package com.ibm.appsure.app ;   
/*-----------------------------------------------------------*/
/*   AppSureTool                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AppSureTool                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The AppSureTool is a GUI front end to the AppSureSystem. This
 * class is the main starting point for the application
 *
 *
 * @version 1.0, 7/27/2001 
 * @author  David Lentz
 */
import java.util.* ;
import java.io.* ;
import java.sql.* ;
import com.ibm.appsure.app.system.* ;
import com.ibm.appsure.app.gui.* ;
import com.ibm.appsure.common.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   AppSureTool                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class AppSureTool extends Object {

   /**
     *  The property file which holds all of our properties
     **/
   private Hashtable properties = null ;

   /**
     *  The single database connection the application uses
     **/
   private static Connection dbConn = null ;

   /**
     *  This is the main starting point for the application. The properties
     * will be read and then the LogonDlg will be brought up
     **/
   public static void main ( String args[] ) {
      new AppSureTool ( args ) ;
   }


   /**
     *  This is the constructor for the application
     *
     * @param args The command line args passed
     **/
   public AppSureTool ( String[] args ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our properties                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      initializeProperties ( args ) ;
      new GUISystem() ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Log                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      Log log = new Log ( properties.get ( "LogFileHome" )+"appsuremonitor.log" ) ;
      log.setLogFileMaxSize ( Integer.parseInt ( ""+properties.get ( "MaxLogFileSize" ) ) ) ;              

      logonUser() ;
   }

   /**
     *  This method will log on the user to the database. This effort creates
     * a connection to the app sure database
     *
     * @return TRUE if connected, FALSE if not
     **/
   private boolean logonUser() {
      InitWindow initWindow = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, log the user in                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogonDlg logonDlg = new LogonDlg () ;
      logonDlg.getResponse() ;

      initWindow = new InitWindow () ;
      initWindow.setText ( "Logging into AppSure Database" ) ;
      if ( connectToDatabase ( logonDlg.getUserId(), logonDlg.getPassword() ) ) {
         new MainWindow () ;
         initWindow.dispose() ;
         return( true ) ;
      }
      return( false ) ;
   }

   /**
     *  This method will initialize all properties for the application,
     * including the properties from the command line and property file
     *
     * @param args The command line arguments
     **/
   private void initializeProperties ( String[] args ) {
      properties = new Hashtable () ;
      readProperties() ;
      processCommandLineArgs ( args ) ;
   }


   /**
     *  This method will read in all of our properties and populate our
     * property hashtable.
     **/
   private void readProperties () {
      Properties propertyFile = new Properties() ;

      try {
         Enumeration enum = null ;

         propertyFile.load ( new FileInputStream ( new File ( "AppSureTool.properties" ) ) ) ;
         enum = propertyFile.propertyNames() ;

         while ( enum.hasMoreElements() ) {
            Object nextElement = enum.nextElement() ;

            properties.put ( nextElement, propertyFile.get ( nextElement ) ) ;
         }
      } catch ( Exception e ) {
         System.err.println ( "Cannot find AppSureTool.properties file. Please make sure this file exists in the directory in which you executed the AppSure Monitor." ) ;
      }
   }


   /**
     *  This method will replace all current properties in the 'properties' hash table
     * with the command line args
     **/
   private void processCommandLineArgs ( String[] args ) {
      try {
         for ( int i = 0; i < args.length; i++ ) {
            int    equalIndex    = args[i].indexOf ( "=" ) ;
            if ( equalIndex >= 0 ) {
               String propertyName  = args[i].substring ( 0, equalIndex ) ;
               String propertyValue = args[i].substring ( equalIndex+1 ) ;

               properties.remove ( propertyName ) ;
               properties.put ( propertyName, propertyValue ) ;
            }
         }
      } catch ( Exception e ) {
      }
   }


   /**
     *  This method will attempt to make a connection to the AppSure database.
     *
     * @param userId The User Id to use to log in with
     * @param password The password to use to log in with
     *
     * @return TRUE if connected
     **/
   private boolean connectToDatabase ( String userId, String password ) {
      boolean  rc       = false ;
      String   database = (String)properties.get ( "Database" ) ;
      String   server   = (String)properties.get ( "DatabaseServer" ) ;
      String   portNum  = (String)properties.get ( "JDBCPortNumber" ) ;
      Log.log ( "Connecting to Database "+database+" on server "+server+" with port number "+portNum ) ;

      try {
         String      driverName = "COM.ibm.db2.jdbc.net.DB2Driver" ;

         Class.forName ( driverName ).newInstance() ;

         dbConn = DriverManager.getConnection( "jdbc:db2://"+server+":"+portNum+"/"+database, userId, password );

         rc = ( dbConn != null ) ;

      } catch ( Exception e ) {
         if ( e.toString().indexOf ( "username and/or" ) != -1 ) {
            GUISystem.printBox ( "Info", "The User Id or Password supplied is not correct" ) ;
            logonUser() ;
         } else {
            GUISystem.printBox ( "Error", "Cannot connect to the AppSure Database. Please check the log file for more information" ) ;
            Log.log ( "Cannot connect to database "+database+" on server "+server+" with port number "+portNum ) ;
            Log.log ( e ) ;
            System.exit(-1) ;
         }
      }
      return( rc ) ;
   }


   /**
     *  This method will return the database connection for the application
     *
     * @return Connection
     **/
   public static Connection getConnection() {
      return( dbConn ) ;
   }

} /* AppSureTool */
