package com.ibm.appsure.app.gui ;       
/*-----------------------------------------------------------*/
/*   AppSureCheckDlg                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AppSureCheckDlg                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.appsure.common.* ;
import com.ibm.appsure.* ;
import com.ibm.appsure.app.* ;
import com.ibm.appsure.db.* ;
import com.ibm.appsure.app.system.* ;
import com.ibm.appsure.app.shared.gui.* ;
import com.ibm.appsure.app.shared.util.* ;  
/**
 * This dialog holds all of the logs/failures for an application 
 *
 *
 * @version 1.0, 8/7/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AppSureCheckDlg                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class AppSureCheckDlg extends JFrame implements ListSelectionListener, Runnable, ActionListener {

   /**
     *  This vector holds all Failure objects for the application
     **/
   private Vector failures = null ;

   /**
     *  This vector holds all logs for the application
     **/
   private Vector logs = null ;

   /**
     *  These are the titles for the Logs window
     **/
   private String[] logTitles = { "Log Index", "Time", "Keep"} ;

   /**
     *  These are the titles for the failures
     **/
   private String[] failureTitles = { "Failure", "Notified", "Closed", "Working"} ;

   /**
     *  This is the application we are monitoring
     **/
   private Application application = null ;

   /**
     *  This is the 'Ok' Button
     **/
   private DButton pb_OK = new DButton ( "Ok" ) ;

   /**
     *  This is the list of Logs
     **/
   private MultiList cnr_LOGS = new MultiList ( GUISystem.getFontUtil() ) ;

   /**
     *  This is the list of Failures
     **/
   private MultiList cnr_FAILURES = new MultiList ( GUISystem.getFontUtil() ) ;

   /**
     *  This is the MLE to show the logs
     **/
   private JTextArea mle_MAINLOG = new JTextArea () ;

   /**
     *  This is the MLE To show the logs that appears in the failures
     **/
   private JTextArea mle_FAILURE_LOG = new JTextArea () ;

   /**
     *  This is the check box for working
     **/
   private DButton pb_WORK_ON = new DButton ( "Work On" ) ;

   /**
     *  This is the check box for keep
     **/
   private JCheckBox ck_KEEP = new JCheckBox ( "Keep" ) ;

   /**
     *  This is the MLE for the resolution
     **/
   private JTextArea mle_RESOLUTION = new JTextArea() ;

   /**
     *  This is the Notebook for the main dialog which holds the Fail and Logs tabs
     **/
   private JTabbedPane mainNotebook = new JTabbedPane () ;

   /**
     *  This is the Notebook for the Log/Resolution tabs under the Logs tab
     **/
   private JTabbedPane resolutionNotebook = new JTabbedPane () ;

   /**
     *  This is the 'Close' button that closes a failure
     **/
   private DButton pb_CLOSE = new DButton ( "Close" );

   /**
     *  This is the Delete button that will delete a log
     **/
   private DButton pb_DELETE = new DButton ( "Delete" ) ;

   /**
     *  This controil holds our Ok button
     **/
   private ButtonPanel mainButtonBar = new ButtonPanel () ;

   /**
     *  This is the Scroll Pane around the Main Log MLE
     **/
   private JScrollPane scr_LOGS = new JScrollPane ( mle_MAINLOG ) ;

   /**
     *  This is the button panel which holds the Delete and View buttons in the
     * Logs panel
     **/
   private ButtonPanel logButtonPanel = new ButtonPanel () ;

   /**
     *  Default Constructor
     **/
   public AppSureCheckDlg ( Frame parent, Application application ) {
      super ( "AppSure - "+application.getName() ) ;

      this.application = application ;

      initialize() ;

      setSize ( 640, 480 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true );
      new Thread ( this ).start() ;
   }


   /**
     *  This method will create the entire GUI And will add it to
     * us
     **/
   private void initialize () {
      JPanel      failPanel     = new JPanel () ;
      JPanel      logPanel      = new JPanel () ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_LOGS.setColumnHeadings ( logTitles ) ;
      cnr_FAILURES.setColumnHeadings ( failureTitles ) ;    
      cnr_LOGS.setColumnWidth ( 0, 75 ) ;
      cnr_LOGS.setColumnWidth ( 1, 200 ) ;
      cnr_FAILURES.setColumnWidth ( 0, 350 ) ;
      cnr_FAILURES.setColumnWidth ( 1, 200 ) ;
      cnr_FAILURES.setColumnWidth ( 2, 200 ) ;
      cnr_FAILURES.setColumnWidth ( 3, 200 ) ;
      mle_RESOLUTION.setFont ( pb_OK.getFont() ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener ( this ) ;
      pb_WORK_ON.addActionListener ( this ) ;
      ck_KEEP.addActionListener ( this ) ;
      pb_DELETE.addActionListener ( this ) ;
      cnr_LOGS.addListSelectionListener ( this ) ;
      cnr_FAILURES.addListSelectionListener ( this ) ;
      pb_OK.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      logPanel.setLayout ( null ) ;
      failPanel.setLayout ( null ) ;
      getContentPane().setLayout ( null );

      failPanel.add ( cnr_FAILURES ) ;
      failPanel.add ( pb_WORK_ON ) ;
      failPanel.add ( resolutionNotebook ) ;
      failPanel.add ( pb_CLOSE ) ;

      logPanel.add ( cnr_LOGS ) ;
      logPanel.add ( ck_KEEP ) ;
      logPanel.add ( scr_LOGS ) ;
      logButtonPanel.add ( pb_DELETE ) ;
      logPanel.add ( logButtonPanel ) ;

      mainNotebook.addTab ( "Failures", failPanel );
      mainNotebook.addTab ( "Logs",     logPanel ) ;
      resolutionNotebook.addTab ( "Log", new JScrollPane ( mle_FAILURE_LOG ) ) ;
      resolutionNotebook.addTab ( "Resolution", new JScrollPane ( mle_RESOLUTION ) ) ;
      mainButtonBar.add ( pb_OK ) ;

      getContentPane().add ( mainNotebook, BorderLayout.CENTER ) ;
      getContentPane().add ( mainButtonBar, BorderLayout.SOUTH ) ;
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;

   }


   /**
     *  This method will layout and position all of our components
     **/
   public void doLayout () {
      Dimension size         = getSize() ;
      int       rowHeight    = 18 ;
      int       x            = 5 ;
      int       y            = 5 ;

      super.doLayout() ;
      size = getSize() ;

      mainNotebook.setBounds ( 5, 5, size.width-10, size.height-70 ) ;
      mainButtonBar.setBounds ( 5, size.height-60, size.width-10, 25 ) ;

      y=5 ;
      size=new Dimension ( size.width-10, size.height-100 ) ;
      cnr_LOGS.setBounds ( x, y, size.width-(x*3), (size.height/2)-50 ) ;
      y+=((size.height/2)-50)+3 ;
      ck_KEEP.setBounds  ( x, y, size.width-(x*3), rowHeight ) ;
      y+=rowHeight+3 ;
      scr_LOGS.setBounds ( x, y, size.width-(x*3), size.height-y-30 ) ;
      y+=(size.height-y-30)+5 ;
      logButtonPanel.setBounds ( x, y, size.width-(x*2), 25 ) ;


      y=5 ;
      cnr_FAILURES.setBounds ( x, y, size.width-(x*3), (size.height/2)-50 ) ;
      y+=((size.height/2)-50)+3 ;
      resolutionNotebook.setBounds ( x, y, size.width-(x*3), size.height-y-30 ) ;
      y+=(size.height-y-30)+5 ;
      pb_CLOSE.setBounds ( x, y, 80, 25 ) ;
      pb_WORK_ON.setBounds  ( x+85, y, 85, 25 ) ;
      y+=rowHeight+3 ;

   }


   /**
     *  This is the method that is called when buttons are hit
     **/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == pb_OK )
         dispose() ;

      else if ( e.getSource() == pb_WORK_ON ) {
      }

      else if ( e.getSource() == ck_KEEP ) {
         LogFile log = (LogFile)cnr_LOGS.getSelectedItem() ;

         if ( log != null ) {
            log.setKeep ( ck_KEEP.isSelected() ) ;
            try {
               log.writeToDatabase ( AppSureTool.getConnection() ) ;
            } catch ( AppSureException e1 ) {
               Log.log ( e1 ) ;
               GUISystem.printBox ( "Error", "There was an error in the last action you took, please check the logs" ) ;
            }
         }
      }

      else if ( e.getSource() == pb_DELETE ) {
         LogFile log = (LogFile)cnr_LOGS.getSelectedItem() ;

         if ( log != null ) {
            log.setKeep ( ck_KEEP.isSelected() ) ;
            try {
               log.updateRecStatus ( log.REC_DELETE ) ;
               log.writeToDatabase ( AppSureTool.getConnection() ) ;
               cnr_LOGS.remove ( log ) ;
            } catch ( AppSureException e1 ) {
               Log.log ( e1 ) ;
               GUISystem.printBox ( "Error", "There was an error in the last action you took, please check the logs" ) ;
            }
         }
      }

   }

   /**
     *  This thread simply reads in all of our information
     **/
   public void run () {
      Connection con = AppSureTool.getConnection() ;

      try {
         Failure   failure = null ;
         Statement stmt    = con.createStatement() ;
         ResultSet rs      = stmt.executeQuery ( "SELECT A.LOGINDEX, B.DESCRIPTION, A.NOTIFIED, A.CLOSED, "+
                                                 "       A.WORKING,  A.RESOLUTION "+
                                                 "FROM APPSURE.FAILURE A, "+
                                                 "     APPSURE.FAILURETYPE B "+
                                                 "WHERE A.LOGINDEX IN (SELECT LOGINDEX FROM APPSURE.LOGS WHERE APPCODE = '"+application.getAppCode()+"') AND "+
                                                 "      A.FAILURETYPE = B.FAILURETYPE "+
                                                 "FOR FETCH ONLY" ) ;

         failures = new Vector ( 1 ) ;
         cnr_FAILURES.setData ( failures ) ;
         while ( rs.next() ) {
            LogFile logFile = new LogFile ( rs.getInt(1) ) ;

            failure = new Failure ( ) ;
            failure.setLogFile ( logFile ) ;
            failure.setFailureDescription ( rs.getString(2).trim() ) ;
            failure.setNotified           ( rs.getString(3) ) ;
            failure.setClosed             ( rs.getString(4) ) ;
            failure.setWorking            ( rs.getString(5) ) ;
            failure.setResolution         ( rs.getString(6) ) ;

            failures.addElement ( failure ) ;
         }

         cnr_FAILURES.setData ( failures ) ;
         rs.close() ;


         rs = stmt.executeQuery ( "SELECT LOGINDEX, LOGTIMESTAMP, KEEP "+
                                  "FROM APPSURE.LOGS "+
                                  "WHERE APPCODE = '"+application.getAppCode()+"' FOR FETCH ONLY" ) ;

         logs = new Vector ( 1 ) ;
         cnr_LOGS.setData ( logs ) ;
         while ( rs.next() ) {
            LogFile logFile = new LogFile ( rs.getInt(1) ) ;
            logFile.setLogTimestamp ( rs.getString(2) ) ;
            logFile.setKeep ( rs.getString(3).equals ( "Y" ) ) ;
            logs.addElement ( logFile ) ;
         }

         cnr_LOGS.setData ( logs ) ;


      } catch ( Exception e ) {
         Log.log ( e ) ;
         GUISystem.printBox ( "Info", "Cannot read Failures/Logs, Please check the log file for the application" ) ;
      }
   }


   /**
     *  This method will be called when the user selects a Log or Failure
     **/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( e.getSource() == cnr_LOGS )
         refreshLog() ;
      else if ( e.getSource() == cnr_FAILURES )
         refreshFailure() ;
   }


   /**
     *  This method will refresh the information being displayed in the log
     * notebook tab
     **/
   private void refreshLog () {
      LogFile log = (LogFile)cnr_LOGS.getSelectedItem() ;

      try {
         if ( log == null ) {
            mle_MAINLOG.setText ( "" ) ;
            ck_KEEP.setSelected ( false ) ;
         } else {
            if ( log.getLogContents() == null )
               AppSureSQL.readLogContents ( AppSureTool.getConnection(), log ) ;

            if ( log.getLogContents() == null )
               mle_MAINLOG.setText ( "NO LOG FILE AVAILABLE" ) ;
            else
               mle_MAINLOG.setText ( log.getLogContents() ) ;

            ck_KEEP.setSelected ( log.getKeep() ) ;
         }
      } catch ( AppSureException e ) {
         Log.log ( e ) ;
         GUISystem.printBox ( "Error", "There was an error written to the log in the last action you took" ) ;
      }
   }


   /**
     *  This method will refresh the failure information
     **/
   private void refreshFailure () {
      Failure failure = (Failure)cnr_FAILURES.getSelectedItem() ;

      try {
         if ( failure == null ) {
            mle_FAILURE_LOG.setText ( "" ) ;
            mle_RESOLUTION.setText ( "" ) ;
         } else {
            String  resolution = failure.getResolution() ;
            LogFile log        = failure.getLogFile() ;
            
            if ( log.getLogContents() == null )
               AppSureSQL.readLogContents ( AppSureTool.getConnection(), log ) ;

            if ( ( resolution == null ) || ( resolution.length() == 0 ) )
               resolution="Please enter the resolution here and press 'Save'" ;

            mle_RESOLUTION.setText ( resolution  ) ;

            if ( log.getLogContents() == null )
               mle_FAILURE_LOG.setText ( "NO LOG FILE AVAILABLE" ) ;
            else
               mle_FAILURE_LOG.setText ( log.getLogContents() ) ;
         }


      } catch ( AppSureException e ) {
         Log.log ( e ) ;
         GUISystem.printBox ( "Error", "There was an error written to the log in the last action you took" ) ;
      }
   }



   /**
     *  This method will save all information about a failure
     **/
   private void saveFailure () {
      Failure failure = (Failure)cnr_FAILURES.getSelectedItem() ;

      if ( failure != null ) {
         failure.setResolution ( mle_RESOLUTION.getText() ) ;
      }
   }
} /* AppSureCheckDlg */
