package com.ibm.appsure.app.gui ;
/*-----------------------------------------------------------*/
/*   LogonDlg                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LogonDlg                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.io.* ;
import com.ibm.appsure.app.shared.util.* ;
import com.ibm.appsure.app.shared.gui.* ;
import com.ibm.appsure.app.system.* ;
/**
 * A Dialog that asks the user to log on 
 *
 *
 * @version 1.0, 1/30/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   LogonDlg                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class LogonDlg extends JDialog implements Runnable, KeyListener, ActionListener {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextField      ef_USERID          = new JTextField      ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 50 ), "", 0 ) ;
   private JPasswordField  ef_PASSWORD        = new JPasswordField  ( "" ) ;
   private JLabel          st_HELPTEXT        = new JLabel          ( "Please enter your AppSure ID and Password" ) ;
   private JLabel          st_USERID          = new JLabel          ( "User Id" ) ;
   private JLabel          st_PASSWORD        = new JLabel          ( "Password" ) ;
   private JLabel          st_IMAGE           = new JLabel          ( "" ) ;
   private JButton         pb_OK              = new JButton         ( "Ok" ) ;
   private JButton         pb_CANCEL          = new JButton         ( "Cancel" ) ;
   private ButtonPanel     buttonPanel        = new ButtonPanel     () ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean rc = false ;
   private JFrame  parent = null  ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constructor                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public LogonDlg (  ) {
      super ( new JFrame(), "AppSure Login", true ) ;                                         
      parent = (JFrame)getParent() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write to the 'splashClose' file so that the splash      */
      /* closes                                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      FileUtil.saveFile ( "closeSplash", "close" ) ;


      setResizable ( false ) ;

      setSize ( 300, 170 ) ;
      WinUtil.centerWindow ( this) ;
   }


   /*------------------------------------------------------------*/
   /* getResponse
    */
   /**
    *    This method will pop up our window
    *  and will wait for the user to hit enter
    *  or cancel. If they hit cancel this method returns falsse
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if Ok is pressed, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean getResponse () {
      try {

         createControls() ;
         new Thread ( this ).start() ;
         setVisible ( true ) ;

      } catch ( Exception e ) {
      }
      parent.dispose() ;
      return( rc ) ;
   }


   /*-----------------------------------------------------------*/
   /*   createControls                                          */          
   /*                                                           */
   /*   Description:                                            */
   /*      Creates and attaches our controls                    */                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void createControls () {
      Container contentPane = getContentPane() ;

      contentPane.setLayout ( null ) ;
      contentPane.add ( st_HELPTEXT ) ;
      contentPane.add ( st_USERID ) ;
      contentPane.add ( ef_USERID ) ;
      contentPane.add ( st_PASSWORD ) ;
      contentPane.add ( ef_PASSWORD ) ;
      contentPane.add ( buttonPanel ) ;
      contentPane.add ( st_IMAGE ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_CANCEL ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addKeyListener          ( this ) ;
      pb_CANCEL.addKeyListener      ( this ) ;
      pb_OK.addActionListener       ( this ) ;
      pb_CANCEL.addActionListener   ( this ) ;
      ef_USERID.addActionListener   ( this ) ;
      ef_PASSWORD.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_OK ) ;
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      st_HELPTEXT.setHorizontalAlignment ( SwingConstants.CENTER ) ;                              
   }



   /*-----------------------------------------------------------*/
   /* layoutControls
    */
   /**
    *    Moves and resizes all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       x    = 10 ;
      int       y    = 45 ;
      int       cx   = GUISystem.getRowHeight() ;
      Dimension size = getSize() ;

      super.doLayout() ;

      st_IMAGE.setBounds ( x, y-4, 44, 44 ) ;
      st_HELPTEXT.setBounds ( x, 0, size.width-20, 45 ) ;
      st_USERID.setBounds ( x+50, y, 75, cx ) ;
      ef_USERID.setBounds ( x+125, y, 100, cx ) ;
      y+=cx ;

      st_PASSWORD.setBounds ( x+50, y, 75, cx ) ;
      ef_PASSWORD.setBounds ( x+125, y, 100, cx ) ;
      y=size.height-58 ;

      buttonPanel.setBounds ( 0, y, size.width, 25 ) ;
   }


   /*-----------------------------------------------------------*/
   /*   actionPerformed                                         */     
   /*                                                           */
   /*   Description:                                            */
   /*      Responds to buttons being pressed                    */                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( ( e.getSource() == pb_OK ) ||
           ( e.getSource () == ef_PASSWORD ) ||
           ( e.getSource () == ef_USERID ) )
         processOk() ;

      else if ( e.getSource() == pb_CANCEL )
         dispose() ;

   } /* buttonHit */




   /*------------------------------------------------------------*/
   /* processOk
    */
   /**
    *    this method will process our Ok button
    *  hit
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void processOk () {
      if ( saveData() ) {
         rc = true ;
         dispose() ;
      }
   }



   /*-----------------------------------------------------------*/
   /*   saveData                                                */    
   /*                                                           */
   /*   Description:                                            */
   /*      Saves the data on the window                         */                   
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      TRUE if everything was saved                         */   
   /*-----------------------------------------------------------*/
   public boolean saveData () {
      boolean  rc       = false ;
      String   userId   = ef_USERID.getText() ;
      String   password = new String ( ef_PASSWORD.getPassword() ) ;

      if ( ( userId == null ) || ( userId.length() == 0 ) ) {
         GUISystem.printBox ( "Info", "Please enter your User Id" ) ;
         ef_USERID.requestFocus() ;
      } else if ( ( password == null ) || ( password.length() == 0 ) ) {
         GUISystem.printBox ( "Info", "Please enter your password" ) ;
         ef_PASSWORD.requestFocus() ;
      } else 
         rc = true ;


      return( rc ) ;
   } /* saveData */




   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    Refreshes all of the data on our window
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData () {

      try {
            ef_USERID.requestFocus() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

   }

   public void keyReleased ( KeyEvent e ) {
   }
   public void keyTyped ( KeyEvent e ) {
   }

   public void keyPressed ( KeyEvent e ) {

      switch ( e.getKeyCode() ) {
         

         case KeyEvent.VK_ENTER :
            if ( e.getSource() == pb_OK )
               processOk() ;
            else if ( e.getSource() == pb_CANCEL ) {
               dispose() ;
            }
            break ;

         case KeyEvent.VK_SPACE :
            if ( e.getSource() == pb_OK )
               processOk() ;
            else if ( e.getSource() == pb_CANCEL ) {
               dispose() ;
            }
            break ;


      }
   }

   public void run () {
      try {
         Thread.currentThread().sleep ( 100 ) ;
         refreshData() ;
      } catch ( Exception e ) {
      }
   }

   public String getUserId () {
      return ( ef_USERID.getText() ) ;
   }

   public String getPassword () {
      return ( ef_PASSWORD.getText() ) ;
   }
} /* LogonDlg */
