package com.ibm.appsure.app.gui ;  
/*-----------------------------------------------------------*/
/*   MainWindow                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MainWindow                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.appsure.app.shared.util.* ;
import com.ibm.appsure.app.shared.gui.* ;
import com.ibm.appsure.app.system.* ;
import com.ibm.appsure.common.* ;
import com.ibm.appsure.app.* ;
import com.ibm.appsure.* ;
/**
 * This is the AppSure Main Window for the Application 
 *
 *
 * @version 1.0, 7/27/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   MainWindow                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class MainWindow extends JFrame implements WindowListener, Runnable, ActionListener {

   /**
     *  This is the list of title headings for the application list
     **/
   private String[] appTitles = { "Application Code", "Application Name", "Description"} ;

   /**
     *  The name of the thread that will initialize the window
     **/
   private static final String THREAD_INITIALIZE = "INIT" ;

   /**
     *  The only instance of this window allowed
     **/
   private static MainWindow instance = null ;

   /**
     *  This is the MultiList which will hold all of our applications
     **/
   private MultiList cnr_APPS = null ;

   /**
     *  The 'New' Button that will create a new application
     **/
   private DButton pb_NEW = new DButton ( "New" ) ;

   /**
     *  The 'Edit' Button that will edit an application
     **/
   private DButton pb_EDIT = new DButton ( "Edit" ) ;

   /**
     *  This is the 'Delete' button that will delete an application
     **/
   private DButton pb_DELETE = new DButton ( "Delete" ) ;

   /**
     *  This is the 'Check' button that will check an application
     **/
   private DButton pb_CHECK = new DButton ( "Check" ) ;

   /**
     *  This is the button panel that holds all action buttons
     **/
   private ButtonPanel actionPanel = null ;

   /**
     *  Default Constructor
     **/
   public MainWindow () {
      super ( "AppSure" ) ;
      setSize ( 640, 480 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;
      instance = this ;
      addWindowListener ( this ) ;

      createGUI () ;
      new Thread ( this, THREAD_INITIALIZE ).start() ;

   }


   /**
     *  This method will return the one instance of the main window for the application
     **/
   public static MainWindow getInstance() {
      return( instance ) ;
   }


   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e ) {
   }
   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ) {
   }
   public void windowClosed ( WindowEvent e ) {
   }
   public void windowOpened ( WindowEvent e ) {
   }
   public void windowClosing ( WindowEvent e ) {
      System.out.println ( "CLOSING" ) ;
      System.exit(0) ;
   }


   /**
     * This method resizes and positions our components
     **/
   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;

      if ( cnr_APPS != null )
         cnr_APPS.setBounds ( 5, 5, size.width-15, size.height-70 ) ;
      if ( actionPanel != null )
         actionPanel.setBounds ( 5, size.height-60, size.width-15, 25 ) ;
   }

   /**
     *  This method will create and add all of our GUI
     **/
   private void createGUI () {
      Container c = getContentPane() ;
      Dimension prefButtonSize = new Dimension ( 80, 25 ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */     
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_APPS = new MultiList ( GUISystem.getFontUtil() ) ;
      actionPanel = new ButtonPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */        
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_APPS.setColumnHeadings ( appTitles ) ;
      cnr_APPS.setColumnWidth ( 1, 300 ) ;
      cnr_APPS.setColumnWidth ( 2, 400 ) ;
      pb_EDIT.setPreferredSize ( prefButtonSize ) ;
      pb_NEW.setPreferredSize ( prefButtonSize ) ;
      pb_CHECK.setPreferredSize ( prefButtonSize ) ;
      pb_DELETE.setPreferredSize ( prefButtonSize ) ;

      pb_EDIT.addActionListener ( this ) ;
      pb_DELETE.addActionListener ( this ) ;
      pb_NEW.addActionListener ( this ) ;
      pb_CHECK.addActionListener ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */  
      /*                                                           */
      /*-----------------------------------------------------------*/
      actionPanel.add ( pb_CHECK )  ;
      actionPanel.add ( pb_NEW ) ;
      actionPanel.add ( pb_EDIT ) ;
      actionPanel.add ( pb_DELETE ) ;

      c.setLayout ( null ) ;
      c.add ( cnr_APPS, BorderLayout.CENTER ) ;
      c.add ( actionPanel, BorderLayout.SOUTH ) ;
      GUISystem.setPropertiesOnPanel ( actionPanel ) ;
   }


   /**
     *  This thread does a lot of things. The function that is done
     * is handled through the name of the thread
     **/
   public void run () {
      String name = Thread.currentThread().getName() ;

      if ( name.equals ( THREAD_INITIALIZE ) ) {
         refreshAppList() ;
         validate() ;
      }
   }



   /**
     *  This method will refresh the list of applications we are presenting
     **/
   protected void refreshAppList () {
      Connection conn = AppSureTool.getConnection() ;
      try {
         Vector    apps = new Vector ( 1 ) ;
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT APPCODE, NAME, SHORTDESCRIPT "+
                                              "FROM APPSURE.APPLICATION WHERE APPCODE <> 'DEFAULT' FOR FETCH ONLY" ) ;

         setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)) ;
         while ( rs.next() ) {
            Application app = new Application ( rs.getString(1).trim() ) ;

            app.setName ( rs.getString(2).trim() ) ;
            app.setShortDescript ( rs.getString(3).trim() ) ;
            apps.addElement ( app ) ;
         }

         cnr_APPS.setData ( apps ) ;

      } catch ( Exception e ) {
         Log.log ( e ) ;
         GUISystem.printBox ( "Info", "Unable to refresh list of applications" ) ;
      }
      setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
   }



   /**
     *  This method will be called when the user hits a button or double clicks the list
     **/
   public void actionPerformed ( ActionEvent e ) {

      /**
        * This method will edit the currently selected Application
        **/
      if ( e.getSource() == pb_EDIT ) {
         Application app = (Application) cnr_APPS.getSelectedItem() ;

         if ( app != null ) {
            try {
               if ( app.readFromDatabase ( AppSureTool.getConnection() ) ) {
                  System.out.println ( "AppEditDlg Cllaing Starting!" ) ;
                  new AppEditDlg ( this, app ) ;
               }
            } catch ( AppSureException e1 ) {
               GUISystem.printBox ( "Info", "Cannot edit that application. Check log file for more information" ) ;
               Log.log ( e1 ) ;
            }
         }
      } /* pb_EDIT  */


      /**
        * This is the 'New' button
        **/
      else if ( e.getSource() == pb_NEW ) {
         Application app = new Application() ;

         app.updateRecStatus ( app.REC_INSERT ) ;
         new AppEditDlg ( this, app ) ;
      }


      /**
        *  This is the 'Check' button
        **/
      else if ( e.getSource() == pb_CHECK ) {
         Application app = (Application) cnr_APPS.getSelectedItem() ;

         if ( app != null ) {
            try {
               if ( app.readFromDatabase ( AppSureTool.getConnection() ) ) {
                  new AppSureCheckDlg ( this, app ) ;
               }
            } catch ( AppSureException e1 ) {
               GUISystem.printBox ( "Info", "Cannot Check that application. Check log file for more information" ) ;
               Log.log ( e1 ) ;
            }
         }
      }
   }


} /* MainWindow */
