package com.ibm.appsure.app.shared.gui ;                          
/*-----------------------------------------------------------*/
/*   BrightnessImageFilter                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BrightnessImageFilter               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.image.* ;
/**
 * This image filter will either lighten or darken an image 
 *
 *
 * @version 1.0, 9/5/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BrightnessImageFilter                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class BrightnessImageFilter extends RGBImageFilter {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   int brightness = 0 ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our image filter
    *  set at the brightness specified (negative values)
    *  go darker
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BrightnessImageFilter ( int brightness ) {
      this.brightness = brightness ;
      canFilterIndexColorModel = true ;
   }




   /*------------------------------------------------------------*/
   /* filterRGB
    */
   /**
    *    This method will filter the RGB value passed in
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int filterRGB ( int x, int y, int rgb ) {
      int r  = (rgb >> 16) & 0xff ;
      int g  = (rgb >> 8 ) & 0xff ;
      int b  = (rgb >> 0 ) & 0xff ;

      r+=(brightness*r) /100 ;
      g+=(brightness*g) /100 ;
      b+=(brightness*b) /100 ;

      r = Math.min ( Math.max(0,r), 255 ) ;
      g = Math.min ( Math.max(0,g), 255 ) ;
      b = Math.min ( Math.max(0,b), 255 ) ;

      return( rgb & 0xff000000) | ( r <<16) | ( g << 8 ) | ( b << 0 ) ;
   }
} /* BrightnessImageFilter */
