package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   DButton                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DButton                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This is a subclass of the JButton. The only reason it has 
 * been subclasses is so that the actionPerformed is called on
 * a seperate thread. This will increase the responsivness of the
 * SWING gui. Please see the JButton Documentation any JButton
 * References. 
 *
 * The DButton also has the following features added to the
 * JButton:
 *
 * Ability to act like 'URL' Link. Please see actAsLink()
 *
 * @version 1.0, 1/18/1999 
 */
import javax.swing.* ;
import java.awt.event.* ;
import java.awt.* ;



/*-----------------------------------------------------------*/
/*                                                           */
/*   DButton                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class DButton extends JButton implements Runnable, MouseListener {


   private static Insets      margins             = new Insets ( 0, 0, 0, 0 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean     borderRollover      = false ;
   private ActionEvent actionEvent         = null ;
   private boolean     actAsLink           = false ;
   private Color       highlightForeground = Color.white ;
   private Color       oldForeground       = null ;





   /*------------------------------------------------------------*/
   /* Constructors
    */
   /**
    *    All Constructors are mapped to the JButton
    *  Constructors
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DButton() {
      super() ;
      addMouseListener ( this ) ;
      setMargin ( margins ) ;
   }
   public DButton(Icon icon) {
      super(icon) ;
      addMouseListener ( this ) ;
      setMargin ( margins ) ;
   }
   public DButton(String text) {
      super(text) ;
      addMouseListener ( this ) ;
      setMargin ( margins ) ;
   }
   public DButton(String text, Icon icon) {
      super(text,icon) ;
      addMouseListener ( this ) ;
      setMargin ( margins ) ;
   }
   public DButton(String text, Icon icon, String actionCommand ) {
      super(text,icon) ;
      addMouseListener ( this ) ;
      setMargin ( margins ) ;
      setActionCommand ( actionCommand ) ;
   }


   protected void threadedFireActionPerformed ( ActionEvent e ) {
      this.actionEvent = e ;

      if ( actionEvent == null )
         actionEvent = new ActionEvent ( this, 0, this.toString() ) ;

      new Thread ( this ).start() ;
   }
   protected void fireActionPerformed ( ActionEvent e ) {
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    The run method will run the thread that
    *  will call all action listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      super.fireActionPerformed ( actionEvent ) ;
   }





   /*------------------------------------------------------------*/
   /* actAsLink
    */
   /**
    *    This method will either turn on or off the 
    *  'act as link feature' When this feature is on,
    *  The text of the button will underline and the text
    *  will change to the highlightedForegroundColor.
    *
    *  @param enable TRUE = enable the act as link
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actAsLink ( boolean enable ) {
      this.actAsLink = enable ;
   }




   /*------------------------------------------------------------*/
   /* setHighlightForeground
    */
   /**
    *    This method will set the text color of the
    *  button when the mouse is over the button
    *  if the 'actAsLink' is enabled
    *
    *  @param color The new color
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setHighlightForeground ( Color color ) {
      this.highlightForeground = color ;
   }

   public void mouseClicked  ( MouseEvent e ) {
   }

   public void mousePressed  ( MouseEvent e ) {
   }
   
   public void mouseReleased ( MouseEvent e ) {
      if ( actAsLink ) {
         if ( e.getClickCount() == 1 ) 
            threadedFireActionPerformed ( null ) ;
      } else
         threadedFireActionPerformed ( null ) ;
   }

   public void mouseEntered  ( MouseEvent e ) {
      if ( actAsLink ) {
         oldForeground = getForeground() ;
         setForeground ( highlightForeground ) ;
         setCursor( Cursor.getPredefinedCursor(Cursor.HAND_CURSOR)) ;
      }

      if ( borderRollover )
         setBorderPainted ( true ) ;
   }
   public void mouseExited   ( MouseEvent e  ) {
      if ( actAsLink ) {
         setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
         setForeground ( oldForeground ) ;
      }

      if ( borderRollover )
         setBorderPainted ( false ) ;
   }





   /*------------------------------------------------------------*/
   /* setBorderRollover
    */
   /**
    *    THis method will turn the Border Rollover
    *  effect. This effect will make the button have
    *  no border unless the mouse is over the button
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBorderRollover ( boolean borderRollover ) {
      this.borderRollover = borderRollover ;
      setBorderPainted ( !borderRollover ) ;
   }
} /* DButton */

