package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   DList                                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DList                               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The DList is a subclass of JList. It has been subclassed 
 * to fire selection events in a thread. It will also fire
 * action events in the case of a double click on return key hit
 *
 *
 * @version 1.0, 1/19/1999 
 * @author David Lentz
 */
import javax.swing.* ;
import javax.swing.event.* ;
import java.util.Vector ;
import java.awt.event.* ;
import java.awt.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   DList                                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class DList extends JList implements Runnable, MouseListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   int     firstIndex  = 0 ;
   int     lastIndex   = 0 ;
   boolean isAdjusting = false ;
   Vector  actionVec   = null ;
   Image   backImage   = null ;


   public DList() {
      super() ;
      addMouseListener ( this ) ;
   }
   public DList(ListModel dataModel) {
      super ( dataModel ) ;
      addMouseListener ( this ) ;
   }
   public DList(Object[] listData) {
      super ( listData ) ;
      addMouseListener ( this ) ;
   }
   public DList(Vector listData) {
      super ( listData ) ;
      addMouseListener ( this ) ;
   }


   protected  void fireSelectionValueChanged ( int firstIndex, int lastIndex, boolean isAdjusting ) {
      this.firstIndex  = firstIndex ;
      this.lastIndex   = lastIndex ;
      this.isAdjusting = isAdjusting ;
      new Thread ( this, "SELECTION" ).start() ;
   }

   public void run () {
      String operation = Thread.currentThread().getName() ;

      if ( operation.equals ( "SELECTION" ) )
         super.fireSelectionValueChanged ( firstIndex, lastIndex, isAdjusting ) ;
      else if ( operation.equals ( "ACTION" ) )
         fireActionEvent () ;

   }





   /*------------------------------------------------------------*/
   /* fireActionEvent
    */
   /**
    *    This method will fire an action event to all
    *  action listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void fireActionEvent () {
      if ( actionVec != null ) {
         ActionEvent actionEvent = null ;
         int         i           = 0 ;
         int         length      = actionVec.size() ;

         actionEvent = new ActionEvent ( this, ActionEvent.ACTION_PERFORMED, "" ) ;

         while ( i < length )
            ((ActionListener)actionVec.elementAt(i++)).actionPerformed ( actionEvent ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener
    *  to the list
    *  
    *
    *  @param actionListener The new ActionListener
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener actionListener ) {
      if ( actionVec == null )
         actionVec = new Vector ( 1, 1 ) ;

      actionVec.addElement ( actionListener ) ;
   }





   /*------------------------------------------------------------*/
   /* removeActionListener
    */
   /**
    *    This method will remove an action listener
    *  from the list
    *  
    *
    *  @param actionListener The ActionListener to remove
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeActionListener ( ActionListener actionListener ) {
      if ( actionVec != null )
         actionVec.removeElement ( actionListener ) ;
   }



   public void mouseClicked ( MouseEvent e ) {
      if ( e.getClickCount() > 1 )
         new Thread ( this, "ACTION" ).start() ;
   }
   public void mousePressed ( MouseEvent e ) {}
   public void mouseReleased ( MouseEvent e ) {}
   public void mouseEntered ( MouseEvent e ) {}
   public void mouseExited ( MouseEvent e ) {}



   public void setBackground ( Image image ) {
      this.backImage = image ;
   }

   //public void paintComponent ( Graphics g ) {
   //   if ( backImage != null ) {
   //      GraphicTools.wallPaper ( this, g, backImage, new Rectangle ( 0, 0, getPreferredScrollableViewportSize().width, getPreferredScrollableViewportSize().height ) ) ;
   //      paintChildren(g) ;
   //   } else
   //      super.paintComponent ( g) ;
   //}

} /* DList */
