package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   DPanel                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DPanel                              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This simple subclass only captures information about the location and size of the panel and
 * makes each public so no matter when you reference height or width you know that the current
 * height and width values are right.<br><br>
 *
 * This subclass also allows you to specify a preferred width or a preferred height or both. Please
 * see the setPreferedSize method for more info.
 *
 *
 * @version 1.0, 12/14/1998 
 * @author David Lentz
 */
import com.ibm.appsure.app.shared.util.* ;
import javax.swing.* ;
import java.awt.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   DPanel                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class DPanel extends JPanel {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public int       x      = 0 ;
   public int       y      = 0 ;
   public int       width  = 0 ;
   public int       height = 0 ;
   public Rectangle rect   = new Rectangle ( 0, 0, 0, 0 ) ;
   public Image     backImage = null ;

   private Dimension prefSize = new Dimension ( -1, -1 ) ;


   public void setBounds ( int x, int y, int cx, int cy ) {
      if ( ( cx > prefSize.width ) && ( prefSize.width != -1 ) )
         cx = prefSize.width ;
      if ( ( cy > prefSize.height ) && ( prefSize.height != -1 ) )
         cy = prefSize.height ;

      this.x = x ;
      this.y = y ;
      this.width = cx ;
      this.height = cy ;
      rect.width = width ;
      rect.height = height ;
      super.setBounds ( x, y, cx, cy ) ;
   }

   public void setLocation ( int x, int y ) {
      this.x = x ;
      this.y = y ;
      super.setLocation ( x, y ) ;
   }

   public void setLocation ( Point pt ) {
      this.x = pt.x ;
      this.y = pt.y ;
      super.setLocation ( pt ) ;
   }

   public void setSize ( int cx, int cy ) {
      if ( ( cx > prefSize.width ) && ( prefSize.width != -1 ) )
         cx = prefSize.width ;
      if ( ( cy > prefSize.height ) && ( prefSize.height != -1 ) )
         cy = prefSize.height ;
      this.width = cx ;
      this.height = cy ;
      rect.width = width ;
      rect.height = height ;
      super.setSize ( cx, cy ) ;
   }

   public void setSize ( Dimension size ) {
      if ( ( size.width > prefSize.width ) && ( prefSize.width != -1 ) )
         size.width = prefSize.width ;
      if ( ( size.height > prefSize.height ) && ( prefSize.height != -1 ) )
         size.height = prefSize.height ;
      this.width = size.width ;
      this.height = size.height ;
      rect.width = width ;
      rect.height = height ;
      super.setSize ( size ) ;
   }


   public void setBackground ( Image image ) {
      this.backImage = image ;
   }

   public void paintComponent ( Graphics g ) {
      if ( backImage != null ) {
         WinUtil.wallPaper ( rect, this, g, backImage ) ;
         paintChildren ( g ) ;
      } else
         super.paintComponent ( g ) ;
   }

   /**
     *  This method will set the preferred size. If you wish
     * to only set the width or only set the height, pass a -1
     * for the value you wish NOT to set. For example, if you
     * want to set the preferred width to 100 but do not want
     * to set the preferred height, you would pass a Dimension
     * object whose width is 100 and height is -1. The -1 tells
     * the getPreferredSize and getMinimumSize methods to use
     * the super's value.
     **/
   public void setPreferredSize ( Dimension size ) {
      this.prefSize = size ;
   }

   /**
     *  This method will return the preferred size
     **/
   public Dimension getPreferredSize () {
      Dimension defaultSize = new Dimension ( 1,1 ) ;

      try {
         defaultSize = super.getPreferredSize() ;

         if ( prefSize.width != -1 )
            defaultSize.width = prefSize.width ;

         if ( prefSize.height != -1 )
            defaultSize.height = prefSize.height ;
      } catch ( Exception e ) {
      }

      return( defaultSize ) ;
   }

   /**
     *  This method will simply return getPreferredSize
     **/
   public Dimension getMinimumSize () {
      Dimension defaultSize = new Dimension ( 1,1 ) ;

      try {
         defaultSize = super.getMinimumSize() ;

         if ( prefSize.width != -1 )
            defaultSize.width = prefSize.width ;

         if ( prefSize.height != -1 )
            defaultSize.height = prefSize.height ;
      } catch ( Exception e ) {
      }

      return( defaultSize ) ;
   }
   /**
     *  This method will simply return getPreferredSize
     **/
   public Dimension getMaximumSize () {
      Dimension defaultSize = new Dimension ( 1,1 ) ;

      try {
         defaultSize = super.getMaximumSize() ;

         if ( prefSize.width != -1 )
            defaultSize.width = prefSize.width ;

         if ( prefSize.height != -1 )
            defaultSize.height = prefSize.height ;
      } catch ( Exception e ) {
      }

      return( defaultSize ) ;
   }

   /**
     *  This method will update the preferred height only
     **/
   public void setPreferredHeight ( int height ) {
      prefSize.height = height ;
   }

   /**
     *  This method will update the preferred width only
     **/
   public void setPreferredWidth ( int width ) {
      prefSize.width = width ;
   }
} /* DPanel */
