package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   DTextArea                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DTextArea                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This JPanel holds a JTextArea and status information. The status information
 * holds the maximum allowed characters and a percentage full if a maximumText
 * property has been set. It also holds the current cursor position by row and
 * column
 *
 *
 * @version 1.0, 9/8/1999 
 */
import javax.swing.* ;
import javax.swing.border.* ;
import javax.swing.event.* ;
import java.awt.* ;
import java.awt.event.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   DTextArea                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class DTextArea extends JPanel implements DocumentListener, FocusListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea    textArea      = new JTextArea() ;
   private JLabel       st_MAX        = new JLabel ("0" ) ;
   private JLabel       st_CURSORPOS  = new JLabel ("0") ;
   private JProgressBar gauge         = new JProgressBar() ;
   private JPanel       statusPanel   = new JPanel() ;
   private JScrollPane  scrollPane    = new JScrollPane ( textArea ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Variables                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int          maximumText          = 0 ;
   private MaskDocument maskDocument         = null ;





   /*------------------------------------------------------------*/
   /* Constructor(1)
    */
   /**
    *    This constructor will create an initially blank
    *  DTextArea
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DTextArea
    *//*---------------------------------------------------------*/
   public DTextArea () {
      initialize() ;
   }


   /*------------------------------------------------------------*/
   /* Constructor(2)
    */
   /**
    *    This constructor will create and set the text of the
    *  JTextArea to the text passed in
    *  
    *
    *  @param text The initial text to show
    *  @param None
    *  @param None
    *
    *  @return DTextArea
    *//*---------------------------------------------------------*/
   public DTextArea ( String text ) {
      setText ( text ) ;
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* Constructor(3)
    */
   /**
    *    This constructor creates a blank 
    *  DTextArea with a maximumText property
    *  set
    *
    *  @param maximumText The maximum amount of text to allow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DTextArea ( int maximumText ) {
      setMaximumText ( maximumText ) ;
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of the JTextArea
    *  
    *  
    *
    *  @param text The text to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText ( String text ) {
      textArea.setText ( text ) ;
   } /* setText  */




   /*------------------------------------------------------------*/
   /* getText
    */
   /**
    *    This method will return the text that is
    *  displayed in the JTextArea
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getText () {
      return( textArea.getText() ) ;
   } /* getText  */




   /*------------------------------------------------------------*/
   /* setEditable
    */
   /**
    *    This method will turn the editable property
    *  on and off
    *  
    *
    *  @param editable TRUE = allow character edits, FALSE = Do not allow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEditable ( boolean editable ) {
      textArea.setEditable ( editable ) ;
   } /* setEditable  */





   /*------------------------------------------------------------*/
   /* setMaximumText
    */
   /**
    *    This method will set the maximum allowable
    *  length of text displayed in the text area
    *  
    *
    *  @param maximumText The new maximumText. Must be greater than 0
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMaximumText ( int maximumText ) {
      if ( maximumText > 0 ) {
         this.maximumText = maximumText ;
         maskDocument = new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, maximumText ) ;
         maskDocument.addDocumentListener ( this ) ;
         textArea.setDocument ( maskDocument ) ;
         st_MAX.setText ( ""+maximumText ) ;
         gauge.setMaximum ( maximumText ) ;
      } /* if  */
   } /* setMaximumText */




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize the control
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      setLayout ( new BorderLayout() ) ;
      add ( scrollPane, BorderLayout.CENTER ) ;
      add ( statusPanel, BorderLayout.SOUTH ) ;

      statusPanel.setLayout ( new BorderLayout(5,5) ) ;
      statusPanel.add ( st_MAX, BorderLayout.EAST ) ;
      statusPanel.add ( st_CURSORPOS, BorderLayout.WEST ) ;
      statusPanel.add ( gauge, BorderLayout.CENTER ) ;

      textArea.setLineWrap ( true ) ;
      textArea.setWrapStyleWord ( true ) ;
      gauge.setFont ( new Font ( "SansSerif", Font.PLAIN, 10 ) ) ;
      gauge.setStringPainted ( true ) ;
      gauge.setString ( "Length:0" ) ;
      setBorder ( new BevelBorder ( BevelBorder.LOWERED ) ) ;
      addFocusListener ( this ) ;
      st_MAX.addFocusListener ( this ) ;
      gauge.addFocusListener ( this ) ;
      st_CURSORPOS.addFocusListener ( this ) ;
   }

   //public void caretUpdate ( CaretEvent e )  {
   //   try {
   //      int offSet = e.getDot() ;
   //      int line   = textArea.getLineOfOffset(offSet) ;
   //      int pos    = offSet-textArea.getLineStartOffset(line) ;

   //      pos++ ;
   //      line++ ;
   //      st_CURSORPOS.setText ( pos+":"+line ) ;
   //   } catch ( Exception ex ) {
   //      ex.printStackTrace() ;
   //   }
   //}

   private void updateGauge() {
      int length        = textArea.getText().length() ;
      int newPercentage = 0 ;

      //if ( ( length > 0 ) && ( maximumText > 0 ) )
      //   newPercentage = new Integer ( length * 100 / (maximumText - 0) ).intValue() ;

      gauge.setValue ( length ) ;
      gauge.setString ( "Length:"+length ) ;
   }

   public void insertUpdate ( DocumentEvent e ) {
      updateGauge() ;
   }
   public void removeUpdate ( DocumentEvent e ) {
      updateGauge() ;
   }
   public void changedUpdate ( DocumentEvent e ) {
      updateGauge() ;
   }





   /*------------------------------------------------------------*/
   /* getTextArea
    */
   /**
    *    This method will return the JTextArea
    *  instance that this control is using.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JTextArea
    *//*---------------------------------------------------------*/
   public JTextArea getTextArea () {
      return( textArea ) ;
   }


   public void focusGained ( FocusEvent e ) {
      textArea.requestFocus() ;
   }

   public void focusLost ( FocusEvent e ) {
   }
} /* DTextArea */

