package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   DTree                                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DTree                               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import javax.swing.event.* ;
import javax.swing.tree.* ;
import java.awt.event.* ;
import java.util.* ;
/**
 * This is a subclass of JTree so that the fired events are
 * fired in a seperate thread. It also makes it so that you
 * can add yourself as an ActionListener, which will notify
 * you when something is double-clicked on
 *
 * @version 1.0, 3/1/1999
 * @author David Lentz 
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DTree                                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class DTree extends JTree implements Runnable, MouseListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Thread Names                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_FIRECHANGED         = "FireChanged" ;
   private static final String THREAD_FIREEXPANDED        = "FireExpanded" ;
   private static final String THREAD_FIRECOLLAPSED       = "FireCollapsed" ;
   private static final String THREAD_FIREWILLEXPANDED    = "FireWillExpanded" ;
   private static final String THREAD_FIREWILLCOLLAPSED   = "FireWillCollapsed" ;
   private static final String THREAD_FIREACTIONPERFORMED = "FireAction" ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private TreePath           curPath            = null ;
   private TreeSelectionEvent event              = null ;
   private Vector             actionListeners    = null ;
   private boolean            mouseListenerAdded = false ;


   public void fireTreeCollapsed ( TreePath path ) {
      this.curPath = path ;
      new Thread ( this, THREAD_FIRECOLLAPSED ).start() ;
   }

   public void fireTreeExpanded ( TreePath path) {
      this.curPath = path ;
      new Thread ( this, THREAD_FIREEXPANDED ).start() ;
   }

   public void fireTreeWillCollapse ( TreePath path ) {
      this.curPath = path ;
      new Thread ( this, THREAD_FIREWILLCOLLAPSED ).start() ;
   }

   public void fireTreeWillExpand ( TreePath path ) {
      this.curPath = path ;
      new Thread ( this, THREAD_FIREWILLEXPANDED ).start() ;
   }

   protected void fireValueChanged ( TreeSelectionEvent e ) {
      event = e ;
      new Thread ( this, THREAD_FIRECHANGED ).start() ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our events in a thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName() ;

      try {
         if ( name.equals ( THREAD_FIRECHANGED ) )
            super.fireValueChanged ( event ) ;

         else if ( name.equals ( THREAD_FIREWILLEXPANDED ) )
            super.fireTreeWillExpand ( curPath ) ;

         else if ( name.equals ( THREAD_FIREWILLCOLLAPSED ) )
            super.fireTreeWillCollapse ( curPath ) ;

         else if ( name.equals ( THREAD_FIREEXPANDED ) )
            super.fireTreeExpanded ( curPath ) ;

         else if ( name.equals ( THREAD_FIRECOLLAPSED ) )
            super.fireTreeCollapsed ( curPath ) ;

         else if ( name.equals ( THREAD_FIREACTIONPERFORMED ) )
            fireActionPerformed() ;

      } catch ( javax.swing.tree.ExpandVetoException e ) {
         e.printStackTrace() ;
      }
   }

   public void mouseExited ( MouseEvent e ) {
   }

   public void mouseEntered ( MouseEvent e ) {
   }

   public void mouseClicked ( MouseEvent e ) {
      if ( e.getClickCount() > 1 )
         new Thread ( this, THREAD_FIREACTIONPERFORMED ).start() ;
   }

   public void mousePressed ( MouseEvent e) {
   }

   public void mouseReleased ( MouseEvent e ) {
   }





   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener to
    *  this JTree. If an item is double-clicked on, this
    *  tree will fire an action performed event in a separate thread
    *
    *  @param listener The listener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      if ( actionListeners == null )
         actionListeners = new Vector () ;

      actionListeners.addElement ( listener ) ;

      if ( !mouseListenerAdded ) {
         addMouseListener ( this ) ;
         mouseListenerAdded = true ;
      }
   }




   /*------------------------------------------------------------*/
   /* removeActionListener
    */
   /**
    *    This method will remove an action listener
    *  
    *  
    *
    *  @param listener The listener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeActionListener ( ActionListener listener ) {
      if ( actionListeners != null )
         actionListeners.removeElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* fireActionPerformed
    */
   /**
    *    This method will fire an action performed
    *  to all registered listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void fireActionPerformed () {
      if ( actionListeners != null ) {
         int         i    = 0 ;
         int         size = actionListeners.size() ;
         ActionEvent e    = new ActionEvent ( this, 0, toString() ) ;

         while ( i < size ) {
            ((ActionListener)actionListeners.elementAt(i)).actionPerformed ( e ) ;
            i++ ;
         }
      }
   }

} /* DTree */
