package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   DetailTreeCellComponent                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DetailTreeCellComponent             */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.appsure.app.shared.util.* ;
import javax.swing.* ;
import java.awt.* ;      
/**
 * This is the component which does the actual drawing work for items in
 * a JTree that has a Renderer of DetailTreeCellRenderer.
 *
 *
 * @version 1.0, 3/7/2000
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   DetailTreeCellComponent                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class DetailTreeCellComponent extends JComponent {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected Object                 data             = null ;
   protected boolean                selected         = false ;
   protected boolean                leaf             = false ;
   protected boolean                expanded         = false ;
   private   int                    defaultHeight    = 18 ;
   private   Dimension              prefSize         = new Dimension ( 120, defaultHeight ) ;
   private   Dimension              noShowDim        = new Dimension ( 120, 0 ) ;
   private   boolean                noShow           = false ;
   private   DetailTreeCellRenderer treeRenderer     = null ;
   private   int[]                  colDim           = null ;
   private   FontUtil               fontUtil         = null ;
   private   String                 tempString       = null ;
   private   int                    tempStringLength = 0 ;
   private   boolean                wordWrap         = false ;
   private   boolean                evenRow          = false ;
   private   Color                  selectedBackgroundColor = new Color ( 220, 220, 255 ) ;
   private   Color                  lightGray               = new Color ( 240, 240, 240 ) ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a blank DefaultTreeComponent
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DetailTreeCellComponent ( DetailTreeCellRenderer treeRenderer, int columnNum, FontUtil fontUtil ) {
      this.fontUtil     = fontUtil ;
      this.treeRenderer = treeRenderer ;
      prefSize = new Dimension ( 120, 18 ) ;
      colDim = new int[columnNum] ;

      for ( int i = 0; i < columnNum; i++ )
         colDim[i] = 100 ;
      prefSize.width = (columnNum*100)+400 ;

   }



   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data in
    *  memory
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object newData, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus ) {
      data          = newData ;
      this.selected = selected ;
      this.expanded = expanded ;
      this.leaf     = leaf ;                                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   IF this is a MutableTreeWrapper, ask him for the data   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( data instanceof MutableTreeWrapper ) && ( !(data instanceof ViewTreeNode ) ) )
         data = ((MutableTreeWrapper)data).getDataObject() ;



      noShow = ( ( data == null ) || 
                 ( data.toString() == null ) || 
                 ( data.toString().length() == 0 ) ) ;

      evenRow = ( ((row/2)*2) != row ) ;
   }

   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method is subclassed because the height
    *  of our component depends on the data we
    *  are showing
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {

      if ( noShow )
         return( noShowDim ) ;

      prefSize.height = defaultHeight ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to word wrap, we must define our height here  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( wordWrap ) {
         if ( ( data instanceof MultiListRow ) || ( data instanceof ExtendedMultiListRow ) ) {
            prefSize.height = getPreferredHeightFromMultiListRow ( (MultiListRow) data ) ;
         } 
      } 

      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* setPreferredWidth
    */
   /**
    *    This method will set the preferred width of
    *  all items in the list
    *  
    *
    *  @param width The preferred width
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPreferredWidth ( int width ) {
      prefSize.width = width ;
   }


   /*------------------------------------------------------------*/
   /* paint
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param g The Graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paint ( Graphics g ) {

      getPreferredSize() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are selected, paint our selected background       */                                        
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( selected ) {
         g.setColor ( selectedBackgroundColor ) ;
         g.fillRect ( 0, 0, prefSize.width, prefSize.height ) ;
      } 
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, clear it                                     */          
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         if ( evenRow )
            g.setColor ( Color.white ) ;
         else
            g.setColor ( lightGray ) ;
         g.fillRect ( 0, 0, prefSize.width, prefSize.height ) ;
      }

      g.setColor ( Color.black ) ;

      if ( data instanceof MultiListRow )
         paintMultiListRow ( g, (MultiListRow) data ) ;

      else
         paintData ( g, data, 0, 11, 1800, prefSize.height, (Rectangle)g.getClip() ) ;

      if ( selected ) {
         g.setColor ( new Color ( 170, 170, 255 ) ) ;
         g.drawRect ( 0, 0, prefSize.width-2, prefSize.height-2 ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* paintMultiListRow
    */
   /**
    *    This method will paint a row of information
    *  
    *  
    *
    *  @param MultiListRow The row to paint
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void paintMultiListRow ( Graphics g, MultiListRow row ) {
      int       i         = 0 ;
      int       x         = 0 ;
      Rectangle clip      = (Rectangle)g.getClip() ;


      if ( colDim.length > 0 ) {
         int columnNum = colDim.length ;

         while ( i < columnNum ) {//&& ( clip.x < (x+colDim[i] ) ) ) {

            if ( row instanceof ExtendedMultiListRow )
               ((ExtendedMultiListRow)row).adjustColumnGraphics ( columnNum, g ) ;

            if ( data instanceof String )
               paintData ( g, row.getColumnData(i), x, 11, treeRenderer.getColumnWidth(i), prefSize.height, clip ) ;
            else
               paintData ( g, row.getColumnData(i), x, 0, treeRenderer.getColumnWidth(i), prefSize.height, clip ) ;

            x+=treeRenderer.getColumnWidth(i) ;
            i++ ;
         }
      }// else 
      //  paintData ( g, row.getColumnData(0), 0, 10, 1800, prefSize.height, null ) ;

   } /* paintMultiListRow  */





   /*------------------------------------------------------------*/
   /* getPreferredHeightFromMultiListRow
    */
   /**
    *    Thie method is used to get the preferred
    *  height needed to display a MultiListRow Fully
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The height in pixels
    *//*---------------------------------------------------------*/
   private int getPreferredHeightFromMultiListRow ( MultiListRow row ) {
      int i         = 0 ;
      int retHeight = 0 ;

      if ( colDim.length > 0 ) {
         int    columnNum = colDim.length ;
         Object data      = null ;

         while ( i < columnNum ) {

            data = row.getColumnData(i) ;

            if ( data instanceof String[] ) {
               if ( retHeight < ((String[])data).length*defaultHeight )
                  retHeight = ((String[])data).length*defaultHeight ;
            }

            else if ( data instanceof Image ) {
               if ( retHeight < ((Image)data).getHeight ( this ) )
                  retHeight = ((Image)data).getHeight ( this ) ;
            }

            else {
               if ( retHeight < defaultHeight )
                  retHeight = defaultHeight ;
            }

            i++ ;
         } /* while */
      } /* if */

      return( retHeight ) ;
   } /* getPreferredHeightFromMultiListRow */





   /*------------------------------------------------------------*/
   /* paintData
    */
   /**
    *    This method will simply try to paint
    *  data
    *  
    *
    *  @param g The graphics context
    *  @param data The data to paint
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintData ( Graphics g, Object data, int x, int y, int width, int height, Rectangle clip ) {

      if ( data != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paint an Image                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( data instanceof Image )
            g.drawImage ( (Image)data, x, y, this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paint an Image Icon                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( data instanceof ImageIcon )
            g.drawImage ( ((ImageIcon)data).getImage(), x, y, this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paint an Array of Strings                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( data instanceof String[] ) {
            String[] stringArray      = (String[]) data ;
            int      i                = 0 ;
            int      tempStringLength = 0 ;
            int      tempY            = y+11 ;

            if ( stringArray.length == 1 )
               tempY+=2 ;

            for ( i = 0; i < stringArray.length; i++ ) {
               tempStringLength = stringArray[i].length() ;


               if ( tempStringLength > 0 ) {
                  /* Set up the font util for our font  */
                  if ( fontUtil.getFont() != g.getFont() )
                     fontUtil.setFont ( g.getFont() ) ;

                  fontUtil.drawCharsEtc ( g, 
                                          stringArray[i].toCharArray(), 
                                          tempStringLength, 
                                          x, 
                                          tempY, 
                                          clip, 
                                          width ) ;
                  tempY+=defaultHeight ;
               }
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paint everything else                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            tempString = data.toString() ;
            if ( tempString != null ) {
               tempStringLength = tempString.length() ;


               if ( tempStringLength > 0 ) {
                  /* Set up the font util for our font  */
                  if ( fontUtil.getFont() != g.getFont() )
                     fontUtil.setFont ( g.getFont() ) ;

                  fontUtil.drawCharsEtc ( g, 
                                          tempString.toCharArray(), 
                                          tempStringLength, 
                                          x, 
                                          ( (wordWrap) && ( prefSize.height != defaultHeight) ) ? y+11 : fontUtil.getYCentered(height-1), 
                                          clip, 
                                          width ) ;
               }
            }
         }
      }
   }




   /*------------------------------------------------------------*/
   /* setColumnWidth
    */
   /**
    *    This method will set the column width for
    *  a specified column
    *  
    *
    *  @param column The column to set the width for
    *  @param width The width in pixels
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setColumnWidth ( int column, int width ) {
      colDim[column] = width ;
      prefSize.width=0 ;
      for ( int i = 0; i < colDim.length; i++ )
         prefSize.width+=colDim[i] ;
      prefSize.width+=400 ;
   }





   /*------------------------------------------------------------*/
   /* setWordWrap
    */
   /**
    *    This method will turn word wrapping on or
    *  off. If you are wondering how to do this in a JTree, please
    *  look at the DetailTreeCellRenderer class to do this.
    *  
    *
    *  @param wordWrap
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setWordWrap ( boolean wordWrap ) {
      this.wordWrap = wordWrap ;
   }
} /* DefaultTreeComponent */
