package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   DragWindow                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DragWindow                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The DragWindow is a simple extension of Window that can
 * be dragged around by the user.
 *
 * @version 1.0, 5/13/1998 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.appsure.app.shared.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   DragWindow                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class DragWindow extends JWindow implements MouseMotionListener, MouseListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int RESIZE_SE      = 1 ;
   private static final int RESIZE_NE      = 2 ;
   private static final int RESIZE_E       = 3 ;
   private static final int RESIZE_SW      = 4 ;
   private static final int RESIZE_NW      = 5 ;
   private static final int RESIZE_W       = 6 ;
   private static final int RESIZE_N       = 7 ;
   private static final int RESIZE_S       = 8 ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected int            width          = 0 ;
   protected int            height         = 0 ;
   protected int            x              = 0 ;
   protected int            y              = 0 ;
   private   int            oldWidth       = -1 ;
   private   int            oldHeight      = -1 ;
   protected int            oldX           = -1 ;
   protected int            oldY           = -1 ;
   private   boolean        sizeDrag       = false ;
   private   int            dragX          = 0 ;
   private   int            dragY          = 0 ;
   protected int            sizeDir        = 0 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constructors                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public DragWindow ( Frame parent, JPanel panel, int width, int height ) {
      super ( parent ) ;
      init() ;
      getContentPane().setLayout ( new BorderLayout()) ;
      getContentPane().add ( panel, BorderLayout.CENTER ) ;
      setSize ( width, height ) ;

      //addMouseListeners ( panel ) ;
   }


   /*-----------------------------------------------------------*/
   /*   init                                                    */
   /*                                                           */
   /*   Description:                                            */
   /*      Initializes the DragWindow                           */                                          
   /*                                                           */
   /*   Parameters:                                             */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   private void init ( ) {
      addMouseListener ( this ) ;
      addMouseMotionListener ( this ) ;
   }

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setSize subclassed to set variables                     */                  
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setSize ( int width, int height ) {
      this.width = width ;
      this.height = height ;
      super.setSize ( width, height ) ;
   }



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setSize subclassed to set variables                     */                  
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setSize ( Dimension size ) {
      this.width = size.width ;
      this.height = size.height ;
      super.setSize ( size ) ;
   }



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setBounds subclassed to store variables                 */                      
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setBounds ( int x, int y, int cx, int cy ) {
      this.width = cx ;
      this.height = cy ;
      this.x = x ;
      this.y = y ;
      super.setBounds ( x, y, cx, cy ) ;
   }



   /*-----------------------------------------------------------*/
   /*   mouseDragged                                            */           
   /*                                                           */
   /*   Description:                                            */
   /*      processes the mouse drag messsages sent              */                                                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      MouseEvent - The mouse event for us                  */                           
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void mouseDragged ( MouseEvent e ) {
      drag ( e.getX(), e.getY(), dragX, dragY ) ;
   }

   public void drag ( int mx, int my, int dragX, int dragY ) {

      sizeDrag = true ;
      this.dragX = dragX ;
      this.dragY = dragY ;
      /* if sizeDir is 0, then we are moving the
         component, otherwise we are sizing it */
      if ( sizeDir == 0 ) {
         mx+=x ;
         my+=y ;

         setLocation ( mx-dragX, my-dragY ) ;
      } else {

         switch ( sizeDir ) {
            case RESIZE_S :
               setBounds ( x, y, width, my+3 ) ;
               break ;

            case RESIZE_E :
               setBounds ( x, y, mx+3, height ) ;
               break ;

            case RESIZE_SE :
               setBounds ( x, y, mx+3, my+3 ) ;
               break ;

            case RESIZE_N :
               setBounds ( x, y+my, width, height-my ) ;
               break ;

            case RESIZE_W :
               setBounds ( x+mx, y, width-mx, height ) ;
               break ;

            case RESIZE_NW :
               setBounds ( x+mx, y+my, width-mx, height-my ) ;
               break ;

            case RESIZE_NE :
               setBounds ( x, y+my, mx+3, height-my ) ;
               break ;

            case RESIZE_SW :
               setBounds ( x+mx, y, width-mx, my+3 ) ;
               break ;

         }
      }
   }


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setLocation subclassed to store variables               */                        
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setLocation ( int x, int y ) {
      if ( oldX == -1 )
         oldX=x ;
      else
         oldX=this.x ;
      if ( oldY == -1 )
         oldY=y ;
      else
         oldY=this.y ;

      this.x = x ;
      this.y = y ;
      super.setLocation(x,y) ;
   }

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setLocation subclassed to store variables               */                        
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setLocation ( Point pt ) {

      if ( oldX == -1 )
         oldX=pt.x ;
      else
         oldX=this.x ;
      if ( oldY == -1 )
         oldY=pt.y ;
      else
         oldY=this.y ;

      this.x = pt.x ;
      this.y = pt.y ;
      super.setLocation ( pt ) ;
   }

   /*-----------------------------------------------------------*/
   /*   mouseMoved                                              */         
   /*                                                           */
   /*   Description:                                            */
   /*      processes mouse move messages                        */                                              
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void mouseMoved ( MouseEvent e ) {
      int mx = e.getX() ;
      int my = e.getY() ;

      if ( ( mx < 5 ) && ( my < 5 ) ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.NW_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_NW ;
      } else if ( ( mx < 5 ) && ( my > height-5 ) ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.SW_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_SW ;
      } else if ( ( mx > width-5 ) && ( my < 5 ) ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.NE_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_NE ;
      } else if ( ( mx > width-5 ) && ( my > height-5 ) ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.SE_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_SE ;
      } else if ( mx < 5 ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.W_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_W ;
      } else if ( my < 5 ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.N_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_N ;
      } else if ( my > height-5 ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.S_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_S ;
      } else if ( mx > width-5 ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.E_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_E ;
      } else {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.DEFAULT_CURSOR ) ) ;
         sizeDir = 0 ;
      }

   }

   public void mouseReleased ( MouseEvent e ) {
      sizeDrag = false ;
      setBounds ( x, y, width, height ) ;
   }

   public void mousePressed ( MouseEvent e ) {

      sizeDrag = true ;

      /* Mark the mouse location for use in the full drag */
      dragX = e.getX() ;
      dragY = e.getY() ;
   }

   public void mouseClicked ( MouseEvent e ) {
      sizeDrag = false ;
   }

   public void mouseEntered ( MouseEvent e ) {
   }

   public void mouseExited ( MouseEvent e ) {
   }


   /*------------------------------------------------------------*/
   /* addMouseListener
    */
   /**
    *    This method will add A mouse listener
    *  for all Components on the panel passed in 
    *  recursivly. 
    *
    *  @param panel The Panel to traverse through
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addMouseListeners ( Container panel ) {
      Component   c         = null ;
      Component[] compArray = panel.getComponents() ;
      int         i         = 0 ;

      panel.addMouseListener ( this ) ;

      while ( i < compArray.length ) {

         c = compArray[i] ;    
         c.addMouseListener ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the component is another container, call us again    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( c instanceof Container ) {
            addMouseListeners ( (Container)c ) ;

            i++ ;
         }
      }
   }


} /* DragWindow */

