package com.ibm.appsure.app.shared.gui ;    
/*-----------------------------------------------------------*/
/*   HotLinkGroup                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HotLinkGroup                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.event.* ;

/**
 * This object allows multiple HotLinkLabels to be grouped 
 * together. This allows 1 HotLinkLabel in a group to be selected.
 * 
 *
 * @version 1.0, 12/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HotLinkGroup                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class HotLinkGroup extends Object implements MouseListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector       labels        = new Vector ( 1 ) ;
   private HotLinkLabel selectedLabel = null ;
   private boolean      multiSelect   = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default Constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkGroup () {
   }




   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will add a HotLinkLabel
    *  to the group
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void add ( HotLinkLabel label ) {
      label.addMouseListener ( this ) ;
      label.setAutoSelect ( false ) ;
      labels.addElement ( label ) ;
   }



   /*------------------------------------------------------------*/
   /* deselectLabels
    */
   /**
    *    This method will deselect all labels
    *  except the label in the 'selectedLabel' property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void deselectLabels () {
      int i    = 0 ;
      int size = labels.size() ;

      while ( i < size ) {
         ((HotLinkLabel)labels.elementAt(i)).setSelected ( false ) ;
         ((HotLinkLabel)labels.elementAt(i)).repaint() ;
         i++ ;
      }
   }




   /*------------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove all HotLinkLabels
    *  from our group
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAll () {
      labels = new Vector ( 1, 1 ) ;
   }




   /*------------------------------------------------------------*/
   /* setMultiSelect
    */
   /**
    *    This method will turn multi select on or off
    *  
    *  
    *
    *  @param multiSelect TRUE = allow multiple selections
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMultiSelect ( boolean multiSelect ) {
      this.multiSelect = multiSelect ;
      if ( !multiSelect )
         deselectLabels() ;
   }





   /*------------------------------------------------------------*/
   /* mouseEntered
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseEntered ( MouseEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* mouseExited
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseExited ( MouseEvent e ) {
   }





   /*------------------------------------------------------------*/
   /* mousePressed
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mousePressed ( MouseEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* mouseReleased
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseReleased ( MouseEvent e ) {
   }





   /*------------------------------------------------------------*/
   /* mouseClicked
    */
   /**
    *    This method will deselect the appropriate HotLinkLabels
    *    since another was selected
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseClicked ( MouseEvent e ) {                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If multi select, only select multiples if               */
      /* the control key is down                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( multiSelect ) {
         if ( !e.isControlDown() )
            deselectLabels () ;
      } else
         deselectLabels () ;

      ((HotLinkLabel)e.getSource()).setSelected ( true ) ;
      ((HotLinkLabel)e.getSource()).revalidate() ;
      //new Thread ( (HotLinkLabel)e.getSource(), 
      //             HotLinkLabel.THREAD_FIREACTION ).start() ;
   }


} /* HotLinkGroup */
