package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   HotLinkLabel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HotLinkLabel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
/**
 * The HotLinkLabel is a subclass of JComponent that displays text and will change
 * color to blue and underline itself if the mouse enters
 * the bounds of the component. Once the mouse has left, it will
 * return to it's normal state.<br><br>
 *
 * If the mouse is pressed, any registered ActionListener will
 * be notified VIA a actionPerformed call. This default behavior
 * can be modified by calling the 'setAutoSelect' method.<br><br>
 *
 * The HotLinkLabel is great to use as a small (  less than 50 items ) list. To
 * use the HotLinkLabel in a list, use the HotLinkGroup object to keep the selections
 * correct.
 *
 * @version 1.0, 1/11/2000
 * @author David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HotLinkLabel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class HotLinkLabel extends JComponent implements Runnable, MouseListener, MouseMotionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_FIREACTION = "FA" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector     listenerVec   = null ;
   private String     text          = null ;
   private int        fontLeading   = 0 ;
   private int        fontDescent   = 0 ;
   private int        fontHeight    = 0 ;
   private int        fontWidth     = 0 ;
   private int        imageWidth    = 0 ;
   private int        imageHeight   = 0 ;
   private boolean    mouseOver     = false ;
   private boolean    mouseDown     = false ;
   private ImageIcon  imageIcon     = null ;
   private Dimension  minSize       = new Dimension (0,0) ;
   private String     actionCommand = null ;
   private boolean    enabled       = true ;
   private int        buttonId      = 0 ;
   private Object     data          = null ;
   private boolean    selected      = false ;
   private boolean    showSelection = true ;
   private boolean    autoSelect    = true ;
   private Font       currentFont   = null ;


   private Color selectedForeground = null ;
   private Color selectedBackground = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initially
    *  as a blank string
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel () {
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String passed in.
    *  
    *
    *  @param text The text to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text ) {
      initialize() ;
      setText ( text ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the ImageIcon passed in.
    *  
    *
    *  @param imageIcon The ImageIcon to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( ImageIcon imageIcon ) {
      initialize() ;
      setIcon ( imageIcon ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String and ImageIcon passed in.
    *  
    *
    *  @param text The text to display
    *  @param imageIcon The ImageIcon to display
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, ImageIcon imageIcon ) {
      initialize() ;
      setText ( text ) ;
      setIcon ( imageIcon ) ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String and ImageIcon passed in.
    *  
    *
    *  @param text The text to display
    *  @param imageIcon The ImageIcon to display
    *  @param actionCommand An action command for this label
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, ImageIcon imageIcon, String actionCommand ) {
      initialize() ;
      setText ( text ) ;
      setActionCommand ( actionCommand ) ;
      setIcon ( imageIcon ) ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String and ImageIcon passed in.
    *  
    *
    *  @param text The text to display
    *  @param imageIcon The ImageIcon to display
    *  @param actionCommand An action command for this label
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, ImageIcon imageIcon, String actionCommand, int id ) {
      initialize() ;
      setId ( id ) ;
      setText ( text ) ;
      setActionCommand ( actionCommand ) ;
      setIcon ( imageIcon ) ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String passed in.
    *  
    *
    *  @param text The text to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, int id ) {
      initialize() ;
      setId ( id ) ;
      setText ( text ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the ImageIcon passed in.
    *  
    *
    *  @param imageIcon The ImageIcon to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( ImageIcon imageIcon, int id ) {
      initialize() ;
      setIcon ( imageIcon ) ;
      setId ( id ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String and ImageIcon passed in.
    *  
    *
    *  @param text The text to display
    *  @param imageIcon The ImageIcon to display
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, ImageIcon imageIcon, int id ) {
      initialize() ;
      setText ( text ) ;
      setId ( id ) ;
      setIcon ( imageIcon ) ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String and ImageIcon passed in.
    *  
    *
    *  @param text The text to display
    *  @param imageIcon The ImageIcon to display
    *  @param actionCommand An action command for this label
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, ImageIcon imageIcon, String actionCommand, int id, Object data ) {
      initialize() ;
      setData ( data ) ;
      setId ( id ) ;
      setText ( text ) ;
      setActionCommand ( actionCommand ) ;
      setIcon ( imageIcon ) ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String passed in.
    *  
    *
    *  @param text The text to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, int id, Object data ) {
      initialize() ;
      setData ( data ) ;
      setId ( id ) ;
      setText ( text ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the ImageIcon passed in.
    *  
    *
    *  @param imageIcon The ImageIcon to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( ImageIcon imageIcon, int id, Object Data ) {
      initialize() ;
      setData ( data ) ;
      setIcon ( imageIcon ) ;
      setId ( id ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel initialized
    *  to the String and ImageIcon passed in.
    *  
    *
    *  @param text The text to display
    *  @param imageIcon The ImageIcon to display
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, ImageIcon imageIcon, int id, Object data ) {
      initialize() ;
      setData ( data ) ;
      setText ( text ) ;
      setId ( id ) ;
      setIcon ( imageIcon ) ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates the HotLinkLabel with the
    *  text passed in, and will add the
    *  ActionListener passed to the listener list.
    *
    *  @param text The text to initialize to
    *  @param listener The ActionListener to add
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HotLinkLabel ( String text, ActionListener listener ) {
      initialize() ;
      setText ( text ) ;
      addActionListener ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize all internal
    *  listeners and variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      addMouseListener ( this ) ;
      addMouseMotionListener ( this ) ;
      
      if ( currentFont == null ) 
         currentFont = (Font)javax.swing.UIManager.getDefaults().get ( "Button.font" ) ;

      if ( selectedForeground == null )
         selectedForeground = (Color)javax.swing.UIManager.getDefaults().get ( "List.selectionForeground" ) ;
      
      if ( selectedBackground == null )
         selectedBackground = (Color)javax.swing.UIManager.getDefaults().get ( "List.selectionBackground" ) ;

      setFont ( currentFont ) ;
   }


   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of the
    *  HotLinkLabel.
    *  
    *
    *  @param text The new text to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText ( String text ) {
      this.text = text ;
      fontWidth = Toolkit.getDefaultToolkit().getFontMetrics (getFont()).stringWidth ( text ) ;
      revalidate() ;
   }



   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an ActionListener
    *  to be notified if either the left or
    *  right mouse button is pressed.
    *
    *  @param listener The ActionListener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      if ( listenerVec == null )
         listenerVec = new Vector ( 1, 1 ) ;

      listenerVec.addElement ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will repaint our
    *  component.
    *  
    *
    *  @param g The Graphics context to draw into
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      Dimension size = getSize() ;

      super.paintComponent ( g ) ;
      size.height-- ;
      size.width-- ;            

      if ( selected ) {
         if ( showSelection ) {
            g.setColor ( selectedBackground ) ;
            g.fillRect ( 0, 0, size.width, size.height ) ;
         }
      } else if ( isOpaque() ) {
         g.setColor ( getBackground() ) ;
         g.fillRect ( 0, 0, size.width, size.height ) ;
      }


      if ( enabled ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the correct color                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !mouseOver )
            g.setColor ( getForeground() ) ;
         else
            g.setColor ( selectedBackground ) ;

         if ( ( mouseDown ) && ( mouseOver ) )
            g.setColor ( Color.red ) ;
      } else
         g.setColor ( Color.darkGray ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If text, draw it                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( text != null ) {
         int         space    = (size.height-fontHeight)/2 ;
         int         y        = size.height-space-fontDescent-(fontLeading/2) ;

         if ( selected ) {
            if ( showSelection )
               g.setColor ( selectedForeground ) ;
         } else
            g.setColor ( getForeground() ) ;

         g.setFont ( currentFont ) ;
         g.drawString ( text, imageWidth+1, y ) ;                     

         if ( imageIcon != null )
            g.drawImage ( imageIcon.getImage(), 0, (size.height/2)-(imageHeight/2), this ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw the line beneath the text                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( mouseOver ) && ( enabled ) ) 
            g.drawLine ( 0, y+1, fontWidth+imageWidth, y+1 ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* setFont
    */
   /**
    *    This method will set our current
    *  font
    *  
    *
    *  @param font The new font to draw the text with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFont ( Font font ) {
      FontMetrics fontMetrics = Toolkit.getDefaultToolkit().getFontMetrics (font) ;

      fontHeight      = fontMetrics.getHeight() ;
      fontDescent     = fontMetrics.getDescent() ;
      fontLeading     = fontMetrics.getLeading() ;

      if ( text != null ) 
         fontWidth = Toolkit.getDefaultToolkit().getFontMetrics (font).stringWidth ( text ) ;

      currentFont = font ;
      super.setFont ( font ) ;
   }





   /*------------------------------------------------------------*/
   /* mouseEntered
    */
   /**
    *    This method will be called once the
    *  mouse has entered our component. This
    *  method will paint the component to reflect
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseEntered ( MouseEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* mouseExited
    */
   /**
    *    This method will be called once the
    *  mouse has left the component. This method
    *  will paint our component to reflect that
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseExited ( MouseEvent e ) {
      mouseOver = false ;
      repaint() ;
   }




   /*------------------------------------------------------------*/
   /* mousePressed
    */
   /**
    *    This method will be called if the
    *  mouse is pressed inside of our
    *  control
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mousePressed ( MouseEvent e ) {
      if ( e.getX() < fontWidth+imageWidth ) {
         if ( !mouseDown ) {
            setCursor( Cursor.getPredefinedCursor(Cursor.HAND_CURSOR)) ;
            mouseDown = true ;
            repaint() ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* mouseReleased
    */
   /**
    *    This method will be called once the
    *  mouse has been pressed and is now
    *  being released.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseReleased ( MouseEvent e ) {
      mouseDown = false ;

      if ( ( enabled ) && ( mouseOver )  ) {
         new Thread ( this, THREAD_FIREACTION ).start() ;
         if ( autoSelect )
            setSelected ( true ) ;
      }

   }




   /*------------------------------------------------------------*/
   /* mouseClicked
    */
   /**
    *    This method will be called if the
    *  mouse is pressed and released .
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseClicked ( MouseEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* mouseMoved
    */
   /**
    *    This method will be called when
    *  the mouse is moved within our
    *  component.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseMoved ( MouseEvent e ) {
      if ( e.getX() < (fontWidth+imageWidth) ) {

         if ( !mouseOver ) {
            mouseOver = true ;
            setCursor( Cursor.getPredefinedCursor(Cursor.HAND_CURSOR)) ;
            repaint() ;
         }
      }

      else if ( mouseOver ) {
         setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
         mouseOver = false ;
         repaint() ;
      }
   }





   /*------------------------------------------------------------*/
   /* mouseDragged
    */
   /**
    *    This method will be called if the
    *  mouse is dragged while it is inside
    *  of our component.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseDragged ( MouseEvent e ) {
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run certain opertaions
    *  in a thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_FIREACTION                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_FIREACTION ) )
         fireActionEvent () ;
   }





   /*------------------------------------------------------------*/
   /* fireActionEvent
    */
   /**
    *    This method will fire an action event
    *  to all registered listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void fireActionEvent () {
      if ( listenerVec != null ) {
         ActionEvent event = null ;
         int         size  = listenerVec.size() ;
         int         i     = 0 ;

         if ( actionCommand == null )
            event = new ActionEvent ( this, ActionEvent.ACTION_FIRST, text, 0 ) ;
         else
            event = new ActionEvent ( this, ActionEvent.ACTION_FIRST, actionCommand, 0 ) ;

         while ( i < size ) {
            ((ActionListener)listenerVec.elementAt(i)).actionPerformed ( event ) ;
            i++ ;
         } /* while */

      } /* if */

   } /* fireActionEvent */





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will simply return the
    *  minimum size
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize() {
      return( getMinimumSize() ) ;
   }

   public Dimension getMaximumSize () {
      return( getMinimumSize() ) ;
   }




   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will determine the minimum
    *  amount of space that this control actually
    *  needs to take up
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize() {
      minSize.height = fontHeight ;
      if ( fontWidth == 0 )
         setText ( text ) ;
      
      minSize.width  = fontWidth+3+imageWidth+(4*text.length()) ;

      if ( imageIcon != null ) {
         int imageHeight = imageIcon.getImage().getHeight ( this )+4 ;

         if ( imageHeight > minSize.height )
            minSize.height = imageHeight ;

      }
      return( minSize ) ;
   }





   /*------------------------------------------------------------*/
   /* setIcon
    */
   /**
    *    This method will set the icon that
    *  is being displayed
    *  
    *
    *  @param imageIcon The ImageIcon to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIcon ( ImageIcon imageIcon ) {
      this.imageIcon = imageIcon ;
      if ( imageIcon != null ) {
         imageWidth = imageIcon.getImage().getWidth   ( this ) ;
         imageHeight = imageIcon.getImage().getHeight ( this ) ;
      } else {
         imageWidth  = 0 ;
         imageHeight = 0 ;
      }
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* setActionCommand
    */
   /**
    *    This method will set the string that is
    *  passed to any ActionEvent listeners when
    *  this HotLinkLabel is pressed
    *
    *  @param actionCommand The new action command
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setActionCommand ( String actionCommand ) {
      this.actionCommand = actionCommand ;
   }





   /*------------------------------------------------------------*/
   /* getText
    */
   /**
    *    This method will return the text that
    *  is being displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getText () {
      return( text ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon that
    *  is being displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( imageIcon ) ;
   }




   /*------------------------------------------------------------*/
   /* getActionCommand
    */
   /**
    *    This method will return the action command
    *  associated to this HotLinkLabel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getActionCommand () {
      return( actionCommand ) ;
   }




   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable
    *  the hot link label
    *  
    *
    *  @param enable TRUE = allow user interaction, FALSE = do not allow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enable ) {
      this.enabled = enable ;
      repaint() ;
   }





   /*------------------------------------------------------------*/
   /* setId
    */
   /**
    *    This method will simply set an Id that may
    *  be used by the programmer. Please keep in
    *  mind that this Id is not used by this control
    *
    *  @param id The ID to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setId ( int id ) {
      buttonId = id ;
   }




   /*------------------------------------------------------------*/
   /* getId
    */
   /**
    *    This method will return the Id that this
    *  control has been holding
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getId ( ) {
      return( buttonId ) ;
   }





   /*------------------------------------------------------------*/
   /* setData
    */
   /**
    *    This method will set a programmable data
    *  object to this button. This data object can
    *  be retrieved through the 'getData' method. This 'Data' object can be anything
    *
    *  @param data The Data Object to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setData ( Object data ) {
      this.data = data ;
   }




   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data object that has
    *  been set on this HotLinkLabel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( data ) ;
   }





   /*------------------------------------------------------------*/
   /* setSelected
    */
   /**
    *    This method will set the selected state of
    *  this control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelected ( boolean selected ) {
      this.selected = selected ;
      repaint() ;
   }




   /*------------------------------------------------------------*/
   /* getSelected
    */
   /**
    *    This method will return the selected
    *  state
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getSelected () {
      return( selected ) ;
   }





   /*------------------------------------------------------------*/
   /* setSelectedForeground
    */
   /**
    *    This method will set the selected foreground
    *  color
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelectedForeground ( Color color ) {
      selectedForeground = color ;
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* setSelectedBackground
    */
   /**
    *    This method will set the background color
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelectedBackground ( Color color ) {
      selectedBackground = color ;
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* showSelection
    */
   /**
    *    This method will turn on/off the selection
    *  state of the HotLinkLabel
    *  
    *
    *  @param true show selection, false = do not show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showSelection ( boolean showSelection ) {
      this.showSelection = showSelection ;
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* setAutoSelect
    */
   /**
    *    This method will true auto-selecting on or off. If
    *  turned off, no action performed or repaint will
    *  happen when the control is selected
    *
    *  @param autoSelect TRUE = Select and send the actionPerformed when selected, FALSE = do not 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAutoSelect ( boolean autoSelect ) {
      this.autoSelect = autoSelect ;
   }

} /* HotLinkLabel */
