package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   InfoBoxDlg                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: InfoBoxDlg                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.ActionListener ;
import java.awt.event.ActionEvent ;
import com.ibm.appsure.app.shared.util.* ;
/**
 * This Dialog will show an InfoBoxPanel. 
 *
 *
 * @version 1.0, 12/11/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   InfoBoxDlg                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class InfoBoxDlg extends JDialog implements ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private InfoBoxPanel infoBoxPanel         = null ;
   private boolean      returnCode           = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a default InfoBox
    *  with a description
    *  
    *
    *  @param parent The parent frame of this dialog
    *  @param title The Title of this InfoBox
    *  @param description The Description to show
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxDlg ( Frame parent, String title, String description ) {
      super ( parent, title, true ) ;
      createControls() ;
      setDescription ( description ) ;
   }
   public InfoBoxDlg ( Dialog parent, String title, String description ) {
      super ( parent, title, true ) ;
      createControls() ;
      setDescription ( description ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a default InfoBox
    *  with a description and an image
    *  
    *
    *  @param parent The parent frame of this dialog
    *  @param title The Title of this InfoBox
    *  @param description The Description to show
    *  @param image The image to show
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxDlg ( Frame parent, String title, String description, ImageIcon image ) {
      super ( parent, title, true ) ;
      createControls() ;
      setDescription ( description ) ;
      setImageIcon ( image ) ;
   }
   public InfoBoxDlg ( Dialog parent, String title, String description, ImageIcon image ) {
      super ( parent, title, true ) ;
      createControls() ;
      setDescription ( description ) ;
      setImageIcon ( image ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a default InfoBox
    *  with a description and an image
    *  
    *
    *  @param parent The parent frame of this dialog
    *  @param title The Title of this InfoBox
    *  @param description The Description to show
    *  @param image The image to show
    *  @param iconType This determines what buttons to show. see setIconType
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxDlg ( Frame parent, String title, String description, ImageIcon image, int iconType ) {
      super ( parent, title, true ) ;
      createControls() ;
      setDescription ( description ) ;
      setImageIcon ( image ) ;
      setIconType ( iconType ) ;
   }
   public InfoBoxDlg ( Dialog parent, String title, String description, ImageIcon image, int iconType ) {
      super ( parent, title, true ) ;
      createControls() ;
      setDescription ( description ) ;
      setImageIcon ( image ) ;
      setIconType ( iconType ) ;
   }







   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will return what button the user
    *  has pressed. This method will block execution
    *  until the user closes the window, or pushes a button
    *
    *  @param dispose TRUE = dispose the dialog once the user hits a button,
    *                 FALSE = Just hide the dialog
    *  @param None
    *  @param None
    *
    *  @return TRUE = User pressed Yes or Ok, FALSE = User pressed No or Cancel
    *//*---------------------------------------------------------*/
   public boolean getResult ( boolean dispose ) {
      updateDisplay() ;
      setVisible ( true ) ;
      if ( dispose )
         dispose() ;
      return( returnCode ) ;
   }




   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {

      /* Create our controls */
      infoBoxPanel = new InfoBoxPanel () ;

      /* Add our controls */
      getContentPane().add ( infoBoxPanel ) ;

      infoBoxPanel.addActionListener ( this ) ;

      setResizable ( false ) ;
   }





   /*------------------------------------------------------------*/
   /* setDescription
    */
   /**
    *    This method will set the description that
    *  will show
    *  
    *
    *  @param description The new description
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescription ( String description ) {
      infoBoxPanel.setDescription ( description ) ;
      updateDisplay() ;
   }




   /*------------------------------------------------------------*/
   /* setImageIcon
    */
   /**
    *    This method will update the image that is
    *  being displayed. Please note that it will
    *  not be visually updated until you call updateDisplay
    *
    *  @param image The image to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setImageIcon ( ImageIcon image ) {
      infoBoxPanel.setImage ( image ) ;
   }





   /*------------------------------------------------------------*/
   /* updateDisplay
    */
   /**
    *    This method must be called to show any
    *  changes to an image, description or button
    *  text
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void updateDisplay () {
      infoBoxPanel.updateDisplay() ;
      pack() ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method is subclassed so that the window
    *  auto-sizes to encompass the entire InfoBoxPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      super.doLayout() ;
      //WinUtil.centerWindow ( this ) ;
   }


   /*------------------------------------------------------------*/
   /* setYes
    */
   /**
    *    This method will set the text that is displayed
    *  in the yes button
    *  
    *
    *  @param yesStr The Yes Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setYes ( String yesStr ) {
      infoBoxPanel.setYes ( yesStr ) ;
   }


   /*------------------------------------------------------------*/
   /* setCancel
    */
   /**
    *    This method will set the text that is displayed
    *  in the cancel button
    *  
    *
    *  @param cancelStr The Cancel Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCancel ( String cancelStr ) {
      infoBoxPanel.setCancel ( cancelStr ) ;
   }




   /*------------------------------------------------------------*/
   /* setIconType
    */
   /**
    *    This method will set what type of icon
    *  we will use
    *  
    *
    *  @param iconType ( InfoBoxPanel.ICON_ERROR, InfoBoxPanel.ICON_INFO, InfoBoxPanel.ICON_QUESTION )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIconType ( int iconType ) {
      infoBoxPanel.setIconType ( iconType ) ;
   }


   /*------------------------------------------------------------*/
   /* setOk
    */
   /**
    *    This method will set the text that is displayed
    *  in the ok button
    *  
    *
    *  @param okStr The Ok Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOk ( String okStr ) {
      infoBoxPanel.setNo ( okStr ) ;
   }


   /*------------------------------------------------------------*/
   /* setNo
    */
   /**
    *    This method will set the text that is displayed
    *  in the no button
    *  
    *
    *  @param noStr The No Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNo ( String noStr ) {
      infoBoxPanel.setNo ( noStr ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  we have decided to listen to is fired
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      returnCode = (infoBoxPanel.actionEventToBoolean(e)==InfoBoxPanel.ACTION_OK) ;
      if ( !returnCode ) {
         returnCode = (infoBoxPanel.actionEventToBoolean(e)==InfoBoxPanel.ACTION_YES) ;
      }
      setVisible ( false ) ;
   }

} /* InfoBoxDlg */

