package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   InfoBoxPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: InfoBoxPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.awt.event.ActionListener ;
import com.ibm.appsure.app.shared.util.* ;
/**
 * This subclass of Object holds an icon, a label and three buttons. The 
 * image, text and buttons that are displayed are configured using the
 * ICON_XXX type. The configuration is as follows:
 *
 * Icon Type      Ok    Yes    No   
 * -------------- ----- ------ -----
 * ICON_INFO      X
 * ICON_ERROR     X
 * ICON_QUESTION        X      X
 * 
 * The default icon type is ICON_INFO
 *
 * Use the addControls(Container) method to add controls to your container
 *
 * Please note. The setDescription, setIconType etc.. methods will not
 *              update anything visually. Please call updateDisplay() to
 *              update the display. This includes any constructors that
 *              are passed items to update
 *
 * @version 1.0, 12/11/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   InfoBoxPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class InfoBoxPanel extends JPanel {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */   
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int ACTION_UNDEFINED        = 0 ;
   public static final int ACTION_OK               = 1 ;
   public static final int ACTION_CANCEL           = 2 ;
   public static final int ACTION_YES              = 3 ;
   public static final int ACTION_NO               = 4 ;

   public static final int ICON_INFO      = 1 ;
   public static final int ICON_QUESTION  = 2 ;
   public static final int ICON_ERROR     = 3 ;
   

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton   pb_OK                  = null ;
   private DButton   pb_CANCEL              = null ;
   private JLabel    st_DESCRIPT            = null ;
   private JLabel    st_IMAGE               = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String           description             = null ;
   private int              iconType                = ICON_INFO ;
   private ImageIcon        imageIcon               = null ;
   private String           str_OK                  = "Ok" ;
   private String           str_CANCEL              = "Cancel" ;
   private String           str_NO                  = "No" ;
   private String           str_YES                 = "Yes" ;
   private InfoBoxLayoutMgr layoutMgr               = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a blank
    *  InfoBoxPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxPanel () {
      createControls() ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a default panel with a description
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxPanel ( String description ) {
      setDescription ( description ) ;
      createControls() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create an InfoBoxPanel that will
    *  contain a description and an image
    *  
    *
    *  @param description The Description to show
    *  @param image The image to show
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InfoBoxPanel ( String Description, Image image ) {
      setDescription ( description ) ;
      setImage ( image ) ;
      createControls() ;
   }



   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our
    *  controls to the InfoLayoutMgr.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {


      /* First, create our layout manager */
      setLayout ( layoutMgr = new InfoBoxLayoutMgr() )  ;

      /* Now, create all of our controls */
      add ( "PB_OK",       pb_OK       = new DButton ( str_OK ) ) ;
      add ( "PB_CANCEL",   pb_CANCEL   = new DButton ( str_CANCEL ) ) ;
      add ( "ST_DESCRIPT", st_DESCRIPT = new JLabel (  ) ) ;
      add ( "ST_IMAGE",    st_IMAGE    = new JLabel    ( "" ) ) ;

      /* Create our image icon for the st_IMAGE */
      //imageIcon = new ImageIcon () ;

      /* Set some properties on our controls */
      st_IMAGE.setVerticalAlignment ( SwingConstants.TOP ) ;

   }




   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener to
    *  each of our buttons
    *  
    *
    *  @param listener The ActionListener to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      pb_OK.addActionListener ( listener ) ;
      pb_CANCEL.addActionListener ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* setDescription
    */
   /**
    *    This method will set what description
    *  we are currently using
    *  
    *
    *  @param description The new description to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescription( String description ) {
      this.description = description ;
   }


   /*------------------------------------------------------------*/
   /* setIconType
    */
   /**
    *    This method will set what icon type is currently
    *  being used
    *  
    *
    *  @param iconType ( ICON_ERROR, ICON_INFO or ICON_QUESTION )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIconType ( int iconType ) {
      this.iconType = iconType ;
   }




   /*------------------------------------------------------------*/
   /* setImage
    */
   /**
    *    This method will set what image we are
    *  currently displaying on the window
    *  
    *
    *  @param image The new image
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setImage ( Image image ) {
      if ( image != null )
         imageIcon.setImage ( image ) ;
   }





   /*------------------------------------------------------------*/
   /* setImage
    */
   /**
    *    This method will set what image we are
    *  currently displaying
    *  
    *
    *  @param imageIcon The ImageIcon to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setImage ( ImageIcon image ) {
     if ( image != null )
         this.imageIcon = image ;
   }





   /*------------------------------------------------------------*/
   /* updateDisplay
    */
   /**
    *    This method will update our display. This
    *  includes the button text, description and icons.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void updateDisplay () {

      /* First, depending on which ICON_XXX Type, show
         buttons */
      switch ( iconType ) {
         
         case ICON_QUESTION :
            pb_OK.setText ( str_YES ) ;
            pb_CANCEL.setText ( str_NO ) ;
            pb_CANCEL.setVisible ( true ) ;
            break ;

         default:
            pb_OK.setText ( str_OK ) ;
            pb_CANCEL.setVisible ( false ) ;
            break ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, add our HTML around the description                 */                              
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_DESCRIPT.setText ( "<html><body><font size=3 face=\"Arial, Helvetica\">"+description+"</body></html>" ) ;

      if ( imageIcon != null )
         st_IMAGE.setIcon ( imageIcon ) ;
      invalidate() ;
   }




   /*------------------------------------------------------------*/
   /* setYes
    */
   /**
    *    This method will set the text that is displayed
    *  in the yes button
    *  
    *
    *  @param yesStr The Yes Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setYes ( String yesStr ) {
      this.str_YES = yesStr ;
   }


   /*------------------------------------------------------------*/
   /* setNo
    */
   /**
    *    This method will set the text that is displayed
    *  in the no button
    *  
    *
    *  @param noStr The No Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNo ( String noStr ) {
      this.str_NO = noStr ;
   }

   /*------------------------------------------------------------*/
   /* setOk
    */
   /**
    *    This method will set the text that is displayed
    *  in the ok button
    *  
    *
    *  @param okStr The Ok Str
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOk ( String okStr ) {
      this.str_OK = okStr ;
   }


   /*------------------------------------------------------------*/
   /* setCancel
    */
   /**
    *    This method will set the text that is displayed
    *  in the cancel button
    *  
    *
    *  @param cancelStr The Cancel Str
    *
    *  @return Cancelthing
    *//*---------------------------------------------------------*/
   public void setCancel ( String cancelStr ) {
      this.str_CANCEL = cancelStr ;
   }

   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will simply return what our
    *  layout manager says our size should be
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( layoutMgr.minimumLayoutSize ( this ) ) ;
   }
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }
   public Dimension getMaximumSize () {
      return( getPreferredSize() ) ;
   }





   /*------------------------------------------------------------*/
   /* getAction
    */
   /**
    *    This method will return either ACTION_UNDEFINED, ACTION_OK,
    *  ACTION_CANCEL, ACTION_YES or ACTION_NO depending on what the user
    *  hits. The following table shows what value is returned for what
    *  button
    *
    *  Button     ICON_ERROR/ICON_INFO   ICON_QUESTION
    *  ---------- ---------------------- ----------------------------
    *  Ok         ACTION_OK              ACTION_YES
    *  Cancel     ACTION_CANCEL          ACTION_NO
    *
    *
    *  @param e The ActionEvent to analyze. This action event should have been fired
    *           from one of the buttons on the InfoBoxPanel. Use the addActionListeners
    *           method to receive these notifications
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int actionEventToBoolean ( ActionEvent e ) {
      int rc = ACTION_UNDEFINED ;

      if ( e.getSource() == pb_OK )
         rc = ACTION_OK ;
      if ( e.getSource() == pb_CANCEL ) 
         rc = ACTION_CANCEL ;

      if ( iconType == ICON_QUESTION )
         rc+=2 ;

      return( rc ) ;
   }
} /* InfoPanel */
