package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   ListDlg                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ListDlg                             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.appsure.app.shared.gui.* ;
import com.ibm.appsure.app.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
/**
 * This simple window displays a vector of information in a MultiList. 
 * It will not return the item selected currently.
 *
 * @version 1.0, 3/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ListDlg                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class ListDlg extends JDialog implements ActionListener {              

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private, Static Members                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] defaultTitle = {""};


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Members                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton      pb_OK       = null ;
   private DButton      pb_CANCEL   = null ;
   private MultiList    cnr_DATA    = null ;
   private ButtonPanel  buttonPanel = null ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector    selVec      = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create, and initialize the window.
    *  It will return immediatly. To show the window please
    *  call getResult.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ListDlg ( Frame parent, String title, Vector dataList, FontUtil fontUtil ) {
      super ( parent, title, true ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK       = new DButton ( "Ok" ) ;
      pb_CANCEL   = new DButton ( "Cancel" ) ;
      cnr_DATA    = new MultiList ( fontUtil ) ;
      buttonPanel = new ButtonPanel () ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_DATA.setColumnHeadings ( defaultTitle ) ;
      cnr_DATA.showColumnHeadings ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      Container contentPane = getContentPane() ;

      contentPane.setLayout ( null ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      contentPane.add ( cnr_DATA,    BorderLayout.CENTER ) ;
      contentPane.add ( buttonPanel, BorderLayout.SOUTH ) ;  
      pb_OK.setMinimumSize ( new Dimension ( 70, 25 ) ) ;
      pb_CANCEL.setMinimumSize ( new Dimension ( 80, 25 ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   RefreshData                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      new QuickSort ( dataList ) ;
      cnr_DATA.add ( dataList ) ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our size and position                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 300, 300 ) ;
      WinUtil.centerWindow ( this ) ;
   }






   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the window and will
    *  wait until the user selects an item.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getResult () {
      setVisible ( true ) ;

      return( selVec ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to button's being hit
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      selVec = cnr_DATA.getSelection() ;
      super.dispose() ;
   }





   /*------------------------------------------------------------*/
   /* setOkText
    */
   /**
    *    This method will set the text on the Ok Button
    *  
    *  
    *
    *  @param text The new text
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOkText ( String text ) {
      pb_OK.setText ( text ) ;
   }





   /*------------------------------------------------------------*/
   /* setCancelText
    */
   /**
    *    This method will set the text on the cancel
    *  button
    *  
    *
    *  @param text The text to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCancelText ( String text ) {
      pb_CANCEL.setText ( text ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will layout and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;
      cnr_DATA.setBounds ( 5, 5, size.width-15, size.height-80 ) ;
      buttonPanel.setBounds ( 0, size.height-60, size.width-5, 25 ) ;
   }

   public DButton getOkButton () {
      return( pb_OK ) ;
   }

   public DButton getCancelButton () {
      return( pb_CANCEL ) ;
   }
} /* ListDlg */
