package com.ibm.appsure.app.shared.gui ;                                      
/*-----------------------------------------------------------*/
/*   MultiLineTableCellRenderer                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MultiLineTableCellRenderer          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.table.* ;
import javax.swing.* ;
import java.awt.* ;
/**
 * This TableCellRenderer is used in JTables and extends 
 * the normal functionality of the DefaultTableCellRenderer. This
 * advanced functionality includes the following:<br><br>
 *
 * Multiple Lines of text in a single Cell<br>
 * <br>
 *
 * To use this renderer, you must call myTable.setDefaultRenderer ( String[].class, new MultiLineTableCellRenderer() ) ;<br>
 * 
 *
 *
 * @version 1.0, 4/4/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   MultiLineTableCellRenderer                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class MultiLineTableCellRenderer extends Component implements TableCellRenderer {      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   private members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTable  table       = null ;
   private Object  value       = null ;
   private boolean hasFocus    = false ;
   private boolean isSelected  = false ;
   private int     row         = 0 ;
   private int     column      = 0 ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default Constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MultiLineTableCellRenderer () {
   }



   /*------------------------------------------------------------*/
   /* getTableCellEditorComponent
    */
   /**
    *    This method will return an instance of this
    *  class since this class is the one responsible
    *  for providing the UI
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return this
    *//*---------------------------------------------------------*/
   public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)  {
      this.table      = table ;
      this.value      = value ;
      this.isSelected = isSelected ;
      this.row        = row ;
      this.column     = column ;
      this.hasFocus   = hasFocus ;

      return( this ) ;
   }

   public void paint ( Graphics g ) {
      String[]  stringArray      = (String[]) value ;
      int       i                = 0 ;
      int       tempStringLength = 0 ;
      int       x                = 0 ;
      int       tempY            = 10 ;

      if ( isSelected ) {
         Dimension size = getSize() ;

         if ( table != null )
            g.setColor ( table.getSelectionBackground() ) ;
         else
            g.setColor ( Color.blue ) ;
         g.fillRect ( 0, 0, size.width, size.height ) ;

         if ( table != null )
            g.setColor ( table.getSelectionForeground() ) ;
         else
            g.setColor ( Color.white ) ;
      } else {
         if ( table != null )
            g.setColor ( table.getForeground() ) ;
         else
            g.setColor ( Color.black ) ;
      }

      for ( i = 0; i < stringArray.length; i++ ) {
         tempStringLength = stringArray[i].length() ;


         if ( tempStringLength > 0 ) {

            g.drawString ( stringArray[i], x, tempY ) ;
            tempY+=18 ;
         }
      }

   }

} /* MultiLineTableCellRenderer */
