package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   MutliList                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MutliList                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The MultiList is a JPanel consisting of a JScrollPane, JList and header information. 
 * The list may contain multiple columns containing any kind of data ( text, icon, numeric etc.. ).
 * Since the JScrollPane can hold JComponents on any boundraies, you may use the JScrollPane 
 * to add custom controls to the MultiList.
 *
 * The best way to support Multiple Columns in the MultiList is to provide a drawing mechanism that
 * will draw each item. This mechanism can be provided by subclassing javax.swing.DefaultCellRenderer. In
 * the subclass, over ride the method getCellRendererComponent() to return a JComponent of your choice. Each
 * line in the list when drawn will call the JComponent's paint. Since Mutliple Columns require data to be
 * organized in columns that might not be the same width, the getColumnX() method of the MultiList may
 * be used to determine where to start drawing each column.
 *
 * The default MultiColumnCellRenderer will require a MultiListRow object that will return any column data
 * requested. Please see the MultiListRow Interface for more information.
 *
 *
 * @version 1.0, 2/5/1999 
 * @author David Lentz
 */
import javax.swing.* ;
import javax.swing.event.* ;
import javax.swing.border.* ;
import java.util.Vector ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.appsure.app.shared.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   MutliList                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class MultiList extends DPanel implements DetailList, Runnable, MouseListener, ListSelectionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private   JScrollPane         scr_PANE           = null ;
   protected JList               lb_LIST            = null ;
   private   DetailColumnHeader  titlePanel         = null ;
   private   Panel               pnl_SPLIT          = new Panel() ;
   private   DPanel              pnl_HOLDPANEL      = new DPanel() ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector               headings           = null ;
   private MultiListRowRenderer cellRenderer       = null ;
   private FontUtil             fontUtil           = null ;
   private int                  columnCount        = 0 ;
   private DefaultListModel     listModel          = null ;
   private Color                oddLinesColor      = Color.white ;
   private Vector               actionVec          = null ;
   private Vector               selectionVec       = null ;
   private int                  firstIndex         = 0 ;
   private int                  lastIndex          = 0 ;
   private boolean              isAdjusting        = false ;
   private int                  sortColumn         = -1 ;
   private int                  sortType           = SORT_ASCENDING ;
   private ListSelectionEvent   lastSelectionEvent = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates a default
    *  MultiList with no column definition and no data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MultiList ( FontUtil fontUtil ) {
      initialize(fontUtil) ;
   }

   MultiList ( Vector selectionVec, ListSelectionEvent e ) {
      this.selectionVec       = selectionVec ;
      this.lastSelectionEvent = e ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates a default MultiList
    *  with specific column headings
    *  
    *
    *  @param columnHeadings A Vector containing objects to use. Each object specifies a column
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MultiList ( FontUtil fontUtil, Vector columnHeadings ) throws NullPointerException {
      initialize(fontUtil) ;
      setColumnHeadings ( columnHeadings ) ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates a default MultiList
    *  with specific column headings
    *  
    *
    *  @param columnHeadings An array containing objects to use. Each object specifies a column
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MultiList ( FontUtil fontUtil, Object[] columnHeadings ) {
      initialize(fontUtil) ;
      setColumnHeadings ( columnHeadings ) ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a MultiList with
    *  the specified Headings and Data
    *  
    *
    *  @param columnHeadings An array containing objects to use. Each object specifies a column
    *  @param data The data to insert into the list
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MultiList ( FontUtil fontUtil, Object[] columnHeadings, Object[] data ) {
      initialize(fontUtil) ;
      setColumnHeadings ( columnHeadings ) ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a MultiList with
    *  the specified Headings and Data
    *  
    *
    *  @param columnHeadings A Vector containing objects to use. Each object specifies a column
    *  @param data The data to insert into the list
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MultiList ( FontUtil fontUtil, Vector columnHeadings, Vector data ) {
      initialize( fontUtil ) ;
      setColumnHeadings ( columnHeadings ) ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will iniitalize the MultiList
    *  control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize ( FontUtil fontUtil ) {

      this.fontUtil = fontUtil ;
      titlePanel = new DetailColumnHeader (this) ;
      listModel = new DefaultListModel() ;

      setLayout ( null ) ;
      add ( pnl_HOLDPANEL ) ;
      pnl_HOLDPANEL.setLayout ( new BorderLayout() ) ;
      pnl_HOLDPANEL.add ( scr_PANE = new JScrollPane ( lb_LIST = new DList() ), BorderLayout.CENTER ) ;
      add ( pnl_SPLIT ) ;
      pnl_SPLIT.setVisible ( false ) ;
      scr_PANE.setBorder ( new BevelBorder ( BevelBorder.LOWERED ) ) ;
      scr_PANE.getViewport().putClientProperty ("EnableWindowBlit", Boolean.TRUE);

      /* Initialize our list  */
      cellRenderer = new MultiListRowRenderer ( this, fontUtil ) ;
      lb_LIST.setCellRenderer ( cellRenderer ) ;
      lb_LIST.setModel ( listModel ) ;

      /* Adjust some properties  */
      lb_LIST.addListSelectionListener ( this ) ;
      pnl_SPLIT.setBackground ( Color.black ) ;
      setBackground ( Color.white ) ;
      lb_LIST.setBackground ( Color.white ) ;

      lb_LIST.addMouseListener ( this ) ;
   }




   /*------------------------------------------------------------*/
   /* setWordWrap
    */
   /**
    *    This method will turn word wrapping on or
    *  off. If word wrapping is turned on, an Array
    *  of Strings can be returned from a MultiList getColumnData
    *  method.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWordWrap ( boolean wordWrap ) {
      cellRenderer.setWordWrap ( wordWrap ) ;
      setRowHeight ( -1 ) ;
   }


   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;

      pnl_HOLDPANEL.setBounds ( 0, 0, size.width, size.height ) ;
   }





   /*------------------------------------------------------------*/
   /* setColumnHeadings
    */
   /**
    *    This method will set the objects that are to
    *  be used for the heading of each column.
    *  
    *
    *  @param columnHeadings A Vector containing objects to use for the headings
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnHeadings ( Vector columnHeadings ) {
      int i      = 0 ;
      int length = 0 ;

      titlePanel = new DetailColumnHeader (this) ;

      if ( columnHeadings == null ) {
         titlePanel.removeAll() ;
      } else {
         scr_PANE.setColumnHeaderView ( titlePanel ) ;

         length = columnHeadings.size() ;
         headings = columnHeadings;

         while ( i < length ) {
            titlePanel.addColumn ( columnHeadings.elementAt(i) ) ;
            i++ ;
         }

         cellRenderer.setRowWidth ( titlePanel.getPreferredSize().width ) ;
         //cellRenderer.setRowHeight ( -1 ) ;
         cellRenderer.setColumnNum ( length ) ;
         columnCount = length ;
      }

   }


   /*------------------------------------------------------------*/
   /* setColumnHeadings
    */
   /**
    *    This method will set the objects that are to
    *  be used for the heading of each column.
    *  
    *
    *  @param columnHeadings A Vector containing objects to use for the headings
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnHeadings ( Object[] columnHeadings ) {
      if ( columnHeadings != null ) {
         int      i = 0 ;
         Vector   vec = new Vector ( columnHeadings.length ) ;

         while ( i < columnHeadings.length )
            vec.addElement ( columnHeadings[i++] ) ;

         setColumnHeadings ( vec ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setColumnHeadingBorder
    */
   /**
    *    This method will set the border that appears
    *  around the heading of each column
    *  
    *
    *  @param Border The new Border
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnHeadingBorder ( javax.swing.border.Border border ) {
      titlePanel.setColumnHeadingBorder ( border ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnHeadings
    */
   /**
    *    This method will set the column headings
    *  used for each column
    *  
    *
    *  @param columnHeadings An array of objects to use for the headings
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getColumnHeadings () {
      return( headings ) ;
   }






   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    This method will return the number of
    *  columns being displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnCount () {
      if ( headings != null )
         return( headings.size() ) ;

      return( 0 ) ;
   }



   /*------------------------------------------------------------*/
   /* setData
    */
   /**
    *    This method will set the data that
    *  the list will display
    *  
    *
    *  @param data A vector containing MultiListRow objects representing the data
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setData ( Vector data ) {
      removeAll() ;
      add ( data ) ;
   }


   /*------------------------------------------------------------*/
   /* setData
    */
   /**
    *    This method will set the data that
    *  the list will display
    *  
    *
    *  @param dataArray An array containing MultiListRow objects representing the data
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setData ( Object[] dataArray ) {
      removeAll() ;
      if ( dataArray != null ) {
         add ( dataArray ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getColumnX
    */
   /**
    *    This method will return the x coordinate in
    *  pixels of the column passed
    *  
    *
    *  @param column The column to return the x coordinate for
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnX ( int column ) {
      return( titlePanel.getColumnX(column) ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnWidth
    */
   /**
    *    This method will return the current width
    *  of a column
    *  
    *
    *  @param column The column number to return the width of
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getColumnWidth ( int column ) {
      return( titlePanel.getColumnWidth ( column ) ) ;
   }




   /*------------------------------------------------------------*/
   /* setColumnWidth
    */
   /**
    *    This method will set a column's width in
    *  pixels.
    *  
    *
    *  @param columnNumber The column number to adjust
    *  @param width The width in pixels
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnWidth ( int columnNum, int width ) {
      titlePanel.setColumnWidth ( columnNum, width ) ;                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, if we have any ExtendedMultiListRows in,           */
      /* we must tell them the new column width                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshExtendedColumnWidths ( columnNum, width ) ;


      refreshColumns() ;
   }





   /*------------------------------------------------------------*/
   /* refreshExtendedColumnWidths
    */
   /**
    *    This method will notify each ExtendedMultiListRow
    *  in our data set that the column number specified
    *  is now at the pixel width specificed
    *
    *  @param column The column
    *  @param pixelWidth The width in pixels
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshExtendedColumnWidths ( int columnNum, int width ) {
      int i    = 0 ;
      int size = listModel.size() ;

      while ( i < size ) {
         if ( listModel.getElementAt(i) instanceof ExtendedMultiListRow )
            ((ExtendedMultiListRow)listModel.getElementAt(i)).columnSized ( columnNum, width ) ;
         i++ ;
      }
   }



   /*------------------------------------------------------------*/
   /* refreshExtendedColumnWidths
    */
   /**
    *    This method will notify each ExtendedMultiListRow
    *  in our data set to refresh it's column width
    *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshExtendedColumnWidths ( ) {
      try {
         int i    = 0 ;
         int size = listModel.size() ;
         int headIndex = 0 ;
         int headSize  = headings.size() ;

         while ( i < size ) {
            if ( listModel.getElementAt(i) instanceof ExtendedMultiListRow ) {
               headIndex = 0 ;

               while ( headIndex < headSize ) {
                  ((ExtendedMultiListRow)listModel.getElementAt(i)).columnSized ( headIndex, titlePanel.getColumnWidth ( headIndex ) ) ;
                  headIndex++ ;
               }
            }
            i++ ;
         }
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* refreshColumns
    */
   /**
    *    This method will refresh and reposition all
    *  of the columns in the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshColumns() {
      invalidate() ;
      scr_PANE.invalidate() ;
      scr_PANE.revalidate() ;
      revalidate() ;
      scr_PANE.repaint() ;
      cellRenderer.setRowWidth ( titlePanel.getPreferredSize().width ) ;
      cellRenderer.setColumnNum ( columnCount ) ;
      lb_LIST.setFixedCellWidth ( titlePanel.getPreferredSize().width ) ;
   }

   public void showDivider ( int x ) {
      pnl_SPLIT.setVisible ( true ) ;
      pnl_SPLIT.setBounds ( x, 0, 2, height ) ;

   }
   public void hideDivider () {
      pnl_SPLIT.setVisible ( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getJList
    */
   /**
    *    This method will return the JList that is
    *  displaying the information
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JList
    *//*---------------------------------------------------------*/
   public JList getJList() {
      return( lb_LIST ) ;
   }


   public void setBorder ( javax.swing.border.Border border ) {
      if ( scr_PANE != null )
         scr_PANE.setBorder ( border ) ;
   }





   /*------------------------------------------------------------*/
   /* setRowHeight
    */
   /**
    *    This method will set how high each row is
    *  
    *  
    *
    *  @param height The height in pixels
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRowHeight ( int height ) {
      lb_LIST.setFixedCellHeight ( height ) ;
      if ( cellRenderer != null )
         cellRenderer.setRowHeight ( height ) ;
   }





   /*------------------------------------------------------------*/
   /* getJScrollPane
    */
   /**
    *    This method will return the instance of the
    *  JScroll pane that this multi list is using
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JScrollPane
    *//*---------------------------------------------------------*/
   public JScrollPane getJScrollPane() {
      return( scr_PANE ) ;
   }




   /*------------------------------------------------------------*/
   /* setBackground
    */
   /**
    *    This method will set the background of
    *  the list to the color specified
    *  
    *
    *  @param color The background color
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBackground ( Color color ) {
      super.setBackground ( color ) ;

      try {
         if ( lb_LIST != null )
            lb_LIST.setBackground ( color ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }





   /*------------------------------------------------------------*/
   /* setBackground
    */
   /**
    *    This method will set the background
    *  to the image specified. The image will be
    *  titled across the list
    *
    *  @param image The image to use as a background
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBackground ( Image image ) {
      cellRenderer.setBackground ( image ) ;
      pnl_HOLDPANEL.setBackground ( image ) ;
      lb_LIST.setOpaque ( false ) ;
   }





   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will add a single object to the
    *  bottom of the list
    *  
    *
    *  @param object The object to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void add ( MultiListRow object ) {

      if ( sortColumn != -1 )
         listModel.add ( sortedIndex ( (MultiListSort)object ), object ) ;
      else
         listModel.add ( listModel.getSize(), object  ) ;

   }





   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will add an entire vector
    *  to the end of the list. If the list is sorted, each
    *  element in the vector will be added into it's correctly
    *  sorted place.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void add ( Vector vec ) {
      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int         i      = 0 ;
         int         length = vec.size() ;
         Object[]    dataArray = null ;
         Object[]    curArray  = null ;               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Sort the list if need be                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( sortColumn >= 0 ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, we must get the current data and new data into   */
            /*   a single list.                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            curArray = listModel.toArray() ;

            /* If we have current data, add the arrays together */
            if ( curArray != null ) {
               dataArray = new Object [curArray.length+vec.size() ] ;

               while ( i < length ) {
                  dataArray[i+curArray.length]=vec.elementAt(i) ;
                  i++ ;
               }
            }

            /* If we don't have any data, just copy the new data in */
            else {
               dataArray = new Object [ vec.size() ] ;
               vec.copyInto ( dataArray ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Sort the list                                           */    
            /*                                                           */
            /*-----------------------------------------------------------*/
            new MultiListSorter ( dataArray, sortColumn, sortType ) ;           

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add the data back in                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            i=0 ;
            removeAll() ;
            listModel.ensureCapacity ( dataArray.length ) ;
            while ( i < dataArray.length ) {
               listModel.add ( i, dataArray[i] ) ;
               i++ ;
            }

         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If no sort, just add the items to the list              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            int listModelSize = 0 ;

            listModel = new DefaultListModel() ;

            listModel.ensureCapacity ( listModel.getSize()+length ) ;

            while ( i < length ) {
               listModel.add ( listModelSize++, vec.elementAt(i) ) ;
               i++ ;
            } /* while */
            lb_LIST.setModel ( listModel ) ;
            scr_PANE.getHorizontalScrollBar ().setValue ( 0 ) ;
         } /* else */

      } /* if vec is good */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now adjust the width on all columns for any         */
      /* ExtendedMultiListRow                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshExtendedColumnWidths () ;
      invalidate() ;
      lb_LIST.invalidate() ;
      scr_PANE.invalidate() ;
      revalidate() ;


   }


   /*------------------------------------------------------------*/
   /* selectAll
    */
   /**
    *    This method will either select or deselect everyting
    *  
    *  
    *
    *  @param select TRUE = select Everything, FALSE = deselect
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectAll ( boolean select ) {
      if ( select ) {
         int[] index = new int[listModel.getSize()] ;

         for ( int i = 0; i < index.length ; i++ )
            index[i]=i ;

         lb_LIST.setSelectedIndices ( index ) ;
      } else
         lb_LIST.clearSelection() ;
   }




   /*------------------------------------------------------------*/
   /* remove
    */
   /**
    *    This method will remove the object
    *  specified.
    *  
    *
    *  @param object The Object to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void remove ( MultiListRow object ) {
      if ( object != null )
         listModel.removeElement ( object ) ;
   }





   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the vector that is
    *  displaying in the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getData () {
      Vector   vec       = new Vector ( 10, 1 ) ;
      Object[] dataArray = listModel.toArray() ;
      int      i         = 0 ;

      if ( dataArray != null ) {
         while ( i < dataArray.length ) {
            vec.addElement ( dataArray[i] ) ;
            i++ ;
         }
      }

      return( vec ) ;
   }





   /*------------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove everything
    *  from the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAll () {
      listModel.removeAllElements() ;
      listModel = new DefaultListModel() ;
      listModel.ensureCapacity ( 1000 ) ;
      lb_LIST.setModel ( listModel ) ;
   }





   /*------------------------------------------------------------*/
   /* getSelection
    */
   /**
    *    This method will return a vector of
    *  selected values
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getSelection () {
      Object[] obj = lb_LIST.getSelectedValues() ;
      Vector   retVec = new Vector ( obj.length ) ;
      int i = 0 ;

      while ( i < obj.length ) {
         retVec.addElement ( obj[i] ) ;
         i++ ;
      }
      return( retVec ) ;
   }



   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will add an array of objects
    *  to the end of the list
    *  
    *
    *  @param objectArray The array of objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void add ( Object[] objectArray ) {
      if ( objectArray != null ) {
         int i = 0 ;
         Vector vec = null ;

         vec = new Vector ( 10, 10 ) ;

         while ( i < objectArray.length ) {
            vec.addElement ( objectArray[i] ) ;
            i++ ;
         }
         add ( vec ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* addPanel
    */
   /**
    *    This method will add another Panel to either the
    *  left, right, top or bottom of the list. 
    *  
    *
    *  @param component The panel to add
    *  @param location The location, either BorderLayout.WEST, BorderLayout.EAST, BorderLayout.NORTH, BorderLayout.SOUTH
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addPanel ( Component component, String location ) {
      if ( !location.equals ( BorderLayout.CENTER ) )
         pnl_HOLDPANEL.add ( component, location ) ;
   }




   /*------------------------------------------------------------*/
   /* removePanel
    */
   /**
    *    This method will remove the panel
    *  that was added through the 'addPanel'
    *  method
    *
    *  @param component The panel to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removePanel ( Component component ) {
      pnl_HOLDPANEL.remove ( component ) ;
   }





   /*------------------------------------------------------------*/
   /* removeSelectedItems
    */
   /**
    *    This method will simply remove all items
    *  that are selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeSelectedItems () {
      Vector vec = getSelection() ;

      if ( vec != null ) {
         int i    = 0 ;
         int size = vec.size() ;

         while ( i < size ) {
            listModel.removeElement ( vec.elementAt(i) ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* showColumnHeadings
    */
   /**
    *    This method will either show or hide
    *  the column headings
    *  
    *
    *  @param show = TRUE or FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showColumnHeadings ( boolean show ) {
      titlePanel.setShowing ( show ) ;
   }





   /*------------------------------------------------------------*/
   /* setSelectedItem
    */
   /**
    *    This method will set the currently selected
    *  item
    *  
    *
    *  @param data The data object to set
    *  @param select TRUE = set selected, FALSE = set not selected
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelectedItem ( Object data, boolean select ) {
      lb_LIST.setSelectedValue ( data, select ) ;
   }





   /*------------------------------------------------------------*/
   /* setSelectedIndices
    */
   /**
    *    This method will select all of the indexes
    *  passed in
    *  
    *
    *  @param selIndex The array of indexes to select
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelectedIndices ( int[] index ) {
      lb_LIST.setSelectedIndices ( index ) ;
   }





   /*------------------------------------------------------------*/
   /* getSelectedItem
    */
   /**
    *    This method will return the currently
    *  selected item
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getSelectedItem () {
      return( lb_LIST.getSelectedValue () ) ;
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    Fires events in a thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      if ( operation.equals ( "SELECTION" ) ) {
         fireSelectionEvent () ;
      } else if ( operation.equals ( "ACTION" ) )
         fireActionEvent () ;

   }





   /*------------------------------------------------------------*/
   /* fireActionEvent
    */
   /**
    *    This method will fire an action event to all
    *  action listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void fireActionEvent () {
      if ( actionVec != null ) {
         ActionEvent actionEvent = null ;
         int         i           = 0 ;
         int         length      = actionVec.size() ;

         actionEvent = new ActionEvent ( this, ActionEvent.ACTION_PERFORMED, "" ) ;

         while ( i < length )
            ((ActionListener)actionVec.elementAt(i++)).actionPerformed ( actionEvent ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* fireSelectionEvent
    */
   /**
    *    This method will fire an action event to all
    *  action listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void fireSelectionEvent () {
      if ( selectionVec != null ) {
         int                i           = 0 ;
         int                length      = selectionVec.size() ;
         ListSelectionEvent event       = null ;

         event = new ListSelectionEvent ( this, 
                                          lastSelectionEvent.getFirstIndex(), 
                                          lastSelectionEvent.getLastIndex(), 
                                          lastSelectionEvent.getValueIsAdjusting() ) ;
         while ( i < length ) {
            ((ListSelectionListener)selectionVec.elementAt(i++)).valueChanged ( event ) ;
         }
      }
   }







   /*------------------------------------------------------------*/
   /* addListSelectionListener
    */
   /**
    *    This method will add an action listener
    *  to the list
    *  
    *
    *  @param actionListener The new ListSelectionListener
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addListSelectionListener ( ListSelectionListener actionListener ) {
      if ( selectionVec == null )
         selectionVec = new Vector ( 1, 1 ) ;

      selectionVec.addElement ( actionListener ) ;
   }





   /*------------------------------------------------------------*/
   /* removeListSelectionListener
    */
   /**
    *    This method will remove an action listener
    *  from the list
    *  
    *
    *  @param actionListener The ListSelectionListener to remove
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeListSelectionListener ( ListSelectionListener actionListener ) {
      if ( actionVec != null )
         actionVec.removeElement ( actionListener ) ;
   }


   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener
    *  to the list
    *  
    *
    *  @param actionListener The new ActionListener
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener actionListener ) {
      if ( actionVec == null )
         actionVec = new Vector ( 1, 1 ) ;

      actionVec.addElement ( actionListener ) ;
   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    Called when a value in the list has changed
    *  
    *  
    *
    *  @param listSelectionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged(ListSelectionEvent e) {      
      this.lastSelectionEvent = e ;

      new Thread ( this, "SELECTION" ).start() ;
   }



   /*------------------------------------------------------------*/
   /* removeActionListener
    */
   /**
    *    This method will remove an action listener
    *  from the list
    *  
    *
    *  @param actionListener The ActionListener to remove
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeActionListener ( ActionListener actionListener ) {
      if ( actionVec != null )
         actionVec.removeElement ( actionListener ) ;
   }



   public void mouseClicked ( MouseEvent e ) {
   }
   public void mousePressed ( MouseEvent e ) {}
   public void mouseReleased ( MouseEvent e ) {
      //if ( e.getClickCount() == 1 )
      //   new Thread ( this, "SELECTION" ).start() ;
      //fireSelectionEvent() ;
      if ( e.getClickCount() > 1 ) {
         if ( actionVec != null )
            new Thread ( this, "ACTION" ).start() ;
         //fireActionEvent() ;
      }

   }
   public void mouseEntered ( MouseEvent e ) {}
   public void mouseExited ( MouseEvent e ) {}





   /*------------------------------------------------------------*/
   /* containsItem
    */
   /**
    *    This method will determine if this list
    *  contains the item specified
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean containsItem ( Object object ) {
      boolean rc = false ;
      Object[] dataArray = listModel.toArray() ;

      if ( dataArray != null ) {
         int i = 0 ;

         while ( ( !rc ) && ( i < dataArray.length ) ) {
            if ( dataArray[i] != null )
               rc = dataArray[i].equals ( object ) ;
            i++ ;
         }
      }
      return( rc );
   }





   /*------------------------------------------------------------*/
   /* setSort
    */
   /**
    *    This method will set what type of
    *  sort, SORT_ASCENDING or SORT_DESCENDING
    *  
    *
    *  @param sortType The type of sort
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSort ( int sortType ) {
      if ( ( sortType == SORT_ASCENDING ) || ( sortType == SORT_DESCENDING ) )
         this.sortType = sortType ;
   }




   /*------------------------------------------------------------*/
   /* switchSortType
    */
   /**
    *    This method will switch the sort type
    *  from ascending to descending, or 
    *  from descending to ascending. PLEASE NOTE, this method will
    *  not automatically reorder the list. A subsequent call to setSortColumn
    *  will re-sort the columns.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void switchSortType () {
      if ( sortType == SORT_ASCENDING )
         sortType = SORT_DESCENDING ;
      else
         sortType = SORT_ASCENDING ;
   }



   /*------------------------------------------------------------*/
   /* getSortColumn
    */
   /**
    *    This method will return what column
    *  is being sorted
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getSortColumn () {
      return( sortColumn ) ;
   }


   /*------------------------------------------------------------*/
   /* setSortColumn
    */
   /**
    *    This method will set what column
    *  we are currently sorting on. This sort
    *  will apply to any additional data added
    *
    *  @param columnNum The column Number
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSortColumn ( int columnNum ) {
      Object[] dataArray = listModel.toArray() ;

      this.sortColumn = columnNum ;

      if ( dataArray != null ) {
         Object selRec = getSelectedItem() ;

         removeAll() ;
         add ( dataArray ) ;
         setSelectedItem ( selRec, true ) ;
      }
      titlePanel.repaint() ;
   }





   /*------------------------------------------------------------*/
   /* sortedIndex
    */
   /**
    *    This method will return the sorted index
    *  of the object passed in.
    *  
    *
    *  @param object The object to figure out the sorted Index of
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int sortedIndex ( MultiListSort object ) {
      int i           = 0 ;
      int sortedIndex = 0 ;
      int size = listModel.getSize() ;

      while ( ( i < size ) && ( sortedIndex == 0 ) ) {


         if ( ( sortType == SORT_DESCENDING ) &&
              ((MultiListSort)listModel.elementAt(i)).compareTo(object, sortColumn ) == 1 )
            sortedIndex = i-1 ;
         else if ( ((MultiListSort)listModel.elementAt(i)).compareTo(object, sortColumn ) == 1 )
            sortedIndex = i+1 ;


         i++ ;
      }

      if ( sortedIndex < 0 )
         sortedIndex = 0;

      return( sortedIndex ) ;
   }





   /*------------------------------------------------------------*/
   /* setOddLinesColor
    */
   /**
    *    This method can be used to set the
    *  color off all ODD numbered lines.
    *  
    *
    *  @param Color The color to set the odd colored lines to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOddLinesColor ( Color color ) {
      this.oddLinesColor = color ;
      repaint() ;
   }




   /*------------------------------------------------------------*/
   /* getOddLinesColor
    */
   /**
    *    This method will return the color
    *  that all odd lines will have as their
    *  background
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Color getOddLinesColor ( ) {
      return( oddLinesColor ) ;
   }






   /*------------------------------------------------------------*/
   /* setColumnHeadingsBackground
    */
   /**
    *    This method will set the background color
    *  of the column headers
    *  
    *
    *  @param color The new color
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnHeadingsBackground ( Color color ) {
      titlePanel.setBackground ( color ) ;
   }


   /*------------------------------------------------------------*/
   /* getSortType
    */
   /**
    *    This method will return the type of sort
    *  being used.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return SORT_ASCENDING or SORT_DESCENDING
    *//*---------------------------------------------------------*/
   public int getSortType () {
      return( sortType ) ;
   }





   /*------------------------------------------------------------*/
   /* printRow
    */
   /**
    *    This method will paint the given row
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void printRow ( MultiListRow row, Graphics g ) {
      cellRenderer.getListCellRendererComponent ( lb_LIST, row, 1, false, false ) ;
      cellRenderer.paint ( g ) ;
   }

   public void print ( Graphics g, int pageNum, Point topLeft, Dimension size ) {
      Panel p = new Panel () ;
      DList printList = new DList () ;

      printList.setCellRenderer ( cellRenderer ) ;
      printList.setModel ( listModel ) ;

      SwingUtilities.paintComponent ( g, 
                                      printList, 
                                      p, 
                                      topLeft.x, 
                                      topLeft.y+(size.height*pageNum), 
                                      size.width, 
                                      size.height+(topLeft.y+(size.height*pageNum)) ) ;
      //for ( int i =0 ;i < 10; i++ ) {
      //   printRow ( (MultiListRow)listModel.elementAt(i), g ) ;
      //   g.translate ( 30, i*25 ) ;
      //}
      p=null;
      printList=null ;
   }

   public int getNumOfPages ( Dimension size ) {
      try {
         return ( (lb_LIST.getSize().height/size.height)+1 ) ;
      } catch ( Exception e ) {
      }
      return ( 1 ) ;
   }
} /* MutliList */


