package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   MultiListSorter                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MultiListSorter                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 Cogent                           */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This object orders a vector of MultiListSort objects.
 *
 *
 * @version 1.0, 7/1/1998 
 */
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   MultiListSorterer                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class MultiListSorter extends Object {

    private IllegalArgumentException err1 =
    new IllegalArgumentException("Stack Overflow");

    int COMP_LESS  = -1;
    int COMP_EQUAL =  0;
    int COMP_GRTR  =  1;
    int sortType   = 0 ;

    private class StackItem {
        public int left;
        public int right;
    }



    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This constructor creates the MultiListSorter
     *  object and sorts the array passed in. It will
     *  also sort ascending.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public MultiListSorter ( Object [] list, int columnNum, int sortType ) {
        Sort(list,sortType == MultiList.SORT_DESCENDING, columnNum );
    }




    /*------------------------------------------------------------*/
    /* Constructor(1)
     */
    /**
     *    This constructor will sort the vector passed
     *  in
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public MultiListSorter ( Vector vec, int columnNum, int sortType ) {
        Object[] list = null ;
        int      i    = 0 ;

        if ( ( vec != null ) && ( vec.size() > 0 ) ) {
            list = new Object[vec.size()] ;
            vec.copyInto ( list ) ;
            Sort ( list, sortType == MultiList.SORT_DESCENDING, columnNum ) ;

            /* Now, copy the values back into the vector  */
            vec.removeAllElements() ;
            while ( i < list.length ) {
                vec.addElement ( list[i] ) ;
                i++ ;
            }

            list=null ;
        }
    }




    /*------------------------------------------------------------*/
    /* sort
     */
    /**
     *    This method does the actual work of the
     *  MultiListSorter
     *  
     *
     *  @param list The array to sort
     *  @param descending TRUE = sort Descending, FALSE = sort Ascending
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void Sort ( Object [] list, boolean descending, int columnNum ) {
        final int stackSize = 32;
        StackItem [] stack = new StackItem [stackSize];

        for ( int n = 0; n < 32; ++n )
            stack[n] = new StackItem();

        int stackPtr = 0;

        // determine direction of sort
        int comp;

        if ( descending )
            comp = COMP_GRTR;
        else
            comp = COMP_LESS;

        // size of minimum partition to median-of-three
        final int Threshold = 7;

        // sizes of left and right partitions
        int lsize, rsize;

        // create working indexes
        int l, r, mid, scanl, scanr, pivot;

        // set initial values
        l = 0;
        r = list.length - 1;

        Object temp;

        // main loop
        while ( true ) {
            while ( r > l ) {
                if ( (r - l) > Threshold ) {
                    // "median-of-three" partitioning
                    mid = (l + r) / 2;

                    // three-sort left, middle, and right elements
                    if ( compare(list[mid],list[l], columnNum ) == comp ) {
                        temp      = list[mid];
                        list[mid] = list[l];
                        list[l]   = temp;
                    }

                    if ( compare(list[r],list[l], columnNum ) == comp ) {
                        temp    = list[r];
                        list[r] = list[l];
                        list[l] = temp;
                    }

                    // three-sort left, middle, and right elements
                    if ( compare(list[r],list[mid], columnNum ) == comp ) {
                        temp      = list[mid];
                        list[mid] = list[r];
                        list[r]   = temp;
                    }

                    // set-up for partitioning
                    pivot = r - 1;

                    temp        = list[mid];
                    list[mid]   = list[pivot];
                    list[pivot] = temp;

                    scanl = l + 1;
                    scanr = r - 2;
                } else {
                    // set-up for partitioning
                    pivot = r;
                    scanl = l;
                    scanr = r - 1;
                }

                for ( ;; ) {
                    // scan from left for element >= to pivot
                    while ( (compare(list[scanl],list[pivot], columnNum ) == comp) && (scanl < r) )
                        ++scanl;

                    // scan from right for element <= to pivot
                    while ( (compare(list[pivot],list[scanr], columnNum ) == comp) && (scanr > l) )
                        --scanr;

                    // if scans have met, exit inner loop
                    if ( scanl >= scanr )
                        break;

                    // exchange elements
                    temp        = list[scanl];
                    list[scanl] = list[scanr];
                    list[scanr] = temp;

                    if ( scanl < r )
                        ++scanl;

                    if ( scanr > l )
                        --scanr;
                }

                // exchange final element
                temp        = list[scanl];
                list[scanl] = list[pivot];
                list[pivot] = temp;

                // place largest partition on stack
                lsize = scanl - l;
                rsize = r - scanl;

                if ( lsize > rsize ) {
                    if ( lsize != 1 ) {
                        ++stackPtr;

                        if ( stackPtr == stackSize )
                            throw err1;

                        stack[stackPtr].left  = l;
                        stack[stackPtr].right = scanl - 1;
                    }

                    if ( rsize != 0 )
                        l = scanl + 1;
                    else
                        break;
                } else {
                    if ( rsize != 1 ) {
                        ++stackPtr;

                        if ( stackPtr == stackSize )
                            throw err1;

                        stack[stackPtr].left  = scanl + 1;
                        stack[stackPtr].right = r;
                    }

                    if ( lsize != 0 )
                        r = scanl - 1;
                    else
                        break;
                }
            }

            // iterate with values from stack
            if ( stackPtr != 0 ) {
                l = stack[stackPtr].left;
                r = stack[stackPtr].right;

                --stackPtr;
            } else
                break;
        }
    }

    int compare ( Object x1, Object x2, int columnNum ) {

        try {
            if ( ( x1 != null ) && ( x2 != null ) ) {
                if ( (x1 instanceof String) &&  (x2 instanceof String) ) {
                    int c = ((String)x1).compareTo((String)x2);

                    if ( c < 0 )
                        return COMP_LESS;
                    else
                        if ( c > 0 )
                        return COMP_GRTR;
                    else
                        return COMP_EQUAL;
                }if ( ( x1 instanceof MultiListSort ) && ( x2 instanceof MultiListSort ) ) {
                    int c = ((MultiListSort)x1).compareTo((MultiListSort)x2, columnNum) ;

                    if ( c < 0 )
                        return COMP_LESS;
                    else
                        if ( c > 0 )
                        return COMP_GRTR;
                    else
                        return COMP_EQUAL;


                } else {
                    int c = (x1.toString()).compareTo(x2.toString());

                    if ( c < 0 )
                        return COMP_LESS;
                    else
                        if ( c > 0 )
                        return COMP_GRTR;
                    else
                        return COMP_EQUAL;
                }
            }
        } catch ( Exception e ) {
        }
        return( COMP_EQUAL ) ;
    }

}

