package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   SpellCheckPanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SpellCheckPanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel holds a spell checker button, and spell checker controls 
 * and can basically be tied to a specific TextComponent through
 * the 'setTextComponent' method. Please note that a SpellChecker Object
 * MUST be created prior to this control being created.
 *
 *
 * @version 1.0, 8/20/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.border.* ;
import javax.swing.text.* ;
import com.ibm.nzna.shared.spell.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   SpellCheckPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class SpellCheckPanel extends JPanel implements ActionListener, ItemListener, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Statics                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Dimension expandedPrefSize = new Dimension ( 360, 21 ) ;
   private static Dimension normPrefSize     = new Dimension ( 80, 21 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean   expanded        = false ;
   private Dimension prefSize        = normPrefSize ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton         pb_SPELLCHECK  = new DButton ( "Spell Check" ) ;
   private DButton         pb_NEXT        = new DButton ( "Next" ) ;
   private DButton         pb_REPLACE     = new DButton ( "Replace" ) ;
   private DButton         pb_LEARN       = new DButton ( "Learn" ) ;
   private DButton         pb_STOP        = new DButton ( "Stop" ) ;
   private JComboBox       cb_WORDS       = new JComboBox () ;
   private JLabel          st_WORDS       = new JLabel ( "Words" ) ;
   private BevelBorder     indentBorder   = new BevelBorder ( BevelBorder.LOWERED ) ;
   private TextComponent   textComponent  = null ;
   private JTextComponent  jTextComponent = null ;
   private JSpellToolkit   spellToolkit   = null ;
   private JSpellErrorInfo errorInfo      = null ;
   private int             posOffset      = 0 ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SpellCheckPanel () {

      spellToolkit = new SpellChecker().getToolkit() ;

      setLayout ( null ) ;
      add ( pb_SPELLCHECK ) ;
      add ( pb_REPLACE ) ;
      add ( pb_NEXT ) ;
      add ( pb_LEARN ) ;
      add ( pb_STOP ) ;
      add ( cb_WORDS ) ;
      add ( st_WORDS ) ;

      pb_SPELLCHECK.addActionListener ( this ) ;
      pb_REPLACE.addActionListener ( this ) ;
      pb_NEXT.addActionListener ( this ) ;
      pb_LEARN.addActionListener ( this ) ;
      pb_STOP.addActionListener ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred size
    *  of our window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return the minimum size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 4 ;
      int       y         = 2 ;
      int       rowHeight = 18 ;

      super.doLayout() ;

      if ( expanded ) {
         st_WORDS.setBounds   ( x, y, 40, rowHeight-2 ) ;
         x+=40 ;

         cb_WORDS.setBounds   ( x, y, 100, rowHeight-2 ) ;
         x+=100 ;

         pb_REPLACE.setBounds ( x, y, 60, rowHeight-2 ) ;
         x+=60 ;

         pb_NEXT.setBounds ( x, y, 45, rowHeight-2 ) ;
         x+=45 ;

         pb_LEARN.setBounds ( x, y, 60, rowHeight-2 ) ;
         x+=60 ;

         pb_STOP.setBounds ( x, y, 50, rowHeight-2 ) ;
         x+=50 ;

      }

      else {
         pb_SPELLCHECK.setBounds ( 0, 0, size.width, size.height ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to buttons
    *  being pressed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_SPELLCHECK                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_SPELLCHECK ) {
         if ( ( textComponent != null ) || ( jTextComponent != null ) ) {
            expanded = true ;
            prefSize = expandedPrefSize ;
            invalidate() ;
            getParent().invalidate() ;
            st_WORDS.setVisible ( true ) ;
            cb_WORDS.setVisible ( true ) ;
            pb_STOP.setVisible  ( true ) ;
            pb_LEARN.setVisible ( true ) ;
            pb_NEXT.setVisible  ( true ) ;
            pb_SPELLCHECK.setVisible ( false ) ;
            setBorder ( indentBorder ) ;
            revalidate() ;

            /* Start Checking */
            if ( textComponent != null )
               spellToolkit.initRawCheck ( textComponent.getText() ) ;
            else
               spellToolkit.initRawCheck ( jTextComponent.getText() ) ;

            posOffset=0 ;
            new Thread ( this ).start() ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_STOP                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_STOP ) {
         expanded = false ;
         prefSize = normPrefSize ;
         st_WORDS.setVisible ( false ) ;
         cb_WORDS.setVisible ( false ) ;
         pb_STOP.setVisible  ( false ) ;
         pb_LEARN.setVisible ( false ) ;
         pb_NEXT.setVisible  ( false ) ;
         pb_SPELLCHECK.setVisible ( true ) ;
         invalidate() ;
         getParent().invalidate() ;
         setBorder ( null ) ;
         revalidate() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_NEXT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_NEXT ) {
         if ( errorInfo.getWord() != null )
            spellToolkit.addIgnore ( errorInfo.getWord() ) ;
         new Thread ( this ).start() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_REPLACE                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REPLACE )
         replaceText() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_LEARN                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_LEARN ) {
         if ( errorInfo.getWord() != null )
            spellToolkit.addLearned ( errorInfo.getWord() ) ;
         new Thread ( this ).start() ;
      }

   }





   /*------------------------------------------------------------*/
   /* setEnableAll
    */
   /**
    *    this method will either disable or enable
    *  all components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setEnableAll ( boolean enable ) {
      st_WORDS.setEnabled ( enable ) ;
      cb_WORDS.setEnabled ( enable ) ;
      pb_STOP.setEnabled  ( enable ) ;
      pb_LEARN.setEnabled ( enable ) ;
      pb_NEXT.setEnabled  ( enable ) ;
   }


   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our spell thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      /* Disable the controls for now */
      setEnableAll ( false ) ;

      errorInfo = spellToolkit.rawCheck() ;

      if ( errorInfo != null ) {
         remove ( cb_WORDS ) ;
         if ( errorInfo.getSuggestions().length > 0 ) {
            try {
               cb_WORDS = new JComboBox ( errorInfo.getSuggestions() ) ;
            } catch ( Exception e ) {
               cb_WORDS = new JComboBox() ;
            }
         } else
            cb_WORDS = new JComboBox() ;

         add ( cb_WORDS ) ;
         cb_WORDS.addItemListener ( this ) ;

         /* Now, select the bad word */
         if ( textComponent != null )
            textComponent.select ( errorInfo.getPosition()+posOffset, errorInfo.getEndPosition()+posOffset ) ;
         else
            jTextComponent.select ( errorInfo.getPosition()+posOffset, errorInfo.getEndPosition()+posOffset ) ;

      } else {
         expanded = false ;
         prefSize = normPrefSize ;
         st_WORDS.setVisible ( false ) ;
         cb_WORDS.setVisible ( false ) ;
         pb_STOP.setVisible  ( false ) ;
         pb_LEARN.setVisible ( false ) ;
         pb_NEXT.setVisible  ( false ) ;
         pb_SPELLCHECK.setVisible ( true ) ;
         invalidate() ;
         getParent().invalidate() ;
         setBorder ( null ) ;
      }
      /* Disable the controls for now */
      setEnableAll ( true ) ;
      revalidate() ;

   }




   /*------------------------------------------------------------*/
   /* replaceText
    */
   /**
    *    This method will replace the text that
    *  is selected, and then will kick off the thread
    *  again
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void replaceText () {
      String wholeText   = null ;
      String replaceWord = (String) cb_WORDS.getSelectedItem() ;

      if ( textComponent != null )
         wholeText = textComponent.getText() ;
      else
         wholeText = jTextComponent.getText() ;

      //if ( ( replaceWord.length() > errorInfo.getWord().length() ) )
      //   posOffset+=(replaceWord.length()-errorInfo.getWord().length() ) ;
      //else
      //   posOffset-=(replaceWord.length()-errorInfo.getWord().length() ) ;


      if ( replaceWord != null )
         wholeText = wholeText.substring ( 0, errorInfo.getPosition()+posOffset )+replaceWord+wholeText.substring ( errorInfo.getEndPosition() ) ;
      else
         wholeText = wholeText.substring ( 0, errorInfo.getPosition()+posOffset )+wholeText.substring ( errorInfo.getEndPosition() ) ;

      if ( textComponent != null )
         textComponent.setText ( wholeText ) ;
      else {
         jTextComponent.setText ( wholeText ) ;
         if ( jTextComponent instanceof JComponent )
            ((JComponent)jTextComponent).revalidate() ;
      }

      spellToolkit.initRawCheck(wholeText) ;
      new Thread ( this ).start() ;

   }





   /*------------------------------------------------------------*/
   /* setTextComponent
    */
   /**
    *    This method will set the text component
    *  that this spell checker will check
    *  
    *
    *  @param textComponent The TextComponent to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTextComponent ( TextComponent textComponent ) {
      this.textComponent = textComponent ;
   }

   /*------------------------------------------------------------*/
   /* setTextComponent
    */
   /**
    *    This method will set the text component
    *  that this spell checker will check. This is for the Swing
    *  
    *
    *  @param textComponent The TextComponent to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTextComponent ( JTextComponent jTextComponent ) {
      this.jTextComponent = jTextComponent ;
   }





   /*------------------------------------------------------------*/
   /* itemStateChanged
    */
   /**
    *    This method will be called if the user
    *  selects a word from the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void itemStateChanged ( ItemEvent e ) {
      replaceText() ;
   }

} /* SpellCheckPanel */
