package com.ibm.appsure.app.shared.gui ;
/*-----------------------------------------------------------*/
/*   StraightLayoutMgr                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StraightLayoutMgr                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 Cogent                           */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;  
import java.util.* ;
/**
 * This Layout manager just takes components and aligns each one on the same line. 
 *
 *
 * @version 1.0, 12/14/1998 
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   StraightLayoutMgr                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class StraightLayoutMgr extends Object implements LayoutManager {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector    compVec        = new Vector ( 10 ) ;
   private Dimension minimumSize    = new Dimension ( 10, 10 ) ;
   private int       componentCount = 0 ;

   /*------------------------------------------------------------*/
   /* addLayoutComponent
    */
   /**
    *    This method is subclassed to add components
    *  to our InfoBoxPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addLayoutComponent ( String name, Component comp ) {
      compVec.addElement ( comp ) ;
      componentCount++ ;
   }

   public void removeLayoutComponent ( Component comp ) {
      compVec.removeElement ( comp ) ;
      componentCount-- ;
   }


   public void layoutContainer ( Container parent ) {

      /* Synchronize this in case someone takes
         off a component while sizing. This won't let em'! */
      synchronized ( parent.getTreeLock() ) {
         int        i        = 0 ;
         int        length   = compVec.size() ;
         Dimension  prefSize = null ;
         Component  comp     = null ;
         int        x        = 0 ;
         Dimension  size     = parent.getSize() ;

         while ( i < length ) {
            comp = (Component) compVec.elementAt(i) ;

            if ( ( comp != null ) && ( comp.isVisible() ) ) {
               prefSize = comp.getMinimumSize() ;
               comp.setBounds ( x, 0, prefSize.width, size.height ) ;
               x+=prefSize.width+5 ;
            }
            i++ ;
         }

      } /* sync */

   }


   public Dimension minimumLayoutSize ( Container parent ) {
      int        i        = 0 ;
      int        length   = compVec.size() ;
      Dimension  prefSize = null ;
      int        width    = 10 ;
      Component  comp     = null ;

      while ( i < length ) {
         comp = (Component) compVec.elementAt(i) ;

         if ( ( comp != null ) && ( comp.isVisible() ) ) {
            prefSize = comp.getMinimumSize() ;
            width+=prefSize.width+5 ;
            
            if ( minimumSize.height < prefSize.height )
               minimumSize.height = prefSize.height ;
         }
         i++ ;
      }

      minimumSize.width = width+5 ;

      return( minimumSize ) ;
   }




   /*------------------------------------------------------------*/
   /* preferredLayoutSize
    */
   /**
    *    This method will return the preferred layout
    *  size. We will simply return minimumLayoutSize.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension preferredLayoutSize ( Container parent ) {
      return( minimumLayoutSize ( parent ) ) ;
   }




   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return a description of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( "StraightLayoutMgr - David Lentz" ) ;
   }





   /*------------------------------------------------------------*/
   /* componentCount
    */
   /**
    *    This method will return the number of components
    *  in the layout
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int componentCount() {
      return( componentCount ) ;
   }





   /*------------------------------------------------------------*/
   /* componentFromIndex
    */
   /**
    *    This method will return the component
    *  that was added at the specified index
    *  
    *
    *  @param index The index to get the component from
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Component componentFromIndex ( int index ) {
      return( (Component) compVec.elementAt(index) ) ;
   }

} /* StraightLayoutMgr */
