package com.ibm.appsure.app.shared.gui.form;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      MultiLineFormComponent 
/*   Creation Date: APRIL 19, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
/**
 * The MultiLineFormComponent is an extension of the FormComponent and
 * allows a single JTextArea control to be edited in the form.
 *
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* MultiLineFormComponent 
/*
/*----------------------------------------------------*/
public class MultiLineFormComponent extends FormComponent {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
      * The JTextArea control for displaying the data
      **/
    private JTextArea textArea = null ;

    /**
      * The JScrollPane control used for scrolling the JTextArea
      **/
    private JScrollPane scrollPane = null ;

    /**
      *  This property tells the constructors if this control
      * should use a JScrollPane or not. Default is to use a 
      * JScrollPane
      **/
    private boolean useScrollPane = true ;

    /**
      *  This default constructor initializes the FormComponent
      * with no text.
      **/
    public MultiLineFormComponent ( String title ) {
       super ( title ) ;
       initialize() ;
    }

    /**
      * This constructor takes the title of the FormComponent
      * and the initial text that will show in the JTextArea.
      **/
    public MultiLineFormComponent ( String title, String initialText ) {
        super ( title ) ;
        initialize () ;
        setText ( initialText ) ;
    }

    /**
      * This constructor takes the title of the FormComponent
      * and the initial text that will show in the JTextArea. It also
      * allows you to specify if you want the JTextArea inside of a
      * JScrollPane or not.
      **/
    public MultiLineFormComponent ( String title, String initialText, boolean useScrollPane ) {
        super ( title ) ;
        this.useScrollPane = useScrollPane ;
        initialize () ;
        setText ( initialText ) ;
    }

    /**
      *  This method will initialize our JTextArea control
      **/
    private void initialize () {
       textArea = new JTextArea () ;

       /*-----------------------------------------------------------*/
       /*                                                           */
       /*   Adjust properties                                       */        
       /*                                                           */
       /*-----------------------------------------------------------*/
       textArea.setLineWrap ( true ) ;
       textArea.setWrapStyleWord ( true ) ;
       textArea.setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Label.font" ) ) ;

       if ( useScrollPane ) 
          setFormComponent ( new JScrollPane ( textArea ) ) ;
       else 
          setFormComponent ( textArea ) ;
    } /* init  */


    /**
      * This method will set the text that is shown in the JTextArea
      **/
    public void setText ( String text ) {
        if ( text == null )
            textArea.setText ( "" ) ;
        else
            textArea.setText ( text ) ;
    }

    /**
      * This method will return the instance of the JTextArea control that
      * we are displaying.
      **/
    public JTextArea getJTextArea () {
        return ( textArea ) ;
    }


    /**
      * This method will return the text that the user has typed
      **/
    public String getText () {
        return ( textArea.getText() ) ;
    }


}
/* Class1 */
