package com.ibm.appsure.app.shared.gui.wizard ;
/*-----------------------------------------------------------*/
/*   Wizard                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Wizard                              */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import javax.swing.border.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.Vector ;
import java.io.Serializable ;
import com.ibm.appsure.app.shared.gui.* ;
/**
 * The Wizard control is actually a simple subclass of JPanel that  
 * can be added to a JFrame, Frame, JDialog, Dialog or JInternal Frame.<br><br>
 *
 * The main responsibility of the Wizard object is to manage all of the 
 * different steps of the Wizard.<br><br>
 *
 * Use the addStep and removeStep methods to add and remove different steps.<br><br>
 *
 * The WizardSteps may be used for any information display purposes. You may
 * use the InfoStep to simply display an information string to the user.
 *
 *
 * @version 1.0, 12/21/1998 
 * @version 2.0, 9/26/2000 <- Changed look to new Windows 2000 look (DAL)
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Wizard                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class Wizard extends JPanel implements Serializable, ActionListener, Runnable {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Constants                                               */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static final int    MAX_STEPS          = 10 ;
    private static final int    NOTIFY_CANCEL      = 1 ;
    private static final int    NOTIFY_FINISH      = 2 ;
    private static final int    NOTIFY_NEXT        = 3 ;
    private static final int    NOTIFY_BACK        = 4 ;

    private static final String THREAD_PROCESSSHOW = "ProcessShow" ;


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Controls                                                */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private JLabel   st_WIZARDIMAGE       = null ;
    private JPanel   titlePanel           = null ;
    private JLabel   st_TITLE             = null ;
    private DButton  pb_NEXT              = null ;
    private DButton  pb_CANCEL            = null ;
    private DButton  pb_BACK              = null ;
    private DButton  pb_FINISH            = null ;

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Vars                                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private Image       wizardImage     = null ;
    private boolean     initialized     = false ;
    private int         stepIndex       = 0 ;
    private int         stepIndexMax    = 0 ;
    private WizardStep  curStep         = null ;
    private Vector      listeners       = new Vector ( 1, 10 ) ;
    private boolean     autoAdvance     = true ;



    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This will create a blank Wizard displaying
     *  only the next and cancel buttons.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Wizard() {
        init() ;
        initialized = true ;
    }





    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This will create a wizard displaying the next
     *  and cancel buttons and also the image passed.
     *  The image is drawn in the left side of the wizard.
     *
     *  @param wizardImage The Image to draw
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Wizard ( ImageIcon wizardImage ) {
        init() ;
        setImage ( wizardImage ) ;
        initialized = true ;
    }











    /*------------------------------------------------------------*/
    /* init
     */
    /**
     *    This method will initialize all of our
     *  default controls
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void init () {

        /* Create all of the controls */
        st_WIZARDIMAGE = new JLabel () ;
        st_TITLE       = new JLabel ( "TITLE" ) ;
        pb_NEXT        = new DButton ( "Next" ) ;
        pb_BACK        = new DButton ( "Back" ) ;
        pb_CANCEL      = new DButton ( "Cancel" ) ;
        pb_FINISH      = new DButton ( "Finish" ) ;
        titlePanel     = new JPanel() ;


        /* Set the properties of our controls */
        st_WIZARDIMAGE.setBorder ( null ) ;
        st_WIZARDIMAGE.setBackground ( Color.white ) ;
        st_TITLE.setBackground ( Color.white ) ;
        titlePanel.setBackground ( Color.white ) ;
        st_TITLE.setVerticalAlignment ( SwingConstants.TOP ) ;

        /* Add our listeners */
        pb_NEXT.addActionListener ( this ) ;
        pb_BACK.addActionListener ( this ) ;
        pb_CANCEL.addActionListener ( this ) ;
        pb_FINISH.addActionListener ( this ) ;

        /* Add the controls to ourselves */
        setLayout ( null ) ;
        titlePanel.setLayout ( new BorderLayout() ) ;
        titlePanel.add ( st_TITLE,       BorderLayout.CENTER ) ;
        titlePanel.add ( st_WIZARDIMAGE, BorderLayout.EAST ) ;
        add ( titlePanel ) ;
        add ( pb_NEXT ) ;
        add ( pb_BACK ) ;
        add ( pb_CANCEL ) ;
        add ( pb_FINISH ) ;

        /* Set up our controls */
        pb_BACK.setVisible ( false ) ;
        pb_FINISH.setVisible ( false ) ;

    }





    /*------------------------------------------------------------*/
    /* setCurrentStep
     */
    /**
     *    This method will set the current step that
     *  is showing to the step passed
     *  
     *
     *  @param wizardStep The Step to show
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setCurrentStep ( WizardStep step ) {
        showStep(step) ;
    }



    /*------------------------------------------------------------*/
    /* getCurrentStep
     */
    /**
     *    This method will return the current
     *  step that is being shown
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return step The WizardStep that is being shown
     *//*---------------------------------------------------------*/
    public WizardStep getCurrentStep () {
        return( curStep ) ;
    }




    /*------------------------------------------------------------*/
    /* getImage
     */
    /**
     *    This method will return the image that is
     *  being displayed on the left side of the
     *  wizard
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return ImageIcon
     *//*---------------------------------------------------------*/
    public ImageIcon getImage () {
        return( (ImageIcon) st_WIZARDIMAGE.getIcon() ) ;
    }


    /*------------------------------------------------------------*/
    /* setImage 
     */
    /**
     *    This method will change the image that is
     *  being displayed in the left side of the Wizard.
     *  
     *
     *  @param wizardImage The new image to draw
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setImage ( ImageIcon imageIcon ) {
        if ( imageIcon != null ) {
            st_WIZARDIMAGE.setIcon ( imageIcon ) ;
            wizardImage = imageIcon.getImage() ;
        }
        doLayout() ;
    }


    /*------------------------------------------------------------*/
    /* paint
     */
    /**
     *    This method will paint our component
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void paintComponent ( Graphics g ) {
        Dimension size = getSize();

        super.paintComponent(g) ;
        g.setColor ( Color.darkGray ) ;
        g.drawLine ( 0, 59, size.width, 59 ) ;
        g.drawLine ( 0, size.height-50, size.width, size.height-50 ) ;
        g.setColor ( Color.white ) ;
        g.drawLine ( 0, 60, size.width, 60 ) ;
        g.drawLine ( 0, size.height-49, size.width, size.height-49 ) ;
    }





    /*------------------------------------------------------------*/
    /* doLayout
     */
    /**
     *    This method will lay out all of our components
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void doLayout () {

        if ( initialized ) {
            Dimension size = getSize() ;

            super.doLayout() ;
            titlePanel.setBounds ( 0, 0, size.width, 58 ) ;

            if ( pb_NEXT.isVisible() ) {
                pb_BACK.setBounds   ( size.width-(83*3)-5, size.height-30, 78, 23 ) ;
                pb_NEXT.setBounds   ( size.width-(83*2)-10, size.height-30, 78, 23 ) ;
            } else {
                pb_BACK.setBounds   ( size.width-(83*2)-5, size.height-30, 78, 23 ) ;
            }


            pb_CANCEL.setBounds ( size.width-83, size.height-30, 78, 23 ) ;
            pb_FINISH.setBounds ( size.width-83, size.height-30, 78, 23 ) ;


            if ( curStep != null )
                //curStep.setBounds ( 35, 81, size.width-70, size.height-145 ) ;
               curStep.setBounds ( 35, 62, size.width-70, size.height-135 ) ;

        } /* if */
    }




    /*------------------------------------------------------------*/
    /* showStep
     */
    /**
     *    This method will show the passed step
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private synchronized void showStep ( WizardStep newStep ) {

        /* Remove the old step if we have one */
        if ( curStep != null ) {
            remove ( curStep ) ;
        }

        /* Add and set our new step */
        curStep = newStep ;
        add ( curStep ) ;
        doLayout() ;
        newStep.setVisible ( true ) ;
        newStep.doLayout() ;
        st_TITLE.setText ( newStep.getTitle() ) ;
        repaint() ;

        /* Run the process show thread */
        new Thread ( this, THREAD_PROCESSSHOW ).start() ;
    }





    /*------------------------------------------------------------*/
    /* actionPerformed
     */
    /**
     *    This method will be called when events we
     *  are registered to listen to are calling us
     *  
     *
     *  @param e The ActionEvent
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void actionPerformed ( ActionEvent e ) {
        if ( e.getSource() == pb_NEXT ) {
            processNext() ;
        }

        else if ( e.getSource() == pb_BACK ) {
            if ( curStep != null )
                curStep.backOut() ;
            notifyListeners ( NOTIFY_BACK ) ;
            new Thread ( this, THREAD_PROCESSSHOW ).start() ;
        }

        else if ( e.getSource() == pb_CANCEL )
            notifyListeners ( NOTIFY_CANCEL ) ;

        else if ( e.getSource() == pb_FINISH ) {
            if ( ( curStep != null ) && ( curStep.saveInfo() ) )
                notifyListeners ( NOTIFY_FINISH ) ;
        }
    }



    /*------------------------------------------------------------*/
    /* run
     */
    /**
     *    This method will run our many processes
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void run () {
        String name = Thread.currentThread().getName() ;


        if ( name.equals ( THREAD_PROCESSSHOW ) ) {
            if ( curStep != null ) {
                curStep.setWizard ( this ) ;
                curStep.refreshInfo() ;
            }
        }

    }





    /*------------------------------------------------------------*/
    /* notifyListeners
     */
    /**
     *    This method will go through all of the listeners
     *  of the listener vector and notify each one
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private synchronized void notifyListeners ( int notify ) {
        int            i        = 0 ;
        int            length   = listeners.size() ;
        WizardListener listener = null ;
        WizardEvent    event    = null ;

        event = new WizardEvent ( curStep, this ) ;

        while ( i < length ) {
            listener = (WizardListener) listeners.elementAt(i) ;
            if ( listener != null ) {

                switch ( notify ) {
                case NOTIFY_CANCEL :
                    listener.cancelPerformed ( event ) ;
                    break ;

                case NOTIFY_FINISH :
                    listener.finishPerformed ( event ) ;
                    break ;

                case NOTIFY_NEXT :
                    listener.nextPerformed ( event ) ;
                    break ;

                case NOTIFY_BACK :
                    listener.backPerformed ( event ) ;
                    break ;
                }

            }
            i++ ;
        }
    }





    /*------------------------------------------------------------*/
    /* addWizardListener
     */
    /**
     *    This method will add a WizardListener
     *  to be notified of Wizard events.
     *  
     *
     *  @param wizardListener The listener to add
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public synchronized void addWizardListener ( WizardListener listener ) {
        listeners.addElement ( listener ) ;
    }




    /*------------------------------------------------------------*/
    /* removeWizardListener
     */
    /**
     *    This method will remove a listener from
     *  the list of listeners.
     *  
     *
     *  @param listener The listener to remove
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public synchronized void removeWizardListener ( WizardListener listener ) {
        listeners.removeElement ( listener ) ;
    }

    /*------------------------------------------------------------*/
    /* enableControls
     */
    /**
     *    This method will either enable or disable
     *  the next, back, cancel and finish buttons.
     *  
     *
     *  @param enable TRUE = enable the buttons, FALSE = disable the buttons
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void enableControls ( boolean enable ) {
        pb_NEXT.setEnabled ( enable ) ;
        pb_BACK.setEnabled ( enable ) ;
        pb_CANCEL.setEnabled ( enable ) ;
        pb_FINISH.setEnabled ( enable ) ;
    }



    /*------------------------------------------------------------*/
    /* showNext
     */
    /**
     *    This method will either show or hide the 
     *  next button.
     *  
     *
     *  @param show TRUE = show the Next button, FALSE = do not show
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void showNext ( boolean show ) {
        pb_NEXT.setVisible ( show ) ;
    }


    /*------------------------------------------------------------*/
    /* showCancel
     */
    /**
     *    This method will either show or hide the
     *  cancel button
     *  
     *
     *  @param show TRUE = show the cancel button, FALSE = Hide the cancel button
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void showCancel ( boolean show ) {
        pb_CANCEL.setVisible ( show ) ;
    }

    /*------------------------------------------------------------*/
    /* showFinish
     */
    /**
     *    This method will either show or hide the
     *  finish button
     *  
     *
     *  @param show TRUE = show the finish button, FALSE = Hide the finish button
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void showFinish ( boolean show ) {
        pb_FINISH.setVisible ( show ) ;
    }

    /*------------------------------------------------------------*/
    /* showBack
     */
    /**
     *    This method will either show or hide the
     *  back button
     *  
     *
     *  @param show TRUE = show the back button, FALSE = Hide the back button
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void showBack ( boolean show ) {
        pb_BACK.setVisible ( show ) ;
    }





    /*------------------------------------------------------------*/
    /* showButtons
     */
    /**
     *    This is a convience method for setting the
     *  visibility state of all 4 buttons. Please note that
     *  the cancel button and finish button cannot be shown together!
     *
     *  @param showNext - TRUE = show the Button, FALSE = do not show
     *  @param showBack - TRUE = show the Button, FALSE = do not show
     *  @param showCancel - TRUE = show the Button, FALSE = do not show
     *  @param showFinish - TRUE = show the Button, FALSE = do not show
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void showButtons ( boolean showNext, boolean showBack, boolean showCancel, boolean showFinish ) {
        showNext ( showNext ) ;
        showBack ( showBack ) ;
        showCancel ( showCancel ) ;
        showFinish ( showFinish ) ;
        doLayout() ;
    }





    /*------------------------------------------------------------*/
    /* processNext
     */
    /**
     *    This method will simulate the user pressing
     *  the 'Next' button
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void processNext () {
        if ( ( curStep != null ) && ( curStep.saveInfo() ) ) {
            notifyListeners ( NOTIFY_NEXT ) ;
            new Thread ( this, THREAD_PROCESSSHOW ).start() ;
        } else if ( curStep == null ) {
            notifyListeners ( NOTIFY_NEXT ) ;
            new Thread ( this, THREAD_PROCESSSHOW ).start() ;
        }
    }




    /*------------------------------------------------------------*/
    /* setNextText
     */
    /**
     *    This method will set the text on the
     *  next button
     *  
     *
     *  @param text The text to set
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setNextText ( String text ) {
        pb_NEXT.setText ( text ) ;
    }


    /*------------------------------------------------------------*/
    /* setBackText
     */
    /**
     *    This method will set the text on the
     *  Back button
     *  
     *
     *  @param text The text to set
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setBackText ( String text ) {
        pb_BACK.setText ( text ) ;
    }



    /*------------------------------------------------------------*/
    /* setFinishText
     */
    /**
     *    This method will set the text on the
     *  Finish button
     *  
     *
     *  @param text The text to set
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setFinishText ( String text ) {
        pb_FINISH.setText ( text ) ;
    }



    /*------------------------------------------------------------*/
    /* setCancelText
     */
    /**
     *    This method will set the text on the
     *  Cancel button
     *  
     *
     *  @param text The text to set
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setCancelText ( String text ) {
        pb_CANCEL.setText ( text ) ;
    }





} /* Wizard */
