package com.ibm.appsure.app.shared.gui.wizard ;
/*-----------------------------------------------------------*/
/*   WizardStep                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WizardStep                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.JPanel ;
/**
 * The WizardStep Panel defines functions that the Wizard will 
 * call during it's lifetime. 
 *
 * A Wizard Step is simply a single step in the process of taking a user
 * through a series of commands. For example, an Installation wizard would
 * use one step to get destination directory, the next step would be to
 * get shortcut information etc..
 *
 * Any time the wizard processes a 'Next' or 'Back' action, the methods
 * called are in seperate threads. This is done to make the Wizard more
 * repsonsive during long refreshInfo() calls. This also allows a WizardStep
 * to process information. 
 *
 * A WizardStep can use the next() and back() methods in the Wizard to auto-push
 * the next or back buttons.
 *
 *
 * @version 1.0, 12/21/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WizardStep                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class WizardStep extends JPanel {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */      
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String stepTitle_ = "" ;
   



   /*------------------------------------------------------------*/
   /* setWizard
    */
   /**
    *    This method will be called once to set
    *  the wizard that this WizardStep is going
    *  to display in
    *
    *  @param wizard The Wizard that will be used to display this step
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWizard ( Wizard wizard ) {}





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will be called only if the user hits the
    *  next or finish button. It is used by the Wizard
    *  to determine if processing the next step of the Wizard
    *  can continue. If the information collected on this WizardStep
    *  is not sufficient to continue processing, it should return
    *  false. No action will be taken by the Wizard if this method
    *  returns 'FALSE'.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Wizard can continue processing, FALSE if Wizard should disregard the last action from the user
    *//*---------------------------------------------------------*/
   public boolean saveInfo() { return( true ) ;}





   /*------------------------------------------------------------*/
   /* refreshInfo
    */
   /**
    *    This method will be called every time the Wizard
    *  displays the step. This is the time for any Wizard
    *  to set the data in any GUI controls it has. This is not the time
    *  to create any GUI controls or to do any CPU intensive actions. Doing
    *  so will slow down the process of showing this step.
    *
    *  This WizardStep is not displayed until after this method is called.
    *
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshInfo ( ) {}




   /*------------------------------------------------------------*/
   /* backOut
    */
   /**
    *    This method will be called when the user
    *  has backed out of this WizardStep.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public  void backOut () {}






   /*------------------------------------------------------------*/
   /* close
    */
   /**
    *    This method will be called during the Wizard's
    *  close processing. Any objects/GUI controls etc.. should
    *  be freed in this method. Any threads running for this
    *  WizardStep should also be stopped.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public  void close() {}





   /*------------------------------------------------------------*/
   /* getId
    */
   /**
    *    This method can be subclassed to return
    *  an integer id of this WizardStep. The Default
    *  return is 0
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getId() {
      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method should return the title of this
    *  step
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( stepTitle_ ) ;
   }




   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set the title that is returned
    *  from this step
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitle ( String title ) {
      stepTitle_ = title ;
   }

} /* WizardStep */
