package com.ibm.appsure.app.shared.util ;
/*-----------------------------------------------------------*/
/*   CDate                                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CDate                               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.text.* ;
/**
 * This class holds many useful functions for manipulating dates 
 *
 *
 * @version 1.0, 7/26/2000 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   CDate                                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class CDate extends Object {



    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Date and Time Constants                                 */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static final int DATE_STAMP           = 1 ;  // 1997-01-01-08.00.00.00000
    public static final int DATE_DATE            = 2 ;  // 1/1/1997
    public static final int DATE_TIME            = 3 ;  // 12:00:00
    public static final int DATE_HOUR            = 4 ;  // 12
    public static final int DATE_MINUTE          = 5 ;  // 00
    public static final int DATE_SECOND          = 6 ;  // 00
    public static final int DATE_YEAR            = 7 ;  // 1997
    public static final int DATE_MONTH           = 8 ;  // 01
    public static final int DATE_DAY             = 9 ;  // 01
    public static final int DATE_DBDATE          = 10 ; // 1997/01/01
    public static final int DATE_CCEXPIRE        = 11 ;  // 01/1997
    public static final int DATE_PRETTYDATE      = 12 ;  // 01/01/1997 at 12:00pm




    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Public Variables                                        */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public int    hour            = 0 ;
    public int    minute          = 0 ;
    public int    second          = 0 ;
    public int    year            = 0 ;
    public int    month           = 0 ;
    public int    day             = 0 ;
    public int    dateType        = DATE_DATE ;
    public String strDate         = "" ;

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Constructors                                            */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public CDate ( ) {
        this.strDate  = today() ;
    } /* CDate */

    public CDate ( int dateType ) {
        this.dateType = dateType ;
        this.strDate  = today () ;
    }

    public CDate ( String date, int dateType ) {

        this.dateType = dateType ;
        this.strDate  = date ;

    } /* CDate */        

    /*-----------------------------------------------------------*/
    /*   today()                                                 */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will return todays date in the current   */
    /*   date type.                                              */
    /*                                                           */
    /*   Parameters:                                             */
    /*                                                           */
    /*                                                           */
    /*   Returns:                                                */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public String today ( ) {
        SimpleDateFormat  date      = null ;
        Date              todayDate = new Date() ;
        String            retStr    = "" ;

        switch ( dateType ) {
            
            case DATE_TIME :
                date = new SimpleDateFormat ( "hh:mm" ) ;
                break ;

            case DATE_DATE :
                date = new SimpleDateFormat ( "MM/dd/yyyy" ) ;
                break ;

            case DATE_STAMP :
                date = new SimpleDateFormat ( "yyyy-MM-dd-kk.mm.ss.000000" ) ;
                break ;

            case DATE_CCEXPIRE :
                date = new SimpleDateFormat ( "MM/yyy" ) ;
                break ;

            case DATE_YEAR :
                date = new SimpleDateFormat ( "yyyy" ) ;
                break ;

            case DATE_MONTH :
                date = new SimpleDateFormat ( "MM" ) ;
                break ;

            case DATE_DAY :
                date = new SimpleDateFormat ( "dd" ) ;
                break ;


        } /* switch */

        date.setTimeZone ( TimeZone.getDefault() ) ;
        retStr = date.format(todayDate) ;
        return( retStr ) ;
    } /* today */ 



    /*-----------------------------------------------------------*/
    /* getDate
     */
    /**
     *    This method will return a certain type of date
     *  
     *  
     *             
     *  @param dateType The type of date to get
     *  @param 
     *  @param 
     *
     *  @return String representation of current date/time
     */
    public String getDate ( int dateType ) {
        int    tempDateType = this.dateType ;
        String retStr       = "" ;

        this.dateType = dateType ;
        retStr = today () ;
        this.dateType = tempDateType ;

        return( retStr ) ;
    }





    /*------------------------------------------------------------*/
    /* convertDate
     */
    /**
     *    This method will convert a date from
     *  the fromDateType to the toDateType
     *  
     *
     *  @param fromDateType The current type of date being passed
     *  @param toDateType The type of date to return
     *  @param date The actual string date
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public static String convertDate ( int fromDateType, int toDateType, String date ) {
        return( createDate ( parse ( date, DATE_YEAR, fromDateType ),
                             parse ( date, DATE_MONTH, fromDateType ),
                             parse ( date, DATE_DAY, fromDateType ),
                             parse ( date, DATE_HOUR, fromDateType ),
                             parse ( date, DATE_MINUTE, fromDateType ),
                             parse ( date, DATE_SECOND, fromDateType ),
                             toDateType ) ) ;
    }

    /**
      *  This method will simply return the 4 digit year
      **/
    public int getYear () {
       return ( new Integer ( format4Digit ( parse ( today(),
                                                     DATE_YEAR,
                                                     dateType ) ) ).intValue() ) ;
    }

    /**
      *  This method will simply return the month
      **/
    public int getMonth () {
       return ( new Integer ( parse ( today(),
                              DATE_MONTH,
                              dateType ) ).intValue() ) ;
    }

    /**
      *  This method will simply return the day of the month
      **/
    public int getDayOfMonth () {
       return ( new Integer ( parse ( today(),
                                      DATE_DAY,
                                      dateType ) ).intValue() ) ;
    }

    /*-----------------------------------------------------------*/
    /*   parse()                                                 */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will pull out 1 value from a date or     */
    /*   time string. If you pass a time string, the dateType    */
    /*   must be DATE_TIME.                                      */
    /*                                                           */
    /*   Parameters:                                             */
    /*      date     - The Date/Time string to parse from        */
    /*      parse    - What value to parse. ex DATE_HOUR         */
    /*      dateType - What type of string was passed.           */
    /*                                                           */
    /*   Returns:                                                */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static int parse ( String date, int parse, int dateType ) {
        StringBuffer retBuf   = new StringBuffer ( 1 ) ;
        int          retInt   = 0 ;
        int          delCount = 0 ;
        int          startPos = 0 ;
        int          del      = 1 ;
        boolean      cont     = true ;
        Integer      integer  = null ;
        int          count    = 0 ;

        if ( ( date != null ) && ( date.length()>0 ) ) {

            try {

                retBuf = new StringBuffer ( date.length() ) ;                  

                /* First, because we don't know if the number delimeter
                   is a :, -, ., or /, change it to a ~, so we know what
                   the delimeter is */
                date = date.replace ( ':', '' ) ;
                date = date.replace ( '/', '' ) ;
                date = date.replace ( '-', '' ) ;
                date = date.replace ( '.', '' ) ;
                date = date.replace ( ' ', '' ) ;

                switch ( dateType ) {
                    
                    case DATE_TIME :
                        switch ( parse ) {
                            case DATE_HOUR :
                                delCount = 0 ;
                                break ;

                            case DATE_MINUTE :
                                delCount = 1 ;
                                break ;

                            case DATE_SECOND :
                                delCount = 2 ;
                                break ;
                        } /* switch parse */
                        break ;

                    case DATE_DBDATE :
                        switch ( parse ) {
                            case DATE_YEAR :
                                delCount = 0 ;
                                break ;

                            case DATE_MONTH :
                                delCount = 1 ;
                                break ;

                            case DATE_DAY :
                                delCount = 2 ;
                                break ;
                        } /* switch parse */
                        break ;

                    case DATE_CCEXPIRE :
                        switch ( parse ) {
                            case DATE_YEAR :
                                delCount = 1 ;
                                break ;

                            case DATE_MONTH :
                                delCount = 0 ;
                                break ;
                        } /*switch*/
                        break ;

                    case DATE_DATE :
                        switch ( parse ) {
                            case DATE_YEAR :
                                delCount = 2 ;
                                break ;

                            case DATE_MONTH :
                                delCount = 0 ;
                                break ;

                            case DATE_DAY :
                                delCount = 1 ;
                                break ;
                        } /* switch parse */
                        break ;



                        /* 1997-07-26-22.50.25.090000 */
                        /* 0    1  2  3  4  5  6      */

                    case DATE_STAMP :
                        switch ( parse ) {
                            case DATE_YEAR :
                                delCount = 0 ;
                                break ;

                            case DATE_DAY :
                                delCount = 2 ;
                                break ;

                            case DATE_MONTH :
                                delCount = 1 ;
                                break ;

                            case DATE_HOUR :
                                delCount = 3 ;
                                break ;

                            case DATE_MINUTE :
                                delCount = 4 ;
                                break ;

                            case DATE_SECOND :
                                delCount = 5 ;
                                break ;

                        } /* switch parse */
                        break ;

                } /* switch dateType */


                /* Ok, we have our delimeter count. Walk down the
                   string and find the starting position of the 
                   parse var. */
                startPos = 0 ;
                count = 0 ;
                while ( count != delCount ) {
                    startPos = date.indexOf ( del, startPos+1 ) ;
                    count++ ;    
                }
                if ( delCount > 0 )
                    startPos++ ; // Advance to our first character 

                /* Ok, build a string with the parse value in it */
                while ( (startPos < date.length() ) && ( cont ) ) {
                    retBuf.append ( date.charAt(startPos) ) ;
                    startPos++ ;

                    /* Check to see if we have got the next delimeter */
                    if ( startPos < date.length() )
                        if ( date.charAt(startPos) == '' )
                            cont = false ;
                } /* while */

                date = date.replace ( '', ' ' ) ;
                integer = new Integer ( retBuf.toString().trim() ) ;
                retInt = integer.intValue() ;
            } catch ( NumberFormatException e1 ) {
            } catch ( Exception e ) {
               e.printStackTrace() ;
            } /* catch */
        } /* if date */

        return( retInt ) ;
    } /* parse */

    /*-----------------------------------------------------------*/
    /*   julian()                                                */
    /*                                                           */
    /*   Description:                                            */
    /*      USE toJulain FOR NOW!                                */
    /*                                                           */
    /*   Parameters:                                             */
    /*      date - The String representation of the date         */
    /*                                                           */
    /*   Returns:                                                */
    /*      a double representing the date                       */
    /*-----------------------------------------------------------*/
    public static double toJulian ( String date, int dateType ) {
        double retJulian = 0.0 ;

        if ( date.length() > 0 ) {
            int    year      = parse ( date, DATE_YEAR,  dateType ) ;
            int    month     = parse ( date, DATE_MONTH, dateType ) ;
            int    day       = parse ( date, DATE_DAY,   dateType ) ;

            /* If this is a CCEXPIRE Date, it has no day, so assume
               it is 1 */
            if ( dateType == DATE_CCEXPIRE )
                day = 1 ;


            //retJulian = (long)367*year-(long)(7*(year+(long)((month+9)/12))/4)+(long)((275*month)/9)+day;
            retJulian = (long)367*year-(long)Math.floor((7*(year+(long)Math.floor(((month+9)/12)))/4))+(long)Math.floor(((275*month)/9))+day;

            retJulian += 1721013.5;
        }

        return( retJulian ) ;
    }          

    /*-----------------------------------------------------------*/
    /*   julianToGregorian()                                     */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will translate a julian date to a        */
    /*   gregorian date.                                         */
    /*                                                           */
    /*   Parameters:                                             */
    /*      julian   - The Double Julian date                    */
    /*      dateType - The type of date you want returned        */
    /*                                                           */
    /*   Returns:                                                */
    /*      String representation of the julian date passed      */
    /*-----------------------------------------------------------*/
    public static String julianToGregorian ( double julian, int dateType ) {
        double a=0, b=0, c=0, d=0, e=0, f=0, g=0;
        long   z      = 0 ;
        int    day    = 0 ;
        int    month  = 0 ;
        int    year   = 0 ;
        String retStr = "" ;

        try {
            z = (long) Math.floor ( julian ) ;
            f = julian + 0.5 - z;

            if ( z < 2299161 )
                a = z;
            else {
                g = (z - 1867216.25) / 36524.25;
                a = z + 1 + g - (long)(g / 4.0);
            } /* else */

            b = a + 1524.0;
            c = Math.floor (((b - 122.1) / 365.25));
            d = Math.floor (365.25 * c);
            e = Math.floor ((b - d) / 30.6001);

            day   = (int) Math.floor ( b - d - (long)(30.6001 * e) + f ) ;
            month = (int) Math.floor ( (e < 13.5)     ? e - 1    : e - 13 ) ;
            year  = (int) Math.floor ( (month > 2.5) ? c - 4716 : c - 4715 ) ;

            retStr = createDate ( year, month, day, 0, 0, 0, dateType ) ;

        } catch ( Exception ex ) {
            ex.printStackTrace() ;
        }

        return( retStr ) ;
    } /* julianToGregorian */

    /*-----------------------------------------------------------*/
    /*   createDate()                                            */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will create a date string from the       */
    /*   int values passed.                                      */
    /*                                                           */
    /*   Parameters:                                             */
    /*      int values for year,month,day,hour,minute,second     */
    /*      dateType - The type of date to return                */
    /*                                                           */
    /*   Returns:                                                */
    /*      The string rep of the date                           */
    /*-----------------------------------------------------------*/
    public static String createDate ( int year, int month,  int day,
                                      int hour, int minute, int second, int dateType ) {
        String           retStr = "" ;

        switch ( dateType ) {
            
            case DATE_TIME :
                retStr = format2Digit(hour)+":"+format2Digit(minute) ;
                break ;

            case DATE_DATE :
                retStr = format2Digit(month)+"/"+format2Digit(day)+"/"+format4Digit(year) ;
                break ;

            case DATE_STAMP :
                retStr = format4Digit(year)+"-"+format2Digit(month)+"-"+format2Digit(day)+"-"+format2Digit(hour)+"."+format2Digit(minute)+"."+format2Digit(second)+".000000" ;
                break ;

            case DATE_CCEXPIRE :
                retStr = format2Digit(month)+"/"+format4Digit(year) ;
                break ;

            case DATE_DBDATE :
                retStr = format4Digit(year)+"/"+format2Digit(month)+"/"+format2Digit(day) ;
                break ;

            case DATE_PRETTYDATE :
                String  time = format2Digit(hour)+":"+format2Digit(minute) ;
                boolean isPM = isPM ( time ) ;

                if ( hour > 12 ) {
                    hour-=12 ;
                    time = format2Digit(hour)+":"+format2Digit(minute) ;
                }

                if ( isPM )
                    time = time+" pm" ;
                else
                    time = time+" am" ;

                retStr = format2Digit(month)+"/"+format2Digit(day)+"/"+format4Digit(year)+" at "+time ;
                break ;

        } /* switch */

        return( retStr ) ;
    } /* CreateDate */

    /*-----------------------------------------------------------*/
    /*   differenceDate()                                        */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will return the difference between two   */
    /*   dates in days.                                          */
    /*                                                           */
    /*   Parameters:                                             */
    /*      date1    - The first date                            */
    /*      date2    - The second date                           */
    /*      dateType - The type of dates that were passed        */
    /*                                                           */
    /*   Returns:                                                */
    /*      number of days between                               */
    /*-----------------------------------------------------------*/
    public static long differenceDate ( String date1, String date2, int dateType ) {
        double   jul1 = 0 ;
        double   jul2 = 0 ;
        long     diff = 0 ;

        if ( (date1 != null) && (date2 != null) && (date1.length()>0) && (date2.length()> 0) ) {
            jul1 = toJulian ( date1, dateType ) ;
            jul2 = toJulian ( date2, dateType ) ;

            diff = Math.round( jul1-jul2 ) ;

        }

        return( diff ) ;
    } /* difference */

    /*-----------------------------------------------------------*/
    /*   compareDate()                                           */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will compare two dates, and return       */
    /*   1, 0, or -1.                                            */
    /*                                                           */
    /*   Parameters:                                             */
    /*      date1    - The first date                            */
    /*      date2    - The second date                           */
    /*      dateType - The type of dates that were passed        */
    /*                                                           */
    /*   Returns:                                                */
    /*     1 = Date1 is greater than Date 2                      */
    /*     0 = Date1 is equal to Date 2                          */
    /*    -1 = Date2 is greater than Date 1                      */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static int compareDate ( String date1, String date2, int dateType ) {
        double   jul1 = 0 ;
        double   jul2 = 0 ;
        int      ret  = 0 ;

        if ( ( date1.length()>0 ) && ( date2.length()> 0 ) ) {
            jul1 = toJulian ( date1, dateType ) ;
            jul2 = toJulian ( date2, dateType ) ;

            if ( jul1 > jul2 )
                ret = 1 ;
            else if ( jul1 < jul2 )
                ret = -1 ;
            else
                ret = 0 ;

        }

        return( ret ) ;
    } /* compareDate */


    /*-----------------------------------------------------------*/
    /*   toJulian()                                              */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will return the julian double from the   */
    /*   date passed                                             */
    /*                                                           */
    /*   Parameters:                                             */
    /*      date - The String representation of the date         */
    /*                                                           */
    /*   Returns:                                                */
    /*      a double representing the date                       */
    /*-----------------------------------------------------------*
     public static int toJulian ( String date, int dateType ) {
        int    year      = parse ( date, DATE_YEAR,  dateType ) ;
        int    month     = parse ( date, DATE_MONTH, dateType ) ;
        int    day       = parse ( date, DATE_DAY,   dateType ) ;
        int    julian    = year * 1000 + julianDays (year, month, day);
     
        
        return ( julian ) ;
     } /* toJulian */


    /*-----------------------------------------------------------*/
    /*   julianDays()                                            */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will return the amount of julian days    */
    /*   from the year, month and day passed.                    */
    /*                                                           */
    /*   Returns:                                                */
    /*      the number of julian days                            */
    /*-----------------------------------------------------------*/
    private static int julianDays (int y, int m, int d) {
        int loop   = 0 ;
        int result = 0 ;

        for ( loop = 1; loop < m; loop++ )
            result += daysInMonth (y, loop);

        return(result + d);
    }


    /*-----------------------------------------------------------*/
    /*   daysInMonth()                                           */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will return the amount of days in a      */
    /*   month.                                                  */
    /*                                                           */
    /*   Parameters:                                             */
    /*      m - The month to return days in                      */
    /*      y - The year                                         */
    /*                                                           */
    /*   Returns:                                                */
    /*      The amount of days in a month                        */
    /*-----------------------------------------------------------*/
    private static int daysInMonth (int y, int m) {

        int days = 0;

        switch ( m ) {
            
            case 1:
            case 3:
            case 5:
            case 7:
            case 8:
            case 10:
            case 12: days = 31; break;

            case 4:
            case 6:
            case 9:
            case 11: days = 30; break;

            case 2:  days = (isLeapYear (y) ? 29 : 28); break;
        }

        return(days);
    }


    /*-----------------------------------------------------------*/
    /*   isLeapYear()                                            */
    /*                                                           */
    /*   Description:                                            */
    /*      THis method will determine if a year is a leap year  */
    /*                                                           */
    /*   Parameters:                                             */
    /*      y - The year to check                                */
    /*                                                           */
    /*   Returns:                                                */
    /*      TRUE - Year is a leap year, FALSE if not             */
    /*-----------------------------------------------------------*/
    private static boolean isLeapYear (int y) {

        boolean result = false;

        if ( (y % 4) == 0 )
            if ( (y % 100) == 0 )
                result = ((y % 400) == 0);
            else
                result = true;
        else
            result = false;

        return(result);
    }



    /*-----------------------------------------------------------*/
    /*   formatTime()                                            */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will format the time 15:33:00 to         */
    /*   3:33 pm.                                                */
    /*                                                           */
    /*   Parameters:                                             */
    /*      time     - The time to format                        */
    /*      dateType - The type of date ( timestamp or time )    */
    /*                                                           */
    /*   Returns:                                                */
    /*       the string of formatted time                        */
    /*-----------------------------------------------------------*/
    public static String formatTime ( String time, int dateType ) {
        String formatTime = "" ;

        if ( time != null ) {
            int     hours   = parse ( time, DATE_HOUR,   dateType ) ;
            int     minutes = parse ( time, DATE_MINUTE, dateType ) ;

            formatTime = formatTime ( hours, minutes, dateType ) ;
        } /* if */

        return( formatTime ) ;
    } /* formatTime */

    public static String formatTime ( int hours, int minutes, int dateType ) {
        String formatTime = "" ;
        String strMinute  = "" ;

//               String  suffix  = "am" ;

//               if ( hours > 12 ) {
//                  hours-=12 ;
//                  suffix = "pm" ;
//               }
//               else if ( hours == 12 ) 
//                  suffix = "pm" ;
//               else if ( hours == 0 ) {
//                  hours = 12 ;
//               }
//               else
//                  suffix = "pm" ;

        if ( minutes < 10 )
            strMinute = "0"+minutes ;
        else
            strMinute = ""+minutes ;

        formatTime = hours+":"+strMinute;//+" "+suffix ;


        return( formatTime ) ;
    } /* formatTime */


    /*-----------------------------------------------------------*/
    /*   isPM ()                                                 */
    /*                                                           */
    /*   Description:                                            */
    /*      This function will determine if a time is in the     */
    /*   afternoon.                                              */
    /*                                                           */
    /*   Parameters:                                             */
    /*      time - The time to check                             */
    /*                                                           */
    /*   Returns:                                                */
    /*      TRUE - the time is after 12:00 noon                  */
    /*-----------------------------------------------------------*/
    public static boolean isPM ( String time ) {
        boolean after = false ;
        int     hour  = parse ( time, DATE_HOUR, DATE_TIME ) ;

        if ( hour > 12 )
            after = true ;
        else if ( time.indexOf ( "pm" ) >=0 )
            after = true ;
        else if ( time.indexOf ( "PM" ) >=0 )
            after = true ;

        return( after ) ;

    } /* isPM */

    /*-----------------------------------------------------------*/
    /*   checkDate()                                             */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will check a date to see if it's valid   */
    /*   You can also pass in times.                             */
    /*                                                           */
    /*   Parameters:                                             */
    /*      date     - The date to be checked                    */
    /*      dateType - The type of date                          */
    /*                                                           */
    /*   Returns:                                                */
    /*      The DATE_XXX of the invalid part, 0 if everything OK */
    /*   -1 if passed a null, or failed for unknown reason       */
    /*-----------------------------------------------------------*/
    public static int checkDate ( String date, int dateType ) {
        String errStr  = "" ;
        int    hours   = 0 ;
        int    minutes = 0 ;
        int    seconds = 0 ;
        int    days    = 0 ;
        int    month   = 0 ;
        int    year    = 0 ;
        int    ret     = -1 ;

        if ( date != null ) {
            switch ( dateType ) {
                
                case DATE_DATE :
                    days = parse ( date, DATE_DAY, dateType ) ;
                    month = parse ( date, DATE_MONTH, dateType ) ;
                    year  = parse ( date, DATE_YEAR, dateType ) ;
                    hours = 12 ;
                    minutes = 12 ;
                    seconds = 12 ;
                    break ;

                case DATE_STAMP :
                    days    = parse ( date, DATE_DAY,    dateType ) ;
                    month   = parse ( date, DATE_MONTH,  dateType ) ;
                    year    = parse ( date, DATE_YEAR,   dateType ) ;
                    hours   = parse ( date, DATE_HOUR,   dateType ) ;
                    minutes = parse ( date, DATE_MINUTE, dateType ) ;
                    seconds = parse ( date, DATE_SECOND, dateType ) ;
                    break ;

                case DATE_TIME :
                    days    = 12 ;
                    month   = 12 ;
                    year    = 1996 ;
                    hours   = parse ( date, DATE_HOUR,   dateType ) ;
                    minutes = parse ( date, DATE_MINUTE, dateType ) ;
                    seconds = parse ( date, DATE_SECOND, dateType ) ;
                    break ;

                case DATE_CCEXPIRE :
                    days    = 12 ;
                    month   = parse ( date, DATE_MONTH, dateType ) ;
                    year    = parse ( date, DATE_YEAR, dateType );
                    hours   = 1 ;
                    minutes = 1 ;
                    seconds = 1 ;
                    break ;


            } /* switch */


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Check for the obvious errors                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( days < 1 ) || ( days > 31 ) ) {
                ret = DATE_DAY ;
            } else if ( ( month < 1 ) || ( month > 12 ) ) {
                ret = DATE_MONTH ;
            } else if ( ( year < 1980 ) || ( year > 9999 ) ) {
                ret = DATE_YEAR ;
            } else if ( ( hours < 1 ) || ( hours > 12 ) ) {
                ret = DATE_HOUR ;
            } else if ( ( minutes < 0 ) || ( minutes > 59 ) ) {
                ret = DATE_MINUTE ;
            } else if ( ( seconds < 0 ) || ( seconds > 59 ) ) {
                ret = DATE_SECOND ;
            } else
                ret = 0 ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now make sure the date range is correct for             */
            /* The month                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( month != 0 ) && ( year != 0 ) && ( ret == 0 ) ) {
                int daysInMonth = daysInMonth ( year, month ) ;

                if ( days > daysInMonth )
                    ret = DATE_DAY ;
            }


        } /* if date */

        return( ret ) ;

    } /* checkDate */


    /*-----------------------------------------------------------*/
    /*   format4DigitYear                                        */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will take a date, and make the year      */
    /*   have 4 digits. Year < 80 = 2000+Year, Year > 80 = 1900  */
    /*                                                           */
    /*   Parameters:                                             */
    /*      date       - The string of the date                  */
    /*      dateType   - The type of date, DATE or STAMP         */
    /*                                                           */
    /*   Returns:                                                */
    /*      String with the year in 4 digits                     */
    /*-----------------------------------------------------------*/
    public static String format4DigitYear ( String date, int dateType ) {
        String  retStr    = "" ;
        int     hours     = 0 ;
        int     minutes   = 0 ;
        int     seconds   = 0 ;
        int     days      = 0 ;
        int     month     = 0 ;
        int     year      = 0 ;

        if ( date != null ) {
            date = date.trim() ;

            hours   = parse ( date, DATE_HOUR,   dateType ) ;
            minutes = parse ( date, DATE_MINUTE, dateType ) ;
            seconds = parse ( date, DATE_SECOND, dateType ) ;
            days    = parse ( date, DATE_DAY,    dateType ) ;
            month   = parse ( date, DATE_MONTH,  dateType ) ;
            year    = parse ( date, DATE_YEAR,   dateType ) ;

            if ( ( year >= 0 ) &&
                 ( month > 0 ) &&
                 ( days > 0 ) )
                retStr = createDate ( year, month, days, hours, minutes, seconds, dateType ) ;
        } /* if date */

        return( retStr ) ;
    } /* format4DigitYear */

    /*-----------------------------------------------------------*/
    /*   isFutureDate()                                          */
    /*                                                           */
    /*   Description:                                            */
    /*      This function will determine if a date is in the     */
    /*   furure.                                                 */
    /*                                                           */
    /*   Parameters:                                             */
    /*      date - The date to check                             */
    /*                                                           */
    /*   Returns:                                                */
    /*      TRUE - the date is in the future.                    */
    /*-----------------------------------------------------------*/
    public static boolean isFutureDate (String futuredate, int dateType) {
        CDate today   = new CDate(DATE_DATE);
        double   jul1 = 0 ;
        double   jul2 = 0 ;
        boolean  ret  = false ;

        if ( futuredate != null ) {
            if ( ( futuredate.length() > 0 ) && ( today.strDate.length()> 0 ) ) {
                jul1 = toJulian ( futuredate, dateType ) ;
                jul2 = toJulian ( today.strDate, DATE_DATE ) ;

                if ( jul1 > jul2 )
                    ret = true ;
                else if ( jul1 < jul2 )
                    ret = false ;

            }
        } /* if */

        return( ret ) ;
    }

    /*-----------------------------------------------------------*/
    /*   addDaysToDate                                           */
    /*                                                           */
    /*   Description:                                            */
    /*      Adds the specified number of days to the date        */
    /*                                                           */
    /*                                                           */
    /*   Returns:                                                */
    /*      Date in gregorian format                             */
    /*-----------------------------------------------------------*/
    public static String addDaysToDate (String date, int days, int dateType) {
        if ( (date != null) && (date.length() > 0) ) {
            /*DEBUG*/
            //System.out.println ( "orignal date:"+date+" added days "+days);

            String newDate = null;
            double juliandate = toJulian(date, dateType);
            newDate = julianToGregorian( toJulian(date, dateType) + days , dateType);

            /*DEBUG*/
            //System.out.println ( "new     date:"+newDate+" Julian Days:"+juliandate );

            return( newDate ) ;
        } /* END IF */
        return(null);
    } /* addDaysToDate /*
 
    /*-----------------------------------------------------------*/
    /*   subtractDaysFromDate                                    */
    /*                                                           */
    /*   Description:                                            */
    /*      Adds the specified number of days to the date        */
    /*                                                           */
    /*                                                           */
    /*   Returns:                                                */
    /*      Date in gregorian format                             */
    /*-----------------------------------------------------------*/
    public static String subtractDaysFromDate (String date, int days, int dateType) {
        if ( (date != null) && (date.length() > 0) ) {
            /*DEBUG*/
            //System.out.println ( "orignal date:"+date+" added days "+days);

            String newDate = null;
            double juliandate = toJulian(date, dateType);
            newDate = julianToGregorian( toJulian(date, dateType) - days , dateType);

            /*DEBUG*/
            //System.out.println ( "new     date:"+newDate+" Julian Days:"+juliandate );

            return( newDate ) ;
        } /* END IF */
        return(null);
    } /* subtractDaysToDate /*
 
   /*-----------------------------------------------------------*/
    /*   addMonthsToDate                                         */
    /*                                                           */
    /*   Description:                                            */
    /*      Adds the specified number of Months to the date      */
    /*                                                           */
    /*                                                           */
    /*   Returns:                                                */
    /*      Date in gregorian format                             */
    /*-----------------------------------------------------------*/
    public static String addMonthsToDate (String date, int months, int dateType) {
        String newDate = "";
        if ( date != null ) {
            int day   = parse ( date, DATE_DAY, dateType ) ;
            int month = parse ( date, DATE_MONTH, dateType ) ;
            int year  = parse ( date, DATE_YEAR, dateType ) ;

            month += months;
            year  +=  (month-1) / 12;
            month  = (month % 12) > 0? (month % 12): month;

            newDate = createDate ( year, month,  day, 0, 0, 0, dateType ); 
        } /* if date */
        return(newDate);
    } /* addMonthsToDate */

    /*-----------------------------------------------------------*/
    /*   format2Digit()                                          */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will format a number to a 2 digits string*/
    /*                                                           */
    /*   Parameters:                                             */
    /*      digit                                                */
    /*                                                           */
    /*   Returns:                                                */
    /*      for the number 5, it returns 05                      */
    /*-----------------------------------------------------------*/
    public static String format2Digit ( int digit ) {
        String retStr = "" ;

        if ( digit < 10 )
            retStr = "0"+digit ;
        else
            retStr = ""+digit ;

        return( retStr ) ;
    } /* format2Digit */

    /*-----------------------------------------------------------*/
    /*   format4Digit()                                          */
    /*                                                           */
    /*   Description:                                            */
    /*      This method should ONLY BE USED FOR YEARS. It will   */
    /*   format a year to have 4 digits, instead of two          */
    /*                                                           */
    /*   Parameters:                                             */
    /*      year - The year to be formatted                      */
    /*                                                           */
    /*   Returns:                                                */
    /*      A string with the 4 digit year                       */
    /*-----------------------------------------------------------*/
    public static String format4Digit ( int year ) {
        if ( ( year > 74 ) && ( year < 100 ) )
            year+=1900 ;
        else if ( year < 74 )
            year+=2000 ;
        return( ""+year ) ;
    }

    /*-----------------------------------------------------------*/
    /*   getDayOfWeek  --                                        */
    /*                                                           */
    /*   This method will return the day of the week 0-6         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static int getDayOfWeek ( String date, int dateType ) {
        int weekDay = 0;

        if ( date != null )
            weekDay = (int) (toJulian(date, dateType) + 1.5) % 7;

        return(weekDay);
    }  /* GetDayOfWeek */

    /**
      * This method will return 1 for Jan or January, 2 for Feb or February etc..
      */
    public static int getMonthNumFromDescript ( String month ) {
        int rc = 0 ;

        month = month.toUpperCase() ;

        if ( month.startsWith ( "JAN" ) )
            rc = 1 ;
        else if ( month.startsWith ( "FEB" ) )
            rc = 2 ;
        else if ( month.startsWith ( "MAR" ) )
            rc = 3 ;
        else if ( month.startsWith ( "APR" ) )
            rc = 4 ;
        else if ( month.startsWith ( "MAY" ) )
            rc = 5 ;
        else if ( month.startsWith ( "JUN" ) )
            rc = 6 ;
        else if ( month.startsWith ( "JUL" ) )
            rc = 7 ;
        else if ( month.startsWith ( "AUG" ) )
            rc = 8 ;
        else if ( month.startsWith ( "SEP" ) )
            rc = 9 ;
        else if ( month.startsWith ( "OCT" ) )
            rc = 10 ;
        else if ( month.startsWith ( "NOV" ) )
            rc = 11 ;
        else if ( month.startsWith ( "DEC" ) )
            rc = 12 ;

        return ( rc ) ;
    }
} /* CDate */        

