package com.ibm.appsure.app.shared.util ;
/*-----------------------------------------------------------*/
/*   FileUtil                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FileUtil                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class holds static methods usefull for file manipulations 
 *
 *
 * @version 1.0, 5/12/1999 
 * @author  David Lentz
 */
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   FileUtil                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class FileUtil extends Object {

   /*------------------------------------------------------------*/
   /* readFile
    */
   /**
    *    This method will read in a file and return the
    *  contents of the file in a string
    *  
    *
    *  @param fileName The filename to read
    *  @param None
    *  @param None
    *
    *  @return String 
    *//*---------------------------------------------------------*/
   public static String readFile ( String fileName ) throws FileNotFoundException, IOException  {
      String retStr = null ;
      File              file      = new File ( fileName ) ;
      FileInputStream   in        = new FileInputStream ( file ) ;
      int               maxChunk  = 1024 ;
      byte[]            fByte     = new byte[ maxChunk ] ;
      int               i         = 0 ;
      int               readCount = 0 ;

      retStr = "" ;
      while ( readCount >= 0 ) {
         readCount = in.read ( fByte ) ;
         if ( readCount>= 0 )
            retStr = retStr+new String ( fByte, 0, readCount ) ;
      }

      in.close() ;

      return( retStr ) ;
   } /* loadFile */






   /*------------------------------------------------------------*/
   /* checkFileExist
    */
   /**
    *    This method will return TRUE if the
    *  passed filename exists
    *  
    *
    *  @param filename The filename of the file to check
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean exists ( String fileName ) {
      return( new File ( fileName ).exists() ) ;
   }





   /*------------------------------------------------------------*/
   /* rename
    */
   /**
    *    This method will rename one file to another
    *  filename
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if rename succeeds
    *//*---------------------------------------------------------*/
   public static boolean rename ( String originalFilename, String newFilename ) {
      return( new File ( originalFilename ).renameTo ( new File ( newFilename ) ) ) ;
   }


   /*------------------------------------------------------------*/
   /* saveFile
    */
   /**
    *    This method will save a string to a file.
    *  
    *  
    *
    *  @param filename The filename of the file
    *  @param string The String to save to the file
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean saveFile ( String filename, String string ) {
      boolean rc = false ;

      try {
         File             outFile = new File ( filename ) ;
         FileOutputStream out     = new FileOutputStream ( outFile ) ;

         out.write ( string.getBytes() ) ;
         out.close() ;
         outFile = null ;
         out     = null ;
         rc = true ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* dirExists
    */
   /**
    *    This method will check to see if a
    *  directory name exists in the current directory
    *  
    *
    *  @param dirName The name of the directory
    *  @param None
    *  @param None
    *
    *  @return TRUE = Exists, FALSE = does not exit
    *//*---------------------------------------------------------*/
   public static boolean dirExists ( String dirName ) {
      Properties prop    = System.getProperties() ;

      return( dirExists ( dirName, prop.getProperty ( "user.dir" ) ) ) ;
   }





   /*------------------------------------------------------------*/
   /* dirExists
    */
   /**
    *    This method will check to see if a directory
    *  exists in the specified directory.
    *  
    *
    *  @param dirName The directory name to look for
    *  @param inDir The directory to look in
    *  @param None
    *
    *  @return TRUE = Exists, FALSE = does not exit
    *//*---------------------------------------------------------*/
   public static boolean dirExists ( String dirName, String inDir ) {
      File     dir     = new File ( inDir ) ;
      String[] dirList = dir.list() ;
      int      i       = 0 ;
      boolean  found   = false ;

      while ( ( i < dirList.length ) && ( !found ) ) {
         if ( ( dirList[i] != null ) &&
              ( dirList[i].equals ( dirName ) ) )
            found = true ;
         i++ ;
      }

      return( found ) ;
   }

   /*------------------------------------------------------------*/
   /* deleteFile
    */
   /**
    *    This method will delete the specified
    *  file
    *  
    *
    *  @param fileName The name of the file to delete
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void deleteFile ( String fileName ) throws Exception {
      File file = new File ( fileName ) ;

      if ( file != null )
         file.delete() ;
   }





   /*------------------------------------------------------------*/
   /* appendToFile
    */
   /**
    *    This method will append the fileString passed
    *  to the fileName file.
    *  
    *
    *  @param fileName The name of the file to append to
    *  @param fileString The string to append to the file
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void appendToFile ( String fileName, String fileString ) throws Exception {
      RandomAccessFile file      = new RandomAccessFile ( fileName, "rw" ) ;

      file.seek( file.length() );
      file.write ( fileString.getBytes() ) ;
      file.close() ;
   }



   /*-----------------------------------------------------------*/
   /* copyFile
    */
   /**
    *    This method will copy a file
    *  
    *  
    *             
    *  @param source The source file
    *  @param dest The destination file
    *  @param 
    *
    *  @return 
    */
   public static boolean copyFile ( String source, String dest ) throws Exception {
      boolean rc = false ;
      File             inFile    = new File ( source ) ;
      File             outFile   = new File ( dest ) ;
      FileInputStream  in        = new FileInputStream ( inFile ) ;
      FileOutputStream out       = new FileOutputStream ( outFile ) ;
      int              readCount = 0 ;
      int              maxChunk  = 4096 ;
      byte[]           fByte     = new byte[ maxChunk ] ;

      copyStream ( in, out ) ;

      out.flush() ;
      in.close() ;
      out.close() ;

      in=null ;
      out=null;
      inFile=null ;
      outFile=null ;
      fByte=null ;
      System.gc();

      rc = true ;
      return( rc ) ;
   }

   /*------------------------------------------------------------*/
   /* copyStream
    */
   /**
    *    This method will copy the InputStream passed in
    *  to the OutputStream passed in
    *  
    *
    *  @param source The source (InputStream)
    *  @param dest The destination (OutputStream)
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void copyStream ( InputStream source, OutputStream dest ) throws IOException {

      synchronized(source) {
         synchronized(dest) {
            byte[] buffer    = new byte[256] ;
            int    bytesRead = 0 ;

            while ( true ) {
               bytesRead = source.read ( buffer ) ;
               if ( bytesRead == -1 )
                  break ;
               dest.write ( buffer, 0, bytesRead ) ;
            }
         }
      }
   }

   /*------------------------------------------------------------*/
   /* readBytesFromFile
    */
   /**
    *    This method will read in a file and return the
    *  contents of the file in a  byte array
    *  
    *
    *  @param fileName The filename to read
    *  @param None
    *  @param None
    *
    *  @return String 
    *//*---------------------------------------------------------*/
   public static byte[] readBytesFromFile ( String fileName ) throws FileNotFoundException, IOException  {
      String retStr = null ;
      File              file      = new File ( fileName ) ;
      FileInputStream   in        = new FileInputStream ( file ) ;
      int               maxChunk  = 1024 ;
      byte[]            fByte     = new byte[ maxChunk ] ;
      byte[]            retBytes  = null ;
      int               i         = 0 ;
      int               readCount = 0 ;

      retStr = "" ;
      while ( readCount >= 0 ) {
         readCount = in.read ( fByte ) ;
         if ( readCount>= 0 ) {
            if ( retBytes == null ) {
               retBytes = new byte[readCount] ;
               System.arraycopy ( fByte, 0, retBytes, 0, readCount ) ;
            } else {
               byte[] tempbyte = new byte[retBytes.length] ;

               System.arraycopy ( retBytes, 0, tempbyte, 0, retBytes.length ) ;
               retBytes = new byte [ tempbyte.length+readCount ] ;
               System.arraycopy ( tempbyte, 0, retBytes, 0, tempbyte.length ) ;
               System.arraycopy ( fByte, 0, retBytes, tempbyte.length, readCount ) ;
            }
         }
      }

      in.close() ;

      return( retBytes ) ;
   } /* loadFile */



   /*------------------------------------------------------------*/
   /* readObjectFromDisk
    */
   /**
    *    This method will deserialize a previously
    *  serialized object from the user's local
    *  hard drive
    *
    *  @param filename The filename of the file to deserialize
    *  @param None
    *  @param None
    *
    *  @return NULL if no file found or serialization error, Object otherwise
    *//*---------------------------------------------------------*/
   public static Object readObjectFromDisk ( String filename ) throws Exception {
      Object retObject = null ;
      FileInputStream f1 = new FileInputStream ( filename ) ;
      ObjectInputStream s1 = new ObjectInputStream ( f1 ) ;

      retObject = (Object) s1.readObject () ;

      s1.close() ;
      f1.close() ;

      return( retObject ) ;
   }





   /*------------------------------------------------------------*/
   /* writeObjectToDisk
    */
   /**
    *    This method will write a Object to the user's
    *  hard drive
    *  
    *
    *  @param filename The filename to save the object as
    *  @param Object The Object to serialize down
    *  @param None
    *
    *  @return TRUE if write was a success.
    *//*---------------------------------------------------------*/
   public static boolean writeObjectToDisk ( String filename, Object object ) throws Exception  {
      boolean            rc = false ;
      FileOutputStream   f  = new FileOutputStream ( filename ) ;
      ObjectOutputStream s  = new ObjectOutputStream ( f ) ;

      s.writeObject ( object ) ;
      s.flush() ;
      f.close() ;
      s.close() ;

      rc = true ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* containsPathInfo
    */
   /**
    *    This method will return TRUE if the filename
    *  passed contains any path information.
    *  
    *
    *  @param filename The filename to check
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public static boolean containsPathInfo ( String filename ) {
      return( filename.indexOf ( File.separator ) >= 0 ) ;
   }




   /*------------------------------------------------------------*/
   /* filenameFromPath
    */
   /**
    *    This method will return the filename of the
    *  file specified in the full path, stripping out
    *  all path information
    *
    *  @param filename The filename to strip and return
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String filenameFromPath ( String filename ) {
      if ( containsPathInfo ( filename ) ) {
         return( filename.substring ( filename.lastIndexOf ( File.separator ) ) ) ;
      }

      return( filename ) ;
   }

} /* FileUtil */
