package com.ibm.appsure.app.shared.util ;   
   /*********************************************************
   * This class automatically determines how to find a resource
   * needed by a servlet or application.  This class is a class loader which
   * will find a file by the classpath of the VM in which this
   * servlet is running.  This allows path independance in 
   * environments which can not be controlled in production
   * installations.  Due to the search method, which searches the
   * entire classpath for your resource, your resources should be
   * fully qualified to prevent any conflict with other applications
   * which may be sharing the same VM in the production environment.
   * This class also allow us to have one central property file for
   * global parameters attributes for all applications running in the
   * same environment.  This is used in development for turning on 
   * debugging across all applications at once.
   * The getResourceAsStream method will load any file from the 
   * classpath of the VM.  Use this for loading property files.
   * 
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 3/28/100  7:38pm
   * @author Chad Meadows
   * @version 1.0
   *********************************************************/
import java.io.* ;
import java.util.* ;
import com.ibm.nzna.shared.io.* ;

public class ResourceLoader extends Object {
    static ClassLoader resourceClassLoader = null ;
    /*********************************************************
    * Finds a resource from the classpath and returns an
    * inputStream for reading the resource.
    * 
    *
    * Author:  Chad Meadows
    * Created: 3/29/100  3:38pm
    * @param
    * @return
    *********************************************************/
    /**
     * A resource is some data (images, audio, text, etc) that wants to be
     * accessed by some class code in a way that is independent of the
     * location of the code.  Resources are found with cooperation of the
     * class loaders, since they are the only ones who know where the class
     * actually came from. <p>
     *
     * System resources are those that are handled by the host implemenation
     * directly.  For example, they may be located in the CLASSPATH.<p>
     *
     * The name of a resource is a "/"-separated sequence of identifiers.
     * The class Class provides convenience methods for accessing resources;
     * the methods implement a convention where the package name is prefixed
     * to the short name of the resource.<p>
     * 
     * Resources can be accessed as an InputStream, or as a URL.
     *
     * @see Class
     */

    /**
     * Get an InputStream on a given resource..  Will return null if no
     * resource with this name is found. <p>
     *
     * The resource name may be any system resource (e.g. follows CLASSPATH
     * order).
     *
     * @param  name  the name of the resource, to be used as is.
     * @return an InputStream on the resource, or null if not found.
     * @since   JDK1.1
     */
    public static InputStream getInputStream(String name ) {
        /* Get the class Loader of our Loader */
        ClassLoader classLoader = getClassLoader() ;
        if ( classLoader != null ) {
           return classLoader.getResourceAsStream(name);
        }
        else {
           System.out.println("ResourceLoader unable to get ClassLoader") ;
           return ( null ) ;
        }
    }

   public static ClassLoader getClassLoader() {
        if ( resourceClassLoader == null ) {
            ResourceLoader resourceLoader = new ResourceLoader () ;
            Class classObject = resourceLoader.getClass() ;
            ClassLoader classLoader = classObject.getClassLoader() ;
            resourceClassLoader = classLoader ;
        }
        return ( resourceClassLoader ) ;
    }
    public static byte[] load(String name ) {
       byte[] bytes = null ;
       /* Get the input stream to the file using */
       /* the class loader                       */
       InputStream io = getInputStream( name ) ;
       /* if the resource is found */
       if ( io != null ) {
          /* Read the contents of the input stream  */
          bytes = InputStreamLoader.load ( io ) ;
       }
       return ( bytes ) ;
    }
    /*********************************************************
    * Get fully qualified path to the resource by searching the
    * classpath.
    * 
    *
    * Author:  
    * Created: 6/21/100  7:16pm
    * @param
    * @return The full path or null if not found.
    *********************************************************/
    public static final String getPath (String name ) {
       /* We must find the classpath entry which contains this resource */
       Properties properties = System.getProperties() ;
       String classpath       = (String) properties.get("java.class.path") ;
       String pathSeparator   = (String) properties.get("path.separator" ) ;
       StringTokenizer tokenizer = new StringTokenizer( classpath, pathSeparator ) ;
       while (tokenizer.hasMoreElements() ) {
          String path = (String ) tokenizer.nextElement() ;
          /* If this is a zip or jar file skip */
          if ((path.endsWith(".zip")) || (path.endsWith(".jar"))) {
             continue ;
          }
          try {
             File file = new File( path, name  ) ;
             if ( file.exists() ) {
                return ( path ) ;
             }
          }
          catch (Exception e ) {}
       }
       return ( null ) ;
    }
}
