package com.ibm.appsure.app.shared.util ;
/*-----------------------------------------------------------*/
/*   StoreUtil                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StoreUtil                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class holds static methods for use with storage mechanisms ( Vectors, arrays, Hashtables etc.. ) 
 *
 *
 * @version 1.0, 5/5/1999 
 * @author  David Lentz
 */
import java.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   StoreUtil                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class StoreUtil extends Object {





   /*------------------------------------------------------------*/
   /* vectorContainsInteger
    */
   /**
    *    This method will check a vector to see if
    *  it contains the int passed. The vector must
    *  contain only Integer objects.
    *
    *  @param vec The vector to look in
    *  @param num The number to check for
    *  @param None
    *
    *  @return TRUE if the num value is  in the vector, FALSE if otherwise
    *//*---------------------------------------------------------*/
   public static boolean vectorContainsInteger ( Vector vec, int num ) {
      boolean found  = false ;
      int     i      = 0 ;
      int     length = 0 ;

      if ( vec != null ) {
         Integer intObject = null ;

         length = vec.size() ;

         while ( ( !found ) && ( i < length ) ) {
            intObject = (Integer) vec.elementAt(i) ;
            found = ( intObject.intValue() == num ) ;
            i++ ;
         }
      }

      return ( found ) ;
   }

   /*------------------------------------------------------------*/
   /* vectorContains
    */
   /**
    *    This method will return TRUE if the
    *  passed vector contains the passed object. This
    *  is different then the Vector.contains because it calls
    *  the object.equals method to compare, instead of instance comparing
    *
    *  @param vec The Vector to check
    *  @param object The object to look for
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean vectorContains ( Vector vec, Object object ) {
      boolean rc   = false ;

      if ( vec != null ) {
         int     i    =  0 ;
         int     size = vec.size() ;

         try {
            while ( ( !rc ) && ( i < size ) ) {
               rc = vec.elementAt(i).equals ( object ) ;
               i++ ;
            }
         } catch ( Exception e ) {
            e.printStackTrace() ;
         }
      }

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* vectorContains
    */
   /**
    *    This method will remove an object from the        
    *  passed vector. This method  
    *  is different then the Vector.removeElement because it calls
    *  the object.equals method to compare, instead of instance comparing
    *  with the operator. It will also remove all instances of the object
    *  passed in.
    *
    *  @param vec The Vector to check
    *  @param object The object to remove
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeFromVector ( Vector vec, Object object ) {

      if ( vec != null ) {
         int     i    =  0 ;
         int     size = vec.size() ;

         try {

            while ( i < size ) {
               if ( vec.elementAt(i).equals ( object ) ) {
                  vec.removeElementAt(i) ;
                  i=0 ;
                  size-- ;
               }

               i++ ;
            } /* while */

         } catch ( Exception e ) {
            e.printStackTrace() ;
         }
      }

   }


} /* StoreUtil */
