package com.ibm.appsure.app.shared.util ;
/*-----------------------------------------------------------*/
/*   Text                                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Text                                */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This is a simple class of usefull, static methods for manipulating text
 *
 *
 * @version 1.0, 12/28/99 
 * @author  David Lentz
 */
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   Text                                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class Text extends Object {





    /*------------------------------------------------------------*/
    /* exceptionToString
     */
    /**
     *    This method will take an Exception and
     *  write out the complete stack information to
     *  a string
     *
     *  @param e The exception
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public static String exceptionToString ( Exception e ) {
        StringWriter    sw      = new StringWriter () ;
        PrintWriter     pw      = new PrintWriter ( sw ) ;
        String          retStr  = null ;

        /* Get the stackTrace */
        e.printStackTrace ( pw ) ;
        retStr = sw.toString() ;
        sw = null ;
        pw = null ;
        e  = null ;

        return( retStr ) ;
    }





    /*------------------------------------------------------------*/
    /* replaceStrInStr
     */
    /**
     *    This method will replace the first occurance of searchStr
     *  with replaceStr, inside of the string passed
     *  
     *
     *  @param string The String to replace inside of
     *  @param searchStr The String to search for
     *  @param replaceStr The String to replace searchStr with
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static String replaceInStr ( String string, String searchStr, String replaceStr ) {
        int strIndex = string.indexOf ( searchStr ) ;

        if ( strIndex > -1 )
            return( string.substring ( 0, strIndex )+replaceStr+string.substring ( strIndex+searchStr.length() ) ) ;
        else
            return( string ) ;

    } /* replaceStrInStr */

    /*------------------------------------------------------------*/
    /* replaceStrInStrFrom
     */
    /**
     *    This method will replace the first occurance of searchStr
     *  with replaceStr, inside of the string passed, from the index
     *  fromPoint.
     *  
     *
     *  @param string The String to replace inside of
     *  @param fromPoint The index of the character to start replacing after
     *  @param searchStr The String to search for
     *  @param replaceStr The String to replace searchStr with
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static String replaceStrInStrFrom ( String string, int fromPoint, String searchStr, String replaceStr ) {
        String fromString = string.substring ( fromPoint, string.length() ) ;
        int strIndex = fromString.indexOf ( searchStr ) ;

        if ( strIndex > -1 ) {
            fromString = fromString.substring ( 0, strIndex )+replaceStr+fromString.substring ( strIndex+searchStr.length() ) ;
            return( string.substring ( 0, fromPoint )+fromString ) ;
        } else
            return( string ) ;

    } /* replaceInStrFrom */


    /*------------------------------------------------------------*/
    /* replaceAllStrInStr
     */
    /**
     *    This method will replace all occurances of searchStr
     *  with replaceStr, inside of the string passed
     *  
     *
     *  @param string The String to replace inside of
     *  @param searchStr The String to search for
     *  @param replaceStr The String to replace searchStr with
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static String replaceAllStrInStr ( String string, String searchStr, String replaceStr ) {
        String retStr         = "" ;
        int    startIndex     = string.indexOf ( searchStr ) ;
        int    prevStartIndex = startIndex ;
        int    stopIndex      = string.length() ;
        int    replaceStrLen  = searchStr.length() ;

        /* If we don't need to worry about the
           string, don't */
        if ( startIndex > -1 ) {
            StringBuffer retBuffer  = new StringBuffer ( string.substring ( 0, startIndex )+replaceStr ) ;
            boolean      cont       = true ;

            while ( startIndex > -1 ) {
                prevStartIndex = startIndex ;
                startIndex = string.indexOf ( searchStr, startIndex+replaceStrLen ) ;

                if ( startIndex > -1 ) {
                    retBuffer.append ( string.substring ( prevStartIndex+replaceStrLen, startIndex ) ) ;
                    retBuffer.append ( replaceStr ) ;
                }
            }

            retBuffer.append ( string.substring ( prevStartIndex+replaceStrLen ) ) ;
            retStr = retBuffer.toString() ;
            retBuffer = null ;

        } else
            retStr = string ;


        return( retStr ) ;
    } /* replaceAllInStr */

    /*------------------------------------------------------------*/
    /* replaceAllStrInStrFrom
     */
    /**
     *    This method will replace all occurances of searchStr
     *  with replaceStr, inside of the string passed, after the
     *  specified index.
     *  
     *
     *  @param string The String to replace inside of
     *  @param fromPoint The point to start replacing after
     *  @param searchStr The String to search for
     *  @param replaceStr The String to replace searchStr with
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static String replaceAllStrInStrFrom ( String string, int fromPoint, String searchStr, String replaceStr ) {
        String retStr = "" ;
        String fromString = string.substring ( fromPoint, string.length() ) ;

        retStr = replaceInStr ( fromString, searchStr, replaceStr ) ;

        while ( retStr.indexOf ( searchStr ) >= 0 ) {
            retStr = replaceInStr ( retStr, searchStr, replaceStr ) ;
        }

        retStr = string.substring ( 0, fromPoint )+retStr ;

        return( retStr ) ;
    } /* replaceAllInStringFrom */


    /*------------------------------------------------------------*/
    /* replaceAllStrInStrFromTo
     */
    /**
     *    This method will replace all occurances of searchStr
     *  with replaceStr, inside of the string passed from the index
     *  passed as fromPoint, to the index passed as toPoint
     *  
     *
     *  @param string The String to replace inside of
     *  @param searchStr The String to search for
     *  @param replaceStr The String to replace searchStr with
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static String replaceAllInStrFrom ( String string, int fromPoint, int toPoint, String searchStr, String replaceStr ) {
        String retStr = "" ;
        String fromString = string.substring ( fromPoint, toPoint ) ;

        retStr = replaceInStr ( fromString, searchStr, replaceStr ) ;

        while ( retStr.indexOf ( searchStr ) >= 0 ) {
            retStr = replaceInStr ( retStr, searchStr, replaceStr ) ;
        }

        retStr = string.substring ( 0, fromPoint )+retStr+string.substring ( toPoint, string.length() ) ;

        return( retStr ) ;
    } /* replaceAllInStringFromTo */

    /*------------------------------------------------------------*/
    /* stripPath
     */
    /**
     *    This method will strip the path from a filename
     *  
     *  
     *
     *  @param filename The Filename to strip
     *  @param None
     *  @param None
     *
     *  @return The Stripped filename
     *//*---------------------------------------------------------*/
    public static String stripPath ( String filename ) {
        String retStr = filename ;

        try {
            if ( filename.indexOf ( File.separator ) != -1 )
                retStr = filename.substring ( filename.lastIndexOf ( File.separator )+1, filename.length() ) ;
        } catch ( Exception e ) {
            e.printStackTrace() ;
        }

        return( retStr ) ;
    }


    /*-----------------------------------------------------------*/
    /* containsHTML
     */
    /**
     *    This method will determine if a string
     *  contains any  HTML tags.
     *  
     *             
     *  @param string The string to check
     *  @param 
     *  @param 
     *
     *  @return TRUE The string contains HTML Tags
     */
    public static boolean containsHTML ( String string ) {
        boolean rc = false ;

        if ( string != null ) {
            String checkStr = string.toUpperCase() ;

            rc = ( ( checkStr.indexOf ( "<HTML" ) != -1 ) || 
                   ( checkStr.indexOf ( "</" ) != -1 ) || 
                   ( checkStr.indexOf ( "<BODY" ) != -1 ) || 
                   ( checkStr.indexOf ( "<FONT" ) != -1 ) || 
                   ( checkStr.indexOf ( "IMG SRC" ) != -1 ) || 
                   ( checkStr.indexOf ( "<B>" ) != -1 ) || 
                   ( checkStr.indexOf ( "<I>" ) != -1 ) || 
                   ( checkStr.indexOf ( "HTML" ) != -1 ) || 
                   ( checkStr.indexOf ( "HREF" ) != -1 ) || 
                   ( checkStr.indexOf ( "<PRE" ) != -1 ) || 
                   ( checkStr.indexOf ( "<TABLE" ) != -1 ) || 
                   ( checkStr.indexOf ( "<TD" ) != -1 ) || 
                   ( checkStr.indexOf ( "<TR" ) != -1 ) ||
                   ( checkStr.indexOf ( "HELVETICA" ) != -1 ) || 
                   ( checkStr.indexOf ( "COURIER" ) != -1 ) || 
                   ( checkStr.indexOf ( "BGCOLOR" ) != -1 ) || 
                   ( checkStr.indexOf ( "APPLET" ) != -1 ) || 
                   ( checkStr.indexOf ( "FORM" ) != -1 ) || 
                   ( checkStr.indexOf ( "<HR" ) != -1 ) || 
                   ( checkStr.indexOf ( "COLOR=" ) != -1 ) || 
                   ( checkStr.indexOf ( "COLOR =" ) != -1 ) || 
                   ( checkStr.indexOf ( "<!" ) != -1 ) || 
                   ( checkStr.indexOf ( "FGCOLOR" ) != -1 ) ) ;

        }

        return( rc ) ;
    }


    /*------------------------------------------------------------*/
    /* formatStringLineBreaks
     */
    /**
     *    This method will format a string for a certain
     *  number of chars per line noticing the Line
     *  breaks in the String.
     *
     *  @param string The String
     *  @param length The number of chars per line
     *  @param None
     *
     *  @return String[] An array of strings broken down
     *//*---------------------------------------------------------*/
    public static String[] formatStringLineBreaks ( String string, int length ) {
        String[]          retArray  = null ;

        if ( string != null ) {
            Vector            retVec    = new Vector ( 1, 10 ) ;
            Vector            stringVec = new Vector ( 1, 10 ) ;
            StringTokenizer   strtok    = null ;

            /* We add a space between two carriage returns
               because the tokenizer won't pick them up if
               they appear together! */
            string = replaceAllStrInStr ( string, "\n\n", "\n \n" ) ;
            strtok = new StringTokenizer ( string, "\n" ) ; 

            if ( strtok.countTokens() <= 1 )
                return( formatStringLines ( string, length ) ) ;
            else {

                /* Add all of the string arrays to our vector */
                while ( strtok.hasMoreTokens() )
                    retVec.addElement ( formatStringLines ( strtok.nextToken(), length ) ) ;
            } /* else */

            retVec.trimToSize() ;
            if ( retVec.size() > 1 ) {
                int      i         = 0 ;
                int      vLength    = retVec.size() ;
                int      a         = 0 ;
                String[] tempArray = null ;

                while ( i < vLength ) {
                    tempArray = (String[]) retVec.elementAt(i) ;
                    if ( tempArray != null ) {
                        for ( a = 0; a < tempArray.length; a++ )
                            stringVec.addElement(tempArray[a]) ;
                    }
                    i++ ;
                }
                stringVec.trimToSize() ;
                retArray = new String[stringVec.size()] ;
                stringVec.copyInto ( retArray ) ;
            }
            retVec = null ;
            stringVec = null ;
            return( retArray ) ;
        }

        retArray = new String[1] ;
        retArray[0] = new String ( "" ) ;

        return( retArray ) ;
    } /* formatStringLineBreaks */

    /*-----------------------------------------------------------*/
    /* formatStringLength
     */
    /**
     *    This method will format a string length to
     *  a specified length per row
     *  
     *
     *  @param string The String to format
     *  @param length The number of characters per line to return
     *  @param None
     *
     *  @return String The formatted string
     *//*---------------------------------------------------------*/
    public static String formatStringLength ( String string, int length ) {
        return( formatStringLength ( string, length, true ) ) ;
    }
    public static String formatStringLength ( String string, int length, boolean cleanString ) {
        boolean         cont    = true ;
        int             nextRet = length ;
        int             index   = 0 ;
        String          retStr  = "" ;
        StringTokenizer token   = null ;
        StringBuffer    buf     = null ;

        if ( string != null ) {

            /* First, clean up the string we were sent */
            if ( cleanString )
                string = cleanString ( string ) ;


            /*---------------------------------------------------------------*/
            /*                                                               */
            /*   If the length is shorter than the string, wrap the string   */
            /*                                                               */
            /*---------------------------------------------------------------*/
            if ( length < string.length() ) {

                System.out.println ( "Wrapping. Length:"+length+"  String Length:"+string.length() ) ;
                /* Create our String Tokenizer */
                token = new StringTokenizer ( string ) ;

                /* Ok, now create a string with the words and single spaces in
                   between the words */
                while ( token.hasMoreTokens () ) {
                    retStr = retStr.concat ( token.nextToken () ) ;
                    retStr = retStr.concat ( " " ) ;
                } /* while */


                if ( retStr != null ) {

                    if ( retStr.length() > length ) {

                        buf = new StringBuffer ( retStr ) ;

                        /* Loop through ,making each space closest to the 80 a carriage return */
                        while ( cont ) {

                            /* Get the last position of a space, before
                               the length */
                            index = retStr.lastIndexOf ( " ", nextRet ) ;

                            /* If there are no more spaces, get out of here */
                            if ( index < 1 )
                                cont = false ;
                            else {

                                /* If this index is valid, put a return character
                                   in and set our index back */
                                if ( index < retStr.length() ) {
                                    buf.setCharAt ( index, '\n' ) ;
                                    nextRet=index+length ;
                                    if ( nextRet > retStr.length() )
                                        cont = false ;
                                } else
                                    cont = false ;
                            } /* else */

                        } /* cont */
                    } /* if length */
                    else
                        buf = new StringBuffer ( retStr ) ;
                } /* if */

                retStr = buf.toString().trim() ;
            } /* if */

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, pad the string                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
                buf = new StringBuffer ( length ) ;

                buf.insert (0, string ) ;
                while ( buf.length() <= length )
                    buf.append (" ") ;

                retStr = buf.toString() ;
            }
        } /* if string  */
        return( retStr ) ;

    } /* formatStringLength */


    /*-----------------------------------------------------------*/
    /* formatStringLines
     */
    /**
     *    This function is similar to the formatString function. It will
     *  format a string to a specified length per line, but
     *  will return the string broken into an array of lines.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return array of string
     *//*---------------------------------------------------------*/
    public static String[] formatStringLines ( String string, int length ) {
        String[]         retStr    = null ;

        if ( string != null ) {
            boolean          cont      = true ;
            int              nextRet   = length ;
            int              index     = 0 ;
            StringBuffer     buf       = new StringBuffer(string) ;
            int              lastIndex = 0 ;
            int              i         = 0 ;
            int              len       = 0 ;
            Vector           vector    = new Vector ( 30, 5 ) ;
            String           newStr    = null ;
            String           nextWord  = "" ;
            StringTokenizer  token     = null ;

            try {

                /* First, send this string through our normal formatter, so
                   we know where each line is supposed to be */
                string = cleanString ( string ) ;

                if ( string != null ) {


                    /* First of all, if this is just 1 line of text,
                       make 1 element in the array and populate it */
                    if ( string.length() < length ) {
                        retStr = new String[1] ;
                        retStr[0] = new String ( string ) ;
                    } else {

                        token = new StringTokenizer ( string ) ;

                        /* Ok, loop through all of the words */
                        newStr = "" ;
                        while ( token.hasMoreTokens () ) {

                            /* Get the next word */
                            nextWord = token.nextToken () ;

                            /* Ok, if the length we have now, plus the length
                               of this word is greater than our format length,
                               start a new line */
                            if ( (newStr.length()+nextWord.length()+1) > length ) {
                                vector.addElement ( newStr ) ;
                                newStr = new String ( nextWord ) ;
                                newStr = newStr.concat ( " " ) ;
                            } /* if */
                            else {
                                newStr = newStr.concat ( nextWord ) ;
                                newStr = newStr.concat ( " " ) ;
                            } /* else */
                        } /* while */   

                        /* OK, trim our vector, create an array, and copy
                           our strings over to it */
                        if ( newStr.length () > 0 )
                            vector.addElement ( newStr ) ;
                        vector.trimToSize () ;
                        retStr = new String [ vector.size() ] ;
                        vector.copyInto ( retStr ) ;
                    } /* if */
                } /* if */
            } catch ( Exception e ) {
                e.printStackTrace() ;
            }
        }
        return( retStr ) ;

    } /* formatStringLines */

    /*-----------------------------------------------------------*/
    /* cleanString
     */
    /**
     *    This method will strip a string of all cariage returns,
     *  and will trim the string
     *  
     *
     *  @param string The String to clean
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static String cleanString ( String string ) {
        return( string ) ;
    } /* cleanString */   




    /*-----------------------------------------------------------*/
    /* cleanDBString
     */
    /**
     *    Cleans a string for use in a database
     *  
     *  
     *
     *  @param String the String to clean
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static String cleanDBString ( String string ) {
        String retStr = string ;

        if ( retStr  != null ) {
            if ( retStr.indexOf ( "''" ) >= 0 )
                retStr = replaceAllStrInStr ( retStr, "''", "'" ) ;

            retStr = replaceAllStrInStr ( retStr, "'", "''" ) ;
            retStr = cleanString ( retStr ).trim() ;
        } else
            retStr = "NULL" ;

        return( retStr ) ;
    }


    /*------------------------------------------------------------*/
    /* cleanNullableDBString
     */
    /**
     *     This method will do the following to
     *  any string passed:<br><br>
     *
     *  1. If the String passed is NULL, the string will be assigned the value 'NULL'
     *  2. If the String passed is Not null, all single quotes will be changed to two single quotes
     *  3. The String returned will be enclosed in single quotes.<br><br>
     *
     *  The string returned can be used directly in an SQL String, for example:<br>
     *  INSERT INTO MYTABLE ( FIRSTNAME ) values ( "+cleanNullableDBString ( firstName )+" )
     *
     *  Examples:<br><br>
     *
     *  <pre>
     *  Passed String          String Returned<br>
     *  ---------------------- -------------------------<br>
     *  I can't wait           'I can''t wait'<br>
     *  NULL value             NULL<br>
     *  DLENTZ                 'DLENTZ'<br>
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return cleaned string
     *//*---------------------------------------------------------*/
    public static String cleanNullableDBString ( String dirtyString ) {
        if ( dirtyString == null )
            return ( "NULL" ) ;

        if ( dirtyString.indexOf ( "'" ) >= 0 )
            dirtyString = replaceAllStrInStr ( dirtyString, "'", "''" ) ;

        dirtyString = cleanString ( dirtyString.trim() ) ;

        return ( "'"+dirtyString+"'" ) ;
    }



    /*------------------------------------------------------------*/
    /* stripHTML
     */
    /**
     *    This method will strip out all HTML in the
     *  string provided and will return the new string
     *  without HTML
     *
     *  @param htmlString The String containing HTML
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public static String stripHTML ( String htmlString ) {
        String returnStr = htmlString ;

        if ( ( returnStr != null ) && ( returnStr.length() > 0 ) ) {
            StringTokenizer strtok = new StringTokenizer ( htmlString, "<" ) ;
            String          curtok = null ;

            returnStr = "" ;
            while ( strtok.hasMoreTokens() ) {
                curtok = strtok.nextToken() ;
                if ( curtok.indexOf ( ">" ) >= 0 ) {
                    curtok = curtok.substring ( curtok.indexOf ( ">" )+1, curtok.length() ) ;
                }

                returnStr+=" "+curtok ;
            }

            returnStr = replaceAllStrInStr ( returnStr, "&nbsp;", "" ) ;
            returnStr = replaceAllStrInStr ( returnStr, "&NBSP;", "" ) ;
        }

        return( returnStr ) ;
    }





    /*------------------------------------------------------------*/
    /* stripLineBreaks
     */
    /**
     *    This method will replace all line breaks in the
     *  line passed in with nothing, effectivly removing
     *  each line break
     *
     *  @param string The String to remove line breaks from
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public static String stripLineBreaks ( String string ) {
        return( replaceAllStrInStr ( string, "\n", "" ) ) ;
    }





    /*------------------------------------------------------------*/
    /* getYN
     */
    /**
     *    This method will return the string 'Y' or the
     *  string 'N' depending on the value of the
     *  boolean value passed in
     *
     *  @param value The boolean value to translate
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static String getYN ( boolean value ) {
        return( value ? "Y" : "N" ) ;
    }
} /* Text */

