package com.ibm.appsure.app.shared.util ;    
/*-----------------------------------------------------------*/
/*   URLValidator                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: URLValidator                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.net.* ;
/**
 * This class will basically only validate that a URL exists and returns data 
 *
 *
 * @version 1.0, 5/8/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   URLValidator                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class URLValidator extends Object {





   /*------------------------------------------------------------*/
   /* validateUrl
    */
   /**
    *    This method will validate that a URL exists
    *  by performing the following tasks:<br><pre>
    *
    *  1. Connect to the URL
    *  2. Read in all bytes from the URL<br></pre>
    *
    *  If any of the steps fail, an exception will be thrown
    *  that indicates the failure.
    *  
    *
    *  @param urlStr The Universal Resource Locater to validate
    *
    *  @return TRUE if the URL was loaded, FALSE if not
    *//*---------------------------------------------------------*/
   public static boolean validateUrl ( String urlStr ) throws Exception {
      boolean             loaded    = false ;
      URL                 url       = new URL ( urlStr ) ;
      URLConnection       urlConn   = url.openConnection() ;
      BufferedInputStream inStr     = new BufferedInputStream ( urlConn.getInputStream() ) ;
      byte[]              readBytes = new byte[1024] ;
      int                 byteCount = 0 ;
      int                 readCount = 0 ;

      while ( readCount >= 0 ) {
         readCount = inStr.read ( readBytes, 0, readBytes.length ) ;
         if ( readCount > -1 )
            byteCount+=readCount ;
      }

      loaded = ( byteCount > 0 ) ;
      inStr.close() ;
      url = null ;
      urlConn = null ;

      return( loaded ) ;
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will simply return the title to the
    *  HTML Page that is requested through the
    *  URL passed in
    *
    *  @param url The URL to load
    *  @param None
    *  @param None
    *
    *  @return String, NULL if the page could not be loaded
    *//*---------------------------------------------------------*/
   public static String getTitle ( String urlStr ) throws Exception {
      boolean             loaded    = false ;
      URL                 url       = new URL ( urlStr ) ;
      URLConnection       urlConn   = url.openConnection() ;
      BufferedInputStream inStream  = new BufferedInputStream ( urlConn.getInputStream() ) ;
      byte[]              readBytes = new byte[1024] ;
      int                 byteCount = 0 ;
      int                 readCount = 0 ;
      String              readStr   = "" ;
      String              title     = null ;

      while ( readCount >= 0 ) {
         readCount = inStream.read ( readBytes ) ;
         if ( readCount>= 0 )
            readStr = readStr+new String ( readBytes, 0, readCount ) ;
      }

      if ( ( readStr.length() > 0 ) && (  readStr != null ) ) {
         int titleIndexStart = readStr.toUpperCase().indexOf ( "<TITLE>" ) ;
         int titleIndexEnd   = readStr.toUpperCase().indexOf ( "</TITLE>" ) ;

         if ( ( titleIndexStart >= 0 ) && 
              ( titleIndexEnd >= 0 ) && 
              ( titleIndexEnd > titleIndexStart ) ) {
            title = readStr.substring ( titleIndexStart+7, titleIndexEnd ) ;
         } else
            title="" ;
      }

      loaded = ( byteCount > 0 ) ;
      inStream.close() ;
      url = null ;
      urlConn = null ;

      return( title ) ;
   }

} /* URLValidator */
