package com.ibm.appsure.app.shared.util ;    
/*-----------------------------------------------------------*/
/*   Unzip                                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Unzip                               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.io.* ;
import java.util.zip.* ;
import java.util.* ;
/**
 * This is the program will basically unzip the file that is 
 * passed in as a parameter. If there is a gif file called 'unzip.gif'
 * in the directory that this program was envoked 
 * it will display that gif while loading. Otherwise it will only
 * display a white box.<br><br>
 *
 * The zip file passed in as parameter 1 will be unzipped in the directory
 * that this program was invoked.<br><br>
 *
 * If you wish to use this class to unzip a file from within another Java
 * program you may simply call the static method 'unzip' and pass it the
 * name of the zip file you wish to unzip.
 *
 *
 * @version 1.0, 1/14/1999 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Unzip                                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class Unzip {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Frame        tempFrame        = null ;
   private UnzipWindow  outputWindow     = null ;



   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This is the main entry point for the application
    *  The command line parameters taken is
    *  the URL of the code update.
    *
    *  @param args Command line args
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String[] args ) {

      if ( ( args == null ) || ( args.length != 1 ) )
         displayUsage() ;
      else
         new Unzip ( args[0] ) ;
   }




   /*------------------------------------------------------------*/
   /* displayUsage
    */
   /**
    *    Shows how to call the application
    *  from the command line
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void displayUsage () {
      System.out.println ( "Usage:" ) ;
      System.out.println ( "java Unzip <zip filename>" ) ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will construct our frame and
    *  GUI system
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Unzip ( String zipfile ) {
      Image  image        = Toolkit.getDefaultToolkit().getImage ( "unzip.gif" ) ;
      int    width        = 0 ;
      int    height       = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */     
      /*                                                           */
      /*-----------------------------------------------------------*/
      outputWindow = new UnzipWindow ( tempFrame = new Frame() ) ;
      waitForImage ( outputWindow, image ) ;
      width=image.getWidth ( outputWindow ) ;
      height=image.getHeight ( outputWindow ) ;

      outputWindow.setImage ( image ) ;
      outputWindow.setSize  ( width, height ) ;
      WinUtil.centerWindow  ( outputWindow ) ;
      outputWindow.setVisible ( true ) ;

      unzip ( zipfile ) ;
      System.exit(0) ;
   }




   /*-----------------------------------------------------------*/
   /* waitForImage
    */
   /**
    *    Waits for an image to load an then returns
    *  
    *  
    *
    *  @param component The ImageObserver to be used
    *  @param image The image to wait for
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void waitForImage(Component component, Image image) {
      MediaTracker tracker = new MediaTracker(component);
      if ( ( image != null ) && ( component != null ) ) {
         try {
            tracker.addImage(image, 0);
            tracker.waitForID(0);
         } catch ( InterruptedException e ) {
         }
      }
   }





   /*------------------------------------------------------------*/
   /* unzip
    */
   /**
    *    This method will unzip a file into the current
    *  directory
    *  
    *
    *  @param zipfile The zip file to unzip
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything was unzipped
    *//*---------------------------------------------------------*/
   public static boolean unzip ( String zipfile ) {
      Properties  prop          = System.getProperties();
      String      outDir        = "." ;
      String      filePath      = null ;
      File        file          = null ;
      File        root          = null ;
      ZipFile     zin           = null ;
      Enumeration enum          = null ;
      byte[]      buffer        = null ;
      String      currDir       = null ;
      ZipEntry    ze            = null ;
      String      name          = null ;
      File        dir           = null ;
      boolean     cont          = true ;
      int         fileCount     = 0 ;
      int         fileIndex     = 0 ;
      boolean     rc            = true ;
      int         length        = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the log system                                   */            
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {

         if ( outDir.endsWith(File.separator) )
            outDir = outDir.substring(0, outDir.length() - 1);

         // read the .zip file
         zin = new ZipFile( zipfile );
         root = new File(outDir);

         if ( !root.exists() )
            root.mkdirs();

         enum = zin.entries();
         buffer = new byte[1024];
         currDir = outDir;               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, count the entries                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( enum.hasMoreElements() ) {
            fileCount++ ;
            enum.nextElement() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now loop through writing each file                  */                             
         /*                                                           */
         /*-----------------------------------------------------------*/
         enum = zin.entries() ;
         while ( ( enum.hasMoreElements() ) && ( cont ) ) {

            fileIndex++ ;

            ze = (ZipEntry)enum.nextElement();


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the Zip Entry is null, we are probably done,         */
            /* otherwise we must process the file                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ze != null ) {
               name = ze.getName();

               /* Replace the / characters with the true separator character */
               if ( File.separatorChar != '/' )
                  name = name.replace('/', File.separatorChar) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If this is a directory, make sure the directory exists  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ze.isDirectory() ) {

                  /* Chop off the trailing file separator if needed */
                  if ( name.endsWith(File.separator) )
                     name = name.substring(0, name.length() - 1);

                  dir = new File ( outDir+File.separator+name ) ;
                  if ( !dir.exists() )
                     dir.mkdirs() ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Append this directory to our complete path              */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  currDir = outDir + File.separator + name;
               } else {

                  filePath = outDir + File.separator + name;
                  file = new File(filePath);
                  dir  = new File(file.getParent()) ;

                  /* We must do this check again because the directory
                     that this file is in might not have been in the zip file
                     yet */
                  if ( !dir.exists() ) {
                     dir.mkdirs () ;
                  }


                  try {

                     InputStream      in  = zin.getInputStream(ze);
                     FileOutputStream out = new FileOutputStream(filePath);

                     length = 1 ;
                     while ( length >= 1 ) {
                        length = in.read(buffer);
                        if ( length > 0 )
                           out.write(buffer, 0, length);
                     }

                     out.close();
                     in.close() ;
                     file=null ;
                     dir=null ;
                  } 
                  
                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Pick apart the exception to show the user a nice msg    */                                           
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  catch ( Exception e ) {
                     String error = e.getMessage() ;

                     rc = false ;

                     /* If this is the end of zlib, ignore it */
                     if ( error.indexOf ( "end of ZLIB" ) == -1 ) {

                        /* Ok, if they ran out of disk space, we will get a write error. If
                           we get this exception, put up a pretty message to the user */
                        if ( error.indexOf ( "write error" ) != -1 ) {
                           e.printStackTrace() ;
                           cont=false ;
                        }

                        /* Otherwise, throw the exception */
                     } else {
                        e.printStackTrace() ;
                        cont=false ;
                     }

                  } /* catch  */
               } /* else if this was a file  */
            } /* if ze  */
            else
               cont = false ;

         } /* while  */

         zin.close();
      } catch ( Exception e ) {
         rc = false ;
         e.printStackTrace() ;
      }

      return( rc ) ;

   } /* unzip */



} /* Unzip */



class UnzipWindow extends Window {

   private Image image = null ;

   public UnzipWindow ( Frame frame ) {
      super ( frame ) ;
   }

   protected void setImage ( Image image ) {
      this.image = image ;
   }

   public void paint ( Graphics g ) {
      if ( image != null )
         g.drawImage ( image, 0, 0, this ) ;
   }

}


