package com.ibm.appsure.app.system ;
/*-----------------------------------------------------------*/
/*   GUISystem                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: GUISystem                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The GUI System has static methods useful for building
 * GUI Applications
 *
 * @version 1.0, 1/9/100 
 * @author  David Lentz
 */
import javax.swing.* ;
import javax.swing.border.* ;
import java.awt.* ;
import java.io.* ;
import java.util.* ;
import com.ibm.appsure.app.gui.* ;
import com.ibm.appsure.app.shared.util.* ;
import com.ibm.appsure.app.shared.gui.* ;
import com.ibm.appsure.app.shared.gui.form.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   GUISystem                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class GUISystem extends Object  {



    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Vars                                            */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static InfoBoxDlg          printBox     = null ;         
    private static FontUtil            fontUtil     = null ;




    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Public Static Vars                                      */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static final TitledBorder etchedBorder    = new TitledBorder("") ;
    public static final BevelBorder  loweredBorder   = new BevelBorder(BevelBorder.LOWERED) ;
    public static final BevelBorder  raisedBorder    = new BevelBorder(BevelBorder.RAISED) ;
    public static final LineBorder   blackBorder     = new LineBorder ( Color.black ) ;
    public static final EmptyBorder  emptyBorder     = new javax.swing.border.EmptyBorder(0,0,0,0) ;
    public static final EmptyBorder  emptyFillBorder = new javax.swing.border.EmptyBorder(2,2,2,2) ;
    public static final LineBorder   grayBorder      = new LineBorder ( Color.darkGray ) ;
    public static final Dimension    defaultNavSize  = new Dimension ( 200, 200 ) ;
    public static       boolean      os2             = false ;


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Colors                                                  */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static final Color       oddLinesColor = new Color ( 240, 240, 240 ) ;
    public static final Color       defaultGray   = new Color ( 214, 212 ,206 ) ;




    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create the one instance of
     *  GUI System
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public GUISystem () {

        setLookAndFeel() ;

        //os2 = PropertySystem.getBool ( PropertySystem.PROPERTY_OS2 ) ;

    }


    /*------------------------------------------------------------*/
    /* setLookAndFeel
     */
    /**
     *    This method will set the look and feel
     *  for the Quest Application to the look and feel passed in.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setLookAndFeel ( String lookFeel ) {

        try {
            JPanel jp = new JPanel() ; // Created only for passing to the ImageSystem.getImageIcon method

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the UI Look and Feel                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            UIManager.setLookAndFeel ( lookFeel ) ;                                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Initialize our properties                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            initializeProperties() ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Load our own Default Values into the Swing UI           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            //UIManager.getDefaults().put ("Tree.expandedIcon",    ImageSystem.getImageIcon ( jp, ImageSystem.OPEN_TREE ) ) ;
            //UIManager.getDefaults().put ("Tree.collapsedIcon",   ImageSystem.getImageIcon ( jp, ImageSystem.CLOSE_TREE ) ) ;
            //UIManager.getDefaults().put ("Form.foreground",      PropertySystem.get ( PropertySystem.PROPERTY_FORM_FOREGROUND ) ) ;
            //UIManager.getDefaults().put ("Form.valueForeground", PropertySystem.get ( PropertySystem.PROPERTY_FORM_VALUE_FOREGROUND ) ) ;
            //UIManager.getDefaults().put ("Form.titleForeground", PropertySystem.get ( PropertySystem.PROPERTY_FORM_TITLE_FOREGROUND ) ) ;
            //UIManager.getDefaults().put ("Form.font",            PropertySystem.get ( PropertySystem.PROPERTY_FORM_FONT ) ) ;
            //UIManager.getDefaults().put ("Form.valueFont",       PropertySystem.get ( PropertySystem.PROPERTY_FORM_VALUE_FONT ) ) ;
            //UIManager.getDefaults().put ("Form.titleFont",       PropertySystem.get ( PropertySystem.PROPERTY_FORM_TITLE_FONT ) ) ;

            if ( MainWindow.getInstance() != null )
                SwingUtilities.updateComponentTreeUI(MainWindow.getInstance());


        } catch ( Exception e ) {
            e.printStackTrace() ;
        }

    }




    /*------------------------------------------------------------*/
    /* setLookAndFeel
     */
    /**
     *    This is the default setLookAndFeel method will set the look and feel
     *  for the Quest Application from what is currently in the PropertySystem.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setLookAndFeel () {
        try {
           UIManager.setLookAndFeel ( "com.sun.java.swing.plaf.windows.WindowsLookAndFeel" ) ;                                                    
        } catch ( Exception e ) {
        }
    }


    /*------------------------------------------------------------*/
    /* setLookAndFeel
     */
    /**
     *    This method will set the look and feel
     *  for the an external window. Please call the default setLookAndFeel
     *  to set the application look and feel
     *  
     *
     *  @param JDialog the Dialog to set the LANDF for
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setLookAndFeel ( JDialog dialog ) {
        if ( dialog != null )
            SwingUtilities.updateComponentTreeUI(dialog);

    }


    /*------------------------------------------------------------*/
    /* setLookAndFeel
     */
    /**
     *    This method will set the look and feel
     *  for the an external window. Please call the default setLookAndFeel
     *  to set the application look and feel
     *  
     *
     *  @param JDialog the Dialog to set the LANDF for
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setLookAndFeel ( JFrame frame ) {
        String lookFeel = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel" ;

        if ( frame != null )
            SwingUtilities.updateComponentTreeUI(frame);

    }


    /*------------------------------------------------------------*/
    /* getRowHeight
     */
    /**
     *    This method will return the average
     *  height of a row of Components
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static int getRowHeight () {
        return( 18 ) ;
    }


    /*-----------------------------------------------------------*/
    /*   PrintBox()                                              */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will put up a Print Box                  */
    /*                                                           */
    /*   Parameters:                                             */
    /*                                                           */
    /*                                                           */
    /*   Returns:                                                */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static boolean printBox ( String title, String text ) {
        return( printBox (  MainWindow.getInstance(), title, text, -1 ) ) ;
    }
    public static boolean printBox ( Frame parent, String title, String text ) {
        return( printBox ( parent, title, text, -1 ) ) ;
    }


    /*------------------------------------------------------------*/
    /* printBox
     */
    /**
     *    Puts up a cached message box
     *  informing the user of something
     *  
     *
     *  @param parent The Frame or Dialog parent
     *  @param title The title of the window
     *  @param text The text of the window
     *  @param type InfoBoxPanel.ICON_ERROR, ICON_INFO, ICON_QUESTION
     *
     *  @return TRUE - User hit 'Ok' or 'Yes', FALSE = User hit 'Cancel' or 'No'
     *//*---------------------------------------------------------*/
    public static boolean printBox ( Component parent, String title, String text, int type ) {
        boolean     ret        = false ;
        String      retStr     = "" ;
        Frame       tempParent = null ;                           


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create only one instance of PrintBox                    */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( printBox == null ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   We must have a valid parent, create one if we do not    */
            /* have one                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( parent == null ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Always take the main Window's instance first            */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( MainWindow.getInstance() != null )
                    parent = MainWindow.getInstance() ;
                else {
                    tempParent = new Frame () ;
                    parent     = tempParent ;
                }
            } /* if parent  */

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the instance                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( parent instanceof JFrame )
                printBox = new InfoBoxDlg ( (JFrame)parent, "", "" ) ;
            else if ( parent instanceof Dialog )
                printBox = new InfoBoxDlg ( (Dialog)parent, "", "" ) ;
            else
                printBox = new InfoBoxDlg ( tempParent, "", "" ) ;
        }

        try {
            if ( ( title != null ) && ( text != null ) ) {
                ImageIcon   icon      = null ;
                String      tempTitle = title.toUpperCase() ;


                /* First, lets see what icontype to give the printbox */
                if ( type < 0 ) {
                    if ( ( tempTitle.indexOf ( "ERROR" )   >= 0 ) ||
                         ( tempTitle.indexOf ( "BAD" )     >= 0 ) ||
                         ( tempTitle.indexOf ( "SQL" )     >= 0 ) ||
                         ( tempTitle.indexOf ( "MISSING" ) >= 0 ) ) {
                        type = InfoBoxPanel.ICON_ERROR ;
                        //IMAGE icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.ERROR ) ;
                    } /* if this is an error box */
                    else
                        if ( ( tempTitle.indexOf ( "?" )        >= 0 ) ||
                             ( tempTitle.indexOf ( "UESTIO" )   >= 0 ) ||
                             ( tempTitle.indexOf ( "WHO" )      >= 0 ) ||
                             ( tempTitle.indexOf ( "WHAT" )     >= 0 ) ||
                             ( tempTitle.indexOf ( "WHEN" )     >= 0 ) ||
                             ( tempTitle.indexOf ( "WHERE" )    >= 0 ) ||
                             ( tempTitle.indexOf ( "WHY" )      >= 0 ) ) {
                        type = InfoBoxPanel.ICON_QUESTION ;
                        //IMAGE icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.QUESTION ) ;
                    } /* else, this is a question box */
                    else {
                        type = InfoBoxPanel.ICON_INFO ;
                        //IMAGE icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.INFO ) ;
                    } /* else, this is an info box */

                } else {
                    switch ( type ) {
                        case 2 :
                            //IMAGE icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.QUESTION ) ;
                            break ;

                        case 1 :
                            //IMAGE icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.ERROR ) ;
                            break ;

                        default:
                            //IMAGE icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.INFO ) ;
                            break ;
                    }
                }

                printBox.setDescription ( text ) ;
                printBox.setTitle ( title ) ;
                printBox.setIconType ( type ) ;

                if ( icon != null )
                    printBox.setImageIcon ( icon ) ;

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   If we had to create a temporary Frame, just center      */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( tempParent != null )
                    WinUtil.centerWindow ( printBox );

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Otherwise, center in the parent                         */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else
                    WinUtil.centerChildInParent ( printBox, parent ) ;

                if ( ( printBox != null ) && ( printBox.getContentPane() != null ) ) {
                    GUISystem.setPropertiesOnPanel ( printBox.getContentPane() ) ;

                    ret = printBox.getResult(false) ;
                }

            } /* if */
        } catch ( Exception e ) {
           e.printStackTrace() ;
        }

        if ( tempParent != null )
            tempParent.dispose() ;

        /* for now, destroy the print box and recreate it next time  */
        printBox.dispose() ;
        printBox = null ;

        return( ret ) ;
    } /* PrintBox */





    /*------------------------------------------------------------*/
    /* setPreferredButton
     */
    /**
     *    This method will take a button and
     *  looks at it's size. If it is normally larger
     *  than 70x25, it will leave the button alone,
     *  otherwise the button's preferred size will be set.
     *
     *  @param button The button to resize
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setPreferredButton ( JComponent button ) {
        Dimension size = button.getPreferredSize() ;

        if ( size.width < 70 )
            size.width = 70 ;
        if ( size.height < 25 )
            size.height = 25 ;

        button.setMinimumSize ( size ) ;
        button.setSize ( size ) ;
    }





    /*------------------------------------------------------------*/
    /* getParentDefWin
     */
    /**
     *    This method will return the parent
     *  AppDefaultWin of a component passed
     *  in
     *
     *  @param comp
     *  @param None
     *  @param None
     *
     *  @return AppDefaultWin instance, NULL if none
     *//*---------------------------------------------------------*/
    public static JFrame getParentDefWin ( Component c ) {
        JFrame ret = null ;

        while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
            if ( c instanceof JFrame )
                ret = (JFrame)c ;
        }

        if ( ret == null )
            ret = MainWindow.getInstance() ;


        return( ret ) ;
    }



    /*------------------------------------------------------------*/
    /* getParentFrame
     */
    /**
     *    This method will return the parent
     *  Frame of a component passed
     *  in
     *
     *  @param comp
     *  @param None
     *  @param None
     *
     *  @return AppDefaultWin instance, NULL if none
     *//*---------------------------------------------------------*/
    public static Frame getParentFrame ( Component c ) {
        Frame ret = null ;

        while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
            if ( c instanceof Frame )
                ret = (Frame)c ;
        }

        if ( ret == null ) {
            ret = MainWindow.getInstance() ;
        }


        return( ret ) ;
    }






    /*------------------------------------------------------------*/
    /* setPropertiesOnPanel
     */
    /**
     *    This method will set all GUI Properties
     *  for all Components on the panel passed in 
     *  recursivly. This includes Font, Color etc..
     *
     *  @param panel The Panel to traverse through
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setPropertiesOnPanel ( Container panel ) {

        if ( !os2 ) {
            Component   c         = null ;
            Component[] compArray = panel.getComponents() ;
            int         i         = 0 ;

            //panel.setBackground ( defaultGray ) ;

            while ( i < compArray.length ) {

                c = compArray[i] ;    

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   AbstractButton - Set Font                               */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( c instanceof AbstractButton ) {
                        if ( ( c instanceof DButton ) || ( c instanceof JButton ) )
                            setPreferredButton ( (JComponent)c ) ;
                    }

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   If the component is another container, call us again    */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    else if ( c instanceof Container ) {
                        if ( c instanceof JScrollPane ) {
                            ((JScrollPane)c).setBorder ( loweredBorder ) ;
                        } else if ( c instanceof ButtonPanel ) {
                            //IMAGE ((ButtonPanel)c).setArrowImage ( ImageSystem.getImageIcon ( MainWindow.getInstance() ,ImageSystem.BUTTONPANEL_LEFT ) ,
                            //IMAGE                                  ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.BUTTONPANEL_RIGHT ) ) ;
                        }

                        setPropertiesOnPanel ( (Container)c ) ;
                    }
                i++ ;
            } /* while  */
        } /* if !os2 */
    }




    /*------------------------------------------------------------*/
    /* getFontUtil
     */
    /**
     *    This method will return the FontUtil
     *  object for the application
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static FontUtil getFontUtil() {
        if ( fontUtil == null )
            fontUtil = new FontUtil() ;

        return( fontUtil ) ;
    }


    /*------------------------------------------------------------*/
    /* getFileOpen
     */
    /**
     *    This method will return a file object that is
     *  received from a file open dialog.
     *  
     *
     *  @param title The title of the window
     *  @param currentDirectory The current directory to open in
     *  @param loadDlg TRUE = show the Load Dialog, FALSE = show the Save Dialog
     *
     *  @return File[] An array of files. The first file is the actual file, the second is a directory file
     *//*---------------------------------------------------------*/
    public static File[] getFileOpen ( Frame parent, String title, String currentDirectory, String initFileName, boolean loadDlg ) {
        File[] file = null ;

        /* Check to see what file open dialog we use. For some systems, the
           swing file open may not work correctly. */
        try {
                FileDialog fileDialog = new FileDialog ( parent, title, loadDlg ? FileDialog.LOAD : FileDialog.SAVE ) ;
                String     fileName   = "" ;
                String     directory  = "" ;

                if ( currentDirectory != null )
                    fileDialog.setDirectory ( currentDirectory ) ;

                if ( initFileName != null )
                    fileDialog.setFile ( initFileName ) ;

                fileDialog.setVisible ( true ) ;

                fileName = fileDialog.getFile() ;
                directory = fileDialog.getDirectory() ;

                if ( ( fileName != null ) && ( fileName.length() > 0 ) ) {
                    file = new File[2] ;
                    file[0] = new File ( directory+File.separator+fileName ) ;
                    file[1] = new File ( directory ) ;
                }
        } catch ( Exception e ) {
            e.printStackTrace() ;
        }

        return( file ) ;
    }


    /*------------------------------------------------------------*/
    /* getFileOpen
     */
    /**
     *    This method will return a file object that is
     *  received from a file open dialog
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static File[] getFileOpen ( Frame parent, String title, String currentDirectory, boolean loadDlg ) {
        return( getFileOpen ( parent, title, currentDirectory, null, loadDlg ) ) ;
    }


    /*------------------------------------------------------------*/
    /* getFileOpen
     */
    /**
     *    This method will return a file object that is
     *  received from a file open dialog.
     *  
     *
     *  @param title The title of the window
     *  @param currentDirectory The current directory to open in
     *
     *  @return File[] An array of files. The first file is the actual file, the second is a directory file
     *//*---------------------------------------------------------*/
    public static File[] getFileOpen ( Frame parent, String title, String currentDirectory ) {
        return( getFileOpen ( parent, title, currentDirectory, null, true ) ) ;
    }





    /*------------------------------------------------------------*/
    /* setComponentOpaque
     */
    /**
     *    This method will set the component passed
     *  and any children recursivly to the opaque false
     *  passed
     *
     *  @param component The component to change
     *  @param opaque FALSE/TRUE
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setComponentOpaque ( JComponent component, boolean opaque ) {

        component.setOpaque ( opaque ) ;

        if ( component instanceof Container ) {
            Component   c         = null ;
            Component[] compArray = component.getComponents() ;


            if ( compArray != null )
                for ( int i = 0 ; i < compArray.length; i++ ) {
                    if ( compArray[i] instanceof JComponent )
                        setComponentOpaque ( (JComponent) compArray[i], opaque ) ;
                }

        }
    }





    /*------------------------------------------------------------*/
    /* 
     */
    /**
     *    This little method simply prints out the names
     *  of each Swing UI default
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void printSwingUIDefaults () {
        UIDefaults defaults = UIManager.getDefaults() ;
        Enumeration e = defaults.keys() ;

        while ( e.hasMoreElements() ) {
            //LogSystem.log ( 1, e.nextElement().toString() ) ;
        }

    }


    /**
      *  This method will set all properties in the PropertySystem if 
      * the PropertySystem does not have the value set yet
      **/
    private static void initializeProperties () {
    }


    /**
      *  This method will set the Font and Color properties on every control in the
      * FormComponent passed in
      **/
    public static void setPropertiesOnFormComponent ( Container component ) {
        Component   c         = null ;
        Component[] compArray = component.getComponents() ;
        int         i         = 0 ;

        while ( i < compArray.length ) {

            c = compArray[i] ;       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Value Labels                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( c instanceof ValueLabel ) {
                ValueLabel valueLabel = (ValueLabel) c ;

                valueLabel.setOpaque ( false ) ;

                ((JComponent)valueLabel.getComponent()).setOpaque ( false ) ;
                valueLabel.getJLabel().setOpaque ( false ) ;

                valueLabel.setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Form.font" ) ) ;
                valueLabel.setForeground ( (Color)javax.swing.UIManager.getDefaults().get ( "Form.foreground" ) ) ;
                valueLabel.getComponent().setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Form.valueFont" ) ) ;
                valueLabel.getComponent().setForeground ( (Color)javax.swing.UIManager.getDefaults().get ( "Form.valueForeground" ) ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Radio Buttons and Check Boxes                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( ( c instanceof JRadioButton ) ||
                      ( c instanceof JCheckBox ) ) {
                c.setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Form.font" ) ) ;
                c.setForeground ( (Color)javax.swing.UIManager.getDefaults().get ( "Form.foreground" ) ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   JTextArea and JTextField                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( ( c instanceof JTextArea ) || ( c instanceof JTextField ) ) {
                c.setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Form.valueFont" ) ) ;
                c.setForeground ( (Color)javax.swing.UIManager.getDefaults().get ( "Form.valueForeground" ) ) ;
            }



            else if ( ( c instanceof JPanel ) ||
                      ( c instanceof JScrollPane ) ||
                      ( c instanceof Panel ) ) {
                System.out.println ( "Recurse!" ) ;
                setPropertiesOnFormComponent ( (Container)c ) ;
            }

            i++ ;
        }

        if ( component instanceof FormComponent ) {
            ((FormComponent)component).setForeground ( (Color)UIManager.getDefaults().get ( "Form.titleForeground" ) ) ;
            ((FormComponent)component).setFont ( (Font)UIManager.getDefaults().get ( "Form.titleFont" ) ) ;
        }

    }
} /* GUISystem */



