package com.ibm.appsure.common ;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      Application
/*   Creation Date: MARCH 26, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.appsure.* ;
/**
 * The Application class holds all information about a specific 
 * Application in the App database schema. This class also holds 
 * methods of reading and writing Application objects to the Database.
 *
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* Application
/*
/*----------------------------------------------------*/
public class Application extends DatabaseObject implements com.ibm.appsure.app.shared.gui.MultiListRow {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This is a simple human readable name for this application. The max
    * length for this field is 254 characters.
    */
   private String name = null ;

   /**
    * This AppCode is a simple Application specific code. This code MUST
    * be unique. 
    */
   private String appCode  = null ;

   /**
    * This is a simple 1,024 character field that should tell what this
    * application does etc.. HTML should be used to provide links to more
    * in depth information
    */
   private String about = null ;

   /**
    * This is the location that the program is run from. This may be a
    * directory or a server name or whatever you need to indicate a
    * location with. The max length for this field is 100 characters.
    */
   private String runLocation = null ;

   /**
     *  This property defines if the application can run more than
     * one instance
     *
     **/
   private boolean multipleInst = false ;

   /**
     *  THis is the short description of the application
     **/
   private String shortDescript = null ;

   /**
     * This is the average amount of time in minutes that this application runs
     **/
   private int averageRunMin = 0 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     * This constructo creates an Application object used when creating
     * a new application in the database
     **/
   public Application () {
   }


   /**
     * This constructor creates an Application object used when reading
     * from the database<br><br>
     *
     * <b>Please Note</b>: This method will not actually read the object from the database. To read it, use the readFromDatabase method
     *
     * @param appCode The Application Code to read from the database
     **/
   public Application ( String appCode ) {
      this.appCode = appCode ;
   }

   /**
     *  This method returns the name of this application            
     *
     * @returns The name
     **/
   public String getName() {
      return(name);
   }

   /**
     *  This method sets the name for this application. The name
     * can be anything.
     *
     * @param The name of the application
     **/
   public void setName(String name) {
      this.name = name;
   }


   /**
     *  This method will set the short description
     *
     * @param String shortDescription
     **/
   public void setShortDescript ( String shortDescript ) {
      this.shortDescript= shortDescript ;
   }

   public String getShortDescript () {
      return( shortDescript ) ;
   }
   /**
     *  This method will return the application code unique to this
     * application
     *
     * @returns The application code
     **/
   public String getAppCode() {
      return(appCode);
   }

   /**
     *  This method will set the application code unique to this 
     * application.
     *
     * @param appCode The Application Code
     **/
   public void setAppCode(String appCode) {
      this.appCode = appCode;
   }

   /**
     *  This method will return simple text describing this
     * application.
     **/
   public String getAbout() {
      return(about);
   }

   /**
     *  This method will set the simple descriptive text
     * describing this application
     *
     * @param aboutText
     **/
   public void setAbout(String about) {
      this.about = about;
   }

   /**
     *  This method will return a string describing where
     * this application runs.
     *
     * @returns the Run Location
     **/
   public String getRunLocation() {
      return(runLocation);
   }

   /**
     *  This method will set the run location.
     *
     * @param runLocation A simple text describing where this application runs
     **/
   public void setRunLocation(String runLocation) {
      this.runLocation = runLocation;
   }

   /**
     *  This method will read this record in from the database. If the appCode
     * property is not set this method will do nothing.
     *
     * @return 0 if read from the database, the SQLCode if the record failed reading
     **/
   public boolean readFromDatabase ( Connection conn ) throws AppSureException {
      boolean rc = false ;

      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT NAME, ABOUT, RUNLOCATION, MULTIPLEINST, shortdescript, averageRunTimeMin "+
                                              "FROM APPSURE.APPLICATION "+
                                              "WHERE APPCODE = '"+getAppCode()+"'" ) ;

         if ( rs.next() ) {
            setName          ( rs.getString (1).trim() ) ;
            setAbout         ( rs.getString (2) ) ;
            setRunLocation   ( rs.getString (3).trim() ) ;
            setMultipleInst  ( rs.getString (4).equals ( "Y" ) ) ;
            setShortDescript ( rs.getString (5) ) ;
            setAverageRunMin ( rs.getInt    (6) ) ;
            updateRecStatus ( REC_NOCHANGE ) ;
            rc = true ;
         }
         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         throw ( new AppSureException ( e ) ) ;
      }

      return( rc ) ;
   }


   /**
     *  This method will save this information to the database. 
     *
     * @return 0 if the record was saved, the SQLCode if the record was not saved
     **/
   public int writeToDatabase ( Connection conn ) throws AppSureException {
      int rc = 0 ;

      try {
         Statement stmt = conn.createStatement() ;

         switch ( getRecStatus() ) {
            case REC_INSERT :
               stmt.executeUpdate ( "INSERT INTO APPSURE.APPLICATION "+
                                    "values "+
                                    "( '"+appCode+"', "+
                                    "  '"+name+"', "+
                                    "  '"+about+"', "+
                                    "  '"+runLocation+"',"+
                                    "  '"+(multipleInst ? "Y" : "N")+"', "+
                                    "  '"+shortDescript+"', "+
                                    "   "+averageRunMin+") ") ;
               break ;

            case REC_UPDATE :
               stmt.executeUpdate ( "UPDATE APPSURE.APPLICATION "+
                                    "SET NAME = '"+name+"', "+
                                    "    ABOUT = '"+about+"', "+
                                    "    RUNLOCATION = '"+runLocation+"',"+
                                    "    MULTIPLEINST = '"+(multipleInst ? "Y" : "N")+"', "+
                                    "    SHORTDESCRIPT = '"+shortDescript+"', "+
                                    "    AVERAGERUNTIMEMIN = "+averageRunMin+" WHERE APPCODE = '"+appCode+"' ") ;
               break ;

            case REC_DELETE :
               stmt.executeUpdate ( "DELETE FROM APPSURE.FAILURE "+
                                    "WHERE LOGINDEX IN (SELECT LOGINDEX FROM APPSURE.LOGS WHERE APPCODE = '"+appCode+"')" ) ;
               stmt.executeUpdate ( "DELETE FROM APPSURE.LOGS WHERE APPCODE = '"+appCode+"'" ) ;
               stmt.executeUpdate ( "DELETE FROM APPSURE.APPPROCESS WHERE APPCODE = '"+appCode+"'" ) ;
               stmt.executeUpdate ( "DELETE FROM APPSURE.RUNTIME WHERE APPCODE = '"+appCode+"'" ) ;
               stmt.executeUpdate ( "DELETE FROM APPSURE.CONTACT WHERE APPCODE = '"+appCode+"'" ) ;
               stmt.executeUpdate ( "DELETE FROM APPSURE.APPLICATION WHERE APPCODE = '"+appCode+"'" ) ;
               break ;
         }

         stmt.close() ;
         rc=0 ;

      } catch ( SQLException e ) {
         throw new AppSureException (e ) ;
      }

      return( rc ) ;
   }



   /**
     * This method will turn the 'Multiple Instance' piece on and off
     *
     * @param multipleInst TRUE or FALSE
     **/
   public void setMultipleInst ( boolean multipleInst ) {
      this.multipleInst = multipleInst ;
   }

   /**
     * This method will return the multiplace instance property
     *
     * @returns boolean
     **/
   public boolean getMultipleInst ( ) {
      return( multipleInst ) ;
   }


   /**
     *  This method will simply return the values needed to display in a MultiListRot
     **/
   public Object getColumnData ( int column ) {
      switch ( column ) {
         case 0 :
            return( appCode ) ;

         case 1 :
            return( name ) ;

         case 2 :
            return( shortDescript ) ;

         default:
            return( "" ) ;
      }
   }


   /**
     * This method will return the number of minutes this application runs
     *
     * @return int
     **/
   public int getAverageRunMin () {
      return( averageRunMin ) ;
   }

   /**
     * This method will set the average run minutes
     **/
   public void setAverageRunMin ( int averageRunMin ) {
      this.averageRunMin = averageRunMin ;
      updateRecStatus ( REC_UPDATE ) ;
   }
}
/* Class1 */
