package com.ibm.appsure.common;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      Contact
/*   Creation Date: MAY 7, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
/**
 * A 'Contact' is the joining of a Person to an Application. This
 * contact has a type of contact which could be 'Project Manager',
 * 'Development Lead' etc.. The information in this class is stored
 * in the APP.CONTACT table.
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* Contact
/*
/*----------------------------------------------------*/
public class Contact extends DatabaseObject {

   public static final int CONTACT_PRIMARY     = 1 ;
   public static final int CONTACT_SECONDARY   = 2 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This is the application unique key for an application
    * in the AppSure system. This key must be defined in the
    * APP.APPLICATION table. 
    */
   private String appCode = null ;

   /**
     *  This is the index of the person that this contact represents
     **/
   private int personIndex = 0 ;

   /**
     *  This is the person object that this contact represents
     **/
   private Person person = null ;

   /**
    * This is what type of Contact this record represents. This type
    * must be defined in the APP.CONTACTTYPE table. An example
    * of a Contact Type would be a 'Project Manager', 'Development Lead',
    * 'Architect' etc.. 
    */
   private int contactType = 0 ;







   /**
     *  This constructor creates a new Contact object that may be used
     * to save to the database
     **/
   public Contact () {
      updateRecStatus ( REC_INSERT ) ;
   }

   /**
     *  This constructor creates a new Contact object that is fully populated
     *
     * @param appCode The appCode defined in the APP.APPLICATION table
     * @param personIndex The Primary key for this person
     * @param contactType The type of contact
     *
     **/
   public Contact ( String appCode, int personIndex, int contactType ) {
      setAppCode     ( appCode ) ;
      setPersonIndex ( personIndex ) ;
      setContactType ( contactType ) ;
      updateRecStatus ( REC_NOCHANGE ) ;
   }


   /**
     *  This method will return the appCode that is unique to the
     * application this contact is for
     **/
   public String getAppCode() {
      return appCode;
   }

   /**
     *  This method will set the appCode that this contact is
     * associated to
     *
     * @param appCode The appCode defined in the APP.APPLICATION table
     **/
   public void setAppCode(String appCode) {
      this.appCode = appCode;
   }

   /**
     *  This method will return the primary key for the person that
     * this contact represents. This primary key is defined in the APP.PERSON table
     *
     * @returns int
     **/
   public int getPersonIndex() {
      return personIndex;
   }

   /**
     *  This method will set the primary key pointing to the person that
     * this contact represents. This primary key is defined in the APP.PERSON table
     *
     * @param personIndex The Primary key for this person
     **/
   public void setPersonIndex(int personIndex) {
      this.personIndex = personIndex;
   }

   /**
     *  This method will return what type of contact this is. The integer
     * returned may be used to read from the APP.CONTACTTYPE table
     *
     * @return int
     **/
   public int getContactType() {
      return contactType;
   }

   /**
     *  This method will set what type of contact this is. The contactType
     * passed must be present in the APP.CONTACTTYPE table.
     *
     * @param contactType The type of contact
     **/
   public void setContactType(int contactType) {
      this.contactType = contactType;
   }

   /**
     *  This method will read this record in from the database. If the runIndex
     * property is not set this method will do nothing.
     *
     * @return 0 if read from the database, the SQLCode if the record failed reading
     **/
   public int readFromDatabase () {
      return( 0 ) ;
   }


   /**
     *  This method will save this information to the database. 
     *
     * @return 0 if the record was saved, the SQLCode if the record was not saved
     **/
   public int saveToDatabase () {
      return( 0 ) ;
   }


   /**
     *  This method will return the person that this Contact represents
     *
     * @return Person
     **/
   public Person getPerson() {
      return( person ) ;
   }


   /**
     *  This method will set the person that this Contact represents
     **/
   public void setPerson ( Person person ) {
      this.person = person ;
      personIndex = person.getPersonIndex() ;
      updateRecStatus ( REC_UPDATE ) ;
   }
} /* Contact */
