package com.ibm.appsure.common;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      ContactType
/*   Creation Date: MAY 7, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
/**
 * This object represents a single type of contact. The information
 * in this class is stored in the APP.TYPECONTACT database table. A
 * Type of Contact could be 'Project Manager', 'Manager', 'Developer',
 * 'Architect' etc..
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* ContactType
/*
/*----------------------------------------------------*/
public class ContactType extends DatabaseObject {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
    * This is the primary key for this type of contact 
    */
   private int contactType = 0 ;

   /**
    * This is the human readable description of this type
    * of contact 
    */
   private String description = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/


   /**
     *  This constructor creates a new ContactType records suitable
     * for creation in the AppSure system
     **/
   public ContactType () {
      updateRecStatus ( REC_INSERT ) ;
   }

   /**
     *  This constructor fully populates a ContactType object
     *
     * @param contactType This is the primary key for this contact type and must
     *                    exist in the APP.CONTACTTYPE table
     * @param description The new user readable description of this contact
     **/
   public ContactType ( int contactType, String description ) {
      this.contactType = contactType ;
      setDescription ( description ) ;
      updateRecStatus ( REC_NOCHANGE ) ;
   }

   /**
     *  This method will return the primary key for this contact
     * type. This primary key will be in the APP.CONTACTTYPE table.
     *
     * @return int
     **/
   public int getContactType(){ 
      return contactType;
   }


   /**
     *  This method will return the description of this contact
     * type. This description is suitable for user reading.
     *
     * @return String
     **/
   public String getDescription(){ 
      return description;
   }

   /**
     *  This method will set the description of this contact
     * type.
     *
     * @param description The new user readable description of this contact
     **/
   public void setDescription(String description){ 
      this.description = description;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will read this record in from the database. If the runIndex
     * property is not set this method will do nothing.
     *
     * @return 0 if read from the database, the SQLCode if the record failed reading
     **/
   public int readFromDatabase () {
      return ( 0 ) ;
   }


   /**
     *  This method will save this information to the database. 
     *
     * @return 0 if the record was saved, the SQLCode if the record was not saved
     **/
   public int saveToDatabase () {
      return ( 0 ) ;
   }


} /* ContactType */
