package com.ibm.appsure.common ;
/*-----------------------------------------------------------*/
/*   DatabaseObject                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DatabaseObject                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.FileNotFoundException ;
/**
 * This is the base class for all objects in the AppSure System 
 * that wish to write, read, delete etc.. from different data sources.<br><br>
 * This class also holds stub methods for reading and writing this object to
 * different data sources.
 *
 * @version 1.0, 3/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DatabaseObject                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class DatabaseObject extends Object implements java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int REC_NOCHANGE      = 0 ;
   public static final int REC_UPDATE        = 1 ;
   public static final int REC_INSERT        = 2 ;
   public static final int REC_DELETE        = 3 ;
   public static final int REC_NOINSERT      = 4 ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int recStatus = REC_NOCHANGE ;                                                          




   /*------------------------------------------------------------*/
   /* valid
    */
   /**
    *    This method will return FALSE if this record's
    *  current status is either REC_DELETE or
    *  REC_NOINSERT.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean valid () {
      return( ( recStatus != REC_DELETE) && ( recStatus != REC_NOINSERT ) ) ;
   }




   /*------------------------------------------------------------*/
   /* modified
    */
   /**
    *    This method will return TRUE if this record's
    *  current status is either REC_INSERT, REC_UPDATE or
    *  REC_DELETE.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean modified () {
      return( ( recStatus == REC_INSERT ) || ( recStatus == REC_UPDATE ) || 
              ( recStatus == REC_DELETE )  ) ;
   }




   /*------------------------------------------------------------*/
   /* updateRecStatus
    */
   /**
    *    This method will update this record's current
    *  status to the newStatus passed in. It is not guarenteed that
    *  the current status will change to the status you pass though,
    *  take for example if this record's current status is REC_INSERT,
    *  and you call this method, trying to change the status to REC_UPDATE.
    *  Since this record has not been writed to it's data source, the record
    *  will retain it's REC_INSERT status.
    *  
    *
    *  @param newStatus The new status to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void updateRecStatus ( int newStatus) {

      if ( (newStatus == REC_NOCHANGE) || (newStatus == REC_NOINSERT) )
         recStatus = newStatus ;
      else {
         switch ( recStatus ) {
            case REC_NOINSERT:
               if ( newStatus == REC_INSERT )
                  recStatus = REC_INSERT;
               else if ( newStatus == REC_UPDATE )
                  recStatus = REC_INSERT;
               else if ( newStatus == REC_DELETE )
                  recStatus = REC_NOINSERT;
               break;
            case REC_INSERT:
               if ( newStatus == REC_UPDATE )
                  recStatus = REC_INSERT;
               else if ( newStatus == REC_DELETE )
                  recStatus = REC_NOINSERT;
               break;
            case REC_UPDATE:
               if ( newStatus == REC_INSERT )
                  recStatus = REC_UPDATE;
               else if ( newStatus == REC_DELETE )
                  recStatus = REC_DELETE;
               break;
            case REC_DELETE:
               if ( newStatus == REC_INSERT )
                  recStatus = REC_UPDATE;
               else if ( newStatus == REC_UPDATE )
                  recStatus = REC_UPDATE;
               break;

            default :
               recStatus = newStatus ;
         }  /* switch CurrStatus */
      }  /* else */
   } /* updateRecordStatus */







   /*------------------------------------------------------------*/
   /* getRecStatus
    */
   /**
    *    This method will return the status of this
    *  record. Valid return values are as follows:<br>
    *
    *  REC_INSERT<br>
    *  REC_UPDATE<br>
    *  REC_DELETE<br>
    *  REC_NOINSERT<br>
    *  REC_NOCHANGE<br>  
    *
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getRecStatus () {
      return( recStatus ) ;
   }





   /*------------------------------------------------------------*/
   /* writeToXML
    */
   /**
    *    This method should write this object in XML
    *  format to the filename specified
    *  
    *
    *  @param filename The filename to create and write in
    *  @param None
    *  @param None
    *
    *  @return int. 0 should be good return
    *//*---------------------------------------------------------*/
   public int writeToXML ( String filename ) throws FileNotFoundException {
      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* writeToDatabase
    */
   /**
    *    This method should write this object to a
    *  database.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int 0 Should mean default
    *//*---------------------------------------------------------*/
   public int writeToDatabase () {
      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* writeToFile
    */
   /**
    *    This method should simply write this object 
    *  to the filename specified.
    *  
    *
    *  @param filename The filename to write the object to
    *  @param None
    *  @param None
    *
    *  @return int 0 Should mean default
    *//*---------------------------------------------------------*/
   public int writeToFile ( String filename ) throws FileNotFoundException {
      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* readFromDatabase
    */
   /**
    *    This method should read this object
    *  from a database.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int 0 Should mean default
    *//*---------------------------------------------------------*/
   public int readFromDatabase () {
      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* readFromXML
    */
   /**
    *    This method should populate this object's
    *  value from an XML file.
    *  
    *
    *  @param filename The XML file to read in. the .XML extension should be passed as well
    *  @param None
    *  @param None
    *
    *  @return int 0 Should be default
    *//*---------------------------------------------------------*/
   public int readFromXML ( String filename ) throws FileNotFoundException {
      return( 0 ) ;
   }






   /*------------------------------------------------------------*/
   /* readFromFile
    */
   /**
    *    This method should read this object from a
    *  file on the user's local file system.
    *  
    *
    *  @param filename The filename to load
    *  @param None
    *  @param None
    *
    *  @return int 0 Should mean default
    *//*---------------------------------------------------------*/
   public int readFromFile ( String filename ) throws FileNotFoundException {
      return( 0 ) ;
   }






   /*------------------------------------------------------------*/
   /* toXML
    */
   /**
    *    This method should return an XML string
    *  that represents this object.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String toXML () {
      return( null ) ;
   }






   /*------------------------------------------------------------*/
   /* toSQL
    */
   /**
    *    This method should return the SQL needed
    *  to save this object to the database entirely.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String toSQL () {
      return( null ) ;
   }
} /* PersitentRec  */
