package com.ibm.appsure.common ;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      Log
/*   Creation Date: MARCH 26, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.io.* ;
import java.util.* ;
import java.text.* ;
import java.sql.* ;
import com.ibm.appsure.* ;
import com.ibm.appsure.common.* ;
/**
 * The Log class handles logging of any messages to a single
 * log file.
 *
 *
 * @author  David Lentz
 * @version 1
 */

public class Log extends Object {

   /**
     * The log filename
     **/
   private static String logFilename = "appsure.out" ;

   /**
     * This is the max number of bytes the log file is allowed to grow
     **/
   private static long logFileMaxSize = 1024 ;


   /**
     * Default Constructor
     *
     * @param logFile The name of the log file
     **/
   public Log ( String logFilename ) {
      this.logFilename = logFilename ;
      log ( "\n\n" ) ;
      log ( "Application Started. "+getTimestamp() ) ;
   }


   /**
     * This method will set the max size for the log file
     *
     * @param logFileMaxSize The max size in bytes
     **/
   public void setLogFileMaxSize ( int logFileMaxSize ) {
      this.logFileMaxSize = logFileMaxSize ;
   }


   /**
     * This method will log a message to the log file
     **/
   public static void log ( String log ) {
      appendToFile ( getTimestamp()+"   "+log ) ;
   }


   /**
     *  This method will log an exception to the log file
     **/
   public static void log ( Exception ex ) {
      String          logStr  = getTimestamp()+"   Exception Occurred\n-----------------------------------------------\n" ;
      StringWriter    sw      = new StringWriter () ;
      PrintWriter     pw      = new PrintWriter ( sw ) ;

      /* Get the stackTrace */
      ex.printStackTrace ( pw ) ;

      logStr = sw.toString() ;

      appendToFile ( logStr ) ;
   }


   /**
     *  This method will specially log an AppSureException
     **/
   public static void log ( AppSureException ex ) {
      String          logStr  = getTimestamp()+"   AppSureException Occurred\n------------------------------------------------------\n" ;
      StringWriter    sw      = new StringWriter () ;
      PrintWriter     pw      = new PrintWriter ( sw ) ;

      /* Get the stackTrace */
      ex.printStackTrace ( pw ) ;

      logStr = sw.toString() ;

      appendToFile ( logStr ) ;
   }


   /**
     *  This method will return the current timestamp
     **/
   private static String getTimestamp () {
      String           retStr    = null ;
      SimpleDateFormat date      = new SimpleDateFormat ( "yyyy-MM-dd-kk.mm.ss.000000" ) ;
      java.util.Date   todayDate = new java.util.Date() ;

      date.setTimeZone ( TimeZone.getDefault() ) ;
      retStr = date.format(todayDate) ;

      return( retStr ) ;
   }


   /**
     *  This method will simply append a string to the file
     **/
   private static void appendToFile ( String contents ) {                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, truncate the file if needed                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contents+="\n" ;
      checkAndTruncate ( contents.length() ) ;

      try {
         RandomAccessFile file      = new RandomAccessFile ( logFilename, "rw" ) ;

         file.seek( file.length() );
         file.write ( contents.getBytes() ) ;
         file.close() ;
      } catch ( Exception e ) {
         System.err.println ( "Cannot write to log file "+logFilename+"\n"+contents ) ;
      }
   }


   /**
     *  This method will check the log file size and truncate it if needed. Truncation
     * will occur when the file is larger than the logFileMaxSize property. If truncated,
     * the first few lines of the file will be cut.          
     *
     * @param addSize The number of bytes to add to the file's length 
     **/
   private static void checkAndTruncate ( int addSize ) {
      File file = new File ( logFilename ) ;

      if ( file.length() > (logFileMaxSize-addSize) ) {
         try {

            if ( addSize > file.length() )
               clearLog() ;
            else
               saveFile ( logFilename, readFile ( logFilename ).substring ( addSize ) ) ;
         } catch ( Exception e ) {
            System.err.println ( "Cannot truncate file" ) ;
            e.printStackTrace() ;
         }
      }
   }


   /*------------------------------------------------------------*/
   /* readFile
    */
   /**
    *    This method will read in a file and return the
    *  contents of the file in a string
    *  
    *
    *  @param fileName The filename to read
    *  @param None
    *  @param None
    *
    *  @return String 
    *//*---------------------------------------------------------*/
   public static String readFile ( String fileName ) throws FileNotFoundException, IOException  {
      String retStr = null ;
      File              file      = new File ( fileName ) ;
      FileInputStream   in        = new FileInputStream ( file ) ;
      int               maxChunk  = 1024 ;
      byte[]            fByte     = new byte[ maxChunk ] ;
      int               i         = 0 ;
      int               readCount = 0 ;

      retStr = "" ;
      while ( readCount >= 0 ) {
         readCount = in.read ( fByte ) ;
         if ( readCount>= 0 )
            retStr = retStr+new String ( fByte, 0, readCount ) ;
      }

      in.close() ;

      return( retStr ) ;
   } /* loadFile */







   /*------------------------------------------------------------*/
   /* saveFile
    */
   /**
    *    This method will save a string to a file.
    *  
    *  
    *
    *  @param filename The filename of the file
    *  @param string The String to save to the file
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean saveFile ( String filename, String string ) {
      boolean rc = false ;

      try {
         File             outFile = new File ( filename ) ;
         FileOutputStream out     = new FileOutputStream ( outFile ) ;

         out.write ( string.getBytes() ) ;
         out.close() ;
         outFile = null ;
         out     = null ;
         rc = true ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( rc ) ;
   }


   /**
     *  This method will clear the current log file
     **/
   public static void clearLog () {
      saveFile ( logFilename, "" ) ;
   }


   /**
     *  This method will simply skip a line in the log file
     **/
   public static void skipLine () {
      appendToFile ( "" ) ;
   }
}
