package com.ibm.appsure.common;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      Person
/*   Creation Date: MAY 7, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
/**
 * The Person class identifies a single person in the AppSure system.
 * The Person is usually associated to an Application and is then called
 * a Contact.
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* Person
/*
/*----------------------------------------------------*/
public class Person extends DatabaseObject {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Members                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
    * This is the person's first name 
    */
   private String firstName;

   /**
    * This is the person's last name 
    */
   private String lastName;

   /**
    * This is the person's eMail Address. This eMail address
    * must contain the user id, the 'at' sign and the domain. 
    */
   private String email;

   /**
    * This is the person's phone number 
    */
   private String phone;

   /**
    * This is the eMail address used to page this person 
    */
   private String pagerEMail;

   /**
    * This is the person's pager phone number 
    */
   private String pager;

   /**
    * This is the person's primary key in the AppSure database 
    */
   private int personIndex;





   /**
     *  This constructor creates a Person object suitable to
     * save as a new person in the database
     **/
   public Person () {
   }


   /**
     *  This constructor creates a Person object suitable to
     * read in from the database.
     *
     * @param personIndex The Primary Key used to read this person
     *                    in from the database
     **/
   public Person ( int personIndex ) {
      this.personIndex = personIndex ;
   }




   /**
     *  This method will return this person's first name
     * 
     *  @return String
     **/
   public String getFirstName() { 
      return firstName; 
   }

   /**
     *  This method will set this person's first name
     *
     * @param firstName This person's new first name
     **/
   public void setFirstName(String firstName){ 
      this.firstName = firstName;
   }

   /**
     *  This method will return this person's last name
     *
     * @return String
     **/
   public String getLastName() { 
      return lastName;
   }

   /**
     *  This method will set this person's last name
     *
     * @param lastName This person's new last name
     **/
   public void setLastName(String lastName){ 
      this.lastName = lastName;
   }

   /**
     *  This method will return this person's eMail
     * address.
     *
     * @returns String
     **/
   public String getEmail(){ 
      return email;
   }

   /**
     *  This method will set this person's eMail address.
     *
     * @param email This person's eMail address
     **/
   public void setEmail(String email){ 
      this.email = email;
   }

   /**
     *  This method will return this person's phone number
     *
     * @return String
     **/
   public String getPhone(){ 
      return phone;
   }

   /**
     *  This method will set this person's phone number
     *
     * @param phone This person's phone number
     **/
   public void setPhone(String phone){ 
      this.phone = phone;
   }

   /**
     *  This method will return this person's eMail address
     * to their pager.
     *
     * @return String
     **/
   public String getPagerEMail(){ 
      return pagerEMail;
   }

   /**
     *  This method will set this person's eMail address for
     * their pager
     *
     * @param pagerEMail
     **/
   public void setPagerEMail(String pagerEMail){ 
      this.pagerEMail = pagerEMail;
   }

   /**
     *  This method will return this person's pager number
     * 
     * @return String
     **/
   public String getPager(){ 
      return pager;
   }

   /**
     *  This method will set this person's pager number
     *
     * @param The pager number
     **/
   public void setPager(String pager){ 
      this.pager = pager;
   }

   /**
     *  This method will return this person's primary key
     * in the AppSure system.
     *
     * @return int
     **/
   public int getPersonIndex(){ 
      return personIndex;
   }

} /* Person */
