package com.ibm.appsure.common;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      UTIL
/*   Creation Date: JULY 23, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.io.* ;
import java.text.* ;
import java.util.* ;
/**
 *
 *
 * This class holds Utility Methods that the AppSure system globally uses
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* Class1
/*
/*----------------------------------------------------*/
public class Util {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/


   /*------------------------------------------------------------*/
   /* readFile
    */
   /**
    *    This method will read in a file and return the
    *  contents of the file in a string
    *  
    *
    *  @param fileName The filename to read
    *  @param None
    *  @param None
    *
    *  @return String 
    *//*---------------------------------------------------------*/
   public static String readFile ( String fileName ) throws FileNotFoundException, IOException  {
      String retStr = null ;
      File              file      = new File ( fileName ) ;
      FileInputStream   in        = new FileInputStream ( file ) ;
      int               maxChunk  = 1024 ;
      byte[]            fByte     = new byte[ maxChunk ] ;
      int               i         = 0 ;
      int               readCount = 0 ;

      retStr = "" ;
      while ( readCount >= 0 ) {
         readCount = in.read ( fByte ) ;
         if ( readCount>= 0 )
            retStr = retStr+new String ( fByte, 0, readCount ) ;
      }

      in.close() ;

      return( retStr ) ;
   } /* loadFile */




   /**
     *  This method will return a 3 character abbreviation for what
     * the current day is
     **/
   public static String getDay() {
      Date              todayDate = new Date() ;
      String            day    = "" ;
      SimpleDateFormat  format = null ;

      format = new SimpleDateFormat ( "E" ) ;
      format.setTimeZone ( TimeZone.getDefault() ) ;
      day = format.format ( todayDate ).substring(0,3).toLowerCase() ;

      return( day ) ;

   }

   /**
     *  This method will return a time in standard DB/2 Format
     **/
   public static String getTime() {
      Date              todayDate = new Date() ;
      String            time      = "" ;
      SimpleDateFormat  format    = null ;

      format = new SimpleDateFormat ( "hh:mm:00" ) ;
      format.setTimeZone ( TimeZone.getDefault() ) ;
      time = format.format ( todayDate ) ;

      return( time ) ;

   }


   /**
     *  This method will simply add 5 minutes to the time give
     *
     * @param time The time to add 5 minutes to
     **/
   public static String getAfterTime ( String time ) {
      int    numOfMinutes = 5 ;
      int    colonIndex   = time.indexOf ( ":" ) ;
      int    hour         = Integer.parseInt ( time.substring ( 0, colonIndex ) ) ;
      int    minute       = Integer.parseInt ( time.substring ( colonIndex+1, time.indexOf ( ":", colonIndex+1 ) ) ) ;

      for ( int i = 0; i < numOfMinutes; i++ ) {
         minute++ ;
         if ( minute > 59 ) {
            minute=0 ;
            hour++ ;
            if ( hour > 23 )
               hour=0 ;
         }
      }

      time="";
      if ( hour < 10 )
         time = "0" ;
      time+=""+hour+":" ;

      if ( minute < 10 )
         time+="0" ;
      time+=""+minute+":00" ;

      return( time ) ;
   }


   /**
     *  This method will simply add 5 minutes to the time give
     *
     * @param time The time to add 5 minutes to
     **/
   public static String getBeforeTime ( String time ) {
      int    numOfMinutes = 5 ;
      int    colonIndex   = time.indexOf ( ":" ) ;
      int    hour         = Integer.parseInt ( time.substring ( 0, colonIndex ) ) ;
      int    minute       = Integer.parseInt ( time.substring ( colonIndex+1, time.indexOf ( ":", colonIndex+1 ) ) ) ;

      for ( int i = 0; i < numOfMinutes; i++ ) {
         minute-- ;
         if ( minute < 0 ) {
            minute=59 ;
            hour-- ;
            if ( hour < 0 )
               hour=23 ;
         }
      }

      time="";
      if ( hour < 10 )
         time = "0" ;
      time+=""+hour+":" ;

      if ( minute < 10 )
         time+="0" ;
      time+=""+minute+":00" ;

      return( time ) ;
   }


   /**
     *  This method will create a full timestamp from the time passed in. The
     * date put into the timestamp will be today's date.
     **/
   public static String createTimestampFromTime ( String time ) {
      Date              todayDate = new Date() ;
      String            stamp     = "" ;
      SimpleDateFormat  format    = null ;

      format = new SimpleDateFormat ( "yyyy-MM-dd" ) ;
      format.setTimeZone ( TimeZone.getDefault() ) ;
      stamp = format.format ( todayDate ) ;

      stamp+="-"+time+".000000" ;
      stamp=replaceAllStrInStr ( stamp, ":", "." ) ;

      return( stamp ) ;

   }


   /*------------------------------------------------------------*/
   /* replaceAllStrInStr
    */
   /**
    *    This method will replace all occurances of searchStr
    *  with replaceStr, inside of the string passed
    *  
    *
    *  @param string The String to replace inside of
    *  @param searchStr The String to search for
    *  @param replaceStr The String to replace searchStr with
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String replaceAllStrInStr ( String string, String searchStr, String replaceStr ) {
      String retStr         = "" ;
      int    startIndex     = string.indexOf ( searchStr ) ;
      int    prevStartIndex = startIndex ;
      int    stopIndex      = string.length() ;
      int    replaceStrLen  = searchStr.length() ;

      /* If we don't need to worry about the
         string, don't */
      if ( startIndex > -1 ) {
         StringBuffer retBuffer  = new StringBuffer ( string.substring ( 0, startIndex )+replaceStr ) ;
         boolean      cont       = true ;

         while ( startIndex > -1 ) {
            prevStartIndex = startIndex ;
            startIndex = string.indexOf ( searchStr, startIndex+replaceStrLen ) ;

            if ( startIndex > -1 ) {
               retBuffer.append ( string.substring ( prevStartIndex+replaceStrLen, startIndex ) ) ;
               retBuffer.append ( replaceStr ) ;
            }
         }

         retBuffer.append ( string.substring ( prevStartIndex+replaceStrLen ) ) ;
         retStr = retBuffer.toString() ;
         retBuffer = null ;

      } else
         retStr = string ;


      return( retStr ) ;
   } /* replaceAllInStr */


   /**
     *  Cheap hack at getting the diff in time in minutes
     **/
   public static int getTimeDiff ( String timestamp1, String timestamp2 ) {
      int hour1   = CDate.parse ( timestamp1, CDate.DATE_HOUR, CDate.DATE_STAMP ) ;
      if ( hour1 == 0 ) hour1 = 24 ;
      int hour2   = CDate.parse ( timestamp2, CDate.DATE_HOUR, CDate.DATE_STAMP ) ;
      if ( hour2 == 0 ) hour2 = 24 ;
      int minute1 = CDate.parse ( timestamp1, CDate.DATE_MINUTE, CDate.DATE_STAMP ) ;
      int minute2 = CDate.parse ( timestamp2, CDate.DATE_MINUTE, CDate.DATE_STAMP ) ;
      int hours   = Math.abs ( hour1-hour2 ) ;
      int minutes = Math.abs ( minute1-minute2 ) ;
      int diff    = (hours*60)+minutes ;


      return( diff ) ;
   }
} /* Util */
