package com.ibm.appsure;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      AppSure
/*   Creation Date: MAY 7, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.appsure.common.* ;
import com.ibm.appsure.db.* ;
/**
 * The AppLog object holds all information about a specific log
 * file.
 *
 * @author  David Lentz
 * @version 1
 */


public class AppLog extends Object {

    /**
      * This property contains the entire contents of the log file
      **/
    private String logFileContents = null ;

    /**
      * This is the unique index for the log
      **/
    private int logIndex = 0 ;

    /**
      * This is the 'keep' property notifying the AppSure Clean up to
      * keep this log forever
      **/
    private boolean keep = false ;

    /**
      * This is the timestamp on the log file. The timestamp is in format YYYY-MM-DD-HH-MM-SS-000000
      **/
    private String timestamp = null ;


    /**
      * Default Constructor
      **/
    private AppLog () {
        updateTimestamp() ;
    }

    /**
      * This constructor initializes this record for an insert into the AppSure system
      **/
    private AppLog ( String logFileContents, boolean keep ) {
        setLogFileContents ( logFileContents ) ;
        setKeep ( keep ) ;
        updateTimestamp() ;
    }


    /**
      * This method will update the timestamp to the current time
      **/
    public void updateTimestamp () {
        timestamp = new CDate ( CDate.DATE_STAMP ).today() ;
    }


    /**
      * This method will set the keep property
      *
      * @boolean keep. TRUE = Keep this log forever
      **/
    public void setKeep ( boolean keep ) {
        this.keep = keep ;
    }

    /**
      * This method will set the Log File Contents
      *
      * @param String logFileContents The contents of the log file
      **/
    public void setLogFileContents ( String logFileContents ) {
        this.logFileContents = logFileContents ;
    }

    /**
      * This method will return the contents of the log file
      *
      * @return String
      **/
    public String getLogFileContents () {
        return( logFileContents ) ;
    }

    /**
      * This method will return the Keep property
      *
      * @return boolean
      **/
    public boolean getKeep () {
        return( keep ) ;
    }


    /**
      * This method will return the Log File Index
      *
      * @return int
      **/
    public int getLogFileIndex () {
        return( logIndex ) ;
    }

    /**
      * This method will return the timestamp of this
      * log file.
      *
      * @return String The DB/2 Timestamp of the Log File
      **/
    public String getLogFileTimestamp () {
        return( timestamp ) ;
    }
}
