package com.ibm.appsure.db ;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      AppSureSQL
/*   Creation Date: MARCH 26, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.appsure.* ;
import com.ibm.appsure.common.* ;
/**
 * The AppSureSQL object holds static SQL Methods that the AppSure
 * API Uses.
 *
 *
 * @author  David Lentz
 * @version 1
 */

public class AppSureSQL {


   /**
     * This method will return TRUE or FALSE as to if the Application
     * passed in is currently running
     *
     * @param String appCode The AppCode of the Application to check
     *
     * @return TRUE if the application is running, FALSE if not running or not found
     **/
   public static boolean isAppRunning ( Connection conn, String appCode ) throws AppSureException {
      boolean running = false ;
      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT STARTTIME "+
                                              "FROM APPSURE.appProcess "+
                                              "WHERE APPCODE = '"+appCode+"' AND "+
                                              "STOPTIME IS NULL" ) ;

         running = rs.next() ;

         rs.close() ;
         stmt.close() ;

      } catch ( SQLException e ) {
         throw ( new AppSureException ( e ) ) ;
      }

      return( running ) ;
   }



   /**
     *  This method will log an application as running.
     **/
   public static int logApplicationRunning ( Connection conn, String appCode ) throws AppSureException {
      int       processIndex = -1 ;

      try {
         Statement stmt         = conn.createStatement() ;

         processIndex = getNextProcessIndex ( conn, appCode ) ;

         stmt.executeUpdate ( "INSERT INTO APPSURE.appProcess "+
                              "values ( "+processIndex+", '"+appCode+"', CURRENT TIMESTAMP, NULL )" ) ;
         stmt.close() ;
      } catch ( SQLException e ) {
         throw ( new AppSureException ( e ) ) ;
      }
      return( processIndex ) ;
   }


   /**
     *  This method will log an application as stopped.
     *
     * @param conn The Connection to the database
     * @param processIndex The process Index for the application
     * @param appCode The Application to stop
     **/
   public static void logApplicationStopped ( Connection conn, int processIndex, String appCode ) throws AppSureException {
      try {
         Statement stmt = conn.createStatement() ;

         stmt.executeUpdate ( "UPDATE APPSURE.APPPROCESS SET STOPTIME = CURRENT TIMESTAMP WHERE PROCESSINDEX = "+processIndex ) ;
         stmt.close() ;
      } catch ( SQLException e ) {
         throw ( new AppSureException ( e ) ) ;
      }
   }



   /**
     *  This method creates a new log file in the AppSure system
     *
     * @param conn The Connection to the database
     * @param appCode The application code to log under
     * @param log The contents of the log file to log
     **/
   public static int createLog ( Connection conn, String appCode, String logFile ) throws AppSureException {
      int logIndex = getNextLogIndex ( conn ) ;

      try {
         PreparedStatement stmt     = null ;

         if ( logIndex > 0 ) {
            stmt = conn.prepareStatement ( "INSERT INTO APPSURE.logs "+
                                           "values ( "+logIndex+", '"+appCode+"', ?, CURRENT TIMESTAMP, 'Y' )" ) ;

            stmt.setBytes ( 1, logFile.getBytes() ) ;
            stmt.execute() ;
            stmt.close() ;
         } else
            throw new AppSureException ( "Cannot create any log files. LogIndex corrupt" ) ;
      } catch ( SQLException e ) {
         throw ( new AppSureException ( e ) ) ;
      } /* catch  */

      return( logIndex ) ;
   }


   /**
     *  This method simply returns the next available log index that
     * can be used in the AppSure.LOGS table
     *
     * @param conn The Connection object
     *
     * @return the Log Index
     **/
   private static int getNextLogIndex ( Connection conn ) {
      int rc = 0 ;
      try {
         Statement stmt = conn.createStatement () ;
         ResultSet rs   = stmt.executeQuery ( "SELECT MAX(LOGINDEX) FROM APPSURE.LOGS" ) ;

         if ( rs.next() )
            rc = rs.getInt(1)+1 ;

         rs.close() ;
         stmt.close() ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
         rc = -1 ;
      }

      return( rc ) ;
   }


   /**
     *  This method will create a failure in the AppSure.FAILURES table
     *
     * @param conn The Database Connection
     * @param appCode The Application code to log the failure against
     * @param failureType The type of failure that is going to be logged
     * @param logIndex The index of the log files in the AppSure.Logs table
     *
     * @return TRUE if a failure was created
     **/
   public static boolean createFailure ( Connection conn, String appCode, int failureType, int logIndex ) throws AppSureException {
      boolean created = false ;

      try {
         PreparedStatement stmt     = null ;

         stmt = conn.prepareStatement ( "INSERT INTO APPSURE.failure "+
                                        "values ( "+logIndex+", "+failureType+", NULL, NULL, NULL, '')" ) ;

         stmt.execute() ;
         stmt.close() ;
         created = true ;
      } catch ( SQLException e ) {
         throw ( new AppSureException ( e ) ) ;
      } /* catch  */

      return( created ) ;
   }


   /**
     * This method will return the Failure for the last time an application
     * was run.
     *
     *
     * @param conn The Database connection to the AppSure System
     * @param appCode The Application Code to look up
     **/
   public static Failure getLastFailure ( Connection conn, String appCode ) throws AppSureException {
      Failure failure = new Failure() ;

      try {
         Statement stmt = conn.createStatement () ;
         ResultSet rs   = stmt.executeQuery ( "SELECT LOGS.LOGINDEX,  LOGS.LOG,            LOGS.LOGTIMESTAMP, "+
                                              "       LOGS.KEEP,      FAILURE.FAILURETYPE, FAILURE.NOTIFIED, "+
                                              "       FAILURE.CLOSED, FAILURE.WORKING,     FAILURE.RESOLUTION "+
                                              "FROM APPSURE.LOGS LOGS, APPSURE.FAILURE FAILURE "+
                                              "WHERE LOGS.APPCODE = '"+appCode+"' AND "+
                                              "      FAILURE.LOGINDEX = LOGS.LOGINDEX AND "+
                                              "	FAILURE.WORKING  IS NULL AND "+
                                              "	FAILURE.CLOSED IS NULL" ) ;


         if ( rs.next() ) {
            LogFile logFile = new LogFile ( rs.getInt(1) ) ;

            logFile.setLogContents   ( rs.getString(2) ) ;
            logFile.setLogTimestamp  ( rs.getString(3).trim() ) ;
            logFile.setKeep          ( rs.getString(4).equals ( "Y" ) ) ;
            failure.setFailureType   ( rs.getInt ( 5 ) ) ;
            failure.setNotified      ( rs.getString( 6 ) ) ;
            failure.setClosed        ( rs.getString( 7 ) ) ;
            failure.setWorking       ( rs.getString( 8 ) ) ;
            failure.setResolution    ( rs.getString(9) ) ;
            failure.setLogFile       ( logFile ) ;
         }


         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         e.printStackTrace() ;
         throw ( new AppSureException ( e ) ) ;
      }

      return( failure ) ;
   }



   /**
     * This method will obtain the next available processIndex in the 
     * process table
     **/
   private static int getNextProcessIndex ( Connection conn, String appCode ) throws AppSureException {
      int    rc       = 0 ;
      Failure failure = new Failure() ;

      try {
         Statement stmt = conn.createStatement () ;
         ResultSet rs   = stmt.executeQuery ( "SELECT MAX(PROCESSINDEX), MIN(PROCESSINDEX) "+
                                              "FROM APPSURE.APPPROCESS" ) ;


         if ( rs.next() ) {
            int max = rs.getInt(1) ;
            int min = rs.getInt(2) ;

            if ( min > 1 )
               rc = min-1 ;
            else
               rc = max+1 ;

            if ( rc <= 0 )
               rc = max+1 ;

         } else
            rc = 1 ; // No processes


         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         e.printStackTrace() ;
         throw new AppSureException ( e ) ;
      }

      return( rc ) ;
   }



   /**
     * This method will simply return a property from the APPSURE.PROPERTIES table. 
     *
     * @param connection       The connection to the database
     * @param propertyName     The property name to retrieve
     * @param defaultProperty  If any error occurers or no property match is found, this value is returned
     **/
   public static String getProperty ( Connection conn, String propertyName, String defaultProperty ) {
      String property = null ;

      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT VALUE FROM APPSURE.PROPERTIES WHERE PROPERTY = '"+propertyName+"'" ) ;

         if ( rs.next() )
            property = rs.getString(1).trim() ;

         rs.close() ;
         stmt.close() ;

      } catch ( Exception e ) {
         property = defaultProperty ;
      }

      return( property ) ;
   }



   /**
     *  This method will read in all unique AppCodes from the AppSure.Application table and
     * returns a Vector containing them. The Vector contains String objects
     *
     * @param conn The Database Connection to use
     **/
   public static Vector getUniqueApplications ( Connection conn ) throws Exception {
      Vector appCodes = new Vector ( 10 ) ;

      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT APPCODE FROM APPSURE.APPLICATION where APPCODE <> 'DEFAULT'" ) ;

         while ( rs.next() )
            appCodes.addElement ( rs.getString(1).trim() ) ;

         rs.close() ;
         stmt.close() ;

      } catch ( Exception e ) {
         throw ( e ) ;
      }

      return( appCodes ) ;

   }



   /**
     *  This method will read the Failure Type and return a FailureType object
     * populated from the information in the database
     *
     * @param conn The Connection to the database
     * @param failureType The Failure Type to read in
     *
     * @return FailureType object
     **/
   public static FailureType readFailureType ( Connection conn, int failureTypeInd ) throws AppSureException {
      FailureType failureType = new FailureType( failureTypeInd ) ;

      try {
         Statement stmt = conn.createStatement () ;
         ResultSet rs   = stmt.executeQuery ( "SELECT FAILURETYPE, DESCRIPTION, EMAIL, PAGE, SEVERITY "+
                                              "FROM APPSURE.FAILURETYPE "+
                                              "WHERE FAILURETYPE = "+failureTypeInd ) ;


         if ( rs.next() ) {
            failureType.setDescription ( rs.getString(2).trim() ) ;
            failureType.setEmail ( rs.getString(3).equals ( "Y" ) ) ;
            failureType.setPage  ( rs.getString(4).equals ( "Y" ) ) ;
            failureType.setSeverity ( rs.getInt(5) ) ;
         }


         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         e.printStackTrace() ;
         throw ( new AppSureException ( e ) ) ;
      }

      return( failureType ) ;
   }



   /**
     *  This method will read a person object from the database. The person must be associated
     * with the application specified in 'AppCode' and must be of type contactType.
     *
     * @param conn The Database Connection
     * @param appCode The Application to read the person from
     * @param personType The type of person to read
     *
     * @return Contact object
     **/
   public static Contact getContact ( Connection conn, String appCode, int contactType ) throws AppSureException {
      Contact contact = null ;

      try {
         Statement stmt = conn.createStatement () ;
         ResultSet rs   = stmt.executeQuery ( "SELECT PERSONINDEX, CONTACTTYPE "+
                                              "FROM APPSURE.CONTACT "+
                                              "WHERE APPCODE = '"+appCode+"' AND "+
                                              "      CONTACTTYPE = "+contactType ) ;


         if ( rs.next() )
            contact = new Contact ( appCode, rs.getInt(1), rs.getInt(2) ) ;

         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         e.printStackTrace() ;
         throw ( new AppSureException ( e ) ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read in the person info                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( contact != null )
         contact.setPerson ( readPerson ( conn, contact.getPersonIndex() ) ) ;



      return( contact ) ;
   }




   /**
     *  This method will read in all of a person's profile. It will store
     * the information in the Person object returned
     *
     * @param conn The Database connection
     * @param personIndex The Primary Key (or person index) for the person
     *
     * @return Person
     **/
   public static Person readPerson ( Connection conn, int personIndex ) throws AppSureException {
      Person person = null ;

      try {
         Statement stmt = conn.createStatement () ;
         ResultSet rs   = stmt.executeQuery ( "SELECT FIRST_NAME, LAST_NAME, EMAIL, PHONE, PAGER, PAGEREMAIL "+
                                              "FROM APPSURE.PERSON "+
                                              "WHERE PERSONINDEX = "+personIndex ) ;


         if ( rs.next() ) {
            person = new Person ( personIndex ) ; 
            person.setFirstName  ( rs.getString ( 1 ).trim() ) ;
            person.setLastName   ( rs.getString ( 2 ).trim() ) ;
            person.setEmail      ( rs.getString ( 3 ).trim() ) ;
            person.setPager      ( rs.getString ( 4 ).trim() ) ;
            person.setPagerEMail ( rs.getString ( 5 ).trim() ) ;
         }

         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         e.printStackTrace() ;
         throw ( new AppSureException ( e ) ) ;
      }

      return( person ) ;
   }



   /**
     *  This method will read in all application run times for a specific day and application
     *
     * @param conn The Database Connection
     * @param appCode The Application Code to read
     * @param day The 3 character day to read
     * @param time If given, run times will only be read if they have already passed
     *
     * @return Vector of RunTimeRec objects
     **/
   public static Vector readApplicationRunTimes ( Connection conn, String appCode, String day, String time ) throws AppSureException {
      Vector runTimes = new Vector ( 1 ) ;

      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT RUNINDEX, STARTTIME "+
                                              "FROM APPSURE.RUNTIME "+
                                              "WHERE APPCODE = '"+appCode+"' AND "+
                                              "      DAY     = '"+day+"'"+
                                              ((time==null) ? "" : "AND STARTTIME < '"+time+"'") ) ;

         while ( rs.next() ) {
            RunTimeRec runTimeRec = new RunTimeRec ( rs.getInt(1) ) ;
            runTimeRec.setDay ( day ) ;
            runTimeRec.setAppCode ( appCode ) ;
            runTimeRec.setStartTime ( rs.getString(2).trim() ) ;
            runTimes.addElement ( runTimeRec ) ;
         }

         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         throw new AppSureException ( e ) ;
      }
      return( runTimes ) ;
   }


   /**
     *  This methodw will update the notification time to the current time
     * for the failure representing the log file index passed in
     *
     * @param conn The Database Connection
     * @param logIndex The Logfile index
     **/
   public static void updateNotifyTime ( Connection conn, int logIndex ) throws AppSureException {
      try {
         Statement stmt = conn.createStatement() ;
         stmt.executeUpdate ( "UPDATE APPSURE.FAILURE SET NOTIFIED = CURRENT TIMESTAMP WHERE LOGINDEX = "+logIndex ) ;
         stmt.close() ;
      } catch ( SQLException e ) {
         throw new AppSureException ( e ) ;
      }
   }



   /**
     *  Returns the average run time in minutes for an application
     *
     * @param conn The Database Connection
     * @param appCode The application to check
     *
     * @return int
     **/
   public static int getAverageRunTimeMin ( Connection conn, String appCode ) throws AppSureException {
      int averageRunMin = 0 ;

      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT AVERAGERUNTIMEMIN "+
                                              "FROM APPSURE.APPLICATION "+
                                              "WHERE APPCODE = '"+appCode+"'" ) ;

         if ( rs.next() )
            averageRunMin = rs.getInt(1) ;

         rs.close() ;
         stmt.close() ;

      } catch ( SQLException e ) {
         throw new AppSureException ( e ) ;
      }
      return ( averageRunMin ) ;
   }


   /**
     *  This method will read all open processes (processes that have not ended) from
     * the AppSure database.
     *
     * @param conn The Database Connection
     * @param appCode The Application to read
     *
     * @return Vector of 
     **/
   public static Vector readApplicationOpenProcesses ( Connection conn, String appCode ) throws AppSureException {
      Vector processes = new Vector ( 1 ) ;

      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT STARTTIME "+
                                              "FROM APPSURE.APPPROCESS "+
                                              "WHERE APPCODE = '"+appCode+"' AND "+
                                              "      STOPTIME IS NULL " ) ;

         while ( rs.next() ) {
            ProcessRec processRec = new ProcessRec() ;

            processRec.setAppCode ( appCode ) ;
            processRec.setStartTime ( rs.getString ( 1 ).trim() ) ;

            processes.addElement ( processRec ) ;
         }

         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         throw new AppSureException ( e ) ;
      }
      return( processes ) ;
   }


   /**
     *  This method will populate a LogFile object's contents
     **/
   public static void readLogContents ( Connection conn, LogFile log ) throws AppSureException {
      try {
         Statement stmt = conn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT LOG "+
                                              "FROM APPSURE.LOGS "+
                                              "WHERE LOGINDEX = "+log.getLogIndex()+" FOR FETCH ONLY" ) ;

         if ( rs.next() )
            log.setLogContents ( new String ( rs.getBytes(1) ) ) ;

         rs.close() ;
         stmt.close() ;
      } catch ( SQLException e ) {
         throw new AppSureException ( e ) ;
      }
   }
}


