-- *********************************************************
-- *                                                       *
-- * AppSure v1.0                                          *
-- *                                                       *
-- * This DDL Script creates all of the tables,            *
-- * foreign keys and indexes for the AppSure product.     *
-- *                                                       *
-- *                                                       *
-- *********************************************************


-- *********************************************************
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *                  Table Creation                       *
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *********************************************************


-- *********************************************************
-- *                                                       *
-- * APPSURE.APPLICATION                                   *
-- *                                                       *
-- *********************************************************
create table appSure.application (
        appCode           char(10)        not null,
        name              char(254)       not null,
        about             varchar(1024),
        runLocation       char(100)       not null,
        multipleInst      char(1)         not null,
        shortDescript     varchar(254),
        averageRunTimeMin smallint        not null,
        
Primary Key (appCode));


-- *********************************************************
-- *                                                       *
-- * APPSURE.LOGS                                          *
-- *                                                       *
-- *********************************************************
create table appSure.logs (
        logIndex        integer         not null,
        appCode         char(10)        not null,
        log             clob(10M)       not null,
        logTimestamp    timestamp       not null,
        keep            char(1)         not null,
Primary Key (logIndex),
CONSTRAINT keep CHECK ( keep IN ('Y', 'N' ) ) );


-- *********************************************************
-- *                                                       *
-- * APPSURE.PROPERTIES                                    *
-- *                                                       *
-- *********************************************************
create table appSure.properties (
        property        char(50)        not null,
        value           char(254)       not null);
        
-- *********************************************************
-- *                                                       *
-- * APPSURE.RUNTIME                                       *
-- *                                                       *
-- *********************************************************
create table appSure.runTime (
        runIndex        smallint        not null,
        appCode         char(10)        not null,
        startTime       time            not null,
        day             char(3)         not null,
Primary Key (runIndex, appCode),
CONSTRAINT day     CHECK ( day IN ('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' ) ) );


-- *********************************************************
-- *                                                       *
-- * APPSURE.APPPROCESS                                    *
-- *                                                       *
-- *********************************************************
create table appSure.appProcess (
        processIndex    smallint        not null,
        appCode         char(10)        not null,
        startTime       timestamp       not null,
        stopTime        timestamp       ,
Primary Key (processIndex));




-- *********************************************************
-- *                                                       *
-- * APPSURE.FAILURE                                       *
-- *                                                       *
-- *********************************************************
create table appSure.failure (
        logIndex        integer         not null,
        failureType     smallInt        not null,
        notified        timestamp       ,
        closed          timestamp       ,
        working         timestamp       ,
        resolution      varchar(1024)   ,
primary key (logIndex));        


-- *********************************************************
-- *                                                       *
-- * APPSURE.FailureType                                   *
-- *                                                       *
-- *********************************************************
create table appSure.failureType (
        failureType     smallint        not null,
        description     char(50)        not null,
        eMail           char(1)         not null,
        page            char(1)         not null,
        severity        smallint        not null,
primary key (failureType));


-- *********************************************************
-- *                                                       *
-- * APPSURE.CONTACT                                       *
-- *                                                       *
-- *********************************************************
create table appSure.contact (
        appCode         char(10)        not null,
        personIndex     integer         not null,
        contactType     smallInt        not null,
primary key (appCode, personIndex, contactType));

-- *********************************************************
-- *                                                       *
-- * APPSURE.CONTACTTYPE                                   *
-- *                                                       *
-- *********************************************************
create table appSure.contactType (
        contactType     smallInt        not null,
        description     char(50)        not null,
primary key (contacttype));


-- *********************************************************
-- *                                                       *
-- * APPSURE.PERSON                                        *
-- *                                                       *
-- *********************************************************
create table appSure.person (
        personIndex     integer         not null,
        first_name      char(50)        not null,
        last_name       char(50)        not null,
        eMail           char(100)       not null,
        phone           char(25)        not null,
        pager           char(25)        not null,
        pagerEMail      char(100)       not null,
primary key (personIndex));        
        
        
        

-- *********************************************************
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *                  Foreign Keys                         *
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *********************************************************
ALTER TABLE appSure.logs ADD CONSTRAINT appCode
  foreign KEY (appCode) REFERENCES appSure.application
  ON DELETE CASCADE ;

ALTER TABLE appSure.appProcess ADD CONSTRAINT appCode
  foreign KEY (appCode) REFERENCES appSure.application
  ON DELETE CASCADE ;

ALTER TABLE appSure.failure ADD CONSTRAINT logIndex
  foreign KEY (logIndex) REFERENCES appSure.logs
  ON DELETE CASCADE ;
  
ALTER TABLE appSure.failure ADD CONSTRAINT failureType
  foreign KEY (failureType) REFERENCES appSure.failureType
  ON DELETE CASCADE ;

ALTER TABLE appSure.runtime ADD CONSTRAINT appCode
  foreign KEY (appCode) REFERENCES appSure.application
  ON DELETE CASCADE ;
  
ALTER TABLE appSure.contact ADD CONSTRAINT appCode
  foreign KEY (appCode) REFERENCES appSure.application
  ON DELETE CASCADE ;

ALTER TABLE appSure.contact ADD CONSTRAINT personIndex
  foreign KEY (personIndex) REFERENCES appSure.person
  ON DELETE CASCADE ;
  
ALTER TABLE appSure.contact ADD CONSTRAINT contactType
  foreign KEY (contactType) REFERENCES appSure.contactType
  ON DELETE CASCADE ;












-- *********************************************************
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *                  Indexes                              *
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *                                                       *
-- *********************************************************
CREATE INDEX name    ON appSure.application ( name    );
CREATE INDEX closed  ON appSure.failure     ( closed  );
CREATE INDEX working ON appSure.failure     ( working );

INSERT INTO APPSURE.FAILURETYPE VALUES ( 0, 'Default Non Severity', 'N', 'N', 0 ) ;
INSERT INTO APPSURE.FAILURETYPE VALUES ( 1, 'Severity 1', 'N', 'N', 1 ) ;
INSERT INTO APPSURE.FAILURETYPE VALUES ( 2, 'Severity 2', 'N', 'N', 2 ) ;

INSERT INTO APPSURE.APPLICATION VALUES ( 'SAMPLE1', 'Sample Application #1', 'Simple Sample', 'Client', 'N', 'Just a simple sample', 1);
INSERT INTO APPSURE.RUNTIME VALUES ( 1, 'SAMPLE1', '12:00:00', 'wed');
