package com.ibm.appsure.monitor ;       
/*-----------------------------------------------------------*/
/*   AppSureMonitor                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AppSureMonitor                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.net.* ;
import java.io.* ;
import java.util.* ;
import java.sql.* ;
import com.ibm.appsure.common.* ;
import com.ibm.appsure.db.* ;
import com.ibm.appsure.* ;
/**
 * This is the main starting point for the AppSureMonitor program 
 *
 *
 * @version 1.0, 7/24/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AppSureMonitor                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class AppSureMonitor extends Object implements Runnable {

   /**
     *  This is the amount of time (in minutes) that the monitor
     * should sleep for before recycling
     **/
   private int sleepMinutes=10 ;

   /**
     *  A Hashtable containing all command line args and properties
     **/
   private Hashtable properties = null ;

   /**
     *  This is the Database connection that the Daemon will use
     **/
   private Connection dbConn = null ;



   /**
     *  The main starting point for the AppSure Monitor application
     **/
   public static void main ( String args[] ) {
      new AppSureMonitor ( args ) ;
   }


   /**
     *  The default constructor for the AppSureMonitor takes the command
     * line arguments passed to the application
     *
     * @param args The command line arguments
     **/
   AppSureMonitor ( String args[] ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize the properties                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      initializeProperties ( args ) ;                                      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check to see if another instance of the monitor is      */
      /* running                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( checkMultipleInstances() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our Log                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         Log log = new Log ( properties.get ( "LogFileHome" )+"appsuremonitor.log" ) ;
         log.setLogFileMaxSize ( Integer.parseInt ( ""+properties.get ( "MaxLogFileSize" ) ) ) ;              


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Connect to the database                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( connectToDatabase() ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now ,start the deamon                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            new Thread ( this ).start() ;
         }

      }



   }


   /**
     *  This method will initialize all properties for the application,
     * including the properties from the command line and property file
     *
     * @param args The command line arguments
     **/
   private void initializeProperties ( String[] args ) {
      properties = new Hashtable () ;
      readProperties() ;
      processCommandLineArgs ( args ) ;
      sleepMinutes = Integer.parseInt ( ""+properties.get ( "SleepTime" ) ) ;

      if ( properties.get ( "SMTP" ) != null )
         new AppSureEMailer ( (String)properties.get ( "SMTP" ) ) ;
      else {
         System.err.println ( "No SMTP Server specified. Application exiting. Please specify an SMTP server in the AppSureMonitor.properties file.\nFor example: SMTP=mySMTP.mydomain.com" ) ;
         System.exit(-1) ;
      }
   }


   /**
     *  This method will read in all of our properties and populate our
     * property hashtable.
     **/
   private void readProperties () {
      Properties propertyFile = new Properties() ;

      try {
         Enumeration enum = null ;

         propertyFile.load ( new FileInputStream ( new File ( "AppSureMonitor.properties" ) ) ) ;
         enum = propertyFile.propertyNames() ;

         while ( enum.hasMoreElements() ) {
            Object nextElement = enum.nextElement() ;

            properties.put ( nextElement, propertyFile.get ( nextElement ) ) ;
         }
      } catch ( Exception e ) {
         System.err.println ( "Cannot find AppSureMonitor.properties file. Please make sure this file exists in the directory in which you executed the AppSure Monitor." ) ;
      }
   }


   /**
     *  This method will replace all current properties in the 'properties' hash table
     * with the command line args
     **/
   private void processCommandLineArgs ( String[] args ) {
      try {
         for ( int i = 0; i < args.length; i++ ) {
            int    equalIndex    = args[i].indexOf ( "=" ) ;
            if ( equalIndex >= 0 ) {
               String propertyName  = args[i].substring ( 0, equalIndex ) ;
               String propertyValue = args[i].substring ( equalIndex+1 ) ;

               properties.remove ( propertyName ) ;
               properties.put ( propertyName, propertyValue ) ;
            }
         }
      } catch ( Exception e ) {
      }
   }


   /**
     *  This method will attempt to open a ServerSocket for the port number
     * defined in the AppSureMonitor.properties file as property name MonitorStartPortNum. If
     * the Server Socket is opened sucesfully then that means that the AppSureMonitor is not
     * running on this client. If the ServerSocket cannot be opened then the application will
     * exit.
     *
     * @return TRUE if the application was opened, FALSE if not
     **/
   private boolean checkMultipleInstances () {
      int portNum = Integer.parseInt ( ""+properties.get ( "MonitorStartPortNum" ) ) ;

      try {
         ServerSocket socket = new ServerSocket ( portNum ) ;
         return( true ) ;
      } catch ( Exception e ) {
         System.err.println ( "Cannot start due to port number "+portNum+" already being used. This might be an indication that the AppSure Monitor is already running" ) ;
         System.exit(0) ;
      }
      return( false ) ;
   }


   /**
     *  This method will attempt to obtain a database connection to
     * the Database Name, Server and JDBC Port number listed in the
     * properties
     **/
   private boolean connectToDatabase () {
      String database = (String)properties.get ( "Database" ) ;
      String server   = (String)properties.get ( "DatabaseServer" ) ;
      String portNum  = (String)properties.get ( "JDBCPortNumber" ) ;

      Log.log ( "Connecting to Database "+database+" on server "+server+" with port number "+portNum ) ;
      try {
         String      driverName = "COM.ibm.db2.jdbc.net.DB2Driver" ;
         Connection  conn       = null ;

         Class.forName ( driverName ).newInstance() ;

         dbConn = DriverManager.getConnection( "jdbc:db2://"+server+":"+portNum+"/"+database, "dlentz", "m0seley" );

      } catch ( Exception e ) {
         Log.log ( "Cannot connect to database "+database+" on server "+server+" with port number "+portNum ) ;
         Log.log ( e ) ;
         return( false ) ;
      }
      return( true ) ;
   }

   /**
     *  This run method runs the actual Daemon that loops forever
     **/
   public void run () {
      boolean rc = true ;

      while ( rc ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Make sure our database connection is still good         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !verifyConnection ( ) ) {
            Log.log ( "Database Connection failed. Trying reconnect" ) ;
            rc = connectToDatabase() ;
         }




         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Check the software                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         Log.skipLine() ;
         Log.skipLine() ;
         Log.log ( "Monitor Checking Software" ) ;
         rc = checkAppSureSoftware() ; 

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, sleep                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            if ( sleepMinutes != 1 )
               Log.log ( "Monitor Sleeping for "+sleepMinutes+" Minutes" ) ;
            else
               Log.log ( "Monitor Sleeping for "+sleepMinutes+" Minute" ) ;

            Thread.currentThread().sleep ( sleepMinutes*60000 ) ;
         } catch ( Exception e ) {
            Log.log ( "Monitor awoken with error" ) ;
            Log.log ( e ) ;
            rc = false ;
         }
      }
   }



   /**
     *  This method does the actual check against all software
     * running
     **/
   private boolean checkAppSureSoftware () {
      try {
         Vector appCodeVec = AppSureSQL.getUniqueApplications ( dbConn ) ;
         int    i          = 0 ;
         int    size       = appCodeVec.size() ;

         if ( size > 0 ) {
            String appCode = null ;
            Log.log ( "Checking "+appCodeVec.size()+" Application(s)" ) ;

            while ( i < size ) {
               appCode = (String) appCodeVec.elementAt(i) ;
               checkApplication ( appCode ) ;
               i++ ;
            }
            System.out.println ( "Done Check" ) ;
         } else
            Log.log ( "No Applications to check" ) ;

      } catch ( Exception e ) {
         Log.log ( "Exception caught while processing applications" ) ;
         Log.log ( e ) ;
         return( true ) ;
      }
      return( true ) ;
   }



   /**
     *  This method will ensure that the application passed meets the
     * following criteria:<br><br><pre>
     *
     * 1. That is ran Ok last time. If it did not then an alert will go out
     * 2. That the application is not running past it's normal run time. If it is still
     *    running then an alert will go out</pre>
     *
     * @param appCode The AppCode for the Application Code to check
     **/
   private void checkApplication ( String appCode ) {
      Failure failure = null ;                          

      try {

         /* Check to see if the application ran in it's  */
         /* Designated run time for today                */
         Log.log ( "   Checking Application "+appCode ) ;
         //if ( checkApplicationStart   ( appCode ) )
         checkApplicationRuntime ( appCode ) ;
         checkApplicationFailure ( appCode ) ;

      } catch ( AppSureException e ) {
         Log.log ( "***Cannot check application "+appCode+" due to exceptions" ) ;
         e.printStackTrace() ;
      } catch ( Exception e ) {
         Log.log ( "***Cannot check application "+appCode+" due to exceptions" ) ;
         e.printStackTrace() ;
      }
   }


   /**
     *  This method will check an application for any failures that may have
     * been logged
     *
     * @param appCode The Application Code for the failure
     **/
   private void checkApplicationFailure ( String appCode ) throws Exception, AppSureException {
      Failure failure = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check for a failure in the last run                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      Log.log ( "      Checking "+appCode+" for failures" ) ;
      failure = AppSureSQL.getLastFailure ( dbConn, appCode ) ;

      if ( failure != null ) {
         if ( !failure.isSevZero() ) {
            if ( failure.getNotified() == null )
               failure.sendAlert ( dbConn, appCode ) ;
         } else
            Log.log ( "      "+appCode+" ended without error" ) ;
      } else
         Log.log ( "      Could not find any failures" ) ;
   }



   /**
     *  This method will check to see if an application is running longer
     * than expected. If it is then the failureType 1 will be sent out. 
     *
     * @param appCode The Application code to check
     **/
   private void checkApplicationRuntime ( String appCode ) {

      Log.log ( "      Checking Application Run Time for "+appCode ) ;
      try {
         int    averageRunMin = AppSureSQL.getAverageRunTimeMin ( dbConn, appCode ) ;
         Vector processes     = AppSureSQL.readApplicationOpenProcesses ( dbConn, appCode ) ;

         if ( ( processes != null ) && ( processes.size() > 0 ) ) {
            ProcessRec processRec = null ;
            int        i          = 0 ;
            int        size       = processes.size() ;
            String     today      = new CDate ( CDate.DATE_STAMP ).today() ;

            while ( i < size ) {
               processRec = (ProcessRec)processes.elementAt(i) ;
               int diff = Util.getTimeDiff ( today, processRec.getStartTime() ) ;

               if ( diff > averageRunMin ) {
                  int logIndex = AppSureSQL.createLog ( dbConn, appCode, appCode+" is still running after "+diff+" minute(s). The average run time for this application is "+averageRunMin+"(minutes)" ) ;

                  AppSureSQL.createFailure ( dbConn, appCode, 3, logIndex ) ;

                  Log.log ( "      Application "+appCode+" is still running after "+diff+" minute(s)" ) ;

               } else
                  Log.log ( "      Application "+appCode+" is running in it's allotted time" ) ;

               i++ ;
            }
         }

      } catch ( AppSureException e ) {
         Log.log ( "******Cannot Check Application Runtime for "+appCode ) ;
         Log.log ( e ) ;
      }
   }



   /**
     *  This method will simply try to select any value from the APPSURE.PROPERTIES
     * table. This is to determine if the database connection that the monitor
     * is using is still good. 
     *
     * @returns TRUE if connection is good, FALSE if not
     **/
   private boolean verifyConnection () {
      boolean rc = false ;

      try {
         Statement stmt = dbConn.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT * from APPSURE.PROPERTIES" ) ;

         rc =  rs.next() ;

         rs.close() ;
         stmt.close() ;
      } catch ( Exception e ) {
         rc = false ;
      }
      return( rc ) ;
   }


   /**
     *  This method will make sure that an application has started when it
     * said it was going to start
     *
     * @return TRUE if the application has started, FALSE if not
     **/
   private boolean checkApplicationStart ( String appCode ) {
      boolean checked = false ;

      try {
         Vector runTimes = AppSureSQL.readApplicationRunTimes ( dbConn, appCode, Util.getDay(), Util.getTime() ) ;

         if ( ( runTimes != null ) && ( runTimes.size() > 0 ) ) {
            int        i          = 0 ;
            int        size       = runTimes.size() ;
            RunTimeRec runTimeRec = null ;

            while ( i < size ) {
               runTimeRec = (RunTimeRec) runTimes.elementAt(i) ;

               try {
                  checked = runTimeRec.didAppRun ( dbConn ) ;                                              

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   If the application did not run, create a failure for it */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( !checked ) {
                     int logIndex = AppSureSQL.createLog ( dbConn, appCode, "Application did not start at specified time of "+runTimeRec.getStartTime() ) ;
                     AppSureSQL.createFailure ( dbConn, appCode, 2, logIndex ) ;
                     Log.log ( "      Application "+appCode+" did not run at "+runTimeRec.getStartTime() ) ;
                  } else
                     Log.log ( "      Application "+appCode+" ran at "+runTimeRec.getStartTime() ) ;

               } catch ( Exception e ) {
                  Log.log ( "******Could not check Application Run Time for RunTimeIndex:"+runTimeRec.getRunTimeIndex() ) ;
                  Log.log ( e ) ;
               }
               i++ ;
            }
         }

      } catch ( AppSureException e ) {
         Log.log ( "******Cannot check Application Start. Exception:" ) ;
         Log.log ( e ) ;
      }

      return( checked ) ;
   }
} /* AppSureMonitor */
