package com.ibm.nzna.projects.OASearchExport ;         
/*-----------------------------------------------------------*/
/*   OASearchExport                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OASearchExport                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
/**
 * This batch extracts Symptom information from the OA Tables of Quest. Three
 * files are produced from this batch and used in creating the Search Engine
 * database. These files include:<br><br>
 *
 * <pre>
 * Filename             File Description<br>
 * -------------        ---------------------------------------------<br>
 * OA_Unclassified.txt  All Unclassified Symptoms<br>
 * OA_Dealer.txt        All  Dealer Confidential Symptoms.<br>
 * OA_Confidential.txt  All Confidential Symptoms (used for Info Tips)<br>
 *
 * To run this batch, the OASearchExport.property file must be in the client
 * directory. This file holds information such as Database Name, Server etc..<br><br>
 *
 *
 * @version 1.0, 11/3/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OASearchExport                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class OASearchExport extends Batch {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Properties properties          = null ;
   private boolean    continueProcess     = true ;
   private boolean    shutdownDatabase    = false ;




   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This is the main entry point for the application. It
    *  will kick off and run the batch. No arguments are currently
    *  taken.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String args[] ) {
      new OASearchExport() ;
      System.exit(0) ;
   }




   /*------------------------------------------------------------*/
   /* OASearchExport
    */
   /**
    *    This is the constructor that will initialize and
    *  then run our batch
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OASearchExport () {
      initializeLogFile() ;
      startBatch() ;
   }




   /*------------------------------------------------------------*/
   /* initializeLogFile
    */
   /**
    *    This method will initialize our batch log
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeLogFile () {
      new BatchLogSystem ( "OASearchExport", this ) ;
   }





   /*------------------------------------------------------------*/
   /* startBatch
    */
   /**
    *    This method will start the batch program and
    *  run through everything
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void startBatch() {                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Load in our properties                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         properties = new Properties() ;
         properties.load ( new FileInputStream ( new File ( "OASearchExport.property" ) ) ) ;
      } catch ( Exception e ) {
         logError ( "Cannot load Property file OASearchExport.property" ) ;
         logSevereError ( e ) ;
         e.printStackTrace() ;
         System.exit(-1) ;
      }



      /*------------------------------------------------------------*/
      /*                                                            */
      /*   Connect to the database. If there isn't a DatabaseSystem */
      /* instance then create one                                   */
      /*                                                            */
      /*------------------------------------------------------------*/
      if ( DatabaseSystem.getInstance() == null ) {
         try {
            new DatabaseSystem ( true, BatchLogSystem.getInstance(), true ) ;
            shutdownDatabase = true ;


         } catch ( Exception e ) {
            logSevereError ( e ) ;
            System.exit(-1) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we get a connection, continue                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( DatabaseSystem.createConnection ( DatabaseSystem.QUEST,
                                             properties.getProperty ( "DATABASENAME",   "QUEST" ),
                                             properties.getProperty ( "DATABASESERVER", "LOCALHOST" ),
                                             "",
                                             "quest",
                                             "j0hnny" ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Process the Confidential                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( continueProcess )
            exportData ( "confidential.import", TypeList.DOCCLASS_IBM_CONFIDENTIAL ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Process the Dealer Confidential                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( continueProcess )
            exportData ( "dealer.import", TypeList.DOCCLASS_DEALER_CONFIDENTIAL ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Process the Unclassified                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( continueProcess )
            exportData ( "unclassified.import", TypeList.DOCCLASS_UNCLASSIFIED ) ;


      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Shutdown the database if need be                        */                       
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( shutdownDatabase ) {
         DatabaseSystem.shutdown() ;
      }
      

   }


   /*------------------------------------------------------------*/
   /* stopBatch
    */
   /**
    *    This method will stop the batch elegantly
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopBatch () {
      continueProcess = false ;
   }





   /*------------------------------------------------------------*/
   /* exportData
    */
   /**
    *    This method will export all of the data to the
    *  specified filename.
    *  
    *
    *  @param filename The filename of the file to create and export data to
    *  @param docClassInd The indicator of the doc class to read as defined in TIGRIS.TYPEDOCCLASS
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean exportData ( String filename, int docClassInd ) {
      boolean rc = false ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "exportData ( "+filename+", "+docClassInd+" )", 60 ) ;

      try {
         Statement        stmt = sqlMethod.createStatement() ;                        
         FileOutputStream out  = new FileOutputStream ( new File ( filename ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the statement is good continue                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( stmt != null ) {
            int       geoInd = 0 ;
            int       i      = 2000 ;
            ResultSet rs     = stmt.executeQuery ( "SELECT DISTINCT A.SYMPTOMIND, C.GEOIND, A.TITLE, "+
                                                   "                A.KEYWORDS,   C.COUNTRYCODEIND,  "+
                                                   "                D.BRANDIND                       "+
                                                   "FROM OA.SYMPTOMTITLE      A,                     "+
                                                   "     OA.SYMPTOMTITLECNTRY B,                     "+
                                                   "     TIGRIS.GEOCOUNTRY    C,                     "+
                                                   "     OA.SYMPTOM           D                      "+
                                                   "WHERE A.SYMPTOMIND      = D.SYMPTOMIND AND       "+
                                                   "      A.STATUSIND       = "+Status.TYPESTATUS_FLAWED+" AND "+
                                                   "      D.ARCHIVED        = 'N' AND                "+
                                                   "      A.SYMPTOMTITLEIND = B.SYMPTOMTITLEIND AND  "+
                                                   "      B.COUNTRYIND      = C.COUNTRYCODEIND AND   "+
                                                   "      A.DOCCLASSIND     = "+docClassInd+"        "+
                                                   "FOR FETCH ONLY" ) ;                                 

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, loop through and write each value out              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( ( continueProcess ) && ( rs.next() ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If the Country is US or Canada, set the geography       */
               /* special. This should be taken out later.                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( rs.getInt(5) == 19 )
                  geoInd = 202 ;
               else if ( rs.getInt(5) == 110 )
                  geoInd = 201 ;
               else if ( rs.getInt(5) == 124 )
                  geoInd = 6 ;
               else
                  geoInd = rs.getInt(2) ;                 


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Write the line to the file                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               out.write ( new String ( Text.formatStringLength ( ""+i,            8 )+
                                        Text.formatStringLength ( ""+rs.getInt(1), 8 )+
                                        Text.formatStringLength ( ""+geoInd,       8 )+
                                        Text.formatStringLength ( ""+rs.getInt(6), 8 )+
                                        Text.formatStringLength ( ""+docClassInd,  8 )+
                                        rs.getString(3).trim()+" "+
                                        rs.getString(4).trim()+"\n" ).getBytes() ) ;

               i++ ;
            }

            rs.close() ;
            out.close() ;

         } else {
            logError ( "Cannot create statement object" ) ;
            continueProcess = false ;
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         logSevereError ( e ) ;
         continueProcess = false ;
      }

      sqlMethod.close() ;
      return( rc ) ;
   }

} /* OASearchExport */
