package com.ibm.nzna.projects.PEWLookup;
 
import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.util.* ;

/**
 * PEWLookup.fetch Servlet
 * @(#)fetch.java	1.0 03/29/2000
 * @author Mike Rychener, IBM IGS for PSG Helpcenter
 **/

/* specs 
 *
 * takes in GET/POST parameters mtmdl (4 chars become <type>), country, serial
 * returns an HTML document displaying warranty info or error message
 *
 */

/* HISTORY
 *  03/24  mdr  created skeleton
 *  03/27  mdr  fill in URL construction, raw XML print
 *  03/28  mdr  convert to using buffered byte array and setting content length at end
 *                (p. 128 in Java Servlet Programming, J. Hunter & W. Crawford)
 *  03/29  mdr  incorporate output HTML formats & input CGI parameters
 *                - then retracted the CGI getParameter stuff, was causing servletrunner error?
 *  03/30  mdr  parse XML to fill in HTML forms
 *  03/31  mdr  finish preceding
 *  04/04  mdr  split out pew_select_template to file
 *  04/05  mdr  connect to actual remote PEW
 *  04/10  mdr  get properties: template & URL paths from file
 *              query string parameters + path info from calling URL
 *  04/12  mdr  add warranty calculation and error code check; expand fields displayed (Infotips)
 *  04/13  mdr  fill in gaps in display (now full Infotips content)
 *  04/14  mdr  convert to use Chad Meadows Base class from borg; add package & imports; rename
 *                -- using HelloWorldServlet example
 *              now a better way to read properties file = base class sets properties field var
 *  04/19  mdr  make some outputs conditional on site, eg, barcode not on eSupport
 *  04/24  mdr  'test' won't do as path info, because it contains 'es' -- stick with 2-letter te
 *  04/25  mdr  check that <validate> and </validate> both exist
 *              Javadoc texts added
 *  04/26  mdr  change read(xbuf) to iterate, make sure end of stream reached
 *  04/27  mdr  more writeLog's; embed parsing in try-catch, with stack dump into comment;
 *                error messages for parameters more friendly; full XML into comment
 *  04/29  mdr  change reading to be from InputStreamReader, xbuf becomes char array
 *              Calendar calc for today's date was bad: need Get, not just field names
 *              change default strings when various values not found in XML (3rd of 3)
 *              calculate and utilize partEnd in parsing <part> expr's
 *  05/01  mdr  adjust month values set in Calendar objects for 0..11 convention
 *  05/05  mdr  add message if no parts service info table is output (no <part>s from PEW)
 *  05/07  mdr  add today's date substitution for __Today__ in templates' main title
 *              new fn formatCalDate, used for current date, et al to get them printable
 *                - it adds leading 0's for small numbers
 *              new template value RowFont22, used in certain locations (warranty descr's)
 *              revise error messages per customer requirements, some site-specific
 *              expand __selfUrl__ when appropriate in WhiteEnd template string
 *  05/08  mdr  include mfg part numbers in bp display, new customer req
 *              remove some of the logging around I/O
 *              new method subSplice used in the __selfUrl__ section; also for __Today__
 *  05/10  mdr  modify url connect exception message -- add more <p>'s, just in case long junk
 *              embed 'no service info' message for parts in <table>, try for better placement
 *  06/08  mdr  add 'local service center' substitution of ServiceLink read from template file
 *  06/20  mdr  add __selfHost__ and __selfGeo__ splices in WhiteEnd section (latter only for BP)
 *                and new properties entry webHost
 *              add pickup of socksProxyHost and -Port from properties, + setting in system
 *  06/27  mdr  allow <head> in template files, so they have <meta> tags; replace constant prints
 *                of header stuff with template array strings
 *              exclude <head> section when displaying 'es' version of template
 *  07/13  mdr  add timing of PEW database servlet, printed to log
 *              allow '/' in PEWhost name -- only use prefix as real host, rest as path fragment
 *  07/25  mdr  remove socks property settings -- now done exclusively in global.properties base
 *  -----  TO DO:
 *              allow documents to be fetched from GWA site, ie, explicit w[3,ww].pc.ibm.com
 *                in fixed links could be subSplice'd -- webHost? new property docHost?
 *                - would have to put __selfHost__ or __docHost__ (new) into template files
 *                - maybe all those links should be relative, no hostname??
 *              new department? naming conventions? should use our customers' dept.?
 *              can pew_select form incorporate navigation info to fill in blank boxes?
 *              how about timing out of the PEW call -- in case some problem over there
 *              search for 'later' in comments below for basic requirements yet to do
 *              make the input stream (xbuf) arbitrary size? BufferedInputStream??
 *              what about logging the interactions, for live debugging purposes (don't trust
 *                their PEW servlet completely)
 * END HISTORY
 */

/**********************************************************************
 * MyException is a bare-bones exception handler for minor internal PEWLookup.fetch errors.
 * @param a message String.
 **********************************************************************/
class MyException extends Exception {
    public MyException(String s) { super(s); }
}

/**********************************************************************
 * The main work of the PEWLookup.fetch servlet is done by the servletService method.
 * The init method of the ServletBase class will read fetch.properties from the ClassPath.
 * We just need to tell it here what the name of the properties file is.
 * @author Mike Rychener
 * @version 1.0
 **********************************************************************/
public class fetch extends ServletBase
{ 
    private final String propertyFilename = "fetch" ;
    protected String getPropertyFileName() {
        return ( propertyFilename ) ;
    }

    private String formatCalDate(int yr, int mo, int dt) {
        // this simple convenience method turns 3 int's into a date string
        return (
            Integer.toString(yr) + ((mo < 10) ? "-0" : "-") + Integer.toString(mo)
            + ((dt < 10) ? "-0" : "-") + Integer.toString(dt)
        );
    }

    private String subSplice(String str, String sub, String snu) {
        // take a given sequence sub out of a string str and splice the remainders together,
        // inserting a third string snu where sub used to be (only if it was there)
        if (str.indexOf(sub) < 0) return str;
        if (str.indexOf(sub) == 0) return (snu + str.substring(sub.length()));
        return ( str.substring(0, str.indexOf(sub)) + snu
                 + str.substring(str.indexOf(sub) + sub.length()) );
    }

/**********************************************************************
 * servletService takes in CGI parameters, calls the PEW servlet and parses
 * XML into HTML for printing to the user.  It reads a template file based on
 * which site (from PathInfo) and location information in fetch.properties.
 * @param request -- the standard servlet request object.
 * @param response -- the standard servlet response object.
 * @param (Get/Post) CGI value mtmdl is the machine-type model string, eg, 954971F.  Only
 * the first 4 characters are used.  The parameter of the same name in eSupport
 * is acceptable here.
 * @param (Get/Post) CGI value country is a 3-digit code.  See the HTML select menu
 * in pew_select_white.html for details.  The length must be 3.
 * @param (Get/Post) CGI value serial is a 7-character string.  Length is not enforced.
 * @param (Get/Post) CGI tag PathInfo (via request.getPathInfo) is either 'it', 'es'
 * or 'bp' according to which site is invoking the servlet (this tag is
 * placed by static html or TCL on the site).  Pathinfo is 'te' for personal test site.
 **********************************************************************/   
    public void servletService (
	HttpServletRequest	request,
	HttpServletResponse	response
        ) throws ServletException, IOException, MalformedURLException
    {
        InputStreamReader       instr = null;
        BufferedReader          tbuf = null;
        int                     cbytes = 0;
        int                     sel1 = 0;
        int                     sel2 = 0;
        char                    xbuf[];
        String                  partype = "";
        String                  parcountry = "";
        String                  parserial = "";

        // loadable resources:
        String WhiteHead[] = new String[100];  // initial html text for head tag, meta, title
        String WhiteStart[] = new String[100];  // initial html text for WhiteSpace
        String Separator = "";
        String RowColor2 = "";  // strings that define size=2 headers, etc.
        String RowFont2 = "";
        String RowFont22 = "";
        String ColEnd2 = "";
        String RowEnd2 = "";        
        String WarrInfoStart[] = new String[100];  // further sections of html content
        String WarrDescStart[] = new String[100];
        String FRUInfoStart[] = new String[100];
        String ServiceLink = "";
        String WhiteEnd[] = new String[100];
        String templateTag = "";

        WhiteHead[0] = "";
        WhiteStart[0] = "";
        WarrInfoStart[0] = "";
        WarrDescStart[0] = "";
        FRUInfoStart[0] = "";
        WhiteEnd[0] = "";
        
        xbuf = new char[12288];  // this covers machine (max ~1400) + 50 parts @ 200 char's
        writeLog("fetch starting");
        String insite = new String(request.getPathInfo().toLowerCase());
        if (insite == null || insite.length() == 0) insite = "/it";
        response.setContentType("text/html");
        // write the data of the response -- PrintWriter wrapped around byte stream buffer
        ByteArrayOutputStream bytes = new ByteArrayOutputStream(1024);
	PrintWriter out = new PrintWriter(bytes, true);  // force flushing
        if (insite.indexOf("es") >= 0) templateTag = "_es";  // tag for eSupport
        else if (insite.indexOf("bp") >= 0) templateTag = "_bp";  // tag for Business Partner
        // tag will be empty if it's "it" (infotips) or "te" or whatever

        try {  // template reader
            FileReader templt = new FileReader(
                properties.getProperty("templatePath")
                + "pew_status_template" + templateTag + ".html");
            tbuf = new BufferedReader(templt);
            int jj = 0;
            WhiteHead[jj++] = tbuf.readLine();
            while (WhiteHead[jj-1].indexOf("__String_WhiteStart__") < 0)
                WhiteHead[jj++] = tbuf.readLine();
            WhiteHead[jj-1] = "";  // erase the separator
            jj = 0;
            WhiteStart[jj++] = tbuf.readLine();
            while (WhiteStart[jj-1].indexOf("__String_RowColor2__") < 0)
                WhiteStart[jj++] = tbuf.readLine();
            WhiteStart[jj-1] = "";  // erase the separator
            RowColor2 = tbuf.readLine();  // assume each of these is a single line
            Separator = tbuf.readLine();
            if (Separator.indexOf("__String_RowFont2__") >= 0) RowFont2 = tbuf.readLine();
            Separator = tbuf.readLine();
            if (Separator.indexOf("__String_RowFont22__") >= 0) RowFont22 = tbuf.readLine();
            Separator = tbuf.readLine();
            if (Separator.indexOf("__String_ColEnd2__") >= 0) ColEnd2 = tbuf.readLine();
            Separator = tbuf.readLine();
            if (Separator.indexOf("__String_RowEnd2__") >= 0) RowEnd2 = tbuf.readLine();
            Separator = tbuf.readLine();
            jj = 0;
            if (Separator.indexOf("__String_WarrInfoStart__") >= 0) {
                WarrInfoStart[jj++] = tbuf.readLine();
                while (WarrInfoStart[jj-1].indexOf("__String_WarrDescStart__") < 0)
                    WarrInfoStart[jj++] = tbuf.readLine();
            }
            WarrInfoStart[jj-1] = "";  // erase the separator
            jj = 0;
            WarrDescStart[jj++] = tbuf.readLine();
            while (WarrDescStart[jj-1].indexOf("__String_FRUInfoStart__") < 0)
                WarrDescStart[jj++] = tbuf.readLine();
            WarrDescStart[jj-1] = "";  // erase the separator
            jj = 0;
            FRUInfoStart[jj++] = tbuf.readLine();
            while (FRUInfoStart[jj-1].indexOf("__String_ServiceLink__") < 0)
                FRUInfoStart[jj++] = tbuf.readLine();
            FRUInfoStart[jj-1] = "";  // erase the separator
            ServiceLink = tbuf.readLine();
            Separator = tbuf.readLine();
            if (Separator.indexOf("__String_WhiteEnd__") >= 0) {
                jj = 0;
                WhiteEnd[jj++] = tbuf.readLine();
                while (WhiteEnd[jj-1].indexOf("__That's_All_Folks__") < 0)
                    WhiteEnd[jj++] = tbuf.readLine();
                WhiteEnd[jj-1] = "";  // if it gets to here, must have read the separator
            }
            // loop may end with an IOException at end of file (or other error)
        }   // end template reader try
        catch (FileNotFoundException f1) {  // dummy output if no template
            out.println("<HTML><BODY> PEWLookup.fetch no template file -- Installation error --"
                        + f1.getMessage() + " </BODY></HTML>");
            writeLog("fetch exception template -- " + parserial
                     + ":" + partype + ":" + parcountry);
            response.setContentLength(bytes.size());
            bytes.writeTo(response.getOutputStream());
            // System.exit(1);  -- NOT: kills servletrunner??
            return;
            }
        catch (IOException e1) {  // if IO error or end of file, just stop loading the arrays
            }
        catch (ArrayIndexOutOfBoundsException e2) {  // just stop if bad array index
            }
        finally { try { tbuf.close(); } catch (Exception e) {} }

        long tmstart = System.currentTimeMillis();
        try {  // parameters & URL connection
            // extract CGI parameters and check their validity
            partype = request.getParameter("mtmdl");
            if (partype == null || partype.length() < 4 )
                throw new MyException("Invalid Machine Type.  Format: XXXX (num) Example: 6282.  ");
            else partype = partype.substring(0,4);
            // check that machine type is valid -- all digits? -- later (see wrapqtx for rules)
            parcountry = request.getParameter("country");
            if (parcountry == null || parcountry.length() < 3 )
                throw new MyException("Select a country or location from the drop-down list.  ");
            else {
                parcountry = parcountry.substring(0,3);
                if (!Character.isDigit(parcountry.charAt(0))
                    || !Character.isDigit(parcountry.charAt(1))
                    || !Character.isDigit(parcountry.charAt(2))) parcountry = "892"; // default US
            }
            // later, may country be 1 or 2 digits in length?  above assumes 3 digits
            parserial =  request.getParameter("serial");
            if (parserial == null || parserial.length() < 6 )
                throw new MyException("Invalid Serial Number.  Format: XXXXXXX (alphanum) Example: 23B0536.  ");
            // later, are there any constraints on serial numbers?  always 7 alphanum's?
            // set up any proxy host specified: OUT
            // if (properties.getProperty("socksProxyHost") != null
            //      && properties.getProperty("socksProxyPort") != null) {
            //    Properties p = System.getProperties();
            //    p.put("socksProxyHost", properties.getProperty("socksProxyHost"));
            //    p.put("socksProxyPort", properties.getProperty("socksProxyPort"));
            //    System.setProperties(p);
            // }
            // construct the URL for my mock PEW servlet: -- DEBUG
            // String partype = "2144";
            // String parserial = "23lclkt";
            // String parcountry = "892";
            String exthost = properties.getProperty("PEWhost");
            String remhost = "";
            if (exthost.indexOf('/') > -1) {
                remhost = exthost.substring(exthost.indexOf('/'));
                exthost = exthost.substring(0, exthost.indexOf('/'));
            }
            URL myurl = new URL("http", exthost,
                        Integer.parseInt(properties.getProperty("PEWport")),
                        remhost + "/servlet/" + properties.getProperty("PEWservletPathName")
                        + "?XML=<warrantyInfo><type>" + partype + "</type>"
                        + "<serial>" + parserial + "</serial>"
                        + "<country>" + parcountry + "</country></warrantyInfo>"
                        );
            writeLog("fetch connecting -- " + parserial + ":" + partype + ":" + parcountry);
            URLConnection conn = myurl.openConnection();
            conn.connect();
            // writeLog("fetch connected -- " + parserial + ":" + partype + ":" + parcountry);
            instr = new InputStreamReader(conn.getInputStream());
            int rcnt = 0;
            while (rcnt >= 0) {  // read until get a -1 return, indicating end of stream
                cbytes += rcnt;
                rcnt = instr.read(xbuf, cbytes, xbuf.length);
                Integer ircnt = new Integer(rcnt);
                // writeLog("fetch read " + ircnt.toString() + " bytes -- " + parserial
                //         + ":" + partype + ":" + parcountry);
            }
        }   // end parameters & URL connection try
        catch (Exception ue) { 
            if (ue.getMessage().equals("null") || ue.getMessage() == null) {
                out.println("<HTML><BODY>"
                            + "<p><font size=4 face=\"Helvetica, Arial\" color=Red>"
                            + "URL Connection, Network, or Unknown Parameter Error.  "
                            + "Please try again.</font></p>"
                            + "<!-- PEWLookup.fetch exception from PEW with null message -->"
                            + "</BODY></HTML>");
            } else {
                out.println("<HTML><BODY>"
                            + "<p><font size=4 face=\"Helvetica, Arial\" color=Blue>" 
                            + "Connection failed.</p>"
                            + "<p>" + ue.getMessage() + "</p>"
                            + "<p>Please try again.</font></p>"
                            + "<!-- PEWLookup.fetch exception from PEW "
                            + "(URL Connection, Network, or Parameter Error) -->"
                            + "</BODY></HTML>");
            }
            writeLog("fetch exception parameter/URL -- " + ue.getMessage() + " -- " + parserial
                     + ":" + partype + ":" + parcountry);
            response.setContentLength(bytes.size());
            bytes.writeTo(response.getOutputStream());
            // System.exit(1);  -- NOT: kills servletrunner
            return;
        }
        finally { try { instr.close(); } catch (Exception e) {} }
        
        long tmend = System.currentTimeMillis();
        writeLog("PEW lookup time: " + (tmend - tmstart) + " msec");
        String xbufstr = new String(xbuf,0,cbytes);  // convert read-in XML to a String

	// write the data of the response
        if (insite.indexOf("es") < 0) {
            // loop thru WhiteHead[] and print it, if not eSupport site
            for (int ii = 0; ii < WhiteHead.length; ii++) 
                if (WhiteHead[ii] != null) out.println(WhiteHead[ii]);
        } else {
            // must suppress printing of WhiteHead[], except 1st leading comment line
            out.println(WhiteHead[0]);
        }

        Integer icbytes = new Integer(cbytes);
        out.println("<!-- XML total length = " + icbytes.toString() + " -->");
        // out.println("DEVELOPMENT VERSION");
        writeLog("fetch received " + icbytes.toString() + " bytes -- " + parserial
                     + ":" + partype + ":" + parcountry);
        out.println("<!-- XML received = |" + xbufstr + "| -->");
        // if (cbytes > 150) writeLog("fetch XML = |" + xbufstr.substring(0,150) + "...");
        // else writeLog("fetch XML = |" + xbufstr + "|");
        Calendar currdate = Calendar.getInstance();
        // currdate.setTime(currdate.getTime());  this is redundant
        currdate.set(currdate.get(Calendar.YEAR), currdate.get(Calendar.MONTH),
             currdate.get(Calendar.DATE), 0, 0, 0);
        for (int ii = 0; ii < WhiteStart.length; ii++) 
            if (WhiteStart[ii] != null) {
                if (WhiteStart[ii].indexOf("__Today__") > -1)
                    out.println(subSplice(WhiteStart[ii], "__Today__",
                                          formatCalDate(currdate.get(Calendar.YEAR),
                                                        currdate.get(Calendar.MONTH)+1,
                                                        currdate.get(Calendar.DATE))
                                // added 1 to month due to Java convention, see below
                                ));
                else out.println(WhiteStart[ii]);
            }

      try {  // all the XML parsing try ...
        // check the validate numbers
        if (xbufstr.indexOf("<validate>") > -1 && xbufstr.indexOf("</validate>") > -1) {
            String valstr = xbufstr.substring(xbufstr.indexOf("<validate>")
                                              + "<validate>".length(),
                                              xbufstr.indexOf("</validate>"));
            int valdat = Integer.parseInt(valstr);
            int recdat = xbufstr.length() - "<validate></validate>".length() - valstr.length();
            Integer irecdat = new Integer(recdat);
            if (valdat != recdat) {
                out.println("PEWLookup.fetch Input error: <validate> mismatch, possible data loss: "
                            + valstr + " expected, not " + irecdat.toString());
            }
        } else {
            out.println("<p><font size=4 face=\"Helvetica, Arial\" COLOR=Red>"
                        + "PEWLookup.fetch Data Validation Tag is missing.  Information in this "
                        + "report is incomplete.</font></p>");
        }
        // write the XML expression received in raw form:
        // out.println("DEBUG: " + xbufstr + "<br>");
        writeLog("fetch parsing -- " + parserial
                 + ":" + partype + ":" + parcountry);
        // String excep = new String("");  // debug
        // partype = excep.substring(3);  // debug -- erroneous usage, causes exception
        String table1[][] = { { "<countryDesc>", "</countryDesc>", "Unknown" },
                              { "<type>", "</type>", "xxxx" },
                              { "<model>", "</model>", "ooo" },
                              { "<serial>", "</serial>", "error" },
                              { "<shipDate>", "</shipDate>", "n/a" }  // must be 4th, see loop below
        };
        // ---------------------------------- Product Info Table ------------------------------
        out.println(RowColor2);
        for (int ii = 0; ii < table1.length; ii++) {  // set up 5 columns of 1st displayed table
            // except no shipDate if on eSupport or Business Partner sites
            if (ii == 4 && (insite.indexOf("es") >= 0 || insite.indexOf("bp") >= 0)) continue;
            sel1 = xbufstr.indexOf(table1[ii][0]);
            sel2 = xbufstr.indexOf(table1[ii][1]);
            if (sel1 < 0 || sel2 < 0) {
                out.println(RowFont2 + table1[ii][2] + ColEnd2);
            } else {
                sel1 += table1[ii][0].length();
                out.println(RowFont2 + xbufstr.substring(sel1,sel2) + ColEnd2);
            }
        }
        out.println(RowEnd2);
        // ---------------------------------- Warranty Info Table ------------------------------
        for (int ii = 0; ii < WarrInfoStart.length; ii++)
            if (WarrInfoStart[ii] != null) out.println(WarrInfoStart[ii]);
        
        String table2[][] = { { "<wed>", "</wed>", "1900/01/01" },
                              { "<mStartDate>", "</mStartDate>", "1900/01/01" },
                              { "<mEndDate>", "</mEndDate>", "1900/01/01" },
                              { "<shipDate>", "</shipDate>", "1900/01/01" }
        };
        int wdates[][] = new int[4][3];  // 4 dates (above) come in as yyyy-mm-dd strings
        String wdate0 = "";

        out.println(RowColor2);
        for (int ii = 0; ii < table2.length; ii++) {  // extract dates from XML
            sel1 = xbufstr.indexOf(table2[ii][0]);
            sel2 = xbufstr.indexOf(table2[ii][1]);
            if (sel1 < 0 || sel2 < 0) {
                wdates[ii][0] = 1900;
                wdates[ii][1] = 1;  // unknown == Jan. 1, 1900
                wdates[ii][2] = 1;
            } else {
                sel1 += table2[ii][0].length();  // numbers in loop correlate with array above
                wdate0 = xbufstr.substring(sel1,sel2);  // & wdate ii's below
                sel1 = 0;
                sel2 = wdate0.indexOf("-");
                wdates[ii][0] = Integer.parseInt(wdate0.substring(sel1,sel2));
                sel1 = sel2 + 1;
                sel2 = wdate0.indexOf("-",sel1);
                wdates[ii][1] = Integer.parseInt(wdate0.substring(sel1,sel2));
                sel1 = sel2 + 1;
                wdates[ii][2] = Integer.parseInt(wdate0.substring(sel1));
            }
        }
        Calendar warxdate = Calendar.getInstance();  // condense numbers into Calendar objects
        int iiwarx = 0;  // NOTE: Java convention means that month values have to be -1
                         //   for consistency with the current time obtained by getInstance()
        warxdate.set(wdates[iiwarx][0], wdates[iiwarx][1] - 1, wdates[iiwarx][2], 0, 0, 0);
        Calendar srvsdate = Calendar.getInstance();
        int iisrvs = 1;
        srvsdate.set(wdates[iisrvs][0], wdates[iisrvs][1] - 1, wdates[iisrvs][2], 0, 0, 0);
        Calendar srvddate = Calendar.getInstance();
        int iisrvd = 2;
        srvddate.set(wdates[iisrvd][0], wdates[iisrvd][1] - 1, wdates[iisrvd][2], 0, 0, 0);
        Calendar shipdate = Calendar.getInstance();
        int iishpd = 3;
        shipdate.set(wdates[iishpd][0], wdates[iishpd][1] - 1, wdates[iishpd][2], 0, 0, 0);
        // currdate is now set above, before Whitestart printed
        int warStatus = 0;
        if (xbufstr.indexOf("<error>") > -1
            && Character.isDigit(xbufstr.charAt(xbufstr.indexOf("<error>") + "<error>".length())))
        {
            // warStatus == 0 for this branch
            if ('1' == xbufstr.charAt(xbufstr.indexOf("<error>") + "<error>".length())) {
                out.println(RowFont2 + "Unknown. Proof of Purchase Required" + ColEnd2);
                out.println(RowFont2 + "N/A" + ColEnd2);
                out.println(RowEnd2);
                for (int ii = 0; ii < WarrDescStart.length; ii++)
                    if (WarrDescStart[ii] != null) out.println(WarrDescStart[ii]);      
                out.println(RowEnd2);
                out.println(RowColor2);
                if (insite.indexOf("it") >= 0)
                    out.println(RowFont22
                                + "Machine Type and Serial Not Found in PEW.  "
                                + "Please verify your input and try your query again.  "
                                + "If your query is still unsuccessful, warranty service "
                                + "and support may still be obtained by providing a copy "
                                + "of the original proof of purchase to IBM.  "
                                + ColEnd2);
                else out.println(RowFont22
                                 + "Machine Type and Serial Not Found in our Database.  "
                                 + "Please verify your input and try your query again.  "
                                 + "If your query is still unsuccessful, warranty service "
                                 + "and support may still be obtained by providing a copy "
                                 + "of the original proof of purchase to IBM.  "
                                 + ColEnd2);
            }  // end error code 1
            else {
                out.println(RowFont2 + "System Error" + ColEnd2);
                out.println(RowFont2 + "N/A" + ColEnd2);
                out.println(RowEnd2);
                for (int ii = 0; ii < WarrDescStart.length; ii++)
                    if (WarrDescStart[ii] != null) out.println(WarrDescStart[ii]);      
                out.println(RowEnd2);
                out.println(RowColor2);
                if (insite.indexOf("it") >= 0)
                    out.println(RowFont22
                                + "PSG Entitlement Warehouse (PEW) is currently unavailable.  "
                                + "Please try again in a few minutes.  "
                                + "If the problem persists, please contact the IGS Help Desk at "
                                + "Tie-Line 526-5335 to open a Problem Report.  "
                                + "Please send the Problem Report number to "
                                + "MISSupport/Raleigh/IBM(IBMUSM35) "
                                + "so that the PEW administrator can follow up on the resolution.  "
                                + ColEnd2);
                else out.println(RowFont22 + "System Unavailable.  "
                                 + "Warranty information cannot be provided.  "
                                 + "Please try again in a few minutes.  "
                                 + ColEnd2);
            }  // end error code 2
        }  // end handling of <error>
        else if (currdate.after(srvsdate) && currdate.before(srvddate) 
                || currdate.equals(srvsdate) || currdate.equals(srvddate)) {
            out.println(RowFont2 + "In Warranty" + ColEnd2);
            warStatus = 1;
            out.println(RowFont2
                        + formatCalDate(wdates[iisrvd][0], wdates[iisrvd][1], wdates[iisrvd][2])
                        + ColEnd2);
            out.println(RowEnd2);
            for (int ii = 0; ii < WarrDescStart.length; ii++)
                if (WarrDescStart[ii] != null) out.println(WarrDescStart[ii]);      
            out.println(RowEnd2);
            out.println(RowColor2);
            sel1 = xbufstr.indexOf("<mSDFDesc>");
            sel2 = xbufstr.indexOf("</mSDFDesc>");
            if (sel1 < 0 || sel2 < 0) {
                out.println(RowFont22 + "N/A" + ColEnd2);
            } else {
                sel1 += "<mSDFDesc>".length();
                out.println(RowFont22
                  + subSplice(xbufstr.substring(sel1,sel2), "local service center", ServiceLink)
                  + ColEnd2);
            }
        }  // end service entitled
        else if (currdate.after(shipdate) && currdate.before(warxdate) ||
                 currdate.equals(shipdate) || currdate.equals(warxdate)) {
            out.println(RowFont2 + "In Warranty" + ColEnd2);
            warStatus = 2;
            out.println(RowFont2
                        + formatCalDate(wdates[iiwarx][0], wdates[iiwarx][1], wdates[iiwarx][2])
                        + ColEnd2);
            out.println(RowEnd2);
            for (int ii = 0; ii < WarrDescStart.length; ii++)
                if (WarrDescStart[ii] != null) out.println(WarrDescStart[ii]);      
            out.println(RowEnd2);
            out.println(RowColor2);
            sel1 = xbufstr.indexOf("<sdfDesc>");
            sel2 = xbufstr.indexOf("</sdfDesc>");
            if (sel1 < 0 || sel2 < 0) {
                out.println(RowFont22 + "N/A" + ColEnd2);
            } else {
                sel1 += "<sdfDesc>".length();
                out.println(RowFont22
                  + subSplice(xbufstr.substring(sel1,sel2), "local service center", ServiceLink)
                  + ColEnd2);
            }
        }  // end warranty entitled
        else {
            out.println(RowFont2 + "Out of Warranty" + ColEnd2);
            warStatus = 3;
            if (srvddate.after(warxdate)) {
                out.println(RowFont2
                         + formatCalDate(wdates[iisrvd][0], wdates[iisrvd][1], wdates[iisrvd][2])
                         + ColEnd2);
            }
            else out.println(RowFont2
                        + formatCalDate(wdates[iiwarx][0], wdates[iiwarx][1], wdates[iiwarx][2])
                        + ColEnd2);
            out.println(RowEnd2);
            for (int ii = 0; ii < WarrDescStart.length; ii++)
                if (WarrDescStart[ii] != null) out.println(WarrDescStart[ii]);      
            out.println(RowEnd2);
            out.println(RowColor2);
            sel1 = xbufstr.indexOf("<sdfDesc>");
            sel2 = xbufstr.indexOf("</sdfDesc>");
            if (sel1 < 0 || sel2 < 0) {
                out.println(RowFont22 + "N/A" + ColEnd2);
            } else {
                sel1 += "<sdfDesc>".length();
                out.println(RowFont22
                  + subSplice(xbufstr.substring(sel1,sel2), "local service center", ServiceLink)
                  + ColEnd2);
            }
        }  // end Out of Warranty
        out.println(RowEnd2);
        // ---------------------------------- FRU Parts Service Table -------------------------
        int partPos = 0;
        int partEnd = 0;
        if (xbufstr.indexOf("<part>", partPos) > 0) {
          for (int ii = 0; ii < FRUInfoStart.length; ii++)
            if (FRUInfoStart[ii] != null) out.println(FRUInfoStart[ii]);
        }
        else if (warStatus > 0) {
            out.println("<tr bgcolor=cccccc><tr><td><font size=2 color=Black>&nbsp;&nbsp;No service "
                        + "information on additional parts was found.</td></tr></table>");
        }
        // construct the HTML form, or fill in template (may be a variable-rows table)
        // NOTE: must pick up both partNumber and fruPart; also fruDesc, serviceFlag
        String table3[][] = { { "<partNumber>", "</partNumber>", "no Part" },  // must be 0th, ...
                              { "<fruPart>", "</fruPart>", "no FRU" },
                              { "<fruDesc>", "</fruDesc>", "N/A" },
                              { "<serviceFlag>", "</serviceFlag>", "?" },
                              { "<barcode>", "</barcode>", "N/A" }  // must be 4th, see loop below
        };
        while (xbufstr.indexOf("<part>", partPos) > 0) {  // iterate over all <part> expr's
            partPos = xbufstr.indexOf("<part>", partPos) + 5;  // skip almost all of <part>
            partEnd = xbufstr.indexOf("</part>", partPos);  // set up a boundary for this part
            if (partEnd < 0) partEnd = xbufstr.indexOf("</output>", partPos);  // error...later
            out.println(RowColor2);
            for (int ii = 0; ii < table3.length; ii++) {  // set up 5 columns of displayed table
                // except no mfg. part number if eSupport
                if (ii == 0 && insite.indexOf("es") >= 0) continue;
                // also except no barcode if eSupport site
                if (ii == 4 && insite.indexOf("es") >= 0) continue;
                sel1 = xbufstr.indexOf(table3[ii][0], partPos);
                sel2 = xbufstr.indexOf(table3[ii][1], partPos);
                if (sel1 < 0 || sel2 < 0 || sel1 > partEnd) {
                    out.println(RowFont2 + table3[ii][2] + ColEnd2);
                } else {
                    sel1 += table3[ii][0].length();
                    out.println(RowFont2 + xbufstr.substring(sel1,sel2) + ColEnd2);
                }
            }
            out.println(RowEnd2);
        } // end while
      }  // end try XML Parsing
      catch ( Exception e ) {
          StringWriter    sw      = new StringWriter ();
          PrintWriter     pw      = new PrintWriter ( sw );

          /* Get the stackTrace */
          e.printStackTrace ( pw );
          out.println("<p><font size=4 face=\"Helvetica, Arial\" color=Red>"
              + "Error interpreting warranty data.  Possible format error or data loss.  "
              + "Please try again.</font></p>"
              + "<!-- " + e.getMessage() + " -->"
              + "<!-- " + pw.toString() + " -->");
          writeLog("fetch exception XML -- " + parserial
                   + ":" + partype + ":" + parcountry);
      }
        
        for (int ii = 0; ii < WhiteEnd.length; ii++)
            if (WhiteEnd[ii] != null) {
                if (WhiteEnd[ii].indexOf("__selfUrl__") > -1) {
                    String selfUrl = new String(request.getQueryString());
                    writeLog("fetch query string:|" + selfUrl + "|" + parserial
                             + ":" + partype + ":" + parcountry);
                    selfUrl = subSplice(selfUrl, "page=root&", "");
                    selfUrl = subSplice(selfUrl, "&page=root", "");  // just in case at end
                    selfUrl = subSplice(selfUrl, "upsave=", "up=");
                    selfUrl = subSplice(selfUrl, "utsave=", "ut=");
                    out.println(subSplice(WhiteEnd[ii], "__selfUrl__", selfUrl));
                }
                else if (WhiteEnd[ii].indexOf("__selfGeo__") > -1 || 
                         WhiteEnd[ii].indexOf("__selfHost__") > -1) {
                    // if __selfGeo__ splice in us, ca, emea, la, ap based on pathinfo tail
                    String selfLine = WhiteEnd[ii];
                    int inbpit = insite.indexOf("bp");
                    if (inbpit >= 0 && insite.length() == inbpit + 2)
                        insite = insite + "us";  // default selfGeo = us
                    if (inbpit >= 0) selfLine = subSplice(selfLine, "__selfGeo__",
                                                          insite.substring(inbpit + 2));
                    else inbpit = insite.indexOf("it");
                    if (properties.getProperty("webHost") != null)
                        selfLine = subSplice(selfLine, "__selfHost__",
                                             properties.getProperty("webHost"));
                    // if __selfHost__ and no webHost property specified, 
                    //   splice in chaos.raleigh.ibm.com:8002 (es) or :9003 (it, bp)
                    else if (inbpit < 0) selfLine = subSplice(selfLine, "__selfHost__",
                                                         "chaos.raleigh.ibm.com:8002");
                    else selfLine = subSplice(selfLine, "__selfHost__",
                                              "chaos.raleigh.ibm.com:9003");
                    out.println(selfLine);
                }
                else out.println(WhiteEnd[ii]);
            }  // end for-loop if (WhiteEnd != null)

        // out.println("<p>Parameters " + partype + ":" + parcountry + ":" + parserial);
        if (insite.indexOf("it") >= 0 || insite.indexOf("bp") >= 0
                || insite.indexOf("te") >= 0) {
            out.println("</BODY></HTML>");  // don't do this for eSupport site
        }
        writeLog("fetch returning normally -- " + parserial
                 + ":" + partype + ":" + parcountry);
        response.setContentLength(bytes.size());
        bytes.writeTo(response.getOutputStream());
    }
}

