
/*
 *                
 * @(#)BrandFamily.java	1.0 04/24/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch;

import java.util.*; 

/**
 * The all brands, families and associations
 *
 * @version 1.0, 07/31/2001 
 * @author  Steve Wang
 */

public class BrandFamily extends Object { 
    /* 
     * This map stores brand and its child families with:
     *     Key: brand CategoryRec
     *    Value: sorted set of family CategoryRec
     */
    private TreeMap brandFamilyMap = new TreeMap(new RecComparator());

    /**
     * Constructor
     */
    public BrandFamily (TreeMap bf) {
        brandFamilyMap = bf; 
    }

    /**
     * Get the brand sorted set 
     */
    public TreeSet getBrandSet() {
        TreeSet brand_set = new TreeSet(new RecComparator());
        Set keys = brandFamilyMap.keySet();
        Iterator keyIt = keys.iterator();
        while (keyIt.hasNext()) {
            // The brand CategoryRec is the key
            CategoryRec key = (CategoryRec)keyIt.next();
            brand_set.add(key); 
        }
        return brand_set; 
    }

    /**
     * Get brand CategoryRec given brandind
     * 
     * @param brandInd The brand indicator
     * @return CategoryRec The brand rec
     */
    public CategoryRec getBrand(int brandInd) {
        // Iterate through each key (CategoryRec of brands) to get the rec
        Set keys  = brandFamilyMap.keySet();
        Iterator keyIt = keys.iterator();
        while (keyIt.hasNext()) {
            CategoryRec key = (CategoryRec)keyIt.next();
            if (key.getInd()==brandInd) {
                return key; 
            }
        }
        return null; 
    }

    /**
     * Get brand CategoryRec given brand name
     * 
     * @param brand The brand name
     * @return CategoryRec The brand rec
     */
    public CategoryRec getBrand(String brand) {
        // Iterate through each key (CategoryRec of brands) to get the rec
        Set keys  = brandFamilyMap.keySet();
        Iterator keyIt = keys.iterator();
        while (keyIt.hasNext()) {
            CategoryRec key = (CategoryRec)keyIt.next();
            if (key.getDescript().equals(brand)) {
                return key; 
            }
        }
        return null; 
    }

    /**
     * Get the sorted family set given brandInd 
     * 
     * @param brandInd The brand indicator
     * @return TreeSet The family sorted set 
     */
    public TreeSet getBrandFamilySet(int brandInd) {
        CategoryRec brand = getBrand(brandInd); 
        return (TreeSet)brandFamilyMap.get(brand); 
    }

    /**
     * Get family categoryRec given brandInd and familyInd
     * 
     * @param brandInd The brand indicator
     * @param familyInd The brand indicator
     * @return CategoryRec The family rec
     */
    public CategoryRec getBrandFamily(int brandInd, int familyInd) {
        TreeSet familySet = getBrandFamilySet(brandInd); 
        if (familySet != null) {
            Iterator it = familySet.iterator();
            while (it.hasNext()) {
                CategoryRec family = (CategoryRec)it.next();
                if (family.getInd()==familyInd) {
                    return family; 
                }
            }
        }
        return null; 
    }

    /** 
     * Get brand rec given a familyInd
     *
     * @param familyInd The family ind
     * @return CategoryRec The brand rec
     */
    public CategoryRec getBrandRec (int familyInd) {
        // Iterate through each key (CategoryRec of brands) to get the rec
        Set keys  = brandFamilyMap.keySet();
        Iterator keyIt = keys.iterator();
        while (keyIt.hasNext()) {
            CategoryRec brand = (CategoryRec)keyIt.next();
            // If the corresponding family rec is found, then the brand is found 
            if (getBrandFamily(brand.getInd(), familyInd) != null) {
                return brand; 
            } 
        }
        return null; 
    }
}

