/*
 *                
 * @(#)Docgen.java	1.0 06/05/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch;

import java.util.*; 
import java.io.*; 
import java.sql.*; 
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.nzna.projects.batch.exception.*; 
import com.ibm.nzna.projects.batch.event.*; 
import com.ibm.nzna.projects.batch.intf.*; 
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.util.*; 
import com.ibm.nzna.projects.common.quest.*; 
import com.ibm.nzna.projects.common.quest.doc.*; 
import com.ibm.appsure.*; 

/**
 * This class defines the main docgen batch class. It initializes with the BatchInit
 * object and loads the supplied plugins. It will register each plugins as its 
 * listerner. The docgen class will run some sql queries to get required eocument and fire
 * BatchEvent to the registered plugin listeners.  
 *
 * @version 1.0, 06/05/2001 
 * @author  Steve Wang
 */

public class Docgen extends Batch implements DatabaseStatusListener, 
                                             DatabaseSystemListener {
    /**
     * Publish Now.   
     */
    private static boolean publishNow = false; 

    /**
     * The Connection  
     */
    private Connection conn = null; 

    /**
     * The database system
     */
    private DatabaseSystem dbs = null; 

    /**
     * docgen app name
     */
    public static String docgen_app_name = "";  

    /**
     * website  - a vector of WebSite objects
     */
    private Vector website = new Vector(); 

    /**
     * Get PublishNow.   
     *
     * @param none.  
     * @return publishNow.
     */
    public static boolean getPublishNow () throws IOException {
        // The publishNow should be set through command line param. If not set,
        // DocGen is assumed starting without publishNow. 
        String pubNow = BatchInit.getDocgenPublishNow();
        if (pubNow!=null && pubNow.compareTo("true")==0) {
            publishNow = true;
        } else {
            publishNow = false; 
        }
        return publishNow; 
    }

    /**
     * Get fullupdate property 
     */
    public boolean getFullUpdate () throws IOException {
        if (BatchInit.getDocgenFullUpdate().compareTo("true")==0) {
           return true;
        } else {
           return false;
        }
    }

    /**
     * Get changedtime  
     */
    public String getChangedTime (Connection conn) throws BatchException, 
                                                          SQLException, 
                                                          IOException {
        if (getFullUpdate()) {
            //Get the time as far as possible: 
            return Constants.FULL_UPDATE_TIMESTAMP; 
        } else {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            //
            //Get the timestamp from tigris.constants table
            //Now we might have one pass for all sites including esupport, bpartner
            //and infotips. Thus we need to get the minimum timestamp of for 
            // the followins:
            //
            // TS_DOCGEN_ESUPPORT, TS_DOCGEN_BPARTNER, and TS_DOCGEN_TECHTIPS
            //
            // We will reach docs as far as the minimum timestamp
            //    
            rs = stmt.executeQuery (
                "SELECT min(strconst) " +
                "FROM tigris.constants " + 
                "WHERE constant = '" + Constants.DOCGEN_PREFIX + "ESUPPORT' " + 
                " OR   constant = '" + Constants.DOCGEN_PREFIX + "IPARTNER' " +
                " OR   constant = '" + Constants.DOCGEN_PREFIX + "INFOTIPS' " +
                "FOR FETCH ONLY"
                ); 
            if (rs.next()) {
                String timestamp = rs.getString(1).trim(); 
                rs.close(); 
                stmt.close(); 
                return timestamp; 
            } else {
                throw new BatchException (
                    "Get last docgen update timestamp error. ", Constants.ERROR, Constants.BATCH_EXCEPTION); 
            }
        }
    }
    
    /** 
     * Start and initialize the bach. 
     *
     * @param none.  
     * @return none.
     */
    public void startDocgenBatch () throws Exception, AppSureException {
        super.startBatch ();

        // Initiate the log files 
        String logdir = BatchInit.getLogdir();
        if (getPublishNow()) {
            docgen_app_name = BatchInit.getAppName(Constants.DocgenPubNowNameKey); 
            BatchInit.logSystem = new BatchLogSystem (logdir+docgen_app_name, this); 
            BatchUtil.logit("LogSystem created for application " + docgen_app_name, Constants.INFORMATIONAL, 0); 
        } else {
            docgen_app_name = BatchInit.getAppName(Constants.DocgenNameKey); 
            BatchInit.logSystem = new BatchLogSystem (logdir+docgen_app_name, this); 
            BatchUtil.logit("LogSystem created for application " + docgen_app_name, Constants.INFORMATIONAL, 0); 
        }
        
        // create database connections
        if (BatchInit.getDb2Driver()) {
            dbs = new DatabaseSystem (Constants.debug, BatchInit.logSystem, true);  /*true for app driver*/
        } else {
            dbs = new DatabaseSystem (Constants.debug, BatchInit.logSystem, false);  /*false for net driver*/
        }

        DatabaseSystem.addStatusListener (this);
        DatabaseSystem.addDatabaseSystemListener (this);
        if (!DatabaseSystem.createConnection(DatabaseSystem.QUEST, 
                                             BatchInit.getDbName(), 
                                             BatchInit.getDbServerName (), 
                                             BatchInit.getDbPort(), 
                                             BatchInit.getDbUserId(), 
                                             BatchInit.getDbUserPasswd())) {
            throw new BatchException (
                   "DocGen terminating. Database connection failed, \n"            
                +  "  Name: " + BatchInit.getDbName()           + "\n"
                +  "  Server: " + BatchInit.getDbServerName()   + "\n"
                +  "  Port: " + BatchInit.getDbPort()           + "\n"
                +  "  User: " + BatchInit.getDbUserId(),
                  Constants.FATAL); 
        }

        BatchUtil.logit("DatabaseSystem connection created: \n" 
                        +  "  Name: " + BatchInit.getDbName()           + "\n"
                        +  "  Server: " + BatchInit.getDbServerName()   + "\n"
                        +  "  Port: " + BatchInit.getDbPort()           + "\n"
                        +  "  User: " + BatchInit.getDbUserId(),
                        Constants.INFORMATIONAL, 0); 
        
        conn = DatabaseSystem.getConnection (DatabaseSystem.QUEST); 
        BatchInit.appSure = new AppSure(); 
        if (!BatchInit.appSure.startApp(conn, docgen_app_name)) {  
            // Only one instance of Docgen is allowed to run. The startApp would fail for
            // multiple instances. 
            BatchUtil.logit( "AppSure startApp() returned false for " + docgen_app_name 
                             + ". Maybe another instance is running.", Constants.WARNING, 0);
            throw new AppSureException("AppSure startApp() docgen failed"); 
        }
        BatchUtil.logit("AppSure started for appcode " + docgen_app_name, Constants.INFORMATIONAL, 0); 

        // Check if docgen or docgenpub now can run right now
        BatchUtil.logit("Checking if "+docgen_app_name+" is allowed to run", Constants.INFORMATIONAL, 0);
        if (!isDocgenOkToRun(BatchInit.appSure)) {
            if (getPublishNow()) {
                BatchUtil.logit("DocGen PublishNow stopped: Daily Docgen or Navgen not finished", Constants.INFORMATIONAL, 0);
            } else {
                BatchUtil.logit("DocGen stopped: Navgen not finished", Constants.WARNING, 0);
            }

            // This will terminate the process at this point
            // return
            stopBatch(0); 
        }

        // Otherwise the batch is allowed to continue
        BatchUtil.logit(docgen_app_name+" is allowed to execute", Constants.INFORMATIONAL, 0);

        // load the tigris.constants table
        BatchInit.tConstants.refreshConstants(BatchInit.logSystem);

        // create website
        String sites = ""; 
        Vector sitevec = BatchInit.getWebSiteNames(); 
        for (int i=0; i<sitevec.size(); i++) {
            String ws = (String)sitevec.elementAt(i);  
            website.add(new WebSite(conn, BatchInit.getSiteType(ws), ws, BatchInit.getLangInd())); 
            sites += ws + " "; 
        }

        if (website.size()==0) {
            BatchUtil.logit("No website created ", Constants.WARNING, 0); 
        } else {
            BatchUtil.logit("Websites created for " + sites, Constants.INFORMATIONAL, 0); 
        }

        // load plugins
        if (initPlugin()==0) { 
            BatchUtil.logit("No loadable plugins found", Constants.WARNING, 0); 
        } 
    }

    /** 
     * DatabaseSystemListener interface implementing
     */
    public void databaseAdded (Database db) {
    }

    /** 
     * DatabaseSystemListener interface implementing
     */
    public void databaseRemoved (Database db) {
    }

    /** 
     * DatabaseSystemListener interface implementing
     */
    public void methodAdded (Database db, SQLMethod m) {
    }

    /** 
     * DatabaseSystemListener interface implementing
     */
    public void methodRemoved (Database db, SQLMethod m) {
    }

    /** 
     * DatabaseStatusListener interface implementing
     */
    public void databaseStatusChange (int statusType, Database db) {
    }

    /**
     * Stop batch
     *
     * @param none.  
     * @return none.
     */
    public void stopBatch (int n) {
        super.stopBatch ();
        //add any app specific shut down info here. 
        if (BatchInit.appSure != null) {
            try {
                BatchUtil.logit("Appsure shutting down ", Constants.INFORMATIONAL, 0); 
                BatchInit.appSure.shutDown(BatchLogSystem.getErrorFile() /*docgen_app_name*/, null); 
            } catch (AppSureException e) {
                BatchUtil.logit("Appsure shut down failed: "+e, Constants.ERROR, Constants.UNABLE_STOP_BATCH); 
                e.printStackTrace(); 
            }
        } else {
            BatchUtil.logit("Appsure is null; Shutdown not called ", Constants.ERROR, 0); 
        }

        // Print error stat for docgen 
        BatchUtil.logit("****************************************************", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("*     " + docgen_app_name + " Error Stat ", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("****************************************************", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Total Warnings: " + BatchInit.batchWarning, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Total Errors:   " + BatchInit.batchError, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Total Fatals:   " + BatchInit.batchFatal, Constants.INFORMATIONAL, 0); 
        
        System.exit(n); 
    }
    
    /**
     * This function initializes the plugin classes. It will add each plugin
     * as a listerner for the Docgen. The plugins will use docinit to obtain 
     * the properties. 
     *
     * @param none.  
     * @return int the number of plugins actually loaded.
     */
    public int initPlugin() throws Exception {
        int n = 0; 
        String plugins = BatchInit.getPlugins();
        BatchUtil.logit ("Loading plugins for " + plugins, Constants.INFORMATIONAL, 0); 
        if (plugins.length()>0) {
            StringTokenizer st = new StringTokenizer(plugins, "|");
            while (st.hasMoreTokens()) {
                String plugin = st.nextToken();
                //
                // We obtain each plugin class name and instantiate each class for 
                // each of the website. We will add each of the 'class - website'
                // instances to the batchListeners
                //
                for (int i=0; i<website.size(); i++) {
                    WebSite wsite = (WebSite)website.elementAt(i); 

                    // Skip load XmlDocgen plugin with none-esupport website
                    if (plugin.indexOf("XmlDocgen")!=-1 && wsite.getSiteName().compareTo(Constants.WEB_SITE_ESUPPORT)!=0) {
                        continue; 
                    }

                    try {
                        // try to load plugins which implements DocgenIntf and batchlistener
                        Object plgIn = Class.forName(plugin).newInstance(); 
                        if ((plgIn instanceof DocgenIntf) && (plgIn instanceof BatchListener)) {
                            ((DocgenIntf)plgIn).init(wsite, conn); 
                            addBatchListener((BatchListener)plgIn);
                            n++; 
                            BatchUtil.logit("  Added plugin " + plugin + " for " + wsite.getSiteName(), Constants.INFORMATIONAL, 0); 
                        } else {
                            BatchUtil.logit("  Plugin " + plugin + " not loaded: not a docgen listener", Constants.INFORMATIONAL, 0); 
                        }
                    } catch (Exception e) {
                        BatchUtil.logit("  Plugin " + plugin + " not loaded: " + e, Constants.WARNING, 0); 
                    }
                }
            }
        } else {
            BatchUtil.logit ("  No Docgen plugin found. Plugins not loaded ", Constants.INFORMATIONAL, 0); 
        }
        return n; 
    }

    /** 
     * This method fires the batchEvent 
     * 
     * @param e A batchEvent object
     * @return none
     */
    public void fireBatchEvent (BatchEvent e) throws BatchException, IOException, SQLException {
        Vector batchListenerVec = this.getBatchListeners(); 
        if (batchListenerVec != null ) {
            Object[] listeners = batchListenerVec.toArray();
            for ( int i = 0; i<listeners.length; i++ ) {
                try {
                    // We might have other types of listeners which is not a docgenIntf
                    // for example, BatchLogSystem is anotehr listener in the array
                    if (listeners[i] instanceof DocgenIntf) {
                        switch (e.getID()) {
                        case BatchEvent.NEW_DOCS_EVENT_ID:
                            ((DocgenIntf)listeners[i]).processDocs((DocsEvent)e);
                            break;
                        case BatchEvent.NEW_DOCS_EVENT_END_ID:
                            ((DocgenIntf)listeners[i]).finishDocs();
                            break;
                        default:
                            break;
                        }
                    }
                } catch (Exception ex) {
                    BatchUtil.logit("  Plugin " + listeners[i].toString() + " failed to process event" 
                                    + e.getID() + ". Exception: " +ex, Constants.ERROR, Constants.SEV2); 
                }
            }
        }
    }

    /**
     * Create a docsEvent
     * 
     * @param conn A db connection
     * @param src The event source
     * @param id The batchEvent id
     * @param docind Docind
     * @param doctypeind Doctypeind
     * @param docclassind Docclassind
     * @param siteind Siteind
     * @return DocsEvent A docsEvent object
     */
    public static DocsEvent createDocsEvent (Connection conn, Object src, int id, int docind, int doctypeind, 
                                      int docclassind, int siteind) throws Exception {
        Document doc = new Document(docind);
        int ret = 0;
        int ret2 = 0; 
        
        // Looks like we need pretty much everything from the document
        try {
            ret = DocumentReader.readDocument(conn, doc); 
            BatchUtil.logit("Read document " + doc.getFilename() + " docind " + docind + " doctype " + doctypeind, Constants.INFORMATIONAL, 0); 
        } catch (Exception ex) {
            // Reading doc failed for this one. Don't process 
            BatchUtil.logit("Reading document " + doc.getFilename() + " docind " + docind + " failed " + ex, Constants.WARNING, 0); 
            return null; 
        }
/*
        //populate the doc table  
        BatchUtil.logit("Reading document base", Constants.DEBUG, 0); 
        ret = DocumentReader.readDocumentBase(conn, doc); 

        // The following is not vital to the program execution 
        try {
            BatchUtil.logit("Reading document query text", Constants.DEBUG, 0); 
            ret = DocumentReader.readDocumentQueryText(conn, doc); 
        } catch (Exception e) {
            BatchUtil.logit("read document querytext failed " + e, Constants.DEBUG, 0); 
        } 

        try {
            BatchUtil.logit("Reading document locale", Constants.DEBUG, 0); 
            ret = DocumentReader.readDocumentLocale(conn, doc); 
        } catch (Exception e) {
            BatchUtil.logit("read document locale failed " + e, Constants.DEBUG, 0); 
        }
        
        //populate the quest doc body
        BatchUtil.logit("Reading document body", Constants.DEBUG, 0); 
        ret = DocumentReader.readDocumentBody(conn, doc); 
        
        //populate the quest doc body
        BatchUtil.logit("Reading document properties", Constants.DEBUG, 0); 
        ret = DocumentReader.readDocumentProperties(conn, doc); 
        
        //populate the graphics attachment of the doc
        BatchUtil.logit("Reading graphics attachment", Constants.DEBUG, 0); 
        ret = DocumentReader.readDocumentGraphics(conn, doc); 
        
        //populate the media attachment of the doc - no longer publish 
        //try {
        //    ret = doc.readDocumentFiles(conn, doc); 
        //} catch (SQLException e) {
        //    ret2 = -1; 
        //    BatchUtil.logit("readDocFiles error: " + e, Constants.ERROR, 0); 
        //}
*/       
        return new DocsEvent (src, id, doc, doctypeind, docclassind, siteind);
    }
     
    /** 
     * This method checks if docgen can proceed 
     *
     * @param appSure AppSure object
     * @return boolean Returns true if no navgen running.  
     */
    public static boolean isDocgenOkToRun (AppSure appSure) throws IOException, AppSureException { 
        String navgen_name   = BatchInit.getAppName(Constants.NavgenNameKey);
        long sleep_max = BatchInit.getAppMaxSleepTime(Constants.DocgenNameKey); 
        long idling = 0; 

        // If it's publishNow, if normal docgen or navgen is running, pubNow should stop here.
        // There is no need to wait because it will try to run in the next 15 min
        if (getPublishNow()) {
            if (appSure.isAppRunning(navgen_name)) {
                BatchUtil.logit("  Navgen running ", Constants.INFORMATIONAL, 0); 
                return false; 
            } else if (appSure.isAppRunning(BatchInit.getAppName(Constants.DocgenNameKey))) {
                BatchUtil.logit("  Docgen running ", Constants.INFORMATIONAL, 0); 
                return false; 
            } else {
                return true; 
            }
        } else {
            // This is normal daily docgen running. If at this time Navgen or docgenPubNow 
            // not finished, it will wait for maximum 1 hour
            while (appSure.isAppRunning(navgen_name) || appSure.isAppRunning(BatchInit.getAppName(Constants.DocgenPubNowNameKey))) {
                if (appSure.isAppRunning(navgen_name)) {
                    BatchUtil.logit("  DocGen waiting for Navgen finishing. Checking back in 30 seconds ...", Constants.INFORMATIONAL, 0); 
                } else {
                    BatchUtil.logit("  DocGen waiting for PublishNow finishing. Checking back in 30 seconds ...", Constants.INFORMATIONAL, 0); 
                }

                if (idling > sleep_max) {
                    // max sleep time exceeded. needs attention
                    BatchUtil.logit("DocGen max waiting " + sleep_max + " exceeded.", Constants.ERROR, Constants.UNABLE_START_BATCH); 
                    return false; 
                }
                //sleep for 1 min at a time 
                try {
                    Thread.sleep(30000); 
                    idling += 30000; 
                } catch (InterruptedException e) { 
                    BatchUtil.logit("DocGen waiting interrupted: " + e, Constants.ERROR, Constants.UNABLE_START_BATCH); 
                    return false; 
                }
            } 
            return true; 
        }
    }

    /**
     * Docgen main function. To get rid of bunch 'docgen.' for calling 
     * the Docgen member functions if the code is inside main(). 
     *
     * @param args Command arguments passed in
     * @return none
     */
    public void docgen_main (String args[]) {
        // Batch starting 
        try {
            // Try to load the property file for docgen. 
            // The command line parameters should look like
            //      java Docgen -database.server:wangtp.raleigh.ibm.com 
            // The option overrides the one in the docgen properties. The 
            // property file is in com/ibm/nzna/properties/batch directory
            String fsep = File.separator; 
            new StandardPropertyLoader(
                "com"+fsep+"ibm"+fsep+"nzna"+fsep+"properties"+fsep+"batch"+fsep+"docgen.properties", args);
             
            // Try to start up
            startDocgenBatch(); 

            int counter = 0; 
            if (getPublishNow()) {
                BatchUtil.logit("PublishNow publishing starts ", Constants.INFORMATIONAL, 0); 
                // Process docgen as publishNow
                Statement stmt = conn.createStatement();
                ResultSet rs;

                //Get all the docs in the tigris.PublishNow table. Save the docind
                //in a vector so that all the docs in that vector will
                //be deleted upon successful publishing. 
                Vector docind_vec = new Vector(); 
                rs = stmt.executeQuery (
                    "SELECT distinct "          +
                           "pnow.docind, "      + 
                           "d.doctypeind, "     +
                           "d.docclassind, "    +
                           "d.changedtime, "    +
                           "d.webable, "        +
                           //"d.show, "           +  //don't check for show
                           "d.publish "         +
                    "FROM tigris.documents d, tigris.publishnow pnow " + 
                    "WHERE d.docind = pnow.docind " + 
                    "FOR FETCH ONLY"
                    ); 
                while (rs.next()) {
                    DocsEvent e = createDocsEvent(conn, 
                                                  this, 
                                                  BatchEvent.NEW_DOCS_EVENT_ID,
                                                  rs.getInt(1), 
                                                  rs.getInt(2), 
                                                  rs.getInt(3), 
                                                  0); // don't need docsiteind infor
                    if (e!=null) {
                        e.setChangedSince(BatchUtil.substitute(rs.getString(4).trim(), " ", "-")); 
                        e.webable = rs.getString(5).equals("Y");
                        //e.show    = (rs.getString(6).compareToIgnoreCase("Y")==0);
                        e.publish = rs.getString(6).equals("Y");
                        fireBatchEvent(e); 
                        docind_vec.add(Integer.toString(rs.getInt(1))); 
                        counter++; 
                    }
                }

                //Now notify the plugins the end of docs
                fireBatchEvent(new DocsEvent(this, BatchEvent.NEW_DOCS_EVENT_END_ID, null ,0,0,0)); 
                BatchUtil.logit("Documents finished ", Constants.INFORMATIONAL, 0); 

                // Upon successful publishing the publishnow table, reset the table 
                // by deleting the entries processed. 
                if (counter>0) {
                    for (int i=0; i<docind_vec.size(); i++) {
                        rs = stmt.executeQuery (
                            "DELETE FROM tigris.publishnow pnow " + 
                            "WHERE pnow.docind = " + (String)docind_vec.elementAt(i)   
                            ); 
                    }
                }
                rs.close(); 
                stmt.close();
            } else {
                BatchUtil.logit("Document publishing starts for documents changed since "+getChangedTime(conn), 
                                Constants.INFORMATIONAL, 0); 
                // Process as normal docgen batch 
                Statement stmt = conn.createStatement();
                ResultSet rs;

                //Get the changed doc 
                rs = stmt.executeQuery (
                    "SELECT distinct "          +
                           "d.docind, "         + 
                           "d.doctypeind, "     +
                           "d.docclassind, "    +
                           "d.changedtime, "    +
                           "d.webable, "        +
                           //"d.show, "           +  //no check for show
                           "d.publish "         +
                    "FROM tigris.documents d " + 
                    "WHERE  " +

                   // We don't need the following because we want to
                   // be able to delete outdated files from the site

                   // "AND d.publish = 'Y' " +
                   // "AND d.webable = 'Y' " +
                   // "AND d.recycled = 'N' " +

                    // We use the minimum of the last docgen update time
                    // of the websites
                    " d.changedtime > '" + getChangedTime(conn) + "' " +
                    "FOR FETCH ONLY"
                    ); 
                while (rs.next()) {
                    DocsEvent e = createDocsEvent(conn, 
                                                  this, 
                                                  BatchEvent.NEW_DOCS_EVENT_ID,
                                                  rs.getInt(1),  // docind
                                                  rs.getInt(2),  // doctypeind 
                                                  rs.getInt(3),  // docclassind 
                                                  0); // no longer need siteind 
                    // The db2 output the time stamp without the middle position dash
                    // Add it here so that it can be compared with other timestamp
                    if (e!=null) {
                        e.setChangedSince(BatchUtil.substitute(rs.getString(4).trim(), " ", "-")); 
                        e.webable = rs.getString(5).equals("Y");
                        //e.show    = (rs.getString(6).compareToIgnoreCase("Y")==0);
                        e.publish = rs.getString(6).equals("Y");
                        fireBatchEvent(e); 
                        counter++;
                    }

                    // Debug only. 
                    if (Constants.debug) {
                        if (counter == 10) {
                            BatchUtil.logit("Published first " + counter + " documents", Constants.DEBUG, 0); 
                            break;
                        }
                    }
                }

                //Now notify the plugins the end of docs
                fireBatchEvent(new DocsEvent(this, BatchEvent.NEW_DOCS_EVENT_END_ID, null, 0,0,0)); 
                BatchUtil.logit("Document publishing finished ", Constants.INFORMATIONAL, 0); 

                rs.close();
                stmt.close();
            }
        } catch (BatchException batchEx) {
            BatchUtil.logit(docgen_app_name + " failed due to Application error. ", batchEx, Constants.FATAL, batchEx.getErrorCode()); 
            batchEx.printStackTrace(); 
            stopBatch(1); 
        } catch (AppSureException e) {
            BatchUtil.logit(docgen_app_name + " failed due to AppSure exception. " + e, Constants.FATAL, Constants.SEV1); 
            e.printStackTrace(); 
            stopBatch(2); 
        } catch (SQLException e) {
            BatchUtil.logit(docgen_app_name + " failed due to sql exception ", e, Constants.FATAL, Constants.SEV1); 
            e.printStackTrace(); 
            stopBatch(3); 
        } catch (Exception e) {
            BatchUtil.logit(docgen_app_name + " failed due to unknown exception ", e, Constants.FATAL, Constants.SEV1); 
            e.printStackTrace(); 
            stopBatch(4); 
        } 

        // Shut down
        stopBatch(0); 
    }

    /** 
     * The main function
     * 
     * @param args Command arguments
     * @return none
     */
    public static void main (String argv[]) {   
        Docgen docgen = new Docgen();
        docgen.docgen_main(argv); 
    }
}

