/*
 *                
 * @(#)Navgen.java	1.0 07/14/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch;

import java.util.*; 
import java.io.*; 
import java.sql.*; 
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.nzna.projects.batch.exception.*; 
import com.ibm.nzna.projects.batch.event.*; 
import com.ibm.nzna.projects.batch.intf.*; 
import com.ibm.nzna.projects.batch.plugin.*; 
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.util.*; 
import com.ibm.nzna.projects.common.quest.*; 
import com.ibm.nzna.projects.common.quest.type.*; 
import com.ibm.nzna.projects.common.quest.doc.*; 
import com.ibm.appsure.*; 

/**
 * This class defines the main navgen batch class. It initializes with the BatchInit
 * object and loads the supplied plugins. It will register each plugins as its 
 * listerner. The navgen class will run some sql queries to get product data and fire
 * BatchEvent to the registered plugin listeners.  
 *
 * @version 1.0, 04/24/2001 
 * @author  Steve Wang
 */

public class Navgen extends Batch implements DatabaseStatusListener, 
                                             DatabaseSystemListener {
    /**
     * The Connection  
     */
    private Connection conn = null; 

    /**
     * The database system
     */
    private DatabaseSystem dbs = null; 

    /** 
     * Navgen app name
     */
    private String navgen_app_name = "";

    /**
     * website  - a vector of WebSite objects
     */
    private Vector website = new Vector(); 

    /**
     * A sorted set with all Brand CategoryRec objects 
     */
    public static TreeSet brandSet           = new TreeSet(new RecComparator());
    
    /**
     * A map with 
     *       Key : BrandKey
     *      Value: Sorted set of child family categoryRec objects 
     */
    public static TreeMap brandChildMap      = new TreeMap();

    /**
     * A map with 
     *       Key : BrandFamilyKey
     *      Value: Sorted set of child machine categoryRec objects 
     */
    public static TreeMap familyChildMap     = new TreeMap();

    /**
     * A map with 
     *       Key : BFMachineKey
     *      Value: Sorted set of child model categoryRec objects 
     */
    public final static TreeMap machineChildMap     = new TreeMap(); 

    /** 
     * A map with 
     *       Key :  machine typeCatInd
     *      Value: Vector of BrandFamilyKey type object
     */
    public static TreeMap machineParentMap    = new TreeMap();

    /** 
     * A map with 
     *       Key :  model typeCatInd
     *      Value: Vector of BFMachineKey type object
     */
    public static TreeMap modelParentMap      = new TreeMap();

   /**
    * A map table with
    *         Key : BFMModelKey
    *        Value: Product Description String 
    */
   public static TreeMap productMap           = new TreeMap();
   
   /**
    * A TreeMap with key types of BrandKey, BrandFamilyKey, BFMachineKey
    * and BFMModelKey with descending levels. The values will be a hDocs 
    * hashtable object. The docsMap has key types of DocTypeCatKey and 
    * value of a vector of docId. 
    * 
    *     TreeMap prodDocsMap 
    *          -- Key: brandKey, brandFamilyKey, BFMachineKey or BFMModelKey
    *          -- Value: hDocs of hashtable object. 
    *                          -- key: DocTypeCatKey 
    *                          -- value: a vector of docIds 
    */
   public static TreeMap prodDocsMap          = new TreeMap(); 

   public final static String SYSMAN_BRAND    = "Systems Management";

   public final static CategoryRec ALL_CATREC = new CategoryRec(0, "All", 1); 

    /** 
     * Start and initialize the bach. 
     *
     * @param none.  
     * @return none.
     */
    public void startNavgenBatch () throws Exception, AppSureException {
        super.startBatch ();

        // Initiate the log files 
        String logdir = BatchInit.getLogdir();
        navgen_app_name = BatchInit.getAppName(Constants.NavgenNameKey); 
        BatchInit.logSystem = new BatchLogSystem (logdir+Constants.NavgenNameKey, this); 

        // create database connections
        dbs = new DatabaseSystem (Constants.debug, BatchInit.logSystem, true); 
        DatabaseSystem.addStatusListener (this);
        DatabaseSystem.addDatabaseSystemListener (this);
        if (!DatabaseSystem.createConnection(DatabaseSystem.QUEST, 
                                             BatchInit.getDbName(), 
                                             BatchInit.getDbServerName (), 
                                             BatchInit.getDbPort(), 
                                             BatchInit.getDbUserId(), 
                                             BatchInit.getDbUserPasswd())) {
            throw new BatchException (
                  "Database connection failed, \n"                  
               +  "  Name: "    + BatchInit.getDbName()        + "\n"    
               +  "  Server: "  + BatchInit.getDbServerName()  + "\n"   
               +  "  Port: "    + BatchInit.getDbPort()        + "\n"   
               +  "  User: "    + BatchInit.getDbUserId(),
                  Constants.FATAL); 
        }

        BatchUtil.logit("DatabaseSystem connection created: \n" 
                        +  "  Name: " + BatchInit.getDbName()           + "\n"
                        +  "  Server: " + BatchInit.getDbServerName()   + "\n"
                        +  "  Port: " + BatchInit.getDbPort()           + "\n"
                        +  "  User: " + BatchInit.getDbUserId(),
                        Constants.INFORMATIONAL, 0); 
        
        conn = DatabaseSystem.getConnection(DatabaseSystem.QUEST); 
        BatchInit.appSure = new AppSure(); 
        if (!BatchInit.appSure.startApp(conn, navgen_app_name)) {  
            // Only one instance of Navgen is allowed to run. The startApp would fail for
            // multiple instances. 
            BatchUtil.logit( "AppSure startApp() returns false for " + navgen_app_name 
                             + ". Maybe another navgen instance is running.", Constants.WARNING, 0);
            throw new AppSureException("AppSure startApp() navgen failed"); 
        }
        BatchUtil.logit("AppSure started for appcode " + navgen_app_name, Constants.INFORMATIONAL, 0); 

        // load the tigris.constants table
        BatchInit.tConstants.refreshConstants(BatchInit.logSystem);

        // create website
        String sites = ""; 
        Vector sitevec = BatchInit.getWebSiteNames(); 
        for (int i=0; i<sitevec.size(); i++) {
            String ws = (String)sitevec.elementAt(i);  
            website.add(new WebSite(conn, BatchInit.getSiteType(ws), ws, BatchInit.getLangInd())); 
            sites += ws + " "; 
        }

        if (website.size()==0) {
            BatchUtil.logit("No website found", Constants.WARNING, 0); 
        } else {
            BatchUtil.logit("The websites are: " + sites, Constants.INFORMATIONAL, 0); 
        }

        // load plugins
        if (initPlugin()==0) { 
            BatchUtil.logit("No loadable plugins found", Constants.WARNING, 0); 
        }
    }

    /** 
     * DatabaseSystemListener interface implementing
     */
    public void databaseAdded (Database db) {
    }

    /** 
     * DatabaseSystemListener interface implementing
     */
    public void databaseRemoved (Database db) {
    }

    /** 
     * DatabaseSystemListener interface implementing
     */
    public void methodAdded (Database db, SQLMethod m) {
    }

    /** 
     * DatabaseSystemListener interface implementing
     */
    public void methodRemoved (Database db, SQLMethod m) {
    }

    /** 
     * DatabaseStatusListener interface implementing
     */
    public void databaseStatusChange (int statusType, Database db) {
    }

    /**
     * Stop batch
     *
     * @param none.  
     * @return none.
     */
    public void stopBatch (int n) {
        super.stopBatch ();
        
        // Print error stat for navgen 
        BatchUtil.logit("****************************************************", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("*     " + navgen_app_name + " Error Stat ", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("****************************************************", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Total Warnings: " + BatchInit.batchWarning, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Total Errors:   " + BatchInit.batchError, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Total Fatals:   " + BatchInit.batchFatal, Constants.INFORMATIONAL, 0); 
        
        // Shut down appsure 
        if (BatchInit.appSure != null) {
            try {
                BatchInit.appSure.shutDown(BatchLogSystem.getErrorFile(), null); 
            } catch (AppSureException e) {
                BatchUtil.logit("Appsure shut down failed: "+e, Constants.ERROR, 0); 
                e.printStackTrace(); 
            }
        }

        System.exit(n); 
    }
     
    /**
     * This function initializes the plugin classes. It will add each plugin
     * as a listerner for the Navgen. The plugins will use BatchInit to obtain 
     * the properties. 
     *
     * @param none.  
     * @return int the number of plugins actually loaded.
     */
    public int initPlugin() throws Exception {
        int n = 0; 
        String plugins = BatchInit.getPlugins();
        BatchUtil.logit ("Loading plugins for " + plugins, Constants.INFORMATIONAL, 0); 
        if (plugins.length()>0) {
            StringTokenizer st = new StringTokenizer(plugins, "|");
            while (st.hasMoreTokens()) {
                String plugin = st.nextToken();
                //
                // We obtain each plugin class name and instantiate each class for 
                // each of the website. We will add each of the 'class - website'
                // instances to the batchListeners
                //
                for (int i=0; i<website.size(); i++) {
                    WebSite wsite = (WebSite)website.elementAt(i); 
                    try {
                        // try to load plugins which implements NavgenIntf and batchlistener
                        Object plgIn = Class.forName(plugin).newInstance(); 
                        if (plgIn instanceof NavgenIntf && plgIn instanceof BatchListener) {
                            ((NavgenIntf)plgIn).init(wsite, conn); 
                            addBatchListener((BatchListener)plgIn);
                            n++; 
                            BatchUtil.logit("Added plugin: " + plugin + " for " + wsite.getSiteName(), Constants.INFORMATIONAL, 0); 
                        } else {
                            BatchUtil.logit("Plugin " + plugin + " not loaded: not a navgen listener", Constants.INFORMATIONAL, 0); 
                        }
                    } catch (ClassNotFoundException e) {
                        BatchUtil.logit("Plugin " + plugin + " not loaded: " + e, Constants.WARNING, 0); 
                    }
                }
            }
        } else {
            BatchUtil.logit ("  No Navgen plugin found. Plugins not loaded ", Constants.INFORMATIONAL, 0); 
        }
        return n; 
    }


    /**
     * Fire the navgen BatchEvent
     *
     * @param e The batch event object 
     * @return none
     */
    public void fireBatchEvent(BatchEvent e) throws BatchException, IOException, SQLException { 
        Vector batchListenerVec = this.getBatchListeners(); 
        if (batchListenerVec != null ) {
            Object[] listeners = batchListenerVec.toArray();
            // We might have other types of listeners which is not a docgenIntf
            // for example, BatchLogSystem is anotehr listener in the array
            for ( int i = 0; i < listeners.length; i++ ) {
                if (listeners[i] instanceof NavgenIntf) {
                    switch (e.getID()) {
                    case BatchEvent.PRODUCT_EVENT_ID: 
                        ((NavgenIntf)listeners[i]).processProd((ProdEvent)e);
                        break;
                    case BatchEvent.PRODUCT_EVENT_END_ID:
                        ((NavgenIntf)listeners[i]).finishProd();
                        break;
                    case BatchEvent.NEW_DOCS_EVENT_ID:
                        ((NavgenIntf)listeners[i]).processDocs((DocsEvent)e);
                        break;
                    case BatchEvent.NEW_DOCS_EVENT_END_ID:
                        ((NavgenIntf)listeners[i]).finishDocs();
                        break;
                    case BatchEvent.BRAND_FAMILY_ID:
                        ((NavgenIntf)listeners[i]).brandFamilySet(e);
                        break;
                    case BatchEvent.DOC_CATEGORY_ID:
                        ((NavgenIntf)listeners[i]).docCategorySet((DocCategoryEvent)e);
                        break;
                    case BatchEvent.FINISH_PROCESS_ID:
                        ((NavgenIntf)listeners[i]).finishProcess(e);
                        break;
                    case BatchEvent.MK_EVENT_ID:
                        ((NavgenIntf)listeners[i]).processMk((MkEvent)e);
                        break; 
                    default:
                        break;
                    }//end switch
                }//end if navgenintf
            }// end for
        }//end if vec=null
    }
    /**
     * Add product related map for all plugins to use  
     *
     * @param e The ProdEvent object
     * @return none
     */
    public void addProdMaps(ProdEvent e) {  
        BatchUtil.logit("Add product maps ", Constants.DEBUG, 0); 
        // Insert the new product into the product table
        boolean isOption = false; 
        if (e.brand.indexOf("IBM Options")!=-1) {
            isOption = true; 
        }                               

        // Save to the brand sorted set
        brandSet.add(new CategoryRec(e.brandInd, e.brand, e.brandOrder)); 

        // Build the brand key
        BrandKey brandKey = new BrandKey(e.brandInd);

        // Save the child map to the brand key
        addChildMap (new CategoryRec(e.familyInd, e.family, e.familyOrder), brandKey, brandChildMap); 
        
        // Build the family key
        BrandFamilyKey familyKey = new BrandFamilyKey(e.brandInd, e.familyInd);
        machineParentMap.put(new Integer(e.machineInd), familyKey); 

        // Save to the child map. 
        // The 'All' cases below are mainly used in saving the profiledb
        if (!familyChildMap.containsKey(familyKey)) {
            // Add 'All' machine to the the machine set initially
            addChildMap(ALL_CATREC/*new CategoryRec(0, "All", 1)*/, familyKey, familyChildMap);
            String prod_desc = "";
            if (isOption) {
                prod_desc = e.family; 
            } else {
                prod_desc = "All types/models for " + e.family;
            }
            productMap.put(new BFMModelKey(e.brandInd, e.familyInd, "All", "All"), prod_desc); 
            // Create the all model as well
            addChildMap(ALL_CATREC/*new CategoryRec(0, "All", 1)*/, new BFMachineKey(e.brandInd, e.familyInd, "All"), machineChildMap);
        }
        addChildMap (new CategoryRec(e.machineInd, e.machine, e.machineOrder), familyKey, familyChildMap); 
        
        // Build the machine key
        BFMachineKey machineKey = new BFMachineKey(e.brandInd, e.familyInd, e.machine); 
        modelParentMap.put(new Integer(e.modelInd), machineKey); 

        // Save to the child map. 
        if (!machineChildMap.containsKey(machineKey)) {
            String prod_desc = "";
            if (isOption) {
                prod_desc = e.family + "," + e.machine; 
            } else {
                prod_desc = "All models for " + e.family + "," + e.machine;
            }
            productMap.put(new BFMModelKey(e.brandInd, e.familyInd, e.machine, "All"), prod_desc); 
            // Add to the all model as well
            addChildMap(ALL_CATREC/*new CategoryRec(0, "All", 1)*/, machineKey, machineChildMap);
        }
        addChildMap (new CategoryRec(e.modelInd, e.model, e.modelOrder), machineKey, machineChildMap); 

        // Build the model key
        BFMModelKey modelKey = new BFMModelKey(e.brandInd, e.familyInd, e.machine, e.model);
        
        productMap.put(modelKey,    e.getProductDescript()); 

        // The vector will hold DocsEvent Object
        // Build the brand key
        prodDocsMap.put(brandKey,   new Vector()/*new Hashtable()*/);

        // Build the family key
        prodDocsMap.put(familyKey,  new Vector()/*new Hashtable()*/);

        // Build the machine key
        prodDocsMap.put(machineKey, new Vector()/*new Hashtable()*/);

        // Build the model key
        prodDocsMap.put(modelKey,   new Vector()/*new Hashtable()*/);

        BatchUtil.logit("Product " + e.getProductInd() + " inserted to all the maps" , Constants.DEBUG, 0); 
    }

    /** 
     * Add to child categoryRec to the parent map
     *
     * @param childRec A child categoryRec object (i.e., a familyRec)
     * @param key The key to the childMap (i.e., the brandChildMap)
     * @param childMap The child map to add to 
     * @return none
     */
    private void addChildMap (CategoryRec childRec, Object key, TreeMap childMap) {
        if (childMap.containsKey(key)) {
            TreeSet childSet = (TreeSet)childMap.get(key); 
            childSet.add(childRec); 
        } else {
            TreeSet descSet = new TreeSet(new RecComparator());
            descSet.add(childRec);
            childMap.put(key, descSet); 
        }
    }

    /** 
     * Add sysman brand data
     */
    private void addSysMan (int brandInd, String brand, TreeSet familySet) {
        // If the brand is Systems Management, we need to insert it into the
        // various product tables because they don't show up in the readProd 
        // query 
        prodDocsMap.put(new BrandKey(brandInd), new Vector());

        // Add the family keys as well
        Iterator it = familySet.iterator(); 
        while (it.hasNext()) {
            prodDocsMap.put(new BrandFamilyKey(brandInd, ((CategoryRec)it.next()).getInd()), new Vector());  
        }
    }

    /**
     * Retrieves the marketing catalog entries
     */
    public int readMkCatalog () throws SQLException, BatchException, IOException {
        BatchUtil.logit("Begin readMkCatalog ", Constants.INFORMATIONAL, 0); 
        if (Constants.debug) {
            return 0; 
        }
        Statement stmt = conn.createStatement();
        ResultSet rs = stmt.executeQuery (
            "SELECT gkey, gval " + 
            " FROM navgen.gdbm "          +  
            " WHERE siteind = 10 "        + //marketing catalog 
            " FOR FETCH ONLY " 
            ); 
        int counter = 0; 
        while (rs.next()) {
            fireBatchEvent(new MkEvent(this, BatchEvent.MK_EVENT_ID, rs.getString(1).trim(), rs.getString(2).trim())); 
            counter++; 
        }
        BatchUtil.logit(" read " + counter + " market catalog key&vals", Constants.INFORMATIONAL, 0); 
        rs.close();
        stmt.close(); 
        return counter; 
    }

    /**
     * This function queries the typecategory table to obtain all the brands 
     * and families. It builds the BrandFamily objects and a BrandFamily event 
     * containing the all brands families info will be fired. 
     *
     * @param none
     * @return int Number of brands
     */
    public int readAllBrandFamily () throws SQLException, BatchException, IOException {
        BatchUtil.logit("Begin readAllBrandFamily ", Constants.INFORMATIONAL, 0); 
        Statement stmt = conn.createStatement();
        ResultSet rs = stmt.executeQuery (
            "SELECT distinct "  + 
            " b.typecatind,"    +  //brand
            " b.description,"   + 
            " b.ordernum,"      + 
            " f.typecatind,"    +  //family
            " f.description,"   +
            " f.ordernum "      +
            "FROM product.typecategory b, " + 
            "     product.typecategory f  " + 
            "WHERE b.categorylevel = '" + TypeCategory.LEVEL_BRAND + "' " +
            "AND   f.parentcatind  = b.typecatind "  +
            "AND   f.categorylevel = '" + TypeCategory.LEVEL_FAMILY + "' " +
            "ORDER BY 1, 4 " + 
            "FOR FETCH ONLY"
            ); 
        
        // The brandFamilies stores a sorted family set corresponding to 
        // each of the sorted brand categoryRec key 
        TreeMap brandFamilyMap = new TreeMap(new RecComparator());
        TreeSet familySet      = new TreeSet(new RecComparator());

        // Iterate through the rows  
        String brand   = "";
        int brandOrder = 0; 
        int prevBrand  = 0;
        int prevBrandOrder  = 0; 
        String prevBrandStr = ""; 
        while (rs.next()) {
            int brandInd  = rs.getInt(1);
            brand         = rs.getString(2).trim();
            brandOrder    = rs.getInt(3); 
            CategoryRec familyRec = new CategoryRec(rs.getInt(4)/*familyInd*/, rs.getString(5).trim(), rs.getInt(6));
            if (prevBrand != brandInd) {
                // A new brand row
                if (familySet.size()>0) {
                    if (Constants.debug) {
                        BatchUtil.logit("Brandind =  " + prevBrand + " brand= "+prevBrandStr, Constants.DEBUG, 0); 
                        BatchUtil.logit("familySet=  " + BatchUtil.getLists(familySet, "{", "}", " "), Constants.DEBUG, 0); 
                    }
                    // Add the child familySet for this new brand.
                    brandFamilyMap.put(new CategoryRec(prevBrand, prevBrandStr, prevBrandOrder),familySet);
                    
                    // If the brand is Systems Management, we need to insert it into the
                    // various product tables because they don't show up in the readProd 
                    // query
                    if (prevBrandStr.equals(SYSMAN_BRAND)) {
                        addSysMan (prevBrand, prevBrandStr, familySet); 
                    }

                    // Last update the familySet for new ones
                    familySet = new TreeSet(new RecComparator()); 
                    familySet.add(familyRec); 
                } else {
                    familySet.add(familyRec); 
                }

                // Reset the previous brandind
                prevBrand      = brandInd; 
                prevBrandStr   = brand; 
                prevBrandOrder = brandOrder; 
            } else {
                // In the same brand group
                familySet.add(familyRec); 
            }
        }
        rs.close();
        stmt.close(); 

        if (Constants.debug) {
            BatchUtil.logit("Brandind =  " + prevBrand + " brand= "+prevBrandStr, Constants.DEBUG, 0); 
            BatchUtil.logit("familySet=  " + BatchUtil.getLists(familySet, "{", "}", " "), Constants.DEBUG, 0); 
        }
        // Add the last group of the familySet to brand 
        if (familySet.size()>0) {
            brandFamilyMap.put(new CategoryRec(prevBrand, brand, brandOrder),familySet);
            if (brand.equals(SYSMAN_BRAND)) {
                addSysMan (prevBrand, brand, familySet);
            }
        }
        
        // Fire the brandFamilyEvent
        BrandFamilyEvent e = new BrandFamilyEvent (
                                     this, 
                                     BatchEvent.BRAND_FAMILY_ID,
                                     new BrandFamily(brandFamilyMap)
                                     );
        fireBatchEvent(e); 
        return brandFamilyMap.size(); 
    }

    /**
     * This functions gets all doccategories (200+) in a sorted set of 
     * CategoryRec object for each of the brand. The key is brandKey.
     *
     * @param none
     * @return int Number of doccategories
     */
    public int readAllDocCategories () throws SQLException, BatchException, IOException {
        BatchUtil.logit("Begin readAllDocCategories ", Constants.INFORMATIONAL, 0); 
        Vector docCategory = new Vector();
        Statement stmt = conn.createStatement();
        ResultSet rs = stmt.executeQuery (
            "SELECT distinct "  + 
            " b.typecatind,"    +  //brand
            " b.description,"   + 
            " b.ordernum,"      + 
            " c.typecatind,"    +  //doc category
            " c.description,"   + 
            " c.ordernum "      +
            "FROM product.typecategory b, "          + 
            "     product.typecategory c  "          + 
            "WHERE b.categorylevel = '" + TypeCategory.LEVEL_BRAND + "' " +
            " AND   c.parentcatind  = b.typecatind "  +
            "AND   c.categorylevel = '" + TypeCategory.LEVEL_DOC_CATEGORY + "' " +
            " ORDER BY 1, 4 " + 
            "FOR FETCH ONLY"
            ); 

        // The complete doc category map with brandKey. Each value is a 
        // sorted set of document categories corresponding to the brand
        //TreeMap docCategoryMap = new TreeMap(); 
        //TreeSet docCatSet      = new TreeSet(new RecComparator()); 
        TreeSet docCatSet      = new TreeSet(); 

        // Iterate through the rows  
        int prevBrand  = 0;
        while (rs.next()) {
            docCatSet.add(rs.getString(5).trim()); 
            /*
            int brandInd  = rs.getInt(1);
            CategoryRec docCatRec = new CategoryRec(rs.getInt(4), rs.getString(5).trim(), rs.getInt(6));
            if (prevBrand != brandInd) {
                // A new brand row
                if (docCatSet.size()>0) {
                    if (Constants.debug) {
                        BatchUtil.logit("Brandind =  " + prevBrand, Constants.DEBUG, 0); 
                        //BatchUtil.logit("docCatSet=  " + BatchUtil.getLists(docCatSet, "{", "}", " "), Constants.DEBUG, 0); 
                    }
                    // Add the child doccatSet for this new brand.
                    docCategoryMap.put(new BrandKey(prevBrand),docCatSet);
                    docCatSet = new TreeSet(new RecComparator()); 
                    docCatSet.add(docCatRec); 
                } else {
                    docCatSet.add(docCatRec); 
                }
                prevBrand = brandInd; 
            } else {
                // In the same brand group
                docCatSet.add(docCatRec); 
            }
            */
        }
        rs.close();
        stmt.close();

        /*
        if (Constants.debug) {
            BatchUtil.logit("Brandind =  " + prevBrand, Constants.DEBUG, 0); 
            //BatchUtil.logit("docCatSet=  " + BatchUtil.getLists(docCatSet, "{", "}", " "), Constants.DEBUG, 0); 
        }
        
        // Add the last group of the docCategorySet to brand 
        if (docCatSet.size()>0) {
            docCategoryMap.put(new BrandKey(prevBrand),docCatSet);
        }
        */
        
        // Fire the DocCategoryEvent
        DocCategoryEvent e = new DocCategoryEvent(this, 
                                                  BatchEvent.DOC_CATEGORY_ID,
                                                  docCatSet/*docCategoryMap*/);
        fireBatchEvent(e);
        return docCatSet.size();
    }

    /**
     * This function gets the quest docid, filename and related information. It 
     * executes the sql: 
     * 
     * <br> <br>
     * 
     <font color="#3333FF"> 
     * SELECT d.docind, d.lndocid, d.doctypeind, d.title, d.faq, d.flashdate,   <br> 
     *        d.docclassind, d.changedtime <br> 
     * FROM tigris.documents d      <br>  
     * WHERE d.publish = 'Y'        <br> 
     *       and d.webable = 'Y'    <br> 
     *       and d.recycled = 'N'   <br>  
     *        ORDER BY 1            <br>
     * <br> 
     </font>  
     * An appropriate event will be fired to the listeners in the function to 
     * notify the doc related data. This query will provide data for type 3, 4 
     * and 5 keys defined in Builddb2 plugin. 
     *
     * The docclassind will be used to determine if the document is publishable
     * for a particular web site. The minimum publishable docclassind for a 
     * certain website can be obtained by the following query: 
     * <br> <br>
     <font color="#3333FF"> 
     *  SELECT min(docclassind)     <br>  
     *  FROM docgen.sitedocclass    <br> 
     *  WHERE siteind = ?           <br> 
     *  FOR FETCH ONLY              <br>
     </font> 
     *
     * @param none
     * @return int The number of rows of data 
     */
    public int readDocIdQuery () throws Exception {
        BatchUtil.logit("Begin readDocIdQuery ", Constants.INFORMATIONAL, 0); 
        //
        // Query for all the publishable documents. Get   
        // all the types and categories for any of the publishable document.
        //  <doc, siteind, docTypeDescript, doc_typecategoryRec>
        //
        // To get the release date, we are doing a Union. We use 'publish', which should 
        // always be 'Y' for padding purpose.
        //
        // Add the typecategories which belong to the same doc to a vector typeVec
        int counter = 0; 
        int ret = 0; 
        Statement stmt = conn.createStatement();
        ResultSet rs   = stmt.executeQuery (
            "SELECT distinct d.docind,    d.docclassind,    d.lndocid, tc.typecatind,  tc.description, " + 
            "       tc.ordernum,          tc.categorylevel, d.title,   d.faq,          d.flashnews, " + 
            "       d.doctypeind,         d.changedtime,    d.flashdate " + 
            "FROM tigris.documents d, tigris.category doccat, product.typecategory tc " + 
            "WHERE d.docind = doccat.docind " +           //join to get doc category
            " AND doccat.typecatind = tc.typecatind " +    
            " AND d.publish = 'Y' " +
            " AND d.webable = 'Y' " +
            // "AND d.show = 'Y' " +  //no docind has show 'Y' right now
            " Order by 12 desc, 1 " +                                  
            "FOR FETCH ONLY"
            /*
            "SELECT distinct d.docind,    d.docclassind,    d.lndocid, tc.typecatind,  tc.description, " + 
            "       tc.ordernum,          tc.categorylevel, d.title,   d.faq,          d.flashnews, " + 
            "       d.doctypeind,         d.publish,        d.flashdate " + 
            "FROM tigris.documents d, tigris.category doccat, product.typecategory tc " +
            "WHERE d.docind = doccat.docind " +           //join to get doc category
            " AND doccat.typecatind = tc.typecatind " +    //join to get release date
            " AND d.publish = 'Y' " +
            " AND d.webable = 'Y' " +
            " AND d.docind not in ( " + 
            "           select docind from tigris.docproperties " + 
            "           where propertyind= " + DocumentProperty.RELEASE_DATE  + ") " +  // should be 3 
            // "AND d.show = 'Y' " +  //no docind has show 'Y' right now
            " UNION " +
            "SELECT distinct d.docind,    d.docclassind,    d.lndocid, tc.typecatind,  tc.description, " + 
            "       tc.ordernum,          tc.categorylevel, d.title,   d.faq,          d.flashnews, " + 
            "       d.doctypeind,         dp.value,         d.flashdate " + 
            "FROM tigris.documents d, tigris.category doccat, product.typecategory tc, " +
            "     tigris.docproperties dp " + 
            "WHERE d.docind = doccat.docind " +           //join to get doc category
            " AND doccat.typecatind = tc.typecatind " +    //join to get release date
            " AND dp.docind = d.docind " + 
            " AND d.publish = 'Y' " +
            " AND d.webable = 'Y' " +
            " AND dp.docind = d.docind " + 
            " AND (dp.propertyind = " + DocumentProperty.RELEASE_DATE  +  // 3
            "   or dp.propertyind = " + DocumentProperty.LAST_UPDATE_CTS_TIPS + ") " + //6
            // "AND d.show = 'Y' " +  //no docind has show 'Y' right now
            " Order by 1, 4 " +                                  
            "FOR FETCH ONLY"
            */
            ); 

        int prevDocInd = 0; 
        Vector typeVec      = new Vector(); 
        DocsEvent e         = null;
        DocsEvent prevEvent = null; 
        while (rs.next()) {
            int docInd = rs.getInt(1); 
            CategoryRec rec = new CategoryRec(rs.getInt(4), rs.getString(5).trim(), rs.getInt(6));
            rec.setCategoryLevel(rs.getString(7).trim());
            //
            // We will use the docind and doccatind to create a Document and its
            // TypeCategoryRec object. The two objects and some other parameters
            // will be passed in to the DocsEvent handler.
            //
            if (prevDocInd != docInd) {
                if (e!=null) {
                    prevEvent = e; 
                }

                e = new DocsEvent (this, 
                                   BatchEvent.NEW_DOCS_EVENT_ID, 
                                   rs.getInt(1),      //doc, 
                                   rs.getInt(2),      //docclass
                                   rec
                                   );
                if (rs.getString(3)==null) {
                    BatchUtil.logit("Docind " + docInd + " has no lndocd name. Ignored", Constants.WARNING, 0); 
                    continue; 
                } else {
                    e.lndocId = rs.getString(3).trim();
                }
                e.title   = rs.getString(8).trim();
                e.faq     = (rs.getString(9).compareTo("Y")==0)?  true : false;    // Y or N         
                e.hotNews = (rs.getString(10).compareTo("Y")==0)? true : false;   // Y or N 
                e.doctypeind = rs.getInt(11);   

                // Get the release date
                e.releaseDate = rs.getString(12).trim().substring(0, 10);

                // Get the flash date
                String tmp = rs.getString(13);
                if (tmp==null) {
                    e.flashdate = ""; 
                } else {
                    // get the flashdate in the format of "yyyy-mm-dd"
                    // e.flashdate = BatchUtil.getCompTime(tmp.trim()); 
                    e.flashdate = tmp.trim(); 
                }

                // This is the place to determine if the hot news has expired
                if (e.flashdate.length()>0) {
                    String currTime = BatchUtil.getCurrTime("y-m-d");
                    e.hotNews = e.flashdate.compareTo(currTime)>0;
                } else {
                    e.hotNews = false; 
                }

                counter++; 

                if (typeVec.size()>0) {
                    // The typeVec is filled
                    prevEvent.typevec = typeVec; 
                    fireBatchEvent(prevEvent); 

                    // Save the current type rec into a new vector
                    typeVec = new Vector();
                    typeVec.add(rec); 
                } else {
                    typeVec.add(rec); 
                }
                prevDocInd = docInd; 

                // debug 
                if (Constants.debug) {
                    BatchUtil.logit("Reading doc " + rs.getInt(1) 
                         + " ; Associated level tcInd: " + rs.getInt(4)
                         + " Descript: " + rec.toString() 
                         + " Category Level: " + rec.getCategoryLevel(), Constants.DEBUG, 0); 
                    if (counter == 10) {
                        BatchUtil.logit("Published first " + counter + " documents", Constants.DEBUG, 0); 
                        break;
                    }
                }
            } else {
                // In the same doc group
                typeVec.add(rec); 
            }
        }

        // Fire the last doc
        if (e!=null) {
            e.typevec = typeVec; 
            fireBatchEvent(e); 
        }
        
        rs.close();
        stmt.close(); 

        // Notify plugins the end of doc event
        fireBatchEvent(new DocsEvent (this, BatchEvent.NEW_DOCS_EVENT_END_ID, null, 0, 0, 0)); 
        return counter; 
    }

    /**
     * This query gets the information for every valid product in the product table. 
     * The information can be brand, family, type, model, associated document info etc.
     * The results can be used to contruct some types of key-value for the gdbm
     * data file.  
     *
     * The query starts by join the product table with typecategory table to get 
     * brand, family, machine and model information
     */ 
    public int readProdQuery () throws SQLException, BatchException, IOException {
        BatchUtil.logit("Begin readProdQuery ", Constants.INFORMATIONAL, 0); 
        String debug_target=""; 
        if (Constants.debug) {
            debug_target = " and b.typecatind = 4 "; 
        }

        // Get product and the associated brand, family, machine and model. 
        // First process product with date. We expect the rows will look like:
        //    prod1     date1 (announced)
        //    prod1     date2 (withdrawn) 
        // etc.
        // Union the prod with and without a product date
        // We will save the date into a vector to be used by the plugins
        int prevProdInd = 0; 
        int counter1  = 0; 
        int counter2  = 0; 
        Vector DatesVec = new Vector(); 
        ProdEvent e = null; 
        ProdEvent prevEvent = null; 

        Statement stmt = conn.createStatement(); 
        ResultSet rs = stmt.executeQuery (
            "SELECT DISTINCT " +
                "b.typecatind, b.description, f.typecatind,  f.description, " + 
                "t.typecatind, t.description, m.typecatind,  m.description, " +   
                "p.productind, p.prodnum,     p.description, t.ordernum, "    +
                "m.ordernum,   b.ordernum,    f.ordernum,    pdate.actual, "  + // min date for possible multiple country
                "pdate.typedateind " +
            "FROM   product.product p, product.category pcat, " +  
            "    product.typecategory b, product.typecategory prj, product.typecategory f, " +
            "    product.typecategory t, product.typecategory m, product.dates pdate " +
            "WHERE  p.productind = pcat.productind "  +   // join to get product 
            " AND   pcat.typecatind = m.typecatind  " +       
            " AND   (m.categorylevel = '" + TypeCategory.LEVEL_MODEL + "' " + 
            "     OR m.categorylevel = '" + TypeCategory.LEVEL_PARTNUMBER + "') " + 
            " AND   m.parentcatind    = t.typecatind "    +
            " AND   t.parentcatind    = prj.typecatind "  +
            " AND   prj.parentcatind  = f.typecatind "    +
            " AND   f.parentcatind    = b.typecatind "    +
            " AND   pdate.productind  = p.productind "    +  // join to get the product date
            " AND   p.publish         = 'Y' " + 
            //" AND   p.productind    = d.productind " +   // Get the date in plugins
            //" AND   d.typedateind   = " + TypeList.DATE_ANNOUNCED +  
            
            debug_target + 
            // !!! This line is for debug only
            //" and b.typecatind = 3 " + 
            
            " ORDER BY 1, 3, 6, 8 " + 
            "FOR FETCH ONLY "  
            ); 
        while (rs.next()) {
            int prodInd = rs.getInt(9); 
            ProductDate pdate = null; 
            //BatchUtil.logit("got product ind = " + prodInd, Constants.INFORMATIONAL, 0); 
            String tmpdate = rs.getString(16); 
            if (tmpdate==null) {
                pdate = new ProductDate(rs.getInt(9)/*prodind*/, rs.getInt(17)/*date type*/, "1970/01/31"/*date*/);
            } else {
                pdate = new ProductDate(rs.getInt(9)/*prodind*/, rs.getInt(17)/*date type*/, tmpdate.trim()/*date*/);
            }

            if (prevProdInd != prodInd) {
                if (e!=null) {
                    prevEvent = e; 
                }
                // A new product comes, creating a ProdEvent for the previous one
                //
                // Contruct a prod event 
                //
                e = new ProdEvent (this, 
                                   BatchEvent.PRODUCT_EVENT_ID,
                                   rs.getInt(9),              //productInd 
                                   rs.getString(10).trim(),   //productNum
                                   rs.getString(11).trim()    //product desc
                                   );
                e.brandInd      = rs.getInt(1);              //brandInd 
                e.brand         = rs.getString(2).trim();    //brand 
                e.familyInd     = rs.getInt(3);              //familyInd 
                e.family        = rs.getString(4).trim();    //family 
                e.machineInd    = rs.getInt(5);             //typeInd 
                e.machine       = rs.getString(6).trim();    //type
                e.modelInd      = rs.getInt(7);              //modelInd
                e.model         = rs.getString(8).trim();    //model
                e.machineOrder  = rs.getInt(12);         //machine ordernum
                e.modelOrder    = rs.getInt(13);         //model ordernum
                e.brandOrder    = rs.getInt(14);         //brand ordernum
                e.familyOrder   = rs.getInt(15);         //family ordernum
                counter1++; 

                if (DatesVec.size()>0) {
                    // The DatesVec is filled 
                    prevEvent.datesvec = DatesVec;  

                    addProdMaps(prevEvent); 
                    fireBatchEvent(prevEvent); 

                    // Save the current date into a new vector
                    DatesVec = new Vector();
                    DatesVec.add(pdate); 
                } else {
                    DatesVec.add(pdate); 
                }
                prevProdInd = prodInd; 

                // debug 
                if (Constants.debug) {
                    if (counter1 == 100) {
                        BatchUtil.logit("Published first " + counter1 + " products with dates", Constants.DEBUG, 0); 
                        break;
                    }
                }
            } else {
                // In the same product group
                DatesVec.add(pdate); 
            }
        }

        // Fire the last product 
        if (e!=null) {
            e.datesvec = DatesVec; 
            addProdMaps(e); 
            fireBatchEvent(e); 
        }
        rs.close(); 
        stmt.close(); 

        // Now Process product without announce date 
        BatchUtil.logit("Begin the second part of readProdQuery ", Constants.INFORMATIONAL, 0); 
        stmt = conn.createStatement(); 
        rs = stmt.executeQuery (
            "SELECT DISTINCT " +
                "b.typecatind, b.description, f.typecatind,  f.description, " + 
                "t.typecatind, t.description, m.typecatind,  m.description, " +   
                "p.productind, p.prodnum,     p.description, t.ordernum, "    +
                "m.ordernum,   b.ordernum,    f.ordernum "                    + 
            "FROM   product.product p, product.category pcat, " +  
            "    product.typecategory b, product.typecategory prj, product.typecategory f, " +
            "    product.typecategory t, product.typecategory m " +
            "WHERE  p.productind = pcat.productind "  +   // join to get product 
            " AND   pcat.typecatind = m.typecatind  " +       
            " AND   (m.categorylevel = '" + TypeCategory.LEVEL_MODEL + "' " + 
            "     OR m.categorylevel = '" + TypeCategory.LEVEL_PARTNUMBER + "') " + 
            " AND   m.parentcatind    = t.typecatind "    +
            " AND   t.parentcatind    = prj.typecatind "  +
            " AND   prj.parentcatind  = f.typecatind "    +
            " AND   f.parentcatind    = b.typecatind "    +
            " AND   p.productind not in (select productind from product.dates)"    +  // prod not having date
            " AND   p.publish         = 'Y' " + 
            
            debug_target + 
            // !!! This line is for debug only
            //" and b.typecatind = 3 " + 
            
            " ORDER BY 1, 3, 6, 8 " + 
            "FOR FETCH ONLY "  
            ); 
        while (rs.next()) {
            e = new ProdEvent (this, 
                               BatchEvent.PRODUCT_EVENT_ID,
                               rs.getInt(9),              //productInd 
                               rs.getString(10).trim(),   //productNum
                               rs.getString(11).trim()    //product desc
                               );
            e.brandInd      = rs.getInt(1);              //brandInd 
            e.brand         = rs.getString(2).trim();    //brand 
            e.familyInd     = rs.getInt(3);              //familyInd 
            e.family        = rs.getString(4).trim();    //family 
            e.machineInd    = rs.getInt(5);             //typeInd 
            e.machine       = rs.getString(6).trim();    //type
            e.modelInd      = rs.getInt(7);              //modelInd
            e.model         = rs.getString(8).trim();    //model
            e.machineOrder  = rs.getInt(12);         //machine ordernum
            e.modelOrder    = rs.getInt(13);         //model ordernum
            e.brandOrder    = rs.getInt(14);         //brand ordernum
            e.familyOrder   = rs.getInt(15);         //family ordernum
            e.datesvec      = null; 
            counter2++; 

            addProdMaps(e); 
            fireBatchEvent(e); 
            if (Constants.debug) {
                if (counter2 == 10) {
                    BatchUtil.logit("Published next " + counter2 + " products without dates", Constants.DEBUG, 0); 
                    break;
                }
            }
        }

        rs.close(); 
        stmt.close(); 

        /*
        if (Constants.debug) {
            BatchUtil.logit("Printing brandChildMap: " + BatchUtil.getTreeMapContent(brandChildMap), Constants.DEBUG, 0); 
            BatchUtil.logit("Printing familyChildMap: " + BatchUtil.getTreeMapContent(familyChildMap), Constants.DEBUG, 0); 
            BatchUtil.logit("Printing machineChildMap: " + BatchUtil.getTreeMapContent(machineChildMap), Constants.DEBUG, 0); 
            BatchUtil.logit("Printing machineParentMap: " + BatchUtil.getTreeMapContent(machineParentMap), Constants.DEBUG, 0); 
            BatchUtil.logit("Printing modelParentMap: " + BatchUtil.getTreeMapContent(modelParentMap), Constants.DEBUG, 0); 
            BatchUtil.logit("Printing productMap: " + BatchUtil.getTreeMapContent(productMap), Constants.DEBUG, 0); 
        }
        */
        
        // Notify the plugins the end of product event 
        fireBatchEvent(new ProdEvent (this, BatchEvent.PRODUCT_EVENT_END_ID, 0, "", "")); 
        return counter1 + counter2; 
    }

    /**
     * Navgen main function. To get rid of bunch 'navgen.' for calling 
     * the Navgen member functions if the code is inside main(). 
     *
     * @param args Command arguments passed in
     * @return none    
     */
    public void navgen_main(String args[]) {
        // Batch starting 
        try {
            // Try to load the property file for docgen. 
            // The command line parameters should look like
            //      java Docgen -database.server:wangtp.raleigh.ibm.com 
            // The option overrides the one in the docgen properties. The 
            // property file is in com/ibm/nzna/properties/batch directory
            String fsep = File.separator; 
            new StandardPropertyLoader(
                "com"+fsep+"ibm"+fsep+"nzna"+fsep+"properties"+fsep+"batch"+fsep+"navgen.properties", args);
            startNavgenBatch(); 

            int counter = 0;

            // First read the market catalog
            readMkCatalog(); 

            // Get all brands and families
            readAllBrandFamily(); 

            // Get all the doc categories
            readAllDocCategories();

            // Qeury the products and docs tables and fire appropriate
            // events to the plugins.
            counter = readProdQuery(); 
            BatchUtil.logit("Processed " + counter + " products", Constants.INFORMATIONAL, 0); 

            counter = readDocIdQuery(); 
            BatchUtil.logit("Processed " + counter + " documents", Constants.INFORMATIONAL, 0); 

            //
            //If counter is bigger than 0, we'll ftp the db files and 
            //update the navgen timestamp constant. We are re-using 
            //TS_NAVGEN_ESUPPORT for the combined timestamp.
            //
            if (counter>0) {
                // Fire the finishing event for the plugins to process 
                fireBatchEvent(new BatchEvent(this, BatchEvent.FINISH_PROCESS_ID));
            }

            // Successfully stopped
            stopBatch(0);          
            
            // Call the stopBatch method of each plugin
            Vector batchListenerVec = this.getBatchListeners(); 
            if (batchListenerVec != null) {
                Object[] listeners = batchListenerVec.toArray();
                // We might have other types of listeners which is not a docgenIntf
                // for example, BatchLogSystem is anotehr listener in the array
                for ( int i = 0; i < listeners.length; i++ ) {
                    if (listeners[i] instanceof NavgenIntf) {
                        ((NavgenIntf)listeners[i]).stopNavgen(0);
                    }
                }// end for
            }
        
            BatchUtil.logit("NavGen finished successfully.", Constants.INFORMATIONAL, 0); 
        }  catch (BatchException batchEx) {
            BatchUtil.logit("NavGen failed due to Application error: ", batchEx, Constants.FATAL, 0); 
            batchEx.printStackTrace(); 
            stopBatch(1); 
        } catch (AppSureException e) {
            BatchUtil.logit("NavGen failed due to AppSure exception: " + e, Constants.FATAL, 0); 
            e.printStackTrace(); 
            stopBatch(2); 
        } catch (SQLException e) {
            BatchUtil.logit("NavGen failed due to sql exception: ", e, Constants.FATAL, 0); 
            e.printStackTrace(); 
            stopBatch(3); 
        } catch (Exception e) {
            BatchUtil.logit("NavGen failed due to unknown exception: ", e, Constants.FATAL, 0); 
            e.printStackTrace(); 
            stopBatch(4); 
        } 
    }

    /** 
     * The main function
     * 
     * @param args Command arguments
     * @return none
     */
    public static void main (String args[]) {
        Navgen navgen = new Navgen();
        navgen.navgen_main(args); 
    }
}

