/*
 *                
 * @(#)WebSite.java	1.0 06/05/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch;

import java.sql.*;
import java.util.*; 
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.nzna.projects.batch.exception.*; 

/**
 * docgen.typesite  table 
 * SITEIND DESCRIPT
 * ------- --------------------------------------------------
 *     1 TECHTIPS
 *     2 BPARTNER
 *     3 ESUPPORT
 *     4 INFOTIPS
 *     5 IPARTNER
 * @author  Steve Wang                                     
 */

public class WebSite extends Object {
    /**
     * web site type 
     */
    private int sitetype; 
    
    /**
     * web site name 
     */
    private String sitename; 
    
    /**
     * site language id 
     */
    private int langInd; 

    /**
     * web site classfication 
     */
    private int miniclassification;

    /** 
     * The last docgen update for this site
     */
    private String lastDocgenUpdate; 

    /**
     * image location 
     */
    private String imageLoc; 

    /**
     * Templates. The vector contains CategoryRec objects
     */
    private Vector templates = new Vector(); 

    /**
     * Doctype description. The Vector contains CategoryRec objects  
     */
    private Vector docTypes = new Vector(); 

    /**
     * showcat. The vector contains CategoryRec objects
     */
    private Vector showCat = new Vector(); 

    /**
     * Website constructor
     */
    public WebSite (Connection conn, int sitetype, String sitename, int langid) throws BatchException {
        this.sitetype    = sitetype;  
        this.langInd     = langid; 
        this.sitename    = sitename; 

        // Verify if there is any connection
        if (conn == null) {
            throw new BatchException("No connection(conn=null). Creating Website failed sitename=" + sitename 
                                     +"  sitetype=" + sitetype, Constants.ERROR); 
        }

        // If it's publish now or full update, set to fullupdate for
        // this site
        try {
            if (BatchInit.getDocgenFullUpdate().compareTo("true")==0 || 
                BatchInit.getDocgenPublishNow().compareTo("true")==0) {
                lastDocgenUpdate = Constants.FULL_UPDATE_TIMESTAMP;  
            } else {
                lastDocgenUpdate = readLastDocgenUpdate (conn); 
            }
        } catch (java.io.IOException e) {
            BatchUtil.logit("Get full update failed. Using last update from db for " + sitename, Constants.INFORMATIONAL, 0); 
            lastDocgenUpdate = readLastDocgenUpdate (conn); 
        }

        int rc1          = readDocTemplate(conn, sitetype, langid, templates); 
        int rc2          = readDocTypes(conn, sitetype, docTypes, showCat);
        miniclassification = readMiniSiteClassification(conn, sitetype);
        imageLoc = readImageLocation(conn, sitetype, sitename); 

        // Check if there is any error in creating the website 
        if (lastDocgenUpdate==null || rc1!=0 || rc2!=0 || miniclassification==-1 || 
            imageLoc==null) {
            // Error in creating WebSite object
            throw new BatchException("Creating Website failed sitename=" + sitename 
                                     +"  sitetype=" + sitetype, Constants.ERROR); 
        }
    }  

    /** 
     * Get templates vector
     *
     * @param none
     * @return vector A vector of template CategoryRec
     */
    public Vector getTemplates () {
        return templates; 
    }

    /** 
     * Get docTypes vector
     *
     * @param none
     * @return vector A vector of doctypes CategoryRec
     */
    public Vector getDocTypes () {
        return docTypes; 
    }

    /** 
     * Get showCat vector
     *
     * @param none
     * @return vector A vector of showCat CategoryRec
     */
    public Vector getShowCat () {
        return showCat; 
    }

    /**
     * Given doctype, returns template for the website
     *
     * @param doctype The doctype
     * @return String the html template corresponding to the doctype 
     */  
    public String getTemplateDescript (int doctype) {
        return BatchUtil.findDescript(templates, doctype); 
    }
    
    /**
     * Given doctype, returns docdescription 
     *
     * @param doctype The doctype 
     * @return String The doctype description
     */  
    public String getDoctypeDescription (int doctype) {
        return BatchUtil.findDescript(docTypes, doctype); 
    }

    /**
     * Given doctype, returns showcat 
     *
     * @param doctype The doctype
     * @return String The showcat parameter
     */  
    public String getShowCatDescript (int doctype) {
        return BatchUtil.findDescript(showCat, doctype); 
    }

    /**
     * Get image location
     *
     * @param none
     * @return String The image location for the website
     */
    public String getImageLoc () {
        return imageLoc; 
    }

    /**
     * Get miniclassification
     *
     * @param none
     * @return int The minimum classification for the website
     */
    public int getMiniClassification () {
        return miniclassification; 
    }
    
    /**
     * Get the website name
     * 
     * @param none
     * @return String The website name
     */
    public String getSiteName () {
        return sitename; 
    }

    /**
     * Get the last docgen update time
     *
     * @param none
     * @return String The last docgen update for this site
     */
    public String getLastDocgenUpdate () {
        return lastDocgenUpdate; 
    }

    /**
     * Get the last docgen update for this site
     *
     * @param conn A database connection
     * @return String The last docgen update timestamp for this site
     */
    public String readLastDocgenUpdate (Connection conn) {
        try {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            //
            // TS_DOCGEN_ESUPPORT, TS_DOCGEN_BPARTNER, and TS_DOCGEN_TECHTIPS
            //    
            rs = stmt.executeQuery (
                "SELECT strconst " +
                "FROM tigris.constants " + 
                "WHERE constant = '" + Constants.DOCGEN_PREFIX + sitename + "' " + 
                "FOR FETCH ONLY"
                ); 
            if (rs.next()) {
                String timestamp = rs.getString(1).trim(); 
                rs.close(); 
                stmt.close(); 
                return timestamp; 
            } else {
                //timestamp not found. Assume a fullupdate 
                BatchUtil.logit("Get last docgen update timestamp error. ", Constants.ERROR, 0); 
                return null;
            }
        } catch (SQLException e) {
            BatchUtil.logit("Get last docgen update timestamp error. ", Constants.ERROR, 0); 
            return null;
        } 
    }
    
    /**
     * Get language ind
     *
     * @param none
     * @return int The language ind for the website
     */
    public int getLangInd () {
        return langInd; 
    }
    
    /**
     * Get site type 
     * 
     * @param none
     * @return int The sitetype ind
     */
    public int getSiteType () {
        return sitetype; 
    }
    
    /** 
     * Given a sitetype and langind, returns a vector of templates and doctypeind 
     * 
     * Example:
     * DOCTYPEIND SITEIND LANGUAGEIND
        ---------- ------- -----------
         2       1           1
         3       1           1
         4       1           1
         8       1           1
         9       1           1
        10       1           1
     *
     * @param conn A database connection
     * @param sitetype The site typeind
     * @param langid The language ind
     * @param templates The templates vec
     * @return int None 0 indicating error
     */
    public static int readDocTemplate (Connection conn, 
                                         int sitetype,      //in param
                                         int langid,        //in param
                                         Vector templates   //out param
                                        ) {  //out param
        BatchUtil.logit("Load website template for website " + sitetype, Constants.DEBUG, 0); 
        
        //get header and footer from dochtmlcommon
        String header = "";
        String footer = ""; 
        int rc = 0; 
        try {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            //get the header and footer 
            rs = stmt.executeQuery (
                "SELECT commonind, template " +
                "FROM docgen.dochtmlcommon "  +
                "WHERE siteind = " + Integer.toString(sitetype) + " " +
                "AND languageind = " + Integer.toString(langid) + " " +
                "FOR FETCH ONLY"
                ); 
            while (rs.next()) {
                switch (rs.getInt (1)) {
                case 1: //common: header in docgen.,typecommon
                    header = rs.getString(2).trim(); 
                    BatchUtil.logit("  Header templates loaded", Constants.DEBUG, 0); 
                    break;
                case 2: //common: footer in docgen.typecommon
                    footer = rs.getString(2).trim();
                    BatchUtil.logit("  Footer templates loaded", Constants.DEBUG, 0); 
                    break;
                }
            } 
            rs.close(); 
        } catch (SQLException e) {
            rc = 1; 
            BatchUtil.logit("  Could not get footer or header", Constants.ERROR, 0); 
        } 
        //BatchUtil.logit("Header and footer templates loaded", Constants.DEBUG, 0); 

        try {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            //get the templates 
            rs = stmt.executeQuery (
                "SELECT doctypeind, template " +
                "FROM docgen.dochtmltemp " + 
                "WHERE siteind = " + Integer.toString(sitetype) + " " + 
                "AND   languageind = " + Integer.toString(langid) + " " +
                "FOR FETCH ONLY"
                ); 
            while (rs.next()) {
                int docTypeInd = rs.getInt(1); 
                String src     = rs.getString(2).trim(); 
                
                // Now, replace the ###HEADER### and ###FOOTER### tags with
                // the appropriate "subtemplate."
                src = BatchUtil.substitute(src, "###Common:HEADER###", header );
                src = BatchUtil.substitute(src, "###Common:FOOTER###", footer );
                templates.add(new CategoryRec(docTypeInd, src, 0)); 
                BatchUtil.logit("  added header and footer for docType: " + docTypeInd, Constants.DEBUG, 0); 
            } 
            rs.close(); 
            stmt.close(); 
        } catch (SQLException e) {
            rc = 1; 
            BatchUtil.logit("  Could not get doc template", Constants.ERROR, 0); 
        }
        return rc; 
    }

    /** 
     * Given a sitetype indicator, returns a vector of doctypeind, descript and 
     * showcat  
     * 
     * Example:
     
        DOCTYPEIND DESCRIPT                                           SITEIND
        ---------- -------------------------------------------------- -------
         2 Parts information                                        1
         3 Product information                                      1
         4 Hints and tips                                           1
         8 Service publications                                     1
         9 Downloadable files                                       1
        10 Preloaded software information                           1
     * 
     * @param conn A database connection
     * @param sitetype The site typeind
     * @param docType The doctypeind vec
     * @param docTypeDesc The doctype description vec
     * @param showcat The showcat vec
     * @return int None 0 for error
     */
    public static int readDocTypes (Connection conn, 
                                      int sitetype,         //in param 
                                      Vector docType,       //out param 
                                      Vector showcat) {     //out param
        BatchUtil.logit("Load docTypes for site " + Integer.toString(sitetype), Constants.DEBUG, 0); 
        int rc = 0; 
        try {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            //get the timestamp from tigris.constants table
            rs = stmt.executeQuery (
                "SELECT doctypeind, descript, showcat " + 
                "FROM docgen.sitedoctype " +
                "WHERE siteind = " + sitetype + " " +
                "Order by 2 " + 
                "FOR FETCH ONLY"
                ); 
            while (rs.next()) {
                docType.add(new CategoryRec(rs.getInt(1), rs.getString(2).trim(), 0)); 
                showcat.add(new CategoryRec(rs.getInt(1), rs.getString(3).trim(), 0)); 
                BatchUtil.logit("  added docTypes " + rs.getString(2).trim(), Constants.DEBUG, 0); 
            } 
            rs.close(); 
            stmt.close(); 
        } catch (SQLException e) {
            rc = 1; 
            BatchUtil.logit("  Could not get doctypes ", Constants.ERROR, 0); 
        }
        return rc; 
    }


    /** 
     * Given a sitetype indicator, returns site mini classification 
     * 
     * @param conn A database connection
     * @param sitetype The sitetype ind
     * @return int The minimum classification
     */
    public static int readMiniSiteClassification (Connection conn, int sitetype) { 
        BatchUtil.logit("Load classification for site " + sitetype, Constants.DEBUG, 0); 
        int cf = Constants.UNCLASSIFIED; 
        try {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            //get the timestamp from tigris.constants table
            rs = stmt.executeQuery (
                "SELECT min(docclassind) " +
                "FROM docgen.sitedocclass " + 
                "WHERE siteind = " + sitetype + " " +
                "FOR FETCH ONLY"                
                ); 
            if (rs.next()) {
                cf = rs.getInt (1); 
                BatchUtil.logit("  Classification for the site " + cf, Constants.DEBUG, 0); 
            } else {
                BatchUtil.logit("  No miniclassification found, assuming unclassified ", Constants.DEBUG, 0); 
            }
            rs.close(); 
            stmt.close(); 
        } catch (SQLException e) {
            cf = -1; 
            BatchUtil.logit("  No miniclassification found, assuming unclassified " + e, Constants.DEBUG, 0); 
        }
        return cf; 
    }

    /** 
     * Given a sitetype indicator, returns image loc 
     * 
     * @param conn A database connection
     * @param sitetype The sitetype ind
     * @param siteName The site name
     * @return String The image location
     */
    public static String readImageLocation (Connection conn, int sitetype, String siteName) { 
        BatchUtil.logit("Load image location for site " + sitetype, Constants.DEBUG, 0); 
        String imageLoc = new String();
        try {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            //get the timestamp from tigris.constants table
            rs = stmt.executeQuery (
                "SELECT strconst " + 
                "FROM tigris.constants " + 
                "WHERE constant = 'IMAGELOC_" + siteName + "' " + 
                "FOR FETCH ONLY"            
                ); 
            if (rs.next()) {
                imageLoc = rs.getString(1).trim(); 
                BatchUtil.logit("  Image Location: " + imageLoc, Constants.DEBUG, 0); 
            } else {
                BatchUtil.logit("  Image Location not found ", Constants.WARNING, 0); 
            }
            rs.close(); 
            stmt.close(); 
        } catch (SQLException e) {
            BatchUtil.logit("  Image Location not found " + e, Constants.WARNING, 0); 
            return null; 
        }
        return imageLoc; 
    }

    /**
     * Set the latest doc publish time for this website
     * 
     * @param tStamp The timestamp
     * @return none
     */
    public int updateDocgenTimestamp(Connection conn, String tStamp) {
        BatchUtil.logit("Update docgen timestamp for website: " + sitename, 
              Constants.INFORMATIONAL, 0); 
        int rc = 0; 
        try {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            //get the timestamp from tigris.constants table
            rs = stmt.executeQuery ( 
                "UPDATE tigris.constants SET strconst = " + tStamp + " " +  
                "WHERE  constant = 'TS_DOCGEN_" + sitename + "'"
                ); 
            rs.close(); 
            stmt.close(); 
        } catch (SQLException e) {
            rc = 1; 
            BatchUtil.logit("Update docgen timestamp error for website: " + sitename, Constants.WARNING, 0); 
        }
        return rc; 
    }
}

