/*
 *                
 * @(#)DocsEvent.java	1.0 06/05/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch.event;

import java.util.*; 
import com.ibm.nzna.projects.batch.*;
import com.ibm.nzna.projects.common.quest.type.*; 
import com.ibm.nzna.projects.common.quest.doc.*; 

/**
 * This defines the DocsEvent. This even is fired by Docgen.
 *  
 * @version 1.0, 06/04/2001 
 * @author  Steve Wang
 */

public class DocsEvent extends BatchEvent  {
    /**
     * New document
     */
    private Document doc;

    /**
     * Document ind
     */
    private int docInd;       

    /**
     * doc classid
     */
    private int docclassind;

    /** 
     * changedSince timestamp
     */
    private String changedSince; 
    
    /**
     * site ind
     */
    private int docsiteind;

    /**
     * doctype description
     */
    private String docTypeDescript = new String(); 

    /**
     * doc typecategory rec
     */
    private CategoryRec docCatRec  = new CategoryRec(); 

    /**
     * Document name
     */
    public String lndocId; 

    /**
     * doctype ind
     */
    public int doctypeind;
    
    /**
     * publish indicator
     */
    public boolean publish = true;

    /**
     * show
     */
    public boolean show = true;

    /**
     * webable
     */
    public boolean webable = true; 

    /**
     * title
     */
    public String title = "NULL";

    /** 
     * faq
     */
    public boolean faq = false; 

    /**
     * hot news
     */
    public boolean hotNews = false; 

    /**
     * Release date
     */
    public String releaseDate = ""; 

    /**
     * Flash date
     */
    public String flashdate = ""; 

    /**
     * The vector of category types (contains doccategory, doc brand, etc )
     */
    public Vector typevec = null; 

    /**
     * Constructs a Batch object with the specified source object and type.
     *
     * @param source the object where the event originated           
     * @id the event type
     */
    public DocsEvent(Object source, 
                        int id, 
                        Document d,
                        int typeid,
                        int classind,
                        int siteind
                        ) {
        super(source, id);
        doc = d;
        doctypeind  = typeid;
        docclassind = classind;
        docsiteind  = siteind;
    }
    
    /**
     * Constructs a Batch object with the specified source object and type.
     *
     * @param source the object where the event originated           
     * @id the event type
     * @param siteind Siteind in docgen.sitedoctype table
     * @param dtDescrpt Document type descript (e.g. Downloadable files, Hints Tips, etc)
     * @param dcInd Document typecategory ind
     * @param dcDescrpt Document type descript (e.g. All, Diagnosis, etc) 
     * @param dcLevel Document type category level (e.g. TypeCategory.LEVEL_FAMILY, etc) 
     */
    public DocsEvent(   Object          source, 
                        int             id, 
                        int             docind, //Document        d,
                        int             docclass, 
                        CategoryRec     catRec
                        ) {
        super(source, id);
        docInd          = docind;
        docclassind     = docclass;  
        docCatRec       = catRec; 
    }
    
    /**
     * Returns the document.
     */
    public Document getDoc() {
        return doc;
    }
    
    /**
     * get doctypeind
     */
    public int getDocTypeind() {
        return doctypeind;
    }
    
    /**
     * get docsiteind
     */
    public int getDocSiteind() {
        return docsiteind;
    }
    
    /**
     * get docclassind
     */
    public int getDocClassind() {
        return docclassind;
    }

    /**
     * get docind
     */
    public int getDocind() {
        return docInd;
    }

    /**
     * Get doc typecategory rec
     */
    public CategoryRec getDocCatRec (){
        return docCatRec; 
    }

    /**
     * Get doctype descript
     */
    public String getDocTypeDescript () {
        return docTypeDescript;
    }

    /** 
     * Get changedSince
     */
    public String getChangedSince() {
        return changedSince; 
    }

    /**
     * Set changed since
     */
    public void setChangedSince(String changed) {
        changedSince = changed;
    }

    public int getDocumentType() {
        return doctypeind;
    }

    public String getFilename() {
        return lndocId; 
    }

    public boolean getFaq(){
        return faq;
    }

    public boolean getHotNews() {
        return hotNews; 
    }

    public String getReleaseDate() {
        return releaseDate; 
    }

    public String getTitle() {
        return title; 
    }

    /**
     * To string method
     */
    public String toString() {
        return docInd+"_"+lndocId+"_"+docCatRec.toString(); 
    }
}
