
/*
 *                
 * @(#)NavgenIntf.java	1.0 04/24/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch.intf;

import java.sql.*;
import java.util.*; 
import java.io.*;
import com.ibm.nzna.projects.batch.event.*;
import com.ibm.nzna.projects.batch.exception.*; 
import com.ibm.nzna.projects.batch.*;  

/**
 * The Navgen interface which should be implemented by all
 * navgen listeners 
 * 
 * @version 1.0, 04/24/2001 
 * @author  Steve Wang
 */

public interface NavgenIntf {
    /**
     * The null separator. Since we can not save this into db2, we 
     * will use \t to replace it and will change it back to \0
     * in the extracting program 
     */
    public final static String NULLSEP = "\\0"; 

    /**
     * The pipe separator
     */
    public final static char PIPESEP = '|';

    /** 
     * The machine type
     */
    public final static char MACHINE_TYPE = 'M';

    /**
     * The Option type
     */
    public final static char OPTION_TYPE = 'O'; 

    /**
     * Init function
     */
    public void init (WebSite site, Connection conn) throws BatchException; 

    /**
     * Interface function to force the implementer to overwrite. 
     * BrandFamilyEvent handler.  
     */
    public void brandFamilySet(BatchEvent e) throws BatchException; 

    /**
     * DocCategoryEvent handler.  
     */
    public void docCategorySet(DocCategoryEvent e) throws BatchException; 

    /**
     * DocIdEvent handler.  
     */
    public void processDocs(DocsEvent e) throws BatchException, IOException, SQLException ; 

    /**
     * Finish docs
     */
    public void finishDocs() throws BatchException, IOException, SQLException ; 

    /**
     * ProdEvnet handler.  
     */
    public void processProd(ProdEvent e) throws BatchException; 
    
    /**
     * Finish prod
     */
    public void finishProd() throws BatchException; 

    /**
     * End of query process
     */
    public void finishProcess(BatchEvent e) throws BatchException, IOException;  

    /**
     * Market catalog event
     */
    public void processMk(MkEvent e) throws BatchException;

    /**
     * Write gdbm function
     */
   // public void writeGdbm(Connection conn, String gkey, String gval, int website) throws WriteGdbmException;

    /**
     * batch stop
     * 
     * @param n The stop condiction
     */
    public void stopNavgen(int n) throws Exception; 
}
