/*
 *                
 * @(#)BuildDB2.java	1.0 07/24/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch.plugin;

import java.util.*; 
import java.io.*; 
import java.sql.*; 
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.projects.batch.*;
import com.ibm.nzna.projects.batch.event.*;
import com.ibm.nzna.projects.batch.intf.*; 
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.nzna.projects.batch.exception.*; 
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.util.*; 
import com.ibm.nzna.shared.ftp.FTP.*; 
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.quest.doc.*; 

/**
 * The BuildDB2 plugin main class. The class will save the following key and value
 * to the gdbm file: <br>
 * 1. <br>
 *   keyStr = machType-model <br> 
 *   contentStr = "realpage" + '\0' + brand + '|' + family + '|' + machType + '|' + model
            + '\0' + "family" + '\0' + family + '\0'; <br> 
 * <br>
 * 2. <br> 
 *  keyStr = machType; <br>
 *  contentStr = "realpage" + '\0' + brand + '|' + family + '|' + machType + '\0'; <br> 
 * <br>
 * 3. <br> 
 *  keyStr = fileName <br> 
 *  contentStr = "title" + '\0' + docTitle  <br> 
 *        + '\0' + "date" + '\0' + docTime.getDate().asString("%Y-%m-%d")  <br> 
 *        + '\0' + "faq" + '\0' + ( faq ? YES : NO )    <br> 
 *        + '\0' + "news" + '\0' + ( news ? YES : NO )  <br> 
 *        + '\0';   <br> 
 * <br>
 * 4. <br> 
 *  keyStr = docID <br> 
 *  contentStr = "realpage" + '\0' + fileName + '\0'; <br> 
 * <br>
 * 5. <br> 
 *  keyStr = docID.change("-", "") <br> 
 *  contentStr = "realpage" + '\0' + fileName + '\0';  <br> 
 * <br>
 * 6. <br> 
 *  keyStr = "root"; <br> 
 *  contentStr = "title" + '\0' + "Welcome"                 <br> 
 *        + '\0' + "template" + '\0' + "welcome"            <br> 
 *        + '\0' + "child_prompt" + '\0' + "brand"          <br> 
 *        + '\0' + "look" + '\0' + "welcome"                <br> 
 *        + '\0' + "children" + '\0' + rootChildren         <br> 
 *        + '\0';                                           <br> 
 * <br>
 * 7. <br> 
 *  keyStr = brand + '|' + family + '|' + machType + '|' + model    <br> 
 *  contentStr = "title" + '\0' + family                            <br> 
 *              + '\0' + "subtitle" + '\0' + description            <br> 
 *              + '\0' + "mtmdl" + '\0' + machType + "-" + model    <br> 
 *              + '\0' + "template" + '\0' + "techinfo"             <br> 
 *              + '\0' + "look" + '\0' + look                       <br>
 *              + '\0' + "brand" + '\0' + brand                     <br> 
 *              + '\0' + "mktg_suffix" + '\0' + techtipsHack(brand) <br> 
 *              + '\0' ;                                            <br> 
 *              + "child_prompt" + '\0' + childPrompt               <br> 
 *              + '\0' + "children" + '\0' + children               <br> 
 *              + "docs." + curr + '\0' + '{' + docs.getDocsForDocType(curr, LISTSEP) + '}' + '\0'; <br>
 *              + "doc_types" + '\0' + '{' + docTypeStr + '}'       <br>
 *              + '\0';                                             <br> 
 * <br>
 * 8. <br> 
 *  keyStr = brand                                                  <br> 
 *  contentStr = "title" + '\0' + brand                             <br> 
 *              + '\0' + "template" + '\0' + "techinfo"             <br> 
 *              + '\0' + "look" + '\0' + look                       <br>
 *              + '\0' + "brand" + '\0' + brand                     <br> 
 *              + '\0' + "mktg_suffix" + '\0' + techtipsHack(brand) <br> 
 *              + '\0' ;                                            <br> 
 *              + "child_prompt" + '\0' + childPrompt               <br> 
 *              + '\0' + "children" + '\0' + children               <br> 
 *              + "docs." + curr + '\0' + '{' + docs.getDocsForDocType(curr, LISTSEP) + '}' + '\0'; <br>
 *              + "doc_types" + '\0' + '{' + docTypeStr + '}'       <br>
 *              + '\0';                                             <br> 
 * <br>
 * 9. <br> 
 *  keyStr = brand + '|' + family                                   <br> 
 *  contentStr = "title" + '\0' + brand                             <br> 
 *              + '\0' + "subtitle" + '\0' + family                 <br> 
 *              + '\0' + "template" + '\0' + "techinfo"             <br> 
 *              + '\0' + "look" + '\0' + look                       <br>
 *              + '\0' + "brand" + '\0' + brand                     <br> 
 *              + '\0' + "mktg_suffix" + '\0' + techtipsHack(brand) <br> 
 *              + '\0' ;                                            <br> 
 *              + "child_prompt" + '\0' + childPrompt               <br> 
 *              + '\0' + "children" + '\0' + children               <br> 
 *              + "docs." + curr + '\0' + '{' + docs.getDocsForDocType(curr, LISTSEP) + '}' + '\0'; <br>
 *              + "doc_types" + '\0' + '{' + docTypeStr + '}'       <br>
 *              + '\0';                                             <br> 
 * <br>
 * 10. <br> 
 *  keyStr = brand + '|' + family + '|' + machType                  <br> 
 *  contentStr = "title" + '\0' + family                            <br> 
 *              + '\0' + "subtitle" + '\0' + machType               <br> 
 *              + '\0' + "template" + '\0' + "techinfo"             <br> 
 *              + '\0' + "look" + '\0' + look                       <br>
 *              + '\0' + "brand" + '\0' + brand                     <br> 
 *              + '\0' + "mktg_suffix" + '\0' + techtipsHack(brand) <br> 
 *              + '\0' ;                                            <br> 
 *              + "child_prompt" + '\0' + childPrompt               <br> 
 *              + '\0' + "children" + '\0' + children               <br> 
 *              + "docs." + curr + '\0' + '{' + docs.getDocsForDocType(curr, LISTSEP) + '}' + '\0'; <br>
 *              + "doc_types" + '\0' + '{' + docTypeStr + '}'       <br>
 *              + '\0';                                             <br> 
 * <br>
 * 
 * @version 1.0, 04/24/2001 
 * @author  Steve Wang
 */

public class BuildDb2 extends Object implements BatchListener, 
                                                NavgenIntf {
    /**
     * index_not_found return code
     */
    public final static int INDEX_NOT_FOUND = -100; 

    /**
     * product not found return code
     */
    public final static int PROD_NOT_FOUND = -101; 

    /**
     * Define the All category type
     */
    public final static int ALL_CATEGORY    = -1; 
    
    /**
     * Define the 'Hot news' document type
     */
    public final static int HOTNEWS_DOCTYPE = 0; 

    /**
     * all category index
     */
    public static int allcat_doccat_index   = 0; 
    
    /**
     * hot news
     */
    public final static String HOT_NEWS     = "Hot news";

    /**
     * faq
     */
    public final static String FAQ          = "Frequently asked questions"; 

    /**
     * mb 
     */
    public final static long MB             = 1024*1024; 

    /**
     * Website objects.
     */
    private WebSite website = null; 

    /**
     * Db Connection 
     */
    private Connection conn = null; 

    /**
     * BrandFamily  
     */
    private BrandFamily brandFamilies = null;    

    /** 
     * All doctypes (8 doctypes, i.e., Downloadable files, Hints and Tips etc). 
     * The vector contains CategoryRec objects
     */
    //private Vector allDocTypes = new Vector(); 
    private Object allDocTypes[] = null;   

    /**
     * DoccategorySet (200+ doccategories, i.e., All, Doagnosis, etc). 
     * vector contains TypeCategoryRec objects. This will be set from the 
     * docCategoryEvent. 
     *       Key: BrandKey
     *      Value: TreeSet of docCategory 
     */
    //private TreeMap docCategoryMap = null; 
    private Object allDocCategories[] = null; 

    /**
     * For fewer new string() calls, build the strings earlier
     */
    private String allDocTypeCatStrings[][] = null; 
    private String allDocTypeStrings[]      = null; 

    /**
     * A static int recording docind. The docind is expected to be grouped in the 
     * the result set such as:
     *      docInd5 ....
     *      docInd5 ....
     *      docInd8 ....
     *      docInd8 ...
     * Thus it's possible to know all info for a single document has been processed
     */
     private int prevDocInd = 0;      

     private String prevMachine = "";

     // for statistics
     private long num_newdocvec = 0; 

    /** 
     * Obtain the current time as db2 timestamp. It will be
     * used to update the navgen timestamp upon finish the run 
     */ 
    private String tmspBefore = null; 
    
    // gdbm file name
    private String dbFileName = null; 

    /**
     * The current docInd being processed
     */
    private int docIndCount = 0; 

    /**
     * The product set for this website. The set contains BFMModelKey object and
     * will be used as a benchmark for delete product from the prodDocsMap which
     * contains a whole product. This will be set only through esupport site and
     * used by esupport and bp sites, not infotips site
     */
    private static TreeSet prodSet  = null;

    /**
     * The less document set (compared to infotips) for this website
     * The set contains docId string
     */
    private TreeSet docLessSet = null;

    /**
     * The Hotnews key 
     */
    private static DocTypeCatKey hotNewsKey = new DocTypeCatKey(HOTNEWS_DOCTYPE, ALL_CATEGORY); 

    /**
     * doctypecatkeys 
     */
    private static Vector docTypeCatKeys = new Vector(); 

    /**
     * root Hotnews ids. This is intended for infotips and bp sites 
     */
    private static Vector rootHotnewsVec = null; 

    /**
     * used to compare categoryRec object
     */
    private static RecComparator comp = new RecComparator(); 
    
    /**
     * temporaly holding the typecat vec
     */
    //private Vector docTypeCatVec = new Vector();

    /**
     * websites
     */
    private boolean w_infotips = false; 
    private boolean w_esupport = false;
    private boolean w_ipartner = false; 

    /**
     * removed the 'all' keys from childmap
     */
    private static boolean removedAllKey = false; 
    
    /**
     * Implement the BatchListener methods 
     *
     *  This method should be called when an error
     *  occurs. This error should not be a severe error,
     *  but a simple error, for example, 'Cannot find product 2312-XCX', or 'Cannot find Question 1'
     *
     *  @param errorStr The error string
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logError ( String errorStr ) { 
    }

    /** 
     * logSevereError
     *    This method should log program errors that
     *  prevent operation. For example, 'Cannot Connect To FTP Site'
     *  or 'FTP Site Disconnected'.
     *
     *  @param errorStr The Error message
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logSevereError ( String errorStr ) { 
    }

    /**
     * logSevereError
     *    This method should be called in a catch
     *  block of when an exception happens
     *  
     *
     *  @param e The exception that happened
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logSevereError ( Exception e ) { 
    }

    /**
     * logProgress
     *    This method should be called to notify
     *  progress.
     *  
     *  @param outputStr The output for the progress
     *  @param totalOperations The total number of operations needed to complete the batch
     *  @param operation The current operation that is being done
     *
     *  @return Nothing
     */
    public void logProgress ( String outputStr, int totalOperations, int operation ) { 
    }

    /** batchStart
     *    This method should be called when the
     *  batch starts
     *  
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void batchStart () { 
    }

    /** batchStop
     *    This method should be called when the
     *  batch has finished.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void batchStop () { 
    }

    /**
     * stop navgen batch. 
     *
     * @param n Stop condiftion. n=0 means stop success
     */
    public void stopNavgen(int n) throws BatchException {
        BatchUtil.logit("Navgen stopping for website " + website.getSiteName(), Constants.INFORMATIONAL, 0); 
        /*
        // Have outstanding batch inserts need to finish
        if (insertNum>0 && n==0) {
            BatchUtil.logit("Remaining inserts executed for " + website.getSiteName(), Constants.INFORMATIONAL, 0); 
            execBatchInserts(); 
        }
        */
    }

    /**
     * Init function 
     *
     * @param site WebSite object
     * @param init BatchInit object
     * @return none
     */
    public void init (WebSite site, Connection conn) throws BatchException { 
        website     = site; 
        this.conn   = conn; 
        Vector allDocTypesVec = website.getDocTypes(); 

        // set the website boolean
        w_infotips = website.getSiteName().equals(Constants.WEB_SITE_INFOTIPS); 
        w_esupport = website.getSiteName().equals(Constants.WEB_SITE_ESUPPORT); 
        w_ipartner = website.getSiteName().equals(Constants.WEB_SITE_IPARTNER); 

        // We manually insert the Hotnews document type 
        boolean hotAdded = false; 
        int size = allDocTypesVec.size(); 
        for (int i=0; i<size; i++) {
            if (HOT_NEWS.compareTo(((CategoryRec)allDocTypesVec.elementAt(i)).toString())<0) {
                allDocTypesVec.insertElementAt(new CategoryRec(HOTNEWS_DOCTYPE, HOT_NEWS, 0), i); 
                hotAdded = true; 
                break; 
            }
        }
                                                                            
        // Must be the last one
        if (!hotAdded) {
            allDocTypesVec.add(new CategoryRec(HOTNEWS_DOCTYPE, HOT_NEWS, 0)); 
        }
        
        // Create the all doctypes. The index is doc typeInd -- for performance 
        allDocTypes       = new Object[20]; 
        allDocTypeStrings = new String[20];
        for (int i=0; i<allDocTypesVec.size(); i++) {
            CategoryRec tcRec                 = (CategoryRec)allDocTypesVec.elementAt(i);
            allDocTypes[tcRec.getInd()]       = tcRec; 
            allDocTypeStrings[tcRec.getInd()] = "docs." + tcRec.getDescript(); 
        }
        //allDocTypes = allDocTypesVec.toArray(); 

        /*
        if (Constants.debug) {
            // Print out the doctypes and index
            BatchUtil.logit("allDocTypes array for website " + website.getSiteName(), Constants.INFORMATIONAL, 0); 
            for (int i=0; i<allDocTypes.length; i++) {
                BatchUtil.logit("  index = " + i 
                                + "; type ind = "  + ((CategoryRec)allDocTypes[i]).getInd()
                                + "; type desc = " + ((CategoryRec)allDocTypes[i]).getDescript(), Constants.INFORMATIONAL, 0); 
            }
        }
        */

        // Memory saver
        /*
        for (int i=0; i<allDocTypesVec.size(); i++) {
            CategoryRec docTypeRec = (CategoryRec)allDocTypesVec.elementAt(i); 
            allDocTypeKeys.add(new DocTypeCatKey(docTypeRec.getInd(), ALL_CATEGORY));
        }
       */
        
        //
        // Obtain the current time as db2 timestamp. It will be
        // used to update the docgen timestamp upon finish the run 
        //
        Calendar curr_cal = Calendar.getInstance();
        tmspBefore = BatchUtil.getDb2Timestamp(curr_cal); 

        try {
            dbFileName = BatchInit.getGdbmFile(website.getSiteType()) + BatchInit.getGdbmFileName(); //"db.tmp"; 
        } catch (Exception e) {
            BatchUtil.logit("Could not find db file name for websie " + website.getSiteName(), Constants.WARNING, 0); 
            dbFileName = "./db"; 
        }
        // Delete navgen.gdbm tables for the website 
        //BatchUtil.logit("Deleting previous Navgen.gdbm data for website " + website.getSiteName(), Constants.INFORMATIONAL, 0); 
        //deleteGdbmEntries(conn, website.getSiteType()); 

        if (!w_infotips/*website.getSiteName().equals(Constants.WEB_SITE_INFOTIPS)*/) {
            // creating the prodSet for esupport and bpartner
            docLessSet = new TreeSet(); 
        } else {
            // Set rootHotnewsVec for infotips 
            rootHotnewsVec = new Vector(); 
        }

        // Save some static page gdbm key-val  for esupport site only
        if (w_esupport/*website.getSiteName().equals(Constants.WEB_SITE_ESUPPORT)*/) {
            BatchUtil.logit("Saving static key-val for website " + website.getSiteName(), Constants.INFORMATIONAL, 0); 
            defineStaticPages(conn);
            prodSet = new TreeSet(); 
        }
    }   

    /**
     * brand-family event handler
     *
     * @param e A brandFamilyEvent
     * @return none
     */
    public void brandFamilySet(BatchEvent e) throws BatchException { 
        brandFamilies = ((BrandFamilyEvent)e).getBrandFamilies(); 

        // Need to update the prodSet to include Sys Management brand
        if (w_esupport/*website.getSiteName().equals(Constants.WEB_SITE_ESUPPORT)*/) {
            CategoryRec brand = brandFamilies.getBrand(Navgen.SYSMAN_BRAND);
            if (brand!=null) {
                TreeSet familySet = brandFamilies.getBrandFamilySet(brand.getInd());
                if (familySet!=null) {
                    // If the brand is Systems Management, we need to insert it into the
                    // various product tables because they don't show up in the readProd 
                    // query 
                    prodSet.add(new BrandKey(brand.getInd()));

                    // Add the family keys as well
                    Iterator it = familySet.iterator(); 
                    while (it.hasNext()) {
                        prodSet.add(new BrandFamilyKey(brand.getInd(), ((CategoryRec)it.next()).getInd()));  
                    }
                } else {
                    BatchUtil.logit(" Systems Management families not found", Constants.ERROR, 0); 
                }
            } else {
                BatchUtil.logit(" Systems Management brand not found", Constants.ERROR, 0); 
            }
        } 
    }

    /**
     * DocCategoryEvent handler.  
     *
     * @param e A doccategoryevent 
     * @return none
     */
    public void docCategorySet(DocCategoryEvent e) throws BatchException {
        //docCategoryMap = e.getDocCategory();
        // Add the 'All' cateogry case
        Object tmpArray[]   = e.getDocCategory().toArray();
        allDocCategories    = new Object[e.getDocCategory().size()+1];
        allDocCategories[0] = new String("All"); 
        System.arraycopy(tmpArray, 0, allDocCategories, 1, tmpArray.length); 
        allcat_doccat_index = 0; ; 

        allDocTypeCatStrings = new String[15][allDocCategories.length]; 
        for (int i=0; i<allDocTypes.length; i++) {
            for (int j=0; j<allDocCategories.length; j++) {
                CategoryRec typeRec = (CategoryRec)allDocTypes[i]; 
                if (typeRec!=null) {
                    //BatchUtil.logit("i= " + i + " type ind = " + typeRec.getInd() + "  type = " + typeRec.getDescript()+" cat = " +allDocCategories[j], Constants.INFORMATIONAL, 0); 
                    allDocTypeCatStrings[typeRec.getInd()][j] = "docs."+typeRec.getDescript()/*docTypeDesc*/+"."+allDocCategories[j]/*docCatDesc*/; 
                }
            }
        }

        if (Constants.debug && w_infotips) {
            // Print out the doctypes and index
            BatchUtil.logit("allDocCategories array ", Constants.INFORMATIONAL, 0); 
            for (int i=0; i<allDocCategories.length; i++) {
                BatchUtil.logit("  index = " + i + "; cat desc = " + (String)allDocCategories[i], Constants.INFORMATIONAL, 0); 
            }
        }

        //if (Constants.debug) {
            //BatchUtil.logit("Printing docCategoryMap: " + BatchUtil.getTreeMapContent(docCategoryMap), Constants.DEBUG, 0); 
        //}

        // Memory saver
        // Set every docTypeCatKeys
        /*
        for (int i=0; i<allDocTypesVec.size(); i++) {
            int docTypeInd = ((CategoryRec)allDocTypesVec.elementAt(i)).getInd(); 
            for (int j=0; j<allDocCategories.length; j++) {
                int docCatInd = findDocCatIndex((String)allDocCategories[j]); 
                if (docCatInd!=INDEX_NOT_FOUND) {
                    docTypeCatKeys.add(new DocTypeCatKey(docTypeInd, docCatInd));
                } else {
                    BatchUtil.logit("   docCatInd not found while constructing keys for doctypeind "+docTypeInd, Constants.WARNING, 0); 
                }
            }
        }
        */
    }

    /**
     * Save the marketing catalog info
     */
    public void processMk(MkEvent e) throws BatchException {
        if (e.mkKey!=null && e.mkVal!=null) {
            writeGdbm(conn, e.mkKey, e.mkVal, dbFileName); 
        }
    }

    /** 
     * Finish docs handler
     */
    public void finishDocs() throws BatchException, IOException, SQLException {
        //
        // We iterate through the prodDocsMap key set. Obtain level doc info
        // from the different kinds of keys. Each of the corresponding value is in
        // the format of :
        //          -- Value: hDocs of hashtable object. 
        //                         -- key: DocTypeCatKey 
        //                         -- value: a vector of docIds 
        // We will be able to build the Type7, Type8, Type9 and Type10 gdbm key-val 
        // with the info above
        //
        //if (Constants.debug) {
            // Print again the prodDocsMap
            //BatchUtil.logit("Printing prodDocsMap: " + BatchUtil.getTreeMapContent(Navgen.prodDocsMap), Constants.DEBUG, 0); 
            //BatchUtil.logit("Printing docLessSet: "  + BatchUtil.getLists(docLessSet, "{", "}", " "), Constants.DEBUG, 0); 
        //}
        
        // remove 'all' keys once
        /*  seems the getvalidprodlist would eleminate 'all' 
        if (!removedAllKey) {
            BatchUtil.logit("Removing the 'all' keys ", Constants.INFORMATIONAL, 0); 
            removeAllCatRec(Navgen.familyChildMap);
            removeAllCatRec(Navgen.machineChildMap);
            removedAllKey = true; 
        }
        */

        BatchUtil.logit("Finished adding docs and start saving prod-docs for " + website.getSiteName(), Constants.INFORMATIONAL, 0); 

        docIndCount = 0; 
        Set keys = Navgen.prodDocsMap.keySet(); 
        Iterator keyIt = keys.iterator(); 
        while (keyIt.hasNext()) {
            Object key = keyIt.next();

            // If prodSet is not null, which is for website esupport and bpartner,
            // check if the key is in prodSet. If it's not in, stop here because
            // the website does not contain the product and its parent levels
            if (!w_infotips/*website.getSiteName().equals(Constants.WEB_SITE_INFOTIPS)*/ && prodSet!=null && !prodSet.contains(key)) {
                continue; 
            }

            if (key instanceof BFMModelKey) {
                // Type7 key-val
                BatchUtil.logit("Saving model key for " + website.getSiteName(), Constants.DEBUG, 0); 
                saveModel(conn, (BFMModelKey)key); 
                docIndCount++; 
            } else if (key instanceof BFMachineKey) {
                // Type10 key-val
                BatchUtil.logit("Saving machine key for " + website.getSiteName(), Constants.DEBUG, 0); 
                saveMachine(conn, (BFMachineKey)key);         
                docIndCount++; 
            } else if (key instanceof BrandFamilyKey) {
                // Type9 key-val
                BatchUtil.logit("Saving family key for " + website.getSiteName(), Constants.DEBUG, 0); 
                saveFamily(conn, (BrandFamilyKey)key); 
                docIndCount++; 
            } else if (key instanceof BrandKey) {
                // Type8 key-val
                BatchUtil.logit("Saving brand key for " + website.getSiteName(), Constants.DEBUG, 0); 
                saveBrand(conn, (BrandKey)key); 
                docIndCount++; 
            } else {
                BatchUtil.logit("The prodDocsMap key " + key.toString() + " is not a bfmm type key. No action taken", Constants.WARNING, 0); 
            }

            /*
            if (docIndCount%1000==0) {
                // Get some memory info every 500 doc
                Runtime rt = Runtime.getRuntime();
                BatchUtil.logit("The free memory is " + rt.freeMemory()/MB + " MB." + " Prod-Docs processed " + docIndCount + " for " + website.getSiteName() , Constants.INFORMATIONAL, 0); 
                BatchUtil.logit("The total memory is " + rt.totalMemory()/MB + " MB.", Constants.INFORMATIONAL, 0); 
                if (docIndCount%2000==0) {
                    BatchUtil.logit("  Calling gc()", Constants.INFORMATIONAL, 0); 
                    System.gc(); 
                }
            }
            */
        }

        // We can save Type6 Key-val right now
        BatchUtil.logit("Saving root key for " + website.getSiteName(), Constants.DEBUG, 0); 
        saveRoot(conn, brandFamilies.getBrandSet()); 
    }

    // delete the ALL_CATREC from the familychildmap and machinecchildmap
    public void removeAllCatRec(TreeMap childMap) {
        Set keys = childMap.keySet(); 
        Iterator keyIt = keys.iterator();
        Vector removeKeys = new Vector(); 
        while (keyIt.hasNext()) {
            Object key = keyIt.next();
            CategoryRec rec = (CategoryRec)childMap.get(key); 
            if (rec.getDescript().equals("All") || rec.getDescript().equals("ALL")) {
                removeKeys.add(key); 
            }
        }

        // remove the keys from the map
        for (int i=0; i<removeKeys.size(); i++) {
            childMap.remove(removeKeys.elementAt(i));
        }

        // throw them away (hopefully the gc would do the job)
        removeKeys = null; 
    }
    
    // find the doccat index from alldoccategories array
    private int findDocCatIndex(String desc) {
        for (int j=0; j<allDocCategories.length; j++) {
            if (desc.equals((String)allDocCategories[j])) {
                return j; 
            }
        }
        return INDEX_NOT_FOUND; 
    }

    /*
    // find the doctype index from alldoctypes array
    private CategoryRec findDocTypeRec(int docTypeInd) {
        for (int j=0; j<allDocTypes.length; j++) {
            CategoryRec docTypeRec = (CategoryRec)allDocTypes[j];
            if (docTypeInd == docTypeRec.getInd()) {
                return docTypeRec; 
            }
        }
        return null; 
    }
    */

    // find the all category doctypekey  for a doctypeind
    /*
    private DocTypeCatKey findAllDocCatKey(int docTypeInd) {
        for (int i=0; i<allDocTypeKeys.size(); i++) {
            DocTypeCatKey key = (DocTypeCatKey)allDocTypeKeys.elementAt(i); 
            if (docTypeInd==key.doctype) {
                return key; 
            }
        }
        return null; 
    }
    */

    // find the doctypekey for a doctypeind and doccatind
    private DocTypeCatKey findDocTypeCatKey(int docTypeInd, int docCatInd) {
        for (int i=0; i<docTypeCatKeys.size(); i++) {
            DocTypeCatKey key = (DocTypeCatKey)docTypeCatKeys.elementAt(i); 
            if (docTypeInd==key.doctype && docCatInd==key.doccat) {
                return key; 
            }
        }
        return null; 
    }

    // append a type&cat string 
    private void addTypeCat(StringBuffer s, int doctype, int doccat) {
        s.append(doctype).append(" ").append(doccat).append(" ");
    }

    /**
     * DocIdEvent handler. 
     * 
     * @param e The DocsEvent object
     * @return none 
     */
    public void processDocs(DocsEvent e) throws BatchException, SQLException { 
        // Rejecting docs which does not meet mini classfication 
        if (e.getDocClassind()<website.getMiniClassification()) {
            BatchUtil.logit("Docind " + e.getDocind() + " classified for " + website.getSiteName() + ". Not added", 
                            Constants.INFORMATIONAL, 0); 
            docLessSet.add(e.getFilename()); 
            return;
        } 
        
        // Save Type3 key-val
        BatchUtil.logit("Saving doc-3 key for " + website.getSiteName(), Constants.DEBUG, 0); 
        saveFileName(conn, e);

        // Save Type4 Key-val
        BatchUtil.logit("Saving doc-4 key for " + website.getSiteName(), Constants.DEBUG, 0); 
        saveDocID(conn, e);

        // Save Type5 Key-val
        BatchUtil.logit("Saving doc-5 key for " + website.getSiteName(), Constants.DEBUG, 0); 
        saveDocID2(conn, e); 

        // We add the doc-prod relations only for infotips site. The esupport or bp 
        // site will use it and the doc difference later.
        if (!w_infotips/*website.getSiteName().equals(Constants.WEB_SITE_INFOTIPS)*/) {
            return; 
        }

        // Find the documentType and docCategoriesInd first
        int docind     = e.getDocind(); 
        int docTypeInd = e.getDocumentType(); 
        int docCatInd  = INDEX_NOT_FOUND; 

        // This vector holds the document categories inds. 
        // Multiple categories could be associated with one document
        //Vector docTypeCatVec = new Vector();
        //docTypeCatVec.clear(); 

        // The doctypecat record, which will be saved to proddocsmap
        DocTypeCat docTC = new DocTypeCat(e.getFilename());

        // Iterating through the typeRec to find the doc category 
        final Vector typeVec = e.typevec; 
        int tsize            = typeVec.size(); 
        int doccat_count     = 0;
        int prev_type        = -1;
        int prev_cat         = -1; 
        for (int i=0; i<tsize; i++) {
            CategoryRec catRec = (CategoryRec)typeVec.elementAt(i); 
            if (catRec.getCategoryLevel().equals(TypeCategory.LEVEL_DOC_CATEGORY)) {
                docCatInd    = findDocCatIndex(catRec.getDescript()); 
                if (prev_type!=-1 && prev_cat!=-1 && docTypeInd==prev_type && docCatInd==prev_cat) {
                    //eleminate dup
                    continue; 
                }

                if (docCatInd!=INDEX_NOT_FOUND && docTypeInd!=0) {
                    //DocTypeCatKey key = findDocTypeCatKey(docTypeInd, docCatInd); 
                    //if (key!=null) {
                        //docTypeCatVec.add(key/*new DocTypeCatKey(docTypeInd, docCatInd)*/);
                        addTypeCat(docTC.typeCats, docTypeInd, docCatInd);
                        prev_type = docTypeInd;
                        prev_cat  = docCatInd; 
                        doccat_count++; 
                        //break; 
                    //} else {
                    //    BatchUtil.logit ("  DocTypeCatKey not found for docind " + docind, Constants.WARNING, 0); 
                    //}
                } 
            } 
        }
/*
        if (docIndCount%500==0) {
            // Get some memory info every 500 doc
            Runtime rt = Runtime.getRuntime();
            BatchUtil.logit("The free memory is " + rt.freeMemory()/MB + " MB." + " Docs processed " + docIndCount , Constants.INFORMATIONAL, 0); 
            BatchUtil.logit("The total memory is " + rt.totalMemory()/MB + " MB.", Constants.INFORMATIONAL, 0); 
            BatchUtil.logit("The number of new docvec is " + num_newdocvec, Constants.INFORMATIONAL, 0); 
            if (docIndCount%1000==0) {
                BatchUtil.logit("  Calling gc()", Constants.INFORMATIONAL, 0); 
                System.gc(); 
            }
        }
*/
        docIndCount++; 
        BatchUtil.logit("Adding docind " + docind, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Obtained " + doccat_count + " docCatInd's; docTypeInd " + docTypeInd + " for docind " + docind, Constants.DEBUG, 0); 


        // Check if docCatInd or docTypeInd valid
        if (doccat_count==0) {
            BatchUtil.logit ("Document catInd not found for docind " + docind, Constants.WARNING, 0); 
        } else if (docTypeInd==0) {
            BatchUtil.logit ("Document typeInd not found for docind " + docind, Constants.WARNING, 0); 
        }

        // We get the doc - brand, family, type, model association here
        BatchUtil.logit("    ", Constants.DEBUG, 0); 
        BatchUtil.logit("Starting adding doc to all levels: docind " + docind + " . Size of typevec "+tsize, Constants.DEBUG, 0); 
        // Construct the key. use -1 to denote for all cat. The doc might 
        // belong to several category. 
        /*
        if (doccat_count>0) {
            DocTypeCatKey key = findAllDocCatKey(docTypeInd); 
            if (key!=null) {
                docTypeCatVec.add(key); 
            } else {
                BatchUtil.logit ("Document allcat typeInd not found for docind " + docind, Constants.WARNING, 0); 
            }
            //docTypeCatVec.add(new DocTypeCatKey(docTypeInd, ALL_CATEGORY)); 
        }

        // Determine if the document is a hot news document and add it to the 
        // doctypecatvec if it is a hotnews document
        if (e.getHotNews()) {
            docTypeCatVec.add(hotNewsKey); 
        }
        */
        
        //Add allcategory for this type
        if (doccat_count>0) {
            addTypeCat(docTC.typeCats, docTypeInd, allcat_doccat_index/*ALL_CATEGORY*/); 
        }

        // If hot news, add hot news type cat
        if (e.getHotNews()) {
            addTypeCat(docTC.typeCats, HOTNEWS_DOCTYPE/*findDocTypeIndex(HOTNEWS_DOCTYPE)*/, allcat_doccat_index/*ALL_CATEGORY*/); 
        }

        // Iterate through the typevec vector which contains prod-doc info for 
        // a single document
        String docId  = e.getFilename(); 
        int count = 0;   //counter for added docs
        int typesize = typeVec.size(); 
        for (int i=0; i<typesize; i++) {
            CategoryRec catRec = (CategoryRec)typeVec.elementAt(i); 
            //
            // We get the doc with brand, family, type and model association here.
            // The method for adding docs to the Hashtables is to add the doc to the
            // parent LEVEL (brand, family, machine or model) and add the doc to 
            // ALL the children levels. 
            
            // Later on those docs in each level
            // should be propagated down to all it's children levels -- by definitions how 
            // doc is associated with various typecategories. 
            //
            String tcLevel     = catRec.getCategoryLevel();  
            int tcInd          = catRec.getInd();

            // No document-brand direct relation
            // TypeCategory.LEVEL_BRAND
            if (tcLevel.equals(TypeCategory.LEVEL_FAMILY)) {
                BatchUtil.logit("Adding family for docind " + docind, Constants.DEBUG, 0); 
                CategoryRec brandRec = brandFamilies.getBrandRec(tcInd);
                if (brandRec!=null) {
                    int rc = 0; 
                    int brandInd         = brandRec.getInd(); 
                    BrandFamilyKey bfKey = new BrandFamilyKey(brandInd, tcInd/*familyInd*/); 
                    // Add the family and its parent level doc
                    rc = addLevelDoc(bfKey, docTC);  
                    // If family doc not added, no need to add its parent of children
                    if (rc>0) {
                        count += rc; 
                        count += addLevelDoc(new BrandKey(brandInd), docTC);  

                        // Find all its children and add the doc to it
                        count += addFamilyChildDoc(brandInd, tcInd, docTC); 
                        //BatchUtil.logit("   Added " + rc + " family docs", Constants.INFORMATIONAL, 0); 
                    }
                } else {
                    BatchUtil.logit("Brand not found for family " + tcInd +" while process doc "+docId, Constants.WARNING, 0); 
                }
            } else if (tcLevel.equals(TypeCategory.LEVEL_MACHINE) || 
                       tcLevel.equals(TypeCategory.LEVEL_SUBCATEGORY)) {
                BatchUtil.logit("Adding machine/family type for docind " + docind, Constants.DEBUG, 0); 
                BrandFamilyKey bfKey = (BrandFamilyKey)Navgen.machineParentMap.get(new Integer(tcInd)); 
                if (bfKey!=null) {
                    int rc = 0; 
                    BFMachineKey bfmKey  = new BFMachineKey(bfKey.brandInd, 
                                                            bfKey.familyInd, 
                                                            catRec.getDescript()); 
                    // Add the machine level and its parent level doc
                    rc = addLevelDoc(bfmKey, docTC);  
                    if (rc>0) {
                        count += rc; 
                        count += addLevelDoc(bfKey,  docTC);  
                        count += addLevelDoc(new BrandKey(bfKey.brandInd), docTC);  

                        // Find all its children and add the doc to it
                        count += addMachineChildDoc(bfKey.brandInd, bfKey.familyInd, catRec.getDescript(), docTC);
                        //BatchUtil.logit("   Added " + rc + " machine docs", Constants.INFORMATIONAL, 0); 
                    }
                } else {
                    BatchUtil.logit("Brand, family not found for machine " + catRec.getDescript() + " ("+tcInd+") "+" while process doc "+docId, Constants.WARNING, 0); 
                }
            } else if (tcLevel.equals(TypeCategory.LEVEL_MODEL) ||
                       tcLevel.equals(TypeCategory.LEVEL_PARTNUMBER)) {
                BatchUtil.logit("Adding model/partnum for docind " + docind, Constants.DEBUG, 0); 
                BFMachineKey bfmKey = (BFMachineKey)Navgen.modelParentMap.get(new Integer(tcInd)); 
                if (bfmKey!=null) {
                    int rc = 0; 
                    //BatchUtil.logit("Found parent for adding model/partnum docind " + docind + " . Size of modelparentMap " + Navgen.modelParentMap.size(), Constants.INFORMATIONAL, 0); 
                    BFMModelKey  bfmmKey= new BFMModelKey(bfmKey.brandInd, 
                                                           bfmKey.familyInd, 
                                                           bfmKey.machine,
                                                           catRec.getDescript()); 
                    // Add the machine level and its parent level doc
                    rc = addLevelDoc(bfmmKey, docTC);
                    if (rc>0) {
                        // We only add to the parent if the product is valid
                        count += rc; 
                        count += addLevelDoc(bfmKey,  docTC);  
                        count += addLevelDoc(new BrandFamilyKey(bfmKey.brandInd, bfmKey.familyInd), docTC);  
                        count += addLevelDoc(new BrandKey(bfmKey.brandInd), docTC);  
                    }
                    //BatchUtil.logit("   Added " + rc + " model docs", Constants.INFORMATIONAL, 0); 
                } else {
                    BatchUtil.logit("Brand, family, machine not found for model " + catRec.getDescript() + " ("+tcInd+") "+"while process doc "+docId, Constants.WARNING, 0); 
                }
            }  
        } // end for docsevent
        
        // For hot news, if the doc has been added to any level, add it to root level
        if (count>0 && e.getHotNews()) {
            // Add the docid directly to the rootHotnewsVec
            rootHotnewsVec.add(docTC.docId/*e.getFilename()*/); 
        }
        
        // No use for docsevent now, discard it 
        e = null; 
        
        //BatchUtil.logit("Finished adding doc to all levels: docind " + docind, Constants.DEBUG, 0); 
    }

    /**
     * Add the doc to the different level keys and different typecat in the prodDocsMap
     *
     * @param docTypeCatVec A vector containing doctypecatKey objects the doc belongs to  
     * @param typeInd The type category ind
     * @param level The type cateogry level
     * @param levelKey The level keys (brandkey, familykey etc) 
     * @param e The docsevent 
     * @return none
     */ 
    private int addLevelDoc (Object levelKey, DocTypeCat dtc /*DocsEvent e*/) throws BatchException {
        //
        // We make sure the levelKey is in the prodDocsMap keys. Retrieve the level doc hashtable 
        // with the levelKey. 
        // 
        int rc = 0; 
        //BatchUtil.logit("Adding docId " + docId + " to level " + levelKey.toString() + " with docTypeCatKey " + tcKey.toString(), Constants.DEBUG, 0); 
        
        if (levelKey.toString().indexOf("-All")!=-1) {
            // Don't want to add the docs to 'All' product
            BatchUtil.logit("  The docId " + dtc.docId/*e.getFilename()*/ + " not added because it has a level of All " + levelKey.toString(), Constants.DEBUG, 0); 
            return rc; 
        }

        // Get the level doc hashtable with the levelKey
        //logLargeDoc("  begin addlevel doc for level key "+levelKey.toString()); 
        //Hashtable hDoc =  (Hashtable)Navgen.prodDocsMap.get(levelKey); 
        Vector vDoc = (Vector)Navgen.prodDocsMap.get(levelKey); 
        if (vDoc!=null) {
            if (!vDoc.contains(dtc)) {
                vDoc.add(dtc);
                rc++; 
            }
            /*
            int dtc_size = docTypeCatVec.size();
            for (int i=0; i<dtc_size; i++) {
                //logLargeDoc("  found hDoc "); 
                rc++; 
                // The level doc exists and we need to merge the docId to the level doc 
                DocTypeCatKey tcKey = (DocTypeCatKey)docTypeCatVec.elementAt(i); 
                Vector docVec       = (Vector)hDoc.get(tcKey);
                if (docVec==null) {
                    //logLargeDoc("  docVec is null "); 
                    // No corresponding doc found for the doc type-cat key. Add as new
                    Vector newDocVec = new Vector(); 
                    newDocVec.add(e.getFilename());
                    hDoc.put(tcKey, newDocVec); 
                    num_newdocvec++; 
                } else {
                    if (!docVec.contains(e.getFilename())) {
                        docVec.add(e.getFilename()); 
                    }
                }
            }
            */
        } else {
            // The doc is associated to a level (brand, family etc) which no product has.
            //rc = PROD_NOT_FOUND; 
            BatchUtil.logit("Doc " + dtc.docId/*e.getFilename()*/ 
                + " assoc to level " + levelKey.toString() + " having no product - could be deleted. Not added.", Constants.INFORMATIONAL, 0); 
        }
        //logLargeDoc("  end addlevel doc for level key "+levelKey.toString()); 

        // Discard the levelKey. 
        levelKey = null; 

        return rc; 
    }

    /**
     * Add a doc to all the children of a family
     *
     * @param docTypeCatVec A vector containing doctypecatKey objects the doc belongs to  
     * @param brandInd The brand ind
     * @param familyInd The family ind
     * @param e The docsevent 
     * @return none
     */
    private int addFamilyChildDoc(int brandInd, int familyInd, DocTypeCat dtc) 
            throws BatchException {
        int rc = 0; 
        BrandFamilyKey bfKey = new BrandFamilyKey(brandInd, familyInd); 
        //logLargeDoc("Add family child doc " + bfKey.toString()); 
        // Iterate through the familyChildMap to all the machine children 
        TreeSet machineSet = (TreeSet)Navgen.familyChildMap.get(bfKey); 
        if (machineSet!=null) {
            Iterator machIt = machineSet.iterator(); 
            // Iterate through machine children
            while (machIt.hasNext()) {
                CategoryRec machineRec = (CategoryRec)machIt.next();
                BFMachineKey bfmKey = new BFMachineKey(brandInd, familyInd, machineRec.getDescript()); 

                // Add doc to this machine child
                rc += addLevelDoc(bfmKey, dtc);

                // Add doc to all the grand child model as well 
                rc += addMachineChildDoc(brandInd, familyInd, machineRec.toString(), dtc);
            }
        } else {
            BatchUtil.logit("No child found in familyChildMap for bfKey " + bfKey.toString(), 
                  Constants.INFORMATIONAL, 0); 
        }
        return rc; 
    }

    /**
     * Add a doc to all the children of a machine
     *
     * @param docTypeCatVec A vector containing doctypecatKey objects the doc belongs to  
     * @param brandInd The brand ind
     * @param familyInd The family ind
     * @param machine The machine description
     * @param e The docsevent 
     * @return none
     */
    private int addMachineChildDoc( 
                                    int     brandInd, 
                                    int     familyInd, 
                                    String  machine, 
                                    DocTypeCat dtc) throws BatchException { 
        int rc = 0; 
        BFMachineKey bfmKey = new BFMachineKey(brandInd, familyInd, machine); 
        //logLargeDoc("Add machine child doc " + bfmKey.toString()); 
        TreeSet modelSet = (TreeSet)Navgen.machineChildMap.get(bfmKey); 
        if (modelSet!=null) {
            Iterator modelIt = modelSet.iterator();
            // Then iterate through each model 
            while (modelIt.hasNext()) {
                CategoryRec modelRec = (CategoryRec)modelIt.next(); 
                BFMModelKey bfmModelKey = new BFMModelKey(brandInd, familyInd, machine, modelRec.getDescript());
                rc += addLevelDoc(bfmModelKey, dtc);
            }
        } else {
            BatchUtil.logit("No child found in machineChildMap for bfmKey " + bfmKey.toString(), 
                  Constants.INFORMATIONAL, 0); 
        }
        return rc; 
    }
    
    /**
     * Finish prod handler
     *
     * @param none
     * @return none
     */
    public void finishProd() throws BatchException {
        BatchUtil.logit("Finishing process prod ", Constants.INFORMATIONAL, 0); 

        // Get some memory information
        Runtime rt = Runtime.getRuntime();
        BatchUtil.logit("The free memory is " + rt.freeMemory()/MB + " MB", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("The total memory is " + rt.totalMemory()/MB + " MB", Constants.INFORMATIONAL, 0); 
        
        if (w_esupport/*website.getSiteName().equals(Constants.WEB_SITE_ESUPPORT)*/) {
            // No more product. We can save all the buildProfileDb key-val now.
            // This applies to esupport website only
            BatchUtil.logit("Start saving profile key-val ", Constants.INFORMATIONAL, 0); 
            BuildProfileDb profileGdbm = new BuildProfileDb(conn, Constants.ESUPPORT_PROFILE_TYPE, this); 
            profileGdbm.saveBrandDisplay();
            profileGdbm.saveBrandKeys(Navgen.brandSet, Navgen.brandChildMap, Navgen.familyChildMap, Navgen.machineChildMap, Navgen.productMap); 
        }
        return; 
    }

    /**
     * Verify if a product is valid or not based on website and announce date. 
     * 
     * Every Product should be published to eSupport if (today+1day) >= date_announced and PUBLISH = 'Y'
     * Every Product should be published to Business Partner if (today+1day) >= date_announced and PUBLISH = 'Y'
     * Every Product should be published to Info Tips if PUBLISH = 'Y'
     *
     * Since we already have publish to 'Y' when the product is coming through the 
     * query from navgen, we only compare the dates to validate the product
     * 
     * @param website The website parameter
     * @param prodInd The productInd
     * @param datesvec The vector stores ProductDate 
     */
    public boolean isValid(WebSite website, ProdEvent e) {
        // For infotips site, not affected by the date. 
        if (w_infotips/*website.getSiteName().equals(Constants.WEB_SITE_INFOTIPS)*/) {
            return true; 
        } else {
            if (e.datesvec!=null) {
                // Get the date for the prodInd. Since only about 5% products having dates
                // and for those having dates, only 10% having the date_announced type, we
                // will validate a product only based on the date_announced type. For all
                // products which do not have dates or do not have date_announced type, 
                // we will publish them for esupport or bp sites FOR NOW. 
                int size = e.datesvec.size(); 
                for (int i=0; i<size; i++) {
                    ProductDate pdate = (ProductDate) e.datesvec.elementAt(i); 
                    if (pdate.typedateind==TypeList.DATE_ANNOUNCED) {
                        BatchUtil.logit("actual anno=" + pdate.actual_announce, Constants.DEBUG, 0); 
                        //if (!pdate.actual_announce.equals("NULL")) {
                            CDate today      = new CDate();
                            String today_str = today.getDate(CDate.DATE_DBDATE); 
                            String tomorrow  = CDate.addDaysToDate(today_str, 1, CDate.DATE_DBDATE);

                            // tomorrow >= actual_announce, then publish; otherwise, not
                            if (CDate.compareDate(tomorrow, pdate.actual_announce, CDate.DATE_DBDATE)>=0) {
                                return true;
                            } else {
                                return false; 
                            }
                        //} 
                    }
                } // end for
            } 
        }

        // add the product to the less map
        //BatchUtil.logit("Can not find information to valid Product " + e.getProductInd() + " for " + website.getSiteName() + ". Not published", Constants.WARNING, 0); 
        return true/*false*/; 
    }
     
    /**
     * ProdEvnet handler.  
     *
     * @param e The ProdEvent object
     * @return none
     */
    public void processProd(ProdEvent e) throws BatchException {  
        BatchUtil.logit("Processing product " + e.getProductInd() + " for " + website.getSiteName(), 
                        Constants.INFORMATIONAL, 0); 
        
        // Verify if the product is valid. If it's not valid, don't process 
        // any further
        if (!isValid(website, e)) {
            BatchUtil.logit("Product " + e.getProductInd() + " is not valid for " + website.getSiteName(), 
                            Constants.DEBUG, 0); 
            return; 
        }

        // Add the prodSet for esupport (same for bp) 
        if (prodSet!=null && w_esupport/*website.getSiteName().equals(Constants.WEB_SITE_ESUPPORT)*/) {
            // Build the brand key
            prodSet.add(new BrandKey(e.brandInd));

            // Build the family key
            prodSet.add(new BrandFamilyKey(e.brandInd, e.familyInd));

            // Build the machine key
            prodSet.add(new BFMachineKey(e.brandInd, e.familyInd, e.machine));

            // Build the model key
            prodSet.add(new BFMModelKey(e.brandInd, e.familyInd, e.machine, e.model));
        }

        boolean isOption = false; 
        if (e.brand.indexOf("IBM Options")!=-1) {
            isOption = true; 
        }                               

        // We can save product level key-val here directly
        // Write the type-1 key-val
        if ((e.machine!="All") && (e.model!="All") && !isOption) { 
            String gkey = e.machine + '-' + e.model;
            String gval = 
                "realpage" + NULLSEP + e.brand + PIPESEP + e.family + PIPESEP + e.machine + 
                PIPESEP + e.model + NULLSEP + "family" + NULLSEP + e.family + NULLSEP; 
            writeGdbm(conn, gkey, gval, dbFileName); 
        }

        // Write the type-2 key-val
        if (e.machine!="All" && !isOption) { 
            if (prevMachine.length()==0 || prevMachine.compareTo(e.machine)!=0) {
                // A new machine
                String gkey = e.machine; 
                String gval = "realpage" + NULLSEP + e.brand + PIPESEP + e.family + PIPESEP + e.machine + NULLSEP; 
                writeGdbm(conn, gkey, gval, dbFileName); 
                 
                // Update to the new machine
                prevMachine = e.machine; 
            } 
        }
    }

    /**
     * Obtain the docs lists for the key
     *
     * @param pDocsMap The prodDocsMap 
     * @param levelKey The key to prodDocsMap, such as BrandKey, BrandFamilyKey etc.
     * @param valStr The doctype, doccat and id lists which will be saved into gdbm 
     * @return none
     */
    private void buildDocString (TreeMap pDocsMap, Object levelKey, StringBuffer valStr) {
        // Obtaining doc type and cat and docId list
        //Vector docType    = new Vector();
        //TreeMap docCatMap = new TreeMap(); 
        TreeSet typeSet = new TreeSet(comp);
        TreeMap catMap  = new TreeMap(); 
        TreeMap docsMap = new TreeMap(); 
        //Hashtable hDocs   = (Hashtable)pDocsMap.get(levelKey); 
        Vector vDocs    = (Vector)pDocsMap.get(levelKey); 
        //BatchUtil.logit("buildDocString start", Constants.DEBUG, 0); 
        if (vDocs==null) {
            BatchUtil.logit("No docs found for the levelKey " + levelKey.toString(), Constants.DEBUG, 0); 
        } else {
            StringBuffer hotNewsId = new StringBuffer();
            boolean hotNewsAdded   = false; 
            BatchUtil.logit("   vDocs size for levelkey " + levelKey.toString() + " is " +vDocs.size(), Constants.DEBUG, 0); 
            for (int i=0; i<vDocs.size(); i++) {
                DocTypeCat docTC = (DocTypeCat)vDocs.elementAt(i); 
                if (!w_infotips) {
                    if (docLessSet.contains(docTC.docId)) {
                        continue; 
                    }
                }

                // docTC.typeCats is formatted as "i j i j ...etc"
                // todo: all category special case
                StringTokenizer st = new StringTokenizer(docTC.typeCats.toString());
                if (Constants.debug) {
                    BatchUtil.logit("   the type&cat string is  " + docTC.typeCats.toString(), Constants.DEBUG, 0); 
                    BatchUtil.logit("   the total tokens are " + st.countTokens(), Constants.DEBUG, 0); 
                }        
                while (st.hasMoreTokens()) {
                    int docTypeInd  = 0;
                    int docCatIndex = 0;
                    try {
                        docTypeInd  = Integer.parseInt(st.nextToken());
                        docCatIndex = Integer.parseInt(st.nextToken());
                    } catch (NumberFormatException e) {
                        // Nothing major to worry
                        BatchUtil.logit("  The token string is  " + docTC.typeCats.toString(), Constants.WARNING, 0); 
                        continue; 
                    }
                    //BatchUtil.logit("Find docTypeIndex " + docTypeIndex + " and docCatIndex " + docCatIndex, Constants.DEBUG, 0); 
                    CategoryRec docTypeRec = (CategoryRec)allDocTypes[docTypeInd]; //(CategoryRec)allDocTypes[docTypeIndex]; 
                    if (docTypeRec==null) {
                        BatchUtil.logit("  Incorrect docTypeInd= " + docTypeInd, Constants.WARNING, 0); 
                        continue; 
                    }
                    typeSet.add(docTypeRec/*new Integer(docTypeInd)*/); 

                    // If it's a hot news, add it to the hotnews vector
                    if (docTypeInd==HOTNEWS_DOCTYPE) {
                        // Hot news type
                        hotNewsId.append("{").append(docTC.docId).append(".html} "); 
                        hotNewsAdded = true; 
                        continue; 
                    }

                    //String docCatDesc  = (String)allDocCategories[docCatIndex]; 
                    String key = allDocTypeCatStrings[docTypeRec.getInd()][docCatIndex]; //new String("docs."+docTypeRec.getDescript()+"."+docCatDesc); 
                    StringBuffer idStr = (StringBuffer)docsMap.get(key); 
                    if (idStr!=null) {
                        idStr.append("{").append(docTC.docId).append(".html} "); 
                    } else {
                        idStr = new StringBuffer().append("{").append(docTC.docId).append(".html} ");
                        docsMap.put(key, idStr); 
                    }

                    key = allDocTypeStrings[docTypeRec.getInd()]; //new String("docs."+docTypeRec.getDescript()); 
                    TreeSet catSet = (TreeSet)catMap.get(key);
                    if (catSet!=null) {
                        catSet.add(allDocCategories[docCatIndex]/*new Integer(docCatIndex)*/); 
                    } else {
                        catSet = new TreeSet();
                        catSet.add(allDocCategories[docCatIndex]/*new Integer(docCatIndex)*/); 
                        catMap.put(key, catSet); 
                    }
                } // end while
            } // end for

            // Iterate through the catMap to build the docCat lists for each of 
            // the doc type
            BatchUtil.logit("   catMap size is " +catMap.size(), Constants.DEBUG, 0); 
            Set keys = catMap.keySet(); 
            Iterator keyIt = keys.iterator(); 
            while (keyIt.hasNext()) {
                Object key = keyIt.next();
                TreeSet catSet = (TreeSet)catMap.get(key); 
                // The key is a string of "docs.types" format
                StringBuffer catStr = new StringBuffer(); 
                int i = 0; 
                int size = catSet.size(); 
                Iterator it = catSet.iterator(); 
                while (it.hasNext()) {
                    //Integer docCatIndex = (Integer)it.next(); 
                    catStr.append("{").append((String)it.next()/*allDocCategories[docCatIndex.intValue()]*/).append("}"); 
                    if (i<(size-1)) {
                        catStr.append(" "); 
                    }
                    i++; 
                }
                valStr.append((String)key).append(NULLSEP).append(catStr).append(NULLSEP); 
            }

            // Add the hot news 
            if (hotNewsAdded) {
                valStr.append("docs."+HOT_NEWS).append(NULLSEP).append(hotNewsId).append(NULLSEP); 
            }

            // Iterate through the docsMap to build the docId list for
            // each of the docType and docCat
            BatchUtil.logit("   docsMap size is " +docsMap.size(), Constants.DEBUG, 0); 
            keys  = docsMap.keySet();
            keyIt = keys.iterator();
            while (keyIt.hasNext()) {
                Object key = keyIt.next();
                // The key is a string of "docs.type.cat" format
                valStr.append((String)key).append(NULLSEP).append((StringBuffer)docsMap.get(key)).append(NULLSEP); 
            }

            // Finally build the doc_types string
            StringBuffer typeStr = new StringBuffer(); 
            int i = 0; 
            int size = typeSet.size(); 
            Iterator it = typeSet.iterator(); 
            while (it.hasNext()) {
                //Integer docTypeInd = (Integer)it.next(); 
                CategoryRec docTypeRec = (CategoryRec)it.next(); //findDocTypeRec(docTypeInd.intValue()); 
                typeStr.append("{").append(docTypeRec.getDescript()).append("}"); 
                if (i<(size-1)) {
                    typeStr.append(" "); 
                }
                i++; 
            }
            valStr.append("doc_types").append(NULLSEP).append(typeStr).append(NULLSEP); 
        } 

        // Hopes this will help deallocate memory for gc. 
        typeSet = null;
        catMap  = null;
        docsMap = null;
    }

    /**
     * The function will kick out the docId which is in the docLessSet
     *
     * @param idVec A vector of doc id constructed for infotips site
     * @param lessDoc The difference (less) docs for the website
     * @param Vector A vector containing doc id's just for the website
     */
    /*
    private Vector getValidDocs(TreeSet idVec, TreeSet lessDoc) {
        Vector validDocs = new Vector(); 
        Iterator it = idVec.iterator(); 
        while (it.hasNext()) {
            DocRec docrec = (DocRec)it.next();
            if (!lessDoc.contains(docrec.docId)) {
                validDocs.add(docrec.docId); 
            }
        }
        return validDocs; 
    }
    */

    /**
     * Write the brand key-value to gdbm 
     *
     * @param conn A db connection
     * @param key A BrandKey key
     * @retunr none
     */ 
    public void saveBrand (Connection conn, BrandKey key) throws WriteGdbmException {  
        CategoryRec brandRec = brandFamilies.getBrand(key.brandInd);
        TreeSet familySet    = brandFamilies.getBrandFamilySet(key.brandInd); 
        if (brandRec!=null) {
            String brand = brandRec.toString(); 

            // look = lowercase brand with "IBM " removed and spaces converted to "_"
            String look = BatchUtil.substitute(brand, "IBM ", "");
            look = BatchUtil.substitute(look, " ", "-");
            look = look.toLowerCase(); 

            // write brand data
            String gkey = brand;
            String gval = "title" + NULLSEP + brand
              + NULLSEP + "template" + NULLSEP + "techinfo"
              + NULLSEP + "look" + NULLSEP + look
              + NULLSEP + "brand" + NULLSEP + brand
              + NULLSEP + "mktg_suffix" + NULLSEP + techtipsHack(brand)
              + NULLSEP;

            if (familySet != null) {
                String childPrompt = "family";
                String children    = //BatchUtil.getLists(familySet, "{", "}", " ");   
                    w_infotips? BatchUtil.getLists(familySet, "{", "}", " ") :
                                BatchUtil.getValidProdLists(familySet, key, prodSet, "{", "}", " "); 
                gval += "child_prompt" + NULLSEP + childPrompt
                  + NULLSEP + "children" + NULLSEP + children
                  + NULLSEP;
            } else {
                BatchUtil.logit("No family found for brand " + brand, Constants.WARNING, 0); 
            }

            // Obtaining doc type and cat and docId list
            StringBuffer gval2 = new StringBuffer(gval); 
            buildDocString(Navgen.prodDocsMap, key, gval2); 
            writeGdbm(conn, gkey, gval2.toString(), dbFileName); 
        } else {
            BatchUtil.logit(" No gdbm data written for Type8 key: no brand found for BrandKey " + key.brandInd, Constants.WARNING, 0); 
        }
    }

    /**
     * Write the brand-family key-value to gdbm 
     *
     * @param conn A db connection
     * @param key A BrandFamilyKey key
     * @retunr none
     */ 
    public void saveFamily (Connection conn, BrandFamilyKey key) throws WriteGdbmException {  
        CategoryRec brandRec  = brandFamilies.getBrand(key.brandInd);
        CategoryRec familyRec = brandFamilies.getBrandFamily(key.brandInd, key.familyInd); 
        if (brandRec != null) {
            String brand  = brandRec.toString(); 
            String family = familyRec.toString();

            // look = lowercase brand with "IBM " removed and spaces converted to "_"
            String look = BatchUtil.substitute(brand, "IBM ", "");
            look = BatchUtil.substitute(look, " ", "-");
            look = look.toLowerCase(); 

            // write brand family data
            String gkey = brand + PIPESEP + family;
            String gval = "title" + NULLSEP + brand
              + NULLSEP + "subtitle" + NULLSEP + family 
              + NULLSEP + "template" + NULLSEP + "techinfo"
              + NULLSEP + "look" + NULLSEP + look
              + NULLSEP + "brand" + NULLSEP + brand
              + NULLSEP + "mktg_suffix" + NULLSEP + techtipsHack(brand)
              + NULLSEP;

            // Build the machine child list for the family
            TreeSet childSet   = (TreeSet)Navgen.familyChildMap.get(key);
            if (childSet!=null) {
                String childPrompt = "machine type";
                String children    = //BatchUtil.getLists(childSet, "{", "}", " ");   
                    w_infotips? BatchUtil.getLists(childSet, "{", "}", " ") :
                                BatchUtil.getValidProdLists(childSet, key, prodSet, "{", "}", " "); 
                gval += "child_prompt" + NULLSEP + childPrompt
                  + NULLSEP + "children" + NULLSEP + children
                  + NULLSEP;
            } else {
                BatchUtil.logit("No machine child found for brand-family: " + brand + "-" + family, Constants.WARNING, 0); 
            }

            // Obtaining doc type and cat and docId list
            StringBuffer gval2 = new StringBuffer(gval); 
            buildDocString(Navgen.prodDocsMap, key, gval2); 
            writeGdbm(conn, gkey, gval2.toString(), dbFileName); 
        } else {
            BatchUtil.logit(" No gdbm data written for Type9 key: no brand found for BrandKey " + key.brandInd, Constants.WARNING, 0); 
        }
    }
    
    /**
     * Write the brand-family-machine key-value to gdbm 
     *
     * @param conn A db connection
     * @param key A BFMachineKey key
     * @retunr none
     */ 
    public void saveMachine (Connection conn, BFMachineKey key) throws WriteGdbmException {  
        CategoryRec brandRec  = brandFamilies.getBrand(key.brandInd);
        CategoryRec familyRec = brandFamilies.getBrandFamily(key.brandInd, key.familyInd); 
        String machine = key.machine; 
        if (brandRec != null) {
            String brand  = brandRec.toString(); 
            String family = familyRec.toString();

            // look = lowercase brand with "IBM " removed and spaces converted to "_"
            String look = BatchUtil.substitute(brand, "IBM ", "");
            look = BatchUtil.substitute(look, " ", "-");
            look = look.toLowerCase(); 

            // write brand family, machine data
            //String gkey = brand + PIPESEP + family + PIPESEP + machine;
            StringBuffer gkey = new StringBuffer().append(brand).append(PIPESEP).append(family)
                                  .append(PIPESEP).append(machine);
            /*
            String gval = "title" + NULLSEP + family
                    + NULLSEP + "subtitle" + NULLSEP + machine
                    + NULLSEP + "template" + NULLSEP + "techinfo"
                    + NULLSEP + "look" + NULLSEP + look
                    + NULLSEP + "brand" + NULLSEP + brand
                    + NULLSEP + "mktg_suffix" + NULLSEP + techtipsHack(brand)
                    + NULLSEP;
            */
            StringBuffer gval = new StringBuffer().append("title").append( NULLSEP).append( family)
                   .append( NULLSEP).append( "subtitle")   .append( NULLSEP).append( machine)
                   .append( NULLSEP).append( "template")   .append( NULLSEP).append( "techinfo")
                   .append( NULLSEP).append( "look")       .append( NULLSEP).append( look)
                   .append( NULLSEP).append( "brand")      .append( NULLSEP).append( brand)
                   .append( NULLSEP).append( "mktg_suffix").append( NULLSEP).append( techtipsHack(brand))
                   .append( NULLSEP);
            // Build the model child list for the machine
            TreeSet childSet   = (TreeSet)Navgen.machineChildMap.get(key);
            if (childSet!=null) {
                String childPrompt = "model";
                String children    = 
                    w_infotips? BatchUtil.getLists(childSet, "{", "}", " ") :
                                BatchUtil.getValidProdLists(childSet, key, prodSet, "{", "}", " "); 
                //gval += "child_prompt" + NULLSEP + childPrompt
                //  + NULLSEP + "children" + NULLSEP + children
                //  + NULLSEP;
                gval.append("child_prompt").append( NULLSEP).append( childPrompt)
                 .append( NULLSEP).append( "children").append( NULLSEP).append( children)
                 .append( NULLSEP);
            } else {
                BatchUtil.logit("No model found for brand|family|machine: " + 
                      brand + "|" + family +"|" + machine, Constants.WARNING, 0); 
            }

            // Obtaining doc type and cat and docId list
            //gval += buildDocString(Navgen.prodDocsMap, key); 
            buildDocString(Navgen.prodDocsMap, key, gval); 
            writeGdbm(conn, gkey.toString(), gval.toString(), dbFileName); 
        } else {
            BatchUtil.logit(" No gdbm data written for Type10 key: no brand found for BrandKey " + key.brandInd, Constants.WARNING, 0); 
        }
    }
    
    /**
     * Write the brand-family-type-model key-value to gdbm 
     *
     * @param conn A db connection
     * @param key A BFMModelKey key
     * @retunr none
     */ 
    public void saveModel (Connection conn, BFMModelKey key) throws WriteGdbmException {  
        CategoryRec brandRec  = brandFamilies.getBrand(key.brandInd);
        CategoryRec familyRec = brandFamilies.getBrandFamily(key.brandInd, key.familyInd); 
        String machine = key.machine; 
        String model   = key.model; 
        if (brandRec != null) {
            String brand  = brandRec.toString(); 
            String family = familyRec.toString();

            // look = lowercase brand with "IBM " removed and spaces converted to "_"
            String look = BatchUtil.substitute(brand, "IBM ", "");
            look = BatchUtil.substitute(look, " ", "-");
            look = look.toLowerCase(); 

            // write brand family, machine, model data      
            //String gkey = brand + PIPESEP + family + PIPESEP + machine + PIPESEP + model;
            StringBuffer gkey = new StringBuffer().append(brand).append(PIPESEP).append(family)
                                   .append(PIPESEP).append(machine).append(PIPESEP).append(model);
            /*
            String gval = "title" + NULLSEP + family
                + NULLSEP + "subtitle" + NULLSEP + (String)Navgen.productMap.get(key)
                + NULLSEP + "mtmdl" + NULLSEP + machine + "-" + model
                + NULLSEP + "template" + NULLSEP + "techinfo"
                + NULLSEP + "look" + NULLSEP + look
                + NULLSEP + "brand" + NULLSEP + brand
                + NULLSEP + "mktg_suffix" + NULLSEP + techtipsHack(brand)
                + NULLSEP ;
            */
            StringBuffer gval = new StringBuffer("title").append( NULLSEP).append( family)
               .append( NULLSEP).append( "subtitle")   .append( NULLSEP).append( (String)Navgen.productMap.get(key))
               .append( NULLSEP).append( "mtmdl")      .append( NULLSEP).append( machine).append( "-").append( model)
               .append( NULLSEP).append( "template")   .append( NULLSEP).append( "techinfo")
               .append( NULLSEP).append( "look")       .append( NULLSEP).append( look)
               .append( NULLSEP).append( "brand")      .append( NULLSEP).append( brand)
               .append( NULLSEP).append( "mktg_suffix").append( NULLSEP).append( techtipsHack(brand))
               .append( NULLSEP);

            // Obtaining doc type and cat and docId list
            buildDocString(Navgen.prodDocsMap, key, gval); 
            writeGdbm(conn, gkey.toString(), gval.toString(), dbFileName); 
        } else {
            BatchUtil.logit(" No gdbm data written for Type7 key: no brand found for BrandKey " + key.brandInd, Constants.WARNING, 0); 
        }
    }
    
    /**
     * Write the root key-value to gdbm 
     * 
     * @param conn A db connection
     * @param brands A sorted set of brand typecategory objects 
     */ 
    public void saveRoot (Connection conn, TreeSet brands) throws WriteGdbmException { 
        String rootChildren = BatchUtil.getLists(brands, "{", "}", " "); 

        String gkey = "root";
        String gval = "title" + NULLSEP + "Welcome"
          + NULLSEP + "template" + NULLSEP + "welcome"
          + NULLSEP + "child_prompt" + NULLSEP + "brand"
          + NULLSEP + "look" + NULLSEP + "welcome"
          + NULLSEP + "children" + NULLSEP + rootChildren
          + NULLSEP;
        // only for infotips and bp
        if (website.getSiteName().compareTo(Constants.WEB_SITE_ESUPPORT)!=0) {
            if (rootHotnewsVec!=null && rootHotnewsVec.size()>0) {
                if (docLessSet!=null) {
                    // Bp site
                    gval += "doc_types" + NULLSEP + "{Hot news}" + NULLSEP 
                           +"docs.Hot news" + NULLSEP + BatchUtil.getValidLists(rootHotnewsVec, docLessSet, "{", ".html}", " ") 
                           + NULLSEP;  
                } else {
                    // Infotips site  
                    gval += "doc_types" + NULLSEP + "{Hot news}" + NULLSEP 
                           +"docs.Hot news" + NULLSEP + BatchUtil.getLists(rootHotnewsVec, "{", ".html}", " ") 
                           + NULLSEP;  
                }
            }
        }
        writeGdbm(conn, gkey, gval, dbFileName); 
    }
    
    /**
     * Write the docid key-value to gdbm 
     *
     * @param conn A db connection
     * @param e A DocsEvent object
     */ 
    public void saveDocID(Connection conn, DocsEvent e) throws WriteGdbmException {  
        String docId = e.getFilename();
        String gkey  = docId;  
        //String gval  = "realpage" + NULLSEP + docId +".html" + NULLSEP;
        StringBuffer gval  = new StringBuffer().append("realpage").append(NULLSEP).append(docId)
                                               .append(".html")   .append(NULLSEP);
        writeGdbm(conn, gkey, gval.toString(), dbFileName); 
    }
    
    /**
     * Write the docid (without dash) key-value to gdbm 
     *
     * @param conn A db connection
     * @param e A DocsEvent object
     */ 
    public void saveDocID2(Connection conn, DocsEvent e) throws WriteGdbmException {  
        String docId = e.getFilename();
        String gkey  = BatchUtil.substitute(docId, "-", "");  
        String gval  = "realpage" + NULLSEP + docId +".html" + NULLSEP;
        writeGdbm(conn, gkey, gval, dbFileName); 
    }
    
    /**
     * Write the quest file name key-value to gdbm 
     *
     * @param conn A db connection
     * @param e A DocsEvent object
     * @return none
     */ 
    public void saveFileName (Connection conn, DocsEvent e) throws WriteGdbmException, SQLException { 
        // We'll get the the date of the document first. If it's SSD doc 
        // (doctypeind = 9), we'll get the release date from docsproperties table.
        String docTime = e.getReleaseDate();
        /*
        String gkey = e.getFilename() + ".html";
        String gval = "title" + NULLSEP + e.getTitle()
          + NULLSEP + "date"  + NULLSEP + docTime
          + NULLSEP + "faq"   + NULLSEP + (e.getFaq()? "Yes" : "No")
          + NULLSEP + "news"  + NULLSEP + (e.getHotNews()? "Yes" : "No")
          + NULLSEP;
        */
        StringBuffer gkey = new StringBuffer().append(e.getFilename()).append(".html");
        StringBuffer gval = new StringBuffer().append("title").append(NULLSEP).append(e.getTitle())
          .append( NULLSEP) .append( "date")  .append( NULLSEP) .append( docTime)
          .append( NULLSEP) .append( "faq" )  .append( NULLSEP) .append( (e.getFaq()? "Yes" : "No"))
          .append( NULLSEP) .append( "news")  .append( NULLSEP) .append( (e.getHotNews()? "Yes" : "No"))
          .append( NULLSEP);
        writeGdbm(conn, gkey.toString(), gval.toString(), dbFileName); 
    }
    
    /**
     * This method executes the C++ command which extracts key and value from 
     * gdbm table and creates db files
     *
     * @param sitename The website name  
     * @return boolean True for success and false for failure
     */
    public boolean createDbFile (String sitename) throws BatchException, IOException {
        /* Although we can pass these env to the DbGen command, the java seems 
           not passing the system env such as libpath etc to the command. 
        // Contruct env variables to be used in the createDb command
        String envp[] = {
            "dbName="    + BatchInit.getDbName(),
            "dbUser="    + BatchInit.getDbUserId(),
            "dbPasswd="  + BatchInit.getDbUserPasswd(),
            "dbOutPath=" + BatchInit.getGdbmFileOutPath(website.getSiteName()),
            "webSite="   + sitename
        };
        */

        try {
            Runtime rt = Runtime.getRuntime();
            String commandDir = "./";
            if (BatchInit.getCommandDir().length()>0) {
                commandDir = BatchInit.getCommandDir(); 
            }
            
            String dbgenCmd = commandDir + BatchInit.getCreateDbCommand() 
                            + " -U " + BatchInit.getDbUserId()
                            + " -P " + BatchInit.getDbUserPasswd()
                            + " -D " + BatchInit.getDbName()
                            + " -w " + sitename
                            + " -O " + BatchInit.getGdbmFileOutPath(sitename); 

            BatchUtil.logit("Creating db file for website " + sitename, Constants.INFORMATIONAL, 0);
            BatchUtil.logit("The command is " + dbgenCmd, Constants.INFORMATIONAL, 0);
            
            //Process proc = rt.exec(BatchInit.getCreateDbCommand()/*command name*/, envp, new File(commandDir)); 
            Process proc = rt.exec(dbgenCmd/*command name*/); 
            
            BatchUtil.logit("Waiting for the process to finish " + commandDir + BatchInit.getCreateDbCommand(), Constants.INFORMATIONAL, 0);
            int rc = proc.waitFor();

            // Prepare messages from the process
            String proc_error_msg = "DbGen error log message: "; 
            String proc_log_msg = "DbGen log message: "; 
            final int BUFF_SIZE = 1024*20;  
            byte[] buff = new byte[BUFF_SIZE]; 
            
            // Determine if the process is finished normally or not
            if (rc==0) {
                // The subprocess finished normally
                // Get log message
                int n = proc.getInputStream().read(buff);
                if (n>0 && n<BUFF_SIZE) {
                    proc_log_msg += new String(buff, 0, n); 
                }
                BatchUtil.logit("DbGen finished normally. rc=" + rc + " \n" + proc_log_msg, Constants.INFORMATIONAL, 0);
                return true; 
            } else {
                // Abnormal termination of the subprocess
                // Get error message
                int n = proc.getErrorStream().read(buff);
                proc_error_msg += new String(buff, 0, n); 

                // Get any output message
                n = proc.getInputStream().read(buff);
                if (n>0 && n<BUFF_SIZE) {
                    proc_log_msg += new String(buff, 0, n); 
                }
                BatchUtil.logit("DbGen erro: rc=" + rc + "\n" + proc_error_msg + "\n" + proc_log_msg, Constants.ERROR, 0);
            }
        } catch (Exception e) {
            throw new CreateDbCommandException("Error in executing DbGen: " + e, Constants.ERROR);  
        }
        return false; 
    }

    /**
     * Delete the entries in the navgen.gdbm for the current website
     */
    public void deleteGdbmEntries(Connection conn, int sitetype) throws BatchException { 
        try {
            Statement stmt = conn.createStatement();
            ResultSet rs;

            // First test if the table is full or empty
            rs = stmt.executeQuery (
                   "SELECT count(*) FROM navgen.gdbm g " + 
                   "WHERE g.siteind = " + sitetype   
                   ); 

            if (rs.next()) {
                int items = rs.getInt(1); 
                if (items>0) {
                    BatchUtil.logit("There are " + items + " to delete for website "+sitetype, Constants.INFORMATIONAL, 0);
                    rs = stmt.executeQuery (
                           "DELETE FROM navgen.gdbm g " + 
                           "WHERE g.siteind = " + sitetype   
                           ); 
                } else {
                    BatchUtil.logit("Nothing to delete for website "+sitetype, Constants.INFORMATIONAL, 0);
                }
            }
            
            // If it's esupport site, we need to delete the profile entries as well
            if (sitetype == Constants.ESUPPORT_TYPE) {
                // First test if the table is full or empty
                rs = stmt.executeQuery (
                       "SELECT count(*) FROM navgen.gdbm g " + 
                       "WHERE g.siteind = " + Constants.ESUPPORT_PROFILE_TYPE   
                       ); 
                if (rs.next()) {
                    int items = rs.getInt(1); 
                    if (items>0) {
                        BatchUtil.logit("There are " + items + " to delete for profile" , Constants.INFORMATIONAL, 0);
                        rs = stmt.executeQuery (
                               "DELETE FROM navgen.gdbm g " + 
                               "WHERE g.siteind = " + Constants.ESUPPORT_PROFILE_TYPE   
                               ); 
                    } else {
                        BatchUtil.logit("Nothing to delete for profile", Constants.INFORMATIONAL, 0);
                    }
                }
            } //end if sitetype = esupport_type

            // Market categlog needs to delete its own entries
            rs.close(); 
            stmt.close(); 
        } catch (SQLException e) {
            throw new BatchException("Failed to delete entries in navgen.gdbm for website " + sitetype + e, Constants.WARNING);
        }
    }

    /**                 
     * This method is called when a FtpDdFileEvent is received. The db files will
     * be ftped to website-appropriate locations. It should report any errors through 
     * the AppSure system.  
     *
     * @param site_name The current website name
     * @return none 
     */
    public void ftpDbFile (String site_name) throws BatchException, IOException {
        //Create the ftp object and get related ftp parameters
        Ftp ftp         = new Ftp(); 
        String host     = BatchInit.getFtpHost(site_name);
        String host_dir = BatchInit.getFtpHostDir(site_name);  
        String loc_dir  = BatchInit.getGdbmFileOutPath(site_name);
        String user_id  = BatchInit.getFtpUserId(site_name); 

        //ftp the db file 
        BatchUtil.logit("Ftp db file at " + loc_dir, Constants.INFORMATIONAL, 0);
        Vector fnames = new Vector();
        if (site_name.compareTo(Constants.WEB_SITE_ESUPPORT_PROFILE)==0) {
            fnames.add(BatchInit.getProfileDbFileName()/*"profileDb.tmp"*/); 
        } else {
            fnames.add(BatchInit.getGdbmFileName()/*db.tmp*/); 
        }
        BatchUtil.ftpFiles (
                            host, 
                            host_dir, 
                            loc_dir, 
                            user_id, 
                            BatchInit.getFtpPassword(site_name),
                            "image", 
                            "put", 
                            fnames); /* "db.tmp" */ 
    }

    /**
     * Finish up by creating db files and ftp the db files to the appropriate
     * locations
     *
     */
    public void finishProcess(BatchEvent e) throws BatchException, IOException {
        String sitename = website.getSiteName(); 
        boolean rc = true; 
        /* As the db file is created through the jni function writeDb, there is no
           need to create the db file from navgen.gdbm table 
        // Create gdbm file for the current website
        boolean rc = createDbFile(sitename); 

        // For esupport site, we need also to build the profile gdbm
        if (sitename.compareTo(Constants.WEB_SITE_ESUPPORT)==0) {
            rc = createDbFile(Constants.WEB_SITE_ESUPPORT_PROFILE); 
        }
        */

        if (rc) {
            // Ftp the db file for the current website
            ftpDbFile(sitename); 

            // For esupport site, we need also to ftp the profile gdbm
            if (sitename.compareTo(Constants.WEB_SITE_ESUPPORT)==0) {
                ftpDbFile(Constants.WEB_SITE_ESUPPORT_PROFILE); 
            }

            // The entries in navgen.gdbm table will not be deleted right away. 
            // The data will be kept until the next running of the buildDB2
            // so that we can examine the previous data anytime in between.

            // Update the corresponding website constants
            try {
                Statement stmt = conn.createStatement();
                ResultSet rs;
                rs = stmt.executeQuery (
                    "UPDATE tigris.constants SET strconst = '" + tmspBefore + "' "+  
                    " WHERE constant = 'TS_BUILDDB2_" + sitename + "'"
                    );
                rs.close(); 
            } catch (SQLException e2) {
                throw new BatchException("Failed to update TS_BUILDDB2 constants for website " + sitename + e2, Constants.WARNING);
            }
        } else {
            BatchUtil.logit("Failed to create db file for " + website + ". Timestamp has not been updated", Constants.FATAL, 0);
        }
    }
    
    /**
     * Define the static page data (used to get TCL template file name for pages
     *  like logon, profile, etc.)
     * 
     * @param conn A db connecion
     * @return none
     */
    public void defineStaticPages(Connection conn) throws WriteGdbmException {
        // Each pagetype requires
        String pages[] = {
          "buildhelp"
          , "help"
          , "profile"
          , "search"
          , "searchhelp"
          , "submit_profile"
          , "tmshelp"
          , "hottips"
          , "helpcenter"
          , "logon"
          , "resetpw"
          , "submit_passwd"
          , "helpwwhc"
        };

        for (int i = 0; i < 13; i++ ) {
          String gkey = pages[i];
          String gval = "template" + NULLSEP + pages[i] + NULLSEP;
          writeGdbm(conn, gkey, gval, dbFileName);
        }
    }
    
    /**
    *  This function defines mktg_suffix (marketing catalog related) value. 
    *  @param Brand 
    *
    *  @return techtips value
    */
    public String techtipsHack(String brand) {
      if ( brand.compareTo("IBM Aptiva")==0 ) {
   	return "aptiva";
      } else if ( brand.compareTo("IBM Monitors")==0) {
    	return "mon";
      } else if ( brand.compareTo("IBM Options")==0) {
    	return "obi";
      } else if ( brand.compareTo("IBM PC")==0 ) {
    	return "pc";
      } else if ( brand.compareTo("IBM PC Server")==0 ) {
    	return "pcsrv";
      } else if ( brand.compareTo("IBM ThinkPad")==0 ) {
    	return "tp";
      } else if ( brand.compareTo("Ambra")==0  ) {
    	return "amb";
      } else {
        return "";
      }
    }

    static {
        String libpath = System.getProperty("java.library.path"); 
        System.out.println("the libpath is " + libpath); 
        BatchUtil.logit("Loading libwriteDb.so ", Constants.INFORMATIONAL, 0);
        System.loadLibrary("writeDb");
    }

    public static native int writeDb(String filename, String gkey, String gval);
    
    /**
     * write into the gdbm file
     */
    public static void writeGdbm(Connection conn, String gkey, String gval, String dbfile) throws WriteGdbmException { 
        BatchUtil.logit("INSERT INTO db file VALUES ("+ "'" + gkey + "'," + dbfile + ",'" + gval + "')", Constants.DEBUG, 0);
        int n = writeDb(dbfile, gkey, gval); 
        if (n!=0) {
            throw new WriteGdbmException(
                "rc = " + n + ". Writing to db file " + dbfile + " failed " 
                + " with \n\tkey: " + gkey + "\n\tvalue: " + gval,
                0); 
        }
    }

    /**
     * for debugging use
     */
    private void logLargeDoc(String msg) {
        if (docIndCount>33000) {
            BatchUtil.logit(msg, Constants.INFORMATIONAL, 0);
        }
    }
} 
