/*
 *                
 * @(#)HtmlDocgen.java	1.0 06/05/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch.plugin;

import java.util.*; 
import java.io.*; 
import java.sql.*; 
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.projects.batch.*;
import com.ibm.nzna.projects.batch.event.*;
import com.ibm.nzna.projects.batch.intf.*; 
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.nzna.projects.batch.exception.*; 
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.util.*; 
import com.ibm.nzna.shared.ftp.FTP.*; 
//import com.ibm.nzna.projects.common.quest.*;
import com.ibm.nzna.projects.common.quest.doc.*; 
import com.ibm.nzna.projects.common.quest.type.*; 

/**
 * @author  Steve Wang                                     
 */

public class HtmlDocgen extends Object implements BatchListener, 
                                                  DocgenIntf {

    /**
     * Website objects
     */
    private WebSite website; 

    /**
     * A db connection
     */
    private Connection conn; 
     
    /**
     * Obtain the current time as db2 timestamp. It will be
     * used to update the docgen timestamp upon finish the run 
     */
    static String tmspBefore = null; 
    /**
     * Updated quest files (docs and graphics) list 
     */ 
    private Vector pubFileList = new Vector(); 

    /**
     * Unpublishable (deleted) docs and graphics list
     */
    private Vector deleteFileList = new Vector();

    /**
     * Counter for published docs
     */
    private int docs_count = 0; 

    /**
     * Counter for published graphics 
     */
    private int graphics_count = 0; 

    /**
     * Counter for the docs ftped
     */
    private int ftp_docs_count = 0;

    /**
     * Counter for the graphics ftped
     */
    private int ftp_graphics_count = 0;
    
    /**
     * Counter for deleted files
     */
    private int files_deleted_count = 0; 

    /**
     * Counter for files to be deleted
     */
    private int files_deleting_count = 0; 

    /** 
     * Get docs count
     *
     * @param none
     * @return int The number of docs published
     */
    public int getDocsCount() {
        return docs_count; 
    }

    /**
     * Get graphics count
     *
     * @param none
     * @return int The number of graphics published
     */
    public int getGraphicsCount() {
        return graphics_count; 
    }

    /**
     * Get ftped docs count
     *
     * @param none
     * @return int The number of docs ftped 
     */
    public int getFtpDocsCount() {
        return ftp_docs_count; 
    }

    /**
     * Get ftped graphics count
     *
     * @param none
     * @return int The number of graphics ftped 
     */
    public int getFtpGraphicsCount() {
        return ftp_graphics_count; 
    }
    
    /**
     * Get deleted files count
     *
     * @param none
     * @return int The number of files deleted 
     */
    public int getFilesDeleteCount() {
        return files_deleted_count; 
    }

    /**
     * Get files to be deleted count
     *
     * @param none
     * @return int The number of files to be deleted 
     */
    public int getFilesDeletingCount() {
        return files_deleting_count; 
    }
    
    /**
     * Print out publishing statistics
     *
     * @param none
     * @retrun none
     */
    public void publishStat() {
        BatchUtil.logit("****************************************************", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("*      " + website.getSiteName() + " " + Docgen.docgen_app_name + " Publish Stat ", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("*         " + tmspBefore, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("****************************************************", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Document Published: " + docs_count, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Images   Published: " + graphics_count, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Document Deleted:   " + files_deleted_count, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Document ftped:     " + ftp_docs_count, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Images   ftped:     " + ftp_graphics_count, Constants.INFORMATIONAL, 0); 
    }
    
    /**
     * Implement the BatchListener methods 
     *
     *  This method should be called when an error
     *  occurs. This error should not be a severe error,
     *  but a simple error, for example, 'Cannot find product 2312-XCX', or 'Cannot find Question 1'
     *
     *  @param errorStr The error string
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logError ( String errorStr ) { 
    }

    /** 
     * logSevereError
     *    This method should log program errors that
     *  prevent operation. For example, 'Cannot Connect To FTP Site'
     *  or 'FTP Site Disconnected'.
     *
     *  @param errorStr The Error message
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logSevereError ( String errorStr ) { 
    }

    /**
     * logSevereError
     *    This method should be called in a catch
     *  block of when an exception happens
     *  
     *
     *  @param e The exception that happened
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logSevereError ( Exception e ) { 
    }

    /**
     * logProgress
     *    This method should be called to notify
     *  progress.
     *  
     *  @param outputStr The output for the progress
     *  @param totalOperations The total number of operations needed to complete the batch
     *  @param operation The current operation that is being done
     *
     *  @return Nothing
     */
    public void logProgress ( String outputStr, int totalOperations, int operation ) { 
    }

    /** batchStart
     *    This method should be called when the
     *  batch starts
     *  
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void batchStart () { 
    }

    /** batchStop
     *    This method should be called when the
     *  batch has finished.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void batchStop () { 
    }

    /**
     * Init function 
     *
     * @param site WebSite object
     * @param init BatchInit object
     * @return none
     */
    public void init (WebSite site, Connection conn) { 
        website   = site; 
        this.conn = conn; 

        // Obtain the current time as db2 timestamp. It will be
        // used to update the docgen timestamp upon finish the run 
        Calendar curr_cal = Calendar.getInstance();
        tmspBefore = BatchUtil.getDb2Timestamp(curr_cal); 
    }                             

    /**
     * Interface function to force the implementer to overwrite. 
     * DocsEvent handler.  
     *
     * @param e DocsEvent object
     * @return none
     */
    public void processDocs (DocsEvent e) throws BatchException, IOException, SQLException { 
        Document doc    = e.getDoc();
        String html_doc = new String(); 
        String fname    = doc.getFilename() + ".html"; 
        
        // Select Document is publishable, webable, having the minimum classification
        // and the last docgen update time for the site is greater than the retrieve 
        // time
        BatchUtil.logit(
             "  changed since " + e.getChangedSince() + "\n"
            +"  last update " + website.getLastDocgenUpdate() 
            + " for website " + website.getSiteName() + "\n" 
            +"  classification " + doc.getDocumentClass(),
            Constants.DEBUG, 0); 

        // First test if the document is changed for the website. 
        if (e.getChangedSince().compareTo(website.getLastDocgenUpdate())<0) {
            BatchUtil.logit("Not published. " + fname + " not changed for " + website.getSiteName(), Constants.DEBUG, 0); 
            return; 
        }

        if (e.webable && 
            //e.show    && 
            e.publish &&
            doc.getDocumentClass() >= website.getMiniClassification()) {

            // Create the html document
            html_doc = toHtml(e);     

            //save the doc to a file
            String path = BatchInit.getHtmlDocOutPath(website.getSiteName());
            BatchUtil.createDoc(path+fname, html_doc); 
            BatchUtil.logit(fname + " created for " + website.getSiteName(), Constants.INFORMATIONAL, 0); 

            //add to the published doc file name to the list
            pubFileList.add(fname);
            BatchUtil.logit(fname + " added to the pub list", Constants.DEBUG, 0); 

            //increase the published doc counter
            docs_count++; 

            //add the associated doc graphics names to the list 
            Vector graphics = doc.getGraphics(); 
            if (graphics!=null) {
                for (int i=0; i<graphics.size(); i++) {
                    String g_name = ((DocumentGraphic)graphics.elementAt(i)).getFilename(); 
                    if (g_name != null) {
                        pubFileList.add(g_name); 
                        BatchUtil.logit("  Assoc. graphic " + g_name + " added to the pub list", Constants.DEBUG, 0); 

                        //increase the published graphics counter
                        graphics_count++; 
                    } else {
                        BatchUtil.logit("  Assoc. graphic has a null filename for " + fname, Constants.WARNING, 0); 
                    }
                }
            }
            
            // attachement names
            // Not needed. 
        } else if (doc.getDocumentClass() >= website.getMiniClassification()) {
            //This doc is non-publishable and should be deleted 
            //Add to the deleted doc file name to the list
            BatchUtil.logit("  " + fname + " unpublishable for " + website.getSiteName() + ". Add to delete list", 
                            Constants.INFORMATIONAL, 0); 
            deleteFileList.add(fname);
            files_deleting_count++; 

            //add the associated doc graphics names to the list 
            Vector graphics = doc.getGraphics(); 
            if (graphics!=null) {
                for (int i=0; i<graphics.size(); i++) {
                    String g_name = ((DocumentGraphic)graphics.elementAt(i)).getFilename(); 
                    if (g_name != null) {
                        deleteFileList.add(g_name); 
                        files_deleting_count++; 
                        BatchUtil.logit("  Assoc. graphic " + g_name + " added to delete list ", Constants.DEBUG, 0); 
                    } else {
                        BatchUtil.logit("  Assoc. delete graphic has a null filename for " + fname, Constants.WARNING, 0); 
                    }
                }
            }
        } else {
            BatchUtil.logit("  " + doc.getFilename() + " unpublishable. Confiential for " + website.getSiteName(), 
                            Constants.INFORMATIONAL, 0); 
        }
    }
    
    /**
     * Finished processing docs
     *
     * @param none
     * @return none
     */
    public void finishDocs() throws BatchException, IOException, SQLException {
        //Create docgen.ftp.list file
        //This file contains all publishable html doc and graphics 
        String sitename   = website.getSiteName();
        String path       = BatchInit.getHtmlDocOutPath(sitename);
        String full_fname = path + Docgen.docgen_app_name + Constants.HTML_FTP_LIST; //"docgen.ftp.list"
        String file_list  = new String(); 
        for (int i=0; i<pubFileList.size(); i++) {
            file_list += (String)(pubFileList.elementAt(i)) + "\n"; 
        }
        BatchUtil.createDoc(full_fname, file_list); 

        //Create docgen.delete.list file
        //This file contains all files (docs and graphics) need to be deleted
        full_fname = path + Docgen.docgen_app_name + Constants.HTML_DELETE_LIST; //"docgen.delete.list";
        file_list  = new String(); 
        for (int i=0; i<deleteFileList.size(); i++) {
            file_list += (String)(deleteFileList.elementAt(i)) + "\n"; 
        }
        BatchUtil.createDoc(full_fname, file_list); 

        //ftp the html docs and the associated graphics 
        try {
            if (pubFileList.size()>0) {
                ftpNewDocs();
            }
        } catch (BatchFtpException ex) {
            // Log it, however, don't terminate the process
            BatchUtil.logit("Ftp html doc failed ", ex, Constants.ERROR, 0); 
        }

        //delete docs/graphics no longer publishable 
        if (deleteFileList.size()>0) {
            files_deleted_count = BatchUtil.deleteDocs(path, deleteFileList); 
        }

        //Update the docgen timestamp for normal docgen (not publishNow)
        if (!Docgen.getPublishNow()) {
            // Obtain the current time as db2 timestamp. It will be
            // used to update the docgen timestamp upon finish the run 
            if (pubFileList.size()>0) {
                BatchUtil.logit("Setting timestamp to " + tmspBefore, Constants.INFORMATIONAL, 0); 
                Statement stmt = conn.createStatement();
                ResultSet rs;
                try {
                    rs = stmt.executeQuery (
                        "UPDATE tigris.constants SET strconst = '" + tmspBefore + "'" +  
                        " WHERE constant = 'TS_DOCGEN_" + sitename + "'"
                        );
                    rs.close();
                    BatchUtil.logit("Set the last Docgen update timestamp to " + tmspBefore + " for " + sitename, Constants.DEBUG, 0); 
                } catch (SQLException e) {
                    BatchUtil.logit("Failed to set TS_DOCGEN_" + sitename+ " for website " + sitename + e, Constants.ERROR, 0); 
                }
            } else {
                BatchUtil.logit("No document published. Timestamp not reset ", Constants.INFORMATIONAL, 0); 
            }
        }

        //Print statistics
        publishStat(); 
    }

    /**
     * Ftp new doc event handling
     *
     * @param none
     * @return none
     */
    public void ftpNewDocs () throws BatchException, IOException {  
        int site_type    = website.getSiteType(); 
        String site_name = website.getSiteName(); 
        
        //Create the ftp object and get related ftp parameters
        Ftp ftp         = new Ftp(); 
        String host     = BatchInit.getFtpHost(site_name);
        String host_dir = BatchInit.getFtpHostDir(site_name);  
        String loc_dir  = BatchInit.getHtmlDocOutPath(site_name);
        String user_id  = BatchInit.getFtpUserId(site_name); 
        
        //Separate the html and image files  
        Vector html_files  = new Vector();
        Vector image_files = new Vector();
        for (int i=0; i<pubFileList.size(); i++) {
            String fn = (String) pubFileList.elementAt(i);
            if (fn.indexOf(".html") != -1) {
                html_files.add(fn); 
            } else {
                image_files.add(fn); 
            }
        }

        //ftp the html docs
        BatchUtil.logit("Ftp quest html files for website " + website.getSiteName(), Constants.INFORMATIONAL, 0);
        ftp_docs_count = BatchUtil.ftpFiles (
                            host, 
                            host_dir, 
                            loc_dir, 
                            user_id, 
                            BatchInit.getFtpPassword(site_name),
                            "ascii", 
                            "put", 
                            html_files);

        //ftp the images files
        String host_image_dir = BatchInit.getHostImageDir(site_name);  
        String loc_image_dir  = BatchInit.getLocalImageDir(site_name);
        BatchUtil.logit("Ftp quest images for website " + website.getSiteName(), Constants.INFORMATIONAL, 0);
        ftp_graphics_count = BatchUtil.ftpFiles (
                            host, 
                            host_image_dir, 
                            loc_image_dir, 
                            user_id, 
                            BatchInit.getFtpPassword(site_name),
                            "image", 
                            "put", 
                            image_files);
    }

    /**
     * Generate html doc from a Document object
     *
     * @param e DocsEvent object
     * @return String The html doc 
     */
    public String toHtml (DocsEvent e) throws BatchException {
        Document doc    = e.getDoc();
        int docTypeInd  = doc.getDocumentType();
        int docClassInd = e.getDocClassind();

	String template = website.getTemplateDescript(docTypeInd);   
        if ( (template == null) || (template.length()==0) ) {
            throw new HtmlTemplateNotFoundException
                ("Html template not found for doctype: " + Integer.toString(docTypeInd) + " " + 
                 "for website " + website.getSiteName(), Constants.ERROR); 
        }

        Calendar today = Calendar.getInstance(); 

	// Simple replacements:
	//    Title, Body, Notes' Document ID, Document Type,
	//    LastUpdate, abstract, description, keywords, location, 
        //    and lastmodified for countries

	template = BatchUtil.substitute( template,  "###TITLE###", doc.getTitle() );

        if (doc.getDocumentBody() != null) {
            template = BatchUtil.substitute( template,  "###MAINBODY###", doc.getDocumentBody().getBody());
        } else {
            template = BatchUtil.substitute( template,  "###MAINBODY###", " ");
            BatchUtil.logit("Get document body failed " + doc.getDocInd(), Constants.ERROR, 0);
        }
	
        template = BatchUtil.substitute( template,  "###DOCID###", doc.getFilename());

        String doctype_desc = website.getDoctypeDescription(docTypeInd);
        if (doctype_desc != null) {
            template = BatchUtil.substitute( template,  "###DOCTYPE###", doctype_desc);
        } else {
            template = BatchUtil.substitute( template,  "###DOCTYPE###", " ");
            BatchUtil.logit("Get doctype desc failed for docTypeInd " + docTypeInd 
                            + " for docind " + doc.getDocInd(), Constants.WARNING, 0);
        }

        if (doc.getLastTouched() != null && doc.getLastTouched().length()>=10) {
            // Excepect getting a timestamp string
            template = BatchUtil.substitute( template,  "###LASTUPDATE###", doc.getLastTouched().substring(0, 11));
        } else {
            template = BatchUtil.substitute( template,  "###LASTUPDATE###", BatchUtil.getCurrTime("y-m-d"));
        }

	template = BatchUtil.substitute( template,  "###LASTPUB###", BatchUtil.getCurrTime("y-m-d") );
	template = BatchUtil.substitute( template,  "###FAXABLE###", ( doc.getFaxable() ? "Yes" : "No" ) );
	template = BatchUtil.substitute( template,  "###DOCSUMMARY###", doc.getEMailComments() );
        template = BatchUtil.substitute( template,  "###YEAR###", BatchUtil.getCurrTime(Calendar.YEAR)); 

        // Curretly only doctypeind = 9 and siteind = 3,4,5 (esupport, infotips
        // and bp) has ###MODEL### in the templates. For BRAND, FAMILY, 
        // MACHINETYPE, the sites are for 1 and 2 (BPARTNER and TECHTIPS) which 
        // do not exist (?). 

	// Brand and Family
        Vector appl_brands = doc.getApplicableBrands();
        Vector appl_family = doc.getApplicableFamilies();
        Vector appl_machine= doc.getApplicableMachines();
        Vector appl_models = doc.getModels();   // todo: use a query to get applicable one

        if (appl_brands != null) {
            template = BatchUtil.substitute( template,  "###BRAND###", BatchUtil.getLists(appl_brands, "", "", ", ") );
        } else {
            template = BatchUtil.substitute( template,  "###BRAND###", " ");
            BatchUtil.logit("Get applicable brands failed " + doc.getDocInd(), Constants.WARNING, 0);
        }

        if (appl_family != null) {
            template = BatchUtil.substitute( template,  "###FAMILY###", BatchUtil.getLists(appl_family, "", "", ", ") );
        } else {
            template = BatchUtil.substitute( template,  "###FAMILY###", " " );
            BatchUtil.logit("Get applcable family failed " + doc.getDocInd(), Constants.WARNING, 0);
        }

	// MachineType, Model
        if (appl_machine != null) {
            template = BatchUtil.substitute( template,  "###MACHINETYPE###", BatchUtil.getLists(appl_machine, "", "", ", ") );
        } else {
            template = BatchUtil.substitute( template,  "###MACHINETYPE###", " ");
            BatchUtil.logit("Get applicable machine failed " + doc.getDocInd(), Constants.WARNING, 0);
        }

        if (appl_models != null) {
            template = BatchUtil.substitute( template,  "###MODEL###", BatchUtil.getLists(appl_models, "", "", ", ") );
        } else {
            template = BatchUtil.substitute( template,  "###MODEL###", " ");
            BatchUtil.logit("Get applcable model failed " + doc.getDocInd(), Constants.WARNING, 0);
        }


	// Special (computed) Tags include:
	//  ###DOCIDNODASH### = DocID w/ No Dash
	//  ###TITLE1### = Title w/ TP expanded to ThinkPad, Brand name
	//     replaced by brand sans 'IBM' and w/ 'Support' appended
	//  ###CLASSIFICATION### = Classification as
	//     - "IBM Confidential" in red
	//     - "IBM Dealer Confidential" in blue (993333) followed by
	//        "This document is provided to IBM and our Business Partners to help
	//         sell and/or service IBM products. It is not to be distributed beyond
	//         that audience or used for any other purpose.<p>"
	//  ###CLASSIFICATIONC### = Classification only if Confidential!
	//  ###COUNTRY### = Geography is Country is all, (if Geo = "", then it should be "World-Wide")

	// ###DOCIDNODASH###
	if ( template.indexOf( "###DOCIDNODASH###" ) != -1 ) {
            String docId = doc.getFilename(); 
	    template = BatchUtil.substitute( template,  "###DOCIDNODASH###", BatchUtil.substitute(docId, "-", "") );
	}

	// ###TITLE1###
	if ( template.indexOf( "###TITLE1###" ) != -1 ) {
	    String tmpTitle = new String(doc.getTitle());
	    BatchUtil.substitute(tmpTitle, " TP ", " ThinkPad " );
	    BatchUtil.substitute(tmpTitle, " IBM ", " " );
	    template = BatchUtil.substitute( template,  "###TITLE1###", tmpTitle );
	}

        // first meta tag search change
        template = BatchUtil.substitute( template,  "###ABSTRACT###", doc.getTitle() );              
        
        // second meta tag search change
        template = BatchUtil.substitute( template,  "###DESCRIPTION###", doc.getTitle() );                                 
        
        //System.out.println (template); 
        //third meta tag search change
        if (doc.getQueryText() != null) {
            template = BatchUtil.substitute( template,  "###KEYWORDS###", doc.getQueryText().getQueryText() );                                   
        } else {
            template = BatchUtil.substitute( template,  "###KEYWORDS###", " " );                                   
            BatchUtil.logit("Get document query text failed " + doc.getDocInd(), Constants.WARNING, 0);
        }
        
        //forth meta tag search change todo-needs to check out dom
        //Currently all we know is the applicable countries for this document. The 
        //Location parameter should be coming from the cgi query string and replaced
        //there. 
        template = BatchUtil.substitute( template,  "###LOCATION###", "" );                                   
        
        //fifth meta tag search change
        template = BatchUtil.substitute( template,  "###LASTMODIFIED###", BatchUtil.getCurrTime("y-m-d") ); 


	// ###CLASSIFICATION###
	if ( template.indexOf( "###CLASSIFICATION###" ) != -1) {
	    String tmpClass = new String();
	    switch (docClassInd) {
		case Constants.IBM_CONFIDENTIAL:
			tmpClass = "<font color=#FF0000>IBM Confidential</font>";
			break;
		case Constants.IBM_DEALER_CONFIDENTIAL:
			tmpClass = "<font size=2 color=#993333><b>This document is provided to IBM and our Business Partners to help ";
			tmpClass += "sell and/or service IBM products. It is not to be distributed beyond ";
			tmpClass += "that audience or used for any other purpose.</b></font><p>";
			break;
		default:
			tmpClass = "";
	    }
            template = BatchUtil.substitute( template,  "###CLASSIFICATION###", tmpClass );
	}

	// ###CLASSIFICATIONC###
	if ( template.indexOf( "###CLASSIFICATIONC###" ) != -1 ) {
	    String tmpClass = new String();
	    switch (docClassInd) {
		case Constants.IBM_CONFIDENTIAL:
			tmpClass = "IBM Confidential";
			break;
		default:
			tmpClass = "";
                        break;
	    }
    	    template = BatchUtil.substitute( template,  "###CLASSIFICATIONC###", tmpClass);
	}

	// ###COUNTRY###  
	if ( template.indexOf( "###COUNTRY###" ) != -1 ) {
            // get document locale
            com.ibm.nzna.projects.common.quest.LocaleRec locr = doc.getLocale(); 
            if (locr != null) {
                template = BatchUtil.substitute( template,  "###COUNTRY###", locr.getLocaleList());
            } else {
                BatchUtil.logit("Get document locale text fail " + doc.getDocInd(), Constants.WARNING, 0);
            }
        }

        // Change the Image location.
        template = BatchUtil.substitute( template,  "###IMAGELOC###", website.getImageLoc());

        // Now changes unique for specific documents
        switch (docTypeInd) {
        case TypeList.DOCTYPE_SSD:
            template = toSSDHtml(doc, template); 
            break;
        case TypeList.DOCTYPE_SERVICE_PUB:
            template = toServicePubHtml(doc, template); 
            break;
        case TypeList.DOCTYPE_PARTS_INFO:
            template = toPartsInfoHtml(doc, template); 
            break;
        case TypeList.DOCTYPE_PRODUCT_INFO:
            template = toProdInfoHtml(doc, template); 
            break;
        case TypeList.DOCTYPE_SERVICE_HINTS_TIPS:
            template = toHintsTipsHtml(doc, template); 
            break;
        case TypeList.DOCTYPE_PRELOAD:
            template = toPreloadHtml(doc, template); 
            break;
        }

	return template;
    }

    /**
     * Generate ssd html doc 
     *
     * @param e DocsEvent object
     * @param tempalte The document template
     * @return String The html doc 
     */
    public String toSSDHtml (Document doc, String template) throws BatchException {
        BatchUtil.logit("Substitiution for SSD doc " + doc.getDocInd(), Constants.DEBUG, 0);
        // Disk Description
        String description = doc.getDescription();
        if (description != null) {
            template = BatchUtil.substitute(template, "###DISKDESC###", BatchUtil.substitute( description, "\n", "<br>" ) );
        } else {
            //template = BatchUtil.substitute(template, "###DISKDESC###", " ");
            BatchUtil.logit("Get document description failed " + doc.getDocInd(), Constants.WARNING, 0);
        }

        // Disk Release Date
        String releaseDate = doc.getReleaseDate();
        if (releaseDate!=null) {
            template = BatchUtil.substitute(template, "###DISKRELEASEDATE###", releaseDate);
        } else {
            BatchUtil.logit("Get document disk release date failed " + doc.getDocInd(), Constants.WARNING, 0);
        }

        // Disk Version
        String version = doc.getVersion();
        if (version != null) {
            template = BatchUtil.substitute(template, "###DISKVERSION###", version);
        } else {
            BatchUtil.logit("Get document disk version failed " + doc.getDocInd(), Constants.WARNING, 0);
        }

        if ( template.indexOf( "###FILETABLEROWS###" ) != -1 ) {
          // format file name and file size as a table
          String fileTableRows = new String();
          Vector files = doc.getFileAttachments(); 
          for (int i=0; i<files.size(); i++) {
            DocumentFile doc_file = (DocumentFile) files.elementAt(i); 
            String url      = doc_file.getURL();
            String fileName = doc_file.getFilename();

            // file name as url
            fileTableRows += "<TR VALIGN=\"TOP\">";
            fileTableRows += "<TD><FONT FACE=\"ARIAL,HELVETICA,GENEVA\" SIZE=2>";
            if (url!=null && url.length() > 0 ) {
              if ( url.indexOf(".pdf") != -1 ) {
                // Open PDF document links in new browser window
                fileTableRows += "<a href=\"" + url + "\" target=\"_blank\">" +
                                 fileName + "</a>";
              } else {
                fileTableRows += "<a href=\"" + url + "\">" + fileName + "</a>";
              }
            } else {
              fileTableRows += fileName;
            }
            fileTableRows += "</FONT></TD>";

            // blank
            fileTableRows += "<TD>&nbsp;</TD>";

            // file size
            fileTableRows += "<TD ALIGN=RIGHT><FONT FACE=\"ARIAL,HELVETICA,GENEVA\" SIZE=2>";

            fileTableRows += insertCommas(String.valueOf(doc_file.getSize()));

            fileTableRows += "</FONT></TD>";

            // blank
            fileTableRows += "<TD>&nbsp;</TD>\n";

            // file description
            fileTableRows += "<TD ALIGN=LEFT><FONT FACE=\"ARIAL,HELVETICA,GENEVA\" SIZE=2>";
            fileTableRows += doc_file.getDescription();
            fileTableRows += "<br><br></FONT></TD>";
            fileTableRows += "</TR>\n";
          }

          if (fileTableRows.length()==0) {
            fileTableRows += "<TR><TD COLSPAN=2>";
            fileTableRows += "<FONT FACE=\"ARIAL,HELVETICA,GENEVA\" SIZE=2>";
            fileTableRows += "<b>No available downloads</b></FONT>";
            fileTableRows += "</TD></TR>";
          }

          template = BatchUtil.substitute(template, "###FILETABLEROWS###", fileTableRows );
          //template = BatchUtil.substitute(template, "###FILEURL###", fileURL );
          //template = BatchUtil.substitute(template, "###ATTACHARC###", attachArc );
          //template = BatchUtil.substitute(template, "###DISKSIZE###", attachArc );
        }

        String addlInfo = doc.getAdditionalInfo();
        if ( addlInfo != null && addlInfo.length()>0) {
          if ( template.indexOf( "###ADDLINFO###" ) != -1 ) {
            template = BatchUtil.substitute(template, "###ADDLINFO###", addlInfo );
          }

          if ( template.indexOf( "###ADDITIONALINFO###" ) != -1 ) {
            String addInfo = "\\n"
              +"<tr>\\n"
              +"  <td colspan=2 valign=top align=left>\\n"
              +"    <table cellspacing=0 cellpadding=0 border=0>\\n"
              +"      <tr>\\n"
              +"        <td align=left valign=top>\\n"
              +"          <font face=\"Arial, Helvetica\" size=2>\\n"
              +"            <em>\\n"
              +"              <u>\\n"
              +"                Additional Information:\\n"
              +"              </u>\\n"
              +"            </em>\\n"
              +"          </font>\\n"
              +"        </td>\\n"
              +"      </tr>\\n"
              +"      <tr>\\n"
              +"        <td align=left valign=top>\\n";
            addInfo += addlInfo;
            addInfo += "\\n"
              +"        </td>\\n"
              +"      </tr>\\n"
              +"    </table>\\n"
              +"  </td>\\n"
              +"</tr>\\n";

            template = BatchUtil.substitute(template, "###ADDITIONALINFO###", addInfo );
          }
        } else {
          template = BatchUtil.substitute(template, "###ADDITIONALINFO###", "" );
          template = BatchUtil.substitute(template, "###ADDLINFO###", "" );
        }

        /*
        String ctryComment = doc.getCountryComment(); 
        if (ctryComment != null && ctryComment.length() > 0 && template.indexOf( "###COUNTRYINFO###")!=-1 ) {
          String tmp = "\\n"
            +"<tr>\\n"
            +"  <td colspan=2 valign=top align=left>\\n"
            +"    <table cellspacing=0 cellpadding=0 border=0>\\n"
            +"      <tr>\\n"
            +"        <td align=left valign=top>\\n"
            +"          <font face=\"Arial, Helvetica\" size=2>\\n"
            +"            <em>\\n"
            +"              <u>\\n"
            +"                Country Specific Information:\\n"
            +"              </u>\\n"
            +"            </em>\\n"
            +"          </font>\\n"
            +"        </td>\\n"
            +"      </tr>\\n"
            +"      <tr>\\n"
            +"        <td align=left valign=top>\\n"
            +"          <font face=\"Arial, Helvetica\" size=2>\\n";
          tmp += ctryComment;
          tmp += "</font></td></tr></table></td></tr>";

          template = BatchUtil.substitute(template, "###COUNTRYINFO###", tmp );
        } else {
          template = BatchUtil.substitute(template, "###COUNTRYINFO###", "" );
        }
        */
        return template; 
    }

    // A private util function for ssd and service pub doc
    private String insertCommas(String str) {
      String ret = new String(); 
      int len = str.length();
      int i=(len-3); 
      for (; i>0; i-=3) {
          ret = "," + str.substring(i, i+3) + ret;
        //str.insert(',', i);
      } 
      return str.substring(0, i+3) + ret;
    }
    
    /**
     * Generate service pub html doc 
     *
     * @param e DocsEvent object
     * @param tempalte The document template
     * @return String The html doc 
     */
    public String toServicePubHtml (Document doc, String template) throws BatchException {
        BatchUtil.logit("Substitiution for Service Pub doc " + doc.getDocInd(), Constants.DEBUG, 0);
        // Disk Description
        String description = doc.getDescription();
        if (description != null) {
            template = BatchUtil.substitute(template, "###DISKDESC###", BatchUtil.substitute( description, "\n", "<br>" ) );
        } else {
            //template = BatchUtil.substitute(template, "###DISKDESC###", " ");
            BatchUtil.logit("Get document description failed " + doc.getDocInd(), Constants.WARNING, 0);
        }

        //  // Disk Release Date
        //  template = BatchUtil.substitute(template, "###DISKRELEASEDATE###", diskReleaseDate.asString("%Y-%m-%d") );
        String releaseDate = doc.getReleaseDate();
        if (releaseDate!=null) {
            template = BatchUtil.substitute(template, "###DISKRELEASEDATE###", releaseDate);
        } else {
            BatchUtil.logit("Get document disk release date failed " + doc.getDocInd(), Constants.WARNING, 0);
        }

        if ( template.indexOf( "###FILETABLEROWS###" ) != -1 ) {
          // format file name and file size as a table
          String fileTableRows = new String();
          String version = new String();
          Vector files = doc.getFileAttachments(); 
          for (int i=0; i<files.size(); i++) {
            DocumentFile doc_file = (DocumentFile) files.elementAt(i); 
            String url      = doc_file.getURL();
            String fileName = doc_file.getFilename();

            // file name as url
            fileTableRows += "<TR VALIGN=\"TOP\">";
            fileTableRows += "<TD><FONT FACE=\"ARIAL,HELVETICA,GENEVA\" SIZE=2>";
            if (url!=null && url.length() > 0 ) {
              if ( url.indexOf(".pdf") != -1 ) {
                // Open PDF document links in new browser window
                fileTableRows += "<a href=\"" + url + "\" target=\"_blank\">" +
                                 fileName + "</a>";
              } else {
                fileTableRows += "<a href=\"" + url + "\">" + fileName + "</a>";
              }
            } else {
              fileTableRows += fileName;
            }
            fileTableRows += "</FONT></TD>";

            // blank
            fileTableRows += "<TD>&nbsp;</TD>";

            // file size
            fileTableRows += "<TD ALIGN=RIGHT><FONT FACE=\"ARIAL,HELVETICA,GENEVA\" SIZE=2>";
            fileTableRows += insertCommas( String.valueOf(doc_file.getSize()) );
            fileTableRows += "</FONT></TD>";

            // blank
            fileTableRows += "<TD>&nbsp;</TD>\n";

            // file description
            fileTableRows += "<TD ALIGN=LEFT><FONT FACE=\"ARIAL,HELVETICA,GENEVA\" SIZE=2>";
            fileTableRows += doc_file.getDescription();
            fileTableRows += "<br><br></FONT></TD>";
            fileTableRows += "</TR>\n";


            // Version is only set once for all files
            //      if (version != "") version += ", ";
            //version += curr.getFileVersion();
            version = doc_file.getVersion();
            if (version != null) {
                template = BatchUtil.substitute(template, "###DISKVERSION###", version);
            } else {
                BatchUtil.logit("Get document disk version failed " + doc.getDocInd(), Constants.WARNING, 0);
            }
          }

          if (fileTableRows == "") {
            fileTableRows += "<TR><TD COLSPAN=2>";
            fileTableRows += "<FONT FACE=\"ARIAL,HELVETICA,GENEVA\" SIZE=2>";
            fileTableRows += "<b>No available downloads</b></FONT>";
            fileTableRows += "</TD></TR>";
          }

          template = BatchUtil.substitute(template, "###FILETABLEROWS###", fileTableRows );
          //template = BatchUtil.substitute(template, "###FILEURL###", fileURL );
          //template = BatchUtil.substitute(template, "###ATTACHARC###", attachArc );
          //template = BatchUtil.substitute(template, "###DISKSIZE###", attachArc );

          template = BatchUtil.substitute(template, "###DISKVERSION###", version);

        } else if ( template.indexOf( "###FILEURL###" )!=-1 ) {

          String fileURL   = new String();
          String attachArc = new String();
          String version   = new String();
          Vector files = doc.getFileAttachments(); 
          for (int i=0; i<files.size(); i++) {
            DocumentFile doc_file = (DocumentFile) files.elementAt(i); 
            String url      = doc_file.getURL();
            String fileName = doc_file.getFilename();
            
            if (url != null) {
              if (fileURL != null) 
                  fileURL += ",<br> ";
              fileURL += "<a href=\"" + url;
              // use target=_blank for PDF document links
              //               to open new browser window
              if ( url.indexOf(".pdf")!=-1) {
                fileURL += "\" target=\"_blank\">" + fileName + "</a>";
              } else {
                fileURL += "\">" + fileName + "</a>";
              }
            }
            if (attachArc != null) { 
                attachArc += ",<br> ";
            }
            attachArc += fileName + " : " + doc_file.getSize();
            if (version.length()==0) {
                version = doc_file.getVersion();
            }
          }
          if (fileURL.length()==0) {
            fileURL = "<b>No available downloads</b>";
          }

          template = BatchUtil.substitute(template, "###FILEURL###", fileURL );

          template = BatchUtil.substitute(template, "###ATTACHARC###", attachArc );

          template = BatchUtil.substitute(template, "###DISKVERSION###", version);

        }

        String addlInfo = doc.getAdditionalInfo();
        if ( template.indexOf( "###ADDLINFO###" ) != -1 ) {
          String addInfo = new String("");
          //      "<tr><td colspan=2 valign=top align=left> \
          // <table cellspacing=0 cellpadding=0 border=0> \
          //  <tr><td align=left valign=top><font face=\"Arial, Helvetica\" size=2><em><u>Additional Information:</u></em></font></td></tr> \
            //  <tr><td align=left valign=top>";
          addInfo += addlInfo;
          //    addInfo += "</td></tr></table></td></tr>";

          template = BatchUtil.substitute(template, "###ADDLINFO###", addInfo );
        }
        return template;
    }

    /**
     * Generate part info html doc 
     *
     * @param e DocsEvent object
     * @param tempalte The document template
     * @return String The html doc 
     */
    public String toPartsInfoHtml (Document doc, String template) throws BatchException {
        // No specific changes right now 
        return template;
    }
    
    /**
     * Generate preloaded  html doc 
     *
     * @param e DocsEvent object
     * @param tempalte The document template
     * @return String The html doc 
     */
    public String toPreloadHtml (Document doc, String template) throws BatchException {
        // No specific changes right now 
        return template;
    }
    
    /**
     * Generate prod info html doc 
     *
     * @param e DocsEvent object
     * @param tempalte The document template
     * @return String The html doc 
     */
    public String toProdInfoHtml (Document doc, String template) throws BatchException {
        // No specific changes right now 
        return template;
    }
    
    /**
     * Generate hints and tips html doc 
     *
     * @param e DocsEvent object
     * @param tempalte The document template
     * @return String The html doc 
     */
    public String toHintsTipsHtml (Document doc, String template) throws BatchException {
        // No specific changes right now 
        return template;
    }
}

