/****************
 *                
 * @(#)SearchDocgen.java	1.0 08/20/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 ****************/

package com.ibm.nzna.projects.batch.plugin;

import java.util.*;
import java.io.*;
import java.sql.*;
import com.ibm.nzna.shared.util.*;    // LogSystem, Text
import com.ibm.nzna.shared.db.*;      // DatabaseSystem
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.projects.batch.*;
import com.ibm.nzna.projects.batch.event.*;
import com.ibm.nzna.projects.batch.intf.*; 
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.nzna.projects.batch.exception.*; 
import com.ibm.nzna.projects.common.quest.doc.*;
import com.ibm.nzna.projects.common.quest.type.*;

/**
 * @author  Mike Rychener 
 * based on skeleton from Steve Wang
 */

/***************
 *   HISTORY
 *    08/19     mdr   created from Steve Wang's HtmlDocgen.java
 *    08/28     mdr   added addlInfo and file descriptions (for SSD's), keywords, etc.
 *    09/21     mdr   revise to be more in line with Steve's new htmldocgen plugin
 *                     - empty out batchStart and put things in init
 *    09/26     mdr   debug / expand error messages; use other logit when exception occurs
 *    09/28     mdr   more info added to error messages; avoid double conflicting messages
 *    10/01     mdr   use iConn = new DatabaseSystem constants, not just DatabaseSystem.QUEST
 *    10/03     mdr   remove dbs DatabaseSystem instance -- not needed, already done by Docgen
 *    10/04     mdr   add a diagnostic message for failed getconnection + warn log if init fail
 *                    more detail in a couple of log messages
 *                    add rollback if sql fails
 *    10/08     mdr   radioactive becomes "localhost" in db connection create's
 *    10/09     mdr   log sql exceptions better
 *                    use docgen.properties for search databases (all within init())
 *    10/12     mdr   update APST tables
 *    10/14     mdr   and revise all queries to use Prepared stmts for efficient loops
 *    10/15-16  mdr   replace setClob with setAsciiStream (we shall see)
 *                    add setAutoCommit(false) to connections
 *    10/17     mdr   change method name in new sqlmethod call
 *         TO DO   --------------
 *    08/28     what about other doc types -- all covered here? (beyond specials for SSD)
 *              part numbers?
 *              need to count errors, warnings, etc. and display at BatchStop time
 *               = fill in publishStat
 *    10/09     what about updating symsearch?  need daily script it appears
 *
 ***************/

public class SearchDocgen extends Object implements BatchListener, 
                                                  DocgenIntf {
    /**
     * Website objects
     **/
    private WebSite website; 

    /**
     * A db connection
     **/
    private Connection conn = null; 
    private int iConn = 0;
     
    /**
     * SQL Method for this thread
     **/
    SQLMethod sqlm = null;
    PreparedStatement stmtQ0 = null;  // 'Q' queries check if document exists in given table
    ResultSet         rsQ0   = null;
    PreparedStatement stmtQ1 = null;
    ResultSet         rsQ1   = null;
    PreparedStatement stmtQ2 = null;
    ResultSet         rsQ2   = null;
    PreparedStatement stmtQ3 = null;
    ResultSet         rsQ3   = null;
    PreparedStatement stmtQ4 = null;
    ResultSet         rsQ4   = null;
    PreparedStatement stmtU0 = null;  // 'U' queries update existing docs
    PreparedStatement stmtU1 = null;
    PreparedStatement stmtU2 = null;
    PreparedStatement stmtU3 = null;
    PreparedStatement stmtU4 = null;
    PreparedStatement stmtI0 = null;  // 'I' queries insert new docs
    PreparedStatement stmtI1 = null;
    PreparedStatement stmtI2 = null;
    PreparedStatement stmtI3 = null;
    PreparedStatement stmtI4 = null;
    PreparedStatement stmtD0 = null;  // 'D' queries delete existing docs
    PreparedStatement stmtD1 = null;
    PreparedStatement stmtD2 = null;
    PreparedStatement stmtD3 = null;
    PreparedStatement stmtD4 = null;
    PreparedStatement stmtB0 = null;  // 'B' query finds list of brands for a document
    ResultSet         rsB0   = null;
    
    /**
     * Obtain the current time as db2 timestamp. It will be
     * used to update the docgen timestamp upon finish the run 
     **/
    String tmspBefore = null; 
    /**
     * Updated quest files (docs and graphics) list 
     */ 
    private Vector pubFileList = new Vector(); 

    /**
     * buffer to hold CLOB being formed
     **/
    private String clobStr = new String();

    /**
     * Counter for published docs, etc.
     */
    private int docs_count = 0; 
    private int dels_count = 0;

    /**************** 
     * Get docs count
     *
     * @param none
     * @return int The number of docs published
     ****************/
    public int getDocsCount() {
        return docs_count; 
    } // end getDocsCount
    
    /****************
     * Print out publishing statistics
     *
     * @param none
     * @retrun none
     ****************/
    public void publishStat() {
    } // end publishStat
    
    /****************
     * Implement the BatchListener methods 
     *
     *  This method should be called when an error
     *  occurs. This error should not be a severe error,
     *  but a simple error, for example, 'Cannot find product 2312-XCX',
     *  or 'Cannot find Question 1'
     *
     *  @param errorStr The error string
     *  @param None
     *  @param None
     *
     *  @return Nothing
     ****************/
    public void logError ( String errorStr ) { 
    } // end logError

    /****************
     * logSevereError
     *    This method should log program errors that
     *  prevent operation. For example, 'Cannot Connect To FTP Site'
     *  or 'FTP Site Disconnected'.
     *
     *  @param errorStr The Error message
     *  @param None
     *  @param None
     *
     *  @return Nothing
     ****************/
    public void logSevereError( String errorStr ) { 
    } // end logSevereError(error string)

    /****************
     * logSevereError
     *    This method should be called in a catch
     *  block of when an exception happens
     *  
     *
     *  @param e The exception that happened
     *  @param None
     *  @param None
     *
     *  @return Nothing
     ****************/
    public void logSevereError ( Exception sev ) { 
    } // end logSevereError(exception)

    /****************
     * logProgress
     *    This method should be called to notify
     *  progress.
     *  
     *  @param outputStr The output for the progress
     *  @param totalOperations The total number of operations needed to complete the batch
     *  @param operation The current operation that is being done
     *
     *  @return Nothing
     ****************/
    public void logProgress ( String outputStr, int totalOperations, int operation ) { 
    } // end logProgress

    /****************
     *  batchStart
     *    This method should be called when the batch starts
     *  
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     ****************/
    public void batchStart () {
    }
    
    /***************
     * Init function 
     *
     * @param site WebSite object
     * @param init BatchInit object
     * @return none
     ***************/
    public void init (WebSite site, Connection conn) { 
        website   = site; 
    
        LogSystem logSystem = BatchInit.logSystem; 

      try {        
        // create database connection, one of the 3 production search databases
        // NO NEED - static: dbs = new DatabaseSystem (Constants.debug, logSystem, true); 
        // DatabaseSystem.addStatusListener (this);
        // DatabaseSystem.addDatabaseSystemListener (this);
        Properties p1 = StandardPropertyLoader.getProperties();
        // the preceding assumes docgen_main() loaded docgen.properties
        if (p1 == null || p1.getProperty("database.ESsearch.dbname") == null
            || p1.getProperty("database.ESsearch.dbname").length() == 0) {
            BatchUtil.logit(
                      "SearchDocgen [" + website.getSiteName()
                      + "] will fail init -- crucial properties missing.",
                      Constants.WARNING,
                      0);
        }
        if (website.getMiniClassification() == Constants.UNCLASSIFIED) {
            iConn = DatabaseSystem.QUESTES1;
            if (!DatabaseSystem.createConnection(iConn, 
                                                 p1.getProperty("database.ESsearch.dbname"),
                                                 p1.getProperty("database.ESsearch.server"), 
                                                 p1.getProperty("database.ESsearch.port"), 
                                                 p1.getProperty("database.ESsearch.userid"), 
                                                 p1.getProperty("database.ESsearch.password"))) {
                this.conn = null;
                iConn = 0;
                throw new BatchException (
                    "SearchDocgen plugin terminating. Database connection failed, \n"
                    +  "  Name: " + p1.getProperty("database.ESsearch.dbname") + "\n"
                    +  "  Server: " + p1.getProperty("database.ESsearch.server") + "\n"
                    +  "  Port: " + p1.getProperty("database.ESsearch.port") + "\n"
                    +  "  User: " + p1.getProperty("database.ESsearch.userid"),
                    Constants.FATAL);
            }
        }
        if (website.getMiniClassification() == Constants.IBM_DEALER_CONFIDENTIAL) {
            iConn = DatabaseSystem.QUESTBP1;
            if (!DatabaseSystem.createConnection(iConn, 
                                                 p1.getProperty("database.BPsearch.dbname"),
                                                 p1.getProperty("database.BPsearch.server"), 
                                                 p1.getProperty("database.BPsearch.port"), 
                                                 p1.getProperty("database.BPsearch.userid"), 
                                                 p1.getProperty("database.BPsearch.password"))) {
                this.conn = null;
                iConn = 0;
                throw new BatchException (
                    "SearchDocgen plugin terminating. Database connection failed, \n"
                    +  "  Name: " + p1.getProperty("database.BPsearch.dbname") + "\n"
                    +  "  Server: " + p1.getProperty("database.BPsearch.server") + "\n"
                    +  "  Port: " + p1.getProperty("database.BPsearch.port") + "\n"
                    +  "  User: " + p1.getProperty("database.BPsearch.userid"),
                    Constants.FATAL); 
            }
        }
        if (website.getMiniClassification() == Constants.IBM_CONFIDENTIAL) {
            iConn = DatabaseSystem.QUESTIT1;
            if (!DatabaseSystem.createConnection(iConn, 
                                                 p1.getProperty("database.ITsearch.dbname"),
                                                 p1.getProperty("database.ITsearch.server"), 
                                                 p1.getProperty("database.ITsearch.port"), 
                                                 p1.getProperty("database.ITsearch.userid"), 
                                                 p1.getProperty("database.ITsearch.password"))) {
                this.conn = null;
                iConn = 0;
                throw new BatchException (
                    "SearchDocGen plugin terminating. Database connection failed, \n"            
                    +  "  Name: " + p1.getProperty("database.ITsearch.dbname") + "\n"
                    +  "  Server: " + p1.getProperty("database.ITsearch.server") + "\n"
                    +  "  Port: " + p1.getProperty("database.ITsearch.port") + "\n"
                    +  "  User: " + p1.getProperty("database.ITsearch.userid"),
                    Constants.FATAL); 
            }
        }
      } catch (Exception inx) {
          BatchUtil.logit(
                    "SearchDocgen [" + website.getSiteName()
                    + "] failed init -- sent exception to error log.",
                    Constants.WARNING,
                    0);
          BatchUtil.logit(
                    "SearchDocgen [" + website.getSiteName() + "] failed init.",
                    inx,
                    Constants.FATAL,
                    0);
          return;
      }

        if (iConn != 0) this.conn = DatabaseSystem.getConnection(iConn);
        if (this.conn != null) {
            try { conn.setAutoCommit(false);  // recommended to avoid thread confusion (dlentz told me)
            } catch (SQLException se0) {
                BatchUtil.logit(
                          "SearchDocgen [" + website.getSiteName() + "] failed setAutoCommit.",
                          se0,
                          Constants.WARNING,
                          0);
            }
            // the following uses the above connection AND closing it will release the connection
            // the close is done in finishDocs() below
            sqlm = new SQLMethod( iConn, "SearchDocgen.init", 10 );
            
            // 'Q' queries check if document exists in given table
            PreparedStatement stmtQ0 = sqlm.createPreparedStatement(
                "select DOCIND from TIGRIS.DOCSEARCH where DOCIND = ?"
                );
            PreparedStatement stmtQ1 = sqlm.createPreparedStatement(
                "select DOCIND from TIGRIS.DOCSEARCHA where DOCIND = ?"
                );
            PreparedStatement stmtQ2 = sqlm.createPreparedStatement(
                "select DOCIND from TIGRIS.DOCSEARCHP where DOCIND = ?"
                );
            PreparedStatement stmtQ3 = sqlm.createPreparedStatement(
                "select DOCIND from TIGRIS.DOCSEARCHS where DOCIND = ?"
                );
            PreparedStatement stmtQ4 = sqlm.createPreparedStatement(
                "select DOCIND from TIGRIS.DOCSEARCHT where DOCIND = ?"
                );
            
            // 'U' queries update existing docs
            PreparedStatement stmtU0 = sqlm.createPreparedStatement(
                "update TIGRIS.DOCSEARCH aa set aa.CBODY = ? , aa.TITLE = ? "
                + "WHERE aa.DOCIND = ? and aa.RECNUM = 1"
                );
            PreparedStatement stmtU1 = sqlm.createPreparedStatement(
                "update TIGRIS.DOCSEARCHA aa set aa.CBODY = ? , aa.TITLE = ? "
                + "WHERE aa.DOCIND = ? and aa.RECNUM = 1"
                );
            PreparedStatement stmtU2 = sqlm.createPreparedStatement(
                "update TIGRIS.DOCSEARCHP aa set aa.CBODY = ? , aa.TITLE = ? "
                + "WHERE aa.DOCIND = ? and aa.RECNUM = 1"
                );
            PreparedStatement stmtU3 = sqlm.createPreparedStatement(
                "update TIGRIS.DOCSEARCHS aa set aa.CBODY = ? , aa.TITLE = ? "
                + "WHERE aa.DOCIND = ? and aa.RECNUM = 1"
                );
            PreparedStatement stmtU4 = sqlm.createPreparedStatement(
                "update TIGRIS.DOCSEARCHT aa set aa.CBODY = ? , aa.TITLE = ? "
                + "WHERE aa.DOCIND = ? and aa.RECNUM = 1"
                );

            // 'I' queries insert new docs
            PreparedStatement stmtI0 = sqlm.createPreparedStatement(
                "insert into TIGRIS.DOCSEARCH "
                + "(DOCIND,RECNUM,LNDOCID,DOCTYPEIND,TITLE,CBODY) VALUES (?,1,?,?,?,?)"
                );
            PreparedStatement stmtI1 = sqlm.createPreparedStatement(
                "insert into TIGRIS.DOCSEARCHA "
                + "(DOCIND,RECNUM,LNDOCID,DOCTYPEIND,TITLE,CBODY) VALUES (?,1,?,?,?,?)"
                );
            PreparedStatement stmtI2 = sqlm.createPreparedStatement(
                "insert into TIGRIS.DOCSEARCHP "
                + "(DOCIND,RECNUM,LNDOCID,DOCTYPEIND,TITLE,CBODY) VALUES (?,1,?,?,?,?)"
                );
            PreparedStatement stmtI3 = sqlm.createPreparedStatement(
                "insert into TIGRIS.DOCSEARCHS "
                + "(DOCIND,RECNUM,LNDOCID,DOCTYPEIND,TITLE,CBODY) VALUES (?,1,?,?,?,?)"
                );
            PreparedStatement stmtI4 = sqlm.createPreparedStatement(
                "insert into TIGRIS.DOCSEARCHT "
                + "(DOCIND,RECNUM,LNDOCID,DOCTYPEIND,TITLE,CBODY) VALUES (?,1,?,?,?,?)"
                );

            // 'D' queries delete existing docs
            PreparedStatement stmtD0 = sqlm.createPreparedStatement(
                "DELETE FROM TIGRIS.DOCSEARCH aa WHERE aa.DOCIND = ? and aa.RECNUM = 1"
            );
            PreparedStatement stmtD1 = sqlm.createPreparedStatement(
                "DELETE FROM TIGRIS.DOCSEARCHA aa WHERE aa.DOCIND = ? and aa.RECNUM = 1"
            );
            PreparedStatement stmtD2 = sqlm.createPreparedStatement(
                "DELETE FROM TIGRIS.DOCSEARCHP aa WHERE aa.DOCIND = ? and aa.RECNUM = 1"
            );
            PreparedStatement stmtD3 = sqlm.createPreparedStatement(
                "DELETE FROM TIGRIS.DOCSEARCHS aa WHERE aa.DOCIND = ? and aa.RECNUM = 1"
            );
            PreparedStatement stmtD4 = sqlm.createPreparedStatement(
                "DELETE FROM TIGRIS.DOCSEARCHT aa WHERE aa.DOCIND = ? and aa.RECNUM = 1"
            );

            // 'B' query finds list of brands for a document
            PreparedStatement stmtB0 = sqlm.createPreparedStatement(
                "SELECT DISTINCT cc.docind,dd.PARENTCATIND "
                + "FROM PRODUCT.TYPECATEGORY dd,TIGRIS.CATEGORY cc,PRODUCT.TYPECATCHILD ee "
                + "WHERE cc.DOCIND = ? AND cc.TYPECATIND = ee.TYPECATIND "
                + "AND ee.PARENTCATIND = dd.TYPECATIND AND dd.CATEGORYLEVEL = 'family'"
                );

        } else BatchUtil.logit("SearchDocgen [" + website.getSiteName()
                               + "] Connection is null.",
                               Constants.WARNING,
                               0);
	BatchUtil.logit("SearchDocgen [" + website.getSiteName() + "] plugin started successfully.",
                        Constants.INFORMATIONAL,
                        0);

    } // end init

    /****************
     * batchStop
     *    This method should be called when the
     *  batch has finished.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     ****************/
    public void batchStop () { 
    } // end batchStop

    /****************
     * Interface function to force the implementer to overwrite. 
     * DocsEvent handler.  
     *
     * @param e DocsEvent object
     * @return none
     ****************/
    public void processDocs (DocsEvent e) throws BatchException,SQLException { 
        Document doc    = e.getDoc();
        String html_doc = new String(); 
        String fname    = doc.getFilename() + ".html"; 
        
        // First test if the document is changed for the website. 
        if (e.getChangedSince().compareTo(website.getLastDocgenUpdate()) < 0) {
            BatchUtil.logit("SearchDocgen[" + website.getSiteName() +
                            "]: Not published on this website. " + fname + " not changed for "
                            + website.getSiteName() + " -- " + e.getChangedSince()
                            + " is before " + website.getLastDocgenUpdate(),
                            Constants.INFORMATIONAL,
                            0); 
            return; 
        }
        if (conn == null) return;
        if (doc.getWebable() && 
            !doc.getRecycled() &&         // same as publishable??
            doc.getDocumentClass() >= website.getMiniClassification()) {
            // Document is publishable on this site,
            // so form the CLOB to be indexed:
	    // title & keywords x 4
            clobStr = doc.getTitle();
            if (doc.getQueryText() != null) clobStr += " " + doc.getQueryText();                                   

            clobStr += " " + clobStr;
            clobStr += " " + clobStr;  // that is two doublings to get 4 replications
	    // lndocid (two ways)
            String fileStr = doc.getFilename(); // with hyphen, if any
            int hyp = fileStr.indexOf("-");  // and without hyphen:
            if (hyp > 0) {  // assume hyphen is not the first character
                fileStr += " " + fileStr.substring(0, hyp) + fileStr.substring(hyp+1);
            }
            fileStr += " " + fileStr;
            fileStr += " " + fileStr;  // weighted 4 times, like title
            clobStr += " " + fileStr;
	    // summary
            clobStr += " " + doc.getEMailComments();

        // partnum (if any) ??
		
            if (doc.getDocumentType() == TypeList.DOCTYPE_SSD) { // SSD descriptions:
                clobStr += " " + doc.getDescription();
                Vector files = doc.getFileAttachments(); 
                for (int i=0; i<files.size(); i++) {
                    DocumentFile doc_file = (DocumentFile) files.elementAt(i); 
                    String fileName = doc_file.getFilename();  // add full file name
                    int dotp = fileName.indexOf(".");  // also add file name without ext.
                    if (dotp > 0) { fileName += " " + fileName.substring(0, dotp); }
                    fileName += " " + fileName;
                    fileName += " " + fileName;  // weight filename like title is above, x 4
                    clobStr += " " + fileName;
                    clobStr += " " + doc_file.getDescription();
                    } // end for files
            } // end SSD descriptions

            // addinfo (if any)
            String addlInfo = doc.getAdditionalInfo();
            if ( addlInfo != null && addlInfo.length()>0) clobStr += " " + addlInfo;

            // add in docbody rows (if any) (loop while they last -- was limited to 15)
            clobStr += " " + doc.getDocumentBody().getBody();

      	    try {
                FileUtil.saveFile("tempdocbody.txt", clobStr);
                // would prefer something like: InputStream clobInput = new StringReader(clobStr);
                FileInputStream clobInput = new FileInputStream("tempdocbody.txt");
                // first see if the row is there -- if so, update; if not, insert;
                // would anyone ever change doctypeind or lndocid here? -- assume not
                stmtQ0.setInt(1, doc.getDocInd());
                rsQ0 = stmtQ0.executeQuery();
                if (rsQ0 == null) {
                    stmtI0.setInt(1, doc.getDocInd());
                    stmtI0.setString(2, doc.getFilename());
                    stmtI0.setInt(3, doc.getDocumentType());
                    stmtI0.setString(4, doc.getTitle());
                    stmtI0.setAsciiStream(5, clobInput, clobStr.length());
                    stmtI0.executeUpdate();
                } else {
                    stmtU0.setAsciiStream(1, clobInput, clobStr.length());
                    stmtU0.setString(2, doc.getTitle());
                    stmtU0.setInt(3, doc.getDocInd());
                    stmtU0.executeUpdate();
                }
                rsQ0.close();
                // need to put change in appropriate brand table too! (if any)
                // get a list of brands for this doc:
                stmtB0.setInt(1, doc.getDocInd());
                rsB0 = stmtB0.executeQuery();
                // -- if that result set not empty, possibly update DOCSEARCHx:
                while (rsB0.next()) {
                    if (rsB0.getInt(2) == 3) {         // update docsearcha row (Aptiva)
                        stmtQ1.setInt(1, doc.getDocInd());
                        rsQ1 = stmtQ1.executeQuery();
                        if (rsQ1 == null) {
                            stmtI1.setInt(1, doc.getDocInd());
                            stmtI1.setString(2, doc.getFilename());
                            stmtI1.setInt(3, doc.getDocumentType());
                            stmtI1.setString(4, doc.getTitle());
                            stmtI1.setAsciiStream(5, clobInput, clobStr.length());
                            stmtI1.executeUpdate();
                        } else {
                            stmtU1.setAsciiStream(1, clobInput, clobStr.length());
                            stmtU1.setString(2, doc.getTitle());
                            stmtU1.setInt(3, doc.getDocInd());
                            stmtU1.executeUpdate();
                        }
                        rsQ1.close();
                    }
                    else if (rsB0.getInt(2) == 4 || rsB0.getInt(2) == 7) {
                        // update docsearchp row -- PC | IntelliStation
                        stmtQ2.setInt(1, doc.getDocInd());
                        rsQ2 = stmtQ2.executeQuery();
                        if (rsQ2 == null) {
                            stmtI2.setInt(1, doc.getDocInd());
                            stmtI2.setString(2, doc.getFilename());
                            stmtI2.setInt(3, doc.getDocumentType());
                            stmtI2.setString(4, doc.getTitle());
                            stmtI2.setAsciiStream(5, clobInput, clobStr.length());
                            stmtI2.executeUpdate();
                        } else {
                            stmtU2.setAsciiStream(1, clobInput, clobStr.length());
                            stmtU2.setString(2, doc.getTitle());
                            stmtU2.setInt(3, doc.getDocInd());
                            stmtU2.executeUpdate();
                        }
                        rsQ2.close();
                    }
                    else if (rsB0.getInt(2) == 8) { // update docsearchs row -- server
                        stmtQ3.setInt(1, doc.getDocInd());
                        rsQ3 = stmtQ3.executeQuery();
                        if (rsQ3 == null) {
                            stmtI3.setInt(1, doc.getDocInd());
                            stmtI3.setString(2, doc.getFilename());
                            stmtI3.setInt(3, doc.getDocumentType());
                            stmtI3.setString(4, doc.getTitle());
                            stmtI3.setAsciiStream(5, clobInput, clobStr.length());
                            stmtI3.executeUpdate();
                        } else {
                            stmtU3.setAsciiStream(1, clobInput, clobStr.length());
                            stmtU3.setString(2, doc.getTitle());
                            stmtU3.setInt(3, doc.getDocInd());
                            stmtU3.executeUpdate();
                        }
                        rsQ3.close();
                    }
                    else if (rsB0.getInt(2) == 10) { // update docsearcht row -- ThinkPad
                        stmtQ4.setInt(1, doc.getDocInd());
                        rsQ4 = stmtQ4.executeQuery();
                        if (rsQ4 == null) {
                            stmtI4.setInt(1, doc.getDocInd());
                            stmtI4.setString(2, doc.getFilename());
                            stmtI4.setInt(3, doc.getDocumentType());
                            stmtI4.setString(4, doc.getTitle());
                            stmtI4.setAsciiStream(5, clobInput, clobStr.length());
                            stmtI4.executeUpdate();
                        } else {
                            stmtU4.setAsciiStream(1, clobInput, clobStr.length());
                            stmtU4.setString(2, doc.getTitle());
                            stmtU4.setInt(3, doc.getDocInd());
                            stmtU4.executeUpdate();
                        }
                        rsQ4.close();
                    }
                } // end while rsB0.next()
                rsB0.close();
                clobInput.close();
            } catch (Exception e1) {
                BatchUtil.logit(
                    "SearchDocgen[" + website.getSiteName() + "]: Failed to update row for "
                    + fname, Constants.WARNING, 0);
                BatchUtil.logit(
                    "SearchDocgen [" + website.getSiteName() + "] update query failed -- "
                    + fname,
                    e1,
                    Constants.WARNING,
                    0);
                sqlm.rollBack();
                return;
            }
            docs_count++; 
            BatchUtil.logit("SearchDocgen[" + website.getSiteName() + "]: Updated row for "
                            + fname + " with CLOB: " + clobStr.length() + " characters.",
                            Constants.INFORMATIONAL, 0);
        } // end if doc publishable on this site
        else if (doc.getDocumentClass() >= website.getMiniClassification()) {
            //This doc is non-publishable and should be deleted 
            try {
                // following will get exception if row not already there
                stmtD0.setInt(1, doc.getDocInd());
                stmtD0.executeUpdate();
                // get a list of brands for this doc:
                stmtB0.setInt(1, doc.getDocInd());
                rsB0 = stmtB0.executeQuery();
                // need to delete from appropriate brand table too! (if any)
                while (rsB0.next()) {
                    if (rsB0.getInt(2) == 3) {         // update docsearcha row (Aptiva)
                        stmtD1.setInt(1, doc.getDocInd());
                        stmtD1.executeUpdate();
                    } else if (rsB0.getInt(2) == 4 || rsB0.getInt(2) == 7) {
                        // update docsearchp row -- PC | IntelliStation
                        stmtD2.setInt(1, doc.getDocInd());
                        stmtD2.executeUpdate();
                    } else if (rsB0.getInt(2) == 8) { // update docsearchs row -- server
                        stmtD3.setInt(1, doc.getDocInd());
                        stmtD3.executeUpdate();
                    } else if (rsB0.getInt(2) == 10) { // update docsearcht row -- ThinkPad
                        stmtD4.setInt(1, doc.getDocInd());
                        stmtD4.executeUpdate();
                    }
                } // end while rsB0.next()
                rsB0.close();
            } catch (Exception e2) { 
                BatchUtil.logit( 
                    "SearchDocgen[" + website.getSiteName() + "]: Failed to delete row for "
                    + fname, Constants.WARNING, 0); 
                BatchUtil.logit(
                    "SearchDocgen [" + website.getSiteName() + "] delete query failed -- "
                    + fname,
                    e2,
                    Constants.WARNING,
                    0);
                sqlm.rollBack();
                return;
            }
 	    BatchUtil.logit("SearchDocgen[" + website.getSiteName() + "]: Deleted row for "
                            + fname, Constants.INFORMATIONAL, 0);
            dels_count++; 
        } // end else if non-publishable (delete from this site)
    } // end processDocs
    
    /****************
     * Finished processing docs
     *
     * @param none
     * @return none
     ****************/
    public void finishDocs() throws BatchException {
      if (sqlm != null) {
        try {
            stmtQ0.close();  // closing these will commit the changes
            stmtQ1.close();
            stmtQ3.close();
            stmtQ3.close();
            stmtQ4.close();
            stmtI0.close();
            stmtI1.close();
            stmtI3.close();
            stmtI3.close();
            stmtI4.close();
            stmtU0.close();
            stmtU1.close();
            stmtU3.close();
            stmtU3.close();
            stmtU4.close();
            stmtD0.close();
            stmtD1.close();
            stmtD3.close();
            stmtD3.close();
            stmtD4.close();
            stmtB0.close();
            // close the connection to the database (one of 3) -- implicit in closing sqlMethod
            sqlm.close();
        } catch (SQLException se1) {
            BatchUtil.logit(
                      "SearchDocgen [" + website.getSiteName() + "] failed stmtxx.close()",
                      se1,
                      Constants.WARNING,
                      0);
        }
      }
	// summarize what was done for each of the 3
	// display count: warnings, errors, etc.
        BatchUtil.logit("SearchDocgen [" + website.getSiteName() + "] plugin processed "
                        + docs_count + " documents.",
                        Constants.INFORMATIONAL, 0);
        //Print statistics
        publishStat(); 
    } // end finishDocs
    
    /**
     * Ftp new doc event handling
     *
     * @param none
     * @return none
     */
    public void ftpNewDocs () throws BatchException, IOException {  
        // empty for now
    }

} // end class

