/*
 *                
 * @(#)XmlDocgen.java	1.0 06/05/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch.plugin;

import java.util.*; 
import java.io.*; 
import java.sql.*; 
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.projects.batch.*;
import com.ibm.nzna.projects.batch.event.*;
import com.ibm.nzna.projects.batch.intf.*; 
import com.ibm.nzna.projects.batch.utils.*; 
import com.ibm.nzna.projects.batch.exception.*; 
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.util.*; 
import com.ibm.nzna.shared.ftp.FTP.*; 
//import com.ibm.nzna.projects.common.quest.*;
import com.ibm.nzna.projects.common.quest.doc.*; 
import com.ibm.nzna.projects.common.quest.type.*; 

/**
 * @author  Steve Wang                                     
 */

public class XmlDocgen extends Object implements BatchListener, 
                                                 DocgenIntf {
    /**
     * Website object
     */
    private WebSite website; 

    /**
     * A db connection
     */
    private Connection conn; 

    /**
     * Updated xml files list 
     */ 
    private Vector pubFileList = new Vector(); 

    /**
     * Unpublishable (deleted) xml list
     */
    private Vector deleteFileList = new Vector();

    /**
     * Counter for published docs
     */
    private int docs_count = 0; 

    /**
     * Counter for the docs ftped
     */
    private int ftp_docs_count = 0;

    /**
     * Counter for deleted files
     */
    private int files_deleted_count = 0; 

    /**
     * Counter for files to be deleted
     */
    private int files_deleting_count = 0; 

    /** 
     * Get docs count
     * @param none
     * @return int The number of docs published
     */
    public int getDocsCount() {
        return docs_count; 
    }

    /**
     * Get ftped docs count
     * @param none
     * @return int The number of docs ftped 
     */
    public int getFtpDocsCount() {
        return ftp_docs_count; 
    }

    /**
     * Get deleted files count
     * @param none
     * @return int The number of files deleted 
     */
    public int getFilesDeleteCount() {
        return files_deleted_count; 
    }

    /**
     * Get files to be deleted count
     * @param none
     * @return int The number of files to be deleted 
     */
    public int getFilesDeletingCount() {
        return files_deleting_count; 
    }
    
    /**
     * Print out publishing statistics
     * @param none
     * @retrun none
     */
    public void publishStat() {
        BatchUtil.logit("****************************************************", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("*     " + website.getSiteName() + " " + Docgen.docgen_app_name + " Xml Publish Stat ", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("*        " + HtmlDocgen.tmspBefore, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("****************************************************", Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Xml Doc Published: " + docs_count, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Xml Doc Deleted:   " + files_deleted_count, Constants.INFORMATIONAL, 0); 
        BatchUtil.logit("Xml Doc ftped:     " + ftp_docs_count, Constants.INFORMATIONAL, 0); 
    }
    
    /**
     * Implement the BatchListener methods 
     *
     *  This method should be called when an error
     *  occurs. This error should not be a severe error,
     *  but a simple error, for example, 'Cannot find product 2312-XCX', or 'Cannot find Question 1'
     *
     *  @param errorStr The error string
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logError ( String errorStr ) { 
    }

    /** 
     * logSevereError
     *    This method should log program errors that
     *  prevent operation. For example, 'Cannot Connect To FTP Site'
     *  or 'FTP Site Disconnected'.
     *
     *  @param errorStr The Error message
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logSevereError ( String errorStr ) { 
    }

    /**
     * logSevereError
     *    This method should be called in a catch
     *  block of when an exception happens
     *  
     *
     *  @param e The exception that happened
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void logSevereError ( Exception e ) { 
    }

    /**
     * logProgress
     *    This method should be called to notify
     *  progress.
     *  
     *  @param outputStr The output for the progress
     *  @param totalOperations The total number of operations needed to complete the batch
     *  @param operation The current operation that is being done
     *
     *  @return Nothing
     */
    public void logProgress ( String outputStr, int totalOperations, int operation ) { 
    }

    /** batchStart
     *    This method should be called when the
     *  batch starts
     *  
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void batchStart () { 
    }

    /** batchStop
     *    This method should be called when the
     *  batch has finished.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     */
    public void batchStop () { 
    }

    /**
     * init function 
     */
    public void init (WebSite site, Connection conn) { 
        website   = site; 
        this.conn = conn; 
    }

    /**
     * Interface function to force the implementer to overwrite. 
     * DocsEvent handler.  
     */
    public void processDocs (DocsEvent e) throws BatchException, IOException, SQLException { 
        Document doc   = e.getDoc();
        String xml_doc = new String(); 
        String fname   = doc.getFilename() + ".xml"; 

        // First test if the document is changed for the website. 
        if (e.getChangedSince().compareTo(website.getLastDocgenUpdate())<0) {
            BatchUtil.logit("  Not published. " + fname + " not changed for " + website.getSiteName(), Constants.DEBUG, 0); 
            return; 
        }

        // Select Document is publishable, webable, having the minimum classification
        // and the last docgen update time for the site is greater than the retrieve 
        // time
        if (e.webable && 
            //e.show && 
            e.publish && 
            doc.getDocumentClass() >= website.getMiniClassification()) {
            //generate the xml file for the publishable docs
            xml_doc = toXml(e); 

            //save the doc to a file
            String path  = BatchInit.getXmlDocOutPath(website.getSiteName());
            BatchUtil.createDoc(path+fname, xml_doc); 

            //add to the published doc file name to the list
            pubFileList.add(fname);
            BatchUtil.logit(fname + " added to the pub list", Constants.DEBUG, 0); 

            //increase the published doc counter
            docs_count++; 
        } else if (doc.getDocumentClass() >= website.getMiniClassification()) {
            //This doc is non-publishable and should be deleted 
            //Add to the deleted doc file name to the list
            deleteFileList.add(fname);
            files_deleting_count++; 
            BatchUtil.logit("  " + fname + " unpublishable. Add to delete list", 
                            Constants.DEBUG, 0); 
        } else {
            BatchUtil.logit("  " + doc.getFilename() + " xml doc unpublishable. Confiential for " + website.getSiteName(), 
                            Constants.DEBUG, 0); 
        }
    }

    /**
     * Finished processing docs
     * @param none
     * @return none
     */
    public void finishDocs() throws BatchException, IOException, SQLException {
        String path = BatchInit.getXmlDocOutPath(website.getSiteName());
        
        //Create docgen.ftp.xmllist file
        //This file contains all publishable xml doc 
        String sitename   = website.getSiteName();
        String full_fname = path + Docgen.docgen_app_name + Constants.XML_FTP_LIST; //"docgen.ftp.xmllist";
        String file_list  = new String(); 
        for (int i=0; i<pubFileList.size(); i++) {
            file_list += (String)(pubFileList.elementAt(i)) + "\n"; 
        }
        BatchUtil.createDoc(full_fname, file_list); 

        //Create docgen.delete.xmllist file
        //This file contains all xml files need to be deleted
        full_fname = path + Docgen.docgen_app_name + Constants.XML_DELETE_LIST; //"docgen.delete.xmllist";
        file_list  = new String(); 
        for (int i=0; i<deleteFileList.size(); i++) {
            file_list += (String)(deleteFileList.elementAt(i)) + "\n"; 
        }
        BatchUtil.createDoc(full_fname, file_list); 

        //ftp the xml docs - currently we ftp xml to Lionbridge with 
        //another cron job. 
        //ftpNewDocs();

        //delete docs/graphics no longer publishable 
        files_deleted_count = BatchUtil.deleteDocs(path, deleteFileList); 
  
        //Print statistics
        publishStat(); 
    }

    /**
     * Ftp new doc event handling
     */
    public void ftpNewDocs () throws BatchException, IOException {  
        int site_type    = website.getSiteType(); 
        String site_name = website.getSiteName(); 
        
        //Create the ftp object and get related ftp parameters
        Ftp ftp         = new Ftp(); 
        String host     = BatchInit.getXmlFtpHost(site_name);
        String host_dir = BatchInit.getXmlFtpHostDir(site_name);  
        String loc_dir  = BatchInit.getXmlDocOutPath(site_name);
        String user_id  = BatchInit.getFtpUserId(site_name); 
        
        //ftp the xml docs
        BatchUtil.logit("Ftp quest xml files for website " + website.getSiteName(), Constants.INFORMATIONAL, 0);
        ftp_docs_count = BatchUtil.ftpFiles (
                            host, 
                            host_dir, 
                            loc_dir, 
                            user_id, 
                            BatchInit.getFtpPassword(site_name),
                            "ascii", 
                            "put", 
                            pubFileList);
    }

    /**
     * Generate xml doc from a Document object
     */
    public String toXml (DocsEvent e) throws BatchException, IOException {
        Document doc    = e.getDoc();
        int docTypeInd  = e.getDocTypeind();
        int docClassInd = e.getDocClassind();

        String template = BatchInit.getXmlTemplate(website.getSiteName());
        if (template == null || template.length()==0) {
            throw new XmlTemplateNotFoundException ("Xml template not found from property file", Constants.ERROR); 
        }
        
        template = BatchUtil.substitute( template,  "###CREATION_DATE###", BatchUtil.getCurrTime("m-d-y"));
        template = BatchUtil.substitute( template,  "###DOCID###", doc.getFilename());
        template = BatchUtil.substitute( template,  "###HTML_FILE_NAME###",  doc.getFilename() + ".html");
        template = BatchUtil.substitute( template,  "###PUBLISH_DATE###", doc.getLastTouched());
        template = BatchUtil.substitute( template,  "###TITLE###", doc.getTitle());
        template = BatchUtil.substitute( template,  "###DOCTYPE###", website.getDoctypeDescription(docTypeInd));

        // Write document classification
        String docClassification = new String();
        switch (docClassInd) {
          case Constants.IBM_CONFIDENTIAL:
            docClassification = "IBM Confidential";
            break;
          case Constants.IBM_DEALER_CONFIDENTIAL:
            docClassification = "Dealer Confidential";
            break;
          case Constants.UNCLASSIFIED:
            docClassification = "Unclassified";
            break;
          default:
            docClassification = "Unknown";
            break;
        }
        template = BatchUtil.substitute( template, "###CLASSIFICATION###", docClassification);

        // Get list of geographies
        com.ibm.nzna.projects.common.quest.LocaleRec locr = doc.getLocale(); 
        template = BatchUtil.substitute( template,  "###GEO###", locr.getLocaleList());

        // Generate Machines/Options data
        String machines = new String();
        String options  = new String(); 

        Vector brands   = doc.getApplicableBrands();
        Vector families = doc.getApplicableFamilies();
        Vector types    = doc.getApplicableMachines();
        Vector models   = doc.getModels(); 
        String brand  = "";
        String family = "";
        String type   = "";
        String model  = "";
        if (brands!=null) {
            for (int i=0; i<brands.size(); i++) {
               brand  = ((TypeCategoryRec)brands.elementAt(i)).toString();
               try {
                   family = ((TypeCategoryRec)families.elementAt(i)).toString();
               } catch (Exception e1) {
                   BatchUtil.logit("  getApplicableFamilies error", Constants.INFORMATIONAL, 0);
               }
               try {
                   type   = ((TypeCategoryRec)types.elementAt(i)).toString();
               } catch (Exception e2) {
                   BatchUtil.logit("  getApplicableTypes error", Constants.INFORMATIONAL, 0);
               }
               try {
                   model  = ((TypeCategoryRec)models.elementAt(i)).toString(); 
               } catch (Exception e3) {
                   BatchUtil.logit("  getModels error", Constants.INFORMATIONAL, 0);
               }

               if (brand.compareTo("IBM Options")==0) {
                 options += "\t<OPTION>\n";
                 options += "\t\t<BRAND>"       + brand  + "</BRAND>\n";
                 options += "\t\t<SUPCATEGORY>" + family + "</SUPCATEGORY>\n";
                 options += "\t\t<SUBCATEGORY>" + type   + "</SUBCATEGORY>\n";
                 options += "\t\t<PARTNUMBER>"  + model  + "</PARTNUMBER>\n";
                 options += "\t</OPTION>\n";
               } else {
                 machines += "\t<MACHINE>\n";
                 machines += "\t\t<BRAND>"       + brand  + "</BRAND>\n";
                 machines += "\t\t<FAMILY>"      + family + "</FAMILY>\n";
                 machines += "\t\t<MACHINETYPE>" + type   + "</MACHINETYPE>\n";
                 machines += "\t\t<MODEL>"       + model  + "</MODEL>\n";
                 machines += "\t</MACHINE>\n";
               }
            }
        } else {
            BatchUtil.logit("  getApplicableBrands returns null", Constants.INFORMATIONAL, 0);
        }

        template = BatchUtil.substitute( template, "###MACHINES###", machines);
        template = BatchUtil.substitute( template, "###OPTIONS###",  options);

        //History
        template = BatchUtil.substitute( template, "###HISTORY###", doc.getEMailComments());  

        return template;
    }
}

