package com.ibm.nzna.projects.batch.prodgen;
import java.sql.*;
import java.util.*;
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.mail.*;
import com.ibm.nzna.projects.common.quest.product.*;  
import com.ibm.nzna.projects.common.quest.type.TypeCategory;  
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.db.*;
import java.io.*;
import java.lang.*;
//import com.ibm.appsure;


/* -------------------------------------------------------------- */
/* Misfits MisfitsMailer                                                    */
/*                                                                */
/* @(#) Misfits MisfitsMailer.java	       
/* This batch will send out email to bpms when misfit products are created */
/* @version 1.0 04/17/2000
 /* @author Tracey Watkins ,Souvik Dutta   */
/*  @Modified 9/05/01			    */
/******************************************************************/


public class MisfitsMailer implements Runnable {
    private Mail        mail                    =  null;
    private String      mailHost            =  null;
    private String      mailFrom           =  null;
    private boolean   okToContinue    =  true;
    private Vector     bpmList              =  null;
    private Vector     misfitList             =  null;
    private String      questdbHost        =  null;
    private String      questdbName      =  null;
    private String      questdbUser        =  null;
    private String      questdbPassword =  null;
    private String      questdbPort         =  null;

    static ProdGenInit prodGenInit = null;

    /**
     * Constructor .
     *
     * 	This method will instantiate the class.
     *  @param	ProdGenInit  The batch that instantiates this class.
     *  @return	Nothing						                                     
    **/
    public MisfitsMailer ( ProdGenInit prodGenInit) {
        try {
            this.prodGenInit = prodGenInit;
            prodGenInit.logFile.beginLogSection("MisfitsMailer");
            questdbName = prodGenInit.props.getProperty("db.quest.name");
            questdbUser = prodGenInit.props.getProperty("db.quest.uid");
            questdbPassword = prodGenInit.props.getProperty("db.quest.pwd");
            questdbHost = prodGenInit.props.getProperty("db.quest.host");
            questdbPort = prodGenInit.props.getProperty("db.quest.port");
            mailHost=prodGenInit.props.getProperty ("mail.host");
            prodGenInit.logFile.log("Mail host: " + mailHost);
            mailFrom =prodGenInit.props.getProperty ("mail.from");
            prodGenInit.logFile.log("Mail sender:  " + mailFrom);
            bpmList = new Vector();
            misfitList = new Vector();
            misfitList = prodGenInit.misfitsVec;
            prodGenInit.logFile.log("There are " + misfitList.size() + " misfits to be processed by MisfitsMailer.");
            mail = new Mail(mailFrom, mailHost);
            prodGenInit.logFile.log("Mail instantiated.");
            run();
        } catch ( Exception e ) {
            prodGenInit.logFile.logError(e.getMessage());
            okToContinue=false;
        }
    }   //end of MisfitsMailer constructor


    /**
     * createBPMList(ResultSet)
     * This method processes the all the rows of the result of an SQLQuery.<p>
     *
     *  @param ResultSet ResultSet of the SQLQuery that is to be processed.
     *  @return Nothing
     **/
    public boolean createBPMList(ResultSet r) {
        prodGenInit.logFile.log ( "Creating the BPM List."); 
        BPM bpm = null;
        try {
            while ( r.next() ) {
                bpm = new BPM();
                bpm.userId = r.getString(1).trim();
                bpm.name = r.getString(2).trim();
                bpm.emailAddr = r.getString(3).trim();
                bpm.brandInd  = r.getInt(4);
                bpm.brandDesc = r.getString(5).trim();
                prodGenInit.logFile.log("User: " + bpm.userId + "\tEMail: " + bpm.emailAddr + " \tBrandInd: " + bpm.brandInd +
                                        "\tDescription: " + bpm.brandDesc );
                bpmList.addElement(bpm);
            } 
            prodGenInit.logFile.log("There are " + bpmList.size()+ " BPMs in the list. ");
        } //end of try
        catch ( SQLException e ) {
            prodGenInit.logFile.logSevereError("Error in Misfits Mailer- creating BPM List");
            return( false );
        } //end of catch
        return( true );
    } //end of method createBPMList(ResultSet)

    /**
     * procesResults()
     * This method processes the all the rows of the result and sends appropriate mail notification.<p>
     *
     *  @param None
     *  @return Nothing
     **/
    public void processResults() {
        prodGenInit.logFile.log ( "Processing misfits..."); 
        BPM  bpm = new BPM();
//        prodGenInit.logFile.log("BPM instantiated.");
        Misfit misfit = new Misfit();
//        prodGenInit.logFile.log("Misfit instantiated.");
        Enumeration bpms = null;
        Enumeration misfits = null;

        try {
            if ( misfitList.isEmpty() ) {
                prodGenInit.logFile.log("There are no misfits to process at this time.");
//                return;
            }
            bpms = bpmList.elements();

            while ( bpms.hasMoreElements() ) {
                prodGenInit.logFile.log("Getting the next BPM.");
                bpm = (BPM) bpms.nextElement();
                String subject = "Product Misfit Items for OPIC/M Brand:   " + bpm.brandDesc;
                StringBuffer messageDetails =  new StringBuffer();
                prodGenInit.logFile.log("Getting list of misfits.");

                misfits = misfitList.elements();

                int counter = 0;
                while ( misfits.hasMoreElements() ) {
                    ProductDraft pd = new ProductDraft();
                    pd = (ProductDraft)misfits.nextElement();
//                    prodGenInit.logFile.log("Setting misfit info from product draft misfit.");
                    misfit.productInd = pd.getProductInd();
//                    prodGenInit.logFile.log("productInd: " + misfit.productInd);
                    misfit.brandInd = pd.getOPICMBrand().getInd();
//                    prodGenInit.logFile.log("Brand: " + misfit.brandInd);
                    misfit.description=pd.getOPICMDescript(); 
//                    prodGenInit.logFile.log("Description: " + misfit.description);
                    misfit.productNum = pd.getProdNum();
//                    prodGenInit.logFile.log("Product Number: " + misfit.productNum);
//                    prodGenInit.logFile.log("Checking misfit brand against BPM.");
                    if ( bpm.brandInd!=misfit.brandInd ) {
//                        prodGenInit.logFile.log("BPM.brand: " + bpm.brandInd + " is not equal to Misfit.brand: " + misfit.brandInd);
                        continue;
                    }
                    messageDetails.append( misfit.toString() );
                    counter++;
                } //end while misfits

                StringBuffer message = new StringBuffer("The following " + counter + " misfits require your attention: \n\nProductInd\tProduct Number\tOPIC/M Description\n") ;
                message.append(messageDetails.toString());
                mail.setRecipients(bpm.emailAddr);
                prodGenInit.logFile.log("BPM eMail: " + bpm.emailAddr);
                mail.setSubject(subject);
                prodGenInit.logFile.log("Subject: " + subject);
                mail.setText(message.toString());
                prodGenInit.logFile.log("Message: \n" + message); 
                mail.send();
                prodGenInit.logFile.log ("Mail sent.");
            } // while hasMoreBPM
        } //end of try
        catch ( Exception e ) {
            okToContinue = false;
        } //end of catch
    } //end of method processResults

    /**
     * getOkToContinue()
     * This is returns whether or not the program should continue to execute.
     *
     * @param None
     * @return boolean True if successful; otherwise false.
     **/
    public boolean getOkToContinue() {
//        prodGenInit.logFile.log("Getting okToContinue from MisfitsMailer.");
        return okToContinue;
    } //end of method getOkToContinue

    /**
     * run()
     * This is main class for the ProdGen BPM Misfit eMailer Batch.
     *
     * @param None
     * @return Nothing
     **/
    public void run() {
        try {
            prodGenInit.logFile.log ( "MisfitMailer.run()");
            if ( ! prodGenInit.dbSys.createConnection(DatabaseSystem.QUEST, questdbName, questdbHost, questdbPort, questdbUser,questdbPassword) ) {
                prodGenInit.logFile.logSevereError("MisfitsMailer: Unable to connect to quest.");
                //  AppSure.logFailure("MisfitsMailer: Unable to connect to quest.");
                okToContinue = false;
                return;
            } else {
                prodGenInit.logFile.log("Successfully connected to Quest");
//                SQLMethod sqlMethod = new SQLMethod(1, "MisfitsMailer", 60);
//                Statement statement = sqlMethod.createStatement();
//                prodGenInit.logFile.log("Statement Created"); 
                Statement statement = prodGenInit.dbSys.getConnection(DatabaseSystem.QUEST).createStatement();
                prodGenInit.logFile.log("Statement Created"); 

                /* Get all the BPM's details. */
                prodGenInit.logFile.log ( "before  BPM "); 
                String qryStr =  "select a.userid, a.name, a.emailaddr, b.typecatind, b.description  "+
                                 "from tigris.userid a, prodraft.typecategory b " + //", tigris.useridcountry c  "+
                                 "where b.categorylevel = '" + TypeCategory.LEVEL_OPICM_BRAND +
                                 "' and b.valid = 'Y' and a.userid=b.owner ";  // and a.userid=c.userid " ) ; 
                prodGenInit.logFile.log("Current  query:  \n" + qryStr);
                ResultSet result = statement.executeQuery ( qryStr );
                prodGenInit.logFile.log ( "Query executed.");  
                if ( createBPMList(result) )
                    processResults();
            }//end of else
        }//end of try
        catch ( Exception ex ) {
            prodGenInit.logFile.logError(ex.getMessage());
            okToContinue=false;
            return;
        } //end of catch
    } //end of run
}//end of class MisfitsMailer


    /****************************************************************
     * BPM
     * This class is used to hold all information about a BPM necessary 
     * to email a BPM information about his/her misfits.<p>
     *
     ***************************************************************/
    class BPM {
        String  userId = null;
        String  name = null;
        String  emailAddr = null;
        int brandInd = 0;
        String brandDesc = null;

        /**
         * toString()
         * This method returns all the elements as a string value.<p>
         *
         *  @param None
         *  @return String  The String representation of a BPM object.
         **/
        public String toString() {
            StringBuffer outStr = new StringBuffer("Userid: " + userId + "\t ");
            outStr.append("Name: " + name + "\t ");
            outStr.append( "Email address: " + emailAddr + "\t ");
            outStr.append( "Brand Indicator: " + brandInd + "\t");
            outStr.append( "Brand Description: " + brandDesc + "\n"); 
            return(outStr.toString());
        } //end of method toString()
    } //end of class BPM


    /****************************************************************
     * Misfit
     * This class is used to hold all information about a misfit necessary
     * to email a BPM information about his/her misfits.<p>
     *
     ***************************************************************/
    class Misfit {
        int productInd = 0;
        int brandInd = 0;
        String description = null;
        String productNum = null;

        /**
         * toString()
         * This method returns all the elements as a string value.<p>
         *
         *  @param None
         *  @return String  The String representation of a BPM object.
         **/
        public String toString() {
            StringBuffer outStr = new StringBuffer(productInd + "\t\t ");
            outStr.append(productNum + "\t ");
            outStr.append( description + "\n ");
            return(outStr.toString());
        } //end of method toString()
    }//end of class Misfit