package com.ibm.nzna.projects.batch.prodgen;
/* -------------------------------------------------------------------------------------------------------- */
/* OpicmExport                                                                                                             */
/* This class executes a ksh script passing it  database connection variables.               */
/* The ksh script name and all of the parameters for this script from a property file.  */
/*                                                                                                                                     */
/* @Tracey Watkins                                                                                                       */
/* @version 1.0                                                                                                               */
/* --------------------------------------------------------------------------------------------------------- */

import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.db.*;
import java.util.*;
import java.io.*;
import java.lang.*;
//import com.ibm.appsure.*;

public class OpicmExport {
    /************************Private * Members **************************/
    private boolean okToContinue = true;
    private String logFileName = null;
    private String backupDir = null;
    private String opicmDBName = null;
    private String opicmDBUser = null;
    private String opicmDBPwd = null;
    private String questDBName = null;
    private String questDBUser = null;
    private String questDBPwd = null;
    static ProdGenInit prodGenInit = null;

    /*****************************Methods*******************************/
    /**
    * Constructor
    *    	This method will instantiate the class.
    *  @param	None
    *  @return	Nothing
    */
    public OpicmExport ( ProdGenInit pgi) { //throws AppSureException{
        this.prodGenInit = pgi;
        prodGenInit.logFile.beginLogSection("OpicmExport");
        backupDir = prodGenInit.props.getProperty("prodgen.backup.dir");
        opicmDBName = prodGenInit.props.getProperty("db.opicm.name");
        opicmDBUser = prodGenInit.props.getProperty("db.opicm.uid");
        opicmDBPwd = prodGenInit.props.getProperty("db.opicm.pwd");
        questDBName = prodGenInit.props.getProperty("db.quest.name");
        questDBUser = prodGenInit.props.getProperty("db.quest.uid");
        questDBPwd = prodGenInit.props.getProperty("db.quest.pwd");
        logFileName = prodGenInit.props.getProperty("prodgen.logfile");
        prodGenInit.logFile.log("OpicmExport instantiated.");
    }  //end of constructor

    public boolean runScript() { //throws AppSureException{
        try {
            prodGenInit.logFile.log("runScript()");
            Runtime r = Runtime.getRuntime();
            prodGenInit.logFile.log("Runtime().getRuntime()");
            Process proc = null;
            prodGenInit.logFile.log("Process instantiated.");
            String cmdStr = prodGenInit.props.getProperty("prodgen.getData.script");
            prodGenInit.logFile.log("The command to run is : " + cmdStr );
            cmdStr += " " + opicmDBName + " " + opicmDBUser + " " + opicmDBPwd ;
            cmdStr += " " + questDBName + " " + questDBUser + " " + questDBPwd ;
            cmdStr += " " + logFileName + " " + backupDir + " >> " + logFileName ;
            prodGenInit.logFile.log("The command to run is : " + cmdStr );
            proc= r.exec( cmdStr );
            proc.waitFor();
            int exitValue = proc.exitValue();
            prodGenInit.logFile.log("Command : " + cmdStr + " is finished.");
            switch ( exitValue ) {
            case 0: 
                prodGenInit.logFile.log("Exit value : " + exitValue + "\tNormal execution.");
                break;
            case 1:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when connecting to the opicmods database. ");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when connecting to the opicmods database. ");
                okToContinue=false;
                break;
            case 2:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when connecting to the quest database.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when connecting to the quest database.");
                okToContinue=false;
                break;
            case 3:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when exporting data from table opicm.es_country_cpsl.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when exporting data from table opicm.es_country_cpsl.");
                okToContinue=false;
                break;
            case 4:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when exporting data from table opicm.es_country_cto.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when exporting data from table opicm.es_country_cto.");
                okToContinue=false;
                break;
            case 5:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when exporting data from table opicm.es_country_sol.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when exporting data from table opicm.es_country_sol.");
                okToContinue=false;
                break;
            case 6:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when importing data from table opicm.es_country_cpsl.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when importing data from table opicm.es_country_cpsl.");
                okToContinue=false;
                break;
            case 7:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when importing data from table opicm.es_country_cto.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when importing data from table opicm.es_country_cto.");
                okToContinue=false;
                break;
            case 8:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when importing data from table opicm.es_country_soll.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when importing data from table opicm.es_country_soll.");
                okToContinue=false;
                break;
            case 9:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when loading dummy.ixf to table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when loading dummy.ixf to table opicm.odsimport.");
                okToContinue=false;
                break;
            case 10:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting data from table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting data from table opicm.odsimport.");
                okToContinue=false;
                break;
            case 11:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting non EMEA and non US data from table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting non EMEA and non US data from table opicm.odsimport.");
                okToContinue=false;
                break;
            case 12:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting US data from table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting US data from table opicm.odsimport.");
                okToContinue=false;
                break;
            case 13:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting EMEA Client Products data from table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting EMEA Client Products data from table opicm.odsimport.");
                okToContinue=false;
                break;
            case 14:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting EMEA Mobile Systems data from table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting EMEA Mobile Systems data from table opicm.odsimport.");
                okToContinue=false;
                break;
            case 15:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting EMEA Visual Products data from table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting EMEA Visual Products data from table opicm.odsimport.");
                okToContinue=false;
                break;
            case 16:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting EMEA PC Servers data from table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting EMEA PC Servers data from table opicm.odsimport.");
                okToContinue=false;
                break;
            case 17:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting EMEA Options and Peripheral data from opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting EMEA Options and Peripheral data from opicm.odsimport.");
                okToContinue=false;
                break;
            case 18:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when deleting all other data from table opicm.odsimport.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when deleting all other data from table opicm.odsimport.");
                okToContinue=false;
                break;
            case 19:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when updating table opicm.odsimport setting ship_supported_date to NULL where it was 'NULL'.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when updating table opicm.odsimport setting ship_supported_date to NULL where it was 'NULL'.");
                okToContinue=false;
                break;
            case 20:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\nFailed when Failed when updating table opicm.odsimport targ_setting ship_supported_date to NULL where it was 'NULL'.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\nFailed when Failed when updating table opicm.odsimport targ_setting ship_supported_date to NULL where it was 'NULL'.");
                okToContinue=false;
                break;
            default:
                prodGenInit.logFile.log("Exit value : " + exitValue + "\tUnknown error.  Check prodgen.log file.");
                prodGenInit.logFile.logError("Exit value : " + exitValue + "\tUnknown error.  Check prodgen.log file.");
//                okToContinue=false;
            }   //end of switch
            prodGenInit.logFile.log("Exit value : " + exitValue);

        } //end of try
        catch ( IOException ioe ) {
            okToContinue = false;
            prodGenInit.logFile.log("There was an IO Exception. " + ioe.getMessage() );
            ioe.printStackTrace();
//            prodGenInit.appSure.logFailure( prodGenInit.UNABLE_TO_RETRIEVE_OPICM_DATA, prodGenInit.logFile.getErrorFile(), prodGenInit.logFile.getErrorFile() );
        } //end of catch IOException
        catch ( InterruptedException ie ) {
            okToContinue = false;
            prodGenInit.logFile.log( "There was an Interrupted Exception. " + ie.getMessage() );
            ie.printStackTrace();
//            prodGenInit.appSure.logFailure( prodGenInit.UNABLE_TO_RETRIEVE_OPICM_DATA, prodGenInit.logFile.getErrorFile(), prodGenInit.logFile.getErrorFile() );
        } //end of catch InterruptedException
        catch ( Exception e ) {
            okToContinue = false;
            prodGenInit.logFile.log( "There was an exception.  " + e.getMessage() );
            e.printStackTrace();
//            prodGenInit.appSure.logFailure( prodGenInit.UNABLE_TO_RETRIEVE_OPICM_DATA, prodGenInit.logFile.getErrorFile(), prodGenInit.logFile.getErrorFile() );
        } //end of catch Exception
        return okToContinue;
    } //end of method runScript
} // end of class OpicmExport
