package com.ibm.nzna.projects.batch.prodgen;
//Author Tracey and Souvik 4/16/2000
//Changed 7/27/01


import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.db.*;
//import com.ibm.appsure.*;
import com.ibm.nzna.shared.util.StandardPropertyLoader;
import com.ibm.nzna.projects.common.quest.type.*;
import java.util.*;
import java.io.*;
import java.lang.*;


/**
 * ProdGenInit class initializes the batch, makes connections to opicm
 * @author Souvik Dutta and Tracey Watkins
 * @version 1.0
 */
public class ProdGenInit extends Batch implements FailureTypes {

    /**
     * This flag determines which driver is used.  If true, then the App Driver is used.  If false, then the Net Driver is used.
     */
    protected boolean appDriver = false;
    /**
     * This is the file name of the log for the ProdGen Batch.
     */
    protected BatchLogSystem logFile = null;

    /**
     * This flag determines whether or not the batch can continue.  If true, then the batch can run.  If false, then the batch should cease running.
     */
    private boolean okToContinue = true;

    /**
     * This flag shows what type of publishing is to be processed.  If true, then a full publish will be run.  If false, then only a delta publish will be run based on changes made since the date listed in the datePublish field.
     */
    private boolean fullPublish = false;

    /**
     * This date is used for delta publishing such that only a delta publish will be run based on changes made since the date listed in this field.
     */
    private String datePublish = null;


    // These variable is used for appSure database server, name, id, password
    private String appCode = "PRODGEN";

    /**
    * This should be read from property file. It is used to make each run/instance of the batch unique (in case there are multiple instances). 
    */
    private int runIndex = 1;   

    protected boolean debug = false;
    protected int debugLevel = 1;
    protected  TypeList typeList = null;
    protected  int APPSURE  = 0;
    protected int failureType = 0;

    protected String logFileName = null;
    protected Properties props = null;
    protected Vector misfitsVec = new Vector();
    protected DatabaseSystem dbSys = null;
//    protected AppSure appSure = null;
//    public static OpicmExport opicmExport = null;
//    public static ProdGenManager pgMan = null;
//    public static MisfitsMailer misfitsMailer = null;

/**********************************METHODS***********************************/
    public static void main( String[] args ) {
        new ProdGenInit( args );
    }  //end of main method

    public ProdGenInit( String[] args ) {
        try {
            logFileName = "logs"+System.getProperty("file.separator") + "prodgen";
            logFile = new BatchLogSystem( logFileName, this);
            typeList = new TypeList();
            typeList.initialize(false);
            okToContinue = initialize( args );              
//            appSure = new AppSure();

            if ( okToContinue ) {
                startBatch();
            }
//            if ( okToContinue ) {
            OpicmExport opicmExport = new OpicmExport(this);
//            }
            logFile.log("opicmExport.runScript()");
            okToContinue = opicmExport.runScript();
            logFile.log("opicmExport.runScript() complete.");
            if ( okToContinue ) {
                ProdGenManager prodGenMan = new ProdGenManager(this);
                okToContinue = prodGenMan.processProducts();
                misfitsVec = prodGenMan.getMisfitsList();
                logFile.log("Number of misfits created: " + misfitsVec.size());
            } else {
                logFile.logSevereError("Failure during OpicmExport.");
//                appSure.logFailure(UNABLE_TO_RETRIEVE_OPICM_DATA, logFile.getErrorFile(), logFile.getErrorFile());
                stopBatch();
            }
//            if ( okToContinue ) {
            if (misfitsVec.size() > 0) {
                MisfitsMailer misfitMailer = new MisfitsMailer(this);
//                logFile.log("MisfitsMailer instantiated.");
//                okToContinue = misfitsMailer.getOkToContinue();
//                logFile.log("misfitsMailer.getOkToContinue() : " + okToContinue);
//                if ( okToContinue ) {
//                logFile.log("Calling MisfitsMailer.run()....");
//                    misfitMailer.run();
                    logFile.log("MisfitsMailer.run() complete.");
//                }
                okToContinue = misfitMailer.getOkToContinue();
                logFile.log("misfitsMailer.getOkToContinue() : " + okToContinue);
//            } else {
//                logFile.logSevereError("Failure during ProdGenManager.");
//                appSure.logFailure(BATCH_EXCEPTION, logFile.getErrorFile(), logFile.getErrorFile());
//                stopBatch();
//            }
            }
            else 
                logFile.log("There are no misfits to process.");
            if ( okToContinue ) {
                okToContinue = updateTimestamp();
                if ( !okToContinue ) {
                    logFile.logSevereError("Failure during ProdGenInit.");
//                    appSure.logFailure(UNABLE_TO_UPDATE_TIMESTAMP, logFile.getErrorFile(), logFile.getErrorFile() );
                }
                stopBatch();
            } else {
                logFile.logSevereError("Failure during MisfitsMailer.\n");
//                appSure.logFailure(UNABLE_TO_SEND_EMAIL, logFile.getErrorFile(), logFile.getErrorFile());
                stopBatch();
            }
//        }//end of try
//        catch ( AppSureException ase ) {
//            logFile.logSevereError("Appsure threw an exception in ProdGenInit constructor.");
//            logFile.logSevereError(ase.getMessage());
        } catch ( Exception e ) {
//			appSure.logFailure(BATCH_EXCEPTION, logFile.getErrorFile(), logFile.getErrorFile());
            logFile.logSevereError("Exception caught in ProdGenInit constuctor.\n" + e.getMessage() );
        }
    } //end of constructor ProdGenInit

    public boolean getOkToContinue() {
        return okToContinue;
    }  //end of method getOkToContinue

    public boolean initialize( String[] args ) {
        boolean returnCode = true;
        try {
//            String FS = System.getProperty("file.separator");
//            new StandardPropertyLoader("com"+FS +"ibm"+FS+"nzna"+FS+"properties"+"prodgen.properties", args);
//            StandardPropertyLoader stdPropLdr = new StandardPropertyLoader("prodgen.properties");
//                        String path = stdPropLdr.getPath();
//                        logFile.log("Property path is " + path);
//            props = StdPropLdr.getProperties();
            props = this.load("prodgen.properties", args);
            if ( props.isEmpty() ) {
                logFile.logSevereError("ProdGenInit: Unable to read properties file or command line parameters.");
                //                              appSure.logFailure(BATCH_EXCEPTION, logFile.getErrorFile(), logFile.getErrorFile());
                return( returnCode = false );
            }

            logFile.log("properties loaded");

            /*Get the appsure properties.*/
            APPSURE = new Integer(props.getProperty("db.prodgen.APPSURE")).intValue();
            runIndex = new Integer(props.getProperty("db.appsure.run.index")).intValue();
            debug =  new Boolean(props.getProperty("prodgen.DEBUG")).booleanValue();
            debugLevel = new Integer(props.getProperty("prodgen.debug.level")).intValue();
            logFile.log("APPSURE: " + APPSURE + "\trunIndex: " + runIndex + "\tdebug: " + debug + "\tdebug level: " + debugLevel);
            appDriver =  new Boolean( props.getProperty("db.quest.appDriver") ).booleanValue() ;
            DatabaseSystem dbSys = new DatabaseSystem(debug, logFile, appDriver);
        } //end of try
        catch ( Exception e ) {
            logFile.logSevereError("Exception caught during initialization");
            logFile.logSevereError(e.getMessage());
            returnCode = false;
        }//end of catch
        return returnCode;
    } //end of method initialize


    /* This method will start the batch */
    public void startBatch() {
//        try {
//            if ( okToContinue ) {
//                if ( appSure.isAppRunning(appCode) ) {
//                    logFile.log("Application is already running.");
//                    stopBatch();
//                } else {
//                    okToContinue = appSure.startApp(dbSys.getConnection(APPSURE), appCode);
//                    if (okToContinue) {
//                    }
//                    logFile.log("AppSure.startApp for " + appCode + " successful.");
//                }
//            } //end of if okToContinue
//        }//end of try
//        catch ( AppSureException ase ) {
//            logFile.logSevereError("AppSureException caught in startBatch. \n" + ase.getMessage());
//        } catch ( Exception e ) {
//            logFile.logSevereError("Exception caught in startBatch.\n" + e.getMessage());
//        }
    }//end of method startBatch

    /* This method will stop the batch */
    public void stopBatch() {
        try {
            DatabaseSystem.shutdown();
//            okToContinue=appSure.shutDown(logFile.getErrorFile(), logFile.getErrorFile());
//        }//end of try
//        catch ( AppSureException ase ) {
//            logFile.logSevereError("AppSureException caught in stopBatch. \n" + ase.getMessage());
        } catch ( Exception e ) {
            logFile.logSevereError("Exception caught in stopBatch.\n" + e.getMessage());
        }
        System.exit( okToContinue ? 1 : 0 );
    }//end of method stopBatch

    public boolean updateTimestamp() {
        logFile.log( "Updating PRODGEN_TIMESTAMP in the database.");
        boolean rc = true;
        try {
            //Get db properties from property file.
            String dbHost = props.getProperty("db.quest.host" );
            String dbName = props.getProperty("db.quest.name" );
            String dbPort = props.getProperty("db.quest.port" );
            String dbUser = props.getProperty("db.quest.uid" );
            String dbPassword = props.getProperty("db.quest.pwd" );
            boolean useAppDriver =  new Boolean( props.getProperty("db.quest.appDriver") ).booleanValue() ;
            logFile.log( "Host: " + dbHost + "\tName: " + dbName + "\tPort: " + dbPort + "\nUser: " +  dbUser +
                         "\tPwd: " + dbPassword + "\nappDriver: " + useAppDriver );

            //Create a connection to the database.
            if ( DatabaseSystem.createConnection(DatabaseSystem.QUEST, dbName, dbHost, dbPort, dbUser, dbPassword) ) {
                logFile.log( "Connection successfully created to the quest database." );
            } else {
                logFile.logError( "Unable to connect to the Quest database." );
//                appSure.logFailure( UNABLE_TO_UPDATE_TIMESTAMP, logFile.getErrorFile(), logFile.getErrorFile() );
                rc=false;
                return( rc );
            }
            if ( rc ) {
                //Create the query used in the Prepared Statement.
                String updateTimestampQry = "UPDATE tigris.constants SET STRCONST=CURRENT TIMESTAMP WHERE constant='PRODGEN_TIMESTAMP'";
                System.out.println("updateTimestampQry: " + updateTimestampQry);
                SQLMethod sqlMethod = new SQLMethod(1, "ProdGen",  5);
                System.out.println("SQLMethod created.");
                QuestStatement updateStmt = new QuestStatement( sqlMethod.createStatement( ) );
                System.out.println("QuestStatement created.");
                if ( updateStmt.executeUpdate(updateTimestampQry) != 1 ) {
                    logFile.log( "Unable to update PRODGEN_TIMESTAMP." );
                    rc=false;
                } else {
                    logFile.log( "PRODGEN_TIMESTAMP updated in tigris.constants." ) ;
                }

                updateStmt.close();
                sqlMethod.close();
            }
        } //end of try
        catch ( Exception e ) {
            rc = false;
//            try {
//                appSure.logFailure(UNABLE_TO_UPDATE_TIMESTAMP, logFile.getErrorFile(), logFile.getErrorFile() );
//            } catch ( AppSureException ase ) {
//                logFile.logSevereError("Appsure threw an exception in ProdGenInit update timestamp method.");
//                logFile.logSevereError(ase.getMessage());
//            }
            logFile.logSevereError("Exception caught during updateTimestamp(). " );
            logFile.logSevereError( e.getMessage() );
//        } catch ( AppSureException ase ) {
//            logFile.logSevereError("Appsure threw an exception in ProdGenInit update timestamp method.");
//            logFile.logSevereError(ase.getMessage());
        }

        return rc;
    }  //end of method updateTimestamp()


    /*------------------------------------------------------------*/
    /* load
     */
    /**
     *    This method should parse through the given parameters
     *
     *  @param args Command line arguments as an array of String
     *  @return True if successful, otherwise false.
     *//*---------------------------------------------------------*/
    public Properties  load ( String propFile, String[] args ) {
        logFile.log( "Loading the properties/command line args." );
        Properties properties = null;
        try {
            properties = StandardPropertyLoader.load( propFile );
            String tempStr = null;
            String key = null;
            String value = null;
            for ( int i=0; i < args.length; i++ ) {
                tempStr = args[i];
                key = tempStr.substring(1, tempStr.indexOf(":"));
                value = tempStr.substring(tempStr.indexOf(":")+1);
                properties.put(key, value);
                logFile.log( key + "  =  " + properties.get(key));
            }
        } catch ( Exception e ) {
            logFile.log(e.getMessage());
            e.printStackTrace();
        }
        return properties;
    } //end of load method

}//end of ProdGenInit class
