package com.ibm.nzna.projects.batch.prodgen;
//Author Tracey, Souvik 04/16/20001

import com.ibm.nzna.projects.common.quest.product.*;  //ProductDraft, Product, CreateProductDraft, OpicmMapper
import com.ibm.nzna.shared.batch.*;
import com.ibm.nzna.shared.db.*;
import com.ibm.appsure.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.lang.*;
import com.ibm.nzna.projects.common.quest.type.*;  //TypeList, TypeCategoryRec , TypeDateRec, TypeOfferingRec
import com.ibm.nzna.projects.common.quest.*;  //LocaleRec
import com.ibm.nzna.projects.qit.admin.keyword.offering.TypeOfferingList;
import com.ibm.nzna.projects.qit.admin.keyword.date.TypeDateList;
import com.ibm.nzna.shared.util.CDate;


/**
 * This class will pull all of the required information from OPIC/M to the quest database.
 * It converts OPIC/M products into Quest product drafts.
 */
public class ProdGenManager extends Product {

/********************************MEMBERS***************************************/
    /**
     * This holds the timestamp of when prodgen was last run.
     */
    private String lastRun= null;

    /**
     * This vector hold all the products that are in opicm but not in quest
     */
//    private Vector keyListNew = null;

    /**
     * This vector hold all the products that have been updated in opicm but not in quest
     */
//    private Vector keyListUpdate = new Vector();

    /**
     * This vector holds all the products that are not in opicm but are in quest
     */
//    private Vector keyListDelete = new Vector();

    /**
     * This is the vector of misfit products that need to be emailed to the BPMs.
     */
    private Vector misfitList = new Vector ();

    static ProdGenInit prodGenInit = null;

    private boolean okToContinue = true;

    private OPICMMapper opicmMap = null;

    private CDate todayDate = new CDate();
/********************************METHODS***************************************/
    /**
    *  Constructor
    *  This method will instantiate the class.
    *  @param	ProdGenInit The batch that calls ProdGenManager.
    *  @return	Nothing
    */
    public ProdGenManager ( ProdGenInit pgi ) {
        this.prodGenInit = pgi;
        prodGenInit.logFile.beginLogSection("ProdGenManager");
    }  //end of constructor


    /**
    *  setProductDates
    *  This method will set the target and actual dates for each opicm product
    *   to the quest geographies and countries.
    *  @param OpicmProduct The OpicmProduct object
    *  @param TypeCountryCodeRec The TypeCountryCodeRec for the quest country associated with this product.
    *  @return	Vector of dates for the product.
    */
    public Vector setProductDates (OpicmProduct opicmProduct, TypeCountryCodeRec country ) {
//        public Vector setProductDates (OpicmProduct opicmProduct, TypeCategoryRec country ) {
        prodGenInit.logFile.log("ProdGenManager.setProductDates");
        OpicmProduct opcm = opicmProduct;
        Vector productDatesVec = new Vector();
        Vector prodTypeDateRecs = new Vector();
        prodTypeDateRecs = TypeList.getInstance().getTypeList ( TypeList.TYPE_DATE ) ;

        for ( int i=0; i<prodTypeDateRecs.size(); i++ ) {
            TypeDateRec tdr = (TypeDateRec)prodTypeDateRecs.elementAt(i);
            int tdrInd = tdr.getInd();
            ProductDate prodDate = new ProductDate();

            switch ( tdrInd ) {
            case TypeList.DATE_ANNOUNCED:
                prodGenInit.logFile.log("Date Type is Announced.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.cntyAnnounceDate != null &&  !opcm.cntyAnnounceDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Country Actual: " + opcm.cntyAnnounceDate);
                    prodDate.setActualDate(opcm.cntyAnnounceDate);
                } else if ( opcm.actualAnnounceDate != null && !opcm.actualAnnounceDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.actualAnnounceDate);
                    prodDate.setActualDate(opcm.actualAnnounceDate);
                }
                if ( opcm.targCntyAnnounceDate !=null  && !opcm.targCntyAnnounceDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Country Target: " + opcm.targCntyAnnounceDate);
                    prodDate.setTargetDate(opcm.targCntyAnnounceDate);
                }
                break;
            case TypeList.DATE_PRE_ANNOUNCED:
                prodGenInit.logFile.log("Date type is PreAnnounced.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.preAnnounceDate !=null && !opcm.preAnnounceDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.preAnnounceDate);
                    prodDate.setActualDate(opcm.preAnnounceDate);
                }
                if ( opcm.targPreAnnounceDate != null && !opcm.targPreAnnounceDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.targPreAnnounceDate);
                    prodDate.setTargetDate(opcm.targPreAnnounceDate);
                }
                break;
            case TypeList.DATE_GA_AVAILABLE:
                prodGenInit.logFile.log("Date type is GA Available.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.gaAvailabilityDate !=null && !opcm.gaAvailabilityDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.gaAvailabilityDate);
                    prodDate.setActualDate(opcm.gaAvailabilityDate);
                }
                if ( opcm.targGaAvailabilityDate != null && !opcm.targGaAvailabilityDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.targGaAvailabilityDate);
                    prodDate.setTargetDate(opcm.targGaAvailabilityDate);
                }
                break;
            case TypeList.DATE_WITHDRAWN:
                prodGenInit.logFile.log("Date type is Withdrawn.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.withdrawnDate !=null && !opcm.withdrawnDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.withdrawnDate);
                    prodDate.setActualDate(opcm.withdrawnDate);
                }
                if ( opcm.targWithdrawnDate != null && !opcm.targWithdrawnDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.targWithdrawnDate);
                    prodDate.setTargetDate(opcm.targWithdrawnDate);
                }
                break;
            case TypeList.DATE_END_OF_SERVICE:
                prodGenInit.logFile.log("Date type is End of Service.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.endOfServiceDate != null && !opcm.endOfServiceDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.endOfServiceDate);
                    prodDate.setActualDate(opcm.endOfServiceDate);
                }
                if ( opcm.targEndOfServiceDate !=null && !opcm.targEndOfServiceDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.targEndOfServiceDate);
                    prodDate.setTargetDate(opcm.targEndOfServiceDate);
                }
                break;
            case TypeList.DATE_END_OF_MANUFACT:
                prodGenInit.logFile.log("Date type is End of Manufacturing.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.endOfManufactDate != null && opcm.endOfManufactDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.endOfManufactDate);
                    prodDate.setActualDate(opcm.endOfManufactDate);
                }
                if ( opcm.targEndOfManufactDate !=null && !opcm.targEndOfManufactDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.targEndOfManufactDate);
                    prodDate.setTargetDate(opcm.targEndOfManufactDate);
                }
                break;
            case TypeList.DATE_END_OF_MARKET:
                prodGenInit.logFile.log("Date type is End of Marketing.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.endOfMarketDate !=null && !opcm.endOfMarketDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.endOfMarketDate);
                    prodDate.setActualDate(opcm.endOfMarketDate);
                }
                if ( opcm.targEndOfMarketDate != null && !opcm.targEndOfMarketDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.endOfMarketDate);
                    prodDate.setTargetDate(opcm.targEndOfMarketDate);
                }
                break;
            case TypeList.DATE_FIRST_ORDER:
                prodGenInit.logFile.log("Date type is first order.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.firstOrderDate != null && !opcm.firstOrderDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.firstOrderDate);
                    prodDate.setActualDate(opcm.firstOrderDate);
                }
                if ( opcm.targFirstOrderDate != null && !opcm.targFirstOrderDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.targFirstOrderDate);
                    prodDate.setTargetDate(opcm.targFirstOrderDate);
                }
                break;
            case TypeList.DATE_SHIP_SUPPORTED:
                prodGenInit.logFile.log("Date type is ship supported.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.shipSupportedDate !=null  && !opcm.shipSupportedDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.shipSupportedDate);
                    prodDate.setActualDate(opcm.shipSupportedDate);
                }
                if ( opcm.targShipSupportedDate !=null && opcm.targShipSupportedDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.targShipSupportedDate);
                    prodDate.setTargetDate(opcm.targShipSupportedDate);
                }
                break;
            case TypeList.DATE_END_OF_PRODUCTION:
                prodGenInit.logFile.log("Date type is End of Production.");
                prodDate = new ProductDate(tdrInd);
                if ( opcm.endOfProdDate !=null && !opcm.endOfProdDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Actual " + opcm.endOfProdDate);
                    prodDate.setActualDate(opcm.endOfProdDate);
                }
                if ( opcm.targEndOfProdDate != null && !opcm.targEndOfProdDate.equals("NULL") ) {
                    prodGenInit.logFile.log("Target " + opcm.targEndOfProdDate);
                    prodDate.setTargetDate(opcm.targEndOfProdDate);
                }
                break;
            default:
                prodGenInit.logFile.log("Date type is unknown.");
                break;
            }
            if ( (prodDate.getActualDate() != null && !prodDate.getActualDate().equals("NULL"))|| 
                 (prodDate.getTargetDate() != null ) && !prodDate.getTargetDate().equals("NULL") ) {
                if ( country != null ) {
                    prodDate.setCountryCodeInd(country.getInd());
                } else {
                    //This is a hack to get the dates set if no mapping exists for the OPIC/M country to the Quest country.
                    prodDate.setCountryCodeInd(124); 
                }
                productDatesVec.addElement(prodDate);
            }
        } //end of for
//        prodGenInit.logFile.log("Returning from setProductDates().");
        return productDatesVec;
    }  //end of method setProductDates()

    /**
    *  setOfferingType
    *  This method will set the offering type for CTO products.
    *  @param	String The description of this product's offering.
    *  @return	The offeringTypeInd
    */
    public int setOfferingType (String offeringDesc ) {
        prodGenInit.logFile.log("Setting offering type.");
        String offeringDescript = offeringDesc;
        Vector offeringTypeRecs = TypeList.getInstance().getTypeList ( TypeList.TYPE_OFFERING ) ;

        int offeringTypeInd = 0; 
        for ( int i=0; i<offeringTypeRecs.size(); i++ ) {
            TypeOfferingRec totr = (TypeOfferingRec)offeringTypeRecs.elementAt(i);
            if ( offeringDescript.endsWith("CTO") && totr.getDescript().indexOf("CTO") > 0 ) {
                offeringTypeInd = totr.getInd();
            }
            //Other offering types can be set here.
        }
        return offeringTypeInd;
    }  //end of method setOfferingType()


    /**
      *  This method will add an OPIC/M Brand to the database
      **/
    public TypeCategoryRec createNewOBrand(String opicmBrandDescript ) {
        prodGenInit.logFile.log("Creating new OPIC/M brand.");
        String oBrand = opicmBrandDescript;
        TypeCategoryRec oDraftBrandRec = null;
        if ( ( oBrand != null ) && ( oBrand.length() > 0 ) ) {
            TypeCategoryRec oBrandRec = TypeCategory.createTypeCategory ( 0,
                                                                          oBrand,
                                                                          TypeCategory.LEVEL_OPICM_BRAND,
                                                                          0,
                                                                          "N",
                                                                          "OPICM" ) ;

            oDraftBrandRec = TypeCategory.createDraftTypeCategory ( oBrandRec.getInd(),
                                                                    0,
                                                                    oBrand,
                                                                    TypeCategory.LEVEL_OPICM_BRAND,
                                                                    0,
                                                                    "N",
                                                                    "OPICM" ) ;
            int returnCode = TypeCategory.writeToDatabase();
            int rc = oDraftBrandRec.writeToDatabase();
            System.out.println("Opicm brand (Category)" + oBrand + "\t Return code: " + returnCode);
            System.out.println("Opicm brand (Draft Category)" + oBrand + "\t Return code: " + rc);
        }
        return oDraftBrandRec;
    }  //end of method createNewOBrand

    /**
      *  This method will add an OPIC/M Family to the database
      **/
    private TypeCategoryRec createNewOFamily ( String opicmFamilyDescript, TypeCategoryRec oDraftBrandRec ) {
        String oFamily =opicmFamilyDescript;
        TypeCategoryRec oDraftFamilyRec = null;
        if ( ( oFamily != null ) && ( oFamily.length() > 0 ) ) {
            TypeCategoryRec oBrandRec  = oDraftBrandRec;
            TypeCategoryRec oFamilyRec = TypeCategory.createTypeCategory ( oBrandRec.getInd(),
                                                                           oFamily,
                                                                           TypeCategory.LEVEL_OPICM_FAMILY,
                                                                           0,
                                                                           "N",
                                                                           "OPICM" ) ;

            oDraftFamilyRec = TypeCategory.createDraftTypeCategory ( oFamilyRec.getInd(),
                                                                     oBrandRec.getInd(),
                                                                     oFamily,
                                                                     TypeCategory.LEVEL_OPICM_FAMILY,
                                                                     0,
                                                                     "N",
                                                                     "OPICM" ) ;
            int returnCode = TypeCategory.writeToDatabase();
            int rc = oDraftFamilyRec.writeToDatabase();
            System.out.println("Opicm family " + oFamily + "\t Return code: " + returnCode);
        }
        return oDraftFamilyRec;
    } /* createNewOFamily  */

    /**
      *  This method will add an OPIC/M Series to the database
      **/
    private TypeCategoryRec createNewOSeries ( String opicmSeriesDescript, TypeCategoryRec oDraftFamilyRec ) {
        String oSeries = opicmSeriesDescript;
        TypeCategoryRec oDraftSeriesRec = null;
        if ( ( oSeries != null ) && ( oSeries.length() > 0 ) ) {
            TypeCategoryRec oFamilyRec = oDraftFamilyRec;
            TypeCategoryRec oSeriesRec = TypeCategory.createTypeCategory ( oFamilyRec.getInd(),
                                                                           oSeries,
                                                                           TypeCategory.LEVEL_OPICM_SERIES,
                                                                           0,
                                                                           "N",
                                                                           "OPICM" ) ;

            oDraftSeriesRec = TypeCategory.createDraftTypeCategory ( oSeriesRec.getInd(),
                                                                     oFamilyRec.getInd(),
                                                                     oSeries,
                                                                     TypeCategory.LEVEL_OPICM_SERIES,
                                                                     0,
                                                                     "N",
                                                                     "OPICM" ) ;
            int returnCode = TypeCategory.writeToDatabase();
            int rc = oDraftSeriesRec.writeToDatabase();
            System.out.println("Opicm series " + oSeries + "\t Return code: " + returnCode);
        }
        return oDraftSeriesRec;
    } /* createNewOSeries  */


    /**
      *  This method will add an OPIC/M Geography to the database
      **/
    public TypeCategoryRec createNewOGeo(String opicmGeoDescript ) {
        prodGenInit.logFile.log("Creating new OPIC/M geography.");
        String oGeo = opicmGeoDescript;
        TypeCategoryRec oDraftGeoRec = null;
        if ( ( oGeo != null ) && ( oGeo.length() > 0 ) ) {
            TypeCategoryRec oGeoRec = TypeCategory.createTypeCategory ( 0,
                                                                        oGeo,
                                                                        TypeCategory.LEVEL_OPICM_GEOGRAPHY,
                                                                        0,
                                                                        "N",
                                                                        "OPICM" ) ;

            oDraftGeoRec = TypeCategory.createDraftTypeCategory ( oGeoRec.getInd(),
                                                                  0,
                                                                  oGeo,
                                                                  TypeCategory.LEVEL_OPICM_GEOGRAPHY,
                                                                  0,
                                                                  "N",
                                                                  "OPICM" ) ;
            int returnCode = TypeCategory.writeToDatabase();
            int rc = oDraftGeoRec.writeToDatabase();
            System.out.println("Opicm geography: " + oGeo + "\t Return code: " + returnCode);
        }
        return oDraftGeoRec;
    }  //end of method createNewOGeo

    /**
      *  This method will add an OPIC/M Country to the database
      **/
    private TypeCategoryRec createNewOCountry ( String opicmCountryDescript, TypeCategoryRec oDraftGeoRec ) {
        String oCountry =opicmCountryDescript;
        TypeCategoryRec oDraftCountryRec = null;
        if ( ( oCountry != null ) && ( oCountry.length() > 0 ) ) {
            TypeCategoryRec oGeoRec  = oDraftGeoRec;
            TypeCategoryRec oCountryRec = TypeCategory.createTypeCategory ( oGeoRec.getInd(),
                                                                            oCountry,
                                                                            TypeCategory.LEVEL_OPICM_COUNTRY,
                                                                            0,
                                                                            "N",
                                                                            "OPICM" ) ;

            oDraftCountryRec = TypeCategory.createDraftTypeCategory ( oCountryRec.getInd(),
                                                                      oGeoRec.getInd(),
                                                                      oCountry,
                                                                      TypeCategory.LEVEL_OPICM_COUNTRY,
                                                                      0,
                                                                      "N",
                                                                      "OPICM" ) ;
            int returnCode1 = TypeCategory.writeToDatabase();
            int rc = oDraftCountryRec.writeToDatabase();
            System.out.println("Opicm country " + oCountry + "\t Return code: " + returnCode1);
        }
        return oDraftCountryRec;
    } /* createNewOCountry  */


    /**
    * processProducts
    *  This method gets the lists of new, updated and deleted products in OPIC/M.  It creates Quest product 
    *  drafts for each product.  If a new brand, family, series, project, machine, model, geography or country exists
    *  in OPIC/M that does not exist in Quest, then it is created in Quest.
    *  @param	None
    *  @return	True if successful; otherwise false.
    */
    public boolean processProducts () {
        boolean rc = true;
        try {
            String questDBName = prodGenInit.props.getProperty("db.quest.name");
            String questDBUser = prodGenInit.props.getProperty("db.quest.uid");
            String questDBPwd = prodGenInit.props.getProperty("db.quest.pwd");
            String questDBHost = prodGenInit.props.getProperty("db.quest.host");
            String questDBPort = prodGenInit.props.getProperty("db.quest.port");

            if ( !prodGenInit.dbSys.createConnection( DatabaseSystem.QUEST, questDBName, questDBHost, questDBPort, questDBUser, questDBPwd) ) {
                prodGenInit.logFile.logSevereError("Unable to connect to the quest database.");
                rc=false;
                return rc;
            }

            /* Get the BPM Announce Days */
            int days = 0;
            Statement newProduct = prodGenInit.dbSys.getConnection(DatabaseSystem.QUEST).createStatement();
            String bpmDays = prodGenInit.props.getProperty("prodgen.bpm.days");
            if ( bpmDays == null ) {
                String queryStr = "select strconst from tigris.constants where constant = 'BPM_ANNOUNCE_DAYS'";
                ResultSet results = newProduct.executeQuery( queryStr );
                if ( results.next() ) {
                    days = results.getInt(1);
                } else {
                    prodGenInit.logFile.logSevereError("Unable to get the bpmDays from Quest.");
//                    AppSure.logFailure(failureType, null, prodGenInit.logFile.getProgressFile());
                    rc = false;
                    return rc;
                }
            } /* if bpmDays is null */
            todayDate = new CDate(CDate.DATE_DBDATE);
            prodGenInit.logFile.log("Today is " + todayDate.getDate(CDate.DATE_DBDATE));
            bpmDays = todayDate.addDaysToDate(todayDate.today(), days, todayDate.dateType);
            prodGenInit.logFile.log("bpmDays = " + bpmDays);

            prodGenInit.logFile.log("todayDate is " + todayDate.getDate(CDate.DATE_DBDATE));
            prodGenInit.logFile.log("bpmDays is " + bpmDays);

            /* Get the last run timestamp from database if not in properties file. */
            if ( lastRun == null ) {
                String queryStr = "select strconst from tigris.constants where constant = 'PRODGEN_TIMESTAMP'";
                prodGenInit.logFile.log("queryStr : \n" + queryStr);
                ResultSet results =newProduct.executeQuery( queryStr );
                if ( results.next() ) {
                    lastRun = results.getString(1);
                    lastRun = lastRun.trim();
                    prodGenInit.logFile.log("ProdGen last ran successfully on " + lastRun);
                } else
                    prodGenInit.logFile.log("lastRun is null.");
                results.close();
            }

            opicmMap = new OPICMMapper();

            /* Get list of new OPIC/M products to be inserted into Quest.  Create product drafts for each new product,
                 set the appropriate fields and save the product draft to the database. */
            String queryNew= "select * from opicm.odsimport where key not in (select key from opicm.primarylink) and (se_valfrom >'"+lastRun;
            queryNew += "') and  ((cnty_Announce_Date <='"+bpmDays+"') or (actual_announce_date <= '" +bpmDays;
            queryNew += "') or (target_announce_Date <= '" + bpmDays + "')) order by market_description, variant_part_num_desc ";
            prodGenInit.logFile.log("Query for new products: \n" + queryNew);
            ResultSet resultsNew =newProduct.executeQuery( queryNew);

            int count = 0;
            int retCode = 0;
            ProductDraft prodDraft = null;
            while ( resultsNew.next() ) {
                prodDraft = new ProductDraft();
                OpicmProduct opcm = new OpicmProduct();
                opcm = getOpicmProduct(resultsNew);
                TypeCategoryRec questFamily = (TypeCategoryRec)opicmMap.getQuestFamilyFromOPICM(opcm.subGroupName, opcm.familyName, opcm.seriesName);
                TypeCountryCodeRec questCountry = opicmMap.getQuestCountryFromOPICM(opcm.subGeoCode, opcm.countryName);

                if ( questCountry == null ) {
                    /* Get the OPIC/M geo/country. */
                    TypeCategoryRec opicmGeo = TypeCategory.getDraftCategoryFromDescription(opcm.subGeoCode, TypeCategory.LEVEL_OPICM_GEOGRAPHY, 0);
                    if ( opicmGeo == null ) {
                        opicmGeo = createNewOGeo(opcm.subGeoCode);
                    }
                    Vector opicmGeoVec = new Vector();
                    opicmGeoVec.addElement(opicmGeo);

                    TypeCategoryRec opicmCountry = TypeCategory.getDraftCategoryFromDescription(opcm.countryName, TypeCategory.LEVEL_OPICM_COUNTRY, opicmGeo.getInd());
                    if ( opicmCountry == null ) {
                        opicmCountry = createNewOCountry(opcm.countryName, opicmGeo);
                    }
                    Vector opicmCountryVec = new Vector();
                    opicmCountryVec.addElement(opicmCountry);

                    TypeCategory.writeToDatabase();
                } //Country not mapped in Quest.

                /* Set project/machine/model or project/subcategory/part number. */
                TypeCategoryRec questBrand = null;
                TypeCategoryRec opicmBrand = null;
                TypeCategoryRec opicmFamily = null;
                TypeCategoryRec opicmSeries = null;
                TypeCategoryRec project = null;
                TypeCategoryRec subCategory=null;
                TypeCategoryRec partNumber=null;
                TypeCategoryRec machine = null;
                TypeCategoryRec model = null;

                CreateProductDraft cpd = null;
                if ( opcm.subGroupName.startsWith("OPTIONS") ) { //Product is an OPTION
                    if ( questFamily!=null ) {
                        questBrand = TypeCategory.getCategory(questFamily.getParentCatInd());

                        project = TypeCategory.getDraftCategoryFromDescription( opcm.projectName, TypeCategory.LEVEL_PROJECT, questFamily.getInd() );
                        if ( project == null ) {
                            project = TypeCategory.createDraftTypeCategory ( questFamily.getInd() , opcm.projectName, TypeCategory.LEVEL_PROJECT, 0, "N", "OPICM" ) ;
                            retCode = project.writeToDatabase();
                        }

                        subCategory = TypeCategory.getDraftCategoryFromDescription( project.getDescript(), TypeCategory.LEVEL_SUBCATEGORY, project.getInd() );
                        if ( subCategory == null ) {
                            subCategory = TypeCategory.createDraftTypeCategory ( project.getInd() , project.getDescript(), TypeCategory.LEVEL_SUBCATEGORY, 0, "N", "OPICM" ) ;
                            retCode = subCategory.writeToDatabase();
                        }

                        partNumber = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum, TypeCategory.LEVEL_PARTNUMBER, subCategory.getInd() );
                        if ( partNumber == null ) {
                            partNumber = TypeCategory.createDraftTypeCategory ( subCategory.getInd() , opcm.variantPartNum, TypeCategory.LEVEL_PARTNUMBER, 0, "N", "OPICM" ) ;
                            retCode = partNumber.writeToDatabase();
                        }

                        retCode = TypeCategory.writeToDatabase();
                        try {
                            cpd = prodDraft.createProductDraft(opcm.variantPartNum);
                            prodDraft = this.getProductDraft(cpd);
                            prodDraft.setWorkRequired(TypeList.WORKREQ_EDIT);
                        } catch ( Exception e ) {
                            prodDraft.setWorkRequired(TypeList.WORKREQ_CREATE);
                            prodGenInit.logFile.log("Exception caught: " + e.getMessage());
                            e.printStackTrace();
                        }
                        prodDraft.setBrand(  questBrand );
                        prodDraft.setFamily(  questFamily );
                    } else {  //Product is an OPTION who's OPICM Brand/Family/Series has not been mapped to Quest

                        opicmBrand = TypeCategory.getDraftCategoryFromDescription(opcm.subGroupName, TypeCategory.LEVEL_OPICM_BRAND, 0);
                        if ( opicmBrand == null ) {
                            opicmBrand = createNewOBrand( opcm.subGroupName );
                        }

                        opicmFamily = TypeCategory.getDraftCategoryFromDescription(opcm.familyName, TypeCategory.LEVEL_OPICM_FAMILY, opicmBrand.getInd());
                        if ( opicmFamily == null ) {
                            opicmFamily = createNewOFamily(opcm.familyName, opicmBrand);
                        }

                        opicmSeries = TypeCategory.getDraftCategoryFromDescription(opcm.seriesName, TypeCategory.LEVEL_OPICM_SERIES, opicmFamily.getInd());
                        if ( opicmSeries == null ) {
                            opicmSeries = createNewOSeries(opcm.seriesName, opicmFamily);
                        }

                        project = TypeCategory.getDraftCategoryFromDescription( "Default", TypeCategory.LEVEL_PROJECT, opicmSeries.getInd() );
                        if ( project == null ) {
                            project = TypeCategory.createDraftTypeCategory ( opicmSeries.getInd() , opcm.projectName, TypeCategory.LEVEL_PROJECT, 0, "N", "OPICM" ) ;
                        }

                        subCategory = TypeCategory.getDraftCategoryFromDescription( opcm.projectName, TypeCategory.LEVEL_SUBCATEGORY, project.getInd() );
                        if ( subCategory == null ) {
                            subCategory = TypeCategory.createDraftTypeCategory ( project.getInd() , opcm.projectName, TypeCategory.LEVEL_SUBCATEGORY, 0, "N", "OPICM" ) ;
                        }

                        partNumber = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum, TypeCategory.LEVEL_PARTNUMBER, subCategory.getInd() );
                        if ( partNumber == null ) {
                            partNumber = TypeCategory.createDraftTypeCategory ( subCategory.getInd() , opcm.variantPartNum, TypeCategory.LEVEL_PARTNUMBER, 0, "N", "OPICM" ) ;
                        }

                        retCode = TypeCategory.writeToDatabase();

                        prodDraft.setOPICMBrand(  opicmBrand );
                        prodDraft.setOPICMFamily(  opicmFamily );
                        prodDraft.setOPICMSeries( opicmSeries );
                    }
                    prodDraft.setProject( project );
                    prodDraft.setSubCategory(subCategory);
                    prodDraft.setPartNumber(partNumber);
                    prodDraft.setProductType(TypeList.PRODUCTTYPE_OPTION);
                } else {  //Product is not an OPTION
                    if ( questFamily!= null ) {  //Non-Option product whose OPICM Brand/Family/Series has been mapped in Quest.
                        questBrand = TypeCategory.getCategory(questFamily.getParentCatInd());
                        prodGenInit.logFile.log("Quest Brand: " + questBrand.getDescript() + "\tQuest Family: " + questFamily.getDescript());

                        project = TypeCategory.getDraftCategoryFromDescription( opcm.projectName, TypeCategory.LEVEL_PROJECT, questFamily.getInd() );
                        if ( project == null ) {
                            project = TypeCategory.createDraftTypeCategory ( questFamily.getInd() , opcm.projectName, TypeCategory.LEVEL_PROJECT, 0, "N", "OPICM" ) ;
                            retCode = project.writeToDatabase();
                        }
                        prodGenInit.logFile.log("Project: " + project.getDescript() + "\tParent: " + project.getParentCatInd());

                        machine = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum.substring(0,4), TypeCategory.LEVEL_MACHINE, project.getInd() );
                        if ( machine == null ) {
                            machine = TypeCategory.createDraftTypeCategory ( project.getInd() , opcm.variantPartNum.substring(0,4), TypeCategory.LEVEL_MACHINE, 0, "N", "OPICM" ) ;
                            retCode = machine.writeToDatabase();

                        }
                        prodGenInit.logFile.log("Machine: " + machine.getDescript() + "\tParent: " + machine.getParentCatInd());

                        model = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum.substring(4), TypeCategory.LEVEL_MODEL, machine.getInd() );
                        if ( model == null ) {
                            model = TypeCategory.createDraftTypeCategory ( machine.getInd() , opcm.variantPartNum.substring(4), TypeCategory.LEVEL_MODEL, 0, "N", "OPICM" ) ;
                            model.writeToDatabase();
                        }
                        prodGenInit.logFile.log("Model: " + model.getDescript() + "\tParent: " + model.getParentCatInd());

                        retCode = TypeCategory.writeToDatabase();
                        prodGenInit.logFile.log("TypeCategory.writeToDatabase(): " + retCode);

                        prodGenInit.logFile.log("ProductDraft.createProductDraft(" + questFamily.getDescript() + "," +machine.getDescript() + "," +
                                                model.getDescript() + ")" );
//                        cpd = prodDraft.createProductDraft(questFamily, opcm.variantPartNum.substring(0,4), opcm.variantPartNum.substring(4));
                        try {
                            cpd = prodDraft.createProductDraft(questFamily, machine.getDescript(), model.getDescript());
                            prodDraft = this.getProductDraft(cpd);
                            prodDraft.setWorkRequired(TypeList.WORKREQ_EDIT);
                        } catch ( Exception e ) {
                            prodDraft.setWorkRequired(TypeList.WORKREQ_CREATE);
                            prodGenInit.logFile.log("Exception caught: " + e.getMessage());
                            e.printStackTrace();
                        }
                        prodDraft.setBrand(  questBrand );
                        prodDraft.setFamily(  questFamily );
                    }//if questFamily != null
                    else {  //Non-Option product whose OPICM Brand/Family/Series has not been mapped in Quest.
                        opicmBrand = TypeCategory.getDraftCategoryFromDescription(opcm.subGroupName, TypeCategory.LEVEL_OPICM_BRAND, 0);
                        if ( opicmBrand == null ) {
                            opicmBrand = createNewOBrand( opcm.subGroupName );
                        }

                        opicmFamily = TypeCategory.getDraftCategoryFromDescription(opcm.familyName, TypeCategory.LEVEL_OPICM_FAMILY, opicmBrand.getInd());
                        if ( opicmFamily == null ) {
                            opicmFamily = createNewOFamily(opcm.familyName, opicmBrand);
                        }

                        opicmSeries = TypeCategory.getDraftCategoryFromDescription(opcm.seriesName, TypeCategory.LEVEL_OPICM_SERIES, opicmFamily.getInd());
                        if ( opicmSeries == null ) {
                            opicmSeries = createNewOSeries(opcm.seriesName, opicmFamily);
                        }

                        project = TypeCategory.getDraftCategoryFromDescription( opcm.projectName, TypeCategory.LEVEL_PROJECT, opicmSeries.getInd() );
                        if ( project == null ) {
                            project = TypeCategory.createDraftTypeCategory ( opicmSeries.getInd() , opcm.projectName, TypeCategory.LEVEL_PROJECT, 0, "N", "OPICM" ) ;
                        }

                        machine = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum.substring(0,4), TypeCategory.LEVEL_MACHINE, project.getInd() );
                        if ( machine == null ) {
                            machine = TypeCategory.createDraftTypeCategory ( project.getInd() , opcm.variantPartNum.substring(0,4), TypeCategory.LEVEL_MACHINE, 0, "N", "OPICM" ) ;
                        }

                        model = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum.substring(4), TypeCategory.LEVEL_MODEL, machine.getInd() );
                        if ( model == null ) {
                            model = TypeCategory.createDraftTypeCategory ( machine.getInd() , opcm.variantPartNum.substring(4), TypeCategory.LEVEL_MODEL, 0, "N", "OPICM" ) ;
                        }

                        retCode = TypeCategory.writeToDatabase();

                        prodDraft.setOPICMBrand(  opicmBrand );
                        prodDraft.setOPICMFamily(  opicmFamily );
                        prodDraft.setOPICMSeries( opicmSeries );
                    } //Else OPIC/M Series is not mapped.

                    prodDraft.setProject( project );
                    prodDraft.setMachine(machine);
                    prodDraft.setModel(model);

                    if ( opcm.subGroupName.startsWith("Visual") )
                        prodDraft.setProductType(TypeList.PRODUCTTYPE_MONITOR);
                    else
                        prodDraft.setProductType(TypeList.PRODUCTTYPE_SYSTEM);
                } //Else product is not an Option.

                /* Set the work required type for the product draft. */
//                prodDraft.setWorkRequired(TypeList.WORKREQ_CREATE);

                /* Set fields using set methods in ProductDraft. */
                prodDraft.setComment("Inserted by OPIC/M");

                /* Set the misfit flag is not mapped to a Quest Family. */
                if ( questFamily == null )
                    prodDraft.setMisfit(true);

                /* Set the OPIC/M and Quest descriptions. */
                if ( opcm.marketDesc != null && !opcm.marketDesc.equals("NULL") ) {
                    prodDraft.setDescription( opcm.marketDesc );
                    prodDraft.setOPICMDescript( opcm.marketDesc );
                } else {
                    if ( opcm.variantPartNumDesc != null && !opcm.variantPartNumDesc.equals("NULL") ) {
                        prodDraft.setDescription( opcm.variantPartNumDesc );
                        prodDraft.setOPICMDescript( opcm.variantPartNumDesc );
                    }
                }

                /* Set the base model. */
                if ( opcm.offeringPartNum != null && !opcm.offeringPartNum.equals("NULL") ) {
                    prodDraft.setBaseModel(opcm.offeringPartNum);
                }

                /* Set the offering type from db and set only when offering is CTO. */
                if ( opcm.variantPartNumDesc !=null && !opcm.variantPartNumDesc.equals("NULL") ) {
                    prodDraft.setOffering(this.setOfferingType(opcm.variantPartNumDesc));
                } else {
                    if ( opcm.marketDesc != null && !opcm.marketDesc.equals("NULL") ) {
                        prodDraft.setOffering(this.setOfferingType(opcm.marketDesc));
                    }
                }

                /* Set the input, last touched by, created by, created on, source and import from opicm fields. */
                prodDraft.setInput("OPIC/M");
                prodDraft.setLastTouchedBy("PRODGEN");
                prodDraft.setCreatedBy("OPIC/M");
                prodDraft.setCreatedOn(opcm.valFrom);
                prodDraft.setSource("OPIC/M");
                prodDraft.setOPICMImport(true);

                /*Set all product dates, locale and write to database if and only if the opicm country is mapped to a quest country */
                if ( questCountry != null ) {

                    /*Set the dates. */
                    prodDraft.setDates(setProductDates(opcm, questCountry));

                    /* Set Country and Geography */
                    LocaleRec  localeRec = new LocaleRec();

                    /* Set the Quest Country and Geo in the LocalRec for the product draft. */
                    localeRec.addCountry(questCountry.getInd());
                    localeRec.addGeography(questCountry.getGeoInd());

                    /* Set the LocalRec for the product draft. */
                    if ( (localeRec.countryCount() > 0) || (localeRec.geoCount() > 0) ) {
                        prodDraft.setLocale( localeRec );
                    }

                    /* Determine if the product draft is a misfit.  If so, add it to the misfits Vector to be processed by */
                    /* MisfitsMailer.                                                                                                                               */
                    if ( prodDraft.getMisfit() ) {
                        misfitList.addElement( prodDraft );
                        prodGenInit.logFile.log("Added product " + prodDraft.getProductInd() + " to misfits vector.");
                    }

                    /* Add entry to PrimaryLink table for opicm key and product draft ind. */
                    prodDraft.setOPICMKey(opcm.key);

                    /* Save the product draft to the Quest database */
                    prodDraft.writeToDatabase();

                    /* Increment the product draft count. */
                    count++;
                } else {
                    prodGenInit.logFile.log("This section should be logged to APPSURE as a Sev 3 email only for notification purposes.");
                    prodGenInit.logFile.log("There is no quest country mapped for this product, therefore, we are unable to save this draft.");
                    prodGenInit.logFile.log("OPIC/M Geography: " + opcm.subGeoCode + "\t Country: " + opcm.countryName);
                    prodGenInit.logFile.log("OPIC/M Brand: " + opcm.subGroupName + "\tFamily: " + opcm.familyName+
                                            "\tSeries: " + opcm.seriesName + "\tProduct: " + opcm.variantPartNum);
                }

                prodGenInit.logFile.log(count + " products processed.");
            } //end of while resultsNew
            prodGenInit.logFile.log("There are " + count + " new opicm products.");
            resultsNew.close();

            /* Get list of  updated OPIC/M products to be modified in Quest.  Create product drafts for each updated  */
            /* product, set the appropriate fields and save the product draft to the database.                                            */
//            String queryUpdate= "select * from opicm.odsimport " +
//                                                    "where key in (select key from opicm.primarylink l, prodraft.product d " +
//                                                    "where d.importopicm='Y' and l.productind = p.productind ) and (se_valfrom >'" + lastRun + "') " + 
//                                                    "and ((cnty_Announce_Date <='" + bpmDays + "') or (actual_announce_date <= '" +bpmDays +"') "+
//                                                    "or (target_announce_Date <= '" + bpmDays + "')) order by market_description, variant_part_num_desc ";
            String queryUpdate= "select * from opicm.odsimport " +
                                "where key in (select key from opicm.primarylink  " +
                                "where productind in (select productind from prodraft.product " +
                                "where importopicm='Y' )) and se_valfrom >'" + lastRun + "' " + 
                                "and (cnty_Announce_Date <='" + bpmDays + "' or actual_announce_date <= '" +bpmDays +"' "+
                                "or target_announce_Date <= '" + bpmDays + "') order by market_description, variant_part_num_desc ";
            prodGenInit.logFile.log("Query for updated products: \n" + queryUpdate);
            ResultSet resultsUpdate =newProduct.executeQuery( queryUpdate );

            count=0;
            /* Process updated products. */
            while ( resultsUpdate.next() ) {
                prodDraft = new ProductDraft();
                OpicmProduct opcm = new OpicmProduct();
                opcm = getOpicmProduct( resultsUpdate );

                TypeCategoryRec questFamily = (TypeCategoryRec)opicmMap.getQuestFamilyFromOPICM(opcm.subGroupName, opcm.familyName, opcm.seriesName);
                TypeCountryCodeRec questCountry = opicmMap.getQuestCountryFromOPICM(opcm.subGeoCode, opcm.countryName);

                if ( questCountry == null ) {
                    /* Get the OPIC/M geo/country. */
                    TypeCategoryRec opicmGeo = TypeCategory.getDraftCategoryFromDescription(opcm.subGeoCode, TypeCategory.LEVEL_OPICM_GEOGRAPHY, 0);
                    if ( opicmGeo == null ) {
                        opicmGeo = createNewOGeo(opcm.subGeoCode);
                    }
                    Vector opicmGeoVec = new Vector();
                    opicmGeoVec.addElement(opicmGeo);

                    TypeCategoryRec opicmCountry = TypeCategory.getDraftCategoryFromDescription(opcm.countryName, TypeCategory.LEVEL_OPICM_COUNTRY, opicmGeo.getInd());
                    if ( opicmCountry == null ) {
                        opicmCountry = createNewOCountry(opcm.countryName, opicmGeo);
                    }
                    Vector opicmCountryVec = new Vector();
                    opicmCountryVec.addElement(opicmCountry);

                    TypeCategory.writeToDatabase();
                } //Country not mapped in Quest.

                /* Set project/machine/model or project/subcategory/part number. */
                TypeCategoryRec questBrand = null;
                TypeCategoryRec opicmBrand = null;
                TypeCategoryRec opicmFamily = null;
                TypeCategoryRec opicmSeries = null;
                TypeCategoryRec project = null;
                TypeCategoryRec subCategory=null;
                TypeCategoryRec partNumber=null;
                TypeCategoryRec machine = null;
                TypeCategoryRec model = null;

                CreateProductDraft cpd = null;
                if ( opcm.subGroupName.startsWith("OPTIONS") ) { //Product is an OPTION
                    if ( questFamily!=null ) {
                        questBrand = TypeCategory.getCategory(questFamily.getParentCatInd());

                        project = TypeCategory.getDraftCategoryFromDescription( opcm.projectName, TypeCategory.LEVEL_PROJECT, questFamily.getInd() );
                        if ( project == null ) {
                            project = TypeCategory.createDraftTypeCategory ( questFamily.getInd() , opcm.projectName, TypeCategory.LEVEL_PROJECT, 0, "N", "OPICM" ) ;
                            retCode = project.writeToDatabase();
                        }

                        subCategory = TypeCategory.getDraftCategoryFromDescription( project.getDescript(), TypeCategory.LEVEL_SUBCATEGORY, project.getInd() );
                        if ( subCategory == null ) {
                            subCategory = TypeCategory.createDraftTypeCategory ( project.getInd() , project.getDescript(), TypeCategory.LEVEL_SUBCATEGORY, 0, "N", "OPICM" ) ;
                            retCode = subCategory.writeToDatabase();
                        }

                        partNumber = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum, TypeCategory.LEVEL_PARTNUMBER, subCategory.getInd() );
                        if ( partNumber == null ) {
                            partNumber = TypeCategory.createDraftTypeCategory ( subCategory.getInd() , opcm.variantPartNum, TypeCategory.LEVEL_PARTNUMBER, 0, "N", "OPICM" ) ;
                            retCode = partNumber.writeToDatabase();
                        }

                        retCode = TypeCategory.writeToDatabase();
                        try {
                            cpd = prodDraft.createProductDraft(opcm.variantPartNum);
                            prodDraft = this.getProductDraft(cpd);
                            prodDraft.setWorkRequired(TypeList.WORKREQ_EDIT);
                        } catch ( Exception e ) {
                            prodDraft.setWorkRequired(TypeList.WORKREQ_CREATE);
                            prodGenInit.logFile.log("Exception caught: " + e.getMessage());
                            e.printStackTrace();
                        }

                        prodDraft.setBrand(  questBrand );
                        prodDraft.setFamily(  questFamily );
                    } else {  //Product is an OPTION who's OPICM Brand/Family/Series has not been mapped to Quest

                        opicmBrand = TypeCategory.getDraftCategoryFromDescription(opcm.subGroupName, TypeCategory.LEVEL_OPICM_BRAND, 0);
                        if ( opicmBrand == null ) {
                            opicmBrand = createNewOBrand( opcm.subGroupName );
                        }

                        opicmFamily = TypeCategory.getDraftCategoryFromDescription(opcm.familyName, TypeCategory.LEVEL_OPICM_FAMILY, opicmBrand.getInd());
                        if ( opicmFamily == null ) {
                            opicmFamily = createNewOFamily(opcm.familyName, opicmBrand);
                        }

                        opicmSeries = TypeCategory.getDraftCategoryFromDescription(opcm.seriesName, TypeCategory.LEVEL_OPICM_SERIES, opicmFamily.getInd());
                        if ( opicmSeries == null ) {
                            opicmSeries = createNewOSeries(opcm.seriesName, opicmFamily);
                        }

                        project = TypeCategory.getDraftCategoryFromDescription( "Default", TypeCategory.LEVEL_PROJECT, opicmSeries.getInd() );
                        if ( project == null ) {
                            project = TypeCategory.createDraftTypeCategory ( opicmSeries.getInd() , opcm.projectName, TypeCategory.LEVEL_PROJECT, 0, "N", "OPICM" ) ;
                        }

                        subCategory = TypeCategory.getDraftCategoryFromDescription( opcm.projectName, TypeCategory.LEVEL_SUBCATEGORY, project.getInd() );
                        if ( subCategory == null ) {
                            subCategory = TypeCategory.createDraftTypeCategory ( project.getInd() , opcm.projectName, TypeCategory.LEVEL_SUBCATEGORY, 0, "N", "OPICM" ) ;
                        }

                        partNumber = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum, TypeCategory.LEVEL_PARTNUMBER, subCategory.getInd() );
                        if ( partNumber == null ) {
                            partNumber = TypeCategory.createDraftTypeCategory ( subCategory.getInd() , opcm.variantPartNum, TypeCategory.LEVEL_PARTNUMBER, 0, "N", "OPICM" ) ;
                        }

                        retCode = TypeCategory.writeToDatabase();

                        prodDraft.setOPICMBrand(  opicmBrand );
                        prodDraft.setOPICMFamily(  opicmFamily );
                        prodDraft.setOPICMSeries( opicmSeries );
                    }
                    prodDraft.setProject( project );
                    prodDraft.setSubCategory(subCategory);
                    prodDraft.setPartNumber(partNumber);
                    prodDraft.setProductType(TypeList.PRODUCTTYPE_OPTION);
                } else {  //Product is not an OPTION
                    if ( questFamily!= null ) {  //Non-Option product whose OPICM Brand/Family/Series has been mapped in Quest.
                        questBrand = TypeCategory.getCategory(questFamily.getParentCatInd());

                        project = TypeCategory.getDraftCategoryFromDescription( opcm.projectName, TypeCategory.LEVEL_PROJECT, questFamily.getInd() );
                        if ( project == null ) {
                            project = TypeCategory.createDraftTypeCategory ( questFamily.getInd() , opcm.projectName, TypeCategory.LEVEL_PROJECT, 0, "N", "OPICM" ) ;
                            retCode = project.writeToDatabase();
                        }

                        machine = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum.substring(0,4), TypeCategory.LEVEL_MACHINE, project.getInd() );
                        if ( machine == null ) {
                            machine = TypeCategory.createDraftTypeCategory ( project.getInd() , opcm.variantPartNum.substring(0,4), TypeCategory.LEVEL_MACHINE, 0, "N", "OPICM" ) ;
                            retCode = machine.writeToDatabase();
                        }

                        model = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum.substring(4), TypeCategory.LEVEL_MODEL, machine.getInd() );
                        if ( model == null ) {
                            model = TypeCategory.createDraftTypeCategory ( machine.getInd() , opcm.variantPartNum.substring(4), TypeCategory.LEVEL_MODEL, 0, "N", "OPICM" ) ;
                            retCode = model.writeToDatabase();
                        }


                        try {
                            cpd = prodDraft.createProductDraft(questFamily, opcm.variantPartNum.substring(0,4), opcm.variantPartNum.substring(4));
                            prodDraft = this.getProductDraft(cpd);
                            prodDraft.setWorkRequired(TypeList.WORKREQ_EDIT);
                        } catch ( Exception e ) {
                            prodDraft.setWorkRequired(TypeList.WORKREQ_CREATE);
                            prodGenInit.logFile.log("Exception caught: " + e.getMessage());
                            e.printStackTrace();
                        }
                        prodDraft.setBrand(  questBrand );
                        prodDraft.setFamily(  questFamily );
                    }//if questFamily != null
                    else {  //Non-Option product whose OPICM Brand/Family/Series has not been mapped in Quest.
                        opicmBrand = TypeCategory.getDraftCategoryFromDescription(opcm.subGroupName, TypeCategory.LEVEL_OPICM_BRAND, 0);
                        if ( opicmBrand == null ) {
                            opicmBrand = createNewOBrand( opcm.subGroupName );
                        }

                        opicmFamily = TypeCategory.getDraftCategoryFromDescription(opcm.familyName, TypeCategory.LEVEL_OPICM_FAMILY, opicmBrand.getInd());
                        if ( opicmFamily == null ) {
                            opicmFamily = createNewOFamily(opcm.familyName, opicmBrand);
                        }

                        opicmSeries = TypeCategory.getDraftCategoryFromDescription(opcm.seriesName, TypeCategory.LEVEL_OPICM_SERIES, opicmFamily.getInd());
                        if ( opicmSeries == null ) {
                            opicmSeries = createNewOSeries(opcm.seriesName, opicmFamily);
                        }

                        project = TypeCategory.getDraftCategoryFromDescription( opcm.projectName, TypeCategory.LEVEL_PROJECT, opicmSeries.getInd() );
                        if ( project == null ) {
                            project = TypeCategory.createDraftTypeCategory ( opicmSeries.getInd() , opcm.projectName, TypeCategory.LEVEL_PROJECT, 0, "N", "OPICM" ) ;
                        }

                        machine = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum.substring(0,4), TypeCategory.LEVEL_MACHINE, project.getInd() );
                        if ( machine == null ) {
                            machine = TypeCategory.createDraftTypeCategory ( project.getInd() , opcm.variantPartNum.substring(0,4), TypeCategory.LEVEL_MACHINE, 0, "N", "OPICM" ) ;
                        }

                        model = TypeCategory.getDraftCategoryFromDescription( opcm.variantPartNum.substring(4), TypeCategory.LEVEL_MODEL, machine.getInd() );
                        if ( model == null ) {
                            model = TypeCategory.createDraftTypeCategory ( machine.getInd() , opcm.variantPartNum.substring(4), TypeCategory.LEVEL_MODEL, 0, "N", "OPICM" ) ;
                        }

                        retCode = TypeCategory.writeToDatabase();

                        prodDraft.setOPICMBrand(  opicmBrand );
                        prodDraft.setOPICMFamily(  opicmFamily );
                        prodDraft.setOPICMSeries( opicmSeries );
                    } //Else OPIC/M Series is not mapped.

                    prodDraft.setProject( project );
                    prodDraft.setMachine(machine);
                    prodDraft.setModel(model);

                    if ( opcm.subGroupName.startsWith("Visual") )
                        prodDraft.setProductType(TypeList.PRODUCTTYPE_MONITOR);
                    else
                        prodDraft.setProductType(TypeList.PRODUCTTYPE_SYSTEM);
                } //Else product is not an Option.

                /* Set fields using set methods in ProductDraft */
                prodDraft.setComment("Updated by Opicm");

                /* Set the work required type for the product draft. */
//                prodDraft.setWorkRequired(prodGenInit.typeList.WORKREQ_EDIT);


                /* Set the offering type for the product draft. */
                if ( opcm.marketDesc != null && !opcm.marketDesc.equals("NULL") ) {
                    prodDraft.setDescription( opcm.marketDesc );
                    prodDraft.setOPICMDescript( opcm.marketDesc );
                } else {
                    if ( opcm.variantPartNumDesc != null && !opcm.variantPartNumDesc.equals("NULL") ) {
                        prodDraft.setDescription( opcm.variantPartNumDesc );
                        prodDraft.setOPICMDescript( opcm.variantPartNumDesc );
                    }
                }

                /* Set the base model for the product draft. */
                if ( opcm.offeringPartNum != null && !opcm.offeringPartNum.equals("NULL") ) {
                    prodDraft.setBaseModel(opcm.offeringPartNum);
                }

                /* Set the Quest Brand and Family for the product draft. */
                if ( questFamily == null )
                    prodDraft.setMisfit(true);

                /* Set the offering type based on the variant part number description or the marketing description. */
                if ( opcm.variantPartNumDesc !=null && !opcm.variantPartNumDesc.equals("NULL") ) {
                    prodDraft.setOffering(this.setOfferingType(opcm.variantPartNumDesc));
                } else {
                    if ( opcm.marketDesc != null && !opcm.marketDesc.equals("NULL") ) {
                        prodDraft.setOffering(this.setOfferingType(opcm.marketDesc));
                    }
                }

                prodDraft.setInput("OPIC/M");
                prodDraft.setOPICMImport(true);
                prodDraft.setLastTouchedBy("PRODGEN");
                if ( !prodDraft.getSource().startsWith("OPIC/M") ) {
                    prodDraft.setSource("OPIC/M" + prodDraft.getSource());
                }

                /*Set all product dates, locale and write to database if and only if the opicm country is mapped to a quest country */
                if ( questCountry != null ) {

                    /*Set the dates. */
                    prodDraft.setDates(setProductDates(opcm, questCountry));

                    LocaleRec  localeRec = new LocaleRec();

                    /* Set the Quest Country and Geo in the LocalRec for the product draft. */
                    localeRec.addCountry(questCountry.getInd());
                    localeRec.addGeography(questCountry.getGeoInd());

                    /* Set the LocalRec for the product draft. */
                    if ( (localeRec.countryCount() > 0) || (localeRec.geoCount() > 0) ) {
                        prodDraft.setLocale( localeRec );
                    }

                    /* Determine if the product draft is a misfit.  If so, add it to the misfits Vector to be processed by */
                    /* MisfitsMailer.                                                                                                                               */
                    if ( prodDraft.getMisfit() ) {
                        misfitList.addElement( prodDraft );
                        prodGenInit.logFile.log("Added product " + prodDraft.getProductInd() + " to misfits vector.");
                    }

                    /* Save the product draft to the Quest database */
                    prodDraft.writeToDatabase();

                    /* Increment the product draft count. */
                    count++;
                } else {
                    prodGenInit.logFile.log("This section should be logged to APPSURE as a Sev 3 email only for notification purposes.");
                    prodGenInit.logFile.log("There is no quest country mapped for this product, therefore, we are unable to save this draft.");
                    prodGenInit.logFile.log("OPIC/M Geography: " + opcm.subGeoCode + "\t Country: " + opcm.countryName);
                    prodGenInit.logFile.log("OPIC/M Brand: " + opcm.subGroupName + "\tFamily: " + opcm.familyName+
                                            "\tSeries: " + opcm.seriesName + "\tProduct: " + opcm.variantPartNum);
                }

                prodGenInit.logFile.log(count + " product processed.");
            } //end of while resultsUpdate
            prodGenInit.logFile.log("There are " + count + " updated opicm products saved.");

            /* Close the ResultSet for the updated products. */
            resultsUpdate.close();


            /* Get list of products deleted from OPIC/M  to be deleted in Quest.  Create product drafts for each deleted */
            /* product, set the appropriate fields and save the product draft to the database.                                               */
            Statement delProduct = prodGenInit.dbSys.getConnection(DatabaseSystem.QUEST).createStatement();
//            String queryDelete= "select * from opicm.primarylink where key not in (select key from opicm.odsimport) " +
//                                "and productind in (select productind from prodraft.product where importopicm='Y') ";
            String queryDelete= "select productind from opicm.primarylink where key not in (select key from opicm.odsimport) " +
                                "and productind in (select productind from prodraft.product where importopicm='Y') ";
            prodGenInit.logFile.log("Query for deleted products: \n" + queryUpdate);
            ResultSet deleteResults =delProduct.executeQuery(queryDelete);


            count=0;
            while ( deleteResults.next() ) {
                int productInd = deleteResults.getInt("PRODUCTIND");
//                prodDraft= new ProductDraft(productInd);
                prodDraft= new ProductDraft();
                CreateProductDraft cpd = prodDraft.createProductDraft(productInd);
                prodDraft = this.getProductDraft(cpd);
                prodDraft.setComment("Deleted by OPIC/M");
                prodDraft.setWorkRequired(prodGenInit.typeList.WORKREQ_DELETE);

                /* Save the product draft to the Quest database */
                prodDraft.writeToDatabase();

                /* Increment the product draft count. */
                count++;
                prodGenInit.logFile.log(count + " product processed.");
            } //end of while deleteResults.next()

            prodGenInit.logFile.log("There are " + count + " deleted opicm products saved.");

            /* Close the ResultSet for the deleted products. */
            deleteResults.close();

        } //end of try
        catch ( Exception e ) {
            prodGenInit.logFile.logSevereError(e);
            okToContinue=false;
        } //end of catch
        return okToContinue;
    } //end of processProducts method

    public ProductDraft getProductDraft ( CreateProductDraft cpd) {
        ProductDraft pd     =new ProductDraft();
        int sqlCode         =0;
        int draftType = cpd.getReturnCode();
        prodGenInit.logFile.log(cpd.toOut());
        switch ( draftType ) {
        case  CreateProductDraft.DRAFT_ALREADY_EXISTS :
            pd =cpd.getProductDraft();
//            pd.setWorkRequired(prodGenInit.typeList.WORKREQ_EDIT);
            break;
        case CreateProductDraft.DRAFT_CREATED :
            pd = cpd.getProductDraft();
//            pd.setWorkRequired(TypeList.WORKREQ_CREATE);
            break;
        case CreateProductDraft.DRAFT_MISFIT:
            pd= cpd.getProductDraft();
//            pd.setWorkRequired(TypeList.WORKREQ_CREATE);
            break;
        case CreateProductDraft.DRAFT_RECYCLED :
            pd =  cpd.getProductDraft();
//            pd.setWorkRequired(prodGenInit.typeList.WORKREQ_EDIT);
            break;
        case CreateProductDraft.DRAFT_SQL_ERROR :
            sqlCode =  cpd.getSqlCode();
            prodGenInit.logFile.log("The SQL Code is " + sqlCode );
            break;
        default:
            break;
        }
        return pd;
    }   //end of method getProductDraft(CreateProductDraft)

    public OpicmProduct getOpicmProduct( ResultSet results ) {
        OpicmProduct opcm = new OpicmProduct();
        try {
            opcm.key = results.getString("KEY");
            opcm.subGeoCode = results.getString("GEOGRAPHY");
            opcm.countryName = results.getString("COUNTRY");
            opcm.subGroupName = results.getString("BRAND");
            opcm.familyName = results.getString("FAMILY");
            opcm.seriesName = results.getString("SERIES");
            opcm.projectName = results.getString("PROJECT");
            opcm.offeringPartNum = results.getString("OFFERING_PART_NUMBER");
            opcm.offeringPnumbDesc = results.getString("OFFERING_PART_NUM_DESC");
            opcm.marketDesc = results.getString("MARKET_DESCRIPTION");
            opcm.variantPartNum = results.getString("VARIANT_PART_NUMBER");
            opcm.variantPartNumDesc = results.getString("VARIANT_PART_NUM_DESC");
            opcm.actualAnnounceDate = results.getString("ACTUAL_ANNOUNCE_DATE");
            opcm.targetAnnounceDate = results.getString("TARGET_ANNOUNCE_DATE");
            opcm.cntyAnnounceDate = results.getString("CNTY_ANNOUNCE_DATE");
            opcm.preAnnounceDate = results.getString("PREANNOUNCE_DATE");
            opcm.gaAvailabilityDate = results.getString("GAVAILABILITY_DATE");
            opcm.withdrawnDate = results.getString("WITHDRAWN_DATE");
            opcm.endOfServiceDate = results.getString("END_OF_SERVICE_DATE");
            opcm.endOfProdDate = results.getString("END_OF_PROD_DATE");
            opcm.endOfManufactDate = results.getString("END_OF_MANUFAC_DATE");
            opcm.endOfMarketDate = results.getString("END_OF_MARKET_DATE");
            opcm.firstOrderDate = results.getString("FIRST_ORDER_DATE");
            opcm.shipSupportedDate  = results.getString("SHIP_SUPPORTED_DATE");
            opcm.targCntyAnnounceDate = results.getString("TARG_CNTY_ANNOUNCE_DATE");
            opcm.targPreAnnounceDate = results.getString("TARG_PREANNOUNCE_DATE");
            opcm.targGaAvailabilityDate = results.getString("TARG_GAVAILABILITY_DATE");
            opcm.targWithdrawnDate = results.getString("TARG_WITHDRAWN_DATE");
            opcm.targEndOfServiceDate = results.getString("TARG_END_OF_SERVICE_DATE");
            opcm.targEndOfProdDate = results.getString("TARG_END_OF_PROD_DATE");
            opcm.targEndOfManufactDate = results.getString("TARG_END_OF_MANUFAC_DATE");
            opcm.targEndOfMarketDate = results.getString("TARG_END_OF_MARKET_DATE");
            opcm.targFirstOrderDate = results.getString("TARG_FIRST_ORDER_DATE");
            opcm.targShipSupportedDate = results.getString("TARG_SHIP_SUPPORTED_DATE");
            opcm.status = results.getString("STATUS");
            opcm.valFrom = results.getString("SE_VALFROM");
        } catch ( Exception e ) {
            prodGenInit.logFile.logSevereError(e);
            okToContinue = false;
        }

        return opcm;
    }//end of method getOpicmProduct

    public Vector getMisfitsList() {
        return   misfitList;
    } // end of method getMisfitsList

    class OpicmProduct {
        /**
         * This holds the opicm database key.
         */
        private String key = null;

        /**
         * This holds the opicm subgeocode.  This field maps to the quest geography.
         */
        private String subGeoCode = null;

        /**
         * This holds the opicm COUNTRYNAME.  This field maps to the quest country.
         */
        private String countryName = null;

        /**
         * This holds the opicm SUBGROUPNAME.  This field maps to the quest brand.
         */
        private String subGroupName = null;


        /**
         * This holds the opicm FAMILYNAME.  This field and the opicm SERIESNAME map to the quest family.
         */
        private String familyName = null;

        /**
         * This holds the opicm SERIESNAME.  This field and the opicm FAMILYNAME map to the quest family.
         */
        private String seriesName = null;

        /**
         * This holds the opicm PROJECTNAME.  Currently, this does not map to a field in quest.
         */
        private String projectName = null;

        /**
         * This holds the opicm offering part number.  This field maps to the quest partnum.
         */
        private String offeringPartNum = null;
        /**
         * This holds the opicm offering part number description.
         */
        private String offeringPnumbDesc = null;

        /**
         * This holds the opicm solution marketing description.
         */
        private String marketDesc = null;

        /**
         * This holds the opicm variant solution part number.
         */
        private String variantPartNum = null;

        /**
         * This holds the opicm country variant part number description.
         */
        private String variantPartNumDesc = null;

        /**
         * This holds the opicm actual Announcement date.
         */
        private String actualAnnounceDate = null;

        /**
         * This holds the opicm target Announcement date.
         */
        private String targetAnnounceDate = null;

        /**
         * This holds the opicm actual Announcement date for the country.
         */
        private String cntyAnnounceDate = null;

        /**
         * This holds the opicm actual Preannouncement date for the country.
         */
        private String preAnnounceDate  = null;

        /**
         * This holds the opicm actual Available date (GA date).
         */
        private String gaAvailabilityDate = null;

        /**
         * This holds the opicm actual Withdrawn date.
         */
        private String withdrawnDate = null;

        /**
         * This holds the opicm actual End of Service date.
         */
        private String endOfServiceDate = null;

        /**
         * This holds the opicm actual End of Production date.
         */
        private String endOfProdDate = null;

        /**
         * This holds the opicm acutal End of Manufacturing date.
         */
        private String endOfManufactDate = null;

        /**
         * This holds the opicm actual End of Marketing date.
         */
        private String endOfMarketDate = null;

        /**
         * This holds the opicm actual First Order date.
         */
        private String firstOrderDate = null;

        /**
         * This holds the opicm actual Ship Supported date.
         */
        private String shipSupportedDate = null;

        /**
         * This holds the opicm target Preannouncement date for the counntry.
         */
        private String targCntyAnnounceDate = null;

        /**
         * This holds the opicm target Preannouncement date for the country.
         */
        private String targPreAnnounceDate = null;

        /**
         * This holds the opicm target Availability date (GA date).
         */
        private String targGaAvailabilityDate = null;

        /**
         * This holds the opicm target Withdrawn date.
         */
        private String targWithdrawnDate = null;

        /**
         * This holds the opicm target End of Service date.
         */
        private String targEndOfServiceDate = null;

        /**
         * This holds the opicm target End of Production date.
         */
        private String targEndOfProdDate = null;

        /**
         * This holds the opicm target End of Manufacturing date.
         */
        private String targEndOfManufactDate = null;

        /**
         * This holds the opicm target End of Marketing date.
         */
        private String targEndOfMarketDate=null;

        /**
         * This holds the opicm target First Order date.
         */
        private String targFirstOrderDate=null;

        /**
         * This holds the opicm target Ship Supported date.
         */
        private String targShipSupportedDate=null;

        /**
         * This holds the opicm status of a product.
         */
        private String status =null;

        /**
         * This holds the opicm timestamp of when the record was last changed. This is equivalent to the changedtime in quest.
         */
        private String valFrom= null;
    } //end of class OpicmProduct
} //end of class ProdGenManager

