/*
 *                
 * @(#)BatchUtil.java	1.0 06/05/01
 *                                        
 * @Copyright  2001 IBM          
 *                                      
 */

package com.ibm.nzna.projects.batch.utils;

import java.util.*; 
import java.io.*; 
import java.sql.*;
import com.ibm.nzna.projects.batch.*; 
import com.ibm.nzna.projects.batch.exception.*;
import com.ibm.nzna.projects.batch.plugin.*; 
import com.ibm.nzna.shared.ftp.FTP.*; 
import com.ibm.nzna.shared.ftp.*; 
import com.ibm.appsure.*; 


/**
 * This class defines static utility functions for the batch package
 * @author  Steve Wang                                     
 */

public class BatchUtil {
    /**
     * Subsititue the 'src' string with 'text' when the 'pat' matches
     *
     * @param src  The source string
     * @param pat  The pattern string
     * @param text The subsititute text string for the pattern
     * @return String The new string after substitution
     */
    public static String substitute (String src, String pat, String text) {
        int fromIndex = 0;
        int i = -1;
        int patlen = pat.length();
        String temp = new String();
        while ((i=src.indexOf(pat, fromIndex))>=0) {
          temp += src.substring (fromIndex, i);
          temp += text;
          fromIndex = i + pat.length();
        }
        if (fromIndex < src.length()-1) {
          temp += src.substring (fromIndex, src.length());
        }
        return temp;
   }

   /**
    * Get current time with the format year (xxxx)-date(xx)-month(xx) 
    * Allowed format: m-d-y, y-m-d, d-m-y.  Default to m-d-y
    * 
    * @param format The format string. It can be one of these: m-d-y, y-m-d, d-m-y.
    * @return String The current time string for that format
    */
    public static String getCurrTime (String format) {
        Calendar ca = Calendar.getInstance();
        String y = Integer.toString(ca.get(Calendar.YEAR)); 
        String m = new String();
        String d = new String();

        // Calendar's month starting with 0
        int n = ca.get(Calendar.MONTH)+1;
        if (n<=9) {
            m = "0"+Integer.toString(n); 
        } else {
            m = Integer.toString(n); 
        }
        
        n = ca.get(Calendar.DATE);
        if (n<=9) {
            d = "0"+Integer.toString(n); 
        } else {
            d = Integer.toString(n); 
        }

        if (format.compareToIgnoreCase("m-d-y")==0) {
            return m + "-" + d + "-" + y; 
        } else if (format.compareToIgnoreCase("y-m-d")==0) {
            return y + "-" + m + "-" + d; 
        } else if (format.compareToIgnoreCase("d-m-y")==0) {
            return d + "-" + m + "-" + y; 
        } else {
            return m + "-" + d + "-" + y; 
        }
    }

   /**
    * Get current year (xxxx), date(xx) or month(xx) 
    *
    * @param int The Calendar.xxx integer
    * @return String The current year, date or month
    */
    public static String getCurrTime (int t) {
        Calendar ca = Calendar.getInstance();
        switch (t) {
        case Calendar.YEAR:
            return Integer.toString(ca.get(Calendar.YEAR)); 
        case Calendar.MONTH:
            // Calendar's month starting with 0
            int m = ca.get(Calendar.MONTH)+1;
            if (m<=9) {
                return "0"+Integer.toString(m); 
            } else {
                return Integer.toString(m); 
            }
        case Calendar.DATE:
            int d = ca.get(Calendar.DATE);
            if (d<=9) {
                return "0"+Integer.toString(d); 
            } else {
                return Integer.toString(d); 
            }
        default:
            break;
        }
        return Integer.toString(ca.get(Calendar.YEAR)); 
    }

    /**
     * Get the db2 sytle timestamp for a given calendar object. An example of a db2
     * timestamp:  2001-07-12-03.04.44.540184  
     *
     * @param cal A calendar object
     * @return String A db2 style timestamp
     */
    public static String getDb2Timestamp (Calendar cal) {
        int year    = cal.get(Calendar.YEAR); 
        int month   = cal.get(Calendar.MONTH) + 1; // month starting with 0
        int date    = cal.get(Calendar.DATE);
        int hour    = cal.get(Calendar.HOUR_OF_DAY);
        int minute  = cal.get(Calendar.MINUTE);
        int second  = cal.get(Calendar.SECOND); 
        int millisec= cal.get(Calendar.MILLISECOND);

        return 
            Integer.toString(year)        + "-" + 
            getTwoDigitIntAsString(month) + "-" + 
            getTwoDigitIntAsString(date)  + "-" + 
            getTwoDigitIntAsString(hour)  + "." + 
            getTwoDigitIntAsString(minute)+ "." + 
            getTwoDigitIntAsString(second)+ "." + 
            getMilliSecAsString(millisec); 
    } 

    /**
     * Convert Java timestamp object to a db2 timestamp string
     *
     * @param jstamp A java timestamp 
     * @return String A db2 timestamp string
     */
    public static String jstampToDb2Timestamp (Timestamp jstamp) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(jstamp);
        return getDb2Timestamp(cal);
    }

    /** 
     * Convert db2 timestamp to java sql timestamp
     * 
     * @param db2stamp A db2 timestamp (i.e., in the format of 2001-07-12-03.04.44.000184)
     * @return String A java sql timestamp (i.e, format of 2001-07-12 03:04:44.184)
     */
    public static String db2stampTojstamp (String db2stamp) {
        String jstamp = db2stamp.substring(0, 10) + " "           // 2001-07-12 
            + db2stamp.substring(11, 19).replace('.', ':') + "."; // 03:04:44.

        // Get the remaining
        String rem = db2stamp.substring(20, 26);

        int i = 0; 
        // Find the next none 0 pos, that's where to cut off all the leading 0's
        for (; i<rem.length(); i++) {
            if (rem.charAt(i)!='0') {
                break;
            }
        }

        if (i==6) {
            // All digits are 0
            return jstamp + "0"; 
        } else {
            return jstamp + rem.substring(i, 6); 
        }

        /*
        StringTokenizer st = new StringTokenizer(db2stamp, "-");
        String year  = st.nextToken();
        String month = st.nextToken();
        String date  = st.nextToken();
        String more  = st.nextToken(); 

        st. = new StringTokenizer(more, ".");
        String hour = st.nextToken();
        String min  = st.nextToken();
        String sec  = st.nextToken();
        String msec = st.nextToken();

        return year+"-"+month+"-"+date+" "+hour+":"
        */
    }

    /** 
     * Get the db date mm/dd/yyyy in the comparable formate of
     * yyyy-mm-dd
     *
     * @param date The db date
     * @param String The new date format
     */
    public static String getCompTime(String stamp) {
        StringTokenizer st = new StringTokenizer(stamp, "/");
        if (st.hasMoreTokens()) {
            String month = st.nextToken();
            String date  = st.nextToken();
            String year  = st.nextToken();
            return year+"-"+month+"-"+date; 
        }
        // not in the format, return the original 
        return stamp; 
    }

    /**
     * Get the string rep of a two digits integer. If the integer is in single digit, a 
     * '0' is padded in the first digit. 
     *
     * @param int The integer
     * @return String The string rep of the integer such as xx
     */
    public static String getTwoDigitIntAsString (int i) {
        if (i<=9) {
            return "0"+Integer.toString(i); 
        } else {
            return Integer.toString(i); 
        }
    }

    /**
     * The function gets the 6 digits string for a millisecond. Use '0' as padding
     *
     * @param int A millisec integer
     * @return String A six digits string
     */
    public static String getMilliSecAsString (int ms) {
        String s = Integer.toString(ms);
        int len = s.length(); 
        for (int i=0; i<6-len; i++) {
            s = "0" + s; 
        }
        return s; 
    }
    
    /*
     * Combine the description of a vector or set of TypeCategoryRec into to a string with
     * format of "left_brace description right_brace separator ...."
     * Suppose e1, e2 ... represents the description of each typecategory Rec, then
     * an example would be "e1, e2, ...", where left_brace=right_brace="" and 
     * separor = ", ". 
     * Another example would be: "{e1} {e2} ...", where left_brace="{", 
     * right_brace="}" and separator = " ".  
     *
     * @param tcr A TypeCategoryRec (Object) vector or set
     * @param left_brace The left brace of the description
     * @param right_brace The right brace of the description
     * @param sep The seperator between the description
     * @return String A combined string of the type category list
     */
    /*
    public static String getLists (Object tcr, String left_brace, String right_brace, String sep) {
        String descpt = new String();
        // Make sure the object tcr has toArray implemented
        if (tcr instanceof java.util.Collection) {
            int i = 0; 
	    int size = ((Collection)tcr).size(); 
            Iterator it = ((Collection)tcr).iterator(); 
            while (it.hasNext()) {
                descpt += left_brace + it.next().toString() + right_brace; 
                if (i<(size-1)) {
                    descpt += sep; 
                }
                i++; 
            }
        }
        return descpt; 
    }
    */
    public static String getLists (Object tcr, String left_brace, String right_brace, String sep) {
        // expecting the tcr is a vector of quest docId of MIGR-4TEST type 
        // Make sure the object tcr has toArray implemented
        StringBuffer sbuff = new StringBuffer(); 
        if (tcr instanceof java.util.Collection) {
            int i = 0; 
	    int size = ((Collection)tcr).size(); 
            Iterator it = ((Collection)tcr).iterator(); 
            int curr_len = 0; 
            while (it.hasNext()) {
                sbuff.append(left_brace + it.next().toString() + right_brace); 
                if (i<(size-1)) {
                    sbuff.append(sep); 
                }
                i++; 
            }
        }
        return sbuff.toString(); 
    }  

    /*
     * Combine the description of a vector or set of TypeCategoryRec into to a string with
     * format of "left_brace description right_brace separator ...."
     * Suppose e1, e2 ... represents the description of each typecategory Rec, then
     * an example would be "e1, e2, ...", where left_brace=right_brace="" and 
     * separor = ", ". 
     * Another example would be: "{e1} {e2} ...", where left_brace="{", 
     * right_brace="}" and separator = " ".  
     *
     * @param tcr A TypeCategoryRec (Object) vector or set
     * @param invalidSet Only tcr object not in this set will be in the list
     * @param left_brace The left brace of the description
     * @param right_brace The right brace of the description
     * @param sep The seperator between the description
     * @return String A combined string of the type category list
     */
    public static String getValidLists (Vector tcr, TreeSet inValidSet, String left_brace, String right_brace, String sep) {
                // expecting the tcr is a vector of quest docId of MIGR-4TEST type 
        StringBuffer sbuff = new StringBuffer(); 
        // Make sure the object tcr has toArray implemented
        if (tcr instanceof java.util.Collection) {
            int i = 0; 
	    int size = ((Collection)tcr).size(); 
            Iterator it = ((Collection)tcr).iterator(); 
            while (it.hasNext()) {
                String docId = it.next().toString(); 
                if (!inValidSet.contains(docId)) {
                    sbuff.append(left_brace + docId + right_brace); 
                    if (i<(size-1)) {
                        sbuff.append(sep); 
                    }
                }  
                i++; 
            }
        }
        return sbuff.toString();  
    }  
    /*
     * Get the valid product list for esupport and bp
     *
     * @param tcr A TypeCategoryRec (Object) vector or set
     * @param key The parent level key of tcr object (such as if tcr is machine record, levelKey 
     *        is the correponding brandFamilyKey
     * @param prodSet Valid products  
     * @param left_brace The left brace of the description
     * @param right_brace The right brace of the description
     * @param sep The seperator between the description
     * @return String A combined string of the type category list
     */
    public static String getValidProdLists (Object tcr, Object levelKey, TreeSet prodSet, String left_brace, String right_brace, String sep) {
        StringBuffer sbuff = new StringBuffer(); 
        if (tcr instanceof java.util.Collection) {
            int i = 0; 
	    int size = ((Collection)tcr).size(); 
            Iterator it = ((Collection)tcr).iterator(); 
            while (it.hasNext()) {
                //if (key instanceof BFMModelKey) {  // not possible
                //} else 
                // Constuct the product key and test if it's in prodSet
                Object childkey = null; 
                CategoryRec rec = (CategoryRec)it.next(); 
                if (levelKey instanceof BFMachineKey) {
                    // From saveMachine() call
                    //String model = it.next().toString(); 
                    childkey =  new BFMModelKey(((BFMachineKey)levelKey).brandInd, 
                                                ((BFMachineKey)levelKey).familyInd, 
                                                ((BFMachineKey)levelKey).machine, 
                                                rec.toString()); 
                } else if (levelKey instanceof BrandFamilyKey) {
                    // From saveFamily() call
                    //String machine = it.next().toString(); 
                    childkey =  new BFMachineKey(((BrandFamilyKey)levelKey).brandInd, 
                                                 ((BrandFamilyKey)levelKey).familyInd, 
                                                 rec.toString()); 
                } else if (levelKey instanceof BrandKey) {
                    // From saveBrand() call 
                    childkey =  new BrandFamilyKey(((BrandKey)levelKey).brandInd, rec.getInd()); 
                } else {
                    BatchUtil.logit("  Invalid call to this method. Key " + levelKey.toString(), Constants.WARNING, 0); 
                }
                if (childkey!=null && prodSet.contains(childkey)) {
                    sbuff.append(left_brace + rec.toString() + right_brace); 
                    if (i<(size-1)) {
                        sbuff.append(sep); 
                    }
                }  
                i++; 
            }
        }
        return sbuff.toString();  
    }  
    /** 
     * For debug purpose. Getting the content of a TreeMap
     */
    public static String getTreeMapContent(TreeMap tmap) {
        Set keys = tmap.keySet(); 
        String ret = "\nThe treemap Key Set =  " + BatchUtil.getLists(keys, "{", "}", " ");
        Iterator keyIt = keys.iterator(); 
        while (keyIt.hasNext()) {
            Object key = keyIt.next();
            Object content = tmap.get(key); 
            if (content instanceof Set) {
                // for printing familyChildMap etc. 
                Set set = (Set)tmap.get(key); 
                ret += "\nTreeMap Set =  " + BatchUtil.getLists(set, "{", "}", " ");
            } else if (content instanceof String) {
                // For printing productMap
                ret += "\nTreeMap String =  " + (String)content;
            } else if (content instanceof Hashtable) {
                // For printing prodDocsMap
                ret += "\nTreeMap hastable with treemap key: " + key.toString(); 
                Set hkeys = ((Hashtable)content).keySet();
                ret += "\n\tHastable docCatTypeKey set = " + BatchUtil.getLists(hkeys, "[", "]", " ");; 
                Iterator hkeyIt = hkeys.iterator();
                while (hkeyIt.hasNext()) {
                    DocTypeCatKey hkey = (DocTypeCatKey)hkeyIt.next();
                    TreeSet docIds = (TreeSet)((Hashtable)content).get(hkey);
                    ret += "\n\tHastable DocIds = " + BatchUtil.getLists(docIds, "(", ")", " ");; 
                }
            }
        }
        return ret; 
    }

    /** 
     * Print hashtable -- for debug purpose
     */
    public static String getHashtableContent(Hashtable htab) {
        // For printing hDocs
        String ret = "Hastable: "; 
        Set hkeys = htab.keySet();
        Iterator hkeyIt = hkeys.iterator();
        while (hkeyIt.hasNext()) {
            DocTypeCatKey hkey = (DocTypeCatKey)hkeyIt.next();
            Vector docIds = (Vector)htab.get(hkey);
            ret += "\n\tKey = " + hkey.toString() + " DocIds = " + BatchUtil.getLists(docIds, "(", ")", " ");; 
        }
        return ret; 
    }

    /**
     * Find the description given a type ind for a Vector of CategoryRec
     *
     * @param tvec The vector of CategoryRec objects
     * @param typeInd The typeInd of a CategoryRec 
     * @return String The description 
     */
    public static String findDescript (Vector tvec, int typeInd) {
        for (int i=0; i<tvec.size(); i++) {
            Object rec = tvec.elementAt(i);
            if (rec instanceof CategoryRec) {
                CategoryRec tr = (CategoryRec)tvec.elementAt(i); 
                if (typeInd==tr.getInd()) {
                    return tr.getDescript(); 
                }
            }
        }
        return null;  
    }
    
    /**
     * Create a file with given filename and content
     * 
     * @param fname The full name (path+name) of a file to be created
     * @param html_doc The content to be written to the file
     * @return none
     */
    public static void createDoc (String fname, String html_doc) throws BatchException {
        try {
            FileOutputStream fout = new FileOutputStream(fname);
            fout.write (html_doc.getBytes()); 
            fout.close();
        } catch (Exception e) {
            throw new OutputDocException("Error creating doc " + fname + ": " + e, 
                                         Constants.ERROR); 
        }
    }

    /**
     * Delete files with a path and list of file names
     *
     * @param path The path of the files
     * @param flist The list of files to be deleted
     * @return int Number of deleted files
     */
    public static int deleteDocs (String path, Vector flist) throws BatchException {
        //Set the deleted file counter to 0
        int num_del = 0; 
        String full_name = ""; 
        try {
            for (int i=0; i<flist.size(); i++) {
                full_name = path + (String)flist.elementAt(i);

                //Construct the file with the abstract path full_name
                //Delete any file which exists
                File f = new File(full_name); 
                if (f.exists()) {
                    f.delete();
                    num_del++; 
                    BatchUtil.logit("\nDeleted file " + full_name, Constants.DEBUG, 0); 
                }
            }
        } catch (Exception e) {
            throw new DeleteFileException("Error deleting file " + full_name + ": " + e, 
                                          Constants.WARNING);
        }
        return num_del; 
    }

    /*
     * Ftp list of files
     *
     * @param host      The ftp host to connect to
     * @param host_dir  The remote directory 
     * @param loc_dir   The locale directory
     * @param user_id   The ftp user id
     * @param password  Password to login
     * @param file_type The file type to be ftped. Either ascii or image
     * @param load_type Put or Get
     * @param fnames    A list of files to be ftped
     * @return int      The total number of files ftped
     */
    public static int ftpFiles  (String host,       //ftp host name
                                 String host_dir,   //host dir to ftp to
                                 String loc_dir,    //local dir where file resides
                                 String user_id,    //ftp user id
                                 String password,   //ftp password
                                 String file_type,  //transfer type: bin or ascii
                                 String load_type,  //put or get
                                 Vector fnames)     //files to be ftped
                          throws BatchFtpException {  

        // If the host name is null, don't do ftp (comment out the ftp host
        // properties will do this)
        if (host.length()==0) {
            logit("The ftp host name is not specified. Files not ftped", Constants.INFORMATIONAL, 0);
            return 0; 
        }

        logit("Host: " + host, Constants.INFORMATIONAL, 0);
        logit("Host Dir: " + host_dir, Constants.INFORMATIONAL, 0);
        logit("Local Dir: " + loc_dir, Constants.INFORMATIONAL, 0);
        logit("User id: " + user_id, Constants.INFORMATIONAL, 0);
        
        //Set ftped file count to 0
        int num_ftp = 0; 

        //Set ftp paramerters
        Ftp ftp = new Ftp(); 
        ftp.setHost(host);
        ftp.setUser(user_id);
        ftp.setPassword(password); 

        try {
            logit("Connecting...", Constants.INFORMATIONAL, 0);
            
            ftp.connect();
            ftp.login();
            ftp.command("SITE umask 000"); 

            logit("Connected to " + host, Constants.INFORMATIONAL, 0);
            
            if (file_type.compareToIgnoreCase("ascii") == 0) {
                ftp.setType(FtpClient.ASCII); 
                logit("Transfer type set to ASCII", Constants.INFORMATIONAL, 0);
            } else {
                ftp.setType(FtpClient.IMAGE); 
                logit("Transfer type set to IMAGE", Constants.INFORMATIONAL, 0);
            }
            
            if (ftp.isConnected()) {
                ftp.cd(host_dir); 
                logit("Changed to Remote dir: " + host_dir, Constants.INFORMATIONAL, 0);

                //Going through the file list and to upload/download each one
                for (int i=0; i<fnames.size(); i++) {
                    String loc_file = loc_dir + /*File.separator +*/ (String)fnames.elementAt(i); 
                    if (load_type.compareToIgnoreCase("put") == 0) {
                        try {
                            ftp.put(loc_file); 
                            logit("Put file " + loc_file, Constants.DEBUG, 0);
                            num_ftp++; 
                        } catch (Exception ex) {
                            logit("Error in PUT file "+loc_file, ex, Constants.ERROR, 0);
                        }
                    } else { 
                        try {
                            ftp.get((String)fnames.elementAt(i), loc_file); 
                            logit("Get file " + loc_file, Constants.DEBUG, 0);
                            num_ftp++; 
                        } catch (Exception ex) {
                            logit("Error in GET file " + loc_file, ex, Constants.ERROR, 0);
                        }
                    }
                }
                ftp.disconnect();
            }
        } catch (FileNotFoundException e) {
            logit("Host or local file/dir not found", e, Constants.ERROR, 0);
        } catch (Exception e) {
            logit("Ftp Error: ", e, Constants.ERROR, 0);
            //throw new BatchFtpException("Ftp Error: " + e.toString(), Constants.ERROR); 
        } 
        return num_ftp; 
    }

    /**
     * log label. 
     *
     * @param level The log level
     * @param int Control (may be used for component
     * @return String A label in front of all the messsages. It has the format of 
     *        D000000 where D denotes for debugging, I for informaltional, E for
     *        error and F for fatal error. The number for hour, minute and seconds
     */
    public static String getLabel(int level, int ctrl) {
        Calendar ca = Calendar.getInstance();
        int h = ca.get(Calendar.HOUR_OF_DAY); 
        String m = getTwoDigitIntAsString(ca.get(Calendar.MINUTE)); 
        String s = getTwoDigitIntAsString(ca.get(Calendar.SECOND)); 
        switch (level) {
        case Constants.DEBUG:
            return "D"+h+m+s+" "; 
        case Constants.INFORMATIONAL:
            return "I"+h+m+s+" "; 
        case Constants.WARNING:
            return "W"+h+m+s+" "; 
        case Constants.ERROR:
            return "E"+h+m+s+" "; 
        case Constants.FATAL:
            return "F"+h+m+s+" "; 
        }
        return "X"+h+m+s+" "; 
    }

    /**
     * logit
     *
     * @param msg message to log
     * @param level log level
     * @param ctrl control to be used to connect this log to the logsystem 
     */
    public static void logit (String msg, int level, int ctrl) {
        // If the specified level is lower than the logLevel, don't log
        if (level<BatchInit.getLogLevel()) {
            return; 
        }

        // Get the log prefix label
        String stamp = getLabel(level, ctrl);

        // Print on screen
        if (level >= Constants.DEBUG) {
           System.out.println (stamp+msg); 
        }

        // Put in log files
        if (level >= Constants.FATAL) {
            BatchInit.logSystem.logSevereError(stamp+msg);
            BatchInit.batchFatal++;

            // call appsure.logFailure 
            callAppsure(ctrl, stamp);
        } else if (level >= Constants.ERROR) {
            BatchInit.logSystem.logError(stamp+msg);
            BatchInit.batchError++; 

            // call appsure.logFailure 
            callAppsure(ctrl, stamp);
        } else if (level >= Constants.WARNING) {
            BatchInit.logSystem.logProgress(stamp+msg, 0, 0);
            BatchInit.batchWarning++;
        } else if (level >= Constants.INFORMATIONAL) {
            BatchInit.logSystem.logProgress(stamp+msg, 0, 0);
        } else {
            BatchInit.logSystem.logProgress(stamp+msg, 0, 0);
        }
    }

    /**
     * logit exception to log exceptions
     *
     * @param msg  String message 
     * @param ex An exception to log 
     * @param level log level
     * @param ctrl control to be used to connect this log to the logsystem 
     */
    public static void logit (String msg, Exception ex, int level, int ctrl) {
        // If the specified level is lower than the logLevel, don't log
        if (level<BatchInit.getLogLevel()) {
            return; 
        }

        // Get the log prefix label
        String stamp = getLabel(level, ctrl);

        // Print on screen 
        if (level >= Constants.DEBUG) {
            ex.printStackTrace(); 
        }

        // Save to the error log
        if (level >= Constants.FATAL) {
            BatchInit.logSystem.logSevereError(stamp+msg);
            BatchInit.logSystem.logSevereError(ex);
            BatchInit.batchFatal++;

            callAppsure(ctrl, stamp);
        } else if (level >= Constants.ERROR) {
            BatchInit.logSystem.logSevereError(stamp+msg);
            BatchInit.logSystem.logSevereError(ex);
            BatchInit.batchError++; 
            
            callAppsure(ctrl, stamp);
        } 
    }

    // call appsure.logFailure
    private static boolean callAppsure(int ctrl, String stamp) {
        // call appsure.logFailure 
        try {
            return BatchInit.appSure.logFailure(ctrl, BatchInit.logSystem.getErrorFile(), null);
        } catch (AppSureException appEx) {
            BatchInit.logSystem.logSevereError(stamp+"Appsure failed in logFailure()");
            BatchInit.batchFatal++;
        } catch (Exception ex) {
            BatchInit.logSystem.logSevereError(stamp+"Appsure failed in logFailure()");
            BatchInit.logSystem.logSevereError(ex);
            BatchInit.batchFatal++;
        }

        return false; 
    }
}

