package com.ibm.nzna.projects.common.quest ; 
/*-----------------------------------------------------------*/
/*   Constants                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Constants                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import java.util.* ;
import java.sql.* ;
import COM.ibm.db2.app.* ;
/**
 * This class holds methods for retrieving constants from the TIGRIS.CONSTANTS table 
 *
 *
 * @version 1.0, 3/24/1998 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   Constants                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class Constants extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Vector constVec = new Vector ( 100, 100 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constructor                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public Constants () {
   }





   /*------------------------------------------------------------*/
   /* refreshConstants
    */
   /**
    *    This method will refresh all of the currently
    *  cached constants with the constants from
    *  the database
    *
    *  @param log The LogSystem for error logging
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean refreshConstants ( LogSystem log ) {
      boolean   rc      = false ;
      String    rtnName = "refreshConstants" ;
      int       sqlCode = 0 ;
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;

      constVec.removeAllElements() ;

      try {
         Statement stmt = sql.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT CONSTANT, INTCONST, STRCONST, APPLICATION "+
                                               "FROM TIGRIS.CONSTANTS" ) ;

            while ( rs.next() ) {
               constVec.addElement ( new ConstantsRec ( rs.getString(1).trim(),      // Constant
                                                        rs.getInt(2),                // int Constant
                                                        rs.getString(3).trim(),      // Str Constant
                                                        rs.getString(4).trim() ) ) ; // Application
            }

            rs.close() ;
         }

         rc = true ;

      } catch ( Exception e ) {
         if ( log != null ) {
            log.log ( 1, "Exception while reading constants" ) ;
            log.log ( 1, e ) ;
         } else {
            System.out.println ( "Exception while reading constants" ) ;
            e.printStackTrace() ;
         }
         sql.rollBack() ;
         rc = false ;
      }

      sql.close() ;
      return( rc ) ;

   }


   /*-----------------------------------------------------------*/
   /*   getStrConst                                             */
   /*                                                           */
   /*   Description:                                            */
   /*      Returns the String version of a constant             */                                                     
   /*                                                           */
   /*   Parameters:                                             */
   /*      constant - The constant to get                       */                      
   /*                                                           */
   /*   Returns:                                                */
   /*      String                                               */
   /*-----------------------------------------------------------*/
   public static String getStrConst ( String constant ) {
      int          length      = constVec.size() ;
      int          i           = 0 ;
      ConstantsRec constantsRec = null ;
      boolean      found       = false ;
      String       retStr      = null ;

      if ( constant != null ) {
         while ( ( i < length ) && ( !found ) ) {
            constantsRec = (ConstantsRec) constVec.elementAt(i) ;
            if ( ( constantsRec != null ) && ( constantsRec.constant.equals ( constant ) ) )
               found = true ;
            else
               i++ ;
         }

         if ( found )
            retStr = constantsRec.strConst ;
      }

      return( retStr ) ;
   }

   /*-----------------------------------------------------------*/
   /*   getIntConst                                             */
   /*                                                           */
   /*   Description:                                            */
   /*      Returns the Integer version of a constant            */                                                     
   /*                                                           */
   /*   Parameters:                                             */
   /*      constant - The constant to get                       */                      
   /*                                                           */
   /*   Returns:                                                */
   /*      int                                                  */
   /*-----------------------------------------------------------*/
   public static int getIntConst ( String constant ) {
      int          length      = constVec.size() ;
      int          i           = 0 ;
      ConstantsRec constantsRec = null ;
      boolean      found       = false ;
      int          rc          = 0 ;

      if ( constant != null ) {
         while ( ( i < length ) && ( !found ) ) {
            constantsRec = (ConstantsRec) constVec.elementAt(i) ;
            if ( ( constantsRec != null ) && ( constantsRec.constant.equals ( constant ) ) )
               found = true ;
            else
               i++ ;
         }

         if ( found )
            rc = constantsRec.intConst ;
      }

      return( rc ) ;
   }

   /*-----------------------------------------------------------*/
   /*   getApplicationConst                                     */
   /*                                                           */
   /*   Description:                                            */
   /*      Returns the application version of a constant        */                                                     
   /*                                                           */
   /*   Parameters:                                             */
   /*      constant - The constant to get                       */                      
   /*                                                           */
   /*   Returns:                                                */
   /*      String                                               */
   /*-----------------------------------------------------------*/
   public static String getApplicationConst ( String constant ) {
      int          length      = constVec.size() ;
      int          i           = 0 ;
      ConstantsRec constantsRec = null ;
      boolean      found       = false ;
      String       retStr      = null ;

      if ( constant != null ) {
         while ( ( i < length ) && ( !found ) ) {
            constantsRec = (ConstantsRec) constVec.elementAt(i) ;
            if ( ( constantsRec != null ) && ( constantsRec.constant.equals ( constant ) ) )
               found = true ;
            else
               i++ ;
         }

         if ( found ) {
            retStr = constantsRec.application ;
         }
      }

      return( retStr ) ;
   }

   /*-----------------------------------------------------------*/
   /*   getConstantVector                                       */
   /*                                                           */
   /*   Description:                                            */
   /*      Returns a vector holding all of the constants        */                                                          
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public Vector getConstantVector () {
      return( constVec ) ;
   }

   /*-----------------------------------------------------------*/
   /*   constRec                                                */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will add a constant                      */                                            
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public static void addConstants ( ConstantsRec constRec ) {
      constVec.addElement ( constRec ) ;
   }

   /*-----------------------------------------------------------*/
   /*   removeConstants                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      Removes a constant from the vector                   */                                               
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public static void removeConstants ( ConstantsRec constRec ) {
      constVec.removeElement ( constRec ) ;
   }


} /* Constants */
