package com.ibm.nzna.projects.common.quest ;
/*-----------------------------------------------------------*/
/*   Counter                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Counter                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import java.sql.* ;
/**
 * This class holds static methods to manipulate TIGRIS.COUNTERS.
 * All methods in this class assume that a DatabaseSystem object
 * has already been created.
 *
 *
 * @version 1.0, 4/9/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   Counter                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class Counter extends Object {




   /*------------------------------------------------------------*/
   /* getCounter
    */
   /**
    *    This method will read a counter from the
    *  TIGRIS.COUNTER table. It will then update
    *  that counter.
    *
    *  @param log The LogSystem to use for error trapping
    *  @param counter The string name of the counter to return
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public static int getCounter ( LogSystem log, String counter ) {
      String    rtnName = "getCounter ( "+counter+" )" ;
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      int       ret     = 0 ;

      try {
         Statement stmt = sql.createStatement() ;                            


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If stmt is good, we have a connection                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT VALUE "+
                                               "FROM TIGRIS.COUNTER "+
                                               "WHERE COUNTER = '"+counter+"' "+
                                               "FOR UPDATE OF VALUE, DBUSER, CHANGEDTIME" ) ;          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Read in the counter                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.next() )
               ret = rs.getInt(1) ;


            rs.close() ;
            stmt.close() ;

            stmt = sql.createStatement() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, immediatly update the counter                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.executeUpdate ( "UPDATE TIGRIS.COUNTER "+
                                 "SET VALUE = "+(ret+1)+" "+
                                 "WHERE COUNTER = '"+counter+"'" ) ;

         } /* if stmt is good */

      } catch ( SQLException e ) {
         sql.rollBack() ;
         if ( log != null )
            log.log ( 1, e ) ;
         else
            e.printStackTrace() ;
      } /* catch */

      sql.close() ;

      return( ret ) ;
   } /* getCounter */



   /*------------------------------------------------------------*/
   /* getCounter
    */
   /**
    *    This method will read a counter from the
    *  TIGRIS.COUNTER table. It will then update
    *  that counter.
    *  <p>
    *  This version of getCounter is meant to be used to get
    *  a counter value from within a Java stored procedure
    *  where the DatabaseSystem object is not available.  
    *
    *  @param conn    The Connection object
    *  @param counter The string name of the counter to return
    *  @exception java.sql.SQLException if DB2 error occurs
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public static int getCounter ( Connection conn, String counter ) throws SQLException {
      int       ret  = 0;
      Statement stmt = null;

      try {
         stmt = conn.createStatement();                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If stmt is good, we have a connection                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT VALUE "+
                                               "FROM TIGRIS.COUNTER "+
                                               "WHERE COUNTER = '"+counter+"' "+
                                               "FOR UPDATE OF VALUE, DBUSER, CHANGEDTIME" ) ;          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Read in the counter                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.next() )
               ret = rs.getInt(1) ;

            rs.close() ;
            stmt.close() ;

            stmt = conn.createStatement() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, immediatly update the counter                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.execute ( "UPDATE TIGRIS.COUNTER "+
                           "SET VALUE = "+(ret+1)+" "+
                           "WHERE COUNTER = '"+counter+"'" ) ;

         } /* if stmt is good */
      } finally {
         if ( stmt != null ) stmt.close();
      }

      return( ret ) ;
   } /* getCounter */


} /* Counter */

