package com.ibm.nzna.projects.common.quest.brand ;
/*-----------------------------------------------------------*/
/*   Brands                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Brands                              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
import java.sql.* ;
/**
 * This object holds associations between brands and families. It 
 * holds static methods usefull for getting family and brand associations
 *
 * @version 1.0, 3/20/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Brands                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class Brands {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the main hard-coded brand indicator for options */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int BRAND_OPTIONS     = 6 ;
   public static final int BRAND_MONITORS    = 5 ;
   public static final int BRAND_CROSS       = 2 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Vector      brandsVec         = null ;
   private static boolean     cache             = false ;
   private static String      cacheDir          = "" ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the Brands object.
    *  It will not read the associations by default. To
    *  read the associations, you must call 'readAssociations'
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Brands () {
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the Brands
    *  class so that the associations will be read
    *  from cache first, then database. The default is to
    *  have cache turned off.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Brands ( boolean newCache ) {
      cache = newCache ;
   }





   /*------------------------------------------------------------*/
   /* readAssociations
    */
   /**
    *    If caching is turned on, this method will
    *  check for any updates in the Quest database. If
    *  there are updates it will read all associations, if there
    *  are not updates and the cache property is turned On,
    *  then this method will simply read the Associations from the
    *  brands_assoc.cache file
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean readAssociations ( LogSystem log ) {
      TypeCategory.getCategories ( TypeCategory.LEVEL_FAMILY ) ;
      TypeCategory.getCategories ( TypeCategory.LEVEL_BRAND ) ;
      TypeCategory.getCategories ( TypeCategory.LEVEL_PROJECT ) ;
      return( true ) ;
   }

   /*------------------------------------------------------------*/
   /* readAssociationsFromDatabase
    */
   /**
    *    This method will read in the associations from
    *  the Quest database. Please note that this method 
    *  requires that a DatabaseSystem be created.
    *
    *  @param log The LogSystem to use for error logging
    *  @param None
    *  @param None
    *
    *  @return true if everything was read ok
    *//*---------------------------------------------------------*/
   private static boolean readAssociationsFromDatabase ( LogSystem log ) {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* associationsHaveChanged
    */
   /**
    *    This method will determine if any associations
    *  have changed since the last cache time.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean associationsHaveChanged ( String lastCacheTime, LogSystem log ) {
      boolean changed = false ;

      return( changed ) ;
   }





   /*------------------------------------------------------------*/
   /* readAssociationsFromCache
    */
   /**
    *    This method will read in the associations
    *  from the cache file. If the cache file is not
    *  present it will call the readAssociationsFromDatabase method
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean readAssociationsFromCache ( LogSystem log ) {
      boolean read = false ;

      return( read ) ;
   }


   /**
    * This method will return the brands that are in the Quest Database. Each
    * brand returned is a TypeCategoryRec.
    */
   public static Vector getBrands () {
      return( TypeCategory.getCategories ( TypeCategory.LEVEL_BRAND ) ) ;
   }

   /**
    * This method will return the parent brand to the family passed
    * in
    **/
   public static TypeCategoryRec getBrandFromFamily ( TypeFamilyRec familyRec ) {
      return( (TypeCategoryRec)TypeCategory.getCategory ( familyRec.getParentCatInd (), TypeCategory.LEVEL_BRAND ) ) ;
   }

   /**
    * This method will return the parent brand of the family indicator
    * passed in
    **/
   public static TypeCategoryRec getBrandFromFamily ( int familyInd ) {
      return( getBrandFromFamily ( (TypeFamilyRec)TypeCategory.getCategory ( familyInd, TypeCategory.LEVEL_FAMILY ) ) ) ;
   }

   /**
     * This method will return all families under a brand
     **/
   public static Vector getFamilies ( TypeCategoryRec brandRec ) {
      return( TypeCategory.getChildCategories ( brandRec.getInd(), TypeCategory.LEVEL_FAMILY ) ) ;
   }

   /**
     * This method will return all families under all brands
     */
   public static Vector getFamilies () {
      return( TypeCategory.getCategories ( TypeCategory.LEVEL_FAMILY ) ) ;
   }

   /**
     * This method will return all families under a brand
     **/
   public static Vector getFamilies ( int brandInd ) {
      return( TypeCategory.getChildCategories ( brandInd, TypeCategory.LEVEL_FAMILY ) ) ;
   }


   /**
     * This method will create a new brand in the Quest database.
     **/
   public static TypeCategoryRec createBrand ( String descript ) {
      return( null ) ;
   }

   /**
    * This method will return NULL if an error occurs. It will return a valid
    * TypeFamilyRec object if a family was created under the brand passed in
    **/
   public static TypeFamilyRec createFamily ( TypeCategoryRec brandRec, String descript ) {
      return( createFamily ( brandRec.getInd(), descript ) ) ;
   }

   /**
    * This method will return NULL if an error occurs. It will return a valid
    * TypeFamilyRec object if a family was created under the brand passed in
    **/
   public static TypeFamilyRec createFamily ( int brandInd, String descript ) {
      return( null ) ;
   }

   /**
    * This method will delete a family from the database
    */
   public static boolean deleteFamily ( int familyInd ) {
      return( false ) ;
   }

   /**
     * This method will delete a family from the database
     **/
   public static boolean deleteFamily ( TypeFamilyRec familyRec ) {
      return( deleteFamily ( familyRec.getInd() ) ) ;
   }

   /**
     * This method will save all of the changed associations. To
     * make a change to an association, the TypeCategoryRec or TypeFamilyRec
     * must have been changed prior to calling this method
     **/
   public static boolean saveAssociations () {
      return( false ) ;
   }

   /**
     * This method will return the owner of a brand
     **/
   public static String getOwner (int brandInd) {
      return( TypeCategory.getCategory ( brandInd, TypeCategory.LEVEL_BRAND ).getOwner() ) ;
   }


   /*------------------------------------------------------------*/
   /* getCacheDir
    */
   /**
    *    This method will return the fully qualified
    *  path to where the Cache files are written
    *  and read from, including the last Path Separator
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getCacheDir () {
      return( null ) ;
   }


   /*------------------------------------------------------------*/
   /* setCacheDir
    */
   /**
    *    This method will set the directory
    *  that the cache files are written and read
    *  from
    *
    *  @param dir The full path excluding the trailing path separator
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setCacheDir ( String dir ) {
      if ( dir == null )
         cacheDir = "" ;
      else
         cacheDir = dir ;
   }

   /**
    * This method will return the TypeCategoryRec of the brandInd
    * passed in
    **/
   public static TypeCategoryRec getBrand ( int brandInd ) {
      return( (TypeCategoryRec)TypeCategory.getCategory ( brandInd, TypeCategory.LEVEL_BRAND ) ) ;
   }

   /**
     * This method will return the TypeFamilyRec of the familyInd
     * passed in
     */
   public static TypeFamilyRec getFamily ( int familyInd ) {
      return( (TypeFamilyRec)TypeCategory.getCategory ( familyInd, TypeCategory.LEVEL_FAMILY ) ) ;
   }

   /**
     * This method will return the first matching brand to the
     * name passed in
     *
     * @param brandName The name of the brand
     */
   public static TypeCategoryRec getBrand ( String brandName ) {
      return( (TypeCategoryRec) TypeCategory.getCategory ( brandName, TypeCategory.LEVEL_BRAND ) ) ;
   }

   /**
     * This method will return the first matching family to the
     * name passed in
     *
     * @param familyName The name of the family
     */
   public static TypeCategoryRec getFamily ( String familyName ) {
      return( (TypeCategoryRec) TypeCategory.getCategory ( familyName, TypeCategory.LEVEL_FAMILY ) ) ;
   }


   /**
     *  This method will return TRUE if the brand passed is a brand of OPTION
     **/
   public static boolean isBrandOption ( TypeCategoryRec brandRec ) {
      if ( brandRec != null )
         return( isBrandOption ( brandRec.getInd() ) ) ;
      return ( false ) ;
   }

   /**
     *  This method will return TRUE if the brand indicator passed in is a OPTION
     **/
   public static boolean isBrandOption ( int brandInd ) {
      return( brandInd == 6 ) ;
   }



   /**
     *  This method will simply return all Machines and SubCategories (as TypeCategoryRec) objects
     * that fall under the family indicator given.
     *
     * @param familyInd The TypeCatInd of the family to look for
     *
     * @return Vector
     **/
   public static Vector getMachinesSubCategoriesFromFamily ( int familyInd ) {
      Vector      returnVec = null ;
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getMachinesSubCategoriesFromFamily ( "+familyInd+" )", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.TYPECATIND, A.DESCRIPTION, A.PARENTCATIND, A.CATEGORYLEVEL, "+
                                              "       A.ORDERNUM,   A.CHILDSORT,   A.OWNER,        A.VALID "+
                                              "FROM PRODUCT.TYPECATEGORY A, "+
                                              "     PRODUCT.TYPECATCHILD B "+
                                              "WHERE A.CATEGORYLEVEL in ( '"+TypeCategory.LEVEL_MACHINE+"', '"+TypeCategory.LEVEL_SUBCATEGORY+"') AND "+
                                              "      A.VALID = 'Y' AND "+
                                              "      A.TYPECATIND = B.TYPECATIND AND "+
                                              "      B.PARENTCATIND = "+familyInd+" FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            TypeCategoryRec categoryRec = new TypeCategoryRec ( rs.getInt(1) ) ;

            categoryRec.setDescript      ( rs.getString(2).trim() ) ;
            categoryRec.setParentCatInd  ( rs.getInt(3) ) ;
            categoryRec.setCategoryLevel ( rs.getString(4).trim() ) ;
            categoryRec.setOrderNum      ( rs.getInt(5) ) ;
            categoryRec.setChildSort     ( rs.getString(6) ) ;
            categoryRec.setOwner         ( rs.getString(7).trim() ) ;
            categoryRec.updateRecStatus  ( TypeCategoryRec.REC_NOCHANGE ) ;

            if ( returnVec == null )
               returnVec = new Vector ( 1 ) ;

            returnVec.addElement ( categoryRec ) ;
         }

         rs.close() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   }


   /**
     *  This method will simply return all Machines and SubCategories (as TypeCategoryRec) objects
     * that fall under the project indicator given.
     *
     * @param projectInd The TypeCatInd of the project to look for
     *
     * @return Vector
     **/
   public static Vector getMachinesSubCategoriesFromProject ( int projectInd ) {
      Vector      returnVec = null ;
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getMachinesSubCategoriesFromProject ( "+projectInd+" )", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.TYPECATIND, A.DESCRIPTION, A.PARENTCATIND, A.CATEGORYLEVEL, "+
                                              "       A.ORDERNUM,   A.CHILDSORT,   A.OWNER,        A.VALID "+
                                              "FROM PRODUCT.TYPECATEGORY A, "+
                                              "     PRODUCT.TYPECATCHILD B "+
                                              "WHERE A.CATEGORYLEVEL in ( '"+TypeCategory.LEVEL_MACHINE+"', '"+TypeCategory.LEVEL_SUBCATEGORY+"') AND "+
                                              "      A.VALID = 'Y' AND "+
                                              "      A.TYPECATIND = B.TYPECATIND AND "+
                                              "      B.PARENTCATIND = "+projectInd+" FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            TypeCategoryRec categoryRec = new TypeCategoryRec ( rs.getInt(1) ) ;

            categoryRec.setDescript      ( rs.getString(2).trim() ) ;
            categoryRec.setParentCatInd  ( rs.getInt(3) ) ;
            categoryRec.setCategoryLevel ( rs.getString(4).trim() ) ;
            categoryRec.setOrderNum      ( rs.getInt(5) ) ;
            categoryRec.setChildSort     ( rs.getString(6) ) ;
            categoryRec.setOwner         ( rs.getString(7).trim() ) ;
            categoryRec.updateRecStatus  ( TypeCategoryRec.REC_NOCHANGE ) ;

            if ( returnVec == null )
               returnVec = new Vector ( 1 ) ;

            returnVec.addElement ( categoryRec ) ;
         }

         rs.close() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   }

   /**
     *  This method will simply return all Machines and SubCategories (as TypeCategoryRec) objects
     * that fall under the project indicator given.
     *
     * @param projectInd The TypeCatInd of the project to look for
     *
     * @return Vector
     **/
   public static Vector getDraftMachinesSubCategoriesFromProject ( int projectInd ) {
      Vector      returnVec = null ;
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDraftMachinesSubCategoriesFromProject ( "+projectInd+" )", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.TYPECATIND, A.DESCRIPTION, A.PARENTCATIND, A.CATEGORYLEVEL, "+
                                              "       A.ORDERNUM,   A.CHILDSORT,   A.OWNER,        A.VALID "+
                                              "FROM PRODRAFT.TYPECATEGORY A, "+
                                              "     PRODRAFT.TYPECATCHILD B "+
                                              "WHERE A.CATEGORYLEVEL in ( '"+TypeCategory.LEVEL_MACHINE+"', '"+TypeCategory.LEVEL_SUBCATEGORY+"') AND "+
                                              "      A.VALID = 'Y' AND "+
                                              "      A.TYPECATIND = B.TYPECATIND AND "+
                                              "      B.PARENTCATIND = "+projectInd+" FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            TypeCategoryRec categoryRec = new TypeCategoryRec ( rs.getInt(1) ) ;

            categoryRec.setDescript      ( rs.getString(2).trim() ) ;
            categoryRec.setParentCatInd  ( rs.getInt(3) ) ;
            categoryRec.setCategoryLevel ( rs.getString(4).trim() ) ;
            categoryRec.setOrderNum      ( rs.getInt(5) ) ;
            categoryRec.setChildSort     ( rs.getString(6) ) ;
            categoryRec.setOwner         ( rs.getString(7).trim() ) ;
            categoryRec.setIsDraft       ( true ) ;
            categoryRec.updateRecStatus  ( TypeCategoryRec.REC_NOCHANGE ) ;

            if ( returnVec == null )
               returnVec = new Vector ( 1 ) ;

            returnVec.addElement ( categoryRec ) ;
         }

         rs.close() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   }


   /**
     *  This method will return the TypeCategoryRec representing the 'Option' brand
     *
     * @return TypeCategoryRec
     **/
   public static TypeCategoryRec getOptionBrand () {
      return ( TypeCategory.getCategory (  BRAND_OPTIONS ) ) ;
   }

   /**
     *  This method will return the TypeCategoryRec representing the 'Option' brand
     *
     * @return TypeCategoryRec
     **/
   public static TypeCategoryRec getMonitorBrand () {
      return ( TypeCategory.getCategory (  BRAND_MONITORS ) ) ;
   }

   /**
     *  This method will return TRUE if the Brand Indicator passed is the Cross Brand indicator
     **/
   public static boolean isCrossBrand ( int ind ) {
      return ( ind == BRAND_CROSS ) ;
   }


   /**
     *  This method will return the TypeCategoryRec representing the 'Cross Brand'
     **/
   public static TypeCategoryRec getCrossBrand () {
      return ( TypeCategory.getCategory ( BRAND_CROSS ) ) ;
   }
} /* Brands */