package com.ibm.nzna.projects.common.quest.doc;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      CreateDocDraft
/*   Creation Date: APRIL 17, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
/**
 * This class simply holds all information about the attempt to
 * create a document draft. Since the creation of a document draft
 * can fail for many reasons this class was created.
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* CreateDocDraft
/*
/*----------------------------------------------------*/
public class CreateDocDraft {     

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  Nothing was done
     **/
   public static final int DRAFT_NONE              = 0 ;

   /**
     *  A Product Draft already exists for the Existing Product. The
     * productDraft property is populated with that draft.
     **/
   public static final int DRAFT_ALREADY_EXISTS    = 1 ;

   /**
     *  A Product Draft already exists, but the draft has been put in
     * the recycling bin. The productDraft property is populated with 
     * that draft and can be used to obtain information about who
     * recycled the productDraft and when.
     **/
   public static final int DRAFT_RECYCLED          = 2 ;

   /**
     *  A Product Draft already exists but is a Misfit product. The
     * productDraft property will be populated with the current record,
     * however Quest Input should not allow a user to edit this misfit.
     **/
   public static final int DRAFT_MISFIT            = 3 ;

   /**
     *  A new Product Draft was created. The productDraft property
     * holds that new Product Draft.
     **/
   public static final int DRAFT_CREATED           = 4 ;

   /**
     *  An SQL Error occurred while creating the new ProductDraft. You
     * may use the 'getSQLCode' method to obtain the SQL Error generated
     **/
   public static final int DRAFT_SQL_ERROR         = 5 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This is the DocumentDraft object that resulted in the
    * Draft Creation Process. This property may be null, for
    * example, if the creation process stopped because of an 
    * SQL Error. 
    */
   private DocumentDraft documentDraft;

   /**
    * If the returnCode for this document is DRAFT_SQL_ERROR 
    * then this property will contain the SQL Code that was
    * generated. 
    */
   private int sqlCode;

   /**
    * This is the return code for the Draft Creation process. This
    * property will contain one of the predefined DRAFT_ values
    * defined in this class. 
    */
   private int returnCode;

   /**
     * The Default Constructor initializes this class
     **/
   public CreateDocDraft ( int returnCode, int sqlCode, DocumentDraft documentDraft ) {
      this.returnCode   = returnCode ;
      this.sqlCode      = sqlCode ;
      this.documentDraft = documentDraft ;
   }

   /**
     *  This method will return the newly created DocumentDraft object
     **/
   public DocumentDraft getDocumentDraft() {
      return documentDraft;
   }

   /**
     *  This method will return the SQL Code that was generated if an SQL Code was
     * generated
     **/
   public int getSqlCode() {
      return sqlCode;
   }

   /**
     *  This method will return the returnCode from the creation of the draft process
     **/
   public int getReturnCode() {
      return returnCode;
   }
} /* CreateDocDraft */
