package com.ibm.nzna.projects.common.quest.doc;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      CreateDocumentDraft
/*   Creation Date: APRIL 17, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
/**
 * This class simply holds all information about the attempt to
 * create a document draft. Since the creation of a document draft
 * can fail for many reasons this class was created
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* CreateDocumentDraft
/*
/*----------------------------------------------------*/
public class CreateDocumentDraft {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Static Members                                          */     
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
      *  Nothing was done. If you receive this it is good to look
      * at the log files that the LogSystem is writing to
      **/
    public static final int DRAFT_NONE              = 0 ;

    /**
      *  A Document Draft already exists for the Existing Document. The
      * documentDraft property is populated with that draft.
      **/
    public static final int DRAFT_ALREADY_EXISTS    = 1 ;

    /**
      *  A Document Draft already exists, but the draft has been put in
      * the recycling bin. The documentDraft property is populated with 
      * that draft and can be used to obtain information about who
      * recycled the documentDraft and when.
      **/
    public static final int DRAFT_RECYCLED          = 2 ;


    /**
      *  A new Document Draft was created. The documentDraft property
      * holds that new Document Draft.
      **/
    public static final int DRAFT_CREATED           = 4 ;

    /**
      *  An SQL Error occurred while creating the new DocumentDraft. You
      * may use the 'getSQLCode' method to obtain the SQL Error generated
      **/
    public static final int DRAFT_SQL_ERROR         = 5 ;


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
     * This is the DocumentDraft object that resulted in the
     * Draft Creation Process. This property may be null, for
     * example, if the creation process stopped because of an 
     * SQL Error. 
     */
    private DocumentDraft documentDraft;

    /**
     * If the returnCode for this document is DRAFT_SQL_ERROR 
     * then this property will contain the SQL Code that was
     * generated. 
     */
    private int sqlCode;

    /**
     * This is the return code for the Draft Creation process. This
     * property will contain one of the predefined DRAFT_ values
     * defined in this class. 
     */
    private int returnCode;

    /**
      * The Default Constructor initializes this class
      **/
    public CreateDocumentDraft ( int returnCode, int sqlCode, DocumentDraft documentDraft ) {
        this.returnCode   = returnCode ;
        this.sqlCode      = sqlCode ;
        this.documentDraft = documentDraft ;
    }

    public DocumentDraft getDocumentDraft() {
        return (documentDraft);
    }

    /**
      * If the getReturnCode method returns DRAFT_SQL_ERROR then
      * this method will return the DB/2 error 
      */
    public int getSqlCode() {
        return (sqlCode);
    }

    /**
      * This method will return the return code as defined in this class. This
      * return code will be either DRAFT_CREATED, DRAFT_SQL_ERROR, DRAFT_RECYCLED,
      * DRAFT_NONE or DRAFT_ALREADY_EXISTS
      *
      * @return int
      */
    public int getReturnCode() {
        return (returnCode); 
    }
} /* CreateDocumentDraft */
