package com.ibm.nzna.projects.common.quest.doc ;   
/*-----------------------------------------------------------*/
/*   DocReadTest                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocReadTest                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.db.* ;
import java.util.* ;
/**
 * This small program simply connects to the quest database 
 * and reads a document. The database information is
 * passed in from command line as is the document indicator
 *
 *
 * @version 1.0, 2/23/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocReadTest                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocReadTest extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String databaseName      = null ;
   private String server            = null ;
   private String userId            = null ;
   private String password          = null ;
   private String portNumber        = null ;
   private int    docInd            = 0 ;




   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This is the execution point
    *  
    *  
    *
    *  @param databaseName The name of the database
    *  @param Server The name of the server
    *  @param port The JDBC Port number
    *  @param userId The user id to connect with
    *  @param password The password to connect with
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String args[] ) {
      new com.ibm.nzna.shared.util.LogSystem ( 1, "DocReadTest.out" ) ;
      new DocReadTest ( args ) ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method kicks off our test
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocReadTest ( String args[] ) {
      try {
         databaseName=args[0] ;
         server=args[1] ;
         portNumber=args[2] ;
         userId=args[3] ;
         password=args[4] ;
         docInd = new Integer ( args[5] ).intValue() ;
      } catch ( Exception e ) {
         System.out.println ( "Usage:\njava com.ibm.nzna.projects.common.quest.doc.DocReader <databaseName> <serverName> <portNumber> <userId> <password> <doc indicator>" ) ;
         System.exit(0) ;
      }


      try {
         DatabaseSystem d = new DatabaseSystem ( false, null, false ) ;

         System.out.print ( "Connecting to Quest..." ) ;
         if ( d.createConnection ( DatabaseSystem.QUEST,
                                   databaseName,
                                   server,
                                   portNumber,
                                   userId,
                                   password ) ) {

             System.out.println ( "Done" ) ;

            for ( int i = docInd; i < (docInd+5); i++ ) {
                System.out.print ( "Reading Document :"+i ) ;
               Document doc         = new Document ( i ) ;
               long     startMillis = System.currentTimeMillis() ;
               int      rc          = doc.readFromDatabase() ;


               if ( rc == 0 )
                  System.out.println ( "Done" ) ; //printDocument ( doc ) ;
               else if ( rc == 100 )
                  System.out.println ( "Could not find document" ) ;
               else
                  System.out.println ( "Error" ) ;

               System.out.println ( "Total Time Taken:"+((System.currentTimeMillis()-startMillis))+"(ms)" ) ;
            }




         } else
            System.out.println ( "Could not connect to "+databaseName+" on "+server+" on port "+portNumber ) ;
      } catch ( Exception e1 ) {
         e1.printStackTrace() ;
         System.out.println ( "Could not connect to "+databaseName+" on "+server+" on port "+portNumber ) ;

      }
   }

   private static void printDocument ( Document doc ) {
      System.out.println ( "TITLE:         "+doc.getTitle() ) ;
      System.out.println ( "NOTES UID:     "+doc.getUniversalNotesId() ) ;
      if ( doc.getDocumentBody() != null )
         System.out.println ( "BODY LENGTH:   "+doc.getDocumentBody().toString().length() ) ;
      else
         System.out.println ( "BODY LENGTH:   NO BODY" ) ;
      System.out.println ( "DBUSER:        "+doc.getLastTouchedBy() ) ;
      System.out.println ( "CHANGEDTIME:   "+doc.getLastTouched() ) ;
      System.out.println ( "WEBABLE:       "+doc.getWebable() ) ;
      System.out.println ( "WEBONLY:       "+doc.getWebOnly() ) ;
      System.out.println ( "FAXABLE:       "+doc.getFaxable() ) ;
      System.out.println ( "EMAILABLE:     "+doc.getEMailable() ) ;
      System.out.println ( "EMAILABLEDATE: "+doc.getEMailableDate() ) ;

      if ( doc.getCategory() != null )
         System.out.println ( "CATEGORIES:    "+doc.getCategory().size()+" Categories associated" ) ;
      else
         System.out.println ( "CATEGORIES:    NONE" ) ;

      if ( doc.getFileAttachments() != null ) {
         Vector fileVec = doc.getFileAttachments() ;

         System.out.println ( "FILE ATTACHMENTS:\n------------------\n" ) ;
         for ( int i = 0; i < fileVec.size() ; i++ ) {
            System.out.println ( "File:"+((DocumentFile)fileVec.elementAt(i)).getFilename()+"\n   Location:"+((DocumentFile)fileVec.elementAt(i)).getLocation() ) ;
            System.out.println ( "   Size:"+((DocumentFile)fileVec.elementAt(i)).getSize() ) ;
         }
         System.out.println ( "" ) ;
      }

      if ( doc.getGraphics() != null ) {
         Vector fileVec = doc.getGraphics() ;

         System.out.println ( "GRAPHICS:\n------------------\n" ) ;
         for ( int i = 0; i < fileVec.size() ; i++ ) {
            System.out.println ( "Filename:"+((DocumentGraphic)fileVec.elementAt(i)).getFilename() ) ;
         }
         System.out.println ( "" ) ;
      }

   }
} /* DocReadTest */
