package com.ibm.nzna.projects.common.quest.doc ;
/*-----------------------------------------------------------*/
/*   Document                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Document                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
/**
 * This class represents a single document. Information 
 * in this class is stored in the TIGRIS.DOCUMENT table. All
 * subclasses of this class should implement java.io.Serializable<br><br>
 *
 * The Document object has a list of properties that will fire events when they
 * are changed. Since the document has so many properties, any DocumentPropertyListener
 * will be notified through 1 method that a property has changed and what the property
 * that changed was.<br><br>
 *
 * <b>FAQ Section</b><br>
 * <i>How do I remove a File Attachment?</i><br>
 * To remove a File Attachment, simply get the DocumentFile for the file attachment
 * you wish to remove and call 'documentFile.updateRecStatus ( documentFile.REC_DELETE )'.<br><br>
 *
 * <i>What does 'Applicable Brands, Families, Machines and Sub Categories' mea?</i><br>
 * Please note that the 'applicable'
 * brands, families and machines are not direct associations to this document.
 * but are applied through the direct association of either a brand, family, machine
 * or model. For example, if a document is associated to the IBM Product
 * 2168-57C, then that is all that is returned from Document.getModels. No brand is
 * returned from Document.getBrands() because the document is not associated to a brand,
 * it is associated to a model only. The 'Applied' Brands, Families, Machines and
 * Sub Categories are created from that direct association. So, in our example, the
 * getApplicableBrands() method would return 'IBM Aptiva' because the direct association
 * of the 2168-57C product makes that brand 'Applicable.<br><br>
 *
 * @version 1.0, 10/16/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Document                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class Document extends PersistentRec implements java.io.Serializable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Metric Types                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int METRIC_NOTAPPLICABLE             = 0 ;
   public static final int METRIC_SYSTEMSUPPORTDISK         = 1 ;
   public static final int METRIC_RETAINTIP                 = 2 ;
   public static final int METRIC_FEEDBACK                  = 3 ;
   public static final int METRIC_EDITORINPUT               = 4 ;
   public static final int METRIC_CMVC                      = 5 ;
   public static final int METRIC_RICK                      = 6 ;  



   /*-----------------------------------------------------------------*/
   /*                                                                 */
   /*  All Properties that are thrown to listeners of the Document    */
   /*                                                                 */
   /*-----------------------------------------------------------------*/
   public static final int TITLE                  = 1 ;
   public static final int BRAND                  = 2 ;
   public static final int FAMILY                 = 3 ;
   public static final int MACHINE                = 4 ;
   public static final int MODEL                  = 5 ;
   public static final int PART_NUMBER            = 6 ;
   public static final int FILE_ATTACHMENTS       = 7 ;
   public static final int FAQ                    = 8 ;
   public static final int FAXABLE                = 9 ;
   public static final int ADDITIONAL_INFO        = 10 ;
   public static final int AUTHORITY              = 11 ;
   public static final int APPLICABLE_BRAND       = 12 ;
   public static final int APPLICABLE_FAMILY      = 13 ;
   public static final int APPLICABLE_MACHINE     = 14 ;
   public static final int APPLICABLE_SUBCATEGORY = 16 ;
   public static final int SUBCATEGORY            = 17 ;
   public static final int CATEGORY               = 18 ;
   public static final int CREATE_DATE            = 19 ;
   public static final int DESCRIPTION            = 20 ;
   public static final int BODY                   = 21 ;
   public static final int CLASS                  = 22 ;
   public static final int LINKS                  = 23 ;
   public static final int TYPE                   = 24 ;
   public static final int EMAILABLE              = 25 ;
   public static final int EMAILABLEDATE          = 26 ;
   public static final int EMAILCOMMENTS          = 27 ;
   public static final int FILENAME               = 28 ;
   public static final int GRAPHICS               = 29 ;
   public static final int HOTNEWSDATE            = 30 ;
   public static final int KNOWLEDGEBASE          = 31 ;
   public static final int LASTTOUCHEDBY          = 32 ;
   public static final int LASTTOUCHEDON          = 33 ;
   public static final int LOCALE                 = 34 ;
   public static final int LOCKED                 = 35 ;
   public static final int METRIC                 = 36 ;
   public static final int MONTHS_TO_REVIEW       = 37 ;
   public static final int OWNER                  = 38 ;
   public static final int PUBLICATION_TYPE       = 39 ;
   public static final int QUERY_TEXT             = 40 ;
   public static final int RECYCLED               = 41 ;
   public static final int REFERRING_LINKS        = 42 ;
   public static final int RELEASE_DATE           = 43 ;
   public static final int SOURCE                 = 44 ;
   public static final int STATUS                 = 45 ;
   public static final int URL                    = 46 ;
   public static final int VERSION                = 47 ;
   public static final int WEBABLE                = 48 ;
   public static final int WEBONLY                = 49 ;

   /**
     *  This is a special property that will not get fired in the
     * property change events. This is only for the undo capabilities
     **/
   public static final int UNDO_BFTMSP_CHANGE     = 50 ; 
   public static final int UNDO_ADD_FILE          = 51 ;
   public static final int UNDO_REMOVE_FILE       = 52 ;


   public class FireThread extends Object implements Runnable {

      int      property  = 0 ;
      Vector   listeners = null ;
      Document document  = null ;

      public FireThread ( Document document, int property, Vector listeners ) {
         this.document  = document ;
         this.property  = property ;
         this.listeners = listeners ;

         new Thread ( this ).start() ;
      }

      public void run () {
         int i     = 0 ;
         int size  = listeners.size() ;

         while ( i < size ) {
            try {
               ((DocumentPropertyListener)listeners.elementAt(i)).documentChanged ( document, property ) ;
            } catch ( Exception e ) {
            }
            i++ ;
         }
      }
   }

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This is the vector which holds all undoable edits
     **/
   private Vector undoableEdits = null ;

   /**
     *  This property determines if undo events and objects will
     * be created when this document is altered. By default this is
     * false.
     **/
   private boolean undoEnabled = false ;

   /**
     *  This is the boolean property that enables/disables the 
     * firing of events when a property is changed
     **/
   private boolean fireEventEnabled = true ;

   /**
     * This is the primary key of Documents
     **/
   protected int docInd = 0 ;

   /**
     * This is the title of the document
     **/
   private String title = null ;

   /**
     * The Owner is the person responsible for this document. This is usually the User Id of a Brand 
     * Program Manager.
     **/
   private String owner = null ;

   /**
     *  This tells if this document is checked out
     **/
   private boolean maintLock = false ;

   /**
     * This is the Recycled Flag. If a Document Draft is Recycled it can be thought of
     * as deleted, however it can be resurrected
     */
   private boolean recycled = false ;


   /**
     * The source is the 'Source of Information', basically where the info in this document came from
     **/
   private String source = null ;

   /**
     *  This locked value determines if someone has this document open in the Quest database and
     * therefore no saves can take place
     **/
   private boolean locked = false ;

   /**
     * The filename is the name of the document on the web. Currently this goes with the Lotus Notes CTS Tips Id
     **/
   private String filename = null ;

   /**
     *  Holds all of the comments from 5 previous publishings of a document
     **/
   private Vector commentHistory = null ;

   /**
     *  This part number is used mostly for SSD and Service Publication documents. This Part Number is
     * in no way associated to the Quest PRODUCT schema
     **/
   private String partNumber = null ;

   /**
     * This is the last person who touched the document. This is the User's User Id
     **/
   private String lastTouchedBy   = null ;

   /**
     * This description is really only needed for SSDs. This describes the pack of downloads
     **/
   private String description = null ;

   /**
     * This is the time the document was last changed
     **/
   private String lastTouched = null ;

   /**
     * The Query Text can be used for the Search Engine to Index against.
     **/
   private DocumentQueryText queryText = null ;

   /**
     * The body is HTML code that represents the entire body of the document
     **/
   private DocumentBody body = null ;

   /**
     * Webable means that this document can go out to the web. Default is TRUE
     **/
   private boolean webable = true ;

   /**
     * Webonly means that this document should only go to the web, not CTS Tips. Default is FALSE
     **/
   private boolean webOnly = false ;

   /**
     * Faxable means that this document can be faxed through the Faxback System
     **/
   private boolean faxable = false ;

   /**
     * Hot News means that this document is part of the 'Hot News' items on the web sites
     **/
   private boolean hotNews = false ;

   /**
     * This date is the date that the hotNews should be considered old and obsolete
     **/
   private String hotNewsDate = null ;

   /**
     * eMailable means that this document can be eMailed through the Proactive eMail system
     **/
   private boolean eMailable = false ;

   /**
     * The eMailable date is the date that this document should be eMailed.
     **/
   private String eMailableDate = null ;

   /**
     * The FAQ flag determines if this document goes in the 'Frequently Asked Question' section of the web site
     **/
   private boolean faq = false ;


   /**
     * The KnowledgeBase flag determines if this document is used in the Avalon (OA) section of Quest at all
     **/
   private boolean knowledgeBase = false ;

   /**
     * The 'Months To Review' is the amount of time before this document needs to be reviewed for accuracy.
     **/
   private int monthsToReview = 12 ;


   /**
     * The eMail Comments are the comments that are sent in the body of the eMail.
     **/
   private String eMailComments = "" ;

   /**
     *  Additional information can be stored for any document, however currently should only be stored for
     *  a document of type Service Publication
     **/
   private String additionalInfo = null ;


   /**
     * This is the type of document classification. This is either Confidential, Unclassified or Dealer Confidential,
     * and is the actual indicator representing the classification. This information is stored
     * in the TIGRIS.TYPEDOCCLASS table. Defaults to Confidential
     **/
   private int docClass = TypeList.DOCCLASS_IBM_CONFIDENTIAL ;

   /**
     * This is the type of document. The different types of documents are defined in the TIGRIS.TYPEDOC
     * table. Defaults to Service Hint and Tip
     **/
   private int documentType = TypeList.DOCTYPE_SERVICE_HINTS_TIPS ;


   /**
     * The publication type is only meant for Document Types of Service Publication. Any attempt to set
     * this value with the document type not Service Publication will result in an IllegalArgumentException being thrown
     **/
   private int publicationType = 0 ;

   /**
     * The Metric is a simple means for the authors to get statistics on the document. The data
     * for metrics can be found in the TIGRIS.TYPEMETRIC table
     **/
   private int metric = 0 ;

   /**
     * The authority level needed to edit this document. The authority levels are defined in TIGRIS.TYPEAUTH
     **/
   private int authority = 0 ;

   /**
     * The status of the document. The different status' are defined in TIGRIS.TYPESTATUS with a status type of 1,
     * any other status that is applied to the document will result in an IllegalArgumentException error.
     **/
   private DocumentStatus status = null ;

   /**
     * This is a vector of DocumentGraphic. Each DocumentGraphic object is represented in the BODY of the document. Graphic
     * data is stored in the TIGRIS.GRAPHIC table. Documents are associated to graphics in the TIGRIS.DOCGRAPHIC table.
     **/
   private Vector graphics = null ;

   /**
     * This is a vector of DocumentFile objects. Each DocumentFile object represents a file attachment to this document,
     * currently FileAttachments are allowed for all types of documents.
     **/
   private Vector fileAttachments = null ;

   /**
     * The Document Categories for this document. Each item in the vector MUST be a TypeDocCatRec object,
     * the data for the Document Categories is stored in TIGRIS.TYPEDOCCAT and is associated to
     * documents in the TIGRIS.DOCCATEGORY table
     **/
   private Vector categories = null ;

   /**
     * Document Properties are external, dynamic properties that can be set on a document. These properties
     * are stored in the TIGRIS.DOCPROPERTY table. Each object in the vector MUST be a DocumentProperty
     * object.
     **/
   private Vector properties = null ;

   /**
     * The Document Links vector holds all of the documents that this document links to. Each object
     * in the Vector MUST be of DocumentLink.
     **/
   private Vector documentLinks = null ;

   /**
     * The Referring Document Links is a list of Documents that link to this document
     **/
   private Vector referringDocumentLinks = null ;

   /**
     * The document's locale information containing Geography and Country associations
     */
   private LocaleRec locale = null ;


   /**
     *  This vector holds the TypeCategoryRec objects representing the associated brands
     **/
   private Vector brands = null ;

   /**
     *  This vector holds the TypeCategoryRec objects representing the associated families
     **/
   private Vector families = null ;

   /**
     *  This vector holds the TypeCategoryRec objects representing the associated machines
     **/
   private Vector machines = null ;

   /**
     *  This vector holds the TypeCategoryRec objects representing the associated models
     **/
   private Vector models = null ;


   /**
     *  This flag tells the DocumentWriter if the Brand list has changed
     **/
   protected boolean brandUpdated = false ;

   /**
     *  This flag tells the DocumentWriter if the Family list has changed
     **/
   protected boolean familyUpdated = false ;

   /**
     *  This flag tells the DocumentWriter if the machine list has changed
     **/
   protected boolean machineUpdated = false ;

   /**
     *  This flag tells the DocumentWriter if the Model list has changed
     **/
   protected boolean modelUpdated = false ;

   /**
     * This vector holds TypeCategoryRec objects to families that are applicable
     * to this document. Please see the getApplicableFamilies method for more information
     **/
   private Vector appliedFamilies = null ;

   /**
     * This vector holds TypeCategoryRec objects to Brands that are applicable
     * to this document. Please see the getApplicableBrands method for more information
     **/
   private Vector appliedBrands = null ;

   /**
     * This vector holds TypeCategoryRec objects to Machines that are applicable
     * to this document. Please see the getApplicableMachines method for more information
     **/
   private Vector appliedMachines = null ;

   /**
     * This vector holds TypeCategoryRec objects to Sub Cateogries that are applicable
     * to this document. Please see the getApplicableSubCategories method for more information
     **/
   private Vector appliedSubCategories = null ;

   /**
     * This vector holds all registered listeners
     **/
   private Vector listeners = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor and
    *  creates a blank document. This document's
    *  recStatus will be set to REC_INESRT
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Document () {
      updateRecStatus ( REC_INSERT ) ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates a document
    *  with the specifed document indicator. This implies that this
    *  document already exists in the database
    *  
    *
    *  @param docInd The Document Indicator as defined in TIGRIS.DOCUMENTS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Document ( int docInd ) {
      this.docInd = docInd ;
      updateRecStatus ( REC_UPDATE ) ;
   }




   /*------------------------------------------------------------*/
   /* setDocumentLinks
    */
   /**
    *    This method will set the links in a Document.
    *  Each object in this vector <b>MUST</b> be
    *  a DocumentLink object
    *
    *  @param vector A Vector of DocumentLink Objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setDocumentLinks ( Vector vector ) {
      this.documentLinks = vector ;
      firePropertyChanged ( LINKS ) ;
   }




   /*------------------------------------------------------------*/
   /* getDocumentLinks
    */
   /**
    *    This method will return the document
    *  links that this document points to.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector A Vector of DocumentLink objects
    *//*---------------------------------------------------------*/
   public final Vector getDocumentLinks () {
      if ( documentLinks != null )
         return( (Vector) documentLinks.clone() ) ;
      return( new Vector () ) ;
   }


   /*------------------------------------------------------------*/
   /* setReferringDocumentLinks
    */
   /**
    *    This method will set the links in a ReferringDocument.
    *  Each object in this vector <b>MUST</b> be
    *  a ReferringDocumentLink object
    *
    *  @param vector A Vector of ReferringDocumentLink Objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setReferringDocumentLinks ( Vector vector ) {
      this.referringDocumentLinks = vector ;
      firePropertyChanged ( REFERRING_LINKS ) ;
   }




   /*------------------------------------------------------------*/
   /* getReferringDocumentLinks
    */
   /**
    *    This method will return the document
    *  links that this document points to.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector A Vector of ReferringDocumentLink objects
    *//*---------------------------------------------------------*/
   public final Vector getReferringDocumentLinks () {
      if ( referringDocumentLinks != null )
         return( (Vector) referringDocumentLinks.clone() ) ;
      return( new Vector () ) ;
   }





   /*------------------------------------------------------------*/
   /* getDocInd
    */
   /**
    *    This method will return the document indicator
    *  of this document. This is the primary key for
    *  this document in the database.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final int getDocInd () {
      return( docInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title of our
    *  document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String version of the Title
    *//*---------------------------------------------------------*/
   public final String getTitle () {
      return( title ) ;
   }





   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set the document's title.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setTitle ( String title ) throws IllegalArgumentException {
      int max = 254 ;

      if ( this.title == null )
         this.title="" ;

      if ( ( title != null ) && ( title.length() <= max ) ) {
         if ( ( this.title == null ) || ( !this.title.equals ( title ) ) ) {
            addUndo ( TITLE, this.title ) ;
            this.title = title.trim() ;
            updateRecStatus ( REC_UPDATE ) ;
            firePropertyChanged ( TITLE ) ;
         }
      } else
         throw new IllegalArgumentException ( "TITLE cannot be NULL or longer than "+max+" characters" ) ;
   } /* setTitle  */





   /*------------------------------------------------------------*/
   /* setOwner
    */
   /**
    *    This method will set the Owner of this
    *  document. Normally the Owner should be the
    *  Brand Program Manager but it really can be anyone.
    *
    *  @param owner The new owner
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setOwner ( String owner ) throws IllegalArgumentException  {
      int max = 8 ;

      if ( owner != null ) {
         if ( owner.length() <= max ) {
            if ( ( this.owner == null ) || ( !this.owner.equals ( owner ) ) ) {
               addUndo ( OWNER, this.owner ) ;
               this.owner = owner.trim() ;
               updateRecStatus ( REC_UPDATE ) ;
               firePropertyChanged ( OWNER ) ;
            }
         } else
            throw new IllegalArgumentException ( "OWNER cannot be NULL or longer than "+max+" characters. Current Length:"+(owner.length()) ) ;
      } else {
         this.owner = "" ;
      }
   } /* setOwner  */





   /*------------------------------------------------------------*/
   /* getOwner
    */
   /**
    *    This method will return the owner of this
    *  document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getOwner () {
      return( owner ) ;
   }





   /*------------------------------------------------------------*/
   /* setSource
    */
   /**
    *    This method will set the source of this
    *  document. 
    *  
    *
    *  @param source The Document's Source
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setSource ( String source ) throws IllegalArgumentException {

      if ( this.source == null )
         this.source="" ;

      if ( source != null ) {
         if ( !this.source.equals ( source ) ) {
            addUndo ( SOURCE, this.source ) ;
            this.source = source.trim() ;
            firePropertyChanged ( SOURCE ) ;
            updateRecStatus ( REC_UPDATE ) ;
         }
      } else
         this.source=null ;

   }


   /*------------------------------------------------------------*/
   /* getSource
    */
   /**
    *    This method will return our source property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getSource () {
      return( source ) ;
   }




   /*------------------------------------------------------------*/
   /* setFilename
    */
   /**
    *    This method will set the filename that this
    *  document publishes to when it goes to the web
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setFilename ( String filename ) {
      if ( this.filename == null ) {
         if ( filename != null ) {
            this.filename = filename.trim() ;
            firePropertyChanged ( FILENAME ) ;
         } else
            this.filename = null ;
      }
   }




   /*------------------------------------------------------------*/
   /* generateFilename
    */
   /**
    *    This method is currently private and is never
    *  called. Once CTS Tips is erased and we generate
    *  our own filenames, this method will generate the
    *  filename. This method also generates a DocInd if one
    *  has not yet been set.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void generateFilename () {
      if ( ( filename == null ) || ( filename.length() == 0 ) ) {
         filename = "MIGR-"+docInd ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   } /* generateFilename  */





   /*------------------------------------------------------------*/
   /* getFilename
    */
   /**
    *    This method will return our filename
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getFilename () {
      return( filename ) ;
   }




   /*------------------------------------------------------------*/
   /* getLastTouchedBy
    */
   /**
    *    This method will return the LastTouchedBy property.
    *  This is the User Id of the person who last touched
    *  this document
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final String getLastTouchedBy () {
      return( lastTouchedBy ) ;
   }




   /*------------------------------------------------------------*/
   /* setLastTouchedBy
    */
   /**
    *    This method will set the LastTouchedBy property. This
    *  property is the User Id of the person who
    *  last touched the document
    *
    *  @param lastTouchedBy The User Id
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setLastTouchedBy ( String lastTouchedBy ) throws IllegalArgumentException {
      int max = 8 ;

      if ( ( lastTouchedBy != null ) && ( lastTouchedBy.length() <= max ) ) {
         if ( ( this.lastTouchedBy == null ) || ( !this.lastTouchedBy.equals ( lastTouchedBy ) ) ) {
            this.lastTouchedBy = lastTouchedBy.trim() ;
            updateRecStatus ( REC_UPDATE ) ;
            firePropertyChanged ( LASTTOUCHEDBY ) ;
         }
      } else
         throw new IllegalArgumentException ( "Last Touched By cannot be NULL or longer than "+max+" characters. Tried to set "+lastTouchedBy ) ;
   }




   /*------------------------------------------------------------*/
   /* getLastTouched
    */
   /**
    *    This method will return a timestamp of the
    *  time that this document was last touched
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final String getLastTouched () {
      return( lastTouched ) ;
   }




   /*------------------------------------------------------------*/
   /* setLastTouched
    */
   /**
    *    This method will set the time that this document
    *  was last updated
    *  
    *
    *  @param lastTouched The new Changed Time DB/2 Timestamp
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected final void setLastTouched ( String lastTouched ) {
      if ( lastTouched != null ) {
         this.lastTouched = lastTouched.trim() ;
         firePropertyChanged ( LASTTOUCHEDON ) ;
      } else
         this.lastTouched = null ;
   }



   /*------------------------------------------------------------*/
   /* getQueryText
    */
   /**
    *    This method will return the Search Engine
    *  Query Text
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final DocumentQueryText getQueryText () {
      return( queryText ) ;
   }




   /*------------------------------------------------------------*/
   /* setQueryText
    */
   /**
    *    This method will set the Query Text of
    *  this document.
    *  
    *
    *  @param queryText The new query Text
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setQueryText ( DocumentQueryText queryText ) throws IllegalArgumentException {
      if ( queryText != null ) {
         addUndo ( QUERY_TEXT, this.queryText ) ;
         this.queryText = queryText ;
         firePropertyChanged ( QUERY_TEXT ) ;
      } else
         throw new IllegalArgumentException ( "QUERYTEXT cannot be NULL" ) ;
   }




   /*------------------------------------------------------------*/
   /* getDocumentBody
    */
   /**
    *    This method will return the DocumentBody
    *  object for this document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final DocumentBody getDocumentBody () {
      return( body ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocumentBody
    */
   /**
    *    This method will set the DocumentBody
    *  object for this document
    *  
    *
    *  @param body The new DocumentBody object to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setDocumentBody ( DocumentBody body ) {
      addUndo ( BODY, this.body ) ;
      this.body = body ;
      firePropertyChanged ( BODY ) ;
   }





   /*------------------------------------------------------------*/
   /* setWebable
    */
   /**
    *    This method will either turn the webable
    *  property on or off. If this property is turned off,
    *  this document will be erased from all Web Sites
    *
    *  @param webable TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setWebable ( boolean webable ) {
      addUndo ( WEBABLE, new Boolean ( this.webable ) ) ;
      if ( this.webable != webable ) {
         this.webable = webable ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( WEBABLE ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* setWebOnly
    */
   /**
    *    This method will set our web only
    *  property. If this property is TRUE then this
    *  document will only go to the web. It will not go to any other place
    *  that this document might go (CTS Tips etc..) 
    *
    *  @param webonly TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setWebOnly ( boolean webOnly ) {

      addUndo ( WEBONLY, new Boolean ( this.webOnly ) );
      if ( this.webOnly != webOnly ) {
         this.webOnly = webOnly ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( WEBONLY ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* getWebable
    */
   /**
    *    This method will return TRUE or
    *  FALSE if this document is webable or not
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final boolean getWebable () {
      return( webable ) ;
   }





   /*------------------------------------------------------------*/
   /* getWebOnly
    */
   /**
    *    This method will return our WebOnly
    *  property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public final boolean getWebOnly () {
      return( webOnly ) ;
   }





   /*------------------------------------------------------------*/
   /* setFaxable
    */
   /**
    *    This method will return the Faxable property
    *  on or off (TRUE/FALSE)
    *  
    *
    *  @param faxable (TRUE/FALSE)
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setFaxable ( boolean faxable ) {
      addUndo ( FAXABLE, new Boolean ( this.faxable ) ) ;
      if ( this.faxable != faxable ) {
         this.faxable = faxable ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( FAXABLE ) ;
      }
   } /* setFaxable  */





   /*------------------------------------------------------------*/
   /* getFaxable
    */
   /**
    *    This method will return our Faxable
    *  property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public final boolean getFaxable () {
      return( faxable ) ;
   }





   /*------------------------------------------------------------*/
   /* setHotNewsDate
    */
   /**
    *    This method will set our hot news date. If
    *  NULL, this document will no be considered
    *  hot news
    *
    *  @param hotNewsDate
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setHotNewsDate ( String hotNewsDate ) {

      addUndo ( HOTNEWSDATE, this.hotNewsDate ) ;
      this.hotNewsDate = hotNewsDate ;
      hotNews = ( hotNewsDate != null ) ;
      if ( hotNewsDate != null ) {
         if ( ( hotNewsDate.charAt(2) != '/' ) &&
              ( hotNewsDate.charAt(2) != '-' ) ) {
            hotNewsDate = CDate.createDate ( CDate.parse ( hotNewsDate, CDate.DATE_YEAR,  CDate.DATE_DBDATE ),
                                             CDate.parse ( hotNewsDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                             CDate.parse ( hotNewsDate, CDate.DATE_DAY,   CDate.DATE_DBDATE ),
                                             0,0,0, CDate.DATE_DATE ) ;
         }
         this.hotNewsDate = hotNewsDate.trim() ;
         firePropertyChanged ( HOTNEWSDATE ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getHotNews
    */
   /**
    *    This method will return our hot news
    *  property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final boolean getHotNews () {
      return( hotNews ) ;
   }




   /*------------------------------------------------------------*/
   /* getHotNewsDate
    */
   /**
    *    This method will return our hot news
    *  date
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String date
    *//*---------------------------------------------------------*/
   public final String getHotNewsDate () {
      return( hotNewsDate ) ;
   }





   /*------------------------------------------------------------*/
   /* setEMailable
    */
   /**
    *    This method will set our eMailable flag. If TRUE
    *  this document will be able to go out in
    *  proactive eMail
    *
    *  @param eMailable TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setEMailable ( boolean eMailable ) {
      addUndo ( EMAILABLE, new Boolean ( this.eMailable ) ) ;
      if ( this.eMailable != eMailable ) {
         this.eMailable = eMailable ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( EMAILABLE ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getEMailable
    */
   /**
    *    This method will return out eMailable
    *  property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return eMailable (TRUE/FALSE)
    *//*---------------------------------------------------------*/
   public final boolean getEMailable () {
      return( eMailable ) ;
   }




   /*------------------------------------------------------------*/
   /* setEMailableDate
    */
   /**
    *    This method will set the date in which this
    *  eMail is supposed to go out. If this date is NULL,
    *  the eMailable property is automatically set to FALSE
    *
    *  @param eMailableDate The new date
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setEMailableDate ( String eMailableDate ) {
      addUndo ( EMAILABLEDATE, this.eMailableDate ) ;

      if ( eMailableDate == null ) {
         this.eMailableDate = eMailableDate ;
         this.eMailable     = false ;

         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( EMAILABLEDATE ) ;
      } else {

         if ( ( eMailableDate.charAt(2) != '-' ) &&
              ( eMailableDate.charAt(2) != '/' ) ) {
            eMailableDate = CDate.createDate ( CDate.parse  ( eMailableDate, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                               CDate.parse  ( eMailableDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                               CDate.parse  ( eMailableDate, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                               0,
                                               0,
                                               0,
                                               CDate.DATE_DATE ) ;
         }

         this.eMailableDate = eMailableDate ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( EMAILABLEDATE ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getEMailableDate
    */
   /**
    *    This method will return our eMailableDate
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String date
    *//*---------------------------------------------------------*/
   public final String getEMailableDate () {
      return( eMailableDate ) ;
   }





   /*------------------------------------------------------------*/
   /* setFaq
    */
   /**
    *    This method will set our faq flag. If TRUE
    *  this document will be considered a 
    *  'FAQ'.
    *
    *  @param faq TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setFaq ( boolean faq ) {
      addUndo ( FAQ, new Boolean ( this.faq ) ) ;
      if ( this.faq != faq ) {
         this.faq = faq ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( FAQ ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getFaq
    */
   /**
    *    This method will return out faq
    *  property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return faq (TRUE/FALSE)
    *//*---------------------------------------------------------*/
   public final boolean getFaq () {
      return( faq ) ;
   }





   /*------------------------------------------------------------*/
   /* setKnowledgeBase
    */
   /**
    *    This method will set our knowledgeBase flag. If TRUE
    *  this document will be considered a 
    *  part of the OnlineAssistant.
    *
    *  @param knowledgeBase TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setKnowledgeBase ( boolean knowledgeBase ) {
      addUndo ( KNOWLEDGEBASE, new Boolean ( this.knowledgeBase ) ) ;
      if ( this.knowledgeBase != knowledgeBase ) {
         this.knowledgeBase = knowledgeBase ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( KNOWLEDGEBASE ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getKnowledgeBase
    */
   /**
    *    This method will return out knowledgeBase
    *  property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return knowledgeBase (TRUE/FALSE)
    *//*---------------------------------------------------------*/
   public final boolean getKnowledgeBase () {
      return( knowledgeBase ) ;
   }






   /*------------------------------------------------------------*/
   /* setMonthsToReview
    */
   /**
    *    This method will set our monthsToReview value
    *
    *  @param monthsToReview int
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setMonthsToReview ( int monthsToReview ) {
      addUndo ( MONTHS_TO_REVIEW, new Integer ( this.monthsToReview ) ) ;
      if ( this.monthsToReview != monthsToReview ) {
         this.monthsToReview = monthsToReview ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( MONTHS_TO_REVIEW ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getMonthsToReview
    */
   /**
    *    This method will return out monthsToReview
    *  property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return monthsToReview (int)
    *//*---------------------------------------------------------*/
   public final int getMonthsToReview () {
      return( monthsToReview ) ;
   }





   /*------------------------------------------------------------*/
   /* setEMailComments
    */
   /**
    *    This method will set the EMailComments of this
    *  document. Normally the EMailComments should be the
    *  eMailed in Proactive eMail
    *
    *  @param eMailComments The new eMailComments
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setEMailComments ( String eMailComments ) throws IllegalArgumentException  {
      int max = 1000 ;

      if ( eMailComments != null ) {
         if ( eMailComments.length() <= max ) {
            if ( ( this.eMailComments == null ) || ( !this.eMailComments.equals ( eMailComments ) ) ) {
               addUndo ( EMAILCOMMENTS, this.eMailComments ) ;
               this.eMailComments = eMailComments.trim() ;
               updateRecStatus ( REC_UPDATE ) ;
               firePropertyChanged ( EMAILCOMMENTS ) ;

            }
         } else
            throw new IllegalArgumentException ( "SUMMARY cannot be NULL or longer than "+max+" characters" ) ;
      }
   } /* setEMailComments  */





   /*------------------------------------------------------------*/
   /* getEMailComments
    */
   /**
    *    This method will return the eMailComments of this
    *  document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getEMailComments () {
      return( eMailComments ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocumentClass
    */
   /**
    *    This method will set the classification of
    *  this document. The classification must be a
    *  indicator defined in TIGRIS.TYPEDOCCLASS
    *
    *  @param docClass The Document Classification indicator
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setDocumentClass ( int docClass ) {
      if ( this.docClass != docClass ) {
         addUndo ( CLASS, new Integer ( this.docClass ) ) ;
         this.docClass = docClass ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( CLASS ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getDocumentClass
    */
   /**
    *    This method will return the document
    *  classification indicator of this document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The Document Classification as defined in TIGRIS.TYPEDOCCLASS
    *//*---------------------------------------------------------*/
   public final int getDocumentClass () {
      return( docClass ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocumentType
    */
   /**
    *    This method will set the type of document
    *  that this is. The Type is defined in TIGRIS.TYPEDOCUMENT
    *  
    *
    *  @param documentTypeInd The indicator as defined in TIGRIS.TYPEDOCUMENT
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setDocumentType ( int documentType ) {
      if ( this.documentType != documentType ) {
         addUndo ( TYPE, new Integer ( this.documentType ) ) ;
         this.documentType = documentType ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( TYPE ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getDocumentType
    */
   /**
    *    This method will return our type of
    *  document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final int getDocumentType () {
      return( documentType ) ;
   }





   /*------------------------------------------------------------*/
   /* setPublicationType
    */
   /**
    *    This method will set the PublicationType
    *  of Service Publication documents. 
    *
    *  @param int The Type of publication as defined in TIGRIS.TYPEPUBLICATION
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setPublicationType ( int publicationType ) throws IllegalArgumentException {
      if ( publicationType != this.publicationType ) {
         addUndo ( PUBLICATION_TYPE, new Integer ( this.publicationType ) ) ;
         this.publicationType = publicationType ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( PUBLICATION_TYPE ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getPublicationType
    */
   /**
    *    This method will return our publication
    *  type
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The publication indicator as defined in TIGRIS.TYPEPUBLICATION
    *//*---------------------------------------------------------*/
   public final int getPublicationType () {
      return( publicationType ) ;
   }





   /*------------------------------------------------------------*/
   /* setMetric
    */
   /**
    *    This method will set the metric of this document.
    *  The metrics are defined in TIGRIS.TYPEMETRIC
    *  
    *
    *  @param metric The Metric indicator as defined in TIGRIS.TYPEMETRIC
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setMetric ( int metric ) {
      if ( this.metric != metric ) {
         addUndo ( METRIC, new Integer ( this.metric ) ) ;
         this.metric = metric ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( METRIC ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getMetric
    */
   /**
    *    This method will return the metric of
    *  this document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return metric The Metric Indication as defined in TIGRIS.TYPEMETRIC
    *//*---------------------------------------------------------*/
   public final int getMetric () {
      return( metric ) ;
   }





   /*------------------------------------------------------------*/
   /* setAuthority
    */
   /**
    *    This method will set the authority of this document.
    *  The authoritys are defined in TIGRIS.TYPEAUTH
    *  
    *
    *  @param authority The Authority indicator as defined in TIGRIS.TYPEAUTH
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setAuthority ( int authority ) {
      if ( this.authority != authority ) {
         this.authority = authority ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( AUTHORITY ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getAuthority
    */
   /**
    *    This method will return the authority of
    *  this document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return authority The Authority Indication as defined in TIGRIS.TYPEAUTH
    *//*---------------------------------------------------------*/
   public final int getAuthority () {
      return( authority ) ;
   }





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set this document's current
    *  status
    *  
    *
    *  @param status The DocumentStatus object representing our status
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setStatus ( DocumentStatus status ) {
      addUndo ( STATUS, this.status ) ;
      this.status = status ;
      firePropertyChanged ( STATUS ) ;

   }




   /*------------------------------------------------------------*/
   /* getStatus
    */
   /**
    *    This method will return our current status in
    *  a DocumentStatus object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DocumentStatus
    *//*---------------------------------------------------------*/
   public final DocumentStatus getStatus () {
      return( status ) ;
   }





   /*------------------------------------------------------------*/
   /* getGraphics
    */
   /**
    *    This method will return a Vector of 
    *  DocumentGraphic objects. Each object represents
    *  a single graphic in the body of a document
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public final Vector getGraphics () {
      Vector returnGraphics = null ;

      if ( graphics != null ) {
         returnGraphics = (Vector)graphics.clone() ;
      }

      return( returnGraphics ) ;
   }




   /*------------------------------------------------------------*/
   /* setGraphics
    */
   /**
    *    This method will set the graphics of a Document.
    *  Each record in the vector passed in MUST
    *  be a DocumentGraphic.
    *
    *  @param graphics The Vector of graphics
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setGraphics ( Vector graphics ) {
      this.graphics = graphics ;
      firePropertyChanged ( GRAPHICS ) ;

   }





   /*------------------------------------------------------------*/
   /* getFileAttachments
    */
   /**
    *    This method will return all of the File Attachments
    *  for this document. This vector contains
    *  DocumentFile objects
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public final Vector getFileAttachments () {
      Vector returnFileAttachments = null ;
      if ( fileAttachments != null )
         returnFileAttachments = (Vector)fileAttachments.clone() ;
      return( returnFileAttachments ) ;
   }




   /*------------------------------------------------------------*/
   /* setFileAttachments
    */
   /**
    *    This method will set the file attachments for
    *  this document. Each record in the document must
    *  be a DocumentFile Object.
    *
    *  @param fileAttachments The Vector containing DocumentFile objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setFileAttachments ( Vector fileAttachments ) {
      addUndo ( FILE_ATTACHMENTS, this.fileAttachments ) ;
      this.fileAttachments = fileAttachments ;
      firePropertyChanged ( FILE_ATTACHMENTS ) ;

   }





   /*------------------------------------------------------------*/
   /* setCategory
    */
   /**
    *    This method will set the Document's Categories.
    *  Each record in the Vector passed in must be
    *  a TypeDocCatRec object.
    *
    *  @param categories The new Categories
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setCategory ( Vector categories ) {
      if ( categories != null )
         addUndo ( CATEGORY, this.categories ) ;
      else
         addUndo ( CATEGORY, new Vector ( 1 ) ) ;
      this.categories = (Vector)categories.clone() ;
      updateRecStatus ( REC_UPDATE ) ;
      firePropertyChanged ( CATEGORY ) ;

   }





   /*------------------------------------------------------------*/
   /* getCategory
    */
   /**
    *    This method will return a list of the categories
    *  that are associated to this document. The Vector
    *  returned contains TypeDocCatRec Objects.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public final Vector getCategory () {
      if ( categories != null )
         return( (Vector)categories.clone() ) ;
      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* generateKeys
    */
   /**
    *    This method will generate the primary keys
    *  for this document. Currently the primary keys
    *  includes only Document Indicator. If this document
    *  already has a document indicator assigned, nothing is done
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void generateKeys () {
      if ( docInd == 0 )
         docInd = Counter.getCounter ( LogSystem.getInstance(), "DOCIND" ) ;
   }





   /*------------------------------------------------------------*/
   /* readFromDatabase
    */
   /**
    *    This method will read this document from the
    *  database.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if read, FALSE if not read
    *//*---------------------------------------------------------*/
   public int readFromDatabase () {
      String    rtnName   = "readFromDatabase" ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      int       rc        = 0 ;

      if ( true ) {
         Connection conn = sqlMethod.getConnection() ;

         try {
            rc = DocumentReader.readDocument ( conn, this ) ;
         } catch ( Exception e ) {
            e.printStackTrace() ;
         }
      } else if ( sqlMethod != null ) {
         try {
            CallableStatement stmt  = sqlMethod.createStoredProc ( "QUEST.DocumentReader", 3 ) ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up the call                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.setInt               ( 1, getDocInd() ) ;
            stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
            stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            LogSystem.log ( 1, "Reading Document:"+getDocInd() ) ;
            stmt.execute() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the StoredProcRec from the Lob                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Throw an error if need be                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( retRec != null ) && ( retRec.error ) ) {
               LogSystem.beginLogSection ( "DocumentReader Returned Error" ) ;
               LogSystem.log ( 1, "sqlCode:   "+retRec.sqlCode ) ;
               LogSystem.log ( 1, "userError: "+retRec.userError ) ;
               LogSystem.log ( 1, "errorStr:  "+retRec.errorStr ) ;
               LogSystem.log ( 1, "error:     "+retRec.error ) ;
               try {
                  rc = new Integer ( retRec.sqlCode ).intValue() ;
               } catch ( Exception e1 ) {
                  LogSystem.log ( 1, "Cannot translate retRec.sqlCode to Integer for RC" ) ;
               }
               LogSystem.endLogSection () ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, continue                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get our return object                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               initializeProperties ( (Document) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ) ;
               forceRecStatus ( REC_NOCHANGE ) ;
               rc = 0 ;

            } /* Else if we have no error  */

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            rc = -1 ;
         }
      } /* if sqlMethod is not null  */

      sqlMethod.close() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getPartNumber
    */
   /**
    *    This method will return the part
    *  number field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getPartNumber () {
      return( partNumber ) ;
   }




   /*------------------------------------------------------------*/
   /* setPartNumber
    */
   /**
    *    This method will set the part number
    *  for this document. The part number
    *  cannot exceed 50 characters
    *
    *  @param partNumber The new value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setPartNumber ( String partNumber ) throws IllegalArgumentException {
      if ( partNumber != null ) {
         if ( partNumber.length() <= 50 ) {
            addUndo ( PART_NUMBER, this.partNumber ) ;
            this.partNumber = partNumber.trim() ;
            updateRecStatus ( REC_UPDATE ) ;
            firePropertyChanged ( PART_NUMBER ) ;

         } else
            throw new IllegalArgumentException ( "Part Number cannot exceed 50 bytes" ) ;

      } else {
         this.partNumber = null ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* getAdditionalInfo
    */
   /**
    *    This method will return the Additional
    *  info field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getAdditionalInfo () {
      return( additionalInfo ) ;
   }




   /*------------------------------------------------------------*/
   /* setAdditionalInfo
    */
   /**
    *    This method will set the Additional info
    *  for this document. The Additional info
    *  cannot exceed 32k
    *
    *  @param additionalInfo The new value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setAdditionalInfo ( String additionalInfo ) throws IllegalArgumentException {
      if ( additionalInfo != null ) {
         if ( additionalInfo.length() <= 32767 ) {
            addUndo ( ADDITIONAL_INFO, this.additionalInfo ) ;
            this.additionalInfo = additionalInfo.trim() ;
            updateRecStatus ( REC_UPDATE ) ;
            firePropertyChanged ( ADDITIONAL_INFO ) ;

         } else
            throw new IllegalArgumentException ( "Additional Information cannot exceed 32,767 bytes" ) ;

      } else {
         this.additionalInfo = null ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   }

   /*------------------------------------------------------------*/
   /* getLocked
    */
   /**
    *    This method will determine if this document is
    *  locked. The User Id passed in may be null. If the
    *  user is null, the DBUSER field will not be used in 
    *  determining if the document is truly locked by another
    *  individual
    *
    *  @param user The User Id to compare with, can be NULL
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final boolean getLocked ( String user ) {
      if ( ( user != null ) && ( lastTouchedBy != null ) ) {
         return( ( locked ) && ( lastTouchedBy.equals ( user ) ) ) ;
      }

      return( locked ) ;
   }





   /*------------------------------------------------------------*/
   /* setReleaseDate
    */
   /**
    *    This method will set the ReleaseDate property. The release
    * date passed MUST be in the form MM/DD/YYYY
    *  
    *  
    *
    *  @param releaseDate The property value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setReleaseDate ( String releaseDate ) {
      if ( releaseDate != null ) {
         DocumentProperty property = getProperty ( DocumentProperty.RELEASE_DATE ) ;

         addUndo ( RELEASE_DATE, property.getValue() ) ;
         property.setValue ( CDate.createDate ( CDate.parse ( releaseDate, CDate.DATE_YEAR, CDate.DATE_DATE ),
                                                CDate.parse ( releaseDate, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                                CDate.parse ( releaseDate, CDate.DATE_DAY, CDate.DATE_DATE ),
                                                0,0,0,CDate.DATE_DBDATE ) ) ;
         firePropertyChanged ( RELEASE_DATE ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getReleaseDate
    */
   /**
    *    This method will return the ReleaseDate
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getReleaseDate () {
      return( getProperty ( DocumentProperty.RELEASE_DATE ).getValue() ) ;
   }




   /*------------------------------------------------------------*/
   /* setURL
    */
   /**
    *    This method will set the URL property
    *  
    *  
    *
    *  @param url The property value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setURL ( String url ) {
      if ( url != null ) {
         DocumentProperty property = getProperty ( DocumentProperty.URL ) ;

         addUndo ( URL, property.getValue() ) ;
         property.setValue ( url ) ;
         firePropertyChanged ( URL ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getURL
    */
   /**
    *    This method will return the URL
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getURL () {
      return( getProperty ( DocumentProperty.URL ).getValue() ) ;
   }




   /*------------------------------------------------------------*/
   /* setVersion
    */
   /**
    *    This method will set the Version property
    *  
    *  
    *
    *  @param version The property value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setVersion ( String version ) {
      if ( version != null ) {
         DocumentProperty property = getProperty ( DocumentProperty.VERSION ) ;

         addUndo ( VERSION, property.getValue() ) ;
         property.setValue ( version ) ;
         firePropertyChanged ( VERSION ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getVersion
    */
   /**
    *    This method will return the Version
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getVersion () {
      return( getProperty ( DocumentProperty.VERSION ).getValue() ) ;
   }




   /*------------------------------------------------------------*/
   /* setDescription
    */
   /**
    *    This method will set the Description property
    *  
    *  
    *
    *  @param description The property value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setDescription ( String description ) {
      int              currentIndex = 0 ;
      int              length       = description.length() ;
      String           currentChunk = null ;
      int              i            = 1 ;
      DocumentProperty newProperty  = null ;

      addUndo ( DESCRIPTION, getDescription() ) ;                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, remove all of our properties                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      removeProperty ( DocumentProperty.DESCRIPTION ) ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, set them back                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      boolean cont = true ;
      while ( cont ) {
         if ( currentIndex+254 > length ) {
            currentChunk = description.substring ( currentIndex ) ;
            cont  = false ;
         } else {
            currentChunk = description.substring ( currentIndex, currentIndex+254 ) ;
            currentIndex+=254 ;
         }

         newProperty = new DocumentProperty ( getDocInd() ) ;
         newProperty.setPropertyInd ( DocumentProperty.DESCRIPTION ) ;
         newProperty.setOrderNum ( i ) ;
         newProperty.setValue ( currentChunk ) ;
         addProperty ( newProperty ) ;
         i++ ;
      }


      this.description = description ;
      firePropertyChanged ( DESCRIPTION ) ;

   }





   /*------------------------------------------------------------*/
   /* getDescription
    */
   /**
    *    This method will return the Description
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getDescription () {
      return( description ) ;
   }


   /*------------------------------------------------------------*/
   /* setCreateDate
    */
   /**
    *    This method will set the CreateDate property. This date
    * must be in the form of MM/DD/YYYY
    *  
    *  
    *
    *  @param createDate The property value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setCreateDate ( String createDate ) {
      if ( createDate != null ) {
         DocumentProperty property = getProperty ( DocumentProperty.CREATE_DATE ) ;

         property.setValue ( CDate.createDate ( CDate.parse ( createDate, CDate.DATE_YEAR, CDate.DATE_DATE ),
                                                CDate.parse ( createDate, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                                CDate.parse ( createDate, CDate.DATE_DAY, CDate.DATE_DATE ),
                                                0,0,0,CDate.DATE_DBDATE ) ) ;
         firePropertyChanged ( CREATE_DATE ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* getCreateDate
    */
   /**
    *    This method will return the CreateDate
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getCreateDate () {
      return( getProperty ( DocumentProperty.CREATE_DATE ).getValue() ) ;
   }


   /*------------------------------------------------------------*/
   /* setUniversalNotesId
    */
   /**
    *    This method will set the UniversalNotesId property
    *  
    *  
    *
    *  @param universalNotesId The property value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setUniversalNotesId ( String universalNotesId ) {
      if ( universalNotesId != null ) {
         DocumentProperty property = getProperty ( DocumentProperty.UNIVERSAL_NOTES ) ;

         property.setValue ( universalNotesId ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getUniversalNotesId
    */
   /**
    *    This method will return the UniversalNotesId
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public final String getUniversalNotesId () {
      return( getProperty ( DocumentProperty.UNIVERSAL_NOTES ).getValue() ) ;
   }





   /*------------------------------------------------------------*/
   /* getProperty
    */
   /**
    *    This method will go through the properties
    *  of the document and return the property 
    *  that matches the property passed in. If there is no
    *  match for a property to find, a blank DocumentProperty
    *  record will be returned. The blank record will be of type
    *  propertyInd passed.
    *
    *  @param propertyInd The property indicator as defined in DocumentProperty object
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private DocumentProperty getProperty ( int propertyInd ) {
      Integer          propertyInteger = new Integer ( propertyInd ) ;
      DocumentProperty property        = null ;
      int              i               = 0 ;                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, search for the property                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( properties != null ) {
         int size = properties.size() ;

         while ( ( i < size ) && ( property == null ) ) {
            if ( ((DocumentProperty)properties.elementAt(i)).equals ( propertyInteger ) )
               property = (DocumentProperty)properties.elementAt(i) ;
            i++ ;
         }

      }  /* if */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we didn't find the property, create a new one    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( property == null ) {
         property = new DocumentProperty ( docInd ) ;
         property.setPropertyInd ( propertyInd ) ;
         if ( properties == null )
            properties= new Vector ( 10 ) ;
         properties.addElement ( property ) ;
      }


      return( property ) ;
   } /* getProperty */





   /*------------------------------------------------------------*/
   /* initializeProperties
    */
   /**
    *    This method will initialize our data to the
    *  data from the Document passed in
    *  
    *
    *  @param document The Document to copy the data from
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initializeProperties ( Document document ) {

      this.docInd                 = document.docInd ;
      this.title                  = document.title ;
      this.owner                  = document.owner ;
      this.source                 = document.source ;
      this.locked                 = document.locked ;
      this.filename               = document.filename ;
      this.partNumber             = document.partNumber ;
      this.lastTouchedBy          = document.lastTouchedBy ;
      this.maintLock              = document.maintLock ;
      this.description            = document.description ;
      this.lastTouched            = document.lastTouched ;
      this.queryText              = document.queryText ;
      this.webable                = document.webable ;
      this.webOnly                = document.webOnly ;
      this.faxable                = document.faxable ;
      this.hotNews                = document.hotNews ;
      this.hotNewsDate            = document.hotNewsDate ;
      this.eMailable              = document.eMailable ;
      this.eMailableDate          = document.eMailableDate ;
      this.faq                    = document.faq ;
      this.knowledgeBase          = document.knowledgeBase ;
      this.monthsToReview         = document.monthsToReview ;
      this.eMailComments          = document.eMailComments ;
      this.additionalInfo         = document.additionalInfo ;
      this.docClass               = document.docClass ;
      this.documentType           = document.documentType ;
      this.publicationType        = document.publicationType ;
      this.metric                 = document.metric ;
      this.authority              = document.authority ;

      if ( document.commentHistory != null )
         this.commentHistory = (Vector)document.commentHistory.clone() ;

      if ( document.status != null )
         this.status                 = new DocumentStatus ( docInd,
                                                            document.status.getStatus(),
                                                            document.status.getStatusDate() )  ;

      if ( document.body != null )
         this.body = (DocumentBody)document.body.clone() ;

      if ( document.locale != null )
         this.locale = (LocaleRec)document.getLocale();

      if ( document.graphics != null )
         this.graphics               = (Vector)document.graphics.clone() ;
      if ( document.fileAttachments != null )
         this.fileAttachments        = (Vector)document.fileAttachments.clone() ;
      if ( document.categories != null )
         this.categories             = (Vector)document.categories.clone() ;
      if ( document.properties != null )
         this.properties             = (Vector)document.properties.clone() ;
      if ( document.documentLinks != null )
         this.documentLinks          = (Vector)document.documentLinks.clone() ;
      if ( document.referringDocumentLinks != null )
         this.referringDocumentLinks = (Vector)document.referringDocumentLinks.clone() ;
      if ( document.appliedBrands != null )
         this.appliedBrands = (Vector)document.appliedBrands.clone() ;
      if ( document.appliedFamilies != null )
         this.appliedFamilies = (Vector)document.appliedFamilies.clone() ;
      if ( document.appliedMachines != null )
         this.appliedMachines = (Vector)document.appliedMachines.clone() ;
      if ( document.appliedSubCategories != null )
         this.appliedSubCategories = (Vector)document.appliedSubCategories.clone() ;

      if ( document.brands != null )
         this.brands = (Vector)document.brands.clone() ;
      if ( document.families != null )
         this.families = (Vector)document.families.clone() ;
      if ( document.machines != null )
         this.machines = (Vector)document.machines.clone() ;
      if ( document.models != null )
         this.models = (Vector)document.models.clone() ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we were passed a DocumentDraft to initialize     */
      /* with, take that document draft's Existing Doc Indicator   */
      /* as our own                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( document instanceof DocumentDraft ) {
         this.docInd = ((DocumentDraft)document).getExistingDocInd () ;
      }




      forceRecStatus ( document.getRecStatus() ) ;
   }





   /*------------------------------------------------------------*/
   /* forceRecStatus
    */
   /**
    *    This method is extermely dangerous and 
    *  should not be used except for when reading
    *  in a document. It forces the recstatus to the status passed in
    *
    *  @param int recStatus the new rec status
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void forceRecStatus ( int recStatus ) {
      updateRecStatus ( REC_NOCHANGE ) ;
      updateRecStatus ( recStatus ) ;
   }





   /*------------------------------------------------------------*/
   /* saveToDatabase
    */
   /**
    *    This method will save this document to the
    *  database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return sqlCode , 0 if Ok
    *//*---------------------------------------------------------*/
   public int writeToDatabase () {
      int rc = -1 ;

      try {
         generateKeys() ;
         generateFilename() ;
         rc = DocumentWriter.writeToDatabase ( this ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getLocale
    */
   /**
    *    This method will return the LocaleRec
    *  that assigns countries and geographies to
    *  this document. If a document locale has not yet been
    *  assigned a new one will be created
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return LocaleRec
    *//*---------------------------------------------------------*/
   public final LocaleRec getLocale () {
      if ( locale == null )
         locale = new LocaleRec () ;

      return( (LocaleRec)locale.clone() ) ;
   }




   /*------------------------------------------------------------*/
   /* setLocale
    */
   /**
    *    This method will set the LocaleRec
    *  record which assigns countries and geographies
    *  to a document
    *
    *  @param localeRec The LocaleRec to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setLocale ( LocaleRec newLocale ) {
      if ( this.locale != null )
         addUndo ( LOCALE, this.locale ) ;
      else
         addUndo ( LOCALE, new LocaleRec() ) ;

      this.locale = newLocale ;
      firePropertyChanged ( LOCALE ) ;

   }


   /*------------------------------------------------------------*/
   /* addFileAttachment
    */
   /**
    *     This method will add a DocumentFile
    *  object (File Attachment) to this document.
    *  
    *  
    *
    *  @param file The DocumentFile object to add
    *  @param None
    *  @param None
    *
    *  @return void - Nothing
    *//*---------------------------------------------------------*/
   public void addFileAttachment ( DocumentFile file ) {
      if ( fileAttachments == null )
         fileAttachments = new Vector ( 1 ) ;

      fileAttachments.addElement ( file ) ;
   }

   /**
     *  This method will remove a Document File object from
     * this document. This means that the file will be deleted from
     * the database.
     *
     * @param file The file to remove
     **/
   public void removeFileAttachment ( DocumentFile file ) {
      if ( fileAttachments != null )
         file.updateRecStatus ( DocumentFile.REC_DELETE ) ;
   }



   /**
     * This method will turn the Recycled property either on or
     * off. If the recycle flag on a document is TRUE then the document
     * will not go out to the web sites or any other place
     *
     * @param recycled TRUE=This document is recycled, FALSE=This document is webable.
     */
   public final void setRecycled ( boolean recycled ) {
      this.recycled = recycled ;
      updateRecStatus ( REC_NOCHANGE ) ;
      firePropertyChanged ( RECYCLED ) ;

   }


   /**
     * This method will return the Recycled Property
     *
     * @return boolean TRUE=This document is recycled, FALSE=This document is webable.
     */
   public final boolean getRecycled () {
      return( recycled ) ;
   }


   /**
     *  This method will return all of the brands that this document is associated to. Please
     * note that any brand returned should be considered that this document is associated to
     * all machines under that brand.
     *
     * @return A Vector of TypeCategoryRec objects, each object representing a single brand
     **/
   public final Vector getBrands () {
      return( brands ) ;
   }

   /**
     *  This method will return all of the families that this document is associated to. Please
     * note that any family returned should be considered that this document is associated to
     * all machines/models under that family
     *
     * @return A Vector of TypeCategoryRec objects, each object representing a single family
     **/
   public final Vector getFamilies () {
      return( families ) ;
   }


   /**
     *  This method will return all of the machines that this document is associated to. Please
     * note that any machine returned should be considered that this document is associated to
     * all models under that machine.
     *
     * @return A Vector of TypeCategoryRec objects, each object representing a single machine
     **/
   public final Vector getMachines () {
      return( machines ) ;
   }

   /**
     *  This method will return all of the models that this document is associated to. Please note
     * that this method returns <b>only directly associated</b> models. Therefore if a document is
     * associated to the brand IBM Aptiva only, this method returns nothing since the document is
     * not associated directly to a model.
     *
     * @return A Vector of TypeCategoryRec objects, each object representing a single model.
     **/
   public final Vector getModels () {
      return( models ) ;
   }


   /**
     *  This method will set the brands that this document is associated to. These brands indicate
     * that this document is associated to all Families, Machines and Models under that brand.
     *
     * @param brands The vector of TypeCategoryRec objects, each object representing the brand.
     **/
   public final void setBrands ( Vector brands ) {
      addBFTMUndo () ;
      this.brands = brands ;
      brandUpdated = true ;
      firePropertyChanged ( BRAND ) ;

   }

   /**
     *  This method will set the families that this document is associated to. These families indicate
     * that this document is associated to all Machines and Models under that family.
     *
     * @param families The vector of TypeCategoryRec objects, each object representing the family.
     **/
   public final void setFamilies ( Vector families ) {
      addBFTMUndo () ;
      this.families = families ;
      familyUpdated = true ;
      firePropertyChanged ( FAMILY ) ;

   }

   /**
     *  This method will set the machines that this document is associated to. These machines indicate
     * that this document is associated to all models under that machine.
     *
     * @param machines The vector of TypeCategoryRec objects, each object representing the machine.
     **/
   public final void setMachines ( Vector machines ) {
      addBFTMUndo () ;
      this.machines = machines ;
      machineUpdated = true ;
      firePropertyChanged ( MACHINE ) ;

   }


   /**
     *  This method will set the models that this document is associated to. These models indicate
     * that this document is associated directly to the model
     *
     * @param models The vector of TypeCategoryRec objects, each object representing the model
     **/
   public final void setModels ( Vector models ) {
      addBFTMUndo () ;
      this.models = models ;
      modelUpdated = true ;
      firePropertyChanged ( MODEL ) ;

   }

   /**
     * This method will return a list of brands that this document is associated with by means
     * of the associated families, machines and models. Please note that the list of brands returned
     * from this method does NOT mean that this document is associated to all families under the brand. 
     *
     * @returns Vector of TypeCategoryRec objects. Each object represents a brand
     **/
   public final Vector getApplicableBrands() {
      return( appliedBrands ) ;
   }


   /**
     * This method will return a list of families that this document is associated with by means
     * of the associated machines and models. Please note that the list of families returned
     * from this method does NOT mean that this document is associated to all machines under the family. 
     *
     * @returns Vector of TypeCategoryRec objects. Each object represents a family
     **/
   public final Vector getApplicableFamilies () {
      return( appliedFamilies ) ;
   }

   /**
     * This method will return a list of machines that this document is associated with by means
     * of the associated models. Please note that the list of machines returned
     * from this method does NOT mean that this document is associated to all models under the machines. 
     *
     * @returns Vector of TypeCategoryRec objects. Each object represents a machine
     **/
   public final Vector getApplicableMachines () {
      return( appliedMachines ) ;
   }

   /**
     * This method will return a list of Sub Categories that this document is associated with by means
     * of the associated paret numbers. Please note that the list of sub categories returned
     * from this method does NOT mean that this document is associated to all part numbers under the sub categories. 
     *
     * @returns Vector of TypeCategoryRec objects. Each object represents a machine
     **/
   public final Vector getApplicableSubCategories () {
      return( appliedSubCategories ) ;
   }


   /**
     * This method will set the list of applicable brands
     *
     * @param appliedBrands A Vector containing TypeCategoryRec objects representing the Applicable brands
     **/
   public final void setApplicableBrands ( Vector appliedBrands ) {
      this.appliedBrands = appliedBrands ;
      firePropertyChanged ( APPLICABLE_BRAND ) ;

   }

   /**
     * This method will set the list of applicable machines
     *
     * @param appliedMachines A Vector containing TypeCategoryRec objects representing the Applicable machines
     **/
   public final void setApplicableMachines ( Vector appliedMachines ) {
      this.appliedMachines = appliedMachines ;
      firePropertyChanged ( APPLICABLE_MACHINE ) ;
   }

   /**
     * This method will set the list of applicable families
     *
     * @param appliedFamilies A Vector containing TypeCategoryRec objects representing the Applicable families
     **/
   public final void setApplicableFamilies ( Vector appliedFamilies ) {
      this.appliedFamilies = appliedFamilies ;
      firePropertyChanged ( APPLICABLE_FAMILY ) ;
   }

   /**
     * This method will set the list of applicable sub categories
     *
     * @param appliedSubCategories A Vector containing TypeCategoryRec objects representing the Applicable Sub Categories
     **/
   public final void setApplicableSubCategories ( Vector appliedSubCategories ) {
      this.appliedSubCategories = appliedSubCategories ;
      firePropertyChanged ( APPLICABLE_SUBCATEGORY ) ;
   }


   /**
     * This method will add a DocumentPropertyListener to this document. This listener
     * will be notified whenever a property in this document changes
     *
     * @param listener The DocumentPropertyListener to add
     **/
   public void addDocumentPropertyListener ( DocumentPropertyListener listener ) {
      if ( listeners == null )
         listeners = new Vector ( 1 ) ;

      listeners.addElement ( listener ) ;
   }


   /**
     * This method will remove a DocumentPropertyListener from this document
     *
     * @param listener The DocumentPropertyListener to remove
     **/
   public void removeDocumentPropertyListener ( DocumentPropertyListener listener ) {
      if ( listeners != null ) {
         listeners.removeElement ( listener ) ;
         if ( listeners.size() == 0 )
            listeners = null ;
      }
   }


   /**
     * This method will fire an event to all registered listeners
     *
     * @param property The Property to fire
     **/
   protected void firePropertyChanged ( int property ) {
      if ( ( listeners != null ) && ( fireEventEnabled ) )
         new FireThread ( this, property, listeners ) ;
   }


   /**
     *  This method will add a single Product to the document. This method
     * will also adjust the Applicable Brands, Families, Machines and Sub Categories
     * as well.
     *
     * @param product The TypeCategoryRec representing either the Machine or Part Number
     **/
   public void addProduct ( TypeCategoryRec categoryRec ) {
      addProduct ( categoryRec, true ) ;
   }

   /**
     *  This method will add a single Product to the document. This method
     * will also adjust the Applicable Brands, Families, Machines and Sub Categories
     * as well.
     *
     * @param product The TypeCategoryRec representing either the model or Part Number
     **/
   public void addProduct ( TypeCategoryRec categoryRec, boolean notify ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only add if the Category is a Model   or Part Num.      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categoryRec  != null ) &&
           ( ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_MODEL ) ) ||
             ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_PARTNUMBER ) ) ) ) {

         if ( models == null )
            models = new Vector ( 1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only add this if the product is not already here        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            if ( !models.contains ( categoryRec ) ) {
               TypeCategoryRec machine = null ;
               TypeCategoryRec family  = null ;
               TypeCategoryRec brand   = null ;


               if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_PARTNUMBER ) )
                  machine = (TypeCategoryRec)TypeCategory.getParentCategory ( categoryRec, TypeCategory.LEVEL_SUBCATEGORY ) ;
               else
                  machine = (TypeCategoryRec)TypeCategory.getParentCategory ( categoryRec, TypeCategory.LEVEL_MACHINE ) ;
               family  = (TypeCategoryRec)TypeCategory.getParentCategory ( categoryRec, TypeCategory.LEVEL_FAMILY ) ;
               brand  = (TypeCategoryRec)TypeCategory.getParentCategory ( categoryRec, TypeCategory.LEVEL_BRAND ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the product                                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               models.addElement ( categoryRec ) ;                       

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, adjust the Applicable items                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( appliedBrands == null )
                  appliedBrands = new Vector ( 1 ) ;
               if ( !appliedBrands.contains ( brand ) )
                  appliedBrands.addElement ( brand ) ;

               if ( appliedFamilies == null )
                  appliedFamilies = new Vector ( 1 ) ;
               if ( !appliedFamilies.contains ( family ) )
                  appliedFamilies.addElement ( family ) ;

               if ( machine.getCategoryLevel().equals ( TypeCategory.LEVEL_MACHINE ) ) {
                  if ( appliedMachines == null )
                     appliedMachines = new Vector ( 1 ) ;
                  if ( !appliedMachines.contains ( machine ) )
                     appliedMachines.addElement ( machine ) ;
               } else {
                  if ( appliedSubCategories == null )
                     appliedSubCategories = new Vector ( 1 ) ;
                  if ( !appliedSubCategories.contains ( machine ) )
                     appliedSubCategories.addElement ( machine ) ;
               }

               if ( notify ) {
                  firePropertyChanged ( MODEL ) ;
                  firePropertyChanged ( APPLICABLE_BRAND ) ;
                  firePropertyChanged ( APPLICABLE_FAMILY ) ;
                  firePropertyChanged ( APPLICABLE_MACHINE ) ;
                  firePropertyChanged ( APPLICABLE_SUBCATEGORY ) ;
               }
            }
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
         }
      }
   } /* addProduct  */


   /**
     *  This method will add all products in the Vector passed in and then
     * fire one PropertyChanged event for all changes. Each item in the 
     * vector passed MUST be a TypeCategory object representing the Model
     * or part number
     * 
     * @param products The Products to add
     **/
   public void addProducts ( Vector products ) {
      if ( ( products != null ) && ( products.size() > 0 ) ) {
         int i    = 0 ;
         int size = products.size() ;

         while ( i < size ) {
            addProduct ( (TypeCategoryRec) products.elementAt(i), false ) ;
            i++ ;
         }

         firePropertyChanged ( MODEL ) ;
         firePropertyChanged ( APPLICABLE_BRAND ) ;
         firePropertyChanged ( APPLICABLE_FAMILY ) ;
         firePropertyChanged ( APPLICABLE_MACHINE ) ;
         firePropertyChanged ( APPLICABLE_SUBCATEGORY ) ;
      }
   }


   /**
     *  This method will add all products in the Vector passed in and then
     * fire one PropertyChanged event for all changes. Each item in the 
     * vector passed MUST be a ProductRow object representing the Product. Please
     * note that ProductRow's Model property must already be set
     * 
     * @param products The Products to add
     **/
   public void addProductRows ( Vector products ) {
      if ( ( products != null ) && ( products.size() > 0 ) ) {
         int        i    = 0 ;
         int        size = products.size() ;
         ProductRow row  = null ;

         while ( i < size ) {
            row = (ProductRow)products.elementAt(i) ;
            addProduct ( row.getModel(), false ) ;
            i++ ;
         }

         firePropertyChanged ( MODEL ) ;
         firePropertyChanged ( APPLICABLE_BRAND ) ;
         firePropertyChanged ( APPLICABLE_FAMILY ) ;
         firePropertyChanged ( APPLICABLE_MACHINE ) ;
         firePropertyChanged ( APPLICABLE_SUBCATEGORY ) ;
      }
   }



   /**
     *  This method will return an exact copy of this document instance
     **/
   public Object clone () {
      Document document = new Document ( getDocInd() ) ;

      document.initializeProperties ( this ) ;

      return( document ) ;
   }


   /**
     *  This method wil enable/disable the event firing when a
     * property is changed. 
     *
     * @param enable TRUE = Send out property change events when something is changed
     **/
   public void setPropertyChangeEnabled ( boolean enable ) {
      this.fireEventEnabled = enable ;
   }


   /**
     *  This method turns the undoable feature of the Document object on or off. By default
     * this feature is disabled. If enabled, every 'set' method will generate a DocumentUndoableEdit
     * Object. A Vector of these objects can be obtained using the getUndoableEdits() method. Please remember
     * that even the document body is undoable, so if you edit the body 15 times, that document body will
     * be in memory 15 times. Also see the DocumentUndoableListener to be notified when items are added/removed
     * from the document undo. Changing the status of this will wipe out all current undos.
     *
     * @param enable TRUE/FALSE
     **/
   public void enableUndo ( boolean enable ) {
      undoEnabled = enable ;
   }

   /**
     *  This method will not do anything except for wiping out all of the undo items that have been collected
     * so far.
     **/
   public void writeToMemory () {
      undoableEdits=null ;
   }

   /**
     *  This method will return a Vector of all DocumentUndoableEdit objects that have been collected since
     * the last 'writeToMemory' method was called.
     **/
   public Vector getUndoableEdits () {
      return( undoableEdits ) ;
   }


   /**
     *  This method will add a new DocumentUndoableEdit object to our list. If the
     * document's enableUndo property is FALSE, then this method will do nothing
     *
     * @param propertyInd The indicator for the property that was changed
     * @param data The data object that resides with the change
     **/
   protected void addUndo ( int propertyInd, Object data ) {

      if ( ( undoEnabled ) && ( data != null ) ) {

         if ( undoableEdits == null )
            undoableEdits = new Vector ( 1 ) ;
         undoableEdits.addElement ( new DocumentUndoableEdit ( propertyInd,
                                                               data ) ) ;

      }
   }


   /**
     *  This method will create a single UNDO action for the editing
     * of Brands, Families, Machines, Models, Part Numbers and Sub Categories.
     **/
   private void addBFTMUndo () {
      Hashtable undoHash = new Hashtable () ;

      if ( brands != null )
         undoHash.put ( "BRANDS", brands ) ;
      if ( families != null )
         undoHash.put ( "FAMILIES", families ) ;
      if ( machines != null )
         undoHash.put ( "MACHINES", machines ) ;
      if ( models != null )
         undoHash.put ( "MODELS", models ) ;
   }


   /**
     *  This method will attempt to return the first available brand that
     * this document is associated to.
     *
     * @return TypeCategoryRec
     **/
   public TypeCategoryRec getFirstAvailBrand () {
      TypeCategoryRec returnBrand = null ;

      if ( ( brands != null ) && ( brands.size() > 0 ) )
         returnBrand = (TypeCategoryRec)brands.elementAt(0) ;
      else if ( ( appliedBrands != null ) && ( appliedBrands.size() > 0 ) )
         returnBrand = (TypeCategoryRec)appliedBrands.elementAt(0) ;

      return( returnBrand ) ;
   }

   /**
     *  This method will set the document indicator for this document
     **/
   protected void setDocInd ( int docInd ) {
      this.docInd = docInd ;
   }


   /**
     *  This method will nullify the filename for this document
     **/
   protected void clearFilename () {
      this.filename = null ;
   }


   /**
     *  This method will undo the last undoable action
     **/
   public DocumentUndoableEdit undo () {
      DocumentUndoableEdit undo          = null ;

      System.out.println ( "Undo!" ) ;
      if ( ( undoableEdits != null ) && ( undoableEdits.size() > 0 ) ) {
         boolean              currentUndoEnabled = undoEnabled ;
         int                  lastIndex          = undoableEdits.size()-1 ;

         System.out.println ( "Undoable Edits Size:"+undoableEdits.size() ) ;

         undo  = (DocumentUndoableEdit)undoableEdits.elementAt ( lastIndex ) ;
         undoEnabled = false ;
         undo.undo ( this ) ;
         undoEnabled=currentUndoEnabled ;
         undoableEdits.removeElementAt(lastIndex ) ;
      }

      return( undo ) ;
   }


   /**
     *  This method will check out the document. This means that no QIT user
     * should be able to change this document when it is checked out
     *
     * @param userId The User Id to check out
     **/
   public void checkOut ( String userId ) {
      if ( docInd != 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "Document.checkOut ( "+userId+" )", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            stmt.executeUpdate ( "UPDATE TIGRIS.DOCUMENTS SET MAINTLOCK = 'Y' WHERE DOCIND = "+docInd ) ;
            stmt.executeUpdate ( "DELETE FROM TIGRIS.DOCPROPERTIES WHERE DOCIND = "+docInd+" AND "+
                                 "PROPERTYIND = "+DocumentProperty.CHECKED_OUT_BY ) ;
            stmt.executeUpdate ( "INSERT INTO TIGRIS.DOCPROPERTIES "+
                                 "(DOCIND, PROPERTYIND, ORDERNUM, VALUE, DBUSER, CHANGEDTIME) "+
                                 "values "+
                                 "("+docInd+", "+DocumentProperty.CHECKED_OUT_BY+", 0, '"+userId+"', 'QUEST', CURRENT TIMESTAMP)" ) ;
            stmt.close() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            sqlMethod.rollBack() ;
         }
         sqlMethod.close() ;
      }
   }

   /**
     *  This method will check in the document. This means that QIT users
     * should be able to change this document when it is not checked out
     *
     **/
   public void checkIn (  ) {
      if ( docInd != 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "Document.checkIn ()", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            stmt.executeUpdate ( "UPDATE TIGRIS.DOCUMENTS SET MAINTLOCK = 'N' WHERE DOCIND = "+docInd ) ;
            stmt.executeUpdate ( "DELETE FROM TIGRIS.DOCPROPERTIES WHERE DOCIND = "+docInd+" AND "+
                                 "PROPERTYIND = "+DocumentProperty.CHECKED_OUT_BY ) ;
            stmt.close() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            sqlMethod.rollBack() ;
         }
         sqlMethod.close() ;
      }
   }

   /**
     *  This method will return TRUE if this document is currently checked out by someone
     **/
   public boolean isCheckedOut () {
      return( maintLock ) ;
   }

   /**
     *  This method will return the user id of the person who has this document checked out
     **/
   public String getCheckedOutBy () {
      DocumentProperty property = getProperty ( DocumentProperty.CHECKED_OUT_BY ) ;

      if ( property != null )
         return( property.getValue() ) ;

      return( null ) ;
   }


   /**
     *  This method will set the maint lock property
     **/
   protected void setMaintLock ( String maintLock ) {
      this.maintLock = maintLock.equals ( "Y" ) ;
   }


   /*------------------------------------------------------------*/
   /* setCheckedOutBy
    */
   /**
    *    This method will set the user who has this document checked out
    *  
    *  
    *
    *  @param checkedOutBy the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setCheckedOutBy ( String user ) {
      if ( user != null ) {
         DocumentProperty property = getProperty ( DocumentProperty.CHECKED_OUT_BY ) ;

         property.setValue ( user ) ;
      }
   }


   /**
     *  This method will simply return all properties that this document has assigned
     **/
   public Vector getProperties ( ) {
      return( properties ) ;
   }


   /**
     *  this method will remove all document properties of the type specified
     **/
   private void removeProperty ( int propertyInd ) {
      if ( ( properties != null ) && ( properties.size() > 0 ) ) {
         int i    = 0;
         int size = properties.size() ;

         while ( i < size ) {
            if ( ((DocumentProperty)properties.elementAt(i)).getPropertyInd() == propertyInd ) {
               properties.removeElementAt(i) ;
               size-- ;
            } else
               i++ ;
         }
      }
   }

   /**
     *  This method will simply add a DocumentProperty object to this document
     **/
   public void addProperty ( DocumentProperty property ) {
      if ( properties == null )
         properties = new Vector ( 1 ) ;

      properties.addElement ( property ) ;
   }




   /*------------------------------------------------------------*/
   /* getCommentHistory
    */
   /**
    *    This method will return the comment history. This
    *  is simply a Vector of DocumentComment objects.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getCommentHistory () {
      return( commentHistory ) ;
   }





   /*------------------------------------------------------------*/
   /* addCommentToHistory
    */
   /**
    *    This method will add a comment to the
    *  history. There is no logic here to place the
    *  comment in it's ordered place, comments are just added FIFO
    *
    *  @param comment The DocumentComment object to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void addCommentToHistory ( DocumentComment comment ) {
      if ( commentHistory == null ) {
         commentHistory = new Vector ( 1 ) ;
      }
      commentHistory.addElement ( comment ) ;
   }



   /**
     *  This method will return the comment history for this document as one
     * long string
     **/
   public String getCommentHistoryAsText () {
      Vector history = getCommentHistory() ;
      String retStr  = "" ;

      if ( ( history != null ) && ( history.size() > 0 ) ) {
         int i    = 0 ;
         int size = history.size() ;

         while ( i < size ) {
            retStr+=((DocumentComment)history.elementAt(i)).getComment()+"\n\n" ;
            i++ ;
         }
      }

      return( retStr ) ;
   }

} /* Document */
