package com.ibm.nzna.projects.common.quest.doc ;              
/*-----------------------------------------------------------*/
/*   DocumentBody                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentBody                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
/**
 * This object holds the Document Body of a document 
 *
 *
 * @version 1.0, 2/12/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentBody                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentBody extends com.ibm.nzna.shared.pom.PersistentRec implements java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;      

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector body = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor initializes our
    *  variables
    *  
    *
    *  @param body The initial Query Text
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentBody ( String body ) throws IllegalArgumentException {
      updateRecStatus ( REC_INSERT ) ;
      setBody ( body ) ;
   }





   /*------------------------------------------------------------*/
   /* setBody
    */
   /**
    *    This method will set the query text of the
    *  object
    *  
    *
    *  @param body The Query Text to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBody ( String body ) throws IllegalArgumentException {
      if ( body != null ) {
         int    length     = body.length() ;
         int    maxSize    = 32700 ;

         if ( this.body != null )
            this.body.removeAllElements() ;
         else
            this.body = new Vector ( 10 ) ;                              


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Break the document down into 32k chunks                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( length > maxSize ) {
            int    chunks     = body.length() / maxSize ;
            String bodyPart   = null ;
            int    startIndex = 0 ;

            /* add 1 to the chunk size because if the length of the
               body is less than DB_MAXLONGVARCHARSIZE, then chunks is 0! */
            chunks++ ;

            /* Now we insert each chunk as a seperate row in the database    */
            for ( int recNum = 0 ; recNum < chunks ; recNum++ ) {
               String strStmt = "" ;


               if ( startIndex > ( length-maxSize ) )
                  bodyPart = body.substring ( startIndex, body.length() ) ;
               else
                  bodyPart = body.substring ( startIndex, startIndex+maxSize ) ;

               startIndex+=maxSize ;

               this.body.addElement ( bodyPart ) ;

            } // End of for loop for inserts of chunks

         }

         else
            this.body.addElement ( body ) ;

         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "BODY cannot be NULL" ) ;
   }




   /*------------------------------------------------------------*/
   /* getBody
    */
   /**
    *    This method will return the Body
    *  for the document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getBody () {
      String retStr = "" ;

      if ( body != null ) {
         for ( int i = 0; i < body.size(); i++ )
            retStr+=body.elementAt(i) ;
      }
      return( retStr ) ;
   }





   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return the Query Text
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( getBody() ) ;
   }





   /*------------------------------------------------------------*/
   /* getBodyChunks
    */
   /**
    *    This method will return an array of strings,
    *  each String contains 32k of the body. The order
    *  of the Strings in the array is in correct.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getBodyChunks () {
      return( body ) ;
   }





   /*------------------------------------------------------------*/
   /* addBodyChunk
    */
   /**
    *    This method will add a 32k chunk to
    *  the bottom of the document body
    *  
    *
    *  @param bodyChunk The 32k Chunk to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addBodyChunk ( String bodyChunk ) {
      if ( body == null )
         body = new Vector ( 1 ) ;

      body.addElement ( bodyChunk ) ;
   }


   /**
     *  This method will clone this document body
     **/
   public Object clone () {
      return ( new DocumentBody ( getBody() ) ) ;
   }
} /* DocumentBody */
