package com.ibm.nzna.projects.common.quest.doc ;       
/*-----------------------------------------------------------*/
/*   DocumentDraftReader                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentDraftReader                 */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Blob ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
/**
 * This class holds static methods used when reading a document
 * from the QUEST.DOCUMENT table into a DocumentDraft object 
 *
 *
 * @version 1.0, 2/22/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentDraftReader                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentDraftReader extends StoredProc {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec       = new StoredProcRec () ;




   /*------------------------------------------------------------*/
   /* readDocumentDraft
    */
   /**
    *    This method will be called by DB/2 when
    *  a request for this stored procedure is given.
    *  
    *
    *  @param docInd The DocumentDraft Indicator to read in
    *  @param documentBlob The Blob to store the document in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readDocument ( int docInd, Blob documentBlob, Blob storedProcReturn ) throws Exception {

      try {
         Connection    conn     = getConnection() ;
         DocumentDraft document = new DocumentDraft ( docInd ) ;
         int           rc       = 0 ;

         rc = readDocumentDraft ( conn, document ) ;

         documentBlob = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( documentBlob, document ) ;
         set ( 2, documentBlob ) ;

         returnRec.sqlCode  = rc ;
         returnRec.error    = ( rc != 0 ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Commit                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         conn.commit() ;
         conn.close() ;

      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcReturn = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcReturn, returnRec ) ;

      set ( 3, storedProcReturn ) ;

   } /* readDocumentDraft  */



   /*------------------------------------------------------------*/
   /* readDocumentDraft
    */
   /**
    *    This method will read the document record
    *  passed in. The document record passed in
    *  will be populated from whatever is in the QUEST.DOCUMENTS table
    *
    *  @param Connection The DB/2 Connection to use to read with
    *  @param document The DocumentDraft object to populate
    *  @param None
    *
    *  @return TRUE if the document was read Ok, FALSE if not
    *//*---------------------------------------------------------*/
   protected  int readDocumentDraft ( Connection conn, DocumentDraft document ) throws Exception {
      int rc = 0 ;                                                                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in the QUEST.DOCUMENTS table info           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rc = readDocumentDraftBase ( conn, document ) ;                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read the document's body parts only if this is not an   */
      /* SSD                                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentBody ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Body" ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read the document's Graphics                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentGraphics ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Body" ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read the DocumentDraft's Categories                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentDraftCategories ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Categories" ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read the DocumentDraft's Properties                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentDraftProperties ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Properties" ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read the DocumentDraft's Query Text                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentQueryText ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Query Text" ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the document's links                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentLinks ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Links" ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the referring links                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentDraftReferringLinks ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Referring Links" ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the document location information               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentLocale ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Locale" ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the Comment History                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 )
         readCommentHistory ( conn, document ) ;

      /*------------------------------*/
      /*                              */
      /*  Read in the document status */
      /*                              */
      /*------------------------------*/
      if ( rc == 0 ) {
         rc = readDocumentStatus ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document status" ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the Brand/Family/Machine/Model/Part Numbers     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc == 0 )
         rc = readDocumentBrands ( conn, document ) ;
      if ( rc == 0 )
         rc = readDocumentFamilies ( conn, document ) ;
      if ( rc == 0 )
         rc = readDocumentMachines ( conn, document ) ;
      if ( rc == 0 )
         rc = readDocumentModels ( conn, document ) ;
      if ( rc == 0 )
         rc = readDocumentPartNumbers ( conn, document ) ;
      if ( rc == 0 )
         rc = readDocumentApplicable ( conn, document ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this is an SSD or Service Pub, read the document's   */
      /* file attachments                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( rc == 0 ) && 
           ( ( document.getDocumentType() == TypeList.DOCTYPE_SSD ) ||
             ( document.getDocumentType() == TypeList.DOCTYPE_SERVICE_PUB ) ) ) {
         rc = readDocumentFiles ( conn, document ) ;
         if ( rc != 0 )
            returnRec.errorStr = "Error reading Document Locale" ;
      }



      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* readDocumentDraftBase
    */
   /**
    *    This method will read all information from
    *  the QUEST.DOCUMENTS table and
    *  populates the document object passed
    *
    *  @param document The DocumentDraft object to populate
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private  int readDocumentDraftBase ( Connection conn, DocumentDraft document ) throws Exception {
      int       rc   = 0 ;
      Statement stmt = conn.createStatement() ;
      ResultSet rs   = null ;
      String    sql  = null ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Form our SQL                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT DOCTYPEIND,     DOCCLASSIND, TITLE,       OWNER,         REVIEWPERIOD, "+
            "       AUTHIND,        WEBABLE,     WEBONLY,     FAXABLE,       FLASHNEWS,    "+
            "       FLASHDATE,      FAQ,         EMAILABLE,   EMAILABLEDATE, SUMMARY,      "+
            "       METRICIND,      ADDINFO,     PARTNUM,     PUBTYPE,       LNDOCID,      "+
            "       SOURCE,         MAINTLOCK,   DBUSER,      CHANGEDTIME,   KNOWLEDGEBASE,"+
            "       EXISTINGDOCIND, REQUESTOR,   WORKIND,     COMMENT,       STARTDATE,    "+
            "       STOPDATE,       RECYCLED                                               "+
            "FROM   QUEST.DOCUMENTS                                                        "+
            "WHERE  DOCIND   = "+document.getDocInd()+"                                    "+
            "       FOR FETCH ONLY                                                         " ;    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Run the SQL                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Populate our data                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rs.next() ) {
         document.setDocumentType      ( rs.getInt(1) ) ;
         document.setDocumentClass     ( rs.getInt(2) ) ;
         document.setTitle             ( rs.getString(3).trim() ) ;
         document.setOwner             ( rs.getString(4).trim() ) ;
         document.setMonthsToReview    ( rs.getInt(5) ) ;
         document.setAuthority         ( rs.getInt(6) ) ;
         document.setWebable           ( rs.getString(7).equals ( "Y" ) ) ;
         document.setWebOnly           ( rs.getString(8).equals ( "Y" ) ) ;
         document.setFaxable           ( rs.getString(9).equals ( "Y" ) ) ;
         document.setHotNewsDate       ( rs.getString(11) ) ;
         document.setFaq               ( rs.getString(12).equals ( "Y" ) ) ;
         document.setEMailable         ( rs.getString(13).equals ( "Y" ) ) ;
         document.setEMailableDate     ( rs.getString(14) ) ;
         document.setEMailComments     ( rs.getString(15) ) ;
         document.setMetric            ( rs.getInt(16) ) ;
         document.setAdditionalInfo    ( rs.getString(17) ) ;
         document.setPartNumber        ( rs.getString(18) ) ;
         document.setPublicationType   ( rs.getInt(19) ) ;
         document.setFilename          ( rs.getString(20) ) ;
         document.setSource            ( rs.getString(21) ) ;
         document.setMaintLock         ( rs.getString(22) ) ;
         document.setLastTouchedBy     ( rs.getString(23).trim() ) ;
         document.setLastTouched       ( rs.getString(24).trim() ) ;
         document.setKnowledgeBase     ( rs.getString(25).equals ( "Y" ) ) ;
         document.setExistingDocInd    ( rs.getInt(26) ) ;
         document.setRequestor         ( rs.getString(27) ) ;
         document.setWorkRequired      ( rs.getInt(28) ) ;
         document.setOriginalComments  ( rs.getString(29) ) ;
         document.setStartDate         ( rs.getString(30) ) ;
         document.setStopDate          ( rs.getString(31) ) ;
      } else {
         rc = 100 ;
         returnRec.errorStr = "Document "+document.getDocInd()+" Not Found in QUEST.DOCUMENTS table" ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close our ResultSet                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs.close() ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close our statement                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      stmt.close() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* readDocumentGraphics
    */
   /**
    *    This method will read in all files attachted
    *  to the document passed in
    *  
    *
    *  @param conn The DB/2 connection to use
    *  @param document The DocumentDraft object to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private  int readDocumentGraphics ( Connection conn, DocumentDraft document ) throws  SQLException {
      int       rc   = 0 ;
      Statement stmt = conn.createStatement() ;
      ResultSet rs   = null ;
      String    sql  = null ;     
      Vector    vec  = new Vector ( 1, 1 ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up the SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT FILENAME "+
            "FROM QUESTNEW.GRAPHICS "+
            "WHERE DOCIND = "+document.getDocInd()+" "+
            "FOR FETCH ONLY" ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Run the SQL                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;


      while ( rs.next() ) {
         DocumentGraphic graphic = new DocumentGraphic ( document.getDocInd(), rs.getString(1).trim() ) ;

         graphic.resetRecStatus() ;
         vec.addElement ( graphic ) ;
      }

      document.setGraphics ( vec ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close our ResultSet and Statement                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs.close() ;
      stmt.close() ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* readDocumentFiles
    */
   /**
    *    This method will read in all files pertaining
    *  to the document passed in
    *  
    *
    *  @param conn The DB/2 connection to use
    *  @param document The DocumentDraft object to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static int readDocumentFiles ( Connection conn, Document document ) throws  SQLException {
      int       rc   = 0 ;
      Statement stmt = conn.createStatement() ;
      ResultSet rs   = null ;
      String    sql  = null ;     
      Vector    vec  = new Vector ( 1, 1 ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up the SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT FILENAME, DESCRIPT, SIZE, URL, VERSION, FTPMODE, ORDERNUM "+
            "FROM QUEST.DOCFILES "+
            "WHERE DOCIND = "+document.getDocInd()+" "+
            "ORDER BY ORDERNUM FOR FETCH ONLY" ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Run the SQL                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;


      while ( rs.next() ) {
         DocumentFile file = new DocumentFile ( document.getDocInd(), rs.getString(1).trim() ) ;

         file.setDescription ( rs.getString(2).trim() ) ;
         file.setSize        ( rs.getInt(3) ) ;
         file.setLocation    ( rs.getString(4) ) ;
         file.setURL         ( rs.getString(4) ) ;
         file.setVersion     ( rs.getString(5).trim() ) ;
         file.setFTPMode     ( rs.getInt(6) ) ;
         file.setOrderNum    ( rs.getInt(7) ) ;
         file.resetRecStatus() ;
         vec.addElement ( file ) ;
      }

      document.setFileAttachments ( vec ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close our ResultSet and Statement                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs.close() ;
      stmt.close() ;

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* readDocumentBody
    */
   /**
    *    This method will read this document's
    *  body
    *  
    *
    *  @param conn The connection to use when reading
    *  @param document The DocumentDraft object to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private  int readDocumentBody ( Connection conn, DocumentDraft document ) throws Exception {
      int       rc   = 0;
      Statement stmt = conn.createStatement () ;
      ResultSet rs   = null ;
      String    sql  = null ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Format our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT BODY "+
            "FROM QUEST.DOCBODY "+
            "WHERE DOCIND = "+document.getDocInd()+" "+
            "ORDER BY RECNUM" ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Exceute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read all of the bodies                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rs.next() ) {
         DocumentBody body = new DocumentBody ( rs.getString(1).trim() ) ;

         while ( rs.next() ) {
            body.addBodyChunk ( rs.getString(1).trim() ) ;
         }

         document.setDocumentBody ( body ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close our ResultSet and Statement                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs.close() ;
      stmt.close() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* readDocumentDraftCategories
    */
   /**
    *    This method will read this document's
    *  categories
    *  
    *
    *  @param conn The connection to use when reading
    *  @param document The DocumentDraft object to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private  int readDocumentDraftCategories ( Connection conn, DocumentDraft document ) throws Exception {
      Statement stmt            = conn.createStatement() ;
      int       rc              = 0 ;
      ResultSet rs              = null ;                          
      boolean   continueProcess = true ;
      Vector    categoryVec       = new Vector ( 1, 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT A.TYPECATIND,      B.DESCRIPTION, B.PARENTCATIND, "+
                               "       B.ORDERNUM,        B.CHILDSORT,   B.OWNER "+
                               "FROM QUEST.CATEGORY A, "+
                               "     PRODUCT.TYPECATEGORY B "+
                               "WHERE A.DOCIND = "+document.getDocInd()+" AND "+
                               "      A.TYPECATIND = B.TYPECATIND AND "+
                               "      B.CATEGORYLEVEL = '"+TypeCategory.LEVEL_DOC_CATEGORY+"' "+
                               "FOR FETCH ONLY" ) ;

      while ( rs.next() ) {
         TypeCategoryRec categoryRec = new TypeCategoryRec ( rs.getInt(1) ) ;

         categoryRec.setDescript      ( rs.getString(2).trim() ) ;
         categoryRec.setCategoryLevel ( TypeCategory.LEVEL_MACHINE ) ;
         categoryRec.setParentCatInd  ( rs.getInt(3) ) ;
         categoryRec.setOrderNum      ( rs.getInt(4) ) ;
         categoryRec.setChildSort     ( rs.getString(5).trim() ) ;
         categoryRec.setOwner         ( rs.getString(6).trim() ) ;
         categoryVec.addElement ( categoryRec ) ;
      }


      rs.close() ;
      stmt.close() ;

      document.setCategory ( categoryVec ) ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* readDocProperties
    */
   /**
    *    This method will read in all of the Doc Properties
    *  
    *  
    *
    *  @param conn The connection to use when reading
    *  @param document The DocumentDraft object to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private  int readDocumentDraftProperties ( Connection conn, DocumentDraft document ) throws Exception {
      Statement     stmt           = conn.createStatement () ;
      String        sql            = null ;
      ResultSet     rs             = null ;                    
      String        descript       = "" ;
      String        countryComment = "" ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT PROPERTYIND, ORDERNUM, VALUE "+
            "FROM QUEST.DOCPROPERTIES "+
            "WHERE DOCIND = "+document.getDocInd()+" "+
            "ORDER BY ORDERNUM "+
            "FOR FETCH ONLY" ;            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the statement                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;

      while ( rs.next() ) {
         if ( rs.getString(3) != null ) {
            switch ( rs.getInt(1) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Lotus Notes Universal Id                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               case DocumentProperty.UNIVERSAL_NOTES :
                  document.setUniversalNotesId ( rs.getString(3).trim() ); 
                  break ;

               case DocumentProperty.DESCRIPTION :
                  descript+= ( rs.getString(3) ) ;
                  break ;

               case DocumentProperty.RELEASE_DATE :
                  document.setReleaseDate ( CDate.dateFromDBDate ( rs.getString(3).trim() ) ) ;
                  break ;

               case DocumentProperty.COUNTRYCOMMENT :
                  break ;

               case DocumentProperty.CREATE_DATE :
                  document.setCreateDate ( CDate.dateFromDBDate ( rs.getString(3).trim() ) ) ;
                  break ;

               case DocumentProperty.LAST_UPDATE_CTS_TIPS :
                  /* Last Update in CTS Tips  */
                  break ;

               case DocumentProperty.URL :
                  document.setURL ( rs.getString(3).trim() ) ;
                  break ;

               case DocumentProperty.VERSION :
                  document.setVersion ( rs.getString(3).trim() ) ;
                  break ;

               case DocumentProperty.CHECKED_OUT_BY :
                  document.setCheckedOutBy ( rs.getString(3).trim() ) ;
                  break ;

            }
         }
      }

      if ( descript.length() > 0 )
         document.setDescription ( descript.trim() ) ;

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }


   /*------------------------------------------------------------*/
   /* readDocumentQueryText
    */
   /**
    *    This method will read the document Query Text
    *
    *  @param conn The connection to use when reading
    *  @param document The DocumentDraft object to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private  int readDocumentQueryText ( Connection conn, DocumentDraft document ) throws Exception {
      Statement     stmt       = conn.createStatement() ;
      ResultSet     rs         = null ;
      String        sqlStmt    = null ; 
      StringBuffer  comments   = new StringBuffer() ;
      int           searchInd  = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Form the SQL                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlStmt = "SELECT QUERYTEXT "+
                "FROM QUEST.DOCQUERYTEXT "+
                "WHERE  DOCIND = "+document.getDocInd() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sqlStmt ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rs.next() )
         document.setQueryText ( new DocumentQueryText ( rs.getString(1).trim() ) ) ;

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* readDocumentLinks
    */
   /**
    *    This method will read in all of the links
    *  that this document contains
    *  
    *
    *  @param conn The Database Connetion to use when reading
    *  @param document The DocumentDraft to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   private  int readDocumentLinks ( Connection conn, DocumentDraft document ) throws SQLException {
      int       rc    = 0 ;
      Statement stmt  = conn.createStatement() ;
      ResultSet rs    = null ;
      Vector    links = new Vector ( 10, 10 );

      rs = stmt.executeQuery ( "SELECT A.DOCLINKIND, B.TITLE FROM "+
                               "QUEST.DOCLINKS A, "+
                               "QUEST.DOCUMENTS B "+
                               "WHERE A.DOCLINKIND = B.DOCIND AND "+
                               "A.DOCIND = "+document.getDocInd() ) ;

      while ( rs.next() ) {
         DocumentLink link = new DocumentLink ( document.getDocInd(),
                                                rs.getInt(1), 
                                                rs.getString(2).trim() );

         links.addElement ( link ) ;
      }

      document.setDocumentLinks ( links ) ;

      rs.close() ;
      stmt.close() ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* readReferringDocumentLinks
    */
   /**
    *    This method will read in all of the links
    *  that this document contains
    *  
    *
    *  @param conn The Database Connetion to use when reading
    *  @param document The DocumentDraft to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   private  int readDocumentDraftReferringLinks ( Connection conn, DocumentDraft document ) throws SQLException {
      int       rc    = 0 ;
      Statement stmt  = conn.createStatement() ;
      ResultSet rs    = null ;
      Vector    links = new Vector ( 10, 10 );

      rs = stmt.executeQuery ( "SELECT A.DOCLINKIND, B.TITLE FROM "+
                               "QUEST.DOCLINKS A, "+
                               "QUEST.DOCUMENTS B "+
                               "WHERE A.DOCIND = B.DOCIND AND "+
                               "A.DOCLINKIND = "+document.getDocInd() ) ;

      while ( rs.next() ) {
         DocumentLink link = new DocumentLink ( rs.getInt(1),
                                                document.getDocInd(), 
                                                rs.getString(2).trim() );

         links.addElement ( link ) ;
      }

      document.setReferringDocumentLinks ( links ) ;

      rs.close() ;
      stmt.close() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* readDocumentLocale
    */
   /**
    *    This method will read in all Geography
    *  and country information on a document
    *  
    *
    *  @param conn The DB/2 Connection to use to read with
    *  @param document The DocumentDraft object to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   private  int readDocumentLocale ( Connection conn, DocumentDraft document ) throws Exception {
      LocaleRec locale          = new LocaleRec ( conn ) ;
      Statement stmt            = conn.createStatement() ;
      int       rc              = 0 ;
      ResultSet rs              = null ;                          
      boolean   continueProcess = true ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT A.GEOIND, B.DESCRIPT "+
                               "FROM QUEST.DOCGEO A, "+
                               "     TIGRIS.TYPEGEOGRAPHY B "+
                               "WHERE DOCIND = "+document.getDocInd()+" AND "+
                               "      A.GEOIND = B.GEOIND "+
                               "FOR FETCH ONLY" ) ;

      while ( ( continueProcess ) && ( rs.next() ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is world wide, do not process any more          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rs.getInt(1) == TypeList.GEO_WORLD_WIDE ) {
            locale.setWorldWide ( true ) ;
            continueProcess = false ;
         }
         else
            locale.addGeography ( new TypeGeoRec ( rs.getInt(1), rs.getString(2).trim() ) ) ;
      }

      rs.close() ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read in all of the countries                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( continueProcess )
         rs = stmt.executeQuery ( "SELECT A.COUNTRYCODEIND, B.PHONECOUNTRYCODE, B.DESCRIPT,     B.ZIPMASK,   "+
                                  "       B.PHONEMASK,      B.ZIPDIGITS,        B.TIMEDIFF,     B.MULTIZONE, "+
                                  "       B.USESDST,        B.CURRENCY,         B.CURRENCYMASK, B.LANG01,    "+
                                  "       B.LANG02,         B.LANG03,           C.GEOIND,       B.ISOCODE    "+
                                  "FROM QUEST.DOCCOUNTRY A,                                                  "+
                                  "     TIGRIS.TYPECOUNTRYCODE B,                                            "+
                                  "     TIGRIS.GEOCOUNTRY C                                                  "+
                                  "WHERE DOCIND = "+document.getDocInd()+"  AND                              "+
                                  "      A.COUNTRYCODEIND = B.COUNTRYCODEIND AND                             "+
                                  "      A.COUNTRYCODEIND = C.COUNTRYCODEIND                                 "+
                                  "FOR FETCH ONLY" ) ;

      while ( ( continueProcess ) && ( rs.next() ) ) {
         locale.addCountry ( new TypeCountryCodeRec ( rs.getInt(1), 
                                                      rs.getString(2), 
                                                      rs.getString(3).trim(), 
                                                      rs.getString(4),
                                                      rs.getString(5), 
                                                      (short)rs.getInt(6),
                                                      rs.getString(7),
                                                      rs.getString(8), 
                                                      rs.getString(9),
                                                      rs.getString(10),
                                                      rs.getString(11),
                                                      (short)rs.getInt(12),
                                                      (short)rs.getInt(13),
                                                      (short)rs.getInt(14),
                                                      (short)rs.getInt(15),
                                                      rs.getString(16) ) ) ;
      } /* while  */

      document.setLocale ( locale ) ;

      rs.close() ;
      stmt.close() ;

      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* readCommentHistory
    */
   /**
    *    This method will read up to 5 of the latest
    *  comments for this document. This method
    *  requires a valid value in the existingDocInd property
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static int readCommentHistory ( Connection con, DocumentDraft document) throws Exception {
      Statement     stmt       = con.createStatement() ;
      ResultSet     rs         = null ;
      String        sqlStmt    = null ; 
      int           searchInd  = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Form the SQL                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlStmt = "SELECT DOCIND, COMMENT "+
                "FROM QUEST.DOCUMENTS "+
                "WHERE  EXISTINGDOCIND = "+document.getExistingDocInd()+" AND "+
                "       STOPDATE IS NOT NULL AND "+
                "       RECYCLED = 'N' AND "+
                "       TEMPLATE = 'N' "+
                "ORDER BY STOPDATE DESC FETCH FIRST 5 ROWS ONLY" ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sqlStmt ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {
         if ( rs.getString(2) != null )
            document.addCommentToHistory ( new DocumentComment ( rs.getInt ( 1 ),
                                                                 rs.getString(2).trim() ) ) ;
         else
            document.addCommentToHistory ( new DocumentComment ( rs.getInt ( 1 ),
                                                                 "N/A" ) ) ;
      }

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }




   /*------------------------------------------------------------*/
   /* readDocumentBrands
    */
   /**
    *    This method will read in all brands that this document is
    *  associated with. 
    *  
    *
    *  @param conn The DB/2 Connection to use to read with
    *  @param document The Document object to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   private static int readDocumentBrands ( Connection conn, Document document ) throws Exception {
      Statement stmt            = conn.createStatement() ;
      int       rc              = 0 ;
      ResultSet rs              = null ;                          
      boolean   continueProcess = true ;
      Vector    brandVec        = new Vector ( 1, 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT A.TYPECATIND,      B.DESCRIPTION, B.PARENTCATIND, "+
                               "       B.ORDERNUM,        B.CHILDSORT,   B.OWNER "+
                               "FROM QUEST.CATEGORY A, "+
                               "     PRODUCT.TYPECATEGORY B "+
                               "WHERE A.DOCIND = "+document.getDocInd()+" AND "+
                               "      A.TYPECATIND = B.TYPECATIND AND "+
                               "      B.CATEGORYLEVEL = '"+TypeCategory.LEVEL_BRAND+"' "+
                               "FOR FETCH ONLY" ) ;

      while ( rs.next() ) {
         TypeCategoryRec brandRec = new TypeCategoryRec ( rs.getInt(1) ) ;

         brandRec.setDescript      ( rs.getString(2) ) ;
         brandRec.setCategoryLevel ( TypeCategory.LEVEL_BRAND ) ;
         brandRec.setParentCatInd  ( rs.getInt(3) ) ;
         brandRec.setOrderNum      ( rs.getInt(4) ) ;
         brandRec.setChildSort     ( rs.getString(5) ) ;
         brandRec.setOwner         ( rs.getString(6) ) ;
         brandVec.addElement ( brandRec ) ;
      }


      rs.close() ;
      stmt.close() ;

      document.setBrands ( brandVec ) ;

      return( rc ) ;
   }

   /*------------------------------------------------------------*/
   /* readDocumentFamilies
    */
   /**
    *    This method will read in all families that this document is
    *  associated with. Please note that this method will populate the
    *  brands as Integer objects. Each integer object is translated to the
    *  correct TypeCategoryObject once the document has been returned from
    *  the stored procedure, in the Document.readFromDatabase method
    *  
    *
    *  @param conn The DB/2 Connection to use to read with
    *  @param document The Document object to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   private static int readDocumentFamilies ( Connection conn, Document document ) throws Exception {
      Statement stmt            = conn.createStatement() ;
      int       rc              = 0 ;
      ResultSet rs              = null ;                          
      boolean   continueProcess = true ;
      Vector    familyVec        = new Vector ( 1, 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT A.TYPECATIND,      B.DESCRIPTION, B.PARENTCATIND, "+
                               "       B.ORDERNUM,        B.CHILDSORT,   B.OWNER "+
                               "FROM QUEST.CATEGORY A, "+
                               "     PRODUCT.TYPECATEGORY B "+
                               "WHERE A.DOCIND = "+document.getDocInd()+" AND "+
                               "      A.TYPECATIND = B.TYPECATIND AND "+
                               "      B.CATEGORYLEVEL = '"+TypeCategory.LEVEL_FAMILY+"' "+
                               "FOR FETCH ONLY" ) ;

      while ( rs.next() ) {
         TypeCategoryRec familyRec = new TypeCategoryRec ( rs.getInt(1) ) ;

         familyRec.setDescript      ( rs.getString(2) ) ;
         familyRec.setCategoryLevel ( TypeCategory.LEVEL_FAMILY ) ;
         familyRec.setParentCatInd  ( rs.getInt(3) ) ;
         familyRec.setOrderNum      ( rs.getInt(4) ) ;
         familyRec.setChildSort     ( rs.getString(5) ) ;
         familyRec.setOwner         ( rs.getString(6) ) ;
         familyVec.addElement ( familyRec ) ;
      }


      rs.close() ;
      stmt.close() ;

      document.setFamilies ( familyVec ) ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* readDocumentMachines
    */
   /**
    *    This method will read in all machines that this document is
    *  associated with. Please note that this method will populate the
    *  machines as Integer objects. Each integer object is translated to the
    *  correct TypeCategoryObject once the document has been returned from
    *  the stored procedure, in the Document.readFromDatabase method
    *  
    *
    *  @param conn The DB/2 Connection to use to read with
    *  @param document The Document object to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   private static int readDocumentMachines ( Connection conn, Document document ) throws Exception {
      Statement stmt            = conn.createStatement() ;
      int       rc              = 0 ;
      ResultSet rs              = null ;                          
      boolean   continueProcess = true ;
      Vector    machineVec        = new Vector ( 1, 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT A.TYPECATIND,      B.DESCRIPTION, B.PARENTCATIND, "+
                               "       B.ORDERNUM,        B.CHILDSORT,   B.OWNER, "+
                               "       B.CATEGORYLEVEL "+
                               "FROM QUEST.CATEGORY A, "+
                               "     PRODUCT.TYPECATEGORY B "+
                               "WHERE A.DOCIND = "+document.getDocInd()+" AND "+
                               "      A.TYPECATIND = B.TYPECATIND AND "+
                               "      B.CATEGORYLEVEL in ('"+TypeCategory.LEVEL_MACHINE+"', '"+TypeCategory.LEVEL_SUBCATEGORY+"') "+
                               "FOR FETCH ONLY" ) ;

      while ( rs.next() ) {
         TypeCategoryRec machineRec = new TypeCategoryRec ( rs.getInt(1) ) ;

         machineRec.setDescript      ( rs.getString(2) ) ;
         machineRec.setCategoryLevel ( rs.getString(7) ) ;
         machineRec.setParentCatInd  ( rs.getInt(3) ) ;
         machineRec.setOrderNum      ( rs.getInt(4) ) ;
         machineRec.setChildSort     ( rs.getString(5) ) ;
         machineRec.setOwner         ( rs.getString(6) ) ;
         machineVec.addElement ( machineRec ) ;
      }


      rs.close() ;
      stmt.close() ;

      document.setMachines ( machineVec ) ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* readDocumentModels
    */
   /**
    *    This method will read in all models that this document is
    *  associated with. Please note that this method will populate the
    *  models as Integer objects. Each integer object is translated to the
    *  correct TypeCategoryObject once the document has been returned from
    *  the stored procedure, in the Document.readFromDatabase method
    *  
    *
    *  @param conn The DB/2 Connection to use to read with
    *  @param document The Document object to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   private static int readDocumentModels ( Connection conn, Document document ) throws Exception {
      Statement stmt            = conn.createStatement() ;
      int       rc              = 0 ;
      ResultSet rs              = null ;                          
      boolean   continueProcess = true ;
      Vector    modelVec        = new Vector ( 1, 1 ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT A.TYPECATIND,      B.DESCRIPTION, B.PARENTCATIND, "+
                               "       B.ORDERNUM,        B.CHILDSORT,   B.OWNER "+
                               "FROM QUEST.CATEGORY A, "+
                               "     PRODUCT.TYPECATEGORY B "+
                               "WHERE A.DOCIND = "+document.getDocInd()+" AND "+
                               "      A.TYPECATIND = B.TYPECATIND AND "+
                               "      B.CATEGORYLEVEL = '"+TypeCategory.LEVEL_MODEL+"' "+
                               "FOR FETCH ONLY" ) ;

      while ( rs.next() ) {
         TypeCategoryRec modelRec = new TypeCategoryRec ( rs.getInt(1) ) ;

         modelRec.setDescript      ( rs.getString(2) ) ;
         modelRec.setCategoryLevel ( TypeCategory.LEVEL_MODEL ) ;
         modelRec.setParentCatInd  ( rs.getInt(3) ) ;
         modelRec.setOrderNum      ( rs.getInt(4) ) ;
         modelRec.setChildSort     ( rs.getString(5) ) ;
         modelRec.setOwner         ( rs.getString(6) ) ;
         modelVec.addElement ( modelRec ) ;
      }


      rs.close() ;
      stmt.close() ;

      document.setModels ( modelVec ) ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* readDocumentPartNumbers
    */
   /**
    *    This method will read in all partNumbers that this document is
    *  associated with. Please note that this method will populate the
    *  partNumbers as Integer objects. Each integer object is translated to the
    *  correct TypeCategoryObject once the document has been returned from
    *  the stored procedure, in the Document.readFromDatabase method
    *  
    *
    *  @param conn The DB/2 Connection to use to read with
    *  @param document The Document object to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   private static int readDocumentPartNumbers ( Connection conn, Document document ) throws Exception {
      Statement stmt            = conn.createStatement() ;
      int       rc              = 0 ;
      ResultSet rs              = null ;                          
      boolean   continueProcess = true ;
      Vector    partNumberVec        = document.getModels() ;

      if ( partNumberVec == null )
         partNumberVec = new Vector ( 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT A.TYPECATIND,      B.DESCRIPTION, B.PARENTCATIND, "+
                               "       B.ORDERNUM,        B.CHILDSORT,   B.OWNER "+
                               "FROM QUEST.CATEGORY A, "+
                               "     PRODUCT.TYPECATEGORY B "+
                               "WHERE A.DOCIND = "+document.getDocInd()+" AND "+
                               "      A.TYPECATIND = B.TYPECATIND AND "+
                               "      B.CATEGORYLEVEL = '"+TypeCategory.LEVEL_PARTNUMBER+"' "+
                               "FOR FETCH ONLY" ) ;

      while ( rs.next() ) {
         TypeCategoryRec partNumberRec = new TypeCategoryRec ( rs.getInt(1) ) ;

         partNumberRec.setDescript      ( rs.getString(2) ) ;
         partNumberRec.setCategoryLevel ( TypeCategory.LEVEL_PARTNUMBER ) ;
         partNumberRec.setParentCatInd  ( rs.getInt(3) ) ;
         partNumberRec.setOrderNum      ( rs.getInt(4) ) ;
         partNumberRec.setChildSort     ( rs.getString(5) ) ;
         partNumberRec.setOwner         ( rs.getString(6) ) ;
         partNumberVec.addElement ( partNumberRec ) ;
      }


      rs.close() ;
      stmt.close() ;

      document.setModels ( partNumberVec ) ;

      return( rc ) ;
   }                       


   /**
     * This method will read in the Document status
     *
     * @param conn The connection read with
     * @param document The Document to read for
     **/
   public static int readDocumentStatus ( Connection conn, Document document ) throws Exception {
      DocumentStatus statusRec       = null ;
      Statement      stmt            = conn.createStatement() ;
      int            rc              = 0 ;
      ResultSet      rs              = null ;                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT STATUSIND, STATUSDATE "+
                               "FROM QUEST.DOCSTATUS "+
                               "WHERE DOCIND = "+document.getDocInd()+" "+
                               "ORDER BY RECNUM FOR FETCH ONLY" ) ;

      while ( rs.next() )
         statusRec = new DocumentStatus ( document.getDocInd(), rs.getInt(1), rs.getString(2).trim() ) ;

      rs.close() ;                        
      stmt.close() ;

      document.setStatus ( statusRec ) ;

      return( rc ) ;
   }


   /**
     *  This method will read in all applicable Brands, Families, Machines and Sub Categories
     **/
   public static int readDocumentApplicable ( Connection conn, Document document ) throws SQLException {
      Statement stmt    = conn.createStatement() ;
      Vector    brands  = new Vector (1 ) ;
      Vector    family  = new Vector ( 1 ) ;
      Vector    machine = new Vector ( 1 ) ;
      ResultSet rs   = stmt.executeQuery ( "SELECT A.TYPECATIND, A.DESCRIPTION, A.CATEGORYLEVEL, A.PARENTCATIND "+
                                           "FROM PRODUCT.TYPECATEGORY A, "+
                                           "     PRODUCT.TYPECATCHILD B "+
                                           "WHERE A.TYPECATIND = B.PARENTCATIND AND "+
                                           "      B.TYPECATIND IN (SELECT TYPECATIND FROM QUEST.CATEGORY WHERE DOCIND = "+document.getDocInd()+")" ) ;

      while ( rs.next() ) {
         TypeCategoryRec categoryRec = new TypeCategoryRec ( rs.getInt(1) ) ;

         categoryRec.setDescript ( rs.getString ( 2 ).trim() ) ;
         categoryRec.setCategoryLevel ( rs.getString(3).trim() ) ;
         categoryRec.setParentCatInd ( rs.getInt(4) ) ;
         categoryRec.updateRecStatus ( TypeCategoryRec.REC_NOCHANGE ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Brand                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_BRAND ) ) {
            if ( !brands.contains ( categoryRec ) )
               brands.addElement ( categoryRec ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Family                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_FAMILY ) ) {
            if ( !family.contains ( categoryRec ) )
               family.addElement ( categoryRec ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Machine                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_MACHINE ) ) {
            if ( !machine.contains ( categoryRec ) )
               machine.addElement ( categoryRec ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Sub Category                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_SUBCATEGORY ) ) {
            if ( !machine.contains ( categoryRec ) )
               machine.addElement ( categoryRec ) ;
         }



      }
      document.setApplicableBrands ( brands ) ;
      document.setApplicableFamilies ( family ) ;
      document.setApplicableMachines ( machine ) ;

      rs.close() ;
      stmt.close() ;
      return( 0 ) ;
   }

} /* DocumentDraftReader */
