package com.ibm.nzna.projects.common.quest.doc ;       
/*-----------------------------------------------------------*/
/*   DocumentDraftWriter                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentDraftWriter                 */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.*;
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class has static methods used when saving a DocumentDraft object to Quest in
 * the QUEST.DOCUMENTS table.
 *
 *
 * @version 1.0, 2/21/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentDraftWriter                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentDraftWriter extends Object {





   /*------------------------------------------------------------*/
   /* writeToDatabase
    */
   /**
    *    This method will write a document to the 
    *  database.
    *  
    *
    *  @param document The DocumentDraft Object to write
    *  @param None
    *  @param None
    *
    *  @return 0 if everything is Ok, SQLCode if something failed. However, the following
    *          codes will be returned if an internal error occurred outside of DB/2:<br><br>
    *
    * Error Code  Description<br>
    * ----------- ------------------------------<br>
    * -2          Couldn't write base information (QUEST.DOCUMENTS)<br>
    * -3          Couldn't write Graphic informtion (QUEST.DOCGRAPHIC)<br>
    * -4          Couldn't write Locale Information (QUEST.DOCCOUNTRY, QUEST.DOCGEO)<br>
    * -5          Couldn't write the DocumentDraft Categories ( QUEST.DOCCATEGORY)<br>
    * -6          Couldn't write the DocumentDraft Body  (QUEST.DOCBODY)<br>
    * -7          Couldn't write the DocumentDraft File Attachments - Only for SSDs and Service Publications (QUEST.DOCMEDIA)<br>
    * -8          Coundn't write the document status<br>
    * -9          Couldn't write document BFTM<br>
    * -10         Couldn't write Query Text<br>
    * -11         Couldn't write Properties<br>
    *//*---------------------------------------------------------*/
   protected static int writeToDatabase ( DocumentDraft document ) throws Exception {
      StringBuffer sqlBuffer = new StringBuffer () ;
      int          sqlCode   = 0 ;                                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, delete from a couple of tables that we only      */
      /* support inserts (no updates) of.                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlBuffer.append ( "DELETE FROM QUEST.CATEGORY WHERE DOCIND = "+document.getDocInd()+SqlRunner.END_DELIM ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now,   we must save the base information of the         */
      /* document. This base information is written to             */
      /* QUEST.DOCUMENTS                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( writeBaseInfo ( document, sqlBuffer ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write the Graphics information                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( writeGraphics ( document, sqlBuffer ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Write the Locale information                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( writeLocale ( document, sqlBuffer ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Write the Categories                                    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( writeCategories ( document, sqlBuffer ) ) {

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Write the File Attachments                              */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( writeFileAttachments ( document, sqlBuffer ) ) {


                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Write the Status                                        */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( writeStatus ( document, sqlBuffer ) ) {

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Write the Properties                                    */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        if ( writeProperties ( document, sqlBuffer ) ) {

                           /*-----------------------------------------------------------*/
                           /*                                                           */
                           /*   Write the BFTM                                          */
                           /*                                                           */
                           /*-----------------------------------------------------------*/
                           if ( writeBFTM ( document, sqlBuffer ) ) {


                              /*-----------------------------------------------------------*/
                              /*                                                           */
                              /*   This method will write our query text                   */
                              /*                                                           */
                              /*-----------------------------------------------------------*/
                              if ( writeQueryText ( document, sqlBuffer ) ) {
                                 /*-----------------------------------------------------------*/
                                 /*                                                           */
                                 /*   Ok, if we have any SQL to run, do so now                */
                                 /*                                                           */
                                 /*-----------------------------------------------------------*/
                                 if ( sqlBuffer.length() > 1 ) {
                                    sqlCode = SqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST, sqlBuffer.toString() ) ;

                                    if ( sqlCode == 0 ) {

                                       if ( document.getDocumentType() == TypeList.DOCTYPE_SSD )
                                          writeAdditionalInfo ( document ) ;
                                       else
                                          writeBody ( document ) ;

                                    }
                                 } /* if sqlBuffer.length  */

                              } /* query Text  */
                              else
                                 sqlCode = -10 ;

                           } /* BFTM  */
                           else
                              sqlCode = -9 ;
                        } /* properties  */
                        else
                           sqlCode = -11 ;

                     } /* status  */
                     else
                        sqlCode = -8 ;

                  } /* writeFileAttachments  */
                  else
                     sqlCode = -7 ;

               } /* if writeCategories */
               else
                  sqlCode = -5 ;

            } /* if writeLocale */
            else
               sqlCode = -4 ;

         } /* if writeGraphics */
         else
            sqlCode = -3 ;

      } /* if writeBaseInfo */
      else sqlCode = -2 ;


      return( sqlCode ) ;
   } /* writeToDatabase  */




   /*------------------------------------------------------------*/
   /* writeBaseInfo
    */
   /**
    *    This method will write all information needed
    *  to save the passed document to the
    *  QUEST.DOCUMENTS table
    *
    *  @param document the DocumentDraft to save
    *  @param sqlBuffer The Buffer to put the SQL needed to save in
    *  @param None
    *
    *  @return TRUE if everything went Ok
    *//*---------------------------------------------------------*/
   private static boolean writeBaseInfo ( DocumentDraft document, StringBuffer sqlBuffer ) {
      boolean rc        = true ;                                                                 
      String  comments  = document.getOriginalComments()+"\n"+document.getComments() ;

      comments = Text.cleanNullableDBString ( comments ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Make sure that no template will be saved with a filename*/
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( document.getTemplate() )
         document.clearFilename() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Depending on the Record Status, write the SQL           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( document.getRecStatus () ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   For a delete, we only mark PUBLISH to No and            */
         /* Recycled to Yes                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case document.REC_DELETE :
            sqlBuffer.append ( "DELETE FROM QUEST.DOCMEDIA WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCBODY WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCCOUNTRY WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCGEO WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUESTNEW.GRAPHICS WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCHISTORY WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCLINKS WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCPROPERTIES WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCQUERYTEXT WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCSTATUS WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.PRODDOCS WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.NOTIFICATION WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.CATEGORY WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            sqlBuffer.append ( "DELETE FROM QUEST.DOCUMENTS WHERE DOCIND = "+document.getDocInd()+""+SqlRunner.END_DELIM ) ;
            break ;                          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   For an update, we update all fields                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case document.REC_UPDATE :
            sqlBuffer.append ( "UPDATE QUEST.DOCUMENTS "+
                               "SET RECYCLED       = 'N', "+
                               "    DBUSER         = '"+document.getLastTouchedBy()+"', "+
                               "    CHANGEDTIME    = CURRENT TIMESTAMP, "+
                               "    DOCTYPEIND     = "+document.getDocumentType()+", "+
                               "    DOCCLASSIND    = "+document.getDocumentClass()+", "+
                               "    TITLE          = "+Text.cleanNullableDBString ( document.getTitle() )+", "+
                               "    OWNER          = "+Text.cleanNullableDBString ( document.getOwner() )+", "+
                               "    REVIEWPERIOD   = "+document.getMonthsToReview()+", "+
                               "    AUTHIND        = 0, "+
                               "    KNOWLEDGEBASE  = '"+Text.getYN ( document.getKnowledgeBase() )+"', "+
                               "    WEBABLE        = '"+Text.getYN ( document.getWebable() )+"', "+
                               "    WEBONLY        = '"+Text.getYN ( document.getWebOnly() )+"', "+
                               "    FAXABLE        = '"+Text.getYN ( document.getFaxable() )+"', "+
                               "    FLASHNEWS      = '"+Text.getYN ( document.getHotNews() )+"', "+
                               "    FLASHDATE      =  "+Text.cleanNullableDBString ( document.getHotNewsDate() )+", "+
                               "    FAQ            = '"+Text.getYN ( document.getFaq() )+"', "+
                               "    EMAILABLE      = '"+Text.getYN ( document.getEMailable() )+"', "+
                               "    EMAILABLEDATE  =  "+Text.cleanNullableDBString ( document.getEMailableDate() )+", "+
                               "    SUMMARY        =  "+Text.cleanNullableDBString ( document.getEMailComments() )+", "+
                               "    METRICIND      =  "+document.getMetric()+", "+
                               "    PARTNUM        =  "+Text.cleanNullableDBString ( document.getPartNumber() )+", "+
                               "    PUBTYPE        =  "+document.getPublicationType()+", "+
                               "    LNDOCID        =  "+Text.cleanNullableDBString ( document.getFilename() )+", "+
                               "    SOURCE         =  "+Text.cleanNullableDBString ( document.getSource() )+", "+
                               "    COMMENT        =  "+comments+", "+
                               "    STARTDATE      =  '"+document.getStartDate()+"', "+
                               "    STOPDATE       =   "+Text.cleanNullableDBString ( document.getStopDate() )+", "+
                               "    EXISTINGDOCIND =  "+document.getExistingDocInd()+", "+
                               "    REQUESTOR      =   "+Text.cleanNullableDBString ( document.getRequestor() )+", "+
                               "    WORKIND        = "+document.getWorkRequired()+", "+
                               "    MAINTLOCK      = 'N' "+
                               "WHERE DOCIND = "+document.getDocInd()+
                               SqlRunner.END_DELIM
                             );
            break ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   For an INSERT, put in all fields                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case document.REC_INSERT :
            String stopDate = document.getStopDate() ;

            if ( ( stopDate == null ) || ( stopDate.length() == 0 ) )
               stopDate = "NULL" ;
            else
               stopDate = "'"+stopDate+"'" ;

            sqlBuffer.append ( "INSERT INTO QUEST.DOCUMENTS "+
                               "( DOCIND, "+
                               "  RECYCLED, "+
                               "  DBUSER, "+
                               "  CHANGEDTIME, "+
                               "  DOCTYPEIND, "+
                               "  DOCCLASSIND, "+
                               "  TITLE, "+
                               "  OWNER, "+
                               "  REVIEWPERIOD, "+
                               "  AUTHIND, "+
                               "  KNOWLEDGEBASE, "+
                               "  WEBABLE, "+
                               "  WEBONLY, "+
                               "  FAXABLE, "+
                               "  FLASHNEWS, "+
                               "  FLASHDATE, "+
                               "  FAQ, "+
                               "  EMAILABLE, "+
                               "  EMAILABLEDATE, "+
                               "  SUMMARY, "+
                               "  METRICIND, "+
                               "  PARTNUM, "+
                               "  PUBTYPE, "+
                               "  LNDOCID, "+
                               "  SOURCE, "+
                               "  WORKIND, "+
                               "  MAINTLOCK, "+
                               "  COMMENT,   "+
                               "  STARTDATE, "+
                               "  STOPDATE,  "+
                               "  EXISTINGDOCIND, "+
                               "  SHOW, "+
                               "  TEMPLATE, "+
                               "  REQUESTOR ) values "+
                               "(    "+document.getDocInd()+", "+
                               "     'N', "+
                               "     '"+document.getLastTouchedBy()+"', "+
                               "     CURRENT TIMESTAMP, "+
                               "     "+document.getDocumentType()+", "+
                               "     "+document.getDocumentClass()+", "+
                               "     "+Text.cleanNullableDBString ( document.getTitle() )+", "+
                               "     "+Text.cleanNullableDBString ( document.getOwner() )+", "+
                               "     "+document.getMonthsToReview()+", "+
                               "     0, "+
                               "     '"+Text.getYN ( document.getKnowledgeBase() )+"', "+
                               "     '"+Text.getYN ( document.getWebable() )+"', "+
                               "     '"+Text.getYN ( document.getWebOnly() )+"', "+
                               "     '"+Text.getYN ( document.getFaxable() )+"', "+
                               "     '"+Text.getYN ( document.getHotNews() )+"', "+
                               "      "+Text.cleanNullableDBString ( document.getHotNewsDate() )+", "+
                               "     '"+Text.getYN ( document.getFaq() )+"', "+
                               "     '"+Text.getYN ( document.getEMailable() )+"', "+
                               "      "+Text.cleanNullableDBString ( document.getEMailableDate() )+", "+
                               "      "+Text.cleanNullableDBString ( document.getEMailComments() )+", "+
                               "      "+document.getMetric()+", "+
                               "      "+Text.cleanNullableDBString ( document.getPartNumber() )+", "+
                               "      "+document.getPublicationType()+", "+
                               "      "+Text.cleanNullableDBString ( document.getFilename() )+", "+
                               "      "+Text.cleanNullableDBString ( document.getSource() )+", "+
                               "     "+document.getWorkRequired()+", "+
                               "     'N', "+
                               "      "+comments+", "+
                               "     '"+document.getStartDate()+"', "+
                               "     "+Text.cleanNullableDBString ( document.getStopDate() )+", "+
                               "     "+document.getExistingDocInd()+", "+
                               "      'Y', "+
                               "      '"+Text.getYN ( document.getTemplate() )+"', "+
                               "      "+Text.cleanNullableDBString ( document.getRequestor() )+" ) "+
                               SqlRunner.END_DELIM
                             );
            break ;

      }


      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* writeGraphics
    */
   /**
    *    This method will save all of the graphics
    *  for the document. This does not includes making
    *  an entry in the QUEST.GRAPHICS table. This method will
    *  fail if an entry is not in QUEST.GRAPHICS for each graphic
    *  associated with the document.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean writeGraphics ( DocumentDraft document, StringBuffer sqlBuffer ) {
      boolean rc       = true ;
      Vector  graphics = document.getGraphics() ;                                         


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, delete all graphics in the DOCGRAPHIC table      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlBuffer.append ( "DELETE FROM QUESTNEW.GRAPHICS WHERE DOCIND = "+document.getDocInd()+SqlRunner.END_DELIM ) ;                                             


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have a valid vector, continue to walk through it  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( document.valid() ) && ( graphics != null ) && ( graphics.size() > 0 ) ) {
         DocumentGraphic graphic = null ;
         int             i       = 0 ;
         int             size    = graphics.size() ;              

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Walk through the vector                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            graphic = (DocumentGraphic) graphics.elementAt(i) ;

            if ( ( graphic != null ) && ( graphic.valid() ) ) {

               /*------------------------------------------------------------*/
               /*                                                            */
               /*   Save the graphic as an INSERT regardless of the graphics */
               /* rec status because we already deleted all graphics above   */
               /*                                                            */
               /*------------------------------------------------------------*/
               sqlBuffer.append ( "INSERT INTO QUESTNEW.GRAPHICS "+
                                  "( DOCIND, FILENAME )"+
                                  "values "+
                                  "( "+document.getDocInd()+", "+
                                  "  '"+graphic.getFilename()+"')"+SqlRunner.END_DELIM ) ;
            }

            i++ ;
         }
      }



      return( rc ) ;
   } /* writeGraphics */





   /*------------------------------------------------------------*/
   /* writeLocale
    */
   /**
    *    This method will update/insert/delete the
    *  entries in the QUEST.DOCCOUNTRY table
    *  and QUEST.DOCGEO table.
    *
    *  @param document The document to save
    *  @param sqlBuffer The buffer to place the SQL Commands in
    *  @param None
    *
    *  @return TRUE if everything was written Ok, FALSE if not
    *//*---------------------------------------------------------*/
   private static boolean writeLocale ( DocumentDraft document, StringBuffer sqlBuffer ) {
      LocaleRec localeRec = document.getLocale() ;
      boolean   rc        = false ;                                  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, delete all Countries and Geographies             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlBuffer.append ("DELETE FROM QUEST.DOCGEO     WHERE DOCIND = "+document.getDocInd()+SqlRunner.END_DELIM ) ;
      sqlBuffer.append ("DELETE FROM QUEST.DOCCOUNTRY WHERE DOCIND = "+document.getDocInd()+SqlRunner.END_DELIM ) ;                               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, if we have a locale rec, process it                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( document.valid() ) && ( localeRec != null ) ) {
         TypeCountryCodeRec countryRec   = null ;
         TypeGeoRec         geoRec       = null ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are World Wide, insert 1 geography with indicator */
         /* of 6 meaning 'World Wide'                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( localeRec.isWorldWide() ) {
            sqlBuffer.append ( "INSERT INTO QUEST.DOCGEO "+
                               "( DOCIND, GEOIND, DBUSER, CHANGEDTIME )"+
                               "VALUES "+
                               " ( "+document.getDocInd()+", "+       // DocInd
                               "   6, "+                              // Geography indicator
                               "   '"+document.getLastTouchedBy()+"', "+     // DBUser
                               "   CURRENT TIMESTAMP )"+              // Changed Time
                               SqlRunner.END_DELIM ) ;  
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise add each geography                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            Vector geoVec     = localeRec.getGeoListContainingAll() ;
            Vector countryVec = localeRec.getSingleCountryList() ;
            int    i          = 0 ;                  

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, save the geographies                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( geoVec != null ) && ( geoVec.size() > 0 ) ) {
               int length = geoVec.size() ;

               while ( i < length ) {
                  sqlBuffer.append ( "INSERT INTO QUEST.DOCGEO "+
                                     "(DOCIND, GEOIND, DBUSER, CHANGEDTIME )"+
                                     "VALUES "+
                                     " ( "+document.getDocInd()+", "+                       // DocInd
                                     "   "+((TypeGeoRec)geoVec.elementAt(i)).getInd()+","+  // Geography indicator
                                     "   '"+document.getLastTouchedBy()+"', "+                     // DBUser
                                     "   CURRENT TIMESTAMP )"+                              // Changed Time
                                     SqlRunner.END_DELIM ) ;  

                  i++ ;
               }
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now that the geos are saved, get the countries      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
               int length = countryVec.size() ;

               i = 0 ;
               while ( i < length ) {
                  sqlBuffer.append ( "INSERT INTO QUEST.DOCCOUNTRY "+
                                     "(DOCIND, COUNTRYCODEIND, DBUSER, CHANGEDTIME )"+
                                     "VALUES "+
                                     "( "+document.getDocInd()+", "+
                                     "  "+((TypeCountryCodeRec)countryVec.elementAt(i)).getInd()+", "+
                                     "  '"+document.getLastTouchedBy()+"', "+
                                     "  CURRENT TIMESTAMP )"+
                                     SqlRunner.END_DELIM ) ;
                  i++ ;
               } /* while i < length */
            } /* if country vec is good and has at least 1 entry */
         } /*  if we are not world wide */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are here then nothing went wrong                  */                             
         /*                                                           */
         /*-----------------------------------------------------------*/
         rc = true ;

      } else
         rc = true ;

      return( rc ) ;

   }





   /*------------------------------------------------------------*/
   /* writeCategories
    */
   /**
    *    This method will save all of the categories
    *  of a document
    *  
    *
    *  @param document The DocumentDraft to save
    *  @param sqlBuffer The SQLBuffer to write to
    *  @param None
    *
    *  @return TRUE if everything was written Ok
    *//*---------------------------------------------------------*/
   private static boolean writeCategories ( DocumentDraft document, StringBuffer sqlBuffer ) {
      boolean rc         = false ;
      Vector  categories = document.getCategory() ;                                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, if we have data, save it                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( document.valid() ) && ( categories != null ) && ( categories.size() > 0 ) ) {
         int             i        = 0 ;
         int             size     = categories.size() ;
         TypeCategoryRec category = null ;

         while ( i < size ) {

            category = (TypeCategoryRec)categories.elementAt(i) ;
            sqlBuffer.append ( "INSERT INTO QUEST.CATEGORY "+
                               "(DOCIND, TYPECATIND )"+
                               "values "+
                               "( "+document.getDocInd()+", "+
                               "  "+category.getInd()+")"+SqlRunner.END_DELIM ) ;

            i++ ;
         } /* while */

      } /* if categories */

      rc = true ;

      return( rc ) ;
   } /* writeCategories */






   /*------------------------------------------------------------*/
   /* writeBody
    */
   /**
    *    This method will write the document's
    *  body. Please note that the document must
    *  already be saved to the QUEST.DOCUMENTS table
    *  prior to using this method because it does not write to
    *  a buffer but does the update/insert/delete in the database
    *
    *  @param document The DocumentDraft to write
    *  @param 
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean writeBody ( DocumentDraft document ) throws Exception {
      SQLMethod         sqlMethod  = new SQLMethod ( DatabaseSystem.QUEST, "writeBody", 5 ) ;
      DocumentBody      body       = document.getDocumentBody() ;
      boolean           rc         = true ;
      Statement         deleteStmt = sqlMethod.createStatement() ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, delete all body chunks from the database         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         deleteStmt.executeUpdate ("DELETE FROM QUEST.DOCBODY WHERE DOCIND = "+document.getDocInd() ) ;
         deleteStmt.close() ;


         if ( ( body != null ) && 
              ( document.valid() ) &&
              ( ( body.getRecStatus () == body.REC_INSERT ) ||
                ( body.getRecStatus () == body.REC_UPDATE ) ) ) {
            Vector bodyChunks = body.getBodyChunks() ;

            if ( ( bodyChunks != null ) && ( bodyChunks.size() > 0 ) ) {
               int i    = 0 ;
               int size = bodyChunks.size() ;

               while ( i < size ) {
                  PreparedStatement stmt = sqlMethod.createPreparedStatement ( "INSERT INTO QUEST.DOCBODY "+
                                                                               "(DOCIND, recnum, BODY, DBUSER, CHANGEDTIME) "+
                                                                               "values ( "+document.getDocInd()+", "+
                                                                               "         "+(i+1)+", "+
                                                                               "         ?, "+
                                                                               "         '"+document.getLastTouchedBy()+"', "+
                                                                               "         CURRENT TIMESTAMP )" ) ;
                  System.out.println ( "***\n***\n*** Body Chunk SIze:"+((String)bodyChunks.elementAt(i)).length() ) ;
                  stmt.setString ( 1, (String)bodyChunks.elementAt(i) ) ;
                  stmt.execute() ;
                  stmt.close() ;
                  i++ ;
               } /* while */

            }

         }
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* writeAdditionalInfo
    */
   /**
    *    This method will write the document's additional information
    *
    *  @param document The DocumentDraft to write
    *  @param 
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean writeAdditionalInfo ( DocumentDraft document ) throws Exception {
      SQLMethod         sqlMethod  = new SQLMethod ( DatabaseSystem.QUEST, "writeAddInfo", 5 ) ;
      String            addInfo    = document.getAdditionalInfo() ;
      boolean           rc         = true ;
      Statement         deleteStmt = sqlMethod.createStatement() ;

      try {
         PreparedStatement stmt = sqlMethod.createPreparedStatement ( "UPDATE QUEST.DOCUMENTS "+
                                                                      "SET ADDINFO = ? "+
                                                                      "WHERE DOCIND = "+document.getDocInd() ) ;
         stmt.setString ( 1, addInfo ) ;
         stmt.execute() ;
         stmt.close() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* writeFileAttachments
    */
   /**
    *    This method will write the document's
    *  file attachments. Please note that the entries in the
    *  QUEST.MULTIMEDIA table must already be present prior to calling
    *  this method.
    *
    *  @param document The DocumentDraft to write
    *  @param sqlBuffer The StringBuffer to write the SQL to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean writeFileAttachments ( DocumentDraft document, StringBuffer sqlBuffer ) throws Exception {
      boolean           rc         = true ;
      Vector            files      = document.getFileAttachments() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, delete all entries                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlBuffer.append ( "DELETE FROM QUEST.DOCFILES WHERE DOCIND = "+document.getDocInd()+SqlRunner.END_DELIM ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, walk the list if we have one                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( document.valid() ) && ( files != null ) && ( files.size() > 0 ) ) {
         int          i       = 0 ;
         int          size    = files.size() ;
         DocumentFile file    = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Walk the list saving each file attachment               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            file = (DocumentFile) files.elementAt(i) ;

            if ( ( file != null ) &&
                 ( file.valid() ) ) {
               sqlBuffer.append ( "INSERT INTO QUEST.DOCFILES (DOCIND, FILENAME, DESCRIPT, SIZE, URL, VERSION, FTPMODE, ORDERNUM) "+
                                  "values "+
                                  "( "+document.getDocInd()+", "+
                                  "  '"+file.getFilename()+"', "+
                                  "  '"+Text.cleanDBString ( file.getDescription() )+"', "+
                                  "  "+file.getSize()+", "+
                                  "  '"+Text.cleanDBString ( file.getURL() )+"', "+
                                  "  '"+Text.cleanDBString ( file.getVersion() )+"', "+
                                  "  "+file.getFTPMode()+", "+
                                  "  "+file.getOrderNum()+" )"+SqlRunner.END_DELIM ) ;
            }
            i++ ;
         } /* while  */
      } /* if  */


      return( rc ) ;
   }

   /**
     *  This method will create the SQL needed to save the document status
     **/
   private static boolean writeStatus ( Document document, StringBuffer sqlBuffer ) {
      if ( ( document.valid() ) && ( document.getStatus() != null ) ) {
         document.getStatus().setDocInd ( document.getDocInd() ) ;
         sqlBuffer.append ( document.getStatus().toDraftSQL () ) ;
      }
      return( true ) ;
   }


   /**
     *  This method will create the SQL needed to save the document status
     **/
   private static boolean writeQueryText ( Document document, StringBuffer sqlBuffer ) {
      sqlBuffer.append ( "DELETE FROM QUEST.DOCQUERYTEXT WHERE DOCIND = "+document.getDocInd()+SqlRunner.END_DELIM ) ;

      if ( ( document.valid() ) && ( document.getQueryText() != null ) && ( document.getQueryText().toString().length() > 0 ) ) {
         sqlBuffer.append ( "INSERT INTO QUEST.DOCQUERYTEXT "+
                            "values ( "+document.getDocInd()+", "+
                            "         '"+Text.cleanDBString ( document.getQueryText().toString() )+"', "+
                            "         'QUEST', "+
                            "         CURRENT TIMESTAMP)"+SqlRunner.END_DELIM ) ;
      }
      return( true ) ;
   }



   /**
     *  This method will create the SQL needed to write out the BFTM
     **/
   private static boolean writeBFTM  ( Document document, StringBuffer sqlBuffer ) {
      if ( document.valid() ) {
         Vector brands   = document.getBrands() ;
         Vector families = document.getFamilies() ;
         Vector models   = document.getModels () ;
         Vector machines = document.getMachines() ;
         Vector all      = new Vector() ;

         if ( ( brands != null ) && ( brands.size() > 0 ) ) {
            int i    = 0 ;
            int size = brands.size() ;

            while ( i < size ) {
               if ( !all.contains ( brands.elementAt(i) ) )
                  all.addElement ( brands.elementAt(i) ) ;
               i++ ;
            }
         }

         if ( ( families != null ) && ( families.size() > 0 ) ) {
            int i    = 0 ;
            int size = families.size() ;

            while ( i < size ) {
               if ( !all.contains ( families.elementAt ( i ) ) )
                  all.addElement ( families.elementAt(i) ) ;
               i++ ;
            }
         }

         if ( ( models != null ) && ( models.size() > 0 ) ) {
            int i    = 0 ;
            int size = models.size() ;

            while ( i < size ) {
               if ( !all.contains ( models.elementAt(i) ) )
                  all.addElement ( models.elementAt(i) ) ;
               i++ ;
            }
         }


         if ( ( machines != null ) && ( machines.size() > 0 ) ) {
            int i    = 0 ;
            int size = machines.size() ;

            while ( i < size ) {
               if ( !all.contains ( machines.elementAt(i) ) )
                  all.addElement ( machines.elementAt(i) ) ;
               i++ ;
            }
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, loop through all records and write the SQL         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( all.size() > 0 ) {
            int i    = 0 ;
            int size = all.size() ;

            while ( i < size ) {
               sqlBuffer.append ( "INSERT INTO QUEST.CATEGORY (DOCIND, TYPECATIND) "+
                                  "VALUES ( "+document.getDocInd()+", "+((TypeCategoryRec)all.elementAt(i)).getInd()+")"+
                                  SqlRunner.END_DELIM ) ;
               i++ ;
            }
         }
      }
      return( true ) ;
   }


   /**
     *  This method will write out all of the SQL needed to save all of the properties
     **/
   private static boolean writeProperties ( DocumentDraft document, StringBuffer sqlBuffer ) {
      Vector properties = document.getProperties() ;

      sqlBuffer.append ("DELETE FROM QUEST.DOCPROPERTIES WHERE DOCIND = "+document.getDocInd()+SqlRunner.END_DELIM ) ;

      if ( ( document.valid() ) && ( properties != null ) && ( properties.size() > 0 ) ) {
         int              i        = 0 ;
         int              size     = properties.size() ;
         DocumentProperty property = null ;

         while ( i < size ) {
            property = (DocumentProperty)properties.elementAt(i) ;
            if ( property.getValue() != null )
               sqlBuffer.append ( "INSERT INTO QUEST.DOCPROPERTIES VALUES ("+
                                  ""+document.getDocInd()+", "+
                                  ""+property.getPropertyInd()+", "+
                                  ""+property.getOrderNum()+", "+
                                  "'"+Text.cleanDBString ( property.getValue() )+"', "+
                                  "USER, "+
                                  "CURRENT TIMESTAMP)"+SqlRunner.END_DELIM ) ;
            i++ ;
         }
      }

      return( true ) ;
   }
} /* DocumentDraftWriter */
