package com.ibm.nzna.projects.qit.doc ;
/*-----------------------------------------------------------*/
/*   DocumentFamily                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentFamily                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This record holds information about the association between a Family and Document 
 *
 *
 * @version 1.0, 6/2/1999 
 * @author  David Lentz
 */
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentFamily                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentFamily extends TypeGroupRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean      associateAll  = false ;
   private TypeBrandRec brandRec      = null ;
   private TypeGroupRec groupRec      = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a blank DocumentFamily
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentFamily () {
      super ( 0, "",0, 0 ) ;
   }

   /*------------------------------------------------------------*/
   /* setData
    */
   /**
    *    This method will set the data in this record from the TypeGroupRec
    *  passed,
    *  
    *
    *  @param brandRec The TypeBrandRec that the GroupRec belongs to
    *  @param groupRec The TypeGroupRec to set the data from
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setData ( TypeBrandRec brandRec, TypeGroupRec groupRec ) {
      if ( ( brandRec != null ) && ( groupRec != null ) ) {
         this.brandRec = brandRec ;
         this.groupRec = groupRec ;
         setInd ( groupRec.getInd() ) ;
         setDescript ( groupRec.getDescript() ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* getTypeGroupRec
    */
   /**
    *    This method will return the TypeGroupRec
    *  that this association represents
    *  
    *
    *  @return TypeGroupREc
    *//*---------------------------------------------------------*/
   public TypeGroupRec getTypeGroupRec () {
      return( groupRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getTypeBrandRec
    */
   /**
    *    This method will return the TypeBrandRec
    *  that is part of this association
    *  
    *
    *
    *  @return TypeBrandRec
    *//*---------------------------------------------------------*/
   public TypeBrandRec getTypeBrandRec () {
      return( brandRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getAll
    */
   /**
    *    This method will return TRUE if this
    *  Doc Family/Brand association represents
    *  all machines under the Family
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean getAll () {
      return( associateAll ) ;
   }




   /*------------------------------------------------------------*/
   /* setAll
    */
   /**
    *    This method will set the 'All' flag which means
    *  if the flag is TRUE, this record represents all
    *  machines under this family
    *
    *  @param TRUE or FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAll ( boolean all ) {
      this.associateAll = all ;
   }





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will return TRUE if the
    *  object passed is equal to this instance
    *  
    *
    *  @param data The data to compare
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean equals ( Object compareRec ) {
      boolean equal = false ;

      if ( compareRec instanceof DocumentFamily ) {
         equal = ( ( brandRec.getInd()       == ((DocumentFamily)compareRec).brandRec.getInd() ) &&
                   ( groupRec.getInd()       == ((DocumentFamily)compareRec).groupRec.getInd() ) ) ;

      }

      else if ( compareRec instanceof TypeBrandRec ) {
         equal = ( ( brandRec.getInd() == ((TypeBrandRec)compareRec).getInd() ) ) ;
      }

      else if ( compareRec instanceof TypeGroupRec ) {
         equal = ( ( groupRec.getInd() == ((TypeGroupRec)compareRec).getInd() ) ) ;
      }

      return( equal ) ;
   }




   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return a string representation
    *  of this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String toString () {
      String retStr = super.toString() ;//"BrandInd:"+brandRec.getInd()+"   FamilyInd:"+groupRec.getInd() ;

      if ( associateAll )
         retStr = retStr+"(All)" ; //" Brand:"+brandRec.getInd()+")" ;

      return( retStr ) ;
   }


   public Object clone () {
      DocumentFamily retRec = new DocumentFamily() ;

      retRec.groupRec      = groupRec ;
      retRec.brandRec      = brandRec ;
      retRec.setDescript ( getDescript() ) ;
      retRec.setInd ( getInd() ) ;

      return( retRec ) ;
   }

} /* DocumentFamily */
