package com.ibm.nzna.projects.common.quest.doc ;  
/*-----------------------------------------------------------*/
/*   DocumentFile                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentFile                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.net.* ;
import java.sql.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
/**
 * The DocumentFile represents a single File Attachment to a document 
 *
 *
 * @version 1.0, 10/16/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentFile                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentFile extends QuestRecord {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /**
     *  This Rec Status is only for the saving routines to know
     * if the user chose to deassociate this file.
     **/
   public static final int REC_DEASSOCIATE = -100 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   File Locations. These are the only valid file locations */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static String[] locations = {
      new String ( "access_ibm_en" ),
      new String ( "access_ibm_jp" ),
      new String ( "access_ibm_fr" ),
      new String ( "access_ibm_fi" ),
      new String ( "access_ibm_ne" ),
      new String ( "access_ibm_no" ),
      new String ( "access_ibm_sp" ),
      new String ( "access_ibm_dk" ),
      new String ( "access_ibm_sv" ),
      new String ( "access_ibm_gr" ),
      new String ( "ambra" ),
      new String ( "aptiva" ),
      new String ( "backlevel" ),
      new String ( "bp_aptiva" ),
      new String ( "bp_netvista" ),
      new String ( "bp_options" ),
      new String ( "bp_pc" ),
      new String ( "bp_server" ),
      new String ( "bp_sysmgmt" ),
      new String ( "bp_thinkpad" ),
      new String ( "commercial_desktop" ),
      new String ( "communications" ),
      new String ( "eprm" ),
      new String ( "dos_util" ),
      new String ( "intellistation" ),
      new String ( "ispirati" ),
      new String ( "mobiles" ),
      new String ( "mobiles_pdf" ),
      new String ( "netfinity_manager" ),
      new String ( "netvista" ),
      new String ( "network" ),
      new String ( "network_station" ),
      new String ( "options" ),
      new String ( "options_iso" ),
      new String ( "os2_ews" ),
      new String ( "pc_servers" ),
      new String ( "pc_servers_iso" ),
      new String ( "pc_servers_pdf" ),
      new String ( "pcug_news" ),
      new String ( "powerpc" ),
      new String ( "private/network" ),
      new String ( "private/options" ),
      new String ( "private/server_hidden" ),
      new String ( "private/thinkpad" ),
      new String ( "private/netfinity" ),
      new String ( "ps1" ),
      new String ( "refdisks" ),
      new String ( "ssg" ),
      new String ( "sysmgmt" ),
      new String ( "system_specific" ),
      new String ( "ta_ibm_tips" ),
      new String ( "ta_support" ),
      new String ( "tcpip" ),
      new String ( "tpnls" ),
      new String ( "valuepnt" ),
      new String ( "visuals" )
   } ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  The filename of the file. This is the filename which will
     *  be placed up on the FTP Site. It should not contain any
     *  path information
     **/
   private String filename    = null ;

   /**
     *  This is the order number in which the file should be displayed
     * in a list.
     **/
   private int orderNum = 0 ;

   /**
     *  The local filename of the file. This should be the complete
     *  path to the file on the client machine.
     **/
   private String localFilename = null ;

   /**
     *  The location of the file on the FTP site. This should be the
     *  directory under /home/ftp/pub/pccbbs of the file. Please see
     *  the 'locations' array above.
     **/
   private String location = null ;

   /**
     *  This is the size of the file in bytes
     **/
   private long size = 0 ;

   /**
     *  The document indicator for this file. 
     **/
   private int documentInd  = 0 ;

   /**
     *  This should be a human readable description of the file. This is stored in the
     *  TIGRIS.MULTIMEDIA table
     **/
   private String description = null ;

   /**
     *  This should be the complete URL including Protocol and location of the file
     **/
   private String url = null ;

   /**
     *  This is the string version of the file, stored in the TIGRIS.MULTIMEDIA
     *  table 
     **/
   private String version = null ;


   /**
     *  This is the FTP Mode . 0 = Binary, 1 = Ascii
     **/
   private int ftpMode = 0 ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is a blank constructor that initializes
    *  nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentFile ( ) {
      updateRecStatus ( REC_INSERT ) ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a DocumentFile
    *  record initialized to the filename passed in.
    *  If the filename includes a path, then the filename
    *  passed will be stored in the localFilename property, and the
    *  filename only (with path stripped out) will be stored in the
    *  filename property.<br><br>
    *
    *  If the filename begins with FTP://ftp.pc.ibm.com or ftp://ftp.pc.ibm.com then the location
    *  is derived from the URL 
    *
    * <b>Please Note</b>This Constructor initializes the record to a
    * RECSTATUS of Update, not Insert. Please use the 3rd Constructor
    * to specify a rec status to initialize to.
    *
    *  @param documentInd This is the document indicator for this file.
    *  @param String filename
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentFile ( int documentInd, String filename ) {
      this.documentInd = documentInd ;
      if ( filename.indexOf ( "ftp.pc.ibm.com" ) >= 0 ) {
         String ftpStart = "ftp.pc.ibm.com/pub/pccbbs/" ;

         setFilename ( filename.substring ( filename.lastIndexOf ("/" )+1 ) ) ;
         setLocalFilename ( filename.substring ( filename.lastIndexOf ("/" ) ) ) ;
         setLocation ( filename.substring ( filename.indexOf ( ftpStart )+ftpStart.length(),
                                            filename.lastIndexOf ( "/" ) ) ) ;
      }

      else if ( FileUtil.containsPathInfo ( filename ) ) {
         setLocalFilename ( filename ) ;
         setFilename ( FileUtil.filenameFromPath ( filename ) ) ;
      } else
         setFilename ( filename ) ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a DocumentFile
    *  record initialized to the filename passed in.
    *  If the filename includes a path, then the filename
    *  passed will be stored in the localFilename property, and the
    *  filename only (with path stripped out) will be stored in the
    *  filename property.<br><br>
    *
    *  @param documentInd This is the document indicator to use, if it is 0, it will be considered a new file
    *  @param String filename
    *  @param recStatus The initial record status as defined in PersistentRec
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentFile ( int documentInd, String filename, int recStatus ) {
      this.documentInd = documentInd ;
      updateRecStatus ( recStatus ) ;
      if ( FileUtil.containsPathInfo ( filename ) ) {
         setLocalFilename ( filename ) ;
         setFilename ( FileUtil.filenameFromPath ( filename ) ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* setLocalFilename
    */
   /**
    *    This method will set the local filename property.
    *  Please see the variable documentation above
    *  for a description of this property
    *
    *  @param localFilename The new localFilename to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLocalFilename ( String localFilename ) {
      this.localFilename = localFilename ;
      updateRecStatus ( REC_UPDATE ) ;
   }




   /*------------------------------------------------------------*/
   /* setFilename
    */
   /**
    *    This method will set the filename of this file.
    *  Please see the variable documentation above
    *  for a description of this property
    *  
    *
    *  @param filename The new filename to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFilename ( String filename ) {
      if ( ( filename != null ) && ( filename.length() < 50 ) ) {
         this.filename = filename ;
         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Filenames must not be null and be less than 50 characters in length" ) ;
   }





   /*------------------------------------------------------------*/
   /* setVersion
    */
   /**
    *    This method will set the version of this
    *  file
    *  Please see the variable documentation above
    *  for a description of this property
    *  
    *
    *  @param version The new version to set to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setVersion ( String version ) {
      if ( ( version != null ) && ( version.length() < 25 ) ) {
         this.version = version ;
         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "The Version must not be null and be less than 25 characters in length" ) ;
   }





   /*------------------------------------------------------------*/
   /* setLocation
    */
   /**
    *    This method will set the location of this file on
    *  the FTP site. Please note that if the location passed
    *  is NULL or is not in the list of predefined locations,
    *  an IllegalArgumentException is thrown<br><br>
    *  Please see the variable documentation above
    *  for a description of this property
    *
    *  @param location The new location
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLocation ( String location ) {
      if ( location != null ) {
         // && ( isValidLocation ( location ) ) ) {

         if ( location.indexOf ( "ftp.pc.ibm.com" ) >= 0 ) {
            String ftpStart = "ftp.pc.ibm.com/pub/pccbbs/" ;

            location = location.substring ( location.indexOf ( ftpStart )+ftpStart.length(),
                                            location.lastIndexOf ( "/" ) )  ;
         }

         if ( isValidLocation ( location ) ) {
            this.location = location ;
            updateRecStatus ( REC_UPDATE ) ;
         }
      } else
         throw new IllegalArgumentException ( "The Location must not be null and must be already defined in the DocumentFile.java class. Cannot set location to:"+location ) ;
   }






   /*------------------------------------------------------------*/
   /* setSize
    */
   /**
    *    This method will set the size of the file,
    *  
    *  
    *
    *  @param size The new size of the file
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSize ( long size ) {
      this.size = size ;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /*------------------------------------------------------------*/
   /* isValidLocation
    */
   /**
    *    This method will return TRUE if the location
    *  passed is valid for a file
    *  
    *
    *  @param location The location to check
    *  @param None
    *  @param None
    *
    *  @return TRUE if valid, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean isValidLocation ( String location ) {
      boolean valid = false ;
      int     i     = 0 ;
      int     size  = locations.length ;

      while ( ( !valid ) && ( i < size ) ) {
         valid = locations[i].equals ( location ) ;
         i++ ;
      }

      return( valid ) ;
   }





   /*------------------------------------------------------------*/
   /* setDescription
    */
   /**
    *    This method will set the description of this
    *  file
    *  
    *
    *  @param description The new description to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescription ( String description ) {
      if ( ( description != null ) && ( description.length() < 254 ) ) {
         this.description = description ;
         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Descriptions cannot be null and must be less than 254 characters in length" ) ;

   }




   /*------------------------------------------------------------*/
   /* getDescription
    */
   /**
    *    This method will return the description of
    *  this file
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getDescription () {
      return( description ) ;
   }





   /*------------------------------------------------------------*/
   /* setURL
    */
   /**
    *    This method will set the URL of the file
    *  passed in
    *
    *  @param url The URL of the file (Please see notes above on property description)
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setURL ( String url ) {
      if ( ( url != null ) && ( url.length() < 1500 ) ) {
         //URL urlObject = new URL ( url ) ;

         this.url = url ;


         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "The URL value cannot be NULL and must be less than 1500 characters in length" ) ;
   }





   /*------------------------------------------------------------*/
   /* getFilename
    */
   /**
    *    This method will return the full filename
    *  that is on the FTP site
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getFilename () {
      return( filename ) ;
   }




   /*------------------------------------------------------------*/
   /* getLocalFilename
    */
   /**
    *    This method will return the local filename of this
    *  file on the client. If this record has just been
    *  read from the database then this value will be NULL
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getLocalFilename () {
      return( localFilename ) ;
   }




   /*------------------------------------------------------------*/
   /* getVersion
    */
   /**
    *    This method will return the version of this
    *  file
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getVersion () {
      return( version ) ;
   }




   /*------------------------------------------------------------*/
   /* getURL
    */
   /**
    *    This method will return the URL of this
    *  file. This is the full URL that can download
    *  this file from an FTP site
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getURL () {
      return( url ) ;
   }




   /*------------------------------------------------------------*/
   /* getSize
    */
   /**
    *    This method will return the size of this file
    *  in bytes
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return long
    *//*---------------------------------------------------------*/
   public long getSize () {
      return( size ) ;
   }




   /*------------------------------------------------------------*/
   /* getLocation
    */
   /**
    *    This method will return the location
    *  of this file
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getLocation () {
      return( location ) ;
   }






   /*------------------------------------------------------------*/
   /* fileExistsInQuest
    */
   /**
    *    This method will return the document indicator
    *  of the document that this file belongs to
    *  in the Quest database. This method handles any ROLLBACKS
    *  due to SQLException or Exception thrown.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return docInd as defined in TIGRIS.DOCMEDIA, or 0 if not found
    *//*---------------------------------------------------------*/
   public int fileExistsInQuest() throws SQLException, Exception {
      return( fileExistsInQuest ( filename ) ) ;
   }


   /*------------------------------------------------------------*/
   /* fileExistsInQuest
    */
   /**
    *    This method will return the Document Indicator
    *  if the Filename passed exists 
    *  in the Quest database. This method handles any ROLLBACKS
    *  due to SQLException or Exception thrown.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return docInd as defined in TIGRIS.DOCMEDIA, 0 if no document was found
    *//*---------------------------------------------------------*/
   public static int fileExistsInQuest ( String filename ) throws SQLException, Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "fileExistInQuest ( "+filename+" )", 5 ) ;
      int       rc        = 0 ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT A.DOCIND "+
                                                   "FROM TIGRIS.DOCMEDIA A, "+
                                                   "     TIGRIS.MULTIMEDIA B  "+
                                                   "WHERE B.FILENAME = '"+filename+"' AND "+
                                                   "      A.MULTIMEDIAIND = B.MULTIMEDIAIND "+
                                                   "FOR FETCH ONLY" ) ;

         if ( rs.next() )
            rc = rs.getInt(1) ;

         rs.close() ;
         stmt.close() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack () ;
         throw e ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* getLocations
    */
   /**
    *    This method will return all valid locations of
    *  a file on the FTP.PC.IBM.COM site
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String Array
    *//*---------------------------------------------------------*/
   public static String[] getLocations () {
      return( locations );
   }


   /*------------------------------------------------------------*/
   /* toSQL
    */
   /**
    *    This method will return all SQL that needs to
    *  take place to save this record to the database.
    *  Each SQL statement is separated by the SQLRunner.END_SQL_DELIM so
    *  that this SQL may be passed to the SQLRunner. Please note that
    *  this method DOES NOT set the record status back to REC_NOCHANGE
    *  since this method does not know if the operation succeeded
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toSQL () {
      String sql = "" ;

      switch ( getRecStatus() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   REC_INSERT                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case REC_INSERT :
            sql = "INSERT INTO TIGRIS.DOCFILES "+
                  " ( DOCIND, FILENAME, DESCRIPT, SIZE, URL, VERSION ) "+
                  "VALUES ( "+
                  " "+getDocumentInd()+", "+
                  " '"+getFilename()+"', "+
                  " '"+Text.cleanDBString ( getDescription())+"', "+
                  " '"+Text.cleanDBString ( getLocation() )+"', "+
                  " "+getSize()+", "+
                  " '"+Text.cleanDBString ( getURL() )+"', "+
                  " '"+Text.cleanDBString ( getVersion() )+"' )" ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   REC_UPDATE :                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case REC_UPDATE :
            sql = "UPDATE TIGRIS.DOCFILES "+
                  "SET   DOCIND   = "+getDocumentInd()+", "+
                  "      FILENAME = '"+getFilename()+"', "+
                  "      DESCRIPT = '"+Text.cleanDBString(getDescription())+"', "+
                  "      SIZE     =  "+getSize()+", "+
                  "      URL      = '"+Text.cleanDBString(getURL())+"', "+
                  "      VERSION  = '"+Text.cleanDBString(getVersion())+"' "+
                  "WHERE DOCIND   = "+getDocumentInd()+" AND "+
                  "      FILENAME = '"+getFilename()+"'" ;
            break ; 

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   REC_DELETE                                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case REC_DELETE :
            sql = "DELETE FROM TIGRIS.DOCFILE "+
                  "WHERE DOCIND   = "+getDocumentInd()+" AND "+
                  "      FILENAME = '"+getFilename()+"'" ;
            break ;

      }

      return( sql.toString () ) ;
   }





   /*------------------------------------------------------------*/
   /* resetRecStatus
    */
   /**
    *    This method will force the rec status to
    *  REC_NOCHANGE
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void resetRecStatus () {
      updateRecStatus ( REC_NOCHANGE ) ;
   }




   /*------------------------------------------------------------*/
   /* getdocumentInd
    */
   /**
    *    This method will return the doc indicator
    *  of this file
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getDocumentInd () {
      return( documentInd ) ;
   }


   /**
     *  This method will simply return the name of the file
     **/
   public String toString () {
      return( filename ) ;
   }




   /*------------------------------------------------------------*/
   /* toDraftSQL
    */
   /**
    *    This method will return all SQL that needs to
    *  take place to save this record to the database as a Draft.
    *  Each SQL statement is separated by the SQLRunner.END_SQL_DELIM so
    *  that this SQL may be passed to the SQLRunner. Please note that
    *  this method DOES NOT set the record status back to REC_NOCHANGE
    *  since this method does not know if the operation succeeded
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toDraftSQL () {
      String sql = "" ;

      switch ( getRecStatus() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   REC_INSERT                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case REC_INSERT :
            sql = "INSERT INTO QUEST.DOCFILES "+
                  " ( DOCIND, FILENAME, DESCRIPT, SIZE, URL, VERSION, FTPMODE ) "+
                  "VALUES ( "+
                  " "+getDocumentInd()+", "+
                  " '"+getFilename()+"', "+
                  " '"+Text.cleanDBString ( getDescription())+"', "+
                  " '"+Text.cleanDBString ( getLocation() )+"', "+
                  " "+getSize()+", "+
                  " '"+Text.cleanDBString ( getURL() )+"', "+
                  " '"+Text.cleanDBString ( getVersion() )+"', "+
                  "  "+ftpMode+" )" ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   REC_UPDATE :                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case REC_UPDATE :
            sql = "UPDATE QUEST.DOCFILES "+
                  "SET   DOCIND   = "+getDocumentInd()+", "+
                  "      FILENAME = '"+getFilename()+"', "+
                  "      DESCRIPT = '"+Text.cleanDBString(getDescription())+"', "+
                  "      SIZE     =  "+getSize()+", "+
                  "      URL      = '"+Text.cleanDBString(getURL())+"', "+
                  "      VERSION  = '"+Text.cleanDBString(getVersion())+"', "+
                  "      ftpMode  = "+ftpMode+" "+
                  "WHERE DOCIND   = "+getDocumentInd()+" AND "+
                  "      FILENAME = '"+getFilename()+"'" ;
            break ; 

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   REC_DELETE                                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case REC_DELETE :
            sql = "DELETE FROM QUEST.DOCFILE "+
                  "WHERE DOCIND   = "+getDocumentInd()+" AND "+
                  "      FILENAME = '"+getFilename()+"'" ;
            break ;

      }

      return( sql.toString () ) ;
   }


   /**
     *  This method will return the FTP Mode to transfer the file in
     *
     * @return 0 = Binary, 1 = ASCII
     **/
   public int getFTPMode () {
      return( ftpMode ) ;
   }


   /**
     *  This method will set the FTP Mode to transfer the file in
     **/
   public void setFTPMode ( int ftpMode ) {
      this.ftpMode = ftpMode ;
   }


   /**
     *  This method will set the order number
     *
     * @param orderNum The new order number
     **/
   public void setOrderNum ( int orderNum ) {
      if ( this.orderNum != orderNum ) {
         this.orderNum = orderNum ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   }


   /**
     *  This method will return the orderNum for this file
     *
     * @return int
     **/
   public int getOrderNum ( ) {
      return( orderNum ) ;
   }
} /* DocumentFile */
