package com.ibm.nzna.projects.common.quest.doc ;    
/*-----------------------------------------------------------*/
/*   DocumentLink                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentLink                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.pom.* ;
/**
 * This object holds information specific to a Document Link 
 *
 *
 * @version 1.0, 10/16/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentLink                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentLink extends PersistentRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * The Document Indicator of the Document that this link belongs to
     **/
   private int    docInd         = 0 ;       

   /**
     *  The Document Indicator of the Document that this link points to
     **/
   private int    linkedDocInd      = 0 ;

   /**
     *  The title of the document that is being linked to. This value is
     *  not saved, rather it is only used for reference.
     **/
   private String linkedTitle       = null ;

   /**
     * This property defines if this document links is a referring link. This
     * means that this link represents a link in another document (defined as docInd)
     * to the linkedDocInd
     **/
   private boolean referringLink = false ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates a new document link. The
    *  document title may be null
    *  
    *
    *  @param docInd The Document indicator that this link belongs to
    *  @param linkedDocInd The Document indicator that this link points to
    *  @param title The title of the document that is being linked to
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentLink ( int docInd, int linkedDocInd, String title ) {
      setDocInd ( docInd ) ;
      setLinkedDocInd ( linkedDocInd ) ;
      setTitle ( title ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocInd
    */
   /**
    *    This method will set the document indicator
    *  that this link belongs to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocInd ( int docInd ) {
      this.docInd = docInd ;
   }




   /*------------------------------------------------------------*/
   /* setLinkedDocInd
    */
   /**
    *    This method will set the document indicator
    *  that is being linked to
    *  
    *
    *  @param linkedDocInd The Document Indicator
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLinkedDocInd ( int linkedDocInd ) {
      this.linkedDocInd = linkedDocInd ;
   }




   /*------------------------------------------------------------*/
   /* getDocInd
    */
   /**
    *    This method will return the document indicator
    *  that this record belongs to.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getDocInd () {
      return( docInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getLinkedDocInd
    */
   /**
    *    This method will return the linked document
    *  indicator that this document belongs to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getLinkedDocInd () {
      return( linkedDocInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set the title of this DocumentLink
    *  
    *  
    *
    *  @param title The title to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setTitle ( String linkedTitle ) {
      this.linkedTitle = linkedTitle.trim() ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title of this
    *  object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( linkedTitle ) ;
   }





   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return a human readable
    *  output of this document link
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( docInd+"->"+linkedDocInd+"("+linkedTitle+")" ) ;
   }


   /**
     * This method will return the referring link property
     *
     * @returns boolean
     **/
   public boolean getReferringLink() {
      return ( referringLink ) ;
   }


   /**
     * This method will set the referring link property
     *
     * @param referring The referring property
     **/
   public void setReferringLink ( boolean referring ) {
      this.referringLink = referring ;
   }
} /* DocumentLink */
