package com.ibm.nzna.projects.common.quest.doc ;   
/*-----------------------------------------------------------*/
/*   DocumentRow                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentRow                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.*  ;
import java.awt.Image ;
/**
 * This class is used to show a Document object in a MultiList
 * or Detailed JTree. Basically this object is the MultiListRow
 * object for a document.<br><br>
 *
 * This class also has methods that can reorder the data that
 * is returned for a particular column
 *
 *
 * @version 1.0, 4/18/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentRow                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentRow extends Object implements java.io.Serializable, ExtendedMultiListRow, MultiListSort, MultiListRow {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int COL_DOCUMENTIND      = 1 ;
   public static final int COL_TITLE            = 2 ;
   public static final int COL_DOCCLASSIND      = 3 ;
   public static final int COL_FILENAME         = 4 ;
   public static final int COL_LASTTOUCHEDBY    = 5 ;
   public static final int COL_LASTTOUCHED      = 6 ;
   public static final int COL_EMAILABLE_DATE   = 7 ;
   public static final int MAX_COLUMNS          = 8 ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  The title of the document
     **/
   private String title = null ;

   /**
     * This is the document's eMailable date
     **/
   private String eMailableDate = null ;

   /**
     *  This is the array of lines that our title has. Each 
     * String in the array represents a line in the title. This is
     * to support the title wrapping.
     **/
   private String[] titleArray = null ;

   /**
     *  The existing document indicator
     **/
   private String existingDocumentInd = null  ;

   /**
     *  The draft document indicator
     **/
   private String documentInd = null  ;

   /**
     *  The Classification
     **/
   private int docClassInd = 0  ;

   /**
     *  This is the original value if the setLastTouchedBy method was called
     **/
   private String originalLastTouchedBy = null ;

   /**
     *  This is the timestamp of the last touched
     **/
   private String lastTouchedStamp = null ;

   /**
     *  The Filename
     **/
   private String filename = null ;

   /**
     *  The Work Required Indicator
     **/
   private int workRequiredInd = 0  ;

   /**
     *  The owner of this document
     **/
   private String owner = null ;

   /**
     *  This is the image that is returned for this
     *  document classification
     **/
   private Image docClassImage = null ;

   /**
     *  The last person who touched this record
     **/
   private String lastTouchedBy = null ;

   /**
     *  The last time this document was touched
     **/
   private String lastTouched = null ;

   /**
     *  This array holds the order in which the columns
     * will be viewed.
     **/
   private int[] columnOrder = null ;

   /**
     *  If this is a draft, this flag will be TRUE
     **/
   private boolean draft = false ;

   /**
     *  This object is the document that this row represents. Please
     * note that this object may be NULL if the document has not been 
     * read in yet. Please use this method's readDocument method to
     * read in this document and populate this property
     **/
   private Document document = null ;

   /**
     * Creates a blank row
     *
     * @param documentInd The Document Indicator
     * @param draft TRUE if this documetn is a draft
     **/
   public DocumentRow ( String documentInd,
                        boolean draft ) {
      this.documentInd = documentInd ;
      this.draft       = draft ;
   }

   /**
     * Creates a blank row
     *
     * @param documentInd The Document Indicator
     * @param existingDocumentInd The Existing Document Indicator
     * @param draft TRUE if this documetn is a draft
     **/
   public DocumentRow ( String documentInd,
                        String existingDocInd,
                        boolean draft ) {
      this.documentInd = documentInd ;
      this.existingDocumentInd = existingDocInd ;
      this.draft       = draft ;
   }

   /**
     *  Creates a new DocumentRow based on the draft passed in
     **/
   public DocumentRow ( DocumentDraft draft ) {
      this ( ""+draft.getDocInd(),
             ""+draft.getExistingDocInd(),
             draft.getTitle(),
             draft.getOwner(),
             draft.getDocumentClass(),
             draft.getFilename(),
             draft.getWorkRequired(),
             draft.getLastTouchedBy(),
             draft.getLastTouched(),
             draft.getLastTouched(),
             true ) ;

      document = draft ;
   }
   /**
     *  This default constructor takes all properties. The params
     *  are<br><br>
     * 1. DOCIND - The Document Indicator from QUEST.DOCUMENTS. If reading a list of<br>
     *             Documents from TIGRIS.DOCUMENTS, use a '0' in place of this column in<br>
     *             the select clause<br>
     * 2. EXISTINGDOCIND - The Existing Document Indicator from TIGRIS.DOCUMENTS.<br>
     * 3. TITLE - The Title of the document<br>
     * 4. OWNER - The Owner of the document<br>
     * 5. DOCCLASSIND - The Classification of the document<br>
     * 6. LNDOCID - The Lotus Notes ID of the Document<br>
     * 7. WORKIND - The Work Required Indicator<br>
     * 8. DBUSER - The DBUSER column from the XXX.DOCUMENTS table<br>
     * 9. CHANGEDTIME - The CHANGEDTIME column from the XXX.DOCUMENTS table<br>
     **/
   public DocumentRow ( String  documentInd,
                        String  existingDocumentInd,
                        String  title,
                        String  owner,
                        int     docClassInd,
                        String  filename,
                        int     workRequiredInd,
                        String  lastTouchedBy,
                        String  lastTouched,
                        String  lastTouchedStamp,
                        boolean draft ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our local vars                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.documentInd         = documentInd ;
      this.existingDocumentInd = existingDocumentInd ;
      this.title               = title ;
      this.owner               = owner ;
      this.docClassInd         = docClassInd ;
      this.filename            = filename ;
      this.workRequiredInd     = workRequiredInd ;
      this.lastTouchedBy       = lastTouchedBy ;
      this.lastTouched         = lastTouched ;  
      this.lastTouchedStamp    = lastTouchedStamp ;
      this.draft               = draft ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our default order                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      columnOrder = new int[MAX_COLUMNS] ;
      columnOrder [ 0 ] = COL_DOCCLASSIND ;
      columnOrder [ 1 ] = COL_DOCUMENTIND ;
      columnOrder [ 2 ] = COL_TITLE ;
      columnOrder [ 3 ] = COL_FILENAME ;
      columnOrder [ 4 ] = COL_LASTTOUCHEDBY ;
      columnOrder [ 5 ] = COL_LASTTOUCHED ;
   }






   /**
    *    This method will return the column data
    *  for a specific column
    *  
    *
    *  @param column The column to return data for
    *
    *  @return Object
    */
   public Object getColumnData ( int column ) {
      return( getData ( columnOrder [ column ] ) ) ;
   }

   /**
     *  This method will look up to see what data
     * to return for the index in columnOrder
     **/
   private Object getData ( int columnOrderIndex ) {
      switch ( columnOrderIndex ) {
         case COL_DOCUMENTIND :
            return( documentInd ) ;
         case COL_TITLE :
            return( titleArray ) ;
         case COL_DOCCLASSIND :
            return( docClassImage ) ;
         case COL_FILENAME :
            return( filename ) ;
         case COL_LASTTOUCHEDBY :
            return( lastTouchedBy ) ;
         case COL_LASTTOUCHED :
            return( lastTouched ) ;
         case COL_EMAILABLE_DATE :
            return( eMailableDate ) ;
      }
      return( "" ) ;
   }

   public String getLastTouchedBy () {
      return( lastTouchedBy ) ;
   }

   public void setLastTouchedBy ( String lastTouchedBy ) {
      if ( originalLastTouchedBy == null )
         originalLastTouchedBy = this.lastTouchedBy ;

      this.lastTouchedBy = lastTouchedBy ;
   }


   public int getDocClassInd () {
      return( docClassInd ) ;
   }

   public void setDocClassImage ( Image image ) {
      this.docClassImage = image ;
   }

   public boolean isDraft () {
      return( draft ) ;
   }

   public void columnSized ( int column, int width ) {
      if ( columnOrder[column] == COL_TITLE )
         titleArray = Text.formatStringLines ( title, width/6 ) ;
   }

   public void setColumnOrder ( int column, int columnId ) {
      columnOrder[column] = columnId ;
   }

   public void adjustColumnGraphics ( int column, java.awt.Graphics g ) {
   }

   public int getExistingDocumentInd () {
      return( new Integer ( existingDocumentInd ).intValue() ) ;
   }

   public int getDocumentInd () {
      return( new Integer ( documentInd ).intValue() ) ;
   }


   /**
     *  This method will return our title
     **/
   public String getTitle () {
      return( title ) ;
   }

   /**
     *  This method will read in the document or document draft that
     * this row represents.
     *
     * @param forceRead If this value is TRUE, the read will happen
     *                  regardless if the document has already been read
     *                  in.
     *
     * @returns The SQLCode if an error, zero if no error
     **/
   public int readDocument ( boolean forceRead ) {
      int returnCode  = 0 ;

      if ( forceRead )
         document = null ;

      if ( document == null ) {
         int documentInd = new Integer ( this.documentInd ).intValue() ;

         if ( isDraft() )
            document = new DocumentDraft ( documentInd ) ;
         else
            document = new Document ( documentInd ) ;

         returnCode = document.readFromDatabase() ;
      }

      return( returnCode ) ;
   }

   /**
     *  This method will return the Document object that this row represents. If
     * the document has not been read in (through the readDocument method) then
     * this method will return NULL
     **/
   public Document getDocument () {
      return( document ) ;
   }


   /**
     *  This toString method will contain the following text:<br><br>
     *
     * (Filename)Description
     **/
   public String toString () {
      return( "("+filename+") "+title ) ;
   }


   /**
     * This method will return the filename of this document
     **/
   public String getFilename () {
      return( filename ) ;
   }


   /**
     * This method will set the emailable date
     **/
   public void setEmailableDate ( String eMailableDate ) {
      this.eMailableDate = eMailableDate ;
   }

   /**
     * This method will return the eMailable date
     **/
   public String getEMailableDate () {
      return( eMailableDate ) ;
   }

   public void freeDocument () {
      document = null ;
   }


   /**
     * This method will return the compare results of the object
     * passed in to us
     **/
   public int compareTo ( MultiListSort object, int column ) {
      int rc = 0 ;


      if ( object instanceof DocumentRow ) {
         DocumentRow compareRow = (DocumentRow) object ;

         switch ( columnOrder [ column ] ) {
            case COL_DOCUMENTIND :
               int compareInd = new Integer ( compareRow.documentInd ).intValue() ;
               int thisInd    = new Integer ( documentInd ).intValue() ;

               if ( compareInd > thisInd )
                  rc = 1 ;
               else if ( compareInd < thisInd ) 
                  rc = -1 ;
               else
                  rc = 0 ;
               break ;

            case COL_TITLE :
               try {
                  rc = title.compareTo ( compareRow.title ) ;
               } catch ( NullPointerException e ) {
                  rc = 1 ;
               }
               break ;

            case COL_DOCCLASSIND :
               if ( compareRow.docClassInd > docClassInd )
                  rc = 1 ;
               else if ( compareRow.docClassInd < docClassInd )
                  rc = -1 ;
               else 
                  rc = 0 ;
               break ;

            case COL_FILENAME :
               if ( ( filename == null ) || ( filename.length() == 0 ) )
                  rc = 1 ;
               else 
                  rc = filename.compareTo ( compareRow.filename ) ;
               break ;

            case COL_LASTTOUCHEDBY :
               rc = lastTouchedBy.compareTo ( compareRow.lastTouchedBy ) ;
               break ;

            case COL_LASTTOUCHED :
               rc = CDate.compareDate ( compareRow.lastTouchedStamp, lastTouchedStamp, CDate.DATE_STAMP ) ;
               break ;

            case COL_EMAILABLE_DATE :
               rc = CDate.compareDate ( compareRow.eMailableDate, eMailableDate, CDate.DATE_WW_DATE ) ;
               break ;

         }
      }

      return( rc ) ;
   }

} /* DocumentRow */
