package com.ibm.nzna.projects.common.quest.doc ;   
/*-----------------------------------------------------------*/
/*   DocumentSQL                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentSQL                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.Vector ;
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class holds static methods that issue SQL calls 
 * that return information about a document. There are no
 * calls to read a document (Please use Document.readFromDatabase)
 * for that.
 *
 *
 * @version 1.0, 8/1/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentSQL                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentSQL {

   /**
     *  This method will return all Documents associated to the typeCatInd
     * param passed in. The Documents are returned in a Vector of 
     * DocumentRow Objects.
     *
     * @param typeCatInd The TypeCatInd representing the TypeCategory in the PRODUCT.TYPECATEGORY table
     *
     * @returns Vector of DocumentRow Objects
     **/
   public static Vector getDocumentsFromTypeCategory ( int typeCatInd ) throws SQLException  {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocumentsFromFamily ( "+typeCatInd+" )", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT A.DOCIND,  A.TITLE,  A.OWNER, A.DOCCLASSIND, "+
                                                   "       A.LNDOCID, A.DBUSER, A.CHANGEDTIME "+
                                                   "FROM TIGRIS.DOCUMENTS A, "+
                                                   "     TIGRIS.CATEGORY B "+
                                                   "WHERE A.DOCIND = B.DOCIND AND "+
                                                   "      A.PUBLISH = 'Y' AND "+
                                                   "      B.TYPECATIND = "+typeCatInd+" FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            returnVec.addElement ( new DocumentRow ( ""+rs.getInt(1),
                                                     "",
                                                     rs.getString(2).trim(),
                                                     rs.getString(3).trim(),
                                                     rs.getInt(4),
                                                     rs.getString(5).trim(),
                                                     0,
                                                     rs.getString(6).trim(),
                                                     rs.getString(7),
                                                     rs.getString(7),
                                                     false ) ) ;
         }

         rs.close() ;

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   } /* getDocumentsFromFamily  */



   /**
     *  This method will return a Vector of Documents that have been previously deleted as DocumentRow objects
     * that are associated to the Family passed in. Please note
     * that this means that a document must be associated directly
     * to the Family to be returned. If a document is associated
     * to a product in that family then the document IS NOT returned.
     *
     * @param familyInd The TypeCatInd representing the family
     *
     * @returns Vector
     **/
   public static Vector getDeletedDocumentsFromFamily ( int familyInd ) throws SQLException  {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocumentsFromFamily ( "+familyInd+" )", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT A.DOCIND,  A.TITLE,  A.OWNER, A.DOCCLASSIND, "+
                                                   "       A.LNDOCID, A.DBUSER, A.CHANGEDTIME "+
                                                   "FROM TIGRIS.DOCUMENTS A, "+
                                                   "     TIGRIS.CATEGORY B "+
                                                   "WHERE A.DOCIND = B.DOCIND AND "+
                                                   "      A.PUBLISH = 'N' AND "+
                                                   "      B.TYPECATIND = "+familyInd+" FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            returnVec.addElement ( new DocumentRow ( ""+rs.getInt(1),
                                                     "",
                                                     rs.getString(2).trim(),
                                                     rs.getString(3).trim(),
                                                     rs.getInt(4),
                                                     rs.getString(5).trim(),
                                                     0,
                                                     rs.getString(6).trim(),
                                                     rs.getString(7),
                                                     rs.getString(7),
                                                     false ) ) ;
         }

         rs.close() ;

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   } /* getDocumentsFromFamily  */

   /**
     *  This method will return a Vector of DocumentRow objects
     * that are associated to the Family passed in. Please note
     * that this means that a Document Template must be associated directly
     * to the Family to be returned. If a document is associated
     * to a product in that family then the document IS NOT returned.
     *
     * @param familyInd The TypeCatInd representing the family
     *
     * @returns Vector
     **/
   public static Vector getDocumentTemplatesFromFamily ( int familyInd ) throws SQLException  {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocumentsFromFamily ( "+familyInd+" )", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT A.DOCIND,      A.EXISTINGDOCIND, A.TITLE,  A.OWNER, "+
                                                   "       A.DOCCLASSIND, A.LNDOCID,        A.DBUSER, A.CHANGEDTIME "+
                                                   "FROM QUEST.DOCUMENTS A, "+
                                                   "     QUEST.CATEGORY B "+
                                                   "WHERE A.DOCIND = B.DOCIND AND "+
                                                   "      B.TYPECATIND = "+familyInd+" AND "+
                                                   "      A.TEMPLATE = 'Y' FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            returnVec.addElement ( new DocumentRow ( ""+rs.getInt(1),
                                                     ""+rs.getInt(2),
                                                     rs.getString(3).trim(),
                                                     rs.getString(4).trim(),
                                                     rs.getInt(5),
                                                     rs.getString(6).trim(),
                                                     0,
                                                     rs.getString(7).trim(),
                                                     rs.getString(8),
                                                     rs.getString(9),
                                                     true ) ) ;
         }

         rs.close() ;

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   } /* getDocumentTemplatesFromFamily  */

   /**
     *  This method will return a Vector of DocumentRow objects
     * that are associated to the Family passed in. Please note
     * that this means that a Document History must be associated directly
     * to the Family to be returned. If a document is associated
     * to a product in that family then the document IS NOT returned.
     *
     * @param familyInd The TypeCatInd representing the family
     *
     * @returns Vector
     **/
   public static Vector getDocumentHistoryFromFamily ( int familyInd ) throws SQLException  {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocumentsFromFamily ( "+familyInd+" )", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT A.DOCIND,      A.EXISTINGDOCIND, A.TITLE,  A.OWNER, "+
                                                   "       A.DOCCLASSIND, A.LNDOCID,        A.DBUSER, A.CHANGEDTIME "+
                                                   "FROM QUEST.DOCUMENTS A, "+
                                                   "     QUEST.CATEGORY B "+
                                                   "WHERE A.DOCIND = B.DOCIND AND "+
                                                   "      B.TYPECATIND = "+familyInd+" AND "+
                                                   "      A.TEMPLATE = 'N' FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            returnVec.addElement ( new DocumentRow ( ""+rs.getInt(1),
                                                     ""+rs.getInt(2),
                                                     rs.getString(3).trim(),
                                                     rs.getString(4).trim(),
                                                     rs.getInt(5),
                                                     rs.getString(6).trim(),
                                                     0,
                                                     rs.getString(7).trim(),
                                                     rs.getString(8),
                                                     rs.getString(9),
                                                     true ) ) ;
         }

         rs.close() ;

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( returnVec ) ;
   } /* getDocumentTemplatesFromFamily  */



   /**
     *  This method will delete all Document Category instances of the category
     * passed in. This does not delete the documents, only the document associations
     *
     * @param categoryDescription The description of the document category
     **/
   public static boolean removeAllDocumentCategory ( String categoryDescription ) throws SQLException {
      boolean   rc                  = false ;
      SQLMethod sqlMethod           = new SQLMethod ( DatabaseSystem.QUEST, 
                                                      "DocumentSQL.removeAllDocumentCategory ( "+categoryDescription+" )", 
                                                      5 ) ;

      categoryDescription = Text.cleanDBString ( categoryDescription ) ;

      try {
         Statement stmt                = sqlMethod.createStatement() ;
         String    subSelectTypeClause = "(SELECT TYPECATIND FROM PRODUCT.TYPECATEGORY "+
                                         " WHERE DESCRIPTION = '"+categoryDescription+"' AND "+
                                         "       CATEGORYLEVEL = '"+TypeCategory.LEVEL_DOC_CATEGORY+"')" ;

         stmt.executeUpdate ( "DELETE FROM TIGRIS.CATEGORY "+
                              "WHERE TYPECATIND IN "+subSelectTypeClause ) ;
         stmt.executeUpdate ( "DELETE FROM QUEST.CATEGORY "+
                              "WHERE TYPECATIND IN "+subSelectTypeClause ) ;

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }
      sqlMethod.close() ;

      return( rc ) ;
   }


   /**
     *  This method will simply return the existing document indicator for a draft indicator.
     *
     * @param draftInd The Draft Document Indicator to look up
     *
     * @returns int
     **/
   public static int getExistingDocInd ( int docInd ) throws SQLException  {
      SQLMethod sqlMethod      = new SQLMethod ( DatabaseSystem.QUEST, "getExistingDocInd ( "+docInd+" )", 5 ) ;
      int       existingDocInd = 0 ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT EXISTINGDOCIND FROM QUEST.DOCUMENTS WHERE DOCIND = "+docInd+" FOR FETCH ONLY" ) ;


         if ( rs.next() )
            existingDocInd = rs.getInt(1) ;

         rs.close() ;

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( existingDocInd ) ;
   } /* getExistingDocInd  */



   /**
     *  This method will simply return the doc indicator of the notes id passed in. If the
     * filename is not found in the TIGRIS.XXX tables then a 0 is returned
     **/
   public static int getExistingDocInd ( String filename ) throws Exception {
      SQLMethod sqlMethod      = new SQLMethod ( DatabaseSystem.QUEST, "getExistingDocInd ( "+filename+" )", 5 ) ;
      int       existingDocInd = 0 ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT DOCIND FROM TIGRIS.DOCUMENTS WHERE LNDOCID = '"+filename+"' FOR FETCH ONLY" ) ;


         if ( rs.next() )
            existingDocInd = rs.getInt(1) ;

         rs.close() ;

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( existingDocInd ) ;
   } /* getExistingDocInd  */


   /**
     *  This method will simply return the doc indicator of the notes id passed in. If the
     * filename is not found in the QUEST.XXX tables then a 0 is returned
     **/
   public static int getDraftDocInd ( String filename ) throws Exception {
      SQLMethod sqlMethod      = new SQLMethod ( DatabaseSystem.QUEST, "getExistingDocInd ( "+filename+" )", 5 ) ;
      int       docInd = 0 ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT DOCIND FROM QUEST.DOCUMENTS WHERE LNDOCID = '"+filename+"' FOR FETCH ONLY" ) ;


         if ( rs.next() )
            docInd = rs.getInt(1) ;

         rs.close() ;

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( docInd ) ;
   } /* getExistingDocInd  */


} /* DocumentSQL */
