package com.ibm.nzna.projects.common.quest.doc ;       
/*-----------------------------------------------------------*/
/*   DocumentStatus                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentStatus                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.pom.PersistentRec ;
import com.ibm.nzna.projects.common.storedProc.extSqlRunner.* ;
import com.ibm.nzna.shared.util.CDate ;
/**
 * This object holds information about a single document status 
 *
 *
 * @version 1.0, 2/12/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentStatus                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentStatus extends PersistentRec implements java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String date        = null ;
   private int    status      = 0 ;
   private int    documentInd = 0 ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor initializes our
    *  values
    *  
    *
    *  @param status The Status indicator as defined in TIGRIS.TYPESTATUS
    *  @param date The date that the status takes effect
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentStatus ( int documentInd, int status, String date ) {
      updateRecStatus ( REC_INSERT ) ;
      setStatus ( status ) ;
      setDate   ( date ) ;
   }




   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the type of status
    *  that this is.
    *  
    *
    *  @param status The type of status. Defined in TIGRIS.TYPESTATUS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatus ( int status ) {
      if ( this.status != status ) {
         this.status = status ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setDate
    */
   /**
    *    This method will set the date for this
    *  status. The date is the date that the status
    *  takes effect
    *
    *  @param date The String version of the date
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDate ( String date ) throws IllegalArgumentException {
      if ( ( date != null ) && ( date.length() > 5 ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, if the date that was passed to us has either        */                                       
         /* a dash or forward slash in the third character then the   */
         /* date field should be ok                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( date.charAt(2) != '-' ) &&
              ( date.charAt(2) != '/' ) ) {
            date = CDate.createDate ( CDate.parse  ( date, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                      CDate.parse  ( date, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                      CDate.parse  ( date, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                      0,
                                      0,
                                      0,
                                      CDate.DATE_DATE ) ;
            }

         

         if ( ( this.date == null ) || ( !this.date.equals ( date ) ) ) {
            this.date = date ;
            updateRecStatus ( REC_UPDATE ) ;
         }
      } else
         throw new IllegalArgumentException ( "STATUSDATE is invalid:"+date ) ;
   }





   /*------------------------------------------------------------*/
   /* getStatus
    */
   /**
    *    This method will return the status indicator
    *  as defined in TIGRIS.TYPESTATUS
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getStatus () {
      return( status ) ;
   }





   /*------------------------------------------------------------*/
   /* getStatusDate
    */
   /**
    *    This method will return our status
    *  date
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getStatusDate () {
      return( date ) ;
   }



   /**
     * This method will return the SQL needed to save this status to the
     * database
     **/
   public String toSQL () {
      String sql = "" ;

      switch ( getRecStatus() ) {
         case REC_INSERT :
         case REC_UPDATE :
            sql="DELETE FROM TIGRIS.DOCSTATUS WHERE DOCIND = "+documentInd+ExtSqlRunner.END_DELIM ;
            sql+="INSERT INTO TIGRIS.DOCSTATUS "+
                 "( DOCIND, STATUSIND, RECNUM, COUNTRYCODEIND, STATUSDATE, REFERENCE, DBUSER, CHANGEDTIME )"+
                 "values "+
                 "( "+documentInd+", "+
                 " "+status+", "+
                 "  1, "+
                 "  124, "+
                 "  '"+date+"', "+
                 "  '', "+
                 "  'QUEST', "+
                 "  CURRENT TIMESTAMP)"+ExtSqlRunner.END_DELIM ;
            break ;

         case REC_DELETE :
            sql="DELETE FROM TIGRIS.DOCSTATUS WHERE DOCIND = "+documentInd+ExtSqlRunner.END_DELIM ;
            break ;

      }

      return( sql ) ;
   }



   /**
     * This method will return the SQL needed to save this status to the
     * database
     **/
   public String toDraftSQL () {
      String sql = "" ;

      switch ( getRecStatus() ) {
         case REC_NOCHANGE :
         case REC_INSERT :
         case REC_UPDATE :
            sql="DELETE FROM QUEST.DOCSTATUS WHERE DOCIND = "+documentInd+ExtSqlRunner.END_DELIM ;
            sql+="INSERT INTO QUEST.DOCSTATUS "+
                 "( DOCIND, STATUSIND, RECNUM, COUNTRYCODEIND, STATUSDATE, REFERENCE, DBUSER, CHANGEDTIME )"+
                 "values "+
                 "( "+documentInd+", "+
                 " "+status+", "+
                 "  1, "+
                 "  124, "+
                 "  '"+date+"', "+
                 "  '', "+
                 "  'QUEST', "+
                 "  CURRENT TIMESTAMP)"+ExtSqlRunner.END_DELIM ;
            break ;

         case REC_DELETE :
            sql="DELETE FROM QUEST.DOCSTATUS WHERE DOCIND = "+documentInd+ExtSqlRunner.END_DELIM ;
            break ;

      }

      return( sql ) ;
   }


   /**
     *  This method will set the document indicator that this status rec uses
     **/
   public void setDocInd ( int docInd ) {
      this.documentInd = docInd ;
   }
} /* DocumentStatus */
