package com.ibm.nzna.projects.common.quest.doc ;                       
/*-----------------------------------------------------------*/
/*   DocumentUndoableEdit                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentUndoableEdit                */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.common.quest.LocaleRec ;
/**
 * This object holds all information needed to store a single undo  
 * action. The actual data that will be undone is passed in as
 * the propertyInd property. This propertyInd is defined as one
 * of the properties in the Document.java file. 
 *
 * @version 1.0, 8/16/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentUndoableEdit                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentUndoableEdit extends Object implements java.io.Serializable {


   /**
     *  This is the property that we can undo
     **/
   private int propertyInd = 0 ;

   /**
     *  This is the data that can be undone
     **/
   private Object data = null ;


   /**
     *  Default Constructor
     *
     * @param propertyInd The property that this undo is for
     * @param data The data that this undo is for
     **/
   public DocumentUndoableEdit ( int propertyInd, Object data ) {
      this.propertyInd = propertyInd ;
      this.data        = data ;
      //System.out.println ( "Document Field Set:\n   "+data+"\n---------------------------------\n------------------------------" ) ;
   }

   public String toString () {
      return ( "Document Undo - "+getPresentationName()+"\n   Property:"+propertyInd+"\n   data:"+data ) ;
   }



   public boolean canRedo () {
      return( false ) ;
   }

   public void die () {
      data=null;
   }
   public String getPresentationName () {
      return( getUndoPresentationName() ) ;
   }

   public String getRedoPresentationName () {
      return( "" ) ;
   }

   /**
     *  This method will return the human readable descrpition of
     * what this undo is
     **/
   public String getUndoPresentationName () {
      String retStr = null ;

      switch ( propertyInd ) {
         case Document.TITLE :
            retStr = "Title Change" ;
            break ;
         case Document.UNDO_BFTMSP_CHANGE :
            retStr = "BFTMSP Change" ;
            break ;

         case Document.UNDO_ADD_FILE :
            retStr = "Add File" ;
            break ;

         case Document.UNDO_REMOVE_FILE :
            retStr = "Remove File" ;
            break ;

         case Document.FAQ :
            retStr = "FAQ Change" ;
            break ;
         case Document.FAXABLE :
            retStr = "Faxable Change" ;
            break ;
         case Document.ADDITIONAL_INFO :
            retStr = "Additional Info Change" ;
            break ;
         case Document.AUTHORITY :
            retStr = "Authority Change" ;
            break ;
         case Document.CATEGORY :
            retStr = "Category Change" ;
            break ;
         case Document.DESCRIPTION :
            retStr = "Description Change" ;
            break ;
         case Document.BODY :
            retStr = "Body Change" ;
            break ;
         case Document.CLASS :
            retStr = "Classification Change" ;
            break ;
         case Document.TYPE :
            retStr = "Type Change" ;
            break ;
         case Document.EMAILABLE :
            retStr = "eMailable Change" ;
            break ;
         case Document.EMAILABLEDATE :
            retStr = "eMailable Date Change" ;
            break ;
         case Document.EMAILCOMMENTS :
            retStr = "eMail Comments Change" ;
            break ;
         case Document.HOTNEWSDATE :
            retStr = "Hot News Date Change" ;
            break ;
         case Document.KNOWLEDGEBASE :
            retStr = "Online Assistant Change" ;
            break ;
         case Document.LOCALE :
            retStr = "Location Change" ;
            break ;
         case Document.METRIC :
            retStr = "Metric Change" ;
            break ;
         case Document.MONTHS_TO_REVIEW :
            retStr = "Months To Review Change" ;
            break ;
         case Document.OWNER :
            retStr = "Owner Change" ;
            break ;
         case Document.PUBLICATION_TYPE :
            retStr = "Publication Type Change" ;
            break ;
         case Document.QUERY_TEXT :
            retStr = "Query Text Change" ;
            break ;
         case Document.RELEASE_DATE :
            retStr = "Release Date Change" ;
            break ;
         case Document.SOURCE :
            retStr = "Source Change" ;
            break ;
         case Document.STATUS :
            retStr = "Status Change" ;
            break ;
         case Document.URL :
            retStr = "URL Change" ;
            break ;
         case Document.VERSION :
            retStr = "Version Change" ;
            break ;
         case Document.WEBABLE :
            retStr = "Webable Change" ;
            break ;
         case Document.WEBONLY :
            retStr = "Web Only Change" ;
            break ;
         default:
            retStr = "" ;
      }

      return( retStr ) ;
   }


   /**
     *  This method will actually perform the Undo by setting the correct property
     * back to the value this undo record recorded
     **/
   public void undo ( Document document ) {

      switch ( propertyInd ) {
         case Document.TITLE :
            System.out.println ( "UNdoing title. Setting new title to "+data ) ;
            document.setTitle ( (String)data ) ;
            break ;

         case Document.UNDO_BFTMSP_CHANGE :
            Vector brands   = (Vector)((Hashtable)data).get ( "BRANDS" ) ;
            Vector families = (Vector)((Hashtable)data).get ( "FAMILIES" ) ;
            Vector machines = (Vector)((Hashtable)data).get ( "MACHINES" ) ;
            Vector models   = (Vector)((Hashtable)data).get ( "MODELS" ) ;

            document.setBrands ( brands ) ;
            document.setModels ( models ) ;
            document.setFamilies ( families ) ;
            document.setMachines ( machines ) ;
            break ;

         case Document.UNDO_ADD_FILE :
            document.addFileAttachment ( (DocumentFile)data ) ;
            break ;

         case Document.UNDO_REMOVE_FILE :
            document.removeFileAttachment ( (DocumentFile)data ) ;
            break ;

         case Document.FAQ :
            document.setFaq ( ((Boolean)data).booleanValue() ) ;
            break ;

         case Document.FAXABLE :
            document.setFaxable ( ((Boolean)data).booleanValue() ) ;
            break ;
         case Document.ADDITIONAL_INFO :
            document.setAdditionalInfo ( (String)data ) ;
            break ;
         case Document.CATEGORY :
            System.out.println ( "Undoing Category. Size:"+((Vector)data).size() ) ;
            document.setCategory ( (Vector) data ) ;
            break ;
         case Document.DESCRIPTION :
            document.setDescription ( (String)data ) ;
            break ;
         case Document.BODY :
            document.setDocumentBody ( (DocumentBody)data ) ;
            break ;
         case Document.CLASS :
            document.setDocumentClass ( ((Integer)data).intValue() ) ;
            break ;
         case Document.TYPE :
            document.setDocumentType ( ((Integer)data).intValue() ) ;
            break ;
         case Document.EMAILABLE :
            document.setEMailable ( ((Boolean)data).booleanValue() ) ;
            break ;
         case Document.EMAILABLEDATE :
            document.setEMailableDate ( (String)data ) ;
            break ;
         case Document.EMAILCOMMENTS :
            document.setEMailComments ( (String)data ) ;
            break ;
         case Document.HOTNEWSDATE :
            document.setHotNewsDate ( (String)data ) ;
            break ;
         case Document.KNOWLEDGEBASE :
            document.setKnowledgeBase ( ((Boolean)data).booleanValue() ) ;
            break ;
         case Document.LOCALE :
            System.out.println ( "Undoing locale. Country Count:"+((LocaleRec)data).countryCount() ) ;
            document.setLocale ( (LocaleRec)data ) ;
            break ;
         case Document.METRIC :
            document.setMetric ( ((Integer)data).intValue() ) ;
            break ;
         case Document.MONTHS_TO_REVIEW :
            document.setMonthsToReview ( ((Integer)data).intValue() ) ;
            break ;
         case Document.OWNER :
            document.setOwner ( (String) data ) ;
            break ;
         case Document.PUBLICATION_TYPE :
            document.setPublicationType ( ((Integer)data).intValue() ) ;
            break ;
         case Document.QUERY_TEXT :
            document.setQueryText ( (DocumentQueryText) data ) ;
            break ;
         case Document.RELEASE_DATE :
            document.setReleaseDate ( (String) data ) ;
            break ;
         case Document.SOURCE :
            document.setSource ( (String) data ) ;
            break ;
         case Document.STATUS :
            document.setStatus ( (DocumentStatus) data ) ;
            break ;
         case Document.URL :
            document.setURL ( (String) data ) ;
            break ;
         case Document.VERSION :
            document.setVersion ( (String) data ) ;
            break ;
         case Document.WEBABLE :
            document.setWebable ( ((Boolean)data).booleanValue() ) ;
            break ;
         case Document.WEBONLY :
            document.setWebOnly ( ((Boolean)data).booleanValue() ) ;
            break ;
      }
   }

   public boolean isSignificant () {
      return true ;
   }

   public void redo () {
   }

   public boolean canUndo () {
      return ( true ) ;
   }
} /* DocumentUndoableEdit */
