
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import java.util.Vector;

/**
 * @version 1.7
 * @author  Vishnu Kumar
**/
class ActionDraftWriter {

  /**
   * Return the SQL for this Action.
  **/
  static String getSQL(ActionDraft ad) {

    StringBuffer sb = new StringBuffer();

    sb.append( getActionDraftSQL(ad) );
    if ( ad.valid() ) {
      // insert/update the sub-objects as necessary
      sb.append( getObjectSQL(ad) );
    }

    if ( !ad.modified() && sb.length() > 0 ) {
      // Action wasn't modified but sub-objects were - update timestamp
      sb.append("UPDATE oadraft.action " +
                "SET changedTime=CURRENT TIMESTAMP " +
                "WHERE actionInd=")
        .append( ad.getInd() ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save or delete the Action's 
   * Title, AddtlInfo and DocLink objects to the datbase.
   * 
   * @param ActionDraft        the actiondraft to be saved
  **/
  static String getObjectSQL(ActionDraft actionDraft) {

    StringBuffer sb = new StringBuffer();
    Vector v;

    //---------------------------------------------------------------------
    // Loop through each subobject contained in the ActionDraft object.
    // Update the sortOrder, parentInd and dbUser for the
    // subobject.  setSortOrder() may affect the modification state of
    // the subobject if its order has changed, but setParentInd() and
    // setDbUser() will not affect the modification state.
    //---------------------------------------------------------------------
 
    v = actionDraft.getTitles();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        Title t = (Title) v.elementAt(i);
        if ( t.valid() ) { 
          t.setSortOrder( sortOrder++ ); 
          t.setParentInd( actionDraft.getInd() );
          t.setDbUser( actionDraft.getDbUser() );
        }
        sb.append( ActionTitleDraftWriter.getSQL(t) );
      }
    }

    v = actionDraft.getAdditionalInfo();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        AddtlInfo ai = (AddtlInfo) v.elementAt(i);
        if ( ai.valid() ) { 
          ai.setSortOrder( sortOrder++ ); 
          ai.setParentInd( actionDraft.getInd() );
          ai.setDbUser( actionDraft.getDbUser() );
        }
        sb.append( ActionInfoDraftWriter.getSQL(ai) );
      }
    }

    v = actionDraft.getDocLinks();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        DocLink d = (DocLink) v.elementAt(i);
        if ( d.valid() ) { 
          d.setSortOrder( sortOrder++ );
          d.setParentInd( actionDraft.getInd() );
          d.setDbUser( actionDraft.getDbUser() );
        }
        sb.append( ActionLinkDraftWriter.getSQL(d) );
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the ActionDraft to the database.
  **/
  static String getActionDraftSQL(ActionDraft ad) { 
    String sql;

    switch ( ad.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(ad);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(ad);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(ad);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  /**
   * Returns the SQL String to insert the Action in the database.
  **/
  static String getInsertSQL(ActionDraft ad) { 
    // Note: oadraft.action.created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oadraft.action " +
         "(actionInd, activeActionInd, actionTypeInd, brandInd, " +
         "typeWorkInd, docClassInd, comments, " +
         "maintLock, startDate, stopDate, recycled, " +
         "archived, created, dbUser, changedTime) " +
       "VALUES ("
    );
    // actionInd:
    sb.append( ad.getInd()                                 ).append(',');

    // activeActionInd:
    if (ad.getActiveActionInd() == 0) {
      sb.append( "NULL"                                    ).append(',');
    } else {
      sb.append( ad.getActiveActionInd()                   ).append(',');
    }
	
    // actionTypeInd:
    sb.append( ad.getActionType().getInd()                 ).append(',');

    // brandInd:
    sb.append( ad.getBrand().getInd()                      ).append(',');

    // typeWorkInd:
    sb.append( ad.getTypeWorkRequired().getInd()           ).append(',');

    // docClassInd:
    sb.append( ad.getDocClass().getInd()                   ).append(',');

    // comments:
    String comments = ad.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append("NULL,");
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) ).append(',');
    }

    // maintLock:
    sb.append( DatabaseUtil.booleanToDB( ad.getMaintLock() ) ).append(','); 

    // startDate:
    sb.append( "CURRENT DATE"                               ).append(',');

    // stopDate:
    sb.append( "NULL"                                       ).append(',');
    
    // recycled:
    sb.append( DatabaseUtil.booleanToDB( ad.isRecycled() ) ).append(',');
    
    // archived:
    sb.append( DatabaseUtil.booleanToDB( ad.isArchived() ) ).append(',');

    // created:
    sb.append( "CURRENT TIMESTAMP"                         ).append(',');
    
    // dbUser:
    sb.append( DatabaseUtil.stringToDB( ad.getDbUser() )   ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // insert the countries into oadraft.actionCountry
    sb.append( CountryWriter.getInsertSQL( "oadraft.actionCountry",
                                           ad.getInd(),
                                           ad.getCountryList(),
                                           ad.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns the SQL String to update the Action in the database.
  **/
  static String getUpdateSQL(ActionDraft ad) {  

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oadraft.action SET");

    sb.append( " actionTypeInd=" )
      .append( ad.getActionType().getInd() );

    sb.append( ",brandInd=" )
      .append( ad.getBrand().getInd()  );

    sb.append( ",typeWorkInd=" )
      .append( ad.getTypeWorkRequired().getInd() );

    sb.append( ",docClassInd=" )
      .append( ad.getDocClass().getInd() );

    sb.append( ",comments=" );
    String comments = ad.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append( "NULL" );
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) );
    }

    sb.append( ",maintLock=" )
      .append( DatabaseUtil.booleanToDB( ad.getMaintLock() ) );

    sb.append( ",recycled=" )
      .append( DatabaseUtil.booleanToDB( ad.isRecycled() ) );  

    sb.append( ",archived=" )
      .append( DatabaseUtil.booleanToDB( ad.isArchived() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( ad.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE actionInd=").append( ad.getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oadraft.actionCountry",
                                           "actionInd",
                                            ad.getInd(),
                                            ad.getCountryList(),
                                            ad.getOriginalCountryList(),
                                            ad.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns the SQL String to delete the Action from the database.
  **/
  static String getDeleteSQL(ActionDraft ad) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);
    int actionDraftInd = ad.getInd();

    // Delete the Action Titles
    sb.append( ActionTitleDraftWriter.getDeleteAllForActionSQL(actionDraftInd) );

    // Delete the Action Additional Information 
    sb.append( ActionInfoDraftWriter.getDeleteAllForActionSQL(actionDraftInd) );

    // Delete the Action DocLinks
    sb.append( ActionLinkDraftWriter.getDeleteAllForActionSQL(actionDraftInd) );

    // Delete the Action itself
    sb.append(
      "DELETE from oadraft.actioncountry where actionind="
    ).append( actionDraftInd ).append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oadraft.action where actionind="
    ).append( actionDraftInd ).append(SqlRunner.END_DELIM);
    return sb.toString();
  }
}
