
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.util.Vector;

/**
 * @version 1.11
 * @author  Mark DeBiase
**/

class ActionInfoWriter {

   private static final String SCHEMA = "oa";

   static String getSQL(AddtlInfo a) {
      String sql;

      switch ( a.getRecStatus() ) {
         case PersistentRec.REC_INSERT:
            sql = getInsertSQL(a, SCHEMA);
            break;

         case PersistentRec.REC_UPDATE:
            sql = getUpdateSQL(a, SCHEMA);
            break;

         case PersistentRec.REC_DELETE:
            sql = getDeleteSQL(a, SCHEMA);
            break;

         default:
            sql = "";
      }
      return sql;
   }

   static String getInsertSQL(AddtlInfo a, String schema) {
      // Note: created defaults to CURRENT TIMESTAMP, not inserted
      // Note: the additional info text will be updated by the
      //       ActionWriter via a PreparedStatement because there
      //       is a limit of 4000 bytes on literal strings.

      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      sb.append(
               "INSERT INTO " + schema + ".actionAddInfo " +
               "(actionAddInfoInd, actionInd, sortOrder, addInfo, " +
               "titleInd, redirectFlag, languageInd, lastTranslated, " +
               "docClassInd, dbUser, changedTime) "+
               "VALUES ("
               );
      // actionAddInfoInd:
      sb.append( a.getInd()                                      ).append(',');

      // actionInd:
      sb.append( a.getParentInd()                                ).append(',');

      // sortOrder:
      sb.append( a.getSortOrder()                                ).append(',');

      // addInfo:
      sb.append( "'To be updated by PreparedStatement'"          ).append(',');

      // titleInd:
      if ( a.getParentTitleInd() == 0 ) {
         sb.append( "null"                                        ).append(',');
      } else {
         sb.append( a.getParentTitleInd()                         ).append(',');
      }

      // redirectFlag:
      sb.append( DatabaseUtil.booleanToDB( a.getRedirectFlag() ) ).append(',');

      // languageInd:
      sb.append( a.getLanguage().getInd()                        ).append(',');

      // lastTranslated:
      sb.append(
               DatabaseUtil.timestampToDB(a.getLastTranslationDate() )  ).append(',');

      // docClassInd:
      sb.append( a.getDocClass().getInd()                        ).append(',');

      // dbUser:
      sb.append( DatabaseUtil.stringToDB( a.getDbUser() )        ).append(',');

      // changedTime:
      sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

      /*-----------------------------------------------------------------------*/

      // only insert countries/conditions if language is English
      if ( a.getLanguage().getInd() == 1 ) {

         // insert the countries
         sb.append( CountryWriter.getInsertSQL( schema + ".actionInfoCountry",
                                                a.getInd(),
                                                a.getCountryList(),
                                                a.getDbUser() ) );

         // insert the conditions
         sb.append( ConditionWriter.getInsertSQL( schema + ".actionInfoCond",
                                                  a.getInd(), 
                                                  a.getConditions(), 
                                                  a.getDbUser() ) );
      }

      // insert the embedded links & graphics
      sb.append( getEmbeddedInsertSQL(a, schema) );

      return sb.toString();
   }

   static String getUpdateSQL(AddtlInfo a, String schema) { 
      // Note: the additional info text will be updated by the
      //       ActionWriter via a PreparedStatement because there
      //       is a limit of 4000 bytes on literal strings.

      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      //sb.append ( "DELETE FROM "+schema+".actionInfoCountry WHERE actionAddInfoInd = "+a.getInd() ) ;
      //sb.append ( SqlRunner.END_DELIM ) ;
      //sb.append ( "DELETE FROM "+schema+".actionInfoCond WHERE actionAddInfoInd = "+a.getInd() ) ;
      //sb.append ( SqlRunner.END_DELIM ) ;
      //sb.append ( "DELETE FROM "+schema+".actionAddInfo WHERE actionAddInfoInd = "+a.getInd() ) ;
      //sb.append ( SqlRunner.END_DELIM ) ;
      //sb.append ( getInsertSQL ( a, schema ) ) ;
      sb.append("UPDATE " + schema + ".actionAddInfo SET");

      sb.append( " sortOrder=" )
      .append( a.getSortOrder() );

      sb.append( ",addInfo=" )
      .append( "'To be updated by PreparedStatement'" );

      sb.append(",titleInd=");
      if ( a.getParentTitleInd() == 0 ) {
         sb.append( "null" );
      } else {
         sb.append( a.getParentTitleInd() );
      }

      sb.append( ",redirectFlag=")
      .append( DatabaseUtil.booleanToDB( a.getRedirectFlag() ) );

//    sb.append( ",languageInd=" )
//      .append( a.getLanguage().getInd() );

      sb.append( ",lastTranslated=" )
      .append( DatabaseUtil.timestampToDB(a.getLastTranslationDate() ) );

      sb.append( ",docClassInd=" )
      .append( a.getDocClass().getInd() );

      sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( a.getDbUser() ) );

      sb.append( ",changedTime=CURRENT TIMESTAMP");

      sb.append(" WHERE actionAddInfoInd=").append( a.getInd() )
      .append(" AND languageInd=").append( a.getLanguage().getInd() )
      .append(SqlRunner.END_DELIM);

      /*-----------------------------------------------------------------------*/

      // only update countries/conditions if language is English
      if ( a.getLanguage().getInd() == 1 ) {

         // update the country list (updated via delete/insert)
         sb.append( CountryWriter.getUpdateSQL( schema + ".actionInfoCountry",
                                                "actionAddInfoInd",
                                                a.getInd(),
                                                a.getCountryList(),
                                                a.getOriginalCountryList(),
                                                a.getDbUser() ) );

         // update the conditions (updated via delete/insert)
         sb.append( ConditionWriter.getUpdateSQL( schema + ".actionInfoCond",
                                                  "actionAddInfoInd",
                                                  a.getInd(),
                                                  a.getConditions(),
                                                  a.getDbUser() ) );
      }

      // update the embedded links & graphics
      sb.append( getEmbeddedUpdateSQL(a, schema) );

      return sb.toString();
   }

   static String getDeleteSQL(AddtlInfo a, String schema) {
      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      // delete the countries
      sb.append( CountryWriter.getDeleteSQL( schema + ".actionInfoCountry",
                                             "actionAddInfoInd",
                                             a.getInd() ) );

      // delete the conditions
      sb.append( ConditionWriter.getDeleteSQL( schema + ".actionInfoCond",
                                               "actionAddInfoInd",
                                               a.getInd() ) );

      // delete the embedded links & graphics
      sb.append( getEmbeddedDeleteSQL(a, schema) );

      // finally, delete the additional information
      sb.append("DELETE FROM " + 
                schema + ".actionAddInfo WHERE actionAddInfoInd=")
      .append( a.getInd() ).append(SqlRunner.END_DELIM);

      return sb.toString();
   }

   static String getEmbeddedInsertSQL(AddtlInfo a, String schema) {

      // sql to insert the embeded documents and graphics
      Vector v;
      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      v = a.getEmbeddedDocInds();
      if ( v != null ) {
         for ( int i=0; i<v.size(); i++ ) {
            int docInd = ((Integer) v.elementAt(i)).intValue();
            sb.append("INSERT INTO " + schema + ".actionInfoLinks VALUES(");
            sb.append( a.getInd() ).append(',');
            sb.append( docInd     ).append(',');
            sb.append( DatabaseUtil.stringToDB( a.getDbUser() ) ).append(',');
            sb.append( "CURRENT TIMESTAMP)").append(SqlRunner.END_DELIM);
         }
      }

      v = a.getEmbeddedGraphics();
      if ( v != null ) {
         for ( int i=0; i<v.size(); i++ ) {
            AddtlInfoGraphic g = (AddtlInfoGraphic) v.elementAt(i);
            if ( ( g.getRecStatus() != g.REC_DELETE ) && ( g.getRecStatus() != g.REC_NOINSERT ) ) {
               sb.append("INSERT INTO " + schema + ".actionInfoGraphic VALUES(");
               sb.append( a.getInd() ).append(',');
               sb.append( g.getGraphicInd() ).append(',');
               sb.append( g.getLanguage().getInd() ).append(',');
               sb.append( DatabaseUtil.timestampToDB( g.getLastTranslationDate() ) )
               .append(',');
               sb.append( g.getDocClass().getInd() ).append(',');
               sb.append( DatabaseUtil.stringToDB( a.getDbUser() ) ).append(',');
               sb.append( "CURRENT TIMESTAMP)").append(SqlRunner.END_DELIM);
            }
         }
      }

      return sb.toString();
   }

   static String getEmbeddedUpdateSQL(AddtlInfo a, String schema) {
      return getEmbeddedDeleteSQL(a, schema) + getEmbeddedInsertSQL(a, schema);
   }

   static String getEmbeddedDeleteSQL(AddtlInfo a, String schema) {
      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      sb.append("DELETE FROM " + 
                schema + ".actionInfoLinks WHERE actionAddInfoInd=")
      .append( a.getInd() ).append(SqlRunner.END_DELIM);

      sb.append("DELETE FROM " + 
                schema + ".actionInfoGraphic WHERE actionAddInfoInd=")
      .append( a.getInd() ).append(SqlRunner.END_DELIM);

      return sb.toString();
   }

   /**
    * Return SQL to delete all the Additional Information for an Action.
   **/
   static String getDeleteAllForActionSQL(int actionInd) {
      return getDeleteAllForActionSQL(actionInd, SCHEMA);
   }

   static String getDeleteAllForActionSQL(int actionInd, String schema) {
      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      sb.append(
               "DELETE from "+ schema +".actioninfocond where actionaddinfoind in " +
               "(select distinct actionaddinfoind " +
               "from " + schema + ".actionaddinfo where actionind="
               ).append( actionInd ).append(')').append(SqlRunner.END_DELIM);
      sb.append(
               "DELETE from "+ schema +".actioninfocountry where actionaddinfoind in " +
               "(select distinct actionaddinfoind " +
               "from " + schema + ".actionaddinfo where actionind="
               ).append( actionInd ).append(')').append(SqlRunner.END_DELIM);
      sb.append(
               "DELETE from "+ schema +".actioninfolinks where actionaddinfoind in " +
               "(select distinct actionaddinfoind " +
               "from " + schema + ".actionaddinfo where actionind="
               ).append( actionInd ).append(')').append(SqlRunner.END_DELIM);
      sb.append(
               "DELETE from "+ schema +".actioninfographic where actionaddinfoind in " +
               "(select distinct actionaddinfoind " +
               "from " + schema + ".actionaddinfo where actionind="
               ).append( actionInd ).append(')').append(SqlRunner.END_DELIM);
      sb.append(
               "DELETE from " + schema + ".actionaddinfo where actionind="
               ).append( actionInd ).append(SqlRunner.END_DELIM);

      return sb.toString();
   }
}
