
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;

import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant Actions.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.8
 * @author  Mark DeBiase
**/
public class ActionReader {

   /**
    * Read and populate the specified Action object.
    * The Action object's indicator should contain the actionInd to be read.
    * @param conn  the database Connection object
    * @param a     the Action object to populate
    * @param lang  the language to read the sub-objects for (0 = all languages)
    * @param mode  an OAMode value (i.e., OAMode.DEFAULT or OAMode.WEB).
    * @return true if the Action was successfully read
              false if the Action was not found in the database
   **/
   public static boolean readFromDB(Connection conn, Action a, 
                                    int languageInd, int mode) 
   throws SQLException
   {
      boolean    foundAction;
      Statement  stmt = conn.createStatement();
      ResultSet  rs;

      // turn on DB2 explain:
      //stmt.executeUpdate("SET CURRENT EXPLAIN MODE YES");
      //stmt.executeUpdate("SET CURRENT EXPLAIN SNAPSHOT YES");

      rs = stmt.executeQuery(
                            "SELECT " +
                            "actionTypeInd, brandInd, docClassInd, " +
                            "archived, created, dbUser, changedTime, openDraft " +
                            "FROM oa.action " +
                            "WHERE actionInd=" + a.getInd()  + " " +
                            "FOR FETCH ONLY"
                            );

      if ( !rs.next() ) {
         foundAction = false;
         stmt.close();
      } else {
         foundAction = true;

         //---------------------------------------------------------------------
         // Read the base Action data
         //---------------------------------------------------------------------
         a.setActionType( new TypeActionRec( rs.getShort(1), "" ) );
         a.setBrand( new TypeBrandRec( rs.getInt(2), "", 0 ) );
         a.setDocClass( new TypeDocClassRec( rs.getShort(3), "" ) );
         a.setArchived( DatabaseUtil.booleanFromDB( rs.getString(4) ) );
         a.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
         a.setDbUser( rs.getString(6).trim() );
         a.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
         a.setOpenDraft( DatabaseUtil.booleanFromDB( rs.getString(8) ) );

         stmt.close();

         //---------------------------------------------------------------------
         // Read the Action Countries
         //---------------------------------------------------------------------
         a.setCountryList(
                         CountryReader.readCountries( conn, "oa.actionCountry",
                                                      "actionInd", a.getInd() ) );

         //---------------------------------------------------------------------
         // Read the Action Titles/AddtlInfo based on the language passed in
         // Pass the mode to AddtlInfoReader for possible optimizations.
         //---------------------------------------------------------------------
         TitleReader.readActionTitles( conn, a, languageInd );
         AddtlInfoReader.readActionAddtlInfo( conn, a, languageInd, mode );

         //---------------------------------------------------------------------
         // Read the Action DocLinks
         //---------------------------------------------------------------------
         DocLinkReader.readActionDocLinks( conn, a );

         a.updateRecStatus( a.REC_NOCHANGE );
      }

      return (foundAction);
   }
}
