
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import com.ibm.nzna.shared.util.* ;
import java.util.Vector;

/**
 * @version 1.10
 * @author  Mark DeBiase
**/
class ActionTitleWriter {

   private static final String SCHEMA = "oa";





   /*------------------------------------------------------------*/
   /* getSQL
    */
   /**
    *    This method will return the SQL needed to
    *  save this Action Title's data
    *  
    *
    *  @param t The title to save
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   static String getSQL(Title t) {
      String sql;

      switch ( t.getRecStatus() ) {
         case PersistentRec.REC_INSERT:
            log ( "Saving Action Title for INSERT" ) ;
            sql = getInsertSQL(t, SCHEMA);
            break;

         case PersistentRec.REC_UPDATE:
            log ( "Saving Action Title for UPDATE" ) ;
            sql = getUpdateSQL(t, SCHEMA);
            break;

         case PersistentRec.REC_DELETE:
            log ( "Saving Action Title for DELETE" ) ;
            sql = getDeleteSQL(t, SCHEMA);
            break;

         default:
            sql = "";
      }
      return (sql);
   }

   static String getInsertSQL(Title t, String schema) {
      // Note: created defaults to CURRENT TIMESTAMP, not inserted

      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      sb.append(
               "INSERT INTO " + schema + ".actionTitle " +
               "(actionTitleInd, actionInd, sortOrder, title, languageInd, " +
               "lastTranslated, docClassInd, dbUser, changedTime) "+
               "VALUES ("
               );

      // actionTitleInd:
      sb.append( t.getInd()                                     ).append(',');

      // actionInd:
      sb.append( t.getParentInd()                               ).append(',');

      // sortOrder:
      sb.append( t.getSortOrder()                               ).append(',');

      // title:
      sb.append( DatabaseUtil.stringToDB( t.getTitle() )        ).append(',');

      // languageInd:
      sb.append( t.getLanguage().getInd()                       ).append(',');

      // lastTranslated:
      sb.append(
               DatabaseUtil.timestampToDB(t.getLastTranslationDate() ) ).append(',');

      // docClassInd:
      sb.append( t.getDocClass().getInd()                       ).append(',');

      // dbUser:
      sb.append( DatabaseUtil.stringToDB( t.getDbUser() )       ).append(',');

      // changedTime:
      sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

      /*-----------------------------------------------------------------------*/

      // only insert countries/conditions if language is English
      if ( t.getLanguage().getInd() == 1 ) {

         // insert the countries
         sb.append( CountryWriter.getInsertSQL( schema + ".actionTitleCountry",
                                                t.getInd(),
                                                t.getCountryList(),
                                                t.getDbUser() ) );

         // insert the conditions
         sb.append( ConditionWriter.getInsertSQL( schema + ".actionTitleCond",
                                                  t.getInd(), 
                                                  t.getConditions(), 
                                                  t.getDbUser() ) );
      }

      return (sb.toString());
   }

   static String getUpdateSQL(Title t, String schema) { 

      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      //sb.append("DELETE FROM "+schema+".actionTitleCountry WHERE ACTIONTITLEIND = "+t.getInd() ) ;
      //sb.append(SqlRunner.END_DELIM);
      //sb.append("DELETE FROM "+schema+".actionTitleCond WHERE ACTIONTITLEIND = "+t.getInd() ) ;
      //sb.append(SqlRunner.END_DELIM);
      //sb.append("DELETE FROM "+schema+".actionTitle WHERE ACTIONTITLEIND = "+t.getInd() ) ;
      //sb.append(SqlRunner.END_DELIM);
      //sb.append ( getInsertSQL ( t, schema ) ) ;

      sb.append("UPDATE " + schema + ".actionTitle SET");

      sb.append( " sortOrder=" )
      .append( t.getSortOrder() );

      sb.append( ",title=" )
      .append( DatabaseUtil.stringToDB( t.getTitle() ) );

//    sb.append( ",languageInd=" )
//      .append( t.getLanguage().getInd() );

      sb.append( ",lastTranslated=" )
      .append( DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) );

      sb.append( ",docClassInd=" )
      .append( t.getDocClass().getInd() );

      sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( t.getDbUser() ) );

      sb.append( ",changedTime=CURRENT TIMESTAMP");

      sb.append(" WHERE actionTitleInd=").append( t.getInd() )
      .append(" AND languageInd=").append( t.getLanguage().getInd() )
      .append(SqlRunner.END_DELIM);

      /*-----------------------------------------------------------------------*/

      // only update countries/conditions if language is English
      if ( t.getLanguage().getInd() == 1 ) {

         // update the country list (updated via delete/insert)
         sb.append( CountryWriter.getUpdateSQL( schema + ".actionTitleCountry",
                                                "actionTitleInd",
                                                t.getInd(),
                                                t.getCountryList(),
                                                t.getOriginalCountryList(),
                                                t.getDbUser() ) );

         // update the conditions (updated via delete/insert)
         sb.append( ConditionWriter.getUpdateSQL( schema + ".actionTitleCond",
                                                  "actionTitleInd",
                                                  t.getInd(),
                                                  t.getConditions(),
                                                  t.getDbUser() ) );
      }

      return (sb.toString());
   }

   static String getDeleteSQL(Title t, String schema) {
      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      // delete the countries
      sb.append( CountryWriter.getDeleteSQL( schema + ".actionTitleCountry",
                                             "actionTitleInd",
                                             t.getInd() ) );

      // delete the conditions
      sb.append( ConditionWriter.getDeleteSQL( schema + ".actionTitleCond",
                                               "actionTitleInd",
                                               t.getInd() ) );

      sb.append("DELETE FROM " + schema + ".actionTitle WHERE actionTitleInd=")
      .append( t.getInd() ).append(SqlRunner.END_DELIM);

      return (sb.toString());
   }

   /**
    * Return SQL to delete all the Titles for an Action.
   **/
   static String getDeleteAllForActionSQL(int actionInd) {
      return (getDeleteAllForActionSQL(actionInd, SCHEMA));
   }

   static String getDeleteAllForActionSQL(int actionInd, String schema) {
      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      sb.append(
               "DELETE from " + schema + ".actiontitlecond where actiontitleind in " +
               "(select distinct actiontitleind " +
               "from " + schema + ".actiontitle where actionind="
               ).append( actionInd ).append(')').append(SqlRunner.END_DELIM);
      sb.append(
               "DELETE from " + schema + ".actiontitlecountry where actiontitleind in " +
               "(select distinct actiontitleind " +
               "from " + schema + ".actiontitle where actionind="
               ).append( actionInd ).append(')').append(SqlRunner.END_DELIM);
      sb.append(
               "DELETE from " + schema + ".actiontitle where actionind="
               ).append( actionInd ).append(SqlRunner.END_DELIM);

      return (sb.toString());
   }


   /*------------------------------------------------------------*/
   /* log
    */
   /**
    *    This method checks for an instance of the LogSystem
    *  and if it is valid, uses it to log a message
    *  
    *
    *  @param logMessage The message to log
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void log ( String logMessage ) {
      if ( LogSystem.getInstance() != null )
         LogSystem.log ( 1, logMessage ) ;
   }

}
