
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import java.sql.*;
import java.util.Vector;

/**
 * @version 1.25
 * @author  Mark DeBiase
**/
public class ActionWriter {

  /**
   * Return the SQL for this Action.
  **/
  static String getSQL(Action a) {
    boolean subObjectsModified;

    StringBuffer sb = new StringBuffer();

    sb.append( getActionSQL(a) );
    if ( a.valid() ) {
      // insert/update the sub-objects as necessary
      sb.append( getObjectSQL(a) );
    }

    if ( !a.modified() && sb.length() > 0 ) {
      // Action wasn't modified but sub-objects were - update timestamp
      sb.append("UPDATE oa.action " +
                "SET changedTime=CURRENT TIMESTAMP " +
                "WHERE actionInd=")
        .append( a.getInd() ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save the Action's 
   * Title, AddtlInfo and DocLink objects to the datbase.
   * 
   * @param Action        the action to be saved
  **/
  static String getObjectSQL(Action action) {

    StringBuffer sb = new StringBuffer();
    Vector v;

    //---------------------------------------------------------------------
    // Loop through each subobject contained in the Action object.
    // Update the sortOrder, parentInd and dbUser for the
    // subobject.  setSortOrder() may affect the modification state of
    // the subobject if its order has changed, but setParentInd() and
    // setDbUser() will not affect the modification state.
    //---------------------------------------------------------------------
 
    v = action.getTitles();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        Title t = (Title) v.elementAt(i);
        if ( t.valid() ) { 
          t.setSortOrder( sortOrder++ ); 
          t.setParentInd( action.getInd() );
          t.setDbUser( action.getDbUser() );
        }
        sb.append( ActionTitleWriter.getSQL(t) );
      }
    }

    v = action.getAdditionalInfo();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        AddtlInfo a = (AddtlInfo) v.elementAt(i);
        if ( a.valid() ) { 
          a.setSortOrder( sortOrder++ ); 
          a.setParentInd( action.getInd() );
          a.setDbUser( action.getDbUser() );
        }
        sb.append( ActionInfoWriter.getSQL(a) );
      }
    }

    v = action.getDocLinks();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        DocLink d = (DocLink) v.elementAt(i);
        if ( d.valid() ) { 
          d.setSortOrder( sortOrder++ );
          d.setParentInd( action.getInd() );
          d.setDbUser( action.getDbUser() );
        }
        sb.append( ActionLinkWriter.getSQL(d) );
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the Action to the database.
  **/
  static String getActionSQL(Action a) {
    String sql;

    switch ( a.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(a);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(a);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(a);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  /**
   * Returns the SQL String to insert the Action in the database.
  **/
  static String getInsertSQL(Action a) {
    // Note: oa.action.created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.action " +
         "(actionInd, actionTypeInd, brandInd, docClassInd, " +
         "archived, dbUser, changedTime, openDraft) " +
       "VALUES ("
    );
    // actionInd:
    sb.append( a.getInd()                                 ).append(',');

    // actionTypeInd:
    sb.append( a.getActionType().getInd()                 ).append(',');

    // brandInd:
    sb.append( a.getBrand().getInd()                      ).append(',');

    // docClassInd:
    sb.append( a.getDocClass().getInd()                   ).append(',');

    // archived:
    sb.append( DatabaseUtil.booleanToDB( a.isArchived() ) ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( a.getDbUser() )   ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP, 'N')")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // insert the countries into oa.actionCountry
    sb.append( CountryWriter.getInsertSQL( "oa.actionCountry",
                                           a.getInd(),
                                           a.getCountryList(),
                                           a.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns the SQL String to update the Action in the database.
  **/
  static String getUpdateSQL(Action a) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oa.action SET");

    sb.append( " actionTypeInd=" )
      .append( a.getActionType().getInd() );

    sb.append( ",brandInd=" )
      .append( a.getBrand().getInd()  );

    sb.append( ",docClassInd=" )
      .append( a.getDocClass().getInd() );

    sb.append( ",archived=" )
      .append( DatabaseUtil.booleanToDB( a.isArchived() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( a.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE actionInd=").append( a.getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oa.actionCountry",
                                           "actionInd",
                                            a.getInd(),
                                            a.getCountryList(),
                                            a.getOriginalCountryList(),
                                            a.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns the SQL String to delete the Action from the database.
  **/
  static String getDeleteSQL(Action a) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);
    int actionInd = a.getInd();

    // Delete the Action Titles
    sb.append( ActionTitleWriter.getDeleteAllForActionSQL( actionInd ) );

    // Delete the Action Additional Information 
    sb.append( ActionInfoWriter.getDeleteAllForActionSQL( actionInd ) );

    // Delete the Action DocLinks
    sb.append( ActionLinkWriter.getDeleteAllForActionSQL( actionInd ) );

    // Delete the Action itself
    sb.append(
      "DELETE from oa.actioncountry where actionind="
    ).append( actionInd ).append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oa.action where actionind="
    ).append( actionInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return the SQL to publish an Action from an ActionDraft object.
   * 
   * @return a String containing SQL statements separated by 
   *         SqlRunner.END_DELIM.
  **/
  public static String getPublishSQL(Connection conn, ActionDraft draft)
                       throws Exception
  {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);


    // construct a new Action object from the draft object
    // the recstatus and indicators will be set appropriately
    Action active = new Action( draft );
    active.assignDatabaseKeys(conn);  // throws Exception on failures

    // if there is already an active object, then the Titles, AddtlInfo
    // and DocLinks associated with it must be wiped out 
    if (active.getRecStatus() == PersistentRec.REC_UPDATE) {
      int actionInd = active.getInd();
      sb.append( ActionTitleWriter.getDeleteAllForActionSQL(actionInd) );
      sb.append( ActionInfoWriter.getDeleteAllForActionSQL(actionInd) );
      sb.append( ActionLinkWriter.getDeleteAllForActionSQL(actionInd) );
    }

    // generate the sql to to publish the action
    sb.append( getSQL(active) );

    // update the actionResponse table:
    // 1. delete all existing entries for this actionInd
    // 2. insert all geos contained in oa.actionCountry
    //    for this actionInd, setting the response to 'N'
    // MAD 6/18: put countries instead of geos in actionResponse table
    sb.append( "DELETE FROM oa.actionResponse WHERE actionInd=")
      .append( active.getInd() ).append(SqlRunner.END_DELIM);
    sb.append(
       "INSERT INTO oa.actionResponse " +
          "SELECT DISTINCT " +
             "a.actionInd, gc.geoInd, 'N', a.dbUser, a.changedTime " +
          "FROM oa.action a, oa.actionCountry ac, tigris.geocountry gc " +
          "WHERE a.actionInd=" + active.getInd() +
          "  AND ac.actionInd=a.actionInd "+
          "  AND ac.countryInd = gc.countryCodeInd "
    ).append(SqlRunner.END_DELIM);

    // update the oadraft.node and oadraft.edgepreanswer tables with 
    // the active action indicator.  this only has to be done for drafts
    // which have never been published before.
    if (active.getRecStatus() == PersistentRec.REC_INSERT) {
      sb.append(
         "UPDATE oadraft.node " +
         "SET activeActionInd=" + active.getInd() + " " +
         "WHERE draftActionInd=" + draft.getInd()
      ).append(SqlRunner.END_DELIM);

      sb.append(
         "UPDATE oadraft.edgePreanswer " +
         "SET activeActionInd=" + active.getInd() + " " +
         "WHERE draftActionInd=" + draft.getInd()
      ).append(SqlRunner.END_DELIM);
    }
    

    // update the openDraft flag.  this is done seperately at the end
    // in case there are any failures publishing the draft.
    sb.append(
      "UPDATE oa.action " +
      "SET openDraft='N' " +
      "WHERE actionInd="
    ).append( active.getInd() ).append(SqlRunner.END_DELIM);

    // finally, update the stopDate & activeActionInd on the draft object
    sb.append(
      "UPDATE oadraft.action " +
      "SET stopDate=CURRENT DATE, " +
          "changedTime=CURRENT TIMESTAMP, " +
          "activeActionInd=" + active.getInd() + " " +
      "WHERE actionInd=" + draft.getInd() 
    ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

}
