
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.shared.pom.* ;
import java.util.Vector;

/**
 * Contains information about a graphics file embedded in the
 * text of an Additional Information object.
 *
 * <pre><br>
 * Change History<br>
 * ----------------------------------------------------------<br>
 * David Lentz    Made extend PersistentRec so it has recStatus<br>
 *
 * </pre>
 *
 * @version 1.6
 * @author  Mark DeBiase
 *
**/
public class AddtlInfoGraphic extends PersistentRec implements java.io.Serializable {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   private int              graphicInd_      = 0;
   private TypeLanguageRec  language_        = null;
   private String           lastTranslated_  = null;
   private TypeDocClassRec  docClass_        = null;
   private String           filename_        = null ;


   /**
    * New object.
    * @param graphicInd the indicator from tigris.graphics
   **/
   public AddtlInfoGraphic(int graphicInd) {
      graphicInd_ = graphicInd;
   }

   /**
    * Get the graphics indicator.
   **/
   public int getGraphicInd() {
      return (graphicInd_);
   }

   /**
    * Gets the language.
    * @return the language
   **/
   public TypeLanguageRec getLanguage() {
      return (language_);
   }

   /**
    * Sets the language.
    * @parm language the language
   **/
   public void setLanguage(TypeLanguageRec language) {
      language_ = language;
   }

   /**
    * Get the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @return a String representing a DB2 timestamp
   **/
   public String getLastTranslationDate() {
      return (lastTranslated_);
   }

   /**
    * Set the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @param translationDate a String representing a DB2 timestamp
   **/
   public void setLastTranslationDate(String translationDate) {
      lastTranslated_ = translationDate;
   }

   /**
    * Get the document classification for this object.
    * @return the document classification
   **/
   public TypeDocClassRec getDocClass() {
      return (docClass_);
   }

   /**
    * Set the document classification for this object.
    * @param docClassRec the document classification
   **/
   public void setDocClass(TypeDocClassRec docClass) {
      docClass_ = docClass;
   }

   /**
    * Update the TypeRec objects in this object with the proper descriptions.
    * Called after the object is read from the database.
   **/
   void updateTypeRecData() {
      TypeList t = TypeList.getInstance();

      // docClass
      TypeDocClassRec newDocClass;
      newDocClass = (TypeDocClassRec) t.objectFromInd( docClass_.getInd(),
                                                       TypeList.TYPE_DOCCLASS );
      if ( newDocClass != null ) {
         docClass_ = newDocClass;
      }

      // language
      TypeLanguageRec newLangRec;
      newLangRec = (TypeLanguageRec) t.objectFromInd( language_.getInd(),
                                                      TypeList.TYPE_LANGUAGE );
      if ( newLangRec != null ) {
         language_ = newLangRec;
      }

      return;
   }





   /*------------------------------------------------------------*/
   /* setFilename
    */
   /**
    *    This method will set the filename that this
    *  graphic represents
    *  
    *
    *  @param filename The new filename
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFilename ( String filename ) {
      filename_ = filename ;
   }





   /*------------------------------------------------------------*/
   /* getFilename
    */
   /**
    *    This method will return the filename of this
    *  graphic
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getFilename () {
      return ( filename_ ) ;
   }
}

