
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;



/**
 * Reader for Online Assistant Answer drafts.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.7
 * @author VishnuKumar.E
**/ 
class AnswerDraftReader {

  /**
   * Read and populate the Answer objects for the specified QuestionDraft.
   * @param conn  the database Connection object
   * @param a     the Question whose Answers are to be read
   * @param lang  the language to read the sub-objects for (0 = all languages)
  **/
  static void readAnswersForQuestion(Connection conn, QuestionDraft q, 
                                     int languageInd) 
              throws SQLException 
  {
  
    Statement stmt = conn.createStatement();
    ResultSet rs;

    rs = stmt.executeQuery( "SELECT " + 
                              "answerInd, activeAnswerInd, " +
                              "sortOrder, docClassInd, " +
                              "created, dbUser, changedTime " +
                            "FROM oadraft.answer " + 
                            "WHERE questionInd=" + q.getInd() + " " +
                            "ORDER BY sortOrder " + 
                            "FOR FETCH ONLY");
  
    //-----------------------------------------------------------------------
    // Read all the AnswerDrafts for the QuestionDraft
    //-----------------------------------------------------------------------
    Vector v = new Vector();
    while( rs.next() ) {
      Answer ans = new Answer( rs.getInt(1) );
      ans.setParentInd( q.getInd() );
      ans.setDraft(true);

      ans.setActiveAnswerInd( rs.getInt(2) );
      ans.setSortOrder( rs.getShort(3) );
      ans.setDocClass( new TypeDocClassRec( rs.getShort(4), "" ) );
      ans.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(5) ) );
      ans.setDbUser( rs.getString(6).trim() );
      ans.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
      ans.setDraft(true);

      //---------------------------------------------------------------------
      // Read the Answer conditions/countries
      //---------------------------------------------------------------------
      // MAD performance: answer conditions are now read all at once below
      //ans.setConditions(
      //   ConditionReader.readConditions( conn, "oadraft.answerCond",
      //                                   "parentAnswerInd", ans )
      //);
      // MAD performance: answer countries are now read all at once below
      //ans.setCountryList(
      //   CountryReader.readCountries( conn, "oadraft.answerCountry",
      //                                "answerInd", ans.getInd() )
      //);

      //---------------------------------------------------------------------
      // Read the Answer Titles based on the language passed in
      //---------------------------------------------------------------------
      // MAD performance: answer titles are now read all at once below
      //if (languageInd == 0) {
      //  // read for all languages
      //  TitleReader.readAnswerDraftTitles( conn, ans );
      //} else {
      //  // read for specific language
      //  TitleReader.readAnswerDraftTitles( conn, ans, languageInd );
      //}

      //---------------------------------------------------------------------
      // Read the Answer Conclusions
      //---------------------------------------------------------------------
      // MAD performance: answer conclusions are now read all at once below
      //AnswerConclReader.readConclForAnswerDraft( conn, ans );

      ans.updateRecStatus(PersistentRec.REC_NOCHANGE);
      v.addElement(ans);
    }  
    
    rs.close();
    stmt.close();

    q.setAnswers(v);

    //-----------------------------------------------------------------------
    // Read the Conditions/Countries for all Answers
    //-----------------------------------------------------------------------
    ConditionReader.readAllAnswerConditions(conn, q, "oadraft");
    CountryReader.readAllAnswerCountries(conn, q, "oadraft");

    //---------------------------------------------------------------------
    // Read all Answer Titles based on the language passed in
    //---------------------------------------------------------------------
    TitleReader.readAllAnswerDraftTitles( conn, q, languageInd );

    //-----------------------------------------------------------------------
    // Read all Answer Conclusions
    //-----------------------------------------------------------------------
    AnswerConclReader.readAllDraftAnswerConcl( conn, q );
  }
}
