
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import java.util.Vector;

/**
 * @version 1.5
 * @author  Mark DeBiase
**/
class AnswerDraftWriter {

  /**
   * Return the SQL for this Answer.
  **/
  static String getSQL(Answer ans) {

    StringBuffer sb = new StringBuffer();

    sb.append( getAnswerDraftSQL(ans) );
    if ( ans.valid() ) {
      // insert/update the sub-objects as necessary
      sb.append( getObjectSQL(ans) );
    }

    if ( !ans.modified() && sb.length() > 0 ) {
      // Answer wasn't modified but sub-objects were - update timestamp
      sb.append("UPDATE oadraft.answer " +
                "SET changedTime=CURRENT TIMESTAMP " +
                "WHERE answerInd=")
        .append( ans.getInd() ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save the Answers's 
   * Title and AnswerConcl objects to the datbase.
   * 
   * @param Answer        the Answer to be saved
  **/
  static String getObjectSQL(Answer ans) {

    StringBuffer sb = new StringBuffer();
    Vector v;
 
    v = ans.getTitles();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        Title t = (Title) v.elementAt(i);
        if ( t.valid() ) { 
          t.setSortOrder( sortOrder++ ); 
          t.setParentInd( ans.getInd() );
          t.setDbUser( ans.getDbUser() );
        }
        sb.append( AnswerTitleDraftWriter.getSQL(t) );
      }
    }

    v = ans.getConclusions();
    if (v != null) {
      // there is no sort order for conclusions
      for (int i=0; i<v.size(); i++) {
        AnswerConcl concl = (AnswerConcl) v.elementAt(i);
        if ( concl.valid() ) { 
          concl.setFromAnswerInd( ans.getInd() );
          concl.setDbUser( ans.getDbUser() );
        }
        sb.append( AnswerConclDraftWriter.getSQL(concl) );
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the Answer to the database.
  **/
  static String getAnswerDraftSQL(Answer ans) {
    String sql;

    switch ( ans.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(ans);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(ans);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(ans);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(Answer ans) {
    // Note: answer.created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oadraft.answer " +
         "(answerInd, activeAnswerInd, questionInd, sortOrder, docClassInd, " +
         "dbUser, changedTime) " +
       "VALUES ("
    );
    sb.append( ans.getInd()                                  ).append(',');
    if ( ans.getActiveAnswerInd() == 0 ) {
      sb.append( "NULL"                                     ).append(',');
    } else {
      sb.append( ans.getActiveAnswerInd()                    ).append(','); 
    }
    sb.append( ans.getParentInd()                            ).append(',');
    sb.append( ans.getSortOrder()                            ).append(',');
    sb.append( ans.getDocClass().getInd()                    ).append(',');
    sb.append( DatabaseUtil.stringToDB( ans.getDbUser() )    ).append(',');
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    // insert the countries
    sb.append( CountryWriter.getInsertSQL( "oadraft.answerCountry",
                                           ans.getInd(),
                                           ans.getCountryList(),
                                           ans.getDbUser() ) );

    // insert the conditions
    sb.append( ConditionWriter.getInsertSQL("oadraft.answerCond",
                                            ans.getInd(), 
                                            ans.getConditions(), 
                                            ans.getDbUser() ) );

    return sb.toString();
  }

  static String getUpdateSQL(Answer ans) {
    // Note: answer.questionInd is not updated

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oadraft.answer SET");
    sb.append( " sortOrder=" )
      .append( ans.getSortOrder() );
    sb.append( ",docClassInd=" )
      .append( ans.getDocClass().getInd() );
    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( ans.getDbUser() ) );
    sb.append( ",changedTime=CURRENT TIMESTAMP");
    sb.append(" WHERE answerInd=").append( ans.getInd() )
      .append(SqlRunner.END_DELIM);

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oadraft.answerCountry",
                                           "answerInd",
                                            ans.getInd(),
                                            ans.getCountryList(),
                                            ans.getOriginalCountryList(),
                                            ans.getDbUser() ) );

    // update the conditions (updated via delete/insert)
    sb.append( ConditionWriter.getUpdateSQL("oadraft.answerCond",
                                            "parentAnswerInd",
                                            ans.getInd(),
                                            ans.getConditions(),
                                            ans.getDbUser() ) );

    return sb.toString();
  }

  static String getDeleteSQL(Answer ans) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete the answer conclusions
    sb.append( AnswerConclDraftWriter.getDeleteAllForAnswerSQL(ans.getInd()) );
    
    // delete the answer titles
    sb.append( AnswerTitleDraftWriter.getDeleteAllForAnswerSQL(ans.getInd()) );
    
    // delete the countries
    sb.append( CountryWriter.getDeleteSQL( "oadraft.answerCountry",
                                           "answerInd",
                                            ans.getInd() ) );

    // delete the conditions
    sb.append( ConditionWriter.getDeleteSQL( "oadraft.answerCond",
                                             "parentAnswerInd",
                                             ans.getInd() ) );

    sb.append("DELETE FROM oadraft.answer WHERE answerInd=")
      .append( ans.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete ALL Answers to a Question.
  **/
  static String getDeleteAllForQuestionSQL(int questionInd) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // Delete the Conclusions for all Answers to the Question
    sb.append( AnswerConclDraftWriter.getDeleteAllForQuestionSQL(questionInd) );
		
    // Delete the Titles for all Answers to the Question
    sb.append( AnswerTitleDraftWriter.getDeleteAllForQuestionSQL(questionInd) );
		
    // Delete all the Answers
    sb.append(
      "DELETE from oadraft.answercond where parentanswerind in " +
        "(select distinct answerind from oadraft.answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oadraft.answercountry where answerind in " +
        "(select distinct answerind from oadraft.answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oadraft.answer where questionind="
    ).append( questionInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
