
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.*;



/**
 * Reader for Online Assistant Answers.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.11
 * @author VishnuKumar.E
**/ 
class AnswerReader {

  /**
   * Read and populate the Answer objects for the specified Question.
   * @param conn  the database Connection object
   * @param a     the Question whose Answers are to be read
   * @param lang  the language to read the sub-objects for (0 = all languages)
  **/
  static void readAnswersForQuestion(Connection conn, Question q, 
                                     int languageInd) 
              throws SQLException 
  {

    Statement stmt = conn.createStatement();
    ResultSet rs;

    rs = stmt.executeQuery( "SELECT " + 
                              "answerInd, sortOrder, docClassInd, " +
                              "created, dbUser, changedTime " +
                            "FROM oa.answer " + 
                            "WHERE questionInd=" + q.getInd() + " " +
                            "ORDER BY sortOrder " +
                            "FOR FETCH ONLY");
  
    //-----------------------------------------------------------------------
    // Read all the Answers for the Question
    //-----------------------------------------------------------------------
    Vector v = new Vector();
    while( rs.next() ) {
      Answer ans = new Answer( rs.getInt(1) );
      ans.setParentInd( q.getInd() );
      ans.setDraft(false);

      ans.setSortOrder( rs.getShort(2) );
      ans.setDocClass( new TypeDocClassRec( rs.getShort(3), "" ) );
      ans.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(4) ) );
      ans.setDbUser( rs.getString(5).trim() );
      ans.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(6) ) );

      //---------------------------------------------------------------------
      // Read the Answer conditions/countries
      //---------------------------------------------------------------------
      // MAD performance: answer conditions are now read all at once below
      //ans.setConditions(
      //   ConditionReader.readConditions( conn, "oa.answerCond",
      //                                   "parentAnswerInd", ans )
      //);
      // MAD performance: answer countries are now read all at once below
      //ans.setCountryList(
      //   CountryReader.readCountries( conn, "oa.answerCountry",
      //                                "answerInd", ans.getInd() )
      //);

      //---------------------------------------------------------------------
      // Read the Answer Titles based on the language passed in
      //---------------------------------------------------------------------
      // MAD performance: answer titles are now read all at once below
      //if (languageInd == 0) {
      //  // read for all languages
      //  TitleReader.readAnswerTitles( conn, ans );
      //} else {
      //  // read for specific language
      //  TitleReader.readAnswerTitles( conn, ans, languageInd );
      //}

      //---------------------------------------------------------------------
      // Read the Answer Conclusions
      //---------------------------------------------------------------------
      // MAD performance: answer conclusions are now read all at once below
      //AnswerConclReader.readConclForAnswer( conn, ans );

      ans.updateRecStatus(PersistentRec.REC_NOCHANGE);
      v.addElement(ans);
    }  
    
    rs.close();
    stmt.close();

    q.setAnswers(v);

    //-----------------------------------------------------------------------
    // Read the Conditions/Countries for all Answers
    //-----------------------------------------------------------------------
    ConditionReader.readAllAnswerConditions(conn, q, "oa");
    CountryReader.readAllAnswerCountries(conn, q, "oa");

    //---------------------------------------------------------------------
    // Read all Answer Titles based on the language passed in
    //---------------------------------------------------------------------
    TitleReader.readAllAnswerTitles( conn, q, languageInd );

    //-----------------------------------------------------------------------
    // Read all Answer Conclusions
    //-----------------------------------------------------------------------
    AnswerConclReader.readAllAnswerConcl( conn, q );
  }
}
