
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.util.Vector;

/**
 * @version 1.9
 * @author  Mark DeBiase
**/
class AnswerTitleWriter {

  private static final String SCHEMA = "oa";

  static String getSQL(Title t) {
    String sql;

    switch ( t.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(t, SCHEMA);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(t, SCHEMA);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(t, SCHEMA);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(Title t, String schema) {
    // Note: created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO " + schema + ".answerTitle " +
         "(answerTitleInd, answerInd, sortOrder, title, languageInd, " +
         "lastTranslated, docClassInd, dbUser, changedTime) "+
       "VALUES ("
    );

    // answerTitleInd:
    sb.append( t.getInd()                                     ).append(',');

    // answerInd:
    sb.append( t.getParentInd()                               ).append(',');

    // sortOrder:
    sb.append( t.getSortOrder()                               ).append(',');

    // title:
    sb.append( DatabaseUtil.stringToDB( t.getTitle() )        ).append(',');

    // languageInd:
    sb.append( t.getLanguage().getInd()                       ).append(',');

    // lastTranslated:
    sb.append( 
      DatabaseUtil.timestampToDB(t.getLastTranslationDate() ) ).append(',');

    // docClassInd:
    sb.append( t.getDocClass().getInd()                       ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( t.getDbUser() )       ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // only insert countries/conditions if language is English
    if ( t.getLanguage().getInd() == 1 ) {

      // insert the countries 
      sb.append( CountryWriter.getInsertSQL( schema + ".answerTitleCountry",
                                             t.getInd(),
                                             t.getCountryList(),
                                             t.getDbUser() ) );

      // insert the conditions 
      sb.append( ConditionWriter.getInsertSQL( schema + ".answerTitleCond",
                                               t.getInd(), 
                                               t.getConditions(), 
                                               t.getDbUser() ) );
    }

    return sb.toString();
  }

  static String getUpdateSQL(Title t, String schema) { 
    // Note: created is not updated

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    //sb.append ( "DELETE FROM "+schema+".answerTitleCond WHERE answerTitleInd = "+t.getInd() ) ;
    //sb.append ( SqlRunner.END_DELIM ) ;
    //sb.append ( "DELETE FROM "+schema+".answerTitleCountry WHERE answerTitleInd = "+t.getInd() ) ;
    //sb.append ( SqlRunner.END_DELIM ) ;
    //sb.append ( "DELETE FROM "+schema+".answerTitle WHERE answerTitleInd = "+t.getInd() ) ;
    //sb.append ( SqlRunner.END_DELIM ) ;
    //sb.append ( getInsertSQL ( t, schema ) ) ;

    sb.append("UPDATE " + schema + ".answerTitle SET");

    sb.append( " sortOrder=" )
      .append( t.getSortOrder() );

    sb.append( ",title=" )
      .append( DatabaseUtil.stringToDB( t.getTitle() ) );

    sb.append( ",languageInd=" )
      .append( t.getLanguage().getInd() );

    sb.append( ",lastTranslated=" )
      .append( DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) );

    sb.append( ",docClassInd=" )
      .append( t.getDocClass().getInd() );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( t.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE answerTitleInd=").append( t.getInd() )
      .append(" AND languageInd=").append( t.getLanguage().getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // only update countries/conditions if language is English
    if ( t.getLanguage().getInd() == 1 ) {

      // update the country list (updated via delete/insert)
      sb.append( CountryWriter.getUpdateSQL( schema + ".answerTitleCountry",
                                             "answerTitleInd",
                                              t.getInd(),
                                              t.getCountryList(),
                                              t.getOriginalCountryList(),
                                              t.getDbUser() ) );

      // update the conditions (updated via delete/insert)
      sb.append( ConditionWriter.getUpdateSQL( schema + ".answerTitleCond",
                                               "answerTitleInd",
                                               t.getInd(),
                                               t.getConditions(),
                                               t.getDbUser() ) );
    }

    return sb.toString();
  }

  static String getDeleteSQL(Title t, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete the countries
    sb.append( CountryWriter.getDeleteSQL( schema + ".answerTitleCountry",
                                           "answerTitleInd",
                                            t.getInd() ) );

    // delete the conditions
    sb.append( ConditionWriter.getDeleteSQL( schema + ".answerTitleCond",
                                             "answerTitleInd",
                                             t.getInd() ) );

    sb.append("DELETE FROM " + schema + ".answerTitle WHERE answerTitleInd=")
      .append( t.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the Titles for ALL Answers to a Question.
  **/
  static String getDeleteAllForQuestionSQL(int questionInd) {
    return getDeleteAllForQuestionSQL(questionInd, SCHEMA);
  }

  static String getDeleteAllForQuestionSQL(int questionInd, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "DELETE from " + schema + ".answertitlecond where answertitleind in " +
        "(select distinct answertitleind " +
         "from " + schema + ".answertitle t, " + schema + ".answer a " +
           "where t.answerind = a.answerind " +
           "and   a.questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from " + schema + ".answertitlecountry where answertitleind in " +
        "(select distinct answertitleind " +
         "from " + schema + ".answertitle t, " + schema + ".answer a " +
           "where t.answerind = a.answerind " +
           "and   a.questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from " + schema + ".answertitle where answerind in " +
        "(select distinct answerind " +
         "from " + schema + ".answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the Titles for a single Answer.
  **/
  static String getDeleteAllForAnswerSQL(int answerInd) {
    return getDeleteAllForAnswerSQL(answerInd, SCHEMA);
  }

  static String getDeleteAllForAnswerSQL(int answerInd, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "DELETE from " + schema + ".answertitlecond where answertitleind in " +
        "(select distinct answertitleind " +
         "from " + schema + ".answertitle where answerind="
    ).append( answerInd ).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from " + schema + ".answertitlecountry where answertitleind in " +
        "(select distinct answertitleind " +
         "from " + schema + ".answertitle where answerind="
    ).append( answerInd ).append(')').append(SqlRunner.END_DELIM);

    sb.append(
      "DELETE from " + schema + ".answertitle where answerind="
    ).append( answerInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
