
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import java.util.Vector;

/**
 * @version 1.14
 * @author  Mark DeBiase
**/
class AnswerWriter {

  /**
   * Return the SQL for this Answer.
  **/
  static String getSQL(Answer a) {

    StringBuffer sb = new StringBuffer();

    sb.append( getAnswerSQL(a) );
    if ( a.valid() ) {
      // insert/update the sub-objects as necessary
      sb.append( getObjectSQL(a) );
    }

    if ( !a.modified() && sb.length() > 0 ) {
      // Answer wasn't modified but sub-objects were - update timestamp
      sb.append("UPDATE oa.answer " +
                "SET changedTime=CURRENT TIMESTAMP " +
                "WHERE answerInd=")
        .append( a.getInd() ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save the Answers's 
   * Title and AnswerConcl objects to the datbase.
   * 
   * @param Answer        the Answer to be saved
  **/
  static String getObjectSQL(Answer answer) {

    StringBuffer sb = new StringBuffer();
    Vector v;
 
    v = answer.getTitles();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        Title t = (Title) v.elementAt(i);
        if ( t.valid() ) { 
          t.setSortOrder( sortOrder++ ); 
          t.setParentInd( answer.getInd() );
          t.setDbUser( answer.getDbUser() );
        }
        sb.append( AnswerTitleWriter.getSQL(t) );
      }
    }

    v = answer.getConclusions();
    if (v != null) {
      // there is no sort order for conclusions
      for (int i=0; i<v.size(); i++) {
        AnswerConcl a = (AnswerConcl) v.elementAt(i);
        if ( a.valid() ) { 
          a.setFromAnswerInd( answer.getInd() );
          a.setDbUser( answer.getDbUser() );
        }
        sb.append( AnswerConclWriter.getSQL(a) );
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the Answer to the database.
  **/
  static String getAnswerSQL(Answer a) {
    String sql;

    switch ( a.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(a);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(a);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(a);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(Answer a) {
    // Note: oa.answer.created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.answer " +
         "(answerInd, questionInd, sortOrder, docClassInd, " +
         "dbUser, changedTime) " +
       "VALUES ("
    );
    sb.append( a.getInd()                                  ).append(',');
    sb.append( a.getParentInd()                            ).append(',');
    sb.append( a.getSortOrder()                            ).append(',');
    sb.append( a.getDocClass().getInd()                    ).append(',');
    sb.append( DatabaseUtil.stringToDB( a.getDbUser() )    ).append(',');
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    // insert the countries into oa.answerCountry
    sb.append( CountryWriter.getInsertSQL( "oa.answerCountry",
                                           a.getInd(),
                                           a.getCountryList(),
                                           a.getDbUser() ) );

    // insert the conditions into oa.answerCond
    sb.append( ConditionWriter.getInsertSQL("oa.answerCond",
                                            a.getInd(), 
                                            a.getConditions(), 
                                            a.getDbUser() ) );

    return sb.toString();
  }

  static String getUpdateSQL(Answer a) {
    // Note: oa.answer.questionInd is not updated

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oa.answer SET");
    sb.append( " sortOrder=" )
      .append( a.getSortOrder() );
    sb.append( ",docClassInd=" )
      .append( a.getDocClass().getInd() );
    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( a.getDbUser() ) );
    sb.append( ",changedTime=CURRENT TIMESTAMP");
    sb.append(" WHERE answerInd=").append( a.getInd() )
      .append(SqlRunner.END_DELIM);

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oa.answerCountry",
                                           "answerInd",
                                            a.getInd(),
                                            a.getCountryList(),
                                            a.getOriginalCountryList(),
                                            a.getDbUser() ) );

    // update the conditions (updated via delete/insert)
    sb.append( ConditionWriter.getUpdateSQL("oa.answerCond",
                                            "parentAnswerInd",
                                            a.getInd(),
                                            a.getConditions(),
                                            a.getDbUser() ) );

    return sb.toString();
  }

  static String getDeleteSQL(Answer a) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete the answer conclusions
    sb.append( AnswerConclWriter.getDeleteAllForAnswerSQL( a.getInd() ) );

    // delete the answer titles
    sb.append( AnswerTitleWriter.getDeleteAllForAnswerSQL( a.getInd() ) );

    // delete the countries
    sb.append( CountryWriter.getDeleteSQL( "oa.answerCountry",
                                           "answerInd",
                                            a.getInd() ) );

    // delete the conditions
    sb.append( ConditionWriter.getDeleteSQL( "oa.answerCond",
                                             "parentAnswerInd",
                                             a.getInd() ) );

    sb.append("DELETE FROM oa.answer WHERE answerInd=")
      .append( a.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete ALL Answers to a Question.
  **/
  static String getDeleteAllForQuestionSQL(int questionInd) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // Delete the Conclusions for all Answers to the Question
    sb.append( AnswerConclWriter.getDeleteAllForQuestionSQL( questionInd ) );

    // Delete the Titles for all Answers to the Question
    sb.append( AnswerTitleWriter.getDeleteAllForQuestionSQL( questionInd ) );

    // Delete all the Answers
    sb.append(
      "DELETE from oa.answercond where parentanswerind in " +
        "(select distinct answerind from oa.answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oa.answercountry where answerind in " +
        "(select distinct answerind from oa.answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oa.answer where questionind="
    ).append( questionInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete the Conditions/Contries, Titles and Conclusions
   * for alll Answers to a Question.
   * Used when publishing a Question.
  **/
  static String getDeleteAnswerDataForQuestionSQL(int questionInd) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // Delete the Conclusions for all Answers to the Question
    sb.append( AnswerConclWriter.getDeleteAllForQuestionSQL( questionInd ) );

    // Delete the Titles for all Answers to the Question
    sb.append( AnswerTitleWriter.getDeleteAllForQuestionSQL( questionInd ) );

    // Delete all the Answers
    sb.append(
      "DELETE from oa.answercond where parentanswerind in " +
        "(select distinct answerind from oa.answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oa.answercountry where answerind in " +
        "(select distinct answerind from oa.answer where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
