
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import java.util.Vector;

/**
 * Writer for Online Assistant conditions.
 *
 * @version 1.6
 * @author  Mark DeBiase
**/
class ConditionWriter {

  static String getInsertSQL(String tableName, int parentInd, 
                             Conditions conditions, String dbUser)
  {
    String insertSql = "";

    if (conditions != null && conditions.size() > 0) {
      StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

      Vector v = conditions.getTerms();
      for (int i=0; i<v.size(); i++) {
        ConditionTerm term = (ConditionTerm) v.elementAt(i);

        //*****************************************************************
        // Note: this code assumes the following column order on the 
        // condition table:
        //   parentInd, condSortOrder, questionInd, answerInd, operators, 
        //   dbuser, changedtime
        //*****************************************************************
        sb.append("INSERT INTO ").append(tableName)
          .append(" VALUES (");
        sb.append( parentInd                                      ).append(',');
        sb.append( (i + 1)                                        ).append(',');
        sb.append( term.getQuestionInd()                          ).append(',');
        // if answerInd is 0, then term applies to all answers - insert null
        if (term.getAnswerInd() == 0) {
          sb.append( "null,");
        } else {
          sb.append( term.getAnswerInd()                          ).append(',');
        }
        sb.append( DatabaseUtil.stringToDB( term.getOperators() ) ).append(',');
        sb.append( DatabaseUtil.stringToDB( dbUser )              ).append(',');
        sb.append("CURRENT TIMESTAMP)")
          .append(SqlRunner.END_DELIM);
      }
      insertSql = sb.toString();
    }

    return insertSql;
  }

  static String getUpdateSQL(String tableName, String columnName, int parentInd,
                             Conditions conditions, String dbUser) 
  { 
    // only update the conditions if they have been modified
    if (conditions.isModified()) {
      // use delete/insert to update conditions...
      return ConditionWriter.getDeleteSQL(tableName, columnName, parentInd)  +
             ConditionWriter.getInsertSQL(tableName, parentInd,
                                          conditions, dbUser);
    } else {
      return "";
    }
  }

  static String getDeleteSQL(String tableName, String columnName, int parentInd)
  {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("DELETE FROM ").append(tableName) 
      .append(" WHERE ").append(columnName).append('=').append(parentInd)
      .append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
