
package com.ibm.nzna.projects.common.quest.oa;

import java.sql.*;

/**
 * Miscellaneous database functions placed here until their proper
 * home or equivalent in quest is found.
 *
 * @version 1.4
 * @author  Mark DeBiase
**/
public class DatabaseUtil {

  static final int DEFAULT_SB_SIZE     = 250;
  static final int DEFAULT_VECTOR_SIZE = 5;

  /**
   * Add single quotes around a String to be written to the database.
   * Also double-up ('') any single quotes found within the string.
  **/
  public static String stringToDB(String s) {

    java.util.StringTokenizer st = new java.util.StringTokenizer(s, "'", true);

    // this length is a bit of an over-estimate, but close enough
    int sbLen = s.length() + 2 + st.countTokens();

    StringBuffer sb = new StringBuffer(sbLen);

    sb.append('\'');
    while (st.hasMoreTokens()) {
      String tok = st.nextToken();
      if (tok.equals("'")) {
        sb.append("''");
      }
      else {
        sb.append(tok);
      }
    }
    sb.append('\'');

    return sb.toString();
  }

  /**
   * Convert a boolean true/false value to a String Y/N value enclosed
   * in single quotes to be written to the database.
  **/
  public static String booleanToDB(boolean b) {
    return ( (b == true) ? "'Y'" : "'N'" );
  }

  /**
   * Convert a String Y/N value read from the database to a boolean
   * true/false value.
  **/
  public static boolean booleanFromDB(String s) {
    return ( (s.equals("Y")) ? true : false );
  }

  /**
   * Remove the microseconds from the end of a timestamp String read 
   * from the database.
  **/
  public static String timestampFromDB(String timestamp) {
    String retValue;

    if (timestamp == null) {
      retValue = "";
    } else {
      retValue = timestamp.substring(0, timestamp.lastIndexOf('.'));
    }

    return retValue;
  }

  /**
   * Format a timestamp String for the database. Return a database NULL
   * value if the timestamp String is null or empty; otherwise return 
   * the timestamp String enclosed in single quotes.
  **/
  public static String timestampToDB(String timestamp) {
    String retValue;

    if (timestamp == null || timestamp.equals("")) {
      retValue = "NULL";
    } else {
      retValue = "'" + timestamp + "'";
    }

    return retValue;
  }
}
