
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant DocLink objects.
 * This class is used by the OA stored procedures.
 * 
 *  @version 1.6
 *  @author Vishnukumar. E
**/

class DocLinkReader {

  private static final String ACTIVE_SCHEMA = "oa";
  private static final String DRAFT_SCHEMA  = "oadraft";

//=============================================================================

  /**
   *  Read Action DocLinks (oa schema).
  **/
  static void readActionDocLinks(Connection conn, Action a) throws SQLException
  {
    readActionDocLinks(conn, a, ACTIVE_SCHEMA);
  }

  /**
   *  Read Action DocLinks (oadraft schema).
  **/
  static void readActionDraftDocLinks(Connection conn, Action a) 
         throws SQLException
  {
    readActionDocLinks(conn, a, DRAFT_SCHEMA);
  }

  /**
   *  Read Action DocLinks (common function).
  **/
  private static void readActionDocLinks(Connection conn, Action a, 
                                         String schema) 
          throws SQLException
  {

    Statement stmt = conn.createStatement();
    ResultSet rs;

    rs = stmt.executeQuery(
           "SELECT " +
             "actionLinkInd, sortOrder, linkInd, linkGroupInd, " +
             "docClassInd, created, dbUser, changedTime " +
           "FROM " + schema + ".actionLink " +
           "WHERE actionInd=" + a.getInd() + " " +
           "ORDER BY sortOrder " +
           "FOR FETCH ONLY"
    );

    //populate the doclink
    Vector v = new Vector();
    while( rs.next() )
    {
      DocLink dl = new DocLink( rs.getInt(1) );
      dl.setParentInd( a.getInd() );

      dl.setSortOrder( rs.getShort(2) );
      dl.setLinkInd( rs.getInt(3) );
      dl.setLinkGroupInd( rs.getInt(4) );
      dl.setDocClass( new TypeDocClassRec( rs.getShort(5), "" ) );
      dl.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(6) ) );
      dl.setDbUser( rs.getString(7).trim() );
      dl.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(8) ) );

      // read the conditions/countries
      // MAD performance: doclink conditions are now read all at once below
      //dl.setConditions(
      //   ConditionReader.readConditions( conn, schema + ".actionLinkCond",
      //                                   "actionLinkInd", dl )
      //);
      // MAD performance: doclink countries are now read all at once below
      //dl.setCountryList(
      //   CountryReader.readCountries( conn, schema + ".actionLinkCountry",
      //                                "actionLinkInd", dl.getInd() )
      //);

      dl.updateRecStatus(dl.REC_NOCHANGE);
      v.addElement(dl);
    }

    rs.close();
    stmt.close();

    a.setDocumentLinks(v);

    // if we read any doclink objects, then get their condtions/countries
    // doclink is optional, so it is possible that there is none
    if (v.size() > 0) {
      ConditionReader.readAllActionDocLinkConditions(conn, a, schema);
      CountryReader.readAllActionDocLinkCountries(conn, a, schema);
    }
  }  

//=============================================================================

  /**
   *  Read Question DocLinks (oa schema).
  **/
  static void readQuestionDocLinks(Connection conn, Question q) 
              throws SQLException
  {
    readQuestionDocLinks(conn, q, ACTIVE_SCHEMA);
  }

  /**
   *  Read Question DocLinks (oadraft schema).
  **/
  static void readQuestionDraftDocLinks(Connection conn, Question q) 
              throws SQLException
  {
    readQuestionDocLinks(conn, q, DRAFT_SCHEMA);
  }

  /**
   *  Read Question DocLinks (common function).
  **/
  private static void readQuestionDocLinks(Connection conn, Question q,
                                           String schema) 
                 throws SQLException
  {

    Statement stmt = conn.createStatement();
    ResultSet rs;

    rs = stmt.executeQuery(
           "SELECT " + 
             "questionLinkInd, sortOrder, linkInd, linkGroupInd, " +
             "docClassInd, created, dbUser, changedTime " + 
           "FROM " + schema + ".questionLink " + 
           "WHERE questionInd=" + q.getInd() + " " +
           "ORDER BY sortOrder " +
           "FOR FETCH ONLY"
    );

    //populate the doclink
    Vector v = new Vector();
    while( rs.next() )
    {
      DocLink dl = new DocLink( rs.getInt(1) );
      dl.setParentInd( q.getInd() );

      dl.setSortOrder( rs.getShort(2) );
      dl.setLinkInd( rs.getInt(3) );
      dl.setLinkGroupInd( rs.getInt(4) );
      dl.setDocClass( new TypeDocClassRec( rs.getShort(5), "" ) );
      dl.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(6) ) );
      dl.setDbUser( rs.getString(7).trim() );
      dl.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(8) ) );

      // read the conditions/countries
      // MAD performance: doclink conditions are now read all at once below
      //dl.setConditions(
      //   ConditionReader.readConditions( conn, schema + ".questionLinkCond",
      //                                   "questionLinkInd", dl )
      //);
      // MAD performance: doclink countries are now read all at once below
      //dl.setCountryList(
      //   CountryReader.readCountries( conn, schema + ".questionLinkCntry",
      //                                "questionLinkInd", dl.getInd() )
      //);

      dl.updateRecStatus(dl.REC_NOCHANGE);
      v.addElement(dl);
    }
  
    rs.close();
    stmt.close();

    q.setDocLinks(v);

    // if we read any doclink objects, then get their condtions/countries
    // doclink is optional, so it is possible that there is none
    if (v.size() > 0) {
      ConditionReader.readAllQuestionDocLinkConditions(conn, q, schema);
      CountryReader.readAllQuestionDocLinkCountries(conn, q, schema);
    }
  }  

 }
