
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;

/**
 * Represents an Action to be added to the Action list.
 * All the Actions in the Action list are displayed when
 * an ALIST Node is encountered in a Symptom logic flow.
 *
 * @version 1.10
 * @author  Mark DeBiase
**/

public class EdgeAlist extends EdgeObject {

  private int   activeActionInd_ = 0;
  private int   draftActionInd_  = 0;

  /**
   * New EdgeAlist object.
   * @param a the Action to be added to the action list.
   * @exception IllegalArgumentException if the Action's indicator is 0.
  **/
  public EdgeAlist(Action a) {
    if (a instanceof ActionDraft) {
      ActionDraft ad   = (ActionDraft) a;
      activeActionInd_ = ad.getActiveActionInd();
      draftActionInd_  = ad.getInd();
    } else {
      activeActionInd_ = a.getInd();
    }

    if (activeActionInd_ == 0 && draftActionInd_ == 0) {
      throw new IllegalArgumentException("actionInd is 0");
    }

    updateRecStatus(REC_INSERT);
  }

  /**
   * New EdgeAlist object based on a <b><i>published</i></b> Action.
   * @param actionInd the indicator of the <i>published</i> Action 
                      to be addd to the action list.
   * @exception IllegalArgumentException if actionInd is 0.
   * @deprecated use EdgeAlist(Action) constructor instead
  **/
  public EdgeAlist(int actionInd) {
    super();
    activeActionInd_   = actionInd;
    if (activeActionInd_ == 0) {
      throw new IllegalArgumentException("actionInd is 0");
    }
    updateRecStatus(REC_INSERT);
  }

  /**
   * Constructor used by draft Reader.
  **/
  EdgeAlist(int activeActionInd, int draftActionInd) {
    activeActionInd_ = activeActionInd;
    draftActionInd_  = draftActionInd;
  }

  /**
   * Gets indicator of the <b>active</b> Action to be added to the action list.
   * Same as getActiveActionInd().
  **/
  public int getActionInd() {
    return activeActionInd_;
  }

  /**
   * Gets indicator of the published Action to be added to the action list.
   * Same as getActionInd().
  **/
  public int getActiveActionInd() {
    return activeActionInd_;
  }

  /**
   * Gets indicator of the draft Action to be added to the action list.
  **/
  public int getDraftActionInd() {
    return draftActionInd_;
  }

  /**
   * Sets indicator of the draft Action to be added to the action list.
  **/
  // For use by Avalon.
  public void setDraftActionInd(int draftActionInd) {
    draftActionInd_ = draftActionInd;
  }

  /**
   * Returns a String representation of this object.
  **/
  public String toString() {
    return "EdgeAlist " + getInd() + ": actionInd=" + activeActionInd_ +
                                     "/" +  draftActionInd_;
  }
}
