
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant EdgeGroups.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.4
 * @author  Mark DeBiase
**/


class EdgeGroupDraftReader {

  /**
   * Read SymptomDraft EdgeGrups.
  **/
  static void readEdgeGroupsForSymptom(Connection conn, Symptom s) 
              throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read all EdgeGroups
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "edgeGroupInd, headNodeInd, tailNodeInd, " + 
         "xPos, yPos, cxPos, cyPos, zOrder, dbUser, changedTime " +
       "FROM oadraft.edgeGroup " +
       "WHERE symptomInd=" + s.getInd() + " " +
       "ORDER BY zOrder " +
       "FOR FETCH ONLY"
    );

    Vector v = new Vector();
    while( rs.next() )
    {
      int edgeGroupInd = rs.getInt(1);
      Node headNode    = s.getNodeByInd( rs.getInt(2) );
      Node tailNode    = s.getNodeByInd( rs.getInt(3) );

      EdgeGroup eg = new EdgeGroup(edgeGroupInd, headNode, tailNode);
      eg.setGuiData( rs.getInt(4), rs.getInt(5),     // xPos, yPos
                     rs.getInt(6), rs.getInt(7),     // cxPos, cyPos
                     rs.getInt(8) );                 // zOrder

      eg.setDbUser( rs.getString(9).trim() );
      eg.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(10 ) ) );

      //---------------------------------------------------------------------
      // Read all the Edges in this EdgeGroup
      //---------------------------------------------------------------------
      // MAD performance: edges are now read all at once below
      //EdgeDraftReader.readEdgesForEdgeGroup( conn, eg, s );

      eg.updateRecStatus(eg.REC_NOCHANGE);
      v.addElement( eg );
    }
    
    rs.close();
    stmt.close();
  
    s.setEdgeGroups(v);

    //-----------------------------------------------------------------------
    // Read all the Edges in this Symptom
    //-----------------------------------------------------------------------
    EdgeDraftReader.readAllEdgesForSymptom(conn, s);
  }
}
