
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import java.util.Vector;

/**
 * @version 1.7
 * @author  Mark DeBiase
**/
class EdgeGroupWriter {

  /**
   * Return the SQL for this EdgeGroup and its contained objects.
  **/
  static String getSQL(EdgeGroup eg) {

    StringBuffer sb = new StringBuffer();


    sb.append( getEdgeGroupSQL(eg) );
    if ( eg.valid() ) {
      // insert/update the sub-objects as necessary
      sb.append( getObjectSQL(eg) );
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save the EdgeGroup's 
   * Edge objects to the datbase.
   * 
   * @param EdgeGroup     the EdgeGroup to be saved
  **/
  static String getObjectSQL(EdgeGroup group) {

    StringBuffer sb = new StringBuffer();
    Vector v;
 
    v = group.getEdges();
    if (v != null) {
      // there is no sort order for edges
      for (int i=0; i<v.size(); i++) {
        Edge e = (Edge) v.elementAt(i);
        if ( e.valid() ) { 
            e.setEdgeGroupInd( group.getInd() );
            e.setDbUser( group.getDbUser() );
          }
        sb.append( EdgeWriter.getSQL(e) );
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the EdgeGroup to the database.
  **/
  static String getEdgeGroupSQL(EdgeGroup e) {
    String sql;

    switch ( e.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(e);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(e);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(e);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  /**
   * Returns the SQL String to insert the EdgeGroup in the database.
  **/
  static String getInsertSQL(EdgeGroup e) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.edgeGroup " +
         "(edgeGroupInd, symptomInd, headNodeInd, tailNodeInd, " +
         "xPos, yPos, cxPos, cyPos, zOrder, " +
         "dbUser, changedTime) " +
      "VALUES ("
    );

    // edgeGroupInd:
    sb.append( e.getInd()                                  ).append(',');

    // symptomInd:
    sb.append( e.getSymptomInd()                           ).append(',');

    // headNodeInd:
    sb.append( e.getHeadNodeInd()                          ).append(',');
    if (e.getHeadNodeInd() == 0) {
      throw new IllegalStateException("Head Node indicator is 0 -- " +
                                      "the Node was probably not put in " +
                                      "the Symptom's Vector of Nodes");
    }

    // tailNodeInd:
    sb.append( e.getTailNodeInd()                          ).append(',');
    if (e.getTailNodeInd() == 0) {
      throw new IllegalStateException("Tail Node indicator is 0 -- " +
                                      "the Node was probably not put in " +
                                      "the Symptom's Vector of Nodes");
    }

    // xPos, yPos, cxPos, cyPos, zOrder:
    sb.append( e.getXPos()   ).append(',');
    sb.append( e.getYPos()   ).append(',');
    sb.append( e.getcXPos()  ).append(',');
    sb.append( e.getcYPos()  ).append(',');
    sb.append( e.getZOrder() ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( e.getDbUser() )    ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Returns the SQL String to update the EdgeGroup in the database.
   * Only the GUI data (xPos, yPos, cxPos, cxPos, zOrder) for
   * the EdgeGroup can be updated.
  **/
  static String getUpdateSQL(EdgeGroup e) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oa.edgeGroup SET");

    sb.append( " xPos=" )
      .append( e.getXPos() );

    sb.append( ",yPos=" )
      .append( e.getYPos() );

    sb.append( ",cxPos=" )
      .append( e.getcXPos() );

    sb.append( ",cyPos=" )
      .append( e.getcYPos() );

    sb.append( ",zOrder=" )
      .append( e.getZOrder() );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( e.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE edgeGroupInd=").append( e.getInd() )
      .append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Returns the SQL String to delete the EdgeGroup from the database.
  **/
  static String getDeleteSQL(EdgeGroup e) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append( EdgeWriter.getDeleteAllForEdgeGroupSQL( e.getInd() ) );

    sb.append("DELETE FROM oa.edgeGroup WHERE edgeGroupInd=")
      .append( e.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the EdgeGroups for a Symptom.
  **/
  static String getDeleteAllForSymptomSQL(int symptomInd) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append( EdgeWriter.getDeleteAllForSymptomSQL(symptomInd) );

    sb.append("DELETE from oa.edgegroup where symptomind=")
      .append( symptomInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
