
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.shared.util.*;
import java.sql.*;

/**
 * Base class for EdgePreanswer, EdgeAlist, EdgeUnanswer and EdgeReturn
 * objects.  Holds some common values and functions.
 * 
 * @version 1.10
 * @author  Mark DeBiase
**/

public abstract class EdgeObject extends PersistentRec {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object or its subclasses, YOU MUST UPDATE THIS NUMBER.    */
  /* Adding methods is fine as long as you know what you are   */
  /* doing!                                                    */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  private int indicator_ = 0;
  private int edgeInd_   = 0;

  // these value correspond to the preanswerType in the edgePreanswer table
  public static final short EDGE_PREANSWER = 0;
  public static final short EDGE_ALIST     = 1;
  public static final short EDGE_UNANSWER  = 2;
  public static final short EDGE_RETURN    = 3;

  /**
   * New EdgeObect.
  **/
  public EdgeObject() {
    indicator_ = 0;
    edgeInd_   = 0;
  }

  /**
   * Gets this object's indicator.
  **/
  public int getInd() {
    return indicator_;
  }

  /**
   * Sets this object's indicator.
  **/
  void setInd(int indicator) {
    indicator_ = indicator;
  }

  /**
   * Gets the parent Edge indicator.
  **/
  int getEdgeInd() {
    return edgeInd_;
  }

  /**
   * Sets the parent Edge indicator.
  **/
  void setEdgeInd(int edgeInd) {
    edgeInd_ = edgeInd;
  }

  /**
   * Assign a database key for this EdgeObject.
   * @return true if a database key was assigned, otherwise false
  **/
  boolean assignDatabaseKey(boolean isDraft) {
    boolean success = true;
    String counterName;

    if (isDraft) {
      counterName = "DEDGPREANS";
    } else {
      counterName = "EDGEPREANS";
    }

    if (indicator_ == 0 && valid()) {
      LogSystem.log(1, "Getting database key for EdgeObject: " + counterName);
      indicator_ = Counter.getCounter( LogSystem.getInstance(), counterName );
      if (indicator_ == 0) {
        LogSystem.log(2, "Failed to get database key for EdgeObject");
        success = false;
      }
    }

    return success;
  }
  // version to be called by publish stored procedure:
  void assignDatabaseKey(Connection conn, boolean isDraft) throws Exception {
    String counterName;

    if (isDraft) {
      counterName = "DEDGPREANS";
    } else {
      counterName = "EDGEPREANS";
    }

    if (indicator_ == 0) {
      indicator_ = Counter.getCounter( conn, counterName );
      if (indicator_ == 0) {
        throw new Exception("Failed to get database key for EdgeObject");
      }
    }

  }

  /**
   * Reset values in this object.
   * Sets the indicator to 0, sets the recStatus to REC_INSERT.
  **/
  public void resetNew() {
    setInd(0);
    setEdgeInd(0);
    updateRecStatus(REC_NOCHANGE) ;
    updateRecStatus(REC_INSERT);
  }
}

