
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;

/**
 * Represents a Question to be unanswered.
 * An EdgeUnanswer object represents a Question whose Answer should
 * be removed from the answer history if it was previously answered.
 * 
 * @version 1.10
 * @author  Mark DeBiase
**/

public class EdgeUnanswer extends EdgeObject {

  private int  activeQuestionInd_ = 0;
  private int  draftQuestionInd_  = 0;

  /**
   * New EdgeUnanswer object.
   * @param q the Question to be unanswered.
   * @exception IllegalArgumentExecption if the Question's indicator is 0.
  **/
  public EdgeUnanswer(Question q) {
    if (q instanceof QuestionDraft) {
      QuestionDraft qd   = (QuestionDraft) q;
      activeQuestionInd_ = qd.getActiveQuestionInd();
      draftQuestionInd_  = qd.getInd();
    } else {
      activeQuestionInd_ = q.getInd();
    }

    if (activeQuestionInd_ == 0 && draftQuestionInd_ == 0) {
      throw new IllegalArgumentException("questionInd is 0");
    }

    updateRecStatus(REC_INSERT);
  }

  /**
   * New EdgeUnanswer object based on a <b><i>published</i></b> Question.
   * @param questionInd the indicator of the <i>published</i>Question 
                        to be unanswered
   * @exception IllegalArgumentExecption if the questionInd is 0.
   * @deprecated use EdgeUnanswer(Question) constructor instead
  **/
  public EdgeUnanswer(int questionInd) {
    super();
    activeQuestionInd_ = questionInd;
    if (activeQuestionInd_ == 0) {
      throw new IllegalArgumentException("questionInd is 0");
    }
    updateRecStatus(REC_INSERT);
  }

  /**
   * Constructor used by draft Reader.
  **/
  EdgeUnanswer(int activeQuestionInd, int draftQuestionInd) {
    activeQuestionInd_ = activeQuestionInd;
    draftQuestionInd_  = draftQuestionInd;
  }

  /**
   * Gets indicator of the Question to be unanswered.
  **/
  public int getQuestionInd() {
    return activeQuestionInd_;
  }

  /**
   * Gets indicator of the published Question to be unanswered.
   * Same as getQuestionInd().
  **/
  public int getActiveQuestionInd() {
    return activeQuestionInd_;
  }

  /**
   * Gets indicator of the draft Question to be unanswered.
  **/
  public int getDraftQuestionInd() {
    return draftQuestionInd_;
  }

  /**
   * Sets indicator of the draft Question to be unanswered.
  **/
  // For use by Avalon.
  public void setDraftQuestionInd(int draftQuestionInd) {
    draftQuestionInd_ = draftQuestionInd;
  }


  /**
   * Returns a String representation of this object.
  **/
  public String toString() {
    return "EdgeUnanswer " + getInd() + ": questionInd=" + activeQuestionInd_ +
                                        "/" +  draftQuestionInd_;
  }
}

