
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.storedProc.oaReader.*;

/**
 * Online Assistant Document Link to an external URL.
 *
 * @see QuestLink
 * @see ExternalLink
 *
 * @version 1.13
 * @author  Mark DeBiase
**/
public class ExternalLink extends Link {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  protected String  url_      = null;
  protected boolean urlValid_ = false;

  /**
   * New ExternalLink object.
  **/
  public ExternalLink() { }

  /**
   * Create a new ExternalLink with the specified title.
   * The title contains the text to be displayed on the web -
   * used in the HTML anchor tag.
   * <p>
   * This constructor will create a LanguageText object with
   * a TypeLanguageRec of <b>English</b> for the title and add it
   * to the Vector of LanguageText objects in the Link object
   * superclass.
   * 
   * @see Link
   *
   * @param url   the link's url
   * @param title the link's title
  **/
  public ExternalLink(String url, String title) {
    super(title);
    url_ = url;
  }

  /**
   * ExternalLink to be read from the database.
   * @param linkInd the link indicator
  **/
  public ExternalLink(int linkInd) {
    super(linkInd);
  }

  /**
   * Initialize from an existing Link object.
  **/
  ExternalLink(ExternalLink el) {
    super(el);

    this.url_      = el.url_;
    this.urlValid_ = el.urlValid_;
  }

  /**
   * Initialize from an existing LinkDraft object.
  **/
  ExternalLink(ExternalLinkDraft eld) {
    super( (LinkDraft) eld );

    this.url_      = eld.url_;
    this.urlValid_ = eld.urlValid_;
  }

  /**
   * Get the external URL for this link.
  **/
  public String getUrl() {
    return url_;
  }

  /**
   * Set the external URL for this link.
   * Also sets the URL to invalid by calling setUrlValid(false).
  **/
  public void setUrl(String url) {
    url_ = url;
    setUrlValid(false);
    updateRecStatus(REC_UPDATE);
  }

  /**
   * Returns true if the URL is valid.
  **/
  public boolean isUrlValid() {
    return urlValid_;
  }

  /**
   * Sets the flag which indicates whether this URL is valid or not.
  **/
  public void setUrlValid(boolean urlValid) {
    urlValid_ = urlValid;
  }

  /**
   * Return the URL string for the ExternalLink.
   * The value returned is the same as getUrl().
  **/
  public String getUrlString() {
    return getUrl();
  }

}
