
package com.ibm.nzna.projects.common.quest.oa;

import java.util.Hashtable;

/**
 * Wrapper around java.util.Hashtable that allows a key to be put() in
 * the Hashtable with no value specified - a dummy object is used as
 * the value.
 * @see java.util.Hashtable
 *
 * @version 1.1
 * @author  Mark DeBiase
**/
public class HashList extends Hashtable {

  private static final Object NULL = new Object();

  /**
   * Put the specified object into the HashList.
   * @param key the object to add
  **/
  public void put(Object key) {
    super.put(key, NULL);
  }

  /**
   * Merge all the elements of the specified HashList into this HashList.
   * Duplicate objects will not be added.
   * @param h the HashList to be merged in
  **/
  public synchronized void merge(HashList h) {
    java.util.Enumeration e = h.keys();
    while (e.hasMoreElements()) {
      Object o = e.nextElement();
      this.put(o);
    }
  }

  /**
   * Return a String representation of this object.
   * Prints the keys only (values are all the same).
  **/
  public synchronized String toString() {
    StringBuffer sb = new StringBuffer( this.size() * 4 );
    int last  = this.size() - 1;
    int count = 0;

    sb.append('{');
    java.util.Enumeration e = this.keys();
    while (e.hasMoreElements()) {
      String key = (String) e.nextElement();
      sb.append(key);
      if (count < last) {
        sb.append(", ");
      }
      count++;
    }
    sb.append('}');

    return sb.toString();
  }

  // Reminders that these operations are not supported.
  public Object put(Object key, Object value) {
    throw new Error("put(): operation not supported");
  }

  public java.util.Enumeration elements() {
    throw new Error("elements(): operation not supported");
  }

  public boolean contains(Object value) {
    throw new Error("contains(): operation not supported");
  }
}
