
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.type.*;

/**
 * Generic object to hold text for a for a specific language.
 *
 * @version 1.7
 * @author  Mark DeBiase
**/

public class LanguageText extends PersistentRec {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;    

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   toStringTypes                                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  TEXT returns only the straight text
     **/
   public static final int TEXT         = 1 ;

   /**
     *  HTML returns regular HTML to show the text
     **/
   public static final int HTML         = 2 ;

   /**
     *  HTML_BOLD returns the text wrapped in HTML so that the text appears <b>bold</b>
     **/
   public static final int HTML_BOLD    = 3 ;

   /**
     *  HTML_TITLE returns the text wrapped in HTML so that the text is large
     **/
   public static final int HTML_TITLE    = 4 ;

   /**
     *  DIAG returns some diagnostic information along with the text
     **/
   public static final int DIAG           = 5 ;


   private String          text_           = null; // the text
   private TypeLanguageRec language_       = null; // the language
   private String          lastTranslated_ = null; // last translation date
   private String          changedTime_    = null; // date object was updated
   private String          dbuser_         = null; // last user to update this 
                                                   // object
   private int             toStringType    = TEXT ;

   /**
    * Create an empty LanguageText object.
   **/
   public LanguageText() { 
      updateRecStatus(REC_INSERT);
   }

   /**
    * Create a new LanguageText object with a TypeLanguageRec for English.
    * This constructor will use the TypeList class to retrieve the 
    * the appropriate TypeLanguageRec.  If the TypeList can not be
    * retrieved, a new TypeLanguageRec with indicator 1 and no description
    * will be created.
   **/
   public LanguageText(String text) { 
      text_ = text; 

      Object o = null;  // bypass the type system -- tempoarary
/*
    Object o = TypeList.getInstance().objectFromInd(1, TypeList.TYPE_LANGUAGE);
*/
      if ( o != null && o instanceof TypeLanguageRec ) {
         language_ = (TypeLanguageRec) o;
      } else {
         language_ = new TypeLanguageRec(1, "");
      }

      updateRecStatus(REC_INSERT);
   }

   /**
    * Create a new LanguageText object.
    * @param text  the text.
    * @param lang  the language.
   **/
   public LanguageText(String text, TypeLanguageRec lang) {
      text_     = text;
      language_ = lang;
      updateRecStatus(REC_INSERT);
   }

   /**
    * Gets the text value.
   **/
   public String getText() {
      return text_;
   }

   /**
    * Sets the text value.
   **/
   public void setText(String text) {
      text_ = text;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Gets the language.
    * @return TypeLanguageRec - the language
   **/
   public TypeLanguageRec getLanguage() {
      return language_;
   }

   /**
    * Sets the language.
    * @parm language the language
   **/
   public void setLanguage(TypeLanguageRec language) {
      language_ = language;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Get the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @return a String representing a DB2 timestamp
   **/
   public String getLastTranslationDate() {
      return lastTranslated_;
   }

   /**
    * Set the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @param translationDate a String representing a DB2 timestamp
   **/
   public void setLastTranslationDate(String translationDate) {
      lastTranslated_ = translationDate;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Gets the last changed date/time for this object.
   **/
   public String getChangedTime() {
      return changedTime_;
   }

   /**
    * Sets the last changed date/time for this object.
    * Called by Reader classes.
   **/
   void setChangedTime(String changedTime) {
      changedTime_ = changedTime;
   }

   /**
    * Gets the user who last changed this object.
   **/
   public String getDbUser() {
      return dbuser_;
   }

   /**
    * Sets the user who last changed this object.
   **/
   public void setDbUser(String dbuser) {
      dbuser_ = dbuser.toUpperCase();
   }

   /**
    * Update the TypeRec objects in the LanguageText object with 
    * the proper descriptions.
    * Called after a LanguageText object is read from the database.
   **/
   void updateTypeRecData() {
      TypeList t = TypeList.getInstance();

      // language
      TypeLanguageRec newLangRec;
      newLangRec = (TypeLanguageRec) t.objectFromInd( language_.getInd(),
                                                      TypeList.TYPE_LANGUAGE );
      if ( newLangRec != null ) {
         language_ = newLangRec;
      }

      return;
   }




   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    this method will return the text that is associated
    *  to this language text
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( toString ( toStringType ) ) ;
   }




   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return the output requested
    *  from the 'toString' method of the type toStringType
    *  
    *
    *  @param toStringType The type of output to get
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString ( int toStringType ) {
      String retStr = text_ ;


      switch ( toStringType ) {
         case DIAG :
            retStr = ""+language_+": "+text_ ;
            break;

         case HTML :
            retStr = "<html><body><table><tr><td><font = \"Helvetica, Arial\" size=2>"+text_+"</font></td></tr></table></body></html>" ;
            break ;

         case HTML_BOLD :
            retStr = "<html><body><font = \"Helvetica, Arial\" size=2><b>"+text_+"</b></font></body></html>" ;
            break ;

         case HTML_TITLE :
            retStr = "<html><body><font = \"Helvetica, Arial\" size=4>"+text_+"</font></body></html>" ;
            break ;

         case TEXT :
            break ;


      }

      return (retStr);
   }




   /*------------------------------------------------------------*/
   /* setToStringType
    */
   /**
    *    This method will set the type of output
    *  returned from the toString method
    *  
    *
    *  @param toStringType The type of toString (HTML, HTML_BOLD, HTML_TITLE and TEXT)
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setToStringType ( int toStringType ) {
      this.toStringType = toStringType ;
   }
}
