
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.util.Vector;

/**
 * @version 1.4
 * @author  Mark DeBiase
**/
class LinkDraftWriter {

  /**
   * Return the SQL for the draft Link.
  **/
  static String getSQL(LinkDraft linkDraft) {
    String sql;
    Link   link = linkDraft.asLink();

    switch ( link.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(linkDraft);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(linkDraft);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(linkDraft);
        break;

      default:
        sql = "";
    }

    // if the Link object hasn't been deleted, 
    // write the titles to the database
    if ( link.getRecStatus() != PersistentRec.REC_DELETE ) {
      sql += getLinkTitleSQL(link);
    }

    if ( !link.modified() && sql.length() > 0 ) {
      // Link wasn't modified but titles were - update timestamp
      sql += "UPDATE oadraft.link " +
             "SET changedTime=CURRENT TIMESTAMP " +
             "WHERE linkInd=" + link.getInd() + 
             SqlRunner.END_DELIM;
    }

    return sql;
  }

  static String getInsertSQL(LinkDraft ld) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);
    Link link = ld.asLink();

    sb.append(
      "INSERT INTO oadraft.link " +
         "(linkInd, activeLinkInd, linkType, docInd, url, urlValid, " +
         "typeWorkInd, comments, maintLock, startDate, stopDate, recycled, " +
         "archived, dbUser, created, changedTime) " +
       "VALUES ("
    );

    // linkInd:
    sb.append( link.getInd() ).append(',');

    // activeLinkInd:
    if ( ld.getActiveLinkInd() == 0 ) {
      sb.append("NULL,");
    } else {
      sb.append( ld.getActiveLinkInd() ).append(',');
    }

    // linkType, docInd, url, urlValid:
    if ( ld instanceof ExternalLink ) {
      ExternalLink e = (ExternalLink) ld;
      sb.append( Link.LINK_TYPE_EXTERNAL                    ).append(',');
      sb.append( "NULL"                                     ).append(',');
      sb.append( DatabaseUtil.stringToDB( e.getUrl() )      ).append(',');
      sb.append( DatabaseUtil.booleanToDB( e.isUrlValid() ) ).append(',');
    } else {
      QuestLink q = (QuestLink) ld;
      sb.append( Link.LINK_TYPE_QUEST ).append(',');
      sb.append( q.getDocInd()        ).append(',');
      sb.append( "NULL,NULL,"         );  // url and urlValid columns
    }

    // typeWorkInd:
    sb.append( ld.getTypeWorkRequired().getInd()           ).append(',');

    // comments:
    String comments = ld.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append("NULL,");
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) ).append(',');
    }

    // maintLock:
    sb.append( DatabaseUtil.booleanToDB( ld.getMaintLock() )  ).append(',');

    // startDate:
    sb.append( "CURRENT DATE"                                 ).append(',');

    // stopDate:
    sb.append( "NULL"                                         ).append(',');

    // recycled:
    sb.append( DatabaseUtil.booleanToDB( ld.isRecycled() )    ).append(',');

    // archived:
    sb.append( DatabaseUtil.booleanToDB( link.isArchived() )  ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( link.getDbUser() )    ).append(',');

    // created, changedTime:
    sb.append("CURRENT TIMESTAMP, CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  static String getUpdateSQL(LinkDraft ld) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);
    Link link = ld.asLink();

    // for both QuestLinkDraft and ExternalLinkDraft, the maintlock,
    // recycled and archived values can be updated.  for an
    // ExternalLinkDraft, the url can also be updated.

    sb.append("UPDATE oadraft.link SET");

    sb.append( " typeWorkInd=" )
      .append( ld.getTypeWorkRequired().getInd() );

    sb.append( ",comments=" );
    String comments = ld.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append( "NULL" );
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) );
    }

    sb.append( ",maintLock=" )
      .append( DatabaseUtil.booleanToDB( ld.getMaintLock() ) );

    sb.append( ",recycled=" )
      .append( DatabaseUtil.booleanToDB( ld.isRecycled() ) );  

    sb.append( ",archived=" )
      .append( DatabaseUtil.booleanToDB( link.isArchived() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( link.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    if (ld instanceof ExternalLink) {
      ExternalLink el = (ExternalLink) ld;
      sb.append( ",url=" )
        .append( DatabaseUtil.stringToDB( el.getUrl() ) );
      sb.append( ",urlValid=")
        .append( DatabaseUtil.booleanToDB( el.isUrlValid() ) );
    }

    sb.append(" WHERE linkInd=").append( link.getInd() );

    sb.append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  static String getDeleteSQL(LinkDraft ld) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);
    Link link = ld.asLink();

    sb.append("DELETE FROM oadraft.linkTitle WHERE linkInd=")
      .append( link.getInd() ).append(SqlRunner.END_DELIM);

    sb.append("DELETE FROM oadraft.link WHERE linkInd=")
      .append( link.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  static String getLinkTitleSQL(Link link) {
    String sql = "";

    Vector tv = link.getLinkTitles();
    for (int i=0; i<tv.size(); i++) {
      LanguageText title = (LanguageText) tv.elementAt(i);
      switch ( title.getRecStatus() ) {
        case title.REC_INSERT:
          sql += getLinkTitleInsertSQL( link, title );
          break;
        case title.REC_UPDATE:
          sql += getLinkTitleUpdateSQL( link, title );
          break;
        case title.REC_DELETE:
          sql += getLinkTitleDeleteSQL( link, title );
          break;
      }
    }
    return sql;
  }

  static String getLinkTitleInsertSQL(Link l, LanguageText t) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oadraft.linkTitle " +
      "(linkInd, languageInd, lastTranslated, title, dbUser, changedTime) " +
      "VALUES("
    );
    sb.append( l.getInd() ).append(',');
    sb.append( t.getLanguage().getInd() ).append(',');
    sb.append( DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) )
      .append(',');
    sb.append( DatabaseUtil.stringToDB( t.getText() ) ).append(',');
    sb.append( DatabaseUtil.stringToDB( l.getDbUser() ) ).append(',');
    sb.append( "CURRENT TIMESTAMP)" )
      .append( SqlRunner.END_DELIM );

    return sb.toString();
  }

  static String getLinkTitleUpdateSQL(Link l, LanguageText t) {

    return 
       "UPDATE oadraft.linkTitle SET" + 
          " title=" + DatabaseUtil.stringToDB( t.getText() ) +
          ",lastTranslated=" +
                   DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) +
          ",dbUser=" + DatabaseUtil.timestampToDB( l.getDbUser() ) +
          ",changedTime=CURRENT TIMESTAMP " +
       "WHERE linkInd=" + l.getInd() + 
       "  AND languageInd=" + t.getLanguage().getInd() +
       SqlRunner.END_DELIM;
  }

  static String getLinkTitleDeleteSQL(Link l, LanguageText t) {
    return "DELETE FROM oadraft.linkTitle " +
           "WHERE linkInd="     + l.getInd() +
           "  AND languageInd=" + t.getLanguage().getInd() +
           SqlRunner.END_DELIM;
  }
}
