package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.util.Vector;

/**
 * Online Assistant draft LinkGroup Writer.
 * 
 * @version 1.3
 * @author Vishnu Kumar. E
**/

class LinkGroupDraftWriter {

  static String getSQL(LinkGroupDraft lg) {
    String sql;

    switch ( lg.getRecStatus() ) {
      case lg.REC_INSERT:
        sql = getInsertSQL(lg);
	break;

      case lg.REC_UPDATE:
        sql = getUpdateSQL(lg);
	break;

      case lg.REC_DELETE:
        sql = getDeleteSQL(lg);
	break;

      default:
        sql = "";
    }

    if (lg.getRecStatus() != lg.REC_DELETE ) {
      sql += getLinkGroupTitleSQL(lg);
    }

    if ( !lg.modified() && sql.length() > 0 ) {
      // LinkGroup wasn't modified but titles were - update timestamp
      sql += "UPDATE oadraft.linkGroup " +
             "SET changedTime=CURRENT TIMESTAMP " +
             "WHERE linkGroupInd=" + lg.getInd() + 
             SqlRunner.END_DELIM;
    }

    return sql;
  }

  //=========================================================================
  // Methods for oadraft.linkGroup table (insert/update/delete):
  //=========================================================================

  static String getInsertSQL(LinkGroupDraft lg) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oadraft.linkGroup " +
         "(linkGroupInd, activeLinkGroupInd,  " +
         "typeWorkInd, comments, maintLock, startDate, stopDate, recycled, " +
	 "archived, dbUser, created, changedTime) " + 
      "VALUES ("
    );

    // linkGroupInd:
    sb.append( lg.getInd() ).append(',');

    // activeLinkGroupInd:
    if (lg.getActiveLinkGroupInd() == 0) {
      sb.append( "NULL"                                    ).append(',');
    } else {
      sb.append( lg.getActiveLinkGroupInd()                ).append(',');
    }

    // typeWorkInd:
    sb.append( lg.getTypeWorkRequired().getInd()           ).append(',');

    // comments:
    String comments = lg.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append("NULL,");
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) ).append(',');
    }

    // maintLock:
    sb.append( DatabaseUtil.booleanToDB( lg.getMaintLock() ) ).append(','); 

    // startDate:
    sb.append( "CURRENT DATE"                               ).append(',');

    // stopDate:
    sb.append( "NULL"                                       ).append(',');
    
    // recycled:
    sb.append( DatabaseUtil.booleanToDB( lg.isRecycled() )  ).append(',');

    // archived:
    sb.append( DatabaseUtil.booleanToDB(lg.isArchived() )).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB(lg.getDbUser() ) ).append(',');

    // created, changedTime:
    sb.append( "CURRENT TIMESTAMP, CURRENT TIMESTAMP)" )
      .append(SqlRunner.END_DELIM);
     
    // insert the links
    sb.append( getLinkGroupLinksInsertSQL(lg) );
    
    return sb.toString();
  } 
    
  static String getUpdateSQL(LinkGroupDraft lg) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oadraft.linkGroup SET" );

    sb.append( " typeWorkInd=" )
      .append( lg.getTypeWorkRequired().getInd() );

    sb.append( ",comments=" );
    String comments = lg.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append( "NULL" );
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) );
    }

    sb.append( ",maintLock=" )
      .append( DatabaseUtil.booleanToDB( lg.getMaintLock() ) );

    sb.append( ",recycled=" )
      .append( DatabaseUtil.booleanToDB( lg.isRecycled() ) );  

    sb.append(",archived=")
      .append( DatabaseUtil.booleanToDB( lg.isArchived() ) );

    sb.append(",dbUser=")
      .append( DatabaseUtil.stringToDB( lg.getDbUser() ) );

    sb.append(",changedTime=CURRENT TIMESTAMP ");

    sb.append("WHERE linkGroupInd=").append( lg.getInd() )
      .append(SqlRunner.END_DELIM);
      
    // update the Links in the LinkGroup via delete/insert
    sb.append( getLinkGroupLinksDeleteSQL(lg) );
    sb.append( getLinkGroupLinksInsertSQL(lg) );

    return sb.toString();
  }
   
  static String getDeleteSQL(LinkGroupDraft lg) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete links
    sb.append( getLinkGroupLinksDeleteSQL(lg) );

    // delete titles
    sb.append( "DELETE FROM oadraft.linkGroupTitle WHERE linkGroupInd=")
      .append( lg.getInd() ).append(SqlRunner.END_DELIM);

    // delete link group
    sb.append("DELETE FROM oadraft.linkGroup WHERE linkgroupInd=")
      .append( lg.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  //=========================================================================
  // Methods for oadraft.linkGroupLinks table (insert/delete):
  //=========================================================================
       
  static String getLinkGroupLinksInsertSQL(LinkGroupDraft lg) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    Vector linkVec = lg.getLinks();
    for(int j=0; j<linkVec.size(); j++) {
      sb.append(
        "INSERT INTO oadraft.linkGroupLinks " +
           "(linkGroupInd, linkInd, sortOrder, dbUser, changedTime) " +
        "VALUES ("
      );

      Link currentLink = (Link) linkVec.elementAt(j);

      // linkGroupInd:
      sb.append( lg.getInd()                               ).append(',');

      // linkInd:
      sb.append( currentLink.getInd()                      ).append(',');

      // sortOrder:
      sb.append( (j+1)                                     ).append(',');

      // dbUser:
      sb.append( DatabaseUtil.stringToDB( lg.getDbUser() ) ).append(',');

      // changedTime:
      sb.append( "CURRENT TIMESTAMP)" )
        .append(SqlRunner.END_DELIM);
    }
	 
    return sb.toString();
          
  }
 
  static String getLinkGroupLinksDeleteSQL(LinkGroupDraft lg) {

    return "DELETE FROM oadraft.linkGroupLinks " + 
           "WHERE linkgroupInd=" + lg.getInd() +
           SqlRunner.END_DELIM;
  }    


  //=========================================================================
  // Methods for oadraft.linkGroupTitle table (insert/update/delete):
  //=========================================================================
    
  static String getLinkGroupTitleSQL(LinkGroupDraft lg) {
    String sql = "";

    Vector tv = lg.getLinkGroupTitles();
    for (int i=0; i<tv.size(); i++) {
      LanguageText title = (LanguageText) tv.elementAt(i);
      switch ( title.getRecStatus() ) {
        case title.REC_INSERT:
          sql += getLinkGroupTitleInsertSQL( lg, title );
          break;
        case title.REC_UPDATE:
          sql += getLinkGroupTitleUpdateSQL( lg, title );
          break;
        case title.REC_DELETE:
          sql += getLinkGroupTitleDeleteSQL( lg, title );
          break;
      }
    }

    return sql;
  }

  static String getLinkGroupTitleInsertSQL(LinkGroupDraft lg, LanguageText t) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
       "INSERT INTO oadraft.linkGroupTitle " +
         "(linkgroupInd, languageInd, lastTranslated, title, " + 
         "dbUser, changedTime) " +
       "VALUES("
    );

    sb.append( lg.getInd() ).append(',');
    sb.append( t.getLanguage().getInd() ).append(',');
    sb.append( DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) )
      .append(',');
    sb.append( DatabaseUtil.stringToDB( t.getText() ) ).append(',');
    sb.append( DatabaseUtil.stringToDB( lg.getDbUser() ) ).append(',');
    sb.append( "CURRENT TIMESTAMP)" )
      .append( SqlRunner.END_DELIM );

    return sb.toString();
  }

  static String getLinkGroupTitleUpdateSQL(LinkGroupDraft lg, LanguageText t) {

    return
      "UPDATE oadraft.linkGroupTitle SET" +
          " title=" + DatabaseUtil.stringToDB( t.getText() ) +
          ",lastTranslated=" +
                   DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) +
          ",dbUser=" + DatabaseUtil.timestampToDB( lg.getDbUser() ) +
          ",changedTime=CURRENT TIMESTAMP " +
       "WHERE linkGroupInd=" + lg.getInd() +
       "  AND languageInd="  + t.getLanguage().getInd() +
       SqlRunner.END_DELIM;
  }

  static String getLinkGroupTitleDeleteSQL(LinkGroupDraft lg, LanguageText t) {

    return "DELETE FROM oadraft.linkGroupTitle " +
           "WHERE linkGroupInd=" + lg.getInd() +
           "  AND languageInd="  + t.getLanguage().getInd() +
           SqlRunner.END_DELIM;
  }

}

       
      
   

    
