package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.sql.*;
import java.util.Vector;

/**
 * Online Assistant Link Group Writer.
 * 
 * @version 1.2
 * @author Vishnu Kumar. E
**/

public class LinkGroupWriter {

  static String getSQL(LinkGroup lg) {
    String sql;

    switch ( lg.getRecStatus() ) {
      case lg.REC_INSERT:
        sql = getInsertSQL(lg);
	break;

      case lg.REC_UPDATE:
        sql = getUpdateSQL(lg);
	break;

      case lg.REC_DELETE:
        sql = getDeleteSQL(lg);
	break;

      default:
        sql = "";
    }

    if (lg.getRecStatus() != lg.REC_DELETE ) {
      sql += getLinkGroupTitleSQL(lg);
    }

    if ( !lg.modified() && sql.length() > 0 ) {
      // LinkGroup wasn't modified but titles were - update timestamp
      sql += "UPDATE oa.linkGroup " +
             "SET changedTime=CURRENT TIMESTAMP " +
             "WHERE linkGroupInd=" + lg.getInd() + 
             SqlRunner.END_DELIM;
    }

    return sql;
  }

  //=========================================================================
  // Methods for oa.linkGroup table (insert/update/delete):
  //=========================================================================

  static String getInsertSQL(LinkGroup lg) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.linkGroup " +
         "(linkGroupInd, archived, dbUser, " +
	 "created, changedTime, openDraft) " + 
      "VALUES ("
    );

    // linkGroupInd:
    sb.append( lg.getInd() ).append(',');

    // archived:
    sb.append( DatabaseUtil.booleanToDB(lg.isArchived() )).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB(lg.getDbUser() ) ).append(',');

    // created, changedTime, openDraft:
    sb.append( "CURRENT TIMESTAMP, CURRENT TIMESTAMP, 'N')" )
      .append(SqlRunner.END_DELIM);
     
    // insert the links
    sb.append( getLinkGroupLinksInsertSQL(lg) );
    
    return sb.toString();
  } 
    
  static String getUpdateSQL(LinkGroup lg) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oa.linkGroup SET" );

    sb.append(" archived=")
      .append( DatabaseUtil.booleanToDB( lg.isArchived() ) );

    sb.append(",dbUser=")
      .append( DatabaseUtil.stringToDB( lg.getDbUser() ) );

    sb.append(",changedTime=CURRENT TIMESTAMP ");

    sb.append("WHERE linkGroupInd=").append( lg.getInd() )
      .append(SqlRunner.END_DELIM);
      
    // update the Links in the LinkGroup via delete/insert
    sb.append( getLinkGroupLinksDeleteSQL(lg) );
    sb.append( getLinkGroupLinksInsertSQL(lg) );

    return sb.toString();
  }
   
  static String getDeleteSQL(LinkGroup lg) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete links
    sb.append( getLinkGroupLinksDeleteSQL(lg) );

    // delete titles
    sb.append( "DELETE FROM oa.linkGroupTitle WHERE linkGroupInd=")
      .append( lg.getInd() ).append(SqlRunner.END_DELIM);

    // delete link group
    sb.append("DELETE FROM oa.linkGroup WHERE linkgroupInd=")
      .append( lg.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  //=========================================================================
  // Methods for oa.linkGroupLinks table (insert/delete):
  //=========================================================================
       
  static String getLinkGroupLinksInsertSQL(LinkGroup lg) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    Vector linkVec = lg.getLinks();
    for(int j=0; j<linkVec.size(); j++) {
      sb.append(
        "INSERT INTO oa.linkGroupLinks " +
           "(linkGroupInd, linkInd, sortOrder, dbUser, changedTime) " +
        "VALUES ("
      );

      Link currentLink = (Link) linkVec.elementAt(j);

      // linkGroupInd:
      sb.append( lg.getInd()                               ).append(',');

      // linkInd:
      sb.append( currentLink.getInd()                      ).append(',');

      // sortOrder:
      sb.append( (j+1)                                     ).append(',');

      // dbUser:
      sb.append( DatabaseUtil.stringToDB( lg.getDbUser() ) ).append(',');

      // changedTime:
      sb.append( "CURRENT TIMESTAMP)" )
        .append(SqlRunner.END_DELIM);
    }
	 
    return sb.toString();
          
  }
 
  static String getLinkGroupLinksDeleteSQL(LinkGroup lg) {

    return "DELETE FROM oa.linkGroupLinks " + 
           "WHERE linkgroupInd=" + lg.getInd() +
           SqlRunner.END_DELIM;
  }    


  //=========================================================================
  // Methods for oa.linkGroupTitle table (insert/update/delete):
  //=========================================================================
    
  static String getLinkGroupTitleSQL(LinkGroup lg) {
    String sql = "";

    Vector tv = lg.getLinkGroupTitles();
    for (int i=0; i<tv.size(); i++) {
      LanguageText title = (LanguageText) tv.elementAt(i);
      switch ( title.getRecStatus() ) {
        case title.REC_INSERT:
          sql += getLinkGroupTitleInsertSQL( lg, title );
          break;
        case title.REC_UPDATE:
          sql += getLinkGroupTitleUpdateSQL( lg, title );
          break;
        case title.REC_DELETE:
          sql += getLinkGroupTitleDeleteSQL( lg, title );
          break;
      }
    }

    return sql;
  }

  static String getLinkGroupTitleInsertSQL(LinkGroup lg, LanguageText t) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
       "INSERT INTO oa.linkGroupTitle " +
         "(linkgroupInd, languageInd, lastTranslated, title, " + 
         "dbUser, changedTime) " +
       "VALUES("
    );

    sb.append( lg.getInd() ).append(',');
    sb.append( t.getLanguage().getInd() ).append(',');
    sb.append( DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) )
      .append(',');
    sb.append( DatabaseUtil.stringToDB( t.getText() ) ).append(',');
    sb.append( DatabaseUtil.stringToDB( lg.getDbUser() ) ).append(',');
    sb.append( "CURRENT TIMESTAMP)" )
      .append( SqlRunner.END_DELIM );

    return sb.toString();
  }

  static String getLinkGroupTitleUpdateSQL(LinkGroup lg, LanguageText t) {

    return
      "UPDATE oa.linkGroupTitle SET" +
          " title=" + DatabaseUtil.stringToDB( t.getText() ) +
          ",lastTranslated=" +
                   DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) +
          ",dbUser=" + DatabaseUtil.timestampToDB( lg.getDbUser() ) +
          ",changedTime=CURRENT TIMESTAMP " +
       "WHERE linkGroupInd=" + lg.getInd() +
       "  AND languageInd="  + t.getLanguage().getInd() +
       SqlRunner.END_DELIM;
  }

  static String getLinkGroupTitleDeleteSQL(LinkGroup lg, LanguageText t) {

    return "DELETE FROM oa.linkGroupTitle " +
           "WHERE linkGroupInd=" + lg.getInd() +
           "  AND languageInd="  + t.getLanguage().getInd() +
           SqlRunner.END_DELIM;
  }

  /**
   * Return the SQL to publish a LinkGroup from an LinkGroupDraft object.
   * 
   * @return a String containing SQL statements separated by 
   *         SqlRunner.END_DELIM.
  **/
  public static String getPublishSQL(Connection conn, LinkGroupDraft draft)
                       throws Exception
  {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);


    // construct a new LinkGroup object from the draft object
    // the recstatus and indicators will be set appropriately
    LinkGroup active = new LinkGroup( draft );
    active.assignDatabaseKeys(conn);  // throws Exception on failures

    // if there is already an active object, then the Titles
    // associated with it must be wiped out 
    if (active.getRecStatus() == PersistentRec.REC_UPDATE) {
      sb.append(
        "DELETE FROM oa.linkGroupTitle " + 
        "WHERE linkGroupInd="
      ).append( active.getInd() ).append(SqlRunner.END_DELIM);
    }

    // generate the sql to to publish the linkgroup
    sb.append( getSQL(active) );

    // update the openDraft flag.  this is done seperately at the end
    // in case there are any failures publishing the draft.
    sb.append(
      "UPDATE oa.linkGroup " +
      "SET openDraft='N' " +
      "WHERE linkGroupInd="
    ).append( active.getInd() ).append(SqlRunner.END_DELIM);

    // finally, update the stopDate & activeLinkGroupInd on the draft object
    sb.append(
      "UPDATE oadraft.linkGroup " +
      "SET stopDate=CURRENT DATE, " +
          "changedTime=CURRENT TIMESTAMP, " +
          "activeLinkGroupInd=" + active.getInd() + " " +
      "WHERE linkGroupInd=" + draft.getInd() 
    ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

}

       
      
   

    
