
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.sql.*;
import java.util.Vector;

/**
 * @version 1.10
 * @author  Mark DeBiase
**/
public class LinkWriter {

  /**
   * Return the SQL for the Link.
  **/
  static String getSQL(Link l) {
    String sql;

    switch ( l.getRecStatus() ) {
      case l.REC_INSERT:
        sql = getInsertSQL(l);
        break;

      case l.REC_UPDATE:
        sql = getUpdateSQL(l);
        break;

      case l.REC_DELETE:
        sql = getDeleteSQL(l);
        break;

      default:
        sql = "";
    }

    // if the Link object hasn't been deleted, 
    // write the titles to the database
    if ( l.getRecStatus() != l.REC_DELETE ) {
      sql += getLinkTitleSQL(l);
    }

    if ( !l.modified() && sql.length() > 0 ) {
      // Link wasn't modified but titles were - update timestamp
      sql += "UPDATE oa.link " +
             "SET changedTime=CURRENT TIMESTAMP " +
             "WHERE linkInd=" + l.getInd() + 
             SqlRunner.END_DELIM;
    }

    return sql;
  }

  static String getInsertSQL(Link l) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.link " +
         "(linkInd, linkType, docInd, url, urlValid, " +
         "archived, dbUser, created, changedTime, openDraft) " +
       "VALUES ("
    );

    // linkInd:
    sb.append( l.getInd() ).append(',');

    // linkType, docInd, url, urlValid:
    if ( l instanceof ExternalLink ) {
      ExternalLink e = (ExternalLink) l;
      sb.append( l.LINK_TYPE_EXTERNAL                       ).append(',');
      sb.append( "NULL"                                     ).append(',');
      sb.append( DatabaseUtil.stringToDB( e.getUrl() )      ).append(',');
      sb.append( DatabaseUtil.booleanToDB( e.isUrlValid() ) ).append(',');
    } else {
      QuestLink q = (QuestLink) l;
      sb.append( l.LINK_TYPE_QUEST ).append(',');
      sb.append( q.getDocInd()     ).append(',');
      sb.append( "NULL,NULL,"      );  // url and urlValid columns
    }

    // archived:
    sb.append( DatabaseUtil.booleanToDB( l.isArchived() )  ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( l.getDbUser() )    ).append(',');

    // created, changedTime, openDraft:
    sb.append("CURRENT TIMESTAMP, CURRENT TIMESTAMP, 'N')")
      .append(SqlRunner.END_DELIM);

    // MAD 6/8:
    // if this is a QuestLink, then update the 'knowledgebase' flag 
    // in tigris.documents in quest.documents to 'Y'
    if ( l instanceof QuestLink) {
      QuestLink q = (QuestLink) l;

      sb.append(
        "UPDATE tigris.documents " +
        "SET knowledgebase='Y' " +
        "WHERE docInd=" + q.getDocInd()
      ).append(SqlRunner.END_DELIM);

      sb.append(
        "UPDATE quest.documents " +
        "SET knowledgebase='Y' " +
        "WHERE existingDocInd=" + q.getDocInd()
      ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  static String getUpdateSQL(Link l) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // for a QuestLink, only the archived flag can be updated.
    // for an ExternalLink, the archived flag and the url can 
    // be updated.

    sb.append("UPDATE oa.link SET");

    sb.append( " archived=" )
      .append( DatabaseUtil.booleanToDB( l.isArchived() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( l.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    if (l instanceof ExternalLink) {
      ExternalLink el = (ExternalLink) l;
      sb.append( ",url=" )
        .append( DatabaseUtil.stringToDB( el.getUrl() ) );
      sb.append( ",urlValid=")
        .append( DatabaseUtil.booleanToDB( el.isUrlValid() ) );
    }

    sb.append(" WHERE linkInd=").append( l.getInd() );

    sb.append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  static String getDeleteSQL(Link l) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("DELETE FROM oa.linkTitle WHERE linkInd=")
      .append( l.getInd() ).append(SqlRunner.END_DELIM);

    sb.append("DELETE FROM oa.link WHERE linkInd=")
      .append( l.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  static String getLinkTitleSQL(Link link) {
    String sql = "";

    Vector tv = link.getLinkTitles();
    for (int i=0; i<tv.size(); i++) {
      LanguageText title = (LanguageText) tv.elementAt(i);
      switch ( title.getRecStatus() ) {
        case title.REC_INSERT:
          sql += getLinkTitleInsertSQL( link, title );
          break;
        case title.REC_UPDATE:
          sql += getLinkTitleUpdateSQL( link, title );
          break;
        case title.REC_DELETE:
          sql += getLinkTitleDeleteSQL( link, title );
          break;
      }
    }
    return sql;
  }

  static String getLinkTitleInsertSQL(Link l, LanguageText t) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.linkTitle " +
      "(linkInd, languageInd, lastTranslated, title, dbUser, changedTime) " +
      "VALUES("
    );
    sb.append( l.getInd() ).append(',');
    sb.append( t.getLanguage().getInd() ).append(',');
    sb.append( DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) )
      .append(',');
    sb.append( DatabaseUtil.stringToDB( t.getText() ) ).append(',');
    sb.append( DatabaseUtil.stringToDB( l.getDbUser() ) ).append(',');
    sb.append( "CURRENT TIMESTAMP)" )
      .append( SqlRunner.END_DELIM );

    return sb.toString();
  }

  static String getLinkTitleUpdateSQL(Link l, LanguageText t) {

    return 
       "UPDATE oa.linkTitle SET" + 
          " title=" + DatabaseUtil.stringToDB( t.getText() ) +
          ",lastTranslated=" +
                   DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) +
          ",dbUser=" + DatabaseUtil.timestampToDB( l.getDbUser() ) +
          ",changedTime=CURRENT TIMESTAMP " +
       "WHERE linkInd=" + l.getInd() + 
       "  AND languageInd=" + t.getLanguage().getInd() +
       SqlRunner.END_DELIM;
  }

  static String getLinkTitleDeleteSQL(Link l, LanguageText t) {
    return "DELETE FROM oa.linkTitle " +
           "WHERE linkInd="     + l.getInd() +
           "  AND languageInd=" + t.getLanguage().getInd() +
           SqlRunner.END_DELIM;
  }

  /**
   * Return the SQL to publish a Link from a LinkDraft object.
   * 
   * @return a String containing SQL statements separated by 
   *         SqlRunner.END_DELIM.
  **/
  public static String getPublishSQL(Connection conn, LinkDraft draft)
                       throws Exception
  {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);


    // construct a new Link object from the draft object
    // the recstatus and indicators will be set appropriately
    Link active;
    if (draft instanceof QuestLinkDraft) {
      active = new QuestLink( (QuestLinkDraft) draft );
    } else {
      active = new ExternalLink( (ExternalLinkDraft) draft );
    }
    active.assignDatabaseKeys(conn);  // throws Exception on failures

    // if there is already an active object, then the Titles
    // associated with it must be wiped out 
    if (active.getRecStatus() == PersistentRec.REC_UPDATE) {
      sb.append(
        "DELETE FROM oa.linkTitle " + 
        "WHERE linkInd="
      ).append( active.getInd() ).append(SqlRunner.END_DELIM);
    }

    // generate the sql to to publish the link
    sb.append( getSQL(active) );

    // update the openDraft flag.  this is done seperately at the end
    // in case there are any failures publishing the draft.
    sb.append(
      "UPDATE oa.link " +
      "SET openDraft='N' " +
      "WHERE linkInd="
    ).append( active.getInd() ).append(SqlRunner.END_DELIM);

    // finally, update the stopDate & activeActionInd on the draft object
    sb.append(
      "UPDATE oadraft.link " +
      "SET stopDate=CURRENT DATE, " +
          "changedTime=CURRENT TIMESTAMP, " +
          "activeLinkInd=" + active.getInd() + " " +
      "WHERE linkInd=" + draft.asLink().getInd() 
    ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
