
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;

/**
 * Reader for Online Assistant Nodes.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.3
 * @author  Mark DeBiase
**/


class NodeDraftReader {

  /**
   * Read SymptomDraft Nodes.
  **/
  static void readNodesForSymptom(Connection conn, Symptom s) 
              throws SQLException
  {
    Statement  stmt = conn.createStatement();
    ResultSet  rs;

    //---------------------------------------------------------------------
    // Read all Nodes
    //---------------------------------------------------------------------
    rs = stmt.executeQuery( 
       "SELECT " +
         "nodeInd, nodeType, " +
         "activeActionInd, draftActionInd, " + 
         "activeQuestionInd, draftQuestionInd, " + 
         "activeSymptomInd, draftSymptomInd, " +
         "xPos, yPos, width, height, zOrder, " +
         "dbUser, changedTime " +
       "FROM oadraft.node " +
       "WHERE symptomInd=" + s.getInd() + " " +
       "ORDER BY zOrder " +
       "FOR FETCH ONLY"
    );

    Vector v = new Vector();
    while( rs.next() )
    {
      Node n = new Node();
      n.setInd( rs.getInt(1) );
      n.setSymptomInd( s.getInd() );

      n.setNodeType( rs.getShort(2) );

      switch ( n.getNodeType() ) {
        case Node.NODE_TYPE_ACTION:
          n.setActiveObjectInd( rs.getInt(3) );
          n.setDraftObjectInd( rs.getInt(4) );
          break;
        case Node.NODE_TYPE_QUESTION:
          n.setActiveObjectInd( rs.getInt(5) );
          n.setDraftObjectInd( rs.getInt(6) );
          break;
        case Node.NODE_TYPE_SYMPTOM:
          n.setActiveObjectInd( rs.getInt(7) );
          n.setDraftObjectInd( rs.getInt(8) );
          break;
        case Node.NODE_TYPE_RESOLVED:
        case Node.NODE_TYPE_UNRESOLVED:
        case Node.NODE_TYPE_RETURN:
        case Node.NODE_TYPE_ALIST:
          n.setOAObjectInd( 0 );
          break;
      }

      int x = rs.getInt(9);
      int y = rs.getInt(10);
      int w = rs.getInt(11);
      int h = rs.getInt(12);
      int z = rs.getInt(13);
      n.setGuiData(x, y, w, h, z);

      n.setDbUser( rs.getString(14).trim() );
      n.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(15) ) );


      n.updateRecStatus(n.REC_NOCHANGE);
      v.addElement(n);
    }
    
    rs.close();
    stmt.close();
  
    s.setNodes(v);
  }
}
