
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import java.util.Vector;

/**
 * @version 1.4
 * @author  Mark DeBiase
**/
class NodeWriter {

  /**
   * Returns the SQL String to write the Node to the database.
  **/
  static String getSQL(Node n) {
    String sql;

    switch ( n.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(n);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(n);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(n);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  /**
   * Returns the SQL String to insert the Node in the database.
  **/
  static String getInsertSQL(Node n) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.node " +
         "(nodeInd, symptomInd, nodeType, actionInd, questionInd, " +
         "childSymptomInd, xPos, yPos, width, height, zOrder, " +
         "dbUser, changedTime) " +
      "VALUES ("
    );

    // nodeInd:
    sb.append( n.getInd()                                  ).append(',');

    // symptomInd:
    sb.append( n.getSymptomInd()                           ).append(',');

    // nodeType:
    short nodeType = n.getNodeType();
    sb.append( nodeType                                    ).append(',');

    // actionInd:
    if ( nodeType == Node.NODE_TYPE_ACTION ) {
      sb.append( n.getOAObjectInd()                        ).append(',');
    } else {
      sb.append( "NULL," );
    }

    // questionInd:
    if ( nodeType == Node.NODE_TYPE_QUESTION ) {
      sb.append( n.getOAObjectInd()                        ).append(',');
    } else {
      sb.append( "NULL," );
    }

    // childSymptomInd:
    if ( nodeType == Node.NODE_TYPE_SYMPTOM ) {
      sb.append( n.getOAObjectInd()                        ).append(',');
    } else {
      sb.append( "NULL," );
    }

    // xPos, yPos, width, height, zOrder:
    sb.append( n.getXPos()   ).append(',');
    sb.append( n.getYPos()   ).append(',');
    sb.append( n.getWidth()  ).append(',');
    sb.append( n.getHeight() ).append(',');
    sb.append( n.getZOrder() ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( n.getDbUser() )    ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Returns the SQL String to update the Node in the database.
   * Only the GUI data (xPos, yPos, width, height, zOrder) for
   * the Node can be updated.
  **/
  static String getUpdateSQL(Node n) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oa.node SET");

    sb.append( " xPos=" )
      .append( n.getXPos() );

    sb.append( ",yPos=" )
      .append( n.getYPos() );

    sb.append( ",width=" )
      .append( n.getWidth() );

    sb.append( ",height=" )
      .append( n.getHeight() );

    sb.append( ",zOrder=" )
      .append( n.getZOrder() );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( n.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE nodeInd=").append( n.getInd() )
      .append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Returns the SQL String to delete the Node from the database.
  **/
  static String getDeleteSQL(Node n) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("DELETE from oa.node WHERE nodeInd=")
      .append( n.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the Nodes for a Symptom.
  **/
  static String getDeleteAllForSymptomSQL(int symptomInd) {
    return "DELETE FROM oa.node WHERE symptomInd=" + symptomInd +
           SqlRunner.END_DELIM;
  }
}
