package com.ibm.nzna.projects.common.quest.oa;    
/*-----------------------------------------------------------*/
/*   ProductConcl                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductConcl                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import java.sql.*;
import java.util.* ;

/**
 * This class represents a single Product Conclusion. This consists 
 *
 *
 * @version 1.2, 6/8/2000 
 * @author  Mark Debiase
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductConcl                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductConcl extends PersistentRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String questionTitle        = null ;
   private String answerTitle          = null ;
   private int    prodInt              = 0 ;
   private int    brandGroupInd        = 0 ;
   private int    questionInd          = 0 ;
   private int    answerInd            = 0 ;
   private String htmlString           = "" ;
   private String dbUser               = null ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default constructor sets rec status to INSERT
    *  
    *  
    *
    *  @param prodInt The product that this record will represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductConcl ( int prodInt ) {
      updateRecStatus ( REC_INSERT ) ;
      this.prodInt = prodInt ;
   }

   ProductConcl () {
   }

   /**
    * Get the product conclusions for the specific product (machinetype-model).
    * 
    * @param productInd  the product indicator (prodint) from tigris.products.
    * 
    * @return a Hastable which contains the question/answer pairs for each
    *         conclusion -- the Hashtable key will be the question indicator
    *         and the value will be the answer indicator.  The Hashtable will
    *         be empty if there were no conclusions for the product.
    *         A <b>null</b> value will be returned if an error is encountered
    *         reading the product conclusions.
   **/
   public static Hashtable getProductConclusions(int productInd) {
      Hashtable productConclusions = new Hashtable();

      SQLMethod sqlMethod = new SQLMethod( DatabaseSystem.QUEST, 
                                           "ProductConclusion.getProductionConclusions()", 1);

      try {
         Statement stmt = sqlMethod.createStatement();
         ResultSet   rs = stmt.executeQuery( "SELECT a.questionInd, a.answerInd " +
                                             "FROM oa.answer a, oa.productConcl p " +
                                             "WHERE p.productInd=" + productInd + " " +
                                             "AND a.answerInd=p.answerInd " +
                                             "FOR READ ONLY"
                                           );

         while ( rs.next() ) {
            int questionInd = rs.getInt(1);
            int answerInd   = rs.getInt(2);

            productConclusions.put( new Integer(questionInd), 
                                    new Integer(answerInd) );
         }
      } catch (Exception e) {
         productConclusions = null;
         LogSystem.log( 1, e );
         sqlMethod.rollBack();
      } finally {
         sqlMethod.close();
      }

      return productConclusions;

   }





   /*------------------------------------------------------------*/
   /* getProductConclRecs
    */
   /**
    *    This method will return a Vector of
    *  ProductConcl objects that make up the
    *  conclusion list for the product given
    *
    *  @param prodInt The product id to look for
    *  @param languageInd The language indicator to read
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getProductConclRecs ( int prodInt, int languageInd ) {
      Vector retVec = null ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, 
                                            "ProductConcl.getProductConclRecs ( "+prodInt+" )",
                                            5 ) ;

      try {
         Statement    stmt    = sqlMethod.createStatement() ;
         ResultSet    rs      = null ;
         ProductConcl readRec = null ;

         rs = stmt.executeQuery ( "SELECT QUESTIONTITLE.TITLE, ANSWERTITLE.TITLE, ANSWER.QUESTIONIND, ANSWER.ANSWERIND, PRODUCTCONCL.DBUSER "+
                                  "FROM OA.QUESTIONTITLE QUESTIONTITLE, "+
                                  "     OA.ANSWERTITLE   ANSWERTITLE,   "+
                                  "     OA.ANSWER        ANSWER,        "+
                                  "     OA.PRODUCTCONCL  PRODUCTCONCL   "+
                                  "WHERE PRODUCTCONCL.PRODUCTIND   = "+prodInt+" AND "+
                                  "      PRODUCTCONCL.ANSWERIND    = ANSWER.ANSWERIND AND "+
                                  "      QUESTIONTITLE.QUESTIONIND = ANSWER.QUESTIONIND AND "+
                                  "      QUESTIONTITLE.SORTORDER   = (SELECT MAX(SORTORDER) FROM OA.QUESTIONTITLE WHERE LANGUAGEIND = "+languageInd+" AND QUESTIONIND = ANSWER.QUESTIONIND) AND "+
                                  "      ANSWERTITLE.ANSWERIND     = ANSWER.ANSWERIND AND "+
                                  "      ANSWERTITLE.SORTORDER     = (SELECT MAX(SORTORDER) FROM OA.ANSWERTITLE WHERE LANGUAGEIND = "+languageInd+" AND ANSWERIND = ANSWER.ANSWERIND)  "+
                                  "FOR FETCH ONLY" ) ;

         retVec = new Vector ( 1, 1 ) ;
         while ( rs.next() ) {
            readRec = new ProductConcl () ;
            readRec.setProdInt ( prodInt ) ;
            readRec.setQuestionAnswer ( rs.getInt ( 3 ),
                                        rs.getInt ( 4 ),
                                        rs.getString ( 1 ),
                                        rs.getString ( 2 ) ) ;
            readRec.setDbUser ( rs.getString(5).trim() ) ;
            retVec.addElement ( readRec ) ;
         }

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;

      return( retVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getProdInt
    */
   /**
    *    This method will return the prodInt that
    *  this record represents. This prodint is defined
    *  in TIGRIS.PRODUCTS
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getProdInt () {
      return( prodInt ) ;
   }




   /*------------------------------------------------------------*/
   /* setQuestionAnswer
    */
   /**
    *    This method will set the question/answer
    *  pair that this record represents
    *  
    *
    *  @param questionInd The QuestionInd as defined in OA.QUESTION
    *  @param answerInd The AnswerInd as defined in OA.ANSWER
    *  @param questionTitle The title of the question. This value can be NULL
    *  @param answerTitle The title of the answer. This value can be NULL
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setQuestionAnswer ( int questionInd, int answerInd, String questionTitle, String answerTitle ) {
      this.questionInd   = questionInd ;
      this.answerInd     = answerInd ;
      if ( questionTitle != null )
         this.questionTitle = questionTitle.trim() ;
      else
         this.questionTitle = null ;

      if ( answerTitle != null )
         this.answerTitle   = answerTitle.trim() ;
      else
         answerTitle = null ;

      updateRecStatus ( REC_UPDATE ) ;
      htmlString =  "<html><body><b>"+questionTitle+"</b><br>&nbsp;&nbsp;&nbsp;"+answerTitle+"</body></html>" ;
   }




   /*------------------------------------------------------------*/
   /* getQuestion
    */
   /**
    *    This method will return the QuestionIndicator
    *  that this record represents. This Question
    *  indicator is defined in OA.QUESTION
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getQuestion () {
      return( questionInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getAnswer
    */
   /**
    *    This method will return the answer that this
    *  record represents. This indicator is defined in
    *  OA.ANSWER
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getAnswer () {
      return( answerInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getQuestionTitle
    */
   /**
    *    This method will return the title of the question
    *  that this record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The title, NULL if no title is given
    *//*---------------------------------------------------------*/
   public String getQuestionTitle () {
      return( questionTitle ) ;
   }




   /*------------------------------------------------------------*/
   /* getAnswerTitle
    */
   /**
    *    This method will return the answer title that
    *  this record represents.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String title NULL if no title
    *//*---------------------------------------------------------*/
   public String getAnswerTitle () {
      return( answerTitle ) ;
   }





   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return an HTML representation
    *  of this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( htmlString ) ;
   }




   /*------------------------------------------------------------*/
   /* writeToDatabase
    */
   /**
    *    This method will write this record to the
    *  database and return 0 if everything went ok
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int writeToDatabase () {
      int         rc        = 1 ;
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "ProductConcl.writeToDatabase", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         switch ( getRecStatus() ) {
            case REC_INSERT :
               stmt.executeUpdate ( getInsertSQL () ) ;
               break ;

            case REC_UPDATE :
               stmt.executeUpdate ( getUpdateSQL () )  ;
               break ;

            case REC_DELETE :
               stmt.executeUpdate ( getDeleteSQL () ) ;
               break ;
         }

         rc = 0 ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
         rc = -2 ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getDbUser
    */
   /**
    *    This method will return the last user who
    *  touched this record
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getDbUser () {
      return( dbUser ) ;
   }




   /*------------------------------------------------------------*/
   /* setDbUser
    */
   /**
    *    This method will set the dbuser field
    *  
    *  
    *
    *  @param dbUser The dbUser to set to. This user id must be in TIGRIS.USERID
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDbUser ( String dbUser ){
      updateRecStatus ( REC_UPDATE ) ;
      this.dbUser = dbUser ;
   }




   /*------------------------------------------------------------*/
   /* getInsertSQL
    */
   /**
    *    This method will return the Insert SQL statement
    *  for this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getInsertSQL () {
      return( "INSERT INTO OA.PRODUCTCONCL "+
              "( PRODUCTIND, ANSWERIND, BRANDIND, DBUSER, CHANGEDTIME )"+
              "VALUES "+
              "( "+prodInt+", "+
              "  "+answerInd+", "+
              "  0, "+
              " '"+dbUser+"', "+
              "  CURRENT TIMESTAMP )" ) ;
   }




   /*------------------------------------------------------------*/
   /* getUpdateSQL
    */
   /**
    *    This method will return the SQL to update
    *  this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getUpdateSQL () {
      return( "UPDATE OA.PRODUCTCONCL "+
              "SET ANSWERIND    = "+answerInd+", "+
              "    BRANDIND   = 0, "+
              "    DBUSER       = '"+dbUser+"', "+
              "    CHANGEDTIME  = CURRENT TIMESTAMP "+
              "WHERE PRODUCTIND = "+prodInt ) ;
   }




   /*------------------------------------------------------------*/
   /* getDeleteSQL
    */
   /**
    *    This method will return the SQL needed to
    *  delete this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getDeleteSQL () {
      return(  "DELETE FROM OA.PRODUCTCONCL WHERE PRODUCTIND = "+prodInt+" AND ANSWERIND = "+answerInd ) ;
   }




   /*------------------------------------------------------------*/
   /* saveProductConclList
    */
   /**
    *    This method will save a Vector of ProductConcl
    *  recs at the same time. This is much more efficient
    *  than calling 'writeToDatabase' on a large list of ProductConcl
    *
    *  @param listVec The vector that contains the ProductConcl objects to save
    *  @param None
    *  @param None
    *
    *  @return rc TRUE if everything went ok
    *//*---------------------------------------------------------*/
   public static boolean saveProductConclList ( Vector listVec ) {
      boolean rc = false ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "ProductConcl.saveProductConclList", 5 ) ;

      if ( ( listVec != null ) && ( listVec.size() > 0 ) ) {
         try {
            CallableStatement stmt     = sqlMethod.createStoredProc ( "QUEST.sqlRunner", 2 );
            String            sqlToRun = "" ;
            int               i        = 0 ;
            int               size     = listVec.size() ;                       
            ProductConcl      curRec   = null ;
            StoredProcRec     retRec   = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, get all of the SQL To run                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( i < size ) {
               curRec = (ProductConcl) listVec.elementAt(i) ;

               switch ( curRec.getRecStatus() ) {
                  case REC_DELETE :
                     sqlToRun+=curRec.getDeleteSQL()+SqlRunner.END_DELIM ;
                     break ;

                  case REC_INSERT :
                     sqlToRun+=curRec.getInsertSQL()+SqlRunner.END_DELIM ;
                     break ;

                  case REC_UPDATE :
                     sqlToRun+=curRec.getUpdateSQL()+SqlRunner.END_DELIM ;
                     break ;
               }
               i++ ;
            }

            if (sqlToRun.length() == 0) {
               LogSystem.log(1, "No changes to be written");
               return( false ) ;
            }

            String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                          SqlRunner.END_DELIM, "\n" );

            LogSystem.log ( 1, "Starting to send statements to SQLRUNNER" ) ;
            LogSystem.log ( 1, debugString ) ;

            /*-----------------------------------------------------------*/
            /*   Set up the Callable Stmt                                */
            /*-----------------------------------------------------------*/
            stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
            stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
            stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      

            /*-----------------------------------------------------------*/
            /*   Execute the SQL                                         */
            /*-----------------------------------------------------------*/
            stmt.execute() ;

            /*-----------------------------------------------------------*/
            /*   Get the StoredProcRec from the Lob                      */
            /*-----------------------------------------------------------*/
            retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 2 );
            stmt.close() ;

            if (retRec == null) {
               LogSystem.log( 2, "StoredProcRec is null - error status unknown");
            }

            if ( ( retRec != null ) && ( retRec.error ) ) {
               LogSystem.log( 2, retRec.errorStr ) ;
               rc = false ;
            } else
               rc = true ;

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e ) ;
         }
      } else
         rc = true ; // TRUE since we did nothing

      sqlMethod.close() ;

      return( rc ) ;
   }

   private void setProdInt ( int prodInt ) {
      this.prodInt = prodInt ;
   }
} /* ProductConcl */
