
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.storedProc.oaReader.*;

/**
 * Online Assistant Document Link to a Quest Document.
 *
 * @version 1.12
 * @author  Mark DeBiase
**/
public class QuestLink extends Link {
  /*-----------------------------------------------------------*/
  /*                                                           */
  /*   This is the serialization ID. If you add fields to this */
  /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
  /* fine as long as you know what you are doing!              */
  /*                                                           */
  /*-----------------------------------------------------------*/
  static final long serialVersionUID = 1000000 ;

  protected int    docInd_  = 0;
  protected String lnDocId_ = "";

  /**
   * Create a new QuestLink with the specified title.
   * The title contains the text to be displayed on the web -
   * used in the HTML anchor tag.
   * <p>
   * This constructor will create a LanguageText object with
   * a TypeLanguageRec of English for the title and add it
   * to the Vector of LanguageText objects in the Link object
   * superclass.
   * 
   * @see Link
   *
   * @param docInd the Quest document indicator
   * @param title  the link title
  **/
  public QuestLink(int docInd, String title) {
    super(title);
    docInd_ = docInd;
  }

  /**
   * QuestLink to be read from the database.
   * @param linkInd the link indicator
  **/
  public QuestLink(int linkInd) {
    super(linkInd);
  }

  /**
   * Initialize from an existing Link object.
  **/
  QuestLink(QuestLink ql) {
    super(ql);

    this.docInd_  = ql.docInd_;
    this.lnDocId_ = ql.lnDocId_;
  }

  /**
   * Initialize from an existing LinkDraft object.
  **/
  QuestLink(QuestLinkDraft qld) {
    super( (LinkDraft) qld );

    this.docInd_  = qld.docInd_;
    this.lnDocId_ = qld.lnDocId_;
  }

  /**
   * Get the Quest Document indicator.
  **/
  public int getDocInd() {
    return docInd_;
  }

  /**
   * Set the Quest Document indicator.
  **/
  void setDocInd(int docInd) {
    docInd_ = docInd;
  }

  /**
   * Gets the LNDOCID from tigris.documents for the Quest document indicator.
  **/
  public String getLNDocId() {
    return lnDocId_;
  }

  /**
   * Sets the LNDOCID from tigris.documents for the Quest document indicator. 
  **/
  void setLNDocId(String lnDocId) {
    lnDocId_ = lnDocId;
  }

  /**
   * Return the URL String for the QuestLink.
   * The return value will be "###doclink###" + getLNDocId() + ".html".
   * The pattern "###doclink###" must be replaced with the appropriate
   * path for the web site the Link will be displayed on (i.e.,
   * "/helpcenter/infotips/techinfo/" for the Infotips web site).
  **/
  public String getUrlString() {
    return "###doclink###" + lnDocId_ + ".html";
  }

}
