
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import java.util.Vector;

/**
 * @version 1.8
 * @author  Vishnu Kumar. E
**/
class QuestionDraftWriter {

  /**
   * Return the SQL for this QuestionDraft.
  **/
  static String getSQL(QuestionDraft qd) {

    StringBuffer sb = new StringBuffer();

    sb.append( getQuestionDraftSQL(qd) );
    if ( qd.valid() ) {
       // insert/update the sub-objects as necessary
       sb.append( getObjectSQL(qd) );
    }

    if ( !qd.modified() && sb.length() > 0 ) {
      // Question wasn't modified but sub-objects were - update timestamp
      sb.append("UPDATE oadraft.question " +
                "SET changedTime=CURRENT TIMESTAMP " +
                "WHERE questionInd=")
        .append( qd.getInd() ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save the Question's 
   * Answer, Title, AddtlInfo and DocLink objects to the datbase.
   * 
   * @param QuestionDraft        the QuestionDraft to be saved
  **/
  static String getObjectSQL(QuestionDraft questionDraft) {

    StringBuffer sb = new StringBuffer();
    Vector v;

    //---------------------------------------------------------------------
    // Loop through each subobject contained in the QuestionDraft object.
    // Update the sortOrder, parentInd and dbUser for the
    // subobject.  setSortOrder() may affect the modification state of
    // the subobject if its order has changed, but setParentInd() and
    // setDbUser() will not affect the modification state.
    //---------------------------------------------------------------------
 
    v = questionDraft.getAnswers();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        Answer a = (Answer) v.elementAt(i);
        if ( a.valid() ) {
          a.setSortOrder( sortOrder++ );
          a.setParentInd( questionDraft.getInd() );
          a.setDbUser( questionDraft.getDbUser() );
        }
        sb.append( AnswerDraftWriter.getSQL(a) );
      }
    }
 
    v = questionDraft.getTitles();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        Title t = (Title) v.elementAt(i);
        if ( t.valid() ) {
          t.setSortOrder( sortOrder++ );
          t.setParentInd( questionDraft.getInd() );
          t.setDbUser( questionDraft.getDbUser() );
        }
        sb.append( QuestionTitleDraftWriter.getSQL(t) );
      }
    }

    v = questionDraft.getAdditionalInfo();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        AddtlInfo ai = (AddtlInfo) v.elementAt(i);
        if ( ai.valid() ) {
          ai.setSortOrder( sortOrder++ );
          ai.setParentInd( questionDraft.getInd() );
          ai.setDbUser( questionDraft.getDbUser() );
        }
        sb.append( QuestionInfoDraftWriter.getSQL(ai) );
      }
    }

    v = questionDraft.getDocLinks();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        DocLink d = (DocLink) v.elementAt(i);
        if ( d.valid() ) {
          d.setSortOrder( sortOrder++ );
          d.setParentInd( questionDraft.getInd() );
          d.setDbUser( questionDraft.getDbUser() );
        }
        sb.append( QuestionLinkDraftWriter.getSQL(d) );
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the Action to the database.
  **/
  static String getQuestionDraftSQL(QuestionDraft qd) {
    String sql;

    switch ( qd.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(qd);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(qd);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(qd);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(QuestionDraft qd) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oadraft.question " +
         "(questionInd, activeQuestionInd, questionTypeInd, " +
          "hiddenFlag, autoFillFlag, " +
          "brandInd, typeWorkInd, docClassInd, comments, " +
          "maintLock, startDate, stopDate, recycled, " +
          "archived, created, dbUser, changedTime) " + 
       "VALUES ("
    );
    // questionInd:
    sb.append( qd.getInd()                                       ).append(',');

    // activeQuestionInd:
    if (qd.getActiveQuestionInd() == 0) {
      sb.append( "NULL"                                         ).append(',');
    } else {
      sb.append( qd.getActiveQuestionInd()                       ).append(',');
    }
    
    // questionTypeInd:
    sb.append( qd.getQuestionType().getInd()                     ).append(',');

    // hiddenFlag:
    sb.append( DatabaseUtil.booleanToDB( qd.getHiddenAnswers() ) ).append(',');

    // autoFillFlag:
    sb.append( DatabaseUtil.booleanToDB( qd.getAutoFill() )      ).append(',');

    // brandInd:
    sb.append( qd.getBrand().getInd()                            ).append(',');

    // typeWorkInd:
    sb.append( qd.getTypeWorkRequired().getInd()                 ).append(',');

    // docClassInd:
    sb.append( qd.getDocClass().getInd()                         ).append(',');

    // comments:
    String comments = qd.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append("NULL,");
    } else {
      sb.append( DatabaseUtil.stringToDB( comments )  ).append(',');
    }

    // maintLock:
    sb.append( DatabaseUtil.booleanToDB( qd.getMaintLock() )     ).append(',');
    
    // startDate:
    sb.append( "CURRENT DATE"                                   ).append(',');

    // stopDate:
    sb.append( "NULL"                                           ).append(',');
    
    // recycled:
    sb.append( DatabaseUtil.booleanToDB( qd.isRecycled() )       ).append(',');
    
    // archived:
    sb.append( DatabaseUtil.booleanToDB( qd.isArchived() )       ).append(',');
    
    // created:
    sb.append( "CURRENT TIMESTAMP"                              ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( qd.getDbUser() )         ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // insert the countries
    sb.append( CountryWriter.getInsertSQL( "oadraft.questionCountry",
                                           qd.getInd(),
                                           qd.getCountryList(),
                                           qd.getDbUser() ) );

    return sb.toString();
  }

  static String getUpdateSQL(QuestionDraft qd) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oadraft.question SET");

    sb.append( " questionTypeInd=" )
      .append( qd.getQuestionType().getInd() );

    sb.append( ",hiddenFlag=" )
      .append( DatabaseUtil.booleanToDB( qd.getHiddenAnswers() ) );

    sb.append( ",autoFillFlag=" )
      .append( DatabaseUtil.booleanToDB( qd.getAutoFill() ) );

    sb.append( ",brandInd=" )
      .append( qd.getBrand().getInd() );

    sb.append( ",typeWorkInd=" )
      .append( qd.getTypeWorkRequired().getInd() );

    sb.append( ",docClassInd=" )
      .append( qd.getDocClass().getInd() );

    sb.append( ",comments=" );
    String comments = qd.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append( "NULL" );
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) );
    }

    sb.append( ",maintLock=" )
      .append( DatabaseUtil.booleanToDB( qd.getMaintLock() ) );

    sb.append( ",recycled=" )
      .append( DatabaseUtil.booleanToDB( qd.isRecycled() ) );
      
    sb.append( ",archived=" )
      .append( DatabaseUtil.booleanToDB( qd.isArchived() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( qd.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE questionInd=").append( qd.getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oadraft.questionCountry",
                                           "questionInd",
                                            qd.getInd(),
                                            qd.getCountryList(),
                                            qd.getOriginalCountryList(),
                                            qd.getDbUser() ) );

    return sb.toString();
  }

  static String getDeleteSQL(Question qd) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);
    int questionDraftInd = qd.getInd();

    // Delete ALL the Answers to the Question
    sb.append( AnswerDraftWriter.getDeleteAllForQuestionSQL(questionDraftInd) );

    // Delete the Question Titles
    sb.append( 
      QuestionTitleDraftWriter.getDeleteAllForQuestionSQL(questionDraftInd)
    );

    // Delete the Question Additional Info
    sb.append(
      QuestionInfoDraftWriter.getDeleteAllForQuestionSQL(questionDraftInd)
    );

    // Delete the Question DocLinks
    sb.append(
      QuestionLinkDraftWriter.getDeleteAllForQuestionSQL(questionDraftInd)
    );

    // Delete the Question itself
    sb.append(
      "DELETE from oadraft.questioncountry where questionind="
    ).append( questionDraftInd ).append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oadraft.question where questionind="
    ).append( questionDraftInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
