
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.util.Vector;

/**
 * @version 1.7
 * @author  Mark DeBiase
**/

class QuestionLinkWriter {

  private static final String SCHEMA = "oa";

  static String getSQL(DocLink d) {
    String sql;

    switch ( d.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(d, SCHEMA);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(d, SCHEMA);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(d, SCHEMA);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(DocLink d, String schema) {
    // Note: created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO " + schema + ".questionLink " +
         "(questionLinkInd, questionInd, sortOrder, linkInd, linkGroupInd, " +
         "docClassInd, dbUser, changedTime) " +
       "VALUES ("
    );
    sb.append( d.getInd()                                     ).append(',');
    sb.append( d.getParentInd()                               ).append(',');
    sb.append( d.getSortOrder()                               ).append(',');
    // determine whether the DocLink represents a Link or LinkGroup
    if ( d.hasLinkGroup() ) {
      sb.append( "NULL,");
      sb.append( d.getLinkGroupInd() ).append(',');
    } else {
      sb.append( d.getLinkInd() ).append(',');
      sb.append( "NULL,");
    }
    sb.append( d.getDocClass().getInd()                       ).append(',');
    sb.append( DatabaseUtil.stringToDB( d.getDbUser() )       ).append(',');
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    // insert the countries
    sb.append( CountryWriter.getInsertSQL( schema + ".questionLinkCntry",
                                           d.getInd(),
                                           d.getCountryList(),
                                           d.getDbUser() ) );

    // insert the conditions
    sb.append( ConditionWriter.getInsertSQL( schema + ".questionLinkCond",
                                             d.getInd(), 
                                             d.getConditions(), 
                                             d.getDbUser() ) );

    return sb.toString();
  }

  static String getUpdateSQL(DocLink d, String schema) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // for a DocLink, only the sortOrder & docClass (and countries, 
    // conditions) can be updated.  the link/linkgroup cannot be updated.

    sb.append("UPDATE " + schema + ".questionLink SET");
    sb.append( " sortOrder=" )
      .append( d.getSortOrder() );
    sb.append( ",docClassInd=" )
      .append( d.getDocClass().getInd() );
    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( d.getDbUser() ) );
    sb.append( ",changedTime=CURRENT TIMESTAMP");
    sb.append(" WHERE questionLinkInd=").append( d.getInd() )
      .append(SqlRunner.END_DELIM);

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( schema + ".questionLinkCntry",
                                           "questionLinkInd",
                                            d.getInd(),
                                            d.getCountryList(),
                                            d.getOriginalCountryList(),
                                            d.getDbUser() ) );

    // update the conditions (updated via delete/insert)
    sb.append( ConditionWriter.getUpdateSQL( schema + ".questionLinkCond",
                                             "questionLinkInd",
                                             d.getInd(),
                                             d.getConditions(),
                                             d.getDbUser() ) );

    return sb.toString();
  }

  static String getDeleteSQL(DocLink d, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete the countries
    sb.append( CountryWriter.getDeleteSQL( schema + ".questionLinkCntry",
                                           "questionLinkInd",
                                            d.getInd() ) );

    // delete the conditions
    sb.append( ConditionWriter.getDeleteSQL( schema + ".questionLinkCond",
                                             "questionLinkInd",
                                             d.getInd() ) );

    sb.append("DELETE FROM " + schema + ".questionLink " +
              "WHERE questionLinkInd=")
      .append( d.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the DocLinks for an Question.
  **/
  static String getDeleteAllForQuestionSQL(int questionInd) {
    return getDeleteAllForQuestionSQL(questionInd, SCHEMA);
  }

  static String getDeleteAllForQuestionSQL(int questionInd, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "DELETE from " + schema + ".questionlinkcond where questionlinkind in " +
        "(select distinct questionlinkind " +
          "from " + schema + ".questionlink where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".questionlinkcntry where questionlinkind in " +
        "(select distinct questionlinkind " +
          "from " + schema + ".questionlink where questionind=" 
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".questionlink where questionind="
    ).append( questionInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
