
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.shared.util.* ;
import java.sql.*;


/**
 * Reader for Online Assistant Questions.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.9
 * @author VishnuKumar.E
**/
public class QuestionReader {

  /**
   * Read and populate the specified Question object.
   * The Question object's indicator should contain the questionInd to be read.
   * @param conn  the database Connection object
   * @param q     the Question to populate
   * @param lang  the language to read the sub-objects for (0 = all languages)
   * @param mode  an OAMode value (i.e., OAMode.DEFAULT or OAMode.WEB).
   * @return true if the Question was successfully read
             false if the Question was not found in the database
  **/
  public static boolean readFromDB(Connection conn, Question q, 
                                   int languageInd, int mode)
                        throws SQLException
  {
    boolean foundQuestion;
    Statement stmt = conn.createStatement();
    ResultSet rs;

    // turn on DB2 explain:
    //stmt.executeUpdate("SET CURRENT EXPLAIN MODE YES");
    //stmt.executeUpdate("SET CURRENT EXPLAIN SNAPSHOT YES");

    rs = stmt.executeQuery( 
         "SELECT " + 
            "questionTypeInd, hiddenFlag, autoFillFlag, " +
            "brandInd, docClassInd, archived, created, " +
            "dbUser, changedTime, openDraft " +
         "FROM oa.question " + 
         "WHERE questionInd=" + q.getInd() + " " +
         "FOR FETCH ONLY"
    );

    if (!rs.next()) {
      foundQuestion = false;
      rs.close() ;
      stmt.close();
    } else {
      foundQuestion = true;
  
      //---------------------------------------------------------------------
      // Read the base Question data
      //---------------------------------------------------------------------
      q.setQuestionType( new TypeQuestionRec( rs.getShort(1), "" ) );
      q.setHiddenAnswers( DatabaseUtil.booleanFromDB( rs.getString(2) ) );
      q.setAutoFill( DatabaseUtil.booleanFromDB( rs.getString(3) ) ); 
      q.setBrand( new TypeBrandRec( rs.getInt(4), "", 0 ) );
      q.setDocClass( new TypeDocClassRec( rs.getShort(5), "" ) );
      q.setArchived( DatabaseUtil.booleanFromDB( rs.getString(6) ) );
      q.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
      q.setDbUser( rs.getString(8).trim() );
      q.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(9) ) );
      q.setOpenDraft( DatabaseUtil.booleanFromDB( rs.getString(10) ) );

      rs.close() ;
      stmt.close();

      //---------------------------------------------------------------------
      // Read the Question Countries
      //---------------------------------------------------------------------

       q.setCountryList(
          CountryReader.readCountries( conn, "oa.questionCountry",
                                       "questionInd", q.getInd() ) );

      //---------------------------------------------------------------------
      // Read the Question Answers
      //---------------------------------------------------------------------
      AnswerReader.readAnswersForQuestion( conn, q, languageInd );

      //---------------------------------------------------------------------
      // Read the Question Titles/AddtlInfo based on the language passed in
      // Pass the mode to AddtlInfoReader for possible optimizations.
      //---------------------------------------------------------------------
      TitleReader.readQuestionTitles( conn, q, languageInd );
      AddtlInfoReader.readQuestionAddtlInfo( conn, q, languageInd, mode );

      //---------------------------------------------------------------------
      // Read the Question DocLinks
      //---------------------------------------------------------------------
      DocLinkReader.readQuestionDocLinks( conn, q );

      q.updateRecStatus( q.REC_NOCHANGE );
      q.answersByInd_ = null;   // dont send Hashtable back from stored proc
    }

    return foundQuestion;
  }
}
