
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.util.Vector;

/**
 * @version 1.11
 * @author  Mark DeBiase
**/
class QuestionTitleWriter {

  private static final String SCHEMA = "oa";

  static String getSQL(Title t) {
    String sql;

    switch ( t.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(t, SCHEMA);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(t, SCHEMA);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(t, SCHEMA);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(Title t, String schema) {
    // Note: created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO " + schema + ".questionTitle " +
         "(questionTitleInd, questionInd, sortOrder, title, languageInd, " +
         "lastTranslated, docClassInd, dbUser, changedTime) "+
       "VALUES ("
    );

    // questionTitleInd:
    sb.append( t.getInd()                                  ).append(',');

    // questionInd:
    sb.append( t.getParentInd()                            ).append(',');

    // sortOrder:
    sb.append( t.getSortOrder()                            ).append(',');

    // title:
    sb.append( DatabaseUtil.stringToDB( t.getTitle() )     ).append(',');

    // languageInd:
    sb.append( t.getLanguage().getInd()                    ).append(',');

    // lastTranslated:
    sb.append( 
      DatabaseUtil.timestampToDB(t.getLastTranslationDate() ) ).append(',');

    // docClassInd:
    sb.append( t.getDocClass().getInd()                    ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( t.getDbUser() )    ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // only insert countries/conditions if language is English
    if ( t.getLanguage().getInd() == 1 ) {

      // insert the countries
      sb.append( CountryWriter.getInsertSQL( schema + ".questionTitleCntry",
                                             t.getInd(),
                                             t.getCountryList(),
                                             t.getDbUser() ) );

      // insert the conditions
      sb.append( ConditionWriter.getInsertSQL( schema + ".questionTitleCond",
                                               t.getInd(), 
                                               t.getConditions(), 
                                               t.getDbUser() ) );
    }

    return sb.toString();
  }

  static String getUpdateSQL(Title t, String schema) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    //sb.append ( "DELETE FROM "+schema+".questionTitleCntry WHERE QUESTIONTITLEIND = "+t.getInd() ) ;
    //sb.append ( SqlRunner.END_DELIM ) ;
    //sb.append ( "DELETE FROM "+schema+".questionTitleCond WHERE QUESTIONTITLEIND = "+t.getInd() ) ;
    //sb.append ( SqlRunner.END_DELIM ) ;
    //sb.append ( "DELETE FROM "+schema+".questionTitle WHERE QUESTIONTITLEIND = "+t.getInd() ) ;
    //sb.append ( SqlRunner.END_DELIM ) ;
    //sb.append ( getInsertSQL ( t, schema ) ) ;

    sb.append("UPDATE " + schema + ".questionTitle SET");

    sb.append( " sortOrder=" )
      .append( t.getSortOrder() );

    sb.append( ",title=" )
      .append( DatabaseUtil.stringToDB( t.getTitle() ) );

//  sb.append( ",languageInd=" )
//    .append( t.getLanguage().getInd() );

    sb.append( ",lastTranslated=" )
      .append( DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) );

    sb.append( ",docClassInd=" )
      .append( t.getDocClass().getInd() );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( t.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE questionTitleInd=").append( t.getInd() )
      .append(" AND languageInd=").append( t.getLanguage().getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // only update countries/conditions if language is English
    if ( t.getLanguage().getInd() == 1 ) {

      // update the country list (updated via delete/insert)
      sb.append( CountryWriter.getUpdateSQL( schema + ".questionTitleCntry",
                                             "questionTitleInd",
                                              t.getInd(),
                                              t.getCountryList(),
                                              t.getOriginalCountryList(),
                                              t.getDbUser() ) );

      // update the conditions (updated via delete/insert)
      sb.append( ConditionWriter.getUpdateSQL( schema + ".questionTitleCond",
                                               "questionTitleInd",
                                               t.getInd(),
                                               t.getConditions(),
                                               t.getDbUser() ) );
    }

    return sb.toString();
  }

  static String getDeleteSQL(Title t, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete the countries
    sb.append( CountryWriter.getDeleteSQL( schema + ".questionTitleCntry",
                                           "questionTitleInd",
                                            t.getInd() ) );

    // delete the conditions
    sb.append( ConditionWriter.getDeleteSQL( schema + ".questionTitleCond",
                                             "questionTitleInd",
                                             t.getInd() ) );

    sb.append("DELETE FROM " + schema + ".questionTitle " +
              "WHERE questionTitleInd=")
      .append( t.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the Titles for an Question.
  **/
  static String getDeleteAllForQuestionSQL(int questionInd) {
    return getDeleteAllForQuestionSQL(questionInd, SCHEMA);
  }

  static String getDeleteAllForQuestionSQL(int questionInd, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "DELETE from " + schema + ".questiontitlecond " +
      "where questiontitleind in " +
        "(select distinct questiontitleind " +
          "from " + schema + ".questiontitle where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".questiontitlecntry " +
      "where questiontitleind in " +
        "(select distinct questiontitleind " +
          "from " + schema + ".questiontitle where questionind="
    ).append( questionInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".questiontitle where questionind="
    ).append( questionInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
