
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import java.sql.*;
import java.util.Vector;

/**
 * @version 1.22
 * @author  Mark DeBiase
**/
public class QuestionWriter {

  /**
   * Return the SQL for this Question.
  **/
  static String getSQL(Question q) {

    StringBuffer sb = new StringBuffer();

    sb.append( getQuestionSQL(q) );
    if ( q.valid() ) {
       // insert/update the sub-objects as necessary
       sb.append( getObjectSQL(q) );
    }

    if ( !q.modified() && sb.length() > 0 ) {
      // Question wasn't modified but sub-objects were - update timestamp
      sb.append("UPDATE oa.question " +
                "SET changedTime=CURRENT TIMESTAMP " +
                "WHERE questionInd=")
        .append( q.getInd() ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save the Question's 
   * Answer, Title, AddtlInfo and DocLink objects to the datbase.
   * 
   * @param question        the Question to be saved
  **/
  static String getObjectSQL(Question question) {

    StringBuffer sb = new StringBuffer();
    Vector v;

    //---------------------------------------------------------------------
    // Loop through each subobject contained in the Questoin object.
    // Update the sortOrder, parentInd and dbUser for the
    // subobject.  setSortOrder() may affect the modification state of
    // the subobject if its order has changed, but setParentInd() and
    // setDbUser() will not affect the modification state.
    //---------------------------------------------------------------------
 
    v = question.getAnswers();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        Answer a = (Answer) v.elementAt(i);
        if ( a.valid() ) {
          a.setSortOrder( sortOrder++ );
          a.setParentInd( question.getInd() );
          a.setDbUser( question.getDbUser() );
        }
        sb.append( AnswerWriter.getSQL(a) );
      }
    }
 
    v = question.getTitles();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        Title t = (Title) v.elementAt(i);
        if ( t.valid() ) {
          t.setSortOrder( sortOrder++ );
          t.setParentInd( question.getInd() );
          t.setDbUser( question.getDbUser() );
        }
        sb.append( QuestionTitleWriter.getSQL(t) );
      }
    }

    v = question.getAdditionalInfo();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        AddtlInfo a = (AddtlInfo) v.elementAt(i);
        if ( a.valid() ) {
          a.setSortOrder( sortOrder++ );
          a.setParentInd( question.getInd() );
          a.setDbUser( question.getDbUser() );
        }
        sb.append( QuestionInfoWriter.getSQL(a) );
      }
    }

    v = question.getDocLinks();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        DocLink d = (DocLink) v.elementAt(i);
        if ( d.valid() ) {
          d.setSortOrder( sortOrder++ );
          d.setParentInd( question.getInd() );
          d.setDbUser( question.getDbUser() );
        }
        sb.append( QuestionLinkWriter.getSQL(d) );
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the Action to the database.
  **/
  static String getQuestionSQL(Question q) {
    String sql;

    switch ( q.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(q);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(q);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(q);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(Question q) {
    // Note: oa.question.created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oa.question " +
         "(questionInd, questionTypeInd, hiddenFlag, autoFillFlag, " +
         "brandInd, docClassInd, archived, dbUser, changedTime, openDraft) " + 
       "VALUES ("
    );
    // questionInd:
    sb.append( q.getInd()                                       ).append(',');

    // questionTypeInd:
    sb.append( q.getQuestionType().getInd()                     ).append(',');

    // hiddenFlag:
    sb.append( DatabaseUtil.booleanToDB( q.getHiddenAnswers() ) ).append(',');

    // autoFillFlag:
    sb.append( DatabaseUtil.booleanToDB( q.getAutoFill() )      ).append(',');

    // brandInd:
    sb.append( q.getBrand().getInd()                            ).append(',');

    // docClassInd:
    sb.append( q.getDocClass().getInd()                         ).append(',');

    // archived:
    sb.append( DatabaseUtil.booleanToDB( q.isArchived() )       ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( q.getDbUser() )         ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP, 'N')")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // insert the countries into oa.questionCountry
    sb.append( CountryWriter.getInsertSQL( "oa.questionCountry",
                                           q.getInd(),
                                           q.getCountryList(),
                                           q.getDbUser() ) );

    return sb.toString();
  }

  static String getUpdateSQL(Question q) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oa.question SET");

    sb.append( " questionTypeInd=" )
      .append( q.getQuestionType().getInd() );

    sb.append( ",hiddenFlag=" )
      .append( DatabaseUtil.booleanToDB( q.getHiddenAnswers() ) );

    sb.append( ",autoFillFlag=" )
      .append( DatabaseUtil.booleanToDB( q.getAutoFill() ) );

    sb.append( ",brandInd=" )
      .append( q.getBrand().getInd() );

    sb.append( ",docClassInd=" )
      .append( q.getDocClass().getInd() );

    sb.append( ",archived=" )
      .append( DatabaseUtil.booleanToDB( q.isArchived() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( q.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE questionInd=").append( q.getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oa.questionCountry",
                                           "questionInd",
                                            q.getInd(),
                                            q.getCountryList(),
                                            q.getOriginalCountryList(),
                                            q.getDbUser() ) );

    return sb.toString();
  }

  static String getDeleteSQL(Question q) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);
    int questionInd = q.getInd();

    // Delete ALL the Answers to the Question
    sb.append( AnswerWriter.getDeleteAllForQuestionSQL(questionInd) );

    // Delete the Question Titles
    sb.append( QuestionTitleWriter.getDeleteAllForQuestionSQL(questionInd) );

    // Delete the Question Additional Info
    sb.append( QuestionInfoWriter.getDeleteAllForQuestionSQL(questionInd) );

    // Delete the Question DocLinks
    sb.append( QuestionLinkWriter.getDeleteAllForQuestionSQL(questionInd) );

    // Delete the Question itself
    sb.append(
      "DELETE from oa.questioncountry where questionind="
    ).append( questionInd ).append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oa.question where questionind="
    ).append( questionInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return the SQL to publish a Question from an QuestionDraft object.
   * 
   * @return a String containing SQL statements separated by 
   *         SqlRunner.END_DELIM.
  **/
  public static String getPublishSQL(Connection conn, QuestionDraft draft)
                       throws Exception
  {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);

    // construct a new Question object from the draft object
    // the recstatus and indicators will be set appropriately
    Question active = new Question( draft );
    active.assignDatabaseKeys(conn);  // throws Exception on failures

    // if there is already an active object, then delete the Conditions,
    // Countries, Titles and Conclusions for all Answers and delete the
    // Titles, AddtlInfo and DocLinks for the Question.
    if (active.getRecStatus() == PersistentRec.REC_UPDATE) {
      int questionInd = active.getInd();
      sb.append( AnswerWriter.getDeleteAnswerDataForQuestionSQL(questionInd) );
      sb.append( QuestionTitleWriter.getDeleteAllForQuestionSQL(questionInd) );
      sb.append( QuestionInfoWriter.getDeleteAllForQuestionSQL(questionInd) );
      sb.append( QuestionLinkWriter.getDeleteAllForQuestionSQL(questionInd) );

      // find any answers which exist in the oa schema but do not exist in 
      // the oadraft schema for the question being published -- these answers
      // should be deleted.
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(
              "SELECT answerind " +
              "FROM oa.answer " +
              "WHERE questionInd=" + active.getInd() + " " +
              "AND answerInd NOT IN (" +
                  "SELECT activeAnswerInd " +
                  "FROM oadraft.answer " +
                  "WHERE questionInd=" + draft.getInd() + " " +
                  "AND activeAnswerInd IS NOT NULL" +
              ")"
      );
      while( rs.next() ) {
        int answerInd = rs.getInt(1);
        // the countries, titles, conclusions, etc. have already been deleted
        // above so we just have to delete the answer itself....
        sb.append( "DELETE FROM oa.answer WHERE answerInd=")
          .append(answerInd).append(SqlRunner.END_DELIM);
        //sb.append( AnswerWriter.getDeleteSQL(answerInd) );
      }
      stmt.close();
    }

    // generate the sql to to publish the action
    sb.append( getSQL(active) );

    // update the oadraft.node and oadraft.edgepreanswer tables with 
    // the active question indicator.  this only has to be done for drafts
    // which have never been published before.
    if (active.getRecStatus() == PersistentRec.REC_INSERT) {
      sb.append(
         "UPDATE oadraft.node " +
         "SET activeQuestionInd=" + active.getInd() + " " +
         "WHERE draftQuestionInd=" + draft.getInd()
      ).append(SqlRunner.END_DELIM);

      sb.append(
         "UPDATE oadraft.edgePreanswer " +
         "SET activeQuestionInd=" + active.getInd() + " " +
         "WHERE draftQuestionInd=" + draft.getInd()
      ).append(SqlRunner.END_DELIM);
    }

    // update the activeAnswerInd in oadraft.answer for any new answers.
    // also update oadraft.edge and oadraft.edgepreanswer.
    Vector answerVec = active.getAnswers();
    for (int i=0; i<answerVec.size(); i++) {
      Answer ans = (Answer) answerVec.elementAt(i);
      if ( ans.getRecStatus() == ans.REC_INSERT ) {
        sb.append(
          "UPDATE oadraft.answer " +
          "SET activeAnswerInd=" + ans.getInd() + " " +
          "WHERE answerInd=" + ans.getDraftAnswerInd()
        ).append(SqlRunner.END_DELIM);

        sb.append(
          "UPDATE oadraft.edge " +
          "SET activeAnswerInd=" + ans.getInd() + " " +
          "WHERE draftAnswerInd=" + ans.getDraftAnswerInd()
        ).append(SqlRunner.END_DELIM);

        sb.append(
          "UPDATE oadraft.edgePreanswer " +
          "SET activeAnswerInd=" + ans.getInd() + " " +
          "WHERE draftAnswerInd=" + ans.getDraftAnswerInd()
        ).append(SqlRunner.END_DELIM);
      } 
    }

    // update the openDraft flag.  this is done seperately at the end
    // in case there are any failures publishing the draft.
    sb.append(
      "UPDATE oa.question " +
      "SET openDraft='N' " +
      "WHERE questionInd="
    ).append( active.getInd() ).append(SqlRunner.END_DELIM);

    // finally, update the stopDate & activeQuestionInd on the draft object
    sb.append(
      "UPDATE oadraft.question " +
      "SET stopDate=CURRENT DATE, " +
          "changedTime=CURRENT TIMESTAMP, " +
          "activeQuestionInd=" + active.getInd() + " " +
      "WHERE questionInd=" + draft.getInd()
    ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
