
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.oaReader.*;
import com.ibm.nzna.projects.common.storedProc.oaPublish.*;
import com.ibm.nzna.projects.common.storedProc.oaValidate.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;



/** 
 * Online Assistant Symptom Draft.
 * 
 * @version 1.24
 * @author Vishnu Kumar
 *
**/

public class SymptomDraft extends Symptom implements DraftObject {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   private int         activeSymptomInd_  = 0;
   private boolean     maintLock_;
   private String      startDate_ = "";
   private String      stopDate_  = null;
   private boolean     recycled_;
   private TypeWorkRec typeWork_  = null;
   private String      comments_  = "";

   public SymptomDraft() {
      super(0);

      TypeList t = TypeList.getInstance();
      typeWork_  = (TypeWorkRec) t.objectFromInd( 1, TypeList.TYPE_WORKREQUIRED );
   }

   public SymptomDraft(int symptomDraftInd) {
      super(symptomDraftInd);
   }

   /**
    * Create a new SymptomDraft object from an active Symptom.
   **/
   SymptomDraft(Symptom publishedSymptom) {
      super(publishedSymptom);

      activeSymptomInd_ = publishedSymptom.symptomInd_;
   }




   /*------------------------------------------------------------*/
   /* resetAsNew
    */
   /**
    *    This method will reset all variables in this Symptom
    *  Draft so that the draft is saved as a new draft
    *  and the new draft creates a new Symptom
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void resetAsNew () {
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, set this indicator and active indicator to 0     */                                          
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeSymptomInd_ = 0 ;
      setInd(0) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Let all child objects know of our new condition         */                                      
      /*                                                           */
      /*-----------------------------------------------------------*/
      initSubObjects ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, make sure a new Symptom is created                 */                              
      /*                                                           */
      /*-----------------------------------------------------------*/
      setTypeWorkRequired ( (TypeWorkRec)TypeList.getInstance().objectFromInd ( TypeList.WORKREQ_CREATE, TypeList.TYPE_WORKREQUIRED ) ) ;
      
   }

   public int getActiveSymptomInd() {
      return activeSymptomInd_;
   }

   void setActiveSymptomInd(int activeSymptomInd) {
      activeSymptomInd_ = activeSymptomInd;
   }

   public boolean getMaintLock() {
      return maintLock_;
   }

   public void setMaintLock(boolean maintLock)
   {
      maintLock_ = maintLock;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns the date the draft was opened.
   **/
   public String getStartDate() {  
      return startDate_;
   }

   /**
    * Set the date the draft was opened.
    * <b>Note:</b> This value may be set, but it will not
    * used when the object is written to the database
    * (CURRENT DATE will be used).
   **/
   public void setStartDate(String startDate) {
      startDate_ = startDate;
   }

   /**
    * Returns the date the draft was closed or null if the draft is still open.
   **/
   public String getStopDate() {
      return stopDate_;
   }

   /**
    * Set the date the draft was closed.
    * <b>Note:</b> This value may be set, but it will not
    * used when the object is written to the database
    * (CURRENT DATE will be used).
   **/
   public void setStopDate(String stopDate) {
      stopDate_ = stopDate;
   }

   /**
    * Returns true if this object is recycled.
   **/
   public boolean isRecycled() {
      return recycled_;
   }

   /**
    * Set recycled flag for this object.
   **/
   public void setRecycled(boolean recycled) {
      recycled_ = recycled;
      updateRecStatus(REC_UPDATE);
   }

   public TypeWorkRec getTypeWorkRequired() {
      return typeWork_;
   }

   public void setTypeWorkRequired(TypeWorkRec typeWork) {
      typeWork_ = typeWork;
      updateRecStatus(REC_UPDATE);
   }

   public String getComments() {
      return comments_;
   }

   public void setComments(String comments) {
      comments_ = comments;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Validate this object for publishing.  
    * 
    * @return a ValidationRec with the results of the validation or
    *         null if an error occurs.  Check the ValidationRec to 
    *         determine if publishing can proceed.
    * @see com.ibm.nzna.projects.common.storedProc.oaValidate.ValidationRec
   **/
   public ValidationRec validate() {
      LogSystem.log(2, ">>> Starting SymptomDraft.validate()");
      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "", 10 );
      ValidationRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ( "QUEST.OAValidate", 3 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter( 3, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, this.getInd() );         // object ind
         stmt.setInt( 2, OAValidate.SYMPTOM );   // object type

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (ValidationRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "ValidationRec was returned null");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            retRec = null;   // return error to the user
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         retRec = null;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> SymptomDraft.validate() complete");

      return retRec;
   }

   /**
    * Assign a database key for this draft Symptom.
    * @return true if a database key was assigned, otherwise false
   **/
   boolean assignMyDatabaseKey() {
      boolean success = true;

      if ( symptomInd_ == 0 && valid() ) {
         LogSystem.log(1, "Getting database key for SymptomDraft");
         symptomInd_ = Counter.getCounter( LogSystem.getInstance(), "DSYMPTMIND" );
         if ( symptomInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for SymptomDraft");
            success = false;
         }
      }

      return success;
   }

   /**
    * Write the Symptom draft and all its contained objects to the database.
    * This method assumes that a DatabaseSystem object for the
    * DatabaseSystem.QUEST database has already been connected.
    * 
    * @return 0 for success, nonzero for failure.
   **/
   public int writeToDatabase() {
      LogSystem.log(2, ">>> Starting SymptomDraft.writeToDatabase()");

      /*--------------------------------------------------------------*/
      /*   Assign database keys for this Question and its subobjects. */
      /*--------------------------------------------------------------*/
      if ( !assignDatabaseKeys() ) {
         return -1;
      }

      SQLMethod sql = new SQLMethod(DatabaseSystem.QUEST,
                                    "SymptomDraft.writeToDatabase()",10);
      int returnCode = 0 ;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sql.createStoredProc("QUEST.ExtSqlRunner", 3);
         String sqlToRun = SymptomDraftWriter.getSQL(this);
         if ( sqlToRun.length() == 0 ) {
            LogSystem.log(1, "No changes to be written");
            return 0;
         }
         String debugString = Text.replaceAllStrInStr( sqlToRun, 
                                                       SqlRunner.END_DELIM, "\n" );

         LogSystem.log ( 1, "Starting to send statements to EXTSQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlToRun.getBytes() ) ;      
         stmt.setInt   ( 2, 0 );  // commit interval

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement(stmt, 3);

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            returnCode = retRec.sqlCode ;
            if ( returnCode == 0 ) returnCode = -1;
         }

         stmt.close() ;

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
         returnCode = -1 ;
      } finally {
         sql.close() ;
      }

      LogSystem.log(2, ">>> SymptomDraft.writeToDatabase() complete");

      return returnCode;
   }

   /**
    * Publish the draft Symptom.
    * Any changes to the draft must be saved to the database by
    * calling writeToDatabase() before the draft is published (the
    * publish method will not save the current object).
    * 
    * @return true for success, false for failure.
   **/
   public boolean publish() {
      LogSystem.log(2, ">>> Starting SymptomDraft.publish()");

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, 
                                      "SymptomDraft.publish()", 10 );
      boolean success = true;
      StoredProcRec retRec = null;

      try {
         CallableStatement stmt = sqlm.createStoredProc ( "QUEST.OAPublish", 3 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, this.getInd() );       // object ind
         stmt.setInt( 2, OAPublish.SYMPTOM );   // object type

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the StoredProcRec from the Lob                      */
         /*-----------------------------------------------------------*/
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 3 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         } else {
            // the userError String will contain the SQL generated for
            // publishing if DEBUG=true in the OAPublish stored procedure.
            if ( retRec.userError.length() > 0 ) {
               LogSystem.log(1, "Publish SQL:");
               LogSystem.log(2, retRec.userError);
            }
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
            success = false;
         }
      } catch ( Exception e ) {
         sqlm.rollBack() ;
         LogSystem.log ( 3, e ) ;
         success = false;
      } finally {
         sqlm.close() ;
      }

      LogSystem.log(2, ">>> SymptomDraft.publish() complete");

      return success;
   }

   /**
    * Read a SymptomDraft object from the database.
    * 
    * @param symptomDraftInd  the database indicator of the 
    *                         SymptomDraft to be read.
    * @param languageInd the language to read the text items for 
    *                    (0 = read all languages).
    * @return the SymptomDraft object or null if an error occured or
    *         the object was not found.  The SymptomDraft object is
    *         returned as a Symptom object -- it must be cast.
   **/
   public static Symptom readFromDatabase(int symptomDraftInd, int languageInd) {
      LogSystem.log(2, ">>> Starting SymptomDraft.readFromDatabase()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "SymptomDraft.readFromDatabase()", 5 );
      StoredProcRec retRec     = null;
      Symptom       retSymptom = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, symptomDraftInd );        // object ind
         stmt.setInt( 2, OAReader.SYMPTOM );       // object type
         stmt.setInt( 3, OAReader.READ_DRAFT );    // read active object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Symptom and the StoredProcReturnRec objects     */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retSymptom = (Symptom)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      LogSystem.log(2, ">>> SymptomDraft.readFromDatabase() completed");

      if ( retSymptom != null ) {
         retSymptom.updateTypeRecData(); // update the Type*Rec objects
         retSymptom.setRecNoChange();    // sets the original country list
      } else {
         LogSystem.log(2, "No object found for indicator " + symptomDraftInd);
      }

      return retSymptom;
   }

   /**
    * Get a draft Symptom.  This method will return the open draft of an
    * Symptom if one exists.  If there is no open draft, a new one will
    * be created.
    * <b>Note:</b> If a new draft is created, the openDraft flag in the
    * oa.action table will be set to 'Y'.  This flag will be reset to
    * 'N' when the draft is published.  If the draft is not saved, the
    * openDraft flag on oa.symptom will have to be manually reset.
    * 
    * 
    * @param symptomInd  the database indicator of the published Symptom 
    *                    whose draft is to be read.
    * @param languageInd the language to read the text items for 
    *                    (0 = read all languages).
    * @param typeWork    the TypeWorkRec to be set into the
    *                    returned draft object if it is a new draft
    * @return the SymptomDraft object or null if an error occured or
    *         the object was not found.
    * @exception PendingDraftException if there is a pending draft
   **/
   public static SymptomDraft getDraft(int symptomInd, int languageInd,
                                       TypeWorkRec typeWork)
   throws PendingDraftException
   {
      LogSystem.log(2, ">>> Starting SymptomDraft.getDraft()");
      SQLMethod sql = new SQLMethod( DatabaseSystem.QUEST, 
                                     "SymptomDraft.getDraft()", 5 );
      StoredProcRec retRec     = null;
      SymptomDraft  retSymptom = null;

      try {
         CallableStatement stmt = sql.createStoredProc ( "QUEST.OAReader", 7 );

         /*-----------------------------------------------------------*/
         /*   Set up the Callable Stmt                                */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 7, Types.LONGVARBINARY ) ;
         stmt.setInt( 1, symptomInd );             // object ind
         stmt.setInt( 2, OAReader.SYMPTOM );       // object type
         stmt.setInt( 3, OAReader.GET_DRAFT );     // read draft object
         stmt.setInt( 4, languageInd );            // language ind
         stmt.setInt( 5, OAMode.getMode() );       // current mode

         /*-----------------------------------------------------------*/
         /*   Execute the SQL                                         */
         /*-----------------------------------------------------------*/
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*   Get the Symptom and the StoredProcReturnRec objects     */
         /*   from the returned LOBs                                  */
         /*-----------------------------------------------------------*/
         retSymptom = 
         (SymptomDraft)StoredProcUtil.getObjectFromStatement( stmt, 6 );
         retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement( stmt, 7 );
         stmt.close() ;

         if ( retRec == null ) {
            LogSystem.log( 2, "StoredProcRec is null - error status unknown");
         }

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log( 2, retRec.errorStr ) ;
         } else {
            if ( retSymptom == null )
               LogSystem.log(2, "No object found for indicator " + symptomInd);
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 3, e ) ;
      }

      sql.close() ;
      LogSystem.log(2, ">>> SymptomDraft.getDraft() completed");

      // throw exception if the return code indicates a pending draft
      if ( retRec != null && retRec.error && 
           retRec.sqlCode == OAReader.PENDING_DRAFT_RC ) {
         throw new PendingDraftException();
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Finish up reading                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retSymptom != null )
         finishGetDraft ( retSymptom, typeWork ) ;

      return retSymptom;
   }


   /**
    * Get a draft Symptom.  This method will return the open draft of an
    * Symptom if one exists.  If there is no open draft, a new one will
    * be created.<br><br><b>Note! This uses JDBC Only, NO STORED PROCEDURE CALL, it is recommended to use the getDraft method which calls the Stored Procedure!</b><br><br>
    * <b>Note:</b> If a new draft is created, the openDraft flag in the
    * oa.action table will be set to 'Y'.  This flag will be reset to
    * 'N' when the draft is published.  If the draft is not saved, the
    * openDraft flag on oa.symptom will have to be manually reset.
    * 
    * 
    * @param symptomInd  the database indicator of the published Symptom 
    *                    whose draft is to be read.
    * @param languageInd the language to read the text items for 
    *                    (0 = read all languages).
    * @param typeWork    the TypeWorkRec to be set into the
    *                    returned draft object if it is a new draft
    * @return the SymptomDraft object or null if an error occured or
    *         the object was not found.
    * @exception PendingDraftException if there is a pending draft
   **/
   public static SymptomDraft getDraftJDBC(int symptomInd, int languageInd,
                                           TypeWorkRec typeWork)
   throws PendingDraftException
   {
      Connection    conn       = DatabaseSystem.getConnection ( DatabaseSystem.QUEST ) ;
      StoredProcRec retRec     = null;
      SymptomDraft  retSymptom = null;

      LogSystem.log(1, ">>> Starting SymptomDraft.getDraft()");

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the Draft                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retSymptom = SymptomDraftReader.getDraft( conn, 
                                                   symptomInd,
                                                   languageInd );

         if ( retSymptom == null )
            LogSystem.log(1, "No object found for indicator " + symptomInd);

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      DatabaseSystem.releaseConnection ( DatabaseSystem.getDatabaseFromConnection ( conn ) ) ;
      LogSystem.log(1, ">>> SymptomDraft.getDraft() completed");

      // throw exception if the return code indicates a pending draft
      if ( retRec != null && retRec.error && 
           retRec.sqlCode == OAReader.PENDING_DRAFT_RC ) {
         throw new PendingDraftException();
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Finish up reading                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retSymptom != null )
         finishGetDraft ( retSymptom, typeWork ) ;

      return retSymptom;
   }





   /*------------------------------------------------------------*/
   /* finishGetDraft
    */
   /**
    *    This method will be called by either getDraft
    *  or getDraftJDBC to finish up the work required
    *  to read in a symptom draft. This work includes
    *  reading all Type*** objects and setting the TypeWork
    *
    *  @param retSymptom The SymptomDraft to finish up with
    *  @param typeWork The type of work required on the symptom
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void finishGetDraft ( SymptomDraft retSymptom, TypeWorkRec typeWork ) {
      if ( retSymptom.getRecStatus() == REC_INSERT ) {
         retSymptom.setTypeWorkRequired( typeWork );
      } else {
         retSymptom.setRecNoChange();  // sets original country list
      }
      retSymptom.updateTypeRecData(); // update the Type*Rec objects
   }




   /**
    * Update the TypeRec objects in the Symptom's subobjects with 
    * the proper descriptions.
    * Called by readFromDatabase();
   **/
   void updateTypeRecData() {
      super.updateTypeRecData();

      TypeList t = TypeList.getInstance();

      // type work required
      TypeWorkRec newTypeWork;
      newTypeWork = (TypeWorkRec) t.objectFromInd( typeWork_.getInd(),
                                                   TypeList.TYPE_WORKREQUIRED );
      if ( newTypeWork != null ) {
         typeWork_ = newTypeWork;
      }

      return;
   }


   /*------------------------------------------------------------*/
   /* forceCloseDraft
    */
   /**
    *    This method will set the value for 'OPENDRAFT'
    * in the OA.SYMPTOM table to 'N' if there are no
    * open drafts.
    *  
    *  
    *
    *  @param databaseInd The database indicator for the SYMPTOM
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void forceCloseDraft ( int databaseInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "forceCloseDraft", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         // update openDraft only if there are no open drafts
         stmt.executeUpdate (
                            "UPDATE oa.symptom SET openDraft='N' " +
                            "WHERE symptomInd=" + databaseInd + " " +
                            "AND (" +
                            "SELECT symptomInd FROM oadraft.symptom " +
                            "WHERE activeSymptomInd=" + databaseInd + " " +
                            "AND stopDate IS NULL" +
                            ") IS NULL"
                            );

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }
      sqlMethod.close() ;
   }


}  

