
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import java.util.Vector;

/**
 * @version 1.7
 * @author  Vishnu kumar. E
**/
class SymptomDraftWriter {

  /**
   * Return the SQL for this Symptom.
  **/
  static String getSQL(SymptomDraft sd) {

    StringBuffer sb = new StringBuffer();

    sb.append( getSymptomSQL(sd) );
    if ( sd.valid() ) {
      // insert/update the sub-objects as necessary
      sb.append( getObjectSQL(sd) );
    }

    // if this is a new symptom, update the root node indicator
    // (it was inserted as NULL due to circular dependency with node table)
    if ( sd.getRecStatus() == sd.REC_INSERT ) {
      sb.append( getUpdateRootNodeSQL(sd) );
    }

    if ( !sd.modified() && sb.length() > 0 ) {
      // Symptom wasn't modified but sub-objects were - update timestamp
      sb.append("UPDATE oadraft.symptom " +
                "SET changedTime=CURRENT TIMESTAMP " +
                "WHERE symptomInd=")
        .append( sd.getInd() ).append(SqlRunner.END_DELIM);
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to save the Symptom's 
   * SymptomTitle, Node and EdgeGroup/Edge objects.
   * 
   * @param symptom        the symptom draft to be saved
  **/
  static String getObjectSQL(SymptomDraft symptom) {

    StringBuffer sb = new StringBuffer();
    Vector v;

    //---------------------------------------------------------------------
    // Loop through each subobject contained in the Symptom object.
    // Update the sortOrder, parentInd and dbUser for the
    // subobject.  setSortOrder() may affect the modification state of
    // the subobject if its order has changed, but setParentInd() and
    // setDbUser() will not affect the modification state.
    //---------------------------------------------------------------------
 
    v = symptom.getSymptomTitles();
    if (v != null) {
      int sortOrder = 1;
      for (int i=0; i<v.size(); i++) {
        SymptomTitle t = (SymptomTitle) v.elementAt(i);
        if ( t.valid() ) { 
          t.setSortOrder( sortOrder++ ); 
          t.setSymptomInd( symptom.getInd() );
          t.setDbUser( symptom.getDbUser() );
        }
        sb.append( SymptomTitleDraftWriter.getSQL(t) );
      }
    }

    v = symptom.getNodes();
    if (v != null) {
      for (int i=0; i<v.size(); i++) {
        Node n = (Node) v.elementAt(i);
        if ( n.valid() ) { 
          n.setSymptomInd( symptom.getInd() );
          n.setDbUser( symptom.getDbUser() );
        }
        sb.append( NodeDraftWriter.getSQL(n) );
      }
    }

    v = symptom.getEdgeGroups();
    if (v != null) {
      for (int i=0; i<v.size(); i++) {
        EdgeGroup e = (EdgeGroup) v.elementAt(i);
        if ( e.valid() ) { 
          e.setSymptomInd( symptom.getInd() );
          e.setDbUser( symptom.getDbUser() );
        }
        sb.append( EdgeGroupDraftWriter.getSQL(e) );
      }
    }

    return sb.toString();
  }

  /**
   * Returns the SQL String to write the Symptom to the database.
  **/
  static String getSymptomSQL(SymptomDraft sd) {
    String sql;

    switch ( sd.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(sd);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(sd);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(sd);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  /**
   * Returns the SQL String to insert the Symptom in the database.
  **/
  static String getInsertSQL(SymptomDraft sd) {

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO oadraft.symptom " +
         "(symptomInd, activeSymptomInd, symptomTypeInd, " + 
         "rootNodeInd, brandInd, typeWorkInd, docClassInd, statusInd, " +
         "comments, maintLock, startDate, stopDate, recycled, " +
         "archived, created, dbUser, changedTime) " +
       "VALUES ("
    );
    // symptomInd:
    sb.append( sd.getInd()                                  ).append(',');

    // activeSymptomInd:
    if (sd.getActiveSymptomInd() == 0) {
      sb.append( "NULL"                                     ).append(',');
    } else {
      sb.append( sd.getActiveSymptomInd()                   ).append(',');
    }

    // symptomTypeInd:
    sb.append( sd.getSymptomType().getInd()                 ).append(',');

    // rootNodeInd:
    // due to circular dependency between rootNodeInd in oadraft.symptom and
    // symptomInd in oadraft.node, we must insert the root node as null and
    // then go back and update it after the nodes are inserted.
    sb.append( "NULL," );

    // oaBrandInd:
    sb.append( sd.getBrand().getInd()                       ).append(',');

    // typeWorkInd:
    sb.append( sd.getTypeWorkRequired().getInd()            ).append(',');

    // docClassInd:
    sb.append( sd.getDocClass().getInd()                    ).append(',');

    // statusInd:
    sb.append( sd.getSymptomStatus().getInd()               ).append(',');

    // comments:
    String comments = sd.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append("NULL,");
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) ).append(',');
    }

    // maintLock:
    sb.append( DatabaseUtil.booleanToDB( sd.getMaintLock() ) ).append(','); 

    // startDate:
    sb.append( "CURRENT DATE"                               ).append(',');

    // stopDate:
    sb.append( "NULL"                                       ).append(',');
    
    // recycled:
    sb.append( DatabaseUtil.booleanToDB( sd.isRecycled() ) ).append(',');

    // archived:
    sb.append( DatabaseUtil.booleanToDB( sd.isArchived() )  ).append(',');

    // created:
    sb.append( "CURRENT TIMESTAMP"                          ).append(',');

    // dbUser:
    sb.append( DatabaseUtil.stringToDB( sd.getDbUser() )    ).append(',');

    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // insert the countries
    sb.append( CountryWriter.getInsertSQL( "oadraft.symptomCountry",
                                           sd.getInd(),
                                           sd.getCountryList(),
                                           sd.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns SQL to update the Symptom root node after all the 
   * Nodes/Egdges have been inserted when creating a new Symptom.
  **/
  static String getUpdateRootNodeSQL(SymptomDraft sd) {

    // first check for missing/improperly added root node

    if ( sd.getRootNode() == null ) {
      throw new IllegalStateException("Root Node has not been set");
    }

    if ( sd.getRootNodeInd() == 0 ) {
      throw new IllegalStateException("Root Node indicator is 0 -- " +
                                      "the Node was probably not put in " +
                                      "the Symptom's Vector of Nodes");
    }

    return "UPDATE oadraft.symptom set rootNodeInd=" + sd.getRootNodeInd() +
           " WHERE symptomInd=" + sd.getInd() +
           SqlRunner.END_DELIM;
  }

  /**
   * Returns the SQL String to update the Symptom in the database.
  **/
  static String getUpdateSQL(SymptomDraft sd) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append("UPDATE oadraft.symptom SET");

    sb.append( " symptomTypeInd=" )
      .append( sd.getSymptomType().getInd() );

    sb.append( ",brandInd=" )
      .append( sd.getBrand().getInd()  );

    sb.append( ",typeWorkInd=" )
      .append( sd.getTypeWorkRequired().getInd() );

    sb.append( ",docClassInd=" )
      .append( sd.getDocClass().getInd() );

    sb.append( ",statusInd=" )
      .append( sd.getSymptomStatus().getInd() );

    sb.append( ",comments=" );
    String comments = sd.getComments();
    if (comments == null || comments.length() == 0) {
      sb.append( "NULL" );
    } else {
      sb.append( DatabaseUtil.stringToDB( comments ) );
    }

    sb.append( ",maintLock=" )
      .append( DatabaseUtil.booleanToDB( sd.getMaintLock() ) );

    sb.append( ",recycled=" )
      .append( DatabaseUtil.booleanToDB( sd.isRecycled() ) );  

    sb.append( ",archived=" )
      .append( DatabaseUtil.booleanToDB( sd.isArchived() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( sd.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE symptomInd=").append( sd.getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // update the country list (updated via delete/insert)
    sb.append( CountryWriter.getUpdateSQL( "oadraft.symptomCountry",
                                           "symptomInd",
                                            sd.getInd(),
                                            sd.getCountryList(),
                                            sd.getOriginalCountryList(),
                                            sd.getDbUser() ) );

    return sb.toString();
  }

  /**
   * Returns the SQL String to delete the Symptom from the database.
  **/
  static String getDeleteSQL(SymptomDraft sd) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE * 2);
    int symptomInd = sd.getInd();

    // Update the rootNode indicator to NULL so we can delete the nodes
    sb.append( "UPDATE oadraft.symptom SET rootNodeInd=NULL WHERE symptomInd=" )
      .append( symptomInd ).append(SqlRunner.END_DELIM);

    // Delete the Nodes & Edges
    sb.append( EdgeGroupDraftWriter.getDeleteAllForSymptomSQL(symptomInd) );
    sb.append( NodeDraftWriter.getDeleteAllForSymptomSQL(symptomInd) );

    // Delete the Symptom Titles
    sb.append( SymptomTitleDraftWriter.getDeleteAllForSymptomSQL(symptomInd) );

    // Delete the Symptom itself
    sb.append(
      "DELETE from oadraft.symptomcountry where symptomind="
    ).append( symptomInd ).append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from oadraft.symptom where symptomind="
    ).append( symptomInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
