
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.projects.common.quest.type.*;
import java.sql.*;
import java.util.Vector;


/**
 * Reader for Online Assistant Actions.
 * This class is used by the OA stored procedures.
 * 
 * @version 1.11
 * @author  Vishnukumar. E
**/
public class SymptomReader {

  private static final short TYPE_STATUS_SYMPTOM = 4;

  /**
   * Read and populate the specified Symptom object.
   * The Symptom object's indicator should contain the symptomInd to be read.
   * @param conn  the database Connection object
   * @param s     the Symptom to populate
   * @param lang  the language to read the sub-objects for (0 = all languages)
   * @param mode  an OAMode value (i.e., OAMode.DEFAULT or OAMode.WEB).
   * @return true if the Symptom was successfully read,
             false if the Symptom was not found in the database
  **/
  public static boolean readFromDB(Connection conn, Symptom s, 
                                   int languageInd, int mode) 
                        throws SQLException
  {
    Statement   stmt = conn.createStatement();
    ResultSet   rs;
    boolean     foundSymptom;

    // turn on DB2 explain:
    //stmt.executeUpdate("SET CURRENT EXPLAIN MODE YES");
    //stmt.executeUpdate("SET CURRENT EXPLAIN SNAPSHOT YES");

    rs = stmt.executeQuery( 
         "SELECT " +
           "symptomTypeInd, rootNodeInd, " +
           "brandInd, docClassInd, statusInd, " +
           "archived, created, dbUser, changedTime, openDraft " +
         "FROM oa.symptom " +
         "WHERE symptomInd=" + s.getInd() + " " +
         "FOR FETCH ONLY"
    );

    if ( !rs.next() ) {
      foundSymptom = false;
      stmt.close();
    } else {
      foundSymptom = true;

      int rootNodeInd;
                      
      //---------------------------------------------------------------------
      // Read the base Symptom data
      //---------------------------------------------------------------------
      s.setSymptomType( new TypeSymptomRec( rs.getShort(1), "" ) );
      rootNodeInd = rs.getInt(2);
      s.setBrand( new TypeBrandRec( rs.getInt(3), "", 0 ) );
      s.setDocClass( new TypeDocClassRec( rs.getShort(4), "" ) );
      s.setSymptomStatus( new TypeStatusRec( rs.getShort(5), 
                                             TYPE_STATUS_SYMPTOM, "" ) );
      s.setArchived( DatabaseUtil.booleanFromDB( rs.getString(6) ) );
      s.setCreationTime( DatabaseUtil.timestampFromDB( rs.getString(7) ) );
      s.setDbUser( rs.getString(8).trim() );
      s.setChangedTime( DatabaseUtil.timestampFromDB( rs.getString(9) ) );
      s.setOpenDraft( DatabaseUtil.booleanFromDB( rs.getString(10) ) );
 
      stmt.close();

      //---------------------------------------------------------------------
      // Read the Symptom Countries
      //---------------------------------------------------------------------
      s.setCountryList(
         CountryReader.readCountries( conn, "oa.symptomCountry",
                                      "symptomInd", s.getInd() ) );

      //---------------------------------------------------------------------
      // Read the Symptom Titles based on the language passed in
      //---------------------------------------------------------------------
      TitleReader.readSymptomTitles( conn, s, languageInd );

      //---------------------------------------------------------------------
      // Read the Symptom Nodes and EdgeGroups/Edges
      //---------------------------------------------------------------------
      NodeReader.readNodesForSymptom( conn, s );
      EdgeGroupReader.readEdgeGroupsForSymptom( conn, s, mode );

      // set the root node:
      Node n = s.getNodeByInd( rootNodeInd );
      s.setRootNode(n);


      s.updateRecStatus( s.REC_NOCHANGE );
      // dont sent the following data back from the stored procedure --
      // it can be rebuilt on the client (reason: less network traffic?).
      s.edgesByInd_ = null;
      s.nodesByInd_ = null;
    }

    return foundSymptom;
  }
}
