
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.quest.Counter;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.shared.util.*;
import java.sql.*;
import java.util.Vector;

/**
 * Symptom Title.
 *
 * @version 1.13
 * @author  Mark DeBiase
**/
public class SymptomTitle extends PersistentRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* object, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;     

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Output Methods                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int HTML           = 2 ;
   public static final int DEBUG          = 1 ;
   public static final int TEXT           = 0 ;


   private int             titleInd_       = 0;
   private int             symptomInd_     = 0;
   private int             sortOrder_      = 0;
   private String          title_          = null;
   private String          keywords_       = "";
   private TypeLanguageRec language_       = null;
   private String          lastTranslated_ = null;
   private TypeDocClassRec docClass_       = null;
   private boolean         indexable_      = false;
   private String          dbUser_         = null;
   private String          changedTime_    = null;
   private Vector          countries_      = null; 
   private Vector          origCountries_  = null;
   private boolean         textUpdated_    = false;
   private int             toStringType    = TEXT ;

   /**
    * New SymptomTitle.
   **/
   public SymptomTitle() {
      this("");

      /* This call sets the Rec Status to REC_UPDATE! */
      setLanguage ( new TypeLanguageRec ( TypeList.LANGUAGE_ENGLISH, "English" ) ) ;
   }

   /**
    * New SymptomTitle, with specified title text.
   **/
   public SymptomTitle(String title) {
      super();
      title_ = title;
      updateRecStatus(REC_INSERT);


      /* This call sets the Rec Status to REC_UPDATE! */
      setLanguage ( new TypeLanguageRec ( TypeList.LANGUAGE_ENGLISH, "English" ) ) ;
   }

   /**
    * SymptomTitle object read from the database.
   **/
   SymptomTitle(int symptomTitleInd) {
      titleInd_ = symptomTitleInd;
      updateRecStatus(REC_NOCHANGE);
   }

   /**
    * Assign a database key for this SymptomTitle.
   **/
   boolean assignDatabaseKey(boolean isDraft) {
      boolean success = true;
      String counterName;

      if ( isDraft ) {
         counterName = "DSYMPTMTTL";
      } else {
         counterName = "SYMPTMTITL";
      }

      if ( titleInd_ == 0 && valid() ) {
         titleInd_ = Counter.getCounter( LogSystem.getInstance(), counterName );
         if ( titleInd_ == 0 ) {
            LogSystem.log(2, "Failed to get database key for SymptomTitle");
            success = false;
         }
      }

      return success;
   }
   // version to be called by publish stored procedure:
   void assignDatabaseKey(Connection conn, boolean isDraft) throws Exception {
      String counterName;

      if ( isDraft ) {
         counterName = "DSYMPTMTTL";
      } else {
         counterName = "SYMPTMTITL";
      }

      if ( titleInd_ == 0 ) {
         titleInd_ = Counter.getCounter( conn, counterName );
         if ( titleInd_ == 0 ) {
            throw new Exception("Failed to get database key for SymptomTitle");
         }
      }

   }

   /**
    * Reset values in this object.
    * Sets the indicator to 0, sets the recStatus to REC_INSERT.
   **/
   public void resetNew() {
      setInd(0);
      setSymptomInd(0);
      // Force the insert
      updateRecStatus(REC_NOCHANGE);
      updateRecStatus(REC_INSERT);
   }

   /**
    * Get the indicator for this SymptomTitle.
   **/
   public int getInd() {
      return titleInd_;
   }

   /**
    * Set the indicator for this SymptomTitle.
   **/
   void setInd(int titleInd) {
      titleInd_ = titleInd;
   }

   /**
    * Get the parent Symptom's indicator.
   **/
   int getSymptomInd() {
      return symptomInd_;
   }

   /**
    * Set the parent Symptom's indicator.
   **/
   void setSymptomInd(int symptomInd) {
      symptomInd_ = symptomInd;
   }

   /**
    * Get the title string.
   **/
   public String getTitle() {
      return title_;
   }

   /**
    * Set the title string.
   **/
   public void setTitle(String title) {
      title_       = title;
      textUpdated_ = true;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Called by Readers to reset the textUpdated flag after reading a Title
    * from the database.
   **/
   void setTextUpdated(boolean textUpdated) {
      textUpdated_ = textUpdated;
   }

   /**
    * Returs true if the title text has been updated.
   **/
   boolean wasTextUpdated() {
      return textUpdated_;
   }

   /**
    * Get the keywords.
   **/
   public String getKeywords() {
      return keywords_;
   }

   /**
    * Set the keywords.
   **/
   public void setKeywords(String keywords) {
      keywords_ = keywords;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Returns true if this object is archived.
   **/
   public boolean isIndexable() {
      return indexable_;
   }

   /**
    * Set the archive state of this object.
   **/
   public void setIndexable(boolean indexable) {
      indexable_ = indexable;
      updateRecStatus(PersistentRec.REC_UPDATE);
   }

   /**
    * Set the title sort order.
   **/
   int getSortOrder() {
      return sortOrder_;
   }

   /**
    * Set the title sort order.
   **/
   void setSortOrder(int sortOrder) {
      sortOrder_ = sortOrder;
   }

   /**
    * Gets the language.
    * @return TypeLanguageRec the language
   **/
   public TypeLanguageRec getLanguage() {
      return language_;
   }

   /**
    * Sets the language.
    * @parm language the language
   **/
   public void setLanguage(TypeLanguageRec language) {
      language_ = language;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Get the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @return a String representing a DB2 timestamp
   **/
   public String getLastTranslationDate() {
      return lastTranslated_;
   }

   /**
    * Set the last translation date for this Title.
    * A null value indicates this Title has never been translated.
    * @param translationDate a String representing a DB2 timestamp
   **/
   public void setLastTranslationDate(String translationDate) {
      lastTranslated_ = translationDate;
   }

   /**
    * Get the document classification for this object.
    * @return the document classification
    * @see com.ibm.nzna.projects.common.quest.type.TypeDocClassRec
   **/
   public TypeDocClassRec getDocClass() {
      return docClass_;
   }

   /**
    * Set the document classification for this object.
    * @param docClassRec the document classification
    * @see com.ibm.nzna.projects.common.quest.type.TypeDocClassRec
   **/
   public void setDocClass(TypeDocClassRec docClass) {
      docClass_ = docClass;
      updateRecStatus(PersistentRec.REC_UPDATE);
   }

   /**
    * Gets the last changed date/time for this object.
    * The changed time is a String in DB2 timestamp format.
   **/
   public String getChangedTime() {
      return changedTime_;
   }

   /**
    * Sets the last changed date/time for this object.
    * Called by database routines.
    * @param changedTime a String in DB2 timestamp format.
   **/
   void setChangedTime(String changedTime) {
      changedTime_ = changedTime;
   }

   /**
    * Gets the user who last changed this object.
   **/
   public String getDbUser() {
      return dbUser_;
   }

   /**
    * Sets the user who last changed this object.
    * Used by Writer classes - the value from the parent object is used
    * when writing to the database.
   **/
   void setDbUser(String dbUser) {
      dbUser_ = dbUser;
   }

   /**
    * Gets the list of country indicators for this object.
    * @return a Vector of TypeCountryCodeRec objects.
    * @see com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec
   **/
   public Vector getCountryList() {
      return countries_;
   }

   /**
    * Sets the list of country indicators for this object.
    * @param countries a Vector of TypeCountryCodeRec objects.
    * @see com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec
   **/
   public void setCountryList(Vector countries) {
      countries_ = countries;
      updateRecStatus(REC_UPDATE);
   }

   /**
    * Get the original country list.
    * The original country list is the list of countries that the object
    * contained when it was read from the database (or when it was last
    * saved to the database).  It is used to compare against the current 
    * country list to determine if any countries have been added/deleted.
   **/
   Vector getOriginalCountryList() {
      return origCountries_;
   }

   /**
    * Set the original country list to the current country list.
    * The original country list is the list of countries that the object
    * contained when it was read from the database (or when it was last
    * saved to the database).  It is used to compare against the current 
    * country list to determine if any countries have been added/deleted.
   **/
   void setOriginalCountryListFromCurrent() {
      if ( countries_ != null ) {
         origCountries_ = (Vector) countries_.clone();
      } else {
         origCountries_ = null;
      }
   }

   /**
    * Perform a shallow copy of this object.
   **/
   public Object clone() {
      try {
         return super.clone();
      } catch ( CloneNotSupportedException e ) {
         throw new InternalError(e.toString());
      }
   }

   /**
    * Return a String representation of this object.
   **/
   public String toString() {
      return ( toString ( toStringType ) ) ;
   }

   /**
    * Return a String representation of this object.
   **/
   public String toString ( int stringType ) {
      switch ( toStringType ) {
         case DEBUG :
            return( "SymptomTitle " + titleInd_ + ": \"" + title_ + 
                    "\" (lang=" + language_.getInd() + ")" ) ;

         case HTML :
            return( "<html><body><font size=2 face=\"Helvetica\"><b>"+title_+"</b></font></body></html>" ) ;

         case TEXT :
         default:
            return( title_ ) ;
      }
   }

   /**
    * Update the TypeRec objects in this object with the proper descriptions.
    * Called after a SymptomTitle is read from the database.
   **/
   void updateTypeRecData() {
      TypeList t = TypeList.getInstance();

      // language
      TypeLanguageRec newLangRec;
      newLangRec = (TypeLanguageRec) t.objectFromInd( language_.getInd(),
                                                      TypeList.TYPE_LANGUAGE );
      if ( newLangRec != null ) {
         language_ = newLangRec;
      }

      // docClass
      TypeDocClassRec newDocClass;
      newDocClass = (TypeDocClassRec) t.objectFromInd( docClass_.getInd(),
                                                       TypeList.TYPE_DOCCLASS );
      if ( newDocClass != null ) {
         docClass_ = newDocClass;
      }

      // country list
      if ( countries_ != null ) {
         Vector newCountries = new Vector();
         for ( int i=0; i<countries_.size(); i++ ) {
            TypeCountryCodeRec country, newCountry;
            country = (TypeCountryCodeRec) countries_.elementAt(i);
            newCountry = (TypeCountryCodeRec) t.objectFromInd( country.getInd(), 
                                                               TypeList.TYPE_COUNTRYCODE );
            if ( newCountry != null ) {
               newCountries.addElement( newCountry );
            } else {
               newCountries.addElement( country );
            }
         }
         countries_ = newCountries;
      }
   }

   /**
    * Set the recStatus to REC_NOCHANGE.
    * Also saves a copy of the current country list.
   **/
   void setRecNoChange() {
      updateRecStatus(REC_NOCHANGE);
      // save the country list to compare for future modifications
      setOriginalCountryListFromCurrent();
   }

   /**
    * Find the SymptomTitle with the specified indicator in a
    * Vector of SymptomTitles.
    * This method will find the first occurence of the 
    * specified SymptomTitle.
    * @param  ind  the SymptomTitle indicator to search for
    * @param  v    the Vector of SymptomTitles to search
    * @return the SymptomTitle with the specified indicator or 
    *         null if it was not found in the Vector.
   **/
   public static SymptomTitle findTitleInVector(int ind, Vector v) {
      return findTitleInVector(ind, v, 1);
   }

   /**
    * Find the nth occurence of the SymptomTitle with the specified
    * indicator in a Vector of SymptomTitles.
    * @param  ind  the SymptomTitle indicator to search for
    * @param  v    the Vector of SymptomTitles to search
    * @param  n    the occurence to find -- must be &gt;= 1
    * @return the SymptomTitle with the specified indicator or 
    *         null if it was not found in the Vector at the specified
    *         occurence.
   **/
   public static SymptomTitle findTitleInVector(int ind, Vector v, int n) {
      SymptomTitle foundObject = null;
      int occurence = 1;  // number of times we found the specified object

      if ( v != null ) {
         for ( int i=0; foundObject == null && i < v.size(); i++ ) {
            SymptomTitle tmp = (SymptomTitle) v.elementAt(i);
            if ( tmp.getInd() == ind ) {
               if ( occurence == n ) {
                  foundObject = tmp;
               } else {
                  occurence++;
               } 
            }
         }
      }

      return foundObject;
   }


   public void updateRecStatus ( int recStatus ) {
      super.updateRecStatus ( recStatus ) ;
   }




   /*------------------------------------------------------------*/
   /* setToStringType
    */
   /**
    *    this method will set what type of output is
    *  returned from the 'toString()' method. The different
    *  types are DEFAULT (only the title), DEBUG (extended info) or HTML (Pretty Info)
    *
    *  @param toStringType The output type as defined in this class (HTML, DEFAULT or DEBUG )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setToStringType ( int toStringType ) {
      this.toStringType = toStringType ;
   }
}
