
package com.ibm.nzna.projects.common.quest.oa;

import com.ibm.nzna.shared.pom.PersistentRec;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec;
import com.ibm.nzna.projects.common.quest.type.TypeLanguageRec;
import java.util.Vector;

/**
 * @version 1.6
 * @author  Mark DeBiase
**/
class SymptomTitleWriter {

  private static final String SCHEMA = "oa";

  static String getSQL(SymptomTitle t) {
    String sql;

    switch ( t.getRecStatus() ) {
      case PersistentRec.REC_INSERT:
        sql = getInsertSQL(t, SCHEMA);
        break;

      case PersistentRec.REC_UPDATE:
        sql = getUpdateSQL(t, SCHEMA);
        break;

      case PersistentRec.REC_DELETE:
        sql = getDeleteSQL(t, SCHEMA);
        break;

      default:
        sql = "";
    }
    return sql;
  }

  static String getInsertSQL(SymptomTitle t, String schema) {
    // Note: created defaults to CURRENT TIMESTAMP, not inserted

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    sb.append(
      "INSERT INTO " + schema + ".symptomTitle " +
         "(symptomTitleInd, symptomInd, sortOrder, title, keywords, " +
         "languageInd, lastTranslated, docClassInd, indexable, " +
         "dbUser, changedTime) " +
      "VALUES ("
    );
    // symptomTitleInd:
    sb.append( t.getInd()                                     ).append(',');
    // symptomInd:
    sb.append( t.getSymptomInd()                              ).append(',');
    // sortOrder:
    sb.append( t.getSortOrder()                               ).append(',');
    // title:
    sb.append( DatabaseUtil.stringToDB( t.getTitle() )        ).append(',');
    // keywords:
    sb.append( DatabaseUtil.stringToDB( t.getKeywords() )     ).append(',');
    // languageInd:
    sb.append( t.getLanguage().getInd()                       ).append(',');
    // lastTranslated:
    sb.append( 
      DatabaseUtil.timestampToDB(t.getLastTranslationDate() ) ).append(',');
    // docClassInd:
    sb.append( t.getDocClass().getInd()                       ).append(',');
    // indexable:
    sb.append( DatabaseUtil.booleanToDB( t.isIndexable() )    ).append(',');
    // dbUser:
    sb.append( DatabaseUtil.stringToDB( t.getDbUser() )       ).append(',');
    // changedTime:
    sb.append("CURRENT TIMESTAMP)")
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // only insert countries if language is English
    if ( t.getLanguage().getInd() == 1 ) {

      // insert the countries
      sb.append( CountryWriter.getInsertSQL( schema + ".symptomTitleCntry",
                                             t.getInd(),
                                             t.getCountryList(),
                                             t.getDbUser() ) );
    }

    return sb.toString();
  }

  static String getUpdateSQL(SymptomTitle t, String schema) { 

    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    //sb.append ( "DELETE FROM "+schema+".symptomTitleCntry WHERE SYMPTOMTITLEIND = "+t.getInd() ) ;
    //sb.append ( SqlRunner.END_DELIM ) ;
    //sb.append ( "DELETE FROM "+schema+".symptomTitle WHERE SYMPTOMTITLEIND = "+t.getInd() ) ;
    //sb.append ( SqlRunner.END_DELIM ) ;
    //sb.append ( getInsertSQL ( t, schema ) ) ;

    sb.append("UPDATE " + schema + ".symptomTitle SET");

    sb.append( " sortOrder=" )
      .append( t.getSortOrder() );

    sb.append( ",title=" )
      .append( DatabaseUtil.stringToDB( t.getTitle() ) );

    sb.append( ",keywords=" )
      .append( DatabaseUtil.stringToDB( t.getKeywords() ) );

//    sb.append( ",languageInd=" )
//      .append( t.getLanguage().getInd() );

    sb.append( ",lastTranslated=" )
      .append( DatabaseUtil.timestampToDB( t.getLastTranslationDate() ) );

    sb.append( ",docClassInd=" )
      .append( t.getDocClass().getInd() );

    sb.append( ",indexable=" )
      .append( DatabaseUtil.booleanToDB( t.isIndexable() ) );

    sb.append( ",dbUser=" )
      .append( DatabaseUtil.stringToDB( t.getDbUser() ) );

    sb.append( ",changedTime=CURRENT TIMESTAMP");

    sb.append(" WHERE symptomTitleInd=").append( t.getInd() )
      .append(" AND languageInd=").append( t.getLanguage().getInd() )
      .append(SqlRunner.END_DELIM);

    /*-----------------------------------------------------------------------*/

    // only update countries if language is English
    if ( t.getLanguage().getInd() == 1 ) {

      // update the country list (updated via delete/insert)
      sb.append( CountryWriter.getUpdateSQL( schema + ".symptomTitleCntry",
                                             "symptomTitleInd",
                                              t.getInd(),
                                              t.getCountryList(),
                                              t.getOriginalCountryList(),
                                              t.getDbUser() ) );
    }

    return sb.toString();
  }

  static String getDeleteSQL(SymptomTitle t, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // delete the countries
    sb.append( CountryWriter.getDeleteSQL( schema + ".symptomTitleCntry",
                                           "symptomTitleInd",
                                            t.getInd() ) );

    sb.append("DELETE FROM " + schema + ".symptomTitle WHERE symptomTitleInd=")
      .append( t.getInd() ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }

  /**
   * Return SQL to delete all the Titles for a Symptom.
  **/
  static String getDeleteAllForSymptomSQL(int symptomInd) {
    return getDeleteAllForSymptomSQL(symptomInd, SCHEMA);
  }

  static String getDeleteAllForSymptomSQL(int symptomInd, String schema) {
    StringBuffer sb = new StringBuffer(DatabaseUtil.DEFAULT_SB_SIZE);

    // Delete the Symptom Titles
    sb.append(
      "DELETE from " + schema + ".symptomtitlecntry where symptomtitleind in " +
        "(select distinct symptomtitleind from " + schema + ".symptomtitle " + 
          "where symptomind="
    ).append( symptomInd ).append(')').append(SqlRunner.END_DELIM);
    sb.append(
      "DELETE from " + schema + ".symptomtitle where symptomind="
    ).append( symptomInd ).append(SqlRunner.END_DELIM);

    return sb.toString();
  }
}
