
package com.ibm.nzna.projects.common.quest.oa;

import java.util.Vector;

/**
 * Vector utility functions.
 *
 * @version 1.4
 * @author  Mark DeBiase
**/
public class VectorUtil {

  /**
   * Return true if two vectors contain the same elements.
   * Element comparison is done with Vector.contains() which uses
   * the equals() method for comparisons.
  **/
  public static boolean vectorsEqual(Vector v1, Vector v2) {
    boolean equal = true;

    // first do some quick checks:
    // * check to see if both vectors are actually the same
    // * check to see if one vector is null and the other is not
    // * check to see if the number of elements is different
    // if none of these conditions hold true, then do an element
    // by element comparison of the two vectors.
    if (v1 == v2) {
      equal = true;  // v1 and v2 are both the same vector or are both null
    } else if (v1 == null && v2 != null) {
      equal = false;
    } else if (v1 != null && v2 == null) {
      equal = false;
    } else if (v1.size() != v2.size()) {
      equal = false;
    } else {
      // check elements of v1 against v2:
      for (int i=0; i<v1.size() && equal; i++) {
        equal = v2.contains( v1.elementAt(i) );
      }
    }

    return equal;
  }

  /**
   * Swap the positions of two objects in a Vector.
   * No error checking is done - it is assumed that both indexes are 
   * valid for the specified Vector.
  **/
  public static void swapElements(Vector v, int index1, int index2) {

    synchronized (v) {
      Object temp = v.elementAt(index1);

      v.setElementAt( v.elementAt(index2), index1 );
      v.setElementAt( temp,                index2 );
    }
  }
}
