package com.ibm.nzna.projects.common.quest.product;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      CreateProductDraft
/*   Creation Date: APRIL 17, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
/**
 * This class simply holds all information about the attempt to
 * create a product draft. Since the creation of a product draft
 * can fail for many reasons this class was created
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* CreateProductDraft
/*
/*----------------------------------------------------*/
public class CreateProductDraft {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  Nothing was done
     **/
   public static final int DRAFT_NONE              = 0 ;

   /**
     *  A Product Draft already exists for the Existing Product. The
     * productDraft property is populated with that draft.
     **/
   public static final int DRAFT_ALREADY_EXISTS    = 1 ;

   /**
     *  A Product Draft already exists, but the draft has been put in
     * the recycling bin. The productDraft property is populated with 
     * that draft and can be used to obtain information about who
     * recycled the productDraft and when.
     **/
   public static final int DRAFT_RECYCLED          = 2 ;

   /**
     *  A Product Draft already exists but is a Misfit product. The
     * productDraft property will be populated with the current record,
     * however Quest Input should not allow a user to edit this misfit.
     **/
   public static final int DRAFT_MISFIT            = 3 ;

   /**
     *  A new Product Draft was created. The productDraft property
     * holds that new Product Draft.
     **/
   public static final int DRAFT_CREATED           = 4 ;

   /**
     *  An SQL Error occurred while creating the new ProductDraft. You
     * may use the 'getSQLCode' method to obtain the SQL Error generated
     **/
   public static final int DRAFT_SQL_ERROR         = 5 ;

   /**
     *  If a draft wsa created for an active product indicator that does not exist
     **/
   public static final int DRAFT_NO_ACTIVE_PRODUCT = 6 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This is the ProductDraft object that resulted in the
    * Draft Creation Process. This property may be null, for
    * example, if the creation process stopped because of an 
    * SQL Error. 
    */
   private ProductDraft productDraft;

   /**
    * If the returnCode for this product is DRAFT_SQL_ERROR 
    * then this property will contain the SQL Code that was
    * generated. 
    */
   private int sqlCode;

   /**
    * This is the return code for the Draft Creation process. This
    * property will contain one of the predefined DRAFT_ values
    * defined in this class. 
    */
   private int returnCode;

   /**
     * The Default Constructor initializes this class
     **/
   public CreateProductDraft ( int returnCode, int sqlCode, ProductDraft productDraft ) {
      this.returnCode   = returnCode ;
      this.sqlCode      = sqlCode ;
      this.productDraft = productDraft ;
   }


   public ProductDraft getProductDraft() {
      return(productDraft);
   }

   /**
     * If the getReturnCode method returns DRAFT_SQL_ERROR then
     * this method will return the DB/2 error 
     */
   public int getSqlCode() {
      return(sqlCode);
   }

   /**
     * This method will return the return code as defined in this class. This
     * return code will be either DRAFT_CREATED, DRAFT_SQL_ERROR, DRAFT_RECYCLED,
     * DRAFT_NONE or DRAFT_ALREADY_EXISTS
     *
     * @return int
     */
   public int getReturnCode() {
      return(returnCode); 
   }


   /**
     *  This method will simply write out all of our properties to a string
     * and return it
     **/
   public String toOut() {
      String retStr = "" ;

      retStr+="Contains Product Draft:"+((productDraft==null) ? "No" : "Yes" )+"\n" ;
      retStr+="sqlCode:               "+sqlCode+"\n" ;
      retStr+="returnCode:            " ;

      switch ( returnCode ) {
         case DRAFT_NONE :
            retStr+="DRAFT_NONE" ;
            break ;

         case DRAFT_ALREADY_EXISTS :
            retStr+="DRAFT_ALREADY_EXISTS" ;
            break ;

         case DRAFT_RECYCLED :
            retStr+="DRAFT_RECYCLED" ;
            break ;

         case DRAFT_MISFIT :
            retStr+="DRAFT_MISFIT" ;
            break ;

         case DRAFT_CREATED :
            retStr+="DRAFT_CREATED" ;
            break ;

         case DRAFT_SQL_ERROR :
            retStr+="DRAFT_SQL_ERROR" ;
            break ;

         case DRAFT_NO_ACTIVE_PRODUCT :
            retStr+="DRAFT_NO_ACTIVE_PRODUCT" ;
            break ;
      }

      retStr+="\n" ;

      return ( retStr ) ;
   }
} /* CreateProductDraft */
