package com.ibm.nzna.projects.common.quest.product ;   
/*-----------------------------------------------------------*/
/*   DraftReadTest                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DraftReadTest                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import java.util.* ;
/**
 * This small program simply connects to the quest database 
 * and reads a document Draft. The database information is
 * passed in from command line as is the document indicator
 *
 *
 * @version 1.0, 2/23/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DraftReadTest                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DraftReadTest extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String databaseName      = null ;
   private String server            = null ;
   private String userId            = null ;
   private String password          = null ;
   private String portNumber        = null ;
   private int    docInd            = 0 ;




   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This is the execution point
    *  
    *  
    *
    *  @param databaseName The name of the database
    *  @param Server The name of the server
    *  @param port The JDBC Port number
    *  @param userId The user id to connect with
    *  @param password The password to connect with
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String args[] ) {
      new com.ibm.nzna.shared.util.LogSystem ( 1, "DraftReadTest.out" ) ;
      new DraftReadTest ( args ) ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method kicks off our test
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DraftReadTest ( String args[] ) {
      try {
         databaseName=args[0] ;
         server=args[1] ;
         portNumber=args[2] ;
         userId=args[3] ;
         password=args[4] ;
         docInd = new Integer ( args[5] ).intValue() ;
      } catch ( Exception e ) {
         System.out.println ( "Usage:\njava com.ibm.nzna.projects.common.quest.doc.DocReader <databaseName> <serverName> <portNumber> <userId> <password> <doc indicator>" ) ;
         System.exit(0) ;
      }


      try {
         DatabaseSystem d = new DatabaseSystem ( false, null, false ) ;

         if ( d.createConnection ( DatabaseSystem.QUEST,
                                   databaseName,
                                   server,
                                   portNumber,
                                   userId,
                                   password ) ) {

            System.out.print ( "Initializing TypeList...  " ) ;
            new TypeList() ;
            TypeList.getInstance().initialize ( false ) ;
            System.out.println ( "Done" ) ;

            for ( int i = docInd; i < (docInd+5); i++ ) {
               ProductDraft  product     = new ProductDraft ( i ) ;
               long          startMillis = 0 ;
               int           rc          = product.readFromDatabase() ;

               product = (ProductDraft)product.findFromCategory ( "IBM Aptiva", "PS/1", "2165", "55C" ) ;
               if ( product != null )
                  System.out.println ( "Found Product "+product.getProductInd()+" Through Product.find" ) ;
               product = new ProductDraft ( i ) ;
               
               startMillis = System.currentTimeMillis() ;
               rc = product.readFromDatabase() ;
               System.out.println ( "--------------------------------------------------\nTotal Time Taken:"+((System.currentTimeMillis()-startMillis))+"(ms)\n-----------------------------------------------" ) ;

               if ( rc == 0 )
                  System.out.println ( product.toOut() ) ;
               else if ( rc == 100 )
                  System.out.println ( "Could not find Product" ) ;
               else
                  System.out.println ( "Error:"+rc ) ;
            }




         } else
            System.out.println ( "Could not connect to "+databaseName+" on "+server+" on port "+portNumber ) ;
      } catch ( Exception e1 ) {
         e1.printStackTrace() ;
         System.out.println ( "Could not connect to "+databaseName+" on "+server+" on port "+portNumber ) ;

      }
   }

} /* DraftReadTest */
