package com.ibm.nzna.projects.common.quest.product;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      product
/*   Creation Date: APRIL 11, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
/**
 * The Product class is the super class for all Products relating
 * to Quest. This class holds all properties associated
 * with a Product and should represent a Product in the 
 * PRODUCT.PRODUCT table.
 *
 *
 * @author  David Lentz
 * @version 1
 * @since 4/11/2001
 */

/*----------------------------------------------------
/*
/* Product
/*
/*----------------------------------------------------*/
public class Product extends PersistentRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /*------------------------------------*/
   /*                                    */
   /*  These are the property indicators */
   /*                                    */
   /*------------------------------------*/
   public static final int BASEMODEL     = 1  ;
   public static final int BRAND         = 2 ;
   public static final int MACHINE       = 3 ;
   public static final int MODEL         = 4 ;
   public static final int FAMILY        = 5 ;
   public static final int CHECKEDOUTBY  = 6 ;
   public static final int CREATEDBY     = 7 ;
   public static final int CREATEDON     = 8 ;
   public static final int CUSTOMER      = 9 ;
   public static final int DATES         = 10 ;
   public static final int DESCRIPTION   = 11 ;
   public static final int INPUT         = 12 ;
   public static final int LOCALE        = 13 ;
   public static final int OFFERING      = 14 ;
   public static final int OPICMIMPORT   = 16 ;
   public static final int PRODUCTIND    = 17 ;
   public static final int PRODUCTTYPE   = 18 ;
   public static final int SOURCE        = 19 ;
   public static final int WARRANTY      = 20 ;
   public static final int XREF          = 21 ; 
   public static final int PARTNUMBER    = 22 ;
   public static final int SUBCATEGORY   = 23 ;
   public static final int PROJECT       = 24 ;


   public class FireThread extends Object implements Runnable {

      int      property  = 0 ;
      Vector   listeners = null ;
      Product product  = null ;

      public FireThread ( Product product, int property, Vector listeners ) {
         this.product   = product ;
         this.property  = property ;
         this.listeners = listeners ;

         new Thread ( this ).start() ;
      }

      public void run () {
         int i     = 0 ;
         int size  = listeners.size() ;

         while ( i < size ) {
            ((ProductPropertyListener)listeners.elementAt(i)).productChanged ( product, property ) ;
            i++ ;
         }
      }
   }



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * This string is used to define the source of a Product as the 
     * author
     */
   private static String SOURCE_AUTHOR = "Author" ;

   /**
     *  This vector holds all Comment History items
     **/
   private Vector commentHistory = null ;

   /**
     * This string is used to define the source of a Product as the 
     * OPIC/M Database
     */
   private static String SOURCE_OPICM = "OPIC/M" ;

   /**
     *  This is the Product Indicator that is the Primary Key that
     * identifies a Unique Active Product
     **/
   private int    productInd    = 0 ;

   /**
     *  This boolean flag determines if any properties are fired to
     * listeners when they change
     **/
   private boolean fireEventEnabled = true ;

   /**
     *  This is the Publish flag. If TRUE, this product should be published
     * to all appropriate web sites and 3rd party vendors
     **/
   private boolean publish = true ;

   /**
     *  This is the Customer Indicator that relates to the product.TypeCustomer table
     **/
   private int    customer      = 0 ;

   /**
     *  This is the list of listeners that will be notified of changes
     **/
   private Vector listeners = null ;

   /**
     *  This is the Warranty Indicator that relates to the product.TypeWarranty table
     **/
   private int    warranty      = 0 ;

   /**
     *  This is the offering indicator that relates to the product.TypeOffering Table. An
     * example of the data in the TypeOffering table would be 'LE Direct', 'Special Bid'
     **/
   private int    offering      = 0 ;

   /**
     *  What type of Product this is. This is the productTypeInd and relates to the
     * product.TypeProduct table. Examples of this data would include 'System', 'Monitor' or 'Option'
     **/
   private int    type          = 0 ;

   /**
     *  The Product Number for this System. This does not have a set method because it is always
     * set by taking the Machine, adding a Dash, and then adding the Model, so if the machine was
     * 2168 and the model was 56C then the prodNum would automatically be set to '2168-56C'
     **/
   private String prodNum       = null ;

   /**
     *  This is the flag telling ProdGen if it should import OPIC/M Data over this product or not
     **/
   private boolean opicmImport = false ;

   /**
     *  The Part Number is only used for Products with a ProductTypeInd of Option. This is the
     * unique Option Part Number. 
     **/
   private TypeCategoryRec partNum       = null ;

   /**
     *  This is the human readable descriptionion of this product.
     **/
   private String description      = null ;

   /**
     *  This is the base model that this product is based off of. This value can be NULL indicating
     * that this product is not based off of any other product.
     **/
   private String baseModel     = null ;

   /**
     *  The Input field indicates how the data was entered into the database. This value will either
     * be a Name or the text 'OPIC/M'. If the value is not 'OPIC/M' it is assumed that the text is an
     * author's name.
     **/
   private String input         = null ;

   /**
     *  This is the source of the information (Customer Wise) of this product. This is a free form
     * text field that the customer enters.
     **/
   private String source        = SOURCE_AUTHOR ;

   /**
     *  This property holds the User Id of who is currently editing this product. This could be 
     * NULL. If this property is NULL, then no one is editing this product. If the value is NON-NULL
     * then it should be assumed that the person currently has this Product up in Quest Input and is
     * editing the product.
     **/
   private String checkedOutBy  = null ;

   /**
     *  This is the date that this product was created. This date can be NULL however. A NULL value
     * usually indicates that this product was created prior to Quest 6.0
     **/
   private String createdOn     = null ;

   /**
     *  This is the User Id of the person who created this product. 
     **/
   private String createdBy     = null ;

   /**
     * This is the time that this product was last touched or saved. This can be NULL in the event
     * that this product has not been saved to the database yet.
     **/
   private String lastTouchedOn = null ;

   /**
     * This is the user id of the person who last touched or saved this product. This can be NULL in the
     * event that this product has not been saved to the database yet.
     **/
   private String lastTouchedBy = null ;

   /**
    * This XRef property is a Vector of XRef objects. Each object represents a row in the 
    * PRODUCT.XREF table. Please see the XRef object documentation for more information 
    */
   private Vector xRef = null ;

   /**
    * The LocaleRec describes the Country and Geography associations
    * to this product. This includes any 'All' Geographies and any 'World Wide'
    * information 
    */
   private LocaleRec locale = null ;

   /**
     *  This Vector holds all ProductDate objects regarding this product. If you use
     * this vector please keep in mind that you should not ever remove an object from the
     * vector. Doing so will not delete the date from the database. If you wish to remove
     * a date from this product, simply call the 'removeDate' method.
     **/
   protected Vector dates = null ;



   /**
     *  This flag tells the DocumentWriter if the Brand list has changed
     **/
   protected boolean brandUpdated = false ;

   /**
     * This flag tells the DocumentWriter if the Project list has changed
     **/
   protected boolean projectUpdated = false ;

   /**
     *  This flag tells the DocumentWriter if the Part Number has changed
     **/
   protected boolean partNumUpdated = false ;

   /**
     *  This flag tells the DocumentWriter if the Family list has changed
     **/
   protected boolean familyUpdated = false ;

   /**
     *  This flag tells the DocumentWriter if the machine list has changed
     **/
   protected boolean machineUpdated = false ;

   /**
     *  This flag tells the DocumentWriter if the Model list has changed
     **/
   protected boolean modelUpdated = false ;

   /**
     *  This is the product's brand
     **/
   private TypeCategoryRec brand = null ;

   /**
     *  This is the product's family
     **/
   private TypeCategoryRec family = null ;

   /**
     *  This is the product's model
     **/
   private TypeCategoryRec model = null ;

   /**
     *  This is the product's machine
     **/
   private TypeCategoryRec machine = null ;

   /**
     *  This is the product's project
     **/
   protected TypeCategoryRec project = null ;

   /**
     *  This is the product's Family Type. This value is only valid
     * if the type of this product is an option
     **/
   private TypeCategoryRec subCategory = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/


   /**
     * This constructor will create a blank product record. Normally
     * you will only use this constructor when creating a new Product
     * for the Quest database.
     **/
   public Product () {
      updateRecStatus ( REC_INSERT ) ;
   }

   /**
     * This constructor will create a product object that represents
     * a Product already in the PRODUCT.PRODUCT table. If you simply
     * wish to read in all information about this product, use this
     * constructor and then call Product.readFromDatabase or Product.readFromXML
     **/
   public Product ( int productInd ) {
      setProductInd ( productInd ) ;
   }



   /*------------------------------------------------------------*/
   /* getProductInd
    */
   /**
    *    This method will return the current Product
    *  Primary Key. If the key has not been set, it will be 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getProductInd() {
      generateKeys() ;
      return(productInd);
   }


   /**
     * This method will simply return the product indicator without
     * generating one if there isn't one
     */
   public int getProductIndNow() {
      return( productInd ) ;
   }


   /**
     * This method will set this product's Product Indicator property
     **/
   protected void setProductInd ( int productInd ) {
      this.productInd = productInd ;
      firePropertyChanged ( PRODUCTIND ) ;
   }

   /**
     * This method will clone this product and all properties in this
     * product. This means all Vectors, Strings etc.. properties in this
     * record will be duplicated. However, the actual Objects inside of a
     * vector will not be duplicated.
     */
   public Object cloneAll () {
      Product returnRec = (Product)clone() ;

      returnRec.productInd = this.productInd ;

      return( returnRec ) ;
   }


   /*------------------------------------------------------------*/
   /* getCustomer
    */
   /**
    *    This method will return the customerTypeInd
    *  that can be used with the TypeList class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getCustomer() {
      return(customer);
   }




   /*------------------------------------------------------------*/
   /* setCustomerType
    */
   /**
    *    This method will set the customer type. This
    *  number MUST be in the PRODUCT.TYPECUSTOMER
    *  table or when you save you will get an error
    *
    *  @param customer The customer to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCustomer( int customer ) {
      this.customer = customer;
      updateRecStatus ( REC_UPDATE ) ;
      firePropertyChanged( CUSTOMER ) ;
   }




   /*------------------------------------------------------------*/
   /* getWarrantyType
    */
   /**
    *    This method returns the warrantyTypeInd
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getWarranty() {
      return(warranty);
   }




   /*------------------------------------------------------------*/
   /* setWarranty
    */
   /**
    *    This method will set the warranty property
    *  
    *  
    *
    *  @param warranty The new warranty
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWarranty(int warranty) {
      this.warranty = warranty;
      updateRecStatus ( REC_UPDATE ) ;
      firePropertyChanged ( WARRANTY ) ;
   }




   /*------------------------------------------------------------*/
   /* getOffering
    */
   /**
    *    This method will return the offeringTypeInd
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getOffering() {
      return(offering);
   }




   /*------------------------------------------------------------*/
   /* setOffering
    */
   /**
    *    This method will set the offering property
    *  
    *  
    *
    *  @param offering The new offering
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOffering(int offering) {
      this.offering = offering;
      updateRecStatus ( REC_UPDATE ) ;
      firePropertyChanged ( OFFERING ) ;
   }




   /*------------------------------------------------------------*/
   /* getProductType
    */
   /**
    *    This method will return the productTypeInd
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getProductType() {
      return(type);
   }



   /*------------------------------------------------------------*/
   /* getProdNum
    */
   /**
    *    This method will return the product number
    *  that this product has assigned to it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getProdNum() {
      return(prodNum);
   }




   /*------------------------------------------------------------*/
   /* getPartNumber
    */
   /**
    *    This method will return the part number. This
    *  property will most likely be NULL unless this is
    *  a product of type Option
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public TypeCategoryRec getPartNumber() {
      return(partNum);
   }




   /*------------------------------------------------------------*/
   /* setPartNumber
    */
   /**
    *    This method will set the part number for
    *  this product
    *  
    *
    *  @param partNum The part number to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPartNumber ( TypeCategoryRec partNum ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the part number we have is null, use the one passed  */
      /* in, however, if the part number we have is not null       */
      /* then simple set the description of our current partnumber */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( this.partNum == null )
         this.partNum = partNum;
      else {
         if ( partNum != null ) {
            this.partNum.setDescript ( partNum.toString() ) ;
            this.partNum.setIsDraft  ( partNum.getIsDraft() ) ;
            this.partNum.setParentCatInd ( partNum.getParentCatInd() ) ;
         } else
            this.partNum = null ;
      }

      partNumUpdated = true ;
      firePropertyChanged ( PARTNUMBER ) ;
   }




   /*------------------------------------------------------------*/
   /* getDescript
    */
   /**
    *    This method will return the descriptionion
    *  of this product
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getDescription() {
      return(description);
   }




   /*------------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    This method will set the descriptionion of this
    *  product. A NULL descriptionion is not allowerd
    *  
    *
    *  @param description The Description to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescription ( String description ) throws IllegalArgumentException {
      if ( ( description == null ) || ( description.length() > 254 ) )
         throw new IllegalArgumentException ( "Product Descriptions cannot "+
                                              "be null and must be less than "+
                                              "254 characters" ) ;
      else {
         this.description = description;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( DESCRIPTION ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* getBaseModel
    */
   /**
    *    This method will return the model that this
    *  product is based off of.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getBaseModel() {
      return(baseModel);
   }




   /*------------------------------------------------------------*/
   /* setBaseModel
    */
   /**
    *    This method will set the model that this
    *  product is based off of
    *  
    *
    *  @param baseModel The model that this product is based off of
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBaseModel(String baseModel) {
      this.baseModel = baseModel;
      updateRecStatus ( REC_UPDATE ) ;
      firePropertyChanged ( BASEMODEL ) ;
   }




   /*------------------------------------------------------------*/
   /* getInput
    */
   /**
    *    This method will return the input property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getInput() {
      return(input);
   }




   /*------------------------------------------------------------*/
   /* setInput
    */
   /**
    *    This method will set the input field for
    *  this product.
    *  
    *
    *  @param input The new input
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setInput(String input) throws IllegalArgumentException { 
      if ( ( input == null ) || ( input.length() > 25 ) )
         throw new IllegalArgumentException ( "The Input property of a"+
                                              "Product cannot be NULL or "+
                                              "exceed 25 characters" ) ;
      else {
         this.input = input;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( INPUT ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* getSource
    */
   /**
    *    This method will return the source of this
    *  product's information
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getSource() {
      return(source);
   }




   /*------------------------------------------------------------*/
   /* setSource
    */
   /**
    *    This method will set the source of this
    *  product's information
    *  
    *
    *  @param source The new source of this product's information
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSource ( String source ) throws IllegalArgumentException { 
      if ( ( source == null ) || ( source.length() > 254 ) )
         throw new IllegalArgumentException ( "The Product Source cannot be "+
                                              "NULL and it cannot exceed "+
                                              "254 characters" ) ;
      else {
         this.source = source;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( SOURCE ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* getCheckedOutBy
    */
   /**
    *    This method will return the user id of the
    *  user who currently has this product checked out
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getCheckedOutBy() {
      return(checkedOutBy);
   }




   /*------------------------------------------------------------*/
   /* setCheckedOutBy
    */
   /**
    *    This method will set the user id of the
    *  person who should be considered the one who
    *  is editing this product. Please note that this does not
    *  automatically update the database. To update the database automatically
    *  and set the 'Checked Out By' property please use the 'checkOut' method.
    *
    *  @param checkedOutBy A user id
    *  @param None
    *  @param None
    *
    *  @see checkOut
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCheckedOutBy ( String checkedOutBy ) throws IllegalArgumentException { 
      if ( checkedOutBy != null ) {
         if ( checkedOutBy.length() > 12 )
            throw new IllegalArgumentException ( "The Product CheckedOutBy cannot "+
                                                 "exceed 12 characters in length" ) ;
         else {
            this.checkedOutBy = checkedOutBy;
            updateRecStatus ( REC_UPDATE ) ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* getCreatedOn
    */
   /**
    *    This method will return the date that this
    *  product was created on
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getCreatedOn() {
      return(createdOn);
   }




   /*------------------------------------------------------------*/
   /* getCreatedBy
    */
   /**
    *    This method will return the User Id of the
    *  person who originally created this product
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getCreatedBy() {
      return(createdBy);
   }






   /*------------------------------------------------------------*/
   /* getLastTouchedOn
    */
   /**
    *    This method will return the timestamp of when
    *  this product was last saved/touched
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getLastTouchedOn() {
      return(lastTouchedOn);
   }




   /*------------------------------------------------------------*/
   /* getLastTouchedBy
    */
   /**
    *    This method will return the user id of the
    *  person who last saved/touched this product
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getLastTouchedBy() {
      return(lastTouchedBy);
   }

   /**
    * This method will return a Vector of XRef objects. Each XRef
    * object represents a single row in the PRODUCT.XREF table.
    * Please note
    * that if you wish to delete an XRef entry from this product,
    * it must remain in the XRef Vector. You should use the
    * 'XRef.updateRecStatus ( XRef.REC_DELETE)' method to delete an
    * XRef item. If you delete an XRef item from the vector then that
    * XRef row will remain in the database.
    **/
   public Vector getXRef() {
      return(xRef);
   }

   /**
     * This method will set the list of XRef objects.  Please note
     * that if you wish to delete an XRef entry from this product,
     * it must remain in the XRef Vector. You should use the
     * 'XRef.updateRecStatus ( XRef.REC_DELETE)' method to delete an
     * XRef item. If you delete an XRef item from the vector then that
     * XRef row will remain in the database.
     **/
   public void setXRef ( Vector xRef ) {
      this.xRef = xRef;
      firePropertyChanged ( XREF ) ;
   }

   /**
     * This method will return the LocaleRec for this product
     **/
   public LocaleRec getLocale() {
      if ( locale != null )
         return( (LocaleRec)locale.clone() ) ;
      return(new LocaleRec());
   }

   /**
     * This method will set the Locale rec of this product. The Locale
     * rec is an Object which holds both the Geography and Country
     * information regarding this product
     **/
   public void setLocale ( LocaleRec locale ) {
      this.locale = locale;
      firePropertyChanged ( LOCALE ) ;
   }

   /**
     * This method will return the TypeCategoryRec representing this product's
     * Brand. This value may be null. To get
     * the actual String representation, use the TypeCategoryRec.getDescription()
     * method.
     **/
   public TypeCategoryRec getBrand () {
      return( brand ) ;
   }

   /**
     * This method will return the TypeCategoryRec representing this product's
     * Family. This value may be null. To get
     * the actual String representation, use the TypeCategoryRec.getDescription()
     * method.
     **/
   public TypeCategoryRec getFamily () {
      return( family ) ;
   } /* getFamily  */

   /**
     * This method will return the TypeProjectRec representing the
     * project that this product belongs to. If this product does not belong
     * to a project, then the blank project for this product's family will be
     * returned. In any event, if you wish to change the project you should never
     * set the description on the project, you should either create a new project
     * or get another project from the TypeCategory class. If this product
     * does not have a family yet and this product does not have a project
     * then this method will return NULL.<br><br>
     *
     * <b>Please Note</b>: If the blank project is not created then this method
     * will create it and save it to the database using the TypeCategory.writeToDatabase method
     **/
   public TypeCategoryRec getProject () {
      if ( project == null ) {
         if ( family != null ) {
            TypeCategoryRec blankProject = TypeCategory.getCategoryFromDescription ( "Default", 
                                                                                     TypeCategory.LEVEL_PROJECT,
                                                                                     family.getInd() ) ;

            if ( blankProject == null ) {
               blankProject = TypeCategory.createTypeCategory ( family.getInd(),
                                                                "Default",
                                                                TypeCategory.LEVEL_PROJECT,
                                                                0,
                                                                "N",
                                                                "QUEST" ) ;

               TypeCategory.writeToDatabase() ;
            }

            projectUpdated = true ;
            this.project = blankProject ;
         } /* if family  */
      }

      return( project ) ;
   } /* getProject  */

   /**
     * This method will return the TypeCategoryRec representing this product's
     * Machine Type. This value may be null. To get
     * the actual String representation, use the TypeCategoryRec.getDescription()
     * method.
     **/
   public TypeCategoryRec getMachine () {
      return( machine ) ;
   } /* getMachine  */

   /**
     * This method will return the TypeCategoryRec representing this product's
     * Model. This value may be null. To get
     * the actual String representation, use the TypeCategoryRec.getDescription()
     * method. 
     **/
   public TypeCategoryRec getModel () {
      return( model ) ;
   } /* getModel  */

   /**
     * This method will return the TypeCategoryRec representing this product's
     * Family Type. This value may be null. To get
     * the actual String representation, use the TypeCategoryRec.getDescription()
     * method.
     **/
   public TypeCategoryRec getSubCategory () {
      return( subCategory ) ;
   } /* getSubCategory  */

   /**
     * This method will save all information that is unsaved to the database.
     * This method will return 0 if no error occurred and the SQLCode of
     * an error if an error did occur
     **/
   public int writeToDatabase () {
      String sql = toSQL () ;
      int    rc  = 0 ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have at least 1 SQL, run it                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( sql != null ) && ( sql.length() > 0 ) ) {
         rc = SqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST, sql ) ;

         if ( rc == 0 ) {
            setLastTouchedOn ( ProductSQL.getDBTimestampFromDatabase() ) ;
            updateRecStatus ( REC_NOCHANGE ) ;
            resetUpdateVariables() ;
         }
      }

      return( rc ) ;
   } /* writeFromDatabase  */

   /**
     * This method will read in all information for this product draft.
     * This method will return 0 if no error occurred and the SQLCode of
     * an error if an error did occur
     */
   public int readFromDatabase () {
      String    rtnName   = "Product.readFromDatabase" ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      int       rc        = 0 ;

      if ( sqlMethod != null ) {
         try {
            CallableStatement stmt  = sqlMethod.createStoredProc ( "QUEST.ProductReader", 3 ) ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up the call                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.setInt               ( 1, getProductInd() ) ;
            stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
            stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            LogSystem.log ( 1, "Reading Product:"+getProductInd() ) ;
            stmt.execute() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the StoredProcRec from the Lob                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Throw an error if need be                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( retRec != null ) && ( retRec.error ) ) {
               LogSystem.beginLogSection ( "ProductReader Returned Error" ) ;
               LogSystem.log ( 1, "sqlCode:   "+retRec.sqlCode ) ;
               LogSystem.log ( 1, "userError: "+retRec.userError ) ;
               LogSystem.log ( 1, "errorStr:  "+retRec.errorStr ) ;
               LogSystem.log ( 1, "error:     "+retRec.error ) ;
               try {
                  rc = new Integer ( retRec.sqlCode ).intValue() ;
               } catch ( Exception e ) {
                  LogSystem.log ( 1, "Could not translate retRec.sqlCode to integer" ) ;
                  rc=-1;
               }
               LogSystem.endLogSection () ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, continue                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get our return object                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               initializeProperties ( (Product) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ) ;
               forceRecStatus ( REC_NOCHANGE ) ;
               rc = 0 ;

            } /* Else if we have no error  */

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            rc = -1 ;
         }
      } /* if sqlMethod is not null  */

      sqlMethod.close() ;

      return( rc ) ;
   } /* readFromDatabase  */

   /**
     *  This method will remove a specific date from this product.
     **/
   public void removeDate ( ProductDate date ) throws IllegalArgumentException  {
      if ( ( dates != null ) && ( dates.indexOf ( date ) > 0 ) )
         date.updateRecStatus ( REC_DELETE ) ;
      else
         throw new IllegalArgumentException ( "Cannot remove date from product because "+
                                              "product does not contain date" ) ;
   } /* removeDate  */

   /**
     *  This method will add a date to this product
     **/
   public void addDate ( ProductDate date ) {
      if ( dates == null )
         dates = new Vector ( 10, 1 ) ;

      dates.addElement ( date ) ;
   } /* addDate  */


   /**
     *  This method will return a vector of all of the Product Dates. Each object
     * in the Vector returned will be of ProductDate. Please note however
     * that each item in this vector will be saved to the PRODUCT.DATE table. Do
     * not ever remove objects from the vector that you receive from getDates
     * and expect those items to be deleted from the database. To delete a
     * date from the product simply call that ProductDate Object's
     * updateRecStatus ( ProductDate.REC_DELETE ) method or use the
     * convience method 'removeDate' in this class.
     **/
   public Vector getDates () {
      if ( dates != null )
         return( (Vector)dates.clone() ) ;
      return( new Vector ( 1 ) ) ;
   }

   /**
     * This method will set the dates for this product. Please note however
     * that each item in this vector will be saved to the PRODUCT.DATE table. Do
     * not ever remove objects from the vector that you receive from getDates
     * and expect those items to be deleted from the database. To delete a
     * date from the product simply call that ProductDate Object's
     * updateRecStatus ( ProductDate.REC_DELETE ) method or use the
     * convience method 'removeDate' in this class.
     **/
   public void setDates ( Vector dates ) {
      this.dates = dates ;

      if ( ( dates != null ) && ( dates.size() > 0 ) ) {
         int i     = 0 ;
         int size  = dates.size() ;

         while ( i < size ) {
            ((ProductDate)dates.elementAt(i)).setProductInd ( getProductInd() ) ;
            i++ ;
         }

         firePropertyChanged( DATES) ;
      }
   }


   /**
     *  This method will remove an XRef entry from this product.
     **/
   public void removeXRef ( XRef xRefObject ) throws IllegalArgumentException {
      if ( ( xRef != null ) && ( xRef.indexOf ( xRefObject ) > 0 ) )
         xRefObject.updateRecStatus ( REC_DELETE ) ;
      else
         throw new IllegalArgumentException ( "Cannot remove XRef from product because "+
                                              "product does not contain XRef" ) ;
   }


   /**
     *  This method will add an Xref object to our product.
     **/
   public void addXRef ( XRef xRefObject ) {
      if ( xRef == null ) {
         xRef = new Vector ( 10, 1 ) ;
      }

      xRef.addElement ( xRefObject ) ;
   }

   /**
     * This method will create a ProductDraft Object from this Product object.
     * This method will return a CreateProductDraft object
     **/
   public CreateProductDraft createProductDraft () {
      return( ProductDraft.createProductDraft ( this ) ) ;
   }

   /**
     * This method will immeditaly check out this product record. Please
     * keep in mind that this method automatically overwrites the current
     * checked out state in the database.
     *
     * @Param userId The User Id that checks the product Out. NULL if no one
     **/
   public void checkOut ( String userId ) throws Exception {
      if ( productInd != 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "checkOut ( "+userId+" )", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            setCheckedOutBy ( userId ) ;
            if ( userId != null )
               stmt.executeUpdate ( "UPDATE PRODUCT.PRODUCT SET CHECKEDOUTBY = '"+userId+"' where productInd = "+productInd ) ;
            else
               stmt.executeUpdate ( "UPDATE PRODUCT.PRODUCT SET CHECKEDOUTBY = NULL where productInd = "+productInd ) ;

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            throw ( e ) ;
         }

         sqlMethod.close() ;
      } else
         throw new IllegalArgumentException ( "Cannot lock a product without a Product Indicator" ) ;
   }

   /**
     * This method will make a complete clone of this product except
     * for the ProductInd property. If you wish to clone this product
     * and the ProductInd property, please use the cloneAll method
     **/
   public Object clone () {
      Product returnRec = new Product() ;

      returnRec.customer = customer ;
      returnRec.warranty = warranty ;
      returnRec.offering = offering ;
      returnRec.type = type ;
      returnRec.prodNum = prodNum ;
      returnRec.partNum = partNum ;
      returnRec.description = description ;
      returnRec.baseModel = baseModel ;
      returnRec.input = input ;
      returnRec.source = source ;
      returnRec.checkedOutBy = checkedOutBy ;
      returnRec.createdOn = createdOn ;
      returnRec.createdBy = createdBy ;
      returnRec.lastTouchedOn = lastTouchedOn ;
      returnRec.lastTouchedBy = lastTouchedBy ;
      returnRec.xRef = xRef ;
      returnRec.locale = locale ;
      returnRec.dates = dates ;
      returnRec.brand = brand ;
      returnRec.family = family ;
      returnRec.model = model ;
      returnRec.machine = machine ;
      returnRec.project = project ;
      returnRec.subCategory = subCategory ;

      return( returnRec ) ;
   }


   /**
     *  This method will initialize our properties based on the 
     * Product object passed in. 
     **/
   public void initializeProperties ( Product product ) {
      this.customer      = product.customer ;
      this.warranty      = product.warranty ;
      this.offering      = product.offering ;
      this.type          = product.type ;
      this.prodNum       = product.prodNum ;
      this.partNum       = product.partNum ;
      this.description   = product.description ;
      this.baseModel     = product.baseModel ;
      this.input         = product.input ;
      this.source        = product.source ;
      this.checkedOutBy  = product.checkedOutBy ;
      this.createdOn     = product.createdOn ;
      this.createdBy     = product.createdBy ;
      this.lastTouchedOn = product.lastTouchedOn ;
      this.lastTouchedBy = product.lastTouchedBy ;
      this.xRef          = product.xRef ;
      this.locale        = product.getLocale() ;
      this.dates         = product.getDates() ;
      this.opicmImport   = product.opicmImport ;

      if ( product.commentHistory != null )
         commentHistory = (Vector)product.commentHistory.clone() ;

      if ( product.brand != null )
         setBrand ( product.brand ) ;

      if ( product.family != null )
         setFamily ( product.family ) ;

      if ( product.model != null )
         setModel ( product.model ) ;

      if ( product.machine != null )
         setMachine ( product.machine ) ;

      if ( product.project != null )
         setProject (product.project ) ;

      if ( product.subCategory != null )
         setSubCategory ( product.subCategory ) ;

      if ( product.partNum != null )
         setPartNumber ( product.partNum ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set us as the product indicator for the dates           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( dates != null ) {
         int i     = 0 ;
         int size  = dates.size() ;

         while ( i < size ) {
            ((ProductDate)dates.elementAt(i)).setProductInd ( getProductInd() ) ;
            i++ ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our Locale Rec status so it will save               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      locale.updateRecStatus ( REC_INSERT ) ;


   }

   /**
     *  This method will return the publish property
     **/
   public String getPublish () {
      return( publish ? "Y" : "N" ) ;
   }


   /**
     *  This method will set the publish property
     *
     * @param publish The Publish flag (Either Y or N, defaults to Y)
     **/
   public void setPublish ( String flag ) {
      if ( flag.equals ( "N" ) )
         publish = false ;
      else
         publish = true ;
   }


   /**
     *  This method will set the timestamp for the time that this
     * product object was last touched
     *
     * @param lastTouchedOn The DB/2 Timestamp
     **/
   protected void setLastTouchedOn ( String lastTouchedOn ) {
      this.lastTouchedOn = lastTouchedOn ;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will set the user id of the person who
     * last touched this record
     *
     * @param userId The User Id
     **/
   public void setLastTouchedBy ( String userId ) {
      this.lastTouchedBy = userId ;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will set what brand this product belongs to
     *
     * @brand The TypeCategoryRec representing the brand
     **/
   public void setBrand ( TypeCategoryRec brand ) throws IllegalArgumentException {
      if ( this.brand == null ) {
         this.brand = brand ;
         brandUpdated = true ;
      } else if ( brand != null ) {
         if ( this.brand.getInd() != brand.getInd() ) {
            this.brand = brand ;
            brandUpdated = true ;
            firePropertyChanged ( BRAND ) ;
         }
      } else
         this.brand=null ;
   }

   /**
     *  This method will set what family this product belongs to
     *
     * @family The TypeCategoryRec representing the family
     **/
   public void setFamily ( TypeCategoryRec family ) throws IllegalArgumentException {
      if ( family != null ) {
         if ( this.family == null ) {
            this.family = family ;
            familyUpdated = true ;
            firePropertyChanged ( FAMILY ) ;
         } else if ( this.family.getInd() != family.getInd() ) {
            this.family = family ;
            familyUpdated = true ;
            firePropertyChanged ( FAMILY ) ;
         }
      } else
         this.family = null ;

   }

   /**
     *  This method will set what machine this product belongs to
     *
     * @machine The TypeCategoryRec representing the machine
     **/
   public void setMachine ( TypeCategoryRec machine ) throws IllegalArgumentException {
      if ( machine != null ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we don't have a Machine yet, update ours             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( this.machine == null ) {
            machineUpdated = true ;
            this.machine = machine ;
            firePropertyChanged ( MACHINE ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we do have a machine and it is different than ours,  */
         /*   then update ours                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( this.machine.getInd() != machine.getInd() ) {
            this.machine = machine ;
            machineUpdated = true ;
            firePropertyChanged ( MACHINE ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the Product Number automatically                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( model != null )
            prodNum = machine.toString()+"-"+model.toString() ;
         else
            prodNum = machine.toString() ;

      } else
         this.machine = null ;

   }

   /**
     *  This method will set what model this product belongs to
     *
     * @model The TypeCategoryRec representing the model
     **/
   public void setModel ( TypeCategoryRec model ) throws IllegalArgumentException {
      if ( model != null ) {

         /*------------------------------------------------------------*/
         /*                                                            */
         /*   If we do not have a TypeCategoryRec yet for the model    */
         /* then use the one passed in. However, if we already have a  */
         /* TypeCategoryRec then we just simply change the description */
         /* of it .                                                    */
         /*                                                            */
         /*------------------------------------------------------------*/
         if ( this.model == null ) {
            this.model = model ;
         } else {
            this.model.setDescript ( model.toString() ) ;
            this.model.setIsDraft  ( model.getIsDraft() ) ;
            this.model.setParentCatInd ( model.getParentCatInd() ) ;

            model = this.model ;
         }

         modelUpdated = true ;
         firePropertyChanged ( MODEL ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the product automatically                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( machine != null )
            prodNum = machine.toString()+"-"+model.toString() ;
         else
            prodNum = model.toString() ;

      } else
         this.model = null ;

   }

   /**
     *  This method will set the model to the TypeCategoryRec passed in. This
     * is very dangerous and you should never use this method. This method is
     * only for the ProductDraft object. Noramlly this object tries to reuse old model
     * information by simply changing the text of the TypeCategoryRec. This method will
     * completely remove the old information without deleting it from the database.
     **/
   protected void hardSetModel ( TypeCategoryRec categoryRec ) {
      this.model = categoryRec ;
      modelUpdated= true ;
   }

   /**
     *  This method will set the part number to the TypeCategoryRec passed in. This
     * is very dangerous and you should never use this method. This method is
     * only for the ProductDraft object. Noramlly this object tries to reuse old part number
     * information by simply changing the text of the TypeCategoryRec. This method will
     * completely remove the old information without deleting it from the database.
     **/
   protected void hardSetPartNumber ( TypeCategoryRec categoryRec ) {
      this.partNum = categoryRec ;
      partNumUpdated= true ;
   }



   /**
     *  This method will set what project this product belongs to
     *
     * @project The TypeCategoryRec representing the project
     **/
   public void setProject ( TypeCategoryRec project ) throws IllegalArgumentException {
      this.project = project ;
      updateRecStatus ( REC_UPDATE ) ;
      projectUpdated = true ;
      firePropertyChanged ( PROJECT ) ;
   }

   /**
     *  This method will set what subCategory this product belongs to
     *
     * @subCategory The TypeCategoryRec representing the subCategory
     **/
   public void setSubCategory ( TypeCategoryRec subCategory ) throws IllegalArgumentException {
      if ( ( subCategory != null ) && ( this.subCategory != null ) ) {
         if ( this.subCategory.getInd() != subCategory.getInd() ) {
            this.subCategory = subCategory ;
            machineUpdated = true ;
            updateRecStatus ( REC_UPDATE ) ;
            firePropertyChanged ( SUBCATEGORY ) ;
         }
      } else {
         this.subCategory = subCategory ;
         machineUpdated = true ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( SUBCATEGORY ) ;
      }
   }


   /**
     *  This method will set the user id of the person who created this record. Normally
     * you should never use this method unless you are working with a new product.
     *
     * @param createdBy The user id of the user who created this product
     **/
   public void setCreatedBy ( String createdBy ) throws IllegalArgumentException {
      if ( ( createdBy != null ) && ( createdBy.length() <= 12 ) ) {
         this.createdBy = createdBy ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( CREATEDBY ) ;
      } else
         throw new IllegalArgumentException ( "Cannot set Created By to NULL or set to > 12 characters. Tried to set created by to :"+createdBy+":" ) ;
   }


   /**
     *  This method will set the ustime that this record was created. Normally
     * you should never use this method unless you are working with a new product.
     *
     * @param createdOn The DB/2 Timestamp
     **/
   public void setCreatedOn ( String createdOn ) throws IllegalArgumentException {
      if ( createdOn != null ) {
         this.createdOn = createdOn ;
         updateRecStatus ( REC_UPDATE ) ;
         firePropertyChanged ( CREATEDON ) ;
      } else
         throw new IllegalArgumentException ( "Cannot set Created By to NULL. Tried to set created by to :"+createdOn+":" ) ;
   }

   /**
     *  This method will return the OPIC/M Import flag. If the flag is FALSE
     * then this product should never be over written by the prod gen batch
     *
     * @return boolean
     **/
   public boolean getOPICMImport () {
      return( opicmImport ) ;
   }

   /**
     *  This method will set the OPIC/M Import flag. Be very careful when using
     * this API since it tells ProdGen if it should overwrite data in this product
     * or not. Setting this flag to FALSE will tell ProdGen never to import data
     * over this product. Setting this flag to TRUE tells ProdGen that it is Ok
     * to overwrite this data 
     *
     * @param opicmImport TRUE/FALSE
     **/
   public void setOPICMImport ( boolean opicmImport ) {
      this.opicmImport = opicmImport ;
      updateRecStatus ( REC_UPDATE ) ;
      firePropertyChanged ( OPICMIMPORT ) ;
   }

   /**
     *  This method will set the product type. Please note that the integer passed in
     * must already reside in the PRODUCT.TYPEPRODUCT table.
     **/
   public void setProductType ( int productType ) {
      this.type = productType ;
      updateRecStatus ( REC_UPDATE ) ;
      firePropertyChanged ( PRODUCTTYPE ) ;
   }

   /**
     *  This method will return the SQL needed to save the information in this object
     * to the Quest Database. Each SQL String returned will have the SqlRunner.END_DELIM
     * appended to it so that the SQL may be passed to the SqlRunner stored procedure
     *
     * @return String
     **/
   public String toSQL () {
      String sql = "" ;                                                               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Generate the keys if needed                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( generateKeys() ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, generate the SQL needed to save this product to  */
         /* the PRODUCT.PRODUCT table                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( getRecStatus() ) {
            case REC_DELETE :
               int modelInd = 0 ;

               if ( partNum != null )
                  modelInd = partNum.getInd() ;
               else if ( model != null )
                  modelInd = model.getInd() ;

               sql = "UPDATE PRODUCT.PRODUCT "+
                     "SET PUBLISH = 'N',"+
                     "    LASTTOUCHEDBY = '"+lastTouchedBy+"', "+
                     "    LASTTOUCHEDON = CURRENT TIMESTAMP "+
                     "WHERE PRODUCTIND = "+productInd+""+SqlRunner.END_DELIM+
                     ""+
                     "DELETE FROM TIGRIS.CATEGORY WHERE TYPECATIND = "+modelInd+""+SqlRunner.END_DELIM+
                     ""+
                     "DELETE FROM QUEST.CATEGORY WHERE TYPECATIND = "+modelInd; ;
               break ;

            case REC_INSERT :

               sql = "INSERT INTO PRODUCT.PRODUCT "+
                     "( PRODUCTIND,     PRODNUM,         DESCRIPTION,      BASEMODEL, TYPECUSTOMERIND, "+
                     "  TYPEPRODUCTIND, TYPEOFFERINGIND, TYPEWARRANTYIND,  INPUT,     SOURCE, "+
                     "  PUBLISH,        CHECKEDOUTBY,    IMPORTOPICM,      CREATEDON, CREATEDBY, "+
                     "  LASTTOUCHEDON,  LASTTOUCHEDBY ) values ( "+
                     " "+productInd+", "+
                     " '"+Text.cleanDBString ( prodNum )+"', "+
                     " '"+Text.cleanDBString ( description )+"', "+
                     " "+Text.cleanNullableDBString ( baseModel )+", "+
                     " "+customer+", "+
                     " "+type+", "+
                     " "+offering+", "+
                     " "+warranty+", "+
                     " '"+Text.cleanDBString ( input ) +"', "+
                     " '"+Text.cleanDBString ( source ) +"', "+
                     " 'Y', "+
                     " NULL, "+
                     " 'Y', "+
                     " CURRENT TIMESTAMP, "+
                     " '"+createdBy+"', "+
                     " CURRENT TIMESTAMP, "+
                     " '"+lastTouchedBy+"' )" ;
               break ;

            case REC_UPDATE :
               sql = "UPDATE PRODUCT.PRODUCT "+
                     "SET PRODNUM     = '"+Text.cleanDBString ( prodNum )+"', "+
                     "    DESCRIPTION     = '"+Text.cleanDBString ( description )+"', "+
                     "    BASEMODEL       = '"+Text.cleanDBString ( baseModel )+"', "+
                     "    TYPECUSTOMERIND = "+customer+", "+
                     "    TYPEPRODUCTIND  = "+type+", "+
                     "    TYPEOFFERINGIND = "+offering+", "+
                     "    TYPEWARRANTYIND = "+warranty+", "+
                     "    INPUT           = '"+Text.cleanDBString ( input )+"', "+
                     "    SOURCE          = '"+Text.cleanDBString ( source )+"', "+
                     "    PUBLISH         = '"+(publish ? "Y" : "N")+"', "+
                     "    CHECKEDOUTBY    = "+Text.cleanNullableDBString ( checkedOutBy )+", "+
                     "    IMPORTOPICM     = '"+(opicmImport ? "Y" : "N")+"', "+
                     "    LASTTOUCHEDON   = CURRENT TIMESTAMP, "+
                     "    LASTTOUCHEDBY   = '"+Text.cleanDBString ( lastTouchedBy )+"' "+
                     "WHERE PRODUCTIND = "+productInd ;
               break ;
         }

         if ( sql.length() > 0 )
            sql=sql+SqlRunner.END_DELIM ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, generate the SQL needed to save the dates          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql=sql+datesToSQL () ;                                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Generate the SQL needed to save the Cross Products      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql=sql+xRefToSQL () ;                                 

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Generate the SQL needed to save the Locale              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql=sql+localeToSQL () ;                                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Generate the SQL needed to save the categorization      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql=sql+categoryToSQL() ;
      } else {
         String error = "Could not generate a Product Indicator. No INSERT can occur. Please check logs for more info" ;
         LogSystem.log ( 1, error ) ;
      }



      return( sql ) ;
   }


   /**
     *  This method will return the SQL needed to save all dates for this product.
     *  Each SQL String returned will have the SqlRunner.END_DELIM
     *  appended to it so that the SQL may be passed to the SqlRunner stored procedure
     *
     **/
   protected String datesToSQL () {
      String sql   = "" ;                              
      Vector dates = getDates() ;
      int         i    = 0 ;
      int         size = dates.size() ;
      ProductDate date = null ;

      sql="DELETE FROM PRODUCT.DATES WHERE PRODUCTIND = "+getProductInd()+SqlRunner.END_DELIM ;
      while ( i < size ) {

         date = ((ProductDate)dates.elementAt(i)) ;
         date.setProductInd ( getProductInd() ) ;
         sql+=date.toSQL () ;

         i++ ;
      }

      return( sql ) ;
   }


   /**
     *  This method will return the SQL needed to save all XRef objects for
     * this product. Each SQL String returned will have the SqlRunner.END_DELIM
     *  appended to it so that the SQL may be passed to the SqlRunner stored procedure
     *
     * @returns String
     **/
   protected String xRefToSQL () {
      String sql = "" ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only generate SQL if there are changes                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( POMUtil.vectorContainsChanges ( xRef ) ) {
         int  i       = 0 ;
         int  size    = xRef.size() ;
         XRef xRefRec = null ;

         while ( i < size ) {

            xRefRec = (XRef)xRef.elementAt(i) ;

            if ( xRefRec.modified() )
               sql+=xRefRec.toSQL () ;

            i++ ;
         }
      }

      return( sql ) ;
   }


   /**
     *  This method will return the SQL needed to save all locale information (Country
     * and Geography) for this product. Each SQL String returned will have the SqlRunner.END_DELIM
     *  appended to it so that the SQL may be passed to the SqlRunner stored procedure
     *
     * @returns String
     **/
   protected String localeToSQL () {
      String sql = "" ;                                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only generate the SQL if the locale was modified        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( locale != null ) && ( locale.modified() ) ) {
         Vector countries = locale.getCountryList() ;
         Vector geos      = locale.getGeoListContainingAll() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, we need to delete all dates out of the table,    */
         /* we then will reinsert them                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql = "DELETE FROM PRODUCT.GEO WHERE PRODUCTIND = "+productInd+SqlRunner.END_DELIM ;
         sql+="DELETE FROM PRODUCT.COUNTRY WHERE PRODUCTIND = "+productInd+SqlRunner.END_DELIM ;                                                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this locale is representing WORLD-WIDE then we only  */
         /* need to do 1 insert statement, otherwise we must process  */
         /* each country and geography                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( locale.isWorldWide() ) {
            sql+="INSERT INTO PRODUCT.GEO ( PRODUCTIND, GEOIND ) "+
                 "values ( "+productInd+", "+TypeList.GEO_WORLD_WIDE+" )"+SqlRunner.END_DELIM ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, it wasn't world wide so we must save everything     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, save the countries                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( countries != null ) && ( countries.size() > 0 ) ) {
               int                i       = 0 ;
               int                size    = countries.size() ;
               TypeCountryCodeRec country = null ;

               while ( i < size ) {
                  country = (TypeCountryCodeRec) countries.elementAt(i) ;
                  sql+="INSERT INTO PRODUCT.COUNTRY ( PRODUCTIND, COUNTRYCODEIND ) "+
                       "values ( "+productInd+", "+country.getInd()+" )"+SqlRunner.END_DELIM ;

                  i++ ;
               } /* while  */


            } /* if  */

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, save the geographies                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( geos != null ) && ( geos.size() > 0 ) ) {
               int          i    = 0 ;
               int          size = geos.size() ;
               TypeGeoRec   geo  = null ;

               while ( i < size ) {
                  geo = (TypeGeoRec) geos.elementAt(i) ;
                  sql+="INSERT INTO PRODUCT.GEO ( PRODUCTIND, GEOIND ) "+
                       "values ( "+productInd+", "+geo.getInd()+" )"+SqlRunner.END_DELIM ;

                  i++ ;
               } /* while  */


            } /* if  */
         } /* else, not world wide  */
      } /* if modified  */

      return( sql ) ;
   }


   /**
     *  This method will save all of our category information. After the information is saved,
     * this method will issue a delete of all Models, Machines and Part Numbers that are no longer referenced
     * in the PRODUCT.CATEGORY and TIGRIS.CATEGORY tables. This will clean up all models, machines and 
     **/
   protected String categoryToSQL () {
      String sql = "" ;                          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the brand was updated, change it                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( brandUpdated ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, delete the current brand if the product status   */
         /* is not INSERT. We will not have an entry in the database  */
         /* if the rec status is INSERT.                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getRecStatus() != REC_INSERT )
            sql+= "DELETE FROM PRODUCT.CATEGORY "+
                  "WHERE PRODUCTIND = "+productInd+" AND "+
                  "      TYPECATIND = (select b.typecatind "+
                  "                    from product.category a, "+
                  "                         product.typecategory b "+
                  "                    where productInd      = "+productInd+" and "+
                  "                          a.typecatind    = b.typecatind and "+
                  "                          b.categorylevel = 'brand')"+SqlRunner.END_DELIM ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create our new entry                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql+="INSERT INTO PRODUCT.CATEGORY "+
              "(PRODUCTIND, TYPECATIND) values ("+
              ""+productInd+", "+
              ""+brand.getInd()+" )"+SqlRunner.END_DELIM ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the family was updated, change it                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( familyUpdated ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, delete the current family if the product status  */
         /* is not INSERT. We will not have an entry in the database  */
         /* if the rec status is INSERT.                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getRecStatus() != REC_INSERT )
            sql+= "DELETE FROM PRODUCT.CATEGORY "+
                  "WHERE PRODUCTIND = "+productInd+" AND "+
                  "      TYPECATIND = (select b.typecatind "+
                  "                    from product.category a, "+
                  "                         product.typecategory b "+
                  "                    where productInd      = "+productInd+" and "+
                  "                          a.typecatind    = b.typecatind and "+
                  "                          b.categorylevel = 'family')"+SqlRunner.END_DELIM ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create our new entry                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql+="INSERT INTO PRODUCT.CATEGORY "+
              "(PRODUCTIND, TYPECATIND) values ("+
              ""+productInd+", "+
              ""+family.getInd()+" )"+SqlRunner.END_DELIM ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the project was updated, change it                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( project == null ) || ( projectUpdated ) ) {
         TypeCategoryRec project = getProject() ; // This should create a new Project if it was null

         if ( project != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the SQL to save the machine type category           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sql+=project.toSQL() ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, delete the current brand if the product status   */
            /* is not INSERT. We will not have an entry in the database  */
            /* if the rec status is INSERT.                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( getRecStatus() != REC_INSERT )
               sql+= "DELETE FROM PRODUCT.CATEGORY "+
                     "WHERE PRODUCTIND = "+productInd+" AND "+
                     "      TYPECATIND = (select b.typecatind "+
                     "                    from product.category a, "+
                     "                         product.typecategory b "+
                     "                    where productInd      = "+productInd+" and "+
                     "                          a.typecatind    = b.typecatind and "+
                     "                          b.categorylevel = '"+TypeCategory.LEVEL_PROJECT+"')"+SqlRunner.END_DELIM ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, create our new entry                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sql+="INSERT INTO PRODUCT.CATEGORY "+
                 "(PRODUCTIND, TYPECATIND) values ("+
                 ""+productInd+", "+
                 ""+project.getInd()+" )"+SqlRunner.END_DELIM ;                                           


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, delete all machines that are not referenced in     */
            /* PRODUCT.CATEGORY, PRODRAFT.CATEGORY, TIGRIS.CATEGORY or   */
            /* QUEST.CATEGORY                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            //sql+="DELETE FROM PRODUCT.TYPECATEGORY "+
            //     "WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_PROJECT+"' AND "+
            //     "TYPECATIND NOT IN ( SELECT TYPECATIND FROM PRODUCT.CATEGORY) AND "+
            //     "TYPECATIND NOT IN ( SELECT TYPECATIND FROM TIGRIS.CATEGORY) "+SqlRunner.END_DELIM ;

         } /* if projectUpdated  */
      } /* if projectUpdateds  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the machine was updated, change it                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( machineUpdated ) {
         boolean isOption = ( getProductType() == TypeList.PRODUCTTYPE_OPTION ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the SQL to save the machine type category           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( isOption )
            sql+=getSubCategory().toDraftSQL() ;
         else
            sql+=getMachine().toDraftSQL() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, delete the current brand if the product status   */
         /* is not INSERT. We will not have an entry in the database  */
         /* if the rec status is INSERT.                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getRecStatus() != REC_INSERT )
            sql+= "DELETE FROM PRODUCT.CATEGORY "+
                  "WHERE PRODUCTIND = "+getProductInd()+" AND "+
                  "      TYPECATIND = (select b.typecatind "+
                  "                    from prodraft.category a, "+
                  "                         prodraft.typecategory b "+
                  "                    where a.productInd      = "+getProductInd()+" and "+
                  "                          a.typecatind    = b.typecatind and "+
                  "                          b.categorylevel = IN ('"+TypeCategory.LEVEL_MACHINE+"', '"+TypeCategory.LEVEL_SUBCATEGORY+"')"+SqlRunner.END_DELIM ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create our new entry                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( isOption )
            sql+="INSERT INTO PRODUCT.CATEGORY "+
                 "(PRODUCTIND, TYPECATIND) values ("+
                 ""+getProductInd()+", "+
                 ""+getSubCategory().getInd()+" )"+SqlRunner.END_DELIM ;
         else
            sql+="INSERT INTO PRODUCT.CATEGORY "+
                 "(PRODUCTIND, TYPECATIND) values ("+
                 ""+getProductInd()+", "+
                 ""+getMachine().getInd()+" )"+SqlRunner.END_DELIM ;                                           



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, delete all machines that are not referenced in     */
         /* PRODUCT.CATEGORY, PRODRAFT.CATEGORY, TIGRIS.CATEGORY or   */
         /* QUEST.CATEGORY                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         //sql+="DELETE FROM PRODUCT.TYPECATEGORY "+
         //     "WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_MACHINE+"' AND "+
         //     "TYPECATIND NOT IN ( SELECT TYPECATIND FROM PRODUCT.CATEGORY) AND "+
         //     "TYPECATIND NOT IN ( SELECT TYPECATIND FROM TIGRIS.CATEGORY) "+SqlRunner.END_DELIM ;


      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the part number was updated, change it               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( partNumUpdated ) {

         if ( partNum != null ) {
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the SQL to update the Model information             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sql+=partNum.toSQL () ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, create our new entry only if we are inserting this */
            /* as a new product. If we are updating, we have already     */
            /* updated the Model description text in save for the        */
            /* TypeCategoryRec                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( getRecStatus() == REC_INSERT )
               sql+="INSERT INTO PRODUCT.CATEGORY "+
                    "(PRODUCTIND, TYPECATIND) values ("+
                    ""+productInd+", "+
                    ""+partNum.getInd()+" )"+SqlRunner.END_DELIM ;
         } else
            sql+="DELETE FROM PRODUCT.CATEGORY WHERE PRODUCTIND = "+getProductInd()+" AND "+
                 "TYPECATIND IN (SELECT TYPECATIND FROM PRODUCT.TYPECATEGORY WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_PARTNUMBER+"')"+SqlRunner.END_DELIM ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the model, part Nated, change it                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( modelUpdated ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the SQL to update the Model information             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( model != null ) {
            sql+=model.toSQL () ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, create our new entry only if we are inserting this */
            /* as a new product. If we are updating, we have already     */
            /* updated the Model description text in save for the        */
            /* TypeCategoryRec                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( getRecStatus() == REC_INSERT )
               sql+="INSERT INTO PRODUCT.CATEGORY "+
                    "(PRODUCTIND, TYPECATIND) values ("+
                    ""+productInd+", "+
                    ""+model.getInd()+" )"+SqlRunner.END_DELIM ;
         } else
            sql+="DELETE FROM PRODUCT.CATEGORY WHERE PRODUCTIND = "+productInd+" AND "+
                 "TYPECATIND IN (SELECT TYPECATIND FROM PRODUCT.TYPECATEGORY WHERE CATEGORYLEVEL = '"+TypeCategory.LEVEL_MODEL+"')"+SqlRunner.END_DELIM ;

      }
      return( sql ) ;
   }



   /**
     *  This method will generate the primary keys for this class if the primary keys
     * have not yet been assigned.
     **/
   protected boolean generateKeys () {
      if ( productInd == 0 )
         setProductInd ( Counter.getCounter ( LogSystem.getInstance(), "PRODUCT" ) ) ;
      return( productInd != 0 ) ;
   }



   /**
     *  This method will return a simple string describing the entire product and all
     * objects
     *
     * @returns String
     **/
   public String toOut () {
      String retStr="" ;

      /*------------------------------------*/
      /*                                    */
      /*  Product Indicator and Description */
      /*                                    */
      /*------------------------------------*/
      retStr+="Product Ind:            "+getProductIndNow()+"\n" ;
      retStr+="Description:            "+getDescription()+"\n" ;                                        


      /*--------*/
      /*        */
      /*  Brand */
      /*        */
      /*--------*/
      if ( getBrand() != null )
         retStr+="Brand:                  "+getBrand().toString()+"\n" ;
      else
         retStr+="Brand:                  NONE\n" ;                                      

      /*---------*/
      /*         */
      /*  Family */
      /*         */
      /*---------*/
      if ( getFamily() != null )
         retStr+="Family:                 "+getFamily().toString()+"\n" ;
      else
         retStr+="Family:                  NONE\n" ;                                    

      /*----------*/
      /*          */
      /*  Project */
      /*          */
      /*----------*/
      if ( getProject() != null ) {
         if ( project.toString().equals ( "Default" ) )
            retStr+="Project:                BLANK PROJECT\n" ;
         else
            retStr+="Project:                "+getProject().toString()+"\n" ;
      } else
         retStr+="Project:                NULL!\n" ;                                    

      /*----------*/
      /*          */
      /*  Machine */
      /*          */
      /*----------*/
      try {
         retStr+="Machine:                "+getMachine().toString()+"\n" ;
         retStr+="Model:                  "+getModel().toString()+"\n" ;
      } catch ( Exception e ) {
         retStr+="Machine:                NONE\n" ;
         retStr+="Model:                  NONE\n" ;
      }


      /*---------------*/
      /*               */
      /*  Part Number  */
      /*               */
      /*---------------*/
      try {
         retStr+="Part Number:            "+getPartNumber().toString()+"\n" ;
      } catch ( Exception d1 ) {
         retStr+="Part Number:            NONE\n" ;
      }


      /*-------------*/
      /*             */
      /*  Other Info */
      /*             */
      /*-------------*/
      retStr+="publish:                "+getPublish()+"\n" ;
      retStr+="Checked Out By:         "+getCheckedOutBy()+"\n" ;
      retStr+="Source:                 "+getSource()+"\n" ;
      retStr+="Offering:               "+getOffering()+"\n" ;
      retStr+="Customer:               "+getCustomer()+"\n" ;
      retStr+="Warranty:               "+getWarranty()+"\n" ;

      if ( ( xRef != null ) && ( xRef.size() > 0 ) ) {
         int i    = 0 ;
         int size = xRef.size() ;

         while ( i < size ) {
            retStr+=((XRef)xRef.elementAt(i)).toOut()+"\n\n" ;
            i++ ;
         }

      } else
         retStr+="\nNo XRef Objects\n" ;

      if ( locale != null )
         retStr+=locale.toOut()+"\n\n" ;
      else
         retStr+="No locale information\n\n" ;

      return( retStr ) ;
   }

   /**
     *  This method will return a blank ProductDraft object (even though the method is
     * coded to return a Product object) that matches the Product Indicator passed and 
     * is initialized with the Product Indicator passed. To see the data in the record
     * you must call 'ProductDraft.readFromDatabase'
     *
     * @param productInd The Product Indicator to look for.
     *
     * @return A blank ProductDraft object.
     **/
   public static Product find ( int productInd ) {
      ProductDraft productDraft = null ;
      SQLMethod    sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "ProductDraft.find ( "+productInd+" )", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT PRODUCTIND FROM PRODUCT.PRODUCT WHERE PRODUCTIND = "+productInd+" FOR FETCH ONLY" ) ;

         if ( rs.next() )
            productDraft = new ProductDraft ( productInd ) ;

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      return( (Product)productDraft ) ;
   }



   /**
     *  This method will return a blank ProductDraft object (even though the method is
     * coded to return a Product object) that matches the categorization passed and 
     * is initialized with the Product Indicator of the product found. To see the data in the record
     * you must call 'ProductDraft.readFromDatabase'
     *
     * @param brand THe brand indicator of the Brand
     * @param familyInd The Family indicator of the family
     * @param machineInd The Machine indicator of the machine
     * @param modelInd The model indiator of the model
     *
     * @return A blank ProductDraft object.
     **/
   public static Product findFromCategory ( int brandInd, int familyInd, int machineInd, int modelInd ) {
      ProductDraft productDraft = null ;
      SQLMethod    sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "ProductDraft.find ( "+familyInd+", "+machineInd+", "+modelInd+" )", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT PRODUCTIND "+
                                              "FROM   PRODUCT.PRODUCT "+
                                              "WHERE "+
                                              "PRODUCTIND IN ( "+
                                              "    SELECT PRODUCTIND "+
                                              "    FROM   PRODUCT.CATEGORY "+
                                              "    WHERE  TYPECATIND = "+familyInd+") AND "+
                                              "PRODUCTIND IN ( "+
                                              "    SELECT PRODUCTIND "+
                                              "    FROM   PRODUCT.CATEGORY "+
                                              "    WHERE  TYPECATIND = "+brandInd+") AND "+
                                              "PRODUCTIND IN ( "+
                                              "    SELECT PRODUCTIND "+
                                              "    FROM   PRODUCT.CATEGORY "+
                                              "    WHERE  TYPECATIND = "+machineInd+") AND "+
                                              "PRODUCTIND IN ( "+
                                              "    SELECT PRODUCTIND "+
                                              "    FROM   PRODUCT.CATEGORY "+
                                              "    WHERE  TYPECATIND = "+modelInd+") FOR FETCH ONLY " ) ;
         if ( rs.next() )
            productDraft = new ProductDraft ( rs.getInt(1) ) ;

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      return( (Product)productDraft ) ;
   }


   /**
     *  This method will return a blank ProductDraft object (even though the method is
     * coded to return a Product object) that matches the categorization passed and 
     * is initialized with the Product Indicator of the product found. To see the data in the record
     * you must call 'ProductDraft.readFromDatabase'
     *
     * @param brand The exact matching description of the brand
     * @param family The exact matching description of the family
     * @param machine The exact matching description of the machine
     * @param model The exact matching description of the model
     *
     * @return A blank ProductDraft object.
     **/
   public static Product findFromCategory ( String brand, String family, String machine, String model ) {
      TypeCategoryRec brandRec   = TypeCategory.getCategory ( brand, TypeCategory.LEVEL_BRAND ) ;
      TypeCategoryRec familyRec  = TypeCategory.getCategory ( family, TypeCategory.LEVEL_FAMILY ) ;
      TypeCategoryRec machineRec = TypeCategory.getCategory ( machine, TypeCategory.LEVEL_MACHINE ) ;
      TypeCategoryRec modelRec   = TypeCategory.getCategory ( model, TypeCategory.LEVEL_MODEL ) ;

      if ( ( familyRec  != null ) &&
           ( machineRec != null ) &&
           ( modelRec   != null ) )
         return( findFromCategory ( brandRec.getInd(), familyRec.getInd(), machineRec.getInd(), modelRec.getInd() ) ) ;
      return( null ) ;
   }


   /**
     *  This method will force all POM Objects to be the rec status passed. Please
     * use this carefully. If something should be updated and you call this method, it could
     * no longer be saved in a call to writeToDatabase()
     **/
   public void forceRecStatus ( int recStatus ) {
      updateRecStatus ( REC_NOCHANGE ) ;
      updateRecStatus ( recStatus ) ;

      if ( recStatus == REC_NOCHANGE )
         resetUpdateVariables() ;
   }


   /**
     *  This method will set our Machine/Model categorization from the draft object passed in. it
     * will try and find a match for the draft machine/model in the active tables. If a match is
     * found then the machine/model will be set to the matching active machine/model. If no match
     * is found, then a new Machine or Model will be created and set.
     **/
   protected void createCategoryFromDraft ( ProductDraft productDraft ) {
      TypeCategoryRec currentMachine     = null ;
      TypeCategoryRec currentModel       = null ;
      TypeCategoryRec currentProject     = null ;
      TypeCategoryRec currentSubCategory = null ;
      TypeCategoryRec currentPartNum     = null ;
      TypeCategoryRec draftMachine       = productDraft.getMachine() ;
      TypeCategoryRec draftModel         = productDraft.getModel() ;
      TypeCategoryRec draftProject       = productDraft.getProject() ;
      TypeCategoryRec draftPartNum       = productDraft.getPartNumber() ;
      TypeCategoryRec draftSubCategory   = productDraft.getSubCategory() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Project                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( draftProject != null ) {
         currentProject = TypeCategory.getCategory ( draftProject.getInd() ) ;

         /* If we did not find one, create one now  */
         if ( currentProject == null ) {
            currentProject = TypeCategory.createTypeCategory ( draftProject.getInd(),
                                                               getFamily().getInd(),
                                                               draftProject.toString(),
                                                               TypeCategory.LEVEL_PROJECT,
                                                               draftProject.getOrderNum(),
                                                               draftProject.getChildSort(),
                                                               draftProject.getOwner() ) ;
            currentProject.writeToDatabase() ;
            setProject ( currentProject ) ;
         } else {

            /* If this is an insert, we must insert the project  */
            if ( getRecStatus() != REC_INSERT )
               projectUpdated = false ;
            else
               projectUpdated = true ;

            this.project = currentProject ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Machine                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( draftMachine != null ) {
         currentMachine = TypeCategory.getCategory ( draftMachine.getInd() ) ;

         /* If we did not find one, create one now  */
         if ( currentMachine == null ) {
            currentMachine = TypeCategory.createTypeCategory ( draftMachine.getInd(),
                                                               currentProject.getInd(),
                                                               draftMachine.toString(),
                                                               TypeCategory.LEVEL_MACHINE,
                                                               draftMachine.getOrderNum(),
                                                               draftMachine.getChildSort(),
                                                               draftMachine.getOwner() ) ;
            currentMachine.writeToDatabase() ;
            setMachine ( currentMachine ) ;
         } else {

            /* If this is an insert, we must insert the machine  */
            if ( getRecStatus() != REC_INSERT )
               machineUpdated = false ;
            else
               machineUpdated = true ;
            this.machine = currentMachine ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Model                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( draftModel != null ) {
         currentModel = TypeCategory.getCategory ( draftModel.getInd() ) ;

         /* If we did not find one, create one now  */
         if ( currentModel == null ) {
            currentModel = TypeCategory.createTypeCategory ( draftModel.getInd() ,
                                                             draftMachine.getInd(),
                                                             draftModel.toString(),
                                                             TypeCategory.LEVEL_MODEL,
                                                             draftModel.getOrderNum(),
                                                             draftModel.getChildSort(),
                                                             draftModel.getOwner() ) ;

            this.model = null ;
            setModel ( currentModel ) ;
         } else {
            modelUpdated = false ;
            this.model = currentModel ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the model's parent regardless                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( currentMachine != null ) && ( model != null ) ) {
         model.setParentCatInd ( currentMachine.getInd() ) ;
         currentModel.writeToDatabase() ;
      }




      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Sub Category                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( draftSubCategory != null ) {
         currentSubCategory = TypeCategory.getCategory ( draftSubCategory.getInd() );

         /* If we did not find one, create one now  */
         if ( currentSubCategory == null ) {
            currentSubCategory = TypeCategory.createTypeCategory ( draftSubCategory.getInd(),
                                                                   currentProject.getInd(),
                                                                   draftSubCategory.toString(),
                                                                   TypeCategory.LEVEL_SUBCATEGORY,
                                                                   draftSubCategory.getOrderNum(),
                                                                   draftSubCategory.getChildSort(),
                                                                   draftSubCategory.getOwner() ) ;
            currentSubCategory.writeToDatabase() ;
            setSubCategory ( currentSubCategory ) ;
         } else {
            /* If this is an insert, we must insert the Sub Category  */
            if ( getRecStatus() != REC_INSERT )
               machineUpdated = false ;
            else
               machineUpdated = true ;
            this.subCategory = currentSubCategory ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Part Number                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( draftPartNum != null ) {
         currentPartNum = TypeCategory.getCategory ( draftPartNum.getInd() ) ;

         /* If we did not find one, create one now  */
         if ( currentPartNum == null ) {
            currentPartNum = TypeCategory.createTypeCategory ( draftPartNum.getInd(),
                                                               draftSubCategory.getInd(),
                                                               draftPartNum.toString(),
                                                               TypeCategory.LEVEL_PARTNUMBER,
                                                               draftPartNum.getOrderNum(),
                                                               draftPartNum.getChildSort(),
                                                               draftPartNum.getOwner() ) ;
            currentPartNum.writeToDatabase() ;
            currentPartNum.updateRecStatus ( REC_NOCHANGE ) ;
            this.partNum = null ;
            setPartNumber ( currentPartNum ) ;
         } else {
            partNumUpdated = false ;
            this.partNum = currentPartNum ;
         }
      }
   }


   /**
     * This method will reset all of the Product Update variables in this class to false.
     * The variables that are reset are the machineUpdated, modelUpdated etc..
     **/
   protected void resetUpdateVariables () {
      machineUpdated = false ;
      modelUpdated   = false ;
      partNumUpdated = false ;
      brandUpdated   = false ;
      familyUpdated  = false ;
      projectUpdated = false ;

      if ( dates != null ) {
         int i    = 0 ;
         int size = dates.size() ;

         while ( i < size ) {
            if ( ((ProductDate)dates.elementAt(i)).valid() )
               ((ProductDate)dates.elementAt(i)).updateRecStatus ( REC_NOCHANGE ) ;
            i++ ;
         }
      }


      if ( locale != null ) {
         if ( locale.valid() )
            locale.updateRecStatus ( REC_NOCHANGE ) ;
      }


      if ( brand != null )
         brand.updateRecStatus ( REC_NOCHANGE ) ;
      if ( family != null )
         family.updateRecStatus ( REC_NOCHANGE ) ;
      if ( project != null )
         project.updateRecStatus ( REC_NOCHANGE ) ;
      if ( machine != null )
         machine.updateRecStatus ( REC_NOCHANGE ) ;
      if ( subCategory != null )
         subCategory.updateRecStatus ( REC_NOCHANGE ) ;
      if ( model != null )
         model.updateRecStatus ( REC_NOCHANGE ) ;
      if ( partNum != null )
         partNum.updateRecStatus ( REC_NOCHANGE ) ;

   }


   /**
     * This method will add a ProductPropertyListener to this product. This listener
     * will be notified whenever a property in this product changes
     *
     * @param listener The ProductPropertyListener to add
     **/
   public void addProductPropertyListener ( ProductPropertyListener listener ) {
      if ( listeners == null )
         listeners = new Vector ( 1 ) ;

      listeners.addElement ( listener ) ;
   }


   /**
     * This method will remove a ProductPropertyListener from this product
     *
     * @param listener The ProductPropertyListener to remove
     **/
   public void removeProductPropertyListener ( ProductPropertyListener listener ) {
      if ( listeners != null ) {
         listeners.removeElement ( listener ) ;
         if ( listeners.size() == 0 )
            listeners = null ;
      }
   }


   /**
     * This method will fire an event to all registered listeners
     *
     * @param property The Property to fire
     **/
   protected void firePropertyChanged ( int property ) {
      if ( ( listeners != null ) && ( fireEventEnabled ) )
         new FireThread ( this, property, listeners ) ;
   }

   /**
     *  This method wil enable/disable the event firing when a
     * property is changed. 
     *
     * @param enable TRUE = Send out property change events when something is changed
     **/
   public void setPropertyChangeEnabled ( boolean enable ) {
      this.fireEventEnabled = enable ;
   }

   /**
     * This method will return the list of comment history
     */
   public Vector getCommentHistory() {
      return(commentHistory);
   }

   /**
     *  This method will add a string to history.
     *
     * @param The String to add
     **/
   protected void addCommentToHistory ( ProductComment comment ) {
      if ( commentHistory == null )
         commentHistory = new Vector (1) ;

      commentHistory.addElement ( comment ) ;
   }
   /**
     *  This method will return the product comment history as text
     **/
   public String getCommentHistoryAsText () {
      String retStr = "" ;

      if ( ( commentHistory != null ) && ( commentHistory.size() > 0 ) ) {
         int i     = 0;
         int size  = commentHistory.size () ;

         while ( i < size ) {
            retStr+=((ProductComment)commentHistory.elementAt(i)).toString() ;
            if ( i < (size-1) )
               retStr+="\n\n" ;
            i++ ;
         }
      }

      return( retStr ) ;
   }

} /* Product */