package com.ibm.nzna.projects.common.quest.product;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      ProductDate
/*   Creation Date: APRIL 17, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.shared.pom.PersistentRec ;
/**
 * The ProductDate class holds information regarding a single date
 * of a product. This could be the Announced, Withdrawn etc.. date. Each
 * date also has a specific country.
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* ProductDate
/*
/*----------------------------------------------------*/
public class ProductDate extends PersistentRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000001 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  ProductInd is the Primary Key of the product that this date is for
     **/
   private int productInd = 0 ;

   /**
     *  This holds what is returned from toString
     **/
   private String toString = "" ;

   /**
     *  This is the actual date String representation of the date. 
     **/
   private String actualDate = null ;

   /**
     *  This is the target date String representation of the date. This value
     * must not be null and must not be taken seriously by any publishing scripts.
     **/
   private String targetDate = null ;

   /**
     *  This is the country that this date is valid for. 
     **/
   private int countryCodeInd = 0 ;

   /**
     *  This is the type of date. This type may be 'Announced', 'Withdrawn', etc..
     * Please keep in mind that this indicator must already exist in the
     * PRODUCT.TYPEDATE table.
     **/
   private int typeDateInd = 0 ;


   /**
     * The Default Constructor.
     **/
   public ProductDate () {
   }

   /**
     * This constructor can be used when creating a new date for the database
     *
     * @param The type of date
     **/
   public ProductDate ( int typeDateInd ) {
      this.typeDateInd = typeDateInd ;
   }


   /**
     *  This constructor should be used when creating a new date for the database.
     *
     *  @param The productInd that this date is for
     *  @param The type of date that this object represents
     **/
   public ProductDate ( int productInd, int typeDateInd ) {
      this.productInd = productInd ;
      this.typeDateInd = typeDateInd ;
      updateRecStatus ( REC_INSERT ) ;
   }

   /**
     *  This constructor should be used when creating a record from a row that the
     * already in the database.
     **/
   public ProductDate ( int productInd, String actualDate, String targetDate, int typeDateInd, int countryCodeInd ) throws IllegalArgumentException {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our local vars                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.productInd  = productInd ;
      this.typeDateInd = typeDateInd ;
      setActualDate         ( actualDate ) ;
      setTargetDate         ( targetDate ) ;
      setCountryCodeInd     ( countryCodeInd ) ;                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the status to no change                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      updateRecStatus ( REC_NOCHANGE ) ;
   }


   /**
     *  This method will set the actual date for this date type
     **/
   public void setActualDate ( String date ) {
      this.actualDate = date ;
      updateRecStatus ( REC_UPDATE ) ;
   }

   /**
     *  This method will set the target date for this date type
     **/
   public void setTargetDate ( String date ) throws IllegalArgumentException {
      if ( ( date != null ) && ( date.length() <= 10 ) ) {
         targetDate = date ;
         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Target date of "+date+" is invalid" ) ;
   }

   /**
     *  This method will set the country that this date represents
     **/
   public void setCountryCodeInd ( int countryCodeInd ) {
      this.countryCodeInd = countryCodeInd ;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /**
     *  This method will return the SQL needed to save this object. The String
     * that is returned will have the SqlRunner.END_DELIM appended to the end. The
     * SQL returned from this method is suitable for saving in the PRODUCT.DATE table.
     *
     * @return String
     **/
   public String toSQL () {
      StringBuffer buffer         = null ;
      String       saveTargetDate = null ;
      String       saveActualDate = null ;

      if ( targetDate != null ) {
         saveTargetDate = targetDate ; //CDate.createDate ( CDate.parse ( targetDate, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                       //      CDate.parse ( targetDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                       //      CDate.parse ( targetDate, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                       //      0,0,0,CDate.DATE_DATE ) ;
      }
      if ( actualDate != null )
         saveActualDate = actualDate ; //CDate.createDate ( CDate.parse ( actualDate, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                       //      CDate.parse ( actualDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                       //      CDate.parse ( actualDate, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                       //      0,0,0,CDate.DATE_DATE ) ;

      if ( productInd > 0 ) {

         if ( saveTargetDate == null ) {
            saveTargetDate = saveActualDate ;
         }

         buffer = new StringBuffer ( "INSERT INTO PRODUCT.DATES (PRODUCTIND, TYPEDATEIND, COUNTRYCODEIND, ACTUAL, TARGET ) VALUES (" ) ;
         buffer.append ( productInd+", " ) ;
         buffer.append ( typeDateInd+", " ) ;
         buffer.append ( countryCodeInd+", " ) ;
         buffer.append ( Text.cleanNullableDBString ( saveActualDate )+", " ) ;
         buffer.append ( "'"+Text.cleanDBString ( saveTargetDate )+"')" ) ;
         buffer.append ( SqlRunner.END_DELIM ) ;

         if ( buffer != null )
            return( buffer.toString() ) ;
      }

      return( "" ) ;
   }


   /**
     *  This method will return the SQL needed to save this object. The String
     * that is returned will have the SqlRunner.END_DELIM appended to the end. The
     * SQL returned from this method is suitable for saving in the PRODRAFT.DATE table.
     *
     * @return String
     **/
   public String toDraftSQL () {
      StringBuffer buffer         = null ;
      String       saveTargetDate = null ;
      String       saveActualDate = null ;

      if ( targetDate != null )
         saveTargetDate = targetDate ; //CDate.createDate ( CDate.parse ( targetDate, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                       //      CDate.parse ( targetDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                       //      CDate.parse ( targetDate, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                       //      0,0,0,CDate.DATE_DATE ) ;
      if ( actualDate != null )
         saveActualDate = actualDate ; //CDate.createDate ( CDate.parse ( actualDate, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                       //      CDate.parse ( actualDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                       //      CDate.parse ( actualDate, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                       //      0,0,0,CDate.DATE_DATE ) ;

      if ( productInd > 0 ) {

         if ( saveTargetDate == null ) {
            saveTargetDate = saveActualDate ;
         }

         buffer = new StringBuffer ( "INSERT INTO PRODRAFT.DATES (PRODUCTIND, TYPEDATEIND, COUNTRYCODEIND, ACTUAL, TARGET ) VALUES (" ) ;
         buffer.append ( productInd+", " ) ;
         buffer.append ( typeDateInd+", " ) ;
         buffer.append ( countryCodeInd+", " ) ;
         buffer.append ( Text.cleanNullableDBString ( saveActualDate )+", " ) ;
         buffer.append ( "'"+Text.cleanDBString ( saveTargetDate )+"')" ) ;
         buffer.append ( SqlRunner.END_DELIM ) ;

         if ( buffer != null )
            return( buffer.toString() ) ;
      }

      return( "" ) ;
   }


   /**
     * This method will set the product indicator
     *
     * @param productInd The product indicator to set
     **/
   protected void setProductInd ( int productInd ) {
      this.productInd = productInd ;
   }


   /**
     *  This method will return the actual date for this date
     **/
   public String getActualDate () {
      return( actualDate ) ;
   }

   /**
     *  This method will return the target date for this date
     **/
   public String getTargetDate () {
      return( targetDate ) ;
   }


   /**
     *  This method will return the country code indicator for the date
     **/
   public int getCountryCodeInd () {
      return( countryCodeInd) ;
   }


   /**
     *  This method will return the product indicator of the product that this
     * date is tied to
     **/
   public int getProductInd () {
      return( productInd ) ;
   }


   /**
     *  This method will return the date type 
     **/
   public int getDateType () {
      return( typeDateInd ) ;
   }

   /**
     *  This method will compare this ProductDate with another date
     * by comparing only the Country and Date Type
     **/
   public boolean equals ( Object object ) {
      if ( object instanceof ProductDate )
         return( ( ((ProductDate)object).getCountryCodeInd() == countryCodeInd ) &&
                 ( ((ProductDate)object).getDateType()       == typeDateInd ) ) ;
      return( super.equals ( object ) ) ;
   }

   /**
     *  This method will set what is returned from the toString method
     **/
   public void setToString ( String toString ) {
      this.toString = toString ;
   }

   public String toString () {
      return( toString ) ;
   }
}
/* Class1 */
