package com.ibm.nzna.projects.common.quest.product ;      
/*-----------------------------------------------------------*/
/*   ProductDateRow                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductDateRow                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.gui.* ; 
import com.ibm.nzna.shared.util.* ; 
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This is a simple wrapper for the ProductDate object so that it can be put into a MultiList 
 *
 *
 * @version 1.0, 8/30/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductDateRow                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductDateRow extends Object implements ExtendedMultiListRow, MultiListRow, MultiListSort {

   /**
     *  This is the type of date that will be displayed in the first column
     **/
   private String dateType = null ;

   /**
     *  This is the indicator for the date type
     **/
   private int dateTypeInd = 0 ;


   /**
     *  This is the date that is shown
     **/
   private String date = null ;

   /**
     *  And this is the ProductDate object 
     **/
   private ProductDate productDate = null ;

   /**
     *  Tells if this is a target date or an actual date
     **/
   private boolean target = false ;

   /**
     *  Default Constructor
     **/
   public ProductDateRow ( ProductDate productDate ) {
      if ( productDate != null ) {
         setProductDate ( productDate ) ;
         this.dateType = TypeList.getInstance().objectFromInd ( productDate.getDateType(), TypeList.TYPE_DATE ).toString() ;
      }
   }

   /**
     *  This method will set the ProductDate object this row will represent
     **/
   public void setProductDate ( ProductDate productDate ) {
      if ( ( productDate.getActualDate() == null ) || ( productDate.getActualDate().length() == 0 ) ) {
         target = true ;
         date = productDate.getTargetDate() ;
      } else {
         target = false ;
         date = productDate.getActualDate() ;
      }

      date = CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR,  CDate.DATE_DATE ),
                                CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                CDate.parse ( date, CDate.DATE_DAY,   CDate.DATE_DATE ),
                                0,
                                0,
                                0,
                                CDate.DATE_WW_DATE ) ;
      this.dateTypeInd = productDate.getDateType() ;
   }

   /**
     *  This constructor creates a blank Date row
     **/
   public ProductDateRow ( TypeDateRec typeDateRec ) {
      this.productDate = null ;
      dateType         = typeDateRec.toString() ;
      dateTypeInd      = typeDateRec.getInd() ;
      date = "" ;
   }

   /**
     *  This method will compare this object to another MultiListRow object that is an instance
     * of ProductDateRow
     **/
   public int compareTo ( MultiListSort sortObject, int column ) {
      if ( ( sortObject instanceof ProductDateRow ) && ( date != null ) ) {
         ProductDateRow otherDate = (ProductDateRow ) sortObject ;

         switch ( column ) {
            case 0 :
               return( dateType.compareTo ( otherDate.dateType ) ) ;

            case 1 :
               if ( date.length() > 0 ) {
                  return( CDate.compareDate ( date,
                                              otherDate.date,
                                              CDate.DATE_WW_DATE ) ) ;
               }
               return( 0 ) ;

         }
      }
      return( 0 ) ;
   }


   /**
     *  This method will return our column data
     **/
   public Object getColumnData ( int column ) {
      switch ( column ) {
         case 0 :
            if ( dateType != null )
               return( dateType ) ;

         case 1 :
            if ( date != null )
               return( date ) ;

         default:
            return( "" ) ;
      }
   }


   /**
     *  This method will return the ProductDate object we are representing
     *
     * @return ProductDate
     **/
   public ProductDate getProductDate () {
      return( productDate ) ;
   }


   /**
     *  This method will return the type of date we are repesnting
     **/
   public int getDateType () {
      return( dateTypeInd ) ;
   }


   public void adjustColumnGraphics ( int column, java.awt.Graphics g ) {
      if ( column == 1 ) {
         if ( target )
            g.setColor ( java.awt.Color.red ) ;
         else
            g.setColor ( java.awt.Color.black ) ;
      }
   }

   public void columnSized ( int i, int k ) {
   }

} /* ProductDateRow */
