package com.ibm.nzna.projects.common.quest.product;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      ProductReader
/*   Creation Date: APRIL 17, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Blob ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
/**
 * This class has the static methods needed to read a Product from the
 * database. Please use the Product object's 'readFromDatabase' method to
 * read information and not this class.
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* ProductReader
/*
/*----------------------------------------------------*/
public class ProductReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec       = new StoredProcRec () ;

   /*------------------------------------------------------------*/
   /* readProductDraft
    */
   /**
    *    This method will be called by DB/2 when
    *  a request for this stored procedure is given.
    *  
    *
    *  @param docInd The ProductDraft Indicator to read in
    *  @param documentBlob The Blob to store the document in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readProduct ( int productInd, Blob productBlob, Blob storedProcReturn ) throws Exception {
      Connection conn = getConnection() ;

      try {
         Product product = new Product ( productInd ) ;
         int     rc           = 0 ;

         rc = readFromDatabase ( conn, product ) ;

         productBlob = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( productBlob, product ) ;
         set ( 2, productBlob ) ;

         returnRec.sqlCode  = rc ;
         returnRec.errorStr = "ProductReader Returned rc:"+rc ;
         returnRec.error    = ( rc != 0 ) ;

      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      conn.commit() ;
      conn.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcReturn = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcReturn, returnRec ) ;

      set ( 3, storedProcReturn ) ;

   } /* readProduct  */



   /**
     * This method will read an entire product record from the database. It
     * will return 0 if all read ok, or and SQL Code in the event of an
     * error. The Product record passed will be populated and must already
     * have the productInd property set.<br><br>
     *
     * @returns  0 = Everything read Ok<br>
     *          -1 = The Base Info could not be read<br>
     *          -2 = The Dates could not be read<br>
     *          -3 = The XRef could not be read<br>
     *          -4 = The Categorization could not be read<br>
     *          -5 = The Locale could not be read<br>
     *          -6 = Could not read the comment history<br><br>
     **/
   protected static int readFromDatabase ( Connection conn, Product product ) throws Exception {
      int rc = 0 ;

      try {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read the Product's Base Information                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rc =  readBaseInfo ( conn, product ) ;

         if ( rc == 0 ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Read in the Product Dates                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( readDates ( conn, product ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in the Cross Reference (XREF) info                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( readCrossProduct ( conn, product ) ) {

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Read in the Geographies and countries                   */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( readLocale ( conn, product ) ) {


                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Read in the Brand, Family, Machine, Model and part num  */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( readCategories ( conn, product ) ) {
                        if ( readCommentHistory ( conn, product ) == 0 )
                           rc = 0 ;
                        else
                           rc = -6 ;
                     } else
                        rc = -4 ;

                  } else
                     rc = -5 ;

               } else /* create Cross Product  */
                  rc = -3 ;


            } else /* read Dates  */
               rc = -2 ;

         }
      } catch ( Exception e ) {
         e.printStackTrace() ;
         throw ( e ) ;
      }

      return( rc ) ;
   }


   /**
     *  This method will read in all base information from the PRODUCT.PRODUCT
     * table. It will populate the Product object passed with this information
     *
     * @param conn The DB/2 Connection to use
     * @param product The Product object to populate
     *
     * @return TRUE if all read Ok, FALSE If did not read or SQL Error
     **/
   public static int readBaseInfo ( Connection conn, Product product ) throws Exception {
      int       rc   = 0 ;
      Statement stmt = conn.createStatement() ;
      ResultSet rs   = stmt.executeQuery ( "SELECT DESCRIPTION,    BASEMODEL,       TYPECUSTOMERIND, "+
                                           "       TYPEPRODUCTIND, TYPEOFFERINGIND, TYPEWARRANTYIND, INPUT, "+
                                           "       SOURCE,         PUBLISH,         CHECKEDOUTBY,    CREATEDON, "+
                                           "       CREATEDBY,      LASTTOUCHEDON,   LASTTOUCHEDBY,   IMPORTOPICM "+
                                           "FROM PRODUCT.PRODUCT "+
                                           "WHERE PRODUCTIND = "+product.getProductInd()+" FOR FETCH ONLY" ) ;

      if ( rs.next() ) {
         product.setDescription   ( rs.getString(1).trim() ) ;
         product.setBaseModel     ( rs.getString(2) ) ;
         product.setCustomer      ( rs.getInt ( 3 ) ) ;
         product.setProductType   ( rs.getInt ( 4 ) ) ;
         product.setOffering      ( rs.getInt ( 5 ) ) ;
         product.setWarranty      ( rs.getInt ( 6 ) ) ;
         product.setInput         ( rs.getString ( 7 ).trim() ) ;
         product.setSource        ( rs.getString ( 8 ).trim() ) ;
         product.setPublish       ( rs.getString ( 9  ) ) ;
         product.setCheckedOutBy  ( rs.getString ( 10 ) ) ;
         product.setCreatedOn     ( rs.getString ( 11 ).trim() ) ;
         product.setCreatedBy     ( rs.getString ( 12 ).trim() ) ;
         product.setLastTouchedOn ( rs.getString ( 13 ).trim() ) ;
         product.setLastTouchedBy ( rs.getString ( 14 ).trim() ) ;
         if ( rs.getString(15) != null )
            product.setOPICMImport        ( rs.getString ( 15 ).equals ( "Y" ) ) ;

      } else {
         rc=100 ;
      }

      rs.close() ;
      stmt.close() ;

      return( rc ) ;

   }


   /**
     *  This method will read in all of the product dates. It will populate
     * the product object passed in with these dates.
     *
     * @param conn The DB/2 Connection to use
     * @param product The Product object to populate
     *
     * @return TRUE if all read Ok, FALSE If did not read or SQL Error
     **/
   public static boolean readDates ( Connection conn, Product product ) throws Exception {
      Statement stmt    = conn.createStatement() ;
      Vector    dateVec = new Vector ( 1 ) ;
      ResultSet rs      = stmt.executeQuery ( "SELECT TYPEDATEIND, COUNTRYCODEIND, ACTUAL, TARGET "+
                                              "FROM   PRODUCT.DATES "+
                                              "WHERE PRODUCTIND = "+product.getProductInd()+" FOR FETCH ONLY" ) ;

      while ( rs.next() ) {
         String actualDate = rs.getString(3) ;
         String targetDate = rs.getString(4) ;

         if ( actualDate != null )
            actualDate = CDate.createDate ( CDate.parse ( actualDate, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                            CDate.parse ( actualDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                            CDate.parse ( actualDate, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                            0,0,0,CDate.DATE_DATE ) ;
         if ( targetDate != null )
            targetDate = CDate.createDate ( CDate.parse ( targetDate, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                            CDate.parse ( targetDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                            CDate.parse ( targetDate, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                            0,0,0,CDate.DATE_DATE ) ;
         dateVec.addElement ( new ProductDate ( product.getProductInd(),
                                                actualDate,
                                                targetDate,
                                                rs.getInt(1),
                                                rs.getInt(2) ) ) ;

      }

      product.setDates ( dateVec ) ;
      rs.close() ;
      stmt.close() ;

      return( true ) ;
   } /* readDates  */



   /**
     *  This method will read in all XRef objects from the PRODUCT.XREF table. It will
     * populate the product record passed with that information
     *
     * @param conn The DB/2 Connection to use
     * @param product The Product object to populate
     *
     * @return TRUE if all read Ok, FALSE If did not read or SQL Error
     **/
   public static boolean readCrossProduct ( Connection conn, Product product ) throws Exception {
      Statement stmt    = conn.createStatement() ;
      Vector    xref    = new Vector ( 1 ) ;
      ResultSet rs      = stmt.executeQuery ( "SELECT TYPEXREFIND, NUMBER "+
                                              "FROM   PRODUCT.XREF "+
                                              "WHERE PRODUCTIND = "+product.getProductInd()+" FOR FETCH ONLY" ) ;

      while ( rs.next() ) {
         xref.addElement ( new XRef ( product.getProductInd(),
                                      rs.getInt(1),
                                      rs.getString(2).trim() ) ) ;

      }

      product.setXRef ( xref ) ;
      rs.close() ;
      stmt.close() ;

      return( true ) ;
   } /* readDates  */


   /*------------------------------------------------------------*/
   /* readLocale
    */
   /**
    *    This method will read in all Geography
    *  and country information on a product
    *  
    *
    *  @param conn The DB/2 Connection to use to read with
    *  @param document The product object to populate
    *  @param None
    *
    *  @return 0 if everything went ok
    *//*---------------------------------------------------------*/
   public static boolean readLocale ( Connection conn, Product product ) throws Exception {
      LocaleRec locale          = new LocaleRec ( conn ) ;
      Statement stmt            = conn.createStatement() ;
      boolean   rc              = false ;
      ResultSet rs              = null ;                          
      boolean   continueProcess = true ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read in all of the Geographies                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( "SELECT A.GEOIND, B.DESCRIPT "+
                               "FROM PRODUCT.GEO A, "+
                               "     TIGRIS.TYPEGEOGRAPHY B "+
                               "WHERE PRODUCTIND = "+product.getProductInd()+" AND "+
                               "      A.GEOIND = B.GEOIND "+
                               "FOR FETCH ONLY" ) ;

      while ( ( continueProcess ) && ( rs.next() ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is world wide, do not process any more          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rs.getInt(1) == 6 ) {
            continueProcess = false ;
            locale.setWorldWide ( true ) ;
         } else
            locale.addGeography ( new TypeGeoRec ( rs.getInt(1), rs.getString(2).trim() ) ) ;
      }

      rs.close() ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read in all of the countries                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( continueProcess )
         rs = stmt.executeQuery ( "SELECT A.COUNTRYCODEIND, B.PHONECOUNTRYCODE, B.DESCRIPT,     B.ZIPMASK,   "+
                                  "       B.PHONEMASK,      B.ZIPDIGITS,        B.TIMEDIFF,     B.MULTIZONE, "+
                                  "       B.USESDST,        B.CURRENCY,         B.CURRENCYMASK, B.LANG01,    "+
                                  "       B.LANG02,         B.LANG03,           C.GEOIND                     "+
                                  "FROM PRODUCT.COUNTRY A,                                                   "+
                                  "     TIGRIS.TYPECOUNTRYCODE B,                                            "+
                                  "     TIGRIS.GEOCOUNTRY C                                                  "+
                                  "WHERE PRODUCTIND = "+product.getProductInd()+"  AND                       "+
                                  "      A.COUNTRYCODEIND = B.COUNTRYCODEIND AND                             "+
                                  "      A.COUNTRYCODEIND = C.COUNTRYCODEIND                                 "+
                                  "FOR FETCH ONLY" ) ;

      while ( ( continueProcess ) && ( rs.next() ) ) {
         locale.addCountry ( new TypeCountryCodeRec ( rs.getInt(1), 
                                                      rs.getString(2), 
                                                      rs.getString(3).trim(), 
                                                      rs.getString(4),
                                                      rs.getString(5), 
                                                      (short)rs.getInt(6),
                                                      rs.getString(7),
                                                      rs.getString(8), 
                                                      rs.getString(9),
                                                      rs.getString(10),
                                                      rs.getString(11),
                                                      (short)rs.getInt(12),
                                                      (short)rs.getInt(13),
                                                      (short)rs.getInt(14),
                                                      (short)rs.getInt(15),
                                                      "" ) ) ;
      } /* while  */

      rs.close() ;
      stmt.close() ;
      rc = true ;

      product.setLocale ( locale ) ;

      return( rc ) ;
   }



   /**
     *  This method will read in all of the categories that a Product is associated with. 
     * remember that these categories include Brand, Family, Machine, Model, Family Type,
     * project, and part number. All of that information will be read and populated into
     * the product object passed in.
     *
     * @param conn The Connection object to read with
     * @param product The Product object to populate
     *
     * @return TRUE if no error.
     **/
   public static boolean readCategories ( Connection conn, Product product ) throws Exception {
      Statement       stmt        = conn.createStatement() ;
      TypeCategoryRec categoryRec = null ;
      ResultSet       rs          = stmt.executeQuery ( "SELECT TYPECATEGORY.TYPECATIND, TYPECATEGORY.DESCRIPTION, "+
                                                        "       TYPECATEGORY.PARENTCATIND, TYPECATEGORY.CATEGORYLEVEL, "+
                                                        "       TYPECATEGORY.ORDERNUM,     TYPECATEGORY.CHILDSORT, "+
                                                        "       TYPECATEGORY.OWNER "+
                                                        "FROM PRODUCT.TYPECATEGORY TYPECATEGORY, "+
                                                        "     PRODUCT.CATEGORY     CATEGORY "+
                                                        "WHERE CATEGORY.PRODUCTIND = "+product.getProductInd()+" AND "+
                                                        "      TYPECATEGORY.TYPECATIND = CATEGORY.TYPECATIND "+
                                                        "FOR FETCH ONLY" ) ;

      while ( rs.next() ) {

         categoryRec = new TypeCategoryRec ( rs.getInt ( 1 ) ) ;
         categoryRec.setDescript      ( rs.getString(2).trim() ) ;
         categoryRec.setParentCatInd  ( rs.getInt(3) ) ;
         categoryRec.setCategoryLevel ( rs.getString ( 4 ).trim() ) ;
         categoryRec.setOrderNum      ( rs.getInt ( 5 ) ) ;
         categoryRec.setChildSort     ( rs.getString ( 6 ).trim() ) ;
         categoryRec.setOwner         ( rs.getString ( 7 ) ) ;
         categoryRec.updateRecStatus ( categoryRec.REC_NOCHANGE ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Brand                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_BRAND ) )
            product.setBrand ( categoryRec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Family                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_FAMILY ) )
            product.setFamily ( categoryRec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Machine                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_MACHINE ) )
            product.setMachine ( categoryRec ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Model                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_MODEL ) )
            product.setModel ( categoryRec ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Project                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_PROJECT ) )
            product.setProject ( categoryRec ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Sub Category                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_SUBCATEGORY ) )
            product.setSubCategory ( categoryRec ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Part Number                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( categoryRec.getCategoryLevel().equals ( TypeCategory.LEVEL_PARTNUMBER ) )
            product.setPartNumber ( categoryRec ) ;

      }

      rs.close() ;
      stmt.close() ;

      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* readCommentHistory
    */
   /**
    *    This method will read up to 5 of the latest
    *  comments for this product. 
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static int readCommentHistory ( Connection con, Product product) throws Exception {
      Statement     stmt       = con.createStatement() ;
      ResultSet     rs         = null ;
      String        sqlStmt    = null ; 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Form the SQL                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlStmt = "SELECT PRODUCTIND, COMMENTS "+
                "FROM PRODRAFT.PRODUCT "+
                "WHERE  EXISTINGPRODUCTIND = "+product.getProductInd()+" AND "+
                "       STOPDATE IS NOT NULL AND "+
                "       RECYCLED = 'N' AND "+
                "       TEMPLATE = 'N' "+
                "ORDER BY STOPDATE DESC FETCH FIRST 5 ROWS ONLY" ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sqlStmt ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {
         if ( rs.getString(2) != null )
            product.addCommentToHistory ( new ProductComment ( rs.getInt ( 1 ),
                                                               rs.getString(2).trim() ) ) ;
         else
            product.addCommentToHistory ( new ProductComment ( rs.getInt ( 1 ),
                                                               "N/A" ) ) ;
      }

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }
} /* ProductReader */
