package com.ibm.nzna.projects.common.quest.product ;   
/*-----------------------------------------------------------*/
/*   ProductRow                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductRow                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.util.Text ;
import com.ibm.nzna.projects.common.quest.type.* ;
import java.awt.Image ;
/**
 * This class is used to show a Product object in a MultiList
 * or Detailed JTree. 
 *
 * This class also has methods that can reorder the data that
 * is returned for a particular column
 *
 *
 * @version 1.0 
 * @author  
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductRow                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductRow extends Object implements java.io.Serializable, ExtendedMultiListRow, MultiListRow, MultiListSort {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int COL_LOCKED           = 1 ;
   public static final int COL_PRODNUM          = 2 ;
   public static final int COL_DESCRIPTION      = 3 ;
   public static final int COL_ANNOUNCEDATE     = 4 ;
   public static final int COL_CREATEDATE       = 5 ;
   public static final int COL_LASTTOUCHEDBY    = 6 ;
   public static final int COL_LASTTOUCHEDON    = 7 ;
   public static final int COL_WORKREQUIRED     = 8 ;
   public static final int COL_PROJECT          = 9 ;
   public static final int MAX_COLUMNS          = 10 ;




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  The description of the product
     **/
   private String description = null ;

   /**
     *  This is the array of lines that our description has. Each 
     * String in the array represents a line in the description. This is
     * to support the description wrapping.
     **/
   private String[] descriptionArray = null ;

   /**
     *  The existing Product indicator
     **/
   private int existingProductInd = 0  ;

   /**
     *  The draft product indicator
     **/
   private int productInd = 0  ;

   /**
     *  The Prodnum corresponding to the Product
     **/
   private String prodNum = null  ;

   /**
     *  This is the image that can be displayed to the far left on the row
     **/
   private Image image = null ;

   /**
     *  This is the original value if the setLastTouchedBy method was called
     **/
   private String originalLastTouchedBy = null ;

   /**
     *  This is the timestamp of when this record was last changed
     **/
   private String lastTouchedOn = null ;

   /**
     *  This is the timestamp of when this record was created
     **/
   private String createdOn = null ;

   /**
     *  This is the userid of the user who first created the Product
     **/
   private String createdBy = null ;

   /**
     *  The actual announce date
     **/
   private String actualDate = null ;

   /**
     *  The target announce date
     **/
   private String targetDate = null  ;

   /**
     *  The last person who touched this product
     **/
   private String lastTouchedBy = null ;

   /**
     * This property will hold the description of the type of work required. This description
     * is normally found in the TIGRIS.TYPEWORKREQUIRED table
     **/
   private String workRequired = null ;

   /**
     *  This object represents either the Part Number or Model of the Product
     * that this object is representing
     **/
   private TypeCategoryRec model = null ;


   /**
     *  This array holds the order in which the columns
     * will be viewed.
     **/
   private int[] columnOrder = null ;

   /**
     *  If this is a draft, this flag will be TRUE
     **/
   private boolean draft = false ;

   /**
     * This method will save the 'project' information for a product. Please note
     * that this value is not always populated
     **/
   private String project = null ;

   /**
     *  This object is the product that this row represents. Please
     * note that this object may be NULL if the product has not been 
     * read in yet. Please use this method's readDocument method to
     * read in this document and populate this property
     **/
   private Product product = null ;




   public ProductRow ( ProductDraft draft ) {
      this ( draft.getProductInd(),
             draft.getExistingProductInd(),
             (draft.getProdNum()!=null) ? draft.getProdNum() : draft.getPartNumber().toString(),
             draft.getDescription(),
             draft.getCreatedOn(),
             draft.getCreatedBy(),
             "","",
             draft.getLastTouchedBy(),
             draft.getLastTouchedOn(),
             true ) ;
      this.product = draft ;
   }
   /**
     *  This default constructor takes all properties. The params
     *  are<br><br>
     * 1. PRODUCTIND - The Product Indicator from PRODUCT.PRODUCT. 
     * 2. EXISTINGPRODUCTIND - The Existing Product Indicator from PRODRAFT.PRODUCT.<br>
     * 3. PRODNUM - The Prodnum for the Product.<br>
     * 4. DESCRIPTION - The Description of the product.<br>
     * 5. CREATEDON - The CreatedOn value for the product.<br>
     * 6. CREATEDBY - The CreatedBy value for the product.<br>
     * 7. ACTUALDATE - The Actual Announce Date from the XXX.DATES table.<br>
     * 8. TARGETDATE - The Target Announce Date from the XXX.DATES table.<br>
     * 9. LASTTOUCHEDBY - The LastTouchedBy value for this product.<br>
     * 10. LASTTOUCHEDON - The LastTouchedOn value for this product.<br>
     * 
     **/
   public ProductRow (  int     productInd,
                        int     existingProductInd,
                        String  prodNum,
                        String  description,
                        String  createdOn,
                        String  createdBy,
                        String  actualDate,
                        String  targetDate,
                        String  lastTouchedBy,
                        String  lastTouchedOn,
                        boolean draft ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our local vars                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/

      this.productInd          = productInd ;
      this.existingProductInd  = existingProductInd ;
      this.description         = description ;
      this.createdOn           = createdOn ;
      this.createdBy           = createdBy ;
      this.actualDate          = actualDate ;
      this.targetDate          = targetDate ;
      this.lastTouchedBy       = lastTouchedBy ;  
      this.lastTouchedOn       = lastTouchedOn ;
      this.draft               = draft ;
      this.prodNum             = prodNum ;                            

      /*--------------*/
      /*              */
      /*  Trim Values */
      /*              */
      /*--------------*/
      if ( prodNum != null )
         prodNum = prodNum.trim() ;

      if ( description != null )
         description = description.trim() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our default order                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      columnOrder = new int[MAX_COLUMNS] ;
      setColumnOrderData ( 0, COL_LOCKED  ) ;
      setColumnOrderData ( 1, COL_PRODNUM  ) ;
      setColumnOrderData ( 2, COL_DESCRIPTION  ) ;
      setColumnOrderData ( 3, COL_ANNOUNCEDATE  ) ;
      setColumnOrderData ( 4, COL_CREATEDATE  ) ;
      setColumnOrderData ( 5, COL_LASTTOUCHEDBY  ) ;
      setColumnOrderData ( 6, COL_LASTTOUCHEDON  ) ;
      setColumnOrderData ( 7, COL_WORKREQUIRED ) ;
      setColumnOrderData ( 8, COL_PROJECT ) ;

      columnSized ( 1, 500 ) ;
   }






   /**
    *    This method will return the column data
    *  for a specific column
    *  
    *
    *  @param column The column to return data for
    *
    *  @return Object
    */
   public Object getColumnData ( int column ) {
      return( getData ( columnOrder [ column ] ) ) ;
   }

   /**
     *  This method will look up to see what data
     * to return for the index in columnOrder
     **/
   private Object getData ( int columnOrderIndex ) {
      switch ( columnOrderIndex ) {
         case COL_LOCKED :
            return( image ) ;

         case COL_PROJECT :
            return( project ) ;

         case COL_PRODNUM :
            return( prodNum ) ;

         case COL_DESCRIPTION :
            return( descriptionArray  ) ;

         case COL_WORKREQUIRED :
            return(workRequired ) ;

         case COL_ANNOUNCEDATE :
            return( ( (actualDate != null ) && ( actualDate.length() > 0 ) ) ? actualDate : targetDate ) ;

         case COL_CREATEDATE :
            return( createdOn ) ;

         case COL_LASTTOUCHEDBY :
            return( lastTouchedBy ) ;

         case COL_LASTTOUCHEDON :
            return( lastTouchedOn ) ;
      }
      return( "" ) ;
   }

   public String getLastTouchedBy () {
      return( lastTouchedBy ) ;
   }

   public void setLastTouchedBy ( String lastTouchedBy ) {
      if ( originalLastTouchedBy == null )
         originalLastTouchedBy = this.lastTouchedBy ;

      this.lastTouchedBy = lastTouchedBy ;
   }


   public boolean isDraft () {
      return( draft ) ;
   }

   public void adjustColumnGraphics ( int column, java.awt.Graphics g ) {
      if ( columnOrder[column] == COL_ANNOUNCEDATE ) {
         boolean drawRed = false ;

         if ( ( actualDate == null ) || ( actualDate.length() == 0 ) )
            drawRed = true ;


         if ( ( drawRed ) && ( targetDate != null ) )
            g.setColor ( java.awt.Color.red ) ;
         else
            g.setColor ( java.awt.Color.black ) ;
      } else {
         g.setColor ( java.awt.Color.black ) ;
      }
   }

   public int getExistingProductInd () {
      return( new Integer ( existingProductInd ).intValue() ) ;
   }

   public int getProductInd () {
      return( new Integer ( productInd ).intValue() ) ;
   }


   /**
     *  This method will return our description
     **/
   public String getDescription () {
      return( description ) ;
   }

   /**
     *  This method will read in the product or product draft that
     * this row represents.
     *
     * @param forceRead If this value is TRUE, the read will happen
     *                  regardless if the product has already been read
     *                  in.
     *
     * @returns The SQLCode if an error, zero if no error
     **/
   public int readProduct ( boolean forceRead ) {
      int returnCode  = 0 ;

      if ( forceRead )
         product = null ;

      if ( product == null ) {

         if ( isDraft() )
            product = new ProductDraft ( productInd ) ;
         else
            product = new Product ( productInd ) ;

         returnCode = product.readFromDatabase() ;
      }

      return( returnCode ) ;
   }

   /**
     *  This method will return the Product object that this row represents. If
     * the Product has not been read in (through the readProduct method) then
     * this method will return NULL
     **/
   public Product getProduct () {
      return( product ) ;
   }

   /**
     *  This method will set what the product number is
     *
     * @param productNum The Product Number to show
     **/
   public void setProdNum ( String productNum ) {
      this.prodNum = productNum ;

      if ( this.prodNum != null )
         this.prodNum = this.prodNum.trim() ;
   }


   /**
     *  This method will return the following string:<br><br>
     *
     *  (Product Number) description
     **/
   public String toString () {
      return( "("+prodNum+") "+description ) ;
   }


   public void columnSized ( int column, int width ) {
      if ( columnOrder[column] == COL_DESCRIPTION )
         descriptionArray = Text.formatStringLines ( description, width/6 ) ;
   }


   /**
     *  This method will return the TypeCategoryRec representing either
     * the model or part number
     **/
   public TypeCategoryRec getModel () {
      return( model ) ;
   }

   /**
     *  This method will set the TypeCategoryRec representing either
     * the model or part number
     **/
   public void setModel ( TypeCategoryRec model ) {
      this.model = model ;
   }

   /**
     *  This method will set the image that is displayed on the far left
     **/
   public void setImage ( Image imageIcon ) {
      this.image = imageIcon ;
   }



   /**
     *  This method will compare this ProductRow to another for sorting purposes
     *
     * @return -1 if less, 0 if equal 1 if greater
     **/
   public int compareTo ( MultiListSort compareObject, int column ) {
      if ( compareObject instanceof ProductRow ) {
         ProductRow compareProduct = (ProductRow) compareObject ;

         switch ( columnOrder [ column ] ) {
            case COL_LOCKED :
               if ( ( image != null ) && ( compareProduct.image == null ) )
                  return( 1 ) ;
               else if ( ( image == null ) && ( compareProduct.image != null ) )
                  return( -1 ) ;
               return( 0 ) ;

            case COL_PRODNUM :
               if ( ( prodNum == null ) || ( prodNum.length() == 0 ) )
                  return( -1 ) ;
               else if ( ( compareProduct.prodNum == null ) || ( compareProduct.prodNum.length() == 0 ) )
                  return( 1  );
               return( prodNum.compareTo ( compareProduct.prodNum ) ) ;

            case COL_DESCRIPTION :
               if ( ( description == null ) || ( description.length() == 0 ) )
                  return( -1 ) ;
               else if ( ( compareProduct.description == null ) || ( compareProduct.description.length() == 0 ) )
                  return( 1  );
               return( description.compareTo ( compareProduct.description ) ) ;

            case COL_ANNOUNCEDATE :
               if ( ( actualDate == null ) || ( actualDate.length() == 0 ) )
                  return( -1 ) ;
               else if ( ( compareProduct.actualDate == null ) || ( compareProduct.actualDate.length() == 0 ) )
                  return( 1 ) ;
               return( CDate.compareDate ( actualDate, compareProduct.actualDate, CDate.DATE_WW_DATE ) ) ;

            case COL_CREATEDATE :
               if ( ( createdOn == null ) || ( createdOn.length() == 0 ) )
                  return( -1 ) ;
               else if ( ( compareProduct.createdOn == null ) || ( compareProduct.createdOn.length() == 0 ) )
                  return( 1 ) ;
               return( CDate.compareDate ( createdOn, compareProduct.createdOn, CDate.DATE_WW_DATE ) ) ;

            case COL_LASTTOUCHEDBY :
               if ( ( lastTouchedBy == null ) || ( lastTouchedBy.length() == 0 ) )
                  return( -1 ) ;
               else if ( ( compareProduct.lastTouchedBy == null ) || ( compareProduct.lastTouchedBy.length() == 0 ) )
                  return( 1  );
               return( lastTouchedBy.compareTo ( compareProduct.lastTouchedBy ) ) ;

            case COL_PROJECT :
               if ( ( project == null ) || ( project.length() == 0 ) )
                  return( -1 ) ;
               else if ( ( compareProduct.project == null ) || ( compareProduct.project.length() == 0 ) )
                  return( 1  );
               return( project.compareTo ( compareProduct.project ) ) ;

            case COL_WORKREQUIRED :
               if ( ( workRequired == null ) || ( workRequired.length() == 0 ) )
                  return( -1 ) ;
               else if ( ( compareProduct.workRequired == null ) || ( compareProduct.workRequired.length() == 0 ) )
                  return( 1  );
               return( workRequired.compareTo ( compareProduct.workRequired ) ) ;

            case COL_LASTTOUCHEDON :
               if ( ( lastTouchedOn == null ) || ( lastTouchedOn.length() == 0 ) )
                  return( -1 ) ;
               else if ( ( compareProduct.lastTouchedOn == null ) || ( compareProduct.lastTouchedOn.length() == 0 ) )
                  return( 1 ) ;
               return( CDate.compareDate ( lastTouchedOn, compareProduct.lastTouchedOn, CDate.DATE_WW_DATETIME ) ) ;
         }
      }
      return( 0 ) ;
   }

   public void setActualDate ( String actualDate ) {
      this.actualDate = actualDate ;
   }

   public String getActualDate () {
      return( actualDate ) ;
   }

   public void setTargetDate ( String targetDate ) {
      this.targetDate = targetDate ;
   }

   public String getTargetDate () {
      return( targetDate ) ;
   }

   public void setWorkRequired ( String workRequired ) {
      this.workRequired = workRequired ;
   }

   public String getWorkRequired ( ) {
      return( workRequired ) ;
   }

   public void setProject ( String project ) {
      this.project = project ;
   }

   public String getProject ( ) {
      return( project ) ;
   }

   /**
     * This method will set what data is shown in what column. 
     *
     * @param column This is the zero-based column number
     * @param dataId The is the id of the data to show. This is COL_XXXX data defined in this class
     **/
   public void setColumnOrderData ( int column, int dataId ) {
      columnOrder[column] = dataId ;
   }


   public String getProductNumber () {
      return ( prodNum ) ;
   }
} /* ProductRow */
