package com.ibm.nzna.projects.common.quest.product ;  
/*-----------------------------------------------------------*/
/*   ProductSQL                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductSQL                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.Vector ;
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class contains generic static SQL methods that can be
 * used to gather information about a product or list of products.
 *
 *
 * @version 1.0, 8/1/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductSQL                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductSQL {


   /**
     *  This method will return all products that fall under the
     * family passed in. Each object in the Vector returned will be 
     * a ProductRow object
     *
     * @param familyInd The TypeCatInd of the Family to read for
     *
     * @return Vector
     **/
   public static Vector getProductsFromFamily ( int familyInd ) throws SQLException {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getProductsFromFamily ( "+familyInd+" )", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.PRODUCTIND, A.PRODNUM,       A.DESCRIPTION,  A.CREATEDON, "+
                                              "       A.CREATEDBY,  A.LASTTOUCHEDBY, A.LASTTOUCHEDON "+
                                              "FROM PRODUCT.PRODUCT A, "+
                                              "     PRODUCT.CATEGORY B "+
                                              "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      A.PUBLISH    = 'Y' AND "+
                                              "      B.TYPECATIND = "+familyInd+" FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            returnVec.addElement ( new ProductRow ( rs.getInt ( 1 ),
                                                    0,
                                                    rs.getString(2).trim(),
                                                    rs.getString(3).trim(),
                                                    rs.getString(4).trim(),
                                                    rs.getString(5).trim(),
                                                    "",
                                                    "",
                                                    rs.getString(6).trim(),
                                                    rs.getString(7),
                                                    false ) ) ;
         }

         rs.close() ;                        

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, grab the options information                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec.size() > 0 )
         populateActiveOptionPartNumbers ( returnVec ) ;

      return( returnVec ) ;
   }

   /**
     *  This method will return all products that have been deleted previously and fall under the
     * family passed in. Each object in the Vector returned will be 
     * a ProductRow object
     *
     * @param familyInd The TypeCatInd of the Family to read for
     *
     * @return Vector
     **/
   public static Vector getDeletedProductsFromFamily ( int familyInd ) throws SQLException {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getProductsFromFamily ( "+familyInd+" )", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.PRODUCTIND, A.PRODNUM,       A.DESCRIPTION,  A.CREATEDON, "+
                                              "       A.CREATEDBY,  A.LASTTOUCHEDBY, A.LASTTOUCHEDON "+
                                              "FROM PRODUCT.PRODUCT A, "+
                                              "     PRODUCT.CATEGORY B "+
                                              "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      A.PUBLISH    = 'N' AND "+
                                              "      B.TYPECATIND = "+familyInd+" FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            returnVec.addElement ( new ProductRow ( rs.getInt ( 1 ),
                                                    0,
                                                    rs.getString(2).trim(),
                                                    rs.getString(3).trim(),
                                                    rs.getString(4).trim(),
                                                    rs.getString(5).trim(),
                                                    "",
                                                    "",
                                                    rs.getString(6).trim(),
                                                    rs.getString(7),
                                                    false ) ) ;
         }

         rs.close() ;                        

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, grab the options information                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec.size() > 0 )
         populateActiveOptionPartNumbers ( returnVec ) ;

      return( returnVec ) ;
   }



   /**
     *  This method will take a Vector of ProductRow objects and will populate
     * any Option information for any potential options in the list
     *
     * @param productVec A Vector of ProductRow objects
     *
     * @returns NOTHING
     **/
   public static void populateDraftOptionPartNumbers ( Vector productVec ) throws SQLException {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "populateOptionPartNumbers()", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT B.PRODUCTIND, A.DESCRIPTION FROM "+
                                              "PRODRAFT.TYPECATEGORY A, "+
                                              "PRODRAFT.CATEGORY B "+
                                              "WHERE A.TYPECATIND = B.TYPECATIND AND "+
                                              "A.CATEGORYLEVEL = '"+TypeCategory.LEVEL_PARTNUMBER+"' AND "+
                                              "B.PRODUCTIND IN "+createInClauseFromProductRows ( productVec )+" "+
                                              "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            ProductRow row = getProductRowFromInd ( rs.getInt(1), productVec ) ;

            row.setProdNum ( rs.getString(2) ) ;
         }

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }
      sqlMethod.close() ;
   }

   /**
     *  This method will take a Vector of ProductRow objects and will populate
     * any Option information for any potential options in the list
     *
     * @param productVec A Vector of ProductRow objects
     *
     * @returns NOTHING
     **/
   public static void populateActiveOptionPartNumbers ( Vector productVec ) throws SQLException {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "populateOptionPartNumbers()", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT B.PRODUCTIND, A.DESCRIPTION FROM "+
                                              "PRODUCT.TYPECATEGORY A, "+
                                              "PRODUCT.CATEGORY B "+
                                              "WHERE A.TYPECATIND = B.TYPECATIND AND "+
                                              "A.CATEGORYLEVEL = '"+TypeCategory.LEVEL_PARTNUMBER+"' AND "+
                                              "B.PRODUCTIND IN "+createInClauseFromProductRows ( productVec )+" "+
                                              "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            ProductRow row = getProductRowFromInd ( rs.getInt(1), productVec ) ;

            row.setProdNum ( rs.getString(2) ) ;
         }

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }
      sqlMethod.close() ;
   }



   /**
     *  This method will create the SQL 'IN' clause from a Vector of ProductRow objects. The 'IN' clause
     * is: SELECT * FROM PRODUCT.PRODUCT WHERE PRODUCTIND <b>IN (1,2,3,4)</b>. This method will return the entire
     * in clause including the parenthesis
     *
     * @param productVec A Vector of ProductRow objects
     * 
     * @return String
     **/
   public static String createInClauseFromProductRows ( Vector productVec ) {
      int i    = 0 ;
      int size = productVec.size() ;
      String returnString = "(" ;

      while ( i < size ) {
         returnString+=((ProductRow)productVec.elementAt(i)).getProductInd() ;
         if ( i < (size-1) )
            returnString+="," ;
         i++ ;
      }

      return( returnString+")") ;
   }


   /**
     *  This method will return the ProductRow object from a Vector of ProductRow
     * objects that match the ProductInd passed
     *
     * @param productInd The Product Indicator
     * @param productVec The Vector of ProductRow objects
     *
     * @return ProductRow
     **/
   public static ProductRow getProductRowFromInd ( int productInd, Vector productVec ) {
      int        i         = 0 ;
      ProductRow returnRec = null ;
      int        size      = productVec.size() ;

      while ( ( returnRec == null ) && ( i < size ) ) {
         if ( ((ProductRow)productVec.elementAt(i)).getProductInd() == productInd )
            returnRec = (ProductRow)productVec.elementAt(i) ;
         i++ ;
      }

      return( returnRec ) ;
   }



   /**
     *  This method will return all Product Templates that fall under the
     * family passed in. Each object in the Vector returned will be 
     * a ProductRow object
     *
     * @param familyInd The TypeCatInd of the Family to read for
     *
     * @return Vector
     **/
   public static Vector getProductTemplatesFromFamily ( int familyInd ) throws SQLException {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getProductsFromFamily ( "+familyInd+" )", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.PRODUCTIND, A.EXISTINGPRODUCTIND, A.PRODNUM,       A.DESCRIPTION, "+
                                              "       A.CREATEDON,  A.CREATEDBY,          A.LASTTOUCHEDBY, A.LASTTOUCHEDON "+
                                              "FROM PRODRAFT.PRODUCT A, "+
                                              "     PRODRAFT.CATEGORY B "+
                                              "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      A.TEMPLATE   = 'Y' AND "+
                                              "      B.TYPECATIND = "+familyInd+" FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            returnVec.addElement ( new ProductRow ( rs.getInt ( 1 ),
                                                    0,
                                                    rs.getString(2).trim(),
                                                    rs.getString(3).trim(),
                                                    rs.getString(4).trim(),
                                                    rs.getString(5).trim(),
                                                    "",
                                                    "",
                                                    rs.getString(6).trim(),
                                                    rs.getString(7),
                                                    false ) ) ;
         }

         rs.close() ;                        

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, grab the options information                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec.size() > 0 )
         populateDraftOptionPartNumbers ( returnVec ) ;

      return( returnVec ) ;
   }


   /**
     *  This method will return all Products in history that fall under the
     * family passed in. Each object in the Vector returned will be 
     * a ProductRow object
     *
     * @param familyInd The TypeCatInd of the Family to read for
     *
     * @return Vector
     **/
   public static Vector getProductHistoryFromFamily ( int familyInd ) throws SQLException {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getProductsFromFamily ( "+familyInd+" )", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.PRODUCTIND, A.EXISTINGPRODUCTIND, A.PRODNUM,       A.DESCRIPTION, "+
                                              "       A.CREATEDON,  A.CREATEDBY,          A.LASTTOUCHEDBY, A.LASTTOUCHEDON "+
                                              "FROM PRODRAFT.PRODUCT A, "+
                                              "     PRODRAFT.CATEGORY B "+
                                              "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      A.TEMPLATE   = 'N' AND "+
                                              "      B.TYPECATIND = "+familyInd+" FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            returnVec.addElement ( new ProductRow ( rs.getInt ( 1 ),
                                                    0,
                                                    rs.getString(2).trim(),
                                                    rs.getString(3).trim(),
                                                    rs.getString(4).trim(),
                                                    rs.getString(5).trim(),
                                                    "",
                                                    "",
                                                    rs.getString(6).trim(),
                                                    rs.getString(7),
                                                    false ) ) ;
         }

         rs.close() ;                        

      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, grab the options information                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec.size() > 0 )
         populateDraftOptionPartNumbers ( returnVec ) ;

      return( returnVec ) ;
   }



   /**
     * This method will return the product indicator for the model passed in
     *
     * @param model The model to read
     *
     * @return int
     **/
   public static int getProductIndFromModel ( TypeCategoryRec model ) throws SQLException {
      int         productInd = 0 ;
      SQLMethod   sqlMethod  = new SQLMethod ( DatabaseSystem.QUEST, "ProductSQL.getProductIndFromModel ( "+model.toString()+" )", 5 ) ;

      try {
         Statement   stmt       = sqlMethod.createStatement() ;
         ResultSet   rs         = stmt.executeQuery ( "SELECT PRODUCTIND FROM PRODUCT.CATEGORY WHERE TYPECATIND = "+model.getInd()+" FOR FETCH ONLY" ) ;

         if ( rs.next() )
            productInd = rs.getInt(1) ;

         rs.close();
      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( productInd ) ;
   }

   /**
     * This method will return the draft product indicator for the model passed in
     *
     * @param model The model to read
     *
     * @return int
     **/
   public static int getDraftProductIndFromModel ( TypeCategoryRec model ) throws SQLException {
      int         productInd = 0 ;
      SQLMethod   sqlMethod  = new SQLMethod ( DatabaseSystem.QUEST, "ProductSQL.getDraftProductIndFromModel ( "+model.toString()+" )", 5 ) ;

      try {
         Statement   stmt       = sqlMethod.createStatement() ;
         ResultSet   rs         = stmt.executeQuery ( "SELECT PRODUCTIND FROM PRODRAFT.CATEGORY WHERE TYPECATIND = "+model.getInd()+" FOR FETCH ONLY" ) ;

         if ( rs.next() )
            productInd = rs.getInt(1) ;

         rs.close();
      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( productInd ) ;
   }


   /**
     * This method will return the existing product indicator from a draft product indicator
     **/
   public static int getExistingProductInd ( int productInd ) throws SQLException {
      int         existingProductInd = 0 ;
      SQLMethod   sqlMethod          = new SQLMethod ( DatabaseSystem.QUEST, "ProductSQL.getExistingProductInd ( "+productInd+" )", 5 ) ;

      try {
         Statement   stmt               = sqlMethod.createStatement() ;
         ResultSet   rs                 = stmt.executeQuery ( "SELECT EXISTINGPRODUCTIND FROM PRODRAFT.PRODUCT WHERE PRODUCTIND="+productInd+" FOR FETCH ONLY" ) ;

         if ( rs.next() )
            existingProductInd = rs.getInt(1) ;

         rs.close();
      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         sqlMethod.close() ;
         throw ( e ) ;
      }

      sqlMethod.close() ;

      return( existingProductInd ) ;
   }


   /*------------------------------------------------------------*/
   /* getDBTimestampFromDatabase
    */
   /**
    *    This method should never be called except
    *  for the getDBTimestamp method and the
    *  time keeping thread. Please do not ever call it!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getDBTimestampFromDatabase() {
      String    retTimestamp = null; 
      SQLMethod sql          = null ;

      try {
         Statement stmt = null ;
         ResultSet rs   = null ;

         sql  = new SQLMethod ( DatabaseSystem.QUEST, "getDBTimestamp", 5 ) ;

         if ( sql != null ) {
            stmt = sql.createStatement() ;
            rs   = stmt.executeQuery ( "SELECT CURRENT TIMESTAMP FROM TIGRIS.USERID FETCH FIRST 1 ROWS ONLY" ) ;

            if ( rs.next() )
               retTimestamp = rs.getString(1).trim() ;

            rs.close() ;
         }
      } catch ( Exception e ) {
         sql.rollBack() ;
      }

      if ( sql != null )
         sql.close() ;

      return( retTimestamp ) ;
   }


} /* ProductSQL */
