package com.ibm.nzna.projects.common.quest.product ;
/*-----------------------------------------------------------*/
/*   TestNewProduct                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TestNewProduct                      */      
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.product.* ; 
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This small program will create a new product in the database 
 *
 *
 * @version 1.0, 6/20/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TestNewProduct                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class TestNewProduct extends Object {

    private String databaseName      = null ;
    private String server            = null ;
    private String userId            = null ;
    private String password          = null ;
    private String portNumber        = null ;
    private int    prodIndex         = 1 ;


    /* Main  */
    public static void main ( String args[] ) {
        new com.ibm.nzna.shared.util.LogSystem ( 1, "TestNewProduct.out" ) ;
        new TestNewProduct ( args ) ;
    }




    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method kicks off our test
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public TestNewProduct ( String args[] ) {
        try {
            databaseName=args[0] ;
            server=args[1] ;
            portNumber=args[2] ;
            userId=args[3] ;
            password=args[4] ;
        } catch ( Exception e ) {
            System.out.println ( "Usage:\njava com.ibm.nzna.projects.common.quest.product.TestNewProduct <databaseName> <serverName> <portNumber> <userId> <password>" ) ;
            System.exit(0) ;
        }


        try {
            DatabaseSystem d = new DatabaseSystem ( false, null, false ) ;

            System.out.print ( "Connecting to Quest..." ) ;
            if ( d.createConnection ( DatabaseSystem.QUEST,
                                      databaseName,
                                      server,
                                      portNumber,
                                      userId,
                                      password ) ) {

                System.out.println ( "Done" ) ;
                for ( int i = 0; i < 10; i++ )
                    createProduct() ;

            } else
                System.out.println ( "Could not connect to "+databaseName+" on "+server+" on port "+portNumber ) ;
        } catch ( Exception e1 ) {
            e1.printStackTrace() ;
            System.out.println ( "Could not connect to "+databaseName+" on "+server+" on port "+portNumber ) ;

        }
        System.exit(0) ;
    }


    /**
      *  This method will create the product
      **/
    private void createProduct () {
        Product         product  = new Product() ;
        TypeCategoryRec machine  = null ; 
        TypeCategoryRec model    = null ;
        TypeCategoryRec project  = null ;
        TypeCategoryRec family   = TypeCategory.getCategory ( "PS/1", TypeCategory.LEVEL_FAMILY ) ;
        TypeCategoryRec brand    = TypeCategory.getCategory ( "IBM Aptiva", TypeCategory.LEVEL_BRAND ) ;
        int             rc       = 0 ;

        new TypeCategoryRec ( "216"+prodIndex, TypeCategory.LEVEL_MACHINE ) ;
        product.setFamily  ( family ) ;
        product.setBrand   ( brand ) ;
        project = TypeCategory.createTypeCategory ( family.getInd(),
                                                     "216"+prodIndex+"  Project!",
                                                     TypeCategory.LEVEL_PROJECT,
                                                     0,
                                                     "N",
                                                     "QUEST" ) ;
        machine = TypeCategory.createTypeCategory ( project.getInd(),
                                                    "216"+prodIndex,
                                                    TypeCategory.LEVEL_MACHINE,
                                                    0,
                                                    "N",
                                                    "QUEST" ) ;
        model = TypeCategory.createTypeCategory ( machine.getInd(),
                                                  "5"+prodIndex+"C",
                                                  TypeCategory.LEVEL_MODEL,
                                                  0,
                                                  "N",
                                                  "QUEST" ) ;


        product.setDescription ( "This is a new product, Product Number "+prodIndex++ ) ;
        product.setMachine ( machine ) ;
        product.setModel   ( model ) ;
        product.setProject ( project ) ;
        product.setProductType    ( 1 ) ;
        product.addXRef    ( new XRef ( product.getProductInd(), 0, "2168-123" ) ) ;
        product.addXRef    ( new XRef ( product.getProductInd(), 0, "2168-124" ) ) ;
        product.addXRef    ( new XRef ( product.getProductInd(), 0, "2168-125" ) ) ;

        System.out.print ( "Saving Product  " ) ;
        long currentTime=System.currentTimeMillis() ;
        rc = product.writeToDatabase() ;
        switch ( rc ) {
        case 0 :
            System.out.println ( "OK! Product Created:"+product.getProductInd() ) ;
            break ;

        default:
            System.out.println ( "Failed. RC:"+rc ) ;
        }

        System.out.println ( "Product Save Time:"+(System.currentTimeMillis()-currentTime)+"(ms)" ) ;
    }


    public static void showProds () {
        SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "r", 5 ) ;

        LogSystem.log ( 1, "Creating SQL Method" ) ;
        try {
            Statement stmt =sqlMethod.createStatement() ;
            ResultSet rs = stmt.executeQuery ( "SELECT PRODUCTIND FROM PRODUCT.PRODUCT" ) ;

            while ( rs.next () ) {
                System.out.println ( rs.getInt(1) ) ;
            }
            rs.close() ;
        } catch ( Exception e ) {
            e.printStackTrace() ;
        }
        sqlMethod.close() ;
    }
} /* TestNewProduct */
