package com.ibm.nzna.projects.common.quest.product;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      XREF
/*   Creation Date: APRIL 16, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
/**
 * The XRef object represents a single row in the PRODUCT.XREF table. This is
 * the association of a Product to a Number
 *
 *
 * @author  David Lentz
 * @version 1
 */

/*----------------------------------------------------
/*
/* XRef
/*
/*----------------------------------------------------*/
public class XRef extends PersistentRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     * The Primary Key for the Product
     **/
   private int productInd = 0 ;

   /**
     * The type of cross reference this association is
     **/
   private int typeXRefInd = 0 ;

   /**
     * This is the actual number that the XRef represents
     **/
   private String number = null ;


   /**
     *  This Constructor creates a default, new XRef associated to
     * the product number given. Use this constructor if you wish to
     * create a brand new XRef for a product
     **/
   public XRef ( int productInd ) {
      this.productInd = productInd ;
      updateRecStatus ( REC_INSERT ) ;
   }

   /**
     *  This Constructor creats a default XRef that is already in the
     * database.
     **/
   public XRef ( int productInd, int typeXRefInd, String number ) {
      this.productInd = productInd ;
      setTypeXRefInd ( typeXRefInd ) ;
      setNumber ( number ) ;
      updateRecStatus ( REC_NOCHANGE ) ;
   }


   /**
     *  This method willr return the number that this XRef represents
     **/
   public String getNumber () {
      return( number ) ;
   }

   /**
     *  This method will set the number that this record represents
     **/
   public void setNumber ( String number ) throws IllegalArgumentException {
      if ( ( number != null ) && ( number.length() < 12 ) ) {
         this.number = number ;
         updateRecStatus ( REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Number cannot be NULL or longer "+
                                              "than 12 characters" ) ;
   } /* setNumber  */

   /**
     *  This method will return the type of XRef association this is
     **/
   public int getTypeXRefInd () {
      return( typeXRefInd ) ;
   }

   /**
     *  This method will set the type of XRef that this association represents
     **/
   public void setTypeXRefInd ( int typeXRefInd ) {
      this.typeXRefInd = typeXRefInd ;
      updateRecStatus ( REC_UPDATE ) ;
   }


   /**
     *  This method will return a String of SQL that will save this record to the
     * database. The string returned will have the SqlRunner.END_DELIM character
     * appended. The SQL Returned is suitable for the PRODUCT.XREF table.
     *
     * @return String
     **/
   public String toSQL () {
      StringBuffer buffer = null ;

      switch ( getRecStatus() ) {
         

         case REC_INSERT :
            buffer = new StringBuffer ( "INSERT INTO PRODUCT.XREF ( PRODUCTIND, TYPEXREFIND, NUMBER ) VALUES (" ) ;
            buffer.append ( productInd ) ;
            buffer.append ( "," ) ;
            buffer.append ( typeXRefInd ) ;
            buffer.append ( ",'" ) ;
            buffer.append ( number ) ;
            buffer.append ( "')" ) ;
            buffer.append ( SqlRunner.END_DELIM ) ;
            break ;

         case REC_UPDATE :
            buffer = new StringBuffer ( "UPDATE PRODUCT.XREF SET " ) ;
            buffer.append ( "NUMBER = '"+number+"'" ) ;
            buffer.append ( "WHERE PRODUCTIND = "+productInd+" AND TYPEXREFIND + "+typeXRefInd ) ;
            buffer.append ( SqlRunner.END_DELIM ) ;
            break ;

         case REC_DELETE :
            buffer = new StringBuffer ( "DELETE FROM PRODUCT.XREF " ) ;
            buffer.append ( "WHERE PRODUCTIND  = "+productInd+" AND " ) ;
            buffer.append ( "      TYPEXREFIND = "+typeXRefInd+" AND " ) ;
            buffer.append ( "      NUMBER      = '"+number+"'" ) ;
            buffer.append ( SqlRunner.END_DELIM ) ;
            break ;

      }

      if ( buffer != null ) {
         return( buffer.toString() ) ;
      }
      return( "" ) ;
   } /* toSQL  */

   /**
     *  This method will return a String of SQL that will save this record to the
     * database. The string returned will have the SqlRunner.END_DELIM character
     * appended. The SQL Returned is suitable for the PRODRAFT.XREF table.
     *
     * @return String
     **/
   public String toDraftSQL () {
      StringBuffer buffer = null ;

      switch ( getRecStatus() ) {
         

         case REC_INSERT :
            buffer = new StringBuffer ( "INSERT INTO PRODRAFT.XREF ( PRODUCTIND, TYPEXREFIND, NUMBER ) VALUES (" ) ;
            buffer.append ( productInd ) ;
            buffer.append ( "," ) ;
            buffer.append ( typeXRefInd ) ;
            buffer.append ( ",'" ) ;
            buffer.append ( number ) ;
            buffer.append ( "')" ) ;
            buffer.append ( SqlRunner.END_DELIM ) ;
            break ;

         case REC_UPDATE :
            buffer = new StringBuffer ( "UPDATE PRODRAFT.XREF SET " ) ;
            buffer.append ( "NUMBER = '"+number+"'" ) ;
            buffer.append ( "WHERE PRODUCTIND = "+productInd+" AND TYPEXREFIND + "+typeXRefInd ) ;
            buffer.append ( SqlRunner.END_DELIM ) ;
            break ;

         case REC_DELETE :
            buffer = new StringBuffer ( "DELETE FROM PRODRAFT.XREF " ) ;
            buffer.append ( "WHERE PRODUCTIND  = "+productInd+" AND " ) ;
            buffer.append ( "      TYPEXREFIND = "+typeXRefInd+" AND " ) ;
            buffer.append ( "      NUMBER      = '"+number+"'" ) ;
            buffer.append ( SqlRunner.END_DELIM ) ;
            break ;

      }

      if ( buffer != null ) {
         return( buffer.toString() ) ;
      }
      return( "" ) ;
   } /* toDraftSQL  */


   /**
     *  This method will return a simple string describing all properties
     * of this class
     **/
   public String toOut () {
      return ( "XRef Product Ind: "+productInd+"\n"+
               "Type XRef:        "+typeXRefInd+"\n"+
               "Number:           "+number+"\n" ) ;
   }


}
/* XRef */
