package com.ibm.nzna.projects.common.quest.status ;
/*-----------------------------------------------------------*/
/*   Status                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Status                              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
/**
 * This object holds static methods useful for manipulating types of Status in the Quest database. 
 * Remember that the TIGRIS.TYPESTATUS holds different status types for both documents and products!
 *
 * Before using any of the methods in this class, you must first have already created an Instance of LogSystem,
 * and created an instance of TypeList or the methods will fail
 *
 * @version 1.0, 3/22/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Status                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class Status extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Status Types                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int TYPESTATUS_ANNOUNCED                 = 0 ;
   public static final int TYPESTATUS_ARTICLENOTVERIFIED        = 4 ;
   public static final int TYPESTATUS_ARTICLEVERIFIED           = 5 ;
   public static final int TYPESTATUS_CLOSEDNOTAPPROVED         = 6 ;
   public static final int TYPESTATUS_APPROVED                  = 8 ;
   public static final int TYPESTATUS_INPROCESS                 = 9 ;
   public static final int TYPESTATUS_NEWREQUEST                = 10 ;
   public static final int TYPESTATUS_RETURNEDTOOWNER           = 11 ;
   public static final int TYPESTATUS_INSUFFICIENTDATA          = 12 ;
   public static final int TYPESTATUS_VALIDATED                 = 13 ;
   public static final int TYPESTATUS_FLAWED                    = 14 ;


   public static final int STATUS_DOCUMENT                      = 1 ;
   public static final int STATUS_PRODUCT                       = 2 ;
   public static final int STATUS_OA                            = 4 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Initializer only reads in the type lists         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static {
   }

   /*-----------------------------------------------------------*/
   /* getStatusList
    */
   /**
    *    This method will return a new vector of
    *  status with the type passed in
    *  
    *
    *  @param typeStatus The type of status to get
    *  @param None
    *  @param None
    *
    *  @return Vector of status
    *//*---------------------------------------------------------*/
   public static Vector getStatusList ( int typeStatus ) {
      Vector        retVec    = new Vector ( 10, 10 ) ;
      int           i         = 0 ;
      Vector        typeList  = TypeList.getInstance().getTypeList ( TypeList.TYPE_STATUS ) ;
      int           length    = typeList.size() ;
      TypeStatusRec statusRec = null ;

      while ( i < length ) {
         statusRec = (TypeStatusRec) typeList.elementAt(i) ;
         if ( ( statusRec != null ) && 
              ( statusRec.getStatusType() == typeStatus ) &&
              ( statusRec.getInd() != TYPESTATUS_APPROVED ) ) //Never let them select approved!
            retVec.addElement(statusRec) ;
         i++ ;
      }

      retVec.trimToSize() ;
      return ( retVec ) ;
   }

   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh a Choice control
    *  with a specific type of status
    *  
    *
    *  @param choice The Choice control to populate
    *  @param typeStatus The type of status to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( Choice choice, int typeStatus ) {
      int           i         = 0 ;
      Vector        typeList  = TypeList.getInstance().getTypeList ( TypeList.TYPE_STATUS ) ;
      int           length    = typeList.size() ;
      TypeStatusRec statusRec = null ;

      while ( i < length ) {
         statusRec = (TypeStatusRec) typeList.elementAt(i) ;
         if ( ( statusRec != null ) && ( statusRec.getStatusType() == typeStatus ) )
            choice.addItem ( statusRec.toString() ) ;
         i++ ;
      } /* while */

   } /* refreshList */


   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh a Choice control
    *  with a specific type of status
    *  
    *
    *  @param list A list to populate
    *  @param typeStatus The type of status to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( JList list, int typeStatus ) {
      int           i         = 0 ;
      Vector        typeList  = TypeList.getInstance().getTypeList ( TypeList.TYPE_STATUS ) ;
      int           length    = typeList.size() ;
      TypeStatusRec statusRec = null ;
      Vector        vec       = new Vector ( 10, 10 ) ;

      while ( i < length ) {
         statusRec = (TypeStatusRec) typeList.elementAt(i) ;
         if ( ( statusRec != null ) && ( statusRec.getStatusType() == typeStatus ) )
            vec.addElement ( statusRec) ;
         i++ ;
      } /* while */

      list.setListData ( vec ) ;

   } /* refreshList */

   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh a Choice control
    *  with a specific type of status
    *  
    *
    *  @param choice The Choice control to populate
    *  @param typeStatus The type of status to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( JComboBox comboBox, int typeStatus ) {
      int           i         = 0 ;
      Vector        typeList  = TypeList.getInstance().getTypeList ( TypeList.TYPE_STATUS ) ;
      int           length    = typeList.size() ;
      TypeStatusRec statusRec = null ;

      while ( i < length ) {
         statusRec = (TypeStatusRec) typeList.elementAt(i) ;
         if ( ( statusRec != null ) && ( statusRec.getStatusType() == typeStatus ) )
            comboBox.addItem ( statusRec ) ;
         i++ ;
      } /* while */

   } /* refreshList */

   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh a ContainerWin control
    *  with a specific type of status
    *  
    *
    *  @param choice The Choice control to populate
    *  @param typeStatus The type of status to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( MultiList cnr, int typeStatus ) {
      int           i         = 0 ;
      Vector        typeList  = TypeList.getInstance().getTypeList ( TypeList.TYPE_STATUS ) ;
      int           length    = typeList.size() ;
      TypeStatusRec statusRec = null ;

      while ( i < length ) {
         statusRec = (TypeStatusRec) typeList.elementAt(i) ;
         if ( ( statusRec != null ) && ( statusRec.getStatusType() == typeStatus ) )
            cnr.add ( statusRec ) ;
         i++ ;
      } /* while */
   } /* refreshList */



} /* Status */
