package com.ibm.nzna.projects.common.quest.type;
/*-----------------------------------------------------------*/
/*   Source File
/*
/*   Filename:      TYPECATEGORY
/*   Creation Date: APRIL 11, 2001
/*
/*
/*
/*         @Copyright  2001
/*          All Rights Reserved
/*
/*
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.storedProc.extSqlRunner.* ;
/**
 * This is the class responsible for holding the integrity between Java and
 * DB/2. This class holds all of the methods needed to read/update/delete/create
 * TypeCategory records in the PRODUCT.TYPECATEGORY table.<br><br>
 *
 * Please note that there should only be 1 instance of the TypeCategory class
 * used per JVM.
 *
 *
 * @author  David Lentz
 * @version 1
 * @since 4/11/2001
 */

/*----------------------------------------------------
/*
/* TypeCategory
/*
/*----------------------------------------------------*/
public class TypeCategory {

   /**
     *  This constant is defined to hold the value for
     * the Brands in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_BRAND    = "brand" ;

   /**
     *  This constant is defined to hold the value for
     * the Families in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_FAMILY   = "family" ;

   /**
     *  This constant is defined to hold the value for
     * the Machines in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_MACHINE  = "machine" ;

   /**
     *  This constant is defined to hold the value for
     * the Models in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_MODEL    = "model" ;

   /**
     *  This constant is defined to hold the value for
     * the Part Numbers in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_PARTNUMBER    = "partnum" ;

   /**
     *  This constant is defined to hold the value for
     * the OPIC/M Brands in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_OPICM_BRAND = "opicmbrand" ;

   /**
     *  This constant is defined to hold the value for
     * the OPIC/M Families in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_OPICM_FAMILY = "opicmfamily" ;


   /**
     *  This constant is defined to hold the value for
     * the OPIC/M Series in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_OPICM_SERIES = "opicmseries" ;


   /**
     *  This constant is defined to hold the value for
     * the OPIC/M Countries in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_OPICM_COUNTRY = "opicmcountry" ;


   /**
     *  This constant is defined to hold the value for
     * the OPIC/M Geographies in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_OPICM_GEOGRAPHY = "opicmgeo" ;


   /**
     *  This constant is defined to hold the value for
     * the Document Categories in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_DOC_CATEGORY = "doccategory" ;


   /**
     *  This constant is defined to hold the value for
     * the Projects in the PRODUCT.TYPECATEGORY table's CATEGORYLEVEL column
     **/
   public static final String LEVEL_PROJECT = "project" ;

   /**
     *  This constant is defined to hold the value for
     * the Family Type in the PRODUCT.TYPECATEGORY table
     **/
   public static final String LEVEL_SUBCATEGORY = "subcategory" ;


   /**
     *  This Vector holds all registered listeners
     **/
   private static Vector listeners = new Vector ( 1 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * This cached list of categories that have been read from the database
     *
     **/
   private static Vector categories    = null ;


   /**
     * This property enables/disables caching. If caching is enabled,
     * this object will read in the categories from disk that were read
     * in the last time this object was instantiated. Each category
     * that has been cached will be checked for updates. The cache filename is
     * TypeCategory.cache
     **/
   private static boolean cache = false ;

   /**
     * The Cache Directory is the directory that the cache file is written
     * to. The cache filename is TypeCategory.cache
     **/
   private static String cacheDirectory = null ;

   /**
     *  This is the vector which holds only memory cached draft categories
     **/
   private static Vector draftCategories = null ;

   /**
    * This is the default constructor. It will create an instance of the
    * TypeCategory class without caching.
    */
   public TypeCategory () {
   }

   /**
    * This constructor will create a TypeCategory class with caching enabled
    * to the directory passed.
    */
   public TypeCategory ( String cacheDirectory ) {
      setCache ( true ) ;
      setCacheDirectory ( cacheDirectory ) ;
   }

   /**
    * This method will return the categories that match the categoryLevel passed
    * in. If these categories have not been read in then ALL categories
    * that match the categoryLevel passed in will be read in. Please pay
    * close attention to how many TypeCategoryRec objects are created from this
    * method. CategoryLevels such as machine and model should not normally be
    * read in through this method as they contain hundreds and thousands of
    * objects. This class will support reading in such a large list but it is
    * not recommended performance wise.
    * @author David Lentz
    */
   public static Vector getCategories ( String categoryLevel ) {
      Vector returnVec = readCategoriesFromMemory ( categoryLevel ) ;                                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the returnVec is null, then we do not have the list  */
      /* in memory. We should read that in now                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec == null ) {
         readCategoriesFromDatabase ( -1, categoryLevel ) ;
         returnVec = readCategoriesFromMemory ( categoryLevel ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Never pass back a NULL, pass back an empty Vector       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec == null )
         returnVec = new Vector () ;


      return( returnVec ) ;
   }

   /**
    * This method will return all children categories
    * under the parentCatInd passed in. If the child
    * categories have not been read in yet then they
    * will be read now. Please look at the warning in method
    * getCategories regarding large sets of data. The
    * Vector returned will contain TypeCategoryRec objects.
    * @since 4/11/2001 
    */
   public static Vector getChildCategories ( int parentCatInd ) {
      Vector returnVec = readCategoriesFromMemory ( parentCatInd ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the returnVec is null, then we do not have the list  */
      /* in memory. We should read that in now                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec == null ) {
         readCategoriesFromDatabase ( parentCatInd, "" ) ;
         returnVec = readCategoriesFromMemory ( parentCatInd ) ;
      }

      return( returnVec ) ;
   }

   /**
    * This method will return all child categories that fall under the parentCatInd 
    * passed. Each child category returned will be of type categoryLevel passed. The
    * Vector returned will contain TypeCategoryRec objects. Please see the warning 
    * on the getCategories method regarding large data sets. 
    */
   public static Vector getChildCategories ( int parentCatInd, String categoryLevel ) {
      Vector returnVec = readCategoriesFromMemory ( parentCatInd, categoryLevel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the returnVec is null, then we do not have the list  */
      /* in memory. We should read that in now                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec == null )
         returnVec = readCategoriesFromDatabase ( parentCatInd, categoryLevel ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Never pass back a NULL, pass back an empty Vector       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec == null )
         returnVec = new Vector () ;

      return( returnVec ) ;
   }


   /**
    * This method will return all child categories that fall under the parentCatInd 
    * passed. Each child category returned will be of type categoryLevel passed. The
    * Vector returned will contain TypeCategoryRec objects. Please see the warning 
    * on the getCategories method regarding large data sets. 
    */
   public static Vector getChildDraftCategories ( int parentCatInd, String categoryLevel ) {
      Vector returnVec = readDraftCategoriesFromMemory ( parentCatInd, categoryLevel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the returnVec is null, then we do not have the list  */
      /* in memory. We should read that in now                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( returnVec == null ) || ( returnVec.size() == 0 ) ) {
         returnVec = readDraftCategoriesFromDatabase ( parentCatInd, categoryLevel ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Never pass back a NULL, pass back an empty Vector       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnVec == null )
         returnVec = new Vector () ;

      return( returnVec ) ;
   }

   /**
    * This method will create a new TypeCategoryRec object that may be 
    * used immediatly. Please note however that this TypeCategoryRec 
    * will not be saved to the database until a call to 'writeToDatabase'
    * is made. 
    *
    * This 'lazy' method does not try to obtain a Primary Key for the
    * Type Category object created. To create a Type Category <b>AND</b>
    * obtain a Primary Key for it, use the normal createTypeCategory method. The
    * primary key will be obtained only when saving
    */
   public static TypeCategoryRec lazyCreateTypeCategory ( int parentCatInd,
                                                          String description,
                                                          String categoryLevel,
                                                          int    orderNum,
                                                          String childSort,
                                                          String owner ) throws IllegalArgumentException {
      TypeCategoryRec          newRec = new TypeCategoryRec() ;

      newRec.setParentCatInd  ( parentCatInd ) ;
      newRec.setDescript      ( description ) ;
      newRec.setCategoryLevel ( categoryLevel ) ;
      newRec.setOrderNum      ( orderNum ) ;
      newRec.setChildSort     ( childSort ) ;
      newRec.setOwner         ( owner ) ;
      newRec.updateRecStatus  ( newRec.REC_NOCHANGE ) ;
      newRec.updateRecStatus  ( newRec.REC_INSERT ) ;

      return( newRec ) ;
   }

   /**
    * This method will create a new TypeCategoryRec object that may be 
    * used immediatly. Please note however that this TypeCategoryRec 
    * will not be saved to the database until a call to 'writeToDatabase'
    * is made. 
    *
    * This 'lazy' method does not try to obtain a Primary Key for the
    * Type Category object created. To create a Type Category <b>AND</b>
    * obtain a Primary Key for it, use the normal createTypeCategory method. The
    * primary key will be obtained only when saving
    */
   public static TypeCategoryRec lazyCreateDraftTypeCategory ( int parentCatInd,
                                                               String description,
                                                               String categoryLevel,
                                                               int    orderNum,
                                                               String childSort,
                                                               String owner ) throws IllegalArgumentException {
      TypeCategoryRec          newRec = new TypeCategoryRec() ;

      newRec.setIsDraft       ( true ) ;
      newRec.setParentCatInd  ( parentCatInd ) ;
      newRec.setDescript      ( description ) ;
      newRec.setCategoryLevel ( categoryLevel ) ;
      newRec.setOrderNum      ( orderNum ) ;
      newRec.setChildSort     ( childSort ) ;
      newRec.setOwner         ( owner ) ;
      newRec.updateRecStatus  ( newRec.REC_NOCHANGE ) ;
      newRec.updateRecStatus  ( newRec.REC_INSERT ) ;

      return( newRec ) ;
   }

   /**
     * This method will create a new TypeCategoryRec object that may be 
     * used immediatly. Please note however that this TypeCategoryRec 
     * will not be saved to the database until a call to 'writeToDatabase'
     * is made. The TypeCatInd will be generated automatically from the
     * TIGRIS.COUNTERS table.<br><br>
     **/
   public static TypeCategoryRec createTypeCategory ( int parentCatInd,
                                                      String description,
                                                      String categoryLevel,
                                                      int    orderNum,
                                                      String childSort,
                                                      String owner ) throws IllegalArgumentException {
      TypeCategoryRec retRec = lazyCreateTypeCategory ( parentCatInd, description, categoryLevel, orderNum, childSort, owner ) ;
      retRec.generateKeys() ;
      addCategory ( retRec ) ;

      return( retRec ) ;
   }

   /**
     * This method will create a new TypeCategoryRec object that may be
     * used immediatly. Please note however that this TypeCategoryRec
     * will not be saved to the database until a call to 'writeToDatabase'
     * is made.
     *
     * The TypeCategoryRec object created using this method will create a
     * category in the PRODRAFT.TYPECATEGORY table.
     *
     */
   public static TypeCategoryRec createDraftTypeCategory ( int parentCatInd,
                                                           String description,
                                                           String categoryLevel,
                                                           int    orderNum,
                                                           String childSort,
                                                           String owner ) throws IllegalArgumentException {
      TypeCategoryRec newRec = lazyCreateDraftTypeCategory (parentCatInd,description,categoryLevel,orderNum,childSort,owner ) ;

      if ( newRec != null )
         newRec.generateKeys() ;

      addCategory ( newRec ) ;

      return( newRec ) ;
   }

   /**
     * This method will create a new TypeCategoryRec object that may be
     * used immediatly. Please note however that this TypeCategoryRec
     * will not be saved to the database until a call to 'writeToDatabase'
     * is made.
     *
     * The TypeCategoryRec object created using this method will create a
     * category in the PRODRAFT.TYPECATEGORY table.
     *
     *
     * <b>Note</b> DO NOT USE THIS METHOD UNLESS YOU ABSOLUTLY NEED TO
     * SPECIFY THE CATEGORY IND. If you DO NOT NEED to specify the
     * CATEGORY IND, you should use the other createTypeCategory.
     */
   public static TypeCategoryRec createDraftTypeCategory ( int typeCatInd,
                                                           int parentCatInd,
                                                           String description,
                                                           String categoryLevel,
                                                           int    orderNum,
                                                           String childSort,
                                                           String owner ) throws IllegalArgumentException {
      TypeCategoryRec          newRec = new TypeCategoryRec(typeCatInd) ;

      newRec.setParentCatInd  ( parentCatInd ) ;
      newRec.setDescript      ( description ) ;
      newRec.setCategoryLevel ( categoryLevel ) ;
      newRec.setOrderNum      ( orderNum ) ;
      newRec.setChildSort     ( childSort ) ;
      newRec.setOwner         ( owner ) ;
      newRec.setIsDraft       ( true ) ;
      newRec.updateRecStatus  ( newRec.REC_NOCHANGE ) ;
      newRec.updateRecStatus  ( newRec.REC_INSERT ) ;

      addCategory ( newRec );

      return( newRec ) ;
   }


   /**
    * This method will create a new TypeCategoryRec object that may be 
    * used immediatly. Please note however that this TypeCategoryRec 
    * will not be saved to the database until a call to 'writeToDatabase'
    * is made. 
    *
    * <b>Note</b> DO NOT USE THIS METHOD UNLESS YOU ABSOLUTLY NEED TO
    * SPECIFY THE CATEGORY IND. If you DO NOT NEED to specify the
    * CATEGORY IND, you should use the other createTypeCategory.
    */
   public static TypeCategoryRec createTypeCategory ( int typeCatInd,
                                                      int parentCatInd,
                                                      String description,
                                                      String categoryLevel,
                                                      int    orderNum,
                                                      String childSort,
                                                      String owner ) throws IllegalArgumentException {
      TypeCategoryRec          newRec = new TypeCategoryRec(typeCatInd) ;

      //newRec.setInd           ( typeCatInd ) ;
      newRec.setParentCatInd  ( parentCatInd ) ;
      newRec.setDescript      ( description ) ;
      newRec.setCategoryLevel ( categoryLevel ) ;
      newRec.setOrderNum      ( orderNum ) ;
      newRec.setChildSort     ( childSort ) ;
      newRec.setOwner         ( owner ) ;
      newRec.updateRecStatus  ( newRec.REC_NOCHANGE ) ;
      newRec.updateRecStatus  ( newRec.REC_INSERT ) ;

      addCategory ( newRec );

      return( newRec ) ;
   }

   /**
    * This method will write all changed TypeCategoryRecs that have been 
    * cached in memory to the database. When updating more than 1 TypeCategoryRec 
    * it is a good practice to make both modiciations and call this method 
    * once. 
    *
    * @returns 0 for success, an SQL Code of a failure
    */
   public static int writeToDatabase () {
      int          sqlCode   = -1 ;                         
      StringBuffer sqlString = new StringBuffer ( "" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, grab all of the SQL to save                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int          i         = 0 ;
         int          size      = categories.size() ;                   


         fireWriteStart () ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ask each object for it's SQL                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            sqlString.append ( ((TypeCategoryRec)categories.elementAt(i)).toSQL() ) ;
            i++ ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, grab all of the SQL to save                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         int          i         = 0 ;
         int          size      = draftCategories.size() ;                   


         fireWriteStart () ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ask each object for it's SQL                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            System.out.println ( "Saving Draft Category. Rec Status:"+((TypeCategoryRec)draftCategories.elementAt(i)).getRecStatus()+": "+((TypeCategoryRec)draftCategories.elementAt(i)).toString() ) ;
            sqlString.append ( ((TypeCategoryRec)draftCategories.elementAt(i)).toSQL() ) ;
            i++ ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, run the SQL                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( sqlString.length() > 0 ) {
         sqlCode = ExtSqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST,
                                                sqlString.toString() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, remove any invalid objects                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( sqlCode == 0 ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Normal Categories                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( categories != null ) && ( categories.size() > 0 ) ) {
               int          i         = 0 ;
               int          size      = categories.size() ;                   
               size=categories.size() ;

               while ( i < size ) {
                  if ( !((TypeCategoryRec)categories.elementAt(i)).valid() ) {
                     categories.removeElementAt(i) ;
                     size=categories.size() ;
                  } else {
                     ((TypeCategoryRec)categories.elementAt(i)).updateRecStatus ( TypeCategoryRec.REC_NOCHANGE ) ;
                     i++ ;
                  }
               }
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Draft  Categories                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
               int          i         = 0 ;
               int          size      = draftCategories.size() ;                   
               size=draftCategories.size() ;

               while ( i < size ) {
                  if ( !((TypeCategoryRec)draftCategories.elementAt(i)).valid() ) {
                     draftCategories.removeElementAt(i) ;
                     size=draftCategories.size() ;
                  } else {
                     ((TypeCategoryRec)draftCategories.elementAt(i)).updateRecStatus ( TypeCategoryRec.REC_NOCHANGE ) ;
                     i++ ;
                  }
               }
            }


         }

         fireWriteStop() ;
      } else
         sqlCode = 0 ;                      //Nothing was changed


      return( sqlCode ) ;
   }

   /**
    * This method will attempt to read all categories from the PRODUCT.TYPECATEGORY 
    * table which fall under the parentInd passed in. This method will return FALSE 
    * if an error occurs while reading the information. This method will return
    * a non zero if an error occurs while reading. It will return 0 if data was read   
    * This method uses the <b>TypeCatReader</b> stored
    * procedure to do it's database work.
    *
    * @return The Vector of categories read in
    */
   private static Vector readCategoriesFromDatabase ( int parentInd ) {
      return( readCategoriesFromDatabase ( parentInd, "" ) ) ;
   }

   /**
    * This method will attempt to read in the categories which fall under the parentCatInd 
    * passed in and which match the categoryLevel passed in. This method will return
    * a non zero if an error occurs while reading. It will return 0 if data was read   
    * This method uses the <b>TypeCatReader</b> stored
    * procedure to do it's database work.
    *
    * @param parentInd The parent category indicator to read. -1 should be passed
    *                  if the parentInd should not be used to read
    * @param categoryLevel The Category Level to read from. An empty string should
    *                  be passed in the category level should not be used to read
    *
    * @return The Vector of categories read in
    */
   private static Vector readCategoriesFromDatabase ( int parentInd,
                                                      String categoryLevel ) {
      Vector    returnVec = null ;
      String    rtnName   = "readCategoriesFromDatabase" ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;

      fireReadStart ( getDescriptFromCategoryLevel ( categoryLevel ) ) ;

      if ( sqlMethod != null ) {
         try {
            CallableStatement stmt  = sqlMethod.createStoredProc ( "QUEST.TypeCatReader", 4 ) ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up the call                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.setString            ( 1, categoryLevel ) ;
            stmt.setInt               ( 2, parentInd ) ;
            stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;
            stmt.registerOutParameter ( 4, Types.LONGVARBINARY ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, "Reading Categories. ParentCatInd:"+parentInd+"   Category Level:"+categoryLevel ) ;

            stmt.execute() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the StoredProcRec from the Lob                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 4 ) ;                


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Throw an error if need be                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( retRec != null ) && ( retRec.error ) ) {
               if ( LogSystem.getInstance() != null ) {
                  LogSystem.beginLogSection ( "TypeCatReader Returned Error" ) ;
                  LogSystem.log ( 1, "sqlCode:   "+retRec.sqlCode ) ;
                  LogSystem.log ( 1, "userError: "+retRec.userError ) ;
                  LogSystem.log ( 1, "errorStr:  "+retRec.errorStr ) ;
                  LogSystem.log ( 1, "error:     "+retRec.error ) ;
                  LogSystem.endLogSection () ;
               }
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, continue and add the new categories to our   */
            /* list                                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
               returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;

               if ( ( returnVec != null ) && ( returnVec.size() > 0 ) ) {
                  int i    = 0 ;
                  int size = returnVec.size() ;

                  while ( i < size ) {
                     addCategory ( (TypeCategoryRec)returnVec.elementAt(i) ) ;
                     i++ ;
                  }
               }

            } /* Else if we have no error  */

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            returnVec = null ;
         }
      } /* if sqlMethod is not null  */

      sqlMethod.close() ;
      fireReadStop() ;

      return( returnVec ) ;
   }


   /**
     *  This method will return all categories that have been read in
     * and have the categoryLevel matching the param passed in
     *
     * @param categoryLevel The CategoryLevel to match
     *
     * @returns A vector of TypeCategoryRe objects matching the CategoryLevel passed in, NULL if nothing matched
     **/
   private static Vector readCategoriesFromMemory ( String categoryLevel ) {
      Vector returnVec = new Vector ( 10 ) ;

      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int             i           = 0 ;
         int             size        = categories.size() ;

         while ( i < size ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).getCategoryLevel().equals ( categoryLevel ) ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) )
               returnVec.addElement ( categories.elementAt(i) ) ;
            i++ ;
         }
      }


      returnVec.trimToSize() ;
      if ( returnVec.size() == 0 )
         returnVec = null ;

      return( returnVec ) ;
   }






   /**
     *  This method will return all categories that have been read in
     * and have the categoryLevel and parentCatINd matching the param passed in
     *
     * @param parentCatInd The parent category to match
     * @param categoryLevel The CategoryLevel to match
     *
     * @returns A vector of TypeCategoryRec objects matching the CategoryLevel passed in, NULL if nothing matched
     **/
   private static Vector readDraftCategoriesFromMemory ( int parentCatInd, String categoryLevel ) {
      Vector returnVec = new Vector ( 10 ) ;

      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         TypeCategoryRec categoryRec = getDraftCategory ( parentCatInd ) ;
         int             i           = 0 ;
         int             size        = draftCategories.size() ;

         while ( i < size ) {
            if ( ( ((TypeCategoryRec)draftCategories.elementAt(i)).getCategoryLevel().equals ( categoryLevel ) ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getParentCatInd() == parentCatInd ) ) {
               returnVec.addElement ( draftCategories.elementAt(i) ) ;
            }
            i++ ;
         }
      }


      returnVec.trimToSize() ;
      if ( returnVec.size() == 0 )
         returnVec = null ;

      return( returnVec ) ;
   }



   /**
     *  This method will return all categories that have been read in
     * and have the categoryLevel and parentCatINd matching the param passed in
     *
     * @param parentCatInd The parent category to match
     * @param categoryLevel The CategoryLevel to match
     *
     * @returns A vector of TypeCategoryRec objects matching the CategoryLevel passed in, NULL if nothing matched
     **/
   private static Vector readCategoriesFromMemory ( int parentCatInd, String categoryLevel ) {
      Vector returnVec = new Vector ( 10 ) ;

      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         TypeCategoryRec categoryRec = getCategory ( parentCatInd ) ;
         int             i           = 0 ;
         int             size        = categories.size() ;

         while ( i < size ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).getCategoryLevel().equals ( categoryLevel ) ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getIsDraft() == categoryRec.getIsDraft() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getParentCatInd() == parentCatInd ) )
               returnVec.addElement ( categories.elementAt(i) ) ;
            i++ ;
         }
      }


      returnVec.trimToSize() ;
      if ( returnVec.size() == 0 )
         returnVec = null ;

      return( returnVec ) ;
   }




   /**
     *  This method will return all categories that have been read in
     * and have the parentCatInd matching the param passed in
     *
     * @param parentCatInd The parent category to match
     *
     * @returns A vector of TypeCategoryRec objects matching the parentCatInd passed in, NULL if nothing matched
     **/
   private static Vector readCategoriesFromMemory ( int parentCatInd ) {
      Vector            returnVec = new Vector ( 10 ) ;
      TypeCategoryRec   category  = getCategory ( parentCatInd ) ;

      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int i    = 0 ;
         int size = categories.size() ;

         while ( i < size ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).getParentCatInd() == parentCatInd ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getIsDraft() == category.getIsDraft() ) )
               returnVec.addElement ( categories.elementAt(i) ) ;
            i++ ;
         }
      }


      returnVec.trimToSize() ;
      if ( returnVec.size() == 0 )
         returnVec = null ;

      return( returnVec ) ;
   }





   /**
    * This method will turn caching either on or off. If caching is turned on
    * then all values will first try to be read from the local disk, then from
    * the database. Also, a check against everthing loaded from disk will occur
    * and any updates in the database will be read.
    */
   public static void setCache ( boolean cacheValue ) {
      cache = cacheValue ;
   }

   /**
     * This method will set the directory that the TypeCategory class will
     * write and read it's cached information to.
     */
   public static void setCacheDirectory ( String cacheDirectoryValue ) {
      cacheDirectory = cacheDirectoryValue ;
   }

   /**
    * This method returns the full path to where the cache files generated
    * by this class are kept
    **/
   public static String getCacheDirectory () {
      return( cacheDirectory ) ;
   }


   /**
     *  This method will return a TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database.
     * 
     * @param description The description to look for
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getCategoryFromDescription ( String description ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int i    = 0 ;
         int size = categories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).toString().equals ( description ) ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) &&
                 ( !((TypeCategoryRec)categories.elementAt(i)).getIsDraft() ) )

               returnRec = (TypeCategoryRec)categories.elementAt(i) ;
            i++ ;
         }

      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getCategoryFromDescription ( "+description+" )",
                                               5 ) ;

         fireReadStart ( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                          "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                          "       Owner "+
                                                          "from product.TypeCategory "+
                                                          "where description = '"+Text.cleanDBString ( description )+"' and "+
                                                          "      valid = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }


   /**
     *  This method will return a TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database.
     * 
     * @param description The description to look for
     * @param level The Category Level to look for
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getCategoryFromDescription ( String description, String level ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int i    = 0 ;
         int size = categories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).toString().equals ( description ) ) &&
                 ( !((TypeCategoryRec)categories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getCategoryLevel().equals ( level ) ) )
               returnRec = (TypeCategoryRec)categories.elementAt(i) ;
            i++ ;
         }

      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getCategoryFromDescription ( "+description+" )",
                                               5 ) ;

         fireReadStart ( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                          "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                          "       Owner "+
                                                          "from product.TypeCategory "+
                                                          "where description   = '"+description+"' and "+
                                                          "      categoryLevel = '"+level+"' AND "+
                                                          "      valid         = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }


   /**
     *  This method will return a TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database.
     * 
     * @param description The description to look for
     * @param parentCatInd The parent indicator to look under
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getCategoryFromDescription ( String description, int parentCatInd ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int i    = 0 ;
         int size = categories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).toString().equals ( description ) ) &&
                 ( !((TypeCategoryRec)categories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getParentCatInd() == parentCatInd ) )
               returnRec = (TypeCategoryRec)categories.elementAt(i) ;
            i++ ;
         }

      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getCategoryFromDescription ( "+description+" )",
                                               5 ) ;

         fireReadStart ( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select a.TypeCatInd,    a.Description, a.ParentCatInd, "+
                                                          "       a.CategoryLevel, a.OrderNum,    a.ChildSort,    "+
                                                          "       a.Owner "+
                                                          "from product.TypeCategory a, "+
                                                          "     product.typeCatChild b "+
                                                          "where a.description  = '"+description+"' and "+
                                                          "      a.typeCatInd   = b.typeCatInd and "+
                                                          "      b.parentCatInd = "+parentCatInd+" AND "+
                                                          "      a.valid        = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }


   /**
     *  This method will return a TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database.
     * 
     * @param description The description to look for
     * @param parentCatInd The parent indicator to look under
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getDraftCategoryFromDescription ( String description, int parentCatInd ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         int i    = 0 ;
         int size = draftCategories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)draftCategories.elementAt(i)).toString().equals ( description ) ) &&
                 ( !((TypeCategoryRec)draftCategories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getParentCatInd() == parentCatInd ) )
               returnRec = (TypeCategoryRec)draftCategories.elementAt(i) ;
            i++ ;
         }

      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getDraftCategoryFromDescription ( "+description+", "+parentCatInd+" )",
                                               5 ) ;

         fireReadStart ( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select a.TypeCatInd,    a.Description, a.ParentCatInd, "+
                                                          "       a.CategoryLevel, a.OrderNum,    a.ChildSort,    "+
                                                          "       a.Owner "+
                                                          "from prodraft.TypeCategory a, "+
                                                          "     prodraft.typeCatChild b "+
                                                          "where a.description  = '"+description+"' and "+
                                                          "      a.typeCatInd   = b.typeCatInd and "+
                                                          "      b.parentCatInd = "+parentCatInd+" AND "+
                                                          "      a.valid        = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.setIsDraft       ( true ) ;
               readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }


   /**
     *  This method will return a TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database.
     * 
     * @param description The description to look for
     * @param level The category level to look for
     * @param parentCatInd The parent indicator to look under
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getCategoryFromDescription ( String description, String level, int parentCatInd ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int i    = 0 ;
         int size = categories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).toString().equals ( description ) ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getCategoryLevel().equals ( level ) ) &&
                 ( !((TypeCategoryRec)categories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getParentCatInd() == parentCatInd ) )
               returnRec = (TypeCategoryRec)categories.elementAt(i) ;
            i++ ;
         }

      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getCategoryFromDescription ( "+description+" )",
                                               5 ) ;

         fireReadStart( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                          "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                          "       Owner "+
                                                          "from product.TypeCategory "+
                                                          "where description   = '"+description+"' and "+
                                                          "      categoryLevel = '"+level+"' and "+
                                                          "      parentCatInd  = "+parentCatInd+" and "+
                                                          "      valid         = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }


   /**
     *  This method will return a Draft TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database.
     * 
     * @param description The description to look for
     * @param level The category level to look for
     * @param parentCatInd The parent indicator to look under
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getDraftCategoryFromDescription ( String description, String level, int parentCatInd ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         int i    = 0 ;
         int size = draftCategories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)draftCategories.elementAt(i)).toString().equals ( description ) ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getCategoryLevel().equals ( level ) ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getParentCatInd() == parentCatInd ) )
               returnRec = (TypeCategoryRec)draftCategories.elementAt(i) ;
            i++ ;
         }

      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getDraftCategoryFromDescription ( "+description+" )",
                                               5 ) ;

         fireReadStart ( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                          "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                          "       Owner "+
                                                          "from prodraft.TypeCategory "+
                                                          "where description   = '"+description+"' and "+
                                                          "      categoryLevel = '"+level+"' and "+
                                                          "      parentCatInd  = "+parentCatInd+" and "+
                                                          "      valid         = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.setIsDraft       ( true ) ;
               readRec.updateRecStatus  ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }


   /**
     *  This method will return a Draft TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database.
     * 
     * @param description The description to look for
     * @param level The category level to look for
     * @param parentCatInd The parent indicator to look under
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getDraftCategoryFromDescription ( String description, String level ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         int i    = 0 ;
         int size = draftCategories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)draftCategories.elementAt(i)).toString().equals ( description ) ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getCategoryLevel().equals ( level ) ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).valid() ) )
               returnRec = (TypeCategoryRec)draftCategories.elementAt(i) ;
            i++ ;
         }

      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getDraftCategoryFromDescription ( "+description+" )",
                                               5 ) ;

         fireReadStart ( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                          "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                          "       Owner "+
                                                          "from prodraft.TypeCategory "+
                                                          "where description   = '"+description+"' and "+
                                                          "      categoryLevel = '"+level+"' and "+
                                                          "      valid         = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.setIsDraft       ( true ) ;
               readRec.updateRecStatus  ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }



   /**
     * This method will attempt to read a single category from the database. 
     *
     * @param typeCatInd The category indicator of the category to read in
     * @param level The category level of the category to read in
     *
     * @return The TypeCategoryRec read in
     */
   private static TypeCategoryRec readCategoryFromDatabase ( int typeCatInd, String level ) {
      SQLMethod        sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeCategory.readCategoryFromDatabase ( "+typeCatInd+", "+level+" )", 5 ) ;
      TypeCategoryRec  returnRec = null ;

      fireReadStart ( getDescriptFromCategoryLevel ( level ) ) ;
      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                              "       CategoryLevel, OrderNum,    ChildSort,    "+
                                              "       Owner "+
                                              "from product.TypeCategory "+
                                              "where typeCatInd = "+typeCatInd+" and "+
                                              "      categoryLevel = '"+level+"' and "+
                                              "      valid = 'Y' for Fetch only" ) ;

         if ( rs.next() ) {
            returnRec = new TypeCategoryRec ( rs.getInt(1) ) ;
            returnRec.setDescript      ( rs.getString(2).trim() ) ;
            returnRec.setParentCatInd  ( rs.getInt(3) ) ;
            returnRec.setCategoryLevel ( rs.getString(4).trim() ) ;
            returnRec.setOrderNum      ( rs.getInt(5) ) ;
            returnRec.setChildSort     ( rs.getString(6).trim() ) ;
            returnRec.setOwner         ( rs.getString(7) ) ;
            returnRec.updateRecStatus ( returnRec.REC_NOCHANGE ) ;

         }

         rs.close() ;

      } catch ( Exception e ) {
         returnRec = null ;
         sqlMethod.rollBack () ;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            e.printStackTrace() ;
      }

      sqlMethod.close() ;
      fireReadStop() ;

      return( returnRec ) ;
   }



   /**
     * This method will attempt to read a single category from the database. 
     *
     * @param typeCatInd The category indicator of the category to read in
     *
     * @return The TypeCategoryRec read in
     */
   private static TypeCategoryRec readCategoryFromDatabase ( int typeCatInd ) {
      SQLMethod        sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeCategory.readCategoryFromDatabase ( "+typeCatInd+" )", 5 ) ;
      TypeCategoryRec  returnRec = null ;

      fireReadStart ( ""+typeCatInd ) ;
      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                              "       CategoryLevel, OrderNum,    ChildSort,    "+
                                              "       Owner "+
                                              "from product.TypeCategory "+
                                              "where typeCatInd = "+typeCatInd+" and "+
                                              "      valid = 'Y' for Fetch only" ) ;

         if ( rs.next() ) {
            returnRec = new TypeCategoryRec ( rs.getInt(1) ) ;
            returnRec.setDescript      ( rs.getString(2).trim() ) ;
            returnRec.setParentCatInd  ( rs.getInt(3) ) ;
            returnRec.setCategoryLevel ( rs.getString(4).trim() ) ;
            returnRec.setOrderNum      ( rs.getInt(5) ) ;
            returnRec.setChildSort     ( rs.getString(6).trim() ) ;
            returnRec.setOwner         ( rs.getString(7) ) ;
            returnRec.updateRecStatus ( returnRec.REC_NOCHANGE ) ;

         }

         rs.close() ;

      } catch ( Exception e ) {
         returnRec = null ;
         sqlMethod.rollBack () ;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            e.printStackTrace() ;
      }

      sqlMethod.close() ;
      fireReadStop() ;

      return( returnRec ) ;
   }



   /**
     * This method will attempt to read a single category from the database. 
     *
     * @param categoryDescript The description of the category to read in
     * @param level The category level of the category to read in
     *
     * @return The TypeCategoryRec read in
     */
   private static TypeCategoryRec readCategoryFromDatabase ( String categoryDescript, String level ) {
      SQLMethod        sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeCategory.readCategoryFromDatabase ( "+categoryDescript+", "+level+" )", 5 ) ;
      TypeCategoryRec  returnRec = null ;

      fireReadStart ( getDescriptFromCategoryLevel ( level ) ) ;
      try {
         Statement stmt = sqlMethod.createStatement() ;
         if ( stmt != null ) {
            ResultSet rs   = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                 "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                 "       Owner "+
                                                 "from product.TypeCategory "+
                                                 "where Description   = '"+categoryDescript+"' and "+
                                                 "      categoryLevel = '"+level+"' and "+
                                                 "      valid         = 'Y' for Fetch only" ) ;

            if ( rs.next() ) {
               returnRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec.setDescript      ( rs.getString(2).trim() ) ;
               returnRec.setParentCatInd  ( rs.getInt(3) ) ;
               returnRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               returnRec.setOrderNum      ( rs.getInt(5) ) ;
               returnRec.setChildSort     ( rs.getString(6).trim() ) ;
               returnRec.setOwner         ( rs.getString(7) ) ;
               returnRec.updateRecStatus ( returnRec.REC_NOCHANGE ) ;

            }

            rs.close() ;
         } else {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, "TypeCategoryRec.readCategoryFromDatabase could not establish a Statement object" ) ;
         }

      } catch ( Exception e ) {
         returnRec = null ;
         sqlMethod.rollBack () ;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            e.printStackTrace() ;
      }

      sqlMethod.close() ;
      fireReadStop() ;

      return( returnRec ) ;
   }

   /**
     * This method will return a single category based on the
     * category indicator passed in. If the category
     * is not found in memory then this method will try to read
     * the information from the database.
     *
     * @param typeCatInd The Category Indicator to read
     */
   public static TypeCategoryRec getCategory ( int typeCatInd ) {
      TypeCategoryRec returnRec        = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First try to read the information from cached memory    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int         i    = 0 ;
         int         size = categories.size() ;

         while ( ( returnRec == null ) && ( i < size ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).getInd() == typeCatInd ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) )
               returnRec = (TypeCategoryRec) categories.elementAt(i) ;
            i++ ;
         } /* while  */

      } /* if we have categories to look through  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to read this information from the database,   */
      /* do so now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         returnRec = readCategoryFromDatabase (  typeCatInd ) ;

         /* Add this for future reading  */
         addCategory ( returnRec ) ;
      }

      return( returnRec ) ;
   }
   /**
     * This method will return a single category based on the
     * category indicator and level passed in. If the category
     * is not found in memory then this method will try to read
     * the information from the database.
     *
     * @param typeCatInd The Category Indicator to read
     * @param level The Category level to read
     */
   public static TypeCategoryRec getCategory ( int typeCatInd, String level ) {
      TypeCategoryRec returnRec        = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First try to read the information from cached memory    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int         i    = 0 ;
         int         size = categories.size() ;

         while ( ( returnRec == null ) && ( i < size ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).getInd() == typeCatInd ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getCategoryLevel().equals ( level ) ) )
               returnRec = (TypeCategoryRec) categories.elementAt(i) ;
            i++ ;
         } /* while  */

      } /* if we have categories to look through  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to read this information from the database,   */
      /* do so now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         returnRec = readCategoryFromDatabase (  typeCatInd, level ) ;

         /* Add this for future reading  */
         addCategory ( returnRec ) ;
      }

      return( returnRec ) ;
   }


   /**
     * This method will return a single category based on the
     * category description and level passed in. If the category
     * is not found in memory then this method will try to read
     * the information from the database.
     *
     * @param description The Category description to read
     * @param categoryLevel The Category categoryLevel to read
     */
   public static TypeCategoryRec getCategory ( String description, String categoryLevel ) {
      TypeCategoryRec returnRec        = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First try to read the information from cached memory    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int i    = 0 ;
         int size = categories.size() ;

         while ( ( returnRec == null ) && ( i < size ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).getDescript().equals ( description ) ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getCategoryLevel().equals ( categoryLevel ) ) )
               returnRec = (TypeCategoryRec) categories.elementAt(i) ;
            i++ ;
         } /* while  */

      } /* if we have categories to look through  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to read this information from the database,   */
      /* do so now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null )
         addCategory ( returnRec = readCategoryFromDatabase (  description, categoryLevel ) ) ;

      return( returnRec ) ;
   }


   /**
     *  This method will read, recurse, all parents of the TypeCategoryObject
     * passed in. Each parent will be returned as a TypeCategoryRec object.
     *
     * @param typeCategoryRec The TypeCategoryRec to look up parents for
     *
     * @return Vector of TypeCategoryRec objects
     **/
   public static Vector getCategoryParents ( TypeCategoryRec categoryRec ) {
      Vector          returnVec = new Vector ( 1 ) ;
      if ( categoryRec != null ) {
         boolean         process   = true ;
         TypeCategoryRec parentRec = categoryRec ;

         while ( process ) {
            if ( parentRec != null ) {
               if ( parentRec.getParentCatInd() == 0 )
                  process = false ;
               else
                  returnVec.addElement ( parentRec = getCategory ( parentRec.getParentCatInd() ) ) ;
            } else
               process = false ;
         }
      }

      return( returnVec ) ;
   }


   /**
     *  This method will return the specific parent of the TypeCategory passed in
     * with the specific category level
     **/
   public static TypeCategoryRec getParentCategory ( TypeCategoryRec categoryRec, String categoryLevel ) {
      Vector          parents   = getCategoryParents ( categoryRec ) ;
      TypeCategoryRec returnRec = null ;

      if ( ( parents != null ) && ( parents.size() > 0 ) ) {
         int i    = 0 ;
         int size = parents.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ((TypeCategoryRec)parents.elementAt(i)).getCategoryLevel().equals ( categoryLevel ) )
               returnRec = (TypeCategoryRec) parents.elementAt(i) ;
            i++ ;
         }
      }
      return( returnRec ) ;
   }


   /**
     *  This method will read, recurse, all parents of the typeCatInd
     * passed in. Each parent will be returned as a TypeCategoryRec object.
     *
     * @param typeCatInd The typeCatInd to look up parents for
     *
     * @return Vector of TypeCategoryRec objects
     **/
   public static Vector getCategoryParents ( int typeCatInd ) {
      return( getCategoryParents ( getCategory ( typeCatInd ) ) ) ;
   }




   /**
     * This method will attempt to read a single category from the database. 
     *
     * @param categoryDescript The description of the category to read in
     * @param level The category level of the category to read in
     *
     * @return The TypeCategoryRec read in
     */
   private static TypeCategoryRec readDraftCategoryFromDatabase ( String categoryDescript, String level ) {
      SQLMethod        sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeCategory.readDraftCategoryFromDatabase ( "+categoryDescript+", "+level+" )", 5 ) ;
      TypeCategoryRec  returnRec = null ;

      fireReadStart ( getDescriptFromCategoryLevel ( level ) ) ;
      try {
         Statement stmt = sqlMethod.createStatement() ;
         if ( stmt != null ) {
            ResultSet rs   = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                 "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                 "       Owner "+
                                                 "from proDraft.TypeCategory "+
                                                 "where Description   = '"+categoryDescript+"' and "+
                                                 "      categoryLevel = '"+level+"' and "+
                                                 "      valid         = 'Y' for Fetch only" ) ;

            if ( rs.next() ) {
               returnRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec.setDescript      ( rs.getString(2).trim() ) ;
               returnRec.setParentCatInd  ( rs.getInt(3) ) ;
               returnRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               returnRec.setOrderNum      ( rs.getInt(5) ) ;
               returnRec.setChildSort     ( rs.getString(6).trim() ) ;
               returnRec.setOwner         ( rs.getString(7) ) ;
               returnRec.setIsDraft       ( true ) ;
               returnRec.updateRecStatus ( returnRec.REC_NOCHANGE ) ;
               addCategory ( returnRec ) ;

            }

            rs.close() ;
         } else {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, "TypeCategoryRec.readCategoryFromDatabase could not establish a Statement object" ) ;
         }

      } catch ( Exception e ) {
         returnRec = null ;
         sqlMethod.rollBack () ;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            e.printStackTrace() ;
      }

      sqlMethod.close() ;
      fireReadStop() ;

      return( returnRec ) ;
   }

   /**
     * This method will return a single category based on the
     * category indicator passed in. If the category
     * is not found in memory then this method will try to read
     * the information from the database.
     *
     * @param typeCatInd The Category Indicator to read
     */
   public static TypeCategoryRec getDraftCategory ( int typeCatInd ) {
      TypeCategoryRec returnRec        = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First try to read the information from cached memory    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         int         i    = 0 ;
         int         size = draftCategories.size() ;

         while ( ( returnRec == null ) && ( i < size ) ) {
            if ( ( ((TypeCategoryRec)draftCategories.elementAt(i)).getInd() == typeCatInd ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getIsDraft() ) )
               returnRec = (TypeCategoryRec) draftCategories.elementAt(i) ;
            i++ ;
         } /* while  */

      } /* if we have draftCategories to look through  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to read this information from the database,   */
      /* do so now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null )
         addCategory ( returnRec = readDraftCategoryFromDatabase (  typeCatInd ) ) ;

      return( returnRec ) ;
   }
   /**
     * This method will return a single category based on the
     * category indicator and level passed in. If the category
     * is not found in memory then this method will try to read
     * the information from the database.
     *
     * @param typeCatInd The Category Indicator to read
     * @param level The Category level to read
     */
   public static TypeCategoryRec getDraftCategory ( int typeCatInd, String level ) {
      TypeCategoryRec returnRec        = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First try to read the information from cached memory    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         int         i    = 0 ;
         int         size = draftCategories.size() ;

         while ( ( returnRec == null ) && ( i < size ) ) {
            if ( ( ((TypeCategoryRec)draftCategories.elementAt(i)).getInd() == typeCatInd ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getCategoryLevel().equals ( level ) ) )
               returnRec = (TypeCategoryRec) draftCategories.elementAt(i) ;
            i++ ;
         } /* while  */

      } /* if we have draftCategories to look through  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to read this information from the database,   */
      /* do so now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null )
         addCategory ( returnRec = readDraftCategoryFromDatabase (  typeCatInd, level ) ) ;

      return( returnRec ) ;
   }


   /**
     * This method will return a single category based on the
     * category description and level passed in. If the category
     * is not found in memory then this method will try to read
     * the information from the database.
     *
     * @param description The Category description to read
     * @param categoryLevel The Category categoryLevel to read
     */
   public static TypeCategoryRec getDraftCategory ( String description, String categoryLevel ) {
      TypeCategoryRec returnRec        = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First try to read the information from cached memory    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         int i    = 0 ;
         int size = draftCategories.size() ;

         while ( ( returnRec == null ) && ( i < size ) ) {
            if ( ( ((TypeCategoryRec)draftCategories.elementAt(i)).getDescript().equals ( description ) ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).valid() ) &&
                 ( ((TypeCategoryRec)draftCategories.elementAt(i)).getCategoryLevel().equals ( categoryLevel ) ) )
               returnRec = (TypeCategoryRec) draftCategories.elementAt(i) ;
            i++ ;
         } /* while  */

      } /* if we have draftCategories to look through  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to read this information from the database,   */
      /* do so now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null )
         addCategory ( returnRec = readDraftCategoryFromDatabase (  description, categoryLevel ) ) ;

      return( returnRec ) ;
   }


   /**
     * This method will attempt to read a single category from the database. 
     *
     * @param typeCatInd The category indicator of the category to read in
     *
     * @return The TypeCategoryRec read in
     */
   private static TypeCategoryRec readDraftCategoryFromDatabase ( int typeCatInd ) {
      SQLMethod        sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeCategory.readDraftCategoryFromDatabase ( "+typeCatInd+" )", 5 ) ;
      TypeCategoryRec  returnRec = null ;

      fireReadStart ( ""+typeCatInd ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                              "       CategoryLevel, OrderNum,    ChildSort,    "+
                                              "       Owner "+
                                              "from proDraft.TypeCategory "+
                                              "where typeCatInd = "+typeCatInd+" and "+
                                              "      valid = 'Y' for Fetch only" ) ;

         if ( rs.next() ) {
            returnRec = new TypeCategoryRec ( rs.getInt(1) ) ;
            returnRec.setDescript      ( rs.getString(2).trim() ) ;
            returnRec.setParentCatInd  ( rs.getInt(3) ) ;
            returnRec.setCategoryLevel ( rs.getString(4).trim() ) ;
            returnRec.setOrderNum      ( rs.getInt(5) ) ;
            returnRec.setChildSort     ( rs.getString(6).trim() ) ;
            returnRec.setOwner         ( rs.getString(7) ) ;
            returnRec.setIsDraft       ( true ) ;
            returnRec.updateRecStatus  ( returnRec.REC_NOCHANGE ) ;
            addCategory ( returnRec ) ;
         }

         rs.close() ;

      } catch ( Exception e ) {
         returnRec = null ;
         sqlMethod.rollBack () ;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            e.printStackTrace() ;
      }

      sqlMethod.close() ;
      fireReadStop() ;

      return( returnRec ) ;
   }



   /**
     * This method will attempt to read all categories under the parent/level from the database. 
     *
     * @param typeCatInd The category indicator of the category to read in
     * @param level The category level of the category to read in
     *
     * @return The TypeCategoryRec read in
     */
   private static Vector readDraftCategoriesFromDatabase ( int parentCatInd, String level ) {
      SQLMethod        sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeCategory.readDraftCategoriesFromDatabase ( "+parentCatInd+", "+level+" )", 5 ) ;
      Vector           returnVec = new Vector ( 1 ) ;

      fireReadStart ( getDescriptFromCategoryLevel ( level ) ) ;
      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                              "       CategoryLevel, OrderNum,    ChildSort,    "+
                                              "       Owner "+
                                              "from proDraft.TypeCategory "+
                                              "where parentCatInd  = "+parentCatInd+" and "+
                                              "      categoryLevel = '"+level+"' and "+
                                              "      valid = 'Y' for Fetch only" ) ;

         while ( rs.next() ) {
            TypeCategoryRec returnRec = new TypeCategoryRec ( rs.getInt(1) ) ;
            returnRec.setDescript      ( rs.getString(2).trim() ) ;
            returnRec.setParentCatInd  ( rs.getInt(3) ) ;
            returnRec.setCategoryLevel ( rs.getString(4).trim() ) ;
            returnRec.setOrderNum      ( rs.getInt(5) ) ;
            returnRec.setChildSort     ( rs.getString(6).trim() ) ;
            returnRec.setOwner         ( rs.getString(7) ) ;
            returnRec.updateRecStatus ( returnRec.REC_NOCHANGE ) ;
            returnVec.addElement ( returnRec ) ;
            returnRec.setIsDraft       ( true ) ;
            addCategory ( returnRec ) ;
         }

         rs.close() ;

      } catch ( Exception e ) {
         returnVec = null ;
         sqlMethod.rollBack () ;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            e.printStackTrace() ;
      }

      sqlMethod.close() ;
      fireReadStop() ;

      return( returnVec ) ;
   }



   /**
     * This method will attempt to read a single category from the database. 
     *
     * @param typeCatInd The category indicator of the category to read in
     * @param level The category level of the category to read in
     *
     * @return The TypeCategoryRec read in
     */
   private static TypeCategoryRec readDraftCategoryFromDatabase ( int typeCatInd, String level ) {
      SQLMethod        sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeCategory.readDraftCategoryFromDatabase ( "+typeCatInd+", "+level+" )", 5 ) ;
      TypeCategoryRec  returnRec = null ;

      fireReadStart ( getDescriptFromCategoryLevel ( level ) ) ;
      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                              "       CategoryLevel, OrderNum,    ChildSort,    "+
                                              "       Owner "+
                                              "from proDraft.TypeCategory "+
                                              "where typeCatInd = "+typeCatInd+" and "+
                                              "      categoryLevel = '"+level+"' and "+
                                              "      valid = 'Y' for Fetch only" ) ;

         if ( rs.next() ) {
            returnRec = new TypeCategoryRec ( rs.getInt(1) ) ;
            returnRec.setDescript      ( rs.getString(2).trim() ) ;
            returnRec.setParentCatInd  ( rs.getInt(3) ) ;
            returnRec.setCategoryLevel ( rs.getString(4).trim() ) ;
            returnRec.setOrderNum      ( rs.getInt(5) ) ;
            returnRec.setChildSort     ( rs.getString(6).trim() ) ;
            returnRec.setOwner         ( rs.getString(7) ) ;
            returnRec.setIsDraft       ( true ) ;
            returnRec.updateRecStatus ( returnRec.REC_NOCHANGE ) ;
            addCategory ( returnRec ) ;

         }

         rs.close() ;

      } catch ( Exception e ) {
         returnRec = null ;
         sqlMethod.rollBack () ;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            e.printStackTrace() ;
      }

      sqlMethod.close() ;
      fireReadStop() ;

      return( returnRec ) ;
   }


   /**
     *  This method will add a TypeCategory listener 
     *
     * @param listener The TypeCategory listener to add
     **/
   public static void addTypeCategoryListener ( TypeCategoryListener listener ) {
      listeners.addElement ( listener ) ;
   }

   /**
     *  This method will remove a TypeCategory listener
     *
     * @param listener The TypeCategory listener to remove
     **/
   public static void removeTypeCategoryListener ( TypeCategoryListener listener ) {
      listeners.removeElement ( listener ) ;
   }



   /**
     *  This method will call the 'typeCategoryReadStart' method
     *
     * @param categoryName The name of the category that is being read
     **/
   private static void fireReadStart ( String categoryName ) {
      int i    = 0 ;
      int size = listeners.size() ;

      while ( i <size ) {
         ((TypeCategoryListener)listeners.elementAt(i)).typeCategoryReadStart ( categoryName ) ;
         i++ ;
      }
   }


   /**
     *  This method will call the 'typeCategoryReadStop' method
     *
     **/
   private static void fireReadStop ( ) {
      int i    = 0 ;
      int size = listeners.size() ;

      while ( i <size ) {
         ((TypeCategoryListener)listeners.elementAt(i)).typeCategoryReadStop () ;
         i++ ;
      }
   }


   /**
     *  This method will call the 'typeCategoryWriteStart' method
     *
     **/
   private static void fireWriteStart ( ) {
      int i    = 0 ;
      int size = listeners.size() ;

      while ( i <size ) {
         ((TypeCategoryListener)listeners.elementAt(i)).typeCategoryWriteStart ( ) ;
         i++ ;
      }
   }


   /**
     *  This method will call the 'typeCategoryWriteStop' method
     *
     **/
   private static void fireWriteStop ( ) {
      int i    = 0 ;
      int size = listeners.size() ;

      while ( i <size ) {
         ((TypeCategoryListener)listeners.elementAt(i)).typeCategoryWriteStop () ;
         i++ ;
      }
   }


   /**
     *  This method will simply return a hard coded human readable description for
     * a category level
     *
     * @param level The Category Level
     *
     * @return String
     **/
   public static String getDescriptFromCategoryLevel ( String level ) {

      if ( level.equals ( LEVEL_BRAND ) )
         return( "Brand" ) ;
      else if ( level.equals ( LEVEL_FAMILY   ) )
         return( "Family" ) ;
      else if ( level.equals ( LEVEL_MACHINE  ) )
         return( "Machine" ) ;
      else if ( level.equals ( LEVEL_MODEL ) )
         return( "Model" ) ;
      else if ( level.equals ( LEVEL_PARTNUMBER ) )
         return( "Part Number" ) ;
      else if ( level.equals ( LEVEL_OPICM_BRAND ) )
         return( "OPIC/M Brand" ) ;
      else if ( level.equals ( LEVEL_OPICM_FAMILY ) )
         return( "OPIC/M Family" ) ;
      else if ( level.equals ( LEVEL_OPICM_SERIES ) )
         return( "OPIC/M Series" ) ;
      else if ( level.equals ( LEVEL_OPICM_COUNTRY ) )
         return( "OPIC/M Country" ) ;
      else if ( level.equals ( LEVEL_OPICM_GEOGRAPHY ) )
         return( "OPIC/M Geography" ) ;
      else if ( level.equals ( LEVEL_DOC_CATEGORY ) )
         return( "Document Category" ) ;
      else if ( level.equals ( LEVEL_PROJECT ) )
         return( "Project" ) ;
      else if ( level.equals ( LEVEL_SUBCATEGORY ) )
         return( "Sub Category" ) ;

      return( "Unknown" ) ;
   }



   /**
     * This method will add the category passed to the categories cache vector. Currently
     * the only Categories that are cached in memory MUST be one of the following Category Levels:<br><br>
     *
     * LEVEL_FAMILY<br>
     * LEVEL_BRAND<br>
     * LEVEL_OPICM_BRAND<br>
     * LEVEL_OPICM_FAMILY<br>
     * LEVEL_OPICM_COUNTRY<br>
     * LEVEL_OPICM_GEOGRAPHY<br>
     * LEVEL_OPICM_SERIES<br>
     * LEVEL_DOC_CATEGORY<br>
     * LEVEL_PROJECT<br>
     *
     * @param category The TypeCategoryRec to add
     **/
   private static void addCategory ( TypeCategoryRec categoryRec ) {
      try {
         if ( categoryRec != null ) {
            boolean dup = true ;

            if ( categories == null )
               categories = new Vector ( 1 ) ;

            if ( draftCategories == null )
               draftCategories = new Vector ( 1 ) ;

            if ( categoryRec.getIsDraft() )
               dup = draftCategories.contains ( categoryRec ) ;
            else
               dup = categories.contains ( categoryRec ) ;

            if ( !dup ) {
               if ( ( categoryRec.getCategoryLevel().equals ( LEVEL_FAMILY ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_BRAND ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_OPICM_BRAND ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_OPICM_FAMILY ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_OPICM_COUNTRY ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_OPICM_GEOGRAPHY ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_OPICM_SERIES ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_PROJECT ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_SUBCATEGORY ) ) ||
                    ( categoryRec.getCategoryLevel().equals ( LEVEL_DOC_CATEGORY ) ) ) {
                  if ( categoryRec.getIsDraft() ) {
                     draftCategories.addElement ( categoryRec ) ;
                  } else
                     categories.addElement ( categoryRec ) ;
               }
            }
         }
      } catch ( NullPointerException e ) {
         e.printStackTrace() ;
      }
   } /* addCategory  */




   /**
     * This method will return the draft version of an active category. If the 
     * draft category does not exist then a new one will be created and returned
     *
     * @param activeCategory
     *
     * @return Draft TypeCategoryRec
     **/
   public static TypeCategoryRec getDraftCategoryFromCategory ( TypeCategoryRec activeCategory ) {
      TypeCategoryRec draftCategory = TypeCategory.getDraftCategoryFromDescription ( activeCategory.toString(), 
                                                                                     activeCategory.getCategoryLevel(), 
                                                                                     activeCategory.getParentCatInd() ) ;

      if ( draftCategory == null ) {
         draftCategory = createDraftTypeCategory ( activeCategory.getParentCatInd() ,
                                                   activeCategory.toString(),
                                                   activeCategory.getCategoryLevel(),
                                                   activeCategory.getOrderNum(),
                                                   activeCategory.getChildSort(),
                                                   activeCategory.getOwner() ) ;
         addCategory ( draftCategory ) ;
         writeToDatabase() ;
      }

      return( draftCategory ) ;
   }



   /**
     *  This method will read, recurse, all parents of the TypeCategoryObject
     * passed in. Each parent will be returned as a TypeCategoryRec object.
     *
     * @param typeCategoryRec The TypeCategoryRec to look up parents for
     *
     * @return Vector of TypeCategoryRec objects
     **/
   public static Vector getDraftCategoryParents ( TypeCategoryRec categoryRec ) {
      Vector          returnVec = new Vector ( 1 ) ;
      if ( categoryRec != null ) {
         boolean         process   = true ;
         TypeCategoryRec parentRec = categoryRec ;

         while ( process ) {
            if ( parentRec != null ) {
               if ( parentRec.getParentCatInd() == 0 )
                  process = false ;
               else
                  returnVec.addElement ( parentRec = getDraftCategory ( parentRec.getParentCatInd() ) ) ;
            } else
               process = false ;
         }
      }

      return( returnVec ) ;
   }

   /**
     *  This method will read, recurse, all parents of the typeCatInd
     * passed in. Each parent will be returned as a TypeCategoryRec object.
     *
     * @param typeCatInd The typeCatInd to look up parents for
     *
     * @return Vector of TypeCategoryRec objects
     **/
   public static Vector getDraftCategoryParents ( int typeCatInd ) {
      return( getDraftCategoryParents ( getCategory ( typeCatInd ) ) ) ;
   }



   /**
     *  This method will return a TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database. This method will make sure that the TypeCategoryRec pased
     * matches the description/level anywhere under the parent cat indicator passed
     * 
     * @param description The description to look for
     * @param level The category level to look for
     * @param parentCatInd The parent indicator to look under
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getCategoryFromDescriptionUnderParent ( String description, String level, int parentCatInd ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int i    = 0 ;
         int size = categories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).toString().equals ( description ) ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getCategoryLevel().equals ( level ) ) &&
                 ( !((TypeCategoryRec)categories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) ) {
               //( ((TypeCategoryRec)categories.elementAt(i)).getParentCatInd() == parentCatInd ) )
               Vector parents = getCategoryParents ( ((TypeCategoryRec)categories.elementAt(i)).getInd() ) ;

               if ( parents != null ) {
                  int parentIndex    = 0 ;
                  int parentSize     = parents.size() ;

                  while ( ( returnRec == null ) && ( parentIndex < parentSize ) ) {
                     if ( ((TypeCategoryRec)parents.elementAt(parentIndex)).getInd() == parentCatInd )
                        returnRec = (TypeCategoryRec)categories.elementAt(i) ;
                     parentIndex++ ;
                  }
               }
            } /* if the Description/Level Matched  */
            i++ ;
         }
      } /* if categories  */



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getCategoryFromDescriptionUndeoParent ( "+description+", "+level+", "+parentCatInd+" )",
                                               5 ) ;

         fireReadStart( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select a.TypeCatInd,    a.Description, a.ParentCatInd, "+
                                                          "       a.CategoryLevel, a.OrderNum,    a.ChildSort,    "+
                                                          "       a.Owner "+
                                                          "from product.TypeCategory A  "+
                                                          "where a.description   = '"+description+"' and "+
                                                          "      a.categoryLevel = '"+level+"' and "+
                                                          "      a.parentCatInd  in (select parentcatind from product.typecatchild where typecatind = a.typecatind) and "+
                                                          "      a.valid         = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }


   /**
     *  This method will return a TypeCategory object from the description
     * passed in. This method will first look in the objects that have been
     * cached in memory. If an object is not found, a search will be performed
     * in the database. This method will make sure that the TypeCategoryRec pased
     * matches the description/level anywhere under the parent cat indicator passed
     * 
     * @param description The description to look for
     * @param level The category level to look for
     * @param parentCatInd The parent indicator to look under
     *
     * @returns The TypeCategoryRec matching the description passed or NULL if not found
     **/
   public static TypeCategoryRec getDraftCategoryFromDescriptionUnderParent ( String description, String level, int parentCatInd ) {
      TypeCategoryRec returnRec = null ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, look through memory                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draftCategories != null ) && ( draftCategories.size() > 0 ) ) {
         int i    = 0 ;
         int size = draftCategories.size() ;

         while ( ( i < size ) && ( returnRec == null ) ) {
            if ( ( ((TypeCategoryRec)categories.elementAt(i)).toString().equals ( description ) ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getCategoryLevel().equals ( level ) ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).getIsDraft() ) &&
                 ( ((TypeCategoryRec)categories.elementAt(i)).valid() ) ) {
               //( ((TypeCategoryRec)categories.elementAt(i)).getParentCatInd() == parentCatInd ) )
               Vector parents = getDraftCategoryParents ( ((TypeCategoryRec)categories.elementAt(i)).getInd() ) ;

               if ( parents != null ) {
                  int parentIndex    = 0 ;
                  int parentSize     = parents.size() ;

                  while ( ( returnRec == null ) && ( parentIndex < parentSize ) ) {
                     if ( ((TypeCategoryRec)parents.elementAt(parentIndex)).getInd() == parentCatInd )
                        returnRec = (TypeCategoryRec)categories.elementAt(i) ;
                     parentIndex++ ;
                  }
               }
            } /* if the Description/Level Matched  */
            i++ ;
         }
      } /* if categories  */



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we didn't find it, look through the database         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( returnRec == null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST,
                                               "TypeCategory.getDraftCategoryFromDescriptionUndeoParent ( "+description+", "+level+", "+parentCatInd+" )",
                                               5 ) ;

         fireReadStart( "" ) ;
         try {
            TypeCategoryRec readRec = null ;
            Statement       stmt    = sqlMethod.createStatement() ;
            ResultSet       rs      = stmt.executeQuery ( "select a.TypeCatInd,    a.Description, a.ParentCatInd, "+
                                                          "       a.CategoryLevel, a.OrderNum,    a.ChildSort,    "+
                                                          "       a.Owner "+
                                                          "from prodraft.TypeCategory A  "+
                                                          "where a.description   = '"+description+"' and "+
                                                          "      a.categoryLevel = '"+level+"' and "+
                                                          "      a.parentCatInd  in (select parentcatind from prodraft.typecatchild where typecatind = a.typecatind) and "+
                                                          "      a.valid         = 'Y' for Fetch only" ) ;

            while ( rs.next() ) {
               readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
               returnRec = readRec ;
               readRec.setDescript      ( rs.getString(2).trim() ) ;
               readRec.setParentCatInd  ( rs.getInt(3) ) ;
               readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
               readRec.setOrderNum      ( rs.getInt(5) ) ;
               readRec.setChildSort     ( rs.getString(6).trim() ) ;
               readRec.setOwner         ( rs.getString(7) ) ;
               readRec.setIsDraft       ( true ) ;
               readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
               addCategory ( readRec ) ;
            }

            rs.close() ;
         } catch ( Exception e ) {
            if ( LogSystem.getInstance() != null )
               LogSystem.log ( 1, e, false ) ;
            else
               e.printStackTrace() ;
            sqlMethod.rollBack() ;
         }

         sqlMethod.close() ;
         fireReadStop() ;
      }



      return( returnRec ) ;
   }








////  /**
////    *  This method will read all categories under the categoryLevel passed in.
////    *
////    * @param categoryLevel The Category Level to read with
////    **/
////  public static void getCategories ( String categoryLevel ) {
////     SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getCategories ( "+categoryLevel+")", 5 ) ;

////     try {
////        TypeCategoryRec readRec = null ;
////        Statement       stmt    = conn.createStatement() ;
////        ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
////                                                      "       CategoryLevel, OrderNum,    ChildSort,    "+
////                                                      "       Owner "+
////                                                      "from product.TypeCategory "+
////                                                      "where CategoryLevel = '"+categoryLevel+"' and "+
////                                                      "      valid         = 'Y' for Fetch Only" ) ;

////        while ( rs.next() ) {
////           readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
////           readRec.setDescript      ( rs.getString(2).trim() ) ;
////           readRec.setParentCatInd  ( rs.getInt(3) ) ;
////           readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
////           readRec.setOrderNum      ( rs.getInt(5) ) ;
////           readRec.setChildSort     ( rs.getString(6).trim() ) ;
////           readRec.setOwner         ( rs.getString(7) ) ;
////           readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
////           returnVec.addElement ( readRec ) ;
////        }

////        rs.close() ;
////     } catch ( Exception e ) {
////        LogSystem.log ( 1, e , false ) ;
////     }

////     sqlMethod.close() ;

////  } /* readCategories  */



////  /**
////    *  This method will read all categories under the parentCatInd passed in.
////    *
////    * @param parentCatInt The Parent Category Indicator to read from
////    **/
////  private void getCategories ( int parentCatInd ) throws Exception {
////     SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getCategories ("+parentCatInd+")", 5 ) ;

////     try {
////        TypeCategoryRec readRec = null ;
////        Statement       stmt    = conn.createStatement() ;
////        ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
////                                                      "       CategoryLevel, OrderNum,    ChildSort,    "+
////                                                      "       Owner "+
////                                                      "from product.TypeCategory "+
////                                                      "where parentCatInd = "+parentCatInd+" and "+
////                                                      "      valid         = 'Y' for Fetch Only" ) ;

////        while ( rs.next() ) {
////           readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
////           readRec.setDescript      ( rs.getString(2).trim() ) ;
////           readRec.setParentCatInd  ( rs.getInt(3) ) ;
////           readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
////           readRec.setOrderNum      ( rs.getInt(5) ) ;
////           readRec.setChildSort     ( rs.getString(6).trim() ) ;
////           readRec.setOwner         ( rs.getString(7) ) ;
////           readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
////           returnVec.addElement ( readRec ) ;
////        }

////        rs.close() ;
////     } catch ( Exception e ) {
////        LogSystem.log ( 1, e ,false ) ;
////     }
////     sqlMethod.close() ;

////  } /* readCategories  */

////  /**
////    *  This method will read all categories under the parentCatInd and
////    * category Level passed in.
////    *
////    * @param categoryLevel The Category Level to read from
////    * @param parentCatInt The Parent Category Indicator to read from
////    **/
////  private void getCategories ( String categoryLevel, int parentCatInd ) throws Exception {
////     SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getCategories ("+categoryLevel+", "+parentCatInd+")", 5 ) ;

////     try {
////        TypeCategoryRec readRec = null ;
////        Statement       stmt    = conn.createStatement() ;
////        ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
////                                                      "       CategoryLevel, OrderNum,    ChildSort,    "+
////                                                      "       Owner "+
////                                                      "from product.TypeCategory "+
////                                                      "where parentCatInd = "+parentCatInd+" and "+
////                                                      "      valid         = 'Y' and "+
////                                                      "      categoryLevel = '"+categoryLevel+"' for Fetch only" ) ;

////        while ( rs.next() ) {
////           readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
////           readRec.setDescript      ( rs.getString(2).trim() ) ;
////           readRec.setParentCatInd  ( rs.getInt(3) ) ;
////           readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
////           readRec.setOrderNum      ( rs.getInt(5) ) ;
////           readRec.setChildSort     ( rs.getString(6).trim() ) ;
////           readRec.setOwner         ( rs.getString(7) ) ;
////           readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
////           returnVec.addElement ( readRec ) ;
////        }

////        rs.close() ;
////     } catch ( Exception e ) {
////        LogSystem.log ( 1, e , false ) ;
////     }

////     sqlMethod.close() ;

////  } /* readCategories  */

} /* TypeCategory */

