package com.ibm.nzna.projects.common.quest.type ;                 
/*-----------------------------------------------------------*/
/*   TypeCategoryReader                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeCategoryReader                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.*;
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Blob ;
/**
 * This DB/2 Stored Procedure will read in rows from the PRODUCT.TYPECATEGORY
 * table. It does so by specifying Category Level, Parent Indicator, or
 * either or. If you wish to only search by category level then pass 
 * a parentCatInd of -1. If you wish to read by parentCatInd only, then pass
 * an empty string as the categoryLevel.<br><br><br>
 *
 * Parameters:<br><pre>
 * Type   Name                   Description
 * ------------------------------------------------------------------------
 * In     categoryLevel          The categoryLevel to include in the search,
 *                               if an Empty String is passed, the search
 *                               will be performed without the categoryLevel.
 * In     parentCatInd           The parentCatInd to include in the search.
 *                               if this is equal to -1 then the parentCatInd
 *                               will not be used in the search.
 * Out    vector                 This vector will contain TypeCategoryRec objects
 *                               that are a result from the search.
 * Out    StoredProcReturnRec    The standard Stored Procedure Return Record
 *                               Holding all of the return error codes</pre>
 *
 *
 * @version 1.0, 5/11/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeCategoryReader                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeCategoryReader extends StoredProc {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This is the record which holds error return codes and all
     **/
   private StoredProcRec returnRec       = new StoredProcRec () ;

   /**
     *  This is the vector to store data in before it is sent back
     **/
   private Vector returnVec = new Vector ( 10 ) ;

   /**
     *  This is the database conncetion
     **/
   private Connection conn = null ;




   /**
    *    This method will be called by DB/2 when
    *  a request for this stored procedure is given.
    *  
    *
    *  @param categoryLevel The Category Level to read from, empty string if none
    *  @param parentCatInd The parent category indicator to read from. -1 if none
    *  @param categoryBlob This does not need to be used
    *  @param storedProcReturn This does not need to be used
    *
    *//*---------------------------------------------------------*/
   public void readCategories ( String categoryLevel, int parentCatInd, Blob categoryBlob, Blob storedProcReturn ) throws Exception {
      conn = getConnection() ;

      try {    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read the Data                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( categoryLevel != null ) && ( categoryLevel.length() > 0 ) && ( parentCatInd == -1 ) )
            getCategories ( categoryLevel ) ;
         else if ( ( categoryLevel != null ) && ( categoryLevel.length() == 0 ) && ( parentCatInd >= 0 ) )
            getCategories ( parentCatInd ) ;
         else if ( ( categoryLevel != null ) && ( categoryLevel.length() > 0 ) && ( parentCatInd >= 0 ) )
            getCategories ( categoryLevel, parentCatInd ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our return vector                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         categoryBlob = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( categoryBlob, returnVec ) ;
         set ( 3, categoryBlob );

      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      conn.commit() ;
      conn.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcReturn = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcReturn, returnRec ) ;

      set ( 4, storedProcReturn ) ;
   } /* readCategory  */


   /**
     *  This method will read all categories under the categoryLevel passed in.
     *
     * @param categoryLevel The Category Level to read with
     **/
   public static void getCategories ( String categoryLevel ) throws Exception {
      TypeCategoryRec readRec = null ;
      Statement       stmt    = conn.createStatement() ;
      ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                    "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                    "       Owner "+
                                                    "from product.TypeCategory "+
                                                    "where CategoryLevel = '"+categoryLevel+"' and "+
                                                    "      valid         = 'Y' for Fetch Only" ) ;

      while ( rs.next() ) {
         readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
         readRec.setDescript      ( rs.getString(2).trim() ) ;
         readRec.setParentCatInd  ( rs.getInt(3) ) ;
         readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
         readRec.setOrderNum      ( rs.getInt(5) ) ;
         readRec.setChildSort     ( rs.getString(6).trim() ) ;
         readRec.setOwner         ( rs.getString(7) ) ;
         readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
         returnVec.addElement ( readRec ) ;
      }

      rs.close() ;
      stmt.close() ;

   } /* readCategories  */



   /**
     *  This method will read all categories under the parentCatInd passed in.
     *
     * @param parentCatInt The Parent Category Indicator to read from
     **/
   private void getCategories ( int parentCatInd ) throws Exception {
      TypeCategoryRec readRec = null ;
      Statement       stmt    = conn.createStatement() ;
      ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                    "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                    "       Owner "+
                                                    "from product.TypeCategory "+
                                                    "where parentCatInd = "+parentCatInd+" and "+
                                                    "      valid         = 'Y' for Fetch Only" ) ;

      while ( rs.next() ) {
         readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
         readRec.setDescript      ( rs.getString(2).trim() ) ;
         readRec.setParentCatInd  ( rs.getInt(3) ) ;
         readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
         readRec.setOrderNum      ( rs.getInt(5) ) ;
         readRec.setChildSort     ( rs.getString(6).trim() ) ;
         readRec.setOwner         ( rs.getString(7) ) ;
         readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
         returnVec.addElement ( readRec ) ;
      }

      rs.close() ;
      stmt.close() ;

   } /* readCategories  */

   /**
     *  This method will read all categories under the parentCatInd and
     * category Level passed in.
     *
     * @param categoryLevel The Category Level to read from
     * @param parentCatInt The Parent Category Indicator to read from
     **/
   private void getCategories ( String categoryLevel, int parentCatInd ) throws Exception {
      TypeCategoryRec readRec = null ;
      Statement       stmt    = conn.createStatement() ;
      ResultSet       rs      = stmt.executeQuery ( "select TypeCatInd,    Description, ParentCatInd, "+
                                                    "       CategoryLevel, OrderNum,    ChildSort,    "+
                                                    "       Owner "+
                                                    "from product.TypeCategory "+
                                                    "where parentCatInd = "+parentCatInd+" and "+
                                                    "      valid         = 'Y' and "+
                                                    "      categoryLevel = '"+categoryLevel+"' for Fetch only" ) ;

      while ( rs.next() ) {
         readRec = new TypeCategoryRec ( rs.getInt(1) ) ;
         readRec.setDescript      ( rs.getString(2).trim() ) ;
         readRec.setParentCatInd  ( rs.getInt(3) ) ;
         readRec.setCategoryLevel ( rs.getString(4).trim() ) ;
         readRec.setOrderNum      ( rs.getInt(5) ) ;
         readRec.setChildSort     ( rs.getString(6).trim() ) ;
         readRec.setOwner         ( rs.getString(7) ) ;
         readRec.updateRecStatus ( readRec.REC_NOCHANGE ) ;
         returnVec.addElement ( readRec ) ;
      }

      rs.close() ;
      stmt.close() ;

   } /* readCategories  */


} /* TypeCategoryReader */
