package com.ibm.nzna.projects.common.quest.type ;
/*-----------------------------------------------------------*/
/*   TypeCountryCodeRec                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeCountryCodeRec                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * Holds information on a specific country. It extends TypeRec so that 
 * we can treat it as a Type Record
 *
 *
 * @version 1.0, 2/26/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeCountryCodeRec                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeCountryCodeRec extends TypeRec implements MultiListRow {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public short   phoneCountryCode = 0 ;
   public String  countryCode      = "" ;   // The 3 character country code for each country
   public String  zipMask          = "" ;
   public String  phoneMask        = "" ;
   public String  timeDiff         = "" ;
   public String  currency         = "" ;
   public String  currencyMask     = "" ;

   public short   zipDigits       = 0 ;
   public short   lang01          = 0 ;
   public short   lang02          = 0 ;
   public short   lang03          = 0 ;
   public short   geoInd          = 0 ;
   public boolean multiZone       = false ;
   public boolean usesDST         = false ;
   public String  isoCode         = null ;

   public TypeCountryCodeRec () {
      updateRecStatus ( REC_INSERT ) ;
   }
   public TypeCountryCodeRec ( int ind, String descript ) {
      super ( ind, descript ) ;
   }

   public TypeCountryCodeRec ( int ind, String phoneCountryCode, String descript, String zipMask, String phoneMask, short zipDigits,
                               String timeDiff, String multiZone, String usesDST, String currency, String currencyMask,short lang01, 
                               short lang02, short lang03, short geoInd, String isoCode ) {
      super ( ind, descript ) ;
      String tmpMultiZone = "T" ;
      String tmpUsesDST   = "T" ;

      if ( timeDiff.equals ( "00" ) )
         timeDiff = "00:00:00" ;
      else
         timeDiff = timeDiff.trim() ;

      if ( ( currency == null ) || ( currency.length() == 0 ) || ( currency.equals ( "NULL" ) ) )
         currency = "US Dollars" ;
      else
         currency = currency.trim() ;

      if ( ( currencyMask == null ) || ( currencyMask.length() == 0 ) || ( currencyMask.equals ( "NULL" ) ) )
         currencyMask = "99.99" ;
      else
         currencyMask = currencyMask.trim() ;

      if ( ( countryCode == null ) || ( countryCode.length() == 0 ) )
         countryCode = "001" ;

      if ( ( phoneMask == null ) || ( phoneMask.length() == 0 ) )
         phoneMask = "(999)-999-9999" ;
      else
         phoneMask = phoneMask.trim() ;

      if ( ( zipMask == null ) || ( zipMask.length() == 0 ) )
         zipMask = "99999" ;
      else
         zipMask = zipMask.trim() ;

      try {
         this.phoneCountryCode = new Integer ( phoneCountryCode ).shortValue() ;
      } catch ( Exception e ) {
         this.phoneCountryCode = 0 ;
      }

      try {
         this.isoCode          = isoCode.trim() ;
      } catch ( NullPointerException npe ) {
         this.isoCode = null ;
      }

      this.countryCode      = ""+ind ;
      this.zipMask          = zipMask ;
      this.phoneMask        = phoneMask ;
      this.zipDigits        = zipDigits ;
      this.timeDiff         = timeDiff ;
      this.currency         = currency ;
      this.currencyMask     = currencyMask ;
      this.lang01           = lang01 ;
      this.lang02           = lang02 ;
      this.lang03           = lang03 ;
      this.geoInd           = geoInd ;

      if ( ( multiZone != null ) && (  multiZone.equals( tmpMultiZone ) ) )
         this.multiZone = true ;
      else
         this.multiZone = false ;

      if ( usesDST.equals( tmpUsesDST ) )
         this.usesDST = true ;
      else
         this.usesDST = false ;


   }


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a new TypeCountryCodeRec
    *  with the data from an old one.
    *  
    *             
    *  @param oldRec The TypeCountryCodeRec to get data from
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public TypeCountryCodeRec ( TypeCountryCodeRec oldRec ) {
      phoneCountryCode = oldRec.phoneCountryCode ;
      countryCode      = oldRec.countryCode ;
      zipMask          = oldRec.zipMask ;
      phoneMask        = oldRec.phoneMask ;
      timeDiff         = oldRec.timeDiff ;
      currency         = oldRec.currency ;
      currencyMask     = oldRec.currencyMask ;
      zipDigits        = oldRec.zipDigits ;
      lang01           = oldRec.lang01 ;
      lang02           = oldRec.lang02 ;
      lang03           = oldRec.lang03 ;
      multiZone        = oldRec.multiZone ;
      usesDST          = oldRec.usesDST ;
      setDescript ( oldRec.getDescript() ) ;
      setInd ( oldRec.getInd() ) ;
      geoInd           = oldRec.geoInd ;
   }



   /*-----------------------------------------------------------*/
   /* columnCount
    */
   /**
    *    This method will return how many columns
    *  should appear in a ContainerWin to display
    *  this record
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return int
    */
   public int columnCount () {
      return( 5 ) ;
   }



   /*-----------------------------------------------------------*/
   /* getDisplayDescript
    */
   /**
    *    This method will return what should appear
    *  in a certain column in a ContainerWin
    *  
    *             
    *  @param column The column to return data for
    *  @param 
    *  @param 
    *
    *  @return A character array representing the column's data
    */
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = getDescript() ;
            break ;

         case 1 :
            retObj = ""+getInd() ;
            break ;

         case 2 :
            retObj = zipMask ;
            break ;

         case 3 :
            retObj = phoneMask ;
            break ;

         case 4 :
            retObj = ""+zipDigits ;
            break ;
      }

      return( retObj ) ;
   }


   /*-----------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will check to see if another
    *  TypeCountryCodeRec is a match to this one
    *  
    *             
    *  @param countryRec The TypeCountryCodeRec to check
    *  @param 
    *  @param 
    *
    *  @return TRUE The record passed is a match
    */
   public boolean equals ( Object countryRec ) {
      boolean rc = false ;

      if ( countryRec instanceof TypeCountryCodeRec ) {
         if ( ( phoneCountryCode == ((TypeCountryCodeRec)countryRec).phoneCountryCode ) &&
              ( countryCode.equals ( ((TypeCountryCodeRec)countryRec).countryCode ) ) &&
              ( zipMask.equals ( ((TypeCountryCodeRec)countryRec).zipMask ) ) &&
              ( phoneMask.equals ( ((TypeCountryCodeRec)countryRec).phoneMask ) ) &&
              ( timeDiff.equals ( ((TypeCountryCodeRec)countryRec).timeDiff ) ) &&
              ( currency.equals ( ((TypeCountryCodeRec)countryRec).currency ) ) &&
              ( currencyMask.equals ( ((TypeCountryCodeRec)countryRec).currencyMask ) ) &&
              ( zipDigits == ((TypeCountryCodeRec)countryRec).zipDigits ) && 
              ( geoInd    == ((TypeCountryCodeRec)countryRec).geoInd ) &&
              ( multiZone == ((TypeCountryCodeRec)countryRec).multiZone ) &&
              ( usesDST   == ((TypeCountryCodeRec)countryRec).usesDST ) )
            rc = true ;
      }

      return( rc ) ;
   }







   /*------------------------------------------------------------*/
   /* cleanDBChars
    */
   /**
    *    cleans all of our strings for the database
    *  
    *  
    *
    *  @param sql The database to clean with
    *  @param typeRec The TypeRecord to clean
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cleanDBChars ( ) {
      super.cleanDBChars ( ) ;
      countryCode  = Text.cleanDBString ( countryCode ) ;
      zipMask      = Text.cleanDBString ( zipMask ) ;   
      phoneMask    = Text.cleanDBString ( phoneMask ) ;  
      timeDiff     = Text.cleanDBString ( timeDiff ) ;  
      currency     = Text.cleanDBString ( currency ) ;
      currencyMask = Text.cleanDBString ( currencyMask ) ;
   } /* cleanDBString */





   /*------------------------------------------------------------*/
   /* getDateMask
    */
   /**
    *    This method will get the date mask associated
    *  to this country
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getDateMask () {
      return( "99/99/9999" ) ;
   }


   /*------------------------------------------------------------*/
   /* getCurrencyMask
    */
   /**
    *    This method will get the currency mask associated
    *  to this country
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getCurrencyMask () {
      return( currencyMask ) ;
   }



   /*------------------------------------------------------------*/
   /* getZipMask
    */
   /**
    *    This method will get the zip mask associated
    *  to this country
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getZipMask () {
      return( zipMask ) ;
   }





   /*------------------------------------------------------------*/
   /* getGeoInd
    */
   /**
    *    This method will return the geography indicator
    *  associated to this country
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getGeoInd() {
      return( geoInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getPhoneCountryCode
    */
   /**
    *    This method will return our phone country code
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public short getPhoneCountryCode () {
      return( phoneCountryCode ) ;
   }



   /**
     * This method will currently only DELETE this country from the
     * database.
     **/
   public int writeToDatabase () {
      int       sqlCode   = 0 ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeCountryCodeRec.writeToDatabase", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         switch ( getRecStatus() ) {
            case REC_DELETE :
               stmt.executeUpdate ( "DELETE FROM PRODUCT.COUNTRY WHERE COUNTRYCODEIND = "+getInd() ) ;
               stmt.executeUpdate ( "DELETE FROM PRODRAFT.COUNTRY WHERE COUNTRYCODEIND = "+getInd() ) ;
               stmt.executeUpdate ( "DELETE FROM TIGRIS.DOCCOUNTRY WHERE COUNTRYCODEIND = "+getInd() ) ;
               stmt.executeUpdate ( "DELETE FROM QUEST.DOCCOUNTRY WHERE COUNTRYCODEIND = "+getInd() ) ;
               stmt.executeUpdate ( "DELETE FROM TIGRIS.GEOCOUNTRY WHERE COUNTRYCODEIND = "+getInd() ) ;
               stmt.executeUpdate ( "DELETE FROM TIGRIS.TYPECOUNTRYCODE WHERE COUNTRYCODEIND = "+getInd() ) ;
               break ;

         }



      } catch ( SQLException e ) {
         sqlCode = e.getErrorCode() ;
         sqlMethod.rollBack() ;
      }

      return( sqlCode ) ;

   }
} /* TypeCountryCodeRec */

