package com.ibm.nzna.projects.common.quest.type ;
/*-----------------------------------------------------------*/
/*   TypeGeoRec                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeGeoRec                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The TypeGeoRec holds specific information about a single geography
 *
 *
 * @version 1.0, 2/28/100 
 * @geoor  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeGeoRec                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeGeoRec extends TypeRec {


   /**
     * This is the vector of countries that belong to this geography
     */
   private Vector countryVec = null ;

   /**
     *  This property enables/disables enhanced output. Basically
     * enhanced output is a String of HTML Tags that makes the
     * description Bold when viewing in a browser
     **/
   private boolean enhanceOutput = false ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default and only constructor
    *  
    *  
    *
    *  @param ind The Geo Indicator
    *  @param descript The description of the brand
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TypeGeoRec ( int geoInd, String descript ) {
      super ( geoInd, descript ) ;
   }



   /*------------------------------------------------------------*/
   /* getCountryVec
    */
   /**
    *    This method will return a vector containing
    *  all of the countries that are located in this
    *  geography
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getCountryVec() {
      if ( countryVec == null ) {
         Vector typeCountryVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;
         int    i              = 0 ;
         int    length         = 0 ;

         countryVec = new Vector ( 10 ) ;

         if ( typeCountryVec != null ) {
            length = typeCountryVec.size() ;

            while ( i < length ) {
               if ( ((TypeCountryCodeRec)typeCountryVec.elementAt(i)).getGeoInd() == getInd() )
                  countryVec.addElement ( typeCountryVec.elementAt(i) ) ;
               i++ ;
            }
         }
      }

      return( countryVec ) ;

   }


   /**
     * This method will read in all of the countries that this geography has using
     * the Connection object passed in.
     *
     * @param conn The Database Connection to use
     */
   public Vector getCountryVec ( Connection conn ) throws SQLException {
      if ( countryVec == null ) {
         Statement stmt      = conn.createStatement() ;
         ResultSet rs        = stmt.executeQuery ( "SELECT A.COUNTRYCODEIND, A.DESCRIPT "+
                                                   "FROM TIGRIS.TYPECOUNTRYCODE A, "+
                                                   "     TIGRIS.GEOCOUNTRY B "+
                                                   "WHERE B.GEOIND = "+getInd()+" AND "+
                                                   "      A.COUNTRYCODEIND = B.COUNTRYCODEIND" ) ;

         countryVec = new Vector ( 10 ) ;
         while ( rs.next() )
            countryVec.addElement ( new TypeCountryCodeRec ( rs.getInt(1), rs.getString(2).trim() ) ) ;

         rs.close() ;
         stmt.close() ;
      }

      return( countryVec ) ;

   }


   /**
     *  This method will set the enhanceOutput property. Please
     * see the enhanceOutput property above for more information.
     *
     * @param enhanceOutput TRUE/FALSE
     **/
   public void setEnhanceOutput ( boolean enhanceOutput ) {
      this.enhanceOutput = enhanceOutput ;
   }


   /**
     *  This method will return the string description of
     * a Geography
     **/
   public String toString () {
      if ( enhanceOutput )
         return( super.toString()+" (All Countries)" ) ;
      return( super.toString() ) ;
   }


   public int writeToDatabase() {
      int       sqlCode   = 0 ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "TypeGeoRec.writeToDatabase", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         switch ( getRecStatus() ) {
            
            case REC_UPDATE :
               stmt.executeUpdate ( "UPDATE TIGRIS.TYPEGEOGRAPHY SET CHANGEDTIME = CURRENT TIMESTAMP, DESCRIPT = '"+Text.cleanDBString ( toString() )+"' "+
                                    "WHERE GEOIND = "+getInd() ) ;
               break ;

         }


      } catch ( SQLException e ) {
         sqlCode = e.getErrorCode() ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;

      return( sqlCode ) ;
   }


   public Object clone () {
      TypeGeoRec geoRec = new TypeGeoRec ( getInd(), getDescript() ) ;

      if ( countryVec != null )
         geoRec.countryVec = (Vector)countryVec.clone() ;
      geoRec.enhanceOutput  = enhanceOutput ;
      return ( geoRec ) ;
   }
} /* TypeGeoRec */
