package com.ibm.nzna.projects.common.quest.type ;
/*-----------------------------------------------------------*/
/*   TypeList                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeList                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * A collection of static methods that allow easy manipulation of TypeLists. Please
 * Note that per Application you must first create a new instance of TypeList. Once
 * the instance is created you must call the initialize method.<br><br><br>
 *
 * To add a new Type List to this object, you must:<br>
 * 1. Add a new constant for the TYPE_XXX. Remember to update the max type list as well<br>
 * 2. Create a new class that extends TypeRec for the new Type List<br>
 * 3. Add a method in com.ibm.nzna.projects.common.storedProc.typeListReader.TypeListReader to read the new type list <br>
 * 4. Add a table name in the getTableName method for the new list if the table can be cached
 *
 *
 * @version 1.0, 3/5/1998 
 * @author  David Lentz
 */
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.storedProc.typeListReader.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import javax.swing.* ;
import java.util.* ;
import java.sql.* ;
import java.awt.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeList                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeList extends Object implements java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int TYPE_AUTH         = 1 ;
   public static final int TYPE_COUNTRYCODE  = 3 ;
   public static final int TYPE_DOC          = 4 ;
   public static final int TYPE_DOCCLASS     = 5 ;
   public static final int TYPE_GEOGRAPHY    = 6 ;
   public static final int TYPE_LANGUAGE     = 8 ;
   public static final int TYPE_TEAM         = 9  ;
   public static final int TYPE_DOCCAT       = 10 ;
   public static final int TYPE_METRIC       = 11 ;
   public static final int TYPE_PROPERTIES   = 12 ;
   public static final int TYPE_PUB          = 13 ;
   public static final int TYPE_WORKREQUIRED = 14 ;
   public static final int TYPE_STATUS       = 15 ;
   public static final int TYPE_RESOLUTION   = 16 ;
   public static final int TYPE_ACTION       = 17 ;
   public static final int TYPE_QUESTION     = 18 ;
   public static final int TYPE_SYMPTOM      = 19 ;
   public static final int TYPE_OABRAND      = 20 ;
   public static final int TYPE_LINK         = 21 ;
   public static final int TYPE_CUSTOMVIEW   = 22 ;
   public static final int TYPE_PRODUCT      = 23 ;
   public static final int TYPE_OFFERING     = 24 ;
   public static final int TYPE_CUSTOMER     = 25 ;
   public static final int TYPE_WARRANTY     = 26 ;
   public static final int TYPE_DATE         = 27 ;
   public static final int TYPE_CANCEL       = 28 ;
   public static final int MAX_TYPE_LIST     = 29 ;                                             

   /*---------------------------------------------------------------*/
   /*                                                               */
   /*   Static Members that are related to a table in the database  */
   /*                                                               */
   /*---------------------------------------------------------------*/
   public static final int WORKREQ_NOTAPPLICABLE            = 0 ;   // Defined in TIGRIS.TYPEWORKREQ
   public static final int WORKREQ_CREATE                   = 1 ;   // Defined in TIGRIS.TYPEWORKREQ
   public static final int WORKREQ_EDIT                     = 2 ;   // Defined in TIGRIS.TYPEWORKREQ
   public static final int WORKREQ_DELETE                   = 3 ;   // Defined in TIGRIS.TYPEWORKREQ

   public static final int DOCCLASS_IBM_CONFIDENTIAL        = 1 ;   // Defined in TIGRIS.TYPEDOCCLASS
   public static final int DOCCLASS_DEALER_CONFIDENTIAL     = 2 ;   // Defined in TIGRIS.TYPEDOCCLASS
   public static final int DOCCLASS_UNCLASSIFIED            = 3 ;   // Defined in TIGRIS.TYPEDOCCLASS

   public static final int LANGUAGE_ENGLISH                 = 1 ;   // Defined in TIGRIS.TYPELANGUAGE
   public static final int LANGUAGE_GERMAN                  = 2 ;   // Defined in TIGRIS.TYPELANGUAGE
   public static final int LANGUAGE_SPANISH                 = 3 ;   // Defined in TIGRIS.TYPELANGUAGE
   public static final int LANGUAGE_FRENCH                  = 4 ;   // Defined in TIGRIS.TYPELANGUAGE
   public static final int LANGUAGE_PORTUGESE               = 5 ;   // Defined in TIGRIS.TYPELANGUAGE
   public static final int LANGUAGE_ITALIAN                 = 6 ;   // Defined in TIGRIS.TYPELANGUAGE

   public static final int GEO_WORLD_WIDE                   = 6 ;   // Defined in TIGRIS.TYPEGEOGRAPHY
   public static final int COUNTRY_WORLD_WIDE               = 124 ; // Defined in TIGRIS.TYPECOUNTRYCODE

   public static final int DOCTYPE_PARTS_INFO               = 2 ;   // Defined in TIGRIS.TYPEDOCUMENT
   public static final int DOCTYPE_PRODUCT_INFO             = 3 ;   // Defined in TIGRIS.TYPEDOCUMENT
   public static final int DOCTYPE_SERVICE_HINTS_TIPS       = 4 ;   // Defined in TIGRIS.TYPEDOCUMENT
   public static final int DOCTYPE_SSD                      = 9 ;   // Defined in TIGRIS.TYPEDOCUMENT
   public static final int DOCTYPE_SERVICE_PUB              = 8 ;   // Defined in TIGRIS.TYPEDOCUMENT
   public static final int DOCTYPE_PRELOAD                  = 10 ;  // Defined in TIGRIS.TYPEDOCUMENT

   public static final int PUBLICATIONTYPE_NOTAPPLICABLE    = 0 ;   // Defined in TIGRIS.TYPEPUBLICATION
   public static final int PUBLICATIONTYPE_FUNCTIONALSPECS  = 1 ;   // Defined in TIGRIS.TYPEPUBLICATION
   public static final int PUBLICATIONTYPE_HARDWAREMANUAL   = 2 ;   // Defined in TIGRIS.TYPEPUBLICATION
   public static final int PUBLICATIONTYPE_USERMANUAL       = 3 ;   // Defined in TIGRIS.TYPEPUBLICATION

   public static final int DATE_WITHDRAWN         = 1 ;  // Defined in PRODUCT.TYPEDATE
   public static final int DATE_ANNOUNCED         = 2 ;  // Defined in PRODUCT.TYPEDATE
   public static final int DATE_DELETED_FROM_XREF = 3 ;  // Defined in PRODUCT.TYPEDATE
   public static final int DATE_PRE_ANNOUNCED     = 4 ;  // Defined in PRODUCT.TYPEDATE
   public static final int DATE_GA_AVAILABLE      = 5;   // Defined in PRODUCT.TYPEDATE
   public static final int DATE_END_OF_SERVICE    = 6;   // Defined in PRODUCT.TYPEDATE
   public static final int DATE_END_OF_MANUFACT   = 7;   // Defined in PRODUCT.TYPEDATE
   public static final int DATE_END_OF_MARKET     = 8;   // Defined in PRODUCT.TYPEDATE
   public static final int DATE_END_OF_PRODUCTION = 9;   // Defined in PRODUCT.TYPEDATE
   public static final int DATE_SHIP_SUPPORTED    = 10 ; // Defined in PRODUCT.TYPEDATE
   public static final int DATE_FIRST_ORDER       = 11 ; // Defined in PRODUCT.TYPEDATE

   public static final int OFFERING_DEFAULT        = 0 ; //Defined in PRODUCT.TYPEOFFERING
   public static final int OFFERING_CTO            = 1 ; //Defined in PRODUCT.TYPEOFFERING



   public static final int PRODUCTTYPE_SYSTEM      = 1 ; // Defined in PRODUCT.TYPEPRODUCT
   public static final int PRODUCTTYPE_OPTION      = 2 ; // Defined in PRODUCT.TYPEPRODUCT
   public static final int PRODUCTTYPE_MONITOR     = 3 ; // Defined in PRODUCT.TYPEPRODUCT

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private        Vector        typeLists         = null ;
   public  static boolean       cacheEnabled      = false ;
   public  static TypeList      instance          = null ;
   public  static int           languageInd       = 1 ;     // Default to english
   public  static Vector        statusListenerVec = null ;
   public         String        cacheDir          = "" ;




   /*------------------------------------------------------------*/
   /* Constructor #1
    */
   /**
    *    Default constructor.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TypeList () {
      if ( instance == null )
         instance = this ;
   }

   public void initialize ( boolean cache ) {
      this.cacheEnabled = cache ;
      initializeTypeList() ;
   }


   /*------------------------------------------------------------*/
   /* setInstance
    */
   /**
    *    This method can be used to set the instance
    *  of TypeList that all static methods will use
    *  
    *
    *  @param TypeList The new instance
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setInstance ( TypeList typeList ) {
      this.instance = typeList ;
   }


   /*------------------------------------------------------------*/
   /* initializeTypeList
    */
   /**
    *    This method will create each of our
    *  type list objects in our TypeListRec array
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeTypeList () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our vector of lists                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      typeLists = new Vector ( MAX_TYPE_LIST, 1 ) ;

      initializeTypeList (  TYPE_AUTH )  ;
      initializeTypeList (  TYPE_COUNTRYCODE )  ;
      initializeTypeList (  TYPE_DOC )  ;
      initializeTypeList (  TYPE_DOCCLASS )  ;
      initializeTypeList (  TYPE_GEOGRAPHY )  ;
      initializeTypeList (  TYPE_LANGUAGE )  ;
      initializeTypeList (  TYPE_TEAM )  ;
      initializeTypeList (  TYPE_DOCCAT )  ;
      initializeTypeList (  TYPE_METRIC )  ;
      initializeTypeList (  TYPE_PROPERTIES )  ;
      initializeTypeList (  TYPE_PUB )  ;
      initializeTypeList (  TYPE_WORKREQUIRED )  ;
      initializeTypeList (  TYPE_STATUS )  ;
      initializeTypeList (  TYPE_RESOLUTION )  ;    
      initializeTypeList (  TYPE_ACTION )  ;    
      initializeTypeList (  TYPE_QUESTION )  ;    
      initializeTypeList (  TYPE_SYMPTOM )  ;    
      initializeTypeList (  TYPE_OABRAND )  ;    
      initializeTypeList (  TYPE_LINK )  ;    
      initializeTypeList (  TYPE_CUSTOMVIEW )  ;    
      initializeTypeList (  TYPE_DATE )  ;    
      initializeTypeList (  TYPE_PRODUCT )  ;    
      initializeTypeList (  TYPE_OFFERING )  ;    
      initializeTypeList (  TYPE_CUSTOMER )  ;    
      initializeTypeList (  TYPE_WARRANTY )  ;    
      initializeTypeList (  TYPE_CANCEL )  ;    

   }




   /*------------------------------------------------------------*/
   /* initializeTypeList
    */
   /**
    *    This method will check to see if a
    *  type list is cached on disk. If it is then
    *  the type list is read in and compared against
    *  the database. This compare simply compares the greatest
    *  changedtime in the list from cache and the greatest changedtime
    *  in the database. If the database has a greater changed time then
    *  the list is marked as invalid so that it rereads when requested to
    *  through the getTypeList method.
    *
    *  @param typeListInd The indicator of the type list
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeTypeList ( int typeListInd ) {
      TypeListRec listRec     = null ;
      String      lastUpdated = null ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only read from cache if it is enabled                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cacheEnabled ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, read in the file from disk                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            listRec     = (TypeListRec)FileUtil.readObjectFromDisk ( getCacheDir()+"TypeList_"+typeListInd+".cache" ) ;
            lastUpdated = (String)FileUtil.readObjectFromDisk ( getCacheDir()+"TypeList_"+typeListInd+"_CacheTime.cache" ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the list is not null, make sure that the timestamp   */
            /* for the list is less than what is in the database         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( listRec != null ) {
               if ( hasTypeListChanged ( typeListInd, lastUpdated ) ) {
                  listRec = null ;
               }
            }
         } /* if cache  */


      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The Exception is usually File Not Found, or a           */
      /* ClassCastException. Regardless of what exception is       */
      /* thrown we want to reread the list                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      catch ( java.io.FileNotFoundException e1 ) {
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      if ( listRec == null )
         listRec = new TypeListRec ( typeListInd ) ;

      typeLists.addElement ( listRec ) ;
   }






   /*------------------------------------------------------------*/
   /* hasTypeListChanged
    */
   /**
    *    This method will determine if a type list has changed
    *  by querying the type list table for the number of records
    *  that have a timestamp greater than the timestamp passed in
    *
    *  @param typeListInd The Type List Indicator
    *  @param timeStamp The timestamp to compare with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean hasTypeListChanged ( int typeListInd, String timeStamp ) {
      String      tableName = getTableName ( typeListInd ) ;
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "hasTypeListChanged", 5 ) ;
      boolean     rc        = false ;

      if ( ( tableName != null ) && ( tableName.length() > 0 ) ) {
         try {
            Statement stmt = sqlMethod.createStatement() ;
            ResultSet rs   = stmt.executeQuery ( "SELECT changedtime FROM "+tableName+" where changedtime > '"+timeStamp+"' FETCH FIRST 1 ROWS ONLY" ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Fetch the latest changedtime                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rc = rs.next() ;

            rs.close() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   In any exception, return a really low timestamp         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         catch ( Exception e ) {
            sqlMethod.rollBack() ;
            e.printStackTrace() ;
            rc = false ;
         }
      } /* if Table Name is good  */
      else
         rc = true ; //Assume that the table has changed


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always close our SQLMethod!                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getTableName
    */
   /**
    *    This method will return the actual Quest
    *  table name for the Type List specified
    *  
    *
    *  @param typeListInd The Type List return the table name of 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static String getTableName ( int typeListInd ) {
      String tableName = "" ;

      switch ( typeListInd ) {
         case TYPE_AUTH :
            tableName = "TIGRIS.TYPEAUTH" ;
            break ;

         case TYPE_COUNTRYCODE :
            tableName = "TIGRIS.TYPECOUNTRYCODE" ;
            break ;

         case TYPE_DOC :
            tableName = "TIGRIS.TYPEDOCUMENT" ;
            break ;

         case TYPE_DOCCLASS :
            tableName = "TIGRIS.TYPEDOCCLASS" ;
            break ;

         case TYPE_GEOGRAPHY :
            tableName = "TIGRIS.TYPEGEOGRAPHY" ;
            break ;

         case TYPE_LANGUAGE :
            tableName = "TIGRIS.TYPELANGUAGE" ;
            break ;

         case TYPE_TEAM :
            tableName = "TIGRIS.USERGROUP" ;
            break ;

         case TYPE_DOCCAT :
            tableName = "TIGRIS.TYPEDOCCAT" ;
            break ;

         case TYPE_METRIC :
            tableName = "TIGRIS.TYPEMETRIC" ;
            break ;

         case TYPE_PROPERTIES :
            break ;

         case TYPE_PUB :
            tableName = "TIGRIS.TYPEPUBLICATION" ;
            break ;

         case TYPE_WORKREQUIRED :
            tableName = "TIGRIS.TYPEWORKREQUIRED" ;
            break ;

         case TYPE_STATUS :
            tableName = "TIGRIS.TYPESTATUS" ;
            break ;

         case TYPE_RESOLUTION :
            tableName = "QUEST.TYPERESOLUTION" ;
            break ;

         case TYPE_ACTION :
            tableName = "OA.TYPEACTION" ;
            break ;

         case TYPE_QUESTION :
            tableName = "OA.TYPEQUESTION" ;
            break ;

         case TYPE_SYMPTOM :
            tableName = "OA.TYPESYMPTOM" ;
            break ;

         case TYPE_OABRAND :
            tableName = "OA.TYPEBRAND" ;
            break ;

         case TYPE_LINK :
            tableName = "OA.TYPELINK" ;
            break ;

         case TYPE_CUSTOMVIEW :
            tableName = "QIT.TYPECUSTOMVIEW" ;
            break ;

      }

      return( tableName ) ;
   }



   /*------------------------------------------------------------*/
   /* typeListRecFromInd
    */
   /**
    *    This method will find the TypeListRec
    *  in our vector that matches the typeListInd
    *  passed in
    *
    *  @param typeListInd The Type List Indicator ( TYPE_AUTH, TYPE_DOC etc.. )
    *  @param None
    *  @param None
    *
    *  @return TypeListRec
    *//*---------------------------------------------------------*/
   protected final TypeListRec typeListRecFromInd ( int typeListInd ) {
      TypeListRec retRec     = null ;

      if ( typeLists != null ) {
         int         i          = 0 ;
         int         length     = typeLists.size() ;
         TypeListRec compareRec = null ;

         while ( ( retRec == null ) && ( i < length ) ) {
            compareRec = (TypeListRec) typeLists.elementAt(i) ;
            if ( compareRec.getTypeListInd() == typeListInd )
               retRec = compareRec ;
            i++ ;
         }
      } /* if typeLists  */

      return( retRec ) ; 
   }




   /*------------------------------------------------------------*/
   /* getTypeListVec
    */
   /**
    *    This method will return a vector of all
    *  type lists
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getTypeListVec () {
      return( typeLists ) ;
   }


   public static TypeList getInstance () {
      if ( instance == null )
         instance = new TypeList() ;
      return( instance ) ;
   }


   /*-----------------------------------------------------------*/
   /* getTypeList
    */
   /**
    *    This method will return a copy of the Vector
    *  holding the type list requested
    *  
    *
    *  @param typeList The type list to retrieve ( TYPE_AUTH, TYPE_PRICE etc.. )
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getTypeList ( int typeListInd ) {
      TypeListRec typeListRec = typeListRecFromInd ( typeListInd ) ;

      if ( validateTypeList ( typeListRec ) )
         return( typeListRec.getTypeListVec() ) ;

      return( null ) ;
   }

   /*-----------------------------------------------------------*/
   /* stringFromInd
    */
   /**
    *    This method will return the description of a
    *  Type List element from the indicator of the element.
    *  
    *
    *  @param ind The indicator of the element
    *  @param typeList The type list to look in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public  String stringFromInd ( int ind, int typeList ) {
      Vector  vec     = getTypeList ( typeList ) ;
      String  retStr  = "" ;
      TypeRec typeRec = null ;

      if ( vec != null ) {
         int     length = vec.size() ;
         int     i      = 0 ;
         boolean found  = false ;

         while ( ( i < length ) && ( !found ) ) {
            typeRec = (TypeRec) vec.elementAt ( i ) ;

            if ( typeRec != null )
               found = typeRec.isInd ( ind ) ;
            i++ ;
         }

         if ( found )
            retStr = typeRec.toString() ;
      }
      return( retStr ) ;
   }



   /*-----------------------------------------------------------*/
   /* indFromString
    */
   /**
    *    This method will return the indicator of a type list
    *  element that matches the string passed in.
    *  
    *
    *  @param descript The description string of the type list
    *  @param typeList The Type List to look in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public  int indFromString ( String descript, int typeList ) {
      Vector  vec     = getTypeList ( typeList ) ;
      int     retInd  = 0 ;
      TypeRec typeRec = null ;
      int     ind     = 0 ;

      if ( vec != null ) {
         int     length = vec.size() ;
         int     i      = 0 ;
         boolean found  = false ;

         while ( ( i < length ) && ( !found ) ) {
            typeRec = (TypeRec) vec.elementAt ( i ) ;

            if ( typeRec != null )
               found = typeRec.toString().equals ( descript ) ;
            i++ ;
         }

         if ( found )
            retInd = typeRec.getInd() ;
      }
      return( retInd ) ;
   }




   /*------------------------------------------------------------*/
   /* objectFromInd
    */
   /**
    *    This method will return an Object from 
    *  an indicator
    *  
    *
    *  @param ind The TypeList indicator to look for
    *  @param typeList the list to look in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public  Object objectFromInd ( int ind, int typeList ) {
      Vector   vec     = getTypeList ( typeList ) ;
      Object   retObj = null ;
      TypeRec  typeRec = null ;

      if ( vec != null ) {
         int     length = vec.size() ;
         int     i      = 0 ;
         boolean found  = false ;

         while ( ( i < length ) && ( !found ) ) {
            if ( vec.elementAt(i) instanceof TypeRec ) {
               typeRec = (TypeRec) vec.elementAt ( i ) ;

               if ( typeRec != null )
                  found = typeRec.isInd ( ind ) ;
            } else
               if ( vec.elementAt(i) instanceof String )
               System.err.println ( "Error. TypeList of: "+typeList+" has a Vector of Strings! These records were expected to be TypeRecs." ) ;
            else
               System.err.println ( "Error. TypeList of: "+typeList+" has a Vector of non-TypeRecs." ) ;
            i++ ;
         }

         if ( found )
            retObj = typeRec ;
      }
      return( retObj ) ;

   }



   /*-----------------------------------------------------------*/
   /* objectFromString
    */
   /**
    *    This method will return an Object from 
    *  an indicator
    *  
    *
    *  @param descript The description to look with
    *  @param typeList the list to look in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public  Object objectFromString ( String descript, int typeList ) {
      Vector   vec     = getTypeList ( typeList ) ;
      Object   retObj = null ;
      TypeRec  typeRec = null ;

      if ( vec != null ) {
         int     length = vec.size() ;
         int     i      = 0 ;
         boolean found  = false ;

         while ( ( i < length ) && ( !found ) ) {
            typeRec = (TypeRec) vec.elementAt ( i ) ;

            if ( typeRec != null )
               found = typeRec.toString().equals ( descript ) ;
            i++ ;
         }

         if ( found )
            retObj = typeRec ;
      }
      return( retObj ) ;

   }




   /*-----------------------------------------------------------*/
   /* addToList
    */
   /**
    *    This method will add an object to a type list.
    *  
    *  
    *
    *  @param typeRec The TypeRec to add to the list
    *  @param typeList The type list to add to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addToList ( TypeRec typeRec, int typeList ) {
      getInstance().typeListRecFromInd ( typeList ).addToList ( typeRec ) ;
      getInstance().setTypeList ( typeList, getInstance().getTypeList ( typeList ) ) ;
   }




   /*-----------------------------------------------------------*/
   /* removeFromList
    */
   /**
    *    This method will remove an item from a type list
    *  
    *  
    *
    *  @param typeRec The item to remove
    *  @param typeList The Type List to remove it from
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeFromList ( TypeRec typeRec, int typeList ) {
      getInstance().typeListRecFromInd ( typeList ).removeFromList ( typeRec ) ;
      getInstance().setTypeList ( typeList, getInstance().getTypeList ( typeList ) ) ;
   }



   public static void replaceList ( Vector vec, int typeListInd ) {
      getInstance().typeListRecFromInd ( typeListInd ).setTypeList ( vec ) ;
   }

   /*-----------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove all type lists
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeAll () {
      int i      = 0 ;
      int length = getInstance().typeLists.size() ;

      while ( i < length )
         ((TypeListRec)getInstance().typeLists.elementAt(i++)).removeAll() ;
      i++ ;
   }


   /*-----------------------------------------------------------*/
   /* readStartupLists
    */
   /**
    *    This method will read all of the TypeLists we
    *  read when the application initializes
    *  
    *
    *  @param LanguageInd The language to read the lists in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean readStartupLists ( int languageInd ) {
      boolean           rc                = false ;

      /*----------------------------------------------------------------*/
      /*                                                                */
      /*   If the cache is enabled, try to read the lists from cache    */
      /*                                                                */
      /*----------------------------------------------------------------*/
      if ( cacheEnabled ) {
         if ( !getInstance().typeListRecFromInd ( TYPE_GEOGRAPHY ).isValid() )
            getInstance().getTypeList ( TYPE_GEOGRAPHY ) ;

         if ( !getInstance().typeListRecFromInd ( TYPE_COUNTRYCODE ).isValid() )
            getInstance().getTypeList ( TYPE_COUNTRYCODE ) ;

         if ( !getInstance().typeListRecFromInd ( TYPE_DOC ).isValid() )
            getInstance().getTypeList ( TYPE_DOC ) ;

         if ( !getInstance().typeListRecFromInd ( TYPE_DOCCLASS ).isValid() )
            getInstance().getTypeList ( TYPE_DOCCLASS ) ;

         if ( !getInstance().typeListRecFromInd ( TYPE_CUSTOMVIEW ).isValid() )
            getInstance().getTypeList ( TYPE_CUSTOMVIEW ) ;

         if ( !getInstance().typeListRecFromInd ( TYPE_AUTH ).isValid() )
            getInstance().getTypeList ( TYPE_AUTH ) ;

         if ( !getInstance().typeListRecFromInd ( TYPE_WORKREQUIRED ).isValid() )
            getInstance().getTypeList ( TYPE_WORKREQUIRED ) ;

         if ( !getInstance().typeListRecFromInd ( TYPE_METRIC ).isValid() )
            getInstance().getTypeList ( TYPE_METRIC ) ;

         if ( !getInstance().typeListRecFromInd ( TYPE_LANGUAGE ).isValid() )
            getInstance().getTypeList ( TYPE_LANGUAGE ) ;

         rc = true ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, read in the lists through stored procedure   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         SQLMethod         sqlMethod         = new SQLMethod ( DatabaseSystem.QUEST, "TypeList.readStartupLists", 5 ) ;
         CallableStatement stmt              = sqlMethod.createStoredProc ( "QUEST.TypeListReader", 4 ) ;
         TypeListReaderRec typeListReaderRec = null ;

         try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up our parameters                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.setInt ( 1, languageInd ) ;          // Our Language Indicator
            stmt.setInt ( 2, -1 ) ;                                // Our Type List to read
            stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ; // The return Lob for TypeListReaderRec
            stmt.registerOutParameter ( 4, Types.LONGVARBINARY ) ; // The return Lob for StoredProcRetRec

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            stmt.execute() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the StoredProcRec from the Lob                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 4 ) ;

            if ( ( retRec != null ) && ( retRec.error ) ) {
               LogSystem.log ( 1, retRec.errorStr ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise continue                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get our return object                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               typeListReaderRec = (TypeListReaderRec) StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;           

               getInstance().setTypeList ( TYPE_GEOGRAPHY    , typeListReaderRec.geographyVec ) ;
               getInstance().setTypeList ( TYPE_COUNTRYCODE  , typeListReaderRec.countryCodeVec ) ;
               getInstance().setTypeList ( TYPE_DOC          , typeListReaderRec.docTypeVec ) ;
               getInstance().setTypeList ( TYPE_DOCCLASS     , typeListReaderRec.docClassVec ) ;
               getInstance().setTypeList ( TYPE_CUSTOMVIEW   , typeListReaderRec.customViewVec ) ;
               getInstance().setTypeList ( TYPE_AUTH         , typeListReaderRec.authVec ) ;
               getInstance().setTypeList ( TYPE_WORKREQUIRED , typeListReaderRec.workVec ) ;
               getInstance().setTypeList ( TYPE_METRIC       , typeListReaderRec.metricVec ) ;
               getInstance().setTypeList ( TYPE_LANGUAGE     , typeListReaderRec.languageVec ) ;

               rc = true ;

            }

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
         }

         sqlMethod.close() ;
      }
      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* addStatusListener
    */
   /**
    *    This method will add a status listener to our
    *  vector
    *  
    *
    *  @param listener The TypeListStatusListener to receive status notifications
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addStatusListener ( TypeListStatusListener listener ) {
      if ( statusListenerVec == null )
         statusListenerVec = new Vector ( 1, 1 );

      statusListenerVec.addElement ( listener ) ;
   }






   /*------------------------------------------------------------*/
   /* fireStatusBeginEvent
    */
   /**
    *    This method will fire a statusBegin message
    *  
    *  
    *
    *  @param msg The message to fire
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void fireStatusBeginEvent ( String msg ) {
      if ( statusListenerVec != null ) {
         int i    = 0 ;
         int size = statusListenerVec.size() ;

         while ( i < size ) {
            ((TypeListStatusListener)statusListenerVec.elementAt(i)).typeListStatusChangeBegin ( msg ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* fireStatusEndEvent
    */
   /**
    *    This method will fire a statusEnd message
    *  
    *  
    *
    *  @param msg The message to fire
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void fireStatusEndEvent ( ) {
      if ( statusListenerVec != null ) {
         int i    = 0 ;
         int size = statusListenerVec.size() ;

         while ( i < size ) {
            ((TypeListStatusListener)statusListenerVec.elementAt(i)).typeListStatusChangeEnd (  ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* removeStatusListener
    */
   /**
    *    This method will remove a TypeListStatusListener
    *  from the TypeList.
    *  
    *
    *  @param listener The TypeListStatusListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeStatusListener ( TypeListStatusListener listener ) {
      if ( statusListenerVec != null )
         statusListenerVec.removeElement ( listener ) ;
   }


   /*-----------------------------------------------------------*/
   /* validateTypeList
    */
   /**
    *    This method will check to see if a TypeList
    *  is still good. If it is it will return TRUE, if it is not, it
    *  will try to read the type list from the database and return TRUE
    *
    *  @param typeList The type List to check
    *  @param languageInd The language to validate in
    *  @param None
    *
    *  @return TRUE - The Type List is good. FALSE - The type list is bad
    *//*---------------------------------------------------------*/
   public boolean validateTypeList ( TypeListRec typeListRec ) {
      boolean     rc          = false ;

      if ( typeListRec != null ) {

         try {
            if ( !typeListRec.isValid() ) {
               readTypeList ( typeListRec ) ;
            }

            rc = typeListRec.isValid() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, "Exception Validating Type List" ) ;
            LogSystem.log ( 1, e, false ) ;
         }

      } else
         System.out.println ( "Cannot validate NULL TypeListRec" ) ;

      return( rc ) ;
   }

   /*-----------------------------------------------------------*/
   /* readTypeList
    */
   /**
    *    This method will read the specified TypeList.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void readTypeList ( TypeListRec typeListRec ) {
      boolean           rc                = false ;
      SQLMethod         sqlMethod         = new SQLMethod ( DatabaseSystem.QUEST, "TypeList.readTypeList ( "+typeListRec+") ", 5 ) ;
      CallableStatement stmt              = sqlMethod.createStoredProc ( "QUEST.TypeListReader", 4 ) ;
      TypeListReaderRec typeListReaderRec = null ;


      fireStatusBeginEvent ( "Reading Type List "+typeListRec.toString() ) ;
      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up our parameters                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setInt ( 1, languageInd ) ;          // Our Language Indicator
         stmt.setInt ( 2, typeListRec.getTypeListInd() ) ;      // Our Type List to read
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ; // The return Lob for TypeListReaderRec
         stmt.registerOutParameter ( 4, Types.LONGVARBINARY ) ; // The return Lob for StoredProcRetRec

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 4 ) ;

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise continue                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            typeListReaderRec = (TypeListReaderRec) StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;           

            getInstance().setTypeList ( typeListRec.getTypeListInd(), typeListReaderRec.retVec ) ;                                   

            rc = true ;

         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;
      fireStatusEndEvent() ;

   }






   /*------------------------------------------------------------*/
   /* setCacheDir
    */
   /**
    *    This method will set the directory
    *  that the cache files are written and read
    *  from
    *
    *  @param dir The full path excluding the trailing path separator
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCacheDir ( String dir ) {
      if ( dir == null )
         cacheDir = "" ;
      else
         this.cacheDir = dir ;
   }





   /*------------------------------------------------------------*/
   /* setLanguage
    */
   /**
    *    This method will set what language all future
    *  reads will be done for
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setLanguage ( int newLanguageInd ) {
      languageInd = newLanguageInd ;
   }



   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will populate a Choice control
    *  with a type list
    *  
    *
    *  @param choice The choice control to populate
    *  @param typeList The Type List to populate with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( Choice choice, int typeList, TypeListListener listener ) {
      new TypeListRefresher ( choice, typeList, listener ) ;
   }


   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will populate a list control
    *  with a type list
    *  
    *
    *  @param list The multi list to populate
    *  @param typeList The Type List to populate with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( MultiList list, int typeList, TypeListListener listener ) {
      new TypeListRefresher ( list, typeList, listener ) ;
   }


   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will populate a JList control
    *  with a type list
    *  
    *
    *  @param choice The JList control to populate
    *  @param typeList The Type List to populate with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( JList list, int typeList, TypeListListener listener ) {
      new TypeListRefresher ( list, typeList, listener ) ;
   }

   /*------------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh a JComboBox
    *  with the type list requested
    *  
    *
    *  @param comboBox The JComboBox to populate
    *  @param listType The type of list
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( JComboBox comboBox, int listType, TypeListListener listener ) {
      new TypeListRefresher ( comboBox, listType, listener  ) ;
   }






   /*------------------------------------------------------------*/
   /* setTypeList
    */
   /**
    *    This method will set the type list specfiied to the
    *  vector passed in
    *  
    *
    *  @param typeListInd The Type List to set the vector on
    *  @param Vector a Vector of TypeList objects
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setTypeList ( int typeListInd, Vector typeList ) {
      TypeListRec typeListRec = getInstance().typeListRecFromInd ( typeListInd ) ;

      typeListRec.setTypeList ( typeList ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to cache, then save this type list            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( cacheEnabled ) {
         try {
            FileUtil.writeObjectToDisk ( getCacheDir()+"TypeList_"+typeListInd+".cache", typeListRec ) ;
            FileUtil.writeObjectToDisk ( getCacheDir()+"TypeList_"+typeListInd+"_CacheTime.cache", new CDate ( CDate.DATE_STAMP ).today() ) ;
         } catch ( Exception e ) {
            e.printStackTrace() ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* getCacheDir
    */
   /**
    *    This method will return the fully qualified
    *  path to where the Cache files are written
    *  and read from, including the last Path Separator
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getCacheDir () {
      if ( ( cacheDir == null ) || ( cacheDir.length() == 0 ) )
         return( "" ) ;
      return( cacheDir+java.io.File.separator ) ;
   }



} /* TypeList */            


/*-----------------------------------------------------------*/
/*   TypeListRefresher                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeListRefresher                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/*                                                           */
/*   The TypeListRefresher refreshes controls in a thread    */                                         
/*                                                           */
/*-----------------------------------------------------------*/
class TypeListRefresher extends Object implements Runnable {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Component        c        = null ;
   private int              listType = 0 ;
   private TypeListListener listener = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will take a Component and
    *  a type list and kick off the thread to
    *  refresh them.
    *
    *  @param c The Component who's list needs refreshing
    *  @param listType The type list to refresh with ( TYPE_AUTH etc.. )
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TypeListRefresher ( Component c, int listType, TypeListListener listener ) {
      this.c        = c ;
      this.listener = listener ;
      this.listType = listType ;
      new Thread ( this ).start() ;
      //run() ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will execute our thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      if ( c instanceof JComboBox )
         refreshList ( (JComboBox) c, listType ) ;

      else if ( c instanceof Choice )
         refreshList ( (Choice) c, listType ) ;

      else if ( c instanceof JList )
         refreshList ( (JList) c, listType ) ;

      else if ( c instanceof MultiList )
         refreshList ( (MultiList) c, listType ) ;

      if ( listener != null )
         listener.listRefreshed ( listType ) ;
   }







   /*------------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh a JComboBox 
    *  control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( JComboBox comboBox, int listType ) {
      Vector dataVec = TypeList.getInstance().getTypeList ( listType ) ;

      if ( dataVec != null ) {
         int    i       = 0 ;
         int    length  = dataVec.size() ;

         while ( i < length )
            comboBox.addItem ( dataVec.elementAt(i++ ) ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh a ComboBox
    *  control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( Choice choice, int listType ) {
      try {
         Vector vec = TypeList.getInstance().getTypeList ( listType ) ;
         int    i   = 0 ;

         if ( vec != null ) {
            int length = vec.size() ;

            while ( i < length ) {
               choice.addItem ( vec.elementAt(i).toString() ) ;
               i++ ;
            }
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }


   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will populate a MultiList
    *  with a type list.
    *  
    *
    *  @param dataCnr The MultiList to populate with
    *  @param typeList The type list to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( MultiList dataCnr, int typeList ) {
      try {
         Vector vec = TypeList.getInstance().getTypeList(typeList) ;

         dataCnr.removeAll() ;
         if ( vec != null )
            dataCnr.add ( vec ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   } /* refreshList */


   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will populate a JList control
    *  with a type list
    *  
    *
    *  @param choice The choice control to populate
    *  @param listType The Type List to populate with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void refreshList ( JList list, int listType ) {
      try {
         Vector vec = TypeList.getInstance().getTypeList ( listType ) ;

         list.setListData ( vec ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

   }
}
