package com.ibm.nzna.projects.common.quest.type ;
/*-----------------------------------------------------------*/
/*   TypeListRec                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeListRec                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * Holds information about a specific type list and the actual
 * data returned from the TIGRIS.TYPEXXXXX table.
 *
 * @version 1.0, 3/5/1998 
 * @author  David Lentz
 */
import java.util.* ;
import java.sql.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeListRec                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeListRec extends Object implements java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector  typeList           = null ;
   private boolean valid              = false ;     // Determines if this list is still valid
   private int     typeListInd        = 0 ;         // The type list this object represents


   /*-----------------------------------------------------------*/
   /* constructor
    */
   /**
    *    Creates a blank TypeListRec
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public TypeListRec ( int typeListInd ) {
      this.typeListInd       = typeListInd ;
   }


   /*-----------------------------------------------------------*/
   /* setTypeList
    */
   /**
    *    This method will set what type list that
    *  this record will monitor
    *  
    *             
    *  @param typeList A Vector of TypeRec objects containing the TYPEXXX data
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setTypeList ( Vector typeList ) {
      this.typeList = typeList ;
      if ( typeList != null )
         valid = true ;
   }


   /*-----------------------------------------------------------*/
   /* invalidate
    */
   /**
    *    This method will make it so that this 
    *  type list is no longer valid.
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void invalidate() {
      valid = false ;
   }




   /*------------------------------------------------------------*/
   /* isValid
    */
   /**
    *    this method will return TRUE if the data
    *  contained in this list is valid
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isValid () {
      return( valid ) ;
   }


   /*-----------------------------------------------------------*/
   /* addToList
    */
   /**
    *    This method will add a TypeRec to our
    *  list of data
    *  
    *             
    *  @param typeRec The TypeRec to add
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void addToList ( TypeRec typeRec ) {
      if ( typeList == null )
         typeList = new Vector ( 50, 10 ) ;
      typeList.addElement ( typeRec ) ;

   }


   /*-----------------------------------------------------------*/
   /* removeFromList
    */
   /**
    *    This method will remove a TypeRec from
    *  the current list
    *  
    *             
    *  @param typeRec The TypeRec to remove
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void removeFromList( TypeRec typeRec ) {
      if ( typeList != null )
         typeList.removeElement ( typeRec ) ;
   }


   /*-----------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove all items from
    *  the current list
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void removeAll () {
      typeList = null ;
      valid = false ;
   }




   /*------------------------------------------------------------*/
   /* checkForUpdate
    */
   /**
    *    This method will basically do a select for
    *  count of this type table. If the count is greater
    *  than 0, it returns TRUE
    *
    *
    *  @return TRUE if data needs to be downloaded
    *//*---------------------------------------------------------*/
   public boolean checkForUpdate ( ) {
      boolean  rc      = true ;
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return the descriptive string
    *  of this type list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {

       switch ( typeListInd ) {
           case TypeList.TYPE_DOC :
               return( "Document" ) ;

           case TypeList.TYPE_STATUS :
               return( "Type Status" ) ;

           case TypeList.TYPE_AUTH :
               return( "Type Auth" ) ;

           case TypeList.TYPE_COUNTRYCODE :
               return( "Type Country Code" ) ;

           case TypeList.TYPE_DOCCLASS :
               return( "Type Document Class" ) ;

           case TypeList.TYPE_GEOGRAPHY :
               return( "Type Geography" ) ;

           case TypeList.TYPE_LANGUAGE :
               return( "Type Language" ) ;

           case TypeList.TYPE_TEAM :
               return( "Type Teams" ) ;

           case TypeList.TYPE_DOCCAT :
               return( "Type Document Category" ) ;

           case TypeList.TYPE_METRIC :
               return( "Type Metric" ) ;

           case TypeList.TYPE_PROPERTIES :
               return( "Type Properties" ) ;

           case TypeList.TYPE_PUB :
               return( "Type Publication" ) ;

           case TypeList.TYPE_WORKREQUIRED :
               return( "Type Work" ) ;

           case TypeList.TYPE_RESOLUTION :
               return( "Type Resolution" ) ;

       }
       return( "" ) ;
   }



   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will determine if the object passed
    *  in is equal to this object in any way.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean equals ( Object compareRec ) {
      boolean rc = false ;

      if ( compareRec instanceof TypeListRec )
         rc = ( ((TypeListRec)compareRec).typeListInd == typeListInd ) ;
      else if ( compareRec instanceof String ) {
         if ( ((String)compareRec).equals ( toString() ) ) {
            rc = true ;
         }
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getTypeListInd
    */
   /**
    *    This method will return the indicator that
    *  represents the type list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ind 
    *//*---------------------------------------------------------*/
   public int getTypeListInd () {
      return( typeListInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getTypeListVec
    */
   /**
    *    This method will return the vector representing
    *  the type list data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vec
    *//*---------------------------------------------------------*/
   public Vector getTypeListVec () {
      return( typeList ) ;
   }


} /* TypeListRec */
