package com.ibm.nzna.projects.common.quest.type ;
/*-----------------------------------------------------------*/
/*   TypeRec                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeRec                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.util.Text ;
import com.ibm.nzna.shared.pom.PersistentRec ;
import com.ibm.nzna.shared.gui.MultiListRow ;
/**
 * The superclass of all Type List records. This record holds
 * the integer indicator and a String description.
 *
 *
 * @version 1.0, 4/11/2001
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeRec                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeRec extends PersistentRec implements java.io.Serializable, MultiListRow {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /**
     * This is the indicator that is unique to this Type Record.
     **/
   private int    ind                      = 0 ;

   /**
     * This is the textual description of this category. This is usually the
     * brand name, family name, country name etc...
     **/
   private String description         = null ;

   /**
     * This is the description that is returned from the toString method. Normalyl
     * this is the same as the description, however in some cases it may be different
     **/
   private String toString = null ;


   /**
     * Default Constructor
     **/
   public TypeRec () {
   }

   /**
     *  This constructor creates a TypeRec with only an indicator
     **/
   public TypeRec ( int ind ) {
      this (ind, "" ) ;
   }

   /**
     * This method creates a default TypeRec with an Indicator and a
     * Description
     **/
   public TypeRec ( int ind, String descript ) {
      setInd ( ind ) ;
      setDescript ( descript ) ;
      updateRecStatus ( REC_NOCHANGE ) ;
   }

   /**
    * The toString method first tries to return the description. If the description is not available then a Blank string is returned. 
    */
   public String toString() {
      if ( toString != null ) 
         return ( toString ) ;
      if ( description != null ) 
         return( description ) ;
      return( "" ) ;
   }

   /**
    * This method will return the data needed to view this row in a MultiList or JTable. 
    * This method returns the following data based on the Column parameter passed:<br><br>
    * 
    * Column=0: A string holding the Indicator
    * Column=1: A string holding the description 
    */
   public Object getColumnData ( int column ) {
      if ( column == 0 )
         return( toString() ) ;
      else if ( column == 1 )
         return( new String ( ""+ind ) ) ;
      return( "" ) ;
   }

   /**
    * This method simply compares the indicator passed with this object's indicator.
    * It will return TRUE if equal, FALSE if not 
    */
   public boolean isInd ( int ind ) {
      return( this.ind == ind ) ;
   }



   /**
     * This method will return the Primary Key indicator for this type object
     **/
   public int getInd () {
      return( ind ) ;
   }





   /**
     * This method will compare any object to this TypeRec. If the object
     * passed is a TypeRec, then both the indicator and description are
     * compared for equality
     **/
   public boolean equals ( Object object ) {
      boolean rc = false ;

      if ( object instanceof TypeRec ) {
         TypeRec typeRec = (TypeRec) object ;

         try {
            if ( ( ind == typeRec.ind ) && ( description.equals ( description ) ) )
               rc = true ;
         } catch ( Exception e ) {
            // Do nothing. This is always a NullPointerException if there is no description
         }

      }

      return( rc ) ;
   }





   /**
     * This method will update the description of this
     * Type element. Max field limit is 50 characters
     **/
   public void setDescript ( String description ) {
      if ( description != null ) {
         this.description = description.trim() ;
         updateRecStatus ( REC_UPDATE ) ;
      }
   }


   /**
     *  This method will set the indicator for this type record. Please be
     * very careful using this method since it does in fact set the Primary Key
     *
     * @param ind The new indicator
     **/
   protected void setInd ( int ind ) {
      this.ind = ind ;
   }


   /**
     * This method will return the description of the Type entry
     *
     * @return String
     */
   public String getDescript () {
      return( description ) ;
   }


   /**
     * This method will clean all text fields for DB/2. This involves
     * changing single quotes to two single quotes etc..
     */
   public void cleanDBChars () {
      description = Text.cleanDBString ( description ) ;
   }


   /**
     * This method will set what is returned from the 'toString' method.
     * this is useful if you want a JList, JTree or MultiList to display
     * something different then what the TypeRec actually is.<br><br>
     *
     * Please note that this does not actually change the description or
     * change any database information. This simply changes what the 
     * return value for the toString call is.
     *
     * @param toString The actual string to return
     **/
   public void setToString ( String toString ) {
      this.toString = toString ;
   }


} /* TypeRec */