package com.ibm.nzna.projects.common.storedProc ;          
/*-----------------------------------------------------------*/
/*   CreateStoredProcs                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CreateStoredProcs                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This application creates all of the stored procedures for the IBM GAD Dept.
 *
 * @version 1.0, 7/11/1999 
 * @author  David Lentz
 */
import java.sql.* ;
import COM.ibm.db2.jdbc.app.*;  // DB2 UDB JDBC classes
import COM.ibm.db2.app.*;    // StoredProc and associated classes
import com.ibm.nzna.shared.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   CreateStoredProcs                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class CreateStoredProcs extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Stored Procedure array. This is where you add           */
   /*   a stored procedure to the list to be created            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String[][] storedProc = {
      { "QUEST.TypeListReader",                                                            // Stored Procedure Name
         "com.ibm.nzna.projects.common.storedProc.typeListReader.TypeListReader!readLists",  // Package.ClassName.Invoke Method
         "( in  languageInd int, "+                                                       // Param 1 The language to read
         "  in  typeListInd int, "+                                                       // Param 2 The Type List Indicator
         "  out typeListInitRec  blob(3M), "+                                             // Param 3 (Returned Information)
         "  out storedProcRetRec blob(2M) ) "},                                           // Param 4 (Returned Information)

      { "QUEST.sqlRunner",                                                                // Stored Procedure Name
         "com.ibm.nzna.projects.common.storedProc.sqlRunner.SqlRunner!runSQL",            // Package.ClassName.Invoke Method
         "( in  inSql            clob(3M),  "+                                        // Param 1 The sql buffer to run
         "  out storedProcRetRec blob(1M) ) "},                                      // Param 2 (Returned Information)

      { "QUEST.ExtSqlRunner",  
         "com.ibm.nzna.projects.common.storedProc.extSqlRunner.ExtSqlRunner!runSQL",
         "( in  inSql            clob(10M),      "+ 
         "  in  commitInterval   integer,        " +
         "  out storedProcRetRec blob(2M) )      "},

      { "QUEST.brandReader",                                                              // Stored Procedure Name
         "com.ibm.nzna.projects.common.storedProc.brandReader.BrandReader!readBrands",    // Package.ClassName.Invoke Method
         "( in  brandInd         int,           "+                                        // Param 1 The Brand Indicator to read, -1 for all
         "  in  inSql            blob(1M), "+                                        // Param 2 The sql buffer to run
         "  out storedProcRetRec blob(1M) ) "},                                      // Param 2 (Returned Information)

      { "QUEST.OAReader",
         "com.ibm.nzna.projects.common.storedProc.oaReader.OAReader!readOAObject",
         "( in  objectInd        integer,  " +                                            // Param 1:  database indicator of the object to be read
         "  in  objectType       integer,  " +                                            // Param 2:  the type of object to be read
         "  in  active           integer,  " +                                            // Param 3:  1 = read active object; 0 = read draft object
         "  in  languageInd      integer,  " +                                            // Param 4:  mode corresponding to OAMode values
         "  in  mode             integer,  " +                                            // Param 5:  languageInd for reading text items (0 = all languages)
         "  out returnedObject   blob(5M), " +                                            // Param 6:  returns the object that was read from the database
         "  out storedProcRetRec blob(1M) )"},                                            // Param 7:  returns status information

      { "QUEST.OAPublish",
         "com.ibm.nzna.projects.common.storedProc.oaPublish.OAPublish!publishOAObject",
         "( in  draftObjectInd   integer,  " +                                            // Param 1:  database indicator of the draft object to be published
         "  in  objectType       integer,  " +                                            // Param 2:  the type of object to be published
         "  out storedProcRetRec blob(10M) )"},                                           // Param 3:  returns status information

      { "QUEST.OAValidate",
         "com.ibm.nzna.projects.common.storedProc.oaValidate.OAValidate!validate",
         "( in  objectInd        integer,  " +                                              // Param 1:  database indicator of the draft object to be validated
         "  in  objectType       integer,  " +                                              // Param 2:  the type of object to be validated
         "  out validationRec    blob(10M) )"},                                              // Param 3:  returns validation results

      { "QUEST.NotifyReader",
         "com.ibm.nzna.projects.qit.notify.storedProc.notifyReader.NotifyReader!read",
         "( in  userId              varchar(254),  " +                                      // Param 1:  The user id to read notifications for
         "  out retNotifyReaderRec  blob(4M),  " +                                          // Param 2:  The object to populate and return
         "  out storedProcRetRec    blob(4M) )"},                                           // Param 3:  returns validation results

      { "QUEST.ReadUsers",
         "com.ibm.nzna.projects.qit.storedProc.readUsers.ReadUsers!readUsers",
         "( in  userListClob        clob(5M),     " +                                       // Param 1:  See StoredProcedure docs
         "  out retReadUsersRec     blob(8M),  " +                                          // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )"},                                           // Param 3:  returns validation results

      { "QUEST.ActDraftReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.actReader.ActDraftReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.ActReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.actReader.ActReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.SymDraftReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.symReader.SymDraftReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.SymReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.symReader.SymReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.QstDraftReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.qstReader.QstDraftReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.QstReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.qstReader.QstReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.AnsDraftReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.ansReader.AnsDraftReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.AnsReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.ansReader.AnsReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.LnkDraftReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.lnkReader.LnkDraftReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.LnkReader",
         "com.ibm.nzna.projects.qit.avalon.storedProc.lnkReader.LnkReader!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results


      { "QUEST.LnkGrpDraftRdr",
         "com.ibm.nzna.projects.qit.avalon.storedProc.lnkGrpRdr.LnkGrpDraftRdr!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.LnkGrpRdr",
         "com.ibm.nzna.projects.qit.avalon.storedProc.lnkGrpRdr.LnkGrpRdr!readList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.LoginUser",
         "com.ibm.nzna.projects.qit.storedProc.loginUser.LoginUser!loginUser",
         // "( in  userId           varchar(12),  " +                                             // Param 1:  The user id to log in
         //"  in  password         varchar(50),  " +                                             // Param 2:  The password of the user
         "(  out loginUserRecBlob blob(2000000),     " +                                             // Param 3:  The LogonUserRec to get information from
         "  out storedProcRetRec blob(1M) )"},                                                 // Param 4:  The return record

      { "QUEST.ProductListReader",
         "com.ibm.nzna.projects.qit.product.storedProc.productListReader.ProductListReader!readList",
         "( in  whereClause      varchar(254),  " +                                            // Param 1:  The where clause to issue
         "  out results          blob(1M),      " +                                            // Param 2:  The Buffer to place the Vector of returned Objects
         "  out sqlCode          integer,       " +                                            // Param 3:  The SQL Code to return
         "  out errorString      varchar(254) )"},                                             // Param 4:  The return error

      { "QUEST.DocListReader",
         "com.ibm.nzna.projects.qit.doc.storedProc.docListReader.DocListReader!readDocList",
         "( in  query               clob(1M), " +                                           // Param 1:  The query to run
         "  in  maxRecordReturn     integer,       "+
         "  in  startReadIndex      integer,       "+
         "  out actualRecordCount   integer,       "+
         "  out returnBlob          blob(3M),      " +                                      // Param 2:  The object to return
         "  out storedProcRetRec    blob(1M) )     "},                                      // Param 3:  returns validation results

      { "QUEST.DocReader",
         "com.ibm.nzna.projects.qit.doc.storedProc.docReader.DocReader!readDocument",
         "( in  docInd           int,           " +                                            // Param 1:  The Document Request Indicator
         "  in  existingDocInd   int,           " +                                            // Param 2:  The Active Document Indicator
         "  in  request          int,           " +                                            // Param 3:  1 = Request, 0 = Active
         "  in  readLevel        int,           " +                                            // Param 4:  Read Level, either Full or Preview
         "  out results          blob(3M),      " +                                            // Param 5:  The results from the Stored PRocedure
         "  out sqlCode          integer,       " +                                            // Param 6:  The SQL Code returned from the Stored Procedure
         "  out errorString      varchar(254))  "},                                             // Param 7:  The return error

      { "QUEST.DocumentReader",
         "com.ibm.nzna.projects.common.quest.doc.DocumentReader!readDocument",         
         "( in  docInd           int,           " +                                            // Param 1:  The Document Indicator
         "  out documentBlob     blob(3M),      " +                                            // Param 2:  The Document blob to return in
         "  out storedProcRetRec blob(1M))      "},                                             // Param 3:  The return error

      { "QUEST.DocDraftReader",
         "com.ibm.nzna.projects.common.quest.doc.DocumentDraftReader!readDocument",         
         "( in  docInd           int,           " +                                            // Param 1:  The Document Indicator
         "  out documentBlob     blob(3M),      " +                                            // Param 2:  The Document blob to return in
         "  out storedProcRetRec blob(1M))      "},                                            // Param 3:  The return error

      { "QUEST.ProDraftReader",
         "com.ibm.nzna.projects.common.quest.product.ProductDraftReader!readProduct",         
         "( in  productInt       int,           " +                                            // Param 1:  The Product Indicator
         "  out productBlob      blob(3M),      " +                                            // Param 2:  The Product blob to return in
         "  out storedProcRetRec blob(1M))      "},                                            // Param 3:  The return error

      { "QUEST.ProductReader", 
         "com.ibm.nzna.projects.common.quest.product.ProductReader!readProduct",         
         "( in  productInt       int,           " +                                            // Param 1:  The Product Indicator
         "  out productBlob      blob(3M),      " +                                            // Param 2:  The Product blob to return in
         "  out storedProcRetRec blob(1M))      "},                                            // Param 3:  The return error


      { "QUEST.TypeCatReader",
         "com.ibm.nzna.projects.common.quest.type.TypeCategoryReader!readCategories",         
         "( in  categoryLevel    varchar(254),  " +                                            // Param 1:  The category level
         "  in  parentCatInd     int,           " +                                            // Param 1:  The parent category indicator
         "  out documentBlob     blob(3M),      " +                                            // Param 2:  The blob for holding the Vetor of TypeCategoryRecs
         "  out storedProcRetRec blob(1M))      "},                                            // Param 3:  The return error

      { "QUEST.TestStoredProc",
         "com.ibm.nzna.projects.common.storedProc.testStoredProc.TestStoredProc!test",         
         "( in  inParam varchar(1024),          " +                                            // Param 1:  The blob for sending and receiving the UserRec to sign in
         "  out outParam varchar(1024))       "}                                             // Param 2:  The return error
   } ;





   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This method will kick off the application
    *  
    *  
    *
    *  @param Database Name
    *  @param user id 
    *  @param password
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String args[] ) {
      new LogSystem ( 1, "CreateStoredProcs.out" ) ;
      if ( ( args == null ) || ( args.length != 5 ) ) {
         System.out.println ( "Usage: java CreateStoredProcs <database> <server> <jdbc port number> <user id> <password>" ) ;
      } else {
         new CreateStoredProcs ( args[3], args[4], args[0], args[1], args[2] ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will run the create stored proc
    *  application
    *  
    *
    *  @param database name
    *  @param user id
    *  @param password
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CreateStoredProcs ( String userid, String password, String database, String server, String portNum ) {
      boolean rc = false ;

      try {
         Connection con = null ;
         String     url = "jdbc:db2//"+server+":"+portNum+":"+database ;

         /* Register the driver  */
         System.out.println ( "Registering DB/2 Net Driver" ) ;
         //Class.forName ("COM.ibm.db2.jdbc.app.DB2Driver").newInstance ();
         Class.forName ("COM.ibm.db2.jdbc.net.DB2Driver").newInstance ();

         // connect with user-provided username and password
         System.out.println ( "Connecting to "+database+" on "+server ) ;
         con = DriverManager.getConnection(url, userid, password);

         rc = createStoredProcedures(con) ;

      } catch ( SQLException ex ) {
         if ( ex.getErrorCode() == -1403 )
            System.out.println ( "The user id and password is not valid" ) ;
         else if ( ex.getErrorCode() == -1013 )
            System.out.println ( "The database name: "+database+" could not be connected to." ) ;
         else
            ex.printStackTrace() ;
      } catch ( Exception e ) {

         e.printStackTrace() ;
      }

      if ( !rc )
         System.out.println ( "Application aborted" ) ;
      else
         System.out.println ( "Application completed, all stored procedures created" ) ;
   }





   /*------------------------------------------------------------*/
   /* createStoredProcedures
    */
   /**
    *    This method will create all of our stored procedures
    *  that are listed in the storedProc array
    *  
    *
    *  @param con The Database Connection to use
    *
    *//*---------------------------------------------------------*/
   private boolean createStoredProcedures ( Connection con ) throws Exception {
      boolean   rc     = false ;
      int       i      = 0 ;
      int       length = storedProc.length ;

      try {
         Statement stmt   = con.createStatement() ;

         while ( i < length ) {
            System.out.print ( "Creating "+storedProc[i][0] ) ;

            try {
               // drop the stored procedure if it exists
               Statement dropStmt = con.createStatement ();
               dropStmt.executeUpdate ("DROP PROCEDURE " + storedProc[i][0]);
               dropStmt.close ();
            } catch ( SQLException e ) {
               //e.printStackTrace() ;
               // ignore this error
            }

            //stmt.executeUpdate ( "DROP PROCEDURE QUEST.TESTSTOREDPROC" ) ;
            //stmt.executeUpdate ( "CREATE PROCEDURE QUEST.TESTSTOREDPROC (OUT outParam char(18)) "+
            //                     "DYNAMIC RESULT SETS 0 "+
            //                     "LANGUAGE JAVA "+
            //                     "PARAMETER STYLE JAVA "+
            //                     "NO DBINFO "+
            //                     "FENCED "+
            //                     "MODIFIES SQL DATA "+
            //                     "PROGRAM TYPE SUB "+
            //                     "EXTERNAL NAME 'TestStoredProc.test'" ) ;

            stmt.executeUpdate ("CREATE PROCEDURE " + storedProc[i][0] + storedProc[i][2] +
                                " LANGUAGE JAVA " +
                                " PARAMETER STYLE DB2GENERAL FENCED "+
                                " EXTERNAL NAME '" + storedProc[i][1] + "'");
            LogSystem.log ( 1, "CREATE PROCEDURE " + storedProc[i][0] + storedProc[i][2] +
                            " LANGUAGE JAVA " +
                            " PARAMETER STYLE DB2GENERAL FENCED "+
                            " EXTERNAL NAME '" + storedProc[i][1] + "'" ) ;

            System.out.println ( "  Done." ) ;
            i++ ;
            //i=length+1 ;
         }
         rc = true ;

         stmt.close() ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( rc ) ;
   }



} /* CreateStoredProcs */

