package com.ibm.nzna.projects.common.storedProc ;
/*-----------------------------------------------------------*/
/*   StoredProcUtil                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StoredProcUtil                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
//import com.ibm.nzna.projects.qit.storedProc.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.Clob ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This class holds useful static methods for dealing with Java Stored Procedures 
 *
 *
 * @version 1.0, 2/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   StoredProcUtil                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class StoredProcUtil extends Object {




    /*------------------------------------------------------------*/
    /* setObjectInBlob
     */
    /**
     *    This method will copy a serialized object into a blob.
     *  
     *
     *  @param blob The Blob
     *  @param object The Object to serialize
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void setObjectInBlob ( Blob blob, Object object ) throws Exception {
        ObjectOutputStream s = new ObjectOutputStream ( blob.getOutputStream() ) ;

        s.writeObject ( object ) ;
        s.flush() ;
        s.close() ;
    }





    /*------------------------------------------------------------*/
    /* getObjectBytes
     */
    /**
     *    This method will return a byte array
     *  from an Object
     *  
     *
     *  @param object The Object to get bytes from
     *  @param None
     *  @param None
     *
     *  @return byte[]
     *//*---------------------------------------------------------*/
    public static byte[] getObjectBytes ( Object object ) throws Exception {
        byte[] retArray = null ;
        try {
            ByteArrayOutputStream byteOut = new ByteArrayOutputStream() ;
            ObjectOutputStream    out     = new ObjectOutputStream ( byteOut ) ;
            byte[]                array   = null ;

            out.writeObject ( object ) ;

            array = byteOut.toByteArray() ;
            retArray = new byte[array.length+10] ;
            System.arraycopy ( array, 0, retArray, 0, array.length ) ;
            retArray[retArray.length-1] = (byte)'\0' ;

            int i=0 ;
            System.out.println ( "Returing:\n" ) ;
            while ( i < retArray.length ) {
                System.out.print ( retArray[i] ) ;
                i++ ;
            }
            System.out.println ( ":end" ) ;
        } catch ( Exception e ) {
            e.printStackTrace() ;
        }
        return( retArray ) ;
    }





    /*------------------------------------------------------------*/
    /* getObjectFromStatement
     */
    /**
     *    This method will form an Object from a CallablleStatement
     *  containing a Blob and returns the object.
     *  
     *
     *  @param stmt The CallableStatement to ask for the Blob from
     *  @param paramNum The parameter in the CallableStatement that holds the Blob
     *  @param None
     *
     *  @return Object
     *//*---------------------------------------------------------*/
    public static Object getObjectFromStatement ( CallableStatement stmt, int paramNum ) throws Exception {
        ByteArrayInputStream f1     = new ByteArrayInputStream ( stmt.getBytes(paramNum) ) ;
        ObjectInputStream    s1     = new ObjectInputStream ( f1 ) ;
        Object               retObj = null ;

        retObj = s1.readObject() ; 
        s1.close() ;
        f1.close() ;

        return( retObj ) ;
    }





    /*------------------------------------------------------------*/
    /* getObjectFromBlob
     */
    /**
     *    This method will deserialize an Object
     *  from a Blob
     *  
     * THIS METHOD DOES NOT WORK!!!! SO DON'T EXPECT IT TO!
     *
     *  @param blob The Blob to take the object from
     *  @param None
     *  @param None
     *
     *  @return Object
     *//*---------------------------------------------------------*/
    public static Object getObjectFromBlob ( Blob blob ) throws Exception {
        Object retObject = null ;

        if ( ( blob != null ) && ( blob.getInputStream() != null ) ) {
            ByteArrayInputStream byteArrayIn  = null ;
            InputStream          blobIn       = blob.getInputStream() ;
            ObjectInputStream    objectIn     = null ;
            byte[]               inBytes      = new byte [ (int)blob.size() ] ;
            int                  i            = 0 ;


            blobIn.read ( inBytes ) ;

            byteArrayIn = new ByteArrayInputStream ( inBytes ) ;
            objectIn    = new ObjectInputStream    ( byteArrayIn ) ;

            retObject = objectIn.readObject() ; 
            objectIn.close() ;
            throw ( new Exception ( "Read in "+inBytes.length+" bytes. First two:"+inBytes[0]+" "+inBytes[1] ) ) ;
        }

        return( retObject ) ;
    }




    /*------------------------------------------------------------*/
    /* writeException
     */
    /**
     *    This method will write a Throwable object (Exception 
     *  or Error) entirely to the error field of a StoredProcRec.
     *  
     *
     *  @param t The Throwable object to write
     *  @param storedProcRec The record to write to
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void writeException ( Throwable t, StoredProcRec storedProcRec ) {
        StringWriter    sw         = new StringWriter () ;
        PrintWriter     pw         = new PrintWriter ( sw ) ;
        StringTokenizer strtok     = null ;
        String          stackTrace = "" ;

        /* Get the stackTrace */
        t.printStackTrace ( pw ) ;
        stackTrace = sw.toString() ;

        if ( stackTrace == null )
            stackTrace = "No Stack Available" ;

        storedProcRec.errorStr = stackTrace ;
        storedProcRec.error    = true ;

        if ( t instanceof SQLException ) {
            storedProcRec.sqlCode = ((SQLException)t).getErrorCode() ;
        }
    }



    /*------------------------------------------------------------*/
    /* getObjectFromResultSet
     */
    /**
     *    This method will form an Object from a ResultSet
     *  containing a Blob and returns the object.
     *  
     *
     *  @param rs The ResultSet to ask for the Blob from
     *  @param paramNum The parameter in the CallableStatement that holds the Blob
     *  @param None
     *
     *  @return Object
     *//*---------------------------------------------------------*/
    public static Object getObjectFromResultSet ( ResultSet rs, int paramNum ) throws Exception {
        ByteArrayInputStream f1     = new ByteArrayInputStream ( rs.getBytes(paramNum) ) ;
        ObjectInputStream    s1     = new ObjectInputStream ( f1 ) ;
        Object               retObj = null ;


        retObj = s1.readObject() ; 
        s1.close() ;
        f1.close() ;

        return( retObj ) ;
    }



    /*------------------------------------------------------------*/
    /* getStringFromClob
     */
    /**
     *    This method will attempt to pull a character
     *  String from the Clob passed in
     *  
     *
     *  @param lob The Clob to pull from
     *  @param None
     *  @param None
     *
     *  @return The String pulled.
     *//*---------------------------------------------------------*/
    public static String getStringFromClob ( Clob lob ) throws Exception {
       Reader        reader    = lob.getReader() ;
       int           max       = 256 ;
       char[]        clobData  = new char[max];           
       int           read      = 0 ;
       StringBuffer  retStr    = new StringBuffer() ; 

       /*-----------------------------------------------------------*/
       /*                                                           */
       /*   Copy over the bytes                                     */
       /*                                                           */
       /*-----------------------------------------------------------*/
       while ( ( read > -1  ) && ( reader != null ) ) {
          read = reader.read ( clobData ) ;
          if ( read > -1 )
             retStr.append ( new String ( clobData ).substring(0,read) ) ;
       }

       return( retStr.toString() ) ;
    }

} /* StoredProcUtil */
