package com.ibm.nzna.projects.common.storedProc ;          
/*-----------------------------------------------------------*/
/*   TestStoredProcs                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TestStoredProcs                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This application will test the following stored procedures by calling  
 * each one in order:<br><br>
 *
 * DocListReader
 * ExtSqlRunner
 * 
 *
 * @version 1.0, 7/11/1999 
 * @author  David Lentz
 */
import java.util.* ;
import java.sql.* ;
import COM.ibm.db2.jdbc.app.*;  // DB2 UDB JDBC classes
import COM.ibm.db2.app.*;    // StoredProc and associated classes
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   TestStoredProcs                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class TestStoredProcs extends Object {



   Connection con = null ;



   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This method will kick off the application
    *  
    *  
    *
    *  @param Database Name
    *  @param user id 
    *  @param password
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String args[] ) {
      new LogSystem ( 1, "TestStoredProcs.out" ) ;
      if ( ( args == null ) || ( args.length != 4 ) ) {
         System.out.println ( "Usage: java TestStoredProcs <database> <server> <user id> <password>" ) ;
      } else {
         new TestStoredProcs ( args[2], args[3], args[0], args[1] ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will run the create stored proc
    *  application
    *  
    *
    *  @param database name
    *  @param user id
    *  @param password
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TestStoredProcs ( String userid, String password, String database, String server ) {
      boolean rc = false ;

      try {
         String     url = "jdbc:db2//"+server+":5703:"+database ;

         /* Register the driver  */
         System.out.println ( "Registering DB/2 Net Driver" ) ;
         //Class.forName ("COM.ibm.db2.jdbc.app.DB2Driver").newInstance ();
         Class.forName ("COM.ibm.db2.jdbc.net.DB2Driver").newInstance ();

         // connect with user-provided username and password
         System.out.println ( "Connecting to "+database+" on "+server ) ;
         con = DriverManager.getConnection(url, userid, password);

         // Test
         testStoredProc () ;

      } catch ( SQLException ex ) {
         if ( ex.getErrorCode() == -1403 )
            System.out.println ( "The user id and password is not valid" ) ;
         else if ( ex.getErrorCode() == -1013 )
            System.out.println ( "The database name: "+database+" could not be connected to." ) ;
         else
            ex.printStackTrace() ;
      } catch ( Exception e ) {

         e.printStackTrace() ;
      }

   }


   /**
     * Tests
     **/
   private void testStoredProc () {
      try {
         CallableStatement stmt  = createStoredProc ( "QUEST.TestStoredProc", 1 ) ;                                    


         if ( stmt != null ) {
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have a query and can continueProcessing           */
            /* then call the stored procedure                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            int     count        = 0 ;
            int     actualCount  = 0 ;
            String  query        = null ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up the call                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            System.out.println ( "Setting up Call" ) ;
            stmt.registerOutParameter ( 1, Types.VARCHAR ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            LogSystem.log ( 1, "Executing Doc List Query:"+query ) ;
            System.out.println ( "Calling DocListReader Stored Procedure" ) ;
            stmt.execute() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            System.out.println ( "Returned:"+stmt.getString ( 1 )  ) ;


         } /* stmt  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         e.printStackTrace() ;
      }

   }



   /*------------------------------------------------------------*/
   /* createStoredProc
    */
   /**
    *    This method will create a stored
    *  procedure and returns the CallableStatement
    *  object for it
    *
    *  @param name The name of the Stored Procedure
    *  @param params The number of parameters in/out passed
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CallableStatement createStoredProc ( String name, int params ) {
      CallableStatement retStmt = null ;
      String            strStmt = constructDB2StoredProcCall ( name, params ) ;

      try {
         System.out.print ( "Preparing Call for "+name+" ..." ) ;
         retStmt = con.prepareCall ( strStmt ); 
         System.out.println ( "Ok" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         System.out.println ( "Bad. Check Logs" ) ;
      }


      return( retStmt ) ;
   }

   /*------------------------------------------------------------*/
   /* constructDB2StoredProcCall
    */
   /**
    *    This method will create the actual
    *  string that is passed to DB/2 for a 
    *  stored procedure
    *
    *  @param name The name of the stored proc.
    *  @param params The number of params
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String constructDB2StoredProcCall ( String name, int params ) {
      String retStr = "CALL "+name+" ( ";
      int    i      = 0 ;

      for ( i = 0; i < params; i++ ) {
         retStr = retStr+"?" ;
         if ( i != (params-1) )
            retStr = retStr+", " ;
      }

      LogSystem.log ( 1, "SQLMethod Creating Stored Procedure Call:"+retStr+")" ) ;

      return( retStr+")" ) ;

   }

} /* TestStoredProcs */

