package com.ibm.nzna.projects.common.storedProc.brandReader ;   
/*-----------------------------------------------------------*/
/*   BrandReader                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BrandReader                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read all Brand/Group combinations from the TIGRIS.BRANDS table
 * It will then return an Array of BrandGroupRecs.
 * 
 * Main Execution Point: logonUser
 *
 * Param Number   Descript
 * -------------- -------------------------------------------
 * 1              Brand Ind to read with. -1 for all. Currently not supported, defaults to reading ALL                      
 * 2              BrandReaderRec (Passed back with populated values)
 * 3              StoredProcRec ( Allocated and returned by BrandReader )
 *
 * 
 *
 * @version 1.0, 2/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BrandReader                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class BrandReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec  returnRec     = new StoredProcRec () ;
   private Connection     con           = null ;
   private Vector         brandGroupVec = new Vector ( 1, 1 ) ;





   /*------------------------------------------------------------*/
   /* readLists
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readBrands ( int brandInd, Blob retVec, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;                   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read all of the brand groups                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         readBrandGroups() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our vector to return                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retVec = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( retVec, brandGroupVec );
         set ( 2, retVec ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 3, storedProcRetRec ) ;
   }





   /*------------------------------------------------------------*/
   /* readBrandGroups
    */
   /**
    *    This method will read all entries from the
    *  TIGRIS.BRANDS table
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readBrandGroups () throws Exception {
      Statement     stmt    = con.createStatement () ;
      BrandGroupRec readRec = null ;
      ResultSet     rs      = stmt.executeQuery ( "SELECT BRANDGROUPIND, BRANDIND, GROUPIND "+
                                                  "FROM TIGRIS.BRANDS "+
                                                  "ORDER BY BRANDGROUPIND FOR FETCH ONLY " ) ;

      int i = 0;
      while ( ( i < 1000 ) && ( rs.next() ) )  {
         readRec = new BrandGroupRec ( rs.getInt(2), rs.getInt(3), rs.getInt(1) ) ;
         brandGroupVec.addElement ( readRec ) ;
         i++ ;
      }

      rs.close() ;
      stmt.close() ;
   }
} /* BrandReader */
