
package com.ibm.nzna.projects.common.storedProc.oaValidate;

import com.ibm.nzna.projects.common.storedProc.*;
import java.util.*;

/**
 * Object to be returned by OAValidate stored procedure with results
 * of validation for publishing.
 * The publishable field indicates whether the object may be published.
 * 
 * @version 1.4
 * @author  Mark DeBiase
**/
public class ValidationRec extends StoredProcRec {

  /**
   * Will be set to true if the object can be published or set to false
   * if the object cannot be published due to errors.
  **/
  public boolean publishable = false;

  /**
   * Vector of UsageRec objects which indicate where (by which objects) 
   * the object to be published is being used.
   * If the publishable flag is set to true, then the objects in this Vector
   * are for information only.  If the publishable flag is set to false, then
   * some of the UsageRec objects <i>may</i> have their isError flag set to
   * true.
   * <p>
   * This Vector can be populated when validating any type of object --
   * Action, Question, Symptom, Link or LinkGroup.
   * 
   * @see UsageRec
  **/
  public Vector usageVec = new Vector();

  /**
   * Vector of FailureRec objects which indicate validation failures.
   * This Vector will only be populated for Question and Symptom validation.
   * 
   * @see FailureRec
  **/
  public Vector failureVec = new Vector();


  /**
   * Vector of Integer objects representing indicators of open draft
   * Actions that are co-requisites to publishing the object that
   * is being validated.
   * The co-requiste Actions must be published at the same time as
   * the object being validated is published.
   * <p>
   * The actionCoreqVec will never be null but it may be empty
   * if there are no co-requisites.
  **/
  public Vector actionCoreqVec    = new Vector(1,5);

  /**
   * Vector of String objects representing the the default titles of the
   * Actions in the actionCoreqVec.  There is a one-to-one correspondence
   * between the actionCoreqVec and the actionCoreqTitles Vectors:
   * actionCoreqTitle.elementAt(i) holds the title for the actionInd in
   * actionCoreqVec.elementAt(i)
  **/
  public Vector actionCoreqTitles = null;

  /**
   * Vector of Integer objects representing indicators of open draft
   * Questions that are co-requisites to publishing the object that
   * is being validated.
   * The co-requiste Questions must be published at the same time as
   * the object being validated is published.
   * <p>
   * The questionCoreqVec will never be null but it may be empty
   * if there are no co-requisites.
  **/
  public Vector questionCoreqVec    = new Vector(1,5);

  /**
   * Vector of String objects representing the the default titles of the
   * Questions in the questionCoreqVec.  There is a one-to-one correspondence
   * between the questionCoreqVec and the questionCoreqTitles Vectors:
   * questionCoreqTitle.elementAt(i) holds the title for the questionInd in
   * questionCoreqVec.elementAt(i)
  **/
  public Vector questionCoreqTitles = null;

  /**
   * Vector of Integer objects representing indicators of open draft
   * Symptoms that are co-requisites to publishing the object that
   * is being validated.
   * The co-requiste Symptoms must be published at the same time as
   * the object being validated is published.
   * <p>
   * The symptomCoreqVec will never be null but it may be empty
   * if there are no co-requisites.
  **/
  public Vector symptomCoreqVec    = new Vector(1,5);

  /**
   * Vector of String objects representing the the default titles of the
   * Symptoms in the symptomCoreqVec.  There is a one-to-one correspondence
   * between the symptomCoreqVec and the symptomCoreqTitles Vectors:
   * symptomCoreqTitle.elementAt(i) holds the title for the questionInd in
   * symptomCoreqVec.elementAt(i)
  **/
  public Vector symptomCoreqTitles = null;
}

