package com.ibm.nzna.projects.common.storedProc.sqlRunner ; 
/*-----------------------------------------------------------*/
/*   SqlRunner                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SqlRunner                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This Stored Procedure will pick apart the CLOB sent to it and
 * will run whatever SQL is inside of the CLOB. The CLOB must adhere
 * to this format:
 *
 * SQL STATEMENT<END_DELIM>SQL STATEMENT<END_DELIM>
 *
 * For example:
 * DELETE FROM TIGRIS.DOCBODY WHERE DOCIND = 123<END>INSERT INTO TIGRIS.DOCBODY<END>
 *
 * It will return the SQL Code and any messages in the StoredProcRetRec
 *
 * The END_DELIM must be sqlRunner.END_DELIM ;
 *
 * @version 1.0, 9/15/1999 
 * @author  David Lentz
 */
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import java.sql.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Clob ;
import COM.ibm.db2.app.Blob ;
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   SqlRunner                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class SqlRunner extends StoredProc {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String END_DELIM           = new Character ( (char)3).toString() ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Program Vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int           sqlCode          = 0 ;          // The SQLCODE we will return
   private Connection    con              = null ;       // Our connection to DB/2
   private String        strClob          = null ;       // The string holding the clob
   private String        errorStr         = null ;       // The string used to hold errors.
   private StoredProcRec returnRec        = new StoredProcRec() ;


   /*------------------------------------------------------------*/
   /* runSQL    
    */
   /**
    *    This method will be called by DB/2 when
    *  we are called by a client. It will process reading
    *  all of the document information
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*--------------------------------------------------------*/
   public void runSQL ( Clob inSql, Blob storedProcRetRec ) throws Exception  {                                                       
      String     sqlToRun[] = null ;            
      Connection con        = getConnection() ;
      con.setAutoCommit(false);      // MAD 4/27
      Statement  stmt       = con.createStatement() ;
      String     lastSQL    = "" ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we can get the string out, continue                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getStringFromClob ( inSql ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Pull out the SQL to run                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sqlToRun = parseSql() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have SQL to run, loop through and run it          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( sqlToRun != null ) && ( sqlToRun.length > 0 ) ) {
               int i    = 0 ;

               while ( i < sqlToRun.length ) {
                  lastSQL = sqlToRun[i] ;
                  stmt.executeUpdate ( sqlToRun[i] ) ;
                  i++ ;
               }
            }
         } else
            this.sqlCode = -1 ;                              

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Close our statement                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.close() ;

      } catch ( SQLException sqlEx ) {
         StoredProcUtil.writeException ( sqlEx, returnRec ) ;
         returnRec.errorStr+= "\nLast SQL Run:"+lastSQL ;
         returnRec.sqlCode  = sqlEx.getErrorCode()  ;
         con.rollback() ;
      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
         returnRec.errorStr+= "\nLast SQL Run:"+lastSQL ;
         returnRec.sqlCode  = sqlCode ;
         con.rollback() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   CLose up shop                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 2, storedProcRetRec ) ;

   }





   /*------------------------------------------------------------*/
   /* getStringFromClob
    */
   /**
    *    This method will attempt to pull a character
    *  String from the Clob passed in
    *  
    *
    *  @param lob The Clob to pull from
    *  @param None
    *  @param None
    *
    *  @return TRUE = Pull was good, FALSE = Pull was bad
    *//*---------------------------------------------------------*/
   private boolean getStringFromClob ( Clob lob ) throws Exception {
      Reader        reader    = lob.getReader() ;
      int           max       = 256 ;
      char[]        clobData  = new char[max];           
      int           read      = 0 ;
      StringBuffer  retStr    = new StringBuffer() ; 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Copy over the bytes                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( read > -1  ) && ( reader != null ) ) {
         read = reader.read ( clobData ) ;
         if ( read > -1 )
            retStr.append ( new String ( clobData ).substring(0,read) ) ;
      }

      if ( retStr.length() > 0 ) {
         strClob = retStr.toString() ;
         return( true ) ;
      }

      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* parseSql
    */
   /**
    *    This method will parse through the SQL Passed
    *  in and will pull out the valid statements
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return An array of strings. Each string is a statement
    *//*---------------------------------------------------------*/
   private String[] parseSql () {
      StringTokenizer strtok = new StringTokenizer(strClob, END_DELIM) ;
      String[]        array  = null ;
      int             i      = 0 ;

      if ( strtok.countTokens() > 0 ) {
         array = new String[strtok.countTokens()] ;

         while ( strtok.hasMoreTokens() ) {
            array[i] = strtok.nextToken() ;

            /* Pull out the delimeter */
            array[i] = replaceInStr ( array[i], END_DELIM, "" ) ;
            i++ ;
         }
      }

      return( array ) ;
   }

   /*------------------------------------------------------------*/
   /* replaceInStr
    */
   /**
    *    This method will replace a string inside of a
    *  string with another string. It will only replace the
    *  first instance of the match.
    *  
    *
    *  @param string0 The full string
    *  @param string1 The string that needs to be replaced
    *  @param string2 The string that replaces string1
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static String replaceInStr ( String string0, String string1, String string2 ) {
      int strIndex = string0.indexOf ( string1 ) ;

      if ( strIndex > -1 )
         return( string0.substring ( 0, strIndex )+string2+string0.substring ( strIndex+string1.length() ) ) ;
      else
         return( string0 ) ;

   } /* replaceInStr */



   /**
     *  This method may be called outside of a stored procedure from a Java Client. It
     * will simply call the SqlRunner stored procedure with the SQL passed and returns
     * the SQL Code for that stored procedure run.
     *
     * @param databaseId The Database Id that the DatabaseSystem is connected to
     * @param sqlRunnerString The string of SQL to run
     **/
   public static int sqlRunnerCode ( int databaseId, String sqlRunnerString ) {
      SQLMethod sql     = new SQLMethod ( databaseId, "sqlRunner", 5 ) ;
      int       sqlCode = 0 ;

      try {
         CallableStatement stmt        = sql.createStoredProc ( "QUEST.sqlRunner", 2 ) ;
         String            debugString = Text.replaceAllStrInStr ( sqlRunnerString, END_DELIM, "\n" ) ;

         LogSystem.log ( 1, "Starting to send statements to EXT SQLRUNNER" ) ;
         LogSystem.log ( 1, debugString ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the Callable Stmt                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.registerOutParameter ( 1, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.setBytes ( 1, sqlRunnerString.getBytes() ) ;      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute the SQL                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log( 2, "Running SQLRunner:" ) ;
         LogSystem.log( 2, Text.replaceAllStrInStr ( sqlRunnerString, END_DELIM, "\n    " ) ) ;
         stmt.execute() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;

         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            sqlCode = retRec.sqlCode ;
         }

         stmt.close() ;

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         sqlCode=-1 ;
      }

      sql.close() ;

      return( sqlCode ) ;
   }


   /**
     *  This method may be called outside of a stored procedure from a Java Client. It
     * will simply execute 1 SQL Statement. The SQLStatement must NOT end with a END_DELIM.
     *
     * @param databaseId The Database Id that the DatabaseSystem is connected to
     * @param sqlRunnerString The string of SQL to run
     **/
   public static int runSingleSQL ( int databaseId, String sqlRunnerString ) {
      SQLMethod sql     = new SQLMethod ( databaseId, "runSingleSQL", 5 ) ;
      int       sqlCode = 0 ;

      try {
         Statement stmt = sql.createStatement() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute the SQL                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log( 2, "Running Single SQL:" ) ;
         LogSystem.log( 2, sqlRunnerString ) ;
         stmt.executeUpdate ( sqlRunnerString ) ;

         stmt.close() ;

      } catch ( SQLException e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         sqlCode=e.getErrorCode() ;
      }

      sql.close() ;

      return( sqlCode ) ;
   }

} /* SqlRunner */
