package com.ibm.nzna.projects.common.storedProc.typeListReader ;   
/*-----------------------------------------------------------*/
/*   TypeListReader                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeListReader                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Blob ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read all start up type lists that the application
 * needs to read in
 * 
 * Main Execution Point: logonUser
 *
 * Param Number   Descript
 * -------------- -------------------------------------------
 * 1              Language Indicator is the language to read
 * 2              Type List Indicator as defined in TypeList.java. A -1 means to read all start up lists
 * 3              TypeListReaderRec (Passed back with populated values)
 * 4              StoredProcRec ( Allocated and returned by TypeListReader )
 *
 * 
 *
 * @version 1.0, 2/23/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeListReader                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeListReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec      returnRec           = new StoredProcRec () ;
   private TypeListReaderRec  typeListReaderRec   = null ;
   private Connection         con                 = null ;





   /*------------------------------------------------------------*/
   /* readLists
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readLists ( int languageInd, int typeListInd, Blob retTypeListReaderRec, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our Type List Reader Record                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         typeListReaderRec = new TypeListReaderRec () ;                                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read our Type Lists                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If they pass a -1 as a Type List Indicator, read the    */
         /*  startup lists.                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( typeListInd == - 1 )
            readStartupLists ( languageInd ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, just read the single type list               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            switch ( typeListInd ) {
               

               case TypeList.TYPE_LANGUAGE :
                  typeListReaderRec.retVec = readTypeLanguage ( ) ;
                  break ;

               case TypeList.TYPE_AUTH :
                  typeListReaderRec.retVec = readTypeAuth ( languageInd ) ;
                  break ;

               case TypeList.TYPE_DOC :
                  typeListReaderRec.retVec = readTypeDocument ( languageInd ) ;
                  break ;

               case TypeList.TYPE_COUNTRYCODE :
                  typeListReaderRec.retVec = readTypeCountryCode ( ) ;
                  break ;

               case TypeList.TYPE_DOCCLASS :
                  typeListReaderRec.retVec = readTypeDocClass ( languageInd ) ;
                  break ;

               case TypeList.TYPE_DOCCAT :
                  typeListReaderRec.retVec = readTypeDocCat ( languageInd ) ;
                  break ;

               case TypeList.TYPE_METRIC :
                  typeListReaderRec.retVec = readTypeMetric ( languageInd ) ;
                  break ;

               case TypeList.TYPE_PROPERTIES :
                  typeListReaderRec.retVec = readTypeProperty ( languageInd ) ;
                  break ;

               case TypeList.TYPE_PUB :
                  typeListReaderRec.retVec = readTypePub ( languageInd ) ;
                  break ;

               case TypeList.TYPE_WORKREQUIRED :
                  typeListReaderRec.retVec = readTypeWorkRequired ( languageInd ) ;
                  break ;

               case TypeList.TYPE_STATUS :
                  typeListReaderRec.retVec = readTypeStatus ( languageInd ) ;
                  break ;

               case TypeList.TYPE_RESOLUTION :
                  typeListReaderRec.retVec = readTypeResolution ( languageInd ) ;
                  break ;

               case TypeList.TYPE_TEAM :
                  typeListReaderRec.retVec = readTypeTeam ( languageInd ) ;
                  break ;

               case TypeList.TYPE_ACTION :
                  typeListReaderRec.retVec = readTypeAction ( languageInd ) ;
                  break ;

               case TypeList.TYPE_QUESTION :
                  typeListReaderRec.retVec = readTypeQuestion ( languageInd ) ;
                  break ;

               case TypeList.TYPE_SYMPTOM :
                  typeListReaderRec.retVec = readTypeSymptom ( languageInd ) ;
                  break ;

               case TypeList.TYPE_GEOGRAPHY :
                  typeListReaderRec.retVec = readTypeGeography ( languageInd ) ;
                  break ;

               case TypeList.TYPE_CUSTOMVIEW :
                  typeListReaderRec.retVec = readTypeCustomView ( languageInd ) ;
                  break ;

               case TypeList.TYPE_DATE :
                  typeListReaderRec.retVec = readTypeDate ( languageInd ) ;
                  break ;

               case TypeList.TYPE_CUSTOMER :
                  typeListReaderRec.retVec = readTypeCustomer ( languageInd ) ;
                  break ;

               case TypeList.TYPE_WARRANTY :
                  typeListReaderRec.retVec = readTypeWarranty ( languageInd ) ;
                  break ;

               case TypeList.TYPE_OFFERING :
                  typeListReaderRec.retVec = readTypeOffering ( languageInd ) ;
                  break ;

               case TypeList.TYPE_CANCEL :
                  typeListReaderRec.retVec = readTypeCancel ( languageInd ) ;
                  break ;

               case TypeList.TYPE_PRODUCT :
                  typeListReaderRec.retVec = readTypeProduct ( languageInd ) ;
                  break ;


            } /* switch */
         } /* else */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our TypeListReaderRec                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retTypeListReaderRec = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( retTypeListReaderRec, typeListReaderRec );
         set ( 3, retTypeListReaderRec ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 4, storedProcRetRec ) ;
   }





   /*------------------------------------------------------------*/
   /* readStartupLists
    */
   /**
    *    This method will read in all of our startup
    *  lists
    *  
    *
    *  @param languageInd The language to read them in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readStartupLists ( int languageInd ) throws Exception  {

      typeListReaderRec.geographyVec   = readTypeGeography    ( languageInd ) ;
      typeListReaderRec.countryCodeVec = readTypeCountryCode  ( ) ;
      typeListReaderRec.docTypeVec     = readTypeDocument     ( languageInd ) ;
      typeListReaderRec.docClassVec    = readTypeDocClass     ( languageInd ) ;
      typeListReaderRec.customViewVec  = readTypeCustomView   ( languageInd ) ;
      typeListReaderRec.workVec        = readTypeWorkRequired ( languageInd ) ;
      typeListReaderRec.authVec        = readTypeAuth         ( languageInd ) ;
      typeListReaderRec.metricVec      = readTypeMetric       ( languageInd ) ;
      typeListReaderRec.languageVec    = readTypeLanguage     ( ) ;
   }


   /*------------------------------------------------------------*/
   /* readTypeLanguage
    */
   /**
    *  Reads all information from the TIGRIS.TYPELANGUAGE table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeLanguage ( ) throws Exception {
      Vector          retVec   = new Vector ( 1, 1 ) ;
      Statement       stmt     = con.createStatement () ;
      String          sql      = "SELECT LANGUAGEIND, DESCRIPT "+
                                 "FROM  TIGRIS.TYPELANGUAGE "+
                                 "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs       = stmt.executeQuery ( sql ) ;
      TypeLanguageRec langRec  = null ;


      while ( rs.next() ) {
         langRec = new TypeLanguageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( langRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readTypeAuth
    */
   /**
    *  Reads all information from the TIGRIS.TYPEAUTH table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeAuth ( int languageInd ) throws Exception {
      Vector          retVec   = new Vector ( 1, 1 ) ;
      Statement       stmt     = con.createStatement () ;
      String          sql      = "SELECT AUTHIND, DESCRIPT "+
                                 "FROM  TIGRIS.TYPEAUTH "+
                                 "WHERE LANGUAGEIND = "+languageInd+" "+
                                 "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs       = stmt.executeQuery ( sql ) ;
      TypeAuthRec authRec  = null ;


      while ( rs.next() ) {
         authRec = new TypeAuthRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( authRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readTypeDocument
    */
   /**
    *  Reads all information from the TIGRIS.TYPEDOCUMENT table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeDocument ( int languageInd ) throws Exception {
      Vector       retVec   = new Vector ( 1, 1 ) ;
      Statement    stmt     = con.createStatement () ;
      String       sql      = "SELECT DOCTYPEIND, DESCRIPT "+
                              "FROM  TIGRIS.TYPEDOCUMENT "+
                              "WHERE LANGUAGEIND = "+languageInd+" "+
                              "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet    rs       = stmt.executeQuery ( sql ) ;
      TypeDocRec   docRec = null ;


      while ( rs.next() ) {
         docRec = new TypeDocRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( docRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readTypeCountryCode
    */
   /**
    *  Reads all information from the TIGRIS.TYPECOUNTRYCODE table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeCountryCode ( ) throws Exception {
      Vector             retVec     = new Vector ( 1, 1 ) ;
      Statement          stmt       = con.createStatement () ;
      String             sql        = "SELECT A.COUNTRYCODEIND, A.PHONECOUNTRYCODE, A.DESCRIPT,     A.ZIPMASK, "+
                                      "       A.PHONEMASK,      A.ZIPDIGITS,        A.TIMEDIFF,     A.MULTIZONE, "+
                                      "       A.USESDST,        A.CURRENCY,         A.CURRENCYMASK, A.LANG01,    "+
                                      "       A.LANG02,         A.LANG03,           B.GEOIND,       A.ISOCODE "+
                                      "FROM  TIGRIS.TYPECOUNTRYCODE A, "+
                                      "      TIGRIS.GEOCOUNTRY B "+
                                      "WHERE A.COUNTRYCODEIND = B.COUNTRYCODEIND AND "+
                                      "      A.DESCRIPT <> 'All' "+
                                      "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet          rs         = stmt.executeQuery ( sql ) ;
      TypeCountryCodeRec countryRec = null ;

      while ( rs.next() ) {
         countryRec = new TypeCountryCodeRec ( rs.getInt(1),                                  // Country Code Ind
                                               rs.getString(2),                               // Phone Country Code
                                               rs.getString(3).trim(),                        // Description
                                               rs.getString(4),                               // Zip Mask
                                               rs.getString(5),                               // Phone Mask
                                               (short)rs.getInt(6),                           // Zip Digits
                                               rs.getString(7),                               // Time Difference
                                               rs.getString(8),                               // Multi Zone
                                               rs.getString(9),                               // Uses DST
                                               rs.getString(10),                              // Currency
                                               rs.getString(11),                              // Currency Mask
                                               (short)rs.getInt(12),                          // Language 1
                                               (short)rs.getInt(13),                          // Language 2
                                               (short)rs.getInt(14),                          // Language 3
                                               (short)rs.getInt(15),                          // Geography Indicator
                                               rs.getString(16) ) ;                           // ISO Country Code

         retVec.addElement ( countryRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readTypeDocClass
    */
   /**
    *  Reads all information from the TIGRIS.TYPEDOCCLASS table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeDocClass ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT DOCCLASSIND, DESCRIPT "+
                                    "FROM  TIGRIS.TYPEDOCCLASS "+
                                    "WHERE LANGUAGEIND = "+languageInd+" "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypeDocClassRec docClassRec = null ;


      while ( rs.next() ) {
         docClassRec = new TypeDocClassRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( docClassRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }




   /*------------------------------------------------------------*/
   /* readTypeDocCat
    */
   /**
    *  Reads all information from the TIGRIS.TYPEDOCCLASS table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeDocCat ( int languageInd ) throws Exception {
      Vector        retVec    = new Vector ( 1, 1 ) ;
      Statement     stmt      = con.createStatement () ;
      String        sql       = "SELECT DOCCATIND, DESCRIPT "+
                                "FROM  TIGRIS.TYPEDOCCAT "+
                                "WHERE LANGUAGEIND = "+languageInd+" "+
                                "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet     rs        = stmt.executeQuery ( sql ) ;
      TypeDocCatRec docCatRec = null ;


      while ( rs.next() ) {
         docCatRec = new TypeDocCatRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( docCatRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* readTypeTeam
    */
   /**
    *  Reads all information from the TIGRIS.USERGROUP table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeTeam ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT USERGROUP, DESCRIPT, LONGDESCRIPT "+
                                    "FROM  TIGRIS.USERGROUP "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypeTeamRec teamRec = null ;


      while ( rs.next() ) {
         teamRec = new TypeTeamRec ( rs.getInt(1), rs.getString(2).trim(), rs.getString(3).trim() ) ;
         retVec.addElement ( teamRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readTypeMetric
    */
   /**
    *  Reads all information from the TIGRIS.TYPEMETRIC table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeMetric ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT METRICIND, DESCRIPT "+
                                    "FROM  TIGRIS.TYPEMETRIC "+
                                    "WHERE LANGUAGEIND = "+languageInd+" "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypeMetricRec metricRec = null ;


      while ( rs.next() ) {
         metricRec = new TypeMetricRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( metricRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* readTypeProperty
    */
   /**
    *  Reads all information from the TIGRIS.TYPEPROPERTIES table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeProperty ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT PROPERTYIND, DESCRIPT "+
                                    "FROM  TIGRIS.TYPEPROPERTIES "+
                                    "WHERE LANGUAGEIND = "+languageInd+" "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypePropertyRec propertyRec = null ;


      while ( rs.next() ) {
         propertyRec = new TypePropertyRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( propertyRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* readTypePub
    */
   /**
    *  Reads all information from the TIGRIS.TYPEPUBLICATION table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypePub ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT PUBTYPE, DESCRIPT "+
                                    "FROM  TIGRIS.TYPEPUBLICATION "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypePubRec pubRec = null ;


      while ( rs.next() ) {
         pubRec = new TypePubRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( pubRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* readTypeWorkRequired
    */
   /**
    *  Reads all information from the TIGRIS.TYPEWORKREQUIRED table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeWorkRequired ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT WORKIND, DESCRIPT "+
                                    "FROM  TIGRIS.TYPEWORKREQUIRED "+
                                    "WHERE LANGUAGEIND = "+languageInd+" "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypeWorkRec     workRec = null ;


      while ( rs.next() ) {
         workRec = new TypeWorkRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( workRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readTypeStatus
    */
   /**
    *  Reads all information from the TIGRIS.TYPESTATUS table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeStatus ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT STATUSIND, STATUSTYPE, DESCRIPT "+
                                    "FROM  TIGRIS.TYPESTATUS "+
                                    "WHERE LANGUAGEIND = "+languageInd+" "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypeStatusRec   statusRec = null ;


      while ( rs.next() ) {
         statusRec = new TypeStatusRec ( rs.getInt(1), rs.getInt(2), rs.getString(3).trim() ) ;
         retVec.addElement ( statusRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }




   /*------------------------------------------------------------*/
   /* readTypeResolution
    */
   /**
    *  Reads all information from the QUEST.TYPERESOLUTION table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeResolution ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT RESOLIND, DESCRIPT "+
                                    "FROM  QUEST.TYPERESOLUTION "+
                                    "WHERE LANGUAGEIND = "+languageInd+" "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypeResolutionRec   resRec = null ;


      while ( rs.next() ) {
         resRec = new TypeResolutionRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( resRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* readTypeGeography
    */
   /**
    *  Reads all information from the QUEST.TYPERESOLUTION table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeGeography ( int languageInd ) throws Exception {
      Vector     retVec = new Vector ( 1, 1 ) ;
      Statement  stmt   = con.createStatement () ;
      String     sql    = "SELECT GEOIND, DESCRIPT "+
                          "FROM  TIGRIS.TYPEGEOGRAPHY "+
                          "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet  rs     = stmt.executeQuery ( sql ) ;
      TypeGeoRec geoRec = null ;


      while ( rs.next() ) {
         geoRec = new TypeGeoRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( geoRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readTypeAction
    */
   /**
    *  Reads all information from the QUEST.TYPEACTION table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeAction ( int languageInd ) throws Exception {
      Vector        retVec    = new Vector ( 1, 1 ) ;
      Statement     stmt      = con.createStatement () ;
      String        sql       = "SELECT TYPEACTIONIND, DESCRIPT "+
                                "FROM  OA.TYPEACTION "+
                                "WHERE LANGUAGEIND = "+languageInd+" "+
                                "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet     rs        = stmt.executeQuery ( sql ) ;
      TypeActionRec actionRec = null ;


      while ( rs.next() ) {
         actionRec = new TypeActionRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( actionRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readTypeQuestion
    */
   /**
    *  Reads all information from the QUEST.TYPEQUESTION table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeQuestion ( int languageInd ) throws Exception {
      Vector          retVec      = new Vector ( 1, 1 ) ;
      Statement       stmt        = con.createStatement () ;
      String          sql         = "SELECT TYPEQUESTIONIND, DESCRIPT "+
                                    "FROM  OA.TYPEQUESTION "+
                                    "WHERE LANGUAGEIND = "+languageInd+" "+
                                    "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet       rs          = stmt.executeQuery ( sql ) ;
      TypeQuestionRec questionRec = null ;


      while ( rs.next() ) {
         questionRec = new TypeQuestionRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( questionRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readTypeSymptom
    */
   /**
    *  Reads all information from the QUEST.TYPESYMPTOM table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeSymptom ( int languageInd ) throws Exception {
      Vector         retVec     = new Vector ( 1, 1 ) ;
      Statement      stmt       = con.createStatement () ;
      String         sql        = "SELECT TYPESYMPTOMIND, DESCRIPT "+
                                  "FROM  OA.TYPESYMPTOM "+
                                  "WHERE LANGUAGEIND = "+languageInd+" "+
                                  "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet      rs         = stmt.executeQuery ( sql ) ;
      TypeSymptomRec symptomRec = null ;


      while ( rs.next() ) {
         symptomRec = new TypeSymptomRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( symptomRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readTypeCustomView
    */
   /**
    *  Reads all information from the QIT.TYPECUSTOMVIEW table   
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeCustomView ( int languageInd ) throws Exception {
      Vector            retVec        = new Vector ( 1, 1 ) ;
      Statement         stmt          = con.createStatement () ;
      String            sql           = "SELECT TYPECUSTOMVIEWIND, DESCRIPT "+
                                        "FROM  QIT.TYPECUSTOMVIEW "+
                                        "WHERE LANGUAGEIND = "+languageInd+" "+
                                        "ORDER BY DESCRIPT FOR FETCH ONLY" ;
      ResultSet         rs            = stmt.executeQuery ( sql ) ;
      TypeCustomViewRec customViewRec = null ;


      while ( rs.next() ) {
         customViewRec = new TypeCustomViewRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( customViewRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readTypeDate
    */
   /**
    *  Reads all information from the PRODUCT.TYPEDATE 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeDate ( int languageInd ) throws Exception {
      Vector            retVec        = new Vector ( 1, 1 ) ;
      Statement         stmt          = con.createStatement () ;
      String            sql           = "SELECT TYPEDATEIND, DESCRIPTION "+
                                        "FROM  PRODUCT.TYPEDATE "+
                                        "ORDER BY DESCRIPTION FOR FETCH ONLY" ;
      ResultSet         rs            = stmt.executeQuery ( sql ) ;
      TypeDateRec       dateRec       = null ;


      while ( rs.next() ) {
         dateRec = new TypeDateRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( dateRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }

   /*------------------------------------------------------------*/
   /* readTypeCustomer
    */
   /**
    *  Reads all information from the PRODUCT.TYPEDATE 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeCustomer ( int languageInd ) throws Exception {
      Vector            retVec        = new Vector ( 1, 1 ) ;
      Statement         stmt          = con.createStatement () ;
      String            sql           = "SELECT TYPECUSTOMERIND, DESCRIPTION "+
                                        "FROM  PRODUCT.TYPECUSTOMER "+
                                        "ORDER BY DESCRIPTION FOR FETCH ONLY" ;
      ResultSet         rs            = stmt.executeQuery ( sql ) ;
      TypeCustomerRec       dateRec       = null ;


      while ( rs.next() ) {
         dateRec = new TypeCustomerRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( dateRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }

   /*------------------------------------------------------------*/
   /* readTypeCancel
    */
   /**
    *  Reads all information from the PRODUCT.TYPEDATE 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeCancel ( int languageInd ) throws Exception {
      Vector            retVec        = new Vector ( 1, 1 ) ;
      Statement         stmt          = con.createStatement () ;
      String            sql           = "SELECT TYPECANCELIND, DESCRIPTION "+
                                        "FROM  PRODRAFT.TYPECANCEL "+
                                        "ORDER BY DESCRIPTION FOR FETCH ONLY" ;
      ResultSet         rs            = stmt.executeQuery ( sql ) ;
      TypeCancelRec       dateRec       = null ;


      while ( rs.next() ) {
         dateRec = new TypeCancelRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( dateRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readTypeOffering
    */
   /**
    *  Reads all information from the PRODUCT.TYPEDATE 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeOffering ( int languageInd ) throws Exception {
      Vector            retVec        = new Vector ( 1, 1 ) ;
      Statement         stmt          = con.createStatement () ;
      String            sql           = "SELECT TYPEOFFERINGIND, DESCRIPTION "+
                                        "FROM  PRODUCT.TYPEOFFERING "+
                                        "ORDER BY DESCRIPTION FOR FETCH ONLY" ;
      ResultSet         rs            = stmt.executeQuery ( sql ) ;
      TypeOfferingRec       dateRec       = null ;


      while ( rs.next() ) {
         dateRec = new TypeOfferingRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( dateRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }

   /*------------------------------------------------------------*/
   /* readTypeProduct
    */
   /**
    *  Reads all information from the PRODUCT.TYPEDATE 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeProduct ( int languageInd ) throws Exception {
      Vector            retVec        = new Vector ( 1, 1 ) ;
      Statement         stmt          = con.createStatement () ;
      String            sql           = "SELECT TYPEPRODUCTIND, DESCRIPTION "+
                                        "FROM  PRODUCT.TYPEPRODUCT "+
                                        "ORDER BY DESCRIPTION FOR FETCH ONLY" ;
      ResultSet         rs            = stmt.executeQuery ( sql ) ;
      TypeProductRec       dateRec       = null ;


      while ( rs.next() ) {
         dateRec = new TypeProductRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         retVec.addElement ( dateRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readTypeWarranty
    */
   /**
    *  Reads all information from the PRODUCT.TYPEDATE 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readTypeWarranty ( int languageInd ) throws Exception {
      Vector            retVec        = new Vector ( 1, 1 ) ;
      Statement         stmt          = con.createStatement () ;
      String            sql           = "SELECT TYPEWARRANTYIND, DESCRIPTION, LONGDESCRIPT "+
                                        "FROM  PRODUCT.TYPEWARRANTY "+
                                        "ORDER BY DESCRIPTION FOR FETCH ONLY" ;
      ResultSet         rs            = stmt.executeQuery ( sql ) ;
      TypeWarrantyRec   warrantyRec   = null ;


      while ( rs.next() ) {
         warrantyRec = new TypeWarrantyRec ( rs.getInt(1), rs.getString(2).trim() ) ;
         if ( rs.getString(3) != null )
            warrantyRec.setLongDescription ( rs.getString(3).trim() ) ;
         else
            warrantyRec.setLongDescription ( null ) ;
         retVec.addElement ( warrantyRec ) ;
      }

      rs.close() ;
      stmt.close() ;
      return( retVec ) ;
   }


} /* TypeListReader */
