package com.ibm.nzna.projects.firewalker.invoker;

import com.ibm.mq.*;
import java.lang.*;
import java.io.*;
import java.util.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.projects.firewalker.base.* ;
import com.ibm.nzna.shared.mq.FireWalkerMQI ;

/**************************************************************************************
* This program reads strings from a queue and prints on the standard output. The program 
* terminates after fetching the string "end".
**************************************************************************************/
public class InvokerApp {
    public static Properties properties = new Properties() ;
    public static UrlAcl     urlAcl     = new UrlAcl() ;
    public static boolean    debug      = false ;
    public static String     version    = "v1.00a" ;
    public static void main(String args[])
    {
        FireWalkerMQI fwMQI = null;
        CGIData cgiData = new CGIData();
        String MsgId;
        String Msg;

        /* Load the property list for this application */
        try {
            System.out.println("InvokerApp("+InvokerApp.version+"):  starting.") ;
            System.out.println("InvokerApp:  loading InvokerApp.properties") ;
            FileInputStream propertyFile = new FileInputStream ("InvokerApp.properties") ;
            properties.load ( propertyFile ) ;
        } catch ( Exception e ) {
            e.printStackTrace() ;
            return ;
        }

        /* Load the Access Control List for this application */
        try {
            System.out.println("InvokerApp:  loading InvokerApp.acl") ;
            FileInputStream propertyFile = new FileInputStream ("InvokerApp.acl") ;
            urlAcl.load ( propertyFile ) ;
        } catch ( Exception e ) {
            e.printStackTrace() ;
            return ;
        }


        String debugProperty = properties.getProperty( "debug", "false" ) ;
        if (debugProperty.equalsIgnoreCase("true")) {
            InvokerApp.debug = true ;            
        }


        try {

            /*************************************************************************
            * Create an instance of FireWalkerMQI which makes a MQI Connection to the
            * MQServer. MsgId parameter specifies an ID for the posted message and the
            * filter ID to get messages using this connection.
            *************************************************************************/
            fwMQI = new FireWalkerMQI(  MQC.MQOO_INPUT_AS_Q_DEF | MQC.MQOO_INQUIRE ,
                                        properties.getProperty("QUEUEMANAGER") ,
                                        properties.getProperty("INPUTQUEUE") ,
                                        "",
                                        properties.getProperty("SERVER") ,
                                        properties.getProperty("CLIENTCHANNEL") ,
                                        Integer.parseInt( properties.getProperty("PORT")) ,
                                        MQC.MQWI_UNLIMITED,
                                        Integer.parseInt( properties.getProperty("MQEXPIRE")),
                                        "",
                                        "");


            /* Open the connection to the queue manager */
            fwMQI.open() ;
            System.out.println("InvokerApp:  initialized");

            int sessionId = 0 ;

            do {
                /* Get the object waiting in the queue */
                Object[] mqObjectArray = fwMQI.getObject();
                /* Gets the object placed in the message by the PostHandlerServlet */
                cgiData =(CGIData) mqObjectArray[0] ;
                /* Get the id which was set by the PostHandlerServlet */
                cgiData.setResponseId( (String) mqObjectArray[1]  ) ;

                System.out.println("==================================================");
                System.out.println("InvokerApp("+InvokerApp.version+", "+sessionId+"):  begin:  "+ new Date()) ;
                System.out.println("InvokerApp:  MQObject received:  "+cgiData.getResponseId() ) ;
                /* Check to see if this url is authorized before proceeding */
                if ( urlAcl.isAuthorized( cgiData.getRequestURL())) {
                    System.out.println("InvokerApp:  URL Authorized.") ;
                    String[][] headers = cgiData.getRequestHeaders();
                    /* Start the Invoker Thread.  This will process the url request */
                    /* and return the result to the MQSeries return queue */
                    new InvokerThread(cgiData, sessionId).start();
                } else {
                    System.out.println("InvokerApp:  URL Not Authorized:  "+ cgiData.getRequestURL()) ;
                    String content = StandardError.get( InvokerApp.properties.getProperty("ERRORTEMPLATE") ,
                                                        "eSupport",
                                                        "This service is currently unavailable at this moment.",
                                                        "Information:  ( FWI, 200 )");
                    try {
                        HTTPData httpData = new HTTPData() ;
                        httpData.setHTMLOutput( content );
                        FireWalkerMQI fwOutMQI = new FireWalkerMQI(   MQC.MQOO_OUTPUT |
                                                                      MQC.MQOO_SET_IDENTITY_CONTEXT | MQC.MQOO_INQUIRE,
                                                                      InvokerApp.properties.getProperty("QUEUEMANAGER") ,
                                                                      InvokerApp.properties.getProperty("OUTPUTQUEUE") ,
                                                                      "",
                                                                      InvokerApp.properties.getProperty("SERVER") ,
                                                                      InvokerApp.properties.getProperty("CLIENTCHANNEL") ,
                                                                      Integer.parseInt( InvokerApp.properties.getProperty("PORT")) ,
                                                                      MQC.MQWI_UNLIMITED,
                                                                      Integer.parseInt( InvokerApp.properties.getProperty("MQEXPIRE")),
                                                                      "",
                                                                      "");

                        fwOutMQI.open() ;
                        fwOutMQI.putObject(httpData, cgiData.getResponseId());
                        System.out.println("InvokerApp:  Placed Object in queue, "+cgiData.getResponseId()) ;
                    }

                    catch (MQException ex) {
                        System.out.println("InvokerThread:  MQError (cc,rc): "+ex.completionCode +", "+ex.reasonCode);
                    } catch (Exception e ) {
                        e.printStackTrace() ;
                    }
                }

                sessionId++ ;
            } 
            while (true);
        } catch ( MQException ex ) {
            System.out.println("An MQSeries error occured : completion code "+
                               ex.completionCode + " Reason Code " + ex.reasonCode);

        } catch ( IOException ex) {
            ex.printStackTrace() ;
        } catch ( ClassNotFoundException ex ) {
           ex.printStackTrace() ;
        } finally {
            try {
                /****************************************************************
                * close() releases the MQI connection to the MQServer.
                ****************************************************************/
                fwMQI.close();
                System.out.println("InvokerApp:  Ended") ;
            } catch ( MQException ex) {
               ex.printStackTrace() ;
            } catch ( java.io.IOException ex) {
               ex.printStackTrace() ;
            }
        }
    }
}

