package com.ibm.nzna.projects.firewalker.invoker;
import java.net.*;
import java.io.*;
import java.util.* ;
import com.ibm.mq.*;
import HTTPClient.* ;
import com.ibm.nzna.shared.net.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.projects.firewalker.base.* ;
import com.ibm.nzna.shared.mq.FireWalkerMQI ;

/*
* A server thread that can create another server thread once it
* has accepted a connection.
*/
class InvokerThread extends Thread {
    static final   Object   LOCK           = new Object();
    static int              activeCount;
    int                     sessionId      = 0 ;

    private CGIData cgiData = null;

    public InvokerThread(CGIData cgiData, int sessionId)
    {
        this.cgiData = cgiData;
        this.sessionId = sessionId ;
    }

    public static int getActiveCount()
    {
        synchronized (LOCK)
        {
            return activeCount;
        }
    }

    public void run()
    {
        System.out.println("InvokerThread("+sessionId+"):  begin") ;
        long                 time           = System.currentTimeMillis() ;
        FireWalkerMQI        fwMQI          = null;
        HTTPData             httpData       = new HTTPData();
        HTTPConnection       httpConnection = null ;
        StringBuffer         content        = new StringBuffer() ;
        String               inputLine      = null ;
        try {
            synchronized (LOCK)
            {
                ++activeCount;
                LOCK.notifyAll();
            }
            if (InvokerApp.debug) {
                System.out.println("InvokerThread:  Connecting to : "+cgiData.getRequestURL()) ;
            } else {
                String requestUrl = cgiData.getRequestURL() ;
                // If we are not in debug mode.  Show only part of the URL.
                int    length     = requestUrl.length() ;
                int    end        = requestUrl.lastIndexOf ("." ) ;
                if ( end != -1) {
                    length = end ;
                }
                System.out.println("InvokerThread:  Connecting to : "+cgiData.getRequestURL().substring(0,length)) ;
            }

            URL urlContent    = new URL(cgiData.getRequestURL());
            httpConnection    = new HTTPConnection ( urlContent ) ;
            //HTTPSocket httpSocket = new HTTPSocket( urlContent ) ;
            NVPair form_data[]= null ;

            try {
                /* get the timeout value for waiting for a response */
                int httpWait = Integer.parseInt( InvokerApp.properties.getProperty("HTTPWAIT")) ;

                /* set the timeout value.  httpWait is in seconds and must be converted to milli */
                httpConnection.setDefaultTimeout( httpWait * 1000 ) ;

                //httpSocket.setTimeout( httpWait * 1000 ) ;

                String postData[][] = cgiData.getPostData() ;

                /* Process post data */
                if ((postData != null ) && ( postData.length > 0)) {
                    form_data = new NVPair[ postData.length ] ;
                    for ( int i=0; i < postData.length ; i++ ) {
                        form_data[i] = new NVPair( postData[i][0], postData[i][1] ) ;
                        if (InvokerApp.debug) {
                            System.out.println( "InvokerThread: formdata:  "+ form_data[i] ) ; 
                        }
                    }
                }

                /* set headers */
                NVPair headers[] = new NVPair[1] ;
                headers[0] = new NVPair( "Accept" , "*/*" ) ;
                //headers[1] = new NVPair( "Accept
                httpConnection.setDefaultHeaders( headers ) ;

                HTTPResponse httpResponse = null ;
                /* Perform the conection and post the data */
                if ( postData != null ) {
                    httpResponse = httpConnection.Post( urlContent.getFile(), form_data ) ;
                    //content = httpSocket.post ( postData ) ;
                }
                /* Perform a get request */
                else {
                    httpResponse = httpConnection.Get( urlContent.getFile()) ;
                    //content = httpSocket.get() ;
                }


                if (InvokerApp.debug) {
                    System.out.println("InvokerThread:  Connecting to for Input: "+cgiData.getRequestURL()) ;
                }

                // Open an input stream to read the contents from the url connection
                InputStream       inputStream       = httpResponse.getInputStream() ;
                InputStreamReader inputStreamReader = new InputStreamReader( inputStream ) ;
                BufferedReader    bufferedReader    = new BufferedReader( inputStreamReader ) ;

                // Read the contents of the input stream into a string
                int linesRead = 0 ;
                while ( (inputLine = bufferedReader.readLine()) != null ) {
                    content.append(inputLine) ;
                    content.append("\n") ;
                    linesRead++ ;
                }

                bufferedReader.close() ;
            } catch (IOException e ) {
                e.printStackTrace() ;
                System.out.println("InvokerThread:  timed out waiting for response!" ) ;
                content.append(StandardError.get( InvokerApp.properties.getProperty("ERRORTEMPLATE") ,
                                                  "eSupport",
                                                  "This service is currently unavailable at this moment.",
                                                  "Information:  ( FWI, 100 )") );
            }



            httpData.setHTMLOutput( content.toString() );

            try {
                fwMQI = new FireWalkerMQI(  MQC.MQOO_OUTPUT |
                                            MQC.MQOO_SET_IDENTITY_CONTEXT | MQC.MQOO_INQUIRE,
                                            InvokerApp.properties.getProperty("QUEUEMANAGER") ,
                                            InvokerApp.properties.getProperty("OUTPUTQUEUE") ,
                                            "",
                                            InvokerApp.properties.getProperty("SERVER") ,
                                            InvokerApp.properties.getProperty("CLIENTCHANNEL") ,
                                            Integer.parseInt( InvokerApp.properties.getProperty("PORT")) ,
                                            MQC.MQWI_UNLIMITED,
                                            Integer.parseInt( InvokerApp.properties.getProperty("MQEXPIRE")),
                                            "",
                                            "");
                fwMQI.open() ;

                fwMQI.putObject(httpData, cgiData.getResponseId());
                System.out.println("InvokerThread:  Placed Object in queue, "+cgiData.getResponseId()) ;
            }

            catch (MQException ex) {
                System.out.println("InvokerThread:  MQError (cc,rc): "+ex.completionCode +", "+ex.reasonCode);
            } catch (Exception ex) {
                System.out.println("InvokerThread:  Error: "+ex.getMessage()+","+ex.toString());
            } finally {
                try {
                    fwMQI.close();
                } catch (Exception ex) {
                }
            }

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            synchronized (LOCK)
            {
                --activeCount;
                LOCK.notifyAll();
            }
        }
        System.out.println("InvokerThread("+sessionId+"):  ended : "+((float)( System.currentTimeMillis() - time ) / 1000)+ " seconds") ;   
    }
}

