package com.ibm.nzna.projects.firewalker.servlet;

import   java.io.*;
import   java.util.*;

import   com.ibm.mq.*;
import   javax.servlet.*;
import   javax.servlet.http.*;
import   javax.servlet.http.HttpUtils.*;
import   HTTPClient.* ;
import   com.ibm.nzna.shared.servlet.* ;
import   com.ibm.nzna.projects.firewalker.base.* ;
import   com.ibm.nzna.shared.mq.FireWalkerMQI ;

public class PostHandlerServlet extends HttpServlet
{
   // Default value for limit of GET & POST information that we'll accept
   // so as to protect ourselves from a DOS attack.
   protected      int                  postContentLimit  = 100*1024;   // 100KB limit.
   protected      int                  getContentLimit   = 1*1024;      // 1KB limit.
   protected      Properties           properties        = new Properties() ;
   protected      int                  sessionId         = 0 ;
   public         static final String  version           = "v1.00a" ;
   public         static boolean       debug             = false ;
   
   // Override getServletInfo().
   public String getServletInfo()
   {
      return("POST request tester : ariva");
   }
   
   public void init ( ServletConfig config) throws ServletException {
      super.init( config ) ;
      try {
         String serverRoot       = System.getProperty("server.root") ;
         String propertyFilePath = serverRoot + "servlets/com/ibm/nzna/project/firewalker/servlet/FireWalkerServlet.properties" ;
         FileInputStream propertyFile = new FileInputStream ( propertyFilePath ) ;
         properties.load ( propertyFile ) ;
         String debug = properties.getProperty( "debug", "false" ) ;
         if (debug.equalsIgnoreCase("true")) {
            this.debug = true ;            
         }
      }
      catch ( Exception e ) {
         e.printStackTrace() ;
         return ;
      }

   }
   
   protected void service (HttpServletRequest request, HttpServletResponse response)
   throws ServletException, IOException
   {  
      int                 sessionId             = this.sessionId++ ;
      long                time                  = System.currentTimeMillis() ;
      CGIData             cgiData               = new CGIData();
      HTTPData            httpData              = new HTTPData();
      /* These should be moved to be part of the class instance */
      /* This would allow more efficient messaging with the new */
      /* FireWalkerMQI connections do not need to be created    */
      /* for each message delivery.                             */
      FireWalkerMQI       fwMQIOut              = null;
      FireWalkerMQI       fwMQIIn               = null;

      System.out.println("PostHandlerServlet("+PostHandlerServlet.version+", "+sessionId+"):  service, begin:  "+ new Date()) ;

      // Check to limit Denial of Service attack.
      // Limit POST information to a reasonable size.
      if (postContentLimit < request.getContentLength())
      {
         // Set the response type.
         response.setContentType ("text/plain");
         ServletOutputStream output = response.getOutputStream();
         output.println(response+" Exceeded POST data length limit. Go away punk!");
         return;
      }

      /* The extra path info contains the URL which will be invoked by */
      /* The invoker application.                                      */
      String cgiURL = "http:/" + request.getPathInfo();
      if(request.getQueryString()!=null)
         cgiURL = cgiURL + "?" + request.getQueryString();

      cgiData.setRequestURL(cgiURL);

      String headerName = null;
      int numHeaderLines = 0;
      Enumeration headers = request.getHeaderNames();

      while(headers.hasMoreElements()) //=========== count number of headers
      {
         numHeaderLines++ ;
         headers.nextElement();
      }

      String[][] requestHeaders = new String[numHeaderLines][2];

      headers = request.getHeaderNames();
      for(int i=0; i<numHeaderLines; i++) //=========== form array of headers.
      {
         headerName = (String) headers.nextElement();
         requestHeaders[i][0] = headerName;
         requestHeaders[i][1] = request.getHeader(headerName);
      }
      cgiData.setRequestHeaders(requestHeaders);
      requestHeaders = null;
      cgiData.setRequestMethod(request.getMethod());

      if (0 < request.getContentLength()) //========== Deal with the POST contents.
      {
         // Get the parameters to do the POST
         Enumeration e = request.getParameterNames();
         int e_len = 0 ;
         for ( e_len=0 ; e.hasMoreElements() ; e_len++ ) 
            e.nextElement() ;

         String form_data[][] = new String[e_len][2] ;
         e = request.getParameterNames() ;

         for ( int i=0; e.hasMoreElements() ; i++ )  {
            String name = (String)e.nextElement();
            form_data[i][0] =  name ;
            form_data[i][1] =  request.getParameter(name) ;
            if (debug) {
               System.out.println( "PostHandlerServlet: formdata:  "+ form_data[i] ) ; 
            }
         }

         cgiData.setPostData( form_data );
      }



      try
      {
         /* If we do not have a connection for reading from the queue */
         /* we will create one here */
         if ( fwMQIOut == null ) {
            fwMQIOut = new FireWalkerMQI( MQC.MQOO_OUTPUT |
                        MQC.MQOO_SET_IDENTITY_CONTEXT,
                        properties.getProperty("QUEUEMANAGER") ,
                        properties.getProperty("OUTPUTQUEUE") ,
                        "", 
                        properties.getProperty("SERVER") ,
                        properties.getProperty("CLIENTCHANNEL") ,
                        Integer.parseInt( properties.getProperty("PORT")) ,
                        Integer.parseInt( properties.getProperty("MQWAIT")) ,
                        Integer.parseInt( properties.getProperty("MQEXPIRE")),
                        "",
                        "");
            /* Make the connection to the queue manager */
            fwMQIOut.open() ;
         }

         String key = cgiData.toString()+"."+System.currentTimeMillis() ;
         //cgiData.setResponseId( key );

         //byte msgId[] = fwMQIOut.putObject(cgiData, "Request");
         key = fwMQIOut.putObject(cgiData);

         System.out.println ( "PostHandlerServlet:  Placed Object in queue") ;
            
         /* If we do not have a connection for writing to the queue */
         /* we will create one here */
         if ( fwMQIIn == null ) {
            fwMQIIn = new FireWalkerMQI(  MQC.MQOO_INPUT_AS_Q_DEF | MQC.MQOO_INQUIRE ,
                        properties.getProperty("QUEUEMANAGER") ,
                        properties.getProperty("INPUTQUEUE") ,
                        "",
                        properties.getProperty("SERVER") ,
                        properties.getProperty("CLIENTCHANNEL") ,
                        Integer.parseInt( properties.getProperty("PORT")) ,
                        Integer.parseInt( properties.getProperty("MQWAIT")) ,
                        Integer.parseInt( properties.getProperty("MQEXPIRE")),
                        "",
                        "");
            /* Make the connection to the queue manager */
            fwMQIIn.open() ;
         }

         System.out.println( "PostHandlerServlet:  Waiting for object return with key: "+ key) ;
         httpData = (HTTPData) fwMQIIn.getObject(key);
         System.out.println( "PostHandlerServlet:  MQObject received") ;

         DataOutputStream output = new DataOutputStream(response.getOutputStream());
         if (debug) {
            System.out.println("PostHandlerServlet:  output:  "+httpData.getHTMLOutput()) ;
         }
         output.writeBytes(httpData.getHTMLOutput());
         output.flush();
         output.close();

         /* close the connections to the queue managers */
         fwMQIOut.close();
         fwMQIIn.close(); 


      }
      catch(MQException ex)
      {
         /* Handle exception generated by a timeout */
         if ( ex.reasonCode == 2033) {
            System.out.println( "PostHandlerServlet:  wait on MQSeries queue timed out!") ;
            DataOutputStream output = new DataOutputStream(response.getOutputStream());
            /* Construct an error message to the user */
            String errorMsg = StandardError.get( properties.getProperty("ERRORTEMPLATE") ,
                                                  "eSupport",
                                                  "This service is currently unavailable at this moment.",
                                                  "Information:  ( FWS, 100 )") ;
            output.writeBytes( errorMsg );
            output.flush();
            output.close();
         }
         else {
            System.out.println( "PostHandlerServlet:  MQSeries error!") ;
            DataOutputStream output = new DataOutputStream(response.getOutputStream());
            /* Construct an error message to the user */
            String errorMsg = StandardError.get( properties.getProperty("ERRORTEMPLATE") ,
                                                  "eSupport",
                                                  "This service is currently unavailable at this moment.",
                                                  "Information:  ( FWS, 101 )") ;
            output.writeBytes( errorMsg );
            output.flush();
            output.close();
         }
         /* An error has occurred close the connection */
         try {
            fwMQIOut.close();
            fwMQIIn.close(); 
         }
         catch(Exception e ) {
            e.printStackTrace() ;
         }
         fwMQIOut = null ;
         fwMQIIn  = null ;
         ex.printStackTrace() ;
         // Set the response type.
      }
      catch (Exception e) {
         e.printStackTrace() ;
      }
      finally {
         System.out.println("PostHandlerServlet("+sessionId+"):  service, ended : "+((float)( System.currentTimeMillis() - time ) / 1000)+ " seconds") ;   
      }
   }
}

