package com.ibm.nzna.projects.ftpsync ;
/**-----------------------------------------------------------*
 * The FTP Manager can be used to transfer and manipulate
 * files on an FTP Server
 *
 * @version 1.1
 * @author  Maria Paredes
 *
 * History
 * ----------------------------------------------- ----
 * 7/13/2000    Enabled Deleting of incoming       David Lentz
 * 12/12/2000   Added ability to write 'chmod'     
 *              script.
 */

import java.io.* ;
import java.util.* ;
import java.net.UnknownHostException ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.ftp.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   FTP Manager                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPMan {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Constants                                               */
    /*                                                           */
    /*-----------------------------------------------------------*/
    final   String scriptFilename   = "chmod_script" ;
    final   String FILE_NAME        = "Filename: " ;
    final   String FILE_AREA        = "filearea: " ;
    final   String ACTION_TYPE      = "actiontype: " ;
    final   String INSERT           = "insert" ;
    final   String DELETE           = "delete" ;
    final   String UPDATE           = "update" ;
    final   String REMOTEHOST       = ftpSync.properties.getProperty( "remoteHost" ) ;
    final   String REMOTE_DIRNAME   = ftpSync.properties.getProperty( "remoteDir" ) ;    // remote base directory
    final   String BASE_PATH        = ftpSync.properties.getProperty( "basePath" ) ;     // path to incoming and areas dirs
    final   String INCOMING_DIRNAME = ftpSync.properties.getProperty( "incomingDir" ) ;  // local incoming directory
    final   String SENT_DIRNAME     = ftpSync.properties.getProperty( "sentDir" ) ;      // local sent directory
    final   String LINEFEED         = System.getProperty( "line.separator" ) ;
    final   String SLASH            = System.getProperty( "file.separator" ) ;
    final   String USERID           = ftpSync.properties.getProperty( "userid" ) ;
    final   String PASSWORD         = ftpSync.properties.getProperty( "password" ) ;

    final   char ASCII  = 'A';               // Transfer type ASCII
    final   char BINARY = 'I';              // Transfer type IMAGE (binary) 


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Vars                                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    static File    basePath       = null ;   // home/ftp/pub/pccbbs the base path
    static File    incomingDir    = null ;   // path to the incoming directory
    static String  fileList[]     = null ;   // string listing file names in a directory
    static String  hdrFileStr     = null ;   // string of header file names
    static String  logFilename    = null ;   // name of the log file
    static String  filename       = null ;   // name of the target file
    static String  fileArea       = null ;   // name of the target area or directory
    static String  actionType     = null ;   // name of the action on the target file
    static String  inFilename     = null ;   // path to the target file in the incoming directory
    static String  headerFileName = null ;   // a header file name
    static String  localAreaFile  = null ;   // path to the target file in the local area or directory
    static Ftp     ftp            = null ;   // The FTP Bean
    static FTPSync ftpSync        = null ;
    static Vector  deleteFileVec  = null ;   // The delete


    /*------------------------------------------------------------*/
    /* FTPMan Constructor
     */
    /**
     *    This method creates an instance of the FTP Manager and 
     *    sets the initial values with the specified parameters 
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    public FTPMan ( FTPSync ftpSync) {
        this.ftpSync = ftpSync ;                                  

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   First, clear out the script to change perms             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        FileUtil.saveFile ( scriptFilename, "" ) ;


        CDate cDate = new CDate(CDate.DATE_STAMP) ;
        logFilename = "ftpsync_" + cDate.today() + ".out" ;

        ftpSync.logProgress ( "FTP Manager Starting", 0, 0 ) ;

        basePath    = new File ( BASE_PATH ) ;            // home/ftp/pub/pccbbs
        incomingDir = new File ( INCOMING_DIRNAME ) ;     // path to incoming directory

        deleteFileVec = new Vector ( 10, 10 ) ;

    }  // FTPMan Const.


    /*------------------------------------------------------------*/
    /* connect
     */
    /**
     *    This method will make the connection to
     *  the current ftp host.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    private boolean connect( ) {
        boolean    connected  = false ;

        ftpSync.logSystem.beginLogSection ( "Connecting to FTP server :"+REMOTEHOST ) ;          

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create the FTP Bean                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        ftp = new Ftp() ;          

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Now, try to connect                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        try {
            ftp.connect( REMOTEHOST ) ;
            ftpSync.logProgress ( "Connected to "+REMOTEHOST+". Complete", 0, 0 ) ;
            ftpSync.logProgress ( "   FTP Reply:"+ftp.getReply(), 0, 0 ) ;
            connected = true ;
        } catch ( UnknownHostException uHE ) {
            ftpSync.logError ( "Could not create connection to "+REMOTEHOST+". (Unknown Host) Failed" ) ;
            ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
            ftpSync.logSevereError ( uHE ) ;
        } catch ( IOException ioE ) {
            ftpSync.logError ( "Could not create connection to "+REMOTEHOST+". (IOException) Failed" ) ;
            ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
            ftpSync.logSevereError (   ioE ) ;
        }

        ftpSync.logSystem.endLogSection () ;

        return( connected ) ;

    }  // connect

    /*------------------------------------------------------------*/
    /* login
     */
    /**
     *    This method will login the current user into
     *  the current ftp host.
     *  
     *
     *  @param user is the userid of current user
     *  @param password of the current userid
     *
     *//*---------------------------------------------------------*/
    public boolean login ( String user, String password ) {
        boolean done = false ;

        ftpSync.logSystem.beginLogSection ( "Logging into server" ) ;

        try {
            ftp.login ( user, password ) ;
            ftpSync.logProgress ( "Logged into "+ftp.getHost()+".", 0, 0 ) ;
            ftpSync.logProgress ( "   FTP Reply:"+ftp.getReply(), 0, 0 ) ;
            done = true ;
        } catch ( FtpError ftpe ) {
            ftpSync.logError ( "Could not login to host "+ftp.getHost()+" with User Id:"+user+", Password:"+password ) ;
            ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
            ftpSync.logSevereError (   ftpe ) ;
        } catch ( IOException ioe ) {
            ftpSync.logError ( "Could not login to host "+ftp.getHost()+" with User Id:"+user+", Password:"+password+". (IOException)" ) ;
            ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
            ftpSync.logSevereError (   ioe ) ;
        }

        ftpSync.logSystem.endLogSection() ;

        return(done );
    }  // login


    /*------------------------------------------------------------*/
    /* logout
     */
    /**
     *    This method will logout the current user from
     *  the current ftp host.
     *  
     *
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    public void logout () {

        ftpSync.logSystem.beginLogSection ( "Logging off of "+ftp.getHost() ) ;

        try {
            ftp.logout() ;

            ftpSync.logProgress ( "Logged out successfully", 0, 0 ) ;
        } catch ( FtpError ftpe ) {
            ftpSync.logError ( "Could not logout from "+ftp.getHost() ) ;
            ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
            ftpSync.logSevereError (   ftpe ) ;
        } catch ( IOException ioe ) {
            ftpSync.logError ( "Could not logout from "+ftp.getHost()+" (IOException)" ) ;
            ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
            ftpSync.logSevereError (   ioe ) ;
        }

        ftpSync.logSystem.endLogSection() ;
    } // logout


    /*------------------------------------------------------------*/
    /* disconnect
     */
    /**
     *    This method will disconnect the current user from
     *  the current ftp host.
     *  
     *
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    public void disconnect () {

        ftpSync.logSystem.beginLogSection ( "Disconnecting from "+ftp.getHost() ) ;

        try {
            ftp.disconnect() ;
            ftpSync.logProgress ( "Disconnected from "+ftp.getHost(), 0, 0 ) ;

        } catch ( IOException ioe ) {
            ftpSync.logError ( "Could not disconnect from "+ftp.getHost()+" (IOException)" ) ;
            ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
            ftpSync.logSevereError (   ioe ) ;
        }

        ftpSync.logSystem.endLogSection () ;
    }  // disconnect


    /*------------------------------------------------------------*/
    /* startFtp
     */
    /**
     *    This method will start the process of file transfers
     *  from reading the file names to reading the header files, 
     *  to manipulating the files.
     *
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    public void startFtp () {

        ftpSync.logSystem.beginLogSection ( "Starting FTP Process" ) ;

        if ( basePath.exists() && incomingDir.exists() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Read the incoming directory                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( readIncomingDir() ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Connect to the FTP Server                               */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( connect() ) {


                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Login to the FTP Server                                 */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( login ( USERID, PASSWORD ) ) {

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Now, process each file                                  */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        for ( int i = 0; i < fileList.length; i++ ) {

                            /*-----------------------------------------------------------*/
                            /*                                                           */
                            /*   Read the header file                                    */
                            /*                                                           */
                            /*-----------------------------------------------------------*/
                            if ( readHdrFile ( i ) ) {

                                ftpSync.logSystem.beginLogSection ( "Processing File "+filename ) ;                             

                                /*-----------------------------------------------------------*/
                                /*                                                           */
                                /*   First, copy the file to the local DASD                  */
                                /*                                                           */
                                /*-----------------------------------------------------------*/
                                if ( copyToLocalArea() ) {

                                    /*-----------------------------------------------------------*/
                                    /*                                                           */
                                    /*   Transfer the file via FTP                               */
                                    /*                                                           */
                                    /*-----------------------------------------------------------*/
                                    if ( transferFile() ) {

                                        /*-----------------------------------------------------------*/
                                        /*                                                           */
                                        /*   delete the file that is in the incoming directory       */
                                        /*                                                           */
                                        /*-----------------------------------------------------------*/
                                        if ( deleteIncomingFile() ) {

                                            /*-----------------------------------------------------------*/
                                            /*                                                           */
                                            /*   finally, move the header file to the 'sent' directory   */
                                            /*                                                           */
                                            /*-----------------------------------------------------------*/
                                            moveHdrToSent() ;
                                            ftpSync.logProgress ( "All actions for file:"+filename+" complete.", 0, 0 ) ;

                                        } /* if we deleted the file from the incoming directory */

                                    } /* if transfer file went ok */

                                } /* if copy to local area went ok */

                                ftpSync.logSystem.endLogSection() ;

                            } /* if we read the header file */
                        } /* for */


                    } /* if logged in */

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Disconnect from the FTP Site                            */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    disconnect() ;                 

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Now, write the files that should be deleted             */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    try {
                        FileUtil.writeObjectToDisk ( "deleteFiles.ser", deleteFileVec ) ;
                    } catch ( Exception e ) {
                        ftpSync.logSystem.logError ( "Could not write deleteFiles.ser" ) ;
                    }

                } /* if connected */
            } /* if read files */


        } else {
            ftpSync.logError ( "Could not find directory for files. Please check base path and incoming path properties" ) ;
        }

        ftpSync.logSystem.endLogSection() ;
    }  // startFtp

    /*------------------------------------------------------------*/
    /* readIncomingDir
     */
    /**
     *    This method will read filenames, in the specified basePath,
     *  into a string.
     *  
     *
     *  @returns TRUE if we have files to process
     *
     *//*---------------------------------------------------------*/
    private boolean readIncomingDir () {
        boolean rc = false ;

        fileList = incomingDir.list ( new HDRFilter() ) ;   // will hold the filenames in 
                                                            // the given basePath
        if ( fileList.length <= 0 )
            ftpSync.logProgress ( "No files in incoming directory to process", 0, 0 ) ;
        else {
            ftpSync.logProgress ( "Going to process "+fileList.length+" files from incoming directory", 0, 0 ) ;
            rc = true ;
        }

        return( rc ) ;

    }  // readIncomingDir

    /*------------------------------------------------------------*/
    /* readHdrFile
     */
    /**
     *    This method will read the header file specified 
     *  
     *  
     *
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    private boolean readHdrFile ( int nextFile ) {
        boolean read = false ;                            


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   First, read in all of the information                   */
        /*                                                           */
        /*-----------------------------------------------------------*/
        try {
            hdrFileStr = FileUtil.readFile ( incomingDir + SLASH + fileList[ nextFile ] ) ;
            headerFileName = fileList[nextFile] ;

            ftpSync.logProgress ( "Read Header File:"+headerFileName, 0, 0 ) ;

            filename    = getHdrFileVars( hdrFileStr, FILE_NAME ) ;
            fileArea    = getHdrFileVars( hdrFileStr, FILE_AREA ) ;
            fileArea    = fileArea.substring( 1 ) ;
            actionType  = getHdrFileVars( hdrFileStr, ACTION_TYPE ) ;

            read = true ;                  


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Cannot find the header file                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
        } catch ( FileNotFoundException e ) {
            ftpSync.logError ( "Could not read header file for :"+headerFileName+". File Not Found" ) ;
            ftpSync.logSevereError ( e ) ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Cannot read the header file                             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        catch ( IOException ioE ) {
            ftpSync.logError ( "Could not read header file for :"+headerFileName+". (IOException)" ) ;
            ftpSync.logSevereError ( ioE ) ;
        } // try-catch

        return( read ) ;

    }  // readHdrFile



    /*------------------------------------------------------------*/
    /* getHdrFileVars
     */
    /**
     *    This method will get the variables Filename, Filearea and 
     *  ActionType from the header files, in order to manipulate the
     *  files
     *
     *  @param hdrFileContent a list of header filenames
     *  @param hdrVariable the current header variable to read
     *
     *//*---------------------------------------------------------*/
    public String getHdrFileVars ( String hdrFileContent, String hdrVariable ) {

        String varValue = null ;
        int lastPos = 0 ;

        if ( hdrFileContent.length() > 0 ) {

            try {
                if ( (lastPos = hdrFileContent.indexOf( hdrVariable )) != -1 ) {
                    lastPos  = lastPos + hdrVariable.length() ;

                    /* Changed the code for the program to work with Windows generated files-KVG*/
                    //varValue = hdrFileContent.substring( lastPos, hdrFileContent.indexOf(LINEFEED, lastPos) - 1 ) ; // strip CR-LF chars
                    varValue = hdrFileContent.substring( lastPos, hdrFileContent.indexOf(LINEFEED, lastPos)) ; // strip CR-LF chars
                    varValue = varValue.trim();

                } else
                    ftpSync.logError ( "Could not find variable "+hdrVariable+" in header file :"+headerFileName ) ;

            } catch ( StringIndexOutOfBoundsException siobe ) {
                ftpSync.logSevereError ( siobe ) ;
            }   // try

        } else   // hdrFileContent is empty
            ftpSync.logError ( "Header file is empty. Filename:"+headerFileName ) ;

        return(varValue );

    } // getHdrFileVars

    /*------------------------------------------------------------*/
    /* transferFile
     */
    /**
     *    This method will process the file based upon the parameter 
     *  specified in the header file: insert, update, delete. 
     *  
     *
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    private boolean transferFile() {
        File   remoteDir      = new File ( REMOTE_DIRNAME + SLASH + fileArea ) ;
        String remoteFilename = REMOTE_DIRNAME + SLASH + fileArea + SLASH + filename ;
        boolean done          = false ;                         

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Handle Inserts/Updates as the same                      */
        /*                                                           */
        /*-----------------------------------------------------------*/
        //if ( actionType.equalsIgnoreCase( INSERT ) || actionType.equalsIgnoreCase( UPDATE )) {
        if ( actionType.equalsIgnoreCase( INSERT ) || actionType.equalsIgnoreCase( UPDATE ) ) {

            try {
                ftp.setType ( BINARY ) ;
                ftp.put( inFilename, remoteFilename, false ) ;        // false means Do not append
                ftpSync.logProgress ( "Transferred File "+filename+" success.", 0, 0 ) ;
                ftpSync.logProgress ( "   FTP Reply:"+ftp.getReply(), 0, 0 ) ;
                done = true ; 
            } catch ( FtpError ftpe ) {
                ftpSync.logError ( "Cannot FTP "+inFilename+" to "+remoteFilename ) ;
                ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
                ftpSync.logSevereError ( ftpe ) ;
            } catch ( IOException ioe ) {
                ftpSync.logError ( "Cannot FTP "+inFilename+" to "+remoteFilename+". (IOException)" ) ;
                ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
                ftpSync.logSevereError ( ioe ) ;
            } // try-catch

        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Handle Deletes only if the deleteFiles flag is set      */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( (FTPSync.deleteFiles ) && ( actionType.equalsIgnoreCase( DELETE ) ) ) {

            try {
                ftp.delete( remoteFilename ) ;
                ftpSync.logProgress ( "Deleted File "+remoteFilename+" from host "+ftp.getHost()+" success.", 0, 0 ) ;
                ftpSync.logProgress ( "   FTP Reply:"+ftp.getReply(), 0, 0 ) ;
                done = true ;

            } catch ( FtpError ftpe ) {
                String ftpReply = ftp.getReply().toString() ;                               

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Only report the error if it is not a 550                */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( ftpReply.indexOf ( "550" ) == -1 ) {
                    ftpSync.logError ( "Could not delete host file "+remoteFilename ) ;
                    ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
                    ftpSync.logSevereError (   ftpe ) ;
                }

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Because 550 means that the file was not there to delete */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else {
                    ftpSync.logProgress ( "Could not delete host file "+remoteFilename+" because it was not there", 0, 0 ) ;
                    done = true ;
                }


            } catch ( IOException ioe ) {
                ftpSync.logError ( "Could not delete host file "+remoteFilename+". (IOException)" ) ;
                ftpSync.logError ( "   FTP Reply:"+ftp.getReply() ) ;
                ftpSync.logSevereError (   ioe ) ;
            } // try-catch

        } else {     // no Action Type specified
            ftpSync.logProgress ( "No Action for file:"+filename+".", 0, 0 ) ;
            done = false ;
        }

        return(done );

    } // transferFile


    /*------------------------------------------------------------*/
    /* copyToLocalArea 
     */
    /**
     *    This method will copy the file into the basePath specified
     *  in the filearea variable provided in the header file.
     *  
     *
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    private boolean copyToLocalArea() {
        boolean rc = false ;                         

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Handle updates/inserts as the same                      */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( actionType.equalsIgnoreCase( INSERT ) || actionType.equalsIgnoreCase( UPDATE ) ) {

            localAreaFile = BASE_PATH + SLASH + fileArea + SLASH + filename ;
            inFilename    = incomingDir + SLASH + filename ;                      // ie., home/ftp/pub/pccbbs/incoming/xyz.txt

            try {

                if ( FileUtil.exists( inFilename ) )
                    if ( FileUtil.copyFile ( inFilename, localAreaFile ) ) {

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Add this file to the script that changes file perms.    */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        appendFileToChmodScript ( localAreaFile ) ;


                        ftpSync.logProgress ( "Copied file "+filename+" to correct local area.", 0, 0 ) ;
                        rc = true ;
                    } else
                        ftpSync.logError ( "Could not copy file "+inFilename+" to "+localAreaFile ) ;
                else
                    ftpSync.logError ( "Could not copy file "+inFilename+" to "+localAreaFile+" (File Not Found)" ) ;

            } catch ( Exception e ) {
                ftpSync.logError ( "Cannot copy file to local file area." ) ;
                ftpSync.logError ( "   incoming file:"+inFilename ) ;
                ftpSync.logError ( "   to file:"+localAreaFile ) ;
                ftpSync.logSevereError ( e ) ;
            }
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Handle Deletes                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( actionType.equalsIgnoreCase ( DELETE ) ) {

            localAreaFile = BASE_PATH + SLASH + fileArea + SLASH + filename ;
            inFilename    = incomingDir + SLASH + filename ;                      // ie., home/ftp/pub/pccbbs/incoming/xyz.txt

            try {
                if ( FileUtil.exists ( localAreaFile ) ) {
                    FileUtil.deleteFile ( localAreaFile ) ;   // done = true holds
                    ftpSync.logProgress ( "Delete local file:"+localAreaFile+" success.", 0, 0 ) ;
                    rc = true ;
                } else {
                    ftpSync.logProgress ( "Cannot delete local file "+localAreaFile+" because it is not there to delete.", 0, 0 ) ;
                    rc = true ;
                }
            } catch ( FileNotFoundException fnfe ) {
                ftpSync.logSystem.logError ( "Cannot find local file for delete. Filename:"+localAreaFile ) ;
                rc = true ; // It's not there to delete
            } catch ( Exception e ) {
                ftpSync.logSystem.logError ( "Cannot delete file from local area:"+localAreaFile ) ;
                ftpSync.logSystem.logSevereError ( e ) ;
                rc = false ;
            }

        }

        return( rc ) ;

    } // copyToLocalArea

    /*------------------------------------------------------------*/
    /* deleteIncomingFile
     */
    /**
     *    This method will delete a file if the variable in the 
     *  header file indicates so.
     *  
     *
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    private boolean deleteIncomingFile() {
        boolean rc = false ;                                          

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add this file to the delete vec so the Cleanup.java     */
        /* program that runs after this batch can delete the file    */
        /*                                                           */
        /*-----------------------------------------------------------*/
        deleteFileVec.addElement ( inFilename ) ;
        rc = true ;


        return( rc ) ;

    } //deleteIncomingFile

    /*------------------------------------------------------------*/
    /* moveHdrToSent
     */
    /**
     *    This method will move the header file to the Sent basePath
     *  
     *  
     *
     *  @param None
     *  @param None
     *
     *//*---------------------------------------------------------*/
    private boolean moveHdrToSent() {
        boolean rc               = true ;
        String  inHdrFile        = incomingDir+SLASH+headerFileName ;
        String  sentHdrFile      = SENT_DIRNAME+SLASH+headerFileName ;

        try {
            if ( FileUtil.copyFile ( inHdrFile, sentHdrFile ) ) {              //copy header to local sent directory
                FileUtil.deleteFile ( inHdrFile ) ;                             //delete from incoming directory
                ftpSync.logProgress ( "Moved header file of "+sentHdrFile, 0, 0 ) ;
                rc = true ;
            } else
                ftpSync.logError ( "Could not move header file "+headerFileName+" to "+SENT_DIRNAME ) ;
        } catch ( FileNotFoundException fnfe ) {
            ftpSync.logSystem.logError ( "Cannot find header file for move. Header filename:"+headerFileName ) ;
            ftpSync.logSystem.logSevereError ( fnfe ) ;
        } catch ( Exception e ) {
            ftpSync.logSystem.logError ( "Cannot move header file. Header filename:"+headerFileName ) ;
            ftpSync.logSystem.logSevereError ( e ) ;
        }

        return( rc ) ;

    } // moveHdrToSent





    /*------------------------------------------------------------*/
    /* appendFileToChmodScript
     */
    /**
     *    This method will ammend a filename to
     *  the script that is run after the batch is complete.
     *  This script will set the permissions for all files this batch processes
     *
     *  @param filename The name of the file to append
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void appendFileToChmodScript ( String filename ) {
        try {
            String currentScript = FileUtil.readFile ( scriptFilename ) ;

            currentScript+="\nchmod 664 "+filename ;

            FileUtil.saveFile ( scriptFilename, currentScript ) ;
        } catch ( Exception e ) {
            ftpSync.logSevereError ( e ) ;
        }
    }



}  // End of FTPMan
