package com.ibm.nzna.projects.ftpsync;
/*-----------------------------------------------------------*/
/*   FTPSync                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FTPSync                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The FTPSync batch synchronizes the f3.pc.ibm.com and ftp.pc.ibm.com directories. 
 * Please review the documentation on http://borg.raleigh.ibm.com for more information
 *
 *
 * @version 1.0, 3/27/2000 
 * @author  Maria Paredes
 */
import java.io.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.batch.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   FTPSync                                                 */
/*                                                           */
/*    This will load properties, such as local host,         */
/*    remote host names, path names, userid, password, etc., */
/*    neccessary to run and will start the batch             */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPSync extends Batch {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Public Members                                          */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static  FTPMan          ftpMan     = null ;
    public static  Properties      properties = new Properties() ;             // system properties necessary to run the FTP Manager
    public static  BatchLogSystem  logSystem  = null ;

    protected static boolean deleteFiles                = false;     //DAL: TRUE if we are to delete files.


    /*------------------------------------------------------------*/
    /* main
     */
    /**
     *    The main entry point for the application. To run the
     * program in addOnly mode, java FTPSync -addOnly true
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void main ( String[] args ) throws FileNotFoundException, IOException {

        //Suneel: To do add Only
        if ( args.length > 0 ) {
            String command = args[0];
            //Check if -addOnly argument is passed or not
            if ( command != null && command.equalsIgnoreCase("-deleteFiles") ) {
                deleteFiles = true ;
            }
        }//--


        new FTPSync() ;
    }





    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create and initialize our
     *  batch
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public FTPSync () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   First create our BatchLogSystem                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        initializeLogSystem() ;


        try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Initialize our properties                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            initializeProperties() ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the FTP Manager                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftpMan = new FTPMan(this) ;


        } catch ( IOException ioe ) {
            logError ( "Could not load properties file" ) ;
            logError ( ioe + "\n" ) ;
        }


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Start the FTP Manager                                   */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( ftpMan != null )
            ftpMan.startFtp() ;
    }





    /*------------------------------------------------------------*/
    /* initializeProperties
     */
    /**
     *    This method will read in out property file
     *  and creates our property object from it
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void initializeProperties () throws IOException {
        FileInputStream propertiesFile = new FileInputStream( "FtpSync.properties" ) ;                                        // Load the Properties File

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create a property object with our properties file       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        properties.load ( propertiesFile ) ;                                           // load the properties

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Now, if we loaded the properties, write out each one    */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( properties != null ) {
            Enumeration propNames = properties.propertyNames() ;

        }


    }





    /*------------------------------------------------------------*/
    /* initializeLogSystem
     */
    /**
     *    This method will initialize our loggin system
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void initializeLogSystem () {
        CDate  cDate       = new CDate(CDate.DATE_STAMP) ;
        String logFilename = "logs"+File.separator+"ftpsync" ;

        logSystem = new BatchLogSystem ( logFilename, this ) ;              
        logSystem.deleteOldLogs ( 2 ) ;

    }
}  //  FTPSync

