package com.ibm.nzna.projects.htmlProductImport ;
/*-----------------------------------------------------------*/
/*   HTMLParse                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HTMLParse                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object holds functions to parse html. The HTMLParse
 * object can be used to parse through the tags of an HTML Page.
 * Some of the functions should only be used with HTML that guarentees
 * a begin/end pair. These functions include:
 *
 * getTagBlock()
 *
 * @version 1.0, 8/15/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HTMLParse                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class HTMLParse extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String     HTMLString  = null ;
   private HTMLTag[]  tagArray    = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a new HTMLParse Object
    *  that can be used to parse through HTML Tags
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HTMLParse ( String HTMLString ) {
      setHTMLString ( HTMLString ) ;

   } /* Constructor */





   /*------------------------------------------------------------*/
   /* getAllTags
    */
   /**
    *    This method will parse out all of the tags in
    *  the HTMLString.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void getAllTags() {
      if ( HTMLString != null ) {
         StringTokenizer strtok = new StringTokenizer ( HTMLString, "<" ) ;
         int             length = strtok.countTokens() ;

         if ( length > 0 ) {
            int    i       = 0 ;
            int    k       = 0 ;
            int    hLength = HTMLString.length() ;
            int    endTag  = 0 ;
            String newTag  = null ;

            tagArray = new HTMLTag [ length ] ;

            while ( ( i < hLength ) && ( k < length ) ) {
               if ( HTMLString.charAt(i) == '<' ) {
                  endTag = HTMLString.indexOf ( ">", i ) ;

                  if ( endTag >= 0 ) {
                     newTag = HTMLString.substring ( i, endTag ) ;
                     tagArray[k] = new HTMLTag ( newTag+">", i, k ) ;
                     k++ ;
                  } /* if */
               } /* if */
               i++ ;
            } /* while */

         } /* if */
      } /* if */
   } /* getAllTags */




   /*------------------------------------------------------------*/
   /* getTag
    */
   /**
    *    This method will return the tag at the specified
    *  tagNum.
    *  
    *
    *  @param tagNum The 0 based index number that the tag is.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTag ( int tagNum ) {
      String retStr = null ;

      if ( ( tagArray != null ) && ( tagNum < tagArray.length ) && ( tagArray[tagNum] != null ) )
         retStr = tagArray[tagNum].getTag() ;

      return( retStr ) ;
   } /* getTag */



   /*-----------------------------------------------------------*/
   /* getTagRec
    */
   /**
    *    This method will return a HTMLTag record
    *  that is at the tagNum index passed in.
    *  
    *             
    *  @param tagNum The 0 based index of the HTML tag in the HTML String.
    *  @param 
    *  @param 
    *
    *  @return HTMLTag
    */
   public HTMLTag getTagRec ( int tagNum ) {
      HTMLTag tagRec = null ;

      if ( ( tagArray != null ) && ( tagNum < tagArray.length ) )
         tagRec = tagArray[tagNum] ;

      return( tagRec ) ;
   } /* getTagRec */



   /*------------------------------------------------------------*/
   /* replaceTag
    */
   /**
    *    This method will replace a tag at the tagNum index
    *  with another tag.
    *  
    *
    *  @param tagNum the 0 based index number of the tag to replace
    *  @param tag The tag to replace with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean replaceTag ( int tagNum, String tag ) {
      boolean replaced = false ;

      if ( ( tagArray != null ) && ( tagNum < tagArray.length ) ) {
         int startTagIndex = tagArray[tagNum].getCharPos() ;
         int oldLength     = tagArray[tagNum].getTag().length() ;
         int stopTagIndex  = startTagIndex+oldLength ;
         int newLength     = tag.length() ;
         int i             = tagNum+1 ;
         int lengthDiff    = newLength-oldLength ;
         String oldStr = HTMLString ;
         String oldTag = tagArray[tagNum].getTag() ;

         HTMLString = HTMLString.substring ( 0, startTagIndex )+tag+HTMLString.substring ( stopTagIndex, HTMLString.length() ) ;
         tagArray[tagNum].setTag ( tag ) ;

         /* Ok, now add to the charPos of each of the tags after this tag */
         if ( lengthDiff != 0 ) {
            while ( i < tagArray.length ) {
               tagArray[i].setCharPos ( tagArray[i].getCharPos()+lengthDiff ) ;
               i++ ;
            }
         }

         replaced = true ;

      } /* if */

      return( replaced ) ;
   } /* replaceTag */




   /*------------------------------------------------------------*/
   /* getHTMLString
    */
   /**
    *    This method will return the HTMLString
    *  that the HTMLParse might have modified with
    *  calls to replaceTag etc..
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getHTMLString () {
      return( HTMLString ) ;
   } /* getHTMLString */




   /*------------------------------------------------------------*/
   /* setHTMLString
    */
   /**
    *    This method will set the HTML String that
    *  this parser will use.
    *  
    *
    *  @param HTMLString The new HTML String to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setHTMLString ( String HTMLString ) {
      this.HTMLString = HTMLString ;
      getAllTags() ;
   } /* setHTMLString */


   /*------------------------------------------------------------*/
   /* getTags
    */
   /**
    *    This method will return an array of HTMLTag Objects
    *  that appear in the HTMLString used by the
    *  parser.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return HTMLTag[] An array of HTML Tag objects
    *//*---------------------------------------------------------*/
   public HTMLTag[] getTags () {
      return( tagArray ) ;
   } /* getTags */





   /*------------------------------------------------------------*/
   /* findTag
    */
   /**
    *    This method will find the first nearest matching tag. In other
    *  words if a tag holds all of the letters in the searchStr,
    *  the tag will be returned.
    *
    *  @param searchStr The string to search with
    *  @param None
    *  @param None
    *
    *  @return tag The string HTML Tag
    *//*---------------------------------------------------------*/
   public HTMLTag findTag ( String searchStr ) {
      HTMLTag retTag = null ;

      if ( tagArray != null ) {
         int    i                  = 0 ;
         int    length             = tagArray.length ;
         String searchStrUpperCase = searchStr.toUpperCase() ;

         while ( ( i < length ) && ( retTag == null ) ) {
            if ( tagArray[i].getTagUpperCase().indexOf ( searchStrUpperCase ) >= 0 )
               retTag = tagArray[i] ;
            i++ ;
         } /* while */

      } /* if */

      return( retTag ) ;
   } /* findTag */




   /*------------------------------------------------------------*/
   /* findNextTag
    */
   /**
    *    This method will find the tag after the specified
    *  tagNum.
    *  
    *
    *  @param searchStr The string to search with
    *  @param None
    *  @param None
    *
    *  @return HTMLTag object
    *//*---------------------------------------------------------*/
   public HTMLTag findNextTag ( int tagNum, String searchStr ) {
      HTMLTag retTag = null ;

      if ( ( tagArray != null ) && ( tagNum < tagArray.length ) ) {
         int    i                  = tagNum ;
         int    length             = tagArray.length ;
         String searchStrUpperCase = searchStr.toUpperCase() ;

         while ( ( i < length ) && ( retTag == null ) ) {
            if ( tagArray[i].getTagUpperCase().indexOf ( searchStrUpperCase ) >= 0 )
               retTag = tagArray[i] ;
            i++ ;
         } /* while */

      } /* if */

      return( retTag ) ;
   } /* findTag */


   /*------------------------------------------------------------*/
   /* findAndReplaceTag
    */
   /**
    *    This method will find and replace the first
    *  occurance of the specified tag in the HTML
    *  string.
    *
    *  @param tag The Current HTMLTag object representing the tag
    *  @param newTag The new tag to replace with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void findAndReplaceTag ( HTMLTag tag, String newTag ) {
      setHTMLString ( Text.replaceStrInStrFrom ( HTMLString, tag.getCharPos()-1, tag.getTag(), newTag ) ) ;
   } /* findAndReplaceTag */

   /*------------------------------------------------------------*/
   /* globalFindAndReplaceTag
    */
   /**
    *    This method will find and replace all
    *  occurances of the specified tag in the HTML
    *  string.
    *
    *  @param oldTag The old tag to be replaced
    *  @param newTaf The new tag to replace with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void globalFindAndReplaceTag ( String oldTag, String newTag ) {
      setHTMLString ( Text.replaceAllStrInStr ( HTMLString, oldTag, newTag ) ) ;
   } /* globalFindAndReplaceTag */




   /*------------------------------------------------------------*/
   /* removeTag
    */
   /**
    *    This method will remove a tag from the HTML string
    *  
    *  
    *
    *  @param tagNum The tag number to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeTag ( int tagNum ) {
      int i             = tagNum+1 ;

      try {
      if ( ( tagArray != null ) && ( tagNum < tagArray.length ) ) {
         int startTagIndex = tagArray[tagNum].getCharPos() ;
         int oldLength     = tagArray[tagNum].getTag().length() ;
         int stopTagIndex  = startTagIndex+oldLength ;

         HTMLString = HTMLString.substring ( 0, startTagIndex )+HTMLString.substring ( stopTagIndex, HTMLString.length() ) ;
         tagArray[tagNum].setTag ( "" ) ;

         /* Ok, now add to the charPos of each of the tags after this tag */
         while ( i < tagArray.length ) {
            tagArray[i].setCharPos ( tagArray[i].getCharPos()-oldLength ) ;
            i++ ;
         }
      } /* if */
      } catch ( Exception e ) {
         if ( tagArray[tagNum] != null )
            System.out.println ( "Could not remove tag:"+tagArray[tagNum].getTag() ) ;
         else
            System.out.println ( "Coult not remove tag#"+tagNum ) ;
      }

   } /* removeTag */






   /*------------------------------------------------------------*/
   /* tagCount
    */
   /**
    *    This method will return the number of tags
    *  in the html document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The number of tags
    *//*---------------------------------------------------------*/
   public int tagCount () {
      int rc = 0 ;

      if ( tagArray != null )
         rc = tagArray.length ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* removeAllTags
    */
   /**
    *    This method will remove all tags from the HTML
    *  String.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAllTags () {
      int length = tagArray.length ;
      int i      = 0 ;

      while ( i < length )
         removeTag ( i++ ) ;
   } /* removeAllTags */





   /*-----------------------------------------------------------*/
   /* getSizeFromFontTag
    */
   /**
    *    This method will return the font size from the
    *  font tag passed in.
    *  
    *             
    *  @param fontTag The font tag to look into. This tag must include FONT SIZE="x"
    *  @param 
    *  @param 
    *
    *  @return size of the font.
    */
   public static int getSizeFromFontTag ( String fontTag ) {
      int size       = 0 ;
      int startIndex = 0 ;
      int endIndex   = 0 ;

      startIndex = fontTag.indexOf ( "SIZE" ) ;

      if ( startIndex >= 0 ) {

         /* Bump the index to the position after the double quote */
         startIndex = fontTag.indexOf ( "\"", startIndex )+1 ;
         endIndex = fontTag.indexOf ( "\"", startIndex ) ;

         if ( startIndex >= 0 ) {
            String tempStr = fontTag.substring ( startIndex, endIndex ) ;

            try {
               size = new Integer ( tempStr ).intValue() ;
            } catch ( Exception e ) {
               size = 0 ;
            } /* catch */
         } /* if */
      } /* if */

      return( size ) ;
   } /* getSizeFromFontTag */


   /*-----------------------------------------------------------*/
   /* getImageLoc
    */
   /**
    *    This method will return the image location in an image tag.
    *  
    *             
    *  @param imageTag The image tag to check in ( <IMG SRG="HTTP://MYSERVER/IMAGES/IMAGES.GIF"> )
    *  @param 
    *  @param 
    *
    *  @return If the above tag is passed, this method will return HTTP://MYSERVER/IMAGES/IMAGES.GIF
    */
   public static String getImageLoc ( String imageTag ) {
      String retStr     = null ;
      int    startIndex = 0 ;
      int    endIndex   = 0 ;

      imageTag = Text.replaceInStr ( imageTag, "src", "SRC" ) ;

      startIndex = imageTag.indexOf ( "SRC" ) ;

      if ( startIndex >= 0 ) {

         /* Bump the index to the position after the double quote */
         startIndex = imageTag.indexOf ( "\"", startIndex )+1 ;
         endIndex = imageTag.indexOf ( "\"", startIndex ) ;

         if ( ( startIndex >= 0 ) && ( endIndex > startIndex ) )
            retStr = imageTag.substring ( startIndex, endIndex ) ;
      } /* if */

      return( retStr ) ;
   } /* getImageLoc */





   /*------------------------------------------------------------*/
   /* replaceTagPair
    */
   /**
    *    This method will replace a begin and end tag pair 
    *  with another begin and end tag.
    *  
    *  NOT IMPLEMENTED YET
    *
    *  @param origBeginTag The original begin tag to look for
    *  @param origEndTag The original end tag to look for
    *  @param newBeginTag The new begin tag to replace the original with
    *  @param newEndTag The new end tag to replace the original with
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void replaceTagPair ( String origBeginTag, String origEndTag, String newBeginTag, String newEndTag ) {
   } /*  */


   /*-----------------------------------------------------------*/
   /* getImageSource
    */
   /**
    *    This method will return the source of the image
    *  in a full image tag ( IMG SRC="http://www.www.com/test.jpg" ). This
    *  tag would return 'http://www.www.com/test.jpg'
    *  
    *             
    *  @param imageTag The full image tag
    *  @param 
    *  @param 
    *
    *  @return String The Name
    */
   public static String getImageSource ( String imageTag ) {
      String retStr = null ;

      if ( imageTag != null ) {
         String tag = imageTag.toUpperCase() ;
         int    index = imageTag.indexOf ( "SRC=\"" ) ;

         if ( index >=0 ) {
            retStr = imageTag.substring ( index+5, imageTag.indexOf ( "\"", index+7 ) ) ;
         } /* index */

      } /* if */

      return( retStr ) ;
   } /* getImageSource */


   /*-----------------------------------------------------------*/
   /* getAnchorName
    */
   /**
    *    This method will return the name of an anchor
    *  from a full anchor tag ( <A NAME="TEST"> )
    *  
    *             
    *  @param anchorTag The full anchor tag
    *  @param 
    *  @param 
    *
    *  @return String The Name
    */
   public String getAnchorName ( String anchorTag ) {
      String retStr = null ;

      if ( anchorTag != null ) {
         String tag = anchorTag.toUpperCase() ;
         int    index = anchorTag.indexOf ( "NAME=\"" ) ;

         if ( index >=0 ) {
            retStr = anchorTag.substring ( index+6, anchorTag.indexOf ( "\"", index+7 ) ) ;
         }

      } /* if */

      return( retStr ) ;
   } /* getAnchorName */




   /*-----------------------------------------------------------*/
   /* getTagBlock
    */
   /**
    *    This method will return the block of text
    *  between a tag and it's ending tag. This block
    *  of text may include other tags.
    *             
    *  @param beginTag The tag to start with.
    *  @param 
    *  @param 
    *
    *  @return String The block of text
    */
   public String getTagBlock ( HTMLTag beginTag ) {
      String retStr = null ;

      if ( beginTag != null ) {
         String tag = getHTMLTag ( beginTag.getTag() ) ;

         if ( tag != null ) {
            int     i           = beginTag.getTagNum();
            int     ignoreCount = 0 ;
            int     endIndex    = 0 ;
            int     startIndex  = beginTag.getCharPos()+beginTag.getTag().length() ;
            HTMLTag nextTag     = null ;

            i++ ;
            while ( ( i < tagArray.length ) && ( endIndex == 0 ) ) {
               nextTag = tagArray[i] ;

               /* If this next tag is not an ending tag */
               if ( !nextTag.getTag().startsWith ( "</"+tag ) ) {

                  /* If this is another tag of the same type, update
                     our ignoreCount */
                  if ( nextTag.getTag().indexOf ( tag ) > -1 )
                     ignoreCount++ ;

               } else {

                  /* This is an ending tag for the tag we were passed. If 
                     we are supposed to ignore it, update the ignoreCount */
                  if ( ignoreCount > 0 )
                     ignoreCount-- ;
                  else
                     endIndex = nextTag.getCharPos() ;

               } /* else */

               i++ ;
            } /* while */

            if ( endIndex > 0 ) {
               retStr = HTMLString.substring ( startIndex, endIndex ) ;
            }

         } /* if tag */

      } /* beginTag */

      return( retStr ) ;
   } /* beginTag */




   /*-----------------------------------------------------------*/
   /* getHTMLTag
    */
   /**
    *    This method will return only the HTML Tag
    *  from a full tag. For example, for the tag: <font size="2">,
    *  this method will return 'font' only.
    *             
    *  @param tagStr The String representation of the tag.
    *  @param 
    *  @param 
    *
    *  @return String the HTML Tag
    */
   public static String getHTMLTag ( String tagStr ) {
      String retStr = null ;

      if ( tagStr != null ) {
         int endIndex = tagStr.indexOf ( " " ) ;

         if ( endIndex == -1 )
            endIndex = tagStr.indexOf ( ">" ) ;

         if ( endIndex > -1 ) {
            retStr = tagStr.substring ( 0, endIndex ).trim() ;

            /* Take out the begin tag mark if it is there */
            if ( retStr.startsWith ( "<" ) )
               retStr = retStr.substring ( 1, retStr.length() ) ;

         } /* endIndex */
      } /* if */

      return( retStr ) ;
   } /* getHTMLTag */








   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will free all resources within
    *  the HTMLParse object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void free () {
      int i       = 0 ;

      HTMLString = null ;

      if ( tagArray != null ) {
         while (i  < tagArray.length ) {
            tagArray[i].free() ;
            tagArray[i]=null ;
            i++ ;
         }
         tagArray = null ;
      }
      System.gc() ;
   }


   public String getTextBetweenTags ( HTMLTag tag1, HTMLTag tag2 ) {
      String retStr = "" ;

      if ( ( tag1 != null ) && ( tag2 != null ) ) {
         retStr = retStr.substring ( tag1.getCharPos()+tag1.length(), tag2.getCharPos() ) ;
      }

      return ( retStr ) ;
   }


} /* HTMLParse */


