package com.ibm.nzna.projects.oa.engine;
 
import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.oa.*;

/**
 * oa.engine.DemoLogic Servlet for Online Assistant Demonstration of LogicAPI
 * @(#)DemoLogic.java	1.0 03/29/2000
 * @author Mike Rychener, IBM IGS for PSG Helpcenter
 **/

/* specs 
 *
 * TBD
 *
 */

/* HISTORY
 *  05/29  mdr  created skeleton from fetch.java
 *  05/31  mdr  exploring OA LogicAPI dispatch exception: java.lang.NoClassDefFoundError
 *               -- OA LogicAPI dispatch exception detail:null
 *                  (tried printing stackTrace but it's empty)
 *  08/21  mdr  add args to LogicAPI constructor to get it to compile properly
 *  08/24  mdr  remove qit_flag
 *
 *  ----- TO DO --
 *
 * END HISTORY
 */

/**********************************************************************
 * The main work of the oa.engine.DemoLogic servlet is done by the servletService method.
 * The init method of the ServletBase class will read DemoLogic.properties, if any.
 * We just need to tell it here what the name of the properties file is.
 * @author Mike Rychener
 * @version 1.0
 **********************************************************************/
public class DemoLogic extends ServletBase
{ 
    private final String propertyFilename = "DemoLogic" ;
    protected String getPropertyFileName() {
        return ( propertyFilename ) ;
    }

    private String formatCalDate(int yr, int mo, int dt) {
        // this simple convenience method turns 3 int's into a date string
        return (
            Integer.toString(yr) + ((mo < 10) ? "-0" : "-") + Integer.toString(mo)
            + ((dt < 10) ? "-0" : "-") + Integer.toString(dt)
        );
    }

    private String subSplice(String str, String sub, String snu) {
        // take a given sequence sub out of a string str and splice the remainders together,
        // inserting a third string snu where sub used to be (only if it was there)
        if (str.indexOf(sub) < 0) return str;
        if (str.indexOf(sub) == 0) return (snu + str.substring(sub.length()));
        return ( str.substring(0, str.indexOf(sub)) + snu
                 + str.substring(str.indexOf(sub) + sub.length()) );
    }

/**********************************************************************
 * servletService takes in CGI parameters, calls the LogicAPI servlet and TBD
 * @param (Get/Post) TBD
 **********************************************************************/   
    public void servletService (
	HttpServletRequest	request,
	HttpServletResponse	response
        ) throws ServletException, IOException, MalformedURLException
    {
	String			title = "oa.engine.DemoLogic Servlet";
        InputStreamReader       instr = null;
        BufferedReader          tbuf = null;
        int                     cbytes = 0;
        int                     sel1 = 0;
        int                     sel2 = 0;
        String                  parcom = "";
        String                  parnode = "";
        String                  parsess = "";
        String                  parseqno = "";
        String                  parsym = "";
        String                  parqfl = "";

        writeLog("DemoLogic starting");
        response.setContentType("text/html");
        // write the data of the response -- PrintWriter wrapped around byte stream buffer
        ByteArrayOutputStream bytes = new ByteArrayOutputStream(1024);
	PrintWriter out = new PrintWriter(bytes, true);  // force flushing
        out.println("<HTML><HEAD><TITLE>");  // headers, etc. unless eSupport site
        out.println(title);
	out.println("</TITLE></HEAD><BODY>");

      try {  // either call LogicAPI with arguments or print a usage message
          parcom = request.getParameter("command");
          if (parcom == null) parcom = request.getParameter("com");
          parnode = request.getParameter("node");
          if (parnode == null) parnode = request.getParameter("nod");
          parsess = request.getParameter("sessionind");
          if (parsess == null) parsess = request.getParameter("ses");
          parseqno = request.getParameter("seqno");
          if (parseqno == null) parseqno = request.getParameter("seq");
          writeLog("DemoLogic check 1 -- " + parcom + ":" + parsess
                   + ":" + parseqno + ":" + parnode);
          // get these and other parameters into an OASubmitVars object:
          OASubmitVars subv = new OASubmitVars(0);
          if (parcom != null) subv.command = parcom;
          if (parnode != null) subv.nodeInd = Integer.parseInt(parnode);
          if (parsess != null) subv.sessionInd = Integer.parseInt(parsess);
          if (parseqno != null) subv.sessionSeqNo = Integer.parseInt(parseqno);
          parsym = request.getParameter("symptom");
          if (parsym == null) parsym = request.getParameter("sym");  // validity checked below
          // parqfl = request.getParameter("qflag");
          // if (parqfl == null) parqfl = request.getParameter("qfl");
          // if (parqfl == null) subv.qit_flag = true;  // default in case no parameter
          // else subv.qit_flag = (parqfl.equals("true")) ? true : false;
          writeLog("DemoLogic check 1.7 -- " + parcom + ":" + parsess
                   + ":" + parseqno + ":" + parnode);
          subv.node_flag = request.getParameter("nflag");
          if (subv.node_flag == null) subv.node_flag = request.getParameter("nfl");
          if (subv.node_flag == null) subv.node_flag = "DQ";  // default in case no parameter
          parqfl = request.getParameter("response");
          if (parqfl == null) parqfl = request.getParameter("res");
          if (parqfl != null) subv.response = Integer.parseInt(parqfl);
          // default response will be what the constructor set it to
          subv.message = request.getParameter("message");
          if (subv.message == null) subv.message = request.getParameter("mes");
          if (subv.message == null) subv.message = "";
          if (parcom != null) {
              writeLog("DemoLogic check 2 -- " + parcom + ":" + parsess
                       + ":" + parseqno + ":" + parnode);
              // now call the dispatch method of Logic API
              LogicAPI oala = new LogicAPI(subv.sessionInd, subv.sessionSeqNo, "/tmp/oa/", null, null);
              OAReturnVars res = oala.dispatch(subv,
                                               (parsym == null) ? 0 : Integer.parseInt(parsym));
              out.println(res.toString());
          } else {
              writeLog("DemoLogic check 3 -- " + parcom + ":" + parsess
                       + ":" + parseqno + ":" + parnode);
              out.println("<pre>");
              out.println("USAGE: put parameters and values after '?' separated by &amp;");
              out.println("where parameters are:");
              out.println("  command = GetNextNode,Get/SetHistory,SaveResolved/Unresolved");
              out.println("  node = node indicator (integer)");
              out.println("  nflag = node-flag (AA, AQ, AS, DA, DQ, DS)");
              out.println("  response = answer indicator (integer)");
              out.println("  message = additional values / parameters (string)");
              out.println("  sessionind = session indicator (integer)");
              out.println("  seqno = sequence within session (integer)");
              // out.println("  qflag = QIT-flag (true or false)");
              out.println("  symptom = symptom indicator (integer)");
              out.println("Parameter names can be abbreviated with the first 3 letters.");
              out.println("Eg, .../demo?com=GetNextNode&nfl=DQ&res=130567&node=150210");
              out.println("</pre>");
          }

      }  // end try either call LogicAPI ...
      catch ( Exception e ) {
        StringWriter    sw      = new StringWriter ();
        PrintWriter     pw      = new PrintWriter ( sw );

        /* Get the stackTrace */
        e.printStackTrace ( pw );
        out.println("<p><font size=4 face=\"Helvetica, Arial\" color=Red>"
              + "Error in main loop.  Possible parameter error.  "
              + "Please try again.</font></p>"
              + "<!-- " + e.getMessage() + " -->"
              + "<!-- " + pw.toString() + " -->");
        writeLog("DemoLogic exception TBD -- " + parcom + ":" + parsess
                   + ":" + parseqno + ":" + parnode);
      }  // end catch exception

        out.println("</BODY></HTML>");
        writeLog("DemoLogic returning normally -- " + parcom + ":" + parsess
                 + ":" + parseqno + ":" + parnode);
        response.setContentLength(bytes.size());
        bytes.writeTo(response.getOutputStream());
    }  // end servletService method
}  // end DemoLogic class

