/* OA LogicAPI: GetNextNode, SaveResolved, SaveUnresolved, SetHistory, GetHistory */

package com.ibm.nzna.projects.oa.engine;
import java.io.*;
import java.lang.*;
import java.sql.*;
import java.util.*;
import com.ibm.nzna.shared.util.* ;  // log system, regular expression, etc.
import com.ibm.nzna.shared.servlet.template.* ;
import com.ibm.nzna.projects.oa.engine.*;

/**
 * LogicAPI class
 * @(#)LogicAPI.java	1.0 05/21/2000
 * @author Mike Rychener, IBM IGS for PSG Helpcenter
 **/

/* specs 
 *
 * takes in OASubmitVars object, extracts values and makes calls to OAEngine methods
 * returns OAReturnVars object with values filled in, suitable for html wrapper or QIT GUI
 *
 */

/* HISTORY
 *  05/21  mdr  created skeleton
 *  05/22  mdr  sketch the main interface methods + dispatch
 *  05/24  mdr  make main process args from command line; add try-catch stuff
 *              populate submitVars and call dispatch as the next test
 *  05/25  mdr  add code to constructor to attach to session, read session index, etc.
 *              add class variables, some of which are set by constructor
 *  05/26  mdr  debug / test changes; remove session state arg from dispatch; it loads & saves it
 *  05/29  mdr  add ERROR branch to dispatch (testing DemoLogic class)
 *  05/31  mdr  change system.exit to return, several places (it closes websphere jvm)
 *              add stacktrace printing and revise trace outputs a little
 *  06/01  mdr  remove node arg from getNextNode, is in session state now
 *              add hooks to include seqno in saved session state object names
 *              new processHistory method (coded by Venu)
 *              add seqno constructor parameter, new curSessionBase
 *              incr seqno when saving file
 *  06/04  mdr  add parsing of title tags and population of return vectors
 *              add synchronized (index) block (for critical section)
 *  06/05  mdr  debug; update documentation (javadoc) comments
 *  06/07  mdr  add passing of brand, docClass, geo, model from SubmitVars to SessionState
 *              removed tag scanning loops (<AnsTitle ###> etc.) -- saved in tagscan.java
 *  06/08  mdr  add 2nd constructor, allow path to be input from properties / servlet base
 *              comment out synchronized block, seems not what I wanted
 *  06/09  mdr  revise logic for case of no index file; restore synchronized blocks (now 3)
 *  06/10  mdr  add custnum (string) passing from submit vars to session state (no hooks there yet)
 *  06/11  mdr  add error message packaging for dispatch exception
 *  06/12  mdr  set seqno before writing session object
 *              add logHistory, endSession methods + commands LogHistory, EndSession
 *              filled in stub for GetHistory command
 *  06/13  mdr  add calls to new OAEngine methods getLanguageIndandSet and getThreeCharCountryCode
 *                so that table lookup can happen on language and country codes
 *              add two new constructor parameters = properties objects for preceding lookups
 *              make class variables here private, should not have been public
 *  06/14  mdr  put myDebug around several system.out.println's; changed null testing in param's
 *              add set of seqno in returnVars from session, after new one written, end of disp.
 *  06/22  mdr  add two methods for gathering database parameters to be passed to OAEngine
 *                (questDbname, etc. and clioDbname, etc.)
 *  06/27  mdr  add parsing of custnum from command line + set defaults of 5 DB parameters, 2 DBs
 *                in main only
 *  07/05  mdr  add oaret fill-in for exception case after standard error display
 *  07/06  mdr  add svar.sessionSeqNo = iseq setting for command-line arg
 *  08/03  mdr  add OAMemoryInterface from submit vars to session state (qitMemory value)
 *              removed a bunch of 'shared' imports
 *  08/10  mdr  update comments for dispatch (show command values) + agenda + various other minor
 *  08/17  mdr  add banner comments in dispatch, other misc. comments
 *              add new oases setup in exception inside processHistory
 *              add history size to session state debug display
 *  08/21  mdr  add arg to logHistory method, namely message string
 *              new class var = logFile, used by LogHistory, et al
 *              add myFutil FileUtil and calls to its method appendToFile, for general logging
 *  08/22  mdr  add more appendToFile for logging, especially inside catch blocks
 *              include calendar date & time as part of log
 *  08/23  mdr  handle processHistory exceptions better - different messages from normal dispatch
 *                and add exception message text (toString)
 *              curSessionFile was not being updated properly when seqno bumped up + sessionPath
 *                was missing in some places
 *              send OAMemoryInterface object if any to OAEngine as a parameter + set source flag
 *              remove qit_flag everywhere -- use qitMemory now
 *  08/24  mdr  don't print the returned object from dispatch in main, OAEngine does it
 *              in catch (main processing for dispatch), check that oaret is not null
 *              revise logic to ensure no error if user doesn't have any country in profile (US)
 *  08/29  mdr  add debug print: sequence number was incremented (used to be in retvars print)
 *  08/30  mdr  add some special settings to processHistory, to get easier input conventions
 *  09/18  mdr  add symptom title to return for Save[Un]Resolved, use title field
 *              fill in endSession and use new method getCalDateTime in a couple of places
 *              expand comments in exception printouts when appendToFile fails
 *  09/20  mdr  expand starting timestamp log message
 *              clean up session objects from /tmp/oa or wherever, on EndSession
 *              sessionPath is now passed to OAEngine and DBAPI for storage of their logs if any
 *              fill in logHistory appendToFile
 *              took out debug stmt setting language to English ALWAYS
 *  09/22  mdr  include 'continuing' timestamp message too (cf. 9/20 starting)
 *              return immediately for several 'final' commands for session
 *  10/02  mdr  add preanswerLists properties and use of addPreanswerItem to set up preanswers
 *  10/09  mdr  fill in details to preanswer brand and geography question (geo only if North Am.)
 *              handle new lang parameter from oasubmitvars
 *  10/10  mdr  error-checking around substitute on brand, including new MyException thrown
 *              (substitute had replaced a splice loop) - need \\+ in that substitute
 *              NO GOOD: scrapped substitute, apparent java bug; restored original loop
 *  10/11  mdr  adjust properties path for LogicAPI.properties (had left out 'servlet')
 *              add myDebug to check whether propertie were loaded
 *  10/12  mdr  add preanswerProc to avoid repeatedly pre-answering
 *              process supported.*.* preanswers
 *  10/16  mdr  add new versions of setQuestNames, setClioNames to allow appDriver parameter
 *                which sets the class var dbUseAppDriver
 *              pass dbUseAppDriver into OAEngine() in dispatch
 *              take out new OAEngine() in main -- not used, plus it's done in dispatch now
 *  10/17  mdr  change RegularExpression.substitute() to string replace()
 *  10/24  tlaw change debug statements and logFile to use LogSystem.log
 *  10/30  mdr  change dbUseAppDriver to be false by default
 *              un-comment-out tlaw's writings (via appendToFile) to special history log (logFile)
          --> TO DO:
                country setting is fudged now, not taking country parameter into account yet
                NOTE: database is questoa2 for command-line (main)
                error if session object not found: advise user to start over? -- currently, it
                  will start over apparently, seqno = 0, but user is not advised (must test and
                  consider how to advise user -- probably needs to be in WebRenderer)
                all System.out: should these be logged instead? (make them debug only?)
                more logging printouts, perhaps + expand the existing ones with useful info (?)
                need better error display -- cf. Venu's way -- esp. when exceptions on oases
 * END HISTORY
 *
 * AGENDA
 *  05/26  mdr  handle exceptions better (more details in messages)
 *              a couple of places simply print error messages -- need to webify / log them
 *              several methods are still stubbed out: endSession, logHistory, save[Un]Resolved
 *              search for 'later' in various comments
 * END AGENDA
 */


/**********************************************************************
 * MyException is a bare-bones exception handler for minor internal LogicAPI errors.
 * @param a message String.
 **********************************************************************/
class MyException extends Exception {
    public MyException(String s) { super(s); }
}


/**********************************************************************
 * The LogicAPI class has methods for logically evaluating a node in the Online
 * Assistant (GetNextNode) and
 * for handling user choices such as SaveResolved, SaveUnresolved, SetHistory and
 * GetHistory -- all coordinated by the main public method, dispatch, based on an
 * OASubmitVars object.  Calls methods in OAEngine class and uses OASessionState object for
 * maintaining values throughout a session.  Variables here are used to help maintain
 * files of session state objects (serialized).  Takes values from OAEngine in the
 * OAReturnVars object.  Takes values from user in the OASubmitVars object.
 * 
 * @author Mike Rychener
 * @version 1.0
 **********************************************************************/
public class LogicAPI {

    private String sessionPath = "/tmp/oa/";
    private String sessionIndex = "LogicAPI.index";
    private Hashtable index = new Hashtable(50);  // will be pairs sessionInd . (Long)timestamp
                                                 // -- (which is serial-obj-file name)
    private Integer curSessionInd = new Integer(1000);
                                 // counter will cycle up to 9999, then jump to 1000
                                 // -- use Integer so hashCode & equals are defined
    private String curSessionFile = "./0.0.0";
    private String curSessionBase = "./0.0";
    private String curSessionKey = "0000";
    private FileUtil myFutil = new FileUtil();
    private String logFile = "OAhistory.log";  // this is the official OA log sent back to authors
    private LogModule  log = null ;   // Logging module for internal diagnostic messages
    private boolean myDebug = false;  // debug (this class) switch
    private Properties countryMapping = null;
    private Properties languageMapping = null;
    private Properties preanswerLists = null;
    private boolean preanswerProc = false;
    private String questDbname = "";
    private String questServer = "";
    private String questPort = "";
    private String questUser = "";
    private String questPasswd = "";
    private boolean dbUseAppDriver = false;
    private String clioDbname = "";
    private String clioServer = "";
    private String clioPort = "";
    private String clioUser = "";
    private String clioPasswd = "";

    /*************************************
     * The LogicAPI constructor uses a given session ID, or finds a new one if first arg is 0,
     * determining the filename of the session or setting up a new one, along
     * with loading or initializing the index hashtable of all sessions.
     * It also takes a sequence number, which is 0 for the first in the session.
     * A third argument is the session path to be used to store objects and index.
     * @param sesno int Session Number
     * @param seqno int Sequence Number within session
     * @param spath String Path where objects and index are stored (writable by web user),
     *   which should come in from a Properties file (loaded by servlet calling this interface),
     *   must begin and end in a separator character for the file system, eg, '/' for AIX.
     * @param cprops Properties a file of associations between country abbreviations and codes
     * @param lprops Properties a file of associations between language abbrev.s and strings
     *************************************/
    public LogicAPI(int sesno, int seqno, String spath, Properties cprops, Properties lprops) {
        // = constructor
        // the only critical section should occur when the index has to be updated with
        //   a new element (several places below are synchronized blocks)
        // curSessionInd (current highest one) will be stored as hash element 0000=curSessionKey
        boolean newIndex = false;

        if (spath != null && spath.length() > 0) sessionPath = spath;
        logFile = sessionPath + logFile;

        String timeStr = getCalDateTime();  // get time stamp
        try {
            LogSystem.beginLogSection("LogicAPI starting/continuing... "+ sesno + "." + seqno 
                                      + " " + timeStr);
            if (myFutil.exists(logFile)) myFutil.appendToFile(logFile, "LogicAPI continuing "
                                                              + sesno + "." + seqno + " "
                                                              + timeStr + "\n");
            else myFutil.saveFile(logFile, "LogicAPI starting " + sesno + "." + seqno + " "
                                  + timeStr + "\n");
        } catch(Exception e) {
//            System.out.println("LogicAPI/timeStr: Log File appendTo/saveFile took Exception.");
            LogSystem.log(1, "LogicAPI/timeStr: Log File appendTo/saveFile took Exception.");
            LogSystem.log(1, e);
        }
        try {
            if (preanswerLists == null)
                preanswerLists = StandardPropertyLoader.load(
                    "com/ibm/nzna/properties/servlet/oa/LogicAPI");
            if (myDebug) {
                if (preanswerLists != null)
//                    System.out.println("LogicAPI init -- " + preanswerLists.size()
//                                       + " properties loaded.");
                LogSystem.log(10, "LogicAPI init -- " + preanswerLists.size()
                                   + " properties loaded.");
                else //System.out.println("LogicAPI init -- NO properties loaded.");
                LogSystem.log(10, "LogicAPI init -- NO properties loaded.");
            }
        } catch(Exception e) {
            try {
                myFutil.appendToFile(logFile,
                                     "LogicAPI/properties: Std. Prop. loader took Exception.");
                LogSystem.log(1, "LogicAPI/properties: Std. Prop. loader took Exception.");
            } catch(Exception e2) {
//                System.out.println("LogicAPI/properties: Std. Prop. loader took Exception.");
                LogSystem.log(1, "LogicAPI/properties: Std. Prop. loader took Exception.");
                LogSystem.log(1, e);
            }
        }
        countryMapping = cprops;
        languageMapping = lprops;
        try {
            FileInputStream ins = new FileInputStream(sessionPath + sessionIndex);
            ObjectInputStream ino = new ObjectInputStream(ins);
            index = (Hashtable)ino.readObject();
            ino.close();
            curSessionInd = (Integer)index.get(curSessionKey);
        } catch(Exception e) {  // usually will be a FileNotFound / ClassNotFound
            if (sesno != 0) {  // this is an error situation: index file missing / unreadable
                curSessionInd = new Integer(sesno);
                index.put(curSessionInd, new Long(System.currentTimeMillis()));  // try re-init
                index.put(curSessionKey, curSessionInd);
//                System.out.println("LogicAPI: Couldn't find the expected index of sessions." +
//                                   "  Attempting to start over on this session.");
                LogSystem.log(1, "LogicAPI: Couldn't find the expected index of sessions." +
                                   "  Attempting to start over on this session.");
                LogSystem.log(1, e);
                try {
                    myFutil.appendToFile(logFile, "Error Encountered - no index of sessions - "
                                         + sesno + "." + seqno + "\n");
                    LogSystem.log(1, "Error Encountered - no index of sessions - "
                                         + sesno + "." + seqno + "\n");
                } catch(Exception e2) {
//                    System.out.println("LogicAPI/index: Log File appendToFile took Exception.");
                    LogSystem.log(1, "LogicAPI/index: Log File appendToFile took Exception.");
                    LogSystem.log(1, e2);
                }
                // here need to write out the index file:
                synchronized (index) {
                  try {
                      FileOutputStream outs = new FileOutputStream(sessionPath + sessionIndex);
                      ObjectOutputStream outo = new ObjectOutputStream(outs);
                      outo.writeObject(index);
                      outo.flush();  // ---------------------- write hashtable to file
                      outo.close();
                  } catch (Exception e2) {
//                      System.out.println("LogicAPI: I/O Error creating new "
//                                         + "index hashtable object (errorIndex): " + sessionIndex
//                                         + " - " + curSessionFile + " - " + e2.toString());
                      LogSystem.log(1, "LogicAPI: I/O Error creating new "
                                         + "index hashtable object (errorIndex): " + sessionIndex
                                         + " - " + curSessionFile + " - " + e2.toString());
                  }
                }  // end synchronize new index
            }
            else newIndex = true;
            // if sesno == 0, will write a new one anyway below, so no further error recov. here
        }  // end try to read in saved hashtable -- at this point, it's an index or a new one
        // -- how know when an existing session times out and can be closed? (later) <<<
        //    (should we check for stale ones every time this starts up?)
        //    (could make a stale list to be processed after the immediate session has cycle)
        if (sesno == 0) {  // create new session
            if (newIndex) {  // create a new index including this new session
                // curSessionInd will be whatever it's initialized to, as a class variable (1000)
              synchronized (index) {
                try {
                index.put(curSessionInd, new Long(System.currentTimeMillis()));
                index.put(curSessionKey, curSessionInd);  // initial current session counter
                FileOutputStream outs = new FileOutputStream(sessionPath + sessionIndex);
                ObjectOutputStream outo = new ObjectOutputStream(outs);
                outo.writeObject(index);
                outo.flush();  // ---------------------- write hashtable to file
                outo.close();
                } catch (Exception e) {
//                    System.out.println("LogicAPI: I/O Error creating new "
//                                       + "index hashtable object (newIndex): " + sessionIndex
//                                       + " - " + curSessionFile + " - " + e.toString());
                    LogSystem.log(1, "LogicAPI: I/O Error creating new "
                                       + "index hashtable object (newIndex): " + sessionIndex
                                       + " - " + curSessionFile + " - " + e.toString());
                    LogSystem.log(1, e);
                    try {
                        myFutil.appendToFile(logFile, "Error Encountered - new index - " + sesno
                                             + "." + seqno + " - " + curSessionFile
                                             + " - " + e.toString() + "\n");
                        LogSystem.log(1, "Error Encountered - new index - " + sesno
                                             + "." + seqno + " - " + curSessionFile
                                             + " - " + e.toString() + "\n");
                        LogSystem.log(1, e);
                    } catch(Exception e2) {
//                        System.out.println("LogicAPI/newIndex: Log File appendToFile took Exception.");
                        LogSystem.log(1, "LogicAPI/newIndex: Log File appendToFile took Exception.");
                        LogSystem.log(1, e2);
                    }
                }
              }  // end synchronize new index
            }  // end create new index
            else {  // update existing index for new session
                // here goes the critical section:
                // multiple threads: the hashtable read in might get updated by others after we read it
                //   so the critical section will have to re-read after it gets it locked, then
                //   write it under lock protection, then release the lock == Java synchronized block
              synchronized (index) {
                try {
                  FileInputStream ins = new FileInputStream(sessionPath + sessionIndex);
                  ObjectInputStream ino = new ObjectInputStream(ins);
                  index = (Hashtable)ino.readObject();
                  ino.close();
                  // update the filed index object for the new session just started
                  curSessionInd = new Integer(((Integer)index.get(curSessionKey)).intValue() + 1);
                  if (index.get(curSessionInd) != null) {
                      // should end the session with that ID, if such exists <<<  (later)
                      // for now, if there's a clash, the old session simply gets deleted
                      String oldbase = curSessionInd.toString() + "."
                                         + ((Long)(index.get(curSessionInd))).toString();
                      int ii = 0;
                      try {
                          for (ii = 0; ii < 9999; ii++) {
                              File oldses =
                                  new File(sessionPath, oldbase + "." + ii);
                              oldses.delete();  // will break when that file doesn't exist
                          }
                      }
                      catch(Exception e) {
//                          System.out.println("LogicAPI: Cleanup ended deleting old session=" + ii);
                          LogSystem.log(1, "LogicAPI: Cleanup ended deleting old session=" + ii);
                          try {
                              myFutil.appendToFile(logFile, "Cleanup - " + curSessionInd.toString()
                                                   + "." + ii + "\n");
                              LogSystem.log(1, "Cleanup - " + curSessionInd.toString()
                                                   + "." + ii + "\n");
                          } catch(Exception e2) {
//                              System.out.println("LogicAPI/cleanup: Log File appendToFile took Exception.");
                              LogSystem.log(1, "LogicAPI/cleanup: Log File appendToFile took Exception.");
                              LogSystem.log(1, e2);
                          }
                      }
                  }
                  index.put(curSessionInd, new Long(System.currentTimeMillis()));
                  index.put(curSessionKey, curSessionInd);  // update the current session counter
                  FileOutputStream outs = new FileOutputStream(sessionPath + sessionIndex);
                  ObjectOutputStream outo = new ObjectOutputStream(outs);
                  outo.writeObject(index);
                  outo.flush();  // ---------------------- write hashtable to file
                  outo.close();
                  // end of critical section (future) -- later -- would unlock at this point
                } catch (Exception e) {
//                    System.out.println("LogicAPI: I/O Error creating new "
//                                       + "index hashtable object (update): " + sessionPath 
//                                       + sessionIndex + " - " + e.toString());
                    LogSystem.log(1, "LogicAPI: I/O Error creating new "
                                       + "index hashtable object (update): " + sessionPath 
                                       + sessionIndex + " - " + e.toString());
                    try {
                        myFutil.appendToFile(logFile, "Error Encountered - new hashtable - "
                                             + sessionPath + sessionIndex + " - " + e.toString()
                                             + "\n");
                        LogSystem.log(1, "Error Encountered - new hashtable - "
                                             + sessionPath + sessionIndex + " - " + e.toString()
                                             + "\n");
                    } catch(Exception e2) {
//                        System.out.println("LogicAPI/updateIndex: Log File appendToFile took Exception.");
                        LogSystem.log(1, "LogicAPI/updateIndex: Log File appendToFile took Exception.");
                        LogSystem.log(1, e2);
                    }
                }
              }  // end synchronized(index)
            }  // end updating index for new session
          try {  // now we must create a new session and write its serialized object to a file
            OASessionState newses = new OASessionState(curSessionInd.intValue());
            // first session state saved will have seq. # = 0; will be read in again by dispatch
            curSessionBase = sessionPath + curSessionInd.toString() + "."
                             + ((Long)(index.get(curSessionInd))).toString();
            curSessionFile = curSessionBase + ".0";
            FileOutputStream outs = new FileOutputStream(curSessionFile);
            ObjectOutputStream outo = new ObjectOutputStream(outs);
            outo.writeObject(newses);
            outo.flush();  // ---------------------- save session object in file
            outo.close();
          } catch (Exception e) {
//              System.out.println("LogicAPI: I/O Error creating new "
//                                 + "saved session state object: " + curSessionFile
//                                 + " - " + e.toString() );
              LogSystem.log(1, "LogicAPI: I/O Error creating new "
                                 + "saved session state object: " + curSessionFile
                                 + " - " + e.toString() );
              try {
                  myFutil.appendToFile(logFile, "Error Encountered - " + curSessionFile
                                       + " - " + e.toString() + "\n");
                  LogSystem.log(1, "Error Encountered - " + curSessionFile
                                       + " - " + e.toString() + "\n");
              } catch(Exception e2) {
//                  System.out.println("LogicAPI/saveNewState: Log File appendToFile took Exception.");
                  LogSystem.log(1, "LogicAPI/saveNewState: Log File appendToFile took Exception.");
                  LogSystem.log(1, e2);
              }
          }  // end try+catch for create new session
        }  // end create new session
        else {
            // this one will actually be recovering data from preceding session in sequence:
            curSessionBase = sessionPath + curSessionInd.toString() + "."
                             + ((Long)(index.get(curSessionInd))).toString();
            curSessionFile = curSessionBase + "." + seqno;
        }
        // note that if an indexed session can't be read from a file, there will be an 
        //   exception thrown later (when session is set up for dispatch), don't need to
        //   check it here, unless error recovery is desired (maybe later)
    }  // end constructor method
        

    /* ************************************
     * abbreviated constructor, allows a default all-writable directory /tmp/oa.
     * ************************************/
    // public LogicAPI(int sesno, int seqno) {  // constructor
       // LogicAPI(sesno, seqno, "/tmp/oa/");
        // note: have different defaults based on site / qit-flag??
    // }  // end constructor 2nd method


    /**********************************************************************
     * The dispatch method breaks down OASubmitVars object to be usable as part of OASessionState
     * and then feeds that into the main interface methods here and in OAEngine;
     * various commands = "GetNextNode", "EndSession", "GetHistory", "LogHistory",
     * "SetHistory", "SaveResolved", and "SaveUnresolved".
     * @param oasub OASubmitVars object with field values from user selection
     * @return OAReturnVars object from OAEngine
     **********************************************************************/
    public OAReturnVars dispatch(OASubmitVars oasub, int symInd) {

        OASessionState oases = null;
        OAReturnVars oaret = null;

        try {
            // -----------------------------
            // restore a saved session state
            // -----------------------------
            if (oasub.command.equals("SetHistory")) {
                // if SetHistory, call processHistory (changes answers, returns oases)
                oases = processHistory(oasub);
            } else {
                // otherwise just read in the current session node
                FileInputStream ins = new FileInputStream(curSessionFile);
                ObjectInputStream ino = new ObjectInputStream(ins);
                oases = (OASessionState)ino.readObject();  // copy in saved session state
                ino.close();
            }
        } catch(Exception e) {
//            System.out.println("LogicAPI: I/O error reading saved session state object: "
//                               + curSessionFile + " - " + e.toString());
            LogSystem.log(1, "LogicAPI: I/O error reading saved session state object: "
                               + curSessionFile + " - " + e.toString());
            try {
                myFutil.appendToFile(logFile, "Error Encountered - reading saved object - "
                                     + curSessionFile + " - " + e.toString() + " - Starting Over\n");
                LogSystem.log(1, "Error Encountered - reading saved object - "
                                     + curSessionFile + " - " + e.toString() + " - Starting Over");
                oases = new OASessionState(curSessionInd.intValue());
                // later? what if oases is null here -- must return, don't go on?
                // or could let null pointer exception happen below and return down there
            } catch(Exception e2) {
//                System.out.println("LogicAPI/readState: Log File appendToFile / new session Exception.");
                LogSystem.log(1, "LogicAPI/readState: Log File appendToFile / new session Exception.");
                LogSystem.log(1, e2);
            }
        }  // end try to restore a saved session state

      try {  // main processing for dispatch
        if (oasub.qitMemory != null) oases.setSourceFlag(true);
        OAEngine oaeo = new OAEngine(questDbname,questServer,questPort,questUser,questPasswd,
                                     oasub.qitMemory,sessionPath,dbUseAppDriver);
        // ----------------------------------------------------
        // here take any new values from oasub and add to oases
        // ----------------------------------------------------
        if (oases.getCurrentSymptom() == 0) {
            if (symInd != 0) oases.setCurrentSymptom(symInd);  // symptom --------
        }
        if (oases.getCurrentNode() == 0) {
            if (oasub.nodeInd != 0) oases.setCurrentNode(oasub.nodeInd);  // node ---------
        }
        if (oasub.sessionSeqNo != 0)  // sequence number -----------
            oases.setCurrentSeqNo(oasub.sessionSeqNo);  // will already be incremented here
            // tail end of this method will write & return seq+1
        if (myDebug)
 //           System.out.println("LogicAPI: QIT_FLAG = " + oases.getSourceFlag()); // DDDEEEBBBUUUGGG
            LogSystem.log(1, "LogicAPI: QIT_FLAG = " + oases.getSourceFlag()); // DDDEEEBBBUUUGGG
        if (oasub.response != 0) oases.setCurrentResponse(oasub.response);  // response ---------
        if (oasub.brand != null && !(oasub.brand.equals(""))) {
            oasub.brand = oasub.brand.replace('+', ' ');             // brand --------
            // while (oasub.brand.indexOf("+") > -1)
            //     oasub.brand = oasub.brand.substring(0, oasub.brand.indexOf("+"))
            //                   + " " + oasub.brand.substring(oasub.brand.indexOf("+") + 1);
            // oasub.brand = RegularExpression.substitute("\\+", " ", oasub.brand); NO GOOD
            oases.setBrand(oasub.brand);
        }
        else throw new MyException("Empty brand.  ");
        if (oasub.docClass != 0) oases.setDocClassInd(oasub.docClass);
        String cocode = "";
        // Country -----------------------------------
        // >>> HERE NEED TO CHECK FOR COUNTRY INPUT PARAMETER ----------------------------- <<<
        // oases.setGeo(oasub.country); -- seems pointless (mdr, 10/09)
        if (oasub.custNum != null && !oasub.custNum.equals("")) {
            oases.setCustNum(Integer.parseInt(oasub.custNum));
            cocode = oaeo.getThreeCharCountryCode(oases,
                             clioDbname,clioServer,clioPort,clioUser,clioPasswd);
        }
        else cocode = "";  // defaults below to USA if no custNum
        if (cocode != null && !cocode.equals("")) {
            if (countryMapping != null) cocode = countryMapping.getProperty(cocode);
            else cocode = "";  // defaults below to USA = 110
            if (cocode == null) cocode = "";
        }
        else cocode = "";
//        if (myDebug) System.out.println("LogicAPI: cocode1 = " + cocode);      // DDDEEEBBBUUUGGG
        if (myDebug) LogSystem.log(1, "LogicAPI: cocode1 = " + cocode);      // DDDEEEBBBUUUGGG
        if (!cocode.equals("")) oases.setCountry(Integer.parseInt(cocode));
        else oases.setCountry(110);  // which is USA
        // Language -----------------------------------
        if (oasub.lang != null) cocode = oasub.lang;
        else cocode = "";
        if (!cocode.equals("")) oaeo.getLanguageIndandSet(cocode, oases);
        else oaeo.getLanguageIndandSet("English", oases);
//        if (myDebug) System.out.println("LogicAPI: cocode2 = " + cocode);      // DDDEEEBBBUUUGGG
        if (myDebug) LogSystem.log(1, "LogicAPI: cocode2 = " + cocode);      // DDDEEEBBBUUUGGG
        // Machine-type-model ------------------------------
        if (!oasub.typeModel.equals("")) {
            if (oasub.typeModel.indexOf("-") > 0 &&
                oasub.typeModel.length() >= (8 + oasub.typeModel.indexOf("-"))) {
                // looks like an Options model: junk before hyphen, then 7 char's
                oasub.typeModel = oasub.typeModel.substring(oasub.typeModel.indexOf("-") + 1);
            }
            if (oasub.typeModel.indexOf("-") < 0 && oasub.typeModel.length() > 4)
                oasub.typeModel = 
                    oasub.typeModel.substring(0, 4) + "-" + oasub.typeModel.substring(4);
            oases.setModel(oasub.typeModel);
        }
        // ----------------------------------------------------
        // fill in some standard preanswers for Brand, Geo, Country, and Type-Model
        // read them in from LogicAPI.properties (which is read in constructor)
        // ----------------------------------------------------
        if (preanswerLists != null && !preanswerProc) {
            preanswerProc = true;
            String brandques = preanswerLists.getProperty(
                "brand." + oasub.brand.replace(' ', '.') + ".question");
            String brandans = preanswerLists.getProperty(
                "brand." + oasub.brand.replace(' ', '.') + ".answer");
            if (brandques != null && brandans != null)
                oases.addPreanswerItem(Integer.parseInt(brandques), Integer.parseInt(brandans));
            if (oasub.country.equals("United States") || oasub.country.equals("Canada")) {
                String geoques = preanswerLists.getProperty("geography.North.America.question");
                String geoans = preanswerLists.getProperty("geography.North.America.answer");
                if (geoques != null && geoans != null)
                    oases.addPreanswerItem(Integer.parseInt(geoques), Integer.parseInt(geoans));
            }
            String suppques = preanswerLists.getProperty(
                "supported." + oasub.brand.replace(' ', '.') + ".question");
            String suppans = preanswerLists.getProperty(
                "supported." + oasub.brand.replace(' ', '.') + ".answer");
            if (suppques != null && suppans != null)
                oases.addPreanswerItem(Integer.parseInt(suppques), Integer.parseInt(suppans));
        }

        if (myDebug) {
            // ---------------------------------
            // display incoming OA Session State (if myDebug)
            // ---------------------------------
//            System.out.println("<p>OA Session State:<br>\n" +                  // DDDEEEBBBUUUGGG
            LogSystem.log(1, "<p>OA Session State:<br>\n" +                  // DDDEEEBBBUUUGGG
                "  Session.Seqno = " + oases.getSessionId() + "." + 
                    oases.getCurrentSeqNo() + "<br>\n" +
                "  Brand = " + oases.getBrand() + "<br>\n" +
                "  Type/Model = " + oases.getModel() + "<br>\n" +
                "  Geo = " + oases.getGeo() + "<br>\n" +
                "  Country Ind = " + oases.getCountry() + "<br>\n" +
                "  Language Ind = " + oases.getLanguage() + "<br>\n" +
                "  Doc Class = " + oases.getDocClassInd() + "<br>\n" +
                "  Response = " + oases.getCurrentResponse() + "<br>\n" +
                "  History size = " + oases.getQuestionHistory().size() + "<br>\n" +
                "  Symptom = " + oases.getCurrentSymptom() + "<br>\n" +
                "  Node = " + oases.getCurrentNode() + "<br>\n" +
                "  Source Flag = " + oases.getSourceFlag() + "<br>");
        }  // end debug
        // -----------------------------------------------------
        myFutil.appendToFile(logFile, "Command = " + oasub.command + " - " + oases.getSessionId()
                             + "." + oases.getCurrentSeqNo() + " - " + oases.getBrand() + ":"
                             + oases.getModel() + " " + oases.getCurrentSymptom() + "/"
                             + oases.getCurrentNode() + "/" + oases.getCurrentResponse() + "\n");
        LogSystem.log(1, "Command = " + oasub.command + " - " + oases.getSessionId()
                             + "." + oases.getCurrentSeqNo() + " - " + oases.getBrand() + ":"
                             + oases.getModel() + " " + oases.getCurrentSymptom() + "/"
                             + oases.getCurrentNode() + "/" + oases.getCurrentResponse() + "\n");
        // -----------------------------------------------------
        // call various methods to actually perform the commands
        // -----------------------------------------------------
        oaret = new OAReturnVars(oases.getSessionId());
        if (oasub.command.equals("GetNextNode")) {
            oaret = getNextNode(oaeo, oases);
        } else if (oasub.command.equals("EndSession")) {
            oaret = endSession(oaeo, oases);
            return oaret;  // return immediately, no need to save new state
        } else if (oasub.command.equals("GetHistory")) {
            oaret = getHistory(oaeo, oases);
        } else if (oasub.command.equals("LogHistory")) {
            oaret = logHistory(oasub.message, oaeo, oases);
        } else if (oasub.command.equals("SetHistory")) {
            oaret = setHistory(oaeo, oases);
        } else if (oasub.command.equals("SaveResolved")) {
            oaret = saveResolved(oaeo, oases);
            return oaret;  // return immediately, no need to save new state
        } else if (oasub.command.equals("SaveUnresolved")) {
            oaret = saveUnresolved(oaeo, oases);
            return oaret;  // return immediately, no need to save new state
        } else {
            oaret = new OAReturnVars(0);
            oaret.addInfo = "LogicAPI: ERROR in command parameter.\n";
        }
      } catch(Throwable t) {
        StringWriter    sw      = new StringWriter ();
        PrintWriter     pw      = new PrintWriter ( sw );

        // ----------------------------------------------
        // set up error return in case exception occurred
        // ----------------------------------------------
        if (oaret == null) try { oaret = new OAReturnVars(oases.getSessionId()); }
                           catch (Exception ee) { oaret = new OAReturnVars(0); }
        oaret.title = "THE ONLINE ASSISTANT ENCOUNTERED AN ERROR";
        oaret.addInfo = "LogicAPI dispatch function received an exception from OA Engine, "
                        + "or had erroneous parameters.  "
                        + "<!-- "
                        + "OA LogicAPI dispatch exception: " + t.toString()
                        + "\nOA LogicAPI dispatch exception detail:" + t.toString() + "\n";
        t.printStackTrace(pw);
        pw.flush();
        oaret.addInfo += sw.getBuffer().toString() + " -->";
        // System.out.println(sw.getBuffer().toString());
        try {
            // these settings fill in potentially missing values from exception:
            oaret.sessionSeqNo = oases.getCurrentSeqNo();
            oaret.nodeInd = oases.getCurrentNode();
            oaret.node_flag = oases.getNodeFlag();
            // here log that an error occurred:
            myFutil.appendToFile(logFile, "Error Encountered - " + oaret.sessionInd
                                 + "." + oaret.sessionSeqNo + " - " + t.toString() + "\n");
            LogSystem.log(1, "Error Encountered - " + oaret.sessionInd
                                 + "." + oaret.sessionSeqNo + " - " + t.toString() + "\n");
        } catch (Exception ee) {
//            System.out.println("LogicAPI/Error: Log File appendToFile took Exception.");            
            LogSystem.log(1, "LogicAPI/Error: Log File appendToFile took Exception.");            
            LogSystem.log(1, ee);
        }
        return oaret;
      }  // end main processing for dispatch
      try {
          // --------------------------------------------------------------------
          // save new session state (with higher seq no for next entry back here)
          // --------------------------------------------------------------------
          curSessionFile = curSessionBase + "." + (oasub.sessionSeqNo + 1);
          oases.setCurrentSeqNo(oasub.sessionSeqNo + 1);
          FileOutputStream outs = new FileOutputStream(curSessionFile);
          ObjectOutputStream outo = new ObjectOutputStream(outs);
          outo.writeObject(oases);  // write out saved session state
          outo.flush();
          outo.close();
          oaret.sessionSeqNo = oases.getCurrentSeqNo();  // bump returned seqNo
          if (myDebug) 
//            System.out.println("LogicAPI: increased SeqNo to " + oases.getCurrentSeqNo());
            LogSystem.log(1, "LogicAPI: increased SeqNo to " + oases.getCurrentSeqNo());
      } catch(Exception e) {
          // leave oases as its default allocation, done above
//          System.out.println("LogicAPI: I/O Error saving session state object: "
//                             + curSessionFile + " - " + e.toString());
          LogSystem.log(1, "LogicAPI: I/O Error saving session state object: "
                             + curSessionFile + " - " + e.toString());
          try {
              myFutil.appendToFile(logFile, "Error saving state - " + curSessionFile
                                   + " - " + e.toString() + "\n");
              LogSystem.log(1, "Error saving state - " + curSessionFile
                                   + " - " + e.toString() + "\n");
          } catch(Exception e2) {
//              System.out.println("LogicAPI/saveState: Log File appendToFile took Exception.");
              LogSystem.log(1, "LogicAPI/saveState: Log File appendToFile took Exception.");
              LogSystem.log(1, e2);
          }
      }  // end try to save session state object (seqNo++)

      return oaret;
    }  // end dispatch


    /**********************************************************************
     * Create a string with current time stamp.
     *
     **********************************************************************/
    public String getCalDateTime() {
        Calendar currdate = Calendar.getInstance();
        String timeStr = currdate.get(Calendar.YEAR)
                         + ((currdate.get(Calendar.MONTH) < 9) ? "-0" : "-")
                         // add 1 to month due to Java convention:
                         + (currdate.get(Calendar.MONTH)+1)
                         + ((currdate.get(Calendar.DATE) < 10) ? "-0" : "-")
                         + currdate.get(Calendar.DATE) + " "
                         + currdate.get(Calendar.HOUR_OF_DAY)
                         + ((currdate.get(Calendar.MINUTE) < 10) ? ":0" : ":")
                         + currdate.get(Calendar.MINUTE)
                         + ((currdate.get(Calendar.SECOND) < 10) ? ":0" : ":")
                         + currdate.get(Calendar.SECOND);
        return timeStr;
    }
    

    /* ---------- next in alphabetical order are the dispatch command methods ---------- */


    /**********************************************************************
     * Terminates the current session, cleaning up session objects, etc.
     *
     **********************************************************************/
    public OAReturnVars endSession(OAEngine oaeo, OASessionState oases)
      throws Exception {
        OAReturnVars oaret = oaeo.displayHistory(oases);
        // webRenderer will handle user confirmation, links, etc.
        try {
            myFutil.appendToFile(logFile, "EndSession - " + oases.getSessionId()
                                 + "." + oases.getCurrentSeqNo() + " - " + oases.getBrand() + ":"
                                 + oases.getModel() + " " + oases.getCurrentSymptom() + "/"
                                 + oases.getCurrentNode() + "/" + oases.getCurrentResponse()
                                 + " - " + getCalDateTime() + "\n");
            LogSystem.log(1, "EndSession - " + oases.getSessionId()
                                 + "." + oases.getCurrentSeqNo() + " - " + oases.getBrand() + ":"
                                 + oases.getModel() + " " + oases.getCurrentSymptom() + "/"
                                 + oases.getCurrentNode() + "/" + oases.getCurrentResponse()
                                 + " - " + getCalDateTime() + "\n");
        } catch(Exception e2) {
//            System.out.println("LogicAPI/endSession: Log File appendToFile took Exception.");
            LogSystem.log(1, "LogicAPI/endSession: Log File appendToFile took Exception.");
            LogSystem.log(1, e2);
        } // end try append to log
        try {
            int ext = 0;
            File objf = new File(curSessionBase + ".0");  // now clean up session object files:
            while (objf.exists()) {
                objf.delete();
                ext++;
                objf = new File(curSessionBase + "." + ext);
            }
        } catch(Exception e2) {
//            System.out.println("LogicAPI/endSession: Cleanup Objects took Exception.");
            LogSystem.log(1, "LogicAPI/endSession: Cleanup Objects took Exception.");
            LogSystem.log(1, e2);
        }

        return oaret;
    }  // end endSession


    /**********************************************************************
     * Gets a list of questions and answers in the current session.
     *
     **********************************************************************/
    public OAReturnVars getHistory(OAEngine oaeo, OASessionState oases)
      throws Exception {
        OAReturnVars oaret = oaeo.displayHistory(oases);
        return oaret;
    }  // end getHistory


    /**********************************************************************
     * Adds a string from the message variable of OASubmitVars to the OA Log.
     *
     **********************************************************************/
    public OAReturnVars logHistory(String msg, OAEngine oaeo, OASessionState oases)
      throws Exception {
        OAReturnVars oaret = new OAReturnVars(oases.getSessionId());
        try {
            myFutil.appendToFile(logFile, "LogHistory - " + oases.getSessionId()
                                 + "." + oases.getCurrentSeqNo() + " - " + oases.getBrand() + ":"
                                 + oases.getModel() + " " + msg
                                 + " - " + getCalDateTime() + "\n");
            LogSystem.log(1,  "LogHistory - " + oases.getSessionId()
                                 + "." + oases.getCurrentSeqNo() + " - " + oases.getBrand() + ":"
                                 + oases.getModel() + " " + msg
                                 + " - " + getCalDateTime() + "\n");
        } catch(Exception e2) {
//            System.out.println("LogicAPI/logHistory: Log File appendToFile took Exception.");
            LogSystem.log(1, "LogicAPI/logHistory: Log File appendToFile took Exception.");
            LogSystem.log(1, e2);
        } // end try append to log
        return oaret;
    }  // end logHistory


    /**********************************************************************
     * Gets a new node by following up on flowchart logic coded by authors based
     * on session answers up to the current node, on various conditions, and
     * on new input from the user.
     *
     **********************************************************************/
    public OAReturnVars getNextNode(OAEngine oaeo, OASessionState oases)
      throws Exception {
        OAReturnVars oaret = null;
        // oases.setCurrentNode(newNode); not needed
        oaret = oaeo.processNode(oases);
        return oaret;
    }  // end getNextNode


    /**********************************************************************
     * Ends a successful online assistant session.
     *
     **********************************************************************/
    public OAReturnVars saveResolved(OAEngine oaeo, OASessionState oases)
      throws Exception {
        OAReturnVars oaret = endSession(oaeo, oases);
        // now add any additional message before returning, current symptom title:
        oaret.title = oaeo.getSymptomTitleAsString(oases);
        return oaret;
    }  // end saveResolved


    /**********************************************************************
     * Ends an unsuccessful online assistant session, saving what happened for
     * later analysis by authors.
     *
     **********************************************************************/
    public OAReturnVars saveUnresolved(OAEngine oaeo, OASessionState oases)
      throws Exception {
        OAReturnVars oaret = endSession(oaeo, oases);
        // now add any additional message before returning, current symptom title:
        oaret.title = oaeo.getSymptomTitleAsString(oases);
        return oaret;
    }  // end saveUnresolved


    /**********************************************************************
     * Changes an answer in the current session's history, as a starting point
     * for continuing the session in a new direction.
     *
     **********************************************************************/

    public OAReturnVars setHistory(OAEngine oaeo, OASessionState oaes) throws Exception {

        // note that processHistory was called above to set up the past session state oaes
        OAReturnVars oars = getNextNode(oaeo, oaes);
        return oars;
    }  // end setHistory

    
    /* ---------- ---------- end of dispatch command methods ---------- ---------- */


    /*********************************************************
    * Goes back in history to revive a past session,
    * when the user calls for a change in the answers.
    * The appropriate session state object is read
    * from a file that was saved in a previous session step.
    *
    * @param seqNo: the sequence number of the question that the
    *               user wants to change (obtained from OASubmitVars)
    * @return oas: the new OASessionState object with the changed
    *              data
    *********************************************************/
    public OASessionState processHistory(OASubmitVars oasub) throws Exception{

        OASessionState oas      = null;
        String oldSessionFile   = ""; 
        
        try {  // restore a saved session state
            oldSessionFile          = sessionPath + oasub.sessionInd + "."
                + ((Long)(index.get(new Integer(oasub.sessionInd)))).toString()
                + "." + oasub.sessionSeqNo;
            FileInputStream ins     = new FileInputStream(oldSessionFile);
            ObjectInputStream ino   = new ObjectInputStream(ins);
            oas = (OASessionState)ino.readObject();  // copy in saved session state
            oasub.sessionSeqNo -= 1;
            oas.setCurrentSeqNo(oasub.sessionSeqNo);  // set state back to previous sequence,
                                                      //   will end up writing over same object
            if (oasub.sessionSeqNo == 0) oas.setCurrentNode(0);  // reset node to start over
            else {
                oasub.response = oas.getPreviousResponse();  // pull in previous one, will be set
                oasub.nodeInd = oas.getPreviousNode();
                oas.setCurrentNode(oas.getPreviousNode());
//                if (myDebug) System.out.println("LogicAPI: found previous response: " + oasub.response
//                                                + " and node: " + oasub.nodeInd);
                if (myDebug) LogSystem.log(1, "LogicAPI: found previous response: " + oasub.response
                                                + " and node: " + oasub.nodeInd);
            }  // else
            ino.close();
        } catch(Exception e) {
            // later warn user here that we're starting over due to error . . .
//            System.out.println("LogicAPI: I/O Error reading "
//                               + "session state object from history: " + oldSessionFile + " - "
//                               + e.toString());
            LogSystem.log(1, "LogicAPI: I/O Error reading "
                               + "session state object from history: " + oldSessionFile + " - "
                               + e.toString() + " - Starting Over");
            myFutil.appendToFile(logFile, "Error Encountered - reading from history - "
                                 + oldSessionFile + " - " + e.toString() + " - Starting Over\n");
            oas = new OASessionState(oasub.sessionInd);  // this might throw an exception?
        } // catch

        return oas;

    }  // end processHistory


    /*********************************************************
     * sets database connection variables for QUEST database
     *********************************************************/
    public void setQuestNames(String dbname,String server,String port,String user,String passwd) {
        questDbname = dbname;
        questServer = server;
        questPort = port;
        questUser = user;
        questPasswd = passwd;
        dbUseAppDriver = false;  // default is to NOT use App Driver (net driver more reliable?)
    }  // end setQuestNames


    /*********************************************************
     * sets database connection variables for QUEST database
     *********************************************************/
    public void setQuestNames(String dbname, String server, String port, 
                              String user, String passwd, boolean appDriver) {
        questDbname = dbname;
        questServer = server;
        questPort = port;
        questUser = user;
        questPasswd = passwd;
        dbUseAppDriver = appDriver;
    }  // end setQuestNames


    /*********************************************************
     * sets database connection variables for CLIO database
     *********************************************************/
    public void setClioNames(String dbname,String server,String port,String user,String passwd) {
        clioDbname = dbname;
        clioServer = server;
        clioPort = port;
        clioUser = user;
        clioPasswd = passwd;
        dbUseAppDriver = false;  // default is to NOT use App Driver (net driver more reliable?)
    }  // end setClioNames


    /*********************************************************
     * sets database connection variables for CLIO database
     *********************************************************/
    public void setClioNames(String dbname, String server, String port,
                             String user, String passwd, boolean appDriver) {
        clioDbname = dbname;
        clioServer = server;
        clioPort = port;
        clioUser = user;
        clioPasswd = passwd;
        dbUseAppDriver = appDriver;
    }  // end setClioNames


    /**********************************************************************
     * main test program (accessible from command line) -- one call executes one
     * logic cycle of the engine.
     * @param args String array, each being parameter=value pair
     **********************************************************************/
    public static void main(String[] args) throws Exception {

      // to function as a CGI program, would take parameters & form a OASubmitVars object
      // -- implement as a servlet when that is appropriate?

      long start = System.currentTimeMillis();
      try {
         
          // EVENTUAL TEST SEQUENCE:

          // sample sequence (cf. interfaces.txt):
          // symptom 1 -> displayQuestion (step 1 in processSymptom)
          // response -> processNode (step 2 in processSymptom)

          /* processNode comes back with addinfo == string separated by various tags, eg,
              <OAActionTitle>, <OAQuestionTitle>, <OAAnswerTitle>,  . . .
              */
          /* initial startup test: 
          OASessionState sest = new OASessionState(1001);
          sest.setCurrentNode(150044);
          sest.setCurrentSymptom(140012);
          OASubmitVars subs = null;
          OAReturnVars s1 = oaeo.processNode(sest);
          System.out.println(s1.toString());
          */

          int ises = 0;
          int ssym = 0;
          int iseq = 0;
          // create submitvars object and fill in default values:
          for (int ii = 0; ii < args.length; ii++) {
              if (args[ii].length() < 3) continue;
              if (!(args[ii].substring(0,3).equals("ses")) && 
                  !(args[ii].substring(0,3).equals("seq"))) continue;
              if (args[ii].indexOf("=") < 0) continue;  // arg error
              // need to pick out the sessionInd & sessionSeqNo parameters first:
              if (args[ii].substring(0,3).equals("ses"))
                  ises = Integer.parseInt(args[ii].substring(1 + args[ii].indexOf("=")));
              else
                  iseq = Integer.parseInt(args[ii].substring(1 + args[ii].indexOf("=")));
          }  // end for loop
          LogicAPI oala = new LogicAPI(ises, iseq, "/tmp/oa/", null, null);
          // set up defaults for databases:
          oala.setClioNames("CLIO","CHAOS","5703","esupport","ESUPP0RT");  // was 5701
          oala.setQuestNames("QUESTOA2","CHAOS","5703", "quest", "j0hnny");  // was 5400
          // OAEngine oaeo = new OAEngine(oala.questDbname,oala.questServer,oala.questPort,
          //  -- not used in main --  oala.questUser,oala.questPasswd,null,oala.sessionPath);
          OASubmitVars svar = new OASubmitVars(ises);
          // here would read in saved session data
          svar.command = "GetNextNode";  // these are DEFAULTS specific to this testing period
          svar.node_flag = "DQ";
          svar.sessionSeqNo = iseq;
          if (args.length == 0) {
              System.out.println(
                  "Usage: LogicAPI parameter=value . . .\n" +
                  "where parameters are:\n" +
                  "  command = GetNextNode,GetHistory,SetHistory,SaveResolved,or SaveUnresolved\n" +
                  "  brand = PC brand, eg, IBM ThinkPad\n" +
                  "  typeModel = 4 + 3 character string, hyphen optional, or Options model (longer)\n" +
                  "  country = 2 + '_' + 2 language_country string\n" +
                  "  docClass = 1, 2, or 3 depending on site (enduser, partner, internal)\n" +
                  "  node = node indicator (integer)\n" +
                  "  nflag = node-flag (AA, AQ, AS, DA, DQ, DS)\n" +
                  "  response = answer indicator (integer)\n" +
                  "  message = additional values / parameters (string)\n" +
                  "  sessionind = session indicator (integer)\n" +
                  "  seqno = sequence within session (integer)\n" +
                  "  (qflag can only be false)\n" +
                  "  symptom = symptom indicator (integer)\n" +
                  "Parameter names can be abbreviated with the first 3 letters.\n" +
                  "Eg, LogicAPI com=GetNextNode nfl=DQ res=130567 nodeInd=150210"
                  );
              return;
          }
          String parstr = "";
          String valstr = "";
          for (int ii = 0; ii < args.length; ii++) {
              // split into parstr, valstr . . . + make sure at least 3 char's
              if (args[ii].length() < 3) continue;
              parstr = args[ii].substring(0, args[ii].indexOf("="));
              if (parstr.length() < 3) continue;  // argument error
              parstr = parstr.substring(0,3);
              if (args[ii].indexOf("=") < 0) continue;  // argument without '='
              valstr = args[ii].substring(1 + args[ii].indexOf("="));
              if (valstr.length() < 1) continue;  // argument error
              if (parstr.equals("ses") || parstr.equals("seq")) continue;  // handled above
              if (parstr.equals("com"))      svar.command = valstr;
              else if (parstr.equals("bra")) svar.brand = valstr;
              else if (parstr.equals("typ")) svar.typeModel = valstr;
              else if (parstr.equals("cou")) svar.country = valstr;
              else if (parstr.equals("cus")) svar.custNum = valstr;
              else if (parstr.equals("doc")) svar.docClass = Integer.parseInt(valstr);
              else if (parstr.equals("nod")) svar.nodeInd = Integer.parseInt(valstr);
              else if (parstr.equals("nfl")) svar.node_flag = valstr;
              else if (parstr.equals("res")) svar.response = Integer.parseInt(valstr);
              else if (parstr.equals("mes")) svar.message = valstr;
              else if (parstr.equals("sym")) ssym = Integer.parseInt(valstr);
          }
          // for debugging, just read in properties from current directory
          // FileInputStream inprop = new FileInputStream("LogicAPI.properties");
          // preanswerLists = StandardPropertyLoader.load(inprop);  obsolete usage

//          System.out.println(svar.toString());  // echo the input
          LogSystem.log(1, svar.toString());  // echo the input
          // now interpret the command:  (dispatch reads saved session state from file)
          OAReturnVars rvar = oala.dispatch(svar, ssym);
//          System.out.println(rvar.toString());  // print the result
          LogSystem.log(1, rvar.toString());  // print the result
          // later, will be converting / expanding the html and displaying to browser
          
      } catch(Throwable t) {
//        System.out.println("OA LogicAPI main exception: " + t.toString()
//                           + "\nOA LogicAPI main exception detail:" + t.getMessage());
        LogSystem.log(1, "OA LogicAPI main exception: " + t.toString()
                           + "\nOA LogicAPI main exception detail:" + t.getMessage());
//        t.printStackTrace();
        return;
      }

      long end = System.currentTimeMillis();
//      System.out.println("Run time: " + (end-start) + " msec");
      LogSystem.log(1, "Run time: " + (end-start) + " msec");
      return;
   
    }  // end main (args)

}  // end class LogicAPI

