/* Object submitted to OA methods from user or higher-level methods */

package com.ibm.nzna.projects.oa.engine;
import java.io.*;
import java.util.*;
import com.ibm.nzna.projects.common.quest.oa.*;

/*********************************************************************
 * Variables that are passed from user back to LogicAPI, where they are
 * transferred into the main OA Session State object.
 *
 *********************************************************************/

public class OASubmitVars implements Serializable,Cloneable {

    /** command takes the values "GetNextNode", "EndSession", "GetHistory", "LogHistory",
      "SetHistory", "SaveResolved", and "SaveUnresolved".
     **/    
    public String command = "GetNextNode";
    /** brand takes values as they appear in the database, eg, "IBM PC", "IBM ThinkPad" **/
    public String brand = "";
    /** typeModel is a 7-character string without hyphens, eg, "6587LGX" -- as in Products table **/
    public String typeModel = "";
    /** country name selected by user or taken from lang parameter (eg, en_US) up the line **/
    public String country = "";
    /** language name selected by user or taken from lang parameter (eg, en_US) up the line **/
    public String lang = "";  // added by mdr 10/09 
    /** custNum is a string of digits, eg, "629600" **/
    public String custNum = "";
    /** docClass is a number (site): 1 (IBM internal), 2 (Business Partner), or 3 (end-user)  **/
    public int docClass = 0;
    public int nodeInd = 0;
    public String node_flag = "AQ";
    /** response is an answer indicator, 0 if just starting out (just asking a question) **/
    public int response = 0;
    /** message is used for logging URL links actually followed by user - LogHistory command. **/
    public String message = "";
    /** sessionInd is internally generated by LogicAPI -- start out with value 0 **/
    public int sessionInd = 0;
    /** sessionSeqNo is internally generated and incremented step by step -- 0 means unknown **/
    public int sessionSeqNo = 0;
    // OUT (23 Aug) public boolean qit_flag = false;  // phased out in favor of qitMemory
    /** qitMemory is an instance of a Quest Input Tool class with methods for accessing memory,
      or null if not in the Quest Input Tool, ie, on the Web **/
    public OAMemoryInterface qitMemory = null;  // mdr added 3 Aug. for QIT memory objects access

    public OASubmitVars(int sessid) {
        // dummy object creation
        this.sessionInd = sessid;
    }

    /**
     * Returns a string representation of this object suitable
     * for display on an HTML capable device.
     *
     * @return String An HTML String representation of this object.
     **/
    public String toString() {

        String sret = "<p><p><b>Submit Vars Object = </b>" + "<br>\n";
        sret += "  Command: " + command + "<br>\n";
        sret += "  Brand: " + brand + "<br>\n";
        sret += "  Type-model: " + typeModel + "<br>\n";
        sret += "  Country: " + country + "<br>\n";
        sret += "  Language: " + lang + "<br>\n";
        sret += "  Customer number: " + custNum + "<br>\n";
        sret += "  Doc-Class: " + docClass + "<br>\n";
        sret += "  Node Indicator: " + nodeInd + "<br>\n";
        sret += "  Node Flag: " + node_flag + "<br>\n";
        sret += "  Response: " + response + "<br>\n";
        sret += "  Message: " + message + "<br>\n";
        sret += "  Session Indicator: " + sessionInd + "<br>\n";
        sret += "  Session Sequence #: " + sessionSeqNo + "<br>\n";
        sret += "  QIT interface: " + ((qitMemory != null) ? "yes" : "no") + "<br>\n";

        return sret;
    }

}  // end OASubmitVars class

