/*-----------------------------------------------------------*/
/*   Export                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Export                              */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
package com.ibm.nzna.projects.oa.export;
import java.io.*;
import java.sql.*;
import java.util.* ;
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.oa.*;
import java.net.*;
import com.ibm.nzna.shared.ftp.*;
import com.ibm.nzna.shared.ftp.FTP.*;
//import dk.eos.net.*;
//import dk.eos.net.ftp.*;

/**
 * This is the class that exports Online Assistant database 
 *  information into XML format.<br><br>
 *
 * @version 1.0, 05/17/2000 
 * @author  Tracey Watkins
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   Export                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class Export extends Batch implements Runnable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static int       LANG            =   0;  
   private static int       QUEST           =   1;
   private static int       CDATETYPE       =   1;
   private BatchLogSystem   batchLog        =   null;
   private String           dbName          =   null;
   private String           serverName      =   null;
   private String           portNumber      =   null;
   private boolean          appDriverFlag   =   false;
   private String           outputDir       =   null;
   private Vector           ftpSite         =   new Vector();
   private String           userId          =   null;
   private String           password        =   null;
   private boolean          fullExportFlag  =   false;
   private String           timeDelta       =   null;
   private String           outputType      =   null;
   private String           brand           =   null;
   private String           classification  =   null;
   private String           geography       =   null;
   private Vector           fileNames       =   new Vector();
   private int              symptomCounter  =   0;
   private Vector           symptoms        =   new Vector();
   private Vector           nodes           =   new Vector();
   private Vector           actions         =   new Vector();
   private Vector           questions       =   new Vector();
   private Vector           edges           =   new Vector();
   private Vector           expLinks        =   new Vector();
   private Thread           exportThread    =   null;
   private boolean          processFlag     =   false;
   private CDate            todayDate       =   new CDate(CDATETYPE);


   /*------------------------------------------------------------*/
   /* Export
    */
   /**
    *    This method is the only constructor
    *  for this batch
    *  
    *  @param args  String of command line arguments.
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Export ( String[] args ) {
      
       //Initialize the BatchLogSystem
       batchLog = new BatchLogSystem( "Export", this );
       batchLog.batchStart();
       batchLog.beginLogSection("Online Assistant Export Batch: ");
       
       logProgress("Initialization of batch & Reading of parameters.", 13, 1);
       
       //Read in the command line parameters
       readParameters( args );

       //Display current command line parameters
       displayParameters();

       exportThread = new Thread(this);

       //Batch Stop
       batchLog.batchStop();
   }


/*------------------------------------------------------------*/
   /* readParameters
    */
   /**
    *    This method should parse through the given parameters
    *  
    *
    *  @param args Command line arguments as an array of String
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readParameters ( String[] args ) {
       System.out.println("Reading Command Line Parameters. # of params read : " + args.length);
       logProgress("Reading command line parameters.  # of parameters read: " + args.length, 13, 1);
       String parameter     =   null; 
       String parameterUpper=   null;

       if ( args.length == 0 ) {
           exportHelp();
           System.exit(1);
       }
       for ( int i=0; i < args.length; i++) {
           parameter = args[i];
           parameterUpper = parameter.toUpperCase();
           if ( parameterUpper.startsWith("/D:") ) {
               dbName = parameter.substring(3);
           }
           else if (parameterUpper.startsWith("/S:")) {
               serverName = parameter.substring(3);
           }
           else if (parameterUpper.startsWith("/P:")) {
               portNumber = parameter.substring(3);
           }
           else if (parameterUpper.equals("/APPDRIVER")) {
               appDriverFlag = true;
           }
           else if (parameterUpper.startsWith("/OUT:")) {
                outputDir = parameter.substring(5);
           }
           else if (parameterUpper.startsWith("/FTP:")) {
               ftpSite.addElement(parameter.substring(5));
           }
           else if (parameterUpper.startsWith("/USER:")) {
               userId = parameter.substring(6);
           }
           else if (parameterUpper.startsWith("/PASSWORD:")) {
               password = parameter.substring(10);
           }
           else if (parameterUpper.equals("/FULLEXPORT")) {
               fullExportFlag = true;
           }
           else if (parameterUpper.startsWith("/BRAND:")) {
               brand = parameter.substring(7);
               fullExportFlag = false;
           }
           else if (parameterUpper.startsWith("/CLASS:")) {
               classification = parameter.substring(7);
               fullExportFlag = false;
           }
           else if (parameterUpper.startsWith("/GEO:")) {
               geography = parameter.substring(5);
               fullExportFlag = false;
           }
           else if (parameterUpper.startsWith("/CHANGEDTIME:")) {
               timeDelta = parameter.substring(13);
               }
           else if (parameterUpper.startsWith("/EXPORT:")) {
               outputType = parameter.substring(8);
           }
           else if (  parameter.startsWith("/?") || parameter.startsWith("?") )
           {
               exportHelp();
               System.exit(1);
           }
           else
           {
                System.out.println("Unknown Parameter:  " + parameter);  
           }
       } //for

       // Default delta to yesterday if no changedtime is passed.
       if ( timeDelta == null) {
           timeDelta = todayDate.subtractDaysFromDate(todayDate.today(), 1, todayDate.dateType);
       }

       //Exits if required parameters are not passed.
       if ( dbName==null || (appDriverFlag==false && (serverName==null ||
                                                      portNumber==null)) ||
            userId==null || password==null || outputDir==null ) {
           exportHelp();
           System.exit(1);
       }


       // Default to full export if no brand, classification or geography passed.
       if ( (brand == null) && (classification == null) && (geography == null) ) {
           fullExportFlag = true;
       }

       // Default output type to XML if no type is passed.
       if ( outputType == null) {
           outputType = "XML";
       }

       System.out.println("Exiting readParameters");
   } //readParameters


   /*------------------------------------------------------------*/
   /* exportHelp
    */
   /**
    *    This method prints out a help message for the parameter
    *  usage for Export.
    *  
    *
    *  @param  None
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void exportHelp () {
      System.out.println("\nUsage:  java Export {/D:database-name /S:server_name \n" +
                         "/P:port-number /OUT:output-directory /FTP:ftp-site \n" +
                         "/USER:user-id /PASSWORD:password /EXPORT:export-type} \n\n");
      System.out.println("\n Parameter      Description");
      System.out.println(" --------------- --------------------------------------");
      System.out.println(" /D:          The database name to export.                ");
      System.out.println("              Example:  /D:QUEST");
      System.out.println(" /S:          The server name that the database resides.");
      System.out.println("              Example:    /S:CHAOS");
      System.out.println(" /P:          The port number of JDBC.");
      System.out.println("              Example:    /P:5703");
      System.out.println(" /APPDRIVER   If this parameter is passed, the batch ");
      System.out.println("              shall use the App Driver instead of the ");
      System.out.println("              default Net Driver.");
      System.out.println("              Example:    /APPDRIVER");
      System.out.println(" /OUT:        The output directory to place the exported ");
      System.out.println("              files.");
      System.out.println("              Example:    /OUT:/quest/oa/ ");
      System.out.println(" /FTP:        The FTP site to place the files.  This ");
      System.out.println("              parameter will be sectioned by semicolons.");
      System.out.println("              Example:");
      System.out.println("              /FTP:testsvr1.raleigh.ibm.com;/home/testid/oaexport;testid;testpw ");
      System.out.println("              /FTP:servername;directory;user;password");
      System.out.println(" /USER:       The user id to connect to the database    ");
      System.out.println("              with. ");
      System.out.println("              Example:    /USER:DBTESTID");
      System.out.println(" /PASSWORD:   The password for the user id.   ");
      System.out.println("              Example:    /PASSWORD:DBTESTPW");
      System.out.println(" /FULLEXPORT  If this parameter is passed, the batch shall  ");
      System.out.println("              produce a full export. All brands, classifications ");
      System.out.println("              and geographies passed will be ignored. ");
      System.out.println("              Example:    /FULLEXPORT");
      System.out.println(" /BRAND       If this parameter is passed, the batch shall  ");
      System.out.println("              produce an export for the specified Brand. ");
      System.out.println("              Example:    /BRAND:290");
      System.out.println(" /CLASS       If this parameter is passed, the batch shall  ");
      System.out.println("              produce an export for the specified Classification. ");
      System.out.println("              Example:    /CLASS:Unclassified");
      System.out.println(" /GEO         If this parameter is passed, the batch shall  ");
      System.out.println("              produce an export for the specified Geography. ");
      System.out.println("              Example:    /GEO:WorldWide");
      System.out.println(" /CHANGEDTIME:If this parameter is passed, the batch ");
      System.out.println("              shall do a delta export using this time as");
      System.out.println("              the delta. ");
      System.out.println("              Example:    /CHANGEDTIME:2000-01-07.00.00.000000");
      System.out.println(" /EXPORT:     This parameter defines whay type of output");
      System.out.println("              is needed.  The default type is XML. ");
      System.out.println("              Example:    /EXPORT:XML");
      System.out.println(" /?           This parameter displays this help message. \n");
   } //exportHelp
   
   
   /*------------------------------------------------------------*/
   /* startBatch
    */
   /**
    *    This method calls the startBatch in Batch
    *  
    *
    *  @param  None
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
    public void startBatch() {
        processFlag = true;
        exportThread.start();
    } //startBatch
   
    
   /*------------------------------------------------------------*/
   /* stopBatch
    */
   /**
    *    This method calls the stopBatch in Batch
    *  
    *
    *  @param  None
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
    public void stopBatch() {
        processFlag = false;
        exportThread.stop();
    } //stopBatch
    
    
    /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method contains the major processing for the Export batch.
    *  
    *
    *  @param  None
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

       boolean hasResults = false;
       ResultSet rs;
       try
       {
           if (processFlag == false) {
               return;
           }
           if (appDriverFlag) {
               System.out.println("if(appDriverFlag)");
               Class.forName( "COM.ibm.db2.jdbc.app.DB2Driver");
               System.out.println("Driver ok");
               new DatabaseSystem(true, batchLog, true);
               System.out.println("DB System object created");
               portNumber = "0";
//               password = null;
//               dbName = "";
               serverName = "";
//               userId = null;
           }
           else {
               new DatabaseSystem(true, batchLog, false);
           }
//           if (DatabaseSystem.createConnection(DatabaseSystem.QUEST, dbName, serverName, portNumber, 
//                                               userId, password) == false) {
           DatabaseSystem.createConnection(DatabaseSystem.QUEST, dbName, serverName, portNumber, 
                                                   userId, password);
//               logError("Database connection failed...!  Program terminated!");
//               System.out.println("Database connection failed...!  Program terminated!");
//               processFlag = false;
//               this.stopBatch();
//           }

           //Find all symptoms to Export
           System.out.println("Database connection successful.");
           logProgress("Database connection successful.", 13, 2);
           SQLMethod sqlMethod = new SQLMethod(1, "Export", 30);
           System.out.println("Db connection id = " + sqlMethod.getDatabaseId());
           logProgress("Db connection id = " + sqlMethod.getDatabaseId(), 13, 2);
           Statement statement = sqlMethod.createStatement();
           System.out.println("Statement creation successful.");
           logProgress("Statement creation successful.", 13, 2);

           String qry = "select distinct a.symptomind from oa.symptom a, oa.symptomcountry b ";
           qry = qry + " where a.changedtime >= '" + timeDelta + "' ";
           if ( fullExportFlag == false ) {
                if (!(brand == null) && fullExportFlag==false) {
                    qry = qry + " and a.brandind = " + brand ;
                }
                else if (!(classification == null) && fullExportFlag == false) {
                    qry = qry + " and a.docclassind = " + classification;
                }
                else if (!(geography == null) && fullExportFlag == false) {
                    qry = qry + " and b.countryind = " + geography;
                }
          }

           //Execute the query and get the changed symptoms
           System.out.println("The query qry:  " + qry);
           logProgress("Query executed:", 13, 2);

            rs = statement.executeQuery(qry);
                if ( !rs.next() ) {
                    hasResults = false;
                    statement.close();
                } 
                else {
                    hasResults = true;

                    System.out.println("Getting next result.");
                    logProgress("Getting next result.", 13, 3);
                    
                    symptoms.addElement(new Symptom(rs.getInt(1)));

                    while ((rs.next())) {
                        System.out.println("Getting next result.");
                        logProgress("Getting next result.", 13, 3);
                        symptoms.addElement(new Symptom(rs.getInt(1)));
                    } //while
                    
                    symptomCounter = symptoms.size();
                    
                    for ( int i=0; i < symptomCounter; i++) {
                        System.out.println("Num of symptoms:  " + symptoms.size());
                        logProgress("Number of changed symptoms:" + symptoms.size(), 13, 3);
                        symptoms.setElementAt(processSymptom((Symptom)symptoms.elementAt(i)), i);
                        logProgress("Symtom " + (Symptom)symptoms.elementAt(i) + " processed.", 13, 6);
                    } //for

                      statement.close();
                  } //else


           // Shutdown the DatabaseSystem object
           DatabaseSystem.shutdown();
           System.out.println("DatabaseSystem shutdown successful.");
           logProgress("DatabaseSystem shutdown successful.", 13, 7);

           //Create the XML output files.
           System.out.println("SymptomCounter:  " + symptomCounter);
           
           for ( int i=0; i < symptoms.size(); i++) {
               toXML((Symptom)symptoms.elementAt(i));
               logProgress("Output generated for symptom: " + symptoms.elementAt(i), 13, 5);
           } //for

           System.out.println("Current List of Files:  ");
           logProgress("Curent list of files:", 13, 6);
           for ( int i=0; i<fileNames.size(); i++) {
               if ( !( fileNames.elementAt(i)== null )) {
                   System.out.println(fileNames.elementAt(i) + "\n"); 
                   logProgress((String)fileNames.elementAt(i), 13, 6);
                } //if
           } //for

           // Ftp files to ftp sites.
           if (!fileNames.isEmpty()) {
               ftpFiles();
               logProgress("Files have been ftped.", 13, 12);
           }

           batchLog.endLogSection();
       } //try
       catch (Exception e) {
           logSevereError(e);
           processFlag = false;
       }
       
       //end batch
//       this.stopBatch();
       logProgress("Batch stopped.", 13, 13);
       System.exit(0);
   } //run


   /*------------------------------------------------------------*/
   /* processSymptom(Symptom)
    */
   /**
    *    This method processes the current symptom.
    *  
    *
    *  @param  symp            Symptom
    *  
    *  @return sympProcessed   Boolean
    *//*---------------------------------------------------------*/
   public Symptom processSymptom (Symptom symp) {
       System.out.println("Reading current symptom from db.");
       logProgress("Reading current symptom from db.", 13, 4);
       try{
           
           if ( !(symp == null) ) {
               
               //Reading symptom information from database
               symp = symp.readFromDatabase( symp.getInd(), LANG);
               System.out.println(symp.toString() + " was read.");
               
               //Getting all associated nodes.
               System.out.println("Getting all associated nodes...");
               nodes = symp.getNodes();
               Node n;
               short nType;
               
               for (int cnt=0; cnt<nodes.size(); cnt++) {
                   System.out.println("Node : " + cnt + " is " + nodes.elementAt(cnt));
                   n = (Node)nodes.elementAt(cnt);
                   n = (Node)symp.getNodeByInd(n.getInd());
                   System.out.println("n is " + n);

                   nType = n.getNodeType();
                   if ( n.isOAObjectNode() ) {
                       switch ( nType ) {
                       case n.NODE_TYPE_ACTION:  
                           System.out.println("This node is an Action.");
                           processAction(n);
                           break;
                       case n.NODE_TYPE_QUESTION:
                           System.out.println("This node is a Question.");
                           processQuestion(n);
                           break;
                       case n.NODE_TYPE_SYMPTOM:
                           System.out.println("This node is a child Symptom.");
                           processChildSymptom(n);
                           break;
                       default:  break; 
                       } //switch
                   } //if 
                   
              } //for

               //Getting all associated edges.
               Vector edgeGroups = new Vector();
               edgeGroups = symp.getEdgeGroups();
               System.out.println("edgeGroup size " + edgeGroups.size());

               for ( int cnt=0; cnt<edgeGroups.size(); cnt++) {
                   if ( !( edgeGroups.elementAt(cnt)== null ) ) {
                       System.out.println("edgeGroup " +cnt+ " was not null.");
                       EdgeGroup eGroup = (EdgeGroup)edgeGroups.elementAt(cnt);
                       System.out.println(eGroup);
                       Vector edgeVector = new Vector();
                       edgeVector = eGroup.getEdges();
                           for (int i=0; i<edgeVector.size(); i++) {
                               if ( !(edgeVector.elementAt(i) == null) ) {
                                   Edge e = (Edge)edgeVector.elementAt(i);
                                   System.out.println("edge: " + e);
                                   processEdge(e);
                               } //if

                           } //for

                       } //if
                   else {
                       System.out.println("edgeGroup: " + edgeGroups.elementAt(cnt) + " was null.");
                   } //else
               } //for
           
           } //if
           else{
               System.out.println("Symptom " + symp.getInd() +
                                  "was null");
           }
       } //try
       catch (Exception e) {
         System.out.println("Exception:  " + e);
         logSevereError(e);
         System.out.println("Problem reading and processing all of symptom " + 
                            symp.getInd() + " from the database.");
       } //catch
       
       return symp;

   } //processSymptom(Symptom)


   /*------------------------------------------------------------*/
   /* processChildSymptom(Node)
    */
   /**
    *    This method processes the current child Symptom.
    *  
    *
    *  @param  n              Node
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void processChildSymptom(Node n) {

       System.out.println("In method processChildSymptom.");
       
       Symptom s = new Symptom(n.getOAObjectInd());
       Symptom symp = new Symptom();
       System.out.println("Current symptoms.size: " + symptoms.size());
       for (int i=0; i<symptoms.size(); i++) {
           symp = (Symptom)symptoms.elementAt(i);

           if (s.getInd() == symp.getInd()) {
               System.out.println("Symptom: " + s.getInd() + 
                                  " is already in the symptoms Vector.");
               return;
           } //if

        } //for

        System.out.println("New childSympNode created: " + s.getInd());
        symptoms.addElement(s);
        s = processSymptom(s);
        symptoms.setElementAt(s, symptoms.size()-1);

   } //processChildSymptom(Node)


   /*------------------------------------------------------------*/
   /* processAction(Node)
    */
   /**
    *    This method processes the current action.
    *  
    *
    *  @param  n              Node
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void processAction (Node n) {
       System.out.println("In method processAction.");
       logProgress("Reading current action from database.", 13, 4);

       Action actNode = new Action(n.getOAObjectInd());
       Action act = new Action();
       System.out.println("Current actions.size: " + actions.size());
       for (int i=0; i<actions.size(); i++) {
           act = (Action)actions.elementAt(i);

           if (actNode.getInd() == act.getInd()) {
               System.out.println("Action: " + actNode.getInd() + 
                                  " is already in the actions Vector.");
               return;
           } //if

       } //for

           System.out.println("New actNode created: " + actNode.getInd());
           actions.addElement(actNode);

           Vector actionDocLinks = new Vector();
           actNode  = actNode.readFromDatabase( actNode.getInd(), LANG);
           if (actNode == null) {
               boolean actionRemoved = false;
               actionRemoved = actions.removeElement(actNode);
               return;
           }
           else {
               System.out.println("Action: " + actNode);
               actions.setElementAt(actNode, actions.size()-1);
               actionDocLinks = actNode.getDocLinks();
               processDocLinks(actionDocLinks);
               toXML(actNode);
           } //else
   } //processAction(Node)


   /*------------------------------------------------------------*/
   /* processDocLinks(Vector)
    */
   /**
    *    This method processes the current vector of DocLinks.
    *  
    *
    *  @param  docLinks       DocLink
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void processDocLinks (Vector docLinks) {
       System.out.println("In method processDocLinks.");
       logProgress("Reading current docLink from database.", 13, 4);
       DocLink dLink;
       LinkGroup lg;
       Vector links = new Vector();
       System.out.println("docLinks size " + docLinks.size());
       for ( int cnt=0; cnt<docLinks.size(); cnt++) {
           if ( !( docLinks.elementAt(cnt)== null ) ) {
               System.out.println("docLink " +cnt+ " was not null.");
               dLink = (DocLink)docLinks.elementAt(cnt);
               System.out.println(dLink);
               if ( dLink.hasLinkGroup() ) {
                   lg = new LinkGroup(dLink.getLinkGroupInd());
                   lg = lg.readFromDatabase(lg.getInd(), LANG);
                   links = lg.getLinks();
                   for (int i=0; i<links.size(); i++) {
                       if ( !(links.elementAt(i) == null) ) {
                           Link l = (Link)links.elementAt(i);
                           System.out.println("link: " + l);
                           Vector lTitles = l.getLinkTitles();
                           System.out.println("lTitles.size() in DocLinks: " + lTitles.size());

                           processLink(l);
                       } //if
                   } //for
               } //if
               else {
                   processLink(dLink.getLinkInd());
               } //else
           } //if
           else {
               System.out.println("docLinks: " + docLinks.elementAt(cnt) + " was null.");
           } //else
       } //for
  } //processDocLinks(Vector)


  /*------------------------------------------------------------*/
  /* processLink(Link)
   */
  /**
   *    This method processes the current Link.
   *  
   *
   *  @param  link           Link
   *  
   *  @return Nothing
   *//*---------------------------------------------------------*/
   public void processLink (Link link) {
       System.out.println("In method processLinks(link).");
       logProgress("Reading current link from database.", 13, 4);

       if (link==null) {
          return;
       }
       else{
           System.out.println("Link: " + link);
           System.out.println("Link Ind: " + link.getInd());
           link = Link.readFromDatabase(link.getInd(), LANG);
           if (link == null) {
               System.out.println("Link was null.");
               return;
           }
           else {
               Vector lTitles = link.getLinkTitles();
               System.out.println("# Titles: " + lTitles.size());
               toXML(link);
           }

//           if (link instanceof QuestLink) {
//               System.out.println("Link is instanceof QuestLink");
//               QuestLink qLink = new QuestLink(link.getInd());
//               qLink.readFromDatabase(qLink.getInd(), LANG);
//               Vector qLTitles = qLink.getLinkTitles();
//               System.out.println("# Titles: " + qLTitles.size());
//               toXML(qLink);
//           }
//           else if (link instanceof ExternalLink) {
//               System.out.println("Link is instanceof ExternalLink");
//               ExternalLink eLink = new ExternalLink(link.getInd());
//               eLink.readFromDatabase(eLink.getInd(), LANG);
//               if (eLink == null) {
//                   System.out.println("eLink is null");
//                   return;
//               }
//               else {
//                   Vector eLTitles = eLink.getLinkTitles();
//                   System.out.println("# Titles: " + eLTitles.size());
//                   toXML(eLink);
//               } //else
//           } //else if
//           else {
//               System.out.println("Unknown Link type");
//               return;
//           }
       } //else

   } //processLink(Link)


  /*------------------------------------------------------------*/
  /* processLink(int)
   */
  /**
   *    This method processes the current Link.
   *  
   *
   *  @param  ind         int
   *  
   *  @return Nothing
   *//*---------------------------------------------------------*/
   public void processLink (int ind) {
       System.out.println("In method processLinks(int).");
       Link link = Link.readFromDatabase(ind, LANG);
       if (link==null) {
          return;
       }
       else{
           System.out.println("Link: " + link);
           processLink(link);
       } //else
   } //processLink(int)


   /*------------------------------------------------------------*/
   /* processQuestion(Node)
    */
   /**
    *    This method processes the current Question.
    *  
    *
    *  @param  n               Node
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void processQuestion (Node n) {
       System.out.println("In method processQuestion.");
       logProgress("Reading current quesiton from database.", 13, 4);

       Question quesNode = new Question(n.getOAObjectInd());
       Question q = new Question();
       System.out.println("Current questions.size: " + questions.size());
       for (int i=0; i<questions.size(); i++) {
           q = (Question)questions.elementAt(i);
           if (quesNode.getInd() == q.getInd()) {
               System.out.println("Question: " + quesNode.getInd() + 
                                  " is already in the questions Vector.");
               return;
           } //if
       } //for
           System.out.println("New quesNode created: " + quesNode.getInd());
           questions.addElement(quesNode);
           quesNode  = quesNode.readFromDatabase( quesNode.getInd(), LANG);
           if (quesNode == null) {
               boolean questionRemoved = false;
               questionRemoved = questions.removeElement(quesNode);
               return;
           }
           else {
               System.out.println("Question: " + quesNode);
               questions.setElementAt(quesNode, questions.size()-1);
               Vector questionDocLinks = new Vector();
               questionDocLinks = quesNode.getDocLinks();
               processDocLinks(questionDocLinks);
               toXML(quesNode);
           } //else
   } //processQuestion(Node)


   /*------------------------------------------------------------*/
   /* processQuestion(Node)
    */
   /**
    *    This method processes the current Question.
    *  
    *
    *  @param  n               Node
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void processEdge (Edge edge) {
       System.out.println("In method processEdge.");
       logProgress("Reading current edge from database.", 13, 4);
       System.out.println("Current edges.size: " + edges.size());
       for (int i=0; i<edges.size(); i++) {
           Edge e = (Edge)edges.elementAt(i);
           if (edge.getInd() == e.getInd()) {
               System.out.println("edge: " + edge.getInd() + 
                                  " is already in the edges Vector.");
               return;
           } //if

       } //for

           System.out.println("New edge created: " + edge.getInd());
           edges.addElement(edge);
           if (edge == null) {
               boolean edgeRemoved = false;
               edgeRemoved = edges.removeElement(edge);
               return;
           }
           else {
               System.out.println("Edge: " + edge);
               edges.setElementAt(edge, edges.size()-1);
               toXML(edge);
           } //else

   } //processEdge(Edge)


   /*------------------------------------------------------------*/
   /* displayParameters
    */
   /**
    *    This method displays all of the current paramters passed
    *  to the Export.
    *  
    *
    *  @param  None
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void displayParameters () {
       if ( !(dbName == null) ) {
           System.out.println("/D: " + dbName); 
       }
       if (!( serverName == null )) {
           System.out.println("/S: " + serverName); 
       }
       if ( ! ( portNumber == null )) {
           System.out.println("/P: " + portNumber); 
       }
       System.out.println("/APPDRIVER " + appDriverFlag); 
       if ( ! ( outputDir == null )) {
           System.out.println("/OUT: " + outputDir); 
       }
       for ( int i=0; i<ftpSite.size(); i++) {
           if ( !( ftpSite.elementAt(i)== null )) {
               System.out.println("/FTP: " + ftpSite.elementAt(i)); 
            } //if
       } //for
       if ( ! ( userId == null )) {
           System.out.println("/USER: " + userId); 
       }
       if ( ! ( password == null )) {
           System.out.println("/PASSWORD: " + password); 
       }
           System.out.println("/FULLEXPORT: " + fullExportFlag); 
       if ( ! ( timeDelta == null)) {
           System.out.println("/CHANGEDTIME: " + timeDelta); 
       }
       if ( ! ( outputType == null )) {
           System.out.println("/EXPORT: " + outputType); 
       }
       if ( ! ( brand == null )) {
           System.out.println("/BRAND: " + brand); 
       }
       if ( ! ( classification == null )) {
           System.out.println("/CLASS: " + classification); 
       }
       if ( ! ( geography == null )) {
           System.out.println("/GEO: " + geography); 
       }
   } //displayParameters


   /*------------------------------------------------------------*/
   /* ftpFiles
    */
   /**
    *    This method ftps all of the output files to each of the specified
    *   ftp sites.
    *  
    *
    *  @param  None
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void ftpFiles () {
       System.out.println("Entering method ftpFiles()");
       logProgress("Ftping files...", 13, 8);
       File oFile;
       Ftp ftp = new Ftp();
       boolean append = false;
       String sep = null;
       System.out.println("Which sep? " + outputDir);
       System.out.println("Index of " + outputDir.indexOf("/"));

       try {
           for ( int i=0; i<ftpSite.size(); i++) {
               if ( !( ftpSite.elementAt(i)== null )) {
                   StringTokenizer strTok = new StringTokenizer((String)ftpSite.elementAt(i), ";");
                   System.out.println("ftpSite: " + i + " " + ftpSite.elementAt(i));
                   String ftpServer = strTok.nextToken(); 
                   String ftpDir =  strTok.nextToken();
                   System.out.println("Which sep? " + outputDir);
                   System.out.println("Index of " + outputDir.indexOf("/"));
                   if (outputDir.endsWith("/") || outputDir.endsWith("\\"))  {
                   }
                   else 
                   {  
                       if (outputDir.indexOf("/") >= 0) {
                           sep = "/";
                       }
                       else {
                           sep = "\\" ;
                       }
                   }
                   System.out.println("outputDir:  " + outputDir + "\n Sep: " + sep);
                   String ftpUser = strTok.nextToken();
                   String ftpPW = strTok.nextToken();
                   System.out.println("ftpServer: " + ftpServer); 
                   System.out.println("ftpDir: " + ftpDir); 
                   System.out.println("ftpUser: " + ftpUser); 
                   System.out.println("ftpPW: " + ftpPW);
                   ftp.connect( ftpServer );
                   System.out.println("Connected to : " + ftpServer);
                   System.out.println(ftp.getReply());
                   ftp.login( ftpUser, ftpPW );
                   System.out.println("     as user: " + ftpUser);
                   System.out.println(ftp.getReply());
                   logProgress("Connected/logged into ftp site: " + ftpServer, 13, 9);
                   ftp.cd(ftpDir);
                   System.out.println("     cd to dir: " + ftpDir);
                   System.out.println(ftp.getReply());
                   logProgress("Changed to directory: " + ftpDir, 13, 10);

                   for ( int j=0; j<fileNames.size(); j++) {
                       if ( !( fileNames.elementAt(j)== null )) {
                           System.out.println("Directory: "  + outputDir);
                           System.out.println("File: " + fileNames.elementAt(j));
                           ftp.put( outputDir + sep + (String)fileNames.elementAt(j), append); 
                           System.out.println(ftp.getReply());
                           System.out.println("FTPed file " + fileNames.elementAt(j) +
                                              " to server " + ftpServer); 
                           logProgress("File: " + fileNames.elementAt(j) + " has been ftped.", 13, 11);
                       } //if  j
                   } //for j
                   ftp.logout();
                   ftp.disconnect();
                   logProgress("Disconnected from ftp site.", 13, 12);

                } //if i
           } //for i
       }//try
       catch (Exception e) {
           System.out.println("Exception while Ftping files:  " + e);
       } //catch
   } //ftpFiles


   /*------------------------------------------------------------*/
   /* toXML(Symptom)
    */
   /**
    *    This method should parse its input and output
    *  in XML format.
    *  
    *
    *  @param symptom Symptom to parse into XML format
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void toXML (Symptom symp) {
       SymptomTitle sTitle = new SymptomTitle();
       Vector sympTitles = symp.getSymptomTitles();
       Vector countries;
       String s = "<!-- Online Assistant Symptom --> \n";
       s = s + "<SYMPTOM> \n";
       s = s + "    <IND>" + symp.getInd() + "</IND> \n";
       s = s + "    <OABRAND>" + symp.getBrand() + "</OABRAND> \n";
       countries = symp.getCountryList();
       s = s + "    <COUNTRIES>";
       if (! (countries.isEmpty() || countries.firstElement()==null) ) 
           s = s + countries.firstElement();
       for ( int j=1; j<countries.size(); j++) {
           if ( !( countries.elementAt(j)== null )) {
               s = s + ", " + countries.elementAt(j);
           } //if j
       } // for j
       s = s + "</COUNTRIES>\n";
       s = s + "    <DOCCLASS>" + symp.getDocClass() + "</DOCCLASS> \n";
       for ( int cnt=0; cnt<sympTitles.size(); cnt++) {
           if ( !( sympTitles.elementAt(cnt)== null )) {
               sTitle = (SymptomTitle)sympTitles.elementAt(cnt);
               s = s + "    <TITLE> \n";
               s = s + "        <IND>" + sTitle.getInd() + "</IND> \n";
               s = s + "        <TEXT>" + sTitle.getTitle() + "</TEXT> \n";
               countries = sTitle.getCountryList();
               s = s + "        <COUNTRIES>";
               if (! (countries.isEmpty() || countries.firstElement()==null) ) 
                   s = s + countries.firstElement();
               for ( int j=1; j<countries.size(); j++) {
                   if ( !( countries.elementAt(j)== null )) {
                       s = s + ", " + countries.elementAt(j);
                   } //if j
               } // for j
               s = s + "</COUNTRIES>\n";
               s = s + "        <DOCCLASS>" + sTitle.getDocClass() + "</DOCCLASS> \n";
               s = s + "        <KEYWORDS>" + sTitle.getKeywords() + "</KEYWORDS> \n";
               s = s + "        <LANGUAGES>" + sTitle.getLanguage() + "</LANGUAGES> \n";
               s = s + "    </TITLE> \n";
            } //if cnt
       } //for cnt
       s = s + "</SYMPTOM> \n";
       try {
           System.out.println(symp.toString());
           File outFile = new File( outputDir, "SYMPTOM_"+symp.getInd()+".XML" );
           FileWriter fileOut = new FileWriter(outFile);
           System.out.println("outFile: " + outFile.toString());
           fileNames.addElement(outFile.getName());
           logProgress("File name: " + outFile.toString() + " has been added to vector.", 13, 6);
           fileOut.write(s, 0, s.length());
           fileOut.close();
           logProgress("File name: " + outFile.toString() + " has been created.", 13, 5);
      } //try
      catch (Exception e) {
           logSevereError(e);
           System.out.println("Exception: " + e);
      } //catch
   } //toXML(Symptom)


   /*------------------------------------------------------------*/
   /* toXML(Edge)
    */
   /**
    *    This method should parse its input and output
    *  in XML format.
    *  
    *
    *  @param edge Edge to parse into XML format
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void toXML (Edge edge) {
       Action act = new Action();
       Vector actionList = edge.getActionList();
       Vector countries;
       String s = "<!-- Online Assistant Edge --> \n";
       s = s + "<EDGE> \n";
       s = s + "    <IND>" + edge.getInd() + "</IND> \n";
       countries = edge.getCountryList();
       s = s + "    <COUNTRIES>";
       if (! (countries.isEmpty() || countries.firstElement()==null) ) 
           s = s + countries.firstElement();
       for ( int j=1; j<countries.size(); j++) {
           if ( !( countries.elementAt(j)== null )) {
               s = s + ", " + countries.elementAt(j);
           } //if j
       } // for j
       s = s + "</COUNTRIES>\n";
       s = s + "    <DOCCLASS>" + edge.getDocClass() + "</DOCCLASS> \n";
//       s = s + "    <HEADNODE>" + edge.getDocClass() + "</HEADNODE> \n";
//       s = s + "    <TAILNODE>" + edge.getDocClass() + "</TAILNODE> \n";
       if ( edge.isAnswerEdge() ) {
           s = s + "    <ANSWER> \n";
           s = s + "        <IND>" + edge.getAnswerInd() + "</IND> \n";
           s = s + "        <QUESTION> \n";
           s = s + "            <IND>" + edge.getQuestionInd() + "</IND> \n";
           s = s + "        </QUESTION> \n";
           s = s + "    </ANSWER> \n";
        } //if cnt
       for ( int cnt=0; cnt<actionList.size(); cnt++) {
           if ( !( actionList.elementAt(cnt)== null )) {
               act = (Action)actionList.elementAt(cnt);
               s = s + "    <ACTION> \n";
               s = s + "        <IND>" + act.getInd() + "</IND> \n";
               s = s + "    </ACTION> \n";
            } //if cnt
       } //for cnt
       s = s + "</EDGE> \n";
       try {
           System.out.println(edge.toString());
           File outFile = new File( outputDir, "EDGE_"+edge.getInd()+".XML" );
           FileWriter fileOut = new FileWriter(outFile);
           System.out.println("outFile: " + outFile.toString());
           fileNames.addElement(outFile.getName());
           logProgress("File name: " + outFile.toString() + " has been added to vector.", 13, 6);
           fileOut.write(s, 0, s.length());
           fileOut.close();
           logProgress("File name: " + outFile.toString() + " has been created.", 13, 5);
      } //try
      catch (Exception e) {
           logSevereError(e);
           System.out.println("Exception: " + e);
      } //catch
   } //toXML(Edge)


   /*------------------------------------------------------------*/
   /* toXML(Link)
    */
   /**
    *    This method should parse its input and output
    *  in XML format.
    *  
    *
    *  @param link Link to parse into XML format
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void toXML (Link link) {
       System.out.println("In method toXML(Link)");
       LanguageText lText = new LanguageText();
       Vector langText = link.getLinkTitles();
       System.out.println("LinkTitle size= " + langText.size());
       String s = "<!-- Online Assistant Document Link --> \n";
       s = s + "<LINK> \n";
       s = s + "    <IND>" + link.getInd() + "</IND> \n";
       for ( int cnt=0; cnt<langText.size(); cnt++) {
           if ( !( langText.elementAt(cnt)== null )) {
               lText = (LanguageText)langText.elementAt(cnt);
               s = s + "    <TITLE> \n";
//               s = s + "        <IND>" + lText.getInd() + "</IND> \n";
               s = s + "        <TEXT>" + lText.getText() + "</TEXT> \n";
//                 s = s + "        <DOCCLASS>" + lText.getDocClass() + "</DOCCLASS> \n";
               s = s + "        <LANGUAGE>" + lText.getLanguage() + "</LANGUAGE> \n";
               s = s + "    </TITLE> \n";
            } //if cnt
       } //for cnt
       s = s + "</LINK> \n";
       try {
           System.out.println(link.toString());
           File outFile = new File( outputDir, "LINK_"+link.getInd()+".XML" );
           FileWriter fileOut = new FileWriter(outFile);
           System.out.println("outFile: " + outFile.toString());
           fileNames.addElement(outFile.getName());
           logProgress("File name: " + outFile.toString() + " has been added to vector.", 13, 6);
           fileOut.write(s, 0, s.length());
           fileOut.close();
           logProgress("File name: " + outFile.toString() + " has been created.", 13, 5);
      } //try
      catch (Exception e) {
           logSevereError(e);
           System.out.println("Exception: " + e);
      } //catch
   } //toXML(Link)


   /*------------------------------------------------------------*/
   /* toXML(Action)
    */
   /**
    *    This method should parse its input and output
    *  in XML format.
    *  
    *
    *  @param action Action to parse into XML format
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void toXML (Action act) {
       Title aTitle = new Title();
       Vector actTitles = act.getTitles();
       AddtlInfo aInfo = new AddtlInfo();
       Vector addInfo = act.getAdditionalInfo();
       Vector countries;
       String s = "<!-- Online Assistant Action --> \n";
       s = s + "<ACTION> \n";
       s = s + "    <IND>" + act.getInd() + "</IND> \n";
       s = s + "    <OABRAND>" + act.getBrand() + "</OABRAND> \n";
       countries = act.getCountryList();
       s = s + "    <COUNTRIES>";
       if (! (countries.isEmpty() || countries.firstElement()==null) ) 
           s = s + countries.firstElement();
       for ( int j=1; j<countries.size(); j++) {
           if ( !( countries.elementAt(j)== null )) {
               s = s + ", " + countries.elementAt(j);
           } //if j
       } // for j
       s = s + "</COUNTRIES>\n";
       s = s + "    <DOCCLASS>" + act.getDocClass() + "</DOCCLASS> \n";
       for ( int cnt=0; cnt<actTitles.size(); cnt++) {
           if ( !( actTitles.elementAt(cnt)== null )) {
               aTitle = (Title)actTitles.elementAt(cnt);
               s = s + "    <TITLE> \n";
               s = s + "        <IND>" + aTitle.getInd() + "</IND> \n";
               s = s + "        <TEXT>" + aTitle.getTitle() + "</TEXT> \n";
               countries = aTitle.getCountryList();
               s = s + "        <COUNTRIES>";
               if (! (countries.isEmpty() || countries.firstElement()==null) ) 
                   s = s + countries.firstElement();
               for ( int j=1; j<countries.size(); j++) {
                   if ( !( countries.elementAt(j)== null )) {
                       s = s + ", " + countries.elementAt(j);
                   } //if j
               } // for j
               s = s + "</COUNTRIES>\n";
               s = s + "        <DOCCLASS>" + aTitle.getDocClass() + "</DOCCLASS> \n";
               s = s + "        <LANGUAGE>" + aTitle.getLanguage() + "</LANGUAGE> \n";
               s = s + "    </TITLE> \n";
            } //if cnt
       } //for cnt
       for ( int cnt=0; cnt<addInfo.size(); cnt++) {
           if ( !( addInfo.elementAt(cnt)== null )) {
               aInfo = (AddtlInfo)addInfo.elementAt(cnt);
               s = s + "    <ADDTLINFO> \n";
               s = s + "        <IND>" + aInfo.getInd() + "</IND> \n";
               s = s + "        <TEXT>" + aInfo.getAddtlInfo() + "</TEXT> \n";
               countries = aInfo.getCountryList();
               s = s + "        <COUNTRIES>";
               if (! (countries.isEmpty() || countries.firstElement()==null) ) 
                   s = s + countries.firstElement();
               for ( int j=1; j<countries.size(); j++) {
                   if ( !( countries.elementAt(j)== null )) {
                       s = s + ", " + countries.elementAt(j);
                   } //if j
               } // for j
               s = s + "</COUNTRIES>\n";
               s = s + "        <DOCCLASS>" + aInfo.getDocClass() + "</DOCCLASS> \n";
               s = s + "        <LANGUAGE>" + aInfo.getLanguage() + "</LANGUAGE> \n";
               s = s + "    </ADDTLINFO> \n";
           } //if cnt
       } //for cnt
       s = s + "</ACTION> \n";
       try {
           System.out.println("Action: " + act.toString());
           File outFile = new File( outputDir, "ACTION_"+act.getInd()+".XML" );
           FileWriter fileOut = new FileWriter(outFile);
           System.out.println("outFile: " + outFile.toString());
           fileNames.addElement(outFile.getName());
           logProgress("File name: " + outFile.toString() + " has been added to vector.", 13, 6);
           fileOut.write(s, 0, s.length());
           fileOut.close();
           logProgress("File name: " + outFile.toString() + " has been created.", 13, 5);
      } //try
      catch (Exception e) {
          logSevereError(e);
          System.out.println("Exception: " + e);
      } //catch
   } //toXML(Action)


   /*------------------------------------------------------------*/
   /* toXML(Question)
    */
   /**
    *    This method should parse its input and output
    *  in XML format.
    *  
    *
    *  @param question Question to parse into XML format
    *  
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void toXML (Question ques) {
       Title qTitle = new Title();
       Vector quesTitles = ques.getTitles();
       AddtlInfo aInfo = new AddtlInfo();
       Vector addInfo = ques.getAdditionalInfo();
       Answer ans = new Answer();
       Vector answers = ques.getAnswers();
       Title aTitle = new Title();
       Vector ansTitles = new Vector();
       Vector countries;
       String s = "<!-- Online Assistant Question --> \n";
       s = s + "<QUESTION> \n";
       s = s + "    <IND>" + ques.getInd() + "</IND> \n";
       s = s + "    <OABRAND>" + ques.getBrand() + "</OABRAND> \n";
       countries = ques.getCountryList();
       s = s + "    <COUNTRIES>";
       if (! (countries.isEmpty() || countries.firstElement()==null) ) 
           s = s + countries.firstElement();
       for ( int j=1; j<countries.size(); j++) {
           if ( !( countries.elementAt(j)== null )) {
               s = s + ", " + countries.elementAt(j);
           } //if j
       } // for j
       s = s + "</COUNTRIES>\n";
       s = s + "    <DOCCLASS>" + ques.getDocClass() + "</DOCCLASS> \n";
       
       //Question title info
       for ( int cnt=0; cnt<quesTitles.size(); cnt++) {
           if ( !( quesTitles.elementAt(cnt)== null )) {
               qTitle = (Title)quesTitles.elementAt(cnt);
               s = s + "    <TITLE> \n";
               s = s + "        <IND>" + qTitle.getInd() + "</IND> \n";
               s = s + "        <TEXT>" + qTitle.getTitle() + "</TEXT> \n";
               countries = qTitle.getCountryList();
               s = s + "        <COUNTRIES>";
               if (! (countries.isEmpty() || countries.firstElement()==null) ) 
                   s = s + countries.firstElement();
               for ( int j=1; j<countries.size(); j++) {
                   if ( !( countries.elementAt(j)== null )) {
                       s = s + ", " + countries.elementAt(j);
                   } //if j
               } // for j
               s = s + "</COUNTRIES>\n";
               s = s + "        <DOCCLASS>" + qTitle.getDocClass() + "</DOCCLASS> \n";
               s = s + "        <LANGUAGE>" + qTitle.getLanguage() + "</LANGUAGE> \n";
               s = s + "    </TITLE> \n";
            } //if cnt
       } //for cnt
       
       //Question additional info 
       for ( int cnt=0; cnt<addInfo.size(); cnt++) {
          if ( !( addInfo.elementAt(cnt)== null )) {
             aInfo = (AddtlInfo)addInfo.elementAt(cnt);
             s = s + "    <ADDTLINFO> \n";
             s = s + "        <IND>" + aInfo.getInd() + "</IND> \n";
             s = s + "        <TEXT>" + aInfo.getAddtlInfo() + "</TEXT> \n";
             countries = aInfo.getCountryList();
             s = s + "            <COUNTRIES>";
             if (! (countries.isEmpty() || countries.firstElement()==null) ) 
                 s = s + countries.firstElement();
             for ( int j=1; j<countries.size(); j++) {
                 if ( !( countries.elementAt(j)== null )) {
                     s = s + ", " + countries.elementAt(j);
                 } //if j
             } // for j
             s = s + "</COUNTRIES>\n";
             s = s + "        <DOCCLASS>" + aInfo.getDocClass() + "</DOCCLASS> \n";
             s = s + "        <LANGUAGE>" + aInfo.getLanguage() + "</LANGUAGE> \n";
             s = s + "    </ADDTLINFO> \n";
          } //if cnt
       } //for cnt
       
       //Question answer info
       for ( int cnt=0; cnt<answers.size(); cnt++) {
          if ( !( answers.elementAt(cnt)== null )) {
             ans = (Answer)answers.elementAt(cnt);
             ans = ques.getAnswerByInd(ans.getInd());
             ansTitles = ans.getTitles();
             s = s + "    <ANSWER> \n";
             s = s + "        <IND>" + ans.getInd() + "</IND> \n";
             countries = ans.getCountryList();
             s = s + "        <COUNTRIES>";
             if (! (countries.isEmpty() || countries.firstElement()==null) ) 
                 s = s + countries.firstElement();
             for ( int j=1; j<countries.size(); j++) {
                 if ( !( countries.elementAt(j)== null )) {
                     s = s + ", " + countries.elementAt(j);
                 } //if j
             } // for j
             s = s + "</COUNTRIES>\n";
             s = s + "        <DOCCLASS>" + ans.getDocClass() + "</DOCCLASS> \n";
             
             //Answer title info
             for ( int i=0; i<ansTitles.size(); i++) {
                 if ( !( ansTitles.elementAt(i)== null )) {
                     aTitle = (Title)ansTitles.elementAt(i);
                     s = s + "        <TITLE> \n";
                     s = s + "            <IND>" + aTitle.getInd() + "</IND> \n";
                     s = s + "            <TEXT>" + aTitle.getTitle() + "</TEXT> \n";
                     countries = aTitle.getCountryList();
                     s = s + "            <COUNTRIES>";
                     if (! (countries.isEmpty() || countries.firstElement()==null) ) 
                         s = s + countries.firstElement();
                     for ( int j=1; j<countries.size(); j++) {
                         if ( !( countries.elementAt(j)== null )) {
                             s = s + ", " + countries.elementAt(j);
                         } //if j
                     } // for j
                     s = s + "</COUNTRIES>\n";
                     s = s + "            <DOCCLASS>" + aTitle.getDocClass() + "</DOCCLASS> \n";
                     s = s + "            <LANGUAGE>" + aTitle.getLanguage() + "</LANGUAGE> \n";
                     s = s + "        </TITLE> \n";
                  } //if i
             } //for i
             s = s + "    </ANSWER> \n";
          } //if cnt
       } //for cnt
       s = s + "</QUESTION> \n";
       try {
           System.out.println("Question: " + ques.toString());
           File outFile = new File( outputDir, "QUESTION_"+ques.getInd()+".XML" );
           FileWriter fileOut = new FileWriter(outFile);
           System.out.println("outFile: " + outFile.toString());
           fileNames.addElement(outFile.getName());
           logProgress("File name: " + outFile.toString() + " has been added to vector.", 13, 6);
           fileOut.write(s, 0, s.length());
           fileOut.close();
           logProgress("File name: " + outFile.toString() + " has been created.", 13, 5);
      } //try
      catch (Exception e) {
          logSevereError(e);
          System.out.println("Exception: " + e);
      } //catch
   } //toXML(Question)


   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This method should instantiate an instance of 
    *  the Export Batch
    *  
    *  @param args  String of command line arguments
    *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String[] args ) {
       Export export = new Export(args);
       export.startBatch();
   } //main
} //Export
