package com.ibm.nzna.projects.oa.imports ;

import java.util.*;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.type.TypeDocClassRec;
import com.ibm.nzna.projects.common.quest.type.TypeBrandRec;
import com.ibm.nzna.projects.common.quest.type.TypeGeoRec;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.pom.PersistentRec ;
import com.oroinc.text.perl.Perl5Util;

public class ImportUtil {
   public static Vector orderByGeo( Vector objectList ) {
      Vector newOrder = new Vector() ;
      Enumeration enum = objectList.elements() ;
      /* Go through the list of conditional objects */
      while (enum.hasMoreElements() ) {
         ConditionalObject condObj = (ConditionalObject) enum.nextElement() ;
         /* Go through the list of countries */
         /* We check to see which geo this belongs to */
         boolean us = false ;
         Vector countryList = condObj.getCountryList() ;
         Enumeration countryEnum = countryList.elements() ;
         while ( countryEnum.hasMoreElements() ) {
            TypeCountryCodeRec countryRec = (TypeCountryCodeRec) countryEnum.nextElement() ;
            /* We search for a U.S. country to determine if this geo was U.S. */
            if ( countryRec.geoInd == 2) {
               us = true ;
               break ;
            }
         }
         /* Is this conditional object in U.S. geo */
         /* All U.S. should be at the end of the vector */
         if ( us ) {
            newOrder.addElement( condObj) ;
         }
         else {

            newOrder.insertElementAt( condObj, 0) ;
         }
      }
      return ( newOrder ) ;
   } // orderByGeo()

   public static Vector orderSymptomTitlesByGeo( Vector objectList ) {
      Vector newOrder = new Vector() ;
      Enumeration enum = objectList.elements() ;
      /* Go through the list of conditional objects */
      try {
         while (enum.hasMoreElements() ) {
            OAImport.logSystem.log( "sorting symptom titles" ) ;
            SymptomTitle titleObj = (SymptomTitle)enum.nextElement() ;
            try {
               /* Go through the list of countries */
               /* We check to see which geo this belongs to */
               boolean us = false ;
               Vector countryList = titleObj.getCountryList() ;
               Enumeration countryEnum = countryList.elements() ;
               while ( countryEnum.hasMoreElements() ) {
                  try {
                     TypeCountryCodeRec countryRec = (TypeCountryCodeRec) countryEnum.nextElement() ;
                     /* We search for a us country to determine if this geo was us */
                     if ( countryRec.geoInd == 2) {
                        us = true ;
                        break ;
                     }
                  } catch ( Exception e ) {
                     OAImport.logSystem.log( "ImportUtil:orderSymptomTitlesByGeo: " + e ) ;
                  }
               }
               /* Is this conditional object in us geo */
               /* All us should be at the end of the vector */
               if ( us ) {
                  newOrder.addElement( titleObj) ;
               }
               else {

                  newOrder.insertElementAt( titleObj, 0) ;
               }
            } catch ( Exception e ) {
               OAImport.logSystem.log( "ImportUtil.java:orderSymptomTuitlesByGeo " + e ) ;
            }
         } // while
      } catch ( Exception e ) {
         OAImport.logSystem.log( "Caught exception " + e ) ;
      }
      return ( newOrder ) ;
   } // orderSymptomTitlesByGeo()

   public static Node findNode ( String symptomID, Symptom mainSymptom, String target, Vector nodeList, Vector indDescript,
                                 Hashtable symptomTable, Hashtable questionTable, Hashtable actionTable , 
                                 Hashtable nodeTable ) {

      Perl5Util regex = new Perl5Util();
      Node node = null ;
      boolean cloned = false ;
      String tmpString = target ;
      
      /* Find the OA object represented by the target in the .r file */
      OAObject oaObject   = null ;
      try {
         /* First search the node table.  The node table contains only the nodes created for */
         /* the current symptom.  It also contains references to cloned questions            */
         if ( (node = (Node)nodeTable.get( symptomID + "." + target + "." + (String)(indDescript.elementAt(1)) )) != null ) {
            return ( node ) ;
         }
         
         /* Check whether this is a cloned question for the target */
         if ( isClone( target ) && regex.match("/-/", target) ) {
            target = regex.preMatch() ;
            cloned = true ;
         }

         OAImport.logSystem.log( "Searching for: " + target + "." + (String)(indDescript.elementAt(1)) ) ;

         if ( (oaObject = (OAObject)symptomTable.get ( target + "." + (String)(indDescript.elementAt(1)))) != null ) {
            OAImport.logSystem.log( "TARGET symptom: " + target + " found in hashtable" ) ;
         } else if ( (oaObject = (OAObject)questionTable.get ( target + "." + (String)(indDescript.elementAt(1)))) != null ) {
            OAImport.logSystem.log( "TARGET question: " + target + " found in hashtable" ) ;
         } else if ( (oaObject = (OAObject)actionTable.get ( target + "." + (String)(indDescript.elementAt(1)))) != null ) {
            OAImport.logSystem.log( "TARGET action: " + target + " found in hashtable" ) ;
         }
      }
      catch ( Exception e ) {
         e.printStackTrace() ;
      }

      if ( oaObject != null ) {
         /* We must create a node for the target.  This node did not exist */
         /* in the old .r file format, but is now required                 */
         node = new Node( oaObject ) ;
         /* Add node to the node table */
         if ( cloned ) {
            nodeTable.put( symptomID + "." + tmpString + "." + (String)(indDescript.elementAt(1)), node ) ;
            OAImport.logSystem.log( "Inserted cloned question node" + tmpString ) ;
         } else
            nodeTable.put( symptomID + "." + target + "." + (String)(indDescript.elementAt(1)), node ) ;

         node.updateRecStatus( PersistentRec.REC_INSERT ) ;
         /* We must write this to the database so an indicator will be set on the node */
         boolean gotDbKeys = mainSymptom.assignDatabaseKeys() ;

         OAImport.logSystem.log( "Node not found for target: " + target + " Created a new node" ) ;
      }
      return ( node ) ;
   } // findNode()

   /*******************************************************/
   /* This method verifies that this is a cloned question */
   /*                                                     */
   /* @return true / false                                */
   /*******************************************************/
   public static boolean isClone( String target ) {
      
      Perl5Util regex = new Perl5Util();
      boolean cloned = false ;

      /* Check for cloned question */
      if ( regex.match( "/-/", target ) ) {
         cloned = true ;
         OAImport.logSystem.log( "Processing Clone: " + target ) ;
      }
      return cloned ;
   } // isClone

   /**************************************************************/
   /* This method is used to find whether an object is already 
   /* present in a list of objects. The passed object is compared
   /* for equality against the list of objects passed in the 
   /* arguments list.
   /* 
   /* @return boolean true if the object was found
   /**************************************************************/
   public static boolean contains( Vector objectList, Object object ) {
      Enumeration objectEnum = objectList.elements() ;
      boolean isEqual = false ;
      Object nextObj = null ;

      while ( objectEnum.hasMoreElements() ) {
         nextObj = objectEnum.nextElement() ;
         if ( nextObj == object ) {
            isEqual = true ;
            break ;
         }
         try {
            if ( ((Node) nextObj).getInd() == ((Node) object).getInd() ) {
               isEqual = true ;
               if ( ((Node) object).getInd() == 0 ) {
                  OAImport.logSystem.log( "Comparing nodes indicators, both indicators are 0" ) ;
               }
               break ;
            }
         } catch ( IllegalStateException e ) {
            OAImport.logSystem.log( "Exception: " + e ) ;
         }
      }
      return isEqual ;
   } // contains

   public static void addNodeToSymptom( Symptom symptom, Node node ) {
      
      Vector nodeList = symptom.getNodes() ;
      boolean assignedDBKey = symptom.assignDatabaseKeys() ;

      if ( !contains( nodeList, node ) ) {
         nodeList.addElement( node ) ;
         OAImport.logSystem.log( "Node added to list of Nodes" ) ;
      } else 
         OAImport.logSystem.log( "Did not add Node to list of nodes" ) ;

   } // addNodeToSymptom

   public static void addEdgeGroupToSymptom ( Symptom symptom, EdgeGroup edgeGroupAdd ) {
      Vector      edgeGroupList = symptom.getEdgeGroups() ;
      Node headNode             = null ;
      Node tailNode             = null ;
      EdgeGroup tmpEdgeGroup    = null ;
      EdgeGroup edgeGroup       = null ;
      Node sourceNode           = edgeGroupAdd.getHeadNode() ;
      Node targetNode           = edgeGroupAdd.getTailNode() ;
      boolean found             = false ;
      if ( edgeGroupList == null ) {
         edgeGroupList = new Vector() ;
         symptom.setEdgeGroups( edgeGroupList ) ;
      }
      Enumeration groupListEnum = edgeGroupList.elements() ;

      /* If edgegroups already exist search the edgegroup list    */
      /* for an existing edgegroup with same head and tail node   */
      if ( edgeGroupList.size() > 0 ) {
         OAImport.logSystem.log( "EDGEGROUPLIST SIZE " + edgeGroupList.size() ) ;
         while ( groupListEnum.hasMoreElements() ) {
           tmpEdgeGroup = (EdgeGroup)groupListEnum.nextElement() ;
           headNode = tmpEdgeGroup.getHeadNode() ;
           tailNode = tmpEdgeGroup.getTailNode() ;
           if ( headNode == sourceNode && tailNode == targetNode ) {   
              edgeGroup = tmpEdgeGroup ;
              found = true ;
              OAImport.logSystem.log( "Found EDGEGROUP" ) ;
              break ;
           } 
         }  // while 
      } // if
      /* Add the group to the group list if not found */
      if ( !found ) {
         //edgeGroup = new EdgeGroup( sourceNode, targetNode ) ;
         OAImport.logSystem.log( "Edgegroup not found in  Edgegrouplist" ) ;
         OAImport.logSystem.log( "Created new edgegroup" ) ;
         /* We add this group to the list since an existing */
         /* edge group was not found                        */
         edgeGroupList.addElement( edgeGroupAdd ) ;
         OAImport.logSystem.log( "ADDED TO EDGEGROUPLIST" ) ;
      }

   }

   /**********************************************************************/ 
   /* This method is used to verify that the source node already exists, */
   /* by checking whether the mainSymptom already contains it.           */
   /*                                                                    */
   /* @return true if the source node is found                           */  
   /*                                                                    */
   /**********************************************************************/ 
   public static boolean nodeExists( Symptom mainSymptom, Node sourceNode ) {

      Vector edgeGroupList = new Vector() ;
      boolean found = false ;

      edgeGroupList = mainSymptom.getEdgeGroups() ;

      for ( int i = 0 ; i < edgeGroupList.size() ; i++ ) {
         if ( ((EdgeGroup)edgeGroupList.elementAt(i)).getHeadNode() == sourceNode ) {
            found = true ;
            break ;
         }
      }
      return found ;
   } // nodeExists
   /*********************************************************
   * Checks to see if this node already exists in the list of nodes
   * 
   *
   * Author:  
   * Created: 11/1/100  5:22pm
   * @param
   * @return
   *********************************************************/
   public static boolean nodeExists( Vector nodes, int indicator ) {
      boolean found = false ;
      Enumeration nodesEnum = nodes.elements() ;
      while ( nodesEnum.hasMoreElements() ) {
         Node node = (Node) nodesEnum.nextElement() ;
         if (node.getActiveObjectInd() == indicator) {
            found = true ;
            break ;
         }
      }

      return ( found ) ;
   }
   /**
    * Returns the node type
    * 
    * NODE_TYPE_ACTION     =  0
    * NODE_TYPE_QUESTION   =  1
    * NODE_TYPE_SYMPTOM    =  2
    * NODE_TYPE_RESOLVED   =  3
    * NODE_TYPE_UNRESOLVED =  4
    * NODE_TYPE_RETURN     =  5
    * NODE_TYPE_ALIST      =  6
    *
    * @param source Source diretory in which to find the Symptom.
    * @param symptomID
    */
   public static short getNodeType ( String objLabel ) {
      short      nodeType = -1;
      Perl5Util  regex    = new Perl5Util();
      OAImport.logSystem.log("PASSED NODELABEL " + objLabel ) ;
      switch ( objLabel.charAt(0) ) {
      case 'A':
         nodeType = Node.NODE_TYPE_ACTION ;
         break ;

      case 'Q':
         nodeType = Node.NODE_TYPE_QUESTION ;
         break ;

      case 'S':
         nodeType = Node.NODE_TYPE_SYMPTOM ;
         break ;

      case '#':
         if ( regex.match( "/#RESOLVED/", objLabel ) ) {
            nodeType = Node.NODE_TYPE_RESOLVED ;
         } else if ( regex.match( "/#UNRESOLVED/", objLabel ) ) {
           OAImport.logSystem.log("UNRESOLVED NODETYPE" ) ;
           nodeType = Node.NODE_TYPE_UNRESOLVED ;
         } else if ( regex.match( "/#RETURN/", objLabel ) ) {
           nodeType = Node.NODE_TYPE_RETURN ;
         } else 
            nodeType = Node.NODE_TYPE_ALIST ;
         break ;
      } // switch

      return nodeType ;

   } // getNodeType()


   public static void listTable( Hashtable table ) {
      
      Enumeration keyEnum = table.keys() ;
      
      while ( keyEnum.hasMoreElements() ) {
         OAImport.logSystem.log( keyEnum.nextElement() + "\n" ) ;
      } 
   } // listTable

    /**
     * Converts the tokenized vector from infix to 
     * postfix 
     *
     * @param condStr  the conditions string
     * @return the postfix vector 
     */
   public static Vector getPostfix( Vector tokenVec ) {
      
      final String bang     = "!" ;
      final String and      = "&" ;
      final String or       = "|" ;
      final String pound    = "#" ;          // flag to indicate the ending of a vector and the process
      String term           = null ;
      String op             = "" ;
      Vector opVec      = new Vector() ;     // operators vector
      Vector infixVec   = tokenVec ;
      Vector postfixVec = new Vector() ;
      char precedOp ;

      infixVec.addElement( pound ) ;
      opVec.addElement( pound ) ;
      op = (String)infixVec.firstElement() ;
      
      while ( op != null || opVec.size() != 0 ) {
         if ( op.equals( bang ) || op.equals( and ) || op.equals( or ) || op.equals( pound ) ) {      // op is an operator
            /* compare new operator precedence over the  */                                                                   
            /* operator in the opVec                     */
            precedOp = getPrecedence( (String)opVec.lastElement(), op ) ;
            switch ( precedOp ) {                                                                                               
            case '>' :     // infix operator has lower precedence                     
               term = (String)postfixVec.lastElement () + (String)opVec.lastElement() ; // add last operator in operator vector to 
                                                                                        // the last term in postfix vector, ie: Q2:2&
               postfixVec.setElementAt( term, postfixVec.size()-1 ) ;                   // set last element to new term ie: Q2:2&
               opVec.removeElement( opVec.lastElement() ) ;                             // remove the last operator in the opVec
               break ;
            case '<' :     // infix operator has greater precedence                         
               opVec.addElement( op ) ;                                                 // add to operator vector
               infixVec.removeElement( infixVec.firstElement() ) ;
               op = (String)infixVec.firstElement() ;
               break ;
            case '=' :     // infix operator has the same precedence                  
               term = (String)postfixVec.lastElement() + (String)opVec.lastElement() ;
               postfixVec.setElementAt( term, postfixVec.size()-1 ) ;
               opVec.removeElement( opVec.lastElement() ) ;
               break ;
            case '?':                                                                  // we reached the flag ending the vectors
               opVec.removeAllElements() ;
               infixVec.removeAllElements() ;
               op = null ;
               break ;
            }
         } else{                                                                 // op is a term
            if ( op != null ) {
               postfixVec.addElement( op ) ;
               OAImport.logSystem.log("Last added to postfixvec " + (String)postfixVec.lastElement() ) ;
               infixVec.removeElement( infixVec.firstElement() ) ;
               op = (String)infixVec.firstElement() ;
            }
         }
         OAImport.logSystem.log( "Postfix Progress: " + postfixVec.toString() ) ;
      } // while

      return postfixVec ;

   }  // getPostfix()

   /**
    * Performs the operator precedence for the specified
    * operators
    *
    * @param op the operator in the operators vector
    * @param newOp the operator from infix vector
    * @return the precedence operator
    */
   public static char getPrecedence( String op, String newOp ) {
      char newOper = newOp.charAt(0) ;
      char precedOp = ' ' ;
      
      switch( op.charAt(0) ) {
      case '!': if ( newOper == '!' ) 
                  precedOp = '=';
                else precedOp = '>' ;    // op > newOp
                break ;

      case '&': if ( newOper == '!')
                  precedOp = '<'  ;                          // op < newOp
                else if ( newOper == '|' || newOper == '#' ) 
                   precedOp = '>' ;                          // op > newOp
                else precedOp = '=' ;
                break ;

      case '|': if ( newOper == '!' || newOper == '&' ) 
                   precedOp = '<' ;                       // op < newOp
                else if ( newOper == '#' ) 
                   precedOp = '>' ;                      // op > newOp
                else
                   precedOp = '=' ;
                break ;
      
      case '(':
         break ;

      case ')':
         break ;

      case '#': if (newOper == '#' )
                   precedOp = '?' ;    // return a flag indicating no more operators and terms
                else precedOp = '<' ;                       // op < newOp
                break ;
      }

      return precedOp ;
   } // getPrecedence

   /**
    * Converts the conditions string to a tokenized vector
    * of terms and conditions. 
    * Ex:  String = Q1:1 | Q2:2 to
    *      Vector = [ Q1:1, |, Q2:2 ]
    *
    * @param condStr  the conditions string
    * @return tokenizedVec the tokenized vector
    * 
    */
   public static Vector tokenizeString( String condStr ) {

     StringTokenizer strTokenizer = new StringTokenizer( condStr, "|&!", true ) ;
     Vector tokenizedVec = new Vector() ;
     String fixedString = null ;

     /* tokenize the condStr by the specified operators */
     while ( strTokenizer.hasMoreTokens() ) {
        tokenizedVec.addElement( strTokenizer.nextToken("()|&!") ) ;
     }
     // to fix bad data in terms, ie:  :Q30:21 should be Q30:21
    for ( int i = 0 ; i < tokenizedVec.size() ; i++ ) {
       if ( ((String)tokenizedVec.elementAt(i)).startsWith(":") ) {
          fixedString = (String)tokenizedVec.elementAt(i) ;
          tokenizedVec.setElementAt( fixedString.substring( 1, fixedString.length() ), i ) ;
          OAImport.logSystem.log("the string has been fixed " + tokenizedVec.elementAt(i) ) ;
       }
    } //for

     return tokenizedVec ;
   } // tokenizeString() 

   public static Vector tokenizeConclusion( String conclStr ) {
      StringTokenizer strTokenizer = new StringTokenizer( conclStr, "&", false ) ;
      Vector tokenizedVec = new Vector() ;
      String fixedString = null ;
      
      /* tokenize the condStr by the specified operators */
      while ( strTokenizer.hasMoreTokens() ) {
         tokenizedVec.addElement( strTokenizer.nextToken("&") ) ;
      }
      /* to fix bad data in terms, ie:  :Q30:21 should be Q30:21 */
     for ( int i = 0 ; i < tokenizedVec.size() ; i++ ) {
        if ( ((String)tokenizedVec.elementAt(i)).startsWith(":") ) {
           fixedString = (String)tokenizedVec.elementAt(i) ;
           tokenizedVec.setElementAt( fixedString.substring( 1, fixedString.length() ), i ) ;
           OAImport.logSystem.log("the string has been fixed " + tokenizedVec.elementAt(i) ) ;
        }
     } //for

      return tokenizedVec ;

   } // tokenizeConclusion

   /**
    * Splits a term string into a pair of question
    * and answer strings in a vector
    * Ex:  take Q1081:2!& return [q1081,2,!&]
    *
    * @param term a string representing a term
    * @return the term pair (question,answer) 
    */
   public static Vector getTerm ( String term ) {
      
      Vector termPair   = new Vector() ;
      Perl5Util  regex  = new Perl5Util () ;
      String ops        = null ;
      String QATerm     = null ;

      if ( regex.match ( "/[!&|].*/", term ) ) {
         QATerm = regex.preMatch() ;
         ops = regex.toString() ;
         termPair = regex.split( "/:/", QATerm ) ;
         termPair.addElement( ops ) ;
      } else
         termPair = regex.split( "/:/", term ) ;
      
      return termPair ;
   } // getTerm()

   /**
    * Gets the condition terms from the specified
    * terms vector and convert them to objects
    * Ex: [q2,2,&|] or [q2,2]
    *
    * @param termsVec, vector of Q&A pairs
    * @param questionTable, hashtable of questions
    * @param key, to look up the question in the table
    *
    * @return a vector of condition term pairs(Q&A objects)
    */
   public static Vector getConditionTerms( Vector termsVec, Hashtable questionTable, String brand ) {
      
      int    index ;
      Vector condTermsVec  = new Vector() ;      // holds the question and answer pairs
      Vector answersVec    = new Vector() ;
      Vector titlesVec     = new Vector() ;
      String key           = "" ;
      Title    title       = new Title() ;
      Question question    = new Question() ;
      Answer   answer      = new Answer() ;

      if ( termsVec != null ) {    // e.g.:termsVec = [Q1,1]
         OAImport.logSystem.log( "termsVec " + termsVec.toString() ) ;
         /* forming the key to look up in the table based on the  */
         /* first elementt of the vector                          */
         key = ((String)termsVec.elementAt(0)).toLowerCase() + "." + brand ; 
         try {
            /* look up the table by key.brand and get the Q&A indicators */
            question = (Question)questionTable.get( key ) ; // question obj from table

            if ( question == null ) {
              OAImport.logSystem.log("QUESTION IN TABLE is null" ) ;
              ImportUtil.listTable( questionTable ) ;
            } else {
               condTermsVec.addElement( question ) ;

               /* Now get the answers */
               answersVec = question.getAnswers() ;                           // list of answers to the question in the hashtable
               /* Check if the answer term has a match in the answers vector */
               if ( Integer.parseInt((String)termsVec.elementAt(1)) <= answersVec.size() ) {
                  index = Integer.parseInt( (String)termsVec.elementAt(1) ) - 1 ;
                  answer = (Answer)answersVec.elementAt( index ) ;               // the answer object

                  if ( answer == null ) {
                     OAImport.logSystem.log("ANSWER IN TABLE is null" ) ;
                  } else
                     condTermsVec.addElement( answer ) ;

                  if ( termsVec.size() == 3 ) {
                     condTermsVec.addElement( termsVec.lastElement() ) ;    // adding the operators string to the returning variable
                  }                                                         // the returning vector would look like: [Qind,Aind,ops]
               } else 
                  /* empty the conditionTerms vector since it already contains the question */
                  condTermsVec.removeAllElements() ;

            } // if - else 
         } catch( Exception e ) {
            OAImport.logSystem.log( "Exception Error, a value is null" + e ) ;
            e.printStackTrace() ;
         }
      }
      return condTermsVec ;
   } // getConditionTerm()

   public static String removeLine ( String string, int pos ) {
      int length = string.length() ;
      /* Find beginning of line */
      int begin = pos ;
      while ( begin >= 0 ) {
         if ( string.charAt(begin) < 32 ) {
            /* This is a newline or carriage return */
            break ;
         }
         begin-- ;
      }
      /* Find end of line */
      int end = pos ;
      while ( end < length ){
         if ( string.charAt(end) < 32 ) {
            /* This is a newline or carriage return */
            break ;
         }
         end++ ;
      }
      /* Move past any other spaces to beginning of next line */
      while (( string.charAt(end) <= 32 ) && ( end < length )){
         end++ ;
      }
      /* Create the new String */
      String newString = string.substring( 0, begin) + string.substring( end, length ) ;
      return ( newString ) ;
   } // removeLine

   /***********************************************************
   * Merges countries for Conditional Objects, from the source 
   * to the target object.
   *
   * Author:  
   * Created: 6/28/100  9:35pm
   * @param
   * @return
   ************************************************************/
   public static void mergeCountries ( ConditionalObject source, ConditionalObject target ) {
      try {
         /* Merge Country List */
         Vector      allCountriesList  = target.getCountryList() ;
         Vector      countryListSource = source.getCountryList() ;
         Enumeration countryListEnum   = countryListSource.elements() ;

         while ( countryListEnum.hasMoreElements() ) {
            Object country = countryListEnum.nextElement() ;
            if ( !allCountriesList.contains( country ) ) {
               allCountriesList.addElement( country ) ;
            }
         } // while 
         /* set the merged list back to the hAction */
         target.setCountryList( allCountriesList ) ;
      }
      catch ( Exception e ) {
         /* If this fails, so be it */
      }
   } // mergeCountries

   /*********************************************************
   * Merges countries for OAObjects, from the source to the 
   * target object.
   *
   * Author:  
   * Created: 6/28/100  9:35pm
   * @param
   * @return
   *********************************************************/
   public static void mergeCountries ( OAObject source, OAObject target ) {
      try {
         /* Merge Country List */
         Vector      allCountriesList  = target.getCountryList() ;
         Vector      countryListSource = source.getCountryList() ;
         Enumeration countryListEnum   = countryListSource.elements() ;

         while ( countryListEnum.hasMoreElements() ) {
            Object country = countryListEnum.nextElement() ;
            if ( !allCountriesList.contains( country ) ) {
               allCountriesList.addElement( country ) ;
            }
         } // while 
         /* set the merged list back to the hAction */
         target.setCountryList( allCountriesList ) ;
      }
      catch ( Exception e ) {
         /* If this fails, so be it */
      }
   }  // mergeCountries

   /*********************************************************
   * Merges countries for Symptom titles, from the source to 
   * the target object.
   *
   * Author:  
   * Created: 6/28/100  9:35pm
   * @param
   * @return
   *********************************************************/
   public static void mergeCountries ( SymptomTitle source, SymptomTitle target ) {
      try {
         /* Merge Country List */
         Vector      allCountriesList  = target.getCountryList() ;
         Vector      countryListSource = source.getCountryList() ;
         Enumeration countryListEnum   = countryListSource.elements() ;

         while ( countryListEnum.hasMoreElements() ) {
            Object country = countryListEnum.nextElement() ;
            if ( !allCountriesList.contains( country ) ) {
               allCountriesList.addElement( country ) ;
            }
         } // while 
         /* set the merged list back to the hAction */
         target.setCountryList( allCountriesList ) ;
      }
      catch ( Exception e ) {
         /* If this fails, so be it */
      }
   } //mergeCountries
   /*********************************************************
   * Merges countries from the source to the target object.
   * 
   *
   * Author:  
   * Created: 6/28/100  9:35pm
   * @param
   * @return
   *********************************************************/
   public static void mergeCountries ( ConditionalObject target, Vector countryListSource ) {
      try {
         /* Merge Country List */
         Vector      allCountriesList  = target.getCountryList() ;
         Enumeration countryListEnum   = countryListSource.elements() ;

         while ( countryListEnum.hasMoreElements() ) {
            Object country = countryListEnum.nextElement() ;
            if ( !allCountriesList.contains( country ) ) {
               allCountriesList.addElement( country ) ;
            }
         } // while 
         /* set the merged list back to the hAction */
         target.setCountryList( allCountriesList ) ;
      }
      catch ( Exception e ) {
         /* If this fails, so be it */
      }
   }

   /*********************************************************
   * Creates a table of DocLink objects for Quest documents.
   *
   * 
   * @param
   * @return
   *********************************************************/
   public static DocLink createQuestDocLink ( Hashtable questDocIdTable, int docId, String linkTitle ) {
      
      Integer docIdObj = new Integer( docId ) ;
      DocLink docLink = null ;
      int docIdKey = 0 ;


      /* create a table of docLink objects                     */
      /* check whether the docId already exists in the table   */
      if ( !questDocIdTable.containsKey( docIdObj ) ) {
         /* set the quest link */
         QuestLink questLink = new QuestLink( docId, linkTitle.trim() ) ;        
         questLink.writeToDatabase() ;
         /* create the docLink object */
         docLink = new DocLink ( questLink ) ;
         /* store the doclink object in the doclink table */
         questDocIdTable.put( docIdObj, docLink ) ;
         OAImport.logSystem.log( "createDocLink: inserted a new doclink " + docIdObj.intValue() ) ;
      } else
         /* return the existing docLink */
         docLink = (DocLink)questDocIdTable.get( docIdObj ) ;
         OAImport.logSystem.log( "docLink already is in the table " + docIdObj.intValue() ) ;

      return docLink ;
   }  // createDocLink

   /***********************************************************
   * Creates a table of DocLink objects for External documents.
   * 
   *
   * 
   * @param
   * @return
   ************************************************************/
   public static DocLink createExternalDocLink ( Hashtable externDocIdTable, String url, String linkTitle ) {
      DocLink docLink = null ;

      /* create a table of docLink objects                     */
      /* check whether the docId already exists in the table   */
      if ( !externDocIdTable.containsKey( url ) ) {
         /* set the external link */
         ExternalLink externalLink = new ExternalLink( url, linkTitle.trim() ) ;        
         externalLink.writeToDatabase() ;
         /* create the docLink object */
         docLink = new DocLink ( externalLink ) ;
         /* store the new doclink in the doclink table */
         externDocIdTable.put( url, docLink ) ;
         OAImport.logSystem.log( "createDocLink: inserted a new doclink " + docLink.getInd() ) ;
      } else
         docLink = (DocLink)externDocIdTable.get( url ) ;
         OAImport.logSystem.log( "docLink is already in the table " + docLink.getInd() ) ;
      
      return docLink ;
   }  // createDocLink
   /*------------------------------------------------------------*/
   /* getDocClassForConditionalObject
    */
   /**
    *    This method compares the docclass conditions of a new 
    *   title to a title in the hash table. Sets the  docclass 
    *   to the default docclass to 'external'
    *  
    *
    *  @param None hTitle, the title of object in hash table
    *  @param None title, the title of new object
    *------------------------------------------------------------*/
   public static void setDocClassForConditionalObject ( ConditionalObject hObject, TypeDocClassRec incomingObjectDocClassRec ) {

      TypeDocClassRec hObjectDocClassRec = hObject.getDocClass() ;
      TypeDocClassRec docClassRec = incomingObjectDocClassRec ;

      /* The hObject may be new without a docclass */
      if ( hObjectDocClassRec != null ) {
         switch ( hObjectDocClassRec.getInd() ) {
         case 1 :         // IBM Confidential
            if ( incomingObjectDocClassRec.getInd() == 2 || incomingObjectDocClassRec.getInd() == 3 ) {
               docClassRec = incomingObjectDocClassRec ;
            } else 
               docClassRec = hObjectDocClassRec ;
            break ;
         case 2 :        // Dealer Confidential
            if ( incomingObjectDocClassRec.getInd() == 3 ) {
               docClassRec = incomingObjectDocClassRec ;
            } else if ( incomingObjectDocClassRec.getInd() == 1 || incomingObjectDocClassRec.getInd() == 2 ) 
               docClassRec = hObjectDocClassRec ;
            break ;
         case 3 :       // Unclassified
               docClassRec = hObjectDocClassRec ;
            break ;
         }
      }
      hObject.setDocClass( docClassRec );
   }  // getDocClassForConditionalObject

} // ImportUtil
