package com.ibm.nzna.projects.oa.imports ;

import java.io.*;
import java.sql.* ;
import java.util.*;
import java.util.zip.*;
import com.oroinc.text.perl.Perl5Util;
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.type.TypeDocClassRec;
import com.ibm.nzna.projects.common.quest.type.TypeBrandRec;
import com.ibm.nzna.projects.common.quest.type.TypeGeoRec;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.pom.PersistentRec ;



/**
 * KBIO is a utility class for reading and writing Knowledge Base objects.
 * Objects can be read from resource (.r) files
 *
 *
 * </pre>
 */
public class KBIO {
   private static Perl5Util regex = new Perl5Util();
    Question  rootQuestion = null ;
    Question  rootSymptomQuestion   = null ;
    Symptom   rootSymptom           = null ;
    Hashtable docIdTable            = null ;
    /**
     * Opens a Knowledge Base resource file for reading.  Using the source and id,
     * opens a resource file for reading.
     *
     * @param source  The source directory for the resource file. Resource files
     *                end with a ".r" (resource) extension.
     * @param id      The id of the object to open.
     *
     * @return BufferedReader
     */
    private synchronized BufferedReader openResourceFile(String source, String id) throws IOException, FileNotFoundException {
        String     filename;
        BufferedReader in =  null;
        
        id = getBaseID(id);
        
        //String tempFilename=source.toLowerCase()+id.toLowerCase();
        String tempFilename=source + id ;
        if (!tempFilename.endsWith(".r")) {
            filename=tempFilename+".r";
        }
        else {filename=tempFilename;}

        String sep = System.getProperty("file.separator");
        
        //if (source.endsWith("\\")) {
        if (source.endsWith(sep)) {
            in = new BufferedReader(new FileReader(filename));
        }
        else {
            //InputStream i = KBIO.class.getResourceAsStream(filename);
           System.out.println ( "KBIO: FILENAME: " + filename + "\n") ;
           FileInputStream i = new FileInputStream( filename ) ;                //KBIO.class.getResourceAsStream(filename);
            in = new BufferedReader(new InputStreamReader(i));
        }

        return in;
    }  // openResourceFile()


    /**
     * Process the translation file in the source directory against the match parameter.
     * Returns the last matching set of preanswers in the translation file.
     *
     * @param sourceDir       The source directory for the translation file.
     * @param translationFile The name of the translation file.
     * @param 
     *
     * @return String The last set of preanswers associated with the specified
     *                matchParam contained in the specified translation file.
     *                If no match if found, then the empty String ("") is returned.
     */
    public Hashtable processTranslationFile( String sourceDir, String translationFile, Hashtable questionTable,
                                             Hashtable translateTable, Vector indDescript ) {
        int       count           = 0 ;
        int       answerLabel     = 0 ;
        Integer   prodInt         = null ;
        Integer   answerInd       = null ;
        Integer   brandGrpInd     = null ;
        Integer   brandInd        = null ;
        String    key             = null ;
        String    astr            = null ;
        String    typeMod         = null ;  
        String    preanswer       = null ;  
        String    questionTerm    = null ;
        String    answerTerm      = null ;
        String    machType        = null ;
        String    machMod         = null ;
        Vector    nextRec         = new Vector() ;
        Vector    typeModList     = new Vector() ;
        Vector    prodIndList     = new Vector() ;
        Vector    answerIndList   = new Vector() ;
        Vector    preTermsList    = new Vector() ;
        Vector    preAnswerList   = new Vector() ;
        Hashtable indicatorsTable = translateTable ;
        Statement statement       = null ;
        ResultSet resultSet       = null ;
        ResultSet brandResultSet  = null ;
        SQLMethod sqlMethod       = null ;
        Question  question        = new Question() ;
        Answer    answer          = new Answer() ;
        Perl5Util regex           = new Perl5Util();
        BufferedReader in ;

        OAImport.logSystem.log( "\nReading: " + sourceDir + translationFile ) ;

        try {
            in = openResourceFile( sourceDir, translationFile ) ;
            
            /* Read in each typeMod record in the translation file */
            while ( (astr = in.readLine()) != null ) {
               /* separate the record into a vector of 2 elements: */
               /* nextRec(0) = machine type/model                  */
               /* nextRec(1) = Q&A pair                            */
               nextRec = regex.split( "/[,]/", astr.trim(), 2 ) ;
               
               if ( nextRec.size() > 1 ) {
                  /* Format the machine type model */
                  typeMod   = ((String)nextRec.elementAt( 0 )).trim() ;

                  /* Check that the typeMod string has the right length */
                  if ( typeMod.length() == 7 ) {
                     machType  = typeMod.substring( 0, 4 ) ;
                     machMod   = typeMod.substring( 4, typeMod.length() ) ;
                     typeMod   = machType + "-" + machMod ;
                     preanswer = (String)nextRec.elementAt( 1 ) ;

                     /* Check if the preanswer is a list of Q&A pairs */
                     if ( preanswer.indexOf( "&" ) != -1 ) {
                        preTermsList = regex.split ( "/&/", preanswer ) ;
                     } else 
                        preTermsList.addElement( preanswer ) ;

                     Enumeration preTermsEnum = preTermsList.elements() ;
                     String nextTerm = null ;
                     boolean addedAnswer = false ;
                     /* Loop through pre answer terms list */
                     while ( preTermsEnum.hasMoreElements() ) {
                        try {
                           nextTerm = ((String)preTermsEnum.nextElement()).trim() ;
                           /* find the question in the hashtable in order to extract   */
                           /* its corresponding answer and obtain the answer indicator */
                           if ( regex.match( "/:/", nextTerm ) )  {
                              questionTerm = regex.preMatch() ;
                              answerTerm   = regex.postMatch() ;
                              answerLabel  = Integer.parseInt( answerTerm ) ;
                           }
                           else
                              OAImport.logSystem.log( "Malformed Preanswer, no matching ':'" ) ;

                           /* check that the answer label is not 0      */
                           /* get the answer's indicator and add it to  */
                           /* the answer indicator vector               */
                           if ( answerLabel > 0 ) { 
                              OAImport.logSystem.log("answerlabel: " + answerLabel ) ;
                              if ( questionTerm.startsWith( "Q" ) ) {
                                 question = (Question)questionTable.get( questionTerm.toLowerCase() + "." + (String)indDescript.elementAt(1) ) ;
                                 answer = (Answer)(question.getAnswers()).elementAt( answerLabel - 1 ) ;
                                 answerInd = new Integer( answer.getInd() ) ;
                                 answerIndList.addElement( answerInd ) ;
                                 addedAnswer = true ;
                              } else
                                 OAImport.logSystem.log( "Malformed Preanswer, no matching 'Q'" ) ;
                           } // if

                        } catch ( Exception e ) {
                           OAImport.logSystem.log( "Failed on TypeMod: " + typeMod + " question ind: " + question.getInd() ) ;
                           OAImport.logSystem.log( "Exception thrown " + e ) ;
                           OAImport.logError( e ) ;
                           e.printStackTrace() ;
                        }
                     } // while

                     /* this is to check that the machine has a preanswer      */
                     /* if the preanswer has been added to the vector then     */
                     /* add also the type model to the typemod vector. In this */
                     /* way the size of the vectors are kept in sync.          */
                     if ( addedAnswer ) {
                        typeModList.addElement( typeMod ) ;
                        preAnswerList.addElement( answerIndList.clone() ) ;
                     }
                     
                     /* clean the vector of terms before processing */
                     /* the next typemod record in the .r file      */
                     preTermsList.removeAllElements() ;
                     answerIndList.removeAllElements() ;
                  } else {
                     OAImport.logSystem.log( "ERROR: TypeMod length is: " + typeMod.length() + "on: " + typeMod ) ;
                  }  // if typeMode.length()

               } // if nextRec.size()>1
               else 
                  OAImport.logSystem.log( "No preanswer available" ) ;
            } // while in.readline
            OAImport.logSystem.log ( "typemodlist.size and preanswerlist.size " + typeModList.size() + " " + preAnswerList.size() ) ;
            /****************************************************************/
            /* Create a Type/Mod.Brand hashtable with the preanswers vector */ 
            /* containing the answerIndicator as the values                 */
            /****************************************************************/
            Hashtable typeModTable = new Hashtable() ;

            for ( int i = 0 ; i < typeModList.size() ; i++ ) {
               key = (String)typeModList.elementAt(i) + "." + indDescript.elementAt(1) ;
               /* if the key does not exist in the tables insert a new typeMod record */
               if ( !typeModTable.containsKey( key ) || !indicatorsTable.contains( key ) ){
                  typeModTable.put( key, preAnswerList.elementAt(i) ) ;
               } else {
                  OAImport.logSystem.log( "key already exists: " + key + " " + (String)indDescript.elementAt(2) ) ;
               }
            } // for

            /******************************************************************/
            /* Get the PRODUCT and BRANDGROUP indicators from TIGRIS.PRODUCTS */
            /* table in the quest database and create a table of product      */
            /* indicators and an answer indicators vector                     */
            /******************************************************************/
            sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "KBIO.processTranslationFile () ", 5 ) ;
            statement = sqlMethod.createStatement() ;
            
            Enumeration typeModEnum = typeModTable.keys() ;
            String typeModKey = null ;
            String machine = null ;
            String model   = null ;
            Vector prodAnsIndList = new Vector() ;

            while ( typeModEnum.hasMoreElements() ) {
               /* the typeMod string is in the form: 1234-EFG.brand */
               typeModKey = ((String)typeModEnum.nextElement()).trim() ;
               OAImport.logSystem.log( "\ntypeModKey: " + typeModKey ) ;
               /* keep the typeMod part only */
               typeMod = typeModKey.substring( 0, typeModKey.indexOf( "." ) ) ;
               machine = typeMod.substring( 0, typeMod.indexOf("-") ) ;
               model = typeMod.substring( typeMod.indexOf("-")+1, typeMod.length() ) ;
               OAImport.logSystem.log("machine/model: " + machine + " " + model ) ;

               OAImport.logSystem.log( "*** Reading from TIGRIS.PRODUCTS" ) ;

               //resultSet = statement.executeQuery( "select prodint, brandgroupind from tigris.products where prodnum = '" +typeMod+"'" ) ;
               resultSet = statement.executeQuery( "select prodint, brandgroupind from tigris.products where machine = '" + machine
                                                   + "' and model = '" + model + "'"  ) ;
               
               boolean wasNull = false ;
               boolean nextRow = false ;

               if ( resultSet.next() ) {
                  nextRow = true ;
                  OAImport.logSystem.log( "resultset column1: " + resultSet.getInt(1) ) ;
                  /* Check whether the SQL statement returned */
                  /* SQL NULL or not                          */
                  if ( resultSet.wasNull() ) {
                     wasNull = true ;
                  } else {
                     OAImport.logSystem.log( "resultset column2: " + resultSet.getInt(2) ) ;
                     if ( resultSet.wasNull() ) {
                        wasNull = true ;
                     }
                  }

                  if ( !wasNull ) {
                     /* Create a table of product-answers indicators */
                     prodInt = new Integer( resultSet.getInt( 1 ) ) ;
                     /* brand group indicator will be used later on */
                     brandGrpInd = new Integer( resultSet.getInt( 2 ) ) ;
                     // create a vector containing the product indicator and its answer indicator list
                     prodAnsIndList.addElement( prodInt ) ;
                     prodAnsIndList.addElement( typeModTable.get( typeModKey ) ) ;
                     OAImport.logSystem.log("prodansindlist.size: " + prodAnsIndList.size() ) ;
                     /* insert in the table the typeModKey and Vector of prodInd and vector of answerInd */
                     if ( !indicatorsTable.containsKey( typeModKey ) ) {
                        indicatorsTable.put( typeModKey, prodAnsIndList.clone() ) ;
                     } else {
                        nextRow = false ;
                        OAImport.logSystem.log( "This key already exists in the table: " + typeModKey ) ;
                     }
                     prodAnsIndList.removeAllElements() ;
                  }
               } // if resultSet

               /***************************************************************/
               /* Query the TIGRIS.BRANDS table to obtain the BRAND indicator */
               /* and create a table of product-brand indicators              */
               /***************************************************************/
               /* if the previous resultset was not null */
               if ( !wasNull && nextRow ) {
                  OAImport.logSystem.log( "*** Reading from TIGRIS.BRANDS" ) ;
                  brandResultSet = statement.executeQuery( "select brandind from tigris.brands where brandgroupind = " + brandGrpInd ) ;
                  Vector indicatorsList = new Vector() ;
                  try {

                     while ( brandResultSet.next() ) {
                        brandInd = new Integer( brandResultSet.getInt( 1 ) ) ;
                        if ( !brandResultSet.wasNull() ) {
                           indicatorsList = (Vector)indicatorsTable.get( typeModKey ) ;
                           indicatorsList.addElement( brandInd ) ;
                           OAImport.logSystem.log( "indicatorsList.size: " + indicatorsList.size() ) ;
                        } else {
                           OAImport.logSystem.log( "Sql returned a null value" ) ;
                        }
                     } // while brandResultSet

                  } catch ( SQLException sqle ) {
                     OAImport.logSystem.log( "SQLException. File - TypeMod: " + sourceDir + translationFile + " " + typeMod ) ;
                     OAImport.logSystem.log( " " + sqle ) ;
                     OAImport.logError( sqle ) ;
                     sqle.printStackTrace() ;
                  }
               } else // if !wasnull
                  OAImport.logSystem.log( "product or brandgroup query was null" ) ;

            }  // while typeModEnum.hasMoreElements

            statement.close() ;
            sqlMethod.close() ;
            
        }
        // catch both of the possible errors from this 'try'
        catch (FileNotFoundException e) {
           OAImport.logSystem.log( "Exception: " + e ) ;
           OAImport.logError( e ) ;
           OAImport.logError( e ) ;
           e.printStackTrace() ;
        }
        catch (IOException e) {
           OAImport.logSystem.log( "Exception: " + e ) ;
           OAImport.logError( e ) ;
           OAImport.logError( e ) ;
           e.printStackTrace() ;
        }
        catch ( SQLException sqle ) {
           sqlMethod.close() ;
           OAImport.logSystem.log( "SQLException: " + sqle ) ;
           OAImport.logError( sqle ) ;
           sqle.printStackTrace() ;
        }

        OAImport.logSystem.log( "Table sizes: indicatorsTable " + indicatorsTable.size() ) ;
        return indicatorsTable ;

    } // processTranslationFile()


    /**
     * readQuestion reads a question resource file and creates a new Question object
     * @param source Source directory in which to find the Question.
     * @param questionID
     * @return Question - Question object read from the resource file.
     */
    public synchronized Question readQuestion( String source, String questionID, Vector indDescript )  throws ArrayIndexOutOfBoundsException {
             
       String     astr          = null ;            // input string
       String     tempinput     = null ;            // temporary container
       Vector     ans           = new Vector () ;
       Vector     cond          = new Vector () ;
       Vector     titlesV       = new Vector () ;
       Vector     ansTitlesV    = new Vector() ;
       Vector     answersV      = new Vector() ;
       BufferedReader in;
       Perl5Util  regex         = new Perl5Util ();
       Title       ansTitle     = new Title() ;
       Answer      answer       = new Answer() ;
       AddtlInfo   addtlInfo    = new AddtlInfo() ;
       Question    mainQuestion = new Question() ;
       Title       qTitle       = null ;
       
       OAImport.logSystem.log( "\n------- Reading Question -------\nFile: " + source + questionID ) ;

       /* set the geo, brand and docclass indicators */
       int             geoInd  = OAImport.typeList.indFromString( (String)indDescript.elementAt( 0 ), OAImport.typeList.TYPE_GEOGRAPHY ) ;
       TypeGeoRec      geoRec  = new TypeGeoRec( geoInd, (String)indDescript.elementAt( 0 ) ) ;
       mainQuestion.setCountryList( geoRec.getCountryVec() ) ;
       mainQuestion.setBrand( (TypeBrandRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(1), OAImport.typeList.TYPE_BRAND ) ) ;
       /* set all questions to be external */
       mainQuestion.setDocClass ( (TypeDocClassRec)OAImport.typeList.objectFromString( "Unclassified", OAImport.typeList.TYPE_DOCCLASS ) ) ;

       tempinput="";
       try {
           
           in = openResourceFile(source, questionID);
           
           // Find the begining of the question text
           while(((astr = in.readLine()) != null) && (!astr.equals("START QUESTION"))) {
           }
           // Read in the text of the question
           while(((astr = in.readLine()) != null) && (!astr.equals("END QUESTION"))) {
               tempinput=tempinput+astr;
           }
           /* Check that the title size does not exceed the */
           /* max size limit in the database                */
           int maxStrLength = 250 ;
           if ( tempinput.length() > maxStrLength ) {
              OAImport.logSystem.log( "Setting the title size to 250 characters." ) ;
              tempinput = tempinput.substring( 0, maxStrLength ) ;
           }
           qTitle = new Title( tempinput.trim() ) ;
           qTitle.setCountryList( geoRec.getCountryVec() ) ;
           qTitle.setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ;
           qTitle.setLanguage( new TypeLanguageRec( 1, "" ) ) ;
           titlesV.addElement( qTitle ) ;
           mainQuestion.setTitles( titlesV );  // Save the question text
           OAImport.logSystem.log( "Reading Question Title " ) ;
           tempinput="";

           OAImport.logSystem.log( "Reading Answers" ) ;
           // Find the beginining of the answer list
           while(((astr = in.readLine()) != null) && (!astr.equals("START ANSWERS"))
                 && (!astr.equals("START ANSWERS HIDDEN"))
                 && (!astr.equals("START ANSWERS AUTOFILL"))
                 && (!astr.equals("START ANSWERS HIDDEN AUTOFILL"))) {
           }

           if ( astr != null ) {
              if (astr.equals("START ANSWERS HIDDEN")) {
                  mainQuestion.setHiddenAnswers(true);
              }
              else if (astr.equals("START ANSWERS HIDDEN AUTOFILL")) {
                  mainQuestion.setHiddenAnswers(true);
                  mainQuestion.setAutoFill(true);
              }
              else if (astr.equals("START ANSWERS AUTOFILL")) {
                  mainQuestion.setAutoFill(true);
              }

              /* Read in the answers FIRST PASS - to store questions and answers in a table   */
              /* without any conditions, so when setting the conditions, we can refer back to */
              /* questions and answers already read in                                        */
              while(((astr = in.readLine()) != null) && (!astr.equals("END ANSWERS"))) {
                  //System.out.println("Reading Answers");
                  // Separate the answer label and conditions from the answer text
                  // The ans vector will contain the answers and conditions in element 0
                  // The answer text will be in element 1
                  ans=regex.split("/[,]/",astr,2);
                  // Separate the answer label from the conditions
                  // cond will contain the answer label in element 0 and the conditionlist
                  // in element 1
                  cond=regex.split("/:/",(String)ans.elementAt(0),2);

                  String label = ((String)cond.elementAt(0)).trim() ;
                  
                  /* check whether the answer label != 0 */
                  if ( !label.equals("0") ) {
                     try {
                        int index = 1 ;
                        answer = new Answer() ;
                        ansTitlesV = new Vector() ;
                        /* set the country and docclass for the Answer object */
                        answer.setCountryList( geoRec.getCountryVec() ) ;
                        answer.setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ; 
                        /* Set the Title Object for the Answer */
                        if ( ans.size() == 1 ) {
                           index = 0 ;
                        }
                        /* Check that the title size does not exceed the */
                        /* max size limit in the database                */
                        String titleString = (String)ans.elementAt( index ) ;
                        if ( titleString.length() > maxStrLength ) {
                           OAImport.logSystem.log( "Setting the answer title size to 250 characters." ) ;
                           titleString = titleString.substring( 0, maxStrLength ) ;
                        }
                        if ( titleString.equals("6862V10") ) {
                           OAImport.logSystem.log( "AnswerTitle: " + titleString ) ;
                        }
                        ansTitle = new Title( titleString.trim() ) ;
                        /* set the country and docclass for the Answer Title object */
                        /* and the Language indicator                               */
                        ansTitle.setCountryList( geoRec.getCountryVec() ) ;
                        ansTitle.setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ;
                        ansTitle.setLanguage( new TypeLanguageRec( 1, "" ) ) ;
                        ansTitlesV.addElement( ansTitle ) ;
                        answer.setTitles( ansTitlesV ) ;        // Save the answer text

                        /* Check for malformed answer labels such as q23.r where answer label 11 shows as 11Q22 */
                        String num = (String)cond.elementAt(0) ;
                        if ( regex.match( "/[A-Za-z.]+/", num ) ) {         
                           num = regex.preMatch() ;
                        }
                        /* check and set the current size of the answers vector */
                        int ansNum = Integer.parseInt( num ) ;               // the answer's label
                        if ( ansNum > answersV.size() ) {
                           answersV.setSize( ansNum ) ;
                           answersV.ensureCapacity( ansNum ) ;
                        }

                        /* Insert the answer in the vector in the position indicated by the answer label */
                        if ( answersV.elementAt( ansNum - 1 ) == null ) {
                           answersV.setElementAt( answer, ansNum - 1 ) ;
                           if ( questionID.equals("q1") && ansNum == 14 ) {
                              OAImport.logSystem.log( "answer title: " + titleString ) ;
                           }
                        }

                        /* Add the answers to the Question */
                        mainQuestion.setAnswers( answersV ) ;

                     } catch( ArrayIndexOutOfBoundsException aioobe )  {
                        OAImport.logSystem.log ( "** Exception Reading: "  + source + questionID+ "\n" ) ;
                        OAImport.logSystem.logSevereError ( "Exception Reading Answer : " + source + questionID+ "\n" ) ;
                        OAImport.logSystem.logSevereError ( "Bad Answer: \n" + aioobe + "\n" ) ;
                        OAImport.logError( aioobe ) ;
                     } catch ( Exception e ) {
                        OAImport.logSystem.log ( "** Exception Caught Reading: "  + source + questionID ) ;
                        OAImport.logSystem.logSevereError ( "Exception Reading Answer : " + source + questionID) ;
                        OAImport.logSystem.logSevereError ( "Bad Answer: \n" + e + "\n" ) ;
                        OAImport.logError( e ) ;
                     }
                  }  // if answer label == 0
               // Save the answer

              }  //  while - END OF FIRST PASS
           } // if astr != null

           
           tempinput="";
           
           // Reset to begining of file to read Additional Information 
           in.close();
           in = openResourceFile(source, questionID);
           
           // Find the begining of the Explain Text
           while(((astr = in.readLine()) != null) && (!astr.equals("START EXPLAIN"))) {
           }
           //System.out.println("Reading ExplainText");
           // Read in the Explain Text for the question
           while(((astr = in.readLine()) != null) && (!astr.equals("END EXPLAIN"))) {
               tempinput=tempinput+astr+"\n";
           }

           /******* Setting the Additional Info Section *********/
           int index ;
           Vector addtlInfoV = new Vector () ;
           if ( !tempinput.equals("") ) {
              OAImport.logSystem.log( "Reading Additional Information" ) ;
              /* Set the additional information object */
              /* the additional info text has an Additional Information area */
              if ( (index = tempinput.indexOf( "Additional Information:" )) != -1 ) {
                 String explain = tempinput.substring( 0, index ) ;
                 mainQuestion = setAddInfoLinks ( mainQuestion, addtlInfo, explain, true ) ;
                 String addInfotxt = tempinput.substring ( index, tempinput.length() ) ;
                 mainQuestion = setAddInfoLinks ( mainQuestion, addtlInfo, addInfotxt, false ) ;

                /* the additional info has <LI> tags only */
              } else if ( (index = tempinput.indexOf( "<LI>" )) != -1 ) {   
                 String explain = tempinput.substring( 0, index ) ;
                 mainQuestion = setAddInfoLinks ( mainQuestion, addtlInfo, explain, true ) ;
                 String addInfotxt = tempinput.substring ( index, tempinput.length() ) ;
                 mainQuestion = setAddInfoLinks ( mainQuestion, addtlInfo, addInfotxt, false ) ;

              } else {  /* process additional info txt with no Add Info area or no <LI> tags in it */
                 mainQuestion = setAddInfoLinks( mainQuestion, addtlInfo, tempinput, true ) ;
              }

              /* set the additional information's Geo and DocClass */
              addtlInfoV = mainQuestion.getAdditionalInfo() ;
              if (addtlInfoV == null ) {
                 addtlInfoV = new Vector () ;
              } else
              for ( int i = 0 ; i < addtlInfoV.size() ; i++ ) {
                 ((AddtlInfo)addtlInfoV.elementAt(i)).setCountryList( geoRec.getCountryVec() ) ;
                 ((AddtlInfo)addtlInfoV.elementAt(i)).setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ;
                 ((AddtlInfo)addtlInfoV.elementAt(i)).setLanguage( new TypeLanguageRec( 1, "" ) ) ;
              }

              mainQuestion.setAdditionalInfo( addtlInfoV ) ;
           } else {
               mainQuestion.setAdditionalInfo( new Vector () );
           }
           tempinput="";

           // Reset to beginning of file
           in.close();
       }
       // catch both of the possible errors from this 'try'
       catch (FileNotFoundException e) {
           System.out.println("FileNotFoundException in readQuestion!\n");
           OAImport.logError( e ) ;
           e.printStackTrace();
           return null;
       }
       catch (IOException e) {
           System.out.println("IOException in readQuestion!\n");
           OAImport.logError( e ) ;
           e.printStackTrace();
           return null;
       }

       /* set the type of Question */
       mainQuestion.setQuestionType( new TypeQuestionRec( 1, "Default" ) ) ;

       return mainQuestion;
    } // readQuestion()

    
    /**
     * readQuestion reads a question resource file and creates a new Question object
     * @param source Source directory in which to find the Question.
     * @param questionID
     * @return Question - Question object read from the resource file.
     */
    public synchronized Hashtable readQuestion( String source, String questionID, Vector indDescript, Hashtable qTable,
                                                String key )  throws ArrayIndexOutOfBoundsException {
       
       Vector     answersV    = new Vector () ;
       Vector     ans         = new Vector () ;
       Vector     cond        = new Vector () ;
       Hashtable  hashtable   = new Hashtable() ;
       String     astr        = null ;                             // input string
       String     tempinput   = null ;                             // temporary container
       BufferedReader in ;
       Perl5Util  regex       = new Perl5Util ();
       Question mainQuestion  = new Question() ;
       
       OAImport.logSystem.log( "\n------- Reading Question -------\nFile: " + source + questionID ) ;

       int             geoInd  = OAImport.typeList.indFromString( (String)indDescript.elementAt( 0 ), OAImport.typeList.TYPE_GEOGRAPHY ) ;
       TypeGeoRec      geoRec  = new TypeGeoRec( geoInd, (String)indDescript.elementAt( 0 ) ) ;

       hashtable = qTable ;
       tempinput="";
       
       try {
           /* retrieve the current question object from the question hash table */
           /* and its corresponding list of answers                             */
           mainQuestion = (Question)hashtable.get( key ) ;
           answersV = mainQuestion.getAnswers() ;
           
           in = openResourceFile(source, questionID);
          
           // Find the begining of the question text -- begining at the Answers section
           // Find the beginining of the answer list
           while(((astr = in.readLine()) != null) && (!astr.equals("START ANSWERS"))
                 && (!astr.equals("START ANSWERS HIDDEN"))
                 && (!astr.equals("START ANSWERS AUTOFILL"))
                 && (!astr.equals("START ANSWERS HIDDEN AUTOFILL"))) {
           }

           if ( astr != null ) {
              /* Read in the answers and set their conditions  */
              while(((astr = in.readLine()) != null) && (!astr.equals("END ANSWERS"))) {
                  
                 // Separate the answer label and conditions from the answer text
                  // The ans vector will contain the answers and conditions in element 0
                  // The answer text will be in element 1
                  ans=regex.split("/[,]/",astr,2) ;
                  // Separate the answer label from the conditions
                  // cond will contain the answer label in element 0 and the conditionlist
                  // in element 1
                  cond=regex.split("/:/",(String)ans.elementAt(0),2) ;
                  
                  /* Fix for condition grouping */
                  if ( cond.size() > 1 && ((String)cond.elementAt(1)).indexOf("(") == -1 ) {

                     String num = (String)cond.elementAt( 0 ) ;
                     String condList = null ;

                     /* checking for answer label == 0 */
                     if ( !num.equals( "0" ) ) {
                        /* to handle .r files data inconsistency where there is no  */
                        /* separator between the answer label and the conditions    */
                        if ( regex.match( "/[A-Za-z.]+/", num ) ) {
                           OAImport.logSystem.log("REGEX " + regex.toString() ) ;
                           num = regex.preMatch() ;
                           OAImport.logSystem.log("NUM " + num ) ;
                           condList = regex.toString() + regex.postMatch() ;
                           OAImport.logSystem.log("CONDLIST " + condList.toString() ) ;
                        }
                        OAImport.logSystem.log( "COND Vector: " + cond.toString() + " size: " + cond.size() ) ;

                        /* set the condition string of cond to condList*/
                        if ( cond.size() > 1 ) {
                           condList = (String) cond.elementAt( 1 ) ;
                           OAImport.logSystem.log("CONDLIST " + condList.toString() ) ;
                        }

                        try {

                           /* get one of the answers for the current question */
                           /* based upon the answer label given by cond first */
                           /* element                                         */
                           Answer answer  = new Answer() ;
                           try {
                              int index      = Integer.parseInt( num ) - 1 ;
                              answer         = (Answer)answersV.elementAt( index ) ;

                           } catch ( NumberFormatException nfe ) {
                              OAImport.logSystem.log( "KBIO: readQuestion: BAD INDEX " + source + questionID + "\n" ) ;
                              OAImport.logSystem.logSevereError( "KBIO: readQuestion: " + source + questionID + "\n" + nfe + "\n" ) ;
                              OAImport.logError( nfe ) ;
                           }
                           if ( answer == null ) {
                              OAImport.logSystem.log( "ANSWER IS NULL" ) ;
                           }

                           /* Set the Answer Conditions here */
                           if ( cond != null ) {

                              if ( cond.size() > 1 ) {                                 // cond has ans label and conditions string
                                 String condStr        = condList ;                     // the string containing the conditions only
                                 Vector tokenVec       = ImportUtil.tokenizeString( condStr ) ;    // the terms and operators in a vector
                                 Vector terms          = new Vector() ;
                                 Vector termsVec       = new Vector() ;
                                 Vector condTermsVec   = new Vector() ;
                                 Vector conditionsVec  = new Vector() ;
                                 String brandGeo       = (String)indDescript.elementAt(1) + "." + (String)indDescript.elementAt(2) ;
                                 Question qObject      = new Question() ;
                                 Answer ansObject      = new Answer() ;
                                 Conditions conditions = answer.getConditions() ;
                                 int qInd, ansInd ;

                                 System.out.println( "ADDING CONDITIONS, file " + key ) ;
                                 OAImport.logSystem.log( "Adding Conditions, file " + key ) ;
                                 OAImport.logSystem.log( "tokenvec " + tokenVec.toString() ) ;
                                 /* case of a single condition for this answer -->  tokenVec = [Q1:5] */
                                 if ( tokenVec.size()  == 1 ) {
                                    /* check that the term contains a Q&A pair */
                                    if ( regex.match( "/:[0-9]+/", (String)tokenVec.elementAt(0) ) ) {
                                       /* obtain the Q&A pair --> [Q1,5] */
                                       terms = ImportUtil.getTerm( (String)tokenVec.elementAt(0) ) ;
                                       OAImport.logSystem.log( "Terms " + terms.toString() ) ;
                                       /* pass the term and hash table to obtain the */
                                       /* corresponding Q&A database indicators      */
                                       termsVec = ImportUtil.getConditionTerms( terms, hashtable, (String)indDescript.elementAt(1) ) ;
                                       if ( termsVec == null ) {
                                          OAImport.logSystem.log( "Could not find term " + termsVec.toString() ) ;
                                       } else {
                                          /* assign the Q&A indicators to form a condition term object */
                                          qObject   = (Question)termsVec.elementAt(0) ;
                                          ansObject = (Answer)termsVec.elementAt(1) ;
                                          ConditionTerm condTerm = new ConditionTerm( qObject, ansObject ) ;
                                          /* this will add the new condition to the answer conditions */
                                          conditions.addTerm( condTerm ) ;
                                       }
                                    } else
                                       OAImport.logSystem.log( "Missing a term: " + (String)tokenVec.elementAt(0) ) ;
                                 } else {
                                    /* case of multiple conditions for this answer  */
                                    /* convert the token vector to a postfix vector */
                                    Vector postfixVec = ImportUtil.getPostfix( tokenVec ) ;
                                    OAImport.logSystem.log( "Postfix Vector " + postfixVec.toString() ) ;
                                    /* if the postfix vector is available, pass it to */ 
                                    /* set the condition terms and set the conditions */
                                    for ( int i = 0 ; i < postfixVec.size() ; i++ ) {
                                       /* check that the term contains a Q&A pair */
                                       if ( regex.match( "/:[0-9]+/", (String)postfixVec.elementAt(i) ) ) {
                                          terms = ImportUtil.getTerm( (String)postfixVec.elementAt(i) ) ;     // terms vector. ie: [q2,4,&]
                                          if ( terms.size() > 1 ) {       // it returned a pair of Q&A
                                             termsVec = ImportUtil.getConditionTerms( terms, hashtable, (String)indDescript.elementAt(1) ) ;    // a vector of: [Qobj,Aobj,ops]
                                          }
                                          if ( termsVec == null ) {
                                             OAImport.logSystem.log( "Could not find term " + termsVec.toString() ) ;
                                          } else { 
                                             /* assign the Q&A indicators to form a condition term object */
                                             qObject   = (Question)termsVec.elementAt(0) ;
                                             ansObject = (Answer)termsVec.elementAt(1) ;
                                             qInd      = qObject.getInd() ;
                                             ansInd    = ansObject.getInd() ;
                                             if ( termsVec.size() == 3 ) {
                                                ConditionTerm condTerm = new ConditionTerm( qInd, ansInd, (String)termsVec.lastElement() ) ;
                                                /* this will add the new condition to the answer conditions */
                                                conditions.addTerm( condTerm ) ;
                                             } else {
                                                ConditionTerm condTerm = new ConditionTerm( qObject, ansObject ) ;
                                                /* this will add the new condition to the answer conditions */
                                                conditions.addTerm( condTerm ) ;
                                             }
                                          }
                                       } // if regex.match ":"
                                       else 
                                          OAImport.logSystem.log( "Missing a term: " + (String)postfixVec.elementAt(i) ) ;
                                    }  // for  
                                 }  // if

                              }  // Answer conditions set
                           } // if cond vector not null

                        } catch( ArrayIndexOutOfBoundsException aioobe )  {
                           OAImport.logSystem.log ( "** Exception Reading Answers: "  + source + questionID+ "\n" ) ;
                           OAImport.logSystem.logSevereError ( "Reading Answer : " + source + questionID+ "\n" ) ;
                           OAImport.logSystem.logSevereError ( "Bad Answer: \n" + aioobe + "\n" ) ;
                           OAImport.logError( aioobe ) ;
                           aioobe.printStackTrace() ;
                        }
                     } // if num != 0
                  } // fix conditions groupings
              } //while
           }  // if astr != null
           

           tempinput = "" ;
           in.close() ;

           /*******************************/
           /* Set the ANSWER CONCLUSIONS  */
           /* Reset to begining of file   */
           /*******************************/

           in = openResourceFile( source, questionID ) ;
           /* Find the begining of the conclusion list */
           while (((astr = in.readLine()) != null) && (!astr.equals("START CONCLUSION"))) {
           }
           
           if ( astr != null ) {
              System.out.println( "Reading Conclusions for question: " + questionID ) ;
              OAImport.logSystem.log( "Reading Conclusions for question: " + questionID ) ;
           }
           
           /* Read in the conclusions */

           Vector conclusions = new Vector() ;
           Answer answer      = new Answer() ;
           int ansIndex ;

           while (((astr = in.readLine()) != null) && (!astr.equals("END CONCLUSION"))) {
               Vector a  = regex.split("/,/",astr,2) ;                     // a(0) = answerlabel + conditions, a(1) = conclusions
               Vector ac = regex.split("/:/",(String)a.elementAt(0),2) ;   // ac(0) = answerlabel, ac(1)=conditions
               try {
                  /* a check for conclusions without an answer label */
                  if ( a.size() > 0 && ac.size() > 0 ) {
                     /* check for answer label == 0, as in Q10:0 */
                     if ( !((String)ac.elementAt(0)).equals( "0" )  ) {
                        ansIndex = Integer.parseInt( (String)ac.elementAt(0) ) ;
                        OAImport.logSystem.log( "conclusions: " + (String)a.elementAt(1) ) ;
                        answer = (Answer)answersV.elementAt( ansIndex - 1 ) ;

                        try {
                           /* check that the conclusion string contains Q&A pairs and no malformed conclusions */
                           if ( (a.size()>1) && regex.match ( "/[[0-9],]/", (String)a.elementAt(1) ) ) {
                              OAImport.logSystem.log( "No conclusions. Malformed conclusion. " + questionID + "/" + (String)a.elementAt(1) ) ;
                           } 
                           else if ( (a.size() > 1) && regex.match( "/Q[0-9]+:[0-9]+[&Q0-9:]*/", (String)a.elementAt(1) ) ) {
                              String conclusionsStr = regex.toString() ;
                              conclusions = setAnswerConclusions( a, indDescript, answer, key, hashtable, conclusionsStr ) ;
                              answer.setConclusions( conclusions ) ;
                           } else {
                              OAImport.logSystem.log( "No Conclusions available for this question/answer " + questionID + "/" + (String)ac.elementAt(0) ) ;

                              /*if (ac.size() > 1) {
                                conclusion.setConditions(((String)ac.elementAt(1)).trim());
                              } */
                           }
                        } catch( ArrayIndexOutOfBoundsException aioobe ) {
                           OAImport.logSystem.log ( "** Exception Reading: "+ source + questionID + "\n" ) ;
                           OAImport.logSystem.logSevereError ( "Bad Conclusion: \n" + aioobe + "\n" ) ;
                           OAImport.logError( aioobe ) ;
                        }
                     }   // if answer label == 0
                  }
               } catch ( Exception e ) {
                  OAImport.logSystem.log("Exception @ setting conclusions: " + astr ) ;
               }

           } // while reading conclusions
           in.close() ;
       }
       // catch both of the possible errors from this 'try'
       catch (FileNotFoundException e) {
           System.out.println( "FileNotFoundException in readQuestion!\n" ) ;
           OAImport.logError( e ) ;
           e.printStackTrace();
           return null;
       }
       catch (IOException e) {
           System.out.println("IOException in readQuestion!\n") ;
           OAImport.logError( e ) ;
           e.printStackTrace() ;
           return null;
       }

       return hashtable ;

    } // readQuestion() 2

    /**
     * setAnswerConclusions
     *
     * this method will actually set the conclusions to a question
     * and returns a vector of all the conclusions found in the resource file
     *
     * @param conclVec, a vector of conclusion strings
     * @param indDescript, a vector containing the geo,brand,and docclass
     * @param answer, the current answer containing the conclusions
     * @param key, the key by which to search a question by brand in the question
     *             table
     * @param questionTable, the table of questions
     * @param concList, a string of conclusions
     * @return a vector of conclusions
     */
    public Vector setAnswerConclusions( Vector conclVec, Vector indDescript, Answer answer, String key,
                                        Hashtable questionTable, String concList ) {
       
       String concStr          = concList ;                                   // the string containing the conclusions only
       Vector tokenVec         = ImportUtil.tokenizeConclusion( concList ) ;  // the Q&A pairs in a vector
       Vector terms            = new Vector() ;
       Vector termsVec         = new Vector() ;
       Vector condTermsVec     = new Vector() ;
       Vector conclusionList   = new Vector() ;
       Question   toQuestion   = new Question() ;
       Answer     toAnswer     = new Answer() ;
       Vector     answersList  = new Vector() ;
       Perl5Util  regex        = new Perl5Util();

       if ( conclVec.size() > 1 ) {                                             // conclVec has ans label and conditions string

         System.out.println( "ADDING CONCLUSIONS, file " + key ) ;
         OAImport.logSystem.log( "Adding Conclusions, file " + key ) ;
         OAImport.logSystem.log("tokenvec " + tokenVec.toString() ) ;
         try {
            /* the tokenized conclusion string is in the form [Q1:5,Q3:2,Q9:1,...]  */
            /* set the condition terms and set the conclusions                      */
            for ( int i = 0 ; i < tokenVec.size() ; i++ ) {
               /* check that the term contains a Q&A pair */
               if ( regex.match( "/:[0-9]+/", (String)tokenVec.elementAt(i) ) ) {
                  /* get the terms vector in the form -->  [q2,4] as strings */
                  terms = ImportUtil.getTerm( (String)tokenVec.elementAt(i) ) ;
                  /* get the condition terms as Q&A objects --> [Qobj,Aobj] */
                  if ( terms.size() > 1 ) {
                     termsVec = ImportUtil.getConditionTerms( terms, questionTable, (String)indDescript.elementAt(1) ) ;
                     if ( !termsVec.isEmpty () ) {
                        /* assign the Q&A indicators to form a condition term object */
                        toQuestion = (Question)termsVec.elementAt(0) ;
                        toAnswer   = (Answer)termsVec.elementAt(1) ;
                        AnswerConcl ansConclusion = new AnswerConcl( toQuestion, toAnswer ) ;
                        ansConclusion.setDocClass( answer.getDocClass() ) ;
                        /* Add the countries from the answer */
                        ansConclusion.setCountryList( answer.getCountryList() ) ;
                        conclusionList.addElement( ansConclusion ) ;
                     }
                  }

               } // if regex.match ":"
               else 
                  OAImport.logSystem.log( "Missing a term: " + (String)tokenVec.elementAt(i) ) ;
            }  //for  
         } catch( Exception e ) {
            OAImport.logSystem.log( "Exception: " + e ) ;
            OAImport.logError( e ) ;
            e.printStackTrace() ;
         }
      
      }  // if cond.size > 1
      OAImport.logSystem.log( "CONCLUSIONLIST: " + conclusionList.toString() ) ;
      return conclusionList ;

    }  /* setAnswerConclusions    */


    /**
     * Sets the document links based on the passed parameter
     *
     * @param question object
     * @param addtlInfo object
     * @param input text to be parsed
     * @param embedded boolean to check for embedded links
     * @param 
     */
    public Question setAddInfoLinks( Question question, AddtlInfo addtlInfo, String input, boolean embedded ) {
       
       Vector        linkTitles   = null ;
       Vector        linkVec      = new Vector() ;
       Question newQuestion  = question ;
       DocLink docLink   = null ;
       
       OAImport.logSystem.log( "Reading Additional Info Links" ) ;
       if ( embedded ) {
          /* search for embedded links here */
          newQuestion = getLinks( newQuestion, addtlInfo, input.trim(), embedded ) ;
          
       } else {  // search for doclinks here
          newQuestion = getLinks( newQuestion, addtlInfo, input.trim(), embedded ) ;
       } // if embedded
       
       if ( newQuestion == null) {
          OAImport.logSystem.log("SETADDINFOLINKS NEW ACTION IS NULL\n") ;
       }
       return newQuestion ;

    } // setAddInfoLinks 

    /**
     * Sets the document links based on the passed parameter
     * @param question object
     * @param addtlInfo object
     * @param input text to be parsed
     * @param embedded boolean flag set to true or false
     */
    public Question getLinks( Question question, AddtlInfo addtlInfo, String input, boolean embedded ) {
       
       final String        docLinkTag      = "###doclink###" ;
             String        aLink        = null ;
             String        questUrl     = "" ;
             String        lnDocId     = null ;
             String        linkTitle    = null ;
             String        addInfoStr   = input.trim() ;
             String        string1      = null ;
             String        string2      = null ;
             int           tagPos       = 0 ;
             int           pos          = 0 ;            // Position of the current HREF being parsed.
             int           nextpos      = 0 ;            // End of the current HREF tag being parsed.
             int           docId        = 0 ;
             Vector        docIdVec     = new Vector() ;
             Vector        links        = new Vector() ; 
             Vector        addInfoVec   = new Vector() ; 
             Vector        titles       = new Vector() ;
             Perl5Util     regex        = new Perl5Util();
             QuestLink     questLink    = null ;
             ExternalLink  externalLink = null ;
             DocLink       docLink      = null ;
             AddtlInfo     nwAddtlInfo  = addtlInfo ;
             Question      newQuestion  = question ;
       /* First we check to see if a link exists in the additional info text */
       /* Be searching for the tag <A HREF                                   */
       if ( input.indexOf( "<A HREF", pos ) != -1  || input.indexOf( "<a href", pos ) != -1 ) {
          //OAImport.logSystem.log( "addinfostr before:" + addInfoStr ) ;
          /*  We parse each link in this while loop until there are no more */
          /*  links to process.  pos is advanced one link at a time.        */
          while ( ( addInfoStr.indexOf( "<A HREF", pos ) != -1  || addInfoStr.indexOf( "<a href", pos ) != -1 ) ) {
             
             /* Set the position of the beginning of the link */
             if ( addInfoStr.indexOf( "<A HREF", pos ) != -1  ) {
               pos = addInfoStr.indexOf( "<A HREF", pos ) ;
             }
             else
               pos = addInfoStr.indexOf( "<a href", pos ) ;
             
             /* Set the ending position of the HREF           */
             nextpos = addInfoStr.indexOf( ">", pos ) ;
             /* Set a substring which contains only the link  */
             aLink = addInfoStr.substring( pos, nextpos ) ;
             
             /* Log our position and links */

             /* We must determine if this link is a quest link */
             /* Match all links which follow this rule         */
             /* <a href followed by one or more of any character */
             /* followed by http followed by 0 or more chars     */
             /* followed by 0 or more question marks             */
             /* followed by 0 or more of any character           */
             /* followed by 1 or more of any uppercase chars     */
             /* followed by a -                                  */
             /* followed by 1 or more uppercase alpha numerics   */
             /* followed by 0 or more quote characters           */
             /* Example. <a href=http://www.pc.ibm.com/support/page=AFDFE-93HDS"> LInk Title </a> */
             /*          -------------------------------------------------------                  */
             /* Dashed portion above would match */
             if ( regex.match ( "/<a href.*http.*[\\?]*.*[A-Z]+-[0-9A-Z]+[\"]*/", aLink ) ||
                  regex.match ( "/<A HREF.*http.*[\\?]*.*[A-Z]+-[0-9A-Z]+[\"]*/", aLink ) ) {
                
                // get the preMatch and postMatch strings 
                /* set the position of the embeddedLink within the addition info */
                tagPos = addInfoStr.indexOf( aLink, 0 ) ;

                /* string1 is All additional info before the tag */
                string1 = addInfoStr.substring( 0, tagPos ) ;
                /* This is all aditional info after the last character of the embeddedLink */
                string2 = addInfoStr.substring( tagPos+aLink.length(), addInfoStr.length() ) ;
                /* We now remove the anchor <a href  */
                /* We do this by matching the http portiion of the embedded link */
                /* until the end of the anchor tag  */
                /* From examle above this would leave */
                /* http://www.pc.ibm.com/support/page=AFDFE-93HDS" */
                /* note: this may or may not have an ending quote */
                if ( regex.match ( "/http.*[\\?]*.*[A-Z]+-[0-9A-Z]+[\"]*/", aLink ) ) {
                   /* This matches if we have an ending quote   */
                   /* This results in removing the ending quote */
                   //if ( regex.match( "/http.*[A-Z]*-[0-9A-Z]*(?=\")/", regex.toString() ) ) {
                   questUrl = regex.toString() ;                              // the url w/o "
                   /* Remove an ending quote if it exists */
                   if ( (questUrl.trim()).endsWith("\"")) {
                      /* Removes the ending quote character */
                      questUrl = questUrl.substring( 0, questUrl.length() - 1 ) ;
                   }
                   /* Match to obtain the Lotus Notes ID */
                   if ( regex.match( "/[A-Z]*-[0-9A-Z]*/", questUrl ) ) 
                      lnDocId = regex.toString() ;                                    // the LN id
                   /* Get the Quest DocId from the database */                     
                   docId = getDocIndFromLNInd( lnDocId.trim() ) ;                            // getting the quest docind from database
                   /* set the title of the link */
                   linkTitle = addInfoStr.substring( addInfoStr.indexOf(">", nextpos)+1, addInfoStr.indexOf("</",nextpos) ) ;   // link title
                   /* Check that the title size does not exceed the */
                   /* max size limit in the database                */
                   int maxStrLength = 250 ;
                   if ( linkTitle.length() > maxStrLength ) {
                      OAImport.logSystem.log( "Setting the link title size to 250 characters." ) ;
                      linkTitle = (linkTitle.substring( 0, maxStrLength )).trim() ;
                   }
                   
                   /* checking whether the docId exists in the database */
                   if ( docId == 0 ) {
                      if ( !embedded ) {
                         /* Reconstruct the additional info replacing the embedded link */
                         /* with our new link which contains the meta tag in place of   */
                         /* the server name and path of the url                         */
                         addInfoStr = ImportUtil.removeLine( addInfoStr, tagPos ) ;
                         /* We have changed the additional info text    */
                         /* update the next pos to the index of string2 */
                         nextpos = string1.length() ;
                         OAImport.logSystem.log( "removing: " + aLink ) ;
                      }
                      OAImport.logSystem.log( "WARNING!!: Requested Document is not valid for: " + lnDocId ) ; 
                   } else {
                      /* If the link is embedded just add the docId to  */
                      /* the emmbeddedind vector                        */
                      if ( embedded ) {
                         /* set the embedded indicator of the Additl. Info object */
                        docIdVec.addElement( new Integer ( docId ) ) ;
                        nwAddtlInfo.setEmbeddedDocInds( docIdVec ) ;                     
                        //links.addElement( questLink ) ;
                      } else { 
                           /* Create a questlink object and add to vector of doclinks */
                           docLink = ImportUtil.createQuestDocLink ( RFilesImporter.questDocLinksTable, docId, linkTitle.trim() ) ;  
                           //questLink = new QuestLink( docId, linkTitle.trim() ) ;            
                           //questLink.writeToDatabase() ;
                           OAImport.logSystem.log( "DOCID, LINKTILE:" + docId + " " + linkTitle ) ;
                           //docLink = new DocLink ( questLink ) ;
                           ImportUtil.setDocClassForConditionalObject( docLink, question.getDocClass() ) ;
                           ImportUtil.mergeCountries ( docLink, question.getCountryList() ) ;
                           //docLink.setDocClass( question.getDocClass() ) ;
                           //docLink.setCountryList( question.getCountryList() ) ;
                           if ( !links.contains(docLink) ) {
                              links.addElement( docLink ) ;
                           }
                      } // if embedded
                      /* substitute the url with the doclink meta tag */
                      aLink = RegularExpression.substitute( "(http.*/[^A-Z]*)([A-Z]+-[0-9A-Z]+.*)", docLinkTag +"/"+"$2", questUrl ) ;
                      /* Reconstruct the additional info replacing the embedded link */
                      /* with our new link which contains the meta tag in place of   */
                      /* the server name and path of the url                         */
                      addInfoStr = string1 + aLink + string2 ;
                      /* We have changed the additional info text    */
                      /* update the next pos to the index of string2 */
                      nextpos = string1.length() + aLink.length() ;
                   } // if docId == 0
                        
                   newQuestion.setDocLinks( links ) ;
                   
                } // if regex.match
             
             } else {  // this is not a quest link
                  
                  if ( (addInfoStr.indexOf( "<A HREF", pos )) != -1 ) {
                     pos = addInfoStr.indexOf( "<A HREF", pos ) ;
                  }
                  else if ( (addInfoStr.indexOf( "<a href", pos )) != -1  ) {
                     pos = addInfoStr.indexOf( "<a href", pos ) ;
                  }
                  nextpos = addInfoStr.indexOf( ">", pos ) ;
                  aLink = addInfoStr.substring( pos, nextpos ) ;        
                  if ( regex.match ( "/HTTP.*(?=\")/", aLink ) || regex.match ( "/http.*(?=\")/", aLink ) ) 
                     aLink = regex.toString() ;                          // the actual url
                  linkTitle = addInfoStr.substring( addInfoStr.indexOf( ">", nextpos )+1, addInfoStr.indexOf( "<", nextpos ) ) ;
                  if ( regex.match("/[^>].*/", linkTitle) ) {
                     linkTitle = regex.toString() ;
                  }
                  /* Check that the title size does not exceed the */
                  /* max size limit in the database                */
                  int maxStrLength = 250 ;
                  if ( linkTitle.length() > maxStrLength ) {
                     OAImport.logSystem.log( "Setting the link title size to 250 characters." ) ;
                     linkTitle = linkTitle.substring( 0, maxStrLength ) ;
                  }
                  // add the links
                  docLink = ImportUtil.createExternalDocLink( RFilesImporter.externDocLinksTable, aLink, linkTitle.trim() ) ;
                  //externalLink = new ExternalLink( aLink, linkTitle.trim() ) ;
                  //externalLink.writeToDatabase() ;
                  
                  //docLink = new DocLink( externalLink ) ;
                  //docLink.setDocClass( question.getDocClass() ) ;
                  //docLink.setCountryList( question.getCountryList() ) ;
                  ImportUtil.setDocClassForConditionalObject( docLink, question.getDocClass() ) ;
                  ImportUtil.mergeCountries ( docLink, question.getCountryList() ) ;
                  if ( !links.contains(docLink) ) {
                     links.addElement ( docLink ) ;
                  }
                  newQuestion.setDocLinks( links ) ;

             }
             /* We must move to the end of the current link */
             /* The next iteration of the while loop will   */
             /* search from this point for the next link    */
             pos = nextpos ;
          }  //while
          //OAImport.logSystem.log( "addinfostr after: " + addInfoStr ) ;
          nwAddtlInfo.setAddtlInfo ( nwAddtlInfo.getAddtlInfo() + addInfoStr ) ;
       } else {
             nwAddtlInfo.setAddtlInfo( nwAddtlInfo.getAddtlInfo() + addInfoStr ) ;
       } 
       addInfoVec.addElement( nwAddtlInfo ) ;
       /* Add the new additional info object to the Question object */
       newQuestion.setAdditionalInfo( addInfoVec ) ;

       return  newQuestion ;

    } //getLinks

                                        
    /**
     * Reads a Knowledge Base Action Resource file into an Action object
     * @param source Source diretory in which to find the Action.
     * @param actionID
     * @param indDescript a vector of indicator descriptions
     */
    public synchronized Action readAction( String source, String actionID, Vector indDescript ) {

        String          astr        = null ;        // input string
        String          tempinput   = null ;        // temporary container
        Vector          addtlInfoV  = new Vector () ;
        Vector          titlesV     = new Vector () ;
        BufferedReader  in          = null ;
        AddtlInfo       addtlInfo   = new AddtlInfo() ;
        Action          mainAction  = new Action() ;
        Title           title       = null ;

        OAImport.logSystem.log( "\n------- Reading Action -------\nFile: " + source + actionID ) ;

        /* set the geo, brand and docclass indicators */
        int             geoInd = OAImport.typeList.indFromString( (String)indDescript.elementAt( 0 ), OAImport.typeList.TYPE_GEOGRAPHY ) ;
        TypeGeoRec      geoRec = new TypeGeoRec( geoInd, (String)indDescript.elementAt( 0 ) ) ;
        
        mainAction.setCountryList( geoRec.getCountryVec() ) ;
        mainAction.setBrand( (TypeBrandRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(1), OAImport.typeList.TYPE_BRAND ) ) ;
        /* set all actions to be external */
        mainAction.setDocClass ( (TypeDocClassRec)OAImport.typeList.objectFromString( "Unclassified", OAImport.typeList.TYPE_DOCCLASS ) ) ;
        tempinput="";

        try {
            in = openResourceFile(source, actionID);

            // Find the begining of the action text
            while(((astr = in.readLine()) != null) && (!astr.equals("START ACTION"))) {
            }
            // Read in the text of the action
            while(((astr = in.readLine()) != null) && (!astr.equals("END ACTION"))) {
                tempinput=tempinput+astr;
            }
            /* Check that the title size does not exceed the */
            /* max size limit in the database                */
            int maxStrLength = 250 ;
            if ( tempinput.length() > maxStrLength ) {
               OAImport.logSystem.log( "Setting the title size to 250 characters." ) ;
               tempinput = tempinput.substring( 0, maxStrLength ) ;
            }
            title = new Title( tempinput.trim() ) ;
            title.setCountryList( geoRec.getCountryVec() ) ;
            title.setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ;
            title.setLanguage( new TypeLanguageRec( 1, "" ) ) ;
            titlesV.addElement( title ) ;
            mainAction.setTitles( titlesV ) ;  // Save the action title
            tempinput="";

            // Find the begining of the Explain Text(Additional Info)
            while(((astr = in.readLine()) != null) && (!astr.equals("START EXPLAIN"))) {
            }

            // Read in the Explain Text for the question
            while(((astr = in.readLine()) != null) && (!astr.equals("END EXPLAIN"))) {
               tempinput=tempinput+astr+"\n";
            }
            int index ;
            if ( !tempinput.equals("") ) {
               OAImport.logSystem.log ( "Reading Additional Information" ) ;
               /* Set the #REDIRECT flag if it exists */
               if ( regex.match( "/#REDIRECT/", tempinput ) ) {
                  addtlInfo.setRedirectFlag( true ) ;
               }
               /* Set the additional information object                       */
               /* the additional info text has an Additional Information area */
               /* boolean flag indicates to look or not for an embedded link  */
               /* true = embedded link                                        */
               if ( (index = tempinput.indexOf( "Additional Information:" )) != -1 ) {
                  String explain = tempinput.substring( 0, index ) ;
                  mainAction = setAddInfoLinks ( mainAction, addtlInfo, explain, true ) ;
                  String addInfotxt = tempinput.substring ( index, tempinput.length() ) ;
                  mainAction = setAddInfoLinks ( mainAction, addtlInfo, addInfotxt, false ) ;
                 
                 /* the additional info has <LI> tags only */
               } else if ( (index = tempinput.indexOf( "<LI>" )) != -1 ) {   
                  String explain = tempinput.substring( 0, index ) ;
                  mainAction = setAddInfoLinks ( mainAction, addtlInfo, explain, true ) ;
                  String addInfotxt = tempinput.substring ( index, tempinput.length() ) ;
                  mainAction = setAddInfoLinks ( mainAction, addtlInfo, addInfotxt, false ) ;

               } else {  /* process additional info txt with no Add Info area or no <LI> tags in it */
                  mainAction = setAddInfoLinks( mainAction, addtlInfo, tempinput, true ) ;
               }

               if ( addtlInfo.getRedirectFlag() ) {
                  OAImport.logSystem.log( "#REDIRECT flag is set" ) ;
               }

               /* set the additional information's Geo and DocClass */
               addtlInfoV = mainAction.getAdditionalInfo() ;
               if (addtlInfoV == null ) {
                  OAImport.logSystem.log( "AddtlInfoV is null, creating a new empty Vector" + actionID ) ;
                  addtlInfoV = new Vector () ;
               } else
               OAImport.logSystem.log( "AddtlInfoV is not Null" ) ;
               for ( int i = 0 ; i < addtlInfoV.size() ; i++ ) {
                  ((AddtlInfo)addtlInfoV.elementAt(i)).setCountryList( geoRec.getCountryVec() ) ;
                  ((AddtlInfo)addtlInfoV.elementAt(i)).setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ;
                  ((AddtlInfo)addtlInfoV.elementAt(i)).setLanguage( new TypeLanguageRec( 1, "" ) ) ;
               }

               mainAction.setAdditionalInfo( addtlInfoV ) ;
            }
            tempinput="";

            in.close() ;
        } /* catch both of the possible errors from this 'try' */
        catch (FileNotFoundException e) {
           OAImport.logSystem.log ( "EXCEPTION: FileNotFoundException, see error log file" + source + actionID ) ;
           OAImport.logSystem.logSevereError ( "EXCEPTION: " + source + actionID + "\n" + e ) ;
           OAImport.logError( e ) ;
           return null;
        }
        catch (IOException e) {
           OAImport.logSystem.log ( "EXCEPTION: IOException, see error log file" + source + actionID ) ;
           OAImport.logSystem.logSevereError ( "EXCEPTION: " + source + actionID + "\n" + e ) ;
           OAImport.logError( e ) ;
           return null;
        }
        /* set the type of Action */
        mainAction.setActionType( new TypeActionRec( 1, "Default" ) ) ;

        return mainAction ;
    } // readAction()

    
    /**
     * Sets the document links based on the passed parameter
     * @param question object
     * @param addtlInfo object
     * @param input text to be parsed
     * @param embedded boolean flag set to true or false
     */
    public Action setAddInfoLinks( Action action, AddtlInfo addtlInfo, String input, boolean embedded ) {
       
       Action newAction  = action ;
       DocLink docLink   = null ;
       
       OAImport.logSystem.log( "Reading Additional Info Links" ) ;
       if ( embedded ) {
          // search for embedded links here
          newAction = getLinks( newAction, addtlInfo, input, embedded ) ;
       } else {  // search for doclinks here
          newAction = getLinks( newAction, addtlInfo, input, embedded ) ;
       } // if embedded
          
       return newAction ;

    } // setAddInfoLinks

    
    public int getDocIndFromLNInd( String lnDocInd ) {

       SQLMethod     sqlMethod    = null ;
       Statement     statement    = null ;
       ResultSet     resultSet    = null ;
       int           docInd       = 0 ;
       String        query        = null ;

       if ( docIdTable == null ) {
          docIdTable = new Hashtable() ;
          sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "KBIO.getDocIndFromLNInd ("+ lnDocInd +") ", 5 ) ;
          statement = sqlMethod.createStatement() ;

          try {
             resultSet = statement.executeQuery( "select docind,lndocid from tigris.documents where publish='Y'") ;

             int count = 0; 
             while ( resultSet.next() ) {
                count++ ;
                if ( (count % 100) == 0) {
                   System.out.print( "Reading docId count: "+ count+ "      \r") ;
                }
                docInd = resultSet.getInt( 1 ) ;
                String lotusId = resultSet.getString( 2 ) ;
                docIdTable.put( lotusId, new Integer( docInd ) ) ;
                //query = sqlMethod.getStatementSQL() ;
             }
             System.out.println("\nFilled docId table") ;
          } catch ( SQLException sqle ) {
             OAImport.logSystem.log ( "SQLException: see error file for message" ) ;
             OAImport.logSystem.logSevereError( "SQLException: lnDocInd: " + lnDocInd + "\n" + sqle.toString() ) ;
             OAImport.logError( sqle ) ;
          } catch ( Exception e ) {
             OAImport.logSystem.log( "WARNING!!: Could not find DocId in database for : " + lnDocInd ) ;
             OAImport.logError( e ) ;
             e.printStackTrace() ;
          }
          sqlMethod.close() ;
       }

       Integer id = (Integer) docIdTable.get( lnDocInd ) ;
       if ( id != null) {
          return ( id.intValue() ) ;
       }
       return( 0 ) ;

    } // getDocIndFromLNInd

    /**
     * Sets the document links based on the passed parameters
     *
     * @param question object
     * @param addtlInfo object
     * @param input text to be parsed
     * @param embedded boolean to check for embedded links
     * @param 
     */
    public Action getLinks( Action action, AddtlInfo addtlInfo, String input, boolean embedded ) {
       
       final String        docLinkTag      = "###doclink###" ;
             String        aLink           = null ;
             String        questUrl        = "" ;
             String        lnDocId         = null ;
             String        linkTitle       = null ;
             String        addInfoStr      = input.trim() ;
             String        string1         = null ;
             String        string2         = null ;
             String        liTag           = "" ;
             int           tagPos          = 0 ;
             int           pos             = 0 ;            // Position of the current HREF being parsed.
             int           nextpos         = 0 ;            // End of the current HREF tag being parsed.
             int           docId           = 0 ;
             Vector        docIdVec        = new Vector() ;
             Vector        links           = new Vector() ; 
             Vector        addInfoVec      = new Vector() ; 
             Vector        titles          = new Vector() ;
             Perl5Util     regex           = new Perl5Util();
             QuestLink     questLink       = null ;
             ExternalLink  externalLink    = null ;
             DocLink       docLink         = null ;
             AddtlInfo     nwAddtlInfo     = addtlInfo ;
             Action        newAction       = action ;

       /* First we check to see if a link exists in the additional info text */
       /* Be searching for the tag <A HREF                                   */
       if ( input.indexOf( "<A HREF", pos ) != -1  || input.indexOf( "<a href", pos ) != -1 ) {
          //OAImport.logSystem.log( "addinfostr before: " + addInfoStr ) ;
          /*  We parse each link in this while loop until there are no more */
          /*  links to process.  pos is advanced one link at a time.        */
          while ( ( addInfoStr.indexOf( "<A HREF", pos ) != -1  || addInfoStr.indexOf( "<a href", pos ) != -1 ) ) {
             
             /* Set the position of the beginning of the link */
             if ( addInfoStr.indexOf( "<A HREF", pos ) != -1  ) {
               pos = addInfoStr.indexOf( "<A HREF", pos ) ;
             }
             else
               pos = addInfoStr.indexOf( "<a href", pos ) ;
             
             /* Set the ending position of the HREF */
             nextpos = addInfoStr.indexOf( ">", pos ) ;
             /* Set a substring which contains only the link --> <a href= "http://..."> */
             aLink = addInfoStr.substring( pos, nextpos ) ;
             
             /* Log our position and links */

             /* We must determine if this link is a quest link   */
             /* Match all links which follow this rule           */
             /* <a href followed by one or more of any character */
             /* followed by http followed by 0 or more chars     */
             /* followed by 0 or more question marks             */
             /* followed by 0 or more of any character           */
             /* followed by 1 or more of any uppercase chars     */
             /* followed by a -                                  */
             /* followed by 1 or more uppercase alpha numerics   */
             /* followed by 0 or more quote characters           */
             /* Example. <a href=http://www.pc.ibm.com/support/page=AFDFE-93HDS"> Link Title </a> */
             /*          -------------------------------------------------------                  */
             /* Dashed portion above would match */
             if ( regex.match ( "/<a href.*http.*[\\?]*.*[A-Z]+-[0-9A-Z]+[\"]*/", aLink ) ||
                  regex.match ( "/<A HREF.*http.*[\\?]*.*[A-Z]+-[0-9A-Z]+[\"]*/", aLink ) ) {
                
                // get the preMatch and postMatch strings 
                /* set the position of the embeddedLink within the addition info */
                tagPos = addInfoStr.indexOf( aLink, 0 ) ;

                /* string1 is All additional info before the tag */
                string1 = addInfoStr.substring( 0, tagPos ) ;
                /* This is all aditional info after the last character of the embeddedLink */
                string2 = addInfoStr.substring( tagPos+aLink.length(), addInfoStr.length() ) ;
                /* We now remove the anchor <a href                             */
                /* We do this by matching the http portion of the embedded link */
                /* until the end of the anchor tag                              */
                /* From examle above this would leave                           */
                /* http://www.pc.ibm.com/support/page=AFDFE-93HDS"              */
                /* note: this may or may not have an ending quote               */
                if ( regex.match ( "/http.*[\\?]*.*[A-Z]+-[0-9A-Z]+[\"]*/", aLink ) ) {
                   /* This matches if we have an ending quote   */
                   /* This results in removing the ending quote */
                   //if ( regex.match( "/http.*[A-Z]*-[0-9A-Z]*(?=\")/", regex.toString() ) ) {
                   questUrl = regex.toString() ;                                       // the url w/o "
                   /* Remove an ending quote if it exists */
                   if ( (questUrl.trim()).endsWith("\"")) {
                      /* Removes the ending quote character */
                      questUrl = questUrl.substring( 0, questUrl.length() - 1 ) ;
                   }
                   /* Match to obtain the Lotus Notes ID */
                   if ( regex.match( "/[A-Z]*-[0-9A-Z]*/", questUrl ) ) 
                      lnDocId = regex.toString() ;                                    // the LN id
                   /* Get the Quest DocId from the database */
                   docId = getDocIndFromLNInd( lnDocId.trim() ) ;                            // getting the quest docind from database
                   /* set the title of the link */
                   linkTitle = addInfoStr.substring( addInfoStr.indexOf(">", nextpos)+1, addInfoStr.indexOf("</",nextpos) ) ;   // link title
                   /* Check that the title size does not exceed the */
                   /* max size limit in the database                */
                   int maxStrLength = 250 ;
                   if ( linkTitle.length() > maxStrLength ) {
                      OAImport.logSystem.log( "Setting the title size to 250 characters." ) ;
                      linkTitle = linkTitle.substring( 0, maxStrLength ) ;
                   }
                   /* checking whether the docId exists in the database */
                   if ( docId == 0 ) {
                      if ( !embedded ) {
                         /* Reconstruct the additional info replacing the embedded link */
                         /* with our new link which contains the meta tag in place of   */
                         /* the server name and path of the url                         */
                         addInfoStr = ImportUtil.removeLine(addInfoStr, tagPos ) ;
                         /* We have changed the additional info text    */
                         /* update the next pos to the index of string2 */
                         nextpos = string1.length() ;
                         OAImport.logSystem.log( "removing: " + aLink ) ;
                      }
                      OAImport.logSystem.log( "WARNING!!: Requested Document is not in the database: " + lnDocId ) ; 
                   } else {
                      /* If the link is embedded */
                      if ( embedded ) {
                         /* set the embedded indicator of the Additl. Info object */
                        docIdVec.addElement( new Integer ( docId ) ) ;
                        nwAddtlInfo.setEmbeddedDocInds( docIdVec ) ;       // add the quest docid to additional info
                        //links.addElement( questLink ) ;
                      } else {
                           /* call this routine to create a table of quest docLinks */
                           docLink = ImportUtil.createQuestDocLink ( RFilesImporter.questDocLinksTable, docId, linkTitle.trim() ) ;  
                           /* Create a questlink object and add to vector of doclinks */
                           //questLink = new QuestLink( docId, linkTitle.trim() ) ;            // set the quest link
                           //questLink.writeToDatabase() ;
                           OAImport.logSystem.log( "DOCID, LINKTITLE:" + docId + " " + linkTitle ) ;
                           //docLink = new DocLink ( questLink ) ;
                           //docLink.setDocClass( action.getDocClass() ) ;      
                           ImportUtil.setDocClassForConditionalObject( docLink, action.getDocClass() ) ;
                           //docLink.setCountryList( action.getCountryList() ) ;
                           ImportUtil.mergeCountries ( docLink, action.getCountryList() ) ;
                           if ( !links.contains(docLink) ) {
                              links.addElement( docLink ) ;
                           }
                      } // if embedded
                      /* substitute the url with the doclink meta tag */
                      aLink = RegularExpression.substitute( "(http.*/[^A-Z]*)([A-Z]+-[0-9A-Z]+.*)", docLinkTag +"/"+"$2", questUrl ) ;
                      /* Reconstruct the additional info replacing the embedded link */
                      /* with our new link which contains the meta tag in place of   */
                      /* the server name and path of the url                         */
                      addInfoStr   = string1 + aLink + string2  ;                     // the addit. info text
                      /* We have changed the additional info text    */
                      /* update the next pos to the index of string2 */
                      nextpos = string1.length()+ aLink.length() ;
                   } // if docId == 0
                        
                   newAction.setDocLinks( links ) ;

                }  // if regex.match
             
             } else {  // this is not a quest link
                  
                  if ( (addInfoStr.indexOf( "<A HREF", pos )) != -1 ) {
                     pos = addInfoStr.indexOf( "<A HREF", pos ) ;
                  }
                  else if ( (addInfoStr.indexOf( "<a href", pos )) != -1  ) {
                     pos = addInfoStr.indexOf( "<a href", pos ) ;
                  }
                  nextpos = addInfoStr.indexOf( ">", pos ) ;
                  aLink = addInfoStr.substring( pos, nextpos ) ;        
                  if ( regex.match ( "/HTTP.*(?=\")/", aLink ) || regex.match ( "/http.*(?=\")/", aLink ) ) 
                     aLink = regex.toString() ;                          // the actual url
                  linkTitle = addInfoStr.substring( addInfoStr.indexOf( ">", nextpos )+1, addInfoStr.indexOf( "<", nextpos ) ) ;
                  if ( regex.match("/[^>].*/", linkTitle) ) {
                     linkTitle = regex.toString() ;
                  }
                  /* Check that the title size does not exceed the */
                  /* max size limit in the database                */
                  int maxStrLength = 250 ;
                  if ( linkTitle.length() > maxStrLength ) {
                     OAImport.logSystem.log( "Setting the title size to 250 characters." ) ;
                     linkTitle = linkTitle.substring( 0, maxStrLength ) ;
                  }
                  
                  docLink = ImportUtil.createExternalDocLink( RFilesImporter.externDocLinksTable, aLink, linkTitle.trim() ) ;
                  // add the links
                  //externalLink = new ExternalLink( aLink, linkTitle.trim() ) ;
                  //externalLink.writeToDatabase() ;
                  //titles.addElement( linkTitle ) ;
                  //externalLink.setLinkTitles( titles ) ;
                  //docLink = new DocLink ( externalLink ) ;
                  //docLink.setDocClass( action.getDocClass() ) ;
                  //docLink.setCountryList( action.getCountryList() ) ;
                  ImportUtil.setDocClassForConditionalObject( docLink, action.getDocClass() ) ;
                  ImportUtil.mergeCountries ( docLink, action.getCountryList() ) ;
                  if ( !links.contains(docLink) ) {
                     links.addElement ( docLink ) ;
                  }
                  newAction.setDocLinks( links ) ;

             }
             /* We must move to the end of the current link */
             /* The next iteration of the while loop will   */
             /* search from this point for the next link    */
             pos = nextpos ;
          }  //while
          //OAImport.logSystem.log( "addinfostr after: " + addInfoStr ) ;
          /* Need to add any addInfo text that comes before any links */
          nwAddtlInfo.setAddtlInfo ( nwAddtlInfo.getAddtlInfo() + addInfoStr ) ;
       } else {
            nwAddtlInfo.setAddtlInfo( nwAddtlInfo.getAddtlInfo() + addInfoStr ) ;
       } 
       addInfoVec.addElement( nwAddtlInfo ) ;
       /* Add the new additional info object to the Action object */
       newAction.setAdditionalInfo( addInfoVec ) ;

       return  newAction ;

    } //getLinks


    /**
     * Reads a Knowledge Base Symptom Resource file into a Symptom object
     * @param source Source diretory in which to find the Symptom.
     * @param symptomID
     */
    public synchronized Symptom readSymptom( String source, String symptomID, Vector indDescript ) {
        
        String       astr;        // input string
        String       tempinput;   // temporary container
        Edge         tempEdge;
        Symptom      mainSymptom  = new Symptom() ;
        SymptomTitle symptomTitle = new SymptomTitle() ;
        Vector       titlesVec    = new Vector() ;
        Vector       tempVector   = new Vector() ;
        Vector       label        = new Vector() ;
        Vector       conclusions  = new Vector() ;
        Vector       nodesVec     = new Vector() ;
        Perl5Util    regex        = new Perl5Util();
        BufferedReader in;
        
        /* Set the geo, brand and docclass indicators */
        int             geoInd = OAImport.typeList.indFromString( (String)indDescript.elementAt( 0 ), OAImport.typeList.TYPE_GEOGRAPHY ) ;
        TypeGeoRec      geoRec = new TypeGeoRec( geoInd, (String)indDescript.elementAt( 0 ) ) ;
        Vector          countryList = geoRec.getCountryVec() ;
        //mainSymptom.setCountryList( geoRec.getCountryVec() ) ;
        mainSymptom.setBrand( (TypeBrandRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(1), OAImport.typeList.TYPE_BRAND ) ) ;
        /* set all actions to be external */
        mainSymptom.setDocClass ( (TypeDocClassRec)OAImport.typeList.objectFromString( "Unclassified", OAImport.typeList.TYPE_DOCCLASS ) ) ;

        OAImport.logSystem.log( "\n------- Reading Symptom -------\nFile: " + source + symptomID ) ;

        tempinput="";

        try {
            in = openResourceFile( source, symptomID );
            titlesVec = new Vector() ;
            int maxStrLength = 250 ;
            // read the data from the file
            while ( ( astr = in.readLine() ) != null ) {
                if ( astr.startsWith("TITLE:") ) {
                    tempinput=astr.substring( 6 );        // TITLE: is 6 chars long, so only want whats after it.
                    /* Check that the title size does not exceed the */
                    /* max size limit in the database                */
                    if ( tempinput.length() > maxStrLength ) {
                       OAImport.logSystem.log( "Setting the title size to 250 characters." ) ;
                       tempinput = tempinput.substring( 0, maxStrLength ) ;
                    }
                    symptomTitle = new SymptomTitle( tempinput.trim() ) ;
                    symptomTitle.setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) );
                    symptomTitle.setLanguage( new TypeLanguageRec( 1, "" ) ) ;
                    symptomTitle.setCountryList( countryList ) ;
                } // if
                if (astr.startsWith("KEYWORDS:")) {
                    tempinput=astr.substring(9);               // KEYWORDS: is 9 chars long only
                    /* Check that the title size does not exceed the */
                    /* max size limit in the database                */
                    if ( tempinput.length() > maxStrLength ) {
                       OAImport.logSystem.log( "Setting the keywords string size to 250 characters." ) ;
                       tempinput = tempinput.substring( 0, maxStrLength ) ;
                    }
                    symptomTitle.setKeywords( tempinput.trim() ) ;    
                } // if keywords
                // if index status
                if (astr.startsWith("INDEX:")) {
                    tempinput=astr.substring(6).trim(); // INDEX: is 6 chars long, so only want
                    if (tempinput.equals("YES")) {           // whats after it.
                       symptomTitle.setIndexable( true );
                    } else {
                        symptomTitle.setIndexable(false);
                    } // if YES
                } // end index status
                
                /* END FIRST PASS */

            } // while input still available
            titlesVec.addElement( symptomTitle ) ;
            mainSymptom.setSymptomTitles( titlesVec );    

            /* set the type of Symptom */
            mainSymptom.setSymptomType( new TypeSymptomRec( 1, "Default" ) ) ;
            /* Set the Symptom Status */
            mainSymptom.setSymptomStatus( new TypeStatusRec( 13, 4, "" ) ) ;

            /* Set the temporary root node with NodeType, with oaObj indicator */
            /* for first pass we use Q1 already in the DB                      */
            int qInd = 1 ;  // the root question indicator is always 1

            /* Load the root question if it has not been loaded */
            if ( rootQuestion == null ) {
               rootQuestion = Question.readFromDatabase( qInd, 1 ) ;
            }
            Node rootNode = new Node( (OAObject)rootQuestion ) ;
            mainSymptom.setRootNode( rootNode ) ;
            nodesVec.addElement( rootNode ) ;
            mainSymptom.setNodes( nodesVec ) ;
            in.close() ;

        } // try
        catch (FileNotFoundException e) { 
           OAImport.logError( e ) ;
           return null ;
        }
        catch (IOException e) { 
         OAImport.logError( e ) ; 
         return null ; 
        }
        catch ( Exception e ) {
           System.out.println( "1st pass failed:  symptom = " + symptomID ) ;
           OAImport.logError( e ) ;
           e.printStackTrace() ;
        }

        return mainSymptom ;

    } // readSymptom() - 1st pass

    
    /**
     * Reads a Knowledge Base Symptom Resource file into a Symptom object
     * @param source Source diretory in which to find the Symptom.
     * @param symptomID, name of the file without the .r extension
     * @param indDescript, vector of geo,brand and docclass descriptions
     * @param symptomTable, hashtable of symptoms
     * @param questionTable, hashtable of questions
     * @param actionTable, hashtable of questions
     *
     * @return hashtable of symtoms
     */
    public synchronized Hashtable readSymptom ( String source, String symptomID, Vector indDescript, Hashtable symptomTable, 
                                                Hashtable questionTable, Hashtable actionTable, Hashtable nodeTable ) {
       BufferedReader in;
       String         astr;        // input string
       String         tempinput;   // temporary container
       Vector         tempVector,label, conclusions;
       Vector         titlesVec     = new Vector() ;
       Vector         nodesVec      = new Vector() ;
       Vector         nodeList      = new Vector() ;
       Vector         edgeList      = new Vector() ;
       Vector         edgeGroupList = new Vector() ;
       //Hashtable      nodeTable     = new Hashtable() ;
       Perl5Util      regex         = new Perl5Util();
       Symptom        mainSymptom   = null ;
       Question       question      = null ;
       Node           node ;
       Edge           edge ;

       OAImport.logSystem.log( "\n------- Reading Symptom 2nd pass -------\nFile: " + source + symptomID ) ;

/*       Enumeration keyEnum = questionTable.keys() ;
       while ( keyEnum.hasMoreElements() ) {
          OAImport.logSystem.log( keyEnum.nextElement() + "\n" ) ;
       }
       OAImport.logSystem.log( "Question Table contents End -----------" ) ;
       OAImport.logSystem.log( "Action Table contents" ) ;
       keyEnum = actionTable.keys() ;
       while ( keyEnum.hasMoreElements() ) {
          OAImport.logSystem.log( keyEnum.nextElement() + "\n" ) ;
       }
       OAImport.logSystem.log( "Action Table contents End -----------" ) ;
       OAImport.logSystem.log( "Symptom Table contents" ) ;
       keyEnum = symptomTable.keys() ;
       while ( keyEnum.hasMoreElements() ) {
          OAImport.logSystem.log( keyEnum.nextElement() + "\n" ) ;
       }
       OAImport.logSystem.log( "Symptom Table contents End -----------" ) ;
*/

       try {
          in = openResourceFile( source, symptomID );

          /* Get the current symptom from the symptom table */
          /* We will then add the nodes to this symptom     */
          mainSymptom = (Symptom)symptomTable.get ( symptomID+"."+ (String)(indDescript.elementAt(1))) ;

          /* Set the geo and docclass indicators */
          int         geoInd           = OAImport.typeList.indFromString( (String)indDescript.elementAt( 0 ), OAImport.typeList.TYPE_GEOGRAPHY ) ;
          TypeGeoRec  geoRec           = new TypeGeoRec( geoInd, (String)indDescript.elementAt( 0 ) ) ;
          Vector      countryList      = geoRec.getCountryVec() ;
          Vector      allCountriesList = mainSymptom.getCountryList() ;
          Enumeration countryListEnum  = countryList.elements() ;
          
          if ( allCountriesList == null ) {
             allCountriesList = new Vector() ;
          }
            
          while ( countryListEnum.hasMoreElements() ) {
             Object country = countryListEnum.nextElement() ;
             if ( !allCountriesList.contains( country ) ) {
                allCountriesList.addElement( country ) ;
             }
          } // while 

          mainSymptom.setCountryList( allCountriesList ) ;

          /*************************************/
          /* read in the source nodes          */
          /*************************************/
          while ( ( astr = in.readLine() ) != null ) {
             short nodeType ;
             if ( astr.trim().equals( "START" ) ) {
                /* read in the question ID */
                astr = in.readLine().trim().toLowerCase() ;
                String nodeLabel = astr ;
                OAImport.logSystem.log( "ASTR - reading nodes " + astr ) ;
                /* If node is not found this method will create */
                /* a Node and add it to the nodeList            */
                node = ImportUtil.findNode( symptomID, mainSymptom, nodeLabel, nodeList, indDescript, symptomTable,
                                             questionTable, actionTable, nodeTable ) ;
                
                /* if the node has a DB Key and is not in the node list */
                /* add it to the list of nodes                          */
                OAImport.logSystem.log( "Adding node to list" ) ;
                ImportUtil.addNodeToSymptom( mainSymptom, node ) ;

                /* Add node to the node table */
                //nodeTable.put( nodeLabel + "." + (String)(indDescript.elementAt(1)), node ) ;

             }  // if astr == START
          } // while in.readLine

          OAImport.logSystem.log( "NODELIST: " + nodeList.toString() ) ;
          /* Add the nodes to the symptom */
          nodeList = mainSymptom.getNodes();
          mainSymptom.updateRecStatus( PersistentRec.REC_UPDATE ) ;
          //mainSymptom.writeToDatabase() ; 
          boolean gotDbKeys = mainSymptom.assignDatabaseKeys() ;

          in.close() ;

          OAImport.logSystem.log( "\n########## 3rd pass of symptoms #########" + nodeList.toString() ) ;
          /*************************************/
          /* 3rd PASS for processing edges     */
          /*************************************/

          /*******************************************************************/
          /* First set the ROOT NODE for the current symptom and the .r file */
          /* real root node which is the first node in the file              */
          /*******************************************************************/

          /* Check to see if additional nodes have been added */
          /* the root node will always exists as element 0    */
          if ( nodeList.size() > 1) {
             Node rootNode  = mainSymptom.getRootNode() ;
             /* Get firstNode after root node */
             Node firstNode        = (Node)nodeList.elementAt(1) ;
             int qInd              = rootNode.getOAObjectInd() ;
             Vector rootEdgeList   = new Vector() ;
             Question rootQuestion = Question.readFromDatabase( qInd, 1 ) ;

             /* set the geo from the source directory passed in */
             String geo = null ;
             StringTokenizer token = new StringTokenizer( source.substring(0, source.length() -1), "/" ) ;
             while ( token.hasMoreElements() ) {
                geo = (String)token.nextElement() ;
             }
             geo = geo.substring( 0, geo.indexOf(".")) ;
             OAImport.logSystem.log( "Creating Root Node edges" ) ;
             Edge rootNodeEdge = null ;
             if ( geo.equals( "us" ) ){
                /* Create the edges to the root Node with a 'TYPE_OTHER' edge */
                rootNodeEdge = new Edge ( rootQuestion ) ;
             } else if ( geo.equals( "aps" ) ) {
                rootNodeEdge = new Edge ( rootQuestion, rootQuestion.getAnswerByInd( 5 ) ) ;
             } else if ( geo.equals( "au" ) ) {
                rootNodeEdge = new Edge ( rootQuestion, rootQuestion.getAnswerByInd( 5 ) ) ;
             } else if ( geo.equals( "ca" ) ) {
                rootNodeEdge = new Edge ( rootQuestion, rootQuestion.getAnswerByInd( 2 ) ) ;
             } else if ( geo.equals( "emea" ) ) {
                rootNodeEdge = new Edge ( rootQuestion, rootQuestion.getAnswerByInd( 1 ) ) ;
             } else if ( geo.equals( "gcg" ) ) {
                rootNodeEdge = new Edge ( rootQuestion, rootQuestion.getAnswerByInd( 4 ) ) ;
             } else if ( geo.equals( "jp" ) ) {
                rootNodeEdge = new Edge ( rootQuestion, rootQuestion.getAnswerByInd( 4 ) ) ;
             } else if ( geo.equals( "la" ) ) {
                rootNodeEdge = new Edge ( rootQuestion, rootQuestion.getAnswerByInd( 3 ) ) ;
             } else
                OAImport.logSystem.log("GEO " + geo ) ;
             //rootNodeEdge.setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ;
             rootNodeEdge.setDocClass ( (TypeDocClassRec)OAImport.typeList.objectFromString( "Unclassified", OAImport.typeList.TYPE_DOCCLASS ) ) ;
             rootNodeEdge.setCountryList( countryList ) ;
             /* Create an EdgeGroup for this node */
             /* connect root node in this symptom to the first node in the list */
             EdgeGroup rootEdgeGroup = new EdgeGroup( rootNode, firstNode ) ;
             rootEdgeList.addElement( rootNodeEdge ) ;
             rootEdgeGroup.setEdges( rootEdgeList ) ;
             ImportUtil.addEdgeGroupToSymptom ( mainSymptom, rootEdgeGroup ) ;
          }

          /*******************************************************************/
          /* connect the root symptom to the root node and symptom           */
          /*                                                                 */
          /*******************************************************************/
          try {
             if ( symptomID.equalsIgnoreCase("s1")) {
                OAImport.logSystem.log("Connecting to Root Symptom" ) ;
                EdgeGroup  edgeGroup            = null ;
                Edge       rootEdge             = null ;
                Vector     rootEdgeVector       = new Vector() ;
                Vector     rootEdgeGroupVector  = null ;
                /* Process only the root symptom of each geo */
                if ( rootSymptom == null ) {
                   /* Read the root symptom */
                   rootSymptom         = Symptom.readFromDatabase( 1, 1 ) ;
                   rootSymptomQuestion = Question.readFromDatabase( 2, 1 ) ;
                }
                rootEdgeGroupVector = rootSymptom.getEdgeGroups() ;
                if ( rootEdgeGroupVector == null ) {
                   rootEdgeGroupVector = new Vector() ;
                }
                /* Answer indicators for brands */
                /* 10 - 'IBM Aptiva'            */
                /* 11 - 'IBM PC'                */
                /* 12 - 'IBM ThinkPad'          */
                /* 13 - 'IBM PC Server'         */
                /* 14 - 'Options By IBM'        */
                /* set the brand from the source directory passed in */
                String brand = null ;
                StringTokenizer brandToken = new StringTokenizer(    source.substring(0, source.length() -1), "/" ) ;
                while ( brandToken.hasMoreElements() ) {
                   brand = (String)brandToken.nextElement() ;
                }
                int brandBegin = brand.indexOf(".") ;
                int brandEnd   = brand.indexOf(".", brandBegin + 1) ;
                brand = brand.substring( brandBegin + 1, brandEnd) ;
                Vector rootNodeList = rootSymptom.getNodes() ;
                /* Is this node already connected */
                if ( !ImportUtil.nodeExists( rootNodeList, mainSymptom.getInd())) {
                   OAImport.logSystem.log("Connecting brand: "+ brand ) ;
                   /* Create a symptom node for the rootNode */
                   Node symptomNode = new Node( mainSymptom ) ;
                   /* Add the new symptom node */
                   rootNodeList.addElement( symptomNode ) ;

                   edgeGroup = new EdgeGroup( rootSymptom.getRootNode(), symptomNode ) ;
                   if ( brand.equals("cdt")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 11 ) ) ;
                   } else if ( brand.equals("consumer")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 10 ) ) ;
                   } else if ( brand.equals("contact")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 14 ) ) ;
                   } else if ( brand.equals("mobiles")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 12 ) ) ;
                   } else if ( brand.equals("nstation")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 11 ) ) ;
                   } else if ( brand.equals("obi")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 14 ) ) ;
                   } else if ( brand.equals("pws")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 11 ) ) ;
                   } else if ( brand.equals("assist")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 11 ) ) ;
                   } else if ( brand.equals("server")) {
                      rootEdge  = new Edge ( rootSymptomQuestion, rootSymptomQuestion.getAnswerByInd( 13 ) ) ;
                   }
                   if ( rootEdge != null ) {
                      rootEdge.setCountryList( allCountriesList ) ;
                      rootEdge.setDocClass ( (TypeDocClassRec)OAImport.typeList.objectFromString( "Unclassified", OAImport.typeList.TYPE_DOCCLASS ) ) ;
                      rootEdgeVector.addElement( rootEdge ) ;
                      edgeGroup.setEdges( rootEdgeVector ) ;
                      rootEdgeGroupVector.addElement( edgeGroup ) ;
                      rootSymptom.writeToDatabase() ;
                      OAImport.logSystem.log("Connection to root symptom complete") ;
                   }
                }
             }
          } catch ( Exception e ) {
             OAImport.logError( e ) ;
             e.printStackTrace() ;
          }

          /***************************************************/
          /* Read In the Nodes and set their Edges           */
          /***************************************************/
          Node sourceNode ;

          in = openResourceFile( source, symptomID );

          while ( ( astr = in.readLine() ) != null ) {
             /* read in nodes */
             if ( astr.trim().equals( "START" ) ) {
                /* read in the Node ID */
                astr = in.readLine().trim().toLowerCase() ;
                String nodeLabel = astr ;
                /* Check for cloned question */
                if ( regex.match("/-/", astr ) ) {
                   astr = regex.preMatch() ;
                   OAImport.logSystem.log( "CLONED QUESTION 3rd pass" ) ;
                }
                OAImport.logSystem.log( "NODELABEL " + nodeLabel ) ;
                /* Find this question object */
                Object objQuestion = questionTable.get( astr + "." + (String)(indDescript.elementAt(1))) ;
                if ( objQuestion != null ) {
                   question = (Question) objQuestion ;
                   OAImport.logSystem.log ( "Got question: " + question + "using " + astr + "." + (String)(indDescript.elementAt(1)) ) ;
                }

                /****************************************************/
                /* Find the node object we are currently working on */
                /* This would be the HEAD NODE                      */
                /****************************************************/
                sourceNode = ImportUtil.findNode ( symptomID, mainSymptom, nodeLabel.toLowerCase(), nodeList, indDescript, symptomTable, questionTable, actionTable , nodeTable ) ;
                
                /* Check whether the source node already exists in the node table */
                /* if it exists then skip to the next node                        */
                if ( ImportUtil.nodeExists( mainSymptom, sourceNode ) ) 
                   continue ;

                /* Add each EDGE to the Head Node object */
                while (((astr = in.readLine()) != null) && (!astr.trim().equals("END"))) {
                   // Separate the edge label and conditions from the target and conclusions
                   // Element 0 will have the label and conditions,
                   // element 1 will have the target and conclusions
                   // (0) = id+cond, (1) = target+concl
                   tempVector = regex.split("/,/",astr); 
                   // Separate the label from the conditions
                   // label(0) = edgeID, label(1) = conditions
                   label = regex.split("/&/",(String)tempVector.elementAt(0),2);
                   // Separate the target from the conclusions (preanswers)
                   // conclusions(0) = target, conclusions(1) = conclusions
                   conclusions = regex.split( "/&/",(String)tempVector.elementAt(1),2);
                   
                   String edgeLabel = ((String)label.elementAt(0)).trim() ;
                   OAImport.logSystem.log("LABEL: " + (String)label.elementAt(0) ) ;
                   
                   /* checking for proper label format */
                   if ( regex.match("/[:]/", edgeLabel) ) {
                      OAImport.logSystem.log( "Error: bad label format. Throwing Exception " + edgeLabel ) ;
                      OAImport.logSystem.log( "BAD DATA in resource file : " + symptomID ) ;
                   }
                   /* Get the answer number for this edge */
                   int answerID = -1 ;
                   if ( !edgeLabel.equals( "#OTHER" ) ) {
                       answerID = Integer.parseInt( edgeLabel ) ;
                       OAImport.logSystem.log( "answerID: " + answerID ) ;
                   } 

                   /* Get the answer list from the question */
                   Vector answerList = question.getAnswers() ;

                   /* Fix for unmatched number of answers between question and symptom .r files */
                   /* Check that the number of answers for the question node                    */
                   /* matches the number of answers of the question itself                      */
                   if ( edgeLabel.equals( "#OTHER" ) || answerID <= answerList.size() ) {
                      /**************************************/
                      /* Get the target value for this edge */
                      /* This would be the TAIL NODE        */
                      /**************************************/
                      String target = (String) conclusions.elementAt(0) ;

                      /* Check to see if this target is a return type */
                      String returnTarget = null ;
                      if ( target.indexOf('!') > 0 ) {
                         Vector vTarget = regex.split("/!/", target ,2);
                         target = (String) vTarget.elementAt(0) ;
                         returnTarget = (String) vTarget.elementAt(1) ;
                         OAImport.logSystem.log("Parsed Node Return: " +target+" "+returnTarget ) ;
                      }

                      Node targetNode ;
                      short nodeType = -1 ;

                      /* find out what type of node we are going to create  */
                      /* for the Tail Node                                */
                      nodeType = ImportUtil.getNodeType( target.trim() ) ;
                      OAImport.logSystem.log( "NODETYPE: " + nodeType ) ;

                      if ( nodeType == Node.NODE_TYPE_RESOLVED ||
                        nodeType == Node.NODE_TYPE_UNRESOLVED  ||
                        nodeType == Node.NODE_TYPE_ALIST   ||
                        nodeType == Node.NODE_TYPE_RETURN ) {
                        OAImport.logSystem.log( "Creating SPECIAL node " ) ;
                        targetNode = new Node( nodeType ) ;
                        /* Add node to the list of nodes */
                        ImportUtil.addNodeToSymptom( mainSymptom, targetNode ) ;
                        
                        /* We must write this to the database so an indicator will be set on the node */
                        //mainSymptom.writeToDatabase() ;
                        boolean assignedKey = mainSymptom.assignDatabaseKeys() ;

                        if ( targetNode == null ) {
                           OAImport.logSystem.log( "Special node is null. NodeType: " + nodeType ) ;
                        }
                      } else {
                        target = target.toLowerCase().trim() ;
                        targetNode = ImportUtil.findNode ( symptomID, mainSymptom, target, nodeList, indDescript, symptomTable, questionTable, actionTable, nodeTable ) ;
                        if ( targetNode == null) {
                           OAImport.logSystem.log("Unexpected: ImportUtil.findNode failed " + target ) ;
                           /* unable to process this node */
                           continue ;
                        }  
                        /* Add node to the list of nodes */
                        ImportUtil.addNodeToSymptom( mainSymptom, targetNode ) ;
                        
                      }

                      /************************************************************/
                      /* Setting the EDGEGROUPS, composed of a Head and Tail Node */
                      /************************************************************/

                      EdgeGroup edgeGroup = new EdgeGroup( sourceNode, targetNode ) ;
                      ImportUtil.addEdgeGroupToSymptom ( mainSymptom, edgeGroup ) ;

                      /******************************/
                      /* Set the EDGES              */
                      /* Get the label for the edge */
                      /******************************/
                      //String edgeLabel = ((String)label.elementAt(0)).trim() ;
                      //OAImport.logSystem.log("LABEL: " + (String)label.elementAt(0) ) ;

                      /*************************************/
                      /* Special case for TYPE_OTHER edges */
                      /*************************************/
                      if ( edgeLabel.equals( "#OTHER" ) ) {       // check the # of answers left in the question
                         OAImport.logSystem.log( "Creating edge type_other" ) ;   
                         edge = new Edge( question ) ;
                      }
                      else {
                         /**************************************************/
                         /* Process normal answers and questions for edges */
                         /**************************************************/

                         /* Get the answer number for this edge */
                         //int answerID = Integer.parseInt( edgeLabel ) ;

                         /* Get the answer list from the question */
                         //Vector answerList = question.getAnswers() ;

                         /* Create our edge */
                         edge = new Edge( question, (Answer)answerList.elementAt( answerID - 1 ) ) ;
                         OAImport.logSystem.log( "Created a normal Edge" ) ;
                      }

                      /*************************************************/
                      /* Check for PREANSWER, NOANSWER and ALIST edges */
                      /*************************************************/
                      if ( conclusions.size() > 1 && conclusions.elementAt(1) != null ) {
                         OAImport.logSystem.log( "Setting Preanswers: " + conclusions.elementAt(1) ) ;
                         /* There could be multiple preanswers to this edge */
                         Vector preansVec       = regex.split ( "/&/", (String)conclusions.elementAt(1) ) ;
                         Enumeration preansEnum = preansVec.elements() ;
                         Vector preansTerms     = new Vector() ;   // list of Q&A pairs
                         Vector preansObjs      = new Vector() ;   // list of Q&A object pairs
                         Vector preanswersList  = new Vector() ;   // list of preanswers edges
                         Vector unanswerList    = new Vector() ;   // list of no answer edges
                         Vector actionList      = new Vector() ;   // list of action list edges
                         String nextElement     = null ;
                         while ( preansEnum.hasMoreElements() ) {
                            nextElement = (String)preansEnum.nextElement() ;
                            OAImport.logSystem.log( "preanswer label: " + (String)nextElement ) ;
                            /* This is a Preanswer/Noanswer if element contains a question */
                            if ( nextElement.startsWith( "Q" ) ) {
                               preansTerms = ImportUtil.getTerm( nextElement ) ;
                              if ( (preansTerms.size() > 1) ) {
                                 if ( ((String)preansTerms.elementAt(1)).equals( "NOANSWER" ) ) {
                                    String questionStr = (((String)preansTerms.elementAt(0)).toLowerCase()).trim() ;
                                    OAImport.logSystem.log( "NOANSWER for question: " + (String)preansTerms.elementAt(0) ) ;
                                    OAImport.logSystem.log( "questionStr: " + questionStr + "." + (String)indDescript.elementAt(1) ) ;
                                    Question noAnsQuestion = (Question)questionTable.get( questionStr + "." + (String)indDescript.elementAt(1) ) ;
                                    if ( noAnsQuestion == null ) {
                                       ImportUtil.listTable( questionTable ) ;
                                    }
                                    int ind = noAnsQuestion.getInd() ;
                                    OAImport.logSystem.log( "noansquestion ind: " + ind ) ;
                                    EdgeUnanswer edgeUnanswer = new EdgeUnanswer( noAnsQuestion ) ;
                                    //unanswerList.addElement( edgeUnanswer ) ;
                                    edge.addUnanswer( edgeUnanswer ) ;
                                 } else {
                                    OAImport.logSystem.log( "Preanswer : " + (String)preansTerms.elementAt(1) ) ;
                                    preansObjs  = ImportUtil.getConditionTerms( preansTerms, questionTable, (String)indDescript.elementAt(1) ) ;
                                    EdgePreanswer edgePreanswer = new EdgePreanswer( (Question)preansObjs.elementAt(0), (Answer)preansObjs.elementAt(1) ) ;
                                    //preanswersList.addElement( edgePreanswer ) ;
                                    edge.addPreanswer( edgePreanswer ) ;
                                 }
                              }
                            } 
                            /* Else this is an Action List */
                            else if ( nextElement.startsWith( "A" ) ) {
                               OAImport.logSystem.log( "preanswer is ALIST: " ) ;
                               Action actionEdge = null ;
                               actionEdge = (Action)actionTable.get( (String)nextElement.toLowerCase() + "." + (String)indDescript.elementAt(1) ) ; 
                               EdgeAlist edgeAlist = new EdgeAlist( actionEdge ) ;
                               //actionList.addElement( edgeAlist ) ;
                               edge.addActionList( edgeAlist ) ;
                            }
                         }  // while
                         //edge.setUnanswers( unanswerList ) ;
                         edge.setUnanswers( edge.getUnanswers() ) ;
                         //edge.setPreanswers( preanswersList ) ;
                         edge.setPreanswers( edge.getPreanswers() ) ;
                         //edge.setActionList( actionList ) ;
                         edge.setActionList( edge.getActionList() ) ;
                      } // end Preanswer NoAnswer Alist edges

                      /***********************************/
                      /* Check for a RETURN type of edge */
                      /***********************************/
                      if ( returnTarget != null ) {
                         /* Get the node to return to */
                         Node returnNode = ImportUtil.findNode ( symptomID, mainSymptom, returnTarget.toLowerCase(), nodeList, indDescript, symptomTable, questionTable, actionTable , nodeTable ) ;
                         /* Add node to the list of nodes */
                         ImportUtil.addNodeToSymptom( mainSymptom, returnNode ) ;
                         
                         EdgeReturn edgeReturn = new EdgeReturn( returnNode ) ;
                         /* set the value in the edge */
                         edge.setReturnNode( edgeReturn ) ;
                         OAImport.logSystem.log( "Set Return Edge for " + returnTarget ) ;   
                      }  // end Return edge

                      /* Set docClass and country indicators */
                      edge.setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ;
                      /* Set the edges country list to the current list for the symptom being read */
                      edge.setCountryList( countryList ) ;

          /*            tempEdge = new Edge(((String)label.elementAt(0)).trim(),((String)conclusions.elementAt(0)).trim());
                      if (label.size() > 1) {
                         tempEdge.setConditions(((String)label.elementAt(1)).trim());
                      }
                      if (conclusions.size() > 1) {
                         tempC = new Conclusion("");
                         tempC.setConclusions(((String)conclusions.elementAt(1)).trim());
                         tempEdge.setConclusions(tempC);
                      }
          */          

                      edgeList = edgeGroup.getEdges() ;
                      if ( edgeList == null ) {
                         edgeList = new Vector() ;
                      }
                      edgeList.addElement( edge ) ;
                      edgeGroup.setEdges( edgeList ) ;
                   }  // Fix for unmatched number of question answers in question and symptom .r files

                } /* read each edge  - while */
                
             }  /* Read each node - if */

             /* Add the edge groups to the symptom */
             //mainSymptom.setEdgeGroups( edgeGroupList ) ;
          
          } // while
          /* Set the correct docClass for edges */
          /* This is done here because edges are*/
          /* not being merged.  They will all be*/
          /* classified unless this is done     */
          Enumeration edgeListEnum = edgeList.elements() ;
          while ( edgeListEnum.hasMoreElements() ) {
             Edge edgeItem = ( Edge ) edgeListEnum.nextElement() ;
             //edgeItem.setDocClass( (TypeDocClassRec)OAImport.typeList.objectFromString( (String)indDescript.elementAt(2), OAImport.typeList.TYPE_DOCCLASS ) ) ;
             edgeItem.setDocClass( mainSymptom.getDocClass() ) ;

          }
       }
       catch ( Exception e ) {
          OAImport.logSystem.log( "Exception: " + e ) ;
          System.out.println( "2nd pass failed:  symptom = " + symptomID ) ;
          OAImport.logError( e ) ;
          e.printStackTrace() ;
       }

       /* List nodes in this symptom */
       Enumeration nodeEnum = edgeGroupList.elements() ;
       OAImport.logSystem.log( "Listing EdgeGroups: " ) ;
       while ( nodeEnum.hasMoreElements() ) {
          EdgeGroup edgeG = (EdgeGroup) nodeEnum.nextElement() ;
          OAImport.logSystem.log( "EdgeGroup: " + edgeG ) ;
       }

       return symptomTable ;

    } // readSymptom - 2nd/3rd pass
     

    /**
     * Given an id for a knowledge base item, returns
     * the root or base id.
     *
     * @param id Id of the item to find the base for.
     *
     * @return String - the base or root id
     */
    public static String getBaseID(String id) {

        Vector v = regex.split("/-/",id);
        String i = (String)v.elementAt(0);

        return i;
    } // getBaseID();

    
}   // class KBIO
