package com.ibm.nzna.projects.oa.imports ;

/*-----------------------------------------------------------*/
/*   OAImport                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OAImport                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.projects.common.quest.type.* ;

/**
 * This is the imports batch. It will create the Log file and
 * an instance of the Batch class that will start processing
 * the imports to the database.
 *
 * Valid Arguments:
 *
 * /DATABASE:       The database name to connect to. Defaults to 'QUEST'
 * /SERVER:         The server to contact. Defaults to 'CHAOS'
 *
 *
 * @version 1.0, 3/27/2000 
 * @author  Maria Paredes
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OAImport                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class OAImport extends Batch {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String                databaseName = "QUEST" ;
   private String                serverName   = "CHAOS" ;
   private String                port         = "5703" ;
   private String                userID       = "quest" ;
   private String                password     = "hgra2l" ;
   public static OAImport        batchLog     = null ;
   public static BatchLogSystem  logSystem    = null ;
   public static TypeList        typeList     = new TypeList() ;
   public static Properties      properties   = new Properties() ;


   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This method is invoked by the JVM when
    *  the os calls for us to execute. We will kick off
    *  our batch now
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String[] args ) {
      new OAImport ( args ) ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create, initialize and start
    *  our batch process.
    *  
    *
    *  @param args[] The arguments for the batch
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OAImport ( String[] args ) {                      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First create our BatchLogSystem                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      batchLog  = this ;
      logSystem = new BatchLogSystem ( "OAImport", this ) ;
      logSystem.setProvideGUI( false ) ;
      
      // Activate the capture of messages from standard err
      // to the log file
      logSystem.setErr() ;

      // Load the properties file 
      StandardPropertyLoader.load( properties, "oaimport" ) ;

      /* Use App Driver */
      boolean useAppDriver = false ;
      String value = properties.getProperty( "db.driver.app" , "true" ) ;
      if ( value.equals("true")) {
         useAppDriver = true ;
      }

      // Create databaseSystem object
      DatabaseSystem dbSystem = new DatabaseSystem( false, logSystem, useAppDriver ) ;      
      dbSystem.setAutoDisconnect( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read our arguments                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      readParams ( args ) ;


      if ( dbSystem.createConnection( dbSystem.QUEST, databaseName, serverName, port, userID, password ) ) {
         System.out.println ("Connected to database: " + databaseName ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   run the batch                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         logSystem.batchStart() ;
         RFilesImporter rFilesImporter = new RFilesImporter() ;
      }
      else { 
         System.out.println( "CANNOT CONNECT" ) ;
      }

      // stop the batch
      logSystem.batchStop() ;
      dbSystem.disconnect( dbSystem.QUEST ) ;
      dbSystem.shutdown() ;
      System.exit(0) ;
   }


   /*------------------------------------------------------------*/
   /* readParams
    */
   /**
    *    This method will traverse through the
    *  array of strings. Each string represents
    *  a parameter
    *
    *  @param args The parameters 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readParams ( String[] args ) {                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have any parameters, read all                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( args != null ) && ( args.length > 0 ) ) {
         int length = args.length ;
         int i      = 0 ;


         logSystem.beginLogSection ( "Reading Command Line Parameters. Number of params passed:"+args.length ) ;
         while ( i < length ) {
            processParam ( args[i].toUpperCase(), args[i] ) ;
            i++ ;
         }
         logSystem.endLogSection () ;
      }

   }



   /*-----------------------------------------------------------*/
   /*   processParam                                            */           
   /*                                                           */
   /*   Description:                                            */
   /*      This method proceses one param                       */                                               
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void processParam ( String param, String mixedCaseParam ) {

      System.out.println ( "Param" ) ;
      logProgress ( "Parameter:"+mixedCaseParam, 1, 1 ) ;

      try {

         if ( param.startsWith ( "/D:" ) )               // D: DATABASE
            databaseName = param.substring ( 3, param.length() ) ;


         else if ( param.startsWith ( "/S:" ) )          // S: SERVER
            serverName = param.substring ( 3, param.length() ) ;
         
         else if ( param.startsWith ( "/P:" ) )          // S: PORT
            port = param.substring ( 3, param.length() ) ;

         else if ( param.startsWith ( "/U:" ) )          // S: USERID
            userID = mixedCaseParam.substring ( 3, param.length() ) ;

         else if ( param.startsWith ( "/PW:" ) )          // S: PASSWORD
            password = mixedCaseParam.substring ( 4, param.length() ) ;

         else if ( param.startsWith ( "/?" ) ) {
            printParams() ;
            System.exit(0) ;
         }

         else if ( param.startsWith ( "?" ) ) {
            printParams() ;
            System.exit(0) ;
         }


         /* If we still don't have the parameter, notify the user */
         else {
            System.out.println ( "Unknown parameter:"+param ) ;
         }
      } catch ( Exception e ) {
         logSevereError ( e ) ;
      }
   }

   /*-----------------------------------------------------------*/
   /* printParams
    */
   /**
    *    This method will print out all of our parameters
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void printParams () {
      System.out.println ( "" ) ;
      System.out.println ( "Usage: java Quest <params>" ) ;
      System.out.println ( "Param       Description                                         Usage" ) ;
      System.out.println ( "----------- -------------------------------------               ----------------" ) ;
      System.out.println ( "/DATABASE   Specify a database name                             /DATABASE:QUEST " ) ;
      System.out.println ( "/SERVER     Specify the database Server                         /SERVER:CHAOS" ) ;
      System.out.println ( "/?          Get this help                                       /? " ) ;

   } 

   /*-----------------------------------------------------------*/
   /* logError
    */
   /**
    *    This method will log all the severe errors into a 
    *  separate file
    *  
    *
    *  @param Exception error 
    *
    *//*---------------------------------------------------------*/
   public static void logError ( Exception e ) {
      batchLog.logSevereError( e ) ;
   }


} /* OAImport */
