package com.ibm.nzna.projects.oa.imports ;


/*-----------------------------------------------------------*/
/*   RFilesImporter                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: RFilesImporter                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.shared.pom.PersistentRec ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.type.TypeDocClassRec;
import com.ibm.nzna.projects.common.quest.type.TypeBrandRec;
import com.ibm.nzna.projects.common.quest.type.TypeGeoRec;
import com.ibm.nzna.projects.common.quest.oa.* ;

/**
 * This is the class to get the resource files to read and 
 * process. 
 *
 * Valid Arguments:
 *
 * @version 1.0
 * @author  
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   RFilesImporter                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class RFilesImporter {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   final String BASEDIR       = OAImport.properties.getProperty( "mainpath" ) ;     //"/home/mparedes/projects/rick/tmp" ;   // parent directory
   final File   EXTERNALDIR   = new File ( BASEDIR + "/production/" ) ;
   final File   INTERNALDIR   = new File ( BASEDIR + "/production.internal/" ) ;
   final File   DEALERDIR     = new File ( BASEDIR + "/production.dealer/" ) ;

   
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private KBIO kbio = new KBIO() ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Members                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   String    geo                 = null ;
   String    brand               = null ;
   String    docClass            = null ;
   String[]  fileList            = null ;                // holds a list of .r file directories names
   File      rFileDir            = null ;                // an .r file directory:  us.mobiles.enduser
   Vector    mainDir             = new Vector( 1 ) ;     // holds docclass directories: production, internal, dealer
   Vector    descriptV           = new Vector( 3 ) ;     // a vector of static indicators descriptions: docclass, brand, geography
   Hashtable actionTable         = new Hashtable() ;
   Hashtable questionTable       = new Hashtable() ;    // to store all question objects
   Hashtable symptomTable        = new Hashtable() ;    // to store all symptom objects
   Hashtable nodeTable           = new Hashtable() ;
   Hashtable transTable          = new Hashtable() ;
   public static Hashtable questDocLinksTable  = new Hashtable() ;
   public static Hashtable externDocLinksTable = new Hashtable() ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the directory pathname 
    *  in order to get to the resource files location.
    *  It will call the method readDir to start the reading 
    *  process.
    *
    *  
    *  @param pathname  the name of the directory to access
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public RFilesImporter ( ) {
      // initialize the vector with the 3 category directories
      mainDir.addElement( INTERNALDIR ) ;                   // reading internal docclass first
      mainDir.addElement( EXTERNALDIR  ) ;                  // ie, /testfs/rick/production/
      mainDir.addElement( DEALERDIR ) ;

      startImports() ;
      OAImport.logSystem.log("IMPORTS DONE") ;
      OAImport.logSystem.log("Writing rFile -> Indicator logs") ;
      logTables() ;

   }  // Constructor

   /*------------------------------------------------------------*/
   /* startImports
    */
   /**
    *    This method will set the directory pathnames needed to 
    *  access the .r files
    *  
    *
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void startImports () {
      String    actionPrefix   = "a" ;
      String    questionPrefix = "q" ;
      String    symptomPrefix  = "s" ;
      String    transPrefix    = "t" ;
      boolean   secondPass     = false ;
      
      /* read in the actions only */
      System.out.println( "\n=== Reading Actions ===" ) ;
      OAImport.logSystem.log( "\n=== Reading Actions ===" ) ;
      actionTable = readFiles( actionPrefix, actionTable, false ) ;
      if ( !actionTable.isEmpty() ) {
         importToDatabase( actionTable, actionPrefix.toUpperCase(), false ) ;
      }
      /* read in the questions only */
      System.out.println( "\n=== Reading Questions ===" ) ;
      OAImport.logSystem.log( "\n=== Reading Questions ===" ) ;
      questionTable = readFiles( questionPrefix, questionTable, false ) ;
      if ( !questionTable.isEmpty() ) {
         importToDatabase( questionTable, questionPrefix.toUpperCase(), false ) ;
      }
      
      /* read in the translate files only */
      System.out.println( "\n=== Reading Translation Files ===" ) ;
      OAImport.logSystem.log( "\n=== Reading Translation Files ===" ) ;
      transTable = readFiles( transPrefix, transTable, false ) ;
      if ( !transTable.isEmpty() ) {
         importToDatabase( transTable, transPrefix.toUpperCase(), false ) ;
      }
      
      /* second pass reading the questions */
      System.out.println( "\n=== Reading Questions - 2nd pass ===" ) ;
      OAImport.logSystem.log( "\n=== Reading Questions - 2nd pass ===" ) ;
      questionTable = readFiles( questionPrefix, questionTable, true ) ;
      if ( !questionTable.isEmpty() ) {
         importToDatabase( questionTable, questionPrefix.toUpperCase(), true ) ;
      }
      /* read in the symptoms only */
      System.out.println( "\n=== Reading Symptoms ===" ) ;
      OAImport.logSystem.log( "\n=== Reading Symptoms ===" ) ;
      symptomTable = readFiles( symptomPrefix, symptomTable, false ) ;
      if ( !symptomTable.isEmpty() ) {
         importToDatabase( symptomTable, symptomPrefix.toUpperCase(), false ) ;
      }
      /* second pass reading the symptoms */
      System.out.println( "\n=== Reading Symptoms - 2nd pass ===" ) ;
      OAImport.logSystem.log( "\n=== Reading Symptoms - 2nd pass ===" ) ;
      symptomTable = readFiles( symptomPrefix, symptomTable, true ) ;
      if ( !symptomTable.isEmpty() ) {
         importToDatabase( symptomTable, symptomPrefix.toUpperCase(), true ) ;
      }
   } // startImports

   /*------------------------------------------------------------*/
   /* readFiles
    */
   /**
    *    This method will set the directory pathnames needed to 
    *  access the .r files
    *  
    *
    *  @param prefix, indicates the type of files to read
    *                 'a' actions,'q' questions,'s' symptoms
    *  @param None
    *
    *  @return hashtable
    *//*---------------------------------------------------------*/
   public Hashtable readFiles ( String prefix, Hashtable filesTable, boolean secondPass ) {
      String  fileList[]  = null ;   // string list of file names in a directory
      String  dirList[]   = null ;   // string list of subdirectories under main directory
      Hashtable hashtable = new Hashtable() ;
      Vector dirVec = new Vector() ;
      String tmpString = null ;
      
      for ( int i = 0 ; i < mainDir.size() ; i++ ) {
         OAImport.logSystem.log( "\n*------------------------------" ) ;
         OAImport.logSystem.log( "*------- Main directory -------\n" + mainDir.elementAt(i) + "\n" ) ;
         /* Show progress */
         System.out.print( "Reading " + mainDir.elementAt(i) + "    \r") ;
         /* .r files directories listed in a string array: aps.cdt.enduser, etc. */
         dirList = ((File)mainDir.elementAt(i)).list() ;
         
         /* rearrange the order of the directories into a vector, */
         /* starting with the US directory                        */
         for ( int k = 0 ; k < dirList.length ; k++ ) {
            if ( dirList[ k ].startsWith( "us" ) ) {
               dirVec.insertElementAt( dirList[k], 0 ) ;
               OAImport.logSystem.log( "Repositioning the US directory: " + dirList[k] ) ;
            } else
               dirVec.addElement( dirList[ k ] ) ;
         }

         if ( dirList.length > 0 ) {
            for ( int j = 0 ; j < dirVec.size() ; j++ ) {
               /* create a file from each directory in the fileList, aps.cdt.enduser is now a File object */
               OAImport.logSystem.log( "This is the current directory: " + (String)dirVec.elementAt(j) ) ;
               rFileDir = new File( ((File)mainDir.elementAt(i)).getPath() + (String)dirVec.elementAt(j) ) ;
                /* get the geography brand and doc class strings */                                                                                                
               setDBDescriptions( rFileDir.getName(), rFileDir.getParent() ) ;
               /* filter the files according to the specified prefix: a, q, or s */
               fileList = rFileDir.list( new RFileFilter( prefix ) ) ;
               /* traverse the vector of .r file directories and process each of them */
               hashtable = processRFiles ( fileList, filesTable, secondPass ) ;

            } // for fileList
         }
         dirVec.removeAllElements() ;
      } // for mainDir

      /* Printing the objects table
      ImportUtil.listTable( hashtable ) ;*/
      
      return hashtable ;
   } // readFiles


   /*------------------------------------------------------------*/
   /* processRFiles
    */
   /**
    *    This method will traverse through the
    *  array of strings. Each string represents
    *  a parameter
    *
    *  @param rfileList, a list of files: actions, questions or symptoms
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Hashtable processRFiles( String[] rfileList, Hashtable filesTable, boolean secondPass ) {

      String[] fileList    = null ;                            // hold list of filenames
      String   rFilename   = null ;                            // a single filename
      String   brandName   = null ;
      String   key         = null ;
      File     rFile       = null ;
      Hashtable hashtable  = filesTable ;
      Action   action      = new Action() ;
      Question question    = new Question() ;
      Symptom  symptom     = new Symptom() ;
      int      count       = 0 ;

      /* traverse the list of files : action, question, or symptom */
      System.out.println( "SubDirectory ==> " + rFileDir.getAbsolutePath() + "\n" ) ;
      
      if ( !secondPass ) {
         /* reading the list of files from a string array */
         /* process each file at a time                   */
         for ( int i = 0 ; i < rfileList.length ; i++ ) {
            try {
               System.out.print("Reading ("+(++count)+") "+rFilename+"   \r") ;
               rFilename = rfileList[i] ;

               if ( rFilename.endsWith ( ".r" ) ) {
                  rFilename = rFilename.substring( 0, rFilename.length() - 2 ) ;

                  if ( rFilename.toLowerCase().startsWith( "a" ) ) {
                     action = kbio.readAction( rFileDir.getPath() + "/", rFilename, getDBDescriptions() ) ;
                     key = rFilename.toLowerCase() + "." + (String)(getDBDescriptions().elementAt(1)) ;

                     /* Filling the table with key: action name.brand and value: action */
                     if ( hashtable.isEmpty() ) {
                        hashtable.put( key, action ) ; 
                     } else {
                        hashtable = compareActions( action, key, hashtable ) ;
                     }
                  } //if "a"  

                  if ( rFilename.toLowerCase().startsWith( "q" ) ) {
                     question = kbio.readQuestion( rFileDir.getPath() + "/", rFilename, getDBDescriptions() ) ;
                     /* First pass - Store questions and answers(no conditions set yet) in a Hashtable */
                     /* key: q#.brand     value: question object                                       */
                     key = rFilename.toLowerCase()+ "." + (String)(getDBDescriptions().elementAt(1)) ; 
                     OAImport.logSystem.log("KEY: " + key ) ;
                     if ( hashtable.isEmpty() ) {
                        hashtable.put( key, question ) ;
                     } else {
                        hashtable = compareQuestions( question, key, hashtable ) ;
                     }
                  }  // if "q"

                  /* Read the translation files */
                  if ( rFilename.toLowerCase().startsWith( "t" ) ) {
                     hashtable = kbio.processTranslationFile( rFileDir.getPath() + "/", rFilename, questionTable, transTable, getDBDescriptions() ) ;
                  } 

                  if ( rFilename.toLowerCase().startsWith( "s" ) ) {
                     Vector titlesV = new Vector();
                     SymptomTitle sympTitle = new SymptomTitle() ;

                     symptom = kbio.readSymptom( rFileDir.getPath() + "/", rFilename, getDBDescriptions() ) ;

                     titlesV = symptom.getSymptomTitles() ;
                     sympTitle = (SymptomTitle)titlesV.elementAt(0) ;

                     key = rFilename.toLowerCase() + "." + (String)(getDBDescriptions().elementAt(1)) ;

                     /* Filling the table with key: action name.brand and value: symptom */
                     if ( hashtable.isEmpty() ) {
                        hashtable.put( key, symptom ) ; 
                     } else{
                        hashtable = compareSymptoms( symptom, key, hashtable ) ;
                     }
                  }  // if "s"
               }   // if .r
            }
            catch ( Exception e ) {
               e.printStackTrace() ;
               OAImport.logSystem.log("ProcessRFiles Exception 1st pass:  " + e) ;
            }
         } // for
      } // if !secondPass 
      else {
      OAImport.logSystem.log("=== SECONDPASS === " ) ;
         /* Depending on the prefix of the files it will work on Symptoms or Questions      */
         /* Second pass through .r files to read Questions Answers and set their conditions */
         for ( int i = 0 ; i < rfileList.length ; i++ ) {
            try {
               rFilename = rfileList[i] ;
               System.out.print("Reading ("+(++count)+") "+rFilename+"   \r") ;
               if ( rFilename.endsWith ( ".r" ) ) {
                  rFilename = rFilename.substring( 0, rFilename.length() - 2 ) ;

                  if ( rFilename.toLowerCase().startsWith( "q" ) ) {
                     key = rFilename.toLowerCase() + "." + (String)(getDBDescriptions().elementAt(1)) ;//+ "." + (String)(getDBDescriptions().elementAt(2)) ;
                     hashtable = kbio.readQuestion( rFileDir.getPath() + "/", rFilename, getDBDescriptions(), filesTable, key ) ;
                  }
                  if ( rFilename.toLowerCase().startsWith( "s" ) ) {
                     key = rFilename.toLowerCase() + "." + (String)(getDBDescriptions().elementAt(1)) ;
                     hashtable = kbio.readSymptom( rFileDir.getPath() + "/", rFilename, getDBDescriptions(), 
                                                   symptomTable, questionTable, actionTable, nodeTable ) ;
                  }
               }
            }
            catch ( Exception e ) {
               e.printStackTrace() ;
               OAImport.logSystem.log("ProcessRFiles Exception 2nd pass:  " + e) ;
            }
         }  // for 
      }
      descriptV.removeAllElements() ;

      return hashtable ;
   } // processRFiles


   /*------------------------------------------------------------*/
   /* setDBDescriptions
    */
   /**
    *    This method will get the geography, brand, and docclass
    *  strings to set the .r files' corresponding static indicators
    *  
    *
    *  @param None
    *  @param None
    *------------------------------------------------------------*/
   public void setDBDescriptions( String dirName, String dirParent ) {

      String tempName = dirName ;
      Vector tempVec = new Vector(3) ;
      IndicatorDescriptionMapping dbMap = new IndicatorDescriptionMapping() ;
      
      geo = tempName.substring( 0, tempName.indexOf( "." ) ) ;
      tempName = tempName.substring( geo.length()+1, tempName.length() ) ;
      
      brand = tempName.substring( 0, tempName.indexOf( "." ) ) ;
      tempName = tempName.substring( brand.length()+1, tempName.length() ) ;
      
      if ( dirParent.endsWith( "production.dealer" ) ) {
         docClass = "dealer" ;
      } else
         docClass = tempName ;
      
      tempVec.addElement ( geo ) ;
      tempVec.addElement ( brand ) ;
      tempVec.addElement ( docClass ) ;

      // do the database mapping add to vector of static indicators

      descriptV.addElement ( dbMap.mapGeoCountry( (String)tempVec.elementAt( 0 ) ) );
      descriptV.addElement ( dbMap.mapBrand( (String)tempVec.elementAt( 1 ) ) ) ;
      descriptV.addElement ( dbMap.mapDocClass( (String)tempVec.elementAt( 2 ) ) ) ;
      
   }

   /*------------------------------------------------------------*/
   /* getDBDescriptions
    */
   /**
    *    This method returns the geography, brand and docclass
    *   descriptions in a Vector
    *  
    *
    *  @param None
    *  @param None
    *------------------------------------------------------------*/
   public Vector getDBDescriptions() {
      return descriptV ;
   }

   /*------------------------------------------------------------*/
   /* compareActions
    */
   /**
    *    This method compare a new action to an action in the
    *   hash table
    *  
    *
    *  @param None
    *  @param None
    *------------------------------------------------------------*/
   public Hashtable compareActions( Action action, String key, Hashtable table ) {

      String    hTitle        = null ;             // a title of an action in the hash table
      Vector    titlesV       = new Vector () ;    // vector of titles for the action to be added
      Vector    hTitlesV      = new Vector () ;    // vector of titles for an action in the hash table
      String    hAddInfo      = null ;             // additonal info text of an add info obj in the hash table
      Vector    addInfoV      = new Vector () ;    // vector of add info for the action to be added
      Vector    hAddInfoV     = new Vector () ;    // vector of add info for an action in the hash table
      Hashtable tmpTable      = table ;
      AddtlInfo hAddInfoObj   = new AddtlInfo() ;
      AddtlInfo addInfoObj    = new AddtlInfo() ;
      Action    hAction       = new Action() ;
      Title     hTitleObj     = new Title() ;

      OAImport.logSystem.log( "Comparing Actions" ) ;

      /* if the action key already exist in the table                      */
      /* get the action value from the table, get the titles of the action */
      /* in the hash table and the current action object                   */
      if ( tmpTable.containsKey( key ) ){
         hAction  = (Action)tmpTable.get( key ) ;
         hTitlesV = hAction.getTitles() ;
         titlesV  = action.getTitles() ;
         boolean found = false ;         
         for ( int i = 0 ; i < hTitlesV.size() ; i++) {
            hTitleObj = (Title)hTitlesV.elementAt( i ) ; 
            hTitle = hTitleObj.getTitle() ;
            /* compare each title in the hash title vector to the current action object    */
            /* if they are the same compare the docclass and set the docclass of the title */
            /* of the hashed action */
            if ( hTitle.equals( titlesV.elementAt( 0 ).toString() ) ) {
               hTitleObj.setDocClass( getDocClassForConditionalObject( (Title)hTitlesV.elementAt( i ), 
                                                        (Title)titlesV.elementAt( 0 ) ) ) ;    // set the prevailing geo and docclass for title
               Title title = (Title)titlesV.elementAt( 0 );
               /* Merge Countries */
               ImportUtil.mergeCountries( title, hTitleObj ) ;

               found = true ;
               break ;
            }
            //OAImport.logSystem.log("ADDEDTITLE: " + (action.getTitles()).elementAt(i) ) ;
         } // for

         if ( !found ) {
            hTitlesV.addElement( (Title)(action.getTitles()).elementAt(0) ) ;
         }
         /* Reorder the titles by geo */
         hTitlesV = ImportUtil.orderByGeo( hTitlesV ) ;
         hAction.setTitles( hTitlesV ) ;

         /* Merge Country List */
         ImportUtil.mergeCountries( action, hAction ) ;
         
         boolean foundAddInfo = false ;

         addInfoV  = action.getAdditionalInfo() ;

         /* Merge the actions additional info */
         if ( action.getAdditionalInfo() != null && !addInfoV.isEmpty() ) {
            hAddInfoV = hAction.getAdditionalInfo() ;
            /* check the additional info objects */
            if ( hAddInfoV == null ) {         // addtln info object might not have add info sometimes
               hAddInfoV = new Vector() ;
            }

            OAImport.logSystem.log( "hAddInfoV.size: " + hAddInfoV.size() ) ;

            if ( hAddInfoV.size() > 0 ) {
               for ( int j = 0 ; j < hAddInfoV.size() ; j++ ) {
                  hAddInfoObj = (AddtlInfo)hAddInfoV.elementAt(j) ;
                  hAddInfo = hAddInfoObj.getAddtlInfo() ;
                  addInfoObj = (AddtlInfo)addInfoV.elementAt( 0 ) ;

                  OAImport.logSystem.log( "Hashed addInfo string: " + hAddInfo ) ;
                  OAImport.logSystem.log( "Incoming addInfo string: " + ((AddtlInfo)addInfoV.elementAt( 0 )).getAddtlInfo() ) ;

                  /* compare each addinfo in the hash addinfo vector to the incoming addInfo object    */
                  /* if they are the same compare the docclass and set the docclass of the addinfo   */
                  /* of the hashed action */
                  if ( hAddInfo.equals( ((AddtlInfo)addInfoV.elementAt( 0 )).getAddtlInfo() ) ) {

                     TypeDocClassRec docClassRec = getDocClassForConditionalObject( hAddInfoObj,
                                                                                    addInfoObj ) ;
                     OAImport.logSystem.log( "AddInfo Question DocClass hash, incoming, result: "+ ((ConditionalObject)hAddInfoObj).getDocClass().getInd()
                                                                                   +","+ ((ConditionalObject)addInfoObj).getDocClass().getInd() 
                                                                                   +","+ docClassRec.getInd()) ;

                     hAddInfoObj.setDocClass( getDocClassForConditionalObject( (AddtlInfo)hAddInfoV.elementAt( j ), 
                                                              (AddtlInfo)addInfoV.elementAt( 0 ) ) ) ;    // set the prevailing geo and docclass for title
                     
                     /* Merge Countries */
                     ImportUtil.mergeCountries( addInfoObj, hAddInfoObj ) ;

                     foundAddInfo = true ;
                     break ;
                  }
               }  // for
            } //if
            
            if ( !foundAddInfo ) {
               hAddInfoV.addElement( (AddtlInfo)(action.getAdditionalInfo()).elementAt(0) ) ;
            }
            /* Reorder the additional info by geo */
            hAddInfoV = ImportUtil.orderByGeo( hAddInfoV ) ;

            hAction.setAdditionalInfo( hAddInfoV ) ;
         }

      } else {
         tmpTable.put ( key,action ) ;
      }

      return tmpTable ;
   } //compareActions

   /*------------------------------------------------------------*/
   /* compareQuestions
    */
   /**
    *    This method compare a new question to an question in the
    *   hash table. Adds Title and additional info if necessary
    *  
    *
    *  @param None
    *  @param None
    *------------------------------------------------------------*/
   public Hashtable compareQuestions( Question question,
                                      String key, Hashtable table ) {

      Vector    titles        = new Vector () ;
      Vector    hTitles       = new Vector () ;
      Vector    addInfoV      = new Vector () ;
      Vector    hAddInfoV     = new Vector () ;
      Vector    answersV      = new Vector () ;
      Vector    hashAnswersV  = new Vector () ;
      Hashtable tmpTable      = table ;
      AddtlInfo hAddInfoObj   = new AddtlInfo() ;
      AddtlInfo addInfoObj    = new AddtlInfo() ;
      Question  hQuestion     = new Question() ;
      Title     hTitleObj     = new Title() ;
      String    hTitle        = null ;
      String    hAddInfo      = null ;
      boolean   found         = false ;

      /* if the question key already exist in the table             */
      /* get the question value from the table and add              */
      /* the titles and additional information of the new question  */
      /* if it does not exist, just insert the new question         */
      OAImport.logSystem.log( "Comparing questions" ) ;
      if ( tmpTable.containsKey( key ) ){
         hQuestion = (Question)tmpTable.get( key ) ;
         hTitles = hQuestion.getTitles() ;
         titles  = question.getTitles() ;
         /* Set title conditions, geo and doclass if necessary and Add titles */
         
         for ( int i = 0 ; i < hTitles.size() ; i++ ) {
            hTitleObj = (Title)hTitles.elementAt( i ) ; 
            hTitle = hTitleObj.getTitle() ;

            if ( hTitle.equals(titles.elementAt( 0 ).toString() ) ) {
               hTitleObj.setDocClass( getDocClassForConditionalObject( (Title)hTitles.elementAt( i ), 
                                                       (Title)titles.elementAt( 0 ) ) ) ;    // set the prevailing geo and docclass for title
               Title title = (Title)titles.elementAt( 0 );
               /* Merge Countries */
               ImportUtil.mergeCountries( title, hTitleObj ) ;

               found = true ;
               break ;
            }
         } // for

         if ( !found ) {
            hTitles.addElement( (Title)(question.getTitles()).elementAt(0) ) ;
         }

         /* We reorder the title by geo here */
         hTitles = ImportUtil.orderByGeo( hTitles ) ;
         hQuestion.setTitles( hTitles ) ;

         /* Merge Country List */
         ImportUtil.mergeCountries( question, hQuestion ) ;

         boolean foundAddInfo = false ;
         
         addInfoV  = question.getAdditionalInfo() ;
         
         /* Merge the questions Additional Information */
         if ( question.getAdditionalInfo() != null && !addInfoV.isEmpty() ) {
            hAddInfoV = hQuestion.getAdditionalInfo() ;
            /* Check the Additional Information Object */
            if ( hAddInfoV == null ) {               // addtln info object might not have add info sometimes
               hAddInfoV = new Vector() ;
            }

            OAImport.logSystem.log( "hAddInfoV.size: " + hAddInfoV.size() ) ;
            
            if ( hAddInfoV.size() > 0 ) {
               OAImport.logSystem.log( "inside if statement, before for loop" ) ;
               for ( int j = 0 ; j < hAddInfoV.size() ; j++ ) {
                  hAddInfoObj = (AddtlInfo)hAddInfoV.elementAt( j ) ;
                  hAddInfo    = hAddInfoObj.getAddtlInfo() ;
                  addInfoObj  = (AddtlInfo)addInfoV.elementAt( 0 ) ;

                  OAImport.logSystem.log( "Hashed addInfo string: " + hAddInfo ) ;
                  OAImport.logSystem.log( "Incoming addInfo string: " + ((AddtlInfo)addInfoV.elementAt( 0 )).getAddtlInfo() ) ;

                  /* compare each addinfo in the hash addinfo vector to the incoming addInfo object    */
                  /* if they are the same compare the docclass and set the docclass of the addinfo */
                  /* of the hashed question */
                  if ( hAddInfo.equals( ((AddtlInfo)addInfoV.elementAt( 0 )).getAddtlInfo() ) ) {

                     TypeDocClassRec docClassRec = getDocClassForConditionalObject( hAddInfoObj,
                                                                                    addInfoObj ) ;    // set the prevailing geo and docclass for title
                     OAImport.logSystem.log( "AddInfo Question DocClass hash, incoming, result: "+ ((ConditionalObject)hAddInfoObj).getDocClass().getInd()
                                                                                   +","+ ((ConditionalObject)addInfoObj).getDocClass().getInd() 
                                                                                   +","+ docClassRec.getInd()) ;

                     hAddInfoObj.setDocClass( getDocClassForConditionalObject( (AddtlInfo)hAddInfoV.elementAt( j ),
                                                              (AddtlInfo)addInfoV.elementAt( 0 ) ) ) ;    // set the prevailing geo and docclass for title
                     
                     /* Merge Countries */
                     ImportUtil.mergeCountries( addInfoObj, hAddInfoObj ) ;

                     foundAddInfo = true ;
                     break ;
                  }
               } // for
            }
            
            if ( !foundAddInfo ) {
               hAddInfoV.addElement( (AddtlInfo)(question.getAdditionalInfo()).elementAt(0) ) ;
            }
            /* Reorder the additional info by geo */
            hAddInfoV = ImportUtil.orderByGeo( hAddInfoV ) ;

            hQuestion.setAdditionalInfo( hAddInfoV ) ;
         }
         /* Compare the Answers from the new Question object and the Hashed Question object */
         answersV = question.getAnswers() ;
         hashAnswersV = hQuestion.getAnswers() ;

         /* Insert in the hashed vector of answers any new answer */
         /* that the incoming answers vector has                  */
         /* So we will parse the hashed answers vector for null   */
         /* elements and compare to the new answers vector to see */
         /* whether the corresponding element contains an object, */
         /* if it contains an answer in that position then we will*/
         /* insert that element into the hashed vector of answers */
         for ( int j = 0 ; (j < hashAnswersV.size()) && (j < answersV.size()) ; j++ ) {
            /* If there is a gap in the answers add the missing element */
            if ( hashAnswersV.elementAt(j) == null && answersV.elementAt(j) != null ) {
               hashAnswersV.removeElementAt(j) ;
               hashAnswersV.insertElementAt( (Answer)answersV.elementAt(j), j ) ;
               OAImport.logSystem.log( "Inserted new answer at position: " + j ) ;
               OAImport.logSystem.log( "Key : " + key ) ;
            }
            /* Compare docClasses of all the answer Titles    */
            /* Set the answer object to the same as the title */
            if ( hashAnswersV.elementAt(j) != null && answersV.elementAt(j) != null ) {
               /* Compare the DocClass of the main Vector to the incoming */
               Answer mergedAnswer     = (Answer) hashAnswersV.elementAt(j) ;
               Answer incomingAnswer   = (Answer) answersV.elementAt(j) ;
               Title  mergedTitle      = (Title) mergedAnswer.getTitles().elementAt(0) ;  //There will only be 1 title.
               Title  incomingTitle    = (Title) incomingAnswer.getTitles().elementAt(0) ;

               TypeDocClassRec docClassRec = getDocClassForConditionalObject( (ConditionalObject) mergedTitle, (ConditionalObject)incomingTitle ) ;
               /* Set the DocClass for the title */
               mergedTitle.setDocClass( docClassRec ) ;
               /* Set the DocClass for the object */
               mergedAnswer.setDocClass( docClassRec ) ;
               /* Debug logging */
               OAImport.logSystem.log( "Answer DocClass hash, incoming, result: "+ ((ConditionalObject)hashAnswersV.elementAt(j)).getDocClass().getInd()
                                                                             +","+ ((ConditionalObject)answersV.elementAt(j)).getDocClass().getInd() 
                                                                             +","+ docClassRec.getInd()) ;
               mergedAnswer.setDocClass( docClassRec ) ;
            }
         } // for

         /* Insert new answers to the hashed vector of answers if */
         /* the incoming vector of answers has more elements      */
         if ( answersV.size() > hashAnswersV.size() ) {
            for ( int i = hashAnswersV.size(); i < answersV.size(); i++ ) {
               hashAnswersV.addElement( answersV.elementAt( i ) ) ;
            }
         }
      } else {
         tmpTable.put ( key,question ) ;
      }

      return tmpTable ;
   } //compareQuestions


   /*------------------------------------------------------------*/
   /* getDocClassForConditionalObject
    */
   /**
    *    This method compares the docclass conditions of a new 
    *   title to a title in the hash table. Sets the  docclass 
    *   to the default docclass to 'external'
    *  
    *
    *  @param None hTitle, the title of object in hash table
    *  @param None title, the title of new object
    *------------------------------------------------------------*/
   public TypeDocClassRec getDocClassForConditionalObject ( ConditionalObject hObject, ConditionalObject incomingObject ) {

      TypeDocClassRec incomingObjectDocClassRec = incomingObject.getDocClass() ;
      TypeDocClassRec hObjectDocClassRec = hObject.getDocClass() ;
      TypeDocClassRec docClassRec = null ;
      
      OAImport.logSystem.log( "Hash object docclass: " + hObjectDocClassRec.getInd() ) ;
      OAImport.logSystem.log( "Incoming object docclass: " + incomingObjectDocClassRec.getInd() ) ;

      switch ( hObjectDocClassRec.getInd() ) {
      case 1 :         // IBM Confidential
         OAImport.logSystem.log( "case 1" ) ;
         if ( incomingObjectDocClassRec.getInd() == 2 || incomingObjectDocClassRec.getInd() == 3 ) {
            docClassRec = incomingObjectDocClassRec ;
         } else 
            docClassRec = hObjectDocClassRec ;
         break ;
      case 2 :        // Dealer Confidential
         OAImport.logSystem.log( "case 2" ) ;
         if ( incomingObjectDocClassRec.getInd() == 3 ) {
            docClassRec = incomingObjectDocClassRec ;
         } else if ( incomingObjectDocClassRec.getInd() == 1 || incomingObjectDocClassRec.getInd() == 2 ) 
            docClassRec = hObjectDocClassRec ;
         break ;
      case 3 :       // Unclassified
         OAImport.logSystem.log( "case 3" ) ;
         docClassRec = hObjectDocClassRec ;
         break ;
      }
      OAImport.logSystem.log( "Docclass set to: " + docClassRec.getInd() ) ;
      return docClassRec ;
   }  // getDocClassForConditionalObject



   /*------------------------------------------------------------*/
   /* compareSymptoms
    */
   /**
    *    This method compare a new symptom to an symptom in the
    *   hash table
    *  
    *
    *  @param None
    *  @param None
    *------------------------------------------------------------*/
   public Hashtable compareSymptoms( Symptom symptom, String key, Hashtable table ) {

      Vector         titles        = new Vector() ;
      Vector         hTitles       = new Vector() ;
      SymptomTitle   hTitle        = null ;
      SymptomTitle   title         = null ;
      Hashtable      tmpTable      = table ;
      Symptom        hSymptom      = new Symptom() ;
      boolean        found         = false ;
      
      OAImport.logSystem.log( "Comparing Symptoms" ) ;
      /* if the symptom key already exist in the table      */
      /* get the symptom value from the table and add       */
      /* the symptom title object of the new symptom        */
      /* if it does not exist, just insert the new symptom  */
      
      if ( tmpTable.containsKey( key ) ) {
         hSymptom = (Symptom)tmpTable.get( key ) ;
         hTitles  = hSymptom.getSymptomTitles() ;
         titles   = symptom.getSymptomTitles() ;
         title    = (SymptomTitle)titles.elementAt(0) ;

         for ( int i = 0 ; i < hTitles.size() ; i++) {
            hTitle = (SymptomTitle)hTitles.elementAt( i ) ;
            String hTitleString = hTitle.getTitle() ;
            String titleString  = title.getTitle() ;
            
            if ( hTitleString.equals( titleString )) {
               /* set the prevailing docclass for the symptom title */
               hTitle.setDocClass(getSymptomTitleDocClass( hTitle, title ) ) ;
               found = true ;
               
               /* Merge Countries */
               ImportUtil.mergeCountries( title, hTitle ) ;

               break ;
            }
         }
         /* If the title does not already exist, add the title */
         if (!found ) {
            hTitles.addElement( title ) ;
            OAImport.logSystem.log( "added symptom title: " + title ) ;
         }
         /* Reorder the titles by geo */
         hTitles = ImportUtil.orderSymptomTitlesByGeo( hTitles ) ;
         hSymptom.setSymptomTitles( hTitles ) ;

      } else {
         tmpTable.put ( key, symptom ) ;
         OAImport.logSystem.log( "symptom is new" ) ;
      }

      return tmpTable ;
   } //compareSymptoms
   

   /*------------------------------------------------------------*/
   /* getSymptomTitleDocClass
    */
   /**
    *    This method compares the docclass conditions of a new 
    *   symptom title to a symptom title in the hash table. 
    *   Sets the  docclass to the default docclass 'external'
    *  
    *
    *  @param None hSymptomTitle, the title of object in hash table
    *  @param None symptomTitle, the title of new object
    *------------------------------------------------------------*/
   public TypeDocClassRec getSymptomTitleDocClass ( SymptomTitle hSymptomTitle, SymptomTitle symptomTitle ) {

      TypeDocClassRec symTitleDocClassRec = symptomTitle.getDocClass() ;
      TypeDocClassRec hSymTitleDocClassRec = hSymptomTitle.getDocClass() ;
      TypeDocClassRec docClassRec = null ;

      switch ( hSymTitleDocClassRec.getInd() ) {
      case 1 :         // IBM Confidential
         if ( symTitleDocClassRec.getInd() == 2 || symTitleDocClassRec.getInd() == 3 ) {
            docClassRec = symTitleDocClassRec ;
         } else 
            docClassRec = hSymTitleDocClassRec ;
         break ;
      case 2 :        // Dealer Confidential
         if ( symTitleDocClassRec.getInd() == 3 ) {
            docClassRec = symTitleDocClassRec ;
         } else if ( symTitleDocClassRec.getInd() == 1 || symTitleDocClassRec.getInd() == 2 ) 
            docClassRec = hSymTitleDocClassRec ;
         break ;
      case 3 :       // Unclassified
            docClassRec = hSymTitleDocClassRec ;
         break ;
      }
      return docClassRec ;
   }  // getSymptomTitleDocClass

   /*------------------------------------------------------------*/
   /* importToDatabase
    */
   /**
    *    This method writes to the database, the actions,
    * questions and symptoms stored in a hashtable.
    *  
    *
    *  @param table, the table containing the OAobjects
    *  @param None
    *------------------------------------------------------------*/
   public void importToDatabase( Hashtable table, String prefix, boolean secondpass  ) {
      Enumeration valEnum = table.elements() ;
      Enumeration keyEnum = table.keys() ;

      System.out.println( "\nImporting objects to database" ) ;
      OAImport.logSystem.log ( "Importing objects to database" ) ;
      int    count = 0 ;
      
      switch ( prefix.charAt( 0 ) ) {
      case 'A':
         Action action  = new Action() ;
         Title titleObj = new Title() ;
         String title = null ;
         System.out.println( "\nTotal Actions: "+ table.size() ) ;
         while ( keyEnum.hasMoreElements() ) {
            try {
               OAImport.logSystem.log ( "\nKEY: " + keyEnum.nextElement() ) ;
               action = (Action)valEnum.nextElement() ;
               Vector titlesV = action.getTitles() ;
               for (int i = 0 ; i < titlesV.size() ; i++) {
                  titleObj = (Title)titlesV.elementAt(i) ;
                  title = (String)titleObj.getTitle() ;
               }
               /* Show Progress */
               System.out.print("Writing Action "+ (++count) +"   \r") ;
               action.writeToDatabase() ;
            }
            catch ( Exception e ) {
               e.printStackTrace() ;
            }
         }
         break ;

      case 'Q':
         String keystr = null ;
         Question question = new Question() ;
         Vector docLinks = new Vector() ;
         System.out.println( "\nTotal Questions: "+ table.size() ) ;
         while ( keyEnum.hasMoreElements() ) {
            keystr = (String)keyEnum.nextElement() ;
            try {
               OAImport.logSystem.log ( "\nKEY: " + keystr ) ;
               question = ( Question ) valEnum.nextElement() ;
               docLinks = question.getDocLinks() ;
               if ( secondpass ) {
                  question.updateRecStatus( PersistentRec.REC_UPDATE ) ;
               }
               /* Fix for missing answers in .r file */
               /* we must copy the vector and remove */
               /* any null items.                    */
               /* We then set the vector back so the */
               /* second pass can match the items    */
               Vector answerListOriginal = question.getAnswers() ;
               Vector answerListCompact  = new Vector() ;
               for ( int i = 0 ; i < answerListOriginal.size() ; i++) {
                  if ( answerListOriginal.elementAt(i) != null) {
                     /* Add the non null answers */
                     answerListCompact.addElement( answerListOriginal.elementAt(i)) ;
                  }
               }

               /* Set the compacted answer list before saving */
               question.setAnswers( answerListCompact ) ;
               /* Write the question to the database */            

               /* Show Progress */
               System.out.print("Writing Question "+ (++count) +"   \r") ;
               question.writeToDatabase() ;
               /* Set the original Vector back so that the     */
               /* Answer numbers will match on the second pass */
               question.setAnswers( answerListOriginal ) ;
            }
            catch ( Exception e ) {
               OAImport.logSystem.log ( "\nKEY: " + keystr + "\n" + e ) ;
               e.printStackTrace() ;
            }
         }
         break ;

      case 'S':
         Symptom symptom = new Symptom() ;
         System.out.println( "\nTotal Symptoms: "+ table.size() ) ;
         while ( keyEnum.hasMoreElements() ) {
            try {
               OAImport.logSystem.log ( "\nKEY: " + keyEnum.nextElement() ) ;
               symptom = ( Symptom ) valEnum.nextElement() ;
               if ( secondpass ) {
                  symptom.updateRecStatus( PersistentRec.REC_UPDATE ) ;
               }
               /* Show Progress */
               System.out.print("Writing Symptom "+ (++count) +"    \r") ;
               symptom.writeToDatabase() ;
            }
            catch ( Exception e ) {
               e.printStackTrace() ;
            }
         }
         break ;

      case 'T':
         /**************************************************/
         /* Translation Files                              */
         /* Write the product, answer and brand indicators */
         /* to the OA.PRODUCTCONCL table                   */
         /**************************************************/
         SQLMethod sqlMethod     = null ;
         Statement statement     = null ;
         Integer   prodInt       = null ;
         Integer   brandInd      = null ;
         Integer   answerInd     = null ;
         Vector    indicatorList = new Vector() ;
         Vector    ansIndList    = new Vector() ;
         int       inserts       = 0 ;
         
         count = 0 ;

         sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "RFilesImporter.importToDatabase()", 5 ) ;
         statement = sqlMethod.createStatement() ;

         OAImport.logSystem.log( "*** UPDATING OA.PRODUCTCONCL Table" ) ;
         if ( !table.isEmpty() ) {
            
            while ( keyEnum.hasMoreElements() ) {
               OAImport.logSystem.log( "KEY: " + keyEnum.nextElement() ) ;
               indicatorList = (Vector)valEnum.nextElement() ;
               OAImport.logSystem.log("indicatorlist.size " + indicatorList.size() ) ;
               prodInt    = (Integer)indicatorList.elementAt(0) ;
               ansIndList = (Vector)indicatorList.elementAt(1) ;
               brandInd   = (Integer)indicatorList.elementAt(2) ;
               
               try {
                  /* for each product indicator in the table, loop  */
                  /* through its corresponding vector of preanswers */
                  for ( int j = 0 ; j < ansIndList.size() ; j++ ) {
                     answerInd = (Integer)ansIndList.elementAt(j) ;
                     OAImport.logSystem.log( "prodint and answerind " + prodInt + " " + answerInd ) ;
                     inserts = statement.executeUpdate( "insert into oa.productconcl( productind, answerind, brandind,dbuser,changedtime) values ("
                                                      + prodInt.intValue() 
                                                      + ", " + answerInd.intValue() 
                                                      + ", " + brandInd.intValue() 
                                                      + ", '', CURRENT TIMESTAMP )" ) ;
                  } //for

                  count++ ;
               } catch ( SQLException sqle ) {
                  OAImport.logSystem.log( "SQLException: " + sqle ) ;
                  sqle.printStackTrace() ;
               } catch ( Exception e ) {
                  OAImport.logSystem.log( "Table size: " + table.size() ) ;
                  e.printStackTrace() ;
               }
            }
            OAImport.logSystem.log( "Number of written product conclusions: " + count ) ;
         } 
         sqlMethod.close() ;
         OAImport.logSystem.log( "Finished processing Translation files\n" ) ;
         break ;

      } // switch

   } // importToDatabase

   public void logTables() {
      String keyName = null ;
      Enumeration keys = actionTable.keys() ;
      while ( keys.hasMoreElements() ) {
         keyName = ( String ) keys.nextElement() ;
         Action action = (Action)actionTable.get( keyName ) ;
         OAImport.logSystem.log( "Action: " + keyName + " --> " + action.getInd() ) ;
      }
      keys = questionTable.keys() ;
      while ( keys.hasMoreElements() ) {
         keyName = ( String ) keys.nextElement() ;
         Question question = (Question)questionTable.get( keyName ) ;
         OAImport.logSystem.log( "Question: " + keyName + " --> " + question.getInd() ) ;
      }
      keys = symptomTable.keys() ;
      while ( keys.hasMoreElements() ) {
         keyName = ( String ) keys.nextElement() ;
         Symptom symptom = (Symptom)symptomTable.get( keyName ) ;
         OAImport.logSystem.log( "Symptom: " + keyName + " --> " + symptom.getInd() ) ;
      }
      keys = nodeTable.keys() ;
      while ( keys.hasMoreElements() ) {
         keyName = ( String ) keys.nextElement() ;
         Node node = (Node)nodeTable.get( keyName ) ;
         OAImport.logSystem.log( "Node: " + keyName + " --> " + node.getInd() ) ;
      }
      keys = transTable.keys() ;
      while ( keys.hasMoreElements() ) {
         keyName = ( String ) keys.nextElement() ;
         Vector vector = (Vector)transTable.get( keyName ) ;
         OAImport.logSystem.log( "Model: " + keyName + " --> " + vector ) ;
      }
   } // logTables

}  // RFilesImporter
               
