package com.ibm.nzna.projects.oa.viewport ;

import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.servlet.template.* ;
import com.ibm.nzna.shared.servlet.ViewPortInput ;
import com.ibm.nzna.shared.mq.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.lang.* ;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;
import com.ibm.nzna.shared.db.*;      // DatabaseSystem
import com.ibm.nzna.projects.common.quest.oa.* ;
import java.sql.*;
import java.util.* ;
import java.io.* ;

   /*********************************************************
   * This class provides a model selection list in html which
   * is created from the OA translation table.
   *
   * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
   * Created: 4/19/100  1:43pm
   * @author 
   * @version
   *********************************************************/
public class OAModelList implements ViewPortInput {
   static final String     fileSeparator     = System.getProperties().getProperty("file.separator") ;
   static final String     modelKey          = "###model_list###" ;
   static final String     brandKey          = "###brand_list###" ;
   static final String     actionKey         = "###actionValue###" ;
   Properties              properties        = new Properties() ;
   Hashtable               modelTable        = new Hashtable() ;
   /* Logging module */
   LogModule               log               = null ;
   /* Template Loader */
   ResourceTemplateLoader  templateLoader    = null ;
   /* values will be loaded from properties file */
   String dbQuestName = null ;
   String dbQuestHost = null ; 
   String dbQuestPort = null ; 
   String dbQuestUid  = null ;
   String dbQuestPwd  = null ;

   /*********************************************************
   * Initialize the input class.  This is provided for classes
   * which may need some prior initialization for efficiency.
   * This would be database connections or other similar tasks.
   * 
   *
   * Author:  
   * Created: 4/19/100  12:38pm
   * @param properties The properties used to initialize this class.
   * @return
   *********************************************************/
   public void init( Properties properties ) {
      /* Get the logging module */
      log = LogManager.getLogModule ( properties ) ;
      
      dbQuestName = properties.getProperty( "db.quest.name" ) ;
      dbQuestHost = properties.getProperty( "db.quest.host" ) ;
      dbQuestPort = properties.getProperty( "db.quest.port" ) ;
      dbQuestUid  = properties.getProperty( "db.quest.uid" ) ; 
      dbQuestPwd  = properties.getProperty( "db.quest.pwd" ) ; 

      DatabaseSystem dbSystem = new DatabaseSystem( false, null, false ) ;
      //dbSystem.createConnection(DatabaseSystem.QUEST,"QUESTOA2","CHAOS","5703","instv5a","instv5a" ) ;
      dbSystem.createConnection( DatabaseSystem.QUEST,dbQuestName,dbQuestHost,dbQuestPort,dbQuestUid,dbQuestPwd ) ;
      /* read in the machines from the oa.productconcl table */
      modelTable = readMachinesFromProductConclTable() ;
      dbSystem.shutdown() ;

      this.properties = properties ;
      /* Create the template loader */
      templateLoader = new ResourceTemplateLoader() ;
      templateLoader.init( properties ) ;
   }

   /*********************************************************
   * Creates a listing of models in html which the user can pick
   * from on the web.  The model list is loaded from the OA Translation
   * table.
   * 
   *
   * Author:  
   * Created: 4/18/100  1:36pm
   * @param sessionState the session state of the web user.
   * @return HTML string to be rendered on the web page.
   *********************************************************/
   public String getHTML ( SessionState sessionState ) throws StandardException {
      
      boolean brandParam = false ;
      String template = null ;
      /* Logging example */
      log.writeLog( "OAModelList:getHTML") ;

      if ( sessionState == null ) {
         log.writeLog("sessionState is null");
      }
      
      /* Get the parameter indicating which template to load   */
      /* from the sessionState                                 */
      brandParam = isBrand( sessionState ) ;

      if ( brandParam ) {
         /* Load the template for the model list */
         template = getTemplate( sessionState ) ;
         template = getModelList( template ) ;
      } else 
         /* Load the template for the brand list */
         template = getBrandList() ;
      
      /* Return Output to Browser */
      return ( template ) ;
   }  // getHTML

   /**********************************************/
   /* Returns true if the parameter is 'oabrand' */
   /*                                            */
   /* @return                                    */
   /**********************************************/
   public boolean isBrand( SessionState sessionState ) {
      boolean oabrand = false ;
      String param = null ;

      /* obtain the request data */
      HttpRequest httpRequest = sessionState.getRequest() ;
      if ( httpRequest == null ) {
         log.writeLog( "isBrand:httprequest is null" ) ;
      }
      int numParam = httpRequest.getNumParameters() ;
      log.writeLog("isBrand:number of parameters: " + numParam ) ;
      
      param = httpRequest.getParameter( "oabrand" ) ;
      if ( param != null ) {
         oabrand = true ;
      }

      return oabrand ;
   }
   /******************************************************/
   /* getModelList()                                     */
   /*                                                    */
   /* This method returns a string of machine models     */
   /* read from the oa.productConcl table                */
   /* via the readMachinesFromProductConclTable method   */
   /*                                                    */
   /* @return a String of machine models with HTML tags  */
   /******************************************************/
   public String getModelList( String template ) {
      
      log.writeLog( "getting the list of models ... " ) ;

      /* Load the template for the model list */
      //String template = templateLoader.loadTemplate( "oa" + fileSeparator + "model_list.html" ) ;
      
      log.writeLog( "modelTable.size: " + modelTable.size() ) ;

      Enumeration productConclEnum  = modelTable.elements() ;
      String modelList              = null ;
      Vector productInfo            = new Vector() ;

      /* Get the model vectors for each brand */
      while ( productConclEnum.hasMoreElements() ) {
         productInfo = (Vector)productConclEnum.nextElement()  ;
         /* Get each model within the vector */
         Enumeration modelEnum = productInfo.elements() ;
         while ( modelEnum.hasMoreElements() ) {
            ProductConclInfo productConclInfo = (ProductConclInfo) modelEnum.nextElement() ;
            if ( modelList == null ) {
               modelList = "<OPTION>" + productConclInfo.descript +"\n" ;
            } else
               modelList = modelList.trim() + "<OPTION>" + productConclInfo.descript +"\n" ;
         }

         /* Put the model list in the template */
         template = RegularExpression.substitute( modelKey , modelList, template ) ;
      }

      /* Return Output to Browser */
      return ( template ) ;
      
   }  // getModelList
      
   /**********************************************************************/
   /* getBrandList                                                       */
   /* This method will return a list of Brands descriptions read in from */
   /* a table of brands obtained from the getBrandsTable method.         */
   /* The string returned includes HTML tags to form a HTML template.    */
   /*                                                                    */
   /* @return a string list of Brand descriptions and HTML tags          */
   /*                                                                    */
   /**********************************************************************/
   public String getBrandList() {
      Hashtable brandsTable = new Hashtable() ;
      /* Load the template for the brand list */
      String template = templateLoader.loadTemplate( "oa" + fileSeparator + "brand_list.html" ) ;

      /* Get the table of brands descriptions */
      brandsTable = getBrandsTable( modelTable ) ;

      Enumeration brandsEnum = brandsTable.elements() ;
      String brandList = null ;

      while ( brandsEnum.hasMoreElements() ) {
         if ( brandList == null ) {
            brandList = "<OPTION>" + (String)brandsEnum.nextElement() ;
         } else
            brandList = brandList.trim() + "<OPTION>" + (String)brandsEnum.nextElement() ;
      }
      
      template = RegularExpression.substitute( brandKey, brandList, template ) ;

      return ( template ) ;

   }  // getBrandList


   /***************************************************************/
   /* getBrandsTable                                              */
   /* This method will return a table of Brands read in from      */
   /* Tigris.typebrand table. The modelTable will provide the     */
   /* brand indicator to perform our query.                       */
   /*                                                             */
   /* @return a table of Brand descriptions and HTML tags         */
   /*                                                             */
   /***************************************************************/
   public Hashtable getBrandsTable( Hashtable modelTable ) {
      
      Hashtable brandsTable   = new Hashtable() ;
      ResultSet rs            = null ;
      Statement statement     = null ;
      Integer   brandKey      = new Integer( 0 ) ;
      String    brandDescript = null ;
      int       brandInd      = 0 ;

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "getBrandsTable()", 2 ) ;
      
      Enumeration brandKeyEnum = modelTable.keys() ;
      
      try {
         while ( brandKeyEnum.hasMoreElements() ) {
            brandKey = (Integer)brandKeyEnum.nextElement() ;
            brandInd = brandKey.intValue() ;

            try {
               statement = sqlm.createStatement() ;
               rs = statement.executeQuery( "SELECT descript from tigris.typebrand where brandind = " + brandInd ) ;
               if ( rs.next() ) {
                  brandDescript = rs.getString( 1 ).trim() ;
                  log.writeLog( "brandDescript: " + brandDescript ) ;
                  brandsTable.put( brandKey, brandDescript ) ;
               } else 
                  log.writeLog( "resultSet returned a null row" ) ;
            } catch ( Exception e ) {
               log.writeLog( e.toString() ) ;
            }
         } // while
         
         rs.close() ;
         sqlm.close() ;
      
      } catch ( SQLException sqle ) {
         log.writeLog( "SQLException:\n" + sqle ) ;
         sqlm.close() ;
      }
      log.writeLog( "brandsTable.size = " + brandsTable.size() ) ;
      return ( brandsTable ) ;
   } // getBrandsTable

   /**
    * Reads the oa.productConcl table and returns a Hashtable whose key
    * is an Integer object containing the brandInd and whose value is
    * a Vector of productConclInfo object.
    * A null value will be returned if a DB2 error is encountered.
    * The ProductConclInfo objects contain the productInd and the String
    * description of the machine.
   **/
   public Hashtable readMachinesFromProductConclTable() {

     log.writeLog("Reading from machine table") ;
     Hashtable machineTable = new Hashtable();
     SQLMethod sqlm = new SQLMethod(DatabaseSystem.QUEST, 
                                    "readMachinesFromProductConclTable()", 2);

     try {
       Statement stmt = sqlm.createStatement();

       // select machineType/model for machines
       // select description/productnum for options
       // rs.getInt(1) == 1 for machines; rs.getInt(1) == 0 for options
       ResultSet rs = stmt.executeQuery(
           "SELECT 1, o.productInd, o.brandInd, " +     // select machines
                  "VARCHAR(p.machine, 20), VARCHAR(p.model, 12) " +
           "FROM oa.productconcl o, tigris.products p " +
           "WHERE o.productind = p.prodint " +
           "AND p.prodint NOT IN (SELECT prodint FROM tigris.prodmachinedesc) " +

           "UNION " +

           "SELECT 0, o.productInd, o.brandind, " +     // select options
                  "VARCHAR(po.descript, 20), VARCHAR(p.prodnum, 12) " +
           "FROM oa.productconcl o, tigris.products p, " +
                "tigris.prodmachinedesc po " +
           "WHERE o.productind = po.prodint " +

           "ORDER BY 3, 4, 5 " +    // order by brandInd, machineType, model
           "FOR READ ONLY"
       );

       int lastBrand = -99;
       Integer brandInteger = null;
       Vector v = null;

       while ( rs.next() ) {
         int isMach      = rs.getInt(1);      // 1 for machines; 0 for options
         int productInd  = rs.getInt(2);
         int brandInd    = rs.getInt(3);
         String type     = rs.getString(4).trim();   // type for machines;
                                                     // descript for options
         String model    = rs.getString(5).trim();   // model for machines;
                                                     // prodnum for options

         // when we encounter a new brand, put the previous brandInteger &
         // Vector of ProductConclInfo objects into the Hastable and then
         // allocate new Integer and Vector objects.
         if (brandInd != lastBrand) {
           if (brandInteger != null) {          // if not the first time through
             machineTable.put(brandInteger, v);
             log.writeLog("Read " + v.size() + 
                              " machines for brandInd " + brandInteger);
           }
           brandInteger = new Integer(brandInd);
           v = new Vector();
           lastBrand = brandInd;
         }

         ProductConclInfo info;
         if ( isMach == 1 ) {
           // for machine, description is "<type>-<model>"
           //info = new ProductConclInfo(productInd, type + "-" + model);
            info = new ProductConclInfo(productInd, type + model);
         } else {
           // for option, description is "<type> <model>"
           info = new ProductConclInfo(productInd, type + model);
         }
         v.addElement(info);
       }

       // put the last brandInteger & Vector of ProductConclInfo objects
       // into the Hashtable -- still check that brandInteger is not null
       // (this could happen if we selected no rows for some reason).
       if (brandInteger != null) {
         machineTable.put(brandInteger, v);
         log.writeLog("Read " + v.size() + 
                          " machines for brandInd " + brandInteger);
       }

       rs.close();
     }
     catch (Exception e) {
       log.writeLog( e.toString());
       e.printStackTrace() ;
       machineTable = null;
     }
     finally {
       sqlm.close();
     }

     return machineTable;
   }  // readMachinesFromProductConclTable

   /*******************************************************/
   /* obtain the list of parameters from the sessionstate */
   /* return these in the form of a query string to be    */
   /* passed to the OA servlet                            */
   /*                                                     */
   /* @return queryStr                                    */
   /*******************************************************/
   public String getQueryString( SessionState sessionState ) {
      String queryStr   = null ;

      /* obtain the request data */
      HttpRequest httpRequest = sessionState.getRequest() ;
      if ( httpRequest == null ) {
         log.writeLog( "httprequest is null" ) ;
      }
      int numParam = httpRequest.getNumParameters() ;
      log.writeLog("number of parameters: " + numParam ) ;

      /* filtering out any mtmdl parameter */
      httpRequest.removeParameter( "mtmdl" ) ;
      queryStr = httpRequest.createQueryString() ;

      log.writeLog( "sending query string: " + queryStr ) ;
      return ( queryStr ) ;
   } //getQueryString

   /****************************************/
   /* Returns the brand or model list html */
   /* template in a string                 */
   /*                                      */
   /* @return the html string              */
   /****************************************/
   public String getTemplate( SessionState sessionState ) {
      
      log.writeLog( "getting the html template ... " ) ;

      /* get the query string */
      String queryString = getQueryString( sessionState ) ;

      /* load the template for the model list */
      String template = templateLoader.loadTemplate( "oa" + fileSeparator + "model_list.html" ) ;
      
      /* get the value of the URL defined in the properties file */
      String oaServletUrl = properties.getProperty( "servlet.oa.url" ) ;

      /* concatenate the query string from the http request */
      /* and the OA servlet URL in the ACTION field of the  */
      /* model list template                                */
      String action = "\"" + oaServletUrl + "?" + queryString + "\"\n" ;
      template = RegularExpression.substitute( actionKey, action, template ) ;

      /* Return Output to Browser */
      return ( template ) ;
      
   }  // getTemplate

} // OAModelList
