package com.ibm.nzna.projects.oa.viewport;
 
import java.io.*;
import java.lang.*;
import java.sql.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.nzna.shared.lang.* ;
import com.ibm.nzna.shared.servlet.* ;
import com.ibm.nzna.shared.servlet.ViewPortInput ;
import com.ibm.nzna.shared.servlet.template.* ;
import com.ibm.nzna.shared.mq.* ;
import com.ibm.nzna.shared.db.*;      // DatabaseSystem
import com.ibm.nzna.shared.util.* ;  // log system, regular expression, etc.
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.*;

/**
 * viewport.StartupOAWeb Servlet
 * @(#)StartupOAWeb.java	1.0 10/03/2000
 * @author Mike Rychener, IBM IGS for PSG Helpcenter
 **/

/* specs 
 *
 * takes in GET/POST parameters mtmdl, (various others) . . .
 *
 * returns an HTML document displaying a form for user to choose country, language,
 *   brand, family, machine type and model.
 *
 */

/* HISTORY
 *  10/03  mdr  created skeleton: database access for lists to populate HTML menus
 *  10/04  mdr  use new templates, fill in more details in fetching data
 *  10/05  mdr  fill in more templates; try to iterate until everything is selected
 *                -- success for testing purposes = SnoopServlet
 *              change this one's properties to have '.' for spaces in brand names
 *              add getLanguagesTable(), getCountriesTable()
 *              add logic for using parameters
 *  10/06  mdr  all get*Table methods: don't add to vector result if empty string
 *              add continueOption string constant
 *              revise logic for using input parameters to pre-select from menu
 *              add skipLists and logic for using it + new getMTModels method
 *  10/07  mdr  convert to be an implementation of ViewPortInput Interface -- cf. OAModelList.java
 *                including main fn is now getHTML
 *  10/08  mdr  new import, shared.lang (for StandardException)
 *              debugging new way; move several list lookups into init() for efficiency
 *  10/09  mdr  replace use of getPathInfo with param docclass, to determine which site we're on
 *              pull in getQueryString from ModelList.java; revise the getting of parameters
 *              revise parameter names, etc. for passing the right thing to WebRenderer
 *                - namely lang, machineType
 *              adjust oaServletUrl value for real use
 *  10/10  mdr  brand comes in with vertical bars, need to weed that out
 *              take out timing messages, already logged elsewhere
 *              also family, et al, come in with vertical bars; type -> machineType
 *              check all input parameters before allowing them to be listed as 'selected'
 *              revise logged messages (brand rather than model, for instance)
 *              first log message includes size of properties
 *              reordered init so brandsList is set up after properties var is set
 *  10/12  mdr  shortcut on en_US and en_* -- pre-set language and country in most common cases
 *                - similarly, French & Canada
 *              remove ut= from queryString
 *              fetch valid models list from productconcl (with MT) and shortcut one iteration
 *                - revised getMTModels to use machine type, return only models
 *  10/13  mdr  remove brand, family, machineType, model and lang from input query string
 *              change DatabaseSystem to use appdriver (true 3rd arg)
 *  10/19  mdr  add more logic in init() to handle possible missing app driver, other defaults
 *              also pick up appdriver switch from a properties file (like WebDaemon does)
 *  10/20  mdr  add checks for null from getParameter on country, et al
 *  10/30  mdr  change default on the appdriver property to be false rather than true
 *  10/31  mdr  add impath variable, images path varies by site; subst. it into header template
 *              use replace() rather than RegularExpression.substitute() in a couple of places
 *              add 2nd arg to getProperty() in getBrandsTable and getHTML, avoid null usages
 *  11/03  mdr  handle 'brand' that comes in from search engine hits -- split into 3 values
 *                (moved parbrand setting earlier to merge with existing site logic)
 *  11/04  mdr  clean up site determination to use PathInfo if present
 *              append docclass to query string explicitly, and remove potentially erroneous one
 *  11/05  mdr  add pathinfo and sessionInd setting to queryString
 *              truncate machineType if it comes in with vertical bar (plain or coded)
 *  -----  TO DO:
 *              save mtmdl lists per brand and mtype (??) (was just brand, but no use for it now)
 *              should mtmdl lists take account of country and language too?
 *              xxx probably need new names for all my parameters, since the eSupport ones differ
 *              xxx  - need to coordinate with Tracey
 *              given a BFTM parameter value, check that it's in the corresponding value list
 *                -- have an example now for brandsList check on parbrand
 *              shrink number of columns if not ready to select type & model
 *              select type-model together? (base on product conclusions)
 *              brand list should be expanded if exist any symptoms -- need loop, make it
 *                more efficient than the old way (loop over all brands, checking each)
 *              what about encoding spaces in resulting query string?? - already done by base
 * END HISTORY
 */


/**********************************************************************
 * MyException is a bare-bones exception handler for minor internal StartupOAWeb errors.
 * @param a message String.
 **********************************************************************/
class MyException extends Exception {
    public MyException(String s) { super(s); }
}


/**********************************************************************
 * The main work of the viewport.StartupOAWeb class is done by the getHTML method.
 * The init method of the ServletBase class will read StartupOAWeb.properties from ClassPath.
 * We just need to tell it here what the name of the properties file is.
 * @author Mike Rychener
 * @version 1.0
 **********************************************************************/
public class StartupOAWeb implements ViewPortInput
{ 
    static final String fileSeparator = System.getProperties().getProperty("file.separator");

    static final String     actionKey         = "###actionValue###" ;
    static final String     imagesKey         = "###imagespath###" ;
    static final String     tableHeadKey      = "###TEMPLATE_HEADER###" ;
    static final String     tableWidthKey1    = "###NUM_OF_COLS_T1###" ;
    static final String     columnHeadKey1    = "###TEMPLATE_TABLE1_HEADER###" ;
    static final String     columnDataKey1    = "###TEMPLATE_TABLE1_DATA###" ;
    static final String     tableWidthKey2    = "###NUM_OF_COLS_T2###" ;
    static final String     columnHeadKey2    = "###TEMPLATE_TABLE2_HEADER###" ;
    static final String     columnDataKey2    = "###TEMPLATE_TABLE2_DATA###" ;
    static final String     componentTypeKey  = "###COMPONENT_TYPE###" ;
    static final String     componentNameKey  = "###COMPONENT_NAME###" ;
    static final String     componentListKey  = "###COMPONENT_OPTIONS###" ;
    static final String     continueOption    = "<OPTION VALUE=\"\">Click on Continue" ;
    
    Properties              properties        = new Properties() ;
    LogModule               log               = null ;  // Logging module
    ResourceTemplateLoader  rtloader          = new ResourceTemplateLoader();
    DatabaseSystem dbSystem                   = null;
    /* values will be loaded from properties file */
    String dbQuestName                        = null ;
    String dbQuestHost                        = null ; 
    String dbQuestPort                        = null ; 
    String dbQuestUid                         = null ;
    String dbQuestPwd                         = null ;
    boolean dbUseAppDrv                       = true ;
    String countryList                        = "";
    String languageList                       = "";
    String brandsList                         = "";

    private final String propertyFilename = "StartupOAWeb" ;
    protected String getPropertyFileName() { return ( propertyFilename ) ; }

    
    /**********************************************************************
     * Initialize the viewport input interface implementation by connecting to
     * database, establishing properties, etc -- Copied and adapted from java
     * class OAModelList.
     * . . .
     * @param properties Properties -- used to provide init details.
     **********************************************************************/   
    public void init( Properties properties ) {
       /* Get the logging module */
       log = LogManager.getLogModule ( properties ) ;

       dbQuestName = properties.getProperty( "db.quest.name", "QUEST" ) ;  // dev: questoa2
       dbQuestHost = properties.getProperty( "db.quest.host", "d04rdb001" ) ;  // dev: Chaos
       dbQuestPort = properties.getProperty( "db.quest.port", "5703" ) ;
       dbQuestUid  = properties.getProperty( "db.quest.uid", "quest" ) ; 
       dbQuestPwd  = properties.getProperty( "db.quest.pwd", "j0hnny" ) ; 
       dbUseAppDrv = properties.getProperty( "db.driver.app", "false" ).equals("true");
       try {
           dbSystem = new DatabaseSystem( false, null, dbUseAppDrv ) ;
           dbSystem.createConnection( DatabaseSystem.QUEST, dbQuestName,
                                      dbQuestHost, dbQuestPort,
                                      dbQuestUid, dbQuestPwd ) ;
       } catch (Exception e) {
           if (dbSystem == null && dbUseAppDrv) {
               dbSystem = new DatabaseSystem( false, null, false ) ;  // try with Net driver
               dbSystem.createConnection( DatabaseSystem.QUEST, dbQuestName,
                                          dbQuestHost, dbQuestPort,
                                          dbQuestUid, dbQuestPwd ) ;
               log.writeLog("StartupOAWeb init -- database connect error (recovered) -- " + e);
           }
       }

       this.properties = properties ;
       if (properties != null)
           log.writeLog("StartupOAWeb init -- " + properties.size() + " properties loaded.");
       else log.writeLog("StartupOAWeb init -- NO properties loaded.");

       /* Create the template loader */
       rtloader = new ResourceTemplateLoader() ;
       rtloader.init( properties ) ;
       
       // pre-calculate several lists from database, since they don't change across calls
       countryList  = optionizeVector(getCountriesTable());
       languageList = optionizeVector(getLanguagesTable());
       brandsList   = optionizeVector(getBrandsTable());

    }

    
    /** This simple convenience method turns 3 int's (year,month,day) into a date string.
     *  Note caller is assumed to have adjusted month value (2nd argument) by 1 to account
     *  for Java calendar's 0-based counting for months.
     **/
    private String formatCalDate(int yr, int mo, int dt) {
        return (
            Integer.toString(yr) + ((mo < 10) ? "-0" : "-") + Integer.toString(mo)
            + ((dt < 10) ? "-0" : "-") + Integer.toString(dt)
        );
    } // end formatCalDate


    /** Take a vector of strings and make them into an <option> list String **/
    private String optionizeVector(Vector itemvec) {
        Enumeration itvenum = itemvec.elements() ;
        if (itemvec.size() == 0 || itvenum == null) return "";
        String itvlist = "<OPTION>" + (String)itvenum.nextElement() ;

        while ( itvenum.hasMoreElements() ) 
           itvlist = itvlist.trim() + "<OPTION>" + (String)itvenum.nextElement() ;
        return itvlist;
    } // end optionizeVector


    /** Make a single substitution of a literal string (string template, key, new substring) **/
    private String subSplice(String str, String sub, String snu) {
        // take a given sequence sub out of a string str and splice the remainders together,
        // inserting a third string snu where sub used to be (only if it was there)
        if (str.indexOf(sub) < 0) return str;
        if (str.indexOf(sub) == 0) return (snu + str.substring(sub.length()));
        return ( str.substring(0, str.indexOf(sub)) + snu
                 + str.substring(str.indexOf(sub) + sub.length()) );
    } // end subSplice


/**********************************************************************
 * Take in CGI parameters, fetch lists of values from database, iterate by
 * calling itself (with a URL action in a <form>) until all parameters are
 * selected by the user, produce a form for user to make selections based on
 * a general set of tabular templates.
 *
 * @param SessionState sessionState -- parameters, etc. from the web.
 **********************************************************************/   
    public String getHTML ( SessionState sessionState ) throws StandardException {
        
        int                     cbytes = 0;
        int                     validSelects = 0;
        String                  parcountry = "";
        String                  parlanguage = "";
        String                  parbrand = "";
        String                  parfamily = "";
        String                  partype = "";
        String                  parmodel = "";

        // loadable resources:
        String                  tempst = "";  // startup main template
        String                  temphe = "";  // header template
        String                  tempch = "";  // component header template
        String                  tempcd = "";  // component data template
        
        HttpRequest request = sessionState.getRequest() ;
        if ( request == null ) {
            log.writeLog( "StartupOAWeb/getHTML: httpRequest is null" ) ;
            return "";
        }
        
        // insite will be "/es" (eSupport site), "/bpggg" (Bus. Partner), or "/it" (Internal)
        //   where ggg is business partner geo abbreviation (la, emea, etc.)
        String insite = request.getPathInfo();
        if (insite == null) insite = "";
        else insite = insite.toLowerCase().substring(0, 3);  // will chop off geo from /bp
        String impath = "/support/images/";   // images path for /es, by default
        String dclass = request.getParameter("docclass");

        /* obtain the request data */
        if (dclass == null || dclass.equals("")) {
            if (insite.equals("/es")) dclass = "3";
            else if (insite.equals("/bp")) dclass = "2";
            else if (insite.equals("/it")) dclass = "1";
            else dclass = "";
        }
        if (dclass.equals("3"))
            impath = properties.getProperty("images.path.es", "/support/images/");
        else if (dclass.equals("2"))
            impath = properties.getProperty("images.path.bp", "/partner/docs/images/");
        else if (dclass.equals("1"))
            impath = properties.getProperty("images.path.it", "/infotips/images/");
        else dclass = "";  // drop erroneous value, allow it to be overridden below

        parbrand = request.getParameter("brand");
        if (parbrand == null) parbrand = "";
        if (parbrand.indexOf(".") > -1) {  // may be a brand from search engine
            String pargeo = parbrand.substring(0, parbrand.indexOf("."));
            // don't know what to do with the goe for now -- later restrict country list??
            parbrand = parbrand.substring(parbrand.indexOf(".") + 1);
            if (parbrand.indexOf(".") > -1) {  // 2nd '.' = more likely from search engine
                String parsite = parbrand.substring(parbrand.indexOf(".") + 1);
                parbrand = parbrand.substring(0, parbrand.indexOf("."));
                if (parbrand.equals("cdt")) parbrand = "IBM PC";
                else if (parbrand.equals("mobiles")) parbrand = "IBM ThinkPad";
                else if (parbrand.equals("server")) parbrand = "IBM PC Server";
                else if (parbrand.equals("consumer")) parbrand = "IBM Aptiva";
                else if (parbrand.equals("obi")) parbrand = "IBM Options";
                else if (parbrand.equals("nstation")) parbrand = "IBM NetVista thin client";
                else parbrand = "";  // not a known brand
                if (dclass.equals("") && parsite.equals("internal")) {
                    insite = "/it";
                    dclass = "1";
                    impath = properties.getProperty("images.path.it", "/infotips/images/");
                }
                else if (dclass.equals("") && parsite.equals("dealer")) {
                    insite = "/bp";
                    dclass = "2";
                    impath = properties.getProperty("images.path.bp", "/partner/docs/images/");
                }
                else if (dclass.equals("") && parsite.equals("enduser")) {
                    insite = "/es";
                    dclass = "3";
                    impath = properties.getProperty("images.path.es", "/support/images/");
                }
            }  // end parbrand with 2nd '.'
        }  // end parbrand with '.'

        log.writeLog("StartupOAWeb/getHTML: starting " + insite + " images: " + impath + ".");
        
        // response.setContentType("text/html");
        // write the data of the response -- PrintWriter wrapped around byte stream buffer
        ByteArrayOutputStream bytes = new ByteArrayOutputStream(1024);
	PrintWriter out = new PrintWriter(bytes, true);  // force flushing

        // long tmstart = System.currentTimeMillis();
        // ---------------------------------- read in templates ------------------------------
        try {  // template reader
            // standard template loader gets from com/ibm/nzna/templates + arg here:
            tempst = rtloader.loadTemplate("oa" + fileSeparator + "template_oastartup.html");
            if (tempst == null)
                throw new MyException("Missing template_oastartup.html file ");
            tempch = rtloader.loadTemplate("oa" + fileSeparator + "template_oaStartupTableHeadCell.html");
            if (tempch == null)
                throw new MyException("Missing template_oaStartupTableHeadCell.html file ");
            tempcd = rtloader.loadTemplate("oa" + fileSeparator + "template_oaStartupTableDataCell.html");
            if (tempcd == null)
                throw new MyException("Missing template_oaStartupTableDataCell.html file ");
            temphe = rtloader.loadTemplate("oa" + fileSeparator + "template_header.html");
            if (temphe == null)
                throw new MyException("Missing template_header.html file ");
            temphe = subSplice(temphe, imagesKey, impath);
        }   // end template reader try
        catch (MyException f1) {  // dummy output if no template
            log.writeLog("StartupOAWeb exception template -- " + f1.getMessage());
            return("<HTML><BODY> viewport/StartupOAWeb -- Installation template error -- "
                        + f1.getMessage() + " </BODY></HTML>");
            // System.exit(1);  -- NOT: kills servletrunner??
        }
        
        try {
            boolean skipLists = false;
            
            if ( request != null ) {
                // get parameters and first check if they're all here
                parcountry = request.getParameter("country");
                if (parcountry == null) parcountry = "";  // avoid null pointer exceptions
                parlanguage = request.getParameter("lang");
                if (parlanguage == null) parlanguage = "";
                // parbrand moved to above
                parfamily = request.getParameter("family");
                if (parfamily == null) parfamily = "";
                partype = request.getParameter("machineType");
                if (partype == null) partype = "";
                parmodel = request.getParameter("model");
                if (parmodel == null) parmodel = "";
            }

            if (partype.indexOf("|") > -1 || partype.indexOf("%7C") > -1) {
                partype = "";  // if vertical bar, can't be one of our selections
            }
            if (parmodel.indexOf("|") > -1 || parmodel.indexOf("%7C") > -1) {
                parmodel = "";  // if vertical bar, can't be one of our selections
            }
            if (parlanguage.equals("en_US")) {
                parlanguage = "English";
                parcountry = "United States";
            }
            else if (parlanguage.indexOf("en_CA") == 0) {
                parcountry = "Canada";
                parlanguage = "English";
            }
            else if (parlanguage.indexOf("en_") == 0) {
                parlanguage = "English";
            }
            else if (parlanguage.indexOf("fr_CA") == 0) {
                parcountry = "Canada";
                parlanguage = "French";
            }
            else if (parlanguage.indexOf("fr_") == 0) {
                parlanguage = "French";
            }
            else if (parlanguage.length() == 5 && parlanguage.indexOf("_") == 2) {
                parlanguage = "";
            }
            if (parcountry != null && parcountry.length() > 0
                    && parlanguage != null && parlanguage.length() > 0
                    && parbrand != null && parbrand.length() > 0
                    && parfamily != null && parfamily.length() > 0
                    && partype != null && partype.length() > 0) {
                tempst = subSplice(tempst,
                                   "</form>",
                                   "The menu of Models has only supported ones.  Select a " +
                                   "model and click on Continue to go ahead.</form>");
                skipLists = true;
            }
            
            // set header of result:
            tempst = subSplice(tempst, tableHeadKey, temphe);

            // extract CGI parameters and check their validity

            // ------------------------------ start first table -----------------------------

            tempst = RegularExpression.substitute(tableWidthKey1, "2", tempst);
            tempst = subSplice(tempst, columnHeadKey1, tempch + tempch);
            tempst = subSplice(tempst, componentTypeKey, "Country");
            tempst = subSplice(tempst, componentTypeKey, "Language");
            // set up for two data columns:
            tempst = subSplice(tempst, columnDataKey1, tempcd + tempcd);

            // ------------------------------ fill in countries ------------------------------
            
            tempst = subSplice(tempst, componentNameKey, "country"); // put name
            tempst = subSplice(tempst, componentNameKey, "Country"); // twice
            // if value comes in as a parameter, make that the selected one
            if (parcountry != null && parcountry.length() > 0) {
                // check country setting validity:
                if (countryList.indexOf(parcountry) > -1)  {
                    validSelects++;
                    tempst = subSplice(tempst,
                                       componentListKey,
                                       "<OPTION VALUE=\"" + parcountry + "\" SELECTED>"
                                       + parcountry + componentListKey);
                }
            }
            tempst = subSplice(tempst, componentListKey, countryList);

            // ------------------------------ fill in languages ------------------------------

            tempst = subSplice(tempst, componentNameKey, "lang"); // put name
            tempst = subSplice(tempst, componentNameKey, "Language"); // twice
            // if value comes in as a parameter, make that the selected one
            if (parlanguage != null && parlanguage.length() > 0) {
                // check language setting validity:
                if (languageList.indexOf(parlanguage) > -1)  {
                    validSelects++;
                    tempst = subSplice(tempst,
                                       componentListKey,
                                       "<OPTION VALUE=\"" + parlanguage + "\" SELECTED>"
                                       + parlanguage + componentListKey);
                }
            }
            tempst = subSplice(tempst, componentListKey, languageList);

            // ------------------------------ start second table -----------------------------

            // really only want 4 columns if both family & brand are known; otherwise, 1 or 2
            tempst = RegularExpression.substitute(tableWidthKey2, "4", tempst);
            tempst = subSplice(tempst, columnHeadKey2, tempch + tempch + tempch + tempch);
            tempst = subSplice(tempst, componentTypeKey, "Brand");
            tempst = subSplice(tempst, componentTypeKey, "Family");
            tempst = subSplice(tempst, componentTypeKey, "Machine-Type");
            tempst = subSplice(tempst, componentTypeKey, "Model");
            // set up for four data columns:
            tempst = subSplice(tempst, columnDataKey2, tempcd + tempcd + tempcd + tempcd);

            // ------------------------------ fill in brands ---------------------------------

            tempst = subSplice(tempst, componentNameKey, "brand"); // put name
            tempst = subSplice(tempst, componentNameKey, "Brand"); // twice
            // if value comes in as a parameter, make that the selected one
            if (parbrand != null && parbrand.length() > 0) {
                if (parbrand.indexOf("|") > -1)
                    parbrand = parbrand.substring(0, parbrand.indexOf("|"));
                if (parbrand.indexOf("%7C") > -1)
                    parbrand = parbrand.substring(0, parbrand.indexOf("%7C"));
                if (brandsList.indexOf(parbrand) < 0)  // validity check for input brand
                    throw new MyException("Brand parameter '" + parbrand
                                          + "' not in supported list of brands. ");
                validSelects++;
                tempst = subSplice(tempst, componentListKey, "<OPTION VALUE=\"" + parbrand
                                   + "\" SELECTED>" + parbrand + componentListKey);
            }
            // normally 'select a brand' is first, already there in template
            tempst = subSplice(tempst, componentListKey, brandsList);  // put list into template

            // ------------------------------ fill in families -------------------------------

            tempst = subSplice(tempst, componentNameKey, "family"); // put name
            tempst = subSplice(tempst, componentNameKey, "Family"); // twice
            // if value comes in as a parameter, make that the selected one
            if (parbrand != null && parbrand.length() > 0) {
                String familiesList = "";
                if (!skipLists) 
                    familiesList = optionizeVector(getFamiliesTable(parbrand));
                if (parfamily != null && parfamily.length() > 0) {
                    if (parfamily.indexOf("|") > -1)
                        parfamily = parfamily.substring(parfamily.indexOf("|") + 1);
                    if (parfamily.indexOf("%7C") > -1)
                        parfamily = parfamily.substring(parfamily.indexOf("%7C") + 1);
                    // check validity of family input:
                    if (skipLists || familiesList.indexOf(parfamily) > -1)  {
                        validSelects++;
                        tempst = subSplice(tempst,
                                           componentListKey,
                                           "<OPTION VALUE=\"" + parfamily + "\" SELECTED>"
                                           + parfamily + componentListKey);
                    }
                }
                tempst = subSplice(tempst, componentListKey, familiesList);  // put list in
            }
            else  // normally 'select a family' is first, already there in template
                tempst = subSplice(tempst, componentListKey, continueOption);

            // ------------------------------ fill in machine-types --------------------------

            tempst = subSplice(tempst, componentNameKey, "machineType"); // put name
            tempst = subSplice(tempst, componentNameKey, "Type"); // twice
            // if value comes in as a parameter, make that the selected one
            if (parbrand != null && parbrand.length() > 0
                    && parfamily != null && parfamily.length() > 0) {
                String typesList = "";
                if (!skipLists) 
                    typesList = optionizeVector(getTypesTable(parbrand, parfamily));
                if (partype != null && partype.length() > 0) {
                    if (partype.indexOf("|") > -1)
                        partype = partype.substring(partype.lastIndexOf("|") + 1);
                    if (partype.indexOf("%7C") > -1)
                        partype = partype.substring(partype.lastIndexOf("%7C") + 1);
                    // check validity of machineType input:
                    if (skipLists || typesList.indexOf(partype) > -1)  {
                        validSelects++;
                        tempst = subSplice(tempst,
                                           componentListKey,
                                           "<OPTION VALUE=\"" + partype + "\" SELECTED>"
                                           + partype + componentListKey);
                    }
                }
                tempst = subSplice(tempst, componentListKey, typesList);  // put list in
            }
            else  // normally 'select a type' is first, already there in template
                tempst = subSplice(tempst, componentListKey, continueOption);

            // ------------------------------ fill in models ---------------------------------

            tempst = subSplice(tempst, componentNameKey, "model"); // put name
            tempst = subSplice(tempst, componentNameKey, "Model"); // twice
            // if value comes in as a parameter, make that the selected one
            if (parbrand != null && parbrand.length() > 0
                    && parfamily != null && parfamily.length() > 0
                    && partype != null && partype.length() > 0) {
                String modelsList = "";  // only supported ones are found, based on prodconcls:
                modelsList = optionizeVector(getMTModels(parbrand, partype));
                if (parmodel != null && parmodel.length() > 0) {
                    // check validity of model input: -- this will probably never happen now
                    //   unless user manually edits the URL
                    if (skipLists || modelsList.indexOf(parmodel) > -1)  {
                        validSelects++;
                        tempst = subSplice(tempst,
                                           componentListKey,
                                           "<OPTION VALUE=\"" + parmodel + "\" SELECTED>"
                                           + parmodel + componentListKey);
                    }
                }
                tempst = subSplice(tempst, componentListKey, modelsList);  // put list in
            }
            else  // normally 'select a model' is first, already there in template
                tempst = subSplice(tempst, componentListKey, continueOption);

        }   // end parameters
        catch (MyException ue) { 
            log.writeLog("StartupOAWeb exception -- parameters -- " + ue.getMessage() + " -- " 
                     + parbrand + ":" + parcountry);
            if (ue.getMessage() == null || ue.getMessage().equals("null")) {
                return("<HTML><BODY>"
                            + "<p><font size=4 face=\"Helvetica, Arial\" color=Red>"
                            + "Unknown Parameter Error.  "
                            + "Please try again.</font></p>"
                            + "<!-- ViewPort.StartupOAWeb exception with null message -->"
                            + "</BODY></HTML>");
            } else {
                return("<HTML><BODY>"
                            + "<p><font size=4 face=\"Helvetica, Arial\" color=Blue>" 
                            + "Parameters are not yet complete.</p>"
                            + "<p>" + ue.getMessage() + "</p>"
                            + "<p>Please try again.</font></p>"
                            + "<!-- ViewPort.StartupOAWeb exception "
                            + "(Parameter Error) -->"
                            + "</BODY></HTML>");
            }
            // System.exit(1);  -- NOT: kills servletrunner
        }  // end catch parameter exceptions
        
        String queryString = getQueryString( request, sessionState ) ;
        if (!dclass.equals("")) queryString = "docclass=" + dclass + "&" + queryString;
        queryString = "sessionInd=0&" + queryString;  // make sure it's a new session
        String oaServletUrl = "";
        if (validSelects >= 5) {
            // now adjust the queryString for the parameters expected by WebRenderer
            // (none for now) could use sessionState.setParameterValue(String param, String val)?
            oaServletUrl = properties.getProperty("servlet.oa.url", "/esupservice/oa"); // success
        }
        else oaServletUrl = properties.getProperty("servlet.startup.url",
                                                   "/esupservice/oaStartupWeb"); // self again
        String action = "\"" + oaServletUrl + insite + "?" + queryString + "\"\n" ;
        tempst = subSplice(tempst, actionKey, action) ;
        
        // long tmend = System.currentTimeMillis();
        // log.writeLog("StartupOAWeb time: " + (tmend - tmstart) + " msec");

        dbSystem.shutdown();
        log.writeLog("StartupOAWeb returning normally -- " + parbrand + ":" + parcountry);
        return(tempst);
    } // end getHTML


    /************************************************************/
    /* Obtain the list of parameters from the sessionstate.     */
    /* Return these in the form of a query string to be         */
    /* passed to the OA servlet.                                */
    /*                                                          */
    /* @author same as ModelList.java (says Mike Rychener, mdr) */
    /* @return String queryStr                                  */
    /************************************************************/
    public String getQueryString( HttpRequest httpRequest, SessionState sessionState ) {
       // mdr added httpRequest argument and removal of ut= from result
       String queryStr   = null ;

       /* obtain the request data */
       if ( httpRequest == null ) {
          log.writeLog( "StartupOAWeb/getQueryString: httpRequest is null." ) ;
          // mdr added tags to all log messages here, identifying source of msg
          return ""; // mdr added error return
       }
       int numParam = httpRequest.getNumParameters() ;
       log.writeLog("StartupOAWeb/getQueryString: number of parameters: " + numParam ) ;

       /* filtering out any mtmdl= and ut= parameter */
       httpRequest.removeParameter( "mtmdl" ) ;
       httpRequest.removeParameter( "ut" ) ;  // ut out because of interfering parameter values
       httpRequest.removeParameter( "brand" ) ;  // these out because our hidden inputs will fill
       httpRequest.removeParameter( "family" ) ;
       httpRequest.removeParameter( "machineType" ) ;
       httpRequest.removeParameter( "model" ) ;
       httpRequest.removeParameter( "lang" ) ;
       httpRequest.removeParameter( "docclass" ) ;
       queryStr = httpRequest.createQueryString() ;

       if (queryStr == null) queryStr = "";  // mdr addition
       log.writeLog( "StartupOAWeb/getQueryString: sending query string: " + queryStr ) ;
       return ( queryStr ) ;
    } //getQueryString

    
    /***************************************************************/
    /* getCountriesTable                                           */
    /* This method will return a table of Countries read in from   */
    /* Tigris.typecountrycode table.                               */
    /*                                                             */
    /* @return a vector of Country descriptions                    */
    /*                                                             */
    /***************************************************************/
    public Vector getCountriesTable() {

       Vector    countryNames    = new Vector(10);
       ResultSet rs              = null ;
       Statement statement       = null ;
       Integer   countryKey      = new Integer( 0 ) ;
       String    countryDescript = null ;
       int       countryInd      = 0 ;

       SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "getCountriesTable()", 2 ) ;

       try {
          statement = sqlm.createStatement() ;
          rs = statement.executeQuery( "SELECT descript from tigris.typecountrycode order by 1" ) ;
          while ( rs.next() ) {
             try {
                countryDescript = rs.getString( 1 ).trim() ;
                // log.writeLog( "countryDescript: " + countryDescript ) ;
                if (countryDescript.length() > 0) countryNames.addElement( countryDescript ) ;
             } catch ( Exception e ) {
                log.writeLog( "StartupOAWeb/getCountriesTable next loop: " + e.toString() ) ;
             }
          } // while

          rs.close() ;
          sqlm.close() ;

       } catch ( SQLException sqle ) {
          log.writeLog( "StartupOAWeb/getCountriesTable SQLException: " + sqle ) ;
          sqlm.close() ;
       }
       if (countryNames.size() < 1)
           log.writeLog( "StartupOAWeb/getCountriesTable: resultSet returned null row" ) ;
       else log.writeLog( "StartupOAWeb/getCountriesTable: # countryNames = "
                          + countryNames.size() ) ;
       return ( countryNames ) ;
    } // getCountriesTable


    /***************************************************************/
    /* getLanguagesTable                                            */
    /* This method will return a table of Languages read in from   */
    /* Tigris.typelanguage table.                                  */
    /*                                                             */
    /* @return a vector of Language descriptions                   */
    /*                                                             */
    /***************************************************************/
    public Vector getLanguagesTable() {

       Vector    languageNames    = new Vector(10);
       ResultSet rs               = null ;
       Statement statement        = null ;
       Integer   languageKey      = new Integer( 0 ) ;
       String    languageDescript = null ;
       int       languageInd      = 0 ;

       SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "getLanguagesTable()", 2 ) ;

       try {
          statement = sqlm.createStatement() ;
          rs = statement.executeQuery( "SELECT descript from tigris.typelanguage order by 1" ) ;
          while ( rs.next() ) {
             try {
                languageDescript = rs.getString( 1 ).trim() ;
                // log.writeLog( "languageDescript: " + languageDescript ) ;
                if (languageDescript.length() > 0) languageNames.addElement( languageDescript ) ;
             } catch ( Exception e ) {
                log.writeLog( "StartupOAWeb/getLanguagesTable next loop: " + e.toString() ) ;
             }
          } // while

          rs.close() ;
          sqlm.close() ;

       } catch ( SQLException sqle ) {
          log.writeLog( "StartupOAWeb/getLanguagesTable SQLException: " + sqle ) ;
          sqlm.close() ;
       }
       if (languageNames.size() < 1)
           log.writeLog( "StartupOAWeb/getLanguagesTable: resultSet returned null row" ) ;
       else log.writeLog( "StartupOAWeb/getLanguagesTable: # languageNames = "
                          + languageNames.size() ) ;
       return ( languageNames ) ;
    } // getLanguagesTable


   /***************************************************************/
   /* getBrandsTable                                              */
   /* This method will return a table of Brands read in from      */
   /* Tigris.typebrand table.  Only brands returned are those     */
   /* with "yes" value in the corresponding property from this    */
   /* servlet's properties file.                                  */
   /*                                                             */
   /* @return a vector of Brand descriptions                      */
   /*                                                             */
   /***************************************************************/
   public Vector getBrandsTable() {
      
      Vector    brandNames    = new Vector(10);
      ResultSet rs            = null ;
      Statement statement     = null ;
      Integer   brandKey      = new Integer( 0 ) ;
      String    brandDescript = null ;
      int       brandInd      = 0 ;

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "getBrandsTable()", 2 ) ;
      
      try {
         statement = sqlm.createStatement() ;
         rs = statement.executeQuery( "SELECT descript from tigris.typebrand order by 1" ) ;
         while ( rs.next() ) {
            try {
               brandDescript = rs.getString( 1 ).trim() ;
               // log.writeLog( "brandDescript: " + brandDescript ) ;
               if (brandDescript.length() > 0 &&
                   properties.getProperty("brand."
                                          + brandDescript.replace(' ', '.'), "no").equals("yes"))
                   brandNames.addElement( brandDescript ) ;
            } catch ( Exception e ) {
               log.writeLog( "StartupOAWeb/getBrandsTable next loop: " + e.toString() ) ;
            }
         } // while
         
         rs.close() ;
         sqlm.close() ;
      
      } catch ( SQLException sqle ) {
         log.writeLog( "StartupOAWeb/getBrandsTable SQLException: " + sqle ) ;
         sqlm.close() ;
      }
      if (brandNames.size() < 1)
          log.writeLog( "StartupOAWeb/getBrandsTable: resultSet returned null row" ) ;
      else log.writeLog( "StartupOAWeb/getBrandsTable: # brandNames = " + brandNames.size() ) ;
      return ( brandNames ) ;
   } // getBrandsTable


   /***************************************************************/
   /* getFamiliesTable                                            */
   /* This method will return a table of Families read in from    */
   /* Tigris.typeGroup table, within a given brand.               */
   /*                                                             */
   /* @return a vector of Family descriptions                     */
   /*                                                             */
   /***************************************************************/
   public Vector getFamiliesTable(String brandString) {

      Vector    familyNames    = new Vector(10);
      ResultSet rs             = null ;
      Statement statement      = null ;
      Integer   familyKey      = new Integer( 0 ) ;
      String    familyDescript = null ;
      int       familyInd      = 0 ;

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "getFamiliesTable()", 2 ) ;

      try {
         statement = sqlm.createStatement() ;
         rs = statement.executeQuery(
             subSplice(
              "SELECT aa.descript " +
              "   from tigris.typegroup aa, " +
              "        tigris.brands bb, " +
              "        tigris.typebrand cc " +
              "   where cc.descript = '##brand##' " +
              "     and cc.brandind = bb.brandind " +
              "     and aa.groupind = bb.groupind " +
              "   order by 1",
              "##brand##",
              brandString)
             );
         while ( rs.next() ) {
            try {
               familyDescript = rs.getString( 1 ).trim() ;
               // log.writeLog( "familyDescript: " + familyDescript ) ;
               if (familyDescript.length() > 0) familyNames.addElement( familyDescript ) ;
            } catch ( Exception e ) {
               log.writeLog( "StartupOAWeb/getFamiliesTable next loop: " + e.toString() ) ;
            }
         } // while

         rs.close() ;
         sqlm.close() ;

      } catch ( SQLException sqle ) {
         log.writeLog( "StartupOAWeb/getFamiliesTable SQLException: " + sqle ) ;
         sqlm.close() ;
      }
      if (familyNames.size() < 1)
          log.writeLog( "StartupOAWeb/getFamiliesTable: resultSet returned null row" ) ;
      else log.writeLog("StartupOAWeb/getFamiliesTable: # familyNames = " + familyNames.size());
      return ( familyNames ) ;
   } // getFamiliesTable


   /***************************************************************/
   /* getTypesTable                                               */
   /* This method will return a table of Types read in from       */
   /* Tigris.typeGroup table, within a given brand & family.      */
   /*                                                             */
   /* @return a vector of Type numbers                            */
   /*                                                             */
   /***************************************************************/
   public Vector getTypesTable(String brandString, String familyString) {

      Vector    typeNames    = new Vector(10);
      ResultSet rs           = null ;
      Statement statement    = null ;
      Integer   typeKey      = new Integer( 0 ) ;
      String    typeDescript = null ;
      int       typeInd      = 0 ;

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "getTypesTable()", 2 ) ;

      try {
         statement = sqlm.createStatement() ;
         rs = statement.executeQuery(
             subSplice(subSplice(
                 "SELECT DISTINCT (A.MACHINE) " +
                  "FROM TIGRIS.PRODUCTS A,TIGRIS.TYPEBRAND B,TIGRIS.TYPEGROUP C,TIGRIS.BRANDS D "+
                  "WHERE B.descript      =  '##brand##' AND " +
                  "      C.descript      =  '##family##' AND " +
                  "      D.BRANDIND      =  B.BRANDIND AND " +
                  "      D.GROUPIND      =  C.GROUPIND AND " +
                  "      A.BRANDGROUPIND =  D.BRANDGROUPIND AND " +
                  "      A.MACHINE       <> 'All' AND " +
                  "      A.MODEL         <> 'All' " +
                  "ORDER BY 1",
                 "##brand##",
                 brandString),
               "##family##",
               familyString)
             );
         while ( rs.next() ) {
            try {
               typeDescript = rs.getString( 1 ).trim() ;
               // log.writeLog( "typeDescript: " + typeDescript ) ;
               if (typeDescript.length() > 0) typeNames.addElement( typeDescript ) ;
            } catch ( Exception e ) {
               log.writeLog( "StartupOAWeb/getTypesTable next loop: " + e.toString() ) ;
            }
         } // while

         rs.close() ;
         sqlm.close() ;

      } catch ( SQLException sqle ) {
         log.writeLog( "StartupOAWeb/getTypesTable SQLException: " + sqle ) ;
         sqlm.close() ;
      }
      if (typeNames.size() < 1)
          log.writeLog( "StartupOAWeb/getTypesTable: resultSet returned null row" ) ;
      else log.writeLog( "StartupOAWeb/getTypesTable: # typeNames = " + typeNames.size() ) ;
      return ( typeNames ) ;
   } // getTypesTable


   /***************************************************************/
   /* getModelsTable                                              */
   /* This method will return a table of Models read in from      */
   /* Tigris.typeGroup table, within a given brand, family        */
   /* & machine type.                                             */
   /*                                                             */
   /* @return a vector of Model numbers                           */
   /*                                                             */
   /***************************************************************/
   public Vector getModelsTable(String brandString, String familyString, String typeString) {

      Vector    modelNames    = new Vector(10);
      ResultSet rs            = null ;
      Statement statement     = null ;
      Integer   modelKey      = new Integer( 0 ) ;
      String    modelDescript = null ;
      int       modelInd      = 0 ;

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "getModelsTable()", 2 ) ;

      try {
         statement = sqlm.createStatement() ;
         rs = statement.executeQuery(
             subSplice(subSplice(subSplice(
                 "SELECT DISTINCT A.MODEL " +
                 " FROM TIGRIS.PRODUCTS A, " +
                 " TIGRIS.TYPEBRAND B, " +
                 " TIGRIS.TYPEGROUP C, " +
                 " TIGRIS.BRANDS D " +
                 " WHERE B.descript      =  '##brand##' AND " +
                 "       C.descript      =  '##family##' AND " +
	         "       A.MACHINE       =  '##type##' AND " +
                 "       D.BRANDIND      =  B.BRANDIND AND " +
                 "       D.GROUPIND      =  C.GROUPIND AND " +
                 "       A.BRANDGROUPIND =  D.BRANDGROUPIND AND " +
                 "       A.MODEL         <> 'All' " +
                 " ORDER BY 1",
                 "##brand##",
                 brandString),
                "##family##",
                familyString),
               "##type##",
               typeString)
             );
         while ( rs.next() ) {
            try {
               modelDescript = rs.getString( 1 ).trim() ;
               // log.writeLog( "modelDescript: " + modelDescript ) ;
               if (modelDescript.length() > 0) modelNames.addElement( modelDescript ) ;
            } catch ( Exception e ) {
               log.writeLog( "StartupOAWeb/getModelsTable next loop: " + e.toString() ) ;
            }
         } // while

         rs.close() ;
         sqlm.close() ;

      } catch ( SQLException sqle ) {
         log.writeLog( "StartupOAWeb/getModelsTable SQLException: " + sqle ) ;
         sqlm.close() ;
      }
      if (modelNames.size() < 1)
          log.writeLog( "StartupOAWeb/getModelsTable: resultSet returned null row" ) ;
      else log.writeLog( "StartupOAWeb/getModelsTable: # modelNames = " + modelNames.size() ) ;
      return ( modelNames ) ;
   } // getModelsTable


   /***************************************************************/
   /* getMTModels                                                 */
   /* This method will return a vector of Machine-Type-Models from*/
   /* product conclusions table, within a given brand             */
   /*                                                             */
   /* @return a vector of 7-character M-T-Models                  */
   /*                                                             */
   /***************************************************************/
   public Vector getMTModels(String brandString, String mtypeString) {

      Vector    modelNames    = new Vector(10);
      ResultSet rs            = null ;
      Statement statement     = null ;
      Integer   modelKey      = new Integer( 0 ) ;
      String    modelDescript = null ;
      int       modelInd      = 0 ;

      SQLMethod sqlm = new SQLMethod( DatabaseSystem.QUEST, "getMTModels()", 2 ) ;

      try {
         statement = sqlm.createStatement() ;
         rs = statement.executeQuery(
             subSplice(subSplice(
                 "SELECT VARCHAR(p.model, 12) " +
                 "FROM oa.productconcl o, tigris.products p, tigris.typebrand b " +
                 "WHERE p.machine = '##mtype##' AND p.prodint = o.productind " +
                 "   AND o.brandind = b.brandind AND b.descript = '##brand##' " +
                 "ORDER BY 1",
                 "##brand##",
                 brandString),
                       "##mtype##",
                       mtypeString)
             );
         while ( rs.next() ) {
            try {
               modelDescript = rs.getString( 1 ).trim() ;
               // log.writeLog( "modelDescript: " + modelDescript ) ;
               if (modelDescript.length() > 0) modelNames.addElement( modelDescript ) ;
            } catch ( Exception e ) {
               log.writeLog( "StartupOAWeb/getMTModelsTable next loop: " + e.toString() ) ;
            }
         } // while

         rs.close() ;
         sqlm.close() ;

      } catch ( SQLException sqle ) {
         log.writeLog( "StartupOAWeb/getMTModelsTable SQLException: " + sqle ) ;
         sqlm.close() ;
      }
      if (modelNames.size() < 1)
          log.writeLog( "StartupOAWeb/getMTModels: resultSet returned null row" ) ;
      else log.writeLog( "StartupOAWeb/getMRModels: # modelNames = " + modelNames.size() ) ;
      return ( modelNames ) ;
   } // getMTModels


} // end StartupOAWeb class

