package com.ibm.nzna.projects.oa.web;

import java.util.*;
//import java.util.zip.*;
import java.io.*;
import java.net.*;
import java.sql.*;
import javax.mail.*;
import javax.mail.internet.*;
import com.ibm.nzna.projects.common.quest.oa.*;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.shared.util.*;    // RegularExpression, LogSystem, Text
import com.ibm.nzna.shared.servlet.template.*;      //ResourceTemplateLoader
import com.ibm.nzna.projects.common.legacy.Pagegen;  //Pagegen needed for encryption
import com.ibm.nzna.properties.*;
import com.ibm.nzna.templates.*;

/* HISTORY
 *
 *  09/14  tlaw   Created based on OAUnresolved.java.
 *
 * END HISTORY
 */

/*********************************************************
 * This Class generates a form that thanks customer for his/her usage,
 * presents customer with a link back to his/her personal support page
 * and present customer with history of current session.
 *
 * Legal: Copyright (c) 2000 IBM. All Rights Reserved.
 * Created: 09/06/2000
 * @author  Tracey Watkins
 * @version 1.0
 *********************************************************/

//public class OAResolved implements Serializable {
public class OAResolved {

    private static int ESUPPORT     = 3;        //Cooresponds to docClassInd 3 (Unclassified)
    private static int BPARTNER     = 2;        //Cooresponds to docClassInd 2 (Dealer Confidential)
    private static int INFOTIPS     = 1;        //Cooresponds to docClassInd 1 (Confidential)
    private static String  TMPSPATH = "com/ibm/nzna/templates/oa/";
    private static String  PROPSPATH= "/com/ibm/nzna/properties/";
    private String  custNum         = "";       //Customer number passed from WebRenderer
    private String  brand           = "";       //Customer's brand passed from WR
    private String  mtmdl           = "";       //Customer's machine type and model from WR
    private String  history         = "";       //OA Session history
    private Vector  histVec         = new Vector(); //OA Session history Vector from WR
    private int     docClassInd     = 3;        //DocClassInd (default to 3--eSupport)
    private String  sympTitle       = "";       //Symptom title passed from WebRenderer
    private int     sympInd         = 0;        //Symptom Ind passed from WebRenderer
    private String  qryStr          = "";       //Query String to customer's personal support page
    private String  imagesPath      = "";       //Path to the images directory for the site
    private String  baseServlet     = "";       //Path and name of the site's home servlet
    private String  oaServlet       = "";       //Path and name of the site's OA serlvet (redirected)
    private String  htmlStr         = "";       //Current HTML in String format

    /* Constructor */
    public OAResolved (String cid, int sympInd, String sympTitle, int docClassInd,
                       String brand, String mtmdl, Vector histVec, String imagesPath,
                       String baseServlet, String oaServlet, String queryStr) {
        try {
            custNum = cid;
            this.brand = brand;
            this.sympInd = sympInd;
            this.sympTitle = sympTitle;
            this.histVec = histVec;
            this.mtmdl = mtmdl;
            this.docClassInd = docClassInd;
            this.baseServlet = baseServlet;
            this.oaServlet = oaServlet;
            this.imagesPath = imagesPath;
            this.qryStr = queryStr;

            boolean loaded;
            String path = StandardPropertyLoader.getPath();
            LogSystem.log(10, "OAResolved:  " +PROPSPATH + "WebDaemon.properties");
            Properties props = StandardPropertyLoader.load(PROPSPATH +"WebDaemon.properties");
            if (props.isEmpty()) {
                loaded = false;
                LogSystem.log(10, "OAResolved:  props file empty.");
                LogSystem.log(10, "OAResolved:  props file=  " + props.toString());
            }
            else {
                loaded = true;
                LogSystem.log(10, "OAResolved:  props file not empty.");
            }
            LogSystem.log(10, "OAResolved:  loaded WebDaemon.properties? " + loaded);

            //Generate the form
            this.generateForm();

        } //try
        catch (Exception e) {
            LogSystem.log(10, "OAResolved:  Exception caught in constructor. ");
            LogSystem.log(1, e);
        } //catch
    } //Constructor

    /**
    * Create the form using the OAResolved template.
    *
    * @param None
    *
    * @return Nothing
    **/
    public void generateForm() {

        String tmpStr = "";
        try {
                ResourceTemplateLoader resTmpLdr = new ResourceTemplateLoader();
                RegularExpression regExp = new RegularExpression();
                htmlStr = resTmpLdr.getTemplate(TMPSPATH + "template_OAResolved.html");
                htmlStr = regExp.substitute("###SYMPTITLE###", sympTitle, htmlStr);
                Integer tmpI = new Integer(sympInd);
                htmlStr = regExp.substitute("###SYMPID###", tmpI.toString(), htmlStr);
                
                tmpStr = resTmpLdr.getTemplate(TMPSPATH + "template_header.html");
                htmlStr = regExp.substitute("###TEMPLATE_HEADER###", tmpStr, htmlStr);

                //Parse History
                history = this.parseHistory(histVec);
                LogSystem.log(10, "OAResolved:  Current history:  " + history);
                htmlStr = regExp.substitute("###HISTORY###", history, htmlStr);
                
                //OA Other conversions--oaservlet, query_string, base servlet and imagespath
                htmlStr = regExp.substitute("###oaservlet###", oaServlet, htmlStr);
                htmlStr = regExp.substitute("###baseservlet###", baseServlet, htmlStr);
                htmlStr = regExp.substitute("###QUERY_STRING###", qryStr, htmlStr);
                htmlStr = regExp.substitute("###actionValue###", oaServlet, htmlStr);
                htmlStr = regExp.substitute("###imagespath###", imagesPath, htmlStr);
        }//try
        catch (Exception e) {
            LogSystem.log(10, "OAResolved:  Exception caught in generateForm().");
            LogSystem.log(1, e);
        }//catch

    }//generateForm()


   /**
   * Create the HTML history from the additional info String passed from WebRenderer.
   *
   * @param Vector histVec
   *
   * @return String histStr
   **/
   public String parseHistory(Vector histVec) {

       LogSystem.log(10, "OAResolved:  parseHistory starting...");
       String histStr = "";
       String tmpStr = "";
       String typeStr = "";
       String histInfoStr="";
       ResourceTemplateLoader resTmpLdr = new ResourceTemplateLoader();
       RegularExpression regExp = new RegularExpression();

       for (int i=0; i<histVec.size(); i++){
          tmpStr = (String)histVec.elementAt(i);
          LogSystem.log(10, "OAresolved:  tmpStr = " + tmpStr);
          typeStr = tmpStr.substring(0,tmpStr.indexOf("="));
          LogSystem.log(10, "OAResolved:  typeStr = " + typeStr);
          histInfoStr = tmpStr.substring(tmpStr.indexOf("=")+1);
          LogSystem.log(10, "OAResolved:  histInfoStr = " + histInfoStr);
          if (typeStr.startsWith("Question") || typeStr.startsWith("Action")) {
              histStr += resTmpLdr.getTemplate(TMPSPATH + "template_boldhistory.html");
              if (typeStr.startsWith("Action")) {
                  histStr = regExp.substitute("###TYPE###", typeStr.substring(0,
                            typeStr.indexOf(" ")), histStr);
              }
              else {
                  histStr = regExp.substitute("###TYPE###",typeStr.substring(0,1), histStr);
              }
              histStr = regExp.substitute("###AITEM###", histInfoStr, histStr);
              LogSystem.log(10, "OAResolved:  histStr = " + histStr);
              }
              else {
                  histStr += resTmpLdr.getTemplate(TMPSPATH + "template_history.html");
                  histStr = regExp.substitute("###TYPE###", typeStr.substring(0,1), histStr);
                  histStr = regExp.substitute("###AITEM###", histInfoStr, histStr);
                  LogSystem.log(10, "OAResolved:  histStr = " + histStr);
              }
       }
       LogSystem.log(10, "OAResolved: Current parsed hitory is= \n" + histStr);
       LogSystem.log(10, "OAResolved:  parseHistory complete.");

       return histStr;
   }//parseHistory(Vector)


    /**
    * Get the current html string
    *
    * @param None
    *
    * @return String
    **/
    public String getHTML() {
        return htmlStr;
    }//getHTML()

}//OAResolved Class
