package com.ibm.nzna.projects.oa.web;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.quest.oa.*;
import com.ibm.nzna.projects.oa.engine.*;
import com.ibm.nzna.shared.servlet.*;
import com.ibm.nzna.shared.util.*;    // RegularExpression, LogSystem, Text
import com.ibm.nzna.shared.servlet.template.*;      //ResourceTemplateLoader
import com.ibm.nzna.projects.common.legacy.*;    //Pagegen for encryption
import com.ibm.nzna.projects.common.quest.oa.*;  //OAMode
import com.ibm.nzna.properties.*;
import com.ibm.nzna.templates.*;
import java.io.*;
import java.sql.*;
import java.util.*;

/**
 * WebRenderer class
 * @author Tracey Watkins
 * @version 1.0
 * This program handles the Online Assistant Questions and Actions.  Child Symptoms
 * (better known as Symptom jumps) are handled in the OAEngine.
 *
 * @param  None
 * @return Nothing
 *
/***********************************************************************/
public class WebRenderer  {

  public static int                 INFOTIPS        = 1;
  public static int                 BPARTNERS       = 2;
  public static int                 ESUPPORT        = 3;
  public    Integer                 sessionNum      =   new Integer(0);
  private   static final int        MAX             =   11;
  public    ResourceTemplateLoader  resTmpLdr       =   new ResourceTemplateLoader();
  protected Properties              props           =   new Properties();
  public    String                  htmlStr         =   null;
  public    SessionState            sessionState;
  public    HttpRequest             httpReq;
  public    LogicAPI                logicAPI;
  public    int                     symptomInd      =   0;
  private   String                  sympTitle       =  "tempTitle";
  private   Vector                  ansIndVec       =   new Vector();
  private   OASubmitVars            oasv;
  private   OAReturnVars            oarv;
  private   String                  commandStr      =   null;
  private   String                  currentURL      =   null;
  private   Vector                  histVec         =   new Vector();
  private   String                  tmpsPath        =   "com/ibm/nzna/templates/oa/";
  private   String                  fileSep         =   System.getProperty("file.separator");
  private   boolean                 appDriver       =   false;  //Defaults to NetDriver
  private   String                  oaServlet       =   "";
  private   String                  baseServlet     =   null;
  private   String                  actionValue     =   null;
  private   String                  imagesPath      =   null;
  private   String                  queryStr     =   null;
//  private   String                  propsPath       =   null;


  public WebRenderer() {
      System.out.println("WebRenderer:  New WebRenderer object instantiated...");
      init();
  } //constructor WebRenderer


  /*------------------------------------------------------------*/
  /* init()
   */
  /**
   *  Initializes the webrendering object by loading the properties files,
   * setting the OAMode to web to improve database performance, loading the
   * database info and instantiating LogicAPI.
   *
   *  @param  None
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
   public void init () {
       
       LogSystem.beginLogSection("WebRenderer:  Initializing...");
       OAMode.setMode(OAMode.WEB);
       LogSystem.log(1, "WebRenderer:  Current OAMode = " + OAMode.getMode());
       Properties cprops = new Properties();
       Properties lprops = new Properties();
       try {
           boolean loaded;
           String path = StandardPropertyLoader.getPath();
           String propsPath = "com/ibm/nzna/properties/oa/";
           LogSystem.log(10, "WebRenderer:  path=  " + path);
           LogSystem.log(10, "WebRenderer:  propsPath=  " + propsPath);
           LogSystem.log(10, "WebRenderer:  " +propsPath + "WebDaemon.properties");
           props = StandardPropertyLoader.load(propsPath +"WebDaemon.properties");
           if (props.isEmpty()) {
               loaded = false;
               LogSystem.log(1, "WebRenderer:  props file empty.");
               LogSystem.log(1, "WebRenderer:  props file=  " + props.toString());
           }
           else {
               loaded = true;
               LogSystem.log(10, "WebRenderer:  props file not empty.");
           }
           LogSystem.log(1, "WebRenderer:  loaded WebDaemon.properties? " + loaded);
           cprops = StandardPropertyLoader.load(props.getProperty("country.mapping"));
           if (cprops.isEmpty()) {
               loaded = false;
           }
           else {
               loaded = true;
           }
           LogSystem.log(10, "WebRenderer:  country map: " + propsPath + props.getProperty("country.mapping"));
           LogSystem.log(1, "WebRenderer:  Country Map Props loaded? " + loaded);
           lprops = StandardPropertyLoader.load(props.getProperty("language.mapping"));
           if (lprops.isEmpty()) {
               loaded = false;
           }
           else {
               loaded = true;
           }
           LogSystem.log(10, "WebRenderer:  language map: " + propsPath + props.getProperty("language.mapping"));
           LogSystem.log(1, "WebRenderer:  Language Map Props loaded? " + loaded);

           LogSystem.log(1, "WebRenderer:  Instantiating LogicAPI...");
           logicAPI = new LogicAPI(0, 0, path + props.getProperty("sessionLogPath"),
                                   cprops, lprops);
           
           LogSystem.log(1, "WebRenderer:  Getting the oa servlet url...");
           oaServlet = props.getProperty("servlet.oa.url");
           oaServlet = oaServlet.substring(oaServlet.indexOf("=")+1);
           if (oaServlet.equals(null))
           {
//               oaServlet = "/oa";
               oaServlet = "/esupservice/redirect.oa/";
           }
           LogSystem.log(1, "WebRenderer:  oaServlet= "+ oaServlet);

           LogSystem.log(1, "WebRenderer:  Getting quest db props...");
           String dbname = null;
           String server = null;
           String port = null;
           String uid = null;
           String pw = null;
           
           String tmpStr = props.getProperty("db.driver.app");
           appDriver = tmpStr.substring(tmpStr.indexOf("=")+1).equalsIgnoreCase("true");
           LogSystem.log(1, "WebRenderer:  appDriver= "+ appDriver);

           dbname = props.getProperty("db.quest.name");
           dbname = dbname.substring(dbname.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  dbname= "+ dbname);
           server = props.getProperty("db.quest.host");
           server = server.substring(server.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  server= "+ server);
           port = props.getProperty("db.quest.port");
           port = port.substring(port.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  port= "+ port);
           uid = props.getProperty("db.quest.uid");
           uid = uid.substring(uid.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  uid= "+ uid);
           pw = props.getProperty("db.quest.pwd");
           pw = pw.substring(pw.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  LogicAPI.setQuestNames...");
           logicAPI.setQuestNames(dbname, server, port, uid, pw, appDriver);

           LogSystem.log(1, "WebRenderer:  Getting Clio props...");
           dbname = props.getProperty("db.clio.name");
           dbname = dbname.substring(dbname.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  dbname= "+ dbname);
           server = props.getProperty("db.clio.host");
           server = server.substring(server.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  server= "+ server);
           port = props.getProperty("db.clio.port");
           port = port.substring(port.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  port= "+ port);
           uid = props.getProperty("db.clio.uid");
           uid = uid.substring(uid.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  uid= "+ uid);
           pw = props.getProperty("db.clio.pwd");
           pw = pw.substring(pw.indexOf("=")+1);
           LogSystem.log(1, "WebRenderer:  LogicAPI.setQuestNames...");
           logicAPI.setClioNames(dbname, server, port, uid, pw, appDriver);

       } //try
       catch (Exception e) {
           LogSystem.log(1, "WebRenderer:  Exception caught in init(). ");
           LogSystem.log(1, e);
       } //catch
   } //method init()


  /*------------------------------------------------------------*/
  /* processRequest(SessionState)
   */
  /**
   *  This method processes each incoming SessionState object.  The bulk of the
   * WebRenderer is handled through this method.  It converts the HttpRequest
   * object to an OASubmitVars object.  That OASubmitVars object is then passed
   * to LogicAPI.  Then, the OAReturnVars object returned from LogicAPI is
   * converted into an HTML String through the use of templates.
   *
   *  @param  SessionState ss
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
   public void processRequest (SessionState ss) {

       LogSystem.log(10, "WebRenderer:  Processing request...");
       try {
          sessionState =  ss;
          LogSystem.log(10, "WebRenderer:  Getting current request...");
          httpReq = sessionState.getRequest();
//          int classInd = Integer.parseInt(httpReq.getParameter("docclass"));
//          imagesPath = this.setImagesPath(classInd);
//          baseServlet = this.setImagesPath(classInd);

          LogSystem.log(10, "WebRenderer:   Num of Parameters= " + httpReq.getNumParameters());
          LogSystem.log(10, "WebRenderer:  Creating OASubmitVars object...");
          oasv = httpReq2SubVars(httpReq);
          LogSystem.log(10, "WebRenderer:  symptomInd = " + symptomInd);
          LogSystem.log(10, "WebRenderer:  oasv created... \n" + oasv.toString());

          if (oasv.command.equals("exit") || oasv.command.equalsIgnoreCase("endSession")) {
              //load template for endSession
              htmlStr = retVars2HTML(oarv);
              LogSystem.log(10, "WebRenderer:  Loading endSession template complete.");
              return;
          }
          else if (oasv.command.equals("createURForm")) {
              //load template for creating the OAUnresolved Form
              LogSystem.log(10, "oasv.CustNum: " + oasv.custNum +
                                 "\nsymptomInd: " + symptomInd +
                                 "\noarv.title: " + oarv.title +
                                 "\noasv.docClass: " + oasv.docClass +
                                 "\noasv.brand: " + oasv.brand +
                                 "\noasv.typeModel: " +oasv.typeModel +
                                 "\nhistVec: " + histVec +
                                 "\nappDriver: " + appDriver +
                                 "\nimagesPath: " + imagesPath +
                                 "\nbaseServlet: " + baseServlet +
                                 "\noaServlet: " + oaServlet +
                                 "\nqueryStr: " + queryStr);
              OAUnresolved oaUnresolved = new OAUnresolved(oasv.custNum, symptomInd, oarv.title,
                                                           oasv.docClass, oasv.brand, oasv.country,
                                                           oasv.typeModel, histVec, appDriver,
                                                           imagesPath, baseServlet, oaServlet, queryStr);
              htmlStr = oaUnresolved.createURForm();
              LogSystem.log(10, "WebRenderer:  Loading OAUnresolvedForm template complete.");
              return;
          }
          else if (oasv.command.equals("createStatPg")) {
              //load template for creating the OAUnresolved Form
              LogSystem.log(10, "oasv.CustNum: " + oasv.custNum +
                                 "\nsymptomInd: " + symptomInd +
                                 "\noarv.title: " + oarv.title +
                                 "\noasv.docClass: " + oasv.docClass +
                                 "\noasv.brand: " + oasv.brand +
                                 "\noasv.typeModel: " +oasv.typeModel +
                                 "\nhistVec: " + histVec +
                                 "\nappDriver: " + appDriver +
                                 "\nimagesPath: " + imagesPath +
                                 "\nbaseServlet: " + baseServlet +
                                 "\noaServlet: " + oaServlet +
                                 "\nqueryStr: " + queryStr);
              OAUnresolved oaUnresolved = new OAUnresolved(oasv.custNum, symptomInd, oarv.title,
                                                           oasv.docClass, oasv.brand, oasv.country,
                                                           oasv.typeModel, histVec, appDriver,
                                                           imagesPath, baseServlet, oaServlet, queryStr);
              htmlStr = oaUnresolved.createStatPage(this.httpReq);
              oasv = new OASubmitVars(0);
              LogSystem.log(10, "WebRenderer:  Loading OAUnresolvedForm template complete.");
              return;
          }

          LogSystem.log(10, "WebRenderer:  Calling LogicAPI.dispatch...");
          oarv = logicAPI.dispatch(oasv, symptomInd);

          LogSystem.log(10, "WebRenderer:  The current oaRetVars object= \n" + oarv.toString());

          if (oasv.command.equals("SaveResolved")){
              //Call OAResolved.
              //Will need to pass the URL w/path and the parsed history HTML String.
              htmlStr = parseHistory(oarv.addInfo);
              LogSystem.log(10, "oasv.CustNum: " + oasv.custNum +
                                 "\nsymptomInd: " + symptomInd +
                                 "\noarv.title: " + oarv.title +
                                 "\noasv.docClass: " + oasv.docClass +
                                 "\noasv.brand: " + oasv.brand +
                                 "\noasv.typeModel: " +oasv.typeModel +
                                 "\nhistVec: " + histVec +
                                 "\nimagesPath: " + imagesPath +
                                 "\nbaseServlet: " + baseServlet +
                                 "\noaServlet: " + oaServlet +
                                 "\nqueryStr: " + queryStr);
              OAResolved oaResolved = new OAResolved (oasv.custNum, symptomInd, oarv.title,
                                                      oasv.docClass, oasv.brand, oasv.typeModel,
                                                      histVec, imagesPath, baseServlet, oaServlet,
                                                      queryStr);
              htmlStr = oaResolved.getHTML();
              LogSystem.log(10, "WebRenderer:  oaResolved.getHTML() = \n" + oaResolved.getHTML());
              LogSystem.log(10, "WebRenderer:  oarv= " + oarv.toString());
              oasv = new OASubmitVars(0);
          }
          else if (oasv.command.equals("SaveUnresolved")){
              //Call OAUnresolved.
              //Will need to pass the URL w/path and the parsed history HTML String.
              String tmpStr = parseHistory(oarv.addInfo);
              //Instantiate OAUnresolved.
              LogSystem.log(10, "WebRenderer:  SaveUnresolved and future OAUnresolved.");
              LogSystem.log(10, "oasv.CustNum: " + oasv.custNum +
                                 "\nsymptomInd: " + symptomInd +
                                 "\noarv.title: " + oarv.title +
                                 "\noasv.docClass: " + oasv.docClass +
                                 "\noasv.brand: " + oasv.brand +
                                 "\noasv.typeModel: " +oasv.typeModel +
                                 "\nhistVec: " + histVec +
                                 "\nappDriver: " + appDriver +
                                 "\nimagesPath: " + imagesPath +
                                 "\nbaseServlet: " + baseServlet +
                                 "\noaServlet: " + oaServlet +
                                 "\nqueryStr: " + queryStr);
              OAUnresolved oaUnresolved = new OAUnresolved(oasv.custNum, symptomInd, oarv.title,
                                                           oasv.docClass, oasv.brand, oasv.country,
                                                           oasv.typeModel, histVec, appDriver,
                                                           imagesPath, baseServlet, oaServlet, queryStr);
              LogSystem.log(10, "WebRenderer:  OAUnresolved instantiation complete.");
              htmlStr = oaUnresolved.generateLinkHistoryForm();
              if (oasv.docClass == INFOTIPS) {
                  oasv = new OASubmitVars(0);
              }
              return;
          }
          else {
              htmlStr = retVars2HTML(oarv);
          }
       } //try
       catch (Exception e) {
           LogSystem.log(1, "WebRenderer:  Exception caught in processRequest(ss)= ");
           LogSystem.log(1, e);
       } //catch
   } //method processRequest(SessionState)


   /*------------------------------------------------------------*/
   /* setBaseServlet(int)
    */
   /**
    *    This method returns the URL of the customer's personal support page based on
    * the website the customer is currently logged into.  This url is in String format.
    *
    *  @param  int      int value of the docClass indicating the website
    *  @return String   String representation of the current base servlet
    *//*---------------------------------------------------------*/
   public String setBaseServlet (int docClass) {

      if (docClass==INFOTIPS) {
          //Current website is InfoTips
          baseServlet = props.getProperty("url.infotips.home");
          if (baseServlet.equals(null))
          {
              baseServlet = "/helpcenter/infotips/";
          }
      }
      else if (docClass==BPARTNERS) {
          //Current website is BPartners
          baseServlet = props.getProperty("url.bp.home");
          if (baseServlet.equals(null))
          {
              baseServlet = "/partner/infotips/";
          }
      }
      else {
          //Current website is eSupport or default to eSupport if invalid
          //docClassInd is passed.
          baseServlet = props.getProperty("url.esupport.home");
          if (baseServlet.equals(null))
          {
              baseServlet="/support/";
          }
      }
       return baseServlet;
   } // setBaseServlet(int)



   /*------------------------------------------------------------*/
   /* setImagesPath(int)
    */
   /**
    *    This method returns the path of the images directory based on the website
    * the customer is currently logged into. 
    *
    *  @param  int      int value of the docClass indicating the website
    *  @return String   String representation of the current images directory
    *//*---------------------------------------------------------*/
   public String setImagesPath (int docClass) {

      String iPath = null;
      if (docClass==INFOTIPS) {
          //Current website is InfoTips
          iPath = props.getProperty("images.path.it");
          if (iPath.equals(null))
          {
              iPath = "/infotips/images/";
          }
      }
      else if (docClass==BPARTNERS) {
          //Current website is BPartners
          iPath = props.getProperty("images.path.bp");
          if (iPath.equals(null))
          {
              iPath = "/partner/docs/images/";
          }
      }
      else {
          //Current website is eSupport or default to eSupport if invalid
          //docClassInd is passed.
          iPath = props.getProperty("images.path.es");
          if (iPath.equals(null))
          {
              iPath = "/support/images/";
          }
      }
       return iPath;
   } // setImagesPath(int)


   /*------------------------------------------------------------*/
   /* getHTML
    */
   /**
    *    This method returns the current HTML String.
    *
    *  @param  none
    *  @return String   String the representation of the current HTML
    *//*---------------------------------------------------------*/
   public String getHTML () {
       return htmlStr;
   } // getHTML()


   /*------------------------------------------------------------*/
   /* httpReq2SubVars(HttpRequest)
    */
   /**
    *    This method converts the current HttpRequest object into
    *   an OASubmitVars object.
    *
    *  @param  httpReq      HttpRequest
    *  @return oaSubVars    OASubmitVars
    *//*---------------------------------------------------------*/
   public OASubmitVars httpReq2SubVars (HttpRequest httpReq) {
       LogSystem.log(10, "WebRenderer:  Converting HttpRequest object to OASubmitVars object...");
       LogSystem.log(10, "WebRenderer:  sessionNum= " + sessionNum.toString());
       LogSystem.log(10, "WebRenderer:  sessionNum.intValue() = " + sessionNum.intValue());

       OASubmitVars oaSubVars = new OASubmitVars(0);
       try {
          oaSubVars = new OASubmitVars(sessionNum.intValue());
          //          String cmd = httpReq.getParameter("command");
          if (sessionNum.intValue() == 0) {
              oaSubVars.sessionSeqNo = 0;
              LogSystem.log(10, "WebRenderer:  sessionNum = 0, so setting oaSubVars.sessionSeqNo=0");
              this.queryStr = httpReq.getQueryString();
          }
//          else if ((sessionNum.intValue() > 0) && !(httpReq.getParameter("page")==null)) {
          else if ((sessionNum.intValue() > 0) && (httpReq.getParameter("page")==null)) {
              oaSubVars.sessionSeqNo = 0;
              oaSubVars.sessionInd = 0;
              LogSystem.log(10, "WebRenderer:  sessionNum > 0 and no page, so resetting ");
              this.queryStr = httpReq.getQueryString();
          }
//          else if ((sessionNum.intValue() > 0) && cmd.equals("createURForm")) {
          else if ((sessionNum.intValue() > 0) &&
                   !(httpReq.getParameter("command")==null) &&
                   (httpReq.getParameter("command").equalsIgnoreCase("createURForm")) ) {
              LogSystem.log(10, "WebRenderer:  cmd=createURForm, so oaSubVars=oasv + cmd");
              oaSubVars = oasv;
              oaSubVars.command = "createURForm";
              return oaSubVars;
          }
          else if ((sessionNum.intValue() > 0) &&
                   !(httpReq.getParameter("command")==null) &&
                   (httpReq.getParameter("command").equalsIgnoreCase("createStatPg")) ) {
              LogSystem.log(10, "WebRenderer:  cmd=createStatPg, so oaSubVars=oasv + cmd");
              oaSubVars = oasv;
              oaSubVars.command = "createStatPg";
              return oaSubVars;
          }
          else {
              LogSystem.log(10, "WebRenderer:  default...");
              oaSubVars.sessionSeqNo = oarv.sessionSeqNo;
              LogSystem.log(10, "WebRenderer:  oaSubVars.sessionSeqNo= " + oaSubVars.sessionSeqNo);
          }
          LogSystem.log(10, "WebRenderer:  Current Query String\n " + httpReq.getQueryString());

          int commandType = 0;     //GetNextNode

          Integer sesSeqNo = new Integer("0");
          Enumeration enum = httpReq.getParameterNames();
          LogSystem.log(10, "\nWebRenderer: httpReq enumeration= ");
          while (enum.hasMoreElements()) {
              String parmName = (String)enum.nextElement();
              LogSystem.log(10, parmName + " = " + httpReq.getParameter(parmName));

              if (parmName.startsWith("exit")) {
                  commandType=5;
              }
              else if (parmName.startsWith("modify")) {
                  commandType=3;
              }
              else if (parmName.startsWith("reset")) {
                  commandType=4;
              }
              else if (parmName.equalsIgnoreCase("nodeInd")) {
                  oaSubVars.nodeInd = Integer.parseInt(httpReq.getParameter("nodeInd"));
              }
              else if (parmName.equalsIgnoreCase("node_flag")) {
                  oaSubVars.node_flag = httpReq.getParameter("node_flag");
              }
              else if (parmName.equalsIgnoreCase("response")) {
                  Integer resp = new Integer("0");
                  String tmpStr = httpReq.getParameter("response");
                  LogSystem.log(10, "WebRenderer:  tmpSr (response)= " + tmpStr);
                  if (tmpStr.equals(null) || tmpStr.equals("")) {
                      oaSubVars.response = 0;
                  }
                  else {
                      resp = Integer.valueOf(tmpStr);
                      int t = resp.intValue();
                      LogSystem.log(10, "ansIndVec.size(): " + ansIndVec.size() + "  resp: " + resp.toString());
                      resp = (Integer)ansIndVec.elementAt(t);
                      oaSubVars.response = resp.intValue();
                  }
              }
              else if (parmName.equalsIgnoreCase("message")) {
                   oaSubVars.message = httpReq.getParameter("message");
               }
               else if (parmName.equalsIgnoreCase("brand")) {
                   String tmpStr = httpReq.getParameter("brand");
                   if (tmpStr.indexOf("|") > 0) {
                        oaSubVars.brand = tmpStr.substring(0, tmpStr.indexOf("|"));
                   }
                   else {
                        oaSubVars.brand = tmpStr;
                   }
               }
               else if (parmName.equalsIgnoreCase("machineType")) {
                   String tmpStr = httpReq.getParameter("machineType");
                   if (tmpStr.lastIndexOf("|") > 0) {
                        if (oaSubVars.typeModel.length() <= 0) {
                             oaSubVars.typeModel = tmpStr.substring(tmpStr.lastIndexOf("|")+1);
                             LogSystem.log(10, "WebRenderer:  oaSubVars.typeModel=" + oaSubVars.typeModel);
                        }
                        else {
                            String modelStr = oaSubVars.typeModel;
                            oaSubVars.typeModel = tmpStr.substring(tmpStr.lastIndexOf("|")+1) + "-" + modelStr;
                        }

                   }
                   else if (oaSubVars.typeModel.length() > 0) {
                       String modelStr = oaSubVars.typeModel;
                       oaSubVars.typeModel = tmpStr + "-" + modelStr;
                           
                   }
                   else oaSubVars.typeModel = tmpStr;
               }
               else if (parmName.equalsIgnoreCase("model")) {
                   String tmpStr = httpReq.getParameter("model");
                   if (tmpStr.lastIndexOf("|") > 0) {
                        LogSystem.log(10, "WebRenderer:  oaSubVars.typeModel=" + oaSubVars.typeModel);
                        if (oaSubVars.typeModel.length() > 0) {
                            oaSubVars.typeModel += "-" + tmpStr.substring(tmpStr.lastIndexOf("|")+1);
                            LogSystem.log(10, "WebRenderer:  oaSubVars.typeModel=" + oaSubVars.typeModel);
                        }
                        else {
                            oaSubVars.typeModel = tmpStr.substring(tmpStr.lastIndexOf("|")-4,
                                                   tmpStr.lastIndexOf("|"));
                            LogSystem.log(10, "WebRenderer:  oaSubVars.typeModel=" + oaSubVars.typeModel);
                            oaSubVars.typeModel += tmpStr.substring(tmpStr.lastIndexOf("|")+1);
                            LogSystem.log(10, "WebRenderer:  oaSubVars.typeModel=" + oaSubVars.typeModel);
                        }
                   }
                   else if (oaSubVars.typeModel.length() <= 0)
                        oaSubVars.typeModel = tmpStr;
                   else oaSubVars.typeModel += "-" + tmpStr;
               }
               else if (parmName.equalsIgnoreCase("lang")) {
                   oaSubVars.lang = httpReq.getParameter("lang");
               }
               else if (parmName.equalsIgnoreCase("country")) {
                   oaSubVars.country = httpReq.getParameter("country");
               }
               else if (parmName.equalsIgnoreCase("docclass")) {
                   oaSubVars.docClass = Integer.parseInt(httpReq.getParameter("docclass"));
               }
               else if (parmName.equalsIgnoreCase("mtmdl")) {
                   oaSubVars.typeModel = httpReq.getParameter("mtmdl");
               }
               else if (parmName.equalsIgnoreCase("up")) {
                   String tmpStr = httpReq.getParameter("up");
                   if (tmpStr.equals(null) || tmpStr.equals("")) {
                       oaSubVars.custNum = tmpStr;
                   }
                   else {
                       tmpStr = Pagegen.decode(tmpStr);
                       oaSubVars.custNum = tmpStr.substring(tmpStr.lastIndexOf("=")+1);
                   }
               }
               else if (parmName.equalsIgnoreCase("symptom")) {
                   symptomInd = Integer.parseInt(httpReq.getParameter("symptom"));
               }
               else if (parmName.equalsIgnoreCase("lang")) {
                   oaSubVars.lang = httpReq.getParameter("lang");
               }
               else if (parmName.equalsIgnoreCase("sesSeqNo")) {
                   sesSeqNo = new Integer(httpReq.getParameter("sesSeqNo"));
               }
               else if (parmName.equalsIgnoreCase("command")) {
                   oaSubVars.command = httpReq.getParameter("command");
                   if (oaSubVars.command.equalsIgnoreCase("resolved")){
                      commandType = 1;
                   }
                   else if (oaSubVars.command.equalsIgnoreCase("unresolved")) {
                      commandType = 2;
                   }
               }
           }//while (enum.hasMoreElements())
          
           //default testing values...
           if (oaSubVars.response<0) {  //setting default response
               oaSubVars.response = 0;
           }
           if (symptomInd <= 0) {  //setting default symptomInd
               symptomInd = this.getSymptomInd(oaSubVars.brand, symptomInd);
           }
           if (oaSubVars.docClass == 0) {  //setting default docClassInd
               oaSubVars.docClass = 3;
           }

           //setting default baseServlet and imagesPath
           baseServlet = setBaseServlet(oaSubVars.docClass);
           LogSystem.log(1, "WebRenderer:  baseServlet = " + baseServlet);
           imagesPath = setImagesPath(oaSubVars.docClass);
           LogSystem.log(1, "WebRenderer:  imagesPath = " + imagesPath);


           oaSubVars.qitMemory=null;

           if (commandType==4) {
               //Reset
               if(sesSeqNo.equals("0")){    //if sesSeqNo = "0", then prev cmd not "GetHistory"
                   oaSubVars.sessionSeqNo = 1;
               }
               else {
                   oaSubVars.sessionSeqNo = sesSeqNo.intValue() + 1;
               }
               symptomInd = this.getSymptomInd(oaSubVars.brand, 0);
               oaSubVars.nodeInd = 0;
               oaSubVars.response=0;
               oaSubVars.command = "SetHistory";
               LogSystem.log(10, "sesSeqNo(cmdType=4): " +sesSeqNo);
           }
           else if (commandType==3) {
               //Modify
               oaSubVars.response=0;
               oaSubVars.nodeInd=0;
               oaSubVars.command = "GetHistory";
           }
           else if (commandType==2) {
               //Exit Save Unresolved
               oaSubVars = oasv;  //Use previous oaSubmitVars object
               oaSubVars.command = "SaveUnresolved";
           }
           else if (commandType==1) {
               //Exit Save Resolved
               oaSubVars = oasv;  //Use previous oaSubmitVars object
               oaSubVars.command = "SaveResolved";
           }
           else if (commandType==5) {
               //Exit (Will need to present user with ability to save as resolved or
               //save as unresolved.)
               oaSubVars.command = "endSession";
           }
       }//try
       catch (Exception e) {
           LogSystem.log(1, "WebRenderer:  Exception caught in httpReq2SubVars= ");
           LogSystem.log(1, e);
       }//catch
           LogSystem.log(10, "WebRenderer:  Converting HttpRequest object to OASubmitVars object completed.");
           return oaSubVars;
   } //httpReq2SubVars(HttpRequest)



   /*------------------------------------------------------------*/
   /* getSymptomInd(String, int)
    */
   /**
    *    This method converts the current OAReturnVars object into
    *   a String object consisting of HTML information.
    *
    *  @param  String   brand
    *  @param  int      sympInd
    *  @return int      symptomInd
    *//*---------------------------------------------------------*/
    public int getSymptomInd(String oaBrand, int symptomInd) {
        
        LogSystem.log(1, "WebRenderer:  Entering method getSymptomInd(String, int) ");
        Integer sympInd = new Integer(0);
        try {
            String brand = oaBrand;
            LogSystem.log(10, "WebRenderer:  brand =  " + brand);
            sympInd = new Integer(symptomInd);
            LogSystem.log(10, "WebRenderer:  symptomInd= " + symptomInd);
            LogSystem.log(10, "WebRenderer:  sympInd = " + sympInd.toString());

            if (sympInd.intValue() <= 0) {
                String tmpStr = "";
                LogSystem.log(10, "WebRenderer:  brand (before replace) =  " + brand);
                brand = "top." + brand.replace(' ', '.');
                LogSystem.log(10, "WebRenderer:  brand (after replace)=  " + brand);
                tmpStr = props.getProperty(brand);
                LogSystem.log(10, "WebRenderer:  tmpStr (property)=  " + tmpStr);
                if ( !tmpStr.equals(null))
                {
                    sympInd = Integer.valueOf(tmpStr.substring(tmpStr.indexOf("=")+1));
                    LogSystem.log(10, "WebRenderer:  sympInd = " + sympInd.toString());
                }
                else sympInd = new Integer(1);
            } //end if sympInd <=0
        }//try
        catch (Exception e) {
            LogSystem.log(1, "WebRenderer:  Exception caught in getSymptomInd= ");
            LogSystem.log(1, e);
        }
        finally {
            LogSystem.log(10, "WebRenderer: returning sympInd = " + sympInd.toString());
            LogSystem.log(1, "WebRenderer:  Exiting method getSymptomInd(String, int) ");
            return sympInd.intValue();
        }
    }//end getSymptomInd(String, int)


   /*------------------------------------------------------------*/
   /* retVars2HTML(OAReturnVars)
    */
   /**
    *    This method converts the current OAReturnVars object into
    *   a String object consisting of HTML information.
    *
    *  @param  oaRetVars    OAReturnVars
    *  @return htmlStr      String
    *//*---------------------------------------------------------*/
   public String retVars2HTML (OAReturnVars oaRetVars) {
       LogSystem.log(10, "WebRenderer:  Converting OAReturnVars object to HTML String object...");

       String tmpStr = "";
       String htmlStr = "";
       RegularExpression regExp = new RegularExpression();
       Vector tmpVec = new Vector();
       String nodeType = "";
       if (oasv.command.equalsIgnoreCase("endSession") || oasv.command.equalsIgnoreCase("exit")){
          htmlStr = this.resTmpLdr.getTemplate(tmpsPath + "template_endsession.html");
          nodeType = "EO";
       }
       else if (oasv.command.equalsIgnoreCase("GetNextNode")){
           htmlStr = this.resTmpLdr.getTemplate(tmpsPath + "template_whitespace.html");
           symptomInd = oaRetVars.objectInd;
           nodeType = oaRetVars.node_flag;
       }
       else {
             htmlStr = this.resTmpLdr.getTemplate(tmpsPath + "template_whitespace.html");
             nodeType = oaRetVars.node_flag;
       }
       Integer node = new Integer(oaRetVars.nodeInd);
       sessionNum = new Integer(oaRetVars.sessionInd);

       if (nodeType.endsWith("Q")) {
           LogSystem.log(10, "WebRenderer:  Current node is a Question.");
           //Question
           LogSystem.log(10, "WebRenderer:  Question conversion starting...");

           //Answers
           LogSystem.log(10, "WebRenderer:  Answer conversion starting...");
           tmpStr = "";
           String ansStr = "";
           tmpVec = oaRetVars.answerTitles;
           ansIndVec = oaRetVars.answerInds;
           Integer tmpI = new Integer("0");
           if (!tmpVec.isEmpty()) {
               ansStr = this.resTmpLdr.getTemplate(tmpsPath + "template_question.html");
               LogSystem.log(10, "WebRenderer:  Number of Answers=  " + tmpVec.size());
               tmpStr="";
               if (tmpVec.size()< MAX) {
                   for (int i=0; i < tmpVec.size(); i++) {
                       tmpStr += this.resTmpLdr.getTemplate(tmpsPath + "template_ansradio.html");
                       tmpStr = regExp.substitute("###ANSWER###", this.resTmpLdr.getTemplate
                                                  (tmpsPath + "template_answer.html"), tmpStr);
                       tmpStr = regExp.substitute("###ANSNO###", tmpI.toString(i), tmpStr);
                       tmpStr = regExp.substitute("###ANSWER###", (String)tmpVec.elementAt(i), tmpStr);
                   } //for
                   ansStr = regExp.substitute("###ANSWERITEMS###", tmpStr, ansStr);
               } //if
               else {
                   String ansListStr = this.resTmpLdr.getTemplate(tmpsPath + "template_anslist.html");
                   for (int i=0; i<tmpVec.size(); i++) {
                       tmpStr += this.resTmpLdr.getTemplate(tmpsPath + "template_answer.html");
                       tmpStr = regExp.substitute("###ANSNO###", tmpI.toString(i), tmpStr);
                       tmpStr = regExp.substitute("###ANSWER###", (String)tmpVec.elementAt(i), tmpStr);
                   } //for
                   ansListStr = regExp.substitute("###ANSLISTITEM###", tmpStr, ansListStr);
                   ansStr = regExp.substitute("###ANSWERITEMS###", ansListStr, ansStr);
               } //else
               htmlStr = regExp.substitute("###NEXTNODE###", ansStr, htmlStr);
           } // if tmpVec not empty
           else {
               htmlStr = regExp.substitute("###NEXTNODE###", " ", htmlStr);
           } //else
           LogSystem.log(10, "WebRenderer:  Answer conversion complete.");
       } //if nodeType == Question
       else if (nodeType.endsWith("A")) {
           //Actions
           LogSystem.log(10, "WebRenderer:  Node is an Action.");
           LogSystem.log(10, "WebRenderer:  Action converion starting...");
           tmpStr = "";
           String actStr = "";
           tmpVec = oaRetVars.actionTitles;
           if (!tmpVec.isEmpty()) {
               actStr = this.resTmpLdr.getTemplate(tmpsPath + "template_action.html");
               for (int i=0; i < tmpVec.size(); i++) {
                   tmpStr += this.resTmpLdr.getTemplate(tmpsPath + "template_actitem.html");
                   tmpStr = regExp.substitute("###ACTION###", (String)tmpVec.elementAt(i), tmpStr);
               } //for
               actStr = regExp.substitute("###ACTIITEM", tmpStr, actStr);

               htmlStr = regExp.substitute("###NEXTNODE###", actStr, htmlStr);
           } // if
           else {
                htmlStr = regExp.substitute("###NEXTNODE###", "", htmlStr);
           }
       }//nodeType = Action
       else if (nodeType.equals("EO")) {
           //Exiting Other
           LogSystem.log(10, "WebRenderer:  Current session is exiting.");
           htmlStr  = resTmpLdr.getTemplate(tmpsPath + "template_endsession.html");
       }  //nodeType = EO (exiting other)
       else {
           LogSystem.log(10, "WebRenderer:  Node type INVALID!!!");
           htmlStr = regExp.substitute("###NEXTNODE###", "", htmlStr);
       } //nodeType unknown

       //Now replace the common pieces...
       //Title
       htmlStr = regExp.substitute("###TITLE###", oaRetVars.title, htmlStr);

       // Links
       LogSystem.log(10, "WebRenderer:  Links conversion starting...");
       tmpStr = "";
       String linkStr = "";
       Vector tmpVec2 = new Vector();
       try {
         if (oaRetVars.linkTitles.isEmpty()) {
            LogSystem.log(10, "WebRenderer:  oaRetVars.linkTitles is empty.");
         }
         else if (oaRetVars.linkTitles.isEmpty()) {
            LogSystem.log(10, "WebRenderer:  oaRetVars.linkURLs is empty.");
         }
         else {
             tmpVec = oaRetVars.linkTitles;
             LogSystem.log(10, "WebRenderer:  tmpVec=oaRetVars.linkTitles complete.\n size = " + tmpVec.size());
             tmpVec2 = oaRetVars.linkURLs;
             LogSystem.log(10, "WebRenderer:  tmpVec=oaRetVars.linkURLs complete.\n size = " + tmpVec.size());
             linkStr = this.resTmpLdr.getTemplate(tmpsPath + "template_links.html");
             for (int i=0; i < tmpVec.size(); i++) {
                tmpStr += this.resTmpLdr.getTemplate(tmpsPath + "template_linkitem.html");
                tmpStr = regExp.substitute("###LINKURL###", (String)tmpVec2.elementAt(i), tmpStr);
                tmpStr = regExp.substitute("###LINKTITLE###", (String)tmpVec.elementAt(i), tmpStr);
             } //for
             linkStr = regExp.substitute("###LINKITEM###", tmpStr, linkStr);
         } //else
         htmlStr = regExp.substitute("###TEMPLATE_LINKS###", linkStr, htmlStr);
       }//try
       catch (Exception e) {
           LogSystem.log(10, "WebRenderer:  retVars2HTML():  Exception caught in link conversion");
           LogSystem.log(1, e);
           htmlStr = regExp.substitute("###TEMPLATE_LINKS###", "", htmlStr);
       }
       LogSystem.log(10, "WebRenderer:  Links conversion complete.");

       //Hidden Info
       LogSystem.log(10, "WebRenderer:  Hidden Info conversion starting...");
       try {
           tmpStr = "";
           Integer tmpI;
           tmpStr = this.resTmpLdr.getTemplate(tmpsPath + "template_hidden_info.html");
           tmpStr = regExp.substitute("###HIDDEN_INFO###", oaRetVars.title, tmpStr);
           tmpI = new Integer(oaRetVars.objectInd);
           tmpStr = regExp.substitute("###objectInd", tmpI.toString(), tmpStr);
           tmpI = new Integer(symptomInd);
           tmpStr = regExp.substitute("###symptom", tmpI.toString(), tmpStr);
           tmpStr = regExp.substitute("###node_flag", oaRetVars.node_flag, tmpStr);
           tmpI = new Integer(oaRetVars.nodeInd);
           tmpStr = regExp.substitute("###nodeInd", tmpI.toString(), tmpStr);
           sessionNum = new Integer(oaRetVars.sessionInd);
           tmpStr = regExp.substitute("###sessionInd", sessionNum.toString(), tmpStr);
           tmpI = new Integer(oaRetVars.sessionSeqNo);
           LogSystem.log(10, "WebRenderer:  oaReturnVars.sessionSeqNo = " + tmpI.toString());
           tmpStr = regExp.substitute("###sessionSeqNo", tmpI.toString(), tmpStr);
           tmpI = new Integer(oasv.response);
           tmpStr = regExp.substitute("###message", oasv.message, tmpStr);
           tmpStr = regExp.substitute("###country", oasv.country, tmpStr);
           tmpStr = regExp.substitute("###language", oasv.lang, tmpStr);
           tmpStr = regExp.substitute("###brand", oasv.brand, tmpStr);
           tmpStr = regExp.substitute("###typeModel", oasv.typeModel, tmpStr);
           String tmp =  Pagegen.encode(oasv.custNum);
           tmpStr = regExp.substitute("###custNum", tmp, tmpStr);
           tmpI = new Integer(oasv.docClass);
           tmpStr = regExp.substitute("###docclass", tmpI.toString(), tmpStr);
           htmlStr = regExp.substitute("###TEMPLATE_HIDDEN_INFO###", tmpStr, htmlStr);
       }  //try
       catch (Exception e) {
           LogSystem.log(1, "WebRenderer:  retVars2HTML():  Exception caught in hidden info conversion");
           LogSystem.log(1, e);
           htmlStr = regExp.substitute("###TEMPLATE_LINKS###", "", htmlStr);
       }
       LogSystem.log(10, "WebRenderer:  Hidden Info conversion complete.");

       //Additional Info
       LogSystem.log(10, "WebRenderer:  Additional Info conversion starting...");
       try {
           tmpStr = "";
           String aiStr="";
           String addString = "";
           if (oaRetVars.addInfo.length() > 0) {
               LogSystem.log(10, "AddInfo = " + oaRetVars.addInfo);
               LogSystem.log(10, "WebRenderer:  oasv.command(addInfo)= " +oasv.command);
               if (oasv.command.equalsIgnoreCase("GetHistory")) {
                   addString = oaRetVars.addInfo;
                   aiStr = "<input type=hidden name=\"reset\" value=\"SetHistory\">";
                   aiStr += this.parseHistory(addString);
               }
               tmpStr = "";
               tmpStr  = this.resTmpLdr.getTemplate(tmpsPath + "template_addinfo.html");
               if (aiStr.equals("")) {
                    tmpStr = regExp.substitute("###ADDINFO###", oaRetVars.addInfo, tmpStr);
               }
               else {
                    tmpStr = regExp.substitute("###ADDINFO###", aiStr, tmpStr);
               }
           } //if addinfo.length > 0
           htmlStr = regExp.substitute("###TEMPLATE_ADDINFO###", tmpStr, htmlStr);
       } //try
       catch (Exception e) {
           LogSystem.log(1, "WebRenderer:  retVars2HTML():  Exception caught in addinfo conversion");
           LogSystem.log(1, e);
           htmlStr = regExp.substitute("###TEMPLATE_LINKS###", "", htmlStr);
       }
       LogSystem.log(10, "WebRenderer:  Additional Info conversion complete.");

       //OA Header
       LogSystem.log(10, "WebRenderer:  OA Header insertion starting...");
       tmpStr = "";
       tmpStr  = this.resTmpLdr.getTemplate(tmpsPath + "template_header.html");
       htmlStr = regExp.substitute("###TEMPLATE_HEADER###", tmpStr, htmlStr);
       LogSystem.log(10, "WebRenderer:  OA Header insertion complete.");

       //OA Footer
       LogSystem.log(10, "WebRenderer:  OA Footer conversion starting...");
       tmpStr = "";
       tmpStr  = this.resTmpLdr.getTemplate(tmpsPath + "template_footer.html");
       htmlStr = regExp.substitute("###TEMPLATE_FOOTER###", tmpStr, htmlStr);
       LogSystem.log(10, "WebRenderer:  OA Footer conversion complete.");

       //OA Other conversions--oaservlet, query_string, base servlet and imagespath
//       htmlStr = regExp.substitute("###oaservlet###", oaServlet, htmlStr);
//       LogSystem.log(10, "WebRenderer:  oaServlet = " + oaServlet);
//       htmlStr = regExp.substitute("###baseservlet###", baseServlet, htmlStr);
//       LogSystem.log(10, "WebRenderer:  baseServlet = " + baseServlet);
//       htmlStr = regExp.substitute("###QUERY_STRING###", queryStr, htmlStr);
//       LogSystem.log(10, "WebRenderer:  queryStr = " + queryStr);
       htmlStr = regExp.substitute("###actionValue###", oaServlet, htmlStr);
       LogSystem.log(10, "WebRenderer:  actionValue = " + oaServlet);
       htmlStr = regExp.substitute("###imagespath###", imagesPath, htmlStr);
       LogSystem.log(10, "WebRenderer:  imagespath = " + imagesPath);


       if  (nodeType.endsWith("A")) {
           LogSystem.log(10, "WebRenderer:  Action conversion complete.");
       }
       else if (nodeType.endsWith("Q")) {
           LogSystem.log(10, "WebRenderer:  Question conversion complete.");
       }
       else {
           LogSystem.log(10, "WebRenderer:  Conversion complete for INVALID node type.");
       }

       LogSystem.log(10, "WebRenderer:  Converting OAReturnVars object to HTML String object completed.");

       return htmlStr;
   } //retVars2HTML(OAReturnVars)



   /**
   * Create the HTML history from the additional info String passed from WebRenderer.
   *
   * @param String history
   *
   * @return String histStr
   **/
   public String parseHistory(String history) {

       LogSystem.log(10, "WebRenderer:  parseHistory starting...");
       RegularExpression regExp = new RegularExpression();
       String histStr = "";
       String tmpStr = "";
       String typeStr = "";
       String histInfoStr="";
       Integer seqNo = new Integer(0);
       if (history.length() > 0) {
          tmpStr = history;
          LogSystem.log(10, "WebRenderer:  tmpStr  = " + tmpStr);
          StringTokenizer strTok = new StringTokenizer(tmpStr, "<");
          LogSystem.log(10, "WebRenderer:  strTok.count()= " +strTok.countTokens());
          while (strTok.hasMoreTokens()) {
              tmpStr = strTok.nextToken();
              LogSystem.log(10, "WR: tmpStr= " + tmpStr);
              typeStr = tmpStr.substring(0, tmpStr.indexOf(">"));
              LogSystem.log(10, "WR: typeStr = " + typeStr);
              histInfoStr = tmpStr.substring(tmpStr.indexOf(">")+1);
              LogSystem.log(10, "WR: histInfoStr = " + histInfoStr);
              histVec.addElement(typeStr + " = " + histInfoStr);
              if (typeStr.startsWith("Question") || typeStr.startsWith("Action")) {
                  LogSystem.log(10, "WR:  seqNo = " +seqNo);
//                  seqNo = new Integer(seqNo.intValue() + 1);
                  LogSystem.log(10, "WR:  seqNo = " +seqNo);
                  histStr += this.resTmpLdr.getTemplate(tmpsPath + "template_addbolditems.html");
                  histStr = regExp.substitute("###SEQ###", seqNo.toString(), histStr);
                  if (typeStr.startsWith("Action")) {
                      histStr = regExp.substitute("###TYPE###", typeStr, histStr);
                  }
                  else {
                      histStr = regExp.substitute("###TYPE###", typeStr.substring(0,1), histStr);
                  }
                  histStr = regExp.substitute("###AITEM###", histInfoStr, histStr);
                  seqNo = new Integer(seqNo.intValue() + 1);
                  LogSystem.log(10, "WR:  seqNo + 1 = " +seqNo);

              }
              else {
                  histStr += this.resTmpLdr.getTemplate(tmpsPath + "template_addinfoitems.html");
                  histStr = regExp.substitute("###SEQ###", seqNo.toString(), histStr);
                  histStr = regExp.substitute("###TYPE###", typeStr.substring(0,1), histStr);
                  histStr = regExp.substitute("###AITEM###", histInfoStr, histStr);
              }
         } //while

       }
       LogSystem.log(15, "WebRenderer: Current parsed hitory is= \n" + histStr);
       LogSystem.log(10, "WebRenderer:  parseHistory complete.");

       return histStr;
   }//parseHistory(String)

} // class WebRenderer
