package com.ibm.nzna.projects.qQuestStats ;   
/*-----------------------------------------------------------*/
/*   QQuestStats                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QQuestStats                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This program provides Quick Quest Statistics (QQuestStats). It 
 * stores the results for the statistics in the TIGRIS.CONSTANTS
 * table under the following values:
 *
 * <pre>
 * Constant (in TIGRIS.CONSTANTS)        StrConst (in TIGRIS.CONSTANTS)   
 * ------------------------------------  -------------------------------  
 * QQ_STAT_DATE                          01/01/1999 (The last time this program ran)
 * QQ_DOC_PUB_THIS_YEAR                  12345      (Documents Published this year)
 * QQ_DOC_PUB_THIS_MONTH                 12345      (Documents Published this month) 
 * QQ_DOC_PUB_THIS_WEEK                  12345      (Documents Published this week)
 * QQ_DOC_PUB_THIS_DAY                   12345      (Documents Published today)
 * QQ_PROD_PUB_THIS_YEAR                 12345      (Products Published this year)
 * QQ_PROD_PUB_THIS_MONTH                12345      (Products Published this month) 
 * QQ_PROD_PUB_THIS_WEEK                 12345      (Products Published this week)
 * QQ_PROD_PUB_THIS_DAY                  12345      (Products Published today)
 * QQ_TOTAL_DOCS                         12345      (Total number of documents)
 * QQ_TOTAL_PRODUCTS                     12346      (Total number of Products)
 * QQ_TOTAL_FAMILIES                     12345      (Total number of Families)
 * QQ_TOTAL_FILES                        12346      (Total number of File Attachments)
 * </pre>
 *
 * You can pass this program the variables for Database Server, Database name and
 * port number. If port number is omitted then the program will try to use the local
 * DB/2 Client.<br><br>
 *
 * The default parameters are as follows:<pre>
 * 
 * Database Name:   quest
 * Database Server: localhost
 * Database Port: None
 * 
 *
 * @version 1.0, 5/31/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QQuestStats                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class QQuestStats extends Object {

   /**
     *  This is the database name
     **/
   private String databaseName = "quest" ;

   /**
     *  This is the database port number
     **/
   private String databasePort = null ;

   /**
     *  This is the database server name
     **/
   private String databaseServer = "localhost" ;

   /**
     *  The main start point of the program. The program currently
     * supports the following command line parameters (Not case sensitive):<br><pre>
     *
     * Param      Description          Usage
     * ---------- -------------------- -------------------------------
     * /S         Server               /S:webdb01.pc.ibm.com
     * /D         Database Name        /D:quest408
     * /P         JDBC Port Number     /P:5703
     * </pre>
     **/
   public static void main ( String args[] ) {
      new QQuestStats ( args ) ;
   }


   /**
     *  Constructor for the program. This constructor initializes
     * everything and then runs the code. This constructor will not
     * return until all code has completed
     *
     * @param args The command line args passed into the program
     **/
   public QQuestStats ( String[] args ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our Log System                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      new LogSystem ( 1, "QQuestStats.out" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Process the command line params                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      processParams ( args )  ;              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Connect to the database                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( connectToDatabase() ) {                  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read and populate the stats                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setStatDate() ;                           


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read and populate the Document Stats                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setDocumentStats() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read the Product Statistics                             */                  
         /*                                                           */
         /*-----------------------------------------------------------*/
         setProductStats() ;
         


      }

      System.exit( 0 ) ;
   }




   /**
     *  This method will read in the command line parameters. It will
     * return FALSE if not command line params were passed
     *
     * @param args The command line params
     *
     * @return boolean
     **/
   private boolean processParams ( String[] args ) {
      boolean rc = false ;

      if ( args != null ) {
         int i = 0 ;

         while ( i < args.length ) {
            if ( args[i] != null ) {
               if ( args[i].toUpperCase().startsWith ( "/S:" ) )
                  databaseServer = args[i].substring ( 3 ) ;
               else if ( args[i].toUpperCase().startsWith ( "/D:" ) )
                  databaseName = args[i].substring ( 3 ) ;
               else if ( args[i].toUpperCase().startsWith ( "/P:" ) )
                  databasePort = args[i].substring ( 3 ) ;
            }
            i++ ;
         }
      }

      LogSystem.log ( 1, "Command Line Paremeters Read. Program Variables:" ) ;
      LogSystem.log ( 1, "Database Name  :"+databaseName ) ;
      LogSystem.log ( 1, "Database Server:"+databaseServer ) ;
      LogSystem.log ( 1, "Database Port  :"+databasePort ) ;
      LogSystem.log ( 1, "\n\n" ) ;


      return( rc ) ;
   }


   /**
     *  This method will try to connect to the database.
     *
     * @return FALSE if a connection could not be made, TRUE if a connetion was made
     **/
   private boolean connectToDatabase() {
      boolean rc = false ;

      try {                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our DatabaseSystem Instance                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         new DatabaseSystem ( true,
                              LogSystem.getInstance(),
                              databasePort == null ) ;              // Use the app driver if we don't have a port number


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Connect to the database                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rc = DatabaseSystem.createConnection ( DatabaseSystem.QUEST,
                                                databaseName,
                                                databaseServer,
                                                databasePort,
                                                "quest",
                                                "j0hnny" ) ;

         if ( rc )
            LogSystem.log ( 2, "Connection Established" ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

      return( rc ) ;
   }


   /**
     *  This method will delete the current constant and create a new
     * one for the statistic passed in
     *
     * @param statName The name of the statistic that is put into the TIGRIS.CONSTANTS - CONSTANT column
     * @param stat     The value of the statistic that is put into thte TIGRIS.CONSTANTS - STRCONST column
     **/
   private void updateStat ( String statName, String stat ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "updateStat", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "DELETE FROM TIGRIS.CONSTANTS WHERE CONSTANT = '"+statName+"'" ) ;
         stmt.executeUpdate ( "INSERT INTO TIGRIS.CONSTANTS VALUES "+
                              "( '"+statName+"', "+
                              "  0, "+
                              "  '"+stat+"', "+
                              "  'QQuestStats', "+
                              "  'QUEST', "+
                              "  CURRENT TIMESTAMP)" ) ;
         stmt.close() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;
   }


   /**
     *  This method will set the constant 'QQ_STAT_DATE' in the TIGRIS.CONSTANTS
     * table to the date on the DB/2 Server
     **/
   private void setStatDate () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setStatDate", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "DELETE FROM TIGRIS.CONSTANTS WHERE CONSTANT = 'QQ_STAT_DATE'" ) ;
         stmt.executeUpdate ( "INSERT INTO TIGRIS.CONSTANTS VALUES "+
                              "( 'QQ_STAT_DATE', "+
                              "  0, "+
                              "  CURRENT DATE, "+
                              "  'QQuestStats', "+
                              "  'QUEST', "+
                              "  CURRENT TIMESTAMP)" ) ;
         stmt.close() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;
   }


   /**
     *  This method will retrieve all document counts and stores the counts
     * in the corresponding constant values. For definition of the constant
     * values please see above.
     *
     **/
   private void setDocumentStats( ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setDocumentStats", 5 ) ;

      try {
         CDate     cDate                  = new CDate () ;
         Statement stmt                   = sqlMethod.createStatement() ;
         int       year                   = cDate.getYear() ;
         int       month                  = cDate.getMonth() ;
         int       dayOfMonth             = cDate.getDayOfMonth() ;
         int       beginningOfWeek        = dayOfMonth-cDate.getDayOfWeek ( cDate.today(), CDate.DATE_DATE ) ;
         ResultSet rs                     = null ;
         int       docsPublishedThisYear  = 0 ;
         int       docsPublishedThisMonth = 0 ;
         int       docsPublishedThisWeek  = 0 ;                            
         int       docsPublishedThisDay   = 0 ;                            
         int       totalDocs              = 0 ;
         int       totalFiles             = 0 ;

         if ( beginningOfWeek == 0 )
            beginningOfWeek = 1; 

         LogSystem.beginLogSection ( "Document Statistics" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Documents published this year                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM TIGRIS.DOCUMENTS "+
                                  "WHERE CHANGEDTIME > '"+year+"-01-01-01.00.00.000000' AND "+
                                  "      RECYCLED = 'N' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            docsPublishedThisYear = rs.getInt(1) ;

         LogSystem.log  ( 1, "docsPublishedThisYear:"+docsPublishedThisYear ) ;
         rs.close() ;                            


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Documents published this month                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM TIGRIS.DOCUMENTS "+
                                  "WHERE CHANGEDTIME > '"+year+"-"+CDate.format2Digit (month)+"-01-01.00.00.000000' AND "+
                                  "      RECYCLED = 'N' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            docsPublishedThisMonth = rs.getInt(1) ;

         LogSystem.log  ( 1, "docsPublishedThisMonth:"+docsPublishedThisMonth ) ;                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Documents published this week                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM TIGRIS.DOCUMENTS "+
                                  "WHERE CHANGEDTIME > '"+year+"-"+CDate.format2Digit (month)+"-"+beginningOfWeek+"-01.00.00.000000' AND "+
                                  "      RECYCLED = 'N' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            docsPublishedThisWeek = rs.getInt(1) ;
         LogSystem.log  ( 1, "docsPublishedThisWeek:"+docsPublishedThisWeek ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Documents published Today                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM TIGRIS.DOCUMENTS "+
                                  "WHERE CHANGEDTIME > '"+year+"-"+CDate.format2Digit (month)+"-"+dayOfMonth+"-01.00.00.000000' AND "+
                                  "      RECYCLED = 'N' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            docsPublishedThisDay = rs.getInt(1) ;
         LogSystem.log  ( 1, "docsPublishedThisDay:"+docsPublishedThisDay ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Total number of docs                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM TIGRIS.DOCUMENTS "+
                                  "WHERE RECYCLED = 'N' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            totalDocs = rs.getInt(1) ;
         LogSystem.log  ( 1, "Total Documents:"+totalDocs ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Total number of File Attachments                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM TIGRIS.DOCUMENTS A, TIGRIS.DOCMEDIA B "+
                                  "WHERE A.DOCIND = B.DOCIND AND "+
                                  "      RECYCLED = 'N' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            totalFiles = rs.getInt(1) ;
         LogSystem.log  ( 1, "Total Files:"+totalFiles ) ;

         updateStat ( "QQ_DOC_PUB_THIS_YEAR",  ""+docsPublishedThisYear ) ;
         updateStat ( "QQ_DOC_PUB_THIS_MONTH", ""+docsPublishedThisMonth ) ;
         updateStat ( "QQ_DOC_PUB_THIS_DAY",   ""+docsPublishedThisDay ) ;
         updateStat ( "QQ_DOC_PUB_THIS_WEEK",  ""+docsPublishedThisWeek ) ;
         updateStat ( "QQ_TOTAL_FILES",        ""+totalFiles ) ;
         updateStat ( "QQ_TOTAL_DOCS",         ""+totalDocs ) ;

         LogSystem.endLogSection() ;
         stmt.close() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;
   }


   /**
     *  This method will retrieve all product counts and stores the counts
     * in the corresponding constant values. For definition of the constant
     * values please see above.
     *
     **/
   private void setProductStats( ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setProductStats", 5 ) ;

      try {
         CDate     cDate                   = new CDate () ;
         Statement stmt                    = sqlMethod.createStatement() ;
         int       year                    = cDate.getYear() ;
         int       month                   = cDate.getMonth() ;
         int       dayOfMonth              = cDate.getDayOfMonth() ;
         int       beginningOfWeek         = dayOfMonth-cDate.getDayOfWeek ( cDate.today(), CDate.DATE_DATE ) ;
         ResultSet rs                      = null ;
         int       prodsPublishedThisYear  = 0 ;
         int       prodsPublishedThisMonth = 0 ;
         int       prodsPublishedThisWeek  = 0 ;                            
         int       prodsPublishedThisDay   = 0 ;                            
         int       totalProducts           = 0 ;
         int       totalFamilies           = 0 ;


         if ( beginningOfWeek == 0 )
            beginningOfWeek = 1; 

         LogSystem.beginLogSection ( "Document Statistics" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Documents published this year                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM PRODUCT.PRODUCT "+
                                  "WHERE LASTTOUCHEDON > '"+year+"-01-01-01.00.00.000000' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            prodsPublishedThisYear = rs.getInt(1) ;

         LogSystem.log  ( 1, "prodsPublishedThisYear:"+prodsPublishedThisYear ) ;
         rs.close() ;                            


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Documents published this month                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM PRODUCT.PRODUCT "+
                                  "WHERE LASTTOUCHEDON > '"+year+"-"+CDate.format2Digit (month)+"-01-01.00.00.000000' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            prodsPublishedThisMonth = rs.getInt(1) ;

         LogSystem.log  ( 1, "prodsPublishedThisMonth:"+prodsPublishedThisMonth ) ;                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Documents published this week                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM PRODUCT.PRODUCT "+
                                  "WHERE LASTTOUCHEDON > '"+year+"-"+CDate.format2Digit (month)+"-"+CDate.format2Digit (beginningOfWeek)+"-01.00.00.000000' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            prodsPublishedThisWeek = rs.getInt(1) ;
         LogSystem.log  ( 1, "prodsPublishedThisWeek:"+prodsPublishedThisWeek ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Documents published Today                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM PRODUCT.PRODUCT "+
                                  "WHERE LASTTOUCHEDON > '"+year+"-"+CDate.format2Digit (month)+"-"+dayOfMonth+"-01.00.00.000000' AND "+
                                  "      PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            prodsPublishedThisDay = rs.getInt(1) ;
         LogSystem.log  ( 1, "prodsPublishedThisDay:"+prodsPublishedThisDay ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Total number of prods                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM PRODUCT.PRODUCT "+
                                  "WHERE PUBLISH  = 'Y'" ) ;
         if ( rs.next() )
            totalProducts = rs.getInt(1) ;
         LogSystem.log  ( 1, "Total Products:"+totalProducts ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the Total number of Families                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT COUNT(*) FROM PRODUCT.TYPECATEGORY WHERE CATEGORYLEVEL = 'family'" ) ;
         
         if ( rs.next() )
            totalFamilies = rs.getInt(1) ;
         LogSystem.log  ( 1, "Total Families:"+totalFamilies ) ;

         updateStat ( "QQ_PROD_PUB_THIS_YEAR",  ""+prodsPublishedThisYear ) ;
         updateStat ( "QQ_PROD_PUB_THIS_MONTH", ""+prodsPublishedThisMonth ) ;
         updateStat ( "QQ_PROD_PUB_THIS_DAY",   ""+prodsPublishedThisDay ) ;
         updateStat ( "QQ_PROD_PUB_THIS_WEEK",  ""+prodsPublishedThisWeek ) ;
         updateStat ( "QQ_TOTAL_FAMILIES",      ""+totalFamilies ) ;
         updateStat ( "QQ_TOTAL_PRODUCTS",      ""+totalProducts ) ;

         LogSystem.endLogSection() ;
         stmt.close() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;
   }


} /* QQuestStats */
