package com.ibm.nzna.projects.qit;
/*-----------------------------------------------------------*/
/*   Quest Input Tool                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Quest Input Tool                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM Corp.                        */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The Beginning of the Quest Input Tool Application. The job
 * of this class is to kick everything off. It will also read and
 * set our command line properties.
 *
 *
 * @version 1.0, 1/7/2000                    
 * @author  David Lentz
 */
import com.ibm.nzna.projects.qit.help.tutor.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.chat.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.help.* ;
import com.ibm.nzna.projects.qit.setup.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   Quest                                                   */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * 
 * @author           dlentz
 * @version          1.0
 */
public class Qit implements AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Statics                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static boolean   appEnabled          = true ;
   private static int       disabled            = 0 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private InitAppWindow initWindow = null ;
   private Str           str        = null ;                       
   private String[]      args       = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Keep an instance of these System                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private BookmarkSystem bookmarkSystem = null ;
   private UserStatSystem userStatSystem = null ;
   private DateSystem     dateSystem     = null ;


   /*-----------------------------------------------------------*/
   /*   main                                                    */
   /*                                                           */
   /*   Description:                                            */
   /*      The Main method called when we are run as an         */
   /*   application                                             */
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public static void main ( String[] args ) {
      System.out.print ( "Quest Initializing\n------------------\n\n" ) ;
      Properties p         = System.getProperties () ;
      String     splatform = p.getProperty ( "os.name" ) ;

      System.out.println ( "Platform:"+splatform ) ;
      new Qit( args ) ;
   }

   public void finalize () {
      try {
         super.finalize() ;
         System.out.println ( "QIT FINALIZED!" ) ;
      } catch ( Throwable e ) {
         System.out.println ( "Exception while finalized" ) ;
      } finally {
      }
   }


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constructor                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public Qit ( String[] args ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Assign our   'Please Wait Initializing' window and vars */
         /*                                                           */
         /*-----------------------------------------------------------*/
         this.args = args ;
         this.initWindow = initWindow ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the Logging System                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         new LogSystem( 0, "quest.debug" ) ;                                              // Create our initial log

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add an alias to the log that will be written to as well */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.addAlias ( "quest.debug."+(new CDate (CDate.DATE_STAMP).today() ) ) ;           
         LogSystem.log ( 0, "Internal Version Number:"+APPLICATION_VERSION ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Redirect StdErr and StdOut                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( PRODUCTION ) {
            LogSystem.getInstance().setErr() ;
            LogSystem.getInstance().setOut() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the Str Object                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         str = new Str ( "english" ) ;
         LogSystem.log ( 0, "Application String Version:"+Str.getApplicationName()) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now kick off the background thread that initializes */
         /* all sub systems                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         //new Thread ( this ).start() ;
         run() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ask the user to log in, if they do so, continue with    */
         /*  initialization. Otherwise exit.                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         new InitializeThread(this, true) ;

         if ( initWindow != null )
            initWindow.dispose() ;


      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }



   /*-----------------------------------------------------------*/
   /*   readParams                                              */         
   /*                                                           */
   /*   Description:                                            */
   /*      Reads the parameters of the application              */                                                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void readParams ( String[] args ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have any parameters, read all                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( args != null ) && ( args.length > 0 ) ) {
         int length = args.length ;
         int i      = 0 ;

         LogSystem.beginLogSection ( "Reading Command Line Parameters" ) ;              

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set some defaults first                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         PropertySystem.putBool ( PropertySystem.PROPERTY_START_DBWINDOW, false ) ;

         while ( i < length ) {
            processParam ( args[i].toUpperCase(), args[i] ) ;
            i++ ;
         }
         LogSystem.endLogSection ( ) ;
      }
   }



   /*-----------------------------------------------------------*/
   /*   processParam                                            */           
   /*                                                           */
   /*   Description:                                            */
   /*      This method proceses one param                       */                                               
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void processParam ( String param, String mixedCaseParam ) {

      try {

         if ( param.startsWith ( "/G:" ) ) {
            try {
               Integer debugLevel = new Integer ( param.substring ( 3, param.length() ) ) ;

               PropertySystem.putInt ( PropertySystem.PROPERTY_DEBUGLEVEL, debugLevel ) ;

            } catch ( Exception e ) {
               /* Do nothing for Exceptions, Normally the NumberFormatException!  */
               System.out.println ( "Could not parse Debug Level of "+param ) ;
            }
         }

         else if ( param.startsWith ( "/D:" ) )
            PropertySystem.putString ( PropertySystem.PROPERTY_APPDB, param.substring ( 3, param.length() ) ) ;

         else if ( param.startsWith ( "/P:" ) )
            PropertySystem.putString ( PropertySystem.PROPERTY_APPDB_PORT, param.substring ( 3, param.length() ) ) ;

         else if ( param.startsWith ( "/S:" ) )
            PropertySystem.putString ( PropertySystem.PROPERTY_APPDB_SERVER, param.substring ( 3, param.length() ) ) ;

         else if ( param.startsWith ( "/U:" ) ) {
            PropertySystem.putString ( PropertySystem.PROPERTY_USERID, param.substring ( 3, param.length() ) ) ;
         }

         else if ( param.startsWith ( "/PW:" ) )
            PropertySystem.putString ( PropertySystem.PROPERTY_PASSWORD, param.substring ( 4, param.length() ) ) ;

         else if ( param.startsWith ( "/TEST" ) )
            PropertySystem.putBool ( PropertySystem.PROPERTY_TESTMODE, true ) ;

         else if ( param.startsWith ( "/L:" ) )
            PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGE, mixedCaseParam.substring ( 3, mixedCaseParam.length() ) ) ;

         else if ( param.startsWith ( "/NOLOGO" ) )
            PropertySystem.putBool ( PropertySystem.PROPERTY_LOGO, false ) ;

         else if ( param.startsWith ( "/?" ) ) {
            printParams() ;
            System.exit(0) ;
         }

         else if ( param.charAt ( 0 ) == '?' ) {
            printParams() ;
            System.exit(0) ;
         }

         else if ( param.startsWith ( "/APPDRIVER" ) )
            PropertySystem.putBool ( PropertySystem.PROPERTY_APPDRIVER, true ) ;

         else if ( param.startsWith ( "/IMPORTFILE:" ) )
            PropertySystem.putString ( PropertySystem.PROPERTY_IMPORTFILE, param.substring ( "/IMPORTFILE:".length(), param.length() ) ) ;

         else if ( param.startsWith ( "/RUNBATCH:" ) ) {
            PropertySystem.putString ( PropertySystem.PROPERTY_RUNBATCH,  param.substring ( "/RUNBATCH:".length(), param.length() ) ) ;
            PropertySystem.putBool   ( PropertySystem.PROPERTY_BATCHMODE, true ) ;
         }

         else if ( param.startsWith ( "/DBWINDOW" ) )
            PropertySystem.putBool ( PropertySystem.PROPERTY_START_DBWINDOW, true ) ;

         /* If we still don't have the parameter, notify the user */
         else {
            System.out.println ( "Unknown parameter:"+param ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }

   /*-----------------------------------------------------------*/
   /* printParams
    */
   /**
    *    This methos will print out all of our paramters
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void printParams () {
      System.out.println ( "" ) ;
      System.out.println ( "Usage: java Quest <params>" ) ;
      System.out.println ( "Param       Description                                         Usage" ) ;
      System.out.println ( "----------- -------------------------------------               ----------------" ) ;
      System.out.println ( "/D:         Specify a database name                             /D:QUEST " ) ;
      System.out.println ( "/P:         Specify a database port                             /P:5703 " ) ;
      System.out.println ( "/S:         Specify the database Server                         /S:SERVER " ) ;
      System.out.println ( "/U:         Specify the user to log on with                     /U:USER " ) ;
      System.out.println ( "/PW:        Specify the password to log on with                 /P:PASSWORD " ) ;
      System.out.println ( "/G:         Start in debug mode                                 /G:DEBUGLEVEL# " ) ;
      System.out.println ( "/L:         Display in a certain language                       /L:English or /L:Spanish" ) ;
      System.out.println ( "/TEST       Use the test User Id and Password                   /TEST" ) ;
      System.out.println ( "/APPDRIVER  Uses the DB/2 App Driver, instead of the Net Driver /APPDRIVER" ) ;
      System.out.println ( "/RUNBATCH   No GUI, just run a predefined batch import          /RUNBATCH:<batchname>" ) ;
      System.out.println ( "/IMPORTFILE Used when in batch mode to identify import file     /IMPORTFILE:<import filename>" ) ;
      System.out.println ( "/?          Get this help                                       /? " ) ;

   }




   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will set the application Enabled
    *  flag. All programming should be done with this
    *  flag. <br>
    *
    *  Please keep in mind that if you call 'setEnabled ( true );' that
    *  doesn't mean that the application will be enabled. If two methods
    *  call 'setEnabled (false)', then two calls need to be made to 'setEnabled (true)'.
    *  Only one call will not enable the application. It is imperative that each
    *  call to setEnabled ( false ), be followed by a setEnabled ( true ).
    *
    *  @param enable TRUE = enable, FALSE = disable
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setEnabled ( boolean enable ) {
      if ( enable ) {
         disabled-- ;
         if ( disabled == 0 )
            appEnabled = true ;
      }

      else {
         disabled++ ;
         appEnabled = false ;
      }
   }





   /*------------------------------------------------------------*/
   /* getEnabled
    */
   /**
    *    This method will return TRUE or FALSE. If
    *  a FALSE is returned, this means that some method
    *  somewhere has requested the application to be disabled.
    *  This means that no button hit, menu hit, list selection etc..
    *  should respond. PLEASE MAKE SURE YOU CALL THIS METHOD TO DETERMINE THAT
    *
    *
    *  @return Nothing TRUE/FALSE
    *//*---------------------------------------------------------*/
   public static boolean getEnabled () {
      return( appEnabled ) ;
   }


   /*------------------------------------------------------------*/
   /* getFirstApplicationRun
    */
   /**
    *    This method will determine if this is the
    *  first time this application has run on the client
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean getFirstApplicationRun () {
      final boolean rc = PropertySystem.getBool ( PropertySystem.PROPERTY_FIRST_APPLICATION_RUN ) ;

      PropertySystem.putBool ( PropertySystem.PROPERTY_FIRST_APPLICATION_RUN, false ) ;

      return( rc ) ;
   }

   public static String writeSomething () {
      System.out.println ( "Something" ) ; return( "AS" ) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write Something out!                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
   }




   /**
    * This method will execute all of the initialization for the Sub Systems in Quest
    */
   public void run () {
      try {
         File  processFile = new File ( "quest.pid" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create the Property System                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         System.out.println ( "Creating Property System" ) ;
         new PropertySystem() ;                  
         //System.out.print ( "." ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Clean out all old log files                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         System.out.println ( "Cleaning Old Log Files" ) ;
         QuestUtil.cleanOldLogFiles() ;
         //System.out.print ( "." ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read the Arguments                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         System.out.println ( "Reading Params" ) ;
         readParams ( args ) ;                                                               
         //System.out.print ( "." ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create the rest of the sub systems.                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log ( 3, "Creating ImageSystem" ) ;
         System.out.println ( "Creating ImageSystem" ) ;
         new ImageSystem() ;
         //System.out.print ( "." ) ;
         System.out.println ( "Creating GUISystem" ) ;
         LogSystem.log ( 3, "Creating GUISystem" ) ;
         //System.out.print ( "." ) ;
         new GUISystem() ;
         System.out.println ( "Creating CacheSystem" ) ;
         LogSystem.log ( 3, "Creating CacheSystem" ) ;
         new CacheSystem() ;
         //System.out.print ( "." ) ;
         System.out.println ( "Creating DateSystem" ) ;
         LogSystem.log ( 3, "Creating DateSystem" ) ;
         dateSystem = new DateSystem() ;
         //System.out.print ( "." ) ;
         System.out.println ( "Creating ThemeSystem" ) ;
         LogSystem.log ( 3, "Creating ThemeSystem" ) ;
         new ThemeSystem() ;
         //System.out.print ( "." ) ;
         System.out.println ( "Creating FontSystem" ) ;
         LogSystem.log ( 3, "Creating FontSystem" ) ;
         new FontSystem() ;
         //System.out.print ( "." ) ;

         System.out.println ( "Creating HelpSystem" ) ;
         LogSystem.log ( 3, "Creating HelpSystem" ) ;
         new HelpSystem() ;
         //System.out.print ( "." ) ;
         System.out.println ( "Creating UserStatSystem" ) ;
         LogSystem.log ( 3, "Creating UserStatSystem" ) ;
         userStatSystem = new UserStatSystem() ;

         System.out.println ( "Creating CustomViewSystem" ) ;
         LogSystem.log ( 3, "Creating CustomViewSystem" ) ;
         //System.out.print ( "." ) ;
         new CustomViewSystem() ;
         //System.out.print ( "." ) ;

         System.out.println ( "Creating BookmarkSystem" ) ;
         LogSystem.log ( 3, "Creating Bookmarks" ) ;
         bookmarkSystem = new BookmarkSystem ( ) ;
         //System.out.print ( "." ) ;

         System.out.println ( "Creating ProcessManagerSystem" ) ;
         LogSystem.log ( 3, "Creating ProcessManagerSystem" ) ;
         new ProcessManagerSystem() ;
         //System.out.print ( "." ) ;
         //LogSystem.log ( 3, "Creating ChatServer" ) ;
         //new com.ibm.nzna.projects.qit.chat.ChatServer() ;

         System.out.println ( "Creating Spell Checker" ) ;
         LogSystem.log ( 3, "Creating Spell Checker" ) ;
         new com.ibm.nzna.shared.spell.SpellChecker() ;
         //System.out.print ( "." ) ;

         System.out.println ( "Creating Initialize, Clipboard, Database System" ) ;
         LogSystem.log ( 3, "Creating initialize" ) ;
         new AppletSystem().initialize() ;
         new ClipSystem () ;
         //System.out.print ( "." ) ;
         new DatabaseSystem ( (PropertySystem.getInt ( PropertySystem.PROPERTY_DEBUGLEVEL ) > 0 ),
                              LogSystem.getInstance(),
                              PropertySystem.getBool ( PropertySystem.PROPERTY_APPDRIVER ) ) ;          
         //System.out.print ( "." ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Mark any properties                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         PropertySystem.putBool ( PropertySystem.PROPERTY_SHUTDOWNCLEAN, !processFile.exists() ) ;              
         LogSystem.setDebugLevel ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEBUGLEVEL ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our process File                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !processFile.exists() ) {
            processFile = null ;
            FileUtil.saveFile ( "quest.pid", "Running" ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the Window Sub System                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         System.out.println ( "Creating WindowSystem" ) ;
         new WindowSystem() ;                                                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Run the Tutorial if this is the first application run   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( getFirstApplicationRun() ) {
            //new TutorWin ( true, -1 ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write our properties                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         System.out.println ( "Creating Writing Properties" ) ;
         PropertySystem.getInstance().writeToDisk() ;
         //System.out.println ( "" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Severe Error", "Cannot Initialize Quest SubSystems. Application will now exit" ) ;
         System.exit(-1) ;
      }
   }

} /* Quest */
