/*-----------------------------------------------------------*/
/*   QuestUpdater                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestUpdater                        */    
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.io.* ;
import java.net.* ;
import java.util.zip.* ;
import java.util.* ;
/**
 * This is the program that will connect to a web server and download an updated Quest 
 *
 *
 * @version 1.0, 1/14/1999 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestUpdater                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestUpdater extends Frame implements Runnable, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_DOWNLOAD     = "Download" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Label   st_PERCENTAGE    = null ;
   private Label   st_STATUS        = null ;
   private Label   st_TITLE         = null ;
   private Button  pb_START         = null ;
   private Button  pb_RESTARTAPP    = null ;
   private Button  pb_QUIT          = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private        String localFile      = "quest.zip" ;
   private static String urlFile        = "" ;




   /*------------------------------------------------------------*/
   /* main
    */
   /**
    *    This is the main entry point for the application
    *  The command line parameters taken is
    *  the URL of the code update.
    *
    *  @param args Command line args
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void main ( String[] args ) {

      if ( ( args == null ) || ( args.length != 1 ) )
         displayUsage() ;
      else {
         urlFile = args[0] ;
         new QuestUpdater() ;
      }
   }




   /*------------------------------------------------------------*/
   /* displayUsage
    */
   /**
    *    Shows how to call the application
    *  from the command line
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void displayUsage () {
      System.out.println ( "Usage:" ) ;
      System.out.println ( "java QuestUpdater urlOfFile" ) ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will construct our frame and
    *  GUI system
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestUpdater () {
      super ( "Quest Auto-Update" ) ;
      createControls() ;
      setBackground ( new Color ( 204, 204, 204 ) ) ;
      setBounds ( 40, 40, 300, 200 ) ;
      setVisible ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and attach all of our
    *  GUI Controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createControls () {
      st_TITLE       = new Label ( "Quest Auto-Update" ) ;
      st_STATUS      = new Label ( "" ) ;
      st_PERCENTAGE  = new Label ( "", Label.CENTER ) ;
      pb_START       = new Button ( "Start" ) ;
      pb_RESTARTAPP  = new Button ( "Run Quest" ) ;
      pb_QUIT        = new Button ( "Quit" ) ;

      st_TITLE.setBackground ( new Color ( 204, 204, 204 ) ) ;
      st_STATUS.setBackground ( new Color ( 204, 204, 204 ) ) ;
      st_PERCENTAGE.setBackground ( new Color ( 204, 204, 204 ) ) ;

      add ( st_TITLE ) ;
      add ( st_STATUS ) ;
      add ( st_PERCENTAGE ) ;
      add ( pb_START ) ;
      add ( pb_RESTARTAPP ) ;
      add ( pb_QUIT ) ;

      /* Adjust some properties */
      st_TITLE.setFont ( new Font ( "SansSerif", Font.BOLD, 16 ) ) ;
      st_STATUS.setForeground ( Color.blue ) ;
      st_PERCENTAGE.setFont ( new Font ( "SansSerif", Font.BOLD, 12 ) ) ;
      pb_START.setFont ( st_PERCENTAGE.getFont() ) ;
      pb_RESTARTAPP.setFont ( st_PERCENTAGE.getFont() ) ;
      pb_QUIT.setFont ( st_PERCENTAGE.getFont() ) ;
      pb_RESTARTAPP.setVisible ( false ) ;


      /* Add our listeners */
      pb_START.addActionListener ( this ) ;
      pb_QUIT.addActionListener ( this ) ;
      pb_RESTARTAPP.addActionListener ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This mehtod will be called when oru controls
    *  are needed to position and size themselve
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;
      int       x    = 5 ;
      int       y    = 18 ;

      st_TITLE.setBounds ( x, y, size.width-(x*2), 40 ) ;
      y+=40 ;

      st_STATUS.setBounds ( x, y, size.width-(x*2), 22 ) ;
      y+=22 ;                              

      st_PERCENTAGE.setBounds ( x, y, size.width-(x*2), 22 ) ;
      y+=22 ;

      pb_START.setBounds ( 5, size.height-30, 70, 25 ) ;
      pb_RESTARTAPP.setBounds ( pb_START.getBounds() ) ;
      pb_QUIT.setBounds ( 80, size.height-30, 70 ,25 ) ;
   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action 
    *  event needs to be processed
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_START )
         new Thread ( this, THREAD_DOWNLOAD ).start() ;

      else if ( e.getSource() == pb_RESTARTAPP ) {
         try {
            Properties p         = System.getProperties () ;
            String     splatform = p.getProperty ( "os.name" ) ;
            Runtime    rt   = Runtime.getRuntime() ;
            String[]   cmds = new String[1] ;

            if ( splatform.indexOf ( "indow" ) != -1 )
               cmds[0] = "quest.bat" ;
            else if (splatform.indexOf ( "OS/2" ) != -1 ) {
               cmds = new String[2] ;
               cmds[0] = "cmd.exe" ;
               cmds[1] = "/C \"quest.cmd\"" ;
            } else if (splatform.indexOf ( "AIX" ) != -1 )
               cmds[0] = "quest" ;

            setVisible ( false ) ;
            rt.exec ( cmds ) ;

            Thread.sleep ( 300 ) ;

            System.exit(0) ;
         } catch ( Exception ex ) {
            st_STATUS.setText ( "Cannot run Quest" ) ;
            ex.printStackTrace() ;
         }
      }

      else if ( e.getSource() == pb_QUIT )
         System.exit(0) ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our download process
    *  in a seperate thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName() ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_DOWNLOAD                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( name.equals ( THREAD_DOWNLOAD ) ) {
         boolean          cont           = false ;

         try {
            File             file           = null ;
            FileOutputStream out            = null ;
            InputStream      in             = null ;
            int              readByteLength = 1024 ; // The number of byte to read at a time
            int              fileLength     = 0 ;
            URL              url            = null ;

            /* First, Disable all controls */
            pb_START.setEnabled ( false ) ;
            pb_RESTARTAPP.setEnabled ( false ) ;            
            pb_QUIT.setEnabled ( false ) ;            


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Open the URL Connection                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            try {
               URLConnection con = null ;

               /* Assume we will fail */
               cont = false ;

               st_STATUS.setText ( "Connecting to Web Site" ) ;
               url = new URL ( urlFile ) ;
               con = url.openConnection() ;

               fileLength = con.getContentLength(); 

               in = con.getInputStream() ;
               cont = true ;
            } catch ( MalformedURLException e ) {
               st_STATUS.setText ( "Cannot locate the URL Specified for download" ) ;
               pb_QUIT.setEnabled ( true ) ;
            } catch ( UnknownHostException e ) {
               st_STATUS.setText ( "Cannot connect to the host specified in the URL" ) ;
               pb_QUIT.setEnabled ( true ) ;
            } catch ( IOException e ) {
               st_STATUS.setText ( "Cannot open socket to URL" ) ;
               pb_QUIT.setEnabled ( true ) ;
            } catch ( Exception e ) {
               System.out.println ( "Exception caught in opening URL:" ) ;
               e.printStackTrace() ;
               pb_QUIT.setEnabled ( true ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Open the local file                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            try {

               /* Assume we will fail */
               cont = false ;

               st_STATUS.setText ( "Creating local file" ) ;

               file = new File ( localFile ) ;
               out = new FileOutputStream ( file ) ;
               cont = true ;
            } catch ( Exception e ) {
               st_STATUS.setText ( "Cannot open local file to download to" ) ;
               e.printStackTrace() ;
               pb_QUIT.setEnabled ( true ) ;
            }



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Download the file                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            try {
               byte[] inByteChunk = new byte [ readByteLength ] ;
               int    bRead       = 1 ;
               int    totalRead   = 0 ;
               int    updateIndex = 0 ;

               /* Assume we will fail */
               cont = false ;

               st_STATUS.setText ( "Downloading "+fileLength ) ;

               while ( bRead > 0 ) {
                  bRead = in.read ( inByteChunk ) ;

                  if ( bRead > 0 ) {
                     totalRead+=bRead ;
                     st_PERCENTAGE.setText ( "%"+( ((int)( ( (double)totalRead/(double)fileLength ) *100 ) ) ) ) ; //+"b of "+fileLength+"b" ) ;
                     out.write ( inByteChunk, 0, bRead ) ;
                  }
               }

               st_PERCENTAGE.setText("" ) ;

               cont = true ;
            } catch ( IOException ex ) {
               st_STATUS.setText ( "Cannot download. Your disk might be full!" ) ;
               ex.printStackTrace() ;
               pb_QUIT.setEnabled ( true ) ;
            } catch ( Exception e ) {
               System.out.println ( "Exception caught while downloading:" ) ;
               e.printStackTrace() ;
               pb_QUIT.setEnabled ( true ) ;
            }



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Unzip the file                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( cont )
               cont = unzipFile() ;

         } catch ( Exception e ) {
            System.out.println ( "Download Thread caught exception:" ) ;
            e.printStackTrace() ;
            pb_QUIT.setEnabled ( true ) ;
         }


         /* Re-enable our controls */
         if ( cont ) {
            pb_START.setVisible ( false ) ;
            pb_RESTARTAPP.setVisible ( true ) ;
            st_TITLE.setText ( "Update Complete!" ) ;
         } else
            st_TITLE.setText ( "Update Failed" ) ;

         pb_START.setEnabled ( true ) ;
         pb_RESTARTAPP.setEnabled ( true ) ;
         pb_QUIT.setEnabled ( true ) ;

      } /* THREAD_DOWNLOAD */


   }



   /*------------------------------------------------------------*/
   /* unzipFile
    */
   /**
    *    This method will unzip the zip file downloaded
    *  by the downloadFile method.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything went OK
    *//*---------------------------------------------------------*/
   private boolean unzipFile ( ) {
      int         argc          = 0;
      String      zipFilename   = localFile ;
      boolean     rc            = true ;
      Properties  prop          = System.getProperties();
      String      fileSeparator = File.separator ;
      String      outDir        = prop.getProperty("user.dir");
      String      filePath      = null ;
      File        file          = null ;
      File        root          = null ;
      ZipFile     zin           = null ;
      Enumeration enum          = null ;
      byte[]      buffer        = null ;
      String      currDir       = null ;
      ZipEntry    ze            = null ;
      String      name          = null ;
      File        dir           = null ;

      st_STATUS.setText ("Extracting" ) ;
      try {

         if ( outDir.endsWith(fileSeparator) )
            outDir = outDir.substring(0, outDir.length() - 1);

         // read the .zip file
         zin = new ZipFile(zipFilename);
         root = new File(outDir);

         if ( !root.exists() )
            root.mkdirs();

         enum = zin.entries();
         buffer = new byte[1024];
         currDir = outDir;

         while ( ( enum.hasMoreElements() ) && ( rc ) ) {

            ze = (ZipEntry)enum.nextElement();

            if ( ze == null )
               break;

            name = ze.getName();

            /* Replace the / characters with the true separator character */
            if ( File.separatorChar != '/' )
               name = name.replace('/', File.separatorChar);

            if ( ze.isDirectory() ) {

               /* Chop off the trailing file separator if needed */

               if ( name.endsWith(fileSeparator) )
                  name = name.substring(0, name.length() - 1);

               dir = new File ( outDir+fileSeparator+name ) ;
               if ( !dir.exists() )
                  dir.mkdirs() ;

               currDir = outDir + fileSeparator + name;
               continue;
            } else {

               filePath = outDir + fileSeparator + name;
               file = new File(filePath);
               dir  = new File(file.getParent()) ;

               /* We must do this check again because the directory
                  that this file is in might not have been in the zip file
                  yet */
               if ( !dir.exists() ) {
                  dir.mkdirs () ;
               }


               try {

                  InputStream in = zin.getInputStream(ze);
                  FileOutputStream out = new FileOutputStream(filePath);
                  for ( ; ; ) {

                     int len = in.read(buffer);
                     if ( len < 0 ) break;
                     out.write(buffer, 0, len);
                  }

                  out.close();
               } catch ( Exception e ) {
                  String error = e.getMessage() ;

                  /* If this is the end of zlib, ignore it */
                  if ( error.indexOf ( "end of ZLIB" ) == -1 ) {

                     /* Ok, if they ran out of disk space, we will get a write error. If
                        we get this exception, put up a pretty message to the user */
                     if ( error.indexOf ( "write error" ) != -1 ) {
                        st_STATUS.setText ( "Error unzipping. Out of disk space" ) ;
                        e.printStackTrace() ;
                        pb_QUIT.setEnabled ( true ) ;
                     }

                     /* Otherwise, throw the exception */
                     else {
                        st_STATUS.setText ( "Error unzipping file:"+file.getName() ) ;
                        e.printStackTrace() ;
                        pb_QUIT.setEnabled ( true ) ;
                     }
                  } else
                     rc = true ;

                  pb_QUIT.setEnabled ( true ) ;
               }
            }

         }

         zin.close();
      } catch ( Exception e ) {
         st_STATUS.setText ( "Error unzipping" ) ;
         e.printStackTrace() ;
         pb_QUIT.setEnabled ( true ) ;
         rc = false ;
      }

      st_STATUS.setText ( "" ) ;

      return( rc ) ;
   }


} /* QuestUpdater */
