package com.ibm.nzna.projects.qit.admin.brandmgmt ;       
/*-----------------------------------------------------------*/
/*   BrandMgmtPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BrandMgmtPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.admin.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
/**
 * This panel allows the user to edit the following information
 * for a brand:<br><br>
 *
 * Add/Edit the BPM (Brand Program Manager) for the brand<br>
 * Add/edit/delete/sort Families<br>
 * Edit/Sort Machines/SubCategories<br>
 * Add/Delete/Sort Document Categories<br>
 *
 *
 * @version 1.0, 6/27/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BrandMgmtPanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class BrandMgmtPanel extends JPanel implements PrintPanel, PreviewPanel, Authority, UserEntryPanelListener, DocConst, ListSelectionListener, QuestPanel, AppConst, AdminConst, ActionListener, ProductConst {

   /**
     *  This is our parent window which is set in the start method
     **/
   private AppDefaultWin parentDefWin = null ;

   /**
     *  This button adds a Category
     **/
   private HotLinkLabel addCategory = createActionHotLink ( Str.getStr ( STR_ADD ) ) ;

   /**
     *  This button removes a Category
     **/
   private HotLinkLabel removeCategory = createActionHotLink ( Str.getStr ( STR_REMOVE ) ) ;

   /**
     *  This button moves a category up
     **/
   private HotLinkLabel upCategory = createActionHotLink ( Str.getStr ( STR_UP ) ) ;

   /**
     *  This button moves a category down
     **/
   private HotLinkLabel downCategory = createActionHotLink ( Str.getStr ( STR_DOWN ) ) ;


   /**
     *  This is the action panel for Family which will
     * hold the action buttons underneath the list
     **/
   private ButtonPanel familyAction = null ;

   /**
     * This is the action panel for machines which will
     * hold the action buttons underneath the list
     **/
   private ButtonPanel machineAction = null ;

   /**
     *  This is the action panel for category which will
     * hold the action buttons underneath the list
     **/
   private ButtonPanel categoryAction = null ;

   /**
     *  this button will add a family
     **/
   private HotLinkLabel addFamily = createActionHotLink ( Str.getStr ( STR_ADD ) ) ;

   /**
     *  this button will edit a family
     **/
   private HotLinkLabel editFamily = createActionHotLink ( Str.getStr ( STR_EDIT ) ) ;

   /**
     *  this button will delete a family
     **/
   private HotLinkLabel removeFamily = createActionHotLink ( Str.getStr ( STR_DELETE ) ) ;

   /**
     *  this button will move a family up
     **/
   private HotLinkLabel upFamily = createActionHotLink ( Str.getStr ( STR_UP ) ) ;

   /**
     *  this button will move a family down
     **/
   private HotLinkLabel downFamily = createActionHotLink ( Str.getStr ( STR_DOWN ) ) ;

   /**
     *  this button will move a machine/Sub Category up
     **/
   private HotLinkLabel upMachine = createActionHotLink ( Str.getStr ( STR_UP ) ) ;

   /**
     *  this button will move a machine down
     **/
   private HotLinkLabel downMachine = createActionHotLink ( Str.getStr ( STR_DOWN ) ) ;

   /**
     *  this button will edit a machine name
     **/
   private HotLinkLabel editMachine = createActionHotLink ( Str.getStr ( STR_EDIT ) ) ;



   /**
     *  This is the list model used by the families
     **/
   private DefaultListModel familyListModel = new DefaultListModel() ;

   /**
     *  This is the list model used by the categories
     **/
   private DefaultListModel categoryListModel = new DefaultListModel() ;

   /**
     *  This is the list model used by the machines
     **/
   private DefaultListModel machineListModel = new DefaultListModel() ;



   /**
     *  This is the brand title that will show
     **/
   private JLabel st_TITLE       = null ;

   /**
     *  This is the Brand Program Manager Field
     **/
   private ValueLabel vl_BPM     = null ;

   /**
     *  This is the entry field for the Brand Program Manager
     **/
   private UserEntryPanel ef_BPM = null ;

   /**
     *  This is the check box for auto-sorting Families
     **/
   private DCheckBox ck_AUTO_SORT_FAMILY = null ;

   /**
     *  This is the check box for auto-sorting Machines/Sub Categories
     **/
   private DCheckBox ck_AUTO_SORT_MACHINE = null ;

   /**
     *  This is the CHeck box for auto-sorting Categogries
     **/
   private DCheckBox ck_AUTO_SORT_CATEGORY = null ;

   /**
     *  This is the title for the Family
     **/
   private JLabel st_FAMILY = null ;

   /**
     *  This is the title for the machine/subcategory
     **/
   private JLabel st_MACHINE = null ;

   /**
     *  This is the close action button
     **/
   private ActionButton pb_CLOSE  = null ;

   /**
     *  This is the List box for the families
     **/
   private JList lb_FAMILY = null ;

   /**
     *  This is the list box for the machines/Sub Categories
     **/
   private JList lb_MACHINE = null ;


   /**
     *  This is the list box for the Categories
     **/
   private JList lb_CATEGORY = null ;

   /**
     *  This is the JScrollPane for the families
     **/
   private JScrollPane scr_FAMILY = null ;

   /**
     *  This is the JScrollPane for the machines/Sub Categories
     **/
   private JScrollPane scr_MACHINE = null ;


   /**
     *  This is the JScrollPane for the Categories
     **/
   private JScrollPane scr_CATEGORY = null ;


   /**
     *  This is the title for the category
     **/
   private JLabel st_CATEGORY = null ;


   /**
     *  This is the brand object that we are editing
     **/
   private TypeCategoryRec brandRec = null ;

   /**
     * This is the JScrollPane which will hold all of our components
     **/
   private JScrollPane scrollPane = null ;

   /**
     * This is the JPanel which will be included in the JScrollPane
     * which holds all of our components
     **/
   private JPanel allComponentPanel = new JPanel () ;


   /**
     *  This constructor will simply assign local vars and return.
     *
     * @brandRec The TypeCategoryRec object representing the brand to edit
     **/
   public BrandMgmtPanel ( TypeCategoryRec brandRec ) {
      this.brandRec = brandRec ;
   }


   /**
     *  This method will initialize and add all of our GUI
     *
     * @return TRUE
     **/
   public boolean initialize () {
      boolean isOption = Brands.isBrandOption ( brandRec ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE    = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE ) ) ;
      st_TITLE    = new JLabel ( brandRec.toString() ) ;
      ef_BPM      = new UserEntryPanel() ;
      vl_BPM      = new ValueLabel ( Str.getStr ( STR_BRAND_PROGRAM_MANAGER ), ef_BPM ) ;        
      st_FAMILY   = new JLabel ( Str.getStr ( STR_FAMILY ) );
      st_MACHINE  = new JLabel ( isOption ? Str.getStr ( STR_SUBCATEGORY ) : Str.getStr ( STR_MACHINE ) ) ;
      st_CATEGORY = new JLabel ( Str.getStr ( STR_CATEGORY ) ) ;
      ck_AUTO_SORT_FAMILY   = new DCheckBox ( Str.getStr ( STR_SORT_ALPHABETICAL ) ) ;
      ck_AUTO_SORT_MACHINE  = new DCheckBox ( Str.getStr ( STR_SORT_ALPHABETICAL ) ) ;
      ck_AUTO_SORT_CATEGORY = new DCheckBox ( Str.getStr ( STR_SORT_ALPHABETICAL ) ) ;
      lb_FAMILY             = new JList () ;
      lb_MACHINE            = new JList () ;
      lb_CATEGORY           = new JList () ;
      scr_FAMILY            = new JScrollPane ( lb_FAMILY ) ;
      scr_MACHINE           = new JScrollPane ( lb_MACHINE ) ;
      scr_CATEGORY          = new JScrollPane ( lb_CATEGORY ) ;
      familyAction          = new ButtonPanel () ;
      machineAction         = new ButtonPanel () ;
      categoryAction        = new ButtonPanel () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      vl_BPM.setBackground ( Color.white ) ;
      vl_BPM.setDescriptionWidth ( 100 ) ;
      setBackground ( Color.white ) ;
      st_TITLE.setFont ( FontSystem.largeTitleFont ) ;  
      ck_AUTO_SORT_FAMILY.setBackground ( Color.white ) ;
      ck_AUTO_SORT_MACHINE.setBackground ( Color.white ) ;
      ck_AUTO_SORT_CATEGORY.setBackground ( Color.white ) ;    
      familyAction.setBackground ( Color.white ) ;
      categoryAction.setBackground ( Color.white ) ;
      machineAction.setBackground ( Color.white ) ;
      lb_FAMILY.setModel ( familyListModel ) ;
      lb_MACHINE.setModel ( machineListModel ) ;
      lb_CATEGORY.setModel ( categoryListModel ) ;
      allComponentPanel.setBackground ( Color.white ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_BPM.addUserEntryPanelListener ( this ) ;
      pb_CLOSE.addActionListener ( this ) ;
      ck_AUTO_SORT_FAMILY.addActionListener ( this ) ;
      ck_AUTO_SORT_MACHINE.addActionListener ( this ) ;
      ck_AUTO_SORT_CATEGORY.addActionListener ( this ) ;
      lb_FAMILY.addListSelectionListener ( this ) ;
      lb_MACHINE.addListSelectionListener ( this ) ;
      lb_CATEGORY.addListSelectionListener ( this ) ;
      upMachine.addActionListener ( this ) ;
      downMachine.addActionListener ( this ) ;
      editMachine.addActionListener ( this ) ;
      upFamily.addActionListener ( this ) ;
      downFamily.addActionListener ( this ) ;
      addFamily.addActionListener ( this ) ;
      removeFamily.addActionListener ( this ) ;
      editFamily.addActionListener ( this ) ;
      upCategory.addActionListener ( this ) ;
      downCategory.addActionListener ( this ) ;
      addCategory.addActionListener ( this ) ;
      removeCategory.addActionListener ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( UserSystem.hasAuthority ( AUTH_FAMILY_KEYWORD ) ) {
         familyAction.add ( addFamily ) ;
         familyAction.add ( editFamily ) ;
         familyAction.add ( removeFamily ) ;
         familyAction.add ( upFamily ) ;
         familyAction.add ( downFamily ) ;
      }

      categoryAction.add ( addCategory ) ;
      categoryAction.add ( removeCategory ) ;
      categoryAction.add ( upCategory ) ;
      categoryAction.add ( downCategory ) ;

      if ( UserSystem.hasAuthority (AUTH_MACHINE_KEYWORD) ) {
         machineAction.add ( editMachine ) ;
         machineAction.add ( upMachine ) ;
         machineAction.add ( downMachine ) ;
      }

      allComponentPanel.setLayout ( null ) ;
      allComponentPanel.add ( st_TITLE ) ;
      allComponentPanel.add ( vl_BPM ) ;
      allComponentPanel.add ( st_FAMILY ) ;
      allComponentPanel.add ( st_CATEGORY ) ;
      allComponentPanel.add ( st_MACHINE ) ;
      if ( UserSystem.hasAuthority ( AUTH_FAMILY_KEYWORD ) )
         allComponentPanel.add ( ck_AUTO_SORT_FAMILY ) ;
      allComponentPanel.add ( scr_FAMILY ) ;
      allComponentPanel.add ( familyAction ) ;
      allComponentPanel.add ( ck_AUTO_SORT_CATEGORY ) ;
      allComponentPanel.add ( scr_CATEGORY ) ;
      allComponentPanel.add ( categoryAction ) ;
      if ( UserSystem.hasAuthority ( AUTH_MACHINE_KEYWORD ) )
         allComponentPanel.add ( ck_AUTO_SORT_MACHINE ) ;
      allComponentPanel.add ( scr_MACHINE ) ;
      allComponentPanel.add ( machineAction ) ;
      allComponentPanel.setMinimumSize   ( new Dimension ( 400, 550 ) ) ;
      allComponentPanel.setPreferredSize ( new Dimension ( 400, 550 ) ) ;
      scrollPane = new JScrollPane ( allComponentPanel ) ;
      setLayout ( new BorderLayout() ) ;
      add ( scrollPane ) ;



      return( true ) ;
   }


   /**
     *  This method is called once we are showing. We will now
     * refresh our data
     **/
   public void start () {
      parentDefWin = GUISystem.getParentDefWin ( this ) ;
      parentDefWin.addActionComponent ( this, pb_CLOSE ) ;
      refresh() ;
   }

   /**
     *  This method will refresh all of our data
     **/
   public void refresh () {
      parentDefWin.setStatus ( Str.getStr ( STR_REFRESHING ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our text                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE.setText ( brandRec.toString() ) ;
      ef_BPM.setText ( brandRec.getOwner() ) ;                                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Our Lists                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshFamilies() ;
      refreshCategories() ;
      refreshMachines() ;
      enableInput ( true ) ;

      parentDefWin.setStatus ( null ) ;
   }

   /**
     *  This method will refresh all of the families listed
     *
     **/
   private void refreshFamilies() {
      TypeCategoryRec familyRec   = (TypeCategoryRec)lb_FAMILY.getSelectedValue() ;
      Vector          families    = Brands.getFamilies ( brandRec ); 

      new QuickSort ( families ) ;
      WinUtil.setListData ( families, familyListModel ) ;
      lb_FAMILY.setModel ( familyListModel ) ;
      ck_AUTO_SORT_FAMILY.setSelected ( brandRec.getChildSort().equals ( "Y" ) ) ;
   }

   /**
     *  This method will refresh the list of machines/sub categories from
     * the selected Family object in the family list.
     **/
   private void refreshMachines () {
      TypeCategoryRec familyRec   = (TypeCategoryRec)lb_FAMILY.getSelectedValue() ;

      if ( familyRec != null ) {
         Vector machines = Brands.getMachinesSubCategoriesFromFamily ( familyRec.getInd() ) ;

         new QuickSort ( machines ) ;
         ck_AUTO_SORT_MACHINE.setEnabled ( true ) ;
         ck_AUTO_SORT_MACHINE.setSelected ( familyRec.getChildSort().equals ( "Y" ) ) ;
         WinUtil.setListData ( machines, machineListModel ) ;
         lb_MACHINE.setModel ( machineListModel ) ;
      }
   }


   /**
     *  This method will refresh the list of categories
     **/
   private void refreshCategories() {
      Vector          categories  = TypeCategory.getChildCategories ( brandRec.getInd(),
                                                                      TypeCategory.LEVEL_DOC_CATEGORY ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Sort the families and categories                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      new QuickSort ( categories ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the lists                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      lb_CATEGORY.setModel ( categoryListModel ) ;
      WinUtil.setListData ( categories, categoryListModel ) ;
      ck_AUTO_SORT_CATEGORY.setSelected ( brandRec.getChildSort().equals ( "Y" ) ) ;
   }


   /**
     *  This method will lay out and position all of our GUI
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 15 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       halfWidth = (size.width/2)-15 ;

      super.doLayout() ;

      st_TITLE.setBounds ( x, y, size.width-(x*2), rowHeight*2) ;
      y+=rowHeight*2 ;
      y+=5 ;

      vl_BPM.setBounds ( x, y, 250, rowHeight ) ;
      y+=rowHeight ;

      st_FAMILY.setBounds ( x, y, halfWidth-20, rowHeight ) ;
      st_CATEGORY.setBounds ( halfWidth+x+20, y, halfWidth-x-25, rowHeight ) ;
      y+=rowHeight ;

      ck_AUTO_SORT_FAMILY.setBounds ( x, y, halfWidth-20, rowHeight ) ;
      ck_AUTO_SORT_CATEGORY.setBounds ( halfWidth+x+20, y, halfWidth-x-25, rowHeight ) ;
      y+=rowHeight ;

      scr_FAMILY.setBounds ( x, y, halfWidth-20, rowHeight*10 ) ;
      scr_CATEGORY.setBounds ( halfWidth+x+20, y, halfWidth-x-25, rowHeight*10 ) ;
      y+=rowHeight*10 ;

      familyAction.setBounds   ( x, y, halfWidth-20, rowHeight ) ;
      categoryAction.setBounds ( halfWidth+x+20, y, halfWidth-x-25, rowHeight ) ;
      y+=rowHeight*2 ;

      st_MACHINE.setBounds ( x, y, halfWidth-20, rowHeight ) ;
      y+=rowHeight ;

      ck_AUTO_SORT_MACHINE.setBounds ( x, y, halfWidth-20, rowHeight ) ;
      y+=rowHeight ;

      scr_MACHINE.setBounds ( x, y, halfWidth-20, rowHeight*10 ) ;
      y+=rowHeight*10 ;

      machineAction.setBounds ( x, y, halfWidth-20, rowHeight ) ;
      y+=rowHeight ;


   }


   /**
     *  This method will return a small ImageIcon for us
     **/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.BRAND_MANAGEMENT_SMALL ) ) ;
   }

   /**
     *  This method will return the name 'Brand Mgmt'
     **/
   public String getName() {
      return( Str.getStr ( STR_BRAND_MANAGEMENT ) ) ;
   }

   /**
     *  This method will return an instance of this panel
     **/
   public JComponent getJComponent () {
      return( this ) ;
   }

   /**
     *  This method will return TRUE
     **/
   public boolean shutdown () {
      return( true ) ;
   }


   /**
     *  This method will be called when an ActionEvent is fired on us
     *
     * @param e The Event
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin ( this ).closePanel ( this, null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ck_AUTO_SORT_FAMILY                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ck_AUTO_SORT_FAMILY ) {
         TypeCategoryRec selectedRec = (TypeCategoryRec)lb_FAMILY.getSelectedValue() ;

         if ( ck_AUTO_SORT_FAMILY.isSelected() ) {
            brandRec.setChildSort ( "Y" ) ;
            zeroCategoryList ( familyListModel ) ;
            refreshFamilies() ;
         } else {
            brandRec.setChildSort ( "N" ) ;
            resetCategoryList ( familyListModel ) ;
         }

         refresh() ;

         if ( selectedRec != null ) {
            lb_FAMILY.removeListSelectionListener ( this ) ;
            lb_FAMILY.setSelectedValue ( selectedRec, true ) ;
            lb_FAMILY.addListSelectionListener ( this ) ;
         }

         enableInput ( true ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   remove Family                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == removeFamily )
         removeFamily() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ck_AUTO_SORT_MACHINE                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ck_AUTO_SORT_MACHINE ) {
         TypeCategoryRec familyRec = (TypeCategoryRec) lb_FAMILY.getSelectedValue() ;

         if ( ck_AUTO_SORT_MACHINE.isSelected() ) {
            familyRec.setChildSort ( "Y" ) ;
            zeroCategoryList ( machineListModel ) ;
         } else {
            familyRec.setChildSort ( "N" ) ;
            resetCategoryList ( machineListModel ) ;
         }

         refreshMachines() ;
         enableInput ( true ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ck_AUTO_SORT_CATEGORY                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ck_AUTO_SORT_CATEGORY ) {
         if ( ck_AUTO_SORT_CATEGORY.isSelected() ) {
            brandRec.setChildSort ( "Y" ) ;
            zeroCategoryList ( categoryListModel ) ;
            refresh() ;
         } else {
            brandRec.setChildSort ( "N" ) ;
            resetCategoryList ( categoryListModel ) ;
            refresh() ;
         }
         enableInput ( true ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Move Family Up                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == upFamily ) {
         lb_FAMILY.removeListSelectionListener ( this ) ;
         moveCategoryUp ( lb_FAMILY ) ;
         lb_FAMILY.addListSelectionListener ( this ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Move Family Down                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == downFamily ) {
         lb_FAMILY.removeListSelectionListener ( this ) ;
         moveCategoryDown ( lb_FAMILY ) ;
         lb_FAMILY.addListSelectionListener ( this ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Move Machine Up                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == upMachine ) {
         lb_MACHINE.removeListSelectionListener ( this ) ;
         moveCategoryUp ( lb_MACHINE ) ;
         lb_MACHINE.addListSelectionListener ( this ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Move Machine Down                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == downMachine ) {
         lb_MACHINE.removeListSelectionListener ( this ) ;
         moveCategoryDown ( lb_MACHINE ) ;
         lb_MACHINE.addListSelectionListener ( this ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Family                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == addFamily )
         addFamily() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Edit Family                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == editFamily )
         editFamily() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Edit Machine                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == editMachine ) {
         boolean isOption = Brands.isBrandOption ( brandRec ) ;
         if ( isOption )
            editSubCategory() ;
         else
            editMachine() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Move DocCategory Up                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == upCategory ) {
         lb_CATEGORY.removeListSelectionListener ( this ) ;
         moveCategoryUp ( lb_CATEGORY ) ;
         lb_CATEGORY.addListSelectionListener ( this ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Move DocCategory Down                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == downCategory ) {
         lb_CATEGORY.removeListSelectionListener ( this ) ;
         moveCategoryDown ( lb_CATEGORY ) ;
         lb_CATEGORY.addListSelectionListener ( this ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Category                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == addCategory )
         addCategory() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remove Category                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == removeCategory )
         removeCategory();

   }



   /**
     *  This method will respond to JList controls getting items selected/deselected. It will
     * adjust the enabled property on some controls depending on if something is selected or not
     **/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( !e.getValueIsAdjusting() ) {
         if ( e.getSource() == lb_FAMILY )
            refreshMachines() ;
      }
      enableInput ( true ) ;
   }


   /**
     *  This method will move the selected category object up. It will also reset the
     * order number values in the list passed
     **/
   private void moveCategoryUp ( JList list ) {
      WinUtil.moveItemUp ( list ) ;
      resetCategoryList ( (DefaultListModel)list.getModel() ) ;
   }


   /**
     *  This method will move the selected category object down. It will also reset the
     * order number values in the list passed
     **/
   private void moveCategoryDown ( JList list ) {
      WinUtil.moveItemDown ( list ) ;
      resetCategoryList ( (DefaultListModel)list.getModel() ) ;
   }


   /**
     *  This method will return TRUE
     **/
   public boolean stop () {
      return( true ) ;
   }


   /**
     *  This method will create a standard HotLinkLabel used as action buttons
     * underneath the lists of data
     **/
   private HotLinkLabel createActionHotLink ( String text ) {
      HotLinkLabel label = new HotLinkLabel ( text ) ;

      label.showSelection ( false ) ;
      return( label );
   }




   /**
     *  This method will set the order num of all category objects
     * passed in to the order number they are in the list (1, 2, 3 etc..)
     *
     * @param listModel The DefaultListModel holding the category objects
     **/
   private synchronized void resetCategoryList ( DefaultListModel listModel ) {
      int           orderNum  = 1 ;
      int           i         = 0 ;
      int           size      = listModel.getSize() ;
      StringBuffer  sqlBuffer = new StringBuffer() ;

      setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;
      upFamily.setEnabled ( false ) ;
      downFamily.setEnabled ( false ) ;
      upMachine.setEnabled ( false ) ;
      downMachine.setEnabled ( false ) ;
      upCategory.setEnabled ( false ) ;
      downCategory.setEnabled ( false ) ;

      upFamily.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;
      downFamily.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;
      upMachine.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;
      downMachine.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;
      upCategory.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;
      downCategory.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;


      while ( i < size ) {
         ((TypeCategoryRec)listModel.elementAt(i)).setOrderNum ( orderNum ) ;
         sqlBuffer.append ( ((TypeCategoryRec)listModel.elementAt(i)).toSQL() ) ;
         orderNum++ ;
         i++ ;
      }

      if ( SqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST, sqlBuffer.toString() ) != 0 )
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;

      enableInput ( true ) ;
      setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
      upFamily.setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
      downFamily.setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
      upMachine.setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
      downMachine.setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
      upCategory.setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
      downCategory.setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
   }


   /**
     *  This method will set the order num of all category objects
     * passed in to zero
     *
     * @param listModel The DefaultListModel holding the category objects
     **/
   private synchronized void  zeroCategoryList ( DefaultListModel listModel ) {
      int orderNum = 1 ;
      int i        = 0 ;
      int size     = listModel.getSize() ;

      while ( i < size ) {
         ((TypeCategoryRec)listModel.elementAt(i)).setOrderNum ( 0 ) ;
         orderNum++ ;
         i++ ;
      }

      setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;
      TypeCategory.writeToDatabase() ;
      setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
   }



   /**
     *  This method will enable/disable appropriate controls based
     * on selections made in the lists
     **/
   private void enableInput ( boolean enable ) {
      if ( enable ) {
         boolean familySelected    = ( lb_FAMILY.getSelectedValue() != null ) ;
         boolean categorySelected  = ( lb_CATEGORY.getSelectedValue() != null ) ;
         boolean machineSelected   = ( lb_MACHINE.getSelectedValue() != null ) ;

         ck_AUTO_SORT_MACHINE.setEnabled ( familySelected ) ;
         scr_MACHINE.setEnabled ( familySelected ) ;

         editFamily.setEnabled ( familySelected ) ;
         removeFamily.setEnabled ( familySelected ) ;

         editMachine.setEnabled ( machineSelected ) ;
         removeCategory.setEnabled ( categorySelected ) ;

         upFamily.setEnabled ( familySelected && !ck_AUTO_SORT_FAMILY.isSelected() ) ;
         downFamily.setEnabled ( familySelected && !ck_AUTO_SORT_FAMILY.isSelected() ) ;

         upMachine.setEnabled ( machineSelected && !ck_AUTO_SORT_MACHINE.isSelected() ) ;
         downMachine.setEnabled ( machineSelected && !ck_AUTO_SORT_MACHINE.isSelected() ) ;

         upCategory.setEnabled ( categorySelected && !ck_AUTO_SORT_CATEGORY.isSelected() ) ;
         downCategory.setEnabled ( categorySelected && !ck_AUTO_SORT_CATEGORY.isSelected() ) ;
      }
   }





   /**
     *  This method will add a family to the database, including making
     * the addition to the CLIO database.
     **/
   private void addFamily () {
      String familyName = getFamilyName ( null ) ;                                    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we are to create the family, do so now           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( familyName != null ) {
         TypeCategoryRec newFamilyRec = TypeCategory.createTypeCategory ( brandRec.getInd(),
                                                                          familyName,
                                                                          TypeCategory.LEVEL_FAMILY,
                                                                          1,
                                                                          "N",
                                                                          UserSystem.getUserId() ) ;

         if ( newFamilyRec != null ) {
            TypeCategoryRec newFamilyDraftRec = TypeCategory.createDraftTypeCategory ( newFamilyRec.getInd(),
                                                                                       brandRec.getInd(),
                                                                                       familyName,
                                                                                       TypeCategory.LEVEL_FAMILY,
                                                                                       1,
                                                                                       "N",
                                                                                       UserSystem.getUserId() ) ;
            int rc = newFamilyRec.writeToDatabase() ;

            if ( rc == 0 )
               rc = newFamilyDraftRec.writeToDatabase() ;

            switch ( rc ) {
               case 0 :
                  //VOY
                  refreshFamilies() ;
                  lb_FAMILY.setSelectedValue ( newFamilyRec, true ) ;
                  break ;

               default:
                  LogSystem.log ( 1, "Cannot save to TypeCategory. Return Code:"+rc ) ;
                  GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
                  break ;
            }
         }

      }
   } /* addFamily  */


   /**
     *  This method will edit a family name. It will make the correct updates in both
     * CLIO and Quest. It will also update the Draft family name in Quest
     **/
   private void editFamily() {
      TypeCategoryRec familyRec  = (TypeCategoryRec)lb_FAMILY.getSelectedValue() ;
      String          familyName = getFamilyName ( familyRec.toString() ) ;

      if ( familyName != null ) {
         int             rc             = 0 ;
         TypeCategoryRec draftFamilyRec = TypeCategory.getDraftCategory ( familyRec.getInd() ) ;

         /* We should always have a draftCategory, but if we didn't find one
            then we must create it  */
         if ( draftFamilyRec == null ) {
            draftFamilyRec = TypeCategory.createDraftTypeCategory ( familyRec.getInd(),
                                                                    brandRec.getInd(),
                                                                    familyName,
                                                                    TypeCategory.LEVEL_FAMILY,
                                                                    1,
                                                                    "N",
                                                                    UserSystem.getUserId() ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust the descriptions                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         draftFamilyRec.setDescript ( familyName ) ;
         familyRec.setDescript ( familyName ) ;

         rc = TypeCategory.writeToDatabase() ;

         switch ( rc ) {
            case 0 :
               //saveToClio
               refreshFamilies() ;
               lb_FAMILY.setSelectedValue ( familyRec, true ) ;
               break ;

            default:
               LogSystem.log ( 1, "Cannot save to TypeCategory. Return Code:"+rc ) ;
               GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
               break ;
         }

      }
   }


   /**
     *  This method will edit a Machine name. It will make the correct updates in both
     * CLIO and Quest. 
     **/
   private void editMachine() {
      TypeCategoryRec machineRec  = (TypeCategoryRec)lb_MACHINE.getSelectedValue() ;
      TypeCategoryRec familyRec   = (TypeCategoryRec)lb_FAMILY.getSelectedValue() ;
      String          machineName = getMachineName ( machineRec.toString(), familyRec ) ;

      if ( machineName != null ) {
         int             rc             = 0 ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust the descriptions                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         machineRec.setDescript ( machineName ) ;

         rc = machineRec.writeToDatabase() ;

         switch ( rc ) {
            case 0 :
               //VOY
               refreshMachines() ;
               lb_MACHINE.setSelectedValue ( machineRec, true ) ;
               break ;

            default:
               LogSystem.log ( 1, "Cannot save to TypeCategory. Return Code:"+rc ) ;
               GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
               break ;
         }

      }
   }


   /**
     *  This method will edit a Sub Category name. It will make the correct updates in both
     * CLIO and Quest. 
     **/
   private void editSubCategory() {
      TypeCategoryRec subCategoryRec  = (TypeCategoryRec)lb_MACHINE.getSelectedValue() ;
      TypeCategoryRec familyRec       = (TypeCategoryRec)lb_FAMILY.getSelectedValue() ;
      String          subCategoryName = getSubCategoryName ( subCategoryRec.toString(), familyRec ) ;

      if ( subCategoryName != null ) {
         int             rc             = 0 ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust the descriptions                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         subCategoryRec.setDescript ( subCategoryName ) ;

         rc = TypeCategory.writeToDatabase() ;

         switch ( rc ) {
            case 0 :
               //VOY
               refreshMachines() ;
               lb_MACHINE.setSelectedValue ( subCategoryRec, true ) ;
               break ;

            default:
               LogSystem.log ( 1, "Cannot save to TypeCategory. Return Code:"+rc ) ;
               GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
               break ;
         }

      }
   }



   /**
     *  This method will ask the user for a family name. It will also
     * check to make sure that the family name is not already taken under
     * this brand. It will not allow duplicate family names under a single
     * brand. This method will also warn the user if the family name entered
     * is part of another brand.
     *
     * @param familyName The initial family name. This value may be null
     **/
   private String getFamilyName ( String familyName ) {
      boolean cont         = true ;

      while ( cont ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the family name                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         familyName   = new SingleEntryDlg ( parentDefWin, 
                                             Str.getStr ( STR_FAMILY ),
                                             Str.getStr ( STR_PLEASE_ENTER_FAMILY ),
                                             (familyName == null ) ? "" : familyName ).getResult();                                                           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only process a good family name                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( familyName != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, check to see if the name already exists under    */
            /* this brand                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( TypeCategory.getCategoryFromDescription ( familyName,
                                                           TypeCategory.LEVEL_FAMILY,
                                                           brandRec.getInd() ) != null ) {
               GUISystem.printBox ( STR_INFO, STR_FAMILY_EXISTS_IN_BRAND ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now check to see if this family exists under        */
            /* another brand                                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( TypeCategory.getCategoryFromDescription ( familyName, TypeCategory.LEVEL_FAMILY ) != null ) {
               TypeCategoryRec otherFamilyRec = TypeCategory.getCategoryFromDescription ( familyName, 
                                                                                          TypeCategory.LEVEL_FAMILY ) ;
               TypeCategoryRec otherBrandRec = TypeCategory.getCategory ( otherFamilyRec.getParentCatInd() ) ;

               cont = !GUISystem.printBox ( Str.getStr ( STR_QUESTION ),
                                            Str.getStr ( STR_FAMILY_EXISTS_IN_THE_BRAND )+" '"+
                                            otherBrandRec.toString()+"' "+
                                            Str.getStr ( STR_DO_YOU_WISH_TO_CREATE_FAMILY ) ) ;

            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, this passed all of our checks, continue             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else
               cont = false ;



         } else {
            cont = false ; //Exit the loop
            familyName = null ;
         }
      }

      return( familyName ) ;
   }


   /**
     *  This method will ask the user for a Machine name. It will also
     * check to make sure that the machine name is not already taken under
     * this family. It will not allow duplicate names under a single
     * family. This method will also warn the user if the name entered
     * is part of another family.
     *
     * @param machineName The initial machine name. This value may be null
     * @param familyRec The TypeCategoryRec representing the family
     **/
   private String getMachineName ( String machineName, TypeCategoryRec familyRec ) {
      boolean cont         = true ;

      while ( cont ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the machine name                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         machineName   = new SingleEntryDlg ( parentDefWin, 
                                              Str.getStr ( STR_MACHINE ),
                                              Str.getStr ( AdminConst.STR_PLEASE_ENTER_MACHINE ),
                                              (machineName == null ) ? "" : machineName, new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 4 ) ).getResult();                                                           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only process a good machine name                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( machineName != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, check to see if the name already exists under    */
            /* this family                                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( TypeCategory.getCategoryFromDescription ( machineName,
                                                           TypeCategory.LEVEL_MACHINE,
                                                           familyRec.getInd() ) != null ) {
               GUISystem.printBox ( STR_INFO, STR_MACHINE_EXISTS_UNDER_FAMILY ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now check to see if this family exists under        */
            /* another family                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( TypeCategory.getCategoryFromDescription ( machineName, TypeCategory.LEVEL_MACHINE ) != null ) {
               TypeCategoryRec otherMachineRec = TypeCategory.getCategoryFromDescription ( machineName, 
                                                                                           TypeCategory.LEVEL_MACHINE ) ;
               TypeCategoryRec otherFamilyRec = TypeCategory.getCategory ( otherMachineRec.getParentCatInd() ) ;

               cont = !GUISystem.printBox ( Str.getStr ( STR_QUESTION ),
                                            Str.getStr ( STR_MACHINE_EXISTS_IN_THE_FAMILY )+" '"+
                                            otherFamilyRec.toString()+"' "+
                                            Str.getStr ( STR_DO_YOU_WISH_TO_CREATE_MACHINE ) ) ;

            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, this passed all of our checks, continue             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else
               cont = false ;



         } else {
            cont = false ; //Exit the loop
            machineName = null ;
         }
      }

      return( machineName ) ;
   }


   /**
     *  This method will ask the user for a SubCategory name. It will also
     * check to make sure that the subCategory category name is not already taken under
     * this family. It will not allow duplicate names under a single
     * family. This method will also warn the user if the name entered
     * is part of another family.
     *
     * @param subCategoryName The initial subCategory name. This value may be null
     * @param familyRec The TypeCategoryRec representing the family
     **/
   private String getSubCategoryName ( String subCategoryName, TypeCategoryRec familyRec ) {
      boolean cont         = true ;

      while ( cont ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the subCategory name                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         subCategoryName   = new SingleEntryDlg ( parentDefWin, 
                                                  Str.getStr ( STR_SUBCATEGORY ),
                                                  Str.getStr ( AdminConst.STR_PLEASE_ENTER_SUBCATEGORY ),
                                                  (subCategoryName == null ) ? "" : subCategoryName, new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 25 ) ).getResult();                                                           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only process a good subCategory name                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( subCategoryName != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, check to see if the name already exists under    */
            /* this family                                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( TypeCategory.getCategoryFromDescription ( subCategoryName,
                                                           TypeCategory.LEVEL_SUBCATEGORY,
                                                           familyRec.getInd() ) != null ) {
               GUISystem.printBox ( STR_INFO, STR_SUBCATEGORY_EXISTS_UNDER_FAMILY ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now check to see if this family exists under        */
            /* another brand                                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( TypeCategory.getCategoryFromDescription ( subCategoryName, TypeCategory.LEVEL_SUBCATEGORY ) != null ) {
               TypeCategoryRec otherSubCategoryRec = TypeCategory.getCategoryFromDescription ( subCategoryName, 
                                                                                               TypeCategory.LEVEL_SUBCATEGORY ) ;
               TypeCategoryRec otherFamilyRec = TypeCategory.getCategory ( otherSubCategoryRec.getParentCatInd() ) ;

               cont = !GUISystem.printBox ( Str.getStr ( STR_QUESTION ),
                                            Str.getStr ( STR_SUBCATEGORY_EXISTS_IN_THE_FAMILY )+" '"+
                                            otherFamilyRec.toString()+"' "+
                                            Str.getStr ( STR_DO_YOU_WISH_TO_CREATE_SUBCATEGORY ) ) ;

            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, this passed all of our checks, continue             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else
               cont = false ;



         } else {
            cont = false ; //Exit the loop
            subCategoryName = null ;
         }
      }

      return( subCategoryName ) ;
   }




   /**
     *  This method will add unique categories to this brand from a predetermined list of categories
     * in the TIGRIS.TYPEDOCCAT table
     **/
   private void addCategory () {
      Vector categories = new JListDlg ( parentDefWin, 
                                         Str.getStr ( STR_DOCUMENT_CATEGORY ),
                                         TypeList.getInstance().getTypeList ( TypeList.TYPE_DOCCAT ) ).getResult() ;

      setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   IF the user selected at least 1 item, process them now  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( categories != null ) {
         int             i          = 0 ;
         int             size       = categories.size() ;
         TypeDocCatRec   docCatRec  = null ;
         int             rc         = 0 ;

         while ( i < size ) {
            docCatRec = (TypeDocCatRec) categories.elementAt(i) ;

            /* only create a new type category if one doesn't exist  */
            if ( TypeCategory.getCategoryFromDescription ( docCatRec.toString(), 
                                                           TypeCategory.LEVEL_DOC_CATEGORY, 
                                                           brandRec.getInd() ) == null ) {
               parentDefWin.setStatus ( Str.getStr ( STR_ASSOCIATING )+" "+docCatRec.toString() ) ;
               TypeCategory.createTypeCategory ( brandRec.getInd(),
                                                 docCatRec.toString(),
                                                 TypeCategory.LEVEL_DOC_CATEGORY,
                                                 1,
                                                 "N",
                                                 UserSystem.getUserId() ) ;
            }
            i++ ;
         }


         /* Ok, now save the information  */
         rc = TypeCategory.writeToDatabase() ;

         switch ( rc ) {
            case 0 :
               refreshCategories() ;
               break ;

            default:
               LogSystem.log ( 1, "Cannot save to TypeCategory. Return Code:"+rc ) ;
               GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
               break ;
         }

      } /* if  */
      setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
      parentDefWin.setStatus ( Str.getStr ( STR_COMPLETE ), 2 ) ;
   }


   /**
     *  This method will remove all selected categories from this brand
     **/
   private void removeCategory() {
      Object[] selected = lb_CATEGORY.getSelectedValues() ;

      if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_DOC_CATEGORY ) ) {
         boolean cont = true ;

         setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)) ;
         parentDefWin.setStatus ( STR_PLEASE_WAIT ) ;
         try {
            if ( ( selected != null ) && ( selected.length > 0 ) ) {
               int rc = 0 ;

               for ( int i = 0; i < selected.length; i++ ) {
                  TypeCategoryRec typeCategoryRec = (TypeCategoryRec)selected[i] ;
                  Vector          documents       = DocumentSQL.getDocumentsFromTypeCategory ( typeCategoryRec.getInd() ) ;

                  /*------------------------------------------------------*/
                  /*                                                      */
                  /*  If we have documents that fall under this category, */
                  /* ask the user if they wish to see the documents       */
                  /*                                                      */
                  /*------------------------------------------------------*/

                  if ( ( documents != null ) && 
                       ( documents.size() > 0 ) && 
                       ( GUISystem.printBox ( Str.getStr ( STR_QUESTION ), 
                                              Str.getStr ( STR_DO_YOU_WISH_TO_SEE_DELETE_DOC_CAT_DOC )+typeCategoryRec.toString() ) ) ) {

                     new DocumentDlg ( parentDefWin, documents ) ;
                     cont = GUISystem.printBox ( STR_QUESTION, STR_STILL_WANT_DELETE_DOC_CAT ) ;
                  }

                  if ( cont )
                     typeCategoryRec.updateRecStatus ( TypeCategoryRec.REC_DELETE ) ;
                  cont = true ;
               }

               /* Ok, now save the information  */
               parentDefWin.setStatus ( Str.getStr ( STR_DELETING )+" "+selected.length ) ;
               rc = TypeCategory.writeToDatabase() ;

               switch ( rc ) {
                  case 0 :
                     refreshCategories() ;
                     break ;

                  default:
                     LogSystem.log ( 1, "Cannot save to TypeCategory. Return Code:"+rc ) ;
                     GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
                     break ;
               }
            }
            parentDefWin.setStatus ( Str.getStr ( STR_COMPLETE ), 2 ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            parentDefWin.setStatus ( null ) ;
         }
         setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) ;
      } /* if GUISystem  */

      parentDefWin.setStatus ( null ) ;
   }



   /**
     *  This method will remove a family safely. This is done by:<br><br>
     *
     * 1. Checking to see if there are any Products (Recycled or not) under the family<br>
     * 2. Checking to see if there are any documents (Recycled or not) associated to the family<br><br>
     *
     * If either #1 or #2 above are TRUE then the user will be prompted with a dialog
     * telling them what Products and Documents are associated and will not allow the delete, otherwise
     * this method will go ahead and make the deletes.
     **/
   private void removeFamily() {
      if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_FAMILY ) ) {
         TypeCategoryRec familyRec = (TypeCategoryRec)lb_FAMILY.getSelectedValue() ;
         StatusWin       statusWin = new StatusWin ( parentDefWin ) ;

         try {
            Vector documents = null ;
            Vector products  = null ;

            if ( GUISystem.printBox ( STR_QUESTION, STR_SHOW_FAMILY_DELETE_OBJECTS ) ) {
               JTreeDlg dlg               = null ;
               Vector   documentTemplates = null ;
               Vector   documentHistory   = null ;
               Vector   productTemplates  = null ;
               Vector   productHistory    = null ;
               Vector   deletedProducts   = null ;
               Vector   deletedDocuments  = null ;

               statusWin.setText ( Str.getStr ( STR_READING ) ) ;
               documentTemplates = DocumentSQL.getDocumentTemplatesFromFamily ( familyRec.getInd() ) ;
               documentHistory   = DocumentSQL.getDocumentHistoryFromFamily   ( familyRec.getInd() ) ;
               deletedDocuments  = DocumentSQL.getDeletedDocumentsFromFamily  ( familyRec.getInd() ) ;
               productTemplates  = ProductSQL.getProductTemplatesFromFamily   ( familyRec.getInd() ) ;
               productHistory    = ProductSQL.getProductHistoryFromFamily     ( familyRec.getInd() ) ;
               deletedProducts   = ProductSQL.getDeletedProductsFromFamily    ( familyRec.getInd() ) ;

               if ( ( documentTemplates.size() > 0 ) ||
                    ( documentHistory.size() > 0 ) ||
                    ( deletedDocuments.size() > 0 ) ||
                    ( productTemplates.size() > 0 ) ||
                    ( productHistory.size() > 0 ) ||
                    ( deletedProducts.size() > 0 ) ) {
                  dlg = new JTreeDlg ( parentDefWin, Str.getStr ( STR_DOCUMENTS_PRODUCTS ) ) ;
                  if ( documentTemplates.size() > 0 )
                     dlg.addChild ( Str.getStr ( STR_DOCUMENT_TEMPLATES ), documentTemplates ) ;
                  if ( deletedDocuments.size() > 0 )
                     dlg.addChild ( Str.getStr ( STR_DELETED_DOCUMENTS ),  deletedDocuments ) ;
                  if ( productTemplates.size() > 0 )
                     dlg.addChild ( Str.getStr ( STR_PRODUCT_TEMPLATES ), productTemplates ) ;
                  if ( productHistory.size() > 0 )
                     dlg.addChild ( Str.getStr ( STR_PRODUCT_HISTORY ), productHistory );
                  if ( deletedProducts.size() > 0 )
                     dlg.addChild ( Str.getStr ( STR_DELETED_PRODUCTS ), deletedProducts ) ;

                  dlg.getResult() ;
               } else
                  GUISystem.printBox ( STR_INFO, STR_NO_AFFECTED_PRODUCTS_OR_DOCS ) ;

            }

            statusWin.setText ( Str.getStr ( STR_CHECKING_FOR_FAMILY_DELETE ) ) ;
            documents = DocumentSQL.getDocumentsFromTypeCategory ( familyRec.getInd() ) ;
            products  = ProductSQL.getProductsFromFamily   ( familyRec.getInd() ) ;

            /* If we have docuemnts or products, we must show the user now  */
            if ( ( documents.size() > 0 ) || ( products.size() > 0 ) ) {
               ProductDocumentDlg dlg = new ProductDocumentDlg ( parentDefWin, products, documents );

               dlg.setLargeTitle ( Str.getStr ( STR_ERROR ) ) ;
               dlg.setDescription ( Str.getStr ( STR_CANNOT_REMOVE_FAMILY_WITH_PROD_DOC ) ) ;
               dlg.setVisible(true) ;
            }

            /* Otherwise, continue  */
            else {

               statusWin.setText ( Str.getStr ( STR_DELETING_FAMILY ) ) ;
               familyRec.updateRecStatus ( familyRec.REC_DELETE ) ;
               TypeCategory.writeToDatabase() ;
               refreshFamilies() ;

            }


         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
         }

         statusWin.dispose() ;
      }
   } /* removeFamily  */


   /**
     *  This method will be called when the user has selected a User in the Brand Program Manager
     *
     * @param userRec The UserRec that was selected
     **/
   public void userSelected ( UserRec userRec ) {
      String oldOwner = brandRec.getOwner() ;
      int    rc       = 0 ;

      setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR )) ;

      brandRec.setOwner ( userRec.getUserId() ) ;
      rc = TypeCategory.writeToDatabase() ;

      if ( rc != 0 ) {
         ef_BPM.setText ( oldOwner ) ;
         LogSystem.log ( 1, "Could not change BPM to "+userRec.getUserId()+". SQLCode:"+rc ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR )) ;
   }


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   The following methods do nothing                        */                       
   /*                                                           */
   /*-----------------------------------------------------------*/
   public boolean copyText() {
      return( false ) ;
   }
   public boolean cutText() {
      return( false ) ;
   }
   public boolean pasteText() {
      return( false ) ;
   }
   public NavPanel getDefaultNavPanel () {
      return( null ) ;
   }
   public void panelSelected () {
   }
   public void panelDeselected () {
   }
   public boolean bookmark () {
      return( false ) ;
   }
    }
     /**
    *  This method will give the Print support to the panel
    **/
    public Object getPrintObject() {
        return( new Print ( allComponentPanel, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_DOCUMENTS ) ) ) ;
    }
    /**
    *  This method will give the Preview support to the panel
    **/
    public Object getPreviewObject() {
        return( new Preview ( allComponentPanel, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_DOCUMENTS ) ) ) ;

} /* BrandMgmtPanel */

