package com.ibm.nzna.projects.qit.admin.brandmgmt ;           
/*-----------------------------------------------------------*/
/*   SelectBrandPanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectBrandPanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.admin.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This panel simply lists all of the available brands and allows the user 
 * to select a brand. Once a brand is selected this panel will close and start
 * a BrandEditorPanel with the selected brand
 *
 * @version 1.0, 6/20/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectBrandPanel                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectBrandPanel extends JPanel implements ActionListener, AdminConst, AppConst, QuestPanel {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  This is the close button
     **/
   private ActionButton pb_CLOSE = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE ) ) ;

   /**
     *  This is the list which holds all of the brands
     **/
   private ListPanel brandList = new ListPanel() ;

   /**
     *  This is the little blurb of help
     **/
   private JLabel st_HELP = new JLabel ( Str.getStr ( STR_PLEASE_SELECT_A_BRAND ) ) ;

   /**
     *  This is the scroll pane which holds the list
     **/
   private JScrollPane scrollPane = new JScrollPane( brandList ) ;



   /**
     *  This method will erase all current brands and insert all brands again
     **/
   public void refresh () {
      Vector brands = Brands.getBrands() ;

      brandList.removeAll() ;

      if ( ( brands != null ) && ( brands.size() > 0 ) ) {
         int          i    = 0 ;
         int          size = brands.size() ;
         HotLinkLabel link = null ;
         Dimension    prefSize = new Dimension ( 250, 35 ) ;

         while ( i < size ) {
            if ( ((TypeCategoryRec)brands.elementAt(i)).getValid() ) {
               link = new HotLinkLabel ( ((TypeCategoryRec)brands.elementAt(i)).toString(),
                                         ImageSystem.getImageIcon ( this, ImageSystem.BRAND_MANAGEMENT ) ) ;

               brandList.add ( link ) ;
               link.addActionListener ( this ) ;
               link.setPreferredSize ( prefSize ) ;
               link.showSelection ( false ) ;
            }
            i++ ;
         } /* while  */
      } /* if  */
      revalidate() ;
   } /* refresh  */

   /**
     *  This method will be called before we are shown. We will
     * just return TRUE
     **/
   public boolean initialize () {        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      WinUtil.setBackgroundAllContainer ( scrollPane, Color.white ) ;
      scrollPane.setBorder ( GUISystem.emptyBorder ) ;
      st_HELP.setOpaque ( false );
      setBackground ( Color.white ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADd listeners                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener ( this ) ;
      



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout ( 20, 20) ) ;
      add ( st_HELP, BorderLayout.NORTH ) ;
      add ( scrollPane, BorderLayout.CENTER) ;
      add ( new JLabel ( "   " ), BorderLayout.WEST ) ;

      return( true ) ;
   }

   /**
     *  This method will be called once we are showing. We will
     * refresh our brands here.
     **/
   public void start() {
      GUISystem.getParentDefWin ( this ).addActionComponent ( this, pb_CLOSE ) ;
      refresh() ;
   }

   /**
     *  This method will return the small Brand Management Icon
     *
     * @return ImageIcon
     **/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.BRAND_MANAGEMENT_SMALL ) ) ;
   }


   /**
     *  This method will be called when one of our brands is selected
     **/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() instanceof HotLinkLabel ) {
         BrandMgmtPanel  panel    = null ;
         TypeCategoryRec brandRec = TypeCategory.getCategory ( ((HotLinkLabel)e.getSource()).getText(),
                                                               TypeCategory.LEVEL_BRAND ) ;

         WindowSystem.createPanel ( panel = new BrandMgmtPanel ( brandRec ) ) ;
      } else
         GUISystem.getParentDefWin(this).closePanel ( this, null ) ;
   }

   public void panelSelected() {
   }
   public void panelDeselected() {
   }
   public boolean copyText() {
      return false ;
   }
   public boolean pasteText() {
      return false ;
   }
   public boolean cutText() {
      return false ;
   }
   public boolean stop () {
      return true;
   }
   public boolean bookmark () {
      return false;
   }
   public boolean shutdown() {
      return true;
   }
   public JComponent getJComponent() {
      return( this );
   } 
   public NavPanel getDefaultNavPanel() {
      return null;
   }

   public String getName () {
      return( Str.getStr ( STR_BRAND_MANAGEMENT ) ) ;
   }

} /* SelectBrandPanel */
