package com.ibm.nzna.projects.qit.admin.keyword ;          
/*-----------------------------------------------------------*/
/*   KeywordPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: KeywordPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.Vector ;
import java.io.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;    
import com.ibm.nzna.projects.qit.print.* ;    
import com.ibm.nzna.projects.qit.admin.* ;    
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel allows the user to adjust keywords in the TYPEXX tables in the tigris database 
 *
 *
 * @version 1.0, 4/9/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   KeywordPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class KeywordPanel extends JPanel implements PrintPanel, PreviewPanel, AdminConst, AppConst, QuestPanel, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  This is simply a holder for the titles in our MultiList
     **/
   private static final String[] listTitle = {"","","","",""} ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Components                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  Our NavPanel
     **/
   private KeywordNavPanel defaultNavPanel    = null ;

   /**
     *  The 'Export' button
     **/
   private ActionButton pb_EXPORT = null ;

   /**
     *  The CLose button
     **/
   private ActionButton    pb_CLOSE           = null ;

   /**
     *  The 'Edit' button
     **/
   private ActionButton    pb_EDIT            = null ;

   /**
     *  The 'New' button
     **/
   private ActionButton    pb_NEW             = null ;

   /**
     *  The 'Delete' button
     **/
   private ActionButton    pb_DELETE          = null ;

   /**
     *  The 'Title' of the type list
     **/
   private JTitle          st_TITLE           = null ;

   /**
     *  The multi list which shows our data
     **/
   private MultiList       cnr_DATA           = null ;      


   /**
     *  This is the KeywordList that is currently being edited
     **/
   private KeywordList currentKeywordList = null ;

   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create all of our GUI and will initialize
    * this panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE        = new ActionButton ( Str.getStr ( STR_CLOSE ),  ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),  Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;            
      pb_EDIT         = new ActionButton ( Str.getStr ( STR_EDIT ),   ImageSystem.getImageIcon ( this, ImageSystem.EDIT ),   Str.getStr ( STR_EDIT ) ) ;            
      pb_NEW          = new ActionButton ( Str.getStr ( STR_NEW   ),  ImageSystem.getImageIcon ( this, ImageSystem.NEW ),    Str.getStr ( STR_NEW ) ) ;            
      pb_DELETE       = new ActionButton ( Str.getStr ( STR_DELETE ), ImageSystem.getImageIcon ( this, ImageSystem.DELETE ), Str.getStr ( STR_DELETE ) ) ;            
      pb_EXPORT       = new ActionButton ( Str.getStr ( STR_EXPORT ), ImageSystem.getImageIcon ( this, ImageSystem.NEW ),    Str.getStr ( STR_EXPORT ) ) ;            
      st_TITLE        = new JTitle ( "" ) ;
      cnr_DATA        = new MultiList ( GUISystem.getFontUtil() ) ;
      defaultNavPanel = new KeywordNavPanel ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_DATA.setColumnHeadings ( listTitle ) ;
      cnr_DATA.setColumnWidth ( 0, 250 ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_EXPORT.addActionListener  ( this ) ;  
      pb_CLOSE.addActionListener  ( this ) ;  
      pb_NEW.addActionListener    ( this ) ;
      pb_EDIT.addActionListener   ( this ) ;
      pb_DELETE.addActionListener ( this ) ;
      cnr_DATA.addActionListener  ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( st_TITLE, BorderLayout.NORTH ) ;
      add ( cnr_DATA, BorderLayout.CENTER ) ;

      return( true ) ;
   }






   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we 
    *  are showing. We will refresh our views now
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_NEW ) ;
      parentWin.addActionComponent ( this, pb_EDIT ) ;
      parentWin.addActionComponent ( this, pb_DELETE ) ;
      parentWin.addActionComponent ( this, pb_EXPORT ) ;
   }


   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      if ( currentKeywordList != null ) {
         Vector data = currentKeywordList.getKeywords() ;

         new com.ibm.nzna.shared.sort.QuickSort ( data ) ;
         cnr_DATA.setData ( data ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_KEYWORD_MANAGEMENT ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon that represents
    *  us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_KEYWORD ) ) ;
   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when we
    *  are selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }

   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when we
    *  are deselected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our navigation
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NavPanel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( defaultNavPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when we need to close.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      removeAll() ;
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when we need
    *  to shutdown. We will return TRUE because
    *  we don't care if the app shuts down
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when we need
    *  to respond to button events
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin ( this ).closePanel ( this, null ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_EDIT ) || ( e.getSource() == cnr_DATA ) ) {
         if ( currentKeywordList != null ) {
            Object selectedObject = cnr_DATA.getSelectedItem() ;
            if ( currentKeywordList.edit ( selectedObject  ) ) {
               refresh() ;
               cnr_DATA.setSelectedItem ( selectedObject, true ) ;
            }
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_DELETE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DELETE ) {
         TypeRec typeRec = (TypeRec) cnr_DATA.getSelectedItem() ;

         if ( typeRec != null ) {
            if ( currentKeywordList != null ) {
               if ( currentKeywordList.delete ( cnr_DATA.getSelectedItem() ) ) {
                  refresh() ;
               }
            }
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_NEW                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_NEW ) {
         if ( currentKeywordList != null ) {
            Object object = currentKeywordList.add() ;

            refresh() ;
            cnr_DATA.setSelectedItem ( object, true ) ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EXPORT                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EXPORT ) {
         if ( currentKeywordList != null )
            export() ;
      }

   }





   /*------------------------------------------------------------*/
   /* setTypeList
    */
   /**
    *    This method will set what type of list that
    *  we are showing. The typeListInd must be one
    *  of the Type Lists defined in com.ibm.nzna.projects.common.quest.type.TypeList.java
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setKeywordList ( KeywordList keywordList ) {
      this.currentKeywordList = keywordList ;

      st_TITLE.setText ( currentKeywordList.getName() ) ;
      cnr_DATA.removeAll() ;
      cnr_DATA.setColumnHeadings ( currentKeywordList.getTitles() ) ;

      /*-------------------------------------------------*/
      /*                                                 */
      /*  If this is the warranty keywords, handle that  */
      /*                                                 */
      /*-------------------------------------------------*/
      if ( keywordList.toString().equals ( Str.getStr ( STR_TYPE_WARRANTY ) ) ) {
         cnr_DATA.setColumnWidth ( 0, 40 ) ;
         cnr_DATA.setColumnWidth ( 1, 100 ) ;
         cnr_DATA.setColumnWidth ( 2, 450 ) ;
      } else
         cnr_DATA.setColumnWidth ( 0, 250 ) ;

      refresh() ;
   }

   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }



   /**
     *  This method will export the current list of data to a CSV files
     **/
   private void export () {
      File[] files = GUISystem.getFileOpen ( GUISystem.getParentDefWin ( this ),
                                             Str.getStr ( STR_EXPORT ),
                                             PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ),
                                             false ) ;

      if ( ( files != null ) && ( files.length > 0 ) ) {
         try {
            cnr_DATA.exportToFile (files[0].toString() ) ;
            GUISystem.printBox ( STR_INFO, STR_COMPLETE ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
         }

      }
   }
   /**
    *  This method will give the Print support to the panel
    **/
   public Object getPrintObject() {
      return( new Print ( cnr_DATA, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }
   /**
    *  This method will give the Preview support to the panel
    **/
   public Object getPreviewObject() {
      return( new Preview ( cnr_DATA, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }


} /* KeywordPanel */
