package com.ibm.nzna.projects.qit.admin.keyword.customer ;
/*-----------------------------------------------------------*/
/*   TypeCustomerList                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeCustomerList                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.admin.keyword.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.admin.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object represents the PRODUCT.TYPECUSTOMER   table for the Keyword Management Applet 
 *
 *
 * @version 1.0, 8/6/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeCustomerList                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeCustomerList extends Object implements AppConst, KeywordList, Authority, AdminConst {



   /**
     *  This method will return our name
     **/
   public String getName () {
      return( Str.getStr ( STR_TYPE_CUSTOMER ) ) ;
   }


   /**
     *  This method will add an item to our date list
     **/
   public Object add () {
      TypeCustomerRec customerRec = null ;
      boolean         rc          = false ;
      String          newDescript = new SingleEntryDlg ( MainWindow.getInstance(),
                                                         Str.getStr ( STR_TYPE_CUSTOMER ),
                                                         Str.getStr ( STR_PLEASE_ENTER_CUSTOMER ),
                                                         "",
                                                         "" ).getResult() ;

      if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {

         if ( TypeList.getInstance().objectFromString ( newDescript, TypeList.TYPE_CUSTOMER ) == null ) {
            int newInd = Counter.getCounter ( LogSystem.getInstance(), "TYPECUST" ) ;

            if ( newInd > 0 ) {
               int saveRC  = 0 ;

               customerRec = new TypeCustomerRec ( newInd, newDescript ) ;
               customerRec.updateRecStatus ( customerRec.REC_INSERT ) ;
               saveRC = customerRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Could not create new TypeCustomer of "+newDescript+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                  customerRec = null ;
               } else {
                  TypeList.getInstance().addToList ( customerRec, TypeList.TYPE_CUSTOMER ) ;
               }
            }
         } else
            GUISystem.printBox ( STR_INFO, STR_THAT_ITEM_ALREADY_EXISTS_IN_KEYWORD_LIST ) ;
      }

      return( customerRec ) ;
   }


   /**
     *  This method will remove an item from our list
     **/
   public boolean delete ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeCustomerRec ) ) {
         TypeCustomerRec customerRec = (TypeCustomerRec) item ;

         if ( customerRec.getInd() == 0 )
            GUISystem.printBox ( STR_INFO, STR_CANNOT_DELETE_DEFAULT ) ;
         else {
            if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_CUSTOMER_LIST ) ) {
               int saveRC = 0 ;

               customerRec.updateRecStatus ( customerRec.REC_DELETE ) ;
               saveRC = customerRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Cannot delete Customer "+customerRec.toString()+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
               } else {
                  rc = true ;
                  TypeList.getInstance().removeFromList ( customerRec, TypeList.TYPE_CUSTOMER ) ;
               }
            }
         }

      }
      return( rc ) ;
   }


   /**
     *  This method will edit the item passed in
     *
     * @param item This is a TypeCustomerRec object to edit
     **/
   public boolean edit ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeCustomerRec ) ) {
         TypeCustomerRec customerRec     = (TypeCustomerRec) item ;
         String          current         = customerRec.toString() ;
         String          newDescript     = new SingleEntryDlg ( MainWindow.getInstance(),
                                                                Str.getStr ( STR_TYPE_DATE ),
                                                                Str.getStr ( STR_PLEASE_ENTER_CUSTOMER ),
                                                                customerRec.toString(),
                                                                "" ).getResult() ;

         if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {
            int saveRC = 0 ;

            customerRec.updateRecStatus ( customerRec.REC_NOCHANGE ) ;
            customerRec.setDescript ( newDescript ) ;
            saveRC = customerRec.writeToDatabase() ;

            if ( saveRC != 0 ) {
               LogSystem.log ( 1, "Could not change TypeCustomer to description "+newDescript+". SQLCode:"+saveRC ) ;
               customerRec.setDescript ( current ) ;
               GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            } else
               rc = true ;
         }
      }

      return( rc ) ;
   }


   /**
     *  This method will return the authorization level needed
     * to edit this list
     **/
   public int getAuthority () {
      return( AUTH_KEYWORD_TYPE_CUSTOMER ) ;
   }


   /**
     *  This method will return a list of TypeCustomerRec objects
     *
     * @return Vector
     **/
   public Vector getKeywords () {
      Vector list = TypeList.getInstance().getTypeList ( TypeList.TYPE_CUSTOMER ) ;
      return( list ) ;
   }


   /**
     *  This method will return our single column's title
     *
     * @return String[]
     **/
   public String[] getTitles () {
      String[] titles = { Str.getStr ( STR_DESCRIPTION ), "Customer Ind"} ;

      return( titles  );
   }


   /**
     *  The toString method must return the name as well
     **/
   public String toString () {
      return( getName() ) ;
   }


} /* TypeCustomerList */


