package com.ibm.nzna.projects.qit.admin.keyword.date ;    
/*-----------------------------------------------------------*/
/*   TypeDateList                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeDateList                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.admin.keyword.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.admin.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object represents the PRODUCT.TYPEDATE table for the Keyword Management Applet 
 *
 *
 * @version 1.0, 8/6/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeDateList                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeDateList extends Object implements AppConst, KeywordList, Authority, AdminConst {



   /**
     *  This method will return our name
     **/
   public String getName () {
      return( Str.getStr ( STR_TYPE_DATE ) ) ;
   }


   /**
     *  This method will add an item to our date list
     **/
   public Object add () {
      TypeDateRec dateRec     = null ;
      String      newDescript = new SingleEntryDlg ( MainWindow.getInstance(),
                                                     Str.getStr ( STR_TYPE_DATE ),
                                                     Str.getStr ( STR_PLEASE_ENTER_TYPE_DATE ),
                                                     "",
                                                     "" ).getResult() ;

      if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {

         if ( TypeList.getInstance().objectFromString ( newDescript, TypeList.TYPE_DATE ) == null ) {
            int newInd = Counter.getCounter ( LogSystem.getInstance(), "TYPEDATE" ) ;

            if ( newInd > 0 ) {
               int         saveRC  = 0 ;

               dateRec = new TypeDateRec ( newInd, newDescript ) ;
               dateRec.updateRecStatus ( dateRec.REC_INSERT ) ;
               saveRC = dateRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Could not create new TypeDate of "+newDescript+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                  dateRec = null ;
               } else {
                  TypeList.getInstance().addToList ( dateRec, TypeList.TYPE_DATE ) ;
               }
            }
         }
      } else
         GUISystem.printBox ( STR_INFO, STR_THAT_ITEM_ALREADY_EXISTS_IN_KEYWORD_LIST ) ;

      return( dateRec ) ;
   }


   /**
     *  This method will remove an item from our list
     **/
   public boolean delete ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeDateRec ) ) {
         TypeDateRec dateRec = (TypeDateRec) item ;

         if ( dateRec.getInd() == 0 )
            GUISystem.printBox ( STR_INFO, STR_CANNOT_DELETE_DEFAULT ) ;
         else {
            if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_TYPE_DATE ) ) {
               int saveRC = 0 ;

               dateRec.updateRecStatus ( dateRec.REC_DELETE ) ;
               saveRC = dateRec.writeToDatabase () ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Cannot delete TypeDate "+dateRec.toString()+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
               } else {
                  rc = true ;
                  TypeList.getInstance().removeFromList ( dateRec, TypeList.TYPE_DATE ) ;
               }
            }
         } /* else  */

      }
      return( rc ) ;
   }


   /**
     *  This method will edit the item passed in
     *
     * @param item This is a TypeDateRec object to edit
     **/
   public boolean edit ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeDateRec ) ) {
         TypeDateRec dateRec     = (TypeDateRec) item ;
         String      current     = dateRec.toString() ;
         String      newDescript = new SingleEntryDlg ( MainWindow.getInstance(),
                                                        Str.getStr ( STR_TYPE_DATE ),
                                                        Str.getStr ( STR_PLEASE_ENTER_TYPE_DATE ),
                                                        dateRec.toString(),
                                                        "" ).getResult() ;

         if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {
            int saveRC = 0 ;

            dateRec.updateRecStatus ( dateRec.REC_NOCHANGE ) ;
            dateRec.setDescript ( newDescript ) ;
            saveRC = dateRec.writeToDatabase() ;

            if ( saveRC != 0 ) {
               LogSystem.log ( 1, "Could not change TypeDate to description "+newDescript+". SQLCode:"+saveRC ) ;
               dateRec.setDescript ( current ) ;
               GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            } else
               rc = true ;
         }
      }

      return( rc ) ;
   }


   /**
     *  This method will return the authorization level needed
     * to edit this list
     **/
   public int getAuthority () {
      return( AUTH_KEYWORD_TYPE_DATE ) ;
   }


   /**
     *  This method will return a list of TypeDateRec objects
     *
     * @return Vector
     **/
   public Vector getKeywords () {
      Vector list = TypeList.getInstance().getTypeList ( TypeList.TYPE_DATE ) ;
      return( list ) ;
   }


   /**
     *  This method will return our single column's title
     *
     * @return String[]
     **/
   public String[] getTitles () {
      String[] titles = { Str.getStr ( STR_DESCRIPTION ), "Date Ind"} ;

      return( titles  );
   }


   /**
     *  The toString method must return the name as well
     **/
   public String toString () {
      return( getName() ) ;
   }


} /* TypeDateList */




