package com.ibm.nzna.projects.qit.admin.keyword.offering ;
/*-----------------------------------------------------------*/
/*   TypeOfferingList                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeOfferingList                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.admin.keyword.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.admin.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object represents the PRODUCT.TYPEOFFERING   table for the Keyword Management Applet 
 *
 *
 * @version 1.0, 8/6/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeOfferingList                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeOfferingList extends Object implements AppConst, KeywordList, Authority, AdminConst {



   /**
     *  This method will return our name
     **/
   public String getName () {
      return( Str.getStr ( STR_TYPE_OFFERING ) ) ;
   }


   /**
     *  This method will add an item to our date list
     **/
   public Object add () {
      TypeOfferingRec offeringRec = null ;
      boolean         rc          = false ;
      String          newDescript = new SingleEntryDlg ( MainWindow.getInstance(),
                                                         Str.getStr ( STR_TYPE_OFFERING ),
                                                         Str.getStr ( STR_PLEASE_ENTER_OFFERING ),
                                                         "",
                                                         "" ).getResult() ;

      if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {

         if ( TypeList.getInstance().objectFromString ( newDescript, TypeList.TYPE_OFFERING ) == null ) {
            int newInd = Counter.getCounter ( LogSystem.getInstance(), "TYPEOFFER" ) ;

            if ( newInd > 0 ) {
               int         saveRC  = 0 ;

               offeringRec = new TypeOfferingRec ( newInd, newDescript ) ;
               offeringRec.updateRecStatus ( offeringRec.REC_INSERT ) ;
               saveRC = offeringRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Could not create new TypeOffering of "+newDescript+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                  offeringRec = null ;
               } else {
                  TypeList.getInstance().addToList ( offeringRec, TypeList.TYPE_OFFERING ) ;
               }
            }
         } else
            GUISystem.printBox ( STR_INFO, STR_THAT_ITEM_ALREADY_EXISTS_IN_KEYWORD_LIST ) ;
      }

      return( offeringRec ) ;
   }


   /**
     *  This method will remove an item from our list
     **/
   public boolean delete ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeOfferingRec ) ) {
         TypeOfferingRec offeringRec = (TypeOfferingRec) item ;

         if ( offeringRec.getInd() == 0 )
            GUISystem.printBox ( STR_INFO, STR_CANNOT_DELETE_DEFAULT ) ;
         else {
            if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_OFFERING ) ) {
               int saveRC = 0 ;

               offeringRec.updateRecStatus ( offeringRec.REC_DELETE ) ;
               saveRC = offeringRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Cannot delete Offering "+offeringRec.toString()+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
               } else {
                  rc = true ;
                  TypeList.getInstance().removeFromList ( offeringRec, TypeList.TYPE_OFFERING ) ;
               }
            }
         }

      }
      return( rc ) ;
   }


   /**
     *  This method will edit the item passed in
     *
     * @param item This is a TypeOfferingRec object to edit
     **/
   public boolean edit ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeOfferingRec ) ) {
         TypeOfferingRec offeringRec     = (TypeOfferingRec) item ;
         String          current         = offeringRec.toString() ;
         String          newDescript     = new SingleEntryDlg ( MainWindow.getInstance(),
                                                                Str.getStr ( STR_TYPE_OFFERING ),
                                                                Str.getStr ( STR_PLEASE_ENTER_OFFERING ),
                                                                offeringRec.toString(),
                                                                "" ).getResult() ;

         if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {
            int saveRC = 0 ;

            offeringRec.updateRecStatus ( offeringRec.REC_NOCHANGE ) ;
            offeringRec.setDescript ( newDescript ) ;
            saveRC = offeringRec.writeToDatabase() ;

            if ( saveRC != 0 ) {
               LogSystem.log ( 1, "Could not change TypeOffering to description "+newDescript+". SQLCode:"+saveRC ) ;
               offeringRec.setDescript ( current ) ;
               GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            } else
               rc = true ;
         }
      }

      return( rc ) ;
   }


   /**
     *  This method will return the authorization level needed
     * to edit this list
     **/
   public int getAuthority () {
      return( AUTH_KEYWORD_TYPE_OFFERING ) ;
   }


   /**
     *  This method will return a list of TypeOfferingRec objects
     *
     * @return Vector
     **/
   public Vector getKeywords () {
      Vector list = TypeList.getInstance().getTypeList ( TypeList.TYPE_OFFERING ) ;
      return( list ) ;
   }


   /**
     *  This method will return our single column's title
     *
     * @return String[]
     **/
   public String[] getTitles () {
      String[] titles = { Str.getStr ( STR_DESCRIPTION ), "Offering Ind"} ;

      return( titles  );
   }


   /**
     *  The toString method must return the name as well
     **/
   public String toString () {
      return( getName() ) ;
   }


} /* TypeOfferingList */


