package com.ibm.nzna.projects.qit.admin.keyword.product ;
/*-----------------------------------------------------------*/
/*   TypeProductList                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeProductList                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.admin.keyword.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.admin.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object represents the PRODUCT.TYPEPRODUCT   table for the Keyword Management Applet 
 *
 *
 * @version 1.0, 8/6/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeProductList                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeProductList extends Object implements AppConst, KeywordList, Authority, AdminConst {



   /**
     *  This method will return our name
     **/
   public String getName () {
      return( Str.getStr ( STR_TYPE_PRODUCT ) ) ;
   }


   /**
     *  This method will add an item to our date list
     **/
   public Object add () {
      TypeProductRec productRec  = null ;
      boolean        rc          = false ;
      String         newDescript = new SingleEntryDlg ( MainWindow.getInstance(),
                                                        Str.getStr ( STR_TYPE_PRODUCT ),
                                                        Str.getStr ( STR_PLEASE_ENTER_PRODUCT ),
                                                        "",
                                                        "" ).getResult() ;

      if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {

         if ( TypeList.getInstance().objectFromString ( newDescript, TypeList.TYPE_PRODUCT ) == null ) {
            int newInd = Counter.getCounter ( LogSystem.getInstance(), "TYPEPROD" ) ;

            if ( newInd > 0 ) {
               int         saveRC  = 0 ;
               
               
               productRec = new TypeProductRec ( newInd, newDescript ) ;
               productRec.updateRecStatus ( productRec.REC_INSERT ) ;
               saveRC = productRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Could not create new TypeProduct of "+newDescript+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                  productRec = null ;
               } else {
                  TypeList.getInstance().addToList ( productRec, TypeList.TYPE_PRODUCT ) ;
               }
            }
         } else
            GUISystem.printBox ( STR_INFO, STR_THAT_ITEM_ALREADY_EXISTS_IN_KEYWORD_LIST ) ;
      }

      return( productRec ) ;
   }


   /**
     *  This method will remove an item from our list
     **/
   public boolean delete ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeProductRec ) ) {
         TypeProductRec productRec = (TypeProductRec) item ;

         if ( productRec.getInd() == 0 )
            GUISystem.printBox ( STR_INFO, STR_CANNOT_DELETE_DEFAULT ) ;
         else {
            if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_PRODUCT ) ) {
               int saveRC = 0 ;

               productRec.updateRecStatus ( productRec.REC_DELETE ) ;
               saveRC = productRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Cannot delete Product "+productRec.toString()+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
               } else {
                  rc = true ;
                  TypeList.getInstance().removeFromList ( productRec, TypeList.TYPE_PRODUCT ) ;
               }
            }
         }

      }
      return( rc ) ;
   }


   /**
     *  This method will edit the item passed in
     *
     * @param item This is a TypeProductRec object to edit
     **/
   public boolean edit ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeProductRec ) ) {
         TypeProductRec productRec     = (TypeProductRec) item ;
         String          current         = productRec.toString() ;
         String          newDescript     = new SingleEntryDlg ( MainWindow.getInstance(),
                                                                Str.getStr ( STR_TYPE_DATE ),
                                                                Str.getStr ( STR_PLEASE_ENTER_PRODUCT ),
                                                                productRec.toString(),
                                                                "" ).getResult() ;

         if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {
            int saveRC = 0 ;

            productRec.updateRecStatus ( productRec.REC_NOCHANGE ) ;
            productRec.setDescript ( newDescript ) ;
            saveRC = productRec.writeToDatabase() ;

            if ( saveRC != 0 ) {
               LogSystem.log ( 1, "Could not change TypeProduct to description "+newDescript+". SQLCode:"+saveRC ) ;
               productRec.setDescript ( current ) ;
               GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            } else
               rc = true ;
         }
      }

      return( rc ) ;
   }


   /**
     *  This method will return the authorization level needed
     * to edit this list
     **/
   public int getAuthority () {
      return( AUTH_KEYWORD_TYPE_PRODUCT ) ;
   }


   /**
     *  This method will return a list of TypeProductRec objects
     *
     * @return Vector
     **/
   public Vector getKeywords () {
      Vector list = TypeList.getInstance().getTypeList ( TypeList.TYPE_PRODUCT ) ;
      return( list ) ;
   }


   /**
     *  This method will return our single column's title
     *
     * @return String[]
     **/
   public String[] getTitles () {
      String[] titles = { Str.getStr ( STR_DESCRIPTION ), "Product Ind"} ;

      return( titles  );
   }


   /**
     *  The toString method must return the name as well
     **/
   public String toString () {
      return( getName() ) ;
   }


} /* TypeProductList */




