package com.ibm.nzna.projects.qit.admin.keyword.warranty ;
/*-----------------------------------------------------------*/
/*   TypeWarrantyList                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TypeWarrantyList                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.admin.keyword.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.admin.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/**
 * This object represents the PRODUCT.TYPEWARRANTY   table for the Keyword Management Applet 
 *
 *
 * @version 1.0, 8/6/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TypeWarrantyList                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class TypeWarrantyList extends Object implements AppConst, KeywordList, Authority, AdminConst {



   /**
     *  This method will return our name
     **/
   public String getName () {
      return( Str.getStr ( STR_TYPE_WARRANTY ) ) ;
   }


   /**
     *  This method will add an item to our date list
     **/
   public Object add () {
      TypeWarrantyRec warrantyRec = null ;
      String          newDescript = new SingleEntryDlg ( MainWindow.getInstance(),
                                                         Str.getStr ( STR_TYPE_WARRANTY ),
                                                         Str.getStr ( STR_PLEASE_ENTER_WARRANTY ),
                                                         "",
                                                         new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 3 ) ).getResult() ;

      if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {
         String newLongDescript = new SingleEntryDlg ( MainWindow.getInstance(),
                                                       Str.getStr ( STR_TYPE_WARRANTY ),
                                                       Str.getStr ( STR_PLEASE_ENTER_WARRANTY )+" (Long Description)",
                                                       "",
                                                       new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;


         if ( ( newLongDescript != null ) && ( newLongDescript.length() > 0 ) ) {
            if ( TypeList.getInstance().objectFromString ( newDescript, TypeList.TYPE_WARRANTY ) == null ) {
               int newInd = Counter.getCounter ( LogSystem.getInstance(), "TYPEWARR" ) ;

               if ( newInd > 0 ) {
                  int         saveRC  = 0 ;

                  warrantyRec = new TypeWarrantyRec ( newInd, newDescript ) ;
                  warrantyRec.setLongDescription ( newLongDescript ) ;
                  warrantyRec.updateRecStatus ( warrantyRec.REC_INSERT ) ;
                  saveRC = warrantyRec.writeToDatabase() ;

                  if ( saveRC != 0 ) {
                     LogSystem.log ( 1, "Could not create new TypeWarranty of "+newDescript+". SQLCode:"+saveRC ) ;
                     GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                     warrantyRec = null ;
                  } else {
                     TypeList.getInstance().addToList ( warrantyRec, TypeList.TYPE_WARRANTY ) ;
                  }

               } /* if  */
            } /* if  */
         } else
            GUISystem.printBox ( STR_INFO, STR_THAT_ITEM_ALREADY_EXISTS_IN_KEYWORD_LIST ) ;
      }

      return( warrantyRec ) ;
   }


   /**
     *  This method will remove an item from our list
     **/
   public boolean delete ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeWarrantyRec ) ) {
         TypeWarrantyRec warrantyRec = (TypeWarrantyRec) item ;

         if ( warrantyRec.getInd() == 0 )
            GUISystem.printBox ( STR_INFO, STR_CANNOT_DELETE_DEFAULT ) ;
         else {
            if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_WARRANTY ) ) {
               int saveRC = 0 ;

               warrantyRec.updateRecStatus ( warrantyRec.REC_DELETE ) ;
               saveRC = warrantyRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Cannot delete Warranty "+warrantyRec.toString()+". SQLCode:"+saveRC ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
               } else {
                  rc = true ;
                  TypeList.getInstance().removeFromList ( warrantyRec, TypeList.TYPE_WARRANTY ) ;
               }
            }
         }

      }
      return( rc ) ;
   }


   /**
     *  This method will edit the item passed in
     *
     * @param item This is a TypeWarrantyRec object to edit
     **/
   public boolean edit ( Object item ) {
      boolean rc = false ;

      if ( ( item != null ) && ( item instanceof TypeWarrantyRec ) ) {
         TypeWarrantyRec warrantyRec     = (TypeWarrantyRec) item ;
         String          current         = warrantyRec.toString() ;
         String          newDescript     = new SingleEntryDlg ( MainWindow.getInstance(),
                                                                "Warranty",
                                                                Str.getStr ( STR_PLEASE_ENTER_WARRANTY )+" (Short Descript)" ,
                                                                warrantyRec.toString(),
                                                                "" ).getResult() ;

         if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {
            String longDescript = new SingleEntryDlg ( MainWindow.getInstance(),
                                                       "Warranty",
                                                       Str.getStr ( STR_PLEASE_ENTER_WARRANTY )+" (Long Descript)" ,
                                                       warrantyRec.getLongDescription(),
                                                       "" ).getResult() ;
            if ( ( longDescript != null ) && ( longDescript.length() > 0 ) ) {
               int saveRC = 0 ;

               warrantyRec.updateRecStatus ( warrantyRec.REC_NOCHANGE ) ;
               warrantyRec.setDescript ( newDescript ) ;
               warrantyRec.setLongDescription ( longDescript ) ;
               saveRC = warrantyRec.writeToDatabase() ;

               if ( saveRC != 0 ) {
                  LogSystem.log ( 1, "Could not change TypeWarranty to description "+newDescript+". SQLCode:"+saveRC ) ;
                  warrantyRec.setDescript ( current ) ;
                  GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
               } else
                  rc = true ;
            }
         }
      }

      return( rc ) ;
   }


   /**
     *  This method will return the authorization level needed
     * to edit this list
     **/
   public int getAuthority () {
      return( AUTH_KEYWORD_TYPE_WARRANTY ) ;
   }


   /**
     *  This method will return a list of TypeWarrantyRec objects
     *
     * @return Vector
     **/
   public Vector getKeywords () {
      Vector list = TypeList.getInstance().getTypeList ( TypeList.TYPE_WARRANTY ) ;
      return( list ) ;
   }


   /**
     *  This method will return our single column's title
     *
     * @return String[]
     **/
   public String[] getTitles () {
      String[] titles = { "Warranty Ind", Str.getStr ( STR_DESCRIPTION ), "Long"+Str.getStr ( STR_DESCRIPTION )} ;

      return( titles  );
   }


   /**
     *  The toString method must return the name as well
     **/
   public String toString () {
      return( getName() ) ;
   }


} /* TypeWarrantyList */


