package com.ibm.nzna.projects.qit.admin.pers ;     
/*-----------------------------------------------------------*/
/*   EditUserPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EditUserPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import java.awt.event.* ;
import java.awt.* ;
import java.util.* ;
/**
 * This panel holds all of the fields needed to edit a UserRec object. It
 * is capable of Creating a new user and editing an existing user
 *
 *
 * @version 1.0, 4/5/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EditUserPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class EditUserPanel extends JPanel implements ActionListener, QuestPanel, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Members                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel         st_NAME            = null ;
   private JLabel         st_USERID          = null ;
   private JLabel         st_PHONE           = null ;
   private JLabel         st_EMAIL           = null ;
   private JLabel         st_TEAM            = null ;
   private JLabel         st_COUNTRY         = null ;
   private JLabel         st_LANGUAGE        = null ;
   private JLabel         st_MANAGER         = null ;
   private JLabel         st_BRANDPM         = null ;
   private JLabel         st_BACKUP          = null ;
   private JLabel         st_REQ_FIELDS      = null ;
   private JTextField     ef_EMAIL           = null ;
   private JCheckBox      ck_ISMANAGER       = null ;
   private JCheckBox      ck_ISBPM           = null ;
   private JComboBox      cb_LANGUAGE        = null ;
   private JList          lb_TEAMS           = null ;
   private JList          lb_COUNTRY         = null ;
   private JList          lb_BRANDPM         = null ;
   private JTextField     ef_NAME            = null ;
   private JTextField     ef_USERID          = null ;
   private JTextField     ef_PHONE           = null ;
   private UserEntryPanel ef_BACKUP          = null ;
   private UserEntryPanel ef_MANAGER         = null ;
   private JScrollPane    scr_TEAMS          = null ;
   private JScrollPane    scr_COUNTRY        = null ;
   private JScrollPane    scr_BRANDPM        = null ;
   private ButtonPanel    teamButtonPanel    = null ;
   private ButtonPanel    countryButtonPanel = null ;
   private ButtonPanel    brandButtonPanel   = null ;
   private HotLinkLabel   pb_ADD_TEAM        = null ;
   private HotLinkLabel   pb_REMOVE_TEAM     = null ;
   private HotLinkLabel   pb_ADD_COUNTRY     = null ;
   private HotLinkLabel   pb_REMOVE_COUNTRY  = null ;
   private HotLinkLabel   pb_ADD_BRAND       = null ;
   private HotLinkLabel   pb_REMOVE_BRAND    = null ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private UserRec        userRec          = null ;       // This is the default userRec that we are using to edit
   private JScrollPane    scrollPane       = null ;       // The scroll pane holding the contentPane
   private JPanel         contentPane      = null ;       // The Content Panel holding all of the GUI controls
   private ActionButton   pb_CLOSE         = null ;       // Closes the panel
   private ActionButton   pb_SAVE          = null ;       // Saves the user
   private PersPanel      persPanel        = null ;       // The PersPanel that launched us, it may be null
   private UserRec        origUserRec      = null ;       // The original User record
   private boolean        saveWhileClosing = true ;       // If TRUE, the panel will try to save when closed



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor assigns the User Record
    *  that we will be editing.
    *  
    *
    *  @param userRec The UserRec to start editing when we show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EditUserPanel ( UserRec userRec ) {
      setUserRec ( userRec ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor assigns the User Record
    *  that we will be editing.
    *  
    *
    *  @param userRec The UserRec to start editing when we show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EditUserPanel ( PersPanel persPanel, UserRec userRec ) {
      this.persPanel = persPanel ;
      setUserRec ( userRec ) ;
   }





   /*------------------------------------------------------------*/
   /* intialize
    */
   /**
    *    This method will be called first, and we will
    *  initialize our component here.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if init went Ok, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean initialize () {
      Vector tempVec = null ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_NAME            = new JLabel ( "* "+Str.getStr ( STR_NAME ) ) ;
      st_USERID          = new JLabel ( "* "+Str.getStr ( STR_USERID ) ) ;
      st_PHONE           = new JLabel ( "* "+Str.getStr ( STR_PHONE ) ) ;
      st_EMAIL           = new JLabel ( "* "+Str.getStr ( STR_EMAIL_ADDRESS ) ) ;
      st_TEAM            = new JLabel ( "* "+Str.getStr ( STR_TEAM ) ) ;
      st_COUNTRY         = new JLabel ( "* "+Str.getStr ( STR_COUNTRY ) ) ;
      st_LANGUAGE        = new JLabel ( "* "+Str.getStr ( STR_LANGUAGE ) ) ;
      st_MANAGER         = new JLabel ( "* "+Str.getStr ( STR_MANAGER ) ) ;
      st_BRANDPM         = new JLabel ( Str.getStr ( STR_BRAND_PROGRAM_MANAGER ) ) ;
      st_BACKUP          = new JLabel ( "* "+Str.getStr ( STR_BACK_UP ) ) ;
      ck_ISMANAGER       = new JCheckBox ( Str.getStr ( STR_IS_A_MANAGER ) ) ;
      ck_ISBPM           = new JCheckBox ( Str.getStr ( STR_BRAND_PROGRAM_MANAGER ) ) ;
      cb_LANGUAGE        = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_LANGUAGE ) ) ;
      lb_TEAMS           = new JList () ;
      lb_COUNTRY         = new JList () ;
      lb_BRANDPM         = new JList () ;
      ef_NAME            = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 50 ), "", 0 ) ;
      ef_USERID          = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 8 ), "", 0 ) ;
      ef_PHONE           = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 17 ), "", 0 ) ;
      ef_BACKUP          = new UserEntryPanel () ;
      ef_MANAGER         = new UserEntryPanel () ;
      ef_EMAIL           = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 50), "", 0 ) ;
      scr_TEAMS          = new JScrollPane ( lb_TEAMS ) ;
      scr_COUNTRY        = new JScrollPane ( lb_COUNTRY ) ;
      scr_BRANDPM        = new JScrollPane ( lb_BRANDPM ) ;
      contentPane        = new JPanel () ;
      scrollPane         = new JScrollPane ( contentPane ) ;
      teamButtonPanel    = new ButtonPanel () ;
      countryButtonPanel = new ButtonPanel () ;
      brandButtonPanel   = new ButtonPanel () ;
      pb_ADD_TEAM        = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE_TEAM     = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_ADD_COUNTRY     = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE_COUNTRY  = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_ADD_BRAND       = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE_BRAND    = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;  
      st_REQ_FIELDS      = new JLabel ( "* = Required Field" ) ;

      pb_CLOSE    = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_SAVE     = new ActionButton ( Str.getStr ( STR_SAVE ),  ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SAVE_USER ) ) ;                        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_REQ_FIELDS.setFont         ( FontSystem.smallTitleFont ) ;
      teamButtonPanel.setBorder     ( GUISystem.grayBorder ) ;
      teamButtonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      teamButtonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                      ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      countryButtonPanel.setBorder     ( GUISystem.grayBorder ) ;
      countryButtonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      countryButtonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                         ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      brandButtonPanel.setBorder     ( GUISystem.grayBorder ) ;
      brandButtonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      brandButtonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                       ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      ef_MANAGER.setShowManagerOnly ( true ) ;
      scr_TEAMS.setBorder ( GUISystem.blackBorder ) ;
      scr_COUNTRY.setBorder ( GUISystem.blackBorder ) ;
      scr_BRANDPM.setBorder ( GUISystem.blackBorder ) ;
      contentPane.setBackground ( Color.white ) ;
      scrollPane.setBackground ( Color.white ) ;
      ck_ISMANAGER.setBackground ( Color.white ) ;
      ck_ISBPM.setBackground ( Color.white ) ;
      setBackground ( Color.white ) ;
      pb_ADD_TEAM.showSelection ( false ) ;
      pb_REMOVE_TEAM.showSelection ( false ) ;
      pb_ADD_COUNTRY.showSelection ( false ) ;
      pb_REMOVE_COUNTRY.showSelection ( false ) ;
      pb_ADD_BRAND.showSelection ( false ) ;
      pb_REMOVE_BRAND.showSelection ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SAVE.addActionListener           ( this ) ;
      pb_CLOSE.addActionListener          ( this ) ;
      pb_ADD_TEAM.addActionListener       ( this ) ;
      pb_REMOVE_TEAM.addActionListener    ( this ) ;
      pb_ADD_COUNTRY.addActionListener    ( this ) ;
      pb_REMOVE_COUNTRY.addActionListener ( this ) ;
      pb_ADD_BRAND.addActionListener      ( this ) ;
      pb_REMOVE_BRAND.addActionListener   ( this ) ;
      ck_ISBPM.addActionListener          ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our buttons                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      teamButtonPanel.add ( pb_ADD_TEAM ) ;
      teamButtonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      teamButtonPanel.add ( pb_REMOVE_TEAM ) ;
      countryButtonPanel.add ( pb_ADD_COUNTRY ) ;
      countryButtonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      countryButtonPanel.add ( pb_REMOVE_COUNTRY ) ;
      brandButtonPanel.add ( pb_ADD_BRAND ) ;
      brandButtonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      brandButtonPanel.add ( pb_REMOVE_BRAND ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI to the panel                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_NAME ) ;
      contentPane.add ( ef_NAME ) ;
      contentPane.add ( st_USERID ) ;
      contentPane.add ( ef_USERID ) ;
      contentPane.add ( st_PHONE  ) ;
      contentPane.add ( ef_PHONE  ) ;
      contentPane.add ( st_BACKUP ) ;
      contentPane.add ( ef_BACKUP ) ;
      contentPane.add ( st_MANAGER ) ;
      contentPane.add ( ck_ISMANAGER ) ;
      contentPane.add ( ef_MANAGER ) ;
      contentPane.add ( st_LANGUAGE ) ;
      contentPane.add ( cb_LANGUAGE ) ;
      contentPane.add ( st_EMAIL ) ;
      contentPane.add ( ef_EMAIL ) ;
      contentPane.add ( st_BRANDPM ) ;
      contentPane.add ( ck_ISBPM ) ;
      contentPane.add ( scr_BRANDPM ) ;
      contentPane.add ( brandButtonPanel ) ;
      contentPane.add ( st_COUNTRY ) ;
      contentPane.add ( scr_COUNTRY ) ;
      contentPane.add ( countryButtonPanel ) ;
      contentPane.add ( st_TEAM ) ;
      contentPane.add ( scr_TEAMS ) ;
      contentPane.add ( teamButtonPanel ) ;
      contentPane.add ( st_REQ_FIELDS ) ;
      contentPane.setMinimumSize ( new Dimension ( 100, 640 ) ) ;
      contentPane.setPreferredSize ( new Dimension ( 100, 640 ) ) ;

      add ( scrollPane, BorderLayout.CENTER ) ;

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_PERSONEL ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return 
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_USER_EDIT ) ) ;
   }

   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing right now
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing right now
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing right now
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we have
    *  been created and are showing. We will
    *  use this time to refresh our data
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_SAVE ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refresh() ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default navigation
    *  panel. Currently that is nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called last when we
    *  are destroyed. It is time to clean up here
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when we
    *  need to be shutdown. If the user needs to
    *  save their changes before shutting down, we can do that here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = It is ok to shutdown, FALSE = It is not
    *//*---------------------------------------------------------*/
   public boolean stop () {
      if ( saveWhileClosing ) {
         //if ( userRec.getRecStatus() != userRec.REC_NOCHANGE ) {
            if ( GUISystem.printBox ( "Question", "Would you like to save this user record?" ) )
               return( saveData() ) ;
            else
               userRec.setData ( origUserRec ) ;
         //}
      }

      if ( persPanel != null )
         persPanel.refresh() ;
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh the data on our screen
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh  () {
      if ( userRec != null ) {
         DefaultListModel listModel = null ;

         if ( userRec.getRecStatus() == userRec.REC_INSERT )
            ef_USERID.setEnabled ( true ) ;
         else
            ef_USERID.setEnabled ( false ) ;

         ef_NAME.setText ( userRec.getName() ) ;
         ef_USERID.setText ( userRec.getUserId() ) ;
         ef_PHONE.setText ( userRec.getPhone() ) ;
         ef_MANAGER.setText ( userRec.getManagerUserId() ) ;
         ef_BACKUP.setText ( userRec.getBackUp() ) ;
         ef_EMAIL.setText ( userRec.getEMail() ) ;
         cb_LANGUAGE.setSelectedItem ( TypeList.getInstance().objectFromInd ( userRec.getLanguage(), TypeList.TYPE_LANGUAGE ) ) ;

         ck_ISMANAGER.setSelected ( userRec.isManager() ) ;
         ck_ISBPM.setSelected ( (userRec.getBrandMgrVec() != null ) && ( userRec.getBrandMgrVec().size() > 0 ) ) ;

         refreshLists() ;

      }
   }





   /*------------------------------------------------------------*/
   /* refreshLists
    */
   /**
    *    This method will refresh all of our list boxes
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshLists () {
      DefaultListModel listModel = null ;

      listModel = new DefaultListModel () ;
      lb_TEAMS.setModel ( listModel ) ;
      populateListModel ( listModel, userRec.getTeams(), TypeList.TYPE_TEAM ) ;

      listModel = new DefaultListModel () ;
      lb_COUNTRY.setModel ( listModel ) ;
      populateListModel ( listModel, userRec.getCountries(), TypeList.TYPE_COUNTRYCODE ) ;

      listModel = new DefaultListModel () ;
      lb_BRANDPM.setModel ( listModel ) ;
      populateListModel ( listModel, userRec.getBrandMgrVec() ) ;

      lb_BRANDPM.setEnabled ( ck_ISBPM.isSelected() ) ;
      pb_ADD_BRAND.setEnabled ( ck_ISBPM.isSelected() ) ;
      pb_REMOVE_BRAND.setEnabled ( ck_ISBPM.isSelected() ) ;

   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to events being fired
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin ( this ).closePanel ( this, persPanel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SAVE = Saves all of our data                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE ) {
         if ( saveData () ) {
            try {
               if ( PropertySystem.getBool ( PropertySystem.PROPERTY_SAVE_AND_CLOSE ) ) {
                  saveWhileClosing = false ;
                  GUISystem.getParentDefWin ( this ).closePanel ( this, persPanel ) ;
               }
            } catch ( Exception ex ) {
            }
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_TEAM                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_TEAM )
         addTeam () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_TEAM                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_TEAM )
         removeTeam() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_COUNTRY                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_COUNTRY )
         addCountry() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_COUNTRY                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_COUNTRY )
         removeCountry () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_BRAND                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_BRAND )
         addBrandMgr() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_BRAND                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_BRAND )
         removeBrandMgr() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ck_ISBPM                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ck_ISBPM ) {
         if ( ck_ISBPM.isSelected() ) {
            lb_BRANDPM.setEnabled ( true ) ;
            pb_ADD_BRAND.setEnabled ( true ) ;
            pb_REMOVE_BRAND.setEnabled ( true ) ;
         } else
            removeAllBrandMgr() ;
      }




   }



   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save all of the data on the
    *  screen and will save the data to the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      boolean passed = false ;
      String  userId = null ;

      if ( userRec != null ) {
         String           name    = ef_NAME.getText() ;
         String           phone   = ef_PHONE.getText() ;
         TypeLanguageRec  langRec = (TypeLanguageRec) cb_LANGUAGE.getSelectedItem() ;
         String           backUp  = ef_BACKUP.getUserId() ;
         String           manager = ef_MANAGER.getUserId() ;
         String           eMail   = ef_EMAIL.getText() ;

         userId  = ef_USERID.getText() ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   We do not need to save the countries, brands or         */
         /* teams here because that was done when the user adjusted   */
         /* them. We only need to check to make sure there are any    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( userRec.getCountries() == null ) ||
              ( userRec.getCountries().length == 0 ) )
            GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_COUNTRY ) ;

         else if ( ( userRec.getTeams() == null ) ||
                   ( userRec.getTeams().length == 0 ) )
            GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_TEAM ) ;

         else if ( ( name == null ) || ( name.length() == 0 ) )
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_NAME ) ;

         else if ( ( userId == null ) || ( userId.length() == 0 ) )
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_USERID ) ;

         else if ( ( phone == null ) || ( phone.length() == 0 ) )
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_PHONE_NUM ) ;

         else if ( ( backUp == null ) || ( backUp.length() == 0 ) )
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_BACKUP ) ;

         else if ( ( eMail == null ) || ( eMail.length() == 0 ) )
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_EMAIL_ADDR ) ;

         else if ( ( manager == null ) || ( manager.length() == 0 ) )
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_MANAGER ) ;

         else if ( langRec == null )
            GUISystem.printBox ( Str.getStr ( STR_INFO ), "Please enter the language" ) ;

         else {
            userRec.setUserId ( userId ) ;
            userRec.setName ( name ) ;
            userRec.setPhone ( phone ) ;
            userRec.setBackUp ( backUp ) ;
            userRec.setManagerUserId ( manager ) ;
            userRec.setLanguage ( langRec.getInd() );                                         
            userRec.setIsManager ( ck_ISMANAGER.isSelected() ) ;
            userRec.setEMail ( eMail ) ;               

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the initial password                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( userRec.getRecStatus() == userRec.REC_INSERT )
               userRec.setPassword ( userId ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, check to see if this user id is already in the      */
            /* database                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( userRec.getRecStatus() == userRec.REC_INSERT ) &&
                 ( UserSystem.getUserRecFromUserId ( userRec.getUserId() ) != null ) ) {
               GUISystem.printBox ( STR_INFO, STR_USER_ALREADY_EXISTS ) ;
               passed = false ;
            } else
               passed = true ;

         } /* else */

      }

      if ( passed ) {
         int recStatus = userRec.getRecStatus() ;

         if ( userRec.saveToDatabase() ) {

            if ( recStatus == userRec.REC_INSERT ) {
               recStatus = userRec.REC_UPDATE ;
               userRec.setToString ( userId ) ;
               //UserSystem.addUserRec ( userRec ) ;
            }

            userRec.updateRecStatus ( userRec.REC_NOCHANGE ) ;
         } else
            passed = false ;
      }
      return( passed ) ;
   }




   /*------------------------------------------------------------*/
   /* setUserRec
    */
   /**
    *    This method will set the user record that we
    *  are using
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setUserRec ( UserRec userRec ) {
      this.userRec     = userRec ;
      this.origUserRec = (UserRec)userRec.clone() ;
   }

   public boolean bookmark () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout ( ) {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ; 
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_REQ_FIELDS.setBounds ( x, y, size.width-(x*3), rowHeight ) ;
      y+=rowHeight ;


      st_USERID.setBounds ( x, y, 200, rowHeight ) ;
      st_NAME.setBounds ( x+350, y, 200, rowHeight ) ;
      y+=rowHeight ;

      ef_USERID.setBounds ( x, y, 200, rowHeight ) ;
      ef_NAME.setBounds ( x+350, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_BACKUP.setBounds ( x, y, 200, rowHeight ) ;
      ck_ISMANAGER.setBounds ( x+350, y, 200, rowHeight ) ;
      y+=rowHeight ;

      ef_BACKUP.setBounds ( x, y, 200, rowHeight ) ;
      ef_MANAGER.setBounds ( x+350, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_PHONE.setBounds ( x, y, 200, rowHeight ) ;
      st_LANGUAGE.setBounds ( x+350, y, 200, rowHeight ) ;
      y+=rowHeight ;

      ef_PHONE.setBounds ( x, y, 200, rowHeight ) ;
      cb_LANGUAGE.setBounds ( x+350, y, 200, rowHeight ) ;
      y+=(rowHeight) ;

      st_EMAIL.setBounds ( x, y, 200, rowHeight ) ;
      y+=rowHeight ;

      ef_EMAIL.setBounds ( x, y, 200, rowHeight ) ;
      y+=(rowHeight*2) ;


      st_BRANDPM.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;
      ck_ISBPM.setBounds ( x, y, 250, rowHeight ) ;
      y+=rowHeight ;
      scr_BRANDPM.setBounds ( x, y, 250, rowHeight*5 ) ;
      y+=(rowHeight*5) ;
      brandButtonPanel.setBounds ( x, y, 250, rowHeight ) ;
      y+=(rowHeight*2) ;

      st_COUNTRY.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;
      scr_COUNTRY.setBounds ( x, y, 250, rowHeight*5 ) ;
      y+=(rowHeight*5);
      countryButtonPanel.setBounds ( x, y, 250, rowHeight ) ;
      y+=(rowHeight*2) ;

      st_TEAM.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;
      scr_TEAMS.setBounds ( x, y, 250, rowHeight*5 ) ;
      y+=(rowHeight*5) ;
      teamButtonPanel.setBounds ( x, y, 250, rowHeight ) ;
      y+=(rowHeight*2) ;


   } /* doLayout */



   /*------------------------------------------------------------*/
   /* addTeam
    */
   /**
    *    This method will add a team to the user
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addTeam () {
      ListDlg listDlg = new ListDlg ( GUISystem.getParentDefWin ( this ), 
                                      Str.getStr ( STR_ADD_TEAM ), 
                                      TypeList.getInstance().getTypeList ( TypeList.TYPE_TEAM ),
                                      GUISystem.getFontUtil() ) ;
      Vector  addVec  = null ;                                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the properties of the list dlg and show it       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listDlg.setOkText     ( Str.getStr ( STR_OK ) ) ;
      listDlg.setCancelText ( Str.getStr ( STR_CANCEL ) );
      addVec = listDlg.getResult() ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add each member selected                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( addVec != null ) {
         int i    = 0 ;
         int size = addVec.size() ;

         while ( i < size ) {
            userRec.addTeam ( ((TypeTeamRec)addVec.elementAt(i)).getInd() ) ;
            i++ ;
         }
      }

      refreshLists() ;

   }




   /*------------------------------------------------------------*/
   /* removeTeam
    */
   /**
    *    This method will remove all selected teams
    *  from the user
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeTeam () {
      Object[] vec = lb_TEAMS.getSelectedValues() ;

      if ( ( vec != null ) && ( vec.length > 0 ) ) {
         int i    = 0 ;
         int size = vec.length ;

         while ( i < size ) {
            userRec.removeTeam ( ((TypeTeamRec)vec[i]).getInd() ) ;
            i++ ;
         }
      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

      refreshLists() ;
   }





   /*------------------------------------------------------------*/
   /* addCountry
    */
   /**
    *    This method will add a team to the user
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addCountry () {
      ListDlg listDlg = new ListDlg ( GUISystem.getParentDefWin ( this ), 
                                      Str.getStr ( STR_ADD_COUNTRY ), 
                                      TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ),
                                      GUISystem.getFontUtil() ) ;
      Vector  addVec  = null ;                                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the properties of the list dlg and show it       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listDlg.setOkText     ( Str.getStr ( STR_OK ) ) ;
      listDlg.setCancelText ( Str.getStr ( STR_CANCEL ) );
      addVec = listDlg.getResult() ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add each member selected                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( addVec != null ) {
         int i    = 0 ;
         int size = addVec.size() ;

         while ( i < size ) {
            userRec.addCountry ( ((TypeCountryCodeRec)addVec.elementAt(i)).getInd() ) ;
            i++ ;
         }
      }

      refreshLists() ;

   }





   /*------------------------------------------------------------*/
   /* removeCountry
    */
   /**
    *    This method will remove all selected
    *  countries from the user
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeCountry () {
      Object[] vec = lb_COUNTRY.getSelectedValues() ;

      if ( ( vec != null ) && ( vec.length > 0 ) ) {
         int i    = 0 ;
         int size = vec.length ;

         while ( i < size ) {
            userRec.removeCountry ( ((TypeCountryCodeRec)vec[i]).getInd() ) ;
            i++ ;
         }

      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

      refreshLists() ;
   } /* removeCountry  */




   /*------------------------------------------------------------*/
   /* populateListModel
    */
   /**
    *    This method will take the vector passed and
    *  will populate the given list model with what
    *  is in the vector
    *
    *  @param listModel The list model to populate
    *  @param vec The Vector of data
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void populateListModel ( DefaultListModel model, Vector vec ) {
      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int size = vec.size() ;
         int i    = 0 ;

         while ( i < size ) {
            model.addElement ( vec.elementAt(i) ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* populateListModel
    */
   /**
    *    This method will populate the given list model
    *  with TypeList Objects. The TypeList Objects
    *  are passed initially with only their indicators in the
    *  the indArray. 
    *
    *  @param listModel The List Model to populate
    *  @param indArray The array of Type List indicators
    *  @param typeList The TypeList to use
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void populateListModel ( DefaultListModel model, int[] indArray, int typeList ) {

      if ( ( indArray != null ) && ( indArray.length > 0 ) ) {
         TypeList typeListInstance = TypeList.getInstance() ;

         for ( int i = 0; i < indArray.length; i++ )
            model.addElement ( typeListInstance.objectFromInd ( indArray[i], typeList ) ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* removeBrandMgr
    */
   /**
    *    This method will remove a brand manager
    *  from the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeBrandMgr () {
      Object[] vec = lb_BRANDPM.getSelectedValues() ;

      if ( ( vec != null ) && ( vec.length > 0 ) ) {
         int i    = 0 ;
         int size = vec.length ;

         while ( i < size ) {
            userRec.removeBrandMgrRec ( ((BrandMgrRec)vec[i]) ) ;
            i++ ;
         }
      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

      refreshLists() ;
   }


   /*------------------------------------------------------------*/
   /* addBrandMgr
    */
   /**
    *    This method will add a brand mgr to our
    *  list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addBrandMgr () {
      Vector   typeVec  = new Vector ( 2 ) ;
      Vector   brandVec = null ;
      ListDlg  listDlg  = null ;

      typeVec.addElement ( new MultiListObject ( Str.getStr ( STR_DOCUMENT ) ) ) ;
      typeVec.addElement ( new MultiListObject ( Str.getStr ( STR_AVALON ) ) ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, get the brand                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listDlg = new ListDlg ( GUISystem.getParentDefWin ( this ),
                              Str.getStr ( STR_BRAND ),
                              Brands.getBrands(),
                              GUISystem.getFontUtil() ) ;
      listDlg.setOkText     ( Str.getStr ( STR_OK ) ) ;
      listDlg.setCancelText ( Str.getStr ( STR_CANCEL ) ) ;
      brandVec = listDlg.getResult() ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, get the type                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( brandVec != null ) {
         listDlg = new ListDlg ( GUISystem.getParentDefWin ( this ),
                                 Str.getStr ( STR_WHAT_BRAND_MGR_TYPE ),
                                 typeVec,
                                 GUISystem.getFontUtil() ) ;
         listDlg.setOkText     ( Str.getStr ( STR_OK ) ) ;
         listDlg.setCancelText ( Str.getStr ( STR_CANCEL ) ) ;
         typeVec = listDlg.getResult() ;

         if ( typeVec != null ) {
            TypeCategoryRec brandRec = (TypeCategoryRec) brandVec.elementAt(0) ;
            String       typeMgr  = ((MultiListObject) typeVec.elementAt(0)).getDescript() ;

            if ( typeMgr.equals ( Str.getStr ( STR_DOCUMENT ) ) )
               typeMgr = "DOC" ;
            else if ( typeMgr.equals ( Str.getStr ( STR_AVALON ) ) )
               typeMgr = "OA" ;

            userRec.addBrandMgrRec ( new BrandMgrRec ( typeMgr, brandRec.getInd() ) ) ;

            refreshLists() ;
         }
      }
   }



   /*------------------------------------------------------------*/
   /* removeAllBrandMgr
    */
   /**
    *    This method will remove all brand managers
    *  from the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeAllBrandMgr () {
      userRec.setBrandMgrVec ( new Vector ( 1, 1 ) ) ;
      refreshLists() ;
   }
} /* EditUserPanel */
