package com.ibm.nzna.projects.qit.admin.pers ;    
/*-----------------------------------------------------------*/
/*   PersNavPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PersNavPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
/**
 * This Navigation Panel is the default navigation panel for the PersPanel. It
 * allows the user to search, and to subsection the list of users by the first
 * letter of their name, the user id, or by team.
 *
 * @version 1.0, 3/5/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   PersNavPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class PersNavPanel extends JPanel implements ActionListener, PropertyListener, NavPanel, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextField   ef_SEARCH       = null ;
   private DButton      pb_SEARCH       = null ;
   private HotLinkLabel pb_BYNAME       = null ;
   private HotLinkLabel pb_BYUSERID     = null ;
   //private HotLinkLabel pb_BYTEAM       = null ;
   private NavListGroup searchGroup     = null ;
   private NavListGroup viewGroup       = null ;
   private JScrollPane  scr_NavList     = null ;
   private NavList      navList         = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private PersPanel persPanel      = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor and it will simply
    *  assign values, since the start and initialize
    *  methods do the rest
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PersNavPanel ( PersPanel persPanel ) {
      this.persPanel = persPanel ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      scr_NavList = new JScrollPane ( navList = new NavList() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust our properties                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      navList.setBackground ( Color.white ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( scr_NavList, BorderLayout.CENTER ) ;          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our listeners                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_BUTTONFONT, this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_DEFAULTFONT, this ) ;
      pb_SEARCH.addActionListener ( this ) ;
      pb_BYUSERID.addActionListener ( this ) ;
      pb_BYNAME.addActionListener ( this ) ;
      //pb_BYTEAM.addActionListener ( this ) ;
      ef_SEARCH.addActionListener ( this ) ;
   }


   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
   }


   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_PERSONEL ) ) ;
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will clean up all of our resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_BUTTONFONT, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_DEFAULTFONT, this ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return ourselves since we
    *  are the JComponent
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will be called on initialization and
    *  when the user switches languages
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      HotLinkGroup group = new HotLinkGroup () ;

      navList.removeAll() ;
      searchGroup = new NavListGroup ( Str.getStr ( STR_SEARCH ) ) ;
      searchGroup.add ( ef_SEARCH   = new JTextField ( "" ) ) ;
      searchGroup.add ( pb_SEARCH   = new DButton ( Str.getStr ( STR_OK ) ) ) ;
      viewGroup   = new NavListGroup ( Str.getStr ( STR_SORT ) ) ;
      viewGroup.add ( pb_BYNAME   = new HotLinkLabel ( Str.getStr ( STR_BYNAME ) )  ) ;
      viewGroup.add ( pb_BYUSERID = new HotLinkLabel ( Str.getStr ( STR_BYUSERID ) ) ) ;
      //viewGroup.add ( pb_BYTEAM   = new HotLinkLabel ( Str.getStr ( STR_BYTEAM ) ) ) ;
      navList.add ( searchGroup ) ;
      navList.add ( viewGroup ) ;
      pb_SEARCH.addActionListener ( this ) ;
      pb_BYUSERID.addActionListener ( this ) ;
      pb_BYNAME.addActionListener ( this ) ;
      //pb_BYTEAM.addActionListener ( this ) ;
      ef_SEARCH.addActionListener ( this ) ;

      group.add ( pb_BYUSERID ) ;
      group.add ( pb_BYNAME ) ;
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      setText() ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are listening for has been fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( ( e.getSource() == pb_SEARCH ) || ( e.getSource() == ef_SEARCH ) )
         persPanel.search ( ef_SEARCH.getText() ) ;

      else if ( e.getSource() == pb_BYUSERID )
         persPanel.setView ( new PersView ( PersView.VIEW_BY_USERID ) ) ;

      else if ( e.getSource() == pb_BYNAME )
         persPanel.setView ( new PersView ( PersView.VIEW_BY_NAME ) ) ;

      //else if ( e.getSource() == pb_BYTEAM )
      //   persPanel.setView ( new PersView ( PersView.VIEW_BY_TEAM ) ) ;
   }
} /* PersNavPanel */
