package com.ibm.nzna.projects.qit.admin.pers ; 
/*-----------------------------------------------------------*/
/*   PersPanel                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PersPanel                           */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
/**
 * This panel holds all of the necessary controls to navigate all users in Quest.
 * It also allows the user to create, edit and delete users. If the logged in user
 * wishes to edit another user, or create another user, The EditUserPanel will
 * be used.
 * 
 *
 * @version 1.0, 3/5/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   PersPanel                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class PersPanel extends JPanel implements PrintPanel, PreviewPanel, QuestPanel, ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Controls                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DTree           cnr_USERS         = null ;
   private PersNavPanel    defaultNavPanel   = null ;
   private JTitle          st_TITLE          = null ;
   private JScrollPane     scr_USERS         = null ;      
   private ActionButton    pb_CLOSE          = null ;
   private ActionButton    pb_EDIT           = null ;
   private ActionButton    pb_NEW            = null ;
   private ActionButton    pb_RESETPASSWORD  = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DetailTreeCellRenderer treeRenderer  = null ;
   private StringTreeNode         root          = null ;
   private DefaultTreeModel       model         = null ;
   private int                    currentViewId = 0 ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method does nothing like all QuestPanel constructors
    *  should do!
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PersPanel () {
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel and
    *  return TRUE if the initialization procedure
    *  was good.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, create our GUI                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE         = new ActionButton ( Str.getStr ( STR_CLOSE ),         ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),          Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_NEW           = new ActionButton ( Str.getStr ( STR_NEW ),           ImageSystem.getImageIcon ( this, ImageSystem.ADD_USER ),       Str.getStr ( STR_NEW_USER ) ) ;
      pb_EDIT          = new ActionButton ( Str.getStr ( STR_EDIT ),          ImageSystem.getImageIcon ( this, ImageSystem.EDIT),            Str.getStr ( STR_EDIT_USER ) ) ;
      pb_RESETPASSWORD = new ActionButton ( Str.getStr ( STR_RESETPASSWORD ), ImageSystem.getImageIcon ( this, ImageSystem.RESET_PASSWORD ), Str.getStr ( STR_RESET_PASSWORD ) ) ;
      st_TITLE         = new JTitle ( Str.getStr ( STR_PERSONEL ) ) ;
      scr_USERS        = new JScrollPane ( cnr_USERS = new DTree() ) ;
      defaultNavPanel  = new PersNavPanel ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up our Tree                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeRenderer = new DetailTreeCellRenderer ( GUISystem.getFontUtil(), 5 ) ;
      cnr_USERS.setCellRenderer ( treeRenderer) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_NEW ) ;
      GUISystem.setPreferredButton ( pb_EDIT ) ;
      GUISystem.setPreferredButton ( pb_RESETPASSWORD ) ;
      scr_USERS.getViewport().putClientProperty ("EnableWindowBlit", Boolean.TRUE);    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener         ( this ) ;
      pb_EDIT.addActionListener          ( this ) ;
      pb_NEW.addActionListener           ( this ) ;
      pb_RESETPASSWORD.addActionListener ( this ) ;
      cnr_USERS.addActionListener        ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout () ) ;
      add ( st_TITLE, BorderLayout.NORTH ) ;
      add ( scr_USERS, BorderLayout.CENTER ) ;


      return( true ) ;
   }


   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we have
    *  been initialized and we are now showing. We
    *  will use this time to refresh any data
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start (){
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_NEW ) ;
      parentWin.addActionComponent ( this, pb_EDIT ) ;
      parentWin.addActionComponent ( this, pb_RESETPASSWORD ) ;

      setView ( new PersView ( PersView.VIEW_BY_USERID ) ) ;
   }



   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the user
    *  selects this panel 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon that should
    *  be used to represent this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_PERSONEL ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default
    *  navigation panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NavPanel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( defaultNavPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method should refresh the view that
    *  we are currently showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      if ( currentViewId != 0 ) {
         setView ( new PersView ( currentViewId ) ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will copy whatever is selected
    *  to the clipboard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will return FALSE since we
    *  do not support pasting of text
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will be called when the user
    *  wants to cut text. We will return false since
    *  we do not support that operation
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called if the user
    *  wishes to shutdown the application. We will
    *  simply return TRUE
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when we need
    *  to be removed from the window in which we
    *  are showing. We will return TRUE after freeing our resources
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name of our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_PERSONEL ) ) ;
   }



   /*------------------------------------------------------------*/
   /* search
    */
   /**
    *    This method will perform a search on the criteria
    *  passed in. Please do not call this method from
    *  a normal thread, call this method from a non-awt thread.
    *
    *  @param searchParam The string to search for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void search ( String params ) {
      Vector allUsers = UserSystem.getAllUserVec () ;
      int    i        = 0  ;

      try {
         if ( allUsers != null ) {
            Object[] exactPath     = new Object[2] ;
            Object[] fuzzyPath     = new Object[2] ;
            TreePath treePath      = null ;
            int      size          = allUsers.size() ;                                              
            int      inNodeIndex   = 0 ;             

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now clear our list                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            root  = new StringTreeNode( "" ) ;
            model = new DefaultTreeModel ( root ) ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create a node for the exact and fuzzy matches           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            StringTreeNode exactNode  = new StringTreeNode ( Str.getStr ( STR_EXACT_MATCHES ) ) ;
            StringTreeNode fuzzyNode  = new StringTreeNode ( Str.getStr ( STR_FUZZY_MATCHES ) ) ;               
            model.insertNodeInto ( exactNode, root, 0 ) ;
            model.insertNodeInto ( fuzzyNode, root, 1 ) ;                                             

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through the users and ask each for an exact match  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            i = 0 ;
            inNodeIndex=0 ;
            while ( i < size ) {
               if ( ((UserRec)allUsers.elementAt(i)).equals ( params ) ) {
                  model.insertNodeInto ( new MutableTreeWrapper ( allUsers.elementAt(i)  ) , exactNode, inNodeIndex ) ;
                  inNodeIndex++ ;
               }
               i++ ;
            } /* while  */

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through and ask each object if they 'fuzzy' match  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            i=0 ;
            inNodeIndex=0 ;
            while ( i < size ) {
               if ( ((UserRec)allUsers.elementAt(i)).fuzzyEquals ( params ) ) {
                  model.insertNodeInto ( new MutableTreeWrapper ( allUsers.elementAt(i) ), fuzzyNode, inNodeIndex ) ;
                  inNodeIndex++ ;
               }
               i++ ;
            } /* while  */



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the model to reflect our changes                    */                           
            /*                                                           */
            /*-----------------------------------------------------------*/
            cnr_USERS.setModel ( model ) ;                           

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the selection so both trees show                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            exactPath[0] = root ;
            exactPath[1] = exactNode ;
            fuzzyPath[0] = root ;
            fuzzyPath[1] = fuzzyNode ;

            WinUtil.expandTree ( cnr_USERS ) ;

         } /* if  */
      } catch ( Exception e ) {
      }
   } /* search  */





   /*------------------------------------------------------------*/
   /* setView
    */
   /**
    *    This method will set the view that we are
    *  currently showing
    *  
    *
    *  @param persView The PersView object to use as the view
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setView ( PersView persView ) {
      currentViewId = persView.getViewId() ;

      if ( persView != null ) {
         cnr_USERS.setModel ( persView.getViewModel() ) ;
         scr_USERS.getHorizontalScrollBar().setValue ( 0 ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when action events
    *  are fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   EDIT                                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == pb_EDIT ) || ( e.getSource() == cnr_USERS ) ) {
         try {
            MutableTreeWrapper wrapper = (MutableTreeWrapper) getSelectedItem() ;
            UserRec            userRec = (UserRec) wrapper.getDataObject() ;                              

            userRec.updateRecStatus ( userRec.REC_UPDATE ) ;
            WindowSystem.createPanel ( new EditUserPanel ( this, userRec ) ) ;
         } catch ( Exception ex ) {
            // The cnr_USERS might not have a wrapper selected, if it does not
            // then this exception will be thrown
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   NEW                                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_NEW ) {
         UserRec userRec = new UserRec ( "", "" ) ;

         userRec.updateRecStatus ( userRec.REC_INSERT ) ;
         WindowSystem.createPanel ( new EditUserPanel ( this, userRec ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_RESETPW                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_RESETPASSWORD ) {
         MutableTreeWrapper wrapper = (MutableTreeWrapper) getSelectedItem() ;
         if ( wrapper != null ) {
            UserRec            userRec = (UserRec) wrapper.getDataObject() ;

            if ( userRec.changePassword ( userRec.getUserId() ) )
               GUISystem.printBox ( STR_INFO, STR_PASSWORD_RESET ) ;
         }

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin(this).closePanel ( this, null ) ;


   }




   /*------------------------------------------------------------*/
   /* getSelectedItem
    */
   /**
    *    This method will get the selected item from
    *  our tree
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return MutableTreeNode
    *//*---------------------------------------------------------*/
   private MutableTreeWrapper getSelectedItem () {
      TreePath paths[]        = cnr_USERS.getSelectionPaths() ;
      Object   selectedObject = null ;

      if ( ( paths != null ) && ( paths.length>0 ) )
         return( (MutableTreeWrapper) paths[paths.length-1].getLastPathComponent() ) ;

      return( null ) ;
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }
   /**
    *  This method will give the Print support to the panel
    **/
   public Object getPrintObject() {
      return( new Print ( cnr_USERS, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_USERS ) ) ) ;
   }
   /**
    *  This method will give the Preview support to the panel
    **/
   public Object getPreviewObject() {
      return( new Preview ( cnr_USERS, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_USERS ) ) ) ;
   }

} /* PersPanel */
