package com.ibm.nzna.projects.qit.admin.pers ;
/*-----------------------------------------------------------*/
/*   PersView                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PersView                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
/**
 * This object holds all information about a specific view supported by 
 * the Personel Maint. Quest Applet.
 *
 * @version 1.0, 3/14/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   PersView                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class PersView extends Object implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Supported Views                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int VIEW_BY_USERID          = 1 ;
   public static final int VIEW_BY_NAME            = 2 ;
   public static final int VIEW_BY_TEAM            = 3 ;      

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int              viewId   =   -1 ;
   private DefaultTreeModel model    = null ;
   private StringTreeNode   root     = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates the complete
    *  view and does not return until the view
    *  is complete. It is a good idea not to instantiate this object in an AWT-Thread
    *
    *  @param viewId The id of the supported view. (VIEW_BY_USERID etc..)
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PersView ( int viewId ) {
      this.viewId = viewId ;
      createView() ;
   }






   /*------------------------------------------------------------*/
   /* createView
    */
   /**
    *    This method will actually create the view
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createView () {                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, create the default model for all views           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      createDefaultModel() ;


      switch ( viewId ) {
         
         case VIEW_BY_USERID :
            createUserIdView() ;
            break ;
         
         case VIEW_BY_NAME :
            createNameView() ;
            break ;
      }
   }






   /*------------------------------------------------------------*/
   /* createUserIdView
    */
   /**
    *    This method does the work needed to create
    *  the VIEW_BY_USERID.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createUserIdView () {
      Vector allUsers = UserSystem.getAllUserVec() ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have the vector, create our tree                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( allUsers != null ) {
         int i    = 1 ;
         int size = allUsers.size() ;

         if ( size > 0 ) {
            UserRec        userRec         = null ;
            int            inNodeIndex     = 0 ;
            int            letterNodeIndex = 1 ;
            StringTreeNode currentNode     = null ;
            String         letter          = null ;
            String         newLetter       = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Alphabetize the vector                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            new QuickSort ( allUsers ) ;                          

            root  = new StringTreeNode ( Str.getStr ( STR_BYUSERID ) ) ;
            model = new DefaultTreeModel ( root ) ;             


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the first item                                   */            
            /*                                                           */
            /*-----------------------------------------------------------*/
            userRec = (UserRec) allUsers.elementAt(i) ;
            letter = userRec.getUserId().substring ( 0, 1 ) ;
            currentNode = new StringTreeNode ( letter ) ;
            model.insertNodeInto ( currentNode, root, 0 ) ;
            model.insertNodeInto ( new MutableTreeWrapper ( userRec ), currentNode, 0 ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Traverse through the ordered vector                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            inNodeIndex = 0 ;
            while ( i < size ) {
               userRec = (UserRec) allUsers.elementAt(i) ;
               newLetter = userRec.getUserId().substring ( 0, 1 ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If this is a new letter, create a new node for it       */                                        
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !letter.equals ( newLetter ) ) {
                  currentNode = new StringTreeNode ( newLetter ) ;
                  model.insertNodeInto ( currentNode, root, letterNodeIndex ) ;
                  model.insertNodeInto ( new MutableTreeWrapper ( userRec ), currentNode, 0 ) ;
                  letter = newLetter ;
                  inNodeIndex=0 ;
                  letterNodeIndex++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise, just add this record                         */                      
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  model.insertNodeInto ( new MutableTreeWrapper ( userRec ), currentNode, inNodeIndex ) ;
                  inNodeIndex++ ;
               } /* else  */
               
               i++ ;
            } /* while  */
         } /* size  */

      } /* if allUsers  */

   } /* createUserIdView  */



   /*------------------------------------------------------------*/
   /* createNameView
    */
   /**
    *    This method does the work needed to create
    *  the VIEW_BY_USERID.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createNameView () {
      Vector allUsers = UserSystem.getAllUserVec() ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have the vector, create our tree                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( allUsers != null ) {
         int i    = 1 ;
         int size = allUsers.size() ;

         if ( size > 0 ) {
            UserRec        userRec         = null ;
            int            inNodeIndex     = 0 ;
            int            letterNodeIndex = 1 ;
            StringTreeNode currentNode     = null ;
            String         letter          = null ;
            String         newLetter       = null ;


            root  = new StringTreeNode ( Str.getStr ( STR_BYNAME ) ) ;
            model = new DefaultTreeModel ( root ) ;             

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Alphabetize the vector, set the name as the toString for*/
            /*   the sort to work.                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( i < size ) {
               ((UserRec)allUsers.elementAt(i)).setToString ( ((UserRec)allUsers.elementAt(i)).getName() ) ;
               i++ ;
            }
            new QuickSort ( allUsers ) ;                          
            i=0 ;
            while ( i < size ) {
               ((UserRec)allUsers.elementAt(i)).setToString ( ((UserRec)allUsers.elementAt(i)).getUserId() ) ;
               i++ ;
            }
            i=1 ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the first item                                   */            
            /*                                                           */
            /*-----------------------------------------------------------*/
            userRec = (UserRec) allUsers.elementAt(i) ;
            letter = userRec.getName().substring ( 0, 1 ) ;
            currentNode = new StringTreeNode ( letter ) ;
            model.insertNodeInto ( currentNode, root, 0 ) ;
            model.insertNodeInto ( new MutableTreeWrapper ( userRec ), currentNode, 0 ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Traverse through the ordered vector                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            inNodeIndex = 0 ;
            while ( i < size ) {
               userRec = (UserRec) allUsers.elementAt(i) ;
               newLetter = userRec.getName().substring ( 0, 1 ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If this is a new letter, create a new node for it       */                                        
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !letter.equals ( newLetter ) ) {
                  currentNode = new StringTreeNode ( newLetter ) ;
                  model.insertNodeInto ( currentNode, root, letterNodeIndex ) ;
                  model.insertNodeInto ( new MutableTreeWrapper ( userRec ), currentNode, 0 ) ;
                  letter = newLetter ;
                  inNodeIndex=0 ;
                  letterNodeIndex++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise, just add this record                         */                      
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  model.insertNodeInto ( new MutableTreeWrapper ( userRec ), currentNode, inNodeIndex ) ;
                  inNodeIndex++ ;
               } /* else  */
               
               i++ ;
            } /* while  */
         } /* size  */

      } /* if allUsers  */

   } /* createNameView  */



   /*------------------------------------------------------------*/
   /* getViewModel
    */
   /**
    *    This method will return the DefaultTreeModel
    *  for a JTree for this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getViewModel () {
      return( model ) ;
   }





   /*------------------------------------------------------------*/
   /* createDefaultModel
    */
   /**
    *    This method will create the default model.
    *  This includes a line of text stating that the
    *  view is not supported
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createDefaultModel () {
      root  = new StringTreeNode ( Str.getStr ( STR_VIEW_NOT_SUPPORTED ) ) ;
      model = new DefaultTreeModel ( root ) ;
   }






   /*------------------------------------------------------------*/
   /* getViewId
    */
   /**
    *    This method will return the View ID
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getViewId () {
      return ( viewId ) ;
   }

} /* PersView */
