package com.ibm.nzna.projects.qit.admin.stats ;  
/*-----------------------------------------------------------*/
/*   StatsExport                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StatsExport                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import java.sql.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;

/**
 * This object will export information from the Quest Database to a flat file.
 *
 * @version 1.0, 7/10/2000 
 * @author  David Lentz
 */

public class StatsExport extends Object implements AppConst {


   /**
     * This export type creates a standard HTML File
     **/
   public static final int OUTPUT_HTML = 1 ;

   /**
     * This export type creates a standard Comma Delimited file
     **/
   public static final int OUTPUT_COMMA = 2 ;

   /**
     * This is the vector which will hold all StatDocRec objects to export
     **/
   private Vector documents = new Vector ( 1000, 1 ) ;

   /**
     * This is the dbUser that the export can be restricted to
     **/
   private String dbUser = null ;

   /**
     * This vector will contain all of the header information for
     * the export
     **/
   private Vector headers = null ;

   /**
     * The number of rows that were exported
     **/
   private int rowsExported = 0 ;

   /**
     * When writing the output, odd lines are colored differently
     **/
   private boolean oddLine = false ;

   /**
     * This is the outputSTream that is created when the output file is
     * opened
     **/
   private FileOutputStream outputStream = null ;

   /**
     * This is the date that the stats will be pulled up to
     **/
   private String toDate = null ;

   /**
     * This is the date
     **/
   private String date = null ;

   /**
     * This is the filename of the file which will be exported to
     **/
   private String filename = null ;

   /**
     * This is the current Export type
     **/
   private int outputType = OUTPUT_COMMA ;

   /**
     * This is the AppDefaultWin
     **/
   private AppDefaultWin defaultWin = null ;

   /**
     * This is the stats panel
     **/
   private StatsPanel statsPanel = null ;

   /**
     * Default Constructor
     *
     * @param defaultWin The AppDefaultWin that will be used to show status
     * @param filename The name of the file to export to
     * @param outputType One of the predefined export types in this class
     *
     * @return 
     **/
   public StatsExport ( AppDefaultWin defaultWin, StatsPanel statsPanel, String filename, int outputType ) {
      this.filename   = filename ;
      this.outputType = outputType ;
      this.defaultWin = defaultWin ;
      this.statsPanel = statsPanel ;
   }


   /**
     * This method will set the dbUser that the exports will be restricted to
     **/
   public void setDBUser ( String dbUser ) {
      if ( ( dbUser != null ) && ( dbUser.length() == 0 ) )
         dbUser = null ;

      this.dbUser = dbUser ;
   }



   /**
     * This method will extract all documents in the Quest database that
     * have the text 'BODY HAS BEEN CHANGED' in the comments of the document.
     **/
   public int exportChangedBodyDocuments () {
      int         rc        = 0 ;
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "StatsExport.exportChangedBodyDocuments", 100 ) ;
      long        startTime = System.currentTimeMillis() ;

      defaultWin.setStatus ( "Reading Initial Document List ...", false ) ;

      try {
         int        count         = 0 ;
         int        updateCount   = 0 ;
         StatDocRec currentDocRec = null ;
         int        lastDocumentInd = -1 ;
         Statement  stmt          = sqlMethod.createStatement() ;
         ResultSet  rs            = stmt.executeQuery ( "SELECT A.DOCIND,         STARTDATE,  STOPDATE,       LNDOCID,     "+
                                                        "         EXISTINGDOCIND, TITLE,      A.OWNER,        REQUESTOR,   "+
                                                        "         METRICIND,      DOCTYPEIND, WORKIND,        DOCCLASSIND, "+
                                                        "       A.CHANGEDTIME,    A.DBUSER,   FAQ,            FLASHDATE, "+
                                                        "         EMAILABLE,      STATUSIND,  D.DESCRIPTION,  D.CATEGORYLEVEL  "+
                                                        "FROM QUEST.DOCUMENTS A, "+
                                                        "     QUEST.DOCSTATUS B, "+
                                                        "     QUEST.CATEGORY  C, "+
                                                        "     PRODUCT.TYPECATEGORY D "+
                                                        "WHERE TEMPLATE = 'N' AND "+
                                                        "      A.DOCIND = B.DOCIND AND "+
                                                        "      B.RECNUM = 2 AND "+
                                                        "      A.CHANGEDTIME > '"+date+"' AND "+
                                                        "      A.CHANGEDTIME < '"+toDate+"' AND "+
                                                        "      COMMENT LIKE '%BODY HAS BEEN CHANGED%' AND "+
                                                        "      A.DOCIND = C.DOCIND AND "+
                                                        "      C.TYPECATIND = D.TYPECATIND "+
                                                        ((dbUser!=null) ? "AND A.DBUSER = '"+dbUser+"' " : "")+
                                                        "ORDER BY STARTDATE, A.DOCIND" ) ;

         createDocumentHeaders() ;
         beginOutFile ( "Changed Documents" ) ;


         while ( rs.next() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this document is different than the one just read in */
            /* then write out the current object and start a new object  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getInt(1) != lastDocumentInd ) {
               if ( currentDocRec != null ) {
                  System.out.println ( "Writing Document "+currentDocRec.existingDocInd ) ;
                  exportDocument ( currentDocRec ) ;
               }
               currentDocRec = null ;
               lastDocumentInd = rs.getInt(1) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have no current product, create one               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( currentDocRec == null ) {
               currentDocRec = new StatDocRec ( rs.getInt(1), rs.getString(2).trim() ) ;

               currentDocRec.stopDate       = rs.getString(3) ;
               currentDocRec.notesId        = rs.getString(4) ;
               currentDocRec.existingDocInd = rs.getInt(5) ;
               currentDocRec.title          = rs.getString(6).trim() ;
               currentDocRec.owner          = rs.getString(7).trim() ;
               currentDocRec.requestor      = rs.getString(8).trim() ;
               currentDocRec.metricInd      = rs.getInt(9) ;
               currentDocRec.docTypeInd     = rs.getInt(10) ;
               currentDocRec.workInd        = rs.getInt(11) ;
               currentDocRec.docClassInd    = rs.getInt(12) ;
               currentDocRec.changedTime    = DateSystem.prettyDateFromStamp ( rs.getString(13) ) ;
               currentDocRec.dbUser         = rs.getString(14).trim() ;
               currentDocRec.faq            = rs.getString(15).trim();
               currentDocRec.hotNewsDate    = rs.getString(16) ;
               currentDocRec.eMailable      = rs.getString(17) ;

               count++ ;
               rc++ ;
               updateCount++ ;
               if ( updateCount > 20 ) {
                  defaultWin.setStatus ( "Exporting Documents. Currently exported "+count+" documents", false ) ;
                  updateCount=0 ;
               }
            } /* if currentDocRec  */

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, assign the categories                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_BRAND ) )
               currentDocRec.brands+=rs.getString(19).trim()+"(All); " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_FAMILY ) )
               currentDocRec.family+=rs.getString(19).trim()+"(All); " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_SUBCATEGORY ) )
               currentDocRec.machines+= rs.getString(19).trim()+"(All); " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_MACHINE ) )
               currentDocRec.machines+= rs.getString(19).trim()+"(All); " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_MODEL ) )
               currentDocRec.models+=rs.getString(19).trim()+"; " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_PARTNUMBER ) )
               currentDocRec.models+=rs.getString(19).trim()+"; " ;

         }

         rs.close() ;
         rowsExported = count ;
         endOutFile() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack();
         e.printStackTrace() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      defaultWin.setStatus ( null ) ;

      return( rc ) ;
   }



   /**
     * This method will export All documents
     **/
   public int exportAllDocuments () {
      int         rc        = 0 ;
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "StatsExport.exportAllDocuments", 100 ) ;
      long        startTime = System.currentTimeMillis() ;

      defaultWin.setStatus ( "Reading Initial Document List ...", false ) ;

      try {

         createDocumentHeaders() ;
         beginOutFile ( "All Documents" ) ;

         int        count         = 0 ;
         int        updateCount   = 0 ;
         StatDocRec currentDocRec = null ;
         int        lastDocumentInd = -1 ;
         Statement  stmt          = sqlMethod.createStatement() ;
         ResultSet  rs            = stmt.executeQuery ( "SELECT A.DOCIND,         STARTDATE,  STOPDATE,       LNDOCID,     "+
                                                        "         EXISTINGDOCIND, TITLE,      A.OWNER,        REQUESTOR,   "+
                                                        "         METRICIND,      DOCTYPEIND, WORKIND,        DOCCLASSIND, "+
                                                        "       A.CHANGEDTIME,    A.DBUSER,   FAQ,            FLASHDATE, "+
                                                        "         EMAILABLE,      STATUSIND,  D.DESCRIPTION,  D.CATEGORYLEVEL  "+
                                                        "FROM QUEST.DOCUMENTS A, "+
                                                        "     QUEST.DOCSTATUS B, "+
                                                        "     QUEST.CATEGORY  C, "+
                                                        "     PRODUCT.TYPECATEGORY D "+
                                                        "WHERE TEMPLATE = 'N' AND "+
                                                        "      A.DOCIND = B.DOCIND AND "+
                                                        "      B.RECNUM = 2 AND "+
                                                        "      A.CHANGEDTIME > '"+date+"' AND "+
                                                        "      A.CHANGEDTIME < '"+toDate+"' AND "+
                                                        "      A.DOCIND = C.DOCIND AND "+
                                                        "      C.TYPECATIND = D.TYPECATIND "+
                                                        ((dbUser!=null) ? "AND A.DBUSER = '"+dbUser+"' " : "")+
                                                        "ORDER BY STARTDATE, A.DOCIND" ) ;

         while ( rs.next() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this document is different than the one just read in */
            /* then write out the current object and start a new object  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getInt(1) != lastDocumentInd ) {
               if ( currentDocRec != null )
                  exportDocument ( currentDocRec ) ;
               currentDocRec = null ;
               lastDocumentInd = rs.getInt(1) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have no current product, create one               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( currentDocRec == null ) {
               currentDocRec = new StatDocRec ( rs.getInt(1), rs.getString(2).trim() ) ;

               currentDocRec.stopDate       = rs.getString(3) ;
               currentDocRec.notesId        = rs.getString(4) ;
               currentDocRec.existingDocInd = rs.getInt(5) ;
               currentDocRec.title          = rs.getString(6).trim() ;
               currentDocRec.owner          = rs.getString(7).trim() ;
               currentDocRec.requestor      = rs.getString(8).trim() ;
               currentDocRec.metricInd      = rs.getInt(9) ;
               currentDocRec.docTypeInd     = rs.getInt(10) ;
               currentDocRec.workInd        = rs.getInt(11) ;
               currentDocRec.docClassInd    = rs.getInt(12) ;
               currentDocRec.changedTime    = DateSystem.prettyDateFromStamp ( rs.getString(13) ) ;
               currentDocRec.dbUser         = rs.getString(14).trim() ;
               currentDocRec.faq            = rs.getString(15).trim();
               currentDocRec.hotNewsDate    = rs.getString(16) ;
               currentDocRec.eMailable      = rs.getString(17) ;

               count++ ;
               rc++ ;
               updateCount++ ;
               if ( updateCount > 20 ) {
                  defaultWin.setStatus ( "Exporting Documents. Currently exported "+count+" documents", false ) ;
                  updateCount=0 ;
               }
            } /* if currentDocRec  */

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, assign the categories                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_BRAND ) )
               currentDocRec.brands+=rs.getString(19).trim()+"(All); " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_FAMILY ) )
               currentDocRec.family+=rs.getString(19).trim()+"(All); " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_SUBCATEGORY ) )
               currentDocRec.machines+= rs.getString(19).trim()+"(All); " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_MACHINE ) )
               currentDocRec.machines+= rs.getString(19).trim()+"(All); " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_MODEL ) )
               currentDocRec.models+=rs.getString(19).trim()+"; " ;
            else if ( rs.getString ( 20 ).trim().equals ( TypeCategory.LEVEL_PARTNUMBER ) )
               currentDocRec.models+=rs.getString(19).trim()+"; " ;

         }

         rs.close() ;
         rowsExported = count ;
         endOutFile() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack();
         e.printStackTrace() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }
      defaultWin.setStatus ( null ) ;

      return( rc ) ;
   }




   /**
     * This method will write out the standard header for all Stat Exports
     **/                                                                  
   private FileOutputStream beginOutFile ( String title ) throws Exception {
      String           exportTypeString = "" ;

      outputStream     = new FileOutputStream ( new File ( filename ) ) ;
      System.out.println ( "Creating file "+filename ) ;

      exportTypeString+=title ;
      if ( dbUser != null )
         exportTypeString+=" for "+UserSystem.getNameFromUserId ( dbUser ) ;


      switch ( outputType ) {
         case OUTPUT_HTML :


            outputStream.write ( new String ( "<html>\n<meta tag=\"Generated by the great Quest\">\n<body>\n<font size=2 color=black face=\"Helvetica, Arial\"><font size=3>\n"+
                                              "<b>Quest Database Export - "+exportTypeString+"</b></font><br>\n"+
                                              "<font color=\"800080\">Date:</font><font size=2>"+DateSystem.getDate ( CDate.DATE_DATE )+"</font><br>\n"+
                                              "<font color=\"800080\">Export Run By:</font>"+UserSystem.getUserName()+"<br><br>\n"+
                                              "<br><br>\nResults<br><br>\n<table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n" ).getBytes() ) ;
            break ;

         case OUTPUT_COMMA :
            outputStream.write ( new String ( "Quest Database Export "+exportTypeString+"\nDate:"+DateSystem.getDate ( CDate.DATE_DATE )+"\nExport Run By:"+UserSystem.getUserName()+"\n").getBytes() ) ;
            break ;
      }

      return( outputStream ) ;
   }



   /*------------------------------------------------------------*/
   /* writeToOutFile
    */
   /**
    *    This mehtod will simply write an entire row to the out
    *  file in the current out type. The data written is passed in
    *  the stringVec. Each String in the Vector will be processed in
    *  order of 0-x. Each String will represent a single column
    *  
    *
    *  @param stringVec The Vector of data to write. Each item in the vector MUST be a string
    *  @param header TRUE = This is a header row, FALSE = every other row
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void write ( Vector stringVec, boolean header ) throws Exception {                                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, prep the row by writing any leading data         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( outputType ) {
         case OUTPUT_HTML :
            if ( oddLine ) {
               oddLine = false ;
               outputStream.write ( ("   <tr bgcolor=\"#F4F4F4\">\n").getBytes() ) ;
            } else {
               oddLine  = true ;
               outputStream.write ( ("   <tr bgcolor=\"#FFFFFF\">\n").getBytes() ) ;
            }
            break ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, loop through all of the variables                  */                             
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( stringVec != null ) {
         int     i         = 0 ;
         int     size      = stringVec.size() ;
         String  curString = null ;

         /* Loop through all but the last string */
         while ( i < (size-1) ) {

            curString = (String) stringVec.elementAt(i) ;

            if ( curString == null )
               curString = "" ;

            switch ( outputType ) {
               case OUTPUT_COMMA :
                  if ( ( curString != null ) && ( curString.indexOf ( "," ) != -1 ) )
                     curString = Text.replaceAllStrInStr ( curString, ",", "" ) ;

                  outputStream.write ( (curString+",").toString().getBytes() ) ;
                  break ;

               case OUTPUT_HTML :
                  if ( header )
                     outputStream.write ( ("      <td><font face=\"Helvetica, Arial\" color=black size=2><b><u>"+curString+"</b></u></font></td>\n").getBytes() ) ;
                  else
                     outputStream.write ( ("      <td><font face=\"Helvetica, Arial\" color=black size=2>"+curString+"</font></td>\n").getBytes() ) ;
                  break ;
            }
            i++ ;
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, write the last column                              */              
         /*                                                           */
         /*-----------------------------------------------------------*/
         curString = (String) stringVec.elementAt(i) ;
         switch ( outputType ) {
            case OUTPUT_COMMA :
               outputStream.write ( (curString).toString().trim().getBytes() ) ;
               break ;

            case OUTPUT_HTML :
               if ( header )
                  outputStream.write ( ("      <td><font face=\"Helvetica, Arial\" color=black size=2><b><u>"+curString+"</b></u></font></td>\n").getBytes() ) ;
               else
                  outputStream.write ( ("      <td><font face=\"Helvetica, Arial\" color=black size=2>"+curString+"</font></td>\n").getBytes() ) ;
               break ;
         } /* switch */
      }/* if stringVec */



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, end the row                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( outputType ) {
         case OUTPUT_HTML :
            outputStream.write ( ("   </tr>\n").getBytes() ) ;
            break ;

         case OUTPUT_COMMA :
            outputStream.write ( ("\n").getBytes() ) ;
      }



   } /* writeToOutFile */


   /**
     * This method will create the headers for the document output
     **/
   private void createDocumentHeaders() {
      headers = new Vector ( 10, 1 ) ;

      headers.addElement ( "Classification" ) ;
      headers.addElement ( "Quest Indicator" ) ;
      headers.addElement ( "Notes Id" ) ;
      headers.addElement ( "Active Indicator" ) ;
      headers.addElement ( "Title" ) ;
      headers.addElement ( "Start Date" ) ;
      headers.addElement ( "Stop Date" ) ;
      headers.addElement ( "Age" ) ;
      headers.addElement ( "Owner" ) ;
      headers.addElement ( "Requestor" ) ;
      headers.addElement ( "Metric" ) ;
      headers.addElement ( "Doc Type" ) ;
      headers.addElement ( "Status" ) ;
      headers.addElement ( "Published User" ) ;
      headers.addElement ( "FAQ Flag" ) ;
      headers.addElement ( "Hot News Date" ) ;
      headers.addElement ( "Work Required" ) ;
      headers.addElement ( "eMailable" ) ;
      headers.addElement ( "Brands" ) ;
      headers.addElement ( "Families" ) ;
      headers.addElement ( "Machines" ) ;
      headers.addElement ( "Models" ) ;
      headers.addElement ( "Categories" ) ;
      headers.addElement ( "Last Update Date" ) ;

   }

   /**
     * This method will create the headers for the product output
     **/
   private void createProductHeaders() {
      headers = new Vector ( 10, 1 ) ;

      headers.addElement ( "Brand" ) ;
      headers.addElement ( "Family" ) ;
      headers.addElement ( "Sub Category" ) ;
      headers.addElement ( "Machine" ) ;
      headers.addElement ( "Model" ) ;
      headers.addElement ( "Part Number" ) ;
      headers.addElement ( "Product Number" ) ;
      headers.addElement ( "Description" ) ;
      headers.addElement ( "Last Touched By" ) ;
      headers.addElement ( "Last Touched When" ) ;
      headers.addElement ( "Export Type" ) ;

   }


   private void exportDocument ( StatDocRec docRec ) throws Exception {
      Vector output = new Vector ( 10 ) ;

      output.addElement ( TypeList.getInstance().objectFromInd ( docRec.docClassInd, TypeList.TYPE_DOCCLASS ).toString() ) ;
      output.addElement ( ""+docRec.docInd ) ;
      output.addElement ( docRec.notesId ) ;
      output.addElement ( ""+docRec.existingDocInd ) ;
      output.addElement ( docRec.title ) ;
      output.addElement ( docRec.startDate ) ;
      output.addElement ( docRec.stopDate ) ;
      output.addElement ( getDocumentAge ( docRec ) ) ;
      output.addElement ( UserSystem.getNameFromUserId ( docRec.owner ) ) ;
      output.addElement ( UserSystem.getNameFromUserId ( docRec.requestor ) ) ;
      output.addElement ( TypeList.getInstance().objectFromInd ( docRec.metricInd, TypeList.TYPE_METRIC ).toString() ) ;
      output.addElement ( TypeList.getInstance().objectFromInd ( docRec.docTypeInd, TypeList.TYPE_DOC ).toString() ) ;
      output.addElement ( docRec.status ) ;
      output.addElement ( UserSystem.getNameFromUserId ( docRec.dbUser ) ) ;
      output.addElement ( docRec.faq ) ;
      output.addElement ( docRec.hotNewsDate ) ;
      output.addElement ( TypeList.getInstance().objectFromInd ( docRec.workInd, TypeList.TYPE_WORKREQUIRED ).toString() ) ;
      output.addElement ( docRec.eMailable ) ;

      output.addElement ( docRec.brands ) ;
      output.addElement ( docRec.family ) ;
      output.addElement ( docRec.machines ) ;
      output.addElement ( docRec.models ) ;
      output.addElement ( docRec.categories ) ;
      output.addElement ( docRec.changedTime ) ;
      write ( output, false ) ;

   }

   /**
     *  This method will write out (export) 1 product to the current file
     **/
   private void exportProduct ( StatProductRec productRec ) throws Exception {
      Vector output = new Vector () ;

      output.addElement ( productRec.brand ) ;
      output.addElement ( productRec.family ) ;
      output.addElement ( productRec.subCategory ) ;
      output.addElement ( productRec.machine ) ;
      output.addElement ( productRec.model ) ;
      output.addElement ( productRec.partNumber ) ;
      output.addElement ( productRec.productNumber ) ;
      output.addElement ( productRec.description ) ;
      output.addElement ( productRec.lastTouchedBy ) ;
      output.addElement ( productRec.lastTouchedOn ) ;
      output.addElement ( productRec.exportType ) ;

      write ( output, false ) ;
   }

   /**
     * This method will determine the age of a document by comparing the start and stop dates
     **/
   private String getDocumentAge ( StatDocRec docRec ) {
      String age = "0" ;

      try {
         docRec.stopDate  = docRec.stopDate.trim() ;
      } catch ( Exception e ) {
         docRec.stopDate = null ;
      }

      if ( ( docRec.stopDate == null ) || ( docRec.stopDate.length() < 2 ) ) {
         age = ""+CDate.differenceDate ( new CDate().getDate ( CDate.DATE_DBDATE ),
                                         docRec.startDate,
                                         CDate.DATE_DBDATE )+" Day(s)" ;
      } else
         age = ""+CDate.differenceDate ( docRec.stopDate,
                                         docRec.startDate,
                                         CDate.DATE_DBDATE )+" Day(s)" ;

      return( age ) ;
   }



   /*------------------------------------------------------------*/
   /* endOutFile
    */
   /**
    *    This method will end the out file and close the
    *  output stream
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void endOutFile ( ) {
      try {
         switch ( outputType ) {
            case OUTPUT_HTML :
               outputStream.write ( ("</table><br><br>Total: "+rowsExported+"</font>\n</body></html>").getBytes() ) ;
               break ;
         }

         outputStream.close() ;
      } catch ( Exception e ) {
      }
   }


   /**
     * This method will retrieve the current categories for a document
     *
     * @param docRec The StatDocRec to provide status for
     *
     * @return The categories delimited by semi-colons
     **/
   private String getDocumentCategories ( Vector documents ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocumentStatus", 5 ) ;
      String    category    = "" ;
      long      startTime   = System.currentTimeMillis() ;

      try {
         TypeList  typeList = TypeList.getInstance() ;
         Statement stmt     = sqlMethod.createStatement() ;
         ResultSet rs       = null ;
         int       i        = 0 ;
         int       size     = documents.size() ;
         String    sql      = "SELECT DOCIND, DOCCATIND FROM QUEST.DOCCATEGORY "+
                              "WHERE DOCIND IN "+createInClause ( documents )+" FOR FETCH ONLY" ;                                    


         /*----------*/
         /*          */
         /*  Execute */
         /*          */
         /*----------*/
         rs = stmt.executeQuery ( sql ) ;


         while ( rs.next() ) {
            int            docInd  = rs.getInt(1) ;
            TypeDocCatRec  catRec  = (TypeDocCatRec)typeList.objectFromInd ( rs.getInt(2), TypeList.TYPE_DOCCAT ) ;
            StatDocRec     statRec = getStatDocRec ( docInd, documents ) ;

            if ( statRec != null ) {
               if ( statRec.categories == null )
                  statRec.categories=catRec.toString() ;
               else
                  statRec.categories+=";"+catRec.toString() ;
            }

         }


         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      System.out.println ( "   Reading Categories Took:"+((System.currentTimeMillis()-startTime))+"(ms)" ) ;

      return( category ) ;
   }

   /**
     * This method will return the Brand/Family/Machine/Model of the
     * document passed
     **/
   private void getDocumentBFTM ( Vector documents ) {
      //SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocumentStatus", 5 ) ;
      //String    status    = "" ;
      //TypeList  typeList  = TypeList.getInstance() ;

      //try {
      //   Statement stmt         = sqlMethod.createStatement() ;
      //   ResultSet rs           = null ;
      //   String    inClause     = createInClause ( documents ) ;
      //   String    sql          = "SELECT DOCIND, BRANDGROUPIND "+
      //                            "FROM QUEST.DOCBRAND DOCBRAND "+
      //                            "WHERE DOCBRAND.DOCIND IN "+inClause+" FOR FETCH ONLY" ;


      //   rs = stmt.executeQuery ( sql ) ;
      //   while ( rs.next() ) {
      //      TypeGroupRec  familyRec = Brands.groupRecFromBrandGroupInd ( rs.getInt(2) ) ;
      //      TypeBrandRec  brandRec  = Brands.brandRecFromBrandGroupInd ( rs.getInt(2) ) ;
      //      StatDocRec    docRec    = getStatDocRec ( rs.getInt(1), documents ) ;

      //      if ( docRec != null ) {
      //         if ( docRec.brands == null )
      //            docRec.brands = brandRec.toString() ;
      //         else if ( docRec.brands.indexOf ( brandRec.toString() ) == -1 )
      //            docRec.brands+=";"+brandRec.toString() ;

      //         if ( docRec.family == null )
      //            docRec.family = familyRec.toString() ;
      //         else if ( docRec.family.indexOf ( familyRec.toString() ) == -1 )
      //            docRec.family+=";"+familyRec.toString() ;
      //      } /* if docRec  */

      //   } /* while  */

      //   rs.close() ;

      //   rs = stmt.executeQuery ( "SELECT A.DOCIND, B.BRANDGROUPIND, B.MACHINE, B.MODEL "+
      //                            "FROM QUEST.PRODDOCS A, "+
      //                            "     TIGRIS.PRODUCTS B "+
      //                            "WHERE A.DOCIND IN "+inClause+" AND "+
      //                            "      A.PRODINT = B.PRODINT FOR FETCH ONLY " ) ;

      //   while ( rs.next() ) {
      //      String        machine   = rs.getString(3).trim() ;
      //      String        model     = rs.getString(4).trim() ;
      //      TypeGroupRec  familyRec = Brands.groupRecFromBrandGroupInd ( rs.getInt(2) ) ;
      //      TypeBrandRec  brandRec  = Brands.brandRecFromBrandGroupInd ( rs.getInt(2) ) ;
      //      StatDocRec    docRec    = getStatDocRec ( rs.getInt(1), documents ) ;

      //      if ( docRec != null ) {

      //         if ( machine.equals ( "All" ) ) {
      //            if ( docRec.family == null )
      //               docRec.family="All "+familyRec.toString() ;
      //            else if ( docRec.family.indexOf ( familyRec.toString() ) == -1 )
      //               docRec.family+=";All "+familyRec.toString() ;

      //         }

      //         else if ( model.equals ( "All" ) ) {
      //            if ( docRec.machines == null )
      //               docRec.machines="All "+machine ;
      //            else if ( docRec.machines.indexOf ( machine ) == -1 )
      //               docRec.machines+=";All "+machine ;
      //         } else {
      //            if ( docRec.machines == null )
      //               docRec.machines=machine ;
      //            else
      //               docRec.machines+=";"+machine ;

      //            if ( docRec.models == null )
      //               docRec.models=model ;
      //            else
      //               docRec.models+=";"+model ;
      //         }
      //      }
      //   } /* while rs  */


      //   rs.close() ;

      //} catch ( Exception e ) {
      //   sqlMethod.rollBack() ;
      //   LogSystem.log ( 1, e, false ) ;
      //}

      //sqlMethod.close() ;
   }


   /**
     * This method will go through a Vector of StatDocRec objects and returns
     * the one matching the Doc Indicator passed in
     *
     * @param docInd The Doc Indicator to look for
     * @param documents The Vector of StatDocRec objects
     *
     * @return StatDocRec
     **/
   private StatDocRec getStatDocRec ( int docInd, Vector documents ) {
      int        i        = 0 ;
      int        size     = documents.size() ;
      StatDocRec foundDoc = null ;

      while ( ( foundDoc == null ) && ( i < size ) ) {
         if ( ((StatDocRec)documents.elementAt(i)).docInd == docInd )
            foundDoc = (StatDocRec)documents.elementAt(i) ;
         i++ ;
      }

      return( foundDoc ) ;
   }

   /**
     * This method will create the 'IN' clause for an SQL Query. The 'IN'
     * clause would be something like 'WHERE DOCIND IN (1,2,3,4)'. This method
     * generates everything inside of the parenthesis including the parenthesis.
     *
     * @param documents The Vector of documents to generate from
     *
     * @return String
     **/
   private String createInClause ( Vector documents ) {
      String inClause = "(" ;
      int    i        = 0 ;
      int    size     = documents.size() ;

      /*------------------------------------------------------*/
      /*                                                      */
      /*  Now, add each document indicator to the 'IN' clause */
      /*                                                      */
      /*------------------------------------------------------*/
      while ( i < size ) {
         inClause+=((StatDocRec)documents.elementAt(i)).docInd ;
         if ( i < (size-1) )
            inClause+="," ;
         i++ ;
      }

      return( inClause+")" ) ;
   }


   /**
     * This method will set the date that the queries will query since
     *
     * @param date The date
     **/
   protected void setDate ( String date ) {
      this.date = CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR, CDate.DATE_DATE ),
                                     CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                     CDate.parse ( date, CDate.DATE_DAY, CDate.DATE_DATE ),
                                     0,
                                     0,
                                     0,
                                     CDate.DATE_STAMP ) ;
   }


   protected void setToDate ( String date ) {
      this.toDate = CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR, CDate.DATE_DATE ),
                                       CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                       CDate.parse ( date, CDate.DATE_DAY, CDate.DATE_DATE ),
                                       0,
                                       0,
                                       0,
                                       CDate.DATE_STAMP ) ;
   }





   /**
     * This method will export All Product Drafts based on the properties of this export.
     **/
   public int exportProducts () {
      int         rc        = 0 ;
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "StatsExport.exportChangedBodyDocuments", 100 ) ;

      try {
         int            count             = 0 ;
         int            updateCount       = 0 ;
         Statement      stmt              = sqlMethod.createStatement() ;
         ResultSet      rs                = null ;                        
         long           startTime         = System.currentTimeMillis() ;
         int            lastProductInd    = 0 ;
         StatProductRec currentProductRec = null ;
         
         
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our status and start our file                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         defaultWin.setStatus ( "Reading Initial Product List ...", false ) ;

         createProductHeaders() ;
         beginOutFile ( "Products" ) ;



         rs = stmt.executeQuery ( "SELECT A.DESCRIPTION, A.CATEGORYLEVEL,  B.DESCRIPTION, "+
                                  "       B.PRODNUM,     B.LASTTOUCHEDBY,  B.LASTTOUCHEDON, "+
                                  "       B.SOURCE,      B.PRODUCTIND,     B.TYPECANCELIND "+
                                  "FROM PRODRAFT.TYPECATEGORY A, "+
                                  "     PRODRAFT.PRODUCT B, "+
                                  "     PRODRAFT.CATEGORY C "+
                                  "WHERE B.STOPDATE IS NOT NULL AND "+
                                  "      B.TEMPLATE = 'N' AND "+
                                  "      B.PRODUCTIND = C.PRODUCTIND AND "+
                                  "      A.TYPECATIND = C.TYPECATIND AND "+
                                  "      B.LASTTOUCHEDON > '"+date+"' AND "+
                                  "      B.LASTTOUCHEDON < '"+toDate+"' "+
                                  ((dbUser!=null) ? "AND A.LASTTOUCHEDBY = '"+dbUser+"' " : "")+
                                  "ORDER BY B.PRODUCTIND "+
                                  "FOR FETCH ONLY" ) ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Process each row                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this product is different than the one just read in  */
            /* then write out the current object and start a new object  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getInt(8) != lastProductInd ) {
               if ( currentProductRec != null )
                  exportProduct ( currentProductRec ) ;
               currentProductRec = null ;
               lastProductInd = rs.getInt(8) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have no current product, create one               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( currentProductRec == null ) {
               String source     = rs.getString(7).trim() ;

               currentProductRec = new StatProductRec () ;
               currentProductRec.description = rs.getString ( 3 ).trim() ;
               currentProductRec.productNumber = rs.getString ( 4 ).trim() ;
               currentProductRec.lastTouchedBy = rs.getString ( 5 ).trim() ;
               currentProductRec.lastTouchedOn = rs.getString ( 6 ).trim() ;  

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Ok, figure out the export type.                         */
               /*                                                           */
               /* Export Type   Description                                 */
               /* --------------------------------------------------------  */
               /* 1             OPICM Products published without editing    */
               /* 2             OPICM Products published with editing       */
               /* 3             Manually created products                   */
               /* 4             Cancelled OPIC/M Products                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( source.indexOf ( "OPIC/M" ) != -1 ) {
                  int cancel = rs.getInt ( 9 ) ;

                  if ( !currentProductRec.lastTouchedBy.equals ( "PRODGEN" ) )
                     currentProductRec.exportType = "2" ;
                  else
                     currentProductRec.exportType = "1" ;

                  if ( cancel != 0 )
                     currentProductRec.exportType = "4" ;

               } else
                  currentProductRec.exportType = "3" ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, populate the record with whatever category         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getString ( 2 ).trim().equals ( TypeCategory.LEVEL_BRAND ) )
               currentProductRec.brand = rs.getString(1).trim() ;
            else if ( rs.getString ( 2 ).trim().equals ( TypeCategory.LEVEL_FAMILY ) )
               currentProductRec.family = rs.getString(1).trim() ;
            else if ( rs.getString ( 2 ).trim().equals ( TypeCategory.LEVEL_SUBCATEGORY ) )
               currentProductRec.subCategory = rs.getString(1).trim() ;
            else if ( rs.getString ( 2 ).trim().equals ( TypeCategory.LEVEL_MACHINE ) )
               currentProductRec.machine = rs.getString(1).trim() ;
            else if ( rs.getString ( 2 ).trim().equals ( TypeCategory.LEVEL_MODEL ) )
               currentProductRec.model = rs.getString(1).trim() ;
            else if ( rs.getString ( 2 ).trim().equals ( TypeCategory.LEVEL_PARTNUMBER ) )
               currentProductRec.partNumber = rs.getString(1).trim() ;

            defaultWin.setStatus ( "Exporting "+rc ) ;

            rc++ ;
         }

         rs.close() ;
         endOutFile() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack();
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }
      defaultWin.setStatus ( null ) ;

      return( rc ) ;

   }



}
