package com.ibm.nzna.projects.qit.admin.stats ;  
/*-----------------------------------------------------------*/
/*   StatsPanel                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StatsPanel                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import java.sql.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;

/**
 * This is the Statistics Administration main panel. This panel allows 
 * authorized users to dump database contents to their screen or hard drive.
 *
 * @version 1.0, 7/10/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   StatsPanel                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class StatsPanel extends JPanel implements DocConst, PrintPanel, PreviewPanel, Runnable, ActionListener, AppConst, QuestPanel {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Export Types                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int EXPORT_HTML         = 1 ;
   private static final int EXPORT_COMMA        = 2 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Blank Output Template                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String template = "<html>"+
                             "<body bgcolor=\"#FFFFFF\">"+
                             "<p><font face=\"Arial, Helvetica, sans-serif\" size=\"4\">STR_STATISTICS</font><font face=\"Arial, Helvetica, sans-serif\" size=\"2\"><br>"+
                             "  <font color=\"800080\">STR_DATE:</font> VALUE_DATE</font></p>"+
                             "<p><font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"800080\">STR_EXPORTNAME:</font><font face=\"Arial, Helvetica, sans-serif\" size=\"2\"> "+
                             "  VALUE_EXPORTNAME<br>"+
                             "  <font color=\"800080\">STR_RUNBY:</font> VALUE_DBUSER</font></p>"+
                             "<p><font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"800080\">STR_ROWS_EXPORTED:</font><font face=\"Arial, Helvetica, sans-serif\" size=\"2\"> "+
                             "  VALUE_ROWS_EXPORTED</font></p>"+
                             "<p><font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"800080\">STR_ABOUT:</font><font face=\"Arial, Helvetica, sans-serif\" size=\"2\"><br>"+
                             "  VALUE_ABOUT</font> </p>"+
                             "<p><font face=\"Arial, Helvetica, sans-serif\" size=\"2\"><br>"+
                             "  </font> </p>" ;
   private String waitTemplate = "<html>"+
                                 "<body bgcolor=\"#FFFFFF\">"+
                                 "<p><font face=\"Arial, Helvetica, sans-serif\" size=\"4\">STR_STATISTICS</font><font face=\"Arial, Helvetica, sans-serif\" size=\"2\"><br>"+
                                 "  <font color=\"800080\">Please wait, your statistics are being exported:</font></p>" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Exports                                                 */
   /*                                                           */
   /*   These are predefined exports that the StatsNavPanel     */
   /*   can use. You must use one of these in a call to         */
   /*   runExport()                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected static final int EXPORT_DOCUMENT_CHANGES    = 1 ;
   protected static final int EXPORT_DOCUMENT_NOCHANGES  = 2 ;
   protected static final int EXPORT_PRODUCTS            = 3 ;          



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Browser      browser    = null ;
   private ActionButton pb_CLOSE   = null ;
   private String       currentOutput = "" ;
   private JTextArea    mle_OUTPUT = new JTextArea() ;
   private JScrollPane  scr_OUTPUT = new JScrollPane ( mle_OUTPUT ) ;




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean       oddLine            = false ;         // Used to alternate rows in HTML so that they have different colors
   private int           currentExport      = 0 ;
   private int           rowsExported       = 0 ;
   private StatsNavPanel navPanel           = null ;
   private boolean       threadRunning      = false ;
   private AppDefaultWin parentWin          = null ;
   private String        userId             = null ;
   private int           exportType         = EXPORT_COMMA ;






   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our statistics panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navPanel = new StatsNavPanel () ;
      browser  = GUISystem.createBrowser() ;

      pb_CLOSE = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE ) ) ;           


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener ( this ) ;        
      navPanel.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque ( false ) ; 
      mle_OUTPUT.setEditable ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( browser.getComponent(), BorderLayout.CENTER ) ;
      add ( scr_OUTPUT ) ;




      return( true ) ;
   }


   public void doLayout () {
      Dimension size = getSize() ;
      super.doLayout() ;
      browser.getComponent().setBounds ( 0, 0, size.width, size.height/2 ) ;
      scr_OUTPUT.setBounds ( 0, size.height/2, size.width, size.height/2 ) ;
   }



   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we have 
    *  been initialized and we are now showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      parentWin = GUISystem.getParentDefWin ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Action Buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;

   }



   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our small icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_STATISTICS ) ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are listening for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE )
         parentWin.closePanel ( this, null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   navPanel                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == navPanel ) {
         if ( !threadRunning ) {
            currentExport = navPanel.getSelectedExport() ;

            browser.setPageContent ( populateWaitTemplate() ) ;
            new Thread ( this ).start() ;
         } /* if */
      } /* else */

   }


   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_STATISTICS ) ) ;
   }




   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will return FALSE, since currently
    *  we do not support bookmarks
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean bookmark () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when this panel
    *  has been selected to show
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when this panel
    *  is called to hide itself. Not close, just hide
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our navigation
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( navPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }






   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will kick off a thread to refresh
    *  the data on our screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      String output = null ;
      currentExport=1;
      output = populateTemplate() ;
      browser.setPageContent ( output+"</body></html>" ) ;
      currentOutput = output ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will return TRUE if we are not
    *  currently running a dump. This means that we
    *  can shutdown the application
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( !threadRunning ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the main window
    *  wants us to close. We will return TRUE if
    *  we are not busy
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      if ( !threadRunning )
         panelDeselected() ;

      return( !threadRunning ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }






   /*------------------------------------------------------------*/
   /* populateWaitTemplate
    */
   /**
    *    This method will populate a 'wait' template and
    *  return the results. This template basically says 'Please wait ...'
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String populateWaitTemplate () {
      String returnTemplate = waitTemplate ;                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Populate the static fields                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_DATE",          Str.getStr ( STR_DATE ) ) ;
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_EXPORTNAME",    Str.getStr ( STR_EXPORT_NAME ) ) ;
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_PLEASEWAIT",    Str.getStr ( STR_PLEASE_WAIT ) ) ;                  
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_ABOUT",         Str.getStr ( STR_ABOUT ) ) ;                  
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_STATISTICS",    Str.getStr ( STR_STATISTICS ) ) ;                                   


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   And now the dynamic fields                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_DATE",  DateSystem.getDate ( CDate.DATE_DATE ) ) ;

      switch ( currentExport ) {
         case EXPORT_DOCUMENT_CHANGES :
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_EXPORTNAME", Str.getStr ( STR_CHANGED_DOCS ) ) ;
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_ABOUT", Str.getStr ( STR_CHANGED_DOCS_ABOUT ) ) ;
            break ;

         case EXPORT_DOCUMENT_NOCHANGES :
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_EXPORTNAME", Str.getStr ( STR_NOCHANGED_DOCS ) ) ;
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_ABOUT", Str.getStr ( STR_NOCHANGED_DOCS_ABOUT ) ) ;
            break ;

         case EXPORT_PRODUCTS :
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_EXPORTNAME", Str.getStr ( STR_PRODUCTS ) ) ;
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_ABOUT", Str.getStr ( STR_PRODUCT_EXPORT_ABOUT ) ) ;
            break ;
      }


      return( returnTemplate ) ;

   }



   /*------------------------------------------------------------*/
   /* populateTemplate
    */
   /**
    *    This method will populate a template and
    *  return the results
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String populateTemplate () {
      String returnTemplate = template ;                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Populate the static fields                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_DATE",          Str.getStr ( STR_DATE ) ) ;
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_EXPORTNAME",    Str.getStr ( STR_EXPORT_NAME ) ) ;
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_RUNBY",         Str.getStr ( STR_RUNBY ) ) ;
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_ROWS_EXPORTED", Str.getStr ( STR_ROWS_EXPORTED ) ) ;                  
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_ABOUT",         Str.getStr ( STR_ABOUT ) ) ;                  
      returnTemplate = Text.replaceInStr ( returnTemplate, "STR_STATISTICS",    Str.getStr ( STR_STATISTICS ) ) ;                  



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Populate the dynamic fields                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_DBUSER",           UserSystem.getUserName() ) ;
      returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_DATE",             DateSystem.getDate ( CDate.DATE_DATE ) ) ;
      returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_ROWS_EXPORTED", ""+rowsExported ) ;


      switch ( currentExport ) {
         case EXPORT_DOCUMENT_CHANGES :
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_EXPORTNAME", Str.getStr ( STR_CHANGED_DOCS ) ) ;
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_ABOUT", Str.getStr ( STR_CHANGED_DOCS_ABOUT ) ) ;
            break ;

         case EXPORT_DOCUMENT_NOCHANGES :
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_EXPORTNAME", Str.getStr ( STR_NOCHANGED_DOCS ) ) ;
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_ABOUT", Str.getStr ( STR_NOCHANGED_DOCS_ABOUT ) ) ;
            break ;

         case EXPORT_PRODUCTS :
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_EXPORTNAME", Str.getStr ( STR_PRODUCTS ) ) ;
            returnTemplate = Text.replaceInStr ( returnTemplate, "VALUE_ABOUT", Str.getStr ( STR_PRODUCT_EXPORT_ABOUT ) ) ;
            break ;
      }


      return( returnTemplate ) ;

   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will kick off one of the exports
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      threadRunning = true ;

      try {
         String  saveExportFilename = null ;
         String  date               = navPanel.getDate() ;
         String  toDate             = navPanel.getToDate() ;

         browser.setPageContent ( populateTemplate() ) ;

         if ( ( date == null ) || ( CDate.checkDate ( date, CDate.DATE_DATE ) != 0 ) )
            GUISystem.printBox ( Str.getStr ( STR_INFO ), "You must enter a valid 'From' Date to export data since" ) ;
         else if ( ( toDate == null ) || ( CDate.checkDate ( toDate, CDate.DATE_DATE ) != 0 ) )
            GUISystem.printBox ( Str.getStr ( STR_INFO ), "You must enter a valid 'To' Date to export data since" ) ;

         else {
            StatFileFilter fileFilter = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Save our information for the export                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            userId = navPanel.getUserId() ;

            if ( navPanel.getExportType().equals ( Str.getStr ( STR_HTML ) ) ) {
               exportType = EXPORT_HTML ;
            } else if ( navPanel.getExportType().equals ( Str.getStr ( STR_COMMA_SEPARATED ) ) ) {
               exportType = EXPORT_COMMA ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ask the user for the filename                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            saveExportFilename = getSaveFilename ( fileFilter ) ;                          

            if ( navPanel.getExportType().equals ( Str.getStr ( STR_HTML ) ) ) {
               if ( saveExportFilename.toUpperCase().indexOf ( ".HTML" ) == -1 )
                  saveExportFilename+=".html" ;
            } else if ( navPanel.getExportType().equals ( Str.getStr ( STR_COMMA_SEPARATED ) ) ) {
               if ( saveExportFilename.toUpperCase().indexOf ( ".CSV" ) == -1 )
                  saveExportFilename+=".csv" ;
            }

            if ( saveExportFilename != null ) {

               switch ( currentExport ) {
                  
                  case EXPORT_DOCUMENT_CHANGES :
                     StatsExport export = new StatsExport ( parentWin, this, saveExportFilename, exportType ) ;

                     export.setDBUser ( navPanel.getUserId() ) ;
                     export.setDate ( date ) ;
                     export.setToDate ( toDate ) ;
                     rowsExported = export.exportChangedBodyDocuments() ;
                     break ;

                  case EXPORT_DOCUMENT_NOCHANGES :
                     StatsExport allExport = new StatsExport ( parentWin, this, saveExportFilename, exportType ) ;

                     allExport.setDBUser ( navPanel.getUserId() ) ;
                     allExport.setDate ( date ) ;
                     allExport.setToDate ( toDate ) ;
                     rowsExported = allExport.exportAllDocuments() ;
                     break ;

                  case EXPORT_PRODUCTS :
                     StatsExport productExport = new StatsExport ( parentWin, this, saveExportFilename, exportType ) ;

                     productExport.setDBUser ( navPanel.getUserId() ) ;
                     productExport.setDate ( date ) ;
                     productExport.setToDate ( toDate ) ;
                     rowsExported = productExport.exportProducts() ;
                     break ;

               }
            }

         } /* if they gave us a filename to export to */

         browser.setPageContent ( populateTemplate() ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      threadRunning = false ;
   }





   /*------------------------------------------------------------*/
   /* getSaveFilename
    */
   /**
    *    This method will prompt the user for the filename
    *  to save the export as
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getSaveFilename ( StatFileFilter fileFilter ) throws Exception {
      File[] openFile = GUISystem.getFileOpen ( GUISystem.getParentDefWin ( this ), 
                                                Str.getStr ( STR_SAVE ), 
                                                PropertySystem.getString ( PropertySystem.PROPERTY_STATS_EXPORT_DIR ),
                                                null,
                                                false,
                                                fileFilter ) ;
      String filename = null ;
      String dir      = null ;

      if ( ( openFile != null ) && ( openFile[0] != null ) && ( openFile[1] != null ) ) {

         filename = openFile[0].getName() ;
         dir      = openFile[1].getAbsolutePath() ;

         if ( ( filename != null ) && ( dir != null ) ) {
            PropertySystem.put ( PropertySystem.PROPERTY_STATS_EXPORT_DIR, dir ) ;
            return( dir+File.separator+filename ) ;
         }
      }

      return( null ) ;
   }

   /**
     * This method will simply append text to the output
     **/
   protected void appendOutput ( String newOutput ) {
      currentOutput = mle_OUTPUT.getText()+newOutput ;
      mle_OUTPUT.setText ( currentOutput ) ;
   }


   /**
    *  This method will give the Print support to the panel
    **/
   public Object getPrintObject() {
      return( new Print ( this, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }

   /**
   *  This method will give the Preview support to the panel
   **/
   public Object getPreviewObject() {
      return( new Preview ( this, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }

} /* StatsPanel */



class StatFileFilter extends javax.swing.filechooser.FileFilter {

   String descript = "" ;
   String ext      = "" ;

   public StatFileFilter ( String descript, String ext ) {
      this.descript = descript ;
      this.ext      = ext; 
   }

   public String getDescription () {
      return( descript ) ;
   }

   public boolean accept ( File file) {
      boolean rc = false ;

      if ( file.toString().toUpperCase().endsWith ( ext.toUpperCase() ) )
         rc = true ;

      return( false ) ;
   }

}
