package com.ibm.nzna.projects.qit.admin.teammaint ;
/*-----------------------------------------------------------*/
/*   TeamMaintPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TeamMaintPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel holds the controls and lists for the Team Management panel 
 *
 *
 * @version 1.0, 4/20/1998 
 * @author  David Lentz
 */
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   TeamMaintPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class TeamMaintPanel extends JPanel implements PrintPanel, PreviewPanel, ActionListener, Runnable, QuestPanel, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REMOVE   = "REMOVE" ;
   private static final String THREAD_REFRESH  = "REFRESH" ;
   private static final String THREAD_ADDUSERS = "ADDUSER" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector       authVec         = new Vector ( 50, 10 ) ;
   private Vector       memberVec       = new Vector ( 100, 20 ) ;
   private TypeTeamRec  currentTeamRec = null ;
   private Vector       newUserVec      = null ;
   private TeamNavPanel defaultNavPanel = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle        st_TEAM         = new JTitle ( "" ) ;
   private JLabel        st_DESCRIPT     = new JLabel ( "" ) ;
   private JTitle        st_USERS        = new JTitle ( Str.getStr ( STR_MEMBERS ) ) ;
   private JTitle        st_AUTHS        = new JTitle ( Str.getStr ( STR_AUTHORITIES ) ) ;
   private MultiList     authorities       = null ;
   private MultiList     memberList       = null ;
   private DButton       pb_GRANT        = new DButton ( Str.getStr ( STR_GRANT ) ) ;
   private DButton       pb_REVOKE       = new DButton ( Str.getStr ( STR_REVOKE ) ) ;
   private DButton       pb_ADD          = new DButton ( Str.getStr ( STR_ADD ) ) ;
   private DButton       pb_REMOVE       = new DButton ( Str.getStr ( STR_REMOVE ) ) ;
   private ActionButton  pb_CLOSE        = null ;
   private ActionButton  pb_NEW          = null ;
   private ActionButton  pb_DELETE       = null ;
   private ActionButton  pb_EDIT         = null ;
   private Insets        buttonInsets    = new Insets ( 2,2,2,2) ;
   private JPanel        authPanel       = new JPanel () ;
   private JPanel        userPanel       = new JPanel () ;
   private JPanel        namePanel       = new JPanel () ;
   private ButtonPanel   authButtonPanel = new ButtonPanel () ;
   private ButtonPanel   userButtonPanel = new ButtonPanel () ;





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will be called when we
    *  need to initialize ourselves before we
    *  are shown
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE
    *//*---------------------------------------------------------*/
   public boolean initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE  = new ActionButton ( Str.getStr ( STR_CLOSE ),  ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),  Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_NEW    = new ActionButton ( Str.getStr ( STR_NEW ),    ImageSystem.getImageIcon ( this, ImageSystem.NEW ) ,   Str.getStr ( STR_CREATE_TEAM ) );
      pb_EDIT   = new ActionButton ( Str.getStr ( STR_EDIT ),   ImageSystem.getImageIcon ( this, ImageSystem.EDIT ),   Str.getStr ( STR_EDIT_TEAM ) ) ;
      pb_DELETE = new ActionButton ( Str.getStr ( STR_DELETE ), ImageSystem.getImageIcon ( this, ImageSystem.DELETE ), Str.getStr ( STR_DELETE_TEAM ) ) ;

      memberList       = new MultiList ( GUISystem.getFontUtil() ) ;
      authorities       = new MultiList ( GUISystem.getFontUtil() ) ;
      defaultNavPanel = new TeamNavPanel ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      memberList.setOpaque ( false ) ;
      authorities.setOpaque ( false ) ;
      memberList.setBorder ( null ) ;
      authorities.setBorder ( null ) ;
      pb_GRANT.setOpaque ( false ) ;
      pb_REVOKE.setOpaque ( false ) ;
      pb_ADD.setOpaque ( false ) ;
      pb_REMOVE.setOpaque ( false ) ;

      st_TEAM.setOpaque ( false ) ;
      st_DESCRIPT.setOpaque ( false ) ;
      st_AUTHS.setOpaque ( false ) ;
      st_AUTHS.setFont ( FontSystem.smallTitleFont ) ;
      st_USERS.setOpaque ( false ) ;
      st_USERS.setFont ( FontSystem.smallTitleFont ) ;
      pb_GRANT.setMargin ( buttonInsets ) ;
      pb_REVOKE.setMargin ( buttonInsets ) ;
      pb_ADD.setMargin ( buttonInsets ) ;
      pb_REMOVE.setMargin ( buttonInsets ) ;
      setBackground ( Color.white ) ;
      namePanel.setOpaque ( false ) ;
      authPanel.setOpaque ( false ) ;
      userPanel.setOpaque ( false ) ;
      userButtonPanel.setOpaque ( false ) ;
      authButtonPanel.setOpaque ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_GRANT.addActionListener ( this ) ;
      pb_REVOKE.addActionListener ( this ) ;
      pb_ADD.addActionListener ( this ) ;
      pb_REMOVE.addActionListener ( this ) ;
      authorities.addActionListener( this ) ;
      memberList.addActionListener( this ) ;
      pb_CLOSE.addActionListener ( this ) ;
      pb_NEW.addActionListener ( this ) ;
      pb_EDIT.addActionListener ( this ) ;
      pb_DELETE.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;

      namePanel.setLayout ( new BorderLayout() ) ;
      namePanel.add ( st_TEAM,     BorderLayout.CENTER ) ;
      namePanel.add ( st_DESCRIPT, BorderLayout.SOUTH ) ;

      authPanel.setLayout ( new BorderLayout() ) ;
      authButtonPanel.add ( pb_GRANT ) ;
      authButtonPanel.add ( pb_REVOKE ) ;
      authPanel.add ( st_AUTHS,        BorderLayout.NORTH ) ;
      authPanel.add ( authorities,       BorderLayout.CENTER ) ;
      authPanel.add ( authButtonPanel, BorderLayout.SOUTH ) ;

      userPanel.setLayout ( new BorderLayout() ) ;
      userButtonPanel.add ( pb_ADD ) ;
      userButtonPanel.add ( pb_REMOVE ) ;
      userPanel.add ( st_USERS,        BorderLayout.NORTH ) ;
      userPanel.add ( memberList,       BorderLayout.CENTER ) ;
      userPanel.add ( userButtonPanel, BorderLayout.SOUTH ) ;

      add ( namePanel ) ;
      add ( authPanel ) ;
      add ( userPanel ) ;

      GUISystem.setPreferredButton ( pb_GRANT ) ;
      GUISystem.setPreferredButton ( pb_REVOKE ) ;
      GUISystem.setPreferredButton ( pb_ADD ) ;
      GUISystem.setPreferredButton ( pb_REMOVE ) ;                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the text on our buttons and columns in the lists    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshAll() ;


      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 0 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;
      namePanel.setBounds ( x, y, size.width-(x*2), rowHeight*3 ) ;
      y+=(rowHeight*3)+5;

      authPanel.setBounds ( x, y, size.width-(x*2), rowHeight*10 ) ;
      y+=(rowHeight*10)+5 ;

      userPanel.setBounds ( x, y, size.width-(x*2), size.height-y-15 ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_NEW ) ;
      parentWin.addActionComponent ( this, pb_EDIT ) ;
      parentWin.addActionComponent ( this, pb_DELETE ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   refresh our panel                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refresh() ;
   }


   /*-----------------------------------------------------------*/
   /*   refreshAll                                              */
   /*                                                           */
   /*   Description:                                            */
   /*      Called when the user switches a language             */                                                     
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public void refreshAll () {
      String[] userTitle = { "", "", "", "", ""} ;
      String[] authTitle = { ""} ;

      pb_GRANT.setText  ( Str.getStr ( STR_GRANT ) ) ;
      pb_REVOKE.setText ( Str.getStr ( STR_REVOKE ) ) ;
      pb_ADD.setText    ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE.setText ( Str.getStr ( STR_REMOVE ) ) ;

      authorities.setColumnHeadings ( authTitle ) ;
      authorities.setColumnWidth ( 0, 250 ) ;

      memberList.setColumnHeadings ( userTitle ) ;
      memberList.setColumnWidth ( 0, 100 ) ;
      memberList.setColumnWidth ( 1, 250 ) ;
      memberList.setColumnWidth ( 2, 250 ) ;
      memberList.setColumnWidth ( 3, 250 ) ;
      memberList.setColumnWidth ( 4, 250 ) ;

      memberList.showColumnHeadings ( false ) ;
      authorities.showColumnHeadings ( false ) ;
   }



   /*-----------------------------------------------------------*/
   /*   run                                                     */
   /*                                                           */
   /*   Description:                                            */
   /*      This thread will either refresh the initial cnrs     */                                                                   
   /*   or refresh a list of authroties.                        */
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( name.equals ( THREAD_REFRESH ) ) {
         if ( currentTeamRec != null ) {
            authorities.removeAll() ;
            memberList.removeAll() ;

            GUISystem.getParentDefWin ( this ).setStatus ( Str.getStr ( STR_READING ) ) ;
            authVec.removeAllElements() ;
            memberVec.removeAllElements() ;
            TeamSQL.readAssoc ( (short) currentTeamRec.getInd(), authVec, memberVec ) ;

            if ( ( authVec != null ) && ( authVec.size() > 0 ) )
               authorities.add ( authVec ) ;

            if ( ( memberVec != null ) && ( memberVec.size() > 0 ) )
               memberList.add ( memberVec ) ;

            st_TEAM.setText ( currentTeamRec.getShortDescript() ) ;
            st_DESCRIPT.setText ( currentTeamRec.getLongDescript() ) ;
            GUISystem.getParentDefWin ( this ).setStatus ( null ) ;

         }
      } /* THREAD_REFRESH */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REMOVE                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( name.equals ( THREAD_REMOVE ) )
         removeUsers() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_ADDUSERS                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( name.equals ( THREAD_ADDUSERS ) )
         addUsers() ;



   }

   /*-----------------------------------------------------------*/
   /*   createTeam                                             */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will bring up the wizard to create a new */                                                           
   /*   group.                                                  */
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public void createTeam () {
      String      teamName     = null ;
      String      teamDescript = null ;
      TypeTeamRec teamRec      = null ;

      teamName = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                      Str.getStr ( STR_NEW ), 
                                      Str.getStr ( STR_TEAM_NAME ), 
                                      "",
                                      "" ).getResult() ;

      teamDescript = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                          Str.getStr ( STR_NEW ), 
                                          Str.getStr ( STR_TEAM_DESCRIPTION ), 
                                          "",
                                          "" ).getResult() ;

      if ( ( teamName != null ) && 
           ( teamDescript != null ) && 
           ( teamName.length() > 0 ) && 
           ( teamDescript.length() > 0 ) ) {

         if ( duplicateTeam ( teamName, teamDescript ) )
            GUISystem.printBox ( "Info", "You cannot create a team that has the same Team Name or Team Description" ) ;

         else {

            teamRec = new TypeTeamRec ( Counter.getCounter ( LogSystem.getInstance(), "TGROUPIND" ),
                                        teamName,
                                        teamDescript ) ;                

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Save this to the Database                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( QITSqlRunner.sqlRunner ( "INSERT INTO TIGRIS.USERGROUP "+
                                          "( USERGROUP, DESCRIPT, LONGDESCRIPT, DBUSER, CHANGEDTIME ) "+
                                          "VALUES "+
                                          "( "+teamRec.getInd()+", '"+teamName+"', '"+teamDescript+"', "+
                                          "'"+UserSystem.getUserId()+"', CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ) ) {
               TypeList.addToList ( teamRec, TypeList.TYPE_TEAM ) ;
               defaultNavPanel.refreshList() ;
               setTeamRec ( teamRec ) ;
            }
         }
      }
   }


   /*-----------------------------------------------------------*/
   /* edit
    */
   /**
    *    This method will edit the currently selected
    *  Group
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void edit () {
      String      teamName     = null ;
      String      teamDescript = null ;

      teamName = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                      Str.getStr ( STR_EDIT ), 
                                      Str.getStr ( STR_TEAM_NAME ), 
                                      currentTeamRec.getShortDescript(),
                                      "" ).getResult() ;

      teamDescript = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                          Str.getStr ( STR_NEW ), 
                                          Str.getStr ( STR_TEAM_DESCRIPTION ), 
                                          currentTeamRec.getLongDescript(),
                                          "" ).getResult() ;

      if ( ( teamName != null ) && 
           ( teamDescript != null ) && 
           ( teamName.length() > 0 ) && 
           ( teamDescript.length() > 0 ) ) {

         currentTeamRec.setDescript ( teamName ) ;
         currentTeamRec.setLongDescript ( teamDescript ) ;                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save this to the Database                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         QITSqlRunner.sqlRunner ( "UPDATE TIGRIS.USERGROUP "+
                                  "SET DESCRIPT     = '"+teamName+"', "+
                                  "    LONGDESCRIPT = '"+teamDescript+"', "+
                                  "    DBUSER       = '"+UserSystem.getUserId()+"', "+
                                  "    CHANGEDTIME  = CURRENT TIMESTAMP "+
                                  "WHERE USERGROUP = "+currentTeamRec.getInd()+
                                  QITSqlRunner.END_DELIM ) ;

         refresh() ;
         setTeamRec ( currentTeamRec ) ;
      }/* if data was good  */
   }


   /*-----------------------------------------------------------*/
   /* delete
    */
   /**
    *    This method will delete a user group from the
    *  database.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void delete () {
      GUISystem.getParentDefWin ( this ).setStatus ( STR_VALIDATING ) ;
      if ( TeamSQL.validateTeamDelete ( currentTeamRec.getInd() ) ) {
         TypeList.removeFromList ( currentTeamRec, TypeList.TYPE_TEAM ) ;
         QITSqlRunner.sqlRunner ( "DELETE FROM TIGRIS.USERGROUPAUTH WHERE USERGROUP = "+currentTeamRec.getInd()+QITSqlRunner.END_DELIM+
                                  "DELETE FROM TIGRIS.USERGROUP     WHERE USERGROUP = "+currentTeamRec.getInd()+QITSqlRunner.END_DELIM ) ;
         defaultNavPanel.refreshList() ;
         refresh() ;
         currentTeamRec = null ;
      } else
         GUISystem.printBox ( STR_INFO, STR_TEAM_DELETE_NOT_VALID ) ;
      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;
   }


   /*-----------------------------------------------------------*/
   /* removeUsers
    */
   /**
    *    This method will remove a user from the container and the
    *    user group
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeUsers () {
      Vector selVec = memberList.getSelection() ;

      if ( ( selVec != null ) && ( selVec.size() > 0 ) ) {
         int    i     = 0 ;
         int    size  = selVec.size() ;
         String sql   = "" ;                                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Remove each item from our list and create our SQL       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            memberList.remove ( (MultiListRow)selVec.elementAt(i) ) ;
            sql+="DELETE FROM TIGRIS.USERIDUSERGROUP WHERE "+
                 "USERGROUP = "+currentTeamRec.getInd()+" AND "+
                 "USERID    = '"+((UserRec)selVec.elementAt(i)).getUserId()+"'"+
                 QITSqlRunner.END_DELIM ;
            memberVec.removeElement ( selVec.elementAt(i));
            i++ ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, save the information to the database               */                                
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( sql != null )
            QITSqlRunner.sqlRunner ( sql ) ;
      }
   } /* remove */


   /*-----------------------------------------------------------*/
   /* revoke
    */
   /**
    *    This method will remove an authority from the container and the
    *    user group
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void revoke () {
      Vector selVec = authorities.getSelection() ;

      if ( ( selVec != null ) && ( selVec.size() > 0 ) ) {
         int    i     = 0 ;
         int    size  = selVec.size() ;
         String sql   = "" ;                                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Remove each item from our list and create our SQL       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            authorities.remove ( (MultiListRow)selVec.elementAt(i) ) ;
            authVec.removeElement ( selVec.elementAt(i) ) ;
            sql+="DELETE FROM TIGRIS.USERGROUPAUTH WHERE "+
                 "USERGROUP = "+currentTeamRec.getInd()+" AND "+
                 "AUTHIND   = "+((TypeAuthRec)selVec.elementAt(i)).getInd()+
                 QITSqlRunner.END_DELIM ;
            i++ ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, save the information to the database               */                                
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( sql != null )
            QITSqlRunner.sqlRunner ( sql ) ;
      }
   } /* revoke */

   /*-----------------------------------------------------------*/
   /* addUsers
    */
   /**
    *    This method will save users to the  database and insert
    *  into the container
    *  
    *
    *  data - vector containing selected users
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addUsers () {
      Vector vec = new SelectUserDlg ( GUISystem.getParentDefWin(this), true ).getResult() ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int     i    = 0 ;
         int     size = vec.size() ;
         String  sql  = "" ;                                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add each user to our list, also create the SQL To save  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            if ( !StoreUtil.vectorContains ( memberVec, vec.elementAt(i) ) ) {
               memberList.add ( (MultiListRow) vec.elementAt(i) ) ;
               sql+="INSERT INTO TIGRIS.USERIDUSERGROUP "+
                    "( USERGROUP, USERID, DBUSER, CHANGEDTIME ) "+
                    "VALUES "+
                    "( "+currentTeamRec.getInd()+", "+
                    " '"+((UserRec)vec.elementAt(i)).getUserId()+"', "+
                    " '"+UserSystem.getUserId()+"', "+
                    " CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ;
               memberVec.addElement ( vec.elementAt(i) ) ;
            }

            i++ ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save the SQL                                            */   
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( sql.length() > 0 )
            QITSqlRunner.sqlRunner ( sql ) ;

      }
   }  /* saveUser */

   /*-----------------------------------------------------------*/
   /* saveAuth
   */
   /**
   *    This method will save authorities to the  database and insert
   *  into the container
   *  
   *
   *  data - vector object with authorities
   *
   *  @return Nothing
   *//*---------------------------------------------------------*/
   public void saveAuth ( Object data ) {
   } /* saveAuth */





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one of our
    *  buttons are pushed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( Qit.getEnabled () ) && ( e.getSource() instanceof DButton ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_NEW                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( pb_NEW == e.getSource() )
            createTeam() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CLOSE                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_CLOSE == e.getSource() ) // pb_CLOSE is defined in MainWinPanel.java
            GUISystem.getParentDefWin(this).closePanel ( this, null );

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_EDIT                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_EDIT == e.getSource() )
            if ( currentTeamRec != null )
               edit() ;
            else
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_DELETE                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_DELETE == e.getSource() )
            if ( currentTeamRec != null )
               delete() ;
            else
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_ADD                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_ADD == e.getSource() )
            if ( currentTeamRec != null )
               addUsers() ;
            else
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_REMOVE                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_REMOVE == e.getSource() )
            if ( currentTeamRec != null )
               removeUsers() ;
            else
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_GRANT                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_GRANT == e.getSource() )
            if ( currentTeamRec != null )
               addAuths() ;
            else
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_REVOKE                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_REVOKE == e.getSource() )
            if ( currentTeamRec != null )
               revoke() ;
            else
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;
      }

   } /* actionPerformed */





   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }



   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when we have
    *  lost focus on the main window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when we are
    *  selected to show on the main window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the title of this
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_TEAMS ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon that represents
    *  this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_TEAMS ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default
    *  navigation panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( defaultNavPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will release our resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      removeAll () ;
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will simply return TRUE
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will simply refresh our vies
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      defaultNavPanel.refreshList() ;
      st_TEAM.setText ( "" ) ;
      st_DESCRIPT.setText ( "" ) ;
      authorities.removeAll() ;
      memberList.removeAll() ;
   }




   /*------------------------------------------------------------*/
   /* setTeamRec
    */
   /**
    *    This method will set the team rec that we
    *  are currently working with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setTeamRec ( TypeTeamRec teamRec ) {
      currentTeamRec = teamRec ;
      new Thread ( this, THREAD_REFRESH ).start() ;
   }




   /*------------------------------------------------------------*/
   /* addAuths
    */
   /**
    *    This method will add authorities to the selected
    *  team
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addAuths () {
      Vector vec = new ListDlg ( GUISystem.getParentDefWin ( this ),
                                 Str.getStr ( STR_ADD ),
                                 TypeList.getInstance().getTypeList ( TypeList.TYPE_AUTH ), GUISystem.getFontUtil() ).getResult() ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         String sql  = "" ;
         int    i    = 0 ;
         int    size = vec.size() ;                                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add each authority, and create the sql to save it       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            if ( !authVec.contains ( vec.elementAt(i) ) ) {
               authorities.add ( (MultiListRow) vec.elementAt(i) ) ;
               sql+="INSERT INTO TIGRIS.USERGROUPAUTH "+
                    "( USERGROUP, AUTHIND, DBUSER, CHANGEDTIME ) "+
                    "VALUES "+
                    "( "+currentTeamRec.getInd()+", "+
                    "  "+((TypeAuthRec)vec.elementAt(i)).getInd()+", "+
                    " '"+UserSystem.getUserId()+"', "+
                    " CURRENT TIMESTAMP)"+QITSqlRunner.END_DELIM ;
               authVec.addElement ( vec.elementAt(i) ) ;
            }
            i++ ;
         } /* while  */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save the SQL                                            */   
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( sql.length() > 0 )
            QITSqlRunner.sqlRunner ( sql ) ;

      }

   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }


   /**
     *  This method will return TRUE if the team name/descript
     * passed is already taken
     **/
   private boolean duplicateTeam ( String teamName, String teamDescript ) {
      Vector  teams = TypeList.getInstance().getTypeList ( TypeList.TYPE_TEAM ) ;
      boolean dup   = false ;

      if ( ( teams != null ) && ( teams.size() > 0 ) ) {
         int i     =0 ;
         int size  = teams.size() ;

         while ( ( !dup ) && ( i < size ) ) {
            TypeTeamRec teamRec = (TypeTeamRec) teams.elementAt(i) ;

            if ( ( teamRec.getShortDescript().equals ( teamName ) ) ||
                 ( teamRec.getLongDescript().equals ( teamDescript ) ) )
               dup=true ;
            i++ ;
         }
      }

      return ( dup ) ;
   }
   /**
    *  This method will give the Print support to the panel
    **/
   public Object getPrintObject() {
      return( new Print ( memberList, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }
   /**
    *  This method will give the Preview support to the panel
    **/
   public Object getPreviewObject() {
      return( new Preview ( memberList, GUISystem.getParentDefWin ( this ), Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }
} /* TeamMaintPanel */

