package com.ibm.nzna.projects.qit.admin.teammaint ;
/*-----------------------------------------------------------*/
/*   TeamSQL                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TeamSQL                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class holds static methods to read/update authorities for the Group Maint App
 *
 *
 * @version 1.0, 4/24/1998 
 * @author  David Lentz
 */
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   TeamSQL                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class TeamSQL extends Object implements AppConst {


    /*-----------------------------------------------------------*/
    /*   readAssoc                                               */
    /*                                                           */
    /*   Description:                                            */
    /*      This method will read the authorities associated     */
    /*   and the users associated to the user group passed       */
    /*                                                           */
    /*   Parameters:                                             */
    /*      userGroup - The integer indicator for the group      */
    /*      authVec   - The vector to populate with authorities  */
    /*      userVec   = The vector to populate with users        */
    /*                                                           */
    /*   Returns:                                                */
    /*      True, False - TRUE - No Errors                       */
    /*-----------------------------------------------------------*/
    public static boolean readAssoc ( int userGroup, Vector authVec, Vector userVec ) {
        String    rtnName  = "readAssoc" ;
        SQLMethod sql      = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
        Vector    retVec   = null ;
        int       sqlCode  = 0 ;
        boolean   rc       = false ;

        if ( (  authVec != null ) && ( userVec != null ) ) {
            try {
                Statement stmt = sql.createStatement() ;
                ResultSet rs   = null ;

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   First, read all of the Authorities                      */
                /*                                                           */
                /*-----------------------------------------------------------*/
                rs = stmt.executeQuery ( "SELECT DISTINCT AUTHIND FROM TIGRIS.USERGROUPAUTH "+
                                    "WHERE USERGROUP = "+userGroup ) ;

                while ( rs.next() )
                    authVec.addElement ( TypeList.getInstance().objectFromInd ( rs.getInt(1), TypeList.TYPE_AUTH ) ) ;

                rs.close() ;

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Now, read all of the users                              */
                /*                                                           */
                /*-----------------------------------------------------------*/
                rs = stmt.executeQuery ( "SELECT DISTINCT USERID FROM TIGRIS.USERIDUSERGROUP "+
                                    "WHERE USERGROUP = "+userGroup ) ;

                while ( rs.next() )
                    userVec.addElement ( UserSystem.getUserRecFromUserId ( rs.getString(1).trim() ) ) ;

                rs.close() ;


                /* We did not get any errors */
                rc = true ;

            } catch ( SQLException e ) {
                sql.rollBack() ;
                LogSystem.log ( 1, e ) ;
            }
        }

        sql.close() ;
        return( rc ) ;
    }





    /*------------------------------------------------------------*/
    /* validateTeamDelete
     */
    /**
     *    This method will validate that it is ok
     *  to delete a team
     *  
     *
     *  @param teamInd The indicator to validate with
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static boolean validateTeamDelete ( int teamInd ) {
        SQLMethod sql   = new SQLMethod ( DatabaseSystem.QUEST, "validateTeamDelete", 5 ) ;
        boolean   valid = false ;

        try {
            Statement stmt = sql.createStatement () ;
            ResultSet rs   = stmt.executeQuery ( "SELECT USERGROUP FROM TIGRIS.USERIDUSERGROUP WHERE USERGROUP = "+teamInd ) ;

            if ( !rs.next() )
                valid = true ;

            stmt.close() ;
        } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
        }
        return( valid ) ;
    }
} /* TeamSQL */
