package com.ibm.nzna.projects.qit.app ;   
/*-----------------------------------------------------------*/
/*   AppletRec                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AppletRec                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing.* ;
/**
 *      This class holds information about a single Quest  
 *   Applet that has been added to the Quest Application.  
 *   DO NOT CHANGE THIS CLASS!!!!!! DOING SO WILL SEVERELY 
 *   CRIPPLE USER'S DESKTOPS.                               
 *
 *
 * @version 1.0, 1/26/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AppletRec                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class AppletRec extends Object implements AppConst, java.io.Serializable, MultiListRow, MultiListSort {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000001 ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String      className    = null ;
   private QuestApplet applet       = null ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the AppletRec to the QuestApplet class
    *  name passed in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AppletRec ( String className ) {
      this.className = className ;
   }





   /*------------------------------------------------------------*/
   /* getClassName
    */
   /**
    *    This method will return the string representation
    *  of the QuestApplet's class name, including
    *  full package information
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getClassName () {
      return( className ) ;
   }





   /*------------------------------------------------------------*/
   /* createQuestApplet
    */
   /**
    *    This method will return a new instance
    *  of the QuestApplet that this AppletRec
    *  represents. Please keep in mind that a new instance of
    *  a QuestApplet does not mean that the QuestApplet has been added
    *  to a GUI, or initialized. It is only a new Instance.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return QuestApplet
    *//*---------------------------------------------------------*/
   public QuestApplet createQuestApplet () throws QuestAppletNotFoundException {
      QuestApplet retApplet = null ;

      try {
         retApplet = (QuestApplet) Class.forName ( className ).newInstance() ;
      } catch ( ClassNotFoundException cnfe ) {
         throw new QuestAppletNotFoundException ( "Cannot Find Quest Applet", className ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( retApplet )  ;
   }




   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will be called when we
    *  are displayed in a MultiList
    *  control
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int columnNum ) {
      Object retObj = null ;

      try {
         if ( applet == null )
            applet = createQuestApplet() ;

         switch ( columnNum ) {
            case 0 :
               retObj =  applet.getSmallIcon() ;
               break ;

            case 1 :
               retObj =  applet.getName() ;
               break ;

            case 2 :
               retObj =  applet.getDescription() ;
               break ;
         }
      } catch ( QuestAppletNotFoundException e ) {
         switch ( columnNum ) {
            case 0 :
               retObj =  ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.APPLET_UNKNOWN ) ;
               break ;

            case 1 :
               retObj =  e.toString() ;
               break ;

            case 2 :
               retObj = "" ;
               break ;
         }

      }

      return( retObj ) ;
   }




   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will be called when the 
    *  user wants to sort a column in the
    *  list we are appearing in
    *
    *  @param object The object we are comparing ourselves to
    *  @param column The column that should be compared
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( MultiListSort object, int column ) {
      AppletRec appletRec = (AppletRec) object ;
      int       rc        = 0 ;

      try {
         if ( applet == null )
            applet = createQuestApplet() ;

         switch ( column ) {
            case 1 :
               rc = appletRec.applet.getDescription().compareTo ( applet.getDescription() ) ;
               break ;

            case 2 :
               rc = appletRec.applet.getName().compareTo ( applet.getName() ) ;
               break ;
         }
      } catch ( Exception e ) {
         rc = 0 ;
      }

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* prepareForSerialization
    */
   /**
    *    This method will prepare this object
    *  so that it can be serialized
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void prepareForSerialization () {
      applet = null ;
   }




   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will determine if the
    *  object passed in is equal to this AppletRec
    *  
    *
    *  @param Obejct the object to compare to
    *  @param None
    *  @param None
    *
    *  @return TRUE = Equal, FALSE = not equal
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {
      if ( object instanceof AppletRec )
         return( ((AppletRec)object).className.equals ( className ) ) ;
      else if ( object instanceof String )
         return( ((String)object).equals ( className ) ) ;

      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name 
    *  of the QuestApplet that this record
    *  represents
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      try {
         if ( applet == null )
            applet = createQuestApplet () ;
         return( applet.getName() ) ;

      } catch ( Exception e ) {
         return( e.toString() ) ;
      } 
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon that
    *  represents the QuestApplet
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      try {
         if ( applet == null )
            applet = createQuestApplet () ;
         return( applet.getIcon() ) ;
      } catch ( Exception e ) {
         return( ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.APPLET_UNKNOWN ) ) ;
      } 
   }




} /* AppletRec */
