package com.ibm.nzna.projects.qit.app ;       
/*-----------------------------------------------------------*/
/*   AppletSystem                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AppletSystem                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.util.* ;       
import com.ibm.nzna.shared.util.* ;       
import java.util.Vector ;
/**
 * The AppletSystem governs all QuestApplets that are available to use. 
 * This system also handles the addition/removal of QuestApplets and has
 * static methods for getting lists of QuestApplets and instantiating a
 * QuestApplet.
 *
 * @version 1.0, 1/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AppletSystem                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class AppletSystem extends Object implements Runnable {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Thread Names                                            */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static final String THREAD_FIRE_APPLET_ADD    = "FAA" ;
    private static final String THREAD_FIRE_APPLET_REMOVE = "FAR" ;

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Static Members                                  */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static       Vector       appletVector     = null ;             
    private static final String       APPLET_FILE      = "quest.applets" ;
    private static       AppletSystem instance         = null ;
    private static       Vector       runningAppletVec = null ;
    private static       Vector       listenerVec      = null ;






    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create but not initialize
     *  the AppletSystem. A call to initialize
     *  will handle the reset
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public AppletSystem () {
        instance = this ;
    }





    /*------------------------------------------------------------*/
    /* initialize
     */
    /**
     *    This method will initialize the AppletSytem.
     *  Initializing includes reading in the available
     *  QuestApplets.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void initialize() {
        try {
            appletVector = (Vector) QuestUtil.readObjectFromDisk ( APPLET_FILE ) ;
            if ( appletVector == null )
                appletVector = new Vector ( 1, 1 ) ;
        } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
        }
    }






    /*------------------------------------------------------------*/
    /* addApplet
     */
    /**
     *    This method will add a QuestApplet
     *  to the list of available applets
     *  
     *
     *  @param appletRec The AppletRec representing the QuestApplet.
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void addApplet ( AppletRec appletRec ) {
        if ( appletVector == null )
            appletVector = new Vector ( 10,1  ) ;

        appletVector.addElement ( appletRec ) ;

        getInstance().writeToDisk() ;
        new Thread ( getInstance(), THREAD_FIRE_APPLET_ADD ).start() ;
    }





    /*------------------------------------------------------------*/
    /* removeApplet
     */
    /**
     *    This method will remove an applet from
     *  the list of available QuestApplets
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void removeApplet ( AppletRec appletRec ) {
        if ( appletVector != null ) {
            appletVector.removeElement ( appletRec ) ;

            getInstance().writeToDisk() ;
            new Thread ( getInstance(), THREAD_FIRE_APPLET_REMOVE ).start() ;
        }
    }

    /*------------------------------------------------------------*/
    /* getAppletList
     */
    /**
     *    This method will return a clone of the
     *  Quest Application's list of AppletRecs. This
     *  list should be freed by the caller
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static Vector getAppletList () {
        return( (Vector)appletVector.clone() ) ;
    }





    /*------------------------------------------------------------*/
    /* getInstance
     */
    /**
     *    Returns an instance of the AppletSystem
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static AppletSystem getInstance() {
        return( instance ) ;
    }





    /*------------------------------------------------------------*/
    /* createApplet
     */
    /**
     *    This method will create, initialize and start
     *  a QuestApplet. The AppletRec passed in
     *  represents that QuestApplet.
     *
     *  @param appletRec The AppletRec representing the QuestApplet
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void createApplet ( AppletRec appletRec ) {
        new InitializeQuestAppletThread ( appletRec ) ;
    }




    /*------------------------------------------------------------*/
    /* run
     */
    /**
     *    This thread will be used to fire
     *  events to our listeners
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void run () {
        String operation = Thread.currentThread().getName() ;       
        if ( ( listenerVec != null ) && ( listenerVec.size() > 0 ) ) {
            int    i         = 0 ;
            int    length    = listenerVec.size() ;

            while ( i < length ) {
                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Fire 'Add' event                                        */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( operation.equals ( THREAD_FIRE_APPLET_ADD ) )
                    ((AppletSystemListener)listenerVec.elementAt(i)).appletAdded() ;

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Fire 'Remove' event                                     */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( operation.equals ( THREAD_FIRE_APPLET_REMOVE ) )
                    ((AppletSystemListener)listenerVec.elementAt(i)).appletRemoved() ;
                i++ ;
            }

        }
    }





    /*------------------------------------------------------------*/
    /* addAppletSystemListener
     */
    /**
     *    This method will add an AppletSystemListener
     *  to the list of listeners
     *  
     *
     *  @param listener The listener to add
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void addAppletSystemListener ( AppletSystemListener listener ) {
        if ( listenerVec == null )
            listenerVec = new Vector ( 1, 1 ) ;

        listenerVec.addElement ( listener ) ;
    }




    /*------------------------------------------------------------*/
    /* removeAppletSystemListener
     */
    /**
     *    This method will remove an AppletSystemListener
     *  
     *  
     *
     *  @param listener The listener to remove
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void removeAppletSystemListener ( AppletSystemListener listener ) {
        if ( listenerVec != null )
            listenerVec.removeElement ( listener ) ;
    }





    /*------------------------------------------------------------*/
    /* writeToDisk
     */
    /**
     *    This method will write our vector
     *  of applets to the user's local drive
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void writeToDisk () {
        int i       = 0 ;
        if ( appletVector != null ) {
            int length  = appletVector.size() ;

            while ( i < length ) {
                ((AppletRec)appletVector.elementAt(i)).prepareForSerialization() ;
                i++;
            }

            QuestUtil.writeObjectToDisk ( APPLET_FILE, appletVector ) ;
        }
    }





    /*------------------------------------------------------------*/
    /* getAppletFromName
     */
    /**
     *    This method will look for the applet named
     *  'name' and return it
     *  
     *
     *  @param name The name of the applet to look for
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static AppletRec getAppletFromName ( String name ) {
        AppletRec appletRec = null ;
        AppletRec tempRec   = null ;
        int       i         = 0 ;

        if ( appletVector != null ) {
            int size = appletVector.size() ;

            while ( ( appletRec == null ) && (  i < size ) ) {
                tempRec = (AppletRec) appletVector.elementAt(i) ;
                if ( ((String)tempRec.getColumnData(1)).equals ( name ) )
                    appletRec = tempRec ;
                i++ ;
            }
        }

        return( appletRec ) ;
    }





    /*------------------------------------------------------------*/
    /* addRunningApplet
     */
    /**
     *    This method will be called when we
     *  need to add a running applet to the
     *  vector of running applets
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    protected static void addRunningApplet ( QuestApplet applet ) {
        if ( runningAppletVec == null )
            runningAppletVec = new Vector ( 1, 1 ) ;

        runningAppletVec.addElement ( applet ) ;
    }


} /* AppletSystem */
