package com.ibm.nzna.projects.qit.app ;   
/*-----------------------------------------------------------*/
/*   BrandMgrRec                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BrandMgrRec                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
/**
 * The BrandMgrRec is the Brand Manager Record for a user. This record 
 * holds the values that are in the TIGRIS.USERIDBRAND table.
 *
 *
 * @version 1.0, 4/7/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BrandMgrRec                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class BrandMgrRec extends PersistentRec implements MultiListRow {

   /*------------------------------------------------------------*/
   /*                                                            */
   /*   This is the serialization ID. If you add fields to this  */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is     */
   /* fine as long as you know what you are doing!               */
   /*                                                            */
   /* If you change this number, you break everyone's Bookmarks  */
   /* to the point where they will not load, so you better write */
   /* a converter or something to run on everyone's bookmarks    */
   /*                                                            */
   /*------------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String          brandMgrType       = null ;
   private int             brandInd           = 0 ;
   private String          prettyBrandMgrType = null ;
   private String          brandStr           = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor initializes our values
    *  
    *  
    *
    *  @param brandMgr The type of brand manager this represents
    *  @param brandInd The brand indicator defined in TIGRIS.TYPEBRAND
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BrandMgrRec ( String brandMgrType, int brandInd ) {
      this.brandMgrType = brandMgrType ;
      this.brandInd     = brandInd ;
   }





   /*------------------------------------------------------------*/
   /* getBrandInd
    */
   /**
    *    This method will return the brand indicator
    *  that this record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getBrandInd () {
      return( brandInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getBrandMgrType
    */
   /**
    *    This method will return the type of brand
    *  manager that this record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getBrandMgrType () {
      return( brandMgrType ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return our data to show
    *  in a column
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      try {
         switch ( column ) {
            case 0 :
               if ( brandStr == null ) {
                  if ( brandInd != 0 )
                     brandStr = Brands.getBrand ( brandInd ).toString() ;
               }
               retObj = brandStr ;
               break ;

            case 1 :
               if ( prettyBrandMgrType == null ) {
                  if ( brandMgrType.equals ( "DOC" ) )
                     prettyBrandMgrType = " (Document)"; //prettyBrandMgrType = " ("+Str.getStr ( STR_DOCUMENT )+")";
                  else if ( brandMgrType.equals ( "OA" ) )
                     prettyBrandMgrType = " (Avalon)" ;//prettyBrandMgrType = " ("+Str.getStr ( STR_AVALON )+")" ;
               }

               retObj = prettyBrandMgrType ;
               break ;
         }
      } catch ( Exception e ) {
         retObj = "" ;
      }

      return( retObj ) ;
   }






   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return a human readable
    *  string
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( getColumnData(0)+" "+getColumnData(1) ) ;
   }





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will return TRUE if the
    *  object passed is equal to this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object  ) {
      boolean equals = false ;

      if ( object instanceof BrandMgrRec ) {
         BrandMgrRec brandMgrRec = (BrandMgrRec) object ;

         try {
            equals = ( ( brandMgrType.equals ( brandMgrRec.brandMgrType ) ) &&
                       ( brandInd == brandMgrRec.brandInd ) ) ;
         } catch ( Exception e ) {
            equals = false ;
         }
      }

      return( equals ) ;
   }
} /* BrandMgrRec */
