package com.ibm.nzna.projects.qit.app ;  
/*-----------------------------------------------------------*/
/*   ChatServer                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ChatServer                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.net.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This server runs through the life of the input tool. It communicates with 
 * other input tools.
 *
 * @version 1.0, 5/8/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ChatServer                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ChatServer extends Object implements Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int LISTEN_PORT         = 7469 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static ChatServer     instance     = null ;
   private static String         ipAddress    = null ;
   private static ServerSocket   serverSocket = null ;

   private boolean      runServer      = true ;
   private Thread       serverThread   = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create and start the
    *  server. If the server is already running, nothing
    *  will happen
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ChatServer ( ) {                                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the instance static var is null, we are not running  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( instance == null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Assign local variables                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         instance = this ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Start the server                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         startServer() ;

      }
   }






   /*------------------------------------------------------------*/
   /* startServer
    */
   /**
    *    This method will start the server that will
    *  monitor our socket for communication from
    *  other tools
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void startServer () {
      try {
         serverSocket = new ServerSocket ( LISTEN_PORT ) ;
         ipAddress = InetAddress.getLocalHost().getHostAddress() ;
         LogSystem.log ( 1, "ChatServer serving from TCP/IP Address:"+ipAddress ) ;

         serverThread = new Thread ( this ) ;
         serverThread.start() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getLocalAddress
    */
   /**
    *    This method will return the local client's
    *  IP Address in the form of xxx.xxx.xxx.xxx
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static final String getLocalAddress () {
      return( ipAddress ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our server 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      while ( runServer ) {
         try {
            Socket socket  = serverSocket.accept() ;
            String command = getStringFromSocket ( socket ) ;
                                
         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* stopServer
    */
   /**
    *    This method will stop the server
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopServer () {
      runServer = false ;

      if ( serverThread != null )
         serverThread.interrupt() ;
   } /* stopServer */






   /*------------------------------------------------------------*/
   /* getStringFromSocket
    */
   /**
    *    This method will read a string from the
    *  socket and will return the string that was
    *  sent in the socket
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private static final String getStringFromSocket ( Socket socket ) throws Exception {
      BufferedInputStream in     = new BufferedInputStream ( socket.getInputStream() ) ;
      String              retStr = "" ;
      byte[]              bytes  = new byte[1024] ;
      boolean             c      = true ;

      while ( c ) {
         if ( in.read ( bytes, 0, bytes.length ) > 0 ) {
            retStr+=new String ( bytes ) ;
         } else
            c = false ;
      }

      in.close() ;
      return ( retStr ) ;
   }
} /* ChatServer */
