package com.ibm.nzna.projects.qit.app ;  
/*-----------------------------------------------------------*/
/*   ClipSystem                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ClipSystem                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
/**
 * This ClipSystem manages the internal Clipboard for the application. 
 * The current clipboard types defined are:<br><br>
 *
 * CLIPBOARD_DOCUMENT<br>
 * CLIPBOARD_PRODUCT<br>
 * CLIPBOARD_USER<br>
 * CLIPBOARD_QUESTION<br>
 * CLIPBOARD_ACTION<br>
 * CLIPBOARD_ANSWER<br>
 * CLIPBOARD_SYMPTOM<br>
 *
 * Although the predefined clipboards can be used, a new clipboard can
 * be created simply by passing a new name. The new name must be different
 * than one of the predefined names.<br>
 *
 * Any object can be placed on the clipboard. If you want a menu item for
 * 'Paste As ...' or 'Copy As ...' you will have to create that yourself.<br>
 *
 * The clipboard items will not be saved when the application closes.<br>
 *
 * There may be multiple clipboard objects. If there is multiple Clipboard
 * objects, you can use the com.ibm.nzna.projects.qit.gui.ClipboardDlg window
 * to show the objects and allow the user to select an object or objects.
 *
 * @version 1.0, 3/24/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ClipSystem                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ClipSystem extends Object implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Predefined Clipboard Types                              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String CLIPBOARD_DOCUMENT   = Str.getStr ( STR_CLIPBOARD_DOCUMENT ) ;
   public static final String CLIPBOARD_PRODUCT    = Str.getStr ( STR_CLIPBOARD_PRODUCT ) ;
   public static final String CLIPBOARD_USER       = Str.getStr ( STR_CLIPBOARD_USER ) ;
   public static final String CLIPBOARD_QUESTION   = Str.getStr ( STR_CLIPBOARD_QUESTION ) ;
   public static final String CLIPBOARD_ACTION     = Str.getStr ( STR_CLIPBOARD_ACTION ) ;
   public static final String CLIPBOARD_ANSWER     = Str.getStr ( STR_CLIPBOARD_ANSWER ) ;
   public static final String CLIPBOARD_SYMPTOM    = Str.getStr ( STR_CLIPBOARD_SYMPTOM ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Hashtable clipboards       = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates our clipboards
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ClipSystem () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the predefined clipboards                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      initializeClipboards() ;


   }






   /*------------------------------------------------------------*/
   /* initializeClipboards
    */
   /**
    *    This method will remove all of the current
    *  clipboards, and will recreate each one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeClipboards () {
      clipboards = null ;
      clipboards = new Hashtable() ;
      clipboards.put ( new Vector(), CLIPBOARD_DOCUMENT ) ;
      clipboards.put ( new Vector(), CLIPBOARD_PRODUCT ) ;
      clipboards.put ( new Vector(), CLIPBOARD_USER ) ;
      clipboards.put ( new Vector(), CLIPBOARD_QUESTION ) ;
      clipboards.put ( new Vector(), CLIPBOARD_ACTION ) ;
      clipboards.put ( new Vector(), CLIPBOARD_ANSWER ) ;
      clipboards.put ( new Vector(), CLIPBOARD_SYMPTOM ) ;
   }





   /*------------------------------------------------------------*/
   /* paste
    */
   /**
    *    get
    *  This method will return a vector of objects
    *  from the clipboard specified
    *
    *  @param clipBoard The clip board to get results from
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector get ( String clipBoard ) {
      Vector vec = (Vector) clipboards.get ( clipBoard ) ;

      return( vec ) ;
   }





   /*------------------------------------------------------------*/
   /* getObjectCount
    */
   /**
    *    This method will return the number of objects
    *  in a specific clipboard.
    *  
    *
    *  @param clipBoard The clipboard to count
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static int getObjectCount ( String clipBoard ) {
      Vector vec = get ( clipBoard ) ;

      if ( vec != null )
         return( vec.size() ) ;

      return( 0 ) ;
   }






   /*------------------------------------------------------------*/
   /* put
    */
   /**
    *    This method will place an object onto one
    *  of the internal clipboards
    *  
    *
    *  @param object The object to place
    *  @param clipBoard The clip board to put the object on
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void put ( Object object, String clipBoard ) {
      Vector vec = get ( clipBoard ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the vector is null, create a new clipboard for the   */
      /* object.                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( vec == null ) {
         clipboards.put ( new Vector(), clipBoard ) ;
         vec = (Vector) clipboards.get ( clipBoard ) ;
      }

      if ( vec != null )
         vec.addElement ( object ) ;
   }





   /*------------------------------------------------------------*/
   /* getAllClipboardNames
    */
   /**
    *    This method will return all of the clipboard
    *  names that is available
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector A Vector of Strings. Each string representing one clipboard
    *//*---------------------------------------------------------*/
   public static Vector getAllClipboardNames () {
      Vector      retVec = new Vector ( 2, 1 ) ;
      Enumeration enum   = clipboards.elements() ;

      while ( enum.hasMoreElements() ) 
         retVec.addElement ( enum.nextElement().toString() ) ;

      return ( retVec  ) ;
   }

} /* ClipSystem */
