package com.ibm.nzna.projects.qit.app ;  
/*-----------------------------------------------------------*/
/*   DateSystem                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DateSystem                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.util.* ;
/**
 * This system simply holds objects and methods useful methods
 * in manipulating Dates. Please DO NOT create instances of the CDate class,
 * use the DateSystem's instance of the class if you need to.
 *
 * @version 1.0, 3/14/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DateSystem                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class DateSystem extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static CDate cDate    = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method simply creates the 1 and only
    *  instance of CDate
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DateSystem () {
      cDate = new CDate () ;
   }





   /*------------------------------------------------------------*/
   /* getCurrentTimestamp
    */
   /**
    *    This method will return the current timestamp
    *  in DB/2 Timestamp format
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getCurrentTimestamp () {
      if ( cDate == null ) {
         cDate = new CDate () ;
      }
      return( cDate.getDate ( CDate.DATE_STAMP ) ) ;
   }


   /*------------------------------------------------------------*/
   /* getDate
    */
   /**
    *    This method will return a date in the current time, in the
    *  specified format.
    *  
    *
    *  @param One of the CDate date types (DATE_DATE, DATE_STAMP, DATE_DBDATE etc..)
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getDate ( int dateType ) {
      if ( cDate == null ) {
         cDate = new CDate () ;
      }
      return( cDate.getDate ( dateType ) ) ;
   }



   /*-----------------------------------------------------------*/
   /* prettyDateFromStamp
    */
   /**
    *    This method will return the 'pretty' print version
    *  of a date from a timestamp
    *  
    *             
    *  @param stamp The Time Stamp
    *  @param 
    *  @param 
    *
    *  @return String The PrettyPrint Date
    */
   public static String prettyDateFromStamp ( String stamp ) {
      if ( cDate == null ) {
         cDate = new CDate () ;
      }
      return( CDate.createDate ( CDate.parse ( stamp, CDate.DATE_YEAR, CDate.DATE_STAMP ),
                                 CDate.parse ( stamp, CDate.DATE_MONTH, CDate.DATE_STAMP ),
                                 CDate.parse ( stamp, CDate.DATE_DAY, CDate.DATE_STAMP ),
                                 CDate.parse ( stamp, CDate.DATE_HOUR, CDate.DATE_STAMP ),
                                 CDate.parse ( stamp, CDate.DATE_MINUTE, CDate.DATE_STAMP ),
                                 0,
                                 CDate.DATE_WW_DATETIME ) ) ;
   }


   public static String dateFromDBDate ( String stamp ) {
      if ( cDate == null ) {
         cDate = new CDate () ;
      }
      return( CDate.createDate ( CDate.parse ( stamp, CDate.DATE_YEAR, CDate.DATE_DBDATE ),
                                 CDate.parse ( stamp, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                 CDate.parse ( stamp, CDate.DATE_DAY, CDate.DATE_DBDATE ),
                                 CDate.parse ( stamp, CDate.DATE_HOUR, CDate.DATE_DBDATE ),
                                 CDate.parse ( stamp, CDate.DATE_MINUTE, CDate.DATE_DBDATE ),
                                 0,
                                 CDate.DATE_DATE ) ) ;
   }



   public static String prettyDateFromDate ( String date ) {
      try {
         if ( cDate == null ) {
            cDate = new CDate () ;
         }

         if ( ( date.charAt(2) != '/' ) &&
              ( date.charAt(2) != '-' ) )
            return( prettyDateFromDBDate ( date ) ) ;

         return( CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR, CDate.DATE_DATE ),
                                    CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                    CDate.parse ( date, CDate.DATE_DAY, CDate.DATE_DATE ),
                                    0,
                                    0,
                                    0,
                                    CDate.DATE_WW_DATETIME ) ) ;
      } catch ( Exception e ) {
      }
      return( date ) ;
   }

   /*-----------------------------------------------------------*/
   /* prettyDateFromDBDate
    */
   /**
    *    This method will return the 'pretty' print version
    *  of a date from a database
    *  
    *             
    *  @param dbDate The date
    *  @param 
    *  @param 
    *
    *  @return String The PrettyPrint Date
    */
   public static String prettyDateFromDBDate ( String dbDate ) {
      return( CDate.createDate ( CDate.parse ( dbDate, CDate.DATE_YEAR,  CDate.DATE_DBDATE ),
                                 CDate.parse ( dbDate, CDate.DATE_MONTH, CDate.DATE_DBDATE ),
                                 CDate.parse ( dbDate, CDate.DATE_DAY,   CDate.DATE_DBDATE ),
                                 0,
                                 0,
                                 0,
                                 CDate.DATE_WW_DATE ) ) ;
   }


   /*-----------------------------------------------------------*/
   /* prettyDateFromStampNoTime
    */
   /**
    *    This method will return the 'pretty' print version
    *  of a date from a timestamp without the time
    *  
    *             
    *  @param stamp The Time Stamp
    *  @param 
    *  @param 
    *
    *  @return String The PrettyPrint Date
    */
   public static String prettyDateFromStampNoTime ( String stamp ) {
      if ( cDate == null ) {
         cDate = new CDate () ;
      }
      return( CDate.createDate ( CDate.parse ( stamp, CDate.DATE_YEAR, CDate.DATE_STAMP ),
                                 CDate.parse ( stamp, CDate.DATE_MONTH, CDate.DATE_STAMP ),
                                 CDate.parse ( stamp, CDate.DATE_DAY, CDate.DATE_STAMP ),
                                 CDate.parse ( stamp, CDate.DATE_HOUR, CDate.DATE_STAMP ),
                                 CDate.parse ( stamp, CDate.DATE_MINUTE, CDate.DATE_STAMP ),
                                 0,
                                 CDate.DATE_WW_DATE ) ) ;
   }

} /* DateSystem */
