package com.ibm.nzna.projects.qit.app ;              
/*-----------------------------------------------------------*/
/*   DebugSenderWizard                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DebugSenderWizard                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import java.io.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.projects.qit.gui.* ;
/**
 * This wizard guides the user through submitting a bug 
 *
 *
 * @version 1.0, 1/24/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DebugSenderWizard                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class DebugSenderWizard extends JDialog implements WizardListener, AppConst {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */      
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Wizard                 wizardPanel        = null ;
   private InfoStep               welcomeStep        = null ;
   private DebugSenderInfoStep    infoStep           = null ;
   private DebugSenderProcessStep processStep        = null ;
   


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor guides the user through
    *  everything and does not return until all
    *  information is gathered
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DebugSenderWizard ( ) {
      super ( new Frame(), "Send Debug", true ) ;

      createWizard() ;

      setSize ( 495, 375 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* createWizard
    */
   /**
    *    This method will create all of our controls
    *  for the wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createWizard () {
      CreateWizardWin statusWin = new CreateWizardWin ( GUISystem.getParentFrame(this), 5 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Wizard panel                                 */     
      /*                                                           */
      /*-----------------------------------------------------------*/
      WinUtil.centerWindow ( statusWin ) ;
      try {
         wizardPanel = new Wizard ( ImageSystem.getImageIcon ( this, "skins"+
                                                               File.separator+
                                                               PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                                                               File.separator+
                                                               "debug_wizard.gif" ) ) ;                     
         statusWin.setValue ( 1 ) ;
      } catch ( Exception e ) {
         wizardPanel = new Wizard () ;
      }
      wizardPanel.addWizardListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our steps                                        */       
      /*                                                           */
      /*-----------------------------------------------------------*/
      welcomeStep = new InfoStep ( Str.getStr ( STR_DEBUG_WIZARD_WELCOME_HELP ),
                                   Str.getStr ( STR_DEBUG_WIZARD_WELCOME ) ) ;
      statusWin.setValue ( 2 ) ;

      infoStep = new DebugSenderInfoStep ( ) ;
      statusWin.setValue ( 3 ) ;

      processStep = new DebugSenderProcessStep ( infoStep ) ;
      statusWin.setValue ( 4 ) ;
      
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our window                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( new BorderLayout () ) ;
      getContentPane().add ( wizardPanel ) ;
      statusWin.setValue ( 5 ) ;
      wizardPanel.setCurrentStep ( welcomeStep ) ;

      statusWin.dispose() ;
   }




   /*------------------------------------------------------------*/
   /* cancelPerformed
    */
   /**
    *    This method will be called if the user
    *  hit the 'Cancel' button on the Wizard
    *  
    *
    *  @param e The wizard event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cancelPerformed ( WizardEvent e ) {
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* finishPerformed
    */
   /**
    *    This method will be called when the
    *  user presses the finish button
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finishPerformed ( WizardEvent e) {
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* nextPerformed
    */
   /**
    *    This method will be called when the
    *  next button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void nextPerformed ( WizardEvent e ) {
      WizardStep step = wizardPanel.getCurrentStep() ;


      if ( step == welcomeStep )
         wizardPanel.setCurrentStep ( infoStep ) ;
      else if ( step == infoStep )
         wizardPanel.setCurrentStep ( processStep ) ;

      adjustButtons () ;
   }


   /*------------------------------------------------------------*/
   /* backPerformed
    */
   /**
    *    This method will be called when the
    *  back button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void backPerformed ( WizardEvent e ) {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == infoStep )
         wizardPanel.setCurrentStep ( welcomeStep ) ;
      else if ( step == processStep )
         wizardPanel.setCurrentStep ( infoStep ) ;

      adjustButtons ( ) ;
   }





   /*------------------------------------------------------------*/
   /* adjustButtons
    */
   /**
    *    This method will look at what step is
    *  currently showing and will decide which
    *  buttons to show in the wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void adjustButtons () {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == welcomeStep )
         wizardPanel.showButtons ( true, false, true, false ) ;
      else if ( step == processStep )
         wizardPanel.showButtons ( false, true, false, true ) ;
      else
         wizardPanel.showButtons ( true, true, true, false ) ;
   }





} /* DebugSenderWizard */
