package com.ibm.nzna.projects.qit.app ;  
/*-----------------------------------------------------------*/
/*   DisplayRec                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DisplayRec                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
/**
 * This DisplayRec interface defines methods that need to be  
 * used by a record that displays another type of record. For example,
 * a UserRec can not be put into a MultiList as it is, But a UserDisplayRec
 * can. The UserDisplayRec holds what UserRec it is displaying  and it implements
 * the MultiListRow so that it can be put into a list. The methods in this interface
 * ensure that you can easily get what actual data is being represented.
 *
 * @version 1.0, 4/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DisplayRec                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public interface DisplayRec {




   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will get the data that the
    *  displayRec is displaying
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public abstract Object getData () ;






   /*------------------------------------------------------------*/
   /* setSort
    */
   /**
    *    This method should set the sorting type for
    *  this object to sort with in a list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void setSort ( int sort ) ;





   /*------------------------------------------------------------*/
   /* containsChildren
    */
   /**
    *    This method should return TRUE or FALSE depending
    *  on wether this display record contains children
    *  display records. This is normally used only in a JTree
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public abstract boolean containsChildren() ;





   /*------------------------------------------------------------*/
   /* getChildren
    */
   /**
    *    This method should return a Vector of
    *  children objects relating to this parent DisplayRec
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public abstract Vector getChildren () ;





   /*------------------------------------------------------------*/
   /* addChild
    */
   /**
    *    This method should add the DisplayRec
    *  passed in as a child of this object
    *  
    *
    *  @param displayRec The DisplayRec to add as a child
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void addChild ( DisplayRec displayRec ) ;





   /*------------------------------------------------------------*/
   /* removeChild
    */
   /**
    *    This method should remove the DisplayRec
    *  from the list of children
    *  
    *
    *  @param displayRec The DisplayRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void removeChild ( DisplayRec displayRec ) ;
} /* DisplayRec */
