package com.ibm.nzna.projects.qit.app ;     
/*-----------------------------------------------------------*/
/*   EntrySystem                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EntrySystem                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.text.* ;
import com.ibm.nzna.projects.qit.type.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The EntrySystem is used to configure JTextFields and JTextAreas in the Quest Application 
 *
 *
 * @version 1.0, 1/26/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EntrySystem                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class EntrySystem extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   public static members                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final MaskDocument defaultMask = new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 1024 ) ;
   public static final MaskDocument defaultDateMask = null ;


   /*------------------------------------------------------------*/
   /* getDateMask
    */
   /**
    *    This method will return the mask that should
    *  be applied to a date field
    *  
    *
    *  @param component a subclass of JTextComponent
    *  @param None
    *  @param None
    *
    *  @return MaskDocument
    *//*---------------------------------------------------------*/
   public static MaskDocument getDateMask ( JTextComponent component ) {
      MaskDocument dateMask = null ;

      try {
         TypeCountryCodeRec countryRec = (TypeCountryCodeRec) QITTypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_COUNTRY ),
                                                                                                        QITTypeList.TYPE_COUNTRYCODE ) ;

         //if ( countryRec != null )
         //   dateMask = new MaskDocument ( MaskDocument.MASKTYPE_STRING, countryRec.getDateMask(), component ) ;
         dateMask = new MaskDocument ( MaskDocument.MASKTYPE_STRING, "99 UAA 9999", component ) ;
      } catch ( Exception e ) {
         dateMask = new MaskDocument ( MaskDocument.MASKTYPE_STRING, "99/99/9999", component ) ;
      }

      return( dateMask ) ;
   }




   /*------------------------------------------------------------*/
   /* getCurrencyMask
    */
   /**
    *    This method will return the mask that should
    *  be used for entering currency.
    *  
    *                                 
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getCurrencyMask () {
      return( "$XX.XX" ) ;
   }





   /*------------------------------------------------------------*/
   /* getStringDateMask
    */
   /**
    *    This method will simply return a
    *  string version of the date mask being used
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getStringDateMask () {
      return( "99/99/9999" ) ;
   }

   /*------------------------------------------------------------*/
   /* fixDateField
    */
   /**
    *    This method will fix a date field so that
    *  the date entered will be valid. For example,
    *  it will append 19 or 20 to a year with only 2 digits
    *
    *  @param textField The JTextField to alter
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void fixDateField ( javax.swing.JTextField textField ) {
   }


   /**
     *  This method will create an automatic focus/action listener
     * for an entry field that displays a date to use. If the entry
     * field uses this listener then the dates entered into it will
     * automatically be fixed (VIA the fixDateField method) when the user
     * tabs out or presses enter
     **/
   public static DateFieldFixer getDateFieldFixer () {
      return ( new DateFieldFixer() ) ;
   }

} /* EntrySystem */


/**
  *  This class listens for the user to tab out of an entry field, or to press enter
  * in an entry field. IF the user does, this class will automatically fix a 2
  * digit date
  **/
 class DateFieldFixer implements FocusListener, ActionListener {

   public void focusGained ( FocusEvent e ) {
   }

   public void focusLost ( FocusEvent e ) {
      if ( e.getSource() instanceof JTextField ) 
         EntrySystem.fixDateField ( (JTextField) e.getSource() ) ;
   }

   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() instanceof JTextField ) 
         EntrySystem.fixDateField ( (JTextField) e.getSource() ) ;
   }

}
