package com.ibm.nzna.projects.qit.app ;   
/*-----------------------------------------------------------*/
/*   GUISystem                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: GUISystem                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The GUI System has static methods useful for building
 * GUI Applications
 *
 * @version 1.0, 1/9/100 
 * @author  David Lentz
 */
import javax.swing.* ;
import javax.swing.border.* ;
import java.awt.* ;
import java.io.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.form.* ;
import java.net.* ;

//import com.l2fprod.gui.*;
//import com.l2fprod.gui.plaf.skin.*;
//import com.l2fprod.util.*;

/*-----------------------------------------------------------*/
/*                                                           */
/*   GUISystem                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class GUISystem extends Object implements AppConst {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static InfoBoxDlg          printBox     = null ;         
   private static FontUtil            fontUtil     = null ;




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public Static Vars                                      */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final TitledBorder etchedBorder    = new TitledBorder("") ;
   public static final BevelBorder  loweredBorder   = new BevelBorder(BevelBorder.LOWERED) ;
   public static final BevelBorder  raisedBorder    = new BevelBorder(BevelBorder.RAISED) ;
   public static final LineBorder   blackBorder     = new LineBorder ( Color.black ) ;
   public static final EmptyBorder  emptyBorder     = new javax.swing.border.EmptyBorder(0,0,0,0) ;
   public static final EmptyBorder  emptyFillBorder = new javax.swing.border.EmptyBorder(2,2,2,2) ;
   public static final LineBorder   grayBorder      = new LineBorder ( Color.darkGray ) ;
   public static final Dimension    defaultNavSize  = new Dimension ( 200, 200 ) ;
   public static       boolean      os2             = false ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Colors                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final Color       oddLinesColor = new Color ( 240, 240, 240 ) ;
   public static final Color       defaultGray   = new Color ( 214, 212 ,206 ) ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the one instance of
    *  GUI System
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public GUISystem () {

      setLookAndFeel() ;

      os2 = PropertySystem.getBool ( PropertySystem.PROPERTY_OS2 ) ;


   }


   /*------------------------------------------------------------*/
   /* setLookAndFeel
    */
   /**
    *    This method will set the look and feel
    *  for the Quest Application to the look and feel passed in.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setLookAndFeel ( String lookFeel ) {

      try {
         JPanel jp = new JPanel() ; // Created only for passing to the ImageSystem.getImageIcon method


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the UI Look and Feel                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log ( 1, "Setting look and feel to "+lookFeel ) ;
         if ( lookFeel.indexOf ( "ick Meta" ) != -1 )
            UIManager.setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
         else
            UIManager.setLookAndFeel ( lookFeel ) ;                                                    
         //setSkinLF() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our properties                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         initializeProperties() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Load our own Default Values into the Swing UI           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         UIManager.getDefaults().put ("Tree.expandedIcon",    ImageSystem.getImageIcon ( jp, ImageSystem.OPEN_TREE ) ) ;
         UIManager.getDefaults().put ("Tree.collapsedIcon",   ImageSystem.getImageIcon ( jp, ImageSystem.CLOSE_TREE ) ) ;
         UIManager.getDefaults().put ("Form.foreground",      PropertySystem.get ( PropertySystem.PROPERTY_FORM_FOREGROUND ) ) ;
         UIManager.getDefaults().put ("Form.valueForeground", PropertySystem.get ( PropertySystem.PROPERTY_FORM_VALUE_FOREGROUND ) ) ;
         UIManager.getDefaults().put ("Form.titleForeground", PropertySystem.get ( PropertySystem.PROPERTY_FORM_TITLE_FOREGROUND ) ) ;
         UIManager.getDefaults().put ("Form.font",            PropertySystem.get ( PropertySystem.PROPERTY_FORM_FONT ) ) ;
         UIManager.getDefaults().put ("Form.valueFont",       PropertySystem.get ( PropertySystem.PROPERTY_FORM_VALUE_FONT ) ) ;
         UIManager.getDefaults().put ("Form.titleFont",       PropertySystem.get ( PropertySystem.PROPERTY_FORM_TITLE_FONT ) ) ;

         if ( MainWindow.getInstance() != null )
            SwingUtilities.updateComponentTreeUI(MainWindow.getInstance());


         PropertySystem.putString ( PropertySystem.PROPERTY_LAF, lookFeel ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

   }




   /*------------------------------------------------------------*/
   /* setLookAndFeel
    */
   /**
    *    This is the default setLookAndFeel method will set the look and feel
    *  for the Quest Application from what is currently in the PropertySystem.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setLookAndFeel () {
      setLookAndFeel ( PropertySystem.getString ( PropertySystem.PROPERTY_LAF ) ) ;
      //setSkinLF() ;
   }


   /*------------------------------------------------------------*/
   /* setLookAndFeel
    */
   /**
    *    This method will set the look and feel
    *  for the an external window. Please call the default setLookAndFeel
    *  to set the application look and feel
    *  
    *
    *  @param JDialog the Dialog to set the LANDF for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setLookAndFeel ( JDialog dialog ) {
      String lookFeel = PropertySystem.getString ( PropertySystem.PROPERTY_LAF ) ;

      if ( dialog != null )
         SwingUtilities.updateComponentTreeUI(dialog);

   }


   /*------------------------------------------------------------*/
   /* setLookAndFeel
    */
   /**
    *    This method will set the look and feel
    *  for the an external window. Please call the default setLookAndFeel
    *  to set the application look and feel
    *  
    *
    *  @param JDialog the Dialog to set the LANDF for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setLookAndFeel ( JFrame frame ) {
      String lookFeel = PropertySystem.getString ( PropertySystem.PROPERTY_LAF ) ;

      if ( frame != null )
         SwingUtilities.updateComponentTreeUI(frame);

   }


   /*------------------------------------------------------------*/
   /* getRowHeight
    */
   /**
    *    This method will return the average
    *  height of a row of Components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static int getRowHeight () {
      return( 18 ) ;
   }


   /**
     * Puts up a standard print box with the Title and Text that are retrieved
     * from the language file. This printBox is created with the MainWindow as it's
     * parent
     *
     * @param titleId The ID of the Title to show
     * @param textId The ID of the text to show
     *
     * @returns TRUE if the user hit either YES or OK, FALSE if the user hit NO or CANCEL
     **/
   public static boolean printBox ( int titleId, int textId ) {
      return( printBox ( Str.getStr ( titleId ), Str.getStr ( textId ) ) ) ;
   }

   /**
     * Puts up a standard Print Box with the title and text given. The parent of this
     * printBox will be the MainWindow.
     *
     * @param title The title to show
     * @param text The text to show
     *
     * @returns TRUE if the user hit either YES or OK, FALSE if the user hit NO or CANCEL
     **/
   public static boolean printBox ( String title, String text ) {
      if ( printBox == null ) {
         printBox = new InfoBoxDlg ( MainWindow.getInstance(), title, text ) ;
      }

      return( configureAndShowPrintBox ( printBox, title, text, true ) ) ;
   }

   /**
     * Puts up a standard InfoBoxDlg with the Title and Text given under the given parent Frame
     *
     * @param parent The parent Frame for the InfoBoxDlg.
     * @param title The title to show
     * @param text The text to show
     *
     * @returns TRUE if the user hit either YES or OK, FALSE if the user hit NO or CANCEL
     **/
   public static boolean printBox ( Frame parent, String title, String text ) {
      InfoBoxDlg newBox = new InfoBoxDlg ( parent, title, text ) ;

      return( configureAndShowPrintBox ( newBox, title, text, false ) ) ;
   }


   /**
     * Puts up a standard InfoBoxDlg with the Title and Text given under the given parent Frame
     *
     * @param parent The parent Frame for the InfoBoxDlg.
     * @param titleId The ID of the Title to show
     * @param textId The ID of the text to show
     *
     * @returns TRUE if the user hit either YES or OK, FALSE if the user hit NO or CANCEL
     **/
   public static boolean printBox ( Frame parent, int title, int text ) {
      return( printBox ( parent, Str.getStr ( title ), Str.getStr ( text ) ) ) ;
   }


   /**
     * Puts up a standard InfoBoxDlg with the Title and Text given under the given parent JDialog
     *
     * @param parent The parent JDialog for the InfoBoxDlg.
     * @param titleId The ID of the Title to show
     * @param textId The ID of the text to show
     *
     * @returns TRUE if the user hit either YES or OK, FALSE if the user hit NO or CANCEL
     **/
   public static boolean printBox ( JDialog parent, int title, int text ) {
      return( printBox ( parent, Str.getStr ( title ), Str.getStr ( text ) ) ) ;
   }


   /**
     * Puts up a standard InfoBoxDlg with the Title and Text given under the given parent JDialog
     *
     * @param parent The parent JDialog for the InfoBoxDlg.
     * @param title The title to show
     * @param text The text to show
     *
     * @returns TRUE if the user hit either YES or OK, FALSE if the user hit NO or CANCEL
     **/
   public static boolean printBox ( JDialog parent, String title, String text ) {
      InfoBoxDlg newBox = new InfoBoxDlg ( parent, title, text ) ;

      return( configureAndShowPrintBox ( newBox, title, text, false ) ) ;
   }

   /**
     * This method will configure a printBox to the title and text specified
     * and will then show it.
     *
     * @param title The title to show in the print box
     * @param text The text of the printBox
     * @param keepAlive TRUE to keep the printBox after the user closes it, FALSE to destroy it
     **/
   private static boolean configureAndShowPrintBox  ( InfoBoxDlg printBox, String title, String text, boolean keepAlive ) {
      boolean ret = false ;

      try {
         ImageIcon   icon      = null ;
         String      tempTitle = title.toUpperCase() ;

         printBox.showOk             ( false ) ;
         printBox.showCancel         ( false ) ;
         printBox.showNo             ( false ) ;
         printBox.showYes            ( false ) ;
         printBox.showNeverShowAgain ( false ) ;

         /*------------------------------------------------*/
         /*                                                */
         /*  Figure out what Icon and buttons to show from */
         /* the title                                      */
         /*                                                */
         /*------------------------------------------------*/
         if ( ( tempTitle.indexOf ( "ERROR" )   >= 0 ) ||
              ( tempTitle.indexOf ( "BAD" )     >= 0 ) ||
              ( tempTitle.indexOf ( "SQL" )     >= 0 ) ||
              ( tempTitle.indexOf ( "MISSING" ) >= 0 ) ) {
            icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.ERROR ) ;
            printBox.showOk ( true ) ;
         } /* if this is an error box */
         else
            if ( ( tempTitle.indexOf ( "?" )        >= 0 ) ||
                 ( tempTitle.indexOf ( "UESTIO" )   >= 0 ) ||
                 ( tempTitle.indexOf ( "WHO" )      >= 0 ) ||
                 ( tempTitle.indexOf ( "WHAT" )     >= 0 ) ||
                 ( tempTitle.indexOf ( "WHEN" )     >= 0 ) ||
                 ( tempTitle.indexOf ( "WHERE" )    >= 0 ) ||
                 ( tempTitle.indexOf ( "WHY" )      >= 0 ) ) {
            icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.QUESTION ) ;
            printBox.showYes ( true ) ;
            printBox.showNo ( true ) ;
         } /* else, this is a question box */
         else {
            icon = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.INFO ) ;
            printBox.showOk ( true ) ;
         } /* else, this is an info box */


         /*--------------------------------------*/
         /*                                      */
         /*  Set the title, description and icon */
         /*                                      */
         /*--------------------------------------*/
         printBox.setDescription ( text ) ;
         printBox.setTitle ( title ) ;

         if ( icon != null ) {
            printBox.setImageIcon ( icon ) ;
         }

         /*------------------------------*/
         /*                              */
         /*  Finally, show the print box */
         /*                              */
         /*------------------------------*/
         printBox.getResult ( keepAlive ) ;

         switch ( printBox.getLastButtonHit () ) {
            case InfoBoxDlg.OK_HIT :
            case InfoBoxDlg.YES_HIT :
               ret = true ;
               break ;

            case InfoBoxDlg.NO_HIT :
            case InfoBoxDlg.CANCEL_HIT :
               ret = false ;
               break ;
         }

      } catch ( Exception e ) {
         LogSystem.log( 1, e, false );
      }

      return( ret ) ;
   } /* PrintBox */





   /*------------------------------------------------------------*/
   /* setPreferredButton
    */
   /**
    *    This method will take a button and
    *  looks at it's size. If it is normally larger
    *  than 70x25, it will leave the button alone,
    *  otherwise the button's preferred size will be set.
    *
    *  @param button The button to resize
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setPreferredButton ( JComponent button ) {
      Dimension size = button.getPreferredSize() ;

      if ( size.width < 70 )
         size.width = 70 ;
      if ( size.height < 25 )
         size.height = 25 ;

      button.setMinimumSize ( size ) ;
      button.setSize ( size ) ;
   }





   /*------------------------------------------------------------*/
   /* getParentDefWin
    */
   /**
    *    This method will return the parent
    *  AppDefaultWin of a component passed
    *  in
    *
    *  @param comp
    *  @param None
    *  @param None
    *
    *  @return AppDefaultWin instance, NULL if none
    *//*---------------------------------------------------------*/
   public static AppDefaultWin getParentDefWin ( Component c ) {
      AppDefaultWin ret = null ;

      while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
         if ( c instanceof AppDefaultWin )
            ret = (AppDefaultWin)c ;
      }

      if ( ret == null )
         ret = MainWindow.getInstance() ;


      return( ret ) ;
   }



   /*------------------------------------------------------------*/
   /* getParentFrame
    */
   /**
    *    This method will return the parent
    *  Frame of a component passed
    *  in
    *
    *  @param comp
    *  @param None
    *  @param None
    *
    *  @return AppDefaultWin instance, NULL if none
    *//*---------------------------------------------------------*/
   public static Frame getParentFrame ( Component c ) {
      Frame ret = null ;

      while ( ( ret == null ) && (( c = c.getParent()) != null ) ) {
         if ( c instanceof Frame )
            ret = (Frame)c ;
      }

      if ( ret == null ) {
         ret = MainWindow.getInstance() ;
      }


      return( ret ) ;
   }






   /*------------------------------------------------------------*/
   /* setPropertiesOnPanel
    */
   /**
    *    This method will set all GUI Properties
    *  for all Components on the panel passed in 
    *  recursivly. This includes Font, Color etc..
    *
    *  @param panel The Panel to traverse through
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setPropertiesOnPanel ( Container panel ) {

      if ( !os2 ) {
         Component   c         = null ;
         Component[] compArray = panel.getComponents() ;
         int         i         = 0 ;

         //panel.setBackground ( defaultGray ) ;

         while ( i < compArray.length ) {

            c = compArray[i] ;    

            if ( !(c instanceof ApplicationTitlePanel ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   ToolBarButton                                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( c instanceof ToolBarButton )
                  ((ToolBarButton)c).setProperties() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   AbstractButton - Set Font                               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( c instanceof AbstractButton ) {
                  if ( ( c instanceof DButton ) || ( c instanceof JButton ) )
                     setPreferredButton ( (JComponent)c ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If the component is another container, call us again    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( c instanceof Container ) {
                  if ( c instanceof JScrollPane ) {
                     ((JScrollPane)c).setBorder ( loweredBorder ) ;
                  } else if ( c instanceof ButtonPanel ) {
                     ((ButtonPanel)c).setArrowImage ( ImageSystem.getImageIcon ( MainWindow.getInstance() ,ImageSystem.BUTTONPANEL_LEFT ) ,
                                                      ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.BUTTONPANEL_RIGHT ) ) ;
                  }

                  setPropertiesOnPanel ( (Container)c ) ;
               }
            }
            i++ ;
         } /* while  */
      } /* if !os2 */
   }




   /*------------------------------------------------------------*/
   /* getFontUtil
    */
   /**
    *    This method will return the FontUtil
    *  object for the application
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static FontUtil getFontUtil() {
      if ( fontUtil == null )
         fontUtil = new FontUtil() ;

      return( fontUtil ) ;
   }


   /*------------------------------------------------------------*/
   /* getFileOpen
    */
   /**
    *    This method will return a file object that is
    *  received from a file open dialog.
    *  
    *
    *  @param title The title of the window
    *  @param currentDirectory The current directory to open in
    *  @param loadDlg TRUE = show the Load Dialog, FALSE = show the Save Dialog
    *
    *  @return File[] An array of files. The first file is the actual file, the second is a directory file
    *//*---------------------------------------------------------*/
   public static File[] getFileOpen ( Frame parent, String title, String currentDirectory, String initFileName, boolean loadDlg ) {
      return( getFileOpen ( parent, title, currentDirectory, initFileName, loadDlg, null ) ) ;
   }


   public static File[] getFileOpen ( Frame parent, String title, String currentDirectory, String initFileName, boolean loadDlg, javax.swing.filechooser.FileFilter fileFilter ) {
      File[] file = null ;

      /* Check to see what file open dialog we use. For some systems, the
         swing file open may not work correctly. */
      try {
         if ( PropertySystem.getBool ( PropertySystem.PROPERTY_OS2 ) ) {
            JFileChooser fileChooser = new JFileChooser ( currentDirectory ) ;

            if ( initFileName != null )
               fileChooser.setSelectedFile ( new File ( initFileName ) ) ;

            fileChooser.setDialogTitle ( title ) ;

            if ( fileFilter != null )
               fileChooser.setFileFilter ( fileFilter ) ;
            

            if ( ( loadDlg ) && ( fileChooser.showOpenDialog ( parent ) == fileChooser.APPROVE_OPTION ) ) {
               file = new File[2] ;
               file[0] = fileChooser.getSelectedFile() ;
               file[1] = fileChooser.getCurrentDirectory() ;
            }

            else if ( ( !loadDlg ) && ( fileChooser.showSaveDialog ( parent ) == fileChooser.APPROVE_OPTION ) ) {
               file = new File[2] ;
               file[0] = fileChooser.getSelectedFile() ;
               file[1] = fileChooser.getCurrentDirectory() ;
            }

         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, use the platform's default file open         */                                      
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            FileDialog fileDialog = new FileDialog ( parent, title, loadDlg ? FileDialog.LOAD : FileDialog.SAVE ) ;
            String     fileName   = "" ;
            String     directory  = "" ;

            if ( currentDirectory != null )
               fileDialog.setDirectory ( currentDirectory ) ;

            if ( initFileName != null )
               fileDialog.setFile ( initFileName ) ;

            fileDialog.setVisible ( true ) ;

            fileName = fileDialog.getFile() ;
            directory = fileDialog.getDirectory() ;

            if ( ( fileName != null ) && ( fileName.length() > 0 ) ) {
               file = new File[2] ;
               file[0] = new File ( directory+File.separator+fileName ) ;
               file[1] = new File ( directory ) ;
            }
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( file ) ;
   }


   /*------------------------------------------------------------*/
   /* getFileOpen
    */
   /**
    *    This method will return a file object that is
    *  received from a file open dialog
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static File[] getFileOpen ( Frame parent, String title, String currentDirectory, boolean loadDlg ) {
      return( getFileOpen ( parent, title, currentDirectory, null, loadDlg ) ) ;
   }


   /*------------------------------------------------------------*/
   /* getFileOpen
    */
   /**
    *    This method will return a file object that is
    *  received from a file open dialog.
    *  
    *
    *  @param title The title of the window
    *  @param currentDirectory The current directory to open in
    *
    *  @return File[] An array of files. The first file is the actual file, the second is a directory file
    *//*---------------------------------------------------------*/
   public static File[] getFileOpen ( Frame parent, String title, String currentDirectory ) {
      return( getFileOpen ( parent, title, currentDirectory, null, true ) ) ;
   }





   /*------------------------------------------------------------*/
   /* setComponentOpaque
    */
   /**
    *    This method will set the component passed
    *  and any children recursivly to the opaque false
    *  passed
    *
    *  @param component The component to change
    *  @param opaque FALSE/TRUE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setComponentOpaque ( JComponent component, boolean opaque ) {

      component.setOpaque ( opaque ) ;

      if ( component instanceof Container ) {
         Component   c         = null ;
         Component[] compArray = component.getComponents() ;


         if ( compArray != null )
            for ( int i = 0 ; i < compArray.length; i++ ) {
               if ( compArray[i] instanceof JComponent )
                  setComponentOpaque ( (JComponent) compArray[i], opaque ) ;
            }

      }
   }





   /*------------------------------------------------------------*/
   /* 
    */
   /**
    *    This little method simply prints out the names
    *  of each Swing UI default
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void printSwingUIDefaults () {
      UIDefaults defaults = UIManager.getDefaults() ;
      Enumeration e = defaults.keys() ;

      while ( e.hasMoreElements() ) {
         LogSystem.log ( 1, e.nextElement().toString() ) ;
      }

   }


   /**
     *  This method will set all properties in the PropertySystem if 
     * the PropertySystem does not have the value set yet
     **/
   private static void initializeProperties () {
      Font defaultFont = (Font)javax.swing.UIManager.getDefaults().get ( "Label.font" ) ;

      if ( PropertySystem.get ( PropertySystem.PROPERTY_FORM_FOREGROUND ) == null )
         PropertySystem.put ( PropertySystem.PROPERTY_FORM_FOREGROUND, Color.black ) ;

      if ( PropertySystem.get ( PropertySystem.PROPERTY_FORM_VALUE_FOREGROUND ) == null )
         PropertySystem.put ( PropertySystem.PROPERTY_FORM_VALUE_FOREGROUND, Color.blue ) ;

      if ( PropertySystem.get ( PropertySystem.PROPERTY_FORM_TITLE_FOREGROUND ) == null )
         PropertySystem.put ( PropertySystem.PROPERTY_FORM_TITLE_FOREGROUND, new Color ( 0, 0, 150 ) ) ;

      if ( PropertySystem.get ( PropertySystem.PROPERTY_FORM_TITLE_FONT ) == null )
         PropertySystem.put ( PropertySystem.PROPERTY_FORM_TITLE_FONT, new Font ( defaultFont.getFontName(),
                                                                                  Font.BOLD,
                                                                                  defaultFont.getSize()+8 ) ) ;

      if ( PropertySystem.get ( PropertySystem.PROPERTY_FORM_VALUE_FONT ) == null )
         PropertySystem.put ( PropertySystem.PROPERTY_FORM_VALUE_FONT, new Font ( defaultFont.getFontName(),
                                                                                  Font.BOLD,
                                                                                  defaultFont.getSize() ) ) ;

      if ( PropertySystem.get ( PropertySystem.PROPERTY_FORM_FONT ) == null )
         PropertySystem.put ( PropertySystem.PROPERTY_FORM_FONT, new Font ( defaultFont.getFontName(),
                                                                            Font.PLAIN,
                                                                            defaultFont.getSize() ) ) ;


   }


   /**
     *  This method will set the Font and Color properties on every control in the
     * FormComponent passed in
     **/
   public static void setPropertiesOnFormComponent ( Container component ) {
      Component   c         = null ;
      Component[] compArray = component.getComponents() ;
      int         i         = 0 ;

      while ( i < compArray.length ) {

         c = compArray[i] ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Value Labels                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( c instanceof ValueLabel ) {
            ValueLabel valueLabel = (ValueLabel) c ;

            valueLabel.setOpaque ( false ) ;

            ((JComponent)valueLabel.getComponent()).setOpaque ( false ) ;
            valueLabel.getJLabel().setOpaque ( false ) ;

            valueLabel.setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Form.font" ) ) ;
            valueLabel.setForeground ( (Color)javax.swing.UIManager.getDefaults().get ( "Form.foreground" ) ) ;
            valueLabel.getComponent().setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Form.valueFont" ) ) ;
            valueLabel.getComponent().setForeground ( (Color)javax.swing.UIManager.getDefaults().get ( "Form.valueForeground" ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Radio Buttons and Check Boxes                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( c instanceof JRadioButton ) ||
                   ( c instanceof JCheckBox ) ) {
            c.setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Form.font" ) ) ;
            c.setForeground ( (Color)javax.swing.UIManager.getDefaults().get ( "Form.foreground" ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   JTextArea and JTextField                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( c instanceof JTextArea ) || ( c instanceof JTextField ) ) {
            c.setFont ( (Font)javax.swing.UIManager.getDefaults().get ( "Form.valueFont" ) ) ;
            c.setForeground ( (Color)javax.swing.UIManager.getDefaults().get ( "Form.valueForeground" ) ) ;
         }



         else if ( ( c instanceof JPanel ) ||
                   ( c instanceof JScrollPane ) ||
                   ( c instanceof Panel ) ) {
            setPropertiesOnFormComponent ( (Container)c ) ;
         }

         i++ ;
      }

      if ( component instanceof FormComponent ) {
         ((FormComponent)component).setForeground ( (Color)UIManager.getDefaults().get ( "Form.titleForeground" ) ) ;
         ((FormComponent)component).setFont ( (Font)UIManager.getDefaults().get ( "Form.titleFont" ) ) ;
      }

   }


   /**
     *  This method should be the preferred method of creating a Browser.
     **/
   public static Browser createBrowser () {
      Browser browser = null ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, try to create Internet Explorer                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //try {
      //browser = new IEBrowser ( ) ;

      //} catch ( Exception e ) {
      //   e.printStackTrace() ;
      //   browser=null ;
      //}

      if ( browser == null )
         browser = new HTMLPaneBrowser() ;

      return( browser ) ;
   }



   /**
     *  This method will set the look and feel to a theme pack using the SkinLF code
     **/
   private static void setSkinLF () {
      //try {
      //   /**
      //    * The skin will be loaded from the following url
      //    */
      //   URL skinURL = new URL("file://themepack.zip" );

      //   /**
      //    * In this case, we are loading a themepack bundled in a zip file.
      //    * We use loadThemePack(url) from SkinLookAndFeel class
      //    */
      //   Skin skin = SkinLookAndFeel.loadThemePack("modernthemepack.zip");//skinURL);
      //   SkinLookAndFeel.setSkin(skin);

      //   /**
      //    * Using custom look and feel with Swing is currently buggy
      //    * because Swing is (most of the time) loaded from the system classpath
      //    * and call to Class.forName use the system class loader
      //    * see http://developer.java.sun.com/developer/bugParade/bugs/4155617.html
      //    * for more information on this bug and workaround
      //    */

      //   /**
      //    * First don't use setLookAndFeel(String) but setLookAndFeel(LookAndFeel)
      //    */
      //   SkinLookAndFeel lnf = new SkinLookAndFeel();
      //   UIManager.setLookAndFeel(lnf);

      //   /**
      //    * Then, specify your own ClassLoader to work around these bugs
      //    * Calls to UIManager.getUI() will use the lnf classloader
      //    * see javax.swing.UIDefaults#getUI method to understand how this works.
      //    */
      //   UIManager.getLookAndFeelDefaults().put("ClassLoader", lnf.getClass().getClassLoader());

      //   /**
      //    * Finally, build the user interface
      //    */
      //} catch ( Exception e ) {
      //   e.printStackTrace();
      //}

   }



   /**
     * this method will create an instance of JLabel which has the text 'Required' in it with
     * a small font
     **/
   public static JLabel getRequiredJLabel () {
      JLabel label = new JLabel ( "Required" ) ;

      label.setFont ( FontSystem.smallFont ) ;

      return( label ) ;
   }
} /* GUISystem */
