package com.ibm.nzna.projects.qit.app ;     
/*-----------------------------------------------------------*/
/*   ImageSystem                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ImageSystem                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import java.io.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.LogSystem ;
/**
 * The ImageSystem is used to cache Images for the Quest Application.
 * Each image is loaded when called for, and held in the ImageCache
 * for the duration of the Application life.
 *
 *
 * @version 1.0, 1/10/100 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   ImageSystem                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ImageSystem extends Object implements AppConst, PropertyListener {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Cached Image Indexes                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    public static final int INFO                                       = 0 ;
    public static final int ERROR                                      = 1 ;
    public static final int QUESTION                                   = 2 ;
    public static final int HELP                                       = 3 ;
    public static final int SMALLCLOSE                                 = 4 ;
    public static final int POPOUT                                     = 5 ;
    public static final int SMALL_SEARCH                               = 6 ;
    public static final int SEARCH                                     = 7 ;
    public static final int CLOSE                                      = 8 ;
    public static final int INDEX                                      = 9 ;
    public static final int SMALL_HELP                                 = 10 ;
    public static final int SHORT_CUT                                  = 11 ;
    public static final int SEE_ALSO                                   = 12 ;
    public static final int WORKSPACE_BACKGROUND                       = 13 ;
    public static final int WORKSPACE_INFOAREA                         = 14 ;
    public static final int BACK                                       = 15 ;
    public static final int FORWARD                                    = 16 ;
    public static final int REFRESH                                    = 17 ;
    public static final int SMALL_WEBBROWSER                           = 18 ;
    public static final int WEBBROWSER                                 = 19 ;
    public static final int LOGON                                      = 20 ;
    public static final int DATABASE_OFF                               = 21 ;
    public static final int DATABASE_ON                                = 22 ;
    public static final int SMALL_ADMIN                                = 23 ;
    public static final int ADMIN                                      = 24 ;
    public static final int PERSONEL                                   = 25 ;
    public static final int STATISTICS                                 = 26 ;
    public static final int CHECKBOOK                                  = 27 ;
    public static final int TEAMS                                      = 28 ;
    public static final int SMALL_CHECKBOOK                            = 29 ;
    public static final int SMALL_TEAMS                                = 30 ;
    public static final int SMALL_STATISTICS                           = 31 ;
    public static final int SMALL_PERSONEL                             = 32 ;
    public static final int SMALL_WORKSPACE                            = 34 ;
    public static final int SMALL_CLOSE_FILLER                         = 35 ;
    public static final int TITLE_BACKGROUND                           = 36 ;
    public static final int ADD_USER                                   = 37 ;
    public static final int DELETE_USER                                = 38 ;
    public static final int RESET_PASSWORD                             = 39 ;
    public static final int OPEN_TREE                                  = 40 ;
    public static final int CLOSE_TREE                                 = 41 ;
    public static final int DOCUMENT_LARGE                             = 42 ;
    public static final int DOCUMENT_SMALL                             = 43 ;
    public static final int PRODUCT_LARGE                              = 44 ;
    public static final int PRODUCT_SMALL                              = 45 ;
    public static final int OA_LARGE                                   = 46 ;
    public static final int OA_SMALL                                   = 47 ;
    public static final int PROMAIL_LARGE                              = 48 ;
    public static final int PROMAIL_SMALL                              = 49 ;
    public static final int NEWBLANK                                   = 50 ;
    public static final int FROMTEMPLATE                               = 51 ;
    public static final int FROMCLIPBOARD                              = 52 ;
    public static final int DOCIMPORT                                  = 53 ;
    public static final int UP                                         = 54 ;
    public static final int DOWN                                       = 55 ;
    public static final int EDIT                                       = 56 ;
    public static final int ACTION                                     = 57 ;
    public static final int DELETE                                     = 58 ;
    public static final int NEW                                        = 59 ;
    public static final int SELECT_USER                                = 60 ;
    public static final int SMALL_BACK_TITLE                           = 61 ;
    public static final int SAVE                                       = 62 ;
    public static final int STAMPEDGRAPHIC                             = 63 ;
    public static final int TABLE                                      = 64 ;
    public static final int TABLEPROP                                  = 65 ;
    public static final int CENTERALIGN                                = 66 ;
    public static final int LEFTALIGN                                  = 67 ;
    public static final int RIGHTALIGN                                 = 68 ;
    public static final int LINK                                       = 69 ;
    public static final int LINKTARGET                                 = 70 ;
    public static final int GRAPHIC                                    = 71 ;
    public static final int HORIZLINE                                  = 72 ;
    public static final int CUT                                        = 73 ;
    public static final int COPY                                       = 74 ;
    public static final int PASTE                                      = 75 ;
    public static final int UNDO                                       = 76 ;
    public static final int FIND                                       = 77 ;
    public static final int FINDAGAIN                                  = 78 ;
    public static final int DOCLINKBP                                  = 79 ;
    public static final int SPELL                                      = 80 ;
    public static final int BOLD                                       = 81 ;
    public static final int ITALIC                                     = 82 ;
    public static final int UNDERLINE                                  = 83 ;
    public static final int STRIKE                                     = 84 ;
    public static final int ORDERLIST                                  = 85 ;
    public static final int NUMERICLIST                                = 86 ;
    public static final int IMPORTHTML                                 = 87 ;
    public static final int INSERTROW                                  = 88 ;
    public static final int INSERTCOL                                  = 90 ;
    public static final int DELETECOL                                  = 91 ;
    public static final int DELETEROW                                  = 92 ;
    public static final int COMMENTS                                   = 93 ;
    public static final int DELCOMMENTS                                = 94 ;
    public static final int BLACK                                      = 95 ;
    public static final int RED                                        = 96 ;
    public static final int BLUE                                       = 97 ;
    public static final int PROPERTY                                   = 98 ;
    public static final int OPEN_MAIL                                  = 99 ;
    public static final int MAIL                                       = 100 ;
    public static final int CUSTOM_QUERY                               = 101 ;
    public static final int IBM_CONFIDENTIAL                           = 102 ;
    public static final int IBM_DEALER_CONFIDENTIAL                    = 103 ;
    public static final int IBM_UNCLASSIFIED                           = 104 ;
    public static final int WIZARD                                     = 105 ;
    public static final int ACTIVE_OBJECT                              = 106 ;
    public static final int DRAFT_OBJECT                               = 107 ;
    public static final int NOTIFY_LARGE                               = 108 ;
    public static final int NOTIFY_SMALL                               = 109 ;
    public static final int FORWARD_NOTIFY                             = 110 ;
    public static final int REPLY_NOTIFY                               = 111 ;
    public static final int OPEN_DRAFT_FOLDER                          = 112 ;  
    public static final int LARGE_KEYWORD                              = 113 ;
    public static final int SMALL_KEYWORD                              = 114 ;
    public static final int VALIDATE_BLANK                             = 115 ;
    public static final int VALIDATE_CHECK                             = 116 ;
    public static final int VALIDATE_FAIL                              = 117 ;
    public static final int VALIDATE_DELETE                            = 118 ;
    public static final int LARGE_RECYCLE_BIN                          = 119 ;
    public static final int SMALL_RECYCLE_BIN                          = 120 ;
    public static final int QUESTION_SMALL                             = 121 ;
    public static final int ACTION_SMALL                               = 122 ;
    public static final int DOC_LINK                                   = 123 ;
    public static final int DOCUMENT_HISTORY_LARGE                     = 124 ;
    public static final int DOCUMENT_HISTORY_SMALL                     = 125 ; 
    public static final int QUESTION_LARGE                             = 126 ;
    public static final int ACTION_LARGE                               = 127 ;
    public static final int SYMPTOM_LARGE                              = 128 ;
    public static final int LINK_LARGE                                 = 129 ;
    public static final int LINK_GROUP_LARGE                           = 130 ;
    public static final int FTP_EXPLORER_SMALL                         = 131 ;
    public static final int FTP_EXPLORER_LARGE                         = 132 ;
    public static final int FILE_DEFAULT                               = 133 ;
    public static final int FILE_EXEC                                  = 134 ;
    public static final int FILE_PDF                                   = 135 ;
    public static final int FILE_TEXT                                  = 136 ;
    public static final int FILE_VIDEO                                 = 137 ;
    public static final int FILE_WAVE                                  = 138 ;
    public static final int OPEN_FOLDER                                = 139 ;
    public static final int CLOSE_FOLDER                               = 140 ;
    public static final int UP_DIR                                     = 141 ;
    public static final int GRID_ON                                    = 142 ;
    public static final int GRID_OFF                                   = 143 ;
    public static final int NODE_ARROW_DOWN                            = 144 ;
    public static final int SYMPTOM_SMALL                              = 145 ;
    public static final int BUTTONPANEL_LEFT                           = 146 ;
    public static final int BUTTONPANEL_RIGHT                          = 147 ;
    public static final int PREVIEW                                    = 148 ;
    public static final int AVALON_WIZARD                              = 149 ;
    public static final int ZOOM_IN                                    = 150 ;
    public static final int ZOOM_OUT                                   = 151 ;
    public static final int CONDITION_WIZARD                           = 152 ;
    public static final int UNPACK                                     = 153 ;
    public static final int LOCKED                                     = 154 ;
    public static final int UNLOCKED                                   = 155 ;
    public static final int CUSTOM_VIEW_WIZARD                         = 156 ;
    public static final int BOOKMARK_WIZARD                            = 157 ;
    public static final int CONCLUSION_WIZARD                          = 158 ;
    public static final int STOP                                       = 159 ;
    public static final int LARGE_NEWOBJECT                            = 160 ;
    public static final int LARGE_SAVEDATABASE                         = 161 ;
    public static final int SMALL_SAVEDATABASE                         = 162 ;
    public static final int SMALL_LINKGROUP                            = 163 ;
    public static final int MARBLE1                                    = 164 ;
    public static final int EDGE_WIZARD                                = 165 ;
    public static final int PROPERTIES                                 = 166 ;
    public static final int TITLES_ON                                  = 167 ;
    public static final int CLIO                                       = 168 ;
    public static final int SMALL_CLIO                                 = 169 ;
    public static final int EXPORT_WIZARD                              = 170 ;
    public static final int EXPORT                                     = 171 ;
    public static final int PRINT                                      = 172 ;
    public static final int SMALL_MASCOT                               = 173 ;
    public static final int CLOSED_DRAFT_FOLDER                        = 174 ;  
    public static final int OPEN_ACTIVE_FOLDER                         = 175 ;  
    public static final int CLOSED_ACTIVE_FOLDER                       = 176 ;       
    public static final int SAVE_DOC                                   = 177 ;
    public static final int CONNECTED                                  = 178 ;
    public static final int CONNECTING                                 = 179 ;
    public static final int DISCONNECTED                               = 180 ;
    public static final int APPLET_UNKNOWN                             = 181 ;
    public static final int CHAT                                       = 182 ;
    public static final int SMALL_PROPERTY_APP                         = 183 ;
    public static final int LARGE_PROPERTY_APP                         = 184 ;
    public static final int NEW_FAMILY_WIZARD                          = 185 ;
    public static final int SMALL_WARNING                              = 186 ;
    public static final int SMALL_ERROR                                = 187 ;
    public static final int FULL_SCREEN                                = 188 ;
    public static final int LARGE_MAIL                                 = 189 ;

    public static final int THEME_TASKBAR                                  = 33 ;
    public static final int THEME_NORTH_TOOLBAR                            = 190 ;
    public static final int THEME_CENTER_TOOLBAR                           = 191 ;
    public static final int THEME_SOUTH_TOOLBAR                            = 192 ;
    public static final int THEME_EAST_TOOLBAR                             = 193 ;
    public static final int THEME_WEST_TOOLBAR                             = 194 ;

    public static final int THEME_NORTH_NAVPANELTITLE                      = 195 ;
    public static final int THEME_CENTER_NAVPANELTITLE                     = 196 ;
    public static final int THEME_SOUTH_NAVPANELTITLE                      = 197 ;
    public static final int THEME_EAST_NAVPANELTITLE                       = 198 ;
    public static final int THEME_WEST_NAVPANELTITLE                       = 199 ;

    public static final int THEME_NORTH_NAVPANEL                           = 200 ;
    public static final int THEME_CENTER_NAVPANEL                          = 201 ;
    public static final int THEME_SOUTH_NAVPANEL                           = 202 ;
    public static final int THEME_EAST_NAVPANEL                            = 203 ;
    public static final int THEME_WEST_NAVPANEL                            = 204 ;

    public static final int THEME_NORTH_CONTENT                            = 205 ;
    public static final int THEME_CENTER_CONTENT                           = 206 ;
    public static final int THEME_SOUTH_CONTENT                            = 207 ;
    public static final int THEME_EAST_CONTENT                             = 208 ;
    public static final int THEME_WEST_CONTENT                             = 209 ;

    public static final int THEME_NORTH_STATUS                             = 210 ;
    public static final int THEME_CENTER_STATUS                            = 211 ;
    public static final int THEME_SOUTH_STATUS                             = 212 ;
    public static final int THEME_EAST_STATUS                              = 213 ;
    public static final int THEME_WEST_STATUS                              = 214 ;

    public static final int THEME_NORTH_ACTIONBAR                          = 215 ;
    public static final int THEME_CENTER_ACTIONBAR                         = 216 ;
    public static final int THEME_SOUTH_ACTIONBAR                          = 217 ;
    public static final int THEME_EAST_ACTIONBAR                           = 218 ;
    public static final int THEME_WEST_ACTIONBAR                           = 219 ;


    public static final int COUNTRY_VIEW                               = 220 ;
    public static final int BLANK_TREE                                 = 221 ;
    public static final int WHATSNEW_RIGHT_ARROW                       = 222 ;
    public static final int WHATSNEW_LEFT_ARROW                        = 223 ;
    public static final int WHATSNEW_MASCOT                            = 224 ;
    public static final int LARGE_CHAT                                 = 225 ;
    public static final int DEBUG_WIZARD                               = 226 ;
    public static final int LOCK_LARGE                                 = 227 ;
    public static final int LOCK_SMALL                                 = 228 ;
    public static final int SAVEDOC_BACKGROUND                         = 229 ;
    public static final int USERSTAT_BACKGROUND                        = 230 ;
    public static final int NEW_DOCUMENT                               = 231 ;
    public static final int ONE                                        = 232 ;
    public static final int TWO                                        = 233 ;
    public static final int THREE                                      = 234 ;
    public static final int LOCALE_DLG                                 = 235 ;
    public static final int BRAND_MANAGEMENT                           = 236 ;
    public static final int BRAND_MANAGEMENT_SMALL                     = 237 ;
    public static final int RIGHT_ARROW                                = 238 ;
    public static final int PRODUCT_COMPARE                            = 239 ;
    public static final int PRODUCT_HISTORY_SMALL                      = 240 ;
    public static final int SMALL_PRODUCT_LOCK                         = 241 ;
    public static final int NEW_PRODUCT_WIZARD                         = 242 ;
    public static final int PRODUCT_COMPARE_LARGE                      = 243 ;
    public static final int PRODUCT_HISTORY_LARGE                      = 244 ;
    public static final int USERSTAT_TOP                               = 245 ;
    public static final int CLOCK_FACE                                 = 246 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Hashtable imageCache = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the Image System
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageSystem () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our cache Array                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      imageCache = new Hashtable() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Make sure we know when the skins change!                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;

   }





   /*------------------------------------------------------------*/
   /* getImage
    */
   /**
    *    This method will return a standard
    *  java.awt.Image for the Image name
    *  passed in. This will not cache the image in memory for future retrieval
    *
    *  @param Object object The Object to read with (If in Jar file, the Object must reside in the Jar file)
    *  @param imageName The name of the image
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Image getImage ( Object object, String imageName ) {
      Image retImage = null ;

      try {
         String filename = "skins"+File.separator+
                           PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                           File.separator+
                           imageName ;

         retImage = new ImageIcon ( filename ).getImage() ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }


      return( retImage ) ;
   }






   /*------------------------------------------------------------*/
   /* getImage
    */
   /**
    *    This method will attempt to get an image
    *  from the image cache. If this image has
    *  not yet been loaded, it will be loaded now
    *
    *  @param imageIndex The index of the image to get
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Image getImage ( Object object, int imageIndex ) {
      ImageIcon icon = getImageIcon ( object, imageIndex ) ;

      if ( icon != null )
         return( icon.getImage() ) ;

      return( null ) ;
   }



   /*------------------------------------------------------------*/
   /* getImageIcon
    */
   /**
    *    This method will return an ImageIcon
    *  from the index in cache. If the image has
    *  not been loaded, it will load the image
    *
    *  @param Object object The Object to read with (If in Jar file, the Object must reside in the Jar file)
    *  @param imageIndex The index of the image to return
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static ImageIcon getImageIcon ( Object object, int imageIndex ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this image has been cached, return the cached image  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( imageCache.contains ( ""+imageIndex ) )
         return( (ImageIcon)imageCache.get ( ""+imageIndex ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, read the image from a JAR of file system     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      imageCache.put ( ""+imageIndex, getImageIcon ( object, imageNameFromIndex ( imageIndex ) ) ) ;

      return( (ImageIcon)imageCache.get ( ""+imageIndex ) ) ;

   }




   /*------------------------------------------------------------*/
   /* getImageIcon
    */
   /**
    *    This method will return an ImageIcon
    *  from the filename passed in.
    *  
    *
    *  @param Object object The Object to read with (If in Jar file, the Object must reside in the Jar file)
    *  @param filename The filename of the image. 
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static ImageIcon getImageIcon ( Object object, String imageName ) {
      if ( imageName != null )
         return( new ImageIcon ( imageName ) ) ;
      return( null ) ;
   }






   /*------------------------------------------------------------*/
   /* imageNameFromIndex
    */
   /**
    *    This method maps a filename to
    *  the Image Index passed in.
    *  
    *
    *  @param imageIndex The index of the cached image
    *  @param None
    *  @param None
    *
    *  @return The filename of the image index
    *//*---------------------------------------------------------*/
   private static String imageNameFromIndex ( int imageIndex ) {
      String filename = null ;
      filename = PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+File.separator ;

      switch ( imageIndex ) {
         case ERROR :
            filename+="error.gif" ;
            break ;

         case COUNTRY_VIEW :
            filename+="view_country.gif" ;
            break ;

         case INFO :
            filename+="info.gif" ;
            break ;

         case SMALLCLOSE :
            filename+="small_close.gif" ;
            break ;

         case POPOUT :
            filename+="popout.gif" ;
            break ;

         case QUESTION :
            filename+="question.gif" ;
            break ;

         case SMALL_SEARCH :
            filename+="small_search.gif" ;
            break ;

         case SEARCH :
            filename+="search.gif" ;
            break ;

         case REFRESH :
            filename+="refresh.gif" ;
            break ;

         case PRODUCT_COMPARE_LARGE :
            filename+="product_compare_large.gif" ;
            break ;

         case BRAND_MANAGEMENT :
            filename+="brandmgmt.gif" ;
            break ;

         case BRAND_MANAGEMENT_SMALL :
            filename+="brandmgmt_small.gif" ;
            break ;

         case FORWARD :
            filename+="forward.gif" ;
            break ;

         case BACK :
            filename+="back.gif" ;
            break ;

         case HELP :
            filename+="help.gif" ;
            break ;

         case CLOSE :
            filename+="close.gif" ;
            break ;

         case APPLET_UNKNOWN :
            filename+="close.gif" ;
            break ;

         case ONE :
            filename+="one.gif" ;
            break ;

         case PRODUCT_COMPARE :
            filename+="product_compare.gif" ;
            break ;

         case PRODUCT_HISTORY_SMALL :
            filename+="product_history_small.gif" ;
            break ;

         case PRODUCT_HISTORY_LARGE :
            filename+="product_history_large.gif" ;
            break ;

         case NEW_PRODUCT_WIZARD :
            filename+="new_product.gif" ;
            break ;

         case RIGHT_ARROW :
            filename+="right_arrow.gif" ;
            break ;

         case SMALL_PRODUCT_LOCK :
            filename+="small_product_lock.gif" ;
            break ;

         case THREE :
            filename+="three.gif" ;
            break ;

         case LOCALE_DLG :
            filename+="country_select.gif" ;
            break ;

         case TWO :
            filename+="two.gif" ;
            break ;

         case INDEX :
            filename+="index.gif" ;
            break ;

         case WHATSNEW_RIGHT_ARROW :
            filename+="whatsnew_right_arrow.gif" ;
            break ;

         case WHATSNEW_LEFT_ARROW :
            filename+="whatsnew_left_arrow.gif" ;
            break ;

         case WHATSNEW_MASCOT :
            filename+="whatsnew.gif" ;
            break ;

         case LARGE_KEYWORD :
            filename+="large_keyword.gif" ;
            break ;

         case VALIDATE_BLANK :
            filename+="validate_blank.gif" ;
            break ;

         case VALIDATE_CHECK :
            filename+="validate_check.gif" ;
            break ;

         case VALIDATE_FAIL :
            filename+="validate_fail.gif" ;
            break ;

         case VALIDATE_DELETE :
            filename+="validate_delete.gif" ;
            break ;

         case LARGE_RECYCLE_BIN :
            filename+="recyclebin.gif" ;
            break ;

         case SMALL_RECYCLE_BIN :
            filename+="small_recyclebin.gif" ;
            break ;

         case DOCUMENT_HISTORY_SMALL :
            filename+="document_history_small.gif" ;
            break ;

         case FTP_EXPLORER_SMALL :
            filename+="ftp_explorer_small.gif" ;
            break ;

         case FTP_EXPLORER_LARGE :
            filename+="ftp_explorer_large.gif" ;
            break ;

         case DOCUMENT_HISTORY_LARGE :
            filename+="document_history_large.gif" ;
            break ;

         case SMALL_KEYWORD :
            filename+="small_keyword.gif" ;
            break ;

         case SMALL_HELP :
            filename+="small_help.gif" ;
            break ;

         case NEW_FAMILY_WIZARD :
            filename+="newfamilywiz.gif" ;
            break ;

         case SHORT_CUT :
            filename+="short_cut.gif" ;
            break ;

         case SEE_ALSO :
            filename+="see_also.gif" ;
            break ;

         case WORKSPACE_INFOAREA :
            filename+="workspace_infoarea.gif" ;
            break ;

         case WORKSPACE_BACKGROUND :
            filename+="workspace_background.gif" ;
            break ;

         case WEBBROWSER :
            filename+="webbrowser.gif" ;
            break ;

         case SMALL_WEBBROWSER :
            filename+="small_webbrowser.gif" ;
            break ;

         case LOGON :
            filename+="logon.gif" ;
            break ;

         case DATABASE_ON :
            filename+="database_on.gif" ;
            break ;

         case DATABASE_OFF :
            filename+="database_off.gif" ;
            break ;

         case SMALL_WORKSPACE :
            filename+="small_workspace.gif" ;
            break ;

         case ADMIN :
            filename+="admin.gif" ;
            break ;

         case SMALL_CLOSE_FILLER :
            filename+="small_close_filler.gif" ;
            break ;

         case SMALL_ADMIN :
            filename+="small_admin.gif" ;
            break ;

         case PERSONEL :
            filename+="pers.gif" ;
            break ;

         case CONNECTING :
            filename+="connecting.gif" ;
            break ;

         case DISCONNECTED :
            filename+="disconnected.gif" ;
            break ;

         case CONNECTED :
            filename+="connected.gif" ;
            break ;

         case THEME_TASKBAR :
            filename+="THEME_taskbar.gif" ;
            break ;

         case SMALL_PERSONEL :
            filename+="small_pers.gif" ;
            break ;

         case CHECKBOOK :
            filename+="checkbook.gif" ;
            break ;

         case SMALL_CHECKBOOK :
            filename+="small_checkbook.gif" ;
            break ;

         case SMALL_LINKGROUP :
            filename+="small_linkgroup.gif" ;
            break ;

         case MARBLE1 :
            filename+="marble1.gif" ;
            break ;

         case TITLES_ON :
            filename+="titles_on.gif" ;
            break ;

         case EXPORT_WIZARD :
            filename+="export_wizard.gif" ;
            break ;

         case EXPORT :
            filename+="small_export.gif" ;
            break ;

         case SMALL_MASCOT :
            filename+="small_mascot.gif" ;
            break ;

         case EDGE_WIZARD :
            filename+="edge_wizard.gif" ;
            break ;

         case CUSTOM_VIEW_WIZARD :
            filename+="custom_query_wizard.gif" ;
            break ;

         case CONCLUSION_WIZARD :
            filename+="conclusion_wizard.gif" ;
            break ;

         case STATISTICS :
            filename+="stats.gif" ;
            break ;

         case SMALL_STATISTICS :
            filename+="small_stats.gif" ;
            break ;


         case SMALL_TEAMS :
            filename+="small_teams.gif" ;
            break ;

         case TEAMS :
            filename+="teams.gif" ;
            break ;

         case TITLE_BACKGROUND :
            filename+="empty_title.gif" ;
            break ;

         case ADD_USER :
            filename+="add_user.gif" ;
            break ;

         case DELETE_USER :
            filename+="delete_user.gif" ;
            break ;

         case RESET_PASSWORD :
            filename+="reset_password.gif" ;
            break ;

         case OPEN_TREE :
            filename+="open_tree.gif" ;
            break ;

         case CLOSE_TREE :
            filename+="closed_tree.gif" ;
            break ;

         case DOCUMENT_LARGE :
            filename+="doc_large.gif" ;
            break ;

         case DOCUMENT_SMALL :
            filename+="doc_small.gif" ;
            break ;

         case OA_LARGE :
            filename+="oa_large.gif" ;
            break ;

         case DRAFT_OBJECT :
            filename+="draft_object.gif" ;
            break ;

         case ACTIVE_OBJECT :
            filename+="active_object.gif" ;
            break ;

         case OA_SMALL :
            filename+="oa_small.gif" ;
            break ;

         case PROMAIL_LARGE :
            filename+="promail_large.gif" ;
            break ;

         case PROMAIL_SMALL :
            filename+="promail_small.gif" ;
            break ;

         case REPLY_NOTIFY :
            filename+="reply_notify.gif" ;
            break ;


         case FORWARD_NOTIFY :
            filename+="forward_notify.gif" ;
            break ;

         case PRODUCT_LARGE :
            filename+="product_large.gif" ;
            break ;

         case PRODUCT_SMALL :
            filename+="product_small.gif" ;
            break ;

         case NEWBLANK :
            filename+="newblank.gif" ;
            break ;

         case UP :
            filename+="up.gif" ;
            break ;

         case DOWN :
            filename+="down.gif" ;
            break ;

         case EDIT :
            filename+="edit.gif" ;
            break ;

         case ACTION :
            filename+="action.gif" ;
            break ;

         case DELETE :
            filename+="delete.gif" ;
            break ;

         case NEW :
            filename+="new.gif" ;
            break ;

         case FROMTEMPLATE :
            filename+="fromtemplate.gif" ;
            break ;

         case AVALON_WIZARD :
            filename+="avalon_wizard.gif" ;
            break ;

         case ZOOM_OUT :
            filename+="zoom_out.gif" ;
            break ;

         case LOCKED :
            filename+="locked.gif" ;
            break ;

         case UNLOCKED :
            filename+="unlocked.gif" ;
            break ;

         case ZOOM_IN :
            filename+="zoom_in.gif" ;
            break ;

         case FROMCLIPBOARD :
            filename+="fromclipboard.gif" ;
            break ;

         case DOCIMPORT :
            filename+="docimport.gif" ;
            break ;

         case SELECT_USER :
            filename+="selectuser.gif";
            break ;

         case SMALL_BACK_TITLE :
            filename+="smallbacktitle.gif";
            break ;

         case SAVE :
            filename+="save.gif";
            break ;

         case STAMPEDGRAPHIC :
            filename+="stamped_graphic.gif" ;
            break ;

         case TABLE :
            filename+="table.gif" ;
            break ;

         case TABLEPROP :
            filename+="table_prop.gif" ;
            break ;

         case CENTERALIGN :
            filename+="align_center.gif" ;
            break ;

         case LEFTALIGN :
            filename+="align_left.gif" ;
            break ;

         case RIGHTALIGN :
            filename+="align_right.gif" ;
            break ;

         case LINK :
            filename+="align_link.gif" ;
            break ;

         case LINKTARGET :
            filename+="link_target.gif" ;
            break ;

         case GRAPHIC :
            filename+="graphic.gif" ;
            break ;

         case HORIZLINE :
            filename+="horiz_line.gif" ;
            break ;

         case CUT :
            filename+="cut.gif" ;
            break ;


         case COPY :
            filename+="copy.gif" ;
            break ;

         case QUESTION_SMALL :
            filename+="small_question.gif" ;
            break ;

         case ACTION_SMALL :
            filename+="small_action.gif" ;
            break ;

         case DOC_LINK :
            filename+="small_doclink.gif" ;
            break ;

         case PASTE :
            filename+="paste.gif" ;
            break ;

         case UNDO :
            filename+="undo.gif" ;
            break ;

         case LARGE_MAIL :
            filename+="large_mail.gif" ;
            break ;

         case SMALL_WARNING :
            filename+="small_warning.gif" ;
            break ;


         case SMALL_ERROR :
            filename+="small_error.gif" ;
            break ;

         case FULL_SCREEN :
            filename+="fullscreen.gif" ;
            break ;


         case SAVE_DOC :
            filename+="save_doc.gif" ;
            break ;

         case FINDAGAIN :
            filename+="find_again.gif" ;
            break ;

         case FIND :
            filename+="find.gif" ;
            break ;

         case DOCLINKBP :
            filename+="linkdoc.gif" ;
            break ;

         case SPELL :
            filename+="spell.gif" ;
            break ;

         case BOLD :
            filename+="bold.gif" ;
            break ;

         case ITALIC :
            filename+="italic.gif" ;
            break ;

         case UNDERLINE :
            filename+="underline.gif" ;
            break ;

         case STRIKE :
            filename+="strike.gif" ;
            break ;

         case ORDERLIST :
            filename+="bullet_list.gif" ;
            break ;

         case NUMERICLIST :
            filename+="numeric_list.gif" ;
            break ;

         case IMPORTHTML :
            filename+="import_html.gif" ;
            break ;

         case INSERTROW :
            filename+="insert_row.gif" ;
            break ;

         case INSERTCOL :
            filename+="insert_col.gif" ;
            break ;


         case DELETECOL :
            filename+="delete_col.gif" ;
            break ;

         case DELETEROW :
            filename+="delete_row.gif" ;
            break ;

         case COMMENTS :
            filename+="comments.gif" ;
            break ;


         case DELCOMMENTS :
            filename+="del_comments.gif" ;

         case BLACK :
            filename+="black.gif" ;
            break ;

         case RED :
            filename+="red.gif" ;
            break ;


         case NOTIFY_SMALL :
            filename+="notify_small.gif" ;
            break ;


         case NOTIFY_LARGE :
            filename+="notify_large.gif" ;
            break ;


         case BLUE :
            filename+="blue.gif" ;
            break ;


         case PROPERTY :
            filename+="property.gif" ;
            break ;


         case OPEN_MAIL :
            filename+="open_mail.gif" ;
            break ;

         case CUSTOM_QUERY :
            filename+="custom_query.gif" ;
            break ;

         case MAIL :
            filename+="mail.gif" ;
            break ;

         case IBM_CONFIDENTIAL :
            filename+="ibm_confidential.gif" ;
            break ;

         case IBM_DEALER_CONFIDENTIAL :
            filename+="ibm_dealer_confidential.gif" ;
            break ;

         case PRINT :
            filename+="print.gif" ;
            break ;

         case CLIO :
            filename+="clio.gif" ;
            break ;


         case SMALL_CLIO :
            filename+="small_clio.gif" ;
            break ;

         case WIZARD :
            filename+="wizard.gif" ;
            break ;

         case IBM_UNCLASSIFIED :
            filename+="unclassified.gif" ;
            break ;

         case FILE_DEFAULT :
            filename+="file_default.gif" ;
            break ;

         case FILE_EXEC :
            filename+="file_exec.gif" ;
            break ;

         case FILE_PDF :
            filename+="file_pdf.gif" ;
            break ;

         case FILE_TEXT :
            filename+="file_text.gif" ;
            break ;

         case ACTION_LARGE :
            filename+="action_large.gif" ;
            break ;

         case QUESTION_LARGE :
            filename+="large_question.gif" ;
            break ;

         case PROPERTIES :
            filename+="configure.gif" ;
            break ;

         case SYMPTOM_LARGE :
            filename+="symptom_large.gif" ;
            break ;

         case LINK_LARGE :
            filename+="doclink_large.gif" ;
            break ;

         case LINK_GROUP_LARGE :
            filename+="doclinkgroup_large.gif" ;
            break ;

         case FILE_VIDEO :
            filename+="file_video.gif" ;
            break ;

         case FILE_WAVE :
            filename+="file_wave.gif" ;
            break ;

         case OPEN_FOLDER :
            filename+="open_folder.gif" ;
            break ;

         case CONDITION_WIZARD :
            filename+="condition_wizard.gif" ;
            break ;

         case UNPACK :
            filename+="unpack.gif" ;
            break ;

         case CLOSE_FOLDER :
            filename+="closed_folder.gif" ;
            break ;

         case PREVIEW :
            filename+="preview.gif" ;
            break ;

         case UP_DIR :
            filename+="up_dir.gif" ;
            break ;

         case GRID_OFF :
            filename+="grid_off.gif" ;
            break ;

         case GRID_ON :
            filename+="grid_on.gif" ;
            break ;

         case NODE_ARROW_DOWN :
            filename+="node_arrow_down.gif" ;
            break ;

         case CHAT :
            filename+="chat.gif" ;
            break ;

         case LARGE_CHAT :
            filename+="large_chat.gif" ;
            break ;

         case DEBUG_WIZARD :
            filename+="debug_wizard.gif" ;
            break ;

         case LOCK_LARGE :
            filename+="lock_large.gif" ;
            break ;

         case LOCK_SMALL :
            filename+="lock_small.gif" ;
            break ;

         case SAVEDOC_BACKGROUND :
            filename+="savedoc_background.gif" ;
            break ;


         case SYMPTOM_SMALL :
            filename+="symptom_small.gif" ;
            break ;

         case BUTTONPANEL_RIGHT :
            filename+="buttonpanel_right.gif" ;
            break ;

         case BUTTONPANEL_LEFT :
            filename+="buttonpanel_left.gif" ;
            break ;

         case BOOKMARK_WIZARD :
            filename+="bookmark_wizard.gif" ;
            break ;

         case STOP :
            filename+="stop.gif" ;
            break ;

         case USERSTAT_BACKGROUND :
            filename+="userstat_background.gif" ;
            break ;

         case CLOCK_FACE :
            filename+="clockface.gif" ;
            break ;
         
         case USERSTAT_TOP :
            filename+="top_stats.gif" ;
            break ;


         case NEW_DOCUMENT :
            filename+="newdocument.gif" ;
            break ;

         case LARGE_NEWOBJECT :
            filename+="large_newobject.gif" ;
            break ;

         case LARGE_PROPERTY_APP :
            filename+="property_app_large.gif" ;
            break ;

         case SMALL_PROPERTY_APP :
            filename+="property_app_small.gif" ;
            break ;

         case LARGE_SAVEDATABASE :
            filename+="large_databasesave.gif" ;
            break ;

         case SMALL_SAVEDATABASE :
            filename+="small_databasesave.gif" ;
            break ;

         case CLOSED_DRAFT_FOLDER :
            filename+="closed_draft_folder.gif" ;
            break ;

         case CLOSED_ACTIVE_FOLDER :
            filename+="closed_active_folder.gif" ;
            break ;

         case OPEN_DRAFT_FOLDER :
            filename+="open_draft_folder.gif" ;
            break ;

         case OPEN_ACTIVE_FOLDER :
            filename+="open_active_folder.gif" ;
            break ;

         case BLANK_TREE :
            filename+="blank_tree.gif" ;
            break ;

         case THEME_NORTH_ACTIONBAR :
            filename+="THEME_north_actionbar.gif" ;
            break ;

         case THEME_SOUTH_ACTIONBAR :
            filename+="THEME_south_actionbar.gif" ;
            break ;

         case THEME_CENTER_ACTIONBAR :
            filename+="THEME_center_actionbar.gif" ;
            break ;

         case THEME_WEST_ACTIONBAR :
            filename+="THEME_west_actionbar.gif" ;
            break ;

         case THEME_EAST_ACTIONBAR :
            filename+="THEME_east_actionbar.gif" ;
            break ;

         case THEME_NORTH_TOOLBAR :
            filename+="THEME_north_toolbar.gif" ;
            break ;

         case THEME_SOUTH_TOOLBAR :
            filename+="THEME_south_toolbar.gif" ;
            break ;

         case THEME_CENTER_TOOLBAR :
            filename+="THEME_center_toolbar.gif" ;
            break ;

         case THEME_WEST_TOOLBAR :
            filename+="THEME_west_toolbar.gif" ;
            break ;

         case THEME_EAST_TOOLBAR :
            filename+="THEME_east_toolbar.gif" ;
            break ;

         case THEME_NORTH_NAVPANELTITLE :
            filename+="THEME_north_navpaneltitle.gif" ;
            break ;

         case THEME_SOUTH_NAVPANELTITLE :
            filename+="THEME_south_navpaneltitle.gif" ;
            break ;

         case THEME_CENTER_NAVPANELTITLE :
            filename+="THEME_center_navpaneltitle.gif" ;
            break ;

         case THEME_WEST_NAVPANELTITLE :
            filename+="THEME_west_navpaneltitle.gif" ;
            break ;

         case THEME_EAST_NAVPANELTITLE :
            filename+="THEME_east_navpaneltitle.gif" ;
            break ;

         case THEME_NORTH_NAVPANEL :
            filename+="THEME_north_navpanel.gif" ;
            break ;

         case THEME_SOUTH_NAVPANEL :
            filename+="THEME_south_navpanel.gif" ;
            break ;

         case THEME_CENTER_NAVPANEL :
            filename+="THEME_center_navpanel.gif" ;
            break ;

         case THEME_WEST_NAVPANEL :
            filename+="THEME_west_navpanel.gif" ;
            break ;

         case THEME_EAST_NAVPANEL :
            filename+="THEME_east_navpanel.gif" ;
            break ;

         case THEME_NORTH_CONTENT :
            filename+="THEME_north_content.gif" ;
            break ;

         case THEME_SOUTH_CONTENT :
            filename+="THEME_south_content.gif" ;
            break ;

         case THEME_CENTER_CONTENT :
            filename+="THEME_center_content.gif" ;
            break ;

         case THEME_WEST_CONTENT :
            filename+="THEME_west_content.gif" ;
            break ;

         case THEME_EAST_CONTENT :
            filename+="THEME_east_content.gif" ;
            break ;

         case THEME_NORTH_STATUS :
            filename+="THEME_north_status.gif" ;
            break ;

         case THEME_SOUTH_STATUS :
            filename+="THEME_south_status.gif" ;
            break ;

         case THEME_CENTER_STATUS :
            filename+="THEME_center_status.gif" ;
            break ;

         case THEME_WEST_STATUS :
            filename+="THEME_west_status.gif" ;
            break ;

         case THEME_EAST_STATUS :
            filename+="THEME_east_status.gif" ;
            break ;



         default:
            filename = null ;
      }

      if ( filename != null )
         filename="skins"+File.separator+filename ;


      return( filename ) ;
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has been changed.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object newValue ) {

      switch ( property ) {
         case PropertySystem.PROPERTY_SKINNAME :
         case PropertySystem.PROPERTY_SHOWSKIN :
            releaseImages() ;
            break ;
      }
   }





   /*------------------------------------------------------------*/
   /* releaseImages
    */
   /**
    *    This method will release all images that are
    *  currently cached. This means that the next
    *  time they are called for, they are reloaded from disk
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void releaseImages () {
       imageCache = new Hashtable() ;
   }




   /*------------------------------------------------------------*/
   /* getIndexFromImage
    */
   /**
    *    This method will return the index (usually specified
    *  in the AppConst interface) of the ImageIcon
    *  passed in
    *
    *  @param imageIcon The ImageIcon to get the index of
    *  @param None
    *  @param None
    *
    *  @return int 0 if the image was not found
    *//*---------------------------------------------------------*/
   //public static int getIndexFromImage ( ImageIcon imageIcon ) {
   //   int retIndex = 0 ;

   //   if ( imageCache != null ) {
   //      int i    = 0 ;

   //      while ( ( retIndex == 0 ) && ( i < imageCache.length ) ) {
   //         if ( imageCache[i] == imageIcon )
   //            retIndex = i ;
   //         i++ ;
   //      }
   //   }

   //   return( retIndex ) ;
   //}

} /* ImageSystem */
