package com.ibm.nzna.projects.qit.app ;            
/*-----------------------------------------------------------*/
/*   InitializeThread                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: InitializeThread                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.* ;     
import com.ibm.nzna.projects.qit.app.util.* ;     
import com.ibm.nzna.projects.qit.gui.* ;     
import com.ibm.nzna.projects.qit.type.* ;     
import com.ibm.nzna.projects.qit.workspace.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.setup.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import java.awt.* ;
import javax.swing.* ;
import java.util.* ;
import java.io.* ;
/**
 * This thread will initialize all variables of the Quest application. 
 * This initialize includes the following steps:<br><pre>
 *
 * 1. Connect to the Quest database
 * 2. Read in the constants
 * 3. Kick off the Updater if needed
 * 4. Log the user in, reading all user ids and user information
 * 5. Read in the start up type lists
 * 6. Read in all users
 * 7. Read in all Brand/Family Combinations
 * 8. Bring up the workspace of the Main Window
 *
 *
 * If for any reason the Initialization fails, a logon window will
 * be put back up, starting the process over again.
 *
 * @version 1.0, 1/20/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   InitializeThread                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class InitializeThread extends Object implements DatabaseStatusListener, Runnable, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private InitializeWindow initWindow      = null ;
   private Qit              qit             = null ;
   private boolean          connectingShown = false ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and kick off
    *  the Initialization thread if the param is true
    *  
    *
    *  @param runThread TRUE = Run the thread, FALSE = do not run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InitializeThread ( Qit qit, boolean runThread ) {
      this.qit = qit ;

      if ( runThread ) {
         new Thread ( this ).start() ;
      }
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run through the initialization
    *  steps
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      boolean continueInitialize = false ;
      boolean productionMode     = !PropertySystem.getBool ( PropertySystem.PROPERTY_TESTMODE ) ;
      String  dbUserId           = "quest" ;
      String  dbPassword         = productionMode ? "k1ngarthur" : "j0hnny" ;

      DatabaseSystem.addStatusListener ( this ) ;


      if ( false ) {
         Qit.setEnabled ( true ) ;
         WindowSystem.createMainWindow() ;
         MainWindow.createWorkspace() ;
      } else {
         Frame tempFrame = new Frame() ;                                

         try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, log in the user                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( new UserLogonDlg ( tempFrame ).getResponse() )
               continueInitialize = true ;
            else {
               continueInitialize = false ;
               new ShutdownThread(false).run() ;
            }

            tempFrame.dispose() ;

            if ( continueInitialize ) {
               Qit.setEnabled ( false ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Begin                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               LogSystem.beginLogSection ( "Initialization" ) ;                 

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Bring up the Initialize Window if need be               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( PropertySystem.getBool ( PropertySystem.PROPERTY_LOGO ) )
                  initWindow = createInitializeWindow() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Clear The Temp Directory                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( PropertySystem.getBool ( PropertySystem.PROPERTY_CLEAR_TEMP_DIR_ON_START ) )
                  QuestUtil.clearTempDir() ;

               /*-------------------------------------------*/
               /*                                           */
               /*  Copy over the images/misc. files to temp */
               /*                                           */
               /*-------------------------------------------*/
               QuestUtil.populateTempDir() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Load in external classes                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               continueInitialize = loadExternalClasses() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Connect to the Quest Database                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               LogSystem.log ( 1, "Connecting to database: " ) ;
               continueInitialize = DatabaseSystem.createConnection ( DatabaseSystem.QUEST,
                                                                      PropertySystem.getString ( PropertySystem.PROPERTY_APPDB ),
                                                                      PropertySystem.getString ( PropertySystem.PROPERTY_APPDB_SERVER ),
                                                                      PropertySystem.getString ( PropertySystem.PROPERTY_APPDB_PORT ),
                                                                      dbUserId, 
                                                                      dbPassword ) ;                              

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Bring up the database window if need be                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( PropertySystem.getBool ( PropertySystem.PROPERTY_START_DBWINDOW ) )
                  new DatabaseDlg() ;

               if ( !continueInitialize )
                  GUISystem.printBox ( STR_ERROR, STR_CANNOT_CONNECT_TO_DATABASE ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in the constants                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Reading Constants" ) ;
                  if ( initWindow != null )
                     initWindow.setText ( STR_CHECKING_NEW_VERSION ) ;
                  continueInitialize = Constants.refreshConstants ( LogSystem.getInstance() ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Check the application version                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( continueInitialize ) && 
                    ( APPLICATION_VERSION != Constants.getIntConst ( "IT_VERSION" ) ) ) {
                  LogSystem.log ( 1, "Checked Application Version. Software Version:"+APPLICATION_VERSION+"  Database Version:"+Constants.getIntConst ( "IT_VER" ) ) ;
                  continueInitialize = false ;
                  if ( initWindow != null )
                     initWindow.dispose() ;
                  updateApplication() ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create the Type List                                    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Creating Type Lists" ) ;
                  if ( initWindow != null )
                     initWindow.setText ( Str.getStr ( STR_SYNCHRONIZING_LOCAL ) ) ;
                  new QITTypeList() ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in the User Information                            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Reading user information" ) ;
                  if ( initWindow != null )
                     initWindow.setText ( Str.getStr ( STR_LOGGING_IN_USER ) ) ;
                  continueInitialize = UserSystem.logonUser ( PropertySystem.getString ( PropertySystem.PROPERTY_USERID ),
                                                              PropertySystem.getString ( PropertySystem.PROPERTY_PASSWORD  ) ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Clear the user's password, since this is the first      */
               /*  thing to do when the user logs in                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               PropertySystem.putString ( PropertySystem.PROPERTY_PASSWORD, "" ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read the startup lists                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Reading Start up lists" ) ;
                  if ( initWindow != null )
                     initWindow.setText ( STR_READING_STARTUP_TYPE_LISTS ) ;
                  TypeList.getInstance().refreshList ( new JList(), TypeList.TYPE_STATUS, null ) ;
                  TypeList.getInstance().refreshList ( new JList(), TypeList.TYPE_DOCCAT, null ) ;
               }



               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the language                                        */
               /*   This will also read in the start up type lists because  */
               /*   the TypeList object automatically reads those in when   */
               /*   the language changes                                    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               //if ( continueInitialize ) {
               //   LogSystem.log ( 1, "Setting Language to Language Ind:"+UserSystem.getLanguage() ) ;
               //   if ( initWindow != null )
               //      initWindow.setText ( STR_SETTING_LANGUAGE ) ;
               //   Str.switchLanguage ( UserSystem.getLanguage() ) ;
               //   PropertySystem.putInt ( PropertySystem.PROPERTY_LANGUAGE, UserSystem.getLanguage() ) ;
               //}

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in all of the valid users                          */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Reading all users" ) ;
                  if ( initWindow != null )
                     initWindow.setText ( STR_CACHING_ALL_USERS ) ;
                  continueInitialize = UserSystem.refreshAllUsers() ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in all of the valid brand                          */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Reading all brand and family information" ) ;
                  if ( initWindow != null )
                     initWindow.setText ( STR_READING_ALL_BRAND_FAMILY ) ;
                  new Brands ( PropertySystem.getBool ( PropertySystem.PROPERTY_CACHE_BRAND_ASSOC ) ) ;
                  Brands.setCacheDir ( CacheSystem.getCacheDir() ) ;
                  continueInitialize = Brands.readAssociations ( LogSystem.getInstance() ) ;
               }



               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Run setup if need be                                    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( continueInitialize ) && ( getRunSetup() ) ) {
                  if ( new SetupWizard().getResult() ) {
                  } else
                     System.exit(0) ; // Exit since they cancelled setup
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in all of the custom views                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Reading all Custom views" ) ;
                  if ( initWindow != null )
                     initWindow.setText ( STR_READING_CUSTOM_VIEWS ) ;
                  CustomViewSystem.readViews() ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in all of the bookmarks                            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Reading all Bookmarks" ) ;
                  if ( initWindow != null )
                     initWindow.setText ( STR_READING_BOOKMARKS ) ;
                  BookmarkSystem.readBookmarks() ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Put up the Quest Workspace                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( continueInitialize ) {
                  LogSystem.log ( 1, "Initialization Complete, bringing Workspace up" ) ;

                  WindowSystem.createMainWindow() ;
                  MainWindow.createWorkspace() ;
                  MainWindow.getInstance().setApplicationInstance ( qit ) ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   End our Log Section                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               LogSystem.endLogSection () ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Throw an error in the log and start over                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !continueInitialize ) {

                  LogSystem.log ( 1, "Initialization Failed. Bringing up Logon Screen Again" ) ;


                  if ( initWindow != null )
                     initWindow.dispose() ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Ask the user to log in, if they do so, continue with    */
                  /*  initialization. Otherwise exit.                          */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  Qit.setEnabled ( true ) ;
                  PropertySystem.putString ( PropertySystem.PROPERTY_PASSWORD, "" ) ;
                  run() ; 
               } else
                  Qit.setEnabled ( true ) ;

            } /* if continueInitialize  */

         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
            Qit.setEnabled ( true ) ;
            run() ;
         }
      }

      DatabaseSystem.removeStatusListener ( this ) ;
      if ( initWindow != null )
         initWindow.dispose() ;
   }





   /*-----------------------------------------------------------*/
   /*   loadExternalClasses                                     */                  
   /*                                                           */
   /*   Description:                                            */
   /*      This method will try and load all of our external    */                                                                  
   /*   classes. If any of them fail, it will report the error  */
   /*   and return false.                                       */
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      TRUE = Loaded all classes                            */
   /*-----------------------------------------------------------*/
   private static boolean loadExternalClasses () {
      boolean rc              = false ;
      String  className       = "" ;
      boolean useAppDriver    = PropertySystem.getBool ( PropertySystem.PROPERTY_APPDRIVER ) ;

      /* Load our initial external classes */
      if ( useAppDriver )
         rc = ( loadClass ( "COM.ibm.db2.jdbc.app.DB2Driver" ) != null ) ;
      else
         rc = ( loadClass ( "COM.ibm.db2.jdbc.net.DB2Driver" ) != null )  ;


      if ( rc )
         LogSystem.log ( 1, "All External Classes Loaded" ) ;


      return( rc ) ;
   } /* loadExternalClasses */





   /*------------------------------------------------------------*/
   /* loadClass
    */
   /**
    *    This method will load the external class
    *  passed in
    *  
    *
    *  @param className The string representation of the class name
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static Class loadClass ( String className ) {
      Class retClass = null ;

      try {

         LogSystem.log ( 1, "Loading Class "+className ) ;
         retClass = Class.forName ( className ) ;

      } catch ( ClassNotFoundException e ) {
         Frame tempFrame = new Frame ( "" ) ;

         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( tempFrame, "Fatal Error", "Cannot load Class:"+className ) ;
         tempFrame.dispose() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( retClass ) ;

   } /* loadClass */


   /*------------------------------------------------------------*/
   /* loadClass
    */
   /**
    *    This method will load the external class
    *  passed in
    *  
    *
    *  @param className The string representation of the class name
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static Class loadClassNoError ( String className ) {
      Class retClass = null ;

      try {

         LogSystem.log ( 1, "Loading Class "+className ) ;
         retClass = Class.forName ( className ) ;

      } catch ( ClassNotFoundException e ) {
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( retClass ) ;

   } /* loadClass */






   /*------------------------------------------------------------*/
   /* databaseStatusChange
    */
   /**
    *    This method will notify us when a status
    *  in the database we are listeneing for has
    *  changed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void databaseStatusChange ( int statusType, Database database ) {
      switch ( statusType ) {
         case DATABASE_STATUS_CONNECTING :
            if ( ( initWindow != null ) && ( !connectingShown ) ) {
               connectingShown = true ;
               initWindow.setText ( Str.getStr ( STR_CONNECTING_DATABASE )+" "+database.getName() ) ;
            }
            break ;

      }
   }


   /*------------------------------------------------------------*/
   /* updateApplication
    */
   /**
    *    This method simply kicks off the executable to update the app
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void updateApplication () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Tell the user                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.printBox ( STR_INFO, STR_NEW_APPLICATION_VERSION ) ;                               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we must download the code. Use the                 */
      /* QuestUpdater class to do this                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      new QuestUpdater() ;


   }




   /*------------------------------------------------------------*/
   /* createInitializeWindow
    */
   /**
    *    This method will attempt to instantiate the
    *  Initialize Window from the current skin directory
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private InitializeWindow createInitializeWindow () {
      InitializeWindow initWin = null ;

      try {
         String packageName = "com.ibm.nzna.projects.qit.skins."+PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+"." ;
         String className   = packageName+"SkinInitializeWindow" ; 
         Class  winClass    = loadClassNoError ( className ) ;

         if ( winClass == null )
            winClass = loadClassNoError ( "com.ibm.nzna.projects.qit.skins.Default.SkinInitializeWindow" ) ;

         if ( winClass != null ) {
            initWin = (InitializeWindow)winClass.newInstance() ;
         } 

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         initWin = null ;
      }


      return( initWin ) ;

   }





   /*------------------------------------------------------------*/
   /* getRunSetup
    */
   /**
    *    This method will determine if the application
    *  needs to run setup.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Yes, FALSE = No
    *//*---------------------------------------------------------*/
   private static boolean getRunSetup () {
      boolean setup = true ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, if the APPLICATIONSETUP property is not set      */
         /* then we need to run setup.                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setup = !PropertySystem.getBool ( PropertySystem.PROPERTY_APPLICATIONSETUP ) ;                                    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Check to make sure the temp directory exists.           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !setup ) {
            File tempDir = new File ( PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ) ;

            setup = !tempDir.exists() ;
         }
      } catch ( Exception e ) {
      }

      return( setup ) ;
   }






} /* InitializeThread */
