package com.ibm.nzna.projects.qit.app ;     
/*-----------------------------------------------------------*/
/*   LoggedUserRec                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LoggedUserRec                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
/**
 * This Object holds specific information about a Logged On User
 *
 *
 * @version 1.0, 3/12/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LoggedUserRec                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class LoggedUserRec extends UserRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*------------------------------------------------------------*/
   /* LoggedUserRec
    */
   /**
    *    This method will create a LoggedUserRec
    *  
    *  
    *
    *  @param userId The User Id
    *  @param name The name of the user
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LoggedUserRec ( String userId, String name ) {
      super ( userId, name ) ;
   }


   /*------------------------------------------------------------*/
   /* clone
    */
   /**
    *    This method will return a clone of this
    *  object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object clone () {
      LoggedUserRec retRec = new LoggedUserRec ( getUserId(), getName() ) ;

      retRec.setManagerUserId ( getManagerUserId() ) ;
      retRec.setIsManager     ( isManager() ) ;
      retRec.setCountries     ( getCountries() ) ;
      retRec.setBrandMgrVec   ( getBrandMgrVec() ) ;
      retRec.setManagerEMail  ( getManagerEMail() ) ;
      retRec.setPhone         ( getPhone() ) ;
      retRec.setEMail         ( getEMail() ) ;
      retRec.setIPAddress     ( getIPAddress() ) ;
      retRec.setLanguage      ( getLanguage() ) ;
      retRec.setTeams         ( getTeams() ) ;
      retRec.setAuths         ( getAuths() ) ;
      retRec.updateRecStatus  ( getRecStatus() ) ;
      retRec.setPassword      ( getPassword() ) ;

      return( retRec ) ;
   }





   /*------------------------------------------------------------*/
   /* isBPM
    */
   /**
    *    This method will simply return TRUE if this
    *  user is a BPM of any brand
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isBPM () {
      Vector  vec = getBrandMgrVec() ;
      boolean rc  = false ;

      rc = ( ( vec != null ) && ( vec.size() > 0 ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always let development have this auth                   */                            
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !rc )
         rc = hasAuth ( Authority.AUTH_SUPERUSER ) ;

      return ( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* isBPM
    */
   /**
    *    This method will return TRUE if the user
    *  is a BPM of the brandInd passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isBPM ( int brandInd ) {
      Vector  vec = getBrandMgrVec () ;
      boolean rc  = false ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i    =  0;
         int size = vec.size() ;

         while ( ( !rc ) && ( i < size ) ) {
            rc = ( ((BrandMgrRec)vec.elementAt(i)).getBrandInd() == brandInd ) ;
            i++ ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always let development have this auth                   */                            
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !rc )
         rc = hasAuth ( Authority.AUTH_SUPERUSER ) ;
      
      return ( rc ) ;
   }
} /* LoggedUserRec */
