package com.ibm.nzna.projects.qit.app ;    
/*-----------------------------------------------------------*/
/*   MainWindow                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MainWindow                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The MainWindow is the Main GUI Processing center of the
 * Quest Application. Although there may be many frames in
 * the Quest Application, there is only one Main Window.
 *
 *
 * @version 1.0, 1/8/2000 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.shared.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   MainWindow                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class MainWindow extends AppDefaultWin {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static MainWindow mainWindow      = null ;
   private static final int      PROPERTY_WINDOW_POSITION      = 101 ;

   private Qit qit = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will simply set the
    *  size of the Main Window and will
    *  show the main window. All other initializing will
    *  be done when the initialize method is called. This will
    *  provide a faster way of showing the user the blank window,
    *  and then the menu bars etc..
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MainWindow () {
      super ( ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our instance                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mainWindow = this ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties and then show our window              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setTitle ( Str.getApplicationName() ) ;
      setSize () ;
      try {
         setVisible ( true ) ;                                 
      } catch ( Exception e ) {
         setVisible ( true ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return the instance
    *  of the main window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static MainWindow getInstance() {
      return( mainWindow ) ;
   }





   /*------------------------------------------------------------*/
   /* windowClosed
    */
   /**
    *    This method will be called when the
    *  window needs to close. We will shutdown
    *  here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowClosing ( WindowEvent e ) {
      PropertySystem.put ( PROPERTY_WINDOW_POSITION, getBounds() ) ;
      new ShutdownThread ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* createWorkspace
    */
   /**
    *    This method will create our workspace. This is
    *  the only window in the QuestApplication
    *  that should show a Workspace
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void createWorkspace() {
      getInstance().addPanel ( new com.ibm.nzna.projects.qit.workspace.WorkspacePanel() ) ;
   }





   /*------------------------------------------------------------*/
   /* setSize
    */
   /**
    *    This method will first try to set our size and
    *  position from the property we have saved. If
    *  there is no property, then we will default
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setSize () {
      boolean useDefault = false ;

      try {
         Rectangle bounds = (Rectangle) PropertySystem.get ( PROPERTY_WINDOW_POSITION ) ;                                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have a good Rectangle, then we have our position  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( bounds != null ) {
            setBounds ( bounds ) ;

         } else
            useDefault = true ;
      } catch ( Exception e ) {
         useDefault= true ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we use our default, reposition and size now          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( useDefault ) {
         setSize ( 640, 480 ) ;
         WinUtil.centerWindow ( this ) ;
      } /* if default */

   } /* setSize */




   /*------------------------------------------------------------*/
   /* setApplicationInstance
    */
   /**
    *    Since the MainWindow is alive throughout the
    *  life of the application, an instance of the application
    *  is kept to keep the Garbage Collector from Finalizing it
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setApplicationInstance ( Qit qit ) {
      this.qit = qit ;
   }
} /* MainWindow */
