package com.ibm.nzna.projects.qit.app ;           
/*-----------------------------------------------------------*/
/*   PropertySystem                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PropertySystem                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing.table.* ;
import javax.swing.* ;
/**
 * The Properties system handles all application properties that must be 
 * stored for future use. These properties will remain after the application
 * is closed and re opened. Any property added or changed will result
 * in the PropertySystem firing a PropertyChanged event to all PropertyListeners.
 * There are certain properties that are pre defined for the application and
 * should never be set by any QuestApplet, any attempt to set these properties
 * by any code but the base will be rejected by Exception. Some properties are
 * set by command line options passed to the Quest Input Application.<br><br>
 *
 * If for any reason you wish to disregard all properties from a previous version
 * of Quest and start over (for example, you have changed the data type of a specific
 * property, You must change the PROPERTY_VERSION variable defined in this class file
 * and recompile it before releasing the code to the client. This will force the PropertySystem
 * to delete all old properties and recreate them from default when the application starts.<br><br>
 * 
 * <pre>
 * These properties are as follows:<br>
 *<br>
 * PROPERTY_PROPERTY_VERSION            The version of the properties. If the version changes from what the application expects, all properties are deleted and the defaults take over
 * PROPERTY_LANGUAGE                    Holds the integer type of what language should be displayed<br>
 * PROPERTY_LANGUAGEFILE                Holds the name of the file that holds the strings for the PROPERTY_LANGUAGE<br>
 * PROPERTY_COUNTRY                     Returns the country that the Quest Application is configured for. This is the COUNTRYCODEIND that is in TIGRIS.TYPECOUNTRYCODE.<br>
 * PROPERTY_LARGETITLEFONT              Holds what font all Large titles should be in<br>
 * PROPERTY_SMALLTITLEFONT              Holds what font all Small titles should be in<br>
 * PROPERTY_DEFAULTFONT                 Holds what font the default font should be<br>
 * PROPERTY_LISTFONT                    Holds what font all lists should be displayed in<br>
 * PROPERTY_BUTTONFONT                  Holds what font all buttons should be displayed in<br>
 * PROPERTY_TEXTFONT                    Holds what font all text should be a part of<br>
 * PROPERTY_SHOWSKIN                    A Boolean value indicating if the app is drawing it's skin<br>
 * PROPERTY_USERID                      Indicates that the current User Id of the application has changed<br>
 * PROPERTY_PASSWORD                    The unencrypted version of the user's password                          * NOT Saved from session to session<br>
 * PROPERTY_APPDB                       Indicates the name of the database that the application uses<br>
 *                                      for User Ids and global Application settings<br>
 * PROPERTY_APPDB_PORT                  The port number that the application database is listening on<br>
 * PROPERTY_APPDB_SERVER                The server name that the application database is listening on<br>
 * PROPERTY_BATCHMODE                   If the application is running in Batch Mode, ie NO GUI!<br>
 * PROPERTY_TESTMODE                    If the application is running under Test<br>
 * PROPERTY_DEBUGLEVEL                  The Debug level. The three debug levels are:<br>
 * PROPERTY_APPDRIVER                   If TRUE, the application is using the DB/2 App Driver, FALSE = DB/2 Net Driver<br>
 * PROPERTY_DEBUGLEVEL                  1 - Show Only Severe Exceptions<br>
 *                                      2 - Show SQL and Severe Error messages<br>
 *                                      3 - Show ALL Debug Messages<br>
 * PROPERTY_MAINWINDOW                  Returns an instance of the main window<br>
 * PROPERTY_CHATSYSTEM                  Returns an instance of the Chat System <br>
 * PROPERTY_IMPORTFILE                  The filename for a batch to import<br>
 * PROPERTY_RUNBATCH                    The batch name to run in batch mode<br>
 * PROPERTY_PASSWORD                    The Unencrypted User's password<br>
 * PROPERTY_SKINNAME                    The name of the current Skin<br>
 * PROPERTY_LAF                         The current Look and Feel<br>
 * PROPERTY_HOMEDIR                     The directory that the Quest Application was started from<br>
 * PROPERTY_DEFAULTFONT                 The default font all controls must use<br>
 * PROPERTY_DEFAULTBOLDFONT             The default font for any bolded text<br>
 * PROPERTY_BUTTONFONT                  The font that all buttons should use<br>
 * PROPERTY_LISTFONT                    The font that all lists should use<br>
 * PROPERTY_TITLEFONT                   The font that should be used to display Titles<br>
 * PROPERTY_SMALLTITLEFONT              The font that should be used to display smaller titles.<br>
 * PROPERTY_SMALLFONT                   The font that should be used to display small text.         <br>
 * PROPERTY_PANELTITLEFONT              The font that displays the title of a panel
 * PROPERTY_FADECOLOR                   The color that every other item in a list will have as a background<br>
 * PROPERTY_ACTIONBUTTONCOLOR           The color of the text that is displayed on action buttons<br>
 * PROPERTY_APPLICATIONSETUP            This value will be false initally, once the user has completed setup, it will return to TRUE<br>
 * PROPERTY_TEMP_DIR                    The temporary directory that will be wiped clean upon every startup. Please note that this directory path does not include a file separator at the end of the path<br>
 * PROPERTY_TOOLBAR_TEXT_ORIENT         The orientation of the toolbar text. Valid values are LEFT, RIGHT, TOP, BOTTOM<br>
 * PROPERTY_TOOLBAR_SHOW_ICON_TEXT      Either ICONTEXT, TEXT or ICON. This determines what is displayed in each toolbar button<br>
 * PROPERTY_TOOLBAR_HELP                If TRUE, each toolbar button will display it's fly over help<br>
 * PROPERTY_TOOLBAR_BUTTON_SIZE         The size of  each button. Currently 'SMALL' or 'DEFAULT'<br>
 * PROPERTY_MDI                         This will be TRUE if the application should have multiple windows, FALSE if everything is contained in one window<br>
 * PROPERTY_DEFAULT_BRAND               This is the default brand indicator (Integer Object) for the user<br>
 * PROPERTY_DEFAULT_GROUP               This is the default group indicator for the user<br>
 * PROPERTY_DETACH_DIRECTORY            This is the default directory for detaching files<br>
 * PROPERTY_USE_SWING_FILE_DIALOGS      This value tells if we should use the host File Open/Save, or the Swings.<br>
 * PROPERTY_IMPORT_DIRECTORY            This value is the default directory for importing files.<br>
 * PROPERTY_SAVE_AND_CLOSE              If this value is TRUE, then any save operation should close a panel, otherwise the panel should remain open<br>
 * PROPERTY_USE_EXTERNAL_WORDPROC       If this value is TRUE, then the user wishes to use a 3rd party word processor.<br>
 * PROPERTY_EXTERNAL_WORDPROC_EXE       This is the full path and executable information for a 3rd party word processor.<br>
 * PROPERTY_EXTERNAL_WORDPROC_NAME      This is the name of the External Word Processor<br>
 * PROPERTY_IMPORT_HTML_DIR             This is the directory to import HTML files from<br>
 * PROPERTY_IMPORT_GRAPHIC_DIR          This is the directory to default loading graphics for content from<br>
 * PROPERTY_IMPORT_PRODUCT_DIR          This is the directory to default loading product import files from<br>
 * PROPERTY_SOCKS_SERVER                This is the name of the socks server the Input Tool will use<br>
 * PROPERTY_SOCKS_PORT                  This is the port number of the socks server<br>
 * PROPERTY_FTP_SAVE_DIR                This is the default load/save directory for the FTP Explorer<br>
 * PROPERTY_CLOSE_EXTERNAL_WORDPROC     If this value is TRUE, the tool will automatically close the external word processor after a re-import of the text that is being edited<br>
 * PROPERTY_BROWSER_ZOOM                This is the current zoom factor of the browser<br>
 * PROPERTY_FIRST_APPLICATION_RUN       This is TRUE if the application has not run on the client yet, FALSE means that this is not the first time the application has run<br>
 * PROPERTY_STATS_EXPORT_DIR            This is the directory to save the exports<br>
 * PROPERTY_USE_DEFAULT_BOOKMARK_NAME   If TRUE, always use the 'Default' Bookmark name instead of allowing the user to change it<br>
 * PROPERTY_LAST_BOOKMARK_FOLDER_USED   The name of the last folder that was used in the bookmarks (String Name)<br>
 * PROPERTY_SHOW_WHATS_NEW              If this value is true, the 'What's New' window can appear. The user can turn this off<br>
 * PROPERTY_MACHINEMODELTREE            This is the last known width of each column in the DetailMachineModelTree control<br>
 * PROPERTY_LAST_SPLIT_SELECTION        This is an array of the last selected indexes in the PopUpSplitTypeList<br>
 * PROPERTY_LAST_SELECTED_BRAND         This is the TypeCatInd of the last selected brand<br>
 * PROPERTY_LAST_SELECTED_FAMILY        This is the TypeCatInd of the last selected family<br>
 * PROPERTY_OS2                         This property is TRUE or FALSE, TRUE if this program is running on OS/2<br>
 * PROPERTY_CLEAR_TEMP_DIR_ON_START     If this is TRUE, the temp directory will be erased when the application starts<br>
 * PROPERTY_LOGO                        If TRUE, the Quest Logo will appear during Startup, FALSE = No Logo<br>
 * PROPERTY_LOGS_KEEP                   The number of days to keep log files for.<br>
 * PROPERTY_FIRST_APPLET_RUN            This is the String name of the first applet that is run by the user<br>
 * PROPERTY_FIRST_APPLET_RUN_COUNT      This is the number of times that the user has run the applet in PROPERTY_FIRST_APPLET_RUN.<br>
 * PROPERTY_STARTUP_APPLET              This is the name of the applet to always start when the application starts. This value can be NULL<br>
 * PROPERTY_ASK_FIRST_APPLET_RUN        This is a boolean value to let the Workspace know if it should ask to auto start the first applet<br>
 * PROPERTY_CACHE_TYPE_LISTS            This is a boolean value that tells wether or not to cache type lists<br>
 * PROPERTY_CACHE_BRAND_ASSOC           This is a boolean value that tells if QIT should cache Brand Family Associations<br>
 * PROPERTY_CURRENT_SUB_VERSION         This is the sub version of this property file, Please see the PROPERTY_SUB_VERSION variable comemnts for more information<br>
 * PROPERTY_LAST_SELECTED_FAMILIES      This is a serialized vector of Integer objects. Each Integer object represents a Group Indicator from the TIGRIS.TYPEGROUP table<br>
 * PROPERTY_ANTI_ALIAS                  This method will turn anti-aliasing on or off
 * PROPERTY_FORM_FOREGROUND             This is the color of the descriptions on the forms in the editors
 * PROPERTY_FORM_VALUE_FOREGROUND       This is the color of the values on the forms in the editors
 * PROPERTY_FORM_TITLE_FOREGROUND       This is the color of the titles in the forms.
 * PROPERTY_FORM_FONT                   This is the font for all forms
 * PROPERTY_FORM_VALUE_FONT             This is the font for all data values drawn on a form
 * PROPERTY_FORM_TITLE_FONT             This is the font of all Form Titles
 * PROPERTY_LAST_GEOGRAPHY              This is the last geography that the user selected
 * PROPERTY_SHOW_INTERNAL_WORDPROC      If TRUE, any word processor Quest Applet should allow the selection of the Quest Internal Word Processor. If FALSE, that should never be an option
 * PROPERTY_EXTERNAL_BROWSER            This is the full path to the browser of the user' choice
 * <pre><br><br>
 *
 *
 * Preallocated blocks of Properties. If your Quest Applet uses it's own defined properties, Please pre-allocate a block
 * of properties here. Please keep your Properties to a minimum. Also, remember to update the MAX_PROPERTY static variable to
 * be large enough to hold the defined properties.
 *
 * <pre>
 * Quest Applet/Class               Numbers reserved<br>
 * -------------------------------- ----------------<br>
 * Quest Application                1-100           <br>
 * MainWindow.class                 101-120         <br>
 * Avalon Quest Applet              121-150         <br>
 * Document Quest Applet            151-200         <br>
 * Product Quest Applet             201-250         <br>
 *    
 *
 * @version 1.0, 1/7/2000
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   PropertySystem                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class PropertySystem extends Object implements Runnable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public static members                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int PROPERTY_LANGUAGE                     = 1 ;
   public static final int PROPERTY_COUNTRY                      = 2 ;
   public static final int PROPERTY_LARGETITLEFONT               = 3 ;
   public static final int PROPERTY_SMALLTITLEFONT               = 4 ;
   public static final int PROPERTY_DEFAULTFONT                  = 5 ;
   public static final int PROPERTY_LISTFONT                     = 6 ;
   public static final int PROPERTY_BUTTONFONT                   = 7 ;
   public static final int PROPERTY_TEXTFONT                     = 8 ;
   public static final int PROPERTY_SHOWSKIN                     = 9 ;
   public static final int PROPERTY_USERID                       = 10 ;
   public static final int PROPERTY_APPDB                        = 11 ;
   public static final int PROPERTY_APPDB_PORT                   = 12 ;
   public static final int PROPERTY_APPDB_SERVER                 = 13 ;
   public static final int PROPERTY_BATCHMODE                    = 14 ;
   public static final int PROPERTY_TESTMODE                     = 15 ;
   public static final int PROPERTY_DEBUGLEVEL                   = 16 ;
   public static final int PROPERTY_APPDRIVER                    = 17 ;
   public static final int PROPERTY_PASSWORD                     = 18 ;
   public static final int PROPERTY_RUNBATCH                     = 19 ;
   public static final int PROPERTY_IMPORTFILE                   = 20 ;
   public static final int PROPERTY_SKINNAME                     = 21 ;
   public static final int PROPERTY_LAF                          = 22 ;
   public static final int PROPERTY_HOMEDIR                      = 23 ;
   public static final int PROPERTY_FADECOLOR                    = 24 ;
   public static final int PROPERTY_SMALLFONT                    = 25 ;
   public static final int PROPERTY_ACTIONBUTTONCOLOR            = 26 ;
   public static final int PROPERTY_SHUTDOWNCLEAN                = 27 ;
   public static final int PROPERTY_PREV_SHUTDOWNCLEAN           = 28 ;
   public static final int PROPERTY_APPLICATIONSETUP             = 29 ;
   public static final int PROPERTY_TEMP_DIR                     = 30 ;
   public static final int PROPERTY_TOOLBAR_TEXT_ORIENT          = 31 ;
   public static final int PROPERTY_TOOLBAR_SHOW_ICON_TEXT       = 32 ;
   public static final int PROPERTY_TOOLBAR_HELP                 = 33 ;
   public static final int PROPERTY_TOOLBAR_BUTTON_SIZE          = 34 ;
   public static final int PROPERTY_MDI                          = 35 ;
   public static final int PROPERTY_LANGUAGEFILE                 = 36 ;
   public static final int PROPERTY_PANELTITLEFONT               = 37 ;
   public static final int PROPERTY_DEFAULT_BRAND                = 38 ;
   public static final int PROPERTY_DEFAULT_GROUP                = 39 ;
   public static final int PROPERTY_DETACH_DIRECTORY             = 40 ;
   public static final int PROPERTY_USE_SWING_FILE_DIALOGS       = 41 ;
   public static final int PROPERTY_IMPORT_DIRECTORY             = 42 ;
   public static final int PROPERTY_SAVE_AND_CLOSE               = 43 ;
   public static final int PROPERTY_USE_EXTERNAL_WORDPROC        = 44 ;
   public static final int PROPERTY_EXTERNAL_WORDPROC_EXE        = 45 ;
   public static final int PROPERTY_IMPORT_HTML_DIR              = 46 ;
   public static final int PROPERTY_IMPORT_GRAPHIC_DIR           = 47 ;
   public static final int PROPERTY_IMPORT_PRODUCT_DIR           = 48 ;
   public static final int PROPERTY_SOCKS_SERVER                 = 49 ;
   public static final int PROPERTY_SOCKS_PORT                   = 50 ;
   public static final int PROPERTY_FTP_SAVE_DIR                 = 51 ;
   public static final int PROPERTY_EXTERNAL_WORDPROC_NAME       = 52 ;
   public static final int PROPERTY_CLOSE_EXTERNAL_WORDPROC      = 53 ;
   public static final int PROPERTY_BROWSER_ZOOM                 = 54 ;
   public static final int PROPERTY_FIRST_APPLICATION_RUN        = 55 ;
   public static final int PROPERTY_STATS_EXPORT_DIR             = 56 ;
   public static final int PROPERTY_USE_DEFAULT_BOOKMARK_NAME    = 57 ;
   public static final int PROPERTY_LAST_BOOKMARK_FOLDER_USED    = 58 ;
   public static final int PROPERTY_PROPERTY_VERSION             = 59 ;
   public static final int PROPERTY_START_DBWINDOW               = 60 ;
   public static final int PROPERTY_COUNTRY_FAVORITES            = 61 ;
   public static final int PROPERTY_BRAND_FAVORITES              = 62 ;
   public static final int PROPERTY_COUNTRY_FAVORITES_AUTO       = 63 ;
   public static final int PROPERTY_BRAND_FAVORITES_AUTO         = 64 ;
   public static final int PROPERTY_SHOW_WHATS_NEW               = 65 ;
   public static final int PROPERTY_MACHINEMODELTREE             = 66 ;
   public static final int PROPERTY_LAST_SPLIT_SELECTION         = 67 ;
   public static final int PROPERTY_LAST_SELECTED_BRAND          = 69 ;
   public static final int PROPERTY_OS2                          = 70 ;
   public static final int PROPERTY_CLEAR_TEMP_DIR_ON_START      = 71 ;
   public static final int PROPERTY_LOGO                         = 72 ;
   public static final int PROPERTY_LOGS_KEEP                    = 73 ;
   public static final int PROPERTY_FIRST_APPLET_RUN             = 74 ;
   public static final int PROPERTY_FIRST_APPLET_RUN_COUNT       = 75 ;
   public static final int PROPERTY_STARTUP_APPLET               = 76 ;
   public static final int PROPERTY_ASK_FIRST_APPLET_RUN         = 77 ;
   public static final int PROPERTY_CACHE_TYPE_LISTS             = 78 ;
   public static final int PROPERTY_CACHE_BRAND_ASSOC            = 79 ;
   public static final int PROPERTY_CURRENT_SUB_VERSION          = 80 ;
   public static final int PROPERTY_LAST_SELECTED_FAMILIES       = 81 ;
   public static final int PROPERTY_ANTI_ALIAS                   = 82 ;
   public static final int PROPERTY_FORM_FOREGROUND              = 83 ;
   public static final int PROPERTY_FORM_VALUE_FOREGROUND        = 84 ;
   public static final int PROPERTY_FORM_TITLE_FOREGROUND        = 85 ;
   public static final int PROPERTY_FORM_FONT                    = 86 ;
   public static final int PROPERTY_FORM_VALUE_FONT              = 87 ;
   public static final int PROPERTY_FORM_TITLE_FONT              = 88 ;
   public static final int PROPERTY_LAST_GEOGRAPHY               = 89 ;
   public static final int PROPERTY_LAST_SELECTED_FAMILY         = 90 ;
   public static final int PROPERTY_SHOW_INTERNAL_WORDPROC       = 91 ;
   public static final int PROPERTY_EXTERNAL_BROWSER             = 92 ;



   private static final int MAX_PROPERTY = 251 ;

   /**
     *  If you alter the PROPERTY_VERSION, all current properties
     *  will be set back to their default values no matter what
     *  is saved in the property file.
     **/
   private static final int PROPERTY_VERSION  = 8 ;

   /**
     *  If you alter the PROPERTY_SUB_VERSION, the 'setNewPropertiesDefault'
     *  method will be called, allowing you to set any new properties default
     *  value
     **/
   private static final int PROPERTY_SUBVERSION = 2 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Static Members                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String QUEST_PROP             = "quest.prop" ;
   private static Hashtable        propHash         = null ;
   private static PropertySystem   instance         = null ;
   private static Vector[]         listenerArray    = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int     propertyFire            = 0 ;
   private Object  valueFire               = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the PropertySystem for the Quest Application.
    *  WARNING!!! - Only one instance of the Property System should be created.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PropertySystem () {
      java.io.File propFile                = new java.io.File ( QUEST_PROP ) ;                                         
      Properties   p                       = System.getProperties () ;
      String       homeDir                 = p.getProperty ( "user.dir" ) ;
      boolean      createFromDefault       = true ;
      boolean      os2                     = false ;
      Properties   prop                    = System.getProperties () ;
      String       splatform               = prop.getProperty ( "os.name" ) ;
      boolean      setNewPropertiesDefault = false ;

      if ( splatform.indexOf ( "OS/2" ) != -1 )
         os2=true ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, if a previous Property file exists, read it      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( propFile.exists() ) {
         createFromDisk() ;
         if ( getInt ( PROPERTY_PROPERTY_VERSION ) != PROPERTY_VERSION )
            createFromDefault = true ;
         else
            createFromDefault = false ;
         if ( getInt ( PROPERTY_CURRENT_SUB_VERSION ) != PROPERTY_SUBVERSION ) {
            putInt ( PROPERTY_CURRENT_SUB_VERSION, PROPERTY_SUBVERSION ) ;
            setNewPropertiesDefault = true ;
         } else
            setNewPropertiesDefault = false ;
      }
      if ( createFromDefault ) {
         propHash = new Hashtable() ;                      
         setDefaultProperties() ;
      }
      if ( setNewPropertiesDefault )
         setNewPropertiesDefault() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, set our default properties                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.instance = this ;                
      putString ( PROPERTY_HOMEDIR, homeDir ) ;
      putBool   ( PROPERTY_APPDRIVER, false ) ;
      putBool   ( PROPERTY_OS2,       os2 ) ;
      putBool   ( PROPERTY_LOGO,      true ) ;
      putBool   ( PROPERTY_TESTMODE,  false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up our listener array                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listenerArray = new Vector [ MAX_PROPERTY ] ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor is mainly used to
    *  kick off the thread that fires property
    *  change events.
    *
    *  @param property The property to fire
    *  @param value The value to fire
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PropertySystem ( int property, Object value ) {
      this.propertyFire = property ;
      this.valueFire    = value ;
   }




   /*------------------------------------------------------------*/
   /* setDefaultProperties
    */
   /**
    *    This method will set the default properties
    *  for the application
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setDefaultProperties() {

      try {
         Properties p       = System.getProperties () ;
         String     homeDir = p.getProperty ( "user.dir" ) ;

         propHash.put ( new Integer ( PROPERTY_DEBUGLEVEL ), new Integer ( 1 ) ) ; // Do not call putInt because we MUST have a DEBUG LEVEL BEFORE THAT!
         putInt    ( PROPERTY_LOGS_KEEP,                  14 ) ;
         putInt    ( PROPERTY_PROPERTY_VERSION,           PROPERTY_VERSION )  ;
         putInt    ( PROPERTY_FIRST_APPLET_RUN_COUNT,     0 ) ;
         putString ( PROPERTY_FIRST_APPLET_RUN,           "" ) ;
         putString ( PROPERTY_STARTUP_APPLET,             "" ) ;
         putString ( PROPERTY_IMPORT_DIRECTORY,           homeDir ) ;
         putString ( PROPERTY_IMPORT_HTML_DIR,            homeDir ) ;
         putString ( PROPERTY_STATS_EXPORT_DIR,           homeDir ) ;
         putString ( PROPERTY_IMPORT_GRAPHIC_DIR,         homeDir ) ;
         putString ( PROPERTY_DETACH_DIRECTORY,           homeDir ) ;
         putString ( PROPERTY_IMPORT_PRODUCT_DIR,         homeDir ) ;
         putString ( PROPERTY_FTP_SAVE_DIR,               homeDir ) ;
         putString ( PROPERTY_SOCKS_SERVER,               "" ) ;
         putString ( PROPERTY_SOCKS_PORT,                 "" ) ;
         putString ( PROPERTY_APPDB,                      "quest" ) ;
         putString ( PROPERTY_APPDB_PORT,                 "5703" ) ;
         putString ( PROPERTY_APPDB_SERVER,               "webdb01.pc.ibm.com" ) ;
         putString ( PROPERTY_SKINNAME,                   "Default" ) ;
         putString ( PROPERTY_LAF,                        "com.sun.java.swing.plaf.windows.WindowsLookAndFeel" ) ;
         putString ( PROPERTY_HOMEDIR,                    homeDir ) ;
         putString ( PROPERTY_LANGUAGEFILE,               "english" ) ;
         putInt    ( PROPERTY_LANGUAGE,                   com.ibm.nzna.projects.common.quest.type.TypeList.LANGUAGE_ENGLISH ) ;
         putInt    ( PROPERTY_COUNTRY,                    110 ) ;
         putInt    ( PROPERTY_DEFAULT_BRAND,              -1 ) ;
         putInt    ( PROPERTY_DEFAULT_GROUP,              -1 ) ;
         put       ( PROPERTY_COUNTRY_FAVORITES,          new Vector ( 1 ) ) ;
         put       ( PROPERTY_BRAND_FAVORITES,            new Vector ( 1 ) ) ;
         put       ( PROPERTY_DEFAULTFONT,                new Font ( "Helvetica", Font.PLAIN, 12 ) ) ;
         put       ( PROPERTY_BUTTONFONT,                 new Font ( "Helvetica", Font.PLAIN, 12 ) ) ;
         put       ( PROPERTY_LISTFONT,                   new Font ( "Helvetica", Font.PLAIN, 12 ) ) ;
         put       ( PROPERTY_TEXTFONT,                   new Font ( "Helvetica", Font.PLAIN, 12 ) ) ;
         put       ( PROPERTY_LARGETITLEFONT,             new Font ( "Helvetica", Font.PLAIN, 24 ) ) ;
         put       ( PROPERTY_SMALLTITLEFONT,             new Font ( "Helvetica", Font.BOLD,  12 ) ) ;
         put       ( PROPERTY_SMALLFONT,                  new Font ( "Helvetica", Font.PLAIN, 10 ) ) ;
         put       ( PROPERTY_PANELTITLEFONT,             new Font ( "Helvetica", Font.BOLD,  16 ) ) ;
         put       ( PROPERTY_FADECOLOR,                  new Color ( 240, 240, 240 ) ) ;
         put       ( PROPERTY_ACTIONBUTTONCOLOR,          Color.white ) ;
         putInt    ( PROPERTY_BROWSER_ZOOM,               3           ) ;
         putBool   ( PROPERTY_ANTI_ALIAS,                 false ) ;
         putBool   ( PROPERTY_CLEAR_TEMP_DIR_ON_START,    false ) ;
         putBool   ( PROPERTY_CACHE_TYPE_LISTS,           true ) ;
         putBool   ( PROPERTY_CACHE_BRAND_ASSOC,          true ) ;
         putBool   ( PROPERTY_COUNTRY_FAVORITES_AUTO,     true ) ;
         putBool   ( PROPERTY_BRAND_FAVORITES_AUTO,       true ) ;
         putBool   ( PROPERTY_USE_DEFAULT_BOOKMARK_NAME,  false       ) ;
         putBool   ( PROPERTY_SHOWSKIN,                   false       ) ;
         putBool   ( PROPERTY_APPLICATIONSETUP,           false       ) ;
         putBool   ( PROPERTY_SHUTDOWNCLEAN,              false       ) ;
         putBool   ( PROPERTY_APPDRIVER,                  false       ) ;
         putBool   ( PROPERTY_USE_SWING_FILE_DIALOGS,     false       ) ;
         putBool   ( PROPERTY_TESTMODE,                   false       ) ;
         putBool   ( PROPERTY_SHOW_WHATS_NEW,             true        ) ;
         putBool   ( PROPERTY_ASK_FIRST_APPLET_RUN,       true        ) ;
         putBool   ( PROPERTY_SHOW_INTERNAL_WORDPROC,     true        ) ;
         putString ( PROPERTY_TOOLBAR_TEXT_ORIENT,        "RIGHT"     ) ;
         putString ( PROPERTY_TOOLBAR_BUTTON_SIZE,        "DEFAULT"   ) ;
         putString ( PROPERTY_TOOLBAR_SHOW_ICON_TEXT,     "ICONTEXT"  ) ;
         putString ( PROPERTY_EXTERNAL_WORDPROC_NAME,     "Ext. Word Proc"  ) ;
         putBool   ( PROPERTY_CLOSE_EXTERNAL_WORDPROC,    true        ) ;
         putBool   ( PROPERTY_TOOLBAR_HELP,               true        ) ;
         putBool   ( PROPERTY_MDI,                        false       ) ;
         putBool   ( PROPERTY_SAVE_AND_CLOSE,             true        ) ;
         putBool   ( PROPERTY_USE_EXTERNAL_WORDPROC,      false       ) ;
         putBool   ( PROPERTY_FIRST_APPLICATION_RUN,      true        ) ;
         putBool   ( PROPERTY_START_DBWINDOW,             false       ) ;


      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }






   /*------------------------------------------------------------*/
   /* createFromDisk
    */
   /**
    *    This method will create our properties
    *  from the QUEST_PROP file on the
    *  user's client machine
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createFromDisk () {
      Hashtable newProp = (Hashtable) QuestUtil.readObjectFromDisk ( QUEST_PROP ) ;
      if ( newProp == null )
         propHash = new Hashtable () ;
      else
         propHash = newProp ;
   }






   /*------------------------------------------------------------*/
   /* writeToDisk
    */
   /**
    *    This method will write our property file
    *  to disk
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void writeToDisk () {

      QuestUtil.writeObjectToDisk ( QUEST_PROP, propHash ) ;
   }

   private int listenerCount () {
      int i       = 0 ;
      int count   = 0 ;

      //Syste.out.println ( "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n" ) ;
      while ( i < listenerArray.length ) {
         if ( listenerArray[i] != null ) {
            int in = 0 ;
            while ( in < listenerArray[i].size() ) {
               //Syste.out.println ( "\n\n-----------------------------------------------\nListener:" + listenerArray[i].elementAt(in) ) ;
               in++ ;
            }
            count+=listenerArray[i].size() ;
         }
         i++ ;
      }

      return( count ) ;
   }


   /*------------------------------------------------------------*/
   /* getInt
    */
   /**
    *    This method will return an 'int' property
    *  from the Quest Application.
    *  
    *
    *  @param property The Description of the property
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static int getInt ( int property )  {
      try {
         return( ((Integer)propHash.get ( new Integer ( property ) )).intValue() ) ;
      } catch ( Exception e ) {
      }
      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* get
    */
   /**
    *    This method will return a single
    *  object from the Quest Application properties
    *  
    *
    *  @param property The name of the property
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Object get ( int property )  {
      return( propHash.get ( new Integer ( property ) ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getString
    */
   /**
    *    This method will return a String
    *  representation of a Quest Application
    *  property
    *
    *  @param property The name of the property
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getString ( int property )  {
      try {
         return( (String)propHash.get ( new Integer ( property ) ) ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* getBool
    */
   /**
    *    This method will return the boolean
    *  value of a Quest Application property
    *  
    *
    *  @param property The propertyion of the property
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public static boolean getBool ( int property ) {
      try {
         return( ((Boolean)propHash.get ( new Integer ( property ) )).booleanValue() ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, "Cannot load BOOLEAN property: "+property ) ;
      }

      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* putBool
    */
   /**
    *    This method will put a boolean value
    *  in the Quest Application Properties
    *  
    *
    *  @param property The name of the property
    *  @param boolean The boolean value to place
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void putBool ( int property, boolean value ) {
      putBool ( property, new Boolean ( value ) ) ;
   }


   /*------------------------------------------------------------*/
   /* putBool
    */
   /**
    *    This method will put a boolean value
    *  in the Quest Application Properties
    *  
    *
    *  @param property The name of the property
    *  @param Boolean The boolean object to place
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void putBool ( int property, Boolean value )  {
      try {
         propHash.put ( new Integer ( property ), value ) ;
         LogSystem.log ( 1, "Property Set. Name:"+property+"   Value:"+value ) ;
         firePropertyChange ( property, value ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, "Cannot put BOOLEAN property into Property Hashtable/ Property:"+property+"   Boolean Value:"+value ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* putString
    */
   /**
    *    This method will put a String value
    *  in the Quest Application Properties
    *  
    *
    *  @param property The name of the property
    *  @param string The String object to place
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void putString ( int property, String value ) {
      try {
         propHash.put ( new Integer ( property ), value ) ;
         if ( property != PROPERTY_PASSWORD )
            LogSystem.log ( 1, "Property Set. Name:"+property+"   Value:"+value ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, "Cannot put String property. Property:"+property+"  String:"+value ) ;
      }

      firePropertyChange ( property, value ) ;
   }



   /*------------------------------------------------------------*/
   /* putInt
    */
   /**
    *    This method will put an int value
    *  in the Quest Application Properties
    *  
    *
    *  @param property The name of the property
    *  @param int The int value to place
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void putInt ( int property, int value ) {
      putInt ( property, new Integer ( value ) ) ;
   }





   /*------------------------------------------------------------*/
   /* put
    */
   /**
    *    This method will just put an Object onto
    *  the Property System
    *  
    *
    *  @param property The name of the property
    *  @param value The object to place
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void put ( int property, Object value ) {
      try {
         if ( value != null ) {
            propHash.put ( new Integer ( property ), value ) ;
            LogSystem.log ( 1, "Property Set. Name:"+property+"    Value:"+value.toString() ) ;
            firePropertyChange ( property, value ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, "Cannot set property Number:"+property+" to Value:"+value ) ;
         LogSystem.log ( 1, e, false ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* putInt
    */
   /**
    *    This method will put an Integer
    *  Object into the Quest Application Properties
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void putInt ( int property, Integer value ) {
      propHash.put ( new Integer ( property ), value ) ;
      LogSystem.log ( 1, "Property Set. Name:"+property+"   Value:"+value ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this is the LANGUAGE property, tell Str to switch!   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( property == PROPERTY_LANGUAGE ) {
         Str.switchLanguage ( value.intValue() ) ;
      }
      firePropertyChange ( property, value ) ;
   }





   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return an instance
    *  of the instantiated PropertySystem
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static PropertySystem getInstance() {
      return( instance ) ;
   }





   /*------------------------------------------------------------*/
   /* addPropertyListener
    */
   /**
    *    This method will add a listener for a
    *  specific property
    *  
    *
    *  @param property The property to listen for
    *  @param listener The listener to notify
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addPropertyListener ( int property, PropertyListener listener ) {
      if ( listenerArray[property] == null )
         listenerArray[property] = new Vector ( 1, 1 ) ;

      listenerArray[property].addElement ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* removePropertyListener
    */
   /**
    *    This method will remove a PropertyListener
    *  from being notified.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removePropertyListener ( int property, PropertyListener listener ) {
      if ( listenerArray[property] != null )
         listenerArray[property].removeElement ( listener ) ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will basically just fire
    *  property change events
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      try {
         int i       = 0 ;
         int size    = listenerArray[propertyFire].size() ;

         while ( i < size ) {
            try {
               ((PropertyListener)listenerArray[propertyFire].elementAt(i)).propertyChanged ( propertyFire, valueFire ) ;
            } catch ( Exception e ) {
            }
            i++ ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* firePropertyChange
    */
   /**
    *    This method will kick off a thread to
    *  fire a property change
    *  
    *
    *  @param property The property to fire
    *  @param value The value to fire
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void firePropertyChange ( int property, Object value ) {
      if ( getInstance() != null )
         getInstance().writeToDisk() ;
      if ( ( listenerArray != null ) && ( listenerArray[property] != null ) ) {
         new Thread ( new PropertySystem(property,value) ).start() ;
      }
   }





   /*------------------------------------------------------------*/
   /* putDetailList
    */
   /**
    *    This method will save the column widths of
    *  a DetailList. The column widths are saved as
    *  an integer array so that when they are read back in,
    *  you can use the 'getDetailList' method to return the
    *  column widths back to the original state
    *
    *  @param property The Property to save under
    *  @param detailList The DetailList to gather column information from
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void putDetailList ( int property, DetailList detailList ) {
      int   columnCount = detailList.getColumnCount() ;

      if ( columnCount > 0 ) {
         int[] widths      = new int[columnCount] ;

         for ( int i = 0; i < columnCount; i++ )
            widths[i] = detailList.getColumnWidth ( i ) ;

         put ( property, widths ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* getDetailList
    */
   /**
    *    This method will read the last known column
    *  widths and set the column widths back. If
    *  there are no column widths to set this method returns FALSe
    *
    *  @param property The property that the widths are saved under
    *  @param detailList The DetailList to set column widths of
    *  @param None
    *
    *  @return TRUE if widths were set, FALSE if not
    *//*---------------------------------------------------------*/
   public static boolean getDetailList ( int property, DetailList detailList ) {
      boolean rc = false ;

      try {
         int[] widths = (int[]) get ( property ) ;

         if ( ( widths != null ) && ( widths.length > 0 ) ) {
            for ( int i = 0; i < widths.length; i++ )
               detailList.setColumnWidth ( i, widths[i] ) ;
            rc = true ;
         }
      } catch ( Exception e ) {
         e.printStackTrace() ;
         rc = false ;
      }

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* getJTable
    */
   /**
    *    This method will read the last known column
    *  widths and set the column widths back. If
    *  there are no column widths to set this method returns FALSe
    *
    *  @param property The property that the widths are saved under
    *  @param table The JTable to adjust
    *  @param None
    *
    *  @return TRUE if widths were set, FALSE if not
    *//*---------------------------------------------------------*/
   public static boolean getJTable ( int property, JTable table ) {
      boolean rc = false ;

      try {
         int[] widths = (int[]) get ( property ) ;

         if ( ( widths != null ) && ( widths.length > 0 ) ) {
            
            for ( int i = 0; i < widths.length; i++ ) 
               ((TableColumn)((TableColumnModel)table.getColumnModel()).getColumn(i)).setPreferredWidth ( widths[i] ) ;

            rc = true ;
         }
      } catch ( Exception e ) {
         e.printStackTrace() ;
         rc = false ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setNewPropertiesDefault
    */
   /**
    *    This method will set any new properties 
    *  default. Please keep in mind that this overrides
    *  what the user has already set
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void setNewPropertiesDefault () {
      putBool ( PROPERTY_CACHE_TYPE_LISTS, true ) ;
      putBool ( PROPERTY_CACHE_BRAND_ASSOC, true ) ;
      putBool ( com.ibm.nzna.projects.qit.product.ProductConst.PROPERTY_PRODUCT_MGMT_SHOW_PROJECT, true ) ;
   }


   /**
     * This method will return the brand indicator that should be selected as the default brand
     **/
   public static int getPreferredBrand () {
      int defaultBrand = getInt ( PROPERTY_DEFAULT_BRAND ) ;
      int lastSelected = getInt ( PROPERTY_LAST_SELECTED_BRAND ) ;

      if ( lastSelected != 0 )
         defaultBrand = lastSelected ;

      return ( defaultBrand ) ;
   }
} /* PropertySystem */

