package com.ibm.nzna.projects.qit.app ;    
/*-----------------------------------------------------------*/
/*   QuestUpdater                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestUpdater                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.net.* ;
import java.awt.* ;
import javax.swing.* ;
import java.util.zip.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.* ;
/**
 * This class updates the Quest Input Tool by doing the following:<br><br>
 *
 * 1. FTP the code from the FTP Site. The code is normally called quest.zip,
 * but that can be changed in the IT_VER constant.<br><br>
 *
 * 2. Unzip the zip file downloaded. The Zip file MUST unzip into a directory
 * called 'new'.<br><br>
 *
 * 3. Kick off the process to copy everything from the new directory to the
 * current directory and then immediatly exit.<br><br>
 *
 * <b>The following steps are done once the tool has exited.</b><br><br>
 *
 * Batch File that is kicked off from the tool does:<br>
 * 1. xcopy everything from $HOME/new to $HOME<br>
 * 2. copy com_ibm_nzna.jar.new to com_ibm_nzna.jar<br>
 * 3. Erase com_ibm_nzna.jar.new<br>
 *
 *
 * @version 1.0, 10/13/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestUpdater                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestUpdater extends Object implements AppConst {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JFrame           tempFrame      = new JFrame () ;
   private StatusWin        statusWin      = new StatusWin ( tempFrame ) ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will kick off our work. It will
    *  never return because a System.exit is
    *  called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestUpdater () {               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, download the code                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( downloadCode() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Unzip the new code                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( unzipCode() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Kick off the batch to copy the files over               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            statusWin.setText ( "Launching Code Copier" ) ;
            statusWin.setValue ( 0 ) ;
            startCopy() ;
         }
      }




      tempFrame.dispose() ;
      System.exit(0) ;

   }





   /*------------------------------------------------------------*/
   /* downloadCode
    */
   /**
    *    This method will download the code from 
    *  whatever the IT_VER String Constant is pointing
    *  to. This will attempt to download the quest.zip file
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean downloadCode () {
      boolean          downloaded     = false ;
      File             file           = null ;
      FileOutputStream out            = null ;
      InputStream      in             = null ;
      int              readByteLength = 1024 ; // The number of byte to read at a time
      int              fileLength     = 0 ;
      URL              url            = null ;                 
      String           urlFile        = Constants.getStrConst ( "IT_VERSION" ) ;
      String           localFile      = "new.zip" ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Bring up the status window                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      statusWin.setText ( "Updating ..." ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Open the URL Connection                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         URLConnection con = null ;

         statusWin.setText ( "Connecting to web site" ) ;
         url = new URL ( urlFile ) ;
         con = url.openConnection() ;

         fileLength = con.getContentLength(); 

         in = con.getInputStream() ;
      } catch ( MalformedURLException e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Cannot locate the URL Specified for download. Please check network connections and try again" ) ;
      } catch ( UnknownHostException e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Cannot connect to the host specified in the URL. Please check network connections and try again" ) ;
      } catch ( IOException e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Cannot connect to the host specified in the URL. Please check network connections and try again" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Cannot connect to the host specified in the URL. Please check network connections and try again" ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Open the local file                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {

         file = new File ( localFile ) ;
         out = new FileOutputStream ( file ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Cannot open local file to download to. This may be because you are out of space, or you do not have write permissions. Please correct and try again" ) ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Download the file                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         byte[] inByteChunk = new byte [ readByteLength ] ;
         int    bRead       = 1 ;
         int    totalRead   = 0 ;
         int    updateIndex = 0 ;

         statusWin.setMaxValue ( fileLength ) ;    

         statusWin.setText ( "Downloading "+fileLength+" Bytes" ) ;
         while ( bRead > 0 ) {
            bRead = in.read ( inByteChunk ) ;

            if ( bRead > 0 ) {
               totalRead+=bRead ;
               statusWin.setValue ( totalRead ) ;
               out.write ( inByteChunk, 0, bRead ) ;
            }
         }

         downloaded = true ;
      } catch ( IOException ex ) {
         LogSystem.log ( 1, ex, false ) ;
         GUISystem.printBox ( "Info", "Cannot save to your workstation. Please manually download the update and install" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_CANNOT_AUTO_UPDATE_AT_THIS_TIME ) ;
      }


      return( downloaded ) ;
   }





   /*------------------------------------------------------------*/
   /* unzipCode
    */
   /**
    *    This method will unzip all of our code that
    *  was just downloaded to the quest home directory
    *  as 'new.zip'.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean unzipCode () {
      Properties  prop          = System.getProperties();
      String      outDir        = PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR ) ;
      String      filePath      = null ;
      File        file          = null ;
      File        root          = null ;
      ZipFile     zin           = null ;
      Enumeration enum          = null ;
      byte[]      buffer        = null ;
      String      currDir       = null ;
      ZipEntry    ze            = null ;
      String      name          = null ;
      File        dir           = null ;
      boolean     cont          = true ;
      int         fileCount     = 0 ;
      int         fileIndex     = 0 ;
      boolean     rc            = true ;
      int         length        = 0 ;

      statusWin.setText ("Extracting" ) ;
      statusWin.setValue ( 0 ) ;
      try {

         if ( outDir.endsWith(File.separator) )
            outDir = outDir.substring(0, outDir.length() - 1);

         // read the .zip file
         zin = new ZipFile("new.zip");
         root = new File(outDir);

         if ( !root.exists() )
            root.mkdirs();

         enum = zin.entries();
         buffer = new byte[1024];
         currDir = outDir;               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, count the entries                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( enum.hasMoreElements() ) {
            fileCount++ ;
            enum.nextElement() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now loop through writing each file                  */                             
         /*                                                           */
         /*-----------------------------------------------------------*/
         statusWin.setMaxValue ( fileCount ) ;
         enum = zin.entries() ;
         while ( ( enum.hasMoreElements() ) && ( cont ) ) {

            fileIndex++ ;
            statusWin.setValue ( fileIndex ) ;

            ze = (ZipEntry)enum.nextElement();


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the Zip Entry is null, we are probably done,         */
            /* otherwise we must process the file                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ze != null ) {
               name = ze.getName();

               

               /* Replace the / characters with the true separator character */
               if ( File.separatorChar != '/' )
                  name = name.replace('/', File.separatorChar) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If this is a directory, make sure the directory exists  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ze.isDirectory() ) {

                  /* Chop off the trailing file separator if needed */
                  if ( name.endsWith(File.separator) )
                     name = name.substring(0, name.length() - 1);

                  dir = new File ( outDir+File.separator+name ) ;
                  if ( !dir.exists() )
                     dir.mkdirs() ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Append this directory to our complete path              */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  currDir = outDir + File.separator + name;
               } else {

                  filePath = outDir + File.separator + name;
                  file = new File(filePath);
                  dir  = new File(file.getParent()) ;

                  /* We must do this check again because the directory
                     that this file is in might not have been in the zip file
                     yet */
                  if ( !dir.exists() ) {
                     dir.mkdirs () ;
                  }


                  try {

                     InputStream      in  = zin.getInputStream(ze);
                     FileOutputStream out = new FileOutputStream(filePath);

                     statusWin.setText ( "Extracting "+filePath.substring ( filePath.lastIndexOf ( File.separatorChar ) )+1 ) ;
                     length = 1 ;
                     while ( length >= 1 ) {
                        length = in.read(buffer);
                        if ( length > 0 )
                           out.write(buffer, 0, length);
                     }

                     out.close();
                     in.close() ;
                     file=null ;
                     dir=null ;
                  } 
                  
                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Pick apart the exception to show the user a nice msg    */                                           
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  catch ( Exception e ) {
                     String error = e.getMessage() ;

                     rc = false ;

                     /* If this is the end of zlib, ignore it */
                     if ( error.indexOf ( "end of ZLIB" ) == -1 ) {

                        /* Ok, if they ran out of disk space, we will get a write error. If
                           we get this exception, put up a pretty message to the user */
                        if ( error.indexOf ( "write error" ) != -1 ) {
                           LogSystem.log ( 1, e, false ) ;
                           GUISystem.printBox ( "Info",  "Error unzipping. Out of disk space" ) ;
                           cont=false ;
                        }

                        /* Otherwise, throw the exception */
                     } else {
                        LogSystem.log ( 1, e, false ) ;
                        GUISystem.printBox ( "Info", "Error unzipping file:"+file.getName() ) ;
                        cont=false ;
                     }

                  } /* catch  */
               } /* else if this was a file  */
            } /* if ze  */
            else
               cont = false ;

         } /* while  */

         zin.close();
      } catch ( Exception e ) {
         rc = false ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Error unzipping" ) ;
      }

      statusWin.setText ( "" ) ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* startCopy
    */
   /**
    *    Since most operating systems do not allow you
    *  to copy a file over a file that is currently in use, we
    *  must kick off a batch to copy the files in a separate
    *  process. This is where we kick off that batch
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void startCopy() {
      Properties p         = System.getProperties () ;
      String     splatform = p.getProperty ( "os.name" ) ;
      Runtime    rt        = Runtime.getRuntime() ;
      String[]   cmds      = new String[2] ;                     


      try {

         if ( splatform.indexOf ( "indow" ) != -1 )
            cmds[0] = "up_win32.bat" ;
         else if ( splatform.indexOf ( "OS/2" ) != -1 )
            cmds[0] = "cmd.exe" ;
         else if ( splatform.indexOf ( "AIX" ) != -1 )
            cmds[0] = "up_aix" ;
         else
            cmds[0] = "up_gen" ;

         /* For OS/2 platforms to kick off a process correctly,
            we had to invoke cmd.exe first, passing it a '/C' parameter
            with the uos2.cmd command file and params */
         if ( splatform.indexOf ( "OS/2" ) != -1 )
            cmds[1] = "/C \"up_os2.cmd " ;
         else
            cmds[1] = " " ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Run the batch                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rt.exec ( cmds ).waitFor() ;   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Sleep for 3 seconds, this will give enough time         */
         /* for slower machines to start the batch.                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         Thread.sleep ( 3000 ) ;           

      } catch ( Exception ex ) {
         if ( splatform.indexOf ( "OS/2" ) != -1 )
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), 
                                 "OS/2 is having trouble running the auto update for you. Please open an OS/2 Command Prompt, change to the quest directory and enter 'uos2'" ) ;
         LogSystem.log ( 1, ex, false ) ;
      }
   }
} /* QuestUpdater */
