package com.ibm.nzna.projects.qit.app ;           
/*-----------------------------------------------------------*/
/*   ShutdownThread                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ShutdownThread                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import java.io.* ;
/**
 * This thread will shut the application down. It will close
 * each QuestApplet, write all information to the disk and
 * will then execute the System.exit. Starting this thread
 * can only be stopped by QuestApplets who wish not to shut down
 *
 *
 * @version 1.0, 1/20/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ShutdownThread                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class ShutdownThread extends Object implements Runnable {





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will simply kick off the
    *  Shutdown thread if a TRUE is passed
    *  
    *
    *  @param runThread TRUE = Run thread now, FALSE = do not run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ShutdownThread ( boolean runThread) {
      if ( runThread )
         new Thread ( this ).start() ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will actually run through
    *  the shut down process
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      try {
         File     processFile      = new File ( "quest.pid" ) ;       
         boolean  continueShutdown = false ;

         LogSystem.beginLogSection ( "Shutdown Started" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Close all panels                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log ( 1, "Shutting WindowSystem Down" ) ;
         continueShutdown = WindowSystem.shutdown() ;                                                                                   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the IP Address of the user to nothing               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         UserSystem.setUserAddress ( "000.000.000.000" ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Shutdown the database connections                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( continueShutdown ) {
            LogSystem.log ( 1, "Disconnecting from all databases" ) ;
            continueShutdown = DatabaseSystem.shutdown () ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Block out certain properties                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( continueShutdown ) {
            PropertySystem.putString ( PropertySystem.PROPERTY_PASSWORD, "" ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Mark this as a clean shutdown                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( continueShutdown ) {
            LogSystem.log ( 1, "Writing Property System To Disk" ) ;
            PropertySystem.getInstance().writeToDisk() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Erase our PID process file                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( continueShutdown ) {
            LogSystem.log ( 1, "Deleting PID file" ) ;
            processFile.delete() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   erase the closeSplash file                              */                 
         /*                                                           */
         /*-----------------------------------------------------------*/
         new File ( "closeSplash" ).delete() ;
         


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Exit the application                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( continueShutdown ) {
            LogSystem.log ( 1, "Exit" ) ;
            LogSystem.endLogSection() ;
            System.exit ( 0 ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }


} /* ShutdownThread */
