package com.ibm.nzna.projects.qit.app ;
/*-----------------------------------------------------------*/
/*   StatusRec                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StatusRec                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This object holds information about a single status, date and comment. 
 *
 * This class may be obsolete with QIT 3.0 goes out
 *
 *
 * @version 1.0, 6/27/1998 
 * @author  David Lentz
 */
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.type.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   StatusRec                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class StatusRec extends com.ibm.nzna.shared.pom.PersistentRec implements java.io.Serializable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int                statusInd          = 0 ;
   private int                recNum             = 0 ;
   private String             statusDate         = null ;
   private String             comment            = null ;
   private TypeCountryCodeRec countryRec         = null ;
   private String             dbUser             = null ;
   private String             changedTime        = null ;
   private String             typeStr            = null ;



   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This creates a blank StatusRec
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public StatusRec () {
   }


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a new statusRec with
    *  the data from another one
    *  
    *             
    *  @param statusRec The StatusRecord to copy from
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public StatusRec ( StatusRec statusRec ) {
      setData ( statusRec ) ;
   }


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This creates and initializes a StatusRec to
    *  a status and statusDate
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public StatusRec ( int statusInd, String statusDate ) {
      setStatusInd ( statusInd ) ;
      setDate ( statusDate ) ;
   }

   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This creates and initializes a StatusRec to
    *  a status, a statusDate and a comment
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public StatusRec ( int statusInd, String statusDate, String comment ) {
      setStatusInd ( statusInd ) ;
      setDate ( statusDate ) ;
      setComment ( comment ) ;
   }



   /*-----------------------------------------------------------*/
   /* columnCount
    */
   /**
    *    This mehtod will return the number of columns
    *  that this record supports
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public int columnCount () {
      return( 3 ) ;
   }


   /*-----------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    Returns the specific string for a column in
    *  a details view container
    *  
    *             
    *  @param column The Column to return data for
    *  @param 
    *  @param 
    *
    *  @return Object
    */
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            if ( typeStr == null )
               typeStr = QITTypeList.getInstance().stringFromInd ( statusInd, QITTypeList.TYPE_STATUS ) ;
            retObj = typeStr ;
            break ;

         case 1 :
            retObj = statusDate ;
            break ;

         case 2 :
            retObj = comment ;
            break ;
      }

      return( retObj ) ;
   }


   /*-----------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return the string representation
    *  of this status
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      if ( typeStr == null )
         typeStr = QITTypeList.getInstance().stringFromInd ( statusInd, QITTypeList.TYPE_STATUS ) ;

      return( typeStr ) ;
   }


   /*-----------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will determine if the StatusRec
    *  passed is a match of this statusRec. The comparison
    *  will be done on the status type
    *  
    *             
    *  @param statusRec The status rec to compare to
    *  @param 
    *  @param 
    *
    *  @return TRUE = a match, FALSE = does not match
    */
   public boolean equals ( Object compareRec ) {
      boolean equal = false ;

      if ( compareRec instanceof StatusRec ) {
         StatusRec statusRec = (StatusRec) compareRec ;

         if ( statusRec.statusInd == statusInd )
            equal = true ;

      }

      else if ( compareRec instanceof Integer ) {
         Integer integer = (Integer) compareRec ;

         equal = integer.intValue() == statusInd ;
      }


      return( equal ) ;

   }



   /*-----------------------------------------------------------*/
   /* setData
    */
   /**
    *    This method will set the data in this status record
    *  to the data that is in another
    *  
    *             
    *  @param statusRec The status record to copy from
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setData ( StatusRec statusRec ) {
      if ( statusRec != null ) {
         statusInd          = statusRec.statusInd ;
         statusDate         = statusRec.statusDate ;
         comment            = statusRec.comment ;
         countryRec         = statusRec.countryRec ;
         dbUser             = statusRec.dbUser ;
         changedTime        = statusRec.changedTime ;
         recNum             = statusRec.recNum ;
      }
   }

   public Object clone () {
      return( new StatusRec(this) ) ;
   }

   public TypeStatusRec getTypeStatus () {
      return( (TypeStatusRec) QITTypeList.getInstance().objectFromInd ( statusInd, TypeList.TYPE_STATUS ) ) ;
   }


   /*-----------------------------------------------------------*/
   /* getId
    */
   /**
    *    We have subclassed this to return our
    *  status indicator
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return The status indicator of this status
    */
   public int getId () {
      return( statusInd ) ;
   }




   /*------------------------------------------------------------*/
   /* setStatusInd
    */
   /**
    *    This method will set our status indicator
    *  
    *  
    *
    *  @param ind The status indicator to represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatusInd ( int ind ) {
      this.statusInd = ind ;
      typeStr = null ;
   }




   /*------------------------------------------------------------*/
   /* setDate
    */
   /**
    *    This method will set the status date
    *  
    *  
    *
    *  @param date The Date
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDate ( String date ) {
      this.statusDate = date ;
   }




   /*------------------------------------------------------------*/
   /* setCountry
    */
   /**
    *    This method will set this status' country
    *  
    *  
    *
    *  @param countryRec The TypeCountryCodeRec to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCountry ( TypeCountryCodeRec countryRec ) {
      this.countryRec = countryRec; 
   }




   /*------------------------------------------------------------*/
   /* setComment
    */
   /**
    *    This method will set this status' comment
    *  
    *  
    *
    *  @param comment The status comment
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setComment ( String comment ) {
      this.comment = comment ;
   }




   /*------------------------------------------------------------*/
   /* setRecNum
    */
   /**
    *    This method will set this status' record number
    *  
    *  
    *
    *  @param recNum The new recNum
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRecNum ( int recNum ) {
      this.recNum = recNum ;
   }




   /*------------------------------------------------------------*/
   /* getRecNum
    */
   /**
    *    This method will return the RecNum of
    *  this status record. The RecNum is a place in history
    *  for this status. 1 is most current, 2 the next, 3 ...
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getRecNum () {
      return( recNum ) ;
   }






   /*------------------------------------------------------------*/
   /* getStatusInd
    */
   /**
    *    This method will return this status' status indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getStatusInd () {
      return( statusInd ) ;
   }


   /*------------------------------------------------------------*/
   /* getComment
    */
   /**
    *    This method will return this status record's
    *  comment. Currently the quest database is
    *  not using this for anything
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getComment () {
      return( comment ) ;
   }





   /*------------------------------------------------------------*/
   /* getDate
    */
   /**
    *    This method will return this status' date
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getDate () {
      return( statusDate ) ;
   }





   /*------------------------------------------------------------*/
   /* getCountry
    */
   /**
    *    This method will return this record's 
    *  country
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TypeCountryCodeRec
    *//*---------------------------------------------------------*/
   public TypeCountryCodeRec getCountry () {
      return( countryRec ) ;
   }




   /*------------------------------------------------------------*/
   /* cleanDBChars
    */
   /**
    *    This method will clean all strings in this record
    *  so that they are acceptable for DB/2
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cleanDBChars () {
   }





   /*------------------------------------------------------------*/
   /* valid
    */
   /**
    *    This method will return TRUE if this instance
    *  of a status is valid. To be valid, this status ind must
    *  be above or equal to 0 and have a status date
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public boolean valid () {
      return( ( statusInd >=0 ) && ( statusDate != null ) && ( statusDate.length() > 0 ) ) ;
   }

}
