package com.ibm.nzna.projects.qit.app ;
/*-----------------------------------------------------------*/
/*   Str                                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Str                                 */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM Corp.                        */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.type.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * All Strings used throughout the Quest Application 
 *
 *
 * @version 1.0, 1/10/2000
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   Str                                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class Str extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Hashtable stringHash  = null ;
   private static int       languageInd = 0 ;                              
   private static String    strFile     = "english.lng" ;
   private static String    appName     = "Quest" ;                                                         

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create and initialize
    *  the String environment for the Quest
    *  application
    *
    *  @param languageFile The name of the language file to read in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Str ( String languageFile ) {
      switchLanguage ( languageFile ) ;
   }


   /*-----------------------------------------------------------*/
   /*   switchLanguage                                          */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will switch what language to use         */
   /*                                                           */
   /*   Parameters:                                             */
   /*      language    - The Language to load                   */
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public static void switchLanguage ( String language ) {
      try {

         if ( language != null ) {
            if ( !language.toLowerCase().endsWith ( ".lng" ) )
               loadStrFile ( language.toLowerCase()+".lng" ) ;
            else
               loadStrFile ( language.toLowerCase() ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         switchLanguage ( "english" ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* switchLanguage
    */
   /**
    *    This method will switch what language to
    *  use
    *  
    *
    *  @param languageInd The Language Indicator as defined in TIGRIS.TYPELANGUAGE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void switchLanguage ( int languageInd ) {
      //try {
      //TypeLanguageRec langRec = (TypeLanguageRec) QITTypeList.getInstance().objectFromInd ( languageInd, QITTypeList.TYPE_LANGUAGE ) ;

      switchLanguage ( "english" ) ; //langRec.toString().toLowerCase() ) ;
      PropertySystem.putString ( PropertySystem.PROPERTY_LANGUAGEFILE, "english" ) ; //langRec.toString().toLowerCase() ) ;

      //} catch ( Exception e ) {
      //   LogSystem.log ( 1, e, false ) ;
      //}
   }


   /*-----------------------------------------------------------*/
   /* getApplicationName
    */
   /**
    *    This method will return the current application
    *  name
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Application Name String
    *//*---------------------------------------------------------*/
   public static String getApplicationName () {
      return( appName ) ;
   }


   /*-----------------------------------------------------------*/
   /*   getString                                               */
   /*                                                           */
   /*   Description:                                            */
   /*      Returns a string from the string array               */                                                   
   /*                                                           */
   /*   Parameters:                                             */
   /*      stringInd - The String Indicator                     */                        
   /*                                                           */
   /*   Returns:                                                */
   /*      None                                                 */
   /*-----------------------------------------------------------*/
   public static String getStr ( int stringInd ) {

      try {

         return( (String)stringHash.get ( ""+stringInd ) ) ;

      } catch ( IndexOutOfBoundsException ex ) {
         LogSystem.log ( 1,  "Cannot Load String Index '"+stringInd+"' out of range in strArray. Language Ind:"+PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ) ;
      } catch ( NullPointerException ex1 ) {
         LogSystem.log ( 1, "strArray in strObject is null. String Index:"+stringInd+"  Language Ind:"+PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ) ;      
      }

      return( "" ) ;
   }


   /*-----------------------------------------------------------*/
   /* loadStrFile
    */
   /**
    *    This method will load the string file into our
    *  vector for later use
    *  
    *
    *  @param filename The filename of the string file
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void loadStrFile ( String filename ) {
      try {
         String            inStr   = null ;
         StringTokenizer   strtok  = null ;
         String            tempStr = null ;
         int               index   = 0 ;          

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read the whole file into a String                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         inStr = FileUtil.readFile ( filename ) ;                                 


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, toke the string into separate strings              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         strtok = new StringTokenizer ( inStr, "\n" ) ;

         stringHash = new Hashtable() ;

         /*-------------------------------------------------------------*/
         /*                                                             */
         /*   Grab the Application Name from the first line in the file */
         /*                                                             */
         /*-------------------------------------------------------------*/
         appName = strtok.nextToken() ;
         appName = appName.substring ( 0, appName.length()-1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, walk through the rest of the lines                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( strtok.hasMoreTokens() ) {

            tempStr = strtok.nextToken() ;

            /* Take the end of line char off */
            if ( ( tempStr != null ) && ( tempStr.length() > 6 ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If a line begins with a '#' character, ignore it        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !tempStr.startsWith ( "#" ) ) {
                  int colonIndex = tempStr.indexOf ( ":" ) ;

                  tempStr = tempStr.substring ( 0, tempStr.length()-1 ).trim() ;
                  index = new Integer ( tempStr.substring ( 0, colonIndex ).trim() ).intValue() ;
                  stringHash.put ( ""+index, tempStr.substring ( colonIndex+1 ) ) ;

               } /* if not a comment  */

            } /* if the temp string is good  */

         } /* while more tokens */

         strFile = filename ;

      } catch ( FileNotFoundException e ) {
         if ( !filename.equals ( "english.lng" ) ) {
            java.awt.Frame tempFrame = new java.awt.Frame() ;

            new InfoBoxDlg ( tempFrame, 
                             "Severe Error", 
                             "Cannot find language file "+filename+". Defaulting to english."
                           ).setVisible(true) ;
            switchLanguage ( "english" ) ;
         } else {
            System.out.println ( "Language Error" ) ;
            System.out.println ( "   Could not locate the language file '"+filename+"' please make sure this file resides in the quest directory" ) ;
            System.exit(0) ; // exit now 
         }
      } catch ( Exception e ) {
         java.awt.Frame tempFrame = new java.awt.Frame() ;

         LogSystem.log ( 1, e, false ) ;
         if ( !filename.equals ( "english.lng" ) ) {
            new InfoBoxDlg ( tempFrame, 
                             "Severe Error", 
                             "Cannot find language file "+filename+". Defaulting to english." ).setVisible ( true ) ;
            switchLanguage ( "english" ) ;
         } else {
            new InfoBoxDlg ( tempFrame, 
                             "Severe Error", 
                             "Cannot find language file "+filename+". Defaulting to english." ).setVisible ( true ) ;
            tempFrame.dispose() ;
            System.exit(0) ;
         }

      }

   }

} /* Str */
