package com.ibm.nzna.projects.qit.app ;   
/*-----------------------------------------------------------*/
/*   ThemeSystem                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ThemeSystem                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This system maintains all variables for a Theme. 
 *
 *
 * @version 1.0, 10/22/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ThemeSystem                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ThemeSystem extends Object implements AppConst, PropertyListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Theme Properties                                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Properties themeProperties = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Like all good Subsystems, this just initializes
    *  our vars
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ThemeSystem () {
      reloadThemeProperty() ;

      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;
   }





   /*------------------------------------------------------------*/
   /* reloadThemeProperty
    */
   /**
    *    This method will reload whatever theme
    *  that the user last had selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void reloadThemeProperty () {
      String themePropertyFile = PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME ) ;
      String homeDir           = PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR ) ;

      themeProperties = new Properties() ;

      if ( themePropertyFile != null ) {

         themePropertyFile = homeDir+File.separator+"skins"+File.separator+themePropertyFile+File.separator+"theme.property" ;                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Try and load the property file                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log ( 1, "Trying to load property file:"+themePropertyFile ) ;
         try {
            themeProperties.load ( new FileInputStream ( new File ( themePropertyFile ) ) ) ;
         } catch ( Exception e ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we cannot find it, load the default one              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            LogSystem.log ( 1, "Cannot load theme properties for filename:"+themePropertyFile ) ;
            LogSystem.log ( 1, e, false ) ;
            try {
               themeProperties.load ( new FileInputStream ( new File ( "skins"+File.separator+"Default"+File.separator+"theme.property" ) ) ) ;
            } catch ( Exception e1 ) {
               LogSystem.log ( 1, "Cannot load Default Theme after previous Exception" ) ;
               LogSystem.log ( 1, e1 , false ) ;
            }
         }
      }

   } /* reloadThemeProperty  */





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has been changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object newValue ) {
      if ( property == PropertySystem.PROPERTY_SKINNAME )
         reloadThemeProperty() ;
   }







   /*------------------------------------------------------------*/
   /* getcolor
    */
   /**
    *    This method will return a Color object
    *  based on a themed properties value.
    *  
    *
    *  @param value The value ( TITLEPANEL_BG_COLOR etc..)
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static final Color getColor ( String value ) {
      String rgbString = themeProperties.getProperty ( value, "0,0,0" ) ;

      return(getColorFromRGBString ( rgbString )) ;
   }


   /*------------------------------------------------------------*/
   /* getColorFromRGBString
    */
   /**
    *    This method wil return a Color object
    *  from a string containing r,g,b values separated
    *  by commas.
    *
    *  @param rgbString The RGB String
    *  @param None
    *  @param None
    *
    *  @return Color
    *//*---------------------------------------------------------*/
   private static Color getColorFromRGBString ( String rgbString ) {
      Color retColor = Color.black ;

      try {
         StringTokenizer strtok = new StringTokenizer ( rgbString, "," ) ;
         int             r      = 0 ;
         int             g      = 0 ;
         int             b      = 0 ;
         String          cur    = null ;

         cur = strtok.nextToken() ;
         if ( cur != null )
            r = new Integer ( cur ).intValue() ;
         cur = strtok.nextToken() ;
         if ( cur != null )
            g = new Integer ( cur ).intValue() ;
         cur = strtok.nextToken() ;
         if ( cur != null )
            b = new Integer ( cur ).intValue() ;

         if ( ( r == -1 ) || ( g == -1 ) || ( b == -1 ) )
            retColor = null ;
         else
            retColor = new Color ( r, g, b ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, "Cannot parse RGB Value from :"+rgbString ) ;
         LogSystem.log ( 1, e, false ) ;
      }

      return( retColor ) ;
   }




   /*------------------------------------------------------------*/
   /* getThemeProperty
    */
   /**
    *    This method will return a raw property from
    *  the theme.property file
    *  
    *
    *  @param name The name of the property
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static final Object getThemeProperty ( String name ) {
      return( themeProperties.getProperty ( name ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getBoolean
    */
   /**
    *    This method will return the boolean value
    *  from the theme.property file of the name passed in
    *  
    *
    *  @param name The name of the property to get the boolean value for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static final boolean getBoolean ( String name ) {
      boolean rc = false ;
      String  property = themeProperties.getProperty ( name ) ;

      if ( property.equalsIgnoreCase ( "TRUE" ) )
         rc = true ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* texturePanel
    */
   /**
    *    This method will texturea panel with the
    *  left,right,top,bottom and center images passed
    *  in. 
    *
    *  @param g The graphics context to paint into
    *  @param left The image to tile the left of the panel
    *  @param right The image to tile the right of the panel
    *  @param top The image to tile the top of the panel
    *  @param bottom The image to tile the bottom of the panel
    *  @param center The image to tile throughout the center of the panel
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void texturePanel ( Graphics g, Image left, Image right, Image top, Image bottom, Image center ) {
   }


} /* ThemeSystem */
