package com.ibm.nzna.projects.qit.app ; 
/*-----------------------------------------------------------*/
/*   UserRec                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: UserRec                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
/**
 * The UserRec holds all basic information about a user. This does not 
 * include authorities or language information or any other 'logged in'
 * information. For that information, the LoggedUserRec can be used
 *
 * If this object is displayed in a list, the following columns will be shown:
 * User Id
 * Name
 * eMail
 * Phone Number
 * Manager
 * Manager eMail
 *
 * @version 1.0, 2/28/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   UserRec                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class UserRec extends PersistentRec implements java.io.Serializable, MultiListSort, MultiListRow {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /**
     *  Returned if the login method passed ok
     **/
   public static final int LOGIN_OK                = 0 ;

   /**
     *  Returned if the user id wasn't found when logging in
     **/
   public static final int LOGIN_USER_NOT_FOUND    = 1 ;

   /**
     *  Returned when the password is wrong when logging in
     **/
   public static final int LOGIN_PASSWORD_WRONG    = 2 ;

   /**
     *  Returned when there has been too many attempts to
     * log in that have failed.
     **/
   public static final int LOGIN_TOO_MANY_ATTEMPTS = 3 ;

   /**
     *  Returned if the login method could not log in due
     * to an SQL error. The LogSystem will have reported
     * the exact error in this case.
     **/
   public static final int LOGIN_SQL_ERROR = 4 ; 


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final char[][]   encrypt         = { { '@','#','$','%','^','&',
         '*','(',')','-','=','[',']',';',
         '1',',','.','/','q','{','}',':','"',
         '<','>','?','`','~','D','e','g','s',
         'u','f','d','!'},
      { 'B','Z','D','C','F','E','H',
         'G','J','I','L','@','N','O','%','#','P','#','R',
         '(','T','W','V','Y','X',
         'A','2','1','4','3','6','5',
         '8','7','0','9'},
      { 'D','C','B','A','H','G','F','E','L',
         'K','J','I','$','[','N','M','@','S','*','Q',')','}','V',
         'U','{','Y','4','/','9',
         '.','8',',','6','5','3',
         '2'}} ;
   private static final char[] decrypt = { 'A',
      'B','C','D',
      'E','F','G','H','I','J','K','L','M','N',
      'O','P','Q','R','S',
      'T','U','V','W','X','Y','Z',
      '1','2','3','4',
      '5','6','7','8','9','0'} ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String   userId       = null ;
   private String   name         = null ;
   private String   ipAddress    = null ;
   private String   manager      = null ;
   private String   eMail        = null ;
   private String   managerEMail = null ;
   private String   phone        = null ;
   private String   password     = null ;
   private String   toString     = null ;
   private String   backUp       = null ;
   private boolean  isManager    = false ;
   private int      languageInd  = 0 ;
   private int      attempts     = 0 ;
   private int[]    teams        = null ;
   private int[]    auths        = null ;
   private int[]    countries    = null ;
   private Vector   brandMgrVec  = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor for this
    *  object
    *  
    *
    *  @param userId The User Id of the user
    *  @param name The name of the user
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public UserRec ( String userId, String name ) {
      setUserId ( userId ) ;
      setName ( name ) ;
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize the record
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      updateRecStatus ( REC_NOCHANGE ) ;
      auths = new int[1] ;
      toString = userId ;
   }




   /*------------------------------------------------------------*/
   /* setUserId
    */
   /**
    *    This method will set the user id that this
    *  object represents
    *  
    *
    *  @param userId The User Id 
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setUserId ( String userId ) {
      this.userId = userId ;
   }




   /*------------------------------------------------------------*/
   /* getUserId
    */
   /**
    *    This method will return the user id that
    *  is represented by this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getUserId () {
      return( userId ) ;
   }





   /*------------------------------------------------------------*/
   /* setName
    */
   /**
    *    This method will set the name of the
    *  user that this object represents
    *  
    *
    *  @param name The name of the user id that this represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setName ( String name ) {
      this.name = name ;
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name of the user
    *  that this user record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( name ) ;
   }


   /*------------------------------------------------------------*/
   /* setManager
    */
   /**
    *    This method will set the manager of the
    *  user that this object represents
    *  
    *
    *  @param manager The manager's userid of the user id that this represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setManagerUserId ( String manager ) {
      this.manager = manager ;
   }





   /*------------------------------------------------------------*/
   /* getManager
    */
   /**
    *    This method will return the manager of the user
    *  that this user record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getManagerUserId () {
      return( manager ) ;
   }

   /*------------------------------------------------------------*/
   /* setEMail
    */
   /**
    *    This method will set the eMail of the
    *  user that this object represents
    *  
    *
    *  @param eMail The eMail of the user id that this represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEMail ( String eMail ) {
      this.eMail = eMail ;
   }





   /*------------------------------------------------------------*/
   /* getEMail
    */
   /**
    *    This method will return the eMail of the user
    *  that this user record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getEMail () {
      return( eMail ) ;
   }

   /*------------------------------------------------------------*/
   /* setManagerEMail
    */
   /**
    *    This method will set the managerEMail of the
    *  user that this object represents
    *  
    *
    *  @param managerEMail The managerEMail of the user id that this represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setManagerEMail ( String managerEMail ) {
      this.managerEMail = managerEMail ;
   }





   /*------------------------------------------------------------*/
   /* getManagerEMail
    */
   /**
    *    This method will return the managerEMail of the user
    *  that this user record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getManagerEMail () {
      return( managerEMail ) ;
   }

   /*------------------------------------------------------------*/
   /* setIPAddress
    */
   /**
    *    This method will set the ipAddress of the
    *  user that this object represents
    *  
    *
    *  @param ipAddress The ipAddress of the user id that this represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIPAddress ( String ipAddress ) {
      this.ipAddress = ipAddress ;
   }





   /*------------------------------------------------------------*/
   /* getIPAddress
    */
   /**
    *    This method will return the ipAddress of the user
    *  that this user record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getIPAddress () {
      return( ipAddress ) ;
   }


   /*------------------------------------------------------------*/
   /* setLanguage
    */
   /**
    *    This method will set the language of the
    *  user that this object represents
    *  
    *
    *  @param language The language of the user id that this represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLanguage ( int languageInd ) {
      this.languageInd = languageInd ;
   }





   /*------------------------------------------------------------*/
   /* getLanguage
    */
   /**
    *    This method will return the language of the user
    *  that this user record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public int getLanguage () {
      return( languageInd ) ;
   }



   /*------------------------------------------------------------*/
   /* setPhone
    */
   /**
    *    This method will set the phone of the
    *  user that this object represents
    *  
    *
    *  @param phone The phone of the user id that this represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPhone ( String phone ) {
      this.phone = phone ;
   }





   /*------------------------------------------------------------*/
   /* getPhone
    */
   /**
    *    This method will return the phone of the user
    *  that this user record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getPhone () {
      return( phone ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return what to show for
    *  a certain column
    *  
    *
    *  @param column The column to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = userId ;
            break ;

         case 1 :
            retObj = name ;
            break ;

         case 2 :
            retObj = eMail ;
            break ;

         case 3 :
            retObj = phone ;
            break ;

         case 4 :
            retObj = manager ;
            break ;

         case 5 :
            retObj = managerEMail ;
            break ;
      }

      if ( retObj == null )
         retObj = "" ;

      return( retObj ) ;
   }



   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will compare this object with another in a list
    *  
    *
    *  @param object The object to sort with
    *  @param column The column to order by
    *
    *//*---------------------------------------------------------*/
   public int compareTo ( MultiListSort object, int column ) {
      int compare = 0 ;

      if ( object instanceof UserRec ) {
         UserRec compareRec = (UserRec) object ;


         switch ( column ) {
            case 0 :
               if ( ( userId != null ) && ( compareRec.userId != null ) )
                  compare = userId.compareTo ( compareRec.userId ) ;
               break ;

            case 1 :
               if ( ( name != null ) && ( compareRec.name != null ) )
                  compare = name.compareTo ( compareRec.name ) ;
               break ;

            case 2 :
               if ( ( eMail != null ) && ( compareRec.eMail != null ) )
                  compare = eMail.compareTo ( compareRec.eMail ) ;
               break ;

            case 3 :
               if ( ( phone != null ) && ( compareRec.phone != null ) )
                  compare = phone.compareTo ( compareRec.phone ) ;
               break ;

            case 4 :
               if ( ( manager != null ) && ( compareRec.manager != null ) )
                  compare = manager.compareTo ( compareRec.manager ) ;
               break ;

            case 5 :
               if ( ( managerEMail != null ) && ( compareRec.managerEMail != null ) )
                  compare = managerEMail.compareTo ( compareRec.managerEMail ) ;
               break ;
         }
      }

      return( compare ) ;
   }





   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will simply return the User's User Id
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String toString () {
      return( toString ) ;
   }





   /*------------------------------------------------------------*/
   /* addTeam
    */
   /**
    *    This method will add this user to a team.
    *  
    *  
    *
    *  @param teamInd The USERGROUP in TIGRIS.USERGROUPS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addTeam ( int teamInd ) {
      if ( teams != null ) {
         if ( !hasTeam ( teamInd ) ) {
            int[] copyArray = new int[teams.length+1] ;

            System.arraycopy ( teams, 0, copyArray, 0, teams.length ) ;
            copyArray[copyArray.length-1]=teamInd ;
            teams = copyArray ;
         }
      } else {
         teams = new int[1] ;
         teams[0] = teamInd ;
      }

   }





   /*------------------------------------------------------------*/
   /* removeTeam
    */
   /**
    *    This method will remove a team from this
    *  user's record
    *  
    *
    *  @param teamInd The USERGROUP that is in TIGRIS.USERGROUP
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeTeam ( int teamInd ) {
      boolean found = false ;
      int     i     = 0 ;

      if ( teams != null ) {
         Vector tempVec = new Vector ( teams.length ) ;

         while ( i < teams.length ) {
            if ( teams[i] != teamInd )
               tempVec.addElement ( new Integer ( teams[i] ) ) ;
            i++ ;
         }

         i=0 ;
         teams = new int[tempVec.size()] ;
         while ( i < tempVec.size() ) {
            teams[i] = ((Integer)tempVec.elementAt(i)).intValue() ;
            i++ ;
         }

      }
   }





   /*------------------------------------------------------------*/
   /* hasAuth
    */
   /**
    *    This method will determine if this user has
    *  the authority passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean hasAuth ( int authInd ) {
      int     i       = 0 ;
      boolean hasAuth = false ;

      while ( ( !hasAuth) && ( i < auths.length ) ) {
         hasAuth = ( auths[i] == authInd ) ;
         if ( !hasAuth )
            hasAuth = ( auths[i] == Authority.AUTH_SUPERUSER ) ;
         i++ ;
      }

      return( hasAuth ) ;
   }


   /*------------------------------------------------------------*/
   /* addAuth
    */
   /**
    *    This method will add this user to a auth.
    *  
    *  
    *
    *  @param authInd The USERGROUP in TIGRIS.USERGROUPS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addAuth ( int authInd ) {
      if ( auths != null ) {
         int[] copyArray = new int[auths.length+1] ;

         System.arraycopy ( auths, 0, copyArray, 0, auths.length ) ;
         copyArray[copyArray.length-1]=authInd ;
         auths = copyArray ;
      } else {
         auths = new int[1] ;
         auths[0] = authInd ;
      }

   }





   /*------------------------------------------------------------*/
   /* removeAuth
    */
   /**
    *    This method will remove a auth from this
    *  user's record
    *  
    *
    *  @param authInd The USERGROUP that is in TIGRIS.USERGROUP
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAuth ( int authInd ) {
      boolean found = false ;
      int     i     = -1 ;

      while ( ( !found ) && ( i < auths.length ) ) {
         i++ ;
         if ( i < auths.length )
            found = ( auths[i] == authInd ) ;
      }

      if ( found ) {
         int[] copyArray = new int[auths.length-1] ;
         int  k          = 0 ;

         while ( i < auths.length ) {
            if ( auths[i] != authInd ) {
               copyArray[k]=auths[i] ;
               k++ ;
            }
            i++ ;
         }
         auths = copyArray ;
      }
   }





   /*------------------------------------------------------------*/
   /* getTeams
    */
   /**
    *    This method will return an array of integers. Each
    *  integer is the team ind, which is in the TIGRIS.USERGROUP
    *  table
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int[] getTeams () {
      return( teams ) ;
   }



   /*------------------------------------------------------------*/
   /* getAuths
    */
   /**
    *    This method will return an array of integers. Each
    *  integer is the authInd, which is in the TIGRIS.TYPEAUTH
    *  table
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int[] getAuths () {
      return( auths ) ;
   }




   /*------------------------------------------------------------*/
   /* setAuths
    */
   /**
    *    This method set the Authorities of this record
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAuths ( int[] auths ) {
      this.auths = auths ;
   }

   /*------------------------------------------------------------*/
   /* setTeams
    */
   /**
    *    This method set the Teams of this record
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTeams ( int[] teams ) {
      this.teams = teams ;
   }





   /*---------------------------------------------------------------*/
   /*   getDecryptChar()                                            */
   /*                                                               */
   /*   Description:                                                */
   /*      This method will take a encrypt character and decrypt it */
   /*                                                               */
   /*   Parameters:                                                 */
   /*      c - Character to be decrypted                            */
   /*                                                               */
   /*   Returns:                                                    */
   /*      decrypted character                                      */
   /*---------------------------------------------------------------*/
   protected static char getDecryptChar ( char c, int pos ) {
      char retc = 0 ;
      int  i    = 0 ;

      while ( ( i < encrypt.length ) && ( retc == 0 ) ) {
         if ( encrypt[pos][i] == c )
            retc = decrypt[i] ;
         i++ ;
      }

      return( retc ) ;
   }

   /*---------------------------------------------------------------*/
   /*   getEncryptChar()                                            */
   /*                                                               */
   /*   Description:                                                */
   /*      This method will take a normal character, and encrypt it */
   /*                                                               */
   /*   Parameters:                                                 */
   /*      c - Character to be encrypted                            */
   /*                                                               */
   /*   Returns:                                                    */
   /*      encrypted character                                      */
   /*---------------------------------------------------------------*/
   protected static char getEncryptChar ( char c, int pos ) {
      char retc = 0 ;
      int  i    = 0 ;

      while ( ( i < decrypt.length ) && ( retc == 0 ) ) {
         if ( decrypt[i] == c )
            retc = encrypt[pos][i] ;
         i++ ;
      }

      return( retc ) ;
   }

   /*-----------------------------------------------------------*/
   /*   encryptPassword                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will encrypt the users password          */
   /*                                                           */
   /*   Parameters:                                             */
   /*      password - String representation of the password     */
   /*                                                           */
   /*   Returns:                                                */
   /*      The encrypted password                               */
   /*-----------------------------------------------------------*/
   public String encryptPassword ( ) {
      int          i      = 0 ;
      StringBuffer buf    = new StringBuffer ( password ) ;
      StringBuffer retBuf = new StringBuffer ( password ) ;
      int          pos    = 0 ;

      /* Ok, go through the password, and substitute the letters */
      for ( i = 0; i < password.length(); i++ ) {
         retBuf.setCharAt ( i, getEncryptChar ( password.charAt(i), pos ) ) ;
         pos++ ;
         if ( pos > 2 )
            pos = 0 ;
      }



      return( retBuf.toString () ) ;

   } /* encryptPassword */

   /*-----------------------------------------------------------*/
   /*   decryptPassword                                         */
   /*                                                           */
   /*   Description:                                            */
   /*      This method will encrypt the users password          */
   /*                                                           */
   /*   Parameters:                                             */
   /*      password - String representation of the password     */
   /*                                                           */
   /*   Returns:                                                */
   /*      The encrypted password                               */
   /*-----------------------------------------------------------*/
   public String decryptPassword ( String password ) {
      int          i      = 0 ;
      int          pos    = 0 ;
      StringBuffer buf    = new StringBuffer ( password ) ;
      StringBuffer retBuf = new StringBuffer ( password ) ;

      /* Ok, go through the password, and substitute the letters */
      for ( i = 0; i < password.length(); i++ ) {
         retBuf.setCharAt ( i, getDecryptChar ( password.charAt(i), pos ) ) ;
         pos++ ;
         if ( pos > 2 )
            pos = 0 ;
      }

      return( retBuf.toString () ) ;

   } /* decryptPassword */





   /*------------------------------------------------------------*/
   /* getEncyptedPassword
    */
   /**
    *    This method will return the password in it's
    *  encrypted form
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getEncryptedPassword () {
      return( encryptPassword() ) ;
   }


   /*------------------------------------------------------------*/
   /* setPassword
    */
   /**
    *    This method will set the password of this
    *  record
    *  
    *
    *  @param password The new password
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPassword ( String password ) {
      this.password = password ;
   }




   /*------------------------------------------------------------*/
   /* getPassword
    */
   /**
    *    This method will return the password that
    *  has been assigned to this user
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getPassword ( ) {
      return( password ) ;
   }


   /*------------------------------------------------------------*/
   /* saveToDatabase                                             */
   /**
    *    This method will save this record to the database.
    *  The operation of save depends on what the
    *  recStatus is. This method will NOT DELETE itself. The reason
    *  it will not delete itself is because there are many foreign keys
    *  to the TIGRIS.USERID table that would prevent the DELETE.<br><br>
    *
    *  This method will also not UPDATE the password. It will however INSERT
    *  a password, see 'changePassword' method to update the password.
    *
    *  This method takes advantage of the SqlRunner stored procedure. This method
    *  will simply form all of the SQL Statements to save the user, and then passes
    *  all SQL Statements to the SqlRunner.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if saved, FALSE if not saved
    *//*---------------------------------------------------------*/
   public boolean saveToDatabase () {
      boolean   rc      = false ;                                   
      String    sqlStmt = null ;

      try {
         String isManagerYN = isManager ? "Y" : "N" ;     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Clean our data                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         //cleanSQL () ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, save the TIGRIS.USERID information first            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( getRecStatus() ) {
            case REC_INSERT :
               sqlStmt = "INSERT INTO TIGRIS.USERID "+
                         "( USERID,       NAME,      ATTEMPTS, PASSWORD, MANAGER, MANAGERUSERID, "+
                         "  BACKUPUSERID, EMAILADDR, PHONENUM, DBUSER, CHANGEDTIME ) "+
                         "values ( "+
                         "'"+userId+"', "+
                         "'"+name+"', "+
                         " "+attempts+", "+
                         "'"+Text.cleanDBString ( getEncryptedPassword() )+"', "+
                         "'"+isManagerYN+"', "+
                         "'"+manager+"', "+
                         "'"+backUp+"', "+
                         "'"+eMail+"', "+
                         "'"+phone+"', "+
                         "'"+UserSystem.getUserId()+"', "+
                         "CURRENT TIMESTAMP )"+
                         QITSqlRunner.END_DELIM ;
               break ;
            case REC_UPDATE :
               sqlStmt = "UPDATE TIGRIS.USERID "+
                         "SET NAME           = '"+name+"', "+
                         "    ATTEMPTS       = "+attempts+", "+
                         "    MANAGER        = '"+isManagerYN+"', "+
                         "    MANAGERUSERID  = '"+manager+"', "+
                         "    BACKUPUSERID   = '"+backUp+"', "+
                         "    PHONENUM       = '"+phone+"', "+
                         "    EMAILADDR      = '"+eMail+"', "+
                         "    DBUSER         = '"+UserSystem.getUserId()+"', "+
                         "    CHANGEDTIME    = CURRENT TIMESTAMP "+
                         "WHERE USERID = '"+userId+"'"+
                         QITSqlRunner.END_DELIM ;
               break ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, if we have a valid start, finish it                 */                              
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( sqlStmt != null ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, delete everything from TIGRIS.USERIDUSERGROUP      */
            /* for this user, then, reinsert all of the values           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sqlStmt+="DELETE FROM TIGRIS.USERIDUSERGROUP WHERE USERID = '"+userId+"' "+QITSqlRunner.END_DELIM ;

            if ( teams != null ) {
               for ( int i = 0 ; i < teams.length; i++ )
                  sqlStmt+="INSERT INTO TIGRIS.USERIDUSERGROUP "+
                           "( USERGROUP, USERID, DBUSER, CHANGEDTIME )"+
                           "VALUES ( "+
                           ""+teams[i]+", "+
                           "'"+userId+"', "+
                           "'"+UserSystem.getUserId()+"', "+
                           "CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ;        
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, delete everything from TIGRIS.USERIDCOUNTRY for    */
            /* this user and then reinsert all of the values             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sqlStmt+="DELETE FROM TIGRIS.USERIDCOUNTRY WHERE USERID = '"+userId+"' "+QITSqlRunner.END_DELIM ;

            if ( countries != null ) {
               for ( int i = 0 ; i < countries.length; i++ )
                  sqlStmt+="INSERT INTO TIGRIS.USERIDCOUNTRY "+
                           "( USERID, COUNTRYCODEIND, DBUSER, CHANGEDTIME )"+
                           "VALUES ( "+
                           "'"+userId+"', "+
                           ""+countries[i]+", "+
                           "'"+UserSystem.getUserId()+"', "+
                           "CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ;        
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Save the Language                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            switch ( getRecStatus() ) {
               case REC_INSERT :
                  sqlStmt+="INSERT INTO TIGRIS.USERLANGUAGE "+
                           "(USERID, LANGUAGEIND, DBUSER, CHANGEDTIME ) "+
                           "VALUES ( "+
                           "'"+userId+"', "+
                           ""+languageInd+", "+
                           "'"+UserSystem.getUserId()+"', "+
                           "CURRENT TIMESTAMP )"+
                           QITSqlRunner.END_DELIM ;
                  break ;

               case REC_UPDATE :
                  sqlStmt+="UPDATE TIGRIS.USERLANGUAGE "+
                           "SET LANGUAGEIND = "+languageInd+", "+
                           "    DBUSER      = '"+UserSystem.getUserId()+"', "+
                           "    CHANGEDTIME = CURRENT TIMESTAMP "+
                           "WHERE USERID = '"+userId+"'"+
                           QITSqlRunner.END_DELIM ;
                  break;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Save the Brand Mgr Values                               */                
            /*                                                           */
            /*-----------------------------------------------------------*/
            sqlStmt+="DELETE FROM TIGRIS.USERIDBRAND WHERE USERID = '"+userId+"' "+QITSqlRunner.END_DELIM ;

            if ( ( brandMgrVec != null ) && ( brandMgrVec.size() > 0 ) ) {
               int i    = 0 ;
               int size = brandMgrVec.size() ;

               while ( i < size ) {
                  sqlStmt+="INSERT INTO TIGRIS.USERIDBRAND "+
                           "( USERID, BRANDIND, TYPEBRANDMGR, DBUSER, CHANGEDTIME )"+
                           "VALUES ( "+
                           "'"+userId+"', "+
                           ""+((BrandMgrRec)brandMgrVec.elementAt(i)).getBrandInd()+", "+
                           "'"+((BrandMgrRec)brandMgrVec.elementAt(i)).getBrandMgrType()+"', "+
                           "'"+UserSystem.getUserId()+"', "+
                           "CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ;        
                  i++ ;
               }
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, send the SQL to the sqlRunner                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rc = QITSqlRunner.sqlRunner ( sqlStmt ) ;

            if ( rc ) {
               if ( getRecStatus() == REC_INSERT )
                  UserSystem.addUserRec ( this ) ;
               updateRecStatus ( REC_NOCHANGE ) ;
            }

         } /* if sqlStmt  */



      } catch ( Exception e ) {
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            LogSystem.logExceptionToFile ( e, "c:\\UserRec.err" ) ;
      }


      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will return TRUE if this record
    *  equals the record passed in. If the record passed
    *  in is a string, then the User Id will be compared to the string
    *
    *  @param Object the object to compare to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {
      boolean equals = false ;

      if ( object != null ) {
         if ( object instanceof UserRec ) {
            UserRec compareRec = (UserRec) object ;

            try {
               equals = ( ( userId.equals ( compareRec.userId ) ) &&
                          ( name.equals ( compareRec.name ) ) &&
                          ( manager.equals ( compareRec.manager ) ) &&
                          ( eMail.equals ( compareRec.eMail ) ) &&
                          ( managerEMail.equals ( compareRec.managerEMail ) ) &&
                          ( phone.equals ( compareRec.phone ) ) &&
                          ( languageInd == compareRec.languageInd ) ) ;
            } catch ( Exception e ) {
               equals = false ;
            }
         }

         else if ( object instanceof String ) {
            String compareRec = (String) object ;

            if ( userId != null ) {
               equals = compareRec.equals ( userId ) ;
            }
         }
      }

      return( equals ) ;
   }





   /*------------------------------------------------------------*/
   /* fuzzyEquals
    */
   /**
    *    This method will perform a 'Fuzzy' match on
    *  the string passed in. The following fields will
    *  be searched in a fuzzy search:
    *
    *  <pre><br>
    *     user id
    *     name
    *     manager
    *     eMail  
    *     managerEMail
    *     phone      
    *   </pre> 
    *
    *  @param fuzzyStr The string to compare with
    *  @param None
    *  @param None
    *
    *  @return TRUE if matches, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean fuzzyEquals ( String fuzzyStr ) {
      boolean equals = false ;

      try {
         fuzzyStr = fuzzyStr.toUpperCase() ;

         if ( userId != null )
            equals = ( userId.toUpperCase().indexOf ( fuzzyStr ) >= 0 ) ;

         if ( ( name != null ) && ( !equals ) )
            equals = ( name.toUpperCase().indexOf ( fuzzyStr ) >= 0 ) ;

         if ( ( manager != null ) && ( !equals ) )
            equals = ( manager.toUpperCase().indexOf ( fuzzyStr ) >= 0 ) ;

         if ( ( eMail != null ) && ( !equals ) )
            equals = ( eMail.toUpperCase().indexOf ( fuzzyStr ) >= 0 ) ;

         if ( ( managerEMail != null ) && ( !equals ) )
            equals = ( managerEMail.toUpperCase().indexOf ( fuzzyStr ) >= 0 ) ;

         if ( ( phone != null ) && ( !equals ) )
            equals = ( phone.toUpperCase().indexOf ( fuzzyStr ) >= 0 )  ;

      } catch ( Exception e ) {
      }
      return( equals ) ;
   }





   /*------------------------------------------------------------*/
   /* setToString
    */
   /**
    *    This method will set what the 'toString' method
    *  will return. By Default it returns the User Id
    *  
    *
    *  @param toString The string to set to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setToString ( String toString ) {
      this.toString = toString ;
   }



   /*------------------------------------------------------------*/
   /* isManager
    */
   /**
    *    This method will return TRUE if this
    *  person is a manager
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public boolean isManager () {
      return( isManager ) ;
   }



   /*------------------------------------------------------------*/
   /* getBackUp
    */
   /**
    *    This method will return this person's
    *  back up user id
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getBackUp( ) {
      return( backUp ) ;
   }





   /*------------------------------------------------------------*/
   /* setBackUp
    */
   /**
    *    This method will set this user's backUp
    *  user id
    *  
    *
    *  @param userId The user id representing this person's back up
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBackUp ( String userId ) {
      this.backUp = userId ;
   }



   /*------------------------------------------------------------*/
   /* addCountry
    */
   /**
    *    This method will add this user to a country.
    *  
    *  
    *
    *  @param countryInd The COUNTRYCODEIND in TIGRIS.TYPECOUNTRYCODE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCountry ( int countryInd ) {
      if ( ( countries != null ) && ( !hasCountry ( countryInd ) ) ) {
         int[] copyArray = new int[countries.length+1] ;

         System.arraycopy ( countries, 0, copyArray, 0, countries.length ) ;
         copyArray[copyArray.length-1]=countryInd ;
         countries = copyArray ;
      } else {
         countries = new int[1] ;

         countries[0] = countryInd ;
      }

   }





   /*------------------------------------------------------------*/
   /* removeCountry
    */
   /**
    *    This method will remove a team from this
    *  user's record
    *  
    *
    *  @param countryCodeInd The COUNTRYCODEIND defined in TIGRIS.TYPECOUNTRYCODE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCountry ( int countryCodeInd ) {
      int     i     = 0 ;

      if ( countries != null ) {
         Vector tempVec = new Vector ( countries.length ) ;

         while ( i < countries.length ) {
            if ( countries[i] != countryCodeInd )
               tempVec.addElement ( new Integer ( countries[i] ) ) ;
            i++ ;
         }

         i=0 ;
         countries = new int[tempVec.size()] ;
         while ( i < tempVec.size() ) {
            countries[i] = ((Integer)tempVec.elementAt(i)).intValue() ;
            i++ ;
         }

      }
   }





   /*------------------------------------------------------------*/
   /* getCountries
    */
   /**
    *    This method will return the list of countries
    *  that this user is associated with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int array. Each int in the array is a CountryCodeInd as defined in TIGRIS.TYPECOUNTRYCODE
    *//*---------------------------------------------------------*/
   public int[] getCountries () {
      return( countries ) ;
   }





   /*------------------------------------------------------------*/
   /* setCountries
    */
   /**
    *    This method will set the entire list of countries
    *  that this user has associated to them
    *  
    *
    *  @param countries An array of countrycodeinds defined in TIGRIS.TYPECOUNTRYCODE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCountries ( int[] countries ) {
      this.countries = countries ;
   }



   /*------------------------------------------------------------*/
   /* setIsManager
    */
   /**
    *    This method will set the 'isManager' property
    *  
    *  
    *
    *  @param isManager TRUE = This user is a manager
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIsManager ( boolean isManager ) {
      this.isManager = isManager ;
   }





   /*------------------------------------------------------------*/
   /* addBrandMgrRec
    */
   /**
    *    This method will add a Brand Manager
    *  record to this user
    *  
    *
    *  @param brandMgrRec The BrandMgrRec to add
    *  @param None
    *  @param None
    *
    *  @return The number of BrandMgrRecs on this user
    *//*---------------------------------------------------------*/
   public int addBrandMgrRec ( BrandMgrRec brandMgrRec ) {
      if ( brandMgrVec == null )
         brandMgrVec = new Vector ( 1, 1 ) ;

      if ( !brandMgrVec.contains ( brandMgrRec ) )
         brandMgrVec.addElement ( brandMgrRec ) ;

      return( brandMgrVec.size() ) ;
   }





   /*------------------------------------------------------------*/
   /* removeBrandMgrRec
    */
   /**
    *    This method will remove a brand manager
    *  record from this user.
    *  
    *
    *  @param brandMgrRec The BrandMgrRec to remove
    *  @param None
    *  @param None
    *
    *  @return The number of BrandMgrRecs on this user
    *//*---------------------------------------------------------*/
   public int removeBrandMgrRec ( BrandMgrRec brandMgrRec ) {
      int rc = 0 ;

      if ( brandMgrVec != null ) {
         brandMgrVec.removeElement ( brandMgrRec ) ;
         rc = brandMgrVec.size() ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getBrandMgrVec
    */
   /**
    *    This method will return a Vector of
    *  BrandMgrRecs.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getBrandMgrVec () {
      return( brandMgrVec ) ;
   }





   /*------------------------------------------------------------*/
   /* setBrandMgrVec
    */
   /**
    *    This method will set the entire list of BrandMgrRecs
    *  that this user has.
    *  
    *
    *  @param brandMgrVec A Vector of BrandMgrRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrandMgrVec ( Vector brandMgrVec ) {
      this.brandMgrVec = brandMgrVec ;
   }





   /*------------------------------------------------------------*/
   /* cleanSQL
    */
   /**
    *    This method will clean all of our values
    *  and will prepare them to be saved to the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void cleanSQL () {
      Text.cleanDBString ( userId ) ;
      Text.cleanDBString ( name ) ;
      Text.cleanDBString ( ipAddress ) ;
      Text.cleanDBString ( manager ) ;
      Text.cleanDBString ( eMail ) ;
      Text.cleanDBString ( managerEMail ) ;
      Text.cleanDBString ( phone ) ;
      Text.cleanDBString ( password ) ;
      Text.cleanDBString ( toString ) ;
      Text.cleanDBString ( backUp ) ;
   }




   /*------------------------------------------------------------*/
   /* hasCountry
    */
   /**
    *    This method will return TRUE if the
    *  user is associated to the country passed
    *  
    *
    *  @param countryInd The country to look for
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean hasCountry ( int countryInd ) {
      boolean rc = false ;

      if ( countries != null ) {
         int i = 0 ;

         while ( ( i < countries.length ) && ( !rc ) ) {
            rc = (countries[i] == countryInd) ;
            i++ ;
         } /* while  */
      } /* if  */

      return( rc ) ;
   } /* hasCountry  */




   /*------------------------------------------------------------*/
   /* hasTeam
    */
   /**
    *    This method will return TRUE if the
    *  user is associated to the team passed
    *  
    *
    *  @param teamInd The team to look for
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean hasTeam ( int teamInd ) {
      boolean rc = false ;

      if ( teams != null ) {
         int i = 0 ;

         while ( ( i < teams.length ) && ( !rc ) ) {
            rc = (teams[i] == teamInd) ;
            i++ ;
         } /* while  */
      } /* if  */

      return( rc ) ;
   } /* hasTeam  */





   /*------------------------------------------------------------*/
   /* clone
    */
   /**
    *    This method will clone this user record
    *  and will return a new UserRec
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public Object clone () {
      UserRec userRec = new UserRec ( userId, name ) ;


      userRec.ipAddress    = ipAddress ;
      userRec.manager      = manager ;
      userRec.eMail        = eMail ;
      userRec.managerEMail = managerEMail ;
      userRec.phone        = phone ;
      userRec.password     = password ;
      userRec.toString     = toString ;
      userRec.backUp       = backUp ;
      userRec.isManager    = isManager ;
      userRec.languageInd  = languageInd ;

      if ( teams != null )
         userRec.setTeams ( teams ) ;

      if ( auths != null ) {
         for ( int i = 0; i < auths.length ; i++ )
            userRec.addAuth ( auths[i] ) ;
      }


      if ( countries != null )
         userRec.setCountries ( countries ) ;

      if ( brandMgrVec != null )
         userRec.brandMgrVec  = (Vector) brandMgrVec.clone() ;

      return( userRec ) ;
   }





   /*------------------------------------------------------------*/
   /* setData
    */
   /**
    *    This method will set the data of this record
    *  from the UserRec passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setData ( UserRec userRec ) {
      this.ipAddress    = userRec.ipAddress ;
      this.manager      = userRec.manager ;
      this.eMail        = userRec.eMail ;
      this.managerEMail = userRec.managerEMail ;
      this.phone        = userRec.phone ;
      this.password     = userRec.password ;
      this.toString     = userRec.toString ;
      this.backUp       = userRec.backUp ;
      this.isManager    = userRec.isManager ;
      this.languageInd  = userRec.languageInd ;

      teams=null ;
      if ( userRec.teams != null )
         setTeams ( userRec.teams ) ;

      auths=null ;
      if ( userRec.auths != null ) {
         for ( int i = 0; i < userRec.auths.length ; i++ )
            addAuth ( userRec.auths[i] ) ;
      }


      countries=null ;
      if ( userRec.countries != null )
         setCountries ( userRec.countries ) ;

      if ( userRec.brandMgrVec != null )
         this.brandMgrVec  = (Vector) userRec.brandMgrVec.clone() ;

   }





   /*------------------------------------------------------------*/
   /* changePassword
    */
   /**
    *    This method will set the user's password
    *  to the password passed.
    *  
    *
    *  @param password The new password for the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean changePassword ( String password ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "changePassword", 5 ) ;
      boolean   rc        = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         this.password = password ;
         stmt.executeUpdate ( "UPDATE TIGRIS.USERID "+
                              "SET PASSWORD    = '"+encryptPassword()+"', "+
                              "    ATTEMPTS    = 0, "+
                              "    DBUSER      = '"+UserSystem.getUserId()+"', "+
                              "    CHANGEDTIME = CURRENT TIMESTAMP "+
                              "WHERE USERID    = '"+userId+"'" ) ;

         rc = true ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         if ( LogSystem.getInstance() != null )
            LogSystem.log ( 1, e, false ) ;
         else
            LogSystem.logExceptionToFile ( e, "c:\\UserRec.err" ) ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }







   /*------------------------------------------------------------*/
   /* getAttempts
    */
   /**
    *    THis method will return how many times the
    *  user has tried to log into the tool
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getAttempts() {
      return( attempts ) ;
   }





   /*------------------------------------------------------------*/
   /* setAttempts
    */
   /**
    *    This method will set the attempts to the
    *  value specified. The attempts is the number of
    *  times the user has failed to log in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAttempts ( int attempts ) {
      this.attempts = attempts ;
   }





   /*------------------------------------------------------------*/
   /* hasBPMAuth
    */
   /**
    *    This method will return TRUE if the user is
    *  a BPM to the brand passed
    *  
    *
    *  @param brandInd The brand indicator to check
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean hasBPMAuth ( int brandInd ) {
      boolean hasAuth = false ;

      if ( ( brandMgrVec != null ) && ( brandMgrVec.size() >0 ) ) {
         int i     = 0 ;
         int size  = brandMgrVec.size() ;

         while ( ( i < size ) && ( !hasAuth ) ) {
            hasAuth = ( ((BrandMgrRec)brandMgrVec.elementAt(i)).getBrandInd() == brandInd ) ;
            i++ ;
         }
      }

      return( hasAuth ) ;
   }






   /*------------------------------------------------------------*/
   /* hasBrandAuth
    */
   /**
    *    This method will return TRUE if the user
    *  has the authority for the brand passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean hasBrandAuth ( int brandInd ) {
      boolean hasAuth = hasBPMAuth ( brandInd ) ;                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If they do not have BPM auth, check for normal brand    */
      /* authorites                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !hasAuth )
         hasAuth = hasAuth ( QuestUtil.brandAuthFromBrandInd ( brandInd ) ) ;

      return( hasAuth ) ;
   }



   /*------------------------------------------------------------*/
   /* hasAvalonBrandAuth
    */
   /**
    *    This method will return TRUE if the user
    *  has the authority for the avalon brand passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean hasAvalonBrandAuth ( int brandInd ) {
      boolean hasAuth = hasBPMAuth ( brandInd ) ;                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If they do not have BPM auth, check for normal brand    */
      /* authorites                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !hasAuth )
         hasAuth = hasAuth ( QuestUtil.avalonBrandAuthFromBrandInd ( brandInd ) ) ;

      return( hasAuth ) ;
   }



   /**
     *  This method will log on the user specified in this object. It will auto-update the
     * attempts value accordingly.
     *
     * @return One of the LOGIN_XXX errors defined in this class or LOGIN_OK if no problem.
     **/
   protected int login () {
      SQLMethod sqlMethod       = new SQLMethod ( DatabaseSystem.QUEST, "UserRec.login", 5 ) ;
      boolean   sqlMethodClosed = false ;
      int       rc              = 0 ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.PASSWORD, A.NAME, A.ATTEMPTS, B.LANGUAGEIND "+
                                              "FROM TIGRIS.USERID A, "+
                                              "     TIGRIS.USERLANGUAGE B "+
                                              "WHERE A.USERID = '"+userId+"' AND "+
                                              "      A.USERID = B.USERID "+
                                              "FOR FETCH ONLY" ) ;

         if ( rs.next() ) {
            String dbPassword = rs.getString(1).trim() ;
            int    attempts   = rs.getInt(3) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If attempts < 5 continue, otherwise throw error         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( attempts < 5 ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Compare passwords                                       */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( dbPassword.equals ( getEncryptedPassword() ) ) {


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Ok, everything is good, now copy our info over          */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  this.userId      = userId ;
                  this.name        = rs.getString(2).trim() ;
                  this.languageInd = rs.getInt(4) ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Read in the rest of the information for this user       */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  rs.close() ;
                  sqlMethod.close() ;
                  sqlMethodClosed = true ;
                  readAuthorities ( ) ;
                  readTeams       ( ) ;
                  readCountry     ( ) ;
                  readBrands      ( ) ;


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   If the attempts is > 0, set them back to 0              */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( attempts > 0 ) {
                     setAttempts ( 0 ) ;
                  }

               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   The password was wrong, return the error and update     */
               /* the amount of attempts to login                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  rc = LOGIN_PASSWORD_WRONG ;
                  setAttempts ( attempts+1 ) ;
               }


            } else
               rc = LOGIN_OK ;

         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise the user has not been found                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else
            rc = LOGIN_USER_NOT_FOUND ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         rc = LOGIN_SQL_ERROR ;
      }


      if ( !sqlMethodClosed )
         sqlMethod.close() ;

      return( rc ) ;
   } /* login  */





   /*------------------------------------------------------------*/
   /* setAttempts
    */
   /**
    *    This method will update the 'ATTEMPTS'
    *  column in 'TIGRIS.USERID' to the amount
    *  of attempts passed in
    *
    *  @param attempts The new amount of attempts
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setAttemptsNow ( int attempts ) throws Exception {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "UserRec.setAttemptsNow", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE TIGRIS.USERID "+
                              "SET ATTEMPTS = "+attempts+" "+
                              "WHERE USERID = '"+userId+"'" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;
   }





   /*------------------------------------------------------------*/
   /* readAuthorities
    */
   /**
    *    This method will read in all of the authories
    *  granted to this user in all of the teams
    *  that the user belongs to
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readAuthorities (  )  {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "UserRec.setAttemptsNow", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.AUTHIND "+
                                              "FROM TIGRIS.USERGROUPAUTH A,                   "+
                                              "     TIGRIS.USERIDUSERGROUP B                  "+
                                              "WHERE B.USERID = '"+userId+"' AND "+
                                              "      A.USERGROUP = B.USERGROUP                "+
                                              "FOR FETCH ONLY                                 " ) ;      


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through and read our values                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() )
            addAuth ( rs.getInt(1) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Close our cursor and exit                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs.close() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* readTeams
    */
   /**
    *    This method will read in all of the teams
    *  granted to this user 
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readTeams ( ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "UserRec.setAttemptsNow", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT USERGROUP                         "+
                                              "FROM TIGRIS.USERIDUSERGROUP              "+
                                              "WHERE USERID = '"+userId+"' "+
                                              "FOR FETCH ONLY                           " ) ;      


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through and read our values                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() )
            addTeam ( rs.getInt(1) ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Close our cursor and exit                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* readCountry
    */
   /**
    *    This method will read in all of the user's
    *  countries.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readCountry ( ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "UserRec.setAttemptsNow", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT COUNTRYCODEIND FROM TIGRIS.USERIDCOUNTRY WHERE USERID = '"+userId+"'" ) ;

         while ( rs.next() )
            addCountry ( rs.getInt(1) ) ;

         rs.close() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* readBrands
    */
   /**
    *    This method will read in all of the user's
    *  brands.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readBrands ( ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "UserRec.setAttemptsNow", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT BRANDIND, TYPEBRANDMGR FROM TIGRIS.USERIDBRAND WHERE USERID = '"+userId+"'" ) ;

         while ( rs.next() )
            addBrandMgrRec ( new BrandMgrRec ( rs.getString(2).trim(), rs.getInt(1) ) );

         rs.close() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
      }

      sqlMethod.close() ;
   }



} /* UserRec */


