package com.ibm.nzna.projects.qit.app ;     
/*-----------------------------------------------------------*/
/*   UserRecEditor                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: UserRecEditor                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import javax.swing.border.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
/**
 * The Editor for the UserRec allows 1 field to be edited. This field 
 * is a JComponent which holds the required GUI to edit that field. The
 * field is identified by one of the FIELD_XXX values defined in this class.<br><br>
 *
 * The JComponent may be added to a JList with many other UserRecEditor classes to
 * provide a complete UserRec edit piece.<br><br>
 *
 * Each UserRecEditor has a 'saveData' method. This method will save whatever data
 * is being represented. If the data is not adequate then a message will be displayed
 * to the user and a 'FALSE' will be returned from the saveData method.
 *
 * @version 1.0, 4/5/2000 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   UserRecEditor                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class UserRecEditor extends JComponent implements DataLengths, ActionListener, TypeListListener, Editor, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Field Ids                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int FIELD_USERID       = 1 ;
   public static final int FIELD_NAME         = 2 ;
   public static final int FIELD_MANAGER      = 3 ;
   public static final int FIELD_EMAIL        = 4 ;
   public static final int FIELD_PHONE        = 5 ;
   public static final int FIELD_PASSWORD     = 6 ;
   public static final int FIELD_TEAM         = 7 ;
   public static final int FIELD_BRAND_MGR    = 8 ;
   public static final int FIELD_LANGUAGE     = 9 ;
   public static final int FIELD_COUNTRY      = 10 ;
   public static final int FIELD_BACKUP       = 11 ;


   public static final String[] defaultTitleHeadings = { ""} ;
   public static final String[] brandMgrHeaders = { Str.getStr ( STR_BRAND ), Str.getStr ( STR_TYPE )} ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Components. To find out what each of these          */
   /* components actually do, please see the 'createGUI' method */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextField       ef_ENTRYFIELD        = null ;
   private JComboBox        cb_COMBOBOX          = null ;
   private JLabel           st_TITLE             = null ;
   private MultiList        cnr_MULTILIST        = null ;
   private JCheckBox        ck_CHECKBOX          = null ;
   private DButton          pb_BUTTON1           = null ;
   private DButton          pb_BUTTON2           = null ;      
   private UserEntryPanel   ef_USERENTRY         = null ;
   private JScrollPane      scr_SCROLLPANE       = null ;
   private ButtonPanel      buttonPanel          = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       editField               = 0  ;   // This is the field we are editing
   private UserRec   userRec                 = null ; // This is the UserRec hat we are editing
   private Dimension prefSize                = null ; // The preferred size in pixels of this control





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor. 
    *  
    *  
    *
    *  @param editField This is the FIELD_XXX value to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public UserRecEditor ( int editField ) {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our local vars up                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.editField = editField ;
      prefSize = new Dimension ( 300, 50 ) ;

      createGUI () ;
   }





   /*------------------------------------------------------------*/
   /* createGUI
    */
   /**
    *    This method will create the necessary GUI
    *  components to edit whatever field in the
    *  user record that we are editing
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createGUI () {
      int title = 0 ;

      switch ( editField ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   FIELD_USERID: We will use the ef_ENTRYFIELD as a        */
         /* entry field to gather the user id                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case FIELD_USERID :
            title = STR_USERID ;
            ef_ENTRYFIELD = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_USERID ), "", 0 ) ;
            break ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_NAME: We will use the ef_ENTRYFIELD as a          */
            /* entry field to gather the user's name                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_NAME :
            title = STR_NAME ;
            ef_ENTRYFIELD = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_NAME ), "", 0 ) ;
            break ;                                              

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_NAME: We will use the ef_USERENTRY  as a          */
            /* entry field to gather the user's backup                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_BACKUP :
            title = STR_BACK_UP ;
            ef_USERENTRY = new UserEntryPanel () ;
            break ;                                              

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_MANAGER: We will use the cb_COMBOBOX as a list    */
            /* to allow the user to select a manager                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_MANAGER :
            title = STR_MANAGER ;
            ef_USERENTRY = new UserEntryPanel () ;
            ef_USERENTRY.setShowManagerOnly ( true ) ;
            ck_CHECKBOX = new JCheckBox ( Str.getStr ( STR_IS_A_MANAGER ) ) ;
            prefSize.height+=30 ;
            ck_CHECKBOX.setOpaque ( false ) ;
            ck_CHECKBOX.addActionListener ( this ) ;
            break ;                                      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_EMAIL: We will use the ef_ENTRYFIELD as a         */
            /* entry field to gather the user's eMail Address            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_EMAIL :
            title = STR_EMAIL_ADDRESS ;
            ef_ENTRYFIELD = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_EMAILADDR ), "", 0 ) ;
            break ;                                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_PHONE: We will use the ef_ENTRYFIELD as a         */
            /* entry field to gather the user's phone number             */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_PHONE :
            title = STR_PHONE ;
            ef_ENTRYFIELD = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_PHONE ), "", 0 ) ;
            break ;                                     


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_TEAM: We will use the MultiList to allow the user */
            /* to select multiple teams for this user. We will also use  */
            /* the pb_BUTTON1 and pb_BUTTON2 as 'Add' and 'Remove'       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_TEAM :
            title = STR_TEAM ;
            cnr_MULTILIST = new MultiList ( GUISystem.getFontUtil() ) ;
            cnr_MULTILIST.setColumnHeadings ( defaultTitleHeadings ) ;
            cnr_MULTILIST.showColumnHeadings ( false ) ;
            pb_BUTTON1 = new DButton ( Str.getStr ( STR_ADD ) ) ;
            pb_BUTTON2 = new DButton ( Str.getStr ( STR_REMOVE ) ) ;
            pb_BUTTON1.addActionListener ( this ) ;
            pb_BUTTON2.addActionListener ( this ) ;
            GUISystem.setPreferredButton ( pb_BUTTON1 ) ;
            GUISystem.setPreferredButton ( pb_BUTTON2 ) ;
            buttonPanel = new ButtonPanel () ;
            buttonPanel.add ( pb_BUTTON1 ) ;
            buttonPanel.add ( pb_BUTTON2 ) ;
            prefSize.height=300 ;
            break ;                                       


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_BRAND_MGR: We will use the ck_CHECKBOX to allow   */
            /* the user to specfiy this user is a brand program manager. */
            /* We will also use the cnr_MULTILIST to display the list    */
            /* of brands.                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_BRAND_MGR :
            title = STR_BRAND_PROGRAM_MANAGER ;
            ck_CHECKBOX   = new JCheckBox ( Str.getStr ( title ) ) ;
            ck_CHECKBOX.setOpaque ( false ) ;
            cnr_MULTILIST = new MultiList ( GUISystem.getFontUtil() ) ;
            cnr_MULTILIST.setColumnHeadings ( brandMgrHeaders ) ;
            ck_CHECKBOX.addActionListener ( this ) ;
            pb_BUTTON1 = new DButton ( Str.getStr ( STR_ADD ) ) ;
            pb_BUTTON2 = new DButton ( Str.getStr ( STR_REMOVE ) ) ;
            pb_BUTTON1.addActionListener ( this ) ;
            pb_BUTTON2.addActionListener ( this ) ;
            GUISystem.setPreferredButton ( pb_BUTTON1 ) ;
            GUISystem.setPreferredButton ( pb_BUTTON2 ) ;
            buttonPanel = new ButtonPanel () ;
            buttonPanel.add ( pb_BUTTON1 ) ;
            buttonPanel.add ( pb_BUTTON2 ) ;
            prefSize.height=300 ;
            break ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_LANGUAGE: We will use the cb_COMBOBOX to          */
            /* allow the user to select 1 language                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_LANGUAGE :
            title = STR_LANGUAGE ;
            cb_COMBOBOX = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_LANGUAGE ) ) ;
            break ;                                            


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_COUNTRY: We will use the cnr_MULTILIST to allow   */
            /* The user to select multiple countries.                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_COUNTRY :
            title = STR_COUNTRY ;
            cnr_MULTILIST = new MultiList ( GUISystem.getFontUtil() ) ;
            cnr_MULTILIST.setColumnHeadings ( defaultTitleHeadings ) ;
            cnr_MULTILIST.showColumnHeadings ( false ) ;
            pb_BUTTON1 = new DButton ( Str.getStr ( STR_ADD ) ) ;
            pb_BUTTON2 = new DButton ( Str.getStr ( STR_REMOVE ) ) ;
            pb_BUTTON1.addActionListener ( this ) ;
            pb_BUTTON2.addActionListener ( this ) ;
            GUISystem.setPreferredButton ( pb_BUTTON1 ) ;
            GUISystem.setPreferredButton ( pb_BUTTON2 ) ;
            buttonPanel = new ButtonPanel () ;
            buttonPanel.add ( pb_BUTTON1 ) ;
            buttonPanel.add ( pb_BUTTON2 ) ;
            prefSize.height=300 ;
            cnr_MULTILIST.setBorder ( null ) ;
            break ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, create our title and add our GUI                    */                           
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE = new JLabel ( Str.getStr ( title ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque ( false ) ;
      st_TITLE.setFont ( FontSystem.smallTitleFont ) ;

      setProperties() ;

      addGUI() ;

   } /* createGUI  */






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This mehtod will lay out and posistion all of 
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int y         = 0 ;
      int x         = 0 ;
      int rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_TITLE.setBounds ( 0, 0, prefSize.width, rowHeight ) ;
      y+=rowHeight+2 ;

      switch ( editField ) {
         case FIELD_USERID :
         case FIELD_NAME :
         case FIELD_EMAIL :
         case FIELD_PHONE :
            ef_ENTRYFIELD.setBounds ( x, y, 250, rowHeight ) ;
            break ;

         case FIELD_LANGUAGE :
            cb_COMBOBOX.setBounds ( x, y, 250, rowHeight ) ;
            break ;


         case FIELD_COUNTRY :
         case FIELD_TEAM :
            cnr_MULTILIST.setBounds ( x, y, 250, prefSize.height-y-45 ) ;
            y=(prefSize.height-y-25 ) ;
            buttonPanel.setBounds ( x, y, 250, 25 ) ;
            break ;

         case FIELD_BRAND_MGR :
            ck_CHECKBOX.setBounds ( x, y, 250, rowHeight ) ;
            y+=rowHeight ;
            cnr_MULTILIST.setBounds ( x, y, 250, prefSize.height-y-45 ) ;
            y+=(prefSize.height-y-45);
            buttonPanel.setBounds ( x,y ,250,25 ) ;
            break ;

         case FIELD_MANAGER :
            ck_CHECKBOX.setBounds ( x, y, 250, rowHeight ) ;
            y+=rowHeight ;

         case FIELD_BACKUP :
            ef_USERENTRY.setBounds ( x, y, 250, rowHeight ) ;
            break ;

      }
   }



   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method should refresh any data on the
    *  Editor.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData () {
      Vector dataVec = new Vector ( 1, 1 ) ;

      switch ( editField ) {   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   FIELD_BACKUP                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case FIELD_BACKUP :
            ef_USERENTRY.setText ( userRec.getBackUp() ) ;
            break ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_TEAMS - Only refresh if we already have the data  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_TEAM :
            int[]  teams   = userRec.getTeams() ;

            if ( teams != null ) {
               for ( int i = 0; i < teams.length; i++ ) {
                  if ( teams[i] > 0 ) 
                     dataVec.addElement ( TypeList.getInstance().objectFromInd ( teams[i], TypeList.TYPE_TEAM ) ) ;
               }

            }

            cnr_MULTILIST.removeAll() ;
            cnr_MULTILIST.add ( dataVec ) ;
            break ;

            /*-------------------------------------------------------------*/
            /*                                                             */
            /*   FIELD_COUNTRY Only refresh if we already have the data    */
            /*                                                             */
            /*-------------------------------------------------------------*/
         case FIELD_COUNTRY :
            int[]  countries    = userRec.getCountries() ;

            if ( countries != null ) {
               for ( int i = 0; i < countries.length; i++ )
                  dataVec.addElement ( TypeList.getInstance().objectFromInd ( countries[i], TypeList.TYPE_COUNTRYCODE ) ) ;

               cnr_MULTILIST.removeAll() ;
               cnr_MULTILIST.add ( dataVec ) ;
            }
            break ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_USERID: We will use the ef_ENTRYFIELD as a        */
            /* entry field to gather the user id                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_USERID :
            ef_ENTRYFIELD.setText ( userRec.getUserId() ) ;  

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is an edit, do not allow the user id to be      */
            /* changed                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ef_ENTRYFIELD.setEditable ( userRec.getRecStatus() != userRec.REC_UPDATE ) ;

            break ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_NAME: We will use the ef_ENTRYFIELD as a          */
            /* entry field to gather the user's name                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_NAME :
            ef_ENTRYFIELD.setText ( userRec.getName() ) ;
            break ;                                              

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_MANAGER: We will use the cb_COMBOBOX as a list    */
            /* to allow the user to select a manager                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_MANAGER :
            ef_USERENTRY.setText ( userRec.getManagerUserId() ) ;
            ck_CHECKBOX.setSelected ( userRec.isManager() ) ;
            break ;                                      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_EMAIL: We will use the ef_ENTRYFIELD as a         */
            /* entry field to gather the user's eMail Address            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_EMAIL :
            ef_ENTRYFIELD.setText ( userRec.getEMail () ) ;
            break ;                                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_PHONE: We will use the ef_ENTRYFIELD as a         */
            /* entry field to gather the user's phone number             */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_PHONE :
            ef_ENTRYFIELD.setText ( userRec.getPhone() ) ;
            break ;                                     


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_BRAND_MGR: We will use the ck_CHECKBOX to allow   */
            /* the user to specfiy this user is a brand program manager. */
            /* We will also use the cnr_MULTILIST to display the list    */
            /* of brands.                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_BRAND_MGR :
            Vector brandMgrVec = userRec.getBrandMgrVec();

            ck_CHECKBOX.setSelected ( ( brandMgrVec != null ) && ( brandMgrVec.size() > 0 ) ) ;

            cnr_MULTILIST.setEnabled ( ck_CHECKBOX.isSelected() ) ;
            pb_BUTTON1.setEnabled ( ck_CHECKBOX.isSelected() ) ;
            pb_BUTTON2.setEnabled ( ck_CHECKBOX.isSelected() ) ;
            cnr_MULTILIST.removeAll() ;

            if ( ( brandMgrVec != null ) && ( brandMgrVec.size() > 0 ) ) {
               cnr_MULTILIST.add ( brandMgrVec ) ;
            }
            break ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_LANGUAGE: We will use the cb_COMBOBOX to          */
            /* allow the user to select 1 language                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_LANGUAGE :
            cb_COMBOBOX.setSelectedItem ( TypeList.getInstance().objectFromInd ( userRec.getLanguage(), TypeList.TYPE_LANGUAGE ) ) ;
            break ;                                            
      }

   } /* refreshData  */





   /*------------------------------------------------------------*/
   /* setEditUserRec
    */
   /**
    *    This method will set what UserRe we
    *  are editing
    *  
    *
    *  @param userRec The UserRec to edit
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEditUserRec ( UserRec userRec ) {
      this.userRec = userRec ;
   }


   /*------------------------------------------------------------*/
   /* listRefreshed
    */
   /**
    *    This method will be called when one of the lists
    *  we have had refreshed is done
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void listRefreshed ( int typeList ) {
   }




   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our minimum size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( getMinimumSize() ) ;
   }




   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return the minimal size
    *  that this control can be reduced to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* addGUI
    */
   /**
    *    This method will add our GUI to this panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addGUI () {
      setLayout ( null ) ;
      add ( st_TITLE ) ;
      if ( ef_ENTRYFIELD != null )
         add ( ef_ENTRYFIELD ) ;
      if ( cb_COMBOBOX != null )
         add ( cb_COMBOBOX ) ;
      if ( st_TITLE != null )
         add ( st_TITLE ) ;
      if ( cnr_MULTILIST != null )
         add ( cnr_MULTILIST ) ;
      if ( ck_CHECKBOX != null )
         add ( ck_CHECKBOX ) ;
      if ( scr_SCROLLPANE != null )
         add ( scr_SCROLLPANE ) ;
      if ( ef_USERENTRY != null )
         add ( ef_USERENTRY ) ;
      if ( buttonPanel != null )
         add ( buttonPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* getBrandList
    */
   /**
    *    This method will return the brand list with
    *  an extra added (Document) and (Online Assistant) 
    *  to each brand name.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector getBrandList () {
      Vector returnVec = new Vector ( 1, 1 ) ;
      Vector brandVec  = Brands.getBrands() ;
      int    i         = 0 ;
      int    size      = brandVec.size() ;

      while ( i < size ) {
         returnVec.addElement ( new String ( brandVec.elementAt(i).toString()+" ("+Str.getStr ( STR_DOCUMENT )+")" ) ) ;
         returnVec.addElement ( new String ( brandVec.elementAt(i).toString()+" ("+Str.getStr ( STR_ONLINE_ASSISTANT )+")" ) ) ;
         i++ ;
      }

      return( returnVec ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when action events 
    *  are fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      switch ( editField ) {
         case FIELD_MANAGER :
            if ( e.getSource() == ck_CHECKBOX ) {
               boolean isManager = ck_CHECKBOX.isSelected() ;

               ef_USERENTRY.setText ( isManager ? userRec.getName() : "" ) ;
               ef_USERENTRY.setEnabled ( !isManager ) ;
            }
            break ;

         case FIELD_BRAND_MGR :
            if ( e.getSource() == ck_CHECKBOX ) {

               if ( !ck_CHECKBOX.isSelected() )
                  removeAllBrandMgr() ;

               cnr_MULTILIST.setEnabled ( ck_CHECKBOX.isSelected() ) ;
               pb_BUTTON1.setEnabled ( ck_CHECKBOX.isSelected() ) ;
               pb_BUTTON2.setEnabled ( ck_CHECKBOX.isSelected() ) ;

            }

            else if ( e.getSource() == pb_BUTTON1 )
               addBrandMgr() ;
            else if ( e.getSource() == pb_BUTTON2 )
               removeBrandMgr() ;
            break ;

         case FIELD_COUNTRY :
            if ( e.getSource() == pb_BUTTON1 )
               addCountry() ;
            else if ( e.getSource() == pb_BUTTON2 )
               removeCountry() ;
            break ;

         case FIELD_TEAM :
            if ( e.getSource() == pb_BUTTON1 )
               addTeam() ;
            else if ( e.getSource() == pb_BUTTON2 )
               removeTeam() ;
            break ;

      }


   }




   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method should save any data being
    *  represented and return TRUE if the data
    *  was saved. Otherwise, put up a message to the user and return FALSE
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      int        errorMsg  = 0 ;
      JComponent errorComp = null ;
      boolean    rc        = true ;

      switch ( editField ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   FIELD_USERID: We will use the ef_ENTRYFIELD as a        */
         /* entry field to gather the user id                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case FIELD_USERID :
            String userId = ef_ENTRYFIELD.getText() ;

            if ( ( userId == null ) || ( userId.length() == 0 ) ) {
               errorMsg  = STR_MUST_ENTER_USERID ;
               errorComp = ef_ENTRYFIELD ;
            } else {

               userRec.setUserId ( userId ) ;                                             

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If this is a new User Id, set the password to their id  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( userRec.getRecStatus() == userRec.REC_INSERT )
                  userRec.setPassword ( userId ) ;

            }


            break ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_NAME: We will use the ef_ENTRYFIELD as a          */
            /* entry field to gather the user's name                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_NAME :
            String name = ef_ENTRYFIELD.getText() ;

            if ( ( name == null ) || ( name.length() == 0 ) ) {
               errorMsg  = STR_MUST_ENTER_NAME ;
               errorComp = ef_ENTRYFIELD ;
            } else
               userRec.setName ( name ) ;
            break ;                                              

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_MANAGER: We will use the cb_COMBOBOX as a list    */
            /* to allow the user to select a manager                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_MANAGER :
            UserRec managerUserRec = ef_USERENTRY.getUserRec() ;

            if ( managerUserRec == null ) {
               errorMsg  = STR_MUST_ENTER_MANAGER ;
               errorComp = ef_ENTRYFIELD ;
            } else
               userRec.setManagerUserId ( managerUserRec.getUserId() ) ;

            userRec.setIsManager ( ck_CHECKBOX.isSelected() ) ;
            break ;                                      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_EMAIL: We will use the ef_ENTRYFIELD as a         */
            /* entry field to gather the user's eMail Address            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_EMAIL :
            String email = ef_ENTRYFIELD.getText() ;

            if ( ( email == null ) || ( email.length() == 0 ) ) {
               errorMsg  = STR_MUST_ENTER_EMAIL_ADDR ;
               errorComp = ef_ENTRYFIELD ;
            } else
               userRec.setEMail ( email ) ;

            break ;                                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_PHONE: We will use the ef_ENTRYFIELD as a         */
            /* entry field to gather the user's phone number             */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_PHONE :
            String phone = ef_ENTRYFIELD.getText() ;

            if ( ( phone == null ) || ( phone.length() == 0 ) ) {
               errorMsg  = STR_MUST_ENTER_PHONE_NUM ;
               errorComp = ef_ENTRYFIELD ;
            } else
               userRec.setPhone ( phone ) ;
            break ;                                     


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_TEAM: We will use the MultiList to allow the user */
            /* to select multiple teams for this user                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_TEAM :
            int[] teams = userRec.getTeams() ;

            if ( ( teams == null ) || ( teams.length == 0 ) ) {
               errorMsg  = STR_MUST_SELECT_ONE_TEAM ;
               errorComp = cnr_MULTILIST ;
            }
            break ;                                       


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_BRAND_MGR: We will use the ck_CHECKBOX to allow   */
            /* the user to specfiy this user is a brand program manager. */
            /* We will also use the cnr_MULTILIST to display the list    */
            /* of brands.                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_BRAND_MGR :
            break ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_LANGUAGE: We will use the cb_COMBOBOX to          */
            /* allow the user to select 1 language                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_LANGUAGE :
            TypeLanguageRec languageRec = (TypeLanguageRec) cb_COMBOBOX.getSelectedItem() ;

            if ( languageRec != null )
               userRec.setLanguage ( languageRec.getInd() ) ;
            else {
               errorMsg = STR_MUST_SELECT_LANGUAGE ;
               errorComp = cb_COMBOBOX ;
            }
            break ;                                               

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_BACKUP                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_BACKUP :
            UserRec backUpUserRec = ef_USERENTRY.getUserRec() ;

            if ( backUpUserRec == null ) {
               errorMsg  = STR_MUST_ENTER_BACKUP ;
               errorComp = ef_USERENTRY ;
            } else
               userRec.setBackUp ( backUpUserRec.getUserId() ) ;
            break ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   FIELD_COUNTRY: We will use the cnr_MULTILIST to allow   */
            /* The user to select multiple countries.                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case FIELD_COUNTRY :
            int[] countries = userRec.getCountries() ;
            Vector countryVec = cnr_MULTILIST.getSelection() ;

            if ( ( countries == null ) || ( countries.length == 0 ) ) {
               errorMsg  = STR_MUST_SELECT_ONE_COUNTRY ;
               errorComp = cnr_MULTILIST ;
            }
            break ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have a message, show it, set the focus to         */                                      
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( errorMsg != 0 ) || ( errorComp != null ) ) {
         if ( errorMsg > 0 )
            GUISystem.printBox ( STR_ERROR, errorMsg ) ;

         if ( errorComp != null )
            errorComp.requestFocus() ;

         rc = false ;
      }


      return( rc ) ;
   } /* saveData  */





   /*------------------------------------------------------------*/
   /* setProperties
    */
   /**
    *    This method will set the properties on all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setProperties () {

      if ( ef_USERENTRY != null ) {
         ef_USERENTRY.setBorder ( LineBorder.createBlackLineBorder() ) ;
      }

      if ( cnr_MULTILIST != null ) {
         cnr_MULTILIST.setBorder ( LineBorder.createBlackLineBorder() ) ;
      }

      if ( pb_BUTTON1 != null )
         pb_BUTTON1.setOpaque ( false ) ;

      if ( pb_BUTTON2 != null )
         pb_BUTTON2.setOpaque ( false ) ;

      if ( buttonPanel != null )
         buttonPanel.setOpaque ( false ) ;
   }





   /*------------------------------------------------------------*/
   /* addTeam
    */
   /**
    *    This method will add a team to the user
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addTeam () {
      ListDlg listDlg = new ListDlg ( GUISystem.getParentDefWin ( this ), 
                                      Str.getStr ( STR_ADD_TEAM ), 
                                      TypeList.getInstance().getTypeList ( TypeList.TYPE_TEAM ),
                                      GUISystem.getFontUtil() ) ;
      Vector  addVec  = null ;                                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the properties of the list dlg and show it       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listDlg.setOkText     ( Str.getStr ( STR_OK ) ) ;
      listDlg.setCancelText ( Str.getStr ( STR_CANCEL ) );
      addVec = listDlg.getResult() ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add each member selected                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( addVec != null ) {
         int i    = 0 ;
         int size = addVec.size() ;

         while ( i < size ) {
            userRec.addTeam ( ((TypeTeamRec)addVec.elementAt(i)).getInd() ) ;
            i++ ;
         }
      }

      refreshData() ;

   }


   /*------------------------------------------------------------*/
   /* addCountry
    */
   /**
    *    This method will add a team to the user
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addCountry () {
      ListDlg listDlg = new ListDlg ( GUISystem.getParentDefWin ( this ), 
                                      Str.getStr ( STR_ADD_COUNTRY ), 
                                      TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ),
                                      GUISystem.getFontUtil() ) ;
      Vector  addVec  = null ;                                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the properties of the list dlg and show it       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listDlg.setOkText     ( Str.getStr ( STR_OK ) ) ;
      listDlg.setCancelText ( Str.getStr ( STR_CANCEL ) );
      addVec = listDlg.getResult() ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add each member selected                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( addVec != null ) {
         int i    = 0 ;
         int size = addVec.size() ;

         while ( i < size ) {
            userRec.addCountry ( ((TypeCountryCodeRec)addVec.elementAt(i)).getInd() ) ;
            i++ ;
         }
      }

      refreshData() ;

   }





   /*------------------------------------------------------------*/
   /* removeCountry
    */
   /**
    *    This method will remove all selected
    *  countries from the user
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeCountry () {
      Vector vec = cnr_MULTILIST.getSelection() ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i    = 0 ;
         int size = vec.size() ;

         while ( i < size ) {
            userRec.removeCountry ( ((TypeCountryCodeRec)vec.elementAt(i)).getInd() ) ;
            i++ ;
         }
      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

      refreshData() ;
   } /* removeCountry  */



   /*------------------------------------------------------------*/
   /* removeTeam
    */
   /**
    *    This method will remove all selected teams
    *  from the user
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeTeam () {
      Vector vec = cnr_MULTILIST.getSelection() ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i    = 0 ;
         int size = vec.size() ;

         while ( i < size ) {
            userRec.removeTeam ( ((TypeTeamRec)vec.elementAt(i)).getInd() ) ;
            i++ ;
         }
      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

      refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* removeBrandMgr
    */
   /**
    *    This method will remove a brand manager
    *  from the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeBrandMgr () {
      Vector vec = cnr_MULTILIST.getSelection() ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i    = 0 ;
         int size = vec.size() ;

         while ( i < size ) {
            userRec.removeBrandMgrRec ( ((BrandMgrRec)vec.elementAt(i)) ) ;
            i++ ;
         }
      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

      refreshData() ;
   }


   /*------------------------------------------------------------*/
   /* removeAllBrandMgr
    */
   /**
    *    This method will remove all brand managers
    *  from the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeAllBrandMgr () {
      userRec.setBrandMgrVec ( new Vector ( 1, 1 ) ) ;
      refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* addBrandMgr
    */
   /**
    *    This method will add a brand mgr to our
    *  list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addBrandMgr () {
      Vector   typeVec  = new Vector ( 2 ) ;
      Vector   brandVec = null ;
      ListDlg  listDlg  = null ;

      typeVec.addElement ( new MultiListObject ( Str.getStr ( STR_DOCUMENT ) ) ) ;
      typeVec.addElement ( new MultiListObject ( Str.getStr ( STR_AVALON ) ) ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, get the brand                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listDlg = new ListDlg ( GUISystem.getParentDefWin ( this ),
                              Str.getStr ( STR_BRAND ),
                              Brands.getBrands(),
                              GUISystem.getFontUtil() ) ;
      listDlg.setOkText     ( Str.getStr ( STR_OK ) ) ;
      listDlg.setCancelText ( Str.getStr ( STR_CANCEL ) ) ;
      brandVec = listDlg.getResult() ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, get the type                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( brandVec != null ) {
         listDlg = new ListDlg ( GUISystem.getParentDefWin ( this ),
                                 Str.getStr ( STR_WHAT_BRAND_MGR_TYPE ),
                                 typeVec,
                                 GUISystem.getFontUtil() ) ;
         listDlg.setOkText     ( Str.getStr ( STR_OK ) ) ;
         listDlg.setCancelText ( Str.getStr ( STR_CANCEL ) ) ;
         typeVec = listDlg.getResult() ;

         if ( typeVec != null ) {
            TypeBrandRec brandRec = (TypeBrandRec) brandVec.elementAt(0) ;
            String       typeMgr  = ((MultiListObject) typeVec.elementAt(0)).getDescript() ;

            if ( typeMgr.equals ( Str.getStr ( STR_DOCUMENT ) ) )
               typeMgr = "DOC" ;
            else if ( typeMgr.equals ( Str.getStr ( STR_AVALON ) ) )
               typeMgr = "OA" ;

            userRec.addBrandMgrRec ( new BrandMgrRec ( typeMgr, brandRec.getInd() ) ) ;

            refreshData() ;
         }
      }
   }
} /* UserRecEditor */
