package com.ibm.nzna.projects.qit.app ;
/*-----------------------------------------------------------*/
/*   UserStatRec                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: UserStatRec                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.gui.* ;
/**
 * The UserStatRec holds information on a single user statistic
 *
 *
 * @version 1.0, 1/27/2000 
 * @author  David Lentz
 */
public class UserStatRec extends Object implements java.io.Serializable, MultiListRow {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private int         uid         = 0 ;
    private int         value       = 0 ;
    private String      description = "" ;



    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create a new UserStatRec
     *  initialized to the values passed in
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public UserStatRec ( int uid, int value, String description ) {
        this.uid = uid ;
        setValue ( value ) ;
        setDescription ( description ) ;
    }




    /*------------------------------------------------------------*/
    /* setValue
     */
    /**
     *    This method will set the value of this
     *  UserStatRec
     *  
     *
     *  @param value The new value
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setValue ( int value ) {
        this.value = value ;
    }




    /*------------------------------------------------------------*/
    /* setDescription
     */
    /**
     *    This method will set the user readable
     *  description of this user statistic
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setDescription ( String description ) {
        this.description = description ;
    }


    /*------------------------------------------------------------*/
    /* getDescription
     */
    /**
     *    This method will return the description
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getDescription () {
        return( description ) ;
    }




    /*------------------------------------------------------------*/
    /* getValue
     */
    /**
     *    This method will return the value of
     *  this user statistic
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public int getValue () {
        return( value ) ;
    }




    /*------------------------------------------------------------*/
    /* incrementValue
     */
    /**
     *    This method will simply update the value
     *  by 1
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void incrementValue () {
        value++ ;
    }




    /*------------------------------------------------------------*/
    /* decrementValue
     */
    /**
     *    This method will decrement the value
     *  by 1
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void decrementValue () {
        value-- ;
    }




    /*------------------------------------------------------------*/
    /* getUID
     */
    /**
     *    This method will return the UID of the
     *  statistic that this UserStatRec represents
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return int
     *//*---------------------------------------------------------*/
    public int getUID () {
        return( uid ) ;
    }





    /*------------------------------------------------------------*/
    /* getColumnData
     */
    /**
     *    This method will return the data that
     *  should appear in a MultiList for this
     *  object
     *
     *  @param column The column to return for
     *  @param None
     *  @param None
     *
     *  @return Object The object to paint
     *//*---------------------------------------------------------*/
    public Object getColumnData ( int column ) {
        if ( column == 0 )
            return ( description ) ;
        return ( ""+value ) ;
    }


}

