package com.ibm.nzna.projects.qit.app ;           
/*-----------------------------------------------------------*/
/*   UserStatSystem                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: UserStatSystem                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
/**
 * The UserStatSystem can be used by any Quest Applet. An Applet
 * may simply 'add' a statistic to the UserStatSystem and update
 * that statistic at any time. Each statistic has a unique integer
 * and a description. 
 *
 * Statistics can be accessed through this sub system as well.
 *
 * Each time the statistics are updated, a check will be done for
 * the current date. If the date changes, all statistics will be
 * wiped to zero.
 *
 *
 * @version 1.0, 1/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   UserStatSystem                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class UserStatSystem extends Object implements Runnable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Thread Names                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_FIRESTATUPDATE = "FSU" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private static Members                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Vector         statVec            = null ;
   private static String         currentStatDate    = null ;
   private static String         USERSTAT_FILE      = "quest.userstat" ;
   private static String         USERSTATDATE_FILE  = "quest.userstat_date" ;      
   private static UserStatSystem instance           = null ;      

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   private members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector listenerVec   = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the UserStatSystem
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public UserStatSystem () {                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign local vars                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      instance = this ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the current date                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      currentStatDate = new CDate ( CDate.DATE_DATE ).today() ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Try and load our variables                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      createFromDisk() ;
   }





   /*------------------------------------------------------------*/
   /* createFromDisk
    */
   /**
    *    This method will attempt to read
    *  in our statVecand current date
    *  from disk
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createFromDisk () {
      Vector    vec  = (Vector) QuestUtil.readObjectFromDisk ( USERSTAT_FILE ) ;
      String    date = (String) QuestUtil.readObjectFromDisk ( USERSTATDATE_FILE ) ;

      if ( vec  != null )
         statVec= vec ;
      else
         statVec= new Vector() ;

      if ( date != null ) {
         if ( !date.equals ( currentStatDate ) )
            clearStats() ;
      }
   }


   /*------------------------------------------------------------*/
   /* clearStats
    */
   /**
    *    This method will reset all statistics
    *  to 0
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void clearStats () {
      int i       = 0 ;

      if ( statVec != null ) {
         int size = statVec.size() ;

         while ( i < size ) {
            ((UserStatRec)statVec.elementAt(i)).setValue ( 0 ) ;
            i++ ;
         }

         writeToDisk() ;
      }
   }





   /*------------------------------------------------------------*/
   /* checkDate
    */
   /**
    *    This method will check the current date
    *  with what we have stored. If the date
    *  has changed, all stats will be cleared
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void checkDate () {
      String date = new CDate ( CDate.DATE_DATE ).today() ;

      if ( !date.equals ( currentStatDate ) ) {
         currentStatDate = date ;
         clearStats() ;
      }
   }






   /*------------------------------------------------------------*/
   /* statUserRecFromUID
    */
   /**
    *    This method will find the StatUserRec
    *  corresponding to the UID passed in
    *  
    *
    *  @param uid The UID of the stat to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static UserStatRec userStatRecFromUID ( int uid ) {
      int         i       = 0 ;
      int         size    = statVec.size() ;
      UserStatRec retRec  = null ;

      while ( ( retRec == null ) && ( i < size ) ) {
         if ( ((UserStatRec)statVec.elementAt(i)).getUID()==uid )
            retRec = (UserStatRec) statVec.elementAt(i) ;
         i++ ;
      }

      return( retRec ) ;
   }





   /*------------------------------------------------------------*/
   /* addStatistic
    */
   /**
    *    This method will add a statistic to the 
    *  list of statistics.
    *  
    *
    *  @param uid The UID of the statistic
    *  @param value The initial value of the statistic
    *  @param description The description of the statistic
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addStatistic ( int uid, int value, String description ) {
      if ( userStatRecFromUID ( uid ) == null ) {
         UserStatRec userStatRec = new UserStatRec ( uid, value, description ) ;

         statVec.addElement ( userStatRec ) ;
         writeToDisk() ;
         new Thread ( getInstance(), THREAD_FIRESTATUPDATE ).start() ;
      }
   }





   /*------------------------------------------------------------*/
   /* incrementStatistic
    */
   /**
    *    This method will increment the given
    *  statistic by 1
    *  
    *
    *  @param uid The UID of the statistic
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void incrementStatus ( int uid ) {
      UserStatRec userStatRec = userStatRecFromUID ( uid ) ;

      if ( userStatRec != null ) {
         userStatRec.incrementValue() ;
         new Thread ( getInstance(), THREAD_FIRESTATUPDATE ).start() ;
      }
   }




   /*------------------------------------------------------------*/
   /* writeToDisk
    */
   /**
    *    This method will write the UserStatSystem
    *  to the user's local drive
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void writeToDisk () {
      QuestUtil.writeObjectToDisk ( USERSTAT_FILE,     statVec ) ;
      QuestUtil.writeObjectToDisk ( USERSTATDATE_FILE, currentStatDate ) ;
   }





   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return an instance of
    *  the UserStatSystem
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static UserStatSystem getInstance() {
      return( instance ) ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will perform operations in a thread.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_FIRESTATUPDATE                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_FIRESTATUPDATE ) ) {
         int i       = 0 ;
         if ( listenerVec != null ) {
            int size = listenerVec.size() ;

            while ( i < size ) {
               ((UserStatListener)listenerVec.elementAt(i)).userStatChanged () ;
               i++ ;
            }
         }
      }
   }





   /*------------------------------------------------------------*/
   /* addUserStatListener
    */
   /**
    *    This method will add a listener to the
    *  list. Remember to remove yourselves when
    *  done
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addUserStatListener ( UserStatListener listener ) {
      if ( getInstance() != null ) {
         if ( getInstance().listenerVec == null )
            getInstance().listenerVec = new Vector ( 1, 1 ) ;

         getInstance().listenerVec.addElement ( listener ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* removeUserStatListener
    */
   /**
    *    This method will remove a listener
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeUserStatListener ( UserStatListener listener ) {
      if ( getInstance().listenerVec != null )
         getInstance().listenerVec.removeElement ( listener ) ;
   }






   /*------------------------------------------------------------*/
   /* getStatList
    */
   /**
    *    This method will return a vector of UserStatRecs
    *  that can be used. The caller of this
    *  method MUST free the vector, as everything returned is a copy
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getStatList() {
      if ( statVec != null )
         return( (Vector)statVec.clone() ) ;
      return ( null ) ;
   }
} /* UserStatSystem */
