package com.ibm.nzna.projects.qit.app ;    
/*-----------------------------------------------------------*/
/*   UserSystem                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: UserSystem                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.io.* ;
import java.util.* ;
import java.net.* ;
import java.awt.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.chat.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.storedProc.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.qit.storedProc.readUsers.* ;
/**
 * This subsystem is used for mainting the current User 
 * that is logged in. This includes holding all specific values
 * for the user (name, id, manager etc..). This also includes
 * all user authentication. See the 'hasAuthority' method for
 * more details.
 *
 * @version 1.0, 2/9/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   UserSystem                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class UserSystem extends Object implements AppConst {




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static LoggedUserRec loggedUser     = null ;
   private static Vector        allUserVec     = null ;




   /*------------------------------------------------------------*/
   /* hasAuthority
    */
   /**
    *    This method will determine if the
    *  user has the authority passed in. This 
    *  authority could be in any of the teams that the user belongs to.
    *
    *  @param auth The authority indicator to check. Please see AppConst for a list of authorities.
    *  @param None
    *  @param None
    *
    *  @return TRUE = User has authority, FALSE = User does not have authority
    *//*---------------------------------------------------------*/
   public static boolean hasAuthority ( int auth ) {
      boolean hasAuth = false ;

      if ( loggedUser != null )
         return( loggedUser.hasAuth(auth) ) ;

      return( hasAuth ) ;
   }






   /*------------------------------------------------------------*/
   /* hasBrandAuth
    */
   /**
    *    This method will return TRUE if the user
    *  has the authority of the brand specified
    *  
    *
    *  @param brandInd The brand indicator to check. This should be a brand ind defined in TIGRIS.TYPEBRAND
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean hasBrandAuth ( int brandInd ) {
      return( loggedUser.hasBrandAuth ( brandInd ) ) ;
   }


   /**
     *  This method will return true if the logged in user is a BPM
     **/
   public static boolean hasBPMAuth () {
      boolean hasAuth = false ;

      if ( ( loggedUser.getBrandMgrVec() != null ) &&
           ( loggedUser.getBrandMgrVec().size() > 0 ) )
         return ( true ) ;

      hasAuth = hasAuthority ( Authority.AUTH_SUPERUSER ) ;

      return ( hasAuth ) ;
   }


   /*------------------------------------------------------------*/
   /* logonUser
    */
   /**
    *    This method will attempt to logon the 
    *  user id and password passed in.
    *  
    *
    *  @param userId The User Id to log in
    *  @param password the UNENCRYPTED password to log in.
    *  @param None
    *
    *  @return TRUE If the user was logged in Ok, FALSE if not
    *//*---------------------------------------------------------*/
   public static boolean logonUser ( String userId, String password ) {
      boolean             loggedIn      = false ;
      LoggedUserRec       loggedUserRec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_LOGGING_IN_USER ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our variables                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      loggedUserRec = new LoggedUserRec ( userId.trim(), "" ) ;
      loggedUserRec.setPassword ( password ) ;
      PropertySystem.put ( PropertySystem.PROPERTY_PASSWORD, "" ) ;      //Clear the password from the properties

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Log the user in                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( loggedUserRec.login() ) {
         
         case UserRec.LOGIN_PASSWORD_WRONG :
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_PASSWORD_WRONG ) ) ;
            break ;

         case UserRec.LOGIN_TOO_MANY_ATTEMPTS :
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_TOO_MANY_ATTEMPTS ) ) ;
            break ;

         case UserRec.LOGIN_USER_NOT_FOUND :
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_YOUR_USERID_NOT_FOUND ) ) ;
            break ;

         case UserRec.LOGIN_SQL_ERROR :
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_LOGIN_UNAVAILABLE ) ) ;
            break ;

         case UserRec.LOGIN_OK :
            loggedIn = true ;
            loggedUser = loggedUserRec ;
            break ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;

      return( loggedIn ) ;
   }






   /*------------------------------------------------------------*/
   /* getUserName
    */
   /**
    *    This method will return the name of
    *  the user who is currently logged into
    *  the application
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getUserName () {
      if ( loggedUser != null )
         return( loggedUser.getName() ) ;
      else
         return( "No User Logged" ) ;
   }





   /*------------------------------------------------------------*/
   /* getUserId
    */
   /**
    *    This method will return the ID of the currently
    *  logged in user
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getUserId () {
      if ( loggedUser != null )
         return( loggedUser.getUserId() ) ;
      else
         return( "NO_USER" ) ;
   }





   /*------------------------------------------------------------*/
   /* getLanguage
    */
   /**
    *    This method will return the language that 
    *  the logged in user is configured for
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The Language Indicator
    *//*---------------------------------------------------------*/
   public static int getLanguage () {
      try {
         return( loggedUser.getLanguage() ) ;
      } catch ( Exception e ) {
      }
      return( 1 ) ; // Default to English
   }





   /*------------------------------------------------------------*/
   /* setLanguage
    */
   /**
    *    This method will set our language that
    *  the user is using. This method also saves
    *  this information to the database, changes the language for the
    *  application and sets the application's properties.
    *
    *  @param languageInd The new Language Indicator to use, as defined in TIGRIS.TYPELANGUAGE
    *  @param None
    *  @param None
    *
    *  @return TRUE = The language was changed, FALSE = The language was not changed
    *//*---------------------------------------------------------*/
   public static boolean setLanguage ( int languageInd ) {
      SQLMethod sql  = new SQLMethod ( DatabaseSystem.QUEST, "UserSystem.setLanguage", 5 ) ;
      Statement stmt = sql.createStatement() ;
      boolean   rc   = false ;

      try {
         stmt.executeUpdate ( "UPDATE TIGRIS.USERLANGUAGE "+
                              "SET LANGUAGEIND = "+languageInd+" "+
                              "WHERE USERID = '"+loggedUser.getUserId()+"'" ) ;

         loggedUser.setLanguage ( languageInd ) ;
         Str.switchLanguage ( languageInd ) ;
         PropertySystem.putInt ( PropertySystem.PROPERTY_LANGUAGE, languageInd ) ;
         rc = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sql.rollBack() ;
      }

      sql.close() ;

      return( rc );
   }





   /*------------------------------------------------------------*/
   /* setPassword
    */
   /**
    *    This method will set the user's current
    *  password. This method also saves the
    *  password to the database in it's encrypted form.
    *
    *  @param password The password to save
    *  @param None
    *  @param None
    *
    *  @return TRUE if the password was changed, FALSE if not
    *//*---------------------------------------------------------*/
   public static boolean setPassword ( String password ) {
      loggedUser.setPassword ( password ) ;
      loggedUser.setAttempts ( 0 ) ;
      return( loggedUser.saveToDatabase() ) ;
   }





   /*------------------------------------------------------------*/
   /* getLoggedUserRec
    */
   /**
    *    This method will return a copy of the LoggedUserRec.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static LoggedUserRec getLoggedUserRec () {
      try {
         return( (LoggedUserRec) loggedUser.clone() ) ;
      } catch ( Exception e ) {
      }
      return( null ) ;
   } /* getLoggedUserRec  */





   /*------------------------------------------------------------*/
   /* setLoggedUserRec
    */
   /**
    *    This method will set what LoggedUserRec
    *  the application is using.
    *  
    *
    *  @param loggedUserRec The new LoggedUserRec
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setLoggedUserRec ( LoggedUserRec loggedUserRec ) {
      loggedUser = loggedUserRec ;
   }




   /*------------------------------------------------------------*/
   /* refreshAllUsers
    */
   /**
    *    This method will re-read all users from the
    *  database. Please use this function wisely, as
    *  it does take a moment to read in everything
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything went Ok
    *//*---------------------------------------------------------*/
   public static boolean refreshAllUsers ( ) {
      boolean rc              = false ;
      Vector  uniqueUserIds   = getChangedCachedUsers() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, check to see if we need to read the users        */
      /*   from the database, or if we can just use our cache      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( uniqueUserIds != null ) {
         int       i             = 0 ;
         Frame     tempFrame     = new Frame() ;
         StatusWin statusWin     = new StatusWin ( tempFrame ) ;


         statusWin.setText ( "Caching "+uniqueUserIds.size()+" users" ) ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our vector if need be                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( allUserVec == null )
            allUserVec = new Vector ( 600, 1 ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in all of the user ids                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( uniqueUserIds.size() > 0 ) {
            int    size          = uniqueUserIds.size() ;
            int    k             = 0 ;
            String userList      = null ;
            int    MAX_USER_READ = 200 ;                              
            int    userCount     = 0 ;

            statusWin.setMaxValue ( size ) ;
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Build the list of comma separated users                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rc = true ;
            while ( ( rc ) && ( i < size ) ) {

               userList="" ;
               userCount = 0 ;
               for ( k =i; k < i+MAX_USER_READ; k++ ) {
                  if ( k < size ) {
                     userList+="'"+(String)uniqueUserIds.elementAt(k)+"'" ;
                     userCount++ ;
                     if ( ( k < (i+(MAX_USER_READ-1)) ) && ( k < (size-1) ) )
                        userList+=",";
                  } else
                     k=i+MAX_USER_READ ;
               }

               statusWin.setText ( "Reading "+i+"-"+(i+userCount)+" of "+size+" users" ) ;
               statusWin.setValue ( i ) ;
               rc = refreshUsersJDBC ( userList ) ;
               i+=MAX_USER_READ ;
            }
         } else
            LogSystem.log ( 1, "Cannot read in World Wide user list." ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, cache it to disk                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc )
            CacheSystem.cacheObject ( "users.allUserVec", allUserVec ) ;


         statusWin.dispose() ;
         tempFrame.dispose() ;
      } else
         rc = true ; // We read the info from cache, so it is good

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* refreshUsers
    */
   /**
    *    This method will read in all user ids that
    *  begin with the startLetter passed. It will then
    *  add that user id to the UserSystem's list of ids
    *
    *  @param startLetter The start letter
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean refreshUsers ( String userList ) {
      boolean           rc         = false ;                                       

      SQLMethod         sqlMethod  = new SQLMethod ( DatabaseSystem.QUEST, "UserSystem.refreshUsers ()", 5 ) ;
      CallableStatement stmt       = sqlMethod.createStoredProc ( "QUEST.ReadUsers", 3 ) ;

      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_READING_ALL_USERS ) ;

      try {
         Vector retVec = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, userList.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            LogSystem.log ( 1, retRec.userError ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            retVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the vector is good, we have our data                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rc = true ;
            if ( ( retVec != null ) && ( retVec.size() > 0 ) ) {
               int i    = 0 ;
               int size = retVec.size() ;

               while ( i < size ) {
                  allUserVec.addElement ( retVec.elementAt(i) ) ;
                  i++ ;
               }
            }

         } /* else  */

         rc = true ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close( ) ;
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;

      return( rc ) ;
   } /* readAllUsers  */



   private static boolean refreshUsersJDBC ( String userList ) {
      boolean           rc         = false ;                                       
      Connection con = DatabaseSystem.getConnection ( DatabaseSystem.QUEST) ;


      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_READING_ALL_USERS ) ;

      try {
         Vector retVec = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our return object                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retVec = ReadUsers.readUsers ( con, userList ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the vector is good, we have our data                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rc = true ;
         if ( ( retVec != null ) && ( retVec.size() > 0 ) ) {
            int i    = 0 ;
            int size = retVec.size() ;

            while ( i < size ) {
               allUserVec.addElement ( retVec.elementAt(i) ) ;
               i++ ;
            }
         }

         rc = true ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      DatabaseSystem.releaseConnection ( DatabaseSystem.getDatabaseFromConnection ( con ) ) ;

      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;

      return( rc ) ;
   } /* readAllUsers  */





   /*------------------------------------------------------------*/
   /* getChangedCachedUsers
    */
   /**
    *    This method will try and read all users from our
    *  cached object, it will then try to read all changed users
    *
    *  <pre>
    *  The following pseudo-code describes this method:
    *  1. Get last time the cache was set and read in the cached user ids
    *  2. Lookup in DB/2 for all changes since that date
    *  3. If there were changes, Generate a Vector of user ids that have changed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return A vector of all changed user ids, NULL if nothing has changed
    *//*---------------------------------------------------------*/
   private static Vector getChangedCachedUsers () {
      String    timestamp     = CacheSystem.getCacheTime ( "users.allUserVec" ) ;             //Get the Timestamp of the last cache
      Vector    cacheVec      = (Vector) CacheSystem.getCachedObject ( "users.allUserVec" ) ; //Get the Cached User Ids
      SQLMethod sqlMethod     = new SQLMethod ( DatabaseSystem.QUEST, "UserSystem.getChangedCachedUsers", 5 ) ;
      Vector    uniqueUserVec = null ;                                          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our vector from the cache, even if the cache     */
      /* is not there                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      allUserVec = cacheVec ;


      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( Str.getStr ( STR_SYNC_USER_CACHE )) ;
      try {
         Statement stmt     = sqlMethod.createStatement() ;
         ResultSet rs       = stmt.executeQuery ( "SELECT DISTINCT USERID FROM TIGRIS.USERID WHERE CHANGEDTIME > '"+timestamp+"'" ) ;                                            


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the rs.next method returns TRUE, there is new info   */
         /*   if it returns false, we can use our cache since there   */
         /*   is nothing new in the database.                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rs.next() ) {
            uniqueUserVec = new Vector ( 1, 1 ) ;                                      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Remove the user from our list and add it to the         */
            /* list of changed users. The user will be re-read back in   */
            /* with the changes after this method in the refreshAllUsers */
            /* method.                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            removeUserId ( rs.getString(1).trim() ) ;
            uniqueUserVec.addElement ( rs.getString(1).trim() ) ;                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through, adding each one                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( rs.next() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Remove the user from our list and add it to the         */
               /* list of changed users. The user will be re-read back in   */
               /* with the changes after this method in the refreshAllUsers */
               /* method.                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               removeUserId ( rs.getString(1).trim() ) ;
               uniqueUserVec.addElement ( rs.getString(1).trim() ) ;
            }

         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, no changes, do not reread, just use the cache       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            uniqueUserVec = null ;
            allUserVec = cacheVec ;
         }

         rs.close() ;
         stmt.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;

      return( uniqueUserVec ) ;
   }






   /*------------------------------------------------------------*/
   /* getAllUserVec
    */
   /**
    *    This method will return a vector containing
    *  UserRec objects. Each object represents 1
    *  user in the database
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector getAllUserVec () {
      return( allUserVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getNameFromUserId
    */
   /**
    *    This method will return the full name of a
    *  user from the User Id passed in
    *  
    *
    *  @param userId The user id to search for
    *  @param None
    *  @param None
    *
    *  @return String The name
    *//*---------------------------------------------------------*/
   public static String getNameFromUserId ( String userId ) {
      UserRec userRec = getUserRecFromUserId ( userId ) ;

      if ( userRec != null )
         return( userRec.getName() ) ;
      return( "?" ) ;
   } /* getNameFromUserId  */


   /*------------------------------------------------------------*/
   /* getUserIdFromName
    */
   /**
    *    This method will return the full name of a
    *  user from the User Id passed in
    *  
    *
    *  @param name The name to search for
    *  @param None
    *  @param None
    *
    *  @return String The name
    *//*---------------------------------------------------------*/
   public static String getUserIdFromName ( String name ) {
      String userId = null ;

      if ( allUserVec != null ) {
         int     i       = 0 ; 
         int     size    = allUserVec.size() ;
         UserRec userRec = null ;

         while ( ( name == null ) && ( i < size ) ) {
            userRec = (UserRec) allUserVec.elementAt(i ) ;
            if ( userRec.getName().equals ( name ) )
               userId = userRec.getUserId() ;
            i++ ;
         }
      }


      return( userId ) ;
   } /* getUserIdFromName  */





   /*------------------------------------------------------------*/
   /* getBrandProgramManager
    */
   /**
    *    This method will return a UserRec representing
    *  the Brand program manager for the brand
    *  passed in and the brand manager type passed in
    *
    *  @param brandInd The brand ind that is in TIGRIS.TYPEBRAND
    *  @param typeBrandMgr Either DOC or OA
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static UserRec getBrandProgramManager ( int brandInd, String typeBrandMgr ) {
      UserRec retRec = null ;

      // if ( allUserVec != null ) {
      //    int     i       = 0 ;
      //    int     size    = allUserVec.size() ;
      //    UserRec userRec = null ;

      //    while ( ( i < size ) && ( retRec == null ) ) {
      //       userRec = (UserRec) allUserVec.elementAt(i) ;
      //       if ( userRec.getBrandInd() == brandInd ) {
      //          if ( ( userRec.getTypeBrandMgr() != null ) &&
      //               ( userRec.getTypeBrandMgr().equals ( typeBrandMgr ) ) )
      //             retRec = userRec ;
      //       }
      //       i++ ;
      //    }
      // }

      return( retRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getBrandProgramManager
    */
   /**
    *    This method will return the Document
    *  brand program manager for a brand
    *  
    *
    *  @param brandInd The brand indicator
    *  @param None
    *  @param None
    *
    *  @return String userId String
    *//*---------------------------------------------------------*/
   public static String getBrandProgramManager ( int brandInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getBrandProgramManager ( "+brandInd+" )", 5 ) ;
      String    userId    = null ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT OWNER FROM TIGRIS.TYPEBRAND WHERE BRANDIND = "+brandInd+" FOR FETCH ONLY" ) ;

         if ( rs.next() )
            userId = rs.getString(1) ;

         if ( userId != null )
            userId = userId.trim() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
      return( userId ) ;
   }


   /*------------------------------------------------------------*/
   /* getUserRecFromName
    */
   /**
    *    This method will return a UserRec representing
    *  the name passed in
    *
    *  @param name The name of the user
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static UserRec getUserRecFromName ( String name ) {
      UserRec retRec = null ;

      if ( allUserVec != null ) {
         int     i       = 0 ;
         int     size    = allUserVec.size() ;
         UserRec userRec = null ;

         while ( ( i < size ) && ( retRec == null ) ) {
            userRec = (UserRec) allUserVec.elementAt(i) ;
            if ( userRec.getName().equals ( name ) )
               retRec = userRec ;
            i++ ;
         }
      }

      return( retRec ) ;
   }


   /*------------------------------------------------------------*/
   /* getUserRecFromUserId
    */
   /**
    *    This method will return a UserRec representing
    *  the user id passed in
    *
    *  @param userId
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static UserRec getUserRecFromUserId ( String userId ) {
      UserRec retRec = null ;

      if ( allUserVec != null ) {
         int     i       = 0 ;
         int     size    = allUserVec.size() ;
         UserRec userRec = null ;

         while ( ( i < size ) && ( retRec == null ) ) {
            userRec = (UserRec) allUserVec.elementAt(i) ;
            if ( userRec.getUserId().equals ( userId ) )
               retRec = userRec ;
            i++ ;
         }
      }

      return( retRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getManagerList
    */
   /**
    *    This method will return a vector of UserRecs.
    *  Each UserRec represents a manager
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getManagerList () {
      Vector retVec = new Vector ( 1, 1 ) ;

      if ( allUserVec != null ) {
         int i    = 0 ;
         int size = allUserVec.size() ;

         while ( i < size ) {
            if ( ((UserRec)allUserVec.elementAt(i)).isManager() ) {
               retVec.addElement ( allUserVec.elementAt(i) ) ;
            }
            i++ ;
         }
      }

      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* addUserRec
    */
   /**
    *    This method will add a user record to our
    *  list of all users
    *  
    *
    *  @param userRec The UserRec to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addUserRec ( UserRec userRec ) {
      if ( allUserVec != null ) 
         allUserVec.addElement ( userRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getUserCountries
    */
   /**
    *    This method will return a list of countries
    *  that a user is associated to
    *  
    *
    *  @param userId The User Id to get the countries for
    *  @param None
    *  @param None
    *
    *  @return Vector of TypeCountryCodeRecs
    *//*---------------------------------------------------------*/
   public static Vector getUserCountries ( String userId ) {
      return( getUserCountries ( getUserRecFromUserId ( userId ) ) ) ;
   }



   /*------------------------------------------------------------*/
   /* getUserCountries
    */
   /**
    *    This method will return a list of countries
    *  that a user is associated to
    *  
    *
    *  @param userRec The UserRec associated to the user
    *  @param None
    *  @param None
    *
    *  @return Vector of TypeCountryCodeRecs
    *//*---------------------------------------------------------*/
   public static Vector getUserCountries ( UserRec userRec ) {

      if ( userRec.getUserId().equals ( getUserId() ) )
         return( getLoggedUserCountries() ) ;

      if ( userRec != null ) {
         int[]    countries   = userRec.getCountries() ;
         Vector   retVec      = new Vector ( 1, 1 ) ;
         TypeList typeList    = TypeList.getInstance() ;

         if ( countries != null ) {

            for ( int i = 0 ; i < countries.length; i++ ) {
               retVec.addElement ( typeList.objectFromInd ( countries[i], TypeList.TYPE_COUNTRYCODE ) ) ;
            }
         }

         return( retVec ) ;
      }
      return( null ) ;
   }






   /*------------------------------------------------------------*/
   /* getLoggedUserCountries
    */
   /**
    *    This method will return the countries that
    *  belong to the user logged in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of TypeCountryCodeRecs
    *//*---------------------------------------------------------*/
   public static Vector getLoggedUserCountries () {

      if ( loggedUser != null ) {
         int[]    countries   = loggedUser.getCountries() ;
         Vector   retVec      = new Vector ( 1, 1 ) ;
         TypeList typeList    = TypeList.getInstance() ;

         if ( countries != null ) {

            for ( int i = 0 ; i < countries.length; i++ ) {
               retVec.addElement ( typeList.objectFromInd ( countries[i], TypeList.TYPE_COUNTRYCODE ) ) ;
            }
         }

         return( retVec ) ;
      }
      return( null ) ;
   }



   /*------------------------------------------------------------*/
   /* setUserAddress
    */
   /**
    *    This method will set the user's I/P address
    *  in the database
    *  
    *
    *  @param ipAddress The IP Address to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setUserAddress ( String ipAddress ) {
      // Uncomment this when Chat is working SQLMethod sqlMethod = null ;

      // Uncomment this when Chat is working try {
      // Uncomment this when Chat is working    sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setUserAddress ( "+ipAddress+" )", 5 ) ;
      // Uncomment this when Chat is working    Statement stmt      = sqlMethod.createStatement () ;

      // Uncomment this when Chat is working    stmt.executeUpdate ( "UPDATE TIGRIS.USERID SET IPADDR = '"+ipAddress+"' WHERE USERID = '"+getUserId()+"'" ) ;

      // Uncomment this when Chat is working    sqlMethod.close() ;
      // Uncomment this when Chat is working } catch ( Exception e ) {
      // Uncomment this when Chat is working    sqlMethod.rollBack() ;
      // Uncomment this when Chat is working    sqlMethod.close() ;
      // Uncomment this when Chat is working }

   }





   /*------------------------------------------------------------*/
   /* getUserCountries
    */
   /**
    *    This method will return all of the countries
    *  that the user logged in has associated to them
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of TypeCountryCodeRecs
    *//*---------------------------------------------------------*/
   public static Vector getUserCountries () {
      return( getUserCountries ( getUserId() ) ) ;
   }





   /*------------------------------------------------------------*/
   /* hasCountries
    */
   /**
    *    This method will return TRUE if the user is
    *  assigned to any country passed in the vector.
    *  
    *
    *  @param countryVec A Vector of TypeCountryCodeRecs
    *  @param None
    *  @param None
    *
    *  @return TRUE if the user has at least 1 of the countries.
    *//*---------------------------------------------------------*/
   public static boolean hasCountries ( Vector countryVec ) {
      boolean has = false ;

      if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = countryVec.size() ;

         while ( ( !has ) && ( i < size ) ) {
            has = hasCountry ( (TypeCountryCodeRec)countryVec.elementAt(i) ) ;
            i++ ;
         }

      } else
         has = true ;

      return( has ) ;
   }



   /*------------------------------------------------------------*/
   /* hasCountry
    */
   /**
    *    This method will determine if a UserId has
    *  the authority for the specfic country passed in
    *  
    *
    *  @param countryCodeRec The TypeCountryCodeRec to compare against
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean hasCountry ( TypeCountryCodeRec countryCodeRec ) {
      return( hasCountry ( countryCodeRec.getInd() ) ) ;
   }


   /*------------------------------------------------------------*/
   /* hasCountry
    */
   /**
    *    This method will determine if a UserId has
    *  the authority for the specfic country passed in
    *  
    *
    *  @param countryCodeInd The CountryCodeInd as defined in TIGRIS.TYPECOUNTRYCODE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean hasCountry ( int countryCode ) {
      return( loggedUser.hasCountry ( countryCode ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getUserBrands
    */
   /**
    *    This method will return a list of brands
    *  that the user is assigned to edit
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of TypeBrandRec objects
    *//*---------------------------------------------------------*/
   public static Vector getUserBrands () {
      Vector authBrands = new Vector ( 1, 1 ) ;
      Vector brands     = Brands.getBrands() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, loop through the list of brands and check each      */
      /* one.                                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( brands != null ) && ( brands.size() > 0 ) ) {
         int i    = 0 ;
         int size = brands.size() ;

         while ( i < size ) {
            if ( hasBrandAuth ( ((TypeBrandRec)brands.elementAt(i)).getInd() ) )
               authBrands.addElement ( brands.elementAt(i) ) ;

            i++ ;
         }
      }


      return( authBrands ) ;
   }





   /*------------------------------------------------------------*/
   /* getUniqueUserIdsFromDatabase
    */
   /**
    *    This method will read in all unique user ids
    *  from the database and returns them in a vector
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static Vector getUniqueUserIdsFromDatabase () {
      Vector readVec = null ;

      SQLMethod         sqlMethod  = new SQLMethod ( DatabaseSystem.QUEST, "UserSystem.getUniqueUserIdsFromDatabase ()", 5 ) ;
      CallableStatement stmt       = sqlMethod.createStoredProc ( "QUEST.ReadUsers", 3 ) ;

      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_READING_ALL_USERS ) ;

      try {
         Vector retVec = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, new String ( "UNIQUE_USERS" ).getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            LogSystem.log ( 1, retRec.userError ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            retVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the vector is good, we have our data                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            readVec = retVec ;

         } /* else  */

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close( ) ;
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;

      return( readVec ) ;
   }





   /*------------------------------------------------------------*/
   /* removeUserId
    */
   /**
    *    This method will remove a user's record
    *  from our memory. This does not remove a 
    *  user id from the database.
    *
    *  @param userId The User Id to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void removeUserId ( String userId ) {

      if ( ( userId != null ) && ( allUserVec != null ) ) {
         UserRec userRec = getUserRecFromUserId ( userId ) ;

         if ( userRec != null )
            allUserVec.removeElement ( userRec ) ;
      }
   }
} /* UserSystem */


