package com.ibm.nzna.projects.qit.app ;    
/*-----------------------------------------------------------*/
/*   WindowSystem                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WindowSystem                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.gui.* ;    
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
/**
 * The WindowSystem is the root of all GUI Applets in the Quest Application.
 * It is responsible for mainting all open AppDefaultWins and the creation
 * of QuestPanels. To create a new panel, simply call the 'createPanel' method.
 *
 * 
 * @version 1.0, 3/5/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WindowSystem                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class WindowSystem extends Object {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Vector     appDefaultWinVec     = null ;
   private static MainWindow mainWindow           = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will simply create an instance of
    *  the WindowSystem. It should only be called
    *  once in the Application's life.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WindowSystem () {         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our vectors                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      appDefaultWinVec = new Vector ( 1, 1 ) ;                      

   }





   /*------------------------------------------------------------*/
   /* createPanel
    */
   /**
    *    This method will create a new panel. If the application
    *  is running in MDI mode, a new window will be created
    *  for the Panel, otherwise a panel will be added to the MainWin instance.
    *
    *  @param questPanel The new Quest Panel to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void createPanel ( QuestPanel questPanel ) {
      try {
         boolean mdiMode = PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ;                          

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If in MDI mode, create a new window                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( mdiMode ) {
            AppDefaultWin newWin = new AppDefaultWin ( questPanel ) ;

            newWin.setSize ( 640, 480 ) ;
            WinUtil.centerWindow ( newWin ) ;

            appDefaultWinVec.addElement ( newWin ) ;
            newWin.setVisible ( true ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, add this panel to the MainWin                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            MainWindow.getInstance().addPanel ( questPanel ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will close all open windows. Each
    *  open window is responsible for closing all panels
    *  inside of that window
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Shutdown completed
    *//*---------------------------------------------------------*/
   public static boolean shutdown () {
      int i            = 0 ;
      int size         = 0 ;
      boolean shutdown = true ;                                         

      if ( appDefaultWinVec != null )
         size = appDefaultWinVec.size() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close all open AppDefaultWins, except the main win      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( shutdown ) && ( i < size ) ) {
         ((AppDefaultWin)appDefaultWinVec.elementAt(i)).dispose() ;
         shutdown = ((AppDefaultWin)appDefaultWinVec.elementAt(i)).isDisposed() ;
         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, close the MainWindow                                */               
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( shutdown ) {
         if ( mainWindow != null ) {
            mainWindow.dispose() ;
            shutdown = mainWindow.isDisposed() ;
         } else
            shutdown = true ;
      }
      return( shutdown ) ;
   }






   /*------------------------------------------------------------*/
   /* findQuestPanel
    */
   /**
    *    This method will traverse through all open windows. Each
    *  open window will be asked to look through it's open panels.
    *  Each open panel's equals() method will be called with the object passed to this method.
    *
    *  @param compareObject The object to call equals() on each QuestPanel with.
    *  @param None
    *  @param None
    *
    *  @return QuestPanel NULL if not found
    *//*---------------------------------------------------------*/
   public static QuestPanel findQuestPanel ( Object compareObject ) {
      QuestPanel returnPanel = null ;

      if ( mainWindow != null )
         returnPanel = mainWindow.findQuestPanel ( compareObject ) ;


      if ( ( appDefaultWinVec != null ) && 
           ( returnPanel      == null ) ) {
         int i       = 0 ;
         int size    = appDefaultWinVec.size() ;

         while ( ( i < size ) && ( returnPanel == null ) ) {
            returnPanel = ((AppDefaultWin)appDefaultWinVec.elementAt(i)).findQuestPanel ( compareObject ) ;
            i++ ;
         }
      }

      return( returnPanel ) ;
   } /* findQuestPanel  */






   /*------------------------------------------------------------*/
   /* showPanel
    */
   /**
    *    This method will make sure whatever window
    *  contains the QuestPanel passed is showing and up
    *  front. It also makes sure that the QuestPanel is showing on the AppDefaultWin
    *
    *  @param questPanel The QuestPanel to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void showPanel ( QuestPanel questPanel ) {
      boolean mdiMode = PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ;                          

      if ( ( mainWindow != null ) && 
           ( mainWindow.applicationPanelFromQuestPanel ( questPanel ).containsQuestPanel ( questPanel ) ) ) {
         mainWindow.setVisible ( true ) ;
         mainWindow.toFront() ;
         mainWindow.requestFocus() ;
         mainWindow.setPanelFocus ( questPanel ) ;
      }

      else if ( appDefaultWinVec != null ) {
         int           i       = 0 ;
         int           size    = appDefaultWinVec.size() ;
         AppDefaultWin win     = null ;
         boolean       found   = false ;

         while ( ( i < size ) && ( !found ) ) {
            win = (AppDefaultWin)appDefaultWinVec.elementAt(i) ;

            if ( ( win != null ) && 
                 ( win.applicationPanelFromQuestPanel ( questPanel ).containsQuestPanel ( questPanel ) ) ) {
               win.toFront() ;
               win.requestFocus() ;
               win.setPanelFocus ( questPanel ) ;
               found = true ;
            }
            i++ ;
         } /* while  */
      } /* else  */
   } /* showPanel  */





   /*------------------------------------------------------------*/
   /* createMainWindow
    */
   /**
    *    This method will create the application's main
    *  window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void createMainWindow ( ) {
      if ( mainWindow == null )
         mainWindow = new MainWindow () ;
   }
} /* WindowSystem */
