package com.ibm.nzna.projects.qit.app.appProp ;
/*-----------------------------------------------------------*/
/*   ApplicationPropertyDlg                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ApplicationPropertyDlg              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing. * ;
import java.awt.* ;           
import java.awt.event.* ;           
/**
 * This dialog holds a notebook with pages for each Application Property. 
 *
 *
 * @version 1.0, 1/23/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ApplicationPropertyDlg                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class ApplicationPropertyDlg extends JDialog implements ActionListener, AppConst, PropertyListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTabbedPane notebook        = null ;
   private DButton     pb_OK           = null ;
   private DButton     pb_CANCEL       = null ;
   private DButton     pb_APPLY        = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Pages                                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private TaskBarPage       taskBarPage     = null ;
   private LanguagePage      languagePage    = null ;
   private InternetPage      internetPage    = null ;
   private LookFeelPage      lookFeelPage    = null ;
   private AppletsPage       appletPage      = null ;
   private SelectCountryPage countryPage     = null ;
   //private FontPage          fontPage        = null ;
   private WordProcPage      wordProcPage    = null ;
   private SkinsPage         skinsPage       = null ;
   private GeneralPage       generalPage     = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the
    *  application properties. It will not return
    *  until the user closes the window
    *
    *  @param parent The parent window
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ApplicationPropertyDlg ( AppDefaultWin parent ) {
      super ( parent, Str.getStr ( STR_PROPERTIES )+"-"+Str.getStr ( STR_APPLICATION ), true ) ;
      ButtonPanel buttonPanel = new ButtonPanel () ;

      setSize ( 400, 500 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      notebook  = new JTabbedPane () ;
      pb_OK     = new DButton ( ) ;
      pb_APPLY  = new DButton ( ) ;
      pb_CANCEL = new DButton ( ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the panels                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parent.setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
      parent.setValueMax ( 11 ) ;
      notebook.addTab ( "df.", generalPage  = new GeneralPage() ) ;    
      parent.setValue ( 1 ) ;
      notebook.addTab ( "df.", taskBarPage  = new TaskBarPage() ) ;    
      parent.setValue ( 2 ) ;
      notebook.addTab ( ".df", languagePage = new LanguagePage() ) ;
      parent.setValue ( 3 ) ;
      notebook.addTab ( "df.", lookFeelPage = new LookFeelPage() ) ;
      parent.setValue ( 4 ) ;
      notebook.addTab ( "df.", countryPage  = new SelectCountryPage() ) ;
      parent.setValue ( 5 ) ;
      notebook.addTab ( "df.", appletPage   = new AppletsPage() ) ;
      parent.setValue ( 6 ) ;
      //notebook.addTab ( "df.", fontPage     = new FontPage() ) ;
      //parent.setValue ( 7 ) ;
      notebook.addTab ( "df.", internetPage = new InternetPage() ) ;
      parent.setValue ( 8 ) ;
      notebook.addTab ( "df.", wordProcPage = new WordProcPage() ) ;
      parent.setValue ( 9 ) ;
      notebook.addTab ( "df.", skinsPage    = new SkinsPage() ) ;
      parent.setValue ( 10 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_APPLY.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE,   this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LAF,        this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_BUTTONFONT, this ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;
      GUISystem.setPreferredButton ( pb_OK ) ;
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      GUISystem.setPreferredButton ( pb_APPLY ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( new BorderLayout (5,5) ) ;
      getContentPane().add ( notebook ) ;
      getContentPane().add ( buttonPanel, BorderLayout.SOUTH ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_APPLY ) ;
      buttonPanel.add ( pb_CANCEL ) ;                                 
      parent.setStatus ( 11 ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Make ourselves look like the user wants it              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;         


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, show ourselves                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parent.setStatus ( null ) ;
      parent.setValue ( 0 );
      setVisible ( true ) ;   
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a 
    *  property we are listening for has been
    *  changed
    *
    *  @param property The property
    *  @param value The new value
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE )
         setText() ;
      else if ( property == PropertySystem.PROPERTY_LAF )
         GUISystem.setLookAndFeel ( this ) ;
      else
         GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
   }




   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on our
    *  buttons and tabs. It is up to the
    *  pages themselves to set their text
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      pb_OK.setText ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL.setText ( Str.getStr ( STR_CANCEL ) ) ;
      pb_APPLY.setText ( Str.getStr ( STR_APPLY ) ) ;

      notebook.setTitleAt ( notebook.indexOfComponent ( generalPage  ),  Str.getStr ( STR_GENERAL ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( taskBarPage  ),  Str.getStr ( STR_TASKBAR ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( languagePage ),  Str.getStr ( STR_LANGUAGE ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( lookFeelPage ),  Str.getStr ( STR_LANDF ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( countryPage  ),  Str.getStr ( STR_COUNTRY ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( appletPage   ),  Str.getStr ( STR_QUESTAPPLET ) ) ;
      //notebook.setTitleAt ( notebook.indexOfComponent ( fontPage     ),  Str.getStr ( STR_FONTS ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( wordProcPage ),  Str.getStr ( STR_WORD_PROCESSOR ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( skinsPage    ),  Str.getStr ( STR_SKIN ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( internetPage ),  Str.getStr ( STR_INTERNET ) ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one
    *  of our buttons is pressed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( e.getSource() == pb_OK ) || ( e.getSource() == pb_APPLY ) ) {
         if ( ( taskBarPage.saveInfo() ) &&
              ( generalPage.saveInfo() ) &&
              ( languagePage.saveInfo()  ) &&
              ( lookFeelPage.saveInfo()  ) &&
              ( countryPage.saveInfo()  ) &&
              ( appletPage.saveInfo()  ) &&
              //( fontPage.saveInfo()  ) &&
              ( internetPage.saveInfo()  ) &&
              ( wordProcPage.saveInfo()  ) &&
              ( skinsPage.saveInfo() ) &&
              ( e.getSource() == pb_OK ) )
            dispose() ;

      }

      else
         dispose() ;
   }




   /*------------------------------------------------------------*/
   /* dispose
    */
   /**
    *    This method is subclassed to remove
    *  ourselves from the PropertySystem listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void dispose () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE,  this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_BUTTONFONT, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LAF, this ) ;

      generalPage.free() ;
      taskBarPage.free() ;
      internetPage.free() ;
      //fontPage.free() ;
      languagePage.free() ;
      lookFeelPage.free() ;
      countryPage.free() ;
      appletPage.free() ;
      wordProcPage.free() ;
      skinsPage.free() ;

      super.dispose() ;
   }
} /* ApplicationPropertyDlg */
