package com.ibm.nzna.projects.qit.app.appProp ;
/*-----------------------------------------------------------*/
/*   FontPage                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FontPage                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import java.util.* ;
import java.awt.* ;
import javax.swing.event.* ;
import javax.swing.* ; 
import java.awt.event.* ;
/**
 * This page allows the user to select different fonts for
 * different GUI controls.
 *
 * @version 1.0, 1/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FontPage                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class FontPage extends WizardStep implements ActionListener, AppConst, PropertyListener, ListSelectionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Temporary Hold Vars for properties                      */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Font defaultFont             = null ;
   private Font largeTitleFont          = null ;
   private Font smallTitleFont          = null ;
   private Font buttonFont              = null ;
   private Font listFont                = null ;
   private Font textFont                = null ;
   private Font smallFont               = null ;
   private Font panelTitleFont          = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_HELP      = null ;
   private JList       lb_CONTROLS  = null ;
   private JLabel      st_SAMPLE    = null ;
   private JScrollPane scr_CONTROLS = null ;
   private JComboBox   cb_FONTS     = null ;
   private JComboBox   cb_SIZES     = null ;
   private JCheckBox   ck_BOLD      = null ;
   private JCheckBox   ck_ITALIC    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Members                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector     fontVec        = null ;
   private String     currentControl = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our page
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FontPage () {                      
      Vector fontVec = new Vector ( 3, 1 ) ;
      Vector sizeVec = new Vector ( 6, 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our font vector                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      fontVec.addElement ( "Helvetica" ) ;
      fontVec.addElement ( "Serif" ) ;
      fontVec.addElement ( "Courier" ) ;

      for ( int i = 8; i < 32; i+=2 )
         sizeVec.addElement ( i+"" ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP      = new JLabel () ;
      lb_CONTROLS  = new JList () ;        
      st_SAMPLE    = new JLabel () ;
      cb_FONTS     = new JComboBox ( fontVec ) ;
      cb_SIZES     = new JComboBox ( sizeVec ) ;
      ck_BOLD      = new JCheckBox () ;
      ck_ITALIC    = new JCheckBox () ;
      scr_CONTROLS = new JScrollPane ( lb_CONTROLS ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;
      st_SAMPLE.setBorder ( GUISystem.loweredBorder ) ;
      lb_CONTROLS.setBorder ( GUISystem.loweredBorder ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
      lb_CONTROLS.addListSelectionListener ( this ) ;
      cb_SIZES.addActionListener           ( this ) ;
      cb_FONTS.addActionListener           ( this ) ;
      ck_BOLD.addActionListener            ( this ) ;
      ck_ITALIC.addActionListener          ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP ) ;
      add ( scr_CONTROLS ) ;       
      add ( cb_FONTS ) ;
      add ( cb_SIZES ) ;
      add ( ck_BOLD ) ;
      add ( ck_ITALIC ) ;
      add ( st_SAMPLE ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our text                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust our temp fonts                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setTempFonts() ;

   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       halfWidth = size.width/2 ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       y         = 110+rowHeight ;

      super.doLayout() ;

      st_HELP.setBounds ( 5, 5, size.width-10, 100 ) ;
      scr_CONTROLS.setBounds ( 5, 110, halfWidth-5, 100 ) ;
      cb_FONTS.setBounds ( halfWidth+5, 110, halfWidth-50, rowHeight ) ;
      cb_SIZES.setBounds ( size.width-40, 110, 35, rowHeight ) ;
      y+=rowHeight ;

      ck_BOLD.setBounds ( halfWidth+5, y, halfWidth-10, rowHeight ) ;
      y+=rowHeight ;

      ck_ITALIC.setBounds ( halfWidth+5, y, halfWidth-10, rowHeight ) ;

      st_SAMPLE.setBounds ( 5, 220, size.width-10, 50 ) ;

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to action events
    *  being fired.
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      st_SAMPLE.setFont ( fontFromGUI () ) ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set all of our
    *  text
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText ( ) {
      try {
         int      language   = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         Vector   controlVec = new Vector ( 1, 1 ) ;

         st_HELP.setText ( Str.getStr ( STR_FONT_SELECT_HELP ) ) ;

         controlVec.addElement ( Str.getStr ( STR_BUTTONS ) ) ;
         controlVec.addElement ( Str.getStr ( STR_LISTS ) ) ;
         controlVec.addElement ( Str.getStr ( STR_TITLES ) ) ;
         controlVec.addElement ( Str.getStr ( STR_TEXT ) ) ;
         controlVec.addElement ( Str.getStr ( STR_SMALL_TITLES ) ) ;
         controlVec.addElement ( Str.getStr ( STR_SMALL_FONT ) ) ;
         controlVec.addElement ( Str.getStr ( STR_PANEL_TITLE_FONT ) ) ;

         lb_CONTROLS.removeAll() ;
         lb_CONTROLS.setListData ( controlVec ) ;

         ck_BOLD.setText ( Str.getStr ( STR_BOLD_TEXT ) ) ;
         ck_ITALIC.setText ( Str.getStr ( STR_ITALIC_TEXT ) ) ;
         st_SAMPLE.setText ( Str.getStr ( STR_SAMPLE ) ) ;

      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when the
    *  list is selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      try {
         String control         = (String)lb_CONTROLS.getSelectedValue() ;                               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save the data if they switched positions                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( currentControl != null ) && ( control != currentControl ) )
            saveData () ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, switch the font controls                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         currentControl = control ;
         refreshData ( fontFromControl ( control ) ) ;

      } catch ( Exception e1 ) {
      }
   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save our properties
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {

      saveData () ;

      if ( !defaultFont.equals ( FontSystem.defaultFont ) )
         PropertySystem.put ( PropertySystem.PROPERTY_DEFAULTFONT, defaultFont ) ;
      if ( !largeTitleFont.equals ( FontSystem.largeTitleFont ) )
         PropertySystem.put ( PropertySystem.PROPERTY_LARGETITLEFONT, largeTitleFont ) ;
      if ( !smallTitleFont.equals ( FontSystem.smallTitleFont ) )
         PropertySystem.put ( PropertySystem.PROPERTY_SMALLTITLEFONT, smallTitleFont ) ;
      if ( !buttonFont.equals ( FontSystem.buttonFont ) )
         PropertySystem.put ( PropertySystem.PROPERTY_BUTTONFONT, buttonFont ) ;
      if ( !listFont.equals ( FontSystem.listFont ) )
         PropertySystem.put ( PropertySystem.PROPERTY_LISTFONT, listFont ) ;
      if ( !textFont.equals ( FontSystem.textFont ) )
         PropertySystem.put ( PropertySystem.PROPERTY_DEFAULTFONT, textFont ) ;
      if ( !smallFont.equals ( FontSystem.smallFont ) )
         PropertySystem.put ( PropertySystem.PROPERTY_SMALLFONT, smallFont ) ;
      if ( !panelTitleFont.equals ( FontSystem.panelTitleFont.getName() ) )
         PropertySystem.put ( PropertySystem.PROPERTY_PANELTITLEFONT, panelTitleFont ) ;

      setTempFonts() ;

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a
    *  property has been changed
    *  
    *
    *  @param property The property that was changed
    *  @param value The new value
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      setText() ;
   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will release our resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh the data on our
    *  controls to a specified font.
    *  
    *
    *  @param font The font to set to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData ( Font font ) {
      String  fontName = font.getName () ;
      int     size     = font.getSize () ;
      boolean bold     = font.isBold () ;
      boolean italic   = font.isItalic () ;


      cb_FONTS.setSelectedItem ( fontName ) ;
      ck_BOLD.setSelected ( bold ) ;
      ck_ITALIC.setSelected ( italic ) ;
      cb_SIZES.setSelectedItem ( size+"" ) ;

      st_SAMPLE.setFont ( font ) ;
   }




   /*------------------------------------------------------------*/
   /* fontFromControl
    */
   /**
    *    This method will return the font object from
    *  the control that is passed in
    *  
    *
    *  @param control The control
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Font fontFromControl ( String control ) {
      if ( control.equals ( Str.getStr ( STR_BUTTONS ) ) )
         return( buttonFont ) ;
      else if ( control.equals ( Str.getStr ( STR_LISTS ) ) )
         return( listFont ) ;
      else if ( control.equals ( Str.getStr ( STR_TEXT ) ) )
         return( textFont ) ;
      else if ( control.equals ( Str.getStr ( STR_TITLES ) ) )
         return( largeTitleFont ) ;
      else if ( control.equals ( Str.getStr ( STR_SMALL_TITLES ) ) )
         return( smallTitleFont ) ;
      else if ( control.equals ( Str.getStr ( STR_SMALL_FONT ) ) )
         return( smallFont ) ;
      else if ( control.equals ( Str.getStr ( STR_PANEL_TITLE_FONT ) ) )
         return( panelTitleFont ) ;


      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save the data that is
    *  up in the font selection controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveData () {
      if ( currentControl != null ) {
         if ( currentControl.equals ( Str.getStr ( STR_BUTTONS ) ) )
            buttonFont = fontFromGUI() ;

         else if ( currentControl.equals ( Str.getStr ( STR_LISTS ) ) )
            listFont = fontFromGUI() ;

         else if ( currentControl.equals ( Str.getStr ( STR_TEXT ) ) )
            textFont = fontFromGUI() ;

         else if ( currentControl.equals ( Str.getStr ( STR_TITLES ) ) )
            largeTitleFont = fontFromGUI() ;

         else if ( currentControl.equals ( Str.getStr ( STR_SMALL_TITLES ) ) )
            smallTitleFont = fontFromGUI() ;

         else if ( currentControl.equals ( Str.getStr ( STR_SMALL_FONT ) ) )
            smallFont = fontFromGUI() ;

         else if ( currentControl.equals ( Str.getStr ( STR_PANEL_TITLE_FONT ) ) ) {
            panelTitleFont = fontFromGUI() ;
         }

      }
   } /* saveData  */




   /*------------------------------------------------------------*/
   /* fontFromGUI
    */
   /**
    *    This method will return a Font object from
    *  what the GUI is displaying
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Font fontFromGUI () {
      Font newFont      = null ;
      int  newFontStyle = Font.PLAIN ;

      if ( ( ck_BOLD.isSelected() ) && ( ck_ITALIC.isSelected() ) )
         newFontStyle = Font.ITALIC | Font.BOLD ;

      else if ( ( !ck_BOLD.isSelected() ) && ( ck_ITALIC.isSelected() ) )
         newFontStyle = Font.ITALIC ;

      else if ( ( ck_BOLD.isSelected() ) && ( !ck_ITALIC.isSelected() ) )
         newFontStyle = Font.BOLD ;

      newFont = new Font ( cb_FONTS.getSelectedItem().toString(), 
                           newFontStyle,
                           new Integer ( cb_SIZES.getSelectedItem().toString() ).intValue() ) ;

      return( newFont ) ;
   }




   /*------------------------------------------------------------*/
   /* setTempFonts
    */
   /**
    *    This method will set the temporary fonts
    *  to the fonts that are currently in the FontSystem
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setTempFonts () {
      defaultFont             = new Font ( FontSystem.defaultFont.getName(), FontSystem.defaultFont.getStyle(), FontSystem.defaultFont.getSize() ) ;
      largeTitleFont          = new Font ( FontSystem.largeTitleFont.getName(), FontSystem.largeTitleFont.getStyle(), FontSystem.largeTitleFont.getSize() ) ;
      smallTitleFont          = new Font ( FontSystem.smallTitleFont.getName(), FontSystem.smallTitleFont.getStyle(), FontSystem.smallTitleFont.getSize() ) ;
      buttonFont              = new Font ( FontSystem.buttonFont.getName(), FontSystem.buttonFont.getStyle(), FontSystem.buttonFont.getSize() ) ;
      listFont                = new Font ( FontSystem.listFont.getName(), FontSystem.listFont.getStyle(), FontSystem.listFont.getSize() ) ;
      textFont                = new Font ( FontSystem.textFont.getName(), FontSystem.textFont.getStyle(), FontSystem.textFont.getSize() ) ;
      smallFont               = new Font ( FontSystem.smallFont.getName(), FontSystem.smallFont.getStyle(), FontSystem.smallFont.getSize() ) ;
      panelTitleFont          = new Font ( FontSystem.panelTitleFont.getName(), FontSystem.panelTitleFont.getStyle(), FontSystem.panelTitleFont.getSize() ) ;

   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title in a Wizard
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_FONT_SELECT_HELP_TITLE ) ) ;
   }
} /* FontPage */
