package com.ibm.nzna.projects.qit.app.appProp ;
/*-----------------------------------------------------------*/
/*   InternetPage                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: InternetPage                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.help.* ;     
import com.ibm.nzna.projects.qit.gui.* ;     
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
/**
 * This page belongs in the ApplicationPropertyDlg's notebook. It holds 
 * the internet settings for the tool. Basically this is the socks server config.
 *
 * @version 1.0, 1/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   InternetPage                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class InternetPage extends WizardStep implements PropertyListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Components                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_HELP          = null ;
   private JLabel      st_SOCKS_SERVER  = null ;
   private JLabel      st_SOCKS_PORT    = null ;
   private JTextField  ef_SOCKS_SERVER  = null ;
   private JTextField  ef_SOCKS_PORT    = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the page and all GUI Components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public InternetPage() {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP         = new JLabel () ;
      st_SOCKS_SERVER = new JLabel () ;
      st_SOCKS_PORT   = new JLabel () ;
      ef_SOCKS_SERVER = new JTextField ( "" ) ;
      ef_SOCKS_PORT   = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC ), "1080", 5 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;
      setText() ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI to our panel                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP ) ;
      add ( st_SOCKS_SERVER ) ;
      add ( ef_SOCKS_SERVER ) ;
      add ( ef_SOCKS_PORT ) ;
      add ( st_SOCKS_PORT ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our GUI Controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      st_HELP.setBounds ( 5, 5, size.width-5, 65 ) ;
      y+=70 ;

      st_SOCKS_SERVER.setBounds ( 5, y, 150, rowHeight ) ;
      ef_SOCKS_SERVER.setBounds ( 155, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_SOCKS_PORT.setBounds ( 5, y, 150, rowHeight ) ;
      ef_SOCKS_PORT.setBounds ( 155, y, 200, rowHeight ) ;
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has been changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE )
         setText() ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on all
    *  of our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText ( ) {
      st_HELP.setText   ( Str.getStr ( STR_INTERNET_PAGE_HELP ) ) ;
      st_SOCKS_SERVER.setText ( Str.getStr ( STR_SOCKS_SERVER ) ) ;
      st_SOCKS_PORT.setText ( Str.getStr ( STR_SOCKS_PORT ) ) ;

   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refesh out list
    *  of Quest Applets
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      try {
         String socksServer = PropertySystem.getString ( PropertySystem.PROPERTY_SOCKS_SERVER ) ;
         String socksPort   = PropertySystem.getString ( PropertySystem.PROPERTY_SOCKS_PORT ) ;

         if ( socksServer != null )
            ef_SOCKS_SERVER.setText ( socksServer ) ;
         if ( socksPort != null )
            ef_SOCKS_PORT.setText ( socksPort ) ;
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will make the final save
    *  for all operations that have been
    *  made on this panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      String socksServer = ef_SOCKS_SERVER.getText() ;
      String socksPort   = ef_SOCKS_PORT.getText() ;

      try {
         if ( ( socksServer != null ) &&
              ( socksPort   != null ) && 
              ( socksServer.length() > 0 ) &&
              ( socksPort.length() > 0 ) ) {
            Properties p = System.getProperties();

            PropertySystem.put ( PropertySystem.PROPERTY_SOCKS_SERVER, socksServer ) ;
            PropertySystem.put ( PropertySystem.PROPERTY_SOCKS_PORT, socksPort ) ;


            p.put("socksProxyHost", socksServer);
            p.put("socksProxyPort", socksPort );
            System.setProperties(p) ;

         } else {
            PropertySystem.put ( PropertySystem.PROPERTY_SOCKS_SERVER, "" ) ;
            PropertySystem.put ( PropertySystem.PROPERTY_SOCKS_PORT, "" ) ;
         }
      } catch ( Exception e ) {
      }

      return( true ) ;

   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will release resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title that is shown
    *  in a wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_INTERNET_PAGE_HELP_TITLE ) ) ;
   }

} /* InternetPage */
