package com.ibm.nzna.projects.qit.app.appProp ;
/*-----------------------------------------------------------*/
/*   LanguagePage                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LanguagePage                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.type.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import java.util.* ;
import java.awt.* ;
import javax.swing.event.* ;
import javax.swing.* ; 
/**
 * This page allows the user to select a different language 
 *
 *
 * @version 1.0, 1/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LanguagePage                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class LanguagePage extends WizardStep implements AppConst, ListSelectionListener, PropertyListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel  st_HELP      = new JLabel () ;
   private JList   lb_LANGUAGES = new JList () ;
   private Vector  langVec      = null ;
   private boolean changed      = false ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our page
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LanguagePage () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP      = new JLabel () ;
      lb_LANGUAGES = new JList () ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;
      lb_LANGUAGES.setBorder ( GUISystem.loweredBorder ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP ) ;
      add ( lb_LANGUAGES ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our text                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      lb_LANGUAGES.addListSelectionListener ( this ) ;                  

   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;

      st_HELP.setBounds ( 5, 5, size.width-10, 150 ) ;
      lb_LANGUAGES.setBounds ( 45, 160, size.width-90, 150 ) ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set all of our
    *  text
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText ( ) {
      try {
         int language = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;

         langVec = new Vector ( 10, 1 ) ;
         langVec.addElement ( Str.getStr ( STR_ENGLISH ) ) ;
         langVec.addElement ( Str.getStr ( STR_FRENCH ) ) ;
         langVec.addElement ( Str.getStr ( STR_GERMAN ) ) ;          
         langVec.addElement ( Str.getStr ( STR_ITALIAN ) ) ;
         langVec.addElement ( Str.getStr ( STR_SPANISH ) ) ;


         st_HELP.setText ( Str.getStr ( STR_LANGUAGE_SELECT_HELP ) ) ;
         lb_LANGUAGES.removeAll() ;
         lb_LANGUAGES.setListData ( langVec ) ;

                        
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Select the correct language                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( language ) {
            case TypeList.LANGUAGE_ENGLISH :
               lb_LANGUAGES.setSelectedValue ( Str.getStr ( STR_ENGLISH ), true ) ;
               break ;

            case TypeList.LANGUAGE_FRENCH :
               lb_LANGUAGES.setSelectedValue ( Str.getStr ( STR_FRENCH ), true ) ;
               break ;

            case TypeList.LANGUAGE_ITALIAN :
               lb_LANGUAGES.setSelectedValue ( Str.getStr ( STR_ITALIAN ), true ) ;
               break ;

            case TypeList.LANGUAGE_SPANISH :
               lb_LANGUAGES.setSelectedValue ( Str.getStr ( STR_SPANISH ), true ) ;
               break ;

            case TypeList.LANGUAGE_GERMAN :
               lb_LANGUAGES.setSelectedValue ( Str.getStr ( STR_GERMAN ), true ) ;
               break ;

         }

      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when the
    *  list is selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( !e.getValueIsAdjusting() ) {
         try {
            String language        = (String)lb_LANGUAGES.getSelectedValue() ;
            int    currentLanguage = ((Integer)PropertySystem.get ( PropertySystem.PROPERTY_LANGUAGE )).intValue() ;

            changed = true ;

            if ( language.equals ( Str.getStr ( STR_ENGLISH ) ) )
               Str.switchLanguage ( "english" ) ;
            else if ( language.equals ( Str.getStr ( STR_FRENCH ) ) )
               Str.switchLanguage ( "french" ) ;
            else if ( language.equals ( Str.getStr ( STR_ITALIAN ) ) )
               Str.switchLanguage ( "italian" ) ;
            else if ( language.equals ( Str.getStr ( STR_SPANISH ) ) )
               Str.switchLanguage ( "spanish" ) ;
            else if ( language.equals ( Str.getStr ( STR_GERMAN ) ) )
               Str.switchLanguage ( "german" ) ;
            else if ( language.equals ( Str.getStr ( STR_JAPANESE ) ) )
               Str.switchLanguage ( "japanese" ) ;
            else if ( language.equals ( Str.getStr ( STR_CHINESE ) ) )
               Str.switchLanguage ( "chinese" ) ;

            st_HELP.setText ( Str.getStr ( STR_LANGUAGE_SELECT_HELP ) ) ;

            Str.switchLanguage ( currentLanguage ) ;
         } catch ( Exception e1 ) {
            LogSystem.log ( 1 ,e1, false ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save our properties
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {

      try {
         String          language        = (String)lb_LANGUAGES.getSelectedValue() ;
         int             currentLanguage = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         TypeLanguageRec langRec         = null ;
         QITTypeList     typeList        = (QITTypeList) QITTypeList.getInstance() ;

         if ( ( language != null ) && ( changed ) ) {
            String databaseLanguage = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, pull out the english description for the database*/
            /*                                                           */
            /* Please note that this databaseLanguage string MUST match  */
            /* exactly what is in the 'DESCRIPT' column in the           */
            /* TIGRIS.TYPELANGUAGE table!!!                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( language.equals ( Str.getStr ( STR_ENGLISH ) ) )
               databaseLanguage = "English" ;
            else if ( language.equals ( Str.getStr ( STR_FRENCH ) ) )
               databaseLanguage = "French" ;
            else if ( language.equals ( Str.getStr ( STR_ITALIAN ) ) )
               databaseLanguage = "Italian" ;
            else if ( language.equals ( Str.getStr ( STR_SPANISH ) ) )
               databaseLanguage = "Spanish" ;
            else if ( language.equals ( Str.getStr ( STR_GERMAN ) ) )
               databaseLanguage = "German" ;
            else if ( language.equals ( Str.getStr ( STR_JAPANESE ) ) )
               databaseLanguage = "Japanese" ;
            else if ( language.equals ( Str.getStr ( STR_CHINESE ) ) )
               databaseLanguage = "Chinese" ;

            langRec = (TypeLanguageRec)  typeList.objectFromString ( databaseLanguage, typeList.TYPE_LANGUAGE ) ;


            if ( langRec.getInd() != currentLanguage )
               UserSystem.setLanguage ( langRec.getInd() ) ;
            changed =false ;
         }
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a
    *  property has been changed
    *  
    *
    *  @param property The property that was changed
    *  @param value The new value
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      setText() ;
   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will release our resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
   }

} /* LanguagePage */
