package com.ibm.nzna.projects.qit.app.appProp ;
/*-----------------------------------------------------------*/
/*   LookFeelPage                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LookFeelPage                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import java.awt.* ;
import java.util.* ;
import javax.swing.* ;
import java.awt.event.* ;
/**
 * This page goes in the ApplicationPropertyDlg notebook. It allows the user 
 * to select the Swing Look and Feel, to turn off/on MDI mode and to turn on/off
 * Anti-Aliasing.
 *
 * @version 1.0, 1/24/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LookFeelPage                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class LookFeelPage extends WizardStep implements ActionListener, AppConst {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Controls                                                */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private JLabel    st_LANDF         = null ;
    private JList     lb_LANDF         = null ;
    private JCheckBox ck_MDI           = null ;
    private JCheckBox ck_ANTIALIAS     = null ;





    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will initialize our panel, including
     *  attaching all GUI components
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public LookFeelPage () {     

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create the GUI                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        st_LANDF     = new JLabel (  ) ;        
        lb_LANDF     = new JList () ;
        ck_MDI       = new JCheckBox () ;
        ck_ANTIALIAS = new JCheckBox () ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust Properties                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        st_LANDF.setVerticalAlignment ( SwingConstants.TOP ) ;       
        lb_LANDF.setBorder ( GUISystem.loweredBorder ) ;
        ck_ANTIALIAS.addActionListener ( this ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add the controls                                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setLayout ( null ) ;
        add ( st_LANDF ) ;   
        add ( lb_LANDF ) ;
        add ( ck_MDI ) ;
        add ( ck_ANTIALIAS ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Set the text                                            */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setText() ;
    }





    /*------------------------------------------------------------*/
    /* setText
     */
    /**
     *    This method will set the text of all of
     *  our GUI Controls
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setText () {
        Vector landfVec = new Vector ( 3, 1 ) ;

        landfVec.addElement ( Str.getStr ( STR_SOLARIS ) ) ;
        landfVec.addElement ( Str.getStr ( STR_SWING ) ) ;
        landfVec.addElement ( Str.getStr ( STR_WINDOWS ) ) ;
        landfVec.addElement ( "Slick Metal" ) ;

        st_LANDF.setText ( Str.getStr ( STR_LANDF_HELP ) ) ;
        ck_MDI.setText ( Str.getStr ( STR_SHOWMDI ) ) ;
        ck_ANTIALIAS.setText ( Str.getStr ( STR_ANTI_ALIAS ) ) ;

        lb_LANDF.removeAll() ;
        lb_LANDF.setListData ( landfVec ) ;

        refreshData() ;

    }





    /*------------------------------------------------------------*/
    /* refreshData
     */
    /**
     *    This method will refresh the data on
     *  our GUI
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void refreshData () {
        try {
            String currentLF = PropertySystem.getString ( PropertySystem.PROPERTY_LAF ) ;

            if ( currentLF.indexOf ( "WindowsLook" ) >= 0 )
                lb_LANDF.setSelectedValue ( Str.getStr (STR_WINDOWS), true ) ;
            else if ( currentLF.indexOf ( "MacLook" ) >= 0 )
                lb_LANDF.setSelectedValue ( Str.getStr ( STR_MACINTOSH ), true ) ;
            else if ( currentLF.indexOf ( "MetalLook" ) >= 0 )
                lb_LANDF.setSelectedValue ( Str.getStr ( STR_SWING ), true ) ;
            else if ( currentLF.indexOf ( "Motif" ) >= 0 )
                lb_LANDF.setSelectedValue ( Str.getStr ( STR_SOLARIS ), true ) ;
            else if ( currentLF.indexOf ( "Slick Metal" ) >= 0 )
                lb_LANDF.setSelectedValue ( "Slick Metal", true ) ;

            ck_MDI.setSelected ( PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) ;
            ck_ANTIALIAS.setSelected ( PropertySystem.getBool ( PropertySystem.PROPERTY_ANTI_ALIAS ) ) ;

        } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
        }
    }



    /*------------------------------------------------------------*/
    /* doLayout
     */
    /**
     *    This method will lay out and position all
     *  of our components
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void doLayout () {
        Dimension size = getSize() ;

        super.doLayout() ;

        st_LANDF.setBounds ( 5, 5, size.width-10, 130 ) ;
        lb_LANDF.setBounds ( 45, 135, size.width-90, 150 ) ;

        ck_MDI.setBounds ( 5, 290, size.width-10, 21 ) ;
        ck_ANTIALIAS.setBounds ( 5, 313, size.width-10, 21 ) ;
    }





    /*------------------------------------------------------------*/
    /* saveInfo
     */
    /**
     *    This method will be called when we are
     *  expected to save all properties on this
     *  panel
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean saveInfo () {
        String lookFeel = (String)lb_LANDF.getSelectedValue() ;

        try {
            String  currentLF = PropertySystem.getString ( PropertySystem.PROPERTY_LAF ) ;
            boolean mdi       = PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ;

            if ( ( lookFeel.equals ( Str.getStr ( STR_SWING ) ) ) && ( !currentLF.equals ( "javax.swing.plaf.metal.MetalLookAndFeel"  ) ) )
                GUISystem.setLookAndFeel ( "javax.swing.plaf.metal.MetalLookAndFeel" ) ;
            else if ( ( lookFeel.equals ( Str.getStr ( STR_WINDOWS ) ) ) && ( !currentLF.equals ( "com.sun.java.swing.plaf.windows.WindowsLookAndFeel" ) ) )
                GUISystem.setLookAndFeel (  "com.sun.java.swing.plaf.windows.WindowsLookAndFeel" ) ;
            else if ( ( lookFeel.equals ( Str.getStr ( STR_SOLARIS ) ) ) && ( !currentLF.equals ( "com.sun.java.swing.plaf.motif.MotifLookAndFeel" ) ) )
                GUISystem.setLookAndFeel (  "com.sun.java.swing.plaf.motif.MotifLookAndFeel" ) ;
            else if ( ( lookFeel.equals ( Str.getStr ( STR_MACINTOSH ) ) ) && ( !currentLF.equals ( "com.sun.java.swing.plaf.mac.MacLookAndFeel" ) ) )
                GUISystem.setLookAndFeel (  "com.sun.java.swing.plaf.mac.MacLookAndFeel" ) ;
            else if ( ( lookFeel.equals ( "Slick Metal" ) ) && ( !currentLF.equals ( "Slick Metal" ) ) )
                GUISystem.setLookAndFeel (  "Slick Metal" ) ;

            if ( mdi != ck_MDI.isSelected() )
                PropertySystem.putBool ( PropertySystem.PROPERTY_MDI, ck_MDI.isSelected() ) ;

            PropertySystem.putBool ( PropertySystem.PROPERTY_ANTI_ALIAS, ck_ANTIALIAS.isSelected() ) ;

        } catch ( Exception e ) {
        }

        return ( true ) ;
    }





    /*------------------------------------------------------------*/
    /* free
     */
    /**
     *    This method will be called to release resources
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    protected void free () {
    }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title for when we
    *  show in a wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
    public String getTitle () {
       return ( Str.getStr ( STR_LANDF_HELP_TITLE ) ) ;
    }


    
    
    
       /*------------------------------------------------------------*/
       /* actionPerformed
        */
       /**
        *    This method will respond to action
        *  events
        *  
        *
        *  @param None
        *  @param None
        *  @param None
        *
        *  @return Nothing
        *//*---------------------------------------------------------*/
    public void actionPerformed ( ActionEvent e ) {
       if ( e.getSource() == ck_ANTIALIAS ) {
          if ( ck_ANTIALIAS.isSelected() ) 
             GUISystem.printBox ( STR_INFO, STR_WARN_ANTI_ALIAS ) ;
       }
    }
} /* LookFeelPage */
