package com.ibm.nzna.projects.qit.app.appProp ;              
/*-----------------------------------------------------------*/
/*   SelectCountryPage                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectCountryPage                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import javax.swing.* ;
import java.awt.* ;      
/**
 * This notebook page goes in the ApplicationPropertyDlg's notebook. It allows 
 * the user to select what country the application is configured for.
 *
 * @version 1.0, 1/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectCountryPage                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectCountryPage extends WizardStep implements AppConst, PropertyListener, TypeListListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel         st_COUNTRYHELP          = null ;
   private JList          lb_COUNTRY              = null ;
   private JScrollPane    scr_COUNTRY             = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the
    *  panel and all GUI components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectCountryPage () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_COUNTRYHELP  = new JLabel () ;
      lb_COUNTRY      = new JList  () ;            
      scr_COUNTRY     = new JScrollPane ( lb_COUNTRY ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the properties                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_COUNTRYHELP.setVerticalAlignment ( SwingConstants.TOP ) ;
      setText() ; 
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_COUNTRYHELP ) ;
      add ( scr_COUNTRY ) ;


   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;

      st_COUNTRYHELP.setBounds ( 5, 5, size.width-10, 130 ) ;
      scr_COUNTRY.setBounds ( 45, 135, size.width-90, 160 ) ;
   }



   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of all GUI
    *  components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      try {

         st_COUNTRYHELP.setText ( Str.getStr ( STR_COUNTRY_SELECT_HELP ) ) ;
         TypeList.refreshList ( lb_COUNTRY, TypeList.TYPE_COUNTRYCODE, this ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save our properties
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo() {
      try {
         TypeCountryCodeRec countryRec = (TypeCountryCodeRec) lb_COUNTRY.getSelectedValue() ;                                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If they have a country rec, reset the application       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( countryRec != null ) {
            int currentCountryCodeInd = PropertySystem.getInt ( PropertySystem.PROPERTY_COUNTRY ) ;

            if ( currentCountryCodeInd != countryRec.getInd() )
               PropertySystem.putInt ( PropertySystem.PROPERTY_COUNTRY, countryRec.getInd() ) ;

         } else
            GUISystem.printBox ( STR_ERROR, STR_MUST_SELECT_COUNTRY ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a
    *  property we are listening for has been
    *  fired
    *
    *  @param property The property that changed
    *  @param value The new value
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE )
         setText() ;
   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will be called when our
    *  panel is freed and needs to release 
    *  resources
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void free () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
   }





   /*------------------------------------------------------------*/
   /* listRefreshed
    */
   /**
    *    This method will be called by the TypeList
    *  object when our list has been refreshed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void listRefreshed ( int typeList ) {
      try {
         TypeCountryCodeRec countryRec     = null ;
         int                countryCodeInd = PropertySystem.getInt ( PropertySystem.PROPERTY_COUNTRY ) ;

         countryRec = (TypeCountryCodeRec) TypeList.getInstance().objectFromInd ( countryCodeInd, TypeList.TYPE_COUNTRYCODE ) ;
         lb_COUNTRY.setSelectedValue ( countryRec, true ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title that is shown
    *  in a wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_COUNTRY_SELECT_HELP ) ) ;
   }
} /* SelectCountryPage */
