package com.ibm.nzna.projects.qit.app.appProp ;
/*-----------------------------------------------------------*/
/*   SkinsPage                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SkinsPage                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import java.awt.* ;
import java.io.* ;
import java.util.* ;
import javax.swing.* ;
/**
 * This page allows the user to select a skin for the app. It also
 * allows the user to turn the skins off.
 *
 * @version 1.0, 1/24/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SkinsPage                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class SkinsPage extends WizardStep implements AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_SKINS         = null ;
   private JList        lb_SKINS         = null ;
   private JScrollPane  scr_SKINS        = null ;
   private JCheckBox    ck_SKIN          = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our panel, including
    *  attaching all GUI components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SkinsPage () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SKINS  = new JLabel (  ) ;        
      lb_SKINS  = new JList () ;
      scr_SKINS = new JScrollPane ( lb_SKINS ) ;
      ck_SKIN   = new JCheckBox ( Str.getStr ( STR_SKIN_APP ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SKINS.setVerticalAlignment ( SwingConstants.TOP ) ;       
      scr_SKINS.setBorder ( GUISystem.loweredBorder ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      //add ( ck_SKIN ) ;
      add ( st_SKINS ) ;   
      add ( scr_SKINS ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the text                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of all of
    *  our GUI Controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText () {
      try {
         Vector   skinsVec   = new Vector ( 3, 1 ) ;
         String   skinStrDir = PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR )+
                               File.separator+
                               "skins" ;                                         
         String[] skins      = new File ( skinStrDir ).list() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read all of the directories in our skins directory      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( skins != null ) {
            for ( int i = 0; i < skins.length; i++ ) {
               if ( new File ( skinStrDir+File.separator+skins[i] ).isDirectory() )
                  skinsVec.addElement ( skins[i] ) ;
            }
         }



         st_SKINS.setText ( Str.getStr ( STR_SKINS ) ) ;

         lb_SKINS.removeAll() ;
         lb_SKINS.setListData ( skinsVec ) ;
         lb_SKINS.setSelectedValue ( PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME ), true ) ;
         ck_SKIN.setSelected ( PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ) ;

         refreshData() ;
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh the data on
    *  our GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
   }



   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all
    *  of our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_SKINS.setBounds ( 5, 5, size.width-10, 80 ) ;
      ck_SKIN.setBounds ( 45, 85, size.width-90, rowHeight ) ;

      scr_SKINS.setBounds ( 45, 85+rowHeight+2, size.width-90, 150 ) ;
   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will be called when we are
    *  expected to save all properties on this
    *  panel. We will only save the properties changed.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      String  skinName = (String)lb_SKINS.getSelectedValue() ;
      boolean skinApp  = false ; //ck_SKIN.isSelected() ;

      try {
         if ( ( skinName == null ) || ( skinName.length() == 0 ) ) {
            skinApp = false ;
            PropertySystem.put ( PropertySystem.PROPERTY_SKINNAME, "Default" ) ;
         } else {
            if ( !skinName.equals ( PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME ) ) )
               PropertySystem.put ( PropertySystem.PROPERTY_SKINNAME, skinName ) ;
         }


         if ( skinApp != PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) )
            PropertySystem.putBool ( PropertySystem.PROPERTY_SHOWSKIN, skinApp ) ;

      } catch ( Exception e ) {
      }
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will be called to release resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title when we show
    *  in a wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_SKINS_TITLE ) ) ;
   }
} /* SkinsPage */
