package com.ibm.nzna.projects.qit.app.appProp ;
/*-----------------------------------------------------------*/
/*   WordProcPage                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WordProcPage                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import java.util.* ;
import java.awt.* ;
import java.io.* ;
import java.awt.event.* ;
import javax.swing.event.* ;
import javax.swing.* ; 
/**
 * This page allows the user to select a word processor
 *
 *
 * @version 1.0, 1/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WordProcPage                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class WordProcPage extends WizardStep implements AppConst, ActionListener, PropertyListener, FocusListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel     st_HELP           = null ;
   private JLabel     st_PATH           = null ;
   private JTextField ef_PATH           = null ;
   private JLabel     st_NAME           = null ;
   private JTextField ef_NAME           = null ;
   private JCheckBox  ck_USEEXTWORDPROC = null ;
   private JCheckBox  ck_AUTO_CLOSE     = null ;
   private DButton    pb_BROWSE         = null ;
   private JCheckBox  ck_SHOW_INTERNAL  = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our page
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WordProcPage () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP           = new JLabel () ;
      st_PATH           = new JLabel () ;
      ck_USEEXTWORDPROC = new JCheckBox () ;
      ck_AUTO_CLOSE     = new JCheckBox () ;
      ef_PATH           = new JTextField () ;
      st_NAME           = new JLabel ( Str.getStr ( STR_NAME ) ) ;
      ef_NAME           = new JTextField ( Str.getStr ( STR_WORD_PROCESSOR ) ) ;
      pb_BROWSE         = new DButton ( Str.getStr ( STR_BROWSE ) ) ;
      ck_SHOW_INTERNAL  = new JCheckBox ( Str.getStr ( STR_SHOW_INTERNAL_WORD_PROC ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
      ef_PATH.addFocusListener ( this ) ;
      ck_USEEXTWORDPROC.addActionListener ( this ) ;
      pb_BROWSE.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP ) ;
      add ( ck_USEEXTWORDPROC ) ;
      add ( ck_AUTO_CLOSE ) ;
      add ( ck_SHOW_INTERNAL ) ;
      add ( st_PATH ) ;       
      add ( ef_PATH ) ;
      add ( pb_BROWSE ) ;
      add ( st_NAME ) ;
      add ( ef_NAME ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our text                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData() ;

   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh the data on our
    *  GUI components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      try {
         String path = PropertySystem.getString ( PropertySystem.PROPERTY_EXTERNAL_WORDPROC_EXE ) ;
         String name = PropertySystem.getString ( PropertySystem.PROPERTY_EXTERNAL_WORDPROC_NAME ) ;

         ck_USEEXTWORDPROC.setSelected ( PropertySystem.getBool ( PropertySystem.PROPERTY_USE_EXTERNAL_WORDPROC ) ) ;

         if ( path != null )
            ef_PATH.setText ( path ) ;

         if ( name != null )
            ef_NAME.setText ( name ) ;

         ck_SHOW_INTERNAL.setEnabled ( PropertySystem.getBool ( PropertySystem.PROPERTY_SHOW_INTERNAL_WORDPROC ) ) ;
         ck_AUTO_CLOSE.setEnabled ( ck_USEEXTWORDPROC.isSelected() ) ;
         ef_PATH.setEnabled ( ck_USEEXTWORDPROC.isSelected() ) ;
         pb_BROWSE.setEnabled ( ck_USEEXTWORDPROC.isSelected() ) ;
         st_PATH.setEnabled ( ck_USEEXTWORDPROC.isSelected() ) ;
         ck_AUTO_CLOSE.setSelected ( PropertySystem.getBool ( PropertySystem.PROPERTY_CLOSE_EXTERNAL_WORDPROC ) ) ;

         if ( !ck_USEEXTWORDPROC.isSelected() ) {
            ck_SHOW_INTERNAL.setSelected ( true ) ;
            ck_SHOW_INTERNAL.setEnabled ( false ) ;
         } else
            ck_SHOW_INTERNAL.setEnabled ( true ) ;

      } catch ( Exception e ) {
      }

   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       y         = 85 ;

      super.doLayout() ;

      st_HELP.setBounds ( 5, 5, size.width-10, 80 ) ;
      ck_USEEXTWORDPROC.setBounds ( 5, y, size.width-10, rowHeight ) ;
      y+=rowHeight ;

      ck_AUTO_CLOSE.setBounds ( 5, y, size.width-10, rowHeight ) ;
      y+=rowHeight ;

      ck_SHOW_INTERNAL.setBounds ( 5, y, size.width-10, rowHeight ) ;
      y+=rowHeight ;

      st_PATH.setBounds ( 5, y, size.width-10, rowHeight ) ;
      y+=rowHeight ;

      ef_PATH.setBounds ( 5, y, size.width-95 , rowHeight ) ;
      pb_BROWSE.setBounds ( size.width-90, y, 80, rowHeight ) ;
      y+=rowHeight+2 ;

      st_NAME.setBounds ( 5, y, size.width-10, rowHeight ) ;
      y+=rowHeight ;

      ef_NAME.setBounds ( 5, y, size.width-10, rowHeight ) ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set all of our
    *  text
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText ( ) {
      try {
         int language = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;

         st_HELP.setText           ( Str.getStr ( STR_WORDPROC_SELECT_HELP ) ) ;
         st_PATH.setText           ( Str.getStr ( STR_FULL_PATH_EXE ) ) ;
         ck_USEEXTWORDPROC.setText ( Str.getStr ( STR_USE_EXTERNAL_WORD_PROC ) ) ;
         st_NAME.setText           ( Str.getStr ( STR_NAME ) ) ;
         pb_BROWSE.setText         ( Str.getStr ( STR_BROWSE ) ) ;
         ck_AUTO_CLOSE.setText     ( Str.getStr ( STR_AUTO_CLOSE_EXTERNAL_WORDPROC ) ) ;

      } catch ( Exception e ) {
      }
   }






   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save our properties
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      boolean rc = true ;

      try {
         String pathExe = ef_PATH.getText() ;

         PropertySystem.putBool ( PropertySystem.PROPERTY_USE_EXTERNAL_WORDPROC, ck_USEEXTWORDPROC.isSelected() ) ;

         if ( ck_USEEXTWORDPROC.isSelected() ) {
            File   file    = new File ( pathExe ) ;

            if ( file.exists() )
               PropertySystem.putString ( PropertySystem.PROPERTY_EXTERNAL_WORDPROC_EXE, ef_PATH.getText() ) ;
            else {
               GUISystem.printBox ( STR_ERROR, STR_EXTERNAL_WORD_PROC_DOES_NOT_EXIST ) ;
               rc = false ;
            }

            PropertySystem.putBool   ( PropertySystem.PROPERTY_SHOW_INTERNAL_WORDPROC, ck_SHOW_INTERNAL.isSelected() ) ;
            PropertySystem.putString ( PropertySystem.PROPERTY_EXTERNAL_WORDPROC_NAME, ef_NAME.getText() ) ;
            PropertySystem.putBool   ( PropertySystem.PROPERTY_CLOSE_EXTERNAL_WORDPROC, ck_AUTO_CLOSE.isSelected() ) ;
         }

      } catch ( Exception e ) {
      }
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a
    *  property has been changed
    *  
    *
    *  @param property The property that was changed
    *  @param value The new value
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      setText() ;
   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will release our resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to action events
    *  being fired
    *  
    *
    *  @param e The action event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      ef_PATH.setEnabled ( ck_USEEXTWORDPROC.isSelected() ) ;
      pb_BROWSE.setEnabled ( ck_USEEXTWORDPROC.isSelected() ) ;
      st_PATH.setEnabled ( ck_USEEXTWORDPROC.isSelected() ) ;
      ck_AUTO_CLOSE.setEnabled ( ck_USEEXTWORDPROC.isSelected() ) ;

      if ( !ck_USEEXTWORDPROC.isSelected() ) {
         ck_SHOW_INTERNAL.setSelected ( true ) ;
         ck_SHOW_INTERNAL.setEnabled ( false ) ;
      } else
         ck_SHOW_INTERNAL.setEnabled ( true ) ;

      if ( e.getSource() == pb_BROWSE ) {
         File[] file = GUISystem.getFileOpen ( WinUtil.getParentFrame ( this ),
                                               Str.getStr ( STR_FIND ),
                                               "",
                                               "*.EXE",
                                               true ) ;
         if ( file != null ) {
            ef_PATH.setText ( file[1].getAbsolutePath()+File.separator+file[0].getName() ) ;
            focusLost ( null ) ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* focusLost
    */
   /**
    *    This method will be called when focus is
    *  lost from the EXEPATH param
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusLost ( FocusEvent e ) {
      String text = ef_PATH.getText() ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Auto-populate the name                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( text.toUpperCase().indexOf ( "DREAMWEAVER" ) >= 0 )
         ef_NAME.setText ( "Dreamweaver" ) ;
      else if ( text.toUpperCase().indexOf ( "NETSCAPE" ) >= 0 )
         ef_NAME.setText ( "Netscape" ) ;
      else if ( text.toUpperCase().indexOf ( "HOMESITE" ) >= 0 )
         ef_NAME.setText ( "HomeSite" ) ;
      else if ( text.toUpperCase().indexOf ( "WORDPRO" ) >= 0 )
         ef_NAME.setText ( "Lotus WordPro" ) ;
      else if ( text.toUpperCase().indexOf ( "WORD" ) >= 0 )
         ef_NAME.setText ( "MS Word" ) ;

   }

   public void focusGained ( FocusEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title that is shown
    *  when we are put into a wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_WORDPROC_SELECT_HELP_TITLE ) ) ;
   }
} /* WordProcPage */
