package com.ibm.nzna.projects.qit.app.userProp ;
/*-----------------------------------------------------------*/
/*   CachePage                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CachePage                           */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.io.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.type.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This page holds what QIT will cache and allows the user to clear cache.
 *
 *
 * @version 1.0, 3/12/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CachePage                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class CachePage extends JPanel implements AppConst, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Variables                                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JCheckBox ck_CACHE_LISTS     = null ;
   private JCheckBox ck_CACHE_BRANDS    = null ;
   private DButton   pb_CLEAR           = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will simply create our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CachePage () {
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI and add's
    *  the GUI to our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI Components                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ck_CACHE_LISTS  = new JCheckBox ( Str.getStr ( STR_CACHE_TYPE_LISTS ) ) ;
      ck_CACHE_BRANDS = new JCheckBox ( Str.getStr ( STR_CACHE_BRANDS_FAMILIES ) ) ;
      pb_CLEAR        = new DButton ( Str.getStr ( STR_CLEAR_CACHE ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLEAR.addActionListener ( this ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( ck_CACHE_LISTS ) ;
      add ( ck_CACHE_BRANDS ) ;
      add ( pb_CLEAR ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData() ;

   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of the fields
    *  on the screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      ck_CACHE_LISTS.setSelected ( PropertySystem.getBool ( PropertySystem.PROPERTY_CACHE_TYPE_LISTS ) ) ;
      ck_CACHE_BRANDS.setSelected ( PropertySystem.getBool ( PropertySystem.PROPERTY_CACHE_BRAND_ASSOC ) ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       y         = 5 ;
      int       x         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      ck_CACHE_LISTS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      ck_CACHE_BRANDS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight*2 ;

      pb_CLEAR.setBounds ( x, y, 100, 25 ) ;
   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to button events
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_CLEAR ) {
         clearListCache() ;
         clearBrandCache() ;
         GUISystem.printBox ( STR_INFO, STR_COMPLETE ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save our information
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE
    *//*---------------------------------------------------------*/
   protected boolean saveInfo () {
      boolean cacheLists  = ck_CACHE_LISTS.isSelected() ;
      boolean cacheBrands = ck_CACHE_BRANDS.isSelected() ;

      PropertySystem.putBool ( PropertySystem.PROPERTY_CACHE_TYPE_LISTS,  cacheLists ) ;
      PropertySystem.putBool ( PropertySystem.PROPERTY_CACHE_BRAND_ASSOC, cacheBrands ) ;

      if ( !cacheLists )
         clearListCache() ;

      if ( !cacheBrands )
         clearBrandCache() ;

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* clearListCache
    */
   /**
    *    This method will delete all files in the
    *  home directory that begin with TypeList and
    *  end in cache.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void clearListCache () {
      File homeDir = new File ( CacheSystem.getCacheDir() ) ;

      if ( homeDir != null ) {
         String files[] = homeDir.list ( new TypeListFilter() ) ;
         int    i       = 0 ;

         if ( ( files != null ) && ( files.length > 1 ) ) {
            while ( i < files.length ) {
               try {
                  FileUtil.deleteFile ( files[i] ) ;
               } catch ( Exception e ) {
               }
               i++ ;
            }
         }

      }
   }


   /*------------------------------------------------------------*/
   /* clearBrandsCache
    */
   /**
    *    This method will delete all files in the
    *  home directory that begin with brands_assoc and
    *  end in cache.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void clearBrandCache () {
      File homeDir = new File ( CacheSystem.getCacheDir() ) ;

      if ( homeDir != null ) {
         String files[] = homeDir.list ( new BrandListFilter() ) ;
         int    i       = 0 ;

         if ( ( files != null ) && ( files.length > 1 ) ) {
            while ( i < files.length ) {
               try {
                  FileUtil.deleteFile ( files[i] ) ;
               } catch ( Exception e ) {
               }
               i++ ;
            }
         }

      }
   }

} /* CachePage */

class TypeListFilter implements FilenameFilter {
   public boolean accept ( File file, String filename ) {
      return( (filename.startsWith ( "TypeList" ) ) &&
              (filename.endsWith ( ".cache" ) ) ) ;
   }
}

class BrandListFilter implements FilenameFilter {
   public boolean accept ( File file, String filename ) {
      return( (filename.startsWith ( "brands_assoc" ) ) &&
              (filename.endsWith ( ".cache" ) ) ) ;
   }
}
