package com.ibm.nzna.projects.qit.app.userProp ;
/*-----------------------------------------------------------*/
/*   PasswordChangeDlg                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PasswordChangeDlg                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The dialog that allows the user to enter the current password,
 * a new password, and a verification of new password.
 *
 *
 * @version 1.0, 2/1/1997 
 * @author  David Lentz
 */
import java.awt.* ;
import javax.swing.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   PasswordChangeDlg                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class PasswordChangeDlg extends JDialog implements DataLengths, AppConst, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton        pb_OK                 = null ;
   private DButton        pb_CANCEL             = null ;
   private JPasswordField ef_CURRENTPASS        = null ;
   private JPasswordField ef_NEWPASS            = null ;
   private JPasswordField ef_NEWPASS1           = null ;
   private JLabel         st_CURRENTPASS        = null ;
   private JLabel         st_NEWPASS            = null ;
   private JLabel         st_NEWPASS1           = null ;
   private JLabel         st_DIRECTIONS         = null ;


   /*------------------------------------------------------------*/
   /* constructor
    */
   /**
    *    This will create the PasswordChangeDlg
    *  
    *  
    *
    *  @param currentPassword The user's current password
    *  @param listener The listener that we should notify
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PasswordChangeDlg ( Frame parent ) {
      super ( parent, Str.getStr(STR_CHANGE_PASSWORD), true ) ;

      setResizable ( false ) ;
      createControls() ;
      setSize ( 300,230 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
      setVisible ( true ) ;
   }






   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create all of our controls
    *  and add them to our window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {

      try {
         Container contentPane = getContentPane() ;     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         st_CURRENTPASS = new JLabel ( Str.getStr(STR_CURRENT_PASSWORD) ) ;                     
         st_NEWPASS     = new JLabel ( Str.getStr(STR_NEW_PASSWORD) ) ;                     
         st_NEWPASS1    = new JLabel ( Str.getStr(STR_NEW_PASSWORD) ) ;                     
         st_DIRECTIONS  = new JLabel ( Str.getStr(STR_PASSWORD_CHANGE_DIRECTIONS) ) ;
         ef_CURRENTPASS = new JPasswordField ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_PASSWORD ), "", 0 ) ;
         ef_NEWPASS     = new JPasswordField ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_PASSWORD ), "", 0 ) ;
         ef_NEWPASS1    = new JPasswordField ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_PASSWORD ), "", 0 ) ;
         pb_OK          = new DButton ( Str.getStr(STR_OK) ) ;
         pb_CANCEL      = new DButton ( Str.getStr(STR_CANCEL) ) ;        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         st_DIRECTIONS.setVerticalAlignment ( SwingConstants.TOP ) ;                      


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the components to our panel                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         contentPane.setLayout ( null ) ;
         contentPane.add ( st_DIRECTIONS ) ;
         contentPane.add ( ef_CURRENTPASS ) ;
         contentPane.add ( st_CURRENTPASS ) ;
         contentPane.add ( ef_NEWPASS ) ;
         contentPane.add ( st_NEWPASS ) ;
         contentPane.add ( ef_NEWPASS1 ) ;
         contentPane.add ( st_NEWPASS1 ) ;
         contentPane.add ( pb_OK ) ;
         contentPane.add ( pb_CANCEL ) ;    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_OK.addActionListener          ( this ) ;
         pb_CANCEL.addActionListener      ( this ) ;
         ef_CURRENTPASS.addActionListener ( this ) ;
         ef_NEWPASS.addActionListener     ( this ) ;
         ef_NEWPASS1.addActionListener    ( this ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      } /* catch */


   } /* createControls */







   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out all of our controls
    *  and size the controls accordingly.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout() {
      int       x          = 15 ;
      int       y          = 15 ;
      Dimension size       = getSize() ;
      int       rowHeight  = GUISystem.getRowHeight() ;

      super.doLayout() ;

      try {
         st_DIRECTIONS.setBounds ( x, y, size.width-(x*2), rowHeight*4 ) ;
         y+=rowHeight*4 ;

         st_CURRENTPASS.setBounds ( x, y, 120, rowHeight ) ;
         ef_CURRENTPASS.setBounds ( x+120, y, 130, rowHeight ) ;
         y+=rowHeight ;


         st_NEWPASS.setBounds ( x, y, 120, rowHeight ) ;
         ef_NEWPASS.setBounds ( x+120, y, 130, rowHeight ) ;
         y+=rowHeight ;

         st_NEWPASS1.setBounds ( x, y, 120, rowHeight ) ;
         ef_NEWPASS1.setBounds ( x+120, y, 130, rowHeight ) ;
         y+=rowHeight ;

         pb_OK.setBounds ( x, size.height-60, 70, 25 ) ;
         pb_CANCEL.setBounds ( x+75, size.height-60, 80, 25 ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      } /* catch */
   } /* layoutControls */




   /*------------------------------------------------------------*/
   /* buttonHit
    */
   /**
    *    This method will respond to buttons being
    *  pressed
    *  
    *
    *  @param id The ID of the button being pressed
    *  @param button The insance of the Button
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() instanceof DButton ) {
         DButton button = (DButton) e.getSource() ;


         if ( button == pb_OK ) {
            if ( saveData() )
               dispose() ;
         }

         else if ( button == pb_CANCEL )
            dispose() ;

      } /* if */

   } /* buttonHit */



   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save all of our data. It will
    *  return FALSE if the data was not entered
    *  correctly.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      boolean       rc              = false ;
      String        cp              = new String (ef_CURRENTPASS.getPassword()).toUpperCase() ;
      LoggedUserRec userRec         = UserSystem.getLoggedUserRec() ;
      String        currentPassword = userRec.getPassword() ;                                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check to see they typed the old password right          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !currentPassword.equals ( cp ) )
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_PASSWORD_WRONG ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check to see the new password's match                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( !new String(ef_NEWPASS.getPassword()).equals ( new String(ef_NEWPASS1.getPassword()) ) )
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_NEW_PASSWORD_NO_MATCH ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Make sure the new password isn't the old one            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( currentPassword.equals ( new String ( ef_NEWPASS.getPassword() ) ) )
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_NEW_PASSWORD_IS_OLD ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Make sure they didn't use the same substring            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( new String ( ef_NEWPASS.getPassword()).indexOf ( currentPassword ) > -1 ) ||
                ( currentPassword.indexOf ( new String ( ef_NEWPASS.getPassword() ) ) > -1 ) )
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_NEW_PASSWORD_CONTAINS_OLD ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, perfect, save the new password                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         currentPassword = new String ( ef_NEWPASS.getPassword() ).toUpperCase() ;
         rc = true ;
      }

      if ( rc ) {

         enableInput ( false ) ;
         rc = userRec.changePassword ( currentPassword ) ;
         enableInput ( true ) ;

         if ( rc ) {
            UserSystem.setLoggedUserRec ( userRec ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_PASSWORD_CHANGED ) ) ;
         }
      } /* if */


      return( rc ) ;
   } /* saveData */




   /*------------------------------------------------------------*/
   /* enableInput
    */
   /**
    *    This method will either enable or disable this
    *  dialog
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void enableInput ( boolean enable ) {
      pb_OK.setEnabled ( enable ) ;
      pb_CANCEL.setEnabled ( enable ) ;
   }
} /* NewUserDlg */
