package com.ibm.nzna.projects.qit.app.userProp ;
/*-----------------------------------------------------------*/
/*   UserPropertyDlg                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: UserPropertyDlg                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing. * ;
import java.awt.* ;           
import java.awt.event.* ;           
/**
 * This dialog holds a notebook with pages for each User Property. 
 *
 *
 * @version 1.0, 1/23/2000
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   UserPropertyDlg                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class UserPropertyDlg extends JDialog implements ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTabbedPane notebook        = null ;
   private DButton     pb_OK           = null ;
   private DButton     pb_CANCEL       = null ;
   private DButton     pb_APPLY        = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Pages                                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private UserPage            userPage            = null ;
   private BrandFamilyPage     brandFamilyPage     = null ;
   private FavoriteCountryPage favoriteCountryPage = null ;
   private CachePage           cachePage           = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the
    *  application properties. It will not return
    *  until the user closes the window
    *
    *  @param parent The parent window
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public UserPropertyDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_USER_PROPERTIES )+"-"+Str.getStr ( STR_APPLICATION ), true ) ;
      ButtonPanel buttonPanel = new ButtonPanel () ;

      setSize ( 400, 400 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      notebook  = new JTabbedPane () ;
      pb_OK     = new DButton ( ) ;
      pb_APPLY  = new DButton ( ) ;
      pb_CANCEL = new DButton ( ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the panels                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      notebook.addTab ( "df.", userPage            = new UserPage() ) ;    
      notebook.addTab ( "df.", brandFamilyPage     = new BrandFamilyPage() ) ;    
      notebook.addTab ( "df.", favoriteCountryPage = new FavoriteCountryPage () ) ;
      notebook.addTab ( "df.", cachePage           = new CachePage() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_APPLY.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;
      GUISystem.setPreferredButton ( pb_OK ) ;
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      GUISystem.setPreferredButton ( pb_APPLY ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( new BorderLayout (15,15) ) ;
      getContentPane().add ( notebook ) ;
      getContentPane().add ( buttonPanel, BorderLayout.SOUTH ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_APPLY ) ;
      buttonPanel.add ( pb_CANCEL ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      setVisible ( true ) ;
   }



   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on our
    *  buttons and tabs. It is up to the
    *  pages themselves to set their text
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      pb_OK.setText ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL.setText ( Str.getStr ( STR_CANCEL ) ) ;
      pb_APPLY.setText ( Str.getStr ( STR_APPLY ) ) ;

      notebook.setTitleAt ( notebook.indexOfComponent ( userPage ),            Str.getStr ( STR_USER ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( brandFamilyPage ),     Str.getStr ( STR_BRAND_FAMILY ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( favoriteCountryPage ), Str.getStr ( STR_FAVORITE_COUNTRY ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( cachePage ),           Str.getStr ( STR_CACHE ) ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one
    *  of our buttons is pressed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( e.getSource() == pb_OK ) || ( e.getSource() == pb_APPLY ) ) {
         if ( ( brandFamilyPage.saveInfo() ) &&
              ( favoriteCountryPage.saveInfo() ) && 
              ( cachePage.saveInfo() ) &&
              ( e.getSource() == pb_OK ) )
            dispose() ;

      }

      else
         dispose() ;
   }
    
} /* UserPropertyDlg */
