package com.ibm.nzna.projects.qit.app.util ;   
/*-----------------------------------------------------------*/
/*   QuestUtil                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestUtil                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This Object holds various generic static methods 
 *
 *
 * @version 1.0, 1/7/2000 
 * @author  David Lentz
 */
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestUtil                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestUtil extends Object implements Runnable, AppConst, Authority {



   /*----------*/
   /*          */                  
   /*  Threads */
   /*          */                  
   /*----------*/
   private static final String THREAD_POPULATE_TEMP_DIR = "TPTD" ;





   /*------------------------------------------------------------*/
   /* readObjectFromDisk
    */
   /**
    *    This method will deserialize a previously
    *  serialized object from the user's local
    *  hard drive
    *
    *  @param filename The filename of the file to deserialize
    *  @param None
    *  @param None
    *
    *  @return NULL if no file found or serialization error, Object otherwise
    *//*---------------------------------------------------------*/
   public static Object readObjectFromDisk ( String filename ) {
      Object retObject = null ;

      try {
         retObject = FileUtil.readObjectFromDisk ( filename ) ;
      } catch ( FileNotFoundException fnfe ) {
         retObject = null ;
      } catch ( Exception e ) {
         retObject = null ;
         LogSystem.log ( 1 ,e , false ) ;
      }

      return( retObject ) ;
   }





   /*------------------------------------------------------------*/
   /* writeObjectToDisk
    */
   /**
    *    This method will write a Object to the user's
    *  hard drive
    *  
    *
    *  @param filename The filename to save the object as
    *  @param Object The Object to serialize down
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean writeObjectToDisk ( String filename, Object object ) {
      boolean rc = false ;

      try {
         rc = FileUtil.writeObjectToDisk ( filename, object ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( rc ) ;
   }


   public static void populateTempDir () {
      new Thread ( new QuestUtil(), THREAD_POPULATE_TEMP_DIR ).start() ;
   }

   /**
     * This method will populate the temp directory by copying over
     * certain files
     **/
   private static void populateTemp () {
      String skinDir = "skins"+File.separator+PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+File.separator ;

      copyFileToTempDir ( skinDir+"doclink.gif" ) ;

      com.ibm.nzna.projects.qit.wordproc.StampedGraphicFactory.copyToTemp() ;                           

   }


   /*------------------------------------------------------------*/
   /* copyFileToTempDir
    */
   /**
    *    This method will copy a file to the temporary
    *  directory.
    *  
    *
    *  @param source The source filename including full path
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean copyFileToTempDir ( String source ) {
      return( copyFileToTempDir ( source, Text.stripPath ( source ) ) ) ;
   }


   /*------------------------------------------------------------*/
   /* copyFileToTempDir
    */
   /**
    *    This method will copy a file to the temporary
    *  directory. You can use the destName param to give the
    *  file a new name once it is copied into the temp dir.
    *  
    *
    *  @param source The source filename including full path
    *  @param destName The new file name to save as. No Path
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean copyFileToTempDir ( String source, String destName ) {
      boolean rc = false ;

      try {
         String tempDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator ;

         if ( tempDir != null ) {
         LogSystem.log ( 2, "Copying file :"+source+"  To :"+tempDir+destName ) ;
         if ( FileUtil.copyFile ( source, 
                                  tempDir+destName ) )
            rc = FileUtil.exists ( tempDir+destName ) ;

         if ( !rc )
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_CANNOT_COPY_FILE_TO_TEMP_DIR )+" - "+source ) ;
         } else
            LogSystem.log ( 2, "Not copying file "+source+" because Temp Directory is invalid or null" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( rc ) ;
   }

   /*------------------------------------------------------------*/
   /* writeTempFile
    */
   /**
    *    This method will write a file to the temporary
    *  directory. The file will be removed when the
    *  application closes if the user has selected to do so in application preferences
    *
    *  @param filename The filename of the file. The filename should not contain a path
    *  @param contents The contents of the file
    *  @param None
    *
    *  @return TRUE if it wrote Ok, FALSE if it did not
    *//*---------------------------------------------------------*/
   public static boolean writeTempFile ( String filename, byte[] contents ) {
      boolean rc = false ;

      try {

         /* Ok, if the filename and contents are both good, continue */
         if ( ( filename != null ) && 
              ( filename.length() > 0 ) && 
              ( contents != null ) && 
              ( contents.length > 0 ) ) {
            try {
               String           tempDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;
               File             outFile = new File ( tempDir+File.separator+filename ) ;
               FileOutputStream out     = new FileOutputStream ( outFile ) ;

               out.write ( contents ) ;

               out.close() ;
               rc = true ;

            } catch ( FileNotFoundException e ) {

               /* Ok, we could not write to the temp dir for some reason,
                  ask the user if the temp dir is correct */
               e.printStackTrace() ;

            } catch ( IOException ioException ) {

               /* Ok, we could not write to the temp dir for some reason,
                  ask the user if the temp dir is correct */
               ioException.printStackTrace() ;

            }

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* writeObjectToTemp
    */
   /**
    *    This method will write a Object to the user's
    *  configured temp directory
    *  
    *
    *  @param filename The filename to save the object as
    *  @param Object The Object to serialize down
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean writeObjectToTemp ( String filename, Object object ) {
      boolean rc = false ;

      try {
         String tempDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

         rc = FileUtil.writeObjectToDisk ( tempDir+File.separator+filename, object ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( rc ) ;
   }

   /*------------------------------------------------------------*/
   /* readObjectFromTemp
    */
   /**
    *    This method will deserialize a previously
    *  serialized object from the user's local
    *  temp directory
    *
    *  @param filename The filename of the file to deserialize
    *  @param None
    *  @param None
    *
    *  @return NULL if no file found or serialization error, Object otherwise
    *//*---------------------------------------------------------*/
   public static Object readObjectFromTemp ( String filename ) {
      Object retObject = null ;

      try {
         String tempDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

         retObject = FileUtil.readObjectFromDisk ( tempDir+File.separator+filename ) ;
      } catch ( FileNotFoundException fnfe ) {
         retObject = null ;
      } catch ( Exception e ) {
         retObject = null ;
         LogSystem.log ( 1 ,e , false ) ;
      }

      return( retObject ) ;
   }

   /*------------------------------------------------------------*/
   /* getDataFromDisplay
    */
   /**
    *    This method will get each data item from each
    *  DisplayRec in the vector passed in and returns a Vector of the
    *  data objects that each DisplayRec.
    *
    *  @param displayVec A Vector containing DisplayRecs
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector getDataFromDisplay ( Vector displayVec ) {
      Vector returnVec = null ;

      if ( displayVec != null ) {
         int size = displayVec.size() ;
         int i    = 0 ;

         returnVec = new Vector ( size ) ;

         while ( i < size ) {
            returnVec.addElement ( ((DisplayRec)displayVec.elementAt(i)).getData() ) ;
            i++ ;
         }
      }

      return( returnVec ) ;
   }




   /*------------------------------------------------------------*/
   /* clearTempDir
    */
   /**
    *    This method will erase all files in the temp
    *  directory
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void clearTempDir () {
      try {
         String   strTempDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;
         File     tempDir    = new File ( strTempDir ) ;
         String[] files      = tempDir.list() ;

         if ( ( files != null ) && ( files.length > 0 ) ) {
            int i = 0 ;

            while ( i < files.length ) {
               new File ( strTempDir+File.separator+files[i] ).delete() ;
               i++ ;
            }
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getCountriesFromGeo
    */
   /**
    *    This method will return a vector of countries
    *  from a geography. If no vector is passed into this
    *  function, a new vector will be created. The vector
    *  returned will contain com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRecs
    *
    *  @param countryVector The vector to add the countries to
    *  @param geoInd The geography indicator to look for
    *  @param None
    *
    *  @see com.ibm.nzna.projects.common.quest.type.TypeCountryCodeRec
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getCountriesFromGeo ( Vector countryVec, int geoInd ) {
      Vector allCountries = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;

      if ( countryVec == null )
         countryVec = new Vector ( 1, 1 ) ;

      if ( allCountries != null ) {
         int size = allCountries.size() ;
         int i    = 0 ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through all of the countries and pull out the geos */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            if ( ((TypeCountryCodeRec)allCountries.elementAt(i)).geoInd == geoInd ) {
               countryVec.addElement ( allCountries.elementAt(i) ) ;
            }
            i++ ;
         }
      }


      return( countryVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getValidData
    */
   /**
    *    This method will go through a Vector of PersistentRecs
    *  and will return a list of PersistentRecs that can
    *  be viewed by the user. It will filter out all REC_NOINSERT and REC_DELETE records
    *
    *  @param vec The vector of PersittentRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getValidData ( Vector vec ) {
      if ( vec != null ) {
         Vector retVec = new Vector ( vec.size() ) ;
         int    i      = 0 ;
         int    size   = vec.size() ;

         while ( i < size ) {
            if ( ((PersistentRec)vec.elementAt(i)).valid() )
               retVec.addElement ( vec.elementAt(i) ) ;
            i++ ;
         }
         return( retVec ) ;
      }

      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* containsValid
    */
   /**
    *    This method will go through a Vector of
    *  PersistentRec objects and returns TRUE if
    *  any one of the objects are valid
    *
    *  @param vector
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public static boolean containsValid ( Vector vec ) {
      boolean contains = false ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i    = 0;
         int size = vec.size() ;

         while ( ( contains ) && ( i < size ) ) {
            if ( ((PersistentRec)vec.elementAt(i)).valid() )
               contains = true ;
            i++ ;
         }
      }

      return( contains ) ;
   }





   /*------------------------------------------------------------*/
   /* brandAuthFromBrandInd
    */
   /**
    *    This method will return the AUTHIND for
    *  the BRANDIND passed in. 
    *  
    *
    *  @param brandInd The Brand indicator to look with
    *  @param None
    *  @param None
    *
    *  @return int The auth indicator as defined in TIGRIS.TYPEAUTH
    *//*---------------------------------------------------------*/
   public static int brandAuthFromBrandInd ( int brandInd ) {
      int auth = -1 ;

      switch ( brandInd ) {
         case 0 :
            auth = 0 ;
            break ;

         case 1 :
            auth = 10 ;
            break ;

         case 2 :
            auth = 7 ;
            break ;

         case 3 :
            auth = 6 ;
            break ;

         case 4 :
            auth = 11 ;
            break ;

         case 5 :
            auth = 12 ;
            break ;

         case 6 :
            auth = 9 ;
            break ;

         case 7 :
            auth = 13 ;
            break ;

         case 8 :
            auth = 14 ;
            break ;

         case 9 :
            auth = 15 ;
            break ;

         case 10 :
            auth = 8 ;
            break ;

         case 13 :
         case 14 :
            auth = 6 ;
            break ;

         default:
            auth = 0 ;
            break ;

      }

      return( auth ) ;
   }






   /*------------------------------------------------------------*/
   /* avalonBrandAuthFromBrandInd
    */
   /**
    *    This method will return the AUTHIND for
    *  the BRANDIND passed in. 
    *  
    *
    *  @param brandInd The Brand indicator to look with
    *  @param None
    *  @param None
    *
    *  @return int The auth indicator as defined in TIGRIS.TYPEAUTH
    *//*---------------------------------------------------------*/
   public static int avalonBrandAuthFromBrandInd ( int brandInd ) {
      int auth = -1 ;

      switch ( brandInd ) {
         case 0 :
            auth = AUTH_SUPERUSER ;
            break ;

         case 1 :
            auth = AUTH_AVALON_AMBRA ;
            break ;

         case 2 :
            auth = AUTH_AVALON_CROSS_BRAND ;
            break ;

         case 3 :
            auth = AUTH_AVALON_APTIVA ;
            break ;

         case 4 :
            auth = AUTH_AVALON_INTELLISTATION ;
            break ;

         case 5 :
            auth = AUTH_AVALON_MONITORS ;
            break ;

         case 6 :
            auth = AUTH_AVALON_OPTIONS ;
            break ;

         case 7 :
            auth = AUTH_AVALON_IBM_PC ;
            break ;

         case 8 :
            auth = AUTH_AVALON_IBM_PC_SERVER ;
            break ;

         case 9 :
            auth = AUTH_AVALON_POWER_PC ;
            break ;

         case 10 :
            auth = AUTH_AVALON_THINKPAD ;
            break ;

         case 13 :
         case 14 :
            auth = AUTH_AVALON_APTIVA ;
            break ;

         default:
            auth = AUTH_SUPERUSER ;
            break ;

      }

      return( auth ) ;
   }





   /*------------------------------------------------------------*/
   /* cleanOldLogFiles
    */
   /**
    *    This method will clean out all of the old log
    *  files, leaving on PROPERTY_LOGS_KEEP
    *  log files kept.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void cleanOldLogFiles () {
      int maxLogFiles = PropertySystem.getInt ( PropertySystem.PROPERTY_LOGS_KEEP ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, try and get a listing of all quest debug files   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         File     dir        = new File ( "." ) ;
         String[] files      = dir.list() ;
         String   start      = "quest.debug." ;
         Vector   debugFiles = new Vector ( 1 ) ;                                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, we have our directory listing from using the        */
         /* dir.list() method above, now walk through the list and    */
         /* grab each valid quest debug file                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( files != null ) && ( files.length > 1 ) ) {
            String today = new CDate ( CDate.DATE_STAMP ).today() ; // Get today's timestamp

            for ( int i = 0; i < files.length; i++ ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Ok, if the filename contains 'quest.debug' and the      */
               /* length of the filename is greater than 'quest.debug'      */
               /* (meaning that there is a date appended to it) then        */
               /* extract the date and add it to our list                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( files[i].indexOf ( "quest.debug" ) >= 0 ) &&
                    ( files[i].length() > start.length()+1 ) ) {
                  String date = files[i].substring ( start.length() ) ;                         

                  /*------------------------------------------------------------*/
                  /*                                                            */
                  /*   Now, use the CDate.dateDiff method                       */
                  /* to figure out if this debug file should be deleted. Always */
                  /* delete the file if the maxLogFiles to keep is zero.        */
                  /*                                                            */
                  /*------------------------------------------------------------*/
                  if ( ( CDate.differenceDate ( today,
                                                date,
                                                CDate.DATE_STAMP ) > maxLogFiles ) ||
                       ( maxLogFiles == 0 ) ) {
                     debugFiles.addElement ( files[i] ) ; // add this file to the list to delete
                  }

               } /* if  */
            } /* for  */
         } /* if files != null  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, delete all of the logs that should be deleted      */
         /* since they were added to our debugFiles vector            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( debugFiles.size() > 0 ) {
            int i    = 0 ;
            int size = debugFiles.size() ; // Never put .size() in the condition of a loop!

            while ( i < size ) {
               LogSystem.log ( 1, "Deleting file OLD log file:"+debugFiles.elementAt(i) ) ;
               FileUtil.deleteFile ( (String)debugFiles.elementAt(i) ) ;
               i++ ;
            }
         }


      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

   }



   /**
     *  This method will attempt to make a connection to the production file ftp server. This
     * server is defined as the IT_FILEHOST constant in TIGRIS.CONSTANTS. 
     **/
   public static Ftp connectToProductionFileFTP () {
      String fileHost = Constants.getStrConst (  "IT_FILEHOST" ) ;
      Ftp    ftp      = null ;

      try {
         ftp = new Ftp ( ) ;

         ftp.setHost ( fileHost ) ;
         ftp.setUser ( FTP_USERID_PRODUCTION ) ;
         ftp.setPassword ( FTP_PASSWORD_PRODUCTION ) ;
         ftp.connect() ;
         ftp.login() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, "Could not log into FTP Server:"+fileHost ) ;
         LogSystem.log ( 1, e, false ) ;
         ftp=null ;
      }

      return( ftp ) ;
   }


   /**
     *  This method will attempt to make a connection to the temp file ftp server. This
     * server is defined as the IT_FILEHOSTTEMP constant in TIGRIS.CONSTANTS. 
     **/
   public static Ftp connectToTempFileFTP () {
      String fileHost = Constants.getStrConst (  "IT_FILEHOSTTEMP" ) ;
      Ftp    ftp      = null ;

      try {
         ftp = new Ftp ( ) ;

         ftp.setHost ( fileHost ) ;
         ftp.setUser ( FTP_USERID_PRODUCTION ) ;
         ftp.setPassword ( FTP_PASSWORD_PRODUCTION ) ;
         ftp.connect() ;
         ftp.login() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, "Could not log into FTP Server:"+fileHost ) ;
         LogSystem.log ( 1, e, false ) ;
         ftp=null ;
      }

      return( ftp ) ;
   }





   /**
     *  This method will attempt to make a connection to the production Graphic ftp server. This
     * server is defined as the IT_IMAGEHOST constant in TIGRIS.CONSTANTS. 
     **/
   public static Ftp connectToProductionGraphicFTP () {
      String fileHost = Constants.getStrConst (  "IT_IMAGEHOST" ) ;
      Ftp    ftp      = null ;

      try {
         ftp = new Ftp ( ) ;

         ftp.setHost ( fileHost ) ;
         ftp.setUser ( FTP_USERID_PRODUCTION ) ;
         ftp.setPassword ( FTP_PASSWORD_PRODUCTION ) ;
         ftp.connect() ;
         ftp.login() ;
         ftp.setType ( ftp.BINARY ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, "Could not log into Graphics Production FTP Server:"+fileHost ) ;
         LogSystem.log ( 1, e, false ) ;
         ftp=null ;
      }

      return( ftp ) ;
   }


   /**
     *  This method will attempt to make a connection to the temp graphic ftp server. This
     * server is defined as the IT_IMAGEHOSTTEMP constant in TIGRIS.CONSTANTS. 
     **/
   public static Ftp connectToTempGraphicFTP () {
      String fileHost = Constants.getStrConst (  "IT_IMAGEHOSTTEMP" ) ;
      Ftp    ftp      = null ;

      try {
         ftp = new Ftp ( ) ;

         ftp.setHost ( fileHost ) ;
         ftp.setUser ( FTP_USERID_PRODUCTION ) ;
         ftp.setPassword ( FTP_PASSWORD_PRODUCTION ) ;
         ftp.connect() ;
         ftp.login() ;
         ftp.setType ( ftp.BINARY ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, "Could not log into temp Graphic FTP Server:"+fileHost ) ;
         LogSystem.log ( 1, e, false ) ;
         ftp=null ;
      }

      return( ftp ) ;
   }





   /**
     * This method will run our threads
     **/
   public void run () {
      String name = Thread.currentThread().getName() ;

      if ( name.equals ( THREAD_POPULATE_TEMP_DIR ) ) {
         Thread.currentThread().setPriority ( Thread.MIN_PRIORITY ) ;
         populateTemp() ;
      }
   }

} /* QuestUtil */
