package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   ActionDisplayRec                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionDisplayRec                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.Action ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
/**
 * This record simply shows a Action in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionDisplayRec                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionDisplayRec extends OAObjectDisplayRec implements IQuickSort, DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int      actionInd         = 0 ;
   private int      docClassInd       = 0 ;
   private String   title             = null ;
   private String   changedTime       = null ;
   private String   dbUser            = null ;
   private String   prettyChangedTime = null ;
   private String   prettyDBUser      = null ;
   private Action   action            = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionDisplayRec () {
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionDisplayRec ( int actionInd, int docClassInd, String title, String changedTime, String dbUser ) {
      this.actionInd        = actionInd ;
      this.docClassInd       = docClassInd ;
      this.title             = title ;
      this.changedTime       = changedTime ;
      this.dbUser            = dbUser ;
   }


   public ActionDisplayRec ( Action action ) {
      this.action      = action ;
      if ( action != null ) {
         this.actionInd   = action.getInd() ;
         this.docClassInd = action.getDocClass().getInd() ;
         this.title       = action.getTitles().elementAt(action.getTitles().size()-1).toString()  ;
         this.changedTime = action.getChangedTime() ;
         this.dbUser      = action.getDbUser() ;
      }
   }

   /*------------------------------------------------------------*/
   /* setActionInd
    */
   /**
    *    This method will set our action indicator
    *  
    *  
    *
    *  @param actionInd The Sympom Indicator to represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setActionInd ( int actionInd ) {
      this.actionInd = actionInd ;
   }




   /*------------------------------------------------------------*/
   /* getActionInd
    */
   /**
    *    This method will return the action indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getActionInd () {
      return( actionInd ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Classification                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            if ( docClassInd == TypeList.DOCCLASS_IBM_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ;
            else if ( docClassInd == TypeList.DOCCLASS_DEALER_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_DEALER_CONFIDENTIAL ) ;
            else
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_UNCLASSIFIED ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Status                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 1 :
            retObj = "" ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Action Indicator                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+actionInd ;
            break;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            if ( getTitleArray() == null )
               createTitleArray ( title ) ;
            retObj = getTitleArray() ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 5 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;

            retObj = prettyDBUser ;
            break ;

         default:
            retObj = "" ;
            break ;
      }


      return( retObj ) ;
   }


   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( action ) ;
   }


   public String toString () {
      return( title ) ;
   }



   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert an entire Vector of
    *  Action objects to ActionDisplayRecs
    *  and will return a vector of the converted records
    *
    *  @param vec The vector to convert
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector vec ) {
      int    size   = vec.size() ;
      Vector retVec = new Vector ( size ) ;
      int    i      = 0 ;

      while ( i < size ) {
         if ( vec.elementAt(i) != null )
            retVec.addElement ( new ActionDisplayRec ( (Action) vec.elementAt(i) ) ) ;
         i++ ;
      }

      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* setAction
    */
   /**
    *    This method will set the action that this
    *  record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAction ( Action action ) {
      this.action = action ;
   }





   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will do the comparison between
    *  another ActionDisplayRec and this one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( IQuickSort sortObject ) {
      int rc = 0 ;

      if ( sortObject instanceof ActionDisplayRec ) {
         ActionDisplayRec displayRec = (ActionDisplayRec) sortObject ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Do the comparison based on what we have to sort         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( getSort() ) {
            case SORT_CHANGEDTIME_ASC :
               rc = CDate.compareDate ( changedTime, displayRec.changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_CHANGEDTIME_DESC :
               rc = CDate.compareDate ( displayRec.changedTime, changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_TITLE_ASC :
               rc = title.compareTo ( displayRec.title ) ;
               break ;

            case SORT_TITLE_DESC :
               rc = displayRec.title.compareTo ( title ) ;
               break ;

            case SORT_DBUSER_ASC :
               rc = dbUser.compareTo ( displayRec.dbUser ) ;
               break ;

            case SORT_DBUSER_DESC :
               rc = displayRec.dbUser.compareTo ( dbUser ) ;
               break ;

            case SORT_DOCCLASS :
               if ( docClassInd < displayRec.docClassInd )
                  rc = -1 ;
               else if ( docClassInd > displayRec.docClassInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_ASC :
               if ( actionInd < displayRec.actionInd )
                  rc = -1 ;
               else if ( actionInd > displayRec.actionInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_DESC :
               if ( actionInd < displayRec.actionInd )
                  rc = 1 ;
               else if ( actionInd > displayRec.actionInd )
                  rc = -1 ;
               else
                  rc = 0 ;
               break ;
         }

      }

      if ( rc > 0 )
         rc = 1 ;
      else if ( rc < 0 )
         rc = -1 ;

      return( rc ) ;
   }


} /* ActionDisplayRec */
