package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   ActionDraftDisplayRec                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionDraftDisplayRec               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
/**
 * This record simply shows a Action in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionDraftDisplayRec                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionDraftDisplayRec extends OAObjectDisplayRec implements IQuickSort,  DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000001 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int      actionInd         = 0 ;
   private int      docClassInd       = 0 ;
   private int      activeActionInd   = 0 ;
   private String   title             = null ;
   private String   changedTime       = null ;
   private String   dbUser            = null ;
   private String   prettyChangedTime = null ;
   private String   prettyDBUser      = null ;
   private String   startDate         = null ;
   private String   stopDate          = null ;
   private boolean  maintLock         = false ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionDraftDisplayRec () {
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionDraftDisplayRec ( int actionInd, int activeActionInd, int docClassInd, String title, String changedTime, String dbUser, String startDate, String stopDate ) {
      this.actionInd         = actionInd ;
      this.activeActionInd   = activeActionInd ;
      this.docClassInd       = docClassInd ;
      this.title             = title ;
      this.changedTime       = changedTime ;
      this.dbUser            = dbUser ;
      this.startDate         = startDate ;
      this.stopDate          = stopDate ;
      this.maintLock         = maintLock ;
   }


   /*------------------------------------------------------------*/
   /* setActionInd
    */
   /**
    *    This method will set our action indicator
    *  
    *  
    *
    *  @param actionInd The Action Draft Indicator to represent
    *  @param activeActionInd The Active Action indicator to represent
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setActionInd ( int actionInd, int activeActionInd ) {
      this.actionInd       = actionInd ;
      this.activeActionInd = activeActionInd ;
   }




   /*------------------------------------------------------------*/
   /* getActionInd
    */
   /**
    *    This method will return the action indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getActionInd () {
      return( actionInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getActiveActionInd
    */
   /**
    *    This method will return the active action
    *  indicator
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getActiveActionInd () {
      return( activeActionInd ) ;
   }

   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Classification                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            if ( docClassInd == TypeList.DOCCLASS_IBM_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ;
            else if ( docClassInd == TypeList.DOCCLASS_DEALER_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_DEALER_CONFIDENTIAL ) ;
            else
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_UNCLASSIFIED ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Status                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 1 :
            if ( maintLock )
               ImageSystem.getImageIcon ( this, ImageSystem.LOCKED ) ;
            else
               ImageSystem.getImageIcon ( this , ImageSystem.UNLOCKED ) ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Action Indicator                                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+actionInd ;
            break;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            if ( getTitleArray() == null )
               createTitleArray ( title ) ;
            retObj = getTitleArray() ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 5 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;

            retObj = prettyDBUser ;
            break ;

         case 6 :
            if ( startDate != null )
               retObj = startDate ;
            else
               retObj = "" ;
            break ;

         case 7 :
            if ( stopDate != null )
               retObj = stopDate ;
            else
               retObj = "" ;
            break ;

         default:
            retObj = "" ;
            break ;

      }


      return( retObj ) ;
   }


   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( null ) ;
   }






   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will do the comparison between
    *  another ActionDraftDisplayRec and this one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( IQuickSort sortObject ) {
      int rc = 0 ;

      if ( sortObject instanceof ActionDraftDisplayRec ) {
         ActionDraftDisplayRec displayRec = (ActionDraftDisplayRec) sortObject ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Do the comparison based on what we have to sort         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( getSort() ) {
            case SORT_CHANGEDTIME_ASC :
               rc = CDate.compareDate ( changedTime, displayRec.changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_CHANGEDTIME_DESC :
               rc = CDate.compareDate ( displayRec.changedTime, changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_TITLE_ASC :
               rc = title.compareTo ( displayRec.title ) ;
               break ;

            case SORT_TITLE_DESC :
               rc = displayRec.title.compareTo ( title ) ;
               break ;

            case SORT_DBUSER_ASC :
               rc = dbUser.compareTo ( displayRec.dbUser ) ;
               break ;

            case SORT_DBUSER_DESC :
               rc = displayRec.dbUser.compareTo ( dbUser ) ;
               break ;

            case SORT_DOCCLASS :
               if ( docClassInd < displayRec.docClassInd )
                  rc = -1 ;
               else if ( docClassInd > displayRec.docClassInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_ASC :
               if ( actionInd < displayRec.actionInd )
                  rc = -1 ;
               else if ( actionInd > displayRec.actionInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_DESC :
               if ( actionInd < displayRec.actionInd )
                  rc = 1 ;
               else if ( actionInd > displayRec.actionInd )
                  rc = -1 ;
               else
                  rc = 0 ;
               break ;
         }

      }

      if ( rc > 0 )
         rc = 1 ;
      else if ( rc < 0 )
         rc = -1 ;

      return( rc ) ;
   }
} /* ActionDraftDraftDisplayRec */
