package com.ibm.nzna.projects.qit.avalon.base ;       
/*-----------------------------------------------------------*/
/*   ActionPreviewRec                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionPreviewRec                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This record holds all of the information used to display a Preview of a Question. Nothing more 
 *
 *
 * @version 1.0, 4/24/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionPreviewRec                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionPreviewRec extends Object implements AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Preview Template                                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String previewTemplate= "<html><body bgcolor=\"#FFFFFF\">"+
                                          "<p><font size=\"5\" face=\"Arial, Helvetica, sans-serif\">TITLE</font></p>"+
                                          "<p><font size=\"2\" face=\"Arial, Helvetica, sans-serif\">"+
                                          "  <font color=\"#800080\">STR_TITLES</font><br>TITLE_TABLE\n"+
                                          "  <font color=\"#800080\">STR_ADDITIONAL_INFO</font>:<br>ADDITIONAL_INFO<br><br>"+
                                          "  <font color=\"#800080\">STR_GEOS</font>:<br>GEOS<br><br>"+
                                          "  <font color=\"#800080\">STR_INTERNAL_NUM</font>:INTERNAL_NUM<br>"+
                                          "  <font color=\"#800080\">STR_CREATE_DATE</font>:CREATEDATE<br>"+
                                          "  <font color=\"#800080\">STR_LAST_TOUCHED_ON</font>:LAST_TOUCHED_ON<br>"+
                                          "  <font color=\"#800080\">STR_LAST_TOUCHED_BY</font>:LAST_TOUCHED_BY<br>"+
                                          "  <font color=\"#800080\">STR_START_DATE</font>:START_DATE<br>"+
                                          "  <font color=\"#800080\">STR_STOP_DATE</font>:STOP_DATE</font><br>"+
                                          "</p>"+
                                          "</body>"+
                                          "</html>" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Members                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String   qualifier         = "oa" ;
   private int      actionInd       = 0 ;
   private String   title             = null ;
   private String   addInfo           = null ;
   private String   prettyCreateDate  = null ;
   private String   prettyChangedTime = null ;
   private String   prettyDbUser      = null ;
   private boolean  dataValid         = false ;
   private String   geos              = null ;
   private String   startDate         = null ;
   private String   stopDate          = null ;
   private String   titleTable        = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor currently for the
    *  ActionPreviewRec. It will take a ActionDisplayRec and
    *  read all preview information from the database from the actionInd
    *  that the ActionDisplayRec represents.
    *  
    *
    *  @param ActionDisplayRec The ActionDisplayRec to copy information from
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionPreviewRec ( ActionDisplayRec displayRec ) {
      actionInd = displayRec.getActionInd() ;
      readPreviewInfo() ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor currently for the
    *  ActionPreviewRec. It will take a ActionDisplayRec and
    *  read all preview information from the database from the actionInd
    *  that the ActionDisplayRec represents.
    *  
    *
    *  @param ActionDisplayRec The ActionDisplayRec to copy information from
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionPreviewRec ( ActionDraftDisplayRec displayRec ) {
      actionInd = displayRec.getActionInd() ;
      readDraftPreviewInfo() ;
   }






   /*------------------------------------------------------------*/
   /* getPreviewHTMLSource
    */
   /**
    *    This method will return the source code for
    *  the HTML needed to display this record
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getPreviewHTMLSource () {
      if ( dataValid ) {
         String retStr = Text.replaceInStr ( previewTemplate, "TITLE", title ) ;                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, replace all the 'Names'                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retStr = Text.replaceInStr ( retStr, "STR_INTERNAL_NUM",    Str.getStr ( STR_INTERNAL_NUM ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_CREATE_DATE",     Str.getStr ( STR_CREATION_DATE ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_LAST_TOUCHED_ON", Str.getStr ( STR_LAST_TOUCHED_ON ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_LAST_TOUCHED_BY", Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_ADDITIONAL_INFO", Str.getStr ( STR_ADDITIONAL_INFORMATION ) ) ;                       
         retStr = Text.replaceInStr ( retStr, "STR_GEOS",            Str.getStr ( STR_GEOGRAPHY ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_START_DATE",      Str.getStr ( STR_START_DATE ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_STOP_DATE",       Str.getStr ( STR_STOP_DATE ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_TITLES",          Str.getStr ( STR_TITLES ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, replace all of the 'Values'                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retStr = Text.replaceInStr ( retStr, "CREATEDATE",          prettyCreateDate ) ;
         retStr = Text.replaceInStr ( retStr, "LAST_TOUCHED_BY",     prettyDbUser ) ;
         retStr = Text.replaceInStr ( retStr, "LAST_TOUCHED_ON",     prettyChangedTime ) ;
         retStr = Text.replaceInStr ( retStr, "INTERNAL_NUM",        ""+actionInd ) ;
         retStr = Text.replaceInStr ( retStr, "ADDITIONAL_INFO",     addInfo ) ;
         retStr = Text.replaceInStr ( retStr, "GEOS",                geos ) ;
         retStr = Text.replaceInStr ( retStr, "START_DATE",          startDate ) ;
         retStr = Text.replaceInStr ( retStr, "STOP_DATE",           stopDate ) ;
         retStr = Text.replaceInStr ( retStr, "TITLE_TABLE",         titleTable ) ;


         return( retStr ) ;
      } else
         return( "<html><body><center>"+Str.getStr ( STR_QUESTION_NOT_FOUND )+"</center><br><br><b>ActionInd:</b>"+actionInd+"</body></html>" );
   }




   /*------------------------------------------------------------*/
   /* readPreviewInfo
    */
   /**
    *    This method will read in all of our preview
    *  information
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readPreviewInfo () {
      AppDefaultWin parentWin = MainWindow.getInstance() ;

      try {
         Action      action  = null ;                  


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the Action object                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parentWin.setStatus ( Str.getStr ( STR_READING ) ) ;
         action = AvalonCache.readAction ( actionInd, false ) ;          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Populate our values                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( action != null ) {
            Vector titles    = action.getTitles() ;    
            Vector addInfos  = action.getAdditionalInfo() ;
            Vector countries = action.getCountryList() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Title                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( titles != null ) && ( titles.size() > 0 ) )
               title = titles.elementAt(titles.size()-1).toString() ;
            else
               title = Str.getStr ( STR_UNKNOWN ) ;             


            if ( ( addInfos != null ) && ( addInfos.size() > 0 ) )
               addInfo = addInfos.elementAt(addInfos.size()-1).toString() ;
            else
               addInfo = " " ;          


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Geographies                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( countries != null ) && ( countries.size() > 0 ) ) {
               int i    = 0 ;
               int size = countries.size() ;

               geos="" ;
               while ( i < size ) {
                  geos+=countries.elementAt(i).toString() ;
                  if ( i < (size-1) )
                     geos+=", " ;
                  i++ ;
               }

            } else
               geos = " " ;



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Dates and User                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            startDate         = " " ;
            stopDate          = " " ;
            prettyCreateDate  = action.getCreationTime() ;
            prettyChangedTime = DateSystem.prettyDateFromStamp ( action.getChangedTime() ) ;
            prettyDbUser      = UserSystem.getNameFromUserId ( action.getDbUser() ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Format the titles                                       */        
            /*                                                           */
            /*-----------------------------------------------------------*/
            formatTitleTable ( action ) ;
            

         }

         dataValid = true ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         dataValid = false ;
      }

      parentWin.setStatus ( null ) ;
   }




   /*------------------------------------------------------------*/
   /* readDraftPreviewInfo
    */
   /**
    *    This method will read in all of our preview
    *  information for a draft
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readDraftPreviewInfo () {      AppDefaultWin parentWin = MainWindow.getInstance() ;

      try {
         ActionDraft action  = null ;                  


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the Action object                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parentWin.setStatus ( Str.getStr ( STR_READING ) ) ;
         action = AvalonCache.readActionDraft ( actionInd, false ) ;          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Populate our values                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( action != null ) {
            Vector titles    = action.getTitles() ;    
            Vector addInfos  = action.getAdditionalInfo() ;
            Vector countries = action.getCountryList() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Title                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( titles != null ) && ( titles.size() > 0 ) )
               title = titles.elementAt(titles.size()-1).toString() ;
            else
               title = Str.getStr ( STR_UNKNOWN ) ;             


            if ( ( addInfos != null ) && ( addInfos.size() > 0 ) )
               addInfo = addInfos.elementAt(addInfos.size()-1).toString() ;
            else
               addInfo = " " ;          


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Geographies                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( countries != null ) && ( countries.size() > 0 ) ) {
               int i    = 0 ;
               int size = countries.size() ;

               geos="" ;
               while ( i < size ) {
                  geos+=countries.elementAt(i).toString() ;
                  if ( i < (size-1) )
                     geos+=", " ;
                  i++ ;
               }

            } else
               geos = " " ;



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Dates and User                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            startDate         = DateSystem.prettyDateFromDBDate ( action.getStartDate() ) ;
            stopDate          = DateSystem.prettyDateFromDBDate ( action.getStopDate() ) ;
            prettyCreateDate  = action.getCreationTime() ;
            prettyChangedTime = DateSystem.prettyDateFromStamp ( action.getChangedTime() ) ;
            prettyDbUser      = UserSystem.getNameFromUserId ( action.getDbUser() ) ;

            if ( stopDate == null )
               stopDate = "" ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Format the titles                                       */        
            /*                                                           */
            /*-----------------------------------------------------------*/
            formatTitleTable ( (Action)action ) ;


         }

         dataValid = true ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         dataValid = false ;
      }

      parentWin.setStatus ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* formatTitleTable
    */
   /**
    *    This method format the titleTable property
    *  given the Action passed in. 
    *  
    *
    *  @param action The Action to format the Title Table fo
    *  @param 
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void formatTitleTable ( Action action ) {
      if ( action != null ) {
         Vector titles = action.getTitles() ;

         if ( ( titles != null ) && ( titles.size() > 0 ) ) {
            int           i    = 0 ;
            int           size = titles.size() ;
            StringBuffer  table = new StringBuffer ( "</font><TABLE BORDER=0>" ) ;
            Title         title = null ;

            while ( i < size ) {

               title = (Title) titles.elementAt(i) ;
               table.append ( "<tr>" ) ;                      

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get the Classification Image in                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               table.append ( "<td><img src=\"file:/" ) ;
               table.append ( PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ) ;
               table.append ( File.separator ) ;

               switch ( ((TypeDocClassRec)title.getDocClass()).getInd() ) {
                  case TypeList.DOCCLASS_IBM_CONFIDENTIAL :
                     table.append ( "ibm_confidential.gif" ) ;
                     break ;

                  case TypeList.DOCCLASS_DEALER_CONFIDENTIAL :
                     table.append ( "ibm_dealer_confidential.gif" ) ;
                     break ;

                  case TypeList.DOCCLASS_UNCLASSIFIED :
                     table.append ( "unclassified.gif" ) ;
                     break;
               }
               table.append ( "\"></td>" );       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Put the Title in                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               table.append ( "<td><font size=\"2\" face=\"Arial, Helvetica, sans-serif\">" ) ;
               table.append ( title.toString() ) ;
               table.append ( "</td>" ) ;  

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   End the row                                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               table.append ( "</tr>" ) ;

               i++ ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   End the table                                           */    
            /*                                                           */
            /*-----------------------------------------------------------*/
            table.append ( "</table><br><font size=\"2\" face=\"Arial, Helvetica, sans-serif\">" ) ;


            titleTable = table.toString() ;

         } else
            titleTable = " " ;

      } else {
         titleTable=" " ;
      }
   }

} /* ActionPreviewRec */
