package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   AddtlInfoGraphicFTP                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AddtlInfoGraphicFTP                 */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.net.* ;
import java.io.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.ftp.* ;      
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;      
/**
 *  
 * This class will FTP all of the graphics from the web sites down to
 * the temp directory for an AddtlInfo object.
 *
 * @version 1.0, 9/8/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AddtlInfoGraphicFTP                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class AddtlInfoGraphicFTP extends Object implements AppConst {




   /*------------------------------------------------------------*/
   /* getGraphicsPub
    */
   /**
    *    This method will get all of the graphics that
    *  are contained in the AddtlInfo object
    *  passed in. The graphics are stored in the temp directory<br><br>
    *  PLEASE NOTE that this method gets graphics only from the published directory
    *
    *  @param parentWin If you want messages posted, pass the window to post the messages in
    *  @param addtlInfo The AddtlInfo object to get graphics from
    *  @param imageCompleteVec The vector of images to discard from the process
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean getGraphicsPub ( AppDefaultWin parentWin, AddtlInfo addtlInfo, Vector imageCompleteVec ) {
      Vector   graphicVec      = addtlInfo.getEmbeddedGraphics() ;
      boolean  readAll         = false ;
      String   currentFilename = "" ;

      if ( ( graphicVec != null ) && ( graphicVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = graphicVec.size() ;

         LogSystem.beginLogSection ( "FTPing graphics for Additional Info Object. FTPing "+size+" Graphic(s)" ) ;
         if ( parentWin != null )
            setStatus ( parentWin,  STR_RECEIVING_FILE ) ;

         try {
            Ftp ftp = new Ftp() ;
            String hostDir  = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_LOC_PUB" ) ;
            String host     = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_PUB" ) ;
            String localDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

            LogSystem.log ( 1, "Host:"+host ) ;
            LogSystem.log ( 1, "Host Directory:"+hostDir ) ;
            LogSystem.log ( 1, "Local Directory:"+localDir ) ;                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the properties of the FTP Bean and connect          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setHost ( host ) ;
            ftp.setUser ( "quest" ) ;
            ftp.setPassword ( "j0hnny" ) ;
            ftp.connect() ;
            ftp.login() ;
            LogSystem.log ( 1, "Connected to "+host ) ;
            ftp.setType ( FtpClient.IMAGE ) ;
            LogSystem.log ( 1, "Set Transfer type to IMAGE" ) ;          


            if ( ftp.isConnected() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the directories                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( hostDir ) ;
               LogSystem.log ( 1, "Entered remote directory:"+hostDir ) ;                       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through, grabbing each file                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {

                  currentFilename = ((AddtlInfoGraphic)graphicVec.elementAt(i)).getFilename() ;

                  if ( !imageProcessed ( currentFilename, imageCompleteVec ) ) {
                     if ( size > 1 )
                        setStatus ( parentWin,  Str.getStr ( STR_RECEIVING_FILE )+" "+currentFilename+"  ("+i+" "+Str.getStr ( STR_OF )+" "+size+")" ) ;
                     else
                        setStatus ( parentWin,  Str.getStr ( STR_RECEIVING_FILE )+" "+currentFilename ) ;

                     LogSystem.log ( 1, "Downloading file:"+currentFilename+":  To :"+localDir+File.separator+currentFilename+":" ) ;
                     ftp.get ( currentFilename, localDir+File.separator+currentFilename ) ;
                     imageCompleteVec.addElement ( currentFilename ) ;
                  }
                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, close the connection and exit                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.disconnect() ;
               readAll = true ;

            } /* if connected */


         } catch ( UnknownHostException uhe ) {
            LogSystem.log ( 1, uhe, false ) ;
            GUISystem.printBox ( STR_INFO, STR_FTP_SITE_NOT_FOUND ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_NOT_ALL_GRAPHICS_DOWNLOADED )+" "+currentFilename ) ;
            readAll = false ;
         }

         LogSystem.endLogSection() ;
      } else
         readAll = true ; // Well, there were none to read!


      return( readAll ) ;
   }



   /*------------------------------------------------------------*/
   /* putGraphicsPub
    */
   /**
    *    This method will put all of the graphics that
    *  are contained in the AddtlInfo object
    *  passed in. The graphics are stored in the temp directory.<br><br>
    *  PLEASE NOTE that this method puts the graphics into the PUBLISHED directory!
    *
    *  @param parentWin If you want messages posted, pass the window to post the messages in
    *  @param addtlInfo The AddtlInfo object to get graphics from
    *  @param imageCompleteVec The vector of all images to discard from the process
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean putGraphicsPub ( AppDefaultWin parentWin, AddtlInfo addtlInfo, Vector imageCompleteVec ) {
      Vector   graphicVec      = addtlInfo.getEmbeddedGraphics() ;
      boolean  putAll          = false ;
      String   currentFilename = "" ;
      int      recStatus       = 0 ;

      if ( ( graphicVec != null ) && ( graphicVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = graphicVec.size() ;

         LogSystem.beginLogSection ( "FTPing graphics for Additional Info Object. FTPing "+size+" Graphic(s)" ) ;
         if ( parentWin != null )
            setStatus ( parentWin,  STR_SENDING_FILE ) ;

         try {
            Ftp ftp = new Ftp() ;
            String hostDir  = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_LOC_PUB" ) ;
            String host     = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_PUB" ) ;
            String localDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

            LogSystem.log ( 1, "Host:"+host ) ;
            LogSystem.log ( 1, "Host Directory:"+hostDir ) ;
            LogSystem.log ( 1, "Local Directory:"+localDir ) ;                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the properties of the FTP Bean and connect          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setHost ( host ) ;
            ftp.setUser ( "quest" ) ;
            ftp.setPassword ( "j0hnny" ) ;
            ftp.connect() ;
            ftp.login() ;
            LogSystem.log ( 1, "Connected to "+host ) ;
            ftp.setType ( FtpClient.IMAGE ) ;
            LogSystem.log ( 1, "Set Transfer type to IMAGE" ) ;          


            if ( ftp.isConnected() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the directories                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( hostDir ) ;
               LogSystem.log ( 1, "Entered remote directory:"+hostDir ) ;                       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through, putting each file                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {

                  currentFilename = ((AddtlInfoGraphic)graphicVec.elementAt(i)).getFilename() ;
                  recStatus       = ((AddtlInfoGraphic)graphicVec.elementAt(i)).getRecStatus() ;

                  if ( !imageProcessed ( currentFilename, imageCompleteVec ) ) {

                     if ( size > 1 )
                        setStatus ( parentWin,  Str.getStr ( (recStatus != AddtlInfoGraphic.REC_DELETE) ? STR_SENDING_FILE : STR_REMOVING_TEMP_FILE )+" "+currentFilename+"  ("+i+" "+Str.getStr ( STR_OF )+" "+size+")" ) ;
                     else
                        setStatus ( parentWin,  Str.getStr ( (recStatus != AddtlInfoGraphic.REC_DELETE) ? STR_SENDING_FILE : STR_REMOVING_TEMP_FILE )+" "+currentFilename ) ;                                            

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   If the status is anything except DELETE and NOINSERT,   */
                     /* then put the file up                                      */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( recStatus != AddtlInfoGraphic.REC_DELETE ) {
                        LogSystem.log ( 1, "Uploading from:"+localDir+File.separator+currentFilename+": To:"+hostDir+"/"+currentFilename ) ;
                        ftp.put ( localDir+File.separator+currentFilename, currentFilename, false ) ;
                     }

                     /*----------------------------------------------------------------*/
                     /*                                                                */
                     /*   Otherwise, delete the graphic if the rec status is DELETE    */
                     /*                                                                */
                     /*----------------------------------------------------------------*/
                     else if ( recStatus == AddtlInfoGraphic.REC_DELETE ) {
                        LogSystem.log ( 1, "Deleting from:"+hostDir+"/"+currentFilename ) ;
                        try {
                           ftp.delete ( currentFilename ) ;
                        } catch ( Exception e ) {
                           LogSystem.log ( 1, e, false ) ;
                        }
                     } else {
                        LogSystem.log ( 1, "No FTP Action for File :"+currentFilename+": with rec status:"+recStatus ) ;
                     }

                     imageCompleteVec.addElement ( currentFilename ) ;
                  }
                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, close the connection and exit                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.disconnect() ;

            } /* if connected */


         } catch ( UnknownHostException uhe ) {
            LogSystem.log ( 1, uhe, false ) ;
            GUISystem.printBox ( STR_INFO, STR_FTP_SITE_NOT_FOUND ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_NOT_ALL_GRAPHICS_DOWNLOADED )+" "+currentFilename ) ;
            putAll = false ;
         }

         LogSystem.endLogSection() ;
      } else
         putAll = true ; // Well, there were none to put!


      setStatus ( parentWin, null ) ;

      return( putAll ) ;
   }



   /*------------------------------------------------------------*/
   /* getGraphicsDraft
    */
   /**
    *    This method will get all of the graphics that
    *  are contained in the AddtlInfo object
    *  passed in. The graphics are stored in the temp directory<br><br>
    *  PLEASE NOTE that this method gets graphics only from the draft directory
    *
    *  @param parentWin If you want messages posted, pass the window to post the messages in
    *  @param addtlInfo The AddtlInfo object to get graphics from
    *  @param imageCompleteVec The vector of images to discard from the process
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean getGraphicsDraft ( AppDefaultWin parentWin, AddtlInfo addtlInfo, Vector imageCompleteVec ) {
      Vector   graphicVec      = addtlInfo.getEmbeddedGraphics() ;
      boolean  readAll         = false ;
      String   currentFilename = "" ;

      if ( ( graphicVec != null ) && ( graphicVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = graphicVec.size() ;

         LogSystem.beginLogSection ( "FTPing graphics for Additional Info Object. FTPing "+size+" Graphic(s)" ) ;
         if ( parentWin != null )
            setStatus ( parentWin,  STR_RECEIVING_FILE ) ;

         try {
            Ftp ftp = new Ftp() ;
            String hostDir  = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_LOC_DRFT" ) ;
            String host     = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_DRFT" ) ;
            String localDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

            LogSystem.log ( 1, "Host:"+host ) ;
            LogSystem.log ( 1, "Host Directory:"+hostDir ) ;
            LogSystem.log ( 1, "Local Directory:"+localDir ) ;                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the properties of the FTP Bean and connect          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setHost ( host ) ;
            ftp.setUser ( "quest" ) ;
            ftp.setPassword ( "j0hnny" ) ;
            ftp.connect() ;
            ftp.login() ;
            LogSystem.log ( 1, "Connected to "+host ) ;
            ftp.setType ( FtpClient.IMAGE ) ;
            LogSystem.log ( 1, "Set Transfer type to IMAGE" ) ;          


            if ( ftp.isConnected() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the directories                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( hostDir ) ;
               LogSystem.log ( 1, "Entered remote directory:"+hostDir ) ;                       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through, grabbing each file                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {

                  currentFilename = ((AddtlInfoGraphic)graphicVec.elementAt(i)).getFilename() ;

                  if ( !imageProcessed ( currentFilename, imageCompleteVec ) ) {
                     if ( size > 1 )
                        setStatus ( parentWin,  Str.getStr ( STR_RECEIVING_FILE )+" "+currentFilename+"  ("+i+" "+Str.getStr ( STR_OF )+" "+size+")" ) ;
                     else
                        setStatus ( parentWin,  Str.getStr ( STR_RECEIVING_FILE )+" "+currentFilename ) ;

                     LogSystem.log ( 1, "Downloading file:"+currentFilename+":  To :"+localDir+File.separator+currentFilename+":" ) ;
                     ftp.get ( currentFilename, localDir+File.separator+currentFilename ) ;
                     imageCompleteVec.addElement ( currentFilename ) ;
                  }
                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, close the connection and exit                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.disconnect() ;


               readAll = true ;
            } /* if connected */


         } catch ( UnknownHostException uhe ) {
            LogSystem.log ( 1, uhe, false ) ;
            GUISystem.printBox ( STR_INFO, STR_FTP_SITE_NOT_FOUND ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_NOT_ALL_GRAPHICS_DOWNLOADED )+" "+currentFilename ) ;
            readAll = false ;
         }

         LogSystem.endLogSection() ;
      } else
         readAll = true ; // Well, there were none to read!


      return( readAll ) ;
   }



   /*------------------------------------------------------------*/
   /* putGraphicsDraft
    */
   /**
    *    This method will put all of the graphics that
    *  are contained in the AddtlInfo object
    *  passed in. The graphics are stored in the temp directory.<br><br>
    *  PLEASE NOTE that this method puts the graphics into the PUBLISHED directory!
    *
    *  @param parentWin If you want messages posted, pass the window to post the messages in
    *  @param addtlInfo The AddtlInfo object to get graphics from
    *  @param completeImageVec The vector containing all images to discard from the process
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean putGraphicsDraft ( AppDefaultWin parentWin, AddtlInfo addtlInfo, Vector completeImageVec ) {
      Vector   graphicVec      = addtlInfo.getEmbeddedGraphics() ;
      boolean  putAll          = false ;
      String   currentFilename = "" ;
      int      recStatus       = 0 ;

      LogSystem.beginLogSection ( "Beginning FTP for Graphics - DRAFT" ) ;

      if ( ( graphicVec != null ) && ( graphicVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = graphicVec.size() ;

         LogSystem.log ( 1, "FTPing "+size+" Graphic(s)" ) ;
         if ( parentWin != null )
            setStatus ( parentWin,  STR_SENDING_FILE ) ;

         try {
            Ftp ftp = new Ftp() ;
            String hostDir  = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_LOC_DRFT" ) ;
            String host     = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_DRFT" ) ;
            String localDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

            LogSystem.log ( 1, "Host:"+host ) ;
            LogSystem.log ( 1, "Host Directory:"+hostDir ) ;
            LogSystem.log ( 1, "Local Directory:"+localDir ) ;                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the properties of the FTP Bean and connect          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setHost ( host ) ;
            ftp.setUser ( "quest" ) ;
            ftp.setPassword ( "j0hnny" ) ;
            ftp.connect() ;
            ftp.login() ;
            LogSystem.log ( 1, "Connected to "+host ) ;
            ftp.setType ( FtpClient.IMAGE ) ;
            LogSystem.log ( 1, "Set Transfer type to IMAGE" ) ;          


            if ( ftp.isConnected() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the directories                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( hostDir ) ;
               LogSystem.log ( 1, "Entered remote directory:"+hostDir ) ;                       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through, grabbing each file                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {

                  currentFilename = ((AddtlInfoGraphic)graphicVec.elementAt(i)).getFilename() ;
                  recStatus       = ((AddtlInfoGraphic)graphicVec.elementAt(i)).getRecStatus() ;

                  if ( !imageProcessed ( currentFilename, completeImageVec ) ) {
                     if ( size > 1 )
                        setStatus ( parentWin,  Str.getStr ( (recStatus != AddtlInfoGraphic.REC_DELETE) ? STR_SENDING_FILE : STR_REMOVING_TEMP_FILE )+" "+currentFilename+"  ("+i+" "+Str.getStr ( STR_OF )+" "+size+")" ) ;
                     else
                        setStatus ( parentWin,  Str.getStr ( (recStatus != AddtlInfoGraphic.REC_DELETE) ? STR_SENDING_FILE : STR_REMOVING_TEMP_FILE )+" "+currentFilename ) ;

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   If the status is anything except DELETE and NOINSERT,   */
                     /* then put the file up                                      */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( recStatus != AddtlInfoGraphic.REC_DELETE ) {
                        LogSystem.log ( 1, "Uploading from:"+localDir+File.separator+currentFilename+": To:"+hostDir+"/"+currentFilename ) ;
                        ftp.put ( localDir+File.separator+currentFilename, currentFilename, false ) ;
                     }

                     /*----------------------------------------------------------------*/
                     /*                                                                */
                     /*   Otherwise, delete the graphic if the rec status is DELETE    */
                     /*                                                                */
                     /*----------------------------------------------------------------*/
                     else if ( recStatus == AddtlInfoGraphic.REC_DELETE ) {
                        LogSystem.log ( 1, "Deleting from:"+hostDir+"/"+currentFilename ) ;
                        try {
                           ftp.delete ( currentFilename ) ;
                        } catch ( Exception e ) {
                           LogSystem.log ( 1 ,e ,false ) ;
                        }
                     } else {
                        LogSystem.log ( 1, "No FTP Action for File :"+currentFilename+": with rec status:"+recStatus ) ;
                     }

                     completeImageVec.addElement ( currentFilename ) ;
                  }
                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, close the connection and exit                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.disconnect() ;

               putAll = true ;

            } /* if connected */


         } catch ( UnknownHostException uhe ) {
            LogSystem.log ( 1, uhe, false ) ;
            GUISystem.printBox ( STR_INFO, STR_FTP_SITE_NOT_FOUND ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_NOT_ALL_GRAPHICS_DOWNLOADED )+" "+currentFilename ) ;
            putAll = false ;
         }

         LogSystem.endLogSection() ;
      } else
         putAll = true ; // Well, there were none to read!


      return( putAll ) ;
   }


   /*------------------------------------------------------------*/
   /* removeGraphicsDraft
    */
   /**
    *    This method will remove all of the graphics for this draft
    *  from the draft directory.
    *
    *  @param parentWin If you want messages posted, pass the window to post the messages in
    *  @param addtlInfo The AddtlInfo object to get graphics from
    *  @param completeImageVec This is the vector of all images to discard from the process
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean removeGraphicsDraft ( AppDefaultWin parentWin, AddtlInfo addtlInfo, Vector completeImageVec ) {
      Vector   graphicVec      = addtlInfo.getEmbeddedGraphics() ;
      boolean  removedAll      = false ;
      String   currentFilename = "" ;

      if ( ( graphicVec != null ) && ( graphicVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = graphicVec.size() ;

         LogSystem.beginLogSection ( "Removing graphics for Additional Info Object. FTPing "+size+" Graphic(s)" ) ;
         if ( parentWin != null )
            setStatus ( parentWin,  STR_REMOVING_TEMP_FILE ) ;

         try {
            Ftp ftp = new Ftp() ;
            String hostDir  = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_LOC_DRFT" ) ;
            String host     = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_DRFT" ) ;
            String localDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

            LogSystem.log ( 1, "Host:"+host ) ;
            LogSystem.log ( 1, "Host Directory:"+hostDir ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the properties of the FTP Bean and connect          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setHost ( host ) ;
            ftp.setUser ( "quest" ) ;
            ftp.setPassword ( "j0hnny" ) ;
            ftp.connect() ;
            ftp.login() ;
            LogSystem.log ( 1, "Connected to "+host ) ;

            if ( ftp.isConnected() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the directories                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( hostDir ) ;
               LogSystem.log ( 1, "Entered remote directory:"+hostDir ) ;                       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through, grabbing each file                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {

                  currentFilename = ((AddtlInfoGraphic)graphicVec.elementAt(i)).getFilename() ;

                  if ( !imageProcessed ( currentFilename, completeImageVec ) ) {
                     if ( parentWin != null ) {
                        if ( size > 1 )
                           setStatus ( parentWin,  Str.getStr ( STR_REMOVING_TEMP_FILE )+" "+currentFilename+"  ("+i+" "+Str.getStr ( STR_OF )+" "+size+")" ) ;
                        else
                           setStatus ( parentWin,  Str.getStr ( STR_REMOVING_TEMP_FILE )+" "+currentFilename ) ;
                     }
                  }


                  LogSystem.log ( 1, "Removing file:"+currentFilename ) ;
                  try {
                     ftp.delete ( currentFilename ) ;
                  } catch ( Exception e ) {
                     LogSystem.log ( 1, e ,false ) ; // Just log the error, it probably says that the file doesn't exist
                  }
                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, close the connection and exit                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.disconnect() ;

               removedAll = true ;

            } /* if connected */


         } catch ( UnknownHostException uhe ) {
            LogSystem.log ( 1, uhe, false ) ;
            GUISystem.printBox ( STR_INFO, STR_FTP_SITE_NOT_FOUND ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_NOT_ALL_GRAPHICS_DOWNLOADED )+" "+currentFilename ) ;
            removedAll = false ;
         }

         LogSystem.endLogSection() ;
      } else
         removedAll = true ; // Well, there were nothing to remove!

      setStatus ( parentWin, null ) ;

      return( removedAll ) ;
   }






   /*------------------------------------------------------------*/
   /* imageProcessed
    */
   /**
    *    this method will determine if an image has
    *  already been processed. With multiple AddtlInfo
    *  objects that share the same graphics, this will be the case many times
    *
    *  @param filename The filename in question
    *  @param fileVec The vector of files already processed
    *  @param None
    *
    *  @return TRUE if the image has been processed, FALSE if not
    *//*---------------------------------------------------------*/
   private static boolean imageProcessed ( String filename, Vector fileVec ) {
      boolean processed = false ;

      if ( ( fileVec != null ) && ( fileVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = fileVec.size() ;

         while ( ( !processed ) && ( i < size ) ) {
            processed = ((String)fileVec.elementAt(i)).equals ( filename ) ;
            i++ ;
         }
      }

      return( processed ) ;
   }





   /*------------------------------------------------------------*/
   /* getGraphicsDraft
    */
   /**
    *    This method will get all of the graphics for
    *  all of the AddtlInfo objects in the vector passed in.
    *  
    *
    *  @param parentWin The parent window to output to. This can be null
    *  @param addInfoVec The vector containing AddtlInfo objects
    *  @param None
    *
    *  @return TRUE if everything processed Ok, FALSE if an error has happened
    *//*---------------------------------------------------------*/
   public static boolean getGraphicsDraft ( AppDefaultWin parentWin, Vector addInfoVec ) {
      boolean processed = false ;

      if ( ( addInfoVec != null ) && ( addInfoVec.size() > 0 ) ) {
         int     i                = 0 ;
         int     size             = addInfoVec.size() ;
         boolean cont             = true ;
         Vector  imageCompleteVec = new Vector ( 10, 10 ) ;

         while ( ( cont ) && ( i < size ) ) {
            cont = getGraphicsDraft ( parentWin, (AddtlInfo)addInfoVec.elementAt(i), imageCompleteVec ) ;
            i++ ;
         }
         processed = cont ;

      } else
         processed = true ;

      setStatus ( parentWin, null ) ;
      return( processed ) ;
   }


   /*------------------------------------------------------------*/
   /* getGraphicsPub
    */
   /**
    *    This method will get all of the graphics for
    *  all of the AddtlInfo objects in the vector passed in.
    *  
    *
    *  @param parentWin The parent window to output to. This can be null
    *  @param addInfoVec The vector containing AddtlInfo objects
    *  @param None
    *
    *  @return TRUE if everything processed Ok, FALSE if an error has happened
    *//*---------------------------------------------------------*/
   public static boolean getGraphicsPub ( AppDefaultWin parentWin, Vector addInfoVec ) {
      boolean processed = false ;

      if ( ( addInfoVec != null ) && ( addInfoVec.size() > 0 ) ) {
         int     i                = 0 ;
         int     size             = addInfoVec.size() ;
         boolean cont             = true ;
         Vector  imageCompleteVec = new Vector ( 10, 10 ) ;

         while ( ( cont ) && ( i < size ) ) {
            cont = getGraphicsPub ( parentWin, (AddtlInfo)addInfoVec.elementAt(i), imageCompleteVec ) ;
            i++ ;
         }
         processed = cont ;

      } else
         processed = true ;

      setStatus ( parentWin, null ) ;
      return( processed ) ;
   }


   /*------------------------------------------------------------*/
   /* putGraphicsPub
    */
   /**
    *    This method will put all of the graphics for
    *  all of the AddtlInfo objects in the vector passed in.
    *  
    *
    *  @param parentWin The parent window to output to. This can be null
    *  @param addInfoVec The vector containing AddtlInfo objects
    *  @param None
    *
    *  @return TRUE if everything processed Ok, FALSE if an error has happened
    *//*---------------------------------------------------------*/
   public static boolean putGraphicsPub ( AppDefaultWin parentWin, Vector addInfoVec ) {
      boolean processed = false ;

      if ( ( addInfoVec != null ) && ( addInfoVec.size() > 0 ) ) {
         int     i                = 0 ;
         int     size             = addInfoVec.size() ;
         boolean cont             = true ;
         Vector  imageCompleteVec = new Vector ( 10, 10 ) ;

         while ( ( cont ) && ( i < size ) ) {
            if ( ((AddtlInfo)addInfoVec.elementAt(i)).valid() )
               cont = putGraphicsPub ( parentWin, (AddtlInfo)addInfoVec.elementAt(i), imageCompleteVec ) ;
            else
               cont = removeGraphicsPub ( parentWin, (AddtlInfo)addInfoVec.elementAt(i), imageCompleteVec ) ;
            i++ ;
         }
         processed = cont ;

      } else
         processed = true ;

      setStatus ( parentWin, null ) ;
      return( processed ) ;
   }



   /*------------------------------------------------------------*/
   /* putGraphicsDraft
    */
   /**
    *    This method will put all of the graphics for
    *  all of the AddtlInfo objects in the vector passed in.
    *  
    *
    *  @param parentWin The parent window to output to. This can be null
    *  @param addInfoVec The vector containing AddtlInfo objects
    *  @param None
    *
    *  @return TRUE if everything processed Ok, FALSE if an error has happened
    *//*---------------------------------------------------------*/
   public static boolean putGraphicsDraft ( AppDefaultWin parentWin, Vector addInfoVec ) {
      boolean processed = false ;

      if ( ( addInfoVec != null ) && ( addInfoVec.size() > 0 ) ) {
         int     i                = 0 ;
         int     size             = addInfoVec.size() ;
         boolean cont             = true ;
         Vector  imageCompleteVec = new Vector ( 10, 10 ) ;

         while ( ( cont ) && ( i < size ) ) {
            if ( ((AddtlInfo)addInfoVec.elementAt(i)).valid() )
               cont = putGraphicsDraft ( parentWin, (AddtlInfo)addInfoVec.elementAt(i), imageCompleteVec ) ;
            else
               cont = removeGraphicsDraft ( parentWin, (AddtlInfo)addInfoVec.elementAt(i), imageCompleteVec ) ;
            i++ ;
         }
         processed = cont ;

      } else
         processed = true ;

      setStatus ( parentWin, null ) ;
      return( processed ) ;
   }


   /*------------------------------------------------------------*/
   /* removeGraphicsDraft
    */
   /**
    *    This method will remove all of the graphics for
    *  all of the AddtlInfo objects in the vector passed in.
    *  
    *
    *  @param parentWin The parent window to outremove to. This can be null
    *  @param addInfoVec The vector containing AddtlInfo objects
    *  @param None
    *
    *  @return TRUE if everything processed Ok, FALSE if an error has happened
    *//*---------------------------------------------------------*/
   public static boolean removeGraphicsDraft ( AppDefaultWin parentWin, Vector addInfoVec ) {
      boolean processed = false ;

      if ( ( addInfoVec != null ) && ( addInfoVec.size() > 0 ) ) {
         int     i                = 0 ;
         int     size             = addInfoVec.size() ;
         boolean cont             = true ;
         Vector  imageCompleteVec = new Vector ( 10, 10 ) ;

         while ( ( cont ) && ( i < size ) ) {
            cont = removeGraphicsDraft ( parentWin, (AddtlInfo)addInfoVec.elementAt(i), imageCompleteVec ) ;
            i++ ;
         }
         processed = cont ;

      } else
         processed = true ;

      return( processed ) ;
   }




   /*------------------------------------------------------------*/
   /* removeGraphicsPub
    */
   /**
    *    This method will remove all of the graphics for this add info
    *  from the pub directory.
    *
    *  @param parentWin If you want messages posted, pass the window to post the messages in
    *  @param addtlInfo The AddtlInfo object to get graphics from
    *  @param completeImageVec This is the vector of all images to discard from the process
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean removeGraphicsPub ( AppDefaultWin parentWin, AddtlInfo addtlInfo, Vector completeImageVec ) {
      Vector   graphicVec      = addtlInfo.getEmbeddedGraphics() ;
      boolean  removedAll      = false ;
      String   currentFilename = "" ;

      if ( ( graphicVec != null ) && ( graphicVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = graphicVec.size() ;

         LogSystem.beginLogSection ( "Removing graphics for Additional Info Object. FTPing "+size+" Graphic(s)" ) ;
         if ( parentWin != null )
            setStatus ( parentWin,  STR_REMOVING_TEMP_FILE ) ;

         try {
            Ftp ftp = new Ftp() ;
            String hostDir  = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_LOC_PUB" ) ;
            String host     = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_PUB" ) ;
            String localDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

            LogSystem.log ( 1, "Host:"+host ) ;
            LogSystem.log ( 1, "Host Directory:"+hostDir ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the properties of the FTP Bean and connect          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setHost ( host ) ;
            ftp.setUser ( "quest" ) ;
            ftp.setPassword ( "j0hnny" ) ;
            ftp.connect() ;
            ftp.login() ;
            LogSystem.log ( 1, "Connected to "+host ) ;

            if ( ftp.isConnected() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the directories                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( hostDir ) ;
               LogSystem.log ( 1, "Entered remote directory:"+hostDir ) ;                       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through, grabbing each file                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {

                  currentFilename = ((AddtlInfoGraphic)graphicVec.elementAt(i)).getFilename() ;

                  if ( !imageProcessed ( currentFilename, completeImageVec ) ) {
                     if ( parentWin != null ) {
                        if ( size > 1 )
                           setStatus ( parentWin,  Str.getStr ( STR_REMOVING_TEMP_FILE )+" "+currentFilename+"  ("+i+" "+Str.getStr ( STR_OF )+" "+size+")" ) ;
                        else
                           setStatus ( parentWin,  Str.getStr ( STR_REMOVING_TEMP_FILE )+" "+currentFilename ) ;
                     }
                  }


                  LogSystem.log ( 1, "Removing file:"+currentFilename ) ;
                  try {
                     ftp.delete ( currentFilename ) ;
                  } catch ( Exception e ) {
                     LogSystem.log ( 1, e ,false ) ;
                  }
                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, close the connection and exit                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.disconnect() ;

               removedAll = true ;

            } /* if connected */


         } catch ( UnknownHostException uhe ) {
            LogSystem.log ( 1, uhe, false ) ;
            GUISystem.printBox ( STR_INFO, STR_FTP_SITE_NOT_FOUND ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_NOT_ALL_GRAPHICS_DOWNLOADED )+" "+currentFilename ) ;
            removedAll = false ;
         }

         LogSystem.endLogSection() ;
      } else
         removedAll = true ; // Well, there were nothing to remove!


      return( removedAll ) ;
   }


   private static void setStatus ( AppDefaultWin parentWin, String status ) {
      if ( parentWin != null )
         parentWin.setStatus ( status ) ;
      else
         MainWindow.getInstance().setStatus ( status ) ;
   }
   private static void setStatus ( AppDefaultWin parentWin, int status ) {
      if ( parentWin != null )
         parentWin.setStatus ( Str.getStr ( status ) ) ;
      else
         MainWindow.getInstance().setStatus ( Str.getStr ( status ) ) ;
   }
} /* AddtlInfoGraphicFTP */
