package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   AddtlInfoImporter                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AddtlInfoImporter                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This class has methods that will parse an HTML file into the AddtlInfo object
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AddtlInfoImporter                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class AddtlInfoImporter extends Object implements AppConst {




   /*------------------------------------------------------------*/
   /* importAddtlInfo
    */
   /**
    *    This method will import a HTML file into
    *  the AddtlInfo object passed in.
    *  
    *
    *  @param htmlFile The HTML File to parse and prepare for the AddtlInfo
    *  @param addtlInfo The object to add the html file to
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean importAddtlInfo ( String htmlFile, AddtlInfo addtlInfo ) {

      try {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, we must process the graphics.                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parseGraphics ( htmlFile, addtlInfo ) ;

         addtlInfo.setAddtlInfo ( htmlFile ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ADD_INFO_NOT_IMPORTED ) ;
         return( false ) ;
      }

      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* createAddtlInfo
    */
   /**
    *    This method will create an AddtlInfo object
    *  from the htmlFile passed in. This htmlFile should
    *  be considered the 'Default' additional information
    *
    *  @param htmlFile The HTML file to import
    *  @param 
    *  @param None
    *
    *  @return AddtlInfo
    *//*---------------------------------------------------------*/
   public static AddtlInfo createAddtlInfo ( String htmlFile ) {
      AddtlInfo addtlInfo = new AddtlInfo (  ) ;

      try {
         Vector        countryVec   = UserSystem.getUserCountries ( UserSystem.getUserId() ) ;                                     


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the additional information default Properties       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         addtlInfo.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                          TypeList.TYPE_LANGUAGE ) ) ;                  
         addtlInfo.setDocClass ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL,
                                                                                          TypeList.TYPE_DOCCLASS ) ) ;
         addtlInfo.setCountryList ( countryVec ) ;                        


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Import the additional information                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !importAddtlInfo ( htmlFile, addtlInfo ) )
            addtlInfo = null ;

      } catch ( Exception e ) {
         addtlInfo = null ;
         LogSystem.log ( 1, e ) ;
      }

      return( addtlInfo ) ;
   }





   /*------------------------------------------------------------*/
   /* prepareHTML
    */
   /**
    *    This method will go through an entire
    *  HTML file and prepare it for the AddtlInfo object. It
    *  will also add any links and graphics
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if processed and added, false if otherwise
    *//*---------------------------------------------------------*/
   private static boolean prepareHTML ( String html, AddtlInfo addtlInfo ) {
      boolean rc = true ;

      try {
         String upperCaseBody = html.toUpperCase() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, take the text inside of the body tags            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         html = html.substring ( 0, upperCaseBody.indexOf ( "</BODY" ) ) ; // Take everything after closing body
         upperCaseBody = html.toUpperCase() ;

         html = html.substring ( upperCaseBody.indexOf ( "<BODY" ), html.length() ) ; 

         html = html.substring ( html.indexOf ( ">" )+1, html.length() ) ;                 // Take everythin after the beginning body tag


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now parse the graphics                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parseGraphics ( html, addtlInfo ) ;

         addtlInfo.setAddtlInfo ( html ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1,e, false ) ;
         GUISystem.printBox ( STR_ERROR, STR_COULD_NOT_PARSE_IMPORT_HTML ) ;
         rc = false ;
      }

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* parseGraphics
    */
   /**
    *    This method will parse the graphics out
    *  of the HTML file passed and will add an
    *  entry into the AddtlInfo object for each graphic
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void parseGraphics ( String htmlFile, AddtlInfo addInfo ) throws Exception {
      Vector currentGraphicVec = addInfo.getEmbeddedGraphics() ;
      Vector newGraphicVec     = getGraphicsFromHTML ( htmlFile ) ;                                             

      /*------------------------------------------------------------*/
      /*                                                            */
      /*   First, find out what graphics have been added. We will   */
      /* Do this by seeing what graphics are in the newGraphicVec   */
      /* but not in the currentGraphicVec. If a graphic is deemed   */
      /* 'new', then it will be added to the additional info object */
      /*                                                            */
      /*------------------------------------------------------------*/
      if ( newGraphicVec != null ) {

         if ( currentGraphicVec == null )
            currentGraphicVec = new Vector ( 1, 1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the newly added graphics                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         getNewGraphics ( newGraphicVec, currentGraphicVec ) ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find out what graphic has been deleted. We         */
         /* will accomplish this by looking at what is currently on   */
         /* the AddtlInfo passed that is not in the newGraphicVec     */
         /* we just parsed out. Any graphic in the current object     */
         /* that is not in the newGraphicVec is considered deleted    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setGraphicsRecStatus ( newGraphicVec, currentGraphicVec ) ;



      }

      addInfo.setEmbeddedGraphics ( currentGraphicVec ) ;


   }





   /*------------------------------------------------------------*/
   /* addtlInfoGraphicFromName
    */
   /**
    *    This method will return an AddtlInfoGraphic
    *  object from the filename given if the graphic
    *  is in the vector passed in. 
    *
    *  @param filename The filename of the graphic. There should be no path information in this filename
    *  @param graphicVec The Vector of AddtlInfoGraphics to look through
    *  @param None
    *
    *  @return AddtlInfoGraphic
    *//*---------------------------------------------------------*/
   private static AddtlInfoGraphic addtlInfoGraphicFromName ( String filename, Vector graphicVec )  {
      AddtlInfoGraphic retGraphic = null ;

      if ( ( graphicVec != null ) && ( graphicVec.size() > 0 ) ) {
         int              i             = 0 ;
         int              size          = graphicVec.size() ;
         AddtlInfoGraphic compareObject = null ;

         while ( ( i < size ) && ( graphicVec == null ) ) {
            compareObject = (AddtlInfoGraphic) graphicVec.elementAt(i) ;
            i++ ;
         }

      }

      return( retGraphic ) ;
   }





   /*------------------------------------------------------------*/
   /* getGraphicsFromHTML
    */
   /**
    *    This method will return all of the graphics that
    *  are present in the HTML. This method only
    *  uses the <IMG tag to parse with
    *
    *  @param html The HTML that contains graphics
    *  @param None
    *  @param None
    *
    *  @return Vector of Strings. Each String is the filename of the graphic
    *//*---------------------------------------------------------*/
   private static Vector getGraphicsFromHTML ( String html ) {
      Vector returnVec = new Vector ( 1, 1 ) ;
      String upperHTML = html.toUpperCase() ;                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only start processing if a simple check reveals that    */
      /* we indeed have at least 1 graphic                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( upperHTML.indexOf ( "<IMG" ) >= 0 ) {
         String  currentToken    = null ;
         int     length          = upperHTML.length() ;
         int     i               = 0 ;
         boolean continueParse   = true ;

         while ( ( i < length ) && ( continueParse ) ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Start by finding the beginning of the <IMG tag          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            i=upperHTML.indexOf ( "<IMG", i ) ;

            if ( ( i >= 0 ) && ( i < length ) ) {
               try {
                  currentToken = html.substring ( i, html.length() ) ;                                             


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Now, bump the index down past the 'SRC' part of the tag */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( currentToken.toUpperCase().indexOf ( "SRC" ) >= 0 )
                     currentToken = currentToken.substring ( currentToken.toUpperCase().indexOf ( "SRC" )+3, currentToken.length() ) ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Now, strip everything else after the source             */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( currentToken.indexOf ( " " ) >= 0 )
                     currentToken= currentToken.substring ( 0, currentToken.indexOf(" ") ).trim() ;                 
                  if ( currentToken.indexOf ( ">" ) >= 0 )
                     currentToken= currentToken.substring ( 0, currentToken.indexOf(">") ).trim() ;                 


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Take out the Double-Quotes                              */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  currentToken = Text.replaceAllStrInStr ( currentToken, "\"", "" ) ;
                  currentToken = Text.replaceAllStrInStr ( currentToken, "=", "" ) ;

                  /*---------------------------------------------------------------------------*/
                  /*                                                                           */
                  /*   If this image is being loaded from another web page, do not count it    */
                  /*                                                                           */
                  /*---------------------------------------------------------------------------*/

                  if ( ( currentToken.indexOf ( "http" ) == -1 ) &&
                       ( currentToken.indexOf ( "HTTP" ) == -1 ) ) {

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Take out the actual gif name                            */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     currentToken = Text.replaceAllStrInStr ( currentToken, "\\", "/" ) ;

                     if ( currentToken.indexOf ( "/" ) >= 0 )
                        currentToken = currentToken.substring ( currentToken.lastIndexOf ( "/" )+1 ) ;

                     returnVec.addElement ( currentToken ) ;
                  }


               } catch ( Exception e ) {
                  LogSystem.log ( 1, e, false ) ;
                  LogSystem.log ( 1, "Received exception parsing token "+currentToken ) ;
               }
            } else
               continueParse = false ;

            i++ ; // Bump down a bit for the next hit
         } /* while  */

      }

      return( returnVec ) ;

   }






   /*------------------------------------------------------------*/
   /* getNewGraphics
    */
   /**
    *    This method takes each item in the 'newGraphic'
    *  vector and looks for it in the 'currentGraphic' 
    *  Vector. If the graphic is not in the currentGraphic vector, 
    *  a new AddtlInfoGraphic object will be created and added. All items
    *  in the newGraphics vector MUST be only the String filename of the
    *  graphic or an ADdtlInfoGraphic object
    *
    *  @param newGraphic The vector containing possible new graphics
    *  @param currentGraphic The vector containing all current graphics
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void getNewGraphics ( Vector newGraphics, Vector currentGraphics ) {
      if ( ( newGraphics != null ) && ( newGraphics.size() > 0 ) ) {
         int i    = 0 ;
         int size = newGraphics.size() ;

         while ( i < size ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the item is a string                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( newGraphics.elementAt(i) instanceof String ) {
               if ( !findGraphicInVec ( (String)newGraphics.elementAt(i), currentGraphics ) ) {
                  LogSystem.log ( 1, "Found new graphic for additional info, Graphic Name:"+newGraphics.elementAt(i) ) ;
                  currentGraphics.addElement ( createAddtlInfoGraphic ( (String)newGraphics.elementAt(i) ) ) ;
               }
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the item is an AddtlInfoGraphic object               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( newGraphics.elementAt(i) instanceof AddtlInfoGraphic ) {
               if ( !findGraphicInVec ( ((AddtlInfoGraphic)newGraphics.elementAt(i)).getFilename(), 
                                        currentGraphics ) ) {
                  LogSystem.log ( 1, "Found new graphic for additional info, Graphic Name:"+((AddtlInfoGraphic)newGraphics.elementAt(i)).getFilename() ) ;
                  currentGraphics.addElement ( newGraphics.elementAt(i) ) ;
               }
            }

            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* setGraphicsRecStatus
    */
   /**
    *    This method takes each item in the 'currentGraphics'
    *  vector and looks for it in the 'newGraphics' 
    *  Vector. If the graphic is not in the newGraphic vector, 
    *  it is assumed deleted and the rec status will be changed on that
    *  AddtlInfoGraphic object. Otherwise the rec status is forced to REC_UPDATE
    *
    *  @param newGraphic The vector containing possible new graphics
    *  @param currentGraphic The vector containing all current graphics
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void setGraphicsRecStatus ( Vector newGraphics, Vector currentGraphics ) {
      if ( ( currentGraphics != null ) && ( currentGraphics.size() > 0 ) ) {
         int i    = 0 ;
         int size = currentGraphics.size() ;

         while ( i < size ) {

            if ( !findGraphicInVec ( ((AddtlInfoGraphic)currentGraphics.elementAt(i)).getFilename(), newGraphics ) )
               ((AddtlInfoGraphic)currentGraphics.elementAt(i)).updateRecStatus ( PersistentRec.REC_DELETE ) ;
            else {
               ((AddtlInfoGraphic)currentGraphics.elementAt(i)).updateRecStatus ( PersistentRec.REC_NOCHANGE ) ;
               ((AddtlInfoGraphic)currentGraphics.elementAt(i)).updateRecStatus ( PersistentRec.REC_UPDATE ) ;
            }


            i++ ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* findGraphicInVec
    */
   /**
    *    This method will look for the String filename in
    *  all of the AddtlInfoGraphic objects that are in the
    *  graphicVec
    *
    *  @param filename The string filename of the graphic to look for
    *  @param graphicVec A Vector containing AddtlInfoGraphic records to look through
    *  @param None
    *
    *  @return TRUE if found, FALSE if not found
    *//*---------------------------------------------------------*/
   private static boolean findGraphicInVec ( String filename, Vector graphicVec ) {
      boolean found = false ;

      if ( ( graphicVec != null ) && ( graphicVec.size() > 0 ) ) {
         int i    = 0  ;
         int size = graphicVec.size() ;

         while ( ( !found) && ( i < size ) ) {
            if ( graphicVec.elementAt(i) instanceof AddtlInfoGraphic ) {
               if ( ((AddtlInfoGraphic)graphicVec.elementAt(i)).getFilename().equals ( filename ) )
                  found = true ;
            } else if ( graphicVec.elementAt(i) instanceof String ) {
               found = ((String)graphicVec.elementAt(i)).equals ( filename ) ;
            }
            i++ ;
         }
      }


      return( found ) ;
   }





   /*------------------------------------------------------------*/
   /* createAddtlInfoGraphic
    */
   /**
    *    This method will create a new AddtlInfoGraphic
    *  object that represents the graphic filename given
    *  
    *
    *  @param filename The filename of the graphic
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static AddtlInfoGraphic createAddtlInfoGraphic ( String filename ) {
      AddtlInfoGraphic graphic = null ;

      LogSystem.log ( 1, "Creating AdditionalInfoGraphic for :"+filename ) ;
      if ( ( filename != null ) && ( filename.length() < 50 ) ) {
         graphic = new AddtlInfoGraphic ( Counter.getCounter ( LogSystem.getInstance(), "GRAPHICIND" ) ) ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save this graphic to the TIGRIS.GRAPHICS table          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( saveGraphicToDatabase ( graphic.getGraphicInd(), filename ) ) {


            graphic.setFilename ( filename ) ;
            graphic.setDocClass ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_UNCLASSIFIED,
                                                                                           TypeList.TYPE_DOCCLASS ) ) ;
            graphic.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                           TypeList.TYPE_LANGUAGE ) ) ;
         } else
            graphic = null ;
      } else
         GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_CANNOT_ADD_GRAPHIC_BAD_FILENAME )+filename ) ;


      return( graphic ) ;

   }






   /*------------------------------------------------------------*/
   /* saveGraphicToDatabase
    */
   /**
    *    This method will insert the graphic
    *  given into the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean saveGraphicToDatabase ( int graphicInd, String filename ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "saveGraphicToDatabase", 5 ) ;
      boolean   rc        = false ;

      LogSystem.log ( 1, "Saving graphic to database. Graphic Ind:"+graphicInd+"  Filename:"+filename ) ;
      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            stmt.executeUpdate ( "INSERT INTO TIGRIS.GRAPHICS "+
                                 "( GRAPHICIND, GRAPHICNAME,    FILENAME, "+
                                 "  VIEWIND,    GRAPHICTYPEIND, DBUSER,   "+
                                 "  CHANGEDTIME ) values "+
                                 "( "+graphicInd+", "+
                                 "  'Avalon Graphic'"+", "+
                                 "  '"+filename+"', "+
                                 "  0, "+
                                 "  1, "+
                                 "  '"+UserSystem.getUserId()+"', "+
                                 "  CURRENT TIMESTAMP )" ) ;

            rc = true ;
         }


      } catch ( SQLException e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_CANNOT_ADD_GRAPHIC )+filename ) ;
         rc = false ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }

} /* AddtlInfoImporter */
