package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   AnswerConclDisplayRec                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AnswerConclDisplayRec               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This record simply shows a AnswerConcl in a List, or preview. 
 *
 * @see com.ibm.nzna.projects.common.quest.oa.AnswerConcl
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AnswerConclDisplayRec                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class AnswerConclDisplayRec extends Object implements DisplayRec, MultiListRow, java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private AnswerConcl answerConcl          = null ;
   private int         displayAnswerInd     = 0 ;
   private int         displayQuestionInd   = 0 ;
   private String      displayQuestionTitle = null ;
   private String      displayAnswerTitle   = null ;
   private String      displayChangedTime   = null ;
   private String      displayDbUser        = null ;
   private String      prettyChangedTime    = null ;
   private String      prettyDBUser         = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *                                     
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AnswerConclDisplayRec ( int answerInd, int questionInd, String answerTitle, String questionTitle, String changedTime, String dbUser ) {
      this.displayAnswerInd         = answerInd ;
      this.displayQuestionInd       = questionInd ;
      this.displayQuestionTitle     = questionTitle ;
      this.displayAnswerTitle       = answerTitle ;
      this.displayChangedTime       = changedTime ;
      this.displayDbUser            = dbUser ;
   }

   public AnswerConclDisplayRec ( AnswerConcl answerConcl ) {
      Question question = null ;
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign local vars                                       */        
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.answerConcl              = answerConcl ;
      this.displayAnswerInd         = answerConcl.getToAnswerInd() ;
      this.displayQuestionInd       = answerConcl.getToQuestionInd() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we must read in the titles                         */                      
      /*                                                           */
      /*-----------------------------------------------------------*/
      question = AvalonCache.readQuestion ( displayQuestionInd, true ) ;
      this.displayChangedTime       = question.getChangedTime() ;
      this.displayDbUser            = question.getDbUser() ;
      displayQuestionTitle = getDefaultQuestionTitle(question) ;
      displayAnswerTitle   = getAnswerTitle(question) ;
      

   }



   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         

         case 0 :
            retObj = "" ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   question title                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 1 :
            retObj = displayQuestionTitle ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   answer title
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+displayAnswerTitle ;
            break;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( displayChangedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( displayDbUser ) ;

            retObj = prettyDBUser ;
            break ;

         default:
            retObj = "" ;
            break ;
      }


      return( retObj ) ;
   }


   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will return the string value of
    *  this object (in HTML!)
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( "<html><body><font size=2><b>"+displayQuestionTitle+"</b><br>&nbsp;&nbsp;&nbsp;"+displayAnswerTitle ) ;
   }





   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert an entire Vector of
    *  AnswerConcl objects to AnswerConclDisplayRecs
    *  and will return a vector of the converted records
    *
    *  @param vec The vector to convert
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector vec ) {
      int    size   = vec.size() ;
      Vector retVec = new Vector ( size ) ;
      int    i      = 0 ;

      while ( i < size ) {
         retVec.addElement ( new AnswerConclDisplayRec ( (AnswerConcl) vec.elementAt(i) ) ) ;
         i++ ;
      }

      return ( retVec ) ;
   }

   public Object getData () {
      return ( answerConcl ) ;
   }



   /*------------------------------------------------------------*/
   /* getDefaultQuestionTitle
    */
   /**
    *    This method will return the default
    *  question title of a question
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getDefaultQuestionTitle ( Question question ) {
      Vector titles = question.getTitles() ;

      return( titles.elementAt(titles.size()-1).toString() ) ;
   }




   /*------------------------------------------------------------*/
   /* getAnswerTitle
    */
   /**
    *    This method will pull out the answer title
    *  from the question
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getAnswerTitle ( Question question ) {
      return ( question.getAnswerByInd ( displayAnswerInd ).toString() ) ;
   }

   public String getAnswerTitle () {
      return ( displayAnswerTitle ) ;
   }

   public String getQuestionTitle () {
      return ( displayQuestionTitle ) ;
   }

   public void setSort ( int sort ) {
   }


   public Vector getChildren () {
      return ( null ) ;
   }

   public boolean containsChildren () {
      return ( false ) ;
   }

   public void addChild ( DisplayRec displayRec ) {
   }

   public void removeChild ( DisplayRec displayRec ) {
   }

} /* AnswerConclDisplayRec */
