package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   AnswerDisplayRec                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AnswerDisplayRec                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This record simply shows a Answer in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AnswerDisplayRec                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class AnswerDisplayRec extends Object implements DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Answer  answer            = null ;
   private int     answerInd         = 0 ;
   private String  title             = null ;
   private String  changedTime       = null ;
   private String  dbUser            = null ;
   private String  prettyChangedTime = null ;
   private String  prettyDBUser      = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AnswerDisplayRec () {
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will construct our object from
    *  the answer passed in
    *  
    *
    *  @param answer The answer
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AnswerDisplayRec ( Answer answer ) {
      Vector titleVec = answer.getTitles() ;

      this.answer      = answer ;
      this.answerInd   = answer.getInd() ;
      this.changedTime = answer.getChangedTime() ;
      this.dbUser      = answer.getDbUser() ;

      if ( titleVec != null )
         title = titleVec.elementAt(titleVec.size()-1).toString() ;

   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *                                     
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AnswerDisplayRec ( int answerInd, String title, String changedTime, String dbUser ) {
      this.answerInd        = answerInd ;
      this.title             = title ;
      this.changedTime       = changedTime ;
      this.dbUser            = dbUser ;
   }


   /*------------------------------------------------------------*/
   /* setAnswerInd
    */
   /**
    *    This method will set our answer indicator
    *  
    *  
    *
    *  @param answerInd The Sympom Indicator to represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAnswerInd ( int answerInd ) {
      this.answerInd = answerInd ;
   }




   /*------------------------------------------------------------*/
   /* getAnswerInd
    */
   /**
    *    This method will return the answer indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getAnswerInd () {
      return( answerInd ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         

         case 0 :
            retObj = "" ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Status                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 1 :
            retObj = "" ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Answer Indicator                                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+answerInd ;
            break;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            retObj = title ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 5 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;

            retObj = prettyDBUser ;
            break ;

         default:
            retObj = "" ;
            break ;
      }


      return( retObj ) ;
   }


   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( answer ) ;
   }





   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert a Vector of Answer
    *  objects to a Vector of AnswerDisplayRecs
    *  
    *
    *  @param answerVec The Vector full of Answer objects
    *  @param None
    *  @param None
    *
    *  @return Vector The Vector full of AnswerDisplayRecs
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector answerVec ) {
      Vector returnVec = new Vector ( 1 ) ;

      if ( ( answerVec != null ) && ( answerVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = answerVec.size() ;

         while ( i < size ) {
            returnVec.addElement ( convert ( (Answer) answerVec.elementAt ( i ) ) ) ;
            i++ ;
         }
      }

      return( returnVec ) ;
   }





   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert a single Answer
    *  object into a single AnswerDisplayRec
    *  
    *
    *  @param answer The answer object to convert
    *  @param None
    *  @param None
    *
    *  @return AnswerDisplayRec
    *//*---------------------------------------------------------*/
   public static AnswerDisplayRec convert ( Answer answer ) {
      return( new AnswerDisplayRec ( answer ) ) ;
   }

   public void setSort ( int sort ) {
   }

   public Vector getChildren () {
      return ( null ) ;
   }

   public boolean containsChildren () {
      return ( false ) ;
   }

   public void addChild ( DisplayRec displayRec ) {
   }

   public void removeChild ( DisplayRec displayRec ) {
   }

} /* AnswerDisplayRec */
