package com.ibm.nzna.projects.qit.avalon.base ;                          
/*-----------------------------------------------------------*/
/*   AvalonMemoryInterface                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AvalonMemoryInterface               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.oa.engine.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.avalon.editors.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This class interfaces with the OAEngine so that the Input Tool
 * can store objects in memory, and still have the OAEngine use those.
 * This basically makes it so that the OAEngine will ask us for knowledge
 * base objects first, then go to the database
 *
 * @version 1.0, 8/8/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AvalonMemoryInterface                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class AvalonMemoryInterface extends OAMemoryInterface implements java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Symptom            symptom     = null ;
   private Vector             symptomVec  = null ;
   private Vector             questionVec = null ;
   private Vector             actionVec   = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates and initializes the AvalonMemoryInterface
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AvalonMemoryInterface ( ) {
   }



   /*------------------------------------------------------------*/
   /* getDraftActionFromMemory
    */
   /**
    *    This method will attempt to return a draft
    *  
    *  
    *
    *  @param actionInd The Action Indicator of the action
    *  @param languageInd The Language to read the action in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Action getDraftActionFromMemory ( int actionInd, int langId ) {
      Action returnAction = null ;

      if ( ( actionVec != null ) && ( actionVec.size() > 0 ) ) {
         int         i           = 0 ;
         int         size        = actionVec.size() ;
         ActionDraft actionDraft = null ;

         while ( ( i < size ) && ( returnAction == null ) ) {
            if ( ((Action)actionVec.elementAt(i)).getInd() == actionInd )
               returnAction = (Action)actionVec.elementAt(i) ;
            i++ ;
         }
      }

      return( returnAction ) ;
   }



   /*------------------------------------------------------------*/
   /* getDraftQuestionFromMemory
    */
   /**
    *    This method will attempt to return a draft
    *  
    *  
    *
    *  @param questionInd The Question Indicator of the Question to read
    *  @param languageInd The Language Indicator
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Question getDraftQuestionFromMemory ( int questionInd, int langId ) {
      Question returnQuestion = null ;

      LogSystem.log ( 1, "AvalonMemoryInterface being asked for question "+questionInd+" in language "+langId );
      if ( ( questionVec != null ) && ( questionVec.size() > 0 ) ) {
         int         i           = 0 ;
         int         size        = questionVec.size() ;
         QuestionDraft QuestionDraft = null ;

         while ( ( i < size ) && ( returnQuestion == null ) ) {
            if ( ((Question)questionVec.elementAt(i)).getInd() == questionInd )
               returnQuestion = (Question)questionVec.elementAt(i) ;
            i++ ;
         }
      }

      if ( returnQuestion != null )
         LogSystem.log ( 1, "AvalonMemoryInterface returning Question Object" ) ;
      else
         LogSystem.log ( 1, "AvalonMemoryInterface returning NULL, Question not found" ) ;


      return( returnQuestion ) ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method must be called every time that
    *  a new Knowledge Base Engine is initialized, it
    *  will re-read all information for the session
    *
    *  @param editorPanel The SymptomEditorPanel to initialize with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize ( SymptomEditorPanel editorPanel ) {
      if ( editorPanel != null ) {
         FlowEditPanel flowEditPanel = editorPanel.getFlowEditPanel() ;

         symptom = editorPanel.getSymptom() ;

         if ( flowEditPanel != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, grab the Symptoms, then Actions and Questions    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            symptomVec  = flowEditPanel.getSymptoms() ;
            actionVec   = flowEditPanel.getActions() ;
            questionVec = flowEditPanel.getQuestions() ;

         }
      }
   }



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method must be called every time that
    *  a new Knowledge Base Engine is initialized, it
    *  will re-read all information for the session
    *
    *  @param Symptom The Symptom to initialize with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize ( Symptom symptom ) {
      if ( symptom != null ) {
         symptomVec = null ;
         actionVec  = null ;
         questionVec = null ;
      }
   }






   /*------------------------------------------------------------*/
   /* getDraftSymptomFromMemory
    */
   /**
    *    This method will return the Symptom asked
    *  for if we have already read it into memory
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Symptom getDraftSymptomFromMemory(int symInd, int langId) {
      Symptom returnSymptom = null ;                                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check if this is the symptom we are editing             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( symptom.getInd() == symInd )
         returnSymptom = symptom ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, go through the symptom vector                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( symptomVec != null ) && ( symptomVec.size() > 0 ) ) {
         int         i           = 0 ;
         int         size        = symptomVec.size() ;
         SymptomDraft SymptomDraft = null ;

         while ( ( i < size ) && ( returnSymptom == null ) ) {
            if ( ((Symptom)symptomVec.elementAt(i)).getInd() == symInd )
               returnSymptom = (Symptom)symptomVec.elementAt(i) ;
            i++ ;
         }
      }


      return( returnSymptom ) ;
   }  // end getDraftSymptomFromMemory




   public Hashtable getEdgesForMemoryNodeAndAnswer(Symptom currentSymptom, int currentNode, int currentResponse) {
      return( new Hashtable() ) ;
   }





   /*------------------------------------------------------------*/
   /* setSymptom
    */
   /**
    *    This method will set the symptom that we
    *  are using
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSymptom ( Symptom symptom ) {
      this.symptom = symptom ;
   }

} /* AvalonMemoryInterface */
