package com.ibm.nzna.projects.qit.avalon.base ;  
/*-----------------------------------------------------------*/
/*   AvalonUtil                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AvalonUtil                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.avalon.editors.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
/**
 * These are usefull, static methods for Avalon 
 *
 *
 * @version 1.0, 6/21/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AvalonUtil                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class AvalonUtil implements AppConst {


   /*------------------------------------------------------------*/
   /* getHeadEdgeGroups
    */
   /**
    *    This method will return the edge groups
    *  that belong to the Node passed in.
    *  
    *
    *  @param node The head node to look with
    *  @param symptom The symptom to look in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getHeadEdgeGroups ( Node node, Symptom symptom ) {
      Vector  edgeGroupVec    = new Vector ( 1 ) ;
      Vector  allEdgeGroupVec = symptom.getEdgeGroups() ;

      if ( ( allEdgeGroupVec != null ) && ( allEdgeGroupVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = allEdgeGroupVec.size() ;

         while ( i < size ) {
            if ( ((EdgeGroup)allEdgeGroupVec.elementAt(i)).getHeadNode() == node )
               edgeGroupVec.addElement ( allEdgeGroupVec.elementAt(i) ) ;
            i++ ;
         }
      }

      return( edgeGroupVec ) ;
   }


   /*------------------------------------------------------------*/
   /* getTailEdgeGroups
    */
   /**
    *    This method will return the edge groups
    *  that point to the node passed in
    *  
    *
    *  @param node The tail node to look with
    *  @param symptom The Symptom to look in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getTailEdgeGroups ( Node node, Symptom symptom ) {
      Vector  edgeGroupVec    = new Vector ( 1 ) ;
      Vector  allEdgeGroupVec = symptom.getEdgeGroups() ;

      if ( ( allEdgeGroupVec != null ) && ( allEdgeGroupVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = allEdgeGroupVec.size() ;

         while ( i < size ) {
            if ( ((EdgeGroup)allEdgeGroupVec.elementAt(i)).getTailNode() == node )
               edgeGroupVec.addElement ( allEdgeGroupVec.elementAt(i) ) ;
            i++ ;
         }
      }

      return( edgeGroupVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getEdgesFromNode
    */
   /**
    *    This method will return all edges that
    *  come from the head node passed
    *  
    *
    *  @param groups The vector of all Edge Groups.
    *  @param headNode The node to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getEdgesFromNode ( Vector groups, Node headNode ) {
      Vector  retVec     = new Vector ( 1, 1 ) ;
      boolean cont       = true ;
      Vector  edgeGroups = getEdgeGroups ( groups, headNode ) ;
      int     egIndex    = 0 ;                                                                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Loop, adding each single Edge from every EdgeGroup      */
      /* found from the 'getEdgeGroups' call.                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( edgeGroups != null ) && ( edgeGroups.size() > 0 ) ) {
         int egSize = edgeGroups.size() ;

         while ( egIndex < egSize ) {
            EdgeGroup edgeGroup = (EdgeGroup)edgeGroups.elementAt(egIndex) ;

            if ( ( edgeGroup != null ) && ( edgeGroup.valid() ) ) {
               Vector edges = edgeGroup.getEdges() ;

               if ( edges != null ) {
                  int i    = 0 ;
                  int size = edges.size() ;

                  while ( i < size )
                     retVec.addElement ( edges.elementAt(i++) ) ;
               }
            } else
               cont = false ;
            egIndex++ ;
         }
      }
      return( retVec ) ;
   } /* getEdgesFromNode */



   /*------------------------------------------------------------*/
   /* getEdgeGroup
    */
   /**
    *    This method will return the EdgeGroup
    *  that contains the head node passed in.
    *
    *  @param groups The vector of all Edge Groups.
    *  @param headNode The head node to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static EdgeGroup getEdgeGroup ( Vector groups, Node headNode ) {
      EdgeGroup edgeGroup = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, go through the list of EdgeGroups to find it     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( groups != null ) {
         int i    = 0 ;
         int size = groups.size() ;

         while ( ( edgeGroup == null ) && ( i < size ) ) {
            if ( ((EdgeGroup)groups.elementAt(i)).getHeadNode() == headNode )
               edgeGroup = (EdgeGroup)groups.elementAt(i) ;
            i++ ;
         } /* while  */
      } /* if  */

      return( edgeGroup ) ;
   }

   /*------------------------------------------------------------*/
   /* getEdgeGroups
    */
   /**
    *    This method will return all EdgeGroups
    *  that contain the head node passed in.
    *
    *  @param groups The vector of all Edge Groups.
    *  @param headNode The head node to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getEdgeGroups ( Vector groups, Node headNode ) {
      Vector retVec = new Vector ( 1, 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, go through the list of EdgeGroups to find it     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( groups != null ) {
         int i    = 0 ;
         int size = groups.size() ;

         while ( i < size ) {
            if ( ((EdgeGroup)groups.elementAt(i)).getHeadNode().equals ( headNode ) ) {
               retVec.addElement ( groups.elementAt(i) ) ;
            }
            i++ ;
         } /* while  */
      } /* if  */

      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* getEdgeGroups
    */
   /**
    *    This method will return the Edge Group
    *  that contains the Edge passed in
    *
    *  @param groups The vector of all Edge Groups.
    *  @param headNode The head node to look for
    *  @param None
    *  @param None
    *
    *  @return edge The Edge
    *//*---------------------------------------------------------*/
   public static EdgeGroup getEdgeGroup ( Vector groups, Edge edge ) {
      EdgeGroup edgeGroup = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, go through the list of EdgeGroups to find it     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( groups != null ) {
         int    i     = 0 ;
         int    size  = groups.size() ;
         Vector edges = null ;

         while ( ( edgeGroup == null ) && ( i < size ) ) {

            edges = ((EdgeGroup)groups.elementAt(i)).getEdges() ;

            if ( edges.contains ( edge ) )
               edgeGroup = (EdgeGroup)groups.elementAt(i) ;
            i++ ;
         } /* while  */
      } /* if  */

      return( edgeGroup ) ;
   }


   /*------------------------------------------------------------*/
   /* getEdgeGroup
    */
   /**
    *    This method will return the EdgeGroup
    *  that contains the head node and tail node passed in.
    *
    *  @param groups The vector of all Edge Groups.
    *  @param headNode The head node to look for
    *  @param tailNode The tail node to look for
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static EdgeGroup getEdgeGroup ( Vector groups, Node headNode, Node tailNode ) {
      EdgeGroup edgeGroup = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, go through the list of EdgeGroups to find it     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( groups != null ) {
         int i    = 0 ;
         int size = groups.size() ;

         while ( ( edgeGroup == null ) && ( i < size ) ) {
            if ( ( ((EdgeGroup)groups.elementAt(i)).getHeadNode() == headNode ) &&
                 ( ((EdgeGroup)groups.elementAt(i)).getTailNode() == tailNode ) )
               edgeGroup = (EdgeGroup)groups.elementAt(i) ;
            i++ ;
         } /* while  */
      } /* if  */

      return( edgeGroup ) ;
   }






   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will return the String version
    *  of the default title of the Symptom passed in
    *  
    *
    *  @param symptom The Symptom
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getDefaultTitle ( Symptom symptom ) {
      String retStr = "" ;

      if ( ( symptom != null ) && ( symptom.getSymptomTitles() != null ) ) {
         Vector titles = symptom.getSymptomTitles() ;
         retStr = titles.elementAt(titles.size()-1).toString() ;
      }

      return( retStr ) ;
   }






   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will return the String version
    *  of the default title of the Action passed in
    *  
    *
    *  @param action The Action
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getDefaultTitle ( Action action ) {
      String retStr = "" ;

      if ( ( action != null ) && ( action.getTitles() != null ) ) {
         Vector titles = action.getTitles() ;
         retStr = titles.elementAt(titles.size()-1).toString() ;
      }

      return( retStr ) ;
   }


   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will return the String version
    *  of the default title of the Question passed in
    *  
    *
    *  @param question The Question
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getDefaultTitle ( Question question ) {
      String retStr = "" ;


      if ( ( question != null ) && ( question.getTitles() != null ) ) {
         Vector titles = question.getTitles() ;
         retStr = titles.elementAt(titles.size()-1).toString() ;
      }

      return( retStr ) ;
   }






   /*------------------------------------------------------------*/
   /* getObjectNodeForNode
    */
   /**
    *    This method will go through the vector of
    *  Object Nodes passed and will return the single
    *  ObjectNode representing the Node
    *
    *  @param objectNodes The vector of ObjectNodes
    *  @param node The node to look for
    *  @param None
    *
    *  @return ObjectNode
    *//*---------------------------------------------------------*/
   public static ObjectNode getObjectNodeForNode ( Vector objectNodes, Node node ) {
      ObjectNode returnNode = null ;

      if ( ( objectNodes != null ) && ( objectNodes.size() > 0 ) ) {
         int i    = 0 ;
         int size = objectNodes.size() ;

         while ( ( returnNode == null ) && ( i < size ) ) {
            if ( ((ObjectNode)objectNodes.elementAt(i)).getNode().equals ( node ) )
               returnNode = (ObjectNode)objectNodes.elementAt(i) ;
            i++ ;
         }
      }

      return( returnNode ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultTitleObject
    */
   /**
    *    This method will return the String version
    *  of the default title of the Symptom passed in
    *  
    *
    *  @param symptom The Symptom
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static SymptomTitle getDefaultTitleObject ( Symptom symptom ) {

      if ( ( symptom != null ) && ( symptom.getSymptomTitles() != null ) ) {
         Vector titles = symptom.getSymptomTitles() ;

         return( (SymptomTitle)titles.elementAt(titles.size()-1) ) ;
      }

      return( null ) ;
   }


   /*------------------------------------------------------------*/
   /* getDefaultTitleObject
    */
   /**
    *    This method will return the String version
    *  of the default title of the Symptom passed in
    *  
    *
    *  @param symptom The Symptom
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static LanguageText getDefaultTitleObject ( LinkGroup linkGroup ) {

      if ( ( linkGroup != null ) && ( linkGroup.getLinkGroupTitles() != null ) ) {
         Vector titles = linkGroup.getLinkGroupTitles() ;

         return( (LanguageText)titles.elementAt(titles.size()-1) ) ;
      }

      return( null ) ;
   }







   /*------------------------------------------------------------*/
   /* getDefaultTitleObject
    */
   /**
    *    This method will return the String version
    *  of the default title of the Action passed in
    *  
    *
    *  @param action The Action
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Title getDefaultTitleObject ( Action action ) {

      if ( ( action != null ) && ( action.getTitles() != null ) ) {
         Vector titles = action.getTitles() ;

         return( (Title)titles.elementAt(titles.size()-1) ) ;
      }

      return( null ) ;
   }


   /*------------------------------------------------------------*/
   /* getDefaultTitleObject
    */
   /**
    *    This method will return the String version
    *  of the default title of the Question passed in
    *  
    *
    *  @param question The Question
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Title getDefaultTitleObject ( Question question ) {

      if ( ( question != null ) && ( question.getTitles() != null ) ) {
         Vector titles = question.getTitles() ;

         return( (Title)titles.elementAt(titles.size()-1) ) ;
      }

      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* prepareAddInfoForView
    */
   /**
    *    This method will prepare an additional information
    *  text for display locally on the user's client. This process
    *  does the following:<br><br>
    *
    *  1. Wraps the entire addinfo in html tags<br>
    *  2. Fixes all of the ###IMAGELOC### tags to have the doc link icon
    *
    *  @param addInfo The additional information to alter
    *  @param None
    *  @param None
    *
    *  @return The altered additional information
    *//*---------------------------------------------------------*/
   public static String prepareAddInfoForView ( String addInfo ) {
      String retInfo  = "" ;
      String skinName = PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME ) ;
      String homeDir  = PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, make sure the doclink.gif file is in the temp dir*/                                               
      /*                                                           */
      /*-----------------------------------------------------------*/
      QuestUtil.copyFileToTempDir ( homeDir+File.separator+"skins"+File.separator+skinName+File.separator+"doclink.gif" ) ;
      System.out.println ( "Copied:"+homeDir+File.separator+"skins"+File.separator+skinName+File.separator+"doclink.gif" ) ;
      

      if ( addInfo != null ) {
         int    startIndex     = 0 ;
         String docLink        = "###doclink###" ;
         int    docLinkLength  = docLink.length() ;
         String tempDir        = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Wrap the body in HTML Tags                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retInfo = "<html><body><font size=2 face=\"Helvetica, Arial\">"+addInfo+"</body></html>" ;                      


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, replace the ###doclink### tags                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         startIndex = retInfo.indexOf ( docLink ) ;
         while ( startIndex >= 0 ) {
            retInfo = retInfo.substring ( 0, startIndex )+"<a href=\"./"+retInfo.substring ( startIndex+docLinkLength, retInfo.length() ) ;
            startIndex = retInfo.indexOf ( ">", startIndex ) ;

            if ( startIndex >= 0 ) {
               retInfo = retInfo.substring ( 0, startIndex )+"\"><img src=\"file:/"+tempDir+File.separator+"doclink.gif></a>"+
                         retInfo.substring ( startIndex+1, retInfo.length() ) ;
               startIndex = retInfo.indexOf ( docLink ) ;
            }
         }

      } /* if  */

      return( retInfo ) ;
   } /* prepareAddInfoForView  */









   /*------------------------------------------------------------*/
   /* getDefaultTitleText
    */
   /**
    *    This method will return the String version
    *  of the default title of the Symptom passed in
    *  
    *
    *  @param symptom The Symptom
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getDefaultTitleText ( Symptom symptom ) {
      String retStr = "" ;

      if ( ( symptom != null ) && ( symptom.getSymptomTitles() != null ) ) {
         Vector titles = symptom.getSymptomTitles() ;
         retStr = ((SymptomTitle)titles.elementAt(titles.size()-1)).toString(SymptomTitle.TEXT) ;
      }

      return( retStr ) ;
   }






   /*------------------------------------------------------------*/
   /* getDefaultTitleText
    */
   /**
    *    This method will return the String version
    *  of the default title of the Action passed in
    *  
    *
    *  @param action The Action
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getDefaultTitleText ( Action action ) {
      String retStr = "" ;

      if ( ( action != null ) && ( action.getTitles() != null ) ) {
         Vector titles = action.getTitles() ;
         retStr = ((Title)titles.elementAt(titles.size()-1)).toString(Title.TEXT) ;
      }

      return( retStr ) ;
   }


   /*------------------------------------------------------------*/
   /* getDefaultTitleText
    */
   /**
    *    This method will return the String version
    *  of the default title of the Question passed in
    *  
    *
    *  @param question The Question
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getDefaultTitleText ( Question question ) {
      String retStr = "" ;


      if ( ( question != null ) && ( question.getTitles() != null ) ) {
         Vector titles = question.getTitles() ;
         retStr = ((Title)titles.elementAt(titles.size()-1)).toString(Title.TEXT) ;
      }

      return( retStr ) ;
   }









} /* AvalonUtil */
