package com.ibm.nzna.projects.qit.avalon.base ;                                
/*-----------------------------------------------------------*/
/*   ConditionTermDisplayRec                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ConditionTermDisplayRec             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.*;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object wraps a ConditionTerm so that it can be displayed in a MultiList 
 *
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ConditionTermDisplayRec                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class ConditionTermDisplayRec extends Object implements AppConst, DisplayRec, MultiListRow {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ConditionTerm      conditionTerm     = null ;
   private String             answerTitle       = null ;
   private String             questionTitle     = null ;
   private String             operators         = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor for  this object. This
    *  constructor will read in the answer and question
    *  titles from the database based on what is in the ConditionTerm.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ConditionTermDisplayRec ( ConditionTerm conditionTerm ) {
      setData ( conditionTerm ) ;
   }

   public void setData ( ConditionTerm conditionTerm ) {
      Question question = null ;

      this.conditionTerm = conditionTerm ;

      question = AvalonCache.readQuestion ( conditionTerm.getQuestionInd(), true ) ;

      if ( conditionTerm.getOperators().equals ( "&" ) )
         operators = Str.getStr ( STR_AND ) ;
      else if ( conditionTerm.getOperators().equals ( "|" ) )
         operators = Str.getStr ( STR_OR ) ;
      else if ( conditionTerm.getOperators().equals ( " |" ) )
         operators = Str.getStr ( STR_OR_GROUP ) ;
      else if ( conditionTerm.getOperators().equals ( " &" ) )
         operators = Str.getStr ( STR_AND_GROUP ) ;
      else if ( conditionTerm.getOperators().equals ( "!&" ) )
         operators = Str.getStr ( STR_NOT_AND ) ;
      else if ( conditionTerm.getOperators().equals ( "!|" ) )
         operators = Str.getStr ( STR_NOT_OR ) ;
      else if ( conditionTerm.getOperators().equals ( "!" ) )
         operators = Str.getStr ( STR_NOT ) ;
      else if ( conditionTerm.getOperators().equals ( "! " ) )
         operators = Str.getStr ( STR_NOT ) ;
      else if ( conditionTerm.getOperators().equals ( "|&" ) )
         operators = Str.getStr ( STR_OR )+" "+Str.getStr ( STR_AND ) ;
      else if ( conditionTerm.getOperators().equals ( "&|" ) )
         operators = Str.getStr ( STR_AND )+" "+Str.getStr ( STR_OR ) ;

      questionTitle = getDefaultQuestionTitle(question) ;
      answerTitle   = getAnswerTitle(question) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the data that should
    *  be drawn in a column in a multi column list
    *  
    *
    *  @param column The column to get data for
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = operators ;
            break ;

         case 1 :
            retObj = questionTitle ;
            break ;

         case 2 :
            retObj = answerTitle ;
            break ;
      }

      return( retObj ) ;
   }




   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that we
    *  represent
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( conditionTerm ) ;
   }





   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert a vector of ConditionTerms
    * to ConditionTermDisplayRecs
    *  
    *  
    *
    *  @param Vector The vector of ConditionTerms
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector conditionTermVec ) {
      Vector retVec = null ;

      if ( conditionTermVec != null ) {
         new Vector ( conditionTermVec.size() ) ;
         int    i      = 0 ;
         int    size   = conditionTermVec.size() ;

         retVec = new Vector ( conditionTermVec.size() ) ;

         while ( i < size ) {
            retVec.addElement ( new ConditionTermDisplayRec ( (ConditionTerm)conditionTermVec.elementAt(i) ) ) ;
            i++ ;
         }
      }

      return( retVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultQuestionTitle
    */
   /**
    *    This method will return the default
    *  question title of a question
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getDefaultQuestionTitle ( Question question ) {
      Vector titles = question.getTitles() ;

      return( titles.elementAt(titles.size()-1).toString() ) ;
   }




   /*------------------------------------------------------------*/
   /* getAnswerTitle
    */
   /**
    *    This method will pull out the answer title
    *  from the question
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getAnswerTitle ( Question question ) {
      if ( question != null ) {
         Answer answer = question.getAnswerByInd ( conditionTerm.getAnswerInd() ) ;

         if ( answer != null )
            return( question.getAnswerByInd ( conditionTerm.getAnswerInd() ).toString() ) ;
         else
            return( "No Answer Title for answerInd:"+conditionTerm.getAnswerInd() ) ;
      }
      return ( "Question is undefined. QuestionInd:"+conditionTerm.getQuestionInd() ) ;
   }

   public void setSort ( int sort ) {
   }


   public Vector getChildren () {
      return ( null ) ;
   }

   public boolean containsChildren () {
      return ( false ) ;
   }

   public void addChild ( DisplayRec displayRec ) {
   }

   public void removeChild ( DisplayRec displayRec ) {
   }

} /* ConditionTermDisplayRec */
