package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   DocLinkDisplayRec                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocLinkDisplayRec                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import java.util.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This record represents a DocLink object. It is basically only 
 * meant to be placed in a MultiList control
 *
 * @version 1.0, 5/9/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocLinkDisplayRec                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocLinkDisplayRec extends Object implements IQuickSort, OASort, AppConst, MultiListRow, DisplayRec {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean   linkGroup         = false ;
   private int       linkInd           = 0 ;
   private int       linkGroupInd      = 0 ;
   private DocLink   docLink           = null ;
   private String    strInd            = "0" ;
   private String    title             = null ;
   private int       sortOn            = 0 ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a DocLinkDisplayRec
    *  from the docLink passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocLinkDisplayRec ( DocLink docLink ) {
      linkGroup    = docLink.hasLinkGroup() ;
      linkInd      = docLink.getLinkInd() ;
      linkGroupInd = docLink.getLinkGroupInd() ;
      strInd       = linkGroup ? ""+linkGroupInd : ""+linkInd ;
      readTitle() ;
      this.docLink = docLink ;
   }





   /*------------------------------------------------------------*/
   /* DocLinkDisplayRec
    */
   /**
    *    This method will create a Link represented
    *  record from the data passed
    *  
    *
    *  @param linkInd The Link indicator to use
    *  @param linkType The link type 0 = QuestLink, 1 = External Link
    *  @param title The title to use
    *
    *  @param None
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocLinkDisplayRec ( int linkInd, int linkType, String title ) {
      this.linkInd = linkInd ;
      strInd       = ""+linkInd ;
      this.title   = title ;
      linkGroup    = false ;
      if ( linkType == 0 )
         docLink = new DocLink ( new QuestLink ( linkInd ) ) ;
      else
         docLink = new DocLink ( new ExternalLink ( linkInd ) ) ;

      docLink.setDocClass ( (TypeDocClassRec)TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL, TypeList.TYPE_DOCCLASS ) ) ;
      docLink.updateRecStatus ( docLink.REC_INSERT ) ;
   }





   /*------------------------------------------------------------*/
   /* DocLinkDisplayRec
    */
   /**
    *    This method will create either a Link or
    *  LinkGroup represented record
    *  
    *import com.ibm.nzna.projects.common.quest.oa.* ;

    *  @param linkInd This is the linkInd if the linkGroup passed
    *  @param title The title of this link
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocLinkDisplayRec ( int linkInd, String title ) {
      this.linkInd = linkInd ;
      docLink = new DocLink ( new LinkGroup ( linkInd ) ) ;
      this.title = title ;

   }



   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the object that should
    *  paint in a certain column
    *  
    *
    *  @param column The column to return
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = linkGroup ? ImageSystem.getImageIcon ( this, ImageSystem.SMALL_LINKGROUP ) : 
                     ImageSystem.getImageIcon ( this, ImageSystem.DOC_LINK ) ;
            break ;

         case 1 :
            retObj = strInd ;
            break ;

         case 2 :
            retObj = title ;
            break ;
      }

      return( retObj ) ;
   }



   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data
    *  that we are representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( docLink ) ;
   }





   /*------------------------------------------------------------*/
   /* readTitle
    */
   /**
    *    This method will read the title that
    *  this object represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readTitle () {                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this is a LinkGroup, read it's title                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( linkGroup ) {
         LinkGroup linkGroupRec = AvalonCache.readLinkGroup ( linkGroupInd ) ;

         if ( linkGroupRec != null )
            title = linkGroupRec.getLinkGroupTitles().elementAt(0).toString() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, read the Quest/External Link's title         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         Object linkObject = AvalonCache.readLink ( linkInd ) ;

         if ( linkObject instanceof Link ) {
            Vector titles = ((Link)linkObject).getLinkTitles() ;

            if ( ( titles != null ) && ( titles.size() > 0 ) )
               title = titles.elementAt(titles.size()-1).toString() ;
         }
      }

   }





   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert a Vector of DocLink
    *  records to a Vector of DocLinkDisplayRecs
    *  and will return the converted vector
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector docLinkVec ) {
      Vector retVec = null ;

      if ( docLinkVec != null ) {
         int size = docLinkVec.size() ;
         int i    = 0 ;

         retVec = new Vector ( size ) ;

         while ( i < size )
            retVec.addElement ( new DocLinkDisplayRec ( (DocLink)docLinkVec.elementAt(i++ ) ) ) ;
      }

      return( retVec ) ;
   }







   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will do the comparison between
    *  another LinkDisplayRec and this one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( IQuickSort sortObject ) {
      int rc = 0 ;

      if ( sortObject instanceof DocLinkDisplayRec ) {
         DocLinkDisplayRec displayRec = (DocLinkDisplayRec) sortObject ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Do the comparison based on what we have to sort         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( sortOn ) {
            case SORT_CHANGEDTIME_ASC :
               rc = 0 ;
               break ;

            case SORT_CHANGEDTIME_DESC :
               rc = 0 ;
               break ;

            case SORT_TITLE_ASC :
               rc = title.compareTo ( displayRec.title ) ;
               break ;

            case SORT_TITLE_DESC :
               rc = displayRec.title.compareTo ( title ) ;
               break ;

            case SORT_DBUSER_ASC :
               rc = 0 ;
               break ;

            case SORT_DBUSER_DESC :
               rc = 0 ;
               break ;

            case SORT_DOCCLASS :
               rc = 0 ;
               break ;

            case SORT_INDICATOR_ASC :
               if ( linkInd < displayRec.linkInd )
                  rc = -1 ;
               else if ( linkInd > displayRec.linkInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_DESC :
               if ( linkInd < displayRec.linkInd )
                  rc = 1 ;
               else if ( linkInd > displayRec.linkInd )
                  rc = -1 ;
               else
                  rc = 0 ;
               break ;
         }

      }

      if ( rc > 0 )
         rc = 1 ;
      else if ( rc < 0 ) 
         rc = -1 ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setSort
    */
   /**
    *    This method will set what we need to sort
    *  on
    *  
    *
    *  @param sortOn The sort as defined in OASort
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSort ( int sortOn ) {
      this.sortOn = sortOn ;
   }

   public Vector getChildren () {
      return ( null ) ;
   }

   public boolean containsChildren () {
      return ( false ) ;
   }

   public void addChild ( DisplayRec displayRec ) {
   }

   public void removeChild ( DisplayRec displayRec ) {
   }

} /* DocLinkDisplayRec */
