package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   EdgeAlistDisplayRec                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EdgeAlistDisplayRec                 */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This record simply shows a EdgeAlist in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EdgeAlistDisplayRec                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class EdgeAlistDisplayRec extends Object implements DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       edgeAlistInd        = 0 ;
   private int       docClassInd       = 0 ;
   private String    title             = null ;
   private String    changedTime       = null ;
   private String    dbUser            = null ;
   private String    prettyChangedTime = null ;
   private String    prettyDBUser      = null ;
   private EdgeAlist edgeAlist         = null ;
   private Action    action            = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EdgeAlistDisplayRec () {
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EdgeAlistDisplayRec ( int edgeAlistInd, int docClassInd, String title, String changedTime, String dbUser ) {
      this.edgeAlistInd        = edgeAlistInd ;
      this.docClassInd       = docClassInd ;
      this.title             = title ;
      this.changedTime       = changedTime ;
      this.dbUser            = dbUser ;
   }


   public EdgeAlistDisplayRec ( EdgeAlist edgeAlist ) {
      this.edgeAlist      = edgeAlist ;
      if ( edgeAlist != null ) {
         this.edgeAlistInd = edgeAlist.getInd() ;
         this.action       = AvalonCache.readAction ( edgeAlist.getActiveActionInd() ) ;
         if ( action != null )
            this.title        = action.getTitles().elementAt(action.getTitles().size()-1).toString()  ;
         else
            title = "Action Not Found!" ;
      }
   }

   /*------------------------------------------------------------*/
   /* setEdgeAlistInd
    */
   /**
    *    This method will set our edgeAlist indicator
    *  
    *  
    *
    *  @param edgeAlistInd The Sympom Indicator to represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEdgeAlistInd ( int edgeAlistInd ) {
      this.edgeAlistInd = edgeAlistInd ;
   }




   /*------------------------------------------------------------*/
   /* getEdgeAlistInd
    */
   /**
    *    This method will return the edgeAlist indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getEdgeAlistInd () {
      return( edgeAlistInd ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Classification                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            if ( docClassInd == TypeList.DOCCLASS_IBM_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ;
            else if ( docClassInd == TypeList.DOCCLASS_DEALER_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_DEALER_CONFIDENTIAL ) ;
            else
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_UNCLASSIFIED ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Status                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 1 :
            retObj = "" ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   EdgeAlist Indicator                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+edgeAlistInd ;
            break;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            retObj = title ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 5 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;

            retObj = prettyDBUser ;
            break ;

         default:
            retObj = "" ;
            break ;
      }


      return( retObj ) ;
   }


   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( edgeAlist ) ;
   }


   public String toString () {
      return( title ) ;
   }



   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert an entire Vector of
    *  EdgeAlist objects to EdgeAlistDisplayRecs
    *  and will return a vector of the converted records
    *
    *  @param vec The vector to convert
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector vec ) {
      int    size   = vec.size() ;
      Vector retVec = new Vector ( size ) ;
      int    i      = 0 ;

      while ( i < size ) {
         if ( vec.elementAt(i) != null )
            retVec.addElement ( new EdgeAlistDisplayRec ( (EdgeAlist) vec.elementAt(i) ) ) ;
         i++ ;
      }

      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* setEdgeAlist
    */
   /**
    *    This method will set the edgeAlist that this
    *  record represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEdgeAlist ( EdgeAlist edgeAlist ) {
      this.edgeAlist = edgeAlist ;
   }

   public void setSort ( int sort ) {
   }

   public Vector getChildren () {
      return ( null ) ;
   }

   public boolean containsChildren () {
      return ( false ) ;
   }

   public void addChild ( DisplayRec displayRec ) {
   }

   public void removeChild ( DisplayRec displayRec ) {
   }

} /* EdgeAlistDisplayRec */
