package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   EdgeUnanswerDisplayRec                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EdgeUnanswerDisplayRec              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.TypeList ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This record simply shows a Question in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EdgeUnanswerDisplayRec                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class EdgeUnanswerDisplayRec extends Object implements DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1001001 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int          questionInd       = 0 ;
   private int          docClassInd       = 0 ;
   private String       title             = null ;
   private String       changedTime       = null ;
   private String       dbUser            = null ;
   private String       prettyChangedTime = null ;
   private String       prettyDBUser      = null ;
   private EdgeUnanswer unanswer          = null ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EdgeUnanswerDisplayRec () {
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EdgeUnanswerDisplayRec ( int questionInd, int docClassInd, String title, String changedTime, String dbUser ) {
      this.questionInd       = questionInd ;
      this.docClassInd       = docClassInd ;
      this.title             = title ;
      this.changedTime       = changedTime ;
      this.dbUser            = dbUser ;
   }

   public EdgeUnanswerDisplayRec ( Question question ) {
      setQuestion ( question ) ;
   }


   public EdgeUnanswerDisplayRec ( EdgeUnanswer unanswer ) {
      if ( unanswer.getDraftQuestionInd() != 0 )
         setQuestion ( AvalonCache.readQuestionDraft ( unanswer.getDraftQuestionInd() ) ) ;
      else
         setQuestion ( AvalonCache.readQuestion ( unanswer.getQuestionInd() ) ) ;
   }

   /*------------------------------------------------------------*/
   /* getQuestionInd
    */
   /**
    *    This method will return the question indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getQuestionInd () {
      if ( unanswer != null ) {
         if ( unanswer.getDraftQuestionInd() != 0 )
            return( unanswer.getDraftQuestionInd() ) ;
         else
            return( unanswer.getQuestionInd() ) ;
      }

      return 0 ;

   }




   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Classification                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            if ( docClassInd == TypeList.DOCCLASS_IBM_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ;
            else if ( docClassInd == TypeList.DOCCLASS_DEALER_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_DEALER_CONFIDENTIAL ) ;
            else
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_UNCLASSIFIED ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Nothing                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 1 :
            retObj = "" ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Question Indicator                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+questionInd ;
            break;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            retObj = title ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 5 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;

            retObj = prettyDBUser ;
            break ;

         default:
            retObj = "" ;
            break ;
      }


      return( retObj ) ;
   }


   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( unanswer ) ;
   }


   public String toString () {
      return( title ) ;
   }


   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert an entire Vector of
    *  Question objects to EdgeUnanswerDisplayRecs
    *  and will return a vector of the converted records
    *
    *  @param vec The vector to convert
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector vec ) {
      int    size   = vec.size() ;
      Vector retVec = new Vector ( size ) ;
      int    i      = 0 ;

      while ( i < size ) {
         retVec.addElement ( new EdgeUnanswerDisplayRec ( (EdgeUnanswer) vec.elementAt(i) ) ) ;
         i++ ;
      }

      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* setQuestion
    */
   /**
    *    This method will set what question that this
    *  display represents
    *  
    *
    *  @param question The question tor represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setQuestion ( Question question ) {
      this.unanswer          = new EdgeUnanswer ( question ) ;
      this.questionInd       = question.getInd() ;
      this.docClassInd       = question.getDocClass().getInd () ;
      this.title             = question.getTitles().elementAt(question.getTitles().size()-1).toString() ;
      this.changedTime       = question.getChangedTime () ;
      this.dbUser            = question.getDbUser() ;

   }


   public void setSort ( int sort ) {
   }

   public Vector getChildren () {
      return ( null ) ;
   }

   public boolean containsChildren () {
      return ( false ) ;
   }

   public void addChild ( DisplayRec displayRec ) {
   }

   public void removeChild ( DisplayRec displayRec ) {
   }


} /* EdgeUnanswerDisplayRec */

