package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   LinkDisplayRec                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkDisplayRec                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;

/**
 * This record simply shows a Link in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkDisplayRec                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkDisplayRec extends OAObjectDisplayRec implements IQuickSort, OASort, DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int     linkInd           = 0 ;
   private int     linkTypeInd       = 0 ;
   private int     docInd            = 0 ;
   private int     docClassInd       = 0 ;
   private boolean valid             = false ;
   private Object  linkObject        = null ;
   private String  title             = null ;
   private String  docFilename       = null ;
   private String  URL               = null ;
   private String  dbUser            = null ;
   private String  changedTime       = null ;
   private String  prettyCreated     = null ;
   private String  prettyChangedTime = null ;
   private String  prettyDBUser      = null ;
   private int     sortOn            = 0 ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkDisplayRec ( int linkInd, 
                           String  title, 
                           boolean valid, 
                           int     linkTypeInd, 
                           int     docInd, 
                           String  docFilename, 
                           String  URL, 
                           String  dbUser, 
                           String  changedTime ) {
      this.valid             = valid ;
      this.title             = title ;
      this.linkInd           = linkInd ;
      this.linkTypeInd       = linkTypeInd ;
      this.docInd            = docInd ;
      this.docFilename       = docFilename ;
      this.URL               = URL ;
      this.dbUser            = dbUser ;
      this.changedTime       = changedTime ;
      this.docClassInd       = docClassInd ;

      if ( linkTypeInd == 0 )
         linkObject = new QuestLink ( linkInd ) ;
      else
         linkObject = new ExternalLink ( linkInd ) ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkDisplayRec ( ) {
   }

   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display. 
    *
    *  DO NOT CHANGE THE ORDER HERE WITHOUT CHANGING THE ORDER IN LinkPreviewRec!
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Classification                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            retObj = "" ;
            break ;

         case 1 :
            retObj = ( valid ? 
                       ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_UNCLASSIFIED ) :
                       ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Link Indicator                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+linkInd ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            if ( getTitleArray() == null )
               createTitleArray ( title ) ;
            retObj = getTitleArray() ;
            break;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 5 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;

            retObj = prettyDBUser ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Target                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 6 :
            if ( URL != null )
               retObj = URL ;
            else
               retObj = docFilename ;
            break ;

         default:
            retObj="" ;
            break ;
      }


      return( retObj ) ;
   }





   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will simply return the title
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( title ) ;
   }

   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( linkObject ) ;
   }




   /*------------------------------------------------------------*/
   /* getLinkInd
    */
   /**
    *    This method will return our link indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getLinkInd () {
      return( linkInd ) ;
   }




   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert a vector of DocLink and
    *  ExternalLink records into LinkDisplayRecs. 
    *  
    *
    *  @param vec The vector to convert
    *  @param None
    *  @param None
    *
    *  @return Vector A Vector of the converted Objects
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector vec ) {
      int    size       = vec.size() ;
      Vector retVec     = new Vector ( size ) ;
      int    i          = 0 ;
      Object linkObject = null ;

      while ( i < size ) {
         linkObject = vec.elementAt(i) ;   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   ExternalLink                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( linkObject instanceof ExternalLink ) {
            ExternalLink exLink = (ExternalLink) linkObject ;

            retVec.addElement ( new LinkDisplayRec ( exLink.getInd(),
                                                     exLink.getLinkTitles().elementAt(0).toString(),
                                                     exLink.isUrlValid(),
                                                     0,
                                                     9,
                                                     null,
                                                     exLink.getUrl(),
                                                     exLink.getDbUser(),
                                                     exLink.getChangedTime() ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   QuestLink                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( linkObject instanceof QuestLink ) {
            QuestLink questLink = (QuestLink) linkObject ;


            retVec.addElement ( new LinkDisplayRec ( questLink.getInd(),
                                                     questLink.getLinkTitles().elementAt(0).toString(),
                                                     true,
                                                     questLink.getDocInd(),
                                                     9,
                                                     questLink.getLNDocId(),
                                                     null,
                                                     questLink.getDbUser(),
                                                     questLink.getChangedTime() ) ) ;
         }


         i++ ;
      }

      return( retVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getLinkType
    */
   /**
    *    This method will return what type of link that 
    *  this record represents. 0 = QuestLink, 1 = ExternalLink
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getLinkType () {
      return( linkTypeInd ) ;
   }






   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will do the comparison between
    *  another LinkDisplayRec and this one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( IQuickSort sortObject ) {
      int rc = 0 ;

      if ( sortObject instanceof LinkDisplayRec ) {
         LinkDisplayRec displayRec = (LinkDisplayRec) sortObject ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Do the comparison based on what we have to sort         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( sortOn ) {
            case SORT_CHANGEDTIME_ASC :
               rc = CDate.compareDate ( changedTime, displayRec.changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_CHANGEDTIME_DESC :
               rc = CDate.compareDate ( displayRec.changedTime, changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_TITLE_ASC :
               rc = title.compareTo ( displayRec.title ) ;
               break ;

            case SORT_TITLE_DESC :
               rc = displayRec.title.compareTo ( title ) ;
               break ;

            case SORT_DBUSER_ASC :
               rc = dbUser.compareTo ( displayRec.dbUser ) ;
               break ;

            case SORT_DBUSER_DESC :
               rc = displayRec.dbUser.compareTo ( dbUser ) ;
               break ;

            case SORT_DOCCLASS :
               rc = 0 ;
               break ;

            case SORT_INDICATOR_ASC :
               if ( linkInd < displayRec.linkInd )
                  rc = -1 ;
               else if ( linkInd > displayRec.linkInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_DESC :
               if ( linkInd < displayRec.linkInd )
                  rc = 1 ;
               else if ( linkInd > displayRec.linkInd )
                  rc = -1 ;
               else
                  rc = 0 ;
               break ;
         }

      }

      if ( rc > 0 )
         rc = 1 ;
      else if ( rc < 0 )
         rc = -1 ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setSort
    */
   /**
    *    This method will set what we need to sort
    *  on
    *  
    *
    *  @param sortOn The sort as defined in OASort
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSort ( int sortOn ) {
      this.sortOn = sortOn ;
   }

   public Vector getChildren () {
      return( null ) ;
   }

   public boolean containsChildren () {
      return( false ) ;
   }

   public void addChild ( DisplayRec displayRec ) {
   }

   public void removeChild ( DisplayRec displayRec ) {
   }





   /*------------------------------------------------------------*/
   /* getDocInd
    */
   /**
    *    This method will return the doc indicator if
    *  we have one. We won't have one if this
    *  is a URL link
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getDocInd () {
      return( docInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our full title
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( title ) ;
   }
} /* LinkDisplayRec */

