package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   LinkGroupDisplayRec                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkGroupDisplayRec                 */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
/**
 * This record simply shows a LinkGroup in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkGroupDisplayRec                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkGroupDisplayRec extends OAObjectDisplayRec implements IQuickSort, OASort, DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int     linkGroupInd      = 0 ;
   private String  title             = null ;
   private String  dbUser            = null ;
   private String  changedTime       = null ;
   private String  prettyChangedTime = null ;
   private String  prettyDBUser      = null ;
   private int     sortOn            = 0 ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupDisplayRec ( int linkGroupInd, String title, String dbUser, String changedTime ) {
      this.linkGroupInd      = linkGroupInd ;
      this.title             = title ;
      this.dbUser            = dbUser ;
      this.changedTime       = changedTime ;
      this.prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;
      this.prettyDBUser      = UserSystem.getNameFromUserId ( dbUser ) ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupDisplayRec ( ) {
   }

   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   LinkGroup Indicator                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+linkGroupInd ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            if ( getTitleArray() == null )
               createTitleArray ( title ) ;
            retObj = getTitleArray() ;
            break ;        

         case 4 :
            retObj = prettyChangedTime ;
            break ;

         case 5 :
            retObj = prettyDBUser ;
            break ;

         default:
            retObj = "" ;
      }


      return( retObj ) ;
   }

   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* getLinkGroupInd
    */
   /**
    *    This method will return the link group indicator
    *  that we will use
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getLinkGroupInd () {
      return( linkGroupInd ) ;
   }







   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will do the comparison between
    *  another LinkGroupDisplayRec and this one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( IQuickSort sortObject ) {
      int rc = 0 ;

      if ( sortObject instanceof LinkGroupDisplayRec ) {
         LinkGroupDisplayRec displayRec = (LinkGroupDisplayRec) sortObject ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Do the comparison based on what we have to sort         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( sortOn ) {
            case SORT_CHANGEDTIME_ASC :
               rc = CDate.compareDate ( changedTime, displayRec.changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_CHANGEDTIME_DESC :
               rc = CDate.compareDate ( displayRec.changedTime, changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_TITLE_ASC :
               rc = title.compareTo ( displayRec.title ) ;
               break ;

            case SORT_TITLE_DESC :
               rc = displayRec.title.compareTo ( title ) ;
               break ;

            case SORT_DBUSER_ASC :
               rc = dbUser.compareTo ( displayRec.dbUser ) ;
               break ;

            case SORT_DBUSER_DESC :
               rc = displayRec.dbUser.compareTo ( dbUser ) ;
               break ;

            case SORT_DOCCLASS :
               rc = 0 ;
               break ;

            case SORT_INDICATOR_ASC :
               if ( linkGroupInd < displayRec.linkGroupInd )
                  rc = -1 ;
               else if ( linkGroupInd > displayRec.linkGroupInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_DESC :
               if ( linkGroupInd < displayRec.linkGroupInd )
                  rc = 1 ;
               else if ( linkGroupInd > displayRec.linkGroupInd )
                  rc = -1 ;
               else
                  rc = 0 ;
               break ;
         }

      }

      if ( rc > 0 )
         rc = 1 ;
      else if ( rc < 0 ) 
         rc = -1 ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setSort
    */
   /**
    *    This method will set what we need to sort
    *  on
    *  
    *
    *  @param sortOn The sort as defined in OASort
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSort ( int sortOn ) {
      this.sortOn = sortOn ;
   }

   public Vector getChildren () {
      return ( null ) ;
   }

   public boolean containsChildren () {
      return ( false ) ;
   }

   public void addChild ( DisplayRec displayRec ) {
   }

   public void removeChild ( DisplayRec displayRec ) {
   }


   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our full title
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( title ) ;
   }

} /* LinkGroupDisplayRec */

