package com.ibm.nzna.projects.qit.avalon.base ;    
/*-----------------------------------------------------------*/
/*   OACustomView                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OACustomView                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import java.awt.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
/**
 * This object represents the Online Assistant (Avalon) custom view 
 *
 *
 * @version 1.0, 4/29/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OACustomView                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class OACustomView extends CustomView implements AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int[]     columnWidths   = { 100, 100, 100, 100, 100, 100, 100} ;
   private int       columnCount    = 3 ;
   private String    fromClause     = null ;
   private String    selectClause   = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OACustomView () {
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a blank
    *  OACustomView based on the type passed
    *  in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OACustomView ( int viewType ) {
      setViewType ( viewType ) ;
   }


   /*------------------------------------------------------------*/
   /* getSelect
    */
   /**
    *    This method will return the entire select
    *  clause including the 'SELECT' keyword
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getSelect () {
      return( selectClause ) ;
   }




   /*------------------------------------------------------------*/
   /* getWhere
    */
   /**
    *    This method will return the entire where clause
    *  including the 'WHERE' statement
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getWhere () {
      Vector          fields      = getFields() ;
      int             i           = 0 ;
      int             size        = fields.size() ;
      String          whereClause = null ;                      
      CustomViewField field       = null ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Start everything off real small                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         whereClause = "WHERE " ;

         while ( i < size ) {
            field = (CustomViewField) fields.elementAt(i) ;                  

            if ( field.getDatabaseOperator() != null ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the field database name                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               whereClause+=field.getColumnName()+" " ;       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the operator                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               whereClause+=field.getDatabaseOperator()+" " ;                                     

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, depending on the column, put the value in          */
               /*                                                           */
               /*-----------------------------------------------------------*/

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Title, Changed time, Additional information             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( field.getColumnName().indexOf ( "CHANGEDTIME" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "URL" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "ADDINFO" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "LNDOCID" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "TITLE" ) >= 0 ) ) {
                  if ( field.getDatabaseOperator().equals ( "LIKE" ) )
                     whereClause+="'%"+field.getValue()+"%' " ;
                  else if ( field.getDatabaseOperator().equals ( "IN" ) )
                     whereClause+="( "+field.getValue()+" ) " ;
                  else
                     whereClause+="'"+field.getValue()+"' " ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   DB User                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "DBUSER" ) >= 0 ) {
                  if ( field.getDatabaseOperator().equals ( "IN" ) )
                     whereClause+="( "+field.getValue()+" ) " ;
                  else
                     whereClause+="'"+field.getValue()+"' " ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Changed Time                                            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "CHANGEDTIME" ) >= 0 ) {
                  if ( field.getDatabaseOperator().equals ( "LIKE" ) )
                     whereClause+="'%"+field.getValue()+"%' " ;
                  else
                     whereClause+="'"+field.getValue()+"' " ;
               }

               whereClause+="AND " ;
            } /* if field was to be used  */

            i++ ;
         } /* where  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now put the standard joins in per view type         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( getViewType() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Link                                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            case CUSTOMVIEW_LINK :
               whereClause+="LINKTITLE.LINKIND = LINK.LINKIND AND "+
                            "LINK.ARCHIVED = 'N' AND "+
                            "LINKTITLE.LANGUAGEIND = "+PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Link Group                                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case CUSTOMVIEW_LINKGROUP :
               whereClause+="LINKGROUPTITLE.LINKGROUPIND = LINKGROUP.LINKGROUPIND AND "+
                            "LINKGROUP.ARCHIVED = 'N' AND "+
                            "LINKGROUPTITLE.LANGUAGEIND = "+PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Action                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case CUSTOMVIEW_ACTION :
               whereClause+="ACTIONTITLE.ACTIONIND = ACTION.ACTIONIND AND "+
                            "ACTION.ARCHIVED = 'N' AND "+
                            "ACTIONTITLE.LANGUAGEIND = "+PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Question                                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case CUSTOMVIEW_QUESTION :
               whereClause+="QUESTIONTITLE.QUESTIONIND = QUESTION.QUESTIONIND AND "+
                            "QUESTION.ARCHIVED = 'N' AND "+
                            "QUESTIONTITLE.LANGUAGEIND = "+PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Symptom                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case CUSTOMVIEW_SYMPTOM :               
               whereClause+="SYMPTOMTITLE.SYMPTOMIND = SYMPTOM.SYMPTOMIND AND "+
                            "SYMPTOM.ARCHIVED = 'N' AND "+
                            "SYMPTOMTITLE.LANGUAGEIND = "+PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
               break ;

            default:
               whereClause+="DO NOT KNOW TYPE!" ;
               break ;

         } /* switch  */


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }


      return( whereClause ) ;
   }




   /*------------------------------------------------------------*/
   /* getTables
    */
   /**
    *    This method will return all of the tables that
    *  this view will select from. This includes the
    *  keyword 'FROM'
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTables () {
      return( fromClause ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnWidths
    */
   /**
    *    This method will return the column widths
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int[]
    *//*---------------------------------------------------------*/
   public int[] getColumnWidths () {
      return( columnWidths ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    This method will return the column count
    *  for this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getColumnCount () {
      return( columnCount ) ;
   }





   /*------------------------------------------------------------*/
   /* instantiateObject
    */
   /**
    *    This method will instaniate an object
    *  of the type passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomView instantiateObject ( int customType ) {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* getQuery
    */
   /**
    *    This method will return our full query
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getQuery () {
      return( getSelect()+" "+getTables()+" "+getWhere() ) ;
   }


   /*------------------------------------------------------------*/
   /* getResults
    */
   /**
    *    This method will kick off the query and will
    *  return the results from the query
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getResults () {
      Vector retVec = null ;

      LogSystem.log ( 1, "Custom View Kicked off. SQL:"+getQuery() ) ;
      switch ( getViewType() ) {
         case CUSTOMVIEW_SYMPTOM :
            retVec = OASQL.readSymptomList ( getQuery() ) ;
            break ;

         case CUSTOMVIEW_ACTION :
            retVec = OASQL.readActionList ( getQuery () ) ;
            break ;

         case CUSTOMVIEW_QUESTION :
            retVec = OASQL.readQuestionList ( getQuery() ) ;
            break ;

         case CUSTOMVIEW_LINK :
            retVec = OASQL.readLinkList ( getQuery() ) ;
            break ;

         case CUSTOMVIEW_LINKGROUP :
            retVec = OASQL.readLinkGroupList ( getQuery () ) ;
            break ;
      }

      return( retVec ) ;
   }




   /*------------------------------------------------------------*/
   /* setViewType
    */
   /**
    *    This method will set what type of query this
    *  is
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setViewType ( int type ) {
      super.setViewType( type ) ;

      columnWidths[0] = 12 ;
      columnWidths[1] = 12 ;
      columnWidths[2] = 50 ;
      columnWidths[3] = 250 ;
      columnWidths[4] = 150 ;
      columnWidths[5] = 150 ;

      switch ( type ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Link                                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case CUSTOMVIEW_LINK :
            //columnWidths = new int [ 6 ] ;
            columnCount=7 ;
            selectClause = "SELECT LINK.LINKIND,  LINK.URL,    LINK.DBUSER,     LINK.CHANGEDTIME, "+
                           "       LINK.URLVALID, LINK.DOCIND, LINKTITLE.TITLE, LINK.LINKTYPE " ;
            fromClause  = "FROM OA.LINK LINK, OA.LINKTITLE LINKTITLE" ;

            fields = new Vector ( 5 ) ;
            fields.addElement ( new CustomViewField ( "LINKTITLE.TITLE",  Str.getStr ( STR_TITLE ) ) ) ;
            fields.addElement ( new CustomViewField ( "LINK.DBUSER",      Str.getStr ( STR_LAST_TOUCHED_BY ) ) ) ;
            fields.addElement ( new CustomViewField ( "LINK.CHANGEDTIME", Str.getStr ( STR_LAST_TOUCHED_ON ) ) ) ;
            fields.addElement ( new CustomViewField ( "LINK.URL",         Str.getStr ( STR_URL ) ) ) ;
            fields.addElement ( new CustomViewField ( "DOC.LNDOCIND",     Str.getStr ( STR_LOTUS_NOTES_ID ) ) ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Link Group                                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case CUSTOMVIEW_LINKGROUP :
            //columnWidths = new int [ 3 ] ;
            columnCount=3 ;
            selectClause = "SELECT DISTINCT (LINKGROUP.LINKGROUPIND), LINKGROUPTITLE.TITLE, LINKGROUP.DBUSER, LINKGROUP.CHANGEDTIME" ;
            fromClause  = "FROM OA.LINKGROUP LINKGROUP, OA.LINKGROUPTITLE LINKGROUPTITLE" ;
            fields = new Vector ( 5 ) ;
            fields.addElement ( new CustomViewField ( "LINKGROUPTITLE.TITLE",  Str.getStr ( STR_TITLE ) ) ) ;
            fields.addElement ( new CustomViewField ( "LINKGROUP.DBUSER",      Str.getStr ( STR_LAST_TOUCHED_BY ) ) ) ;
            fields.addElement ( new CustomViewField ( "LINKGROUP.CHANGEDTIME", Str.getStr ( STR_LAST_TOUCHED_ON ) ) ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Action                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case CUSTOMVIEW_ACTION :
            //columnWidths = new int [ 5 ] ;
            columnCount=5;
            selectClause = "SELECT DISTINCT (ACTION.ACTIONIND), ACTION.DOCCLASSIND, ACTIONTITLE.TITLE, "+
                           "       ACTION.CHANGEDTIME, ACTION.DBUSER" ;
            fromClause  = "FROM OA.ACTION ACTION, OA.ACTIONTITLE ACTIONTITLE" ;
            fields = new Vector ( 5 ) ;
            fields.addElement ( new CustomViewField ( "ACTIONTITLE.TITLE",        Str.getStr ( STR_TITLE ) ) ) ;
            fields.addElement ( new CustomViewField ( "ACTIONADDINFO.ADDINFO",    Str.getStr ( STR_ADDITIONAL_INFORMATION ) ) ) ;
            fields.addElement ( new CustomViewField ( "ACTION.DBUSER",            Str.getStr ( STR_LAST_TOUCHED_BY ) ) ) ;
            fields.addElement ( new CustomViewField ( "ACTION.CHANGEDTIME",       Str.getStr ( STR_LAST_TOUCHED_ON ) ) ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Question                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case CUSTOMVIEW_QUESTION :
            //columnWidths = new int [ 5 ] ;
            columnCount=5 ;
            selectClause = "SELECT DISTINCT (QUESTION.QUESTIONIND), QUESTION.DOCCLASSIND, QUESTIONTITLE.TITLE, "+
                           "       QUESTION.CHANGEDTIME, QUESTION.DBUSER" ;
            fromClause  = "FROM OA.QUESTION QUESTION, OA.QUESTIONTITLE QUESTIONTITLE" ;
            fields = new Vector ( 5 ) ;
            fields.addElement ( new CustomViewField ( "QUESTIONTITLE.TITLE",        Str.getStr ( STR_TITLE ) ) ) ;
            fields.addElement ( new CustomViewField ( "QUESTIONADDINFO.ADDINFO",    Str.getStr ( STR_ADDITIONAL_INFORMATION ) ) ) ;
            fields.addElement ( new CustomViewField ( "QUESTION.DBUSER",            Str.getStr ( STR_LAST_TOUCHED_BY ) ) ) ;
            fields.addElement ( new CustomViewField ( "QUESTION.CHANGEDTIME",       Str.getStr ( STR_LAST_TOUCHED_ON ) ) ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Symptom                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case CUSTOMVIEW_SYMPTOM :               
            //columnWidths = new int [ 5 ] ;
            columnCount = 5 ;
            selectClause = "SELECT DISTINCT (SYMPTOM.SYMPTOMIND), SYMPTOM.DOCCLASSIND, SYMPTOMTITLE.TITLE, "+
                           "       SYMPTOM.CHANGEDTIME, SYMPTOM.DBUSER" ;
            fromClause  = "FROM OA.SYMPTOM SYMPTOM, OA.SYMPTOMTITLE SYMPTOMTITLE" ;
            fields = new Vector ( 5 ) ;
            fields.addElement ( new CustomViewField ( "SYMPTOMTITLE.TITLE",        Str.getStr ( STR_TITLE ) ) ) ;
            fields.addElement ( new CustomViewField ( "SYMPTOM.DBUSER",            Str.getStr ( STR_LAST_TOUCHED_BY ) ) ) ;
            fields.addElement ( new CustomViewField ( "SYMPTOM.CHANGEDTIME",       Str.getStr ( STR_LAST_TOUCHED_ON ) ) ) ;
            break ;

      } /* switch  */
   }






   /*------------------------------------------------------------*/
   /* addFieldRestriction
    */
   /**
    *    This method will add a field restriction
    *  to this query
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addFieldRestriction ( CustomViewField field ) {
      Frame  tempFrame   = new Frame() ;                                          
      Vector operatorVec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always ask the user for what type of operator first     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      operatorVec = new JListDlg ( tempFrame, Str.getStr ( STR_OPERATOR ), field.getOperators() ).getResult() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only continue if they selected an operator              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( operatorVec != null ) && ( operatorVec.size() > 0 ) ) {

         field.setOperator ( (String)operatorVec.elementAt(0) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Additional Information                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( field.getColumnName().indexOf ( "ADDINFO" ) >= 0 ) {
            String addInfo = new SingleEntryDlg ( tempFrame,
                                                  Str.getStr ( STR_ADDITIONAL_INFORMATION ),
                                                  Str.getStr ( STR_PLEASE_ENTER_ADD_INFO_CUSTOM_VIEW ),
                                                  "",
                                                  "" ).getResult() ;
            if ( ( addInfo != null ) && ( addInfo.length() > 0 ) ) {
               field.setValue ( addInfo ) ;
            }

         } /* additionalInformation  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Title                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "TITLE" ) >= 0 ) {
            String title = new SingleEntryDlg ( tempFrame,
                                                Str.getStr ( STR_TITLE ),
                                                Str.getStr ( STR_PLEASE_ENTER_TITLE ),
                                                "",
                                                "" ).getResult() ;
            if ( ( title != null ) && ( title.length() > 0 ) )
               field.setValue ( title ) ;

         } /* title  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Url                                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "URL" ) >= 0 ) {
            String url = new SingleEntryDlg ( tempFrame,
                                              Str.getStr ( STR_URL ),
                                              Str.getStr ( STR_URL ),
                                              "",
                                              "" ).getResult() ;
            if ( ( url != null ) && ( url.length() > 0 ) ) {
               field.setValue ( url ) ;
            }

         } /* url  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   DB User                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "DBUSER" ) >= 0 ) {
            Vector  vector  = new SelectUserDlg ( tempFrame ).getResult() ;
            UserRec userRec = null ;

            if ( ( vector != null ) && ( vector.size() > 0 ) ) {
               int     i    = 0 ;
               int     size = vector.size() ;
               String  str  = "" ;

               while ( i < size ) {
                  str+=((UserRec)vector.elementAt(i)) ;
                  if ( i != (size-1) )
                     str+="," ;
                  i++ ;
               } /* while  */

               field.setValue ( str ) ;

            } /* if  */
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Changed Time                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "CHANGEDTIME" ) >= 0 ) {
            String date = new SingleDateEntryDlg ( tempFrame,
                                                   Str.getStr ( STR_TITLE ),
                                                   Str.getStr ( STR_PLEASE_ENTER_TITLE ),
                                                   new CDate ().today() ).getResult() ;
            if ( ( date != null ) && ( date.length() > 0 ) ) {
               date = Text.replaceAllStrInStr ( date, "/", "-" ) ;
               field.setValue ( date+"-07.00.00.000000" ) ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Changed Time                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "LNDOCID" ) >= 0 ) {
            String lndocid = new SingleEntryDlg ( tempFrame,
                                                  Str.getStr ( STR_LOTUS_NOTES_ID ),
                                                  Str.getStr ( STR_LOTUS_NOTES_ID ),
                                                  "", "" ).getResult() ;
            if ( ( lndocid != null ) && ( lndocid.length() > 0 ) )
               field.setValue ( lndocid ) ;

         }

      } /* if they selected an operator  */

      tempFrame.dispose() ;
   }

} /* OACustomView */
