package com.ibm.nzna.projects.qit.avalon.base ;           
/*-----------------------------------------------------------*/
/*   OAObjectArchiver                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OAObjectArchiver                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.editors.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;

/**
 * This class holds methods that will archive objects in Avalon. 
 *
 *
 * @version 1.0, 11/16/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OAObjectArchiver                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class OAObjectArchiver extends Object implements QuestProcess, AppConst {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static boolean continueProcess = true ;


   /*------------------------------------------------------------*/
   /* archiveObject
    */
   /**
    *    This method will archive the Action
    *  passed in. If this action draft has any usages
    *  anywhere a message will display and it will not archive. Please note
    *  that the draft passed in MUST be saved already to the database with it's
    *  archive flag set to 'TRUE!'.
    *
    *  @param action The Action to archive
    *  @param actionEditorPanel If available the ActionEditorPanel that launched the archive. This can be NULL
    *  @param None
    *
    *  @return TRUE if archived
    *//*---------------------------------------------------------*/
   public static boolean archiveObject ( ActionDraft action, ActionEditorPanel actionEditorPanel ) {
      boolean       archived   = false ;
      AppDefaultWin defaultWin = null ;

      if ( actionEditorPanel != null )
         defaultWin = GUISystem.getParentDefWin ( actionEditorPanel ) ;
      else
         defaultWin = MainWindow.getInstance() ;

      if ( action != null ) {
         Vector usage = null ;            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, grab the usage                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         defaultWin.setStatus ( Str.getStr ( STR_VALIDATING ) ) ;
         usage = ActionUsage.getUsage( action ) ;                           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If there is any usage, tell them now                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( usage.size() > 0 ) {
            boolean checkUsage = false ;

            defaultWin.setStatus ( null ) ;
            checkUsage = GUISystem.printBox ( STR_QUESTION,
                                              STR_ITEM_HAS_USAGE_CANNOT_ARCHIVE ) ;                                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If they chose to see the objects, edit it and show!     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( checkUsage ) {
               if ( actionEditorPanel != null )
                  actionEditorPanel.showPanel ( ActionEditorPanel.PANEL_USAGE ) ;
               else {
                  actionEditorPanel = new ActionEditorPanel ( null, action, ActionEditorPanel.PANEL_USAGE ) ;
                  WindowSystem.createPanel ( actionEditorPanel ) ;
               }
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, archive the action                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            defaultWin.setStatus ( Str.getStr ( STR_SAVING ) ) ;
            archived = action.publish() ;
         }
      }

      defaultWin.setStatus ( null ) ;
      return( archived ) ;
   }




   /*------------------------------------------------------------*/
   /* archiveObject
    */
   /**
    *    This method will archive the Question
    *  passed in. If this question draft has any usages
    *  anywhere a message will display and it will not archive. Please note
    *  that the draft passed in MUST be saved already to the database with it's
    *  archive flag set to 'TRUE!'.
    *
    *  @param question The Question to archive
    *  @param questionEditorPanel If available the QuestionEditorPanel that launched the archive. This can be NULL
    *  @param None
    *
    *  @return TRUE if archived
    *//*---------------------------------------------------------*/
   public static boolean archiveObject ( QuestionDraft question, QuestionEditorPanel questionEditorPanel ) {
      boolean          archived   = false ;
      AppDefaultWin    defaultWin = null ;
      OAObjectArchiver archiver   = new OAObjectArchiver() ;

      continueProcess = true ;
      ProcessManagerSystem.addProcess ( archiver ) ;

      if ( questionEditorPanel != null )
         defaultWin = GUISystem.getParentDefWin ( questionEditorPanel ) ;
      else
         defaultWin = MainWindow.getInstance() ;                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we have a valid question, continue               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( question != null ) {
         Vector  usage        = null ;            
         boolean objectIsUsed = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, grab the usage                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         defaultWin.setStatus ( Str.getStr ( STR_VALIDATING ) ) ;

         objectIsUsed = (QuestionUsage.getQuestionAnswerUsageInActions ( question ).size() > 0 );
         if ( ( !objectIsUsed ) && ( continueProcess ) )
            objectIsUsed = (QuestionUsage.getQuestionAnswerUsageInQuestions ( question ).size() > 0 );
         if ( ( !objectIsUsed ) && ( continueProcess ) )
            objectIsUsed = (QuestionUsage.getQuestionAnswerUsageInSymptoms ( question ).size() > 0 );
         if ( ( !objectIsUsed ) && ( continueProcess ) )
            objectIsUsed = (QuestionUsage.getQuestionProductUsage ( question ).size() > 0 );
         if ( ( !objectIsUsed ) && ( continueProcess ) )
            objectIsUsed = (QuestionUsage.getQuestionUsageInSymptoms ( question ).size() > 0 );


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If there is any usage, tell them now                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( continueProcess ) && ( objectIsUsed ) ) {
            boolean checkUsage = false ;

            defaultWin.setStatus ( null ) ;
            checkUsage = GUISystem.printBox ( STR_QUESTION,
                                              STR_ITEM_HAS_USAGE_CANNOT_ARCHIVE ) ;                                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If they chose to see the objects, edit it and show!     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( checkUsage ) {
               if ( questionEditorPanel != null )
                  questionEditorPanel.showPanel ( QuestionEditorPanel.PANEL_USAGE ) ;
               else {
                  questionEditorPanel = new QuestionEditorPanel ( null, question, QuestionEditorPanel.PANEL_USAGE ) ;
                  WindowSystem.createPanel ( questionEditorPanel ) ;
               }
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, archive the question                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( continueProcess ) {
            defaultWin.setStatus ( Str.getStr ( STR_SAVING ) ) ;
            archived = question.publish() ;
         }
      }


      ProcessManagerSystem.removeProcess ( archiver ) ;
      archiver = null ;
      defaultWin.setStatus ( null ) ;
      return( archived ) ;
   }






   /*------------------------------------------------------------*/
   /* archiveObject
    */
   /**
    *    This method will archive the Symptom
    *  passed in. If this symptom draft has any usages
    *  anywhere a message will display and it will not archive. Please note
    *  that the draft passed in MUST be saved already to the database with it's
    *  archive flag set to 'TRUE!'.
    *
    *  @param symptom The Symptom to archive
    *  @param symptomEditorPanel If available the SymptomEditorPanel that launched the archive. This can be NULL
    *  @param None
    *
    *  @return TRUE if archived
    *//*---------------------------------------------------------*/
   public static boolean archiveObject ( SymptomDraft symptom, SymptomEditorPanel symptomEditorPanel ) {
      boolean          archived   = false ;
      AppDefaultWin    defaultWin = null ;
      OAObjectArchiver archiver   = new OAObjectArchiver() ;

      continueProcess = true ;
      ProcessManagerSystem.addProcess ( archiver ) ;

      if ( symptomEditorPanel != null )
         defaultWin = GUISystem.getParentDefWin ( symptomEditorPanel ) ;
      else
         defaultWin = MainWindow.getInstance() ;                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we have a valid symptom, continue                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( symptom != null ) {
         Vector  usage        = null ;            
         boolean objectIsUsed = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, grab the usage                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         defaultWin.setStatus ( Str.getStr ( STR_VALIDATING ) ) ;

         objectIsUsed = (SymptomUsage.getSymptomUsage ( symptom ).size() > 0 );

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If there is any usage, tell them now                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( continueProcess ) && ( objectIsUsed ) ) {
            boolean checkUsage = false ;

            defaultWin.setStatus ( null ) ;
            checkUsage = GUISystem.printBox ( STR_QUESTION,
                                              STR_ITEM_HAS_USAGE_CANNOT_ARCHIVE ) ;                                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If they chose to see the objects, edit it and show!     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( checkUsage ) {
               if ( symptomEditorPanel != null )
                  symptomEditorPanel.showPanel ( SymptomEditorPanel.PANEL_USAGE ) ;
               else {
                  symptomEditorPanel = new SymptomEditorPanel ( null, symptom, SymptomEditorPanel.PANEL_USAGE ) ;
                  WindowSystem.createPanel ( symptomEditorPanel ) ;
               }
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, archive the symptom                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( continueProcess ) {
            defaultWin.setStatus ( Str.getStr ( STR_SAVING ) ) ;
            archived = symptom.publish() ;
         }
      }


      ProcessManagerSystem.removeProcess ( archiver ) ;
      archiver = null ;
      defaultWin.setStatus ( null ) ;
      return( archived ) ;
   }










   /*------------------------------------------------------------*/
   /* archiveObject
    */
   /**
    *    This method will archive the Link
    *  passed in. If this link draft has any usages
    *  anywhere a message will display and it will not archive. Please note
    *  that the draft passed in MUST be saved already to the database with it's
    *  archive flag set to 'TRUE!'.
    *
    *  @param link The LinkDraft to archive
    *  @param linkEditorPanel If available the LinkEditorPanel that launched the archive. This can be NULL
    *  @param None
    *
    *  @return TRUE if archived
    *//*---------------------------------------------------------*/
   public static boolean archiveObject ( LinkDraft link, LinkEditorPanel linkEditorPanel ) {
      boolean          archived   = false ;
      AppDefaultWin    defaultWin = null ;
      OAObjectArchiver archiver   = new OAObjectArchiver() ;

      continueProcess = true ;
      ProcessManagerSystem.addProcess ( archiver ) ;

      if ( linkEditorPanel != null )
         defaultWin = GUISystem.getParentDefWin ( linkEditorPanel ) ;
      else
         defaultWin = MainWindow.getInstance() ;                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we have a valid link, continue                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( link != null ) {
         Vector  usage        = null ;            
         boolean objectIsUsed = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, grab the usage                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         defaultWin.setStatus ( Str.getStr ( STR_VALIDATING ) ) ;

         objectIsUsed = (LinkUsage.getLinkUsageInActions ( link ).size() > 0 );
         if ( ( continueProcess ) && ( !objectIsUsed ) )
            objectIsUsed = (LinkUsage.getLinkUsageInQuestions ( link ).size() > 0 );
         if ( ( continueProcess ) && ( !objectIsUsed ) )
            objectIsUsed = (LinkUsage.getLinkUsageInLinkGroups ( link ).size() > 0 );

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If there is any usage, tell them now                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( continueProcess ) && ( objectIsUsed ) ) {
            boolean checkUsage = false ;

            defaultWin.setStatus ( null ) ;
            checkUsage = GUISystem.printBox ( STR_QUESTION,
                                              STR_ITEM_HAS_USAGE_CANNOT_ARCHIVE ) ;                                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If they chose to see the objects, edit it and show!     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( checkUsage ) {
               if ( linkEditorPanel != null )
                  linkEditorPanel.showPanel ( LinkEditorPanel.PANEL_USAGE ) ;
               else {
                  
                  if ( link instanceof ExternalLinkDraft )
                     linkEditorPanel = new LinkEditorPanel ( null, (ExternalLinkDraft)link, LinkEditorPanel.PANEL_USAGE ) ;
                  else
                     linkEditorPanel = new LinkEditorPanel ( null, (QuestLinkDraft)link, LinkEditorPanel.PANEL_USAGE ) ;

                  WindowSystem.createPanel ( linkEditorPanel ) ;
               }
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, archive the link                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( continueProcess ) {
            defaultWin.setStatus ( Str.getStr ( STR_SAVING ) ) ;
            archived = link.publish() ;
         }
      }


      ProcessManagerSystem.removeProcess ( archiver ) ;
      archiver = null ;
      defaultWin.setStatus ( null ) ;
      return( archived ) ;
   }




   /*------------------------------------------------------------*/
   /* archiveObject
    */
   /**
    *    This method will archive the LinkGroup
    *  passed in. If this linkGroup draft has any usages
    *  anywhere a message will display and it will not archive. Please note
    *  that the draft passed in MUST be saved already to the database with it's
    *  archive flag set to 'TRUE!'.
    *
    *  @param linkGroup The LinkGroup to archive
    *  @param linkGroupEditorPanel If available the LinkGroupEditorPanel that launched the archive. This can be NULL
    *  @param None
    *
    *  @return TRUE if archived
    *//*---------------------------------------------------------*/
   public static boolean archiveObject ( LinkGroupDraft linkGroup, LinkGroupEditorPanel linkGroupEditorPanel ) {
      boolean          archived   = false ;
      AppDefaultWin    defaultWin = null ;
      OAObjectArchiver archiver   = new OAObjectArchiver() ;

      continueProcess = true ;
      ProcessManagerSystem.addProcess ( archiver ) ;

      if ( linkGroupEditorPanel != null )
         defaultWin = GUISystem.getParentDefWin ( linkGroupEditorPanel ) ;
      else
         defaultWin = MainWindow.getInstance() ;                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we have a valid linkGroup, continue              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( linkGroup != null ) {
         Vector  usage        = null ;            
         boolean objectIsUsed = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, grab the usage                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         defaultWin.setStatus ( Str.getStr ( STR_VALIDATING ) ) ;

         objectIsUsed = (LinkGroupUsage.getLinkGroupUsageInActions ( linkGroup ).size() > 0 );
         if ( ( continueProcess ) && ( !objectIsUsed ) )
            objectIsUsed = (LinkGroupUsage.getLinkGroupUsageInQuestions ( linkGroup ).size() > 0 );

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If there is any usage, tell them now                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( continueProcess ) && ( objectIsUsed ) ) {
            boolean checkUsage = false ;

            defaultWin.setStatus ( null ) ;
            checkUsage = GUISystem.printBox ( STR_QUESTION,
                                              STR_ITEM_HAS_USAGE_CANNOT_ARCHIVE ) ;                                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If they chose to see the objects, edit it and show!     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( checkUsage ) {
               if ( linkGroupEditorPanel != null )
                  linkGroupEditorPanel.showPanel ( LinkGroupEditorPanel.PANEL_USAGE ) ;
               else {
                  linkGroupEditorPanel = new LinkGroupEditorPanel ( null, linkGroup, LinkGroupEditorPanel.PANEL_USAGE ) ;
                  WindowSystem.createPanel ( linkGroupEditorPanel ) ;
               }
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, archive the linkGroup                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( continueProcess ) {
            defaultWin.setStatus ( Str.getStr ( STR_SAVING ) ) ;
            archived = linkGroup.publish() ;
         }
      }


      ProcessManagerSystem.removeProcess ( archiver ) ;
      archiver = null ;
      defaultWin.setStatus ( null ) ;
      return( archived ) ;
   }







   /*------------------------------------------------------------*/
   /* getProcessName
    */
   /**
    *    THis method will return the name of our process
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getProcessName () {
      return( "OAObjectArchiver" ) ;
   }




   /*------------------------------------------------------------*/
   /* stopProcess
    */
   /**
    *    This method will try and stop the current process
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopProcess () {
      continueProcess = false ;
   }

} /* OAObjectArchiver */
