package com.ibm.nzna.projects.qit.avalon.base ;            
/*-----------------------------------------------------------*/
/*   OAObjectDisplayRec                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OAObjectDisplayRec                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;     
import com.ibm.nzna.shared.util.* ;
/**
 * This DisplayRec is the super class to all OA Object Display Records, this 
 * includes the Question, Action and Symptom objects.
 *
 *
 * @version 1.0, 9/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OAObjectDisplayRec                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class OAObjectDisplayRec extends Object implements AvalonConst, OASort, java.io.Serializable, DisplayRec {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;     


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static int charWidth = 0 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private   Vector   childVec   = null ;
   private   int      sort       = SORT_CHANGEDTIME_ASC ;
   private   String[] titleArray = null ;


   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method returns NULL, any subclass of
    *  this class should provide the necessary code
    *  to return the Object
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NULL
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* containsChildren
    */
   /**
    *    This method will return TRUE if this Question has
    *  multiple titles. Each of the titles point to a child.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if this Question has Multiple Titles
    *//*---------------------------------------------------------*/
   public boolean containsChildren () {
      return( ( childVec != null ) && ( childVec.size() > 0 ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getChildren
    */
   /**
    *    This method will return a vector of child
    *  objects
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getChildren () {
      return( childVec ) ;
   }


   /*------------------------------------------------------------*/
   /* addChild
    */
   /**
    *    This method will add a DisplayRec
    *  as a child of this record. 
    *  This is normally the same Object as this record is displaying,
    *  but with a different title
    *
    *  @param displayRec The Child DisplayRec to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addChild ( DisplayRec displayRec ) {
      if ( childVec == null )
         childVec = new Vector ( 1, 1 ) ;

      childVec.addElement ( displayRec ) ;
   }





   /*------------------------------------------------------------*/
   /* removeChild
    */
   /**
    *    This method will remove a child from this
    *  questionDisplayRec
    *  
    *
    *  @param displayRec The DisplayRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeChild ( DisplayRec displayRec ) {
      if ( childVec != null ) {
         childVec.removeElement ( displayRec ) ;
         if ( childVec.size() == 0 )
            childVec = null ;
      }
   }





   /*------------------------------------------------------------*/
   /* setSort
    */
   /**
    *    This method will set the sort that we are
    *  using to sort with. This sort should be one of
    *  the predefined values in OASort
    *
    *  @param sortInd The Sort to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSort ( int sort ) {
      this.sort = sort ;
   }





   /*------------------------------------------------------------*/
   /* getSort
    */
   /**
    *    This method will return the sort that we 
    *  should be sorted with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return sortInd
    *//*---------------------------------------------------------*/
   public int getSort () {
      return( sort ) ;
   }




   /*------------------------------------------------------------*/
   /* createTitleArray
    */
   /**
    *    This method will create the title array from
    *  the title given
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void createTitleArray ( String title ) {

      if ( charWidth == 0 )
         charWidth = FontUtil.getCharWidth ( GUISystem.getFontUtil(), 'k' ) ;

      titleArray = Text.formatStringLines ( title, WIDTH_JTREE_TITLE/charWidth ) ;
   }




   /*------------------------------------------------------------*/
   /* getTitleArray
    */
   /**
    *    This method will return the title array that
    *  has been created
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected String[] getTitleArray () {
      return( titleArray ) ;
   }



} /* OAObjectDisplayRec */
