package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   OAObjectPublisher                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OAObjectPublisher                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import java.io.* ;
import java.net.* ;
import com.ibm.nzna.shared.ftp.* ;      
import com.ibm.nzna.shared.ftp.FTP.* ;      
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.common.storedProc.oaValidate.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This thread publishes a Question by doing these steps:<br><br><pre>
 *
 * 1. Validate each dependency object passed in the ValidationRec<br>
 * 2. If any error occurs during validation, stop immediatly and return an HTML string containing the error<br>
 * 3. If all objects pass validation then publish each object. <br><br></pre>
 *
 * This thread interacts with the PublishPanel passed in by setting the status line and the progress
 * bar accordingly.
 *
 *
 * @version 1.0, 5/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OAObjectPublisher                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class OAObjectPublisher extends Object implements AvalonConst, Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Object        draft         = null ;
   private PublishPanel  publishPanel  = null ;
   private ValidationRec validationRec = null ;
   private int           currentStatus = 0 ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the Thread but
    *  will not start the thread. To start the thread
    *  please call 'start'
    *
    *  @param publishPanel The PublishPanel to interact with
    *  @param validationRec The validationRec to get dependencies with
    *  @param draft The Draft to publish
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OAObjectPublisher ( PublishPanel publishPanel, ValidationRec validationRec, Object draft ) {
      this.publishPanel  = publishPanel ;
      this.validationRec = validationRec ;
      this.draft         = draft ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will start the publishing process,
    *  keep in mind that this method really kicks off another
    *  thread to do the work so this method will immediatly return
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      new Thread ( this ).start() ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread method will publish the question
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {                                        
      int     statusValue     = 0 ;
      boolean continuePublish = false ;

      if ( validationRec != null ) {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, count up all of the objects to save              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( validationRec.actionCoreqVec != null ) &&
              ( validationRec.actionCoreqVec.size() > 0 ) )
            statusValue+=validationRec.actionCoreqVec.size() ;

         if ( ( validationRec.questionCoreqVec != null ) && 
              ( validationRec.questionCoreqVec.size() > 0 ) )
            statusValue+=validationRec.questionCoreqVec.size() ;

         if ( ( validationRec.symptomCoreqVec != null ) &&
              ( validationRec.symptomCoreqVec.size() > 0 ) )
            statusValue+=validationRec.symptomCoreqVec.size() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, set up the GUI text                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         publishPanel.setPublishingText ( Str.getStr ( STR_ACTIONS ) ) ;
         publishPanel.setValue ( 0 ) ;
         publishPanel.setMaximum ( statusValue ) ;          

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Publish the actions                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( validationRec.actionCoreqVec != null ) &&
              ( validationRec.actionCoreqVec.size() > 0 ) )
            continuePublish = publishObjects ( validationRec.actionCoreqVec, AVALON_ACTION ) ;
         else
            continuePublish = true ;
         LogSystem.log ( 1, "OAObjectPublisher: After Publishing actionCoreqVec. Publishing Return Code:"+continuePublish ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Publish the questions                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         publishPanel.setPublishingText ( Str.getStr ( STR_QUESTIONS ) ) ;
         if ( ( continuePublish ) && 
              ( validationRec.questionCoreqVec != null ) &&
              ( validationRec.questionCoreqVec.size() > 0 ) )
            continuePublish = publishObjects ( validationRec.questionCoreqVec, AVALON_QUESTION ) ;
         LogSystem.log ( 1, "OAObjectPublisher: After Publishing questionCoreqVec. Publishing Return Code:"+continuePublish ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Publish the symptoms                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         publishPanel.setPublishingText ( Str.getStr ( STR_SYMPTOMS ) ) ;
         if ( ( continuePublish ) && 
              ( validationRec.symptomCoreqVec != null ) &&
              ( validationRec.symptomCoreqVec.size() > 0 ) )
            continuePublish = publishObjects ( validationRec.symptomCoreqVec, AVALON_SYMPTOM ) ;
         LogSystem.log ( 1, "OAObjectPublisher: After Publishing symptomCoreqVec. Publishing Return Code:"+continuePublish ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now publish the object we were asked to publish     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         publishPanel.setPublishingText ( "" ) ;
         if ( continuePublish ) {

            publishPanel.setStatusText ( Str.getStr ( STR_PUBLISHING ) ) ;

            if ( draft instanceof QuestionDraft )
               continuePublish = ((QuestionDraft)draft).publish () ;
            else if ( draft instanceof SymptomDraft )
               continuePublish = ((SymptomDraft)draft).publish () ;
            else if ( draft instanceof ActionDraft )
               continuePublish = ((ActionDraft)draft).publish () ;
            else if ( draft instanceof QuestLinkDraft )
               continuePublish = ((QuestLinkDraft)draft).publish() ;
            else if ( draft instanceof ExternalLinkDraft )
               continuePublish = ((ExternalLinkDraft)draft).publish() ;
            else if ( draft instanceof LinkGroupDraft )
               continuePublish = ((LinkGroupDraft)draft).publish() ;
            else
               GUISystem.printBox ( "Error", "OAObjectPublisher does not know what type of object to publish!" ) ;

            LogSystem.log ( 1, "OAObjectPublisher: After Publishing Main Draft Object. Publishing Return Code:"+continuePublish ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Clear the GUI                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         publishPanel.setValue ( 0 ) ;
         publishPanel.setStatusText ( "" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now let the publish panel know that we are done     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         LogSystem.log ( 1, "OAObjectPublisher complete. Publishing Return Code:"+continuePublish ) ;
         publishPanel.publishComplete ( continuePublish ) ;
      } else {
         GUISystem.printBox ( "Internal Error", "Could not validate for publish. Publish cannot continue" ) ;
         publishPanel.publishComplete ( false ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* publishObjects
    */
   /**
    *    This method will publish objects to the database
    *  
    *  
    *
    *  @param vec The Vector of objects to publish
    *  @param objectType The type of object to publish ( AVALON_QUESTION, AVALON_ACTION or AVALON_SYMPTOM )
    *  @param None
    *
    *  @return TRUE if everything published Ok
    *//*---------------------------------------------------------*/
   private boolean publishObjects ( Vector vec, int objectType ) {
      boolean published = true ;
      int     i         = 0 ;
      int     size      = vec.size() ;                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, we must publish the graphics                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( publishGraphics ( vec, objectType ) ) {

         while ( ( published ) && ( i < size ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Update the GUI                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            publishPanel.setStatusText ( Str.getStr ( STR_PUBLISHING )+" - "+((Integer)vec.elementAt(i)).intValue() ) ;


            switch ( objectType ) {
               case AVALON_ACTION :
                  ActionDraft actionDraft = new ActionDraft ( ((Integer)vec.elementAt(i)).intValue() ) ;

                  published =  actionDraft.publish() ;
                  break ;

               case AVALON_QUESTION :
                  QuestionDraft draft = new QuestionDraft ( ((Integer)vec.elementAt(i)).intValue() ) ;

                  published =  draft.publish() ;
                  break ;

               case AVALON_SYMPTOM :
                  SymptomDraft symptomDraft = new SymptomDraft ( ((Integer)vec.elementAt(i)).intValue() ) ;

                  published =  symptomDraft.publish() ;
                  break ;
            }

            currentStatus++ ;
            publishPanel.setValue ( currentStatus ) ;
            i++ ;
         }
      } /* if publishGraphics  */

      if ( !published )
         GUISystem.printBox ( STR_ERROR, STR_LAST_OBJECT_DID_NOT_PUBLISH_RETRY ) ;

      return( published ) ;
   }




   /*------------------------------------------------------------*/
   /* publishGraphics
    */
   /**
    *    This method will publish all graphics for
    *  all objects passed in
    *  
    *
    *  @param vec The Objects to publish
    *  @param objectType The type of objects to publish
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean publishGraphics ( Vector vec, int objectType ) {
      boolean published  = true ;
      int     i                = 0 ;
      int     size             = vec.size() ;
      String  indicators       = "" ;                                            
      Vector  graphicFilenames = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, produce a list of indicators. We will use this   */
      /* list to get all graphic filenames that must be FTPed      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( published ) && ( i < size ) ) {

         switch ( objectType ) {
            case AVALON_ACTION :
               ActionDraft actionDraft = new ActionDraft ( ((Integer)vec.elementAt(i)).intValue() ) ;

               indicators+=""+actionDraft.getInd() ;
               break ;

            case AVALON_QUESTION :
               QuestionDraft questionDraft = new QuestionDraft ( ((Integer)vec.elementAt(i)).intValue() ) ;

               indicators+=""+questionDraft.getInd() ;
               break ;

            case AVALON_SYMPTOM :
               SymptomDraft symptomDraft = new SymptomDraft ( ((Integer)vec.elementAt(i)).intValue() ) ;

               indicators+=""+symptomDraft.getInd() ;
               break ;
         }

         if ( i < (size-1) )
            indicators+=", " ;
         currentStatus++ ;
         publishPanel.setValue ( currentStatus ) ;
         i++ ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read in all of the graphics filenames              */                                 
      /*                                                           */
      /*-----------------------------------------------------------*/
      graphicFilenames = readGraphicFilenames ( indicators, objectType ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Update the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( graphicFilenames != null ) ) {

         if ( graphicFilenames.size() > 0 ) {
            publishPanel.setStatusText ( Str.getStr ( STR_PUBLISHING_GRAPHICS ) ) ;
            
            published = ftpGraphics ( graphicFilenames ) ;

            if ( !published )
               GUISystem.printBox ( STR_ERROR, STR_GRAPHICS_NOT_PUBLISHED ) ;
         } else
            published = true ;


      } else
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;


      return( published ) ;


   } /* publishGraphics  */





   /*------------------------------------------------------------*/
   /* readGraphicFilenames
    */
   /**
    *    This method will read in all of the graphics
    *  that an object has. Please note that this only
    *  reads through the DRAFT database tables
    *
    *  @param indicators The comma separated list of Draft Object Indicators
    *  @param objectType The type of object to read
    *  @param None
    *
    *  @return Vector of strings. Each string is a graphic to be FTPed
    *//*---------------------------------------------------------*/
   private Vector readGraphicFilenames ( String indicators, int objectType ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readGraphicFilenames", 5 ) ;
      Vector    retVec    = new Vector ( 1, 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs  = null ;
            String    sql = null ;

            switch ( objectType ) {
               
               case AVALON_ACTION :
                  sql = "SELECT A.FILENAME "+
                        "FROM TIGRIS.GRAPHICS A, "+
                        "     OADRAFT.ACTION B, "+
                        "     OADRAFT.ACTIONINFOGRAPHIC C, "+
                        "     OADRAFT.ACTIONADDINFO D "+
                        "WHERE B.ACTIONIND IN ("+indicators+") AND "+
                        "      B.ACTIONIND        = D.ACTIONIND AND "+
                        "      C.ACTIONADDINFOIND = D.ACTIONADDINFOIND AND "+
                        "      A.GRAPHICIND       = C.GRAPHICIND "+
                        "FOR FETCH ONLY" ;
                  break ;

               case AVALON_QUESTION :
                  sql = "SELECT A.FILENAME "+
                        "FROM TIGRIS.GRAPHICS A, "+
                        "     OADRAFT.QUESTION B, "+
                        "     OADRAFT.QUESTIONINFOGRPHIC C, "+
                        "     OADRAFT.QUESTIONADDINFO D "+
                        "WHERE B.QUESTIONIND IN ("+indicators+") AND "+
                        "      D.QUESTIONIND        = B.QUESTIONIND AND "+
                        "      C.QUESTIONADDINFOIND = D.QUESTIONADDINFOIND AND "+
                        "      A.GRAPHICIND         = C.GRAPHICIND "+
                        "FOR FETCH ONLY" ;
                  break ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have a valid sql, continue                        */                       
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( sql != null ) {
               rs = stmt.executeQuery ( sql ) ;

               while ( rs.next() ) {
                  retVec.addElement ( rs.getString(1).trim() ) ;
               }

               rs.close() ;
            }


         } /* if */
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ,false ) ;
         retVec= null ;
      }
      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* ftpGraphics
    */
   /**
    *    This method will FTP all of the graphics down
    *  from the IT_OA_FTP_GRAP_DRFT site
    *  and publishes them
    *
    *  @param graphicFilenames The Vector of strings. Each string is a graphic's filename
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything went Ok, FALSE If not
    *//*---------------------------------------------------------*/
   private boolean ftpGraphics ( Vector graphicFilenames ) {
      boolean published = false ;

      published = pullGraphicsDown ( graphicFilenames ) ;

      if ( published )
         published = putGraphicsUp ( graphicFilenames ) ;

      return( published ) ;
   }





   /*------------------------------------------------------------*/
   /* pullGraphicsDown
    */
   /**
    *    This method will pull all draft graphics
    *  down from the draft ftp location
    *  
    *
    *  @param graphicFilenames A Vector containing Strings. Each String is a graphic filename
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything was pulled down
    *//*---------------------------------------------------------*/
   private boolean pullGraphicsDown ( Vector graphicFilenames ) {
      boolean  readAll         = false ;
      String   currentFilename = "" ;

      if ( ( graphicFilenames != null ) && ( graphicFilenames.size() > 0 ) ) {
         int i    = 0 ;
         int size = graphicFilenames.size() ;

         LogSystem.beginLogSection ( "FTPing graphics for Publish. FTP getting "+size+" Graphic(s)" ) ;

         try {
            Ftp ftp = new Ftp() ;
            String hostDir  = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_LOC_DRFT" ) ;
            String host     = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_DRFT" ) ;
            String localDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

            LogSystem.log ( 1, "Host:"+host ) ;
            LogSystem.log ( 1, "Host Directory:"+hostDir ) ;
            LogSystem.log ( 1, "Local Directory:"+localDir ) ;                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the properties of the FTP Bean and connect          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setHost ( host ) ;
            ftp.setUser ( FTP_USERID ) ;
            ftp.setPassword ( FTP_PASSWORD ) ;
            ftp.connect() ;
            ftp.login() ;
            LogSystem.log ( 1, "Connected to "+host ) ;
            ftp.setType ( FtpClient.IMAGE ) ;
            LogSystem.log ( 1, "Set Transfer type to IMAGE" ) ;          


            if ( ftp.isConnected() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the directories                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( hostDir ) ;
               LogSystem.log ( 1, "Entered remote directory:"+hostDir ) ;                       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through, grabbing each file                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {

                  currentFilename = (String)graphicFilenames.elementAt(i) ;

                  LogSystem.log ( 1, "Downloading file:"+currentFilename+":  To :"+localDir+File.separator+currentFilename+":" ) ;
                  publishPanel.setStatusText ( Str.getStr ( STR_RECEIVING_FILE )+": "+currentFilename ) ;
                  ftp.get ( currentFilename, localDir+File.separator+currentFilename ) ;

                  LogSystem.log ( 1, "Removing Temp File:"+currentFilename ) ;
                  publishPanel.setStatusText ( Str.getStr ( STR_REMOVING_TEMP_FILE )+": "+currentFilename ) ;
                  ftp.delete ( currentFilename ) ;
                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, close the connection and exit                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.disconnect() ;

            } /* if connected */

            readAll = true ;

         } catch ( UnknownHostException uhe ) {
            LogSystem.log ( 1, uhe, false ) ;
            GUISystem.printBox ( STR_INFO, STR_FTP_SITE_NOT_FOUND ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_NOT_ALL_GRAPHICS_DOWNLOADED )+" "+currentFilename ) ;
            readAll = false ;
         }

         LogSystem.endLogSection() ;
      } else
         readAll = true ; // Well, there were none to read!


      return( readAll ) ;
   }






   /*------------------------------------------------------------*/
   /* putGraphicsUp
    */
   /**
    *    This method will put all graphics passed up
    *  into the Published ftp directory
    *  
    *
    *  @param graphicFilenames A vector of Strings, each string represents a graphic
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean putGraphicsUp ( Vector graphicFilenames ) {
      boolean  putAll          = false ;
      String   currentFilename = "" ;
      int      recStatus       = 0 ;

      if ( ( graphicFilenames != null ) && ( graphicFilenames.size() > 0 ) ) {
         int i    = 0 ;
         int size = graphicFilenames.size() ;

         LogSystem.beginLogSection ( "FTP putting graphics for Publish. FTPing "+size+" Graphic(s)" ) ;

         try {
            Ftp ftp = new Ftp() ;
            String hostDir  = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_LOC_PUB" ) ;
            String host     = com.ibm.nzna.projects.common.quest.Constants.getStrConst ( "IT_OA_FTP_IMG_PUB" ) ;
            String localDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

            LogSystem.log ( 1, "Host:"+host ) ;
            LogSystem.log ( 1, "Host Directory:"+hostDir ) ;
            LogSystem.log ( 1, "Local Directory:"+localDir ) ;                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the properties of the FTP Bean and connect          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.setHost ( host ) ;
            ftp.setUser ( FTP_USERID ) ;
            ftp.setPassword ( FTP_PASSWORD ) ;
            ftp.connect() ;
            ftp.login() ;
            LogSystem.log ( 1, "Connected to "+host ) ;
            ftp.setType ( FtpClient.IMAGE ) ;
            LogSystem.log ( 1, "Set Transfer type to IMAGE" ) ;          


            if ( ftp.isConnected() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the directories                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( hostDir ) ;
               LogSystem.log ( 1, "Entered remote directory:"+hostDir ) ;                       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through, putting each file                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {

                  currentFilename = (String)graphicFilenames.elementAt(i) ;


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   If the status is anything except DELETE and NOINSERT,   */
                  /* then put the file up                                      */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  LogSystem.log ( 1, "Uploading from:"+localDir+File.separator+currentFilename+": To:"+hostDir+"/"+currentFilename ) ;
                  publishPanel.setStatusText ( Str.getStr ( STR_SENDING_FILE )+": "+currentFilename ) ;
                  ftp.put ( localDir+File.separator+currentFilename, currentFilename, false ) ;

                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, close the connection and exit                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               putAll = true ;
               ftp.disconnect() ;

            } /* if connected */


         } catch ( UnknownHostException uhe ) {
            LogSystem.log ( 1, uhe, false ) ;
            GUISystem.printBox ( STR_INFO, STR_FTP_SITE_NOT_FOUND ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_NOT_ALL_GRAPHICS_DOWNLOADED )+" "+currentFilename ) ;
            putAll = false ;
         }

         LogSystem.endLogSection() ;
      } else
         putAll = true ; // Well, there were none to put!


      return( putAll ) ;
   }

} /* OAObjectPublisher */
