package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   OAObjectValidator                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OAObjectValidator                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;     
import com.ibm.nzna.projects.qit.avalon.gui.* ;     
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.storedProc.oaValidate.* ;
/**
 * This object will validate any OAObject before publishing 
 *
 *
 * @version 1.0, 5/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OAObjectValidator                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class OAObjectValidator extends Object implements AppConst {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private PublishPanel  publishPanel   = null ;
   private Object        oaObject       = null ;
   private ValidationRec validationRec  = null ;
   private boolean       errorOddLine   = false ;
   private boolean       errorWarnLine  = false ;
   private boolean       error          = false ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will initialize the object
    *  but do nothing else. To perform the actual
    *  validation, please call 'validate' or 'getHTML'
    *
    *  @param publishPanel The PublishPanel to display output to
    *  @param oaObject The OAObject to verify
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OAObjectValidator ( PublishPanel publishPanel, Object oaObject ) {
      this.publishPanel = publishPanel ;
      this.oaObject     = oaObject ;
   }





   /*------------------------------------------------------------*/
   /* validate
    */
   /**
    *    This method will validate the object passed
    *  only
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void validate() {
      String statusText = Str.getStr ( STR_VALIDATING )+" "+getObjectName() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Call the correct validate for the object passed to us   */
      /*                                                           */
      /*-----------------------------------------------------------*/


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   QuestionDraft                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( oaObject instanceof QuestionDraft ) {
         publishPanel.setStatusText ( statusText+" ("+((QuestionDraft)oaObject).getInd()+")" ) ;
         validationRec = ((QuestionDraft)oaObject).validate() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ActionDraft                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( oaObject instanceof ActionDraft ) {
         publishPanel.setStatusText ( statusText+" ("+((ActionDraft)oaObject).getInd()+")" ) ;
         validationRec = ((ActionDraft)oaObject).validate() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   SymptomDraft                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( oaObject instanceof SymptomDraft ) {
         publishPanel.setStatusText ( statusText+" ("+((SymptomDraft)oaObject).getInd()+")" ) ;
         validationRec = ((SymptomDraft)oaObject).validate() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Quest Link                                              */ 
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( oaObject instanceof QuestLinkDraft ) {
         publishPanel.setStatusText ( statusText+" ("+((QuestLinkDraft)oaObject).getInd()+")" ) ;
         validationRec = ((QuestLinkDraft)oaObject).validate() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   External Link                                           */ 
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( oaObject instanceof ExternalLinkDraft ) {
         publishPanel.setStatusText ( statusText+" ("+((ExternalLinkDraft)oaObject).getInd()+")" ) ;
         validationRec = ((ExternalLinkDraft)oaObject).validate() ;
      } 
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Link Group                                              */ 
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( oaObject instanceof LinkGroupDraft ) {
         publishPanel.setStatusText ( statusText+" ("+((LinkGroupDraft)oaObject).getInd()+")" ) ;
         validationRec = ((LinkGroupDraft)oaObject).validate() ;
      }
      
      
      else
         GUISystem.printBox ( "Info", "OAObjectValidator does not know how to validate the object type passed!" ) ;

      


      publishPanel.setStatusText ( " " ) ;

   }






   /*------------------------------------------------------------*/
   /* getObjectName
    */
   /**
    *    This method will return the user readable
    *  name of the object being validated
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getObjectName () {
      String returnStr = "Unknown" ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   QuestionDraft                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( oaObject instanceof QuestionDraft )
         returnStr = Str.getStr ( STR_QUESTION ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ActionDraft                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( oaObject instanceof ActionDraft )
         returnStr = Str.getStr ( STR_ACTION ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   SymptomDraft                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( oaObject instanceof SymptomDraft )
         returnStr = Str.getStr ( STR_SYMPTOM ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Links                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( oaObject instanceof QuestLinkDraft ) ||
                ( oaObject instanceof ExternalLinkDraft ) )
         returnStr = Str.getStr ( STR_LINK ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Link Group                                              */ 
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( oaObject instanceof LinkGroupDraft )
         returnStr = Str.getStr ( STR_LINK_GROUP ) ;
      

      return( returnStr ) ;

   }



   /*------------------------------------------------------------*/
   /* getHTML
    */
   /**
    *    This method will validate the object passed
    *  and return an HTML representation of
    *  the validation
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getHTML () {    
      String returnHtml = null ;

      try {
         returnHtml = FileUtil.readFile ( "skins"+
                                          File.separator+
                                          PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                                          File.separator+
                                          "avalon"+
                                          File.separator+
                                          "ValidationPreview.html" ) ;             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Put the header text in                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnHtml = Text.replaceInStr ( returnHtml, "STR_VALIDATION",   Str.getStr ( STR_VALIDATION ) ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "STR_ERRORS",       Str.getStr ( STR_ERRORS ) ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "STR_WARNINGS",     Str.getStr ( STR_WARNINGS ) ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "STR_DEPENDENCIES", Str.getStr ( STR_DEPENDENCIES ) ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "STR_QUESTION",     Str.getStr ( STR_QUESTION ) ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "STR_SYMPTOM",      Str.getStr ( STR_SYMPTOM ) ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "STR_ACTION",       Str.getStr ( STR_ACTION ) ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "STR_OBJECT",       getObjectName() ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "STR_DEPENDENCIES", Str.getStr ( STR_DEPENDENCIES ) ) ;
         returnHtml = Text.replaceInStr ( returnHtml, "DATE",             DateSystem.prettyDateFromStamp ( DateSystem.getCurrentTimestamp() ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, validate                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         validate() ;                              

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now put our header on the HTML file                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( validationRec != null ) {
            int failureSize = (validationRec.failureVec      == null ? 0 : validationRec.failureVec.size() ) ;
            int usageSize   = (validationRec.usageVec        == null ? 0 : validationRec.usageVec.size() ) ;
            int coreqSize   = (validationRec.symptomCoreqVec == null ? 0 : validationRec.symptomCoreqVec.size() ) ;

            publishPanel.setStatusText ( Str.getStr ( STR_VALIDATING ) ) ;
            publishPanel.setMaximum ( failureSize+usageSize+coreqSize ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Do the errors                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnHtml = logErrors ( returnHtml ) ;      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Do the warnings                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnHtml = logWarnings ( returnHtml ) ;           


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the dependencies                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnHtml = getDeps ( returnHtml ) ;

         } 

         else {
            returnHtml = Text.replaceInStr ( returnHtml, "STR_VALIDATION:", Str.getStr ( STR_VALIDATION_NO_ERRORS ) ) ;
            returnHtml = Text.replaceInStr ( returnHtml, "WARNING_TABLE", Str.getStr ( STR_NONE ) ) ;
            returnHtml = Text.replaceInStr ( returnHtml, "ERROR_TABLE", Str.getStr ( STR_NONE ) ) ;
         }

      } catch ( Exception e ) {
         e.printStackTrace() ;
         returnHtml = "<html><body><font size=2 face=\"Helvetica\">"+Str.getStr ( STR_CANNOT_VALIDATE )+"</font></body></html>" ;
         error=true ;
      }
      publishPanel.setStatusText ( "" ) ;
      publishPanel.setValue ( 0 ) ;

      return( returnHtml ) ;
   }





   /*------------------------------------------------------------*/
   /* logErrors
    */
   /**
    *    This method will log all of the errors that
    *  the user should see from the failureVec
    *  that is in the ValidationRec
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String logErrors ( String returnHtml ) {

      if ( ( validationRec.failureVec != null ) && ( validationRec.failureVec.size() > 0 ) ) {
         int          i          = 0 ;
         int          size       = validationRec.failureVec.size() ;
         FailureRec   failRec    = null ;
         StringBuffer errorTable = new StringBuffer ( "<TABLE WIDTH=\"100%\">" ) ;

         while ( i < size ) {
            failRec = (FailureRec)validationRec.failureVec.elementAt(i) ;
            errorTable.append ( logErrorLine (  failRec ) ) ;
            i++ ;
            publishPanel.bumpProgress() ;
         }


         errorTable.append ( "</TABLE>" ) ;                             
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Note that we have encountered an error                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         error = true ;

         returnHtml = Text.replaceInStr ( returnHtml, "ERROR_TABLE", errorTable.toString() ) ;


      } else {
         returnHtml = Text.replaceInStr ( returnHtml, "ERROR_TABLE", Str.getStr ( STR_NONE ) ) ;
      }

      return( returnHtml) ;
   }





   /*------------------------------------------------------------*/
   /* logWarnings
    */
   /**
    *    This method will log all of the warnings that
    *  the user should see 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String logWarnings ( String returnHtml ) {

      if ( ( validationRec.usageVec != null ) && ( validationRec.usageVec.size() > 0 ) ) {
         int           i            = 0 ;
         int           size         = validationRec.usageVec.size() ;
         UsageRec      usageRec     = null ;
         StringBuffer  warningTable = new StringBuffer ( "<table width=\"100%\" border=0>" ) ;


         while ( i < size ) {
            usageRec = (UsageRec)validationRec.usageVec.elementAt(i) ;
            warningTable.append ( logWarningLine ( usageRec.objectInd, usageRec.usageType ) ) ;
            i++ ;
            publishPanel.bumpProgress() ;
         }

         warningTable.append ( "</TABLE>" ) ;

         returnHtml = Text.replaceInStr ( returnHtml, "WARNING_TABLE", warningTable.toString() );

      } else
         returnHtml = Text.replaceInStr ( returnHtml, "WARNING_TABLE", Str.getStr ( STR_NONE ) ) ;

      return( returnHtml ) ;
   }





   /*------------------------------------------------------------*/
   /* logErrorLine
    */
   /**
    *    This method will log 1 line in the error list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String logErrorLine ( FailureRec failureRec ) {
      String       bgColor    = "#FFFFFF" ;
      StringBuffer returnHtml = new StringBuffer ( "" ) ;

      if ( errorOddLine ) {
         errorOddLine = false ;
         bgColor = "#FFC0C0" ;
      } else
         errorOddLine = true ;                                               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the first row which contains the Id and error msg   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      returnHtml.append ( "<tr valign=\"top\" bgcolor=\"" ) ;
      returnHtml.append ( bgColor ) ;
      returnHtml.append ( "><td width=\"13%\">" ) ;
      returnHtml.append ( "<font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"#000000\">" ) ;
      returnHtml.append ( failureRec.objectInd+((failureRec.activeObject ? "" : "("+Str.getStr ( STR_DRAFT )+")")) ) ;
      returnHtml.append ( "</font></td><td width=\"84%\">" ) ;
      returnHtml.append ( "<font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"#000000\">" ) ;
      returnHtml.append ( getErrorString ( failureRec ) ) ;
      returnHtml.append ( "</font></td></tr>" ) ;                              


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the next line which holds the title                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      returnHtml.append ( "<tr bgcolor=\"" ) ;
      returnHtml.append ( bgColor ) ;
      returnHtml.append ( "><td width=\"13%\">" ) ;
      returnHtml.append ( "<font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"#000000\">" ) ;
      returnHtml.append ( "&nbsp;&nbsp" ) ;
      returnHtml.append ( "</font></td><td width=\"84%\">" ) ;
      returnHtml.append ( "<font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"#000000\">" ) ;
      returnHtml.append ( failureRec.title ) ;
      if ( failureRec.answerInd > 0 ) {
         returnHtml.append ( "<br><i>" ) ;
         returnHtml.append ( Str.getStr ( STR_ANSWER ) ) ;
         returnHtml.append ( ": " ) ;
         returnHtml.append ( failureRec.answerTitle ) ;
         returnHtml.append ( "</i>" ) ;
      }
      returnHtml.append ( "</font></td></tr>" ) ;



      return( returnHtml.toString() ) ;
   } /* logErrorLine */



   /*------------------------------------------------------------*/
   /* logWarningLine
    */
   /**
    *    This method will log 1 line in the warning list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String logWarningLine ( int indicator, int errorInd ) {
      String       bgColor    = "#FFFFFF" ;
      StringBuffer returnHtml = new StringBuffer ( "" ) ;

      if ( errorWarnLine ) {
         errorWarnLine = false ;
         bgColor = "#FFFFCO" ;
      } else
         errorWarnLine = true ;                                               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the first row which contains the Id and error msg   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      returnHtml.append ( "<tr valign=\"top\" bgcolor=\"" ) ;
      returnHtml.append ( bgColor ) ;
      returnHtml.append ( "><td width=\"13%\">" ) ;
      returnHtml.append ( "<font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"#000000\">" ) ;
      returnHtml.append ( indicator ) ;
      returnHtml.append ( "</font></td><td width=\"84%\">" ) ;
      returnHtml.append ( "<font face=\"Arial, Helvetica, sans-serif\" size=\"2\" color=\"#000000\">" ) ;
      returnHtml.append ( getWarningString ( errorInd ) ) ;
      returnHtml.append ( "</font></td></tr>" ) ;


      return( returnHtml.toString() ) ;
   } /* logErrorLine */






   /*------------------------------------------------------------*/
   /* getErrorString
    */
   /**
    *    This method will return the correct string
    *  for the error indicator given. The error indicator
    *  must be defined in com.ibm.nzna.projects.common.storedProc.oaValidate.FailureRec
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getErrorString ( FailureRec failureRec ) {
      String retStr = "Unknown error ind:"+failureRec.failureReason ;

      switch ( failureRec.failureReason ) {
         case FailureRec.QFAILURE_ACTION_INFO_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_ACTION_INFO_COND ) ;
            break ;

         case FailureRec.QFAILURE_ACTION_LINK_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_ACTION_LINK_COND ) ;
            break ;

         case FailureRec.QFAILURE_ACTION_TITLE_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_ACTION_TITLE_COND ) ;
            break ;

         case FailureRec.QFAILURE_ANSWER_CONCL_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_ANSWER_CONCL_COND ) ;
            break ;

         case FailureRec.QFAILURE_ANSWER_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_ANSWER_COND ) ;
            break ;

         case FailureRec.QFAILURE_ANSWER_TITLE_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_ANSWER_TITLE_COND ) ;
            break ;

         case FailureRec.QFAILURE_EDGE :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_ANSWER_EDGE ) ;
            break ;

         case FailureRec.QFAILURE_EDGE_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_EDGE_COND ) ;
            break ;

         case FailureRec.QFAILURE_EDGE_PREANSWER :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_EDGE_PREANSWER ) ;
            break ;

         case FailureRec.QFAILURE_NO_OPEN_DRAFT_SYMPTOM :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_NO_OPEN_DRAFT_SYMPTOM ) ;
            break ;

         case FailureRec.QFAILURE_QUESTION_INFO_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_QUESTION_INFO_COND ) ;
            break ;

         case FailureRec.QFAILURE_QUESTION_LINK_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_QUESTION_LINK_COND ) ;
            break ;

         case FailureRec.QFAILURE_QUESTION_TITLE_COND :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_QUESTION_TITLE_COND ) ;
            break ;
            
         case FailureRec.FAILURE_MAINTLOCK_SET :
            retStr = Str.getStr ( STR_PUBLISH_FAILURE_OBJECT_LOCKED_BY_USER ) ;
            retStr+=" "+UserSystem.getNameFromUserId ( failureRec.dbUser )+"." ;
            break ;

      }

      return( retStr ) ;
   }





   /*------------------------------------------------------------*/
   /* getWarningString
    */
   /**
    *    This method will return the warning string
    *  that is user readable
    *  
    *
    *  @param warningInd The warning indicator as defined in com.ibm.nzna.projects.common.storedProc.oaValidator.usageRec
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getWarningString ( int warningInd ) {
      String retStr = "Unkown Warning:"+warningInd ;

      switch ( warningInd ) {
         case UsageRec.USAGE_ACTION_DOCLINK :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_ACTION_DOCLINK ) ;
            break ;

         case UsageRec.USAGE_ACTION_INFO_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_ACTION_INFO_COND ) ;
            break ;

         case UsageRec.USAGE_ACTION_LINK_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_ACTION_LINK_COND ) ;
            break ;

         case UsageRec.USAGE_ACTION_TITLE_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_ACTION_TITLE_COND ) ;
            break ;

         case UsageRec.USAGE_ANSWER_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_ANSWER_CONCL_COND ) ;
            break ;

         case UsageRec.USAGE_ANSWER_TITLE_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_ANSWER_TITLE_COND ) ;
            break ;

         case UsageRec.USAGE_EDGE_ALIST :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_EDGE_ALIST ) ;
            break ;

         case UsageRec.USAGE_EDGE_PREANSWER :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_EDGE_PREANSWER ) ;
            break ;

         case UsageRec.USAGE_EDGE_UNANSWER :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_EDGE_UNANSWER ) ;
            break ;

         case UsageRec.USAGE_LINKGROUP_LINK :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_LINKGROUP_LINK ) ;
            break ;

         case UsageRec.USAGE_QUESTION_DOCLINK :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_QUESTION_DOCLINK ) ;
            break ;

         case UsageRec.USAGE_QUESTION_INFO_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_QUESTION_INFO_COND ) ;
            break ;

         case UsageRec.USAGE_QUESTION_LINK_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_QUESTION_LINK_COND ) ;
            break ;

         case UsageRec.USAGE_QUESTION_TITLE_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_QUESTION_TITLE_COND ) ;
            break ;

         case UsageRec.USAGE_SYMPTOM_NODE :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_SYMPTOM_NODE ) ;
            break ;

         case UsageRec.USAGE_ANSWER_CONCL_COND :
            retStr = Str.getStr ( STR_PUBLISH_WARNING_ANSWER_CONCL_COND ) ;
            break ;
      }     


      return( retStr ) ;
   }





   /*------------------------------------------------------------*/
   /* getError
    */
   /**
    *    This method will return TRUE if the validation
    *  that has taken place contained errors,
    *  FALSE if no errors
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getError () {
      return( error ) ;
   }




   /*------------------------------------------------------------*/
   /* getValidationRec
    */
   /**
    *    This method will return the current Validation
    *  Rec that this validator has recieved
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ValidationRec
    *//*---------------------------------------------------------*/
   public ValidationRec getValidationRec () {
      return( validationRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getDeps
    */
   /**
    *    This methos will return the dependencies
    *  that were found during validation
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getDeps ( String returnHtml ) {
      int questionCount = validationRec.questionCoreqVec.size( ) ;
      int actionCount   = validationRec.actionCoreqVec.size() ;
      int symptomCount  = validationRec.symptomCoreqVec.size() ;

      returnHtml = Text.replaceInStr ( returnHtml, "QUESTION_COREQ", ""+questionCount ) ;
      returnHtml = Text.replaceInStr ( returnHtml, "ACTION_COREQ",   ""+actionCount ) ;
      returnHtml = Text.replaceInStr ( returnHtml, "SYMPTOM_COREQ",  ""+symptomCount ) ;

      return( returnHtml ) ;
   }


} /* OAObjectValidator */
