package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   QuestionDisplayRec                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestionDisplayRec                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.Question ;
import com.ibm.nzna.projects.common.quest.type.TypeList ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
/**
 * This record simply shows a Question in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestionDisplayRec                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestionDisplayRec extends OAObjectDisplayRec implements IQuickSort, DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int      questionInd       = 0 ;
   private int      docClassInd       = 0 ;
   private String   title             = null ;
   private String   changedTime       = null ;
   private String   dbUser            = null ;
   private String   prettyChangedTime = null ;
   private String   prettyDBUser      = null ;
   private Question question          = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestionDisplayRec () {
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestionDisplayRec ( int questionInd, int docClassInd, String title, String changedTime, String dbUser ) {
      this.questionInd       = questionInd ;
      this.docClassInd       = docClassInd ;
      this.title             = title ;
      this.changedTime       = changedTime ;
      this.dbUser            = dbUser ;
   }

   public QuestionDisplayRec ( Question question ) {
      this.question          = question ;
      this.questionInd       = question.getInd() ;
      this.docClassInd       = question.getDocClass().getInd () ;
      this.title             = question.getTitles().elementAt(question.getTitles().size()-1).toString() ;
      this.changedTime       = question.getChangedTime () ;
      this.dbUser            = question.getDbUser() ;
   }


   /*------------------------------------------------------------*/
   /* getQuestionInd
    */
   /**
    *    This method will return the question indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getQuestionInd () {
      return( questionInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Classification                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            if ( docClassInd == TypeList.DOCCLASS_IBM_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ;
            else if ( docClassInd == TypeList.DOCCLASS_DEALER_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_DEALER_CONFIDENTIAL ) ;
            else
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_UNCLASSIFIED ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Nothing                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 1 :
            retObj = "" ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Question Indicator                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+questionInd ;
            break;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            if ( getTitleArray() == null )
               createTitleArray ( title ) ;
            retObj = getTitleArray() ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 5 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;

            retObj = prettyDBUser ;
            break ;

         default:
            retObj = "" ;
            break ;
      }


      return( retObj ) ;
   }


   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( question ) ;
   }


   public String toString () {
      return( title ) ;
   }


   /*------------------------------------------------------------*/
   /* convert
    */
   /**
    *    This method will convert an entire Vector of
    *  Question objects to QuestionDisplayRecs
    *  and will return a vector of the converted records
    *
    *  @param vec The vector to convert
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector convert ( Vector vec ) {
      int    size   = vec.size() ;
      Vector retVec = new Vector ( size ) ;
      int    i      = 0 ;

      while ( i < size ) {
         retVec.addElement ( new QuestionDisplayRec ( (Question) vec.elementAt(i) ) ) ;
         i++ ;
      }

      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* setQuestion
    */
   /**
    *    This method will set what question that this
    *  display represents
    *  
    *
    *  @param question The question tor represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setQuestion ( Question question ) {
      this.question = question ;
   }





   /*------------------------------------------------------------*/
   /* readQuestion
    */
   /**
    *    This method will read in the question object that
    *  is represented by this display record. It will 
    *  try to read from cache first, then the database. This method
    *  will not read anything if the question property is already
    *  read.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readQuestion () {
      if ( question == null )
         setQuestion ( AvalonCache.readQuestion ( getQuestionInd() ) ) ;
   }






   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will do the comparison between
    *  another QuestionDisplayRec and this one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( IQuickSort sortObject ) {
      int rc = 0 ;

      if ( sortObject instanceof QuestionDisplayRec ) {
         QuestionDisplayRec displayRec = (QuestionDisplayRec) sortObject ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Do the comparison based on what we have to sort         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( getSort() ) {
            case SORT_CHANGEDTIME_ASC :
               rc = CDate.compareDate ( changedTime, displayRec.changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_CHANGEDTIME_DESC :
               rc = CDate.compareDate ( displayRec.changedTime, changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_TITLE_ASC :
               rc = title.compareTo ( displayRec.title ) ;
               break ;

            case SORT_TITLE_DESC :
               rc = displayRec.title.compareTo ( title ) ;
               break ;

            case SORT_DBUSER_ASC :
               rc = dbUser.compareTo ( displayRec.dbUser ) ;
               break ;

            case SORT_DBUSER_DESC :
               rc = displayRec.dbUser.compareTo ( dbUser ) ;
               break ;

            case SORT_DOCCLASS :
               if ( docClassInd < displayRec.docClassInd )
                  rc = -1 ;
               else if ( docClassInd > displayRec.docClassInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_ASC :
               if ( questionInd < displayRec.questionInd )
                  rc = -1 ;
               else if ( questionInd > displayRec.questionInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_DESC :
               if ( questionInd < displayRec.questionInd )
                  rc = 1 ;
               else if ( questionInd > displayRec.questionInd )
                  rc = -1 ;
               else
                  rc = 0 ;
               break ;
         }

      }

      if ( rc > 0 )
         rc = 1 ;
      else if ( rc < 0 ) 
         rc = -1 ;

      return( rc ) ;
   }


} /* QuestionDisplayRec */
