package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   QuesitonPublisher                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuesitonPublisher                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.common.storedProc.oaValidate.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This thread publishes a Question by doing these steps:<br><br><pre>
 *
 * 1. Validate each dependency object passed in the ValidationRec<br>
 * 2. If any error occurs during validation, stop immediatly and return an HTML string containing the error<br>
 * 3. If all objects pass validation then publish each object. <br><br></pre>
 *
 * This thread interacts with the PublishPanel passed in by setting the status line and the progress
 * bar accordingly.
 *
 *
 * @version 1.0, 5/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuesitonPublisher                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestionPublisher extends Object implements Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int TYPE_QUESTION       = 0 ;
   private static final int TYPE_ACTION         = 1 ;
   private static final int TYPE_SYMPTOM        = 2 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private QuestionDraft question      = null ;
   private PublishPanel  publishPanel  = null ;
   private ValidationRec validationRec = null ;
   private UVector       questionVec   = new UVector ( 1, 1 ) ;
   private UVector       actionVec     = new UVector ( 1, 1 ) ;
   private UVector       symptomVec    = new UVector ( 1, 1 ) ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the Thread but
    *  will not start the thread. To start the thread
    *  please call 'start'
    *
    *  @param publishPanel The PublishPanel to interact with
    *  @param validationRec The validationRec to get dependencies with
    *  @param questionDraft The QuestionDraft to publish
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestionPublisher ( PublishPanel publishPanel, ValidationRec validationRec, QuestionDraft questionDraft ) {
      this.publishPanel  = publishPanel ;
      this.validationRec = validationRec ;
      this.question      = questionDraft ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will start the publishing process,
    *  keep in mind that this method really kicks off another
    *  thread to do the work so this method will immediatly return
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      new Thread ( this ).start() ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread method will publish the question
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, validate all dependencies                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( validate ( validationRec.actionCoreqVec, TYPE_ACTION ) ) {
         if ( validate ( validationRec.questionCoreqVec, TYPE_QUESTION ) ) {
            if ( validate ( validationRec.symptomCoreqVec, TYPE_SYMPTOM ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Ok, now we must publish each item                       */                        
               /*                                                           */
               /*-----------------------------------------------------------*/

               
            }
         }
      }
   }




   /*------------------------------------------------------------*/
   /* validate
    */
   /**
    *    This method will validate all objects in the
    *  vector passed. This method will return FALSE
    *  if any of the objects fail validation this method will stop and return FALSE.
    *  This method also builds the list of dependent objects that must be validated as
    *  well. Each object that is added is also validated here.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean validate ( Vector objects, int type ) {
      boolean rc = true ;

      if ( ( objects != null ) && ( objects.size() > 0 ) ) {
         int size = objects.size() ;
         int i    = 0 ;

         while ( ( rc ) && ( i < size ) ) {
            OAObjectValidator validator = null ;
            DraftObject       object    = null ;

            switch ( type ) {
               case TYPE_ACTION :
                  object = new ActionDraft ( ((Integer)objects.elementAt(i)).intValue() ) ;
                  break ;

               case TYPE_QUESTION :
                  object = new QuestionDraft ( ((Integer)objects.elementAt(i)).intValue() ) ;
                  break ;

               case TYPE_SYMPTOM :
                  object = new SymptomDraft ( ((Integer)objects.elementAt(i)).intValue() ) ;
                  break ;

            }

            new OAObjectValidator ( publishPanel, object ) ;

            validator.validate() ;
            rc =  !( validator.getError() ) ;
            i++ ;
         } /* while  */

      } /* if  */

      return( rc ) ;
   } /* validate  */

} /* QuesitonPublisher */
