package com.ibm.nzna.projects.qit.avalon.base ;     
/*-----------------------------------------------------------*/
/*   SoftValidator                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SoftValidator                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class holds static methods usefull in performing 'soft' validations 
 * on Titles, Additional Informations, Links, Link Groups etc..
 *
 * @version 1.0, 5/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SoftValidator                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class SoftValidator extends Object implements AppConst {




   /*------------------------------------------------------------*/
   /* validateTitles
    */
   /**
    *    This method will validate that all titles
    *  passed in will show on the web in some way
    *  
    *
    *  @param retBuffer The buffer to add the output to
    *  @param titles The vector of Title objects
    *  @param docClassInd The base doc class indicator to look for
    *
    *  @return void
    *//*---------------------------------------------------------*/
   public static void validateTitles ( StringBuffer retBuffer, Vector titles, int docClassInd ) {
      boolean foundError   = false ;

      retBuffer.append ( "<br><font size=2><b>" ) ;
      retBuffer.append ( Str.getStr ( STR_TITLE_VALIDATION ) ) ;
      retBuffer.append ( "</font></b><br>" ) ;

      if ( titles != null ) {
         int     i            = 0 ;
         int     a            = 0 ;
         int     titleSize    = titles.size() ;                                        
         Title   defaultTitle = (Title)titles.elementAt ( titleSize-1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, go through all of the titles and see if any      */
         /* have a document class that is higher than the docClassInd */
         /* passed in. This means that the title will never show      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < titleSize ) {
            if ( ( ((Title)titles.elementAt(i)).valid() ) &&
                 ( ((Title)titles.elementAt(i)).getDocClass().getInd() > docClassInd ) ) {
               retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#990000\"><tr><td><b><font size=2 face = \"Helvetica\" color=\"#FFFFFF\">&nbsp;&nbsp;" ) ;
               retBuffer.append ( Str.getStr ( STR_ERROR ) ) ;
               retBuffer.append ( "</font></b></td></tr></table><font face=\"Helvetica, Arial\" size=2>" ) ;
               retBuffer.append ( Str.getStr ( STR_TITLE_WILL_NEVER_SHOW ) ) ;
               retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;" ) ;
               retBuffer.append ( titles.elementAt(i).toString() ) ;
               retBuffer.append ( "<br><br>" ) ;
               foundError = true ;
            }
            i++ ;
         }
         i=0 ;                                       


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now go through each title and make sure that        */
         /* their order does not conflict with the doc class.         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < titleSize ) {
            docClassInd = ((Title)titles.elementAt(i)).getDocClass().getInd() ;
            a=i+1 ;
            while ( a < titleSize ) {
               if ( ( ((Title)titles.elementAt(a)).valid() ) &&
                    ( ((Title)titles.elementAt(a)).getDocClass().getInd() < docClassInd ) ) {
                  if ( titles.elementAt(a) != defaultTitle ) {
                     retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#FFFF00\"><tr><td><b><font size=2 face = \"Helvetica\" color=black>&nbsp;&nbsp;" ) ;
                     retBuffer.append ( Str.getStr ( STR_WARNING ) ) ;
                     retBuffer.append ( "</font></b></td></tr></table>" ) ;
                     retBuffer.append ( Str.getStr ( STR_TITLE_WILL_NEVER_SHOW_BECAUSE_OF_PREVIOUS_TITLE ) ) ;
                     retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( titles.elementAt(a).toString() ) ;
                     retBuffer.append ( "<br><b>&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( Str.getStr ( STR_TITLE_WILL_NEVER_SHOW_BECAUSE_OF_PREVIOUS_TITLE_2 ) ) ;
                     retBuffer.append ( "</b><br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( titles.elementAt(i).toString() ) ;
                     retBuffer.append ( "<br><br>" ) ;
                     foundError = true ;
                  } /* if  */
               } /* if  */
               a++ ;
            }/* while  */
            i++ ;
         } /* while  */
      }

      if ( !foundError ) {
         retBuffer.append ( "&nbsp;&nbsp;&nbsp;" ) ;
         retBuffer.append ( Str.getStr ( STR_NO_ERRORS_FOUND ) ) ;
         retBuffer.append ( "<br><br>" ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* valdiateAddtlInfos
    */
   /**
    *    This method will validate that all additional information
    *  passed in will show on the web in some way
    *  
    *
    *  @param retBuffer The buffer to add the output to
    *  @param titles The vector of Title objects
    *  @param docClassInd The base doc class indicator to look for
    *
    *  @return void
    *//*---------------------------------------------------------*/
   public static void validateAddtlInfos ( StringBuffer retBuffer, Vector addtlInfos, int docClassInd ) {
      boolean foundError   = false ;

      retBuffer.append ( "<br><font size=2><b>" ) ;
      retBuffer.append ( Str.getStr ( STR_ADD_INFO_VALIDATION ) ) ;
      retBuffer.append ( "</font></b><br>" ) ;

      if ( ( addtlInfos != null ) && ( addtlInfos.size() > 0 ) ) {
         int       i                = 0 ;
         int       a                = 0 ;
         int       titleSize        = addtlInfos.size() ;                                        
         AddtlInfo defaultAddtlInfo = (AddtlInfo)addtlInfos.elementAt ( titleSize-1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, go through all of the title and see if any       */
         /* have a document class that is higher than the docClassInd */
         /* passed in                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < titleSize ) {
            if ( ( addtlInfos.elementAt(i) != null ) &&
                 ( ((AddtlInfo)addtlInfos.elementAt(i)).valid() ) &&
                 ( ((AddtlInfo)addtlInfos.elementAt(i)).getDocClass().getInd() > docClassInd ) ) {
               retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#990000\"><tr><td><b><font size=2  face = \"Helvetica\" color=\"#FFFFFF\">&nbsp;&nbsp;" ) ;
               retBuffer.append ( Str.getStr ( STR_ERROR ) ) ;
               retBuffer.append ( "</font></b></td></tr></table>" ) ;
               retBuffer.append ( Str.getStr ( STR_ADD_INFO_WILL_NEVER_SHOW ) ) ;
               retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;" ) ;
               retBuffer.append ( addtlInfos.elementAt(i).toString() ) ;
               retBuffer.append ( "<br><br>" ) ;
               foundError = true ;
            }
            i++ ;
         }
         i=0 ;                                       


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now go through each title and make sure that        */
         /* their order does not conflict with the doc class.         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < titleSize ) {
            docClassInd = ((AddtlInfo)addtlInfos.elementAt(i)).getDocClass().getInd() ;
            a=i+1 ;
            while ( a < titleSize ) {
               if ( ( ((AddtlInfo)addtlInfos.elementAt(i)).valid() ) && 
                    ( ((AddtlInfo)addtlInfos.elementAt(a)).getDocClass().getInd() > docClassInd ) ) {
                  if ( addtlInfos.elementAt(a) != defaultAddtlInfo ) {
                     retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#FFFF00\"><tr><td><b><font size=2 face = \"Helvetica\" color=black>&nbsp;&nbsp;" ) ;
                     retBuffer.append ( Str.getStr ( STR_WARNING ) ) ;
                     retBuffer.append ( "</font></b></td></tr></table>" ) ;
                     retBuffer.append ( Str.getStr ( STR_ADD_INFO_WILL_NEVER_SHOW_BECAUSE_OF_PREVIOUS_TITLE ) ) ;
                     retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( addtlInfos.elementAt(a).toString() ) ;
                     retBuffer.append ( "<br><b>&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( Str.getStr ( STR_ADD_INFO_WILL_NEVER_SHOW_BECAUSE_OF_PREVIOUS_TITLE_2 ) ) ;
                     retBuffer.append ( "</b><br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( addtlInfos.elementAt(i).toString() ) ;
                     retBuffer.append ( "<br><br>" ) ;
                     foundError = true ;
                  } /* if  */
               } /* if  */
               a++ ;
            }/* while  */
            i++ ;
         } /* while  */
      }

      if ( !foundError ) {
         retBuffer.append ( "&nbsp;&nbsp;&nbsp;" ) ;
         retBuffer.append ( Str.getStr ( STR_NO_ERRORS_FOUND ) ) ;
         retBuffer.append ( "<br><br>" ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* validateSymptomTitles
    */
   /**
    *    This method will validate that all titles
    *  passed in will show on the web in some way
    *  
    *
    *  @param retBuffer The buffer to add the output to
    *  @param titles The vector of SymptomTitle objects
    *  @param docClassInd The base doc class indicator to look for
    *
    *  @return void
    *//*---------------------------------------------------------*/
   public static void validateSymptomTitles ( StringBuffer retBuffer, Vector titles, int docClassInd ) {
      boolean foundError   = false ;

      retBuffer.append ( "<br><font size=2><b>" ) ;
      retBuffer.append ( Str.getStr ( STR_TITLE_VALIDATION ) ) ;
      retBuffer.append ( "</font></b><br>" ) ;

      if ( titles != null ) {
         int          i                   = 0 ;
         int          a                   = 0 ;
         int          titleSize           = titles.size() ;                                        
         SymptomTitle defaultSymptomTitle = (SymptomTitle)titles.elementAt ( titleSize-1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, go through all of the titles and see if any      */
         /* have a document class that is higher than the docClassInd */
         /* passed in. This means that the title will never show      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < titleSize ) {
            if ( ( ((SymptomTitle)titles.elementAt(i)).valid() ) &&
                 ( ((SymptomTitle)titles.elementAt(i)).getDocClass().getInd() < docClassInd ) ) {
               retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#990000\"><tr><td><b><font size=2 face = \"Helvetica\" color=\"#FFFFFF\">&nbsp;&nbsp;" ) ;
               retBuffer.append ( Str.getStr ( STR_ERROR ) ) ;
               retBuffer.append ( "</font></b></td></tr></table>" ) ;
               retBuffer.append ( Str.getStr ( STR_TITLE_WILL_NEVER_SHOW ) ) ;
               retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;" ) ;
               retBuffer.append ( titles.elementAt(i).toString() ) ;
               retBuffer.append ( "<br><br>" ) ;
               foundError = true ;
            }
            i++ ;
         }
         i=0 ;                                       


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now go through each title and make sure that        */
         /* their order does not conflict with the doc class.         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < titleSize ) {
            docClassInd = ((SymptomTitle)titles.elementAt(i)).getDocClass().getInd() ;
            a=i+1 ;
            while ( a < titleSize ) {
               if ( ( ((SymptomTitle)titles.elementAt(i)).valid() ) &&
                    ( ((SymptomTitle)titles.elementAt(a)).getDocClass().getInd() < docClassInd ) ) {
                  if ( titles.elementAt(a) != defaultSymptomTitle ) {
                     retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#FFFF00\"><tr><td><b><font size=2 face = \"Helvetica\" color=black>&nbsp;&nbsp;" ) ;
                     retBuffer.append ( Str.getStr ( STR_WARNING ) ) ;
                     retBuffer.append ( "</font></b></td></tr></table>" ) ;
                     retBuffer.append ( Str.getStr ( STR_TITLE_WILL_NEVER_SHOW_BECAUSE_OF_PREVIOUS_TITLE ) ) ;
                     retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( titles.elementAt(a).toString() ) ;
                     retBuffer.append ( "<br><b>&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( Str.getStr ( STR_TITLE_WILL_NEVER_SHOW_BECAUSE_OF_PREVIOUS_TITLE_2 ) ) ;
                     retBuffer.append ( "</b><br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( titles.elementAt(i).toString() ) ;
                     retBuffer.append ( "<br><br>" ) ;
                     foundError = true ;
                  } /* if  */
               } /* if  */
               a++ ;
            }/* while  */
            i++ ;
         } /* while  */
      }

      if ( !foundError ) {
         retBuffer.append ( "&nbsp;&nbsp;&nbsp;" ) ;
         retBuffer.append ( Str.getStr ( STR_NO_ERRORS_FOUND ) ) ;
         retBuffer.append ( "<br><br>" ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* validateAuthority
    */
   /**
    *    This method will make sure that the user
    *  has the authority to publish this object
    *  
    *
    *  @param retBuffer The buffer to append to
    *  @param brand The brand of the object
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void validateAuthority ( StringBuffer retBuffer, TypeBrandRec brandRec, Vector countryList ) {
      boolean foundError   = false ;

      retBuffer.append ( "<br><font size=2><b>" ) ;
      retBuffer.append ( Str.getStr ( STR_AUTHORITY_VALIDATION ) ) ;
      retBuffer.append ( "</font></b><br>" ) ;

      if ( ( brandRec != null ) && ( !UserSystem.getLoggedUserRec().hasAvalonBrandAuth ( brandRec.getInd() ) ) ) {
         foundError = true ;

         retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#990000\"><tr><td><b><font size=2 face = \"Helvetica\" color=\"#FFFFFF\">&nbsp;&nbsp;" ) ;
         retBuffer.append ( Str.getStr ( STR_ERROR ) ) ;
         retBuffer.append ( "</font></b></td></tr></table>" ) ;
         retBuffer.append ( Str.getStr ( STR_NO_BRAND_AUTH_FOR_PUBLISH ) ) ;
         retBuffer.append ( "<br><br>" ) ;
      }


      if ( ( countryList != null ) && ( !UserSystem.hasCountries ( countryList ) ) ) {
         foundError = true ;

         retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#990000\"><tr><td><b><font size=2 face = \"Helvetica\" color=\"#FFFFFF\">&nbsp;&nbsp;" ) ;
         retBuffer.append ( Str.getStr ( STR_ERROR ) ) ;
         retBuffer.append ( "</font></b></td></tr></table>" ) ;
         retBuffer.append ( Str.getStr ( STR_NO_COUNTRY_AUTHORITY_TO_PUBLISH ) ) ;
         retBuffer.append ( "<br><br>" ) ;
      }

      if ( !foundError ) {
         retBuffer.append ( "&nbsp;&nbsp;&nbsp;" ) ;
         retBuffer.append ( Str.getStr ( STR_NO_ERRORS_FOUND ) ) ;
         retBuffer.append ( "<br><br>" ) ;
      }


   }

} /* SoftValidator */

