package com.ibm.nzna.projects.qit.avalon.base ;              
/*-----------------------------------------------------------*/
/*   SymptomDraftDisplayRec                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomDraftDisplayRec              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
/**
 * This record simply shows a Symptom Draft in a List, or preview. It is not
 * meant to be edited or used in any other way
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomDraftDisplayRec                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomDraftDisplayRec extends OAObjectDisplayRec implements IQuickSort, OASort, DisplayRec, MultiListRow, java.io.Serializable, AvalonConst, AppConst {                                              

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000001 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int     symptomInd        = 0 ;
   private int     docClassInd       = 0 ;
   private int     statusInd         = 0 ;
   private int     activeSymptomInd  = 0 ;
   private String  title             = null ;
   private String  changedTime       = null ;
   private String  dbUser            = null ;
   private String  prettyChangedTime = null ;
   private String  prettyDBUser      = null ;
   private String  startDate         = null ;
   private String  stopDate          = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor only creates the object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomDraftDisplayRec () {
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our object to the
    *  variables passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomDraftDisplayRec ( int symptomInd, int activeSymptomInd, int statusInd, int docClassInd, String title, String changedTime, String dbUser, String startDate, String stopDate ) {
      this.symptomInd        = symptomInd ;
      this.docClassInd       = docClassInd ;
      this.statusInd         = statusInd ;
      this.title             = title ;
      this.changedTime       = changedTime ;
      this.dbUser            = dbUser ;
      this.prettyChangedTime = prettyChangedTime ;
      this.prettyDBUser      = prettyDBUser ;
      this.startDate         = startDate ;
      this.stopDate          = stopDate ;
      this.activeSymptomInd  = activeSymptomInd ;
   }


   /*------------------------------------------------------------*/
   /* setSymptomInd
    */
   /**
    *    This method will set our symptom indicator
    *  
    *  
    *
    *  @param symptomInd The Sympom Indicator to represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSymptomInd ( int symptomInd ) {
      this.symptomInd = symptomInd ;
   }




   /*------------------------------------------------------------*/
   /* getSymptomInd
    */
   /**
    *    This method will return the symptom indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getSymptomInd () {
      return( symptomInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the status of this
    *  record
    *  
    *
    *  @param statusInd The status indicator as defined in TIGRIS.TYPESTATUS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatus ( int statusInd ) {
      this.statusInd = statusInd ;
   }




   /*------------------------------------------------------------*/
   /* getStatus
    */
   /**
    *    This method will return the status indicator
    *  of this record
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getStatus () {
      return( statusInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  needed to display.
    *  
    *
    *  @return column The column to return data for
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Classification                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case 0 :
            if ( docClassInd == TypeList.DOCCLASS_IBM_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ;
            else if ( docClassInd == TypeList.DOCCLASS_DEALER_CONFIDENTIAL )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_DEALER_CONFIDENTIAL ) ;
            else
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_UNCLASSIFIED ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Status                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 1 :
            if ( statusInd == STATUS_FLAWED )
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_CONFIDENTIAL ) ;
            else
               retObj = ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.IBM_UNCLASSIFIED ) ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Symptom Indicator                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 2 :
            retObj = ""+symptomInd ;
            break;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Title                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 3 :
            if ( getTitleArray() == null )
               createTitleArray ( title ) ;
            retObj = getTitleArray() ;
            break ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Changed Time                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 4 :
            if ( prettyChangedTime == null )
               prettyChangedTime = DateSystem.prettyDateFromStamp ( changedTime ) ;

            retObj = prettyChangedTime ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   User Id                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case 5 :
            if ( prettyDBUser == null )
               prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;

            retObj = prettyDBUser ;
            break ;

         case 6 :
            retObj = startDate ;
            break ;

         case 7 :
            if ( stopDate != null )
               retObj = stopDate ;
            else
               retObj = "" ;
            break ;

         default:
            retObj = "" ;
            break ;

      }


      return( retObj ) ;
   }





   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that this
    *  object is representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* getActiveSymptomInd
    */
   /**
    *    This method will return the Draft Symptom's
    *  active indicator
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getActiveSymptomInd () {
      return( activeSymptomInd ) ;
   }






   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will do the comparison between
    *  another SymptomDisplayRec and this one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int compareTo ( IQuickSort sortObject ) {
      int rc = 0 ;

      if ( sortObject instanceof SymptomDraftDisplayRec ) {
         SymptomDraftDisplayRec displayRec = (SymptomDraftDisplayRec) sortObject ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Do the comparison based on what we have to sort         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( getSort() ) {
            case SORT_CHANGEDTIME_ASC :
               rc = CDate.compareDate ( changedTime, displayRec.changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_CHANGEDTIME_DESC :
               rc = CDate.compareDate ( displayRec.changedTime, changedTime, CDate.DATE_STAMP ) ;
               break ;

            case SORT_TITLE_ASC :
               rc = title.compareTo ( displayRec.title ) ;
               break ;

            case SORT_TITLE_DESC :
               rc = displayRec.title.compareTo ( title ) ;
               break ;

            case SORT_DBUSER_ASC :
               rc = dbUser.compareTo ( displayRec.dbUser ) ;
               break ;

            case SORT_DBUSER_DESC :
               rc = displayRec.dbUser.compareTo ( dbUser ) ;
               break ;

            case SORT_DOCCLASS :
               if ( docClassInd < displayRec.docClassInd )
                  rc = -1 ;
               else if ( docClassInd > displayRec.docClassInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_ASC :
               if ( symptomInd < displayRec.symptomInd )
                  rc = -1 ;
               else if ( symptomInd > displayRec.symptomInd )
                  rc = 1 ;
               else
                  rc = 0 ;
               break ;

            case SORT_INDICATOR_DESC :
               if ( symptomInd < displayRec.symptomInd )
                  rc = 1 ;
               else if ( symptomInd > displayRec.symptomInd )
                  rc = -1 ;
               else
                  rc = 0 ;
               break ;
         }

      }

      if ( rc > 0 )
         rc = 1 ;
      else if ( rc < 0 ) 
         rc = -1 ;

      return( rc ) ;
   }


} /* SymptomDraftDisplayRec */
