package com.ibm.nzna.projects.qit.avalon.base ;       
/*-----------------------------------------------------------*/
/*   SymptomPreviewRec                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomPreviewRec                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This record holds all of the information used to display a Preview of a Symptom. Nothing more 
 *
 *
 * @version 1.0, 4/24/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomPreviewRec                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomPreviewRec extends Object implements AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Preview Template                                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String previewTemplate= "<html><body bgcolor=\"#FFFFFF\">"+
                                          "<p><font size=\"5\" face=\"Arial, Helvetica, sans-serif\">TITLE</font></p>"+
                                          "<p><font size=\"2\" face=\"Arial, Helvetica, sans-serif\">"+
                                          "TITLE_TABLE"+
                                          "  <font color=\"#800080\">STR_GEOS</font>:<br>GEOS<br><br>"+
                                          "  <font color=\"#800080\">STR_INTERNAL_NUM</font>:INTERNAL_NUM<br>"+
                                          "  <font color=\"#800080\">STR_ARCHIVED</font>:ARCHIVED<br>"+
                                          "  <font color=\"#800080\">STR_CREATE_DATE</font>:CREATEDATE<br>"+
                                          "  <font color=\"#800080\">STR_LAST_TOUCHED_ON</font>:LAST_TOUCHED_ON<br>"+
                                          "  <font color=\"#800080\">STR_LAST_TOUCHED_BY</font>:LAST_TOUCHED_BY<br>"+
                                          "  <font color=\"#800080\">STR_START_DATE</font>:START_DATE</font><br>"+
                                          "  <font color=\"#800080\">STR_STOP_DATE</font>:STOP_DATE</font><br></font>"+
                                          "</p>"+
                                          "</body>"+
                                          "</html>" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Members                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int      symptomInd         = 0 ;
   private String   title             = null ;
   private String   prettyCreateDate  = null ;
   private String   prettyChangedTime = null ;
   private String   prettyDbUser      = null ;
   private boolean  dataValid         = false ;
   private String   geos              = null ;
   private String   archived          = null ;
   private String   titleTable        = null ;
   private String   stopDate          = null ;
   private String   startDate         = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor currently for the
    *  SymptomPreviewRec. It will take a SymptomDisplayRec and
    *  read all preview information from the database from the symptomInd
    *  that the SymptomDisplayRec represents.
    *  
    *
    *  @param SymptomDisplayRec The SymptomDisplayRec to copy information from
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomPreviewRec ( SymptomDisplayRec displayRec ) {
      symptomInd = displayRec.getSymptomInd() ;
      readPreviewInfo() ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor currently for the
    *  SymptomPreviewRec. It will take a SymptomDisplayRec and
    *  read all preview information from the database from the symptomInd
    *  that the SymptomDisplayRec represents.
    *  
    *
    *  @param SymptomDisplayRec The SymptomDisplayRec to copy information from
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomPreviewRec ( SymptomDraftDisplayRec displayRec ) {
      symptomInd = displayRec.getSymptomInd() ;
      readDraftPreviewInfo() ;
   }






   /*------------------------------------------------------------*/
   /* getPreviewHTMLSource
    */
   /**
    *    This method will return the source code for
    *  the HTML needed to display this record
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getPreviewHTMLSource () {
      if ( dataValid ) {
         String retStr = Text.replaceInStr ( previewTemplate, "TITLE", title ) ;                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, replace all the 'Names'                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retStr = Text.replaceInStr ( retStr, "STR_INTERNAL_NUM",    Str.getStr ( STR_INTERNAL_NUM ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_CREATE_DATE",     Str.getStr ( STR_CREATION_DATE ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_LAST_TOUCHED_ON", Str.getStr ( STR_LAST_TOUCHED_ON ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_LAST_TOUCHED_BY", Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_GEOS",            Str.getStr ( STR_GEOGRAPHY ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_ARCHIVED",        Str.getStr ( STR_ARCHIVED ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_STOP_DATE",       Str.getStr ( STR_STOP_DATE ) ) ;
         retStr = Text.replaceInStr ( retStr, "STR_START_DATE",      Str.getStr ( STR_START_DATE ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, replace all of the 'Values'                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retStr = Text.replaceInStr ( retStr, "CREATEDATE",          prettyCreateDate ) ;
         retStr = Text.replaceInStr ( retStr, "LAST_TOUCHED_BY",     prettyDbUser ) ;
         retStr = Text.replaceInStr ( retStr, "LAST_TOUCHED_ON",     prettyChangedTime ) ;
         retStr = Text.replaceInStr ( retStr, "INTERNAL_NUM",        ""+symptomInd ) ;
         retStr = Text.replaceInStr ( retStr, "GEOS",                geos ) ;
         retStr = Text.replaceInStr ( retStr, "ARCHIVED",            archived ) ;
         retStr = Text.replaceInStr ( retStr, "STOP_DATE",           stopDate ) ;
         retStr = Text.replaceInStr ( retStr, "START_DATE",          startDate ) ;
         retStr = Text.replaceInStr ( retStr, "TITLE_TABLE",         titleTable ) ;


         return( retStr ) ;
      } else
         return( "<html><body><center>"+Str.getStr ( STR_SYMPTOM_NOT_FOUND )+"</center><br><br><b>SymptomInd:</b>"+symptomInd+"</body></html>" );
   }






   /*------------------------------------------------------------*/
   /* readPreviewInfo
    */
   /**
    *    This method will read in all of our preview
    *  information
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readPreviewInfo () {
      AppDefaultWin parentWin = MainWindow.getInstance() ;

      try {
         Symptom      symptom  = null ;                  


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the Symptom object                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parentWin.setStatus ( Str.getStr ( STR_READING ) ) ;
         symptom = AvalonCache.readSymptom ( symptomInd, false ) ;          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Populate our values                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( symptom != null ) {
            Vector titles    = symptom.getSymptomTitles() ;    
            Vector countries = symptom.getCountryList() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Title                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( titles != null ) && ( titles.size() > 0 ) )
               title = titles.elementAt(titles.size()-1).toString() ;
            else
               title = Str.getStr ( STR_UNKNOWN ) ;             


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Geographies                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( countries != null ) && ( countries.size() > 0 ) ) {
               int i    = 0 ;
               int size = countries.size() ;

               geos="" ;
               while ( i < size ) {
                  geos+=countries.elementAt(i).toString() ;
                  if ( i < (size-1) )
                     geos+=", " ;
                  i++ ;
               }

            } else
               geos = " " ;



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Dates and User                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            startDate         = " " ;
            stopDate          = " " ;
            prettyCreateDate  = symptom.getCreationTime() ;
            prettyChangedTime = DateSystem.prettyDateFromStamp ( symptom.getChangedTime() ) ;
            prettyDbUser      = UserSystem.getNameFromUserId ( symptom.getDbUser() ) ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Format the titles                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            formatTitleTable ( symptom ) ;


         }

         dataValid = true ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         dataValid = false ;
      }

      parentWin.setStatus ( null ) ;
   }




   /*------------------------------------------------------------*/
   /* readDraftPreviewInfo
    */
   /**
    *    This method will read in all of our preview
    *  information for a draft
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readDraftPreviewInfo () {      AppDefaultWin parentWin = MainWindow.getInstance() ;

      try {
         SymptomDraft symptom  = null ;                  


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the Symptom object                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parentWin.setStatus ( Str.getStr ( STR_READING ) ) ;
         symptom = AvalonCache.readSymptomDraft ( symptomInd, false ) ;          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Populate our values                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( symptom != null ) {
            Vector titles    = symptom.getSymptomTitles() ;    
            Vector countries = symptom.getCountryList() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Title                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( titles != null ) && ( titles.size() > 0 ) )
               title = titles.elementAt(titles.size()-1).toString() ;
            else
               title = Str.getStr ( STR_UNKNOWN ) ;             


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Geographies                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( countries != null ) && ( countries.size() > 0 ) ) {
               int i    = 0 ;
               int size = countries.size() ;

               geos="" ;
               while ( i < size ) {
                  geos+=countries.elementAt(i).toString() ;
                  if ( i < (size-1) )
                     geos+=", " ;
                  i++ ;
               }

            } else
               geos = " " ;



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Dates and User                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            startDate         = DateSystem.prettyDateFromDBDate ( symptom.getStartDate() ) ;
            stopDate          = DateSystem.prettyDateFromDBDate ( symptom.getStopDate() ) ;
            prettyCreateDate  = symptom.getCreationTime() ;
            prettyChangedTime = DateSystem.prettyDateFromStamp ( symptom.getChangedTime() ) ;
            prettyDbUser      = UserSystem.getNameFromUserId ( symptom.getDbUser() ) ;

            if ( stopDate == null )
               stopDate = "" ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Format the titles                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            formatTitleTable ( (Symptom)symptom ) ;


         }

         dataValid = true ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         dataValid = false ;
      }

      parentWin.setStatus ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* formatTitleTable
    */
   /**
    *    This method format the titleTable property
    *  given the Symptom passed in. 
    *  
    *
    *  @param symptom The Symptom to format the Title Table fo
    *  @param 
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void formatTitleTable ( Symptom symptom ) {
      if ( symptom != null ) {
         Vector titles = symptom.getSymptomTitles() ;

         if ( ( titles != null ) && ( titles.size() > 0 ) ) {
            int           i    = 0 ;
            int           size = titles.size() ;
            StringBuffer  table = new StringBuffer ( "</font><TABLE BORDER=0>" ) ;
            SymptomTitle  title = null ;

            while ( i < size ) {

               title = (SymptomTitle) titles.elementAt(i) ;
               table.append ( "<tr>" ) ;                      

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get the Classification Image in                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               table.append ( "<td><img src=\"file:/" ) ;
               table.append ( PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ) ;
               table.append ( File.separator ) ;

               switch ( ((TypeDocClassRec)title.getDocClass()).getInd() ) {
                  case TypeList.DOCCLASS_IBM_CONFIDENTIAL :
                     table.append ( "ibm_confidential.gif" ) ;
                     break ;

                  case TypeList.DOCCLASS_DEALER_CONFIDENTIAL :
                     table.append ( "ibm_dealer_confidential.gif" ) ;
                     break ;

                  case TypeList.DOCCLASS_UNCLASSIFIED :
                     table.append ( "unclassified.gif" ) ;
                     break;
               }
               table.append ( "\"></td>" );       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Put the Title in                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               table.append ( "<td><font size=\"2\" face=\"Arial, Helvetica, sans-serif\">" ) ;
               table.append ( title.toString() ) ;
               table.append ( "</td>" ) ;  

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   End the row                                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               table.append ( "</tr>" ) ;

               i++ ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   End the table                                           */    
            /*                                                           */
            /*-----------------------------------------------------------*/
            table.append ( "</table><br><font size=\"2\" face=\"Arial, Helvetica, sans-serif\">" ) ;


            titleTable = table.toString() ;

         } else
            titleTable = " " ;

      } else {
         titleTable=" " ;
      }
   }

} /* SymptomPreviewRec */
