package com.ibm.nzna.projects.qit.avalon.base ;                          
/*-----------------------------------------------------------*/
/*   SymptomSoftValidator                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomSoftValidator                */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.*;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.editors.* ;
/**
 * This class holds the methods needed to do a 'soft' validation of a symptom. 
 *
 *
 * @version 1.0, 5/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomSoftValidator                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomSoftValidator extends SoftValidator implements AppConst {




   /*------------------------------------------------------------*/
   /* validateSymptom
    */
   /**
    *    This method will validate a symptom and
    *  will return the string that needs to be displayed
    *  to the user in a Browser control
    *
    *  @param symptom The symptom to validate
    *  @param objectNodes T
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String validateSymptom ( Symptom symptom, Vector objectNodes ) {
      StringBuffer buffer = new StringBuffer ( "<html><head><font face=\"Helvetica\">" ) ;

      validateAuthority ( buffer,
                          symptom.getBrand(),
                          symptom.getCountryList() ) ;

      validateSymptomTitles ( buffer, 
                              symptom.getSymptomTitles(), 
                              symptom.getDocClass().getInd() ) ;

      validateNodes ( buffer,
                      symptom ) ;
      validateEdges ( buffer,
                      objectNodes,
                      symptom ) ;
      buffer.append ( "</font></head></html>" ) ;
      return( buffer.toString() ) ;
   }




   /*------------------------------------------------------------*/
   /* validateNodes
    */
   /**
    *    This method will validate the nodes in a symptom.
    *  It will just make sure that every node has a leading
    *  edgeGroup (Except for the first node of course)
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean validateNodes ( StringBuffer retBuffer, Symptom symptom ) {
      Vector  nodeVec  = symptom.getNodes() ;
      boolean hadError = false ; 

      retBuffer.append ( "<br><font size=2><b>" ) ;
      retBuffer.append ( Str.getStr ( STR_NODE_VALIDATION ) ) ;
      retBuffer.append ( "</font></b><br>" ) ;

      if ( ( nodeVec != null ) && ( nodeVec.size() > 0 ) ) {
         int  i    = 0 ;
         int  size = nodeVec.size() ;
         Node node = null ;

         while ( i < size ) {
            node = (Node) nodeVec.elementAt( i ) ;                                   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   CHeck to see if this node has a leading edge            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( node.valid() ) && ( !nodeHasLeadingEdge ( node, symptom ) ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   This node does not have a leading edge, make sure       */
               /* it is not the root node                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( symptom.getRootNode() != node ) {
                  retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#990000\"><tr><td><b><font size = \"2\" face = \"Helvetica\" color=\"#FFFFFF\">" ) ;
                  retBuffer.append ( Str.getStr ( STR_ERROR ) ) ;
                  retBuffer.append ( "</font></b></td></tr></table>" ) ;
                  retBuffer.append ( Str.getStr ( STR_NODE_NO_LEADING_EDGE ) ) ;
                  retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;" ) ;
                  retBuffer.append ( node.getInd() ) ;
                  retBuffer.append ( "<br><br>" ) ;
                  hadError = true ;
               }
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Check to see if this node is cyclic                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( ( node.valid() ) && ( nodeIsCyclic ( node, symptom ) ) ) {
               retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#990000\"><tr><td><b><font size = \"2\" face = \"Helvetica\" color=\"#FFFFFF\">" ) ;
               retBuffer.append ( Str.getStr ( STR_ERROR ) ) ;
               retBuffer.append ( "</font></b></td></tr></table>" ) ;
               retBuffer.append ( Str.getStr ( STR_NODE_IS_CYCLIC ) ) ;
               retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;" ) ;
               retBuffer.append ( node.getInd() ) ;
               retBuffer.append ( "<br><br>" ) ;
               hadError = true ;
            }

            i++ ;
         } /* while  */

      } /* if  */

      if ( !hadError )
         retBuffer.append ( Str.getStr ( STR_NO_ERRORS_FOUND ) ) ;
      retBuffer.append ( "<br><br>" );

      return( hadError ) ;
   }





   /*------------------------------------------------------------*/
   /* nodeHasLeadingEdge
    */
   /**
    *    This method will determine if the node
    *  passed in has at least 1 edge leading
    *  to it
    *
    *  @param node The node to check for
    *  @param symptom The symptom that the node belongs to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean nodeHasLeadingEdge ( Node node, Symptom symptom ) {
      Vector  edgeGroups = symptom.getEdgeGroups() ;
      boolean found      = false ;

      if ( ( edgeGroups != null ) && ( edgeGroups.size() > 0 ) ) {
         int       i         = 0 ;
         int       size      = edgeGroups.size() ;
         EdgeGroup edgeGroup = null ;

         while ( ( !found ) && ( i < size ) ) {
            edgeGroup = (EdgeGroup)edgeGroups.elementAt(i) ;
            found = ( edgeGroup.getTailNode() == node ) ;
            i++ ;
         }
      }

      return( found ) ;
   }



   /*------------------------------------------------------------*/
   /* nodeIsCyclic
    */
   /**
    *    This method will determine if the node
    *  passed in has an edge going to the Node itself
    *
    *  @param node The node to check for
    *  @param symptom The symptom that the node belongs to
    *  @param None
    *
    *  @return TRUE if the node does
    *//*---------------------------------------------------------*/
   private static boolean nodeIsCyclic ( Node node, Symptom symptom ) {
      Vector  edgeGroups = getEdgeGroups ( node, symptom ) ;
      boolean found      = false ;

      if ( ( edgeGroups != null ) && ( edgeGroups.size() > 0 ) ) {
         int       i         = 0 ;
         int       size      = edgeGroups.size() ;
         EdgeGroup edgeGroup = null ;

         while ( ( !found ) && ( i < size ) ) {
            edgeGroup = (EdgeGroup)edgeGroups.elementAt(i) ;
            found = ( edgeGroup.getTailNode() == node ) ;
            i++ ;
         }
      }

      return( found ) ;
   }





   /*------------------------------------------------------------*/
   /* validateEdges
    */
   /**
    *    This method will make sure that each Question's
    *  answer has an edge, or #OTHER
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean validateEdges ( StringBuffer retBuffer, Vector objectNodeVec, Symptom symptom ) {
      boolean hadError = false ;

      retBuffer.append ( "<br><font size=3><b>" ) ;
      retBuffer.append ( Str.getStr ( STR_EDGE_VALIDATION ) ) ;
      retBuffer.append ( "</font></b><br>" ) ;                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Loop through and process all nodes                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( objectNodeVec != null ) && ( objectNodeVec.size() > 0 ) ) {
         int        i       = 0 ;
         int        size    = objectNodeVec.size() ;
         ObjectNode node    = null ;
         Vector     answers = null ;
         Vector     edges   = null ;

         while ( i < size ) {
            node = (ObjectNode) objectNodeVec.elementAt( i ) ;                                                                 

            if ( node.getNode().valid() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   First, get the answers for the question if this is a    */
               /* question                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               answers = null ;
               if ( ( node.getDraftObject() != null ) && 
                    ( node.getDraftObject() instanceof QuestionDraft ) )
                  answers = ((QuestionDraft)node.getDraftObject()).getAnswers() ;

               if ( ( node.getObject() != null ) &&
                    ( node.getObject() instanceof Question ) )
                  answers = ((Question)node.getObject()).getAnswers() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, get the edges from the node                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               edges = getEdges ( node, symptom ) ;                       

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If there are answers, check them                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( answers != null ) {                          

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   If the answer has no edges, report                      */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( !answersHaveEdges ( answers, edges ) ) {
                     retBuffer.append ( "<table width=\"100\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#990000\"><tr><td><b><font face = \"Helvetica\" color=\"#FFFFFF\">" ) ;
                     retBuffer.append ( Str.getStr ( STR_ERROR ) ) ;
                     retBuffer.append ( "</font></b></td></tr></table>" ) ;
                     retBuffer.append ( Str.getStr ( STR_QUESTION_HAS_ANSWER_NO_EDGE ) ) ;
                     retBuffer.append ( "<br>&nbsp;&nbsp;&nbsp;" ) ;
                     retBuffer.append ( node.getNode().getInd() ) ;
                     retBuffer.append ( "<br><br>" ) ;
                     hadError = true ;
                  }
               }
            }
            i++ ;
         }
      }

      if ( !hadError )
         retBuffer.append ( Str.getStr ( STR_NO_ERRORS_FOUND ) ) ;
      retBuffer.append ( "<br><br>" ) ;
      return( hadError ) ;
   }







   /*------------------------------------------------------------*/
   /* getEdges
    */
   /**
    *    This method will return a Vector of all of the edges for
    *  a particular ObjectNode.
    *  
    *
    *  @param objectNode The ObjectNode to get Edges for
    *  @param symptom The Symptom that the node belongs to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static Vector getEdges ( ObjectNode objectNode, Symptom symptom ) {
      return ( getEdges ( objectNode.getNode(), symptom ) ) ;
   }


   /*------------------------------------------------------------*/
   /* getEdges
    */
   /**
    *    This method will return a Vector of all of the edges for
    *  a particular Node.
    *  
    *
    *  @param node The Node to get Edges for
    *  @param symptom The Symptom that the node belongs to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static Vector getEdges ( Node node, Symptom symptom ) {
      Vector edgeGroups = symptom.getEdgeGroups() ;
      Vector edges      = new Vector ( 1 ) ;

      if ( ( edgeGroups != null ) && ( edgeGroups.size() > 0 ) ) {
         int       i         = 0 ;
         int       size      = edgeGroups.size() ;
         EdgeGroup edgeGroup = null ;

         while ( i < size ) {
            edgeGroup = (EdgeGroup) edgeGroups.elementAt(i) ;

            if ( ( edgeGroup.valid() ) && ( edgeGroup.getHeadNode() == node ) ) {
               Vector newEdges = edgeGroup.getEdges() ;

               if ( ( newEdges != null ) && ( newEdges.size() > 0 ) ) {
                  int k     = 0 ;
                  int nSize = newEdges.size() ;

                  while ( k < nSize ) {
                     edges.addElement ( newEdges.elementAt(k) ) ;
                     k++ ;
                  }
               } /* if  */
            } /* if  */

            i++ ;
         } /* while  */
      }

      return( edges ) ;
   }





   /*------------------------------------------------------------*/
   /* getEdges
    */
   /**
    *    This method will return a Vector of all of the EdgeGroups that
    *  go from the node passed
    *  
    *
    *  @param node The Node to get Edges for
    *  @param symptom The Symptom that the node belongs to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static Vector getEdgeGroups ( Node node, Symptom symptom ) {
      Vector edgeGroups = symptom.getEdgeGroups() ;
      Vector returnVec  = new Vector ( 1 ) ;

      if ( ( edgeGroups != null ) && ( edgeGroups.size() > 0 ) ) {
         int       i         = 0 ;
         int       size      = edgeGroups.size() ;
         EdgeGroup edgeGroup = null ;

         while ( i < size ) {
            edgeGroup = (EdgeGroup) edgeGroups.elementAt(i) ;

            if ( ( edgeGroup.valid() ) && ( edgeGroup.getHeadNode() == node ) ) 
               returnVec.addElement ( edgeGroups.elementAt(i) ) ;
            i++ ;
         } /* while  */
      }

      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* answersHaveEdges
    */
   /**
    *    This method will make sure that each answer
    *  in the answerVec passed has an Edge. It will
    *  first check to see if there is a #OTHER edge. If there is
    *  a #OTHER edge this method will return TRUE
    *
    *  @param answerVec The vector of Answers
    *  @param edgeVec The vector of Edges
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean answersHaveEdges ( Vector answers, Vector edges ) {
      boolean rc = false ;                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First check for a #OTHER edge. If there is one, return  */
      /* TRUE                                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( edges != null ) && ( edges.size() > 0 ) ) {
         int edgeIndex     = 0 ;
         int edgeCount     = edges.size() ;

         while ( ( !rc ) && ( edgeIndex < edgeCount ) ) {
            rc = ( ((Edge)edges.elementAt ( edgeIndex )).getEdgeType() == Edge.EDGE_TYPE_OTHER ) ;
            edgeIndex++ ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, there is no #OTHER, every answer must have an edge  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( !rc ) &&
           ( answers != null ) &&
           ( answers.size() > 0 ) &&
           ( edges != null ) &&
           ( edges.size() > 0 ) ) {
         int     answerSize   = answers.size() ;
         int     answerIndex  = 0 ;
         int     edgeSize     = edges.size() ;
         int     edgeIndex    = 0 ;
         Answer  answer       = null ;
         boolean foundAllAnswers = true ;
         boolean found           = false ;                                           


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through all of the answers and find an edge for    */
         /* each answer                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( ( foundAllAnswers ) && ( answerIndex < answerSize ) ) {

            answer = (Answer)answers.elementAt(answerIndex) ;                                  

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Go through the edges looking for the answer             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            found = false ; // Assume we cannot find the answer in the list of edges
            edgeIndex = 0 ;
            while ( ( edgeIndex < edgeSize ) && ( !found ) ) {
               if ( ( ((Edge)edges.elementAt(edgeIndex)).valid() ) && 
                    ( ((Edge)edges.elementAt(edgeIndex)).getAnswerInd() == answer.getInd() ) )
                  found = true ;

               edgeIndex++ ;
            }

            foundAllAnswers = found ;
            answerIndex++ ;
         }

         rc = foundAllAnswers ;
      }


      return( rc ) ;
   }

} /* SymptomSoftValidator */


