package com.ibm.nzna.projects.qit.avalon.cache ;   
/*-----------------------------------------------------------*/
/*   AvalonCache                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AvalonCache                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This object holds usefull routines that should be used when 
 * reading/writing avalon objects to the database
 *
 *
 * @version 1.0, 5/5/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AvalonCache                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class AvalonCache extends Object {





   /*------------------------------------------------------------*/
   /* readQuestion
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a question cached object matching the questionInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param questionInd the Question Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Question
    *//*---------------------------------------------------------*/
   public static Question readQuestion ( int questionInd ) {
      return( readQuestion ( questionInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readQuestion
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a question cached object matching the questionInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param questionInd the Question Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Question
    *//*---------------------------------------------------------*/
   public static Question readQuestion ( int questionInd, boolean forceReadCache ) {
      String   localTimestamp = null ;
      Question question       = null ;
      String   dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "QUESTION_"+language+"_"+questionInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         question = (Question) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( question == null ) {
            question = Question.readFromDatabase ( questionInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OA.QUESTION WHERE QUESTIONIND = "+questionInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               question = Question.readFromDatabase ( questionInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, question ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( question ) ;
   }




   /*------------------------------------------------------------*/
   /* readQuestionDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a question draft cached object matching the questionInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param questionInd the Question Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param typeWorkRec The TypeWorkRec to open the draft under    
    *
    *  @return Question
    *//*---------------------------------------------------------*/
   public static QuestionDraft readQuestionDraft ( int questionInd ) {
      return( readQuestionDraft ( questionInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readQuestionDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a question draft cached object matching the questionInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param questionInd the Question Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param typeWorkRec The TypeWorkRec to open the draft under
    *
    *  @return Question
    *//*---------------------------------------------------------*/
   public static QuestionDraft readQuestionDraft ( int questionInd, boolean forceReadCache ) {
      String        localTimestamp = null ;
      QuestionDraft question       = null ;
      String        dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "QUESTION_DRAFT_"+language+"_"+questionInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         question = (QuestionDraft) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( question == null ) {
            question = (QuestionDraft)QuestionDraft.readFromDatabase ( questionInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OADRAFT.QUESTION WHERE QUESTIONIND = "+questionInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               question = (QuestionDraft)QuestionDraft.readFromDatabase ( questionInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, question ) ;



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the graphics                                    */           
         /*                                                           */
         /*-----------------------------------------------------------*/
         AddtlInfoGraphicFTP.getGraphicsDraft ( null, question.getAdditionalInfo() ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( question ) ;
   }



   /*------------------------------------------------------------*/
   /* readActionDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a action draft cached object matching the actionInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param actionInd the Action Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param typeWorkRec The TypeWorkRec to open the draft under
    *
    *  @return Action
    *//*---------------------------------------------------------*/
   public static ActionDraft readActionDraft ( int actionInd ) {
      return( readActionDraft ( actionInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readActionDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a action draft cached object matching the actionInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param actionInd the Action Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Action
    *//*---------------------------------------------------------*/
   public static ActionDraft readActionDraft ( int actionInd, boolean forceReadCache ) {
      String        localTimestamp = null ;
      ActionDraft action       = null ;
      String        dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "ACTION_DRAFT_"+language+"_"+actionInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         action = (ActionDraft) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( action == null ) {
            action = (ActionDraft)ActionDraft.readFromDatabase ( actionInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OADRAFT.ACTION WHERE ACTIONIND = "+actionInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               action = (ActionDraft)ActionDraft.readFromDatabase ( actionInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, action ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the graphics                                    */           
         /*                                                           */
         /*-----------------------------------------------------------*/
         AddtlInfoGraphicFTP.getGraphicsDraft ( null, action.getAdditionalInfo() ) ;
         


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( action ) ;
   }





   /*------------------------------------------------------------*/
   /* getDatabaseTimestamp
    */
   /**
    *    This method will execute the query that
    *  it is passed and will return the first result which
    *  should normally be the changedtime field
    *
    *  @param query The query to perform
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static String getDatabaseTimestamp ( String query ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "AvalonCache.getDatabaseTimestamp", 5 ) ;
      String    retStr    = "" ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( query ) ;

         if ( rs.next() )
            retStr = rs.getString(1).trim() ;

         rs.close() ;
         stmt.close() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }
      sqlMethod.close() ;

      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* readLinkGroup
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a link group cached object matching the linkGroupInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param linkGroupInd the LinkGroup Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return LinkGroup
    *//*---------------------------------------------------------*/
   public static LinkGroup readLinkGroup ( int linkGroupInd ) {
      return( readLinkGroup ( linkGroupInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readLinkGroup
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a linkGroup cached object matching the linkGroupInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param linkGroupInd the LinkGroup Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return LinkGroup
    *//*---------------------------------------------------------*/
   public static LinkGroup readLinkGroup ( int linkGroupInd, boolean forceReadCache ) {
      String   localTimestamp = null ;
      LinkGroup linkGroup       = null ;
      String   dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "LINKGROUP_"+language+"_"+linkGroupInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         linkGroup = (LinkGroup) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( linkGroup == null ) {
            linkGroup = LinkGroup.readFromDatabase ( linkGroupInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OA.LINKGROUP WHERE LINKGROUPIND = "+linkGroupInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               linkGroup = LinkGroup.readFromDatabase ( linkGroupInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, linkGroup ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( linkGroup ) ;
   }



   /*------------------------------------------------------------*/
   /* readLink
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a link group cached object matching the linkInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param linkInd the Link Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Object, this could be a QuestLink or ExternalLink
    *//*---------------------------------------------------------*/
   public static Object readLink ( int linkInd ) {
      return( readLink ( linkInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readLink
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a link cached object matching the linkInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param linkInd the Link Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Link
    *//*---------------------------------------------------------*/
   public static Object readLink ( int linkInd, boolean forceReadCache ) {
      String   localTimestamp = null ;
      Object   link           = null ;
      String   dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "LINK_"+language+"_"+linkInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         link = (Link) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( link == null ) {
            link = Link.readFromDatabase ( linkInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OA.LINK WHERE LINKIND = "+linkInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               link = Link.readFromDatabase ( linkInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, link ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( link ) ;
   }


   /*------------------------------------------------------------*/
   /* readLinkDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a link group cached object matching the linkInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param linkInd the Link Indicator to read
    *  @param linkType The link type to use, a 0 passed returns a QuestLink, a 1 passed returns an ExternalLink
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Object, this could be a QuestLink or ExternalLink
    *//*---------------------------------------------------------*/
   public static Object readLinkDraft ( int linkInd, int linkType ) {
      return( readLinkDraft ( linkInd, linkType, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readLinkDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a link cached object matching the linkInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param linkInd the Link Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Link
    *//*---------------------------------------------------------*/
   public static Object readLinkDraft ( int linkInd, int linkType, boolean forceReadCache ) {
      String   localTimestamp = null ;
      Object   link           = null ;
      String   dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "LINKDRAFT_"+language+"_"+linkInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         link = (LinkDraft) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( link == null ) {

            switch ( linkType ) {
               case 0 :
                  link = (QuestLinkDraft)QuestLinkDraft.readFromDatabase ( linkInd, language ) ;
                  break ;

               case 1 :
                  link = (ExternalLinkDraft)ExternalLinkDraft.readFromDatabase ( linkInd, language ) ;
                  break ;
            }

            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OA.LINK WHERE LINKIND = "+linkInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               link = Link.readFromDatabase ( linkInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, link ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( link ) ;
   }





   /*------------------------------------------------------------*/
   /* readSymptom
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a symptom cached object matching the symptomInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param symptomInd the Symptom Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Symptom
    *//*---------------------------------------------------------*/
   public static Symptom readSymptom ( int symptomInd ) {
      return( readSymptom ( symptomInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readSymptom
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a symptom cached object matching the symptomInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param symptomInd the Symptom Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Symptom
    *//*---------------------------------------------------------*/
   public static Symptom readSymptom ( int symptomInd, boolean forceReadCache ) {
      String   localTimestamp = null ;
      Symptom symptom       = null ;
      String   dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "SYMPTOM_"+language+"_"+symptomInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         symptom = (Symptom) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( symptom == null ) {
            symptom = Symptom.readFromDatabase ( symptomInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OA.SYMPTOM WHERE SYMPTOMIND = "+symptomInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               symptom = Symptom.readFromDatabase ( symptomInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, symptom ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( symptom ) ;
   }




   /*------------------------------------------------------------*/
   /* readSymptomDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a symptom draft cached object matching the symptomInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param symptomInd the Symptom Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param typeWorkRec The TypeWorkRec to open the draft under    
    *
    *  @return Symptom
    *//*---------------------------------------------------------*/
   public static SymptomDraft readSymptomDraft ( int symptomInd ) {
      return( readSymptomDraft ( symptomInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readSymptomDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a symptom draft cached object matching the symptomInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param symptomInd the Symptom Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param typeWorkRec The TypeWorkRec to open the draft under
    *
    *  @return Symptom
    *//*---------------------------------------------------------*/
   public static SymptomDraft readSymptomDraft ( int symptomInd, boolean forceReadCache ) {
      String        localTimestamp = null ;
      SymptomDraft symptom       = null ;
      String        dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "SYMPTOM_DRAFT_"+language+"_"+symptomInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         symptom = (SymptomDraft) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( symptom == null ) {
            symptom = (SymptomDraft)SymptomDraft.readFromDatabase ( symptomInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OADRAFT.SYMPTOM WHERE SYMPTOMIND = "+symptomInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               symptom = (SymptomDraft)SymptomDraft.readFromDatabase ( symptomInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, symptom ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( symptom ) ;
   } /* readSymptomDraft  */



   /*------------------------------------------------------------*/
   /* readAction
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a action cached object matching the actionInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param actionInd the Action Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Action
    *//*---------------------------------------------------------*/
   public static Action readAction ( int actionInd ) {
      return( readAction ( actionInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readAction
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a action cached object matching the actionInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param actionInd the Action Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return Action
    *//*---------------------------------------------------------*/
   public static Action readAction ( int actionInd, boolean forceReadCache ) {
      String   localTimestamp = null ;
      Action action       = null ;
      String   dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "ACTION_"+language+"_"+actionInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         action = (Action) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( action == null ) {
            action = Action.readFromDatabase ( actionInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OA.ACTION WHERE ACTIONIND = "+actionInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               action = Action.readFromDatabase ( actionInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, action ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( action ) ;
   }




   /*------------------------------------------------------------*/
   /* readLinkGroup
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a link group cached object matching the linkGroupInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  
    *
    *  @param linkGroupInd the LinkGroup Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return LinkGroup
    *//*---------------------------------------------------------*/
   public static LinkGroupDraft readLinkGroupDraft ( int linkGroupInd ) {
      return( readLinkGroupDraft ( linkGroupInd, false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* readLinkGroupDraft
    */
   /**
    *    This method will do the following:<br><br><pre>
    *
    *  1. Look for a linkGroup cached object matching the linkGroupInd<br>
    *  2. If found, compare the changed time on the object to the changedtime in the database<br>
    *  3. If different, load the object from the database<br>
    *  4. If the same, or the database is lower than cached object, use the cached object<br><br></pre>
    *
    *  Please note however if forceReadCache is TRUE, step 2 will not compare the local record with the
    *  database record!
    *  
    *
    *  @param linkGroupInd the LinkGroup Indicator to read
    *  @param forceReadCache TRUE = Read from cache only, if not in cache, then read from database.
    *  @param None
    *
    *  @return LinkGroup
    *//*---------------------------------------------------------*/
   public static LinkGroupDraft readLinkGroupDraft ( int linkGroupInd, boolean forceReadCache ) {
      String         localTimestamp = null ;
      LinkGroupDraft linkGroup       = null ;
      String         dbTimestamp    = null ;

      try {
         int     language       = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  filename       = "LINKGROUPDRAFT_"+language+"_"+linkGroupInd+".avalon" ;
         boolean cache          = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try to read it from cache                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         linkGroup = (LinkGroupDraft) CacheSystem.getCachedObject ( filename ) ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If not in cache, read it from database                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( linkGroup == null ) {
            linkGroup = (LinkGroupDraft)LinkGroupDraft.readFromDatabase ( linkGroupInd, language ) ;
            cache = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If it was in cache, check the timestamps                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( !forceReadCache ) {
            dbTimestamp = getDatabaseTimestamp ( "SELECT CHANGEDTIME FROM OADRAFT.LINKGROUP WHERE LINKGROUPIND = "+linkGroupInd ) ;
            localTimestamp = CacheSystem.getCacheTime ( filename ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the local timestamp is not greater than the database */
            /* then read it from the database                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( CDate.compareDate ( localTimestamp, dbTimestamp, CDate.DATE_STAMP ) != 1 ) {
               linkGroup = (LinkGroupDraft)LinkGroupDraft.readFromDatabase ( linkGroupInd, language ) ;
               cache = true ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If cache is true, recache this object for next time     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cache )
            CacheSystem.cacheObject ( filename, linkGroup ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      return( linkGroup ) ;
   }



} /* AvalonCache */
