package com.ibm.nzna.projects.qit.avalon.editors ;       
/*-----------------------------------------------------------*/
/*   ActionEditorPanel                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionEditorPanel                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.spell.* ;
/**
 * This panel allows a user to edit everything about a Action. 
 *
 *
 * @version 1.0, 4/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionEditorPanel                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionEditorPanel extends OAEditorPanel implements CountrySelectListener, EditPanelListener, AvalonConst, ActionListener, QuestPanel, AppConst, Runnable, PublishPanelListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_INIT      = "INIT" ;
   private static final String THREAD_REFRESH   = "REFRESH" ;  

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Panel Names                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String PANEL_USAGE       = "USAGE" ;
   public static final String PANEL_DRAFT       = "DRAFT" ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ActionButton     pb_CLOSE         = null ;      
   private ActionButton     pb_SAVE          = null ;      
   private ActionButton     pb_SAVEAS        = null ;      
   private ActionButton     pb_VALIDATE      = null ;
   private AppDefaultWin    parentWin        = null ;
   private JTabbedPane      tabbedPane       = null ;
   private QuestPanel       previousPanel    = null ;
   private DraftEditPanel   draftPanel       = null ;
   private TitleEditPanel   titleEditPanel   = null ;
   private AddInfoEditPanel addInfoEditPanel = null ;
   private LinkBinderPanel  linkPanel        = null ;
   private JLabel           st_TITLE         = null ;
   private SpellPanel       spellPanel       = null ;
   private JPanel           contentPanel     = null ;
   private ToolBarButton    pb_SPELL         = null ;
   private JToolBar         toolBar          = null ;
   private ActionUsagePanel actionUsagePanel = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ActionDraft action         = null ;
   private int           actionInd      = 0 ;
   private boolean       readFromDatabase = false ;
   private boolean       saved            = false ;
   private boolean       forceClose       = false ;
   private boolean       publishing       = false ;
   private PublishPanel  publishPanel     = null ;
   private boolean       published        = false ;       // TRUE = We have published the object
   private boolean       hasAuth          = false ;       // TRUE = The user has authority to save, FALSE = do not save
   private boolean       askCountryDelete = true ;        // TRUE = Ask the user if they want to delete the country
   private boolean       autoPublish      = false ;       // TRUE = Auto Publish the object once read in
   private String        initialPanel     = PANEL_DRAFT ; // The initial panel to show when the editor opens







   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor will create a new
    *  Action.
    *  
    *
    *  @param previousPanel The panel to show once the user closes this panel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionEditorPanel ( QuestPanel previousPanel ) {
      this.previousPanel = previousPanel ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create a new Action                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      action = (ActionDraft) new NewAvalonObjectWizard ( GUISystem.getParentDefWin ( previousPanel.getJComponent() ), AVALON_ACTION ).getNewObject() ;

   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will edit the action
    *  passed in and then set the panel in the tabbed
    *  pane to the panel passed in
    *
    *  @param questPanel The previous panel
    *  @param action The ActionDraft to edit
    *  @param panelName One of the panel names ( PANEL_DRAFT, PANEL_USAGE ) as defined in this class
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionEditorPanel ( QuestPanel questPanel, ActionDraft action, String panelName ) {
      this.previousPanel = previousPanel ;
      this.action        = action ;
      this.initialPanel  = panelName ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will read in the action that represents
    * the actionInd passed in
    *  
    *
    *  @param previousPanel The panel to show once the user closes this panel
    *  @param actionInd The Active Action Indicator indicator to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionEditorPanel ( QuestPanel previousPanel, int actionInd ) {
      this.previousPanel    = previousPanel ;             
      this.actionInd      = actionInd ;
      this.readFromDatabase = true ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will edit the action passed in
    *  
    *
    *  @param previousPanel The panel to show once the user closes this panel
    *  @param action The ActionDraft object to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionEditorPanel ( QuestPanel previousPanel, ActionDraft actionDraft ) {
      this.previousPanel    = previousPanel ;             
      if ( actionDraft != null ) {
         this.action = actionDraft ;
         action.updateRecStatus ( action.REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Cannot edit a NULL quesitonDraft" ) ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will edit the action passed in
    *  
    *
    *  @param previousPanel The panel to show once the user closes this panel
    *  @param action The ActionDraft object to edit
    *  @param autoPublish TRUE = AutoPublish
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionEditorPanel ( QuestPanel previousPanel, ActionDraft actionDraft, boolean autoPublish ) {
      this.previousPanel    = previousPanel ;             
      this.autoPublish      = autoPublish ;
      if ( actionDraft != null ) {
         this.action = actionDraft ;
         action.updateRecStatus ( action.REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Cannot edit a NULL quesitonDraft" ) ;
   }





   /*------------------------------------------------------------*/
   /* ActionEditorPanel
    */
   /**
    *    This constructor is an auto-publish constructor,
    *  meaning that the actionDraftInd passed in will
    *  be read in and the publish button will be hit automatically.
    *
    *  @param previousPanel The Previous Panel to go to after publish
    *  @param actionDraftInd The Draft indicator of the action
    *  @param autoPublish TRUE = AutoPublish
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionEditorPanel ( QuestPanel previousPanel, int actionDraftInd, boolean autoPublish ) {
      this.previousPanel = previousPanel ;

      if ( actionDraftInd != 0 ) {
         this.action = AvalonCache.readActionDraft ( actionDraftInd ) ;

         if ( action != null )
            this.autoPublish = autoPublish ;
      }
   }



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will simply kick off another thread to initialize on
    *  if we have a valid action.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean TRUE if we can show, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our ActionButtons here first, since they need to */
      /* show first                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE         = new ActionButton  ( Str.getStr ( STR_CLOSE ),      ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),       Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;                     
      pb_SAVE          = new ActionButton  ( Str.getStr ( STR_SAVE  ),      ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),        Str.getStr ( STR_SAVE ) ) ;           
      pb_SAVEAS        = new ActionButton  ( Str.getStr ( STR_SAVE_AS ),    ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),        Str.getStr ( STR_SAVE_AS ) ) ;           
      pb_VALIDATE      = new ActionButton  ( Str.getStr ( STR_VALIDATE ),   ImageSystem.getImageIcon ( this, ImageSystem.NEW ),         Str.getStr ( STR_VALIDATE ) ) ;
      toolBar          = new JToolBar() ;
      pb_SPELL         = new ToolBarButton ( Str.getStr ( STR_SPELL ), ImageSystem.getImageIcon ( this, ImageSystem.SPELL ), Str.getStr ( STR_SPELL ) ) ;
      st_TITLE         = new JLabel () ;
      spellPanel       = new SpellPanel () ;
      contentPanel     = new JPanel () ;     
      actionUsagePanel = new ActionUsagePanel ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener    ( this ) ;
      pb_SAVE.addActionListener     ( this ) ;     
      pb_SAVEAS.addActionListener   ( this ) ;     
      pb_VALIDATE.addActionListener ( this ) ;
      pb_SPELL.addActionListener    ( this ) ;

      if ( ( action != null ) || ( readFromDatabase ) ) {
         Color lightGray = new Color ( 240, 240, 240 ) ;                           

         try {
            JScrollPane scrPane     = null ;     
            JScrollPane draftScroll = null ;
            JScrollPane titleScroll = null ;
            JScrollPane linkScroll  = null ;
            JScrollPane usageScroll = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create our GUI                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            draftPanel       = new DraftEditPanel   (  ) ;
            titleEditPanel   = new TitleEditPanel   ( 0 ) ; // Pass a zero as a brand because we don't know it yet
            addInfoEditPanel = new AddInfoEditPanel (  ) ;
            linkPanel        = new LinkBinderPanel  () ;
            tabbedPane       = new JTabbedPane () ;    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add listeners                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            titleEditPanel.addEditPanelListener ( this ) ;
            draftPanel.getCountrySelectPanel().addCountrySelectListener ( this ) ;                                        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Join all CountrySelectPanels to the CountrySelectPanel  */
            /* on the DraftEditPanel                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            titleEditPanel.getCountrySelectPanel().joinCountrySelectPanel ( draftPanel.getCountrySelectPanel() ) ;
            addInfoEditPanel.getCountrySelectPanel().joinCountrySelectPanel ( draftPanel.getCountrySelectPanel() ) ;
            linkPanel.getCountrySelectPanel().joinCountrySelectPanel ( draftPanel.getCountrySelectPanel() ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add our GUI                                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            toolBar.add ( new ToolBarSeparator () ) ;
            toolBar.add ( pb_SPELL ) ;
            tabbedPane.addTab ( Str.getStr ( STR_DRAFT ),                  draftScroll = new JScrollPane ( draftPanel ) ) ;
            tabbedPane.addTab ( Str.getStr ( STR_TITLES ),                 titleScroll = new JScrollPane ( titleEditPanel ) ) ;
            tabbedPane.addTab ( Str.getStr ( STR_ADDITIONAL_INFORMATION ), scrPane     = new JScrollPane ( addInfoEditPanel ) ) ;
            tabbedPane.addTab ( Str.getStr ( STR_LINKS ),                  linkScroll  = new JScrollPane ( linkPanel ) ) ;
            tabbedPane.addTab ( Str.getStr ( STR_ACTION_USAGE ),           usageScroll = new JScrollPane ( actionUsagePanel ) ) ;
            setLayout ( new BorderLayout() ) ;
            add ( contentPanel, BorderLayout.CENTER ) ;
            contentPanel.setLayout ( new BorderLayout() ) ;
            contentPanel.add ( st_TITLE,   BorderLayout.NORTH ) ;
            contentPanel.add ( tabbedPane, BorderLayout.CENTER ) ;                 

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Adjust notebook properties                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            scrPane.setBorder ( null ) ;
            tabbedPane.setBorder ( null ) ;
            tabbedPane.setBackgroundAt ( 0, lightGray ) ;
            tabbedPane.setBackgroundAt ( 1, lightGray ) ;
            tabbedPane.setBackgroundAt ( 2, lightGray ) ;
            tabbedPane.setBackgroundAt ( 3, lightGray ) ;
            tabbedPane.setBackgroundAt ( 4, lightGray ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set properties                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            draftScroll.setBorder ( null ) ;
            titleScroll.setBorder ( null ) ;
            scrPane.setBorder ( null ) ;
            linkScroll.setBorder ( null ) ;
            usageScroll.setBorder ( null ) ;
            linkPanel.setDraftEditPanel ( draftPanel );
            setBackground ( Color.white ) ;
            tabbedPane.setOpaque ( false ) ;
            addInfoEditPanel.showRedirect ( true ) ;

         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
         }
      } /* if Action  */

      return( (action != null ) || ( readFromDatabase ) ) ;

   }




   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing. We will use this time to refresh
    *  our data
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      NavList hotLinkNavList = new NavList() ;
      boolean refresh        = true ;

      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;                                  
      parentWin.addActionComponent ( this, pb_SAVE ) ;                                                                           
      parentWin.addActionComponent ( this, pb_SAVEAS ) ;                                                                           
      parentWin.addActionComponent ( this, pb_VALIDATE ) ;
      addToolBarButtons() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we need to read our action from the database,        */
      /* do it now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( readFromDatabase ) {
         try {
            action = AvalonCache.readActionDraft ( actionInd ) ;

            if ( action == null ) {
               forceClose() ;
               GUISystem.printBox ( this, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
               refresh = false ;
            }  /* if the action was read */
         } catch ( Exception e ) {
            forceClose() ;
         }
      } /* if we were to read the action */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, kick off the thread to refresh our data             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( refresh )
         new Thread ( this, THREAD_REFRESH ).start() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Maintlock if this is not an insert                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( action != null ) && ( !autoPublish ) ) {
         if ( ( !locked() ) && ( action.getRecStatus() != action.REC_INSERT ) )
            setMaintlock ( "Y" ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to auto-publish, do so now                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( !locked() ) && ( autoPublish ) )
         saveAsPublish() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, Now show the initial panel                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !autoPublish )
         showPanel ( initialPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* showPanel
    */
   /**
    *    This method will show the tabbed pane panel
    *  that is passed
    *  
    *
    *  @param panelName The name of the panel as defined in this class ( PANEL_DRAFT etc.. )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showPanel ( String panel ) {

      if ( panel.equals ( PANEL_DRAFT ) )
         tabbedPane.setSelectedComponent ( WinUtil.getParentJScrollPane ( draftPanel ) ) ;

      else if ( panel.equals ( PANEL_USAGE ) ) {
         tabbedPane.setSelectedComponent ( WinUtil.getParentJScrollPane ( actionUsagePanel ) ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when this panel
    *  has been deselected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
   }





   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when this panel
    *  is selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
      addToolBarButtons() ;
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the user wants
    *  us to close. We will close if the data has been
    *  saved 
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      boolean rc = false ;                                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to force close, bypass everything             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !forceClose ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are not publishing, continue with the close       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !publishing ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the user has not saved, ask them to                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            System.out.println ( "Saved Status:"+saved ) ;
            if ( !saved ) {


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If the record is not locked, ask the user               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               System.out.println ( "Locked Status:"+locked() ) ;
               if ( !locked() ) {
                  System.out.println ( "hasAuth Status:"+hasAuth ) ;
                  if ( ( hasAuth ) && ( GUISystem.printBox ( STR_QUESTION, STR_SAVE_ACTION ) ) )
                     rc = save() ;
                  else {
                     /* Unlock the record */
                     if ( action.getRecStatus() == action.REC_INSERT )
                        setOpenDraft ( "N" ) ;
                     rc = true ; // Set the return to TRUE
                  }
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise we cannot save, so return TRUE                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  rc = true ;       // Return TRUE since no action was taken
                  saved = false ;   // set Saved to FALSE since we did not save
               } /* else */

            } else {
               rc = true ; // Set the return to TRUE since they already saved
            }

         } else {
            GUISystem.printBox ( STR_INFO, STR_CANNOT_CLOSE_PUBLISHING ) ;
            rc = false ; // Set the return to FALSE since we cannot close if publishing
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notify our listeners that we have saved                 */
         /* and update our database flags                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc ) {
            if ( !locked() )
               setMaintlock ( "N" ) ;

            if ( saved ) {
               saveComplete ( action ) ;

               if ( published )
                  setOpenDraft ( "N" ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notify the super to stop                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc )
            super.stop() ;


      } else
         rc = true ; // Set return to TRUE since we are forcing the close


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set saved to false for next time around                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      saved = false ;

      if ( rc )
         panelDeselected() ;

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the user
    *  wants to shutdown the applicaiton. We will
    *  return TRUE if the data has been saved
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default navigation
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NavPanel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.ACTION_SMALL ) ) ;
   }



   public void refresh () {
   }


   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh the data on our
    *  screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData () {

      draftPanel.setDraftObject        ( action ) ;
      draftPanel.setObjectType         ( TypeList.TYPE_ACTION ) ;
      draftPanel.setSelectedObjectType ( (Object)action.getActionType() ) ;
      draftPanel.setDraftIndicator     ( ""+action.getInd() ) ;
      draftPanel.setActiveIndicator    ( ""+action.getActiveActionInd() ) ;

      actionUsagePanel.setAction       ( action ) ;
      titleEditPanel.setTitles         ( action.getTitles() ) ;
      titleEditPanel.setBrandInd       ( (action.getBrand() != null ) ? action.getBrand().getInd() : 0 ) ;
      addInfoEditPanel.setAddtlInfos   ( action.getAdditionalInfo() ) ;
      linkPanel.setLinks               ( action.getDocLinks() ) ;
      st_TITLE.setText                 ( AvalonUtil.getDefaultTitleObject ( action ).toString ( Title.HTML_TITLE ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user does not have the authority to edit         */
      /* this object, tell them and take away the save button      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         hasAuth = UserSystem.hasAuthority ( QuestUtil.brandAuthFromBrandInd ( action.getBrand().getInd() ) ) ;

         pb_SAVE.setVisible ( hasAuth ) ;

         if ( !hasAuth )
            GUISystem.printBox ( STR_INFO,
                                 STR_NO_AUTHORITY_EDIT_BRAND ) ;

      } catch ( Exception e ) {
         // Object has no brand, allow the edit
         hasAuth = true ;
         pb_SAVE.setVisible ( true ) ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user is not a BPM or Editor, disable the brand   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         if ( ( !UserSystem.getLoggedUserRec().isBPM() ) &&
              ( !UserSystem.hasAuthority ( AUTH_AUTHOR ) ) )
            draftPanel.setBrandEnabled ( false ) ;

      } catch ( Exception e ) {
         hasAuth = true ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this is locked, tell the user                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( locked() ) {
         GUISystem.printBox ( parentWin,
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_DRAFT_LOCKED_BY )+" "+UserSystem.getNameFromUserId ( action.getDbUser() ) ) ;
         draftPanel.setEnabled       ( false ) ;
         titleEditPanel.setEnabled   ( false ) ;
         addInfoEditPanel.setEnabled ( false ) ;
         pb_SAVE.setVisible ( false ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this is recycled, let them know                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( action.isRecycled() ) {
         GUISystem.printBox ( parentWin,
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_DRAFT_IS_RECYCLED_BY )+" "+UserSystem.getNameFromUserId ( action.getDbUser() ) ) ;
      }


   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an Action
    *  event is fired and we need to respond
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CLOSE                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == pb_CLOSE )
            parentWin.closePanel ( this, previousPanel ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_SAVE                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SAVE )
            processSave() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_SAVEAS                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SAVEAS )
            saveAs() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_VALIDATE                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_VALIDATE ) {
            if ( saveData() ) {
               if ( validateAction() )
                  GUISystem.printBox ( STR_INFO, STR_NO_ERRORS_IN_VALIDATION ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_SPELL                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SPELL ) {
            spellCheck() ;
         }


      } catch ( Exception e1 ) {
         LogSystem.log ( 1, e1 ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_ACTION_EDITOR ) ) ;
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   } /* bookmark  */




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will do a variety of operations. The
    *  operation done depends on the name of the
    *  thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run ( )  {
      String operation = Thread.currentThread().getName() ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_REFRESH ) )
         refreshData() ;

   } /* run */





   /*------------------------------------------------------------*/
   /* saveAsDraft
    */
   /**
    *    This method will save this action
    *  as a draft. It will close the window if the
    *  property is set
    *
    *  @param forPublish - TRUE = This draft is being saved to publish the Action
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveAsDraft ( boolean forPublish ) {                         
      boolean rc   = false ;
      boolean cont = false ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, disable all of our controls                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setEnabled ( false ) ;
      tabbedPane.setEnabled ( false ) ;
      Qit.setEnabled ( false ) ;

      if ( !forPublish )
         cont = AddtlInfoGraphicFTP.putGraphicsDraft ( GUISystem.getParentDefWin ( this ),
                                                       action.getAdditionalInfo() ) ;
      else
         cont = true ;

      if ( cont ) {
         try {
            int saveRc = 0 ;

            parentWin.setStatus ( STR_SAVING ) ;
            saveRc = action.writeToDatabase() ;
            LogSystem.log ( 1, "Write Action Returned:"+saveRc ) ;
            rc = (saveRc == 0) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
            setEnabled ( true ) ;
            tabbedPane.setEnabled ( true ) ;
         }

      }

      if ( ( !rc ) || ( !cont ) )
         GUISystem.printBox ( parentWin, STR_INFO, STR_ACTION_NOT_SAVED ) ;

      tabbedPane.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;
      Qit.setEnabled ( true ) ;

      return( rc ) ;
   } /* saveAsDraft  */




   /*------------------------------------------------------------*/
   /* save
    */
   /**
    *    This method will ask the user what type
    *  of save they want, and will call the appropriate
    *  save methods.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if the object was saved to the database
    *//*---------------------------------------------------------*/
   private boolean save () {
      boolean        rc          = false ;
      int            saveType    = -1 ;
      String         comments    = null ;
      SaveTypeDlg    saveTypeDlg = null ;                                                                                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only continue if the user has entered all of the data   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( saveData() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ask the user what type of save they would like          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         saveTypeDlg = new SaveTypeDlg ( WinUtil.getParentFrame ( this ), 
                                         draftPanel.getOriginalComments(),
                                         draftPanel.getComments(),
                                         action.isArchived(),
                                         action.getTypeWorkRequired().getInd() == TypeList.WORKREQ_DELETE ) ;

         saveType = saveTypeDlg.getResult() ;
         comments = saveTypeDlg.getComments() ;

         action.setComments ( comments ) ;

         switch ( saveType ) {
            case SaveTypeDlg.DRAFT :
               rc = saveAsDraft ( false ) ;
               break ;

            case SaveTypeDlg.PUBLISH :
               if ( validateAction() ) {
                  if ( saveAsDraft ( true ) )
                     saveAsPublish() ;
               }
               break ;

            case SaveTypeDlg.ARCHIVE :
               if ( validateAction() ) {
                  action.setArchived ( true ) ;
                  if ( saveAsDraft ( false ) ) {
                     rc = OAObjectArchiver.archiveObject ( action, this ) ;

                     if ( !rc )
                        reopenDraft() ;
                  }
               }
               break ;

            case SaveTypeDlg.UNARCHIVE :
               if ( validateAction() ) {
                  action.setArchived ( false ) ;
                  if ( saveAsDraft ( true ) )
                     saveAsPublish() ;
               }
               break ;

            case SaveTypeDlg.CANCEL_DRAFT :
               if ( OASQL.cancelDraft ( action ) ) {
                  saved = true ;
                  forceClose = true ;
                  parentWin.closePanel ( this, previousPanel ) ;
               }
               break ;

         } /* switch */
      }

      saved = rc ;
      return( rc ) ;
   } /* save  */




   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will go down through all
    *  our panels and will ask each one if saving
    *  is ok
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      boolean rc = false ;

      rc = ( ( draftPanel.saveData() ) &&
             ( addInfoEditPanel.saveData() ) &&
             ( titleEditPanel.saveData() ) ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save the Links                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc ) {
         Vector titles = action.getTitles() ;

         action.setDocLinks ( linkPanel.getDocLinks( ) ) ;
         action.setBrand ( draftPanel.getBrand() ) ;

         ((Title)titles.elementAt(titles.size()-1)).setDocClass ( draftPanel.getDocClass() ) ;
         action.setTitles ( titles ) ;

         rc = ( ( action.getDocLinks() != null ) &&
                ( action.getBrand()    != null ) &&
                ( action.getDocClass() != null ) ) ;

         if ( !rc )
            showPanel ( PANEL_DRAFT ) ;

      }

      /*--------------------------------------------------------------*/
      /*                                                              */
      /*   If all of the generic panels saved their data, save ours   */
      /*                                                              */
      /*--------------------------------------------------------------*/
      if ( rc ) {
         Vector titles = action.getTitles() ;
         Title  title  = (Title)titles.elementAt(titles.size()-1) ;

         action.setActionType ( (TypeActionRec) draftPanel.getSelectedObjectType() ) ;
         action.setDbUser ( UserSystem.getUserId() ) ;                                        

      }

      return( rc ) ;
   } /* saveData  */





   /*------------------------------------------------------------*/
   /* forceClose
    */
   /**
    *    This method will force close the panel
    *  without asking the user any actions
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void forceClose () {
      forceClose = true ;
      publishing = false ;
      parentWin.closePanel ( this, previousPanel ) ;
   }


   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will get the default title
    *  that shows 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getDefaultTitle () {
      Vector vec = action.getTitles() ;

      return( vec.elementAt(vec.size()-1).toString() ) ;
   }




   /*------------------------------------------------------------*/
   /* editPanelChanged
    */
   /**
    *    This method will be called when something
    *  on the edit panel has changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void editPanelChanged ( EditPanel editPanel ) {
      if ( editPanel == titleEditPanel ) {
         try {
            st_TITLE.setText ( titleEditPanel.getDefaultTitle().toString(Title.HTML_TITLE) ) ;
         } catch ( Exception e ) {
         }
      }
   }



   /*------------------------------------------------------------*/
   /* setMaintLock
    */
   /**
    *    This method will set the maint lock flag
    *  to the value passed
    *  
    *
    *  @param value The value to set maintlock in OADRAFT.QUESTION to. Either Y or N
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setMaintlock ( String value ) {
      if ( action.getRecStatus() != action.REC_INSERT ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setMaintLock("+value+")", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            stmt.executeUpdate ( "UPDATE OADRAFT.ACTION "+
                                 "SET MAINTLOCK = '"+value+"', "+
                                 "    DBUSER    = '"+UserSystem.getUserId()+"', "+
                                 "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE ACTIONIND = "+action.getInd() ) ;
         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e ) ;
         }

         sqlMethod.close() ;
      }
   }




   /*------------------------------------------------------------*/
   /* locked
    */
   /**
    *    This method will return TRUE if this action
    *  draft being edited is locked in the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   private boolean locked () {
      boolean locked = false;

      if ( action != null ) {
         if ( action.getMaintLock() ) {
            locked =  !( action.getDbUser().equals ( UserSystem.getUserId() ) ) ;
         }
      }

      return( locked ) ;
   }




   /*------------------------------------------------------------*/
   /* saveAsPublish
    */
   /**
    *    This method will start the publishing
    *  process
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveAsPublish () {               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put the publish panel up                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( validateAction() ) {
         publishPanel = new PublishPanel ( action ) ;
         publishPanel.setPublishPanelListener ( this ) ;
         contentPanel.remove ( tabbedPane ) ;
         contentPanel.add ( publishPanel, BorderLayout.CENTER ) ;
         pb_SAVE.setVisible ( false ) ;
         pb_VALIDATE.setVisible ( false ) ;
         pb_SAVEAS.setVisible ( false ) ;
         pb_CLOSE.setVisible ( false ) ;
         publishing = true ;
         new Thread ( publishPanel, publishPanel.THREAD_VALIDATE ).start() ;
      }
   }





   /*------------------------------------------------------------*/
   /* publishComplete
    */
   /**
    *    This method will be called once a publish has
    *  completed
    *  
    *
    *  @param complete TRUE = The object was published, FALSE if not
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void publishComplete ( boolean complete ) {
      LogSystem.log ( 1, "ActionEditorPanel, Publish Complete:"+complete ) ;
      if ( complete ) {
         contentPanel.remove ( publishPanel ) ;
         publishing = false ;
         saved = true ;
         published = true ;
         AddtlInfoGraphicFTP.putGraphicsPub ( GUISystem.getParentDefWin ( this ),
                                              action.getAdditionalInfo() ) ;
         AddtlInfoGraphicFTP.removeGraphicsDraft ( GUISystem.getParentDefWin ( this ), 
                                                   action.getAdditionalInfo() ) ;
         parentWin.closePanel ( this, previousPanel ) ;
      } else {
         pb_SAVE.setVisible ( true ) ;
         pb_SAVEAS.setVisible ( true ) ;
         pb_VALIDATE.setVisible ( true ) ;
         pb_CLOSE.setVisible ( true ) ;
         contentPanel.remove ( publishPanel ) ;
         publishPanel = null ;
         contentPanel.add ( tabbedPane ) ;
         publishing = false ;
         setVisible ( false ) ;
         setVisible ( true ) ;
         reopenDraft() ;
      }
   }


   /*------------------------------------------------------------*/
   /* setOpenDraft
    */
   /**
    *    This method will set the open draft flag
    *  to the value passed
    *  
    *
    *  @param value The value to set opendraft in OA.QUESTION to. Either Y or N
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setOpenDraft ( String value ) {
      if ( action != null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setOpenDraft("+value+")", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            stmt.executeUpdate ( "UPDATE OA.ACTION "+
                                 "SET OPENDRAFT = '"+value+"', "+
                                 "    DBUSER    = '"+UserSystem.getUserId()+"', "+
                                 "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE ACTIONIND = "+action.getActiveActionInd() ) ;
         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e ) ;
         }

         sqlMethod.close() ;
      }
   }

   /*------------------------------------------------------------*/
   /* getPublished
    */
   /**
    *    This method will return TRUE if the last
    *  save we did published the object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getPublished () {
      return( published ) ;
   }






   /*------------------------------------------------------------*/
   /* countryAdded
    */
   /**
    *    This method will be called when the user attempts
    *  to add a country to this Question. We do nothing
    *  here
    *
    *  @param countryRec The TypeCountryCodeRec added
    *  @param None
    *  @param None
    *
    *  @return TRUE
    *//*---------------------------------------------------------*/
   public boolean countryAdded ( TypeCountryCodeRec countryRec ) {
      Title  title     = AvalonUtil.getDefaultTitleObject ( action ) ;
      Vector countries = title.getCountryList() ;

      if ( !countries.contains ( countryRec ) )
         countries.addElement ( countryRec ) ;

      title.setCountryList ( countries ) ;
      new Thread ( titleEditPanel ).start() ; // Refresh the titles

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* countryRemoved
    */
   /**
    *    This method will be called when a country
    *  is trying to be removed from this Question. We
    *  first have to ask the user if they want to remove
    *  that country from all properties (titles, add info etc..)
    *  from the Question. If they answer 'Yes', then we allow the removal,
    *  if the answer 'No' then we do not allow the removal
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean countryRemoved ( TypeCountryCodeRec countryRec ) {
      String  questionStr = Str.getStr ( STR_ARE_YOU_SURE_REMOVE_COUNTRY_FROM_OBJECT ) ;
      boolean remove      = false ;

      questionStr = Text.replaceInStr ( questionStr, "%COUNTRYNAME%", countryRec.toString() ) ;

      if ( askCountryDelete ) {
         remove = GUISystem.printBox ( Str.getStr ( STR_QUESTION ), questionStr ) ;
         askCountryDelete = !remove ;
      } else
         remove = true ;

      if ( remove )
         removeCountryFromPanels ( countryRec ) ;

      return( remove ) ;
   }





   /*------------------------------------------------------------*/
   /* countryRemoveDone
    */
   /**
    *    This method will be called once all countries
    *  have been removed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void countryRemoveDone () {
      askCountryDelete = true ;
   }



   /*------------------------------------------------------------*/
   /* removeCountryFromPanels
    */
   /**
    *    This method will go through all of our
    *  panels and remove the country that is passed in
    *  from all objects on them
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeCountryFromPanels ( TypeCountryCodeRec countryRec ) {
      titleEditPanel.removeCountryFromPanel ( countryRec ) ;
      addInfoEditPanel.removeCountryFromPanel ( countryRec ) ;
      linkPanel.removeCountryFromPanel ( countryRec ) ;
   }


   /*------------------------------------------------------------*/
   /* validateAction
    */
   /**
    *    This method will validate the symptom being
    *  edited. It will return TRUE if the validation was
    *  without error
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   private boolean validateAction () {
      return( !(new SoftValidateDlg ( GUISystem.getParentDefWin ( this ), 
                                      ActionSoftValidator.validateAction ( action ) ).getHadErrors()) ) ;
   }




   /*------------------------------------------------------------*/
   /* spellCheck
    */
   /**
    *    This method will spell check all of our
    *  panels.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void spellCheck () {
      add ( spellPanel, BorderLayout.NORTH ) ;
      revalidate() ;
      titleEditPanel.spellCheck ( tabbedPane, spellPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* This method will be called whenever one
    */
   /**
    *    of our EditorPanels has completed a spell
    *  check. Since there are many panels to spell check,
    *  we will use this oppurtunity to kick off the next spell check on the next panel
    *
    *  @param editPanel The editPanel that just got finished with the Spell Check
    *  @param errorsFound TRUE if any errors were found, regardless if they were fixed
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void spellCheckComplete ( EditPanel editPanel, boolean errorsFound ) {                                                                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we just completed checking the Titles, we are        */
      /* done.                                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( editPanel == titleEditPanel ) {
         if ( !errorsFound )
            GUISystem.printBox ( STR_INFO, STR_SPELL_CHECK_COMPLETE ) ;
         remove ( spellPanel ) ;
         revalidate() ;
      }
   }


   /*------------------------------------------------------------*/
   /* addToolBarButtons
    */
   /**
    *    This method will add all of our specific
    *  toolbar buttons
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addToolBarButtons () {
      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 2 ) ;
   }





   /*------------------------------------------------------------*/
   /* processSave
    */
   /**
    *    This method will start processing the
    *  save
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean processSave () {
      boolean rc = false ;

      if ( save() ) {
         if ( PropertySystem.getBool ( PropertySystem.PROPERTY_SAVE_AND_CLOSE ) ) {
            parentWin.closePanel ( this, previousPanel ) ;
            rc = true ;
         } else
            saved = false ;
      } /* if */

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* saveAs
    */
   /**
    *    This method will basically create a question
    *  from the current question. The newly created
    *  question will be a completely new question
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveAs () {
      if ( saveData() ) {
         String newDefaultTitle = null ;
         Title  defaultTitle    = action.getDefaultTitle() ;                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, get the new default title                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         newDefaultTitle = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                                Str.getStr ( STR_TITLE ),
                                                Str.getStr ( STR_SAVE_AS_TITLE ),
                                                defaultTitle.toString(),
                                                "" ).getResult() ;                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, if they gave us a title, continue                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( newDefaultTitle != null ) && ( newDefaultTitle.length() > 0 ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, close the current Symptom                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            setMaintlock ( "N" ) ;
            setOpenDraft ( "N" ) ;                          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, create the new one and save it                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            action.resetAsNew() ;
            defaultTitle.setTitle ( newDefaultTitle ) ;
            if ( processSave() )
               GUISystem.printBox ( STR_INFO, STR_NEW_ACTION_SAVED_AS ) ;
         }
      }
   } /* saveAS  */



   /*------------------------------------------------------------*/
   /* reopenDraft
    */
   /**
    *    This method will reopen the draft we are
    *  editing by setting the stop date to null, it will
    *  also reset the MaintLock to 'Y'.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void reopenDraft () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "reopenDraft", 5 ) ;

      GUISystem.getParentDefWin ( this ).setStatus ( STR_READING ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.ACTION SET STOPDATE=NULL, MAINTLOCK='Y' "+
                              "WHERE ACTIONIND = "+action.getInd() ) ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;                     


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we must reload the object so that all values will  */
      /* now be update.                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      action = AvalonCache.readActionDraft ( action.getInd() ) ;

      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;

   }    
} /* ActionEditorPanel */


