package com.ibm.nzna.projects.qit.avalon.editors ;              
/*-----------------------------------------------------------*/
/*   ActionNodePanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionNodePanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel shows on the 'NodeEdit' panel. It allows the user 
 * to see things about the action 
 *
 * @version 1.0, 5/15/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionNodePanel                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionNodePanel extends JPanel implements NodePanel, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_ACTION     = null ;
   private JLabel       st_TITLE      = null ;
   private JLabel       st_ACTIONIND  = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int           actionInd    = 0 ;
   private boolean       showingDraft = false ;
   private ObjectNode    node         = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will construct our panel with
    *  no Question
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionNodePanel () {
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create our panel initialized
    *  to the Node passed
    *  
    *
    *  @param node The Node representing the Question to show information for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionNodePanel ( FlowNode node ) {
      initialize() ;
      if ( node instanceof ObjectNode )
         setNode ( (ObjectNode)node ) ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_ACTION      = new JLabel ( Str.getStr ( STR_ACTION ) ) ;
      st_ACTIONIND   = new JLabel ( "" ) ;
      st_TITLE       = new JLabel ( "" ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque               ( false ) ;
      st_ACTION.setOpaque     ( false ) ;
      st_TITLE.setOpaque      ( false ) ;
      st_ACTION.setFont       ( FontSystem.largeTitleFont ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_ACTION ) ;
      add ( st_TITLE ) ;
      add ( st_ACTIONIND ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of 
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = 5 ; 
      int       y         = 0 ; 

      st_ACTION.setBounds ( x, y, size.width-(x*2), rowHeight+8 ) ;
      y+=rowHeight+10 ;

      st_TITLE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      st_ACTIONIND.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

   }





   /*------------------------------------------------------------*/
   /* setNode
    */
   /**
    *    This method will set the node that we are
    *  currently editing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNode ( FlowNode node ) {                       
      if ( node instanceof ObjectNode )
         this.node = (ObjectNode)node ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      st_TITLE.setText ( getDefaultTitle () ) ;
      if ( node.isDraft() )
         st_ACTIONIND.setText ( ""+node.getNode().getDraftObjectInd() ) ;
      else
         st_ACTIONIND.setText ( ""+node.getNode().getActiveObjectInd() ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will get the default title of the
    *  object we are representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getDefaultTitle () {
      if ( node != null )
         return( node.getTitle() ) ;
      return( "Unknown" ) ;
   } /* getDefaultTitle */


   public void spellCheckComplete ( EditPanel editPanel, boolean errorsFound ) {
   }

} /* ActionNodePanel */
