package com.ibm.nzna.projects.qit.avalon.editors ;                
/*-----------------------------------------------------------*/
/*   ActionPreviewPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionPreviewPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.awt.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;       
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This preview panel is simply a Browser control. It handles
 * refreshing the information in the view as well as keeping 1
 * copy of the skins/Default/action_preview.html file to render
 *
 *
 * @version 1.0, 4/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionPreviewPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionPreviewPanel extends JPanel implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String         htmlTemplate         = null ;
   private        ActionDraft    action             = null ;
   private        Browser        previewPanel         = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will simply create a blank 
    *  preview window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionPreviewPanel () {
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* ActionPreviewPanel
    */
   /**
    *    This method will create a preview panel
    *  displaying the data from the Action
    *  object passed in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionPreviewPanel ( ActionDraft action ) {
      this.action = action ;
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         previewPanel = new Browser() ;                                   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Load in the htmlTemplate if it is not loaded            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( htmlTemplate == null ) {
            String filename = "skins"+File.separator+
                              PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                              File.separator+
                              "avalon"+
                              File.separator ;

            htmlTemplate = FileUtil.readFile ( filename+"action_preview.html" ) ;

            /* Copy over the background image  */
            QuestUtil.copyFileToTempDir ( filename+"empty_title.gif", "empty_title.gif" ) ;

         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our GUI                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setLayout ( new BorderLayout () ) ;
         add ( previewPanel, BorderLayout.CENTER ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Refresh our Data                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         refreshData() ;


      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

   } /* initialize */






   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh the data on our
    *  preview
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData () {
      String showHtml = null ;                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have the template, fill it in                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( htmlTemplate != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Plug in the 'Name' values                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         showHtml = Text.replaceInStr ( htmlTemplate, "STR_ADDITIONAL_INFO", Str.getStr ( STR_ADDITIONAL_INFORMATION ) ) ;
         showHtml = Text.replaceInStr ( showHtml,     "STR_GEO",             Str.getStr ( STR_GEOGRAPHY ) ) ;                                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have a action, plug in that action's values       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( action != null ) {
            showHtml = Text.replaceInStr ( showHtml, "TITLE",           getDefaultTitle() ) ;
            showHtml = Text.replaceInStr ( showHtml, "GEO",             getGeographyList() ) ;
            showHtml = Text.replaceInStr ( showHtml, "ADDITIONAL_INFO", getDefaultAdditionalInfo() ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, plug in blank values                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            showHtml = Text.replaceInStr ( showHtml, "TITLE",           Str.getStr ( STR_NONE ) ) ;
            showHtml = Text.replaceInStr ( showHtml, "GEO",             Str.getStr ( STR_NONE ) ) ;
            showHtml = Text.replaceInStr ( showHtml, "ADDITIONAL_INFO", Str.getStr ( STR_NONE ) ) ;
         }


      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, put up a default message                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else
         showHtml = "<html><body><center>"+Str.getStr ( STR_CANNOT_FIND_TEMPLATE )+"</center></body></html>" ;            


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, show the content                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      previewPanel.setPageContent ( showHtml ) ;

   }





   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will pull the default title from
    *  the ActionDraft that we are previewing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *  @see com.ibm.nzna.projects.common.quest.oa.ActionDraft
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getDefaultTitle () {
      Vector titleVec = action.getTitles() ;
      String retStr   = Str.getStr ( STR_NONE ) ;

      if ( ( titleVec != null ) && ( titleVec.size() > 0 ) )
         retStr = ((Title)titleVec.elementAt(titleVec.size()-1)).getTitle() ;

      return( retStr ) ;
   }


   /*------------------------------------------------------------*/
   /* getDefaultAddInfo
    */
   /**
    *    This method will pull the default title from
    *  the ActionDraft that we are previewing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *  @see com.ibm.nzna.projects.common.quest.oa.ActionDraft
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getDefaultAdditionalInfo () {
      Vector addInfoVec = action.getAdditionalInfo() ;
      String retStr     = Str.getStr ( STR_NONE ) ;

      if ( ( addInfoVec != null ) && ( addInfoVec.size() > 0 ) )
         retStr = ((AddtlInfo)addInfoVec.elementAt(addInfoVec.size()-1)).getAddtlInfo() ;

      return( retStr ) ;
   }





   /*------------------------------------------------------------*/
   /* getGeographyList
    */
   /**
    *    This method will format a user readable list
    *  of geographies
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getGeographyList () {
      Vector countryVec = action.getCountryList() ;
      String geoStr     = null ;

      if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = countryVec.size() ;

         while ( i < size ) {
            geoStr+=TypeList.getInstance().stringFromInd ( ((TypeCountryCodeRec)countryVec.elementAt(i)).geoInd,
                                                           TypeList.TYPE_GEOGRAPHY ) ;
            if ( i < (size-1) )
               geoStr+=", " ;
            i++ ;
         }
      }

      return( geoStr ) ;
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return ( false ) ;
   }

} /* ActionPreviewPanel */
