package com.ibm.nzna.projects.qit.avalon.editors ;           
/*-----------------------------------------------------------*/
/*   ActionUsagePanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionUsagePanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This panel shows what Symptoms an Action is used in.
 *
 *
 * @version 1.0, 11/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionUsagePanel                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionUsagePanel extends JPanel implements ListSelectionListener, Runnable, ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Column Headers                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String[] columns = { "", "", ""} ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private MultiList       symptomList     = null ;
   private JLabel          st_USAGE        = null ;
   private HotLinkLabel    pb_REFRESH      = null ;
   private ListActionPanel refreshPanel    = null ;   
   private ListActionPanel symptomPanel    = null ;
   private JTextArea       mle_OUTPUT      = null ;
   private JScrollPane     scr_OUTPUT      = null ;
   private HotLinkLabel    pb_EDIT_SYMPTOM = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean           refreshing          = false ;
   private boolean           refreshed           = false ;
   private ActionDraft       action              = null ;
   private Dimension         prefSize            = new Dimension ( 100, 400 ) ;
   private ActionEditorPanel actionEditorPanel   = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our controls,
    *  however the refresh will not happen until the 
    *  setVisible method is called.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionUsagePanel ( ActionEditorPanel actionEditorPanel ) {
      this.actionEditorPanel = actionEditorPanel ;
      initializeGUI () ;
   }




   /*------------------------------------------------------------*/
   /* intializeGUI
    */
   /**
    *    This method will initialize the GUI and add
    *  it to our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeGUI () {          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_USAGE        = new JLabel ( Str.getStr ( STR_ACTION_USAGE_PANEL ) ) ;
      symptomList     = new MultiList ( GUISystem.getFontUtil() ) ;
      pb_REFRESH      = new HotLinkLabel ( Str.getStr ( STR_REFRESH )+" "+Str.getStr ( STR_ALL ) ) ;
      refreshPanel    = new ListActionPanel() ;     
      mle_OUTPUT      = new JTextArea() ;
      scr_OUTPUT      = new JScrollPane ( mle_OUTPUT ) ;
      pb_EDIT_SYMPTOM = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      symptomPanel    = new ListActionPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_USAGE.setOpaque ( false ) ;  
      setListProperties ( symptomList ) ;
      mle_OUTPUT.setEditable ( false ) ;
      mle_OUTPUT.setFont ( FontSystem.defaultFont ) ;
      setBackground ( Color.white ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      refreshPanel.add ( pb_REFRESH ) ;
      symptomPanel.add ( pb_EDIT_SYMPTOM ) ;
      add ( st_USAGE ) ;
      add ( symptomList ) ;
      add ( symptomPanel ) ;
      add ( refreshPanel ) ;
      add ( scr_OUTPUT ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_REFRESH.addActionListener ( this ) ;
      pb_EDIT_SYMPTOM.addActionListener ( this ) ;
      symptomList.addActionListener ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      size.height = prefSize.height ;

      st_USAGE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      symptomList.setBounds ( x, y, size.width-(x*2), rowHeight*10 ) ;
      y+=(rowHeight*10) ;
      
      symptomPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=(rowHeight*2) ;

      scr_OUTPUT.setBounds ( x, y, size.width-(x*2), rowHeight*3 ) ;
      y+=(rowHeight*3) ;

      refreshPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=( rowHeight*2) ;

      super.doLayout() ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called once an action
    *  event is fired on us.
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) { 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REFRESH                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_REFRESH ) {
         if ( !refreshing ) {
            refreshed = false ;
            refresh() ;
         }
      } /* if  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_SYMPTOM                                         */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == symptomList ) || ( e.getSource() == pb_EDIT_SYMPTOM ) ) {
         UsageRec usageRec = (UsageRec)symptomList.getSelectedItem() ;

         if ( usageRec != null ) 
            OAEdit.edit ( usageRec, actionEditorPanel ) ;
      }
      
   } /* actionPerformed  */





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will kick off a thread to refresh
    *  our data. If the refresh thread has already been
    *  started then nothing happens
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refresh () {
      if ( !refreshing )
         new Thread ( this ).start() ;
   } /* refresh  */





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our refresh thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      StatusWin statusWin = new StatusWin ( GUISystem.getParentDefWin ( this ) ) ;

      try {
         if ( action != null ) {

            statusWin.setText ( Str.getStr ( STR_READING_USAGE ) ) ;

            symptomList.removeAll() ;
            symptomList.add ( ActionUsage.getUsage ( action ) ) ;

         }

         refreshed = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }
      statusWin.dispose() ;
      refreshing = false ;
   }




   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method is subclassed so that when we
    *  show ourselves for the first time we refresh
    *  our data
    *
    *  @param g The Graphics Context.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      super.paintComponent ( g ) ;

      if ( !refreshed ) {
         refresh() ;
      }
   }




   /*------------------------------------------------------------*/
   /* setAction
    */
   /**
    *    This method will set what action we are getting
    *  the usage for
    *  
    *
    *  @param actionDraft The action draft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setAction ( ActionDraft actionDraft ) {
      this.action = actionDraft ;
   }


   /*------------------------------------------------------------*/
   /* setListProperties
    */
   /**
    *    This method will set the properties on the
    *  MultiList passed. These properties include
    *  the column headings, column width and to turn off 
    *  the visible column headers
    *
    *  @param multiList The MultiList to set properties on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setListProperties ( MultiList multiList ) {
      multiList.setColumnHeadings ( columns ) ;
      multiList.setColumnWidth ( 0, 20 ) ;
      multiList.setColumnWidth ( 1, 75 ) ;
      multiList.setColumnWidth ( 2, 350 ) ;
      multiList.showColumnHeadings ( false ) ;
      multiList.addListSelectionListener ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when an item in
    *  our list is refreshed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( !e.getValueIsAdjusting() ) {
         if ( symptomList.getSelectedItem() != null ) {
            UsageRec usageRec = (UsageRec)symptomList.getSelectedItem() ;

            mle_OUTPUT.setText ( usageRec.getExplanation() ) ;
         } else
            mle_OUTPUT.setText ( "" ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our default preferred size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will determine the minimal amount
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }
} /* ActionUsagePanel */
