package com.ibm.nzna.projects.qit.avalon.editors ;                                         
/*-----------------------------------------------------------*/
/*   BrandFamilyMachineModelDlg                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BrandFamilyMachineModelDlg          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;

/**
 * This modal dialog will make the user select a single Machine/Model. It will 
 * allow the user to navigate through the BrandFamilyTree control and MachineModelTree
 * control.
 *
 * @version 1.0, 7/12/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BrandFamilyMachineModelDlg                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class BrandFamilyMachineModelDlg extends JDialog implements BookmarkDlgListener, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private MachineModelTree cnr_MACHINEMODEL     = null ;
   private BrandFamilyTree  cnr_BRANDFAMILY      = null ;
   private DButton          pb_OK                = null ;
   private DButton          pb_CANCEL            = null ;
   private JLabel           st_HELP              = null ;
   private ButtonPanel      buttonPanel          = null ;      
   private JScrollPane      scr_MACHINEMODEL     = null ;
   private JScrollPane      scr_BRANDFAMILY      = null ;
   private DButton          pb_BOOKMARK          = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ProductRec productRec = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will not show the dialog
    *  and will return immediatly.
    *  Please call 'getResults' to see what the user has selected
    *
    *  @param parent The parent window of this dialog
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BrandFamilyMachineModelDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_BFTM ), true ) ;

      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* getResults
    */
   /**
    *    This method will show the window and return
    *  the results from the user making a selection. If
    *  the user cancelled the window a null will be returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductRec getResults () {
      setVisible ( true ) ;
      return( productRec ) ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create our GUI, set the size
    *  and center our window.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {   
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_MACHINEMODEL = new MachineModelTree () ;
      cnr_BRANDFAMILY  = new BrandFamilyTree() ;
      pb_OK            = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL        = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      pb_BOOKMARK      = new DButton ( Str.getStr ( STR_BOOKMARK ) ) ;
      buttonPanel      = new ButtonPanel () ;
      st_HELP          = new JLabel ( Str.getStr ( STR_BFTM_DLG_HELP ), JLabel.CENTER ) ;
      scr_MACHINEMODEL = new JScrollPane ( cnr_MACHINEMODEL ) ;
      scr_BRANDFAMILY  = new JScrollPane ( cnr_BRANDFAMILY ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust params                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;  
      cnr_BRANDFAMILY.setMachineModelTree ( cnr_MACHINEMODEL ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_BOOKMARK.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_HELP ) ;
      contentPane.add ( scr_BRANDFAMILY ) ;
      contentPane.add ( scr_MACHINEMODEL ) ;
      contentPane.add ( buttonPanel ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_BOOKMARK ) ;
      buttonPanel.add ( pb_CANCEL ) ;

      GUISystem.setPropertiesOnPanel ( contentPane ) ;


      setSize ( 400, 300 ) ;
      WinUtil.centerChildInParent ( this, getParent ( )  ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       halfWidth = size.width/2 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_HELP.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      scr_BRANDFAMILY.setBounds ( x+3, y, halfWidth-5, size.height-87 ) ;
      scr_MACHINEMODEL.setBounds ( halfWidth+8, y, halfWidth-10-(x*2), size.height-87 ) ;
      buttonPanel.setBounds ( x, size.height-53, size.width-(x*2), 25 ) ;
   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are listening for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         productRec = null ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_BOOKMARK                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_BOOKMARK ) {
         new BookmarkDlg ( GUISystem.getParentDefWin ( this ), this ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_OK ) {
         int prodInt = cnr_MACHINEMODEL.getSelectedMachineModel() ;

         if ( prodInt != 0 ) {
            productRec = new ProductRec ( prodInt ) ;
            int       rc = productRec.readFromDatabase() ;

            if ( rc == 0 )
               dispose() ;
            else
               productRec = null ;
         }
      }
      

   }





   /*------------------------------------------------------------*/
   /* bookmarkSelected
    */
   /**
    *    This method will be called when the user
    *  has selected a bookmark
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void bookmarkSelected ( Vector bookmarkVec ) {
      if ( ( bookmarkVec != null ) && ( bookmarkVec.size() > 0 ) ) {
         Bookmark bookmark = (Bookmark)bookmarkVec.elementAt(0) ;

         if ( bookmark.getBookmarkType() == BOOKMARK_PRODUCT ) {
            productRec = new ProductRec ( ((Integer)bookmark.getObject()).intValue() ) ;

            if ( productRec.readFromDatabase() == 0 )
               dispose() ;
            else {
               GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                                    Str.getStr ( STR_INFO ),
                                    Str.getStr ( STR_PRODUCT_FROM_BOOKMARK_CANNOT_BE_READ ) ) ;
               productRec = null ;
            }
         }
      }
   }

} /* BrandFamilyMachineModelDlg */
