package com.ibm.nzna.projects.qit.avalon.editors ;
/*-----------------------------------------------------------*/
/*   ComponentNode                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ComponentNode                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.image.* ;
import java.util.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.oa.Action ;
import com.ibm.nzna.projects.common.quest.type.TypeList ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * The ComponentNode is a Component that represents a user interactive node. This component
 * allows the user to size and move the node.
 *
 *
 * @version 1.0, 4/19/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ComponentNode                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class ComponentNode extends Component implements  Runnable, FlowNode, FocusListener, AppConst, AvalonConst, MouseListener, MouseMotionListener, MultiListRow {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_FIRE_ACTION_EVENT = "FIREACTION" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int RESIZE_SE      = 1 ;
   private static final int RESIZE_NE      = 2 ;
   private static final int RESIZE_E       = 3 ;
   private static final int RESIZE_SW      = 4 ;
   private static final int RESIZE_NW      = 5 ;
   private static final int RESIZE_W       = 6 ;
   private static final int RESIZE_N       = 7 ;
   private static final int RESIZE_S       = 8 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Node          node                 = null ;
   private Rectangle     nodeBounds           = null ;
   private int           width                = 0 ;
   private int           height               = 0 ;
   private boolean       sizeDrag             = false ;
   private int           dragX                = 0 ;
   private int           dragY                = 0 ;
   private int           x                    = 0 ;
   private int           y                    = 0 ;
   private int           tailNodeTextY        = 0 ;
   private int           sizeDir              = 0 ;
   private Rectangle     oldBounds            = new Rectangle ( -1, -1, -1, -1 ) ;
   private String[]      nodeDescriptionArray = null ;
   private char[]        nodeIndArray         = null ;
   private char[]        draftArray           = null ;
   private FontUtil      fontUtil             = null ;
   private Vector        listeners            = null ;
   private boolean       isDraft              = false ;
   private AppDefaultWin parentWin            = null ;
   private String        nodeDescription      = null ;
   private FontMetrics   fm                   = Toolkit.getDefaultToolkit().getFontMetrics( FontSystem.defaultFont ) ;
   private int           charWidth            = fm.stringWidth ( "li" ) ;
   private boolean       root                 = false ;
   private ObjectNode    objectNode           = null ;
   private VisioPanel    visioPanel           = null ;
   private String        nodeIndString        = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Image focusImage              = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor, it requires
    *  a node.
    *  
    *
    *  @param node The node to represent in the data flow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ComponentNode ( Node node ) {
      setNode ( node ) ;
      initialize() ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a ComponentNode
    *  initialized to the ObjectNode specfied, and the
    *  Node that the ObjectNode represents
    *
    *  @param objectNode The ObjectNode to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ComponentNode ( ObjectNode objectNode ) {
      setObjectNode ( objectNode ) ;
      initialize() ;
   }


   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our component's
    *  properties
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      fontUtil = GUISystem.getFontUtil() ;
      refreshProperties() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our listeners                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addMouseMotionListener ( this ) ;
      addMouseListener ( this ) ;
      addFocusListener ( this ) ;

      draftArray = Str.getStr ( STR_DRAFT ).toCharArray() ;

   }






   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paint ( Graphics g ) {
      int y         = 15 ;
      int rowHeight = GUISystem.getRowHeight () ;
      int textX     = 5 ;

      g.setColor ( getBackground() ) ;
      if ( node != null ) {
         switch ( node.getNodeType() ) {                   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Actions have rounded corners                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_ACTION :
               g.fillRoundRect ( 0, 0, width, height, 28, 28 ) ;
               g.setColor ( Color.yellow ) ;
               g.drawRoundRect ( 0, 0, width, height, 28, 28 ) ;
               g.setColor ( getForeground() ) ;
               break ;          

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Symptoms are Diamonds                                   */            
               /*                                                           */
               /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_SYMPTOM :
               Polygon p = new Polygon() ;

               p.addPoint ( 0+(width/2), 0 ) ;
               p.addPoint ( 0+(width), 0+(height/2) ) ;
               p.addPoint ( 0+(width/2), 0+height ) ;
               p.addPoint ( 0, 0+(height/2) ) ;
               p.addPoint ( 0+(width/2), 0 ) ;

               g.fillPolygon ( p ) ;

               g.setColor ( Color.yellow ) ;
               g.drawPolygon ( p ) ;
               g.setColor ( getForeground().darker() ) ;
               g.drawRect ( 0, 0, width-1, height-1 ) ;
               g.setColor ( getForeground() ) ;
               textX = width/3 ;
               break ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Questions and ever0thing else are bo0es                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
            default:
            case Node.NODE_TYPE_QUESTION :
               g.fillRect ( 0, 0, width, height ) ;               

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Draw the border                                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( focusImage == null )
                  createFocusImage() ;

               if ( sizeDir == 0 )
                  g.drawImage ( focusImage, 0, 0, this ) ;
               break ;
         }
      } else {
         g.fillRect ( x, y, width, height ) ;               
         g.setColor ( getForeground() ) ;
         g.drawRect ( x, y, width, height ) ;               
      }

      g.setColor ( getForeground() ) ;

      if ( objectNode != null )
         objectNode.paintText ( g, 0,0,width,height ) ;


   }





   /*------------------------------------------------------------*/
   /* setNode
    */
   /**
    *    This method will set the node that this ComponentNode
    *  represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNode ( Node node ) {          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our variable                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.node = node ;                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our 'isDraft' property                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( node.getDraftObjectInd() == 0 )
         isDraft = false ;
      else
         isDraft = true ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our description from the ObjectNode                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      nodeDescriptionArray = objectNode.getNodeDescriptionArray() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get our bounds                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      nodeBounds = new Rectangle ( node.getXPos(), node.getYPos(), node.getWidth(), node.getHeight() ) ;
      setBounds ( nodeBounds ) ;                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign the correct indicator to show                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( isDraft() )
         nodeIndArray         = new String ( ""+node.getDraftObjectInd() ).toCharArray() ;
      else
         nodeIndArray         = new String ( ""+node.getActiveObjectInd() ).toCharArray() ;

      refreshProperties() ;
   }




   /*------------------------------------------------------------*/
   /* getNodeBounds
    */
   /**
    *    This method will return the bounds that we
    *  are supposed to be showing in. A rectangle
    *  will be returned.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return rect
    *//*---------------------------------------------------------*/
   protected Rectangle getNodeBounds () {
      nodeBounds.x = node.getXPos() ;
      nodeBounds.y = node.getYPos() ;
      nodeBounds.width = node.getWidth() ;
      nodeBounds.height = node.getHeight() ;
      return( nodeBounds ) ;
   }

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setSize subclassed to set variables                     */                  
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setSize ( int width, int height ) {
      if ( ( width > 25 ) && ( height > 25 ) ) {
         boolean widthChange  = false ;
         boolean heightChange = false ;

         oldBounds.width  = width ;
         oldBounds.height = height ;

         this.width = width ;
         this.height = height ;
         if ( node.valid() )
            node.setGuiData ( x, y, width, height, node.getZOrder() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Find out if our width/height has changed                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         widthChange  = (oldBounds.width  != width ) ;
         heightChange = (oldBounds.height != height ) ;


         if ( ( widthChange ) || ( heightChange ) )
            setNodeDescription ( nodeDescription ) ;

         super.setSize ( width, height ) ;
      }
   }



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setSize subclassed to set variables                     */                  
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setSize ( Dimension size ) {
      if ( ( size.width > 25 ) && ( size.height > 25 ) ) {
         boolean widthChange  = false ;
         boolean heightChange = false ;

         oldBounds.width  = size.width ;
         oldBounds.height = size.height ;


         this.width = size.width ;
         this.height = size.height ;
         if ( node.valid() )
            node.setGuiData ( x, y, width, height, node.getZOrder() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Find out if our width/height has changed                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         widthChange  = (oldBounds.width  != width ) ;
         heightChange = (oldBounds.height != height ) ;

         if ( ( widthChange ) || ( heightChange ) )
            setNodeDescription ( nodeDescription ) ;

         super.setSize ( size ) ;
      }
   }



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setBounds subclassed to store variables                 */                      
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setBounds ( int x, int y, int cx, int cy ) {
      if ( ( x >= 0 ) && ( y >= 0 ) && ( cx > 25 ) && ( cy > 25 ) ) {
         boolean widthChange  = false ;
         boolean heightChange = false ;


         if ( oldBounds.x == -1 )
            oldBounds.x=x ;
         else
            oldBounds.x=this.x ;
         if ( oldBounds.y == -1 )
            oldBounds.y=y ;
         else
            oldBounds.y=this.y ;

         oldBounds.width = width ;
         oldBounds.height = height ;

         this.width = cx ;
         this.height = cy ;
         this.y = y ;
         this.x = x ;                               


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Find out if our width/height has changed                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         widthChange  = (oldBounds.width  != width ) ;
         heightChange = (oldBounds.height != height ) ;


         if ( objectNode != null )
            objectNode.setBounds ( x, y, width, height ) ;

         if ( ( widthChange ) || ( heightChange ) )
            setNodeDescription ( nodeDescription ) ;

         super.setBounds ( x, y, cx, cy ) ;
      }
      if ( visioPanel != null ) {
         //visioPanel.repaint() ;
         visioPanel.revalidate() ;
         visioPanel.ensureVisible ( getBounds() ) ;
      }
   }



   /*-----------------------------------------------------------*/
   /*   mouseDragged                                            */           
   /*                                                           */
   /*   Description:                                            */
   /*      processes the mouse drag messsages sent              */                                                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      MouseEvent - The mouse event for us                  */                           
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void mouseDragged ( MouseEvent e ) {
      drag ( e.getX(), e.getY(), dragX, dragY ) ;
   }

   public void drag ( int mx, int my, int dragX, int dragY ) {

      sizeDrag = true ;
      this.dragX = dragX ;
      this.dragY = dragY ;

      /* if sizeDir is 0, then we are moving the
         component, otherwise we are sizing it */
      if ( sizeDir == 0 ) {

         mx+=x ;
         my+=y ;

         setLocation ( mx-dragX, my-dragY ) ;
      } else {

         switch ( sizeDir ) {
            case RESIZE_S :
               setBounds ( x, y, width, my+3 ) ;
               break ;

            case RESIZE_E :
               setBounds ( x, y, mx+3, height ) ;
               break ;

            case RESIZE_SE :
               setBounds ( x, y, mx+3, my+3 ) ;
               break ;

            case RESIZE_N :
               setBounds ( x, y+my, width, height-my ) ;
               break ;

            case RESIZE_W :
               setBounds ( x+mx, y, width-mx, height ) ;
               break ;

            case RESIZE_NW :
               setBounds ( x+mx, y+my, width-mx, height-my ) ;
               break ;

            case RESIZE_NE :
               setBounds ( x, y+my, mx+3, height-my ) ;
               break ;

            case RESIZE_SW :
               setBounds ( x+mx, y, width-mx, my+3 ) ;
               break ;

         }
      }

   }


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setLocation subclassed to store variables               */                        
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setLocation ( int x, int y ) {
      if ( ( x >= 0 ) && ( y >= 0 ) ) {
         if ( oldBounds.x == -1 )
            oldBounds.x=x ;
         else
            oldBounds.x=this.x ;
         if ( oldBounds.y == -1 )
            oldBounds.y=y ;
         else
            oldBounds.y=this.y ;

         this.x = x ;
         this.y = y ;

         if ( objectNode != null )
            objectNode.setLocation ( x, y ) ;

         node.updateRecStatus ( node.REC_UPDATE ) ;
         super.setLocation(x,y) ;

      }

      if ( visioPanel != null ) {
         //visioPanel.repaint() ;
         visioPanel.revalidate() ;
         visioPanel.ensureVisible ( getBounds() ) ;
      }
   }

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setLocation subclassed to store variables               */                        
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setLocation ( Point pt ) {

      if ( ( x >= 0 ) && ( y >= 0 ) ) {
         if ( oldBounds.x == -1 )
            oldBounds.x=pt.x ;
         else
            oldBounds.x=this.x ;
         if ( oldBounds.y == -1 )
            oldBounds.y=pt.y ;
         else
            oldBounds.y=this.y ;

         this.x = pt.x ;
         this.y = pt.y ;

         if ( objectNode != null )
            objectNode.setLocation ( x, y ) ;

         super.setLocation ( pt ) ;
      }
      if ( visioPanel != null ) {
         //visioPanel.repaint() ;
         visioPanel.revalidate() ;
         visioPanel.ensureVisible ( getBounds() ) ;
      }
   }

   /*-----------------------------------------------------------*/
   /*   mouseMoved                                              */         
   /*                                                           */
   /*   Description:                                            */
   /*      processes mouse move messages                        */                                              
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void mouseMoved ( MouseEvent e ) {
      int mx = e.getX() ;
      int my = e.getY() ;

      if ( ( mx < 5 ) && ( my < 5 ) ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.NW_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_NW ;
      } else if ( ( mx < 5 ) && ( my > height-5 ) ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.SW_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_SW ;
      } else if ( ( mx > width-5 ) && ( my < 5 ) ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.NE_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_NE ;
      } else if ( ( mx > width-5 ) && ( my > height-5 ) ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.SE_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_SE ;
      } else if ( mx < 5 ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.W_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_W ;
      } else if ( my < 5 ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.N_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_N ;
      } else if ( my > height-5 ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.S_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_S ;
      } else if ( mx > width-5 ) {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.E_RESIZE_CURSOR ) ) ;
         sizeDir = RESIZE_E ;
      } else {
         setCursor ( Cursor.getPredefinedCursor ( Cursor.DEFAULT_CURSOR ) ) ;
         sizeDir = 0 ;
      }

   }

   public void mouseReleased ( MouseEvent e ) {
      sizeDrag = false ;
      sizeDir  = 0 ;

      setBounds ( x, y, width, height ) ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Release our current focus image so the                  */
      /* new one draws                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( focusImage != null ) {
         focusImage.flush() ;
         focusImage = null ;
      }

   }

   public void mousePressed ( MouseEvent e ) {

      sizeDrag = true ;
      requestFocus() ;

      /* Mark the mouse location for use in the full drag */
      dragX = e.getX() ;
      dragY = e.getY() ;
   }

   public void mouseClicked ( MouseEvent e ) {
      if ( e.getClickCount() > 1 )
         new Thread ( this, THREAD_FIRE_ACTION_EVENT ).start() ;
      sizeDrag = false ;
   }

   public void mouseEntered ( MouseEvent e ) {
   }

   public void mouseExited ( MouseEvent e ) {
   }





   /*------------------------------------------------------------*/
   /* focusGained
    */
   /**
    *    This method will be called when our focus
    *  has been gained
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusGained ( FocusEvent e ) {
      repaint() ;
   }




   /*------------------------------------------------------------*/
   /* focusLost
    */
   /**
    *    This method will be called when our focus
    *  has been lost
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusLost ( FocusEvent e ) {
      repaint() ;
   }




   /*------------------------------------------------------------*/
   /* isFocusTraversable
    */
   /**
    *    This method will return TRUE since we want
    *  focus events
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isFocusTraversable () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* getNode
    */
   /**
    *    this method will return the node that this
    *  component represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Node getNode () {
      return( node ) ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title depending on what
    *  type of object this is. Currently titles are only returned for
    *  questions, actions and symptoms. If the title has not been read
    *  yet, the title will be read now.
    *  
    *
    *  @return String The title
    *//*---------------------------------------------------------*/
   public String getTitle () {
      if ( nodeDescription == null ) {
         if ( objectNode != null )
            setNodeDescription ( objectNode.getTitle() ) ;
         else
            setNodeDescription ( "????" ) ;
      }

      return( nodeDescription ) ;
   }




   /*------------------------------------------------------------*/
   /* isDraft
    */
   /**
    *    This method will return TRUE if this
    *  node is representing a Draft object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isDraft () {
      return( isDraft ) ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform a variety of functions,
    *  each function depends on the name of the
    *  thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_FIRE_ACTION_EVENT                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_FIRE_ACTION_EVENT ) )
         objectNode.fireActionEvent() ;

   }





   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener to this
    *  ObejctNode
    *  
    *
    *  @param listener The listener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      if ( listeners == null )
         listeners = new Vector ( 1 ) ;

      listeners.addElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* getDraftObject
    */
   /**
    *    This method will return the draft object that this
    *  object node represents. If there is no draft object
    *  available a NULL will be returned. This method also
    *  handles showing the user an error message indicating
    *  why a draft could not be created
    *
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getDraftObject () {
         return( objectNode.getDraftObject() ) ;
   }



   /*------------------------------------------------------------*/
   /* getObject
    */
   /**
    *    This method will return either a Draft Object
    *  or Active Object depending on what this node
    *  represents.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getObject () {
      if ( objectNode != null )
         return( objectNode.getObject() ) ;
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    The to string method will return the title of the
    *  object that this node represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( getTitle() ) ;
   }




   /*------------------------------------------------------------*/
   /* refreshProperties
    */
   /**
    *    This method will refresh all of the properties
    *  that we draw with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void refreshProperties () {
      try {

         setBackground ( (Color)PropertySystem.get ( PROPERTY_FLOWNODE_FOCUS_COLOR ) ) ;
         setForeground ( (Color)PropertySystem.get ( PROPERTY_FLOWNODE_FOCUS_TEXT_COLOR ) ) ;

      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* setNodeDescription
    */
   /**
    *    This method will set the node description
    *  that will be painted in the box to the 
    *  description passed in
    *
    *  @param descript The node description
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setNodeDescription ( String description ) {

      try {
         int propLength = PropertySystem.getInt ( PROPERTY_VISIO_TEXT_LENGTH ) ;

         if ( description.length() > propLength ) {
            description = description.substring ( 0, propLength ) ;
            description+="..." ;
         }

         this.nodeDescription = description ;

         if ( width > 0 ) {
            nodeDescriptionArray = Text.formatStringLines ( description, width/charWidth ) ;

            if ( objectNode != null )
               objectNode.setNodeDescriptionArray ( nodeDescriptionArray ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the status of the
    *  window that we belong to
    *  
    *
    *  @param status The string status
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setStatus ( String status ) {
      if ( parentWin == null )
         parentWin = GUISystem.getParentDefWin ( this ) ;

      if ( parentWin == null )
         parentWin = MainWindow.getInstance() ;

      if ( parentWin != null )
         parentWin.setStatus ( status ) ;
   }




   /*------------------------------------------------------------*/
   /* getCountryList
    */
   /**
    *    This method will return the Vector of countries
    *  that this node's data object represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getCountryList () {
      return( objectNode.getCountryList() ) ;
   }




   /*------------------------------------------------------------*/
   /* getDocClass
    */
   /**
    *    This method will return our doc class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getDocClass () {
      return( objectNode.getDocClass() ) ;
   }





   /*------------------------------------------------------------*/
   /* setRoot
    */
   /**
    *    This method will set wether the node in that
    *  is represented by this control is a root node. Please
    *  note that this is for display purposes only, this does not save to the database as root
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setRoot ( boolean root ) {
      this.root = root ;
   }




   /*------------------------------------------------------------*/
   /* getRoot
    */
   /**
    *    This method will return TRUE if
    *  we are root. Please note that this 'root' property
    *  is only valid for this component, and this component only uses it to determine drawing style.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected boolean getRoot () {
      return( root ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  that should be represented for a specific
    *  column in a MultiList
    *
    *  @param column The column number to return data for
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObject = null ;

      switch ( column ) {
         case 0 :
            retObject = ( ""+node.getInd() ) ;
            break ;

         case 1 :
            retObject = nodeDescription ;
            break ;
      }

      return( retObject ) ;
   }





   /*------------------------------------------------------------*/
   /* setTailNodeTextY
    */
   /**
    *    This method will set the Y position of the tail
    *  Node Text. This text is the text that is shown
    *  above the Object Node. The text has the answer text
    *
    *  @param int
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTailNodeTextY ( int y ) {
      tailNodeTextY = y ;
   }

   public int getTailNodeTextY ( ) {
      return( tailNodeTextY ) ;
   }





   /*------------------------------------------------------------*/
   /* getObjectNode
    */
   /**
    *    This method will return the ObjectNode tied
    *  to this ComponentNode
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ObjectNode
    *//*---------------------------------------------------------*/
   protected ObjectNode getObjectNode () {
      return( objectNode ) ;
   }




   /*------------------------------------------------------------*/
   /* setObjectNode
    */
   /**
    *    This method will set what ObjectNode that
    *  this component is referring to. Please note that
    *  by setting the ObjectNode that the node property is also set
    *
    *  @param objectNode The ObjectNode to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setObjectNode ( ObjectNode objectNode ) {                                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Tell our current Object Node to paint itself again      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( this.objectNode != null )
         this.objectNode.setPaint ( true ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our variables                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.nodeDescriptionArray = objectNode.getNodeDescriptionArray() ;                                         
      this.objectNode           = objectNode ;
      this.root                 = objectNode.getRoot() ;
      this.focusImage           = null ;
      this.nodeDescription      = objectNode.getTitle() ;
      oldBounds.x               = -1 ;
      oldBounds.y               = -1 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Turn off painting for the new object node, and tell     */
      /* it that we are now the ComponentNode                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      objectNode.setPaint ( false ) ;
      objectNode.setComponentNode ( this ) ;                                      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our node, this will refresh all of our data         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setNode ( objectNode.getNode() ) ;
   }






   /*------------------------------------------------------------*/
   /* setVisioPanel
    */
   /**
    *    This method will set the visio panel that this
    *  component will talk to when being dragged
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setVisioPanel ( VisioPanel visioPanel ) {
      this.visioPanel = visioPanel ;
   }


   /*------------------------------------------------------------*/
   /* createFocusImage
    */
   /**
    *    This method will re-create the focus image
    *  of dotted yellow lines around the box
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createFocusImage ( ) {
      Dimension prefSize = getSize() ;

      if ( prefSize.height >= 0 ) {
         int               i         = 0 ;
         MemoryImageSource imgSource = null ;
         int[]             imgBytes  = new int [ prefSize.width*prefSize.height ] ;
         int               yellow    = 0xffffff9c ;

         /* Draw the horizontal lines */
         while ( i < prefSize.width ) {
            imgBytes[i]=yellow ;
            imgBytes[i+(prefSize.width*(prefSize.height-1))]=yellow ;
            i+=2 ;
         }

         i=0 ;
         while ( i < (prefSize.height*(prefSize.width-1)) ) {
            imgBytes[i]=yellow ;
            imgBytes[i+prefSize.width-1]=yellow ;
            i+=prefSize.width*2 ;
         }

         imgSource = new MemoryImageSource ( prefSize.width, prefSize.height, imgBytes, 0, prefSize.width ) ;

         focusImage = this.createImage ( imgSource ) ;
      }

   }




   /*------------------------------------------------------------*/
   /* getSizeDrag
    */
   /**
    *    This method will return TRUE if the mouse
    *  is currently dragging this Component Node
    *  intending to increase the size of the node or intending
    *  to simply move the node
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean getSizeDrag () {
      return( sizeDrag ) ;
   }



} /* ComponentNode */
