package com.ibm.nzna.projects.qit.avalon.editors ;       
/*-----------------------------------------------------------*/
/*   DraftEditPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DraftEditPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This panel holds everything common to a draft object, this
 * includes the Start/Stop Date, Comments, Changed Time and
 * Last touched by, as well as Create Date. Status does not show
 * by default, you will have to call 'setShowStatus ( true )' to show the
 * status drop down
 *
 * @version 1.0, 5/7/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DraftEditPanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class DraftEditPanel extends EditPanel implements AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel             st_CHANGEDTIME        = null ;
   private JLabel             stt_CHANGEDTIME       = null ;
   private JLabel             st_CREATE_DATE        = null ;
   private JLabel             stt_CREATE_DATE       = null ;
   private JLabel             st_DBUSER             = null ;
   private JLabel             stt_DBUSER            = null ;
   private JLabel             st_STARTDATE          = null ;
   private JLabel             stt_STARTDATE         = null ;
   private JLabel             st_STOPDATE           = null ;
   private JLabel             stt_STOPDATE          = null ;
   private JLabel             st_COMMENTS           = null ;
   private JLabel             st_WORKREQ            = null ;
   private JLabel             stt_WORKREQ           = null ;
   private JLabel             st_DRAFT_INDICATOR    = null ;
   private JLabel             stt_DRAFT_INDICATOR   = null ;
   private JLabel             st_ACTIVE_INDICATOR   = null ;
   private JLabel             stt_ACTIVE_INDICATOR  = null ;
   private JTextArea          mle_COMMENTS          = null ;
   private JTextArea          mle_ORIGINAL_COMMENTS = null ;
   private JScrollPane        scr_COMMENTS          = null ;      
   private JTitle             st_OBJECT_TYPE        = null ;
   private JComboBox          cb_OBJECT_TYPE        = null ;
   private JComboBox          cb_BRAND              = null ;
   private JTitle             st_BRAND              = null ;
   private DocClassPanel      docClassPanel         = null ;
   private JComboBox          cb_STATUS             = null ;
   private JTitle             st_STATUS             = null ;
   private CountrySelectPanel countrySelectPanel    = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Dimension   prefSize        = new Dimension ( 400, 500 ) ;
   private OAInterface oaInterface     = null ;
   private boolean     showObjectTypes = true ;
   private boolean     showClass       = true ;
   private boolean     showBrand       = true ;
   private boolean     showStatus      = false ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DraftEditPanel ( OAObject oaInterface ) {
      initialize() ;
      setDraftObject ( oaInterface ) ;
   }

   public DraftEditPanel () {
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* setDraftObject
    */
   /**
    *    This method will set our draft object and will
    *  populate all of our fields
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDraftObject ( OAInterface oaInterface ) {
      try {
         this.oaInterface = oaInterface ;                          

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Put the regular DraftObject data in                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( oaInterface instanceof DraftObject ) {
            DraftObject draft = (DraftObject)oaInterface ;

            stt_WORKREQ.setText              ( draft.getTypeWorkRequired().toString() ) ;
            stt_STARTDATE.setText            ( draft.getStartDate() ) ;
            stt_STOPDATE.setText             ( draft.getStopDate() ) ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add the original comments, and a stamp for the          */
            /* new comments.                                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( draft.getComments() != null ) {
               if ( draft.getComments().length() == 0 )
                  mle_ORIGINAL_COMMENTS.setText    ( Str.getStr ( STR_ON )+" "+DateSystem.getDate ( CDate.DATE_DATE)+" "+UserSystem.getUserName()+" "+Str.getStr ( STR_WROTE )+":" ) ;
               else
                  mle_ORIGINAL_COMMENTS.setText    ( draft.getComments()+"\n"+Str.getStr ( STR_ON )+" "+DateSystem.getDate ( CDate.DATE_DATE)+" "+UserSystem.getUserName()+" "+Str.getStr ( STR_WROTE )+":" ) ;

               mle_COMMENTS.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 1023-mle_ORIGINAL_COMMENTS.getText().length() ) ) ;
            } else
               mle_COMMENTS.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 1023 ) ) ;
         } else
            System.out.println ( "Interface passed in NOT draft object" ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, put the extended information in                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( oaInterface instanceof OAObject ) {
            Vector   brandVec = new Vector ( 1, 1 ) ;
            OAObject oaObject = (OAObject) oaInterface ;                                             

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the user does not have the authority for the brand,  */
            /* then that brand will not show up in the list. We must     */
            /* the object's brand in the list though, so add it!         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            try {
               if ( !UserSystem.hasBrandAuth ( oaObject.getBrand().getInd() ) )
                  cb_BRAND.addItem ( oaObject.getBrand() ) ;
            } catch ( Exception e ) {
               // object has no brand, do not add anything
            }


            docClassPanel.setDocClass         ( oaObject.getDocClass() ) ;
            cb_BRAND.setSelectedItem          ( oaObject.getBrand() ) ;
            countrySelectPanel.setCountryList ( oaObject.getCountryList() ) ;
            stt_CREATE_DATE.setText           ( DateSystem.prettyDateFromStamp ( oaInterface.getCreationTime() ) ) ;
            stt_CHANGEDTIME.setText           ( DateSystem.prettyDateFromStamp ( oaInterface.getChangedTime() ) ) ;
            stt_DBUSER.setText                ( UserSystem.getNameFromUserId ( oaInterface.getDbUser() ) ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      Color  purple       = new Color ( 128, 0, 128 ) ;     
      JPanel commentPanel = new JPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_BRAND              = new JTitle ( Str.getStr ( STR_BRAND ) ) ;
      cb_BRAND              = new JComboBox ( UserSystem.getUserBrands() ) ;
      st_CHANGEDTIME        = new JLabel ( Str.getStr ( STR_LAST_TOUCHED_ON ) ) ;
      stt_CHANGEDTIME       = new JLabel ( "" ) ;
      st_CREATE_DATE        = new JLabel ( Str.getStr ( STR_CREATE_DATE ) ) ;
      stt_CREATE_DATE       = new JLabel ( "" ) ;
      st_DBUSER             = new JLabel ( Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
      stt_DBUSER            = new JLabel ( "" ) ;
      st_STARTDATE          = new JLabel ( Str.getStr ( STR_START_DATE ) ) ;
      stt_STARTDATE         = new JLabel ( "" ) ;
      st_STOPDATE           = new JLabel ( Str.getStr ( STR_STOP_DATE ) ) ;
      stt_STOPDATE          = new JLabel ( "" ) ;
      st_WORKREQ            = new JLabel ( Str.getStr ( STR_WORK_REQUIRED ) ) ;
      stt_WORKREQ           = new JLabel ( "" ) ;
      st_COMMENTS           = new JLabel ( Str.getStr ( STR_COMMENT ) ) ;
      stt_DRAFT_INDICATOR   = new JLabel ( "" ) ;
      st_DRAFT_INDICATOR    = new JLabel ( Str.getStr ( STR_DRAFT_INDICATOR ) ) ;
      stt_ACTIVE_INDICATOR  = new JLabel ( "" ) ;
      st_ACTIVE_INDICATOR   = new JLabel ( Str.getStr ( STR_ACTIVE_INDICATOR ) ) ;
      mle_ORIGINAL_COMMENTS = new JTextArea ( "" ) ;
      mle_COMMENTS          = new JTextArea ( "" ) ;
      scr_COMMENTS          = new JScrollPane ( commentPanel ) ;        
      st_OBJECT_TYPE        = new JTitle ( "" ) ;
      cb_OBJECT_TYPE        = new JComboBox() ;
      docClassPanel         = new DocClassPanel () ;
      st_STATUS             = new JTitle ( Str.getStr ( STR_STATUS ) ) ;
      cb_STATUS             = new JComboBox () ;
      countrySelectPanel    = new CountrySelectPanel () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mle_COMMENTS.setFont ( FontSystem.defaultFont ) ;
      mle_ORIGINAL_COMMENTS.setFont ( FontSystem.defaultFont ) ;
      st_CHANGEDTIME.setForeground ( purple ) ;
      st_CREATE_DATE.setForeground ( purple ) ;
      st_WORKREQ.setForeground ( purple ) ;
      st_DRAFT_INDICATOR.setForeground ( purple ) ;
      st_ACTIVE_INDICATOR.setForeground ( purple ) ;
      st_DBUSER.setForeground ( purple ) ;
      st_STARTDATE.setForeground ( purple ) ;
      st_STOPDATE.setForeground ( purple ) ;
      scr_COMMENTS.setBorder ( GUISystem.blackBorder ) ;   
      setBackground ( Color.white ) ;
      st_OBJECT_TYPE.setFont ( FontSystem.smallTitleFont ) ;
      st_BRAND.setFont ( FontSystem.smallTitleFont ) ;
      st_STATUS.setFont ( FontSystem.smallTitleFont ) ;
      mle_ORIGINAL_COMMENTS.setEditable ( false ) ;
      mle_ORIGINAL_COMMENTS.setBackground ( new Color ( 230, 230, 230 ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      commentPanel.setLayout ( new BorderLayout() ) ;
      commentPanel.add ( mle_ORIGINAL_COMMENTS, BorderLayout.NORTH ) ;
      commentPanel.add ( mle_COMMENTS, BorderLayout.CENTER ) ;
      add ( st_OBJECT_TYPE ) ;
      add ( cb_OBJECT_TYPE ) ;
      add ( st_BRAND ) ;
      add ( cb_BRAND ) ;
      add ( docClassPanel ) ;
      add ( st_STATUS ) ;
      add ( cb_STATUS ) ;
      add ( st_COMMENTS ) ;
      add ( scr_COMMENTS ) ;
      add ( st_DRAFT_INDICATOR ) ;
      add ( stt_DRAFT_INDICATOR ) ;
      add ( st_ACTIVE_INDICATOR ) ;
      add ( stt_ACTIVE_INDICATOR ) ;
      add ( st_WORKREQ ) ;
      add ( stt_WORKREQ ) ;
      add ( st_CREATE_DATE ) ;
      add ( stt_CREATE_DATE ) ;
      add ( st_CHANGEDTIME ) ;
      add ( stt_CHANGEDTIME ) ;
      add ( st_DBUSER ) ;
      add ( stt_DBUSER ) ;
      add ( st_STARTDATE ) ;
      add ( stt_STARTDATE ) ;
      add ( st_STOPDATE ) ;
      add ( stt_STOPDATE ) ;                     
      add ( countrySelectPanel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Auto-select the user's preferred brand                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         TypeBrandRec   brandRec = Brands.getBrand ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEFAULT_BRAND ) ) ;

         if ( brandRec != null )
            cb_BRAND.setSelectedItem ( brandRec ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e , false ) ;
      }


   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout () ;

      countrySelectPanel.setBounds ( 340, y, size.width-345, rowHeight*6 ) ;

      if ( showObjectTypes ) {
         st_OBJECT_TYPE.setBounds ( x, y, 300, rowHeight ) ;
         y+=rowHeight ;
         cb_OBJECT_TYPE.setBounds ( x, y, 300, rowHeight ) ;
         y+=rowHeight ;
      }

      if ( showBrand ) {
         st_BRAND.setBounds ( x, y, 300, rowHeight ) ;
         y+=rowHeight ;
         cb_BRAND.setBounds ( x, y, 300, rowHeight ) ;
         y+=rowHeight ;
      }

      if ( showClass ) {
         docClassPanel.setBounds ( x, y, 300, (rowHeight*2)-1 ) ;
         y+=(rowHeight*2) ;
      }

      if ( showStatus ) {
         st_STATUS.setBounds ( x, y, 300, rowHeight ) ;
         y+=rowHeight ;
         cb_STATUS.setBounds ( x, y, 300, rowHeight ) ;
         y+=rowHeight ;
      }

      //y=5+(rowHeight*8) ;
      st_COMMENTS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      scr_COMMENTS.setBounds ( x, y, size.width-(x*2), size.height-y-(rowHeight*8) ) ;
      y=size.height-(rowHeight*7) ;

      st_ACTIVE_INDICATOR.setBounds ( x, y, 200, rowHeight ) ;
      stt_ACTIVE_INDICATOR.setBounds ( x+200, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_DRAFT_INDICATOR.setBounds ( x, y, 200, rowHeight ) ;
      stt_DRAFT_INDICATOR.setBounds ( x+200, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_WORKREQ.setBounds ( x, y, 200, rowHeight ) ;
      stt_WORKREQ.setBounds ( x+200, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_CREATE_DATE.setBounds ( x, y, 200, rowHeight ) ;
      stt_CREATE_DATE.setBounds ( x+200, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_CHANGEDTIME.setBounds ( x, y, 200, rowHeight ) ;
      stt_CHANGEDTIME.setBounds ( x+200, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_DBUSER.setBounds ( x, y, 200, rowHeight ) ;
      stt_DBUSER.setBounds ( x+200, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_STARTDATE.setBounds ( x, y, 200, rowHeight ) ;
      stt_STARTDATE.setBounds ( x+200, y, 200, rowHeight ) ;
      y+=rowHeight ;

      st_STOPDATE.setBounds ( x, y, 200, rowHeight ) ;
      stt_STOPDATE.setBounds ( x+200, y, 200, rowHeight ) ;
      y+=rowHeight ;
   }



   /*------------------------------------------------------------*/
   /* showObjectTypes
    */
   /**
    *    This method will either hide or show the object
    *  types label and combo box
    *  
    *
    *  @param show TRUE = show, FALSE = hide
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setShowObjectTypes ( boolean show ) {
      st_OBJECT_TYPE.setVisible ( show ) ;
      cb_OBJECT_TYPE.setVisible ( show ) ;
      this.showObjectTypes = show ;
      revalidate() ;
   }



   /*------------------------------------------------------------*/
   /* showDocClass
    */
   /**
    *    This method will either hide or show the class
    *  label and combo box
    *  
    *
    *  @param show TRUE = show, FALSE = hide
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setShowDocClass ( boolean show ) {
      docClassPanel.setVisible ( show ) ;
      if ( show )
         docClassPanel.setDocClass ( TypeList.DOCCLASS_IBM_CONFIDENTIAL ) ;
      this.showClass = show ;
      revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* showBrand
    */
   /**
    *    This method will turn the brand fields
    *  on and off
    *  
    *
    *  @param show TRUE = show brand drop down, FALSE = do not show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setShowBrand ( boolean show ) {
      st_BRAND.setVisible ( show ) ;
      cb_BRAND.setVisible ( show ) ;
      this.showBrand = show ;
      revalidate() ;
   }


   /*------------------------------------------------------------*/
   /* setObjectType
    */
   /**
    *    This method will set what list is displayed to the
    *  user in the drop down object type list
    *  
    *
    *  @param typeInd - The Type indicator for the typelist as defined in com.ibm.nzna.projects.common.quest.type.TypeList
    *  @see  com.ibm.nzna.projects.common.quest.type.TypeList
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setObjectType ( int typeList ) {

      TypeList.getInstance().refreshList ( cb_OBJECT_TYPE, typeList, null ) ;

      switch ( typeList ) {
         case TypeList.TYPE_LINK :
            st_OBJECT_TYPE.setText ( Str.getStr ( STR_LINK_TYPE ) ) ;
            break ;

         case TypeList.TYPE_QUESTION :
            st_OBJECT_TYPE.setText ( Str.getStr ( STR_QUESTION_TYPE ) ) ;
            break ;

         case TypeList.TYPE_SYMPTOM :
            st_OBJECT_TYPE.setText ( Str.getStr ( STR_SYMPTOM_TYPE ) ) ;
            break ;

         case TypeList.TYPE_ACTION :
            st_OBJECT_TYPE.setText ( Str.getStr ( STR_ACTION_TYPE ) ) ;
            break ;
      }
   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save whatever data is on
    *  our screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if data was saved, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      boolean rc = true ;

      ((DraftObject)oaInterface).setComments ( mle_COMMENTS.getText() ) ;

      if ( oaInterface instanceof OAObject ) {
         ((OAObject)oaInterface).setCountryList ( countrySelectPanel.getCountryList() ) ;

         if ( showClass ) {
            if ( getDocClass() != null )
               ((OAObject)oaInterface).setDocClass    ( getDocClass() ) ;
            else
               rc = false ;
         }

      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* getSelectedObjectType
    */
   /**
    *    This method will return whatever is selected
    *  in the object type drop down
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected Object getSelectedObjectType () {
      return( cb_OBJECT_TYPE.getSelectedItem() );
   }




   /*------------------------------------------------------------*/
   /* setSelectedObjectType
    */
   /**
    *    This method will set what type of object
    *  that this represents.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setSelectedObjectType ( Object selObject ) {
      cb_OBJECT_TYPE.setSelectedItem ( selObject ) ;
   }




   /*------------------------------------------------------------*/
   /* getBrand
    */
   /**
    *    This method will return the typeBrand
    *  object that we have selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected TypeBrandRec getBrand () {
      TypeBrandRec brandRec = (TypeBrandRec) cb_BRAND.getSelectedItem() ;

      if ( brandRec == null )
         GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_BRAND ) ;

      return( brandRec ) ;
   }

   /*------------------------------------------------------------*/
   /* getDocClass
    */
   /**
    *    This method will return the typeClass
    *  object that we have selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected TypeDocClassRec getDocClass () {
      TypeDocClassRec docClassRec = docClassPanel.getDocClass() ;

      if ( docClassRec == null )
         GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_DOC_CLASS ) ;

      return( docClassRec  ) ;
   }




   /*------------------------------------------------------------*/
   /* setShowStatus
    */
   /**
    *    This method will show the status combo boxes
    *  or hide the boxes
    *  
    *
    *  @param show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setShowStatus ( boolean show ) {
      this.showStatus = show ;
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* setShowCountry
    */
   /**
    *    This method will hide/show the country
    *  list
    *  
    *
    *  @param show TRUE = show countries, FALSE = do not show countries
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setShowCountry ( boolean show ) {
      countrySelectPanel.setVisible ( show ) ;
      revalidate() ;
   }



   /*------------------------------------------------------------*/
   /* setStatusData
    */
   /**
    *    This method will populate the drop down
    *  of status with the vector passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setStatusData ( Vector statusVec ) {
      if ( statusVec != null ) {
         int i    = 0 ;
         int size = statusVec.size() ;

         while ( i < size ) {
            cb_STATUS.addItem ( statusVec.elementAt(i) ) ;
            i++ ;
         } /* while */

      } /* if */

   } /* setStatusData */





   /*------------------------------------------------------------*/
   /* getStatusData
    */
   /**
    *    This method will return the status that
    *  the status combo box has selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   protected TypeStatusRec getStatusData () {
      return( (TypeStatusRec)cb_STATUS.getSelectedItem() ) ;
   }




   /*------------------------------------------------------------*/
   /* setSelectedStatus
    */
   /**
    *    This method will set what item is selected
    *  in the status drop down
    *  
    *
    *  @param statusRec The TypeStatusRec to select
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setSelectedStatus ( TypeStatusRec statusRec ) {
      if ( statusRec != null ) {
         cb_STATUS.setSelectedItem ( statusRec ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getOriginalComments
    */
   /**
    *    This method will return the 'original comments'
    *  that this draft was edited by. This value may
    *  be NULL
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getOriginalComments () {
      return( mle_ORIGINAL_COMMENTS.getText() ) ;
   }




   /*------------------------------------------------------------*/
   /* getComments
    */
   /**
    *    This method will only return the newly created
    *  comments that the user has entered since
    *  this object was last saved
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getComments () {
      return( mle_COMMENTS.getText() ) ;
   }




   /*------------------------------------------------------------*/
   /* addCountrySelectListener
    */
   /**
    *    This method will add a country select listener
    *  to the CountrySelectPanel that this DraftEditPanel
    *  contains. All notifiations of changes to countries will be sent to that listener
    *
    *  @param listener The CountrySelectListener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCountrySelectListener ( CountrySelectListener listener ) {
      countrySelectPanel.addCountrySelectListener ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* removeCountrySelectListener
    */
   /**
    *    This method will remove the CountrySelectListener
    *  that is passed in from our CountrySelectPanel
    *  
    *
    *  @param listener The CountrySelectListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCountrySelectListener ( CountrySelectListener listener ) {
      countrySelectPanel.removeCountrySelectListener ( listener ) ;
   }


   /*------------------------------------------------------------*/
   /* getCountrySelectPanel
    */
   /**
    *    This method will return an instance of this
    *  panel's CountrySelectPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected CountrySelectPanel getCountrySelectPanel () {
      return( countrySelectPanel ) ;
   }


   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }

   public Dimension getMinimumSize()  {
      return( getPreferredSize() ) ;
   }




   /*------------------------------------------------------------*/
   /* setBrandEnabled
    */
   /**
    *    This method will either enable or disable the
    *  brand combo box
    *  
    *
    *  @param enabled TRUE = enable the drop down, FALSE = disable it
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setBrandEnabled ( boolean enabled ) {
      cb_BRAND.setEnabled ( enabled ) ;
   }





   /*------------------------------------------------------------*/
   /* setDraftIndicator
    */
   /**
    *    This method will set the value that is shown
    *  next to the draft indicator field
    *  
    *
    *  @param draftIndicator The value to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDraftIndicator ( String draftIndicator ) {
      stt_DRAFT_INDICATOR.setText ( draftIndicator );
   }

   /*------------------------------------------------------------*/
   /* setActiveIndicator
    */
   /**
    *    This method will set the value that is shown
    *  next to the active indicator field
    *  
    *
    *  @param activeIndicator The value to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setActiveIndicator ( String activeIndicator ) {
      stt_ACTIVE_INDICATOR.setText ( activeIndicator );
   }
} /* ActiveEditPanel */
