package com.ibm.nzna.projects.qit.avalon.editors ;
/*-----------------------------------------------------------*/
/*   EdgeDlg                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EdgeDlg                             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This dialog holds all answers to the question passed, 
 * and all nodes in the symptom passed, except for the head node passed.
 * It allows the user to assign an answer, and tail node to an edge.
 * The getResult() method will then return a newly created Edge if the user
 * created one, and NULL if the user cancelled
 *
 * @version 1.0, 5/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EdgeDlg                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class EdgeDlg extends JDialog implements WindowListener, ActionListener, Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESH = "REFRESH" ;     

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] columnHeadingsAnswers={"","","","","",""} ;
   private static final String[] columnHeadingsNodes={"",""} ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_IMAGE          = null ;
   private JLabel      st_QUESTION_TITLE = null ;
   private JLabel      st_ANSWERS        = null ;
   private MultiList   cnr_ANSWERS       = null ;
   private MultiList   cnr_NODES         = null ;
   private DButton     pb_OK             = null ;
   private DButton     pb_CANCEL         = null ;
   private DButton     pb_HELP           = null ;
   private ButtonPanel buttonPanel       = null ;      
   private JScrollPane scr_ANSWERS       = null ;
   private DButton     pb_OTHER          = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ImageIcon    wizardImage   = null ;
   private ObjectNode   headNode      = null ;
   private Vector       objectNodes   = null ;
   private Question     question      = null ;
   private Edge         edge          = null ;
   private Symptom      symptom       = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our dialog in entire, but
    *  will not show the dialog. please use 'getResult'
    *  to show the dialog
    *
    *  @param parent The parent frame of this dialog
    *  @param headNode The ObjectNode representing the starting point of the edge
    *  @param objectNodes A list of all ObjectNodes available to the user
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EdgeDlg ( Frame parent, Symptom symptom, ObjectNode headNode, Vector objectNodes, boolean showOther  ) {
      super ( parent, Str.getStr ( STR_NEW_EDGE ), true ) ;
      Container contentPane = getContentPane() ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our local vars                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.symptom     = symptom ;
      this.objectNodes = objectNodes ;
      this.headNode    = headNode ;
      objectNodes.removeElement ( headNode ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardImage       = ImageSystem.getImageIcon ( this, ImageSystem.EDGE_WIZARD ) ;
      st_IMAGE          = new JLabel ( wizardImage ) ;
      st_QUESTION_TITLE = new JLabel () ;
      st_ANSWERS        = new JLabel ( Str.getStr ( STR_ANSWERS ) ) ;
      cnr_NODES         = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_ANSWERS       = new MultiList ( GUISystem.getFontUtil() ) ;
      buttonPanel       = new ButtonPanel () ;
      pb_OK             = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL         = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      pb_HELP           = new DButton ( Str.getStr ( STR_HELP ) ) ;
      pb_OTHER          = new DButton ( Str.getStr ( STR_OTHER ) ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_HELP.addActionListener ( this ) ;
      pb_OTHER.addActionListener ( this ) ;
      addWindowListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_QUESTION_TITLE ) ;
      contentPane.add ( st_ANSWERS ) ;
      contentPane.add ( cnr_ANSWERS ) ;
      contentPane.add ( cnr_NODES ) ;
      contentPane.add ( buttonPanel ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_OTHER ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      buttonPanel.add ( pb_HELP ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPropertiesOnPanel ( contentPane ) ;
      st_QUESTION_TITLE.setFont ( FontSystem.smallTitleFont ) ;
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;
      pb_OTHER.setEnabled ( showOther ) ;
      cnr_ANSWERS.setColumnHeadings ( columnHeadingsAnswers ) ;
      cnr_ANSWERS.setColumnWidth ( 0, 1 ) ;
      cnr_ANSWERS.setColumnWidth ( 1, 1 ) ;
      cnr_ANSWERS.setColumnWidth ( 2, 80 ) ;
      cnr_ANSWERS.setColumnWidth ( 3, 300 ) ;
      cnr_ANSWERS.setColumnWidth ( 4, 150 ) ;
      cnr_ANSWERS.setColumnWidth ( 5, 150 ) ;
      cnr_ANSWERS.showColumnHeadings ( false ) ;
      cnr_NODES.setColumnHeadings ( columnHeadingsNodes ) ;
      cnr_NODES.setColumnWidth ( 0, 80 ) ;
      cnr_NODES.setColumnWidth ( 1, 300 ) ;
      cnr_NODES.showColumnHeadings ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Center our window                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 525, 325 ) ;
      WinUtil.centerWindow ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the dialog and will
    *  not return until the user closes the window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if the user hit Ok, FALSE if user cancelled the window
    *//*---------------------------------------------------------*/
   public boolean getResult () {

      setVisible ( true ) ;

      return( edge != null ) ;
   }

   public Edge getEdge () {
      return( edge ) ;
   }

   public ObjectNode getTailNode () {
      return( (ObjectNode)cnr_NODES.getSelectedItem() ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = wizardImage.getImage().getWidth(this)+10 ;
      int       y         = ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2) ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           y, 
                           wizardImage.getImage().getWidth  ( this ), 
                           wizardImage.getImage().getHeight ( this ) ) ;

      st_QUESTION_TITLE.setBounds ( x, y, size.width-x-10, rowHeight ) ;
      y+=rowHeight ;

      st_ANSWERS.setBounds ( x, y, size.width-x-10, rowHeight ) ;
      y+=rowHeight ;

      cnr_ANSWERS.setBounds ( x, y, size.width-x-10, rowHeight*5 ) ;
      y+=(rowHeight*5)+5 ;

      cnr_NODES.setBounds ( x, y, size.width-x-10, rowHeight*5 ) ;

      buttonPanel.setBounds ( x, size.height-78, size.width-x, 25 ) ;

   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will populate all of our lists
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {

      enableControls ( false ) ;
      st_QUESTION_TITLE.setText ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
      setTitle ( Str.getStr ( STR_PLEASE_WAIT ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have no question. try and read it now             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      question    = (Question) headNode.getObject() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, indicate we are reading                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_QUESTION_TITLE.setText ( headNode.toString() ) ;

      cnr_NODES.add ( getValidObjectNodes ( objectNodes ) ) ;
      cnr_ANSWERS.add ( AnswerDisplayRec.convert ( question.getAnswers() ) ) ;

      setTitle ( Str.getStr ( STR_NEW_EDGE ) ) ;
      enableControls ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* enableControls
    */
   /**
    *    This method will either enable or disable the
    *  controls on our window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void enableControls ( boolean enable ) {
      st_IMAGE.setEnabled ( enable ) ;
      st_QUESTION_TITLE.setEnabled ( enable ) ;
      st_ANSWERS.setEnabled ( enable ) ;
      cnr_ANSWERS.setEnabled ( enable ) ;
      cnr_NODES.setEnabled ( enable ) ;
      pb_OK.setEnabled ( enable ) ;
      pb_CANCEL.setEnabled ( enable ) ;
      pb_HELP.setEnabled ( enable ) ;
      pb_OTHER.setEnabled ( enable ) ;
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will perform a variety of operations.
    *  The operation done is dependant on the name
    *  of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_REFRESH ) )
         refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_OK ) {
         if ( saveData ( false ) )
            dispose() ;
      }

      else if ( e.getSource() == pb_OTHER ) {
         if ( saveData ( true ) )
            dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CANCEL ) {
         edge = null ;
         dispose() ;
      }


   }






   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method wil ltry to create an Edge from
    *  the selected items
    *  
    *
    *  @param other TRUE if this edge should be a #OTHER edge
    *  @param None
    *  @param None
    *
    *  @return TRUE if an edge was created
    *//*---------------------------------------------------------*/
   private boolean saveData ( boolean other ) {
      AnswerDisplayRec  answerDisplay = (AnswerDisplayRec) cnr_ANSWERS.getSelectedItem() ;
      ObjectNode        tailNode      = (ObjectNode) cnr_NODES.getSelectedItem() ;                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If there is a valid node selected, make sure            */
      /* it is not the root node. If it is not, continue           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( tailNode != null ) && ( tailNode.getNode() != symptom.getRootNode() ) ) {                              

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is a #OTHER edge, create it noe                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( other )
            edge = new Edge ( question ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, make sure we have a good answer              */
         /* and then create it.                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( answerDisplay != null ) && ( tailNode != null ) )
            edge = new Edge ( question, (Answer)answerDisplay.getData() ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have an edge created, set the default values      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( edge != null ) {
            edge.setDocClass ( (TypeDocClassRec)TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL, TypeList.TYPE_DOCCLASS ) ) ;
            edge.setCountryList ( UserSystem.getUserCountries ( ) ) ;

            if ( answerDisplay != null )
               edge.setDefaultAnswerTitle ( answerDisplay.getData().toString() ) ;
            else
               edge.setDefaultAnswerTitle ( Str.getStr ( STR_OTHER ) ) ;
         }
      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_VALID_NODE_NOT_ROOT ) ;

      return( edge != null ) ;
   }

   public void windowDeactivated ( WindowEvent e ) {}
   public void windowActivated ( WindowEvent e ) {}
   public void windowOpened ( WindowEvent e ) {new Thread ( this, THREAD_REFRESH ).start();}
   public void windowClosed ( WindowEvent e ) {}
   public void windowClosing ( WindowEvent e ) {}
   public void windowIconified ( WindowEvent e ) {}
   public void windowDeiconified ( WindowEvent e ) {}






   /*------------------------------------------------------------*/
   /* getValidObjectNodes
    */
   /**
    *    This method will return all nodes that should
    *  appear in our list from the Vector passed in
    *  
    *
    *  @param vec A Vector of ObjectNodes
    *  @param None
    *  @param None
    *
    *  @return Vector A Vector of Object Nodes
    *//*---------------------------------------------------------*/
   private Vector getValidObjectNodes ( Vector vec ) {
      Vector returnVec = new Vector ( 1 ) ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i    = 0 ;
         int size = vec.size() ;

         while ( i < size ) {
            if ( ((ObjectNode)vec.elementAt(i)).getNode().valid() )
               returnVec.addElement ( vec.elementAt(i) ) ;
            i++ ;
         }
      }

      new QuickSort ( returnVec ) ;

      return( returnVec ) ;
   }

} /* EdgeDlg */
