package com.ibm.nzna.projects.qit.avalon.editors ;     
/*-----------------------------------------------------------*/
/*   EdgeEditPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EdgeEditPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This panel allows the user to edit everything about an edge. 
 *
 *
 * @version 1.0, 5/19/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EdgeEditPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class EdgeEditPanel extends EditPanel implements PreanswerDlgListener, SelectOAListener, ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle             st_DOC_CLASS           = null ;
   private JTitle             st_ACTION_LIST         = null ;
   private JTitle             st_PRE_ANSWERS         = null ;
   private JTitle             st_UNANSWER            = null ;
   private JTitle             st_EDGE_TITLE          = null ;
   private JComboBox          cb_DOC_CLASS           = null ;
   private JList              lb_ACTION_LIST         = null ;
   private JList              lb_PRE_ANSWERS         = null ;
   private JList              lb_UNANSWER            = null ;
   private ButtonPanel        buttonPanel_ActionList = null ;
   private ButtonPanel        buttonPanel_PreAnswers = null ;
   private ButtonPanel        buttonPanel_UnAnswer   = null ;      
   private ButtonPanel        buttonPanel_Command    = null ;
   private JScrollPane        scr_ACTION_LIST        = null ;
   private JScrollPane        scr_PRE_ANSWERS        = null ;
   private CountrySelectPanel lb_COUNTRIES           = null ;
   private JScrollPane        scr_UNANSWER           = null ;
   private ConditionList      conditionList          = null ;
   private HotLinkLabel       pb_ADD_ACTION          = null ;
   private HotLinkLabel       pb_REMOVE_ACTION       = null ;
   private HotLinkLabel       pb_ADD_PREANSWER       = null ;
   private HotLinkLabel       pb_REMOVE_PREANSWER    = null ;
   private HotLinkLabel       pb_ADD_UNANSWER        = null ;
   private HotLinkLabel       pb_REMOVE_UNANSWER     = null ;
   private HotLinkLabel       pb_SAVE                = null ;
   private HotLinkLabel       pb_CANCEL              = null ;
   private JTitle             st_EDGE_RETURN         = null ;
   private JComboBox          cb_EDGE_RETURN         = null ;
   private JCheckBox          ck_EDGE_RETURN         = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Edge      edge                 = null ;
   private Dimension prefSize             = new Dimension ( 0, 470 ) ;
   private Vector    actionList           = null ;
   private Vector    preanswerList        = null ;
   private Vector    unanswerList         = null ;
   private Vector    objectNodes          = null ;
   private boolean   allowEdgeReturn      = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a new EditEditPanel
    *  iniitalized to the edge passed in
    *  
    *
    *  @param edge The Edge to edit
    *  @param symptom The Symptom that the edge belongs to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EdgeEditPanel ( Edge edge, Vector objectNodes, boolean allowEdgeReturn ) {
      this.edge             = edge ;
      this.objectNodes      = objectNodes ;
      this.allowEdgeReturn  = allowEdgeReturn ;
      initialize() ;
      refreshData() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     
      Dimension tbSize = new Dimension ( 1, GUISystem.getRowHeight() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_DOC_CLASS           = new JTitle ( Str.getStr ( STR_DOC_CLASS ) ) ;
      st_ACTION_LIST         = new JTitle ( Str.getStr ( STR_ACTION_LIST ) ) ;
      st_PRE_ANSWERS         = new JTitle ( Str.getStr ( STR_PRE_ANSWERS ) ) ;
      st_UNANSWER            = new JTitle ( Str.getStr ( STR_UNANSWER ) ) ;
      st_EDGE_TITLE          = new JTitle ( Str.getStr ( STR_EDGE ) ) ;
      st_EDGE_RETURN         = new JTitle ( Str.getStr ( STR_EDGE_RETURN ) ) ;
      cb_DOC_CLASS           = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_DOCCLASS ) ) ;   
      cb_EDGE_RETURN         = new JComboBox ( objectNodes ) ;
      lb_ACTION_LIST         = new JList () ;
      lb_PRE_ANSWERS         = new JList () ;
      conditionList          = new ConditionList() ;
      lb_UNANSWER            = new JList () ;
      buttonPanel_ActionList = new ButtonPanel () ;
      buttonPanel_PreAnswers = new ButtonPanel () ;
      buttonPanel_UnAnswer   = new ButtonPanel () ;
      buttonPanel_Command    = new ButtonPanel () ;
      lb_COUNTRIES           = new CountrySelectPanel() ;
      scr_UNANSWER           = new JScrollPane ( lb_UNANSWER ) ;
      scr_PRE_ANSWERS        = new JScrollPane ( lb_PRE_ANSWERS ) ;
      scr_ACTION_LIST        = new JScrollPane ( lb_ACTION_LIST ) ;        
      pb_SAVE                = new HotLinkLabel ( Str.getStr ( STR_SAVE ) ) ;
      pb_CANCEL              = new HotLinkLabel ( Str.getStr ( STR_CANCEL ) ) ;

      pb_ADD_ACTION          = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE_ACTION       = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_ADD_PREANSWER       = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE_PREANSWER    = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_ADD_UNANSWER        = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE_UNANSWER     = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      ck_EDGE_RETURN         = new JCheckBox ( "" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque ( false ) ;
      scr_UNANSWER.setBorder ( GUISystem.blackBorder ) ;
      scr_PRE_ANSWERS.setBorder ( GUISystem.blackBorder ) ;
      scr_ACTION_LIST.setBorder ( GUISystem.blackBorder ) ;
      st_DOC_CLASS.setFont ( FontSystem.smallTitleFont ) ;
      st_ACTION_LIST.setFont ( FontSystem.smallTitleFont ) ;
      st_EDGE_RETURN.setFont ( FontSystem.smallTitleFont ) ;
      st_PRE_ANSWERS.setFont ( FontSystem.smallTitleFont ) ;
      st_UNANSWER.setFont ( FontSystem.smallTitleFont ) ;
      buttonPanel_ActionList.setBorder ( GUISystem.grayBorder ) ;
      buttonPanel_ActionList.setBackground ( new Color ( 240, 240, 240 ) ) ;
      buttonPanel_ActionList.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                             ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;
      buttonPanel_PreAnswers.setBorder ( GUISystem.grayBorder ) ;
      buttonPanel_PreAnswers.setBackground ( new Color ( 240, 240, 240 ) ) ;
      buttonPanel_PreAnswers.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                             ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;
      buttonPanel_UnAnswer.setBorder ( GUISystem.grayBorder ) ;
      buttonPanel_UnAnswer.setBackground ( new Color ( 240, 240, 240 ) ) ;
      buttonPanel_UnAnswer.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                           ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      buttonPanel_Command.setBorder ( GUISystem.grayBorder ) ;
      buttonPanel_Command.setBackground ( new Color ( 240, 240, 240 ) ) ;
      buttonPanel_Command.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                          ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      ck_EDGE_RETURN.setOpaque ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD_ACTION.addActionListener ( this ) ;
      pb_REMOVE_ACTION.addActionListener ( this ) ;
      pb_ADD_PREANSWER.addActionListener ( this ) ;
      pb_REMOVE_PREANSWER.addActionListener ( this ) ;
      pb_ADD_UNANSWER.addActionListener ( this ) ;
      pb_REMOVE_UNANSWER.addActionListener ( this ) ;
      pb_SAVE.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      ck_EDGE_RETURN.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( buttonPanel_Command ) ;
      add ( st_EDGE_TITLE ) ;
      add ( st_EDGE_RETURN ) ;
      add ( ck_EDGE_RETURN ) ;
      add ( cb_EDGE_RETURN ) ;
      add ( st_DOC_CLASS ) ;
      add ( cb_DOC_CLASS ) ;
      add ( st_ACTION_LIST ) ;
      add ( scr_ACTION_LIST ) ;
      add ( buttonPanel_ActionList ) ;
      add ( st_PRE_ANSWERS ) ;
      add ( scr_PRE_ANSWERS ) ;
      add ( buttonPanel_PreAnswers ) ;
      add ( lb_COUNTRIES ) ;
      add ( conditionList ) ;
      add ( st_UNANSWER ) ;
      add ( scr_UNANSWER ) ;
      add ( buttonPanel_UnAnswer ) ;

      buttonPanel_ActionList.add ( pb_ADD_ACTION ) ;
      buttonPanel_ActionList.add ( new ToolBarSeparator ( tbSize ) ) ;
      buttonPanel_ActionList.add ( pb_REMOVE_ACTION ) ;

      buttonPanel_PreAnswers.add ( pb_ADD_PREANSWER ) ;
      buttonPanel_PreAnswers.add ( new ToolBarSeparator ( tbSize ) ) ;
      buttonPanel_PreAnswers.add ( pb_REMOVE_PREANSWER ) ;

      buttonPanel_UnAnswer.add ( pb_ADD_UNANSWER ) ;
      buttonPanel_UnAnswer.add ( new ToolBarSeparator ( tbSize ) ) ;
      buttonPanel_UnAnswer.add ( pb_REMOVE_UNANSWER ) ;

      buttonPanel_Command.add ( pb_SAVE ) ;
      buttonPanel_Command.add ( new ToolBarSeparator ( tbSize ) ) ;
      buttonPanel_Command.add ( pb_CANCEL ) ;

      ck_EDGE_RETURN.setEnabled ( allowEdgeReturn ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       halfWidth = size.width/2 ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = 0 ;
      int       y         = 0 ;

      super.doLayout() ;

      buttonPanel_Command.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight+2 ;

      st_EDGE_TITLE.setBounds ( x, y, size.width, rowHeight+8 ) ;
      y+=rowHeight+10 ;

      st_DOC_CLASS.setBounds ( x, y, halfWidth, rowHeight ) ;
      ck_EDGE_RETURN.setBounds ( x+halfWidth, y, 20, rowHeight ) ;
      st_EDGE_RETURN.setBounds ( x+halfWidth+20, y, halfWidth, rowHeight ) ;
      y+=rowHeight ;

      cb_DOC_CLASS.setBounds ( x, y, halfWidth-10, rowHeight ) ;
      cb_EDGE_RETURN.setBounds ( x+halfWidth, y, halfWidth, rowHeight ) ;
      y+=rowHeight ;

      st_ACTION_LIST.setBounds ( x, y, halfWidth, rowHeight ) ;
      lb_COUNTRIES.setBounds ( x+halfWidth, y, halfWidth, (rowHeight*6)+5 ) ;
      y+=rowHeight ;

      scr_ACTION_LIST.setBounds ( x, y, halfWidth-5, rowHeight*4 ) ;
      y+=(rowHeight*4)+2 ;

      buttonPanel_ActionList.setBounds ( x, y, halfWidth-5, rowHeight ) ;
      y+=rowHeight+10 ;

      st_PRE_ANSWERS.setBounds ( x, y, halfWidth, rowHeight ) ;
      st_UNANSWER.setBounds ( x+halfWidth, y, halfWidth, rowHeight ) ;
      y+=rowHeight ;

      scr_PRE_ANSWERS.setBounds ( x, y, halfWidth-5, rowHeight*4 ) ;
      scr_UNANSWER.setBounds ( x+halfWidth, y, halfWidth, rowHeight*4 ) ;
      y+=(rowHeight*4)+2 ;

      buttonPanel_PreAnswers.setBounds ( x, y, halfWidth-5, rowHeight ) ;
      buttonPanel_UnAnswer.setBounds ( x+halfWidth, y, halfWidth, rowHeight ) ;
      y+=rowHeight+10 ;

      conditionList.setBounds ( x, y, size.width, rowHeight*7 ) ;
      y+=rowHeight*9 ;

   }


   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }
   public Dimension getMinimumSize () {
      return( prefSize ) ;
   }





   /*------------------------------------------------------------*/
   /*  actionPerformed
    */
   /**
    *    This method will repsond to action events
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_ACTION                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_ADD_ACTION ) {
         SelectOADlg selectDlg = new SelectOADlg() ;

         selectDlg.getInstance().addSelectOAListener ( this ) ;
         selectDlg.getInstance().allowDrafts ( false ) ;
         selectDlg.getInstance().setView ( selectDlg.VIEW_ACTION ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_ACTION                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_ACTION )
         removeAction() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_PREANSWER                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_PREANSWER ) {
         PreanswerDlg dlg = new PreanswerDlg ( GUISystem.getParentDefWin ( this ) ) ;

         dlg.setPreanswerDlgListener ( this ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ck_EDGE_RETURN                                          */     
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ck_EDGE_RETURN ) 
         cb_EDGE_RETURN.setEnabled ( ck_EDGE_RETURN.isSelected() ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_PREANSWER                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_PREANSWER )
         removePreanswer() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_UNANSWER                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_UNANSWER ) {
         SelectOADlg selectDlg = new SelectOADlg() ;

         selectDlg.getInstance().addSelectOAListener ( this ) ;
         selectDlg.getInstance().allowDrafts ( false ) ;
         selectDlg.getInstance().setView ( selectDlg.VIEW_QUESTION ) ;

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_UNANSWER                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_UNANSWER )
         removeUnanswer() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SAVE                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE ) {
         if ( saveData() )
            fireChangedEvent () ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CANCEL )
         fireChangedEvent() ;




   } /* actionPerformed */





   /*------------------------------------------------------------*/
   /* selectOAComplete
    */
   /**
    *    This method will be called when a SelectOADlg
    *  we have started is finished
    *  
    *
    *  @param vec The Vector of data selected by the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectOAComplete ( Vector vec ) {
      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i       = 0 ;
         int size    = vec.size() ;

         while ( i < size ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is an action, add it to the Action List         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( vec.elementAt(i) instanceof ActionDisplayRec )
               addAction ( (ActionDisplayRec)vec.elementAt(i) ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is a question, add it to the unanswer list      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( vec.elementAt(i) instanceof QuestionDisplayRec )
               addUnanswer ( (QuestionDisplayRec)vec.elementAt(i) ) ;

            i++ ;
         }
      } /* if */
   } /* selectOAComplete */





   /*------------------------------------------------------------*/
   /* addAction
    */
   /**
    *    This method will add an action to the Action
    *  List
    *  
    *
    *  @param action The Action to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addAction ( ActionDisplayRec actionDisplayRec ) {
      EdgeAlist           edgeAlist           = null ;
      EdgeAlistDisplayRec edgeAlistDisplayRec = null ;

      if ( actionList == null )
         actionList = new Vector ( 1, 1 ) ;

      edgeAlist = new EdgeAlist ( actionDisplayRec.getActionInd() ) ;
      edgeAlistDisplayRec = new EdgeAlistDisplayRec ( edgeAlist ) ;
      actionDisplayRec.setAction ( AvalonCache.readAction ( actionDisplayRec.getActionInd() ) ) ;
      actionList.addElement ( edgeAlistDisplayRec ) ;
      lb_ACTION_LIST.setListData ( actionList ) ;
   }



   /*------------------------------------------------------------*/
   /* addUnanswer
    */
   /**
    *    This method will add an question to the Question
    *  List
    *  
    *
    *  @param question The Question to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addUnanswer ( QuestionDisplayRec questionDisplayRec ) {
      EdgeUnanswerDisplayRec unanswerDisplayRec = null ;

      if ( unanswerList == null )
         unanswerList = new Vector ( 1, 1 ) ;

      questionDisplayRec.readQuestion() ;
      unanswerDisplayRec = new EdgeUnanswerDisplayRec ( (Question) questionDisplayRec.getData() ) ;
      unanswerList.addElement ( unanswerDisplayRec ) ;
      lb_UNANSWER.setListData ( unanswerList ) ;
   }




   /*------------------------------------------------------------*/
   /* removeAction
    */
   /**
    *    This method will remove all actions that
    *  are selected in the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeAction ( ) {
      Object[] selection = lb_ACTION_LIST.getSelectedValues() ;

      try {
         int selIndex = lb_ACTION_LIST.getSelectedIndex() ;

         for ( int i = 0; i < selection.length; i++ )
            actionList.removeElement ( selection [ i ] ) ;

         lb_ACTION_LIST.setListData ( actionList ) ;
         lb_ACTION_LIST.setSelectedIndex ( selIndex ) ;
      } catch ( Exception e ) {
      }
   }



   /*------------------------------------------------------------*/
   /* conclusionDlgClosed
    */
   /**
    *    This method will be called when a conclusion
    *  dialog that we started has been closed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void preanswerDlgClosed ( EdgePreanswer edgePreanswer, EdgePreanswerDisplayRec displayRec ) {
      if ( edgePreanswer != null ) {

         if ( preanswerList == null )
            preanswerList = new Vector ( 1, 1 ) ;

         preanswerList.addElement ( displayRec ) ;

         lb_PRE_ANSWERS.setListData ( preanswerList ) ;
      }
   }

   /*------------------------------------------------------------*/
   /* removePreanswer
    */
   /**
    *    This method will remove all preanswers that
    *  are selected in the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removePreanswer ( ) {
      Object[] selection = lb_PRE_ANSWERS.getSelectedValues() ;

      try {
         int selIndex = lb_PRE_ANSWERS.getSelectedIndex() ;

         for ( int i = 0; i < selection.length; i++ ) {
            preanswerList.removeElement ( selection [ i ] ) ;
         }

         lb_PRE_ANSWERS.setListData ( preanswerList ) ;
         lb_PRE_ANSWERS.setSelectedIndex ( selIndex ) ;
      } catch ( Exception e ) {
      }
   }

   /*------------------------------------------------------------*/
   /* removeUnanswer
    */
   /**
    *    This method will remove all unanswers that
    *  are selected in the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeUnanswer ( ) {
      Object[] selection = lb_UNANSWER.getSelectedValues() ;

      try {
         int selIndex = lb_UNANSWER.getSelectedIndex() ;

         for ( int i = 0; i < selection.length; i++ )
            unanswerList.removeElement ( selection [ i ] ) ;

         lb_UNANSWER.setListData ( unanswerList ) ;
         lb_UNANSWER.setSelectedIndex ( selIndex ) ;
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save the data on our
    *  screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      ObjectNode returnObjectNode = (ObjectNode)cb_EDGE_RETURN.getSelectedItem() ;

      if ( ( allowEdgeReturn ) && ( ck_EDGE_RETURN.isSelected() ) ) {
         EdgeReturn edgeReturn = new EdgeReturn ( returnObjectNode.getNode() ) ;


         edge.setReturnNode ( edgeReturn ) ;
      } else
         edge.setReturnNode ( null ) ;

      edge.setActionList ( QuestUtil.getDataFromDisplay ( actionList ) ) ;
      edge.setPreanswers ( QuestUtil.getDataFromDisplay ( preanswerList ) ) ;
      edge.setUnanswers  ( QuestUtil.getDataFromDisplay ( unanswerList ) ) ;
      edge.setDocClass   ( (TypeDocClassRec) cb_DOC_CLASS.getSelectedItem() ) ;
      conditionList.getConditions() ;
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      if ( edge != null ) {

         if ( allowEdgeReturn ) {
            if ( edge.getReturnNode() != null ) {
               ObjectNode objectNode = AvalonUtil.getObjectNodeForNode ( objectNodes,
                                                                         ((EdgeReturn)edge.getReturnNode()).getNode() ) ;

               if ( objectNode != null ) {
                  cb_EDGE_RETURN.setSelectedItem ( objectNode ) ;
                  ck_EDGE_RETURN.setSelected ( true ) ;
               }
            }
         }

         cb_EDGE_RETURN.setEnabled ( ck_EDGE_RETURN.isSelected() ) ;

         if ( edge.getActionList () != null )
            actionList    = EdgeAlistDisplayRec.convert ( edge.getActionList() ) ;
         if ( edge.getPreanswers() != null )
            preanswerList = EdgePreanswerDisplayRec.convert ( edge.getPreanswers() ) ;
         if ( edge.getUnanswers() != null )
            unanswerList  = EdgeUnanswerDisplayRec.convert ( edge.getUnanswers() ) ;

         st_EDGE_TITLE.setText ( Str.getStr ( STR_EDGE )+": "+edge.getDefaultAnswerTitle() ) ;
         cb_DOC_CLASS.setSelectedItem ( edge.getDocClass() ) ;

         if ( actionList != null )
            lb_ACTION_LIST.setListData ( actionList ) ;
         if ( preanswerList != null )
            lb_PRE_ANSWERS.setListData ( preanswerList ) ;
         if ( unanswerList != null )
            lb_UNANSWER.setListData ( unanswerList ) ;
         conditionList.setConditions ( edge.getConditions() ) ;
         if ( edge.getCountryList() != null )
            lb_COUNTRIES.setCountryList ( edge.getCountryList() ) ;
      }
   }
} /* EdgeEditPanel */
