package com.ibm.nzna.projects.qit.avalon.editors ;       
/*-----------------------------------------------------------*/
/*   EditPanel                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EditPanel                           */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.util.*;
/**
 * This super class of all ***EditPanels in the editors package 
 *
 *
 * @version 1.0, 5/11/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EditPanel                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class EditPanel extends JPanel {      

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector listenerVec = null ;


   /*------------------------------------------------------------*/
   /* addEditPanelListener
    */
   /**
    *    This method will add a saveListener to this
    *  panel so that it is notified once the object
    *  has been sucessfully saved
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addEditPanelListener ( EditPanelListener listener ) {
      if ( listenerVec == null )
         listenerVec = new Vector ( 1,1 ) ;

      listenerVec.addElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* removeEditPanelListener
    */
   /**
    *    This method will remove a saveListener
    *  from the list of listeners that are notified of
    *  a complete save.
    *
    *  @param saveListener The EditPanelListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeEditPanelListener ( EditPanelListener listener  ) {
      if ( listenerVec != null )
         listenerVec.removeElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* fireChangedEvent
    */
   /**
    *    This method should be called whenever a
    *  save has completed
    *  
    *
    *  @param object The object that was saved
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void fireChangedEvent ( ) {
      if ( listenerVec != null ) {
         new OAEditNotifyThread ( listenerVec, this ).run() ;
      }
   }





   /*------------------------------------------------------------*/
   /* fireSpellCheckEvent
    */
   /**
    *    This method will fire a Spell Check event
    *  to all registered users. This should be called
    *  when a SpellCheck is complete
    *
    *  @param errorsFound TRUE if any errors were found ,regardless if they were fixed
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void fireSpellCheckEvent ( boolean errorsFound ) {
      if ( listenerVec != null )
         new OAEditNotifySpellCheckThread ( listenerVec, errorsFound, this ).run() ;
   }
} /* OAEditPanel */


class OAEditNotifyThread extends Thread implements Runnable {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector    listenerVec       = null ;
   private EditPanel editPanel            = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and start our
    *  thread
    *  
    *
    *  @param listenerVec The list of listeners to notify
    *  @param editPanel The editPanel to notify the listeners with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OAEditNotifyThread ( Vector listenerVec, EditPanel editPanel ) {
      this.listenerVec = listenerVec ;
      this.editPanel      = editPanel ;
   }


   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will simply go down the list
    *  of listeners and notify each one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      int i    = 0 ;

      if ( listenerVec != null ) {
         int size = listenerVec.size() ;

         while ( i < size ) {
            ((EditPanelListener)listenerVec.elementAt(i)).editPanelChanged ( editPanel ) ;
            i++ ;
         } /* while  */
      } /* if  */
   } /* run  */



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }

}/* OAEditNotifyThread  */



class OAEditNotifySpellCheckThread extends Thread implements Runnable {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector    listenerVec          = null ;
   private EditPanel editPanel            = null ;
   private boolean   errorsFound          = false ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and start our
    *  thread
    *  
    *
    *  @param listenerVec The list of listeners to notify
    *  @param editPanel The editPanel to notify the listeners with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OAEditNotifySpellCheckThread ( Vector listenerVec, boolean errorsFound, EditPanel editPanel ) {
      this.listenerVec    = listenerVec ;
      this.editPanel      = editPanel ;
      this.errorsFound    = errorsFound ;
   }


   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will simply go down the list
    *  of listeners and notify each one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      int i    = 0 ;

      if ( listenerVec != null ) {
         int size = listenerVec.size() ;

         while ( i < size ) {
            ((EditPanelListener)listenerVec.elementAt(i)).spellCheckComplete ( editPanel, errorsFound ) ;
            i++ ;
         } /* while  */
      } /* if  */
   } /* run  */


}/* OAEditNotifyThread  */

