package com.ibm.nzna.projects.qit.avalon.editors ;     
/*-----------------------------------------------------------*/
/*   FlowEditPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FlowEditPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.editors.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.oa.Action ;
/**
 * This panel holds all of the controls needed to manipulate a Knowledge flow,
 * including the VISIO chart, the add question/symptom/action buttons etc..
 *
 * @version 1.0, 4/20/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FlowEditPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class FlowEditPanel extends JPanel implements OASaveListener, Runnable, VisioListener, AvalonConst, SelectOAListener, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_PROCESS_NODE_FOCUS_RECEIVED = "TPNFR" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JScrollPane          dataScrollPanel       = null ;
   private JTitle               st_TITLE              = null ;
   private VisioPanel           visioPanel            = null ;
   private JMenuItem            mnu_ADD_ACTION        = null ;
   private JMenuItem            mnu_ADD_QUESTION      = null ;
   private JMenuItem            mnu_ADD_SYMPTOM       = null ;
   private JMenuItem            mnu_ADD_RESOLVED      = null ;
   private JMenuItem            mnu_ADD_UNRESOLVED    = null ;
   private JMenuItem            mnu_ADD_ACTION_LIST   = null ;
   private JMenuItem            mnu_ADD_RETURN        = null ;
   private JMenuItem            mnu_SET_ROOT          = null ;
   private HotLinkLabel         pb_ACTIONS            = null ;
   private HotLinkLabel         pb_REMOVE             = null ;
   private HotLinkLabel         pb_ADD                = null ;
   private ButtonPanel          buttonPanel           = null ;
   private ButtonPanel          nodeOutputPanel       = null ;
   private ButtonPanel          edgePanel             = null ;
   private JLabel               st_NODES              = null ;
   private JLabel               stt_NODES             = null ;
   private JLabel               st_EDGES              = null ;
   private JLabel               stt_EDGES             = null ;
   private SymptomEditorPanel   questPanel            = null ;
   private JScrollPane          nodeScrollPanel       = null ;
   private JPanel               nodePanel             = null ;
   private QuestionNodePanel    questionNodePanel     = null ;
   private ActionNodePanel      actionNodePanel       = null ;
   private SymptomNodePanel     symptomNodePanel      = null ;      
   private JPopupMenu           popupMenu             = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private SymptomDraft       symptom           = null ;
   private ObjectNode         focusNode         = null ;
   private DraftEditPanel         draftPanel        = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our panle
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FlowEditPanel ( SymptomEditorPanel questPanel ) {
      this.questPanel = questPanel ;
      initialize () ;                             
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {          
      Color purple = new Color ( 128, 0, 128 ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create controls                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      visioPanel         = new VisioPanel () ;
      dataScrollPanel    = new JScrollPane ( visioPanel ) ;
      st_TITLE           = new JTitle() ;
      pb_ADD             = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE          = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_ACTIONS         = new HotLinkLabel ( Str.getStr ( STR_ACTIONS ) );
      buttonPanel        = new ButtonPanel () ;
      nodeOutputPanel    = new ButtonPanel () ;
      edgePanel          = new ButtonPanel () ;
      st_EDGES           = new JLabel ( Str.getStr ( STR_EDGES )+":" ) ;
      st_NODES           = new JLabel ( Str.getStr ( STR_NODES )+":" ) ;
      stt_EDGES          = new JLabel ( "0" ) ;
      stt_NODES          = new JLabel ( "0" ) ;
      actionNodePanel    = new ActionNodePanel () ;
      setNodePanel ( null, false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust propeties                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      dataScrollPanel.getViewport().putClientProperty ("EnableWindowBlit", Boolean.TRUE);
      setBackground ( Color.white ) ;
      st_EDGES.setOpaque ( false ) ;
      st_NODES.setOpaque ( false ) ;
      stt_NODES.setOpaque ( false ) ;
      stt_EDGES.setOpaque ( false ) ;
      st_EDGES.setForeground ( purple ) ;
      st_NODES.setForeground ( purple ) ;
      buttonPanel.setBackground ( new Color ( 250, 250, 250 ) ) ;
      nodeOutputPanel.setBackground ( Color.white ) ;
      edgePanel.setBackground ( Color.white ) ;
      buttonPanel.setBorder     ( GUISystem.grayBorder ) ;
      buttonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      edgePanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;           
      nodeOutputPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                      ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD.addActionListener             ( this ) ;
      pb_REMOVE.addActionListener          ( this ) ;
      pb_ACTIONS.addActionListener         ( this ) ;
      visioPanel.addVisioListener          ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.add ( pb_ADD ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_REMOVE ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_ACTIONS ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( buttonPanel ) ;
      add ( nodeOutputPanel ) ;
      add ( edgePanel ) ;
      add ( st_TITLE ) ;
      nodeOutputPanel.add ( st_NODES ) ;
      nodeOutputPanel.add ( stt_NODES ) ;
      edgePanel.add ( st_EDGES ) ;
      edgePanel.add ( stt_EDGES ) ;

      setDataPanel ( visioPanel, false ) ;

   } /* initialize  */






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       y         = 5 ;
      int       x         = 15 ;

      super.doLayout () ;
      buttonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;


      dataScrollPanel.setBounds ( x+205, y, size.width-(x*2)-205, size.height-(y)-rowHeight ) ;
      if ( nodeScrollPanel != null )
         nodeScrollPanel.setBounds ( x, y, 200, size.height-(y)-rowHeight ) ;

      nodeOutputPanel.setBounds ( x+205, size.height-rowHeight, 100, rowHeight ) ;
      edgePanel.setBounds ( x+315, size.height-rowHeight, 100, rowHeight ) ;
   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when action
    *  events are fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_ADD )
         popupAddActions() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Menu - Set Root                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mnu_SET_ROOT )
         setRoot ( focusNode ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ACTIONS                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ACTIONS )
         popupActions() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   menu - Add Question                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mnu_ADD_QUESTION )
         addQuestion() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   menu - Add Action                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mnu_ADD_ACTION )
         addAction() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   menu - Add Symptom                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mnu_ADD_SYMPTOM )
         addSymptom() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Menu - Add Action List                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mnu_ADD_ACTION_LIST )
         addNode ( null, new Node ( Node.NODE_TYPE_ALIST ), false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Menu - Add Resolved                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mnu_ADD_RESOLVED )
         addNode ( null, new Node ( Node.NODE_TYPE_RESOLVED ), false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   menu - Add Unresolved                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mnu_ADD_UNRESOLVED )
         addNode ( null, new Node ( Node.NODE_TYPE_UNRESOLVED ), false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   menu - Add Return                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mnu_ADD_RETURN )
         addNode ( null, new Node ( Node.NODE_TYPE_RETURN ), false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_REMOVE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE )
         removeNode ( focusNode ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Any node in the flow                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() instanceof ObjectNode ) {
         boolean allowEdit = true ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, if the SymptomEditorPanel is in Full Screen      */
         /* mode, we must ask the user to close it first              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( questPanel.getFullScreen() ) {
            allowEdit = GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                                             Str.getStr ( STR_QUESTION ),
                                             Str.getStr ( STR_CANNOT_EDIT_NODE_FULL_SCREEN_CLOSE ) ) ;

            if ( allowEdit )
               questPanel.closeFullScreen() ;
         }

         if ( allowEdit ) {
            if ( ((ObjectNode)e.getSource()).getDraftObject() != null )
               edit ( ((ObjectNode)e.getSource()).getDraftObject() ) ;
            else
               edit ( ((ObjectNode)e.getSource()).getObject() ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* addNode
    */
   /**
    *    This method will add a single node to this
    *  flow
    *  
    *
    *
    *  @param node The node
    *  @param oaObject The OAObject being represented by this Node. NULLs are allowed
    *  @param showWarning If TRUE, and this node is filtered out, a warning will show
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ObjectNode addNode ( OAObject oaObject, Node node, boolean showWarning ) {
      ObjectNode  objectNode = null ;
      boolean     addNode    = true ;                                         


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we were passed an oaObject, and we are supposed      */
      /* to show the warning, do so now                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( oaObject != null ) && ( showWarning ) ) {
         FlowEditorFilter filter     = (FlowEditorFilter) PropertySystem.get ( PROPERTY_FLOW_FILTER ) ;
         boolean          show       = false ;

         show = ( ( filter.showCountry ( oaObject.getCountryList() ) ) &&
                  ( filter.showDocClass ( oaObject.getDocClass().getInd() ) ) ) ;

         if ( !show )
            addNode = GUISystem.printBox ( STR_QUESTION, STR_NODE_BEING_ADDED_IS_FILTERED_OUT_STILL_ADD ) ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are still supposed to be adding this node         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( addNode ) {
         objectNode = visioPanel.addNode ( node ) ;                          

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add ourselves as an Action Listener                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         objectNode.addActionListener ( this ) ;                                                          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the object in the ObjectNode so it doesn't          */
         /* have to read it from cache or the database                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( oaObject instanceof Question ) ||
              ( oaObject instanceof Action ) ||
              ( oaObject instanceof Symptom ) ) {
            objectNode.setOAObject ( oaObject ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the user has not set the root Question yet, ask      */
         /* them if they would like to do so now                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( oaObject instanceof Question ) && ( symptom.getRootNode() == null ) ) {
            if ( GUISystem.printBox ( Str.getStr ( STR_QUESTION ), 
                                      Str.getStr ( STR_NO_ROOT_SET_NOW ) ) ) {
               symptom.setRootNode ( node ) ;
               objectNode.setRoot ( true ) ;
            }
         }




         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Refresh the data on the screen                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         refreshData() ;
         invalidate() ;
         repaint() ;
      }

      return( objectNode ) ;
   }




   /*------------------------------------------------------------*/
   /* addNodes
    */
   /**
    *    This method will add multiple nodes to
    *  this flow
    *  
    *
    *  @param vector A vector of Node objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addNodes ( Vector vector ) {
      Vector objectNodes = null ;                                    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, add each node to the VISIO Panel. This           */
      /* will return a vector of ObjectNodes that correspond to    */
      /* each Node added.                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      objectNodes = visioPanel.addNodes ( vector ) ;                                    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we have Object Nodes, process them now           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( objectNodes != null ) {
         int        i          = 0 ;
         int        size       = objectNodes.size() ;
         ObjectNode objectNode = null ;

         while ( i < size ) {
            objectNode = (ObjectNode) objectNodes.elementAt(i ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add ourselves as an Action Listener                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            objectNode.addActionListener ( this ) ;
            i++ ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the data on the screen                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData() ;
      invalidate() ;
      repaint() ;
   }



   /*------------------------------------------------------------*/
   /* getPaintGrid
    */
   /**
    *    This method will return TRUE if we are
    *  painting our grid now
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean getPaintGrid () {
      return( visioPanel.getPaintGrid() ) ;
   }





   /*------------------------------------------------------------*/
   /* setPaintGrid
    */
   /**
    *    This method will set the paint grid property. If
    *  TRUE, a grid will be painted in the background
    *  
    *
    *  @param grid TRUE = paint Grid, FALSE = do not paint
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPaintGrid ( boolean grid  ) {
      visioPanel.setPaintGrid ( grid ) ;
   }




   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh the data on our screen
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData () {
      refreshCounters() ;
   }




   /*------------------------------------------------------------*/
   /* addQuestion
    */
   /**
    *    This method will add a question to the
    *  flow
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addQuestion () {
      CreateObjectDlg createDlg  = new CreateObjectDlg ( GUISystem.getParentDefWin ( this ) ) ;
      int             result     = createDlg.getResult() ;

      switch ( result ) {
         case CreateObjectDlg.NEW :
            QuestionEditorPanel panel = new QuestionEditorPanel ( questPanel ) ;

            panel.addOASaveListener ( this ) ;
            WindowSystem.createPanel ( panel ) ;
            break ;

         case CreateObjectDlg.BROWSE :
            SelectOADlg dlg = new SelectOADlg ( ) ;

            dlg.getInstance().setView ( SelectOADlg.VIEW_QUESTION ) ;
            dlg.getInstance().addSelectOAListener ( this ) ;
            break ;

         case CreateObjectDlg.EXISTING :
            Question         question = AvalonCache.readQuestion ( createDlg.getExistingInd() ) ;
            Node             node     = new Node ( question ) ;

            if ( question != null )
               addNode ( question, new Node ( question ), true ) ;
            break ;


      }
   }


   /*------------------------------------------------------------*/
   /* addAction
    */
   /**
    *    This method will add a action to the
    *  flow
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addAction () {
      CreateObjectDlg createDlg  = new CreateObjectDlg ( GUISystem.getParentDefWin ( this ) ) ;
      int             result     = createDlg.getResult() ;

      switch ( result ) {
         case CreateObjectDlg.NEW :
            ActionEditorPanel panel = new ActionEditorPanel ( questPanel ) ;

            panel.addOASaveListener ( this ) ;
            WindowSystem.createPanel ( panel ) ;
            break ;

         case CreateObjectDlg.BROWSE :
            SelectOADlg dlg = new SelectOADlg ( ) ;

            dlg.getInstance().setView ( SelectOADlg.VIEW_ACTION ) ;
            dlg.getInstance().addSelectOAListener ( this ) ;
            break ;

         case CreateObjectDlg.EXISTING :
            Action action = AvalonCache.readAction ( createDlg.getExistingInd() ) ;

            if ( action != null )
               addNode ( action, new Node ( action ), true ) ;
            break ;


      }
   }

   /*------------------------------------------------------------*/
   /* addSymptom
    */
   /**
    *    This method will add a symptom to the
    *  flow
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addSymptom () {
      CreateObjectDlg createDlg  = new CreateObjectDlg ( GUISystem.getParentDefWin ( this ) ) ;
      int             result     = createDlg.getResult() ;

      switch ( result ) {
         case CreateObjectDlg.NEW :
            SymptomEditorPanel panel = new SymptomEditorPanel ( questPanel ) ;

            panel.addOASaveListener ( this ) ;
            WindowSystem.createPanel ( panel ) ;
            break ;

         case CreateObjectDlg.BROWSE :
            SelectOADlg dlg = new SelectOADlg ( ) ;

            dlg.getInstance().setView ( SelectOADlg.VIEW_SYMPTOM ) ;
            dlg.getInstance().addSelectOAListener ( this ) ;
            break ;

         case CreateObjectDlg.EXISTING :
            Symptom symptom = AvalonCache.readSymptom ( createDlg.getExistingInd() ) ;

            if ( symptom != null )
               addNode ( symptom, new Node ( symptom ), true ) ;
            break ;


      }
   }


   /*------------------------------------------------------------*/
   /* selectOAComplete
    */
   /**
    *    This method will be called when the user
    *  has made a selection from the SelectOADlg
    *  that we have brought up
    *
    *  @param vec The vector of data that the user has selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectOAComplete ( Vector vec ) {

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int    i             = 0 ;
         int    size          = vec.size() ;
         Vector objectNodeVec = new Vector ( size ) ;

         while ( i < size ) {

            if ( vec.elementAt(i) instanceof QuestionDisplayRec ) {
               Question question = AvalonCache.readQuestion ( ((QuestionDisplayRec)vec.elementAt(i)).getQuestionInd() ) ;

               objectNodeVec.addElement ( addNode ( question, new Node ( question ), true ) ) ;
            }

            else if ( vec.elementAt(i) instanceof QuestionDraftDisplayRec ) {
               QuestionDraft questionDraft = AvalonCache.readQuestionDraft ( ((QuestionDraftDisplayRec)vec.elementAt(i)).getQuestionInd() );

               objectNodeVec.addElement ( addNode ( questionDraft, new Node ( questionDraft ), true ) ) ;
            }

            else if ( vec.elementAt(i) instanceof ActionDisplayRec ) {
               Action action = AvalonCache.readAction ( ((ActionDisplayRec)vec.elementAt(i)).getActionInd() ) ;

               objectNodeVec.addElement ( addNode ( action, new Node ( action ), true ) ) ;
            }

            else if ( vec.elementAt(i) instanceof ActionDraftDisplayRec ) {
               ActionDraft actionDraft = AvalonCache.readActionDraft ( ((ActionDraftDisplayRec)vec.elementAt(i)).getActionInd() ) ;

               objectNodeVec.addElement ( addNode ( actionDraft, new Node ( actionDraft ), true ) ) ;
            }

            else if ( vec.elementAt(i) instanceof SymptomDisplayRec ) {
               Symptom symptom = AvalonCache.readSymptom ( ((SymptomDisplayRec)vec.elementAt(i)).getSymptomInd() ) ;

               objectNodeVec.addElement ( addNode ( symptom, new Node ( symptom ), true ) ) ;
            }

            else if ( vec.elementAt(i) instanceof SymptomDraftDisplayRec ) {
               SymptomDraft symptomDraft = AvalonCache.readSymptomDraft ( ((SymptomDraftDisplayRec)vec.elementAt(i)).getSymptomInd() ) ;

               objectNodeVec.addElement ( addNode ( symptomDraft, new Node ( symptomDraft ), true ) ) ;
            }

            i++ ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, once we are done, cascase the object nodes          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         cascadeObjectNodes ( objectNodeVec ) ;

      } /* if  */

   } /* selectOAComplete  */





   /*------------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove all nodes from the list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAll () {
      visioPanel.removeAll() ;
      refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* setSymptom
    */
   /**
    *    This method will set the symptom that
    *  this FlowEditPanel will edit
    *  
    *
    *  @param symptom The SymptomDraft to edit
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSymptom ( SymptomDraft symptom ) {
      this.symptom = symptom ;                                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   We must assign the database keys, if we don't then      */
      /* nodes might have a node indicator of 0. We cannot allow   */
      /* that because if a node is 0, there is no way to get the   */
      /* edge groups from the node.                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      symptom.assignDatabaseKeys() ;


      visioPanel.removeAll() ;
      visioPanel.setSymptom ( symptom ) ;
      addActionListenerToObjectNodes ( visioPanel.getObjectNodes() ) ;
   }


   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will return the default title
    *  for the symptom
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getDefaultTitle () {
      String retStr = "" ;
      if ( symptom != null ) {
         Vector titles = symptom.getSymptomTitles() ;

         retStr = ((SymptomTitle)titles.elementAt(titles.size()-1)).getTitle() ;
      }
      return( retStr) ;
   }





   /*------------------------------------------------------------*/
   /* setNodePanel
    */
   /**
    *    This method will create the info panel with
    *  the panel passed in
    *  
    *
    *  @param panel The panel to be used inside of the nodePanel
    *  @param revalidate If TRUE, the panel will be revalidated after the panel is added
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setNodePanel ( JComponent panel, boolean revalidate ) {
      if ( nodeScrollPanel != null ) {
         remove ( nodeScrollPanel ) ;
         nodeScrollPanel = null ;
      }

      if ( panel != null )
         nodeScrollPanel = new JScrollPane ( panel ) ;
      else
         nodeScrollPanel = new JScrollPane () ;

      nodeScrollPanel.setBorder ( GUISystem.blackBorder ) ;
      add ( nodeScrollPanel ) ;

      if ( revalidate )
         revalidate() ;
   }


   /*------------------------------------------------------------*/
   /* setDataPanel
    */
   /**
    *    This method will set the panel that is showing on the left
    *  side of this panel  
    *  
    *
    *  @param panel The panel to be used inside of the nodePanel
    *  @param revalidate If TRUE, the panel will be revalidated after the panel is added
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setDataPanel ( Component panel, boolean revalidate ) {
      if ( dataScrollPanel != null ) {
         remove ( dataScrollPanel ) ;
         dataScrollPanel = null ;
      }

      if ( panel != null )
         dataScrollPanel = new JScrollPane ( panel ) ;
      else
         dataScrollPanel = new JScrollPane () ;

      dataScrollPanel.setBorder ( GUISystem.blackBorder ) ;
      dataScrollPanel.setOpaque ( false ) ;
      add ( dataScrollPanel ) ;

      if ( revalidate )
         revalidate() ;

      pb_ADD.setEnabled ( panel == visioPanel ) ;
      pb_REMOVE.setEnabled ( panel == visioPanel ) ;
      pb_ACTIONS.setEnabled ( panel == visioPanel ) ;
      if ( getParent() != null )
         getParent().setEnabled ( panel == visioPanel ) ;

   }




   /*------------------------------------------------------------*/
   /* nodeFocusReceived
    */
   /**
    *    This method will be called when a node
    *  has received focus, we will kick off a thread to do
    *  the processing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void nodeFocusReceived ( ObjectNode objectNode ) {

      if ( objectNode != focusNode ) {

         focusNode = objectNode ;
         new Thread ( this, THREAD_PROCESS_NODE_FOCUS_RECEIVED ).start() ;
      }
   }




   /*------------------------------------------------------------*/
   /* processNodeFocusReceived
    */
   /**
    *    This method will put up the correct Edit panel
    *  and refresh the information for what node is selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void processNodeFocusReceived () {

      /*--------------------------------------------------------------*/
      /*                                                              */
      /*   Depending on the type of node, show the correct NodePanel  */
      /*                                                              */
      /*--------------------------------------------------------------*/
      switch ( focusNode.getNode().getNodeType() ) {
         case Node.NODE_TYPE_QUESTION :
            if ( questionNodePanel == null )
               questionNodePanel = new QuestionNodePanel ( this, visioPanel, symptom ) ;
            questionNodePanel.setNode ( focusNode ) ;
            setNodePanel ( questionNodePanel, true ) ;
            questionNodePanel.refresh() ;
            break ;

         case Node.NODE_TYPE_ACTION :
            actionNodePanel.setNode ( focusNode ) ;
            setNodePanel ( actionNodePanel, true ) ;
            actionNodePanel.refresh() ;
            break ;

         case Node.NODE_TYPE_SYMPTOM :
            if ( symptomNodePanel == null )
               symptomNodePanel = new SymptomNodePanel ( this, focusNode, symptom ) ;
            setNodePanel ( symptomNodePanel, true ) ;
            symptomNodePanel.refresh() ;
            break ;

         default:
            setNodePanel ( null, true ) ;
            break ;
      }
   }




   /*------------------------------------------------------------*/
   /* getNodes
    */
   /**
    *    This method will return all of the nodes that
    *  this flow contains. Each node is an instance
    *  of ObjectNode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   protected Vector getNodes () {
      return( visioPanel.getNodes() ) ;
   }





   /*------------------------------------------------------------*/
   /* getObjectNodes
    */
   /**
    *    This method will return all of the ObjectNodes
    *  that the VisioPanel has in it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   protected Vector getObjectNodes () {
      return( (Vector) visioPanel.getObjectNodes().clone() ) ;
   }






   /*------------------------------------------------------------*/
   /* saveComplete
    */
   /**
    *    This method will be called when a EditorPanel
    *  we started is done and the save is completed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void saveComplete ( OAEditorPanel panel, Object object, boolean published ) {             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add this new node                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addNode ( (OAObject)object, new Node ( (OAObject)object), true ) ;

   }






   /*------------------------------------------------------------*/
   /* getNodesFromActiveInd
    */
   /**
    *    This method will go through our vector of
    *  nodes and tries to match up an existing node
    *  with the type and Active Node passed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getNodesFromActiveInd ( int type, int activeInd ) {
      Vector retVec = new Vector ( 1 ) ;
      Vector objectNodes = visioPanel.getObjectNodes() ;

      if ( ( objectNodes != null ) && ( objectNodes.size() > 0 ) ) {
         int        i           = 0 ;
         int        size        = objectNodes.size() ;
         Node       node        = null ;
         ObjectNode objectNode  = null ;
         int        currentType = 0 ;                         


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, loop through the nodes we have                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            objectNode = (ObjectNode) objectNodes.elementAt(i) ;
            node       = objectNode.getNode() ;
            currentType       = node.getNodeType() ;                                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, make sure this node is of the correct type for      */
            /* what type was passed in                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( currentType == node.NODE_TYPE_QUESTION ) ||
                 ( currentType == node.NODE_TYPE_ACTION ) ||
                 ( currentType == node.NODE_TYPE_SYMPTOM ) ) {

               switch ( type ) {
                  
                  case AVALON_QUESTION :
                     AVALON_QUESTION_DRAFT :
                     if ( node.getActiveObjectInd() == activeInd )
                        retVec.addElement ( objectNode ) ;
                     break ;

                  case AVALON_ACTION :
                     AVALON_ACTION_DRAFT :
                     if ( node.getActiveObjectInd() == activeInd )
                        retVec.addElement ( objectNode ) ;
                     break ;

                  case AVALON_SYMPTOM :
                     AVALON_SYMPTOM_DRAFT :
                     if ( node.getActiveObjectInd() == activeInd )
                        retVec.addElement ( objectNode ) ;
                     break ;
               }
            }
            i++ ;
         }
      }

      return( retVec ) ;
   }



   /*------------------------------------------------------------*/
   /* getNodesFromDraftInd
    */
   /**
    *    This method will go through our vector of
    *  nodes and tries to match up an existing node
    *  with the type and Active Node passed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getNodesFromDraftInd ( int type, int activeInd ) {
      Vector retVec = new Vector ( 1 ) ;
      Vector objectNodes = visioPanel.getObjectNodes() ;

      if ( ( objectNodes != null ) && ( objectNodes.size() > 0 ) ) {
         int        i           = 0 ;
         int        size        = objectNodes.size() ;
         Node       node        = null ;
         ObjectNode objectNode  = null ;
         int        currentType = 0 ;                         


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, loop through the nodes we have                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            objectNode = (ObjectNode) objectNodes.elementAt(i) ;
            node       = objectNode.getNode() ;
            currentType       = node.getNodeType() ;                                         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, make sure this node is of the correct type for      */
            /* what type was passed in                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( currentType == node.NODE_TYPE_QUESTION ) ||
                 ( currentType == node.NODE_TYPE_ACTION ) ||
                 ( currentType == node.NODE_TYPE_SYMPTOM ) ) {

               switch ( type ) {
                  
                  case AVALON_QUESTION_DRAFT :
                     AVALON_QUESTION :
                     if ( node.getDraftObjectInd() == activeInd )
                        retVec.addElement ( objectNode ) ;
                     break ;

                  case AVALON_ACTION_DRAFT :
                     AVALON_ACTION :
                     if ( node.getDraftObjectInd() == activeInd )
                        retVec.addElement ( objectNode ) ;
                     break ;

                  case AVALON_SYMPTOM_DRAFT :
                     AVALON_SYMPTOM :
                     if ( node.getDraftObjectInd() == activeInd )
                        retVec.addElement ( objectNode ) ;
                     break ;
               }
            }
            i++ ;
         }
      }

      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* getPaintTitles
    */
   /**
    *    This method will return TRUE if we are
    *  painting our titles
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getPaintTitles () {
      return( visioPanel.getPaintTitles() ) ;
   }




   /*------------------------------------------------------------*/
   /* setPaintTitles
    */
   /**
    *    This method will set if we paint our titles or
    *  not. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPaintTitles ( boolean paintTitles ) {
      visioPanel.setPaintTitles ( paintTitles ) ;
   }




   /*------------------------------------------------------------*/
   /* setDefaultDataPanel
    */
   /**
    *    This method will set the detault data panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDefaultDataPanel () {
      setDataPanel ( visioPanel, true ) ;
   }


   public void printToPrinter ( PrintJob printJob ) {
      visioPanel.printToPrinter ( printJob ) ;
   }




   /*------------------------------------------------------------*/
   /* removeNode
    */
   /**
    *    This method will remove a node from the
    *  flow only if the node can be removed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeNode ( ObjectNode node ) {

      /*----------------------------------------------------------------*/
      /*                                                                */
      /*   If this is the root node, do not allow the delete            */
      /*                                                                */
      /*----------------------------------------------------------------*/
      if ( symptom.getRootNode() != node.getNode() ) {
         node.getNode().updateRecStatus ( Node.REC_DELETE ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Remove all of the Edge Groups for this Node             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         removeEdgeGroups ( AvalonUtil.getHeadEdgeGroups ( node.getNode(), symptom ) ) ;
         removeEdgeGroups ( AvalonUtil.getTailEdgeGroups ( node.getNode(), symptom ) ) ;
         visioPanel.remove ( node ) ;
         node.setPaint ( false ) ;

         refreshCounters() ;
         invalidate() ;
         repaint () ;
      } else
         GUISystem.printBox ( GUISystem.getParentDefWin ( this ), STR_INFO, STR_CANNOT_REMOVE_ROOT_NODE ) ;
   }





   /*------------------------------------------------------------*/
   /* removeEdgeGroups
    */
   /**
    *    This method will remove the edge groups
    *  that are in the vector passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeEdgeGroups ( Vector edgeGroups ) {
      if ( ( edgeGroups != null ) && ( edgeGroups.size() > 0 ) ) {
         int i    = 0 ;
         int size = edgeGroups.size() ;

         while ( i < size ) {
            ((EdgeGroup)edgeGroups.elementAt(i)).updateRecStatus ( QITRec.REC_DELETE ) ;
            i++ ;
         }
      }

   }






   /*------------------------------------------------------------*/
   /* refreshCounters
    */
   /**
    *    This method will reset the counters that appear
    *  at the bottom of the flow chart
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void refreshCounters () {
      stt_EDGES.setText ( ""+visioPanel.getEdgeGroupCount() ) ;
      stt_NODES.setText ( ""+visioPanel.getNodeCount() ) ;
      st_TITLE.setText ( getDefaultTitle() ) ;
   }



   /*------------------------------------------------------------*/
   /* edit
    */
   /**
    *    This method will edit the currently selected
    *  item
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void edit ( Object object ) {
      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Edit a question                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( object instanceof QuestionDraft ) ||
              ( object instanceof Question ) )
            editQuestion ( object ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Edit an action                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( object instanceof ActionDraft ) ||
                   ( object instanceof Action ) )
            editAction ( object ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   edit a symptom                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( object instanceof Symptom ) ||
                   ( object instanceof SymptomDraft ) )
            editSymptom ( object ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* editQuestion
    */
   /**
    *    This method will edit the question object
    *  passed in
    *  
    *
    *  @param object The object representing either the Question or QuestionDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editQuestion ( Object object ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Question Drafts                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof QuestionDraft ) {
            QuestionEditorPanel editPanel = new QuestionEditorPanel ( questPanel, ((QuestionDraft)object).getInd() ) ;

            editPanel.addOASaveListener ( focusNode ) ;
            WindowSystem.createPanel ( editPanel ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Questions                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof Question ) {
            QuestionDraft questionDraft = null ;                  
            int           questionInd   = ((Question)object).getInd() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Try and read/create a draft                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            questionDraft = OASQL.getQuestionDraft ( GUISystem.getParentDefWin ( this ), questionInd, TypeList.WORKREQ_EDIT ) ;

            if ( questionDraft != null ) {
               QuestionEditorPanel editPanel = new QuestionEditorPanel ( questPanel, questionDraft ) ;

               editPanel.addOASaveListener ( focusNode ) ;
               WindowSystem.createPanel ( editPanel ) ;
            }

         } /* else  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   } /* editQuestion */







   /*------------------------------------------------------------*/
   /* editAction
    */
   /**
    *    This method will edit the action object
    *  passed in
    *  
    *
    *  @param object The object representing either the Action or ActionDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editAction ( Object object ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Action Drafts                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof ActionDraft ) {
            ActionEditorPanel editPanel = new ActionEditorPanel ( questPanel, ((ActionDraft)object).getInd() ) ;

            editPanel.addOASaveListener ( focusNode ) ;
            WindowSystem.createPanel ( editPanel ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Actions                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof Action ) {
            ActionDraft actionDraft = null ;                  
            int           actionInd   = ((Action)object).getInd() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Try and read/create a draft                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            actionDraft = OASQL.getActionDraft ( GUISystem.getParentDefWin ( this ), actionInd, TypeList.WORKREQ_EDIT ) ;

            if ( actionDraft != null ) {
               ActionEditorPanel editPanel = new ActionEditorPanel ( questPanel, actionDraft ) ;

               editPanel.addOASaveListener ( focusNode ) ;
               WindowSystem.createPanel ( editPanel ) ;
            }

         } /* else  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   } /* editAction */







   /*------------------------------------------------------------*/
   /* editSymptom
    */
   /**
    *    This method will edit the symptom object
    *  passed in
    *  
    *
    *  @param object The object representing either the Symptom or SymptomDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editSymptom ( Object object ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Symptom Drafts                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof SymptomDraft ) {
            SymptomEditorPanel editPanel = new SymptomEditorPanel ( questPanel, ((SymptomDraft)object).getInd() ) ;

            editPanel.addOASaveListener ( focusNode ) ;
            WindowSystem.createPanel ( editPanel ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Symptoms                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof Symptom ) {
            SymptomDraft symptomDraft = null ;                  
            int           symptomInd   = ((Symptom)object).getInd() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Try and read/create a draft                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            symptomDraft = OASQL.getSymptomDraft ( GUISystem.getParentDefWin ( this ), symptomInd, TypeList.WORKREQ_EDIT ) ;

            if ( symptomDraft != null ) {
               SymptomEditorPanel editPanel = new SymptomEditorPanel ( questPanel, symptomDraft ) ;

               editPanel.addOASaveListener ( focusNode ) ;
               WindowSystem.createPanel ( editPanel ) ;
            }

         } /* else  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   } /* editSymptom */





   /*------------------------------------------------------------*/
   /* popupAddActions
    */
   /**
    *    This method will pop up the addAction menu
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void popupAddActions () {
      Dimension    size = pb_ADD.getSize() ;
      Point        loc  = pb_ADD.getLocation() ;
      Color        gray = new Color ( 230, 230, 230 ) ;


      mnu_ADD_ACTION      = new JMenuItem ( Str.getStr ( STR_ACTION   ), ImageSystem.getImageIcon ( this, ImageSystem.ACTION_SMALL ) ) ;
      mnu_ADD_QUESTION    = new JMenuItem ( Str.getStr ( STR_QUESTION ), ImageSystem.getImageIcon ( this, ImageSystem.QUESTION_SMALL ) ) ;
      mnu_ADD_SYMPTOM     = new JMenuItem ( Str.getStr ( STR_SYMPTOM  ), ImageSystem.getImageIcon ( this, ImageSystem.SYMPTOM_SMALL ) ) ;
      mnu_ADD_RESOLVED    = new JMenuItem ( Str.getStr ( STR_RESOLVED ) ) ;
      mnu_ADD_UNRESOLVED  = new JMenuItem ( Str.getStr ( STR_UNRESOLVED ) ) ;
      mnu_ADD_ACTION_LIST = new JMenuItem ( Str.getStr ( STR_ACTION_LIST ) ) ;
      mnu_ADD_RETURN      = new JMenuItem ( Str.getStr ( STR_RETURN ) ) ;

      mnu_ADD_QUESTION.setBackground ( gray ) ;
      mnu_ADD_ACTION.setBackground ( gray ) ;
      mnu_ADD_SYMPTOM.setBackground ( gray ) ;
      mnu_ADD_RESOLVED.setBackground ( gray ) ;
      mnu_ADD_UNRESOLVED.setBackground ( gray ) ;
      mnu_ADD_ACTION_LIST.setBackground ( gray ) ;
      mnu_ADD_RETURN.setBackground ( gray ) ;

      mnu_ADD_QUESTION.addActionListener    ( this ) ;
      mnu_ADD_ACTION.addActionListener      ( this ) ;
      mnu_ADD_SYMPTOM.addActionListener     ( this ) ;             
      mnu_ADD_ACTION_LIST.addActionListener ( this ) ;             
      mnu_ADD_RETURN.addActionListener      ( this ) ;             
      mnu_ADD_RESOLVED.addActionListener    ( this ) ;             
      mnu_ADD_UNRESOLVED.addActionListener  ( this ) ;             

      popupMenu = new JPopupMenu () ;
      popupMenu.add ( mnu_ADD_ACTION ) ;
      popupMenu.add ( mnu_ADD_ACTION_LIST ) ;
      popupMenu.add ( mnu_ADD_QUESTION ) ;
      popupMenu.add ( mnu_ADD_SYMPTOM ) ;
      popupMenu.add ( mnu_ADD_RESOLVED ) ;
      popupMenu.add ( mnu_ADD_RETURN ) ;
      popupMenu.add ( mnu_ADD_UNRESOLVED ) ;

      popupMenu.setLightWeightPopupEnabled ( true ) ;
      popupMenu.setBorder ( GUISystem.grayBorder ) ;
      popupMenu.setBackground ( new Color ( 230, 230, 230 ) ) ;
      popupMenu.show ( buttonPanel, loc.x-5, size.height-1 ) ;
   }

   /*------------------------------------------------------------*/
   /* popupActions
    */
   /**
    *    This method will pop up the addAction menu
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void popupActions () {
      Dimension    size = pb_ACTIONS.getSize() ;
      Point        loc  = pb_ACTIONS.getLocation() ;
      Color        gray = new Color ( 230, 230, 230 ) ;


      mnu_SET_ROOT   = new JMenuItem ( Str.getStr ( STR_SET_ROOT   ) ) ;

      mnu_SET_ROOT.setBackground ( gray ) ;

      mnu_SET_ROOT.addActionListener    ( this ) ;

      popupMenu = new JPopupMenu () ;
      popupMenu.add ( mnu_SET_ROOT ) ;

      popupMenu.setLightWeightPopupEnabled ( true ) ;
      popupMenu.setBorder ( GUISystem.grayBorder ) ;
      popupMenu.setBackground ( new Color ( 230, 230, 230 ) ) ;
      popupMenu.show ( buttonPanel, loc.x-5, size.height-1 ) ;
   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will verify that we can save our
    *  data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected boolean saveData () {
      boolean rc = false ;

      if ( symptom.getRootNode() == null )
         GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_ROOT_NODE ) ;
      else
         rc = true ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* clearRoots
    */
   /**
    *    This method will make sure that each
    *  ObjectNode that is in the flow is cleared
    *  of it's root status
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void clearRoots () {
      Vector objectNodes = getObjectNodes() ;

      if ( ( objectNodes != null ) && ( objectNodes.size() > 0 ) ) {
         int i    = 0 ;
         int size = objectNodes.size() ;

         while ( i < size ) {
            ((ObjectNode)objectNodes.elementAt(i)).setRoot ( false ) ;
            i++ ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* setRoot
    */
   /**
    *    This method will set the root to the ObjectNode
    *  passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setRoot ( ObjectNode objectNode ) {
      Vector tailEdges = AvalonUtil.getTailEdgeGroups ( objectNode.getNode(), symptom ) ;

      tailEdges = QuestUtil.getValidData ( tailEdges ) ;

      if ( ( tailEdges == null ) ||     
           ( tailEdges.size() == 0 ) || 
           ( QuestUtil.containsValid ( tailEdges ) ) ) {
         clearRoots() ;
         symptom.setRootNode ( objectNode.getNode() ) ;
         objectNode.setRoot ( true ) ;
         repaint() ;
      } else
         GUISystem.printBox ( STR_INFO, STR_INVALID_ROOT_NODE ) ;
   }





   /*------------------------------------------------------------*/
   /* getVisioPanel
    */
   /**
    *    This method will return the visio panel that
    *  handles the dragging/drawing of the flow
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected VisioPanel getVisioPanel () {
      return( visioPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* cascadeObjectNodes
    */
   /**
    *    This method will cascade position the object
    *  nodes passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void cascadeObjectNodes ( Vector objectNodeVec ) {
      int i    = 0 ;
      int size = objectNodeVec.size() ;
      int x    = 0 ;
      int y    = 0 ;

      while ( i < size ) {
         ((ObjectNode)objectNodeVec.elementAt(i)).setLocation ( x, y ) ;
         x+=15 ;
         y+=15 ;
         i++ ;
      }
   }





   /*------------------------------------------------------------*/
   /* getSymptoms
    */
   /**
    *    This method will return a Vector of Symptom
    *  objects that are loaded into memory and displayed
    *  in the flow
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getSymptoms () {
      Vector returnVec   = new Vector ( 1, 1 ) ;
      Vector objectNodes = visioPanel.getObjectNodes() ;

      if ( ( objectNodes != null ) && ( objectNodes.size() > 0 ) ) {
         int        i          = 0 ;
         int        size       = objectNodes.size() ;
         ObjectNode objectNode = null ;

         while ( i < size ) {
            objectNode = (ObjectNode) objectNodes.elementAt(i) ;

            if ( objectNode.isObjectRead() ) {
               if ( objectNode.getObject() instanceof Symptom )
                  returnVec.addElement ( objectNode.getObject() ) ;
            }
            i++ ;
         }
      }
      return( returnVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getActions
    */
   /**
    *    This method will return a Vector of Action
    *  objects that are loaded into memory and displayed
    *  in the flow
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getActions () {
      Vector returnVec   = new Vector ( 1, 1 ) ;
      Vector objectNodes = visioPanel.getObjectNodes() ;

      if ( ( objectNodes != null ) && ( objectNodes.size() > 0 ) ) {
         int        i          = 0 ;
         int        size       = objectNodes.size() ;
         ObjectNode objectNode = null ;

         while ( i < size ) {
            objectNode = (ObjectNode) objectNodes.elementAt(i) ;

            if ( objectNode.isObjectRead() ) {
               if ( objectNode.getObject() instanceof Action )
                  returnVec.addElement ( objectNode.getObject() ) ;
            }
            i++ ;
         }
      }
      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* getQuestions
    */
   /**
    *    This method will return a Vector of Question
    *  objects that are loaded into memory and displayed
    *  in the flow
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getQuestions () {
      Vector returnVec   = new Vector ( 1, 1 ) ;
      Vector objectNodes = visioPanel.getObjectNodes() ;

      if ( ( objectNodes != null ) && ( objectNodes.size() > 0 ) ) {
         int        i          = 0 ;
         int        size       = objectNodes.size() ;
         ObjectNode objectNode = null ;

         while ( i < size ) {
            objectNode = (ObjectNode) objectNodes.elementAt(i) ;

            if ( objectNode.isObjectRead() ) {
               if ( objectNode.getObject() instanceof Question )
                  returnVec.addElement ( objectNode.getObject() ) ;
            }

            i++ ;
         }
      }
      return( returnVec ) ;
   }





   /*------------------------------------------------------------*/
   /* addActionListenerToObjectNodes
    */
   /**
    *    This method will add this panel as an
    *  action listener to each ObjectNode in the
    *  vector passed in
    *
    *  @param objectNodeVec The Vector containing ObjectNodes
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addActionListenerToObjectNodes ( Vector vec ) {
      int i    = 0 ;
      int size = vec.size() ;

      while ( i < size ) {
         ((ObjectNode)vec.elementAt(i)).addActionListener ( this ) ;
         i++ ;
      }
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      if ( operation.equals ( THREAD_PROCESS_NODE_FOCUS_RECEIVED ) )
         processNodeFocusReceived() ;
   }




   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method must be called when a FlowEditPanel
    *  is no longer being used
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void free () {
      visioPanel.free() ;
   }




   /*------------------------------------------------------------*/
   /* setDraftEditPanel
    */
   /**
    *    This will set the draftPanel property. This property
    *  should be set before any user is allowed to create an
    *  edge because the new Edge gets it's DocClass and countries
    *  from the draft panel
    *
    *  @param draftPanel The DraftEditPanel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setDraftEditPanel ( DraftEditPanel draftPanel ) {
      this.draftPanel = draftPanel ;
   }




   /*------------------------------------------------------------*/
   /* getDraftEditPanel
    */
   /**
    *    this method will return the draft panel
    *  property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return draftPanel
    *//*---------------------------------------------------------*/
   protected DraftEditPanel getDraftEditPanel () {
      return( draftPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* removeCountryFromPanel
    */
   /**
    *    This method will remove the country passed 
    *  in from all Edges that are displayed in this
    *  panel
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void removeCountryFromPanel ( TypeCountryCodeRec countryRec ) {                                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, remove the country from all of the edges         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      removeCountryFromEdges ( countryRec ) ;

   }





   /*------------------------------------------------------------*/
   /* removeCountryFromEdges
    */
   /**
    *    This method will remove a country from
    *  each edge in the Symptom
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void removeCountryFromEdges ( TypeCountryCodeRec countryRec ) {
      Vector edgeGroups = symptom.getEdgeGroups () ;

      if ( ( edgeGroups != null ) && ( edgeGroups.size() > 0 ) ) {
         int i    = 0 ;
         int size = edgeGroups.size() ;

         while ( i < size ) {
            Vector edges = ((EdgeGroup)edgeGroups.elementAt(i)).getEdges() ;

            if ( ( edges != null ) && ( edges.size() > 0 ) ) {
               int k        = 0 ;
               int edgeSize = edges.size() ;

               while ( k < edgeSize ) {
                  ((Edge)edges.elementAt(k)).removeCountry ( countryRec ) ;
                  k++ ;
               }
            }
            i++ ;
         }
      }
   }

} /* FlowEditPanel */
