package com.ibm.nzna.projects.qit.avalon.editors ;           
/*-----------------------------------------------------------*/
/*   FlowEditorFilter                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FlowEditorFilter                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
/**
 * This object has all of the filtering information on the FlowEditor. 
 *
 *
 * @version 1.0, 6/12/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FlowEditorFilter                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class FlowEditorFilter extends Object implements java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public static members                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int SITE_ESUPPORT        = 1 ;
   public static final int SITE_BUSINESSPARTNER = 2 ;
   public static final int SITE_INFOTIPS        = 3 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector countryFilter        = null ;                           // Vector of Integer objects. Each object represents a Country Code Ind.
   private int    siteId               = SITE_ESUPPORT ;          





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our blank FlowEditorFilter
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FlowEditorFilter () {
      countryFilter = new Vector ( 1, 1 ) ;
   }






   /*------------------------------------------------------------*/
   /* setCountryFilter
    */
   /**
    *    This method will set what countries we
    *  are allowing to show.
    *  
    *
    *  @param countryVec A Vector of TypeCountryCodeRcec objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCountryFilterList ( Vector countryVec ) {                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear out the list of countries now                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      countryFilter.removeAllElements() ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now add each country                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
         int i     = 0 ;
         int size  = countryVec.size() ;

         try {
            while ( i < size ) {
            countryFilter.addElement ( new Integer ( ((TypeCountryCodeRec)countryVec.elementAt(i)).getInd() ) ) ;
            i++ ;
         } /* while */
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
         }
      } /* if */
   } /* setCountryFilter */





   /*------------------------------------------------------------*/
   /* getCountryFilterList
    */
   /**
    *    This method will return a Vector of TypeCountryCodeRec
    *  objects that would show in this filter
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getCountryFilterList () {
      int      i        = 0 ;
      int      size     = countryFilter.size() ;
      Vector   retVec   = new Vector ( size ) ;
      TypeList typeList = TypeList.getInstance() ;

      while ( i < size ) {
         retVec.addElement ( typeList.objectFromInd ( ((Integer)countryFilter.elementAt(i)).intValue(), TypeList.TYPE_COUNTRYCODE ) ) ;
         i++ ;
      } /* while */

      return( retVec ) ;
   }






   /*------------------------------------------------------------*/
   /* setSite
    */
   /**
    *    This method will set the site that should
    *  show in this filter. The rules are as follows:<br><br>
    *
    *  eSupport Site = Only Unclassified Objects will show<br>
    *  Business Partner Site = Only Unclassified and Dealer Confidential will show<br>
    *  Info Tips = All objects will show
    *  
    *
    *  @param siteId The Site id. This is defined in this class as one of the SITE_XXX properties
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSite ( int siteId ) {
      this.siteId = siteId ;
   }





   /*------------------------------------------------------------*/
   /* getSite
    */
   /**
    *    This method will return the Site that
    *  this filter is going to show
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return site The id of the site as defined in the SITE_XXX properties of this class
    *//*---------------------------------------------------------*/
   public int getSite () {
      return( siteId ) ;
   }





   /*------------------------------------------------------------*/
   /* showDocClass
    */
   /**
    *    This method will return TRUE if this
    *  filter is allowing the doc classification passed
    *  to show based on the site of this filter.
    *
    *  @param docClass The Document Classification Indicator as defined in TIGRIS.TYPEDOCCLASS
    *  @param None
    *  @param None
    *
    *  @return TRUE = The classification should show, FALSE = The classification should not show
    *//*---------------------------------------------------------*/
   public boolean showDocClass ( int docClassInd ) {
      boolean show = false ;

      switch ( siteId ) {
         case SITE_ESUPPORT :
            show = (docClassInd == TypeList.DOCCLASS_UNCLASSIFIED ) ;
            break ;

         case SITE_BUSINESSPARTNER :
            show = ( (docClassInd == TypeList.DOCCLASS_UNCLASSIFIED ) ||
                     (docClassInd == TypeList.DOCCLASS_DEALER_CONFIDENTIAL ) ) ;
            break ;

         case SITE_INFOTIPS :
            show = true ;
            break ;
      }

      return( show ) ;
   }





   /*------------------------------------------------------------*/
   /* showCountry
    */
   /**
    *    This method will return TRUE if the country
    *  indicator passed is allowed to show.
    *  
    *
    *  @param countryInd The Country Indicator as defined in TIGRIS.TYPECOUNTRYCODE
    *  @param None
    *  @param None
    *
    *  @return TRUE = This country is allowed to show, FALSE = This country is not allowed to show
    *//*---------------------------------------------------------*/
   public boolean showCountry ( int countryInd ) {
      boolean show = false ;

      if ( ( countryFilter != null ) && ( countryFilter.size() > 0 ) ) {
         int i    = 0 ;
         int size = countryFilter.size() ;

         while ( ( !show ) && ( i < size ) ) {
            show = ( ((Integer)countryFilter.elementAt(i)).intValue() == countryInd ) ;
            i++ ;
         }
      }

      return( show ) ;
   }






   /*------------------------------------------------------------*/
   /* showCountry
    */
   /**
    *    This method will determine if any of the
    *  countries in the Vector passed in should
    *  show.
    *
    *  @param countryVec A Vector of TypeCountryCodeRecs to use to check
    *  @param None
    *  @param None
    *
    *  @return TRUE = At least 1 country in the Vector should show, FALSE = No countries in the Vector should show
    *//*---------------------------------------------------------*/
   public boolean showCountry ( Vector countryVec ) {
      boolean show = false ;

      if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
         int i    = 0;
         int size = countryVec.size() ;

         while ( ( !show ) && ( i < size ) ) {
            show = showCountry ( ((TypeCountryCodeRec)countryVec.elementAt(i)).getInd() ) ;
            i++ ;
         }
      } else
         show = true ; // There were no countries to filter out, so show it


      return( show ) ;
   }
} /* FlowEditorFilter */
