package com.ibm.nzna.projects.qit.avalon.editors ;         
/*-----------------------------------------------------------*/
/*   LinkBinderPanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkBinderPanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import java.sql.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This panel binds links to Questions or Actions. It allows the user 
 * link a QuestLink or ExternalLink to a Question or Action and will allow
 * conditions to be set on each link.
 *
 * @version 1.0, 5/9/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkBinderPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkBinderPanel extends EditPanel implements ListSelectionListener, ActionListener, SelectOAListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Titles                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] cnrLinkTitles  = { "", "", ""} ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private HotLinkLabel       pb_ADD_LINK        = null ;
   private HotLinkLabel       pb_REMOVE_LINK     = null ;
   private JTitle             st_LINKS           = null ;
   private MultiList          cnr_LINKS          = null ;
   private HotLinkLabel       pb_LINK_UP         = null ;
   private HotLinkLabel       pb_LINK_DOWN       = null ;      
   private ButtonPanel        buttonPanel        = null ;
   private ButtonPanel        linkButtonPanel    = null ;
   private JPanel             linkPanel          = null ;
   private ConditionList      conditionList      = null ;
   private DocClassPanel      docClassPanel      = null ;
   private CountrySelectPanel countrySelectPanel = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Dimension         prefSize                     = new Dimension ( 300, 500 ) ;
   private DocLink           currentLink                  = null ;
   private Vector            links                        = null ;
   private DraftEditPanel    draftEditPanel               = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initilize our
    *  GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkBinderPanel () {
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD_LINK          = new HotLinkLabel ( Str.getStr ( STR_ADD_LINK ) ) ;
      pb_REMOVE_LINK       = new HotLinkLabel ( Str.getStr ( STR_REMOVE_LINK ) ) ;
      st_LINKS             = new JTitle ( Str.getStr ( STR_LINKS ) ) ;
      cnr_LINKS            = new MultiList ( GUISystem.getFontUtil() ) ;
      pb_LINK_UP           = new HotLinkLabel ( Str.getStr ( STR_UP ) ) ;
      pb_LINK_DOWN         = new HotLinkLabel ( Str.getStr ( STR_DOWN ) ) ;
      buttonPanel          = new ButtonPanel () ;        
      linkButtonPanel      = new ButtonPanel () ;        
      conditionList        = new ConditionList () ;        
      linkPanel            = new JPanel () ;
      docClassPanel        = new DocClassPanel () ;
      countrySelectPanel   = new CountrySelectPanel () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.setBorder     ( GUISystem.grayBorder ) ;
      buttonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;
      linkButtonPanel.setBorder     ( GUISystem.grayBorder ) ;
      linkButtonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      linkButtonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                      ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;
      pb_ADD_LINK.setOpaque ( false ) ;
      pb_REMOVE_LINK.setOpaque ( false ) ;
      setBackground ( Color.white ) ;       
      GUISystem.setPreferredButton ( pb_LINK_UP ) ;
      GUISystem.setPreferredButton ( pb_LINK_DOWN ) ;
      linkPanel.setOpaque ( false ) ;
      cnr_LINKS.setColumnHeadings ( cnrLinkTitles ) ;
      cnr_LINKS.showColumnHeadings ( false ) ;
      cnr_LINKS.setBorder ( GUISystem.blackBorder ) ;
      cnr_LINKS.setColumnWidth ( 0, 18 ) ;
      cnr_LINKS.setColumnWidth ( 1, 100 ) ;
      cnr_LINKS.setColumnWidth ( 2, 350 ) ;
      st_LINKS.setFont ( FontSystem.smallTitleFont ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD_LINK.addActionListener ( this ) ;
      pb_REMOVE_LINK.addActionListener ( this ) ;
      pb_LINK_UP.addActionListener ( this ) ;
      pb_LINK_DOWN.addActionListener ( this ) ;
      cnr_LINKS.addListSelectionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( buttonPanel ) ;
      add ( st_LINKS  ) ;
      add ( cnr_LINKS ) ;
      add ( linkButtonPanel ) ;
      add ( docClassPanel ) ;
      add ( conditionList ) ;
      add ( countrySelectPanel ) ;

      linkButtonPanel.add ( pb_LINK_UP ) ;
      linkButtonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      linkButtonPanel.add ( pb_LINK_DOWN ) ;

      buttonPanel.add ( pb_ADD_LINK ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_REMOVE_LINK ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size       = getSize() ;
      int       rowHeight  = GUISystem.getRowHeight() ;
      int       halfWidth  = size.width/2 ;
      int       y          = 5 ;
      int       x          = 15 ;

      buttonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      cnr_LINKS.setBounds ( x, y, size.width-(x*2), rowHeight*10 ) ;
      y+=(rowHeight*10)+3 ;

      linkButtonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      docClassPanel.setBounds ( x, y, 250, rowHeight*2 ) ;
      y+=( rowHeight*2)+5 ;

      conditionList.setBounds ( x, y, halfWidth-5-x, size.height-y-5  ) ;
      countrySelectPanel.setBounds ( halfWidth, y, halfWidth-5, size.height-y-5 ) ;
   }





   /*------------------------------------------------------------*/
   /* selectOAListener
    */                                                           
   /**
    *    This method will be called when the user
    *  has made a selection from the 'SelectOADlg'
    *  window
    *
    *  @param vec The selection
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectOAComplete ( Vector vec ) {
      try {
         int languageInd = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;


         if ( ( vec != null ) && ( vec.size() > 0 ) ) {
            int    i          = 0 ;
            int    size       = vec.size() ;
            Object selObject  = null ;

            while ( i < size ) {
               selObject = vec.elementAt(i) ;  

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Normal Link                                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( selObject instanceof LinkDisplayRec ) {
                  addLink ( new DocLinkDisplayRec ( ((LinkDisplayRec)selObject).getLinkInd(),
                                                    ((LinkDisplayRec)selObject).getLinkType(),
                                                    ((LinkDisplayRec)selObject).getTitle() ),
                            ((LinkDisplayRec)selObject).getLinkType(),
                            ((LinkDisplayRec)selObject).getDocInd() ) ;

               } else if ( selObject instanceof LinkDraftDisplayRec ) {
                  addLink ( new DocLinkDisplayRec ( ((LinkDraftDisplayRec)selObject).getLinkInd(),
                                                    ((LinkDraftDisplayRec)selObject).getLinkType(),
                                                    ((LinkDraftDisplayRec)selObject).getTitle() ),
                            ((LinkDisplayRec)selObject).getLinkType(),
                            ((LinkDisplayRec)selObject).getDocInd() ) ;

               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Link Group                                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( selObject instanceof LinkGroupDisplayRec ) {
                  addLink ( new DocLinkDisplayRec ( ((LinkGroupDisplayRec)selObject).getLinkGroupInd(),
                                                    ((LinkGroupDisplayRec)selObject).getTitle() ), 1, 0 ) ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Bookmark                                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( selObject instanceof Bookmark ) {

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Link Bookmark                                           */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( ((Bookmark)selObject).getBookmarkType() == BOOKMARK_LINK ) {
                     Link link     = Link.readFromDatabase ( ((Integer)((Bookmark)selObject).getObject()).intValue(),
                                                             languageInd ) ;
                     int  linkType = -1 ;    
                     int  docInd   = 0 ;

                     if ( link instanceof QuestLink ) {
                        linkType = 0 ;
                        docInd = ((QuestLink)link).getDocInd() ;
                     } else
                        linkType = 1 ;  

                     addLink ( new DocLinkDisplayRec ( link.getInd(),
                                                       linkType,
                                                       link.getLinkTitles().elementAt(0).toString() ), linkType, docInd  ) ;
                  }


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   LinkGroup Bookmark                                      */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( ((Bookmark)selObject).getBookmarkType() == BOOKMARK_LINKGROUP ) {
                     LinkGroup linkGroup = LinkGroup.readFromDatabase ( ((Integer)((Bookmark)selObject).getObject()).intValue(),
                                                                        languageInd ) ;

                     addLink ( new DocLinkDisplayRec ( linkGroup.getInd(),
                                                       linkGroup.getLinkGroupTitles().elementAt(0).toString() ), 1, 0 ) ;
                  }

               }
               i++ ;
            }
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* addLink
    */
   /**
    *    This method will add a new link to the
    *  list. It will also refresh the display with the
    *  new link as the current selected link
    *
    *  @param docLink The DocLink to add
    *  @param linkType The type of link. Either 0 for Doc Link, 1 for URL link
    *  @param docInd The Document Indicator if this is a link type of 0
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addLink ( DocLinkDisplayRec linkRec, int linkType, int docInd ) { 
      DocLink link = (DocLink) linkRec.getData() ;

      cnr_LINKS.add ( linkRec ) ;

      if ( links == null )
         links = new Vector ( 1, 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   For URL Links, get the country information from         */
      /* the draft panel                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( linkType == 1 )
         link.setCountryList ( UserSystem.getUserCountries() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, set the list of countries from the document  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else
         link.setCountryList ( getDocCountryList ( docInd ) ) ;

      if ( draftEditPanel != null )
         link.setDocClass ( draftEditPanel.getDocClass() ) ;

      links.addElement ( link ) ;
      //cnr_LINKS.setSelectedItem ( linkRec, true ) ;
      setCurrentLink ( linkRec ) ;
   }





   /*------------------------------------------------------------*/
   /* setCurrentLink
    */
   /**
    *    This method will set what current link
    *  we are displaying
    *  
    *
    *  @param linkDisplayRec The doc link display rec representing the DocLink
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setCurrentLink ( DocLinkDisplayRec linkRec ) {
      if ( linkRec != null )
         setCurrentLink ( (DocLink) linkRec.getData() ) ;
   }





   /*------------------------------------------------------------*/
   /* setCurrentLink
    */
   /**
    *    This method will set the current link we
    *  are using. It will also refresh the data to
    *  
    *
    *  @param docLink The new current DocLink to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setCurrentLink ( DocLink docLink ){
      if ( currentLink != null ) {
         System.out.println ( "setCurrentLink Calling SaveData" ) ;
         saveData() ;
      }

      currentLink = docLink ;
      refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save the data that is on
    *  the screen for the currentLink
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveData () {
      System.out.println ( "Save Data" ) ;
      if ( currentLink != null ) {
         System.out.println ( "   CUrrent Link Not Null in save adata!" ) ;
         currentLink.setDocClass ( docClassPanel.getDocClass() ) ;
         System.out.println ( "Country LIst:"+countrySelectPanel.getCountryList() ) ;
         currentLink.setCountryList ( countrySelectPanel.getCountryList() ) ;
         conditionList.getConditions() ;
      }
   }


   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of the data
    *  on our screen based on what currentLink
    *  is
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      if ( currentLink != null ) {
         try {
            boolean enableEntry = true ;

            if ( cnr_LINKS.getSelectedItem() != null ) {
               enableEntry = !(cnr_LINKS.getSelectedItem() instanceof QuestLink ) ;
            } 

            conditionList.setConditions ( currentLink.getConditions() ) ;
            docClassPanel.setDocClass ( currentLink.getDocClass() ) ;
            countrySelectPanel.setCountryList ( currentLink.getCountryList() ) ;
            System.out.println ( "Refresh Data Country List:"+currentLink.getCountryList() ) ;

            docClassPanel.setEnabled ( enableEntry ) ;
            countrySelectPanel.setEnabled ( enableEntry ) ;
         } catch ( Exception e ) {
            e.printStackTrace() ;
         }

      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an
    *  action event is fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_LINK                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( pb_ADD_LINK == e.getSource() ) {
         SelectOADlg dlg = new SelectOADlg () ;

         /* Save the data that the user has up right now  */
         System.out.println ( "pb_ADD_LINK calling saveData()" ) ;
         saveData() ;

         /* pop up the Browse/Find window  */
         dlg.getInstance().addSelectOAListener ( this ) ;
         dlg.getInstance().allowDrafts ( false ) ;
         dlg.getInstance().setView ( dlg.VIEW_LINK ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_LINK                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_LINK ) {
         if ( currentLink != null ) {
            currentLink.updateRecStatus ( currentLink.REC_DELETE ) ;
            cnr_LINKS.remove ( (MultiListRow)cnr_LINKS.getSelectedItem() ) ;
         } else
            GUISystem.printBox ( this, STR_INFO, STR_PLEASE_SELECT_LINK ) ;

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_LINK_UP                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_LINK_UP )
         moveUp () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_LINK_DOWN                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_LINK_DOWN )
         moveDown() ;



   }




   /*------------------------------------------------------------*/
   /* getDocLinks
    */
   /**
    *    This method will return all of our doc links
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE
    *//*---------------------------------------------------------*/
   public Vector getDocLinks () {
      Vector links    = cnr_LINKS.getData() ;
      Vector docLinks = QuestUtil.getDataFromDisplay ( links ) ;

      return( docLinks ) ;
   }



   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when the user
    *  changes the selection of the current
    *  link
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( !e.getValueIsAdjusting() )
         setCurrentLink ( (DocLinkDisplayRec)cnr_LINKS.getSelectedItem() ) ;

   }




   /*------------------------------------------------------------*/
   /* refreshLinks
    */
   /**
    *    This method will refresh all of our links by clearing
    *  out all of the lists, and populating only the link list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshLinks () {
      conditionList.clear() ;
      cnr_LINKS.removeAll() ;
      if ( links != null )
         cnr_LINKS.add ( DocLinkDisplayRec.convert ( QuestUtil.getValidData ( links ) ) ) ;
   }





   /*------------------------------------------------------------*/
   /* setLinks
    */
   /**
    *    This method will set the list of links that
    *  we display to the user
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLinks ( Vector links ) {
      this.links = links ;
      refreshLinks() ;
   }





   /*------------------------------------------------------------*/
   /* moveUp
    */
   /**
    *    This method will move a link up in the list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void moveUp () {
      MultiListRow row   = (MultiListRow) cnr_LINKS.getSelectedItem() ;
      Vector       data  = cnr_LINKS.getData() ;
      int          index = data.indexOf ( row ) ;

      data.removeElement ( row ) ;
      if ( index > 0 ) {
         data.insertElementAt ( row, index-1 ) ;
         cnr_LINKS.removeAll() ;
         cnr_LINKS.add ( data ) ;
         cnr_LINKS.setSelectedItem ( row, true ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* moveDown
    */
   /**
    *    This method will move a link down in the list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void moveDown () {
      MultiListRow row   = (MultiListRow) cnr_LINKS.getSelectedItem() ;
      Vector       data  = cnr_LINKS.getData() ;
      int          index = data.indexOf ( row ) ;

      data.removeElement ( row ) ;
      if ( index < (data.size()) ) {
         data.insertElementAt ( row, index+1 ) ;
         cnr_LINKS.removeAll() ;
         cnr_LINKS.add ( data ) ;
         cnr_LINKS.setSelectedItem ( row, true ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* removeCountryFromPanel
    */
   /**
    *    This method will not only remove the country
    *  passed in from the panel, but it will also remove
    *  the country from all titles in this panel
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void removeCountryFromPanel ( TypeCountryCodeRec countryRec ) {                                


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remove the country from all of our links                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( links != null ) && ( links.size() > 0 ) ) {
         int    i              = 0 ;
         int    size           = links.size() ;
         Vector linkCountries = null ;

         while ( i < size ) {
            linkCountries = ((DocLink)links.elementAt(i)).getCountryList() ;
            linkCountries.removeElement ( countryRec ) ;
            ((DocLink)links.elementAt(i)).setCountryList ( linkCountries ) ;
            i++ ;
         } /* while  */
      } /* if  */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, refresh our display                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( currentLink != null )
         countrySelectPanel.setCountryList ( currentLink.getCountryList() ) ;


   } /* removeCountryFromPanel  */





   /*------------------------------------------------------------*/
   /* getCountrySelectPanel
    */
   /**
    *    This method will return an instance of this
    *  panel's CountrySelectPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected CountrySelectPanel getCountrySelectPanel () {
      return( countrySelectPanel ) ;
   }  


   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }

   public Dimension getMinimumSize()  {
      return( getPreferredSize() ) ;
   }




   /*------------------------------------------------------------*/
   /* setDraftEditPanel
    */
   /**
    *    This method will set the DraftEditPanel. This will
    *  make it so that when a link is added, the link's
    *  classification is retrieved from the DraftEditPanel 
    *
    *  @param draftEditPanel The DraftEditPanel to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDraftEditPanel ( DraftEditPanel draftEditPanel ) {
      this.draftEditPanel = draftEditPanel ;
   }




   /*------------------------------------------------------------*/
   /* getDocCountryList
    */
   /**
    *    This method will read in all of the countries
    *  that a document is associated to and returns
    *  them in a vector as TypeCountryCodeRecs
    *
    *  @param docInd The document indicator to read
    *  @param None
    *  @param None
    *
    *  @return Vector of TypeCountryCodeRecs
    *//*---------------------------------------------------------*/
   private Vector getDocCountryList ( int docInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocCountryList", 5 ) ;
      Vector    retVec    = new Vector () ;

      try {
         Statement stmt      = sqlMethod.createStatement() ;
         ResultSet rs        = null ;
         LocaleRec localeRec = new LocaleRec() ;
         boolean   cont      = true ;                            


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, read in all of the geographies                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT GEOIND FROM TIGRIS.DOCGEO WHERE DOCIND = "+docInd ) ;

         while ( ( cont ) && ( rs.next() ) ) {
            if ( rs.getInt(1) == 6 ) {
               localeRec.setWorldWide () ;
               cont = false ;
            } else {
               System.out.println ( "Adding Geography:"+rs.getInt(1) ) ;
               localeRec.addGeography ( rs.getInt(1) ) ;
            }
         }

         rs.close() ;                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, read in all of the countries                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt = sqlMethod.createStatement() ;
         rs = stmt.executeQuery ( "SELECT COUNTRYCODEIND FROM TIGRIS.DOCCOUNTRY WHERE DOCIND = "+docInd ) ;

         while ( ( cont ) && ( rs.next() ) ) {
            localeRec.addCountry ( rs.getInt(1) ) ;
            System.out.println ( "Read country:"+rs.getInt(1) ) ;
         }

         rs.close() ;

         retVec = localeRec.getCountryList() ;
         System.out.println ( "RetVec Size:"+retVec.size() ) ;

         for ( int w = 0; w < retVec.size(); w++ )
            System.out.println ( "Returning Country:"+retVec.elementAt(w) ) ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }
      sqlMethod.close() ;
      return( retVec ) ;
   }

} /* LinkBinderPanel */
