package com.ibm.nzna.projects.qit.avalon.editors ;         
/*-----------------------------------------------------------*/
/*   LinkEditorPanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkEditorPanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
/**
 * This editor allows a user to manipulate a Link. 
 *
 *
 * @version 1.0, 4/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkEditorPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkEditPanel extends EditPanel implements KeyListener, SelectDocListener, ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ActionButton    pb_CLOSE      = null ;
   private ActionButton    pb_SAVE       = null ;
   private JTitle          st_TITLE      = null ;
   private JTextField      ef_TITLE      = null ;
   private JRadioButton    rb_URL_LINK   = null ;
   private JRadioButton    rb_QUEST_LINK = null ;
   private JTextField      ef_URL_LINK   = null ;
   private JTextField      ef_QUEST_LINK = null ;
   private HotLinkLabel    pb_VALIDATE   = null ;
   private HotLinkLabel    pb_BROWSE     = null ;
   private JLabel          st_VALIDATED  = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private QuestLinkDraft    questLink         = null ;  // The Quest Link
   private ExternalLinkDraft externalLink      = null ;  // The External Link we are editing
   private QuestPanel        previousPanel     = null ;  // The panel to go back to when we close
   private boolean           saved             = false ; // TRUE if we save this object





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor will initialize as
    *  a 'New' Link. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditPanel () {
      initialize() ;
   }

   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create all of our GUI and will
    *  initialize our local variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true if success
    *//*---------------------------------------------------------*/
   private void initialize () {     
      ButtonGroup buttonGroup = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE      = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_SAVE       = new ActionButton ( Str.getStr ( STR_SAVE ),  ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SAVE_LINK ) ) ;
      st_TITLE      = new JTitle ( Str.getStr ( STR_TITLE ) ) ;
      ef_TITLE      = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ), "", 0 ) ;
      ef_QUEST_LINK = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 12 ), "", 0 ) ;
      ef_URL_LINK   = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_FILEURL ), "", 0 ) ;
      rb_QUEST_LINK = new JRadioButton ( Str.getStr ( STR_QUEST_DOCUMENT_LINK ) ) ;
      rb_URL_LINK   = new JRadioButton ( Str.getStr ( STR_URL_LINK ) ) ;
      pb_VALIDATE   = new HotLinkLabel ( Str.getStr ( STR_VALIDATE ), ImageSystem.getImageIcon ( this, ImageSystem.SMALL_CHECKBOOK ) ) ;
      pb_BROWSE     = new HotLinkLabel ( Str.getStr ( STR_BROWSE ),   ImageSystem.getImageIcon ( this, ImageSystem.SEE_ALSO ) ) ;
      st_VALIDATED  = new JLabel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust our radio buttons                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonGroup = new ButtonGroup () ;
      buttonGroup.add ( rb_URL_LINK ) ;
      buttonGroup.add ( rb_QUEST_LINK ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      st_TITLE.setFont ( FontSystem.smallTitleFont ) ;
      st_VALIDATED.setOpaque  ( false ) ;
      pb_VALIDATE.setOpaque   ( false ) ;
      pb_BROWSE.setOpaque     ( false ) ;
      rb_QUEST_LINK.setOpaque ( false ) ;
      rb_URL_LINK.setOpaque   ( false ) ;
      st_TITLE.setOpaque      ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SAVE.addActionListener       ( this ) ;
      pb_CLOSE.addActionListener      ( this ) ;
      rb_URL_LINK.addActionListener   ( this ) ;
      pb_VALIDATE.addActionListener   ( this ) ;
      pb_BROWSE.addActionListener     ( this ) ;
      rb_QUEST_LINK.addActionListener ( this ) ;
      ef_URL_LINK.addActionListener   ( this ) ;
      ef_QUEST_LINK.addActionListener ( this ) ;
      ef_TITLE.addKeyListener         ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_TITLE ) ;
      add ( ef_TITLE ) ;
      add ( pb_BROWSE ) ;
      add ( rb_URL_LINK ) ;
      add ( ef_URL_LINK ) ;
      add ( rb_QUEST_LINK ) ;
      add ( ef_QUEST_LINK ) ;
      add ( pb_VALIDATE ) ;
      add ( st_VALIDATED ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       y         = 5 ;
      int       x         = 15 ;

      super.doLayout() ;

      st_TITLE.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight+5 ;

      ef_TITLE.setBounds ( x, y, size.width/2, rowHeight ) ;
      y+=rowHeight+10 ;

      rb_QUEST_LINK.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      ef_QUEST_LINK.setBounds ( x+20, y, 150, rowHeight ) ;
      pb_BROWSE.setBounds ( x+180, y, 150, rowHeight ) ;
      y+=rowHeight+10 ;

      rb_URL_LINK.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      ef_URL_LINK.setBounds ( x+20, y, 350, rowHeight ) ;
      pb_VALIDATE.setBounds ( x+380, y, 150, rowHeight ) ;
      y+=rowHeight+10 ;

      st_VALIDATED.setBounds ( x+20, y, 250, rowHeight ) ;


   }



   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will attempt to save the data
    *  on the screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if data was saved, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      boolean rc       = false ;
      String  titleStr = ef_TITLE.getText() ;

      try {

         if ( ( titleStr != null ) && ( titleStr.length() > 0 ) ) {
            Vector       titles = null ;
            LanguageText title  = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Grab the existing title vector                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( questLink != null ) 
               titles = questLink.getLinkTitles() ;
            else if ( externalLink != null ) 
               titles = externalLink.getLinkTitles() ;

            title = (LanguageText)titles.elementAt(0) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up the Title object                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            title.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                         TypeList.TYPE_LANGUAGE ) ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Save the QuestLink if that is what we are editing       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( questLink != null ) {
               if ( validateQuestLink() ) {
                  rc = true ;
               }
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, save the ExternalLink                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( externalLink != null )
               rc = true ;

         } else {
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_TITLE ) ;
            ef_TITLE.requestFocus() ;
         }

      } catch ( Exception e ) {
         rc = false ;
         LogSystem.log ( 1, e, false ) ;
      }


      return( rc ) ;
   } /* saveData  */





   /*------------------------------------------------------------*/
   /* enableEntryFields
    */
   /**
    *    This method will either enable or disable
    *  the URL and Quest Doc Link Entry Fields depending on
    *  what radio button is selected
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void enableEntryFields () {
      ef_URL_LINK.setEnabled ( rb_URL_LINK.isSelected() ) ;
      ef_QUEST_LINK.setEnabled ( rb_QUEST_LINK.isSelected() ) ;
   }

   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an event
    *  is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   rb_URL_LINK                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == rb_URL_LINK ) ||
           ( e.getSource() == rb_QUEST_LINK ) )
         enableEntryFields() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_BROWSE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_BROWSE ) ||
                ( e.getSource() == ef_QUEST_LINK ) )
         new SelectDocDlg ( false ).addSelectDocListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_VALIDATE or ef_URL_LINK                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == ef_URL_LINK ) || 
                ( e.getSource() == pb_VALIDATE ) )
         validateURL() ;



   } /* actionPerformed  */





   /*------------------------------------------------------------*/
   /* setLink
    */
   /**
    *    This method will set what link we are
    *  editing
    *  
    *
    *  @param questLink If we are editing a quest link, show those fields, disable the rest
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLink ( QuestLinkDraft questLink ) {
      this.questLink    = questLink ;
      this.externalLink = null ;

      ef_TITLE.setEnabled ( false ) ;

      rb_URL_LINK.setEnabled ( false ) ;
      ef_URL_LINK.setEnabled ( false ) ;
      pb_VALIDATE.setEnabled ( false ) ;
      rb_QUEST_LINK.setSelected ( true ) ;
      ef_QUEST_LINK.setText ( ""+questLink.getDocInd() ) ;
      ef_TITLE.setText ( questLink.getLinkTitles().elementAt(0).toString() ) ;
   }





   /*------------------------------------------------------------*/
   /* setLink
    */
   /**
    *    This method will set what link we are editing
    *  
    *  
    *
    *  @param docLink The Doc Link that will be edited. All QuestLink fields will be disabled
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLink ( ExternalLinkDraft externalLink ) {
      this.externalLink = externalLink ;
      this.questLink    = null ;

      ef_TITLE.setEnabled ( true ) ;
      rb_QUEST_LINK.setEnabled ( false ) ;
      ef_QUEST_LINK.setEnabled ( false ) ;
      pb_BROWSE.setEnabled ( false ) ;
      rb_URL_LINK.setSelected ( true ) ;
      ef_URL_LINK.setText ( externalLink.getUrl() ) ;
      setValidatedText () ;
      ef_TITLE.setText ( externalLink.getLinkTitles().elementAt(0).toString() ) ;
   }





   /*------------------------------------------------------------*/
   /* setValidatedText
    */
   /**
    *    This method will set the validated text on
    *  if this URL has been validated, it will set the
    *  text unknown if not
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setValidatedText () {
      if ( externalLink != null ) {
         if ( externalLink.isUrlValid() ) {
            st_VALIDATED.setForeground ( Color.blue ) ;
            st_VALIDATED.setText ( Str.getStr ( STR_VALIDATED ) ) ;
         } else {
            st_VALIDATED.setForeground ( new Color ( 150, 0, 0 ) ) ;
            st_VALIDATED.setText ( Str.getStr ( STR_UNKNOWN ) ) ;
         }
      } else
         st_VALIDATED.setText ( "" ) ;
   }




   /*------------------------------------------------------------*/
   /* selectDocComplete
    */
   /**
    *    This method will be called when the user
    *  selects something from the SelectDocDlg
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectDocComplete ( Vector vector ) {
      if ( ( vector != null ) && ( vector.size() > 0 ) ) {
         Object object = vector.elementAt(0) ;

         if ( object instanceof DocRec )
            ef_QUEST_LINK.setText ( ""+((DocRec)object).getExistingDocInd() ) ;
         else if ( object instanceof Bookmark ) {
            Bookmark bookmark = (Bookmark) object ;

            if ( bookmark.getBookmarkType() == BOOKMARK_DOCUMENT )
               ef_QUEST_LINK.setText ( ""+((Integer)bookmark.getObject()).intValue() ) ;
         }

      }
   }





   /*------------------------------------------------------------*/
   /* validateURL
    */
   /**
    *    This method will validate that the URL
    *  in the ef_URL field is valid
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void validateURL () {
      pb_VALIDATE.setEnabled ( false ) ;
      Qit.setEnabled ( false ) ;
      GUISystem.getParentDefWin ( this ).setStatus ( STR_PLEASE_WAIT ) ;
      try {
         externalLink.setUrlValid ( URLValidator.validateUrl ( ef_URL_LINK.getText() ) ) ;
      } catch ( Exception e ) {
         String title = e.toString() ;

         if ( title.indexOf ( "Operation timed out" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_URL_TIMED_OUT ) ;

         else if ( title.indexOf ( "UnknownHostException" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_URL_NOT_FOUND ) ;

         else if ( title.indexOf ( "MalformedURLException: no protocol" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_URL_NO_PROTOCOL ) ;

         else if ( title.indexOf ( "ConnectException: Connection refused" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_URL_CONNECT_REFUSED ) ;

         externalLink.setUrlValid ( false ) ;
      }

      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;
      Qit.setEnabled ( true ) ;
      pb_VALIDATE.setEnabled ( true ) ;

      setValidatedText() ;
   }





   /*------------------------------------------------------------*/
   /* validateQuestLink
    */
   /**
    *    This method will validate that the Doc Indicator
    *  that the user has entered in the ef_QUEST_LINK
    *  field is a valid published document
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean validateQuestLink () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "validateQuestLink", 5 ) ;
      boolean   valid     = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DOCIND "+
                                              "FROM TIGRIS.DOCUMENTS "+
                                              "WHERE DOCIND = "+ef_QUEST_LINK.getText()+" AND "+
                                              "      RECYCLED = 'N' AND "+
                                              "      PUBLISH  = 'Y'" ) ;

         valid = rs.next() ;

         rs.close() ;
         stmt.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;

      return( valid ) ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title of the link
    *  that is being edited
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( ef_TITLE.getText() ) ;
   }


   /*------------------------------------------------------------*/
   /* keyTyped
    */
   /**
    *    We have subclassed this method so that
    *  we will automagically update the title in both
    *  the entry field and in the Title object
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void keyTyped ( KeyEvent e ) {
      fireChangedEvent ( ) ;
   }
   public void keyPressed ( KeyEvent e ) {
   }
   public void keyReleased ( KeyEvent e ) {
   }

} /* LinkEditorPanel */

