package com.ibm.nzna.projects.qit.avalon.editors ;         
/*-----------------------------------------------------------*/
/*   LinkEditorPanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkEditorPanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.qit.Qit ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This editor allows a user to manipulate a Link. 
 *
 *
 * @version 1.0, 4/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkEditorPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkEditorPanel extends OAEditorPanel implements PublishPanelListener, EditPanelListener, AvalonConst, Runnable, ActionListener, QuestPanel, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Thread                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESH      = "REFRESH" ;
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Panel Names                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String PANEL_USAGE       = "USAGE" ;
   public static final String PANEL_DRAFT       = "DRAFT" ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ActionButton    pb_CLOSE      = null ;
   private ActionButton    pb_SAVE       = null ;
   private JTabbedPane     tabbedPane    = null ;
   private LinkEditPanel   linkPanel     = null ;
   private DraftEditPanel  draftPanel    = null ;
   private JLabel          st_TITLE      = null ;
   private PublishPanel    publishPanel  = null ;
   private LinkUsagePanel  usagePanel    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private QuestLinkDraft    questLink         = null ;  // The Quest Link
   private ExternalLinkDraft externalLink      = null ;  // The Doc Link we are editing
   private QuestPanel        previousPanel     = null ;  // The panel to go back to when we close
   private boolean           saved             = false ; // TRUE if we save this object
   private int               linkInd           = 0 ;     // The Draft Link Inidicator to read on startup
   private int               linkType          = -1 ;    // The link type 0 = Quest Link, 1 = ExternalLink
   private boolean           readFromDatabase  = false ; // TRUE if we are to read the linkInd from the database
   private boolean           forceClose        = false ;
   private boolean           publishing        = false ;
   private AppDefaultWin     parentWin         = null ;
   private boolean           published         = false ; // TRUE if the object we were saving was published
   private String            initialPanel      = PANEL_DRAFT ; // The initial panel to show when the editor opens
   private boolean           autoPublish       = false ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor will initialize as
    *  a 'New' Link. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel () {
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will initiailze as an 'Edit'
    *  Link Group
    *  
    *
    *  @param previousPanel The panel to show when this panel closes
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel ( QuestPanel previousPanel ) {
      Object object = new NewAvalonObjectWizard ( GUISystem.getParentDefWin ( previousPanel.getJComponent() ), 
                                                  AVALON_LINK ).getNewObject() ;
      this.previousPanel = previousPanel ;
      if ( object instanceof QuestLinkDraft )
         questLink = (QuestLinkDraft) object ;
      else if ( object instanceof ExternalLinkDraft )
         externalLink = (ExternalLinkDraft) object ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will edit the action
    *  passed in and then set the panel in the tabbed
    *  pane to the panel passed in
    *
    *  @param questPanel The previous panel
    *  @param externalLinkDraft The link to edit
    *  @param panelName One of the panel names ( PANEL_DRAFT, PANEL_USAGE ) as defined in this class
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel ( QuestPanel questPanel, ExternalLinkDraft externalLinkDraft, String panelName ) {
      this.previousPanel     = previousPanel ;
      this.externalLink      = externalLinkDraft ;
      this.initialPanel      = panelName ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will edit the action
    *  passed in and then set the panel in the tabbed
    *  pane to the panel passed in
    *
    *  @param questPanel The previous panel
    *  @param questLinkDraft The link to edit
    *  @param panelName One of the panel names ( PANEL_DRAFT, PANEL_USAGE ) as defined in this class
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel ( QuestPanel questPanel, QuestLinkDraft questLinkDraft, String panelName ) {
      this.previousPanel     = previousPanel ;
      this.questLink         = questLinkDraft ;
      this.initialPanel      = panelName ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our panel and
    *  start editing the ExternalLink passed in
    *  
    *
    *  @param previousPanel The panel to gain focus once we are closed
    *  @param externalLinkDraft The link to edit
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel ( QuestPanel previousPanel, ExternalLinkDraft externalLinkDraft ) {
      this.previousPanel = previousPanel ;
      this.externalLink  = externalLinkDraft ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our panel and
    *  start editing the QuestLink passed in
    *  
    *
    *  @param previousPanel The panel to gain focus once we are closed
    *  @param externalLinkDraft The link to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel ( QuestPanel previousPanel, QuestLinkDraft questLinkDraft ) {
      this.previousPanel = previousPanel ;
      this.questLink     = questLinkDraft ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our panel and
    *  start editing the ExternalLink passed in
    *  
    *
    *  @param previousPanel The panel to gain focus once we are closed
    *  @param externalLinkDraft The link to edit
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel ( QuestPanel previousPanel, ExternalLinkDraft externalLinkDraft, boolean autoPublish ) {
      this.previousPanel = previousPanel ;
      this.externalLink  = externalLinkDraft ;
      this.autoPublish   = autoPublish ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our panel and
    *  start editing the QuestLink passed in
    *  
    *
    *  @param previousPanel The panel to gain focus once we are closed
    *  @param externalLinkDraft The link to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel ( QuestPanel previousPanel, QuestLinkDraft questLinkDraft, boolean autoPublish ) {
      this.previousPanel = previousPanel ;
      this.questLink     = questLinkDraft ;
      this.autoPublish   = autoPublish ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will read in the draft that
    *  is represented by the linkInd passed in
    *  regardless of the type of link (Quest or External)
    *
    *  @param previousPanel The panel to set after we close
    *  @param linkInd The link indicator to read in. This should be the active link indicator
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkEditorPanel ( QuestPanel previousPanel, int linkInd, int linkType ) {
      this.previousPanel = previousPanel ;
      this.linkInd  = linkInd ;
      this.linkType      = linkType ;
      readFromDatabase = true ;
   }

   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create all of our GUI and will
    *  initialize our local variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true if success
    *//*---------------------------------------------------------*/
   public boolean initialize () {     
      Color lightGray = new Color ( 240, 240, 240 ) ;                                

      if ( ( questLink != null ) || ( externalLink != null ) || ( readFromDatabase ) ) {
         JScrollPane usageScroll = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tabbedPane = new JTabbedPane() ;
         draftPanel = new DraftEditPanel () ;
         linkPanel  = new LinkEditPanel () ;        
         usagePanel = new LinkUsagePanel ( this ) ;
         st_TITLE   = new JLabel() ;
         pb_SAVE    = new ActionButton ( Str.getStr ( STR_SAVE ),  ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SAVE ) ) ;
         pb_CLOSE   = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         st_TITLE.setFont ( FontSystem.largeTitleFont ) ;
         setBackground ( Color.white ) ;
         draftPanel.setShowCountry     ( false ) ;
         draftPanel.setShowObjectTypes ( false ) ;           
         draftPanel.setShowDocClass    ( false ) ;
         draftPanel.setShowBrand       ( false ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add action listeners                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_SAVE.addActionListener ( this ) ;
         pb_CLOSE.addActionListener ( this ) ;
         linkPanel.addEditPanelListener ( this ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our GUI                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tabbedPane.addTab ( Str.getStr ( STR_DRAFT ),       draftPanel ) ;
         tabbedPane.addTab ( Str.getStr ( STR_LINK ),        linkPanel ) ;
         tabbedPane.addTab ( Str.getStr ( STR_LINK_USAGE ),  usageScroll = new JScrollPane ( usagePanel ) ) ;
         setLayout ( new BorderLayout() ) ;
         add ( st_TITLE,   BorderLayout.NORTH ) ;
         add ( tabbedPane, BorderLayout.CENTER ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust notebook properties                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tabbedPane.setBorder ( null ) ;
         tabbedPane.setBackgroundAt ( 0, lightGray ) ;
         tabbedPane.setBackgroundAt ( 1, lightGray ) ;
         tabbedPane.setBackgroundAt ( 2, lightGray ) ;
         usageScroll.setBorder ( null ) ;

      }


      return( ( questLink != null ) || ( externalLink != null ) || ( readFromDatabase ) ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we
    *  are showing. We will refresh our data here
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      boolean       refresh   = true ;

      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_SAVE ) ;       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we need to read our question from the database,      */
      /* do it now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( readFromDatabase ) {

         parentWin.setStatus ( STR_PLEASE_WAIT ) ;

         switch ( linkType ) {
            case 0 :
               questLink = (QuestLinkDraft)AvalonCache.readLinkDraft ( linkInd, 
                                                                       linkType, 
                                                                       false ) ;
               break ;

            case 1 :
               externalLink = (ExternalLinkDraft)AvalonCache.readLinkDraft ( linkInd, 
                                                                             linkType, 
                                                                             false ) ;
               break ;
         } /* switch  */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have no valid record to edit, report the error    */                                           
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( questLink == null ) && ( externalLink == null ) ) {
            forceClose = true ;
            GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;
            GUISystem.printBox ( this, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
            refresh = false ;
         }  /* if the question was read */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, if we are locked, tell the user              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( locked() ) {
            GUISystem.printBox ( STR_INFO, STR_DRAFT_LOCKED ) ;
            pb_SAVE.setVisible ( false ) ;
         }


      } /* if we were to read the question */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, lock the record and continue                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( !autoPublish ) && ( getLinkRecStatus() != PersistentRec.REC_INSERT ) )
         setMaintlock ( "Y" ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to auto-publish, do so now                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( !locked() ) && ( autoPublish ) )
         saveAsPublish() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, Now show the initial panel                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !autoPublish )
         showPanel ( initialPanel ) ;

      if ( refresh )
         new Thread ( this, THREAD_REFRESH ).start() ;
      parentWin.setStatus ( null ) ;

   }




   /*------------------------------------------------------------*/
   /* showPanel
    */
   /**
    *    This method will show the tabbed pane panel
    *  that is passed
    *  
    *
    *  @param panelName The name of the panel as defined in this class ( PANEL_DRAFT etc.. )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showPanel ( String panel ) {

      if ( panel.equals ( PANEL_DRAFT ) )
         tabbedPane.setSelectedComponent ( draftPanel ) ;

      else if ( panel.equals ( PANEL_USAGE ) ) {
         tabbedPane.setSelectedComponent ( WinUtil.getParentJScrollPane ( usagePanel ) ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.DOC_LINK ) ) ;
   } /* getIcon  */





   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected() {
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return a EditNavPanel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the user
    *  wants to close us. If our data has not changed
    *  we will return TRUE , otherwise we will prompt the user to save
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean stop () {
      boolean rc = false ;                                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to force close, bypass everything             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !forceClose ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are not publishing, continue with the close       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !publishing ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the user has not saved, ask them to                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !saved ) {


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If the record is not locked, ask the user               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !locked() ) {
                  if ( GUISystem.printBox ( STR_QUESTION, STR_SAVE_LINK ) )
                     rc = save() ;
                  else {
                     /* Unlock the record */
                     if ( getLinkRecStatus() == PersistentRec.REC_INSERT )
                        setOpenDraft ( "N" ) ;
                     rc = true ; // Set the return to TRUE
                  }
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise we cannot save, so return TRUE                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  rc = true ;       // Return TRUE since no link was taken
                  saved = false ;   // set Saved to FALSE since we did not save
               } /* else */

            } else {
               rc = true ; // Set the return to TRUE since they already saved
            }

         } else {
            GUISystem.printBox ( STR_INFO, STR_CANNOT_CLOSE_PUBLISHING ) ;
            rc = false ; // Set the return to FALSE since we cannot close if publishing
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notify our listeners that we have saved                 */
         /* and update our database flags                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc ) {
            if ( !locked() )
               setMaintlock ( "N" ) ;

            if ( published )
               setOpenDraft ( "N" ) ;

            super.stop() ;
         }


      } else
         rc = true ; // Set return to TRUE since we are forcing the close


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set saved to false for next time around                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      saved = false ;

      if ( rc )
         panelDeselected() ;

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the user
    *  wants to shutdown the application. We will
    *  return TRUE since stop was already called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      System.out.println ( "Shutdown" ) ;
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return LinkEditorPanel
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      if ( questLink != null ) {
         Vector vec = questLink.getLinkTitles() ;

         linkPanel.setLink ( questLink ) ;
         draftPanel.setDraftObject ( (OAInterface)questLink ) ;
         st_TITLE.setText ( vec.elementAt(vec.size()-1).toString() ) ;
         draftPanel.setDraftIndicator     ( ""+questLink.getInd() ) ;
         draftPanel.setActiveIndicator    ( ""+questLink.getActiveLinkInd() ) ;
         usagePanel.setLink ( questLink ) ;
      }

      else if ( externalLink != null ) {
         Vector vec = externalLink.getLinkTitles() ;

         linkPanel.setLink ( externalLink ) ;
         draftPanel.setDraftObject ( (OAInterface)externalLink ) ;
         st_TITLE.setText ( vec.elementAt(vec.size()-1).toString() ) ;
         draftPanel.setDraftIndicator     ( ""+externalLink.getInd() ) ;
         draftPanel.setActiveIndicator    ( ""+externalLink.getActiveLinkInd() ) ;
         usagePanel.setLink ( externalLink ) ;
      }


   } /* refresh  */





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_LINK_EDITOR ) ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an event
    *  is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( Qit.getEnabled() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_CLOSE                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == pb_CLOSE )
            GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_SAVE                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SAVE ) {
            if ( save() ) {
               try {
                  if ( PropertySystem.getBool ( PropertySystem.PROPERTY_SAVE_AND_CLOSE ) )
                     GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;
               } catch ( Exception e1 ) {
                  LogSystem.log ( 1, e1 ) ;
               }
            }
         }

      }

   } /* actionPerformed  */







   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save all of the data on our
    *  screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveData() {
      return( ( draftPanel.saveData() ) && ( linkPanel.saveData() )) ;
   }



   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will attempt to save the data
    *  on the screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if data was saved, FALSE if not
    *//*---------------------------------------------------------*/
   private boolean save () {
      boolean        rc          = false ;
      int            saveType    = -1 ;
      String         comments    = null ;
      SaveTypeDlg    saveTypeDlg = null ;                                                                                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only continue if the user has entered all of the data   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( saveData() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ask the user what type of save they would like          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         saveTypeDlg = new SaveTypeDlg ( WinUtil.getParentFrame ( this ), 
                                         draftPanel.getOriginalComments(),
                                         draftPanel.getComments(),
                                         (questLink != null) ? questLink.isArchived() :
                                         externalLink.isArchived(),
                                         (questLink !=null) ? questLink.getTypeWorkRequired().getInd() == TypeList.WORKREQ_DELETE :
                                         externalLink.getTypeWorkRequired().getInd() == TypeList.WORKREQ_DELETE ) ;
         saveType = saveTypeDlg.getResult() ;
         comments = saveTypeDlg.getComments() ;

         if ( questLink != null )
            questLink.setComments ( comments ) ;
         else
            externalLink.setComments ( comments ) ;


         switch ( saveType ) {
            case SaveTypeDlg.DRAFT :
               rc = saveAsDraft() ;
               break ;

            case SaveTypeDlg.PUBLISH :
               if ( validateLink() ) {
                  if ( saveAsDraft() )
                     saveAsPublish() ;
               }
               break ;

            case SaveTypeDlg.ARCHIVE :
               if ( validateLink() ) {
                  if ( questLink != null )
                     questLink.setArchived ( true ) ;
                  else
                     externalLink.setArchived ( true ) ;

                  if ( saveAsDraft ( ) ) {
                     if ( questLink != null )
                        rc = OAObjectArchiver.archiveObject ( questLink, this ) ;
                     else
                        rc = OAObjectArchiver.archiveObject ( externalLink, this ) ;

                     if ( !rc )
                        reopenDraft() ;
                  }
               }
               break ;

            case SaveTypeDlg.UNARCHIVE :
               if ( validateLink() ) {
                  if ( questLink != null )
                     questLink.setArchived ( false ) ;
                  else
                     externalLink.setArchived ( false ) ;
                  if ( saveAsDraft() )
                     saveAsPublish() ;
               }
               break ;

            case SaveTypeDlg.CANCEL_DRAFT :
               boolean saveClose = false ;

               if ( questLink != null )
                  saveClose = OASQL.cancelDraft ( questLink ) ;
               else
                  saveClose = OASQL.cancelDraft ( externalLink ) ;

               if ( saveClose ) {
                  saved = true ;
                  forceClose = true ;
                  parentWin.closePanel ( this, previousPanel ) ;
               }
               break ;

         } /* switch */
      }

      saved = rc ;
      return( rc ) ;
   } /* saveData  */





   /*------------------------------------------------------------*/
   /* saveAsDraft
    */
   /**
    *    This method will save the link as a draft
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if the save was ok
    *//*---------------------------------------------------------*/
   private boolean saveAsDraft () {
      boolean rc = false ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Disable the app                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      Qit.setEnabled ( false ) ;
      tabbedPane.setEnabled ( false ) ;
      GUISystem.getParentDefWin ( this ).setStatus ( STR_SAVING_LINK ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save the Data                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {

         if ( questLink != null )
            rc = ( questLink.writeToDatabase() == 0 ) ;

         else if ( externalLink != null )
            rc = ( externalLink.writeToDatabase() == 0 ) ;

         if ( !rc )
            GUISystem.printBox ( this, STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         rc = false ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Re-enable the app                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      tabbedPane.setEnabled ( true ) ;
      Qit.setEnabled ( true ) ;
      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;               

      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform a variety of
    *  operations. The operation done depends on the
    *  name of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_REFRESH ) )
         refresh() ;

   }




   /*------------------------------------------------------------*/
   /* editPanelChanged
    */
   /**
    *    This method will be called when a panel
    *  we are listening to has changed it's data
    *  
    *
    *  @param editPanel The panel that changed
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void editPanelChanged ( EditPanel editPanel ) {
      st_TITLE.setText ( linkPanel.getTitle() ) ;
   }



   /*------------------------------------------------------------*/
   /* locked
    */
   /**
    *    This method will return TRUE if this link
    *  draft being edited is locked in the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   private boolean locked () {
      boolean   locked = false;
      LinkDraft link   = getLink() ;

      if ( link != null ) {
         if ( link != null ) {
            if ( link.getMaintLock() ) {
               locked =  !( link.getDbUser().equals ( UserSystem.getUserId() ) ) ;
            }
         }
      }

      return( locked ) ;
   }




   /*------------------------------------------------------------*/
   /* setMaintLock
    */
   /**
    *    This method will set the maint lock flag
    *  to the value passed
    *  
    *
    *  @param value The value to set maintlock in OADRAFT.QUESTION to. Either Y or N
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setMaintlock ( String value ) {
      if ( !locked() ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setMaintLock("+value+")", 5 ) ;
         LinkDraft link      = getLink() ;

         if ( link != null ) {
            try {
               Statement stmt = sqlMethod.createStatement() ;

               stmt.executeUpdate ( "UPDATE OADRAFT.LINK "+
                                    "SET MAINTLOCK = '"+value+"', "+
                                    "    DBUSER    = '"+UserSystem.getUserId()+"', "+
                                    "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                    "WHERE LINKIND = "+link.getInd() ) ;
            } catch ( Exception e ) {
               sqlMethod.rollBack() ;
               LogSystem.log ( 1, e, false ) ;
            }

            sqlMethod.close() ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* getLink
    */
   /**
    *    This method will return the LinkDraft
    *  object that we are editing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private LinkDraft getLink () {
      LinkDraft retDraft = null ;

      if ( externalLink != null )
         retDraft = externalLink ;
      else
         retDraft = questLink ;

      return( retDraft ) ;
   }


   /*------------------------------------------------------------*/
   /* saveAsPublish
    */
   /**
    *    This method will start the publishing
    *  process
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveAsPublish () {               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put the publish panel up                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( validateLink() ) {
         if ( questLink != null )
            publishPanel = new PublishPanel ( questLink ) ;
         else
            publishPanel = new PublishPanel ( externalLink ) ;

         publishPanel.setPublishPanelListener ( this ) ;
         remove ( tabbedPane ) ;
         add ( publishPanel, BorderLayout.CENTER ) ;
         pb_SAVE.setVisible ( false ) ;
         pb_CLOSE.setVisible ( false ) ;
         publishing = true ;
         new Thread ( publishPanel, publishPanel.THREAD_VALIDATE ).start() ;
      }
   }





   /*------------------------------------------------------------*/
   /* publishComplete
    */
   /**
    *    This method will be called once a publish has
    *  completed
    *  
    *
    *  @param complete TRUE = The object was published, FALSE if not
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void publishComplete ( boolean complete ) {
      if ( complete ) {
         remove ( publishPanel ) ;
         publishing = false ;
         saved = true ;
         parentWin.closePanel ( this, previousPanel ) ;
      } else {
         pb_SAVE.setVisible ( true ) ;
         pb_CLOSE.setVisible ( true ) ;
         remove ( publishPanel ) ;
         publishPanel = null ;
         add ( tabbedPane ) ;
         publishing = false ;
         setVisible ( false ) ;
         setVisible ( true ) ;
      }
   }

   /*------------------------------------------------------------*/
   /* setOpenDraft
    */
   /**
    *    This method will set the open draft flag
    *  to the value passed
    *  
    *
    *  @param value The value to set opendraft in OA.QUESTION to. Either Y or N
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setOpenDraft ( String value ) {
      if ( getLink() != null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setOpenDraft("+value+")", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;
            LinkDraft link = getLink() ;

            stmt.executeUpdate ( "UPDATE OA.LINK "+
                                 "SET OPENDRAFT = '"+value+"', "+
                                 "    DBUSER    = '"+UserSystem.getUserId()+"', "+
                                 "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE LINKIND = "+link.getActiveLinkInd() ) ;
         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
         }

         sqlMethod.close() ;
      }
   }





   /*------------------------------------------------------------*/
   /* getLinkRecStatus
    */
   /**
    *    This method will return the rec status
    *  of the link we are editing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int getLinkRecStatus () {
      if ( questLink != null )
         return( questLink.getRecStatus() ) ;
      return( externalLink.getRecStatus() ) ;
   }

   public void spellCheckComplete ( EditPanel editPanel, boolean errors ) {
   }




   /*------------------------------------------------------------*/
   /* reopenDraft
    */
   /**
    *    This method will reopen the draft we are
    *  editing by setting the stop date to null, it will
    *  also reset the MaintLock to 'Y'.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void reopenDraft () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "reopenDraft", 5 ) ;

      GUISystem.getParentDefWin ( this ).setStatus ( STR_READING ) ;

      try {
         Statement stmt    = sqlMethod.createStatement() ;
         int       linkInd = (questLink != null) ? questLink.getInd() : externalLink.getInd() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.LINK SET STOPDATE=NULL, MAINTLOCK='Y' "+
                              "WHERE LINKIND = "+linkInd ) ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;                     


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we must reload the object so that all values will  */
      /* now be update.                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( questLink != null )
         questLink = (QuestLinkDraft)AvalonCache.readLinkDraft ( questLink.getInd (), QUEST_LINK ) ;

      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;

   }    


   /*------------------------------------------------------------*/
   /* validateLink
    */
   /**
    *    This method will validate the link being
    *  edited. It will return TRUE if the validation was
    *  without error
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean validateLink () {
      if ( questLink != null )
         return( !(new SoftValidateDlg ( GUISystem.getParentDefWin ( this ), 
                                         LinkSoftValidator.validateLink ( questLink ) ).getHadErrors()) ) ;

      return( !(new SoftValidateDlg ( GUISystem.getParentDefWin ( this ), 
                                      LinkSoftValidator.validateLink ( externalLink ) ).getHadErrors()) ) ;
   }

} /* LinkEditorPanel */


