package com.ibm.nzna.projects.qit.avalon.editors ;         
/*-----------------------------------------------------------*/
/*   LinkGroupEditPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkGroupEditPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This editor allows a user to manipulate a LinkGroup. 
 *
 *
 * @version 1.0, 4/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkGroupEditPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkGroupEditPanel extends EditPanel implements KeyListener, SelectOAListener, ActionListener, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   titles                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] cnrTitles = { "", "", "", "", "", ""} ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ActionButton    pb_CLOSE     = null ;
   private ActionButton    pb_SAVE      = null ;
   private JTitle          st_TITLE     = null ;
   private JTextField      ef_TITLE     = null ;
   private MultiList       cnr_LINKS    = null ;
   private HotLinkLabel    pb_ADD       = null ;
   private HotLinkLabel    pb_REMOVE    = null ;
   private ButtonPanel     buttonPanel  = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private LinkGroupDraft linkGroup         = null ;  // The linkGroup we are editing
   private QuestPanel     previousPanel     = null ;  // The panel to go back to when we close
   private boolean        saved             = false ; // TRUE if we save this object





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor will initialize as
    *  a 'New' Link Group. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupEditPanel () {
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will initiailze as an 'Edit'
    *  Link Group
    *  
    *
    *  @param linkGroup The LinkGroup to edit
    *  @param previousPanel The panel to show when this panel closes
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupEditPanel ( LinkGroupDraft linkGroup ) {
      this.linkGroup = linkGroup ;
      initialize() ;
      refresh() ;
   }



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create all of our GUI and will
    *  initialize our local variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true if success
    *//*---------------------------------------------------------*/
   private void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE     = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_SAVE      = new ActionButton ( Str.getStr ( STR_SAVE ),  ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SAVE_LINK_GROUP ) ) ;
      st_TITLE     = new JTitle ( Str.getStr ( STR_TITLE ) ) ;
      ef_TITLE     = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_OA_TITLE ), "", 0 ) ;
      cnr_LINKS    = new MultiList ( GUISystem.getFontUtil () ) ;
      pb_ADD       = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE    = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      buttonPanel  = new ButtonPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      st_TITLE.setFont ( FontSystem.smallTitleFont ) ;
      GUISystem.setPreferredButton ( pb_ADD ) ;       
      GUISystem.setPreferredButton ( pb_REMOVE ) ;
      cnr_LINKS.setBorder ( GUISystem.blackBorder ) ;
      cnr_LINKS.setColumnHeadings ( cnrTitles ) ;
      cnr_LINKS.showColumnHeadings ( false ) ;
      cnr_LINKS.setColumnWidth ( 0, 12 ) ;
      cnr_LINKS.setColumnWidth ( 1, 12 ) ;
      cnr_LINKS.setColumnWidth ( 2, 50 ) ;
      cnr_LINKS.setColumnWidth ( 3, 250 ) ;
      cnr_LINKS.setColumnWidth ( 4, 150 ) ;
      cnr_LINKS.setColumnWidth ( 5, 150 ) ;
      buttonPanel.setBorder     ( GUISystem.grayBorder ) ;
      buttonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD.addActionListener    ( this ) ;
      pb_REMOVE.addActionListener ( this ) ;
      pb_SAVE.addActionListener   ( this ) ;
      pb_CLOSE.addActionListener  ( this ) ;
      ef_TITLE.addKeyListener     ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      buttonPanel.add ( pb_ADD ) ;
      buttonPanel.add ( new ToolBarSeparator() ) ;
      buttonPanel.add ( pb_REMOVE ) ;
      buttonPanel.add ( new ToolBarSeparator() ) ;
      add ( st_TITLE ) ;
      add ( ef_TITLE ) ;
      add ( cnr_LINKS ) ;
      add ( buttonPanel ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       y         = 5 ;
      int       x         = 5 ;

      super.doLayout() ;


      st_TITLE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=(rowHeight) ;

      ef_TITLE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=(rowHeight*2) ;

      buttonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cnr_LINKS.setBounds ( x, y, size.width-(x*2), size.height-y-5 ) ;
   }

   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      if ( linkGroup != null ) {
         Vector titleVec = linkGroup.getLinkGroupTitles() ;
         Vector linkVec  = linkGroup.getLinks() ;          

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Clear the links                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         cnr_LINKS.removeAll() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our title                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( titleVec.size() > 0 )
            ef_TITLE.setText ( titleVec.elementAt ( 0 ).toString() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Refresh all of our links                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( linkVec != null ) {
            Vector displayVec = LinkDisplayRec.convert ( linkVec ) ;

            cnr_LINKS.add ( displayVec ) ;
         } /* if linkVec  */


      } /* if LinkGroup  */
   } /* refresh  */



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an event
    *  is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_ADD                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_ADD ) {
         SelectOADlg selectDlg = new SelectOADlg().getInstance() ;

         selectDlg.setView ( selectDlg.VIEW_LINK ) ;
         selectDlg.addSelectOAListener ( this ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_REMOVE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE )
         cnr_LINKS.removeSelectedItems() ;

   } /* actionPerformed  */





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will attempt to save the data
    *  on the screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if data was saved, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      int     rc        = 0 ;
      boolean savedData = false ;
      Vector  linkVec   = QuestUtil.getDataFromDisplay ( cnr_LINKS.getData() ) ;

      try {

         if ( saveTitle ( true ) ) {
            if ( ( linkVec != null ) && ( linkVec.size() > 0 ) ) {

               linkGroup.setLinks ( linkVec ) ;

               savedData = true ;
            } else
               GUISystem.printBox ( this, STR_INFO, STR_LINK_GROUP_MUST_CONTAIN_LINK ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         savedData = false ;
      }

      return( savedData ) ;

   } /* saveData  */





   /*------------------------------------------------------------*/
   /* saveTitle
    */
   /**
    *    This method will save the title information
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveTitle ( boolean reportErrors ) {
      String  titleStr  = ef_TITLE.getText() ;
      Vector  titles    = linkGroup.getLinkGroupTitles() ;
      boolean saveData  = false ;

      if ( ( titleStr != null ) && ( titleStr.length() > 0 ) ) {
         if ( titles == null ) {
            LanguageText title = new LanguageText ( titleStr ) ;

            title.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                         TypeList.TYPE_LANGUAGE ) ) ;
            titles = new Vector ( 1 ) ;
            titles.addElement ( title ) ;

         } else
            ((LanguageText)titles.elementAt(0)).setText ( titleStr ) ;

         saveData = true ;
      } else {
         if ( reportErrors ) {
            GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_VALUE ) ;
            ef_TITLE.requestFocus() ;
         }
      }

      return( saveData ) ;
   }


   /*------------------------------------------------------------*/
   /* selectOAComplete
    */
   /**
    *    This method will be called once the SelectOADlg
    *  has completed a selection. We will take the selection
    *  and add it to the list of links
    *
    *  @param vector The vector of links
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectOAComplete ( Vector vector ) {
      if ( vector != null ) {
         Vector currentLinkVec = cnr_LINKS.getSelection() ;                   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   We must ensure no duplicates                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( currentLinkVec != null ) {
            int i          = 0 ;
            int vecSize    = vector.size() ;

            while ( i < vecSize ) {
               if ( vector.elementAt(i) instanceof LinkDisplayRec )
                  if ( !StoreUtil.vectorContains ( currentLinkVec, vector.elementAt(i) ) ) {
                     cnr_LINKS.add ( (MultiListRow) vector.elementAt(i) ) ;
                  }
               i++ ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, just add them all                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            int i    = 0 ;
            int size = vector.size() ;

            while ( i < size ) {
               //cnr_LINKS.add ( new LinkDisplayRec ( (Link)linkVec.elementAt(i) ) ) ;
               i++ ;
            }
         }

      }
   }





   /*------------------------------------------------------------*/
   /* setLinkGroup
    */
   /**
    *    This method will set the LinkGroupDraft that
    *  will be edited by this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setLinkGroup ( LinkGroupDraft linkGroup ) {
      this.linkGroup = linkGroup ;
      refresh() ;
   }



   /*------------------------------------------------------------*/
   /* keyTyped
    */
   /**
    *    We have subclassed this method so that
    *  we will automagically update the title in both
    *  the entry field and in the Title object
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void keyTyped ( KeyEvent e ) {
      saveTitle ( false ) ;
      fireChangedEvent() ;    
   }
   public void keyPressed ( KeyEvent e ) {
      saveTitle ( false ) ;
      fireChangedEvent() ;    
   }
   public void keyReleased ( KeyEvent e ) {
      saveTitle ( false ) ;
      fireChangedEvent() ;    
   }

} /* LinkGroupEditPanel */

