package com.ibm.nzna.projects.qit.avalon.editors ;         
/*-----------------------------------------------------------*/
/*   LinkGroupEditorPanel                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkGroupEditorPanel                */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This editor allows a user to manipulate a LinkGroup. 
 *
 *
 * @version 1.0, 4/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkGroupEditorPanel                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkGroupEditorPanel extends OAEditorPanel implements EditPanelListener, PublishPanelListener, Runnable, ActionListener, QuestPanel, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESH      = "REF" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Panel Names                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String PANEL_USAGE       = "USAGE" ;
   public static final String PANEL_DRAFT       = "DRAFT" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   titles                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] cnrTitles = { "", "", "", "", "", ""} ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ActionButton        pb_CLOSE       = null ;
   private ActionButton        pb_SAVE        = null ;
   private DraftEditPanel      draftPanel     = null ;
   private LinkGroupEditPanel  linkGroupPanel = null ;
   private JTabbedPane         tabbedPane     = null ;
   private PublishPanel        publishPanel   = null ;
   private LinkGroupUsagePanel usagePanel     = null ; 
   private JLabel              st_TITLE       = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean        published         = false ; // TRUE if the object was published Ok
   private LinkGroupDraft linkGroup         = null ;  // The linkGroup we are editing
   private QuestPanel     previousPanel     = null ;  // The panel to go back to when we close
   private boolean        saved             = false ; // TRUE if we save this object
   private int            linkGroupInd      = 0 ;     // The link Group indicator to read on startup
   private boolean        readFromDatabase  = false ; // TRUE if we are to read the linkGroupInd on startup
   private boolean        publishing        = false ; // TRUE if we are currently publishing
   private AppDefaultWin  parentWin         = null ;  // The frame we belong to
   private boolean        forceClose        = false;
   private String         initialPanel      = PANEL_DRAFT ; // The initial panel to show when the editor opens
   private boolean        autoPublish       = false ; // Auto-Publishing


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will initiailze as an 'Edit'
    *  Link Group
    *  
    *
    *  @param previousPanel The panel to show when this panel closes
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupEditorPanel ( QuestPanel previousPanel ) {
      linkGroup = (LinkGroupDraft) new NewAvalonObjectWizard ( GUISystem.getParentDefWin ( previousPanel.getJComponent() ), 
                                                               AVALON_LINK_GROUP ).getNewObject() ;
      this.previousPanel = previousPanel ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the panel
    *  and edit the record passed in
    *  
    *
    *  @param previousPanel The panel to show when this panel closes
    *  @param linkGroupDraft The Link Group Draft to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupEditorPanel ( QuestPanel previousPanel, LinkGroupDraft linkGroupDraft ) {
      this.linkGroup     = linkGroupDraft ;
      this.previousPanel = previousPanel ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the panel
    *  and edit the record passed in
    *  
    *
    *  @param previousPanel The panel to show when this panel closes
    *  @param linkGroupDraft The Link Group Draft to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupEditorPanel ( QuestPanel previousPanel, LinkGroupDraft linkGroupDraft, boolean autoPublish ) {
      this.linkGroup     = linkGroupDraft ;
      this.autoPublish   = autoPublish ;
      this.previousPanel = previousPanel ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the panel
    *  and edit the record passed in
    *  
    *
    *  @param previousPanel The panel to show when this panel closes
    *  @param linkGroupDraft The Link Group Draft to edit
    *  @param initialPanel The initial panel to show
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupEditorPanel ( QuestPanel previousPanel, LinkGroupDraft linkGroupDraft, String initialPanel ) {
      this.linkGroup     = linkGroupDraft ;
      this.previousPanel = previousPanel ;
      this.initialPanel  = initialPanel ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create and show the panel,
    *  after showing it will read in the LinkGroupDraft
    *  indicated by the linkGroupDraftInd
    *
    *  @param previousPanel The panel to show when this panel closes
    *  @param linkGroupDraftInd The indicator of the draft to read on startup
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkGroupEditorPanel ( QuestPanel previousPanel, int linkGroupDraftInd ) {
      this.linkGroupInd  = linkGroupDraftInd ;
      this.previousPanel = previousPanel ;
      readFromDatabase   = true ;
   }

   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create all of our GUI and will
    *  initialize our local variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true if success
    *//*---------------------------------------------------------*/
   public boolean initialize () {     
      Color lightGray = new Color ( 240, 240, 240 ) ;

      if ( ( linkGroup != null ) || ( linkGroupInd != 0 ) ) {
         JScrollPane usageScroll = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_CLOSE       = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
         pb_SAVE        = new ActionButton ( Str.getStr ( STR_SAVE ),  ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SAVE_LINK_GROUP ) ) ;
         linkGroupPanel = new LinkGroupEditPanel () ;
         draftPanel     = new DraftEditPanel () ;
         tabbedPane     = new JTabbedPane () ;
         usagePanel     = new LinkGroupUsagePanel ( this ) ;
         st_TITLE       = new JLabel () ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setBackground ( Color.white ) ;
         tabbedPane.setBorder ( null ) ;
         draftPanel.setShowObjectTypes ( false ) ;
         draftPanel.setShowBrand       ( false ) ;
         draftPanel.setShowCountry     ( false ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_SAVE.addActionListener   ( this ) ;
         pb_CLOSE.addActionListener  ( this ) ;                
         linkGroupPanel.addEditPanelListener ( this ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the tabs to the panel                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tabbedPane.addTab ( Str.getStr ( STR_DRAFT ),             draftPanel ) ;
         tabbedPane.addTab ( Str.getStr ( STR_LINK_GROUP ),        linkGroupPanel ) ;
         tabbedPane.addTab ( Str.getStr ( STR_LINK_GROUP_USAGE ),  usageScroll = new JScrollPane ( usagePanel ) ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the GUI                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setLayout ( new BorderLayout()  ) ;
         add ( st_TITLE,   BorderLayout.NORTH ) ;
         add ( tabbedPane, BorderLayout.CENTER ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust notebook properties                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tabbedPane.setBorder ( null ) ;
         tabbedPane.setBackgroundAt ( 0, lightGray ) ;
         tabbedPane.setBackgroundAt ( 1, lightGray ) ;
         tabbedPane.setBackgroundAt ( 2, lightGray ) ;
         usageScroll.setBorder ( null ) ;
      }


      return( ( linkGroup != null ) || ( linkGroupInd != 0 )  ) ;
   }




   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we
    *  are showing. We will refresh our data here
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      boolean       refresh   = true ;

      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_SAVE ) ;       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we need to read our question from the database,      */
      /* do it now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( readFromDatabase ) {
         try {
            parentWin.setStatus ( STR_PLEASE_WAIT ) ;
            linkGroup = AvalonCache.readLinkGroupDraft ( linkGroupInd ) ;
            if ( linkGroup == null ) {
               forceClose() ;
               GUISystem.printBox ( this, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
               refresh = false ;
            }  /* if the question was read */
         } catch ( Exception e ) {
            forceClose() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise if this record is locked, tell the user       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( locked() ) {
            pb_SAVE.setVisible ( false ) ;
            GUISystem.printBox ( STR_INFO, STR_DRAFT_LOCKED ) ;
         }

      } /* if we were to read the question */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Maintlock if this is not an insert                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( linkGroup != null ) && ( !autoPublish ) ) {
         if ( ( !locked() ) && ( linkGroup.getRecStatus() != linkGroup.REC_INSERT ) )
            setMaintlock ( "Y" ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to auto-publish, do so now                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( !locked() ) && ( autoPublish ) )
         saveAsPublish() ;

      if ( refresh )
         new Thread ( this, THREAD_REFRESH ).start() ;

      parentWin.setStatus ( null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, Now show the initial panel                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !autoPublish )
         showPanel ( initialPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* showPanel
    */
   /**
    *    This method will show the tabbed pane panel
    *  that is passed
    *  
    *
    *  @param panelName The name of the panel as defined in this class ( PANEL_DRAFT etc.. )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showPanel ( String panel ) {

      if ( panel.equals ( PANEL_DRAFT ) )
         tabbedPane.setSelectedComponent ( draftPanel ) ;

      else if ( panel.equals ( PANEL_USAGE ) ) {
         tabbedPane.setSelectedComponent ( WinUtil.getParentJScrollPane ( usagePanel ) ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* forceClose
    */
   /**
    *    This method will force close the panel
    *  without asking the user any questions
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void forceClose () {
      forceClose = true ;
      parentWin.closePanel ( this, previousPanel ) ;
   }



   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.DOC_LINK ) ) ;
   } /* getIcon  */





   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected() {
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return a EditNavPanel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the user
    *  wants to close us. If our data has not changed
    *  we will return TRUE , otherwise we will prompt the user to save
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean stop () {
      boolean rc = false ;                                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to force close, bypass everything             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !forceClose ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are not publishing, continue with the close       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !publishing ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the user has not saved, ask them to                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !saved ) {


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If the record is not locked, ask the user               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !locked() ) {
                  if ( GUISystem.printBox ( STR_QUESTION, STR_SAVE_LINK_GROUP ) )
                     rc = save() ;
                  else {
                     /* Unlock the record */
                     if ( linkGroup.getRecStatus() == linkGroup.REC_INSERT )
                        setOpenDraft ( "N" ) ;
                     rc = true ; // Set the return to TRUE
                  }
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise we cannot save, so return TRUE                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  rc = true ;       // Return TRUE since no linkGroup was taken
                  saved = false ;   // set Saved to FALSE since we did not save
               } /* else */

            } else {
               rc = true ; // Set the return to TRUE since they already saved
            }

         } else {
            GUISystem.printBox ( STR_INFO, STR_CANNOT_CLOSE_PUBLISHING ) ;
            rc = false ; // Set the return to FALSE since we cannot close if publishing
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notify our listeners that we have saved                 */
         /* and update our database flags                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc ) {
            if ( !locked() )
               setMaintlock ( "N" ) ;

            if ( saved ) {
               saveComplete ( linkGroup ) ;

               if ( published )
                  setOpenDraft ( "N" ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notify the super to stop                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc )
            super.stop() ;


      } else
         rc = true ; // Set return to TRUE since we are forcing the close


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set saved to false for next time around                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      saved = false ;

      if ( rc )
         panelDeselected() ;

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the user
    *  wants to shutdown the application. We will
    *  return TRUE since stop was already called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return LinkGroupEditorPanel
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      draftPanel.setDraftObject ( linkGroup ) ;
      linkGroupPanel.setLinkGroup ( linkGroup ) ;
      draftPanel.setDraftIndicator     ( ""+linkGroup.getInd() ) ;
      draftPanel.setActiveIndicator    ( ""+linkGroup.getActiveLinkGroupInd() ) ;
      usagePanel.setLink ( linkGroup ) ;
      st_TITLE.setText ( AvalonUtil.getDefaultTitleObject ( linkGroup ).toString ( LanguageText.HTML_TITLE ) ) ;
   } /* refresh  */





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_LINK_GROUP_EDITOR ) ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an event
    *  is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_CLOSE                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == pb_CLOSE )
            GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_SAVE                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SAVE ) {
            if ( ( save() ) && ( PropertySystem.getBool ( PropertySystem.PROPERTY_SAVE_AND_CLOSE ) ) )
               GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;
         }

      } catch ( Exception e1 ) {
         LogSystem.log ( 1, e1, false ) ;
      }

   } /* actionPerformed  */






   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will attempt to save the data
    *  on the screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      return ( linkGroupPanel.saveData() ) ;
   }

   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will attempt to save the data
    *  to the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if data was saved, FALSE if not
    *//*---------------------------------------------------------*/
   private boolean save () {
      boolean        rc          = false ;
      int            saveType    = -1 ;
      String         comments    = null ;
      SaveTypeDlg    saveTypeDlg = null ;                                                                                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only continue if the user has entered all of the data   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( saveData() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ask the user what type of save they would like          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         saveTypeDlg = new SaveTypeDlg ( WinUtil.getParentFrame ( this ), 
                                         draftPanel.getOriginalComments(),
                                         draftPanel.getComments(),
                                         linkGroup.isArchived(),
                                         linkGroup.getTypeWorkRequired ().getInd() == TypeList.WORKREQ_DELETE ) ;

         saveType = saveTypeDlg.getResult() ;
         comments = saveTypeDlg.getComments() ;

         linkGroup.setComments ( comments ) ;


         switch ( saveType ) {
            case SaveTypeDlg.DRAFT :
               rc = saveAsDraft ( ) ;
               break ;

            case SaveTypeDlg.PUBLISH :
               if ( saveAsDraft ( ) )
                  saveAsPublish() ;
               break ;


            case SaveTypeDlg.ARCHIVE :
               linkGroup.setArchived ( true ) ;
               if ( saveAsDraft ( ) ) {
                  rc = OAObjectArchiver.archiveObject ( linkGroup, this ) ;

                  if ( !rc )
                     reopenDraft() ;
               }
               break ;

            case SaveTypeDlg.UNARCHIVE :
               linkGroup.setArchived ( false ) ;
               if ( saveAsDraft ( ) )
                  saveAsPublish() ;
               break ;

            case SaveTypeDlg.CANCEL_DRAFT :
               if ( OASQL.cancelDraft ( linkGroup ) ) {
                  saved = true ;
                  forceClose = true ;
                  parentWin.closePanel ( this, previousPanel ) ;
               }
               break ;

         } /* switch */
      }

      saved = rc ;
      return( rc ) ;
   } /* saveData  */









   /*------------------------------------------------------------*/
   /* saveAsDraft
    */
   /**
    *    This method will save this action
    *  as a draft. It will close the window if the
    *  property is set
    *
    *  @param forPublish - TRUE = This draft is being saved to publish the Action
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveAsDraft ( ) {                         
      boolean rc   = false ;
      boolean cont = false ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, disable all of our controls                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setEnabled ( false ) ;
      tabbedPane.setEnabled ( false ) ;
      Qit.setEnabled ( false ) ;

      try {
         int saveRc = 0 ;

         parentWin.setStatus ( STR_SAVING ) ;
         saveRc = linkGroup.writeToDatabase() ;
         rc = (saveRc == 0) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         setEnabled ( true ) ;
         tabbedPane.setEnabled ( true ) ;
      }

      if ( !rc )
         GUISystem.printBox ( parentWin, STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;

      tabbedPane.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;
      Qit.setEnabled ( true ) ;

      return( rc ) ;
   } /* saveAsDraft  */





   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform a variety of operations.
    *  The operation done depends on the name of
    *  the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      if ( operation.equals ( THREAD_REFRESH ) )
         refresh() ;
   }




   /*------------------------------------------------------------*/
   /* setMaintLock
    */
   /**
    *    This method will set the maint lock flag
    *  to the value passed
    *  
    *
    *  @param value The value to set maintlock in OADRAFT.QUESTION to. Either Y or N
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setMaintlock ( String value ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setMaintLock("+value+")", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.LINKGROUP "+
                              "SET MAINTLOCK = '"+value+"', "+
                              "    DBUSER    = '"+UserSystem.getUserId()+"', "+
                              "    CHANGEDTIME = CURRENT TIMESTAMP "+
                              "WHERE LINKGROUPIND = "+linkGroup.getInd() ) ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;
   }



   /*------------------------------------------------------------*/
   /* locked
    */
   /**
    *    This method will return TRUE if this question
    *  draft being edited is locked in the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   private boolean locked () {
      boolean locked = false;

      if ( linkGroup != null ) {
         if ( linkGroup.getMaintLock() ) {
            locked =  !( linkGroup.getDbUser().equals ( UserSystem.getUserId() ) ) ;
         }
      }

      return( locked ) ;
   }


   /*------------------------------------------------------------*/
   /* saveAsPublish
    */
   /**
    *    This method will start the publishing
    *  process
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveAsPublish () {               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put the publish panel up                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      publishPanel = new PublishPanel ( linkGroup ) ;

      publishPanel.setPublishPanelListener ( this ) ;
      remove ( tabbedPane ) ;
      add ( publishPanel, BorderLayout.CENTER ) ;
      pb_SAVE.setVisible ( false ) ;
      pb_CLOSE.setVisible ( false ) ;
      publishing = true ;
      new Thread ( publishPanel, publishPanel.THREAD_VALIDATE ).start() ;
   }





   /*------------------------------------------------------------*/
   /* publishComplete
    */
   /**
    *    This method will be called once a publish has
    *  completed
    *  
    *
    *  @param complete TRUE = The object was published, FALSE if not
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void publishComplete ( boolean complete ) {
      if ( complete ) {
         remove ( publishPanel ) ;
         publishing = false ;
         saved = true ;
         published = true ;
         parentWin.closePanel ( this, previousPanel ) ;
      } else {
         pb_SAVE.setVisible ( true ) ;
         pb_CLOSE.setVisible ( true ) ;
         remove ( publishPanel ) ;
         publishPanel = null ;
         add ( tabbedPane ) ;
         publishing = false ;
         setVisible ( false ) ;
         setVisible ( true ) ;
      }
   }

   /*------------------------------------------------------------*/
   /* setOpenDraft
    */
   /**
    *    This method will set the open draft flag
    *  to the value passed
    *  
    *
    *  @param value The value to set opendraft in OA.QUESTION to. Either Y or N
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setOpenDraft ( String value ) {
      if ( linkGroup != null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setOpenDraft("+value+")", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            stmt.executeUpdate ( "UPDATE OA.LINKGROUP "+
                                 "SET OPENDRAFT = '"+value+"', "+
                                 "    DBUSER    = '"+UserSystem.getUserId()+"', "+
                                 "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE LINKGROUPIND = "+linkGroup.getActiveLinkGroupInd() ) ;
         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
         }

         sqlMethod.close() ;
      }
   }

   public void spellCheckComplete ( EditPanel editPanel, boolean errors ) {
   }


   /*------------------------------------------------------------*/
   /* editPanelChanged
    */
   /**
    *    This method will be called when something
    *  on the edit panel has changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void editPanelChanged ( EditPanel editPanel ) {
      if ( editPanel == linkGroupPanel ) {
         st_TITLE.setText ( AvalonUtil.getDefaultTitleObject ( linkGroup ).toString ( LanguageText.HTML_TITLE ) ) ;
      }
   }

   /*------------------------------------------------------------*/
   /* reopenDraft
    */
   /**
    *    This method will reopen the draft we are
    *  editing by setting the stop date to null, it will
    *  also reset the MaintLock to 'Y'.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void reopenDraft () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "reopenDraft", 5 ) ;

      GUISystem.getParentDefWin ( this ).setStatus ( STR_READING ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.LINKGROUP SET STOPDATE=NULL, MAINTLOCK='Y' "+
                              "WHERE LINKGROUPIND = "+linkGroup.getInd() ) ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;                     


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we must reload the object so that all values will  */
      /* now be update.                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      linkGroup = AvalonCache.readLinkGroupDraft ( linkGroup.getInd() ) ;

      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;

   }   
} /* LinkGroupEditorPanel */

