package com.ibm.nzna.projects.qit.avalon.editors ;      
/*-----------------------------------------------------------*/
/*   LinkUsagePanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkUsagePanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.shared.util.* ; 
import com.ibm.nzna.shared.gui.* ; 
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This panel shows the usage of a link. The usage will be reported
 * by showing the Actions and Questions that use the Link.
 *
 *
 * @version 1.0, 11/11/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkUsagePanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkUsagePanel extends JPanel implements ActionListener, AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Column Headers                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String[] columns = { "", "", ""} ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private MultiList       actionList         = null ;
   private MultiList       questionList       = null ;
   private MultiList       linkGroupList      = null ;
   private ListActionPanel actionPanel        = null ;
   private ListActionPanel questionPanel      = null ;
   private ListActionPanel refreshPanel       = null ;      
   private ListActionPanel linkGroupPanel     = null ;
   private JTitle          st_ACTIONS         = null ;
   private JTitle          st_QUESTIONS       = null ;
   private JTitle          st_LINK_GROUPS     = null ;
   private HotLinkLabel    pb_EDIT_ACTION     = null ;
   private HotLinkLabel    pb_EDIT_QUESTION   = null ;
   private HotLinkLabel    pb_EDIT_LINK_GROUP = null ;
   private HotLinkLabel    pb_REFRESH         = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private LinkEditorPanel linkEditorPanel = null ;
   private boolean         refreshing      = false ;
   private boolean         refreshed       = false ;
   private LinkDraft       link            = null ;
   private Dimension       prefSize        = new Dimension ( 100, 400 ) ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public LinkUsagePanel ( LinkEditorPanel linkEditorPanel ) {
      this.linkEditorPanel = linkEditorPanel ;
      initialize() ;
   }






   /*------------------------------------------------------------*/
   /* setLink
    */
   /**
    *    This method will set our link
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLink ( LinkDraft link ) {
      this.link = link ;
      refreshed = false ;
   }


   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will iniitalize our GUI and add
    *  it to our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      actionList         = new MultiList ( GUISystem.getFontUtil() ) ;
      questionList       = new MultiList ( GUISystem.getFontUtil() ) ;
      linkGroupList      = new MultiList ( GUISystem.getFontUtil() ) ;
      st_ACTIONS         = new JTitle ( Str.getStr ( STR_ACTIONS ) ) ;
      st_QUESTIONS       = new JTitle ( Str.getStr ( STR_QUESTIONS ) ) ;
      st_LINK_GROUPS     = new JTitle ( Str.getStr ( STR_LINK_GROUPS ) );
      actionPanel        = new ListActionPanel () ;
      questionPanel      = new ListActionPanel () ;
      refreshPanel       = new ListActionPanel () ;
      linkGroupPanel     = new ListActionPanel () ;
      pb_EDIT_ACTION     = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT_QUESTION   = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT_LINK_GROUP = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;                                
      pb_REFRESH         = new HotLinkLabel ( Str.getStr ( STR_REFRESH )+" "+Str.getStr ( STR_ALL ) ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the Properties of the GUI controls               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_ACTIONS.setFont ( FontSystem.smallTitleFont ) ;
      st_QUESTIONS.setFont ( FontSystem.smallTitleFont ) ;
      st_LINK_GROUPS.setFont ( FontSystem.smallTitleFont ) ;  
      setOpaque ( false ) ;          
      setListProperties ( actionList ) ;
      setListProperties ( questionList ) ;
      setListProperties ( linkGroupList ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_EDIT_ACTION.addActionListener     ( this ) ;
      pb_EDIT_QUESTION.addActionListener   ( this ) ;
      pb_REFRESH.addActionListener         ( this ) ;
      pb_EDIT_LINK_GROUP.addActionListener ( this ) ;      
      questionList.addActionListener       ( this ) ;
      actionList.addActionListener         ( this ) ;
      linkGroupList.addActionListener      ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      refreshPanel.add ( pb_REFRESH ) ;
      actionPanel.add ( pb_EDIT_ACTION ) ;
      questionPanel.add ( pb_EDIT_QUESTION ) ;
      linkGroupPanel.add ( pb_EDIT_LINK_GROUP ) ;
      add ( st_ACTIONS ) ;
      add ( actionList ) ;
      add ( actionPanel ) ;
      add ( st_QUESTIONS ) ;
      add ( questionList ) ;
      add ( questionPanel ) ;
      add ( st_LINK_GROUPS ) ;
      add ( linkGroupList ) ;
      add ( linkGroupPanel ) ;
      add ( refreshPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       rowHeight  = GUISystem.getRowHeight () ;
      int       x          = 0 ;
      int       y          = 0 ;
      Dimension size       = getSize() ;
      int       halfWidth  = size.width/2 ;

      super.doLayout() ;

      st_ACTIONS.setBounds ( x, y, halfWidth-5, rowHeight ) ;
      st_QUESTIONS.setBounds ( x+(halfWidth), y, halfWidth-5, rowHeight ) ;
      y+=rowHeight ;

      actionList.setBounds ( x, y, halfWidth-5, rowHeight*6 ) ;
      questionList.setBounds ( x+halfWidth, y, halfWidth-5, rowHeight*6 );
      y+=rowHeight*6 ;

      actionPanel.setBounds ( x, y, halfWidth-5, rowHeight ) ;
      questionPanel.setBounds ( x+halfWidth, y, halfWidth-5, rowHeight ) ;
      y+=(rowHeight*2) ;

      st_LINK_GROUPS.setBounds ( x, y, halfWidth-5, rowHeight ) ;
      y+=rowHeight ;

      linkGroupList.setBounds ( x, y, halfWidth-5, rowHeight*6 ) ;
      y+=rowHeight*6 ;

      linkGroupPanel.setBounds ( x, y, halfWidth-5, rowHeight ) ;
      y+=(rowHeight*2) ;

      refreshPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
   }


   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method is subclassed so that when we
    *  show ourselves for the first time we refresh
    *  our data
    *
    *  @param g The Graphics Context.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      super.paintComponent ( g ) ;

      if ( !refreshed ) {
         refresh() ;
      }
   }



   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will kick off a thread to refresh
    *  our data. If the refresh thread has already been
    *  started then nothing happens
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refresh () {
      if ( !refreshing )
         new Thread ( this ).start() ;
   } /* refresh  */





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our refresh thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      StatusWin statusWin = null ;

      try {
         statusWin = new StatusWin ( GUISystem.getParentDefWin ( this ) ) ;
         statusWin.setMaxValue ( 3 ) ;

         if ( link != null ) {

            actionList.removeAll() ;
            questionList.removeAll() ;
            linkGroupList.removeAll() ;

            statusWin.setText ( Str.getStr ( STR_READING_USAGE ) ) ;

            actionList.add ( LinkUsage.getLinkUsageInActions ( link ) ) ;
            statusWin.setValue ( 1 ) ;

            questionList.add ( LinkUsage.getLinkUsageInQuestions ( link ) ) ;
            statusWin.setValue ( 2 ) ;

            linkGroupList.add ( LinkUsage.getLinkUsageInLinkGroups ( link ) ) ;
            statusWin.setValue ( 3 ) ;


         }

         refreshed = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      statusWin.dispose() ;
      refreshing = false ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to Action events
    *  being fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) { 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REFRESH                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_REFRESH )  {
         if ( !refreshing ) {
            refreshed = false ;
            refresh() ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_QUESTION                                        */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == questionList ) || ( e.getSource() == pb_EDIT_QUESTION ) ) {
         UsageRec usageRec = (UsageRec)questionList.getSelectedItem() ;

         if ( usageRec != null ) 
            OAEdit.edit ( usageRec, linkEditorPanel ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_ACTION                                          */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == actionList ) || ( e.getSource() == pb_EDIT_ACTION ) ) {
         UsageRec usageRec = (UsageRec)actionList.getSelectedItem() ;

         if ( usageRec != null ) 
            OAEdit.edit ( usageRec, linkEditorPanel ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_LINK_GROUP                                      */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == linkGroupList ) || (  e.getSource() == pb_EDIT_LINK_GROUP ) ) {
         UsageRec usageRec = (UsageRec)linkGroupList.getSelectedItem() ;

         if ( usageRec != null ) 
            OAEdit.edit ( usageRec, linkEditorPanel ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* setListProperties
    */
   /**
    *    This method will set the properties on the
    *  MultiList passed. These properties include
    *  the column headings, column width and to turn off 
    *  the visible column headers
    *
    *  @param multiList The MultiList to set properties on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setListProperties ( MultiList multiList ) {
      multiList.setColumnHeadings ( columns ) ;
      multiList.setColumnWidth ( 0, 20 ) ;
      multiList.setColumnWidth ( 1, 75 ) ;
      multiList.setColumnWidth ( 2, 350 ) ;
      multiList.showColumnHeadings ( false ) ;
   }


} /* LinkUsagePanel */
