package com.ibm.nzna.projects.qit.avalon.editors ;
/*-----------------------------------------------------------*/
/*   OAEdit                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OAEdit                              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This class holds static methods that will edit any Avalon object. 
 *
 *
 * @version 1.0, 7/28/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OAEdit                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class OAEdit extends Object implements AppConst, AvalonConst {



   /*------------------------------------------------------------*/
   /* edit
    */
   /**
    *    This method will edit the currently selected
    *  item
    *  
    *
    *  @param object A MutableTreeWrapper object, holding an OA ***DisplayRec. For Example, QuestionDisplayRec, QuestionDraftDisplayRec etc..
    *  @param previousPanel This is the panel that an Editor Panel will go back to once it is closed
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void edit ( MutableTreeWrapper object, QuestPanel previousPanel ) {
      edit ( (DisplayRec)object.getDataObject(), previousPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* edit
    */
   /**
    *    This method will edit an object of the following
    *  types. 
    *  
    *
    *  @param displayRec The DisplayRec to edit
    *  @param previousPanel This is the panel that an Editor Panel will go back to once it is closed
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void edit ( DisplayRec displayRec, QuestPanel questPanel ) {

      if ( questPanel.getJComponent () != null ) {
         GUISystem.getParentDefWin ( questPanel.getJComponent() ).setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
         GUISystem.getParentDefWin ( questPanel.getJComponent() ).setStatusLock ( true ) ;
      }

      if ( ( displayRec instanceof QuestionDraftDisplayRec ) ||
           ( displayRec instanceof QuestionDisplayRec ) )
         editQuestion ( displayRec, questPanel ) ;

      else if ( ( displayRec instanceof ActionDraftDisplayRec ) ||
                ( displayRec instanceof ActionDisplayRec ) )
         editAction ( displayRec, questPanel ) ;

      else if ( ( displayRec instanceof LinkDisplayRec ) ||
                ( displayRec instanceof LinkDraftDisplayRec ) )
         editLink ( displayRec, questPanel ) ;

      else if ( ( displayRec instanceof SymptomDisplayRec ) ||
                ( displayRec instanceof SymptomDraftDisplayRec ) )
         editSymptom ( displayRec, questPanel ) ;

      else if ( ( displayRec instanceof LinkGroupDisplayRec ) ||
                ( displayRec instanceof LinkGroupDraftDisplayRec ) )
         editLinkGroup ( displayRec, questPanel ) ;

      if ( questPanel.getJComponent() != null ) {
         GUISystem.getParentDefWin ( questPanel.getJComponent() ).setStatusLock ( false ) ;
         GUISystem.getParentDefWin ( questPanel.getJComponent() ).setStatus ( null ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* editQuestion
    */
   /**
    *    This method will edit the question object
    *  passed in
    *  
    *
    *  @param object The object representing either the Question or QuestionDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void editQuestion ( Object object, QuestPanel questPanel ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Question Drafts                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof QuestionDraftDisplayRec )
            WindowSystem.createPanel ( new QuestionEditorPanel ( questPanel, ((QuestionDraftDisplayRec)object).getQuestionInd() ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Questions                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof QuestionDisplayRec ) {
            QuestionDraft questionDraft = null ;                  
            int           questionInd   = ((QuestionDisplayRec)object).getQuestionInd() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Try and read/create a draft                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            questionDraft = OASQL.getQuestionDraft ( GUISystem.getParentDefWin ( questPanel.getJComponent() ), 
                                                     questionInd, 
                                                     TypeList.WORKREQ_EDIT ) ;                        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we created the draft, continue                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( questionDraft != null ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read the graphics from the published site               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( AddtlInfoGraphicFTP.getGraphicsPub ( GUISystem.getParentDefWin ( questPanel.getJComponent() ),
                                                         questionDraft.getAdditionalInfo() ) )
                  WindowSystem.createPanel ( new QuestionEditorPanel ( questPanel, questionDraft ) ) ;
            }

         } /* else  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   } /* editQuestion */



   /*------------------------------------------------------------*/
   /* editAction
    */
   /**
    *    This method will edit the action object
    *  passed in
    *  
    *
    *  @param object The object representing either the Action or ActionDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void editAction ( Object object, QuestPanel questPanel ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Action Drafts                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof ActionDraftDisplayRec ) {
            WindowSystem.createPanel ( new ActionEditorPanel ( questPanel, ((ActionDraftDisplayRec)object).getActionInd() ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Actions                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof ActionDisplayRec ) {
            ActionDraft actionDraft = null ;                  
            int         actionInd   = ((ActionDisplayRec)object).getActionInd() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Try and read/create a draft                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            actionDraft = OASQL.getActionDraft ( GUISystem.getParentDefWin ( questPanel.getJComponent() ),actionInd, TypeList.WORKREQ_EDIT ) ;                                      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   IF we were able to create a new Draft, continue         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( actionDraft != null ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read the graphics from the published site               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( AddtlInfoGraphicFTP.getGraphicsPub ( GUISystem.getParentDefWin ( questPanel.getJComponent() ),
                                                         actionDraft.getAdditionalInfo() ) )
                  WindowSystem.createPanel ( new ActionEditorPanel ( questPanel, actionDraft ) ) ;
            }

         } /* else  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   } /* editAction */



   /*------------------------------------------------------------*/
   /* editLink
    */
   /**
    *    This method will edit the link object
    *  passed in
    *  
    *
    *  @param object The object representing either the Link or LinkDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void editLink ( Object object, QuestPanel questPanel ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Link Drafts                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof LinkDraftDisplayRec )
            WindowSystem.createPanel ( new LinkEditorPanel ( questPanel, ((LinkDraftDisplayRec)object).getLinkInd(), ((LinkDraftDisplayRec)object).getLinkType() ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Links                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof LinkDisplayRec ) {
            LinkDraft    questLink    = null ;
            LinkDraft    externalLink = null ;
            int          linkInd      = ((LinkDisplayRec)object).getLinkInd() ;
            int          linkType     = ((LinkDisplayRec)object).getLinkType() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Try and read/create a draft                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            switch ( linkType ) {
               case 0 :
                  questLink = OASQL.getQuestLinkDraft ( GUISystem.getParentDefWin ( questPanel.getJComponent() ), linkInd, TypeList.WORKREQ_EDIT ) ;

                  if ( questLink != null )
                     WindowSystem.createPanel ( new LinkEditorPanel ( questPanel, (QuestLinkDraft) questLink ) ) ;
                  break ;

               case 1 :
                  externalLink = OASQL.getExternalLinkDraft ( GUISystem.getParentDefWin ( questPanel.getJComponent() ), linkInd, TypeList.WORKREQ_EDIT ) ;

                  if ( externalLink != null )
                     WindowSystem.createPanel ( new LinkEditorPanel ( questPanel, (ExternalLinkDraft) externalLink ) ) ;
                  break ;
            }


         } /* else  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

   } /* editLink */


   /*------------------------------------------------------------*/
   /* editLinkGroup
    */
   /**
    *    This method will edit the link object
    *  passed in
    *  
    *
    *  @param object The object representing either the LinkGroup or LinkGroupDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void editLinkGroup ( Object object, QuestPanel questPanel ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   LinkGroup Drafts                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof LinkGroupDraftDisplayRec ) {
            WindowSystem.createPanel ( new LinkGroupEditorPanel ( questPanel, ((LinkGroupDraftDisplayRec)object).getLinkGroupInd() ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   LinkGroups                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof LinkGroupDisplayRec ) {
            LinkGroupDraft    linkGroup    = null ;
            int               linkInd      = ((LinkGroupDisplayRec)object).getLinkGroupInd() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Try and read/create a draft                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            linkGroup = OASQL.getLinkGroupDraft ( GUISystem.getParentDefWin ( questPanel.getJComponent() ), linkInd, TypeList.WORKREQ_EDIT ) ;

            if ( linkGroup != null )
               WindowSystem.createPanel ( new LinkGroupEditorPanel ( questPanel, (LinkGroupDraft) linkGroup ) ) ;

         } /* else  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

   } /* editLinkGroup */



   /*------------------------------------------------------------*/
   /* editSymptom
    */
   /**
    *    This method will edit the symptom object
    *  passed in
    *  
    *
    *  @param object The object representing either the Symptom or SymptomDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void editSymptom ( Object object, QuestPanel questPanel ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Symptom Drafts                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof SymptomDraftDisplayRec )
            WindowSystem.createPanel ( new SymptomEditorPanel ( questPanel, ((SymptomDraftDisplayRec)object).getSymptomInd() ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Symptoms                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof SymptomDisplayRec ) {
            SymptomDraft symptomDraft = null ;                  
            int         symptomInd   = ((SymptomDisplayRec)object).getSymptomInd() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Try and read/create a draft                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            symptomDraft = OASQL.getSymptomDraft ( GUISystem.getParentDefWin ( questPanel.getJComponent() ),symptomInd, TypeList.WORKREQ_EDIT ) ;

            if ( symptomDraft != null )
               WindowSystem.createPanel ( new SymptomEditorPanel ( questPanel, symptomDraft ) ) ;

         } /* else  */

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   } /* editSymptom */






   /*------------------------------------------------------------*/
   /* edit
    */
   /**
    *    This method will edit whatever is represented
    *  by the UsageRec passed in
    *  
    *
    *  @param usageRec Representing the object to edit
    *  @param questPanel The QuestPanel to return to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void edit ( UsageRec usageRec, QuestPanel questPanel ) {

      switch ( usageRec.getObjectType() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Symptoms                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case AVALON_SYMPTOM :
            edit ( (DisplayRec)new SymptomDisplayRec ( usageRec.getReferenceObjectInd(),
                                                       0,
                                                       0,
                                                       "",
                                                       "",
                                                       "" ),
                   questPanel ) ;
            break ;     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Symptom Drafts                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_SYMPTOM_DRAFT :
            edit ( (DisplayRec)new SymptomDraftDisplayRec ( usageRec.getReferenceObjectInd(),
                                                            0,
                                                            0,
                                                            0,
                                                            "",
                                                            "",
                                                            "",
                                                            "",
                                                            "" ),
                   questPanel ) ;
            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Questions                                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_QUESTION :
            edit ( (DisplayRec)new QuestionDisplayRec ( usageRec.getReferenceObjectInd(),
                                                        0,
                                                        "",
                                                        "",
                                                        "" ),
                   questPanel ) ;

            break ;      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Question Drafts                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_QUESTION_DRAFT :
            edit ( (DisplayRec)new QuestionDraftDisplayRec ( usageRec.getReferenceObjectInd(),
                                                             0,
                                                             0,
                                                             "",
                                                             "",
                                                             "",
                                                             "",
                                                             "" ),
                   questPanel ) ;

            break ;    


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Action                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_ACTION :
            edit ( (DisplayRec)new ActionDisplayRec ( usageRec.getReferenceObjectInd(),
                                                      0,
                                                      "",
                                                      "",
                                                      "" ),
                   questPanel ) ;

            break ;      


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Action Drafts                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_ACTION_DRAFT :
            edit ( (DisplayRec)new ActionDraftDisplayRec ( usageRec.getReferenceObjectInd(),
                                                           0,
                                                           0,
                                                           "",
                                                           "",
                                                           "",
                                                           "",
                                                           "" ),
                   questPanel ) ;

            break ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Links                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_LINK :
            edit ( (DisplayRec) new LinkDisplayRec ( usageRec.getReferenceObjectInd(),
                                                     "",
                                                     true,
                                                     0,
                                                     0,
                                                     "",
                                                     "",
                                                     "",
                                                     "" ),
                   questPanel ) ;
            break ;  

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Link Drafts                                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_LINK_DRAFT :
            edit ( (DisplayRec) new LinkDraftDisplayRec ( usageRec.getReferenceObjectInd(),
                                                          0,
                                                          "",
                                                          false,
                                                          0,
                                                          0,
                                                          "","","","","","" ),
                   questPanel ) ;
            break ;  

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Link Groups                                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_LINK_GROUP :
            edit ( (DisplayRec) new LinkGroupDisplayRec ( usageRec.getReferenceObjectInd(),
                                                          "","","" ), questPanel ) ;
            break ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Link Group Drafts                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case AVALON_LINK_GROUP_DRAFT :
            edit ( (DisplayRec) new LinkGroupDraftDisplayRec ( usageRec.getReferenceObjectInd(),
                                                               0, "", "", "","","" ), questPanel ) ;
            break ;

      } /* switch  */
   }

} /* Edit */
