package com.ibm.nzna.projects.qit.avalon.editors ;     
/*-----------------------------------------------------------*/
/*   OAEditorPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OAEditorPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel is the superclass of all panels that edit an object in the Online Assistant(Avalon) 
 *
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OAEditorPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class OAEditorPanel extends JPanel implements AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector listenerVec       = null ;



   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will free all of our resources. You
    *  should always call 'super.stop()' when you subclass
    *  this panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      if ( listenerVec != null )
         listenerVec.removeAllElements () ;
      listenerVec = null ;
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* addOASaveListener
    */
   /**
    *    This method will add a saveListener to this
    *  panel so that it is notified once the object
    *  has been sucessfully saved
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addOASaveListener ( OASaveListener listener ) {
      if ( listenerVec == null )
         listenerVec = new Vector ( 1,1 ) ;

      listenerVec.addElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* removeOASaveListener
    */
   /**
    *    This method will remove a saveListener
    *  from the list of listeners that are notified of
    *  a complete save.
    *
    *  @param saveListener The OASaveListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeOASaveListener ( OASaveListener listener  ) {
      if ( listenerVec != null )
         listenerVec.removeElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* saveComplete
    */
   /**
    *    This method should be called whenever a
    *  save has completed
    *  
    *
    *  @param object The object that was saved
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void saveComplete ( Object object ) {
      if ( listenerVec != null ) {
         new OAEditorNotifyThread ( this, listenerVec, object ).run() ;
      } 
   }


                                               
   public boolean getPublished () {
      return ( false ) ;
   }


   public String getDefaultTitle () {
      return ( "Unknown Default Title" ) ;
   }

} /* OAEditorPanel */


class OAEditorNotifyThread extends Thread implements Runnable {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector        listenerVec       = null ;
   private Object        object            = null ;
   private OAEditorPanel editorPanel       = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and start our
    *  thread
    *  
    *
    *  @param listenerVec The list of listeners to notify
    *  @param object The object to notify the listeners with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OAEditorNotifyThread ( OAEditorPanel editorPanel, Vector listenerVec, Object object ) {
      this.editorPanel = editorPanel ;
      this.listenerVec = listenerVec ;
      this.object      = object ;
   }


   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will simply go down the list
    *  of listeners and notify each one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      int i    = 0 ;

      if ( listenerVec != null ) {
         int size = listenerVec.size() ;

         while ( i < size ) {
            ((OASaveListener)listenerVec.elementAt(i)).saveComplete ( editorPanel, object, editorPanel.getPublished() ) ;
            i++ ;
         } /* while  */
      } /* if  */
   } /* run  */



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }

}/* OAEditorNotifyThread  */
