package com.ibm.nzna.projects.qit.avalon.editors ;
/*-----------------------------------------------------------*/
/*   ObjectNode                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ObjectNode                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.oa.Action ;
import com.ibm.nzna.projects.common.quest.type.TypeList ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * The ObjectNode is an object that simply allows a node to be painted onto the
 * VisioPanel. This node will not interact with the user though. The VisioPanel will
 * create a ComponentNode to interact with.
 *
 *
 * @version 1.0, 4/19/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ObjectNode                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ObjectNode extends Object implements OASaveListener, FlowNode, AppConst, AvalonConst, MultiListRow {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Point         locationBeginPoint   = new Point ( 0, 0 ) ;
   private Point         locationEndPoint     = new Point ( 0, 0 ) ;
   private Node          node                 = null ;
   private Rectangle     nodeBounds           = null ;
   private String[]      nodeDescriptionArray = null ;
   private char[]        nodeIndArray         = null ;
   private char[]        draftArray           = Str.getStr ( STR_DRAFT ).toCharArray() ;
   private FontUtil      fontUtil             = null ;
   private Vector        listeners            = null ;
   private boolean       isDraft              = false ;
   private Object        oaObject             = null ;
   private AppDefaultWin parentWin            = null ;
   private String        nodeDescription      = null ;
   private FontMetrics   fm                   = Toolkit.getDefaultToolkit().getFontMetrics( FontSystem.defaultFont ) ;
   private int           charWidth            = fm.stringWidth ( "li" ) ;
   private boolean       root                 = false ;
   private boolean       paint                = true ;
   private ComponentNode componentNode        = null ;
   private String        nodeIndStr           = null ;
   private Color         backgroundColor      = Color.white ;
   private Color         foregroundColor      = Color.black ;
   private Dimension     size                 = new Dimension ( 0, 0 ) ;
   private boolean       visible              = true ;
   private int           docClassInd          = 0 ;
   private Vector        countryList          = null ;
   private int           oaObjectInd          = 0 ;
   private Symptom       symptom              = null ;
   private int           edgeTitleY           = 0 ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor, it requires
    *  a node.
    *  
    *
    *  @param node The node to represent in the data flow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ObjectNode ( Symptom symptom, Node node ) {
      this.symptom = symptom ;
      setNode ( node ) ;

      fontUtil = GUISystem.getFontUtil() ;
   }




   /*------------------------------------------------------------*/
   /* paint
    */
   /**
    *    This method will paint our component onto the graphics
    *  context being used
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paint ( Graphics g ) {
      if ( paint ) {
         int x         = node.getXPos() ;
         int y         = node.getYPos() ;
         int width     = node.getWidth() ;
         int height    = node.getHeight() ;

         edgeTitleY = 0 ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw our Shape                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         g.setColor ( getBackground() ) ;

         if ( node != null ) {
            switch ( node.getNodeType() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Actions have rounded corners                            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               case Node.NODE_TYPE_ACTION :
                  g.fillRoundRect ( x, y, width, height, 28, 28 ) ;
                  g.setColor ( getForeground() ) ;
                  g.drawRoundRect ( x, y, width, height, 28, 28 ) ;
                  break ;                      

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Symptoms are Diamonds                                   */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
               case Node.NODE_TYPE_SYMPTOM :
                  Polygon p = new Polygon() ;

                  p.addPoint ( x+(width/2), y ) ;
                  p.addPoint ( x+(width), y+(height/2) ) ;
                  p.addPoint ( x+(width/2), y+height ) ;
                  p.addPoint ( x, y+(height/2) ) ;
                  p.addPoint ( x+(width/2), y ) ;

                  g.fillPolygon ( p ) ;
                  g.setColor ( getForeground() ) ;
                  g.drawPolygon ( p ) ;
                  break ;


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Questions and everything else are boxes                 */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
               default:
               case Node.NODE_TYPE_QUESTION :
                  g.fillRect ( x, y, width, height ) ;               
                  g.setColor ( getForeground() ) ;
                  g.drawRect ( x, y, width, height ) ;
                  break ;
            }
         } else {
            g.fillRect ( x, y, width, height ) ;               
            g.setColor ( getForeground() ) ;
            g.drawRect ( x, y, width, height ) ;               
         }

         if ( root ) {
            for ( int i = 0; i < 3; i++ ) {
               g.drawRect ( x+i, y+i, (width)-i*2, (height)-i*2 ) ;
            }
         }

         paintText ( g, x,y,width,height ) ;
      }
   }


   protected void paintText ( Graphics g, int x, int y, int width, int height ) {
      int rowHeight = GUISystem.getRowHeight () ;
      int y1        = 15 ;

      if ( nodeDescriptionArray == null ) {
         setNodeDescription ( nodeDescription ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Draw the description                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( nodeDescriptionArray != null ) {
         int textX = x+5 ;
         int textY = 0 ;

         if ( node != null ) {
            if ( node.getNodeType() == Node.NODE_TYPE_SYMPTOM ) {
               textX+=(width/3) ;
               textY+=10 ;
            } else if ( node.getNodeType() == Node.NODE_TYPE_ACTION )
               textX+=5 ;
         }

         for ( int i = 0; i < nodeDescriptionArray.length; i++ ) {
            g.drawString ( nodeDescriptionArray[i], textX, textY+y1+y ) ;
            y1+=(rowHeight-4) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Draw the Node indicator                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      fontUtil.drawChars ( g, nodeIndArray,         nodeIndArray.length,         x+5, y1+y, (Rectangle)g.getClip(), x+width ) ;
      y1+=rowHeight ;                    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this is a draft, draw that                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( isDraft() )
         fontUtil.drawChars ( g, draftArray,         draftArray.length,         x+5, y1+y, (Rectangle)g.getClip(), x+width ) ;

      g.drawString ( nodeIndStr, x+5, y+height-20 ) ;

   }



   /*------------------------------------------------------------*/
   /* setNode
    */
   /**
    *    This method will set the node that this ObjectNode
    *  represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNode ( Node node ) {          
      int width  = 0 ;
      int height = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our variable                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.node = node ;                                               

      /*-------------------------------------------------------------*/
      /*                                                             */
      /*   If this node has no indicator at all, we must assign one  */
      /*                                                             */
      /*-------------------------------------------------------------*/
      if ( node.getInd() == 0 ) {
         setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
         node.assignDatabaseKey ( true ) ; //Must be a Draft if we are creating it now
         setStatus ( null ) ;
      }



      if ( node.getDraftObjectInd() == 0 ) {
         isDraft = false ;
         oaObjectInd = node.getActiveObjectInd() ;
      } else {
         isDraft = true ;
         oaObjectInd = node.getDraftObjectInd() ;
      }

      nodeIndStr = "N:"+node.getInd() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the description of this node                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( node.getNodeType() ) {
         
         case Node.NODE_TYPE_SYMPTOM :
            if ( ( oaObject != null ) && ( oaObject instanceof Symptom ) )
               setNodeDescription ( AvalonUtil.getDefaultTitleText ( (Symptom)oaObject ) ) ;
            break ;

         case Node.NODE_TYPE_ACTION :
            if ( ( oaObject != null ) && ( oaObject instanceof Action ) )
               setNodeDescription ( AvalonUtil.getDefaultTitleText ( (Action) oaObject ) ) ;
            break ;

         case Node.NODE_TYPE_QUESTION :
            if ( ( oaObject != null ) && ( oaObject instanceof Question ) )
               setNodeDescription ( AvalonUtil.getDefaultTitleText ( (Question) oaObject ) ) ;
            break ;

         case Node.NODE_TYPE_RESOLVED :
            setNodeDescription ( Str.getStr ( STR_NODE_TYPE_RESOLVED ) ) ;
            break ;

         case Node.NODE_TYPE_UNRESOLVED :
            setNodeDescription ( Str.getStr ( STR_NODE_TYPE_UNRESOLVED ) ) ;
            break ;

         case Node.NODE_TYPE_RETURN :
            setNodeDescription ( Str.getStr ( STR_NODE_TYPE_RETURN ) ) ;
            break ;

         case Node.NODE_TYPE_ALIST :
            setNodeDescription ( Str.getStr ( STR_NODE_TYPE_ALIST ) ) ;
            break ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get our bounds                                          */     
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLocation ( node.getXPos(), node.getYPos() ) ;

      if ( node.getWidth() == 0 )
         width=100 ;

      if ( node.getHeight() == 0 )
         height=100 ;

      setSize ( width, height ) ;


      if ( isDraft() )
         nodeIndArray         = new String ( ""+node.getDraftObjectInd() ).toCharArray() ;
      else
         nodeIndArray         = new String ( ""+node.getActiveObjectInd() ).toCharArray() ;

      refreshProperties() ;
   }




   /*------------------------------------------------------------*/
   /* getNodeBounds
    */
   /**
    *    This method will return the bounds that we
    *  are supposed to be showing in. A rectangle
    *  will be returned.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return rect
    *//*---------------------------------------------------------*/
   protected Rectangle getNodeBounds () {
      if ( nodeBounds == null )
         nodeBounds = new Rectangle() ;
      nodeBounds.x = node.getXPos() ;
      nodeBounds.y = node.getYPos() ;
      nodeBounds.width = node.getWidth() ;
      nodeBounds.height = node.getHeight() ;
      return( nodeBounds ) ;
   }

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setSize subclassed to set variables                     */                  
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setSize ( int width, int height ) {
      setBounds ( -1, -1, width, height ) ;
   }



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   setSize subclassed to set variables                     */                  
   /*                                                           */
   /*-----------------------------------------------------------*/
   public void setSize ( Dimension size ) {
      setBounds ( -1, -1, size.width, size.height ) ;
   }






   /*------------------------------------------------------------*/
   /* setBounds
    */
   /**
    *    Sets the width and height and location of our
    *  object
    *  
    *
    *  @param x The X Position of the top left (-1 to not set)
    *  @param y The Y Coordinate of the top Left (-1 to not set)
    *  @param width The Width in pixels (-1 to not set a width)
    *  @param height The Height in pixels (-1 to not set a height)
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBounds ( int x, int y, int cx, int cy ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, adjust our Node's position                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( node.valid() ) {
         boolean widthChange  = false ;
         boolean heightChange = false ;                               


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Find out if our width/height has changed                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( cx >= 0 )
            widthChange = (cx!=node.getWidth());

         if ( cy >= 0 )
            widthChange = (cy!=node.getHeight()) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now set the position                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( x >= 0 )
            node.setXPos ( x ) ;

         if ( y >= 0 )
            node.setYPos ( y ) ;

         if ( cx > 0 )
            node.setWidth ( cx ) ;

         if ( cy > 0 )
            node.setHeight ( cy ) ;

         //if ( ( widthChange ) || 
         //     ( heightChange ) ) {
         //   setNodeDescription ( nodeDescription ) ;
         //}

      }

      locationBeginPoint.x=x ;
      locationBeginPoint.y=y ;
      locationEndPoint.x=x+node.getWidth() ;
      locationEndPoint.y=y+node.getHeight() ;
   }






   /*------------------------------------------------------------*/
   /* setLocation 
    */
   /**
    *    Sets the new location of this node
    *  
    *  
    *
    *  @param x = The X coordinate
    *  @param y = The Y coordinate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLocation ( int x, int y ) {
      setBounds ( x, y, -1, -1 ) ;
   }





   /*------------------------------------------------------------*/
   /* setLocation
    */
   /**
    *    Sets a new location for this node
    *  
    *  
    *
    *  @param pt The location
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLocation ( Point pt ) {
      setBounds ( pt.x, pt.y, -1, -1 ) ;
   }




   /*------------------------------------------------------------*/
   /* getNode
    */
   /**
    *    this method will return the node that this
    *  component represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Node getNode () {
      return( node ) ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title depending on what
    *  type of object this is. Currently titles are only returned for
    *  questions, actions and symptoms. If the title has not been read
    *  yet, the title will be read now.
    *  
    *
    *  @return String The title
    *//*---------------------------------------------------------*/
   public String getTitle () {

      if ( nodeDescription == null ) {

         switch ( node.getNodeType() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Question                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_QUESTION :
               if ( isDraft() )
                  setNodeDescription ( OASQL.readDefaultQuestionTitle ( node.getDraftObjectInd(), true ) ) ;
               else
                  setNodeDescription ( OASQL.readDefaultQuestionTitle ( node.getActiveObjectInd(), false ) ) ;
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Action                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_ACTION :
               if ( isDraft() )
                  setNodeDescription ( OASQL.readDefaultActionTitle ( node.getDraftObjectInd(), true ) ) ;
               else
                  setNodeDescription ( OASQL.readDefaultActionTitle ( node.getActiveObjectInd(), false ) ) ;
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Symptom                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_SYMPTOM :
               if ( isDraft() )
                  setNodeDescription ( OASQL.readDefaultSymptomTitle ( node.getDraftObjectInd(), true ) ) ;
               else
                  setNodeDescription ( OASQL.readDefaultSymptomTitle ( node.getActiveObjectInd(), false ) ) ;
               break ;

            case Node.NODE_TYPE_ALIST :
               setNodeDescription ( Str.getStr ( STR_ACTION_LIST ) ) ;
               break ;

            case Node.NODE_TYPE_RETURN :
               setNodeDescription ( Str.getStr ( STR_RETURN ) ) ;
               break ;

            case Node.NODE_TYPE_RESOLVED :
               setNodeDescription ( Str.getStr ( STR_RESOLVED ) ) ;
               break ;

            case Node.NODE_TYPE_UNRESOLVED :
               setNodeDescription ( Str.getStr ( STR_UNRESOLVED ) ) ;
               break ;
         }

      } /* if nodeDescription was null  */

      return( nodeDescription ) ;
   }




   /*------------------------------------------------------------*/
   /* isDraft
    */
   /**
    *    This method will return TRUE if this
    *  node is representing a Draft object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isDraft () {
      return( isDraft ) ;
   }




   /*------------------------------------------------------------*/
   /* getDraftObject
    */
   /**
    *    This method will return the draft object that this
    *  object node represents. If there is no draft object
    *  available a NULL will be returned. This method also
    *  handles showing the user an error message indicating
    *  why a draft could not be created
    *
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getDraftObject () {

      if ( ( !isObjectRead() ) && ( isDraft ) ) {
         switch ( node.getNodeType() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Question                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_QUESTION :
               if ( isDraft() )
                  oaObject = AvalonCache.readQuestionDraft ( node.getDraftObjectInd() ) ;
               else {
                  oaObject = OASQL.getQuestionDraft ( MainWindow.getInstance(),
                                                      node.getActiveObjectInd(),
                                                      TypeList.WORKREQ_EDIT ) ;
                  isDraft = true ;
               }

               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Action                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_ACTION :
               if ( isDraft() )
                  oaObject = AvalonCache.readActionDraft ( node.getDraftObjectInd() ) ;
               else {
                  oaObject = OASQL.getActionDraft ( MainWindow.getInstance(),
                                                    node.getActiveObjectInd(),
                                                    TypeList.WORKREQ_EDIT ) ;
                  isDraft = true ;
               }
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Symptom                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_SYMPTOM :
               if ( isDraft() )
                  oaObject = AvalonCache.readSymptomDraft ( node.getDraftObjectInd() ) ;
               else {
                  oaObject = OASQL.getSymptomDraft ( MainWindow.getInstance(),
                                                     node.getActiveObjectInd(),
                                                     TypeList.WORKREQ_EDIT ) ;
                  isDraft = true ;
               }
               break ;
         }
      }

      if ( oaObject != null )
         setDocClassInd ( (OAObject) oaObject ) ;

      setStatus ( null ) ;
      return( oaObject ) ;
   }





   /*------------------------------------------------------------*/
   /* getObject
    */
   /**
    *    This method will return either a Draft Object
    *  or Active Object depending on what this node
    *  represents.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getObject () {                                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only read the object if we have not read it yet         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !isObjectRead() ) {
         setStatus ( Str.getStr ( STR_READING )+" "+nodeDescription ) ;

         switch ( node.getNodeType() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Question                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_QUESTION :
               if ( isDraft() )
                  oaObject = AvalonCache.readQuestionDraft ( node.getDraftObjectInd() ) ;
               else
                  oaObject = AvalonCache.readQuestion ( node.getActiveObjectInd() ) ;
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Action                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_ACTION :
               if ( isDraft() )
                  oaObject = AvalonCache.readActionDraft ( node.getDraftObjectInd() ) ;
               else
                  oaObject = AvalonCache.readAction ( node.getActiveObjectInd() ) ;
               break ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Symptom                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
            case Node.NODE_TYPE_SYMPTOM :
               if ( isDraft() )
                  oaObject = AvalonCache.readSymptomDraft ( node.getDraftObjectInd() ) ;
               else
                  oaObject = AvalonCache.readSymptom ( node.getActiveObjectInd() ) ;
               break ;
         }


         setDocClassInd ( (OAObject) oaObject ) ;

         setStatus ( null ) ;
      }

      return( oaObject ) ;
   }





   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    The to string method will return the title of the
    *  object that this node represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( getTitle() ) ;
   }




   /*------------------------------------------------------------*/
   /* refreshProperties
    */
   /**
    *    This method will refresh all of the properties
    *  that we draw with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void refreshProperties () {
      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the background colors                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( node.getNodeType() == Node.NODE_TYPE_ACTION )
            setBackground ( (Color)PropertySystem.get ( PROPERTY_FLOWNODE_ACTION_COLOR ) ) ;
         else
            setBackground ( (Color)PropertySystem.get ( PROPERTY_FLOWNODE_COLOR ) ) ;


         setForeground ( (Color)PropertySystem.get ( PROPERTY_FLOWNODE_TEXT_COLOR ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the paint property. This will turn us off or on     */
         /* depending on what filters are being used.                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setPaint ( paint ) ;


      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* setNodeDescription
    */
   /**
    *    This method will set the node description
    *  that will be painted in the box to the 
    *  description passed in
    *
    *  @param descript The node description
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setNodeDescription ( String description ) {

      if ( description == null )
         description = "???" ;

      try {
         int propLength = PropertySystem.getInt ( PROPERTY_VISIO_TEXT_LENGTH ) ;
         int width      = node.getWidth() ;

         if ( description.length() > propLength ) {
            description = description.substring ( 0, propLength ) ;
            description+="..." ;
         }

         this.nodeDescription = description ;

         if ( width > 0 ) {
            if ( node.getNodeType() == Node.NODE_TYPE_SYMPTOM )
               width = width/2 ;
            nodeDescriptionArray = Text.formatStringLines ( description, width/charWidth ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    THis method will set the status on the
    *  main window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setStatus ( String status ) {
      MainWindow.getInstance().setStatus ( status ) ;
   }





   /*------------------------------------------------------------*/
   /* setOAObject
    */
   /**
    *    This method will set what OA Object that
    *  this Object Node represents
    *  
    *
    *  @param object Either a Symptom, Question or Action
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOAObject ( Object object ) {
      if ( ( object instanceof Question ) ||
           ( object instanceof Action ) ||
           ( object instanceof Symptom ) ) {

         this.oaObject = object ;

         if ( oaObject != null )
            setDocClassInd ( (OAObject) oaObject ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Question Draft                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof QuestionDraft ) {
            node.setDraftObjectInd ( ((QuestionDraft)object).getInd() ) ;
            docClassInd = ((QuestionDraft)object).getDocClass().getInd() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Action                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof ActionDraft ) {
            node.setDraftObjectInd ( ((ActionDraft)object).getInd() ) ;
            docClassInd = ((ActionDraft)object).getDocClass().getInd() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Symptom                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof SymptomDraft ) {
            node.setDraftObjectInd ( ((SymptomDraft)object).getInd() ) ;
            docClassInd = ((SymptomDraft)object).getDocClass().getInd() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Call setNode because it adjusts all                     */
         /* of our GUI strings and positions                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setNode ( node ) ;




      } /* if instance is correct  */
      else
         throw new IllegalArgumentException ( "Can only set Questions, Actions or Symptoms to an ObjectNode's OAObject" ) ;

   }





   /*------------------------------------------------------------*/
   /* getCountryList
    */
   /**
    *    This method will return the Vector of countries
    *  that this node's data object represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getCountryList () {
      return( countryList ) ;
   }




   /*------------------------------------------------------------*/
   /* getDocClass
    */
   /**
    *    This method will return our doc class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getDocClass () {
      return( docClassInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setVisible
    */
   /**
    *    This method has been subclassed so that
    *  we will not show ourselves if a filter does
    *  not allow
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setVisible ( boolean visible ) {
      this.visible = visible ;
   }





   /*------------------------------------------------------------*/
   /* setRoot
    */
   /**
    *    This method will set wether the node in that
    *  is represented by this control is a root node. Please
    *  note that this is for display purposes only, this does not save to the database as root
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setRoot ( boolean root ) {
      this.root = root ;
   }




   /*------------------------------------------------------------*/
   /* getRoot
    */
   /**
    *    This method will return TRUE if
    *  we are root. Please note that this 'root' property
    *  is only valid for this component, and this component only uses it to determine drawing style.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected boolean getRoot () {
      return( root ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  that should be represented for a specific
    *  column in a MultiList
    *
    *  @param column The column number to return data for
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObject = null ;

      switch ( column ) {
         case 0 :
            retObject = ( ""+node.getInd() ) ;
            break ;

         case 1 :
            retObject = nodeDescription ;
            break ;
      }

      return( retObject ) ;
   }




   /*------------------------------------------------------------*/
   /* isInClip
    */
   /**
    *    This method will determine if this node should
    *  be drawn because it is inside of the clip
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isInClip ( Rectangle rect ) {
      return( ( ( locationBeginPoint.x > rect.x ) &&
                ( locationBeginPoint.y > rect.y ) ) ||
              ( ( locationEndPoint.x < (rect.width+rect.x) ) &&
                ( locationEndPoint.y < (rect.height+rect.y ) ) ) ||
              ( ( locationBeginPoint.x > rect.x ) &&
                ( locationEndPoint.y > rect.y ) ) ||
              ( ( locationEndPoint.x < rect.x ) &&
                ( locationBeginPoint.y > rect.y ) ) ) ;

      // Commented out because the contains method doesn't work
      //rect.contains ( locationBeginPoint ) || ( rect.contains ( locationEndPoint ) ) ) ;
   }





   /*------------------------------------------------------------*/
   /* containsPoint
    */
   /**
    *    This method will return TRUE if the point
    *  object passed is contained inside of this node
    *  
    *
    *  @param point The point to check for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean containsPoint ( Point point ) {
      boolean contains = false ;
      int     cx       = node.getXPos()+node.getWidth() ;
      int     cy       = node.getYPos()+node.getHeight() ;

      contains = ( ( point.x > node.getXPos() ) &&
                   ( point.x < cx ) &&
                   ( point.y > node.getYPos() ) &&
                   ( point.y < cy ) ) ;

      return( contains ) ;
   }





   /*------------------------------------------------------------*/
   /* setPaint
    */
   /**
    *    This method will set paint to TRUE or FALSE. This
    *  means that painting will not occur or it will
    *  
    *
    *  @param TRUE = Paint will occur, FALSE = Paint will not occur
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPaint ( boolean paint ) {
      FlowEditorFilter filter = (FlowEditorFilter) PropertySystem.get ( PROPERTY_FLOW_FILTER ) ;                                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are supposed to paint, make sure that a filter    */
      /* is not set that would hide us.                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( paint ) {
         paint = ( ( filter.showCountry ( getCountryList() ) ) &&
                   ( filter.showDocClass ( getDocClass() ) ) ) ;

         if ( node != null )
            paint = node.valid() ;
      }

      this.paint = paint ;
   }




   /*------------------------------------------------------------*/
   /* fireActionEvent
    */
   /**
    *    This method will fire action events to all
    *  registered users
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void fireActionEvent ( ) {
      if ( listeners != null ) {
         int         i     = 0 ;
         int         size  = listeners.size() ;
         ActionEvent e     = new ActionEvent ( this, 0, null ) ;

         while ( i < size ) {
            ((ActionListener)listeners.elementAt(i)).actionPerformed ( e ) ;
            i++ ;
         }
      }
   }


   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener to this
    *  ObejctNode
    *  
    *
    *  @param listener The listener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      if ( listeners == null )
         listeners = new Vector ( 1 ) ;

      listeners.addElement ( listener ) ;
   }







   /*------------------------------------------------------------*/
   /* getDebugDescript
    */
   /**
    *    This method will simply return a debug
    *  string usefull for determining what exactly
    *  this ObjectNode represetsn
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getDebugDescript () {
      String typeStr = "Unknown" ;

      if ( node != null ) {
         switch ( node.getNodeType() ) {
            case Node.NODE_TYPE_ACTION : 
               typeStr = "Action" ;
               break ;

            case Node.NODE_TYPE_QUESTION :
               typeStr = "Question" ;
               break ;

            case Node.NODE_TYPE_SYMPTOM :
               typeStr = "Symptom" ;
               break ;

            case Node.NODE_TYPE_RESOLVED :
               typeStr = "Resolved";
               break ;

            case Node.NODE_TYPE_UNRESOLVED :
               typeStr = "Unresolved" ;
               break ;

            case Node.NODE_TYPE_RETURN :
               typeStr = "Return" ;
               break ;

            case Node.NODE_TYPE_ALIST :
               typeStr = "AList" ;
               break ;

         }
      }
      return( "Type:"+typeStr+",  Ind:"+(new String (nodeIndArray)) ) ;
   }





   /*------------------------------------------------------------*/
   /* isObjectRead
    */
   /**
    *    This method will determine if the Symptom,Action
    *  or Question object that this node represents has
    *  been read in.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Read in, FALSE = not read in yet
    *//*---------------------------------------------------------*/
   public boolean isObjectRead () {
      return( oaObject != null ) ;
   }





   /*------------------------------------------------------------*/
   /* getPaint
    */
   /**
    *    This method will return TRUE or FALSE. TRUE
    *  if this object is painting
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean getPaint() {
      return( paint ) ;
   }





   /*------------------------------------------------------------*/
   /* saveComplete
    */
   /**
    *    This method will be called if the object we
    *  are showing has been saved. We will simply 
    *  look at that object and apply the changes
    *
    *  @param OAEditorPanel editorPanel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void saveComplete ( OAEditorPanel editorPanel, Object object, boolean published ) {

      setOAObject ( oaObject ) ;

      if ( ( componentNode != null ) && ( componentNode.getObjectNode() == this ) )
         componentNode.setObjectNode ( this ) ;

   }






   /*------------------------------------------------------------*/
   /* setComponentNode
    */
   /**
    *    This method will set the component node that
    *  this Object Node is being referred from
    *  
    *
    *  @param componentNode The component Node
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setComponentNode ( ComponentNode componentNode ) {
      this.componentNode = componentNode ;
   }





   /*------------------------------------------------------------*/
   /* setBackground
    */
   /**
    *    This method will set our background color
    *  
    *  
    *
    *  @param Color the background color
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBackground ( Color color ) {
      backgroundColor = color ;
   }





   /*------------------------------------------------------------*/
   /* getBackground
    */
   /**
    *    This method will return our background
    *  color
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return color The background color
    *//*---------------------------------------------------------*/
   public Color getBackground () {
      return( backgroundColor ) ;
   }

   /*------------------------------------------------------------*/
   /* setForeground
    */
   /**
    *    This method will set our foreground color
    *  
    *  
    *
    *  @param Color the foreground color
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setForeground ( Color color ) {
      foregroundColor = color ;
   }





   /*------------------------------------------------------------*/
   /* getForeground
    */
   /**
    *    This method will return our foreground
    *  color
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return color The foreground color
    *//*---------------------------------------------------------*/
   public Color getForeground () {
      return( foregroundColor ) ;
   }





   /*------------------------------------------------------------*/
   /* getVisible
    */
   /**
    *    This method will return TRUE if this
    *  object is visible in the flow, FALSE if otherwise
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean getVisible () {
      return( visible ) ;
   }





   /*------------------------------------------------------------*/
   /* addCountry
    */
   /**
    *    This method will add to the list of countries
    *  the country passed in
    *  
    *
    *  @param countryRec The TypeCountryCodeRec of the country to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCountry ( TypeCountryCodeRec countryRec ) {
      if ( countryList == null )
         countryList = new Vector ( 1, 1 ) ;

      countryList.addElement ( countryRec ) ;
   }




   /*------------------------------------------------------------*/
   /* setDocClassInd
    */
   /**
    *    This method will set the doc classification
    *  indicator of the node
    *  
    *
    *  @param docClassInd The new Doc Classification as defined in TIGRIS.TYPEDOCCLASS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocClassInd ( int docClassInd ) {
      this.docClassInd = docClassInd ;
   }





   /*------------------------------------------------------------*/
   /* getOAObjectInd
    */
   /**
    *    This method will return the indicator for the
    *  OAObject that this node holds
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getOAObjectInd () {
      return( oaObjectInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getNodeDescriptionArray
    */
   /**
    *    This method will return the NodeDescriptionArray
    *  that is drawn onto the node
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return array
    *//*---------------------------------------------------------*/
   protected String[] getNodeDescriptionArray () {
      return( nodeDescriptionArray ) ;
   }




   /*------------------------------------------------------------*/
   /* setNodeDescriptionArray
    */
   /**
    *    This method will set the NodeDescriptionArray
    *  which is what we paint as text onto the node
    *  
    *
    *  @param array The new array
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setNodeDescriptionArray ( String[] newNodeDescriptionArray ) {
      this.nodeDescriptionArray = newNodeDescriptionArray ;
   }





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will determine equality depending
    *  on the object passed to compare to. The following
    *  objects are compared:<br><br>
    *
    *  Node - If a Node object is passed, this ObjectNode's Node will be compared<br>
    *  ObjectNode - If an ObjectNode is passed, the oaObjectInd is compared<br>
    *  String - If a String is passed, the 'toString()' method returned string will be compared<br>
    *
    *  @param object
    *  @param None
    *  @param None
    *
    *  @return TRUE if equal
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {

      if ( object instanceof Node ) {
         return( node.equals ( (Node)object ) ) ;
      }

      else if ( object instanceof ObjectNode )
         return( ((ObjectNode)object).oaObjectInd == oaObjectInd ) ;

      else if ( object instanceof String )
         return( toString().equals ( (String)object ) ) ;

      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* setEdgeTitleY
    */
   /**
    *    This method will set the edgeTitleY property
    *  
    *  
    *
    *  @param y The new edgeTitleY
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEdgeTitleY ( int y ) {
      this.edgeTitleY = y ;
   }




   /*------------------------------------------------------------*/
   /* getEdgeTitleY
    */
   /**
    *    This method will return the edgeTitleY
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getEdgeTitleY () {
      return( edgeTitleY ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocClassInd
    */
   /**
    *    This method will set our doc class indicator
    *  from whatever the OAObject passed in has
    *  
    *
    *  @param oaObject The OAObject to pull the docClass from
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setDocClassInd ( OAObject oaObject ) {
      if ( oaObject != null )
         docClassInd = oaObject.getDocClass().getInd() ;
   }
} /* ObjectNode */
