package com.ibm.nzna.projects.qit.avalon.editors ;              
/*-----------------------------------------------------------*/
/*   QuestionNodePanel                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestionNodePanel                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel shows on the 'NodeEdit' panel. It allows the user 
 *
 *
 * @version 1.0, 5/15/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestionNodePanel                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestionNodePanel extends JPanel implements MouseListener, EditPanelListener, ActionListener, NodePanel, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle       st_QUESTION     = null ;
   private JLabel       st_TITLE        = null ;
   private JScrollPane  scrollPane      = null ;
   private JList        lb_EDGES        = null ;
   private HotLinkLabel pb_ADD_EDGE     = null ;
   private HotLinkLabel pb_REMOVE_EDGE  = null ;
   private HotLinkLabel pb_EDIT_EDGE    = null ;
   private JLabel       st_EDGES        = null ;
   private JLabel       st_QUESTIONIND  = null ;
   private ButtonPanel  buttonPanel     = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int           questionInd      = 0 ;
   private boolean       showingDraft     = false ;
   private ObjectNode    objectNode       = null ;
   private FlowEditPanel flowPanel        = null ;
   private Symptom       symptom          = null ;
   private VisioPanel    visioPanel       = null ;
   private Vector        edges            = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will construct our panel with
    *  no Question
    *  
    *
    *  @param flowPanel The FlowPanel that this panel is sitting on
    *  @param visioPanel The VisioPan\el that this panel's edges should be drawn on
    *  @param symptom The Symptom that the FlowPanel is editing
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestionNodePanel ( FlowEditPanel flowPanel, VisioPanel visioPanel, Symptom symptom ) {
      this.flowPanel  = flowPanel ;
      this.symptom    = symptom ;
      this.visioPanel = visioPanel ;
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create our panel initialized
    *  to the Node passed
    *  
    *
    *  @param node The Node representing the Question to show information for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestionNodePanel ( FlowNode node ) {
      initialize() ;
      if ( node instanceof ObjectNode )
         setNode ( (ObjectNode)node ) ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_QUESTION    = new JTitle ( Str.getStr ( STR_QUESTION ) ) ;
      st_QUESTIONIND = new JLabel ( "" ) ;
      st_TITLE       = new JLabel ( "" ) ;
      st_EDGES       = new JLabel ( Str.getStr ( STR_EDGES ) ) ;
      lb_EDGES       = new JList  () ;
      scrollPane     = new JScrollPane ( lb_EDGES ) ;
      pb_ADD_EDGE    = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE_EDGE = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_EDIT_EDGE   = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      buttonPanel    = new ButtonPanel () ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque                 ( false ) ;
      st_QUESTION.setOpaque     ( false ) ;
      st_TITLE.setOpaque        ( false ) ;
      st_QUESTION.setFont       ( FontSystem.largeTitleFont ) ;
      st_EDGES.setFont          ( FontSystem.smallTitleFont ) ;
      scrollPane.setBorder      ( GUISystem.blackBorder ) ;
      buttonPanel.setBorder     ( GUISystem.grayBorder ) ;
      buttonPanel.setBackground ( new Color ( 240, 240, 240 ) ) ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action listeners                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD_EDGE.addActionListener    ( this ) ;
      pb_EDIT_EDGE.addActionListener   ( this ) ;
      pb_REMOVE_EDGE.addActionListener ( this ) ;
      lb_EDGES.addMouseListener        ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_QUESTION ) ;
      add ( st_TITLE ) ;
      add ( st_QUESTIONIND ) ;
      add ( st_EDGES ) ;
      add ( scrollPane ) ;
      add ( buttonPanel ) ;
      buttonPanel.add ( pb_ADD_EDGE ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_EDIT_EDGE ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_REMOVE_EDGE ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of 
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = 5 ; 
      int       y         = 0 ; 

      st_QUESTION.setBounds ( x, y, size.width-(x*2), rowHeight+8 ) ;
      y+=rowHeight+10 ;

      st_TITLE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      st_QUESTIONIND.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      st_EDGES.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      scrollPane.setBounds ( x,y , size.width-(x*2), rowHeight*5 );
      y+=rowHeight*5+5 ;

      buttonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
   }





   /*------------------------------------------------------------*/
   /* setNode
    */
   /**
    *    This method will set the node that we are
    *  currently editing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNode ( FlowNode node ) {                                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   We currently only support Object Nodes                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( node instanceof ObjectNode )
         this.objectNode = (ObjectNode)node ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      st_TITLE.setText ( getDefaultTitle () ) ;
      if ( objectNode.isDraft() )
         st_QUESTIONIND.setText ( ""+objectNode.getNode().getDraftObjectInd() ) ;
      else
         st_QUESTIONIND.setText ( ""+objectNode.getNode().getActiveObjectInd() ) ;

      Vector vec = symptom.getEdgeGroups() ;

      setEdges ( AvalonUtil.getEdgesFromNode ( symptom.getEdgeGroups(), objectNode.getNode() ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will get the default title of the
    *  object we are representing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getDefaultTitle () {
      if ( objectNode != null )
         return( objectNode.getTitle() ) ;
      return( "Unknown" ) ;
   } /* getDefaultTitle */





   /*------------------------------------------------------------*/
   /* setEdges
    */
   /**
    *    This method will set the edges that this
    *  panel is displaying
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setEdges ( Vector edges ) {
      this.edges = QuestUtil.getValidData ( edges ) ;

      lb_EDGES.setListData ( this.edges ) ;
   }

   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to buttons being
    *  pressed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_EDGE                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_EDIT_EDGE ) {
         editEdge() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_EDGE                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_EDGE ) {
         Edge      edgeRec       = (Edge)lb_EDGES.getSelectedValue() ;
         EdgeGroup edgeGroup     = null ;
         Vector    allEdgeGroups = symptom.getEdgeGroups() ; 
         int       i             = 0 ;
         boolean   found         = false;

         if ( allEdgeGroups != null ) {
            int size    = allEdgeGroups.size() ;

            while ( ( i < size ) && ( !found ) ) {
               if ( ((EdgeGroup)allEdgeGroups.elementAt(i)).getEdges().contains ( edgeRec ) ) {
                  edgeGroup = (EdgeGroup)allEdgeGroups.elementAt(i) ;                  

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Mark the edge as delete now                             */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  edgeRec.updateRecStatus ( edgeRec.REC_DELETE ) ;


                  /*-------------------------------------------------------------------*/
                  /*                                                                   */
                  /*   If the edge group has no more valid items, mark it as delete    */
                  /*                                                                   */
                  /*-------------------------------------------------------------------*/
                  if ( !QuestUtil.containsValid ( edgeGroup.getEdges() ) )
                     edgeGroup.updateRecStatus ( edgeGroup.REC_DELETE ) ;
                  found=true ;
               }
               i++ ;
            }
         }

         edges.removeElement ( edgeRec ) ;
         lb_EDGES.setListData ( edges ) ;
         symptom.setEdgeGroups ( allEdgeGroups ) ;
         flowPanel.getVisioPanel().refreshEdgeReturns() ;
         flowPanel.refreshData() ;
         flowPanel.repaint() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_EDGE                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_EDGE ) {
         Vector    edgeGroups = symptom.getEdgeGroups() ;
         EdgeGroup edgeGroup  = null ;
         EdgeDlg   edgeDlg    = new EdgeDlg ( GUISystem.getParentDefWin ( this ),
                                              symptom,
                                              objectNode,
                                              flowPanel.getObjectNodes(),
                                              !containsEdgeOther() ) ;                   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Wait for the user to respond                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( edgeDlg.getResult() ) {
            DraftEditPanel draftPanel = flowPanel.getDraftEditPanel() ;
            Edge           edgeRec    = edgeDlg.getEdge() ;
            Vector         edgeVec    = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the edge group that binds the head and tail node    */
            /* and create one if need be                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( draftPanel != null ) {
               edgeRec.setDocClass    ( draftPanel.getDocClass() ) ;
               edgeRec.setCountryList ( draftPanel.getCountrySelectPanel().getCountryList() ) ;
            } else
               LogSystem.log ( 1, "Could not get DraftEditPanel in Edge Creation. Defaulting DocClass and Countries" ) ;

            edgeGroup = AvalonUtil.getEdgeGroup ( edgeGroups, 
                                                  objectNode.getNode(), 
                                                  edgeDlg.getTailNode().getNode() ) ;

            /*----------------------------------------------------------------------*/
            /*                                                                      */
            /*   If we do not have that group, create a new EdgeGroup to hold it    */
            /*                                                                      */
            /*----------------------------------------------------------------------*/
            if ( edgeGroup == null ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If there is no edgeGroupVec, create a new one           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( edgeGroups == null )
                  edgeGroups = new Vector ( 1 ) ;

               edgeGroup = new EdgeGroup ( objectNode.getNode(), edgeDlg.getTailNode().getNode() ) ;
               edgeGroups.addElement ( edgeGroup ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add this new edge to the list of edges                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            edgeVec = edgeGroup.getEdges() ;
            if ( edgeVec == null )
               edgeVec = new Vector ( 1 ) ;
            edgeVec.addElement ( edgeRec ) ;
            edges.addElement   ( edgeRec ) ;
            edgeGroup.setEdges ( edgeVec ) ;
            edgeGroup.updateRecStatus ( edgeGroup.REC_UPDATE ) ;
            setEdges ( edges ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*    Now, set the symptoms in the edge group                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            symptom.setEdgeGroups ( edgeGroups ) ;
            flowPanel.getVisioPanel().addEdgeGroup ( edgeGroup ) ;
            flowPanel.refreshCounters() ;
            flowPanel.repaint() ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* editPanelChanged
    */
   /**
    *    This method will be called when one of our
    *  edit panels has been changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void editPanelChanged ( EditPanel editPanel ) {
      flowPanel.setDefaultDataPanel() ;
      setControlsEnabled ( true ) ;
      flowPanel.getVisioPanel().refreshEdgeReturns() ;
   }





   /*------------------------------------------------------------*/
   /* setControlsEnabled
    */
   /**
    *    This method will either enable or disable
    *  all of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setControlsEnabled ( boolean enabled ) {
      pb_ADD_EDGE.setEnabled ( enabled ) ;
      pb_REMOVE_EDGE.setEnabled ( enabled ) ;
      pb_EDIT_EDGE.setEnabled ( enabled ) ;
      scrollPane.setEnabled ( enabled ) ;
      lb_EDGES.setEnabled ( enabled ) ;
   }




   /*------------------------------------------------------------*/
   /* containsEdgeOther
    */
   /**
    *    This method will determine if any of our
    *  edges are the '#OTHER' edge
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean containsEdgeOther () {
      boolean contains = false ;

      if ( ( edges != null ) && ( edges.size() > 0 ) ) {
         int size = edges.size() ;
         int i    = 0 ;

         while ( ( !contains ) && ( i < size ) ) {
            contains = ((Edge)edges.elementAt(i)).getEdgeType() == Edge.EDGE_TYPE_OTHER ;
            i++ ;
         }
      }

      return( contains ) ;
   }

   public void spellCheckComplete ( EditPanel editPanel, boolean errorsFound ) {
   }





   /*------------------------------------------------------------*/
   /* editEdge
    */
   /**
    *    This method will edit the currently selected
    *  edge
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editEdge () {
      Edge edge = (Edge) lb_EDGES.getSelectedValue() ;

      if ( edge != null ) {
         EdgeGroup     edgeGroup      = AvalonUtil.getEdgeGroup ( symptom.getEdgeGroups(), edge ) ;
         boolean       showReturnEdge = edgeGroup.getTailNode().getNodeType() == Node.NODE_TYPE_SYMPTOM ;
         Vector        objectNodes    = (Vector)visioPanel.getObjectNodes().clone() ;
         EdgeEditPanel edgePanel      = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Take out the root node, the subroutine symptom node     */
         /* and the node that the edges we are displaying is for      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoreUtil.removeFromVector ( objectNodes, symptom.getRootNode() ) ;
         StoreUtil.removeFromVector ( objectNodes, edgeGroup.getTailNode() ) ;
         StoreUtil.removeFromVector ( objectNodes, objectNode.getNode() ) ;

         edgePanel = new EdgeEditPanel ( edge, objectNodes, showReturnEdge ) ;

         edgePanel.addEditPanelListener ( this ) ;
         flowPanel.setDataPanel ( edgePanel, true ) ;
         setControlsEnabled ( false ) ;
      }
   }


   public void mouseReleased ( MouseEvent e ) {}
   public void mousePressed ( MouseEvent e ) {}
   public void mouseEntered ( MouseEvent e ) {}
   public void mouseExited ( MouseEvent e ) {}
   public void mouseClicked ( MouseEvent e ) { if ( e.getClickCount() > 1 ) editEdge() ;}

} /* QuestionNodePanel */
