package com.ibm.nzna.projects.qit.avalon.editors ;                
/*-----------------------------------------------------------*/
/*   QuestionPreviewPanel                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestionPreviewPanel                */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.awt.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;       
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This preview panel is simply a Browser control. It handles
 * refreshing the information in the view as well as keeping 1
 * copy of the skins/Default/question_preview.html file to render
 *
 *
 * @version 1.0, 4/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestionPreviewPanel                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestionPreviewPanel extends JPanel implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String         htmlTemplate         = null ;
   private        QuestionDraft  question             = null ;
   private        Browser        previewPanel         = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will simply create a blank 
    *  preview window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestionPreviewPanel () {
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* QuestionPreviewPanel
    */
   /**
    *    This method will create a preview panel
    *  displaying the data from the Question
    *  object passed in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestionPreviewPanel ( QuestionDraft question ) {
      this.question = question ;
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         previewPanel = new Browser() ;                                   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Load in the htmlTemplate if it is not loaded            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( htmlTemplate == null ) {
            String filename = "skins"+File.separator+
                              PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                              File.separator+
                              "avalon"+
                              File.separator ;

            htmlTemplate = FileUtil.readFile ( filename+"question_preview.html" ) ;

            /* Copy over the background image  */
            QuestUtil.copyFileToTempDir ( filename+"empty_title.gif", "empty_title.gif" ) ;

         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our GUI                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setLayout ( new BorderLayout () ) ;
         add ( previewPanel, BorderLayout.CENTER ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Refresh our Data                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         refresh() ;


      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

   } /* initialize */






   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh the data on our
    *  preview
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      String showHtml = null ;                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have the template, fill it in                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( htmlTemplate != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Plug in the 'Name' values                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         showHtml = Text.replaceInStr ( htmlTemplate, "STR_ADDITIONAL_INFO", Str.getStr ( STR_ADDITIONAL_INFORMATION ) ) ;
         showHtml = Text.replaceInStr ( showHtml,     "STR_GEO",             Str.getStr ( STR_GEOGRAPHY ) ) ;                                            
         showHtml = Text.replaceInStr ( showHtml,     "STR_CHANGEDTIME",     Str.getStr ( STR_LAST_TOUCHED_ON ) ) ;
         showHtml = Text.replaceInStr ( showHtml,     "STR_CREATEDATE",      Str.getStr ( STR_CREATION_DATE ) ) ;
         showHtml = Text.replaceInStr ( showHtml,     "STR_DBUSER",          Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
         showHtml = Text.replaceInStr ( showHtml,     "STR_DATABASEINFO",    Str.getStr ( STR_DATABASE_INFO ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have a question, plug in that question's values   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( question != null ) {
            showHtml = Text.replaceInStr ( showHtml, "TITLE",           getDefaultTitle() ) ;
            showHtml = Text.replaceInStr ( showHtml, "GEO",             getGeographyList() ) ;
            showHtml = Text.replaceInStr ( showHtml, "ADDITIONAL_INFO", getDefaultAdditionalInfo() ) ;
            showHtml = Text.replaceInStr ( showHtml, "DBUSER",          UserSystem.getNameFromUserId ( question.getDbUser() ) ) ;
            showHtml = Text.replaceInStr ( showHtml, "CHANGEDTIME",     DateSystem.prettyDateFromStamp ( question.getChangedTime() ) ) ;
            showHtml = Text.replaceInStr ( showHtml, "CREATEDATE",      DateSystem.prettyDateFromStamp ( question.getCreationTime() ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, plug in blank values                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            showHtml = Text.replaceInStr ( showHtml, "TITLE",           Str.getStr ( STR_NONE ) ) ;
            showHtml = Text.replaceInStr ( showHtml, "GEO",             Str.getStr ( STR_NONE ) ) ;
            showHtml = Text.replaceInStr ( showHtml, "ADDITIONAL_INFO", Str.getStr ( STR_NONE ) ) ;
            showHtml = Text.replaceInStr ( showHtml, "DBUSER",          "" ) ;
            showHtml = Text.replaceInStr ( showHtml, "CHANGEDTIME",     "" ) ;
            showHtml = Text.replaceInStr ( showHtml, "CREATEDATE",      "" ) ;
         }


      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, put up a default message                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else
         showHtml = "<html><body><center>"+Str.getStr ( STR_CANNOT_FIND_TEMPLATE )+"</center></body></html>" ;            


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, show the content                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      previewPanel.setPageContent ( showHtml ) ;

   }





   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will pull the default title from
    *  the QuestionDraft that we are previewing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *  @see com.ibm.nzna.projects.common.quest.oa.QuestionDraft
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getDefaultTitle () {
      Vector titleVec = question.getTitles() ;
      String retStr   = Str.getStr ( STR_NONE ) ;

      if ( ( titleVec != null ) && ( titleVec.size() > 0 ) )
         retStr = ((Title)titleVec.elementAt(titleVec.size()-1)).toString() ;

      return( retStr ) ;
   }


   /*------------------------------------------------------------*/
   /* getDefaultAddInfo
    */
   /**
    *    This method will pull the default title from
    *  the QuestionDraft that we are previewing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *  @see com.ibm.nzna.projects.common.quest.oa.QuestionDraft
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String getDefaultAdditionalInfo () {
      Vector addInfoVec = question.getAdditionalInfo() ;
      String retStr     = Str.getStr ( STR_NONE ) ;

      if ( ( addInfoVec != null ) && ( addInfoVec.size() > 0 ) )
         retStr = ((AddtlInfo)addInfoVec.elementAt(addInfoVec.size()-1)).getAddtlInfo() ;

      return( retStr ) ;
   }





   /*------------------------------------------------------------*/
   /* getGeographyList
    */
   /**
    *    This method will format a user readable list
    *  of geographies
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getGeographyList () {
      Vector countryVec = question.getCountryList() ;
      String geoStr     = "&nbsp;" ;

      if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
         int     i         = 0 ;
         int     size      = countryVec.size() ;
         String  geography = null ;

         while ( i < size ) {
            geography = TypeList.getInstance().stringFromInd ( ((TypeCountryCodeRec)countryVec.elementAt(i)).geoInd,
                                                               TypeList.TYPE_GEOGRAPHY ) ;
            if ( geoStr.indexOf ( geography ) == -1 ) {
               geoStr+=geography ;
               if ( i < (size-1) )
                  geoStr+=", " ;
            }
            i++ ;
         }
      }

      return( geoStr ) ;
   }




   /*------------------------------------------------------------*/
   /* setQuestion
    */
   /**
    *    This method will set the question object
    *  we are displaying. This will not immediatly
    *  refresh the screen. To do this, you must call 'refresh'
    *
    *  @param question The QuestionDraft to set to
    *
    *  @see com.ibm.nzna.projects.common.quest.oa.QuestionDraft
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setQuestion ( QuestionDraft question ) {
      this.question = question ;
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return ( false ) ;
   }

} /* QuestionPreviewPanel */
