package com.ibm.nzna.projects.qit.avalon.editors ;                 
/*-----------------------------------------------------------*/
/*   QuestionUsagePanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestionUsagePanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.prodedit.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This panel holds the 'Usage' information for a Question. This 
 * information shows exactly what 'Uses' the Question. For example, all
 * symptoms who contain this Question will be listed.<br><br>
 *
 * Other conditions that will be listed are as follows:<br>
 * <b>That the answer is not in any:</b><br>
 * 1. Edge in Symptoms, including the Edge Itself, Conditions on the Edge, PreAnswers or UnAnswers on the edge.<br>
 * 2. Conditions for Titles in Actions and other Questions<br>
 * 3. Conditions for Additional Information in Actions and other Questions<br>
 * 4. Conditions on Answers in other Questions<br>
 * 5. Conditions on Links and Link Groups in Actions and other Questions<br>
 * 6. Conclusions on other Questions<br>
 * 7. Conditions on a Conclusion on other Questions<br>
 * 8. Conclusion on any Product<br><br>
 * If any of the above conditions is TRUE, then the QIT will not allow the delete. The user will be able to click on the 'Usage' tab to see why.<br><br><br>
 * The 'Usage' tab that will be added to the Symptom, Action, Link, Link Group and Question editors will appear a lot different in each editor. For example:<br><br><br>
 * <b>The Question Editor's Usage tab will contain a list for each of the following items:</b><br>
 * Symptoms - Showing any usage of the Question as a Node, as a Condition on an Edge,an Unanswer on the edge, or a Preanswer on an Edge.<br>
 * Questions - Showing any usage of the Question as a Condition on any of the titles, as a Condition on any Additional Information, answers answer conclusions, conditions on answer conclusions, and conditions on a Link or Link Group.<br>
 * Actions - Showing any usage of the Question as a Condition on any of the titles, as a Condition on any of the Additional Information, and conditions on a Link or Link Group.<br>
 * Products - Showing any usage as a Conclusion on a Product<br><br>
 *
 *
 *
 * @version 1.0, 9/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestionUsagePanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestionUsagePanel extends EditPanel implements ActionListener, ListSelectionListener, AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Column Headers                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String[] columns = { "", "", ""} ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle            st_SYMPTOMS      = null ;
   private JTitle            st_QUESTIONS     = null ;
   private JTitle            st_PRODUCTS      = null ;
   private JTitle            st_ACTIONS       = null ;
   private JTitle            st_DETAILS       = null ;
   private MultiList         cnr_SYMPTOMS     = null ;
   private MultiList         cnr_QUESTIONS    = null ;
   private MultiList         cnr_PRODUCTS     = null ;
   private MultiList         cnr_ACTIONS      = null ;
   private JTextArea         mle_DETAILS      = null ;
   private JScrollPane       scr_DETAILS      = null ;
   private ListActionPanel   refreshPanel     = null ;
   private HotLinkLabel      pb_REFRESH       = null ;
   private HotLinkLabel      pb_EDIT_SYMPTOM  = null ;
   private HotLinkLabel      pb_EDIT_QUESTION = null ;
   private HotLinkLabel      pb_EDIT_ACTION   = null ;
   private HotLinkLabel      pb_EDIT_PRODUCT  = null ;
   private ListActionPanel   symptomPanel     = null ;
   private ListActionPanel   questionPanel    = null ;
   private ListActionPanel   actionPanel      = null ;
   private ListActionPanel   productPanel     = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean             readData            = false ;
   private Dimension           minSize             = new Dimension ( 100, 363 ) ;
   private QuestionDraft       question            = null ;
   private UsageRec            currentUsageRec     = null ;
   private Vector              questionUsage       = null ;
   private Vector              actionUsage         = null ;
   private Vector              symptomUsage        = null ;
   private Vector              productUsage        = null ;
   private QuestionEditorPanel questionEditorPanel = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    The default constructor only initializes
    *  the GUI fields.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestionUsagePanel ( QuestionEditorPanel questionEditorPanel ) {
      this.questionEditorPanel = questionEditorPanel ;
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize all GUI components
    *  on this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SYMPTOMS   = new JTitle ( Str.getStr ( STR_SYMPTOMS ) ) ;
      st_QUESTIONS  = new JTitle ( Str.getStr ( STR_QUESTIONS ) ) ;
      st_PRODUCTS   = new JTitle ( Str.getStr ( STR_PRODUCTS ) ) ;
      st_ACTIONS    = new JTitle ( Str.getStr ( STR_ACTIONS ) ) ;
      st_DETAILS    = new JTitle ( Str.getStr ( STR_DETAILS ) ) ;
      mle_DETAILS   = new JTextArea ( Str.getStr ( STR_READING ) ) ;
      cnr_SYMPTOMS  = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_QUESTIONS = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_PRODUCTS  = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_ACTIONS   = new MultiList ( GUISystem.getFontUtil() ) ;
      scr_DETAILS   = new JScrollPane ( mle_DETAILS ) ;             
      pb_REFRESH    = new HotLinkLabel ( Str.getStr ( STR_REFRESH ) ) ;
      refreshPanel  = new ListActionPanel() ;
      pb_EDIT_SYMPTOM  = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT_QUESTION = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT_ACTION   = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT_PRODUCT  = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      symptomPanel     = new ListActionPanel () ;
      actionPanel      = new ListActionPanel () ;
      questionPanel    = new ListActionPanel () ;
      productPanel     = new ListActionPanel () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust our GUI Properties                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white );
      st_SYMPTOMS.setFont  ( FontSystem.smallTitleFont ) ;
      st_QUESTIONS.setFont ( FontSystem.smallTitleFont ) ;
      st_PRODUCTS.setFont  ( FontSystem.smallTitleFont ) ;
      st_ACTIONS.setFont   ( FontSystem.smallTitleFont ) ;
      st_DETAILS.setFont   ( FontSystem.smallTitleFont ) ;
      mle_DETAILS.setFont  ( FontSystem.defaultFont ) ;
      mle_DETAILS.setEditable ( false ) ;
      mle_DETAILS.setLineWrap ( true ) ;
      mle_DETAILS.setWrapStyleWord ( true ) ;  
      setListProperties ( cnr_SYMPTOMS ) ;
      setListProperties ( cnr_QUESTIONS ) ;
      setListProperties ( cnr_PRODUCTS ) ;
      setListProperties ( cnr_ACTIONS ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      refreshPanel.add ( pb_REFRESH ) ;
      symptomPanel.add ( pb_EDIT_SYMPTOM ) ;
      productPanel.add ( pb_EDIT_PRODUCT ) ;
      actionPanel.add  ( pb_EDIT_ACTION ) ;
      questionPanel.add ( pb_EDIT_QUESTION ) ;
      add ( st_SYMPTOMS ) ;
      add ( cnr_SYMPTOMS ) ;
      add ( symptomPanel ) ;
      add ( st_QUESTIONS ) ;
      add ( cnr_QUESTIONS ) ;
      add ( questionPanel ) ;
      add ( st_PRODUCTS ) ;
      add ( cnr_PRODUCTS ) ;
      add ( productPanel ) ;
      add ( st_ACTIONS ) ;
      add ( cnr_ACTIONS ) ;
      add ( actionPanel ) ;
      add ( st_DETAILS ) ;
      add ( scr_DETAILS ) ;
      add ( refreshPanel ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_REFRESH.addActionListener ( this ) ;
      pb_EDIT_SYMPTOM.addActionListener ( this ) ;
      pb_EDIT_PRODUCT.addActionListener ( this ) ;
      pb_EDIT_ACTION.addActionListener ( this ) ;
      pb_EDIT_QUESTION.addActionListener ( this ) ;
      cnr_SYMPTOMS.addActionListener ( this ) ;
      cnr_QUESTIONS.addActionListener ( this ) ;
      cnr_PRODUCTS.addActionListener ( this ) ;
      cnr_ACTIONS.addActionListener ( this ) ;


   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and size our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       halfWidth = size.width/2 ;
      int       x         = 5 ;
      int       y         = 5 ;

      super.doLayout() ;

      st_SYMPTOMS.setBounds ( x, y, halfWidth-x, rowHeight ) ;
      st_QUESTIONS.setBounds ( halfWidth+x, y, halfWidth-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cnr_SYMPTOMS.setBounds ( x, y, halfWidth-x, rowHeight*5 ) ;
      cnr_QUESTIONS.setBounds ( halfWidth+x, y, halfWidth-(x*2), rowHeight*5 ) ;
      y+=rowHeight*5 ;

      symptomPanel.setBounds ( x, y, halfWidth-x, rowHeight ) ;
      questionPanel.setBounds ( halfWidth+x, y, halfWidth-(x*2), rowHeight ) ;
      y+=(rowHeight*2) ;

      st_PRODUCTS.setBounds ( x, y, halfWidth-x, rowHeight ) ;
      st_ACTIONS.setBounds ( halfWidth+x, y, halfWidth-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cnr_PRODUCTS.setBounds ( x, y, halfWidth-x, rowHeight*5 ) ;
      cnr_ACTIONS.setBounds ( halfWidth+x, y, halfWidth-(x*2), rowHeight*5 ) ;
      y+=rowHeight*5 ;

      productPanel.setBounds ( x, y, halfWidth-x, rowHeight ) ;
      actionPanel.setBounds ( halfWidth+x, y, halfWidth-(x*2), rowHeight ) ;
      y+=rowHeight*2 ;

      st_DETAILS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      scr_DETAILS.setBounds ( x, y, size.width-(x*2), rowHeight*4 ) ;
      y+=rowHeight*4 ;

      refreshPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;

      y+=rowHeight ;

   }

   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method is subclassed so that when we
    *  show ourselves for the first time we refresh
    *  our data
    *
    *  @param g The Graphics Context.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      super.paintComponent ( g ) ;

      if ( !readData ) {
         readData = true ;
         readData() ;
      }
   }


   /*------------------------------------------------------------*/
   /* readData
    */
   /**
    *    This method will kick off the thread that will
    *  read in all of our data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readData () {
      readData = true ;
      new Thread ( this ).start () ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will read in all of our data
    *  and populate the controls accordingly
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      getUsage() ;
   }




   /*------------------------------------------------------------*/
   /* getUsage
    */
   /**
    *    This method will populate our information
    *  this question has any usages anywhere.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void getUsage() {
      StatusWin statusWin = new StatusWin ( GUISystem.getParentDefWin ( this ) ) ;

      try {
         if ( question != null ) {

            cnr_QUESTIONS.removeAll() ;
            cnr_PRODUCTS.removeAll() ;
            cnr_ACTIONS.removeAll() ;
            cnr_SYMPTOMS.removeAll() ;

            statusWin.setText ( Str.getStr ( STR_READING_USAGE ) ) ;
            statusWin.setMaxValue(5) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Populate the Question Usages                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            questionUsage = QuestionUsage.getQuestionAnswerUsageInQuestions ( question ) ;
            cnr_QUESTIONS.add ( questionUsage ) ;
            statusWin.setValue(1) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Populate the Symptom Usages                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            symptomUsage = QuestionUsage.getQuestionAnswerUsageInSymptoms ( question ) ;
            cnr_SYMPTOMS.add ( symptomUsage ) ;
            statusWin.setValue(2) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Populate the Action                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            actionUsage = QuestionUsage.getQuestionAnswerUsageInActions ( question );
            cnr_ACTIONS.add ( actionUsage ) ;
            statusWin.setValue(3) ;                  

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Populate the Product Usages                             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            productUsage = QuestionUsage.getQuestionProductUsage ( question ) ;
            cnr_PRODUCTS.add ( productUsage ) ;
            statusWin.setValue(4) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add to the Symptoms                                     */          
            /*                                                           */
            /*-----------------------------------------------------------*/
            cnr_SYMPTOMS.add ( QuestionUsage.getQuestionUsageInSymptoms ( question ) ) ;
            statusWin.setValue ( 5 ) ;
            


         }
      } catch ( Exception e ) {
      }
      statusWin.dispose() ;
   }





   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return our minimum size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( minSize ) ;
   }




   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our preferred size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( minSize ) ;
   }




   /*------------------------------------------------------------*/
   /* setQuestion
    */
   /**
    *    This method will set the question that
    *  we are currently displaying
    *  
    *
    *  @param question
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setQuestion ( QuestionDraft question ) {
      readData = false ;
      this.question = question ;
      cnr_SYMPTOMS.removeAll() ;
      cnr_QUESTIONS.removeAll() ;
      cnr_ACTIONS.removeAll() ;
      cnr_PRODUCTS.removeAll() ;
      mle_DETAILS.setText ( Str.getStr ( STR_READING ) ) ;
   }





   /*------------------------------------------------------------*/
   /* setListProperties
    */
   /**
    *    This method will set the properties on the
    *  MultiList passed. These properties include
    *  the column headings, column width and to turn off 
    *  the visible column headers
    *
    *  @param multiList The MultiList to set properties on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setListProperties ( MultiList multiList ) {
      multiList.setColumnHeadings ( columns ) ;
      multiList.setColumnWidth ( 0, 20 ) ;
      multiList.setColumnWidth ( 1, 55 ) ;
      multiList.setColumnWidth ( 2, 550 ) ;
      multiList.showColumnHeadings ( false ) ;
      multiList.addListSelectionListener ( this ) ;
   }




   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when an item in a list
    *  has been selected.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( e.getSource() == cnr_SYMPTOMS )
         currentUsageRec = (UsageRec)cnr_SYMPTOMS.getSelectedItem() ;
      else if ( e.getSource() == cnr_QUESTIONS )
         currentUsageRec = (UsageRec)cnr_QUESTIONS.getSelectedItem() ;
      else if ( e.getSource() == cnr_PRODUCTS )
         currentUsageRec = (UsageRec)cnr_PRODUCTS.getSelectedItem() ;
      else if ( e.getSource() == cnr_ACTIONS )
         currentUsageRec = (UsageRec)cnr_ACTIONS.getSelectedItem() ;

      refreshExplanation() ;
   }




   /*------------------------------------------------------------*/
   /* refreshExplanation
    */
   /**
    *    This method will refresh our explanation
    *  data at the bottom of the screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshExplanation () {
      if ( currentUsageRec != null )
         mle_DETAILS.setText ( currentUsageRec.getExplanation() ) ;
      else
         mle_DETAILS.setText ( "" ) ;
   }





   /*------------------------------------------------------------*/
   /* isAnswerUsed
    */
   /**
    *    This method will check to see if the answerInd
    *  passed is used anywhere in the usage vectors.
    *  If the usage vectors have not been read in, they
    *  are read in now
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected boolean isAnswerUsed ( int answerInd ) {
      boolean used = false ;

      if ( !readData ) {
         readData = true ;
         getUsage() ;
      }

      used = checkAnswerUsageInVec ( answerInd, questionUsage ) ;

      if ( !used )
         used = checkAnswerUsageInVec ( answerInd, productUsage ) ;

      if ( !used )
         used = checkAnswerUsageInVec ( answerInd, actionUsage ) ;

      if ( !used )
         used = checkAnswerUsageInVec ( answerInd, symptomUsage ) ;

      return( used ) ;
   }





   /*------------------------------------------------------------*/
   /* checkAnswerUsageInVec
    */
   /**
    *    This method will check for this answer in 
    *  all UsageRecs in the Vector passed
    *  
    *
    *  @param answerInd The Answer indicator to look for
    *  @param usageVec The Vector of UsageRecs to check
    *
    *  @return TRUE If it is found, FALSE if it is not found
    *//*---------------------------------------------------------*/
   private boolean checkAnswerUsageInVec ( int answerInd, Vector usageVec ) {
      int     i     = 0 ;
      int     size  = usageVec.size() ;
      boolean used  = false ;

      while ( ( i < size ) && ( !used ) ) {
         if ( ((UsageRec)usageVec.elementAt(i)).getAnswerInd() == answerInd ) {
            used = !((UsageRec)usageVec.elementAt(i)).getWarning() ;
         }
         i++ ;
      }

      return( used ) ;
   } /* checkAnswerUsageInVec */




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to action events
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REFRESH                                              */ 
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_REFRESH )
         new Thread ( this ).start() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_SYMPTOM                                         */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == cnr_SYMPTOMS ) || ( e.getSource() == pb_EDIT_SYMPTOM ) ) {
         UsageRec usageRec = (UsageRec)cnr_SYMPTOMS.getSelectedItem() ;

         if ( usageRec != null ) 
            OAEdit.edit ( usageRec, questionEditorPanel ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_QUESTION                                        */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == cnr_QUESTIONS ) || ( e.getSource() == pb_EDIT_QUESTION ) ) {
         UsageRec usageRec = (UsageRec)cnr_QUESTIONS.getSelectedItem() ;

         if ( usageRec != null ) 
            OAEdit.edit ( usageRec, questionEditorPanel ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_ACTION                                          */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == cnr_ACTIONS ) || ( e.getSource() == pb_EDIT_ACTION ) ) {
         UsageRec usageRec = (UsageRec)cnr_ACTIONS.getSelectedItem() ;

         if ( usageRec != null ) 
            OAEdit.edit ( usageRec, questionEditorPanel ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_PRODUCT                                         */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == cnr_PRODUCTS ) || ( e.getSource() == pb_EDIT_PRODUCT ) ) {
         UsageRec      usageRec  = (UsageRec)cnr_PRODUCTS.getSelectedItem() ;
         ProductRec    prodRec   = new ProductRec ( usageRec.getReferenceObjectInd() ) ;
         AppDefaultWin parentWin = GUISystem.getParentDefWin ( questionEditorPanel ) ;

         parentWin.setStatus ( STR_CHECKING_PRODUCT_LOCK ) ;
         if ( prodRec.readProductForEdit() ) {
            ProdEditPanel editProdPanel = new ProdEditPanel ( null, prodRec ) ;

            WindowSystem.createPanel ( editProdPanel ) ;
         }
         parentWin.setStatus ( null ) ;

      }
      
   } /* actionPerformed  */


} /* QuestionUsagePanel */


