package com.ibm.nzna.projects.qit.avalon.editors ;         
/*-----------------------------------------------------------*/
/*   SaveTypeDlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SaveTypeDlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog is used when saving Objects in Avalon. This dialog 
 * has 3 basic buttons normally. Draft, Publish and Cancel.  Here are the
 * return values based on what the user chooses:<br><pre>
 *
 * Button         Return Value<br>
 * -------------- ---------------------------------<br>
 * Draft          DRAFT<br>
 * Publish        PUBLISH<br>  
 * Archive        ARCHIVE<br>
 * UnArchive      UNARCHIVE<br>
 * Cancel Draft   CANCEL_DRAFT<br>
 * Cancel         CANCEL<br></pre>
 *
 * The constructor will not show the window. Please call 'getResult' to show
 * the window and get what the user chose.<br><br>
 *
 * Archive will allow the user to archive a currently non-archived document<br>
 * Unarchive will allow the user to un-archive a currently archived document<br><br>
 *
 * This window also allows the user to set the comments.<br><br>
 *
 *
 * @version 1.0, 4/20/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SaveTypeDlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class SaveTypeDlg extends JDialog implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int CANCEL       = 0 ;
   public static final int DRAFT        = 1 ;
   public static final int PUBLISH      = 2 ;                  
   public static final int ARCHIVE      = 3 ;
   public static final int UNARCHIVE    = 4 ;
   public static final int CANCEL_DRAFT = 5 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private, GUI Members                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton      pb_DRAFT              = null ;
   private DButton      pb_PUBLISH            = null ;
   private DButton      pb_CANCEL_DRAFT       = null ;
   private DButton      pb_ARCHIVE            = null ;
   private DButton      pb_CANCEL             = null ;
   private ButtonPanel  buttonPanel           = null ;
   private JLabel       st_IMAGE              = null ;
   private ImageIcon    wizardImage           = null ;
   private JPanel       actionButtonPanel     = null ;
   private JTextArea    mle_ORIGINAL_COMMENTS = null ;
   private JTextArea    mle_COMMENTS          = null ;  
   private JScrollPane  commentScrollPane     = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int     rc               = CANCEL ;
   private boolean isArchived       = false ;
   private String  originalComments = null ;
   private String  comments         = null ;
   private boolean canArchive       = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the window but
    *  will immediatly return. Please call 'getResult'
    *  to show the window and wait for user response
    *
    *  @param parent The parent of this dialog
    *  @param isArchived This param determines if we will show Archive, or Restore. If isArchived is TRUE, Restore is presented, otherwise Archive is shown
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SaveTypeDlg ( Frame parent, String originalComments, String comments, boolean isArchived, boolean canArchive ) {
      super ( parent, Str.getStr ( STR_SAVE ), true ) ;
      this.isArchived       = isArchived ;
      this.canArchive       = canArchive ;
      this.originalComments = originalComments ;
      this.comments         = comments ;
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the user and waits
    *  until the user responds. Once the user responds
    *  a return value will be returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getResult () {
      setVisible ( true ) ;
      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will completely initialize this window.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {    
      JPanel commentPanel = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardImage           = ImageSystem.getImageIcon ( this, ImageSystem.SAVE_DOC ) ;
      st_IMAGE              = new JLabel ( wizardImage ) ;
      pb_DRAFT              = new DButton ( Str.getStr ( STR_DRAFT ) ) ;
      pb_PUBLISH            = new DButton ( Str.getStr ( STR_PUBLISH ) ) ;
      pb_CANCEL             = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      pb_ARCHIVE            = new DButton ( Str.getStr ( STR_ARCHIVE ) ) ;
      pb_CANCEL_DRAFT       = new DButton ( Str.getStr ( STR_CANCEL_DRAFT ) ) ;
      buttonPanel           = new ButtonPanel () ;
      mle_ORIGINAL_COMMENTS = new JTextArea () ;
      mle_COMMENTS          = new JTextArea () ;
      commentPanel          = new JPanel () ;
      commentScrollPane     = new JScrollPane ( commentPanel ) ;
      actionButtonPanel     = new JPanel() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;
      mle_ORIGINAL_COMMENTS.setEditable ( false ) ;
      mle_ORIGINAL_COMMENTS.setFont ( FontSystem.defaultFont ) ;
      mle_COMMENTS.setFont ( FontSystem.defaultFont ) ;
      buttonPanel.setArrowImage  ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                   ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;
      if ( isArchived )
         pb_ARCHIVE.setText ( Str.getStr ( STR_UNARCHIVE ) ) ;
      if ( originalComments != null ) {
         mle_ORIGINAL_COMMENTS.setText ( originalComments ) ;
         mle_COMMENTS.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 1023-originalComments.length() ) ) ;
      } else
         mle_COMMENTS.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 1023 ) ) ;
      mle_COMMENTS.setText ( comments ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_DRAFT.addActionListener        ( this ) ;
      pb_PUBLISH.addActionListener      ( this ) ;
      pb_CANCEL.addActionListener       ( this ) ;  
      pb_ARCHIVE.addActionListener      ( this ) ;
      pb_CANCEL_DRAFT.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout  ( null ) ;
      actionButtonPanel.setLayout ( new GridLayout ( 3,1 ) ) ;
      commentPanel.setLayout      ( new BorderLayout () ) ;
      commentPanel.add ( mle_ORIGINAL_COMMENTS, BorderLayout.NORTH ) ;
      commentPanel.add ( mle_COMMENTS, BorderLayout.CENTER ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      actionButtonPanel.add ( pb_DRAFT ) ;
      if ( ( !isArchived ) && ( !canArchive ) )
         actionButtonPanel.add ( pb_PUBLISH ) ;
      if ( ( canArchive ) || ( isArchived ) )
         actionButtonPanel.add ( pb_ARCHIVE ) ;
      actionButtonPanel.add ( pb_CANCEL_DRAFT ) ;
      getContentPane().add ( actionButtonPanel ) ;
      getContentPane().add ( st_IMAGE ) ;
      getContentPane().add ( commentScrollPane ) ;
      getContentPane().add ( buttonPanel ) ;

      setResizable ( false ) ;
      setSize ( 495, 325 ) ;
      WinUtil.centerChildInParent ( this, getParent() ) ;
   } /* intialize  */





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = wizardImage.getImage().getWidth(this)+10 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout () ;

      st_IMAGE.setBounds ( 5, 
                           ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2), 
                           wizardImage.getImage().getWidth(this), 
                           wizardImage.getImage().getHeight(this)) ;



      commentScrollPane.setBounds ( x, y, size.width-(x)-10, rowHeight*8 ) ;
      y+=(rowHeight*8) ;

      actionButtonPanel.setBounds ( x, y, size.width-(x)-10, rowHeight*3 ) ;


      buttonPanel.setBounds ( x, size.height-75, size.width-(x*2), 25 ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_DRAFT )
         rc = DRAFT ;
      else if ( e.getSource() == pb_PUBLISH )
         rc = PUBLISH ;
      else if ( e.getSource() == pb_CANCEL_DRAFT )
         rc = CANCEL_DRAFT ;
      else if ( e.getSource() == pb_ARCHIVE ) {
         if ( isArchived )
            rc = UNARCHIVE ;
         else
            rc = ARCHIVE ;
      }

      dispose() ;
   }





   /*------------------------------------------------------------*/
   /* getComments
    */
   /**
    *    This method will return the comments that have been
    *  typed in. This includes the original comments and
    *  the newly typed comments
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getComments () {
      return( mle_ORIGINAL_COMMENTS.getText()+"\n"+mle_COMMENTS.getText() ) ;
   }
} /* SaveTypeDlg */
