package com.ibm.nzna.projects.qit.avalon.editors ;         
/*-----------------------------------------------------------*/
/*   SoftValidateDlg                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SoftValidateDlg                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog is a 'soft' validate window, which means that any validation 
 * displayed here should not be an error that would prevent an object from
 * being saved to the database
 *
 * @version 1.0, 5/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SoftValidateDlg                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class SoftValidateDlg extends JDialog implements ActionListener, AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Browser     browser     = null ;
   private DButton     pb_OK       = null ;
   private DButton     pb_PRINT    = null ;      
   private ButtonPanel buttonPanel = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String  validateText = null ;
   private boolean hadErrors    = false ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the dialog.
    *  It will not return until the user interacts with the
    *  dialog and closes it. However, if there are no errors, a 
    *  message is displayed telling them that.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SoftValidateDlg ( Frame parent, String validateText ) {
      super ( parent, Str.getStr ( STR_VALIDATE ), true ) ;          

      this.validateText = validateText ;                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The Soft Validator's put the STR_ERROR string           */
      /* into the text. If this text is present, then it is an     */
      /* error, otherwise it is not an error, so put up a message  */
      /* instead of showing our window.                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( validateText.indexOf ( Str.getStr ( STR_ERROR ) ) >=0 ) ||
           ( validateText.indexOf ( Str.getStr ( STR_WARNING ) ) >=0 ) ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   We had errors                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         hadErrors = true ;



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our controls                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         browser     = new Browser ( ) ;
         pb_OK       = new DButton ( Str.getStr ( STR_OK ) ) ;
         pb_PRINT    = new DButton ( Str.getStr ( STR_PRINT ) ) ;
         buttonPanel = new ButtonPanel ( ) ;        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         GUISystem.setPreferredButton ( pb_OK ) ;
         GUISystem.setPreferredButton ( pb_PRINT ) ;
         browser.showZoom ( false ) ;
         buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                     ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;
         browser.setBorder ( GUISystem.loweredBorder ) ;           


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_OK.addActionListener    ( this ) ;
         pb_PRINT.addActionListener ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the controls                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         getContentPane().setLayout ( null ) ;
         getContentPane().add ( browser ) ;
         getContentPane().add ( buttonPanel ) ;
         buttonPanel.add ( pb_OK ) ;
         buttonPanel.add ( pb_PRINT ) ;

         setSize ( 500, 450 ) ;
         WinUtil.centerChildInParent ( this, parent ) ;
         new Thread ( this ).start() ;
         setVisible ( true ) ;
      } 

   }


   public void run () {
      try {
         Thread.currentThread().sleep ( 200 ) ;
         browser.setPageContent ( "<html><body><font size=2>"+validateText+"</font></body></html>" ) ;
      } catch ( Exception e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize();

      super.doLayout() ;
      browser.setBounds ( 15, 5, size.width-30, size.height-65 ) ;
      buttonPanel.setBounds ( 15, size.height-55, size.width-30, 25 ) ;

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == pb_OK )
         dispose() ;

   }





   /*------------------------------------------------------------*/
   /* getHadErrors
    */
   /**
    *    This method will return TRUE if the Validation
    *  had errors
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getHadErrors () {
      return( hadErrors ) ;
   }
} /* SoftValidateDlg */
