package com.ibm.nzna.projects.qit.avalon.editors ;           
/*-----------------------------------------------------------*/
/*   SymptomEditorPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomEditorPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.qit.avalon.editors.symptomProp.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.spell.* ;
import javax.swing.* ;
import java.awt.* ;
import java.awt.image.* ;
import java.util.* ;
import java.awt.event.* ;
import java.sql.* ;
/**
 * This panel will allow a user to edit any part of a Symptom. 
 *
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomEditorPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomEditorPanel extends OAEditorPanel implements CountrySelectListener, PrintOwner, PrintPanel, PublishPanelListener, Runnable, EditPanelListener, QuestPanel, AvalonConst, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESH      = "REFRESH" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Panel Names                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String PANEL_USAGE       = "USAGE" ;
   public static final String PANEL_DRAFT       = "DRAFT" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Members                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTabbedPane           tabbedPane           = null ;
   private JScrollPane           scrollPane           = null ;
   private EditNavPanel          defaultNavPanel      = null ;
   private SymptomValidatorPanel validatorPanel       = null ;
   private FlowEditPanel         flowEditPanel        = null ;
   private ToolBarButton         pb_GRID              = null ;
   private ToolBarButton         pb_PROPERTIES        = null ;
   private ToolBarButton         pb_TITLES            = null ;
   private ToolBarButton         pb_FULLSCREEN        = null ;
   private ToolBarButton         pb_SPELL             = null ;
   private JToolBar              toolBar              = null ;
   private SymptomTitleEditPanel titleEditPanel       = null ;
   private DraftEditPanel        draftPanel           = null ;
   private JLabel                st_TITLE             = null ;
   private ActionButton          pb_SAVE              = null ;
   private ActionButton          pb_CLOSE             = null ;
   private ActionButton          pb_VALIDATE          = null ;
   private ActionButton          pb_SAVEAS            = null ;
   private DButton               pb_CLOSE_FULL_SCREEN = null ;
   private JWindow               fullScreenWindow     = null ;
   private SpellPanel            spellPanel           = null ;
   private JPanel                contentPanel         = null ;
   private SymptomUsagePanel     symptomUsagePanel    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private QuestPanel               previousPanel            = null ;
   private SymptomDraft             symptom                  = null ;
   private AppDefaultWin            parentWin                = null ;
   private AvalonMemoryInterface    memoryInterface          = null ;
   private int                      symptomInd               = 0 ;
   private boolean                  readFromDatabase         = false ;
   private boolean                  forceClose               = false ;
   private boolean                  saved                    = false ;
   private boolean                  publishing               = false ;
   private PublishPanel             publishPanel             = null ;
   private boolean                  published                = false ;      // TRUE = We have published the object
   private boolean                  initialRefresh           = true ;
   private boolean                  readingSymptom           = false ;      // TRUE = We are reading the whole symptom
   private Image                    buffImage                = null ;
   private Image                    darkImage                = null ;
   private int                      oldWidth                 = -1 ;
   private int                      oldHeight                = -1 ;
   private boolean                  hasAuth                  = true ;       // TRUE = User has authority to save, FALSE = do not save 
   private boolean                  askRemoveCountryQuestion = true ;
   private boolean                  autoPublish              = false ;      // TRUE = Object will publish once it is read in
   private String                   initialPanel             = PANEL_DRAFT ; // The initial panel to show when the editor opens









   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor will create a blank
    *  Symptom editor with no Symptom
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomEditorPanel ( QuestPanel previousPanel ) {
      this.previousPanel = previousPanel ;

      symptom = (SymptomDraft)new NewAvalonObjectWizard ( GUISystem.getParentDefWin ( previousPanel.getJComponent() ),
                                                          AVALON_SYMPTOM ).getNewObject() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will read in the symptom that represents
    * the symptomInd passed in
    *  
    *
    *  @param previousPanel The panel to show once the user closes this panel
    *  @param symptomInd The Active Symptom Indicator indicator to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomEditorPanel ( QuestPanel previousPanel, int symptomInd ) {
      this.previousPanel    = previousPanel ;             
      this.symptomInd      = symptomInd ;
      this.readFromDatabase = true ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will edit the symptom
    *  passed in and then set the panel in the tabbed
    *  pane to the panel passed in
    *
    *  @param questPanel The previous panel
    *  @param symptom The SymptomDraft to edit
    *  @param panelName One of the panel names ( PANEL_DRAFT, PANEL_USAGE ) as defined in this class
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomEditorPanel ( QuestPanel questPanel, SymptomDraft symptom, String panelName ) {
      this.previousPanel = previousPanel ;
      this.symptom        = symptom ;
      this.initialPanel  = panelName ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will edit the symptom passed in
    *  
    *
    *  @param previousPanel The panel to show once the user closes this panel
    *  @param symptom The SymptomDraft object to edit
    *  @param autoPublish TRUE = AutoPublish
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomEditorPanel ( QuestPanel previousPanel, SymptomDraft symptomDraft, boolean autoPublish ) {
      this.autoPublish      = autoPublish ;
      this.previousPanel    = previousPanel ;             
      if ( symptomDraft != null ) {
         this.symptom = symptomDraft ;
         symptom.updateRecStatus ( symptom.REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Cannot edit a NULL SymptomDraft" ) ;
   }



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will edit the symptom passed in
    *  
    *
    *  @param previousPanel The panel to show once the user closes this panel
    *  @param symptom The SymptomDraft object to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomEditorPanel ( QuestPanel previousPanel, SymptomDraft symptomDraft ) {
      this.previousPanel    = previousPanel ;             
      if ( symptomDraft != null ) {
         this.symptom = symptomDraft ;
         symptom.updateRecStatus ( symptom.REC_UPDATE ) ;
      } else
         throw new IllegalArgumentException ( "Cannot edit a NULL SymptomDraft" ) ;
   }





   /*------------------------------------------------------------*/
   /* SymptomEditorPanel
    */
   /**
    *    This constructor is an auto-publish constructor,
    *  meaning that the symptomDraftInd passed in will
    *  be read in and the publish button will be hit automatically.
    *
    *  @param previousPanel The Previous Panel to go to after publish
    *  @param symptomDraftInd The Draft indicator of the symptom
    *  @param autoPublish TRUE = AutoPublish
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomEditorPanel ( QuestPanel previousPanel, int symptomDraftInd, boolean autoPublish ) {
      this.previousPanel = previousPanel ;

      if ( symptomDraftInd != 0 ) {
         this.symptom = AvalonCache.readSymptomDraft ( symptomDraftInd ) ;

         if ( symptom != null )
            this.autoPublish = autoPublish ;
      }
   }






   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI and variable
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {     
      Dimension   toolPrefSize = new Dimension ( 3, 15 ) ;

      if ( ( symptom != null ) || ( readFromDatabase ) ) {
         Color lightGray = new Color ( 240, 240, 240 ) ;      
         JScrollPane titleScroll      = null ;
         JScrollPane draftPanelScroll = null ;
         JScrollPane usageScroll      = null ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize vars                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         memoryInterface = new AvalonMemoryInterface () ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tabbedPane        = new JTabbedPane() ;
         flowEditPanel     = new FlowEditPanel ( this ) ;
         pb_GRID           = new ToolBarButton ( Str.getStr ( STR_GRID_ON ),     ImageSystem.getImageIcon ( this, ImageSystem.GRID_ON ),     Str.getStr ( STR_TOGGLE_GRID ) ) ;
         pb_PROPERTIES     = new ToolBarButton ( Str.getStr ( STR_PROPERTIES ),  ImageSystem.getImageIcon ( this, ImageSystem.PROPERTIES ),  Str.getStr ( STR_PROPERTIES ) ) ;
         pb_TITLES         = new ToolBarButton ( Str.getStr ( STR_TITLES_ON ),   ImageSystem.getImageIcon ( this, ImageSystem.TITLES_ON ),   Str.getStr ( STR_TITLES ) ) ;
         pb_FULLSCREEN     = new ToolBarButton ( Str.getStr ( STR_FULL_SCREEN ), ImageSystem.getImageIcon ( this, ImageSystem.FULL_SCREEN ), Str.getStr ( STR_SYMPTOM_FULL_SCREEN ) ) ;
         pb_SPELL          = new ToolBarButton ( Str.getStr ( STR_SPELL ),       ImageSystem.getImageIcon ( this, ImageSystem.SPELL ),       Str.getStr ( STR_SPELL ) ) ;
         toolBar           = new JToolBar() ;        
         pb_CLOSE          = new ActionButton ( Str.getStr ( STR_CLOSE ),    ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
         pb_SAVE           = new ActionButton ( Str.getStr ( STR_SAVE ),     ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SAVE ) ) ;
         pb_SAVEAS         = new ActionButton ( Str.getStr ( STR_SAVE_AS ),  ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SAVE_AS ) ) ;
         pb_VALIDATE       = new ActionButton ( Str.getStr ( STR_VALIDATE ), ImageSystem.getImageIcon ( this, ImageSystem.NEW ),   Str.getStr ( STR_VALIDATE ) ) ;
         titleEditPanel    = new SymptomTitleEditPanel ( 0  ) ;
         draftPanel        = new DraftEditPanel () ;
         st_TITLE          = new JLabel() ;
         validatorPanel    = new SymptomValidatorPanel ( memoryInterface, this  ) ;
         contentPanel      = new JPanel () ;
         spellPanel        = new SpellPanel () ;
         symptomUsagePanel = new SymptomUsagePanel ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setBackground ( Color.white ) ;
         toolBar.setBorder ( null ) ;    
         toolBar.setOpaque ( false ) ;           
         st_TITLE.setFont ( FontSystem.largeTitleFont ) ;
         st_TITLE.setOpaque ( false ) ;
         tabbedPane.setOpaque ( false ) ;
         titleEditPanel.getCountrySelectPanel().joinCountrySelectPanel ( draftPanel.getCountrySelectPanel() ) ;
         flowEditPanel.setDraftEditPanel ( draftPanel ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add Listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_GRID.addActionListener           ( this ) ;      
         pb_TITLES.addActionListener         ( this ) ;
         pb_CLOSE.addActionListener          ( this ) ;
         pb_SAVE.addActionListener           ( this ) ;
         pb_PROPERTIES.addActionListener     ( this ) ;
         titleEditPanel.addEditPanelListener ( this ) ;
         pb_VALIDATE.addActionListener       ( this ) ;
         pb_FULLSCREEN.addActionListener     ( this ) ;
         pb_SPELL.addActionListener          ( this ) ;
         pb_SAVEAS.addActionListener         ( this ) ;
         draftPanel.getCountrySelectPanel().addCountrySelectListener ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our toolbar                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         toolBar.add ( new ToolBarSeparator() ) ;
         toolBar.add ( pb_SPELL ) ;
         toolBar.add ( new ToolBarSeparator() ) ;
         toolBar.add ( pb_GRID ) ;
         toolBar.add ( pb_TITLES ) ;
         toolBar.add ( pb_FULLSCREEN ) ;
         toolBar.add ( new ToolBarSeparator() ) ;  
         toolBar.add ( pb_PROPERTIES ) ;
         toolBar.add ( new ToolBarSeparator() ) ;  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our GUI                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tabbedPane.addTab ( Str.getStr ( STR_DRAFT ),                  draftPanelScroll = new JScrollPane ( draftPanel ) ) ;
         tabbedPane.addTab ( Str.getStr ( STR_TITLES ),                 titleScroll      = new JScrollPane ( titleEditPanel ) ) ;
         tabbedPane.addTab ( Str.getStr ( STR_FLOW ),                   flowEditPanel ) ;
         tabbedPane.addTab ( Str.getStr ( STR_VALIDATOR ),              validatorPanel ) ;
         tabbedPane.addTab ( Str.getStr ( STR_SYMPTOM_USAGE ),          usageScroll      = new JScrollPane ( symptomUsagePanel ) ) ;
         setLayout ( new BorderLayout() ) ;
         contentPanel.setLayout ( new BorderLayout() ) ;
         add ( contentPanel, BorderLayout.CENTER ) ;
         contentPanel.add ( st_TITLE, BorderLayout.NORTH ) ;
         contentPanel.add ( tabbedPane, BorderLayout.CENTER ) ;                                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust Scroll Properties                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         draftPanelScroll.setBorder ( null ) ;
         titleScroll.setBorder ( null ) ;
         usageScroll.setBorder ( null ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust notebook properties                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tabbedPane.setBackgroundAt ( 0, lightGray ) ;
         tabbedPane.setBackgroundAt ( 1, lightGray ) ;
         tabbedPane.setBackgroundAt ( 2, lightGray ) ;
         tabbedPane.setBackgroundAt ( 3, lightGray ) ;
         tabbedPane.setBackgroundAt ( 4, lightGray ) ;
      }

      return( ( symptom != null ) || ( readFromDatabase ) ) ;
   }


   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are showing.
    *  We will refresh our data here
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      boolean refresh = false ;

      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_SAVE ) ;                
      parentWin.addActionComponent ( this, pb_SAVEAS ) ;                
      parentWin.addActionComponent ( this, pb_VALIDATE ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our toolbar                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addToolBarButtons() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we need to read our symptom from the database,       */
      /* do it now                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( readFromDatabase ) {
         try {
            parentWin.setStatus ( STR_PLEASE_WAIT ) ;
            symptom = AvalonCache.readSymptomDraft ( symptomInd ) ;
            if ( symptom == null ) {
               forceClose() ;
               GUISystem.printBox ( this, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
            } else
               refresh = true ;
         } catch ( Exception e ) {
            forceClose() ;
         }
      } /* if we were to read the symptom */
      else
         refresh = true ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only continue from this point if we have a symptom      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( symptom != null ) {

         if ( refresh )
            new Thread ( this, THREAD_REFRESH ).start() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the maint lock flag to true if this is an edit      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( (!autoPublish) && ( !locked() ) && ( symptom.getRecStatus() != symptom.REC_INSERT ) ) {
            setMaintlock ( "Y" ) ;
         }

         if ( ( autoPublish ) && ( !locked() ) )
            saveAsPublish() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, Now show the initial panel                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !autoPublish )
            showPanel ( initialPanel ) ;

      }

      parentWin.setStatus ( null ) ;
   }



   /*------------------------------------------------------------*/
   /* showPanel
    */
   /**
    *    This method will show the tabbed pane panel
    *  that is passed
    *  
    *
    *  @param panelName The name of the panel as defined in this class ( PANEL_DRAFT etc.. )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showPanel ( String panel ) {

      if ( panel.equals ( PANEL_DRAFT ) )
         tabbedPane.setSelectedComponent ( WinUtil.getParentJScrollPane ( draftPanel ) ) ;

      else if ( panel.equals ( PANEL_USAGE ) ) {
         tabbedPane.setSelectedComponent ( WinUtil.getParentJScrollPane ( symptomUsagePanel ) ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will be called when we need
    *  to copy text. It will do nothing.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will be called when we need
    *  to paste text. It will do nothing.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will be called when we need
    *  to cut text. It will do nothing.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will simply return an instance
    *  of this class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_SYMPTOM_EDITOR ) ) ;
   }





   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the application
    *  wants to shutdown. We will only return TRUE
    *  if the user has not changed data
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.EDIT ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default navigation
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NavPanel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( defaultNavPanel ) ;
   }


   public void refresh () {
   }


   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh the data on our
    *  screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData () {    

      if ( initialRefresh ) {
         AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;


         parentWin.setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
         tabbedPane.setEnabled ( false ) ;


         flowEditPanel.setSymptom ( symptom ) ;
         parentWin.setStatusLock ( true ) ;
         validatorPanel.setSymptom ( symptom ) ;
         flowEditPanel.refreshData() ;

         draftPanel.setObjectType           ( TypeList.TYPE_SYMPTOM ) ;
         draftPanel.setSelectedObjectType   ( (Object)symptom.getSymptomType() ) ;
         draftPanel.setShowStatus           ( true ) ;
         draftPanel.setStatusData           ( Status.getStatusList ( Status.STATUS_OA ) ) ;
         draftPanel.setSelectedStatus       ( symptom.getSymptomStatus() ) ;
         draftPanel.setDraftObject          ( symptom ) ;
         draftPanel.setDraftIndicator       ( ""+symptom.getInd() ) ;
         draftPanel.setActiveIndicator      ( ""+symptom.getActiveSymptomInd() ) ;
         symptomUsagePanel.setSymptom       ( symptom ) ;
         symptomUsagePanel.setFlowEditPanel ( flowEditPanel ) ;

         titleEditPanel.setTitles         ( symptom.getSymptomTitles() ) ;
         titleEditPanel.setBrandInd       ( symptom.getBrand() != null ? symptom.getBrand().getInd() : 0 ) ;
         st_TITLE.setText                 ( getDefaultTitle() ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the user does not have the authority to edit         */
         /* this object, tell them and take away the save button      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            if ( !UserSystem.hasAuthority ( QuestUtil.brandAuthFromBrandInd ( symptom.getBrand().getInd() ) ) ) {
               GUISystem.printBox ( STR_INFO,
                                    STR_NO_AUTHORITY_EDIT_BRAND ) ;
               hasAuth = false ;
               pb_SAVE.setVisible ( false ) ;
            }
         } catch ( Exception e ) {
            // Symptom does not have a brand, so allow the edit
            hasAuth = true ;
            pb_SAVE.setVisible ( true ) ;
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the user is not a BPM or Editor, disable the brand   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            if ( ( !UserSystem.getLoggedUserRec().isBPM() ) &&
                 ( !UserSystem.hasAuthority ( AUTH_AUTHOR ) ) )
               draftPanel.setBrandEnabled ( false ) ;

         } catch ( Exception e ) {
            hasAuth = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is locked, tell the user                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( locked() ) {
            GUISystem.printBox ( parentWin,
                                 Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_DRAFT_LOCKED_BY )+" "+UserSystem.getNameFromUserId ( symptom.getDbUser() ) ) ;
            pb_SAVE.setVisible ( false ) ;
            draftPanel.setEnabled       ( false ) ;
            titleEditPanel.setEnabled   ( false ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is recycled, let them know                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( symptom.isRecycled() ) {
            GUISystem.printBox ( parentWin,
                                 Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_DRAFT_IS_RECYCLED_BY )+" "+UserSystem.getNameFromUserId ( symptom.getDbUser() ) ) ;
            pb_SAVE.setVisible ( false ) ;
         }

         initialRefresh = false ;

         readingSymptom = false ;
         parentWin.setStatusLock ( false ) ;
         tabbedPane.setEnabled ( true ) ;
         parentWin.setStatus ( null ) ;
      }

   }

   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the
    *  user has selected our panel from a
    *  tabbed pane. If we are in MDI mode, this method is never called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
      addToolBarButtons() ;
   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel onto another. This method
    *  will never be called in the MDI mode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
   }





   /*------------------------------------------------------------*/
   /* addToolBarButtons
    */
   /**
    *    This method will add all of our specific
    *  toolbar buttons
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addToolBarButtons () {
      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 2 ) ;
   }

   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will free all of our resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      boolean rc = false ;                                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are to force close, bypass everything             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !forceClose ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are not publishing, continue with the close       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !publishing ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the user has not saved, ask them to                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !saved ) {


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If the record is not locked, ask the user               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !locked() ) {
                  if ( ( hasAuth ) && ( GUISystem.printBox ( STR_QUESTION, STR_SAVE_SYMPTOM ) ) )
                     rc = save() ;
                  else {
                     /* Unlock the record */
                     if ( symptom.getRecStatus() == symptom.REC_INSERT )
                        setOpenDraft ( "N" ) ;
                     rc = true ; // Set the return to TRUE
                  }
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise we cannot save, so return TRUE                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  rc = true ;       // Return TRUE since no symptom was taken
                  saved = false ;   // set Saved to FALSE since we did not save
               } /* else */

            } else {
               rc = true ; // Set the return to TRUE since they already saved
            }

         } else {
            GUISystem.printBox ( STR_INFO, STR_CANNOT_CLOSE_PUBLISHING ) ;
            rc = false ; // Set the return to FALSE since we cannot close if publishing
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notify our listeners that we have saved                 */
         /* and update our database flags                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc ) {
            if ( !locked() )
               setMaintlock ( "N" ) ;

            if ( saved ) {
               saveComplete ( symptom ) ;

               if ( published )
                  setOpenDraft ( "N" ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notify the super to stop                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc )
            super.stop() ;


      } else
         rc = true ; // Set return to TRUE since we are forcing the close


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set saved to false for next time around                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      saved = false ;

      if ( rc ) {
         panelDeselected() ;
         flowEditPanel.free() ;
      }

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      try {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CLOSE                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == pb_CLOSE )
            GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_FULLSCREEN                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_FULLSCREEN )
            showFullScreen() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CLOSE_FULL_SCREEN                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_CLOSE_FULL_SCREEN )
            closeFullScreen() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_SAVEAS                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SAVEAS )
            saveAs() ;



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_GRID                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_GRID )
            toggleGrid() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_TITLES                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_TITLES )
            toggleTitles() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_SAVE                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SAVE ) {
            processSave() ;
         } /* else  */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_PROPERTIES                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_PROPERTIES ) {
            new SymptomPropertyDlg ( GUISystem.getParentDefWin ( this ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_SPELL                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SPELL )
            spellCheck() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_VALIDATE                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_VALIDATE ) {
            if ( saveData() ) {
               if ( validateSymptom() )
                  GUISystem.printBox ( STR_INFO, STR_NO_ERRORS_IN_VALIDATION ) ;
            }
         }


      } catch ( Exception e1 ) {
         LogSystem.log ( 1, e1, false ) ;
      }


   }





   /*------------------------------------------------------------*/
   /* toggleGrid
    */
   /**
    *    This method will toggle the grid on and off
    *  that is in the FlowEditPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void toggleGrid () {
      boolean grid = !flowEditPanel.getPaintGrid() ;

      flowEditPanel.setPaintGrid ( grid ) ;

      if ( grid ) {
         pb_GRID.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.GRID_OFF ) ) ;
         pb_GRID.setText ( Str.getStr ( STR_GRID_OFF ) ) ;
      } else {
         pb_GRID.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.GRID_ON ) ) ;
         pb_GRID.setText ( Str.getStr ( STR_GRID_ON ) ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* toggleTitles
    */
   /**
    *    This method will toggle the titles on and off
    *  that is in the FlowEditPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void toggleTitles () {
      boolean titles = !flowEditPanel.getPaintTitles() ;

      flowEditPanel.setPaintTitles ( titles ) ;

      if ( titles ) {
         pb_TITLES.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.GRID_OFF ) ) ;
         pb_TITLES.setText ( Str.getStr ( STR_TITLES_OFF ) ) ;
      } else {
         pb_TITLES.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.TITLES_ON ) ) ;
         pb_TITLES.setText ( Str.getStr ( STR_TITLES_ON ) ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }

   public void saveComplete ( Object object ) {
   }



   /*------------------------------------------------------------*/
   /* editPanelChanged
    */
   /**
    *    This method will be called when something
    *  on the edit panel has changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void editPanelChanged ( EditPanel editPanel ) {
      if ( editPanel == titleEditPanel ) {
         try {
            st_TITLE.setText ( titleEditPanel.getDefaultTitle().getTitle() ) ;
         } catch ( Exception e ) {
         }
      }
   }


   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will get the default title
    *  that shows 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getDefaultTitle () {
      Vector vec = symptom.getSymptomTitles() ;

      return( ((SymptomTitle)vec.elementAt(vec.size()-1)).getTitle() ) ;
   }


   /*------------------------------------------------------------*/
   /* setMaintLock
    */
   /**
    *    This method will set the maint lock flag
    *  to the value passed
    *  
    *
    *  @param value The value to set maintlock in OADRAFT.SYMPTOM to. Either Y or N
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setMaintlock ( String value ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setMaintLock("+value+")", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.SYMPTOM "+
                              "SET MAINTLOCK = '"+value+"', "+
                              "    DBUSER    = '"+UserSystem.getUserId()+"', "+
                              "    CHANGEDTIME = CURRENT TIMESTAMP "+
                              "WHERE SYMPTOMIND = "+symptom.getInd() ) ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* setOpenDraft
    */
   /**
    *    This method will set the open draft flag
    *  to the value passed
    *  
    *
    *  @param value The value to set opendraft in OA.SYMPTOM to. Either Y or N
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setOpenDraft ( String value ) {
      if ( symptom != null ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "setOpenDraft("+value+")", 5 ) ;

         try {
            Statement stmt = sqlMethod.createStatement() ;

            stmt.executeUpdate ( "UPDATE OA.SYMPTOM "+
                                 "SET OPENDRAFT = '"+value+"', "+
                                 "    DBUSER    = '"+UserSystem.getUserId()+"', "+
                                 "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE SYMPTOMIND = "+symptom.getActiveSymptomInd() ) ;
         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e ) ;
         }

         sqlMethod.close() ;
      }
   }




   /*------------------------------------------------------------*/
   /* locked
    */
   /**
    *    This method will return TRUE if this symptom
    *  draft being edited is locked in the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   private boolean locked () {
      boolean locked = false;

      if ( symptom != null ) {
         if ( symptom.getMaintLock() ) {
            locked =  !( symptom.getDbUser().equals ( UserSystem.getUserId() ) ) ;
         }
      }

      return( locked ) ;
   }




   /*------------------------------------------------------------*/
   /* forceClose
    */
   /**
    *    This method will force close the panel
    *  without asking the user any questions
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void forceClose () {
      forceClose = true ;
      parentWin.closePanel ( this, previousPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform a variety of operations
    *  The operation that is done depends on the
    *  name of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_REFRESH ) )
         refreshData() ;

   } /* run  */



   /*------------------------------------------------------------*/
   /* saveAsDraft
    */
   /**
    *    This method will save this symptom
    *  as a draft. It will close the window if the
    *  property is set
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveAsDraft () {                         
      boolean rc = false ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, disable all of our controls                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setEnabled ( false ) ;
      tabbedPane.setEnabled ( false ) ;
      Qit.setEnabled ( false ) ;

      try {
         int saveRc = 0 ;

         parentWin.setStatus ( STR_SAVING ) ;
         saveRc = symptom.writeToDatabase() ;
         LogSystem.log ( 1, "Write Symptom Returned:"+saveRc ) ;
         rc = (saveRc == 0) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         setEnabled ( true ) ;
         tabbedPane.setEnabled ( true ) ;
      }

      tabbedPane.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;
      Qit.setEnabled ( true ) ;
      if ( !rc )
         GUISystem.printBox ( parentWin, STR_INFO, STR_SYMPTOM_NOT_SAVED ) ;

      return( rc ) ;
   } /* saveAsDraft  */




   /*------------------------------------------------------------*/
   /* save
    */
   /**
    *    This method will ask the user what type
    *  of save they want, and will call the appropriate
    *  save methods.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if the object was saved to the database
    *//*---------------------------------------------------------*/
   private boolean save () {
      boolean        rc          = false ;
      int            saveType    = -1 ;
      String         comments    = null ;
      SaveTypeDlg    saveTypeDlg = null ;                                                                                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only continue if the user has entered all of the data   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( saveData() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ask the user what type of save they would like          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         saveTypeDlg = new SaveTypeDlg ( WinUtil.getParentFrame ( this ), 
                                         draftPanel.getOriginalComments(),
                                         draftPanel.getComments(),
                                         symptom.isArchived(),
                                         symptom.getTypeWorkRequired().getInd() == TypeList.WORKREQ_DELETE ) ;

         saveType = saveTypeDlg.getResult() ;
         comments = saveTypeDlg.getComments() ;

         symptom.setComments ( comments ) ;


         switch ( saveType ) {
            case SaveTypeDlg.DRAFT :
               rc = saveAsDraft() ;
               break ;

            case SaveTypeDlg.PUBLISH :
               if ( validateSymptom() ) {
                  if ( saveAsDraft() )
                     saveAsPublish() ;
               }
               break ;

            case SaveTypeDlg.ARCHIVE :
               if ( validateSymptom() ) {
                  symptom.setArchived ( true ) ;
                  if ( saveAsDraft ( ) ) {
                     rc = OAObjectArchiver.archiveObject ( symptom, this ) ;

                     if ( !rc )
                        reopenDraft() ;
                  }
               }
               break ;

            case SaveTypeDlg.UNARCHIVE :
               if ( validateSymptom () ) {
                  symptom.setArchived ( false ) ;
                  if ( saveAsDraft() )
                     saveAsPublish() ;
               }
               break ;

            case SaveTypeDlg.CANCEL_DRAFT :
               if ( OASQL.cancelDraft ( symptom ) ) {
                  saved = true ;
                  forceClose = true ;
                  parentWin.closePanel ( this, previousPanel ) ;
               }
               break ;

         } /* switch */
      }

      saved = rc ;
      return( rc ) ;
   } /* save  */


   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will go down through all
    *  our panels and will ask each one if saving
    *  is ok. It will also save ALL data for the symptom
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      boolean rc = false ;

      rc = ( ( draftPanel.saveData() ) &&
             ( flowEditPanel.saveData() ) &&
             ( titleEditPanel.saveData() ) ) ;     
      Vector nodes = symptom.getNodes() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save the Links                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc ) {
         Vector titles = symptom.getSymptomTitles() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save the brand and class information                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         symptom.setBrand ( draftPanel.getBrand() ) ;

         ((SymptomTitle)titles.elementAt(titles.size()-1)).setDocClass ( draftPanel.getDocClass() ) ;
         symptom.setSymptomTitles ( titles ) ;

         if ( draftPanel.getStatusData() != null )
            symptom.setSymptomStatus ( draftPanel.getStatusData() ) ;
         else {
            GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_STATUS );
            rc = false ;
         }

         if ( rc )
            rc = ( ( symptom.getBrand()    != null ) &&
                   ( symptom.getDocClass() != null ) ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save the Nodes                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc )
            symptom.setNodes ( flowEditPanel.getNodes() ) ;

      }

      /*--------------------------------------------------------------*/
      /*                                                              */
      /*   If all of the generic panels saved their data, save ours   */
      /*                                                              */
      /*--------------------------------------------------------------*/
      if ( rc ) {
         symptom.setSymptomType ( (TypeSymptomRec) draftPanel.getSelectedObjectType() ) ;
         symptom.setDbUser ( UserSystem.getUserId() ) ;
      }

      return( rc ) ;
   } /* saveData  */




   /*------------------------------------------------------------*/
   /* saveAsPublish
    */
   /**
    *    This method will start the publishing
    *  process
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveAsPublish () {                        


      if ( validateSymptom () ) {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Unlock the symptom                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( !locked() ) && ( symptom.getRecStatus() != symptom.REC_INSERT ) ) {
            setMaintlock ( "N" ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Put the publish panel up                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         publishPanel = new PublishPanel ( symptom ) ;
         publishPanel.setPublishPanelListener ( this ) ;
         contentPanel.remove ( tabbedPane ) ;
         contentPanel.add ( publishPanel, BorderLayout.CENTER ) ;
         pb_SAVE.setVisible ( false ) ;
         pb_SAVEAS.setVisible ( false ) ;
         pb_VALIDATE.setVisible ( false ) ;
         pb_CLOSE.setVisible ( false ) ;
         publishing = true ;
         new Thread ( publishPanel, publishPanel.THREAD_VALIDATE ).start() ;
      }
   }





   /*------------------------------------------------------------*/
   /* publishComplete
    */
   /**
    *    This method will be called once a publish has
    *  completed
    *  
    *
    *  @param complete TRUE = The object was published, FALSE if not
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void publishComplete ( boolean complete ) {
      if ( complete ) {
         contentPanel.remove ( publishPanel ) ;
         publishing = false ;
         saved = true ;
         published = true ;
         parentWin.closePanel ( this, previousPanel ) ;
      } else {
         pb_SAVE.setVisible ( true ) ;
         pb_SAVEAS.setVisible ( true ) ;
         pb_VALIDATE.setVisible ( true ) ;
         pb_CLOSE.setVisible ( true ) ;
         contentPanel.remove ( publishPanel ) ;
         publishPanel = null ;
         contentPanel.add ( tabbedPane ) ;
         publishing = false ;
         published = false ;
         setVisible ( false ) ;
         setVisible ( true ) ;                    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, we must reopen the draft                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         reopenDraft() ;

      }
   }

   public void printToPrinter ( PrintJob printJob ) {
      flowEditPanel.printToPrinter ( printJob ) ;
   }

   public Object getPrintObject ( ) {
      return( this ) ;
   }


   /*------------------------------------------------------------*/
   /* getPublished
    */
   /**
    *    This method will return TRUE if the last
    *  save we did published the object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getPublished () {
      return( published ) ;
   }





   /*------------------------------------------------------------*/
   /* getFlowEditPanel
    */
   /**
    *    This method will return the instance of the
    *  FlowEditPanel that is being used for this
    *  Symptom Edit
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FlowEditPanel
    *//*---------------------------------------------------------*/
   public FlowEditPanel getFlowEditPanel () {
      return( flowEditPanel ) ;
   }


   /*------------------------------------------------------------*/
   /* getSymptom
    */
   /**
    *    This method will return the symptom that we
    *  are editing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Symptom
    *//*---------------------------------------------------------*/
   public Symptom getSymptom() {
      return( symptom ) ;
   }





   /*------------------------------------------------------------*/
   /* showFullScreen
    */
   /**
    *    This method will show the full screen Visio
    *  Editor
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showFullScreen () {
      Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize () ;

      if ( fullScreenWindow == null ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Remove the Flow PAnel                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         contentPanel.remove ( tabbedPane ) ;
         tabbedPane.setSelectedComponent ( flowEditPanel ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the new Full Screen Window                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         fullScreenWindow = new JWindow ( GUISystem.getParentDefWin ( this ) ) ;                                 

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the button to close the full screen              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_CLOSE_FULL_SCREEN = new DButton ( Str.getStr ( STR_RETURN ) ) ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, add our controls to the full screen window         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         fullScreenWindow.getContentPane().setLayout ( new BorderLayout() ) ;
         fullScreenWindow.getContentPane().add ( pb_CLOSE_FULL_SCREEN, BorderLayout.NORTH ) ;
         fullScreenWindow.getContentPane().add ( tabbedPane, BorderLayout.CENTER ) ;           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add action listeners                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_CLOSE_FULL_SCREEN.addActionListener ( this ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, put up the full window                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      fullScreenWindow.setBounds ( 0, 0, scrSize.width, scrSize.height ) ;
      fullScreenWindow.setVisible ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* getFullScreen
    */
   /**
    *    This method will return TRUE if we are
    *  currently in full screen more
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = We are in full screen, FALSE = we are not in full screen
    *//*---------------------------------------------------------*/
   public boolean getFullScreen () {
      return( fullScreenWindow != null ) ;
   }




   /*------------------------------------------------------------*/
   /* closeFullScreen
    */
   /**
    *    This method will close the full screen window
    *  if it is open and return it to the main window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void closeFullScreen () {
      if ( fullScreenWindow != null ) {
         fullScreenWindow.remove ( tabbedPane ) ;
         contentPanel.add ( tabbedPane ) ;
         fullScreenWindow.dispose() ;
         fullScreenWindow = null ;
         pb_CLOSE_FULL_SCREEN = null ;
         invalidate() ;
         tabbedPane.invalidate() ;
         revalidate() ;
      }
   }





   /*------------------------------------------------------------*/
   /* validateSymptom
    */
   /**
    *    This method will validate the symptom being
    *  edited. It will return TRUE if the validation was
    *  without error
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   private boolean validateSymptom () {
      return( !(new SoftValidateDlg ( GUISystem.getParentDefWin ( this ), 
                                      SymptomSoftValidator.validateSymptom ( symptom, flowEditPanel.getObjectNodes() ) ).getHadErrors()) ) ;
   }

   /*------------------------------------------------------------*/
   /* spellCheck
    */
   /**
    *    This method will spell check all of our
    *  panels.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void spellCheck () {
      add ( spellPanel, BorderLayout.NORTH ) ;
      revalidate() ;
      titleEditPanel.spellCheck ( tabbedPane, spellPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* This method will be called whenever one
    */
   /**
    *    of our EditorPanels has completed a spell
    *  check. Since there are many panels to spell check,
    *  we will use this oppurtunity to kick off the next spell check on the next panel
    *
    *  @param editPanel The editPanel that just got finished with the Spell Check
    *  @param errorsFound TRUE if any errors were found, regardless if they were fixed
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void spellCheckComplete ( EditPanel editPanel, boolean errorsFound ) {                                                                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we just completed checking the Titles, now           */
      /* check the answers                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( editPanel == titleEditPanel ) {
         if ( !errorsFound )
            GUISystem.printBox ( STR_INFO, STR_SPELL_CHECK_COMPLETE ) ;
         remove ( spellPanel ) ;
         revalidate() ;
      }
   }





   /*------------------------------------------------------------*/
   /* saveAs
    */
   /**
    *    This method will basically create a symptom
    *  from the current symptom. The newly created
    *  symptom will be a completely new symptom
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveAs () {
      String       newDefaultTitle = null ;
      SymptomTitle defaultTitle    = symptom.getDefaultTitle() ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, get the new default title                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      newDefaultTitle = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                             Str.getStr ( STR_TITLE ),
                                             Str.getStr ( STR_SAVE_AS_TITLE ),
                                             defaultTitle.getTitle(),
                                             "" ).getResult() ;                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if they gave us a title, continue                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( newDefaultTitle != null ) && ( newDefaultTitle.length() > 0 ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, close the current Symptom                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setMaintlock ( "N" ) ;
         setOpenDraft ( "N" ) ;                          

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create the new one and save it                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         symptom.resetAsNew() ;
         defaultTitle.setTitle ( newDefaultTitle ) ;
         if ( processSave() )
            GUISystem.printBox ( STR_INFO, STR_NEW_SYMPTOM_SAVED_AS ) ;
      }
   } /* saveAS  */




   /*------------------------------------------------------------*/
   /* processSave
    */
   /**
    *    THis method will start the save process
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything saved, FALSE if not
    *//*---------------------------------------------------------*/
   private boolean  processSave () {
      boolean rc = false ;

      if ( save() ) {
         if ( PropertySystem.getBool ( PropertySystem.PROPERTY_SAVE_AND_CLOSE ) )
            parentWin.closePanel ( this, previousPanel ) ;
         else
            saved = false ;
         rc = true ;
      } /* if */

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* countryAdded
    */
   /**
    *    This method will be called when the user attempts
    *  to add a country to this Question. We do nothing
    *  here
    *
    *  @param countryRec The TypeCountryCodeRec added
    *  @param None
    *  @param None
    *
    *  @return TRUE
    *//*---------------------------------------------------------*/
   public boolean countryAdded ( TypeCountryCodeRec countryRec ) {
      SymptomTitle title = AvalonUtil.getDefaultTitleObject ( symptom ) ;
      Vector       countries = title.getCountryList() ;

      if ( !countries.contains ( countryRec ) )
         countries.addElement ( countryRec ) ;

      title.setCountryList ( countries ) ;
      new Thread ( titleEditPanel ).start() ; // Refresh the titles

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* countryRemoved
    */
   /**
    *    This method will be called when a country
    *  is trying to be removed from this Question. We
    *  first have to ask the user if they want to remove
    *  that country from all properties (titles, add info etc..)
    *  from the Question. If they answer 'Yes', then we allow the removal,
    *  if the answer 'No' then we do not allow the removal
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean countryRemoved ( TypeCountryCodeRec countryRec ) {
      String  questionStr = Str.getStr ( STR_ARE_YOU_SURE_REMOVE_COUNTRY_FROM_OBJECT ) ;
      boolean remove      = false ;

      questionStr = Text.replaceInStr ( questionStr, "%COUNTRYNAME%", countryRec.toString() ) ;

      if ( askRemoveCountryQuestion ) {
         remove = GUISystem.printBox ( Str.getStr ( STR_QUESTION ), questionStr ) ;
         askRemoveCountryQuestion = !remove ;
      } else
         remove = true ;

      if ( remove )
         removeCountryFromPanels ( countryRec ) ;

      return( remove ) ;
   }





   /*------------------------------------------------------------*/
   /* countryRemoveDone
    */
   /**
    *    This method will be called once all countries
    *  have been removed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void countryRemoveDone () {
      askRemoveCountryQuestion = true ;
   }



   /*------------------------------------------------------------*/
   /* removeCountryFromPanels
    */
   /**
    *    This method will go through all of our
    *  panels and remove the country that is passed in
    *  from all objects on them
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeCountryFromPanels ( TypeCountryCodeRec countryRec ) {
      titleEditPanel.removeCountryFromPanel ( countryRec ) ;
      flowEditPanel.removeCountryFromPanel  ( countryRec ) ;
   }






   /*------------------------------------------------------------*/
   /* reopenDraft
    */
   /**
    *    This method will reopen the draft we are
    *  editing by setting the stop date to null, it will
    *  also reset the MaintLock to 'Y'.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void reopenDraft () {
      SQLMethod          sqlMethod       = new SQLMethod ( DatabaseSystem.QUEST, "reopenDraft", 5 ) ;
      SymptomEditorPanel newSymptomPanel = null ;

      GUISystem.getParentDefWin ( this ).setStatus ( STR_READING ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.SYMPTOM SET STOPDATE=NULL, MAINTLOCK='Y' "+
                              "WHERE SYMPTOMIND = "+symptom.getInd() ) ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;                     


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we must reload the object so that all values will  */
      /* now be update.                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      WindowSystem.createPanel ( newSymptomPanel = new SymptomEditorPanel ( previousPanel, 
                                                                            symptom.getInd() ) ) ;

      forceClose = true ;
      GUISystem.getParentDefWin ( this ).closePanel ( this, newSymptomPanel ) ;
   }

} /* SymptomEditorPanel */
