package com.ibm.nzna.projects.qit.avalon.editors ;              
/*-----------------------------------------------------------*/
/*   SymptomKeywordDlg                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomKeywordDlg                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog will allow the user to enter some keywords, 
 * and to select 'Update All', 'Update Selected' or 'Cancel'. The
 * dialog's constructor will show the dialog and waits until the user
 * closes the window. After which the constructor will return. The methods
 * getResult() and getKeywords can be used to get what the user entered
 *
 * @version 1.0, 10/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomKeywordDlg                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomKeywordDlg extends JDialog implements ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int UPDATE_ALL        = 1 ;
   public static final int UPDATE_SELECTED   = 2 ;
   public static final int CANCEL            = 3 ;          

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_HELP            = null ;
   private JTextArea    mle_KEYWORDS       = null ;
   private JScrollPane  scr_KEYWORDS       = null ;
   private DButton      pb_UPDATE_ALL      = null ;
   private DButton      pb_UPDATE_SELECTED = null ;
   private DButton      pb_CANCEL          = null ;
   private ButtonPanel  buttonPanel        = null ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String keywords                = null ;
   private int    rc                      = CANCEL ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will show the dialog and waits
    *  for user input. Once the window is closed, the
    *  constructor returns.
    *
    *  @param parent The parent frame
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomKeywordDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_KEYWORDS ), true ) ;

      initializeGUI () ;
      setSize ( 400, 300 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
      setVisible ( true ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will show the dialog and waits
    *  for user input. Once the window is closed, the
    *  constructor returns.
    *
    *  @param parent The parent frame
    *  @param keywords The initial keywords to put in
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomKeywordDlg ( Frame parent, String keywords ) {
      super ( parent, Str.getStr ( STR_KEYWORDS ), true ) ;

      initializeGUI () ;
      mle_KEYWORDS.setText ( keywords ) ;
      setSize ( 400, 300 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
      setVisible ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* initializeGUI
    */
   /**
    *    this method will initialize our GUI and add it
    *  to our contentPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeGUI () {
      Container contentPanel = getContentPane () ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP            = new JLabel ( Str.getStr ( STR_ENTER_KEYWORDS ) ) ;
      mle_KEYWORDS       = new JTextArea ( "" ) ;
      scr_KEYWORDS       = new JScrollPane ( mle_KEYWORDS ) ;
      pb_UPDATE_ALL      = new DButton ( Str.getStr ( STR_UPDATE_ALL ) ) ;
      pb_UPDATE_SELECTED = new DButton ( Str.getStr ( STR_UPDATE_SELECTED ) ) ;
      pb_CANCEL          = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      buttonPanel        = new ButtonPanel () ;    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_UPDATE_ALL.addActionListener      ( this ) ;
      pb_UPDATE_SELECTED.addActionListener ( this ) ;
      pb_CANCEL.addActionListener          ( this ) ;  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPanel.setLayout ( null ) ;
      contentPanel.add ( st_HELP ) ;
      contentPanel.add ( scr_KEYWORDS ) ;
      buttonPanel.add ( pb_UPDATE_ALL ) ;
      buttonPanel.add ( pb_UPDATE_SELECTED ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      contentPanel.add ( buttonPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         rc = CANCEL ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_UPDATE_ALL                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_UPDATE_ALL ) {
         keywords = mle_KEYWORDS.getText() ;
         rc = UPDATE_ALL ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_UPDATE_SELECTED                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_UPDATE_SELECTED ) {
         keywords = mle_KEYWORDS.getText() ;
         rc = UPDATE_SELECTED ;
         dispose() ;
      }
   } /* actionPerformed  */





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       y         = 5 ;
      int       x         = 5 ;
      int       rowHeight = GUISystem.getRowHeight () ;

      super.doLayout() ;

      st_HELP.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      scr_KEYWORDS.setBounds ( x, y, size.width-(x*2), size.height-y-55 ) ;

      buttonPanel.setBounds ( x, size.height-50, size.width-(x*2), 25 ) ;
   }




   /*------------------------------------------------------------*/
   /* getResults
    */
   /**
    *    THis method will return one of the predefined
    *  results (UPDATE_ALL, UPDATE_SELECTED or
    *  CANCEL)
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getResults () {
      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* getKeywords
    */
   /**
    *    This method will return the keywords that the
    *  user has entered
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getKeywords () {
      return( keywords ) ;
   }
} /* SymptomKeywordDlg */
