package com.ibm.nzna.projects.qit.avalon.editors ;           
/*-----------------------------------------------------------*/
/*   SymptomUsagePanel                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomUsagePanel                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This panel shows what Symptoms an Action is used in.
 *
 *
 * @version 1.0, 11/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomUsagePanel                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomUsagePanel extends JPanel implements ListSelectionListener, Runnable, ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Column Headers                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static String[] columns = { "", "", ""} ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private MultiList       subRoutineSymptomList        = null ;
   private MultiList       symptomList                  = null ;
   private JLabel          st_SYMPTOMS                  = null ;
   private JLabel          st_SUBROUTINE_SYMPTOMS       = null ;
   private HotLinkLabel    pb_REFRESH                   = null ;
   private ListActionPanel buttonPanel                  = null ;   
   private JTextArea       mle_OUTPUT                   = null ;
   private JScrollPane     scr_OUTPUT                   = null ;
   private ListActionPanel symptomButtonPanel           = null ;
   private ListActionPanel subRoutineSymptomButtonPanel = null ;
   private HotLinkLabel    pb_EDIT_SYMPTOM              = null ;
   private HotLinkLabel    pb_EDIT_SUBROUTINE_SYMPTOM   = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean            refreshing          = false ;
   private boolean            refreshed           = false ;
   private SymptomDraft       symptom             = null ;
   private Dimension          prefSize            = new Dimension ( 100, 480 ) ;
   private FlowEditPanel      flowEditPanel       = null ;
   private SymptomEditorPanel symptomEditorPanel  = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our controls,
    *  however the refresh will not happen until the 
    *  setVisible method is called.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomUsagePanel ( SymptomEditorPanel symptomEditorPanel ) {
      this.symptomEditorPanel = symptomEditorPanel ;
      initializeGUI () ;
   }




   /*------------------------------------------------------------*/
   /* intializeGUI
    */
   /**
    *    This method will initialize the GUI and add
    *  it to our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeGUI () {          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SUBROUTINE_SYMPTOMS        = new JLabel ( Str.getStr ( STR_SUBROUTINE_SYMPTOMS ) ) ;
      st_SYMPTOMS                   = new JLabel ( Str.getStr ( STR_SYMPTOM_THAT_USE_THIS_SYMPTOM ) ) ;
      subRoutineSymptomList         = new MultiList ( GUISystem.getFontUtil() ) ;
      symptomList                   = new MultiList ( GUISystem.getFontUtil() ) ;
      pb_REFRESH                    = new HotLinkLabel ( Str.getStr ( STR_REFRESH ) ) ;
      buttonPanel                   = new ListActionPanel() ;     
      mle_OUTPUT                    = new JTextArea() ;
      scr_OUTPUT                    = new JScrollPane ( mle_OUTPUT ) ;
      symptomButtonPanel            = new ListActionPanel () ;
      subRoutineSymptomButtonPanel  = new ListActionPanel () ;
      pb_EDIT_SYMPTOM               = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT_SUBROUTINE_SYMPTOM    = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SUBROUTINE_SYMPTOMS.setOpaque ( false ) ;  
      st_SYMPTOMS.setOpaque ( false ) ;
      setListProperties ( subRoutineSymptomList ) ;
      setListProperties ( symptomList ) ;
      mle_OUTPUT.setEditable ( false ) ;
      mle_OUTPUT.setFont ( FontSystem.defaultFont ) ;
      setBackground ( Color.white ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      buttonPanel.add ( pb_REFRESH ) ;
      symptomButtonPanel.add ( pb_EDIT_SYMPTOM ) ;
      subRoutineSymptomButtonPanel.add ( pb_EDIT_SUBROUTINE_SYMPTOM ) ;
      add ( st_SUBROUTINE_SYMPTOMS ) ;
      add ( subRoutineSymptomList ) ;
      add ( subRoutineSymptomButtonPanel ) ;
      add ( st_SYMPTOMS ) ;
      add ( symptomList ) ;
      add ( symptomButtonPanel ) ;
      add ( buttonPanel ) ;
      add ( scr_OUTPUT ) ;    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_REFRESH.addActionListener ( this ) ;
      pb_EDIT_SYMPTOM.addActionListener ( this ) ;
      pb_EDIT_SUBROUTINE_SYMPTOM.addActionListener ( this ) ;
      symptomList.addActionListener ( this ) ;
      subRoutineSymptomList.addActionListener ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      size.height = prefSize.height ;

      st_SUBROUTINE_SYMPTOMS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      subRoutineSymptomList.setBounds ( x, y, size.width-(x*2), rowHeight*5 ) ;
      y+=(rowHeight*5) ;

      subRoutineSymptomButtonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=(rowHeight*2) ;

      st_SYMPTOMS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      symptomList.setBounds ( x, y, size.width-(x*2), rowHeight*5 ) ;
      y+=(rowHeight*5) ;

      symptomButtonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=(rowHeight*2) ;

      buttonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=( rowHeight*2) ;

      scr_OUTPUT.setBounds ( x, y, size.width-(x*2), rowHeight*3 ) ;
      super.doLayout() ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called once an symptom
    *  event is fired on us.
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) { 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REFRESH                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_REFRESH ) {
         if ( !refreshing ) {
            refreshed = false ;
            refresh() ;
         }
      } /* if  */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_SUBROUTINE_SYMPTOM                              */                 
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == symptomList ) || ( e.getSource() == pb_EDIT_SUBROUTINE_SYMPTOM ) )
         OAEdit.edit ( (UsageRec)subRoutineSymptomList.getSelectedItem(),
                       symptomEditorPanel ) ;
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT_SYMPTOM                                         */                 
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == subRoutineSymptomList ) || ( e.getSource() == pb_EDIT_SYMPTOM ) )
         OAEdit.edit ( (UsageRec)symptomList.getSelectedItem(),
                       symptomEditorPanel ) ;

      
   } /* actionPerformed  */





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will kick off a thread to refresh
    *  our data. If the refresh thread has already been
    *  started then nothing happens
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refresh () {
      if ( ( !refreshing ) && ( !refreshed ) )
         new Thread ( this ).start() ;
   } /* refresh  */





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our refresh thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      try {
         if ( ( flowEditPanel != null ) && ( symptom != null ) ) {
            Vector subVec = null ;
            Vector symVec = null ;

            symptom.setNodes ( flowEditPanel.getNodes() ) ;
            GUISystem.getParentDefWin ( this ).setStatus ( Str.getStr ( STR_READING_USAGE ) ) ;

            subVec = SymptomUsage.getSubroutineSymptoms ( symptom ) ;
            symVec = SymptomUsage.getSymptomUsage ( symptom ) ;

            subRoutineSymptomList.removeAll() ;
            symptomList.removeAll() ;

            if ( subVec != null )
               subRoutineSymptomList.add ( subVec ) ;

            if ( symVec != null )
               symptomList.add ( symVec ) ;

         }

         refreshed = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;
      refreshing = false ;
   }




   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method is subclassed so that when we
    *  show ourselves for the first time we refresh
    *  our data
    *
    *  @param g The Graphics Context.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      super.paintComponent ( g ) ;

      refresh() ;
   }




   /*------------------------------------------------------------*/
   /* setAction
    */
   /**
    *    This method will set what symptom we are getting
    *  the usage for
    *  
    *
    *  @param symptomDraft The symptom draft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setSymptom ( SymptomDraft symptomDraft ) {
      this.symptom = symptomDraft ;
   }


   /*------------------------------------------------------------*/
   /* setListProperties
    */
   /**
    *    This method will set the properties on the
    *  MultiList passed. These properties include
    *  the column headings, column width and to turn off 
    *  the visible column headers
    *
    *  @param multiList The MultiList to set properties on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setListProperties ( MultiList multiList ) {
      multiList.setColumnHeadings ( columns ) ;
      multiList.setColumnWidth ( 0, 20 ) ;
      multiList.setColumnWidth ( 1, 75 ) ;
      multiList.setColumnWidth ( 2, 350 ) ;
      multiList.showColumnHeadings ( false ) ;
      multiList.addListSelectionListener ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when an item in
    *  our list is refreshed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( !e.getValueIsAdjusting() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   List of SubRoutine Symptoms                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == subRoutineSymptomList ) {
            if ( subRoutineSymptomList.getSelectedItem() != null ) {
               UsageRec usageRec = (UsageRec)subRoutineSymptomList.getSelectedItem() ;

               mle_OUTPUT.setText ( usageRec.getExplanation() ) ;
            } else
               mle_OUTPUT.setText ( "" ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   List of Symptoms                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == symptomList ) {
            if ( symptomList.getSelectedItem() != null ) {
               UsageRec usageRec = (UsageRec)symptomList.getSelectedItem() ;

               mle_OUTPUT.setText ( usageRec.getExplanation() ) ;
            } else
               mle_OUTPUT.setText ( "" ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our default preferred size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will determine the minimal amount
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }





   /*------------------------------------------------------------*/
   /* setFlowEditPanel
    */
   /**
    *    This method will set the FlowEditPanel that
    *  this panel needs. The need is because the
    *  flow must be saved before validation.
    *
    *  @param flowEditPanel The FlowEditPanel editing the symptom that is being checked
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setFlowEditPanel ( FlowEditPanel flowEditPanel ) {
      this.flowEditPanel = flowEditPanel ;
   }

} /* SymptomUsagePanel */
