package com.ibm.nzna.projects.qit.avalon.editors ;                          
/*-----------------------------------------------------------*/
/*   SymptomValidatorPanel                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomValidatorPanel               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.sql.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import javax.swing.border.* ;
import java.awt.event.* ;
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.oa.engine.* ;
/**
 * This panel allows a user to walk through a symptom. It interfaces
 * with the com.ibm.nzna.projects.oa.engine classes to do the real work.
 *
 *
 * @version 1.0, 6/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomValidatorPanel                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomValidatorPanel extends JPanel implements AvalonConst, AppConst, ActionListener, ListSelectionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextField       ef_MODEL          = null ;
   private JTitle           st_MODEL          = null ;
   private HotLinkLabel     pb_CHANGE         = null ;
   private JList            lb_HISTORY        = null ;
   private JList            lb_ANSWERS        = null ;
   private JTitle           st_HISTORY        = null ;
   private Browser          browser           = null ;
   private JTextArea        mle_SOURCE        = null ;
   private ButtonPanel      buttonPanel       = null ;
   private HotLinkLabel     pb_EDIT           = null ;
   private HotLinkLabel     pb_REMOVE         = null ;
   private JTitle           st_WEBSITE        = null ;
   private JTitle           st_GEOGRAPHY      = null ;
   private JTitle           st_ANSWERS        = null ;
   private JComboBox        cb_WEBSITE        = null ;
   private JComboBox        cb_COUNTRY        = null ;
   private JScrollPane      historyScroll     = null ;
   private JScrollPane      answerScroll      = null ;
   private DButton          pb_START          = null ;
   private DefaultListModel historyListModel  = null ;
   private DefaultListModel answerListModel   = null ;
   private JTabbedPane      outputPane        = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean               started            = false ;
   private SymptomEditorPanel    symptomEditorPanel = null ;
   private LogicAPI              logicAPI           = null ;
   private OAEngine              oaEngine           = null ;
   private TypeBrandRec          brandRec           = null ;
   private OASubmitVars          oaSubmitVars       = null ;
   private OAReturnVars          oaReturnVars       = null ;
   private Properties            countryProp        = new Properties() ;
   private Properties            countryProp2       = new Properties() ;
   private Properties            langProp           = new Properties() ;
   private AvalonMemoryInterface memoryInterface    = null ;
   private Symptom               symptom            = null ;
   private String                machineModel       = null ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our blank panel
    *  
    *  
    *
    *  @param memoryInterface The AvalonMemoryInterface to use with the OAEngine.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomValidatorPanel ( AvalonMemoryInterface memoryInterface, SymptomEditorPanel symptomEditorPanel ) {
      this.symptomEditorPanel = symptomEditorPanel ;
      this.memoryInterface    = memoryInterface ;
      initialize() ;
   }







   /*------------------------------------------------------------*/
   /* setSymptom
    */
   /**
    *    This method will set what symptom this
    *  validator will run through
    *  
    *
    *  @param symptom The symptom to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSymptom ( Symptom symptom ) {
      this.symptom = symptom ;
   }


   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel by adding
    *  all of the gui
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     
      Vector        webSites  = new Vector ( 3 ) ;
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;
      Color         lightGray = new Color ( 240, 240, 240 ) ;

      if ( parentWin != null )
         parentWin = MainWindow.getInstance() ;

      webSites.addElement ( Str.getStr ( STR_ESUPPORT) ) ;
      webSites.addElement ( Str.getStr ( STR_BUSINESS_PARTNER ) ) ;
      webSites.addElement ( Str.getStr ( STR_INFOTIPS ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HISTORY       = new JTitle ( Str.getStr ( STR_HISTORY ) ) ;
      lb_HISTORY       = new JList () ;
      historyScroll    = new JScrollPane ( lb_HISTORY ) ;
      browser          = new Browser () ;
      pb_REMOVE        = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_EDIT          = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;     
      st_WEBSITE       = new JTitle ( Str.getStr ( STR_WEBSITE ) ) ;
      st_GEOGRAPHY     = new JTitle ( Str.getStr ( STR_GEOGRAPHY ) ) ;
      cb_COUNTRY       = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ) ;
      cb_WEBSITE       = new JComboBox ( webSites ) ;
      lb_ANSWERS       = new JList () ;
      answerScroll     = new JScrollPane ( lb_ANSWERS ) ;
      st_ANSWERS       = new JTitle ( Str.getStr ( STR_ANSWERS )  ) ;
      buttonPanel      = new ButtonPanel () ;
      pb_START         = new DButton ( Str.getStr ( STR_START ) ) ;
      historyListModel = new DefaultListModel() ;
      answerListModel  = new DefaultListModel() ;
      pb_CHANGE        = new HotLinkLabel ( Str.getStr ( STR_SET_MODEL ) ) ;
      st_MODEL         = new JTitle  ( Str.getStr ( STR_MODEL ) ) ;
      ef_MODEL         = new JTextField (  ) ;
      outputPane       = new JTabbedPane () ;
      mle_SOURCE       = new JTextArea ( ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_MODEL.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, "XXXX-XXX", ef_MODEL ) ) ;
      browser.setBorder ( GUISystem.blackBorder ) ;
      historyScroll.setBorder ( GUISystem.blackBorder ) ;
      pb_REMOVE.setOpaque ( false ) ;
      pb_EDIT.setOpaque ( false ) ;
      buttonPanel.setBorder ( GUISystem.grayBorder ) ;
      buttonPanel.setBorder     ( GUISystem.grayBorder ) ;
      buttonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;                        
      setStartButton ( true ) ;
      pb_START.setFocusPainted ( false ) ;
      st_GEOGRAPHY.setFont ( FontSystem.smallTitleFont ) ;
      st_HISTORY.setFont ( FontSystem.smallTitleFont ) ;
      st_WEBSITE.setFont ( FontSystem.smallTitleFont ) ;
      st_ANSWERS.setFont ( FontSystem.smallTitleFont ) ;
      st_MODEL.setFont   ( FontSystem.smallTitleFont ) ;
      setBackground ( Color.white ) ;
      browser.showZoom ( false ) ;
      lb_HISTORY.setModel ( historyListModel ) ;
      lb_ANSWERS.setModel ( answerListModel ) ;
      pb_CHANGE.setBackground ( new Color ( 230, 230, 230 ) ) ;
      pb_CHANGE.setBorder ( GUISystem.grayBorder ) ;
      outputPane.setBorder ( GUISystem.grayBorder ) ;
      //pb_CHANGE.setFocusPainted ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_EDIT.addActionListener ( this ) ;
      pb_REMOVE.addActionListener ( this ) ;  
      pb_START.addActionListener ( this ) ;
      pb_CHANGE.addActionListener ( this ) ;
      lb_ANSWERS.addListSelectionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( st_HISTORY ) ;
      add ( st_GEOGRAPHY ) ;
      add ( cb_COUNTRY ) ;
      add ( st_WEBSITE ) ;
      add ( cb_WEBSITE ) ;
      add ( pb_START ) ;
      add ( historyScroll ) ;
      add ( outputPane ) ;
      outputPane.addTab ( Str.getStr ( STR_OUTPUT ), browser ) ;
      outputPane.addTab ( Str.getStr ( STR_SOURCE ), new JScrollPane ( mle_SOURCE ) ) ;
      add ( st_ANSWERS ) ;
      add ( answerScroll ) ;
      add ( buttonPanel ) ;
      add ( st_MODEL ) ;
      add ( ef_MODEL ) ;
      add ( pb_CHANGE ) ;
      buttonPanel.add ( pb_EDIT ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_REMOVE ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Load in our properties                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         countryProp.load ( new FileInputStream ( new File ( "countryProperties.prop" ) ) ) ;
         countryProp2.load ( new FileInputStream ( new File ( "countryProperties2.prop" ) ) ) ;
         langProp.load ( new FileInputStream ( new File ( "languageProperties.prop" ) ) ) ;                    
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( parentWin, STR_INFO, STR_LOCATE_COUNTRY_LANG_PROPERTIES ) ;
         countryProp = null ;
         langProp = null ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our initial values                                  */             
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( PropertySystem.getString ( PROPERTY_VALIDATOR_PRODUCT ) != null )
         ef_MODEL.setText ( PropertySystem.getString ( PROPERTY_VALIDATOR_PRODUCT ) ) ;

      if ( PropertySystem.getInt ( PROPERTY_VALIDATOR_COUNTRY ) > 0 )
         cb_COUNTRY.setSelectedItem ( TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PROPERTY_VALIDATOR_COUNTRY ), TypeList.TYPE_COUNTRYCODE ) ) ;
      
      if ( PropertySystem.getString ( PROPERTY_VALIDATOR_SITE ) != null )
         cb_WEBSITE.setSelectedItem ( PropertySystem.getString ( PROPERTY_VALIDATOR_SITE ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set final properties on GUI                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      outputPane.setBackgroundAt ( 0, lightGray ) ;
      outputPane.setBackgroundAt ( 1, lightGray ) ;




   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = 15 ;
      int       y         = 5 ;

      st_MODEL.setBounds ( x, y, 200, rowHeight ) ;
      st_GEOGRAPHY.setBounds ( x+205, y, 200, rowHeight ) ;
      st_WEBSITE.setBounds ( x+410, y, 200, rowHeight ) ;
      pb_START.setBounds ( x+615, y, 70, rowHeight*2 ) ;
      y+=rowHeight ;

      ef_MODEL.setBounds ( x, y, 125, rowHeight ) ;
      pb_CHANGE.setBounds ( x+128, y, 72, rowHeight ) ;
      cb_COUNTRY.setBounds ( x+205, y, 200, rowHeight ) ;
      cb_WEBSITE.setBounds ( x+410, y, 200, rowHeight ) ;
      buttonPanel.setBounds ( x, size.height-25, 200, rowHeight ) ;
      y+=rowHeight+5 ;

      st_HISTORY.setBounds ( x, y, 200, rowHeight ) ;
      outputPane.setBounds ( x+205, y, size.width-(x*3)-205, size.height-180-y-rowHeight ) ;
      y+=rowHeight ;

      historyScroll.setBounds ( x, y, 200, size.height-30-y ) ;
      y=(size.height-130-y ) ;

      st_ANSWERS.setBounds ( x+205, y, size.width-(x*3)-205, rowHeight ) ;
      y+=rowHeight ;

      answerScroll.setBounds ( x+205, y, size.width-(x*3)-205, size.height-y-5 ) ;


   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are listening for has been fired
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_REMOVE ) {
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CHANGE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CHANGE ) {
         ProductRec productRec = new BrandFamilyMachineModelDlg ( GUISystem.getParentDefWin ( this ) ).getResults() ;

         if ( productRec != null ) {
            ef_MODEL.setText ( ((MachineRec)productRec.getFieldData ( PRODFIELD_MACHINE )).toString()+
                               ((String)productRec.getFieldData ( PRODFIELD_MODEL )) ) ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT ) {
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_START                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_START ) {

         if ( started )
            stopSymptom() ;
         else
            startSymptom() ;
      }


   }





   /*------------------------------------------------------------*/
   /* setStartButton
    */
   /**
    *    This method will either turn the start button
    *  to a 'Start' or 'Stop', depending on the param
    *  passed in.
    *
    *  @param start TRUE = Make the button green, saying 'Start', FALSE = Make the button Red saying 'Stop'
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setStartButton ( boolean start ) {
      if ( start ) {
         pb_START.setBackground ( new Color ( 192, 252,192 ) ) ;
         pb_START.setBorder ( new LineBorder ( new Color ( 0, 128, 0 ) ) ) ;
         pb_START.setForeground ( new Color ( 0, 128, 0 ) ) ;
         pb_START.setText ( Str.getStr ( STR_START ) ) ;
      } else {
         pb_START.setBackground ( new Color ( 255, 192, 192 ) ) ;
         pb_START.setBorder ( new LineBorder ( new Color ( 128, 0, 0 ) ) ) ;
         pb_START.setForeground ( new Color ( 128, 0, 0 ) ) ;
         pb_START.setText ( Str.getStr ( STR_STOP ) ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* startSymptom
    */
   /**
    *    This method will kick off the OAEngine to
    *  start walking through the user's current
    *  symptom
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void startSymptom () {
      AppDefaultWin parentWin   = GUISystem.getParentDefWin ( this ) ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Grab the machine/model                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         machineModel = ef_MODEL.getText() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only continue if a Machine/Model is selected            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( machineModel == null ) || 
              ( machineModel.trim().length() < 5 ) ) {
            GUISystem.printBox ( parentWin,
                                 STR_INFO,
                                 STR_MUST_ENTER_MACHINE_MODEL ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Disable our GUI                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            cb_COUNTRY.setEnabled ( false ) ;
            cb_WEBSITE.setEnabled ( false ) ;
            setStartButton ( false ) ;      

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   We have started                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            started = true ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Save our values to the properties file for next time    */                                           
            /*                                                           */
            /*-----------------------------------------------------------*/
            PropertySystem.putString ( PROPERTY_VALIDATOR_PRODUCT, machineModel ) ;
            PropertySystem.putInt    ( PROPERTY_VALIDATOR_COUNTRY, ((TypeCountryCodeRec)cb_COUNTRY.getSelectedItem()).getInd() ) ;
            PropertySystem.putString ( PROPERTY_VALIDATOR_SITE,    cb_WEBSITE.getSelectedItem().toString() ) ;
            

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the brand from the machine/model entered            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( getBrandFromMachineModel ( machineModel ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Load our properties to pass to the LogicAPI             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               parentWin.setStatus ( Str.getStr ( STR_STARTING_ENGINE ) ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create the Logic API instance                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               logicAPI = new LogicAPI ( 0,
                                         0,
                                         (String)PropertySystem.get ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator,
                                         countryProp,
                                         langProp ) ;                       
               logicAPI.setQuestNames ( "quest", "d04rdb001", "5703", "quest", "j0hnny" ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create the OA Submit Vars object                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               createOASubmitVars() ;  


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Initialize Memory Interface                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( symptomEditorPanel != null )
                  memoryInterface.initialize ( symptomEditorPanel ) ;
               else
                  memoryInterface.initialize ( symptom ) ;          


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Kick off the engine                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               processNextValidatorNode() ;

            } else
               stopSymptom() ;


         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( parentWin,
                              STR_INFO, 
                              STR_CANNOT_START_LOGIC_DAEMON ) ;
      }

      parentWin.setStatus ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* stopSymptom
    */
   /**
    *    This method will stop the current symptom
    *  being run and will free all memory associated with
    *  it
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void stopSymptom () {              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Reenable the selections                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_COUNTRY.setEnabled ( true ) ;
      cb_WEBSITE.setEnabled ( true ) ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear the GUI from the old symptom run                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      historyListModel.removeAllElements() ;
      answerListModel.removeAllElements() ;
      browser.setPageContent ( "<html><body>&nbsp;&nbsp;&nbsp;</body></html>" ) ;
      mle_SOURCE.setText ( "" ) ;
      setStartButton ( true ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our program vars                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      started      = false ;
      oaReturnVars = null ;
      brandRec     = null ;
      machineModel = null ;

   }





   /*------------------------------------------------------------*/
   /* createOASubmitVars
    */
   /**
    *    This method will create our OA Submit Var
    *  object that will be used to talk to the Logic
    *  Engine with
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createOASubmitVars () {
      oaSubmitVars           = new OASubmitVars ( 0 ) ;
      oaSubmitVars.country   = getSelectedCountryFromProperties() ;
      oaSubmitVars.qitMemory = memoryInterface ;
      oaSubmitVars.typeModel = ef_MODEL.getText() ;
      oaSubmitVars.command   = "GetNextNode" ;
      oaSubmitVars.docClass  = 3 ;
      oaSubmitVars.brand     = brandRec.toString() ;
   }





   /*------------------------------------------------------------*/
   /* getSelectedCountryFromProperties
    */
   /**
    *    This method will get what country the user
    *  has selected, and then will return the corresponding
    *  property value
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getSelectedCountryFromProperties() {
      String country = null ;

      if ( countryProp2 != null ) {
         TypeCountryCodeRec countryRec = (TypeCountryCodeRec) cb_COUNTRY.getSelectedItem() ;

         if ( countryRec != null ) {
            int         countryCodeInd    = countryRec.getInd() ;

            country = countryProp2.getProperty ( ""+countryCodeInd, "USA" ) ;

         }
      }

      System.out.println ( "PASSING COUNTRY:"+country+"\n\n\n\n\n" ) ;

      return( country ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshOutput
    */
   /**
    *    This method will refresh all of the output (History,
    *  Answers, Source and Output) from the OAReturnVars
    *  passed in
    *
    *  @param oaReturnVars The OAReturnVars to refresh with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshOutput ( OAReturnVars oaReturnVars ) {
      String addInfo = "<html>\n   <body bgcolor=#FFFFFF>\n      <h1>"+oaReturnVars.title+"</h1>\n      <br>"+oaReturnVars.addInfo+"\n   </body>\n</html>" ;

      browser.setPageContent ( addInfo ) ;
      mle_SOURCE.setText ( addInfo ) ;                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear the answers and insert the new ones               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      answerListModel.removeAllElements() ;
      if ( oaReturnVars.answerTitles != null ) {
         int i    = 0 ;
         int size = oaReturnVars.answerTitles.size() ;

         while ( i < size ) {
            answerListModel.addElement ( oaReturnVars.answerTitles.elementAt ( i ) )  ;
            i++ ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the history                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //refreshHistory() ;

   }





   /*------------------------------------------------------------*/
   /* refreshHistory
    */
   /**
    *    This method will refresh the history list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshHistory () {
      oaSubmitVars.command = "GetHistory" ;

      oaReturnVars = logicAPI.dispatch ( oaSubmitVars, symptom.getInd() ) ;
      System.out.println ( "History Count:"+oaReturnVars.answerTitles.size() ) ;
   }




   /*------------------------------------------------------------*/
   /* getBrandFromMachineModel
    */
   /**
    *    This method will set our brandRec property
    *  from the machine/model passed in. 
    *  
    *
    *  @param machineModel The Machine/Model to look for
    *  @param None
    *  @param None
    *
    *  @return TRUE = The brand was set, FALSE = The brand was not set
    *//*---------------------------------------------------------*/
   public boolean getBrandFromMachineModel ( String machineModel ) {
      boolean   rc        = false ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getBrandFromMachineModel",5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = null ;

         if ( stmt != null ) {
            String machine = machineModel.substring ( 0, machineModel.indexOf ( "-" ) ) ;
            String model   = machineModel.substring ( machineModel.indexOf ( "-" )+1 ) ;

            rs = stmt.executeQuery ( "SELECT BRANDGROUPIND "+
                                     "FROM TIGRIS.PRODUCTS "+
                                     "WHERE MACHINE = '"+machine+"' AND "+
                                     "MODEL = '"+model+"' FOR FETCH ONLY" ) ;

            if ( rs.next() ) {
               brandRec = Brands.brandRecFromBrandGroupInd ( rs.getInt(1) ) ;
               rc = true ;
            }

            rs.close() ;

         } /* if  */

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;                                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we were unable to find a brand, ask the user if      */
      /* they would like to select a brand                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !rc ) {
         if ( GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                                   STR_QUESTION,
                                   STR_CANNOT_FIND_BRAND_FROM_MACHINE_MODEL ) ) {
            Vector brandVec = new JListDlg ( GUISystem.getParentDefWin ( this ),
                                             Str.getStr ( STR_SELECT_A_BRAND ),
                                             Brands.getBrands() ).getResult() ;

            if ( ( brandVec != null ) && ( brandVec.size() > 0 ) ) {
               brandRec = (TypeBrandRec)brandVec.elementAt(0) ;
               rc = true ;
            }
         }
      }

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will respond to the user selecting
    *  items from a list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {

      if ( !e.getValueIsAdjusting() ) {

         if ( e.getSource() == lb_ANSWERS ) {
            processNextValidatorNode() ;
         }

      }

   } /* valueChanged  */





   /*------------------------------------------------------------*/
   /* processNextValidatorNode
    */
   /**
    *    This method will call the 'dispatch' method in
    *  the LogicAPI, and then will refresh the data
    *  on the screen with what is returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void processNextValidatorNode () {
      AppDefaultWin parentWin  = GUISystem.getParentDefWin ( this ) ;
      int           symptomInd = symptom.getInd() ;


      /*--------------------------------------------------------------------*/
      /*                                                                    */
      /*   Capture the new Symptom Ind. if the user has switched symptoms   */
      /*                                                                    */
      /*--------------------------------------------------------------------*/
      if ( oaReturnVars != null ) {
         if ( oaReturnVars.objectInd != 0 )
            symptomInd = oaReturnVars.objectInd ;

         oaSubmitVars.nodeInd = oaReturnVars.nodeInd ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Capture the answer if the user has selected an answer   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( answerListModel.size() > 0 ) {
         try {
            String string = (String) lb_ANSWERS.getSelectedValue() ;
            int    index  = oaReturnVars.answerTitles.indexOf ( string ) ;

            oaSubmitVars.response = ((Integer)oaReturnVars.answerInds.elementAt( index )).intValue() ;

         } catch ( Exception e ) {
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties in OASubmitVars                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      oaSubmitVars.command = "GetNextNode" ;



      parentWin.setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
      oaReturnVars = logicAPI.dispatch ( oaSubmitVars, symptomInd ) ;
      parentWin.setStatus ( null ) ;

      refreshOutput ( oaReturnVars ) ;
   }
} /* SymptomValidatorPanel */
