package com.ibm.nzna.projects.qit.avalon.editors ;                                
/*-----------------------------------------------------------*/
/*   SymptomValidatorWrapper                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomValidatorWrapper             */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.avalon.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This wrapper basically holds the SymptomValidatorPanel and allows it to 
 * be viewed by iteself in an AppDefaultWin.
 *
 * @version 1.0, 8/31/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomValidatorWrapper                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomValidatorWrapper extends JPanel implements Runnable, QuestPanel, AppConst, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Members                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private SymptomValidatorPanel validatorPanel    = null ;
   private ActionButton          pb_CLOSE          = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Symptom               symptom           = null ;
   private QuestPanel            previousPanel     = null ;
   private AvalonMemoryInterface memoryInterface   = null ;
   private int                   symptomInd        = -1 ;
   private boolean               draft             = false ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will simply assign our variables
    *  only
    *  
    *
    *  @param symptom The Symptom to validate
    *  @param previousPanel The panel that should be shown when this panel closes
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomValidatorWrapper ( int symptomInd, boolean draft, QuestPanel previousPanel ) {
      this.previousPanel = previousPanel ;
      this.symptomInd    = symptomInd ;
      this.draft         = draft ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if initializing went Ok
    *//*---------------------------------------------------------*/
   public boolean initialize () {           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize some Vars                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      memoryInterface = new AvalonMemoryInterface () ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      validatorPanel = new SymptomValidatorPanel ( memoryInterface, null ) ;
      pb_CLOSE = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE ) ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener ( this ) ;         


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the components                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( validatorPanel, BorderLayout.CENTER ) ;


      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      parentWin.addActionComponent ( this, pb_CLOSE ) ;

      new Thread ( this ).start() ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called once an action
    *  event is fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the user
    *  wants us to stop. We will free resources and
    *  return TRUE
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      removeAll() ;
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the user
    *  wants to shutdown the Quest Input Tool. We
    *  will return TRUE
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when our panel
    *  is selected from the AppDefWindow it belongs to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }


   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when our panel
    *  is deselected from the AppDefWindow it belongs to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }





   /*------------------------------------------------------------*/
   /* copy
    */
   /**
    *    This method returns FALSE and does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* cut
    */
   /**
    *    This method returns FALSE and does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* paste
    */
   /**
    *    This method returns FALSE and does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }






   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return out name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName( ) {
      return( Str.getStr ( STR_VALIDATOR ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SAVE ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    Since this QuestPanel has no default
    *  nav panel, we will just return null
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will restart the Validator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
   }




   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will bookmark the currently
    *  validating Symptom
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean bookmark () {
      if ( !(symptom instanceof SymptomDraft) ) {
         Bookmark bookmark    = new Bookmark ( ) ;
         Vector   titles      = symptom.getSymptomTitles() ;
         Vector   bookmarkVec = new Vector ( 1 ) ;

         bookmark.setObject ( new Integer ( symptom.getInd() ), BOOKMARK_SYMPTOM ) ;

         if ( ( titles != null ) && ( titles.size() > 0 ) )
            bookmark.setDescript ( titles.elementAt(titles.size()-1).toString() ) ;
         else
            bookmark.setDescript ( ""+symptom.getInd() ) ;

         bookmarkVec.addElement ( bookmark ) ;
         BookmarkSystem.guiAddBookmarks ( bookmarkVec ) ;

      } else
         GUISystem.printBox ( STR_INFO, STR_CANNOT_BOOKMARK_DRAFTS ) ;
      return( true ) ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will reread our symptom
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      Qit.setEnabled ( false ) ;
      validatorPanel.setEnabled ( false ) ;
      parentWin.setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read the Symptom in                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( draft )
         symptom = AvalonCache.readSymptomDraft ( symptomInd ) ;
      else
         symptom = AvalonCache.readSymptom ( symptomInd ) ;

      memoryInterface.setSymptom ( symptom ) ;
      validatorPanel.setSymptom ( symptom ) ;

      Qit.setEnabled ( true ) ;
      validatorPanel.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;
   }


} /* SymptomValidatorWrapper */
