package com.ibm.nzna.projects.qit.avalon.editors ;    
/*-----------------------------------------------------------*/
/*   TitleEditDlg                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TitleEditDlg                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog holds the TitleEditPanel and allows the user to edit titles in a dialog 
 *
 *
 * @version 1.0, 5/6/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TitleEditDlg                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class TitleEditDlg extends JDialog implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ButtonPanel     buttonPanel    = null ;
   private DButton         pb_SAVE        = null ;
   private DButton         pb_CANCEL      = null ;
   private TitleEditPanel  titleEditPanel = null ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean              rc       = false ;
   private TitleEditDlgListener listener = null ;
   private Vector               titles   = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our
    *  window. It will also show it.
    *
    *  @param parent The parent window
    *  @param titles The vector of titles
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TitleEditDlg ( AppDefaultWin parent, Vector titles, int brandInd ) {
      super ( parent, Str.getStr ( STR_TITLES ), false ) ;
      Container contentPane = getContentPane() ;          

      this.titles = titles ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel    = new ButtonPanel () ;
      pb_SAVE        = new DButton ( Str.getStr ( STR_SAVE ) ) ;
      pb_CANCEL      = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      titleEditPanel = new TitleEditPanel ( brandInd, titles ) ;            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SAVE.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_SAVE ) ;
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      titleEditPanel.setBorder ( GUISystem.loweredBorder ) ;  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( titleEditPanel ) ;
      contentPane.add ( buttonPanel ) ;
      buttonPanel.add ( pb_SAVE ) ;
      buttonPanel.add ( pb_CANCEL ) ;


      setSize ( 500, 440 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* setTitleEditDlgListener
    */
   /**
    *    This method will set the 1 listener that
    *  will be notified when this window closes
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitleEditDlgListener ( TitleEditDlgListener listener ) {
      this.listener = listener ;
   }


   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;
      int       x    = 5 ;
      int       y    = 5 ;

      super.doLayout() ;
      titleEditPanel.setBounds ( x, y, size.width-(x*3), size.height-58 ) ;
      buttonPanel.setBounds ( x, size.height-50, size.width-(x*3), 25 ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an
    *  action event is fired
    *  
    *
    *  @param e
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      rc = ( e.getSource() == pb_SAVE ) ;

      if ( rc ) {
         titleEditPanel.saveData() ;
      }

      if ( listener != null )
         listener.titleEditDlgClosed ( rc ? titles : null ) ;

      dispose() ;
   }





   /*------------------------------------------------------------*/
   /* setJoinCountrySelectPanel
    */
   /**
    *    This method will set the CountrySelectPanel
    *  that this dialog's CountrySelectPanel will
    *  be joined to
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setJoinCountrySelectPanel ( CountrySelectPanel countrySelectPanel ) {
      titleEditPanel.getCountrySelectPanel().joinCountrySelectPanel ( countrySelectPanel ) ;
   }

} /* TitleEditDlg */
