package com.ibm.nzna.projects.qit.avalon.editors ;            
/*-----------------------------------------------------------*/
/*   TitleEditPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TitleEditPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import javax.swing.event.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.gui.* ;     
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.shared.gui.* ;     
import com.ibm.nzna.shared.util.* ;     
import com.ibm.nzna.shared.pom.* ;     
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.spell.* ;
/**
 * This panel allows a user to edit a vector of titles. 
 *
 *
 * @version 1.0, 4/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TitleEditPanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class TitleEditPanel extends EditPanel implements SpellPanelListener, Runnable, ListSelectionListener, AppConst, AvalonConst, ActionListener, KeyListener {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Dimension          prefSize           = new Dimension ( 400, 400 ) ;
   private JList              lb_TITLES          = null ;
   private JTextField         ef_TITLE           = null ;
   private JScrollPane        scr_TITLES         = null ;
   private JTitle             st_TITLES          = null ;
   private DocClassPanel      docClassPanel      = null ;
   private JTitle             st_TITLE_TEXT      = null ;
   private HotLinkLabel       pb_ADD_TITLE       = null ;
   private HotLinkLabel       pb_REMOVE_TITLE    = null ;
   private ButtonPanel        buttonPanel        = null ;
   private HotLinkLabel       pb_TITLE_UP        = null ;
   private HotLinkLabel       pb_TITLE_DOWN      = null ;
   private CountrySelectPanel countrySelectPanel = null ;
   private HotLinkLabel       pb_SET_DEFAULT     = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DefaultListModel   listModel         = null ;
   private Vector             titles            = null ;
   private Title              defaultTitle      = null ;
   private Title              currentTitle      = null ;
   private ConditionList      conditionList     = null ;  // Holds all of the conditions
   private ButtonPanel        titleButtonPanel  = null ;  // Holds the 'Up/Down' buttons under the title list
   private Title              spellCheckTitle   = null ;  // The title we are current spell checking
   private JTabbedPane        tabbedPane        = null ;  // The Tabbed Pane we belong to, if we belong to one
   private int                spellIndex        = 0 ;     // The index in the titles vector of what we are spell checking
   private SpellPanel         spellPanel        = null ;  // The spell checker panel that we are communicating with
   private int                brandInd          = 0 ;     // The brand of the object we are editing titles for




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a title panel
    *  initialized to the vector passed in
    *  
    *
    *  @param titles The vector of Title objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TitleEditPanel ( int brandInd, Vector titles ) {

      this.brandInd = brandInd ;
      this.titles   = titles ;                                      

      initialize ( ) ;
   }

   public TitleEditPanel ( int brandInd ) {
      this.brandInd = brandInd ;
      initialize (  ) ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel. It will
    *  add all of the GUI and refresh the data
    *  
    *
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize ( ) {                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI Components                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_TITLE           = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_OA_TITLE ), "", 0 ) ;
      st_TITLES          = new JTitle ( Str.getStr ( STR_TITLES ) ) ;
      st_TITLE_TEXT      = new JTitle ( Str.getStr ( STR_TITLE_TEXT ) ) ;
      pb_ADD_TITLE       = new HotLinkLabel ( Str.getStr ( STR_ADD_TITLE ) ) ;
      pb_REMOVE_TITLE    = new HotLinkLabel ( Str.getStr ( STR_REMOVE_TITLE ) ) ;
      buttonPanel        = new ButtonPanel () ;
      lb_TITLES          = new JList() ;
      pb_TITLE_UP        = new HotLinkLabel ( Str.getStr ( STR_UP ) ) ;
      pb_TITLE_DOWN      = new HotLinkLabel ( Str.getStr ( STR_DOWN ) ) ;
      scr_TITLES         = new JScrollPane ( lb_TITLES ) ;
      docClassPanel      = new DocClassPanel () ;
      conditionList      = new ConditionList() ;
      titleButtonPanel   = new ButtonPanel () ;
      countrySelectPanel = new CountrySelectPanel () ;
      pb_SET_DEFAULT     = new HotLinkLabel ( Str.getStr ( STR_SET_DEFAULT ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      lb_TITLES.setModel ( listModel = new DefaultListModel() ) ;
      st_TITLE_TEXT.setFont  ( FontSystem.smallTitleFont ) ;
      st_TITLES.setFont      ( FontSystem.smallTitleFont ) ;
      GUISystem.setPreferredButton ( pb_ADD_TITLE ) ;
      GUISystem.setPreferredButton ( pb_REMOVE_TITLE ) ;
      scr_TITLES.setBorder ( GUISystem.blackBorder ) ;
      buttonPanel.setBorder     ( GUISystem.grayBorder ) ;
      buttonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      titleButtonPanel.setBorder     ( GUISystem.grayBorder ) ;
      titleButtonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      titleButtonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                       ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      lb_TITLES.setSelectionBackground ( new Color ( 209, 210, 253 ) ) ;
      lb_TITLES.setSelectionForeground ( Color.black ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD_TITLE.addActionListener     ( this ) ;
      pb_REMOVE_TITLE.addActionListener  ( this ) ;
      lb_TITLES.addListSelectionListener ( this ) ;
      ef_TITLE.addKeyListener            ( this ) ;
      pb_TITLE_UP.addActionListener      ( this ) ;
      pb_TITLE_DOWN.addActionListener    ( this ) ;
      pb_SET_DEFAULT.addActionListener   ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_TITLES ) ;
      add ( scr_TITLES ) ;
      titleButtonPanel.add ( pb_TITLE_UP ) ;
      titleButtonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      titleButtonPanel.add ( pb_TITLE_DOWN ) ;
      add ( titleButtonPanel ) ;
      add ( st_TITLE_TEXT ) ;
      add ( ef_TITLE ) ;
      add ( docClassPanel ) ;
      add ( buttonPanel ) ;
      add ( conditionList ) ;
      add ( countrySelectPanel ) ;

      /* Only allow BPMs to set the default title  */
      setBrandInd ( brandInd ) ;

      buttonPanel.add ( pb_SET_DEFAULT ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_ADD_TITLE ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_REMOVE_TITLE ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( titles != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the default title in the large title                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         refreshTitles() ;
         new Thread ( this ).start() ;
      }

   } /* initialize  */





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 15 ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       halfWidth = size.width/2 ;
      int       y         = 5 ;

      buttonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      st_TITLES.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      scr_TITLES.setBounds ( x, y, size.width-(x*2), rowHeight*5 ) ;
      y+=(rowHeight*5)+5 ;

      titleButtonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+10 ;

      st_TITLE_TEXT.setBounds ( x, y, size.width-(x*2)-170, rowHeight ) ;
      docClassPanel.setBounds ( size.width-(x*2)-150, y, 150, rowHeight*2 ) ;
      y+=rowHeight ;

      ef_TITLE.setBounds ( x, y, size.width-(x*2)-170, rowHeight ) ;
      y+=rowHeight ;

      conditionList.setBounds ( x, y, halfWidth-x-5, size.height-y-5 ) ;
      countrySelectPanel.setBounds ( halfWidth, y, halfWidth-x-5, size.height-y-5 ) ;

   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of the data on our
    *  panel
    *  
    *
    *  @param title The Title to refresh with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData ( Title title ) {                                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the title is non-null, refresh the data with it      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( title != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our title text and document classification          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         ef_TITLE.setText ( title.toString(Title.TEXT) ) ;         
         docClassPanel.setDocClass ( title.getDocClass() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the conditions and Countries                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         conditionList.setConditions ( title.getConditions() ) ;
         countrySelectPanel.setCountryList ( title.getCountryList () ) ;

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, clear the fields                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         ef_TITLE.setText ( "" ) ;
         conditionList.setConditions ( null ) ;
         countrySelectPanel.setCountryList ( null ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, make sure our defaultTitle property is pointing    */                                           
      /* to the correct title                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getDefaultTitle() ;

   }

   /*------------------------------------------------------------*/
   /* getDefaultTitle
    */
   /**
    *    This method will return the default title
    *  for the symptom
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected Title getDefaultTitle () {

      if ( titles != null )
         defaultTitle = (Title) titles.elementAt(titles.size()-1 ) ;
      else
         defaultTitle = null ;

      if ( defaultTitle != null )
         defaultTitle.setToStringType ( Title.HTML_BOLD ) ;

      return( defaultTitle ) ;
   }




   /*------------------------------------------------------------*/
   /* addTitle
    */
   /**
    *    This method will pop up a window asking
    *  the user for a title. It will then add the title
    *  to the list and return the title object.
    *
    *  @param parent The parent window for the user pop up
    *  @param None
    *  @param None
    *
    *  @return Title
    *//*---------------------------------------------------------*/
   private Title addTitle ( AppDefaultWin parent ) {
      Title  title        = null ;
      String newTitle     = null ;                                


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save the current data                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      saveData();


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, ask the user for the new title                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      newTitle = new SingleEntryDlg ( parent, 
                                      ( titles == null ) ? Str.getStr ( STR_DEFAULT_TITLE ) : Str.getStr ( STR_TITLE ),
                                      ( titles == null ) ? Str.getStr ( STR_ENTER_DEFAULT_TITLE ) : Str.getStr ( STR_TITLE ),
                                      "",
                                      new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_OA_TITLE ) ).getResult() ;                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user supplied a title, add it                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( newTitle != null ) && ( newTitle.length() > 0 ) ) {
         try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create our title object                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            title = new Title ( newTitle ) ;            

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Assign it's variables                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            title.setDocClass ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL,
                                                                                         TypeList.TYPE_DOCCLASS ) ) ;
            title.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                         TypeList.TYPE_LANGUAGE ) ) ;
            title.setCountryList ( UserSystem.getUserCountries ( UserSystem.getUserId() ) ) ;                 

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add this title to our list                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( titles == null )
               titles = new Vector ( 1, 1 ) ;

            titles.insertElementAt ( title, 0 ) ;    // Adds it to our internal Vector
            listModel.insertElementAt ( title, 0 ) ; // Adds it to the viewable list

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Refresh our data to this new title                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            setCurrentTitle ( title ) ;
            lb_TITLES.setSelectedValue ( title, true ) ;

         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }
      return( title ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  is performed and we need to know about it
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_TITLE                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_ADD_TITLE )
         addTitle ( GUISystem.getParentDefWin ( this ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_TITLE                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_TITLE ) {

         if ( currentTitle != null ) {
            currentTitle.updateRecStatus ( QITRec.REC_DELETE ) ;
            currentTitle = null ;
            refreshTitles() ;
            new Thread ( this ).start() ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_TITLE_UP                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_TITLE_UP ) {
         if ( currentTitle != null )
            moveTitleUp () ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_TITLE_DOWN                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_TITLE_DOWN ) {
         if ( currentTitle != null )
            moveTitleDown() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SET_DEFAULT                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SET_DEFAULT ) {
         if ( currentTitle != null ) {
            getDefaultTitle() ;
            defaultTitle.setToStringType ( Title.HTML ) ;
            titles.removeElement ( currentTitle ) ;
            titles.addElement ( currentTitle ) ;
            refreshTitles() ;
         }
      }


   } /* actionPerformed  */




   /*------------------------------------------------------------*/
   /* refreshTitles
    */
   /**
    *    This method will refresh all of our titles
    *  in the title drop down
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshTitles () {
      Vector validTitles = QuestUtil.getValidData ( titles ) ;                                     

      listModel.removeAllElements() ;

      if ( validTitles != null ) {

         getDefaultTitle() ;

         if ( validTitles != null ) {
            int i    = 0 ;
            int size = validTitles.size() ;

            while ( i < size ) {

               if ( validTitles.elementAt(i) != defaultTitle )
                  ((Title)validTitles.elementAt(i)).setToStringType ( Title.TEXT ) ;

               listModel.add ( listModel.getSize(), validTitles.elementAt(i)   ) ;
               i++ ;
            }
         }
      }

      ef_TITLE.setText ( "" ) ;
      revalidate() ;
   }


   /*------------------------------------------------------------*/
   /* setTitles
    */
   /**
    *    This method will set the vector of titles that
    *  we are using
    *  
    *
    *  @param titles A vector of titles
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitles ( Vector titles ) {
      if ( ( titles != null ) && ( titles.size() > 0 ) ) {
         this.titles = titles ;
         refreshTitles() ;
      }
   }




   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    this method will be called when the user has
    *  selected another title
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( !e.getValueIsAdjusting() ) {
         setCurrentTitle ( (Title)lb_TITLES.getSelectedValue() ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* keyTyped
    */
   /**
    *    We have subclassed this method so that
    *  we will automagically update the title in both
    *  the entry field and in the Title object
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void keyTyped ( KeyEvent e ) {
      saveTitle() ;
      fireChangedEvent() ;    
   }
   public void keyPressed ( KeyEvent e ) {
      saveTitle() ;
      fireChangedEvent() ;    
   }
   public void keyReleased ( KeyEvent e ) {
      saveTitle() ;
      fireChangedEvent() ;    
   }




   /*------------------------------------------------------------*/
   /* saveTitle
    */
   /**
    *    This method will update the title in the
    *  Title object with what is entered on the command
    *  line
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveTitle () {
      if ( currentTitle != null ) {

         currentTitle.setTitle ( ef_TITLE.getText() ) ;
         lb_TITLES.repaint() ;
      }
   }


   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save the data on the
    *  screen to the title
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected boolean saveData () {

      if ( currentTitle != null ) {
         currentTitle.setDocClass ( docClassPanel.getDocClass() ) ;

         conditionList.saveConditions() ;
         currentTitle.setCountryList ( countrySelectPanel.getCountryList() ) ;
      }

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread currently only refreshes our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {                                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Fire a change event to notify our parent panel of       */
      /* a possible title change                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      fireChangedEvent() ;            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, refresh our data                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData ( currentTitle ) ;
   }


   /*------------------------------------------------------------*/
   /* getPreferredSize()
    */
   /**
    *    This method will return the preferred 
    *  size of this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize() {
      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return our minimum size. Actually
    *  this method returns whatever getPreferredSize
    *  returns
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }




   /*------------------------------------------------------------*/
   /* getTitleVec
    */
   /**
    *    This method will return the tile
    *  vector that is being displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected Vector getTitleVec () {
      return( titles ) ;
   }




   /*------------------------------------------------------------*/
   /* moveTitleUp
    */
   /**
    *    This method will move a title in the
    *  title list up a level in the sort order
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void moveTitleUp ( ) {
      moveTitleInList ( false ) ;
   }




   /*------------------------------------------------------------*/
   /* moveTitleDown
    */
   /**
    *    This method will move a title down
    *  in the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void moveTitleDown ( ) {
      moveTitleInList ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* moveTitleFromList
    */
   /**
    *    This method will simply move the title
    *  in the container. This will not have an effect
    *  of the title object yet
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The position it was at before it was moved
    *//*---------------------------------------------------------*/
   public void moveTitleInList ( boolean down ) {
      int    rc     = titles.indexOf ( currentTitle ) ;                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Take the default title out for now                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      defaultTitle = getDefaultTitle() ;

      if ( currentTitle != defaultTitle ) {
         if ( down )
            rc++ ;
         else
            rc-- ;

         if ( ( rc >= 0 ) && ( rc < (titles.size()-1) ) ) {
            int i      = 0 ;
            int length = titles.size() ;

            titles.removeElement ( currentTitle ) ;
            titles.insertElementAt ( currentTitle, rc ) ;

            listModel.removeAllElements() ;

            while ( i < length ) {
               if ( ((PersistentRec)titles.elementAt(i)).valid() ) 
                  listModel.add ( listModel.getSize(), titles.elementAt(i) ) ;
               i++ ;
            } /* while */

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, put the default back in                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            lb_TITLES.setSelectedValue ( currentTitle, true ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* removeCountryFromPanel
    */
   /**
    *    This method will not only remove the country
    *  passed in from the panel, but it will also remove
    *  the country from all titles in this panel
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void removeCountryFromPanel ( TypeCountryCodeRec countryRec ) {                                


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remove the country from all of our titles               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( titles != null ) && ( titles.size() > 0 ) ) {
         int    i              = 0 ;
         int    size           = titles.size() ;
         Vector titleCountries = null ;

         while ( i < size ) {
            titleCountries = ((Title)titles.elementAt(i)).getCountryList() ;
            titleCountries.removeElement ( countryRec ) ;
            ((Title)titles.elementAt(i)).setCountryList ( titleCountries ) ;
            i++ ;
         } /* while  */
      } /* if  */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, refresh our display                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( currentTitle != null )
         countrySelectPanel.setCountryList ( currentTitle.getCountryList() ) ;


   } /* removeCountryFromPanel  */





   /*------------------------------------------------------------*/
   /* getCountrySelectPanel
    */
   /**
    *    This method will return an instance of this
    *  panel's CountrySelectPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected CountrySelectPanel getCountrySelectPanel () {
      return( countrySelectPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* spellCheck
    */
   /**
    *    This method will spell check all titles on this
    *  panel
    *  
    *
    *  @param tabbedPane This can be null. If it is not null, and there
    *                    are misspelled words, this method will turn the
    *                    tabbed pane's page to this panel.
    *  @param None
    *  @param None
    *
    *  @return TRUE if there are no misspelled words
    *//*---------------------------------------------------------*/
   protected boolean spellCheck ( JTabbedPane tabbedPane, SpellPanel spellPanel ) {
      boolean         mispelled    = false ;

      if ( titles.size() > 0 ) {

         this.tabbedPane = tabbedPane ;
         this.spellPanel = spellPanel ;

         /*-------------------------------------------------------------*/
         /*                                                             */
         /*   Add ourselves as a listener so we get notified of status  */
         /*                                                             */
         /*-------------------------------------------------------------*/
         spellPanel.addSpellPanelListener ( this ) ;

         spellIndex = 0 ;
         spellCheck ( (Title)titles.elementAt(spellIndex) ) ;
      } else
         fireSpellCheckEvent ( false ) ;


      return( mispelled ) ;
   }





   /*------------------------------------------------------------*/
   /* spellCheck
    */
   /**
    *    This method will spell check an entire Title
    *  
    *  
    *
    *  @param title The title object to spell check
    *  @param spellToolkit The already created JSpellToolkit
    *  @param None
    *
    *  @return TRUE if no misspelled words
    *//*---------------------------------------------------------*/
   private boolean spellCheck ( Title title ) {
      boolean         rc           = false ;

      spellCheckTitle = title ;
      spellPanel.spellCheck ( spellCheckTitle.toString( Title.TEXT ) ) ;
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* spellCheckComplete
    */
   /**
    *    This method will be called once the SpellPanel
    *  is done checking a title.
    *  
    *
    *  @param errorsFound TRUE if any errors were encountered
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void spellCheckComplete ( boolean errorsFound ) {             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Update the spell Index                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      spellIndex++ ;                                               

      /*------------------------------------------------------------*/
      /*                                                            */
      /*   If we have no more titles to check, notify the listeners */
      /*                                                            */
      /*------------------------------------------------------------*/
      if ( spellIndex >= (titles.size()-1) ) { // subtract 1 from .size() so we don't check the default
         spellPanel.removeSpellPanelListener ( this ) ;
         fireSpellCheckEvent ( errorsFound ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, start spell checking the next title          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         spellCheck ( (Title)titles.elementAt ( spellIndex ) ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* spellCheckWord
    */
   /**
    *    This method will be called when the SpellPanel
    *  has encountered an error
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void spellCheckWord ( String word, int startPos, int endPos ) {

      tabbedPane.setSelectedComponent ( WinUtil.getParentJScrollPane ( this ) ) ;

      saveData() ;
      currentTitle = spellCheckTitle ;
      refreshData ( currentTitle ) ;
      lb_TITLES.setSelectedValue ( currentTitle, true ) ;
      try {
         ef_TITLE.setSelectionStart ( startPos ) ;
         ef_TITLE.setSelectionEnd   ( endPos ) ;
      } catch ( Exception e ) {
      }
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* replaceWord
    */
   /**
    *    This method will replace the word that is passed
    *  in (oldWord) with the newWord passed in. This means
    *  that the user has corrected the type-o
    *
    *  @param oldWord What the word used to be
    *  @param newWord What the new word is
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void replaceWord ( String oldWord, String newWord, int startIndex ) {
      int    startPos = ef_TITLE.getSelectionStart() ;
      String newTitle = spellCheckTitle.toString ( Title.TEXT ) ;

      newTitle = newTitle.substring ( 0, startIndex )+newWord+newTitle.substring ( startIndex+oldWord.length() ) ;

      spellCheckTitle.setTitle ( newTitle ) ;

      refreshData ( spellCheckTitle ) ;

      revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* setCurrentTitle
    */
   /**
    *    This method will set the current title on the
    *  editor. If there already is a title selected, that
    *  title's data will be saved first.
    *
    *  @param title The new current title
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setCurrentTitle ( Title title ) {

      if ( currentTitle != null )
         saveData() ;

      currentTitle = title ;

      getDefaultTitle() ;
      if ( title == defaultTitle ) {
         ef_TITLE.setEnabled ( UserSystem.getLoggedUserRec().isBPM ( brandInd ) ) ;
         docClassPanel.setEnabled ( false ) ;
         pb_TITLE_UP.setEnabled ( false ) ;
         pb_TITLE_DOWN.setEnabled ( false ) ;
         countrySelectPanel.setEnabled ( false ) ;
         conditionList.setEnabled ( false ) ;
         pb_SET_DEFAULT.setEnabled ( false ) ;
         pb_REMOVE_TITLE.setEnabled ( false ) ;
      } else {
         docClassPanel.setEnabled ( true ) ;
         pb_TITLE_UP.setEnabled ( true ) ;
         pb_TITLE_DOWN.setEnabled ( true ) ;
         countrySelectPanel.setEnabled ( true ) ;
         conditionList.setEnabled ( true ) ;
         pb_REMOVE_TITLE.setEnabled ( true ) ;
         pb_SET_DEFAULT.setEnabled ( UserSystem.getLoggedUserRec().isBPM ( brandInd ) ) ;
      }

      new Thread ( this ).start() ;
   }




   /*------------------------------------------------------------*/
   /* setBrandInd
    */
   /**
    *    This method will set the brand indicator of the
    *  object who's titles we are editing. This will
    *  effect the way the 'setDefault' button works. If the user
    *  does not have BPM of the brand passed in then they will not
    *  be able to set the default title
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrandInd ( int brandInd ) {
      boolean enable = UserSystem.getLoggedUserRec().isBPM ( brandInd ) ;

      this.brandInd = brandInd ;
      pb_SET_DEFAULT.setEnabled ( enable ) ;
   }

} /* TitleEditPanel */
