package com.ibm.nzna.projects.qit.avalon.editors ;     
/*-----------------------------------------------------------*/
/*   VisioPanel                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: VisioPanel                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.pom.* ;
/**
 * The VisioPanel is used in the SymptomEditPanel. It allows 
 * the 'VISIO' style edit of a Symptom.
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   VisioPanel                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class VisioPanel extends Container implements MouseMotionListener, MouseListener, PropertyListener, AvalonConst, Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_FIRE_NODE_FOCUS_RECEIVED = "FIREFOCUS" ;             


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector        listeners          = null ;
   private Vector        visibleEdges       = new Vector ( 1, 1 ) ;
   private Vector        visibleObjectNodes = new Vector ( 1, 1 ) ;
   private Vector        focusEdgeGroups    = new Vector ( 1, 1 ) ;
   private Vector        edgeReturns        = new Vector ( 1, 1 ) ;
   private Vector        objectNodes        = null ;
   private Dimension     prefSize           = new Dimension ( 9999, 9999 ) ;
   private int           nodeWidth          = 100 ;
   private int           nodeHeight         = 100 ;
   private int           gridSize           = 10 ;
   private boolean       paintGrid          = false ;
   private boolean       paintEdgeTitle     = false ;
   private ObjectNode    focusNode          = null ;
   private Symptom       symptom            = null ;
   private ComponentNode componentNode      = null ;
   private JViewport     parentViewport     = null ;
   private Point         newScrollPt        = new Point ( 0, 0 ) ;                                          
   private int[]         edgeTitleY         = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Here are the properties that adjust the way we look     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int           lengthEdgeText     = 50 ;
   private Color         gridColor          = null ;
   private Color         edgeFocusColor     = null ;
   private Color         edgeColor          = null ;
   private Color         edgeReturnColor    = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create an empty flow
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public VisioPanel () {
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize all of our variables
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our local vars                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      objectNodes    = new Vector ( 1, 1 ) ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add ourselves as a property listener                    */
      /* Remember that if you add any other properties to listen   */
      /* for here to also remove that property in the free()       */
      /* method!                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PROPERTY_GRID_COLOR,            this ) ;
      PropertySystem.addPropertyListener ( PROPERTY_VISIO_TEXT_LENGTH,     this ) ;
      PropertySystem.addPropertyListener ( PROPERTY_EDGE_FOCUS_COLOR,      this ) ;
      PropertySystem.addPropertyListener ( PROPERTY_EDGE_RETURN_COLOR,     this ) ;
      PropertySystem.addPropertyListener ( PROPERTY_EDGE_COLOR,            this ) ;
      PropertySystem.addPropertyListener ( PROPERTY_FLOW_FILTER,           this ) ;
      PropertySystem.addPropertyListener ( PROPERTY_FLOW_BACKGROUND_COLOR, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust our properties                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      setBackground ( (Color)PropertySystem.get ( PROPERTY_FLOW_BACKGROUND_COLOR ) ) ;        
      gridColor       = (Color)PropertySystem.get ( PROPERTY_GRID_COLOR ) ;
      edgeFocusColor  = (Color)PropertySystem.get ( PROPERTY_EDGE_FOCUS_COLOR ) ;
      edgeColor       = (Color)PropertySystem.get ( PROPERTY_EDGE_COLOR ) ;
      edgeReturnColor = (Color)PropertySystem.get ( PROPERTY_EDGE_RETURN_COLOR ) ;
      lengthEdgeText  = PropertySystem.getInt ( PROPERTY_VISIO_TEXT_LENGTH ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our listeners                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addMouseListener ( this ) ;
      addMouseMotionListener ( this ) ;

   }




   /*------------------------------------------------------------*/
   /* addNode
    */
   /**
    *    This method will add a Node to our flow
    *  
    *  
    *
    *  @param Node The Node to add
    *  @param None
    *  @param None
    *
    *  @return ObjectNode The ObjectNode was created to represent the Node
    *//*---------------------------------------------------------*/
   public ObjectNode addNode ( Node node ) {
      ObjectNode objectNode = new ObjectNode ( symptom, node ) ;

      addObjectNode ( objectNode ) ;
      resetFilter() ;

      return( objectNode ) ;
   }



   /*------------------------------------------------------------*/
   /* addNodes
    */
   /**
    *    This method will add a list Nodes to our flow
    *  
    *
    *  @param object The Vector of Nodes to add
    *  @param None
    *  @param None
    *
    *  @return Vector A Vector containing all of the ObjectNodes that represent each Node passed in
    *//*---------------------------------------------------------*/
   public Vector addNodes ( Vector vector ) {
      Vector retVector = new Vector ( 1 ) ;

      if ( ( vector != null ) && ( vector.size() > 0 ) ) {
         int i    = 0 ;
         int size = vector.size() ;

         while ( i < size ) {
            ObjectNode objectNode = new ObjectNode ( symptom, (Node)vector.elementAt(i) ) ;

            addObjectNode ( objectNode ) ;
            retVector.addElement ( objectNode ) ;
            i++ ;
         }
      }
      resetFilter() ;

      return( retVector ) ;
   }




   /*------------------------------------------------------------*/
   /* addObjectNode
    */
   /**
    *    This method will add an object node to
    *  our panel and initalize it's values
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addObjectNode ( ObjectNode objectNode ) {
      try {
         FlowEditorFilter filter = (FlowEditorFilter) PropertySystem.get ( PROPERTY_FLOW_FILTER ) ;
         Rectangle        bounds = objectNode.getNodeBounds() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust the size                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( bounds.width == 0 )
            bounds.width=100 ;
         if ( bounds.height == 0 )
            bounds.height=100 ;

         objectNode.setSize ( bounds.width, bounds.height ) ;                                      


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this ObjectNode is a root, set that property         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( symptom != null ) && 
              ( symptom.getRootNode() == objectNode.getNode() ) )
            objectNode.setRoot ( true ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add this Object Node to our vector of ObjectNodes       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         objectNodes.addElement ( objectNode ) ;                                           


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, set the visible property from the filter and add    */
         /* this objectNode if it is still visible (not filtered out) */
         /*                                                           */
         /*-----------------------------------------------------------*/
         objectNode.setVisible ( ( ( filter.showCountry ( objectNode.getCountryList() ) ) &&
                                   ( filter.showDocClass ( objectNode.getDocClass() ) ) ) ) ;
         if ( objectNode.getVisible() )
            visibleObjectNodes.addElement ( objectNode ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* addObjectNodes
    */
   /**
    *    This method will add a vector of object nodes
    *  to our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addObjectNodes ( Vector objectNodeVec ) {
      if ( ( objectNodeVec != null ) && ( objectNodeVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = objectNodeVec.size() ;

         while ( i < size ) {
            addObjectNode ( (ObjectNode)objectNodeVec.elementAt(i) ) ;
            i++ ;
         }
      }
   }


   /*------------------------------------------------------------*/
   /* setSymptom
    */
   /**
    *    This method will set the symptom that we
    *  are editing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setSymptom ( Symptom symptom ) {           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our variables                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.symptom = symptom ;                                                                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in all of the data for the nodes                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addObjectNodes ( VisioPanelReader.readNodeData ( symptom ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, force us to take a look at what we just set         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      resetFilter() ; 
   }



   /*------------------------------------------------------------*/
   /* addEdgeGroup
    */
   /**
    *    This method will add a new edge group to the
    *  flow
    *  
    *
    *  @param edgeGroup The Edge Group to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void addEdgeGroup ( EdgeGroup edgeGroup ) {
      ObjectNode tailNode = tailObjectNodeFromEdgeGroup ( edgeGroup ) ;
      ObjectNode headNode = headObjectNodeFromEdgeGroup ( edgeGroup ) ;

      if ( ( tailNode != null ) && ( headNode != null ) &&
           ( tailNode.getVisible() ) && ( headNode.getVisible() ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the group                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            if ( !visibleEdges.contains ( edgeGroup ) )
               visibleEdges.addElement ( edgeGroup ) ;
         } catch ( Exception e ) {
            visibleEdges.addElement ( edgeGroup ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, this group might have a Edge Return, we must       */
         /* refresh our returns now.                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         refreshEdgeReturns() ;

      } else
         GUISystem.printBox ( STR_INFO, STR_EDGE_JUST_ADDED_FILTERED_OUT ) ;
   }


   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our preferred size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }

   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return our preferred size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( prefSize ) ;
   }


   /*------------------------------------------------------------*/
   /* componentShown
    */
   /**
    *    This method will be called when a component
    *  is shown. We will do nothing!
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void componentShown ( ComponentEvent e ) {
   }

   /*------------------------------------------------------------*/
   /* componentHidden
    */
   /**
    *    This method will be called when a component
    *  is shown. We will do nothing!
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void componentHidden ( ComponentEvent e ) {
   }




   public void update ( Graphics g ) {
   }

   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paint ( Graphics g ) {
      Dimension size = getSize() ;

      if ( paintGrid ) {
         g.setColor ( gridColor ) ;

         for ( int x = gridSize; x < size.width; x+=gridSize )
            g.drawLine ( x, 0, x, size.height ) ;

         for ( int y = gridSize; y < size.height; y+=gridSize )
            g.drawLine ( 0, y, size.width, y ) ;

      }
      paintEdges ( g ) ;
      paintNodes ( g ) ;
      super.paint ( g ) ;
   }


   /*------------------------------------------------------------*/
   /* getPaintGrid
    */
   /**
    *    This method will return the current state
    *  of the paintGrid property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean getPaintGrid () {
      return( paintGrid ) ;
   }





   /*------------------------------------------------------------*/
   /* paintNodes
    */
   /**
    *    This method will go through all visible nodes
    *  and paint each one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void paintNodes ( Graphics g ) {
      int i    = 0 ;

      if ( objectNodes != null ) {
         int size = visibleObjectNodes.size() ;
         Rectangle rect = (Rectangle) g.getClip() ;

         while ( i < size ) {
            ObjectNode onode = (ObjectNode)visibleObjectNodes.elementAt(i) ;

            if ( onode.isInClip ( rect ) )
               ((ObjectNode)visibleObjectNodes.elementAt(i)).paint ( g ) ;
            i++ ;
         }
      }
   }



   /*------------------------------------------------------------*/
   /* setPaintGrid
    */
   /**
    *    This method will set the current paintGrid
    *  property. If TRUE, a grid will be painted in the
    *  background of the flow.
    *
    *  @param paintGrid
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPaintGrid ( boolean paintGrid ) {
      this.paintGrid = paintGrid ;
      repaint() ;
   }





   /*------------------------------------------------------------*/
   /* paintEdges
    */
   /**
    *    This method will paint all of the edges
    *  
    *  
    *
    *  @param g The graphics context to draw in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintEdges ( Graphics g ) {                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Paint all of the normal Edges                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( visibleEdges.size() > 0 ) {
         int       i               = 0 ;
         int       edgeSize        = visibleEdges.size() ;
         EdgeGroup edgeGroup       = null ;
         ObjectNode tailObjectNode = null ;

         resetEdgeTitleY ( edgeSize ) ;

         /*------------------------------------------------------------*/
         /*                                                            */
         /*   Loop through and paint each Edge Group                   */
         /*                                                            */
         /*------------------------------------------------------------*/
         g.setColor ( edgeColor ) ;
         if ( paintEdgeTitle ) {
            while ( i < edgeSize ) {
               edgeGroup = (EdgeGroup)visibleEdges.elementAt(i) ;
               tailObjectNode = tailObjectNodeFromEdgeGroup ( edgeGroup ) ;
               tailObjectNode.setEdgeTitleY ( 0 ) ;
               i++ ;
            }
         }

         i=0 ;
         while ( i < edgeSize ) {

            edgeGroup = (EdgeGroup)visibleEdges.elementAt(i) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Only paint the edges if it is visible                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( edgeGroup.valid() )
               paintSingleEdgeGroup ( g, 
                                      edgeGroup.getHeadNode(), 
                                      edgeGroup.getTailNode(), 
                                      edgeGroup.getEdges(),
                                      edgeGroup,
                                      true ) ;
            i++ ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, paint all of the focus groups                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( focusEdgeGroups.size() > 0 ) {
         int       i         = 0 ;
         int       focusSize = focusEdgeGroups.size() ;
         EdgeGroup edgeGroup = null ;

         g.setColor ( edgeFocusColor ) ;
         while ( i < focusSize ) {
            edgeGroup = (EdgeGroup)focusEdgeGroups.elementAt(i) ;

            if ( edgeGroup.valid() )
               paintSingleEdgeGroup ( g, 
                                      edgeGroup.getHeadNode(), 
                                      edgeGroup.getTailNode(), 
                                      edgeGroup.getEdges(),
                                      edgeGroup,
                                      false ) ;
            i++ ;
         } /* while  */
      } /* if  */



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, paint all of the edge returns                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( edgeReturns.size() > 0 ) {
         int                   i          = 0 ;
         int                   edgeSize   = edgeReturns.size() ;
         EdgeReturnDisplayRec  edgeReturn = null ;

         /*------------------------------------------------------------*/
         /*                                                            */
         /*   Loop through and paint each Edge Group                   */
         /*                                                            */
         /*------------------------------------------------------------*/
         g.setColor ( edgeReturnColor ) ;
         while ( i < edgeSize ) {

            edgeReturn = (EdgeReturnDisplayRec)edgeReturns.elementAt(i) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Only paint the edges if it is visible                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            paintSingleEdgeGroup ( g, 
                                   edgeReturn.getHeadNode(), 
                                   edgeReturn.getTailNode(), 
                                   null,
                                   null,
                                   false ) ;
            i++ ;
         }
      } /* paintEdgeReturns  */


   } /* paintComponent  */





   /*------------------------------------------------------------*/
   /* resetEdgeTitleY
    */
   /**
    *    This method resets all 'Y' coordinate values
    *  for the titles to 0
    *  
    *
    *  @param size The number of edges to reset
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void resetEdgeTitleY ( int size ) {
      edgeTitleY = new int[size] ;

      for ( int i = 0; i < size; i++ )
         edgeTitleY[i]=0 ;
   }






   /*------------------------------------------------------------*/
   /* getEdgeGroupIndex
    */
   /**
    *    This method will return the index in the
    *  visibleEdges vector of the EdgeGroup passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   private int getEdgeGroupIndex ( EdgeGroup edgeGroup ) {
      return( visibleEdges.indexOf ( edgeGroup ) ) ;
   }

   /*------------------------------------------------------------*/
   /* paintSingleEdge
    */
   /**
    *    This method will paint one edge from one
    *  node to another
    *  
    *
    *  @param g The graphics context
    *  @param headNode The node to start with
    *  @param tailNode The node to stop with
    *  @param edges All edges in the EdgeGroup
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void paintSingleEdgeGroup ( Graphics g, Node headNode, Node tailNode, Vector edgeVec, EdgeGroup edgeGroup, boolean paintTitles ) {
      boolean    drawEdge      = false ;
      Point      startElbow    = new Point ( 0, 0 ) ;
      Point      startMidElbow = new Point ( 0, 0 ) ;
      Point      stopMidElbow  = new Point ( 0, 0 ) ;
      Point      stopElbow     = new Point ( 0, 0 ) ;
      int        xHalfPoint    = 0 ;
      int        yHalfPoint    = 0 ;
      String     text          = null ;
      int        i             = 0 ;
      int        edgeVecSize   = 0 ;
      Rectangle  clipRect      = (Rectangle) g.getClip() ;                                  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only paint if we have a valid head and tail             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( headNode != null ) && ( tailNode != null ) ) {
         startElbow.y = headNode.getYPos()+headNode.getHeight()+10 ;
         startElbow.x = headNode.getXPos()+(headNode.getWidth()/2) ;

         stopElbow.y  = tailNode.getYPos()-10 ;
         stopElbow.x  = tailNode.getXPos()+(tailNode.getWidth()/2) ;

         xHalfPoint = Math.abs(startElbow.x-stopElbow.x) ;
         yHalfPoint = Math.abs(startElbow.y-stopElbow.y) ;

         if ( stopElbow.x < startElbow.x )
            startMidElbow.x = startElbow.x-xHalfPoint ;
         else
            startMidElbow.x = stopElbow.x-xHalfPoint ;

         startMidElbow.y = startElbow.y ;
         stopMidElbow.y = stopElbow.y ;
         stopMidElbow.x=startMidElbow.x ;         

         if ( ( clipRect.contains ( startElbow ) ) ||
              ( clipRect.contains ( stopElbow ) ) ||
              ( clipRect.contains (startMidElbow ) ) ||
              ( clipRect.contains ( stopMidElbow ) ) )
            drawEdge = true ;
         else
            drawEdge = false ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draw the edge if we are supposed to                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( drawEdge ) {
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, draw the lines                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            g.drawLine ( startElbow.x, headNode.getYPos()+headNode.getHeight(),
                         startElbow.x, startElbow.y ) ;
            g.drawLine ( startElbow.x, startElbow.y,
                         startMidElbow.x, startMidElbow.y ) ;
            g.drawLine ( startMidElbow.x, startMidElbow.y,
                         stopMidElbow.x, stopMidElbow.y ) ;
            g.drawLine ( stopMidElbow.x, stopMidElbow.y,
                         stopElbow.x, stopElbow.y ) ;
            g.drawLine ( stopElbow.x, stopElbow.y,
                         stopElbow.x, tailNode.getYPos() ) ;                                                            


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, draw the first 50 characters of the text           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( edgeGroup != null ) && ( paintEdgeTitle ) && ( paintTitles ) ) {

               if ( ( edgeVec != null ) && ( edgeVec.size() > 0 ) ) {
                  ObjectNode tailObjectNode = tailObjectNodeFromEdgeGroup ( edgeGroup ) ;
                  int        y              = tailObjectNode.getEdgeTitleY() ;
                  int        rowHeight      = GUISystem.getRowHeight() ;

                  if ( y == 0 )
                     y = stopElbow.y ;

                  edgeVecSize = edgeVec.size() ;

                  while ( i < edgeVecSize ) {

                     if ( ((PersistentRec)edgeVec.elementAt(i)).valid() ) {
                        text = edgeVec.elementAt(i).toString() ;

                        if ( text != null ) {
                           if ( text.length() > lengthEdgeText )
                              text = text.substring ( 0, lengthEdgeText )+"..." ;

                           g.drawString ( text, stopElbow.x+3, y-10 ) ;
                           y-=rowHeight ;
                        }
                     }
                     i++ ;
                  } /* while */

                  tailObjectNode.setEdgeTitleY ( y ) ;
               } /* if edgeVec */
            } /* if edgeGroup and paintEdgeTitle  */


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Draw the arrow going into the box                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            g.drawImage ( ImageSystem.getImageIcon ( this, ImageSystem.NODE_ARROW_DOWN ).getImage(), stopElbow.x-5, tailNode.getYPos()-10, this ) ;
         }
      } /* if tail/head are not null */
   }


   /*------------------------------------------------------------*/
   /* focusGained
    */
   /**
    *    This method will be called when a node
    *  has gained focus
    *  
    *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void focusGained ( ) {
      Vector edgeGroups = AvalonUtil.getHeadEdgeGroups ( focusNode.getNode(), symptom ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add all of the edges from the new focus node to         */
      /* the focusEdgeGroups vector so that they will paint        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      focusEdgeGroups.removeAllElements() ;
      focusEdgeGroups = edgeGroups ;

      new Thread ( this, THREAD_FIRE_NODE_FOCUS_RECEIVED ).start() ;
      repaint() ;
      revalidate() ;
   }

   /*------------------------------------------------------------*/
   /* getEdgeGroupCount
    */
   /**
    *    This method will return the number of edge
    *  groups
    *  
    *
    *  @param None                       
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   protected int getEdgeGroupCount () {
      if ( visibleEdges != null )
         return( visibleEdges.size() ) ;

      return( 0 ) ;
   }




   /*------------------------------------------------------------*/
   /* getNodeCount
    */
   /**
    *    This method will return the number of nodes
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   protected int getNodeCount () {
      int count = 1 ; // Always start at one for the focus node
      int i     = 0 ;
      int size  = objectNodes.size() ;

      while ( i < size ) {
         if ( ((ObjectNode)objectNodes.elementAt(i)).getPaint() )
            count++ ;
         i++ ;
      }

      return( count ) ;
   }




   /*------------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    this method will remove all of the nodes and
    *  edges from the screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAll () {
      super.removeAll() ;
      objectNodes.removeAllElements () ;
      visibleEdges.removeAllElements() ;
      edgeReturns.removeAllElements() ;
      revalidate() ;
   }



   /*------------------------------------------------------------*/
   /* addVisioListener
    */
   /**
    *    This method will add a VisioLIstener to
    *  be notified of events
    *  
    *
    *  @param listener The listener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addVisioListener ( VisioListener listener ) {
      if ( listeners == null )
         listeners = new Vector ( 1 ) ;

      listeners.addElement ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* removeVisioListener
    */
   /**
    *    This method will remove a VisioListener
    *  from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeVisioListener ( VisioListener listener ) {
      if ( listeners != null )
         listeners.removeElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* fireNodeFocusReceived
    */
   /**
    *    This method will fire the nodeFocusReceived
    *  event to all listeners. The FlowNode passed to the listeners
    *  will be the node with the current focus
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void fireNodeFocusReceived ( ) {
      if ( listeners != null ) {
         int i    = 0 ;
         int size = listeners.size() ;

         while ( i < size ) {
            ((VisioListener)listeners.elementAt(i)).nodeFocusReceived ( focusNode ) ;
            i++ ;
         }
      }
   } /* fireNodeFocusReceived  */






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run different operations. The
    *  operation run is dependant on the name of the
    *  thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_FIRE_NODE_FOCUS_RECEIVED                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_FIRE_NODE_FOCUS_RECEIVED ) ) {
         fireNodeFocusReceived ( ) ;
      }

   }






   /*------------------------------------------------------------*/
   /* getNodes
    */
   /**
    *    This method will return all Nodes
    *  that are in the flow
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected Vector getNodes ( ) {
      Vector returnVec = null ;

      if ( objectNodes.size() > 0 ) {
         int size = objectNodes.size() ;
         int i    = 0 ;

         returnVec = new Vector ( size ) ;

         while ( i < size ) {
            returnVec.addElement ( ((ObjectNode)objectNodes.elementAt(i)).getNode() ) ;
            i++ ;
         } /* while  */

      } /* if  */

      return( returnVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getObjectNodes
    */
   /**
    *    This method will return all of the object
    *  nodes that this visio drawing has in it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected Vector getObjectNodes () {
      return( objectNodes ) ;
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has been changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {                                                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, we must refresh all component's properties       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshComponentProperties() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Grid Color Property                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( property == PROPERTY_GRID_COLOR )
         gridColor      = (Color)PropertySystem.get ( PROPERTY_GRID_COLOR ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Text Length                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( property == PROPERTY_VISIO_TEXT_LENGTH )
         lengthEdgeText = PropertySystem.getInt ( PROPERTY_VISIO_TEXT_LENGTH ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Edge Focus Color                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( property == PROPERTY_EDGE_FOCUS_COLOR )
         edgeFocusColor = (Color)value ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Edge Color                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( property == PROPERTY_EDGE_COLOR )
         edgeColor = (Color)value ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Edge Return Color                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( property == PROPERTY_EDGE_RETURN_COLOR )
         edgeReturnColor = (Color)value ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The flow background                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( property == PROPERTY_FLOW_BACKGROUND_COLOR )
         setBackground ( (Color) value ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Filter Property                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( property == PROPERTY_FLOW_FILTER )
         resetFilter() ;

      invalidate() ;
      revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* refreshComponentProperties
    */
   /**
    *    This method will tell each component in
    *  our flow to refresh it's properties
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshComponentProperties() {
      try {
         int i    = 0 ;
         int size = objectNodes.size() ;                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Tell all of our nodes to change color                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size )
            ((ObjectNode)objectNodes.elementAt(i++)).refreshProperties() ;

         if ( componentNode != null )
            componentNode.refreshProperties() ;

         revalidate() ;
      } catch ( Exception e ) {
      }
   } /* refreshComponentProperties  */


   /*------------------------------------------------------------*/
   /* getPaintTitles
    */
   /**
    *    This method will return TRUE if we are painting
    *  our titles
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getPaintTitles () {
      return( paintEdgeTitle ) ;
   }





   /*------------------------------------------------------------*/
   /* setPaintTitles
    */
   /**
    *    This method will either set the painting of
    *  titles on or off
    *  
    *
    *  @param paintTitles TRUE = Paint the edge titles
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPaintTitles ( boolean paintTitles ) {
      this.paintEdgeTitle = paintTitles ;
      repaint() ;

   }





   /*------------------------------------------------------------*/
   /* printToPrinter
    */
   /**
    *    This method will print ourselves to a printJob
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void printToPrinter ( PrintJob printJob ) {
      Dimension size     = getSize() ;
      Dimension pageSize = printJob.getPageDimension() ;
      int       x        = 15 ;
      int       y        = 90 ;
      Graphics  g        = null ;
      Image     image    = createImage ( size.width, size.height ) ;

      paint ( image.getGraphics() ) ;

      while ( y < size.height ) {

         x=0 ;
         while ( x < size.width ) {
            g = printJob.getGraphics() ;
            g.translate ( x-(x*2), y-(y*2) ) ;
            g.drawImage ( image, x, y, this ) ;
            g.translate ( 0, 0 ) ;
            g.dispose() ;
            x+=pageSize.width ;
         } /* while  */

         y+=pageSize.height ;
      } /* while  */

      image.flush() ;
      image = null ;
      System.gc() ;

   }





   /*------------------------------------------------------------*/
   /* resetFilter
    */
   /**
    *    This method will reset our filter to the
    *  current filter settings
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void resetFilter () {
      try {
         FlowEditorFilter filter = (FlowEditorFilter) PropertySystem.get ( PROPERTY_FLOW_FILTER ) ;
         int              i      = 0 ;
         int              size   = objectNodes.size() ;
         ObjectNode       oNode  = null ;                  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Assume we will show nothing                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         visibleObjectNodes.removeAllElements() ;
         visibleEdges.removeAllElements() ;                                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Hide the ComponentNode                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( componentNode != null )
            componentNode.setVisible ( false ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through, hiding filtered nodes and edges           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {

            oNode = (ObjectNode) objectNodes.elementAt(i) ;
            oNode.setPaint ( true ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is a visible node, add it to our list           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( oNode.getPaint() ) {
               visibleObjectNodes.addElement ( oNode ) ;
               resetFilterOnEdges ( AvalonUtil.getHeadEdgeGroups ( oNode.getNode(), symptom ) ) ;
               resetFilterOnEdges ( AvalonUtil.getTailEdgeGroups ( oNode.getNode(), symptom ) ) ;
            }

            i++ ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now that we have our newly built list of nodes and  */                                   
         /* edges, refresh our edge returns                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         refreshEdgeReturns() ;



      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }





   /*------------------------------------------------------------*/
   /* resetFilterOnEdges
    */
   /**
    *    This method will either add or remove an EdgeGRoup
    *  from the list of visibleEdgeGroups based on the
    *  flags passed
    *
    *  @param edgeGroupVec The Vector of EdgeGroups to process
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void resetFilterOnEdges ( Vector edgeGroups ) {
      if ( ( edgeGroups != null ) && ( edgeGroups.size() > 0 ) ) {
         int        i          = 0 ;
         int        size       = edgeGroups.size() ;
         ObjectNode tailObject = null ;
         ObjectNode headObject = null ;

         while ( i < size ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Make sure both the head and tail nodes are showing      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            tailObject = tailObjectNodeFromEdgeGroup ( (EdgeGroup) edgeGroups.elementAt(i) ) ;
            headObject = headObjectNodeFromEdgeGroup ( (EdgeGroup) edgeGroups.elementAt(i) ) ;
            if ( ( tailObject != null ) && ( headObject != null ) ) {
               if ( ( tailObject.getPaint() ) && 
                    ( headObject.getPaint() ) ) {

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Catch the IllegalStateException because The             */
                  /* EdgeGroup object will throw that if it is a new record.   */
                  /* If the exception is thrown, just add it to the list of    */
                  /* visible Edges.                                            */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  try {
                     if ( !visibleEdges.contains ( edgeGroups.elementAt(i) ) )
                        visibleEdges.addElement ( edgeGroups.elementAt(i) ) ;
                  } catch ( Exception e ) {
                     visibleEdges.addElement ( edgeGroups.elementAt(i) ) ;
                  }
               }
            } else {
               if ( !visibleEdges.contains ( edgeGroups.elementAt(i) ) )
                  visibleEdges.addElement ( edgeGroups.elementAt(i) ) ;
            }
            i++ ;
         }
      }

   } /* resetFilterOnEdges  */



   /*------------------------------------------------------------*/
   /* tailObjectNodeFromEdgeGroup
    */
   /**
    *    This method will return the tail ObjectNode
    *  for the edge group passed in
    *  
    *
    *  @param edgeGroup The EdgeGroup object to look for a tail on
    *  @param None
    *  @param None
    *
    *  @return ObjectNode
    *//*---------------------------------------------------------*/
   private ObjectNode tailObjectNodeFromEdgeGroup ( EdgeGroup edgeGroup ) {
      ObjectNode returnNode  = null ;
      Vector     objectNodes = getObjectNodes() ;
      int        i           = 0 ;
      int        size        = objectNodes.size() ;

      while ( ( returnNode == null ) && ( i < size ) ) {
         if ( ((ObjectNode)objectNodes.elementAt(i)).getNode() == edgeGroup.getTailNode() )
            returnNode = (ObjectNode) objectNodes.elementAt(i) ;
         i++ ;
      }

      return( returnNode ) ;
   }


   /*------------------------------------------------------------*/
   /* headObjectNodeFromEdgeGroup
    */
   /**
    *    This method will return the head ObjectNode
    *  for the edge group passed in
    *  
    *
    *  @param edgeGroup The EdgeGroup object to look for a head on
    *  @param None
    *  @param None
    *
    *  @return ObjectNode
    *//*---------------------------------------------------------*/
   private ObjectNode headObjectNodeFromEdgeGroup ( EdgeGroup edgeGroup ) {
      ObjectNode returnNode  = null ;
      Vector     objectNodes = getObjectNodes() ;
      int        i           = 0 ;
      int        size        = objectNodes.size() ;

      while ( ( returnNode == null ) && ( i < size ) ) {
         if ( ((ObjectNode)objectNodes.elementAt(i)).getNode() == edgeGroup.getTailNode() )
            returnNode = (ObjectNode) objectNodes.elementAt(i) ;
         i++ ;
      }

      return( returnNode ) ;
   }


   public void revalidate() {
      repaint() ;
      //super.revalidate() ;
      invalidate() ;
//      ((JScrollPane)getParent()).revalidate() ;
   }






   /*------------------------------------------------------------*/
   /* mouseClicked
    */
   /**
    *    This means the user has clicked on a node,
    *  we will try and find that node and set it's
    *  focus
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mousePressed ( MouseEvent e ) {
      processMouseClick ( e ) ;
   }

   public void processMouseClick ( MouseEvent e ) {
      Point       pt   = new Point ( e.getX(), e.getY() ) ;
      ObjectNode  node = objectNodeFromPoint ( pt ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, set the focus                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( node != null ) {

         focusNode = node ;
         setComponentNode ( focusNode ) ;
         componentNode.requestFocus() ;
         focusGained () ;
      }
   }

   public void mouseClicked ( MouseEvent e ) {}
   public void mouseReleased ( MouseEvent e ) {}
   public void mouseMoved ( MouseEvent e ) {}
   public void mouseEntered ( MouseEvent e ) {}
   public void mouseExited ( MouseEvent e ) {}




   /*------------------------------------------------------------*/
   /* objectNodeFromPoint
    */
   /**
    *    This method will return an ObjectNode that
    *  contains the point passed
    *  
    *
    *  @param point The Point to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private ObjectNode objectNodeFromPoint ( Point pt ) {
      int        i          = 0 ;
      ObjectNode returnNode = null ;

      if ( visibleObjectNodes != null ) {
         int size = visibleObjectNodes.size() ;

         while ( (returnNode == null) && ( i < size ) ) {
            if ( ((ObjectNode)visibleObjectNodes.elementAt(i)).containsPoint ( pt ) )
               returnNode = (ObjectNode) visibleObjectNodes.elementAt(i) ;
            i++ ;
         }
      }

      return( returnNode ) ;
   }






   /*------------------------------------------------------------*/
   /* setComponentNode
    */
   /**
    *    This method will set the component node that
    *  is visible. The ComponentNode is the single
    *  component on the panel that can interact with the user
    *
    *  @param objectNode The ObjectNode to set the componentNode as
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setComponentNode ( ObjectNode objectNode ) {                                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the componentNode is null, create one and add it     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( componentNode == null ) {

         componentNode = new ComponentNode ( objectNode ) ;
         super.add ( componentNode ) ;
         parentViewport = (JViewport) getParent() ;
         componentNode.setVisioPanel ( this ) ;
         componentNode.setVisible ( true ) ;
         componentNode.setBounds ( objectNode.getNodeBounds() ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, save the current values and set              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else
         componentNode.setObjectNode ( objectNode ) ;

      if ( componentNode != null )
         componentNode.requestFocus() ;
      componentNode.setVisible ( true ) ;

   }





   /*------------------------------------------------------------*/
   /* ensureVisible
    */
   /**
    *    This method will ensure that the view port is
    *  showing the bounds passed in. Scrolling
    *  will occur if the bounds are not visible
    *
    *  @param bounds The Bounds to ensure is viewing.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void ensureVisible ( Rectangle bounds ) {
      Rectangle viewBounds = parentViewport.getViewRect() ;

      newScrollPt.x = viewBounds.x ;
      newScrollPt.y = viewBounds.y ;

      /* If bounds went off the right side  */
      if ( (viewBounds.x+viewBounds.width) < bounds.x )
         newScrollPt.x+=((viewBounds.x+viewBounds.width)-bounds.x) ;

      /* If bounds went off the left side  */
      else if ( (viewBounds.x) > bounds.x )
         newScrollPt.x = bounds.x ;

      else if ( (viewBounds.x+viewBounds.width) < (bounds.x+bounds.width) )
         newScrollPt.x+=((bounds.x+bounds.width)-(viewBounds.x+viewBounds.width)) ;

      /* if bounds went off the Bottom  */
      if ( (viewBounds.y+viewBounds.height) < bounds.y )
         newScrollPt.y = bounds.y ;

      /* if bounds went off the top  */
      else if ( (viewBounds.y) > bounds.y )
         newScrollPt.y = bounds.y ;

      /* If the bottom went off the bottom  */
      else if ( (viewBounds.y+viewBounds.height) < (bounds.y+bounds.height) )
         newScrollPt.y+=((bounds.y+bounds.height)-(viewBounds.y+viewBounds.height)) ;

      parentViewport.setViewPosition ( newScrollPt ) ;

   }




   /*------------------------------------------------------------*/
   /* remove
    */
   /**
    *    This method will remove an ObjectNode
    *  from the flow
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void remove ( ObjectNode objectNode ) {
      objectNode.getNode().updateRecStatus ( Node.REC_DELETE ) ;
      visibleObjectNodes.removeElement ( objectNode ) ;
      componentNode.setVisible ( false ) ;
   }


   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method MUST be called  when a VisioPanel
    *  is no longer used.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void free () {
      PropertySystem.removePropertyListener ( PROPERTY_GRID_COLOR,            this ) ;
      PropertySystem.removePropertyListener ( PROPERTY_VISIO_TEXT_LENGTH,     this ) ;
      PropertySystem.removePropertyListener ( PROPERTY_EDGE_FOCUS_COLOR,      this ) ;
      PropertySystem.removePropertyListener ( PROPERTY_EDGE_RETURN_COLOR,     this ) ;
      PropertySystem.removePropertyListener ( PROPERTY_EDGE_COLOR,            this ) ;
      PropertySystem.removePropertyListener ( PROPERTY_FLOW_FILTER,           this ) ;
      PropertySystem.removePropertyListener ( PROPERTY_FLOW_BACKGROUND_COLOR, this ) ;
   } /* free */





   /*------------------------------------------------------------*/
   /* refreshEdgeReturns
    */
   /**
    *    This method will take all of the visible EdgeGroups
    *  that we have and creates a new Vector of
    *  edgeReturns from that list.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void refreshEdgeReturns () {
      int       i         = 0 ;
      int       size      = visibleEdges.size() ;
      Vector    edgeVec   = null ;
      EdgeGroup edgeGroup = null ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear all edge returns first                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      edgeReturns.removeAllElements() ;                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, loop through all visible edge groups               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         edgeGroup = (EdgeGroup)visibleEdges.elementAt(i) ;

         if ( edgeGroup.valid() ) {
            int k        = 0 ;
            int edgeSize = 0 ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the edges for the edge group                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            edgeVec = edgeGroup.getEdges() ;
            edgeSize = edgeVec.size() ;                                           

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, loop through all edges and add all edge returns    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            k=0 ;
            while ( k < edgeSize ) {
               try {
                  if ( ( ((Edge)edgeVec.elementAt(k)).getReturnNode() != null ) && 
                       ( ((Edge)edgeVec.elementAt(k)).valid() ) ) {
                     edgeReturns.addElement ( new EdgeReturnDisplayRec ( (Edge)edgeVec.elementAt(k), edgeGroup ) ) ;
                  }
               } catch ( Exception e ) {
                  e.printStackTrace() ;
               }
               k++ ;
            } /* while  */
         } /* if edgeGroup is valid  */
         i++ ;
      } /* while  */

   } /* refreshEdgeReturns  */


   public void mouseDragged ( MouseEvent e ) {
      componentNode.setLocation ( e.getX(), e.getY() ) ;
   }
} /* VisioPanel */
