package com.ibm.nzna.projects.qit.avalon.editors ;           
/*-----------------------------------------------------------*/
/*   VisioPanelReader                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: VisioPanelReader                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object holds methods that the VisioPanel uses 
 * to read information about the nodes it presents.
 *
 *
 * @version 1.0, 10/2/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   VisioPanelReader                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class VisioPanelReader extends Object {





   /*------------------------------------------------------------*/
   /* readNodeData
    */
   /**
    *    This method will read in all of the data needed
    *  for the nodes to display initially. This includes
    *  the Title, Country and Document Classification.<br><br>
    *
    *  This method requires the Symptom param to be non-null and will
    *  take all nodes from the Symptom and create ObjectNodes for 
    *  each
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return A Vector of newly created ObjectNodes
    *//*---------------------------------------------------------*/
   protected static Vector readNodeData ( Symptom symptom ) {
      Vector objectNodeVec = new Vector ( 1, 1 ) ;
      Vector nodeVec       = symptom.getNodes() ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have valid nodes, continue                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( nodeVec != null ) && ( nodeVec.size() > 0 ) ) {
         Vector questions      = null ;
         Vector actions        = null ;
         Vector symptoms       = null ;
         Vector draftQuestions = null ;
         Vector draftActions   = null ;
         Vector draftSymptoms  = null ;                           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, create all of the ObjectNodes                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         createObjectNodesFromNodes ( symptom, objectNodeVec, nodeVec ) ;                                      


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, take apart the nodes into the objects that         */
         /* we read data for.                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         questions      = getNodes ( objectNodeVec, true,  Node.NODE_TYPE_QUESTION ) ;
         actions        = getNodes ( objectNodeVec, true,  Node.NODE_TYPE_ACTION ) ;
         symptoms       = getNodes ( objectNodeVec, true,  Node.NODE_TYPE_SYMPTOM ) ;
         draftQuestions = getNodes ( objectNodeVec, false, Node.NODE_TYPE_QUESTION ) ;
         draftActions   = getNodes ( objectNodeVec, false, Node.NODE_TYPE_ACTION ) ;
         draftSymptoms  = getNodes ( objectNodeVec, false, Node.NODE_TYPE_SYMPTOM ) ;                                                          

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, read in and add all countries and classification   */
         /* information about each node                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         long currentTimeMillis = System.currentTimeMillis() ;
         processCountryClass ( symptom, questions, "OA.QUESTIONCOUNTRY",      "OA.QUESTION", "QUESTIONIND" , "OADRAFT.NODE", "ACTIVEQUESTIONIND" ) ;
         processCountryClass ( symptom, actions,   "OA.ACTIONCOUNTRY",        "OA.ACTION",   "ACTIONIND" ,   "OADRAFT.NODE", "ACTIVEACTIONIND" ) ;
         processCountryClass ( symptom, symptoms,  "OA.SYMPTOMCOUNTRY",       "OA.SYMPTOM",  "SYMPTOMIND" ,  "OADRAFT.NODE", "ACTIVESYMPTOMIND" ) ;

         processCountryClass ( symptom, draftQuestions, "OADRAFT.QUESTIONCOUNTRY", "OADRAFT.QUESTION", "QUESTIONIND" , "OADRAFT.NODE", "DRAFTQUESTIONIND" ) ;
         processCountryClass ( symptom, draftActions,   "OADRAFT.ACTIONCOUNTRY",   "OADRAFT.ACTION",   "ACTIONIND" ,   "OADRAFT.NODE", "DRAFTACTIONIND" ) ;
         processCountryClass ( symptom, draftSymptoms,  "OADRAFT.SYMPTOMCOUNTRY",  "OADRAFT.SYMPTOM",  "SYMPTOMIND" ,  "OADRAFT.NODE", "DRAFTSYMPTOMIND" ) ;

         processTitles ( symptom, questions, "OA.QUESTIONTITLE",      "QUESTIONIND" ) ;
         processTitles ( symptom, actions,   "OA.ACTIONTITLE",        "ACTIONIND"   ) ;
         processTitles ( symptom, symptoms,  "OA.SYMPTOMTITLE",       "SYMPTOMIND"  ) ;

         processTitles ( symptom, draftQuestions, "OADRAFT.QUESTIONTITLE", "QUESTIONIND" ) ;
         processTitles ( symptom, draftActions,   "OADRAFT.ACTIONTITLE",   "ACTIONIND"   ) ;
         processTitles ( symptom, draftSymptoms,  "OADRAFT.SYMPTOMTITLE",  "SYMPTOMIND"  ) ;

         LogSystem.log ( 1, "----------------------------------" ) ;
         LogSystem.log ( 1, "----------------------------------" ) ;
         LogSystem.log ( 1, "End Reading Extra Symptom Inf. Total Time Taken:"+((System.currentTimeMillis()-currentTimeMillis ) ) ) ;
         LogSystem.log ( 1, "----------------------------------" ) ;

      }

      return( objectNodeVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getNodes
    */
   /**
    *    This method will get all nodes of type 'nodeType'
    *  from the vector 'objectNodeVec' and will return
    *  a new vector with the nodes. 
    *
    *  @param objectNodeVec The vector containing all ObjectNodes
    *  @param active  TRUE = only pull Active Nodes, FALSE = only pull Draft Nodes
    *  @param nodeType The type of node to get
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   private static Vector getNodes ( Vector objectNodeVec, boolean active, int nodeType ) {
      Vector returnVec = new Vector ( 1, 1 ) ;
      int    i         = 0 ;
      int    size      = objectNodeVec.size() ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Loop through gathering each node                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the type matches, continue                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ((ObjectNode)objectNodeVec.elementAt(i)).getNode().getNodeType() == nodeType ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is a draft, and we are looking for drafts,      */
            /* add it                                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ((ObjectNode)objectNodeVec.elementAt(i)).isDraft() ) {
               if ( !active )
                  returnVec.addElement ( objectNodeVec.elementAt(i) ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, if it is active, and we are looking for      */
            /* active, add it now.                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else
               if ( active )
               returnVec.addElement ( objectNodeVec.elementAt(i) ) ;
         }
         i++ ;
      }

      return( returnVec ) ;
   } /* getNodes */






   /*------------------------------------------------------------*/
   /* processCountryClass
    */
   /**
    *    This method will read in all of the countries and
    *  classifications for the nodes passed in. For each
    *  node, a new ObjectNode will be created and placed in the objectNodeVec
    *
    *  @param symptom The symptom to look with
    *  @param objectNodeVec The vector to put newly created ObjectNodes in
    *  @param nodeVec The vector of nodes
    *  @param countryTable The fully qualified country table (OA.QUESTIONCOUNTRY or OA.ACTIONCOUNTRY etc..)
    *  @param objectTable The fully qualified object table ( OA.QUESTION or OA.ACTION etc.. )
    *  @param objectInd The object indicator name in the objectTable ( QUESTIONIND or ACTIONIND etc.. )
    *  @param nodeTable The fully qualified node table ( OA.NODE or OADRAFT.NODE )
    *  @param nodeObjectInd The name of the object indicator in the node table (QUESTIONIND, DRAFTQUESTIONIND, ACTIVEQUESTIONIND etc..)
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void processCountryClass ( Symptom symptom, Vector objectNodeVec, String countryTable, String objectTable, String objectInd, String nodeTable, String nodeObjectInd ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only start if we have some nodes                        */                       
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( objectNodeVec.size() > 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "VisioPanelReader.processCountryClass", 5 ) ;

         try {
            Statement stmt       = sqlMethod.createStatement() ;
            int       symptomInd = 0 ;

            if ( symptom instanceof SymptomDraft ) {
               symptomInd = symptom.getInd() ;
               if ( symptomInd == 0 ) 
                  symptomInd = ((SymptomDraft)symptom).getActiveSymptomInd() ;

            } else {
               symptomInd = symptom.getInd() ;
            }

            if ( stmt != null ) {
               ObjectNode objectNode = null ;
               Node       node       = null ;
               ResultSet  rs         = stmt.executeQuery ( "select a."+objectInd+", a.countryind, b.docclassind "+
                                                           "from "+countryTable+" a, "+objectTable+" b "+
                                                           "where a."+objectInd+" in "+
                                                           "      (select c."+nodeObjectInd+" from "+nodeTable+" c "+
                                                           "       where c.symptomInd = "+symptomInd+") and "+
                                                           "      a."+objectInd+" = b."+objectInd+" "+
                                                           "for fetch only" ) ;       

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Process Each Row                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( rs.next() )
                  addCountryClassToObjectNode ( objectNodeVec, rs.getInt(1), rs.getInt(2), rs.getInt(3) ) ;

               rs.close() ;

            } /* if */

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
         }

         sqlMethod.close() ;
      } /* if  */
   }





   /*------------------------------------------------------------*/
   /* createObjectNodesFromNodes
    */
   /**
    *    This method will create object nodes from
    *  the nodes passed in. Each newly created
    *  object node will be placed in the objectNodeVec passed
    *
    *  @param objectNodeVe The Vector to store newly created ObjectNodes
    *  @param nodeVec The nodes to create ObjectNodes from
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void createObjectNodesFromNodes ( Symptom symptom, Vector objectNodesVec, Vector nodesVec ) {
      int i    = 0 ;
      int size = nodesVec.size() ;

      while ( i < size ) {
         objectNodesVec.addElement ( new ObjectNode ( symptom, (Node)nodesVec.elementAt(i) ) ) ;
         i++ ;
      }
   }





   /*------------------------------------------------------------*/
   /* addCountryClassToObjectNode
    */
   /**
    *    This method will add the country and
    *  classification to the ObjectNode who represents
    *  the Indicator passed. If there are multiple objectNodes
    *  for the indicator passed, all ObjectNodes are updated
    *
    *  @param objectNodeVec The Vector holding all ObjectNodes
    *  @param objectInd The indicator of the object to set the country and classification for. Please note that
    *                   This is not the Node Indicator, but the Question/Action/Symptom indicator
    *  @param countryInd The country indicator of the country to add as defined in TIGRIS.TYPECOUNTRYCODE
    *  @param classInd The DocClassInd for classification as defined in TIGRIS.TYPEDOCCLASS
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void addCountryClassToObjectNode ( Vector objectNodeVec, int objectInd, int countryInd, int classInd ) {
      int i    = 0 ;
      int size = objectNodeVec.size() ;

      while ( i < size ) {
         if ( ((ObjectNode)objectNodeVec.elementAt(i)).getOAObjectInd() == objectInd ) {
            ((ObjectNode)objectNodeVec.elementAt(i)).addCountry ( (TypeCountryCodeRec)TypeList.getInstance().objectFromInd ( countryInd, TypeList.TYPE_COUNTRYCODE ) ) ;
            ((ObjectNode)objectNodeVec.elementAt(i)).setDocClassInd ( classInd ) ;
         }
         i++ ;
      }
   }


   /*------------------------------------------------------------*/
   /* processCountryClass
    */
   /**
    *    This method will read in all of the countries and
    *  classifications for the nodes passed in. For each
    *  node, a new ObjectNode will be created and placed in the objectNodeVec
    *
    *  @param symptom The symptom to look with
    *  @param objectNodeVec The vector to put newly created ObjectNodes in
    *  @param nodeVec The vector of nodes
    *  @param titleTable The fully qualified title table (OA.QUESTIONTITLE or OA.ACTIONTITLE etc..)
    *  @param objectInd The object indicator name in the objectTable ( QUESTIONIND or ACTIONIND etc.. )
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void processTitles ( Symptom symptom, Vector objectNodeVec, String titleTable, String objectInd ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only start if we have some nodes                        */                       
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( objectNodeVec.size() > 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "VisioPanelReader.processTitles", 5 ) ;

         try {
            Statement stmt       = sqlMethod.createStatement() ;
            int       symptomInd = 0 ;

            if ( symptom instanceof SymptomDraft ) {
               symptomInd = symptom.getInd() ;
               if ( symptomInd == 0 )
                  symptomInd = ((SymptomDraft)symptom).getActiveSymptomInd() ;
            } else {
               symptomInd = symptom.getInd() ;
            }

            if ( stmt != null ) {
               ObjectNode objectNode = null ;
               Node       node       = null ;
               ResultSet  rs         = stmt.executeQuery ( "select a."+objectInd+", a.title from "+titleTable+" a "+
                                                           "where a."+objectInd+" in ( "+createIndInClause ( objectNodeVec )+" ) "+
                                                           "and a.sortOrder = ( select max(sortorder) from "+titleTable+" where "+objectInd+"=a."+objectInd+" )"+
                                                           "for fetch only" ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Process Each Row                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( rs.next() )
                  setTitlesOnObjectNodes ( objectNodeVec, rs.getInt(1), rs.getString(2).trim() ) ;

               rs.close() ;

            } /* if */

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
         }

         sqlMethod.close() ;
      } /* if  */
   } /* processTitles  */






   /*------------------------------------------------------------*/
   /* createIndInClause
    */
   /**
    *    This method will construct the object indicators
    *  given from the ObjectNodeVec into a string
    *  that can be used in an SQL Statemtent's in clause
    *
    *  @param objectNodeVec The vector of ObjectNodes
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static String createIndInClause ( Vector objectNodeVec ) {
      int    i      = 0 ;
      int    size   = objectNodeVec.size() ;
      String clause = "" ;

      while ( i < size ) {
         if ( i < (size-1) )
            clause+=""+((ObjectNode)objectNodeVec.elementAt(i)).getOAObjectInd()+", " ;
         else
            clause+=""+((ObjectNode)objectNodeVec.elementAt(i)).getOAObjectInd();
         i++ ;
      }

      return( clause ) ;
   }




   /*------------------------------------------------------------*/
   /* setTitlesOnObjectNodes
    */
   /**
    *    This method will set the title (or description) on each
    *  Object Node in the vector that matches the objectInd passed in
    *
    *  @param objectNodeVec The Vector holding all ObjectNodes
    *  @param objectInd The indicator of the object to set the country and classification for. Please note that
    *                   This is not the Node Indicator, but the Question/Action/Symptom indicator
    *  @param title     The title to set
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void setTitlesOnObjectNodes ( Vector objectNodeVec, int objectInd, String title ) {
      int i    = 0 ;
      int size = objectNodeVec.size() ;

      while ( i < size ) {
         if ( ((ObjectNode)objectNodeVec.elementAt(i)).getOAObjectInd() == objectInd ) {
            ((ObjectNode)objectNodeVec.elementAt(i)).setNodeDescription ( title ) ;
         }
         i++ ;
      }
   }
} /* VisioPanelReader */
