package com.ibm.nzna.projects.qit.avalon.editors.symptomProp ;
/*-----------------------------------------------------------*/
/*   FilterPage                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FilterPage                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.help.* ;     
import com.ibm.nzna.projects.qit.gui.* ;     
import com.ibm.nzna.projects.qit.dbgui.* ;     
import com.ibm.nzna.projects.qit.avalon.base.* ;     
import com.ibm.nzna.projects.qit.avalon.editors.* ;     
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import java.io.File ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
/**
 * This page belongs in the SymptomPropertyDlg's notebook. It holds 
 * all general properties that can be adjusted for the Symptom Editor
 *
 * @version 1.0, 1/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FilterPage                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class FilterPage extends JPanel implements AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Components                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private CountrySelectPanel countryPanel  = null ;
   private JLabel             st_HELP       = null ;
   private JLabel             st_SITE       = null ;
   private JComboBox          cb_SITE       = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the page and all GUI Components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FilterPage() {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP       = new JLabel () ;
      countryPanel  = new CountrySelectPanel () ;
      st_SITE       = new JLabel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;
      countryPanel.setBorder ( GUISystem.loweredBorder ) ;
      setText() ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI to our panel                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP ) ;
      add ( countryPanel ) ;
      add ( st_SITE ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our GUI Controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       y         = 85 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      st_HELP.setBounds ( 5, 5, size.width-5, 75 ) ;

      countryPanel.setBounds ( 5, y, size.width-10, rowHeight*5 ) ;
      y+=(rowHeight*6) ;

      st_SITE.setBounds ( 5, y, size.width-10, rowHeight ) ;
      y+=rowHeight ;

      cb_SITE.setBounds ( 5, y, size.width-10, rowHeight ) ;

   } /* doLayout */





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on all
    *  of our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText ( ) {
      Vector sites = new Vector ( 3 ) ;                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the text on our JLabels                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setText ( Str.getStr ( STR_SYMPTOM_FILTER_PAGE_HELP ) ) ;
      st_SITE.setText ( Str.getStr ( STR_WEBSITE ) ) ;                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our vector of web sites                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sites.addElement ( Str.getStr ( STR_ESUPPORT ) ) ;
      sites.addElement ( Str.getStr ( STR_BUSINESS_PARTNER ) ) ;
      sites.addElement ( Str.getStr ( STR_INFOTIPS ) ) ;             


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Recreate the Combo Box                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( cb_SITE != null ) {
         remove ( cb_SITE ) ;
         cb_SITE = null ;
      }

      cb_SITE = new JComboBox ( sites ) ;
      add ( cb_SITE ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refesh out list
    *  of Quest Applets
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      try {
         FlowEditorFilter filter = (FlowEditorFilter)PropertySystem.get ( PROPERTY_FLOW_FILTER ) ;

         countryPanel.setCountryList ( filter.getCountryFilterList() ) ;
         cb_SITE.setSelectedItem ( stringFromSite ( filter.getSite() ) ) ;
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* stringFromSite
    */
   /**
    *    This method will return the String for
    *  the Site Id passed in
    *  
    *
    *  @param siteId The Site Id to return the String for
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String stringFromSite ( int siteId ) {
      String retStr = null ;

      switch ( siteId ) {
         case FlowEditorFilter.SITE_ESUPPORT :
            retStr = Str.getStr ( STR_ESUPPORT ) ;
            break ;

         case FlowEditorFilter.SITE_BUSINESSPARTNER :
            retStr = Str.getStr ( STR_BUSINESS_PARTNER ) ;
            break ;

         case FlowEditorFilter.SITE_INFOTIPS :
            retStr = Str.getStr ( STR_INFOTIPS ) ;
            break ;
      }
      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* siteFromString
    */
   /**
    *    This method will return the Site Indicator
    *  from the string passed in
    *  
    *
    *  @param string The String version of the Web Site
    *  @param None
    *  @param None
    *
    *  @return site
    *//*---------------------------------------------------------*/
   private int siteFromString ( String site ) {
      int retSite = 0 ;

      if ( site.equals ( Str.getStr ( STR_ESUPPORT ) ) )
         retSite = FlowEditorFilter.SITE_ESUPPORT ;
      else if ( site.equals ( Str.getStr ( STR_BUSINESS_PARTNER ) ) )
         retSite = FlowEditorFilter.SITE_BUSINESSPARTNER ;
      else if ( site.equals ( Str.getStr ( STR_INFOTIPS ) ) )
         retSite = FlowEditorFilter.SITE_INFOTIPS ;

      return( retSite ) ;
   }




   /*------------------------------------------------------------*/
   /* saveProperties
    */
   /**
    *    This method will make the final save
    *  for all operations that have been
    *  made on this panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveProperties () {
      try {

         FlowEditorFilter filter = (FlowEditorFilter)PropertySystem.get ( PROPERTY_FLOW_FILTER ) ;

         filter.setCountryFilterList ( countryPanel.getCountryList() ) ;
         filter.setSite ( siteFromString ( (String)cb_SITE.getSelectedItem() ) ) ;

         PropertySystem.put ( PROPERTY_FLOW_FILTER, filter ) ;

      } catch ( Exception e ) {
      }

      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will release resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
   }

} /* FilterPage */

