package com.ibm.nzna.projects.qit.avalon.editors.symptomProp ;
/*-----------------------------------------------------------*/
/*   GeneralPage                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: GeneralPage                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.help.* ;     
import com.ibm.nzna.projects.qit.gui.* ;     
import com.ibm.nzna.projects.qit.avalon.base.* ;     
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import java.io.File ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
/**
 * This page belongs in the SymptomPropertyDlg's notebook. It holds 
 * all general properties that can be adjusted for the Symptom Editor including
 * text widths and colors
 *
 * @version 1.0, 1/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   GeneralPage                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class GeneralPage extends JPanel implements ListSelectionListener, ActionListener, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Components                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_HELP                  = null ;
   private JLabel      st_TEXT_LENGTH           = null ;
   private JTextField  ef_TEXT_LENGTH           = null ;

   private JList       lb_OBJECTS               = null ;
   private JScrollPane scr_OBJECTS              = null ;
   private DButton     pb_OBJECT_COLOR          = null ;
   private JLabel      st_OBJECTS               = null ;
   private JLabel      st_OBJECT_COLOR          = null ;

   private Color flowNodeColor                  = null ;
   private Color flowNodeActionColor            = null ;
   private Color flowNodeFocusColor             = null ;
   private Color gridColor                      = null ;
   private Color flowNodeTextColor              = null ;
   private Color flowNodeFocusTextColor         = null ;
   private Color edgeColor                      = null ;
   private Color edgeFocusColor                 = null ;
   private Color edgeReturnColor                = null ;
   private Color flowBackgroundColor            = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the page and all GUI Components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public GeneralPage() {     
      Vector objectVec = new Vector ( 5 ) ;

      objectVec.addElement ( Str.getStr ( STR_FLOW_BACKGROUND_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_NODE_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_NODE_TEXT_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_NODE_FOCUS_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_NODE_FOCUS_TEXT_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_GRID_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_ACTION_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_EDGE_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_EDGE_FOCUS_COLOR ) ) ;
      objectVec.addElement ( Str.getStr ( STR_EDGE_RETURN_COLOR ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP                  = new JLabel () ;
      st_TEXT_LENGTH           = new JLabel () ;
      st_OBJECTS               = new JLabel () ;
      st_OBJECT_COLOR          = new JLabel () ;
      lb_OBJECTS               = new JList ( objectVec ) ;
      scr_OBJECTS              = new JScrollPane ( lb_OBJECTS )  ;
      ef_TEXT_LENGTH           = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 4 ), "", 0 ) ;
      pb_OBJECT_COLOR          = new DButton ( "" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;
      setText() ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI to our panel                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP ) ;
      add ( st_OBJECTS ) ;
      add ( scr_OBJECTS ) ;
      add ( st_OBJECT_COLOR ) ;
      add ( pb_OBJECT_COLOR ) ;
      add ( st_TEXT_LENGTH ) ;
      add ( ef_TEXT_LENGTH ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our listeners                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OBJECT_COLOR.addActionListener            ( this ) ;
      lb_OBJECTS.addListSelectionListener          ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData() ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our GUI Controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       y         = 85 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      st_HELP.setBounds ( 5, 5, size.width-5, 75 ) ;

      st_OBJECTS.setBounds ( 5, y, 180, rowHeight ) ;
      y+=rowHeight ;

      scr_OBJECTS.setBounds ( 5, y, 180, rowHeight*8 ) ;
      st_OBJECT_COLOR.setBounds ( 190, y, 180, rowHeight ) ;
      y+=rowHeight ;
      pb_OBJECT_COLOR.setBounds ( 190, y, 75, rowHeight ) ;
      y+=rowHeight*8 ;

      st_TEXT_LENGTH.setBounds ( 5, y, 180, rowHeight ) ;
      ef_TEXT_LENGTH.setBounds ( 190, y, 50, rowHeight ) ;
      y+=rowHeight ;

   } /* doLayout */





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on all
    *  of our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText ( ) {
      st_HELP.setText                   ( Str.getStr ( STR_SYMPTOM_GENERAL_PAGE_HELP ) ) ;
      st_TEXT_LENGTH.setText            ( Str.getStr ( STR_TEXT_LENGTH ) ) ;
      st_OBJECTS.setText                ( Str.getStr ( STR_FLOW_OBJECTS ) ) ;
      st_OBJECT_COLOR.setText           ( Str.getStr ( STR_COLOR ) ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refesh out list
    *  of Quest Applets
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      try {

         flowNodeColor                  = (Color)PropertySystem.get ( PROPERTY_FLOWNODE_COLOR ) ;
         flowNodeActionColor            = (Color)PropertySystem.get ( PROPERTY_FLOWNODE_ACTION_COLOR ) ;
         flowNodeFocusColor             = (Color)PropertySystem.get ( PROPERTY_FLOWNODE_FOCUS_COLOR ) ;
         gridColor                      = (Color)PropertySystem.get ( PROPERTY_GRID_COLOR ) ;
         flowNodeTextColor              = (Color)PropertySystem.get ( PROPERTY_FLOWNODE_TEXT_COLOR ) ;
         flowNodeFocusTextColor         = (Color)PropertySystem.get ( PROPERTY_FLOWNODE_FOCUS_TEXT_COLOR ) ;
         edgeColor                      = (Color)PropertySystem.get ( PROPERTY_EDGE_COLOR ) ;
         edgeFocusColor                 = (Color)PropertySystem.get ( PROPERTY_EDGE_FOCUS_COLOR ) ;
         edgeReturnColor                = (Color)PropertySystem.get ( PROPERTY_EDGE_RETURN_COLOR ) ;
         flowBackgroundColor            = (Color)PropertySystem.get ( PROPERTY_FLOW_BACKGROUND_COLOR ) ;


         ef_TEXT_LENGTH.setText ( ""+PropertySystem.getInt   ( PROPERTY_VISIO_TEXT_LENGTH ) ) ;
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* saveProperties
    */
   /**
    *    This method will make the final save
    *  for all operations that have been
    *  made on this panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveProperties () {

      if ( flowNodeColor != PropertySystem.get ( PROPERTY_FLOWNODE_COLOR ) )
         PropertySystem.put     ( PROPERTY_FLOWNODE_COLOR,            flowNodeColor ) ;
      if ( flowNodeColor != PropertySystem.get ( PROPERTY_EDGE_COLOR ) )
         PropertySystem.put     ( PROPERTY_EDGE_COLOR, edgeColor ) ;
      if ( flowNodeColor != PropertySystem.get ( PROPERTY_EDGE_FOCUS_COLOR ) )
         PropertySystem.put     ( PROPERTY_EDGE_FOCUS_COLOR,  edgeFocusColor ) ;
      if ( flowNodeColor != PropertySystem.get ( PROPERTY_EDGE_RETURN_COLOR ) )
         PropertySystem.put     ( PROPERTY_EDGE_RETURN_COLOR, edgeReturnColor ) ;
      if ( flowNodeColor != PropertySystem.get ( PROPERTY_FLOW_BACKGROUND_COLOR ) )
         PropertySystem.put     ( PROPERTY_FLOW_BACKGROUND_COLOR, flowBackgroundColor ) ;
      if ( flowNodeActionColor != PropertySystem.get ( PROPERTY_FLOWNODE_ACTION_COLOR ) )
         PropertySystem.put     ( PROPERTY_FLOWNODE_ACTION_COLOR,     flowNodeActionColor ) ;
      if ( flowNodeFocusColor != PropertySystem.get ( PROPERTY_FLOWNODE_FOCUS_COLOR ) )
         PropertySystem.put     ( PROPERTY_FLOWNODE_FOCUS_COLOR,      flowNodeFocusColor ) ;
      if ( gridColor != PropertySystem.get ( PROPERTY_GRID_COLOR ) )
         PropertySystem.put     ( PROPERTY_GRID_COLOR,                gridColor ) ;
      if ( flowNodeTextColor != PropertySystem.get ( PROPERTY_FLOWNODE_TEXT_COLOR ) )
         PropertySystem.put     ( PROPERTY_FLOWNODE_TEXT_COLOR,       flowNodeTextColor ) ;
      if ( flowNodeFocusTextColor != PropertySystem.get ( PROPERTY_FLOWNODE_FOCUS_TEXT_COLOR ) )
         PropertySystem.put     ( PROPERTY_FLOWNODE_FOCUS_TEXT_COLOR, flowNodeFocusTextColor ) ;

      try {
         Integer textLength = new Integer ( ef_TEXT_LENGTH.getText() ) ;

         if ( textLength.intValue() != PropertySystem.getInt ( PROPERTY_VISIO_TEXT_LENGTH ) )
            PropertySystem.putInt  ( PROPERTY_VISIO_TEXT_LENGTH, textLength ) ;
      } catch ( Exception e ) {

      }
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will release resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to buttons being
    *  pressed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_GRID_COLOR                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_OBJECT_COLOR ) {
         Object object = lb_OBJECTS.getSelectedValue() ;

         if ( object != null ) {
            Color newColor = new JColorChooser().showDialog ( GUISystem.getParentDefWin ( this ),
                                                              (String)object,
                                                              getObjectColor ( (String)object ) ) ;                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we get a new color. use it                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( newColor != null ) {
               setObjectColor ( (String)object, newColor ) ;
               pb_OBJECT_COLOR.setBackground ( newColor ) ;
            } /* if  */
         }

      } /* pb_OBJECT_COLOR  */

   } /* actionPerformed  */



   /*------------------------------------------------------------*/
   /* setObjectColor
    */
   /**
    *    This method will set one of the Flow Object's
    *  colors to what is passed in
    *  
    *
    *  @param objectName The String representation of the object's name
    *  @param color The new color to set to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setObjectColor ( String objectName, Color color ) {
      if ( objectName.equals ( Str.getStr ( STR_NODE_COLOR ) ) )
         flowNodeColor = color ;
      else if ( objectName.equals ( Str.getStr ( STR_NODE_TEXT_COLOR ) ) )
         flowNodeTextColor = color ;
      else if ( objectName.equals ( Str.getStr ( STR_NODE_FOCUS_COLOR ) ) )
         flowNodeFocusColor = color ;
      else if ( objectName.equals ( Str.getStr ( STR_NODE_FOCUS_TEXT_COLOR ) ) )
         flowNodeFocusTextColor = color ;
      else if ( objectName.equals ( Str.getStr ( STR_GRID_COLOR ) ) )
         gridColor = color ;
      else if ( objectName.equals ( Str.getStr ( STR_ACTION_COLOR ) ) )
         flowNodeActionColor = color ;
      else if ( objectName.equals ( Str.getStr ( STR_EDGE_FOCUS_COLOR ) ) ) 
         edgeFocusColor = color ;
      else if ( objectName.equals ( Str.getStr ( STR_EDGE_RETURN_COLOR ) ) )
         edgeReturnColor = color ;
      else if ( objectName.equals ( Str.getStr ( STR_FLOW_BACKGROUND_COLOR ) ) )
         flowBackgroundColor = color ;

   }





   /*------------------------------------------------------------*/
   /* getObjectColor
    */
   /**
    *    This method will return the color of a Symptom
    *  Flow Object.
    *  
    *
    *  @param objectName The String representation of the object
    *  @param None
    *  @param None
    *
    *  @return Color
    *//*---------------------------------------------------------*/
   private Color getObjectColor ( String objectName ) {
      if ( objectName.equals ( Str.getStr ( STR_NODE_COLOR ) ) )
         return( flowNodeColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_NODE_TEXT_COLOR ) ) )
         return( flowNodeTextColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_NODE_FOCUS_COLOR ) ) )
         return( flowNodeFocusColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_NODE_FOCUS_TEXT_COLOR ) ) )
         return( flowNodeFocusTextColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_GRID_COLOR ) ) )
         return( gridColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_ACTION_COLOR ) ) )
         return( flowNodeActionColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_EDGE_COLOR ) ) )
         return ( edgeColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_EDGE_FOCUS_COLOR ) ) ) 
         return ( edgeFocusColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_EDGE_RETURN_COLOR ) ) )
         return ( edgeReturnColor ) ;
      else if ( objectName.equals ( Str.getStr ( STR_FLOW_BACKGROUND_COLOR ) ) )
         return ( flowBackgroundColor ) ;
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when an item in
    *  our list has changed selection state
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {

      if ( !e.getValueIsAdjusting() )
         pb_OBJECT_COLOR.setBackground ( getObjectColor ( (String)lb_OBJECTS.getSelectedValue() ) ) ;

   } /* valueChanged  */


} /* GeneralPage */


