package com.ibm.nzna.projects.qit.avalon.editors.symptomProp ;
/*-----------------------------------------------------------*/
/*   SymptomPropertyDlg                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomPropertyDlg                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing. * ;
import java.awt.* ;           
import java.awt.event.* ;           
/**
 * This dialog holds a notebook with pages for each Symptom Property. 
 *
 *
 * @version 1.0, 1/23/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomPropertyDlg                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomPropertyDlg extends JDialog implements ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTabbedPane notebook        = null ;
   private DButton     pb_OK           = null ;
   private DButton     pb_CANCEL       = null ;
   private DButton     pb_APPLY        = null ;
   private ButtonPanel buttonPanel     = new ButtonPanel () ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Pages                                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private GeneralPage       generalPage     = null ;
   private FilterPage        filterPage      = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the
    *  application properties. It will not return
    *  until the user closes the window
    *
    *  @param parent The parent window
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomPropertyDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_PROPERTIES )+"-"+Str.getStr ( STR_SYMPTOM ), true ) ;

      setSize ( 400, 400 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      notebook  = new JTabbedPane () ;
      pb_OK     = new DButton ( ) ;
      pb_APPLY  = new DButton ( ) ;
      pb_CANCEL = new DButton ( ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the panels                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      notebook.addTab ( "df.", generalPage  = new GeneralPage() ) ;    
      notebook.addTab ( "df.", filterPage   = new FilterPage() ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_APPLY.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      getContentPane().add ( notebook ) ;
      getContentPane().add ( buttonPanel, BorderLayout.SOUTH ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_APPLY ) ;
      buttonPanel.add ( pb_CANCEL ) ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Make ourselves look like the user wants it              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;         


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, show ourselves                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setVisible ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on our
    *  buttons and tabs. It is up to the
    *  pages themselves to set their text
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      pb_OK.setText ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL.setText ( Str.getStr ( STR_CANCEL ) ) ;
      pb_APPLY.setText ( Str.getStr ( STR_APPLY ) ) ;

      notebook.setTitleAt ( notebook.indexOfComponent ( generalPage  ),  Str.getStr ( STR_GENERAL ) ) ;
      notebook.setTitleAt ( notebook.indexOfComponent ( filterPage ),    Str.getStr ( STR_FILTERS ) ) ;
   }



   
   
   
      /*------------------------------------------------------------*/
      /* doLayout
       */
      /**
       *    This method will lay out and position our
       *  controls
       *  
       *
       *  @param None
       *  @param None
       *  @param None
       *
       *  @return Nothing
       *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout () ;
      notebook.setBounds ( 0, 0, size.width, size.height-(rowHeight*3)-10 ) ;
      buttonPanel.setBounds ( size.width-300, size.height-(rowHeight*3)-5, 295, rowHeight+5 ) ;
   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one
    *  of our buttons is pressed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( e.getSource() == pb_OK ) || ( e.getSource() == pb_APPLY ) ) {
         if ( ( generalPage.saveProperties() ) &&
              ( filterPage.saveProperties() ) ) {
            if ( e.getSource() == pb_OK )
            dispose() ;
         }

      }

      else
         dispose() ;
   }




   /*------------------------------------------------------------*/
   /* dispose
    */
   /**
    *    This method is subclassed to remove
    *  ourselves from the PropertySystem listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void dispose () {

      generalPage.free() ;
      filterPage.free () ;

      super.dispose() ;
   }
} /* SymptomPropertyDlg */
