package com.ibm.nzna.projects.qit.avalon.gui ;    
/*-----------------------------------------------------------*/
/*   ConditionDlg                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ConditionDlg                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
/**
 * This dialog allows a user to create a new condition or edit an existing condition. 
 *
 *
 * @version 1.0, 4/28/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ConditionDlg                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class ConditionDlg extends JDialog implements SelectOAListener, Runnable, FocusListener, ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_READ_REFRESH      = "READ_REF" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_IMAGE          = null ;
   private JLabel       st_QUESTION       = null ;
   private JLabel       st_ANSWER         = null ;
   private JLabel       st_QUESTIONTITLE  = null ;
   private JLabel       st_OPERATOR       = null ;
   private JTextField   ef_QUESTION       = null ;
   private JComboBox    cb_ANSWERS        = null ;
   private JComboBox    cb_OPERATORS      = null ;
   private DButton      pb_BROWSE         = null ;
   private DButton      pb_OK             = null ;
   private DButton      pb_CANCEL         = null ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ConditionTerm            conditionTerm    = null ;
   private Question                 question         = null ;
   private boolean                  rc               = false ;
   private ImageIcon                wizardImage      = null ;
   private ConditionDlgListener     listener         = null ;
   private ConditionTermDisplayRec  displayRec       = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the
    *  window and show it. It will then immediatly return
        *
    *  @param parent The parent AppDefaultWin
    *  @param conditionTerm The ConditionTerm to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ConditionDlg ( AppDefaultWin parent ) {
      super ( parent, Str.getStr ( STR_CONDITION ), false ) ;

      initialize() ;
   } /* Constructor */

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the
    *  window and show it. It will then immediatly return. It will
    *  edit the ConditionTermDisplayRec passed in
    *
    *  @param parent The parent AppDefaultWin
    *  @param conditionTerm The ConditionTerm to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ConditionDlg ( AppDefaultWin parent, ConditionTermDisplayRec displayRec ) {
      super ( parent, Str.getStr ( STR_CONDITION ), false ) ;

      this.displayRec = displayRec ;
      initialize() ;

      setQuestion ( ((ConditionTerm)displayRec.getData()).getQuestionInd() ) ;
   } /* Constructor */



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      Container contentPane = getContentPane() ;
      Vector    opsVec      = new Vector ( 2 ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create a vector of operators                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      opsVec.addElement ( Str.getStr ( STR_NOT_AND ) ) ;
      opsVec.addElement ( Str.getStr ( STR_NOT_OR ) ) ;
      opsVec.addElement ( Str.getStr ( STR_AND_GROUP ) ) ;
      opsVec.addElement ( Str.getStr ( STR_OR_GROUP ) ) ;
      opsVec.addElement ( Str.getStr ( STR_AND ) ) ;
      opsVec.addElement ( Str.getStr ( STR_OR ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign Local Vars                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardImage = ImageSystem.getImageIcon ( this, ImageSystem.CONDITION_WIZARD ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI components                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_IMAGE         = new JLabel ( wizardImage ) ;
      st_QUESTIONTITLE = new JLabel ( "" ) ;
      st_QUESTION      = new JLabel ( Str.getStr ( STR_QUESTION ) ) ;
      st_ANSWER        = new JLabel ( Str.getStr ( STR_ANSWER ) ) ;
      st_OPERATOR      = new JLabel ( Str.getStr ( STR_OPERATOR ) ) ;
      ef_QUESTION      = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC ), "", 0 ) ;
      cb_ANSWERS       = new JComboBox () ;
      cb_OPERATORS     = new JComboBox ( opsVec ) ;
      pb_BROWSE        = new DButton ( Str.getStr ( STR_BROWSE ) ) ;
      pb_OK            = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL        = new DButton ( Str.getStr ( STR_CANCEL ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_OK ) ;
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;
      GUISystem.setPropertiesOnPanel ( contentPane );    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CANCEL.addActionListener   ( this ) ;
      pb_OK.addActionListener       ( this ) ;
      pb_BROWSE.addActionListener   ( this ) ;                       
      ef_QUESTION.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI to the content panel                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_QUESTION ) ;
      contentPane.add ( st_QUESTIONTITLE ) ;
      contentPane.add ( st_ANSWER ) ;
      contentPane.add ( st_OPERATOR ) ;
      contentPane.add ( ef_QUESTION ) ;
      contentPane.add ( pb_BROWSE ) ;
      contentPane.add ( cb_ANSWERS ) ;
      contentPane.add ( cb_OPERATORS ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_CANCEL ) ;        



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Center our window                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 495, 325 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;

   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an ActionEvent
    *  is fired on us
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL )
         dispose ( null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_BROWSE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_BROWSE ) {
         SelectOADlg dlg = new SelectOADlg ().getInstance() ;

         dlg.setView ( dlg.VIEW_QUESTION ) ;
         dlg.allowDrafts ( false ) ;
         dlg.addSelectOAListener ( this ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_OK ) {
         if ( saveData() )
            dispose ( conditionTerm ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ef_QUESTION                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ef_QUESTION )
         focusLost ( null ) ;


   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout (){
      Dimension size        = getSize() ;
      int       x           = wizardImage.getImage().getWidth(this)+10 ;
      int       y           = 5 ;
      int       rowHeight   = GUISystem.getRowHeight() ;
      int       titleHeight = 0 ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2), 
                           wizardImage.getImage().getWidth(this), 
                           wizardImage.getImage().getHeight(this)) ;



      st_QUESTION.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      ef_QUESTION.setBounds ( x, y, size.width-(x)-10-100, rowHeight ) ;
      pb_BROWSE.setBounds   ( size.width-90, y, 80, rowHeight ) ;
      y+=rowHeight+5 ;

      st_QUESTIONTITLE.setBounds ( x, y, size.width-(x)-110, rowHeight)  ;
      titleHeight = st_QUESTIONTITLE.getPreferredSize().height ;
      st_QUESTIONTITLE.setBounds ( x, y, size.width-(x)-110, titleHeight)  ;
      y+=titleHeight+5 ;


      st_ANSWER.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      cb_ANSWERS.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      st_OPERATOR.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      cb_OPERATORS.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      pb_OK.setLocation ( x, size.height-75 ) ;
      pb_CANCEL.setLocation ( x+pb_OK.getMinimumSize().width+5, size.height-75 ) ;

   }




   /*------------------------------------------------------------*/
   /* selectOAComplete
    */
   /**
    *    This method will be called when the user
    *  has made a selection in the SelectOADlg
    *  
    *
    *  @param vec The vector of objects that the user selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectOAComplete ( Vector vec ) {

      try {
         if ( ( vec != null ) && ( vec.size() > 0 ) ) {
            if ( vec.elementAt(0) instanceof QuestionDisplayRec )
               setQuestion ( ((QuestionDisplayRec)vec.elementAt(0)).getQuestionInd() ) ;
            else if ( vec.elementAt(0) instanceof Bookmark ) {
               Bookmark bookmark = (Bookmark) vec.elementAt(0 ) ;

               if ( bookmark.getBookmarkType() == BOOKMARK_QUESTION )
                  setQuestion ( ((Integer)bookmark.getObject()).intValue() ) ;
            }

         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* refreshQuestion
    */
   /**
    *    This method will refresh the data on our
    *  screen with the new data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshQuestion () {
      cb_ANSWERS.removeAllItems() ;

      if ( question != null ) {
         Vector answers         = question.getAnswers() ;
         int    selectAnswerInd = -1 ;
         Answer selectAnswer    = null ;                                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we were passed a displayRec, make sure               */
         /* That the answer that it represents is also selected       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( displayRec != null )
            selectAnswerInd = ((ConditionTerm)displayRec.getData()).getAnswerInd() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add all of the answers                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         for ( int i = 0; i < answers.size(); i++ ) {
            if ( ( selectAnswer == null ) && ( selectAnswerInd != -1 ) ) {
               if ( ((Answer)answers.elementAt(i)).getInd() == selectAnswerInd )
                  selectAnswer = (Answer)answers.elementAt(i) ;
            }

            cb_ANSWERS.addItem ( answers.elementAt(i) ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Select the answer if we found it                        */                       
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( selectAnswer != null )
            cb_ANSWERS.setSelectedItem ( selectAnswer ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Populate our fields                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         ef_QUESTION.setText ( ""+question.getInd() ) ;
         try {
            st_QUESTIONTITLE.setText ( AvalonUtil.getDefaultTitleObject ( question ).toString ( Title.HTML ) ) ;
         } catch ( Exception e ) {
            e.printStackTrace() ;
         }
         invalidate() ;
         doLayout() ;
         repaint() ;
      }
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform operations
    *  depending on the name of the thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      try {

         if ( operation.equals ( THREAD_READ_REFRESH ) ) {
            question = readQuestion ( question.getInd() ) ;
            refreshQuestion() ;
         }


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }


   }





   /*------------------------------------------------------------*/
   /* setConditionDlgListener
    */
   /**
    *    This method will set the listener that is notified
    *  of this dialog closing. Currently only 1 listener
    *  is supported
    *
    *  @param listener The listener to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setConditionDlgListener ( ConditionDlgListener listener ) {
      this.listener = listener ;
   }





   /*------------------------------------------------------------*/
   /* dispose
    */
   /**
    *    This method will dispose this window and
    *  will notify the listener of the status passed
    *  
    *
    *  @param conditionTerm The condition term to send to the listeners
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void dispose( ConditionTerm conditionTerm ) {
      super.dispose() ;
      if ( listener != null )
         listener.conditionDlgClosed ( conditionTerm, (displayRec == null) ) ;
   }




   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      boolean rc = false ;
      Answer  answer    = (Answer) cb_ANSWERS.getSelectedItem() ;
      String  operators = (String) cb_OPERATORS.getSelectedItem() ;

      if ( question != null ) {

         rc = true ;            
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, set the operator                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( operators.equals ( Str.getStr ( STR_AND ) ) )
            operators = "& " ;
         else if ( operators.equals ( Str.getStr ( STR_OR ) ) )
            operators = "| " ;
         else if ( operators.equals ( Str.getStr ( STR_NOT_AND ) ) )
            operators = "!&" ;
         else if ( operators.equals ( Str.getStr ( STR_NOT_OR ) ) )
            operators = "!|" ;
         else if ( operators.equals ( Str.getStr ( STR_AND_GROUP ) ) )
            operators = " &" ;
         else if ( operators.equals ( Str.getStr ( STR_OR_GROUP ) ) )
            operators = " |" ;
         else
            rc = false ;                  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the condition object                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         conditionTerm = new ConditionTerm ( question, answer, operators ) ;
         if ( displayRec != null )
            displayRec.setData ( conditionTerm ) ;
      } else
         rc = false ;


      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* answerFromInd
    */
   /**
    *    
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Answer answerFromInd ( int answerInd ) {
      Answer answer = null ;

      if ( question != null ) {
         int    i    = 0 ;
         Vector answers = question.getAnswers() ;
         int    length  = answers.size() ;

         while ( ( i < length ) && ( answer == null ) ) {
            if ( ((Answer)answers.elementAt(i)).getInd() == answerInd )
               answer = (Answer)answers.elementAt(i) ;
            i++ ;
         }
      }

      return( answer ) ;
   }





   /*------------------------------------------------------------*/
   /* readQuestion
    */
   /**
    *    This method will update our GUI and read
    *  in a Question
    *  
    *
    *  @param questionInd The Question Indicator to read in
    *  @param None
    *  @param None
    *
    *  @return Question
    *//*---------------------------------------------------------*/
   private Question readQuestion ( int questionInd ) {
      Question readQuestion = null ;

      st_QUESTION.setText ( Str.getStr ( STR_READING_QUESTION ) ) ;
      st_QUESTION.setForeground ( Color.blue ) ;
      pb_OK.setEnabled ( false ) ;
      pb_CANCEL.setEnabled ( false ) ;

      readQuestion = AvalonCache.readQuestion ( questionInd ) ;

      refreshQuestion() ;
      st_QUESTION.setText ( Str.getStr ( STR_QUESTION ) ) ;
      st_QUESTION.setForeground ( Color.black ) ;
      pb_OK.setEnabled ( true ) ;
      pb_CANCEL.setEnabled ( true ) ;

      return( readQuestion ) ;
   }




   /*------------------------------------------------------------*/
   /* setQuestion
    */
   /**
    *    This method will set our question from
    *  the indicator passed
    *  
    *
    *  @param questionInd
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setQuestion ( int questionInd ) {
      question = new Question ( questionInd ) ;

      new Thread ( this, THREAD_READ_REFRESH ).start() ;
   }/* setQuestion  */




   /*------------------------------------------------------------*/
   /* focusLost
    */
   /**
    *    This method will be called when the user
    *  tabs out of the ef_QUESITON field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusLost ( FocusEvent e ) {
      try {
         int questionInd = new Integer ( ef_QUESTION.getText() ).intValue() ;

         if ( questionInd > 0 )
            setQuestion ( questionInd ) ;
      } catch ( Exception e1 ) {
      }
   }




   /*------------------------------------------------------------*/
   /* focusGained
    */
   /**
    *    This method does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusGained ( FocusEvent e ) {
   }

} /* ConditionDlg */

