package com.ibm.nzna.projects.qit.avalon.gui ;     
/*-----------------------------------------------------------*/
/*   ConditionList                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ConditionList                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.pom.* ;
/**
 * This control holds a MultiList, an Up and Down button. This panel 
 * encompasses the entire maintenance of a Conditions object.
 *
 * @version 1.0, 5/19/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ConditionList                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class ConditionList extends JPanel implements ActionListener, ConditionDlgListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Condition MutliList titles                              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] condTitles = { 
      Str.getStr ( STR_OPERATOR ),
      Str.getStr ( STR_QUESTION ),
      Str.getStr ( STR_ANSWER )
   } ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle       st_TITLE       = null ;
   private MultiList    cnr_CONDITIONS = null ;
   private ButtonPanel  buttonPanel    = null ;
   private HotLinkLabel pb_UP          = null ;
   private HotLinkLabel pb_DOWN        = null ;      
   private HotLinkLabel pb_ADD         = null ;
   private HotLinkLabel pb_REMOVE      = null ;
   private HotLinkLabel pb_EDIT        = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Conditions   conditions     = null ;



   public ConditionList ( ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE = new JTitle ( Str.getStr ( STR_CONDITIONS ) ) ;
      cnr_CONDITIONS = new MultiList ( GUISystem.getFontUtil() ) ;
      buttonPanel    = new ButtonPanel () ;
      pb_UP          = new HotLinkLabel ( Str.getStr ( STR_UP ) ) ;
      pb_DOWN        = new HotLinkLabel ( Str.getStr ( STR_DOWN ) ) ;        
      pb_ADD         = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE      = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_EDIT        = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_UP.setOpaque ( false ) ;
      pb_DOWN.setOpaque ( false ) ;
      pb_EDIT.setOpaque ( false ) ;
      st_TITLE.setFont ( FontSystem.smallTitleFont ) ;
      buttonPanel.setBorder     ( GUISystem.grayBorder ) ;
      buttonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      setOpaque ( false ) ;  
      cnr_CONDITIONS.setBorder  ( GUISystem.blackBorder ) ;
      cnr_CONDITIONS.setColumnHeadings ( condTitles ) ;
      cnr_CONDITIONS.setColumnWidth ( 0, 50 ) ;
      cnr_CONDITIONS.setColumnWidth ( 1, 150 ) ;
      cnr_CONDITIONS.setColumnWidth ( 2, 150 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_UP.addActionListener     ( this ) ;
      pb_DOWN.addActionListener   ( this ) ;
      pb_ADD.addActionListener    ( this ) ;
      pb_REMOVE.addActionListener ( this ) ;
      pb_EDIT.addActionListener   ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_TITLE ) ;
      add ( cnr_CONDITIONS ) ;
      add ( buttonPanel ) ;
      buttonPanel.add ( pb_ADD ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_EDIT ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_REMOVE ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_UP ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
      buttonPanel.add ( pb_DOWN ) ;

   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 0 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      st_TITLE.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      cnr_CONDITIONS.setBounds ( x, y, size.width, size.height-(rowHeight*2)-5 ) ;
      y+=(size.height-(rowHeight*2))-3 ;

      buttonPanel.setBounds ( x, y, size.width, rowHeight ) ;
   }





   /*------------------------------------------------------------*/
   /* setConditions
    */
   /**
    *    This method will set the Conditions that
    *  are being displayed here
    *  
    *
    *  @param conditions The conditions to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setConditions ( Conditions conditions ) {
      this.conditions = conditions ;
      refreshList() ;
   }





   /*------------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh our list of conditions
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshList () {              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remove everything first                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_CONDITIONS.removeAll() ;                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the condition terms and add them                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( conditions != null ) {
         Vector     terms      = ConditionTermDisplayRec.convert ( conditions.getConditionTerms() ) ;         

         if ( terms != null )
            cnr_CONDITIONS.add ( terms ) ;
      }
   } /* refreshList  */




   /*------------------------------------------------------------*/
   /* saveConditions
    */
   /**
    *    This method will save the conditions object  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Conditions saveConditions () {
      if ( conditions != null ) {
         Vector      conditionTermVec = QuestUtil.getDataFromDisplay ( cnr_CONDITIONS.getData() ) ;

         conditions.clear() ;
         if ( conditionTermVec != null )
            for ( int i = 0; i < conditionTermVec.size(); i++ )
               conditions.addTerm ( (ConditionTerm)conditionTermVec.elementAt(i) ) ;

      }
      return( conditions ) ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to action events 
    *  being fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_UP                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_UP )
         moveConditionInList ( (ConditionTermDisplayRec)cnr_CONDITIONS.getSelectedItem(), false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_DOWN                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DOWN )
         moveConditionInList ( (ConditionTermDisplayRec)cnr_CONDITIONS.getSelectedItem(), true ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD )
         addCondition() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE )
         removeCondition() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT )
         editCondition() ;


   }





   /*------------------------------------------------------------*/
   /* addCondition
    */
   /**
    *    This method will add a condition to the list. This
    *  addition process asks the user for the condition
    *  and then will add the condition automagically.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCondition () {
      if ( conditions != null )
         new ConditionDlg ( GUISystem.getParentDefWin ( this ) ).setConditionDlgListener ( this ) ;
   }






   /*------------------------------------------------------------*/
   /* removeCondition
    */
   /**
    *    This method will simply remove all selected
    *  conditions
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCondition () {
      Vector selectionVec = cnr_CONDITIONS.getSelection() ;

      if ( ( selectionVec != null ) && ( selectionVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = selectionVec.size() ;

         while ( i < size ) {
            removeCondition ( (ConditionTermDisplayRec)selectionVec.elementAt(i) ) ;
            i++ ;
         }
      }
   } /* removeCondition  */





   /*------------------------------------------------------------*/
   /* editCondition
    */
   /**
    *    This method will edit the selected condition
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editCondition () {
      Vector selectionVec = cnr_CONDITIONS.getSelection() ;

      if ( ( selectionVec != null ) && ( selectionVec.size() > 0 ) )
         new ConditionDlg ( GUISystem.getParentDefWin ( this ),
                            (ConditionTermDisplayRec)selectionVec.elementAt(0) ).setConditionDlgListener ( this ) ;
      else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;
   }



   /*------------------------------------------------------------*/
   /* removeCondition
    */
   /**
    *    This method will remove the ConditionTermDisplayRec
    *  passed in from the list of conditions and will
    *  take the ConditionTermRec from the Conditions
    *
    *  @param conditionTermDisplayRec The ConditionTermDisplayRec to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void removeCondition ( ConditionTermDisplayRec conditionTermDisplayRec ) {

      if ( conditionTermDisplayRec != null )
         cnr_CONDITIONS.remove ( conditionTermDisplayRec ) ;
   }


   /*------------------------------------------------------------*/
   /* conditionDlgClosed
    */
   /**
    *    This method will be called when a condition
    *  dialog that we have opened is now closed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void conditionDlgClosed ( ConditionTerm conditionTerm, boolean added ) {
      if ( conditionTerm != null ) {

         if ( added ) {
            conditions.addTerm ( conditionTerm ) ;
            cnr_CONDITIONS.add ( new ConditionTermDisplayRec ( conditionTerm ) ) ;
         } else
            revalidate() ;
      }
   }


   /*------------------------------------------------------------*/
   /* moveConditionFromList
    */
   /**
    *    This method will simply move the condition
    *  in the container. This will not have an effect
    *  of the title object yet
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The position it was at before it was moved
    *//*---------------------------------------------------------*/
   public void moveConditionInList ( ConditionTermDisplayRec displayRec, boolean down ) {
      Vector conditions = cnr_CONDITIONS.getData() ;
      int    rc         = conditions.indexOf ( displayRec ) ;

      if ( down )
         rc++ ;
      else
         rc-- ;

      if ( ( rc >= 0 ) && ( rc < conditions.size() ) ) {
         conditions.removeElement ( displayRec ) ;
         cnr_CONDITIONS.removeAll() ;
         conditions.insertElementAt ( displayRec, rc ) ;
         cnr_CONDITIONS.add ( conditions ) ;
         cnr_CONDITIONS.setSelectedItem ( displayRec, true ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* clear
    */
   /**
    *    This method will clear the list and all
    *  conditions
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void clear () {
      cnr_CONDITIONS.removeAll() ;
      conditions = null ;
   }




   /*------------------------------------------------------------*/
   /* getConditions
    */
   /**
    *    This method will save the conditions as
    *  the user has them and returns the Conditions object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Conditions getConditions () {
      return( conditions ) ;
   }





   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable the controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enabled ) {
      cnr_CONDITIONS.setEnabled ( enabled ) ;
      pb_UP.setEnabled ( enabled ) ;
      pb_DOWN.setEnabled ( enabled ) ;
      pb_ADD.setEnabled ( enabled ) ;
      pb_REMOVE.setEnabled ( enabled ) ;
      pb_EDIT.setEnabled ( enabled ) ;
   }
} /* ConditionList */
