package com.ibm.nzna.projects.qit.avalon.gui ;         
/*-----------------------------------------------------------*/
/*   CreateObjectDlg                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CreateObjectDlg                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog is used when creating new Objects in Avalon. This dialog 
 * has 3 basic buttons. New, Existing and Cancel.  Here are the
 * return values based on what the user chooses:<br><pre>
 *
 * Button         Return Value
 * -------------- ---------------------------------
 * New            NEW
 * Existing       EXISTING
 * Cancel         CANCEL<br></pre>
 *
 * The constructor will not show the window. Please call 'getResult' to show
 * the window and get what the user chose.
 *
 *
 * @version 1.0, 4/20/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CreateObjectDlg                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class CreateObjectDlg extends JDialog implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int CANCEL      = 0 ;
   public static final int NEW         = 1 ;
   public static final int EXISTING    = 2 ;                  
   public static final int BROWSE      = 3 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private, GUI Members                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton      pb_NEW         = null ;
   private DButton      pb_BROWSE      = null ;
   private DButton      pb_CANCEL      = null ;
   private ButtonPanel  buttonPanel    = null ;
   private JLabel       st_IMAGE       = null ;
   private JLabel       st_EXISTING    = null ;
   private JTextField   ef_EXISTING    = null ;
   private DButton      pb_GO          = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int rc = CANCEL ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the window but
    *  will immediatly return. Please call 'getResult'
    *  to show the window and wait for user response
    *
    *  @param parent The parent of this dialog
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CreateObjectDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_NEW ), true ) ;

      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the user and waits
    *  until the user responds. Once the user responds
    *  a return value will be returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getResult () {
      setVisible ( true ) ;
      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will completely initialize this window.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_IMAGE    = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.LARGE_NEWOBJECT ) ) ;
      pb_NEW      = new DButton ( Str.getStr ( STR_NEW ) ) ;
      pb_BROWSE   = new DButton ( Str.getStr ( STR_BROWSE ) ) ;
      pb_CANCEL   = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      buttonPanel = new ButtonPanel () ;
      st_EXISTING = new JLabel ( Str.getStr ( STR_EXISTING ) ) ;
      ef_EXISTING = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 10 ), "", 0 ) ;
      pb_GO       = new DButton ( Str.getStr ( STR_GO ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;
      buttonPanel.setArrowImage  ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                   ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_NEW.addActionListener      ( this ) ;
      pb_CANCEL.addActionListener   ( this ) ;
      pb_BROWSE.addActionListener   ( this ) ;  
      ef_EXISTING.addActionListener ( this ) ;
      pb_GO.addActionListener       ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      getContentPane().add ( st_IMAGE ) ;
      getContentPane().add ( st_EXISTING ) ;
      getContentPane().add ( ef_EXISTING ) ;
      getContentPane().add ( pb_GO ) ;
      getContentPane().add ( pb_NEW ) ;
      getContentPane().add ( pb_BROWSE ) ;
      getContentPane().add ( buttonPanel ) ;
      buttonPanel.add ( pb_CANCEL ) ;

      setResizable ( false ) ;
      setSize ( 300, 150 ) ;
      WinUtil.centerChildInParent ( this, getParent() ) ;
   } /* intialize  */





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       y         = 10 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout () ;

      st_IMAGE.setBounds ( 5, 10, 75, 75 ) ;

      st_EXISTING.setBounds ( 100, y, size.width-110, rowHeight ) ;
      y+=rowHeight ;

      ef_EXISTING.setBounds ( 100, y, size.width-165, rowHeight ) ;
      pb_GO.setBounds ( size.width-55, y, 45, rowHeight ) ;
      y+=rowHeight+5 ;

      pb_NEW.setBounds ( 100, y, size.width-110, 25 ) ;
      y+=28 ;

      pb_BROWSE.setBounds ( 100, y, size.width-110, 25 ) ;
      buttonPanel.setBounds ( 5, size.height-50, size.width-10, 25 ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_NEW )
         rc = NEW ;
      else if ( e.getSource() == pb_BROWSE )
         rc = BROWSE ;
      else if ( ( e.getSource() == ef_EXISTING ) ||
                ( e.getSource() == pb_GO ) )
         rc = EXISTING ;

      dispose() ;
   }





   /*------------------------------------------------------------*/
   /* getExistingInd
    */
   /**
    *    This method will return the existing indicator
    *  that the user has entered
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getExistingInd () {
      try {
         return( new Integer ( ef_EXISTING.getText() ).intValue() ) ;
      } catch ( Exception e ) {
      }
      return( -1 ) ;
   }
} /* CreateObjectDlg */
