package com.ibm.nzna.projects.qit.avalon.gui ;                                   
/*-----------------------------------------------------------*/
/*   NewAvalonObjectLinkTargetStep                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NewAvalonObjectLinkTargetStep       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.db.* ;        
import com.ibm.nzna.shared.gui.* ;        
import com.ibm.nzna.shared.util.* ;        
import com.ibm.nzna.shared.gui.wizard.* ;
/**
 * This step of the NewAvalonObjectWizard asks the user for a link target
 *
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NewAvalonObjectLinkTargetStep                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class NewAvalonObjectLinkTargetStep extends WizardStep implements Runnable, SelectDocListener, ActionListener, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_GET_DOC_TITLE      ="GETDOCTITLE" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_HELP_TEXT      = null ;
   private JTextField   ef_LINK           = null ;
   private DButton      pb_BROWSE         = null ;
   private Wizard       wizard            = null ;
   private int          type              = 0 ;
   private JLabel       st_VALID          = null ;
   private DButton      pb_VALIDATE       = null ;
   private String       title             = null ;
   private boolean      valid             = false ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our step
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NewAvalonObjectLinkTargetStep () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP_TEXT = new JLabel ( Str.getStr ( STR_NEW_AVALON_OBJECT_TITLE_HELP ) ) ;
      ef_LINK      = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_OA_TITLE ), "", 0 ) ;               
      pb_BROWSE    = new DButton ( Str.getStr ( STR_BROWSE ) ) ;
      st_VALID     = new JLabel ( "" ) ;
      pb_VALIDATE  = new DButton ( Str.getStr ( STR_VALIDATE ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP_TEXT.setVerticalAlignment ( SwingConstants.TOP ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_LINK.addActionListener   ( this ) ;
      pb_BROWSE.addActionListener ( this ) ;
      pb_VALIDATE.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP_TEXT ) ;
      add ( ef_LINK ) ;
      add ( pb_BROWSE ) ;
      add ( pb_VALIDATE ) ;
      add ( st_VALID ) ;

      setValid ( false ) ;
   }

   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      st_HELP_TEXT.setBounds ( 0, 0, size.width, size.height/2 ) ;
      ef_LINK.setBounds      ( 15, (size.height/2)+2, size.width-155, rowHeight ) ;
      pb_BROWSE.setBounds    ( size.width-135, (size.height/2)+2, 50, rowHeight ) ;
      pb_VALIDATE.setBounds  ( size.width-75, (size.height/2)+2, 50, rowHeight );
      st_VALID.setBounds     ( 15, ((size.height/2)+2)+rowHeight, size.width-60, rowHeight ) ;
   }




   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save the information on our
    *  panel. We will not allow the user to continue
    *  if they have not entered any title
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      ef_LINK.requestFocus() ;
      if ( ( title == null ) && ( type == 0 ) ) {
         return( false ) ;
      }

      return( ( valid ) && ( ef_LINK.getText() != null ) && ( ef_LINK.getText().length() > 0 ) ) ;
   }




   /*------------------------------------------------------------*/
   /* refreshInfo
    */
   /**
    *    This method will refresh our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshInfo () {
      ef_LINK.requestFocus() ;
   }




   /*------------------------------------------------------------*/
   /* getTarget
    */
   /**
    *    this method will return the target that the user
    *  had entered
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTarget () {
      return( ef_LINK.getText() ) ;
   }




   /*------------------------------------------------------------*/
   /* setWizard
    */
   /**
    *    This method will be called when our wizard
    *  has been set. We keep track of it so it the user
    *  hits 'enter' on the command line, we process the next panel
    *
    *  @param wizard Our wizard
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWizard ( Wizard wizard ) {
      this.wizard = wizard ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when the user
    *  hits enter in the entry field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( e.getSource() == ef_LINK ) ||
           ( e.getSource() == pb_VALIDATE ) ) {
         switch ( type ) {
            case 0 :
               st_VALID.setForeground ( Color.blue ) ;
               st_VALID.setText ( Str.getStr ( STR_VALIDATING ) ) ;
               title = null ;
               setValid ( false ) ;
               new Thread ( this, THREAD_GET_DOC_TITLE ).start() ;
               break ;

            case 1 :
               setValid ( validateURL ( ef_LINK.getText() ) ) ;
               break ;
         }
      }

      else if ( e.getSource() == pb_BROWSE ) {
         if ( type == 0 )
            new SelectDocDlg ( false ).addSelectDocListener ( this )  ;
      }
   }




   /*------------------------------------------------------------*/
   /* setType
    */
   /**
    *    This method will set the display of this
    *  panel based on the link type.
    *  
    *
    *  @param type The Link Type 0 = Quest Link, 1 = External Link
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setType ( int type ) {
      this.type = type ;

      switch ( type ) {
         case 0 :
            st_HELP_TEXT.setText ( Str.getStr ( STR_NEW_AVALON_WIZARD_LINK_TARGET_QUEST ) ) ;
            pb_BROWSE.setEnabled ( true ) ;
            pb_VALIDATE.setEnabled ( true ) ;
            ef_LINK.setText ( "" ) ;
            ef_LINK.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 8 ) ) ;
            break ;

         case 1 :
            st_HELP_TEXT.setText ( Str.getStr ( STR_NEW_AVALON_WIZARD_LINK_TARGET_EXTERNAL ) ) ;
            pb_BROWSE.setEnabled ( false ) ;
            pb_VALIDATE.setEnabled ( true ) ;
            ef_LINK.setText ( "" ) ;
            ef_LINK.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 1024 ) ) ;
            break ;
      }
   }





   /*------------------------------------------------------------*/
   /* validateURL
    */
   /**
    *    This method will validate that the URL
    *  in the ef_URL field is valid
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean validateURL ( String url ) {
      boolean valid = false ;

      pb_VALIDATE.setEnabled ( false ) ;
      Qit.setEnabled ( false ) ;
      GUISystem.getParentDefWin ( this ).setStatus ( STR_PLEASE_WAIT ) ;
      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, try and load the URL                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         title = URLValidator.getTitle ( url ) ;                                            
         valid = (title != null) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have the URL, now see if it's already in the db   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( valid ) {
            valid = !checkDupURL ( url ) ;
            if ( !valid )
               GUISystem.printBox ( STR_INFO, STR_DOC_LINK_ALREADY_EXISTS ) ;
         }


      } catch ( Exception e ) {
         String title = e.toString() ;

         if ( title.indexOf ( "Operation timed out" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_URL_TIMED_OUT ) ;

         else if ( title.indexOf ( "UnknownHostException" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_URL_NOT_FOUND ) ;

         else if ( title.indexOf ( "MalformedURLException: no protocol" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_URL_NO_PROTOCOL ) ;

         else if ( title.indexOf ( "ConnectException: Connection refused" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_URL_CONNECT_REFUSED ) ;
      }

      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;
      Qit.setEnabled ( true ) ;
      pb_VALIDATE.setEnabled ( true ) ;

      return( valid ) ;
   }





   /*------------------------------------------------------------*/
   /* getDocTitle
    */
   /**
    *    This method will validate that the Doc Indicator
    *  that the user has entered in the ef_QUEST_LINK
    *  field is a valid published document and will return the title of the doc
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getDocTitle ( int docInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocTitle", 5 ) ;
      String    retTitle  = null ;

      ef_LINK.setEnabled ( false ) ;
      pb_BROWSE.setEnabled ( false ) ;
      title = null ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT TITLE, RECYCLED, PUBLISH, DBUSER, CHANGEDTIME "+
                                              "FROM TIGRIS.DOCUMENTS "+
                                              "WHERE DOCIND = "+docInd+" FOR FETCH ONLY" ) ;

         if ( rs.next() ) {
            String recycled    = rs.getString(2).trim() ;
            String publish     = rs.getString(3).trim() ;
            String dbUser      = UserSystem.getNameFromUserId ( rs.getString(4).trim() ) ;
            String changedTime = DateSystem.prettyDateFromStamp ( rs.getString(5).trim() ) ;

            retTitle = rs.getString(1).trim() ;

            if ( ( recycled.equals ( "Y" ) ) ||
                 ( publish.equals ( "N" ) ) ) {
               retTitle =  null ;
               GUISystem.printBox ( GUISystem.getParentDefWin ( this ), 
                                    Str.getStr ( STR_INFO ), Str.getStr ( STR_DOCUMENT_DELETED_BY )+" "+
                                    dbUser+" "+
                                    Str.getStr ( STR_ON )+" "+
                                    changedTime ) ;
            }
         } else
            GUISystem.printBox ( this, STR_INFO, STR_DOCUMENT_NOT_FOUND ) ;

         rs.close() ;
         stmt.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;
      pb_BROWSE.setEnabled ( true ) ;
      ef_LINK.setEnabled ( true ) ;

      return( retTitle ) ;
   }




   /*------------------------------------------------------------*/
   /* selectDocComplete
    */
   /**
    *    This method will be called when the user
    *  has selected something from the SelectDocDlg.
    *  
    *
    *  @param vec What the user has selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectDocComplete ( Vector vec ) {
      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         Object object = vec.elementAt(0) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   DocRec                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( object instanceof DocRec ) {
            ef_LINK.setText ( ""+((DocRec)object).getExistingDocInd() ) ;
            title = null ;
            new Thread ( this, THREAD_GET_DOC_TITLE ).start() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Bookmark of DocRec                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( object instanceof Bookmark ) {
            Bookmark bookmark = (Bookmark) object ;

            if ( bookmark.getBookmarkType() == BOOKMARK_DOCUMENT ) {
               ef_LINK.setText ( ""+((Integer)bookmark.getObject()).intValue() ) ;
               title = null ;
               new Thread ( this, THREAD_GET_DOC_TITLE ).start() ;
            }
         }
      }
   }





   /*------------------------------------------------------------*/
   /* getLinkTitle
    */
   /**
    *    This method returns the title of the QuestLink
    *  if we have one
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getLinkTitle () {
      return( title ) ;
   }




   /*------------------------------------------------------------*/
   /* getValid
    */
   /**
    *    This method will return TRUE or FALSE if
    *  the URL entered was valid or invalid
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getValid () {
      return( valid ) ;
   }





   /*------------------------------------------------------------*/
   /* setValid
    */
   /**
    *    This method will set our valid/invalid
    *  flag
    *  
    *
    *  @param valid TRUE if valid, FALSE if not valid
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setValid ( boolean valid ) {
      this.valid = valid ;

      if ( valid ) {
         st_VALID.setText ( Str.getStr ( STR_VALID ) ) ;
         st_VALID.setForeground ( Color.blue ) ;
      } else {
         st_VALID.setText ( Str.getStr ( STR_INVALID ) ) ;
         st_VALID.setForeground ( Color.red ) ;

      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Automatically send the user to the next panel           */                                    
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( valid ) {
         wizard.processNext() ;
      }

   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our threads
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_GET_DOC_TITLE                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_GET_DOC_TITLE ) ) {
         String docIndStr = ef_LINK.getText() ;

         if ( ( docIndStr != null ) && ( docIndStr.length() > 0 ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   First, get the title of the document                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            title = getDocTitle ( new Integer ( docIndStr ).intValue() ) ;                                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, if the title is good, check to make sure           */
            /* there are no other links with this docInd                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( title != null ) {

               if ( getLinkInd ( docIndStr ) == 0 )
                  setValid ( true ) ;
               else {
                  setValid ( false ) ;
                  title = null ;
                  GUISystem.printBox ( STR_INFO, STR_DOC_LINK_ALREADY_EXISTS ) ;
               } /* else */

            } /* if */

         } /* if docIndStr */

      } /* if THREAD_GET_DOC_TITLE */
   }





   /*------------------------------------------------------------*/
   /* getLinkInd
    */
   /**
    *    This method will return the LinkInd that matches
    *  the DocLink passed in
    *  
    *
    *  @param docIndStr The String representation of the DocInd
    *  @param None
    *  @param None
    *
    *  @return linkInd The integer Link Indicator
    *//*---------------------------------------------------------*/
   private int getLinkInd ( String docIndStr ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getLinkInd ( "+docIndStr+")", 5 ) ;
      int       linkInd   = -1 ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT LINKIND FROM OA.LINK WHERE DOCIND = "+docIndStr+" FOR FETCH ONLY" ) ;

            if ( rs.next() )
               linkInd = rs.getInt(1) ;
            else
               linkInd = 0 ;

            rs.close() ;
         }
      } catch ( Exception e ) {
         sqlMethod.close() ;
         LogSystem.log ( 1, e ,false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;

      }

      sqlMethod.close() ;

      return( linkInd ) ;
   }


   public String getTitle () {
      return( Str.getStr ( STR_LINK_DESTINATION ) ) ;
   }





   /*------------------------------------------------------------*/
   /* checkDupURL
    */
   /**
    *    This method will attempt to find a duplicate
    *  URL in the databsae for the URL passed in
    *  
    *
    *  @param url The URL to look for
    *  @param None
    *  @param None
    *
    *  @return TRUE if there already is an entry in the database for the URL passed in
    *//*---------------------------------------------------------*/
   private boolean checkDupURL ( String url ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "checkDupURL ( "+url+" )", 5 ) ;
      boolean   rc        = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT LINKIND FROM OA.LINK WHERE translate(URL) = '"+url.toUpperCase()+"' FOR FETCH ONLY" ) ;

            rc = rs.next() ;

            rs.close() ;
            stmt.close() ;

            /*-----------------------------------------------------------------*/
            /*                                                                 */
            /*   Ok, if we haven't found it yet, now look through the drafts   */
            /*                                                                 */
            /*-----------------------------------------------------------------*/
            if ( !rc ) {
               Statement stmt2 = sqlMethod.createStatement() ;
               ResultSet rs2   = stmt2.executeQuery ( "SELECT LINKIND FROM OADRAFT.LINK "+
                                                      "WHERE RECYCLED = 'N' AND "+
                                                      "      STOPDATE IS NULL AND "+
                                                      "      translate(URL) = '"+url.toUpperCase()+"' FOR FETCH ONLY" ) ;

               rc = rs2.next() ;

               rs2.close() ;
            }

         } else
            rc = true ;

      } catch ( Exception e ) {
         rc = true ;
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
      return( rc ) ;
   }
} /* NewAvalonObjectLinkTargetStep */


