package com.ibm.nzna.projects.qit.avalon.gui ;                                   
/*-----------------------------------------------------------*/
/*   NewAvalonObjectLinkTypeStep                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NewAvalonObjectLinkTypeStep         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.gui.* ;        
import com.ibm.nzna.shared.gui.wizard.* ;
/**
 * This step of the NewAvalonObjectWizard asks the user for a Link Type
 *
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NewAvalonObjectLinkTypeStep                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class NewAvalonObjectLinkTypeStep extends WizardStep implements ActionListener, AvalonConst, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_HELP_TEXT      = null ;
   private JList        lb_LIST           = null ;
   private Wizard       wizard            = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our step
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NewAvalonObjectLinkTypeStep () {  
      Vector typeVec = new Vector ( 2 ) ;

      typeVec.addElement ( Str.getStr ( STR_QUEST_LINK ) ) ;
      typeVec.addElement ( Str.getStr ( STR_EXTERNAL_LINK ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP_TEXT = new JLabel ( Str.getStr ( STR_NEW_AVALON_WIZARD_LINK_TYPE ) ) ;
      lb_LIST      = new JList ( ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP_TEXT.setVerticalAlignment ( SwingConstants.TOP ) ;
      lb_LIST.setBorder ( GUISystem.loweredBorder ) ;
      lb_LIST.setListData ( typeVec ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP_TEXT ) ;
      add ( lb_LIST ) ;

   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;

      st_HELP_TEXT.setBounds ( 0, 0, size.width, size.height/2 ) ;
      lb_LIST.setBounds ( 15, (size.height/2)+2, size.width-30, 100 ) ;
   }




   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save the information on our
    *  panel. We will not allow the user to continue
    *  if they have not entered any title
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      String type = (String) lb_LIST.getSelectedValue() ;

      lb_LIST.requestFocus() ;
      return( type != null ) ;
   }




   /*------------------------------------------------------------*/
   /* refreshInfo
    */
   /**
    *    This method will refresh our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshInfo () {
      lb_LIST.requestFocus() ;
   }




   /*------------------------------------------------------------*/
   /* getType
    */
   /**
    *    this method will return the title that the user
    *  had entered
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getType () {
      return( (String)lb_LIST.getSelectedValue() ) ;
   }




   /*------------------------------------------------------------*/
   /* setWizard
    */
   /**
    *    This method will be called when our wizard
    *  has been set. We keep track of it so it the user
    *  hits 'enter' on the command line, we process the next panel
    *
    *  @param wizard Our wizard
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWizard ( Wizard wizard ) {
      this.wizard = wizard ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when the user
    *  hits enter in the entry field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      wizard.processNext() ;
   }




   /*------------------------------------------------------------*/
   /* getIntType
    */
   /**
    *    This method will return the integer type
    *  0 = QuestLink, 1 = External Link
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getIntType () {
      String strType = (String)lb_LIST.getSelectedValue() ;
      int    retType = -1 ;

      if ( strType != null ) {
         if ( strType.equals ( Str.getStr ( STR_QUEST_LINK ) ) )
            retType = 0 ;
         else
            retType = 1 ;
      }
      return( retType ) ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    Returns our title
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_LINK_TYPE_TITLE ) ) ;
   }
} /* NewAvalonObjectLinkTypeStep */
