package com.ibm.nzna.projects.qit.avalon.gui ;                                   
/*-----------------------------------------------------------*/
/*   NewAvalonObjectTitleStep                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NewAvalonObjectTitleStep            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.gui.* ;        
import com.ibm.nzna.shared.gui.wizard.* ;
/**
 * This step of the NewAvalonObjectWizard asks the user for a default title 
 *
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NewAvalonObjectTitleStep                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class NewAvalonObjectTitleStep extends WizardStep implements ActionListener, AvalonConst, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel          st_HELP_TEXT      = null ;
   private JTextField      ef_TITLE          = null ;
   private SpellCheckPanel spellCheckPanel   = null ;
   private Wizard          wizard            = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our step
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NewAvalonObjectTitleStep () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP_TEXT    = new JLabel ( Str.getStr ( STR_NEW_AVALON_OBJECT_TITLE_HELP ) ) ;
      ef_TITLE        = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_OA_TITLE ), "", 0 ) ;               
      spellCheckPanel = new SpellCheckPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP_TEXT.setVerticalAlignment ( SwingConstants.TOP ) ;    
      spellCheckPanel.setTextComponent ( ef_TITLE ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_TITLE.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP_TEXT ) ;
      add ( ef_TITLE ) ;
      add ( spellCheckPanel ) ;

   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      st_HELP_TEXT.setBounds ( 0, 0, size.width, rowHeight*3 ) ;
      spellCheckPanel.setBounds ( 15, (rowHeight*3)+5, size.width-30, rowHeight ) ;
      spellCheckPanel.setBounds ( 15, (rowHeight*3)+5, spellCheckPanel.getPreferredSize().width, spellCheckPanel.getPreferredSize().height ) ;
      ef_TITLE.setBounds ( 15, (rowHeight*4)+7, size.width-30, rowHeight ) ;
   }




   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save the information on our
    *  panel. We will not allow the user to continue
    *  if they have not entered any title
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      ef_TITLE.requestFocus() ;
      return( ( ef_TITLE.getText() != null ) && ( ef_TITLE.getText().length() > 0 ) ) ;
   }




   /*------------------------------------------------------------*/
   /* refreshInfo
    */
   /**
    *    This method will refresh our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshInfo () {
      ef_TITLE.requestFocus() ;
   }




   /*------------------------------------------------------------*/
   /* getObjectTitle
    */
   /**
    *    this method will return the title that the user
    *  had entered
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getObjectTitle () {
      return( ef_TITLE.getText() ) ;
   }




   /*------------------------------------------------------------*/
   /* setWizard
    */
   /**
    *    This method will be called when our wizard
    *  has been set. We keep track of it so it the user
    *  hits 'enter' on the command line, we process the next panel
    *
    *  @param wizard Our wizard
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWizard ( Wizard wizard ) {
      this.wizard = wizard ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when the user
    *  hits enter in the entry field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      wizard.processNext() ;
   }




   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    this method will set the text that is displayed
    *  in the title field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setTitle ( String title ) {
      ef_TITLE.setText ( title ) ;
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title that the
    *  wizard will use
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_OBJECT_TITLE_WIZARD_TITLE ) ) ;
   }

} /* NewAvalonObjectTitleStep */
