package com.ibm.nzna.projects.qit.avalon.gui ;                  
/*-----------------------------------------------------------*/
/*   NewAvalonObjectWizard                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NewAvalonObjectWizard               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;        
import com.ibm.nzna.shared.util.* ;        
import com.ibm.nzna.shared.gui.wizard.* ;
/**
 * This wizard is designed to allow the step through creation of 
 * the following types of Avalon Objects:<br><pre>
 *
 * Question, Action, Symptom<br></pre>
 * 
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NewAvalonObjectWizard                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class NewAvalonObjectWizard extends JDialog implements WizardListener, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int           objectType        = 0 ;
   private Wizard        wizardPanel       = null ;
   private WizardStep[]  steps             = null ;
   private boolean       cancelled         = true ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the Wizard.
    *  It will not return until the wizard is complete
    *  
    *
    *  @param parent The parent window of the wizard
    *  @param objectType The type of object ( AVALON_QUESTION, AVALON_SYMPTOM or AVALON_ACTION )
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NewAvalonObjectWizard ( AppDefaultWin parent, int objectType ) {
      super ( parent, Str.getStr ( STR_NEW ), true ) ;                                  
      CreateWizardWin statusWin = new CreateWizardWin ( parent, 7 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we were passed a valid object type, init             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( objectType == AVALON_QUESTION ) ||
           ( objectType == AVALON_SYMPTOM ) ||
           ( objectType == AVALON_LINK_GROUP ) ||
           ( objectType == AVALON_LINK ) ||
           ( objectType == AVALON_ACTION ) ) {

         this.objectType = objectType ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our Wizard control                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            wizardPanel = new Wizard ( ImageSystem.getImageIcon ( this, ImageSystem.AVALON_WIZARD ) ) ;                     
            wizardPanel.addWizardListener ( this ) ;
         } catch ( Exception e ) {
            wizardPanel = new Wizard () ;
         }

         statusWin.setValue ( 1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our Wizard                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         getContentPane().setLayout ( new BorderLayout() ) ;
         getContentPane().add ( wizardPanel, BorderLayout.CENTER ) ;                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the steps in the wizard                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         statusWin.setValue ( 2 ) ;
         createWizardSteps ( statusWin ) ;           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Start our wizard off                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         wizardPanel.setCurrentStep ( steps[0] ) ;

         setSize ( 495, 375 ) ;
         WinUtil.centerWindow ( this ) ;
         statusWin.dispose() ;
         setVisible ( true ) ;


      }
   }





   /*------------------------------------------------------------*/
   /* createWizardSteps
    */
   /**
    *    This method will create the necessary
    *  steps to create an object type
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createWizardSteps ( CreateWizardWin statusWin ) {
      switch ( objectType ) {
         case AVALON_SYMPTOM :
            steps = new WizardStep[3] ;
            statusWin.setValue ( 3 ) ;
            steps[0] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_SYMPTOM ),
                                                   Str.getStr ( STR_NEW_AVALON_WIZARD_SYMPTOM_TITLE ) ) ;
            statusWin.setValue ( 4 ) ;
            steps[1] = (WizardStep) new NewAvalonObjectTitleStep () ;
            statusWin.setValue ( 5 ) ;
            steps[2] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_FINISH ),
                                                   Str.getStr ( STR_WIZARD_FINISH ) ) ;
            statusWin.setValue ( 6 ) ;
            break ;

         case AVALON_QUESTION :
            steps = new WizardStep[4] ;
            statusWin.setValue ( 3 ) ;
            steps[0] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_QUESTION ),
                                                   Str.getStr ( STR_NEW_AVALON_WIZARD_QUESTION_TITLE ) ) ;
            statusWin.setValue ( 4 ) ;
            steps[1] = (WizardStep) new NewAvalonObjectTitleStep () ;
            statusWin.setValue ( 5 ) ;
            steps[2] = (WizardStep) new NewAvalonObjectAddtlInfoStep () ;
            statusWin.setValue ( 6 ) ;
            steps[3] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_FINISH ),
                                                   Str.getStr ( STR_WIZARD_FINISH ) ) ;
            statusWin.setValue ( 7 ) ;
            break ;

         case AVALON_ACTION :
            steps = new WizardStep[4] ;
            statusWin.setValue ( 3 ) ;
            steps[0] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_ACTION ),
                                                   Str.getStr ( STR_NEW_AVALON_WIZARD_ACTION_TITLE ) ) ;
            statusWin.setValue ( 4 ) ;
            steps[1] = (WizardStep) new NewAvalonObjectTitleStep () ;
            statusWin.setValue ( 5 ) ;
            steps[2] = (WizardStep) new NewAvalonObjectAddtlInfoStep () ;
            statusWin.setValue ( 6 ) ;
            steps[3] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_FINISH ),
                                                   Str.getStr ( STR_WIZARD_FINISH ) ) ;
            statusWin.setValue ( 7 ) ;
            break ;

         case AVALON_LINK :
            steps = new WizardStep[5] ;
            statusWin.setValue ( 3 ) ;
            steps[0] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_LINK ),
                                                   Str.getStr ( STR_NEW_AVALON_WIZARD_LINK_TITLE ) ) ;
            statusWin.setValue ( 4 ) ;
            steps[1] = (WizardStep) new NewAvalonObjectLinkTypeStep () ;
            statusWin.setValue ( 5 ) ;
            steps[2] = (WizardStep) new NewAvalonObjectLinkTargetStep () ;
            statusWin.setValue ( 5 ) ;
            steps[3] = (WizardStep) new NewAvalonObjectTitleStep () ;
            statusWin.setValue ( 6 ) ;
            steps[4] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_FINISH ),
                                                   Str.getStr ( STR_WIZARD_FINISH ) ) ;
            break ;

         case AVALON_LINK_GROUP :
            steps = new WizardStep[3] ;
            statusWin.setValue ( 3 ) ;
            steps[0] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_LINK_GROUP ),
                                                   Str.getStr ( STR_NEW_AVALON_WIZARD_LINK_GROUP_TITLE ) ) ;
            statusWin.setValue ( 4 ) ;
            steps[1] = (WizardStep) new NewAvalonObjectTitleStep () ;
            statusWin.setValue ( 5 ) ;
            steps[2] = (WizardStep) new InfoStep ( Str.getStr ( STR_NEW_AVALON_WIZARD_FINISH ),
                                                   Str.getStr ( STR_WIZARD_FINISH ) ) ;
            statusWin.setValue ( 6 ) ;
            break ;

      }
   }


   /*------------------------------------------------------------*/
   /* cancelPerformed
    */
   /**
    *    This method will be called if the user
    *  hit the 'Cancel' button on the Wizard
    *  
    *
    *  @param e The wizard event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cancelPerformed ( WizardEvent e ) {
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* finishPerformed
    */
   /**
    *    This method will be called when the
    *  user presses the finish button
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finishPerformed ( WizardEvent e) {
      cancelled = false ;
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* nextPerformed
    */
   /**
    *    This method will be called when the
    *  next button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void nextPerformed ( WizardEvent e ) {
      WizardStep step  = wizardPanel.getCurrentStep() ;
      int        index = indexFromStep ( step ) ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Handle Special conditions                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( objectType ) {
         case AVALON_LINK :
            if ( index == 1 ) {
               int type = ((NewAvalonObjectLinkTypeStep)steps[1]).getIntType() ;

               ((NewAvalonObjectLinkTargetStep)steps[2]).setType ( type ) ;
            } else if ( index == 2 ) {
               int type = ((NewAvalonObjectLinkTypeStep)steps[1]).getIntType() ;

               if ( type == 0 ) {
                  //String title = ((NewAvalonObjectLinkTargetStep)steps[2]).getTitle() ;
                  //if ( title != null )
                  //   ((NewAvalonObjectTitleStep)steps[3]).setTitle ( title ) ;
                  finishPerformed(null) ;
               } else {
                  ((NewAvalonObjectTitleStep)steps[3]).setTitle ( ((NewAvalonObjectLinkTargetStep)steps[2]).getLinkTitle() ) ;
               }
            }
      }

      index++ ;
      if ( index >= steps.length )
         index = steps.length-1 ;

      wizardPanel.setCurrentStep ( steps [ index ] ) ;
      adjustButtons ( ) ;
   }


   /*------------------------------------------------------------*/
   /* backPerformed
    */
   /**
    *    This method will be called when the
    *  back button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void backPerformed ( WizardEvent e ) {
      WizardStep step  = wizardPanel.getCurrentStep() ;
      int        index = indexFromStep ( step ) ;

      index-- ;
      if ( index < 0 )
         index = 0 ;

      wizardPanel.setCurrentStep ( steps [ index ] ) ;
      adjustButtons ( ) ;
   }





   /*------------------------------------------------------------*/
   /* adjustButtons
    */
   /**
    *    This method will look at what step is
    *  currently showing and will decide which
    *  buttons to show in the wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void adjustButtons () {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == steps[0] )
         wizardPanel.showButtons ( true, false, true, false ) ;
      else if ( step == steps[steps.length-1] )
         wizardPanel.showButtons ( false, true, false, true ) ;
      else
         wizardPanel.showButtons ( true, true, true, false ) ;
   }





   /*------------------------------------------------------------*/
   /* indexFromStep
    */
   /**
    *    This method will return the index of the WizardStep
    *  passed in
    *  
    *
    *  @param wizardStep
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   private int indexFromStep ( WizardStep wizardStep ) {
      int rc = 0 ;
      int i  = 0 ;

      while ( ( rc == 0 ) && ( i < steps.length ) ) {
         if ( steps[i] == wizardStep )
            rc = i ;
         i++ ;
      }

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* getNewObject
    */
   /**
    *    This method will return the new object that
    *  this wizard created
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getNewObject () {

      if ( !cancelled ) {
         switch ( objectType ) {
            case AVALON_SYMPTOM :
               return( createSymptom() ) ;

            case AVALON_QUESTION :
               return( createQuestion() ) ;

            case AVALON_ACTION :
               return( createAction() ) ;

            case AVALON_LINK :
               return( createLink() ) ;

            case AVALON_LINK_GROUP :
               return( createLinkGroup() ) ;
         }
      }

      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* createSymptom
    */
   /**
    *    This method will create a symptom and returns
    *  the symptom object that was created
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SymptomDraft createSymptom () {
      SymptomDraft symptom = new SymptomDraft() ;

      try {
         Vector        titleVec     = new Vector ( 1, 1 ) ;
         Vector        countryVec   = getFavoriteCountries() ;
         SymptomTitle  title        = new SymptomTitle ( ((NewAvalonObjectTitleStep)steps[1]).getObjectTitle() ) ;              
         Question      question     = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the title object                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         title.setCountryList ( countryVec ) ;
         title.setDocClass ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_UNCLASSIFIED,
                                                                                      TypeList.TYPE_DOCCLASS ) ) ;
         title.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                      TypeList.TYPE_LANGUAGE ) ) ;
         titleVec.addElement ( title ) ;                                                                        
         
         
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the Root Question and create the root node      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         question = AvalonCache.readQuestion ( 1 ) ;
         if ( question != null ) {
            Node   rootNode = new Node ( question ) ;
            Vector nodes    = new Vector ( 1 ) ;

            nodes.addElement ( rootNode ) ;
            symptom.setRootNode ( rootNode ) ;
            symptom.setNodes ( nodes ) ;

         } else {
            GUISystem.printBox ( STR_ERROR, STR_CANNOT_FIND_ROOT_NODE ) ;
            return( null ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Assign the symptom variables                            */                   
         /*                                                           */
         /*-----------------------------------------------------------*/
         symptom.setSymptomTitles    ( titleVec ) ;
         symptom.setCountryList      ( countryVec ) ;
         symptom.setDbUser           ( UserSystem.getUserId() ) ;
         symptom.setChangedTime      ( DateSystem.getCurrentTimestamp() ) ;
         symptom.setStartDate        ( DateSystem.getCurrentTimestamp() ) ;
         symptom.updateRecStatus     ( QITRec.REC_INSERT ) ;

         // Set the Default Work Required to Create
         symptom.setTypeWorkRequired ( (TypeWorkRec)     TypeList.getInstance().objectFromInd ( TypeList.WORKREQ_CREATE, TypeList.TYPE_WORKREQUIRED ) ) ;

         // Set the Default Class to Confidential
         symptom.setDocClass         ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL, TypeList.TYPE_DOCCLASS ) ) ;

         // Set the Default Type to 'Default'
         symptom.setSymptomType      ( (TypeSymptomRec)  TypeList.getInstance().objectFromInd ( 1, TypeList.TYPE_SYMPTOM ) ) ;

         // Set the default status to 'Flawed'
         symptom.setSymptomStatus    ( (TypeStatusRec)   TypeList.getInstance().objectFromInd ( 14, TypeList.TYPE_STATUS ) ) ;

         // Set the brand
         symptom.setBrand            ( getDefaultBrand() ) ;
      } catch ( Exception e ) {
         symptom = null ;
         LogSystem.log ( 1, e ,false ) ;
      }

      return( symptom ) ;
   }




   /*------------------------------------------------------------*/
   /* createQuestion
    */
   /**
    *    This method will create a question from
    *  the data entered into the wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return QuestionDraft
    *//*---------------------------------------------------------*/
   public QuestionDraft createQuestion () {
      QuestionDraft question = new QuestionDraft() ;

      try {
         Vector        titleVec     = new Vector ( 1, 1 ) ;
         Vector        addtlInfoVec = new Vector ( 1, 1 ) ;
         Vector        countryVec   = getFavoriteCountries() ;
         Title         title        = new Title ( ((NewAvalonObjectTitleStep)steps[1]).getObjectTitle() ) ;              
         AddtlInfo     addtlInfo    = AddtlInfoImporter.createAddtlInfo ( ((NewAvalonObjectAddtlInfoStep)steps[2]).getAddtlInfo() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the title object                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         title.setCountryList ( countryVec ) ;
         title.setDocClass ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_UNCLASSIFIED,
                                                                                      TypeList.TYPE_DOCCLASS ) ) ;
         title.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                      TypeList.TYPE_LANGUAGE ) ) ;
         titleVec.addElement ( title ) ;                               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the additional information object                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( addtlInfo.getAddtlInfo().trim().length() > 0 ) {
            addtlInfo.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                             TypeList.TYPE_LANGUAGE ) ) ;                  
            addtlInfo.setDocClass ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL,
                                                                                             TypeList.TYPE_DOCCLASS ) ) ;
            addtlInfo.setCountryList ( getFavoriteCountries() ) ;
            addtlInfoVec.addElement ( addtlInfo ) ;
         }


         question.setAdditionalInfo   ( addtlInfoVec ) ;
         question.setTitles           ( titleVec ) ;
         question.setCountryList      ( countryVec ) ;
         question.setAdditionalInfo   ( addtlInfoVec ) ;
         question.setDbUser           ( UserSystem.getUserId() ) ;
         question.setChangedTime      ( DateSystem.getCurrentTimestamp() ) ;
         question.setStartDate        ( DateSystem.getCurrentTimestamp() ) ;
         question.updateRecStatus     ( QITRec.REC_INSERT ) ;
         question.setTypeWorkRequired ( (TypeWorkRec) TypeList.getInstance().objectFromInd ( TypeList.WORKREQ_CREATE, TypeList.TYPE_WORKREQUIRED ) ) ;
         question.setBrand            ( getDefaultBrand() ) ;
         question.setDocClass         ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL, TypeList.TYPE_DOCCLASS ) ) ;

      } catch ( Exception e ) {
         question = null ;
      }
      return( question ) ;
   }



   /*------------------------------------------------------------*/
   /* createAction
    */
   /**
    *    This method will create an Action from
    *  the data entered into the wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ActionDraft
    *//*---------------------------------------------------------*/
   public ActionDraft createAction () {
      ActionDraft action = new ActionDraft() ;

      try {
         Vector        titleVec     = new Vector ( 1, 1 ) ;
         Vector        addtlInfoVec = new Vector ( 1, 1 ) ;
         Vector        countryVec   = getFavoriteCountries() ;
         Title         title        = new Title ( ((NewAvalonObjectTitleStep)steps[1]).getObjectTitle() ) ;              
         AddtlInfo     addtlInfo    = AddtlInfoImporter.createAddtlInfo ( ((NewAvalonObjectAddtlInfoStep)steps[2]).getAddtlInfo() ) ;

         title.setCountryList ( countryVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the title object                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         title.setCountryList ( countryVec ) ;
         title.setDocClass ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_UNCLASSIFIED,
                                                                                      TypeList.TYPE_DOCCLASS ) ) ;
         title.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                      TypeList.TYPE_LANGUAGE ) ) ;
         titleVec.addElement ( title ) ;                               


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the additional information object                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( addtlInfo.getAddtlInfo().trim().length() > 0 ) {
            addtlInfo.setLanguage ( (TypeLanguageRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                                                             TypeList.TYPE_LANGUAGE ) ) ;                  
            addtlInfo.setDocClass ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL,
                                                                                             TypeList.TYPE_DOCCLASS ) ) ;
            addtlInfo.setCountryList ( getFavoriteCountries() ) ;
            addtlInfoVec.addElement ( addtlInfo ) ;
         }


         action.setAdditionalInfo   ( addtlInfoVec ) ;
         action.setTitles           ( titleVec ) ;
         action.setCountryList      ( countryVec ) ;
         action.setDbUser           ( UserSystem.getUserId() ) ;
         action.setStartDate        ( DateSystem.getCurrentTimestamp() ) ;
         action.updateRecStatus     ( QITRec.REC_INSERT ) ;
         action.setTypeWorkRequired ( (TypeWorkRec) TypeList.getInstance().objectFromInd ( TypeList.WORKREQ_CREATE, TypeList.TYPE_WORKREQUIRED ) ) ;
         action.setBrand            ( getDefaultBrand() ) ;
         action.setDocClass         ( (TypeDocClassRec) TypeList.getInstance().objectFromInd ( TypeList.DOCCLASS_IBM_CONFIDENTIAL,
                                                                                               TypeList.TYPE_DOCCLASS ) ) ;
      } catch ( Exception e ) {
         action = null ;
      }
      return( action ) ;
   }


   /*------------------------------------------------------------*/
   /* createLink
    */
   /**
    *    This method will create an Link from
    *  the data entered into the wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object createLink () {
      QuestLinkDraft    questLink    = null ;
      ExternalLinkDraft externalLink = null ;

      try {
         String  type   = ((NewAvalonObjectLinkTypeStep)steps[1]).getType() ;             
         String  target = ((NewAvalonObjectLinkTargetStep)steps[2]).getTarget() ;              
         boolean valid  = ((NewAvalonObjectLinkTargetStep)steps[2]).getValid() ;             
         String  title  = ((NewAvalonObjectTitleStep)steps[3]).getObjectTitle() ;              


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the link object                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( type.equals ( Str.getStr ( STR_QUEST_LINK ) ) ) {
            questLink = new QuestLinkDraft ( new Integer ( target ).intValue(),
                                             title = ((NewAvalonObjectLinkTargetStep)steps[2]).getLinkTitle() ) ;
            questLink.setDbUser           ( UserSystem.getUserId() ) ;
            questLink.setStartDate        ( DateSystem.getCurrentTimestamp() ) ;
            questLink.updateRecStatus     ( QITRec.REC_INSERT ) ;
            questLink.setTypeWorkRequired ( (TypeWorkRec) TypeList.getInstance().objectFromInd ( TypeList.WORKREQ_CREATE, TypeList.TYPE_WORKREQUIRED ) ) ;
         } else {
            externalLink = new ExternalLinkDraft ( "", title ) ;
            externalLink.setDbUser           ( UserSystem.getUserId() ) ;
            externalLink.setStartDate        ( DateSystem.getCurrentTimestamp() ) ;
            externalLink.updateRecStatus     ( QITRec.REC_INSERT ) ;
            externalLink.setUrl              ( target ) ;
            externalLink.setUrlValid         ( valid ) ;
            externalLink.setTypeWorkRequired ( (TypeWorkRec) TypeList.getInstance().objectFromInd ( TypeList.WORKREQ_CREATE, TypeList.TYPE_WORKREQUIRED ) ) ;
         }

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      if ( questLink != null )
         return( questLink ) ;

      return( externalLink ) ;
   }

   /*------------------------------------------------------------*/
   /* createLinkGroup
    */
   /**
    *    This method will create an LinkGroup from
    *  the data entered into the wizard
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object createLinkGroup () {
      LinkGroupDraft    linkGroup    = null ;

      try {
         String title = ((NewAvalonObjectTitleStep)steps[1]).getObjectTitle() ;              

         linkGroup = new LinkGroupDraft ( title ) ;
         linkGroup.setDbUser           ( UserSystem.getUserId() ) ;
         linkGroup.setStartDate        ( DateSystem.getCurrentTimestamp() ) ;
         linkGroup.updateRecStatus     ( QITRec.REC_INSERT ) ;
         linkGroup.setTypeWorkRequired ( (TypeWorkRec) TypeList.getInstance().objectFromInd ( TypeList.WORKREQ_CREATE, TypeList.TYPE_WORKREQUIRED ) ) ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

      return( linkGroup ) ;
   }






   /*------------------------------------------------------------*/
   /* getDefaultBrand
    */
   /**
    *    This method will return the user's default
    *  brand
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private TypeBrandRec getDefaultBrand () {
      TypeBrandRec brandRec = null ;

      try {
         int defaultBrandInd = PropertySystem.getInt ( PropertySystem.PROPERTY_DEFAULT_BRAND ) ;

         brandRec = (TypeBrandRec)TypeList.getInstance().objectFromInd ( defaultBrandInd, TypeList.TYPE_BRAND ) ;
      } catch ( Exception e ) {
      }

      return( brandRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getFavoriteCountries
    */
   /**
    *    This method will return the user's list of
    *  favorite countries. If this list is not populated, the
    *  countries the user is configured for is returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   private Vector getFavoriteCountries () {
      Vector retVec = null ;

      retVec = (Vector) ((Vector)PropertySystem.get ( PropertySystem.PROPERTY_COUNTRY_FAVORITES )).clone() ;

      if ( ( retVec == null ) ||
           ( retVec.size() == 0 ) ||
           ( PropertySystem.getBool ( PropertySystem.PROPERTY_COUNTRY_FAVORITES_AUTO ) ) )
         retVec = (Vector)UserSystem.getUserCountries ( UserSystem.getUserId() ).clone() ;

      return ( retVec ) ;
   }
} /* NewAvalonObjectWizard */
