package com.ibm.nzna.projects.qit.avalon.gui ;    
/*-----------------------------------------------------------*/
/*   PublishPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PublishPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This panel holds information for the user during the publishing process. Please
 * note that any draft object passed to this panel MUST BE SAVED ALREADY AND UNLOCKED!
 *
 *
 * @version 1.0, 5/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   PublishPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class PublishPanel extends JPanel implements Runnable, ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String THREAD_VALIDATE  = "V" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_PUBLISHING        = new JLabel ( Str.getStr ( STR_PUBLISHING ) )  ;
   private JLabel       stt_PUBLISHING       = new JLabel ( ) ;
   private JLabel       st_STATUS            = new JLabel ( Str.getStr ( STR_STATUS )+": " ) ;
   private JLabel       stt_STATUS           = new JLabel ( ) ;
   private Browser      browser              = new Browser ( ) ;
   private DButton      pb_PUBLISH           = new DButton ( Str.getStr ( STR_PUBLISH ) ) ;
   private DButton      pb_CANCEL            = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
   private DButton      pb_VALIDATE          = new DButton ( Str.getStr ( STR_VALIDATE ) ) ;
   private JProgressBar progressBar          = new JProgressBar () ;      
   private JPanel       statusPanel          = new JPanel () ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Object               publishObject = null ;
   private PublishPanelListener listener      = null ;
   private OAObjectValidator    validator     = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default Constructor
    *  
    *  
    *
    *  @param publishObject The object to publish
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PublishPanel ( Object publishObject ) {                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our local vars                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.publishObject = publishObject ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust GUI Properties                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBorder ( GUISystem.blackBorder ) ;
      setBackground ( Color.white ) ;
      stt_PUBLISHING.setFont ( FontSystem.smallTitleFont ) ;
      stt_PUBLISHING.setForeground ( Color.blue ) ;
      statusPanel.setOpaque ( false ) ;
      stt_STATUS.setFont ( FontSystem.smallTitleFont ) ;
      stt_STATUS.setForeground ( Color.blue ) ;
      progressBar.setOpaque ( false ) ;
      progressBar.setBorder ( GUISystem.blackBorder ) ;
      browser.setBorder ( GUISystem.blackBorder ) ;
      browser.setBackground ( Color.white ) ;               
      browser.showZoom ( false ) ;
      pb_PUBLISH.setEnabled ( false ) ;
      pb_CANCEL.setEnabled ( false ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CANCEL.addActionListener ( this ) ;
      pb_PUBLISH.addActionListener ( this ) ;
      pb_VALIDATE.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI to the panel                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      statusPanel.setLayout ( new BorderLayout( )) ;
      add ( st_PUBLISHING ) ;
      add ( stt_PUBLISHING ) ;
      add ( browser ) ;
      statusPanel.add ( st_STATUS,  BorderLayout.WEST ) ;
      statusPanel.add ( stt_STATUS, BorderLayout.CENTER ) ;
      add ( statusPanel ) ;
      add ( progressBar ) ;
      add ( pb_VALIDATE ) ;
      add ( pb_PUBLISH ) ;
      add ( pb_CANCEL ) ;                   

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will set the lay out of our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = 15 ;
      int       y         = 0 ;

      st_PUBLISHING.setBounds ( x, y, 150, rowHeight ) ;
      stt_PUBLISHING.setBounds ( x+150, y, 250, rowHeight ) ;
      y+=rowHeight ;

      browser.setBounds ( x, y, size.width-(x*2), size.height-y-70 ) ;
      y+=(size.height-y-70)+5 ;

      statusPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      progressBar.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+2 ;

      pb_VALIDATE.setBounds ( x, y, 130, 25 ) ;
      x+=135 ;

      pb_PUBLISH.setBounds ( x, y, 130, 25 ) ;
      x+=135 ;

      pb_CANCEL.setBounds ( x, y, 100, 25 ) ;

   }





   /*------------------------------------------------------------*/
   /* setPublishingText
    */
   /**
    *    This method will set the text that is shown
    *  next to the 'Publishing' label
    *  
    *
    *  @param text The text to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPublishingText ( String text ) {
      stt_PUBLISHING.setText ( text ) ;
   }




   /*------------------------------------------------------------*/
   /* setStatusText
    */
   /**
    *    This method will set the text that is next to the
    *  'Status' label
    *  
    *
    *  @param text The text to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatusText ( String text ) {
      stt_STATUS.setText ( text ) ;
   }






   /*------------------------------------------------------------*/
   /* enablePubish
    */
   /**
    *    This method will either enable or disable
    *  the publish pusbutton
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void enablePublish ( boolean enable ) {
      pb_PUBLISH.setEnabled ( enable ) ;
   }





   /*------------------------------------------------------------*/
   /* enableCancel
    */
   /**
    *    This method will either enable or disable the
    *  cancel button
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void enableCancel ( boolean enable ) {
      pb_CANCEL.setEnabled ( enable ) ;
   }





   /*------------------------------------------------------------*/
   /* setPublishPanelListener
    */
   /**
    *    This method will set the 1 PublishPanelListener
    *  that this PublishPanel will notifiy with
    *  
    *
    *  @param listener The PublishPanelListener
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPublishPanelListener ( PublishPanelListener listener ) {
      this.listener = listener ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an Action
    *  Event is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) { 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_PUBLISH                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_PUBLISH )
         publish() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CANCEL ) {
         if ( listener != null )
            listener.publishComplete ( false ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_VALIDATE                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_VALIDATE )
         validatePublish() ;

   } /* actionPerformed */





   /*------------------------------------------------------------*/
   /* publish
    */
   /**
    *    This method will start the publishing
    *  process
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void publish () {                                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If they have not validated yet, validate now            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( validator == null )
         validatePublish() ;

      else {

         pb_CANCEL.setEnabled ( false ) ;
         pb_VALIDATE.setEnabled ( false ) ;
         pb_PUBLISH.setEnabled ( false ) ;

         new OAObjectPublisher ( this, validator.getValidationRec(), publishObject ).start() ;

      } /* else */

   } /* publish */





   /*------------------------------------------------------------*/
   /* validatePublish
    */
   /**
    *    This method will validate the object
    *  being published and then will populate
    *  the browser with the results
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void validatePublish () {

      validator = new OAObjectValidator ( this, publishObject ) ;

      enablePublish  ( false ) ;
      enableCancel   ( false ) ;
      enableValidate ( false ) ;
      browser.setPageContent ( validator.getHTML() ) ;
      enablePublish  ( !validator.getError() ) ;
      enableCancel   ( true ) ;
      enableValidate ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* enableValidate
    */
   /**
    *    This method will either enable or disable 
    *  the validate push button
    *  
    *
    *  @param enable TRUE = enable the button, FALSE = disable the button
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void enableValidate ( boolean enable ) {
      pb_VALIDATE.setEnabled ( enable ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run a variety of processes
    *  in a thread. The process which is run
    *  is dependant on the name of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_VALIDATE                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_VALIDATE ) )
         validatePublish() ;

   }





   /*------------------------------------------------------------*/
   /* setMaximum
    */
   /**
    *    This method will set the maximum
    *  units that the progress bar indicates
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMaximum ( int maximum ) {
      progressBar.setMaximum ( maximum ) ;
   }




   /*------------------------------------------------------------*/
   /* setValue
    */
   /**
    *    This method will set the value of the
    *  progress bar
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setValue ( int value ) {
      progressBar.setValue ( value ) ;
   }





   /*------------------------------------------------------------*/
   /* bumpProgress
    */
   /**
    *    This method will bump our progress indicator
    *  up by 1 notch
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void bumpProgress () {
      progressBar.setValue ( progressBar.getValue()+1 ) ;
   }





   /*------------------------------------------------------------*/
   /* publishComplete
    */
   /**
    *    This method will be called once the OAPublisher
    *  object has finished publishing the object
    *  
    *
    *  @param complete TRUE = Everything was published. FALSE = not everything was published
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void publishComplete ( boolean complete ) {
      if ( complete ) {
         if ( listener != null )
            listener.publishComplete ( true ) ;
      } else {
         pb_VALIDATE.setEnabled ( true ) ;
         pb_CANCEL.setEnabled ( true ) ;
         pb_PUBLISH.setEnabled ( true ) ;
      }

   }
} /* PublishPanel */
