package com.ibm.nzna.projects.qit.avalon.gui ;   
/*-----------------------------------------------------------*/
/*   SelectOADlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectOADlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.avalon.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.avalon.gui.selectview.* ;
import com.ibm.nzna.projects.qit.avalon.editors.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.exporter.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import javax.swing.border.* ;
import javax.swing.event.* ;
import java.awt.event.* ;
import java.util.* ;
import java.awt.* ;
/**
 * This method will allow the user to search through the Online Assistant 
 * for a Question, Action, Symptom or Link.
 *
 * <br><br>
 * To add a view to this window, follow these steps:<br><pre>
 * 1. Create a class in com.ibm.nzna.projects.qit.avalon.gui.selectview for your<br>
 *    new class. A good idea is to copy one of the existing views to get things started<br>
 * 2. Add that class name to the static 'views' array in this class. Try to keep the names
 *    displayed in alphabetical order
 * <br><br></pre>
 *
 * @version 1.0, 4/19/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectOADlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectOADlg extends JFrame implements AvalonConst, WindowListener, Runnable, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public static Members                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int ENTRYFIELD         = 1 ;
   public static final int USER_ENTRYFIELD    = 2 ;
   public static final int COMBO_BOX          = 3 ;  

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   View Indexs                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int VIEW_ACTION             = 0 ;
   public static final int VIEW_LINK               = 1 ;
   public static final int VIEW_LINKGROUP          = 2 ;
   public static final int VIEW_QUESTION           = 3 ; 
   public static final int VIEW_SYMPTOM            = 4 ; 
   public static final int VIEW_ANSWER             = 5 ; 


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Views                                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] views = { 
      "com.ibm.nzna.projects.qit.avalon.gui.selectview.ActionView", 
      "com.ibm.nzna.projects.qit.avalon.gui.selectview.LinkView" ,
      "com.ibm.nzna.projects.qit.avalon.gui.selectview.LinkGroupView" ,
      "com.ibm.nzna.projects.qit.avalon.gui.selectview.QuestionView" ,
      "com.ibm.nzna.projects.qit.avalon.gui.selectview.SymptomView",
      "com.ibm.nzna.projects.qit.avalon.gui.selectview.AnswerView"
   } ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JScrollPane    bookmarkScroll    = null ;
   private JLabel         st_OBJECT         = null ;
   private JLabel         st_SEARCHARG      = null ;
   private JLabel         st_SEARCH         = null ;
   private JComboBox      cb_OBJECT         = null ;
   private JComboBox      cb_SEARCHON       = null ;
   private JComboBox      cb_SEARCH         = null ;
   private JTextField     ef_SEARCHARG      = null ;
   private JPanel         findPanel         = null ;
   private JPanel         resultPanel       = null ;
   private MultiList      cnr_DATA          = null ;
   private DButton        pb_FIND           = null ;
   private DButton        pb_SELECT         = null ;
   private DButton        pb_EDIT           = null ;
   private DButton        pb_CANCEL         = null ;
   private DButton        pb_EXPORT         = null ;
   private ButtonPanel    buttonPanel       = null ;
   private JCheckBox      ck_SEARCHDRAFT    = null ;
   private UserEntryPanel ef_USER_SEARCH    = null ;
   private BookmarkTree   bookmarkTree      = null ;
   private DButton        pb_BOOKMARKS      = null ;
   private DButton        pb_PRINT          = null ;
   private JLabel         st_BRAND          = null ;
   private JComboBox      cb_BRAND          = null ;
   private HotLinkLabel   pb_CLEAR_HISTORY  = null ;
   private HotLinkLabel   pb_ADD_HISTORY    = null ;
   private HotLinkLabel   pb_REMOVE_HISTORY = null ;
   private ButtonPanel    historyPanel      = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private SelectOAListener        listener         = null ;
   private static SelectOADlg      instance         = null ;
   private boolean                 showingBookmarks = false ;
   private int                     viewInd          = -1 ;
   private boolean                 allowDrafts      = true ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor and it will 
    *  show the window and then immediatly return
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectOADlg () {                                    
      initialize() ;


   } /* SelectOADlg  */







   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create all of our controls
    *  and will then show the window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {

      /*------------------------------------------------------------*/
      /*                                                            */
      /*   If there is already an instance of us, use it, otherwise */
      /* create a new instance.                                     */
      /*                                                            */
      /*------------------------------------------------------------*/
      if ( instance != null ) {
         dispose() ;
         instance.setVisible ( true ) ;
         instance.requestFocus() ;
         instance.allowDrafts ( true ) ;
      }

      else {
         Container    contentPane = getContentPane() ;
         Vector       history     = (Vector)PropertySystem.get ( PROPERTY_SEARCH_HISTORY ) ;
         Vector       brands      = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_BRAND ).clone() ;
         TypeBrandRec allBrandRec = null ;

         instance = this ;
         setTitle ( Str.getStr ( STR_SELECT_OA_OBJECT ) ) ;               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the 'All' brand                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         brands.insertElementAt( allBrandRec = new TypeBrandRec ( -1, Str.getStr ( STR_ALL ), 0 ), 0 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         ef_USER_SEARCH    = new UserEntryPanel () ;
         bookmarkTree      = new BookmarkTree() ;
         ck_SEARCHDRAFT    = new JCheckBox ( Str.getStr ( STR_SHOW_ONLY_DRAFTS ) ) ;       
         st_OBJECT         = new JLabel ( Str.getStr ( STR_OA_OBJECT ) ) ;
         st_SEARCHARG      = new JLabel ( Str.getStr ( STR_CRITERIA ) ) ;
         st_SEARCH         = new JLabel ( Str.getStr ( STR_SEARCH ) ) ;
         pb_CLEAR_HISTORY  = new HotLinkLabel ( Str.getStr ( STR_CLEAR_HISTORY ) ) ;
         cb_OBJECT         = new JComboBox ( getObjectList() ) ;
         cb_SEARCHON       = new JComboBox () ;
         cb_SEARCH         = new JComboBox() ;
         findPanel         = new JPanel () ;
         resultPanel       = new JPanel () ;
         cnr_DATA          = new MultiList ( GUISystem.getFontUtil() ) ;
         pb_FIND           = new DButton ( Str.getStr ( STR_FIND ) ) ;
         pb_SELECT         = new DButton ( Str.getStr ( STR_SELECT ) ) ;
         pb_EDIT           = new DButton ( Str.getStr ( STR_EDIT ) ) ;
         pb_CANCEL         = new DButton ( Str.getStr ( STR_CANCEL ) ) ;        
         pb_BOOKMARKS      = new DButton ( Str.getStr ( STR_BOOKMARKS ) ) ;
         pb_PRINT          = new DButton ( Str.getStr ( STR_PRINT ) ) ;
         pb_EXPORT         = new DButton ( Str.getStr ( STR_EXPORT ) ) ;
         buttonPanel       = new ButtonPanel () ;
         bookmarkScroll    = new JScrollPane ( bookmarkTree ) ;
         st_BRAND          = new JLabel ( Str.getStr ( STR_BRAND ) ) ;
         cb_BRAND          = new JComboBox ( brands ) ;
         //pb_ADD_HISTORY    = new HotLinkLabel ( Str.getStr ( STR_ADD_HISTORY ) ) ;
         //pb_REMOVE_HISTORY = new HotLinkLabel ( Str.getStr ( STR_REMOVE_HISTORY ) ) ;
         //historyPanel      = new ButtonPanel () ;
         ef_SEARCHARG =new JTextField ( "" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setDefaultCloseOperation ( DO_NOTHING_ON_CLOSE ) ;
         addWindowListener ( this ) ;
         bookmarkScroll.setBorder ( GUISystem.loweredBorder ) ;
         findPanel.setBorder   ( new TitledBorder ( Str.getStr ( STR_FIND ) ) ) ;
         resultPanel.setBorder ( new TitledBorder ( Str.getStr ( STR_RESULTS ) ) ) ;    
         cb_SEARCH.setVisible ( false ) ;
         ef_USER_SEARCH.setDescript ( "" ) ;
         ef_USER_SEARCH.setDescriptWidth ( 0 ) ;
         ef_USER_SEARCH.setTopDescript ( false ) ;
         ef_SEARCHARG.setEditable ( true ) ;
         cb_BRAND.setSelectedItem ( TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEFAULT_BRAND ),
                                                                           TypeList.TYPE_BRAND ) ) ;
         //historyPanel.setBorder     ( GUISystem.grayBorder ) ;
         //historyPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
         //historyPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
         //                                 ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add Listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_FIND.addActionListener           ( this ) ;
         pb_BOOKMARKS.addActionListener      ( this ) ;
         pb_EDIT.addActionListener           ( this ) ;
         pb_SELECT.addActionListener         ( this ) ;
         pb_CANCEL.addActionListener         ( this ) ;                    
         pb_EXPORT.addActionListener         ( this ) ;
         cb_OBJECT.addActionListener         ( this ) ;
         cb_SEARCHON.addActionListener       ( this ) ;
         ef_SEARCHARG.addActionListener      ( this ) ;
         pb_PRINT.addActionListener          ( this ) ;
         //pb_CLEAR_HISTORY.addActionListener  ( this ) ;
         //pb_ADD_HISTORY.addActionListener    ( this ) ;
         //pb_REMOVE_HISTORY.addActionListener ( this ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the GUI to the find panel                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         findPanel.setLayout ( null ) ;
         findPanel.add ( st_OBJECT ) ;
         findPanel.add ( cb_OBJECT ) ;
         findPanel.add ( st_SEARCHARG ) ;
         findPanel.add ( cb_SEARCHON ) ;
         findPanel.add ( st_SEARCH ) ;
         findPanel.add ( ef_SEARCHARG ) ;                      
         //historyPanel.add ( pb_CLEAR_HISTORY ) ;
         //historyPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
         //historyPanel.add ( pb_ADD_HISTORY ) ;
         //historyPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) ) ;
         //historyPanel.add ( pb_REMOVE_HISTORY ) ;
         //findPanel.add ( historyPanel ) ;
         findPanel.add ( ef_USER_SEARCH ) ;
         findPanel.add ( cb_SEARCH ) ;
         findPanel.add ( st_BRAND ) ;
         findPanel.add ( cb_BRAND ) ;
         findPanel.add ( ck_SEARCHDRAFT ) ;

         initializeMultiList() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the GUI to the result panel                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         resultPanel.setLayout ( new BorderLayout() ) ;
         resultPanel.add ( cnr_DATA, BorderLayout.CENTER ) ;                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the GUI to our button panel                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         buttonPanel.add ( pb_FIND ) ;
         buttonPanel.add ( pb_SELECT ) ;
         buttonPanel.add ( pb_EDIT ) ;
         buttonPanel.add ( pb_BOOKMARKS ) ;
         buttonPanel.add ( pb_EXPORT ) ;
         buttonPanel.add ( pb_PRINT ) ;
         buttonPanel.add ( pb_CANCEL ) ;  



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our GUI                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         contentPane.setLayout ( null ) ;
         contentPane.add ( buttonPanel ) ;
         contentPane.add ( findPanel ) ;
         contentPane.add ( resultPanel ) ;
         GUISystem.setPropertiesOnPanel ( contentPane ) ;



         setSize ( 565, 390 ) ;
         WinUtil.centerWindow ( this ) ;
         setVisible ( true ) ;

         cb_BRAND.setSelectedItem ( allBrandRec ) ;
         refreshObjectData() ;
      }

   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize () ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust our large panels                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      findPanel.setBounds ( 5, 5, size.width-15, 150 ) ;
      resultPanel.setBounds ( 5, 155, size.width-15, size.height-205 ) ;
      buttonPanel.setBounds ( 5, size.height-50, size.width-15, 25 ) ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the GUI in the Find Panel                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      y = 15 ;
      st_OBJECT.setBounds ( 5, y, 150, rowHeight ) ;
      st_SEARCHARG.setBounds ( 160, y, 150 ,rowHeight ) ;
      st_BRAND.setBounds ( 320, y, 150, rowHeight ) ;
      y+=rowHeight ;

      cb_OBJECT.setBounds ( 5, y, 150, rowHeight ) ;
      cb_SEARCHON.setBounds ( 160, y, 150, rowHeight ) ;
      cb_BRAND.setBounds ( 320, y, 150, rowHeight ) ;
      y+=rowHeight*2 ;

      st_SEARCH.setBounds ( 5, y, size.width-15, rowHeight ) ;
      y+=rowHeight ;

      ef_SEARCHARG.setBounds ( 5, y, size.width-35, rowHeight ) ;
      cb_SEARCH.setBounds ( 5, y, size.width-35, rowHeight ) ;
      ef_USER_SEARCH.setBounds ( 5, y, size.width-35, rowHeight ) ;
      //y+=rowHeight ;

      //historyPanel.setBounds ( 5, y, size.width-35, rowHeight ) ;

      y+=rowHeight ;

      ck_SEARCHDRAFT.setBounds ( 5, y, size.width-35, rowHeight );


   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when the user
    *  has selected one of our GUI Components and that
    *  sent off an ActionEvent
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL )
         setVisible ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ef_SEARCHARG                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ef_SEARCHARG )
         new Thread ( this ).start() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   cb_OBJECT                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == cb_OBJECT )
         refreshObjectData() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   cb_SEARCHON                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == cb_SEARCHON )
         refreshSearchArgData() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_PRINT                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_PRINT )
         new Print ( cnr_DATA, this, cb_OBJECT.getSelectedItem().toString() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == cnr_DATA ) || (  e.getSource() == pb_EDIT ) ) {
         OAEdit.edit ( (DisplayRec)cnr_DATA.getSelectedItem(), Avalon.getOANavPanel() ) ;
         setVisible ( false ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_BOOKMARKS                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_BOOKMARKS ) {
         if ( showingBookmarks ) {
            resultPanel.remove ( bookmarkScroll ) ;
            resultPanel.add ( cnr_DATA, BorderLayout.CENTER ) ;
            pb_BOOKMARKS.setText ( Str.getStr ( STR_BOOKMARKS ) ) ;
            showingBookmarks = false ;
         } else {
            resultPanel.remove ( cnr_DATA ) ;
            resultPanel.add ( bookmarkScroll, BorderLayout.CENTER ) ;
            showingBookmarks = true ;
            pb_BOOKMARKS.setText ( Str.getStr ( STR_BROWSE_DB ) ) ;
         }
         GUISystem.setPreferredButton ( pb_BOOKMARKS ) ;
         buttonPanel.revalidate() ;
         resultPanel.invalidate() ;
         repaint() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Export                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EXPORT ) {
         Vector dataVec = cnr_DATA.getData() ;

         if ( ( dataVec != null ) && ( dataVec.size() > 0 ) )
            new Export ( null, dataVec ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_MUST_NAVIGATE_TO_DATA ) ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_FIND                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_FIND )
         find() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLEAR_HISTORY                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CLEAR_HISTORY ) {
         PropertySystem.put ( PROPERTY_SEARCH_HISTORY, new Vector ( 1 ) ) ;
         ef_SEARCHARG.requestFocus() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_SELECT                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SELECT )
         select() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT )
         edit() ;



   }





   /*------------------------------------------------------------*/
   /* getObjectList
    */
   /**
    *    This method will return a list of all objects
    *  we can search on
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector getObjectList () {
      Vector vector = new Vector ( 1, 1 ) ;

      try {
         for ( int i = 0; i < views.length; i++ )
            vector.addElement ( Class.forName ( views[i] ).newInstance() ) ;
      } catch ( Exception e ) {
      }

      return( vector ) ;
   }




   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return an instance of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectOADlg getInstance() {
      return( instance ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshObjectData
    */
   /**
    *    This method will refesh all of the fields in the
    *  window for the new type of object selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshObjectData () {
      SelectOAView view = (SelectOAView) cb_OBJECT.getSelectedItem() ;
      if ( view != null ) {
         Vector       searchArgVec = view.getSearchArg() ;

         if ( searchArgVec != null ) {
            int i    = 0 ;
            int size = searchArgVec.size() ;

            cb_SEARCHON.removeAllItems() ;
            while ( i < size ) {
               cb_SEARCHON.addItem ( searchArgVec.elementAt(i) ) ;
               i++ ;
            }

            cnr_DATA.removeAll() ;
            cnr_DATA.setColumnHeadings ( view.getColumnHeadings() ) ;
            cnr_DATA.setColumnWidth ( 0, 15 ) ;
            cnr_DATA.setColumnWidth ( 1, 15 ) ;
            cnr_DATA.setColumnWidth ( 2, 100 ) ;
            cnr_DATA.setColumnWidth ( 3, 300 ) ;
            cnr_DATA.setColumnWidth ( 4, 150 ) ;
            cnr_DATA.setColumnWidth ( 5, 150 ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* refreshSearchArgData
    */
   /**
    *    This method will refresh the search argument data
    *  fields. This includes the search entry field/combo box
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshSearchArgData () {
      SelectOAView view = (SelectOAView) cb_OBJECT.getSelectedItem() ;
      String searchArg = (String) cb_SEARCHON.getSelectedItem() ;

      switch ( view.getSearchArgDisplay ( searchArg ) ) {
         case ENTRYFIELD :
            ef_SEARCHARG.setVisible ( true ) ;
            pb_CLEAR_HISTORY.setVisible ( true ) ;
            ef_USER_SEARCH.setVisible ( false ) ;
            cb_SEARCH.setVisible ( false ) ;
            ef_SEARCHARG.requestFocus() ;
            break ;

         case USER_ENTRYFIELD :
            ef_SEARCHARG.setVisible ( false ) ;
            pb_CLEAR_HISTORY.setVisible ( false ) ;
            ef_USER_SEARCH.setVisible ( true ) ;
            cb_SEARCH.setVisible ( false ) ;
            ef_USER_SEARCH.requestFocus() ;
            break ;

         case COMBO_BOX :
            ef_SEARCHARG.setVisible ( false ) ;
            pb_CLEAR_HISTORY.setVisible ( false ) ;
            ef_USER_SEARCH.setVisible ( false ) ;
            cb_SEARCH.setVisible ( true ) ;
            cb_SEARCH.requestFocus() ;
            break ;
      }
   }






   /*------------------------------------------------------------*/
   /* find
    */
   /**
    *    This method will kick off our search
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void find () {
      SelectOAView view      = (SelectOAView) cb_OBJECT.getSelectedItem() ;
      String       searchArg = (String) cb_SEARCHON.getSelectedItem() ;
      String       qualifier = ck_SEARCHDRAFT.isSelected() ? "OADRAFT" : "OA" ;
      int          brandInd  = 0 ;
      String       search    = null ;
      Vector       dataVec   = null ;

      if ( cb_BRAND.getSelectedItem() != null )
         brandInd = ((TypeBrandRec)cb_BRAND.getSelectedItem()).getInd() ;

      if ( brandInd == 0 )
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_BRAND ) ;

      else if ( view == null )
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_OBJECT_TO_FIND ) ;

      if ( ( brandInd != 0 ) && ( searchArg != null ) && ( view != null ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the correct search item                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( view.getSearchArgDisplay ( searchArg ) ) {
            case ENTRYFIELD :
               search = ef_SEARCHARG.getText() ;
               break ;

            case USER_ENTRYFIELD :
               search = ef_USER_SEARCH.getText() ;
               break ;

            case COMBO_BOX :
               search = ef_SEARCHARG.getText() ;
               break ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save this search                                        */       
         /*                                                           */
         /*-----------------------------------------------------------*/
         //addSearchToHistory ( search, brandInd, searchArg, view ) ;          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the data vector                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         enableComponents ( false ) ;
         //resultPanel.remove ( cnr_DATA ) ;
         //cnr_DATA = null ;
         dataVec = view.getData ( searchArg, qualifier, search, brandInd ) ;

         //cnr_DATA = new MultiList ( GUISystem.getFontUtil() ) ;
         //initializeMultiList() ;
         //refreshObjectData() ;
         //resultPanel.add ( cnr_DATA, BorderLayout.CENTER ) ;

         if ( dataVec != null )
            cnr_DATA.setData ( dataVec ) ;

         enableComponents ( true ) ;
         resultPanel.revalidate() ;
      }
   }





   /*------------------------------------------------------------*/
   /* enableComponents
    */
   /**
    *    This method will be either enable
    *  or disable the components on this window
    *  
    *
    *  @param enable TRUE = enable
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void enableComponents ( boolean enabled ) {
      cb_BRAND.setEnabled       ( enabled ) ;
      st_OBJECT.setEnabled      ( enabled ) ;
      pb_BOOKMARKS.setEnabled   ( enabled ) ;
      pb_PRINT.setEnabled       ( enabled ) ;
      st_SEARCHARG.setEnabled   ( enabled ) ;
      st_SEARCH.setEnabled      ( enabled ) ;
      cb_OBJECT.setEnabled      ( enabled ) ;
      cb_SEARCHON.setEnabled    ( enabled ) ;
      cb_SEARCH.setEnabled      ( enabled ) ;
      ef_SEARCHARG.setEnabled   ( enabled ) ;
      findPanel.setEnabled      ( enabled ) ;
      resultPanel.setEnabled    ( enabled ) ;
      cnr_DATA.setEnabled       ( enabled ) ;
      pb_FIND.setEnabled        ( enabled ) ;
      pb_SELECT.setEnabled      ( enabled ) ;
      pb_EDIT.setEnabled        ( enabled ) ;
      pb_CANCEL.setEnabled      ( enabled ) ;
      pb_EXPORT.setEnabled      ( enabled ) ;
      buttonPanel.setEnabled    ( enabled ) ;
      ef_USER_SEARCH.setEnabled ( enabled ) ;

      if ( enabled )
         ck_SEARCHDRAFT.setEnabled ( allowDrafts ) ;
      else
         ck_SEARCHDRAFT.setEnabled ( false ) ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread just calls the 'find' method
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      find () ;
   }





   /*------------------------------------------------------------*/
   /* select
    */
   /**
    *    This method will get everything that is selected
    *  and will close the window, but not before
    *  notifying the listeners. If nothing is selected, nothing happens
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void select () {                                       
      Vector vec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are showing bookmarks, take data from tree        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( showingBookmarks ) {
         try {
            StringTreeNode selectedObject = (StringTreeNode) bookmarkTree.getSelectionPath().getLastPathComponent() ;                                         

            if ( selectedObject != null ) {
               Bookmark bookmark = (Bookmark) selectedObject.getData() ;

               if ( bookmark.isFolder() ) {
                  vec = BookmarkSystem.getBookmarks ( bookmark.getDescript() ) ;
               } else {
                  TreePath[] treePath = bookmarkTree.getSelectionPaths() ;

                  vec = new Vector ( 1, treePath.length ) ;
                  for ( int i = 0; i < treePath.length; i++ )
                     vec.addElement ( ((Bookmark)treePath[i].getLastPathComponent()).getObject() ) ;
               }

               setVisible ( false ) ;
            }
         } catch ( Exception e ) {
         }

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, take the data from the container             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         vec = cnr_DATA.getSelection() ;
      }


      super.setVisible ( false ) ;
      if ( vec != null ) {
         if ( listener != null )
            listener.selectOAComplete ( vec ) ;
         setVisible ( false ) ;
      }

   } /* select  */





   /*------------------------------------------------------------*/
   /* edit
    */
   /**
    *    This method will edit the currently selected
    *  Object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void edit () {
   }





   /*------------------------------------------------------------*/
   /* addSelectOAListener
    */
   /**
    *    This method will add a listener to this
    *  window
    *  
    *
    *  @param listener The SelectOAListener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addSelectOAListener ( SelectOAListener listener ) {
      this.listener = listener ;
   }




   /*------------------------------------------------------------*/
   /* setVisible
    */
   /**
    *    This method has been subclassed to clear
    *  our our listeners once we have been closed
    *  
    *
    *  @param visible TRUE = show, FALSE = do not show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setVisible ( boolean visible ) {
      if ( !visible )
         listener = null ;
      super.setVisible ( visible ) ;
   }





   /*------------------------------------------------------------*/
   /* setView
    */
   /**
    *    This method will set the view that we
    *  are currently displaying
    *  
    *
    *  @param viewIndex The view index. VIEW_XXX etc..
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setView ( int viewIndex ) {
      if ( cb_OBJECT != null ) {

         viewInd = viewIndex ;
         if ( ( cb_OBJECT.getSelectedIndex() != viewIndex ) && 
              ( cb_OBJECT.getItemAt(viewIndex) != null ) ) {
            cb_OBJECT.setSelectedIndex ( viewIndex ) ;
            refreshObjectData() ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* allowDrafts
    */
   /**
    *    This method will allow/disable the check box
    *  for drafts
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void allowDrafts ( boolean allowDrafts ) {
      ck_SEARCHDRAFT.setEnabled ( allowDrafts ) ;
      this.allowDrafts = allowDrafts ;
      if ( !allowDrafts )
         ck_SEARCHDRAFT.setSelected ( false ) ;
      setView ( viewInd ) ;
   }




   /*------------------------------------------------------------*/
   /* windowClosing
    */
   /**
    *    This method will be called when the window
    *  is closing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowClosing ( WindowEvent e) {
      setVisible ( false ) ;
   }

   public void windowClosed ( WindowEvent e ) {
   }

   public void windowOpened ( WindowEvent e ) {
   }

   public void windowIconified ( WindowEvent e ){
   }

   public void windowDeiconified ( WindowEvent e )  {
   }

   public void windowActivated ( WindowEvent e ) {
   }

   public void windowDeactivated ( WindowEvent e ) {
   }





   ///*------------------------------------------------------------*/
   ///* addSearchToHistory
   // */
   ///**
   // *    This method will add a search item to our
   // *  history
   // *  
   // *
   // *  @param None
   // *  @param None
   // *  @param None
   // *
   // *  @return Nothing
   // *//*---------------------------------------------------------*/
   //private void addSearchToHistory ( String search, int brandInd, String searchOn, SelectOAView view ) {
   //   if ( search.length() > 0 ) {
   //      int                      i          = 0 ;
   //      Vector                   history    = (Vector) PropertySystem.get ( PROPERTY_SEARCH_HISTORY ) ;
   //      SelectOASearchHistoryRec historyRec = new SelectOASearchHistoryRec ( search,
   //                                                                           brandInd,
   //                                                                           searchOn,
   //                                                                           view ) ;                     


   //      /*-----------------------------------------------------------*/
   //      /*                                                           */
   //      /*   Only keep the last 10 searches                          */
   //      /*                                                           */
   //      /*-----------------------------------------------------------*/
   //      if ( history.size() > 10 ) {
   //         while ( history.size() > 10 )
   //            history.removeElementAt ( history.size()-1 ) ;
   //      }

   //      if ( !history.contains ( historyRec ) ) {
   //         history.insertElementAt ( historyRec, 0 ) ;
   //         System.out.println ( "Adding to ComboBox:"+historyRec )  ;
   //         ef_SEARCHARG.addItem ( historyRec ) ;
   //      }

   //      PropertySystem.put ( PROPERTY_SEARCH_HISTORY, history ) ;
   //   }
   //}


   private void initializeMultiList () {
      cnr_DATA.setWordWrap ( true ) ;
      cnr_DATA.setOddLinesColor ( new Color ( 240, 240, 240 ) ) ;
      cnr_DATA.addActionListener ( this ) ;
   }
} /* SelectOADlg */
