package com.ibm.nzna.projects.qit.avalon.gui.selectview ;    
/*-----------------------------------------------------------*/
/*   LinkGroupView                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkGroupView                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import java.util.* ;
/**
 * This view will be loaded into the SelectOADlg and allows the user
 * to search through actions by title, author, indicator
 *
 * @version 1.0, 4/19/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkGroupView                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkGroupView extends Object implements java.io.Serializable, SelectOAView, AppConst {


   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method should return the user readable
    *  name of this view. This name will be put into
    *  the select oa dlg.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( Str.getStr ( STR_LINK_GROUP ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getSearchArg
    */
   /**
    *    This method should return the different
    *  arguments that this view is able to search on
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of Strings
    *//*---------------------------------------------------------*/
   public Vector getSearchArg () {
      Vector searchArgVec = new Vector ( 5, 1 ) ;

      searchArgVec.addElement ( Str.getStr ( STR_TITLE ) ) ;
      searchArgVec.addElement ( Str.getStr ( STR_AUTHOR ) ) ;
      searchArgVec.addElement ( Str.getStr ( STR_INTERNAL_NUM ) ) ;

      return( searchArgVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getSearchArgDisplay
    */
   /**
    *    This method should return one of the
    *  predefined values in SelectOADlg that tell
    *  what controls should be displayed. Currently these
    *  values are:
    *  SelectOADlg.ENTRYFIELD
    *  SelectOADlg.COMBO_BOX
    *  SelectOADlg.USER_ENTRYFIELD
    *
    *  @param searchArg The Search argument to return with
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getSearchArgDisplay ( String searchArg ) {
      if ( searchArg != null ) {
         if ( ( searchArg.equals ( Str.getStr ( STR_TITLE ) ) ) ||
              ( searchArg.equals ( Str.getStr ( STR_INTERNAL_NUM ) ) ) )
            return( SelectOADlg.ENTRYFIELD ) ;
         else if ( searchArg.equals ( Str.getStr ( STR_AUTHOR ) ) )
            return( SelectOADlg.USER_ENTRYFIELD ) ;
      }

      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method should return the data from
    *  the view specified.
    *  
    *
    *  @param searchArg The Search Argument that is needed
    *  @param qualifier The qualifier to use in the search
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getData ( String searchArg, String qualifier, String data, int brandInd ) {
      Vector  dataVec     = null ;

      try {
         boolean processSQL  = false ;
         boolean draft       = qualifier.toUpperCase().equals ( "OADRAFT" ) ;
         int     languageInd = PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ) ;
         String  sql         = "SELECT DISTINCT(A.LINKGROUPIND), B.TITLE, "+
                               "       A.DBUSER,     A.CHANGEDTIME "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKGROUPIND " : "" )+
                               "FROM   "+qualifier+".LINKGROUP      A, "+
                               "       "+qualifier+".LINKGROUPTITLE B  " ;


         if ( searchArg != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If Search Arg is TITLE                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( searchArg.equals ( Str.getStr ( STR_TITLE ) ) ) {
               String title = data ;

               if ( ( title != null ) && ( title.length() > 0 ) ) {

                  sql+="WHERE translate(B.TITLE) LIKE '%"+title.toUpperCase()+"%' " ;
                  processSQL = true ;

               } else
                  GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If Search Arg is Author                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( searchArg.equals ( Str.getStr ( STR_AUTHOR ) ) ) {
               UserRec userRec = UserSystem.getUserRecFromName ( data ) ;

               if ( userRec != null ) {

                  sql+="WHERE A.DBUSER = '"+userRec.getUserId()+"' " ;
                  processSQL = true ;

               } else
                  processSQL = false ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If Search Arg is Internal Number                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( searchArg.equals ( Str.getStr ( STR_INTERNAL_NUM ) ) ) {
               String num = data ;

               try {
                  if ( ( num != null ) && ( num.length() > 0 ) ) {

                     sql+="WHERE A.LINKGROUPIND = "+new Integer ( num ).intValue() ;
                     processSQL = true ;

                  } else
                     GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
               } catch ( Exception e ) {
                  GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
               }
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we process the SQL, do so now                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( processSQL ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the standard joins                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               sql+=" AND A.LINKGROUPIND = B.LINKGROUPIND "+
                    " AND B.LANGUAGEIND  = "+languageInd+" "+
                    ( draft ? "AND A.RECYCLED   = 'N' " : " " )+
                    ( draft ? "AND A.STOPDATE IS NULL " : " " ) ;


               if ( draft )
                  dataVec = OASQL.readDraftLinkGroupList ( sql+" FOR FETCH ONLY" ) ;
               else
                  dataVec = OASQL.readLinkGroupList ( sql+" FOR FETCH ONLY" ) ;
            }

         }
      } catch ( Exception e ) {
      }

      return( dataVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnHeadings
    */
   /**
    *    This method should return the column headings
    *  needed to display the view in a MultiList
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String[]
    *//*---------------------------------------------------------*/
   public String[] getColumnHeadings () {
      String[] titles = new String[6] ;

      titles[0] = "" ;
      titles[1] = "" ;
      titles[2] = Str.getStr ( STR_INTERNAL_NUM ) ;
      titles[3] = Str.getStr ( STR_TITLE )  ;
      titles[4] = Str.getStr ( STR_LAST_TOUCHED_ON ) ;
      titles[5] = Str.getStr ( STR_LAST_TOUCHED_BY ) ;

      return( titles ) ;
   }
} /* LinkGroupView */
