package com.ibm.nzna.projects.qit.avalon.gui.selectview ;    
/*-----------------------------------------------------------*/
/*   LinkView                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkView                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.gui.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import java.util.* ;
import java.sql.* ;
/**
 * This view will be loaded into the SelectOADlg and allows the user
 * to search through questions by title, author, indicator
 *
 * @version 1.0, 4/19/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkView                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkView extends Object implements java.io.Serializable, SelectOAView, AppConst {


   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method should return the user readable
    *  name of this view. This name will be put into
    *  the select oa dlg.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( Str.getStr ( STR_LINK ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getSearchArg
    */
   /**
    *    This method should return the different
    *  arguments that this view is able to search on
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of Strings
    *//*---------------------------------------------------------*/
   public Vector getSearchArg () {
      Vector searchArgVec = new Vector ( 5, 1 ) ;

      searchArgVec.addElement ( Str.getStr ( STR_TITLE ) ) ;
      searchArgVec.addElement ( Str.getStr ( STR_AUTHOR ) ) ;
      searchArgVec.addElement ( Str.getStr ( STR_LOTUS_NOTES_ID ) ) ;
      searchArgVec.addElement ( Str.getStr ( STR_INTERNAL_NUM ) ) ;
      searchArgVec.addElement ( Str.getStr ( STR_URL ) ) ;

      return( searchArgVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getSearchArgDisplay
    */
   /**
    *    This method should return one of the
    *  predefined values in SelectOADlg that tell
    *  what controls should be displayed. Currently these
    *  values are:
    *  SelectOADlg.ENTRYFIELD
    *  SelectOADlg.COMBO_BOX
    *  SelectOADlg.USER_ENTRYFIELD
    *
    *  @param searchArg The Search argument to return with
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getSearchArgDisplay ( String searchArg ) {
      if ( searchArg != null ) {
         if ( ( searchArg.equals ( Str.getStr ( STR_TITLE ) ) ) ||
              ( searchArg.equals ( Str.getStr ( STR_INTERNAL_NUM ) ) ) ||
              ( searchArg.equals ( Str.getStr ( STR_LOTUS_NOTES_ID ) ) ) ||
              ( searchArg.equals ( Str.getStr ( STR_URL ) ) ) )
            return( SelectOADlg.ENTRYFIELD ) ;
         else if ( searchArg.equals ( Str.getStr ( STR_AUTHOR ) ) )
            return( SelectOADlg.USER_ENTRYFIELD ) ;
      }

      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method should return the data from
    *  the view specified.
    *  
    *
    *  @param searchArg The Search Argument that is needed
    *  @param qualifier The qualifier to use in the search
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getData ( String searchArg, String qualifier, String data, int brandInd ) {
      boolean processSQL = false ;
      Vector  dataVec    = new Vector ( 100, 10 )  ;                                             


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user is looking for only Notes Ids, only search  */
      /* on Quest Links                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( searchArg.equals ( Str.getStr ( STR_LOTUS_NOTES_ID ) ) )
         getDocLinkData ( dataVec, searchArg, qualifier, data ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, get the URL Links and Doc Links              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         getDocLinkData ( dataVec, searchArg, qualifier, data ) ;
         getURLLinkData ( dataVec, searchArg, qualifier, data ) ;
      }



      new QuickSort ( dataVec ) ;

      return( dataVec ) ;

   }


   /*------------------------------------------------------------*/
   /* getDocLinkData
    */
   /**
    *    This method will look specifically for
    *  links that belong to a document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean getDocLinkData ( Vector dataVec, String searchArg, String qualifier, String data ) {
      String  sql        = null ;
      boolean rc         = false ;
      boolean draft      = qualifier.toUpperCase().equals ( "OADRAFT" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   STR_TITLE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( searchArg.equals ( Str.getStr ( STR_TITLE ) ) ) {
         if ( ( data != null ) && ( data.length() > 0 ) ) {
            sql = "SELECT A.LINKIND,  B.LNDOCID, A.DBUSER,  A.CHANGEDTIME, "+
                  "       A.URLVALID, A.DOCIND,  C.TITLE,   0  "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKIND " : "" )+
                  "FROM "+qualifier+".LINK  A, "+
                  "     TIGRIS.DOCUMENTS B,  "+
                  "     "+qualifier+".LINKTITLE C "+
                  "WHERE translate(C.TITLE)     LIKE '%"+data.toUpperCase()+"%' AND "+
                  "      A.DOCIND               = B.DOCIND AND "+
                  "      C.LINKIND              = A.LINKIND AND "+
                  "      A.LINKTYPE             = 0 AND "+
                  "      A.ARCHIVED             = 'N' "+
                  ( draft ? "  AND A.RECYCLED   = 'N' " : "" )+
                  ( draft ? "  AND A.STOPDATE IS NULL " : "" )+
                  "FOR FETCH ONLY" ;
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   STR_AUTHOR                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( searchArg.equals ( Str.getStr ( STR_AUTHOR ) ) ) {
         if ( ( data != null ) && ( data.length() > 0 ) ) {
            UserRec userRec = UserSystem.getUserRecFromName ( data ) ;

            if ( userRec != null ) {
               data = userRec.getUserId() ;
               sql = "SELECT A.LINKIND,  B.LNDOCID,   A.DBUSER,  A.CHANGEDTIME, "+
                     "       A.URLVALID, A.DOCIND,    C.TITLE,   0 "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKIND " : "" )+
                     "FROM "+qualifier+".LINK  A, "+
                     "     TIGRIS.DOCUMENTS B,  "+
                     "     "+qualifier+".LINKTITLE C "+
                     "WHERE A.DBUSER    = '"+data+"' AND "+
                     "      A.DOCIND    = B.DOCIND   AND "+
                     "      C.LINKIND   = A.LINKIND  AND "+
                     "      A.LINKTYPE  = 0          AND "+
                     "      A.ARCHIVED  = 'N'         "+
                     ( draft ? "AND A.STOPDATE IS NULL " : "" )+
                     ( draft ? "AND A.RECYCLED   = 'N' " : "" )+
                     "FOR FETCH ONLY" ;
            } else
               GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   STR_LOTUS_NOTES_ID                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( searchArg.equals ( Str.getStr ( STR_LOTUS_NOTES_ID ) ) ) {
         if ( ( data != null ) && ( data.length() > 0 ) ) {
            sql = "SELECT A.LINKIND,  B.LNDOCID, A.DBUSER,  A.CHANGEDTIME, "+
                  "       A.URLVALID, A.DOCIND,  C.TITLE,   0 "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKIND " : "" )+
                  "FROM "+qualifier+".LINK  A, "+
                  "     TIGRIS.DOCUMENTS B,  "+
                  "     "+qualifier+".LINKTITLE C "+
                  "WHERE B.LNDOCID     = '"+data+"' AND "+
                  "      A.DOCIND      = B.DOCIND AND "+
                  "      C.LINKIND     = A.LINKIND AND "+
                  "      A.ARCHIVED    = 'N' AND "+
                  "      A.LINKTYPE    = 0  "+
                  ( draft ? " AND A.STOPDATE IS NULL " : "" )+
                  ( draft ? " AND  A.RECYCLED   = 'N' " : "" )+
                  "FOR FETCH ONLY" ;
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   STR_INTERNAL_NUM                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( searchArg.equals ( Str.getStr ( STR_INTERNAL_NUM ) ) ) {
         if ( ( data != null ) && ( data.length() > 0 ) ) {
            try {
               sql = "SELECT A.LINKIND,  B.LNDOCID,  A.DBUSER,  A.CHANGEDTIME, "+
                     "       A.URLVALID, A.DOCIND,   C.TITLE,   0 "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKIND " : "" )+
                     "FROM "+qualifier+".LINK  A, "+
                     "     TIGRIS.DOCUMENTS B,  "+
                     "     "+qualifier+".LINKTITLE C "+
                     "WHERE A.LINKIND   = "+new Integer ( data ).intValue()+" AND "+
                     "      A.DOCIND    = B.DOCIND AND "+
                     "      C.LINKIND   = A.LINKIND AND "+
                     "      A.ARCHIVED    = 'N' "+
                     ( draft ? "  AND A.RECYCLED   = 'N' " : "" )+
                     ( draft ? "  AND A.STOPDATE IS NULL " : "" )+
                     "FOR FETCH ONLY" ;
            } catch ( Exception e ) {
               GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
            }
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
      }

      if ( sql != null )
         rc = getDataFromSQL ( dataVec, sql, 0 ) ;
      return( rc );
   }

   /*------------------------------------------------------------*/
   /* getURLLinkData
    */
   /**
    *    This method will look specifically for
    *  links that go to a URL
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean getURLLinkData ( Vector dataVec, String searchArg, String qualifier, String data ) {
      String  sql        = null ;
      boolean rc         = false ;
      boolean draft      = qualifier.toUpperCase().equals ( "OADRAFT" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   STR_TITLE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( searchArg.equals ( Str.getStr ( STR_TITLE ) ) ) {
         if ( ( data != null ) && ( data.length() > 0 ) ) {
            sql = "SELECT A.LINKIND,  URL,   A.DBUSER,  A.CHANGEDTIME, "+
                  "       A.URLVALID, 0,     C.TITLE, 1 "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKIND " : "" )+
                  "FROM "+qualifier+".LINK  A, "+
                  "     "+qualifier+".LINKTITLE C "+
                  "WHERE translate(C.TITLE)     LIKE '%"+data.toUpperCase()+"%' AND "+
                  "      C.LINKIND   = A.LINKIND       AND "+
                  "      A.LINKTYPE  = 1               AND "+
                  ( draft ? " A.RECYCLED   = 'N' AND " : "" )+
                  ( draft ? " A.STOPDATE IS NULL AND " : "" )+
                  "      A.ARCHIVED    = 'N' "+
                  "FOR FETCH ONLY" ;
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   STR_AUTHOR                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( searchArg.equals ( Str.getStr ( STR_AUTHOR ) ) ) {
         if ( ( data != null ) && ( data.length() > 0 ) ) {
            UserRec userRec = UserSystem.getUserRecFromName ( data ) ;

            if ( userRec != null ) {
               data = userRec.getUserId() ;
               sql = "SELECT A.LINKIND,  URL,  A.DBUSER,  A.CHANGEDTIME, "+
                     "       A.URLVALID, 0,    C.TITLE, 1 "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKIND " : "" )+
                     "FROM "+qualifier+".LINK  A, "+
                     "     "+qualifier+".LINKTITLE C "+
                     "WHERE A.DBUSER    = '"+data+"'  AND "+
                     "      C.LINKIND   = A.LINKIND   AND "+
                     "      A.LINKTYPE  = 1           AND "+
                     ( draft ? "   A.RECYCLED   = 'N' AND " : "" )+
                     ( draft ? "   A.STOPDATE IS NULL AND " : "" )+
                     "      A.ARCHIVED  = 'N' "+
                     "FOR FETCH ONLY" ;
            } else
               GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   STR_INTERNAL_NUM                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( searchArg.equals ( Str.getStr ( STR_INTERNAL_NUM ) ) ) {
         if ( ( data != null ) && ( data.length() > 0 ) ) {
            try {
               sql = "SELECT A.LINKIND,  URL,   A.DBUSER,  A.CHANGEDTIME, "+
                     "       A.URLVALID, 0,     C.TITLE,   1 "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKIND " : "" )+
                     "FROM "+qualifier+".LINK  A, "+
                     "     "+qualifier+".LINKTITLE C "+
                     "WHERE A.LINKIND   = "+(new Integer ( data ).intValue())+" AND "+
                     "      A.LINKTYPE  = 1 AND "+
                     "      C.LINKIND   = A.LINKIND AND "+
                     ( draft ? "   A.RECYCLED   = 'N' AND " : "" )+
                     ( draft ? "   A.STOPDATE IS NULL AND " : "" )+
                     "      A.ARCHIVED    = 'N' "+
                     "FOR FETCH ONLY" ;
            } catch ( Exception e ) {
               GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
            }
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   URL                                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( searchArg.equals ( Str.getStr ( STR_URL ) ) ) {
         if ( ( data != null ) && ( data.length() > 0 ) ) {
            sql = "SELECT A.LINKIND,  URL,   A.DBUSER,  A.CHANGEDTIME, "+
                  "       A.URLVALID, 0,     C.TITLE,   1 "+(draft ? ",A.STARTDATE, A.STOPDATE, A.ACTIVELINKIND " : "" )+
                  "FROM "+qualifier+".LINK  A, "+
                  "     "+qualifier+".LINKTITLE C "+
                  "WHERE translate(A.URL) = '"+data.toUpperCase()+"' AND "+
                  "      A.LINKTYPE  = 1 AND "+
                  "      C.LINKIND   = A.LINKIND AND "+
                  ( draft ? "   A.RECYCLED   = 'N' AND " : "" )+
                  ( draft ? "   A.STOPDATE IS NULL AND " : "" )+
                  "      A.ARCHIVED    = 'N' "+
                  "FOR FETCH ONLY" ;
         } else
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;

      }

      if ( sql != null ) {
         if ( sql != null )
            rc = getDataFromSQL ( dataVec, sql, 1 ) ;

      }

      return( rc );
   }





   /*------------------------------------------------------------*/
   /* getColumnHeadings
    */
   /**
    *    This method should return the column headings
    *  needed to display the view in a MultiList
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String[]
    *//*---------------------------------------------------------*/
   public String[] getColumnHeadings () {
      String[] titles = new String[7] ;

      titles[0] = "" ;
      titles[1] = "" ;
      titles[2] = Str.getStr ( STR_INTERNAL_NUM ) ;
      titles[3] = Str.getStr ( STR_TITLE )  ;
      titles[4] = Str.getStr ( STR_LAST_TOUCHED_ON ) ;
      titles[5] = Str.getStr ( STR_LAST_TOUCHED_BY ) ;
      titles[6] = Str.getStr ( STR_TARGET ) ;

      return( titles ) ;
   }





   /*------------------------------------------------------------*/
   /* getDataFromSQL
    */
   /**
    *    This method will run the SQL passed and
    *  will return the data associated with it
    *  
    *
    *  @param vector The vector to add items to
    *  @param SQL the SQL to run
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean getDataFromSQL ( Vector dataVec, String sql, int linkType ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDataFromSQL ( "+sql+" )", 5 ) ;
      boolean   rc        = false ;
      boolean   draft     = ( sql.indexOf ( "OADRAFT" ) >= 0 ) ;
      try {
         LinkDisplayRec readRec = null ;
         Statement      stmt    = sqlMethod.createStatement() ;
         ResultSet      rs      = stmt.executeQuery ( sql ) ;

         while ( rs.next() ) {
            if ( draft )
               dataVec.addElement ( new LinkDraftDisplayRec ( rs.getInt(1),                                      // Link Indicator
                                                              0,
                                                              rs.getString(7).trim(),                            // Title
                                                              ( rs.getString(5) == null ) ? false : rs.getString(5).trim().equals ( "Y" ),             // Valid
                                                              linkType,                                          // Link Type
                                                              (linkType == 0) ? rs.getInt(6) : 0,                // If reading docs, the doc ind
                                                              (linkType == 0) ? rs.getString(2) : null,          // Doc Filename
                                                              (linkType == 1) ? rs.getString(2) : null,          // If reading URL, the URL
                                                              rs.getString ( 3 ).trim() ,                        // Database User
                                                              rs.getString ( 4 ).trim(),                         // Changed Time
                                                              "", "" ) ) ;                                       // Start Date/Stop Date
            else
               dataVec.addElement ( new LinkDisplayRec ( rs.getInt(1),                                      // Link Indicator
                                                         rs.getString(7).trim(),                            // Title
                                                         ( rs.getString(5) == null ) ? false : rs.getString(5).trim().equals ( "Y" ),             // Valid
                                                         linkType,                                          // Link Type
                                                         (linkType == 0) ? rs.getInt(6) : 0,                // If reading docs, the doc ind
                                                         (linkType == 0) ? rs.getString(2) : null,          // Doc Filename
                                                         (linkType == 1) ? rs.getString(2) : null,          // If reading URL, the URL
                                                         rs.getString ( 3 ).trim() ,                        // Database User
                                                         rs.getString ( 4 ).trim() ) ) ;                      // Changed time
         }
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;
      return( rc ) ;
   }


} /* LinkView */
