package com.ibm.nzna.projects.qit.avalon.oanav ;     
/*-----------------------------------------------------------*/
/*   OANavNavPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OANavNavPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.qit.avalon.oanav.views.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This is the Nav panel for the OA Navigator. 
 *
 *
 * @version 1.0, 3/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OANavNavPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class OANavNavPanel extends JPanel implements CustomViewListener, AvalonConst, NavPanel, AppConst, ActionListener {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private NavListGroup    activeListView           = null ;
   private NavListGroup    activeActionList         = null ;
   private NavListGroup    activeQuestionList       = null ;
   private NavListGroup    activeLinkList           = null ;
   private NavListGroup    activeLinkGroupList      = null ;
   private NavListGroup    activeSymptomList        = null ;

   private NavListGroup    draftListView            = null ;
   private NavListGroup    draftActionList          = null ;
   private NavListGroup    draftQuestionList        = null ;
   private NavListGroup    draftLinkList            = null ;
   private NavListGroup    draftLinkGroupList       = null ;
   private NavListGroup    draftSymptomList         = null ;

   private NavListGroup    activityListView         = null ;
   private NavListGroup    newDraftActivityList     = null ;
   private NavListGroup    closedDraftActivityList  = null ;

   private NavListGroup    archivedListView         = null ;
   private NavListGroup    orphanedListView         = null ;
   private NavListGroup    cprListView              = null ;

   private NavListGroup    customQueryView          = null ;
   private NavListGroup    customQueryQuestionView  = null ;
   private NavListGroup    customQueryActionView    = null ;
   private NavListGroup    customQuerySymptomView   = null ;
   private NavListGroup    customQueryLinkView      = null ;
   private NavListGroup    customQueryLinkGroupView = null ;

   private OANavPanel      navPanel             = null ;
   private JComboBox       cb_BRANDS            = null ;
   private NavListGroup    draftGroup           = null ;
   private NavListGroup    activeGroup          = null ;
   private NavList         navList              = null ;
   private Vector          customViewComponents = new Vector ( 1, 1 ) ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method does nothing but assign our
    *  local vars
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public OANavNavPanel ( OANavPanel navPanel ) {
      this.navPanel = navPanel ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel. This includes
    *  creating all of our controls and attaching
    *  them to this panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {
      JScrollPane scrollPane = null ;     
      Vector      brands     = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_BRAND ).clone() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navList    = new NavList() ;
      scrollPane = new JScrollPane ( navList ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our views                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      createActiveViews      ( ) ;
      createDraftViews       ( ) ;
      createActivityViews    ( ) ;
      createArchivedViews    ( ) ;
      createOrphanViews      ( ) ;
      createCPRViews         ( ) ;
      createCustomQueryViews ( ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our other views                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      brands.insertElementAt ( new TypeBrandRec ( -1, Str.getStr ( STR_ALL ), 0 ), 0 ) ;
      cb_BRANDS = new JComboBox ( brands ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_BRANDS.addActionListener ( this ) ;
      CustomViewSystem.addCustomViewListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout(5,5) ) ;
      add ( cb_BRANDS,  BorderLayout.NORTH ) ;
      add ( scrollPane, BorderLayout.CENTER ) ;


   }




   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing. We will do nothing here
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {

      try {
         TypeBrandRec brandRec =  null ;               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Select the default brand                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         brandRec = (TypeBrandRec) TypeList.getInstance().objectFromInd ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEFAULT_BRAND ),
                                                                          TypeList.TYPE_BRAND ) ;
         if ( brandRec != null ) {
            cb_BRANDS.setSelectedItem ( brandRec ) ;
            navPanel.setBrand ( brandRec ) ;
         } else
            navPanel.setBrand ( (TypeBrandRec) cb_BRANDS.getSelectedItem() ) ;

      } catch ( Exception e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will remove all of our controls
    *  and will free all of our resources
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      CustomViewSystem.removeCustomViewListener ( this ) ;
      removeAll() ;
   }


   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title of this nav
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_VIEWS ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will simply return this class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* createActiveViews
    */
   /**
    *    This method will create all of the necessary
    *  views for Active views.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createActiveViews ( ) {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Base views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeListView = new NavListGroup ( Str.getStr ( STR_ACTIVE ), ImageSystem.getImageIcon ( this, ImageSystem.ACTIVE_OBJECT ) ) ;
      navList.add ( activeListView ) ;              


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Action Views                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeActionList = new NavListGroup ( Str.getStr ( STR_ACTIONS ), ImageSystem.getImageIcon ( this, ImageSystem.ACTION_SMALL ) ) ;
      activeListView.add ( activeActionList ) ;
      activeActionList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ACTION_TYPE ),
                                                null,
                                                OAViews.VIEW_ACTION_TYPE ) ) ;
      activeActionList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ACTION_TITLE ),
                                                null,
                                                OAViews.VIEW_ACTION_TITLE ) ) ;            


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Link Views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeLinkList = new NavListGroup ( Str.getStr ( STR_DOC_LINKS ), ImageSystem.getImageIcon ( this, ImageSystem.DOC_LINK ) ) ;
      activeListView.add ( activeLinkList ) ;
      activeLinkList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DOCLINK_TYPE ),
                                              null,
                                              OAViews.VIEW_DOCLINK_TYPE ) ) ;
      activeLinkList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DOCLINK_STATUS ),
                                              null,
                                              OAViews.VIEW_DOCLINK_STATUS ) ) ;                  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Link Group Views                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeLinkGroupList = new NavListGroup ( Str.getStr ( STR_LINK_GROUPS ), ImageSystem.getImageIcon ( this, ImageSystem.DOC_LINK ) ) ;
      activeListView.add ( activeLinkGroupList ) ;
      activeLinkGroupList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DOCLINK_GROUP_TITLE ),
                                                   null,
                                                   OAViews.VIEW_DOCLINK_GROUP_TITLE ) ) ;                


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Question Views                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeQuestionList = new NavListGroup ( Str.getStr ( STR_QUESTIONS ), ImageSystem.getImageIcon ( this, ImageSystem.QUESTION_SMALL ) ) ;
      activeListView.add ( activeQuestionList ) ;
      activeQuestionList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_QUESTION_TYPE ),
                                                  null,
                                                  OAViews.VIEW_QUESTION_TYPE ) ) ;
      activeQuestionList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_QUESTION_TITLE ),
                                                  null,
                                                  OAViews.VIEW_QUESTION_TITLE ) ) ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Symptom Views                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeSymptomList = new NavListGroup ( Str.getStr ( STR_SYMPTOMS ), ImageSystem.getImageIcon ( this, ImageSystem.SYMPTOM_SMALL ) ) ;
      activeListView.add ( activeSymptomList ) ;
      activeSymptomList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_SYMPTOM_TYPE ),
                                                 null,
                                                 OAViews.VIEW_SYMPTOM_TYPE ) ) ;
      activeSymptomList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_SYMPTOM_STATUS ),
                                                 null,
                                                 OAViews.VIEW_SYMPTOM_STATUS ) ) ;
      activeSymptomList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_SYMPTOM_TITLE ),
                                                 null,
                                                 OAViews.VIEW_SYMPTOM_TITLE ) ) ;                            





      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners to all nav lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeActionList.addActionListener    ( this ) ;
      activeQuestionList.addActionListener  ( this ) ;
      activeLinkList.addActionListener      ( this ) ;
      activeLinkGroupList.addActionListener ( this ) ;
      activeSymptomList.addActionListener   ( this ) ;


   }


   /*------------------------------------------------------------*/
   /* createDraftViews
    */
   /**
    *    This method will create all of the necessary
    *  views for Draft views.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createDraftViews ( ) {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Base views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftListView = new NavListGroup ( Str.getStr ( STR_DRAFTS ), ImageSystem.getImageIcon ( this, ImageSystem.DRAFT_OBJECT ) ) ;
      navList.add ( draftListView ) ;              


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Action Views                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftActionList = new NavListGroup ( Str.getStr ( STR_ACTIONS ), ImageSystem.getImageIcon ( this, ImageSystem.ACTION_SMALL ) ) ;
      draftListView.add ( draftActionList ) ;
      draftActionList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_ACTION_TYPE ),
                                               null,
                                               OAViews.VIEW_DRAFT_ACTION_TYPE ) ) ;
      draftActionList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_ACTION_TITLE ),
                                               null,
                                               OAViews.VIEW_DRAFT_ACTION_TITLE ) ) ;            


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Link Views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftLinkList = new NavListGroup ( Str.getStr ( STR_DOC_LINKS ), ImageSystem.getImageIcon ( this, ImageSystem.DOC_LINK ) ) ;
      draftListView.add ( draftLinkList ) ;
      draftLinkList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_DOCLINK_TYPE ),
                                             null,
                                             OAViews.VIEW_DRAFT_DOCLINK_TYPE ) ) ;
      draftLinkList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_DOCLINK_STATUS ),
                                             null,
                                             OAViews.VIEW_DRAFT_DOCLINK_STATUS ) ) ;                  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Link Group Views                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftLinkGroupList = new NavListGroup ( Str.getStr ( STR_LINK_GROUPS ), ImageSystem.getImageIcon ( this, ImageSystem.DOC_LINK ) ) ;
      draftListView.add ( draftLinkGroupList ) ;
      draftLinkGroupList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_DOCLINK_GROUP_TITLE ),
                                                  null,
                                                  OAViews.VIEW_DRAFT_DOCLINK_GROUP_TITLE ) ) ;                


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Question Views                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftQuestionList = new NavListGroup ( Str.getStr ( STR_QUESTIONS ), ImageSystem.getImageIcon ( this, ImageSystem.QUESTION_SMALL ) ) ;
      draftListView.add ( draftQuestionList ) ;
      draftQuestionList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_QUESTION_TYPE ),
                                                 null,
                                                 OAViews.VIEW_DRAFT_QUESTION_TYPE ) ) ;
      draftQuestionList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_QUESTION_TITLE ),
                                                 null,
                                                 OAViews.VIEW_DRAFT_QUESTION_TITLE ) ) ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Symptom Views                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftSymptomList = new NavListGroup ( Str.getStr ( STR_SYMPTOMS ), ImageSystem.getImageIcon ( this, ImageSystem.SYMPTOM_SMALL ) ) ;
      draftListView.add ( draftSymptomList ) ;
      draftSymptomList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_SYMPTOM_TYPE ),
                                                null,
                                                OAViews.VIEW_DRAFT_SYMPTOM_TYPE ) ) ;
      draftSymptomList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_SYMPTOM_STATUS ),
                                                null,
                                                OAViews.VIEW_DRAFT_SYMPTOM_STATUS ) ) ;
      draftSymptomList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_DRAFT_SYMPTOM_TITLE ),
                                                null,
                                                OAViews.VIEW_DRAFT_SYMPTOM_TITLE ) ) ;                            





      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners to all nav lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftActionList.addActionListener    ( this ) ;
      draftQuestionList.addActionListener  ( this ) ;
      draftLinkList.addActionListener      ( this ) ;
      draftLinkGroupList.addActionListener ( this ) ;
      draftSymptomList.addActionListener   ( this ) ;


   }




   /*------------------------------------------------------------*/
   /* createActivityViews
    */
   /**
    *    This method will create all of the necessary
    *  views for Activity views.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createActivityViews ( ) {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Base views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activityListView = new NavListGroup ( Str.getStr ( STR_ACTIVITY_LOG ), ImageSystem.getImageIcon ( this, ImageSystem.EDIT ) ) ;
      navList.add ( activityListView ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the 'New Drafts' View                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      newDraftActivityList = new NavListGroup ( Str.getStr ( STR_NEW_DRAFTS ) ) ;
      activityListView.add ( newDraftActivityList ) ;
      newDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_ACTION ),
                                                    null,
                                                    OAViews.VIEW_NEW_DRAFT_ACTIVITY_ACTION ) ) ;
      newDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_LINK ),
                                                    null,
                                                    OAViews.VIEW_NEW_DRAFT_ACTIVITY_LINK ) ) ;
      newDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_LINKGROUP ),
                                                    null,
                                                    OAViews.VIEW_NEW_DRAFT_ACTIVITY_LINKGROUP ) ) ;
      newDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_QUESTION ),
                                                    null,
                                                    OAViews.VIEW_NEW_DRAFT_ACTIVITY_QUESTION ) ) ;
      newDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_SYMPTOM ),
                                                    null,
                                                    OAViews.VIEW_NEW_DRAFT_ACTIVITY_SYMPTOM ) ) ;                      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the 'Closed' Drafts view                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      closedDraftActivityList = new NavListGroup ( Str.getStr ( STR_CLOSED_DRAFTS ) ) ;
      activityListView.add ( closedDraftActivityList ) ;
      closedDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_ACTION ),
                                                       null,
                                                       OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_ACTION ) ) ;
      closedDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_LINK ),
                                                       null,
                                                       OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_LINK ) ) ;
      closedDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_LINKGROUP ),
                                                       null,
                                                       OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_LINKGROUP ) ) ;
      closedDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_QUESTION ),
                                                       null,
                                                       OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_QUESTION ) ) ;
      closedDraftActivityList.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_SYMPTOM ),
                                                       null,
                                                       OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_SYMPTOM ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners to all nav lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      newDraftActivityList.addActionListener    ( this ) ;
      closedDraftActivityList.addActionListener  ( this ) ;


   }



   /*------------------------------------------------------------*/
   /* createArchivedViews
    */
   /**
    *    This method will create all of the necessary
    *  views for the archived views.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createArchivedViews ( ) {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Base views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      archivedListView = new NavListGroup ( Str.getStr ( STR_ARCHIVED ), ImageSystem.getImageIcon ( this, ImageSystem.SMALL_KEYWORD ) ) ;
      navList.add ( archivedListView ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Archived Views                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      archivedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ARCHIVED_ACTION ),
                                                null,
                                                OAViews.VIEW_ARCHIVED_ACTION ) ) ;
      archivedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ARCHIVED_LINK ),
                                                null,
                                                OAViews.VIEW_ARCHIVED_LINK ) ) ;
      archivedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ARCHIVED_LINKGROUP ),
                                                null,
                                                OAViews.VIEW_ARCHIVED_LINKGROUP ) ) ;
      archivedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ARCHIVED_QUESTION ),
                                                null,
                                                OAViews.VIEW_ARCHIVED_QUESTION ) ) ;
      archivedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ARCHIVED_SYMPTOM ),
                                                null,
                                                OAViews.VIEW_ARCHIVED_SYMPTOM ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners to all nav lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      archivedListView.addActionListener  ( this ) ;


   }





   /*------------------------------------------------------------*/
   /* createOrphanViews
    */
   /**
    *    This method will create all of the necessary
    *  views for the orphaned views.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createOrphanViews ( ) {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Base views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      orphanedListView = new NavListGroup ( Str.getStr ( STR_ORPHANED ), ImageSystem.getImageIcon ( this, ImageSystem.SMALL_KEYWORD ) ) ;
      navList.add ( orphanedListView ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Orphaned Views                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      orphanedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ORPHANED_ACTION ),
                                                null,
                                                OAViews.VIEW_ORPHANED_ACTION ) ) ;
      orphanedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ORPHANED_LINK ),
                                                null,
                                                OAViews.VIEW_ORPHANED_LINK ) ) ;
      orphanedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ORPHANED_LINKGROUP ),
                                                null,
                                                OAViews.VIEW_ORPHANED_LINKGROUP ) ) ;
      orphanedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ORPHANED_QUESTION ),
                                                null,
                                                OAViews.VIEW_ORPHANED_QUESTION ) ) ;
      orphanedListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_ORPHANED_SYMPTOM ),
                                                null,
                                                OAViews.VIEW_ORPHANED_SYMPTOM ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners to all nav lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      orphanedListView.addActionListener  ( this ) ;


   }





   /*------------------------------------------------------------*/
   /* createCPRViews
    */
   /**
    *    This method will create all of the necessary
    *  views for the cpr views.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createCPRViews ( ) {            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Base views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cprListView = new NavListGroup ( Str.getStr ( STR_CHANGES_PENDING_REVIEW ), ImageSystem.getImageIcon ( this, ImageSystem.SAVE ) ) ;
      navList.add ( cprListView ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the CPR Views                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cprListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CPR_ACTION ),
                                           null,
                                           OAViews.VIEW_CPR_ACTION ) ) ;
      cprListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CPR_LINK ),
                                           null,
                                           OAViews.VIEW_CPR_LINK ) ) ;
      cprListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CPR_LINKGROUP ),
                                           null,
                                           OAViews.VIEW_CPR_LINKGROUP ) ) ;
      cprListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CPR_QUESTION ),
                                           null,
                                           OAViews.VIEW_CPR_QUESTION ) ) ;
      cprListView.add ( new HotLinkLabel ( OAViews.getViewShortName ( OAViews.VIEW_CPR_SYMPTOM ),
                                           null,
                                           OAViews.VIEW_CPR_SYMPTOM ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners to all nav lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cprListView.addActionListener  ( this ) ;


   }





   /*------------------------------------------------------------*/
   /* createCustomQueryViews
    */
   /**
    *    This method will create all of the necessary
    *  views for the custom queries
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createCustomQueryViews ( ) {            
      Vector viewVec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Base views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      customQueryView = new NavListGroup ( Str.getStr ( STR_CUSTOM_VIEWS ), ImageSystem.getImageIcon ( this, ImageSystem.SMALL_ADMIN ) ) ;
      navList.add ( customQueryView ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the main custom query views                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      customQueryView.add ( customQueryActionView    = new NavListGroup ( Str.getStr ( STR_ACTIONS ) ) ) ;
      customQueryView.add ( customQueryLinkView      = new NavListGroup ( Str.getStr ( STR_DOC_LINKS ) ) ) ;
      customQueryView.add ( customQueryLinkGroupView = new NavListGroup ( Str.getStr ( STR_LINK_GROUPS ) ) ) ;
      customQueryView.add ( customQueryQuestionView  = new NavListGroup ( Str.getStr ( STR_QUESTIONS ) ) ) ;
      customQueryView.add ( customQuerySymptomView   = new NavListGroup ( Str.getStr ( STR_SYMPTOMS ) ) ) ;

      viewVec = CustomViewSystem.getViews ( CUSTOMVIEW_ACTION ) ;
      addCustomQueries ( customQueryActionView, viewVec ) ;
      viewVec = CustomViewSystem.getViews ( CUSTOMVIEW_LINK ) ;
      addCustomQueries ( customQueryLinkView, viewVec ) ;
      viewVec = CustomViewSystem.getViews ( CUSTOMVIEW_LINKGROUP ) ;
      addCustomQueries ( customQueryLinkGroupView, viewVec ) ;
      viewVec = CustomViewSystem.getViews ( CUSTOMVIEW_QUESTION ) ;
      addCustomQueries ( customQueryQuestionView, viewVec ) ;
      viewVec = CustomViewSystem.getViews ( CUSTOMVIEW_SYMPTOM ) ;
      addCustomQueries ( customQuerySymptomView, viewVec ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners to all nav lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      customQueryActionView.addActionListener  ( this ) ;
      customQueryQuestionView.addActionListener  ( this ) ;
      customQueryLinkView.addActionListener  ( this ) ;
      customQueryLinkGroupView.addActionListener  ( this ) ;
      customQuerySymptomView.addActionListener  ( this ) ;


   } /* createCustomQueryViews  */





   /*------------------------------------------------------------*/
   /* addCustomQueries
    */
   /**
    *    This method will add custom queries to
    *  the nav list specified. The queries should be in
    *  the queryVec passed
    *
    *  @param navList The nav list to add the views to
    *  @param queryVec The vector containing CustomView records
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addCustomQueries ( NavListGroup navList, Vector queryVec ) {
      if ( ( queryVec != null ) && ( queryVec.size() > 0 ) ) {
         int           i            = 0 ;
         int           size         = queryVec.size() ;
         HotLinkLabel  hotLinkLabel = null ;

         while ( i < size ) {

            navList.add ( hotLinkLabel = new HotLinkLabel ( ((CustomView)queryVec.elementAt(i)).getDescript() ,
                                                            null,
                                                            "CUSTOM_VIEW" ) ) ;
            customViewComponents.addElement ( hotLinkLabel ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() instanceof HotLinkLabel ) {
         String actionCommand = ((HotLinkLabel)e.getSource()).getActionCommand() ;                                        

         /*-------------------------------------------------------------------------*/
         /*                                                                         */
         /*   If this is a custom view, it's action command will be CUSTOM_VIEW     */
         /*                                                                         */
         /*-------------------------------------------------------------------------*/
         if ( actionCommand.equals ( "CUSTOM_VIEW" ) )
            navPanel.setCustomView ( ((HotLinkLabel)e.getSource()).getText() );
         else
            navPanel.setView ( actionCommand ) ;
      }

      else if ( e.getSource() == cb_BRANDS ) {
         TypeBrandRec brandRec = (TypeBrandRec) cb_BRANDS.getSelectedItem() ;

         navPanel.setBrand ( brandRec ) ;
      }
   } /* actionPerformed  */





   /*------------------------------------------------------------*/
   /* customViewRemoved
    */
   /**
    *    This method will be called when the user
    *  erases a custom view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewRemoved ( CustomView customView ) {
      HotLinkLabel label = hotLinkFromCustomView ( customView ) ;
      
      if ( label != null ) 
         label.getParent().remove ( label ) ;
      revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* customViewAdded
    */
   /**
    *    This method is called when the user adds
    *  a custom view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewAdded ( CustomView customView ) {
      HotLinkLabel hotLinkLabel = null ;

      switch ( customView.getViewType() ) {
         case CUSTOMVIEW_ACTION :
            customQueryActionView.add ( hotLinkLabel = new HotLinkLabel ( customView.getDescript(), null, "CUSTOM_VIEW" ) ) ;
            break ;

         case CUSTOMVIEW_QUESTION :
            customQueryQuestionView.add ( hotLinkLabel = new HotLinkLabel ( customView.getDescript(), null, "CUSTOM_VIEW" ) ) ;
            break ;

         case CUSTOMVIEW_LINK :
            customQueryLinkView.add ( hotLinkLabel = new HotLinkLabel ( customView.getDescript(), null, "CUSTOM_VIEW" ) ) ;
            break ;

         case CUSTOMVIEW_LINKGROUP :
            customQueryLinkGroupView.add ( hotLinkLabel = new HotLinkLabel ( customView.getDescript(), null, "CUSTOM_VIEW" ) ) ;
            break ;

         case CUSTOMVIEW_SYMPTOM :
            customQuerySymptomView.add ( hotLinkLabel = new HotLinkLabel ( customView.getDescript(), null, "CUSTOM_VIEW" ) ) ;
            break ;
      }

      if ( hotLinkLabel != null )
         customViewComponents.addElement ( hotLinkLabel ) ;
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* customViewUpdated
    */
   /**
    *    This method will be called when a custom view
    *  is updated
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewUpdated ( CustomView customView ) {
      HotLinkLabel label = hotLinkFromCustomView ( customView ) ;
      
      if ( label != null ) 
         label.setText ( customView.getDescript() ) ;
      revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* hotLinkFromCustomView
    */
   /**
    *    This method will go through all of the
    *  custom view components and will return
    *  the HotLinkLabel for the custom view
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return HotLinkLabel
    *//*---------------------------------------------------------*/
   public HotLinkLabel hotLinkFromCustomView ( CustomView customView ) {
      int          i        = 0 ;
      int          size     = customViewComponents.size() ;
      HotLinkLabel retLabel = null ;

      while ( ( retLabel != null ) && ( i < size ) ) {
         if ( ((HotLinkLabel)customViewComponents.elementAt(i)).getText().equals ( customView.getDescript() ) )
            retLabel = (HotLinkLabel)customViewComponents.elementAt(i) ;

         i++ ;
      }

      return( retLabel ) ;
   }
} /* OANavNavPanel */
