package com.ibm.nzna.projects.qit.avalon.oanav ;                          
/*-----------------------------------------------------------*/
/*   RefreshTreeViewThread                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: RefreshTreeViewThread               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.qit.avalon.oanav.views.* ;
import com.ibm.nzna.projects.qit.avalon.sql.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.sort.* ;
/**
 * This thread will take the path of the currently expanded tree node 
 * and will run the SQL needed to refresh the view. The data will then
 * be populated from this thread
 *
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   RefreshTreeViewThread                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class RefreshTreeViewThread extends Object implements QuestProcess, AvalonConst, OASort, AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Refresh Types                                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int REFRESH_PREBUILT_VIEW  = 1 ;
   private static final int REFRESH_CUSTOM_VIEW    = 2 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private TreePath         paths           = null ;
   private String           view            = null ;
   private CustomView       customView      = null ;
   private DefaultTreeModel model           = null ;
   private AppDefaultWin    defaultWin      = null ;
   private int              refreshType     = 0 ;
   private boolean          continueProcess = true ;
   private static OANavPanel oaNavPanel     = null ;






   /*------------------------------------------------------------*/
   /* RefreshTreeViewThread
    */
   /**
    *    This default constructor does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public RefreshTreeViewThread () {
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the thread and will
    *  start it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public RefreshTreeViewThread ( AppDefaultWin defaultWin, DefaultTreeModel model, TreePath paths, String view, OANavPanel oaNavPanel ) {
      this.paths      = paths ;
      this.oaNavPanel = oaNavPanel ;
      this.defaultWin = defaultWin ;
      this.model      = model ;
      this.view       = view ;
      refreshType     = REFRESH_PREBUILT_VIEW ;
      new Thread ( this ).start () ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the thread and will
    *  start it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public RefreshTreeViewThread ( AppDefaultWin defaultWin, DefaultTreeModel model, CustomView customView, OANavPanel oaNavPanel ) {
      this.defaultWin = defaultWin ;
      this.customView = customView ;
      this.model      = model ;
      this.oaNavPanel = oaNavPanel ;
      refreshType = REFRESH_CUSTOM_VIEW ;
      new Thread ( this ).start () ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform our thread
    *  operation
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create ourselves as a stoppable process                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ProcessManagerSystem.addProcess ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Prebuilt Views                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( refreshType == REFRESH_PREBUILT_VIEW )
         refreshPrebuiltView() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Custom Views                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( refreshType == REFRESH_CUSTOM_VIEW )
         refreshCustomView () ;

      if ( continueProcess )
         ProcessManagerSystem.removeProcess ( this ) ;
   } /* run  */





   /*------------------------------------------------------------*/
   /* refreshCustomView
    */
   /**
    *    This method will refresh a custom view
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshCustomView () {
      Vector dataVec = null ;

      defaultWin.setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
      Qit.setEnabled ( false ) ;
      dataVec = customView.getResults() ;

      populateTree ( model, (StringTreeNode)model.getRoot(), dataVec ) ;

      if ( continueProcess ) {
         Qit.setEnabled ( true ) ;
         defaultWin.setStatus ( null ) ;
      }


   }



   /*------------------------------------------------------------*/
   /* refreshPrebuiltView
    */
   /**
    *    This method will refresh the JTree with
    *  one of the many pre built views
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshPrebuiltView () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, try to get the selected object                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         Object   selectedObject = null ;                           


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we were given a good path, use it                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( paths != null ) {
            selectedObject = paths.getLastPathComponent() ;     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   StringTreeNode                                          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( selectedObject instanceof StringTreeNode ) {
               StringTreeNode stringTreeNode = (StringTreeNode) selectedObject ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   ViewTreeNode                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( selectedObject instanceof ViewTreeNode ) {
               ViewTreeNode viewTreeNode = (ViewTreeNode) selectedObject ;

               if ( viewTreeNode.toSQL() != null ) {

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Symptoms                                                */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( ( view.equals ( OAViews.VIEW_SYMPTOM_TITLE ) ) ||
                       ( view.equals ( OAViews.VIEW_SYMPTOM_STATUS ) ) ||
                       ( view.equals ( OAViews.VIEW_CPR_SYMPTOM ) ) ||
                       ( view.equals ( OAViews.VIEW_ORPHANED_SYMPTOM ) ) ||
                       ( view.equals ( OAViews.VIEW_ARCHIVED_SYMPTOM ) ) ||
                       ( view.equals ( OAViews.VIEW_SYMPTOM_TYPE ) ) )
                     populateTree ( model, viewTreeNode, OASQL.readSymptomList ( viewTreeNode.toSQL() ) ) ;

                  else if ( ( view.equals ( OAViews.VIEW_DRAFT_SYMPTOM_TITLE ) ) ||
                            ( view.equals ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_SYMPTOM ) ) ||
                            ( view.equals ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_SYMPTOM ) ) ||
                            ( view.equals ( OAViews.VIEW_DRAFT_SYMPTOM_STATUS ) ) ||
                            ( view.equals ( OAViews.VIEW_DRAFT_SYMPTOM_TYPE ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readDraftSymptomList ( viewTreeNode.toSQL() ) ) ;


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Questions                                               */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  else if ( ( view.equals ( OAViews.VIEW_QUESTION_TITLE ) ) ||
                            ( view.equals ( OAViews.VIEW_CPR_QUESTION ) ) ||
                            ( view.equals ( OAViews.VIEW_ORPHANED_QUESTION ) ) ||
                            ( view.equals ( OAViews.VIEW_ARCHIVED_QUESTION ) ) ||
                            ( view.equals ( OAViews.VIEW_QUESTION_TYPE ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readQuestionList ( viewTreeNode.toSQL() ) ) ;

                  else if ( ( view.equals ( OAViews.VIEW_DRAFT_QUESTION_TITLE ) ) ||
                            ( view.equals ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_QUESTION ) ) ||
                            ( view.equals ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_QUESTION ) ) ||
                            ( view.equals ( OAViews.VIEW_DRAFT_QUESTION_TYPE ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readDraftQuestionList ( viewTreeNode.toSQL() ) ) ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Actions                                                 */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  else if ( ( view.equals ( OAViews.VIEW_ACTION_TITLE ) ) ||
                            ( view.equals ( OAViews.VIEW_ARCHIVED_ACTION ) ) ||
                            ( view.equals ( OAViews.VIEW_ORPHANED_ACTION ) ) ||
                            ( view.equals ( OAViews.VIEW_CPR_ACTION ) ) ||
                            ( view.equals ( OAViews.VIEW_ACTION_TYPE ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readActionList ( viewTreeNode.toSQL() ) ) ;

                  else if ( ( view.equals ( OAViews.VIEW_DRAFT_ACTION_TYPE ) ) ||
                            ( view.equals ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_ACTION ) ) ||
                            ( view.equals ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_ACTION ) ) ||
                            ( view.equals ( OAViews.VIEW_DRAFT_ACTION_TITLE ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readActionDraftList ( viewTreeNode.toSQL() ) ) ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Links                                                   */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  else if ( ( view.equals ( OAViews.VIEW_DOCLINK_STATUS ) ) ||
                            ( view.equals ( OAViews.VIEW_ARCHIVED_LINK ) ) ||
                            ( view.equals ( OAViews.VIEW_ORPHANED_LINK ) ) ||
                            ( view.equals ( OAViews.VIEW_CPR_LINK ) ) ||
                            ( view.equals ( OAViews.VIEW_DOCLINK_TYPE ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readLinkList ( viewTreeNode.toSQL () ) ) ;

                  else if ( ( view.equals ( OAViews.VIEW_DRAFT_DOCLINK_STATUS ) ) ||
                            ( view.equals ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_LINK ) ) ||
                            ( view.equals ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_LINK ) ) ||
                            ( view.equals ( OAViews.VIEW_DRAFT_DOCLINK_TYPE ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readDraftLinkList ( viewTreeNode.toSQL () ) ) ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Link Groups                                             */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  else if ( ( view.equals ( OAViews.VIEW_DOCLINK_GROUP_TITLE ) ) ||
                            ( view.equals ( OAViews.VIEW_ARCHIVED_LINKGROUP ) ) ||
                            ( view.equals ( OAViews.VIEW_ORPHANED_LINKGROUP ) ) || 
                            ( view.equals ( OAViews.VIEW_CPR_LINKGROUP ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readLinkGroupList ( viewTreeNode.toSQL() ) ) ;

                  else if ( ( view.equals ( OAViews.VIEW_DRAFT_DOCLINK_GROUP_TITLE ) ) ||
                            ( view.equals ( OAViews.VIEW_CLOSED_DRAFT_ACTIVITY_LINKGROUP ) ) ||
                            ( view.equals ( OAViews.VIEW_NEW_DRAFT_ACTIVITY_LINKGROUP ) ) )
                     populateTree ( model,  viewTreeNode, OASQL.readDraftLinkGroupList ( viewTreeNode.toSQL() ) ) ;

               }
            }

         }

      } catch ( Exception ex ) {
         LogSystem.log ( 1, ex );
      }
   }





   /*------------------------------------------------------------*/
   /* populateTree
    */
   /**
    *    this method will populate the node given
    *  with the data vector given. All children nodes
    *  will be deleted, and new ones inserted
    *
    *  @param node The node to work with
    *  @param dataVec A Vector of objects to insert
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void populateTree ( DefaultTreeModel model, DefaultMutableTreeNode node, Vector dataVec ) {
      int   childCount  = node.getChildCount() ;             
      int   sort        = PropertySystem.getInt ( PROPERTY_OANAV_SORT ) ;

      for ( int delete = 0; delete < childCount; delete++ )
         ((DefaultMutableTreeNode)node.getChildAt ( 0 )).removeFromParent() ;

      if ( ( dataVec != null ) && ( dataVec.size() > 0 ) ) {
         int                           i           = 0 ;
         int                           size        = dataVec.size() ;

         defaultWin.setStatus ( Str.getStr ( STR_FOUND )+" "+dataVec.size() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, sort the vector                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( ( continueProcess ) && ( i < size ) ) {
            if ( dataVec.elementAt(i) != null )
               ((DisplayRec)dataVec.elementAt(i)).setSort ( sort ) ;
            i++ ;
         }

         if ( continueProcess ) {
            new QuickSort ( dataVec ) ;

            addChildrenToNode ( dataVec, (MutableTreeWrapper)node ) ;
         } else
            model = null ;
      }

      if ( ( model != null ) && ( node != null ) )
         model.reload ( node ) ;

      defaultWin.setStatus ( null ) ;
   }



   /*------------------------------------------------------------*/
   /* addChildrenToNode
    */
   /**
    *    This method will add a Vector of DisplayRec
    *  objects to the Node passed in
    *  
    *
    *  @param vector The Vector of object
    *  @param node The DefaultMutableTreeNode to add the objects to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addChildrenToNode ( Vector childrenVec, DefaultMutableTreeNode node ) {
      int                size    = childrenVec.size() ;
      int                k       = 0 ;
      MutableTreeWrapper newNode = null ;

      defaultWin.setValueMax ( size ) ;
      while ( ( k < size ) && ( continueProcess ) ) {

         if ( childrenVec.elementAt(k) != null ) {
            node.add ( newNode = new MutableTreeWrapper ( childrenVec.elementAt(k) ) ) ;

            if ( ((DisplayRec)childrenVec.elementAt(k)).containsChildren() )
               addChildrenToNode ( ((DisplayRec)childrenVec.elementAt(k)).getChildren(), 
                                   (DefaultMutableTreeNode) newNode ) ;
         }
         k++ ;
         defaultWin.setValue ( k ) ;
      }

      defaultWin.setValue ( 0 ) ;
   }




   /*------------------------------------------------------------*/
   /* getProcessName
    */
   /**
    *    this method will return our process name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getProcessName () {
      return( "RefreshTreeViewThread" );
   }




   /*------------------------------------------------------------*/
   /* stopProcess
    */
   /**
    *    This method will attempt to stop the process
    *  that is currently running
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopProcess () {
      continueProcess = false ;
      Qit.setEnabled ( true ) ;

      if ( paths != null )
         oaNavPanel.closeTree ( paths ) ;

      ProcessManagerSystem.removeProcess ( this ) ;

   }

} /* RefreshTreeViewThread */
