package com.ibm.nzna.projects.qit.avalon.oanav.searchreplace ;                       
/*-----------------------------------------------------------*/
/*   SearchReplaceProcess                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SearchReplaceProcess                */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This process thread actually does the work for a Search and Replace.
 * It performs the following steps on each object in the SearchReplaceRec's
 * objectChangeVec:<br><pre>
 *
 * 1. Read in the draft from the database
 * 2. Perform the change in memory
 * 3. Save the draft back to the database
 *
 *
 * @version 1.0, 6/7/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SearchReplaceProcess                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class SearchReplaceProcess extends Object implements Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private SearchReplaceRec searchReplaceRec = null ;
   private Frame            parentWin        = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the process but will
    *  not start the thread that does the work. To start
    *  the thread, please call 'start'
    *
    *  @param searchReplaceRec The SearchReplaceRec to use to change the data
    *  @param parent The Parent window of the status window that the processor uses
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SearchReplaceProcess ( SearchReplaceRec searchReplaceRec, Frame parentWin ) {
      this.searchReplaceRec = searchReplaceRec ;
      this.parentWin        = parentWin ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will start the thread and immediatly
    *  return
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      new Thread ( this ).start() ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will do all of the actual processing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      StatusWin statusWin       = new StatusWin ( parentWin ) ;
      Vector    objectChangeVec = searchReplaceRec.getObjectChangeVec () ;
      int       i               = 0 ;
      int       size            = objectChangeVec.size() ;
      boolean   locked          = false ;
      boolean   loaded          = false ;
      String    lockedByUser    = null ;
      Object    object          = null ;

      statusWin.setMaxValue ( size-1 ) ;

      while ( i < size ) {
         statusWin.setText ( Str.getStr ( STR_UPDATING )+" "+( ((Integer)objectChangeVec.elementAt(i)).intValue() ) ) ;

         switch ( searchReplaceRec.getObjectType() ) {
            case SearchReplaceRec.TYPE_ACTION :
               ActionDraft action = AvalonCache.readActionDraft ( ((Integer)objectChangeVec.elementAt(i)).intValue() ) ;
               if ( action != null ) {
                  locked = ( ( action.getMaintLock() ) &&
                             ( UserSystem.getUserId().equals ( action.getDbUser() ) == false ) ) ;
                  if ( locked )
                     lockedByUser = UserSystem.getNameFromUserId ( action.getDbUser() ) ;

                  object = action ;
                  loaded = true ;
               } else {
                  locked = false ;
                  loaded = false ;
               }
               break ;

            case SearchReplaceRec.TYPE_QUESTION :
               QuestionDraft question = AvalonCache.readQuestionDraft ( ((Integer)objectChangeVec.elementAt(i)).intValue() ) ;
               if ( question != null ) {
                  locked = ( ( question.getMaintLock() ) &&
                             ( UserSystem.getUserId().equals ( question.getDbUser() ) == false ) ) ;
                  if ( locked )
                     lockedByUser = UserSystem.getNameFromUserId ( question.getDbUser() ) ;

                  object = question ;
                  loaded = true ;
               } else {
                  locked = false ;
                  loaded = false ;
               }
               break ;

            case SearchReplaceRec.TYPE_SYMPTOM :
               SymptomDraft symptom = AvalonCache.readSymptomDraft ( ((Integer)objectChangeVec.elementAt(i)).intValue() ) ;
               if ( symptom != null ) {
                  locked = ( ( symptom.getMaintLock() ) &&
                             ( UserSystem.getUserId().equals ( symptom.getDbUser() ) == false ) ) ;
                  if ( locked )
                     lockedByUser = UserSystem.getNameFromUserId ( symptom.getDbUser() ) ;

                  object = symptom ;
                  loaded = true ;
               } else {
                  locked = false ;
                  loaded = false ;
               }
               break ;

            case SearchReplaceRec.TYPE_QUEST_LINK :
               QuestLinkDraft questLink = (QuestLinkDraft)AvalonCache.readLinkDraft ( ((Integer)objectChangeVec.elementAt(i) ).intValue(), 0 ) ;

               if ( questLink != null ) {
                  locked = ( ( questLink.getMaintLock() ) &&
                             ( UserSystem.getUserId().equals ( questLink.getDbUser() ) == false ) ) ;
                  if ( locked )
                     lockedByUser = UserSystem.getNameFromUserId ( questLink.getDbUser() ) ;

                  object = questLink ;
                  loaded = true ;
               } else {
                  locked = false ;
                  loaded = false ;
               }
               break ;


            case SearchReplaceRec.TYPE_DOC_LINK :
               ExternalLinkDraft docLink = (ExternalLinkDraft)AvalonCache.readLinkDraft ( ((Integer)objectChangeVec.elementAt(i) ).intValue(), 0 ) ;

               if ( docLink != null ) {
                  locked = ( ( docLink.getMaintLock() ) &&
                             ( UserSystem.getUserId().equals ( docLink.getDbUser() ) == false ) ) ;
                  if ( locked )
                     lockedByUser = UserSystem.getNameFromUserId ( docLink.getDbUser() ) ;

                  object = docLink ;
                  loaded = true ;
               } else {
                  locked = false ;
                  loaded = false ;
               }
               break ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is locked, tell the user                        */                       
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( locked )
            GUISystem.printBox ( Str.getStr ( STR_INFO ), 
                                 Str.getStr ( STR_DRAFT_LOCKED_BY )+" "+lockedByUser ) ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, if it loaded, process it                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( loaded ) {
            processObject ( object ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, tell the user                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else
            GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_COULD_NOT_CREATE_DRAFT ) ) ;

         statusWin.setValue ( i ) ;
         i++ ;
      }

      statusWin.dispose() ;
      GUISystem.printBox ( parentWin, STR_INFO, STR_COMPLETE ) ;

   }





   /*------------------------------------------------------------*/
   /* processObject
    */
   /**
    *    This method will change the data on a certain
    *  object. It will also call the 'writeToDatabase'
    *  method on the object
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void processObject ( Object object ) {

      switch ( searchReplaceRec.getField() ) {
         
         case SearchReplaceRec.FIELD_TITLE :
            changeTitleField ( object ) ;
            break ;

         case SearchReplaceRec.FIELD_ADD_INFO :
            changeAddInfoField ( object ) ;
            break ;

         case SearchReplaceRec.FIELD_KEYWORDS :
            changeKeywordsField ( object ) ;
            break ;

         case SearchReplaceRec.FIELD_ANSWER :
            changeAnswerField ( object ) ;
            break ;
      }
   }





   /*------------------------------------------------------------*/
   /* changeTitleField
    */
   /**
    *    This method will search and replace through
    *  the titles of the object passed
    *  
    *
    *  @param object The object to search and replace with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void changeTitleField ( Object object ) {
      Vector titleVec = null ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, grab the title vector                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( object instanceof QuestionDraft )
         titleVec = ((QuestionDraft)object).getTitles() ;

      else if ( object instanceof ActionDraft )
         titleVec = ((ActionDraft)object).getTitles() ;

      else if ( object instanceof SymptomDraft )
         titleVec = ((SymptomDraft)object).getSymptomTitles() ;

      else if ( object instanceof QuestLinkDraft ) 
         titleVec = ((QuestLinkDraft)object).getLinkTitles() ;
      
      else if ( object instanceof ExternalLinkDraft )
         titleVec = ((ExternalLinkDraft)object).getLinkTitles() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have a title vector, go through and change        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( titleVec != null ) {
         int i    = 0 ;
         int size = titleVec.size() ;

         /* Only allow BPMs to change ALL  */
         if ( !UserSystem.getLoggedUserRec().isBPM() ) {
            
            /* Do not take the default off if this is a link draft  */
            if ( !(object instanceof LinkDraft ) )
               size-- ;
         }

         System.out.println ( "SIZE:"+size ) ;
         while ( i < size ) {
            if ( titleVec.elementAt(i) instanceof Title ) {
               Title title = (Title) titleVec.elementAt(i) ;

               if ( title.getTitle().indexOf ( searchReplaceRec.getFromString() ) >= 0 ) {
                  System.out.println ( "Setting Title!" ) ;
                  title.setTitle ( Text.replaceAllStrInStr ( title.getTitle(), 
                                                             searchReplaceRec.getFromString(),
                                                             searchReplaceRec.getToString() ) ) ;
               }
            }

            else if ( titleVec.elementAt(i) instanceof SymptomTitle ) {
               SymptomTitle title = (SymptomTitle) titleVec.elementAt(i) ;

               if ( title.getTitle().indexOf ( searchReplaceRec.getFromString() ) >= 0 ) {
                  System.out.println ( "Setting Symptom Title!" ) ;
                  title.setTitle ( Text.replaceAllStrInStr ( title.getTitle(), 
                                                             searchReplaceRec.getFromString(),
                                                             searchReplaceRec.getToString() ) ) ;
               }
            }

            else if ( titleVec.elementAt(i) instanceof LanguageText ) {
               LanguageText languageText = (LanguageText) titleVec.elementAt(i) ;

               System.out.println ( "Language Text!" ) ;
               if ( languageText.getText().indexOf ( searchReplaceRec.getFromString() ) >= 0 ) {
                  System.out.println ( "Setting LanguageText!" ) ;
                  languageText.setText ( Text.replaceAllStrInStr ( languageText.getText(), 
                                                                   searchReplaceRec.getFromString(),
                                                                   searchReplaceRec.getToString() ) ) ;
               }
            }
            i++ ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, save the record                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( object instanceof QuestionDraft )
         ((QuestionDraft)object).writeToDatabase() ;

      else if ( object instanceof ActionDraft )
         ((ActionDraft)object).writeToDatabase() ;

      else if ( object instanceof SymptomDraft )
         ((SymptomDraft)object).writeToDatabase() ;


   }



   /*------------------------------------------------------------*/
   /* changeAddInfoField
    */
   /**
    *    This method will search and replace through
    *  the add info of the object passed
    *  
    *
    *  @param object The object to search and replace with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void changeAddInfoField ( Object object ) {
      Vector addInfoVec = null ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, grab the addInfo vector                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( object instanceof QuestionDraft )
         addInfoVec = ((QuestionDraft)object).getAdditionalInfo() ;

      else if ( object instanceof ActionDraft )
         addInfoVec = ((ActionDraft)object).getAdditionalInfo() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have a addInfo vector, go through and change      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( addInfoVec != null ) {
         int i    = 0 ;
         int size = addInfoVec.size() ;

         /* Only allow BPMs to change ALL  */
         if ( !UserSystem.getLoggedUserRec().isBPM() )
            size-- ;

         while ( i < size ) {
            if ( addInfoVec.elementAt(i) instanceof AddtlInfo ) {
               AddtlInfo addInfo = (AddtlInfo) addInfoVec.elementAt(i) ;

               if ( addInfo.getAddtlInfo().indexOf ( searchReplaceRec.getFromString() ) >= 0 ) {
                  addInfo.setAddtlInfo ( Text.replaceAllStrInStr ( addInfo.getAddtlInfo(), 
                                                                   searchReplaceRec.getFromString(),
                                                                   searchReplaceRec.getToString() ) ) ;
               }
            }
            i++ ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, save the record                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( object instanceof QuestionDraft )
         ((QuestionDraft)object).writeToDatabase() ;

      else if ( object instanceof ActionDraft )
         ((ActionDraft)object).writeToDatabase() ;

      else if ( object instanceof SymptomDraft )
         ((SymptomDraft)object).writeToDatabase() ;


   }





   /*------------------------------------------------------------*/
   /* changeKeywordField
    */
   /**
    *    This method will change the keyword field
    *  of the object passed
    *  
    *
    *  @param object The object to change it in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void changeKeywordsField ( Object object ) {
      Vector titleVec = null ;

      if ( object instanceof SymptomDraft )
         titleVec = ((SymptomDraft)object).getSymptomTitles() ;


      if ( titleVec != null ) {
         int          i     = 0 ;
         int          size  = titleVec.size() ;
         SymptomTitle title = null ;

         /* Only allow BPMs to change ALL  */
         if ( !UserSystem.getLoggedUserRec().isBPM() )
            size-- ;

         while ( i < size ) {
            title = (SymptomTitle) titleVec.elementAt(i) ;

            if ( title.getKeywords().indexOf ( searchReplaceRec.getFromString() ) >= 0 ) {
               title.setKeywords ( Text.replaceAllStrInStr ( title.getKeywords(), 
                                                             searchReplaceRec.getFromString(),
                                                             searchReplaceRec.getToString() ) ) ;

               i++ ;
            }
         }
      }

      if ( object instanceof SymptomDraft )
         ((SymptomDraft)object).writeToDatabase() ;
   }


   /*------------------------------------------------------------*/
   /* changeAnswerField
    */
   /**
    *    This method will change the answer field
    *  of the object passed
    *  
    *
    *  @param object The object to change it in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void changeAnswerField ( Object object ) {
      Vector answerVec = null ;

      if ( object instanceof QuestionDraft )
         answerVec = ((QuestionDraft)object).getAnswers() ;


      if ( answerVec != null ) {
         int      i        = 0 ;
         int      size     = answerVec.size() ;
         Answer   answer   = null ;
         Vector   titleVec = null ;

         /* Only allow BPMs to change ALL  */
         if ( !UserSystem.getLoggedUserRec().isBPM() )
            size-- ;

         while ( i < size ) {
            answer = (Answer) answerVec.elementAt(i) ;

            titleVec = answer.getTitles() ;

            if ( titleVec != null ) {
               int   g       = 0 ;
               int   tSize   = titleVec.size() ;
               Title title   = null ;

               /* Only allow BPMs to change ALL  */
               if ( !UserSystem.getLoggedUserRec().isBPM() )
                  tSize-- ;

               while ( g < tSize ) {
                  title = (Title)titleVec.elementAt(g) ;

                  if ( title.getTitle().indexOf ( searchReplaceRec.getFromString() ) >= 0 ) {
                     title.setTitle ( Text.replaceAllStrInStr ( title.getTitle(), 
                                                                searchReplaceRec.getFromString(),
                                                                searchReplaceRec.getToString() ) ) ;
                  }

                  g++ ;
               } /* while */
            } /* if  */

            i++ ;
         }
      }

      if ( object instanceof QuestionDraft )
         ((QuestionDraft)object).writeToDatabase() ;
   }

} /* SearchReplaceProcess */

