package com.ibm.nzna.projects.qit.avalon.oanav.searchreplace ;           
/*-----------------------------------------------------------*/
/*   SearchReplaceRec                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SearchReplaceRec                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.sort.* ;
/**
 * This record holds information that is needed when doing a massive search/replace 
 *
 *
 * @version 1.0, 6/7/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SearchReplaceRec                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class SearchReplaceRec extends Object implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int FIELD_UNKNOWN       = 0 ;
   public static final int FIELD_ANSWER        = 1 ;
   public static final int FIELD_TITLE         = 2 ;
   public static final int FIELD_KEYWORDS      = 3 ;
   public static final int FIELD_ADD_INFO      = 4 ;      

   public static final int TYPE_QUESTION        = 1 ;
   public static final int TYPE_ACTION          = 2 ;
   public static final int TYPE_SYMPTOM         = 3 ;
   public static final int TYPE_QUEST_LINK      = 4 ;
   public static final int TYPE_DOC_LINK        = 5 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector objectChangeVec         = null ;
   private int    fieldId                 = FIELD_UNKNOWN ;
   private String fromString              = null ;
   private String toString                = null ;
   private int    type                    = 0 ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method creates a blank record
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SearchReplaceRec () {
   }




   /*------------------------------------------------------------*/
   /* setObjectChangeVec
    */
   /**
    *    This method will set the vector of Draft objects
    *  that are going to be changed
    *  
    *
    *  @param objectChangeVec The vector of objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setObjectChangeVec ( Vector objectChangeVec ) {
      this.objectChangeVec = objectChangeVec ;
   }





   /*------------------------------------------------------------*/
   /* getObjectChangeVec
    */
   /**
    *    This method will get the vector of objects
    *  to change
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return objectChangeVec (Vector)
    *//*---------------------------------------------------------*/
   public Vector getObjectChangeVec () {
      return( objectChangeVec ) ;
   }


   /*------------------------------------------------------------*/
   /* setFromString
    */
   /**
    *    This method will set the string that will
    *  be searched for
    *  
    *
    *  @param fromString The string to search for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFromString ( String fromString ) {
      this.fromString = fromString ;
   }




   /*------------------------------------------------------------*/
   /* getFromString
    */
   /**
    *    This method will return the from string to search
    *  for
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getFromString () {
      return( fromString ) ;
   }



   /*------------------------------------------------------------*/
   /* setToString
    */
   /**
    *    This method will set the string that will
    *  be searched for
    *  
    *
    *  @param toString The string to search for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setToString ( String toString ) {
      this.toString = toString ;
   }




   /*------------------------------------------------------------*/
   /* getToString
    */
   /**
    *    This method will return the to string to search
    *  for
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getToString () {
      return( toString ) ;
   }





   /*------------------------------------------------------------*/
   /* setField
    */
   /**
    *    This method will set the field to whatever
    *  string is passed. An InvalidArgumentException
    *  will be thrown if the string is not known
    *
    *  @param fieldDescript The description of the field to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setField ( String fieldDescript ) {
      if ( fieldDescript.equals ( Str.getStr ( STR_ANSWER ) ) )
         fieldId = FIELD_ANSWER ;
      else if ( fieldDescript.equals ( Str.getStr ( STR_ADDITIONAL_INFORMATION ) ) )
         fieldId = FIELD_ADD_INFO ;
      else if ( fieldDescript.equals ( Str.getStr ( STR_TITLE ) ) )
         fieldId = FIELD_TITLE ;
      else if ( fieldDescript.equals ( Str.getStr ( STR_KEYWORDS ) ) )
         fieldId = FIELD_KEYWORDS ;
      else
         new IllegalArgumentException ( "Non-supported field description passed:"+fieldDescript ) ;
   }





   /*------------------------------------------------------------*/
   /* setObjectType
    */
   /**
    *    This method will set what type of object
    *  will be changing.
    *  
    *
    *  @param type The type as defined in this class ( TYPE_ACTION, TYPE_QUESTION or TYPE_SYMPTOM )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setObjectType ( int type ) {
      if ( ( type == TYPE_ACTION ) ||
           ( type == TYPE_QUESTION ) ||
           ( type == TYPE_QUEST_LINK ) ||
           ( type == TYPE_SYMPTOM ) )
         this.type = type ;
      else
         throw new IllegalArgumentException ( "Cannot change object type of:"+type ) ;
   }




   /*------------------------------------------------------------*/
   /* getFieldDescripts
    */
   /**
    *    This method will return all of the fields that
    *  may be searched/replaced with the objectChangeVec
    *  that has been set. If the ObjectChangeVec is not set, a NULL is returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of Strings
    *//*---------------------------------------------------------*/
   public Vector getFieldDescripts( ) {
      Vector returnVec = null ;

      if ( objectChangeVec != null ) {

         returnVec = new Vector ( 1, 1 ) ;

         returnVec.addElement ( Str.getStr ( STR_TITLE ) ) ;

         if ( ( type == TYPE_QUESTION ) ||
              ( type == TYPE_ACTION ) )
            returnVec.addElement ( Str.getStr ( STR_ADDITIONAL_INFORMATION ) ) ;

         if ( type == TYPE_QUESTION )
            returnVec.addElement ( Str.getStr ( STR_ANSWER ) ) ;

         if ( type == TYPE_SYMPTOM )
            returnVec.addElement ( Str.getStr ( STR_KEYWORDS ) ) ;

         new QuickSort ( objectChangeVec ) ;
      }

      return( returnVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getObjectType
    */
   /**
    *    This method will return the object type that
    *  needs to be changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getObjectType () {
      return( type ) ;
   }





   /*------------------------------------------------------------*/
   /* getField
    */
   /**
    *    This method will return the field to edit
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getField () {
      return( fieldId ) ;
   }
} /* SearchReplaceRec */
