package com.ibm.nzna.projects.qit.avalon.oanav.searchreplace ;                  
/*-----------------------------------------------------------*/
/*   SearchReplaceWizard                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SearchReplaceWizard                 */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.avalon.cache.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;        
import com.ibm.nzna.shared.util.* ;        
import com.ibm.nzna.shared.gui.wizard.* ;
/**
 * This wizard is designed to allow the step through creation of 
 * the following types of Avalon Objects:<br><pre>
 *
 * Question, Action, Symptom<br></pre>
 * 
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SearchReplaceWizard                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class SearchReplaceWizard extends JDialog implements WizardListener, AvalonConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int              objectType        = 0 ;
   private Wizard           wizardPanel       = null ;
   private WizardStep[]     steps             = null ;
   private boolean          cancelled         = true ;
   private SearchReplaceRec searchReplaceRec  = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the Wizard.
    *  It will not return until the wizard is complete
    *  
    *
    *  @param parent The parent window of the wizard
    *  @param objectType The type of object ( AVALON_QUESTION, AVALON_SYMPTOM or AVALON_ACTION )
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SearchReplaceWizard ( AppDefaultWin parent, SearchReplaceRec searchReplaceRec ) {
      super ( parent, Str.getStr ( STR_NEW ), true ) ;
      CreateWizardWin statusWin = new CreateWizardWin ( parent, 7 ) ;

      this.searchReplaceRec = searchReplaceRec ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our Wizard control                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            wizardPanel = new Wizard ( ImageSystem.getImageIcon ( this, ImageSystem.AVALON_WIZARD ) ) ;                     
            wizardPanel.addWizardListener ( this ) ;
         } catch ( Exception e ) {
            wizardPanel = new Wizard () ;
         }

         statusWin.setValue ( 1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our Wizard                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         getContentPane().setLayout ( new BorderLayout() ) ;
         getContentPane().add ( wizardPanel, BorderLayout.CENTER ) ;                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the steps in the wizard                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         statusWin.setValue ( 2 ) ;
         createWizardSteps ( statusWin ) ;           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Start our wizard off                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         wizardPanel.setCurrentStep ( steps[0] ) ;

         setSize ( 495, 375 ) ;
         WinUtil.centerWindow ( this ) ;
         statusWin.dispose() ;
         setVisible ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* createWizardSteps
    */
   /**
    *    This method will create the necessary
    *  steps to create an object type
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createWizardSteps ( CreateWizardWin statusWin ) {

      steps = new WizardStep[ 4 ] ;

      steps[0] = new InfoStep ( Str.getStr ( STR_SEARCH_REPLACE_WIZARD_WELCOME ) ) ;
      statusWin.setValue ( 3 ) ;

      steps[1] = new SearchReplaceFieldStep ( searchReplaceRec ) ;
      statusWin.setValue ( 4 ) ;

      steps[2] = new SearchReplaceTextStep ( searchReplaceRec ) ;
      statusWin.setValue ( 5 ) ;

      steps[3] = new InfoStep ( Str.getStr ( STR_SEARCH_REPLACE_WIZARD_FINISH ) ) ;
      statusWin.setValue ( 6 ) ;
   }


   /*------------------------------------------------------------*/
   /* cancelPerformed
    */
   /**
    *    This method will be called if the user
    *  hit the 'Cancel' button on the Wizard
    *  
    *
    *  @param e The wizard event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cancelPerformed ( WizardEvent e ) {
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* finishPerformed
    */
   /**
    *    This method will be called when the
    *  user presses the finish button
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finishPerformed ( WizardEvent e) {
      new SearchReplaceProcess ( searchReplaceRec, (Frame)getParent() ).start() ;
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* nextPerformed
    */
   /**
    *    This method will be called when the
    *  next button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void nextPerformed ( WizardEvent e ) {
      WizardStep step  = wizardPanel.getCurrentStep() ;
      int        index = indexFromStep ( step ) ;                

      index++ ;
      if ( index >= steps.length )
         index = steps.length-1 ;

      wizardPanel.setCurrentStep ( steps [ index ] ) ;
      adjustButtons ( ) ;
   }


   /*------------------------------------------------------------*/
   /* backPerformed
    */
   /**
    *    This method will be called when the
    *  back button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void backPerformed ( WizardEvent e ) {
      WizardStep step  = wizardPanel.getCurrentStep() ;
      int        index = indexFromStep ( step ) ;

      index-- ;
      if ( index < 0 )
         index = 0 ;

      wizardPanel.setCurrentStep ( steps [ index ] ) ;
      adjustButtons ( ) ;
   }





   /*------------------------------------------------------------*/
   /* adjustButtons
    */
   /**
    *    This method will look at what step is
    *  currently showing and will decide which
    *  buttons to show in the wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void adjustButtons () {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == steps[0] )
         wizardPanel.showButtons ( true, false, true, false ) ;
      else if ( step == steps[steps.length-1] )
         wizardPanel.showButtons ( false, true, false, true ) ;
      else
         wizardPanel.showButtons ( true, true, true, false ) ;
   }





   /*------------------------------------------------------------*/
   /* indexFromStep
    */
   /**
    *    This method will return the index of the WizardStep
    *  passed in
    *  
    *
    *  @param wizardStep
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   private int indexFromStep ( WizardStep wizardStep ) {
      int rc = 0 ;
      int i  = 0 ;

      while ( ( rc == 0 ) && ( i < steps.length ) ) {
         if ( steps[i] == wizardStep )
            rc = i ;
         i++ ;
      }

      return( rc ) ;
   }

} /* SearchReplaceWizard */
