package com.ibm.nzna.projects.qit.avalon.oanav.views ;
/*-----------------------------------------------------------*/
/*   OAViews                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OAViews                             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
import javax.swing.tree.* ;
import javax.swing.* ;
/**
 * This class holds static methods usefull in manipulating Views in the Navigator.
 * Since there are so many views, the views can be looked at this way. A view has a
 * main view type which is Active, Draft, Orphan or Archived. A View has a sub view type
 * which is Action, Question or Symptom.
 *
 * @version 1.0, 4/10/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OAViews                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class OAViews extends Object implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Integer Views                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int SYMPTOM                    = 1 ;
   public static final int QUESTION                   = 2 ;
   public static final int ACTION                     = 3 ;
   public static final int ACTION_DRAFT               = 4 ;
   public static final int LINK                       = 5 ;
   public static final int LINKGROUP                  = 6 ;
   public static final int NONE                       = 7 ;
   public static final int QUESTION_ACTIVITY_OPEN     = 8 ;
   public static final int ACTION_ACTIVITY_OPEN       = 9 ;
   public static final int SYMPTOM_ACTIVITY_OPEN      = 10 ;
   public static final int LINK_ACTIVITY_OPEN         = 12 ;
   public static final int LINKGROUP_ACTIVITY_OPEN    = 13 ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Views                                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String VIEW_SYMPTOM_TYPE                    = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewSymptomType" ;
   public static final String VIEW_SYMPTOM_STATUS                  = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewSymptomStatus" ;
   public static final String VIEW_SYMPTOM_TITLE                   = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewSymptomTitle" ;

   public static final String VIEW_QUESTION_TYPE                   = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewQuestionType" ;
   public static final String VIEW_QUESTION_TITLE                  = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewQuestionTitle" ;

   public static final String VIEW_ACTION_TYPE                     = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewActionType" ;
   public static final String VIEW_ACTION_TITLE                    = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewActionTitle" ;

   public static final String VIEW_DOCLINK_TYPE                    = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewLinkType" ;
   public static final String VIEW_DOCLINK_STATUS                  = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewLinkStatus" ;

   public static final String VIEW_DOCLINK_GROUP_TITLE             = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewLinkGroup" ;

   public static final String VIEW_DRAFT_SYMPTOM_TYPE              = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftSymptomType" ;
   public static final String VIEW_DRAFT_SYMPTOM_STATUS            = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftSymptomStatus" ;
   public static final String VIEW_DRAFT_SYMPTOM_TITLE             = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftSymptomTitle" ;

   public static final String VIEW_DRAFT_QUESTION_TYPE             = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftQuestionType" ;
   public static final String VIEW_DRAFT_QUESTION_TITLE            = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftQuestionTitle" ;

   public static final String VIEW_DRAFT_ACTION_TYPE               = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftActionType" ;
   public static final String VIEW_DRAFT_ACTION_TITLE              = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftActionTitle" ;

   public static final String VIEW_DRAFT_DOCLINK_TYPE              = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftLinkType" ;
   public static final String VIEW_DRAFT_DOCLINK_STATUS            = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftLinkStatus" ;

   public static final String VIEW_DRAFT_DOCLINK_GROUP_TITLE       = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewDraftLinkGroup" ;

   public static final String VIEW_NEW_DRAFT_ACTIVITY_QUESTION     = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewNewDraftActivityQuestion" ;
   public static final String VIEW_NEW_DRAFT_ACTIVITY_ACTION       = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewNewDraftActivityAction" ;
   public static final String VIEW_NEW_DRAFT_ACTIVITY_SYMPTOM      = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewNewDraftActivitySymptom" ;
   public static final String VIEW_NEW_DRAFT_ACTIVITY_LINK         = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewNewDraftActivityLink" ;
   public static final String VIEW_NEW_DRAFT_ACTIVITY_LINKGROUP    = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewNewDraftActivityLinkGroup" ;

   public static final String VIEW_CLOSED_DRAFT_ACTIVITY_QUESTION  = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewClosedDraftActivityQuestion" ;
   public static final String VIEW_CLOSED_DRAFT_ACTIVITY_ACTION    = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewClosedDraftActivityAction" ;
   public static final String VIEW_CLOSED_DRAFT_ACTIVITY_SYMPTOM   = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewClosedDraftActivitySymptom" ;
   public static final String VIEW_CLOSED_DRAFT_ACTIVITY_LINK      = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewClosedDraftActivityLink" ;
   public static final String VIEW_CLOSED_DRAFT_ACTIVITY_LINKGROUP = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewClosedDraftActivityLinkGroup" ;

   public static final String VIEW_ARCHIVED_ACTION                 = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewArchivedAction" ;
   public static final String VIEW_ARCHIVED_LINK                   = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewArchivedLink" ;
   public static final String VIEW_ARCHIVED_LINKGROUP              = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewArchivedLinkGroup" ;
   public static final String VIEW_ARCHIVED_QUESTION               = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewArchivedQuestion" ;
   public static final String VIEW_ARCHIVED_SYMPTOM                = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewArchivedSymptom" ;

   public static final String VIEW_ORPHANED_ACTION                 = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewOrphanedAction" ;
   public static final String VIEW_ORPHANED_LINK                   = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewOrphanedLink" ;
   public static final String VIEW_ORPHANED_LINKGROUP              = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewOrphanedLinkGroup" ;
   public static final String VIEW_ORPHANED_QUESTION               = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewOrphanedQuestion" ;
   public static final String VIEW_ORPHANED_SYMPTOM                = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewOrphanedSymptom" ;

   public static final String VIEW_CPR_ACTION                      = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewCPRAction" ;
   public static final String VIEW_CPR_LINK                        = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewCPRLink" ;
   public static final String VIEW_CPR_LINKGROUP                   = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewCPRLinkGroup" ;
   public static final String VIEW_CPR_QUESTION                    = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewCPRQuestion" ;
   public static final String VIEW_CPR_SYMPTOM                     = "com.ibm.nzna.projects.qit.avalon.oanav.views.ViewCPRSymptom" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static DefaultTreeModel model        = null ;
   private static StringTreeNode   root         = null ;



   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the entire JTree
    *  Model that needs to be in a JTree to support
    *  the view passed in
    *
    *  @param view One of the VIEW_XXX strings defined in this class
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static DefaultTreeModel getJTreeModel ( String view, TypeBrandRec brandRec ) {       
      DefaultTreeModel model = null ;

      if ( view != null ) {
         try {
            OANavView navView = null ;                                

            LogSystem.log ( 1, "Creating OAView :"+view+":" ) ;
            try {
               navView = (OANavView) Class.forName ( view ).newInstance() ;
            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
            }

            navView.setBrand     ( brandRec ) ;
            model = navView.getJTreeModel() ;

         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }

      return( model ) ;
   }




   /*------------------------------------------------------------*/
   /* getViewName
    */
   /**
    *    This method will return the user readable
    *  description of a view.
    *  
    *
    *  @param view The view to check
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getViewName ( String view ) {
      String retStr = "" ;

      if ( view != null ) {

         view = Text.replaceAllStrInStr ( view, "__DRAFT__", "" ) ;

         try {
            OANavView navView = (OANavView) Class.forName ( view ).newInstance() ;

            retStr = navView.getName() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }

      return( retStr );
   }


   /*------------------------------------------------------------*/
   /* getViewShortName
    */
   /**
    *    This method will return the user readable
    *  description of a view.
    *  
    *
    *  @param view The view to check
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getViewShortName ( String view ) {
      String retStr = "" ;

      if ( view != null ) {

         view = Text.replaceAllStrInStr ( view, "__DRAFT__", "" ) ;

         try {
            OANavView navView = (OANavView) Class.forName ( view ).newInstance() ;

            retStr = navView.getShortName() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
         }
      }

      return( retStr );
   }




   /*------------------------------------------------------------*/
   /* getViewKey
    */
   /**
    *    This method will return the key to a view
    *  
    *  
    *
    *  @param view The name of the view
    *  @param None
    *  @param None
    *
    *  @return String The key
    *//*---------------------------------------------------------*/
   public static String getViewKey ( String view ) {
      String key = "" ;

      if ( view != null ) {
         try {
            OANavView navView = (OANavView) Class.forName ( view ).newInstance() ;

            if ( navView != null )
               key = navView.getKey() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
         }
      }

      return ( key ) ;
   }




} /* OAs */
