package com.ibm.nzna.projects.qit.avalon.oanav.views ;         
/*-----------------------------------------------------------*/
/*   ViewActionType                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewActionType                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This view will list Actions by their geography, and then by their type. Their 
 * type is 'Default', 'Service' etc.. defined in OA.TYPEACTION
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewActionType                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewActionType extends Object implements AvalonConst, OANavView, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  TypeBrandRec           brandRec = null ;
   private  DefaultMutableTreeNode root     = null ;




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_ACTIVE )+" "+Str.getStr ( STR_ACTION )+" "+Str.getStr ( STR_BY_TYPE ) ) ;
   }


   /*------------------------------------------------------------*/
   /* getKey
    */
   /**
    *    This method will return the key, or the names
    *  of the columns returned, from this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getKey () {
      return( Str.getStr ( STR_DOC_CLASS )+", "+
              Str.getStr ( STR_INDICATOR )+", "+
              Str.getStr ( STR_TITLE )+" "+
              Str.getStr ( STR_LAST_TOUCHED_ON )+", "+
              Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( Str.getStr ( STR_BY_TYPE ) ) ;
   }




   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the brand that this
    *  view will be based on
    *  
    *
    *  @param brandRec The BrandRec to base the view on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( TypeBrandRec brandRec ) {
      this.brandRec = brandRec ;
   }





   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the complete DefaultTreeModel
    *  needed to show this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DefaultTreeModel
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getJTreeModel () {
      return( createGeoView() ) ;
   }




   /*------------------------------------------------------------*/
   /* createGeoView
    */
   /**
    *    this method will return the view
    *  based on geography
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private DefaultTreeModel createGeoView () {
      DefaultTreeModel model  = null ;
      Vector           geoVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_GEOGRAPHY ) ;                                         


      model = new DefaultTreeModel ( root = new DefaultMutableTreeNode("") ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Since all views currently drill down by GEO first,      */
      /* we will create each default geo drill down, and then fill */
      /* in the rest.                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( geoVec != null ) && ( geoVec.size() > 0 ) ) {
         int             i       = 0 ;
         int             size    = geoVec.size() ;
         StringTreeNode  node    = null ;
         StringTreeNode  allNode = null ;

         while ( i < size ) {
            if ( ((TypeGeoRec)geoVec.elementAt(i)).getInd() != TypeList.GEO_WORLD_WIDE ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create the Geography Parent                             */                  
               /*                                                           */
               /*-----------------------------------------------------------*/
               node = new StringTreeNode ( ((TypeGeoRec)geoVec.elementAt(i)).toString(), 0 ) ;
               model.insertNodeInto ( node, root, i ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create the 'All' parent                                 */              
               /*                                                           */
               /*-----------------------------------------------------------*/
               allNode = new StringTreeNode ( Str.getStr ( STR_ALL ) ) ;
               model.insertNodeInto ( allNode, node, 0 ) ;
               createSubTypeDrillDown ( model, allNode, 0, (TypeGeoRec) geoVec.elementAt(i) ) ;
               
               createSubCountryDrillDown ( model, node, 1, (TypeGeoRec) geoVec.elementAt(i) ) ;
            }
            i++ ;
         }
      }



      return( model ) ;
   }




   /*------------------------------------------------------------*/
   /* getSubCountryDrillDown
    */
   /**
    *    This method will put the countries under
    *  each geography
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createSubCountryDrillDown ( DefaultTreeModel model, StringTreeNode parentNode, int startIndex, TypeGeoRec geoRec ) {
      Vector countryVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;
      int    i          = 0 ;

      if ( countryVec != null ) {
         int                  size       = countryVec.size() ;
         TypeCountryCodeRec   countryRec = null ;
         StringTreeNode       node       = null ;

         while ( i < size ) {
            countryRec = (TypeCountryCodeRec) countryVec.elementAt(i) ;
            if ( countryRec.getGeoInd() == geoRec.getInd() ) {
               
               node = new StringTreeNode ( countryRec.toString() ) ;
               model.insertNodeInto ( node, parentNode, startIndex ) ;
               startIndex++ ;

               createSubTypeDrillDown ( model, node, 0, countryRec ) ;
            } /* if  */
            i++ ;
         } /* while  */

      } /* if countryVec  */
   } /* createSubCountryDrillDown  */



   /*------------------------------------------------------------*/
   /* createSubTypeDrillDown
    */
   /**
    *    This method will create all sub-items under
    *  a countrygraphy for a view
    *  
    *
    *  @param model The model to user
    *  @param node The node to insert into
    *  @param view The view to use
    *  @param index The index to start adding items at
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createSubTypeDrillDown ( DefaultTreeModel model, StringTreeNode root, int index, TypeCountryCodeRec countryRec ) {
      Vector typeActionVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_ACTION ) ;

      if ( ( typeActionVec != null )&& ( typeActionVec.size() > 0 ) ) {
         int            i              = 0 ;
         int            size           = typeActionVec.size() ;
         TypeActionRec typeActionRec = null ;
         ViewTreeNode   node           = null ;

         while ( i < size ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the Node, and assign our title and SQL           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            node = new ViewTreeNode ( );
            node.setName ( ((TypeActionRec)typeActionVec.elementAt(i)).toString() ) ;
            node.setSQL  ( "SELECT A.ACTIONIND,   B.DOCCLASSIND, B.TITLE, "+
                           "       A.CHANGEDTIME, A.DBUSER,      B.SORTORDER "+     
                           "FROM   OA.ACTION        A, "+
                           "       OA.ACTIONTITLE   B, "+
                           "       OA.ACTIONCOUNTRY C  "+
                           "WHERE  A.ACTIONTYPEIND = "+((TypeActionRec)typeActionVec.elementAt(i)).getInd()+" AND "+
                           "       A.ARCHIVED      = 'N' AND "+
                           ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                           "       A.ACTIONIND     = C.ACTIONIND AND "+
                           "       C.COUNTRYIND    = "+countryRec.getInd()+" AND "+
                           "       B.ACTIONIND     = A.ACTIONIND "+
                           "       ORDER BY A.ACTIONIND, B.SORTORDER DESC "+
                           "FOR FETCH ONLY" ) ;

            model.insertNodeInto ( node, root, 0 ) ;

            /* Insert the 'Please Wait' node */
            model.insertNodeInto ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ), node, 0 ) ;
            i++ ;
         } /* while */
      } /* if */


   } /* createSubCountryDrillDown */




   /*------------------------------------------------------------*/
   /* createSubTypeDrillDown
    */
   /**
    *    This method will create all sub-items under
    *  a countrygraphy for a view
    *  
    *
    *  @param model The model to user
    *  @param node The node to insert into
    *  @param view The view to use
    *  @param index The index to start adding items at
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createSubTypeDrillDown ( DefaultTreeModel model, StringTreeNode root, int index, TypeGeoRec geoRec ) {
      Vector typeActionVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_ACTION ) ;

      if ( ( typeActionVec != null )&& ( typeActionVec.size() > 0 ) ) {
         int            i              = 0 ;
         int            size           = typeActionVec.size() ;
         TypeActionRec typeActionRec = null ;
         ViewTreeNode   node           = null ;

         while ( i < size ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the Node, and assign our title and SQL           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            node = new ViewTreeNode ( );
            node.setName ( ((TypeActionRec)typeActionVec.elementAt(i)).toString() ) ;
            node.setSQL  ( "SELECT A.ACTIONIND,   B.DOCCLASSIND, B.TITLE, "+
                           "       A.CHANGEDTIME, A.DBUSER,      B.SORTORDER "+     
                           "FROM   OA.ACTION        A, "+
                           "       OA.ACTIONTITLE   B, "+
                           "       OA.ACTIONCOUNTRY C  "+
                           "WHERE  A.ACTIONTYPEIND = "+((TypeActionRec)typeActionVec.elementAt(i)).getInd()+" AND "+
                           "       A.ARCHIVED      = 'N' AND "+
                           ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                           "       A.ACTIONIND     = C.ACTIONIND AND "+
                           "       C.COUNTRYIND    IN (SELECT COUNTRYCODEIND FROM TIGRIS.GEOCOUNTRY WHERE GEOIND = "+geoRec.getInd()+") AND "+
                           "       B.ACTIONIND     = A.ACTIONIND "+
                           "       ORDER BY A.ACTIONIND, B.SORTORDER DESC "+
                           "FOR FETCH ONLY" ) ;

            model.insertNodeInto ( node, root, i ) ;

            /* Insert the 'Please Wait' node */
            model.insertNodeInto ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ), node, 0 ) ;
            i++ ;
         } /* while */
      } /* if */


   } /* createSubCountryDrillDown */





} /* ViewActionType */
