package com.ibm.nzna.projects.qit.avalon.oanav.views ;         
/*-----------------------------------------------------------*/
/*   ViewArchivedQuestion                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewArchivedQuestion                */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This view will list Questions by their geography, 
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewArchivedQuestion                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewArchivedQuestion extends Object implements OANavView, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  TypeBrandRec           brandRec = null ;
   private  DefaultMutableTreeNode root     = null ;


   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_ARCHIVED )+" "+Str.getStr ( STR_QUESTIONS ) ) ;
   }

   /*------------------------------------------------------------*/
   /* getKey
    */
   /**
    *    This method will return the key, or the names
    *  of the columns returned, from this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getKey () {
      return( Str.getStr ( STR_DOC_CLASS )+", "+
              Str.getStr ( STR_INDICATOR )+", "+
              Str.getStr ( STR_TITLE )+" "+
              Str.getStr ( STR_LAST_TOUCHED_ON )+", "+
              Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
   }



   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( Str.getStr ( STR_QUESTIONS ) ) ;
   }




   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the brand that this
    *  view will be based on
    *  
    *
    *  @param brandRec The BrandRec to base the view on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( TypeBrandRec brandRec ) {
      this.brandRec = brandRec ;
   }





   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the complete DefaultTreeModel
    *  needed to show this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DefaultTreeModel
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getJTreeModel () {
      DefaultTreeModel model  = null ;
      Vector           geoVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_GEOGRAPHY ) ;                                         


      model = new DefaultTreeModel ( root = new DefaultMutableTreeNode() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Since all views currently drill down by GEO first,      */
      /* we will create each default geo drill down, and then fill */
      /* in the rest.                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( geoVec != null ) && ( geoVec.size() > 0 ) ) {
         int             i         = 0 ;
         int             size      = geoVec.size() ;
         ViewTreeNode    node      = null ;

         while ( i < size ) {
            if ( ((TypeGeoRec)geoVec.elementAt(i)).getInd() != TypeList.GEO_WORLD_WIDE ) {
               node = new ViewTreeNode() ;
               node.setName ( ((TypeGeoRec)geoVec.elementAt(i)).toString() ) ;

               model.insertNodeInto ( node, root, i ) ;

               node.setSQL  ( "SELECT A.QUESTIONIND, B.DOCCLASSIND, B.TITLE, "+
                              "       A.DBUSER,      A.CHANGEDTIME, B.SORTORDER "+
                              "FROM   OA.QUESTION        A, "+
                              "       OA.QUESTIONTITLE   B, "+
                              "       OA.QUESTIONCOUNTRY C  "+
                              "WHERE  A.ARCHIVED       = 'Y' AND "+
                              ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                              "       A.QUESTIONIND    = C.QUESTIONIND AND "+
                              "       C.COUNTRYIND     IN ( SELECT COUNTRYCODEIND FROM TIGRIS.GEOCOUNTRY WHERE GEOIND = "+((TypeGeoRec)geoVec.elementAt(i)).getInd()+") AND "+
                              "       B.QUESTIONIND    = A.QUESTIONIND "+
                              "       ORDER BY A.QUESTIONIND, B.SORTORDER DESC "+
                              "FOR FETCH ONLY" ) ;

               /* Insert the 'Please Wait' node */
               model.insertNodeInto ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ), node, 0 ) ;
            }
            i++ ;
         }
      }



      return( model ) ;
   }





} /* ViewArchivedQuestion */
