package com.ibm.nzna.projects.qit.avalon.oanav.views ;         
/*-----------------------------------------------------------*/
/*   ViewDraftQuestionTitle                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewDraftQuestionTitle              */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This view will list Questions by their geography, 
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewDraftQuestionTitle                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewDraftQuestionTitle extends Object implements OANavView, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  TypeBrandRec           brandRec = null ;
   private  DefaultMutableTreeNode root     = null ;


   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_DRAFT )+" "+Str.getStr ( STR_QUESTION )+" "+Str.getStr ( STR_BY_TITLE_NUMBER ) ) ;
   }

   /*------------------------------------------------------------*/
   /* getKey
    */
   /**
    *    This method will return the key, or the names
    *  of the columns returned, from this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getKey () {
      return( Str.getStr ( STR_DOC_CLASS )+", "+
              Str.getStr ( STR_INDICATOR )+", "+
              Str.getStr ( STR_TITLE )+" "+
              Str.getStr ( STR_LAST_TOUCHED_ON )+", "+
              Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
   }



   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( Str.getStr ( STR_BY_TITLE_NUMBER ) ) ;
   }




   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the brand that this
    *  view will be based on
    *  
    *
    *  @param brandRec The BrandRec to base the view on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( TypeBrandRec brandRec ) {
      this.brandRec = brandRec ;
   }





   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the complete DefaultTreeModel
    *  needed to show this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DefaultTreeModel
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getJTreeModel () {
      DefaultTreeModel model  = null ;
      Vector           geoVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_GEOGRAPHY ) ;                                         


      model = new DefaultTreeModel ( root = new DefaultMutableTreeNode() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Since all views currently drill down by GEO first,      */
      /* we will create each default geo drill down, and then fill */
      /* in the rest.                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( geoVec != null ) && ( geoVec.size() > 0 ) ) {
         int             i         = 0 ;
         int             size      = geoVec.size() ;
         ViewTreeNode    node      = null ;
         StringTreeNode  geoNode   = null ;

         while ( i < size ) {
            if ( ((TypeGeoRec)geoVec.elementAt(i)).getInd() != TypeList.GEO_WORLD_WIDE ) {
               geoNode = new StringTreeNode ( geoVec.elementAt(i).toString() ) ;
               model.insertNodeInto ( geoNode, root, i ) ;

               node = new ViewTreeNode ( Str.getStr ( STR_ALL ), "" ) ;


               node.setSQL  ( "SELECT A.QUESTIONIND, B.DOCCLASSIND, B.TITLE, "+
                              "       A.DBUSER,      A.CHANGEDTIME, A.STARTDATE, A.STOPDATE, A.ACTIVEQUESTIONIND, B.SORTORDER "+
                              "FROM   OADRAFT.QUESTION        A, "+
                              "       OADRAFT.QUESTIONTITLE   B, "+
                              "       OADRAFT.QUESTIONCOUNTRY C  "+
                              "WHERE  A.ARCHIVED       = 'N' AND "+
                              ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                              "       A.QUESTIONIND    = C.QUESTIONIND AND "+
                              "       C.COUNTRYIND     IN ( SELECT COUNTRYCODEIND FROM TIGRIS.GEOCOUNTRY WHERE GEOIND = "+((TypeGeoRec)geoVec.elementAt(i)).getInd()+") AND "+
                              "       B.QUESTIONIND    = A.QUESTIONIND AND "+
                              "       A.STOPDATE       IS NULL "+
                              "       ORDER BY A.QUESTIONIND, B.SORTORDER DESC "+
                              "FOR FETCH ONLY" ) ;            

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Put the 'All' Node in                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               model.insertNodeInto ( node, geoNode, 0 ) ;                              
               model.insertNodeInto ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ),
                                      node,
                                      0 ) ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create all countries now under that geo                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               createSubCountryDrillDown ( model, geoNode, 1, (TypeGeoRec)geoVec.elementAt(i) ) ;

            }
            i++ ;
         }
      }



      return( model ) ;
   }







   /*------------------------------------------------------------*/
   /* createSubCountryDrillDown
    */
   /**
    *    this method will add all countries under
    *  a specific geography to the model passed in
    *  
    *
    *  @param model The model to add nodes to
    *  @param parentNode The node to add nodes to
    *  @param index The index to start inserting nodes at
    *  @param geoRec The TypeGeoRec that we are putting countries in for
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createSubCountryDrillDown ( DefaultTreeModel model, StringTreeNode parentNode, int index, TypeGeoRec geoRec ) {
      Vector countryVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;
      int    i          = 0 ;

      if ( countryVec != null ) {
         int                  size       = countryVec.size() ;
         TypeCountryCodeRec   countryRec = null ;
         ViewTreeNode         node       = null ;

         while ( i < size ) {
            countryRec = (TypeCountryCodeRec) countryVec.elementAt(i) ;
            if ( countryRec.getGeoInd() == geoRec.getInd() ) {
               node = new ViewTreeNode() ;
               node.setName ( countryRec.toString() ) ;
               node.setSQL  ( "SELECT A.QUESTIONIND, B.DOCCLASSIND, B.TITLE, "+
                              "       A.DBUSER,      A.CHANGEDTIME, A.STARTDATE, A.STOPDATE, A.ACTIVEQUESTIONIND, B.SORTORDER "+
                              "FROM   OADRAFT.QUESTION        A, "+
                              "       OADRAFT.QUESTIONTITLE   B, "+
                              "       OADRAFT.QUESTIONCOUNTRY C  "+
                              "WHERE  A.ARCHIVED       = 'N' AND "+
                              ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                              "       A.QUESTIONIND    = C.QUESTIONIND AND "+
                              "       C.COUNTRYIND     = "+countryRec.getInd()+" AND "+
                              "       B.QUESTIONIND    = A.QUESTIONIND AND "+
                              "       A.STOPDATE       IS NULL "+
                              "       ORDER BY A.QUESTIONIND, B.SORTORDER DESC "+
                              "FOR FETCH ONLY" ) ;   
                              
               model.insertNodeInto ( node, parentNode, index ) ;
               index++ ;

               model.insertNodeInto ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ),
                                      node,
                                      0 ) ;
            } /* if  */
            i++ ;
         } /* while  */

      } /* if countryVec  */

   }


} /* ViewDraftQuestionTitle */
