package com.ibm.nzna.projects.qit.avalon.oanav.views ;         
/*-----------------------------------------------------------*/
/*   ViewDraftQuestionType                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewDraftQuestionType               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This view will list Questions by their geography, and then by their type. Their 
 * type is 'Default', 'Component List' etc.. defined in OA.TYPEQUESTION
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewDraftQuestionType                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewDraftQuestionType extends Object implements OANavView, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  TypeBrandRec           brandRec = null ;
   private  DefaultMutableTreeNode root     = null ;




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_DRAFT )+" "+Str.getStr ( STR_QUESTION )+" "+Str.getStr ( STR_BY_TYPE ) ) ;
   }


   /*------------------------------------------------------------*/
   /* getKey
    */
   /**
    *    This method will return the key, or the names
    *  of the columns returned, from this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getKey () {
      return( Str.getStr ( STR_DOC_CLASS )+", "+
              Str.getStr ( STR_INDICATOR )+", "+
              Str.getStr ( STR_TITLE )+" "+
              Str.getStr ( STR_LAST_TOUCHED_ON )+", "+
              Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
   }


   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( Str.getStr ( STR_BY_TYPE ) ) ;
   }




   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the brand that this
    *  view will be based on
    *  
    *
    *  @param brandRec The BrandRec to base the view on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( TypeBrandRec brandRec ) {
      this.brandRec = brandRec ;
   }





   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the complete DefaultTreeModel
    *  needed to show this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DefaultTreeModel
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getJTreeModel () {
      DefaultTreeModel model  = null ;
      Vector           geoVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_GEOGRAPHY ) ;                                         


      model = new DefaultTreeModel ( root = new DefaultMutableTreeNode("") ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Since all views currently drill down by GEO first,      */
      /* we will create each default geo drill down, and then fill */
      /* in the rest.                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( geoVec != null ) && ( geoVec.size() > 0 ) ) {
         int             i       = 0 ;
         int             size    = geoVec.size() ;
         StringTreeNode  geoNode = null ;
         StringTreeNode  node    = null ;

         while ( i < size ) {

            if ( ((TypeGeoRec)geoVec.elementAt(i)).getInd() != TypeList.GEO_WORLD_WIDE ) {
               geoNode = new StringTreeNode ( ((TypeGeoRec)geoVec.elementAt(i)).toString(), 0 ) ;
               model.insertNodeInto ( geoNode, root, i ) ;

               node = new StringTreeNode ( Str.getStr ( STR_ALL ) ) ;
               model.insertNodeInto ( node, geoNode, 0 ) ;
               createSubTypeDrillDown ( model, node, 1, (TypeGeoRec) geoVec.elementAt(i) ) ; 
               createSubCountryDrillDown ( model, geoNode, 1, (TypeGeoRec) geoVec.elementAt(i) ) ;
            }

            i++ ;

         }
      }



      return( model ) ;
   }







   /*------------------------------------------------------------*/
   /* createSubCountryDrillDown
    */
   /**
    *    this method will add all countries under
    *  a specific geography to the model passed in
    *  
    *
    *  @param model The model to add nodes to
    *  @param parentNode The node to add nodes to
    *  @param index The index to start inserting nodes at
    *  @param geoRec The TypeGeoRec that we are putting countries in for
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createSubCountryDrillDown ( DefaultTreeModel model, StringTreeNode parentNode, int index, TypeGeoRec geoRec ) {
      Vector countryVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;
      int    i          = 0 ;

      if ( countryVec != null ) {
         int                  size       = countryVec.size() ;
         TypeCountryCodeRec   countryRec = null ;
         StringTreeNode       node       = null ;

         while ( i < size ) {
            countryRec = (TypeCountryCodeRec) countryVec.elementAt(i) ;
            if ( countryRec.getGeoInd() == geoRec.getInd() ) {
               node = new StringTreeNode ( countryRec.toString() ) ;
                              
               model.insertNodeInto ( node, parentNode, index ) ;
               createSubTypeDrillDown ( model, node, 0, countryRec ) ;
               index++ ;

            } /* if  */
            i++ ;
         } /* while  */

      } /* if countryVec  */

   }





   /*------------------------------------------------------------*/
   /* createSubTypeDrillDown
    */
   /**
    *    This method will create all sub-items under
    *  a Country for a view
    *  
    *
    *  @param model The model to user
    *  @param node The node to insert into
    *  @param view The view to use
    *  @param index The index to start adding items at
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createSubTypeDrillDown ( DefaultTreeModel model, StringTreeNode root, int index, TypeCountryCodeRec countryRec ) {
      Vector typeQuestionVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_QUESTION ) ;

      if ( ( typeQuestionVec != null )&& ( typeQuestionVec.size() > 0 ) ) {
         int            i              = 0 ;
         int            size           = typeQuestionVec.size() ;
         TypeQuestionRec typeQuestionRec = null ;
         ViewTreeNode   node           = null ;

         while ( i < size ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the Node, and assign our title and SQL           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            node = new ViewTreeNode ( );
            node.setName ( ((TypeQuestionRec)typeQuestionVec.elementAt(i)).toString() ) ;
            node.setSQL  ( "SELECT A.QUESTIONIND, B.DOCCLASSIND, B.TITLE, "+
                           "       A.DBUSER,      A.CHANGEDTIME, A.STARTDATE, A.STOPDATE, A.ACTIVEQUESTIONIND, B.SORTORDER "+
                           "FROM   OADRAFT.QUESTION        A, "+
                           "       OADRAFT.QUESTIONTITLE   B, "+
                           "       OADRAFT.QUESTIONCOUNTRY C  "+
                           "WHERE  A.QUESTIONTYPEIND = "+((TypeQuestionRec)typeQuestionVec.elementAt(i)).getInd()+" AND "+
                           "       A.STOPDATE       IS NULL AND "+
                           "       A.ARCHIVED        = 'N' AND "+
                           ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                           "       A.QUESTIONIND     = C.QUESTIONIND AND "+
                           "       C.COUNTRYIND      = "+countryRec.getInd()+" AND "+
                           "       B.QUESTIONIND     = A.QUESTIONIND "+
                           "       ORDER BY A.QUESTIONIND, B.SORTORDER DESC "+
                           "FOR FETCH ONLY" ) ;

            model.insertNodeInto ( node, root, 0 ) ;

            /* Insert the 'Please Wait' node */
            model.insertNodeInto ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ), node, 0 ) ;
            i++ ;
         } /* while */
      } /* if */


   } /* createSubTypeDrillDown */


   /*------------------------------------------------------------*/
   /* createSubTypeDrillDown
    */
   /**
    *    This method will create all sub-items under
    *  a Geography for a view
    *  
    *
    *  @param model The model to user
    *  @param node The node to insert into
    *  @param view The view to use
    *  @param index The index to start adding items at
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createSubTypeDrillDown ( DefaultTreeModel model, StringTreeNode root, int index, TypeGeoRec geoRec ) {
      Vector typeQuestionVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_QUESTION ) ;

      if ( ( typeQuestionVec != null )&& ( typeQuestionVec.size() > 0 ) ) {
         int            i              = 0 ;
         int            size           = typeQuestionVec.size() ;
         TypeQuestionRec typeQuestionRec = null ;
         ViewTreeNode   node           = null ;

         while ( i < size ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the Node, and assign our title and SQL           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            node = new ViewTreeNode ( );
            node.setName ( ((TypeQuestionRec)typeQuestionVec.elementAt(i)).toString() ) ;
            node.setSQL  ( "SELECT A.QUESTIONIND, B.DOCCLASSIND, B.TITLE, "+
                           "       A.DBUSER,      A.CHANGEDTIME, A.STARTDATE, A.STOPDATE, A.ACTIVEQUESTIONIND, B.SORTORDER "+
                           "FROM   OADRAFT.QUESTION        A, "+
                           "       OADRAFT.QUESTIONTITLE   B, "+
                           "       OADRAFT.QUESTIONCOUNTRY C  "+
                           "WHERE  A.QUESTIONTYPEIND = "+((TypeQuestionRec)typeQuestionVec.elementAt(i)).getInd()+" AND "+
                           "       A.STOPDATE       IS NULL AND "+
                           "       A.ARCHIVED        = 'N' AND "+
                           ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                           "       A.QUESTIONIND     = C.QUESTIONIND AND "+
                           "       C.COUNTRYIND      IN (SELECT COUNTRYCODEIND FROM TIGRIS.GEOCOUNTRY WHERE GEOIND = "+geoRec.getInd()+") AND "+
                           "       B.QUESTIONIND     = A.QUESTIONIND "+
                           "       ORDER BY A.QUESTIONIND, B.SORTORDER DESC "+
                           "FOR FETCH ONLY" ) ;

            model.insertNodeInto ( node, root, 0 ) ;

            /* Insert the 'Please Wait' node */
            model.insertNodeInto ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ), node, 0 ) ;
            i++ ;
         } /* while */
      } /* if */


   } /* createSubTypeDrillDown */


} /* ViewDraftQuestionType */
