package com.ibm.nzna.projects.qit.avalon.oanav.views ;         
/*-----------------------------------------------------------*/
/*   ViewNewDraftActivitySymptom                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewNewDraftActivitySymptom         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This view will list Symptoms by their geography, and then by their status. Their 
 * type is 'Flawed', 'Active' etc.. defined in TIGRIS.TYPESTATUS
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewNewDraftActivitySymptom                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewNewDraftActivitySymptom extends Object implements OANavView, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  TypeBrandRec           brandRec = null ;
   private  DefaultMutableTreeNode root     = null ;



   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_NEW_DRAFT )+" "+Str.getStr ( STR_SYMPTOMS ) ) ;
   }


   /*------------------------------------------------------------*/
   /* getKey
    */
   /**
    *    This method will return the key, or the names
    *  of the columns returned, from this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getKey () {
      return( Str.getStr ( STR_DOC_CLASS )+", "+
              Str.getStr ( STR_VALID )+", "+
              Str.getStr ( STR_INDICATOR )+", "+
              Str.getStr ( STR_TITLE )+" "+
              Str.getStr ( STR_LAST_TOUCHED_ON )+", "+
              Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
   }


   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( Str.getStr ( STR_SYMPTOMS ) ) ;
   }




   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the brand that this
    *  view will be based on
    *  
    *
    *  @param brandRec The BrandRec to base the view on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( TypeBrandRec brandRec ) {
      this.brandRec = brandRec ;
   }





   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the complete DefaultTreeModel
    *  needed to show this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DefaultTreeModel
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getJTreeModel () {
      DefaultTreeModel model  = null ;
      Vector           geoVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_GEOGRAPHY ) ;                                         


      model = new DefaultTreeModel ( root = new DefaultMutableTreeNode() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Since all views currently drill down by GEO first,      */
      /* we will create each default geo drill down, and then fill */
      /* in the rest.                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( geoVec != null ) && ( geoVec.size() > 0 ) ) {
         int             i         = 0 ;
         int             size      = geoVec.size() ;
         ViewTreeNode    node      = null ;
         StringTreeNode  geoNode   = null ;

         while ( i < size ) {
            if ( ((TypeGeoRec)geoVec.elementAt(i)).getInd() != TypeList.GEO_WORLD_WIDE ) {
               

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create the geography twistie                            */                   
               /*                                                           */
               /*-----------------------------------------------------------*/
               geoNode = new StringTreeNode ( ((TypeGeoRec)geoVec.elementAt(i)).toString() ) ;
               model.insertNodeInto ( geoNode, root, i ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create the 'All' Twistie under geo                      */                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               node = new ViewTreeNode ( Str.getStr ( STR_ALL ), "" ) ;
               model.insertNodeInto ( node, geoNode, 0 ) ;
               node.setSQL  ( "SELECT A.SYMPTOMIND, B.DOCCLASSIND, A.STATUSIND, B.TITLE, "+
                              "       A.DBUSER,     A.CHANGEDTIME, A.STARTDATE, A.STOPDATE, A.ACTIVESYMPTOMIND, B.SORTORDER "+
                              "FROM   OADRAFT.SYMPTOM        A, "+
                              "       OADRAFT.SYMPTOMTITLE   B, "+
                              "       OADRAFT.SYMPTOMCOUNTRY C  "+
                              "WHERE  A.ARCHIVED       = 'N' AND "+
                              ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                              "       A.SYMPTOMIND     = C.SYMPTOMIND AND "+
                              "       A.STOPDATE       IS NULL AND "+
                              "       C.COUNTRYIND     IN ( SELECT COUNTRYCODEIND FROM TIGRIS.GEOCOUNTRY WHERE GEOIND = "+((TypeGeoRec)geoVec.elementAt(i)).getInd()+") AND "+
                              "       A.RECYCLED       = 'N' AND "+                              "       B.SYMPTOMIND     = A.SYMPTOMIND "+
                              "       ORDER BY SYMPTOMIND, B.SORTORDER DESC "+
                              "FOR FETCH ONLY" ) ;

               /* Insert the 'Please Wait' node */
               model.insertNodeInto ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ), node, 0 ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, put the countries under the geo                    */                           
               /*                                                           */
               /*-----------------------------------------------------------*/
               createSubCountryDrillDown ( model, geoNode, 1, (TypeGeoRec) geoVec.elementAt(i) ) ;
               
            }
            i++ ;
         }
      }



      return( model ) ;
   }



   /*------------------------------------------------------------*/
   /* getSubCountryDrillDown
    */
   /**
    *    This method will put the countries under
    *  each geography
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createSubCountryDrillDown ( DefaultTreeModel model, StringTreeNode parentNode, int startIndex, TypeGeoRec geoRec ) {
      Vector countryVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;
      int    i          = 0 ;

      if ( countryVec != null ) {
         int                  size       = countryVec.size() ;
         TypeCountryCodeRec   countryRec = null ;
         ViewTreeNode         node       = null ;

         while ( i < size ) {
            countryRec = (TypeCountryCodeRec) countryVec.elementAt(i) ;
            if ( countryRec.getGeoInd() == geoRec.getInd() ) {
               node = new ViewTreeNode() ;
               node.setName ( countryRec.toString() ) ;
               node.setSQL  ( "SELECT A.SYMPTOMIND, B.DOCCLASSIND, A.STATUSIND, B.TITLE, "+
                              "       A.DBUSER,     A.CHANGEDTIME, A.STARTDATE, A.STOPDATE, A.ACTIVESYMPTOMIND, B.SORTORDER "+
                              "FROM   OADRAFT.SYMPTOM        A, "+
                              "       OADRAFT.SYMPTOMTITLE   B, "+
                              "       OADRAFT.SYMPTOMCOUNTRY C  "+
                              "WHERE  A.ARCHIVED       = 'N' AND "+
                              ((brandRec.getInd()>=0) ? "A.BRANDIND = "+brandRec.getInd()+" AND " : "" )+
                              "       A.SYMPTOMIND     = C.SYMPTOMIND AND "+
                              "       A.STOPDATE       IS NULL AND "+
                              "       C.COUNTRYIND     = "+countryRec.getInd()+" AND "+
                              "       A.RECYCLED       = 'N' AND "+                              "       B.SYMPTOMIND     = A.SYMPTOMIND "+
                              "       ORDER BY SYMPTOMIND, B.SORTORDER DESC "+
                              "FOR FETCH ONLY" ) ;

               model.insertNodeInto ( node, parentNode, startIndex ) ;
               startIndex++ ;

               model.insertNodeInto ( new ViewTreeNode ( Str.getStr ( STR_PLEASE_WAIT ), null ),
                                      node,
                                      0 ) ;
            } /* if  */
            i++ ;
         } /* while  */

      } /* if countryVec  */
   } /* createSubCountryDrillDown  */




} /* ViewNewDraftActivitySymptom */
