package com.ibm.nzna.projects.qit.avalon.sql ;   
/*-----------------------------------------------------------*/
/*   ActionUsage                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionUsage                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
/**
 * This class looks up what symptoms an Action is used in. 
 *
 *
 * @version 1.0, 11/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionUsage                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionUsage extends Object implements AppConst, AvalonConst {




   /*------------------------------------------------------------*/
   /* getUsage
    */
   /**
    *    This method will return a Vector of Symptoms
    *  that this Action is used in
    *  
    *
    *  @param action The Action to look up usage for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getUsage ( ActionDraft action ) {
      Vector usageVec = null ;                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read the Symptom Usage                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( action.getInd() != 0 ) ||
           ( action.getActiveActionInd() != 0 ) )
         usageVec = getSymptomUsage ( action ) ;


      return( usageVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getSymptomUsage
    */
   /**
    *    This method will look up and return a Vector
    *  of SymptomDisplayRecs. Each symptom in the
    *  vector indicates a symptom that the action passed is used in
    *
    *  @param action The Action to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static Vector getSymptomUsage ( ActionDraft action ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getSymptomUsage", 5 ) ;
      Vector    retVec    = new Vector () ;

      try {
         Statement stmt   = sqlMethod.createStatement () ;
         UsageRec  newRec = null ;
         ResultSet rs     = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, A.RECYCLED, A.CHANGEDTIME, A.DBUSER "+
                                                "FROM OADRAFT.SYMPTOM      A, "+
                                                "     OADRAFT.SYMPTOMTITLE B, "+
                                                "     OADRAFT.NODE         C  "+
                                                "WHERE C.ACTIVEACTIONIND = "+action.getActiveActionInd()+" AND "+
                                                "      a.SYMPTOMIND      = c.SYMPTOMIND AND "+
                                                "      a.STOPDATE        IS NULL        AND "+
                                                "      b.SYMPTOMIND      = c.SYMPTOMIND AND "+
                                                "      B.SORTORDER       = (SELECT MAX (SORTORDER) FROM OADRAFT.SYMPTOMTITLE WHERE SYMPTOMIND = C.SYMPTOMIND) " ) ;                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, read in the drafts                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {
            newRec = new UsageRec ( rs.getInt(1),
                                    rs.getString(2).trim() ) ;
            
            newRec.setWarning ( true ) ;
            newRec.setObjectType ( AVALON_SYMPTOM ) ;
            newRec.setDraft ( true ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this record is recycled, warn them about it          */                                     
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getString(3).trim().equals ( "Y" ) ) {
               newRec.setExplanation ( Str.getStr ( STR_ACTION_USED_IN_DRAFT_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'"+
                                       Str.getStr ( STR_ACTION_USED_IN_RECYCLED_SYMPTOM )+" "+
                                       UserSystem.getNameFromUserId ( rs.getString(5).trim() )+" "+
                                       Str.getStr ( STR_ON )+" "+
                                       DateSystem.prettyDateFromStamp ( rs.getString(4).trim() ) ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, just write the normal warning                */                               
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
               newRec.setExplanation ( Str.getStr ( STR_ACTION_USED_IN_DRAFT_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'" ) ;
            }

            retVec.addElement ( newRec ) ;
         }

         rs.close() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now read in the active Symptoms                     */                          
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, A.OPENDRAFT "+
                                  "FROM OA.SYMPTOM      A, "+
                                  "     OA.SYMPTOMTITLE B, "+
                                  "     OA.NODE         C  "+
                                  "WHERE C.ACTIONIND       = "+action.getActiveActionInd()+" AND "+
                                  "      a.SYMPTOMIND      = c.SYMPTOMIND AND "+
                                  "      a.ARCHIVED        = 'N'          AND "+
                                  "      b.SYMPTOMIND      = c.SYMPTOMIND AND "+
                                  "      B.SORTORDER       = (SELECT MAX (SORTORDER) FROM OA.SYMPTOMTITLE WHERE SYMPTOMIND = C.SYMPTOMIND) " ) ;                
         

         while ( rs.next() ) {

            if ( rs.getString(3).trim().equals ( "N" ) ) {
               newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;

               newRec.setDraft ( false ) ;
               newRec.setWarning ( true ) ;
               newRec.setObjectType ( AVALON_SYMPTOM ) ;

               newRec.setExplanation ( Str.getStr ( STR_ACTION_USED_IN_SYMPTOM )+" '"+
                                       rs.getString ( 2 ).trim()+"'" ) ;

            } else {
               newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;

               newRec.setDraft ( false ) ;
               newRec.setWarning ( false ) ;
               newRec.setObjectType ( AVALON_SYMPTOM ) ;

               newRec.setExplanation ( Str.getStr ( STR_ACTION_USED_IN_SYMPTOM )+" '"+
                                       rs.getString ( 2 ).trim()+"'" ) ;

            }

            retVec.addElement ( newRec ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ,false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
      return( retVec ) ;
   }
} /* ActionUsage */
