package com.ibm.nzna.projects.qit.avalon.sql ; 
/*-----------------------------------------------------------*/
/*   LinkGroupUsage                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LinkGroupUsage                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This class holds useful routines for finding the usage of Link Groups 
 * in Actions, Questions and Link Groups 
 *
 *
 * @version 1.0, 11/11/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LinkGroupUsage                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class LinkGroupUsage extends Object implements AppConst, AvalonConst {




   /*------------------------------------------------------------*/
   /* getLinkGroupUsageInActions
    */
   /**
    *    This method will return a list of actions
    *  that a link is used in. This list can contain
    *  both Action Drafts and Active Actions
    *
    *  @param link The link to look for
    *  @param None
    *  @param None
    *
    *  @return Vector of UsageRecs
    *//*---------------------------------------------------------*/
   public static Vector getLinkGroupUsageInActions ( LinkGroupDraft link ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getLinkGroupUsageInActions", 5 ) ;
      Vector    retVec    = new Vector ( 1, 1 ) ;

      try {
         Statement stmt   = sqlMethod.createStatement() ;
         ResultSet rs     = null ;
         UsageRec  newRec = null ;                                           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, find all of the Draft actions using this Link    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.ACTIONIND, B.TITLE, A.RECYCLED "+
                                  "FROM   OADRAFT.ACTION A, "+
                                  "       OADRAFT.ACTIONTITLE B, "+
                                  "       OADRAFT.ACTIONLINK C "+
                                  "WHERE  C.LINKGROUPIND = "+link.getActiveLinkGroupInd()+" AND "+
                                  "       C.ACTIONIND = A.ACTIONIND AND "+
                                  "       A.STOPDATE IS NULL AND "+
                                  "       B.ACTIONIND = A.ACTIONIND AND "+
                                  "       B.SORTORDER = ( SELECT MAX(SORTORDER) FROM OADRAFT.ACTIONTITLE WHERE ACTIONIND = A.ACTIONIND) "+
                                  "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {

            newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
            newRec.setWarning ( true ) ;
            newRec.setObjectType ( AVALON_ACTION_DRAFT ) ;                              

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is recycled, tell the user that                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getString(3).trim().equals ( "Y" ) )
               newRec.setExplanation ( Str.getStr ( STR_LINK_GROUP_USED_IN_RECYCLED_ACTION )+" '"+
                                       rs.getString(2).trim()+"'" ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, let them know of this open draft             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else
               newRec.setExplanation ( Str.getStr ( STR_LINK_GROUP_USED_IN_DRAFT_ACTION )+" '"+
                                       rs.getString(2).trim()+"'" ) ;

            retVec.addElement ( newRec ) ;
         }

         rs.close() ;                                          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, read in the Active Actions that the link is in     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.ACTIONIND, B.TITLE "+
                                  "FROM   OA.ACTION A, "+
                                  "       OA.ACTIONTITLE B, "+
                                  "       OA.ACTIONLINK C "+
                                  "WHERE  C.LINKGROUPIND = "+link.getActiveLinkGroupInd()+" AND "+
                                  "       C.ACTIONIND = A.ACTIONIND AND "+
                                  "       A.ARCHIVED  = 'N' AND "+
                                  "       B.ACTIONIND = A.ACTIONIND AND "+
                                  "       B.SORTORDER = ( SELECT MAX(SORTORDER) FROM OA.ACTIONTITLE WHERE ACTIONIND = A.ACTIONIND) "+
                                  "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {

            newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
            newRec.setWarning ( false ) ;
            newRec.setObjectType ( AVALON_ACTION ) ;

            newRec.setExplanation ( Str.getStr ( STR_LINK_GROUP_USED_IN_ACTION )+" '"+
                                    rs.getString(2).trim()+"'" ) ;

            retVec.addElement ( newRec ) ;

         }

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getLinkGroupUsageInQuestions
    */
   /**
    *    This method will return a list of actions
    *  that a link is used in. This list can contain
    *  both Question Drafts and Active Questions
    *
    *  @param link The link to look for
    *  @param None
    *  @param None
    *
    *  @return Vector of UsageRecs
    *//*---------------------------------------------------------*/
   public static Vector getLinkGroupUsageInQuestions ( LinkGroupDraft link ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getLinkGroupUsageInQuestions", 5 ) ;
      Vector    retVec    = new Vector ( 1, 1 ) ;

      try {
         Statement stmt   = sqlMethod.createStatement() ;
         ResultSet rs     = null ;
         UsageRec  newRec = null ;                                           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, find all of the Draft actions using this Link    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.QUESTIONIND, B.TITLE, A.RECYCLED "+
                                  "FROM   OADRAFT.QUESTION A, "+
                                  "       OADRAFT.QUESTIONTITLE B, "+
                                  "       OADRAFT.QUESTIONLINK C "+
                                  "WHERE  C.LINKGROUPIND = "+link.getActiveLinkGroupInd()+" AND "+
                                  "       C.QUESTIONIND = A.QUESTIONIND AND "+
                                  "       A.STOPDATE IS NULL AND "+
                                  "       B.QUESTIONIND = A.QUESTIONIND AND "+
                                  "       B.SORTORDER = ( SELECT MAX(SORTORDER) FROM OADRAFT.QUESTIONTITLE WHERE QUESTIONIND = A.QUESTIONIND) "+
                                  "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {

            newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
            newRec.setWarning ( true ) ;
            newRec.setObjectType ( AVALON_QUESTION_DRAFT ) ;                              

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is recycled, tell the user that                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getString(3).trim().equals ( "Y" ) )
               newRec.setExplanation ( Str.getStr ( STR_LINK_GROUP_USED_IN_RECYCLED_QUESTION )+" '"+
                                       rs.getString(2).trim()+"'" ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, let them know of this open draft             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else
               newRec.setExplanation ( Str.getStr ( STR_LINK_GROUP_USED_IN_DRAFT_QUESTION )+" '"+
                                       rs.getString(2).trim()+"'" ) ;

            retVec.addElement ( newRec ) ;
         }

         rs.close() ;                                          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, read in the Active Questions that the link is in   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.QUESTIONIND, B.TITLE "+
                                  "FROM   OA.QUESTION A, "+
                                  "       OA.QUESTIONTITLE B, "+
                                  "       OA.QUESTIONLINK C "+
                                  "WHERE  C.LINKGROUPIND = "+link.getActiveLinkGroupInd()+" AND "+
                                  "       C.QUESTIONIND = A.QUESTIONIND AND "+
                                  "       A.ARCHIVED  = 'N' AND "+
                                  "       B.QUESTIONIND = A.QUESTIONIND AND "+
                                  "       B.SORTORDER = ( SELECT MAX(SORTORDER) FROM OA.QUESTIONTITLE WHERE QUESTIONIND = A.QUESTIONIND) "+
                                  "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {

            newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
            newRec.setWarning ( false ) ;
            newRec.setObjectType ( AVALON_QUESTION ) ;

            newRec.setExplanation ( Str.getStr ( STR_LINK_GROUP_USED_IN_QUESTION )+" '"+
                                    rs.getString(2).trim()+"'" ) ;

            retVec.addElement ( newRec ) ;

         }

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
      return( retVec ) ;
   }



} /* LinkGroupUsage */
