package com.ibm.nzna.projects.qit.avalon.sql ;
/*-----------------------------------------------------------*/
/*   OASQL                                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OASQL                               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class has static methods usefull for the OA Navigator 
 *
 *
 * @version 1.0, 4/17/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OASQL                                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class OASQL extends Object implements AppConst, AvalonConst {




   /*------------------------------------------------------------*/
   /* readSymptomList
    */
   /**
    *    This method will call the symptomListReader
    *  strored procedure with the SQL passed, and will
    *  return a vector of SymptomDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readSymptomList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readSymptomList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.symReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Symptom Query:"+query ) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readActionList
    */
   /**
    *    This method will call the actReader
    *  stored procedure with the SQL passed, and will
    *  return a vector of ActionDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readActionList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readActionList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.ActReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Action Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readActionDraftList
    */
   /**
    *    This method will call the actDraftReader
    *  stored procedure with the SQL passed, and will
    *  return a vector of ActionDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readActionDraftList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readActionDraftList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.ActDraftReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Action Draft Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readQuestionList
    */
   /**
    *    This method will call the qstReader
    *  stored procedure with the SQL passed, and will
    *  return a vector of QuestionDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readQuestionList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readQuestionList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.QstReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Question Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readLinkList
    */
   /**
    *    This method will call the lnkReader
    *  stored procedure with the SQL passed, and will
    *  return a vector of LinkDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readLinkList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readLinkList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.LnkReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Link Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readLinkListGroup
    */
   /**
    *    This method will call the lnkReader
    *  stored procedure with the SQL passed, and will
    *  return a vector of LinkDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readLinkGroupList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readLinkListGroup", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.LnkGrpRdr", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Link Group Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readDraftLinkList
    */
   /**
    *    This method will call the lnkDraftReader
    *  stored procedure with the SQL passed, and will
    *  return a vector of LinkDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readDraftLinkList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readDraftLinkList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.LnkDraftReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Draft Link Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readDraftLinkListGroup
    */
   /**
    *    This method will call the lnkGrpDraftRdr
    *  stored procedure with the SQL passed, and will
    *  return a vector of LinkDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readDraftLinkGroupList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readDraftLinkListGroup", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.LnkGrpDraftRdr", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Draft Link group Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readDraftQuestionList
    */
   /**
    *    This method will call the qstDraftReader
    *  stored procedure with the SQL passed, and will
    *  return a vector of QuestionDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readDraftQuestionList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readDraftQuestionList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.QstDraftReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Question Draft Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }


   /*------------------------------------------------------------*/
   /* readDraftSymptomList
    */
   /**
    *    This method will call the symDraftReader
    *  strored procedure with the SQL passed, and will
    *  return a vector of SymptomDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readDraftSymptomList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readDraftSymptomList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.symDraftReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Symptom Draft Query:"+query ) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readAnswerList
    */
   /**
    *    This method will call the ansReader
    *  stored procedure with the SQL passed, and will
    *  return a vector of AnswerDisplayRec
    *
    *  @param query The query to run
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readAnswerList ( String query ) {
      SQLMethod           sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "OASQL.readAnswerList", 5 ) ;
      boolean             loggedIn     = false ;
      CallableStatement   stmt         = sqlMethod.createStoredProc ( "QUEST.AnsReader", 3 ) ;
      Vector              returnVec    = null ;

      LogSystem.log ( 1, "Answer Query:"+query ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put up our status                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( STR_PLEASE_WAIT ) ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the call                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.setBytes             ( 1, query.getBytes() ) ;
         stmt.registerOutParameter ( 2, Types.LONGVARBINARY ) ;
         stmt.registerOutParameter ( 3, Types.LONGVARBINARY ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         stmt.execute() ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the StoredProcRec from the Lob                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 3 ) ;                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Throw an error if need be                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( retRec != null ) && ( retRec.error ) ) {
            LogSystem.log ( 1, retRec.errorStr ) ;
            System.out.println ( "Error Occurred:\n"+retRec.errorStr ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, continue                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our return object                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            returnVec = (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 2 ) ;           

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         sqlMethod.rollBack() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Like every good SQL Method, close up                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlMethod.close() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear our status                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( MainWindow.getInstance() != null )
         MainWindow.getInstance().setStatus ( null ) ;
      return( returnVec ) ;
   }






   /*------------------------------------------------------------*/
   /* readDefaultActionTitle
    */
   /**
    *    This method will read in the default action
    *  title for the action indicator given
    *  
    *
    *  @param actionInd The Action Indicator to read in
    *  @param draft TRUE = Read from Draft tables, FALSE = Read from Active
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String readDefaultActionTitle ( int actionInd, boolean draft ) {
      SQLMethod sqlMethod   = new SQLMethod ( DatabaseSystem.QUEST, "readDefaultActionTitle ( "+actionInd+", "+draft+")", 5 ) ;
      String    returnTitle = null ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = null ;

         if ( draft )
            rs = stmt.executeQuery ( "SELECT B.TITLE "+
                                     "FROM OADRAFT.ACTION A, "+
                                     "     OADRAFT.ACTIONTITLE B "+
                                     "WHERE A.ACTIONIND = "+actionInd+" AND "+
                                     "      A.ACTIONIND = B.ACTIONIND AND "+
                                     "      B.SORTORDER = (SELECT MAX(SORTORDER) FROM OADRAFT.ACTIONTITLE WHERE ACTIONIND = "+actionInd+") "+
                                     "FOR FETCH ONLY" ) ;
         else
            rs = stmt.executeQuery ( "SELECT B.TITLE "+
                                     "FROM OA.ACTION A, "+
                                     "     OA.ACTIONTITLE B "+
                                     "WHERE A.ACTIONIND = "+actionInd+" AND "+
                                     "      A.ACTIONIND = B.ACTIONIND AND "+
                                     "      B.SORTORDER = (SELECT MAX(SORTORDER) FROM OA.ACTIONTITLE WHERE ACTIONIND = "+actionInd+") "+
                                     "FOR FETCH ONLY" ) ;

         if ( rs.next() )
            returnTitle = rs.getString(1).trim() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;

      return( returnTitle ) ;
   }



   /*------------------------------------------------------------*/
   /* readDefaultQuestionTitle
    */
   /**
    *    This method will read in the default action
    *  title for the action indicator given
    *  
    *
    *  @param actionInd The Question Indicator to read in
    *  @param draft TRUE = Read from Draft tables, FALSE = Read from Active
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String readDefaultQuestionTitle ( int questionInd, boolean draft ) {
      SQLMethod sqlMethod   = new SQLMethod ( DatabaseSystem.QUEST, "readDefaultQuestionTitle ( "+questionInd+", "+draft+")", 5 ) ;
      String    returnTitle = null ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = null ;

         if ( stmt != null ) {
            if ( draft )
               rs = stmt.executeQuery ( "SELECT B.TITLE "+
                                        "FROM OADRAFT.QUESTION A, "+
                                        "     OADRAFT.QUESTIONTITLE B "+
                                        "WHERE A.QUESTIONIND = "+questionInd+" AND "+
                                        "      A.QUESTIONIND = B.QUESTIONIND AND "+
                                        "      B.SORTORDER = (SELECT MAX(SORTORDER) FROM OADRAFT.QUESTIONTITLE WHERE QUESTIONIND = "+questionInd+") "+
                                        "FOR FETCH ONLY" ) ;
            else
               rs = stmt.executeQuery ( "SELECT B.TITLE "+
                                        "FROM OA.QUESTION A, "+
                                        "     OA.QUESTIONTITLE B "+
                                        "WHERE A.QUESTIONIND = "+questionInd+" AND "+
                                        "      A.QUESTIONIND = B.QUESTIONIND AND "+
                                        "      B.SORTORDER = (SELECT MAX(SORTORDER) FROM OA.QUESTIONTITLE WHERE QUESTIONIND = "+questionInd+") "+
                                        "FOR FETCH ONLY" ) ;

            if ( rs.next() )
               returnTitle = rs.getString(1).trim() ;
         }
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;

      return( returnTitle ) ;
   }


   /*------------------------------------------------------------*/
   /* readDefaultSymptomTitle
    */
   /**
    *    This method will read in the default symptom
    *  title for the symptom indicator given
    *  
    *
    *  @param symptomInd The Symptom Indicator to read in
    *  @param draft TRUE = Read from Draft tables, FALSE = Read from Active
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String readDefaultSymptomTitle ( int symptomInd, boolean draft ) {
      SQLMethod sqlMethod   = new SQLMethod ( DatabaseSystem.QUEST, "readDefaultSymptomTitle ( "+symptomInd+", "+draft+")", 5 ) ;
      String    returnTitle = null ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = null ;

         if ( draft )
            rs = stmt.executeQuery ( "SELECT B.TITLE "+
                                     "FROM OADRAFT.SYMPTOM A, "+
                                     "     OADRAFT.SYMPTOMTITLE B "+
                                     "WHERE A.SYMPTOMIND = "+symptomInd+" AND "+
                                     "      A.SYMPTOMIND = B.SYMPTOMIND AND "+
                                     "      B.SORTORDER = (SELECT MAX(SORTORDER) FROM OADRAFT.SYMPTOMTITLE WHERE SYMPTOMIND = "+symptomInd+") "+
                                     "FOR FETCH ONLY" ) ;
         else
            rs = stmt.executeQuery ( "SELECT B.TITLE "+
                                     "FROM OA.SYMPTOM A, "+
                                     "     OA.SYMPTOMTITLE B "+
                                     "WHERE A.SYMPTOMIND = "+symptomInd+" AND "+
                                     "      A.SYMPTOMIND = B.SYMPTOMIND AND "+
                                     "      B.SORTORDER = (SELECT MAX(SORTORDER) FROM OA.SYMPTOMTITLE WHERE SYMPTOMIND = "+symptomInd+") "+
                                     "FOR FETCH ONLY" ) ;

         if ( rs.next() )
            returnTitle = rs.getString(1).trim() ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close() ;

      return( returnTitle ) ;
   }





   /*------------------------------------------------------------*/
   /* getQuestionDraft
    */
   /**
    *    This method will try and read a question draft
    *  from the database. If a draft does not exist,
    *  a new draft will be created
    *
    *  @param parent The parent AppDefaultWin that is loading the draft. This value can be null
    *  @param questionInd The active question indicator
    *  @param typeWorkInd The work indicator as defined in com.ibm.nzna.projects.common.quest.type.TypeList ( TYPEWORK_CREATE etc.. )
    *
    *  @return QuestionDraft. NULL if one cannot be created
    *//*---------------------------------------------------------*/
   public static QuestionDraft getQuestionDraft ( AppDefaultWin parent, int questionInd, int typeWorkInd ) {
      QuestionDraft retDraft = null ;

      if ( parent != null )
         parent.setStatus ( STR_READING ) ;

      try {
         retDraft = QuestionDraft.getDraft ( questionInd,
                                             PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                             (TypeWorkRec)TypeList.getInstance().objectFromInd ( typeWorkInd, TypeList.TYPE_WORKREQUIRED ) ) ;
      } catch ( PendingDraftException pde ) {
         LogSystem.log ( 1, pde, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
      }

      if ( parent != null )
         parent.setStatus ( null ) ;
      return( retDraft ) ;
   }


   /*------------------------------------------------------------*/
   /* getActionDraft
    */
   /**
    *    This method will try and read a action draft
    *  from the database. If a draft does not exist,
    *  a new draft will be created
    *
    *  @param parent The parent AppDefaultWin that is loading the draft. This value can be null
    *  @param actionInd The active action indicator
    *  @param typeWorkInd The work indicator as defined in com.ibm.nzna.projects.common.quest.type.TypeList ( TYPEWORK_CREATE etc.. )
    *
    *  @return ActionDraft. NULL if one cannot be created
    *//*---------------------------------------------------------*/
   public static ActionDraft getActionDraft ( AppDefaultWin parent, int actionInd, int typeWorkInd ) {
      ActionDraft retDraft = null ;

      if ( parent != null )
         parent.setStatus ( STR_READING ) ;

      try {
         retDraft = ActionDraft.getDraft ( actionInd,
                                           PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                           (TypeWorkRec)TypeList.getInstance().objectFromInd ( typeWorkInd, TypeList.TYPE_WORKREQUIRED ) ) ;
      } catch ( PendingDraftException pde ) {
         LogSystem.log ( 1, pde, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
      }

      if ( parent != null )
         parent.setStatus ( null ) ;
      return( retDraft ) ;
   }


   /*------------------------------------------------------------*/
   /* getQuestLinkDraft
    */
   /**
    *    This method will try and read a questLink draft
    *  from the database. If a draft does not exist,
    *  a new draft will be created
    *
    *  @param parent The parent AppDefaultWin that is loading the draft. This value can be null
    *  @param questLinkInd The active questLink indicator
    *  @param typeWorkInd The work indicator as defined in com.ibm.nzna.projects.common.quest.type.TypeList ( TYPEWORK_CREATE etc.. )
    *
    *  @return QuestLinkDraft. NULL if one cannot be created
    *//*---------------------------------------------------------*/
   public static LinkDraft getQuestLinkDraft ( AppDefaultWin parent, int questLinkInd, int typeWorkInd ) {
      LinkDraft retDraft = null ;

      if ( parent != null )
         parent.setStatus ( STR_READING ) ;

      try {
         retDraft = QuestLinkDraft.getDraft ( questLinkInd,
                                              PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                              (TypeWorkRec)TypeList.getInstance().objectFromInd ( typeWorkInd, TypeList.TYPE_WORKREQUIRED ) ) ;
      } catch ( PendingDraftException pde ) {
         LogSystem.log ( 1, pde, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
      }

      if ( parent != null )
         parent.setStatus ( null ) ;
      return( retDraft ) ;
   }


   /*------------------------------------------------------------*/
   /* getExternalLinkDraft
    */
   /**
    *    This method will try and read a externalLink draft
    *  from the database. If a draft does not exist,
    *  a new draft will be created
    *
    *  @param parent The parent AppDefaultWin that is loading the draft. This value can be null
    *  @param externalLinkInd The active externalLink indicator
    *  @param typeWorkInd The work indicator as defined in com.ibm.nzna.projects.common.quest.type.TypeList ( TYPEWORK_CREATE etc.. )
    *
    *  @return ExternalLinkDraft. NULL if one cannot be created
    *//*---------------------------------------------------------*/
   public static LinkDraft getExternalLinkDraft ( AppDefaultWin parent, int externalLinkInd, int typeWorkInd ) {
      LinkDraft retDraft = null ;

      if ( parent != null )
         parent.setStatus ( STR_READING ) ;

      try {
         retDraft = ExternalLinkDraft.getDraft ( externalLinkInd,
                                                 PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                                 (TypeWorkRec)TypeList.getInstance().objectFromInd ( typeWorkInd, TypeList.TYPE_WORKREQUIRED ) ) ;
      } catch ( PendingDraftException pde ) {
         LogSystem.log ( 1, pde, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
      }

      if ( parent != null )
         parent.setStatus ( null ) ;
      return( retDraft ) ;
   }


   /*------------------------------------------------------------*/
   /* getLinkGroupDraft
    */
   /**
    *    This method will try and read a externalLink draft
    *  from the database. If a draft does not exist,
    *  a new draft will be created
    *
    *  @param parent The parent AppDefaultWin that is loading the draft. This value can be null
    *  @param externalLinkInd The active externalLink indicator
    *  @param typeWorkInd The work indicator as defined in com.ibm.nzna.projects.common.quest.type.TypeList ( TYPEWORK_CREATE etc.. )
    *
    *  @return LinkGroupDraft. NULL if one cannot be created
    *//*---------------------------------------------------------*/
   public static LinkGroupDraft getLinkGroupDraft ( AppDefaultWin parent, int externalLinkInd, int typeWorkInd ) {
      LinkGroupDraft retDraft = null ;

      if ( parent != null )
         parent.setStatus ( STR_READING ) ;

      try {
         retDraft = LinkGroupDraft.getDraft ( externalLinkInd,
                                              PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                              (TypeWorkRec)TypeList.getInstance().objectFromInd ( typeWorkInd, TypeList.TYPE_WORKREQUIRED ) ) ;
      } catch ( PendingDraftException pde ) {
         LogSystem.log ( 1, pde, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
      }

      if ( parent != null )
         parent.setStatus ( null ) ;
      return( retDraft ) ;
   }


   /*------------------------------------------------------------*/
   /* getSymptomDraft
    */
   /**
    *    This method will try and read a symptom draft
    *  from the database. If a draft does not exist,
    *  a new draft will be created
    *
    *  @param parent The parent AppDefaultWin that is loading the draft. This value can be null
    *  @param symptomInd The active symptom indicator
    *  @param typeWorkInd The work indicator as defined in com.ibm.nzna.projects.common.quest.type.TypeList ( TYPEWORK_CREATE etc.. )
    *
    *  @return SymptomDraft. NULL if one cannot be created
    *//*---------------------------------------------------------*/
   public static SymptomDraft getSymptomDraft ( AppDefaultWin parent, int symptomInd, int typeWorkInd )  {
      SymptomDraft retDraft = null ;

      if ( parent != null )
         parent.setStatus ( STR_READING ) ;

      try {
         retDraft = SymptomDraft.getDraft( symptomInd,
                                           PropertySystem.getInt ( PropertySystem.PROPERTY_LANGUAGE ),
                                           (TypeWorkRec)TypeList.getInstance().objectFromInd ( typeWorkInd, TypeList.TYPE_WORKREQUIRED ) ) ;
      } catch ( PendingDraftException pde ) {
         LogSystem.log ( 1, pde, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_PENDING_DRAFT_OPEN ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         if ( parent != null )
            GUISystem.printBox ( parent, STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_COULD_NOT_READ_DRAFT ) ;
      }

      if ( parent != null )
         parent.setStatus ( null ) ;
      return( retDraft ) ;
   }





   /*------------------------------------------------------------*/
   /* recycleObject
    */
   /**
    *    This method will recycle the object
    *  passed in. Please keep in mind that Recycling is only for
    *  Drafts. Active Objects cannot be recycled.
    *  
    *
    *  @param objectType The Type of Object ( AVALON_QUESTION_DRAFT, AVALON_ACTION_DRAFT etc..)
    *  @param objectInd The indicator of the object
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void recycleObject ( int objectType, int objectInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "recycleObject", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         String    tableName = "" ;

         switch ( objectType ) {
            case AVALON_QUESTION_DRAFT :
               tableName = "QUESTION" ;
               break ;

            case AVALON_ACTION_DRAFT :
               tableName = "ACTION" ;
               break ;

            case AVALON_LINK_DRAFT :
               tableName = "LINK" ;
               break ;

            case AVALON_LINK_GROUP_DRAFT :
               tableName = "LINKGROUP" ;
               break ;

            case AVALON_SYMPTOM_DRAFT :
               tableName = "SYMPTOM" ;
               break ;
         } /* switch */

         if ( tableName.length() > 0 )
            stmt.executeUpdate ( "UPDATE OADRAFT."+tableName+" "+
                                 "SET RECYCLED    = 'Y', "+
                                 "    DBUSER      = '"+UserSystem.getUserId()+"', "+
                                 "    CHANGEDTIME = CURRENT TIMESTAMP "+
                                 "WHERE "+tableName+"IND = "+objectInd ) ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }





   /*------------------------------------------------------------*/
   /* getSymptomDraftIndFromActiveInd
    */
   /**
    *    This method will return the open draft indicator
    *  for the Active Indicator passed for a Symptom.
    *  If there is no open draft then a 0 is returned
    *
    *  @param activeSymptomInd  The Active Symptom Indicator
    *  @param None
    *  @param None
    *
    *  @return draftSymptomInd (Int)
    *//*---------------------------------------------------------*/
   public int getSymptomDraftIndFromActiveInd ( int activeSymptomInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getSymptomDraftIndFromActiveInd", 5 ) ;
      int       returnInd = 0 ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            ResultSet rs = stmt.executeQuery ( "SELECT SYMPTOMIND "+
                                               "FROM OADRAFT.SYMPTOM "+
                                               "WHERE ACTIVEIND = "+activeSymptomInd+" AND "+
                                               "      STOPDATE IS NULL "+
                                               "FOR FETCH ONLY " ) ;

            if ( rs.next() )
               returnInd = rs.getInt(1) ;

            rs.close() ;
         }
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;
      return( returnInd ) ;
   }



   /*------------------------------------------------------------*/
   /* cancelDraft
    */
   /**
    *    This method will set the stop date and
    *  cancel the current draft
    *  
    *
    *  @param action The ActionDraft to cancel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean cancelDraft ( ActionDraft action ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "cancelDraft", 5 ) ;
      boolean   cancelled = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.ACTION SET STOPDATE = CURRENT DATE, "+
                              "                          CHANGEDTIME = CURRENT TIMESTAMP, "+
                              "                          DBUSER      = '"+UserSystem.getUserId()+"', "+
                              "                          MAINTLOCK   = 'N' "+
                              "WHERE ACTIONIND = "+action.getInd() ) ;

         cancelled = true ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      return ( cancelled ) ;
   }


   /*------------------------------------------------------------*/
   /* cancelDraft
    */
   /**
    *    This method will set the stop date and
    *  cancel the current draft
    *  
    *
    *  @param question The QuestionDraft to cancel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean cancelDraft ( QuestionDraft question ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "cancelDraft", 5 ) ;
      boolean   cancelled = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.QUESTION SET STOPDATE = CURRENT DATE, "+
                              "                          CHANGEDTIME = CURRENT TIMESTAMP, "+
                              "                          DBUSER      = '"+UserSystem.getUserId()+"', "+
                              "                          MAINTLOCK   = 'N' "+
                              "WHERE QUESTIONIND = "+question.getInd() ) ;

         cancelled = true ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      return ( cancelled ) ;
   }


   /*------------------------------------------------------------*/
   /* cancelDraft
    */
   /**
    *    This method will set the stop date and
    *  cancel the current draft
    *  
    *
    *  @param symptom The SymptomDraft to cancel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean cancelDraft ( SymptomDraft symptom ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "cancelDraft", 5 ) ;
      boolean   cancelled = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.SYMPTOM SET STOPDATE = CURRENT DATE, "+
                              "                          CHANGEDTIME = CURRENT TIMESTAMP, "+
                              "                          DBUSER      = '"+UserSystem.getUserId()+"', "+
                              "                          MAINTLOCK   = 'N' "+
                              "WHERE SYMPTOMIND = "+symptom.getInd() ) ;

         cancelled = true ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      return ( cancelled ) ;
   }



   /*------------------------------------------------------------*/
   /* cancelDraft
    */
   /**
    *    This method will set the stop date and
    *  cancel the current draft
    *  
    *
    *  @param link The LinkDraft to cancel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean cancelDraft ( QuestLinkDraft link ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "cancelDraft", 5 ) ;
      boolean   cancelled = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.LINK SET STOPDATE = CURRENT DATE, "+
                              "                          CHANGEDTIME = CURRENT TIMESTAMP, "+
                              "                          DBUSER      = '"+UserSystem.getUserId()+"', "+
                              "                          MAINTLOCK   = 'N' "+
                              "WHERE LINKIND = "+link.getInd() ) ;

         cancelled = true ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      return ( cancelled ) ;
   }




   /*------------------------------------------------------------*/
   /* cancelDraft
    */
   /**
    *    This method will set the stop date and
    *  cancel the current draft
    *  
    *
    *  @param link The LinkDraft to cancel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean cancelDraft ( ExternalLinkDraft link ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "cancelDraft", 5 ) ;
      boolean   cancelled = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.LINK SET STOPDATE = CURRENT DATE, "+
                              "                          CHANGEDTIME = CURRENT TIMESTAMP, "+
                              "                          DBUSER      = '"+UserSystem.getUserId()+"', "+
                              "                          MAINTLOCK   = 'N' "+
                              "WHERE LINKIND = "+link.getInd() ) ;

         cancelled = true ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      return ( cancelled ) ;
   }




   /*------------------------------------------------------------*/
   /* cancelDraft
    */
   /**
    *    This method will set the stop date and
    *  cancel the current draft
    *  
    *
    *  @param linkGroup The LinkGroupDraft to cancel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean cancelDraft ( LinkGroupDraft linkGroup ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "cancelDraft", 5 ) ;
      boolean   cancelled = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "UPDATE OADRAFT.LINKGROUP SET STOPDATE = CURRENT DATE, "+
                              "                          CHANGEDTIME = CURRENT TIMESTAMP, "+
                              "                          DBUSER      = '"+UserSystem.getUserId()+"', "+
                              "                          MAINTLOCK   = 'N' "+
                              "WHERE LINKGROUPIND = "+linkGroup.getInd() ) ;

         cancelled = true ;
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      return ( cancelled ) ;
   }



} /* OASQL */
