package com.ibm.nzna.projects.qit.avalon.sql ;     
/*-----------------------------------------------------------*/
/*   QuestionUsage                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestionUsage                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.quest.oa.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import java.sql.* ;
import java.util.* ;
/**
 * This class holds static SQL methods that can return any information 
 * on the usage of a single question. For more information on Usage, please
 * see the UsageRec class.<br><br>
 *
 * Some things that are checked are:<br>
 *  1. Edge in Symptoms, including the Edge Itself, Conditions on the Edge, PreAnswers or UnAnswers on the edge.<br>
 *  2. Conditions for Titles in Actions and other Questions<br>
 *  3. Conditions for Additional Information in Actions and other Questions<br>
 *  4. Conditions on Answers in other Questions<br>
 *  5. Conditions on Links and Link Groups in Actions and other Questions<br>
 *  6. Conclusions on other Questions<br>
 *  7. Conditions on a Conclusion on other Questions<br>
 *  8. Conclusion on any Product<br>
 *
 * @version 1.0, 9/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestionUsage                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestionUsage extends Object implements AvalonConst, AppConst {





   /*------------------------------------------------------------*/
   /* getQuestionAnswerUsageInQuestions
    */
   /**
    *    This method will return all Errors and Warnings
    *  for the usage of the Question passed in. Please
    *  note that this QuestionInd MUST be the Active Question Indicator.<br><br>
    *
    *  This method will return Usages in the following areas:
    *  All Title Conditions, Add Info Conditions, Answer Conclusions, Answer Conditions
    *  and Answer Conclusion Conditions, Link Conditions
    *
    *  @param Question The Question to look with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getQuestionAnswerUsageInQuestions ( QuestionDraft question ) {
      Vector usageVec = new Vector ( 1, 1 ) ;
      Vector answers  = question.getAnswers() ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, process each answer                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( answers != null ) && ( answers.size() > 0 ) ) {
         int i    = 0 ;
         int size = answers.size() ;                                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Draft  Question Title Conditions          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInDraftQuestionTitleCond ( (Answer)answers.elementAt(i++), usageVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Active Question Title Conditions          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInActiveQuestionTitleCond ( (Answer)answers.elementAt(i++), usageVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Draft  Question Add Info Conditions       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInDraftQuestionAddInfoCond ( (Answer)answers.elementAt(i++), usageVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Active Question Add Info Conditions       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInActiveQuestionAddInfoCond ( (Answer)answers.elementAt(i++), usageVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Draft  Question Answer   Conditions       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInDraftQuestionAnswerCond ( (Answer)answers.elementAt(i++), usageVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Active Question Answer Conditions         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInActiveQuestionAnswerCond ( (Answer)answers.elementAt(i++), usageVec ) ;

      }

      return( usageVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getQuestionAnswerUsageInActions
    */
   /**
    *    This method will return all Errors and Warnings
    *  for the usage of the Question passed in.<br><br>
    *
    *  This method will return Usages in the following areas:
    *  All Title Conditions, Add Info Conditions and Link Conditions
    *
    *  @param Question The Question to look with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getQuestionAnswerUsageInActions ( QuestionDraft question ) {
      Vector usageVec = new Vector ( 1, 1 ) ;
      Vector answers  = question.getAnswers() ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, process each answer                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( answers != null ) && ( answers.size() > 0 ) ) {
         int i    = 0 ;
         int size = answers.size() ;                                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Draft  Question Title Conditions          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInDraftActionTitleCond ( (Answer)answers.elementAt(i++), usageVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Active Question Title Conditions          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInActiveActionTitleCond ( (Answer)answers.elementAt(i++), usageVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Draft Add Info Conditions                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInDraftActionAddInfoCond ( (Answer)answers.elementAt(i++), usageVec ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Active Add Info Conditions                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInActiveActionAddInfoCond ( (Answer)answers.elementAt(i++), usageVec ) ;

      }

      return( usageVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getQuestionAnswerUsageInSymptoms
    */
   /**
    *    This method will return all Errors and Warnings
    *  for the usage of the QuestionInd, AnswerInd passed in. Please
    *  note that this QuestionInd MUST be the Active Question Indicator.<br><br>
    *
    *  This method will return Usages in the following areas:
    *  1. Edge in Symptoms, including the Edge Itself, Conditions on the Edge, PreAnswers or UnAnswers on the edge.<br>
    *
    *  @param Question The Question to look with
    *  @param answerInd The AnswerInd to look with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getQuestionAnswerUsageInSymptoms ( QuestionDraft question ) {
      Vector usageVec = new Vector ( 1, 1 ) ;
      Vector answers  = question.getAnswers() ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, process each answer                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( answers != null ) && ( answers.size() > 0 ) ) {
         int i    = 0 ;
         int size = answers.size() ;                                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Find all Draft  Edges holding these Answers             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size )
            findAnswerInDraftEdge ( (Answer)answers.elementAt(i++), usageVec ) ;                                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Active Edges containg these answers       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInActiveEdge ( (Answer)answers.elementAt(i++), usageVec ) ;                                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Find all Draft Edge Conditions holding these answers    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size )
            findAnswerInDraftEdgeCond ( (Answer)answers.elementAt(i++), usageVec ) ;                                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Active Edges Conditions using answers     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInActiveEdgeCond ( (Answer)answers.elementAt(i++), usageVec ) ;                                


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Find all Draft Edge Preanswers holding these answers    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size )
            findAnswerInDraftEdgePreanswer ( (Answer)answers.elementAt(i++), usageVec ) ;                                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, find all Active Edges Preanswers using answers     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         i=0 ;
         while ( i < size )
            findAnswerInActiveEdgePreanswer ( (Answer)answers.elementAt(i++), usageVec ) ;

      }

      return( usageVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getQuestionUsageInSymptoms
    */
   /**
    *    This method will return the symptoms that contain
    *  this question in their flow.
    *  
    *
    *  @param question The question to look for
    *  @param None
    *  @param None
    *
    *  @return Vector of UsageRecs
    *//*---------------------------------------------------------*/
   public static Vector getQuestionUsageInSymptoms ( QuestionDraft question ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getQuestionUsageInSymptoms", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt   = sqlMethod.createStatement ()  ;
         ResultSet rs     = null ;
         UsageRec  newRec = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, read in all of the occurances in the Drafts      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, A.RECYCLED, A.CHANGEDTIME, A.DBUSER "+
                                  "FROM OADRAFT.SYMPTOM      A, "+
                                  "     OADRAFT.SYMPTOMTITLE B, "+
                                  "     OADRAFT.NODE         C  "+
                                  "WHERE C.ACTIVEQUESTIONIND = "+question.getActiveQuestionInd()+" AND "+
                                  "      C.DRAFTSYMPTOMIND  = A.SYMPTOMIND AND "+
                                  "      B.SYMPTOMIND       = A.SYMPTOMIND AND "+
                                  "      B.SORTORDER        = (SELECT MAX (SORTORDER) FROM OADRAFT.SYMPTOMTITLE WHERE SYMPTOMIND = A.SYMPTOMIND) "+
                                  "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim()) ;
            newRec.setWarning ( true ) ;
            newRec.setObjectType ( AVALON_SYMPTOM ) ;

            if ( rs.getString(3).trim().equals ( "Y" ) )
               newRec.setExplanation ( Str.getStr ( STR_QUESTION_USED_IN_DRAFT_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'"+
                                       Str.getStr ( STR_QUESTION_USED_IN_RECYCLED_SYMPTOM )+" "+
                                       UserSystem.getNameFromUserId ( rs.getString(5).trim() )+" "+
                                       Str.getStr ( STR_ON )+" "+
                                       DateSystem.prettyDateFromStamp ( rs.getString ( 4 ) ) ) ;
            else
               newRec.setExplanation ( Str.getStr ( STR_QUESTION_USED_IN_DRAFT_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'" ) ;

            returnVec.addElement ( newRec ) ;

         }

         rs.close() ;                        


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now check the Active Symptoms                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, A.OPENDRAFT "+
                                  "FROM OA.SYMPTOM      A, "+
                                  "     OA.SYMPTOMTITLE B, "+
                                  "     OA.NODE         C  "+
                                  "WHERE C.QUESTIONIND     = "+question.getActiveQuestionInd()+" AND "+
                                  "      C.SYMPTOMIND      = A.SYMPTOMIND AND "+
                                  "      B.SYMPTOMIND      = A.SYMPTOMIND AND "+
                                  "      B.SORTORDER       = (SELECT MAX (SORTORDER) FROM OA.SYMPTOMTITLE WHERE SYMPTOMIND = A.SYMPTOMIND) "+
                                  "FOR FETCH ONLY" ) ;         

         while ( rs.next() ) {
            if ( rs.getString ( 3 ).trim().equals ( "N" ) ) {
               newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               newRec.setObjectType ( AVALON_SYMPTOM ) ;
               newRec.setWarning ( true ) ;
               newRec.setExplanation ( Str.getStr ( STR_QUESTION_USED_IN_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'" ) ;
            } else {
               newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               newRec.setObjectType ( AVALON_SYMPTOM ) ;
               newRec.setWarning ( false ) ;
               newRec.setExplanation ( Str.getStr ( STR_QUESTION_USED_IN_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'" ) ;

            }
            returnVec.addElement ( newRec ) ;

         }
         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }


      sqlMethod.close() ;
      return( returnVec ) ;

   }



   /*------------------------------------------------------------*/
   /* getQuestionProductUsage
    */
   /**
    *    This method will return all usages of the Question
    *  on a Product Conclusion.
    *  
    *
    *  @param question The Question to look for
    *  @param None
    *  @param None
    *
    *  @return Vector of UsageRecs. Each record should be considered an error
    *//*---------------------------------------------------------*/
   public static Vector getQuestionProductUsage ( Question question ) {
      Vector usageVec = new Vector ( 1, 1 ) ;
      Vector answers  = question.getAnswers() ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, process each answer                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( answers != null ) && ( answers.size() > 0 ) ) {
         int i    = 0 ;
         int size = answers.size() ;                                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Find all Usages                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size )
            getAnswerProductUsage ( (Answer)answers.elementAt(i++), usageVec ) ;                                

      }

      return( usageVec ) ;
   }




   /*------------------------------------------------------------*/
   /* findAnswerInDraftEdge
    */
   /**
    *    This method will find the answer passed
    *  in any draft edge (OADRAFT.EDGE). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftEdge ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftEdge", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE "+
                                                  "FROM OADRAFT.EDGEGROUP A, "+
                                                  "     OADRAFT.SYMPTOMTITLE B, "+
                                                  "     OADRAFT.EDGE C, "+
                                                  "     OADRAFT.SYMPTOM E "+
                                                  "WHERE (C.DRAFTANSWERIND = "+answer.getInd()+" or C.ACTIVEANSWERIND = "+answer.getActiveAnswerInd()+") AND "+
                                                  "      C.EDGEGROUPIND = A.EDGEGROUPIND AND "+
                                                  "      A.SYMPTOMIND   = B.SYMPTOMIND AND "+
                                                  "      A.SYMPTOMIND   = E.SYMPTOMIND AND "+
                                                  "      E.STOPDATE     IS NULL AND "+
                                                  "      B.SORTORDER IN (SELECT MAX(D.SORTORDER) FROM OADRAFT.SYMPTOMTITLE D WHERE D.SYMPTOMIND = A.SYMPTOMIND) "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_SYMPTOM_DRAFT ) ;
               usage.setAnswerInd ( answer.getInd() ) ;
               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_AN_EDGE_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'." ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }



   /*------------------------------------------------------------*/
   /* findAnswerInActiveEdge
    */
   /**
    *    This method will find the answer passed
    *  in any active edge (OA.EDGE). If
    *  there is an occurance, a UsageRec marked as
    *  an error (if there is no open draft) will be put in the usageVec passed.
    *
    *  @param answer The Answer object to look with
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveEdge ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveEdge", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, E.OPENDRAFT "+
                                                  "FROM OA.EDGEGROUP A, "+
                                                  "     OA.SYMPTOMTITLE B, "+
                                                  "     OA.EDGE C, "+
                                                  "     OA.SYMPTOM E "+
                                                  "WHERE C.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                                  "      C.EDGEGROUPIND = A.EDGEGROUPIND AND "+
                                                  "      A.SYMPTOMIND   = B.SYMPTOMIND AND "+
                                                  "      A.SYMPTOMIND   = E.SYMPTOMIND AND "+
                                                  "      B.SORTORDER IN (SELECT MAX(D.SORTORDER) FROM OA.SYMPTOMTITLE D WHERE D.SYMPTOMIND = A.SYMPTOMIND) "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setObjectType ( AVALON_SYMPTOM ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_AN_EDGE_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'."+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_AN_EDGE_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'."+
                                         Str.getStr ( STR_OPEN_DRAFT_FOR_WARNING ) ) ;

               usageVec.addElement ( usage ) ;

            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }



   /*------------------------------------------------------------*/
   /* findAnswerInDraftQuestionTitleCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Question's Title's Conditions. (OADRAFT.QUESTIONTITLECOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftQuestionTitleCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftEdge", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.TITLE "+
                                                  "FROM OADRAFT.QUESTIONTITLE A, "+
                                                  "     OADRAFT.QUESTIONTITLECOND B, "+
                                                  "     OADRAFT.QUESTION C "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+ 
                                                  "      B.QUESTIONTITLEIND = A.QUESTIONTITLEIND AND "+
                                                  "      C.QUESTIONIND      = A.QUESTIONIND AND "+
                                                  "      C.STOPDATE         IS NULL "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_QUESTION_DRAFT ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;
               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_QUESTION_TITLE_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'." ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }

   /*------------------------------------------------------------*/
   /* findAnswerInActiveQuestionTitleCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Question's Title's Conditions. (OA.QUESTIONTITLECOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveQuestionTitleCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveQuestionTitleCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.TITLE, C.OPENDRAFT "+
                                                  "FROM OA.QUESTIONTITLE A, "+
                                                  "     OA.QUESTIONTITLECOND B, "+
                                                  "     OA.QUESTION C "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+ 
                                                  "      B.QUESTIONTITLEIND = A.QUESTIONTITLEIND AND "+
                                                  "      C.QUESTIONIND      = A.QUESTIONIND "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;
               usage.setObjectType ( AVALON_QUESTION ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_QUESTION_TITLE_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. "+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_QUESTION_TITLE_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. " ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* findAnswerInDraftEdgeCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft edge's condition (OADRAFT.EDGECOND). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftEdgeCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftEdgeCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE "+
                                                  "FROM OADRAFT.EDGEGROUP A, "+
                                                  "     OADRAFT.SYMPTOMTITLE B, "+
                                                  "     OADRAFT.EDGECOND C, "+
                                                  "     OADRAFT.EDGE E, "+
                                                  "     OADRAFT.SYMPTOM F "+
                                                  "WHERE (C.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                                  "      E.EDGEIND      = C.EDGEIND AND "+
                                                  "      E.EDGEGROUPIND = A.EDGEGROUPIND AND "+
                                                  "      A.SYMPTOMIND   = B.SYMPTOMIND AND "+
                                                  "      B.SORTORDER IN (SELECT MAX(D.SORTORDER) FROM OADRAFT.SYMPTOMTITLE D WHERE D.SYMPTOMIND = A.SYMPTOMIND) AND "+
                                                  "      F.SYMPTOMIND   = A.SYMPTOMIND AND "+
                                                  "      F.STOPDATE IS NULL "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_SYMPTOM_DRAFT ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_AN_EDGE_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'." ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* findAnswerInActiveEdgeCond
    */
   /**
    *    This method will find the answer passed
    *  in any active edge's condition (OA.EDGECOND). If
    *  there is an occurance, a UsageRec marked as
    *  an error (if there is no open draft) will be put in the usageVec passed.
    *
    *  @param answer The Answer object to look with
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveEdgeCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveEdgeCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, E.OPENDRAFT "+
                                                  "FROM OA.EDGEGROUP A, "+
                                                  "     OA.SYMPTOMTITLE B, "+
                                                  "     OA.EDGECOND C, "+
                                                  "     OA.SYMPTOM E, "+
                                                  "     OA.EDGE F "+
                                                  "WHERE C.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                                  "      F.EDGEIND      = C.EDGEIND AND "+
                                                  "      F.EDGEGROUPIND = A.EDGEGROUPIND AND "+
                                                  "      A.SYMPTOMIND   = B.SYMPTOMIND AND "+
                                                  "      A.SYMPTOMIND   = E.SYMPTOMIND AND "+
                                                  "      B.SORTORDER IN (SELECT MAX(D.SORTORDER) FROM OA.SYMPTOMTITLE D WHERE D.SYMPTOMIND = A.SYMPTOMIND) "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setObjectType ( AVALON_SYMPTOM ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_AN_EDGE_CONDITION_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'."+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_AN_EDGE_CONDITION_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'."+
                                         Str.getStr ( STR_OPEN_DRAFT_FOR_WARNING ) ) ;

               usageVec.addElement ( usage ) ;

            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }



   /*------------------------------------------------------------*/
   /* findAnswerInDraftEdgePreanswer
    */
   /**
    *    This method will find the answer passed
    *  in any draft edge preanswer (OADRAFT.EDGEPREANSWER). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftEdgePreanswer ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftEdgePreanswer", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE "+
                                                  "FROM OADRAFT.EDGEGROUP A, "+
                                                  "     OADRAFT.SYMPTOMTITLE B, "+
                                                  "     OADRAFT.EDGEPREANSWER C, "+
                                                  "     OADRAFT.EDGE E, "+
                                                  "     OADRAFT.SYMPTOM F "+
                                                  "WHERE (C.DRAFTANSWERIND = "+answer.getInd()+" or C.ACTIVEANSWERIND = "+answer.getActiveAnswerInd()+") AND "+
                                                  "      E.EDGEIND      = C.EDGEIND AND "+
                                                  "      E.EDGEGROUPIND = A.EDGEGROUPIND AND "+
                                                  "      A.SYMPTOMIND   = B.SYMPTOMIND AND "+
                                                  "      B.SORTORDER IN (SELECT MAX(D.SORTORDER) FROM OADRAFT.SYMPTOMTITLE D WHERE D.SYMPTOMIND = A.SYMPTOMIND) "+
                                                  "      F.SYMPTOMIND   = A.SYMPTOMIND AND "+
                                                  "      F.STOPDATE  IS NULL "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_SYMPTOM_DRAFT ) ;
               usage.setAnswerInd ( answer.getInd() ) ;

               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_AN_EDGE_PREANSWER_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'." ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* findAnswerInActiveEdgePreanswer
    */
   /**
    *    This method will find the answer passed
    *  in any active edge preanswer (OA.EDGEPREANSWER). If
    *  there is an occurance, a UsageRec marked as
    *  an error (if there is no open draft) will be put in the usageVec passed.
    *
    *  @param answer The Answer object to look with
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveEdgePreanswer ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveEdgePreanswer", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, E.OPENDRAFT "+
                                                  "FROM OA.EDGEGROUP A, "+
                                                  "     OA.SYMPTOMTITLE B, "+
                                                  "     OA.EDGEPREANSWER C, "+
                                                  "     OA.SYMPTOM E, "+
                                                  "     OA.EDGE F "+
                                                  "WHERE C.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                                  "      F.EDGEIND      = C.EDGEIND AND "+
                                                  "      A.EDGEGROUPIND = F.EDGEGROUPIND AND "+
                                                  "      A.SYMPTOMIND   = B.SYMPTOMIND AND "+
                                                  "      A.SYMPTOMIND   = E.SYMPTOMIND AND "+
                                                  "      B.SORTORDER IN (SELECT MAX(D.SORTORDER) FROM OA.SYMPTOMTITLE D WHERE D.SYMPTOMIND = A.SYMPTOMIND) "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setObjectType ( AVALON_SYMPTOM ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_AN_EDGE_PREANSWER_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'."+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_AN_EDGE_PREANSWER_IN_SYMPTOM )+" '"+rs.getString(2).trim()+"'."+
                                         Str.getStr ( STR_OPEN_DRAFT_FOR_WARNING ) ) ;

               usageVec.addElement ( usage ) ;

            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }





   /*------------------------------------------------------------*/
   /* getAnswerProductUsage
    */
   /**
    *    This method will return each instance of this
    *  Question in a Product's Conclusion. (OA.PRODUCTCONCL)
    *  
    *
    *  @param usageVec The vector that will be populated with UsageRecs
    *  @param question The Question to look for. Please note that this draft 
    *                  MUST have an Active Question
    *
    *  @return Vector of UsageRecs
    *//*---------------------------------------------------------*/
   private static void getAnswerProductUsage ( Answer answer, Vector usageVec ) {
      SQLMethod   sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "QuestionUsage.getQuestionProductUsage", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs = stmt.executeQuery ( "SELECT A.PRODINT, A.MACHINE, A.MODEL, A.PRODDESC "+
                                               "FROM TIGRIS.PRODUCTS A, "+
                                               "     OA.PRODUCTCONCL B "+
                                               "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                               "      B.PRODUCTIND = A.PRODINT "+
                                               "      FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1),
                                      rs.getString(2).trim()+"-"+rs.getString(3).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_PRODUCT ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_A_CONCLUSION_ON_PRODUCT )+" "+
                                      rs.getString(2).trim()+"-"+rs.getString(3).trim()+" ('"+rs.getString(4).trim()+"')." ) ;

               usageVec.addElement ( usage ) ;
            }
         }

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }




   /*------------------------------------------------------------*/
   /* findAnswerInDraftActionTitleCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Action's Title's Conditions. (OADRAFT.ACTIONTITLECOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftActionTitleCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftEdge", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.ACTIONIND, A.TITLE "+
                                                  "FROM OADRAFT.ACTIONTITLE A, "+
                                                  "     OADRAFT.ACTIONTITLECOND B, "+
                                                  "     OADRAFT.ACTION C "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+ 
                                                  "      B.ACTIONTITLEIND = A.ACTIONTITLEIND AND "+
                                                  "      C.ACTIONIND      = A.ACTIONIND AND "+
                                                  "      C.STOPDATE         IS NULL "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_ACTION_DRAFT ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;
               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_ACTION_TITLE_COND_ON_ACTION )+" '"+rs.getString(2).trim()+"'." ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }

   /*------------------------------------------------------------*/
   /* findAnswerInActiveActionTitleCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Action's Title's Conditions. (OA.ACTIONTITLECOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveActionTitleCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveActionTitleCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.ACTIONIND, A.TITLE, C.OPENDRAFT "+
                                                  "FROM OA.ACTIONTITLE A, "+
                                                  "     OA.ACTIONTITLECOND B, "+
                                                  "     OA.ACTION C "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+ 
                                                  "      B.ACTIONTITLEIND = A.ACTIONTITLEIND AND "+
                                                  "      C.ACTIONIND      = A.ACTIONIND "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setObjectType ( AVALON_ACTION ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_ACTION_TITLE_COND_ON_ACTION )+" '"+rs.getString(2).trim()+"'. "+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_ACTION_TITLE_COND_ON_ACTION )+" '"+rs.getString(2).trim()+"'. " ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }






   /*------------------------------------------------------------*/
   /* findAnswerInDraftActionTitleCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Action's Add Info's Conditions. (OADRAFT.ACTIONINFOCOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftActionAddInfoCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftActionAddInfoCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.ACTIONIND, A.TITLE "+
                                                  "FROM OADRAFT.ACTIONTITLE A, "+
                                                  "     OADRAFT.ACTIONINFOCOND B, "+
                                                  "     OADRAFT.ACTIONADDINFO D, "+
                                                  "     OADRAFT.ACTION C "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                                  "      B.ACTIONADDINFOIND = D.ACTIONADDINFOIND AND "+
                                                  "      D.ACTIONIND        = C.ACTIONIND AND "+
                                                  "      C.ACTIONIND        = A.ACTIONIND AND "+
                                                  "      A.SORTORDER        = (SELECT MAX(Z.SORTORDER) FROM OADRAFT.ACTIONTITLE Z WHERE Z.ACTIONIND = C.ACTIONIND) AND "+
                                                  "      C.STOPDATE         IS NULL "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_ACTION_DRAFT ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;
               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_ACTION_ADDINFO_COND_ON_ACTION )+" '"+rs.getString(2).trim()+"'." ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }

   /*------------------------------------------------------------*/
   /* findAnswerInActiveActionAddInfoCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Action's Title's Conditions. (OA.ACTIONTITLECOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveActionAddInfoCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveActionAddInfoCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.ACTIONIND, A.TITLE, C.OPENDRAFT "+
                                                  "FROM OA.ACTIONTITLE A, "+
                                                  "     OA.ACTIONINFOCOND B, "+
                                                  "     OA.ACTION C, "+
                                                  "     OA.ACTIONADDINFO D "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+ 
                                                  "      B.ACTIONADDINFOIND = D.ACTIONADDINFOIND AND "+
                                                  "      D.ACTIONIND        = C.ACTIONIND AND "+
                                                  "      A.SORTORDER        = (SELECT MAX(Z.SORTORDER) FROM OA.ACTIONTITLE Z WHERE Z.ACTIONIND = D.ACTIONIND) AND "+
                                                  "      C.ACTIONIND        = A.ACTIONIND "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setObjectType ( AVALON_ACTION ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_ACTION_ADDINFO_COND_ON_ACTION )+" '"+rs.getString(2).trim()+"'. "+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_ACTION_ADDINFO_COND_ON_ACTION )+" '"+rs.getString(2).trim()+"'. " ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }







   /*------------------------------------------------------------*/
   /* findAnswerInDraftQuestionTitleCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Question's Add Info's Conditions. (OADRAFT.QUESTIONINFOCOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftQuestionAddInfoCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftQuestionAddInfoCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.TITLE "+
                                                  "FROM OADRAFT.QUESTIONTITLE A, "+
                                                  "     OADRAFT.QUESTIONINFOCOND B, "+
                                                  "     OADRAFT.QUESTIONADDINFO D, "+
                                                  "     OADRAFT.QUESTION C "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                                  "      B.QUESTIONADDINFOIND = D.QUESTIONADDINFOIND AND "+
                                                  "      D.QUESTIONIND        = C.QUESTIONIND AND "+
                                                  "      C.QUESTIONIND        = A.QUESTIONIND AND "+
                                                  "      A.SORTORDER        = (SELECT MAX(Z.SORTORDER) FROM OADRAFT.QUESTIONTITLE Z WHERE Z.QUESTIONIND = C.QUESTIONIND) AND "+
                                                  "      C.STOPDATE         IS NULL "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_QUESTION_DRAFT ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;
               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_QUESTION_ADDINFO_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'." ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }

   /*------------------------------------------------------------*/
   /* findAnswerInActiveQuestionAddInfoCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Question's Title's Conditions. (OA.QUESTIONTITLECOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveQuestionAddInfoCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveQuestionAddInfoCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.TITLE, C.OPENDRAFT "+
                                                  "FROM OA.QUESTIONTITLE A, "+
                                                  "     OA.QUESTIONINFOCOND B, "+
                                                  "     OA.QUESTION C, "+
                                                  "     OA.QUESTIONADDINFO D "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+ 
                                                  "      B.QUESTIONADDINFOIND = D.QUESTIONADDINFOIND AND "+
                                                  "      D.QUESTIONIND        = C.QUESTIONIND AND "+
                                                  "      A.SORTORDER        = (SELECT MAX(Z.SORTORDER) FROM OA.QUESTIONTITLE Z WHERE Z.QUESTIONIND = D.QUESTIONIND) AND "+
                                                  "      C.QUESTIONIND        = A.QUESTIONIND "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setObjectType ( AVALON_QUESTION ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_QUESTION_ADDINFO_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. "+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_QUESTION_ADDINFO_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. " ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }




   /*------------------------------------------------------------*/
   /* findAnswerInDraftQuestionAnswerCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Question's Answer's Conditions. (OADRAFT.ANSWERCOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftQuestionAnswerCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftQuestionAnswerCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.TITLE "+
                                                  "FROM OADRAFT.QUESTIONTITLE A, "+
                                                  "     OADRAFT.ANSWERCOND B, "+
                                                  "     OADRAFT.ANSWER D, "+
                                                  "     OADRAFT.QUESTION C "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                                  "      B.PARENTANSWERIND    = D.ANSWERIND AND "+
                                                  "      D.QUESTIONIND        = C.QUESTIONIND AND "+
                                                  "      C.QUESTIONIND        = A.QUESTIONIND AND "+
                                                  "      A.SORTORDER          = (SELECT MAX(Z.SORTORDER) FROM OADRAFT.QUESTIONTITLE Z WHERE Z.QUESTIONIND = C.QUESTIONIND) AND "+
                                                  "      C.STOPDATE          IS NULL "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_QUESTION_DRAFT ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;
               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_QUESTION_ANSWER_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'." ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }

   /*------------------------------------------------------------*/
   /* findAnswerInActiveQuestionAnswerCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Question's Answers Conditions. (OA.ANSWERCOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveQuestionAnswerCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveQuestionAnswerCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.TITLE, C.OPENDRAFT "+
                                                  "FROM OA.QUESTIONTITLE A, "+
                                                  "     OA.ANSWERCOND B, "+
                                                  "     OA.QUESTION C, "+
                                                  "     OA.ANSWER D "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+ 
                                                  "      B.PARENTANSWERIND  = D.ANSWERIND AND "+
                                                  "      D.QUESTIONIND      = C.QUESTIONIND AND "+
                                                  "      A.SORTORDER        = (SELECT MAX(Z.SORTORDER) FROM OA.QUESTIONTITLE Z WHERE Z.QUESTIONIND = D.QUESTIONIND) AND "+
                                                  "      C.QUESTIONIND      = A.QUESTIONIND "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setObjectType ( AVALON_QUESTION ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_QUESTION_ANSWER_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. "+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_QUESTION_ANSWER_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. " ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }


   /*------------------------------------------------------------*/
   /* findAnswerInDraftQuestionAnswerTitleCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Question's Answer's Title's Conditions. (OADRAFT.ANSWERCOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInDraftQuestionAnswerTitleCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInDraftQuestionAnswerTitleCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.TITLE, E.TITLE "+
                                                  "FROM OADRAFT.QUESTIONTITLE A, "+
                                                  "     OADRAFT.ANSWERTITLECOND B, "+
                                                  "     OADRAFT.QUESTION C, "+
                                                  "     OADRAFT.ANSWER D, "+
                                                  "     OADRAFT.ANSWERTITLE E "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+
                                                  "      B.PARENTANSWERIND    = D.ANSWERIND AND "+
                                                  "      D.QUESTIONIND        = C.QUESTIONIND AND "+
                                                  "      C.QUESTIONIND        = A.QUESTIONIND AND "+
                                                  "      E.ANSWERTITLEIND     = B.ANSWERTITLEIND AND "+
                                                  "      A.SORTORDER          = (SELECT MAX(Z.SORTORDER) FROM OADRAFT.QUESTIONTITLE Z WHERE Z.QUESTIONIND = C.QUESTIONIND) AND "+
                                                  "      C.STOPDATE           IS NULL "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( false ) ;
               usage.setObjectType ( AVALON_QUESTION ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;
               usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                      Str.getStr ( STR_IS_ON_QUESTION_ANSWER_TITLE_COND_ON_ANSWER )+" '"+rs.getString(3).trim()+"'. "+
                                      Str.getStr ( STR_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. " ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }

   /*------------------------------------------------------------*/
   /* findAnswerInActiveQuestionAnswerTitleCond
    */
   /**
    *    This method will find the answer passed
    *  in any draft Question's Answer's Title's Conditions. (OA.ANSWERTITLECOND ). If
    *  there is an occurance, a UsageRec marked as
    *  an error will be put in the usageVec passed.
    *
    *  @param answerInd The Answer to look for
    *  @param usageVec The vector to populate
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void findAnswerInActiveQuestionAnswerTitleCond ( Answer answer, Vector usageVec ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "findAnswerInActiveQuestionAnswerTitleCond", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         if ( stmt != null ) {
            UsageRec  usage = null ;
            ResultSet rs    = stmt.executeQuery ( "SELECT A.QUESTIONIND, A.TITLE, C.OPENDRAFT, E.TITLE "+
                                                  "FROM OA.QUESTIONTITLE A, "+
                                                  "     OA.ANSWERTITLECOND B, "+
                                                  "     OA.QUESTION C, "+
                                                  "     OA.ANSWER D, "+
                                                  "     OA.ANSWERTITLE E "+
                                                  "WHERE B.ANSWERIND = "+answer.getActiveAnswerInd()+" AND "+ 
                                                  "      B.PARENTANSWERIND    = D.ANSWERIND AND "+
                                                  "      D.QUESTIONIND        = C.QUESTIONIND AND "+
                                                  "      A.SORTORDER          = (SELECT MAX(Z.SORTORDER) FROM OA.QUESTIONTITLE Z WHERE Z.QUESTIONIND = D.QUESTIONIND) AND "+
                                                  "      C.QUESTIONIND        = A.QUESTIONIND "+
                                                  "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               usage = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               usage.setWarning ( rs.getString(3).trim().equals ( "Y" ) ) ;
               usage.setObjectType ( AVALON_QUESTION ) ;
               usage.setAnswerInd ( answer.getActiveAnswerInd() ) ;

               if ( usage.getWarning() )
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_QUESTION_ANSWER_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. "+
                                         Str.getStr ( STR_DRAFT_OPEN_WARNING ) ) ;
               else
                  usage.setExplanation ( Str.getStr ( STR_THE_ANSWER )+" '"+answer.toString()+"' "+
                                         Str.getStr ( STR_IS_ON_QUESTION_ANSWER_COND_ON_QUESTION )+" '"+rs.getString(2).trim()+"'. " ) ;

               usageVec.addElement ( usage ) ;
            }

            rs.close() ;

         } /* if stmt  */
      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;
   }



} /* QuestionUsage */
