package com.ibm.nzna.projects.qit.avalon.sql ;    
/*-----------------------------------------------------------*/
/*   SymptomUsage                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymptomUsage                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import com.ibm.nzna.projects.common.quest.oa.* ;
/**
 * This class holds methods usefull in finding out where a symptom is used 
 *
 *
 * @version 1.0, 11/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymptomUsage                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymptomUsage extends Object implements AppConst, AvalonConst {



   /*------------------------------------------------------------*/
   /* getSubroutineSymptoms
    */
   /**
    *    This method will go rhrough all of the nodes
    *  in the symptom and returns a UsageRec for
    *  each Subroutine symptom in the flow
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector getSubroutineSymptoms ( SymptomDraft symptom ) {
      Vector retVec            = new Vector ( ) ;
      Vector nodes             = symptom.getNodes() ;
      String activeWhereClause = "" ;
      String draftWhereClause  = "" ;

      if ( ( nodes != null ) && ( nodes.size() > 0 ) ) {
         int  i    = 0 ;
         int  size = nodes.size() ;
         Node node = null ;                                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through, creating the where clauses to read        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            node = (Node)nodes.elementAt(i) ;
            if ( node.getNodeType() == Node.NODE_TYPE_SYMPTOM ) {
               if ( node.getDraftObjectInd() == 0 )
                  activeWhereClause+=""+node.getActiveObjectInd()+", " ;
               else
                  draftWhereClause+=""+node.getDraftObjectInd()+", " ;
            }
            i++ ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, read the Draft Subroutine Symptoms               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( draftWhereClause.length() > 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getSubroutineSymptoms (Draft)", 5 ) ;

         try {
            Statement stmt   = sqlMethod.createStatement() ;
            ResultSet rs     = null ;
            UsageRec  newRec = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Take out the last comma                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            draftWhereClause = draftWhereClause.substring ( 0, draftWhereClause.length()-2 ) ;    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Read the data                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, A.RECYCLED "+
                                     "FROM OADRAFT.SYMPTOM A, "+
                                     "     OADRAFT.SYMPTOMTITLE B "+
                                     "WHERE A.SYMPTOMIND IN ( "+draftWhereClause+" ) AND "+
                                     "      A.SYMPTOMIND = B.SYMPTOMIND AND "+
                                     "      B.SORTORDER  IN ( SELECT MAX (SORTORDER) FROM OADRAFT.SYMPTOMTITLE WHERE SYMPTOMIND = A.SYMPTOMIND ) "+
                                     "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               newRec = new UsageRec ( rs.getInt ( 1 ), rs.getString(2).trim() ) ;
               newRec.setWarning ( true ) ;
               newRec.setObjectType ( AVALON_SYMPTOM ) ;
               retVec.addElement ( newRec ) ;
            }

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
         }

         sqlMethod.close() ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now read in the active Subroutine Symptoms          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( activeWhereClause.length() > 0 ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getSubroutineSymptoms (Active)", 5 ) ;

         try {
            Statement stmt   = sqlMethod.createStatement() ;
            ResultSet rs     = null ;
            UsageRec  newRec = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Take out the last comma                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            activeWhereClause = activeWhereClause.substring ( 0, activeWhereClause.length()-2 ) ;    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Read the data                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, A.OPENDRAFT "+
                                     "FROM OA.SYMPTOM A, "+
                                     "     OA.SYMPTOMTITLE B "+
                                     "WHERE A.SYMPTOMIND IN ( "+activeWhereClause+" ) AND "+
                                     "      A.SYMPTOMIND = B.SYMPTOMIND AND "+
                                     "      B.SORTORDER  IN ( SELECT MAX (SORTORDER) FROM OA.SYMPTOMTITLE WHERE SYMPTOMIND = A.SYMPTOMIND ) "+
                                     "FOR FETCH ONLY" ) ;

            while ( rs.next() ) {
               if ( rs.getString(3).trim().equals ( "N" ) ) {
                  newRec = new UsageRec ( rs.getInt ( 1 ), rs.getString(2).trim() ) ;
                  newRec.setWarning ( true ) ;
                  newRec.setObjectType ( AVALON_SYMPTOM ) ;
                  retVec.addElement ( newRec ) ;
               }
            }

         } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
         }

         sqlMethod.close() ;
      }

      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getSymptomUsage
    */
   /**
    *    This method will return all symptoms that the
    *  symptom passed is referenced in.
    *  
    *
    *  @param symptom The symptom to look for
    *  @param None
    *  @param None
    *
    *  @return Vector of UsageRecs
    *//*---------------------------------------------------------*/
   public static Vector getSymptomUsage ( SymptomDraft symptom ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getSymptomUsage", 5 ) ;
      Vector    returnVec = new Vector ( 1 ) ;

      try {
         Statement stmt   = sqlMethod.createStatement ()  ;
         ResultSet rs     = null ;
         UsageRec  newRec = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, read in all of the occurances in the Drafts      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, A.RECYCLED, A.CHANGEDTIME, A.DBUSER "+
                                  "FROM OADRAFT.SYMPTOM      A, "+
                                  "     OADRAFT.SYMPTOMTITLE B, "+
                                  "     OADRAFT.NODE         C  "+
                                  "WHERE C.ACTIVESYMPTOMIND = "+symptom.getActiveSymptomInd()+" AND "+
                                  "      C.DRAFTSYMPTOMIND  = A.SYMPTOMIND AND "+
                                  "      B.SYMPTOMIND       = A.SYMPTOMIND AND "+
                                  "      B.SORTORDER        = (SELECT MAX (SORTORDER) FROM OADRAFT.SYMPTOMTITLE WHERE SYMPTOMIND = A.SYMPTOMIND) "+
                                  "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim()) ;
            newRec.setWarning ( true ) ;
            newRec.setObjectType ( AVALON_SYMPTOM ) ;

            if ( rs.getString(3).trim().equals ( "Y" ) )
               newRec.setExplanation ( Str.getStr ( STR_SYMPTOM_USED_IN_DRAFT_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'"+
                                       Str.getStr ( STR_SYMPTOM_USED_IN_RECYCLED_SYMPTOM )+" "+
                                       UserSystem.getNameFromUserId ( rs.getString(5).trim() )+" "+
                                       Str.getStr ( STR_ON )+" "+
                                       DateSystem.prettyDateFromStamp ( rs.getString ( 4 ) ) ) ;
            else
               newRec.setExplanation ( Str.getStr ( STR_SYMPTOM_USED_IN_DRAFT_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'" ) ;

            returnVec.addElement ( newRec ) ;

         }

         rs.close() ;                        


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now check the Active Symptoms                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( "SELECT A.SYMPTOMIND, B.TITLE, A.OPENDRAFT "+
                                  "FROM OA.SYMPTOM      A, "+
                                  "     OA.SYMPTOMTITLE B, "+
                                  "     OA.NODE         C  "+
                                  "WHERE C.CHILDSYMPTOMIND = "+symptom.getActiveSymptomInd()+" AND "+
                                  "      C.SYMPTOMIND      = A.SYMPTOMIND AND "+
                                  "      B.SYMPTOMIND      = A.SYMPTOMIND AND "+
                                  "      B.SORTORDER       = (SELECT MAX (SORTORDER) FROM OA.SYMPTOMTITLE WHERE SYMPTOMIND = A.SYMPTOMIND) "+
                                  "FOR FETCH ONLY" ) ;         

         while ( rs.next() ) {
            if ( rs.getString ( 3 ).trim().equals ( "N" ) ) {
               newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               newRec.setObjectType ( AVALON_SYMPTOM ) ;
               newRec.setWarning ( true ) ;
               newRec.setExplanation ( Str.getStr ( STR_SYMPTOM_USED_IN_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'" ) ;
            } else {
               newRec = new UsageRec ( rs.getInt(1), rs.getString(2).trim() ) ;
               newRec.setObjectType ( AVALON_SYMPTOM ) ;
               newRec.setWarning ( false ) ;
               newRec.setExplanation ( Str.getStr ( STR_SYMPTOM_USED_IN_SYMPTOM )+" '"+
                                       rs.getString(2).trim()+"'" ) ;

            }
            returnVec.addElement ( newRec ) ;

         }
         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }


      sqlMethod.close() ;
      return( returnVec ) ;

   }
} /* SymptomUsage */
