package com.ibm.nzna.projects.qit.avalon.sql ;
/*-----------------------------------------------------------*/
/*   UsageRec                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: UsageRec                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
/**
 * The UsageRec object is used to hold information about 
 * a single usage of an Object against another Object in Avalon.<br><br>
 *
 * A Usage is a Question being used in another Object's Conditions or a Question
 * being used in a Conclusion, Symptom, Preanswer or UnAnswer.<br><br>
 *
 * A Usage can be a warning or an error. An error means that you cannot touch this
 * particular Question because it is currently being used in an Active object. A
 * Warning means that this Question is being used in an Active Object but their
 * is an open draft of the Active Object so the Draft may be published to delete
 * the usage<br><br>
 *
 * A Usage can also be a Symptom being used inside of another Symptom, or an Action
 * being used inside of a Symptom, or a Link being used inside of a Question/Action.
 *
 * @version 1.0, 9/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   UsageRec                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class UsageRec extends Object implements AppConst, MultiListRow {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * The referenceObjectInd is the Object to which this usage is referring to
     */
   private int     referenceObjectInd      = 0 ;

   /**
     * The referenceObjectTitle is the human readable title to the object this
     * Usage is referring to
     */
   private String  referenceObjectTitle    = "" ;

   /**
     * If this Usage represents a Warning, this value will be TRUE
     */
   private boolean warning                 = false ;

   /**
    *  The draft flag. TRUE = The object being referenced is a Draft
    **/
   private boolean draft                  = false ;

   /**
     * This explanation is a human readble field explaining why this usage is present
     */
   private String explanation             = null ;

   /**
     * Object Type. This is one of the object types as defined in AvalonConst (QUESTION, ACTION etc..)
     */
   private int objectType                 = 0 ;


   /**
     * The Answer Indicator is used whenever the usage indicates an Answer
     **/
   private int answerInd                  = 0 ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor and it will
    *  initialize all variables to what is passed
    *  
    *
    *  @param refObjectInd The referring Object indicator
    *  @param refObjectTitle The title of the referring object
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public UsageRec ( int referenceObjectInd, String referenceObjectTitle ) {
      setReferenceObjectInd   ( referenceObjectInd ) ;
      setReferenceObjectTitle ( referenceObjectTitle ) ;
   }





   /*------------------------------------------------------------*/
   /* setReferenceObjectTitle
    */
   /**
    *    This method will set the referring object's
    *  title. This is the title of the object that is being
    *  'used'.
    *  
    *
    *  @param title The new title
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setReferenceObjectTitle ( String title ) {
      this.referenceObjectTitle = title ;
   }




   /*------------------------------------------------------------*/
   /* setReferenceObjectInd
    */
   /**
    *    This method will set the indicator of the
    *  object that is being 'used'
    *  
    *
    *  @param ind The indicator to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setReferenceObjectInd ( int ind ) {
      this.referenceObjectInd = ind ;
   }





   /*------------------------------------------------------------*/
   /* getReferenceObjectTitle
    */
   /**
    *    This method will return the title of the
    *  object that is being 'used'
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getReferenceObjectTitle () {
      return( referenceObjectTitle ) ;
   }





   /*------------------------------------------------------------*/
   /* getRefenceObjectInd
    */
   /**
    *    This method will return the indicator of the
    *  object being used.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getReferenceObjectInd () {
      return( referenceObjectInd ) ;
   }






   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the data needed to show
    *  in a specific column in a MultiList
    *  
    *
    *  @param column The column to return data for
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObject = null ;

      switch ( column ) {
         case 0 :
            if ( warning )
               retObject = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.SMALL_WARNING ) ;
            else
               retObject = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.SMALL_ERROR ) ;
            break ;

         case 1 :
            retObject = ""+referenceObjectInd ;
            break ;

         case 2 :
            retObject = referenceObjectTitle ;
            break ;

         default:
            retObject = "" ;
            break ;
      }

      return( retObject ) ;
   }





   /*------------------------------------------------------------*/
   /* setDraft
    */
   /**
    *    This method will turn the draft property on
    *  or off.
    *  
    *
    *  @param draft TRUE = The object being referenced is a Draft, FALSE if not
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDraft ( boolean draft ) {
      this.draft = draft ;
   }




   /*------------------------------------------------------------*/
   /* getDraft
    */
   /**
    *    This method will return the draft property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return  TRUE = The object being referenced is a Draft, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean getDraft () {
      return( draft ) ;
   }


   /*------------------------------------------------------------*/
   /* setWarning
    */
   /**
    *    This method will turn the warning property on
    *  or off.
    *  
    *
    *  @param warning TRUE = This usage is a Warning, FALSE = This usage is an error
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWarning ( boolean warning ) {
      this.warning = warning ;
   }




   /*------------------------------------------------------------*/
   /* getWarning
    */
   /**
    *    This method will return the warning property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return  TRUE = This usage is a Warning, FALSE = This usage is an error
    *//*---------------------------------------------------------*/
   public boolean getWarning () {
      return( warning ) ;
   }

   /*------------------------------------------------------------*/
   /* setExplanation
    */
   /**
    *    This method will set the explanation property
    *  
    *
    *  @param explanation A human readable explanation of this usage
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setExplanation ( String explanation ) {
      this.explanation = explanation ;
   }




   /*------------------------------------------------------------*/
   /* getExplanation
    */
   /**
    *    This method will return the explanation property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return  A human readable explanation of this usage
    *//*---------------------------------------------------------*/
   public String getExplanation () {
      return( explanation ) ;
   }



   /*------------------------------------------------------------*/
   /* setObjectType
    */
   /**
    *    This method will set the objectType property
    *  
    *
    *  @param objectType The type of object this is referring to (Question, Action) as defined in AvalonConst
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setObjectType ( int objectType ) {
      this.objectType = objectType ;
   }




   /*------------------------------------------------------------*/
   /* getObjectType
    */
   /**
    *    This method will return the objectType property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return  What type of object this is referring (AVALON_QUESTION, AVALON_ACTION Etc.. )
    *//*---------------------------------------------------------*/
   public int getObjectType () {
      return( objectType ) ;
   }

   /*------------------------------------------------------------*/
   /* setAnswerInd
    */
   /**
    *    This method will set the answerInd property
    *  
    *
    *  @param answerInd The answer that is being used
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAnswerInd ( int answerInd ) {
      this.answerInd = answerInd ;
   }




   /*------------------------------------------------------------*/
   /* getAnswerInd
    */
   /**
    *    This method will return the answerInd property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return  The Answer Indicator
    *//*---------------------------------------------------------*/
   public int getAnswerInd () {
      return( answerInd ) ;
   }


} /* UsageRec */
