package com.ibm.nzna.projects.qit.avalon.storedProc.actReader ;   
/*-----------------------------------------------------------*/
/*   ActReader                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActReader                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read a list of actions for the OA Navigator.
 * The first parameter passed 'query' is a query clause to be run. The Query MUST select
 * the following values in the following order:
 * 
 * <br><pre>
 * 1. ACTIONIND
 * 2. DOCCLASSIND
 * 3. TITLE
 * 4. CHANGEDTIME
 * 5. DBUSER
 * 6. SORTORDER (From OA.ACTIONTITLE)
 * <pre><br><br><br>
 *
 * The query must also SORT on ACTIONIND and SORTORDER.<br><br>
 *
 * The second parameter specifies the schema to read under. 
 * 
 * Main Execution Point: logonUser
 *
 * Param Number   Descript
 * -------------- -------------------------------------------
 * 1              query       - The Query to run
 * 2              vector      - A Vector of SymptomDisplayRecs
 * 3              StoredProcRec ( Allocated and returned by ActReader )
 *
 * 
 *
 * @version 1.0, 2/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActReader                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec = new StoredProcRec () ;
   private Vector        returnVec = null ;
   private Connection    con       = null ;
   private String        query     = null ;





   /*------------------------------------------------------------*/
   /* logonUser
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readList ( Clob query, Blob returnVector, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our return vector                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVec = new Vector ( 1, 10 ) ;
         this.query  = StoredProcUtil.getStringFromClob ( query ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Log the User in                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         readData() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our Vector and return rec                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVector = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( returnVector, returnVec );
         set ( 2, returnVector ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 3, storedProcRetRec ) ;
   }





   /*------------------------------------------------------------*/
   /* readData
    */
   /**
    *    This method will read all of our data in
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readData () throws SQLException {
      Statement         stmt          = con.createStatement() ;
      ResultSet         rs            = stmt.executeQuery ( query ) ;
      ActionDisplayRec  parentRec     = null ;
      ActionDisplayRec  newRec        = null ;
      int               lastSortOrder = -1 ;     
      int               count         = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in all of the data                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( rs.next() ) && ( count < 1500 ) ) {

         newRec  = new ActionDisplayRec ( rs.getInt( 1 ) ,              // Action Ind
                                          rs.getInt( 2 ) ,              // Doc Class Ind
                                          rs.getString ( 3 ).trim() ,   // Title
                                          rs.getString ( 4 ).trim() ,   // Changed Time
                                          rs.getString ( 5 ).trim() ) ; // dbUser


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have not assigned out parent record yet,          */
         /* assign it to the first Action read in. If we have read  */
         /* in our parent record and it's indicator is different      */
         /* than the 'newRec' we just read in, assign our new parent  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( parentRec == null ) || ( parentRec.getActionInd() != newRec.getActionInd() ) ) {
            parentRec = newRec ;
            returnVec.addElement ( parentRec ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, check to see if this newRec is a child       */
         /* record of the parent rec. If the indicators are the same  */
         /* then the newRec is a child and should not be added to     */
         /* the list of records, but as a child to the parentRec      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this new 'orderby' value is different than the one   */
            /* before, process it, otherwise throw it away               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getInt(6) != lastSortOrder ) {

               lastSortOrder = rs.getInt(6) ;
               parentRec.addChild ( newRec ) ;
            }
         }
         count++ ;
      } /* while  */


      rs.close() ;
      stmt.close() ;

   }

} /* ActReader */
