package com.ibm.nzna.projects.qit.avalon.storedProc.ansReader ;   
/*-----------------------------------------------------------*/
/*   AnsReader                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AnsReader                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read a list of answers.
 * The first parameter passed 'query' is a query clause to be run. The Query MUST return
 * the following values in the following order:
 * 
 * <br><pre>
 * 1. ANSWERIND
 * 2. TITLE
 * 3. CHANGEDTIME
 * 4. DBUSER
 * <pre><br>
 *
 * The second parameter specifies the schema to read under. 
 * 
 * Main Execution Point: logonUser
 *
 * Param Number   Descript
 * -------------- -------------------------------------------
 * 1              query       - The Query to run
 * 2              vector      - A Vector of SymptomDisplayRecs
 * 3              StoredProcRec ( Allocated and returned by AnsReader )
 *
 * 
 *
 * @version 1.0, 2/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AnsReader                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class AnsReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec = new StoredProcRec () ;
   private Vector        returnVec = null ;
   private Connection    con       = null ;
   private String        query     = null ;





   /*------------------------------------------------------------*/
   /* logonUser
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readList ( Clob query, Blob returnVector, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our return vector                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVec = new Vector ( 1, 10 ) ;
         this.query  = StoredProcUtil.getStringFromClob ( query ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Log the User in                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         readData() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our Vector and return rec                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVector = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( returnVector, returnVec );
         set ( 2, returnVector ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 3, storedProcRetRec ) ;
   }





   /*------------------------------------------------------------*/
   /* readData
    */
   /**
    *    This method will read all of our data in
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readData () throws SQLException {
      Statement         stmt    = con.createStatement() ;
      ResultSet         rs      = stmt.executeQuery ( query ) ;
      AnswerDisplayRec  readRec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in all of the data                                 */              
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {
         readRec = new AnswerDisplayRec ( rs.getInt( 1 ) ,              // Ansion Ind
                                          rs.getString ( 2 ).trim() ,   // Title
                                          rs.getString ( 4 ).trim() ,   // Changed Time
                                          rs.getString ( 3 ).trim() ) ; // dbUser
         returnVec.addElement ( readRec ) ;
      }
      

      rs.close() ;
      stmt.close() ;

   }

} /* AnsReader */
