package com.ibm.nzna.projects.qit.avalon.storedProc.lnkGrpRdr ;   
/*-----------------------------------------------------------*/
/*   LnkGrpRdr                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LnkGrpRdr                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read a list of link groups for the OA Navigator.
 * The first parameter passed 'query' is the actual query to perform. The Query MUST
 * have these value in the SELECT portion, in this order:<br><br><pre>
 *
 * 1  LINKGROUPIND (OA.LINKGROUP)       <br>
 * 2  TITLE        (OA.LINKGROUPTITLE)  <br>
 * 3  DBUSER       (OA.LINKGROUP)       <br>
 * 4  CHANGEDTIME  (OA.LINKGROUP)       <br><br></pre>
 *
 * Main Execution Point: readList
 *
 * <pre>
 * Param Number   Descript<br>
 * -------------- -------------------------------------------<br>
 * 1              query<br>
 * 2              vector      - A Vector of SymptomDisplayRecs<br>
 * 3              StoredProcRec ( Allocated and returned by LnkGrpRdr )<br>
 *<br></pre>
 * 
 *
 * @version 1.0, 2/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LnkGrpRdr                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class LnkGrpRdr extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec = new StoredProcRec () ;
   private Vector        returnVec = null ;
   private Connection    con       = null ;
   private String        query     = null ;





   /*------------------------------------------------------------*/
   /* logonUser
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readList ( Clob query, Blob returnVector, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our return vector                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVec = new Vector ( 1, 10 ) ;
         this.query  = StoredProcUtil.getStringFromClob ( query ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read the list                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         readData() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our Vector and return rec                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVector = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( returnVector, returnVec );
         set ( 2, returnVector ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 3, storedProcRetRec ) ;
   }





   /*------------------------------------------------------------*/
   /* readData
    */
   /**
    *    This method will read all of our data in
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readData () throws SQLException {
      Statement            stmt     = con.createStatement() ;
      ResultSet            rs       = stmt.executeQuery ( query ) ;
      LinkGroupDisplayRec  readRec  = null ; 
      int                  count    = 0 ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in all of the data                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( rs.next() ) && ( count < 1500 ) ) {
         readRec = new LinkGroupDisplayRec ( rs.getInt(1),                                     // Link Indicator
                                             rs.getString(2).trim(),                           // Title
                                             rs.getString(3).trim(),                           // Database User
                                             rs.getString(4).trim() ) ;                        // Changedtime

         returnVec.addElement ( readRec ) ;
         count++ ;
      } /* while  */


      rs.close() ;
      stmt.close() ;

   }


} /* LnkGrpRdr */


