package com.ibm.nzna.projects.qit.avalon.storedProc.qstReader ;   
/*-----------------------------------------------------------*/
/*   QstDraftReader                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QstDraftReader                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read a list of draft questions for the OA Navigator.
 * The first parameter passed 'query' is a query clause to be run. The Query MUST SELECT
 * the following values in the following order:
 * 
 * <br><pre>
 * 1. QUESTIONIND<br>
 * 2. DOCCLASSIND<br>
 * 3. TITLE<br>
 * 4. DBUSER<br>
 * 5. CHANGEDTIME<br>
 * 6. STARTDATE<br>
 * 7. STOPDATE<br>
 * 8. ACTIVEQUESTIONIND<br>
 * 9. SORTORDER (From the title)<br>
 * <pre><br>
 * <br><br>The Query also MUST order by both QUESTIONIND and SORTORDER
 *
 * The second parameter specifies the schema to read under. 
 * 
 * Main Execution Point: logonUser
 *
 * Param Number   Descript
 * -------------- -------------------------------------------
 * 1              query       - The Query to run
 * 2              vector      - A Vector of SymptomDisplayRecs
 * 3              StoredProcRec ( Allocated and returned by QstDraftReader )
 *
 * 
 *
 * @version 1.0, 2/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QstDraftReader                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class QstDraftReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec = new StoredProcRec () ;
   private Vector        returnVec = null ;
   private Connection    con       = null ;
   private String        schema    = null ;
   private String        query     = null ;





   /*------------------------------------------------------------*/
   /* logonUser
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readList ( Clob query, Blob returnVector, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our return vector                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVec = new Vector ( 1, 10 ) ;
         this.query  = StoredProcUtil.getStringFromClob ( query ) ;
         this.schema = schema ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read the list                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         readData() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our Vector and return rec                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVector = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( returnVector, returnVec );
         set ( 2, returnVector ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 3, storedProcRetRec ) ;
   }





   /*------------------------------------------------------------*/
   /* readData
    */
   /**
    *    This method will read all of our data in
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readData () throws SQLException {
      Statement               stmt          = con.createStatement() ;
      ResultSet               rs            = stmt.executeQuery ( query ) ;
      QuestionDraftDisplayRec newRec        = null ;              
      QuestionDraftDisplayRec parentRec     = null ;              
      int                     lastSortOrder = -1 ;
      int                     count         = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in all of the data                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( rs.next() ) && ( count < 1500 ) ) {

         newRec  = new QuestionDraftDisplayRec ( rs.getInt( 1 ) ,            // Question Ind
                                                 rs.getInt( 8 ) ,            // Active QUestion Indicator
                                                 rs.getInt( 2 ) ,            // Doc Class Ind
                                                 rs.getString ( 3 ).trim() , // Title
                                                 rs.getString ( 5 ).trim() , // Changed Time
                                                 rs.getString ( 4 ).trim() , // DB User
                                                 rs.getString ( 6 ).trim() , // Start Date
                                                 rs.getString ( 7 ) ) ;      // Stop Date

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have not assigned out parent record yet,          */
         /* assign it to the first Question read in. If we have read  */
         /* in our parent record and it's indicator is different      */
         /* than the 'newRec' we just read in, assign our new parent  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( parentRec == null ) || ( parentRec.getQuestionInd() != newRec.getQuestionInd() ) ) {
            parentRec = newRec ;
            returnVec.addElement ( parentRec ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, check to see if this newRec is a child       */
         /* record of the parent rec. If the indicators are the same  */
         /* then the newRec is a child and should not be added to     */
         /* the list of records, but as a child to the parentRec      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this new 'orderby' value is different than the one   */
            /* before, process it, otherwise throw it away               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getInt(9) != lastSortOrder ) {

               lastSortOrder = rs.getInt(9) ;
               parentRec.addChild ( newRec ) ;
            }

         }

         count++ ;
      } /* while */


      rs.close() ;
      stmt.close() ;

   }

} /* QstDraftReader */
