package com.ibm.nzna.projects.qit.avalon.storedProc.symReader ;   
/*-----------------------------------------------------------*/
/*   SymDraftReader                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SymDraftReader                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.qit.avalon.base.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.* ;
import java.sql.* ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read a list of draft symptoms for the OA Navigator.
 * The first parameter passed 'query' is a query clause to be run. The Query MUST select
 * the following values in the following order:
 * 
 * <br><pre>
 * 1.  SYMPTOMIND<br>
 * 2.  DOCCLASSIND<br>
 * 3.  STATUSIND<br>
 * 4.  TITLE<br>
 * 5.  DBUSER<br>
 * 6.  CHANGEDTIME<br>
 * 7.  STARTDATE<br>
 * 8.  STOPDATE<br>
 * 9.  ACTIVESYMPTOMIND<br>
 * 10. SORTORDER (From OADRAFT.SYMPTOMTITLE)<br>
 * <pre><br><br><br>
 *
 * Please note that every query must order by SYMPTOMIND AND SORTORDER
 *
 * Main Execution Point: logonUser <br><pre>
 *
 * Param Number   Descript<br>
 * -------------- -------------------------------------------<br>
 * 1              query       - The Query to run<br>
 * 2              vector      - A Vector of SymptomDisplayRecs<br>
 * 3              StoredProcRec ( Allocated and returned by SymDraftReader )<br>
 *<br></pre>
 * 
 *
 * @version 1.0, 2/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SymDraftReader                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class SymDraftReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec = new StoredProcRec () ;
   private Vector        returnVec = null ;
   private Connection    con       = null ;
   private String        query     = null ;





   /*------------------------------------------------------------*/
   /* logonUser
    */
   /**
    *    This method will log on the user. This method
    *  will be called by DB/2 when this stored
    *  procedure is invoked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readList ( Clob query, Blob returnVector, Blob storedProcRetRec ) throws Exception  {                            

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our connection to DB/2                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our return vector                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVec = new Vector ( 1, 10 ) ;
         this.query  = StoredProcUtil.getStringFromClob ( query ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read the data                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         readData() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our Vector and return rec                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         returnVector = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( returnVector, returnVec );
         set ( 2, returnVector ) ;


      } catch ( Exception e ) {
         StoredProcUtil.writeException ( e, returnRec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Commit                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con.commit() ;
      con.close() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out our return information                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      storedProcRetRec = Lob.newBlob() ;
      StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;

      set ( 3, storedProcRetRec ) ;
   }





   /*------------------------------------------------------------*/
   /* readData
    */
   /**
    *    This method will read all of our data in
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readData () throws SQLException {
      Statement              stmt          = con.createStatement() ;
      ResultSet              rs            = stmt.executeQuery ( query ) ;
      SymptomDraftDisplayRec newRec        = null ;              
      SymptomDraftDisplayRec parentRec     = null ;              
      int                    lastSortOrder = -1 ;
      int                    count         = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in all of the data                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {

         newRec = new SymptomDraftDisplayRec ( rs.getInt( 1 ) ,             // Symptom Indicator
                                               rs.getInt( 9 ) ,             // Active Symptom Indicator
                                               rs.getInt( 3 ) ,             // Status Ind
                                               rs.getInt( 2 ) ,             // Doc Class Indicator
                                               rs.getString ( 4 ).trim() ,  // Title
                                               rs.getString ( 6 ).trim() ,  // Changed Time
                                               rs.getString ( 5 ).trim(),   // DB User
                                               rs.getString ( 7 ).trim(),   // Start Date
                                               rs.getString ( 8 )        ); // Stop Date


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have not assigned out parent record yet,          */
         /* assign it to the first Symptom read in. If we have read  */
         /* in our parent record and it's indicator is different      */
         /* than the 'newRec' we just read in, assign our new parent  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( parentRec == null ) || ( parentRec.getSymptomInd() != newRec.getSymptomInd() ) ) {
            parentRec = newRec ;
            returnVec.addElement ( parentRec ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, check to see if this newRec is a child       */
         /* record of the parent rec. If the indicators are the same  */
         /* then the newRec is a child and should not be added to     */
         /* the list of records, but as a child to the parentRec      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this new 'orderby' value is different than the one   */
            /* before, process it, otherwise throw it away               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( rs.getInt(10) != lastSortOrder ) {

               lastSortOrder = rs.getInt(10) ;
               parentRec.addChild ( newRec ) ;
            }
         } /* if  */

         count++ ;
      } /* while  */


      rs.close() ;
      stmt.close() ;

   }

} /* SymDraftReader */

