package com.ibm.nzna.projects.qit.batchrunner ;        
/*-----------------------------------------------------------*/
/*   BatchRunnerPanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BatchRunnerPanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.io.* ;
import java.util.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;   
import com.ibm.nzna.shared.batch.* ;   
/**
 * This panel shows the output from a batch that is run. 
 *
 *
 * @version 1.0, 4/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BatchRunnerPanel                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class BatchRunnerPanel extends JPanel implements BatchListener, AppConst, QuestPanel {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private GUI Controls                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private JTextArea    mle_PROGRESS      = null ;
    private JTextArea    mle_ERROR         = null ;
    private JTitle       st_PROGRESS       = null ;
    private JTitle       st_ERROR          = null ;
    private JProgressBar progressBar       = null ;
    private JScrollPane  scr_PROGRESS      = null ;
    private JScrollPane  scr_ERROR         = null ;

    private ActionButton pb_COMMIT         = new ActionButton("Commit",ImageSystem.getImageIcon(this, ImageSystem.ACTION),
                                                              "Commit");
    private ActionButton pb_RERUN          = new ActionButton("Rerun",ImageSystem.getImageIcon(this, ImageSystem.ACTION_SMALL),
                                                              "Rerun");



    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private Batch batch     = null ;



    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Variables                                               */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private boolean inThread         = false ;

    private boolean showCommit       = false ;
    private boolean showRerun        = false ;


    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will initialize the BatchRunner
     *  with a Batch
     *  
     *
     *  @param batch The default batch to run
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public BatchRunnerPanel ( Batch batch ) {
        this.batch = batch ;
    }





    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This constructor does nothing
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public BatchRunnerPanel () {

    }


    /*------------------------------------------------------------*/
    /* getIcon
     */
    /**
     *    This method will return the icon that is associated
     *  to this panel
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public ImageIcon getIcon () {
        return( ImageSystem.getImageIcon ( this, ImageSystem.SAVE ) ) ;
    }




    /*------------------------------------------------------------*/
    /* getDefaultNavPanel
     */
    /**
     *    This method will return null since we really
     *  don't have one
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public NavPanel getDefaultNavPanel () {
        return( null ) ;
    }





    /*------------------------------------------------------------*/
    /* initialize
     */
    /**
     *    This method will initialize our panel
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return TRUE
     *//*---------------------------------------------------------*/
    public boolean initialize () {          
        JPanel progressPanel = new JPanel () ;
        JPanel errorPanel    = new JPanel () ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create our controls                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        mle_ERROR         = new JTextArea () ;
        mle_PROGRESS      = new JTextArea () ;
        scr_ERROR         = new JScrollPane ( mle_ERROR ) ;
        scr_PROGRESS      = new JScrollPane ( mle_PROGRESS ) ;
        st_ERROR          = new JTitle ( Str.getStr ( STR_ERRORS ) ) ;
        st_PROGRESS       = new JTitle ( Str.getStr ( STR_PROGRESS ) ) ;        


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust properties                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setBackground ( Color.white ) ;
        mle_ERROR.setEditable ( false ) ;
        mle_PROGRESS.setEditable ( false ) ;
        st_ERROR.setOpaque ( false ) ;
        st_PROGRESS.setOpaque ( false ) ;
        scr_ERROR.setOpaque ( false ) ;
        progressPanel.setOpaque ( false ) ;
        errorPanel.setOpaque ( false ) ;
        scr_PROGRESS.setOpaque ( false ) ;
        mle_ERROR.setOpaque ( false ) ;
        mle_PROGRESS.setOpaque ( false ) ;
        scr_PROGRESS.setBorder ( null ) ;
        scr_ERROR.setBorder ( null ) ;       

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our controls                                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setLayout ( new GridLayout ( 2, 1 ) ) ;
        progressPanel.setLayout ( new BorderLayout () ) ;
        errorPanel.setLayout ( new BorderLayout () ) ;
        progressPanel.add ( st_PROGRESS, BorderLayout.NORTH ) ;
        progressPanel.add ( scr_PROGRESS, BorderLayout.CENTER ) ;
        errorPanel.add ( st_ERROR, BorderLayout.NORTH ) ;
        errorPanel.add ( scr_ERROR, BorderLayout.CENTER ) ;
        add ( progressPanel ) ;
        add ( errorPanel ) ;


        return( true ) ;
    }





    /*------------------------------------------------------------*/
    /* panelDeselected
     */
    /**
     *  This method will be called when a panel is deselected and not
     * showing, but it is still alive on the window.
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void panelDeselected () {
    }




    /*------------------------------------------------------------*/
    /* panelSelected
     */
    /**
     *    This method will be called when the user
     *  has selected this panel to show from
     *  the focus bar
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void panelSelected () {
    }




    /*------------------------------------------------------------*/
    /* pasteText
     */
    /**
     *    This method will do nothing
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean pasteText () {
        return( false ) ;
    }


    /*------------------------------------------------------------*/
    /* copyText
     */
    /**
     *    This method will do nothing
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean copyText () {
        return( false ) ;
    }



    /*------------------------------------------------------------*/
    /* cutText
     */
    /**
     *    This method will do nothing
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean cutText () {
        return( false ) ;
    }





    /*------------------------------------------------------------*/
    /* shutdown
     */
    /**
     *    This method will return TRUE
     *  unless we are running a batch
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean shutdown () {
        return( !inThread ) ;
    }




    /*------------------------------------------------------------*/
    /* stop
     */
    /**
     *    This method will return TRUE unless we
     *  are running a batch
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean stop () {
        return( !inThread ) ;
    }




    /*------------------------------------------------------------*/
    /* refresh
     */
    /**
     *    This method does nothing
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void refresh () {
    }





    /*------------------------------------------------------------*/
    /* getJComponent
     */
    /**
     *    This method will return an instance of this
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public JComponent getJComponent () {
        return( this ) ;
    }




    /*------------------------------------------------------------*/
    /* start
     */
    /**
     *    This method will be called once we are
     *  showing
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void start () {
        if ( batch != null ) {
            AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ; 
            parentWin.addActionComponent( this, pb_COMMIT );
            parentWin.addActionComponent( this, pb_RERUN ); 
            pb_COMMIT.setVisible( showCommit );
            pb_COMMIT.setEnabled( false );
            pb_RERUN.setVisible( showRerun ) ;
            pb_RERUN.setEnabled( false );
            runBatch() ;
        }
    }





    /*------------------------------------------------------------*/
    /* getName
     */
    /**
     *    This method will return the name of this panel
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String getName () {
        return( Str.getStr ( STR_BATCH_RUNNER ) );
    }





    /*------------------------------------------------------------*/
    /* batchStart
     */
    /**
     *    This method will be called when the batch
     *  has been started
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void batchStart () {
        GUISystem.getParentDefWin ( this ).setStatus ( Str.getStr ( STR_RUNNING_BATCH ) ) ;
        inThread = true ;
    }




    /*------------------------------------------------------------*/
    /* batchStop
     */
    /**
     *    This method will be called when the
     *  batch has been stopped
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void batchStop () {
        GUISystem.getParentDefWin ( this ).setStatus ( null ) ;
        inThread = false ;
    }




    /*------------------------------------------------------------*/
    /* logProgress
     */
    /**
     *    This method will be called when we have
     *  progress to log
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void logProgress ( String string, int totalSteps, int step ) {
        mle_PROGRESS.setText ( mle_PROGRESS.getText()+string+"\n" ) ;
    }




    /*------------------------------------------------------------*/
    /* logError
     */
    /**
     *    This method will log an error
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void logError ( String string ) {
        mle_ERROR.setText ( mle_ERROR.getText()+string+"\n" ) ;
    }




    /*------------------------------------------------------------*/
    /* logSevereError
     */
    /**
     *    This method will log a 'severe' error
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void logSevereError ( String string ) {
        mle_ERROR.setText ( mle_ERROR.getText()+"\n\nSEVERE ERROR:\n"+string+"\n" ) ;
    }




    /*------------------------------------------------------------*/
    /* logSevereError
     */
    /**
     *    This method will log a severe error
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void logSevereError ( Exception e ) {
        StringWriter    sw      = new StringWriter () ;
        PrintWriter     pw      = new PrintWriter ( sw ) ;
        StringTokenizer strtok  = null ;
        String          tempStr = "" ;

        /* Get the stackTrace */
        e.printStackTrace ( pw ) ;
        mle_ERROR.setText ( mle_ERROR.getText()+"\n\nEXCEPTION:\n------------------------------------\n"+pw.toString()+"\n\n\n" ) ;

    }


    /*------------------------------------------------------------*/
    /* runBatch
     */
    /**
     *    This method will run the batch that was
     *  set with the 'setBatch' method
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void runBatch() {
        if ( batch != null ) {
            batch.addBatchListener ( this ) ;
            batch.startBatch() ;
        }
    }




    /*------------------------------------------------------------*/
    /* setBatch
     */
    /**
     *    This method will set the batch that will
     *  be run
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setBatch ( Batch batch ) {
        this.batch = batch ;
    }

    public boolean bookmark () {
        return( false ) ;
    }

    /*------------------------------------------------------------*/
    /* getCommitButton
     */
    /**
     *    This method will return the pb_COMMIT ActionButton     
     *    
     *  @param None
     *
     *  @return pb_COMMIT
     *//*---------------------------------------------------------*/

    public ActionButton getCommitButton(){
        return(pb_COMMIT);
    }

    /*------------------------------------------------------------*/
    /* showCommitBtn
     */
    /**
     *   This method sets the showCommit property.       
     *    
     *  @param showCommit TRUE= show button FALSE = Hide button
     *
     *  @return none
     *//*---------------------------------------------------------*/

    public void showCommitBtn(boolean showCommit) {
        this.showCommit = showCommit;
    }

    /*------------------------------------------------------------*/
    /* getRerunButton
     */
    /**
     *    This method will return the pb_RERUN ActionButton     
     *    
     *  @param None
     *
     *  @return pb_RERUN
     *//*---------------------------------------------------------*/

    public ActionButton getRerunButton() {
        return(pb_RERUN);
    }

    /*------------------------------------------------------------*/
   /* showRerunBtn
    */
   /**
    *   This method sets the showRerun property.       
    *    
    *  @param showRerun TRUE= show button FALSE = Hide button
    *
    *  @return none
    *//*---------------------------------------------------------*/

    public void showRerunBtn(boolean showRerun) {
        this.showRerun = showRerun;
    }
  

} /* BatchRunnerPanel */
