package com.ibm.nzna.projects.qit.bookmark ;       
/*-----------------------------------------------------------*/
/*   AddBookmarkDlg                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AddBookmarkDlg                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog will add a bookmark to the BookmarkSystem. It asks 
 * the user for the destination folder, and for the bookmark name.<br><br>
 *
 * This dialog has a special feature property 'multiple' which when passed TRUE
 * to the Constructor will make it so that the 'getFolderUsed' returns the folder
 * that was used to store the bookmark. This makes it so that routines adding multiple
 * bookmarks only have to pop up this dialog once. Then, they can use the 'getFolderUsed'
 * method to add the rest of the bookmarks. <b>PLEASE NOTE</b> that if the user does not
 * check the 'Use folder for the rest of the selected items' or if the check box is not
 * showing then the 'getFolderUsed' method returns FALSE.<br><br>
 *
 * Please note that you must call the 'dispose()' method when you are done with each instance of
 * this dialog!
 *
 * @version 1.0, 5/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AddBookmarkDlg                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class AddBookmarkDlg extends JDialog implements ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_IMAGE                   = null ;
   private JLabel       st_DESCRIPT                = null ;
   private JTextField   ef_DESCRIPT                = null ;
   private JLabel       st_FOLDER                  = null ;
   private DTree        cnr_TREE                   = null ;
   private ButtonPanel  buttonPanel                = null ;
   private JScrollPane  scrollPane                 = null ;
   private DButton      pb_OK                      = null ;
   private DButton      pb_CREATEFOLDER            = null ;
   private DButton      pb_CANCEL                  = null ;      
   private JCheckBox    ck_USE_FOLDER_FOR_MULTIPLE = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean        move               = false ;
   private Bookmark       bookmark           = null ;
   private ImageIcon      wizardImage        = null ;
   private StringTreeNode initialSelectNode  = null ;
   private String         folderUsed         = null ;
   private boolean        multiple           = false ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the window.
    *  It will not return until the window closes
    *  
    *
    *  @param parent The parent frame for this dialog
    *  @param bookmark The bookmark to add
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AddBookmarkDlg ( Frame parent, Bookmark bookmark ) {
      super ( parent, Str.getStr ( STR_ADD_BOOKMARK ), true ) ; 
      this.bookmark = bookmark ;
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the window.
    *  It will not return until the window closes
    *  
    *
    *  @param parent The parent frame for this dialog
    *  @param bookmark The bookmark to add/move
    *  @param move TRUE = Move the bookmark, FALSE = add the bookmark
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AddBookmarkDlg ( Frame parent, Bookmark bookmark, boolean move ) {
      super ( parent, Str.getStr ( STR_ADD_BOOKMARK ), true ) ; 
      this.bookmark = bookmark ;
      this.move     = move ;
      initialize() ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the window.
    *  It will not return until the window closes
    *  
    *
    *  @param parent The parent frame for this dialog
    *  @param bookmark The bookmark to add/move
    *  @param move TRUE = Move the bookmark, FALSE = add the bookmark
    *  @param multiple TRUE = Show the check box allowing the user to choose
    *                  all further moves during this operation to be done on
    *                  the same folder.
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AddBookmarkDlg ( Frame parent, Bookmark bookmark, boolean move, boolean multiple ) {
      super ( parent, Str.getStr ( STR_ADD_BOOKMARK ), true ) ; 
      this.bookmark = bookmark ;
      this.move     = move ;
      this.multiple = multiple ;
      initialize() ;
   }



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize and show our window
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardImage                = ImageSystem.getImageIcon ( this , ImageSystem.BOOKMARK_WIZARD ) ;
      st_IMAGE                   = new JLabel ( wizardImage ) ;
      st_DESCRIPT                = new JLabel ( Str.getStr ( STR_DESCRIPTION ) ) ;
      st_FOLDER                  = new JLabel ( Str.getStr ( STR_FOLDER ) ) ;
      ef_DESCRIPT                = new JTextField ( bookmark.getDescript() ) ;
      cnr_TREE                   = new DTree() ;
      scrollPane                 = new JScrollPane ( cnr_TREE ) ;
      buttonPanel                = new ButtonPanel () ;
      pb_OK                      = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL                  = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      pb_CREATEFOLDER            = new DButton ( Str.getStr ( STR_CREATE_FOLDER ) ) ;    
      ck_USE_FOLDER_FOR_MULTIPLE = new JCheckBox ( Str.getStr ( STR_USE_FOLDER_FOR_MULTIPLE ) );


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener           ( this ) ;
      pb_CANCEL.addActionListener       ( this ) ;
      pb_CREATEFOLDER.addActionListener ( this ) ;  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add The GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      buttonPanel.add ( pb_CREATEFOLDER ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_DESCRIPT ) ;
      contentPane.add ( ef_DESCRIPT ) ;
      contentPane.add ( st_FOLDER ) ;
      contentPane.add ( scrollPane ) ;
      contentPane.add ( ck_USE_FOLDER_FOR_MULTIPLE ) ;
      contentPane.add ( buttonPanel ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Final Properties                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPropertiesOnPanel ( contentPane ) ;
      cnr_TREE.setCellRenderer ( new BookmarkTreeCellRenderer() ) ;
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;
      refreshBookmarkList() ;        
      ck_USE_FOLDER_FOR_MULTIPLE.setEnabled ( multiple ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, select the initial node                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( initialSelectNode != null )
         WinUtil.selectTreeNode ( cnr_TREE, initialSelectNode ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Show the window                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 495, 325 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;



   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize () ;
      int       x         = wizardImage.getImage().getWidth(this)+10 ;
      int       y         = ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2) ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           y, 
                           wizardImage.getImage().getWidth(this), 
                           wizardImage.getImage().getHeight(this)) ;

      st_DESCRIPT.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      ef_DESCRIPT.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight+5 ;

      st_FOLDER.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      scrollPane.setBounds ( x, y, size.width-(x)-10, size.height-y-87-rowHeight ) ;

      ck_USE_FOLDER_FOR_MULTIPLE.setBounds ( x, scrollPane.getSize().height+scrollPane.getLocation().y+5, size.width-(x)-10, rowHeight ) ;

      buttonPanel.setBounds ( x, size.height-78, size.width-x, 25 ) ;

   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when a button
    *  is pressed
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         dispose() ;
         folderUsed = null ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_OK ) {
         if ( move )
            move() ;
         else
            add() ;

         if ( ck_USE_FOLDER_FOR_MULTIPLE.isSelected() ) 
            folderUsed = PropertySystem.getString ( PropertySystem.PROPERTY_LAST_BOOKMARK_FOLDER_USED ) ;

         setVisible ( false ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CREATEFOLDER                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CREATEFOLDER )
         createFolder() ;



   }



   /*------------------------------------------------------------*/
   /* refreshBookmarkList
    */
   /**
    *    This method will refresh all of the bookmarks
    *  in our tree view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshBookmarkList () {
      DefaultTreeModel model      = null ;
      StringTreeNode   root       = null ;
      Vector           folders    = BookmarkSystem.getFolders() ;
      String           lastFolder = PropertySystem.getString ( PropertySystem.PROPERTY_LAST_BOOKMARK_FOLDER_USED ) ;

      model = new DefaultTreeModel ( root = new StringTreeNode ( BookmarkSystem.getRootFolderName() ) ) ;
      insertBookmarkFolder ( model, root, BookmarkSystem.getRootFolder(), lastFolder ) ;
      cnr_TREE.setModel ( model ) ;
      cnr_TREE.setRootVisible ( false ) ;
      cnr_TREE.expandRow(0) ;


   } /* refreshBookmarkList */




   /*------------------------------------------------------------*/
   /* insertBookamrkFolder
    */
   /**
    *    This method will insert a folder into the
    *  DTree. It will also insert all bookmarks and
    *  subfolders in that folder, recursivly
    *
    *  @param model The DefaultTreeModel to use
    *  @param node The node to insert the folder under
    *  @param folder The folder to insert
    *  @param lastFolder The last folder that was selected
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void insertBookmarkFolder ( DefaultTreeModel model, StringTreeNode node, Bookmark folder, String lastFolder ) {
      Vector subFolders = BookmarkSystem.getFolders   ( folder.getDescript() ) ;

      node.add ( node = new StringTreeNode ( folder.getDescript(), folder ) ) ;

      if ( ( lastFolder != null ) && 
           ( folder.getDescript().equals ( lastFolder ) ) )
         initialSelectNode = node ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, put in the sub folders                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( subFolders != null ) && ( subFolders.size() > 0 ) ) {
         int            i        = 0 ;
         int            size     = subFolders.size() ;

         while ( i < size ) {
            insertBookmarkFolder ( model, node, (Bookmark) subFolders.elementAt(i), lastFolder ) ;
            i++ ;
         } /* while */

      } /* if */


   } /* insertBookmarkFolder */





   /*------------------------------------------------------------*/
   /* createFolder
    */
   /**
    *    This method will create a folder
    *  under the currently selected folder. If no folder
    *  is selected, a new folder will be created under root
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createFolder () {
      StringTreeNode selectedObject = null ;

      try {
         Bookmark       parentFolder   = BookmarkSystem.getRootFolder() ;
         Bookmark       newFolder      = new Bookmark() ;
         String         folderName     = new SingleEntryDlg ( WinUtil.getParentFrame ( this ),
                                                              Str.getStr ( STR_DESCRIPTION ),
                                                              Str.getStr ( STR_DESCRIPTION ),
                                                              "",
                                                              "" ).getResult() ;                        

         try {
            selectedObject = (StringTreeNode) cnr_TREE.getSelectionPath().getLastPathComponent() ;
         } catch ( Exception ex1 ) {
            selectedObject = null ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the user gave us a description                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( folderName != null ) && ( folderName.length() > 0 ) ) {

            /*-------------------------------------------------------------*/
            /*                                                             */
            /*   If they have a folder selected, add this new folder in it */
            /*                                                             */
            /*-------------------------------------------------------------*/
            if ( selectedObject != null )
               parentFolder = (Bookmark) selectedObject.getData () ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If they have something else selected, get it's parent   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !parentFolder.isFolder() )
               parentFolder = BookmarkSystem.getBookmark ( parentFolder.getDescript () ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this bookmark doesn't already exist, add it          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( BookmarkSystem.getBookmark(folderName) == null ) {
               newFolder.setDescript ( folderName ) ;
               newFolder.setParentFolder ( parentFolder.getDescript() ) ;
               newFolder.updateRecStatus ( newFolder.REC_INSERT ) ;
               BookmarkSystem.addBookmark ( newFolder ) ;
               refreshBookmarkList() ;
            } else
               GUISystem.printBox ( WinUtil.getParentFrame ( this ),
                                    Str.getStr ( STR_INFO ),
                                    Str.getStr ( STR_CANNOT_NAME_DUPLICATE_BOOKMARK ) ) ;
         }
      } catch ( Exception e ) {
      }

   } /* createFolder */








   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will try to add the bookmark
    *  into the system
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void add () {
      StringTreeNode selectedObject = null ;
      Bookmark       folder         = null ;
      String         descript       = ef_DESCRIPT.getText() ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Grab the folder that the user had selected              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         selectedObject = (StringTreeNode) cnr_TREE.getSelectionPath().getLastPathComponent() ;                                         
      } catch ( Exception ex ) {
         selectedObject = null ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If they gave us a description, continue                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( descript != null ) && ( descript.length() > 0 ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this description is not a duplicate, continue        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( BookmarkSystem.getBookmark ( descript ) == null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If the user did not select a folder, use the root folder*/
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( selectedObject == null )
               folder = BookmarkSystem.getRootFolder() ;
            else
               folder = (Bookmark) selectedObject.getData() ;

            bookmark.setParentFolder ( folder.getDescript() ) ;
            bookmark.updateRecStatus ( bookmark.REC_INSERT ) ;
            bookmark.setDescript     ( descript ) ;

            PropertySystem.put ( PropertySystem.PROPERTY_LAST_BOOKMARK_FOLDER_USED,
                                 folder.getDescript() ) ;
            BookmarkSystem.addBookmark ( bookmark ) ;

         } else
            GUISystem.printBox ( WinUtil.getParentFrame ( this ), 
                                 Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_CANNOT_NAME_DUPLICATE_BOOKMARK ) ) ;

      } else
         GUISystem.printBox ( WinUtil.getParentFrame ( this ), 
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_MUST_ENTER_VALUE ) ) ;

   }

   /*------------------------------------------------------------*/
   /* move
    */
   /**
    *    This method will try to move a bookmark
    *  in the system from one folder to another
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void move () {
      StringTreeNode selectedObject = null ;
      Bookmark       folder         = null ;
      String         descript       = ef_DESCRIPT.getText() ;

      try {
         selectedObject = (StringTreeNode) cnr_TREE.getSelectionPath().getLastPathComponent() ;                                         

      } catch ( Exception ex ) {
         selectedObject = null ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If they gave us a description, continue                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( descript != null ) && ( descript.length() > 0 ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this description is not a duplicate, continue        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( selectedObject == null )
            folder = BookmarkSystem.getRootFolder() ;
         else
            folder = (Bookmark) selectedObject.getData() ;

         PropertySystem.put ( PropertySystem.PROPERTY_LAST_BOOKMARK_FOLDER_USED,
                              folder.getDescript() ) ;

         bookmark.setParentFolder ( folder.getDescript() ) ;
         bookmark.updateRecStatus ( bookmark.REC_UPDATE ) ;
         bookmark.setDescript     ( descript ) ;

         BookmarkSystem.updateBookmark ( bookmark ) ;


      } else
         GUISystem.printBox ( WinUtil.getParentFrame ( this ), 
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_MUST_ENTER_VALUE ) ) ;

   }





   /*------------------------------------------------------------*/
   /* getFolderUsed
    */
   /**
    *    If the 'multiple' property is set through the
    *  constructor of the dialog, and the user selected the
    *  'Use folder for the rest of the selected Items' check box, This 
    *  method returns the folder that the user chose to use for the
    *  rest of the bookmarks. Otherwise this method returns NULL
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getFolderUsed () {
      return( folderUsed ) ;
   }

} /* AddBookmarkDlg */
