package com.ibm.nzna.projects.qit.bookmark ;   
/*-----------------------------------------------------------*/
/*   BookmarkDlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BookmarkDlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog allows the user to edit their bookmarks 
 *
 *
 * @version 1.0, 5/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BookmarkDlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class BookmarkDlg extends JDialog implements ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_IMAGE        = null ;
   private JScrollPane  scrollPane      = null ;
   private BookmarkTree cnr_TREE        = null ;
   private ButtonPanel  buttonPanel     = null ;
   private DButton      pb_EDIT         = null ;
   private DButton      pb_DELETE       = null ;
   private DButton      pb_CANCEL       = null ;      
   private DButton      pb_CREATEFOLDER = null ;
   private DButton      pb_MOVE         = null ;
   private DButton      pb_SELECT       = null ;
   private DButton      pb_SEND         = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ImageIcon           wizardImage       = null ;
   private BookmarkDlgListener listener          = null ;
   private Vector              selectedVec       = null ;







   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will not return until the
    *  window has been closed
    *  
    *
    *  @param parent The parent window of this dialog
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BookmarkDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_BOOKMARKS ), true ) ;     

      initializeAndShow() ;
   }

   public BookmarkDlg ( Frame parent, BookmarkDlgListener listener ) {
      super ( parent, Str.getStr ( STR_BOOKMARKS ), true ) ;     

      setBookmarkDlgListener ( listener ) ;
      initializeAndShow() ;
   }





   /*------------------------------------------------------------*/
   /* initializeAndShow
    */
   /**
    *    This method will iniitalize our window and
    *  then show it. It will not return until the window
    *  is closed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeAndShow() {
      Container contentPane = getContentPane() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardImage     = ImageSystem.getImageIcon ( this, ImageSystem.BOOKMARK_WIZARD ) ;
      st_IMAGE        = new JLabel ( wizardImage ) ;
      cnr_TREE        = new BookmarkTree  () ;
      scrollPane      = new JScrollPane ( cnr_TREE ) ;
      pb_DELETE       = new DButton ( Str.getStr ( STR_DELETE ) ) ;
      pb_EDIT         = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_CANCEL       = new DButton ( Str.getStr ( STR_CLOSE ) ) ;
      pb_CREATEFOLDER = new DButton ( Str.getStr ( STR_CREATE_FOLDER ) ) ;
      pb_MOVE         = new DButton ( Str.getStr ( STR_MOVE ) );
      pb_SELECT       = new DButton ( Str.getStr ( STR_SELECT ) ) ;
      pb_SEND         = new DButton ( Str.getStr ( STR_SEND ) ) ;
      buttonPanel     = new ButtonPanel () ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.add ( pb_SELECT ) ;
      buttonPanel.add ( pb_CREATEFOLDER ) ;
      buttonPanel.add ( pb_EDIT ) ;
      buttonPanel.add ( pb_DELETE ) ;
      buttonPanel.add ( pb_MOVE ) ;
      //buttonPanel.add ( pb_SEND ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( scrollPane ) ;
      contentPane.add ( buttonPanel ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_EDIT.addActionListener         ( this ) ;
      pb_DELETE.addActionListener       ( this ) ;
      pb_CANCEL.addActionListener       ( this ) ;
      pb_CREATEFOLDER.addActionListener ( this ) ;
      pb_MOVE.addActionListener         ( this ) ;
      pb_SELECT.addActionListener       ( this ) ;
      pb_SEND.addActionListener         ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;
      GUISystem.setPropertiesOnPanel ( contentPane ) ; 


      setSize ( 680, 325 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;
   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize () ;
      int       x         = wizardImage.getImage().getWidth(this)+10 ;
      int       y         = ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2) ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           y, 
                           wizardImage.getImage().getWidth(this), 
                           wizardImage.getImage().getHeight(this)) ;


      scrollPane.setBounds ( x, y, size.width-(x)-10, size.height-95 ) ;


      buttonPanel.setBounds ( x, size.height-78, size.width-x, 25 ) ;

   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL )
         dispose() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT )
         edit() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CREATEFOLDER                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CREATEFOLDER )
         createFolder() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_DELETE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DELETE )
         delete() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_MOVE                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_MOVE )
         move() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SELECT                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SELECT ) {
         if ( performSelect() ) {
            dispose() ;
            notifyBookmarkDlgListener() ;
         }
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SEND                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SEND ) {
         if ( performSelect() ) {
            Vector users = new SelectUserDlg ( GUISystem.getParentDefWin ( this ), false ).getResult() ;

            if ( ( users != null ) && ( users.size() > 0 ) ) 
               BookmarkSystem.sendBookmarks ( selectedVec, (UserRec)users.elementAt(0) ) ;
         }
      }




   } /* actionPerformed */






   /*------------------------------------------------------------*/
   /* edit
    */
   /**
    *    This method will edit the currently
    *  selected bookmark's description
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void edit () {
      try {
         StringTreeNode selectedObject = (StringTreeNode) cnr_TREE.getSelectionPath().getLastPathComponent() ;                                         

         if ( selectedObject != null ) {
            Bookmark bookmark = (Bookmark) selectedObject.getData() ;                          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Do not allow the user to edit root!                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !bookmark.getDescript().equals ( BookmarkSystem.getRootFolderName() ) ) {
               String newDescript = new SingleEntryDlg ( WinUtil.getParentFrame ( this ),
                                                         Str.getStr ( STR_DESCRIPTION ),
                                                         Str.getStr ( STR_DESCRIPTION ),
                                                         bookmark.getDescript(),
                                                         "" ).getResult() ;

               if ( ( newDescript != null ) && ( newDescript.length() > 0 ) ) {
                  if ( BookmarkSystem.getBookmark(newDescript) == null ) {
                     bookmark.setDescript ( newDescript ) ;
                     bookmark.updateRecStatus ( bookmark.REC_UPDATE ) ;
                  } else
                     GUISystem.printBox ( WinUtil.getParentFrame ( this ),
                                          Str.getStr ( STR_INFO ),
                                          Str.getStr ( STR_CANNOT_NAME_DUPLICATE_BOOKMARK ) ) ;

               } /* if they gave us a description */
            } /* if bookmark is not root */
         } /* if selectedObject */
      } catch ( Exception e ) {
      }
      repaint() ;
   } /* edit */






   /*------------------------------------------------------------*/
   /* createFolder
    */
   /**
    *    This method will create a folder
    *  under the currently selected folder. If no folder
    *  is selected, a new folder will be created under root
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createFolder () {
      try {
         StringTreeNode selectedObject = (StringTreeNode) cnr_TREE.getSelectionPath().getLastPathComponent() ;                                         
         Bookmark       parentFolder   = BookmarkSystem.getRootFolder() ;
         Bookmark       newFolder      = new Bookmark() ;
         String         folderName     = new SingleEntryDlg ( WinUtil.getParentFrame ( this ),
                                                              Str.getStr ( STR_DESCRIPTION ),
                                                              Str.getStr ( STR_DESCRIPTION ),
                                                              "",
                                                              "" ).getResult() ;                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the user gave us a description                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( folderName != null ) && ( folderName.length() > 0 ) ) {

            /*-------------------------------------------------------------*/
            /*                                                             */
            /*   If they have a folder selected, add this new folder in it */
            /*                                                             */
            /*-------------------------------------------------------------*/
            if ( selectedObject != null )
               parentFolder = (Bookmark) selectedObject.getData () ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If they have something else selected, get it's parent   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !parentFolder.isFolder() )
               parentFolder = BookmarkSystem.getBookmark ( parentFolder.getParentFolder () ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this bookmark doesn't already exist, add it          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( BookmarkSystem.getBookmark(folderName) == null ) {
               newFolder.setDescript ( folderName ) ;
               newFolder.setParentFolder ( parentFolder.getDescript() ) ;
               newFolder.updateRecStatus ( newFolder.REC_INSERT ) ;
               BookmarkSystem.addBookmark ( newFolder ) ;
            } else
               GUISystem.printBox ( WinUtil.getParentFrame ( this ),
                                    Str.getStr ( STR_INFO ),
                                    Str.getStr ( STR_CANNOT_NAME_DUPLICATE_BOOKMARK ) ) ;
         }
      } catch ( Exception e ) {
      }

   } /* createFolder */





   /*------------------------------------------------------------*/
   /* delete
    */
   /**
    *    This method will delete the folder or bookmark
    *  selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void delete () {
      try {
         StringTreeNode selectedObject = (StringTreeNode) cnr_TREE.getSelectionPath().getLastPathComponent() ;                                         

         if ( selectedObject != null ) {
            Bookmark bookmark = (Bookmark) selectedObject.getData() ;
            boolean  delete   = false ;

            if ( !bookmark.getDescript().equals ( BookmarkSystem.getRootFolderName() ) ) {
               if ( bookmark.isFolder() )
                  delete = GUISystem.printBox ( WinUtil.getParentFrame ( this ),
                                                Str.getStr ( STR_QUESTION ),
                                                Str.getStr ( STR_ARE_YOU_SURE_DELETE_BOOKMARK_FOLDER ) ) ;
               else
                  delete = GUISystem.printBox ( WinUtil.getParentFrame ( this ),
                                                Str.getStr ( STR_QUESTION ),
                                                Str.getStr ( STR_ARE_YOU_SURE_DELETE_BOOKMARK ) ) ;                                   


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If we are to delete, delete all selected now            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( delete ) {
                  int[]  selectedRows = cnr_TREE.getSelectionRows() ;
                  Vector deleteVec    = new Vector ( 1, 1 ) ;

                  for ( int i = 0 ; i < selectedRows.length; i++ ) {
                     selectedObject = (StringTreeNode)cnr_TREE.getPathForRow ( selectedRows[i] ).getLastPathComponent() ;

                     bookmark = (Bookmark)selectedObject.getData() ;
                     bookmark.updateRecStatus ( bookmark.REC_DELETE ) ;
                     deleteVec.addElement ( bookmark ) ;
                  }

                  BookmarkSystem.removeBookmarks ( deleteVec ) ;

               }
            } else
               GUISystem.printBox ( WinUtil.getParentFrame ( this ),
                                    Str.getStr ( STR_INFO ),
                                    Str.getStr ( STR_CANNOT_DELETE_ROOT_BOOKMARK ) ) ;


         } 

      } catch ( Exception e ) {
         LogSystem.log ( 1, e , false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* move
    */
   /**
    *    This method will move a bookmark from
    *  one folder to another
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void move () {
      try {
         StringTreeNode selectedObject = (StringTreeNode) cnr_TREE.getSelectionPath().getLastPathComponent() ;                                         

         if ( selectedObject != null ) {
            Bookmark bookmark     = (Bookmark) selectedObject.getData() ;
            boolean  delete       = false ;
            int[]    selectedRows = cnr_TREE.getSelectionRows() ;
            Vector   moveVec      = new Vector ( selectedRows.length ) ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add all selected nodes as bookmarks to our vec          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            for ( int i = 0 ; i < selectedRows.length; i++ ) {


               bookmark = (Bookmark) ((StringTreeNode)cnr_TREE.getPathForRow(selectedRows[i]).getLastPathComponent()).getData() ;                               

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Do not allow the root folder to be moved                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !bookmark.getDescript().equals ( BookmarkSystem.getRootFolderName() ) ) {
                  moveVec.addElement ( bookmark ) ;
               } else
                  GUISystem.printBox ( WinUtil.getParentFrame ( this ),
                                       Str.getStr ( STR_INFO ),
                                       Str.getStr ( STR_CANNOT_DELETE_ROOT_BOOKMARK ) ) ;

            }

            BookmarkSystem.guiMoveBookmarks ( moveVec ) ;
         }
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* setBookmarkDlgListener
    */
   /**
    *    This method will set the one listener that will
    *  be notified of a selection in the BookmarkDlg
    *  
    *
    *  @param listener The listener to be notified
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBookmarkDlgListener ( BookmarkDlgListener listener ) {
      this.listener = listener ;
   }





   /*------------------------------------------------------------*/
   /* notifyBookmarkDlgListener
    */
   /**
    *    This method will notify our listener that something
    *  has been selected. This method will send the
    *  'selectedVec' property to the listener.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void notifyBookmarkDlgListener ( ) {
      if ( listener != null )
         listener.bookmarkSelected ( selectedVec ) ;
   }





   /*------------------------------------------------------------*/
   /* performSelect
    */
   /**
    *    This method will figure out what is selected
    *  in the list and will set the selectedVec property
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean performSelect () {
      int[] selectedRows = cnr_TREE.getSelectionRows() ;

      selectedVec = null ;

      if ( ( selectedRows != null ) && ( selectedRows.length > 0 ) ) {
         StringTreeNode treeNode = null ;                                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up our vector for at least the selection size       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         selectedVec = new Vector ( selectedRows.length ) ;                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, process each selected item                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         for ( int i = 0 ; i < selectedRows.length; i++ ) {
            treeNode = (StringTreeNode) cnr_TREE.getPathForRow ( selectedRows[i] ).getLastPathComponent() ;
            if ( treeNode != null )
               addSelectedItem ( (Bookmark)treeNode.getData() ) ;
         }

      }

      return( selectedVec != null ) ;
   }





   /*------------------------------------------------------------*/
   /* addSelectedItem
    */
   /**
    *    This method will add an item to the vector
    *  of selected items. If the item passed is a bookmark
    *  folder, this method will recursivly call itself to add all elements
    *
    *  @param bookmark The Bookmark to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addSelectedItem ( Bookmark bookmark ) {
      if ( bookmark != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this bookmark is not a folder, just add it           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !bookmark.isFolder() )
            selectedVec.addElement ( bookmark ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, loop through and add each of it's items      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            Vector vec = BookmarkSystem.getBookmarks ( bookmark.getDescript() ) ;

            if ( ( vec != null ) && ( vec.size() > 0 ) ) {
               int i    = 0 ;
               int size = vec.size() ;

               while ( i < size ) {
                  addSelectedItem ( (Bookmark)vec.elementAt(i) ) ;
                  i++ ;
               } /* while */

            } /* if the vec was good */

         } /* else */

      } /* if bookmark */

   } /* addSelectedItem */

} /* BookmarkDlg */
