package com.ibm.nzna.projects.qit.bookmark ;    
/*-----------------------------------------------------------*/
/*   BookmarkHotLinkTree                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BookmarkHotLinkTree                 */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.*;
import com.ibm.nzna.shared.util.*;
import javax.swing.* ;
import javax.swing.tree.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
/**
 * This is a NavList of HotLinkLabels to view Bookmarks.
 *
 *
 * @version 1.0, 5/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BookmarkHotLinkTree                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class BookmarkHotLinkTree extends JPanel implements NavListExpandListener, AppConst, ActionListener, BookmarkListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean          folderOnly = false ;
   private NavList          navList    = null ;
   private Vector           listeners  = new Vector ( 1 ) ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method constructs everything!
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BookmarkHotLinkTree () {
      super() ;

      BookmarkSystem.addBookmarkListener (this ) ;
      refreshBookmarkList() ;

   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method constructs everything!
    *  
    *  
    *
    *  @param folderOnly TRUE = only show folders, FALSE = show all bookmarks
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BookmarkHotLinkTree ( boolean folderOnly ) {
      super() ;

      this.folderOnly = folderOnly ;
      BookmarkSystem.addBookmarkListener (this ) ;
      refreshBookmarkList() ;

   }





   /*------------------------------------------------------------*/
   /* refreshBookmarkList
    */
   /**
    *    This method will refresh all of the bookmarks
    *  in our tree view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshBookmarkList () {
      StringTreeNode   root      = null ;
      Vector           folders   = BookmarkSystem.getFolders() ;

      if ( navList != null )
         remove ( navList ) ;

      navList = new NavList() ;
      navList.setOpaque ( false ) ;
      setLayout ( new BorderLayout() ) ;
      add ( navList, BorderLayout.CENTER ) ;

      insertBookmarkFolder ( navList, BookmarkSystem.getRootFolder() ) ;


   } /* refreshBookmarkList */




   /*------------------------------------------------------------*/
   /* insertBookmarkFolder
    */
   /**
    *    This method will insert a folder into the
    *  DTree. It will also insert all bookmarks and
    *  subfolders in that folder, recursivly
    *
    *  @param folder The folder to insert
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void insertBookmarkFolder ( JPanel parentPanel, Bookmark folder ) {
      try {
         Vector       subFolders = BookmarkSystem.getFolders   ( folder.getDescript() ) ;
         Vector       bookmarks  = BookmarkSystem.getBookmarks ( folder.getDescript() ) ;                    
         NavListGroup newPanel   = null ;

         parentPanel.add ( newPanel = new NavListGroup ( folder.getDescript(), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE_FOLDER ) ) ) ;
         newPanel.setExpandedIcon ( ImageSystem.getImageIcon ( this, ImageSystem.OPEN_FOLDER ) ) ;
         newPanel.setData ( folder ) ;
         newPanel.addExpandListener ( this ) ;
         newPanel.setOpaque ( false ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, put in the sub folders                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( subFolders != null ) && ( subFolders.size() > 0 ) ) {
            int            i        = 0 ;
            int            size     = subFolders.size() ;

            while ( i < size ) {
               insertBookmarkFolder ( newPanel, (Bookmark)subFolders.elementAt(i) ) ;
               i++ ;
            } /* while */

         } /* if */


         if ( !folderOnly ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now put the regular bookmarks in                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( bookmarks != null ) && ( bookmarks.size() > 0 ) ) {
               int          i        = 0 ;
               int          size     = bookmarks.size() ;
               HotLinkLabel newLabel = null ;

               while ( i < size ) {
                  if ( !((Bookmark)bookmarks.elementAt(i)).isFolder() ) {
                     newPanel.add ( newLabel = new HotLinkLabel ( ((Bookmark)bookmarks.elementAt(i)).getDescript(), 
                                                                  ((Bookmark)bookmarks.elementAt(i)).getIcon(),
                                                                  0,
                                                                  bookmarks.elementAt(i) ) ) ;
                     newLabel.setOpaque ( false ) ;
                  }
                  i++ ;
               } /* while */

            } /* if */
         } /* if !foldersOnly */


         //if ( folder == BookmarkSystem.getRootFolder() )
         //   ((NavListGroup)newPanel).expand();
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

   } /* insertBookmarkFolder */


   public void finalize () {
      try {
         super.finalize() ;
         BookmarkSystem.removeBookmarkListener ( this ) ;
      } catch ( Throwable e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* bookmarkAdded
    */
   /**
    *    This method will be called when the BookmarkSystem
    *  has added a bookmark. We will put this bookmark
    *  into our window
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void bookmarkAdded ( Bookmark bookmark ) {
      refreshBookmarkList() ;
   }





   /*------------------------------------------------------------*/
   /* bookmarkRemoved
    */
   /**
    *    This method will be called when the BookmarkSystem
    *  has removed a bookmark. We will also remove
    *  that bookmark
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void bookmarkRemoved ( Bookmark bookmark ) {
      refreshBookmarkList() ;
   }




   /*------------------------------------------------------------*/
   /* setFolderOnly
    */
   /**
    *    This method will set the folder only property.
    *  
    *  
    *
    *  @param folderOnly TRUE = Show only Bookmark Folders, FALSE = Show all bookmarks
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFolderOnly ( boolean folderOnly ) {
      this.folderOnly = folderOnly ;
      refreshBookmarkList() ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will simply call our ActionListeners
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() instanceof HotLinkLabel ) {
         if ( listeners != null ) {
            ActionEvent newEvent = new ActionEvent ( ((HotLinkLabel)e.getSource()).getData(), 0, "Bookmark" ) ;
            int         i        = 0 ;
            int         size     = listeners.size() ;

            while ( i < size ) {
               ((ActionListener)listeners.elementAt(i)).actionPerformed ( newEvent ) ;
               i++ ;
            }
         }
      }
   }




   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an ActionListener to this
    *  control. Receivers of the action events sent
    *  will be notified of a Bookmark being selected, or a
    *  folder being selected. The source of the event will be
    *  the Bookmark object itself
    *
    *  @param listener The listener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      listeners.addElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* removeActionListener
    */
   /**
    *    This method will remove an ActionListener
    *  from this control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeActionListener ( ActionListener listener ) {
      listeners.removeElement ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* expanded
    */
   /**
    *    This method will be notified if a folder is
    *  expanded
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void expanded ( NavListGroup source, boolean expanded ) {
      if ( expanded ) {
         if ( listeners != null ) {
            ActionEvent newEvent = new ActionEvent ( source.getData(), 0, "Bookmark" ) ;
            int         i        = 0 ;
            int         size     = listeners.size() ;

            while ( i < size ) {
               ((ActionListener)listeners.elementAt(i)).actionPerformed ( newEvent ) ;
               i++ ;
            }
         }
      }
   }

} /* BookmarkHotLinkTree */
