package com.ibm.nzna.projects.qit.bookmark ;       
/*-----------------------------------------------------------*/
/*   BookmarkSystem                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BookmarkSystem                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.io.* ;
import java.sql.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.* ;
/**
 * This system is designed to handle all bookmarks in the Quest Application 
 *
 *
 * @version 1.0, 4/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BookmarkSystem                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class BookmarkSystem extends Object implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Vector         bookmarks      = null ;
   private static Vector         listeners      = null ;
   private static boolean        newUser        = true ;
   private static BookmarkSystem instance       = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Like all good sub systems, the constructor
    *  does nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BookmarkSystem ( ) {
      if ( instance == null )
         instance = this ;
   }









   /*------------------------------------------------------------*/
   /* fireBookmarkRemoved
    */
   /**
    *    This method will fire a bookmarkRemoved
    *  message to all registered listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void fireBookmarkRemoved ( Bookmark bookmark ) {
      if ( ( listeners != null ) && ( listeners.size() > 0 ) ) {
         int i    = 0 ;
         int size = listeners.size() ;

         while ( i < size ) {
            ((BookmarkListener)listeners.elementAt(i)).bookmarkRemoved ( bookmark ) ;
            i++ ;
         }
      }
   } /* fireBookmarkRemoved  */

   public void finalize () {
   try {
      super.finalize() ;
      System.out.println ( "Bookmark System FINALIZED!" ) ;
   } catch ( Throwable e ) {
   }
}

   /*------------------------------------------------------------*/
   /* fireBookmarkAdded
    */
   /**
    *    This method will fire a bookmarkAdded
    *  message to all registered listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected static void fireBookmarkAdded ( Bookmark bookmark ) {
      if ( ( listeners != null ) && ( listeners.size() > 0 ) ) {
         int i    = 0 ;
         int size = listeners.size() ;

         while ( i < size ) {
            ((BookmarkListener)listeners.elementAt(i)).bookmarkAdded ( bookmark ) ;
            i++ ;
         }
      }
   } /* fireBookmarkAdded  */





   /*------------------------------------------------------------*/
   /* getBookmarks
    */
   /**
    *    This method will return all bookmarks
    *  under the folder given
    *  
    *
    *  @param folderName The name of the folder
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getBookmarks ( String folderName ) {
      Vector returnVec = new Vector ( 10, 1 ) ;
      int    i         = 0 ;
      int    size      = bookmarks.size() ;

      while ( i < size ) {

         if ( ( ((Bookmark)bookmarks.elementAt(i)).getParentFolder() != null ) &&
              ( ((Bookmark)bookmarks.elementAt(i)).getParentFolder().equals ( folderName ) ) )
            returnVec.addElement ( bookmarks.elementAt(i) ) ;
         i++ ;
      }

      return( returnVec ) ;
   } /* getBookmarks  */






   /*------------------------------------------------------------*/
   /* getFolders
    */
   /**
    *    This method will return a list of folders
    *  who belong to the parent folder passed. This 
    *  list is not recursive
    *
    *  @param parentFolder The name of the parent folder
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getFolders ( String folderName ) {
      Vector returnVec = new Vector ( 10, 1 ) ;
      int    i         = 0 ;
      int    size      = bookmarks.size() ;

      while ( i < size ) {
         if ( ( ((Bookmark)bookmarks.elementAt(i)).getParentFolder() != null ) &&
              ( ((Bookmark)bookmarks.elementAt(i)).getParentFolder().equals ( folderName ) ) &&
              ( ((Bookmark)bookmarks.elementAt(i)).isFolder() ) )
            returnVec.addElement ( bookmarks.elementAt(i) ) ;
         i++ ;
      }

      return( returnVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getFolders
    */
   /**
    *    This method will return all first-level
    *  folders that fall under the root folder
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector getFolders () {
      return( getFolders ( getRootFolderName() ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getRootFolderName
    */
   /**
    *    This method will return the root folder
    *  name
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String getRootFolderName () {
      return( UserSystem.getLoggedUserRec().getName()+Str.getStr ( STR_BOOKMARK_NAME_EXTENSION ) ) ;
   }




   /*------------------------------------------------------------*/
   /* createFolder
    */
   /**
    *    This method will create a new folder
    *  in the bookmark list. This folder will be placed
    *  directly under the root folder
    *
    *  @param folderName The name of the folder
    *  @param None
    *  @param None
    *
    *  @return Bookmark The bookmark representing this folder
    *//*---------------------------------------------------------*/
   public static Bookmark createFolder ( String folderName ) {
      return( createFolder ( folderName, getRootFolderName() ) ) ;
   }




   /*------------------------------------------------------------*/
   /* createFolder
    */
   /**
    *    this method will create a new folder inside
    *  of the parent folder specified.
    *  
    *
    *  @param folderName The name of the folder
    *  @param parentFolder The name of the parent folder
    *  @param None
    *
    *  @return Bookmark The bookmark representing this folder
    *//*---------------------------------------------------------*/
   public static Bookmark createFolder ( String folderName, String parentFolderName ) {
      Bookmark bookmark = new Bookmark () ;

      bookmark.setDescript ( folderName ) ;
      bookmark.setParentFolder ( parentFolderName ) ;
      bookmark.updateRecStatus ( bookmark.REC_INSERT ) ;
      addBookmark ( bookmark ) ;

      return( bookmark ) ;
   }





   /*------------------------------------------------------------*/
   /* addBookmark
    */
   /**
    *    This method will add a bookmark to the
    *  list of bookmarks
    *  
    *
    *  @param bookmark The bookmark to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addBookmark ( Bookmark bookmark ) {
      bookmarks.addElement ( bookmark ) ;
      new Thread ( new BookmarkThread ( bookmark ), BookmarkThread.BOOKMARK_ADDED ).start()  ;
   }



   /*------------------------------------------------------------*/
   /* addBookmarks
    */
   /**
    *    This method will add the Vector of bookmarks to the
    *  list of bookmarks
    *  
    *
    *  @param bookmarkVec The Vector of bookmarks to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addBookmarks ( Vector bookmarkVec ) {
      int size = bookmarkVec.size() ;

      for ( int i = 0; i < size; i++ ) {
         bookmarks.addElement ( bookmarkVec.elementAt(i) ) ;
         fireBookmarkAdded ( (Bookmark) bookmarkVec.elementAt(i) ) ;
      }

      new Thread ( new BookmarkThread ( null ), BookmarkThread.SAVE ).start()  ;
   }

   /*------------------------------------------------------------*/
   /* removeBookmarks
    */
   /**
    *    This method will remove the Vector of bookmarks from the
    *  list of bookmarks
    *  
    *
    *  @param bookmarkVec The Vector of bookmarks to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeBookmarks ( Vector bookmarkVec ) {
      int      size     = bookmarkVec.size() ;
      Bookmark bookmark = null ;

      for ( int i = 0; i < size; i++ ) {
         bookmark = (Bookmark) bookmarkVec.elementAt(i) ;

         if ( bookmark.isFolder() )
            removeAllBookmarksFromFolder ( bookmark.getDescript() ) ;

         bookmarks.removeElement ( bookmark ) ;

         fireBookmarkRemoved ( bookmark ) ;

      }

      new Thread ( new BookmarkThread(null), BookmarkThread.SAVE ).start()  ;
   }



   /*------------------------------------------------------------*/
   /* updateBookmark
    */
   /**
    *    This method will update a bookmark in the
    *  list of bookmarks
    *  
    *
    *  @param bookmark The bookmark to update
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void updateBookmark ( Bookmark bookmark ) {
      new Thread ( new BookmarkThread ( bookmark ), BookmarkThread.BOOKMARK_ADDED ).start()  ;
   }



   /*------------------------------------------------------------*/
   /* removeBookmark
    */
   /**
    *    This method will remove a bookmark from the
    *  list of bookmarks
    *  
    *
    *  @param bookmark The bookmark to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeBookmark ( Bookmark bookmark ) {
      if ( bookmark.isFolder() )
         removeFolder ( bookmark.getDescript() ) ;
      else {
         bookmarks.removeElement ( bookmark ) ;
         new Thread ( new BookmarkThread ( bookmark ), BookmarkThread.BOOKMARK_REMOVED ).start()  ;
      }
   }




   /*------------------------------------------------------------*/
   /* removeFolder
    */
   /**
    *    This method will remove a folder and
    *  all bookmarks that belong inside of that
    *  folder recusrivly. There will be only 1 bookmarkRemoved
    *  fired for all deletes, after all deletes have happened.
    *
    *  @param folderName The name of the folder to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeFolder ( String folderName ) {
      removeAllBookmarksFromFolder ( folderName ) ;
      bookmarks.removeElement ( getBookmark ( folderName ) ) ;
      new Thread ( new BookmarkThread ( null ), BookmarkThread.BOOKMARK_REMOVED ).start() ;
   }

   private static void removeAllBookmarksFromFolder ( String folderName ) {
      int      i         = 0 ;
      Bookmark bookmark  = null ;

      while ( i < bookmarks.size() ) {
         bookmark = (Bookmark) bookmarks.elementAt(i) ;

         if ( bookmark != null ) {
            if ( bookmark.getParentFolder() != null ) {
               if ( ( bookmark.getParentFolder().equals ( folderName ) ) &&
                    ( bookmark.isFolder() ) ) {

                  removeAllBookmarksFromFolder ( bookmark.getDescript() ) ;
                  bookmarks.removeElement ( bookmark ) ;
                  i=0 ;
               } else if ( bookmark.getParentFolder().equals ( folderName ) ) {
                  bookmarks.removeElement ( bookmarks.elementAt(i) ) ;
                  i=0 ;
               }
            }
         }
         i++ ;
      }

   }





   /*------------------------------------------------------------*/
   /* addBookmarkListener
    */
   /**
    *    This method will add a BookmarkListener
    *  to this system
    *  
    *
    *  @param listener The BookmarkListener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addBookmarkListener ( BookmarkListener listener ) {
      if ( listeners == null )
         listeners = new Vector ( 1, 1 ) ;
      if ( listeners != null )
         listeners.addElement ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* removeBookmarkListener
    */
   /**
    *    This method will remove a listener
    *  from the list of registered listeres
    *  
    *
    *  @param listener The BookmarkListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void removeBookmarkListener ( BookmarkListener listener ) {
      if ( listeners != null )
         listeners.removeElement ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* writeBookmarks
    */
   /**
    *    This method will save all bookmarks
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static boolean writeBookmarks () {
      boolean   rc        = false ;
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "BookmarkSystem.writeBookmarks", 5 ) ;

      try {
         PreparedStatement stmt = null ;
         String       strStmt = null ;


         if ( newUser ) {
            strStmt = "INSERT INTO QIT.USERDATA "+
                      "( USERID, BOOKMARKS, DBUSER, CHANGEDTIME ) "+
                      "values "+
                      "( '"+UserSystem.getUserId()+"', "+
                      "  ?, "+
                      " '"+UserSystem.getUserId()+"', "+
                      " CURRENT TIMESTAMP )" ;
         } else {
            strStmt = "UPDATE QIT.USERDATA "+
                      "SET BOOKMARKS         = ?, "+
                      "    DBUSER            = '"+UserSystem.getUserId()+"', "+
                      "    CHANGEDTIME       = CURRENT TIMESTAMP "+
                      "WHERE USERID = '"+UserSystem.getUserId()+"'" ;
         }

         if ( strStmt != null ) {
            FileUtil.writeObjectToDisk ( "tempbookmark.ser", bookmarks ) ;

            File            file        = new File ( "tempbookmark.ser" ) ;
            FileInputStream inputStream = new FileInputStream ( file ) ;
            byte[]          inBytes     = FileUtil.readBytesFromFile ( "tempbookmark.ser" ) ;

            stmt = sqlMethod.createPreparedStatement ( strStmt ) ;
            stmt.setBinaryStream ( 1, new ByteArrayInputStream ( inBytes ), inBytes.length ) ;

            inputStream.close() ;
            FileUtil.deleteFile ( "tempbookmark.ser" ) ;
            stmt.execute() ;
            stmt.close() ;
            stmt = null ;

            rc = true ;

         } /* if  */

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;
      return( rc ) ;
   } /* writeBookmarks */




   /*------------------------------------------------------------*/
   /* readBookmarks
    */
   /**
    *    This method will re-read all of the bookmarks
    *  for the application
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void readBookmarks () {
      Vector newBookmarks = readBookmarks ( UserSystem.getUserId() ) ;

      if ( newBookmarks != null )
         bookmarks = newBookmarks ;
      else
         createRootBookmark() ;


   } /* readBookmarks */


   /*------------------------------------------------------------*/
   /* readBookmarks
    */
   /**
    *    This method will re-read all of the bookmarks
    *  for the application
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static Vector readBookmarks ( String userId ) {
      SQLMethod  sqlMethod    = new SQLMethod ( DatabaseSystem.QUEST, "BookmarkSystem.readBookmarks", 5 ) ;
      Vector     newBookmarks = new Vector ( 1, 1 ) ;

      try {
         Statement    stmt    = sqlMethod.createStatement() ;
         ResultSet    rs      = null ;

         rs = stmt.executeQuery ( "SELECT USERID, BOOKMARKS "+
                                  "FROM QIT.USERDATA "+
                                  "WHERE USERID = '"+userId+"'" ) ;

         if ( rs.next() ) {
            newBookmarks =  (Vector)StoredProcUtil.getObjectFromResultSet ( rs, 2 ) ;
            newUser = false ;
         } else {
            newUser = true ;
            createRootBookmark() ;
         }

         rs.close() ;
         stmt.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         newUser = false ;
         newBookmarks=null ;
      }

      if ( ( newBookmarks != null ) && ( newBookmarks.size() < 1 ) ) {
         newBookmarks = null ;
      }

      sqlMethod.close() ;

      return( newBookmarks ) ;
   } /* readBookmarks */





   /*------------------------------------------------------------*/
   /* getRootFolder
    */
   /**
    *    This method will return the root folder
    *  of all bookmarks
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Bookmark
    *//*---------------------------------------------------------*/
   public static Bookmark getRootFolder () {
      Bookmark root     = null ;
      int      i        = 0 ;
      int      size     = bookmarks.size() ;
      String   rootName = getRootFolderName() ;

      while ( ( i < size ) && ( root == null ) ) {
         if ( ((Bookmark)bookmarks.elementAt(i)).getDescript().equals ( rootName ) )
            root = (Bookmark)bookmarks.elementAt(i) ;
         i++ ;
      }
      return( root ) ;
   }


   /*------------------------------------------------------------*/
   /* getBookmark
    */
   /**
    *    This method will return the bookmark that matches
    *  the description passed
    *  
    *
    *  @param descript The Description of the bookmark you are looking for
    *  @param None
    *  @param None
    *
    *  @return Bookmark
    *//*---------------------------------------------------------*/
   public static Bookmark getBookmark ( String descript ) {
      Bookmark bookmark = null ;
      int      i        = 0 ;
      int      size     = bookmarks.size() ;

      while ( ( i < size ) && ( bookmark == null ) ) {
         if ( ((Bookmark)bookmarks.elementAt(i)).getDescript().equals ( descript ) )
            bookmark = (Bookmark)bookmarks.elementAt(i) ;
         i++ ;
      }

      return( bookmark ) ;
   }




   /*------------------------------------------------------------*/
   /* createRootBookmark
    */
   /**
    *    This method will create the root bookmark
    *  for the user. This will clear out all other
    *  bookmarks
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static void createRootBookmark () {
      Bookmark rootBookmark = new Bookmark() ;
      Bookmark bookmark     = null ;

      LogSystem.log ( 1, "Creating ROOT Bookmarks" ) ;
      rootBookmark.setDescript ( UserSystem.getLoggedUserRec().getName()+Str.getStr ( STR_BOOKMARK_NAME_EXTENSION ) ) ;
      bookmarks = new Vector ( 1, 1 ) ;
      addBookmark ( rootBookmark ) ;                                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the default bookmarks                            */
      /*                                                           */
      /*-----------------------------------------------------------*/

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the 'Web Support' folder                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      bookmark = new Bookmark() ;
      bookmark.setDescript ( Str.getStr ( STR_WEB_SUPPORT ) ) ;
      bookmark.setParentFolder ( rootBookmark.getDescript() );
      bookmark.updateRecStatus ( bookmark.REC_INSERT ) ;
      bookmarks.addElement ( bookmark ) ;                   


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the eSupport Web Site                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      bookmark = new Bookmark() ;
      bookmark.setDescript ( "eSupport" ) ;
      bookmark.setObject ( "http://www.ibm.com/pc/support", BOOKMARK_URL ) ;
      bookmark.setParentFolder ( Str.getStr ( STR_WEB_SUPPORT ) ) ;
      bookmark.updateRecStatus ( bookmark.REC_INSERT ) ;
      bookmarks.addElement ( bookmark ) ;

      bookmark = new Bookmark() ;
      bookmark.setDescript ( "Business Partner" ) ;
      bookmark.setObject ( "http://www.ibm.com/pc/support", BOOKMARK_URL ) ;
      bookmark.setParentFolder ( Str.getStr ( STR_WEB_SUPPORT ) ) ;
      bookmark.updateRecStatus ( bookmark.REC_INSERT ) ;
      bookmarks.addElement ( bookmark ) ;

      bookmark = new Bookmark() ;
      bookmark.setDescript ( "Info Tips" ) ;
      bookmark.setObject ( "http://w3.pc.ibm.com/helpcenter/infotips", BOOKMARK_URL ) ;
      bookmark.setParentFolder ( Str.getStr ( STR_WEB_SUPPORT ) ) ;
      bookmark.updateRecStatus ( bookmark.REC_INSERT ) ;
      bookmarks.addElement ( bookmark ) ;
      new Thread ( new BookmarkThread ( bookmark ), BookmarkThread.BOOKMARK_ADDED ).start()  ;
   }





   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return the instance of the
    *  BookmarkSystem that is being used
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static BookmarkSystem getInstance() {
      return( instance ) ;
   }





   /*------------------------------------------------------------*/
   /* guiAddBookmarks
    */
   /**
    *    This method will go through the vector
    *  passed in and will process each bookmark, asking
    *  the user for information (Description, Folder etc..) for the bookmark.
    *  This is the preferred way of handling the additions of bookmarks.
    *
    *  @param bookmarkVec The vector of Bookmarks
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void guiAddBookmarks ( Vector bookmarkVec ) {
      int            i           = 0 ;
      int            size        = bookmarkVec.size() ;
      Vector         autoAddVec  = new Vector ( size ) ;
      String         useFolder   = null ;
      AddBookmarkDlg bookmarkDlg = null ;
      Bookmark       bookmark    = null ;
      Frame          tempFrame   = new Frame() ;

      while ( i < size ) {

         bookmark = (Bookmark)bookmarkVec.elementAt(i) ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If useFolder is null, ask the user which folder         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( useFolder == null ) {
            bookmarkDlg = new AddBookmarkDlg ( tempFrame,
                                               bookmark, 
                                               false,       // Do not initiate a MOVE
                                               (size > 1) ) ;

            useFolder = bookmarkDlg.getFolderUsed() ;
            bookmarkDlg.dispose() ;

         } /* useFolder  */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, add the bookmark to the auto-add vec         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            bookmark.setParentFolder ( useFolder ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is not a duplicate, add it to the auto-add      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( BookmarkSystem.getBookmark ( bookmark.getDescript() ) == null )
               autoAddVec.addElement ( bookmark ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, we must ask the user for another name        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
               GUISystem.printBox ( tempFrame,
                                    Str.getStr ( STR_INFO ),
                                    Str.getStr ( STR_CANNOT_NAME_DUPLICATE_BOOKMARK ) ) ;
               new AddBookmarkDlg ( tempFrame,
                                    bookmark ).dispose() ;
            }

         } /* else  */
         i++ ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now we must process the auto-Add Vector             */                                  
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( autoAddVec.size() > 0 )
         addBookmarks ( autoAddVec ) ;

      tempFrame.dispose() ;
   } /* guiAddBookmarks  */

   /*------------------------------------------------------------*/
   /* guiMoveBookmarks
    */
   /**
    *    This method will go through the vector
    *  passed in and will process each bookmark, asking
    *  the user for information (Description, Folder etc..) for the bookmark.
    *  This is the preferred way of handling the moves of bookmarks.
    *
    *  @param bookmarkVec The vector of Bookmarks
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void guiMoveBookmarks ( Vector bookmarkVec ) {
      int            i           = 0 ;
      int            size        = bookmarkVec.size() ;
      Vector         autoMoveVec  = new Vector ( size ) ;
      String         useFolder   = null ;
      AddBookmarkDlg bookmarkDlg = null ;
      Bookmark       bookmark    = null ;
      Frame          tempFrame   = new Frame() ;

      while ( i < size ) {

         bookmark = (Bookmark)bookmarkVec.elementAt(i) ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If useFolder is null, ask the user which folder         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( useFolder == null ) {
            bookmarkDlg = new AddBookmarkDlg ( tempFrame,
                                               bookmark, 
                                               true,
                                               (size > 1) ) ;

            useFolder = bookmarkDlg.getFolderUsed() ;
            bookmarkDlg.dispose() ;

         } /* useFolder  */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, move the bookmark to the auto-move vec       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            bookmark.setParentFolder ( useFolder ) ;
         } /* else  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, notify all listeners of this                       */                        
         /*                                                           */
         /*-----------------------------------------------------------*/
         fireBookmarkAdded ( bookmark ) ;

         i++ ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now we must process the auto-Move Vector            */                                  
      /*                                                           */
      /*-----------------------------------------------------------*/
      new Thread ( new BookmarkThread(null), BookmarkThread.SAVE ).start() ;

      tempFrame.dispose() ;
   } /* guiAddBookmarks  */





   /*------------------------------------------------------------*/
   /* sendBookmarks
    */
   /**
    *    This method will send a vector of bookmarks
    *  to another person.   *** NOT WORKING YET ***
    *  
    *
    *  @param bookmarkVec The Vector of Bookmarks to send. These can be folders or whatever
    *  @param userRec The User to send the bookmarks to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void sendBookmarks ( Vector bookmarkVec, UserRec userRec ) {
      StatusWin statusWin = new StatusWin ( MainWindow.getInstance() ) ;

      try {
         int      i               = 0 ;
         int      size            = bookmarkVec.size() ;
         Vector   userBookmarks   = readBookmarks ( userRec.getUserId() ) ;
         Bookmark currentBookmark = null ;                                    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, get the bookmarks for the other person           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         statusWin.setText ( Str.getStr ( STR_SENDING ) ) ;                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Put in the 'Bookmarks for MyName'                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            currentBookmark = (Bookmark)((Bookmark)bookmarkVec.elementAt(i)).clone() ;

            if ( currentBookmark.isFolder() ) {
            }

            i++ ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      statusWin.dispose() ;
   }


} /* BookmarkSystem */


class BookmarkThread extends Object implements Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   String Thread Operations                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected static final String BOOKMARK_REMOVED    = "BR" ;
   protected static final String BOOKMARK_ADDED      = "BA" ;
   protected static final String SAVE                = "SA" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Bookmark actionBookmark = null ;

   /*------------------------------------------------------------*/
   /* BookmarkSystem
    */
   /**
    *    This constructor is only used to 
    *  create a new thread to fire events with
    *  
    *
    *  @param bookmark The bookmark to fire events with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   BookmarkThread ( Bookmark bookmark ) {
      this.actionBookmark = bookmark ;
   }


   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will simply notify our listeners
    *  that an event has ocurred
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      if ( operation.equals ( BOOKMARK_REMOVED ) )
         BookmarkSystem.fireBookmarkRemoved ( actionBookmark ) ;

      else if ( operation.equals ( BOOKMARK_ADDED ) )
         BookmarkSystem.fireBookmarkAdded ( actionBookmark ) ;

      if ( ( operation.equals ( SAVE ) ) ||
           ( operation.equals ( BOOKMARK_ADDED ) ) ||
           ( operation.equals ( BOOKMARK_REMOVED ) ) ) {
         if ( MainWindow.getInstance() != null )
            MainWindow.getInstance().setStatus ( STR_SAVING_BOOKMARKS ) ;
         BookmarkSystem.writeBookmarks() ;
         if ( MainWindow.getInstance() != null )
            MainWindow.getInstance().setStatus ( null ) ;
      }
   }

}
