package com.ibm.nzna.projects.qit.bookmark ;    
/*-----------------------------------------------------------*/
/*   BookmarkTree                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BookmarkTree                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.*;
import com.ibm.nzna.shared.util.*;
import javax.swing.* ;
import javax.swing.tree.* ;
import java.util.* ;
/**
 * This subclass of JTREE holds a user's bookmarks 
 *
 *
 * @version 1.0, 5/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   BookmarkTree                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class BookmarkTree extends JTree implements BookmarkListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTree            tree       = null ;
   private DefaultTreeModel model      = null ;
   private boolean          folderOnly = false ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method constructs everything!
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BookmarkTree () {
      //super() ;

      BookmarkSystem.addBookmarkListener (this ) ;
      setCellRenderer ( new BookmarkTreeCellRenderer() ) ;
      refreshBookmarkList() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method constructs everything!
    *  
    *  
    *
    *  @param folderOnly TRUE = only show folders, FALSE = show all bookmarks
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BookmarkTree ( boolean folderOnly ) {
      //super() ;

      this.folderOnly = folderOnly ;
      BookmarkSystem.addBookmarkListener (this ) ;
      setCellRenderer ( new BookmarkTreeCellRenderer() ) ;
      refreshBookmarkList() ;

   }





   /*------------------------------------------------------------*/
   /* refreshBookmarkList
    */
   /**
    *    This method will refresh all of the bookmarks
    *  in our tree view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshBookmarkList () {
      StringTreeNode   root      = null ;
      Vector           folders   = BookmarkSystem.getFolders() ;

      model = new DefaultTreeModel ( root = new StringTreeNode ( BookmarkSystem.getRootFolderName() ) ) ;
      insertBookmarkFolder ( model, root, BookmarkSystem.getRootFolder() ) ;
      setModel ( model ) ;
      setRootVisible ( false ) ;
      expandRow ( 0 ) ;

   } /* refreshBookmarkList */




   /*------------------------------------------------------------*/
   /* insertBookmarkFolder
    */
   /**
    *    This method will insert a folder into the
    *  DTree. It will also insert all bookmarks and
    *  subfolders in that folder, recursivly
    *
    *  @param model The DefaultTreeModel to use
    *  @param node The node to insert the folder under
    *  @param folder The folder to insert
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void insertBookmarkFolder ( DefaultTreeModel model, StringTreeNode node, Bookmark folder ) {
      try {
         Vector         subFolders = BookmarkSystem.getFolders   ( folder.getDescript() ) ;
         Vector         bookmarks  = BookmarkSystem.getBookmarks ( folder.getDescript() ) ;                    
         StringTreeNode newNode    = null ;

         node.add ( node = new StringTreeNode ( folder.getDescript(), folder ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, put in the sub folders                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( subFolders != null ) && ( subFolders.size() > 0 ) ) {
            int            i        = 0 ;
            int            size     = subFolders.size() ;

            while ( i < size ) {
               insertBookmarkFolder ( model, node, (Bookmark) subFolders.elementAt(i) ) ;
               i++ ;
            } /* while */

         } /* if */


         if ( !folderOnly ) {
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now put the regular bookmarks in                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( bookmarks != null ) && ( bookmarks.size() > 0 ) ) {
               int            i        = 0 ;
               int            size     = bookmarks.size() ;

               while ( i < size ) {
                  if ( !((Bookmark)bookmarks.elementAt(i)).isFolder() )
                     node.add ( newNode = new StringTreeNode ( ((Bookmark)bookmarks.elementAt(i)).getDescript(), 
                                                               bookmarks.elementAt(i) ) ) ;
                  i++ ;
               } /* while */

            } /* if */
         } /* if !foldersOnly */


      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

   } /* insertBookmarkFolder */


   public void finalize () {
      try {
         super.finalize() ;
         BookmarkSystem.removeBookmarkListener ( this ) ;
      } catch ( Throwable e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* bookmarkAdded
    */
   /**
    *    This method will be called when the BookmarkSystem
    *  has added a bookmark. We will put this bookmark
    *  into our window
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void bookmarkAdded ( Bookmark bookmark ) {
      StringTreeNode parentNode = nodeFromDescript ( bookmark.getParentFolder() ) ;
      StringTreeNode tempNode   = nodeFromDescript ( bookmark.getDescript() ) ;

      if ( tempNode != null )
         model.removeNodeFromParent ( tempNode ) ;

      if ( parentNode != null ) {
         StringTreeNode newNode = new StringTreeNode ( bookmark.getDescript(), bookmark ) ;

         model.insertNodeInto ( newNode, parentNode, parentNode.getChildCount() ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* bookmarkRemoved
    */
   /**
    *    This method will be called when the BookmarkSystem
    *  has removed a bookmark. We will also remove
    *  that bookmark
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void bookmarkRemoved ( Bookmark bookmark ) {
      StringTreeNode bookmarkNode = nodeFromDescript ( bookmark.getDescript() ) ;

      if ( bookmarkNode != null )
         model.removeNodeFromParent ( bookmarkNode ) ;

   }





   /*------------------------------------------------------------*/
   /* nodeFromDescript
    */
   /**
    *    This method will return a node from the tree
    *  that is displaying the description passed
    *  
    *
    *  @param descript The text to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private StringTreeNode nodeFromDescript ( String descript ) {
      int            size     = getRowCount() ;
      StringTreeNode retNode  = null ;
      StringTreeNode tempNode = null ;
      int            i        = 0 ;


      while ( ( i < size ) && ( retNode == null ) ) {
         tempNode = (StringTreeNode)getPathForRow(i).getLastPathComponent() ;

         if ( tempNode != null ) {
            if ( tempNode.toString() != null ) {
               if ( tempNode.toString().equals ( descript ) )
                  retNode = tempNode ;
            }
         }
         i++ ;
      }

      return( retNode ) ;
   }





   /*------------------------------------------------------------*/
   /* setFolderOnly
    */
   /**
    *    This method will set the folder only property.
    *  
    *  
    *
    *  @param folderOnly TRUE = Show only Bookmark Folders, FALSE = Show all bookmarks
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFolderOnly ( boolean folderOnly ) {
      this.folderOnly = folderOnly ;
      refreshBookmarkList() ;
   }
} /* BookmarkTree */
