package com.ibm.nzna.projects.qit.bookmark ;
/*-----------------------------------------------------------*/
/*   BookmarkTreeCellRenderer                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: BookmarkTreeCellRenderer            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The BookmarkTreeCellRenderer is just the 'Rubber Stamp' mechanism
 * of the JTree. 
 *
 * @version 1.0, 5/1/2000
 * @author  David Lentz
 */
import javax.swing.tree.* ;
import javax.swing.JTree ;
import java.awt.Component ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   BookmarkTreeCellRenderer                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class BookmarkTreeCellRenderer extends Object implements TreeCellRenderer {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private BookmarkTreeComponent filterComp   = null ;
   private int                     columnCount  = 0 ;
   private int[]                   columnWidths = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create the DefaultTreeRenderer
    *  
    *  
    *
    *  @param columnCount The number of columns that will be displayed
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public BookmarkTreeCellRenderer ( ) {
      filterComp = new BookmarkTreeComponent ( ) ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize the Column Width array                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( columnCount > 0 )
         columnWidths = new int[columnCount] ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the default column widths to 100                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      for ( int i = 0; i < columnCount; i++ )
         setColumnWidth ( i, 100 ) ;
   }





   /*------------------------------------------------------------*/
   /* setColumnWidth
    */
   /**
    *    This method will set the width in pixels
    *  of the column passed in. Columns are 0 based.
    *  
    *
    *  @param column The column to set the width for
    *  @param width The width in pixels
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnWidth ( int column, int width ) {
      columnWidths[column] = width ;
   }



   /*------------------------------------------------------------*/
   /* getTreeCellRendererComponent
    */
   /**
    *    This method will return our ccRendererComponent
    *  instead of the normal JLabel Component
    *  
    *
    *  @param The normal getTreeCellRenderer params
    *
    *  @return Component
    *//*---------------------------------------------------------*/
   public Component getTreeCellRendererComponent ( JTree jTree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus ) {
      filterComp.refreshData ( value, selected, expanded, leaf, row, hasFocus ) ;
      return( (Component) filterComp ) ;
   } /* getTreeCellRendererComponent */





   /*------------------------------------------------------------*/
   /* setPreferredWidth
    */
   /**
    *    This method will set the preferred width of
    *  all items in the list
    *  
    *
    *  @param width The preferred width
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPreferredWidth ( int width ) {
      filterComp.setPreferredWidth ( width ) ;
   }




   /*------------------------------------------------------------*/
   /* setFont
    */
   /**
    *    This method will set the font that we are displaying
    *  the data in
    *  
    *
    *  @param font The font to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFont ( java.awt.Font font ) {
      filterComp.setFont ( font ) ;
   }


} /* DefaultTreeRenderer */
