package com.ibm.nzna.projects.qit.chat ;
/*-----------------------------------------------------------*/
/*   ChatDlg                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ChatDlg                             */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import java.net.* ;
import java.util.* ;
import java.sql.* ;  
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This dialog allows one Quest Input user to communicate with another 
 *
 *
 * @version 1.0, 9/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ChatDlg                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class ChatDlg extends JDialog implements ChatListener, Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Chat messages                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String CHAT_BEGIN    = "CHAT_BEGIN" ;
   private static final String CHAT_SEND     = "CHAT_SEND" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Thread                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_READ_USERS = "ReadUsers" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton          pb_SEND        = null ;
   private DButton          pb_CANCEL      = null ;
   private Browser          chatOutput     = null ;
   private JTextField       entryField     = null ;
   private ButtonPanel      buttonPanel    = null ;
   private JScrollPane      listScroll     = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String[]   columnHeadings = {""} ;
   private ChatSession currentChat    = null ;
   private ChatUser   currentUser    = null ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the window,
    *  and will immediatly return
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ChatDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_MESSAGING ), false ) ;     
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SEND     = new DButton ( Str.getStr ( STR_SEND ) ) ;
      pb_CANCEL   = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      chatOutput  = GUISystem.createBrowser() ;
      entryField  = new JTextField() ;
      buttonPanel = new ButtonPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.setArrowImage  ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                   ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADd the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( listScroll ) ;
      contentPane.add ( chatOutput.getComponent() ) ;
      contentPane.add ( entryField ) ;
      buttonPanel.add ( pb_SEND ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      contentPane.add ( buttonPanel ) ;
      GUISystem.setPropertiesOnPanel ( contentPane ) ;


      setSize ( 375, 200 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
      setVisible ( true ) ;

      new Thread ( this, THREAD_READ_USERS ).start() ;

   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    Lays out all of the controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize () ;
      int       rowHeight = GUISystem.getRowHeight () ;

      super.doLayout() ;
      listScroll.setBounds ( 5, 5, 150, size.height-30 ) ;
      chatOutput.getComponent().setBounds ( 160, 5, size.width-170, size.height-65-rowHeight ) ;
      entryField.setBounds ( 160, size.height-57-rowHeight, size.width-170, rowHeight ) ;
      buttonPanel.setBounds ( 160, size.height-50, size.width-170, 25  ) ;
   }


   public void sendTest () {
      try {
         Socket socket = new Socket ( "9.37.217.49", ChatServer.LISTEN_PORT ) ;

         socket.getOutputStream().write ( new String ( "TESTTEST" ).getBytes() ) ;
         socket.getOutputStream().flush() ;

         socket.close() ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will do different operations. The
    *  operation done depends on the thread name
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_READ_USERS                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //if ( operation.equals ( THREAD_READ_USERS ) )
      //   readUsers() ;

   }

   /*------------------------------------------------------------*/
   /* enableInput
    */
   /**
    *    This method will either enable or disable
    *  input
    *  
    *
    *  @param enable TRUE = allow input, FALSE = do not allow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void enableInput ( boolean enable ) {
      pb_SEND.setEnabled ( enable ) ;
      entryField.setEnabled ( enable ) ;

      if ( enable )
         entryField.requestFocus() ;
   }





   /*------------------------------------------------------------*/
   /* dataReceived
    */
   /**
    *    This method will be called from the ChatServer
    *  when data is received from a previous session
    *  we have started
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object dataReceived ( ChatSession chatObject, String command, Object data ) {
//      Chat

      return( chatObject ) ;
   }

} /* ChatDlg */
