package com.ibm.nzna.projects.qit.chat ;  
/*-----------------------------------------------------------*/
/*   ChatSession                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ChatSession                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.net.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This object is used to communicate with another Input Tool. This object also 
 * represents a single communication session with another tool .
 *
 *
 * @version 1.0, 9/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ChatSession                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ChatSession extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * The IP Address of the machine that is being chatted with
     */
   private String ipAddr      = null ;

   /**
     * Internal unique number for ChatServer to use
     */
   private String uniqueId    = null ;

   /** 
     * The socket this communication is going over
     */
   private Socket socket      = null ;

   /**
     * Chat Listener is the listener who will be notified
     * of messages in this chat
     */
   private ChatListener chatListener = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a blank record
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ChatSession () {
   }




   /*------------------------------------------------------------*/
   /* setIPAddr
    */
   /**
    *    This method will set the IP Address
    *  
    *  
    *
    *  @param ipAddr The new ip address
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setIPAddr ( String ipAddr ) {
      this.ipAddr = ipAddr ;
      createUniqueId() ;
   }





   /*------------------------------------------------------------*/
   /* getIPAddr
    */
   /**
    *    This method will return the IP Address that
    *  this chat session belongs to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getIPAddr () {
      return( ipAddr ) ;
   }




   /*------------------------------------------------------------*/
   /* getUniqueId
    */
   /**
    *    This method will return the Unique Id of the
    *  chat session
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getUniqueId() {
      return( uniqueId ) ;
   }






   /*------------------------------------------------------------*/
   /* createUniqueId
    */
   /**
    *    This method will create the unique id for
    *  this chat. The UniqueId is the current time in milliseconds
    *  added to the I/P Address. For example:
    *
    *  If my I/P Address is 100.100.100.100, then 100100100100 is added
    *  to the current time in milliseconds
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createUniqueId () {
      long            key    = System.currentTimeMillis() ;

      key+=(new Integer ( Text.replaceAllStrInStr ( ipAddr, ".", "" ) ).intValue() ) ;
      uniqueId = ""+key ;
   }





   /*------------------------------------------------------------*/
   /* getSocket
    */
   /**
    *    This method will return the socket that
    *  we are talking with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected Socket getSocket() {
      return( socket ) ;
   }




   /*------------------------------------------------------------*/
   /* setSocket
    */
   /**
    *    This method will set the socket that we
    *  are talking with
    *  
    *
    *  @param socket The socket we are talking over
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setSocket ( Socket socket ) {
      this.socket = socket ;
   }





   /*------------------------------------------------------------*/
   /* getChatListener
    */
   /**
    *    This method will return the chat listener
    *  that is listening to this chat
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ChatListener getChatListener () {
      return( chatListener ) ;
   }




   /*------------------------------------------------------------*/
   /* setChatListener
    */
   /**
    *    This method will set the listener that will
    *  be notified of chat events
    *  
    *
    *  @param chatListener The listener that will be notified
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setChatListener ( ChatListener listener ) {
      this.chatListener = listener ;
   }

} /* ChatSession */
