package com.ibm.nzna.projects.qit.customer ;
/*-----------------------------------------------------------*/
/*   CustomerMachineRec                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomerMachineRec                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This object holds information about a Customer's specific Machine 
 *
 *
 * @version 1.0, 8/6/1999 
 * @author  David Lentz
 */
import java.sql.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.clio.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.gui.* ;
                                     

/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomerMachineRec                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomerMachineRec extends PersistentRec implements MultiListRow, AppConst, java.io.Serializable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String machineType       = "" ;
   private String model             = "" ;
   private String serialNumber      = "" ;
   private String brand             = "" ;
   private String family            = "" ;
   private String partNumber        = null ;
   private String operatingSystem   = "" ;
   private int    machineNum        = 0 ;
   private int    recNum            = 0 ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default constructor does nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomerMachineRec () {
   }





   /*------------------------------------------------------------*/
   /* setMachineType
    */
   /**
    *    This method will set the MachineType
    *  property
    *  
    *
    *  @param machineType The new MachineType
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMachineType ( String machineType ) {
      this.machineType = machineType ;
   }

   /*------------------------------------------------------------*/
   /* setModel
    */
   /**
    *    This method will set the Model
    *  property
    *  
    *
    *  @param model The new Model
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setModel ( String model ) {
      this.model = model ;
   }

   /*------------------------------------------------------------*/
   /* setSerialNumber
    */
   /**
    *    This method will set the SerialNumber
    *  property
    *  
    *
    *  @param serialNumber The new SerialNumber
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSerialNumber ( String serialNumber ) {
      this.serialNumber = serialNumber ;
   }


   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the Brand
    *  property
    *  
    *
    *  @param brand The new Brand
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( String brand ) {
      this.brand = brand ;
   }



   /*------------------------------------------------------------*/
   /* setFamily
    */
   /**
    *    This method will set the Family
    *  property
    *  
    *
    *  @param family The new Family
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFamily ( String family ) {
      this.family = family ;
   }




   /*------------------------------------------------------------*/
   /* getMachineType
    */
   /**
    *    This method will get the MachineType
    *  property
    *  
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getMachineType ( ) {
      return( machineType ) ;
   }

   /*------------------------------------------------------------*/
   /* getModel
    */
   /**
    *    This method will get the Model
    *  property
    *  
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getModel () {
      return( model ) ;
   }

   /*------------------------------------------------------------*/
   /* getSerialNumber
    */
   /**
    *    This method will get the SerialNumber
    *  property
    *  
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getSerialNumber ( ) {
      return( serialNumber ) ;
   }


   /*------------------------------------------------------------*/
   /* getBrand
    */
   /**
    *    This method will get the Brand
    *  property
    *  
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getBrand ( ) {
      return( brand ) ;
   }



   /*------------------------------------------------------------*/
   /* getFamily
    */
   /**
    *    This method will get the Family
    *  property
    *  
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getFamily () {
      return( family ) ;
   }




   /*------------------------------------------------------------*/
   /* getMachineNum
    */
   /**
    *    This method will return the machine
    *  number for this machine
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getMachineNum () {
      return( machineNum ) ;
   }





   /*------------------------------------------------------------*/
   /* getRecNum
    */
   /**
    *    This method will return the record number
    *  of this machine
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getRecNum () {
      return( recNum ) ;
   }





   /*------------------------------------------------------------*/
   /* getBrandGroupInd
    */
   /**
    *    This method will return the brand group
    *  indicator for the brand group of this machine
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getBrandGroupInd () {
      String    rtnName = "getBrandGroupInd" ;
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.CLIO, "getBrandGroupInd", 5 ) ;
      int       bgInd   = 0 ;

      try {
         Statement  stmt = sql.createStatement() ;
         ResultSet  rs   = null ;

         rs = stmt.executeQuery ( "SELECT A.BRANDIND FROM "+
                                  "CLIO.BRANDS A, "+
                                  "CLIO.TYPEBRAND B, "+
                                  "CLIO.TYPEGROUP C "+
                                  "WHERE B.DESCRIPT = '"+brand+"' AND "+
                                  "      B.BRANDID  = A.BRANDID AND "+
                                  "      C.DESCRIPT = '"+family+"' AND "+
                                  "      C.GROUPID  = A.GROUPID "+
                                  "FOR FETCH ONLY" ) ;
         

         if ( rs.next() )
            bgInd = rs.getInt(1) ;

      } catch ( Exception e ) {
      }
      sql.close() ;
      return( bgInd ) ;
   }





   /*------------------------------------------------------------*/
   /* generateMachineNum
    */
   /**
    *    This method will generate a machine 
    *  num only if the current machine num is 0
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void generateMachineNum () {
      if ( machineNum == 0 )
         machineNum = Counter.getCounter ( LogSystem.getInstance(), "MACHINENUM" ) ;
   }




   /*------------------------------------------------------------*/
   /* generateRecNum
    */
   /**
    *    This method will generate a record number
    *  only if the current rec num is 0
    *  
    *
    *  @param custNum This is needed to get the max rec num from CLIO.CUSTMACH
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean generateRecNum ( int custNum ) {
      String    rtnName  = "generateRecNum" ;
      SQLMethod sql      = new SQLMethod ( DatabaseSystem.CLIO, "generateRecNum", 5 ) ;
      boolean   rc       = false;

      recNum = 0 ;

      try {
         Statement stmt = sql.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT RECNUM FROM CLIO.CUSTMACH WHERE CUSTNUM = "+custNum+" ORDER BY RECNUM" ) ;

         while ( rs.next() )
            recNum = rs.getInt(1) ;

         recNum++ ;

         rs.close() ;
      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sql.close() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setPartNumber
    */
   /**
    *    This method will set the part number
    *  of this option
    *  
    *
    *  @param partNumber The part number to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPartNumber ( String partNumber ) {
      this.partNumber = partNumber ;
   }





   /*------------------------------------------------------------*/
   /* getPartNumber
    */
   /**
    *    This method will return the part number
    *  of this option
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getPartNumber ( ) {
      return( partNumber ) ;
   }





   /*------------------------------------------------------------*/
   /* getOperatingSystem
    */
   /**
    *    This method will return the Operating System
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getOperatingSystem () {
      return( operatingSystem ) ;
   }





   /*------------------------------------------------------------*/
   /* setOperatingSystem
    */
   /**
    *    This method will set our operating system
    *  properties
    *  
    *
    *  @param operatingSystem The new operating sytem
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setOperatingSystem ( String operatingSystem ) {
      this.operatingSystem = operatingSystem ;
   }





   /*------------------------------------------------------------*/
   /* setMachineNum
    */
   /**
    *    THis method will set this machine's
    *  machine number
    *  
    *
    *  @param machineNum The new machine number
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMachineNum ( int machineNum ) {
      this.machineNum = machineNum ;
   }

   /*------------------------------------------------------------*/
   /* setRecNum
    */
   /**
    *    This method will set this phone' rec
    *  num
    *  
    *
    *  @param recNum The new Country Code
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRecNum ( int recNum ) {
      this.recNum = recNum ;
   }


   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the data that should
    *  be displayed inside of a MultiList control
    *  
    *
    *  @param column The column number
    *
    *  @return The data to put in the column passed
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            if ( getRecStatus() == REC_DELETE ) {
               if ( partNumber != null )
                  retObj = "X - "+partNumber+" (Deleted)" ;
               else
                  retObj = "X - "+machineType+" (Deleted)" ;
            } else {
               if ( partNumber != null )
                  retObj = partNumber ;
               else
                  retObj = machineType ;
            }
            break ;

         case 1 :
            if ( partNumber != null )
               retObj = machineType ;
            else
               retObj = model ;
            break ;

         case 2 :
            retObj = serialNumber ;
            break ;

         case 3 :
            retObj = brand ;
            break ;

         case 4 :
            retObj = family ;
            break ;

         case 5 :
            retObj = operatingSystem ;
            break ;
      }
      return( retObj ) ;
   }

} /* CustomerMachineRec */
