package com.ibm.nzna.projects.qit.customer ;
/*-----------------------------------------------------------*/
/*   CustomerRec                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomerRec                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This record holds all information about a single Customer in the CLIO database 
 *
 *
 * @version 1.0, 8/5/1999 
 * @author  David Lentz
 */
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.shared.pom.*;
import com.ibm.nzna.shared.db.*;
import com.ibm.nzna.shared.util.*;
import com.ibm.nzna.shared.gui.*;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.clio.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomerRec                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomerRec extends PersistentRec implements MultiListRow, AppConst, java.io.Serializable, CustomerConstants {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       custNum           = 0 ;
   private String    firstName         = "" ;
   private String    middleInitial     = "" ;
   private String    lastName          = "" ;
   private String    suffix            = "" ;
   private String    salutation        = "" ;
   private String    eMailAddress      = "" ;
   private boolean   validEMail        = false ;
   private String    userId            = "" ;
   private String    password          = "" ;
   private String    prettyChangedtime = "" ;
   private String    prettyDBUser      = "" ;
   private String    dbUser            = "" ;
   private String    changedTime       = "" ;
   private String    company           = "" ;
   private String    company2          = "" ;
   private String    question          = "" ;
   private String    answer            = "" ;
   private int       callCount         = 0 ;


   private Vector    phoneVec          = null ;
   private Vector    addressVec        = null ;
   private Vector    machineVec        = null ;
   private Vector    optionVec         = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a blank customer record
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomerRec () {
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a CustomerRec
    *  with the customer number passed in
    *  
    *
    *  @param custNum  The Customer Number to use
    *  @param  
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomerRec ( int custNum ) {
      setCustNum ( custNum ) ;
   }






   /*------------------------------------------------------------*/
   /* setCustNum
    */
   /**
    *    This method will set the customer number
    *  for this customer
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCustNum ( int custNum ) {
      this.custNum = custNum ;
   }





   /*------------------------------------------------------------*/
   /* getCustNum
    */
   /**
    *    This method will return the customer number
    *  of this customer
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getCustNum () {
      return( custNum ) ;
   }


   /*------------------------------------------------------------*/
   /* setFieldData
    */
   /**
    *    This method will set 1 property field with
    *  the data passed in
    *  
    *
    *  @param fieldId The field to set
    *  @param data The data to set the field to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFieldData ( int fieldId, Object data ) {

      switch ( fieldId ) {
         case CUSTFIELD_FIRSTNAME :
            if ( ( data != null ) && ( data instanceof String ) )
               firstName = ((String)data).trim() ;
            break ;

         case CUSTFIELD_VALIDEMAIL :
            if ( ( data != null ) && ( data instanceof String ) ) {
               if ( ((String)data).equals ( "Y" ) )
                  validEMail = true ;
               else
                  validEMail = false ;
            }

            else if ( ( data != null ) && ( data instanceof Boolean ) ) {
               validEMail = ((Boolean)data).booleanValue() ;
            }
            break ;

         case CUSTFIELD_MIDDLEINIT :
            if ( ( data != null ) && ( data instanceof String ) )
               middleInitial = ((String)data).trim() ;
            break ;
         case CUSTFIELD_LASTNAME :
            if ( ( data != null ) && ( data instanceof String ) )
               lastName = ((String)data).trim() ;
            break ;
         case CUSTFIELD_SUFFIX :
            if ( ( data != null ) && ( data instanceof String ) )
               suffix = ((String)data).trim() ;
            break ;
         case CUSTFIELD_SALUTATION :
            if ( ( data != null ) && ( data instanceof String ) )
               salutation = ((String)data).trim() ;
            break ;
         case CUSTFIELD_EMAILADDR :
            if ( ( data != null ) && ( data instanceof String ) )
               eMailAddress = ((String)data).trim() ;
            break ;

         case CUSTFIELD_USERID :
            if ( ( data != null ) && ( data instanceof String ) )
               userId = ((String)data).trim() ;
            break ;

         case CUSTFIELD_PASSWORD :
            if ( ( data != null ) && ( data instanceof String ) )
               password = ((String)data).trim() ;
            break ;

         case CUSTFIELD_COMPANY :
            if ( ( data != null ) && ( data instanceof String ) )
               company = ((String)data).trim() ;
            break ;

         case CUSTFIELD_COMPANY2 :
            if ( ( data != null ) && ( data instanceof String ) )
               company2 = ((String)data).trim() ;
            break ;

         case CUSTFIELD_CHANGEDTIME :
            changedTime = (String) data ;
            prettyChangedtime = DateSystem.prettyDateFromStamp ( changedTime ) ;
            break ;

         case CUSTFIELD_DBUSER : 
            dbUser = (String)data ;
            prettyDBUser = UserSystem.getNameFromUserId ( dbUser ) ;
            break ;



      }
   }





   /*------------------------------------------------------------*/
   /* getFieldData
    */
   /**
    *    This method will return the data that is
    *  associated to the field passed
    *  
    *
    *  @param fieldId The ID of the field to get ( CUSTFIELD_FIRSTNAME, etc.. )
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getFieldData ( int fieldId ) {
      Object retObj = null ;

      switch ( fieldId ) {
         
         case CUSTFIELD_VALIDEMAIL :
            retObj = new Boolean ( validEMail ) ;
            break ;

         case CUSTFIELD_FIRSTNAME :
            retObj = firstName ;
            break ;
         case CUSTFIELD_MIDDLEINIT :
            retObj = middleInitial ;
            break ;
         case CUSTFIELD_LASTNAME :
            retObj = lastName ;
            break ;
         case CUSTFIELD_SUFFIX :
            retObj = suffix ;
            break ;
         case CUSTFIELD_SALUTATION :
            retObj = salutation ;
            break ;
         case CUSTFIELD_EMAILADDR :
            retObj = eMailAddress ;
            break ;

         case CUSTFIELD_USERID :
            retObj = userId ;
            break ;

         case CUSTFIELD_PASSWORD :
            retObj = password ;
            break ;

         case CUSTFIELD_ALLADDRESSES :
            retObj = addressVec ;
            break ;

         case CUSTFIELD_ALLMACHINE :
            retObj = machineVec ;
            break ;

         case CUSTFIELD_ALLPHONE :
            retObj = phoneVec ;
            break ;

         case CUSTFIELD_ALLOPTION :
            retObj = optionVec ;
            break ;

         case CUSTFIELD_COMPANY :
            retObj = company ;
            break ;

         case CUSTFIELD_CHANGEDTIME :
            retObj = changedTime ;
            break ;

         case CUSTFIELD_PRETTYCHANGEDTIME :
            retObj = prettyChangedtime ;
            break ;
      }

      return( retObj ) ;

   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the data that should
    *  be displayed inside of a MultiList control
    *  
    *
    *  @param column The column number
    *
    *  @return The data to put in the column passed
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = new String ( ""+custNum ) ;
            break ;

         case 1 :
            retObj = new String ( firstName+" "+middleInitial+" "+lastName ) ;
            break ;

         case 2 :
            retObj = eMailAddress ;
            break ;

         case 3 :
            retObj = userId ;
            break ;

         case 4 :
            retObj = prettyChangedtime ;
            break ;

         case 5 :
            retObj = prettyDBUser ;
            break ;
      }

      return( retObj ) ;
   }


   /*------------------------------------------------------------*/
   /* addAddressRec
    */
   /**
    *    This method will add a CustomerAddressRec
    *  to this object
    *  
    *
    *  @param addressRec The Address record to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addAddressRec ( CustomerAddressRec addressRec ) {
      if ( addressVec == null )
         addressVec = new Vector ( 1, 1 ) ;
      addressVec.addElement ( addressRec ) ;
   }

   /*------------------------------------------------------------*/
   /* addMachineRec
    */
   /**
    *    This method will add a CustomerMachineRec
    *  to this object
    *  
    *
    *  @param machineRec The Machine record to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addMachineRec ( CustomerMachineRec machineRec ) {
      if ( machineVec == null )
         machineVec = new Vector ( 1, 1 ) ;
      machineVec.addElement ( machineRec ) ;
   }

   /*------------------------------------------------------------*/
   /* addOptionRec
    */
   /**
    *    This method will add a CustomerMachineRec
    *  to this object as an option
    *  
    *
    *  @param machineRec The Machine record to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addOptionRec ( CustomerMachineRec machineRec ) {
      if ( optionVec == null )
         optionVec = new Vector ( 1, 1 ) ;
      optionVec.addElement ( machineRec ) ;
   }


   /*------------------------------------------------------------*/
   /* addPhoneRec
    */
   /**
    *    This method will add a CustomerPhoneRec
    *  to this object
    *  
    *
    *  @param phoneRec The Phone record to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addPhoneRec ( CustomerPhoneRec phoneRec ) {
      if ( phoneVec == null )
         phoneVec = new Vector ( 1, 1 ) ;
      phoneVec.addElement ( phoneRec ) ;
   }






   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will perform a comparison on the
    *  object passed in. If the object passed is a String
    *  object, it will be assumed the eMail address to compare.
    *
    *  @param Object The object to compare to
    *  @param None
    *  @param None
    *
    *  @return TRUE equal, FALSE not equal
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {
      boolean equals = false ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check eMail Address                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( object instanceof String )
         equals = ((String)object).equals ( eMailAddress ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check whole record                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( object instanceof CustomerRec ) {
         CustomerRec compareRec = (CustomerRec) object ;

         try {
            equals = ( ( compareRec.eMailAddress.equals ( eMailAddress ) ) &&
                       ( compareRec.firstName.equals ( firstName ) ) &&
                       ( compareRec.lastName.equals ( lastName ) ) &&
                       ( compareRec.suffix.equals ( suffix ) ) &&
                       ( compareRec.salutation.equals ( salutation ) ) &&
                       ( compareRec.userId.equals ( userId ) ) &&
                       ( compareRec.password.equals ( password ) ) ) ;

         } catch ( Exception e ) {
            equals = false ;
         }

      }

      return( equals ) ;
   }






   /*------------------------------------------------------------*/
   /* existsInDatabase
    */
   /**
    *    This method will try to determine if this
    *  customer exists in the CLIO database. It
    *  will search by eMailAddress only.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return 1 = Found, 0 = Not Found, 2 = error
    *//*---------------------------------------------------------*/
   public int existsInDatabase () {
      SQLMethod sql = new SQLMethod ( DatabaseSystem.CLIO, "existsInDatabase", 5 ) ;
      int       rc      = 0 ;

      try {
         Statement stmt = sql.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT CUSTNUM "+
                                              "FROM CLIO.CUSTNUM "+
                                              "WHERE EMAILADDR = '"+eMailAddress+"' "+
                                              "FOR FETCH ONLY" ) ;

         if ( rs.next() )
            rc = 1 ;

         rs.close() ;
      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         rc=2 ;
      }

      sql.close() ;

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* saveToDatabase
    */
   /**
    *    This method will save this customer to
    *  the CLIO database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if the save was good, FALSE if it went bad
    *//*---------------------------------------------------------*/
   public boolean saveToDatabase () {
      SQLMethod sql = new SQLMethod ( DatabaseSystem.CLIO, "existsInDatabase", 5 ) ;
      String    rtnName   = "saveToDatabase" ;
      boolean   rc        = false ;

      try {
         Statement stmt    = sql.createStatement() ;
         String    sqlStmt = null ;

         switch ( getRecStatus() ) {
            case REC_DELETE :
               if ( custNum > 0 )
                  sqlStmt = "UPDATE CLIO.CUSTNUM SET EDITFLAG = 'N', SHOW = 'N' WHERE CUSTNUM = "+custNum ;
               break ;

            case REC_INSERT :
               if ( custNum == 0 )
                  custNum = Counter.getCounter ( LogSystem.getInstance(), "CUSTNUM" ) ;
               sqlStmt = "INSERT INTO CLIO.CUSTNUM "+
                         "( CUSTNUM,        PARENTCUSTNUM,        PINNUM,                      SALUTATION,        "+
                         "  FNAME,          MI,                   LNAME,                       SUFFIX,            "+
                         "  LANGUAGEIND,    EMAILADDR,            CALLCOUNT,                   ESUPCOUNT,         "+
                         "  SOURCECUSTOMER, CREATEDATE,           LASTUPDATE,                  LASTCALLDATE,      "+
                         "  PINNUMREQUIRED, ORGANIZATION,         CUSTTYPE,                    EDITFLAG,          "+
                         "  SHOW,           VALIDEMAIL,           DBUSER,                      CHANGEDTIME )      "+
                         "VALUES                                                                                  "+
                         "( "+custNum+",    NULL,                 NULL,                        '"+salutation+"',  "+
                         " '"+firstName+"', '"+middleInitial+"', '"+lastName+"',               '"+suffix+"',      "+
                         "  1,              '"+eMailAddress+"',   0,                           0,                 "+
                         " 'CLIOTAB',       CURRENT DATE,         CURRENT DATE,                CURRENT DATE,      "+
                         " 'N',             'N',                  0,                           'Y',               "+
                         " 'Y',             'Y',                 '"+UserSystem.getUserId()+"', CURRENT TIMESTAMP )" ;
               break ;

            case REC_UPDATE :

               /* If we don't have a customer number, try to get one  */
               if ( ( custNum == 0 ) && 
                    ( eMailAddress != null ) && 
                    ( eMailAddress.length() > 0 ) )
                  custNum = findCustNum() ;

               if ( custNum > 0 )
                  sqlStmt = "UPDATE CLIO.CUSTNUM "+
                            "SET PARENTCUSTNUM  = NULL,                         "+
                            "    PINNUM         = NULL,                         "+
                            "    SALUTATION     = '"+salutation+"',             "+
                            "    FNAME          = '"+firstName+"',              "+
                            "    MI             = '"+middleInitial+"',          "+
                            "    LNAME          = '"+lastName+"',               "+
                            "    SUFFIX         = '"+suffix+"',                 "+
                            "    LANGUAGEIND    = 1,                            "+
                            "    EMAILADDR      = '"+eMailAddress+"',           "+
                            "    SOURCECUSTOMER = 'CLIOTAB',                    "+
                            "    LASTUPDATE     = CURRENT DATE,                 "+
                            "    PINNUMREQUIRED = 'N',                          "+
                            "    ORGANIZATION   = 'N',                          "+
                            "    CUSTTYPE       = 0,                            "+
                            "    DBUSER         = '"+UserSystem.getUserId()+"', "+
                            "    CHANGEDTIME    = CURRENT TIMESTAMP             "+
                            "WHERE CUSTNUM = "+custNum ;
               break ;

            default:
               rc=true ;
               break ;
         }

         if ( sqlStmt != null ) {
            stmt.executeUpdate ( sqlStmt ) ;

            if ( ( saveCustMachines ( stmt ) ) &&
                 ( saveCustOptions  ( stmt ) ) &&
                 ( saveCustAddress  ( stmt ) ) &&
                 ( saveCustPhone    ( stmt ) ) &&
                 ( saveCustCompany  ( stmt ) ) &&
                 ( saveCustUserId   ( stmt ) ) )
               rc = true ;
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sql.close() ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* findCustNum
    */
   /**
    *    This method will attempt to locate the 
    *  customer number of this customer by
    *  searching by eMail Address
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return long, -1 if error, 0 if not found
    *//*---------------------------------------------------------*/
   public int findCustNum () {
      String      rtnName     = "findCustNum" ;
      SQLMethod   sql   = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;
      int         rc          = 0 ;

      try {
         Statement stmt = sql.createStatement () ;
         ResultSet rs   = stmt.executeQuery ( "SELECT CUSTNUM FROM CLIO.CUSTNUM WHERE EMAILADDR = '"+eMailAddress+"' FOR FETCH ONLY" ) ;

         if ( rs.next() )
            rc = rs.getInt ( 1 ) ;

         rs.close() ;

      } catch ( Exception e ) {
         sql.rollBack () ;
         LogSystem.log ( 1, e ,false ) ;
         rc=-1 ;
      }

      sql.close() ;
      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* findCustNumByUserId
    */
   /**
    *    This method will attempt to locate the 
    *  customer number of this customer by
    *  searching by User Id
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return long, -1 if error, 0 if not found, customer number if found
    *//*---------------------------------------------------------*/
   public static int findCustNumByUserId ( String userId ) {
      String      rtnName     = "findCustNumByUserId ( "+userId+" )" ;
      SQLMethod   sql   = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;
      int         rc          = 0 ;

      try {
         Statement stmt = sql.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT CUSTNUM FROM CLIO.CUSTPASSWD WHERE CUSTUSERID = '"+userId+"' FOR FETCH ONLY" ) ;

         if ( rs.next() )
            rc = rs.getInt ( 1 ) ;

         rs.close() ;

      } catch ( Exception e ) {
         sql.rollBack () ;
         LogSystem.log ( 1, e, false ) ;
         rc=-1 ;
      }

      sql.close() ;

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* saveCustMachines
    */
   /**
    *    This method will save all machine information
    *  information to the database
    *  
    *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveCustMachines ( Statement stmt ) throws Exception {
      boolean  rc       = false ;

      if ( ( machineVec != null ) && ( machineVec.size() > 0 ) ) {
         int                i       = 0 ;
         int                length  = machineVec.size() ;
         CustomerMachineRec machRec = null ;
         String             strStmt = null ;
         int                bgInd   = 0 ;

         while ( i < length ) {

            machRec = (CustomerMachineRec) machineVec.elementAt(i) ;                    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get the Brand Group Indicator                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            bgInd = machRec.getBrandGroupInd() ;                           

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we can't find it, return an error                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( bgInd == 0 ) {
               GUISystem.printBox ( Str.getStr ( STR_ERROR ),
                                    Str.getStr ( STR_CANNOTLOCATECLIOBRANDFAMILY )+"\nBrand:"+machRec.getBrand()+"\nFamily:"+machRec.getFamily() ) ;
               return( false ) ;
            }

            strStmt = null ;
            switch ( machRec.getRecStatus() ) {
               case REC_DELETE :
                  strStmt = "DELETE FROM CLIO.CUSTMACH "+
                            "WHERE MACHINENUM = "+machRec.getMachineNum()+" AND "+
                            "      CUSTNUM    = "+custNum ;
                  break ;

               case REC_INSERT :
                  if ( machRec.getMachineNum() == 0 )
                     machRec.generateMachineNum() ;

                  if ( machRec.getRecNum() == 0 )
                     machRec.generateRecNum(custNum) ;

                  if ( ( machRec.getMachineNum() > 0 ) && ( machRec.getRecNum() > 0 ) ) {
                     strStmt = "INSERT INTO CLIO.CUSTMACH "+
                               "( CUSTNUM,    RECNUM,     MACHINE,        MODEL,       "+
                               "  SERIAL,     BRANDIND,   SYSPRODNUM,     COMPRODNUM,  "+
                               "  MACHINENUM, SOURCEMACHINE, HARDCOPYPOP, SDTFLAG, "+
                               "  DBUSER,     CHANGEDTIME )               "+
                               "VALUES                                                                                                               "+
                               "( "+custNum+", "+
                               "  "+machRec.getRecNum()+", "+
                               " '"+machRec.getMachineType()+"', "+
                               " '"+machRec.getModel()+"', "+
                               " '"+machRec.getSerialNumber()+"', "+
                               " "+bgInd+", "+
                               " '"+machRec.getMachineType()+"-"+machRec.getModel()+"', "+
                               " '"+machRec.getMachineType()+"-"+machRec.getModel()+"', "+
                               " "+machRec.getMachineNum()+", "+
                               " 'CUST', "+
                               " 'N', "+
                               " ' ', "+
                               " '"+UserSystem.getUserId()+"', "+
                               " CURRENT TIMESTAMP)" ;
                  } else
                     LogSystem.log ( 1, "CustomerRec - Cannot generate Machine Num or Rec Num for Insert. CUSTNUM:"+custNum ) ;
                  break ;

               case REC_UPDATE :
                  if ( machRec.getMachineNum() > 0 ) {
                     strStmt = "UPDATE CLIO.CUSTMACH "+
                               "SET MACHINE      = '"+machRec.getMachineType()+"', "+
                               "    MODEL        = '"+machRec.getModel()+"', "+
                               "    SERIAL       = '"+machRec.getSerialNumber()+"', "+
                               "    BRANDIND     = "+bgInd+", "+
                               "    SYSPRODNUM   = '"+machRec.getMachineType()+"-"+machRec.getModel()+"', "+
                               "    DBUSER       = '"+UserSystem.getUserId()+"', "+
                               "    CHANGEDTIME  = CURRENT TIMESTAMP "+
                               "WHERE CUSTNUM    = "+custNum+" AND "+
                               "      MACHINENUM = "+machRec.getMachineNum() ;
                  }
                  break ;
            } /* switch  */


            if ( strStmt != null )
               stmt.executeUpdate ( strStmt ) ;



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, now make the entry in CLIO.CUSTCOMPONENT            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            switch ( machRec.getRecStatus() ) {
               case REC_DELETE :
                  strStmt = "DELETE FROM CLIO.CUSTCOMPONENT WHERE MACHINENUM = "+machRec.getMachineNum()+
                            " AND CUSTNUM = "+custNum ;
                  break ;

               case REC_INSERT :
                  strStmt = "INSERT INTO CLIO.CUSTCOMPONENT "+
                            "( CUSTNUM,       RECNUM,       BRANDIND,    COMPONENTIND,  "+
                            "  FIRSTCALLDATE, HARDCOPYPOP,  MACHINENUM,  PRODUCTNUM,    "+
                            "  SDTFLAG,       SERVICEIND,   WARRANTY,    DBUSER,        "+
                            "  CHANGEDTIME ) "+
                            "VALUES "+
                            "( "+custNum+", "+
                            "  "+machRec.getRecNum()+", "+
                            "  0, "+
                            "  2, "+
                            "  CURRENT DATE, "+
                            "  'N',          "+
                            "  "+machRec.getMachineNum()+", "+
                            "  '"+machRec.getOperatingSystem()+"', "+
                            "  ' ', "+
                            "  2, "+
                            "  365, "+
                            "  USER, "+
                            "  CURRENT TIMESTAMP )" ;
                  break ;

               case REC_UPDATE :
                  strStmt = "UPDATE CLIO.CUSTCOMPONENT "+
                            "SET PRODUCTNUM   = '"+machRec.getOperatingSystem()+"', "+
                            "    DBUSER       = USER, "+
                            "    CHANGEDTIME  = CURRENT TIMESTAMP "+
                            "WHERE CUSTNUM    = "+custNum+" AND "+
                            "      MACHINENUM = "+machRec.getMachineNum()+" AND "+
                            "      RECNUM     = "+machRec.getRecNum() ;
                  break ;

               default:
                  strStmt = null ;
                  break ;
            }



            i++ ;

            if ( strStmt != null )
               stmt.executeUpdate ( strStmt ) ;

         }

         rc = true ;

      } /* if machineVec  */
      else
         rc = true ;

      return( rc ) ;
   } /* saveCustMachines  */


   /*------------------------------------------------------------*/
   /* saveCustOptions
    */
   /**
    *    This method will save all machine information
    *  information to the database
    *  
    *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveCustOptions ( Statement stmt ) throws Exception {
      boolean   rc        = false ;

      if ( ( optionVec != null ) && ( optionVec.size() > 0 ) ) {
         int                i       = 0 ;
         int                length  = optionVec.size() ;
         CustomerMachineRec machRec = null ;
         String             strStmt = null ;

         while ( i < length ) {

            machRec = (CustomerMachineRec) optionVec.elementAt(i) ;

            strStmt = null ;
            switch ( machRec.getRecStatus() ) {
               case REC_DELETE :
                  strStmt = "DELETE FROM CLIO.CUSTOPTION "+
                            "WHERE RECNUM     = "+machRec.getRecNum()+" AND "+
                            "      CUSTNUM    = "+custNum ;
                  break ;

               case REC_INSERT :
                  if ( machRec.getMachineNum() == 0 )
                     machRec.generateMachineNum() ;

                  if ( machRec.getRecNum() == 0 )
                     machRec.generateRecNum(custNum) ;

                  if ( ( machRec.getMachineNum() > 0 ) && ( machRec.getRecNum() > 0 ) ) {
                     strStmt = "INSERT INTO CLIO.CUSTOPTION "+
                               "( CUSTNUM,    RECNUM,     MACHINE,     SERIAL,  "+
                               "  BRANDIND,   PARTNUM,    HARDCOPYPOP, DBUSER,  "+
                               "  CHANGEDTIME )                                 "+
                               "VALUES                                                                                                               "+
                               "( "+custNum+", "+
                               "  "+machRec.getRecNum()+", "+
                               " '"+machRec.getMachineType()+"', "+
                               " '"+machRec.getSerialNumber()+"', "+
                               " "+machRec.getBrandGroupInd()+", "+
                               " '"+machRec.getPartNumber()+"', "+
                               " 'N', "+
                               " '"+UserSystem.getUserId()+"', "+
                               " CURRENT TIMESTAMP)" ;
                  } else
                     LogSystem.log ( 1, "CustomerRec - Cannot generate Machine Num or Rec Num for Insert. CUSTNUM:"+custNum ) ;
                  break ;

               case REC_UPDATE :
                  if ( machRec.getMachineNum() > 0 ) {
                     strStmt = "UPDATE CLIO.CUSTOPTION "+
                               "SET MACHINE      = '"+machRec.getMachineType()+"', "+
                               "    PARTNUMBER   = '"+machRec.getPartNumber()+"', "+
                               "    SERIAL       = '"+machRec.getSerialNumber()+"', "+
                               "    DBUSER       = '"+UserSystem.getUserId()+"', "+
                               "    CHANGEDTIME  = CURRENT TIMESTAMP "+
                               "WHERE CUSTNUM    = "+custNum+" AND "+
                               "      RECNUM     = "+machRec.getRecNum() ;
                  }
                  break ;
            } /* switch  */


            if ( strStmt != null ) {
               stmt.executeUpdate ( strStmt ) ;
               machRec.updateRecStatus ( REC_NOCHANGE ) ;
            }
            i++ ;
         }

         rc = true ;

      } /* if optionVec  */
      else
         rc = true ;

      return( rc ) ;
   } /* saveCustOptions  */



   /*------------------------------------------------------------*/
   /* forceSaveCustPhone
    */
   /**
    *    This method will force the update of the
    *  first phone in the phoneVec
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   //public boolean forceSaveCustPhone () {
   //   boolean rc = false ;

   //   if ( ( phoneVec != null ) && ( phoneVec.size() > 0 ) ) {
   //      CustomerPhoneRec phoneRec = (CustomerPhoneRec) phoneVec.elementAt(0) ;

   //      if ( phoneRec != null ) {
   //         phoneRec.updateRecStatus ( REC_DELETE ) ;
   //         rc = saveCustPhone () ;
   //         if ( rc ) {
   //            phoneRec.updateRecStatus ( REC_INSERT ) ;
   //            rc = saveCustPhone () ;
   //         }
   //      }
   //   }
   //   return( rc ) ;
   //}


   /*------------------------------------------------------------*/
   /* forceSaveCustAddress
    */
   /**
    *    This method will force the update of the
    *  first address in the addressVec
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   //public boolean forceSaveCustAddress () {
   //   boolean rc = false ;

   //   if ( ( addressVec != null ) && ( addressVec.size() > 0 ) ) {
   //      CustomerAddressRec addressRec = (CustomerAddressRec) addressVec.elementAt(0) ;

   //      if ( addressRec != null ) {
   //         addressRec.updateRecStatus ( REC_DELETE ) ;
   //         rc = saveCustAddress () ;
   //         if ( rc ) {
   //            addressRec.updateRecStatus ( REC_INSERT ) ;
   //            rc = saveCustAddress () ;
   //         }
   //      }
   //   }
   //   return( rc ) ;
   //}





   /*------------------------------------------------------------*/
   /* saveCustAddress
    */
   /**
    *    This method will save all customer
    *  addresses to the CLIO.CUSTADDRESS
    *  table
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveCustAddress ( Statement stmt ) throws Exception {
      boolean   rc        = false ;

      if ( ( addressVec != null ) && ( addressVec.size() > 0 ) ) {
         int                i       = 0 ;
         int                length  = addressVec.size() ;
         CustomerAddressRec addressRec = null ;
         String             strStmt = null ;

         while ( i < length ) {

            addressRec = (CustomerAddressRec) addressVec.elementAt(i) ;

            strStmt = null ;
            switch ( addressRec.getRecStatus() ) {
               case REC_DELETE :
                  strStmt = "DELETE FROM CLIO.CUSTADDRESS "+
                            "WHERE RECNUM     = "+addressRec.getRecNum()+" AND "+
                            "      CUSTNUM    = "+custNum ;
                  break ;

               case REC_INSERT :

                  if ( addressRec.getRecNum() == 0 )
                     addressRec.generateRecNum(custNum) ;

                  if ( addressRec.getRecNum() > 0 ) {
                     strStmt = "INSERT INTO CLIO.CUSTADDRESS "+
                               "( CUSTNUM,    RECNUM,      ADDRESS1,    ADDRESS2,    "+
                               "  ADDRESS3,   ADDRESSIND,  CITY,        STATE,       "+
                               "  ZIPCODE,    COUNTRYCODE, DBUSER,      CHANGEDTIME )"+
                               "VALUES                                                                                                               "+
                               "( "+custNum+", "+
                               "  "+addressRec.getRecNum()+", "+
                               " '"+addressRec.getAddress1()+"', "+
                               " '"+addressRec.getAddress2()+"', "+
                               " '"+addressRec.getAddress3()+"', "+
                               " 1, "+
                               " '"+addressRec.getCity()+"', "+
                               " '"+addressRec.getState()+"', "+
                               " '"+addressRec.getZipcode()+"', "+
                               " '"+addressRec.getCountryCode()+"', "+
                               " '"+UserSystem.getUserId()+"', "+
                               " CURRENT TIMESTAMP)" ;
                  } else
                     LogSystem.log ( 1, "CustomerRec - Cannot generate Machine Num or Rec Num for Insert. CUSTNUM:"+custNum ) ;
                  break ;

               case REC_UPDATE :
                  if ( addressRec.getRecNum() > 0 ) {
                     strStmt = "UPDATE CLIO.CUSTADDRESS "+
                               "SET ADDRESS1     = '"+addressRec.getAddress1()+"', "+
                               "    ADDRESS2     = '"+addressRec.getAddress2()+"', "+
                               "    ADDRESS3     = '"+addressRec.getAddress3()+"', "+
                               "    CITY         = '"+addressRec.getCity()+"', "+
                               "    STATE        = '"+addressRec.getState()+"', "+
                               "    ZIPCODE      = '"+addressRec.getZipcode()+"', "+
                               "    DBUSER       = '"+UserSystem.getUserId()+"', "+
                               "    CHANGEDTIME  = CURRENT TIMESTAMP "+
                               "WHERE CUSTNUM    = "+custNum+" AND "+
                               "      RECNUM     = "+addressRec.getRecNum() ;
                  }
                  break ;
            } /* switch  */


            if ( strStmt != null ) {
               stmt.executeUpdate ( strStmt ) ;
               addressRec.updateRecStatus ( REC_NOCHANGE ) ;
            }
            i++ ;
         }

         rc = true ;

      } /* if addressVec  */
      else
         rc = true ;


      return( rc ) ;

   }





   /*------------------------------------------------------------*/
   /* saveCustPhone
    */
   /**
    *    THis method will save all customer
    *  phonees to the CLIO.CUSTADDRESS
    *  table
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveCustPhone ( Statement stmt ) throws Exception {
      boolean   rc        = false ;

      if ( ( phoneVec != null ) && ( phoneVec.size() > 0 ) ) {
         int                i       = 0 ;
         int                length  = phoneVec.size() ;
         CustomerPhoneRec   phoneRec = null ;
         String             strStmt = null ;

         while ( i < length ) {

            phoneRec = (CustomerPhoneRec) phoneVec.elementAt(i) ;

            strStmt = null ;
            switch ( phoneRec.getRecStatus() ) {
               case REC_DELETE :
                  strStmt = "DELETE FROM CLIO.CUSTPHONE "+
                            "WHERE RECNUM     = "+phoneRec.getRecNum()+" AND "+
                            "      CUSTNUM    = "+custNum ;
                  break ;

               case REC_INSERT :

                  if ( phoneRec.getRecNum() == 0 )
                     phoneRec.generateRecNum(custNum) ;

                  if ( phoneRec.getRecNum() > 0 ) {
                     strStmt = "INSERT INTO CLIO.CUSTPHONE "+
                               "( CUSTNUM,    RECNUM,            AREACODE,    EXTENSION,    "+
                               "  PHONE,      PHONECOUNTRYCODE,  PHONENUM,    DBUSER,       "+
                               "  CHANGEDTIME )"+
                               "VALUES                                                                                                               "+
                               "( "+custNum+", "+
                               "  "+phoneRec.getRecNum()+", "+
                               " '"+phoneRec.getAreaCode()+"', "+
                               " '"+phoneRec.getExtension()+"', "+
                               " '"+phoneRec.getPhone()+"', "+
                               " '"+phoneRec.getPhoneCountryCode()+"', "+
                               " '"+phoneRec.getPhoneNumber()+"', "+
                               " '"+UserSystem.getUserId()+"', "+
                               " CURRENT TIMESTAMP)" ;
                  } else
                     LogSystem.log ( 1, "CustomerRec - Cannot generate Machine Num or Rec Num for Insert. CUSTNUM:"+custNum ) ;
                  break ;

               case REC_UPDATE :
                  if ( phoneRec.getRecNum() > 0 ) {
                     strStmt = "UPDATE CLIO.CUSTPHONE "+
                               "SET AREACODE         = '"+phoneRec.getAreaCode()+"', "+
                               "    EXTENSION        = '"+phoneRec.getExtension()+"', "+
                               "    PHONE            = '"+phoneRec.getPhone()+"', "+
                               "    PHONECOUNTRYCODE = '"+phoneRec.getPhoneCountryCode()+"', "+
                               "    PHONENUM         = '"+phoneRec.getPhoneNumber()+"', "+
                               "    DBUSER           = '"+UserSystem.getUserId()+"', "+
                               "    CHANGEDTIME      = CURRENT TIMESTAMP "+
                               "WHERE CUSTNUM    = "+custNum+" AND "+
                               "      RECNUM     = "+phoneRec.getRecNum() ;
                  }
                  break ;
            } /* switch  */


            if ( strStmt != null ) {
               stmt.executeUpdate ( strStmt ) ;
               phoneRec.updateRecStatus ( REC_NOCHANGE ) ;
            }
            i++ ;
         }

         rc = true ;
      } /* if phoneVec  */
      else
         rc = true ;

      return( rc ) ;

   }





   /*------------------------------------------------------------*/
   /* saveCustCompany
    */
   /**
    *    This method will save the company
    *  information to the CLIO.CUSTCOMPANY
    *  table
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveCustCompany ( Statement stmt ) throws Exception {
      boolean  rc         = false ;
      String    strStmt = null ;

      switch ( getRecStatus() ) {
         case REC_INSERT :
            strStmt = "INSERT INTO CLIO.CUSTCOMPANY         "+
                      "(CUSTNUM, COMPANY, COMPANY2, DBUSER, "+
                      " CHANGEDTIME)                        "+
                      "VALUES                               "+
                      "( "+custNum+",   "+
                      " '"+company+"',  "+
                      " '"+company2+"', "+
                      " '"+UserSystem.getUserId()+"', "+
                      " CURRENT TIMESTAMP )" ;
            break ;

         case REC_DELETE :
            strStmt = "DELETE FROM CLIO.CUSTCOMPANY         "+
                      "WHERE CUSTNUM = "+custNum ;
            break ;

         case REC_UPDATE :
            strStmt = "UPDATE CLIO.CUSTCOMPANY "+
                      "SET COMPANY     = '"+company+"',  "+
                      "    COMPANY2    = '"+company2+"', "+
                      "    DBUSER      = '"+UserSystem.getUserId()+"', "+
                      "    CHANGEDTIME = CURRENT TIMESTAMP "+
                      "WHERE CUSTNUM   = "+custNum ;
            break ;
      }

      if ( strStmt != null ) {
         stmt.executeUpdate ( strStmt ) ;
         rc = true ;
      }


      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* saveCustUserId
    */
   /**
    *    This method will save the user id and password
    *  information to the CLIO.CUSTPASSWD
    *  table
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveCustUserId ( Statement stmt ) throws Exception {
      boolean   rc        = false ;
      String    strStmt = null ;

      switch ( getRecStatus() ) {
         case REC_INSERT :
            strStmt = "INSERT INTO CLIO.CUSTPASSWD                   "+
                      "(CUSTNUM,    ATTEMPTS, QUESTION, ANSWER,      "+
                      " CUSTUSERID, PASSWORD, DBUSER,   CHANGEDTIME) "+
                      "VALUES                                        "+
                      "( "+custNum+",   "+
                      "  0,             "+
                      " '"+question+"',  "+
                      " '"+answer+"', "+
                      " '"+userId+"', "+
                      " '"+password+"', "+
                      " '"+UserSystem.getUserId()+"', "+
                      " CURRENT TIMESTAMP )" ;
            break ;

         case REC_DELETE :
            strStmt = "DELETE FROM CLIO.CUSTPASSWD "+
                      "WHERE CUSTNUM = "+custNum ;
            break ;

         case REC_UPDATE :
            strStmt = "UPDATE CLIO.CUSTPASSWD "+
                      "SET ATTEMPTS    = 0, "+
                      "    CUSTUSERID  = '"+userId+"', "+
                      "    PASSWORD    = '"+password+"', "+
                      "    QUESTION    = '"+question+"', "+
                      "    ANSWER      = '"+answer+"', "+
                      "    DBUSER      = '"+UserSystem.getUserId()+"', "+
                      "    CHANGEDTIME = CURRENT TIMESTAMP "+
                      "WHERE CUSTNUM   = "+custNum ;
            break ;
      }

      if ( strStmt != null ) {
         stmt.executeUpdate ( strStmt ) ;
         rc = true ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* generateUserId
    */
   /**
    *    This method will attempt to generate a 
    *  user id for this customer. These are the steps to creating
    *  the id:
    *
    *  1. Grab the first part of their email address ( dlentz of dlentz@us.ibm.com )
    *  2. Put the first letter of their first name with their last name (blentz of Barbara Lentz)
    *  3. Add a numeric number to the user id of step 1. (dlentz1, dlentz2, dlentz3)
    *
    *  @return The user id created, or NULL if not created
    *//*---------------------------------------------------------*/
   public String generateUserId () {
      StringBuffer userId  = new StringBuffer()  ;
      int          custNum = 0 ;
      String       retUser = null ;
      int          i       = 1 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have an eMail, use it                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( eMailAddress != null ) && ( eMailAddress.indexOf ( "@" ) > 0 ) ) {
         userId.append ( eMailAddress.substring ( 0, eMailAddress.indexOf ( "@" ) ) ) ;

         if ( findCustNumByUserId ( userId.toString() ) == 0 )
            retUser = userId.toString() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, try their name                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retUser == null ) {
         userId.setLength(0) ;

         if ( ( firstName != null ) && ( firstName.length() > 1 ) )
            userId.append ( firstName.charAt(0) ) ;

         if ( lastName != null )
            userId.append ( lastName ) ;

         if ( findCustNumByUserId ( userId.toString() ) == 0 )
            retUser = userId.toString() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, All failed, start adding digits                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( retUser == null ) {

         while ( ( retUser == null ) && ( i < 50 ) ) {
            if ( findCustNumByUserId ( userId.toString()+i ) == 0 )
               retUser = userId.toString()+i ;
            i++ ;
         }
      }

      this.userId = retUser ;
      return( retUser ) ;
   }





   /*------------------------------------------------------------*/
   /* generateCustNum
    */
   /**
    *    This method will assign a customer number
    *  to this record. If there is currently a custnum, 
    *  it is over written
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void generateCustNum () {
      custNum = Counter.getCounter ( LogSystem.getInstance(), "CUSTNUM" ) ;
   }






   /*------------------------------------------------------------*/
   /* readFromDatabase
    */
   /**
    *    This method will read all information
    *  about this customer from the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything went Ok, FALSE if not
    *//*---------------------------------------------------------*/
   public int readFromDatabase () {
      int       rc        = 0 ;
      String    rtnName   = "readFromDatabase" ;
      SQLMethod sql = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;

      if ( custNum > 0 ) {
         try {
            Statement sqlStmt = sql.createStatement() ;
            String    strStmt = null ;
            ResultSet rs      = null ;      


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Build the Query                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            strStmt = "SELECT A.CUSTNUM,     A.FNAME,       A.MI,          A.LNAME,      "+
                      "       A.SUFFIX,      A.SALUTATION,  A.EMAILADDR,   A.VALIDEMAIL, "+
                      "       A.ESUPCOUNT                                                "+
                      "FROM CLIO.CUSTNUM     A                                           "+
                      "WHERE A.CUSTNUM = "+custNum+"                                     "+
                      "FOR FETCH ONLY" ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute the query                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs = sqlStmt.executeQuery ( strStmt ) ;

            if ( rs.next() ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Populate our values                                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               setCustNum ( rs.getInt(1) ) ;
               setFieldData ( CUSTFIELD_FIRSTNAME,  rs.getString(2).trim() ) ;
               setFieldData ( CUSTFIELD_MIDDLEINIT, rs.getString(3).trim() ) ;
               setFieldData ( CUSTFIELD_LASTNAME,   rs.getString(4).trim() ) ;
               setFieldData ( CUSTFIELD_SUFFIX,     rs.getString(5).trim() ) ;
               setFieldData ( CUSTFIELD_SALUTATION, rs.getString(6).trim() ) ;
               setFieldData ( CUSTFIELD_EMAILADDR,  rs.getString(7).trim() ) ;
               setFieldData ( CUSTFIELD_VALIDEMAIL, rs.getString(8).trim() ) ;
               setCallCount ( rs.getInt(9) ) ;               


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read User Id Information                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               strStmt = "SELECT CUSTUSERID, PASSWORD FROM CLIO.CUSTPASSWD "+
                         "WHERE CUSTNUM = "+custNum+" FOR FETCH ONLY" ;
               rs = sqlStmt.executeQuery ( strStmt ) ;

               if ( rs.next () ) {
                  setFieldData ( CUSTFIELD_USERID,     rs.getString(1).trim()) ;
                  setFieldData ( CUSTFIELD_PASSWORD,   rs.getString(2).trim()) ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read Company Info                                       */
               /*                                                           */
               /*-----------------------------------------------------------*/
               strStmt = "SELECT COMPANY, COMPANY2 FROM CLIO.CUSTCOMPANY "+
                         "WHERE CUSTNUM = "+custNum+" FOR FETCH ONLY" ;
               rs = sqlStmt.executeQuery ( strStmt ) ;

               if ( rs.next () ) {
                  try {
                     setFieldData ( CUSTFIELD_COMPANY,    rs.getString(1).trim()) ;
                     setFieldData ( CUSTFIELD_COMPANY2,   rs.getString(2).trim()) ;
                  } catch ( NullPointerException e ) {
                     /* Do nothing. This is caught when Customer2 is null  */
                  }
               }

               rs.close() ;
               sql.close() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in Customer Machines                               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               rc = readCustMachines() ? 1 : 0 ;  

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in Customer Options                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( rc == 1 )
                  rc = readCustOptions() ? 1 : 0 ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in Customer Phones                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( rc == 1 )
                  rc = readCustPhone() ? 1 : 0 ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read in the Customer Addresses                          */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( rc == 1 )
                  rc = readCustAddress() ? 1 : 0 ;


            }

         } catch ( Exception e ) {
            sql.rollBack() ;
            sql.close() ;
            LogSystem.log ( 1, e, false ) ;
         }
      }

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* readCustMachines
    */
   /**
    *    This method will read in all machines
    *  from the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if read, FALSE if error
    *//*---------------------------------------------------------*/
   private boolean readCustMachines () {
      boolean   rc        = false ;
      String    rtnName   = "readCustMachines" ;
      SQLMethod sql = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;

      if ( custNum > 0 ) {
         try {
            CustomerMachineRec  readRec = null ;
            Statement           sqlStmt = sql.createStatement() ;
            String              strStmt = null ;
            ResultSet           rs      = null ;          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Build our SQL                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            strStmt = "SELECT A.MACHINE,  A.MODEL,    A.SERIAL,  B.PRODUCTNUM, "+
                      "       C.DESCRIPT, D.DESCRIPT, A.DEFAULT, A.MACHINENUM  "+
                      "FROM CLIO.CUSTMACH      A, "+
                      "     CLIO.CUSTCOMPONENT B, "+
                      "     CLIO.TYPEBRAND     C, "+
                      "     CLIO.TYPEGROUP     D, "+
                      "     CLIO.BRANDS        E  "+
                      "WHERE A.CUSTNUM    = "+custNum+" AND  "+
                      "      B.MACHINENUM = A.MACHINENUM AND "+
                      "      E.BRANDIND   = A.BRANDIND AND   "+
                      "      D.GROUPID    = E.GROUPID AND    "+
                      "      C.BRANDID    = E.BRANDID        "+
                      "FOR FETCH ONLY" ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute our query                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs = sqlStmt.executeQuery ( strStmt ) ;                 


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop and read each machine                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            machineVec = new Vector ( 10, 1 ) ;
            while ( rs.next() ) {
               readRec = new CustomerMachineRec() ;

               readRec.setMachineType     ( rs.getString(1).trim()) ;
               readRec.setModel           ( rs.getString(2).trim()) ;
               readRec.setSerialNumber    ( rs.getString(3).trim()) ;
               readRec.setOperatingSystem ( rs.getString(4).trim()) ;
               readRec.setBrand           ( rs.getString(5).trim()) ;
               readRec.setFamily          ( rs.getString(6).trim()) ;
               readRec.setMachineNum      ( rs.getInt(8)    ) ;

               machineVec.addElement ( readRec ) ;
            } /* while  */

            rc = true ;

         } catch ( Exception e ) {
            sql.rollBack() ;
            LogSystem.log ( 1,e,true ) ;
         }

      } /* if custNum  */

      sql.close() ;

      return( rc ) ;

   } /* readCustMachines  */



   /*------------------------------------------------------------*/
   /* readCustOptions
    */
   /**
    *    This method will read in all options
    *  from the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if read, FALSE if error
    *//*---------------------------------------------------------*/
   private boolean readCustOptions () {
      boolean   rc        = false ;
      String    rtnName   = "readCustOptions" ;
      SQLMethod sql = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;

      if ( custNum > 0 ) {
         try {
            CustomerMachineRec  readRec = null ;
            Statement           sqlStmt = sql.createStatement() ;
            String              strStmt = null ;
            ResultSet           rs      = null ;          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Build our SQL                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            strStmt = "SELECT A.MACHINE,  A.PARTNUM,  A.SERIAL,  C.DESCRIPT,   "+
                      "       D.DESCRIPT, A.RECNUM "+
                      "FROM CLIO.CUSTOPTION    A, "+
                      "     CLIO.TYPEBRAND     C, "+
                      "     CLIO.TYPEGROUP     D, "+
                      "     CLIO.BRANDS        E  "+
                      "WHERE A.CUSTNUM    = "+custNum+" AND  "+
                      "      E.BRANDIND   = A.BRANDIND AND   "+
                      "      D.GROUPID    = E.GROUPID AND    "+
                      "      C.BRANDID    = E.BRANDID        "+
                      "FOR FETCH ONLY" ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute our query                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs = sqlStmt.executeQuery ( strStmt ) ;                 


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop and read each option                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            optionVec  = new Vector ( 10, 1 ) ;
            while ( rs.next() ) {
               readRec = new CustomerMachineRec() ;

               readRec.setMachineType     ( rs.getString(1).trim() ) ;
               readRec.setPartNumber      ( rs.getString(2).trim() ) ;
               readRec.setSerialNumber    ( rs.getString(3).trim() ) ;
               readRec.setBrand           ( rs.getString(4).trim() ) ;
               readRec.setFamily          ( rs.getString(5).trim() ) ;
               readRec.setRecNum          ( rs.getInt(6)    ) ;

               optionVec.addElement ( readRec ) ;
            } /* while  */

            rc = true ;

         } catch ( Exception e ) {
            sql.rollBack() ;
            LogSystem.log ( 1, e, true ) ;
         }

      } /* if custNum  */

      sql.close() ;

      return( rc ) ;

   } /* readCustOptions  */



   /*------------------------------------------------------------*/
   /* readCustPhone
    */
   /**
    *    This method will read in all phones
    *  from the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if read, FALSE if error
    *//*---------------------------------------------------------*/
   private boolean readCustPhone () {
      boolean   rc        = false ;
      String    rtnName   = "readCustPhone" ;
      SQLMethod sql = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;

      if ( custNum > 0 ) {
         try {
            CustomerPhoneRec readRec = null ;
            Statement        sqlStmt = sql.createStatement() ;
            String           strStmt = null ;
            ResultSet        rs      = null ;          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Build our SQL                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            strStmt = "SELECT PHONENUM, PHONE, PHONECOUNTRYCODE, EXTENSION, RECNUM, AREACODE "+
                      "FROM CLIO.CUSTPHONE "+
                      "WHERE CUSTNUM    = "+custNum+"  "+
                      "FOR FETCH ONLY" ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute our query                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs = sqlStmt.executeQuery ( strStmt ) ;                 


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop and read each phone                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            phoneVec  = new Vector ( 10, 1 ) ;
            while ( rs.next() ) {
               readRec = new CustomerPhoneRec() ;

               if ( rs.getString(1) != null )
                  readRec.setPhoneNumber      ( rs.getString(1).trim() ) ;
               else
                  readRec.setPhoneNumber      ( "" ) ;

               if ( rs.getString(2) != null )
                  readRec.setPhone            ( rs.getString(2).trim() ) ;
               else
                  readRec.setPhone            ( "" ) ;

               if ( rs.getString(3) != null )
                  readRec.setPhoneCountryCode ( rs.getString(3).trim() ) ;
               else
                  readRec.setPhoneCountryCode ( "" ) ;

               if ( rs.getString(4) != null )
                  readRec.setExtension        ( rs.getString(4).trim() ) ;
               else
                  readRec.setExtension        ( "" ) ;

               readRec.setRecNum           ( rs.getInt(5)    ) ;

               if ( rs.getString(6) != null )
                  readRec.setAreaCode         ( rs.getString(6).trim() ) ;
               else
                  readRec.setAreaCode         ( "" ) ;

               phoneVec.addElement ( readRec ) ;
            } /* while  */

            rc = true ;

         } catch ( Exception e ) {
            sql.rollBack() ;
            LogSystem.log( 1, e, true ) ;
         }

      } /* if custNum  */

      sql.close() ;

      return( rc ) ;

   } /* readCustPhone  */



   /*------------------------------------------------------------*/
   /* readCustAddress
    */
   /**
    *    This method will read in all address
    *  from the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if read, FALSE if error
    *//*---------------------------------------------------------*/
   private boolean readCustAddress () {
      boolean   rc        = false ;
      String    rtnName   = "readCustAddress" ;
      SQLMethod sql = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 5 ) ;

      if ( custNum > 0 ) {
         try {
            CustomerAddressRec readRec = null ;
            Statement        sqlStmt = sql.createStatement() ;
            String           strStmt = null ;
            ResultSet        rs      = null ;          

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Build our SQL                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            strStmt = "SELECT A.ADDRESS1, A.ADDRESS2, A.ADDRESS3,    A.CITY,     "+
                      "       A.STATE,    A.ZIPCODE,  A.COUNTRYCODE, B.DESCRIPT, "+
                      "       A.RECNUM                                           "+
                      "FROM CLIO.CUSTADDRESS A,                                  "+
                      "     CLIO.TYPEADDRESS B                                   "+
                      "WHERE A.CUSTNUM    = "+custNum+"  AND                     "+
                      "      A.ADDRESSIND = B.ADDRESSIND                         "+
                      "FOR FETCH ONLY" ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Execute our query                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            rs = sqlStmt.executeQuery ( strStmt ) ;                 


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop and read each phone                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            addressVec  = new Vector ( 10, 1 ) ;
            while ( rs.next() ) {
               readRec = new CustomerAddressRec() ;

               readRec.setAddress1    ( rs.getString(1).trim() ) ;
               readRec.setAddress2    ( rs.getString(2).trim() ) ;
               readRec.setAddress3    ( rs.getString(3).trim() ) ;
               readRec.setCity        ( rs.getString(4).trim() ) ;
               readRec.setState       ( rs.getString(5).trim() ) ;
               readRec.setZipcode     ( rs.getString(6).trim() ) ;
               readRec.setAddressType ( rs.getString(8).trim() ) ;
               readRec.setRecNum      ( rs.getInt(9)    ) ;

               addressVec.addElement ( readRec ) ;
            } /* while  */

            rc = true ;

         } catch ( Exception e ) {
            sql.rollBack() ;
            LogSystem.log ( 1, e, true ) ;
         }

      } /* if custNum  */

      sql.close() ;

      return( rc ) ;

   } /* readCustAddress  */





   /*------------------------------------------------------------*/
   /* setCallCount
    */
   /**
    *    This method will set the call count for
    *  this customer
    *  
    *
    *  @param callCount The new call count
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCallCount ( int callCount ) {
      this.callCount = callCount ;
   }




   /*------------------------------------------------------------*/
   /* getCallCount
    */
   /**
    *    This method will get the call count
    *  from this customer
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getCallCount () {
      return( callCount ) ;
   }





   /*------------------------------------------------------------*/
   /* removeMachineRec
    */
   /**
    *    This method will remove a machine record
    *  and marks it for delete so when the customer is
    *  saved it will be deleted
    *
    *  @param machineRec The CustomerMachineRec
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeMachineRec ( CustomerMachineRec machineRec ) {
      machineRec.updateRecStatus ( REC_DELETE ) ;
   }





   /*------------------------------------------------------------*/
   /* getFirstName
    */
   /**
    *    This method will return the customer's first
    *  name
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getFirstName () {
      return( firstName ) ;
   }




   /*------------------------------------------------------------*/
   /* getLastName
    */
   /**
    *    This method will return the customer's last
    *  name
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getLastName () {
      return( lastName ) ;
   }
} /* CustomerRec */

