package com.ibm.nzna.projects.qit.customer.custedit ;
/*-----------------------------------------------------------*/
/*   CustEditPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustEditPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel holds edit fields to allow the user to edit customer values 
 *
 *
 * @version 1.0, 8/25/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.customer.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   CustEditPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustEditPanel extends JPanel implements DataLengths, ActionListener, QuestPanel, AppConst, CustomerConstants {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   THREADS                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_INITIALIZE   = "INit" ;
   private static final String THREAD_REFRESHDATA  = "RD" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Statics                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] machTitle = { 
      Str.getStr ( STR_MACHINE ), 
      Str.getStr ( STR_MODEL ), 
      Str.getStr ( STR_SERIAL_NUMBER ), 
      Str.getStr ( STR_BRAND ),
      Str.getStr ( STR_FAMILY ),
      Str.getStr ( STR_OS )} ;

   private static final String[] optionTitle = { 
      Str.getStr ( STR_PART_NUMBER ), 
      Str.getStr ( STR_FAMILY ), 
      Str.getStr ( STR_SERIAL_NUMBER ), 
      Str.getStr ( STR_BRAND ),
      Str.getStr ( STR_FAMILY ),
      Str.getStr ( STR_OS )} ;

   private static final String[] phoneTitle = { 
      Str.getStr ( STR_PHONE_TYPE ), 
      Str.getStr ( STR_PHONE_NUMBER ), 
      Str.getStr ( STR_EXTENSION ), 
      Str.getStr ( STR_AREA_CODE ),
      Str.getStr ( STR_PHONE_CODE )};

   private static final String[] addressTitle = { 
      Str.getStr ( STR_ADDRESS_TYPE ), 
      Str.getStr ( STR_ADDRESS1 ), 
      Str.getStr ( STR_CITY ), 
      Str.getStr ( STR_STATE ),
      Str.getStr ( STR_ZIPCODE )};


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JPanel       pnl_DATAHOLD     = null ;
   private JTitle       st_LARGENAME     = null ;
   private JTitle       st_NAMEINFO      = null ;
   private JTitle       st_MACHINEINFO   = null ;
   private JTitle       st_OPTIONINFO    = null ;
   private JTitle       st_PHONEINFO     = null ;
   private JTitle       st_ADDRESSINFO   = null ;
   //private JLabel      st_LASTCALL      = null ;
   private JLabel       st_FIRSTNAME     = null ;
   private JLabel       st_LASTNAME      = null ;
   private JLabel       st_MIDDLEINITIAL = null ;
   private JLabel       st_EMAILADDR     = null ;
   private JTextField   ef_FIRSTNAME     = null ;
   private JTextField   ef_LASTNAME      = null ;
   private JTextField   ef_MIDDLEINITIAL = null ;
   //private JLabel      st_HITCOUNT      = null ;
   private JLabel       st_USERID        = null ;
   private JLabel       st_PASSWORD      = null ;
   private JTextField   ef_USERID        = null ;
   private JTextField   ef_PASSWORD      = null ;
   private JTextField   ef_EMAILADDR     = null ;
   private MultiList    cnr_MACHINES     = null ;
   private MultiList    cnr_PHONES       = null ;
   private MultiList    cnr_OPTIONS      = null ;
   private MultiList    cnr_ADDRESS      = null ;
   private JScrollPane  pnl_SCROLLPANE   = null ;
   private DButton      pb_ADDMACHINE    = null ;
   private DButton      pb_EDITMACHINE   = null ;
   private DButton      pb_DELETEMACHINE = null ;
   private DButton      pb_ADDOPTION     = null ;
   private DButton      pb_EDITOPTION    = null ;
   private DButton      pb_DELETEOPTION  = null ;
   private DButton      pb_ADDADDRESS    = null ;
   private DButton      pb_EDITADDRESS   = null ;
   private DButton      pb_DELETEADDRESS = null ;
   private DButton      pb_ADDPHONE      = null ;
   private DButton      pb_EDITPHONE     = null ;
   private DButton      pb_DELETEPHONE   = null ;
   private ActionButton pb_SAVE          = null ;
   private ActionButton pb_CLOSE         = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private CustomerRec   custRec       = null ;
   private boolean       initialized   = false ;
   private AppDefaultWin parentWin     = null ;
   private QuestPanel    previousPanel = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our default
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustEditPanel ( CustomerRec custRec, QuestPanel previousPanel ) {
      this.custRec       = custRec ;
      this.previousPanel = previousPanel ;
   }



   /*------------------------------------------------------------*/
   /* initlialize
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {
      int x         = 5 ;
      int y         = 5 ;
      int rowHeight = GUISystem.getRowHeight () ;
      int cnrWidth  = 404 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SAVE          = new ActionButton ( Str.getStr ( STR_SAVE ),  ImageSystem.getImageIcon ( this,  ImageSystem.SAVE  ) , Str.getStr(STR_SAVE) ) ;
      pb_CLOSE         = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this,  ImageSystem.CLOSE ) , Str.getStr(STR_CLOSE) ) ;

      st_NAMEINFO      = new JTitle ( Str.getStr ( STR_NAMEINFO ) ) ;
      st_MACHINEINFO   = new JTitle ( Str.getStr ( STR_MACHINEINFO ) ) ;
      st_ADDRESSINFO   = new JTitle ( Str.getStr ( STR_ADDRESSINFO ) ) ;
      st_OPTIONINFO    = new JTitle ( Str.getStr ( STR_OPTIONINFO ) ) ;
      st_PHONEINFO     = new JTitle ( Str.getStr ( STR_PHONEINFO ) ) ;
      st_LARGENAME     = new JTitle ( "" ) ;

      st_FIRSTNAME     = new JLabel ( Str.getStr ( STR_FIRSTNAME ) ) ;
      st_MIDDLEINITIAL = new JLabel ( Str.getStr ( STR_MIDDLEINITIAL ) ) ;
      st_LASTNAME      = new JLabel ( Str.getStr ( STR_LASTNAME ) ) ;
      st_USERID        = new JLabel ( Str.getStr ( STR_USERID ) ) ;
      st_PASSWORD      = new JLabel ( Str.getStr ( STR_PASSWORD ) ) ;
      st_EMAILADDR     = new JLabel ( Str.getStr ( STR_EMAIL_ADDRESS ) ) ;

      ef_FIRSTNAME     = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_FIRSTNAME ),     "", 0 ) ;
      ef_MIDDLEINITIAL = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_MIDDLEINITIAL ), "", 0 ) ;
      ef_LASTNAME      = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_LASTNAME ),      "", 0 ) ;                     
      ef_USERID        = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_CUSTUSERID ),    "", 0 ) ;                     
      ef_PASSWORD      = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_PASSWORD ),      "", 0 ) ;                     
      ef_EMAILADDR     = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_EMAILADDR ),     "", 0 ) ;                     

      cnr_MACHINES = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_OPTIONS  = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_PHONES   = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_ADDRESS  = new MultiList ( GUISystem.getFontUtil() ) ;

      pnl_DATAHOLD = new JPanel() ;

      //st_HITCOUNT = new JLabel ( Str.getStr ( STR_HITCOUNT )+":0" ) ;                 
      //st_LASTCALL = new JLabel ( Str.getStr ( STR_LASTCALL )+":1/1/1970" ) ;                 

      pb_ADDMACHINE    = new DButton ( Str.getStr ( STR_ADD ) ) ;
      pb_EDITMACHINE   = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_DELETEMACHINE = new DButton ( Str.getStr ( STR_DELETE ) ) ;
      pb_ADDOPTION     = new DButton ( Str.getStr ( STR_ADD ) ) ;
      pb_EDITOPTION    = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_DELETEOPTION  = new DButton ( Str.getStr ( STR_DELETE ) ) ;
      pb_ADDPHONE     = new DButton ( Str.getStr ( STR_ADD ) ) ;
      pb_EDITPHONE    = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_DELETEPHONE  = new DButton ( Str.getStr ( STR_DELETE ) ) ;
      pb_ADDADDRESS     = new DButton ( Str.getStr ( STR_ADD ) ) ;
      pb_EDITADDRESS    = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_DELETEADDRESS  = new DButton ( Str.getStr ( STR_DELETE ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our Data hold panel                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_DATAHOLD.setLayout ( null ) ;
      pnl_DATAHOLD.add ( st_LARGENAME     ) ;
      pnl_DATAHOLD.add ( st_NAMEINFO      ) ;
      pnl_DATAHOLD.add ( st_FIRSTNAME     ) ;
      pnl_DATAHOLD.add ( st_LASTNAME      ) ;
      pnl_DATAHOLD.add ( st_MIDDLEINITIAL ) ;
      pnl_DATAHOLD.add ( ef_FIRSTNAME     ) ;
      pnl_DATAHOLD.add ( ef_LASTNAME      ) ;
      pnl_DATAHOLD.add ( ef_MIDDLEINITIAL ) ;
      pnl_DATAHOLD.add ( ef_USERID        ) ;
      pnl_DATAHOLD.add ( st_USERID        ) ;
      pnl_DATAHOLD.add ( ef_PASSWORD      ) ;
      pnl_DATAHOLD.add ( st_PASSWORD      ) ;
      pnl_DATAHOLD.add ( st_EMAILADDR     ) ;
      pnl_DATAHOLD.add ( ef_EMAILADDR     ) ;
      pnl_DATAHOLD.add ( st_MACHINEINFO   ) ;
      pnl_DATAHOLD.add ( cnr_MACHINES     ) ;
      pnl_DATAHOLD.add ( pb_ADDMACHINE    ) ;
      pnl_DATAHOLD.add ( pb_EDITMACHINE   ) ;
      pnl_DATAHOLD.add ( pb_DELETEMACHINE ) ;
      pnl_DATAHOLD.add ( st_OPTIONINFO    ) ;
      pnl_DATAHOLD.add ( cnr_OPTIONS      ) ;
      pnl_DATAHOLD.add ( pb_ADDOPTION     ) ;
      pnl_DATAHOLD.add ( pb_EDITOPTION    ) ;
      pnl_DATAHOLD.add ( pb_DELETEOPTION  ) ;
      pnl_DATAHOLD.add ( st_PHONEINFO     ) ;
      pnl_DATAHOLD.add ( cnr_PHONES       ) ;
      pnl_DATAHOLD.add ( pb_ADDPHONE      ) ;
      pnl_DATAHOLD.add ( pb_EDITPHONE     ) ;
      pnl_DATAHOLD.add ( pb_DELETEPHONE   ) ;
      pnl_DATAHOLD.add ( st_ADDRESSINFO   ) ;
      pnl_DATAHOLD.add ( cnr_ADDRESS      ) ;
      pnl_DATAHOLD.add ( pb_ADDADDRESS    ) ;
      pnl_DATAHOLD.add ( pb_EDITADDRESS   ) ;
      pnl_DATAHOLD.add ( pb_DELETEADDRESS ) ;                     


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to this panel                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout () ) ;
      pnl_SCROLLPANE = new JScrollPane ( pnl_DATAHOLD ) ;
      add ( pnl_SCROLLPANE, BorderLayout.CENTER ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust some properties                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_DATAHOLD.setBackground ( Color.white ) ;
      pnl_DATAHOLD.setBorder ( GUISystem.emptyBorder ) ;

      /* Fast Scroll!  */
      pnl_SCROLLPANE.getViewport().putClientProperty ("EnableWindowBlit", Boolean.TRUE);

      /* No Borders!  */
      pnl_SCROLLPANE.setBorder ( GUISystem.emptyBorder ) ;
      cnr_OPTIONS.setBorder    ( GUISystem.emptyBorder ) ;
      cnr_MACHINES.setBorder   ( GUISystem.emptyBorder ) ;
      cnr_PHONES.setBorder     ( GUISystem.emptyBorder ) ;
      cnr_ADDRESS.setBorder    ( GUISystem.emptyBorder ) ;

      /* See through!  */
      cnr_MACHINES.setOpaque     ( false ) ;
      cnr_OPTIONS.setOpaque      ( false ) ;
      cnr_PHONES.setOpaque       ( false ) ;
      cnr_ADDRESS.setOpaque      ( false ) ;
      st_FIRSTNAME.setOpaque     ( false ) ;
      st_LASTNAME.setOpaque      ( false ) ;
      st_MIDDLEINITIAL.setOpaque ( false ) ;                        
      //st_HITCOUNT.setOpaque      ( false ) ;
      st_NAMEINFO.setOpaque      ( false ) ;

      st_NAMEINFO.setFont    ( FontSystem.smallTitleFont ) ;
      st_MACHINEINFO.setFont ( FontSystem.smallTitleFont ) ;
      st_OPTIONINFO.setFont  ( FontSystem.smallTitleFont ) ;
      st_PHONEINFO.setFont   ( FontSystem.smallTitleFont ) ;
      st_ADDRESSINFO.setFont ( FontSystem.smallTitleFont ) ;
      st_LARGENAME.setFont   ( FontSystem.largeTitleFont ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the titles of the columns in the containers         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_MACHINES.setColumnHeadings ( machTitle ) ;
      cnr_MACHINES.setColumnWidth ( 0, 100 ) ;
      cnr_MACHINES.setColumnWidth ( 1, 75 ) ;
      cnr_MACHINES.setColumnWidth ( 2, 105 ) ;
      cnr_MACHINES.setColumnWidth ( 3, 150 ) ;
      cnr_MACHINES.setColumnWidth ( 4, 150 ) ;
      cnr_MACHINES.setColumnWidth ( 5, 150 ) ;

      cnr_OPTIONS.setColumnHeadings ( optionTitle ) ;
      cnr_OPTIONS.setColumnWidth ( 0, 100 ) ;
      cnr_OPTIONS.setColumnWidth ( 1, 75 ) ;
      cnr_OPTIONS.setColumnWidth ( 2, 105 ) ;
      cnr_OPTIONS.setColumnWidth ( 3, 150 ) ;
      cnr_OPTIONS.setColumnWidth ( 4, 150 ) ;
      cnr_OPTIONS.setColumnWidth ( 5, 150 ) ;

      cnr_PHONES.setColumnHeadings ( phoneTitle ) ;
      cnr_PHONES.setColumnWidth ( 0, 125 ) ;
      cnr_PHONES.setColumnWidth ( 1, 100 ) ;
      cnr_PHONES.setColumnWidth ( 2, 50  ) ;
      cnr_PHONES.setColumnWidth ( 3, 100 ) ;
      cnr_PHONES.setColumnWidth ( 4, 100 ) ;


      cnr_ADDRESS.setColumnHeadings ( addressTitle ) ;
      cnr_ADDRESS.setColumnWidth ( 0, 125 ) ;
      cnr_ADDRESS.setColumnWidth ( 1, 125 ) ;
      cnr_ADDRESS.setColumnWidth ( 2, 105 ) ;
      cnr_ADDRESS.setColumnWidth ( 3, 150 ) ;
      cnr_ADDRESS.setColumnWidth ( 4, 150 ) ;
      cnr_ADDRESS.setColumnWidth ( 5, 150 ) ;

      /* No Borders on the headings!  */
      cnr_MACHINES.setColumnHeadingBorder ( null ) ;
      cnr_OPTIONS.setColumnHeadingBorder  ( null ) ;
      cnr_PHONES.setColumnHeadingBorder   ( null ) ;
      cnr_ADDRESS.setColumnHeadingBorder  ( null ) ;
      cnr_MACHINES.setOpaque ( false ) ;
      cnr_OPTIONS.setOpaque  ( false ) ;
      cnr_PHONES.setOpaque   ( false ) ;
      cnr_ADDRESS.setOpaque  ( false ) ;
      cnr_OPTIONS.setBackground    ( Color.white ) ;
      cnr_MACHINES.setBackground   ( Color.white ) ;
      cnr_PHONES.setBackground     ( Color.white ) ;
      cnr_ADDRESS.setBackground    ( Color.white ) ;           


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADDMACHINE.addActionListener    ( this ) ;
      pb_EDITMACHINE.addActionListener   ( this ) ;
      pb_DELETEMACHINE.addActionListener ( this ) ;
      pb_SAVE.addActionListener ( this ) ;
      pb_CLOSE.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the initial size and position                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_LARGENAME.setBounds ( x, y, 400, rowHeight+(rowHeight/2) ) ;
      y+=rowHeight*2 ;

      st_NAMEINFO.setBounds ( x, y, 400, rowHeight ) ;
      y+=rowHeight+2 ;

      st_FIRSTNAME.setBounds ( x, y, 75, rowHeight ) ;
      ef_FIRSTNAME.setBounds ( x+75, y, 100, rowHeight ) ;
      st_MIDDLEINITIAL.setBounds ( x+180, y, 100, rowHeight ) ;
      ef_MIDDLEINITIAL.setBounds ( x+280, y, 20,  rowHeight ) ;
      st_LASTNAME.setBounds      ( x+305, y, 75, rowHeight ) ;
      ef_LASTNAME.setBounds      ( x+380, y, 100, rowHeight ) ;
      y+=rowHeight+5 ;

      st_USERID.setBounds ( x, y, 75, rowHeight ) ;
      ef_USERID.setBounds ( x+75, y, 100, rowHeight ) ;
      st_PASSWORD.setBounds ( x+180, y,100, rowHeight ) ;
      ef_PASSWORD.setBounds ( x+280, y,100, rowHeight ) ;
      y+=rowHeight+5 ;

      st_EMAILADDR.setBounds ( x, y, 100, rowHeight ) ;
      ef_EMAILADDR.setBounds ( x+100, y, 380, rowHeight ) ;
      y+=rowHeight+25;

      st_MACHINEINFO.setBounds ( x, y, cnrWidth, rowHeight ) ;
      y+=rowHeight+2 ;

      cnr_MACHINES.setBounds ( x, y, cnrWidth, (rowHeight*7)-15 ) ;
      pb_ADDMACHINE.setBounds    ( x+cnrWidth+5, y, 70, 18 ) ;
      pb_EDITMACHINE.setBounds   ( x+cnrWidth+5, y+19, 70, 18 ) ;
      pb_DELETEMACHINE.setBounds ( x+cnrWidth+5, y+38, 70, 18 ) ;
      y+=rowHeight*7 ;

      st_OPTIONINFO.setBounds   ( x, y, cnrWidth, rowHeight ) ;
      y+=rowHeight+2 ;

      cnr_OPTIONS.setBounds     ( x, y, cnrWidth, (rowHeight*7)-15 ) ;
      pb_ADDOPTION.setBounds    ( x+cnrWidth+5, y, 70, 18 ) ;
      pb_EDITOPTION.setBounds   ( x+cnrWidth+5, y+19, 70, 18 ) ;
      pb_DELETEOPTION.setBounds ( x+cnrWidth+5, y+38, 70, 18 ) ;
      y+=rowHeight*7 ;

      st_PHONEINFO.setBounds    ( x, y, cnrWidth, rowHeight ) ;
      y+=rowHeight+2 ;

      cnr_PHONES.setBounds      ( x, y, cnrWidth, (rowHeight*7)-15 ) ;
      pb_ADDPHONE.setBounds     ( x+cnrWidth+5, y, 70, 18 ) ;
      pb_EDITPHONE.setBounds    ( x+cnrWidth+5, y+19, 70, 18 ) ;
      pb_DELETEPHONE.setBounds  ( x+cnrWidth+5, y+38, 70, 18 ) ;
      y+=rowHeight*7 ;

      st_ADDRESSINFO.setBounds ( x, y, cnrWidth, rowHeight ) ;
      y+=rowHeight+2 ;

      cnr_ADDRESS.setBounds      ( x, y, cnrWidth, (rowHeight*7)-15 ) ;
      pb_ADDADDRESS.setBounds    ( x+cnrWidth+5, y, 70, 18 ) ;
      pb_EDITADDRESS.setBounds   ( x+cnrWidth+5, y+19, 70, 18 ) ;
      pb_DELETEADDRESS.setBounds ( x+cnrWidth+5, y+38, 70, 18 ) ;
      y+=rowHeight*7 ;


      pnl_DATAHOLD.setMinimumSize   ( new Dimension ( 450, 800 ) ) ;
      pnl_DATAHOLD.setPreferredSize ( new Dimension ( 450, 800 ) ) ;


      return( true ) ;
   }






   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing and we use this time to finalize
    *  our initialization
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {

      parentWin = GUISystem.getParentDefWin ( this ) ;

      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_SAVE ) ;

      if ( custRec != null )
         refreshData() ;

   }

   /*------------------------------------------------------------*/
   /* setEdit
    */
   /**
    *    This method will set the CustomerRec 
    *  that we are going to edit
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEdit ( CustomerRec customerRec ) {
      this.custRec = customerRec ;
   }



   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all data on our
    *  screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      if ( custRec != null ) {
         String firstName = (String) custRec.getFieldData ( CUSTFIELD_FIRSTNAME ) ;
         String lastName  = (String) custRec.getFieldData ( CUSTFIELD_LASTNAME );

         if ( firstName == null )
            firstName = "" ;
         if ( lastName == null )
            lastName = "" ;

         st_LARGENAME.setText     ( firstName+" "+lastName ) ;
         ef_FIRSTNAME.setText     ( firstName ) ;
         ef_MIDDLEINITIAL.setText ( (String) custRec.getFieldData ( CUSTFIELD_MIDDLEINIT ) ) ;
         ef_LASTNAME.setText      ( (String) custRec.getFieldData ( CUSTFIELD_LASTNAME ) ) ;
         ef_EMAILADDR.setText     ( (String) custRec.getFieldData ( CUSTFIELD_EMAILADDR ) ) ;
         ef_USERID.setText        ( (String) custRec.getFieldData ( CUSTFIELD_USERID ) ) ;
         ef_PASSWORD.setText      ( (String) custRec.getFieldData ( CUSTFIELD_PASSWORD ) ) ;

         cnr_MACHINES.removeAll() ;
         cnr_MACHINES.setData ( (Vector) custRec.getFieldData ( CUSTFIELD_ALLMACHINE ) ) ;

         cnr_OPTIONS.removeAll() ;
         cnr_OPTIONS.setData ( (Vector) custRec.getFieldData ( CUSTFIELD_ALLOPTION ) ) ;

         cnr_PHONES.removeAll() ;
         cnr_PHONES.setData ( (Vector) custRec.getFieldData ( CUSTFIELD_ALLPHONE ) ) ;

         cnr_ADDRESS.removeAll() ;
         cnr_ADDRESS.setData ( (Vector) custRec.getFieldData ( CUSTFIELD_ALLADDRESSES ) ) ;

         //st_HITCOUNT.setText ( "<html><font size=2 face=\"Arial, Helvetica\"><b>"+
         //                      Str.getStr ( STR_HITCOUNT )+":</b><br>"+custRec.getCallCount() ) ;                 
         //st_LASTCALL.setText ( "<html><font size=2 face=\"Arial, Helvetica\"><b>"+
         //                      Str.getStr ( STR_LASTCALL )+":</b><br>"+custRec.getFieldData ( CUSTFIELD_PRETTYCHANGEDTIME ) ) ;

      }
   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save all of the data on
    *  this panel to the customer record
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if data was saved, FALSE if data was missing
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      String     fName      = ef_FIRSTNAME.getText() ;
      String     lName      = ef_LASTNAME.getText() ;
      String     userId     = ef_USERID.getText() ;
      String     password   = ef_PASSWORD.getText() ;
      String     eMailAddr  = ef_EMAILADDR.getText() ;
      String     middleInit = ef_MIDDLEINITIAL.getText() ;
      String     errorStr   = null ;
      Component  errorComp  = null ;
      boolean    rc         = false ;

      try {

         if ( ( fName == null ) || ( fName.length() == 0 ) ) {
            errorStr = Str.getStr ( STR_MUST_ENTER_FIRST_NAME ) ;
            errorComp = ef_FIRSTNAME ;
         }

         else if ( ( lName == null ) || ( lName.length() == 0 ) ) {
            errorStr = Str.getStr ( STR_MUST_ENTER_LAST_NAME ) ;
            errorComp = ef_LASTNAME ;
         }

         else if ( ( userId == null ) || ( userId.length() == 0 ) ) {
            errorStr = Str.getStr ( STR_MUST_ENTER_USERID ) ;
            errorComp = ef_USERID ;
         }

         else if ( ( password == null ) || ( password.length() == 0 ) ) {
            errorStr = Str.getStr ( STR_MUST_ENTER_PASSWORD ) ;
            errorComp = ef_PASSWORD ;
         }

         else if ( ( eMailAddr == null ) || ( eMailAddr.length() == 0 ) ) {
            errorStr = Str.getStr ( STR_MUST_ENTER_EMAIL_ADDR ) ;
            errorComp = ef_EMAILADDR ;
         }


         if ( errorStr != null ) {
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), errorStr ) ;
            errorComp.requestFocus() ;
         } else {
            custRec.setFieldData ( CUSTFIELD_FIRSTNAME,    fName ) ;
            custRec.setFieldData ( CUSTFIELD_LASTNAME,     lName ) ;
            custRec.setFieldData ( CUSTFIELD_MIDDLEINIT,   middleInit ) ;
            custRec.setFieldData ( CUSTFIELD_PASSWORD,     password ) ;
            custRec.setFieldData ( CUSTFIELD_USERID,       userId ) ;
            custRec.setFieldData ( CUSTFIELD_EMAILADDR,    eMailAddr ) ;
            rc = true ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         rc = false ;
      }

      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param e The action event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_SAVE                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_SAVE ) {

         custRec.updateRecStatus ( custRec.REC_UPDATE ) ;

         if ( saveData() ) {
            if ( custRec.saveToDatabase() )
               GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_ADDMACHINE                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADDMACHINE ) {
         CustomerMachineRec machineRec = new CustomerMachineRec() ;

         machineRec.updateRecStatus ( machineRec.REC_INSERT ) ;
         machineRec = new MachineDlg ( parentWin, machineRec ).getResult() ;

         if ( machineRec != null ) {
            custRec.addMachineRec ( machineRec ) ;
            cnr_MACHINES.removeAll() ;
            cnr_MACHINES.setData ( (Vector) custRec.getFieldData ( CUSTFIELD_ALLMACHINE ) ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_EDITMACHINE                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDITMACHINE ) {
         CustomerMachineRec machineRec = (CustomerMachineRec) cnr_MACHINES.getSelectedItem() ;

         if ( machineRec != null ) {
            machineRec = new MachineDlg ( parentWin, machineRec ).getResult() ;

            if ( machineRec != null )
               cnr_MACHINES.repaint() ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_DELETEMACHINE                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DELETEMACHINE ) {
         CustomerMachineRec machineRec = (CustomerMachineRec) cnr_MACHINES.getSelectedItem() ;

         if ( machineRec != null ) {
            custRec.removeMachineRec ( machineRec ) ;
            cnr_MACHINES.repaint() ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;

   } /* actionPerformed  */


   public boolean stop () {
      return( true ) ;
   }

   public boolean shutdown () {
      return( true ) ;
   }

   public JComponent getJComponent () {
      return( this ) ;
   }

   public void panelDeselected () {
   }
   public void panelSelected () {
   }

   public boolean copyText () {
      return( false ) ;
   }
   public boolean cutText () {
      return( false ) ;
   }
   public boolean pasteText () {
      return( false ) ;
   }

   public void refresh () {
   }


   public boolean bookmark () {
      return( false ) ;
   }


   public ImageIcon getIcon () {
      return ( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_CLIO ) ) ;
   }

   public NavPanel getDefaultNavPanel () {
      return ( null ) ;
   }


   public String getName () {
      if ( custRec != null )
         return ( custRec.getFirstName()+" "+custRec.getLastName() ) ;

      return ( Str.getStr ( STR_EDIT ) ) ;
   }

} /* CustEditPanel */
