package com.ibm.nzna.projects.qit.customer.custedit ;
/*-----------------------------------------------------------*/
/*   MachineDlg                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: MachineDlg                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog holds information about a user's specific machine 
 *
 *
 * @version 1.0, 9/15/1999 
 * @author  David Lentz
 */
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.qit.customer.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   MachineDlg                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class MachineDlg extends JDialog implements DataLengths, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_MACHINETYPE       = null ;
   private JLabel       st_MODEL             = null ;
   private JLabel       st_SERIALNUMBER      = null ;
   private JLabel       st_BRAND             = null ;
   private JLabel       st_FAMILY            = null ;
   private JLabel       st_OPERATINGSYS      = null ;
   private JLabel       st_MACHINENUM        = null ;
   private JLabel       st_RECNUM            = null ;
   private JTextField   ef_MACHINETYPE       = null ;
   private JTextField   ef_MODEL             = null ;
   private JTextField   ef_SERIALNUMBER      = null ;
   private JTextField   ef_OPERATINGSYS      = null ;
   private JComboBox    cb_BRAND             = null ;
   private JComboBox    cb_FAMILY            = null ;
   private JCheckBox    ck_DEFAULTMACH       = null ;
   private DButton      pb_OK                = null ;
   private DButton      pb_CANCEL            = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private CustomerMachineRec machineRec     = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor for the window.
    *  It will return immediatly without showing the dialog.
    *  To show the dialog, call the getResult method
    *
    *  @param customerMachineRec The customer machine record to edit
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public MachineDlg ( Frame parent, CustomerMachineRec machineRec ) {
      super ( parent, Str.getStr ( STR_MACHINE ), true ) ;

      this.machineRec = machineRec ;

      createControls() ;
      setSize ( 300, 210 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
   }




   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will return the CustomerMachineRec
    *  that the user has modified.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomerMachineRec getResult() {
      refreshData() ;
      setVisible ( true ) ;
      return( machineRec ) ;
   }





   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will attach all of the controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createControls () {
      Container contentPane = (Container) getContentPane() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_MACHINETYPE  = new JLabel ( Str.getStr ( STR_MACHINE ) ) ;
      st_MODEL        = new JLabel ( Str.getStr ( STR_MODEL ) ) ;
      st_SERIALNUMBER = new JLabel ( Str.getStr ( STR_SERIAL_NUMBER ) ) ;
      st_BRAND        = new JLabel ( Str.getStr ( STR_BRAND ) ) ;
      st_FAMILY       = new JLabel ( Str.getStr ( STR_FAMILY ) ) ;
      st_OPERATINGSYS = new JLabel ( Str.getStr ( STR_OS ) ) ;
      st_MACHINENUM   = new JLabel ( Str.getStr ( STR_MACHINE_NUM ) ) ;
      st_RECNUM       = new JLabel ( Str.getStr ( STR_REC_NUM ) ) ;
      ef_MACHINETYPE  = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 4 ), "", 0 ) ;
      ef_MODEL        = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 3 ), "", 0 ) ;
      ef_SERIALNUMBER = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 25 ), "", 0 ) ;
      ef_OPERATINGSYS = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 12 ), "", 0 ) ;
      cb_BRAND        = new JComboBox ( Brands.getBrands() ) ;
      cb_FAMILY       = new JComboBox ( TypeCategory.getCategories ( TypeCategory.LEVEL_FAMILY ) ) ;
      pb_OK           = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL       = new DButton ( Str.getStr ( STR_CANCEL ) ) ;                           
      ck_DEFAULTMACH  = new JCheckBox ( Str.getStr ( STR_DEFAULT ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our content pane                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_MACHINENUM ) ;
      contentPane.add ( st_RECNUM ) ;
      contentPane.add ( st_MACHINETYPE ) ;
      contentPane.add ( ef_MACHINETYPE ) ;
      contentPane.add ( st_MODEL ) ;
      contentPane.add ( ef_MODEL ) ;
      contentPane.add ( st_SERIALNUMBER ) ;
      contentPane.add ( ef_SERIALNUMBER ) ;
      contentPane.add ( st_OPERATINGSYS ) ;
      contentPane.add ( ef_OPERATINGSYS ) ;
      contentPane.add ( st_BRAND ) ;
      contentPane.add ( cb_BRAND ) ;
      contentPane.add ( st_FAMILY ) ;
      contentPane.add ( cb_FAMILY ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_CANCEL ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the listeners                                       */        
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      
   } /* createControls  */





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       x         = 0 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight() ;
      Dimension size      = getSize() ;

      super.doLayout() ;

      st_MACHINENUM.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      st_RECNUM.setBounds     ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      st_MACHINETYPE.setBounds ( x, y, 130, rowHeight ) ;
      ef_MACHINETYPE.setBounds ( x+130, y, 130, rowHeight ) ;
      y+=rowHeight ;

      st_MODEL.setBounds ( x, y, 130, rowHeight ) ;
      ef_MODEL.setBounds ( x+130, y, 130, rowHeight ) ;
      y+=rowHeight ;

      st_SERIALNUMBER.setBounds ( x, y, 130, rowHeight ) ;
      ef_SERIALNUMBER.setBounds ( x+130, y, 130, rowHeight ) ;
      y+=rowHeight ;

      st_OPERATINGSYS.setBounds ( x, y, 130, rowHeight ) ;
      ef_OPERATINGSYS.setBounds ( x+130, y, 130, rowHeight ) ;
      y+=rowHeight ;

      st_BRAND.setBounds ( x, y, 130, rowHeight ) ;
      cb_BRAND.setBounds ( x+130, y, 130, rowHeight ) ;
      y+=rowHeight ;

      st_FAMILY.setBounds ( x, y, 130, rowHeight ) ;
      cb_FAMILY.setBounds ( x+130, y, 130, rowHeight ) ;
      y+=rowHeight+5 ;


      pb_OK.setBounds ( x, y, 70, 25 ) ;
      pb_CANCEL.setBounds ( x+75, y, 80, 25 ) ;
   }






   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will populate our controls
    *  with the data in the machineRec
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      st_MACHINENUM.setText ( Str.getStr ( STR_MACHINE )+": "+machineRec.getMachineNum() ) ;
      st_RECNUM.setText     ( Str.getStr ( STR_REC_NUM )+": "+machineRec.getRecNum() ) ;
      ef_MACHINETYPE.setText ( machineRec.getMachineType() ) ;
      ef_MODEL.setText ( machineRec.getModel() ) ;
      ef_OPERATINGSYS.setText ( machineRec.getOperatingSystem() ) ;
      ef_SERIALNUMBER.setText ( machineRec.getSerialNumber() ) ;

      try {
         cb_BRAND.setSelectedItem  ( Brands.getBrand ( machineRec.getBrand() ) ) ;
         cb_FAMILY.setSelectedItem ( Brands.getFamily ( machineRec.getFamily() ) ) ;
      } catch ( Exception e ) {
      }

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == pb_OK ) {
         if ( saveData() )
            dispose() ;
      }

      else if ( e.getSource() == pb_CANCEL ) {
         machineRec = null ;
         dispose() ;
      }
   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save the data from the
    *  screen to the machineRec
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if all data was good, FALSE if not
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      boolean   rc           = false ;
      String    machineType  = ef_MACHINETYPE.getText() ;
      String    model        = ef_MODEL.getText() ;
      String    os           = ef_OPERATINGSYS.getText() ;
      String    serialNumber = ef_SERIALNUMBER.getText() ;
      String    brand        = null ;
      String    family       = null ;
      String    errorStr     = null ;
      boolean   defaultMach  = false ;      
      Component errorComp    = null ;


      if ( cb_BRAND.getSelectedItem() != null )
         brand = cb_BRAND.getSelectedItem().toString() ;

      if ( cb_FAMILY.getSelectedItem() != null )
         family = cb_FAMILY.getSelectedItem().toString() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Validate the data                                       */        
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( machineType == null ) || ( machineType.length() == 0 ) ) {
         errorStr = Str.getStr ( STR_MUST_ENTER_MACHINE ) ;
         errorComp = ef_MACHINETYPE ;
      }

      else if ( ( model == null ) || ( model.length() == 0 ) ) {
         errorStr = Str.getStr ( STR_MUST_ENTER_MODEL ) ;
         errorComp = ef_MODEL ;
      }

      else if ( ( os == null ) || ( os.length() == 0 ) ) {
         errorStr = Str.getStr ( STR_MUST_ENTER_OS ) ;
         errorComp = ef_OPERATINGSYS ;
      }

      else if ( ( brand == null ) || ( brand.length() == 0 ) ) {
         errorStr = Str.getStr ( STR_MUST_SELECT_BRAND ) ;
         errorComp = cb_BRAND ;
      }

      else if ( ( family == null ) || ( family.length() == 0 ) ) {
         errorStr = Str.getStr ( STR_MUST_SELECT_GROUP ) ;
         errorComp = cb_FAMILY ;
      }

      if ( errorStr != null ) {
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), errorStr ) ;
         errorComp.requestFocus() ;
      }

      else {
         rc = true ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Copy over our data                                      */         
         /*                                                           */
         /*-----------------------------------------------------------*/
         machineRec.setMachineType     ( machineType ) ;
         machineRec.setModel           ( model ) ;
         machineRec.setSerialNumber    ( serialNumber ) ;
         machineRec.setOperatingSystem ( os ) ;
         machineRec.setBrand           ( brand ) ;
         machineRec.setFamily          ( family ) ;
         machineRec.updateRecStatus ( machineRec.REC_UPDATE ) ;
         
      }

      return ( rc ) ;
   }

} /* MachineDlg */
