package com.ibm.nzna.projects.qit.customer.custmaint ;
/*-----------------------------------------------------------*/
/*   CustMaintPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustMaintPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel allows a user to search for customers 
 *
 *
 * @version 1.0, 8/5/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.customer.* ;
import com.ibm.nzna.projects.qit.customer.custview.* ;
import com.ibm.nzna.projects.qit.customer.custedit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   CustMaintPanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustMaintPanel extends JPanel implements CustomerViewThreadListener, QuestPanel, AppConst, Runnable, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   THREADS                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_INITIALIZE = "Init" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Statics                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] listTitle = { 
      Str.getStr ( STR_CUSTNUM ),
      Str.getStr ( STR_NAME ),
      Str.getStr ( STR_EMAIL_ADDRESS ),
      Str.getStr ( STR_USERID ),
      Str.getStr ( STR_LAST_TOUCHED_ON ),
      Str.getStr ( STR_LAST_TOUCHED_BY )} ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle            st_TITLE    = null ;
   private JTitle            st_STATS    = null ;
   private DButton           pb_IMPORT   = null ;
   private ActionButton      pb_EDIT     = null ;
   private ActionButton      pb_CLOSE    = null ;
   private ActionButton      pb_DELETE   = null ;
   private MultiList         customers   = null ;
   private JPanel            pnl_LIST    = null ;
   private CustMaintNavPanel navPanel    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private AppDefaultWin parentWin            = null ;
   private String        currentView          = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create our blank page and will kick off our
    * init thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustMaintPanel () {
      new Thread ( this, THREAD_INITIALIZE ).start() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize() {
      JPanel textPanel = new JPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      customers = new MultiList ( GUISystem.getFontUtil() ) ;
      pnl_LIST  = new JPanel () ;
      st_TITLE  = new JTitle ( Str.getStr ( STR_WEB_CUSTOMER_MGMT ) ) ;
      st_STATS  = new JTitle ( "No Statistics" ) ;                                 
      pb_EDIT   = new ActionButton ( Str.getStr ( STR_EDIT ),   ImageSystem.getImageIcon ( this, ImageSystem.EDIT   ) ,   Str.getStr(STR_EDIT) ) ;
      pb_DELETE = new ActionButton ( Str.getStr ( STR_DELETE ), ImageSystem.getImageIcon ( this, ImageSystem.DELETE ) ,   Str.getStr(STR_DELETE) ) ;
      pb_CLOSE  = new ActionButton ( Str.getStr ( STR_CLOSE ),  ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ) ,    Str.getStr(STR_CLOSE) ) ;
      navPanel  = new CustMaintNavPanel ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls to the panel                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      pnl_LIST.setLayout ( new BorderLayout () ) ;
      textPanel.setLayout ( new BorderLayout () ) ;
      textPanel.add ( st_STATS, BorderLayout.CENTER ) ;
      textPanel.add ( st_TITLE, BorderLayout.NORTH ) ;

      add ( pnl_LIST, BorderLayout.CENTER ) ;
      pnl_LIST.add ( textPanel,     BorderLayout.NORTH ) ;
      pnl_LIST.add ( customers,  BorderLayout.CENTER ) ;                                   



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we texture the background, set the images            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      customers.setBorder ( GUISystem.emptyBorder ) ;
      st_TITLE.setOpaque ( false ) ;
      st_STATS.setOpaque ( false ) ;
      pnl_LIST.setOpaque ( false ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      customers.setColumnHeadings ( listTitle ) ;
      customers.setColumnWidth ( 0, 75 ) ;
      customers.setColumnWidth ( 1, 200 ) ;
      customers.setColumnWidth ( 2, 100 ) ;
      customers.setColumnWidth ( 3, 75 ) ;
      customers.setColumnWidth ( 4, 100 ) ;
      customers.setColumnWidth ( 5, 100 ) ;
      customers.setOddLinesColor ( new Color ( 240, 240, 240 ) ) ;
      st_STATS.setFont ( FontSystem.smallTitleFont ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener  ( this ) ;
      pb_EDIT.addActionListener   ( this ) ;
      pb_DELETE.addActionListener ( this ) ;
      customers.addActionListener ( this ) ;


      return( true ) ;
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread does a variety of operations. The
    *  operation that is performed is named in the thread
    *  name
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName () ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_INITIALIZE                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( name.equals ( THREAD_INITIALIZE ) ) {
         initialize() ;
      }

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to action events
    *  
    *  
    *
    *  @param e The action event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e  ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == pb_EDIT ) || ( e.getSource() == customers ) )
         editCustomer() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_DELETE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DELETE ) {
         if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_CUSTOMER ) )
            deleteCustomers() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CLOSE )
         GUISystem.getParentDefWin ( this ).closePanel ( this, null ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshView
    */
   /**
    *    This method will refresh our list by kicking off the thread
    *  that will read the customers.
    *  
    *
    *  @param viewStr The View to refresh with
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void refreshView ( String viewStr ) {
      String searchArg  = null ;
      String searchArg2 = null ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our current view                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      currentView = viewStr ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   By Customer Name                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( viewStr.equals ( Str.getStr ( STR_CUST_BY_NAME ) ) ) {
         String tempStr = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                               Str.getStr ( STR_CUST_BY_NAME ),
                                               Str.getStr ( STR_NAME ),
                                               "",
                                               new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;

         /* Break up Temp Str into first name, last name */
         if ( tempStr != null ) {
            if ( tempStr.indexOf ( " " ) > 0 ) {
               searchArg  = tempStr.substring( 0, tempStr.indexOf ( " " ) ) ;
               searchArg2 = tempStr.substring ( tempStr.indexOf ( " " )+1, tempStr.length() ) ;
            } else
               searchArg = tempStr ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   By Customer eMail                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( viewStr.equals ( Str.getStr ( STR_CUST_BY_EMAIL_ADDR ) ) ) {
         String tempStr = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                               Str.getStr ( STR_CUST_BY_EMAIL_ADDR ),
                                               Str.getStr ( STR_EMAIL_ADDRESS ),
                                               "",
                                               new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;

         searchArg = tempStr ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Customer By Customer User Id                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( viewStr.equals ( Str.getStr ( STR_CUST_BY_USERID ) ) ) {
         String tempStr = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                               Str.getStr ( STR_CUST_BY_USERID ),
                                               Str.getStr ( STR_USERID ),
                                               "",
                                               new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;

         searchArg = tempStr ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Customer By Machine/Model                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( viewStr.equals ( Str.getStr ( STR_CUST_BY_MACHINE_MODEL ) ) ) {
         String tempStr = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                               Str.getStr ( STR_CUST_BY_MACHINE_MODEL ),
                                               Str.getStr ( STR_MACHINE_MODEL ),
                                               "",
                                               new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 12 ) ).getResult() ;

         searchArg = tempStr ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Customer By Machines                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( viewStr.equals ( Str.getStr ( STR_CUST_BY_MACHINE ) ) ) {
         String tempStr = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                               Str.getStr ( STR_CUST_BY_MACHINE ),
                                               Str.getStr ( STR_MACHINE ),
                                               "",
                                               new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 12 ) ).getResult() ;

         searchArg = tempStr ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Customer By Customer Number                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( viewStr.equals ( Str.getStr ( STR_CUST_BY_CUSTNUM ) ) ) {
         String tempStr = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                               Str.getStr ( STR_CUST_BY_CUSTNUM ),
                                               Str.getStr ( STR_CUSTNUM ),
                                               "",
                                               new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 12 ) ).getResult() ;

         searchArg = tempStr ;
      }



      if ( searchArg != null ) {
         new CustomerViewThread ( viewStr, searchArg, searchArg2, this ).setAppDefaultWin ( parentWin ) ;

      }
   }


   /*------------------------------------------------------------*/
   /* editCustomer
    */
   /**
    *    This method will read in the customer information
    *  and then set the view to the Customer Editor.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editCustomer () {
      CustomerRec custRec = (CustomerRec) customers.getSelectedItem() ;

      parentWin.setStatus ( Str.getStr ( STR_READING ) ) ;
      if ( custRec != null ) {
         if ( custRec.readFromDatabase() == 1 ) {
            CustEditPanel editPanel = new CustEditPanel ( custRec, this ) ;

            WindowSystem.createPanel ( editPanel ) ;
         }
      }

      parentWin.setStatus ( null );
   }





   /*------------------------------------------------------------*/
   /* deleteCustomers
    */
   /**
    *    This method will delete all selected
    *  customers
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void deleteCustomers () {
      Vector vec = customers.getSelection() ;

      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int     i       = 0 ;
         int     length  = vec.size() ;
         boolean cont    = true ;

         while ( ( cont) && ( i < length ) ) {
            ((CustomerRec)vec.elementAt(i)).updateRecStatus ( CustomerRec.REC_DELETE ) ;
            cont = ((CustomerRec)vec.elementAt(i)).saveToDatabase() ;
            customers.remove ( (MultiListRow)vec.elementAt(i) ) ;
            i++ ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our small icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_CLIO ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_WEB_CUSTOMER_MGMT ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default
    *  navigation panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( navPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called when we are
    *  showing and should finish initializing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      parentWin = GUISystem.getParentDefWin ( this ) ;

      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_EDIT ) ;
      parentWin.addActionComponent ( this, pb_DELETE ) ;
   }

   public boolean stop () {
      return( true ) ;
   }

   public boolean shutdown () {
      return( true ) ;
   }

   public JComponent getJComponent () {
      return( this ) ;
   }

   public void panelDeselected () {
   }
   public void panelSelected () {
   }

   public boolean copyText () {
      return( false ) ;
   }
   public boolean cutText () {
      return( false ) ;
   }
   public boolean pasteText () {
      return( false ) ;
   }

   public void refresh () {
      if ( currentView != null )
         refreshView ( currentView ) ;
   }


   public boolean bookmark () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* setListData
    */
   /**
    *    This method will set the data that we are
    *  displaying
    *  
    *
    *  @param title The title of the data (In Human readable form)
    *  @param data The Vector of Customers to show
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customerViewComplete ( String title, Vector data ) {
      st_TITLE.setText ( title ) ;

      if ( data != null ) {
         customers.removeAll() ;
         st_STATS.setText ( Str.getStr ( STR_VIEWING )+" "+data.size() ) ;
         customers.add ( data ) ;
      }
   }
} /* CustMaintPanel */
