package quest.clio.custmaint ;                          
/*-----------------------------------------------------------*/
/*   RefreshCustListThread                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: RefreshCustListThread               */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This thread reads in and refreshes an entire list of customers 
 *
 *
 * @version 1.0, 8/25/1999 
 * @author  David Lentz
 */
import quest.base.* ;
import quest.sql.* ;
import quest.clio.customer.* ;
import java.util.*;
import java.sql.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   RefreshCustListThread                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class RefreshCustListThread extends Thread implements Const, CustomerConstants {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector         searchVec    = null ;
   private SearchRec      searchRec    = null ;
   private String         searchStr    = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our thread and start
    *  it.
    *  
    *
    *  @param searchRec The search record to use when reading the customers.
    *  @param searchStr The string to search on. This is the string that is visible in the Cust Maint. View list
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public RefreshCustListThread ( SearchRec searchRec, String searchStr ) {
      this.searchRec = searchRec ;
      this.searchStr = searchStr ;
      start() ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will actually run our thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      SystemData.enableApp ( false ) ;

      if ( searchStr != null )
         readCustList () ;

      if ( searchRec.listener != null ) {
         searchRec.resultVec = searchVec ;
         searchRec.listener.searchComplete ( SEARCH_CUSTOMER, searchRec ) ;
      }

      SystemData.enableApp ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* readCustList
    */
   /**
    *    This method will do the actual reading
    *  of customers
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readCustList () {
      Database sql     = SystemData.getDatabase ( DATABASE_CLIO ) ;
      String   strStmt = null ;
      String   rtnName = "readCustList" ;


      SystemData.setStatus ( Str.getStr ( STR_PLEASEWAIT ) ) ;

      try {
         Statement stmt = sql.beginSQL ( rtnName ) ;          


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, set up our SQL                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer Name                                        */     
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( searchStr.equals ( Str.getStr ( STR_CUSTBYNAME ) ) ) 
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B "+
                      "WHERE  A.FNAME     = '"+searchRec.strSearchArg+"' AND   "+
                      "       A.LNAME     = '"+searchRec.strSearchArg2+"' AND  "+
                      "       A.EDITFLAG  = 'Y' AND                            "+
                      "       A.SHOW      = 'Y' AND                            "+
                      "       A.CUSTNUM   = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer User Id                                     */      
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( searchStr.equals ( Str.getStr ( STR_CUSTBYUSERID ) ) ) 
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B "+
                      "WHERE  B.CUSTUSERID = '"+searchRec.strSearchArg+"' AND   "+
                      "       A.EDITFLAG   = 'Y' AND                            "+
                      "       A.SHOW       = 'Y' AND                            "+
                      "       A.CUSTNUM    = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer eMail                                       */      
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( searchStr.equals ( Str.getStr ( STR_CUSTBYEMAILADDR) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B "+
                      "WHERE  A.EMAILADDR = '"+searchRec.strSearchArg+"' AND   "+
                      "       A.EDITFLAG  = 'Y' AND                            "+
                      "       A.SHOW      = 'Y' AND                            "+
                      "       A.CUSTNUM   = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer Cust Num                                    */      
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( searchStr.equals ( Str.getStr ( STR_CUSTBYCUSTNUM) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B "+
                      "WHERE  A.CUSTNUM   = "+searchRec.strSearchArg+" AND   "+
                      "       A.EDITFLAG  = 'Y' AND                          "+
                      "       A.SHOW      = 'Y' AND                          "+
                      "       A.CUSTNUM   = B.CUSTNUM                        "+
                      "FOR FETCH ONLY" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer Machine Model                               */      
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( searchStr.equals ( Str.getStr ( STR_CUSTBYMACHINEMODEL) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B, "+
                      "       CLIO.CUSTMACH C "+
                      "WHERE  C.SYSPRODNUM = '"+searchRec.strSearchArg+"' AND   "+
                      "       A.CUSTNUM    = C.CUSTNUM AND                      "+
                      "       A.EDITFLAG   = 'Y' AND                            "+
                      "       A.SHOW       = 'Y' AND                            "+
                      "       A.CUSTNUM    = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer Machine                                     */      
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( searchStr.equals ( Str.getStr ( STR_CUSTBYMACHINE) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B, "+
                      "       CLIO.CUSTMACH C "+
                      "WHERE  C.MACHINE    = '"+searchRec.strSearchArg+"' AND   "+
                      "       A.CUSTNUM    = C.CUSTNUM AND                      "+
                      "       A.EDITFLAG   = 'Y' AND                            "+
                      "       A.SHOW       = 'Y' AND                            "+
                      "       A.CUSTNUM    = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;

         if ( strStmt != null ) {
            ResultSet    rs          = stmt.executeQuery ( strStmt ) ;
            CustomerRec  readCustRec = null ;
            int          i           = 0 ;

            searchVec = new Vector ( 10, 10 ) ;

            while ( rs.next() ) {

               readCustRec = new CustomerRec () ;
               readCustRec.setCustNum ( rs.getInt(1) ) ;
               readCustRec.setFieldData ( CUSTFIELD_FIRSTNAME,   rs.getString(2) ) ;
               readCustRec.setFieldData ( CUSTFIELD_LASTNAME,    rs.getString(3) ) ;
               readCustRec.setFieldData ( CUSTFIELD_EMAILADDR,   rs.getString(4) ) ;
               readCustRec.setFieldData ( CUSTFIELD_USERID,      rs.getString(5) ) ;
               readCustRec.setFieldData ( CUSTFIELD_DBUSER,      rs.getString(6) ) ;
               readCustRec.setFieldData ( CUSTFIELD_CHANGEDTIME, rs.getString(7) ) ;

               searchVec.addElement ( readCustRec ) ;
               i++ ;
               SystemData.setStatus ( Str.getStr ( STR_READING )+" "+i ) ;
            }
         }

         sql.endSQL ( rtnName, stmt ) ;

      } catch ( Exception e ) {
         sql.rollBack() ;
         SystemData.log ( e, "RefreshCustListThread", rtnName ) ;
      }


      SystemData.setStatus ( null ) ;
   }

} /* RefreshCustListThread */
