package com.ibm.nzna.projects.qit.customer.custview ;
/*-----------------------------------------------------------*/
/*   CustomerViewThread                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomerViewThread                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This thread reads in and refreshes an entire list of customers 
 *
 *
 * @version 1.0, 8/25/1999 
 * @author  David Lentz
 */
import java.util.*;
import java.sql.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.customer.* ;
import com.ibm.nzna.projects.qit.gui.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomerViewThread                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomerViewThread extends Thread implements QuestProcess, AppConst, CustomerConstants {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String                     viewName      = null ;
   private String                     searchArg     = null ;
   private String                     searchArg2    = null ;
   private CustomerViewThreadListener listener      = null ;
   private Vector                     results       = null ;
   private boolean                    continueFetch = true ;
   private AppDefaultWin              defaultWin    = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our thread and start
    *  it.
    *  
    *
    *  @param viewName The name of the view to search on
    *  @param searchArg The first argument to search with
    *  @param listener The listener to notify once the search is complete
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomerViewThread ( String viewName, String searchArg, CustomerViewThreadListener listener  ) {
      this.viewName  = viewName ;
      this.searchArg = searchArg ;
      this.listener  = listener ;
      start() ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our thread and start
    *  it.
    *  
    *
    *  @param viewName The name of the view to search on
    *  @param searchArg The first argument to search with
    *  @param searchArg2 For views that require a second search argument (FirstName-LastName etc..)
    *  @param listener The listener to notify once the search is complete
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomerViewThread ( String viewName, String searchArg, String searchArg2, CustomerViewThreadListener listener  ) {
      this.viewName   = viewName ;
      this.searchArg  = searchArg ;
      this.searchArg2 = searchArg2 ;
      this.listener   = listener ;
      start() ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will actually run our thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      ProcessManagerSystem.addProcess ( this ) ;
      setStatus ( Str.getStr ( STR_READING ) ) ;
      try {

         readCustList () ;

         if ( listener != null )
            listener.customerViewComplete ( viewName, results ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }


      setStatus ( null ) ;
      ProcessManagerSystem.removeProcess ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* readCustList
    */
   /**
    *    This method will do the actual reading
    *  of customers
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readCustList () {
      String     rtnName = "readCustList" ;
      SQLMethod  sql     = new SQLMethod ( DatabaseSystem.CLIO, rtnName, 30 ) ;
      String     strStmt = null ;


      try {
         Statement stmt = sql.createStatement() ;     


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, set up our SQL                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer Name                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( viewName.equals ( Str.getStr ( STR_CUST_BY_NAME ) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B "+
                      "WHERE  A.FNAME     = '"+searchArg+"' AND   "+
                      "       A.LNAME     = '"+searchArg2+"' AND  "+
                      "       A.EDITFLAG  = 'Y' AND                            "+
                      "       A.SHOW      = 'Y' AND                            "+
                      "       A.CUSTNUM   = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer User Id                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( viewName.equals ( Str.getStr ( STR_CUST_BY_USERID ) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B "+
                      "WHERE  B.CUSTUSERID = '"+searchArg+"' AND   "+
                      "       A.EDITFLAG   = 'Y' AND                            "+
                      "       A.SHOW       = 'Y' AND                            "+
                      "       A.CUSTNUM    = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer eMail                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( viewName.equals ( Str.getStr ( STR_CUST_BY_EMAIL_ADDR) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B "+
                      "WHERE  A.EMAILADDR = '"+searchArg+"' AND   "+
                      "       A.EDITFLAG  = 'Y' AND                            "+
                      "       A.SHOW      = 'Y' AND                            "+
                      "       A.CUSTNUM   = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer Cust Num                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( viewName.equals ( Str.getStr ( STR_CUST_BY_CUSTNUM) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B "+
                      "WHERE  A.CUSTNUM   = "+searchArg+" AND   "+
                      "       A.EDITFLAG  = 'Y' AND                          "+
                      "       A.SHOW      = 'Y' AND                          "+
                      "       A.CUSTNUM   = B.CUSTNUM                        "+
                      "FOR FETCH ONLY" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer Machine Model                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( viewName.equals ( Str.getStr ( STR_CUST_BY_MACHINE_MODEL) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B, "+
                      "       CLIO.CUSTMACH C "+
                      "WHERE  C.SYSPRODNUM = '"+searchArg+"' AND   "+
                      "       A.CUSTNUM    = C.CUSTNUM AND                      "+
                      "       A.EDITFLAG   = 'Y' AND                            "+
                      "       A.SHOW       = 'Y' AND                            "+
                      "       A.CUSTNUM    = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   By Customer Machine                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( viewName.equals ( Str.getStr ( STR_CUST_BY_MACHINE) ) )
            strStmt = "SELECT A.CUSTNUM, A.FNAME, A.LNAME, A.EMAILADDR, B.CUSTUSERID, "+
                      "       A.DBUSER,  A.CHANGEDTIME "+
                      "FROM   CLIO.CUSTNUM A, "+
                      "       CLIO.CUSTPASSWD B, "+
                      "       CLIO.CUSTMACH C "+
                      "WHERE  C.MACHINE    = '"+searchArg+"' AND   "+
                      "       A.CUSTNUM    = C.CUSTNUM AND                      "+
                      "       A.EDITFLAG   = 'Y' AND                            "+
                      "       A.SHOW       = 'Y' AND                            "+
                      "       A.CUSTNUM    = B.CUSTNUM                          "+
                      "FOR FETCH ONLY" ;

         if ( strStmt != null ) {
            ResultSet    rs          = stmt.executeQuery ( strStmt ) ;
            CustomerRec  readCustRec = null ;
            int          i           = 0 ;

            results = new Vector ( 10, 10 ) ;

            while ( ( continueFetch ) && ( rs.next() ) ) {

               readCustRec = new CustomerRec () ;
               readCustRec.setCustNum ( rs.getInt(1) ) ;
               readCustRec.setFieldData ( CUSTFIELD_FIRSTNAME,   rs.getString(2) ) ;
               readCustRec.setFieldData ( CUSTFIELD_LASTNAME,    rs.getString(3) ) ;
               readCustRec.setFieldData ( CUSTFIELD_EMAILADDR,   rs.getString(4) ) ;
               readCustRec.setFieldData ( CUSTFIELD_USERID,      rs.getString(5) ) ;
               readCustRec.setFieldData ( CUSTFIELD_DBUSER,      rs.getString(6) ) ;
               readCustRec.setFieldData ( CUSTFIELD_CHANGEDTIME, rs.getString(7) ) ;

               results.addElement ( readCustRec ) ;
               setStatus ( Str.getStr ( STR_READING )+"  "+i ) ;
               i++ ;
            }
         }

      } catch ( Exception e ) {
         sql.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sql.close() ;
   }


   public String getProcessName () {
      return( "Customer View Thread" ) ;
   }

   public void stopProcess () {
      continueFetch = false ;
   }





   /*------------------------------------------------------------*/
   /* setAppDefaultWin
    */
   /**
    *    This method will set the AppDefaultWin
    *  property. All messages from this thread will be displayed
    *  in the status area of the AppDefaultWin passed
    *
    *  @param appDefaultWin The AppDefaultWin to pass messages to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAppDefaultWin ( AppDefaultWin defaultWin ) {
      this.defaultWin = defaultWin ;
   }





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the status on our
    *  AppDefaultWin property if that property
    *  has a valid value
    *
    *  @param status The status to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setStatus ( String status ) {
      if ( defaultWin != null )
         defaultWin.setStatus ( status ) ;
   }



} /* CustomerViewThread */
