package com.ibm.nzna.projects.qit.customview ;  
/*-----------------------------------------------------------*/
/*   CustomView                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomView                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This interface defines the methods needed to maintain a Custom View in a database 
 *<br><br>
 * Please Note! That the ViewInd MUST BE UNIQUE. Currently the following numbers are already
 * used:<br><br>
 *
 * Avalon uses 1-5.
 * Documents use 6
 * <br><br>
 * Please add to this comment field if you reserve another number. REMEMBER THIS NUMBER IS GENERATED
 * in the QIT.TYPECUSTOMVIEW table
 *
 * @version 1.0, 4/29/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomView                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public abstract class CustomView extends com.ibm.nzna.shared.pom.PersistentRec implements MultiListRow {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int    teamInd     = -1 ;
   private int    viewInd     = 0 ;
   private int    viewTypeInd = 0 ;
   private String descript    = null ;
   public  Vector fields      = null ;




   /*------------------------------------------------------------*/
   /* CustomView
    */
   /**
    *    Constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomView () {
   }


   /*------------------------------------------------------------*/
   /* getSelect
    */
   /**
    *    This method should return the select
    *  clause that the custom query will run including the 'SELECT' keyword
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public abstract String getSelect () ;




   /*------------------------------------------------------------*/
   /* getTables
    */
   /**
    *    This method should return the table clause
    *  of a select statement. This includes the 'FROM'
    *  keyword, and all tables
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public abstract String getTables () ;




   /*------------------------------------------------------------*/
   /* getWhere
    */
   /**
    *    This method should return the entire where
    *  clause including the 'WHERE' keyword
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public abstract String getWhere () ;





   /*------------------------------------------------------------*/
   /* setViewType
    */
   /**
    *    This method will set the viewType that this
    *  view represents
    *  
    *
    *  @param viewTypeInd The View Type Indicator as defined in QIT.TYPECUSTOMVIEW
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setViewType ( int viewTypeInd ) {
      this.viewTypeInd = viewTypeInd ;
   }



   /*------------------------------------------------------------*/
   /* getViewType
    */
   /**
    *    This method should return the type of query
    *  that this is. The Type should be defined in
    *  QIT.TYPECUSTOMVIEW
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public final int getViewType () {
      return( viewTypeInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getQuery
    */
   /**
    *    This method should return the entire query
    *  that will perform this custom view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract String getQuery ( ) ;




   /*------------------------------------------------------------*/
   /* getResults
    */
   /**
    *    This method should return the results of this
    *  custom query in a Vector
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public abstract Vector getResults () ;




   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    This method should return the amount of
    *  columns specified in the query
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public abstract int getColumnCount () ;




   /*------------------------------------------------------------*/
   /* instantiateObject
    */
   /**
    *    This method should return a new object
    *  that implements CustomView with the objectType
    *  passed in
    *
    *  @param objectType The Object Type to return a new CustomView for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract CustomView instantiateObject ( int objectType ) ;






   /*------------------------------------------------------------*/
   /* getColumnWidths
    */
   /**
    *    This method should return the width of each
    *  column in pixels. Each integer in the array
    *  specified the amount of pixels needed to display that column in a list
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int[]
    *//*---------------------------------------------------------*/
   public abstract int[] getColumnWidths () ;





   /*------------------------------------------------------------*/
   /* getFields
    */
   /**
    *    This method should return all of the CustomViewField
    *  objects that this query has.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public final Vector getFields () {
      return( fields ) ;
   }




   /*------------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    This method will set the description
    *  of this record
    *  
    *
    *  @param descript The new description for this view
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setDescript ( String descript ) {
      this.descript = descript ;
   }





   /*------------------------------------------------------------*/
   /* getDescript
    */
   /**
    *    This method should return the description
    *  of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final String getDescript () {
      return( descript ) ;
   }




   /*------------------------------------------------------------*/
   /* getTeamInd
    */
   /**
    *    This method should return the team indicator
    *  that this view is available to. It should return
    *  a -1 if no team is specfiied.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public final int getTeamInd () {
      return( teamInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setTeamInd
    */
   /**
    *    This method should set the team indicator
    *  that this view is available to.
    *  
    *
    *  @param teamInd The team indicator to set. The teamInd is defined in TIGRIS.USERGROUP (USERGROUPIND)
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void setTeamInd ( int teamInd ) {
      this.teamInd = teamInd ;
   }




   /*------------------------------------------------------------*/
   /* getViewInd
    */
   /**
    *    This method will return the view indicator
    *  of this view. This view indicator is the CUSTOMVIEWIND
    *  column in the QIT.CUSTOMVIEW table
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public final int getViewInd () {
      return( viewInd ) ;
   }




   /*------------------------------------------------------------*/
   /* assignViewInd
    */
   /**
    *    This method will assign this view a new
    *  view indicator. This method does nothing if the current
    *  viewInd is non-zero.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public final void assignViewInd () {
      if ( viewInd == 0 ) {
         viewInd = Counter.getCounter ( LogSystem.getInstance(), "IT_VIEWS" ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return our column data
    *  for the column passed
    *  
    *
    *  @param column The column data to return
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      return( descript ) ;
   }





   /*------------------------------------------------------------*/
   /* addFieldRestriction
    */
   /**
    *    This method must be subclassed to add
    *  a restriction to a field
    *  
    *
    *  @param viewField The CustomViewField to add a restriction to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void addFieldRestriction ( CustomViewField viewField ) ;





   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will simply return the description
    *  of this View
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( descript ) ;
   }

} /* CustomView */
