package com.ibm.nzna.projects.qit.customview ;     
/*-----------------------------------------------------------*/
/*   CustomViewDlg                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomViewDlg                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog will show the user all of their custom views and will 
 * allow the user to Create, Edit and Delete Custom Views. If the user
 * presses the create button, this window will just close and the 
 * return CREATE from it's getResult Method.
 *
 * @version 1.0, 4/29/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomViewDlg                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomViewDlg extends JDialog implements CustomViewListener, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Return Values                                           */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int ADD         = 1 ;
   public static final int CANCEL      = 2 ;

   public static final String[] titles = { Str.getStr ( STR_CUSTOM_VIEWS )} ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_IMAGE       = null ;
   private MultiList   cnr_VIEWS      = null ;
   private DButton     pb_ADD         = null ;
   private DButton     pb_EDIT        = null ;
   private DButton     pb_DELETE      = null ;
   private DButton     pb_CANCEL      = null ;      
   private ButtonPanel buttonPanel    = null ;
   private DButton     pb_TEST        = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ImageIcon             wizardImage           = null ;
   private int                   rc                    = CANCEL ;
   private CustomViewDlgListener customViewDlgListener = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create but not show our window.
    *  To show the window, you must call 'getResult'
    *
    *  @param parent The parent window for this dialog
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomViewDlg ( Frame parent, CustomViewDlgListener customViewDlgListener ) {
      super ( parent, Str.getStr ( STR_QUERY ), true ) ;
      Container contentPane = null ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign local vars                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.customViewDlgListener = customViewDlgListener ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardImage = ImageSystem.getImageIcon ( this, ImageSystem.CUSTOM_VIEW_WIZARD ) ;
      st_IMAGE    = new JLabel ( wizardImage ) ;
      cnr_VIEWS   = new MultiList ( GUISystem.getFontUtil () ) ;
      pb_ADD      = new DButton ( Str.getStr ( STR_ADD ) ) ;
      pb_EDIT     = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_DELETE   = new DButton ( Str.getStr ( STR_DELETE ) ) ;
      pb_CANCEL   = new DButton ( Str.getStr ( STR_CLOSE ) ) ;        
      pb_TEST     = new DButton ( Str.getStr ( STR_TEST ) ) ;
      buttonPanel = new ButtonPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_VIEWS.setColumnHeadings ( titles ) ;
      cnr_VIEWS.setColumnWidth ( 0, 200 ) ;
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane = getContentPane() ;
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( cnr_VIEWS ) ;
      contentPane.add ( buttonPanel ) ;
      buttonPanel.add ( pb_ADD ) ;
      buttonPanel.add ( pb_EDIT ) ;
      buttonPanel.add ( pb_DELETE ) ;
      buttonPanel.add ( pb_TEST ) ;
      buttonPanel.add ( pb_CANCEL ) ;                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      CustomViewSystem.addCustomViewListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_EDIT.addActionListener   ( this ) ;
      pb_ADD.addActionListener    ( this ) ;
      pb_DELETE.addActionListener ( this ) ;
      pb_TEST.addActionListener   ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshData() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Center our window                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      setSize ( 525, 325 ) ;
      WinUtil.centerWindow ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the window and will
    *  not return until the window is closed.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return rc The return code. ADD or CANCEL as defined in this class
    *//*---------------------------------------------------------*/
   public int getResult () {
      setVisible ( true ) ;
      return( rc ) ;
   }



   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = wizardImage.getImage().getWidth(this)+10 ;
      int       y         = ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2) ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           y, 
                           wizardImage.getImage().getWidth  ( this ), 
                           wizardImage.getImage().getHeight ( this ) ) ;



      cnr_VIEWS.setBounds ( x, y, size.width-(x)-10, size.height-95 ) ;


      buttonPanel.setBounds ( x, size.height-78, size.width-x, 25 ) ;

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when a button
    *  is pressed
    *  
    *
    *  @param e The ActionEvent to process
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         CustomViewSystem.removeCustomViewListener ( this ) ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD ) {
         if ( customViewDlgListener != null )
            new CustomViewWizard ( this, 
                                   customViewDlgListener.getNewCustomView() ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT ) {
         CustomView customView = (CustomView)cnr_VIEWS.getSelectedItem() ;

         if ( customView != null ) {
            customView.updateRecStatus ( customView.REC_UPDATE ) ;
            new CustomViewWizard ( this,
                                   customView ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_DELETE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DELETE ) {
         CustomView customView = (CustomView)cnr_VIEWS.getSelectedItem() ;

         if ( customView != null ) {
            if ( GUISystem.printBox ( Str.getStr ( STR_ARE_YOU_SURE_DELETE ), Str.getStr ( STR_ARE_YOU_SURE_DELETE_CUSTOM_VIEW ) ) )
               CustomViewSystem.removeView ( customView ) ;
         }
      }
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_TEST                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_TEST ) {
         CustomView customView = (CustomView)cnr_VIEWS.getSelectedItem() ;

         if ( customView != null )
            new CustomViewTestDlg ( customView ) ;
      }


   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh the data on our
    *  list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshData () {
      cnr_VIEWS.removeAll() ;
      cnr_VIEWS.add ( CustomViewSystem.getViews() ) ;
   }





   /*------------------------------------------------------------*/
   /* customViewAdded
    */
   /**
    *    this method will be called when a custom view
    *  has been added
    *  
    *
    *  @param customView The CustomView that was added
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewAdded ( CustomView customView ) {
      cnr_VIEWS.add ( customView ) ;
   }



   /*------------------------------------------------------------*/
   /* customViewRemoved
    */
   /**
    *    this method will be called when a custom view
    *  has been added
    *  
    *
    *  @param customView The CustomView that was removed
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewRemoved ( CustomView customView ) {
      cnr_VIEWS.remove ( customView ) ;
   }




   /*------------------------------------------------------------*/
   /* customViewUpdated
    */
   /**
    *    this method will be called when a custom view
    *  has been added
    *  
    *
    *  @param customView The CustomView that was updated
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewUpdated ( CustomView customView ) {
      cnr_VIEWS.repaint() ;
   }

} /* CustomViewDlg */
