package com.ibm.nzna.projects.qit.customview ;         
/*-----------------------------------------------------------*/
/*   CustomViewField                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomViewField                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.*;
/**
 * This object represents a single field in a custom query 
 *
 *
 * @version 1.0, 4/29/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomViewField                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomViewField extends Object implements java.io.Serializable, MultiListRow, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Operators                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int OPERATOR_UNKNOWN       = 0 ;
   public static final int OPERATOR_EQUALS        = 1 ;
   public static final int OPERATOR_NOT_EQUAL     = 2 ;
   public static final int OPERATOR_GREATER_THAN  = 3 ; 
   public static final int OPERATOR_LESS_THAN     = 4 ; 
   public static final int OPERATOR_LIKE          = 5 ; 
   public static final int OPERATOR_IN            = 6 ; 

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Data Types                                              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int DATATYPE_BOOLEAN       = 1 ;
   private static final int DATATYPE_STRING        = 2 ;
   private static final int DATATYPE_INTEGER       = 3 ;
   private static final int DATATYPE_TIME          = 4 ;
   private static final int DATATYPE_TIMESTAMP     = 5 ;      



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String columnName         = null ;
   private String columnDescript     = null ;
   private Object value              = "" ;
   private String operatorString     = null ;
   private int    operatorInd        = 0 ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default Constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomViewField () {
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will construct our object and
    *  will initialize the values to the params given
    *  
    *
    *  @param columnName The name of the column that this field represents. This should be the column name in the database
    *  @param columnDescript The user readable description of this column
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomViewField ( String columnName, String columnDescript ) {
      setColumnName     ( columnName ) ;
      setColumnDescript ( columnDescript ) ;
   }


   /*------------------------------------------------------------*/
   /* setValue
    */
   /**
    *    This method will set the value of this field. The
    *  value will be compared to in the database
    *  
    *
    *  @param value The value to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setValue ( Object value ) {
      this.value = value ;
   }




   /*------------------------------------------------------------*/
   /* setOperator
    */
   /**
    *    This method will set the operator that will
    *  be used to in the compare
    *  
    *
    *  @param operatorInd The operator that is defined in this class ( OPERATOR_EQUALS etc.. )
    *  @param None
    *  @param None
    *
    *  @return None
    *//*---------------------------------------------------------*/
   public void setOperator ( int operatorInd ) {
      this.operatorInd = operatorInd ;

      switch ( operatorInd ) {
         case OPERATOR_EQUALS :
            operatorString = Str.getStr ( STR_EQUALS ) ;
            break ;

         case OPERATOR_NOT_EQUAL :
            operatorString = Str.getStr ( STR_NOT_EQUAL ) ;
            break ;

         case OPERATOR_GREATER_THAN :
            operatorString = ">" ;
            break ;

         case OPERATOR_LESS_THAN :
            operatorString = "<" ;
            break ;

         case OPERATOR_LIKE :
            operatorString = Str.getStr ( STR_LIKE ) ;
            break ;

         case OPERATOR_IN :
            operatorString = Str.getStr ( STR_IN ) ;
            break ;

         default:
            operatorString = "" ;
            break ;
      }
   }


   /*------------------------------------------------------------*/
   /* setOperator
    */
   /**
    *    This method will set the operator that will
    *  be used to in the compare
    *  
    *
    *  @param operatorStr The string operator
    *  @param None
    *  @param None
    *
    *  @return None
    *//*---------------------------------------------------------*/
   public void setOperator ( String operatorStr ) {

      this.operatorString = operatorStr ;

      if ( operatorStr.equals ( Str.getStr ( STR_EQUALS ) ) )
         operatorInd = OPERATOR_EQUALS ;

      else if ( operatorStr.equals ( Str.getStr ( STR_NOT_EQUAL ) ) )
         operatorInd = OPERATOR_NOT_EQUAL ;

      else if ( operatorStr.equals ( ">" ) )
         operatorInd = OPERATOR_GREATER_THAN ;

      else if ( operatorStr.equals ( "<" ) )
         operatorInd = OPERATOR_LESS_THAN ;

      else if ( operatorStr.equals ( Str.getStr ( STR_LIKE ) ) )
         operatorInd = OPERATOR_LIKE ;

      else if ( operatorStr.equals ( Str.getStr ( STR_IN ) ) )
         operatorInd = OPERATOR_IN ;
      else
         operatorInd = OPERATOR_UNKNOWN ;
   }



   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the column data
    *  that should appear in the column passed
    *  
    *
    *  @param column The column to return data for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            retObj = columnDescript ;
            break ;

         case 1 :
            retObj = operatorString ;
            break ;

         case 2 :
            retObj = value ;
            break ;
      }
      return( retObj ) ;
   }





   /*------------------------------------------------------------*/
   /* setColumnName
    */
   /**
    *    This method will set the column name that
    *  is used for this column. This should be the name
    *  of the column in the database
    *
    *  @param columnName The name of this column
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnName ( String columnName ) {
      this.columnName = columnName ;
   }




   /*------------------------------------------------------------*/
   /* setColumnDescript
    */
   /**
    *    This method will set the columnDescript. This value
    *  should be the user readable description of this
    *  column
    *
    *  @param columnDescript The user Readable description of this column
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnDescript ( String columnDescript ) {
      this.columnDescript = columnDescript ;
   }





   /*------------------------------------------------------------*/
   /* getOperatorInd
    */
   /**
    *    This method will return the operator indicator
    *  as defined in this class.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getOperatorInd () {
      return( operatorInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getDatabaseOperator
    */
   /**
    *    This method will return the database operator
    *  that is represented by the operator defined in 
    *  this class
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getDatabaseOperator () {
      String retStr = null ;

      switch ( operatorInd ) {
         case OPERATOR_GREATER_THAN :
            retStr = ">" ;
            break ;

         case OPERATOR_LESS_THAN :
            retStr = "<" ;
            break ;

         case OPERATOR_EQUALS :
            retStr = "=" ;
            break ;

         case OPERATOR_NOT_EQUAL :
            retStr = "!=" ;
            break ;

         case OPERATOR_LIKE :
            retStr = "LIKE" ;
            break ;

         case OPERATOR_IN :
            retStr = "IN" ;
            break ;
      }

      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnName
    */
   /**
    *    This method will return the column name. This
    *  column name should be the name of the column
    *  in the database
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getColumnName () {
      return( columnName ) ;
   }




   /*------------------------------------------------------------*/
   /* getValue
    */
   /**
    *    This method will return the value. This value
    *  should be used in the compare
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getValue () {
      return( value ) ;
   }




   /*------------------------------------------------------------*/
   /* removeRestrictions
    */
   /**
    *    This method will remove all of the restrictions
    *  to this field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeRestrictions () {
      operatorInd    = 0 ;
      operatorString = "?" ;
      value          = "" ;

   }





   /*------------------------------------------------------------*/
   /* getOperators
    */
   /**
    *    This method will return a list of strings. Each
    *  string represents an operator
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getOperators () {
      Vector retVec = new Vector ( 1, 3 ) ;

      retVec.addElement ( Str.getStr ( STR_LIKE ) )  ;
      retVec.addElement ( Str.getStr ( STR_EQUALS ) )  ;
      retVec.addElement ( Str.getStr ( STR_NOT_EQUAL ) )  ;
      retVec.addElement ( ">" ) ;
      retVec.addElement ( "<" ) ;

      return( retVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnDescript
    */
   /**
    *    This method will return the description
    *  of the column
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getColumnDescript () {
      return( columnDescript ) ;
   }

   public String toString () {
      return ( columnDescript ) ;
   }
} /* CustomViewField */
