package com.ibm.nzna.projects.qit.customview ;
/*-----------------------------------------------------------*/
/*   CustomViewSystem                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomViewSystem                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
/**
 * This system handles all of the custom views for Quest
 *
 *
 * @version 1.0, 4/29/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomViewSystem                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomViewSystem extends Object implements Runnable, AppConst {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Thread Names                                            */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static final String THREAD_FIRE_ADDED      = "FA" ;
    private static final String THREAD_FIRE_UPDATED    = "FU" ;
    private static final String THREAD_FIRE_REMOVED    = "FR" ;
    private static final String THREAD_SAVE            = "SA" ;


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static Vector views     = null ;
    private static Vector listeners = null ;
    private CustomView actionObject = null ;




    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create and initialize the system
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public CustomViewSystem () {
        views = new Vector ( 1, 1 ) ;
    }






    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This private constructor will only kick off
     *  the thread to notify listeners of an event
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    CustomViewSystem ( String threadName, CustomView actionObject ) {
        this.actionObject = actionObject ;
        new Thread ( this, threadName ).start() ;
    }



    /*------------------------------------------------------------*/
    /* readViews
     */
    /**
     *    This method will read in all of the user's
     *  views from the database
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void readViews () {
        SQLMethod  sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "CustomViewSystem.readViews", 5 ) ;

        try {
            Statement    stmt    = sqlMethod.createStatement() ;
            ResultSet    rs      = null ;
            CustomView   readRec = null ;

            rs = stmt.executeQuery ( "SELECT CUSTOMVIEWIND, TYPECUSTOMVIEWIND, DESCRIPT, VIEWOBJECT, "+
                                     "       USERGROUPIND,  DBUSER,            CHANGEDTIME "+
                                     "FROM QIT.CUSTOMVIEW WHERE (DBUSER = '"+UserSystem.getUserId()+"' OR "+
                                     "                           USERGROUPIND in (SELECT USERGROUPIND FROM TIGRIS.USERIDUSERGROUP WHERE USERID = '"+UserSystem.getUserId()+"'))" ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through and read each one                          */                     
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( rs.next() ) {
                try {
                    readRec = (CustomView)StoredProcUtil.getObjectFromResultSet ( rs, 4 ) ;

                    if ( views == null )
                        views = new Vector ( 1 ) ;

                    views.addElement ( readRec ) ;

                } catch ( InvalidClassException ex ) {
                    Statement delStmt = sqlMethod.createStatement() ;

                    LogSystem.log ( 1, "Could not Read Custom View. VIEWIND:"+rs.getInt(1)+"  Description:"+rs.getString(3)) ;

                    /* We must now delete this entry from the database */
                    delStmt.executeUpdate ( "DELETE FROM QIT.CUSTOMVIEW WHERE CUSTOMVIEWIND = "+rs.getInt(1) ) ;

                }
            }


            rs.close() ;
            stmt.close() ;

        } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
        }

        sqlMethod.close() ;
    } /* readViews  */






    /*------------------------------------------------------------*/
    /* writeViews
     */
    /**
     *    This method will write all of the views to the
     *  database
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return boolean TRUE if saved, FALSE If not
     *//*---------------------------------------------------------*/
    public static boolean writeViews ( ) {
        boolean   rc        = false ;
        SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "CustomViewSystem.writeViews", 5 ) ;
        String    strStmt   = null ;

        try {
            PreparedStatement stmt = null ;
            int               i    = 0 ;

            if ( ( views != null ) && ( views.size() > 0 ) ) {
                int        size    = views.size() ;
                CustomView view    = null ;
                boolean    cont    = true ;


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Walk the list of views                                  */
                /*                                                           */
                /*-----------------------------------------------------------*/
                while ( i < size ) {

                    view = (CustomView) views.elementAt(i) ;
                    
                    cont = true ;

                    switch ( view.getRecStatus() ) {
                        case CustomView.REC_INSERT :
                            if ( view.getViewInd() == 0 )
                                view.assignViewInd() ;
                            strStmt = "INSERT INTO QIT.CUSTOMVIEW "+
                                      "( CUSTOMVIEWIND, TYPECUSTOMVIEWIND, DESCRIPT, VIEWOBJECT, "+
                                      "  USERGROUPIND,  DBUSER,            CHANGEDTIME ) "+
                                      "values "+
                                      "( "+view.getViewInd()+", "+
                                      "  "+view.getViewType()+", "+
                                      "  '"+Text.cleanDBString ( view.getDescript() )+"', "+
                                      "  ?, "+
                                      "  "+((view.getTeamInd() == -1 ) ? "null" : ""+view.getTeamInd())+", "+
                                      " '"+UserSystem.getUserId()+"', "+
                                      " CURRENT TIMESTAMP )" ;
                            view.updateRecStatus ( view.REC_NOCHANGE ) ;
                            break ;

                        case CustomView.REC_UPDATE :
                            strStmt = "UPDATE QIT.CUSTOMVIEW "+
                                      "SET CUSTOMVIEWIND     = "+view.getViewInd()+", "+
                                      "    TYPECUSTOMVIEWIND = "+view.getViewType()+", "+
                                      "    DESCRIPT          = '"+Text.cleanDBString ( view.getDescript() )+"', "+
                                      "    VIEWOBJECT        = ?, "+
                                      "    USERGROUPIND      = "+((view.getTeamInd() == -1 ) ? "null" : ""+view.getTeamInd())+", "+
                                      "    DBUSER            = '"+UserSystem.getUserId()+"', "+
                                      "    CHANGEDTIME       = CURRENT TIMESTAMP "+
                                      "WHERE CUSTOMVIEWIND = "+view.getViewInd() ;
                            view.updateRecStatus ( view.REC_NOCHANGE ) ;
                            break ;

                        case CustomView.REC_DELETE :
                            Statement stmtDelete = sqlMethod.createStatement() ;

                            stmtDelete.executeUpdate ( "DELETE FROM QIT.CUSTOMVIEW WHERE CUSTOMVIEWIND = "+view.getViewInd() ) ;
                            break ;

                        default:
                            strStmt = null ;
                            cont = false ;
                            break ;
                    } /* switch  */

                    
                    if ( ( strStmt != null ) && ( cont ) ) {
                        FileUtil.writeObjectToDisk ( "tempcustomview.ser", view ) ;

                        File            file        = new File ( "tempcustomview.ser" ) ;
                        FileInputStream inputStream = new FileInputStream ( file ) ;
                        byte[]          inBytes     = FileUtil.readBytesFromFile ( "tempcustomview.ser" ) ;

                        stmt = sqlMethod.createPreparedStatement ( strStmt ) ;
                        stmt.setBinaryStream ( 1, new ByteArrayInputStream ( inBytes ), inBytes.length ) ;

                        LogSystem.log ( 1, strStmt ) ;
                        inputStream.close() ;
                        FileUtil.deleteFile ( "tempcustomview.ser" ) ;
                        stmt.execute() ;
                        stmt.close() ;
                        stmt = null ;
                    }

                    i++ ;
                } /* while  */


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Now, remove all REC_DELETE records from the system      */                                         
                /*                                                           */
                /*-----------------------------------------------------------*/
                i=0 ;
                while ( i < size ) {
                    view = (CustomView) views.elementAt(i) ;

                    if ( !view.valid() ) {
                        views.removeElementAt(i) ;
                        i=0 ;
                        size-- ;
                    } else
                        view.updateRecStatus  ( view.REC_NOCHANGE ) ;
                    i++ ;
                }

                rc = true ;

            } /* if  */

        } catch ( Exception e ) {
            sqlMethod.rollBack() ;
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( STR_ERROR, STR_CANNOT_SAVE_CUSTOM_VIEWS ) ;
        }


        sqlMethod.close() ;
        return( rc ) ;
    }




    /*------------------------------------------------------------*/
    /* getViews
     */
    /**
     *    This method will return all of the views
     *  that fall under the type passed in
     *  
     *
     *  @param viewTypeInd  The type of view to get as defined in QIT.TYPECUSTOMVIEW
     *  @param None
     *  @param None
     *
     *  @return Vector
     *//*---------------------------------------------------------*/
    public static Vector getViews ( int viewTypeInd ) {
        Vector retVec = new Vector ( 1, 1 ) ;

        if ( views != null ) {
            int i     = 0 ;
            int size  = views.size() ;

            while ( i < size ) {
                if ( ( ((CustomView)views.elementAt(i)).getViewType() == viewTypeInd ) &&
                     ( ((CustomView)views.elementAt(i)).valid() ) )
                    retVec.addElement ( views.elementAt(i) ) ;
                i++ ;
            }
        }

        return( retVec ) ;
    }





    /*------------------------------------------------------------*/
    /* addView
     */
    /**
     *    This method will add a view to the list of
     *  views. This view will be saved to the database
     *
     *  @param customView The CustomView to add
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void addView ( CustomView customView ) {
        views.addElement ( customView ) ;
        new CustomViewSystem ( THREAD_FIRE_ADDED, customView ) ;
    }





    /*------------------------------------------------------------*/
    /* getView
     */
    /**
     *    This method will return the CustomView
     *  object that corresponds to the name specified
     *  
     *
     *  @param descript The description of the view
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static CustomView getView ( String descript ) {
        int        i          = 0 ;
        int        size       = views.size() ;
        CustomView customView = null ;

        while ( ( customView == null ) && ( i < size ) ) {
            if ( ((CustomView)views.elementAt(i)).getDescript().equals ( descript ) )
                customView = (CustomView) views.elementAt(i) ;
            i++ ;
        } /* while  */

        return( customView ) ;
    }



    /*------------------------------------------------------------*/
    /* removeView
     */
    /**
     *    This method will remove a view from our
     *  list
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void removeView ( CustomView customView ) {
        customView.updateRecStatus ( customView.REC_DELETE ) ;
        new CustomViewSystem ( THREAD_FIRE_REMOVED, customView ) ;
    }




    /*------------------------------------------------------------*/
    /* getViews
     */
    /**
     *    This method will return all of the views
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Vector containing subclasses of CustomView
     *//*---------------------------------------------------------*/
    public static Vector getViews () {
        return( views ) ;
    }




    /*------------------------------------------------------------*/
    /* getViewFromDescript
     */
    /**
     *    This method will return a CustomView
     *  object from the descript passed in
     *  
     *
     *  @param descript The description to look for
     *  @param None
     *  @param None
     *
     *  @return NULL if not found, CustomView if found
     *//*---------------------------------------------------------*/
    public static CustomView getViewFromDescript ( String descript ) {
        int        i       = 0 ;
        int        size    = views.size() ;
        CustomView retView = null ;

        while ( ( retView == null ) && ( i < size ) ) {
            if ( ((CustomView)views.elementAt(i)).getDescript ().equals( descript ) )
                retView = (CustomView)views.elementAt(i) ;
            i++ ;
        }
        return( retView ) ;
    }





    /*------------------------------------------------------------*/
    /* addCustomViewListener
     */
    /**
     *    This method will add a CustomViewListener
     *  to the CustomView System. Please be very careful
     *  to always call 'removeCustomViewListener' for yourself when you are done,
     *  this way no references to your object will be held for the life of the
     *  application
     *
     *  @param listener The listener to add
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void addCustomViewListener ( CustomViewListener listener ) {
        if ( listeners == null )
            listeners = new Vector ( 1, 10 ) ;

        listeners.addElement ( listener ) ;
    }





    /*------------------------------------------------------------*/
    /* removeCustomViewListener
     */
    /**
     *    This method will remove a listener from the
     *  set of registered listeners
     *  
     *
     *  @param listener The listener to remove
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void removeCustomViewListener ( CustomViewListener listener ) {
        if ( listeners != null )
            listeners.removeElement ( listener ) ;
    }





    /*------------------------------------------------------------*/
    /* run
     */
    /**
     *    This method will fire events to our
     *  listeners
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void run () {
        String operation = Thread.currentThread().getName () ;

        if ( operation.equals ( THREAD_FIRE_ADDED ) )
            fireCustomViewAdded ( actionObject ) ;

        else if ( operation.equals ( THREAD_FIRE_REMOVED ) )
            fireCustomViewRemoved ( actionObject ) ;

        else if ( operation.equals ( THREAD_FIRE_UPDATED ) )
            fireCustomViewUpdated ( actionObject ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Save to the database for added, remove, update          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( ( operation.equals ( THREAD_FIRE_ADDED ) ) ||
             ( operation.equals ( THREAD_FIRE_REMOVED ) ) ||
             ( operation.equals ( THREAD_FIRE_UPDATED ) ) ||
             ( operation.equals ( THREAD_SAVE ) ) ) {
            Qit.setEnabled ( false ) ;
            try {
                MainWindow.getInstance().setStatus ( STR_SAVING_CUSTOM_VIEWS ) ;
                writeViews() ;
                MainWindow.getInstance().setStatus ( null ) ;
            } catch ( Exception e ) {
                LogSystem.log ( 1, e , false ) ;
            }
            Qit.setEnabled ( true ) ;
        }
    }





    /*------------------------------------------------------------*/
    /* fireCustomViewAdded
     */
    /**
     *    This method will fire the customViewAdded
     *  event to all listeners
     *  
     *
     *  @param actionObject The object to pass to the listeners
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void fireCustomViewAdded ( CustomView actionObject ) {
        if ( listeners != null ) {
            int i       = 0 ;
            int size    = listeners.size() ;

            while ( i < size ) {
                ((CustomViewListener)listeners.elementAt(i)).customViewAdded ( actionObject ) ;
                i++ ;
            } /* while  */
        } /* if  */
    } /* fireCustomViewAdded  */




    /*------------------------------------------------------------*/
    /* fireCustomViewRemoved
     */
    /**
     *    This method will fire the customViewRemoved
     *  event to all listeners
     *  
     *
     *  @param actionObject The object to pass to the listeners
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void fireCustomViewRemoved ( CustomView actionObject ) {
        if ( listeners != null ) {
            int i       = 0 ;
            int size    = listeners.size() ;

            while ( i < size ) {
                ((CustomViewListener)listeners.elementAt(i)).customViewRemoved ( actionObject ) ;
                i++ ;
            } /* while  */
        } /* if  */
    } /* fireCustomViewRemoved  */




    /*------------------------------------------------------------*/
    /* fireCustomViewUpdated
     */
    /**
     *    This method will fire the customViewUpdated
     *  event to all listeners
     *  
     *
     *  @param actionObject The object to pass to the listeners
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void fireCustomViewUpdated ( CustomView actionObject ) {
        if ( listeners != null ) {
            int i       = 0 ;
            int size    = listeners.size() ;

            while ( i < size ) {
                ((CustomViewListener)listeners.elementAt(i)).customViewUpdated ( actionObject ) ;
                i++ ;
            } /* while  */
        } /* if  */
    } /* fireCustomViewUpdated  */





    /*------------------------------------------------------------*/
    /* updateView
     */
    /**
     *    This method will update the view in the system
     *  
     *  
     *
     *  @param view The view to update
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void updateView ( CustomView customView ) {
        new CustomViewSystem ( THREAD_FIRE_UPDATED, customView ) ;
    }
} /* CustomViewSystem */
