package com.ibm.nzna.projects.qit.customview ;              
/*-----------------------------------------------------------*/
/*   CustomViewTestDlg                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomViewTestDlg                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;

/**
 * This test dialog will run a custom view for the user and show the results. 
 *
 *
 * @version 1.0, 5/3/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomViewTestDlg                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomViewTestDlg extends JDialog implements AppConst, ActionListener, Runnable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel    st_RESULTS     = null ;
   private MultiList cnr_DATA       = null ;
   private DButton   pb_OK          = null ;      

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private CustomView customView    = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the dialog.
    *  It will immediatly return since this dialog is not
    *  modal
    *
    *  @param customView The CustomView to refresh with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomViewTestDlg ( CustomView customView ) {
      super ( new Frame(), Str.getStr ( STR_TEST ), false ) ;
      Container contentPane = getContentPane() ;                  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign local variables                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.customView = customView ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_RESULTS = new JLabel ( Str.getStr ( STR_RESULTS ) ) ;
      cnr_DATA   = new MultiList ( GUISystem.getFontUtil() ) ;
      pb_OK      = new DButton ( Str.getStr ( STR_OK ) ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_RESULTS ) ;
      contentPane.add ( cnr_DATA ) ;
      contentPane.add ( pb_OK ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPropertiesOnPanel ( contentPane ) ;
      GUISystem.setPreferredButton ( pb_OK ) ;    
      setTitles() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;


      setSize ( 400, 300 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;
      new Thread ( this ).start() ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    this method will lay out and position
    *  all of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size       = getSize () ;
      Dimension okPrefSize = pb_OK.getMinimumSize() ;
      int       y          = 5 ;
      int       x          = 5 ;
      int       rowHeight  = GUISystem.getRowHeight() ;

      super.doLayout() ;
      st_RESULTS.setBounds ( x, y, size.width-(x*3), rowHeight ) ;
      y+=rowHeight ;

      cnr_DATA.setBounds ( x, y, size.width-(x*3), size.height-55-y ) ;

      pb_OK.setBounds ( x, size.height-50, okPrefSize.width, okPrefSize.height ) ;

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param e The action event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      dispose() ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will do the searching and will
    *  populate the data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the results text and color                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_RESULTS.setText ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
      st_RESULTS.setForeground ( Color.blue ) ;
      pb_OK.setEnabled ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Search                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_DATA.removeAll() ;
      cnr_DATA.add ( customView.getResults() ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our text back                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.setEnabled ( true ) ;
      st_RESULTS.setText  ( Str.getStr ( STR_RESULTS ) ) ;
      st_RESULTS.setForeground ( Color.black ) ;



   }





   /*------------------------------------------------------------*/
   /* setTitles
    */
   /**
    *    This method will set the titles up for the container
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setTitles () {
      String[] titles    = new String[customView.getColumnCount()]; 
      int[]    colWidths = customView.getColumnWidths() ;

      for ( int i = 0; i < titles.length; i++ ) 
         titles[i] = " " ;

      cnr_DATA.setColumnHeadings ( titles ) ;
      cnr_DATA.showColumnHeadings ( false ) ;

      for ( int i = 0; i < colWidths.length; i++ )
         cnr_DATA.setColumnWidth ( i, colWidths[i] ) ;

   }
} /* CustomViewTestDlg */
