package com.ibm.nzna.projects.qit.customview ;           
/*-----------------------------------------------------------*/
/*   CustomViewWizard                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomViewWizard                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
/**
 * This wizard guides a user through the creation or edit of a CustomView. 
 *
 *
 * @version 1.0, 4/29/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomViewWizard                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomViewWizard extends JDialog implements AppConst, WizardListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Wizard        wizardPanel    = null ;
   private boolean       rc             = false ;
   private WizardStep[]  steps          = null ;
   private CustomView    customView     = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the wizard.
    *  It will immediatly return. 
    *
    *  @param parent The parent window. This parent will not be the parent
    *                of the wizard, but the parent of the status window that
    *                pops up for a second during initialization
    *  @param customView The CustomView to either Edit or Create
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomViewWizard ( JDialog parent, CustomView customView ) {
      super ( parent, Str.getStr ( STR_EDIT ), true ) ;

      if ( customView.getRecStatus() == customView.REC_INSERT )
         setTitle ( Str.getStr ( STR_NEW ) ) ;

      initialize ( customView ) ;
   }

   public CustomViewWizard ( Frame parent, CustomView customView ) {
      super ( parent, Str.getStr ( STR_EDIT ), true ) ;

      if ( customView.getRecStatus() == customView.REC_INSERT )
         setTitle ( Str.getStr ( STR_NEW ) ) ;

      initialize ( customView ) ;
   }


   private void initialize ( CustomView customView ) {
      CreateWizardWin statusWin = new CreateWizardWin ( MainWindow.getInstance(), 9 ) ;
      int             step      = 0 ;

      statusWin.setText ( "Please wait ..." ) ;
      this.customView = customView ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Wizard control                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         wizardPanel = new Wizard ( ImageSystem.getImageIcon ( this, ImageSystem.CUSTOM_VIEW_WIZARD ) ) ;
         statusWin.setValue ( 2 ) ;
      } catch ( Exception e ) {
         wizardPanel = new Wizard () ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our steps                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      steps = new WizardStep[ ( customView.getRecStatus() == customView.REC_INSERT ? 6 : 5 ) ] ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the welcome step                                 */              
      /*                                                           */
      /*-----------------------------------------------------------*/
      steps[step] = new InfoStep ( Str.getStr ( STR_CUSTOM_VIEW_WIZARD_START ) ) ;
      step++ ;
      statusWin.setValue ( 2 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the 'Select Type' step if this is a new view     */                                          
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( customView.getRecStatus() == customView.REC_INSERT ) {
         steps[step] = new SelectViewTypeStep ( customView );
         step++ ;
      }
      statusWin.setValue ( 3 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The Name step                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      steps[step] = new SelectViewNameStep ( customView ) ;
      step++ ;
      statusWin.setValue ( 4 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The Team step                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      steps[step] = new SelectViewTeamStep ( customView ) ;
      step++ ;
      statusWin.setValue ( 5 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The Team step                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      steps[step] = new SelectViewFieldsStep ( customView ) ;
      step++ ;
      statusWin.setValue ( 6 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the finished step                                */               
      /*                                                           */
      /*-----------------------------------------------------------*/
      steps[step] = new InfoStep ( Str.getStr ( STR_CUSTOM_VIEW_WIZARD_FINISH ) ) ;
      statusWin.setValue ( 7 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties/Add Listeners                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardPanel.addWizardListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our window                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( new BorderLayout () ) ;
      getContentPane().add ( wizardPanel ) ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the first step                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardPanel.setCurrentStep ( steps[0] ) ;
      statusWin.setValue ( 8 ) ;


      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      setSize ( 495, 375 ) ;
      WinUtil.centerWindow ( this ) ;
      statusWin.setValue ( 9 ) ;
      setVisible ( true ) ;
      statusWin.dispose() ;
   }



   /*------------------------------------------------------------*/
   /* cancelPerformed
    */
   /**
    *    This method will be called if the user
    *  hit the 'Cancel' button on the Wizard
    *  
    *
    *  @param e The wizard event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cancelPerformed ( WizardEvent e ) {
      rc = false ;
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* finishPerformed
    */
   /**
    *    This method will be called when the
    *  user presses the finish button
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finishPerformed ( WizardEvent e) {
      rc = true ;

      if ( customView.getRecStatus() == CustomView.REC_INSERT )
         CustomViewSystem.addView ( customView ) ;
      else
         CustomViewSystem.updateView ( customView ) ;

      dispose() ;
   }






   /*------------------------------------------------------------*/
   /* nextPerformed
    */
   /**
    *    This method will be called when the
    *  next button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void nextPerformed ( WizardEvent e ) {
      WizardStep step  = wizardPanel.getCurrentStep() ;
      int        index = indexFromStep ( step ) ;

      index++ ;
      if ( index >= steps.length )
         index = steps.length-1 ;

      wizardPanel.setCurrentStep ( steps [ index ] ) ;
      adjustButtons ( ) ;
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
   }


   /*------------------------------------------------------------*/
   /* backPerformed
    */
   /**
    *    This method will be called when the
    *  back button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void backPerformed ( WizardEvent e ) {
      WizardStep step  = wizardPanel.getCurrentStep() ;
      int        index = indexFromStep ( step ) ;

      index-- ;
      if ( index < 0 )
         index = 0 ;

      wizardPanel.setCurrentStep ( steps [ index ] ) ;
      adjustButtons ( ) ;
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
   }





   /*------------------------------------------------------------*/
   /* adjustButtons
    */
   /**
    *    This method will look at what step is
    *  currently showing and will decide which
    *  buttons to show in the wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void adjustButtons () {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == steps[0] )
         wizardPanel.showButtons ( true, false, true, false ) ;
      else if ( step == steps[steps.length-1] )
         wizardPanel.showButtons ( false, true, false, true ) ;
      else
         wizardPanel.showButtons ( true, true, true, false ) ;
   }





   /*------------------------------------------------------------*/
   /* indexFromStep
    */
   /**
    *    This method will return the index of the WizardStep
    *  passed in
    *  
    *
    *  @param wizardStep
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   private int indexFromStep ( WizardStep wizardStep ) {
      int rc = 0 ;
      int i  = 0 ;

      while ( ( rc == 0 ) && ( i < steps.length ) ) {
         if ( steps[i] == wizardStep )
            rc = i ;
         i++ ;
      }

      return( rc ) ;
   }





} /* CustomViewWizard */
