package com.ibm.nzna.projects.qit.customview ;                 
/*-----------------------------------------------------------*/
/*   SelectViewFieldsStep                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectViewFieldsStep                */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This wizard step will allow the user to select a type of custom view 
 *
 *
 * @version 1.0, 4/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectViewFieldsStep                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectViewFieldsStep extends WizardStep implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Container Title                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] cnrTitles = { 
      Str.getStr ( STR_FIELD ),
      Str.getStr ( STR_OPERATOR ),
      Str.getStr ( STR_VALUE ) 
   } ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private MultiList   fieldList     = null ;
   private DButton     pb_ADD         = null ;
   private DButton     pb_REMOVE      = null ;
   private DButton     pb_TEST        = null ;
   private ButtonPanel buttonPanel    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private CustomView customView       = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the entire
    *  step
    *  
    *
    *  @param CustomView The CustomView to alter
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectViewFieldsStep ( CustomView customView ) {
      this.customView = customView ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      fieldList  = new MultiList ( GUISystem.getFontUtil()) ;
      pb_ADD      = new DButton ( Str.getStr ( STR_ADD_RESTRICTION ) ) ;
      pb_REMOVE   = new DButton ( Str.getStr ( STR_REMOVE_RESTRICTION ) ) ;
      pb_TEST     = new DButton ( Str.getStr ( STR_TEST ) ) ;
      buttonPanel = new ButtonPanel () ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD.addActionListener ( this ) ;
      pb_TEST.addActionListener ( this ) ;
      fieldList.addActionListener ( this ) ;
      pb_REMOVE.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      fieldList.setColumnHeadings ( cnrTitles ) ;
      fieldList.setColumnWidth ( 0, 100 ) ;
      fieldList.setColumnWidth ( 1, 50 ) ;
      fieldList.setColumnWidth ( 2, 100 ) ;
      GUISystem.setPreferredButton ( pb_ADD ) ;
      GUISystem.setPreferredButton ( pb_REMOVE ) ;
      GUISystem.setPreferredButton ( pb_TEST ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( fieldList ) ;
      add ( buttonPanel ) ;
      buttonPanel.add ( pb_ADD ) ;
      buttonPanel.add ( pb_REMOVE ) ;
      buttonPanel.add ( pb_TEST ) ;

      GUISystem.setPropertiesOnPanel ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will position and size our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;

      fieldList.setBounds ( 5, GUISystem.getRowHeight(), size.width-10, size.height-GUISystem.getRowHeight()-35 ) ;
      buttonPanel.setBounds ( 5, size.height-30, size.width-10, 25 ) ;
   }

   public String getTitle () {
      return ( Str.getStr ( STR_SELECT_FIELD_WIZARD_HELP ) ) ;
   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to buttons being
    *  pressed
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_REMOVE ) {
         CustomViewField field = (CustomViewField) fieldList.getSelectedItem() ;

         if ( field != null )
            field.removeRestrictions() ;
         fieldList.repaint() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_ADD ) || ( e.getSource() == fieldList ) ) {
         CustomViewField field = (CustomViewField) fieldList.getSelectedItem() ;

         if ( field != null )
            customView.addFieldRestriction ( field ) ;

         fieldList.repaint() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_TEST                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_TEST )  {
         if ( saveInfo() ) {
            new CustomViewTestDlg ( customView ) ;
         }
      }
      

   }





   /*------------------------------------------------------------*/
   /* refreshInfo
    */
   /**
    *    This method will refresh the data on our
    *  screen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshInfo () {
      fieldList.removeAll() ;
      fieldList.add ( customView.getFields () ) ;
      revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will make sure at least 1 of
    *  the fields has a restriction on it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      Vector  fields = customView.getFields() ;
      int     i      = 0 ;
      int     size   = fields.size() ;
      boolean rc     = false;

      while ( ( i < size ) && ( !rc ) ) {
         if ( ((CustomViewField)fields.elementAt(i)).getOperatorInd() != CustomViewField.OPERATOR_UNKNOWN )
            rc = true ;
         i++ ;
      }

      if ( !rc )
         GUISystem.printBox ( WinUtil.getParentFrame ( this ), Str.getStr ( STR_INFO ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;

      return( rc ) ;
   }


} /* SelectViewFieldsStep */
