package com.ibm.nzna.projects.qit.customview ;                 
/*-----------------------------------------------------------*/
/*   SelectViewNameStep                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectViewNameStep                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This wizard step will allow the user to select a type of custom view 
 *
 *
 * @version 1.0, 4/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectViewNameStep                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectViewNameStep extends WizardStep implements ActionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel     st_HELP        = null ;
   private JTextField ef_NAME        = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private CustomView customView       = null ;
   private Wizard     wizard           = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the entire
    *  step
    *  
    *
    *  @param CustomView The CustomView to alter
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectViewNameStep ( CustomView customView ) {
      this.customView = customView ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP     = new JLabel ( Str.getStr ( STR_NAME ) ) ;
      ef_NAME     = new JTextField ( customView.getDescript() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;
      ef_NAME.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP ) ;
      add ( ef_NAME ) ;

      GUISystem.setPropertiesOnPanel ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will position and size our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;
      int       y    = 30 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_HELP.setBounds ( 0, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      ef_NAME.setBounds ( 5, y, size.width-10, GUISystem.getRowHeight() ) ;
   }



   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save our information
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if information was saved, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      String   descript = ef_NAME.getText() ;
      boolean  rc       = false ;

      if ( ( descript != null ) && ( descript.length() > 0 ) ) {
         if ( CustomViewSystem.getViewFromDescript ( descript ) == null ) {
            customView.setDescript ( descript ) ;
            rc = true ;
         } else if ( customView.getRecStatus () == customView.REC_INSERT )
            GUISystem.printBox ( WinUtil.getParentFrame ( this ), Str.getStr ( STR_INFO ), Str.getStr ( STR_DUPLICATE_CUSTOM_VIEW_NAME ) ) ;
         else
            rc = true ;
      }

      return( rc ) ;
   }


   public void refreshInfo () {
      ef_NAME.setText ( customView.getDescript() );
      ef_NAME.requestFocus() ;
   }

   public void actionPerformed ( ActionEvent e ) {
      wizard.processNext() ;
   }

   public void setWizard ( Wizard wizard ) {
      this.wizard = wizard ;
   }

   public String getTitle () {
      return ( Str.getStr ( STR_ENTER_NAME_WIZARD_HELP ) ) ;
   }


} /* SelectViewNameStep */
