package com.ibm.nzna.projects.qit.customview ;                 
/*-----------------------------------------------------------*/
/*   SelectViewTypeStep                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectViewTypeStep                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This wizard step will allow the user to select a type of custom view 
 *
 *
 * @version 1.0, 4/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectViewTypeStep                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectViewTypeStep extends WizardStep implements MouseListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JList       lb_TYPES       = null ;
   private JScrollPane scr_PANE       = null ;      
   private Wizard      wizard         = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private CustomView customView       = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the entire
    *  step
    *  
    *
    *  @param CustomView The CustomView to alter
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectViewTypeStep ( CustomView customView ) {
      this.customView = customView ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      lb_TYPES    = new JList ( TypeList.getInstance().getTypeList ( TypeList.TYPE_CUSTOMVIEW ) ) ;
      scr_PANE    = new JScrollPane ( lb_TYPES ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      scr_PANE.setBorder ( GUISystem.loweredBorder ) ;  
      lb_TYPES.addMouseListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( scr_PANE ) ;

      GUISystem.setPropertiesOnPanel ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will position and size our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;

      scr_PANE.setBounds ( 5, GUISystem.getRowHeight(), size.width-10, size.height-GUISystem.getRowHeight()-5 ) ;
   }



   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save our information
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if information was saved, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      TypeCustomViewRec customViewRec = (TypeCustomViewRec)lb_TYPES.getSelectedValue() ;
      boolean           rc            = false ;

      if ( customViewRec != null ) {
         customView.setViewType ( customViewRec.getInd() ) ;
         rc = true ;
      }

      return( rc ) ;
   }

   public String getTitle () {
      return( Str.getStr ( STR_SELECT_TYPE_WIZARD_HELP ) ) ;
   }

   public void mouseExited ( MouseEvent e ) {
   }
   public void mouseEntered ( MouseEvent e ) {
   }
   public void mouseReleased ( MouseEvent e ) {
   }
   public void mousePressed ( MouseEvent e ) {
   }
   public void mouseClicked ( MouseEvent e ) {
      if ( e.getClickCount() > 1 )
         wizard.processNext() ;
   }



   /*------------------------------------------------------------*/
   /* setWizard
    */
   /**
    *    This method will be called once to set
    *  the wizard that this WizardStep is going
    *  to display in
    *
    *  @param wizard The Wizard that will be used to display this step
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setWizard ( Wizard wizard ) {
      this.wizard = wizard ;
   }


} /* SelectViewTypeStep */
