package com.ibm.nzna.projects.qit.dbgui ;            
/*-----------------------------------------------------------*/
/*   CountrySelectPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CountrySelectPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;     
import com.ibm.nzna.shared.sort.* ;     
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This panel allows the user to add/remove countries from a list 
 *
 *
 * @version 1.0, 5/21/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CountrySelectPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class CountrySelectPanel extends JPanel implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JList        lb_COUNTRIES        = null ;
   private JScrollPane  scr_COUNTRIES       = null ;
   private JTitle       st_TITLE            = null ;
   private HotLinkLabel pb_ADD              = null ;
   private HotLinkLabel pb_REMOVE           = null ;
   private HotLinkLabel pb_ADD_MY_COUNTRY   = null ;
   private HotLinkLabel pb_ADD_FAVORITES    = null ;
   private ButtonPanel  buttonPanel         = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Variables                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DefaultListModel   countries              = null ;
   private Vector             listeners              = null ;
   private CountrySelectPanel joinCountrySelectPanel = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a blank control
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CountrySelectPanel () {
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE            = new JTitle ( Str.getStr ( STR_COUNTRIES ) ) ;
      buttonPanel         = new ButtonPanel () ;
      pb_ADD              = new HotLinkLabel ( Str.getStr ( STR_ADD ) ) ;
      pb_REMOVE           = new HotLinkLabel ( Str.getStr ( STR_REMOVE ) ) ;
      pb_ADD_MY_COUNTRY   = new HotLinkLabel ( Str.getStr ( STR_ADD_MY_COUNTRIES ) ) ;
      pb_ADD_FAVORITES    = new HotLinkLabel ( Str.getStr ( STR_ADD_FAVORITES ) ) ;
      lb_COUNTRIES        = new JList () ;
      scr_COUNTRIES       = new JScrollPane ( lb_COUNTRIES ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      scr_COUNTRIES.setBorder ( GUISystem.blackBorder ) ;
      buttonPanel.setBorder     ( GUISystem.grayBorder ) ;
      buttonPanel.setBackground ( new Color ( 230, 230, 230 ) ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;             
      st_TITLE.setFont ( FontSystem.smallTitleFont ) ;    
      setBackground ( Color.white ) ;
      lb_COUNTRIES.setModel ( countries = new DefaultListModel() );
      lb_COUNTRIES.setSelectionBackground ( new Color ( 209, 210, 253 ) ) ;
      lb_COUNTRIES.setSelectionForeground ( Color.black ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ADD.addActionListener ( this ) ;
      pb_REMOVE.addActionListener ( this ) ;  
      pb_ADD_MY_COUNTRY.addActionListener ( this ) ;
      pb_ADD_FAVORITES.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      //add ( st_TITLE ) ;
      add ( scr_COUNTRIES ) ;
      add ( buttonPanel ) ;
      buttonPanel.add ( pb_ADD ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) )  ;
      buttonPanel.add ( pb_REMOVE ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) )  ;
      buttonPanel.add ( pb_ADD_MY_COUNTRY ) ;
      buttonPanel.add ( new ToolBarSeparator ( new Dimension ( 1, GUISystem.getRowHeight() ) ) )  ;
      buttonPanel.add ( pb_ADD_FAVORITES ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = getInsets().left ;
      int       y         = getInsets().top ;

      st_TITLE.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      scr_COUNTRIES.setBounds ( x, y, size.width-(x*2), size.height-(rowHeight*2)-5 ) ;
      y+=(size.height-(rowHeight*2))-3 ;

      buttonPanel.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to action events
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_ADD ) {
         Vector addedCountryVec = null ;

         if ( joinCountrySelectPanel != null )
            addedCountryVec = new CountrySelectDlg ( GUISystem.getParentDefWin ( this ),
                                                     joinCountrySelectPanel.getCountryList() ).getResult() ;
         else
            addedCountryVec = new CountrySelectDlg ( GUISystem.getParentDefWin ( this ) ).getResult() ;

         if ( addedCountryVec != null )
            addUniqueCountries ( addedCountryVec ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE )
         remove() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_MY_COUNTRIES                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_MY_COUNTRY )
         addUniqueCountries ( UserSystem.getUserCountries ( UserSystem.getUserId() ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_FAVORITES                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_FAVORITES )
         addUniqueCountries ( (Vector)PropertySystem.get ( PropertySystem.PROPERTY_COUNTRY_FAVORITES ) ) ;



   } /* actionPerformed */





   /*------------------------------------------------------------*/
   /* addUniqueCountries
    */
   /**
    *    This method will add the vector of countries
    *  into the list. If a country in the vector passed
    *  is already in the list, nothing is inserted for that
    *
    *  @param vec The vector containing TypeCountryCodeRecs to insert
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private synchronized void addUniqueCountries ( Vector vec ) {
      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
         int i    = 0 ;
         int size = vec.size() ;                             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have countries, compare each one                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {

            if ( ( !countries.contains ( vec.elementAt(i) ) ) &&
                 ( countryInJoinedPanel ( (TypeCountryCodeRec) vec.elementAt(i) ) ) && 
                 ( checkAddListeners ( (TypeCountryCodeRec) vec.elementAt(i) ) ) )
               countries.addElement ( vec.elementAt(i) ) ;
            i++ ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Sort the new list                                       */        
         /*                                                           */
         /*-----------------------------------------------------------*/
         sort() ;
      }

   }





   /*------------------------------------------------------------*/
   /* setCountryList
    */
   /**
    *    This method will set the country list that will
    *  be displayed to the user
    *  
    *
    *  @param countryVec The vector containing TypeCountryCodeRecs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public synchronized void setCountryList ( Vector newCountryVec ) {
      countries.removeAllElements() ;

      addUniqueCountries ( newCountryVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getCountryList
    */
   /**
    *    This method will return the list of countries
    *  that the user has chosen
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getCountryList () {
      Vector   returnVec = new Vector ( 1, 1 ) ;
      Object[] countryArray = countries.toArray() ;

      if ( countryArray != null )
         for ( int i = 0; i < countryArray.length; i++ )
            returnVec.addElement ( countryArray[i] ) ;

      return( returnVec ) ;
   }





   /*------------------------------------------------------------*/
   /* remove
    */
   /**
    *    This method will remove all selected countries
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void remove ( ) {
      try {
         Object[] selection = lb_COUNTRIES.getSelectedValues() ;
         int      selIndex  = lb_COUNTRIES.getSelectedIndex() ;

         if ( selection != null ) {
            for ( int i = 0; i < selection.length; i++ ) {
               if ( ( UserSystem.hasCountry ( (TypeCountryCodeRec)selection[i] ) ) &&
                    ( checkRemoveListeners ( (TypeCountryCodeRec)selection[i] ) ) )
                  countries.removeElement ( selection[i] ) ;
            }

            sort() ;
            lb_COUNTRIES.setSelectedIndex ( selIndex ) ;
         } /* if */

         notifyRemoveDone() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ,false ) ;
      }

   } /* remove */





   /*------------------------------------------------------------*/
   /* checkRemoveListeners
    */
   /**
    *    This method will ask each listener if the
    *  remove is Ok. If any of the listeners return's
    *  FALSE (meaning that it is not ok to remove) then this method returns FALSE
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return TRUE = Allow the removal, FALSE = do not allow the removal
    *//*---------------------------------------------------------*/
   private boolean checkRemoveListeners ( TypeCountryCodeRec countryRec ) {
      boolean allow = true ;

      if ( ( listeners != null ) && ( listeners.size () > 0 ) ) {
         int i    = 0 ;
         int size = listeners.size() ;

         while ( ( i < size ) && ( allow ) ) {
            allow = ((CountrySelectListener)listeners.elementAt(i)).countryRemoved ( countryRec ) ;
            i++ ;
         }
      }

      return( allow ) ;
   }




   /*------------------------------------------------------------*/
   /* notifyRemoveDone
    */
   /**
    *    This method will notif each listener that all countries
    *  that were requested to be deleted have been deleted.
    *
    *
    *//*---------------------------------------------------------*/
   private void notifyRemoveDone ( ) {

      if ( ( listeners != null ) && ( listeners.size () > 0 ) ) {
         int i    = 0 ;
         int size = listeners.size() ;

         while ( i < size ) {
            ((CountrySelectListener)listeners.elementAt(i)).countryRemoveDone ( ) ;
            i++ ;
         }
      }
   }

   /*------------------------------------------------------------*/
   /* checkAddListeners
    */
   /**
    *    This method will ask each listener if the
    *  remove is Ok. If any of the listeners return's
    *  FALSE (meaning that it is not ok to remove) then this method returns FALSE
    *
    *  @param countryRec The TypeCountryCodeRec to remove
    *  @param None
    *  @param None
    *
    *  @return TRUE = Allow the removal, FALSE = do not allow the removal
    *//*---------------------------------------------------------*/
   private boolean checkAddListeners ( TypeCountryCodeRec countryRec ) {
      boolean allow = true ;

      if ( ( listeners != null ) && ( listeners.size () > 0 ) ) {
         int i    = 0 ;
         int size = listeners.size() ;

         while ( ( i < size ) && ( allow ) ) {
            allow = ((CountrySelectListener)listeners.elementAt(i)).countryAdded ( countryRec ) ;
            i++ ;
         }
      }

      return( allow ) ;
   }





   /*------------------------------------------------------------*/
   /* addCountrySelectListener
    */
   /**
    *    This method will add a CountrySelectListener
    *  to be notified of adding and removing of
    *  countries
    *
    *  @param listener The CountrySelectListener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCountrySelectListener ( CountrySelectListener listener ) {
      if ( listeners == null )
         listeners = new Vector ( 1, 1 ) ;

      listeners.addElement ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* removeCountrySelectListener
    */
   /**
    *    This method will remove a CountrySelectListener
    *  from the list of notifications
    *  
    *
    *  @param listener The CountrySelectListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCountrySelectListener ( CountrySelectListener listener ) {
      if ( listeners != null ) {
         listeners.removeElement ( listener ) ;
         if ( listeners.size() == 0 )
            listeners = null ;
      }
   }




   /*------------------------------------------------------------*/
   /* joinCountrySelectPanel
    */
   /**
    *    This method will join this country select panel
    *  with another. This means that this CountrySelectPanel
    *  will only allow countries to be added that are part of the
    *  country select panel being joined. Please keep in mind that
    *  a CountrySelectPanel can only be joined to one other CountrySelectPanel
    *
    *  @param countrySelectPanel The CountrySelectPanel to join with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void joinCountrySelectPanel ( CountrySelectPanel countrySelectPanel ) {
      joinCountrySelectPanel = countrySelectPanel ;
   }




   /*------------------------------------------------------------*/
   /* countryInJoinedPanel
    */
   /**
    *    This method will determine if the country
    *  passed is selected in the joined panel. If there
    *  is no joined panel a TRUE is returned
    *
    *  @param countryRec The TypeCountryCodeRec to check for
    *  @param None
    *  @param None
    *
    *  @return TRUE if the country passed is in the joined panel, FALSE if not
    *//*---------------------------------------------------------*/
   private boolean countryInJoinedPanel ( TypeCountryCodeRec countryRec ) {
      boolean in = true ;

      if ( joinCountrySelectPanel != null )
         in = joinCountrySelectPanel.getCountryList().contains ( countryRec ) ;

      return( in ) ;
   }




   /*------------------------------------------------------------*/
   /* sort
    */
   /**
    *    This method will do a QuickSort on the
    *  countries
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void sort () {
      Vector countryVec = getCountryList () ;
      int    i          = 0 ;
      int    size       = countryVec.size() ;

      new QuickSort ( countryVec ) ;

      countries.removeAllElements() ;

      while ( i < size )
         countries.addElement ( countryVec.elementAt(i++) ) ;

   }







   /*------------------------------------------------------------*/
   /* getJoinCountrySelectPanel
    */
   /**
    *    This method will return the CountrySelectPanel
    *  that this CountrySelectPanel is joined to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return CountrySelectPanel
    *//*---------------------------------------------------------*/
   public CountrySelectPanel getJoinCountrySelectPanel () {
      return( joinCountrySelectPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable this
    *  control
    *  
    *
    *  @param enable TRUE = enable allowing user input, FALSE= disable not allowing user input
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;
      lb_COUNTRIES.setEnabled ( enable ) ;
      scr_COUNTRIES.setEnabled ( enable ) ;
      st_TITLE.setEnabled ( enable ) ;
      pb_ADD.setEnabled ( enable ) ;
      pb_REMOVE.setEnabled ( enable ) ;
      pb_ADD_MY_COUNTRY.setEnabled ( enable ) ;
      pb_ADD_FAVORITES.setEnabled ( enable ) ;
   }
} /* CountrySelectPanel */
