package com.ibm.nzna.projects.qit.dbgui ;     
/*-----------------------------------------------------------*/
/*   DBStatusLight                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DBStatusLight                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
/**
 * This control displays the current status of the database 
 *
 *
 * @version 1.0, 8/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DBStatusLight                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class DBStatusLight extends JComponent implements PropertyListener, AppConst, DatabaseStatusListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private AppDefaultWin parentWin        = null ;
   private int           status           = DATABASE_STATUS_DISCONNECTED ;
   private ImageIcon     currentImage     = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our
    *  control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DBStatusLight () {
      DatabaseSystem.addStatusListener ( this ) ;                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are connected, set that flag                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( DatabaseSystem.isDatabaseConnected ( DatabaseSystem.QUEST ) )
         status = DATABASE_STATUS_CONNECTED ;

      refreshIcon() ;
   }





   /*------------------------------------------------------------*/
   /* databaseStatusChange
    */
   /**
    *    This method will be called when a status
    *  change has occurred on the database
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void databaseStatusChange ( int statusType, Database database ) {                                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   We only care about changes to the main connection for   */
      /* the icon                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( database.getDatabaseId() == DatabaseSystem.QUEST ) &&
           ( !database.getSecondaryConnection() ) ) {
         status = statusType ;
         refreshIcon() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, put up a message if need be                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( statusType ) {
         case DATABASE_STATUS_CONNECTING :
            setStatus ( Str.getStr ( STR_CONNECTING_DATABASE )+" "+database.getName() );
            break ;

         default:
            setStatus ( null ) ;
            break ;
      }

   }






   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will attempt to set the status
    *  on the parent AppDefWin of this control
    *  
    *
    *  @param status The new status
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setStatus ( String status ) {
      if ( parentWin == null )
         parentWin = GUISystem.getParentDefWin ( this ) ;

      if ( parentWin != null )
         parentWin.setStatus ( status ) ;
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has been changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object newValue ) {
      refreshIcon() ;
   }





   /*------------------------------------------------------------*/
   /* refreshIcon
    */
   /**
    *    This method will refresh the connected/disconnected
    *  icon we are displaying
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshIcon () {

      switch ( status ) {
         
         case DATABASE_STATUS_CONNECTED :
            currentImage = ImageSystem.getImageIcon ( this, ImageSystem.CONNECTED ) ;
            setToolTipText ( Str.getStr ( STR_CONNECTED ) ) ;
            break ;

         case DATABASE_STATUS_CONNECTING :
            currentImage = ImageSystem.getImageIcon ( this, ImageSystem.CONNECTING ) ;
            setToolTipText ( Str.getStr ( STR_CONNECTING ) ) ;
            break ;

         default:
            setToolTipText ( Str.getStr ( STR_DISCONNECTED ) ) ;
            currentImage = ImageSystem.getImageIcon ( this, ImageSystem.DISCONNECTED ) ;
            break ;
      }
   }





   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our status image
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      Dimension size = getSize() ;

      size.width-- ;
      size.height-- ;
      //g.setColor ( Color.darkGray ) ;
      //g.drawLine ( 0, 0, size.width, 0 ) ;
      //g.drawLine ( 0, 0, 0, size.height ) ;

      //g.setColor ( Color.white ) ;
      //g.drawLine ( 0, size.height, size.width, size.height ) ;
      //g.drawLine ( size.width, size.height, size.width, 0 ) ;

      g.drawImage ( currentImage.getImage(), 2, 2, this ) ;
   }
} /* DBStatusLight */
