package com.ibm.nzna.projects.qit.dbgui ;
/*-----------------------------------------------------------*/
/*   DocCategoryDlg                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocCategoryDlg                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import java.awt.event.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This Dialog Allows the user to select Document Categories based on a 
 * list of Brands. Since Document Categories are associated to brands, this
 * Window contains a Drop down that allows the user to select the brand and
 * once selected, presents the user with all of the applicable categories. The
 * user can then select 'Add' to add a category to something
 * The DocCategoryDlgListener will be notified when a document Category is 
 * added 
 *
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   DocCategoryDlg                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocCategoryDlg extends JDialog implements Runnable, ListSelectionListener, ActionListener, AppConst, DocConst {

   /**
     * This is the JLabel which holds the 'ONE' Icon
     */
   private JLabel st_ONE       = null ;

   /**
     * This is the JLabel which holds the 'THREE' Icon
     */
   private JLabel st_THREE       = null ;

   /**
     * This is the JLabel which holds the 'TWO' Icon
     */
   private JLabel st_TWO       = null ;

   /**
     * This is the JLabel which is the title for the Brand drop down
     */
   private JLabel st_SELECT_BRAND = null ;

   /**
     * This is the JLabel which is the title for the Category List
     */
   private JLabel st_SELECT_CATEGORY = null ;

   /**
     * This is the Help/Status text
     */
   private JLabel st_HELP = null ;

   /**
     * This is the JComboBox which holds the geographies
     */
   private JComboBox cb_BRAND = null ;

   /**
     * This is the JList which holds the Countries
     */
   private DList lb_CATEGORIES = null ;

   /**
     * This is the JScrollPane which holds the countries
     */
   private JScrollPane scr_CATEGORIES = null ;

   /**
     * This is the 'Add Selected' button
     */
   private DButton pb_ADD_SELECTED = null ;

   /**
     * This is the 'Done' button
     */
   private DButton pb_DONE = null ;

   /**
     * This is the picture on the left of the dialog
     */
   private JLabel st_SIDE_PIC = null ;

   /**
     * This button panel holds all of the buttons on the
     * dialog. 
     */
   private ButtonPanel buttonPanel = null ;

   /**
     *  This is the list of brands that this dialog will
     * allow to be selected
     **/
   private Vector applicableBrands = null ;


   /**
     *  This is the one listener that is notified when the
     * user has made a request for a Country/Geo addition
     **/
   private DocCategoryDlgListener listener = null ;




   /**
     * This is the default constructor. This method will not show the
     * dialog. Please use the 'waitForUser' method to show the window
     *
     * @param parent The parent of this pop up
     */
   public DocCategoryDlg ( Frame parent, Vector applicableBrands ) {
      super ( parent, Str.getStr ( STR_SELECT_CATEGORY ), true ) ;
      this.applicableBrands = applicableBrands ;
      initialize() ;
   }

   /**
     * This is the default constructor (Which takes a Dialog Parent). This method will not show the
     * dialog. Please use the 'waitForUser' method to show the window
     *
     * @param parent The parent of this pop up
     */
   public DocCategoryDlg ( Dialog parent, Vector applicableBrands ) {
      super ( parent, Str.getStr ( STR_SELECT_CATEGORY ), true ) ;
      this.applicableBrands = applicableBrands ;
      initialize() ;
   }

   public void waitForUser () {
      int             prefBrand  = PropertySystem.getInt ( PropertySystem.PROPERTY_DEFAULT_BRAND ) ;
      TypeCategoryRec brandRec   = (prefBrand!=0) ? (TypeCategoryRec)TypeCategory.getCategory ( prefBrand ) : null ;

      if ( brandRec != null ) {
         cb_BRAND.setSelectedItem ( brandRec ) ;
         new Thread ( this ).start() ;
      }

      setVisible ( true ) ;
   }


   /**
     * This method will initialize our GUI, set properties and add
     * out GUI
     */
   private void initialize () {
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_ONE               = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.ONE ) ) ;
      st_THREE             = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.THREE ) ) ;
      st_TWO               = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.TWO ) ) ;
      st_SELECT_BRAND      = new JLabel ( Str.getStr ( STR_SELECT_A_BRAND ) ) ;
      st_SELECT_CATEGORY   = new JLabel ( Str.getStr ( STR_SELECT_CATEGORY ) ) ;
      st_HELP              = new JLabel ( Str.getStr ( STR_DOC_CATEGORY_DLG_HELP ) ) ;
      cb_BRAND             = new JComboBox ( applicableBrands ) ;
      lb_CATEGORIES        = new DList () ;
      scr_CATEGORIES       = new JScrollPane ( lb_CATEGORIES ) ;
      pb_ADD_SELECTED      = new DButton ( Str.getStr ( STR_ADD_SELECTED ) ) ;
      pb_DONE              = new DButton ( Str.getStr ( STR_DONE ) ) ;
      st_SIDE_PIC          = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.LOCALE_DLG ) ) ;
      buttonPanel          = new ButtonPanel () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SIDE_PIC.setBorder ( GUISystem.loweredBorder ) ;    
      pb_ADD_SELECTED.setEnabled ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_BRAND.addActionListener            ( this ) ;
      pb_ADD_SELECTED.addActionListener     ( this ) ;
      lb_CATEGORIES.addActionListener       ( this ) ;
      pb_DONE.addActionListener             ( this ) ;
      lb_CATEGORIES.addListSelectionListener ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.add ( pb_ADD_SELECTED ) ;
      buttonPanel.add ( pb_DONE ) ;
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_SIDE_PIC ) ;
      contentPane.add ( st_ONE ) ;
      contentPane.add ( st_TWO ) ;
      contentPane.add ( st_THREE ) ;
      contentPane.add ( st_SELECT_BRAND ) ;
      contentPane.add ( st_SELECT_CATEGORY ) ;
      contentPane.add ( st_HELP ) ;
      contentPane.add ( cb_BRAND ) ;
      contentPane.add ( scr_CATEGORIES ) ;
      contentPane.add ( buttonPanel ) ;
      GUISystem.setPropertiesOnPanel ( buttonPanel ) ;


      setSize ( 486, 337 ) ;
      WinUtil.centerChildInParent ( this, getParent() ) ;
      new Thread ( this ).start() ;
   }


   /**
     * This method will lay out and position all of our controls
     */
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = 138 ;
      int       y         = 10 ;

      super.doLayout() ;

      st_SIDE_PIC.setBounds ( 10, 10, 118, 259 ) ;

      st_ONE.setBounds ( x, y, 32, 32 ) ;
      st_SELECT_BRAND.setBounds ( x+35, y, size.width-(x+45), rowHeight ) ;
      y+=rowHeight ;

      cb_BRAND.setBounds ( x+35, y, size.width-(x+45), rowHeight ) ;
      y+=rowHeight ;

      st_TWO.setBounds ( x, y, 32, 32 ) ;
      st_SELECT_CATEGORY.setBounds ( x+35, y, size.width-(x+45), rowHeight ) ;
      y+=rowHeight ;

      scr_CATEGORIES.setBounds ( x+35, y, size.width-(x+45), rowHeight*8 ) ;
      y+=(rowHeight*8);

      st_THREE.setBounds ( x, y, 32, 32 ) ;
      st_HELP.setBounds ( x+35, y, size.width-(x+45), rowHeight*3 ) ;

      buttonPanel.setBounds ( x, size.height-60, size.width-(x+5), 25 ) ;


   }



   /**
     *  This method will be called when the user pushes one of our buttons
     * or makes a selection in the Geography drop down
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_DONE                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_DONE )
         dispose() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   cb_BRAND                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == cb_BRAND ) {
         new Thread ( this ).start() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_ADD_SELECTED                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_ADD_SELECTED ) ||
                ( e.getSource() == lb_CATEGORIES ) ) {
         if ( listener != null )
            listener.docCategoryDlgAdd ( lb_CATEGORIES.getSelectedValues() ) ;
      }

   } /* ActionPerformed  */



   /**
     *  This method will be called when the user has selected an item in the list
     * of countries
     **/
   public void valueChanged ( ListSelectionEvent e ) {
      pb_ADD_SELECTED.setEnabled ( lb_CATEGORIES.getSelectedValue() != null ) ;
   }


   /**
     *  This method will set the one DocCategoryDlgListener that this dialog will
     * notify when Countries/Geographies are added.
     *
     * @param listener The listener
     **/
   public void setDocCategoryDlgListener ( DocCategoryDlgListener listener ) {
      this.listener = listener ;
   }


   /**
     *  This method will refresh our list of categories from the brand that is selected
     **/
   public void run () {
      TypeCategoryRec brandRec = (TypeCategoryRec) cb_BRAND.getSelectedItem() ;

      if ( brandRec != null ) {
         Vector pleaseWaitVec = new Vector () ;

         pleaseWaitVec.addElement ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
         lb_CATEGORIES.setListData ( pleaseWaitVec ) ;
         pb_ADD_SELECTED.setEnabled ( false ) ;
         cb_BRAND.setEnabled ( false ) ;
         lb_CATEGORIES.setListData ( TypeCategory.getChildCategories ( brandRec.getInd(), 
                                                                       TypeCategory.LEVEL_DOC_CATEGORY ) ) ;
         cb_BRAND.setEnabled ( true ) ;
         pb_ADD_SELECTED.setEnabled ( true ) ;

      }
   }
} /* DocCategoryDlg */
