package com.ibm.nzna.projects.qit.dbgui ;
/*-----------------------------------------------------------*/
/*   LocaleDlg                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LocaleDlg                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import java.awt.event.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This Dialog Allows the user to select Geographies and Countries, or
 * all countries in a geography. The only way to retrieve data from this
 * Dialog is through it's listener interface. The LocaleDlgListener
 * will be notified when a country is added or a geography is added.
 *
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   LocaleDlg                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class LocaleDlg extends JDialog implements ListSelectionListener, ActionListener, AppConst, DocConst {

    /**
      * This is the JLabel which holds the 'ONE' Icon
      */
    private JLabel st_ONE       = null ;

    /**
      * This is the JLabel which holds the 'THREE' Icon
      */
    private JLabel st_THREE       = null ;

    /**
      * This is the JLabel which holds the 'TWO' Icon
      */
    private JLabel st_TWO       = null ;

    /**
      * This is the JLabel which is the title for the Geography drop down
      */
    private JLabel st_SELECT_GEOGRAPHY = null ;

    /**
      * This is the JLabel which is the title for the Country List
      */
    private JLabel st_SELECT_COUNTRIES = null ;

    /**
      * This is the Help/Status text
      */
    private JLabel st_HELP = null ;

    /**
      * This is the JComboBox which holds the geographies
      */
    private JComboBox cb_GEOGRAPHY = null ;

    /**
      * This is the JList which holds the Countries
      */
    private DList lb_COUNTRIES = null ;

    /**
      * This is the JScrollPane which holds the countries
      */
    private JScrollPane scr_COUNTRIES = null ;

    /**
      * This is the 'Add Selected' button
      */
    private DButton pb_ADD_SELECTED = null ;

    /**
      * This is the 'Add Entire Geography'
      */
    private DButton pb_ADD_ENTIRE_GEO = null ;

    /**
      * This is the 'Done' button
      */
    private DButton pb_DONE = null ;

    /**
      * This is the picture on the left of the dialog
      */
    private JLabel st_SIDE_PIC = null ;

    /**
      * This button panel holds all of the buttons on the
      * dialog. 
      */
    private ButtonPanel buttonPanel = null ;


    /**
      *  This is the one listener that is notified when the
      * user has made a request for a Country/Geo addition
      **/
    private LocaleDlgListener listener = null ;




    /**
      * This is the default constructor. This method will not show the
      * dialog. Please use the 'waitForUser' method to show the window
      *
      * @param parent The parent of this pop up
      */
    public LocaleDlg ( Frame parent ) {
        super ( parent, Str.getStr ( STR_SELECT_LOCATION ), true ) ;
        initialize() ;
    }

    /**
      * This is the default constructor (Which takes a Dialog Parent). This method will not show the
      * dialog. Please use the 'waitForUser' method to show the window
      *
      * @param parent The parent of this pop up
      */
    public LocaleDlg ( Dialog parent ) {
        super ( parent, Str.getStr ( STR_SELECT_LOCATION ), true ) ;
        initialize() ;
    }

    public void waitForUser () {
        int          lastGeoInd = PropertySystem.getInt ( PropertySystem.PROPERTY_LAST_GEOGRAPHY ) ;
        TypeGeoRec   geoRec     = (lastGeoInd!=0) ? (TypeGeoRec)TypeList.getInstance().objectFromInd ( lastGeoInd, TypeList.TYPE_GEOGRAPHY ) : null ;

        if ( geoRec != null ) {
            cb_GEOGRAPHY.setSelectedItem ( geoRec ) ;
            lb_COUNTRIES.setListData ( geoRec.getCountryVec() ) ;
        }

        setVisible ( true ) ;
    }


    /**
      * This method will initialize our GUI, set properties and add
      * out GUI
      */
    private void initialize () {
        Container contentPane = getContentPane() ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create the GUI                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        st_ONE              = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.ONE ) ) ;
        st_THREE            = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.THREE ) ) ;
        st_TWO              = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.TWO ) ) ;
        st_SELECT_GEOGRAPHY = new JLabel ( Str.getStr ( STR_SELECT_GEOGRAPHY ) ) ;
        st_SELECT_COUNTRIES = new JLabel ( Str.getStr ( STR_SELECT_COUNTRIES ) ) ;
        st_HELP             = new JLabel ( Str.getStr ( STR_LOCALE_DLG_HELP ) ) ;
        cb_GEOGRAPHY        = new JComboBox ( getGeoList() ) ;
        lb_COUNTRIES        = new DList () ;
        scr_COUNTRIES       = new JScrollPane ( lb_COUNTRIES ) ;
        pb_ADD_SELECTED     = new DButton ( Str.getStr ( STR_ADD_SELECTED ) ) ;
        pb_ADD_ENTIRE_GEO   = new DButton ( Str.getStr ( STR_ADD_GEOGRAPHY ) ) ;
        pb_DONE             = new DButton ( Str.getStr ( STR_DONE ) ) ;
        st_SIDE_PIC         = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.LOCALE_DLG ) ) ;
        buttonPanel         = new ButtonPanel () ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust properties                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        st_SIDE_PIC.setBorder ( GUISystem.loweredBorder ) ;    
        pb_ADD_SELECTED.setEnabled ( false ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add listeners                                           */
        /*                                                           */
        /*-----------------------------------------------------------*/
        cb_GEOGRAPHY.addActionListener        ( this ) ;
        pb_ADD_SELECTED.addActionListener     ( this ) ;
        pb_ADD_ENTIRE_GEO.addActionListener   ( this ) ;
        pb_DONE.addActionListener             ( this ) ;
        lb_COUNTRIES.addActionListener        ( this ) ;
        lb_COUNTRIES.addListSelectionListener ( this ) ;



        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our GUI                                             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        buttonPanel.add ( pb_ADD_SELECTED ) ;
        buttonPanel.add ( pb_ADD_ENTIRE_GEO ) ;
        buttonPanel.add ( pb_DONE ) ;
        contentPane.setLayout ( null ) ;
        contentPane.add ( st_SIDE_PIC ) ;
        contentPane.add ( st_ONE ) ;
        contentPane.add ( st_TWO ) ;
        contentPane.add ( st_THREE ) ;
        contentPane.add ( st_SELECT_GEOGRAPHY ) ;
        contentPane.add ( st_SELECT_COUNTRIES ) ;
        contentPane.add ( st_HELP ) ;
        contentPane.add ( cb_GEOGRAPHY ) ;
        contentPane.add ( scr_COUNTRIES ) ;
        contentPane.add ( buttonPanel ) ;
        GUISystem.setPropertiesOnPanel ( buttonPanel ) ;


        setSize ( 486, 337 ) ;
        WinUtil.centerChildInParent ( this, getParent() ) ;
    }


    /**
      * This method will lay out and position all of our controls
      */
    public void doLayout () {
        Dimension size      = getSize() ;
        int       rowHeight = GUISystem.getRowHeight() ;
        int       x         = 138 ;
        int       y         = 10 ;

        super.doLayout() ;

        st_SIDE_PIC.setBounds ( 10, 10, 118, 259 ) ;

        st_ONE.setBounds ( x, y, 32, 32 ) ;
        st_SELECT_GEOGRAPHY.setBounds ( x+35, y, size.width-(x+45), rowHeight ) ;
        y+=rowHeight ;

        cb_GEOGRAPHY.setBounds ( x+35, y, size.width-(x+45), rowHeight ) ;
        y+=rowHeight ;

        st_TWO.setBounds ( x, y, 32, 32 ) ;
        st_SELECT_COUNTRIES.setBounds ( x+35, y, size.width-(x+45), rowHeight ) ;
        y+=rowHeight ;

        scr_COUNTRIES.setBounds ( x+35, y, size.width-(x+45), rowHeight*8 ) ;
        y+=(rowHeight*8);

        st_THREE.setBounds ( x, y, 32, 32 ) ;
        st_HELP.setBounds ( x+35, y, size.width-(x+45), rowHeight*3 ) ;

        buttonPanel.setBounds ( x, size.height-60, size.width-(x+5), 25 ) ;


    }



    /**
      *  This method will be called when the user pushes one of our buttons
      * or makes a selection in the Geography drop down
      **/
    public void actionPerformed ( ActionEvent e ) {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   pb_DONE                                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( e.getSource() == pb_DONE )
            dispose() ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   cb_GEOGRAPHY                                            */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( e.getSource() == cb_GEOGRAPHY ) {
            TypeGeoRec geoRec = ((TypeGeoRec)cb_GEOGRAPHY.getSelectedItem()) ;

            lb_COUNTRIES.setListData ( geoRec.getCountryVec() ) ;
            PropertySystem.putInt ( PropertySystem.PROPERTY_LAST_GEOGRAPHY, geoRec.getInd() ) ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   pb_ADD_SELECTED                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( ( e.getSource() == pb_ADD_SELECTED ) ||
                  ( e.getSource() == lb_COUNTRIES ) ) {
            if ( listener != null )
                listener.localeDlgAddCountries ( lb_COUNTRIES.getSelectedValues() ) ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   pb_ADD_ENTIRE_GEO                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( e.getSource() == pb_ADD_ENTIRE_GEO ) {
            if ( listener != null )
                listener.localeDlgAddGeo ( (TypeGeoRec) cb_GEOGRAPHY.getSelectedItem() ) ;
        }
    } /* ActionPerformed  */



    /**
      *  This method will be called when the user has selected an item in the list
      * of countries
      **/
    public void valueChanged ( ListSelectionEvent e ) {
        pb_ADD_SELECTED.setEnabled ( lb_COUNTRIES.getSelectedValue() != null ) ;
    }


    /**
      *  This method will set the one LocaleDlgListener that this dialog will
      * notify when Countries/Geographies are added.
      *
      * @param listener The listener
      **/
    public void setLocaleDlgListener ( LocaleDlgListener listener ) {
        this.listener = listener ;
    }


    /**
      *  This method will return all geos EXCEPT the World-Wide one
      **/
    private Vector getGeoList () {
        Vector vec = TypeList.getInstance().getTypeList ( TypeList.TYPE_GEOGRAPHY ) ;

        vec.removeElement ( TypeList.getInstance().objectFromInd ( 6, TypeList.TYPE_GEOGRAPHY ) ) ;

        return( vec ) ;
    }
} /* LocaleDlg */
