package com.ibm.nzna.projects.qit.dbgui ;            
/*-----------------------------------------------------------*/
/*   ProductDocumentDlg                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductDocumentDlg                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;     
import com.ibm.nzna.projects.qit.doc.gui.* ;     
import com.ibm.nzna.projects.qit.product.gui.* ;     
/**
 * This Dialog shows the user a list of Products and Documents. It has a 
 * configurable description and the title is also configurable.
 *
 *
 * @version 1.0, 8/1/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductDocumentDlg                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductDocumentDlg extends JDialog implements ActionListener, AppConst {

   /**
     *  This is the description text
     **/
   private JLabel st_DESCRIPTION = new JLabel ("");

   /**
     *  This is the title text
     **/
   private JLabel st_TITLE = new JLabel ( "" ) ;

   /**
     *  This is the list of Documents
     **/
   private DocumentTable cnr_DOCUMENTS = new DocumentTable ( ) ;

   /**
     *  This is the JLabel title for the Document list
     **/
   private JLabel st_DOCUMENTS = new JLabel  ( Str.getStr ( STR_DOCUMENTS ) ) ;

   /**
     *  This is the JLabel title for the Product list
     **/
   private JLabel st_PRODUCTS = new JLabel  ( Str.getStr ( STR_PRODUCTS ) ) ;

   /**
     *  This is the list of Documents that we are showing
     **/
   private Vector documents = null ;

   /**
     *  Thi sis the list of Products that we are showing
     **/
   private Vector products = null ;


   /**
     *  This is the list of Products
     **/
   private ProductTable cnr_PRODUCTS = new ProductTable () ;

   /**
     *  This is the 'Ok' button
     **/
   private DButton pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ;


   /**
     *  This is the default constructor. Please note that the constructor
     * does not show the window. Please call the 'setVisible' method to do that
     *
     * @param parent The paretn window
     * @param products The list of products to show.
     * @param documents The list of documents to show
     **/
   public ProductDocumentDlg ( Frame parent, Vector products, Vector documents ) {
      super ( parent, Str.getStr ( STR_DOCUMENTS_PRODUCTS ), true ) ;

      this.documents = documents ;
      this.products  = products ;

      initialize() ;
   }


   /**
     *  This method will initialize our GUI
     **/
   private void initialize () {
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust propeties                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE.setFont ( FontSystem.largeTitleFont ) ;
      cnr_DOCUMENTS.setDocuments ( documents ) ;
      cnr_PRODUCTS.setProducts ( products ) ;          
      st_DESCRIPTION.setHorizontalAlignment ( SwingConstants.CENTER ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add any listeners                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_TITLE ) ;
      contentPane.add ( st_DESCRIPTION ) ;
      contentPane.add ( st_PRODUCTS ) ;
      contentPane.add ( cnr_PRODUCTS ) ;
      contentPane.add ( st_DOCUMENTS ) ;
      contentPane.add ( cnr_DOCUMENTS ) ;
      contentPane.add ( pb_OK ) ;

      setSize ( 600, 400 ) ;
      WinUtil.centerWindow ( this ) ;
   }


   /**
     *  This method will lay out and position all of our components
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = 5 ;
      int       y         = 5 ;

      super.doLayout() ;

      size.width-=x ;

      if ( st_TITLE.getText().length() > 0 ) {
         st_TITLE.setBounds ( x, y, size.width-(x*2), rowHeight*2 ) ;
         y+=rowHeight*2;
      }

      if ( st_DESCRIPTION.getText().length() > 0 ) {
         st_DESCRIPTION.setBounds ( x, y, size.width-(x*2), rowHeight*2 ) ;
         y+=rowHeight*2 ;
      }

      st_PRODUCTS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cnr_PRODUCTS.setBounds ( x, y, size.width-(x*2), rowHeight*6 ) ;
      y+=rowHeight*6 ;

      st_DOCUMENTS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cnr_DOCUMENTS.setBounds ( x, y, size.width-(x*2), size.height-65-y ) ;

      pb_OK.setBounds ( x, size.height-60, 80, 25 ) ;

   }


   /**
     *  This method will be called when an action event is fired upon us
     **/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == pb_OK )
         dispose() ;
   }


   /**
     *  This method will set the large title being displayed in the window
     *
     * @param largeTitle The new text to display
     **/
   public void setLargeTitle ( String largeTitle ) {
      st_TITLE.setText ( largeTitle ) ;
   }

   /**
     *  This method will set the text that is shown in the description
     *
     * @param descript The Description to show
     **/
   public void setDescription ( String description ) {
      st_DESCRIPTION.setText ( description ) ;
   }
} /* ProductDocumentDlg */
