package com.ibm.nzna.projects.qit.doc ;    
/*-----------------------------------------------------------*/
/*   DocCustomView                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocCustomView                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import java.awt.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
/**
 * This object represents the Document custom view 
 *
 *
 * @version 1.0, 4/29/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocCustomView                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocCustomView extends CustomView implements DocConst, AppConst {

   /*------------------------------------------------------------*/
   /*                                                            */
   /*   This is the serialization ID. If you add fields to this  */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is     */
   /* fine as long as you know what you are doing!               */
   /*                                                            */
   /* If you change this number, you break everyone's Doc Custom */
   /* views to the point where they will not load, so you better */
   /* write a converter or something to run on everyone's        */
   /* bookmarks                                                  */
   /*                                                            */
   /*------------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int[]     columnWidths   = { 20, 75, 300, 100, 200} ;
   private int       columnCount    = 5 ;
   private String    fromClause     = null ;
   private String    selectClause   = null ;
   private boolean   request        = false ;
   private boolean   queryFamily    = false ;
   private boolean   queryBrand     = false ;
   private boolean   queryDocCat    = false ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocCustomView () {
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a blank
    *  DocCustomView based on the type passed
    *  in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocCustomView ( int viewType ) {
      setViewType ( viewType ) ;
   }


   /*------------------------------------------------------------*/
   /* getSelect
    */
   /**
    *    This method will return the entire select
    *  clause including the 'SELECT' keyword
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getSelect () {
      return( selectClause ) ;
   }




   /*------------------------------------------------------------*/
   /* getWhere
    */
   /**
    *    This method will return the entire where clause
    *  including the 'WHERE' statement
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getWhere () {
      Vector          fields      = getFields() ;
      int             i           = 0 ;
      int             size        = fields.size() ;
      String          whereClause = null ;                      
      CustomViewField field       = null ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Start everything off real small                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         whereClause = "WHERE " ;
         queryFamily = false ;
         queryBrand  = false ;
         queryDocCat = false ;

         while ( i < size ) {
            field = (CustomViewField) fields.elementAt(i) ;                  

            if ( field.getDatabaseOperator() != null ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the field database name                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               whereClause+=field.getColumnName()+" " ;       


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the operator                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( !field.getColumnDescript().equals ( Str.getStr ( STR_BRAND ) ) )  &&
                    ( !field.getColumnDescript().equals ( Str.getStr ( STR_FAMILY ) ) ) )
                  whereClause+=field.getDatabaseOperator()+" " ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, depending on the column, put the value in          */
               /*                                                           */
               /*-----------------------------------------------------------*/

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Title, Changed time, Additional information             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( field.getColumnName().indexOf ( "CHANGEDTIME" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "OWNER" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "SUMMARY" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "EMAILABLEDATE" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "EMAILABLE" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "KNOWLEDGEBASE" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "FLASHDATE" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "FLASHNEWS" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "PARTNUM" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "SOURCE" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "COMMENT" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "REVIEWPERIOD" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "FAQ" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "LNDOCIND" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "TITLE" ) >= 0 ) ) {
                  if ( field.getDatabaseOperator().equals ( "LIKE" ) )
                     whereClause+="'%"+field.getValue()+"%' " ;
                  else if ( field.getDatabaseOperator().equals ( "IN" ) )
                     whereClause+="( "+field.getValue()+" ) " ;
                  else
                     whereClause+="'"+field.getValue()+"' " ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Document Type                                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "DOCTYPEIND" ) >=0 )
                  whereClause+=" "+((TypeDocRec)field.getValue()).getInd()+" " ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Doc Classification                                      */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "DOCCLASSIND" ) >=0 )
                  whereClause+=" "+((TypeDocClassRec)field.getValue()).getInd()+" " ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Metric Ind                                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "METRICIND" ) >=0 )
                  whereClause+="  "+((TypeMetricRec)field.getValue()).getInd()+" " ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Publication Type                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "PUBTYPE" ) >=0 )
                  whereClause+=" "+((TypePubRec)field.getValue()).getInd()+" " ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Work Ind                                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "WORKIND" ) >=0 )
                  whereClause+=" "+((TypeWorkRec)field.getValue()).getInd()+" " ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Brand                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_BRAND ) ) ) {
                  whereClause+=" IN (SELECT TYPECATIND FROM PRODUCT.TYPECATCHILD WHERE PARENTCATIND = "+((TypeCategoryRec)field.getValue()).getInd()+") AND DOCUMENTS.DOCIND = BCATEGORY.DOCIND " ;
                  queryBrand = true ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Family                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_FAMILY ) ) ) {
                  whereClause+=" IN (SELECT TYPECATIND FROM PRODUCT.TYPECATCHILD WHERE PARENTCATIND = "+((TypeCategoryRec)field.getValue()).getInd()+") AND DOCUMENTS.DOCIND = FCATEGORY.DOCIND " ;
                  queryFamily = true ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Document Category                                       */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_CATEGORY ) ) ) {
                  whereClause+=" "+((TypeCategoryRec)field.getValue()).getInd() ;
                  whereClause+=" AND DOCUMENTS.DOCIND = DCATEGORY.DOCIND ";
                  queryDocCat = true ;
               }



               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   DB User                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "DBUSER" ) >= 0 ) {
                  if ( field.getDatabaseOperator().equals ( "IN" ) )
                     whereClause+="( "+field.getValue()+" ) " ;
                  else
                     whereClause+="'"+field.getValue()+"' " ;
               }

               whereClause+="AND " ;
            } /* if field was to be used  */

            i++ ;
         } /* where  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Take the last 'AND' from the where clause off           */                                    
         /*                                                           */
         /*-----------------------------------------------------------*/
         whereClause = whereClause.substring ( 0, whereClause.length()-4 ) ;                  


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, adjust the From clause                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( request ) {
            fromClause = "FROM QUEST.DOCUMENTS DOCUMENTS " ;

            if ( queryBrand )
               fromClause+=", QUEST.CATEGORY BCATEGORY " ;
            
            if ( queryFamily )
               fromClause+=", QUEST.CATEGORY FCATEGORY ";
            
            if ( queryDocCat )
               fromClause+=", QUEST.CATEGORY DCATEGORY " ;
         } else {
            fromClause = "FROM TIGRIS.DOCUMENTS DOCUMENTS " ;
            if ( queryBrand )
               fromClause+=", TIGRIS.CATEGORY BCATEGORY " ;
            
            if ( queryFamily )
               fromClause+=", TIGRIS.CATEGORY FCATEGORY ";
            
            if ( queryDocCat )
               fromClause+=", TIGRIS.CATEGORY DCATEGORY " ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, add the standard restrictions to the WHERE clause  */                                            
         /*                                                           */
         /*-----------------------------------------------------------*/
         whereClause+=" AND DOCUMENTS.PUBLISH = 'Y' AND DOCUMENTS.RECYCLED = 'N' " ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }


      return( whereClause ) ;
   }




   /*------------------------------------------------------------*/
   /* getTables
    */
   /**
    *    This method will return all of the tables that
    *  this view will select from. This includes the
    *  keyword 'FROM'
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTables () {
      getWhere() ;
      return( fromClause ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnWidths
    */
   /**
    *    This method will return the column widths
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int[]
    *//*---------------------------------------------------------*/
   public int[] getColumnWidths () {
      return( columnWidths ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    This method will return the column count
    *  for this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getColumnCount () {
      return( columnCount ) ;
   }





   /*------------------------------------------------------------*/
   /* instantiateObject
    */
   /**
    *    This method will instaniate an object
    *  of the type passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomView instantiateObject ( int customType ) {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* getQuery
    */
   /**
    *    This method will return our full query
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getQuery () {
      return( getSelect()+" "+getTables()+" "+getWhere() ) ;
   }


   /*------------------------------------------------------------*/
   /* getResults
    */
   /**
    *    This method will kick off the query and will
    *  return the results from the query
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getResults () {
      Vector       retVec  = new Vector ( 1, 1 ) ;                                              
      DocViewRec   viewRec = new DocViewRec ( DocViewRec.VIEW_EXACT_SQL ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this is going against a Request, we need a different */
      /* Select and From clause                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( request ) {
         selectClause  = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                         "       DOCUMENTS.DOCCLASSIND,      DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                         "       DOCUMENTS.CHANGEDTIME " ;
      } else {
         selectClause         = "SELECT DISTINCT(DOCUMENTS.DOCIND), 0,                 DOCUMENTS.TITLE,   DOCUMENTS.OWNER,  "+
                                "       DOCUMENTS.DOCCLASSIND,      DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER, "+
                                "       DOCUMENTS.CHANGEDTIME " ;
      }

      viewRec.setCategoryInd ( -1 ) ;
      viewRec.setSQLQuery ( selectClause+" "+getTables()+" "+getWhere() ) ;
      viewRec.refreshNow() ;
      retVec = viewRec.getResults() ;

        return( retVec ) ;
   }




   /*------------------------------------------------------------*/
   /* setViewType
    */
   /**
    *    This method will set what type of query this
    *  is
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setViewType ( int type ) {
      super.setViewType( type ) ;

      fields = new Vector ( 5 ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.TITLE",         Str.getStr ( STR_TITLE ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.DOCTYPEIND",    Str.getStr ( STR_DOCUMENT_TYPE ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.DOCCLASSIND",   Str.getStr ( STR_DOC_CLASS ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.OWNER",         Str.getStr ( STR_OWNER ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.REVIEWPERIOD",  Str.getStr ( STR_MONTHS_TO_REVIEW ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.REQUEST",       Str.getStr ( STR_REQUEST ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.FAQ",           Str.getStr ( STR_FAQ ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.EMAILABLE",     Str.getStr ( STR_EMAILABLE ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.KNOWLEDGEBASE", Str.getStr ( STR_KNOWLEDGE_BASE ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.EMAILABLEDATE", Str.getStr ( STR_EMAILABLE_DATE ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.SUMMARY",       Str.getStr ( STR_SUMMARY ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.METRICIND",     Str.getStr ( STR_METRICS ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.PARTNUM",       Str.getStr ( STR_PART_NUMBER ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.PUBTYPE",       Str.getStr ( STR_PUBLICATION ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.SOURCE",        Str.getStr ( STR_SOURCE ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.WORKIND",       Str.getStr ( STR_WORK_REQUIRED ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.COMMENT",       Str.getStr ( STR_COMMENT ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.LNDOCIND",      Str.getStr ( STR_LOTUS_NOTES_ID ) ) ) ;
      fields.addElement ( new CustomViewField ( "BCATEGORY.TYPECATIND",    Str.getStr ( STR_BRAND ) ) ) ;
      fields.addElement ( new CustomViewField ( "FCATEGORY.TYPECATIND",    Str.getStr ( STR_FAMILY ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.DBUSER",        Str.getStr ( STR_LAST_TOUCHED_BY ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.CHANGEDTIME",   Str.getStr ( STR_LAST_TOUCHED_ON ) ) ) ;
      fields.addElement ( new CustomViewField ( "DCATEGORY.TYPECATIND",    Str.getStr ( STR_CATEGORY ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.FLASHNEWS",     Str.getStr ( STR_HOT_NEWS ) ) ) ;
      fields.addElement ( new CustomViewField ( "DOCUMENTS.FLASHDATE",     Str.getStr ( STR_FLASH_DATE ) ) ) ;

      new QuickSort ( fields ) ;
   }






   /*------------------------------------------------------------*/
   /* addFieldRestriction
    */
   /**
    *    This method will add a field restriction
    *  to this query
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addFieldRestriction ( CustomViewField field ) {
      Frame  tempFrame   = new Frame() ;                                          
      Vector operatorVec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always ask the user for what type of operator first for */
      /* most fields                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( field.getColumnName().indexOf ( "REQUEST" ) == -1 ) &&
           ( field.getColumnName().indexOf ( "FAQ" ) == -1 ) &&
           ( field.getColumnName().indexOf ( "FLASHNEWS" ) == -1 ) &&
           ( field.getColumnName().indexOf ( "EMAILABLE" ) == -1 ) &&
           ( field.getColumnName().indexOf ( "KNOWLEDGEBASE" ) == -1 ) )
         operatorVec = new JListDlg ( tempFrame, Str.getStr ( STR_OPERATOR ), field.getOperators() ).getResult() ;
      else {
         operatorVec = new Vector ( 1 ) ;
         operatorVec.addElement ( Str.getStr ( STR_EQUALS ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only continue if they selected an operator              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( operatorVec != null ) && ( operatorVec.size() > 0 ) ) {

         field.setOperator ( (String)operatorVec.elementAt(0) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Title                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( field.getColumnName().indexOf ( "TITLE" ) >= 0 ) {
            String title = new SingleEntryDlg ( tempFrame,
                                                Str.getStr ( STR_TITLE ),
                                                Str.getStr ( STR_TITLE ),
                                                "",
                                                "" ).getResult() ;
            if ( ( title != null ) && ( title.length() > 0 ) ) {
               field.setValue ( title ) ;
            }

         } /* title  */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Comment                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( field.getColumnName().indexOf ( "COMMENT" ) >= 0 ) {
            String comment = new SingleEntryDlg ( tempFrame,
                                                  Str.getStr ( STR_COMMENT ),
                                                  Str.getStr ( STR_COMMENT ),
                                                  "",
                                                  "" ).getResult() ;
            if ( ( comment != null ) && ( comment.length() > 0 ) ) {
               field.setValue ( comment ) ;
            }

         } /* comment  */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Summary                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( field.getColumnName().indexOf ( "SUMMARY" ) >= 0 ) {
            String summary = new SingleEntryDlg ( tempFrame,
                                                  Str.getStr ( STR_SUMMARY ),
                                                  Str.getStr ( STR_SUMMARY ),
                                                  "",
                                                  "" ).getResult() ;
            if ( ( summary != null ) && ( summary.length() > 0 ) ) {
               field.setValue ( summary ) ;
            }

         } /* summary  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Partnum                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( field.getColumnName().indexOf ( "PARTNUM" ) >= 0 ) {
            String partnum = new SingleEntryDlg ( tempFrame,
                                                  Str.getStr ( STR_PART_NUMBER ),
                                                  Str.getStr ( STR_PART_NUMBER ),
                                                  "",
                                                  "" ).getResult() ;
            if ( ( partnum != null ) && ( partnum.length() > 0 ) ) {
               field.setValue ( partnum ) ;
            }

         } /* partnum  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Flash Date                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "FLASHDATE" ) >= 0 ) {
            String date = new SingleDateEntryDlg ( tempFrame,
                                                   Str.getStr ( STR_FLASH_DATE ),
                                                   Str.getStr ( STR_DATE ),
                                                   new CDate ().today() ).getResult() ;
            if ( ( date != null ) && ( date.length() > 0 ) ) {
               date = Text.replaceAllStrInStr ( date, "/", "-" ) ;
               field.setValue ( date ) ;
            }
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Changed Time                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "CHANGEDTIME" ) >= 0 ) {
            String date = new SingleDateEntryDlg ( tempFrame,
                                                   Str.getStr ( STR_LAST_TOUCHED_ON ),
                                                   Str.getStr ( STR_LAST_TOUCHED_ON ),
                                                   new CDate ().today() ).getResult() ;
            if ( ( date != null ) && ( date.length() > 0 ) ) {
               date = Text.replaceAllStrInStr ( date, "/", "-" ) ;
               field.setValue ( date+"-07.00.00.000000" ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Notes Id                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "LNDOC" ) >= 0 ) {
            String lndocid = new SingleEntryDlg ( tempFrame,
                                                  Str.getStr ( STR_LOTUS_NOTES_ID ),
                                                  Str.getStr ( STR_LOTUS_NOTES_ID ),
                                                  "", "" ).getResult() ;
            if ( ( lndocid != null ) && ( lndocid.length() > 0 ) )
               field.setValue ( lndocid ) ;

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Source                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "SOURCE" ) >= 0 ) {
            String source = new SingleEntryDlg ( tempFrame,
                                                 Str.getStr ( STR_SOURCE ),
                                                 Str.getStr ( STR_SOURCE ),
                                                 "", "" ).getResult() ;
            if ( ( source != null ) && ( source.length() > 0 ) )
               field.setValue ( source ) ;

         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Months To Review                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "REVIEWPERIOD" ) >= 0 ) {
            String review = new SingleEntryDlg ( tempFrame,
                                                 Str.getStr ( STR_MONTHS_TO_REVIEW ),
                                                 Str.getStr ( STR_MONTHS_TO_REVIEW ),
                                                 "", new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 2 ) ).getResult() ;
            if ( ( review != null ) && ( review.length() > 0 ) )
               field.setValue ( review ) ;

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Owner                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "OWNER" ) >= 0 ) {
            Vector  vector  = new SelectUserDlg ( tempFrame ).getResult() ;
            UserRec userRec = null ;

            if ( ( vector != null ) && ( vector.size() > 0 ) ) {
               int     i    = 0 ;
               int     size = vector.size() ;
               String  str  = "" ;

               while ( i < size ) {
                  str+=((UserRec)vector.elementAt(i)) ;
                  if ( i != (size-1) )
                     str+="," ;
                  i++ ;
               } /* while  */

               field.setValue ( str ) ;

            } /* if  */
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Category                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "DCATEGORY" ) >= 0 ) {
            Vector vector = new JListDlg ( tempFrame, 
                                           Str.getStr ( STR_CATEGORY ), 
                                           TypeList.getInstance().getTypeList ( TypeList.TYPE_DOCCAT ) ).getResult() ;

            if ( ( vector != null ) && ( vector.size() > 0 ) ) {
               field.setValue ( vector.elementAt(0) ) ;
            }

         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Doc Type                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "DOCTYPEIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_DOCUMENT_TYPE ),
                                                    TypeList.getInstance().getTypeList ( TypeList.TYPE_DOC ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Metrics                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "METRICIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_METRICS ),
                                                    TypeList.getInstance().getTypeList ( TypeList.TYPE_METRIC ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   WorkInd                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "WORKIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_WORK_REQUIRED ),
                                                    TypeList.getInstance().getTypeList ( TypeList.TYPE_WORKREQUIRED ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Pub Type                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "PUBTYPE" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_PUBLICATION ),
                                                    TypeList.getInstance().getTypeList ( TypeList.TYPE_PUB ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Doc Class                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "DOCCLASSIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_DOC_CLASS ),
                                                    TypeList.getInstance().getTypeList ( TypeList.TYPE_DOCCLASS ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Request                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "REQUEST" ) >= 0 ) {
            request = GUISystem.printBox ( tempFrame, STR_QUESTION, STR_CUSTOM_VIEW_FOR_REQUESTS ) ;

            field.setValue ( ""+request ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   FAQ                                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "FAQ" ) >= 0 ) {
            boolean faq = GUISystem.printBox ( tempFrame, STR_QUESTION, STR_CUSTOM_VIEW_FOR_FAQ ) ;

            field.setValue ( ""+(faq ? "Y" : "N") ) ;
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   eMailable Date                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "EMAILABLEDATE" ) >= 0 ) {
            String date = new SingleDateEntryDlg ( tempFrame,
                                                   Str.getStr ( STR_EMAILABLE_DATE ),
                                                   Str.getStr ( STR_EMAILABLE_DATE ),
                                                   new CDate ().today() ).getResult() ;
            if ( ( date != null ) && ( date.length() > 0 ) ) {
               date = Text.replaceAllStrInStr ( date, "/", "-" ) ;
               field.setValue ( date ) ;
            }

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   KNOWLEDGEBASE                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "KNOWLEDGEBASE" ) >= 0 ) {
            boolean kb = GUISystem.printBox ( tempFrame, STR_QUESTION, STR_CUSTOM_VIEW_FOR_KB ) ;

            field.setValue ( ""+(kb ? "Y" : "N") ) ; 
         }




         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   EMAILABLE                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "EMAILABLE" ) >= 0 ) {
            boolean emailable = GUISystem.printBox ( tempFrame, STR_QUESTION, STR_CUSTOM_VIEW_FOR_EMAILABLE ) ;

            field.setValue ( ""+(emailable ? "Y" : "N") ) ; 
         }




         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Flash News                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "FLASHNEWS" ) >= 0 ) {
            boolean flashNews = GUISystem.printBox ( tempFrame, STR_QUESTION, STR_CUSTOM_VIEW_FOR_FLASH_NEWS ) ;

            field.setValue ( ""+(flashNews ? "Y" : "N") ) ; 
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   DB User                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "DBUSER" ) >= 0 ) {
            Vector  vector  = new SelectUserDlg ( tempFrame ).getResult() ;
            UserRec userRec = null ;

            if ( ( vector != null ) && ( vector.size() > 0 ) ) {
               int     i    = 0 ;
               int     size = vector.size() ;
               String  str  = "" ;

               while ( i < size ) {
                  str+=((UserRec)vector.elementAt(i)) ;
                  if ( i != (size-1) )
                     str+="," ;
                  i++ ;
               } /* while  */

               field.setValue ( str ) ;

            } /* if  */
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Brand                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( field.getColumnName().indexOf ( "BCATEGORY" ) >= 0 ) &&
                   ( field.getColumnDescript().equals ( Str.getStr ( STR_BRAND ) ) ) ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_BRAND ),
                                                    Brands.getBrands() ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Family                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( field.getColumnName().indexOf ( "FCATEGORY" ) >= 0 ) &&
                   ( field.getColumnDescript().equals ( Str.getStr ( STR_FAMILY ) ) ) ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_FAMILY ),
                                                    Brands.getFamilies() ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }



      } /* if they selected an operator  */

      tempFrame.dispose() ;
   }





   /*------------------------------------------------------------*/
   /* getRequest
    */
   /**
    *    This method will return TRUE if this query
    *  pulls Document Requests
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getRequest () {
      return( request ) ;
   }
} /* DocCustomView */
