package com.ibm.nzna.projects.qit.doc ;              
/*-----------------------------------------------------------*/
/*   DocTemplateSystem                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocTemplateSystem                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
/**
 * This system maintains all of the document templates for a user 
 *
 *
 * @version 1.0, 3/28/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocTemplateSystem                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocTemplateSystem extends Object {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   private members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static Vector            templateVec    = null ;
    private static boolean           templatesRead  = false ;
    private static DocTemplateSystem instance       = null ;





    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will simply create this object. Templates
     *  will not be read until they are requested for
     *  the first time
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public DocTemplateSystem () {
        templateVec = new Vector ( 1, 1 ) ;
        instance = this ;
    }




    /*------------------------------------------------------------*/
    /* getTemplateVec
     */
    /**
     *    This method will return the vector of templates.
     *  If the templates have not been read in yet, they
     *  will be read in now.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Vector getTemplateVec ( ) {
        if ( !templatesRead )
            readAllTemplates ( ) ;

        return( templateVec ) ;
    }




    /*------------------------------------------------------------*/
    /* readAllTemplates
     */
    /**
     *    This method will read in all of our templates
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private Vector readAllTemplates ( ) {
        DocViewRec viewRec = new DocViewRec ( DocViewRec.VIEW_EXACT_SQL ) ;

        viewRec.setCategoryInd ( -1 ) ;
        viewRec.setSQLQuery ( "SELECT DOCUMENTS.DOCIND,      DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND, DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM QUEST.DOCUMENTS DOCUMENTS "+
                              "WHERE DOCUMENTS.DBUSER = '"+UserSystem.getUserId()+"' AND "+
                              "      DOCUMENTS.TEMPLATE = 'Y' " ) ;

        viewRec.refreshNow() ;

        templateVec   = viewRec.getResults() ;
        templatesRead = true ;

        return( templateVec ) ;
    }




    /*------------------------------------------------------------*/
    /* getInstance
     */
    /**
     *    This method will return the one instance
     *  of the DocTemplateSystem
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return DocTemplateSystem
     *//*---------------------------------------------------------*/
    public static DocTemplateSystem getInstance () {
        return( instance ) ;
    }


    /*------------------------------------------------------------*/
    /* addTemplate
     */
    /**
     *    This method will add a template to our list
     *  
     *  
     *
     *  @param docRec The template to add
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void addTemplate ( DocumentDraft docRec ) {
        if ( templateVec == null )
            templateVec = new Vector ( 1, 1 ) ;

        templateVec.addElement ( new DocumentRow ( docRec ) ) ;
    }





    /*------------------------------------------------------------*/
    /* removeTemplate
     */
    /**
     *    This method will remove a template from
     *  the list of templates
     *  
     *
     *  @param docInd The document indicator of the template to remove
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public static void removeTemplate ( int docInd ) {
        if ( templateVec != null ) {
            int     i       = 0 ;
            int     size    = templateVec.size() ;
            boolean removed = false ;

            while ( ( !removed ) && ( i < size ) ) {
                if ( ((Document)templateVec.elementAt(i)).getDocInd() == docInd ) {
                    templateVec.removeElementAt ( i ) ;
                    removed = true ;
                }
                i++ ;
            }
        }
    }


} /* DocTemplateSystem */
