package com.ibm.nzna.projects.qit.doc.docedit ;     
/*-----------------------------------------------------------*/
/*   CategoryPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CategoryPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import javax.swing.event.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This panel holds the document categories and buttons to:<br><br>
 *
 * Remove a Category<br>
 * Add a Category<br>
 * Remove All Categories<br><br>
 *
 * @version 1.0, 5/2/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CategoryPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class CategoryPanel extends DPanel implements DocumentPropertyListener, DocCategoryDlgListener, ListSelectionListener, ActionListener, AppConst, DocConst {

   /**
     *  This is the HTML That is prepended to the category description
     * when the category is being displayed in a JList
     **/
   private static final String PRE_HTML = "<html><body><font size=2>" ;

   /**
     *  This is the HTML that is appended to the end of the category
     * description when the category is displayed in a JList
     **/
   private static final String SUF_HTML = "</body></html>" ;

   /**
     *  This static variable defines the preferred size of this panel
     **/
   private static final Dimension prefSize = new Dimension ( 100, 300 ) ;

   /**
     *  This is the JList which will hold all of the Document Categories
     **/
   private JList categories      = null ;

   /**
     *  This is the button that will add a category
     **/
   private DButton addCategory   = null ;

   /**
     *  This is the button that will remove a category
     **/
   private DButton removeCategory = null ;

   /**
     *  This is the button that will remove all categories
     **/
   private DButton removeAll = null ;

   /**
     * The List model that our list is using
     */
   private DefaultListModel listModel = null ;

   /**
     *  This is the JScrollPane that the Categories sit int
     **/
   private JScrollPane scrollPane = null ;

   /**
     *  This is the button panel which will hold all of our
     * buttons
     **/
   private ListPanel buttonPanel = null ;

   /**
     *  This is the document that this panel is working with
     **/
   private Document document = null ;


   /**
     *  The default constructor only initializes the GUI
     **/
   public CategoryPanel () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      categories     = new JList () ;
      addCategory    = new DButton ( Str.getStr ( STR_ADD ) ) ;
      removeCategory = new DButton ( Str.getStr ( STR_REMOVE ) ) ;
      removeAll      = new DButton ( Str.getStr ( STR_REMOVE )+" "+Str.getStr ( STR_ALL ) ) ;
      scrollPane     = new JScrollPane ( categories ) ;               
      buttonPanel    = new ListPanel () ;
      categories.setModel ( listModel = new DefaultListModel() ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties on our panel                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      buttonPanel.setBackground ( Color.white ) ;
      removeAll.setEnabled ( false ) ;
      removeCategory.setEnabled ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      categories.addListSelectionListener ( this ) ;
      addCategory.addActionListener       ( this ) ;
      removeCategory.addActionListener    ( this ) ;
      removeAll.addActionListener         ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI to our panel                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.add ( addCategory ) ;
      buttonPanel.add ( removeCategory ) ;
      buttonPanel.add ( removeAll ) ;
      buttonPanel.setPreferredSize ( new Dimension ( 90, 100 ) ) ;
      setPreferredSize ( new Dimension ( 515, 150 ) ) ;
      setLayout ( new BorderLayout () );
      add ( GUISystem.getRequiredJLabel(), BorderLayout.NORTH ) ;
      add ( scrollPane, BorderLayout.CENTER ) ;
      add ( buttonPanel, BorderLayout.EAST ) ;


   } /* Constructor  */


   /**
     *  This method will set the document that we are editing. Any changes made to this 
     * panel will be reflected in the document. Any previous information will be wiped
     * clean.
     **/
   protected void setDocument ( Document document ) {

      if ( this.document != document ) {
         if ( this.document != null )
            this.document.removeDocumentPropertyListener ( this ) ;
         this.document = document ;
         document.addDocumentPropertyListener ( this ) ;
         populateList() ;
      }

      else {
         populateList() ;
      }
   }

   /**
     * This method will populate our JList with information from
     * the document object we have.
     */
   private void populateList () {
      if ( ( document.getCategory() != null ) && ( document.getCategory().size() > 0 ) ) {
         Vector categoryList = prepareCategoryList ( document.getCategory() ) ;

         WinUtil.setListData ( document.getCategory(), listModel ) ;
         removeAll.setEnabled ( true ) ;
      } else {
         listModel.removeAllElements() ;
         removeAll.setEnabled ( false ) ;
      }

      removeCategory.setEnabled ( false ) ;

      if ( listModel.size() == 0 )
         listModel.addElement ( "<html><body><font size=2 name=\"Helvetica, Arial\">No Document Categories, press 'Select' to<br>add categories.</font></body></html>" ) ;

   }


   /**
     * This method will be called when a list selection event from our JList is sent
     */
   public void valueChanged ( ListSelectionEvent e ) {
      if ( isEnabled () )
         removeCategory.setEnabled  ( ( categories.getSelectedValue() != null ) &&
                                      ( !(categories.getSelectedValue() instanceof String) ) ) ;
   }



   /**
     * This method will be called when an action event is thrown to us from one of our
     * buttons
     */
   public void actionPerformed ( ActionEvent e ) {

      if ( isEnabled() ) {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   addCategory                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == addCategory ) {
            Vector applicableFamilies = getAllFamilies () ;


            if ( ( applicableFamilies != null ) && ( applicableFamilies.size() > 0 ) ) {
               Vector         applicableBrands = getBrands ( applicableFamilies ) ;
               DocCategoryDlg dlg              = new DocCategoryDlg ( WinUtil.getParentFrame ( this ),
                                                                      applicableBrands ) ;

               dlg.setDocCategoryDlgListener ( this ) ;
               dlg.waitForUser() ;

            } /* if we have available Categories  */
            else
               GUISystem.printBox ( STR_INFO, STR_CANNOT_ADD_CATEGORIES_WITHOUT_BRAND ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   removeCategory                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == removeCategory ) {
            Object[] values        = categories.getSelectedValues() ;
            Vector   oldCategories = document.getCategory() ;

            if ( ( values != null ) && ( values.length > 0 ) ) {
               for ( int i = 0; i < values.length; i++ )
                  oldCategories.removeElement ( values[i] ) ;

               document.setCategory ( oldCategories ) ;
               populateList() ;
            } /* if  */
         } /* if  */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   removeAll                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == removeAll ) {
            document.setCategory ( new Vector ( 1 ) ) ;
            populateList() ;
         } /* removeAll  */
      }
   }


   /**
     *  This method will create a Vector of unique brands from the
     * vector of families given
     *
     * @param families The Vector of TypeCategoryRec objects 
     *
     * @return Vector of TypeCategoryRec objects. Each one representing a unique brand
     **/
   private Vector getBrands ( Vector families ) {
      int    i      = 0 ;
      int    size   = families.size() ;
      Vector brands = new Vector () ;

      while ( i < size ) {
         TypeCategoryRec brand = TypeCategory.getCategory ( ((TypeCategoryRec)families.elementAt(i)).getParentCatInd() ) ;

         if ( !brands.contains ( brand ) )
            brands.addElement ( brand ) ;
         i++ ;
      }

      return( brands ) ;
   }

   /**
     *  This method will create the model for a JTree to display all possible
     * available categories. 
     **/
   private DefaultTreeModel getAvailableCategories () {
      DefaultTreeModel model = null ;
      StringTreeNode   root  = new StringTreeNode ( Str.getStr ( STR_CATEGORIES_BY_BRAND ) ) ;
      Vector applicableBrands = Brands.getBrands() ; //document.getApplicableBrands() ;

      if ( ( applicableBrands != null ) && ( applicableBrands.size() > 0 ) ) {
         int            i                = 0 ;
         int            size             = applicableBrands.size() ;                         
         StringTreeNode currentBrandNode = null ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our model                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         model = new DefaultTreeModel ( root ) ;
         new QuickSort ( applicableBrands ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Walk the list of applicable brands                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            TypeCategoryRec brandRec        = (TypeCategoryRec)applicableBrands.elementAt(i) ;
            Vector          brandCategories = TypeCategory.getChildCategories ( brandRec.getInd(),
                                                                                TypeCategory.LEVEL_DOC_CATEGORY ) ;                             

            new QuickSort ( brandCategories ) ;
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, add each category from that brand                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( brandCategories != null ) && ( brandCategories.size() > 0 ) ) {
               int catIndex = 0 ;
               int catSize  = brandCategories.size() ;

               model.insertNodeInto ( currentBrandNode = new StringTreeNode ( brandRec.toString() ), root, i ) ;

               while ( catIndex < catSize ) {
                  TypeCategoryRec categoryRec = (TypeCategoryRec)brandCategories.elementAt ( catIndex ) ;
                  model.insertNodeInto ( new MutableTreeWrapper ( categoryRec ), currentBrandNode, catIndex ) ;
                  catIndex++ ;
               }
            }
            i++ ;
         }
      }

      return( model ) ;
   }

   public void docCategoryDlgAdd ( Object[] categories ) {
      if ( categories != null ) {
         Vector currentCategories = document.getCategory() ;

         if ( currentCategories == null )
            currentCategories = new Vector ( categories.length ) ;

         for ( int i = 0; i < categories.length; i++ ) {
            if ( !currentCategories.contains ( categories[i] ) )
               currentCategories.addElement ( categories[i] ) ;
         }

         document.setCategory ( currentCategories ) ;
      }
   }

   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;
      addCategory.setEnabled ( enable ) ;
      removeCategory.setEnabled ( enable ) ;
      removeAll.setEnabled ( enable ) ;
   }


   public void documentChanged ( Document document, int property ) {
      if ( property == document.CATEGORY )
         populateList() ;
   }


   /**
     * This method will return all Applicable Families and directly
     * associated families for the document
     **/
   private Vector getAllFamilies () {
      Vector returnVec = document.getApplicableFamilies() ;
      Vector families           = document.getFamilies() ;

      if ( returnVec == null )
         returnVec = new Vector ( 1 ) ;

      if ( ( families != null ) && ( families.size() > 0 ) ) {
         int i    = 0 ;
         int size = families.size() ;

         while ( i < size ) {
            if ( !returnVec.contains ( families.elementAt(i) ) )
               returnVec.addElement ( families.elementAt(i) ) ;
            i++ ;
         }
      }

      return( returnVec ) ;
   }


   private Vector prepareCategoryList ( Vector categories ) {
      Vector returnList = new Vector () ;

      new QuickSort ( document.getCategory() ) ;


      return ( returnList ) ;
   }
} /* CategoryPanel */

