package com.ibm.nzna.projects.qit.doc.docedit ;    
/*-----------------------------------------------------------*/
/*   DocEditPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocEditPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.doc.DocConst ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.form.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.spell.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.util.LogSystem ;
/**
 * The DocEditPanel is a panel which allows the editing of
 * any property on a Document Draft. This panel extends JPanel
 * so you can place this editor wherever you can place a normal
 * JPanel. <br><br>
 *
 * This panel also handles the saving/publishing of a document that
 * is being edited. Please use the 'saveToDatabase' method to accomplish
 * this. The 'saveToMemory' method will save what the user has changed
 * to the DocumentDraft object in memory only.
 *
 *
 * @version 1.0, 4/25/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocEditPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocEditPanel extends JPanel implements SpellPanelListener, DataLengths, Runnable, PropertyListener, ActionListener, DocumentPropertyListener, FocusListener, DocConst, AppConst {

   /**
     * This thread will set the document body content in the browser
     **/
   private static final String THREAD_SET_DOC_BODY = "DOCBODY" ;

   /**
     *  This represents the FormComponent which allows editing of the title
     * in the layout
     **/
   public static final String FORM_TITLE = Str.getStr ( STR_TITLE ) ;

   /**
     *  This represents the FormComponent which allows the editing of the
     * Document Type and Document Classification
     * in the layout
     **/
   public static final String FORM_TYPE_CLASS = Str.getStr ( STR_TYPE_CLASS ) ;

   /**
     *  This represents the FormComponent that shows the Draft Information
     * in the layout
     **/
   public static final String FORM_DRAFT_INFO = Str.getStr ( STR_DRAFT_INFO ) ;

   /**
     *  This represents the FormComponent that allows the comments to be edited
     * in the layout
     **/
   public static final String FORM_COMMENT = Str.getStr ( STR_COMMENT ) ;

   /**
     *  This represents the FormComponent which shows the Comment History
     * in the layout
     **/
   public static final String FORM_COMMENT_HISTORY = Str.getStr ( STR_COMMENT_HISTORY ) ;

   /**
     *  This represents the FormComponent which allows editing of Publishing Tags
     * in the layout
     **/
   public static final String FORM_PUBLISHING_OPTIONS = Str.getStr ( STR_PUBLISHING_OPTIONS ) ;

   /**
     *  This represents the FormComponent which allows editing of Categories
     * in the layout
     **/
   public static final String FORM_CATEGORY = Str.getStr ( STR_CATEGORY ) ;

   /**
     *  This represents the FormComponent which allows editing of the location
     * in the layout
     **/
   public static final String FORM_DOCUMENT_LOCATION = Str.getStr ( STR_LOCATION ) ;

   /**
     *  This is the ForComponent which allows editing of product assocations
     * in the layout
     **/
   public static final String FORM_DOCUMENT_PRODUCTS = Str.getStr ( STR_PRODUCTS ) ;

   /**
     *  This represents the FormComponent which allows the editing of
     * downloadable file information
     * in the layout
     **/
   public static final String FORM_DOWNLOADABLE_FILE_INFO = Str.getStr ( STR_DOWNLOADABLE_FILE_INFO ) ;

   /**
     *  This represents the FormComponent which allows the editing of File Attachments
     * in the layout
     **/
   public static final String FORM_FILE_ATTACHMENTS = Str.getStr ( STR_FILE_ATTACHMENT ) ;

   /**
     *  This represents the FormComponent which allows the editing of Proactive eMail
     * in the layout
     **/
   public static final String FORM_PROACTIVE_EMAIL = Str.getStr ( STR_PROMAIL ) ;

   /**
     *  This represents the FormComponent which shows doc links
     * in the layout
     **/
   public static final String FORM_THIS_DOCUMENT_LINKS_TO = Str.getStr ( STR_LINKS_FROM_THIS_DOCUMENT ) ;

   /**
     *  This represents the FormComponent which shows Links to this document
     * in the layout
     **/
   public static final String FORM_DOCUMENTS_LINK_TO_THIS = Str.getStr ( STR_LINKS_TO_THIS_DOCUMENT ) ;

   /**
     *  This represents the FormComponent which shows the Quest Document Information
     * in the layout
     **/
   public static final String FORM_QUEST_DOCUMENT_INFO = Str.getStr ( STR_DATABASE_INFO ) ;

   /**
     * This represents the FormComponent which shows the Search Engine Query Index
     **/
   public static final String FORM_SEARCH_ENGINE_QUERY_INDEX = Str.getStr ( STR_SEARCH_ENGINE_QUERY_INDEX ) ;


   /**
     * This is default spell step
     **/
   private static final int SPELL_NONE = 0 ;

   /**
     * This is the step that the spellCheckStep property is set to when checking
     * the title
     **/
   private static final int SPELL_TITLE = 1 ;

   /**
     * This is the step that the spellCheckStep property is set to when checking
     * the proactive eMail body
     **/
   private static final int SPELL_EMAIL = 2 ;

   /**
     *  This is the step that the spellCheckStep properrty is set to when checking
     * the description of an SSD
     **/
   private static final int SPELL_DESCRIPTION = 3 ;

   /**
     *  This is the step that the spellCheckStep properrty is set to when checking
     * the version of an SSD
     **/
   private static final int SPELL_VERSION = 4 ;

   /**
     * This is the last spell check step
     **/
   private static final int SPELL_LAST = 5 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     * This property holds what step in the spell check process we are at
     **/
   private int spellCheckStep = SPELL_NONE ;

   /**
     * If we are spell checking, this is the instance of the spell checker we are using
     **/
   private SpellPanel spellPanel = null ;

   /**
     *  This is the DocumentEditor that we belong to. Please note that this property is
     * only set if and only if the spellCheck method is called
     **/
   private DocumentEditor documentEditor = null ;

   /**
     *  This is the MultiLineFormComponent which holds and allows editing
     * of the Document Draft Title.
     **/
   private MultiLineFormComponent titleComponent      = null ;

   /**
     *  This is the FormComponent which holds all associated Brands, Families,
     *  machines, models, sub categories and part numebers.
     **/
   private ProductPanel productComponent = null ;

   /**
     *  This JTextArea holds the old comments from the current draft. These
     * comments are not yet engraved in History, however they cannot be changed
     **/
   private NoTabJTextArea currentCommentHistory = null ;

   /**
     *  This is the panel which holds the documents that this document links to
     **/
   private DocumentLinkPanel docLinksToPanel = null ;

   /**
     *  This is the panel which holds the referring documents that link to this document
     **/
   private DocumentLinkPanel refDocLinksPanel = null ;


   /**
     *  This is the component which holds the proactive eMail stuff
     **/
   private ProMailPanel proMailPanel = null ;

   /**
     *  This is the File Attachment
     **/
   private FileAttachmentPanel fileAttachments = null ;

   /**
     *  This property defines if we enable the user to edit anything. If not
     * this editor can be thought of as a read only editor
     **/
   private boolean enableInput = true ;

   /**
     *  This JTextArea holds the user editable comments
     **/
   private NoTabJTextArea currentComment = null ;

   /**
     * This is the MultiListFormComponent which holds the comment history.
     * this field is read only
     */
   private MultiLineFormComponent commentHistoryComponent = null ;

   /**
     *  This is the Form Component which holds the Document Classification
     **/
   private RadioFormComponent documentClass = null ;

   /**
     *  This is the FormComponent which holds the Document Type
     **/
   private RadioFormComponent documentType = null ;

   /**
     *  This is the DocumentInfoComponent
     **/
   private DocumentInfoFormComponent documentInfo = null ;

   /**
     * This is the Search Engine Query Index
     **/
   private MultiLineFormComponent searchEngineQueryComponent = null ;

   /**
     *  This is the component which holds all of the categories
     *  assigned to a document
     **/
   private CategoryPanel category = null ;

   /**
     *  This is the Publishing Options panel, which holds
     * the webable, hot news, online assistant and review information
     **/
   private PublishingOptionsPanel publishingOptions = null ;

   /**
     *  This is the Locale Panel which holds all associations to
     * geographies and countries
     **/
   private LocalePanel locale = null ;

   /**
     *  This is the Draft Information panel that shows the
     * Requestor, Content Owner, Document Source and Source Classification
     **/
   private DraftInfoPanel draftInfo = null ;

   /**
     *  This is the Downloadable FIle Information panel
     **/
   private DownloadableInfoPanel downloadablePanel = null ;

   /**
     *  This is the default width and height for all FormComponents
     **/
   private Dimension defaultFormSize = new Dimension ( 600, -1 ) ;

   /**
     *  This is the Document Draft that we are editing
     **/
   private Document document = null ;

   /**
     *  This panel holds all of the data displayed to the user
     **/
   private ListPanel dataPanel = new ListPanel () ;

   /**
     * This panel holds the Field list and the browser 
     **/
   private JPanel fieldBodyPanel = new JPanel () ;

   /**
     *  This is the JScrollPane which holds all of our form components on the data panel
     **/
   private JScrollPane scrollPane = new JScrollPane ( dataPanel ) ;


   /**
     * This is the JToggle Button for the fields
     **/
   private JToggleButton pb_FIELDS = new JToggleButton ( Str.getStr ( STR_FIELDS ) ) ;

   /**
     * This is the JToggleButton for the Body
     **/
   private JToggleButton pb_BODY = new JToggleButton ( Str.getStr ( STR_BODY ) ) ;

   /**
     *  This vector holds all of the GUI Components that we have added ourselves
     * to as a FocusListener
     **/
   private Vector focusGUI = new Vector ( 1 ) ;



   /**
     * This is the browser to show the body
     **/
   private Browser browser = GUISystem.createBrowser() ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*   Methods                                                 */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*                                                           */
   /*-----------------------------------------------------------*/








   /**
     *  This constructor creates and iniitalizes a blank document editor
     **/
   public DocEditPanel () {
      super() ;

      initialize() ;
   }

   /**
     *  This method will initialize all of our form components and will
     *  add each one to this panel
     **/
   private void initialize () {
      ButtonPanel togglePanel = new ButtonPanel () ;
      ButtonGroup group       = new ButtonGroup () ;

      PropertySystem.addPropertyListener ( PROPERTY_EDITOR_LAYOUT, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      scrollPane.getVerticalScrollBar().setUnitIncrement ( GUISystem.getRowHeight() );
      setBackground ( Color.white ) ;               
      setPreferredSize ( new Dimension  ( 630, -1 ) ) ;
      dataPanel.setPadding ( 10 ) ;
      dataPanel.setBackground ( Color.white ) ;
      group.add ( pb_FIELDS ) ;
      group.add ( pb_BODY ) ;
      pb_FIELDS.setSelected ( true ) ;
      togglePanel.setBackground ( Color.white ) ;                        

      /*----------------*/
      /*                */
      /*  Add listeners */
      /*                */
      /*----------------*/
      pb_FIELDS.addActionListener ( this ) ;
      pb_BODY.addActionListener   ( this ) ;                  

      /*-------------------*/
      /*                   */
      /*  Add our Controls */
      /*                   */
      /*-------------------*/
      togglePanel.add ( pb_FIELDS ) ;
      togglePanel.add ( pb_BODY ) ;
      fieldBodyPanel.setLayout ( new CardLayout() ) ;
      fieldBodyPanel.add ( scrollPane, "FIELDS" ) ;
      fieldBodyPanel.add ( browser.getComponent(), "BODY" ) ;


      setLayout ( new BorderLayout () ) ;
      add ( togglePanel, BorderLayout.NORTH ) ;
      add ( fieldBodyPanel ) ;

      GUISystem.setPropertiesOnPanel ( togglePanel ) ;

   }

   /**
     *  This method will set the Document Draft that is being
     * edited. Please use caution when calling this method. If this
     * editor is already editing a document draft, all changes for that
     * draft will be lost and replaced with the document draft passed in
     *
     * @param draft The Document Draft you wish to edit
     **/
   public void setDocument ( Document document ) {
      if ( this.document != null ) {
         this.document.writeToMemory() ;
         this.document.removeDocumentPropertyListener ( this ) ;
      }

      this.document = document ;

      if ( document != null ) {
         document.addDocumentPropertyListener ( this ) ;
         document.enableUndo ( true ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the GUI to ourselves                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         addFormComponents() ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the document                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         documentType.setSelectedItem  ( TypeList.getInstance().objectFromInd ( document.getDocumentType(),
                                                                                TypeList.TYPE_DOC ) ) ;
         documentClass.setSelectedItem  ( TypeList.getInstance().objectFromInd ( document.getDocumentClass(),
                                                                                 TypeList.TYPE_DOCCLASS ) ) ;
         titleComponent.setText            ( document.getTitle () ) ;
         publishingOptions.setDocument     ( document ) ;
         productComponent.setDocument      ( document ) ;
         draftInfo.setDocument             ( document ) ;
         documentInfo.setDocument          ( document ) ;
         category.setDocument              ( document ) ;
         locale.setDocument                ( document ) ;
         proMailPanel.setDocument          ( document ) ;
         docLinksToPanel.setDocumentLinks  ( document.getDocumentLinks() ) ;
         refDocLinksPanel.setDocumentLinks ( document.getReferringDocumentLinks() ) ;
         searchEngineQueryComponent.setText ( (document.getQueryText() != null) ? document.getQueryText().toString() : "" ) ;

         if ( fileAttachments != null ) {
            fileAttachments.setDocument ( document ) ;
            downloadablePanel.setDocument ( document ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Populate the Draft Only Fields                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( document instanceof DocumentDraft ) {
            currentCommentHistory.setText ( ((DocumentDraft)document).getOriginalComments() ) ;
            currentComment.setText ( "" ) ;
            currentComment.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_DOCCOMMENT-((DocumentDraft)document).getOriginalComments().length() ) ) ;
            commentHistoryComponent.setText ( ((DocumentDraft)document).getCommentHistoryAsText() ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the text on the body button                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( document.getDocumentType() != TypeList.DOCTYPE_SSD )
            pb_BODY.setText ( Str.getStr ( STR_BODY ) ) ;
         else
            pb_BODY.setText ( Str.getStr ( STR_ADDITIONAL_INFORMATION ) ) ;

         new Thread ( this, THREAD_SET_DOC_BODY ).start() ;
      } else
         dataPanel.removeAll() ;

      setEnabled ( enableInput ) ;

      revalidate() ;
   }


   /**
     *  This method will save the information on the editor to the
     * ProductDraft object in memory only.<br><br>
     *
     * @param showErrors If TRUE, this method will display errors
     *        to the user regarding missing or incorrect data
     **/
   public boolean saveToMemory ( boolean showErrors ) {
      return( false ) ;
   }





   /**
     *  This method will take the PROPERTY_EDITOR_LAYOUT vector
     * and will add each form component as specified by the vector
     **/
   private void addFormComponents () {
      Vector        layout    = (Vector)PropertySystem.get ( PROPERTY_EDITOR_LAYOUT ) ;
      int           i         = 0 ;
      int           size      = layout.size() ;
      String        current   = null ;                                        
      Container     component = null ;               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, remove everything                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      removeAllFocusListeners() ;
      dataPanel.removeAll() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Loop through each item and add the form components      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {

         current = (String)layout.elementAt(i) ;         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   The Document Title                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( current.equals ( FORM_TITLE ) ) {
            component = titleComponent = new MultiLineFormComponent ( Str.getStr ( STR_TITLE ), "", false ) ;
            titleComponent.getJTextArea().setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN,
                                                                           LENGTH_DOCUMENT_TITLE ) ) ;
            titleComponent.getJTextArea().setBorder ( GUISystem.blackBorder ) ;
            titleComponent.getJTextArea().setFont ( FontSystem.largeTitleFont ) ;
            titleComponent.getJTextArea().setForeground ( new Color ( 0, 0, 150 ) ) ;
            titleComponent.setForeground ( (Color)UIManager.getDefaults().get ( "Form.titleForeground" ) ) ;
            titleComponent.setFont ( (Font)UIManager.getDefaults().get ( "Form.titleFont" ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Product components                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_DOCUMENT_PRODUCTS ) ) {
            productComponent = new ProductPanel () ;
            component = new FormComponent ( Str.getStr ( STR_PRODUCTS ),
                                            productComponent ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Document Type and Classification                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_TYPE_CLASS ) ) {
            JPanel    panel    = new JPanel () ;
            Dimension prefSize = new Dimension ( 320, -1 ) ;

            documentType  = new RadioFormComponent ( Str.getStr ( STR_DOCUMENT_TYPE ),
                                                     TypeList.getInstance().getTypeList ( TypeList.TYPE_DOC ) ) ;
            documentClass = new RadioFormComponent ( Str.getStr ( STR_DOC_CLASS ),
                                                     TypeList.getInstance().getTypeList ( TypeList.TYPE_DOCCLASS ) ) ;

            documentClass.addActionListener ( this ) ;
            documentType.addActionListener ( this ) ;
            GUISystem.setPropertiesOnFormComponent ( documentType ) ;
            GUISystem.setPropertiesOnFormComponent ( documentClass ) ;
            documentType.setPreferredSize ( prefSize ) ;
            documentClass.setPreferredSize ( prefSize ) ;
            panel.setLayout ( new StraightLayoutMgr() ) ;
            panel.add ( documentType, BorderLayout.WEST ) ;
            panel.add ( documentClass, BorderLayout.EAST ) ;
            panel.setBackground ( Color.white ) ;
            component = panel ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Draft Information                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_DRAFT_INFO ) ) {
            draftInfo = new DraftInfoPanel () ;
            component = new FormComponent ( Str.getStr ( STR_DRAFT_INFO ),
                                            draftInfo ) ;
            ((FormComponent)component).setPreferredSize ( defaultFormSize ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Document Information                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_QUEST_DOCUMENT_INFO ) ) {
            documentInfo = new DocumentInfoFormComponent() ;
            component = new FormComponent ( Str.getStr ( STR_QUEST_DOCUMENT_INFO ),
                                            documentInfo ) ;
            ((FormComponent)component).setPreferredSize ( defaultFormSize ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Comment                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_COMMENT ) ) {
            if ( ( document != null ) && ( document instanceof DocumentDraft ) ) {
               JPanel           holdPanel          = new JPanel ( ) ;
               JPanel           scrollHoldPanel    = new JPanel () ;
               int              commentSize        = 1024 ;
               String           currentCommentStr  = null ;
               JTextAreaCounter commentCounter     = null ;

               try {
                  currentCommentStr = ((DocumentDraft)document).getComments().toString() ;
               } catch ( Exception e ) {
                  currentCommentStr= "" ;
               }
               commentSize-=currentCommentStr.length() ;

               currentCommentHistory = new NoTabJTextArea() ;
               currentComment        = new NoTabJTextArea() ;

               currentComment.setFont ( FontSystem.defaultFont ) ;
               currentCommentHistory.setFont ( FontSystem.defaultFont ) ;
               currentCommentHistory.setBackground ( new Color ( 240, 240, 240 ) );
               currentCommentHistory.setEditable ( false ) ;
               currentComment.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, commentSize ) ) ;
               commentCounter = new JTextAreaCounter ( currentComment ) ;

               holdPanel.setLayout ( new BorderLayout() ) ;
               scrollHoldPanel.setLayout ( new BorderLayout() ) ;
               scrollHoldPanel.setBackground ( Color.white ) ;
               holdPanel.add ( currentCommentHistory, BorderLayout.NORTH  ) ;
               holdPanel.add ( currentComment, BorderLayout.CENTER ) ;
               scrollHoldPanel.add ( new JScrollPane ( holdPanel ), BorderLayout.CENTER ) ;
               scrollHoldPanel.add ( commentCounter, BorderLayout.SOUTH ) ;

               component = new FormComponent ( Str.getStr ( STR_COMMENT ), scrollHoldPanel ) ;
               ((FormComponent)component).setPreferredSize ( new Dimension ( 470, 
                                                                             (GUISystem.getRowHeight()*5)+22 ) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Comment History                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_COMMENT_HISTORY ) ) {
            component = commentHistoryComponent =new MultiLineFormComponent ( Str.getStr ( STR_COMMENT_HISTORY ), "" ) ;
            ((MultiLineFormComponent)component).setPreferredSize ( new Dimension ( 470, 
                                                                                   (GUISystem.getRowHeight()*5)+22 ) ) ;
            commentHistoryComponent.getJTextArea().setEditable ( false ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Comment History                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_SEARCH_ENGINE_QUERY_INDEX ) ) {
            component = searchEngineQueryComponent =new MultiLineFormComponent ( Str.getStr ( STR_SEARCH_ENGINE_QUERY_INDEX ), "" ) ;
            ((MultiLineFormComponent)component).setPreferredSize ( new Dimension ( 470, 
                                                                                   (GUISystem.getRowHeight()*5)+22 ) ) ;

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Publishing Options                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_PUBLISHING_OPTIONS ) ) {
            publishingOptions = new PublishingOptionsPanel () ;
            component = new FormComponent ( Str.getStr ( STR_PUBLISHING_OPTIONS ),
                                            publishingOptions ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Links to this document                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_THIS_DOCUMENT_LINKS_TO ) ) {
            docLinksToPanel = new DocumentLinkPanel () ;
            component = new FormComponent ( Str.getStr ( STR_LINKS_FROM_THIS_DOCUMENT ),
                                            docLinksToPanel ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Links From this document                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_DOCUMENTS_LINK_TO_THIS ) ) {
            refDocLinksPanel = new DocumentLinkPanel () ;
            component = new FormComponent ( Str.getStr ( STR_LINKS_TO_THIS_DOCUMENT ),
                                            refDocLinksPanel ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   File Attachment                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_FILE_ATTACHMENTS ) ) {
            if ( ( document != null ) && 
                 ( ( document.getDocumentType() == TypeList.DOCTYPE_SERVICE_PUB ) ||
                   ( document.getDocumentType() == TypeList.DOCTYPE_SSD ) ) ) {
               fileAttachments = new FileAttachmentPanel () ;
               component = new FormComponent ( Str.getStr ( STR_FILE_ATTACHMENT ),
                                               fileAttachments ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   eMail Stuff                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_PROACTIVE_EMAIL ) ) {
            proMailPanel = new ProMailPanel () ;
            component    = new FormComponent ( Str.getStr ( STR_PROMAIL ),
                                               proMailPanel ) ;
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Locale                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_DOCUMENT_LOCATION ) ) {
            locale = new LocalePanel () ;
            component = new FormComponent ( Str.getStr ( STR_DOCUMENT_LOCATION ),
                                            locale ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Category                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_CATEGORY ) ) {
            category  = new CategoryPanel () ;
            component = new FormComponent ( Str.getStr ( STR_CATEGORY ),
                                            category ) ;
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Downloadable File Information                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( current.equals ( FORM_DOWNLOADABLE_FILE_INFO ) ) {
            if ( ( document != null ) && 
                 ( ( document.getDocumentType() == TypeList.DOCTYPE_SERVICE_PUB ) ||
                   ( document.getDocumentType() == TypeList.DOCTYPE_SSD ) ) ) {
               downloadablePanel = new DownloadableInfoPanel () ;
               component = new FormComponent ( Str.getStr ( STR_DOWNLOADABLE_FILE_INFO ), downloadablePanel ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the Component                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( component != null ) {

            if ( ( component instanceof FormComponent ) &&
                 ( component != titleComponent ) )
               GUISystem.setPropertiesOnFormComponent ( (FormComponent)component ) ;

            dataPanel.add ( component ) ;

            addComponentFocus ( component ) ;
         } else
            LogSystem.log ( 1, "Cannot add FormComponent:"+current ) ;


         i++ ;
      } /* while  */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Disable the Document Type Radio Buttons if this is an   */                              
      /* SSD or Service Pub                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( document != null ) {
         if ( ( document.getDocumentType() == TypeList.DOCTYPE_SSD ) ||
              ( document.getDocumentType() == TypeList.DOCTYPE_SERVICE_PUB ) ) {
            documentType.setEnabled ( false ) ;
         }

      }
   }



   /**
     *  This method will go through all children of the component
     * passed and will add us as a FocusListener. This way, if a control
     * that is not visible in our JScrollPane receives focus, we can scroll
     * to it.
     **/
   private void addComponentFocus ( Container component ) {
      Component[] compArray = component.getComponents() ;

      if ( compArray != null ) {

         for ( int i = 0; i < compArray.length; i++ ) {
            if ( ( compArray[i] instanceof JCheckBox ) ||
                 ( compArray[i] instanceof JRadioButton ) ||
                 ( compArray[i] instanceof JList ) ||
                 ( compArray[i] instanceof JTextArea ) ||
                 ( compArray[i] instanceof JTextField ) ||
                 ( compArray[i] instanceof JButton ) ) {
               compArray[i].addFocusListener ( this ) ;
               focusGUI.addElement ( compArray[i] ) ;
            }

            else if ( compArray[i] instanceof Container )
               addComponentFocus ( (Container)compArray[i] ) ;

         }
      }

   }


   /**
     *  This method will be called when a component we have has gained focus
     **/
   public void focusGained ( FocusEvent e ) {
      boolean   c         = true ;
      Component component = (Component)e.getSource() ;
      Rectangle bounds    = component.getBounds() ;
      Rectangle viewRect  = scrollPane.getViewport().getViewRect() ;                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we have to figure out what position this           */
      /* control is in relative to the Data Panel                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( c ) {
         Component parent = component.getParent() ;
         Rectangle parentBounds = parent.getBounds() ;

         c = ( parent != dataPanel ) ;

         if ( c ) {
            bounds.y+=parentBounds.y ;
            component = parent ;
         }
      }

      if ( !viewRect.contains ( bounds.x, bounds.y ) )
         scrollPane.getViewport().setViewPosition ( new Point ( bounds.x, bounds.y ) ) ;


   }

   /**
     *  This method will be notified when any field we are displaying loses focus. We will
     * use this time to save data that appears in non-standard form components.
     **/
   public void focusLost ( FocusEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The title of the document                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == titleComponent.getJTextArea() )
         document.setTitle ( titleComponent.getText( ) ) ;

      /*-------------------------------*/
      /*                               */
      /*  The Search Engine Query Text */
      /*                               */
      /*-------------------------------*/
      else if ( e.getSource() == searchEngineQueryComponent.getJTextArea() ) {
         DocumentQueryText queryText = document.getQueryText() ;

         if ( queryText == null )
            queryText = new DocumentQueryText ( searchEngineQueryComponent.getText() ) ;
         else
            queryText.setQueryText ( searchEngineQueryComponent.getText() ) ;

         document.setQueryText ( queryText ) ;
      }
   }



   /**
     *  This method will be called when any property on the document has changed. We will use
     * this to update any non-standard form components, like the title
     **/
   public void documentChanged ( Document document, int property ) {
      switch ( property ) {
         case Document.TITLE :
            titleComponent.setText ( document.getTitle() ) ;
            break ;

         case Document.TYPE :
            documentType.setSelectedItem ( TypeList.getInstance().objectFromInd ( document.getDocumentType(), TypeList.TYPE_DOC ) ) ;
            break ;

         case Document.CLASS :
            documentClass.setSelectedItem ( TypeList.getInstance().objectFromInd ( document.getDocumentClass(), TypeList.TYPE_DOCCLASS ) ) ;
            break ;

         case Document.LINKS :
            docLinksToPanel.setDocumentLinks ( document.getDocumentLinks() ) ;
            break ;

         case Document.REFERRING_LINKS :
            refDocLinksPanel.setDocumentLinks ( document.getReferringDocumentLinks( )) ;
            break ;


         case Document.ADDITIONAL_INFO :
         case Document.BODY :
            new Thread ( this, THREAD_SET_DOC_BODY ).start() ;
            break ;
      }
   }


   /**
     *  This method will remove us as a FocusListener from everything.
     **/
   private void removeAllFocusListeners () {
      int i    = 0 ;
      int size = focusGUI.size() ;

      while ( i < size ) {
         ((Component)focusGUI.elementAt(i)).removeFocusListener ( this ) ;
         i++ ;
      }

      if ( documentClass != null )
         documentClass.removeActionListener ( this ) ;
      if ( documentType != null )
         documentType.removeActionListener  ( this ) ;
      focusGUI.removeAllElements() ;
   }



   /**
     * This method will start the saving process. It will not return until the save is complete or the
     * user cancels the save. 
     *
     * @return One of the return codes defined in the SaveDocDlg class.
     **/
   public int save () {
      boolean saved = false ;
      boolean cont  = false ;                      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check a couple of things first.                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      /* Status  */
      if ( document.getStatus() != null ) {
         DocumentStatus status = document.getStatus() ;

         if ( CDate.checkDate ( status.getStatusDate(), CDate.DATE_DATE ) != 0 ) {
            GUISystem.printBox ( "Information", "Please enter a valid status and status date" ) ;
         } else {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If The status is Closed, Not Approved, to the user      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( status.getStatus() == Status.TYPESTATUS_CLOSEDNOTAPPROVED ) {
               cont = GUISystem.printBox ( STR_QUESTION, STR_STATUS_IS_CLOSED_NOT_APPROVED ) ;
            } else
               cont= true ;
         }
      } else
         GUISystem.printBox ( "Information", "Please enter a valid status and status date" ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Continue                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( cont ) {

         /*---------------------------------------------------------*/
         /*                                                         */
         /*  First, save all of our information that might not have */
         /* been saved yet.                                         */
         /*                                                         */
         /*---------------------------------------------------------*/
         ((DocumentDraft)document).setComments ( currentComment.getText() ) ;


         /*------------------------------------------*/
         /*                                          */
         /*  Now, bring up the 'SaveDocument' window */
         /*                                          */
         /*------------------------------------------*/
         SaveDocDlg saveDocDlg = new SaveDocDlg ( GUISystem.getParentDefWin ( this ), 
                                                  ((DocumentDraft)document),
                                                  currentComment.getText() ) ;


         return( saveDocDlg.getReturnCode() ) ;

      }

      return( 0 ) ;
   }


   /**
     *  This method will be called when an action event is fired on us. 
     **/
   public void actionPerformed ( ActionEvent e ) {


      /*-------------------------------*/
      /*                               */
      /*  A Document Type Radio Button */
      /*                               */
      /*-------------------------------*/
      if ( e.getSource() == documentType ) {
         int currentType = document.getDocumentType() ;                                    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Be very careful if we have to switch the type because   */
         /* if we do, we must remove everything and re add everything */
         /* since different document types have different fields      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( currentType != ((TypeDocRec)documentType.getSelectedItem()).getInd() ) {
            document.setDocumentType ( ((TypeDocRec)documentType.getSelectedItem()).getInd() ) ;
            document.writeToMemory() ;
            setDocument ( document ) ;
         }

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the classification                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == documentClass ) {
         document.setDocumentClass ( ((TypeDocClassRec)documentClass.getSelectedItem()).getInd() ) ;
      }


      /*---------------------------*/
      /*                           */
      /*  Show the document fields */
      /*                           */
      /*---------------------------*/
      else if ( e.getSource() == pb_FIELDS ) {
         ((CardLayout)fieldBodyPanel.getLayout()).show ( fieldBodyPanel, "FIELDS" ) ;
         revalidate() ;
      }


      /*----------------*/
      /*                */
      /*  Show the body */
      /*                */
      /*----------------*/
      else if ( e.getSource() == pb_BODY ) {
         ((CardLayout)fieldBodyPanel.getLayout()).show ( fieldBodyPanel, "BODY" ) ;
         new Thread ( this, THREAD_SET_DOC_BODY ).start() ;
      }
   } /* actionPerformed  */




   /**
     *  This method will be called when a property we are listening for has changed
     *
     **/
   public void propertyChanged ( int propertyInd, Object data ) {
      setDocument ( document ) ;
   }


   /**
     *  This method must be called whenever this component is removed from another
     **/
   public void free () {
      PropertySystem.removePropertyListener ( PROPERTY_EDITOR_LAYOUT, this ) ;
   }



   /**
     * This method will run our threads. The operation that is done
     * depends on the name of the thread
     **/
   public void run () {
      String operation = Thread.currentThread().getName() ;        

      /*------------------------*/
      /*                        */
      /*  Set the document body */
      /*                        */
      /*------------------------*/
      if ( operation.equals ( THREAD_SET_DOC_BODY ) )
         refreshDocumentBody() ;

   }


   /**
     * This method will populate the browser control with the document body
     **/
   public void refreshDocumentBody() {
      String content = null ;

      if ( pb_BODY.isSelected() ) {
         if ( document.getDocumentType() == TypeList.DOCTYPE_SSD )
            content = document.getAdditionalInfo() ;
         else {
            DocumentBody body = document.getDocumentBody() ;

            if ( body != null )
               content = body.getBody() ;
         }

         if ( ( content == null ) || ( content.length() == 0 ) )
            content = "<html><body>No Body</body></html>" ;

         browser.setPageContent ( content ) ;
      }

   }


   /**
     *  This method will enable/disable the entire editor.
     **/
   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;
      this.enableInput = enable ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, tell all of our components                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         titleComponent.getJTextArea().setEditable ( enable ) ;
         productComponent.setEnabled ( enable ) ;
         proMailPanel.setEnabled ( enable ) ;
         if ( fileAttachments != null )
            fileAttachments.setEnabled ( enable ) ;
         currentComment.setEditable ( enable ) ;
         documentClass.setEnabled ( enable ) ;
         documentType.setEnabled ( enable ) ;
         documentInfo.setEnabled ( enable ) ;
         searchEngineQueryComponent.getJTextArea().setEditable ( enable ) ;
         category.setEnabled ( enable ) ;
         publishingOptions.setEnabled ( enable ) ;
         locale.setEnabled ( enable ) ;
         draftInfo.setEnabled ( enable ) ;
         if ( downloadablePanel != null )
            downloadablePanel.setEnabled ( enable ) ;

         if ( document != null ) {
            if ( ( document.getDocumentType() == TypeList.DOCTYPE_SSD ) ||
                 ( document.getDocumentType() == TypeList.DOCTYPE_SERVICE_PUB ) )
               documentType.setEnabled ( false ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }


   public JPanel getContentPanel () {
      return( dataPanel );
   }



   /**
     * This method will start the spell checker by analyzing the title of the document.
     **/
   protected void spellCheck ( DocumentEditor documentEditor, SpellPanel spellPanel ) {

      this.spellPanel     = spellPanel ;
      this.documentEditor = documentEditor;
      spellCheckStep = SPELL_TITLE ;
      spellPanel.addSpellPanelListener ( this ) ;
      spellPanel.spellCheck ( titleComponent.getText() ) ;
   }


   /*------------------------------------------------------------*/
   /* spellCheckComplete
    */
   /**
    *    This method will be called when a spell
    *  check has completed.
    *  
    *
    *  @param foundErrors TRUE if any errors were found and corrected, FALSE no errors found
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void spellCheckComplete ( boolean foundErrors ) {
      int docTypeInd = document.getDocumentType() ;

      System.out.println ( "spellCheckComplete ( "+foundErrors+" )" ) ;


      spellCheckStep++ ;
      System.out.println ( "New Spell Step "+spellCheckStep ) ;

      switch ( spellCheckStep ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Spell Check THe title                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         case SPELL_TITLE :
            spellPanel.spellCheck ( titleComponent.getText() ) ;
            break ;                 

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Spell Check the eMail body                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
         case SPELL_EMAIL :
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Only spell check if the document is eMailable           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( document.getEMailable() ) {
               spellPanel.spellCheck ( proMailPanel.getJTextArea().getText() ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, go to the next Spell check item              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else
               spellCheckComplete ( foundErrors ) ;
            break ;

         case SPELL_DESCRIPTION :

            if ( ( docTypeInd == TypeList.DOCTYPE_SERVICE_PUB ) ||
                 ( docTypeInd == TypeList.DOCTYPE_SSD ) ) {
               spellPanel.spellCheck ( downloadablePanel.getJTextArea().getText() ) ;
            } else
               spellCheckComplete ( foundErrors ) ;
            break ;

         case SPELL_VERSION :

            if ( ( docTypeInd == TypeList.DOCTYPE_SERVICE_PUB ) ||
                 ( docTypeInd == TypeList.DOCTYPE_SSD ) ) {
               spellPanel.spellCheck ( downloadablePanel.getVersionJTextField().getText() ) ;
            } else
               spellCheckComplete ( foundErrors ) ;
            break ;


         case SPELL_LAST :
            GUISystem.printBox ( "Info", "Spell Check Complete" ) ;
            spellPanel.removeSpellPanelListener ( this ) ;
            spellPanel = null ;
            documentEditor.closeSpellPanel() ;
            break ;
      }
   }




   /*------------------------------------------------------------*/
   /* spellCheckWord
    */
   /**
    *    This method will be called when a misspelled
    *  word has been encountered
    *  
    *
    *  @param word The word that is misspelled
    *  @param startPos The position in the text passed into the SpellPanel of this word
    *  @param endPos The position in the text passed into the SpellPanel of the end of this word
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void spellCheckWord ( String word, int startPos, int endPos ) {

      switch ( spellCheckStep ) {
         case SPELL_TITLE :
            titleComponent.getJTextArea().setCaretPosition ( startPos ) ;
            titleComponent.getJTextArea().moveCaretPosition  ( endPos ) ;
            titleComponent.getJTextArea().requestFocus() ;
            break ;

         case SPELL_EMAIL :
            JTextArea textArea = proMailPanel.getJTextArea() ;

            textArea.setCaretPosition ( startPos ) ;
            textArea.moveCaretPosition  ( endPos ) ;
            textArea.requestFocus() ;
            break ;

         case SPELL_DESCRIPTION :
            JTextArea descriptionArea = downloadablePanel.getJTextArea() ;

            descriptionArea.setCaretPosition ( startPos ) ;
            descriptionArea.moveCaretPosition  ( endPos ) ;
            descriptionArea.requestFocus() ;
            break ;

         case SPELL_VERSION :
            JTextField versionField = downloadablePanel.getVersionJTextField() ;

            versionField.setCaretPosition ( startPos ) ;
            versionField.moveCaretPosition  ( endPos ) ;
            versionField.requestFocus() ;
            break ;
      }
   }





   /*------------------------------------------------------------*/
   /* replaceWord
    */
   /**
    *    This method will be called when the user has
    *  chosen to replace a word in the text that was
    *  passed to the SpellPanel
    *
    *  @param oldWord The old, incorrect word
    *  @param newWord The new, corrected word
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void replaceWord ( String oldWord, String newWord, int startPos ) {

      switch ( spellCheckStep ) {
         case SPELL_TITLE :
            String title = titleComponent.getJTextArea().getText() ;
            title = Text.replaceAllStrInStr ( title, oldWord, newWord ) ;
            titleComponent.getJTextArea().setText ( title ) ;
            document.setTitle ( title ) ;
            break ;

         case SPELL_EMAIL :
            String body = proMailPanel.getJTextArea().getText() ;
            body = Text.replaceAllStrInStr ( body, oldWord, newWord ) ;
            proMailPanel.getJTextArea().setText ( body ) ;
            document.setEMailComments ( body) ;
            break ;

      }
   }


   /**
     * This method will return TRUE if the body of the document is currently showing
     * instead of the Document Forms
     **/
   protected boolean isBodyShowing() {
      return( pb_BODY.isSelected() ) ;
   }


   /**
     * This method will return the filename of the body that was saved temporarily
     **/
   protected String getTempBodyFilename () {
      String filename = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+
                        File.separator+
                        "1_"+System.currentTimeMillis()+"_temp.html" ;
      String content  = null ;

      if ( document.getDocumentType() == TypeList.DOCTYPE_SSD )
         content = document.getAdditionalInfo() ;
      else {
         DocumentBody body = document.getDocumentBody() ;

         if ( body != null )
            content = body.getBody() ;
      }

      if ( ( content == null ) || ( content.length() == 0 ) )
         content = "<html><body>No Body</body></html>" ;

      FileUtil.saveFile ( filename, content ) ;

      return( filename ) ;

   }

} /* DocEditPanel */
