package com.ibm.nzna.projects.qit.doc.docedit ;  
/*-----------------------------------------------------------*/
/*   DocLinkDlg                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocLinkDlg                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog can display a vector of documents. Each document
 * displayed is being displayed as a 'linked' document. The user
 * can then hit the 'Edit' button or the 'Cancel' button.
 *
 *
 * @version 1.0, 11/21/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocLinkDlg                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocLinkDlg extends JDialog implements ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Container Titles                                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] listTitle = { 
      " ",
      Str.getStr ( STR_DOC_IND ),
      Str.getStr ( STR_TITLE )} ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_READ_REFRESH      = "READ_REF" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_DOCUMENT_LINKS = null ;
   private MultiList    documentList      = null ;
   private JLabel       st_IMAGE          = null ;
   private DButton      pb_EDIT           = null ;
   private DButton      pb_CANCEL         = null ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ImageIcon             wizardImage      = null ;
   private Vector                docLinks         = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the
    *  window and show it. It will then not return until the
    *  user has closed the window.
    *
    *  @param parent The parent AppDefaultWin
    *  @param answerConcl The ConclusionTerm to edit
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocLinkDlg ( AppDefaultWin parent, Vector docLinks ) {
      super ( parent, Str.getStr ( STR_DOCUMENT_LINKS ), false ) ;
      this.docLinks = docLinks ;

      initialize() ;
   } /* Constructor */



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign Local Vars                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardImage = ImageSystem.getImageIcon ( this, ImageSystem.CONCLUSION_WIZARD ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI components                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_IMAGE          = new JLabel ( wizardImage ) ;
      st_DOCUMENT_LINKS = new JLabel ( Str.getStr ( STR_DOCUMENT_LINKS ) ) ;
      documentList      = new MultiList ( GUISystem.getFontUtil() ) ;
      pb_EDIT           = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_CANCEL         = new DButton ( Str.getStr ( STR_CANCEL ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_EDIT ) ;
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;
      GUISystem.setPropertiesOnPanel ( contentPane );    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CANCEL.addActionListener   ( this ) ;
      pb_EDIT.addActionListener     ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI to the content panel                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_DOCUMENT_LINKS ) ;
      contentPane.add ( documentList ) ;
      contentPane.add ( pb_EDIT ) ;
      contentPane.add ( pb_CANCEL ) ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */        
      /*                                                           */
      /*-----------------------------------------------------------*/
      documentList.setForeground      ( Color.black ) ;
      documentList.setColumnHeadings  ( listTitle ) ;
      documentList.showColumnHeadings ( true ) ;
      documentList.setColumnWidth     ( 0, 20 ) ;
      documentList.setColumnWidth     ( 1, 75  ) ;
      documentList.setColumnWidth     ( 2, 300 ) ;
      documentList.setColumnWidth     ( 3, 100 ) ;
      documentList.setColumnWidth     ( 4, 200 ) ;

      

      documentList.add ( docLinks ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Center our window                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 495, 325 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;

   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an ActionEvent
    *  is fired on us
    *  
    *
    *  @param e The ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL )
         dispose () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT ) {
         //VOYDocLinkRec docLinkRec = (DocLinkRec)documentList.getSelectedItem() ;

         //VOYif ( docLinkRec != null ) {
         //VOY   DocRec docRec = new DocRec ( ) ;
         //VOY   
         //VOY   docRec.setExistingDocInd ( docLinkRec.getDocInd() ) ;

         //VOY   docRec.edit() ;
         //VOY   dispose() ;
         //VOY}
      } /* else */
 
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout (){
      Dimension size      = getSize() ;
      int       x         = wizardImage.getImage().getWidth(this)+10 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2), 
                           wizardImage.getImage().getWidth(this), 
                           wizardImage.getImage().getHeight(this)) ;



      st_DOCUMENT_LINKS.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      documentList.setBounds ( x, y, size.width-(x)-10, size.height-100 ) ;

      pb_EDIT.setLocation ( x, size.height-75 ) ;
      pb_CANCEL.setLocation ( x+pb_EDIT.getMinimumSize().width+5, size.height-75 ) ;

   }


} /* DocLinkDlg */
