package com.ibm.nzna.projects.qit.doc.docedit ;    
/*-----------------------------------------------------------*/
/*   DocSaveCheck                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocSaveCheck                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.JDialog ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.doc.DocConst ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class checks to see if a document is Ok to save or publish. 
 *
 *
 * @version 1.0, 8/21/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocSaveCheck                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocSaveCheck extends Object implements DocConst, AppConst {


   /**
     *  This method will check to make sure that a document is ok
     * to save as a draft.
     *
     * @param draft The draft to check
     * @param parent The JDialog that is the parent of any error message
     * @param showErrors if TRUE, a message box shows errors/missing fields
     *
     * @return TRUE if the draft was ok
     **/
   public static boolean checkDraft ( DocumentDraft draft, JDialog parent, boolean showErrors ) {
      boolean rc = false ;

      /*---------------------------------------------------*/
      /*                                                   */
      /*  Do not check information for a publish of delete */
      /*                                                   */
      /*---------------------------------------------------*/
      if ( draft.getWorkRequired() != TypeList.WORKREQ_DELETE ) {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Title                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( draft.getTitle() == null ) || ( draft.getTitle().length() == 0 ) ) {
            if ( showErrors )
               GUISystem.printBox ( parent, STR_INFO, STR_PLEASE_ENTER_TITLE ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   At least 1 category                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( draft.getCategory() == null ) || ( draft.getCategory().size() == 0 ) ) {
            if ( showErrors )
               GUISystem.printBox ( parent, STR_INFO, STR_MUST_SELECT_CATEGORY ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   At least 1 machine. This check should always be last    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            int bftmCount = 0  ;

            if ( draft.getBrands() != null )
               bftmCount+=draft.getBrands().size() ;
            if ( draft.getFamilies() != null )
               bftmCount+=draft.getFamilies().size() ;
            if ( draft.getMachines() != null )
               bftmCount+=draft.getMachines().size() ;
            if ( draft.getModels() != null )
               bftmCount+=draft.getModels().size() ;

            if ( bftmCount == 0 ) {
               if ( showErrors )
                  GUISystem.printBox ( parent, STR_INFO, STR_MUST_SELECT_BFTM ) ;
            } else
               rc = true ;
         }
      } else rc = true ; // Always allow the delete

      return( rc )  ;
   }


   public static boolean checkDraft ( DocumentDraft draft, JDialog parent ) {
      return( checkDraft ( draft, parent, true ) ) ;
   }

   public static boolean checkPublish ( DocumentDraft draft, JDialog parent ) {
      return( checkPublish ( draft, parent, true ) ) ;
   }

   /**
     *  This method will check to see if a document can be published.
     *
     * @param DocumentDraft draft The Draft to check
     * @param parent The JDialog that is the parent of any error message
     * @param showErrors if TRUE, a message box shows errors/missing fields
     *
     * @return TRUE if Ok, FALSE If not Ok
     **/
   public static boolean checkPublish ( DocumentDraft draft, JDialog parent, boolean showErrors ) {
      boolean rc = false ;


      /*---------------------------------------------------*/
      /*                                                   */
      /*  Do not check information for a publish of delete */
      /*                                                   */
      /*---------------------------------------------------*/
      if ( draft.getWorkRequired() != TypeList.WORKREQ_DELETE ) {

         rc = checkDraft ( draft, parent, showErrors ) ; // This takes care of checking Title, Category and BFTM

         if ( rc ) {

            rc = false ; // Now, assume it was bad


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Locale                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( draft.getLocale() == null ) ||
                 ( ( draft.getLocale().countryCount() == 0 ) &&
                   ( draft.getLocale().geoCount() == 0 ) ) ) {
               if ( showErrors )
                  GUISystem.printBox ( parent, STR_INFO, STR_MUST_SELECT_ONE_COUNTRY ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Check Knowledge Base and delete                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( ( draft.getWorkRequired() == TypeList.WORKREQ_DELETE ) &&
                      ( draft.getKnowledgeBase() ) ) {
               if ( showErrors )
                  GUISystem.printBox ( parent, STR_INFO, STR_CANNOT_DELETE_OA_DOC ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Check if Delete and has links pointing to this doc      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( ( draft.getWorkRequired() == TypeList.WORKREQ_DELETE ) &&
                      ( draft.getReferringDocumentLinks() != null ) &&
                      ( draft.getReferringDocumentLinks().size() > 0 ) ) {
               if ( showErrors )
                  GUISystem.printBox ( parent, STR_INFO, STR_CANNOT_DELETE_WITH_REFERRING_DOC_LINKS ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Body                                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( ( draft.getDocumentType() != TypeList.DOCTYPE_SSD ) &&
                      ( ( draft.getDocumentBody() == null ) ||
                        ( draft.getDocumentBody().getBody().length() == 0 ) ) ) {
               if ( showErrors )
                  GUISystem.printBox ( parent, STR_INFO, STR_MUST_ENTER_BODY ) ;
            } else
               rc = true ;                                     


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, check specific document type restrictions          */
            /*                                                           */
            /*-----------------------------------------------------------*/

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   SSDs and Service pubs                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( ( draft.getDocumentType() == TypeList.DOCTYPE_SSD ) ||
                   ( draft.getDocumentType() == TypeList.DOCTYPE_SERVICE_PUB ) ) ) {

               if ( rc ) {
                  rc = false ; // Always set back to false

                  if ( draft.getDocumentType() == TypeList.DOCTYPE_SSD ) {


                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   ADditional Info                                         */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( ( draft.getAdditionalInfo() == null ) ||
                          ( draft.getAdditionalInfo().length() == 0 ) ) {
                        if ( showErrors )
                           GUISystem.printBox ( parent, STR_INFO, STR_PLEASE_ENTER_ADD_INFO_CUSTOM_VIEW ) ;
                     }

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Description                                             */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     else if ( ( draft.getDescription() == null ) ||
                               ( draft.getDescription().length() == 0 ) ) {
                        if ( showErrors )
                           GUISystem.printBox ( parent, STR_INFO, STR_MUST_ENTER_DESCRIPTION ) ;
                     }

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Version                                                 */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     else if ( ( draft.getVersion() == null ) ||
                               ( draft.getVersion().length() == 0 ) ) {
                        if ( showErrors )
                           GUISystem.printBox ( parent, STR_INFO, STR_MUST_ENTER_VERSION ) ;
                     }


                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Part Number                                             */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     else if ( ( draft.getPartNumber() == null ) ||
                               ( draft.getPartNumber().length() == 0 ) ) {
                        if ( showErrors )
                           GUISystem.printBox ( parent, STR_INFO, STR_MUST_ENTER_PART_NUMBER ) ;
                     }

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   URL is required only if there are no files attached     */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     else if ( ( ( draft.getFileAttachments() == null ) ||
                                 ( QuestUtil.getValidData ( draft.getFileAttachments() ).size() == 0 ) ) &&
                               ( ( draft.getURL() == null ) ||
                                 ( draft.getURL().length() == 0 ) ) ) {
                        if ( showErrors )
                           GUISystem.printBox ( parent, STR_INFO, STR_MUST_ENTER_URL ) ;
                     } else
                        rc = true ; // Ok, still good
                  }



                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Service Publication                                     */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  else if ( draft.getDocumentType() == TypeList.DOCTYPE_SERVICE_PUB ) {

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Publication Type                                        */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( draft.getPublicationType() == 0 ) {
                        if ( showErrors )
                           GUISystem.printBox ( parent, STR_INFO, STR_MUST_SELECT_PUBLICATION_TYPE ) ;
                     }


                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Description                                             */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     else if ( ( draft.getDescription() == null ) ||
                               ( draft.getDescription().length() == 0 ) ) {
                        if ( showErrors )
                           GUISystem.printBox ( parent, STR_INFO, STR_MUST_ENTER_DESCRIPTION ) ;
                     }

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   URL is required only if there are no files attached     */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     else if ( ( ( draft.getFileAttachments() == null ) ||
                                 ( QuestUtil.getValidData ( draft.getFileAttachments() ).size() == 0 ) ) &&
                               ( ( draft.getURL() == null ) ||
                                 ( draft.getURL().length() == 0 ) ) ) {
                        if ( showErrors )
                           GUISystem.printBox ( parent, STR_INFO, STR_MUST_ENTER_URL ) ;
                     } else
                        rc = true ; // Ok, still good

                  }
               }
            }
         }
      } else rc = true ; // Always allow a delete


      return( rc ) ;
   }
} /* DocSaveCheck */
