package com.ibm.nzna.projects.qit.doc.docedit ;            
/*-----------------------------------------------------------*/
/*   DocSaveStatusPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocSaveStatusPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;     
/**
 * This panel holds the status of a document saving 
 *
 *
 * @version 1.0, 8/22/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocSaveStatusPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocSaveStatusPanel extends JPanel implements AppConst, DocConst {


   /**
     *  This is the Label for saving graphics
     **/
   private JLabel st_GRAPHICS = new JLabel ( Str.getStr ( STR_SAVING_GRAPHICS ) ) ;

   /**
     *  This is the Label for saving File Attachments
     **/
   private JLabel st_FILES = new JLabel ( Str.getStr ( STR_SAVING_FILES ) ) ;

   /**
     *  This is the Label for saving the document
     **/
   private JLabel st_DOCUMENT = new JLabel ( Str.getStr ( STR_SAVING_DOCUMENT ) );

   /**
     *  This is the Label for chesting in
     **/
   private JLabel st_CLEANING = new JLabel (  ) ;

   /**
     *  This is the Check Box for saving graphics
     **/
   private JCheckBox ck_GRAPHICS = new JCheckBox (  ) ;

   /**
     *  This is the Check Box for saving File Attachments
     **/
   private JCheckBox ck_FILES = new JCheckBox (  ) ;

   /**
     *  This is the Check Box for saving the document
     **/
   private JCheckBox ck_DOCUMENT = new JCheckBox (  );

   /**
     *  This is the Check Box for checking in the document
     **/
   private JCheckBox ck_CLEANING = new JCheckBox (  );

   /**
     *  This is the status area
     **/
   private JLabel st_OUTPUT = new JLabel () ;


   /**
     *  Our title
     **/
   private JLabel st_TITLE = new JLabel ( Str.getStr ( STR_SAVING )+" "+Str.getStr ( STR_DOCUMENT ) );
   /**
     *  This property tells us if we are showing the FILES check box
     **/
   private boolean showFiles = false ;


   /**
     *  Default constructor
     **/
   public DocSaveStatusPanel ( DocumentDraft draft ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ck_GRAPHICS.setEnabled ( false ) ;
      ck_FILES.setEnabled ( false ) ;
      ck_DOCUMENT.setEnabled ( false ) ;
      ck_CLEANING.setEnabled ( false ) ;  
      st_GRAPHICS.setEnabled ( false ) ;
      st_FILES.setEnabled ( false ) ;
      st_DOCUMENT.setEnabled ( false ) ;
      st_CLEANING.setEnabled ( false ) ;  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( st_TITLE ) ;
      add ( st_GRAPHICS ) ;
      add ( ck_GRAPHICS ) ;

      if ( ( draft.getDocumentType() == TypeList.DOCTYPE_SSD ) ||
           ( draft.getDocumentType() == TypeList.DOCTYPE_SERVICE_PUB ) ) {
         add ( ck_FILES ) ;
         add ( st_FILES ) ;
         showFiles = true ;
      }

      add ( st_DOCUMENT ) ;
      add ( st_CLEANING ) ;
      add ( st_OUTPUT ) ;
      add ( ck_DOCUMENT ) ;
      add ( ck_CLEANING ) ;
      add ( st_CLEANING ) ;
      add ( st_OUTPUT ) ;


   }



   /**
     *  This method lays out our GUI
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 15 ;
      int       rowHeight = GUISystem.getRowHeight () ;

      super.doLayout() ;

      st_TITLE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      ck_GRAPHICS.setBounds ( x, y, 20, rowHeight ) ;
      st_GRAPHICS.setBounds ( x+20, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      if ( showFiles ) {
         ck_FILES.setBounds ( x, y, 20, rowHeight ) ;
         st_FILES.setBounds ( x+20, y, size.width-(x*2), rowHeight ) ;
         y+=rowHeight ;
      }

      ck_DOCUMENT.setBounds ( x, y, 20, rowHeight ) ;
      st_DOCUMENT.setBounds ( x+20, y, size.width-(x*2), rowHeight ) ;

      y+=rowHeight ;


      ck_CLEANING.setBounds ( x, y, 20, rowHeight ) ;
      st_CLEANING.setBounds ( x+20, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;


      st_OUTPUT.setBounds ( x, size.height-(rowHeight*2), size.width-(x*2), rowHeight*2 ) ;

   }



   /**
     *  This method will enable/disable the Files label. If it is
     * enabled, all other controls will be disabled
     **/
   public void enableFilesLabel ( boolean enable ) {
      st_FILES.setEnabled ( enable ) ;
      if ( enable ) {
         st_DOCUMENT.setEnabled ( !enable ) ;
         st_GRAPHICS.setEnabled ( !enable ) ;
         st_CLEANING.setEnabled ( !enable ) ;
      }
   }



   /**
     *  This method will enable/disable the Documents label. If it is
     * enabled, all other controls will be disabled
     **/
   public void enableDocumentsLabel ( boolean enable ) {
      st_DOCUMENT.setEnabled ( enable ) ;
      if ( enable ) {
         st_GRAPHICS.setEnabled ( !enable ) ;
         st_FILES.setEnabled ( !enable ) ;
         st_CLEANING.setEnabled ( !enable ) ;
      }
   }



   /**
     *  This method will enable/disable the Cleaning label. If it is
     * enabled, all other controls will be disabled
     **/
   public void enableCleaningLabel ( boolean enable ) {
      st_CLEANING.setEnabled ( enable ) ;
      if ( enable ) {
         st_DOCUMENT.setEnabled ( !enable ) ;
         st_FILES.setEnabled ( !enable ) ;
         st_GRAPHICS.setEnabled ( !enable ) ;
      }
   }

   /**
     *  This method will enable/disable the Graphics label. If it is
     * enabled, all other controls will be disabled
     **/
   public void enableGraphicsLabel ( boolean enable ) {
      st_GRAPHICS.setEnabled ( enable ) ;
      if ( enable ) {
         st_DOCUMENT.setEnabled ( !enable ) ;
         st_FILES.setEnabled ( !enable ) ;
         st_CLEANING.setEnabled ( !enable ) ;
      }
   }




   /**
     *  this method will set the selected state of the
     * check box for GRAPHICS
     **/
   public void setGraphicsSelected ( boolean state ) {
      ck_GRAPHICS.setSelected ( state ) ;
   }


   /**
     *  this method will set the selected state of the
     * check box for FILES
     **/
   public void setFilesSelected ( boolean state ) {
      ck_FILES.setSelected ( state ) ;
   }

   /**
     *  this method will set the selected state of the
     * check box for DOCUMENT
     **/
   public void setDocumentsSelected ( boolean state ) {
      ck_DOCUMENT.setSelected ( state ) ;
   }

   /**
     *  this method will set the selected state of the
     * check box for CLEANING
     **/
   public void setCleaningSelected ( boolean state ) {
      ck_CLEANING.setSelected ( state ) ;
   }


   /**
     *  This method will set the status that is shown
     **/
   public void setText ( String status ) {
      st_OUTPUT.setText ( status ) ;
   }
} /* DocSaveStatusPanel */
