package com.ibm.nzna.projects.qit.doc.docedit ;       
/*-----------------------------------------------------------*/
/*   DocumentEditor                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentEditor                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.spell.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.docmaint.* ;
import com.ibm.nzna.projects.qit.doc.history.* ;
import com.ibm.nzna.projects.qit.doc.docedit.property.* ;
import com.ibm.nzna.projects.qit.wordproc.* ;
import com.ibm.nzna.projects.qit.notify.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * The Document Editor is a full QuestPanel that edits a DocumentDraft and
 * allows the saving of the draft. This panel basically holds a DocEditPanel
 * because the DocEditPanel does all the work.
 *
 *
 * @version 1.0, 4/25/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentEditor                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentEditor extends JPanel implements SpellPanelListener, PreviewPanel, PrintPanel, ExtBodyEditProcessListener, Runnable, DocumentPropertyListener, DocConst, ActionListener, QuestPanel, AppConst {


   /**
     * This thread will set our edit panel and it's data
     **/
   private static final String THREAD_SET_EDIT = "SETEDIT" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     * This property defines if we have set the document editor focus button yet
     **/
   private boolean focusButtonSet = false ;

   /**
     *  The DocEditPanel that this panel uses to edit a document draft
     * with.
     **/
   private DocEditPanel editPanel = null ;

   /**
     *  The DocumentDraft object we are editing
     **/
   private DocumentDraft draft = null ;


   /**
     * This is the Save button
     **/
   private ActionButton pb_SAVE = null ;

   /**
     * This is the close button
     **/
   private ActionButton pb_CLOSE = null ;

   /**
     *  This is the Send button
     **/
   private ActionButton pb_SEND = null ;

   /**
     * This is the External Body Edit button
     **/
   private ActionButton pb_EXT_BODY_EDIT = null ;

   /**
     * This is the Internal Body Edit button
     **/
   private ActionButton pb_INT_BODY_EDIT = null ;

   /**
     *  This property is only true when the editor is actually trying to close
     **/
   private boolean editorClosing = false ;

   /**
     * This is the AppDefaultWin this panel belongs to
     **/
   private AppDefaultWin parentWin = null ;


   /**
     *  If TRUE, the data in the document has changed
     **/
   private boolean documentChanged = false ;

   /**
     * This is the menu item to spell check
     **/
   private JMenuItem mnu_SPELLCHECK = new JMenuItem ( "Spell Check" ) ;

   /**
     *  This is the toolbar button for the document history
     **/
   private   ToolBarButton   pb_HISTORY           = null ;

   /**
     * If we are spell checking, this is the spell panel
     **/
   private SpellPanel spellPanel = null ;

   /**
     *  This is the toolbar button for the document undo
     **/
   private ToolBarButton pb_UNDO = null ;

   /**
     *  This is the toolbar that our panel uses
     **/
   private   JToolBar        toolBar              = null ;

   /**
     *  This is the property toolbar button
     **/
   private   ToolBarButton   pb_PROPERTY = null ;

   /**
     * This object holds the process for the 3rd party word processor
     **/
   private ExtBodyEditProcess extBodyEditProcess = null ;

















   /**
     *  This constructor creates a Document Editor for the document draft passed in.
     **/
   public DocumentEditor ( DocumentDraft draft ) {
      this.draft = draft ;
      draft.addDocumentPropertyListener ( this ) ;
   }


   /**
     *  This constructor creates a blank document editor
     **/
   public DocumentEditor ( ) {
   }


   /**
     *  This method initializes all of our GUI and returns TRUE if everything
     * was created Ok
     **/
   public boolean initialize () {
      JPanel holdPanel = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SAVE     = new ActionButton ( Str.getStr ( STR_SAVE ),      ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),                    Str.getStr ( STR_SAVE ) ) ;
      pb_CLOSE    = new ActionButton ( Str.getStr ( STR_CLOSE ),     ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),                   Str.getStr ( STR_CLOSE ) ) ;
      pb_SEND     = new ActionButton ( Str.getStr ( STR_SEND ),      ImageSystem.getImageIcon ( this, ImageSystem.NEW   ),                   Str.getStr ( STR_SEND ) ) ;
      pb_PROPERTY = new ToolBarButton ( Str.getStr ( STR_PROPERTY ), ImageSystem.getImageIcon ( this, ImageSystem.PROPERTIES ),              Str.getStr ( STR_PROPERTY ) ) ;
      pb_HISTORY  = new ToolBarButton ( Str.getStr ( STR_HISTORY ),  ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_HISTORY_SMALL ),  Str.getStr ( STR_HISTORY ) ) ;
      pb_UNDO     = new ToolBarButton ( Str.getStr ( STR_UNDO ),     ImageSystem.getImageIcon ( this, ImageSystem.UNDO ),                    Str.getStr ( STR_UNDO ) ) ;
      toolBar = new JToolBar() ;

      if ( PropertySystem.getBool ( PropertySystem.PROPERTY_SHOW_INTERNAL_WORDPROC ) ) {
         pb_INT_BODY_EDIT = new ActionButton ( Str.getStr ( STR_EDIT_BODY_INT ), ImageSystem.getImageIcon ( this, ImageSystem.EDIT ), Str.getStr ( STR_EDIT_BODY_INT_HELP ) ) ;
         pb_INT_BODY_EDIT.addActionListener ( this ) ;
      }

      if ( PropertySystem.getBool ( PropertySystem.PROPERTY_USE_EXTERNAL_WORDPROC ) ) {
         pb_EXT_BODY_EDIT = new ActionButton ( Str.getStr ( STR_EDIT_BODY_EXT ), ImageSystem.getImageIcon ( this, ImageSystem.EDIT ), Str.getStr ( STR_EDIT_BODY_EXT_HELP ) ) ;
         pb_EXT_BODY_EDIT.addActionListener ( this ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our toolbar buttons                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.add ( pb_UNDO ) ;
      toolBar.add ( pb_PROPERTY ) ;
      toolBar.add ( pb_HISTORY ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      toolBar.setBorder ( GUISystem.emptyBorder ) ;
      toolBar.setOpaque ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SAVE.addActionListener        ( this ) ;
      pb_CLOSE.addActionListener       ( this ) ;
      pb_HISTORY.addActionListener     ( this ) ;
      pb_PROPERTY.addActionListener    ( this ) ;
      pb_SEND.addActionListener        ( this ) ;
      pb_UNDO.addActionListener        ( this ) ;
      mnu_SPELLCHECK.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;

      return( true ) ;
   } /* initialize  */


   /**
     *  This method will layout our scroll pane
     **/
   public void doLayout () {
      Dimension size = getSize() ;
      int       y    = 5 ;

      super.doLayout() ;
      if ( spellPanel != null ) {
         spellPanel.setBounds ( 5, 5, size.width-5, (18*3)+4 ) ;
         y+=(18*3)+4 ;
      }

      if ( editPanel != null )
         editPanel.setBounds ( 5, y, size.width-5, size.height-y-(5) ) ;
   }


   /**
     *  This method will start the DocumentEditor. Since this method
     * is called after we are showing, we will use this time to populate
     * the docEditor.
     **/
   public void start () {

      parentWin = GUISystem.getParentDefWin ( this ) ;

      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_SAVE ) ;

      if ( pb_EXT_BODY_EDIT != null )
         parentWin.addActionComponent ( this, pb_EXT_BODY_EDIT ) ;
      if ( pb_INT_BODY_EDIT != null )
         parentWin.addActionComponent ( this, pb_INT_BODY_EDIT ) ;

      if ( draft != null ) {
         new Thread ( this, THREAD_SET_EDIT ).start() ;
         documentChanged ( draft, 0 ) ;
         documentChanged = false ;
      }

      parentWin.addActionComponent ( this, pb_SEND ) ;

      panelSelected() ;

      checkDocument() ;
   }

   /**
     * This method will run a vareity of functions. The function
     * run depends on the name of the thread
     **/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      if ( operation.equals ( THREAD_SET_EDIT ) ) {
         editPanel = new DocEditPanel() ;
         add ( editPanel ) ;
         editPanel.setDocument ( draft ) ;
         revalidate() ;
         editPanel.setEnabled ( !documentCheckedOut() ) ;
      }
   }


   /**
     *  This method will return the name of our panel
     **/
   public String getName () {
      return( Str.getStr ( STR_EDIT ) ) ;
   }

   /**
     *  This method will return a small icon for us
     **/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.EDIT ) ) ;
   }

   /**
     *  This method will be called when this panel is deselected
     **/
   public void panelDeselected () {
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
      GUISystem.getParentDefWin ( this ).getAppDefaultMenuBar().removeEditMenuItem ( mnu_SPELLCHECK ) ;
   }

   /**
     *  This method will be called when this panel is selected
     **/
   public void panelSelected () {
      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 2 ) ;
      GUISystem.getParentDefWin ( this ).getAppDefaultMenuBar().addEditMenuItem ( mnu_SPELLCHECK ) ;
   }

   /**
     *  This method will be called when the user presses the
     * copy button or menu item
     **/
   public boolean copyText () {
      return( false ) ;
   }


   /**
     *  This method will be called when the user presses the
     * paste button or menu item
     **/
   public boolean pasteText () {
      return( false ) ;
   }

   /**
     *  This method will be called when the user presses the
     * cut button or menu item
     **/
   public boolean cutText () {
      return( false ) ;
   }

   /**
     *  This method will return NULL since the DocumentEditor
     * has no nav panel
     **/
   public NavPanel getDefaultNavPanel () {
      return( null ) ;
   }


   /**
     *  This method will return an instance of this
     **/
   public JComponent getJComponent () {
      return( this ) ;
   }


   /**
     *  This method will be called when the user wishes to
     * close our panel. We will return TRUE only if changes
     * that have been made to the document we are editing are
     * either saved or the user chooses to disregard them
     **/
   public boolean stop () {

      editorClosing = true ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the External Word Procesor is up, Offer to reimport  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !documentCheckedOut() ) {
         if ( extBodyEditProcess != null ) {
            if ( GUISystem.printBox ( "Question", "Do you wish to <b>reimport</b> your body changes?" ) ) {
               extBodyEditProcess.reimportBody() ;
               editorClosing = false ;
               return( false ) ;
            } else
               bodyReimportComplete() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the document has changed, offer to save              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( documentChanged ) {
            if ( GUISystem.printBox ( "Question", "Do you wish to save your document?" ) )
               return( saveDocument() ) ;
         }


         draft.checkIn() ;


      }

      panelDeselected() ;
      editPanel.free() ;
      draft.removeDocumentPropertyListener ( this ) ;

      /*----------------------------------------------------*/
      /*                                                    */
      /*  Now, notify the Document Maint. Applet to refresh */
      /*                                                    */
      /*----------------------------------------------------*/
      DocMaintPanel maintPanel = (DocMaintPanel) WindowSystem.findQuestPanel ( "DOCMAINT" ) ;

      if ( maintPanel != null )
         maintPanel.refresh() ;

      editorClosing = false ;
      return( true ) ;
   }

   /**
     *  This method will be called when the user wishes to
     * close our panel. We will return TRUE only if changes
     * that have been made to the document we are editing are
     * either saved or the user chooses to disregard them
     **/
   public boolean shutdown () {
      return( true ) ;
   }

   /**
     *  This method will bookmark the current Active Document
     * if this DocumentDraft has one.
     **/
   public boolean bookmark () {
      return( false ) ;
   }

   /**
     *  This method does nothing
     **/
   public void refresh () {
      GUISystem.printBox ( "Info", "Refresh not supported here" ) ;
   }


   /**
     * This method is called when the user hits the action button
     **/
   public void actionPerformed ( ActionEvent e ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE )
         parentWin.closePanel ( this, null ) ;

      /*--------------*/
      /*              */
      /*  Spell Check */
      /*              */
      /*--------------*/
      else if ( e.getSource() == mnu_SPELLCHECK )
         spellCheck() ;


      /*----------*/
      /*          */
      /*  History */
      /*          */
      /*----------*/
      else if ( e.getSource() == pb_HISTORY ) {
         WindowSystem.createPanel ( new DocHistPanel ( draft.getExistingDocInd(), true, null ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Undo                                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_UNDO ) {
         DocumentUndoableEdit undo = draft.undo() ;

         if ( undo != null )
            parentWin.setStatus ( undo.getPresentationName()+" Undone", 5 ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SEND                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SEND ) {
         sendNotification() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save the document                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE )
         saveDocument() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Launch the property window                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_PROPERTY )
         new PropertyDlg ( parentWin ) ;


      /*-----------------------*/
      /*                       */
      /*  Internal Body Editor */
      /*                       */
      /*-----------------------*/
      else if ( e.getSource() == pb_INT_BODY_EDIT ) {
         new WordProcFrame ( draft, 
                             (draft.getDocumentType() == TypeList.DOCTYPE_SSD ) ? Document.ADDITIONAL_INFO : Document.BODY ) ;
      }


      /*-----------------------*/
      /*                       */
      /*  External Body Editor */
      /*                       */
      /*-----------------------*/
      else if ( e.getSource() == pb_EXT_BODY_EDIT ) {

         /*-------------------------------------------*/
         /*                                           */
         /*  Start a new process if we don't have one */
         /*                                           */
         /*-------------------------------------------*/
         if ( extBodyEditProcess == null ) {
            extBodyEditProcess = new ExtBodyEditProcess ( draft ) ;
            extBodyEditProcess.setExtBodyEditProcessListener ( this ) ;
            pb_EXT_BODY_EDIT.setText ( Str.getStr ( STR_REIMPORT_BODY ) ) ;
            pb_EXT_BODY_EDIT.setToolTipText ( Str.getStr ( STR_REIMPORT_BODY ) ) ;

         }

         /*------------------------------------------------*/
         /*                                                */
         /*  If we have one, tell that process to reimport */
         /*                                                */
         /*------------------------------------------------*/
         else {
            if ( extBodyEditProcess != null ) {
               extBodyEditProcess.reimportBody() ;
            }
         }
      }

   }



   /**
     * This method will be called when the document we are editing has changed a property,
     * we will use this time to format the focus button text for our panel. This is done
     * by first taking the Document Filename. If the document filename is null, then a
     * document indicator or existing document indicator is used. If both are zero, then
     * a substring of the title is taken. We also make a note that something changed so when
     * we go to close we can ask the user if they wish to save their changes.
     **/
   public void documentChanged ( Document document, int propertyInd  ) {
      documentChanged = true ;

      if ( !focusButtonSet ) {
         String filename = document.getFilename() ;
         String title    = document.getTitle() ;
         int    docInd   = document.getDocInd () ;

         if ( ( filename != null ) && ( filename.length() > 0 ) ) {
            focusButtonSet = true ;
            parentWin.setFocusButtonText ( this, filename ) ;
         }

         else if ( docInd > 0 ) {
            parentWin.setFocusButtonText ( this, ""+docInd ) ;
         }

         else if ( ( title != null ) && ( title.length() > 10 ) ) {
            parentWin.setFocusButtonText ( this, title.substring ( 0, 10 )+"..." ) ;
         }

      }
   }


   /**
     * This method will return TRUE if we are editing the document passed.
     *
     * @param Object The Object to compare to.
     **/
   public boolean equals ( Object object ) {
      if ( object instanceof DocumentDraft ) {
         DocumentDraft compareDraft = (DocumentDraft) object ;

         if ( ( compareDraft.getExistingDocInd() == 0 ) &&
              ( compareDraft.getFilename() != null ) && 
              ( draft.getFilename() != null ) )
            return( compareDraft.getFilename().equals ( draft.getFilename() ) ) ;
         else {
            if ( ( compareDraft.getExistingDocInd() == 0 ) &&
                 ( draft.getExistingDocInd() == 0 ) )
               return( false ) ;
            else
               return( compareDraft.getExistingDocInd() == draft.getExistingDocInd() ) ;
         }
      }

      return( object == this ) ;
   }



   /**
     * This method will be called if the user edits a body externally and
     * reimports it
     **/
   public void bodyReimportComplete () {
      pb_EXT_BODY_EDIT.setText ( Str.getStr ( STR_EDIT_BODY_EXT ) ) ;
      pb_EXT_BODY_EDIT.setToolTipText ( Str.getStr ( STR_EDIT_BODY_EXT_HELP ) ) ;
      extBodyEditProcess = null ;
   }



   /**
     *  This method will send a notification to the BPM of the brand that the
     * document we are editing is for
     **/
   private void sendNotification () {
      TypeCategoryRec brandRec  = draft.getFirstAvailBrand() ;
      NotificationRec notifyRec = new NotificationRec() ;
      boolean         sent      = false ;

      if ( brandRec != null ) {
         UserRec userRec = UserSystem.getBrandProgramManager ( brandRec.getInd(), "DOC" ) ;

         if ( userRec != null )
            notifyRec.toUser = userRec.getUserId() ;
      }

      notifyRec.fromUser = UserSystem.getUserId() ;
      notifyRec.brandRec = brandRec ;
      notifyRec.mustSend = false ;
      notifyRec.resolRec = (TypeResolutionRec) TypeList.getInstance().objectFromInd ( 0, TypeList.TYPE_RESOLUTION ) ;
      notifyRec.openDate = DateSystem.getDate ( CDate.DATE_DATE ) ;
      notifyRec.docInd   = draft.getDocInd() ;
      notifyRec.subject  = Str.getStr ( STR_EDIT )+" - "+notifyRec.docInd ;

      new SendNotifyDlg ( GUISystem.getParentDefWin(this), notifyRec ).getResult() ;
   }



   /**
     *  This method will make attempt to save the document
     **/
   private boolean saveDocument () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the 3rd party editor is open, ask to reimport        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( extBodyEditProcess != null ) {
         if ( GUISystem.printBox ( "Question", "Would you like to reimport your body changes before you save?" ) )
            extBodyEditProcess.reimportBody() ;
         else
            extBodyEditProcess = null ;
      }


      switch ( editPanel.save() ) {
         case SaveDocDlg.SAVE_PUBLISHED :
         case SaveDocDlg.SAVE_CANCEL_DRAFT :
         case SaveDocDlg.SAVE_TEMPLATE :
            documentChanged = false ;
            parentWin.closePanel ( this, null ) ;
            return( true ) ;

         case SaveDocDlg.SAVE_SAVED :
            documentChanged = false ;
            if ( !editorClosing )
               GUISystem.printBox ( "Info", "The document draft has been saved" ) ;
            return( true ) ;
      }

      return( false ) ;
   }


   /**
     *  This method will make sure that the user can edit this document, that is that
     * it is not checked out
     **/
   private void checkDocument ( ) {
      if ( documentCheckedOut() ) {
         pb_SAVE.setVisible ( false ) ;
         if ( pb_EXT_BODY_EDIT != null )
            pb_EXT_BODY_EDIT.setVisible ( false ) ;
         if ( pb_INT_BODY_EDIT != null )
            pb_INT_BODY_EDIT.setVisible ( false ) ;
         GUISystem.printBox ( "Info", UserSystem.getNameFromUserId ( draft.getCheckedOutBy() )+" currently has this document checked out" ) ;
      } else {
         parentWin.setStatus ( "Checking out document" ) ;
         draft.checkOut ( UserSystem.getUserId() ) ;
         parentWin.setStatus ( null ) ;
      }
   }


   /**
     *  This method will return TRUE if the document we are editing should be consiedered
     * as checked out
     **/
   private boolean documentCheckedOut () {
      try {
         if ( ( draft != null ) && 
           ( draft.isCheckedOut() ) &&
           ( !draft.getCheckedOutBy().equals ( UserSystem.getUserId() ) ) )
         return( true ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1 ,e , false );
      }
      return( false ) ;
   }

   /**
     *  This method will give the printing support to the panel
     **/
   public Object getPrintObject() {
      if ( editPanel.isBodyShowing() ) {
         String[] params = new String[2] ;

         params[0] = PropertySystem.getString ( PropertySystem.PROPERTY_EXTERNAL_BROWSER ) ;
         params[1] = editPanel.getTempBodyFilename() ;

         System.out.println ( "Starting Browser!" ) ;
         try {
            Runtime.getRuntime().exec ( params ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( "Info", "Could not launch the 'External Web Browser'. Please verify you have configured the browser correctly by:<br><br>1. Select 'File'->'Preferences'->Application<br>2. When the preferences comes up, look in the 'External Browser Field'" ) ;
         }

         return( null ) ;
      }
      
      return( new Print ( editPanel.getContentPanel(), GUISystem.getParentDefWin(this), Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }
   /**
     *  This method will give the preview support to the panel
     **/

   public Object getPreviewObject() {

      
      if ( editPanel.isBodyShowing() ) {
         String[] params = new String[2] ;

         params[0] = PropertySystem.getString ( PropertySystem.PROPERTY_EXTERNAL_BROWSER ) ;
         params[1] = editPanel.getTempBodyFilename() ;

         try {
            Runtime.getRuntime().exec ( params ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( "Info", "Could not launch the 'External Web Browser'. Please verify you have configured the browser correctly by:<br><br>1. Select 'File'->'Preferences'->Application<br>2. When the preferences comes up, look in the 'External Browser Field'" ) ;
         }

         return( null ) ;
      }

      return( new Preview (editPanel.getContentPanel(),GUISystem.getParentDefWin(this), Str.getStr ( STR_DOCUMENTS ))) ;
   }



   /**
     * This method will initiate the spell check for a document
     **/
   private void spellCheck () {

      /*---------------------------------------*/
      /*                                       */
      /*  Setup and initialize the SpellPanel; */
      /*                                       */
      /*---------------------------------------*/
      spellPanel = new SpellPanel () ;
      spellPanel.addSpellPanelListener ( this ) ;


      /*-----------------------------*/
      /*                             */
      /*  Add our GUI and revalidate */
      /*                             */
      /*-----------------------------*/
      add ( spellPanel ) ;
      revalidate() ;


      /*-----------------------------------------------------*/
      /*                                                     */
      /*  Now, tell the DocEditPanel to start Spell Checking */
      /*                                                     */
      /*-----------------------------------------------------*/
      editPanel.spellCheck ( this, spellPanel ) ;
   }


   public void spellCheckComplete ( boolean foundErrors ) {
   }


   public void spellCheckWord ( String word, int startPos, int endPos ) {
   }

   public void replaceWord ( String oldWord, String newWord, int startPos ) {
   }

   protected void closeSpellPanel () {
      remove ( spellPanel ) ;
      spellPanel = null ;
      revalidate() ;
   }


} /* DocumentEditor */
