package com.ibm.nzna.projects.qit.doc.docedit ;                                      
/*-----------------------------------------------------------*/
/*   DocumentInfoFormComponent                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentInfoFormComponent           */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.Dimension ;
import java.awt.Color ;
import java.awt.Font ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
/**
 * This FormComponent shows the following document information:<br><br> 
 *
 * Active Document Indicator<br>
 * Draft Document Indicator<br>
 * Document Filename<br>
 * Draft Created On<br>
 * Last Touched By<br>
 * Last Touched On<br>
 *
 *
 * @version 1.0, 5/1/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentInfoFormComponent                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentInfoFormComponent extends ListPanel implements DocumentPropertyListener, AppConst {

   /**
     *  This is the Value Label holds the Active Document Indicator
     **/
   private ValueLabel activeDocumentInd = new ValueLabel ( Str.getStr ( STR_ACTIVE_DOCUMENT_IND ),
                                                           new JLabel(""+0) ) ;

   /**
     *  This is the Value Label which holds the Draft Document Indicator
     **/
   private ValueLabel draftDocumentInd = new ValueLabel ( Str.getStr ( STR_DRAFT_DOCUMENT_IND ),
                                                          new JLabel ( ""+0 ) ) ;

   /**
     *  This is the ValueLabel which holds the Document Filename
     **/
   private ValueLabel documentFilename = new ValueLabel ( Str.getStr ( STR_DOCUMENT_FILENAME ),
                                                          new JLabel ( "" ) ) ;

   /**
     *  This is the Value Label which holds the date the draft was created on
     **/
   private ValueLabel draftCreatedOn = new ValueLabel ( Str.getStr ( STR_DRAFT_CREATED_ON ),
                                                        new JLabel ( "" ) ) ;

   /**
     *  This is the Value label which holds the last touched by
     **/
   private ValueLabel lastTouchedBy = new ValueLabel ( Str.getStr ( STR_LAST_TOUCHED_BY ),
                                                       new JLabel ( "" ) ) ;

   /**
     *  This is thet value label which holds the Last Touched ON
     **/
   private ValueLabel lastTouchedOn = new ValueLabel ( Str.getStr ( STR_LAST_TOUCHED_ON ),
                                                       new JLabel ( "" ) ) ;


   /**
     *  This method will create a blank DocumetnInfoFormComponent
     **/
   public DocumentInfoFormComponent () {
      super() ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( activeDocumentInd ) ;
      add ( draftDocumentInd ) ;
      add ( new JLabel ( "_________________________________________" ) ) ;
      add ( documentFilename ) ;
      add ( new JLabel ( "_________________________________________" ) ) ;
      add ( draftCreatedOn ) ;
      add ( lastTouchedBy ) ;
      add ( lastTouchedOn ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the GUI properties                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque ( false ) ;
      setValueLabelProperties ( activeDocumentInd ) ;
      setValueLabelProperties ( draftDocumentInd ) ;
      setValueLabelProperties ( documentFilename ) ;
      setValueLabelProperties ( draftCreatedOn ) ;
      setValueLabelProperties ( lastTouchedBy ) ;
      setValueLabelProperties ( lastTouchedOn ) ;
      setPadding ( 3 ) ;


   }


   /**
     *  This method will set all of the properties for each Value Label
     *  passed in
     **/
   private void setValueLabelProperties ( ValueLabel valueLabel ) {
      valueLabel.setDescriptionWidth ( 150 ) ;
   }

   /**
     *  This method will set the information being displayed to the
     * document passed in
     **/
   public void setDocument ( Document document ) {

      document.removeDocumentPropertyListener ( this ) ;
      document.addDocumentPropertyListener ( this ) ;
      if ( document instanceof DocumentDraft ) {
         activeDocumentInd.setValue ( ""+((DocumentDraft)document).getExistingDocInd() ) ;
         draftDocumentInd.setValue ( ""+((DocumentDraft)document).getDocInd() ) ;
         try {
            draftCreatedOn.setValue ( DateSystem.prettyDateFromDate ( ((DocumentDraft)document).getStartDate() ) ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            draftCreatedOn.setValue ( ((DocumentDraft)document).getStartDate() ) ;
         }
      } else {
         activeDocumentInd.setValue ( ""+document.getDocInd() ) ;
      }

      documentFilename.setValue ( document.getFilename() ) ;
      if ( document.getLastTouchedBy() != null )
         lastTouchedBy.setValue ( UserSystem.getNameFromUserId ( document.getLastTouchedBy() ) ) ;
      else
         lastTouchedBy.setValue ( "-" ) ;
      
      if ( document.getLastTouched() != null )
         lastTouchedOn.setValue ( DateSystem.prettyDateFromStamp ( document.getLastTouched() ) ) ;
      else
         lastTouchedOn.setValue ( "-" ) ;

   }

   public void documentChanged ( Document document, int propertyInd ) {
      setDocument ( document ) ;
   }


} /* DocumentInfoFormComponent */
