package com.ibm.nzna.projects.qit.doc.docedit ;     
/*-----------------------------------------------------------*/
/*   DocumentSaver                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentSaver                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.util.* ;
import java.sql.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;

/**
 * This object has the capability to save/publish any document in any way 
 *
 *
 * @version 1.0, 8/22/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentSaver                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentSaver extends Object implements AppConst, DocConst {


   /**
     *  This method will simply save a draft
     *
     * @param draft The DocumentDraft to save
     *
     * @returns TRUE if saved
     **/
   public static boolean saveDraft ( DocumentDraft draft, DocSaveStatusPanel statusPanel ) {
      boolean saved = false ;                                                                  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Validate the Owner of the document                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( statusPanel != null )
         saved = validateOwner ( draft, WinUtil.getParentJDialog ( statusPanel ), null ) ;
      else
         saved = validateOwner ( draft, null, MainWindow.getInstance() ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First thing to do is to publish any graphics if we      */
      /* have any at all                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( draft.getGraphics() != null ) && ( draft.getGraphics().size() > 0 ) ) {
         if ( saved ) {
            if ( statusPanel != null ) {
               statusPanel.enableGraphicsLabel ( true ) ;
               statusPanel.setGraphicsSelected ( true ) ;
            }
            saved = saveDraftGraphics ( draft.getGraphics(), statusPanel ) ;
         }
      } else
         saved = true ;                      // We are saved since we have nothing to do!


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Next, save any file attachments                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( saved ) && 
           ( draft.getFileAttachments() != null ) && 
           ( draft.getFileAttachments().size() > 0 ) ) {
         if ( statusPanel != null ) {
            statusPanel.enableFilesLabel ( true ) ;
            statusPanel.setFilesSelected ( true ) ;
         }
         saved = saveDraftFiles ( draft.getFileAttachments(), statusPanel ) ;
      } else
         saved = true ;              // We are saved since we have nothing to do!



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Next, Save the document                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( saved ) {
         int sqlCode = 0 ;

         if ( statusPanel != null ) {
            statusPanel.enableDocumentsLabel ( true ) ;
            statusPanel.setDocumentsSelected ( true ) ;
            statusPanel.setText ( "Saving Document Draft" ) ;
         }
         draft.setLastTouchedBy ( UserSystem.getUserId() ) ;                        
         sqlCode = draft.writeToDatabase() ;

         switch ( sqlCode ) {
            case 0 :
               saved = true ;
               draft.updateRecStatus ( DocumentDraft.REC_NOCHANGE ) ;
               break ;

            default:
               LogSystem.log ( 1, "Cannot write document. SQLCode returned:"+sqlCode ) ;
               saved = false ;
               break ;
         }
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check the document back in                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( saved ) {
      }

      return( saved ) ;
   }




   /*------------------------------------------------------------*/
   /* validateOwner
    */
   /**
    *    This method will validate that the current owner
    *  (FIELD_OWNER) is complete. If it is not complete
    *  the BPM of a brand associated to this document will be
    *  assigned. If there are multiple brands, a list will be
    *  given for the user to choose.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean validateOwner ( DocumentDraft draft, JDialog parent, JFrame parent1 ) {
      boolean validated = false ;
      String  owner     = draft.getOwner() ;

      if ( ( owner  == null ) || ( owner.length() == 0 ) ) {
         Vector selectableBrands = null ;
         Vector applicableBrands = draft.getApplicableBrands() ;
         Vector brands           = draft.getBrands() ;

         if ( ( applicableBrands != null ) && ( applicableBrands.size() > 0 ) )
            selectableBrands = applicableBrands ;
         else
            selectableBrands = brands ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have a ProdDocRec, use it first                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( selectableBrands != null ) {
            TypeCategoryRec brandRec = null ;                                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If there is more than 1 brand, ask for which one        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( selectableBrands.size() > 1 ) {
               Vector selection = null ;

               if ( parent != null )
                  selection = new JListDlg ( parent , 
                                             Str.getStr ( STR_SELECT_OWNER_BRAND ),
                                             selectableBrands ).getResult() ;
               else
                  selection = new JListDlg ( parent1 , 
                                             Str.getStr ( STR_SELECT_OWNER_BRAND ),
                                             selectableBrands ).getResult() ;

               if ( ( selection != null ) && ( selection.size() > 0 ) )
                  brandRec = (TypeCategoryRec)selection.elementAt(0) ;

            } else
               brandRec = (TypeCategoryRec) selectableBrands.elementAt(0) ;

            if ( brandRec != null )
               owner = UserSystem.getBrandProgramManager ( brandRec.getInd() ) ;


            validated = (owner != null) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, allow the user to select the owner           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            Vector users = null ;

            if ( parent != null )
               users = new SelectUserDlg ( parent, false ).getResult() ;
            else
               users = new SelectUserDlg ( parent1, false ).getResult() ;

            if ( ( users != null ) && ( users.size() > 0 ) ) {
               owner = ((UserRec)users.elementAt(0)).getUserId() ;
               validated = true ;
            }
         }

      } else
         validated = true ;


      if ( validated )
         draft.setOwner ( owner ) ;

      return( validated ) ;
   }




   /**
     *  This method will simply publish a draft
     *
     * @param draft The DocumentDraft to publish
     *
     * @returns TRUE if saved
     **/
   public static boolean publishDraft ( DocumentDraft draft, DocSaveStatusPanel statusPanel ) {
      boolean saved         = false ;                 
      Document document     = new Document() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now close the draft                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draft.setStopDate ( new CDate ( CDate.DATE_DATE ).today() ) ;
      saved =saveDraft ( draft, statusPanel )  ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First thing to do is to publish any graphics if we      */
      /* have any at all                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( saved ) {

         /*-----------------------------------------------------*/
         /*                                                     */
         /*  Before saving, figure out the RecStatus by looking */
         /* at the Work required of the draft                   */
         /*                                                     */
         /*-----------------------------------------------------*/
         document.initializeProperties ( draft ) ;
         document.updateRecStatus ( Document.REC_NOCHANGE ) ;
         if ( draft.getWorkRequired() == TypeList.WORKREQ_EDIT ) {
            document.updateRecStatus ( Document.REC_UPDATE ) ;
         } else if ( draft.getWorkRequired() == TypeList.WORKREQ_CREATE ) {
            document.updateRecStatus ( Document.REC_INSERT ) ;
         } else if ( draft.getWorkRequired() == TypeList.WORKREQ_DELETE ) {
            document.updateRecStatus ( Document.REC_DELETE ) ;
         }

         saved = false ;

         if ( ( draft.getGraphics() != null ) && ( draft.getGraphics().size() > 0 ) ) {
            if ( statusPanel != null ) {
               statusPanel.enableGraphicsLabel ( true ) ;
               statusPanel.setGraphicsSelected ( true ) ;
            }

            saved = syncGraphics ( draft, statusPanel ) ;
            if ( saved )
               saved = savePublishGraphics ( draft.getGraphics(), statusPanel ) ;

            if ( saved )
               removeDraftGraphics ( draft.getGraphics(), statusPanel ) ;

         } else
            saved = true ;                      // We are saved since we have nothing to do!


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Next, save any file attachments                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( saved ) {
            if ( ( draft.getFileAttachments() != null ) && 
                 ( draft.getFileAttachments().size() > 0 ) ) {
               if ( statusPanel != null ) {
                  statusPanel.enableFilesLabel ( true ) ;
                  statusPanel.setFilesSelected ( true ) ;
               }
               saved=savePublishFiles ( draft, draft.getFileAttachments(), statusPanel ) ;
            } else
               saved = true ;              // We are saved since we have nothing to do!
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Next, Save the document                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( saved ) {
            int sqlCode = 0 ;

            if ( statusPanel != null ) {
               statusPanel.enableDocumentsLabel ( true ) ;
               statusPanel.setDocumentsSelected ( true ) ;
               statusPanel.setText ( "Publishing Document Draft" ) ;
            }


            document.setLastTouchedBy ( UserSystem.getUserId() ) ;                        
            sqlCode = document.writeToDatabase() ;

            switch ( sqlCode ) {
               case 0 :
                  saved = true ;
                  break ;

               default:
                  LogSystem.log ( 1, "Cannot write document. SQLCode returned:"+sqlCode ) ;
                  saved = false ;
                  break ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this draft did not publish, reopen it                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !saved ) {
            if ( statusPanel != null )
               statusPanel.setText ( "Opening old draft" ) ;
            draft.setStopDate ( null ) ;
            draft.writeToDatabase() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, delete all graphics and files from temp      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            if ( statusPanel != null )
               statusPanel.setText ( "Cleaning Draft Servers for Graphics/Files that might be left" ) ;
            cleanTempFTP ( document ) ;
         }


      } else
         LogSystem.log ( 1, "Could not continue publish because save as draft failed" ) ;

      return( saved ) ;
   }


   /**
     *  This method will simply save all graphics for a draft
     **/
   private static boolean saveDraftGraphics ( Vector graphics, DocSaveStatusPanel statusPanel ) {
      boolean rc                 = false ;
      int     i                  = 0 ;
      int     size               = graphics.size() ;
      Ftp     ftp                = new Ftp () ;
      String  host               = Constants.getStrConst ( "IT_IMAGEHOSTTEMP" ) ;
      String  remoteDirectory    = Constants.getStrConst ( "IT_IMAGELOCTEMP" ) ;
      String  localTempDirectory = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up the FTP Host                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ftp.setHost ( host ) ;
      ftp.setUser ( FTP_USERID ) ;
      ftp.setPassword ( FTP_PASSWORD ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Log our information                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.beginLogSection ( "Saving Graphics" ) ;
      LogSystem.log ( 1, "FTP Server:"+host ) ;
      LogSystem.log ( 1, "User Id:"+FTP_USERID ) ;
      LogSystem.log ( 1, "Directory:"+remoteDirectory ) ;

      try {
         DocumentGraphic graphic = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Log in                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( statusPanel != null )
            statusPanel.setText ( "Connecting to Draft Graphics Server" ) ;
         ftp.connect() ;
         ftp.login() ;
         ftp.cd ( remoteDirectory ) ;
         ftp.setType ( ftp.BINARY ) ;        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save each graphic                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            graphic = (DocumentGraphic)graphics.elementAt(i) ;

            if ( graphic.getRecStatus() == DocumentGraphic.REC_DELETE ) {
               if ( statusPanel != null )
                  statusPanel.setText ( "Deleting Graphic "+graphic.getFilename() ) ;
               LogSystem.log ( 1, "Deleting Graphic: "+graphic.getFilename() ) ;
               try {
                  ftp.delete ( graphic.getFilename() ) ;
               } catch ( Exception e ) {
               }
            }

            else {
               if ( statusPanel != null )
                  statusPanel.setText ( "Saving Graphic "+graphic.getFilename() ) ;
               LogSystem.log ( 1, "FTPing Graphic: "+graphic.getFilename() ) ;
               ftp.put ( localTempDirectory+File.separator+graphic.getFilename() ) ;
            }
            i++ ;
         }

         ftp.disconnect() ;
         rc=true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Cannot save graphics" ) ;
         rc = false ;
      }

      LogSystem.endLogSection() ;


      return( rc ) ;

   }



   /**
     *  This method will simply save all graphics for an active document
     **/
   private static boolean savePublishGraphics ( Vector graphics, DocSaveStatusPanel statusPanel ) {
      boolean rc                 = false ;
      int     i                  = 0 ;
      int     size               = graphics.size() ;
      Ftp     ftp                = new Ftp () ;
      String  host               = Constants.getStrConst ( "IT_IMAGEHOST" ) ;
      String  remoteDirectory    = Constants.getStrConst ( "IT_IMAGELOC" ) ;
      String  localTempDirectory = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up the FTP Host                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ftp.setHost ( host ) ;
      ftp.setUser ( FTP_USERID ) ;
      ftp.setPassword ( FTP_PASSWORD ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Log our information                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.beginLogSection ( "Publishing Graphics" ) ;
      LogSystem.log ( 1, "FTP Server:"+host ) ;
      LogSystem.log ( 1, "User Id:"+FTP_USERID ) ;
      LogSystem.log ( 1, "Directory:"+remoteDirectory ) ;

      try {
         DocumentGraphic graphic = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Log in                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( statusPanel != null )
            statusPanel.setText ( "Connecting to Graphics Server" ) ;
         ftp.connect() ;
         ftp.login() ;
         ftp.cd ( remoteDirectory ) ;
         ftp.setType ( ftp.BINARY ) ;        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save each graphic                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            graphic = (DocumentGraphic)graphics.elementAt(i) ;

            if ( !graphic.valid() ) {
               if ( statusPanel != null )
                  statusPanel.setText ( "Deleting Graphic "+graphic.getFilename() ) ;
               LogSystem.log ( 1, "Deleting Graphic: "+graphic.getFilename() ) ;
               try {
                  ftp.delete ( graphic.getFilename() ) ;
               } catch ( Exception e ) {
               }
            }

            else {
               if ( statusPanel != null )
                  statusPanel.setText ( "Saving Graphic "+graphic.getFilename() ) ;
               LogSystem.log ( 1, "FTPing Graphic: "+graphic.getFilename() ) ;
               ftp.put ( localTempDirectory+File.separator+graphic.getFilename() ) ;
            }
            i++ ;
         }

         ftp.disconnect() ;
         rc=true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Cannot save graphics" ) ;
         rc = false ;
      }



      LogSystem.log ( 1, "FTP Process returning "+rc ) ;
      LogSystem.endLogSection() ;

      return( rc ) ;

   }


   /**
     *  This method will remove all draft graphics from the temp directory
     **/
   private static boolean removeDraftGraphics ( Vector graphics, DocSaveStatusPanel statusPanel ) {
      boolean rc                 = false ;
      int     i                  = 0 ;
      int     size               = graphics.size() ;
      Ftp     ftp                = new Ftp () ;
      String  host               = Constants.getStrConst ( "IT_IMAGEHOSTTEMP" ) ;
      String  remoteDirectory    = Constants.getStrConst ( "IT_IMAGELOCTEMP" ) ;
      String  localTempDirectory = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up the FTP Host                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ftp.setHost ( host ) ;
      ftp.setUser ( FTP_USERID_PRODUCTION ) ;
      ftp.setPassword ( FTP_PASSWORD_PRODUCTION ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Log our information                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.beginLogSection ( "Publishing Graphics" ) ;
      LogSystem.log ( 1, "FTP Server:"+host ) ;
      LogSystem.log ( 1, "User Id:"+FTP_USERID ) ;
      LogSystem.log ( 1, "Directory:"+remoteDirectory ) ;

      try {
         DocumentGraphic graphic = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Log in                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( statusPanel != null )
            statusPanel.setText ( "Connecting to Draft Graphics Server" ) ;
         ftp.connect() ;
         ftp.login() ;
         ftp.cd ( remoteDirectory ) ;
         ftp.setType ( ftp.BINARY ) ;        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save each graphic                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            graphic = (DocumentGraphic)graphics.elementAt(i) ;

            if ( statusPanel != null )statusPanel.setText ( "Deleting Draft Graphic "+graphic.getFilename() ) ;
            LogSystem.log ( 1, "Deleting Graphic: "+graphic.getFilename() ) ;
            try {
               ftp.delete ( graphic.getFilename() ) ;
            } catch ( Exception e ) {
            }
            i++ ;
         }

         ftp.disconnect() ;
         rc=true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }



      LogSystem.log ( 1, "FTP Process returning "+rc ) ;
      LogSystem.endLogSection() ;

      return( rc ) ;

   }



   /**
     *  This method will synchronize the draft graphics with the active ones
     * by comparing the two lists.
     **/
   private static boolean syncGraphics ( DocumentDraft draft, DocSaveStatusPanel statusPanel ) {
      SQLMethod sqlMethod        = new SQLMethod ( DatabaseSystem.QUEST, "syncGraphics", 5 ) ;
      boolean   rc               = false ;                                         
      Vector    existingGraphics = new Vector ( 1, 1 ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, first read in the existing document's graphics      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         Statement stmt             = sqlMethod.createStatement() ;
         ResultSet rs               = null ;
         String    sql              = null ;     


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the SQL                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql = "SELECT FILENAME "+
               "FROM TIGNEW.GRAPHICS  "+
               "WHERE DOCIND = "+draft.getExistingDocInd()+" "+
               "FOR FETCH ONLY" ;  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Run the SQL                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( sql ) ;


         while ( rs.next() ) {
            DocumentGraphic graphic = new DocumentGraphic ( draft.getDocInd(), rs.getString(1).trim() ) ;

            existingGraphics.addElement ( graphic ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Close our ResultSet and Statement                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs.close() ;
         stmt.close() ;


         rc = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1 ,e , false ) ;
      }





      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we have done that, see what graphics are in the  */
      /* Existing document but not in the draft. We must delete    */
      /* those from the production site since they were deleted    */
      /* in the draft. We will delete the graphic from the         */
      /* production site by adding it to the draft's graphics as   */
      /* a REC_DELETE, then, when the document graphics are        */
      /* published, poof!                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc ) {
         Vector           draftGraphics  = draft.getGraphics() ;
         int              i              = 0 ;
         int              size           = existingGraphics.size() ;
         DocumentGraphic  currentGraphic = null ;

         while ( i < size ) {
            currentGraphic = (DocumentGraphic) existingGraphics.elementAt(i) ;

            if ( !draftGraphics.contains ( currentGraphic ) ) {
               currentGraphic.updateRecStatus ( DocumentGraphic.REC_NOCHANGE ) ;
               currentGraphic.updateRecStatus ( DocumentGraphic.REC_DELETE ) ;
               draftGraphics.addElement ( currentGraphic ) ;
            }
            i++ ;
         }

         draft.setGraphics ( draftGraphics ) ;

      }


      sqlMethod.close() ;
      LogSystem.log ( 1, "Graphics Sync Returning "+rc ) ;

      return( rc ) ;
   }


   /**
     *  This method will synchronize the draft file attachments with the active ones
     * by comparing the two lists.
     **/
   private static boolean syncFiles ( DocumentDraft draft, DocSaveStatusPanel statusPanel ) {
      SQLMethod sqlMethod        = new SQLMethod ( DatabaseSystem.QUEST, "syncFiles", 5 ) ;
      boolean   rc               = false ;                                         
      Vector    existingFiles    = new Vector ( 1, 1 ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, first read in the existing document's graphics      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         Statement stmt             = sqlMethod.createStatement() ;
         ResultSet rs               = null ;
         String    sql              = null ;     


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the SQL                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql = "SELECT FILENAME "+
               "FROM TIGRIS.DOCFILES  "+
               "WHERE DOCIND = "+draft.getExistingDocInd()+" "+
               "FOR FETCH ONLY" ;  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Run the SQL                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( sql ) ;


         while ( rs.next() ) {
            DocumentFile file = new DocumentFile ( draft.getDocInd(), rs.getString(1).trim() ) ;

            existingFiles.addElement ( file ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Close our ResultSet and Statement                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs.close() ;
         stmt.close() ;


         rc = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1 ,e , false ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we have done that, see what graphics are in the  */
      /* Existing document but not in the draft. We must delete    */
      /* those from the production site since they were deleted    */
      /* in the draft. We will delete the graphic from the         */
      /* production site by adding it to the draft's graphics as   */
      /* a REC_DELETE, then, when the document graphics are        */
      /* published, poof!                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rc ) {
         Vector           draftFiles     = draft.getFileAttachments() ;
         int              i              = 0 ;
         int              size           = existingFiles.size() ;
         DocumentFile  currentFile = null ;

         while ( i < size ) {
            currentFile = (DocumentFile) existingFiles.elementAt(i) ;

            if ( !draftFiles.contains ( currentFile ) ) {
               currentFile.updateRecStatus ( DocumentFile.REC_NOCHANGE ) ;
               currentFile.updateRecStatus ( DocumentFile.REC_DELETE ) ;
               draftFiles.addElement ( currentFile ) ;
            }
            i++ ;
         }

         draft.setFileAttachments ( draftFiles ) ;

      }


      sqlMethod.close() ;

      LogSystem.log ( 1, "File Sync returning "+rc ) ;
      return( rc ) ;
   }







   /**
     *  This method will simply save all file attachments for a draft
     **/
   private static boolean saveDraftFiles ( Vector files, DocSaveStatusPanel statusPanel ) {
      boolean rc                 = false ;
      int     i                  = 0 ;
      int     size               = files.size() ;
      Ftp     ftp                = new Ftp () ;
      String  host               = Constants.getStrConst ( "IT_FILEHOSTTEMP" ) ;
      String  remoteDirectory    = Constants.getStrConst ( "IT_FILELOCTEMP" ) ;
      String  localTempDirectory = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up the FTP Host                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ftp.setHost ( host ) ;
      ftp.setUser ( FTP_USERID ) ;
      ftp.setPassword ( FTP_PASSWORD ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Log our information                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.beginLogSection ( "Saving Files" ) ;
      LogSystem.log ( 1, "FTP Server:"+host ) ;
      LogSystem.log ( 1, "User Id:"+FTP_USERID ) ;
      LogSystem.log ( 1, "Directory:"+remoteDirectory ) ;

      try {
         DocumentFile file = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Log in                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( statusPanel != null )
            statusPanel.setText ( "Connecting to Draft File Server" ) ;
         ftp.connect() ;
         ftp.login() ;
         ftp.cd ( remoteDirectory ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save each file                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            file = (DocumentFile)files.elementAt(i) ;

            if ( file.getLocalFilename() != null ) {

               if ( file.getRecStatus() == DocumentFile.REC_DELETE ) {
                  if ( statusPanel != null )
                     statusPanel.setText ( "Deleting File "+file.getLocalFilename() ) ;
                  LogSystem.log ( 1, "Deleting File: "+file.getLocalFilename() ) ;
                  try {
                     ftp.delete ( file.getLocalFilename() ) ;
                  } catch ( Exception e ) {
                  }
               }

               else {
                  if ( statusPanel != null )
                     statusPanel.setText ( "Saving File "+file.getLocalFilename() ) ;
                  LogSystem.log ( 1, "FTPing File: "+file.getLocalFilename() ) ;
                  ftp.setType ( ( file.getFTPMode() == 0 ) ? Ftp.BINARY : Ftp.ASCII ) ;
                  ftp.put ( file.getLocalFilename() ) ;
               }
            }
            i++ ;
         }

         ftp.disconnect() ;
         rc=true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Info", "Cannot save files" ) ;
         rc = false ;
      }


      LogSystem.log ( 1, "FTP Process returning "+rc ) ;
      LogSystem.endLogSection() ;


      return( rc ) ;

   }



   /**
     *  This method will publish all file attachments with their header files
     **/
   private static boolean savePublishFiles ( DocumentDraft draft, Vector fileAttachments, DocSaveStatusPanel statusPanel ) {
      Ftp          ftp                 = null ;
      Vector       publishFiles        = new Vector ( 1 ) ;                                     
      int          i                   = 0 ;
      int          fileCount           = fileAttachments.size() ;
      boolean      rc                  = false ;
      String       remoteTempDirectory = Constants.getStrConst ( "IT_FILELOCTEMP" ) ;
      DocumentFile currentFile         = null ;
      String       tempDirectory       = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;
      String       incomingDirectory   = Constants.getStrConst ( "IT_FILELOC" ) ;

      LogSystem.beginLogSection ( "Publishing Files" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, download all file attachments from the temp      */
      /* directory. Any file attachment that is downloaded must    */
      /* be published. So, we will make a list of the files which  */
      /* download                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( statusPanel != null )
         statusPanel.setText ( "Connecting to Draft File Server" ) ;
      ftp = QuestUtil.connectToTempFileFTP() ;

      try {

         if ( statusPanel != null )
            statusPanel.setText ( "Downloading changed files" ) ;
         if ( ftp != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Change to the temp directory                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp.cd ( remoteTempDirectory ) ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Process the files                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( i < fileCount ) {

               currentFile = (DocumentFile)fileAttachments.elementAt(i) ;                                          

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Only process if the file is not delete and the file     */
               /* does not have a local filename. If the file has a local   */
               /* filename that means that QIT already knows about the      */
               /* file and it should be FTPed. We don't need to download it */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( currentFile.valid() ) {

                  if ( currentFile.getLocalFilename() != null )
                     publishFiles.addElement ( currentFile ) ;

                  else {
                     try {
                        if ( statusPanel != null )
                           statusPanel.setText ( "Downloading "+currentFile.getFilename()+"("+currentFile.getSize()+" bytes) from Draft File Server" ) ;
                        ftp.setType ( (currentFile.getFTPMode() == 0 ) ? ftp.BINARY : ftp.ASCII ) ;
                        ftp.get ( currentFile.getFilename(), tempDirectory+File.separator+currentFile.getFilename() ) ;
                        publishFiles.addElement ( currentFile ) ;
                     } catch ( Exception e ) {
                     }
                  }
               }
               i++ ;
            }

            ftp.disconnect() ;
         } /* if we logged into the temp  */


         if ( syncFiles ( draft, statusPanel ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, add the existing files to the list of publish      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            fileAttachments = draft.getFileAttachments() ;
            if ( ( fileAttachments != null ) && ( fileAttachments.size() >0 ) ) {
               int size      = fileAttachments.size() ;

               i=0 ;
               while ( i < size ) {
                  if ( !publishFiles.contains ( fileAttachments.elementAt(i) ) )
                     publishFiles.addElement ( fileAttachments.elementAt(i) ) ;
                  /* If the publish files does have this file, make sure the rec status is the same  */
                  else {
                     ((DocumentFile)publishFiles.elementAt(i)).updateRecStatus ( DocumentFile.REC_NOCHANGE ) ;
                     ((DocumentFile)publishFiles.elementAt(i)).updateRecStatus ( ((DocumentFile)fileAttachments.elementAt(i)).getRecStatus() ) ;
                  }
                  i++ ;
               }
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, if we have any files to publish, do so now          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            i=0 ;
            fileCount=publishFiles.size() ;

            if ( statusPanel != null )
               statusPanel.setText ( "Connecting to Publish File Server" ) ;
            ftp = QuestUtil.connectToProductionFileFTP() ;

            try {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Go into the incoming directory                          */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ftp.cd ( incomingDirectory ) ;

               while ( i < fileCount ) {
                  currentFile = (DocumentFile)publishFiles.elementAt(i) ;             

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Create the Header File                                  */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( createFileHeader ( currentFile ) ) {

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   FTP both the header file and file to incoming           */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     ftp.setType ( ftp.BINARY ) ;
                     if ( statusPanel != null )
                        statusPanel.setText ( "Attaching file "+currentFile.getLocalFilename() ) ;
                     ftp.put ( tempDirectory+File.separator+currentFile.getFilename()+".hdr" ) ;
                     if ( currentFile.valid() ) {
                        ftp.setType ( (currentFile.getFTPMode() == 0 ) ? ftp.BINARY : ftp.ASCII ) ;
                        ftp.put ( currentFile.getLocalFilename() ) ;
                     }

                  } else
                     return( false ) ;
                  i++ ;
               }

               ftp.disconnect() ;
               rc = true ;
            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
               GUISystem.printBox ( "Error", "Cannot publish file attachments. This document will not be published" ) ;
               rc = false ;
            }
         } /* if sync files  */
         else
            rc = false ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Error", "Could not FTP file attachments. The Document will not be published" ) ;
         rc = false ;
      }


      LogSystem.log ( 1, "FTP Process returning "+rc ) ;
      LogSystem.endLogSection() ;

      return( rc ) ;
   }



   /**
     *  This method will create the file attachment headers needed to publish a file attachment
     **/
   private static boolean createFileHeader ( DocumentFile documentFile ) {
      String           tempDirectory = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;
      File             header        = new File ( tempDirectory+File.separator+documentFile.getFilename()+".hdr" ) ;
      boolean          created       = false ;

      try {
         FileOutputStream output = new FileOutputStream ( header ) ;

         output.write ( new String ( "Filename: "+documentFile.getFilename()+"\n" ).getBytes() ) ;
         output.write ( new String ( "filesize: "+documentFile.getSize()+"\n" ).getBytes() ) ;
         output.write ( new String ( "filearea: "+documentFile.getLocation()+"\n" ).getBytes() ) ;
         output.write ( new String ( "filetype: "+((documentFile.getFTPMode() == 0 ) ? "binary" : "ascii")+"\n" ).getBytes() ) ;

         if ( documentFile.valid() )
            output.write ( new String ( "actiontype: insert\n" ).getBytes() ) ;
         else
            output.write ( new String ( "actiontype: delete\n" ).getBytes() ) ;

         output.flush() ;
         output.close() ;
         created = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( "Error", "The header for file "+documentFile.getFilename()+" could not be created. Please make sure that you have enough Disk Space left for your temp directory" ) ;
         created = false ;
      }

      return( created ) ;
   }



   /**
     *  This method will remove all graphics and file attachments from the temp servers that a document
     * will use
     **/
   private static void cleanTempFTP ( Document document ) {
      Vector graphics = document.getGraphics() ;
      Vector files    = document.getFileAttachments() ;

      LogSystem.beginLogSection (  "Cleaning temp FTP directory" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the graphics                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( graphics != null ) && ( graphics.size() > 0 ) ) {
         try {
            Ftp     ftp             = QuestUtil.connectToTempGraphicFTP() ;
            String  remoteDirectory = Constants.getStrConst ( "IT_IMAGELOCTEMP" ) ;

            if ( ftp != null ) {
               int    i    = 0 ;
               int    size = graphics.size() ;

               ftp.cd ( remoteDirectory ) ;

               while ( i < size ) {
                  try {
                     ftp.delete ( ((DocumentGraphic)graphics.elementAt(i)).getFilename() ) ;
                  } catch ( Exception e ) {
                  }
                  i++ ;
               }
            }

            ftp.disconnect() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, "Could not clean temp graphics" ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the files                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( files != null ) && ( files.size() > 0 ) ) {
         try {
            Ftp     ftp             = QuestUtil.connectToTempFileFTP() ;
            String  remoteDirectory = Constants.getStrConst ( "IT_FILELOCTEMP" ) ;

            if ( ftp != null ) {
               int    i    = 0 ;
               int    size = files.size() ;

               ftp.cd ( remoteDirectory ) ;

               while ( i < size ) {
                  try {
                     ftp.delete ( ((DocumentFile)files.elementAt(i)).getFilename() ) ;
                  } catch ( Exception e ) {
                  }
                  i++ ;
               }
            }

            ftp.disconnect() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, "Could not clean temp files" ) ;
         }
      }

      LogSystem.endLogSection() ;

   }




   /**
     *  This method will simply publish the document, and then will make an entry into the TIGRIS.PUBLISHNOW table.
     **/
   public static boolean publishDraftNow ( DocumentDraft draft, DocSaveStatusPanel statusPanel ) {
      boolean published = false ;

      if ( publishDraft ( draft, statusPanel ) ) {
         String sql = "DELETE FROM TIGRIS.PUBLISHNOW WHERE DOCIND = "+draft.getExistingDocInd()+SqlRunner.END_DELIM+
                      "INSERT INTO TIGRIS.PUBLISHNOW VALUES ( "+draft.getExistingDocInd()+" )"+SqlRunner.END_DELIM ;
         int    rc  = SqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST, sql ) ;

         if ( rc != 0 ) {
            LogSystem.log ( 1, "Could not execute Publish now. rc:"+rc ) ;
            GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
         } else
            published = true ;
      }
      return( published ) ;
   }
} /* DocumentSaver */

