package com.ibm.nzna.projects.qit.doc.docedit ;                  
/*-----------------------------------------------------------*/
/*   DownloadableInfoPanel                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DownloadableInfoPanel               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;        
import com.ibm.nzna.projects.common.quest.type.* ;        
/**
 * This panel holds the SSD Description, Part Number, Version, URL and Release Date 
 *
 *
 * @version 1.0, 8/16/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DownloadableInfoPanel                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class DownloadableInfoPanel extends DPanel implements DocumentPropertyListener, ActionListener, FocusListener, DocConst, DataLengths, AppConst {

   private boolean processChanges = true ;
   /**
     *  This is the description description :-)
     **/
   private JLabel st_DESCRIPTION = new JLabel ( "*"+Str.getStr ( STR_DESCRIPTION ) ) ;

   /**
     *  This is the NoTabJTextArea for the description
     **/
   private NoTabJTextArea mle_DESCRIPTION = new NoTabJTextArea() ;

   /**
     *  This is the JScrollPane holding the description
     **/
   private JScrollPane scr_DESCRIPTION = new JScrollPane ( mle_DESCRIPTION ) ;

   /**
     *  This is the drop down which holds the Publication Types
     **/
   private JComboBox cb_PUB_TYPE = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_PUB ) ) ;

   /**
     *  This is the value label which holds the text and cb_PUB_TYPE
     **/
   private ValueLabel vl_PUB_TYPE = new ValueLabel ( Str.getStr ( STR_PUBLICATION ),
                                                     cb_PUB_TYPE ) ;

   /**
     *  This is the entry field for the part number
     **/
   private JTextField ef_PARTNUM = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 
                                                                       LENGTH_DOCUMENT_PART_NUMBER ), "", 0 ) ;
   /**
     *  This is the value label for the Part Number
     **/
   private ValueLabel partNumber     = new ValueLabel ( "*"+Str.getStr ( STR_PART_NUMBER ),
                                                        ef_PARTNUM ) ;

   /**
     *  This is the entry field for the version
     **/
   private JTextField ef_VERSION = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 
                                                                       LENGTH_FILEVERSION ), "", 0 ) ;
   /**
     *  This is the Value Lable for the Version
     **/
   private ValueLabel version = new ValueLabel ( "*"+Str.getStr ( STR_VERSION ),
                                                 ef_VERSION ) ;

   /**
     *  This is the entry field for the URL
     **/
   private JTextField ef_URL = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 
                                                                   LENGTH_FILEURL ), "", 0 ) ;
   /**
     *  This is the Value Lable for the URL 
     **/
   private ValueLabel url = new ValueLabel ( "*"+Str.getStr ( STR_URL_FOR_NO_FILE_ATTACH ),
                                             ef_URL ) ;

   private DateEntryField ef_RELEASE_DATE = new DateEntryField ( ) ;

   /**
     *  This is the Value Lable for the Release Date
     **/
   private ValueLabel releaseDate = new ValueLabel ( "*"+Str.getStr ( STR_RELEASE_DATE ),
                                                     ef_RELEASE_DATE ) ;


   /**
     *  This is the document we are editing
     **/
   private Document document = null ;



   /**
     *  Default Constructor creates our controls and does nothing else
     **/
   public DownloadableInfoPanel () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setPreferredSize ( new Dimension ( 500, 220 ) ) ;
      setBackground ( Color.white ) ;
      mle_DESCRIPTION.setFont ( st_DESCRIPTION.getFont() ) ;
      version.setDescriptionWidth ( 150 ) ;
      partNumber.setDescriptionWidth ( 150 ) ;
      releaseDate.setDescriptionWidth ( 150 ) ;
      vl_PUB_TYPE.setDescriptionWidth ( 150 ) ;
      url.setDescriptionWidth ( 150 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mle_DESCRIPTION.addFocusListener ( this ) ;
      ef_PARTNUM.addFocusListener ( this ) ;
      ef_URL.addFocusListener ( this ) ;
      ef_VERSION.addFocusListener ( this ) ;
      ef_RELEASE_DATE.addFocusListener ( this ) ;
      cb_PUB_TYPE.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add GUI                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_DESCRIPTION ) ;
      add ( scr_DESCRIPTION ) ;
      add ( partNumber ) ;
      add ( version ) ;
      add ( url ) ;
      add ( releaseDate ) ;
      add ( vl_PUB_TYPE ) ;
   }


   /**
     *  This method will lay out and position all of our controls
     **/
   public void doLayout ( ) {
      Dimension size      =  getSize() ;
      int       x         = 0 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight () ;

      super.doLayout() ;

      st_DESCRIPTION.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      scr_DESCRIPTION.setBounds ( x, y, size.width, rowHeight*4 ) ;
      y+=(rowHeight*4 ) ;

      partNumber.setBounds ( x, y, 250, rowHeight ) ;
      y+=rowHeight ;

      version.setBounds ( x, y, 200, rowHeight ) ;
      y+=rowHeight ;

      url.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      releaseDate.setBounds ( x, y, 250, rowHeight ) ;
      y+=rowHeight ;

      vl_PUB_TYPE.setBounds ( x, y, 350, rowHeight ) ;
   }



   /**
     *  This method will set what document we are editing
     **/
   public void setDocument ( Document document ) {

      if ( this.document != null )
         this.document.removeDocumentPropertyListener ( this ) ;
      document.addDocumentPropertyListener ( this ) ;
      this.document = document ;

      mle_DESCRIPTION.setText ( document.getDescription () ) ;
      ef_URL.setText ( document.getURL () ) ;
      ef_RELEASE_DATE.setText ( DateSystem.dateFromDBDate ( document.getReleaseDate () ) ) ;
      ef_PARTNUM.setText ( document.getPartNumber () ) ;
      ef_VERSION.setText ( document.getVersion () ) ;
      cb_PUB_TYPE.setSelectedItem ( TypeList.getInstance().objectFromInd ( document.getPublicationType(), TypeList.TYPE_PUB ) ) ;
   }


   /**
     *  This method will be called when a component has lost focus. We will
     * use this time to save any data
     **/
   public void focusLost ( FocusEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   URL                                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == ef_URL ) {
         document.setURL ( ef_URL.getText() ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Version                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ef_VERSION ) {
         document.setVersion ( ef_VERSION.getText() ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Release Date                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ef_RELEASE_DATE ) {
         String releaseDate = ef_RELEASE_DATE.getText() ;

         processChanges = false ;

         if ( releaseDate != null )
            document.setReleaseDate ( releaseDate ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_DATE_ENTERED_WRONG ) ;

         processChanges = true ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Description                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == mle_DESCRIPTION )
         document.setDescription ( mle_DESCRIPTION.getText () );

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Part Number                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ef_PARTNUM ) {
         document.setPartNumber ( ef_PARTNUM.getText() ) ;
      }

   }

   /**
     *  This method will be called when a component has gained focus. We do nothing here
     **/
   public void focusGained ( FocusEvent e ) {
   }


   /**
     *  This method will be called when the user selects a publication type
     **/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == cb_PUB_TYPE )
         document.setPublicationType ( ((TypePubRec)cb_PUB_TYPE.getSelectedItem()).getInd() ) ;
   }


   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;

      cb_PUB_TYPE.setEnabled ( enable ) ;
      ef_URL.setEnabled ( enable ) ;
      ef_RELEASE_DATE.setEnabled ( enable ) ;
      ef_VERSION.setEnabled ( enable ) ;
      mle_DESCRIPTION.setEnabled ( enable ) ;
      ef_PARTNUM.setEnabled ( enable ) ;
   }

   public void documentChanged ( Document document, int property ) {
      if ( processChanges ) {
         if ( ( property == document.DESCRIPTION ) ||
              ( property == document.VERSION ) ||
              ( property == document.PART_NUMBER ) ||
              ( property == document.URL ) ||
              ( property == document.PUBLICATION_TYPE ) ||
              ( property == document.RELEASE_DATE ) ) {
            setDocument ( document ) ;
         }
      }
   }


   /**
     *  This method will return the JTextArea for the description
     **/
   public JTextArea getJTextArea () {
      return ( mle_DESCRIPTION ) ;
   }

   /**
     *  This method will return the JTextField for the version
     **/
   public JTextField getVersionJTextField () {
      return ( ef_VERSION ) ;
   }
} /* DownloadableInfoPanel */
