package com.ibm.nzna.projects.qit.doc.docedit ;       
/*-----------------------------------------------------------*/
/*   DraftInfoPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DraftInfoPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
/**
 * This panel holds information specific to the document draft. 
 * This information includes:<br><br>
 *
 * Requestor<br>
 * Content Owner<br>
 * Document Source<br>
 * Source Classification<br>
 * Document Status<br>
 *
 *
 * @version 1.0, 5/2/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DraftInfoPanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class DraftInfoPanel extends ListPanel implements DocumentPropertyListener, ActionListener, UserEntryPanelListener, AppConst, FocusListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  This is the JComboBox holding the metrics (or source classification if you prefer )
     **/
   private JComboBox cb_SOURCECLASS = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_METRIC ) );
   /**
     *  This is the ValueLabel for the Requestor
     **/
   private ValueLabel requestor = new ValueLabel ( Str.getStr ( STR_REQUESTOR ),
                                                   new JLabel ( "" ) ) ;

   /**
     *  This is the ValueLabel which holds the editable Content Owner
     **/
   private ValueLabel contentOwner = new ValueLabel ( "*"+Str.getStr ( STR_CONTENT_OWNER ),
                                                      new UserEntryPanel () ) ;

   /**
     *  This is the ValueLabel which holds the editable Document Source
     **/
   private ValueLabel source = new ValueLabel ( Str.getStr ( STR_SOURCE ),
                                                new JTextField ( "" ) ) ;

   /**
     *  This is the ValueLabel which holds the source classification
     **/
   private ValueLabel sourceClassification = new ValueLabel ( "*"+Str.getStr ( STR_SOURCE_CLASSIFICATION ),
                                                              cb_SOURCECLASS ) ;


   /**
     * This JComboBox holds all of the document status
     **/
   private JComboBox cb_STATUS = new JComboBox ( Status.getStatusList ( Status.STATUS_DOCUMENT ) ) ;

   /**
     * This is the Value label which holds the combo box for status
     **/
   private ValueLabel vl_STATUS = new ValueLabel ( "*"+Str.getStr ( STR_STATUS ),
                                                   cb_STATUS ) ;

   /**
     * This is the JTextField for the status date
     **/
   private DateEntryField ef_STATUS_DATE = new DateEntryField () ;

   /**
     * This is the Value Label for the status date
     **/
   private ValueLabel vl_STATUS_DATE = new ValueLabel ( "*"+Str.getStr ( STR_STATUS_DATE ) ,
                                                        ef_STATUS_DATE ) ;


   /**
     *  This is the document we are editing
     **/
   private Document document = null ;


   /**
     *  This Constructor creates our blank form
     **/
   public DraftInfoPanel () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADd listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ((UserEntryPanel)contentOwner.getComponent()).addUserEntryPanelListener ( this ) ;
      source.getComponent().addFocusListener ( this ) ;
      cb_SOURCECLASS.addActionListener ( this ) ;  
      ef_STATUS_DATE.addFocusListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( requestor ) ;
      add ( contentOwner ) ;
      add ( source ) ;
      add ( sourceClassification ) ;     
      add ( vl_STATUS ) ;
      add ( vl_STATUS_DATE ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the width of the description on our ValueLabels     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      requestor.setDescriptionWidth ( 150 ) ;
      contentOwner.setDescriptionWidth ( 150 ) ;
      source.setDescriptionWidth ( 150 ) ;
      sourceClassification.setDescriptionWidth ( 150 ) ;
      vl_STATUS.setDescriptionWidth ( 150 ) ;
      vl_STATUS_DATE.setDescriptionWidth ( 150 ) ;
      setPadding ( 5 ) ;

   }



   /**
     *  This method will populate all of our values with what
     * is in the Document object passed in
     **/
   public void setDocument ( Document document ) {

      if ( this.document != null )
         this.document.removeDocumentPropertyListener ( this ) ;
      document.addDocumentPropertyListener ( this ) ;
      this.document = document ;

      if ( document instanceof DocumentDraft ) {
         requestor.setValue ( UserSystem.getNameFromUserId ( ((DocumentDraft)document).getRequestor() ) ) ;
         contentOwner.getComponent().setEnabled ( true ) ;
         source.getComponent().setEnabled ( true ) ;
         sourceClassification.getComponent().setEnabled ( true ) ;
      } else {
         contentOwner.getComponent().setEnabled ( false ) ;
         source.getComponent().setEnabled ( false ) ;
         sourceClassification.getComponent().setEnabled ( false ) ;
      }

      ((UserEntryPanel)contentOwner.getComponent()).setText ( document.getOwner() ) ;
      source.setValue ( document.getSource() ) ;
      ((JComboBox)sourceClassification.getComponent()).setSelectedItem ( TypeList.getInstance().objectFromInd ( document.getMetric(),
                                                                                                                TypeList.TYPE_METRIC ) ) ;


      if ( document.getStatus() != null ) {
         DocumentStatus status = document.getStatus() ;

         /* Remove the listener for now to prevent it from being fired  */
         cb_STATUS.removeActionListener ( this ) ;
         cb_STATUS.setSelectedItem ( (TypeStatusRec) TypeList.getInstance().objectFromInd ( status.getStatus(),
                                                                                            TypeList.TYPE_STATUS ) ) ;
         cb_STATUS.addActionListener ( this ) ;

         ef_STATUS_DATE.setText ( status.getStatusDate() ) ;
      }
   }


   public void focusGained ( FocusEvent e ) {
   }

   /**
     *  This method will be called when a control we have loses focus. We will
     * use this time to save the data that the control has
     **/
   public void focusLost ( FocusEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Source                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == source.getComponent() )
         ((DocumentDraft)document).setSource ( (String) source.getValue() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Status Date                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == ef_STATUS_DATE )
         saveStatus() ;
   }

   /**
     *  This method will be called when the user has selected a metric in the drop down
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Source  Classification ( Metric )                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == cb_SOURCECLASS ) {
         if ( document instanceof DocumentDraft )
            ((DocumentDraft)document).setMetric ( ((TypeMetricRec)cb_SOURCECLASS.getSelectedItem()).getInd() ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Status                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == cb_STATUS ) {
         saveStatus() ;
      }


   }

   /**
     *  This method will be called when a valid user is entered as the content owner
     **/
   public void userSelected ( com.ibm.nzna.projects.qit.app.UserRec userRec ) {
      ((DocumentDraft)document).setOwner ( ((UserEntryPanel)contentOwner.getComponent()).getUserId() ) ;
   }


   /**
     *  This method will save the status information that we are showing
     **/
   public void saveStatus () {
      ((DocumentDraft)document).setStatus ( new DocumentStatus ( document.getDocInd(),
                                                                 ((TypeStatusRec)cb_STATUS.getSelectedItem()).getInd(),
                                                                 ef_STATUS_DATE.getText() ) ) ;
   }


   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;

      ef_STATUS_DATE.setEnabled ( enable ) ;
      cb_STATUS.setEnabled ( enable ) ;
      cb_SOURCECLASS.setEnabled ( enable ) ;
      contentOwner.getComponent().setEnabled ( enable ) ;
      source.getComponent().setEnabled ( enable ) ;
   }

   public void documentChanged ( Document document, int property ) {
      if ( ( property == document.STATUS ) ||
           ( property == document.SOURCE ) ||
           ( property == document.METRIC ) ||
           ( property == document.OWNER ) ) {
         setDocument ( document ) ;
      }
   }

} /* DraftInfoPanel */
