package com.ibm.nzna.projects.qit.doc.docedit ;
/*-----------------------------------------------------------*/
/*   EditDocument                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EditDocument                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.* ;
import java.util.* ;
/**
 * This class holds static routines useful when wanting
 * to start the DocEditPanel from anywhere in the Quest Input Tool
 *
  *
 * @version 1.0, 2/21/2001 
 * @author  David Lentz
 */


public class EditDocument implements AppConst {


   /**
     * This method will start the document editor editing the
     * DocumentRow object passed. If the DocumentRow object
     * passed holds a Document object (not a DocumentDraft Object)
     * then a DocumentDraft will be created and edited
     */
   public static void editDocument ( DocumentRow documentRow ) {
      int sqlCode = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read in the document and edit it                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlCode = documentRow.readDocument ( false ) ;

      switch ( sqlCode ) {
         case 0 :
            if ( documentRow.getDocument() instanceof DocumentDraft )
               editDocument ( (DocumentDraft)documentRow.getDocument() ) ;
            else
               editDocument ( documentRow.getDocument() ) ;
            break ;

         case 100 :
            GUISystem.printBox ( STR_INFO, STR_DOCUMENT_NOT_FOUND ) ;
            break ;

         default:
            LogSystem.log ( 1, "Cannot Read Document For Editing. SQLCode returned from DocumentRow.readDocument: "+sqlCode ) ;
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            break ;
      }

   } /* editDocument  */


   /**
     * This method will start the document editor editing the
     * Document object passed in. A DocumentDraft object will
     * be created.
     */
   public static void editDocument ( Document document ) {
      DocumentDraft draft = createDraft ( document ) ;

      if ( draft != null ) {
         editDocument ( draft ) ;
         readDocumentGraphics ( document ) ;
         System.out.println ( "After read. Rec Status:"+draft.getRecStatus()) ;
      }
   }

   /**
     *  This method will create a draft. It will also tell the user
     * about 'Draft Already Exist', 'Recycled' etc.. It will return
     * a draft if the user wants to edit it
     *
     * @param document The document to create a draft for
     **/
   public static DocumentDraft createDraft ( Document document ) {
      CreateDocumentDraft createDraft = DocumentDraft.createDraft ( document ) ;
      DocumentDraft       returnDraft = null ;

      switch ( createDraft.getReturnCode() ) {
         case CreateDocumentDraft.DRAFT_NONE :
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            break ;

         case CreateDocumentDraft.DRAFT_ALREADY_EXISTS :
            if ( GUISystem.printBox ( Str.getStr ( STR_QUESTION )+" - "+document.getFilename(), 
                                      UserSystem.getNameFromUserId ( createDraft.getDocumentDraft().getLastTouchedBy() )+" "+
                                      Str.getStr ( STR_EDIT_EXISTING_REQUEST ) ) ) {
               returnDraft = createDraft.getDocumentDraft() ;
            returnDraft.updateRecStatus ( DocumentDraft.REC_NOCHANGE ) ;
            returnDraft.updateRecStatus ( DocumentDraft.REC_UPDATE ) ;
            } else
               returnDraft = null ;
            break ;

         case CreateDocumentDraft.DRAFT_RECYCLED :
            if ( GUISystem.printBox ( Str.getStr ( STR_QUESTION ),
                                      UserSystem.getNameFromUserId ( document.getLastTouchedBy() )+" "+
                                      Str.getStr ( STR_EDIT_RECYCLED_REQUEST )+
                                      createDraft.getDocumentDraft().getComments() ) ) {
               returnDraft = createDraft.getDocumentDraft() ;
            }
            break ;

         case CreateDocumentDraft.DRAFT_CREATED :
            returnDraft = createDraft.getDocumentDraft() ;
            break ;
      }

      return( returnDraft ) ;
   } /* editDocument  */


   /**
     * This method will start the document editor editing the
     * DocumentDraft object passed in. If the DocumentDraft
     * object has not been read in then it will be read now. The
     * way of determining if a DocumentDraft object has been read
     * in is to check the title. If the title is NULL, the DocumentDraft
     * object will be read in
     */
   public static void editDocument ( DocumentDraft documentDraft ) {
      QuestPanel panel = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we are read in the draft, do so now              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( documentDraft.getTitle() == null )
         documentDraft.readFromDatabase() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the graphics                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( documentDraft.getRecStatus() != Document.REC_INSERT )
         readDocumentGraphics ( documentDraft ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our default values                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setDefaultValues ( documentDraft ) ;



      /*-----------------------------------------------*/
      /*                                               */
      /*  Find an editor already up with this document */
      /*                                               */
      /*-----------------------------------------------*/
      panel = WindowSystem.findQuestPanel ( documentDraft ) ;

      /*----------------------------------------------------------*/
      /*                                                          */
      /*  If we found a panel, just show it, otherwise create one */
      /*                                                          */
      /*----------------------------------------------------------*/
      if ( panel != null )
         WindowSystem.showPanel ( panel ) ;
      else
         WindowSystem.createPanel ( new DocumentEditor ( documentDraft ) ) ;
   } /* editDocument  */


   /**
     *  This method will set the default values for the document
     **/
   public static void setDefaultValues ( DocumentDraft documentDraft ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign Start Date and Create Date                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( documentDraft.getCreateDate() == null )
         documentDraft.setCreateDate ( new CDate ( CDate.DATE_DATE ).today() ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign the Requestor if it hasn't already been assigned */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( documentDraft.getRequestor() == null )
         documentDraft.setRequestor ( UserSystem.getUserId() ) ;
   }




   /**
     *  This method will read all of the graphics for a document draft
     **/
   private static void readDocumentGraphics ( DocumentDraft draft ) {
      Vector graphics = draft.getGraphics() ;

      if ( ( graphics != null ) && ( graphics.size() > 0 ) ) {
         int     i                  = 0 ;
         int     size               = graphics.size() ;
         Ftp     ftp                = QuestUtil.connectToTempGraphicFTP() ;
         String  remoteDirectory    = Constants.getStrConst ( "IT_IMAGELOCTEMP" ) ;
         String  localTempDirectory = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

         try {
            if ( ftp != null ) {
               DocumentGraphic graphic = null ;

               ftp.cd ( remoteDirectory ) ;

               while ( i < size ) {
                  graphic = (DocumentGraphic)graphics.elementAt(i) ;
                  LogSystem.log ( 1, "Downloading graphic "+graphic.getFilename() );
                  ftp.get ( graphic.getFilename(), localTempDirectory+File.separator+graphic.getFilename() ) ;
                  i++ ;
               }
            }


         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( "Info", "Cannot read document graphics" ) ;
         }
      }
   }



   /**
     *  This method will read in the graphics for an Active Document
     **/
   private static void readDocumentGraphics ( Document document ) {
      Vector graphics = document.getGraphics() ;

      if ( ( graphics != null ) && ( graphics.size() > 0 ) ) {
         int     i                  = 0 ;
         int     size               = graphics.size() ;
         Ftp     ftp                = QuestUtil.connectToProductionGraphicFTP() ;
         String  remoteDirectory    = Constants.getStrConst ( "IT_IMAGELOC" ) ;
         String  localTempDirectory = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

         try {
            if ( ftp != null ) {
               DocumentGraphic graphic = null ;

               ftp.cd ( remoteDirectory ) ;

               while ( i < size ) {
                  graphic = (DocumentGraphic)graphics.elementAt(i) ;
                  LogSystem.log ( 1, "Downloading graphic "+graphic.getFilename() );
                  ftp.get ( graphic.getFilename(), localTempDirectory+File.separator+graphic.getFilename() ) ;
                  i++ ;
               }
            }


         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( "Info", "Cannot read document graphics" ) ;
         }
      }
   }
}
