package com.ibm.nzna.projects.qit.doc.docedit ;       
/*-----------------------------------------------------------*/
/*   ExtBodyEditProcess                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ExtBodyEditProcess                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.sql.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.docedit.property.* ;
import com.ibm.nzna.projects.qit.wordproc.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * The ExtBodyEditProcess is an object whose job it is to control the process
 * that a 3rd party word processor holds while editing a document
 *
 *
 * @version 1.0, 4/25/2001 
 * @author  David Lentz
 */


public class ExtBodyEditProcess extends Object implements DataLengths, DocConst, AppConst, Runnable {

   /**
     * This string is the string QIT puts in all 3rd Party Word Processor docs
     **/
   private String notCreatedByQIT = "<!-- Created by 3rd party editor, NOT QUEST INPUT -->" ;


   /**
     * This is the document that we are editing
     **/
   private DocumentDraft document = null ;

   /**
     * This is the file that was being used to edit with
     **/
   private String documentFile = null ;


   /**
     * This is the listener we will notify when reimports are done
     **/
   private ExtBodyEditProcessListener listener = null ;

   /**
     * This value is TRUE if we are currently reimporting
     **/
   private boolean reimporting = false ;

   /**
     * This is the actual process that the editor will run in
     **/
   private Process  extProcess   = null ;



   /**
     * The default constructor takes a document and launchs the 3rd party
     * editor for that document
     *
     * @param document The document to edit the body of
     **/
   public ExtBodyEditProcess ( DocumentDraft document ) {

      this.document = document ;
      new Thread ( this ).start() ;
   }



   /**
     * This method will start the external process and wait for it
     *
     **/
   public void run () {
      String[] params       = new String[2] ;

      try {
         String content    = getDocumentBody() ;
         String newContent = null ;

         /*--------------------------------*/
         /*                                */
         /*  Create all of the temp. files */
         /*                                */
         /*--------------------------------*/
         document.generateKeys() ;
         documentFile = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator+document.getDocInd()+".html" ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Save the document to the hard drive                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         FileUtil.saveFile ( documentFile, content ) ;                                    

         /*----------------------------------*/
         /*                                  */
         /*  Launch and wait for the process */
         /*                                  */
         /*----------------------------------*/
         params[0] = PropertySystem.getString ( PropertySystem.PROPERTY_EXTERNAL_WORDPROC_EXE ) ;
         params[1] = documentFile ;

         extProcess = Runtime.getRuntime().exec ( params ) ;
         extProcess.waitFor() ;
         extProcess = null ;


         /*----------------------------------------------------------*/
         /*                                                          */
         /*  Now, compare the saved file contents and our contents,  */
         /* if they are different then ask the user if they would    */
         /* like to import. If they are the same, do nothing         */
         /*                                                          */
         /*----------------------------------------------------------*/
         try {
            newContent = FileUtil.readFile ( documentFile ) ;

            if ( ( !newContent.equals ( content ) ) && ( !reimporting ) ) {
               if ( GUISystem.printBox ( STR_QUESTION, STR_FILE_CONTENTS_CHANGED_REIMPORT ) )
                  reimportBody() ;
            }

         } catch ( FileNotFoundException e ) {
            GUISystem.printBox ( STR_ERROR, STR_EDIT_TEMP_FILE_NOT_FOUND ) ;
         }



      } catch ( InterruptedException ie ) {
      } catch ( Exception e ) {
         GUISystem.printBox ( STR_INFO, STR_CANNOT_START_EXT_WORD_PROC ) ;
      }

   }



   /**
     *  This method will return the body that should be edited by
     * the user
     *
     * @return String
     **/
   private String getDocumentBody () {
      String content = null ;

      if ( document.getDocumentType() == TypeList.DOCTYPE_SSD )
         content = document.getAdditionalInfo() ;
      else {
         DocumentBody body = document.getDocumentBody() ;

         if ( body != null )
            content = body.getBody() ;
      }

      if ( content == null )
         content = "<html><body>Erase Me<b>!</b></body></html>" ;

      return( content ) ;
   }






   /*------------------------------------------------------------*/
   /* reimportBody
    */
   /**
    *    This method will reimport the body from a
    *  previously saved body to the temp dir.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/              
   protected void reimportBody () {
      boolean   rc           = true ;
      StatusWin statusWin    = new StatusWin ( MainWindow.getInstance() ) ;

      reimporting = true ;
      LogSystem.beginLogSection ( "Reimporting Body" ) ;

      try {
         String newBody = FileUtil.readFile ( documentFile ) ;

         LogSystem.log ( 1, "Reimporting Body from "+documentFile ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Make sure we read something                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( newBody != null ) {
            String    newBodyUpper = newBody.toUpperCase() ;

            statusWin.setVisible ( true ) ;
            try {
               int docType = document.getDocumentType() ;


               statusWin.setText ( Str.getStr ( STR_IMPORTING_HTML ) ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Take everything inside of the body tags                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( newBodyUpper.indexOf ( "/BODY" ) >= 0 ) {
                  newBody = newBody.substring ( 0, newBodyUpper.indexOf ( "/BODY" ) ) ;
                  newBodyUpper = newBody.toUpperCase() ;
               }

               if ( newBodyUpper.indexOf ( "<BODY" ) >= 0 ) {
                  newBody = newBody.substring ( newBodyUpper.indexOf ( "<BODY" ), newBody.length() ) ;
                  newBodyUpper = newBody.toUpperCase() ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add our special tag in                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               newBody = Text.replaceAllStrInStr ( newBody, notCreatedByQIT, "" ) ;
               newBody=notCreatedByQIT+newBody ;
               newBodyUpper = newBody.toUpperCase() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get the doc links out of it                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               LogSystem.log ( 1, "Parsing Document Links" ) ;
               statusWin.setText ( "Parsing Doc Links" ) ;
               reimportDocLinks ( newBodyUpper, statusWin ) ;       

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Get the graphics                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               statusWin.setText ( "Parsing Graphics" ) ;
               LogSystem.log ( 1, "Parsing Graphics" ) ;
               newBody = reimportGraphics ( newBodyUpper, newBody ) ;


               LogSystem.log ( 1, "Setting Field Data" ) ;

               if ( docType != TypeList.DOCTYPE_SSD ) {
                  document.setDocumentBody ( new DocumentBody ( newBody ) ) ;
               }
               else {
                  if ( newBody.length() < 32000 )
                     document.setAdditionalInfo ( newBody ) ;
                  else {
                     GUISystem.printBox ( Str.getStr ( STR_INFO ), 
                                          Str.getStr ( STR_ADD_INFO_TOO_LONG )+" "+newBody.length()+" "+Str.getStr ( STR_BYTES_LONG ) ) ;
                     rc = false ;
                  }

               }


            } catch ( Exception e ) {
               LogSystem.log ( 1, e ) ;
            }

            LogSystem.log ( 1, "Disposing" ) ;

            if ( PropertySystem.getBool ( PropertySystem.PROPERTY_CLOSE_EXTERNAL_WORDPROC ) ) {
               if ( extProcess != null ) {
                  extProcess.destroy() ;
                  extProcess = null ;
               }
            }


            LogSystem.log ( 1, "Import Complete" ) ;

            statusWin.dispose() ;
            try {

               if ( rc ) {
                  new ReimportDlg ( MainWindow.getInstance(),
                                    document.getGraphics(),
                                    document.getDocumentLinks() ) ;
                  if ( listener != null )
                     listener.bodyReimportComplete() ;
               }
            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
            }
         }
      } catch ( FileNotFoundException e ) {
         GUISystem.printBox ( STR_ERROR, STR_EDIT_TEMP_FILE_NOT_FOUND ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
         GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      statusWin.dispose() ;
      LogSystem.endLogSection() ;
      reimporting = false ;
   }




   /*------------------------------------------------------------*/
   /* reimportDocLinks
    */
   /**
    *    This method will take a 3rd party supplied
    *  body and pull out all of the links
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void reimportDocLinks ( String newBody, StatusWin statusWin ) {
      int          index         = 0 ;
      String       tempStr       = null ;
      Vector       documentLinks = new Vector ( 1, 1 ) ;
      DocumentLink documentLink  = null ;

      while ( index >= 0 ) {
         index = newBody.indexOf ( "<A HREF", index ) ;

         if ( index>=0 ) {
            tempStr = newBody.substring ( index, newBody.indexOf ( "</A>", index ) ) ;

            if ( ( tempStr != null ) && ( tempStr.indexOf ( ".HTML" ) >= 0 ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Extract just the target of the URL link                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               tempStr = tempStr.substring ( tempStr.indexOf ( "=" )+1 ) ;
               tempStr = tempStr.substring ( 0, tempStr.indexOf ( ".HTML" ) ) ;
               tempStr = Text.replaceAllStrInStr ( tempStr, "\"", "" ) ;
               if ( tempStr.indexOf ( "./" ) >= 0 )
                  tempStr = Text.replaceAllStrInStr ( tempStr, "./", "" ) ;
               tempStr = tempStr.trim() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, try and find a document with the lndocid           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( tempStr.length() > 4 ) && ( tempStr.length() <= 11 ) ) {

                  statusWin.setText ( Str.getStr ( STR_READING )+" "+tempStr ) ;
                  documentLink = findDocForLink ( tempStr ) ;                              

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Ok, if we don't already have it, add it                 */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( documentLink != null ) {
                     if ( !documentLinks.contains ( documentLink ) )
                        documentLinks.addElement ( documentLink ) ;
                  }
               }
            }
            index++ ;
         }

         document.setDocumentLinks ( documentLinks ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* reimportDocGraphics
    */
   /**
    *    This method will reimport the documrent graphics
    *  
    *  
    *
    *  @param upperBody The body of the text in uppercase
    *  @param newBody The body of the text
    *  @param None
    *
    *  @return The altered body
    *//*---------------------------------------------------------*/
   private String reimportGraphics ( String upperBody, String newBody ) {
      int              index           = 0 ;
      String           graphicFilename = null ;
      Vector           graphicVec      = document.getGraphics() ;
      DocumentGraphic  graphic         = null ;
      String           tempDir         = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator ;
      int              lastIndex       = 0 ;

      /* First, mark each graphic for delete, we will assume that
         they removed every graphic, then we will go through each
         graphic in the list from the Word Processor and add each one,
         if we marked one for delete and it is still in the document,
         we will mark it for insert */
      if ( graphicVec != null ) {
         int i      = 0 ;
         int length = graphicVec.size() ;

         while ( i < length ) {
            ((DocumentGraphic)graphicVec.elementAt(i)).updateRecStatus ( DocumentGraphic.REC_NOCHANGE ) ;
            ((DocumentGraphic)graphicVec.elementAt(i)).updateRecStatus ( DocumentGraphic.REC_DELETE ) ;
            i++ ;
         }
      } else
         graphicVec = new Vector ( 1 ) ;


      /*---------------*/
      /*               */
      /*  Now, walk it */
      /*               */
      /*---------------*/
      while ( index >= 0 ) {

         lastIndex = index ;
         index = upperBody.indexOf ( "<IMG SRC", index ) ;

         if ( ( index > (upperBody.indexOf ( "<EMBED SRC", lastIndex ) ) ) ||
              ( index < 0 ) ) {
            if ( upperBody.indexOf ( "<EMBED SRC", lastIndex ) >= 0 )
               index = upperBody.indexOf ( "<EMBED SRC", lastIndex ) ;
         }

         if ( index>=0 ) {
            graphicFilename = newBody.substring ( index, newBody.indexOf ( ">", index ) ) ;

            if ( graphicFilename != null ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Extract just the target of the Graphic                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               graphicFilename = graphicFilename.substring ( graphicFilename.indexOf ( "=\"" )+2 ) ;
               graphicFilename = graphicFilename.substring ( 0, graphicFilename.indexOf ( "\"" ) ) ;
               graphicFilename = Text.replaceAllStrInStr ( graphicFilename, "\"", "" ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, add the graphic if we have a valid name and not    */
               /* a predetermined image location.                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( graphicFilename.length() > 0 ) && ( graphicFilename.indexOf ("###IMAGELOC###" ) == -1 ) ) {
                  if ( graphicFilename.startsWith ( "./" ) )
                     graphicFilename = graphicFilename.substring ( 2 ) ;

                  if ( WordProcUtil.notStampedGraphic ( graphicFilename ) ) {
                     DocumentGraphic currentGraphic = getGraphic ( graphicVec, graphicFilename ) ;

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Take off any paltform dependant path separators         */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     try {
                        if ( graphicFilename.indexOf ( "/" ) >= 0 )
                           graphicFilename = graphicFilename.substring ( graphicFilename.lastIndexOf ( "/" )+1,  graphicFilename.length() ) ;
                        if ( graphicFilename.indexOf ( "\\" ) >= 0 )
                           graphicFilename = graphicFilename.substring ( graphicFilename.lastIndexOf ( "\\" )+1, graphicFilename.length() ) ;
                     } catch ( Exception e ) {
                        if ( graphicFilename.indexOf ( "/" ) >= 0 )
                           graphicFilename = graphicFilename.substring ( graphicFilename.lastIndexOf ( "/" ),  graphicFilename.length() ) ;
                        if ( graphicFilename.indexOf ( "\\" ) >= 0 )
                           graphicFilename = graphicFilename.substring ( graphicFilename.lastIndexOf ( "\\" ), graphicFilename.length() ) ;
                     }

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Force the case to lower case                            */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     FileUtil.rename ( tempDir+graphicFilename, tempDir+(graphicFilename.toLowerCase()) ) ;
                     newBody = Text.replaceAllStrInStr ( newBody, graphicFilename, graphicFilename.toLowerCase() ) ;
                     graphicFilename = graphicFilename.toLowerCase() ;                                     



                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Make sure we do not already have this graphic,          */
                     /* if we do mark the graphic for Insert                      */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( currentGraphic != null ) {
                        currentGraphic.updateRecStatus ( DocumentGraphic.REC_NOCHANGE ) ;
                        currentGraphic.updateRecStatus ( DocumentGraphic.REC_INSERT ) ;
                     }

                     else {
                        /* Ok, add the graphic to the list of graphic associations */
                        if ( !verifyGraphic ( graphicFilename ) )
                           GUISystem.printBox ( Str.getStr ( STR_INFO ), 
                                                Str.getStr ( STR_CANNOT_FIND_GRAPHIC )+" "+tempDir+graphicFilename ) ;

                        else {

                           if ( graphicFilename.length() > LENGTH_GRAPHICNAME )
                              GUISystem.printBox ( STR_INFO, STR_GRAPHIC_NAME_TOO_LONG ) ;

                           else {
                              graphicVec.addElement ( graphic = new DocumentGraphic ( Counter.getCounter ( LogSystem.getInstance(),
                                                                                                           "GRAPHICIND" ),
                                                                                      graphicFilename ) ) ;

                              graphic.updateRecStatus ( DocumentGraphic.REC_NOCHANGE ) ;
                              graphic.updateRecStatus ( DocumentGraphic.REC_INSERT ) ;
                           } /* if image name is good  */
                        } /* else, the graphic is verified  */
                     } /* else, this is a new graphic */
                  } /* If this graphic is not a stamped graphic */
               } /* if graphicFilename.length is good */
            } /* if graphicFilename was good */
            index++ ;
         }
      }

      document.setGraphics ( graphicVec );
      return( newBody ) ;
   }

   /*------------------------------------------------------------*/
   /* findDocForLink
    */
   /**
    *    This method will find a document in the
    *  database for the lndocid passed in. It
    *  will populate and return a DocumentLink if found
    *
    *  @param notesId The Lotus Notes Id
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private DocumentLink findDocForLink ( String notesId ) {
      String     rtnName = "findDocForLink" ;
      SQLMethod  sql     = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      Statement  stmt    = null ;
      ResultSet  rs      = null ;
      DocumentLink retRec  = null ;


      try {
         stmt = sql.createStatement () ;

         rs = stmt.executeQuery ( "SELECT DOCIND, TITLE FROM TIGRIS.DOCUMENTS WHERE LNDOCID = '"+notesId+"' FOR FETCH ONLY" ) ;

         if ( rs.next() )
            retRec = new DocumentLink ( document.getDocInd(), rs.getInt(1), rs.getString(2) ) ;

      } catch ( Exception e ) {
         sql.rollBack() ;
      }

      sql.close() ;

      return( retRec ) ;
   }



   /**
     * This method will sort through a list of DocumentGraphic objects and
     * return the one matching the filename passed
     *
     * @param graphics The Vector of DocumentGraphic objects
     * @param filename The filename of the graphic to look for
     **/
   private DocumentGraphic getGraphic ( Vector graphics, String filename ) {
      DocumentGraphic retGraphic = null ;

      if ( ( graphics != null ) && ( graphics.size() > 0 ) ) {
         int i     = 0 ;
         int size  = graphics.size() ;

         while ( ( retGraphic == null ) && ( i < size ) ) {
            if ( ((DocumentGraphic)graphics.elementAt(i)).getFilename().equals ( filename ) )
               retGraphic = (DocumentGraphic)graphics.elementAt(i) ;
            i++ ;
         }
      }

      return( retGraphic ) ;
   }



   /*------------------------------------------------------------*/
   /* verifyGraphic
    */
   /**
    *    This method will verify that a graphic is in
    *  the correct temp directory and is read to upload
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if the graphic exists
    *//*---------------------------------------------------------*/
   private boolean verifyGraphic ( String filename ) {
      String tempDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator ;

      if ( ( filename.indexOf ( ")" ) >=0 ) ||
           ( filename.indexOf ( " " ) >=0 ) ||
           ( filename.indexOf ( "(" ) >=0 ) ||
           ( filename.indexOf ( "*" ) >=0 ) ||
           ( filename.indexOf ( "'" ) >=0 ) ||
           ( filename.indexOf ( "!" ) >=0 ) ||
           ( filename.indexOf ( "@" ) >=0 ) ||
           ( filename.indexOf ( "$" ) >=0 ) ||
           ( filename.indexOf ( "%" ) >=0 ) ||
           ( filename.indexOf ( "^" ) >=0 ) ||
           ( filename.indexOf ( "\"" ) >=0 ) ) {
         GUISystem.printBox ( Str.getStr (STR_INFO ), Str.getStr ( STR_INVALID_FILENAME )+". Filename:"+filename ) ;
         return( false ) ;
      }

      return( FileUtil.exists ( tempDir+filename ) ) ;
   }


   /**
     * This method will set our listener
     **/
   public void setExtBodyEditProcessListener ( ExtBodyEditProcessListener listener ) {
      this.listener = listener ;
   }
}
