package com.ibm.nzna.projects.qit.doc.docedit ;              
/*-----------------------------------------------------------*/
/*   FileAttachmentDlg                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FileAttachmentDlg                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.net.* ;
import java.io.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog allows the user to edit fields on a file attachment 
 *
 *
 * @version 1.0, 8/18/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FileAttachmentDlg                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class FileAttachmentDlg extends JDialog implements ActionListener, AppConst, DocConst, DataLengths {

   /**
     *  This is the DocumentFile object we are editing
     **/
   private DocumentFile documentFile = null ;

   /**
     *  This is the filename text
     **/
   private JLabel st_FILENAME = new JLabel () ;

   /**
     *  This is the size text
     **/
   private JLabel st_SIZE = new JLabel () ;

   /**
     *  this is the description field
     **/
   private JLabel st_DESCRIPTION = new JLabel ( "*"+Str.getStr ( STR_DESCRIPTION ) ) ;

   /**
     *  This is the JTextArea field for description
     **/
   private NoTabJTextArea mle_DESCRIPTION = new NoTabJTextArea ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_FILEGENINFO ) ) ;

   /**
     *  This is the version text
     **/
   private JLabel st_VERSION = new JLabel ( "*"+Str.getStr ( STR_VERSION ) ) ;

   /**
     *  This is the entry field for version
     **/
   private JTextField ef_VERSION = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_FILEVERSION ), "", 2 ) ;

   /**
     *  This is then text for Location
     **/
   private JLabel st_LOCATION = new JLabel ( "*"+Str.getStr ( STR_LOCATION ) ) ;

   /**
     *  This is the drop down for location
     **/
   private JComboBox cb_LOCATION = new JComboBox ( DocumentFile.getLocations()) ;

   /**
     *  This is the text for the URL
     **/
   private JLabel st_URL = new JLabel ( "*"+Str.getStr ( STR_URL ) ) ;

   /**
     *  This is the entry field for the URL
     **/
   private JTextField ef_URL = new JTextField ( "" ) ;

   /**
     *  This is the 'Ok' button
     **/

   private DButton pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     *  This is the Radio Button for Binary
     **/
   private JRadioButton rb_BINARY = new JRadioButton ( Str.getStr ( STR_BINARY ) ) ;

   /**
     *  This is the Radio BUtton for ASCII
     **/
   private JRadioButton rb_ASCII = new JRadioButton ( Str.getStr ( STR_ASCII ) ) ;

   /**
     *  This is the Radio Button Group for the ASCII and Binary Radio Buttons
     **/
   private ButtonGroup buttonGroup = new ButtonGroup () ;


   /**
     *  This is the JLabel for the FTP Mode
     **/
   private JLabel st_FTPMODE = new JLabel ( Str.getStr ( STR_FTP_MODE ) ) ;

   /**
     *  This is the file refresh button
     **/
   private DButton pb_FILE_REFRESH = new DButton ( Str.getStr ( STR_REFRESH_FILE ) ) ;

   /**
     *  This is the cancel button
     **/
   private DButton pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;

   /**
     *  This is the button panel which holds all of our buttons
     **/
   private ButtonPanel buttonPanel = new ButtonPanel () ;

   /**
     *  This is the counter for the description text
     **/
   private JTextAreaCounter mle_DESCRIPTION_COUNTER = null ;





   /**
     *  Default construtor takes a DocumentFile object to edit
     **/
   public FileAttachmentDlg ( Frame parent, DocumentFile documentFile ) {
      super ( parent, Str.getStr ( STR_FILE_ATTACHMENT ), true ) ;

      this.documentFile = documentFile ;

      createGUI () ;
      refreshData() ;

      setResizable ( false ) ;
      setSize ( 400, 390 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;
   }


   /**
     *  This method will add our GUI
     **/
   private void createGUI () {
      Container contentPane = getContentPane() ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create any GUI                                          */     
      /*                                                           */
      /*-----------------------------------------------------------*/
      mle_DESCRIPTION_COUNTER = new JTextAreaCounter ( mle_DESCRIPTION ) ;
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mle_DESCRIPTION.setFont ( pb_OK.getFont() ) ;
      mle_DESCRIPTION.setLineWrap ( true ) ;
      mle_DESCRIPTION.setWrapStyleWord ( true ) ;
      mle_DESCRIPTION.setBorder ( GUISystem.loweredBorder ) ;
      st_FILENAME.setFont ( FontSystem.largeTitleFont ) ;      
      buttonGroup.add ( rb_ASCII ) ;
      buttonGroup.add ( rb_BINARY ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_LOCATION.addActionListener     ( this ) ;
      pb_OK.addActionListener           ( this ) ;
      pb_FILE_REFRESH.addActionListener ( this ) ;
      pb_CANCEL.addActionListener       ( this ) ;
      ef_VERSION.addActionListener      ( this ) ;
      ef_URL.addActionListener          ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADd our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_FILENAME ) ;
      contentPane.add ( st_SIZE ) ;
      contentPane.add ( st_DESCRIPTION ) ;
      contentPane.add ( mle_DESCRIPTION ) ;
      contentPane.add ( mle_DESCRIPTION_COUNTER) ;
      contentPane.add ( st_FTPMODE ) ;
      contentPane.add ( rb_ASCII ) ;
      contentPane.add ( rb_BINARY ) ;
      contentPane.add ( st_VERSION ) ;
      contentPane.add ( ef_VERSION ) ;
      contentPane.add ( st_LOCATION ) ;
      contentPane.add ( cb_LOCATION ) ;
      contentPane.add ( st_URL ) ;
      contentPane.add ( ef_URL ) ;
      contentPane.add ( buttonPanel ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_FILE_REFRESH ) ;
      buttonPanel.add ( pb_CANCEL ) ;

      GUISystem.setPropertiesOnPanel ( contentPane ) ;

   }


   /**
     *  This method will lay out our components
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight () ;

      super.doLayout () ;

      st_FILENAME.setBounds ( x, y, size.width-(x*4), rowHeight*2 ) ;
      y+=(rowHeight*2) ;

      st_SIZE.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight*2 ;

      st_DESCRIPTION.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight;

      mle_DESCRIPTION.setBounds ( x, y, size.width-(x*4), rowHeight*4 ) ;
      y+=rowHeight*4 ;

      mle_DESCRIPTION_COUNTER.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight ;

      st_FTPMODE.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight ;

      rb_ASCII.setBounds ( x, y, (size.width/2)-(x*4), rowHeight ) ;
      rb_BINARY.setBounds ( size.width/2, y, (size.width/2)-(x*4), rowHeight ) ;
      y+=rowHeight ;

      st_VERSION.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight ;

      ef_VERSION.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight ;

      st_LOCATION.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight ;

      cb_LOCATION.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight ;

      st_URL.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight ;

      ef_URL.setBounds ( x, y, size.width-(x*4), rowHeight ) ;
      y+=rowHeight ;

      buttonPanel.setBounds ( x, size.height-55, size.width-(x*4), 25 ) ;
   }



   /**
     *  This method will populate all of our fields
     **/
   private void refreshData () {
      String lastFileLocation = PropertySystem.getString ( PROPERTY_LAST_FILE_LOC ) ;

      if ( lastFileLocation == null )
         lastFileLocation = DocumentFile.getLocations()[0] ;

      st_FILENAME.setText ( documentFile.getFilename() ) ;
      st_SIZE.setText ( documentFile.getSize()+" bytes" ) ;

      /*------------------------------------------------------------------*/
      /*                                                                  */
      /*   If this file is an insert, only try to populate the FTPMode    */
      /*                                                                  */
      /*------------------------------------------------------------------*/
      if ( documentFile.getFilename().equals ( "txt" ) )
         rb_ASCII.setSelected ( true ) ;
      else
         rb_BINARY.setSelected ( true ) ;

      if ( documentFile.getDescription() != null )
         mle_DESCRIPTION.setText ( documentFile.getDescription() ) ;

      if ( documentFile.getVersion() != null )
         ef_VERSION.setText ( documentFile.getVersion() ) ;

      if ( documentFile.getLocation() != null )
         cb_LOCATION.setSelectedItem ( documentFile.getLocation() ) ;
      else
         cb_LOCATION.setSelectedItem ( lastFileLocation ) ;

      if ( documentFile.getURL() != null )
         ef_URL.setText ( documentFile.getURL () ) ;
   }

   /**
     *  This method will respond to users hitting Ok/Refresh File/Cancel
     **/
   public void actionPerformed ( ActionEvent e ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Location combo box                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == cb_LOCATION ) {
         String newLocation = (String)cb_LOCATION.getSelectedItem() ;
         String url         = "" ;

         if ( newLocation.indexOf ( "ate/thinkpad" ) >=0 )
            url = "ftp://thinkpad:ftp4tpad@ftp.pc.ibm.com/pub/"+newLocation +"/"+ documentFile.getFilename() ;
         else if ( newLocation.indexOf ( "ate/network" ) >=0 )
            url = "ftp://network:n3tw0rk@ftp.pc.ibm.com/pub/"+newLocation +"/"+ documentFile.getFilename()  ;
         else if ( newLocation.indexOf ( "ate/options" ) >=0 )
            url = "ftp://options:opt10ns@ftp.pc.ibm.com/pub/"+newLocation +"/"+ documentFile.getFilename()  ;
         else
            url = "ftp://ftp.pc.ibm.com/pub/pccbbs/"+newLocation +"/"+ documentFile.getFilename() ;

         ef_URL.setText ( url ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Cancel                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CANCEL )
         dispose() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok                                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_OK ) ||
                ( e.getSource() == ef_VERSION ) ||
                ( e.getSource() == ef_URL ) ) {
         if ( saveData() )
            dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh File                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_FILE_REFRESH ) {
         if ( refreshFile() )
            pb_CANCEL.setEnabled ( false ) ;
      }


   }



   /**
     *  This method will save all of the data in our fields to the
     * DocumentFile object
     *
     * @return TRUE if all data was saved, FALSE if not
     **/
   private boolean saveData () {
      String  description = mle_DESCRIPTION.getText() ;
      String  version     = ef_VERSION.getText() ;
      String  location    = (String)cb_LOCATION.getSelectedItem() ;
      String  url         = ef_URL.getText() ;
      boolean saved       = false ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Check if data was good                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( description == null ) || ( description.length() == 0 ) ) {
         GUISystem.printBox ( this, STR_ERROR, STR_PLEASE_ENTER_FILE_DESCRIPTION ) ;
         mle_DESCRIPTION.requestFocus() ;
      } else if ( ( version == null ) || ( version.length() == 0 ) ) {
         GUISystem.printBox ( this, STR_ERROR, STR_PLEASE_ENTER_FILE_VERSION ) ;
         ef_VERSION.requestFocus() ;
      } else if ( ( url == null ) || ( url.length() == 0 ) ) {
         GUISystem.printBox ( this, STR_ERROR, STR_PLEASE_ENTER_FILE_URL ) ;
         ef_URL.requestFocus() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, data was good enough to save                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         documentFile.setURL         ( url.trim() ) ;
         documentFile.setDescription ( description.trim() ) ;
         documentFile.setVersion     ( version.trim() );
         documentFile.setLocation    ( location ) ;
         documentFile.setFTPMode     ( rb_BINARY.isSelected() ? 0 : 1 ) ;
         saved = true ;
      }

      return( saved ) ;
   }


   /**
     *  This method will refresh the file that the DocumentFile is associated with.
     **/
   private boolean refreshFile () {
      boolean refreshed = false ;
      File[]  openFile  = GUISystem.getFileOpen ( GUISystem.getParentDefWin ( this ), 
                                                  Str.getStr ( STR_FILE_ATTACHMENT ), 
                                                  PropertySystem.getString ( PropertySystem.PROPERTY_IMPORT_DIRECTORY ) ) ;

      if ( openFile != null ) {
         try {
            File   newFile  = new File ( openFile[0].toString() ) ;
            String filename = Text.stripPath ( openFile[0].toString() ) ;                                                             

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Save the location for next time                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            PropertySystem.putString ( PropertySystem.PROPERTY_IMPORT_DIRECTORY, openFile[1].toString() ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Only allow a refresh if the filename is the same        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !filename.equals ( documentFile.getFilename() ) ) {
               GUISystem.printBox ( STR_INFO, STR_CANNOT_REFRESH_FILE_WITH_DIFF_FILENAME ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Allow                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
               if ( newFile.length() == documentFile.getSize() )
                  GUISystem.printBox ( this, STR_INFO, STR_REFRESH_FILE_WITH_SAME_SIZE ) ;

               documentFile.setLocalFilename ( newFile.toString() ) ;
               documentFile.setSize ( newFile.length() );
               refreshed = true ;
            }
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( this, STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
         }
      }

      return( refreshed ) ;
   }
} /* FileAttachmentDlg */
