package com.ibm.nzna.projects.qit.doc.docedit ;   
/*-----------------------------------------------------------*/
/*   LocalePanel                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LocalePanel                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The LocalePanel holds all of the Geography and Country 
 * associations. This panel also holds 3 buttons to select, remove
 * countries and to associate to world wide.
 *
 * @version 1.0, 5/2/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   LocalePanel                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class LocalePanel extends DPanel implements DocumentPropertyListener, ListSelectionListener, DocConst, LocaleDlgListener, ActionListener, AppConst {

   /**
     *  This is the JList that holds all of the associations
     **/
   private JList locale = new JList () ;

   /**
     *  This is the JScrollPane which holds the locale JList
     **/
   private JScrollPane scroll  = new JScrollPane ( locale )  ;

   /**
     *  This is the small piece of text telling the user what to
     * do when a document is World Wide
     **/
   private JLabel help = new JLabel ( Str.getStr ( STR_LOCALE_PANEL_HELP ) ) ;

   /**
     *  This is the DButton that launches the pop up which
     * allows the user to select geographies and countries
     **/
   private DButton select = new DButton ( Str.getStr ( STR_SELECT ) )  ;

   /**
     *  This is the DButton that removes the selected
     * items from the list
     **/
   private DButton remove = new DButton ( Str.getStr ( STR_REMOVE ) ) ;

   /**
     *  This is the 'World Wide' button that automagically
     * associates a document to world wide
     **/
   private DButton worldWide = new DButton ( Str.getStr ( STR_WORLDWIDE ) ) ;

   /**
     *  This is the JPanel which holds the 3 buttons
     **/
   private ListPanel buttonPanel = new ListPanel () ;


   /**
     * This is our copy of the document record
     */
   private Document document = null ;




   /**
     *  This constructor creates a blank, default Locale panel
     **/
   public LocalePanel () {
      JPanel sidePanel = new JPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the properties                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      buttonPanel.setBackground ( Color.white ) ;
      locale.setModel ( new DefaultListModel() ) ;
      remove.setEnabled ( false ) ;
      help.setVisible ( false ) ;
      sidePanel.setBackground ( Color.white ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      select.addActionListener ( this ) ;
      remove.addActionListener ( this ) ;
      worldWide.addActionListener ( this ) ;
      locale.addListSelectionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sidePanel.setLayout ( new BorderLayout () ) ;
      buttonPanel.add ( select ) ;
      buttonPanel.add ( remove ) ;
      buttonPanel.add ( worldWide ) ;
      buttonPanel.setPreferredSize ( new Dimension ( 75, 100 ) ) ;
      setPreferredSize ( new Dimension ( 500, 200 ) ) ;
      setLayout ( new BorderLayout () );
      add ( GUISystem.getRequiredJLabel(), BorderLayout.NORTH ) ;
      add ( scroll,    BorderLayout.CENTER ) ;
      add ( sidePanel, BorderLayout.EAST ) ;
      sidePanel.add ( buttonPanel, BorderLayout.CENTER ) ;
      sidePanel.add ( help,        BorderLayout.SOUTH ) ;

      sidePanel.setPreferredSize ( new Dimension ( 75, 200 ) ) ;
      sidePanel.setMaximumSize ( new Dimension ( 75, 200 ) ) ;
      sidePanel.setMinimumSize ( new Dimension ( 75, 200 ) ) ;
   }



   /**
     *  This method will set our information from the document passed in
     **/
   public void setDocument ( Document document ) {

      if ( this.document != document ) {

         if ( this.document != null )
            this.document.removeDocumentPropertyListener ( this ) ;

         document.addDocumentPropertyListener ( this ) ;

         this.document = document ;
      }

      /* Now, populate our list  */
      populateList ( ) ;
   }


   /**
     * This method will take a LocaleRec and populate our list with all
     * of it's contents
     *
     * @param LocaleRec The localeRec to use when populating
     */
   public void populateList ( ) {
      DefaultListModel model     = new DefaultListModel() ; //((DefaultListModel)locale.getModel()) ;
      LocaleRec        localeRec = document.getLocale() ;

      /* First, clear out the old list box  */
      model.clear() ;

      if ( localeRec != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Enable/Disable the World Wide Button and the Select/Help*/
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( isEnabled() ) {
            worldWide.setEnabled ( !localeRec.isWorldWide() ) ;
            select.setEnabled ( !localeRec.isWorldWide() ) ;
            help.setVisible ( localeRec.isWorldWide() ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is world wide, simply put the string in there   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( localeRec.isWorldWide() ) {
            Vector vec = new Vector ( 1 ) ;

            model.addElement ( "<html><body><font size=5><b>"+Str.getStr ( STR_WORLDWIDE )+"</b></font></body></html>" ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   otherwise, add all of the items                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            Vector allGeos   = localeRec.getGeoListContainingAll() ;
            Vector countries = localeRec.getSingleCountryList() ;                 


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add all of the Geographies                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( allGeos != null ) && ( allGeos.size() > 0 ) ) {
               int i       = 0 ;
               int size    = allGeos.size() ;

               while ( i < size ) {
                  if ( ((TypeGeoRec)allGeos.elementAt(i)).valid() ) {
                     TypeGeoRec geoRec = (TypeGeoRec)((TypeGeoRec)allGeos.elementAt(i)).clone() ;

                     geoRec.setEnhanceOutput ( true ) ;
                     model.addElement ( geoRec ) ;
                  }
                  i++ ;
               }
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add the associated countries                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( countries != null ) && ( countries.size() > 0 ) ) {
               int i       = 0 ;
               int size    = countries.size() ;

               while ( i < size ) {
                  //if ( ((TypeCountryCodeRec)countries.elementAt(i)).valid() )
                  model.addElement ( countries.elementAt(i) ) ;
                  i++ ;
               }
            }

         }

      }


      if ( model.size() == 0 )
         model.addElement ( "<html><body><font size=1 face=\"Helvetica, Arial\">No Countries or Geographies, press 'Select' to<br>add countries or geographies</font></body></html>" ) ;

      locale.setModel ( model ) ;
   } /* populate list  */


   /**
     * This method will be called when the user hits the 'Select', 'Remove', or
     * World Wide buttons
     */
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The 'Select' Push Button                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == select ) {
         LocaleDlg dlg = new LocaleDlg ( GUISystem.getParentDefWin ( this ) ) ;

         dlg.setLocaleDlgListener ( this ) ;
         dlg.waitForUser() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The 'Remove' Push Button                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == remove ) {
         removeItem() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   World Wide                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource () == worldWide ) {
         LocaleRec localeRec = document.getLocale() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we do not have a LocaleRec yet, create one           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( localeRec == null ) {
            localeRec = new LocaleRec () ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set world wide                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         localeRec.setWorldWide ( true ) ;
         populateList ( ) ;
         document.setLocale ( localeRec ) ;


      }


   }

   /**
     *  This method will be called when a country list is selected by the user
     * in the LocaleDlg pop up
     *
     * @param The countries
     **/
   public void localeDlgAddCountries ( Object[] countries ) {
      try {
         LocaleRec localeRec = document.getLocale() ;                                    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add all of the selected countries                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         for ( int i = 0; i < countries.length; i++ )
            localeRec.addCountry ( (TypeCountryCodeRec)countries[i] ) ;       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Refresh our data                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         document.setPropertyChangeEnabled ( false ) ;
         document.setLocale ( localeRec ) ;
         populateList ( ) ;
         document.setPropertyChangeEnabled ( true ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }

   /**
     *  This method will be called when a geography is selected by the user in
     * the LocaleDlg pop up
     * 
     * @param The geography
     **/
   public void localeDlgAddGeo ( TypeGeoRec geoRec ) {

      try {
         LocaleRec localeRec = document.getLocale() ;                                    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the geography and refresh                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         document.setPropertyChangeEnabled ( false ) ;
         localeRec.addGeography ( geoRec ) ;
         document.setLocale ( localeRec ) ;
         populateList ( ) ;
         document.setPropertyChangeEnabled ( true ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }



   /**
     *  This method will be called when a selection is changed in the
     * list of countries/geos
     **/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( isEnabled() ) {

         /* If the selected item is a string, disable it if it does not have 'No Countries' in it  */
         if ( locale.getSelectedValue() instanceof String )
            remove.setEnabled ( ((String)locale.getSelectedValue()).indexOf ( "No Countries" ) == -1 ) ;
         else if ( locale.getSelectedValue() != null )
            remove.setEnabled ( true ) ;
      }
   }


   /**
     *  This method will remove the item(s) selected in the
     * locale list
     **/
   private void removeItem () {
      Object[]  items     = locale.getSelectedValues() ;
      LocaleRec localeRec = document.getLocale() ;

      if ( ( items != null ) && ( items.length > 0 ) && ( localeRec != null ) ) {

         for ( int i = 0; i < items.length; i++ ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Remove the Geography                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( items[i] instanceof TypeGeoRec )
               localeRec.removeGeography ( (TypeGeoRec) items[i] ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Remove the Country                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( items[i] instanceof TypeCountryCodeRec )
               localeRec.removeCountry ( (TypeCountryCodeRec) items[i] ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   The only string that would be here is the 'World Wide'  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( items[i] instanceof String )
               localeRec.setWorldWide ( false ) ;
         }

         document.setPropertyChangeEnabled ( false ) ;
         document.setLocale ( localeRec ) ;
         populateList ( ) ;
         document.setPropertyChangeEnabled ( true ) ;
      } /* if we have selected items and a locale rec  */
   } /* removeItems  */

   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;
      worldWide.setEnabled ( enable ) ;
      remove.setEnabled ( enable ) ;
      select.setEnabled ( enable ) ;

   }


   public void documentChanged ( Document document, int property ) {
      if ( property == document.LOCALE )
         populateList() ;
   }
} /* LocalePanel */
