package com.ibm.nzna.projects.qit.doc.docedit ;    
/*-----------------------------------------------------------*/
/*   ProMailPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProMailPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel holds all of the fields for a Document's Proactive eMail 
 *
 *
 * @version 1.0, 8/19/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProMailPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProMailPanel extends DPanel implements Runnable, ActionListener, DataLengths, AppConst, DocConst, FocusListener {

   /**
     *  This thread calls the 'checkDate' method
     **/
   private static final String THREAD_CHECK_DATE = "checkDate" ;

   /**
     *  This is the Proactive eMail Date field
     **/
   private DateEntryField ef_DATE = new DateEntryField () ;

   /**
     *  This is the Value Label for the date
     **/
   private ValueLabel vl_DATE = new ValueLabel ( Str.getStr ( STR_WHEN_DOCUMENT_IS_MAILED ),
                                                 ef_DATE ) ;

   /**
     * Determines if any action needs to be processed in Focus or Action
     **/
   private boolean processActions = true ;

   /**
     *  This is the Check box for eMailable
     **/
   private JCheckBox ck_EMAILABLE = new JCheckBox ( "" ) ;

   /**
     *  This is the value label for the check box
     **/
   private ValueLabel vl_EMAILABLE = new ValueLabel ( Str.getStr ( STR_THIS_DOCUMENT_IS_MAILED ),
                                                      ck_EMAILABLE ) ;

   /**
     *  This is the eMail Body text
     **/
   private JLabel st_BODY = new JLabel ( Str.getStr ( STR_EMAIL_BODY ) ) ;

   /**
     *  This is the JTextArea for the body
     **/
   private NoTabJTextArea mle_BODY = new NoTabJTextArea ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_DOCSUMMARY ) ) ;

   /**
     *  This is the JSCrollPane which holds the body
     **/
   private JScrollPane scr_BODY = new JScrollPane ( mle_BODY ) ;

   /**
     *  This is the document we are working on
     **/
   private Document document = null ;

   /**
     *  This is the counter for the summary
     **/
   private JTextAreaCounter mle_BODY_COUNTER = new JTextAreaCounter ( mle_BODY ) ;




   /**
     *  Default Constructor
     **/
   public ProMailPanel () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      setPreferredSize ( new Dimension ( 470, 200 ) ) ;
      vl_EMAILABLE.setDescriptionWidth ( 200 ) ;
      vl_DATE.setDescriptionWidth ( 200 ) ;
      ef_DATE.setEnabled ( false ) ;
      mle_BODY.setEnabled ( false ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_DATE.addFocusListener ( this ) ;
      mle_BODY.addFocusListener ( this ) ;
      ck_EMAILABLE.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add GUI                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( vl_EMAILABLE ) ;
      add ( vl_DATE ) ;
      add ( st_BODY ) ;
      add ( scr_BODY ) ;
      add ( mle_BODY_COUNTER );
   }


   /**
     *  This method will layout and position all of our components 
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 0 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight () ;

      vl_EMAILABLE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      vl_DATE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      st_BODY.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      scr_BODY.setBounds ( x, y, size.width-(x*2), size.height-y-15 ) ;

      mle_BODY_COUNTER.setBounds ( x, size.height-15, size.width-(x*2), 15 ) ;
   }


   /**
     *  This method will set the document we are working on
     **/
   public void setDocument ( Document document ) {
      this.document = document ;

      processActions = false ;
      if ( document.getEMailable() ) {
         ef_DATE.setText ( document.getEMailableDate() ) ;
         mle_BODY.setText ( document.getEMailComments() ) ;
         ef_DATE.setEnabled ( true ) ;
         mle_BODY.setEnabled ( true ) ;
         ck_EMAILABLE.setSelected ( true ) ;
      } else {
         ef_DATE.setEnabled ( false ) ;
         mle_BODY.setEnabled ( false ) ;
         ck_EMAILABLE.setSelected ( false ) ;
         if ( document.getEMailableDate() != null )
            ef_DATE.setText ( document.getEMailableDate() ) ;

         if ( document.getEMailComments() != null )
            mle_BODY.setText ( document.getEMailComments() ) ;
      }
      processActions = true ;
   }

   /**
     *  This method will be called when the user hits our check box
     **/
   public void actionPerformed ( ActionEvent e ) {
      if ( processActions ) {
         if ( isEnabled() ) {
            if ( e.getSource() == ck_EMAILABLE ) {
               document.setEMailable ( ck_EMAILABLE.isSelected() ) ;
               setDocument ( document ) ;
               if ( ck_EMAILABLE.isSelected() ) {
                  ef_DATE.requestFocus() ;
                  ef_DATE.selectAll() ;
               }
            }
         }
      }
   }


   /**
     *  This method will be called when one of our fields loses focus. We
     * will save data here
     **/
   public void focusLost ( FocusEvent e ) {

      if ( processActions ) {
         if ( isEnabled() ) {
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Date                                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( e.getSource() == ef_DATE )
               new Thread ( this, THREAD_CHECK_DATE ).start() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   body                                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( e.getSource() == mle_BODY )
               document.setEMailComments ( mle_BODY.getText() ) ;

         }
      }
   }

   /**
     *  This method will be called when one of our components gets focus.
     **/
   public void focusGained ( FocusEvent e ) {
   }

   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;

      ck_EMAILABLE.setEnabled ( enable ) ;

      if ( document != null ) {
         if ( !document.getEMailable() )
            enable = false ;
      }

      ck_EMAILABLE.removeActionListener ( this ) ;
      ck_EMAILABLE.addActionListener ( this ) ;
      mle_BODY.setEnabled ( enable ) ;
      ef_DATE.setEnabled ( enable ) ;

   }


   /**
     *  This method will return an instance of the JTextArea control we use to allow editing of the
     * proactive eMail body
     **/
   protected JTextArea getJTextArea () {
      return( mle_BODY ) ;
   }

   /**
     *  this method will perform threads based on the name of the thread
     **/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      if ( operation.equals ( THREAD_CHECK_DATE ) )
         checkDate() ;
   }

   /**
     *  This method will check that the date entered is in the future and is correct
     **/
   private void checkDate () {
      String date = ef_DATE.getText() ;

      if ( date != null ) {
         if ( CDate.isFutureDate ( date, CDate.DATE_DATE ) )
            document.setEMailableDate ( date ) ;
         else {
            GUISystem.printBox ( STR_INFO, STR_EMAIL_DATE_IS_FUTURE ) ;
            ef_DATE.setText ( "" ) ;
            ef_DATE.requestFocus() ;
            ef_DATE.selectAll() ;
            }
      } else {
         GUISystem.printBox ( STR_INFO, STR_DATE_ENTERED_WRONG ) ;
         ef_DATE.setText ( "" ) ;
         ef_DATE.requestFocus() ;
         ef_DATE.selectAll() ;
      }
   }
} /* ProMailPanel */
