package com.ibm.nzna.projects.qit.doc.docedit ;    
/*-----------------------------------------------------------*/
/*   ProductPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;   
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * The product panel allows the user to add/remove Brands, Families,
 * machines, models or parts to a document. It also shows the user
 * all currently associated brands, families, machines, models and
 * part numbers.
 *
 *
 * @version 1.0, 7/13/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductPanel extends DPanel implements ActionListener, AppConst, DocConst, Runnable {

   /**
     *  This is the JLabel which shows the output
     **/
   private JTextArea st_OUTPUT      = null ;

   /**
     *  This is the Select button which allows the add/remove of BFTSMP
     **/
   private DButton pb_SELECT     = null ;


   /**
     *  This is the current document
     **/
   private Document document = null ;



   /**
     *  This constructor initializes everything but does not refresh any
     * data. To refresh the data please call 'setDocument'
     **/
   public ProductPanel () {
      ListPanel   listPanel = new ListPanel () ;     
      JScrollPane scroll    = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_OUTPUT = new JTextArea ( Str.getStr ( STR_NO_PRODUCTS_SELECTED ) ) ;
      pb_SELECT = new DButton ( Str.getStr ( STR_SELECT ) ) ;
      listPanel = new ListPanel (); 
      scroll    = new JScrollPane ( st_OUTPUT ) ;

      /*----------------*/
      /*                */
      /*  Add Listeners */
      /*                */
      /*----------------*/
      pb_SELECT.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_OUTPUT.setEditable ( false ) ;
      st_OUTPUT.setWrapStyleWord ( true ) ;
      st_OUTPUT.setLineWrap ( true ) ;
      st_OUTPUT.setFont ( pb_SELECT.getFont() ) ;
      listPanel.setBackground ( Color.white ) ;
      scroll.setBackground ( Color.white ) ;
      listPanel.setPreferredSize ( new Dimension ( 75, 100 ) ) ;  
      setBackground ( Color.white ) ;
      st_OUTPUT.setBorder ( GUISystem.blackBorder ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listPanel.add ( pb_SELECT ) ;
      setLayout ( new BorderLayout() ) ;
      add ( GUISystem.getRequiredJLabel(), BorderLayout.NORTH ) ;
      add ( scroll,                        BorderLayout.CENTER ) ;
      add ( listPanel,                     BorderLayout.EAST ) ;

      setPreferredSize ( new Dimension ( 500, 150 ) ) ;
   }


   /**
     *  This method will reset the data that we are displaying
     *
     * @param document The document to refresh with
     **/
   protected void setDocument ( Document document ) {
      this.document = document ;
      new Thread ( this ).start() ;
   }


   /**
     *  This thread takes all Applicable Brands, Families, Machines, Subcategories
     * and all directly associated Models and Part Numbers and forms the HTML
     * that is needed to show in this control.
     **/
   public void run () {
      String output = "" ;

      /*-----------------------------------------------------------------*/
      /*                                                                 */
      /*   First, grab all of the families that are associated to all    */
      /*                                                                 */
      /*-----------------------------------------------------------------*/
      output+=formFamilyHTML()+"\n\n" ;
      output+=formMachineHTML()+"\n\n" ;
      output+=formModelHTML() ;
      if ( output.length() > 1 )
         st_OUTPUT.setText ( output ) ;
      else
         st_OUTPUT.setText ( Str.getStr ( STR_NO_PRODUCTS_SELECTED ) ) ;


   }


   /**
     *  This method will form the HTML needed to display all Families
     * that are associated as 'All' to this document
     **/
   private String formFamilyHTML () {
      String familyString = "" ;
      Vector families     = document.getFamilies() ;

      if ( ( families != null ) && ( families.size() > 0 ) ) {
         int             i         = 0 ;
         int             size      = families.size() ;
         TypeCategoryRec familyRec = null ;

         while ( i < size ) {
            familyRec = (TypeCategoryRec) families.elementAt(i) ;
            familyString+=familyRec.getDescript()+"("+Str.getStr ( STR_ALL )+")" ;
            if ( i < (size-1) )
               familyString+=", " ;
            i++ ;
         }
      }

      return( familyString ) ;
   }


   /**
     *  This method will make the HTML needed to display the machine
     * information. The machine information that is displayed must be
     * in the format:\n
     * Family Name (machine1, machine2, machine3), Family Name (machine3, machine4) etc..\n\n
     *
     * In this method, the Hashtable called families will hold each unique family and all of the
     * family's machines. The key in the Hashtable will be the family name. The object in the
     * hashtable will be a Vector of TypeCategoryRec objects. Each Object representing a machine.
     *
     **/
   private String formMachineHTML () {
      String    retStr         = "" ;
      Vector    sortBrandVec   = new Vector () ;
      Vector    sortFamilyVec  = new Vector () ;
      Vector    machines       = document.getMachines() ;
      Hashtable brands         = new Hashtable () ;

      if ( ( machines != null ) && ( machines.size() > 0 ) ) {
         TypeCategoryRec machineRec      = null ;
         TypeCategoryRec familyRec       = null ;
         TypeCategoryRec brandRec        = null ;
         int             machineIndex    = 0 ;
         int             machineSize     = machines.size() ;
         int             brandIndex      = 0 ;
         int             brandSize       = 0 ;


         /*-----------------------------------------------------------------------*/
         /*                                                                       */
         /*  First, walk through the list of machines, adding brands and families */
         /*                                                                       */
         /*-----------------------------------------------------------------------*/
         while ( machineIndex < machineSize ) {


            /*-------------------------------------*/
            /*                                     */
            /*  Grab the machine, family and brand */
            /*                                     */
            /*-------------------------------------*/
            machineRec = (TypeCategoryRec)machines.elementAt(machineIndex) ;
            familyRec  = (TypeCategoryRec)TypeCategory.getParentCategory ( machineRec, TypeCategory.LEVEL_FAMILY ) ;
            brandRec   = (TypeCategoryRec)TypeCategory.getParentCategory ( machineRec, TypeCategory.LEVEL_BRAND ) ;


            /*-------------------------------------------------*/
            /*                                                 */
            /*  If we haven't added this brand yet, add it now */
            /*                                                 */
            /*-------------------------------------------------*/
            if ( brands.get ( brandRec ) == null ) {
               Hashtable families   = new Hashtable() ;
               Vector    newMachine = new Vector () ;

               newMachine.addElement ( machineRec ) ;
               families.put ( familyRec, newMachine ) ;
               brands.put ( brandRec, families ) ;
            }

            /*------------------------------------------------------*/
            /*                                                      */
            /*  If we have this brand, check the family and machine */
            /*                                                      */
            /*------------------------------------------------------*/
            else {
               Hashtable families   = (Hashtable) brands.get ( brandRec ) ;
               Vector    newMachine = (Vector)    families.get ( familyRec ) ;

               /*-----------------------------------------------------*/
               /*                                                     */
               /*  If we have no family in the hash table, add it now */
               /*                                                     */
               /*-----------------------------------------------------*/
               if ( newMachine == null ) {
                  newMachine = new Vector ( 1 ) ;
                  families.put ( familyRec, newMachine ) ;
               }

               /*------------------*/
               /*                  */
               /*  Add the machine */
               /*                  */
               /*------------------*/
               newMachine.addElement ( machineRec ) ;
            }


            /*---------------------------------------------------*/
            /*                                                   */
            /*  Add the Brands and Families to our sorting lists */
            /*                                                   */
            /*---------------------------------------------------*/
            if ( !sortBrandVec.contains ( brandRec ) )
               sortBrandVec.addElement ( brandRec ) ;
            if ( !sortFamilyVec.contains ( familyRec ) )
               sortFamilyVec.addElement ( familyRec ) ;



            machineIndex++ ;
         } /* while  */




         /*-----------------------------------------------------*/
         /*                                                     */
         /*  Ok, now that we have the hash table of all brands, */
         /* families and machines. Write out the output         */
         /*                                                     */
         /*-----------------------------------------------------*/
         new QuickSort ( sortBrandVec ) ;
         new QuickSort ( sortFamilyVec ) ;

         brandSize = sortBrandVec.size() ;    

         /*--------------------------*/
         /*                          */
         /*  Walk the list of brands */
         /*                          */
         /*--------------------------*/
         while ( brandIndex < brandSize ) {
            Hashtable families    = (Hashtable)brands.get ( sortBrandVec.elementAt( brandIndex ) ) ;
            int       familyIndex = 0 ;
            int       familySize  = sortFamilyVec.size()  ;

            /*-------------------------------------------------*/
            /*                                                 */
            /*  Now, walk the families under the current brand */
            /*                                                 */
            /*-------------------------------------------------*/
            while ( familyIndex < familySize ) {
               Vector outMachines = (Vector)families.get ( sortFamilyVec.elementAt ( familyIndex ) ) ;

               if ( outMachines != null ) {
                  TypeCategoryRec outMachineRec = null ;


                  machineIndex = 0 ;
                  machineSize  = outMachines.size() ;

                  /*---------------------------------*/
                  /*                                 */
                  /*  Add this family  to our retStr */
                  /*                                 */
                  /*---------------------------------*/
                  retStr+=sortFamilyVec.elementAt(familyIndex)+" (" ;

                  /*-----------------------------------------------*/
                  /*                                               */
                  /*  Now, walk the machines, writing out each one */
                  /*                                               */
                  /*-----------------------------------------------*/
                  while ( machineIndex < machineSize ) {
                     outMachineRec = (TypeCategoryRec) outMachines.elementAt(machineIndex) ;
                     retStr+=outMachineRec.getDescript()+"("+Str.getStr ( STR_ALL )+")" ;

                     if ( machineIndex < (machineSize-1) )
                        retStr+=", " ;
                     machineIndex++ ;
                  }

                  /*--------------------------*/
                  /*                          */  
                  /*  Close this machine list */
                  /*                          */  
                  /*--------------------------*/
                  retStr+=")";
               }


               /*--------------*/
               /*              */
               /*  Add a comma */
               /*              */
               /*--------------*/
               if ( familyIndex < (familySize-1) )
                  retStr+=", " ;

               familyIndex++ ;
            } /* family while  */

            brandIndex++ ;
         } /* while  */



      }
      return( retStr ) ;
   }


   /**
     *  This method will form the HTML needed to display the model information. This information
     * is displayed like:\n
     * Family (Machine, model1, model2, model3) Family (Machine2, model1, model2, model3)
     *
     * @return String
     **/
   private String formModelHTML () {
      String    retStr         = "" ;
      Vector    sortBrandVec   = new Vector () ;
      Vector    sortFamilyVec  = new Vector () ;
      Vector    sortMachineVec = new Vector () ;
      Vector    models         = document.getModels() ;
      Hashtable brands         = new Hashtable () ;

      if ( ( models != null ) && ( models.size() > 0 ) ) {
         TypeCategoryRec machineRec      = null ;
         TypeCategoryRec familyRec       = null ;
         TypeCategoryRec brandRec        = null ;
         TypeCategoryRec modelRec        = null ;
         int             modelIndex      = 0 ;
         int             modelSize       = models.size() ;
         int             brandIndex      = 0 ;
         int             brandSize       = 0 ;

         /*-----------------------------------------------------------------------*/
         /*                                                                       */
         /*  First, walk through the list of machines, adding brands and families */
         /* to our temporary vector. This temporary vector will be used to sort   */
         /* everything later in this method                                       */
         /*                                                                       */
         /*-----------------------------------------------------------------------*/
         while ( modelIndex < modelSize ) {


            /*-------------------------------------*/
            /*                                     */
            /*  Grab the machine, family and brand */
            /*                                     */
            /*-------------------------------------*/
            modelRec   = (TypeCategoryRec)models.elementAt(modelIndex) ;

            if ( modelRec.getCategoryLevel().equals ( TypeCategory.LEVEL_PARTNUMBER ) )
               machineRec = (TypeCategoryRec)TypeCategory.getParentCategory ( modelRec, TypeCategory.LEVEL_SUBCATEGORY ) ;
            else
               machineRec = (TypeCategoryRec)TypeCategory.getParentCategory ( modelRec, TypeCategory.LEVEL_MACHINE ) ;

            if ( machineRec != null ) {
               familyRec  = (TypeCategoryRec)TypeCategory.getParentCategory ( modelRec, TypeCategory.LEVEL_FAMILY ) ;
               brandRec   = (TypeCategoryRec)TypeCategory.getParentCategory ( modelRec, TypeCategory.LEVEL_BRAND ) ;
            }


            /*-------------------------------------------------*/
            /*                                                 */
            /*  If we haven't added this brand yet, add it now */
            /*                                                 */
            /*-------------------------------------------------*/
            if ( brands.get ( brandRec ) == null ) {
               Hashtable families  = new Hashtable() ;
               Hashtable machines  = new Hashtable() ;
               Vector    newModels = new Vector ( 1 ) ;

               newModels.addElement ( modelRec ) ;
               machines.put ( machineRec.toString(), newModels ) ;
               families.put ( familyRec, machines ) ;
               brands.put ( brandRec, families ) ;
            }

            /*------------------------------------------------------*/
            /*                                                      */
            /*  If we have this brand, check the family and machine */
            /*                                                      */
            /*------------------------------------------------------*/
            else {
               Hashtable families   = (Hashtable) brands.get ( brandRec ) ;
               Hashtable machines   = (Hashtable) families.get ( familyRec ) ;
               Vector    newModels  = null ;

               if ( machines != null ) {
                  newModels = (Vector)    machines.get ( machineRec.toString() ) ;
               }

               /*-----------------------------------------------------*/
               /*                                                     */
               /*  If we have no family in the hash table, add it now */
               /*                                                     */
               /*-----------------------------------------------------*/
               if ( newModels == null ) {
                  newModels = new Vector ( 1 ) ;
                  if ( machines == null ) {
                     machines = new Hashtable() ;
                     families.put ( familyRec, machines ) ;
                  }

                  machines.put ( machineRec.toString(), newModels ) ;
                  //families.put ( familyRec, machines ) ;
               }

               /*------------------*/
               /*                  */
               /*  Add the machine */
               /*                  */
               /*------------------*/
               newModels.addElement ( modelRec ) ;
            }


            /*---------------------------------------------------*/
            /*                                                   */
            /*  Add the Brands and Families to our sorting lists */
            /*                                                   */
            /*---------------------------------------------------*/
            if ( !sortBrandVec.contains ( brandRec ) )
               sortBrandVec.addElement ( brandRec ) ;
            if ( !sortFamilyVec.contains ( familyRec ) )
               sortFamilyVec.addElement ( familyRec ) ;
            if ( !sortMachineVec.contains ( machineRec ) )
               sortMachineVec.addElement ( machineRec ) ;



            modelIndex++ ;
         } /* while  */




         /*-----------------------------------------------------*/
         /*                                                     */
         /*  Ok, now that we have the hash table of all brands, */
         /* families and machines. Write out the output         */
         /*                                                     */
         /*-----------------------------------------------------*/
         new QuickSort ( sortBrandVec ) ;
         new QuickSort ( sortFamilyVec ) ;
         new QuickSort ( sortMachineVec ) ;

         brandSize = sortBrandVec.size() ;    

         /*--------------------------*/
         /*                          */
         /*  Walk the list of brands */
         /*                          */
         /*--------------------------*/
         while ( brandIndex < brandSize ) {
            Hashtable families    = (Hashtable)brands.get ( sortBrandVec.elementAt( brandIndex ) ) ;
            int       familyIndex = 0 ;
            int       familySize  = sortFamilyVec.size()  ;

            /*-------------------------------------------------*/
            /*                                                 */
            /*  Now, walk the families under the current brand */
            /*                                                 */
            /*-------------------------------------------------*/
            while ( familyIndex < familySize ) {
               Hashtable machines     = (Hashtable)families.get ( sortFamilyVec.elementAt ( familyIndex ) ) ;
               int       machineIndex = 0 ;

               if ( machines != null ) {
                  int       machineSize  = sortMachineVec.size() ;


                  /*---------------------------------------------------*/
                  /*                                                   */
                  /*  Now, walk through the machines under this family */
                  /*                                                   */
                  /*---------------------------------------------------*/
                  while ( machineIndex < machineSize ) {
                     Vector outModels = (Vector) machines.get ( sortMachineVec.elementAt ( machineIndex ).toString() ) ;

                     if ( outModels != null ) {
                        TypeCategoryRec outModelRec = null ;


                        /*-----------------------------------------*/
                        /*                                         */
                        /*  Add the machine and family description */
                        /*                                         */
                        /*-----------------------------------------*/
                        retStr+=sortFamilyVec.elementAt(familyIndex)+"("+sortMachineVec.elementAt ( machineIndex )+"-" ;


                        modelIndex =0 ;
                        modelSize  = outModels.size() ;

                        while ( modelIndex < modelSize ) {

                           outModelRec = (TypeCategoryRec) outModels.elementAt(modelIndex) ;
                           retStr+=outModelRec.getDescript() ;
                           if ( modelIndex < (modelSize-1) )
                              retStr+=", " ;

                           modelIndex++ ;
                        } /* while modelIndex  */

                        /*-------------------------------------*/
                        /*                                     */
                        /*  Close up the family/machine string */
                        /*                                     */
                        /*-------------------------------------*/
                        retStr+=")" ;

                        if ( machineIndex < (machineSize-1) )
                           retStr+=", " ;

                     } /* if outModels  */


                     machineIndex++ ;
                  } /* while  */

                  /*--------------*/
                  /*              */
                  /*  Add a comma */
                  /*              */
                  /*--------------*/
                  if ( familyIndex < (familySize-1) )
                     retStr+=", " ;

               } /* if machines  */

               familyIndex++ ;
            } /* family while  */

            brandIndex++ ;
         } /* while  */



      }
      return( retStr ) ;
   }



   /**
     * This method responds to action events
     **/
   public void actionPerformed ( ActionEvent e ) {
      new ProductSelectDlg ( GUISystem.getParentDefWin ( this ), document ) ;
      new Thread ( this ).start() ;
   }


   public void setEnabled ( boolean enable ) {
      super.setEnabled ( enable ) ;

      pb_SELECT.setEnabled ( enable ) ;

   }

} /* ProductPanel */
