package com.ibm.nzna.projects.qit.doc.docedit ;    
/*-----------------------------------------------------------*/
/*   ProductSelectBrandPanel                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductSelectBrandPanel             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;   
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This panel holds the controls needed to associate Brands and Families
 * to documents. This panel appears as a tab in the ProductSelectDlg
 *
 *
 * @version 1.0, 7/13/2001 
 * @author  David Lentz
 */


public class ProductSelectBrandPanel extends JPanel implements ListSelectionListener, AppConst, DocConst, ActionListener {

   /**
     * This is the Jlabel for the Brand Drop Down
     **/
   private JLabel st_BRAND = new JLabel ( Str.getStr ( STR_BRAND ) ) ;

   /**
     * This is the JLabel for the associated families
     **/
   private JLabel st_ASSOC_FAMILY = new JLabel ( Str.getStr ( STR_ASSOCIATED_FAMILIES ) ) ;

   /**
     * This is the Drop Down with all brands
     **/
   private JComboBox cb_BRAND = new JComboBox ( Brands.getBrands() ) ;

   /**
     * This is the DList with families
     **/
   private DList  lb_FAMILY = new DList () ;

   /**
     * This is the JScrollPane holding the lb_FAMILY DList
     **/
   private JScrollPane scr_FAMILY = new JScrollPane ( lb_FAMILY ) ;

   /**
     * This is the DList holding the associated families
     **/
   private DList lb_ASSOC_FAMILY = new DList () ;

   /**
     * This is the JScrollPane holding the lb_ASSOC_FAMILY DList
     **/
   private JScrollPane scr_ASSOC_FAMILY = new JScrollPane ( lb_ASSOC_FAMILY ) ;

   /**
     * This is the De associate button
     **/
   private DButton pb_DEASSOC = new DButton ( "<<--" ) ;

   /**
     * This is the Associate Button
     **/
   private DButton pb_ASSOC = new DButton ( "-->>" ) ;

   /**
     * This is the Associate all button
     **/
   private DButton pb_ASSOC_ALL = new DButton ( ">>>>" ) ;


   /**
     * This is the document that we are editing
     **/
   private Document document = null;

   /**
     * This is the list model for the lb_FAMILY
     **/
   private DefaultListModel familyDefaultModel = new DefaultListModel () ;

   /**
     * This is the list model for the lb_ASSOC_FAMILY
     **/
   private DefaultListModel assocFamilyDefaultModel = new DefaultListModel () ;


   /**
     * This is the trailing '(All)' String that is appended to the families that are all Families
     **/
   private String allStr = "("+Str.getStr ( STR_ALL )+")" ;

   /**
     * This is the ProductSelectdlg that we will be using
     **/
   private ProductSelectDlg selectDlg = null ;


   /**
     * The Default constructor creates a blank GUI panel 
     **/
   public ProductSelectBrandPanel ( ProductSelectDlg selectDlg ) {

      this.selectDlg = selectDlg ;                

      /*--------------------*/
      /*                    */
      /*  Adjust properties */
      /*                    */
      /*--------------------*/
      pb_ASSOC_ALL.setForeground ( Color.blue ) ;
      pb_ASSOC_ALL.setEnabled ( false ) ;
      pb_ASSOC.setEnabled ( false ) ;
      pb_DEASSOC.setEnabled ( false ) ;
      lb_FAMILY.setModel ( familyDefaultModel ) ;
      lb_ASSOC_FAMILY.setModel ( assocFamilyDefaultModel );                        

      /*----------------*/
      /*                */
      /*  Add listeners */
      /*                */
      /*----------------*/
      pb_ASSOC_ALL.addActionListener ( this ) ;
      pb_ASSOC.addActionListener     ( this ) ;
      pb_DEASSOC.addActionListener   ( this ) ;
      cb_BRAND.addActionListener     ( this ) ;
      lb_FAMILY.addListSelectionListener       ( this ) ;
      lb_ASSOC_FAMILY.addListSelectionListener ( this ) ;
      lb_FAMILY.addActionListener ( this ) ;
      lb_ASSOC_FAMILY.addActionListener ( this ) ;

      /*------------------------------*/
      /*                              */
      /*  Add the controls to our GUI */
      /*                              */
      /*------------------------------*/
      setLayout ( null ) ;
      add ( st_BRAND ) ;
      add ( cb_BRAND ) ;
      add ( st_ASSOC_FAMILY ) ;
      add ( scr_FAMILY ) ;
      add ( pb_DEASSOC ) ;
      add ( pb_ASSOC ) ;
      add ( pb_ASSOC_ALL ) ;
      add ( scr_ASSOC_FAMILY ) ;

   }




   /**
     * This method will lay out and position our controls
     **/
   public void doLayout () {
      Dimension size        = getSize() ;
      int       halfWidth   = size.width/2 ;
      int       buttonWidth = 20 ;
      int       x           = 5 ;
      int       y           = 5 ;
      int       rowHeight   = GUISystem.getRowHeight () ;

      super.doLayout() ;

      st_BRAND.setBounds ( x, y, halfWidth-10-buttonWidth, rowHeight ) ;
      st_ASSOC_FAMILY.setBounds ( halfWidth+5+buttonWidth, y, halfWidth-10-buttonWidth, rowHeight ) ;
      y+=rowHeight ;

      cb_BRAND.setBounds ( x, y, halfWidth-10-buttonWidth, rowHeight ) ;
      scr_ASSOC_FAMILY.setBounds ( halfWidth+5+buttonWidth, y, halfWidth-10-buttonWidth, size.height-y-5 ) ;
      y+=rowHeight ;

      scr_FAMILY.setBounds ( x, y, halfWidth-10-buttonWidth, size.height-y-5 ) ;


      y=rowHeight*4 ;
      pb_DEASSOC.setBounds ( halfWidth-(buttonWidth), y, buttonWidth*2, 25 ) ;
      y+=30 ;

      pb_ASSOC.setBounds ( halfWidth-(buttonWidth), y, buttonWidth*2, 25 ) ;
      y+=30 ;

      pb_ASSOC_ALL.setBounds ( halfWidth-(buttonWidth), y, buttonWidth*2, 25 ) ;
      y+=30 ;

   }


   /**
     * This method will be called when the user has selected a button
     **/
   public void actionPerformed ( ActionEvent e) {


      /*------------------------------------------------------*/
      /*                                                      */
      /*  If the user selected a brand, populate the families */
      /*                                                      */
      /*------------------------------------------------------*/
      if ( e.getSource() == cb_BRAND ) {
         lb_FAMILY.setListData ( Brands.getFamilies ( (TypeCategoryRec)cb_BRAND.getSelectedItem() ) ) ;
         PropertySystem.putInt ( PropertySystem.PROPERTY_LAST_SELECTED_BRAND, 
                                 ((TypeCategoryRec)cb_BRAND.getSelectedItem()).getInd() ) ;
      }

      /*-----------------------------------------------------*/
      /*                                                     */
      /*  If the user selected to 'Associate All', make that */
      /*                                                     */
      /*-----------------------------------------------------*/
      else if ( e.getSource() == pb_ASSOC_ALL ) {
         associateFamilyAll() ;
      }

      /*-----------------------------------------------*/
      /*                                               */
      /*  If the user selected to 'Associate' a family */
      /*                                               */
      /*-----------------------------------------------*/
      else if ( ( e.getSource() == pb_ASSOC ) || ( e.getSource() == lb_FAMILY ) ) {
         associateApplicableFamily() ;
      }

      /**
        * If the user selected to 'Deassociate' a family
        **/
      else if ( ( e.getSource() == pb_DEASSOC ) || ( e.getSource() == lb_ASSOC_FAMILY ) ) {
         deassociateFamily() ;
      }

   }




   /**
     * This method will refresh the data in our panel with the data from the document passed in. This
     * document that is passed will also be saved to when any changes are made
     **/
   protected void setDocument ( Document document ) {
      this.document = document ;
      clearData() ;
      selectInitialBrand() ;
      refreshData() ;
   }

   /**
     * This method will clear our data
     **/
   private void clearData () {
      Vector noFamily      = new Vector ( 1 ) ;
      Vector noAssocFamily = new Vector ( 1 ) ;

      noFamily.addElement ( Str.getStr ( STR_PLEASE_SELECT_A_BRAND_FROM_ABOVE ) ) ;
      noAssocFamily.addElement ( Str.getStr ( STR_NO_FAMILIES ) ) ;

      lb_FAMILY.setListData ( noFamily ) ;
      lb_ASSOC_FAMILY.setListData ( noAssocFamily ) ;
   }

   /**
     * This method will refresh everything that is displayed on this panel
     **/
   private void refreshData () {

      /*---------------------------------------------------*/
      /*                                                   */
      /*  If we do get passed a document, refresh our info */
      /*                                                   */
      /*---------------------------------------------------*/
      if ( document != null ) {
         Vector          brands      = new Vector () ;
         Vector          families    = null ;
         Vector          allFamilies = new Vector () ;
         TypeCategoryRec family      = null ;
         TypeCategoryRec brand       = null ;

         /*-------------------------------------------------------------------*/
         /*                                                                   */
         /*  First, get all of the directly associated families. We will take */
         /* each Family and add it's brand to the brands vector.              */
         /*                                                                   */
         /*-------------------------------------------------------------------*/
         families = document.getFamilies() ;

         if ( ( families != null ) && ( families.size() > 0 ) ) {
            int             i      = 0 ;
            int             size   = families.size() ;

            while ( i < size ) {
               family = (TypeCategoryRec) families.elementAt(i) ;
               brand  = TypeCategory.getCategory ( family.getParentCatInd() ) ;
               if ( !brands.contains ( brand ) )
                  brands.addElement ( brand ) ;
               if ( !allFamilies.contains ( family ) )
                  allFamilies.addElement ( family ) ;
               family.setToString ( family.getDescript()+" "+allStr ) ;
               i++ ;
            }
         } else {
            Vector noAssocFamily = new Vector ( 1 ) ;

            noAssocFamily.addElement ( Str.getStr ( STR_NO_FAMILIES ) ) ;
            lb_ASSOC_FAMILY.setListData ( noAssocFamily ) ;
         }


         /*------------------------------------------------------------*/
         /*                                                            */
         /*  Now, take all of the implied families from the document,  */
         /* we will add each of their brands to the list of brands     */
         /*                                                            */
         /*------------------------------------------------------------*/
         families = document.getApplicableFamilies () ;

         if ( ( families != null ) && ( families.size() > 0 ) ) {
            int             i      = 0 ;
            int             size   = families.size() ;

            while ( i < size ) {
               family = (TypeCategoryRec) families.elementAt(i) ;
               brand  = TypeCategory.getCategory ( family.getParentCatInd() ) ;

               if ( !brands.contains ( brand ) )
                  brands.addElement ( brand ) ;
               if ( !allFamilies.contains ( family ) )
                  allFamilies.addElement ( family ) ;

               family.setToString ( family.getDescript() ) ;
               i++ ;
            }
         }

         /*-------------------------------*/
         /*                               */
         /*  Now, sort the list of brands */
         /*                               */
         /*-------------------------------*/
         new QuickSort ( brands ) ;


         /*-------------------------------------------------------*/
         /*                                                       */
         /*  Now, create a Vector that is sorted by the brand but */
         /* contains a list of families. This vector will be put  */
         /* into the list of Families                             */
         /*                                                       */
         /*-------------------------------------------------------*/
         if ( brands.size() > 0 ) {
            Vector sortedFamilies = new Vector ( 1 ) ;
            int    i              = 0 ;
            int    size           = brands.size() ;

            while ( i < size ) {
               brand = (TypeCategoryRec)brands.elementAt(i) ;
               families = getFamiliesFromBrand ( brand, allFamilies ) ;
               new QuickSort ( families ) ;
               addFamilies ( families, sortedFamilies ) ;
               i++ ;
            }/* while  */

            /*------------------------------------------*/
            /*                                          */
            /*  And finally, add our list to the output */
            /*                                          */
            /*------------------------------------------*/
            if ( sortedFamilies.size() > 0 )
               lb_ASSOC_FAMILY.setListData ( sortedFamilies ) ;
            else {
               Vector noAssocFamily = new Vector ( 1 ) ;
               noAssocFamily.addElement ( Str.getStr ( STR_NO_FAMILIES ) ) ;
               lb_ASSOC_FAMILY.setListData ( noAssocFamily ) ;
            }

         } /* if brands.size()  */




      } /* else, we have a document  */

   } /* refreshData  */


   /**
     * This method will return a Vector of families from the
     * brand passed in and the list of families passed in
     *
     * @param brand The Brand to look for
     * @param allFamilies The complete list of families to look through
     **/
   private Vector getFamiliesFromBrand ( TypeCategoryRec brand, Vector allFamilies ) {
      Vector returnFamilies = new Vector ( 1 ) ;
      int    i              = 0 ;
      int    size           = allFamilies.size() ;

      while ( i < size ) {
         if ( ((TypeCategoryRec)allFamilies.elementAt(i)).getParentCatInd() == brand.getInd() )
            returnFamilies.addElement ( allFamilies.elementAt(i) ) ;
         i++ ;
      }

      return( returnFamilies ) ;
   }



   /**
     * This method will simply add the families from sourceFamilyVec to
     * targetFamilyVec.
     *
     * @param sourceFamilyVec The families to copy from
     * @param targetFamilyVec The Vector to copy into
     **/
   private void addFamilies ( Vector sourceFamilyVec, Vector targetFamilyVec ) {
      int i    = 0;
      int size = sourceFamilyVec.size() ;

      while ( i < size ) {
         if ( !targetFamilyVec.contains ( sourceFamilyVec.elementAt(i) ) )
            targetFamilyVec.addElement ( sourceFamilyVec.elementAt(i) ) ;
         i++ ;
      }
   }


   /**
     * This method will select the initial brand. If the user has previously
     * selected a brand, then that previously selected brand will select. If the
     * user does not have a previously selected brand, but does have a default brand,
     * then the default brand is selected. If the user has neither a previously selected
     * brand or default brand then nothing is done. If a brand is selected, then the families
     * under that brand are also refreshed and shown
     **/
   private void selectInitialBrand () {
      TypeCategoryRec lastSelectedBrand = (TypeCategoryRec) TypeCategory.getCategory ( PropertySystem.getInt ( PropertySystem.PROPERTY_LAST_SELECTED_BRAND ) ) ;
      TypeCategoryRec favoriteBrand     = (TypeCategoryRec) TypeCategory.getCategory ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEFAULT_BRAND ) ) ;

      if ( lastSelectedBrand == null )
         lastSelectedBrand = favoriteBrand ;

      if ( lastSelectedBrand != null ) {
         cb_BRAND.setSelectedItem ( lastSelectedBrand ) ;
         lb_FAMILY.setListData ( Brands.getFamilies ( lastSelectedBrand ) ) ;
      }
   }

   /**
     * This method will enable/disable the Associate controls
     * depending on what list has what selected.
     *
     * @param e The ListSelectionEvent fired
     **/
   public void valueChanged ( ListSelectionEvent e ) {
      boolean enableAssoc = ( ( lb_FAMILY.getSelectedValue() != null ) &&
                              ( lb_FAMILY.getSelectedValue() instanceof TypeCategoryRec ) );

      pb_DEASSOC.setEnabled ( ( lb_ASSOC_FAMILY.getSelectedValue() != null ) &&
                              ( lb_ASSOC_FAMILY.getSelectedValue() instanceof TypeCategoryRec ) );
      pb_ASSOC.setEnabled ( enableAssoc ) ;
      pb_ASSOC_ALL.setEnabled ( enableAssoc ) ;

   }


   /**
     * This method will take all selected families in the lb_FAMILY list and
     * associate them as an 'All' to the document. It will then add them to
     * our list 
     **/
   private void associateFamilyAll () {
      Object[] selectedFamilies   = lb_FAMILY.getSelectedValues() ;
      Vector   families           = document.getFamilies() ;
      Vector   addedFamilies      = new Vector ( 1 ) ;
      Vector   applicableFamilies = document.getApplicableFamilies() ;
      Vector   machines           = document.getMachines() ;
      Vector   applicableMachines = document.getApplicableMachines() ;
      Vector   models             = document.getModels() ;
      Vector   applicableBrands   = document.getApplicableBrands() ;

      /*----------------------------------*/
      /*                                  */
      /*  Create Vectors if they are null */
      /*                                  */
      /*----------------------------------*/
      if ( families == null )
         families = new Vector ( 1 ) ;
      if ( applicableBrands == null )
         applicableBrands = new Vector ( 1 ) ;


      /*------------------------------------------*/
      /*                                          */
      /*  Loop through, adding each selected item */
      /*                                          */
      /*------------------------------------------*/
      for ( int i = 0; i < selectedFamilies.length; i++ ) {
         TypeCategoryRec family = (TypeCategoryRec)selectedFamilies[i] ;
         TypeCategoryRec brand  = TypeCategory.getParentCategory ( family, TypeCategory.LEVEL_BRAND ) ;

         if ( !families.contains ( family ) ) {
            families.addElement ( family ) ;
            addedFamilies.addElement ( family ) ;

            if ( !applicableBrands.contains ( brand ) )
               applicableBrands.addElement ( brand ) ;
         }


         if ( ( applicableFamilies != null ) && 
              ( applicableFamilies.contains ( family ) ) ) {
            applicableFamilies.removeElement ( family ) ;

            deassociateFamilyChildren ( family, machines ) ;
            deassociateFamilyChildren ( family, applicableMachines ) ;
            deassociateFamilyChildren ( family, models ) ;
         }
      }


      document.setPropertyChangeEnabled ( false ) ;
      document.setApplicableBrands ( applicableBrands ) ;
      document.setModels ( models ) ;
      document.setMachines ( machines ) ;
      document.setApplicableMachines ( machines ) ;
      document.setFamilies ( families ) ;
      document.setPropertyChangeEnabled ( true ) ;
      document.setApplicableFamilies ( applicableFamilies ) ;

      refreshData() ;
      WinUtil.selectItems ( lb_ASSOC_FAMILY, addedFamilies ) ;
   }


   /**
     * This method will take all selected families in the lb_FAMILY list and
     * associate them as an Applicable Family to the document. It will then add them to
     * our list 
     **/
   private void associateApplicableFamily () {
      Object[] selectedFamilies   = lb_FAMILY.getSelectedValues() ;
      Vector   applicableFamilies = document.getApplicableFamilies() ;
      Vector   addedFamilies      = new Vector ( 1 ) ;
      Vector   families           = document.getFamilies() ;
      Vector   applicableBrands   = document.getApplicableBrands() ;


      /*--------------------------------------------*/
      /*                                            */
      /*  Create the vecs if we don't hafe them yet */
      /*                                            */
      /*--------------------------------------------*/
      if ( applicableFamilies == null )
         applicableFamilies = new Vector ( 1 ) ;

      if ( applicableBrands == null )
         applicableBrands = new Vector ( 1 ) ;

      if ( families == null )
         families = new Vector ( 1 ) ;


      /*------------------------------------------------*/
      /*                                                */
      /*  We do a lot of 'set' calls, don't send events */
      /*                                                */
      /*------------------------------------------------*/
      document.setPropertyChangeEnabled ( false ) ;


      for ( int i = 0; i < selectedFamilies.length; i++ ) {

         if ( !applicableFamilies.contains ( selectedFamilies[i] ) ) {
            TypeCategoryRec family = (TypeCategoryRec)selectedFamilies[i] ;
            TypeCategoryRec brand  = TypeCategory.getParentCategory ( family,
                                                                      TypeCategory.LEVEL_BRAND ) ;


            /*---------------------------------------------------------*/
            /*                                                         */
            /*  Add the brand to applicable brands if we don't have it */
            /*                                                         */
            /*---------------------------------------------------------*/
            if ( !applicableBrands.contains ( brand ) )
               applicableBrands.addElement ( brand ) ;

            /*-----------------------------------------------------*/
            /*                                                     */
            /*  Remove this family from the list of 'All' Famiiles */
            /*                                                     */
            /*-----------------------------------------------------*/
            if ( families.contains ( family ) )
               families.removeElement ( family ) ;

            if ( !applicableFamilies.contains ( family ) ) {
               applicableFamilies.addElement ( family ) ;
               addedFamilies.addElement ( family ) ;
               associateApplicableMachines ( family ) ;
            }

         }

         families.removeElement ( selectedFamilies[i] ) ;
      }

      document.setApplicableBrands ( applicableBrands ) ;
      document.setPropertyChangeEnabled ( true ) ;
      document.setFamilies ( families ) ;
      document.setApplicableFamilies ( applicableFamilies ) ;
      refreshData() ;

      WinUtil.selectItems ( lb_ASSOC_FAMILY, addedFamilies ) ;
   }


   /**
     * Associates all machines under the family as Applicable machines
     *
     * @param family The family
     **/
   private void associateApplicableMachines ( TypeCategoryRec family ) {
      Vector          machines        = new Vector ( 1 )  ;
      Vector          projects        = null ;
      Vector          currentMachines = document.getApplicableMachines() ;


      /*-----------------------------------------------------*/
      /*                                                     */
      /*  Read in the applicable machines and sub categories */
      /*                                                     */
      /*-----------------------------------------------------*/
      selectDlg.setText ( Str.getStr ( STR_READING )+" "+family.getDescript() ) ;
      StoreUtil.copyUniqueVector ( Brands.getMachinesSubCategoriesFromFamily ( family.getInd() ),
                                   machines ) ;
      selectDlg.setText ( null ) ;

      /*-------------------------------------------------------------------*/
      /*                                                                   */
      /*  If we do not have any applicable machines yet, create our vector */
      /*                                                                   */
      /*-------------------------------------------------------------------*/
      if ( currentMachines == null )
         currentMachines = new Vector () ;


      /*-------------------*/
      /*                   */
      /*  Add the machines */
      /*                   */
      /*-------------------*/
      if ( ( machines != null ) && ( machines.size() > 0 ) ) {
         int    i               = 0 ;
         int    size            = machines.size() ;

         while ( i < size ) {
            if ( !currentMachines.contains ( machines.elementAt(i) ) ) {
               currentMachines.addElement ( machines.elementAt(i) ) ;
            }
            i++ ;
         }

      }

      document.setApplicableMachines ( currentMachines ) ;
   }

   /**
     *  This method will add all applicable category recs to the 
     * vector passed
     **/
   private void addApplicableMachines ( TypeCategoryRec project, Vector machines, String categoryLevel ) {
      Vector newMachines = TypeCategory.getChildCategories ( project.getInd(), categoryLevel ) ;

      if ( ( newMachines != null ) && ( newMachines.size() > 0 ) ) {
         int i     = 0 ;
         int size  = newMachines.size() ;

         while ( i < size ) {
            if ( !machines.contains ( newMachines.elementAt(i) ) )
               machines.addElement ( newMachines.elementAt(i) ) ;
            i++ ;
         }
      }
   }



   /**
     * This method will take all selected families in the lb_ASSOC_FAMILY list and
     * deassociate them from the document. 
     **/
   private void deassociateFamily () {
      Object[] selectedFamilies   = lb_ASSOC_FAMILY.getSelectedValues() ;

      if ( ( selectedFamilies != null ) && ( selectedFamilies.length > 0 ) ) {
         Vector   applicableFamilies = document.getApplicableFamilies() ;
         Vector   families           = document.getFamilies() ;
         Vector   applicableMachines = document.getApplicableMachines() ;
         Vector   machines           = document.getMachines() ;
         Vector   models             = document.getModels() ;

         document.setPropertyChangeEnabled ( false ) ;

         for ( int i = 0; i < selectedFamilies.length; i++ ) {
            families.removeElement ( selectedFamilies[i] ) ;
            applicableFamilies.removeElement ( selectedFamilies[i] ) ;
            deassociateFamilyChildren ( (TypeCategoryRec)selectedFamilies[i], applicableMachines ) ;
            deassociateFamilyChildren ( (TypeCategoryRec)selectedFamilies[i], machines ) ;
            deassociateFamilyChildren ( (TypeCategoryRec)selectedFamilies[i], models ) ;
         }

         document.setModels ( models ) ;
         document.setMachines ( machines ) ;
         document.setApplicableMachines ( applicableMachines ) ;
         document.setApplicableFamilies ( applicableFamilies ) ;
         document.setPropertyChangeEnabled ( true ) ;
         document.setFamilies ( families ) ;
         refreshData() ;
      }
   } /* deassociateFamily  */

   private void deassociateFamilyChildren ( TypeCategoryRec familyRec, Vector machines ) {
      if ( ( machines != null ) && ( machines.size() > 0 ) ) {
         int i    = 0;
         int size = machines.size() ;

         while ( i < size ) {
            if ( TypeCategory.getParentCategory ( (TypeCategoryRec)machines.elementAt ( i ),
                                                  familyRec.getCategoryLevel() ).getInd() == familyRec.getInd() ) {
               machines.removeElementAt ( i ) ;
               size-- ;
            } else
               i++ ;
         }
      }
   }


   /**
     * This method will clean up all of our resources
     **/
   protected void cleanup () {
   }

}
