package com.ibm.nzna.projects.qit.doc.docedit ;    
/*-----------------------------------------------------------*/
/*   ProductSelectDlg                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductSelectDlg                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * The JDialog allows the user to select any amount of products to  
 * associate to a document.
 *
 *
 * @version 1.0, 7/13/2001 
 * @author  David Lentz
 */


public class ProductSelectDlg extends JDialog implements Runnable, WindowListener, DocConst, AppConst, ActionListener {

   /**
     * This is the Notebook which holds the Brand/Family and Machine/Model tabs.
     **/
   private JTabbedPane notebook  = new JTabbedPane() ;

   /**
     * This is the 'Ok' Button
     **/
   private DButton pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     * This is the 'Cancel' button
     **/
   private DButton pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;


   /**
     * This ButtonPanel holds the Ok and Cancel buttons
     **/
   private ButtonPanel buttonBar = new ButtonPanel () ;

   /**
     * This property tells the closing methods that the user wished to cancel,
     * which means any changes made to the document will be lost
     **/
   private boolean userCancelled = true ;

   /**
     * This is the page which holds all of the brand and family information
     **/
   private ProductSelectBrandPanel brandPanel = null ;

   /**
     * This is the page which holds all of the machine and model information
     **/
   private ProductSelectMachinePanel machinePanel = null ;


   /**
     * This is the document we are working with
     **/
   private Document document = null ;

   /**
     * This is our status
     **/
   private JLabel st_STATUS = null ;

   /**
     *  This is the filename of the serialized file that we will save the documetn
     * to before we adjust any data
     **/
   private Document oldDocument = null ;

   /**
     *  If we are showing a ProductSelectModelPanel, then this is the instance of it
     **/
   private ProductSelectModelPanel modelPanel = null ;

   /**
     * This is the default constructor which shows the window and does not return until
     * the user closes the window
     **/
   public ProductSelectDlg ( Frame parent, Document document ) {
      super ( parent, Str.getStr ( STR_PRODUCTS ), true ) ;

      this.document = document ;
      this.oldDocument = (Document)document.clone() ;

      initialize() ;
      new Thread ( this ).start() ;
      setVisible ( true ) ;
   }


   /**
     * This method will add all of our controls
     **/
   private void initialize () {
      Container contentPane = getContentPane() ;
      Rectangle oldSize     = (Rectangle)PropertySystem.get ( PROPERTY_PRODUCT_SELECT_SIZE ) ;            


      /*----------------------*/
      /*                      */
      /*  Create our notebook */
      /*                      */
      /*----------------------*/
      brandPanel   = new ProductSelectBrandPanel   ( this ) ;
      machinePanel = new ProductSelectMachinePanel ( this ) ;
      notebook.addTab ( Str.getStr ( STR_BRAND_FAMILY ), brandPanel ) ;
      notebook.addTab ( Str.getStr ( STR_MACHINE_MODEL ), machinePanel ) ;
      st_STATUS = new JLabel () ;                        

      /*----------------*/
      /*                */
      /*  Add listeners */
      /*                */
      /*----------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      addWindowListener ( this ) ;

      /*-----------------------------------*/
      /*                                   */
      /*  Add the GUI to our content panel */
      /*                                   */
      /*-----------------------------------*/
      contentPane.setLayout ( null ) ;
      buttonBar.add ( pb_OK ) ;
      buttonBar.add ( pb_CANCEL ) ;
      contentPane.add ( notebook ) ;
      contentPane.add ( st_STATUS ) ;
      contentPane.add ( buttonBar ) ;
      GUISystem.setPropertiesOnPanel ( contentPane ) ;


      if ( oldSize != null )
         setBounds ( oldSize ) ;
      else {
         setSize ( 450, 300 ) ;
         WinUtil.centerWindow ( this ) ;
      }

   }


   /**
     * This method will lay out and position all of our controls
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      if ( modelPanel == null )
         notebook.setBounds ( 5, 5, size.width-(15), size.height-70-rowHeight ) ;
      else
         modelPanel.setBounds ( 5, 5, size.width-(15), size.height-70-rowHeight ) ;
      st_STATUS.setBounds ( 5, size.height-65-rowHeight, size.width-15, rowHeight ) ;
      buttonBar.setBounds ( 5, size.height-60, size.width-15, 25 ) ;
   }



   /**
     * This method will be called when an action event is fired on us
     **/
   public void actionPerformed ( ActionEvent e) {
      if ( e.getSource() == pb_CANCEL )
         userCancelled = true ;
      else
         userCancelled = false ;

      dispose() ;
      windowClosing ( null );

   }


   /**
     * This method will assume the user hit 'cancel'
     **/
   public void windowClosing ( WindowEvent e ) {
      if ( userCancelled ) {
         /* Read in the document from the serialized file we created
            in the constructor so that no changes that were made take
            affect  */
         document.initializeProperties ( oldDocument ) ;
      }

      brandPanel.cleanup() ;
      machinePanel.cleanup() ;
      /*----------------------------------------------------*/
      /*                                                    */
      /*  Now, write out our current position for next time */
      /*                                                    */
      /*----------------------------------------------------*/
      PropertySystem.put ( PROPERTY_PRODUCT_SELECT_SIZE, getBounds() ) ;
   }


   /**
     * This thread will refresh the data in our panels
     **/
   public void run () {
      brandPanel.setDocument   ( document ) ;
      machinePanel.setDocument ( document ) ;
   }


   /**
     * This method will set the text on our status 
     **/
   protected void setText ( String text ) {
      if ( text != null )
         st_STATUS.setText ( text ) ;
      else
         st_STATUS.setText ( "" ) ;
   }


   /**
     *  This method will show the ProductSelectModelPanel and
     * immediatly return
     *
     * @param machineArray An Array of TypeCategoryRec objects representing the machines
     *
     * @return THe ProductSelectModelPanel that was shown
     **/
   protected ProductSelectModelPanel showModelPanel ( Object[] machineArray ) {
      modelPanel = new ProductSelectModelPanel ( this, machineArray, document ) ;

      getContentPane().remove ( notebook ) ;
      getContentPane().add ( modelPanel ) ;
      pb_OK.setVisible ( false ) ;
      pb_CANCEL.setVisible ( false ) ;
      invalidate() ;
      ((JComponent)getContentPane()).revalidate() ;
      ((JComponent)getContentPane()).repaint() ;
      doLayout() ;
      return( modelPanel ) ;
   }


   /**
     *  This method will close the ProductSelectModelPanel being shown
     * and puts the tabbed notebook back on
     **/
   protected void closeModelPanel () {
      if ( modelPanel != null ) {
         getContentPane().remove ( modelPanel ) ;
         getContentPane().add ( notebook ) ;
         pb_OK.setVisible ( true ) ;
         pb_CANCEL.setVisible ( true ) ;
         invalidate() ;
         ((JComponent)getContentPane()).revalidate() ;
         ((JComponent)getContentPane()).repaint() ;
         modelPanel = null ;
         doLayout() ;
      }
   }

   /**
     * This following methods do nothing
     **/
   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ) {
   }
   public void windowClosed ( WindowEvent e ) {
   }
   public void windowOpened ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e) {
   }
   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowOpening ( WindowEvent e ) {
   }


   /**
     * This method will enable/disable our GUI
     **/
   public void setEnabled ( boolean enable ) {
      pb_OK.setEnabled ( enable ) ;
      pb_CANCEL.setEnabled (enable ) ;
      notebook.setEnabled ( enable );
   }
}
