package com.ibm.nzna.projects.qit.doc.docedit ;    
/*-----------------------------------------------------------*/
/*   ProductSelectMachinePanel                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductSelectMachinePanel           */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;   
import java.sql.* ;
import java.awt.event.* ;
import javax.swing.event.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This panel holds the controls needed to associate Machines, Sub Categories, Models and Part Numbers
 * to documents. This panel appears as a tab in the ProductSelectDlg
 *
 *
 * @version 1.0, 7/13/2001 
 * @author  David Lentz
 */


public class ProductSelectMachinePanel extends JPanel implements TreeSelectionListener,  DocumentPropertyListener, AppConst, DocConst, ActionListener {

   /**
     * This is the trailing '(All)' String that is appended to the families that are all Families
     **/
   private String allStr = "("+Str.getStr ( STR_ALL )+")" ;

   /**
     * This is the DList which holds the Machines
     **/
   private JTree tre_MACHINES = new JTree () ;

   /**
     * This is the JScrollPane which holds the machines
     **/
   private JScrollPane scr_MACHINES = new JScrollPane ( tre_MACHINES ) ;


   /**
     * This is the JTree which holds the Machines/Models on the right
     **/
   private JTree tre_MODELS = new JTree () ;

   /**
     * This is the JScrollPane which holds the Machines/Models
     **/
   private JScrollPane scr_MODELS = new JScrollPane ( tre_MODELS ) ;

   /**
     * This is the DButton which associates selected machines
     **/
   private DButton pb_ASSOC = new DButton ( "-->>" ) ;

   /**
     * This is the DButton which deassociates selected models
     **/
   private DButton pb_DEASSOC = new DButton ( "<<--" ) ;

   /**
     * This is the DButton which associates all of a machine
     **/
   private DButton pb_ASSOC_ALL = new DButton ( ">>>>" ) ;

   /**
     * This is the DButton which launches the Bookmarks
     **/
   private DButton pb_BOOKMARKS = new DButton ( Str.getStr ( STR_BOOKMARKS ), ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_SMALL ) ) ;

   /**
     *  This is the ProductSelectDlg we are a part of
     **/

   /**
     * This is the Label over the machines
     **/
   private JLabel st_MACHINES = new JLabel ( Str.getStr ( STR_MACHINE ) ) ;

   /**
     *  This is the tree cell renderer
     **/
   private DefaultTreeRenderer treeRenderer = null ;

   /**
     * This is the Label over the models
     **/
   private JLabel st_MODELS = new JLabel ( Str.getStr ( STR_MODEL ) ) ;

   /**
     * This is the document we are editing
     **/
   private Document document = null ;

   /**
     *  This is the ProductSelectDlg we are a part of
     **/
   private ProductSelectDlg selectDlg = null ;

   /**
     * This tells if we are currently refreshing everything
     **/
   private boolean refreshingAll = false ;


   /**
     * The default constructor simply adds our GUI 
     *
     * @param selectDlg The ProductSelectDlg that we are a part of
     **/
   public ProductSelectMachinePanel ( ProductSelectDlg selectDlg ) {

      this.selectDlg = selectDlg ;            


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust GUI Properties                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ASSOC_ALL.setForeground ( Color.blue ) ;           
      tre_MODELS.setCellRenderer ( treeRenderer = new DefaultTreeRenderer () ) ;
      tre_MACHINES.setCellRenderer ( treeRenderer ) ;
      pb_ASSOC.setEnabled ( false ) ;
      pb_DEASSOC.setEnabled ( false ) ;
      pb_ASSOC_ALL.setEnabled ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ASSOC.addActionListener ( this ) ;
      pb_ASSOC_ALL.addActionListener ( this ) ;
      pb_DEASSOC.addActionListener ( this ) ;    
      tre_MACHINES.addTreeSelectionListener ( this ) ;
      tre_MODELS.addTreeSelectionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_MACHINES ) ;
      add ( scr_MACHINES ) ;
      add ( pb_DEASSOC ) ;
      add ( pb_ASSOC ) ;
      add ( pb_ASSOC_ALL ) ;
      add ( st_MODELS ) ;
      add ( scr_MODELS ) ;
      add ( pb_BOOKMARKS ) ;

      clearData() ;

   }



   /**
     * This method will position and size all of our controls
     **/
   public void doLayout () {
      Dimension size        = getSize() ;
      int       halfWidth   = size.width/2 ;
      int       buttonWidth = 20 ;
      int       x           = 5 ;
      int       y           = 5 ;
      int       rowHeight   = GUISystem.getRowHeight () ;

      super.doLayout() ;

      st_MACHINES.setBounds ( x, y, halfWidth-10-buttonWidth, rowHeight ) ;
      st_MODELS.setBounds ( halfWidth+5+buttonWidth, y, halfWidth-10-buttonWidth, rowHeight ) ;
      y+=rowHeight ;

      scr_MACHINES.setBounds ( x, y, halfWidth-10-buttonWidth, size.height-y-5 ) ;
      scr_MODELS.setBounds ( halfWidth+5+buttonWidth, y, halfWidth-10-buttonWidth, size.height-y-5-22 ) ;
      pb_BOOKMARKS.setBounds ( halfWidth+5+buttonWidth, size.height-22, halfWidth-5-buttonWidth, 20 ) ;
      y+=rowHeight ;


      y=rowHeight*4 ;
      pb_DEASSOC.setBounds ( halfWidth-(buttonWidth), y, buttonWidth*2, 25 ) ;
      y+=30 ;

      pb_ASSOC.setBounds ( halfWidth-(buttonWidth), y, buttonWidth*2, 25 ) ;
      y+=30 ;

      pb_ASSOC_ALL.setBounds ( halfWidth-(buttonWidth), y, buttonWidth*2, 25 ) ;
      y+=30 ;


   }


   /**
     * This method will set the data on our screen according to the document
     * passed in. Any changes to the data will also be reflected in this
     * document.
     *
     * @param document The document to use
     **/
   protected void setDocument ( Document document ) {
      if ( this.document != null )
         this.document.removeDocumentPropertyListener ( this ) ;

      this.document = document ;
      document.addDocumentPropertyListener ( this ) ;

      clearData() ;
      refreshData() ;
   }


   /**
     * This method will clear our lists
     **/
   private void clearData () {
      tre_MACHINES.setModel ( createNoMachineTreeModel() ) ;
   }

   /**
     * This method will be called when the user has selected a button
     **/
   public void actionPerformed ( ActionEvent e) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   User wants to associate single models/parts             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_ASSOC )
         associateMachines() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   User wants to associate a machine to as 'All'           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ASSOC_ALL )
         associateAll() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   User wants to deassociate something                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DEASSOC ) {
         deassociate() ;
      }

   }


   /**
     * This method will clean up all of our resources
     **/
   protected void cleanup () {
      if ( document != null )
         document.removeDocumentPropertyListener ( this ) ;
   }


   public void documentChanged ( Document document, int property ) {
      switch ( property ) {
         
         case Document.APPLICABLE_MACHINE :
         case Document.APPLICABLE_FAMILY :
         case Document.FAMILY :
         case Document.MACHINE :
            refreshData() ;
            break ;

         case Document.MODEL :
            if ( !refreshingAll )
               refreshModels() ;
            break ;
      }
   }


   /**
     *  This method will associate all selected machines
     **/
   private void associateMachines () {
      Object[]   machineArray = null ;
      TreePath[] selection    = tre_MACHINES.getSelectionPaths() ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only process if we have selection                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( selection != null ) && ( selection.length > 0 ) ) {
         Vector machines           = document.getMachines() ;
         Vector applicableMachines = document.getApplicableMachines() ;
         Vector tempVec            = new Vector ( 1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   We don't want notifications to go out here              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         document.setPropertyChangeEnabled ( false ) ;                        

         /* Create the vectors if the document doesn't have m  */
         if ( machines == null )
            machines = new Vector ( 1 ) ;
         if ( applicableMachines == null )
            applicableMachines = new Vector ( 1 ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through the selection                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         for ( int i = 0; i < selection.length; i++ ) {
            StringTreeNode  node       = (StringTreeNode)selection[i].getLastPathComponent() ;
            TypeCategoryRec machine    = (TypeCategoryRec) node.getData() ;

            if ( machines.contains ( machine ) ) {
               machines.addElement ( machine ) ;
            }

            if ( !applicableMachines.contains ( machine ) ) {
               applicableMachines.addElement ( machine ) ;
            }

            tempVec.addElement ( machine ) ;
         }

         document.setApplicableMachines ( applicableMachines) ;
         document.setPropertyChangeEnabled ( true ) ;
         //document.setMachines ( machines ) ;

         machineArray = new Object [ tempVec.size() ] ;
         tempVec.copyInto ( machineArray ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Show the panel of models                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( machineArray != null ) && ( machineArray.length > 0 ) )
         selectDlg.showModelPanel ( machineArray ) ;
   }

   /**
     * This method will refresh our data with what the 
     * document currently has
     **/
   private void refreshData () {
      refreshingAll = true ;
      refreshMachines() ;
      refreshModels() ;
      refreshingAll = false ;
   }


   /**
     *  This method refreshes all machines in the Machine tree
     **/
   private void refreshMachines () {
      Vector          brands             = new Vector () ;
      Vector          allFamilies        = new Vector () ;
      TypeCategoryRec family             = null ;
      TypeCategoryRec brand              = null ;
      TypeCategoryRec machine            = null ;
      Vector          allMachines        = new Vector () ;
      Vector          machines           = document.getMachines() ;
      Vector          applicableMachines = document.getApplicableMachines() ;                                             
      int             scrollY            = (scr_MACHINES.getVerticalScrollBar()!=null) ? scr_MACHINES.getVerticalScrollBar().getValue() : 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create one Vector of all Brands, Families and Machines  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( machines != null ) && ( machines.size() > 0 ) ) {
         int i    = 0 ;
         int size = machines.size() ;

         while ( i < size ) {
            machine = (TypeCategoryRec)machines.elementAt(i) ;
            machine.setToString ( machine.getDescript()+" "+allStr ) ;

            if ( !allMachines.contains ( machine ) ) {
               allMachines.addElement ( machine ) ;
               family = TypeCategory.getParentCategory ( machine, TypeCategory.LEVEL_FAMILY ) ;
               brand  = TypeCategory.getParentCategory ( family,  TypeCategory.LEVEL_BRAND ) ;

               if ( !allFamilies.contains ( family ) )
                  allFamilies.addElement ( family ) ;
               if ( !brands.contains ( brand ) )
                  brands.addElement ( brand ) ;
            }
            i++ ;
         }
      }

      if ( ( applicableMachines != null ) && ( applicableMachines.size() > 0 ) ) {
         int i    = 0 ;
         int size = applicableMachines.size() ;

         while ( i < size ) {
            machine = (TypeCategoryRec)applicableMachines.elementAt(i) ;
            machine.setToString ( machine.getDescript() ) ;

            if ( !allMachines.contains ( machine ) ) {
               allMachines.addElement ( machine ) ;
               family = TypeCategory.getParentCategory ( machine, TypeCategory.LEVEL_FAMILY ) ;
               brand  = TypeCategory.getParentCategory ( machine, TypeCategory.LEVEL_BRAND ) ;

               if ( !allFamilies.contains ( family ) )
                  allFamilies.addElement ( family ) ;
               if ( !brands.contains ( brand ) )
                  brands.addElement ( brand ) ;
            }
            i++ ;
         }
      }

      if ( allMachines.size() > 0 ) {

         /*-------------------------------*/
         /*                               */
         /*  Now, sort the list of brands */
         /*                               */
         /*-------------------------------*/
         new QuickSort ( brands ) ;
         new QuickSort ( allFamilies ) ;
         new QuickSort ( allMachines ) ;

         createMachineTreeModel ( brands, allFamilies, allMachines ) ;

      } else
         tre_MACHINES.setModel ( createNoMachineTreeModel() ) ;

      if ( scrollY != 0 )
         scr_MACHINES.getVerticalScrollBar().setValue ( scrollY ) ;
   } /* refreshMachines */



   /**
  *  This method will create a tree model based on the sorted Brands, Families and Machines
  *
  * @param brands The list of brands
  * @param families The list of families
  * @param machines The list of machines
  **/
   private DefaultTreeModel createMachineTreeModel ( Vector brands, Vector families, Vector machines ) {
      StringTreeNode   rootNode   = new StringTreeNode ( Str.getStr ( STR_MACHINES_AND_SUB_CATEGORIES ) ) ;
      DefaultTreeModel treeModel  = new DefaultTreeModel ( rootNode ) ;
      int              i          = 0 ;
      int              size       = brands.size() ;
      TypeCategoryRec  brand      = null ;
      StringTreeNode   brandNode  = null ;         
      int              count      = 0 ;                                    
      Vector           brandNodes = new Vector ( 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Process each brand. By doing this each Family           */
      /* machine and model will be processed                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         brand = (TypeCategoryRec)brands.elementAt(i) ;
         treeModel.insertNodeInto ( brandNode = new StringTreeNode ( brand.getDescript() ), 
                                    rootNode, count++ ) ;
         insertFamiliesIntoModel ( treeModel, brand, brandNode, families, machines ) ;
         brandNodes.addElement ( brandNode ) ;

         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our model                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      tre_MACHINES.setModel ( treeModel ) ;                    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, expand all of the brands                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      tre_MACHINES.setCellRenderer ( new DefaultTreeRenderer () ) ;
      WinUtil.expandTree ( tre_MACHINES ) ;


      return( treeModel ) ;
   }


   /**
     *  This method will add each family into the tree model passed. In doing this,
     * all machines/sub categories will be added under each family
     *
     * @param treeModel The DefaultTreeModel to alter
     * @param brand The brand to use
     * @param families The list of families
     * @param machines The list of machines
     **/
   private void insertFamiliesIntoModel ( DefaultTreeModel treeModel, TypeCategoryRec brandRec, StringTreeNode brand, Vector families, Vector machines ) {
      int             i          = 0 ;
      int             size       = families.size() ;
      StringTreeNode  familyNode = null ;
      int             brandInd   = brandRec.getInd() ;
      TypeCategoryRec family     = null ;
      int             count      = 0 ;

      while ( i < size ) {

         family = (TypeCategoryRec)families.elementAt(i) ;
         if ( family.getParentCatInd() == brandInd ) {
            familyNode = new StringTreeNode ( family.toString() ) ;
            treeModel.insertNodeInto ( familyNode, brand, count++ ) ;
            insertMachinesIntoModel ( treeModel, family, familyNode, machines ) ;
         }
         i++ ;
      }
   }


   /**
     *  This method will add each machine or Sub category under the Family passed
     *
     * @param treeModel The DefaultTreeModel to alter
     * @param family The family to use
     * @param machines The list of machines
     **/
   private void insertMachinesIntoModel ( DefaultTreeModel treeModel, TypeCategoryRec familyRec, StringTreeNode family, Vector machines ) {
      int                i           = 0 ;
      int                size        = machines.size() ;
      StringTreeNode     machineNode = null ;
      int                familyInd   = familyRec.getInd() ;
      TypeCategoryRec    machine     = null ;
      int                count       = 0 ;

      while ( i < size ) {

         machine = (TypeCategoryRec)machines.elementAt(i) ;
         if ( TypeCategory.getParentCategory ( machine, 
                                               TypeCategory.LEVEL_FAMILY ).getInd() == familyInd ) {
            machineNode = new StringTreeNode ( machine.toString(), machine ) ;
            treeModel.insertNodeInto ( machineNode, family, count++ ) ;
         }
         i++ ;
      }
   }
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/











































   /**
     * This method will return a Vector of families from the
     * brand passed in and the list of families passed in
     *
     * @param brand The Brand to look for
     * @param allFamilies The complete list of families to look through
     **/
   private Vector getFamiliesFromBrand ( TypeCategoryRec brand, Vector allFamilies ) {
      Vector returnFamilies = new Vector ( 1 ) ;
      if ( allFamilies != null ) {
         int    i              = 0 ;
         int    size           = allFamilies.size() ;

         while ( i < size ) {
            if ( ((TypeCategoryRec)allFamilies.elementAt(i)).getParentCatInd() == brand.getInd() )
               returnFamilies.addElement ( allFamilies.elementAt(i) ) ;
            i++ ;
         }
      }

      return( returnFamilies ) ;
   }

   /**
     * This method will return a Vector of machines from the
     * family passed in and the list of machines passed in
     *
     * @param family The Family to look for
     * @param allMachines The complete list of machines to look through
     **/
   private Vector getMachinesFromFamily ( TypeCategoryRec family, Vector allMachines ) {
      Vector          returnMachines = new Vector ( 1 ) ;
      int             i              = 0 ;
      int             size           = allMachines.size() ;
      TypeCategoryRec tempCat        = null ;

      while ( i < size ) {
         tempCat = TypeCategory.getParentCategory ( (TypeCategoryRec)allMachines.elementAt(i), TypeCategory.LEVEL_FAMILY ) ;

         if ( ( tempCat != null ) && ( tempCat.getInd() == family.getInd() ) )
            returnMachines.addElement ( allMachines.elementAt(i) ) ;
         i++ ;
      }

      return( returnMachines ) ;
   }



   /**
     * This method will simply add the families from sourceFamilyVec to
     * targetFamilyVec.
     *
     * @param sourceFamilyVec The families to copy from
     * @param targetFamilyVec The Vector to copy into
     **/
   private void addFamilies ( Vector sourceFamilyVec, Vector targetFamilyVec ) {
      int i    = 0;
      int size = sourceFamilyVec.size() ;

      while ( i < size ) {
         if ( !targetFamilyVec.contains ( sourceFamilyVec.elementAt(i) ) )
            targetFamilyVec.addElement ( sourceFamilyVec.elementAt(i) ) ;
         i++ ;
      }
   }


   /**
     *  This method will refresh the Machines/Model tree
     **/
   private void refreshModels () {
      Vector brands   = new Vector ( 1 ) ;
      Vector families = new Vector ( 1 ) ;
      Vector machines = new Vector ( 1 ) ;
      Vector models   = getModelsAndDescriptions ( document.getModels() ) ;

      if ( ( models != null ) && ( models.size() > 0 ) ) {
         int               i         = 0 ;
         int               modelSize = models.size() ;
         TypeCategoryRec   brand     = null ;
         TypeCategoryRec   model     = null ;
         TypeCategoryRec   family    = null ;
         TypeCategoryRec   machine   = null ;


         selectDlg.setEnabled ( false ) ;
         selectDlg.setText ( "Refreshing Models ..." ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, get all distinct brands, families, machines and  */
         /* sub categories from the models                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < modelSize ) {
            model = (TypeCategoryRec) models.elementAt(i) ;
            machine = TypeCategory.getParentCategory ( model, TypeCategory.LEVEL_MACHINE ) ;
            if ( machine == null )
               machine = TypeCategory.getParentCategory ( model, TypeCategory.LEVEL_SUBCATEGORY ) ;

            if ( machine != null ) {
               family   = TypeCategory.getParentCategory ( machine, TypeCategory.LEVEL_FAMILY ) ;
               brand    = TypeCategory.getParentCategory ( family, TypeCategory.LEVEL_BRAND ) ;

               if ( !brands.contains ( brand ) )
                  brands.addElement ( brand ) ;
               if ( !families.contains ( family ) )
                  families.addElement ( family ) ;
               if ( !machines.contains ( machine ) )
                  machines.addElement ( machine ) ;

            } else
               LogSystem.log ( 1, "Could not find Machine or Sub Categorgy parent for Model:"+model.getInd()+" ("+model.getDescript()+")" ) ;

            i++ ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, sort each vector                                   */            
         /*                                                           */
         /*-----------------------------------------------------------*/
         new QuickSort ( brands ) ;
         new QuickSort ( families ) ;
         new QuickSort ( machines ) ;
         new QuickSort ( models ) ;           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, create our tree                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         createModelTreeModel ( brands, families, machines, models ) ;
      } /* if models  */
      else {
         tre_MODELS.setModel ( createNoModelTreeModel() ) ;
      }

      selectDlg.setText ( "" ) ;
      selectDlg.setEnabled ( true ) ;
   } /* refreshModels  */


   /**
     *  This method will create a blank tree model for use when the document
     * we are displaying for has no direct models associated
     **/
   private DefaultTreeModel createNoModelTreeModel () {
      StringTreeNode rootNode = new StringTreeNode ( Str.getStr ( STR_NO_MODELS_ASSOCIATED ) ) ;

      return( new DefaultTreeModel ( rootNode ) ) ;
   }

   /**
     *  This method will create a blank tree model for use when the document
     * we are displaying for has no direct machines associated
     **/
   private DefaultTreeModel createNoMachineTreeModel () {
      StringTreeNode rootNode = new StringTreeNode ( Str.getStr ( STR_NO_MACHINES ) ) ;

      return( new DefaultTreeModel ( rootNode ) ) ;
   }


   /**
     *  This method will create a tree model based on the sorted Brands, Families, Machines
     * and Models passed in
     *
     * @param brands The list of brands
     * @param families The list of families
     * @param machines The list of machines
     * @param models The list of models
     **/
   private DefaultTreeModel createModelTreeModel ( Vector brands, Vector families, Vector machines, Vector models ) {
      StringTreeNode   rootNode   = new StringTreeNode ( Str.getStr ( STR_MODELS_AND_PARTS ) ) ;
      DefaultTreeModel treeModel  = new DefaultTreeModel ( rootNode ) ;
      int              i          = 0 ;
      int              size       = brands.size() ;
      TypeCategoryRec  brand      = null ;
      StringTreeNode   brandNode  = null ;         
      int              count      = 0 ;                                    
      Vector           brandNodes = new Vector ( 1 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Process each brand. By doing this each Family           */
      /* machine and model will be processed                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         brand = (TypeCategoryRec)brands.elementAt(i) ;
         treeModel.insertNodeInto ( brandNode = new StringTreeNode ( brand.getDescript(), brand ), 
                                    rootNode, count++ ) ;
         insertFamiliesIntoModel ( treeModel, brandNode, families, machines, models ) ;
         brandNodes.addElement ( brandNode ) ;

         i++ ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our model                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      tre_MODELS.setModel ( treeModel ) ;                    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, expand all of the brands                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      WinUtil.expandTree ( tre_MODELS ) ;
      tre_MODELS.setCellRenderer ( treeRenderer = new DefaultTreeRenderer () ) ;


      return( treeModel ) ;
   }


   /**
     *  This method will add each family into the tree model passed. In doing this,
     * all machines/sub categories and models will be added under each family
     *
     * @param treeModel The DefaultTreeModel to alter
     * @param brand The brand to use
     * @param families The list of families
     * @param machines The list of machines
     * @param models The list of models
     **/
   private void insertFamiliesIntoModel ( DefaultTreeModel treeModel, StringTreeNode brand, Vector families, Vector machines, Vector models ) {
      int             i          = 0 ;
      int             size       = families.size() ;
      StringTreeNode  familyNode = null ;
      int             brandInd   = ((TypeCategoryRec)brand.getData()).getInd() ;
      TypeCategoryRec family     = null ;
      int             count      = 0 ;

      while ( i < size ) {

         family = (TypeCategoryRec)families.elementAt(i) ;
         if ( family.getParentCatInd() == brandInd ) {
            familyNode = new StringTreeNode ( family.toString(), family ) ;
            treeModel.insertNodeInto ( familyNode, brand, count++ ) ;
            insertMachinesIntoModel ( treeModel, familyNode, machines, models ) ;
         }
         i++ ;
      }
   }


   /**
     *  This method will add each machine or Sub category under the Family passed. In doing this,
     * all models will be added under each machine/sub category
     *
     * @param treeModel The DefaultTreeModel to alter
     * @param family The family to use
     * @param machines The list of machines
     * @param models The list of models
     **/
   private void insertMachinesIntoModel ( DefaultTreeModel treeModel, StringTreeNode family, Vector machines, Vector models ) {
      int               i           = 0 ;
      int               size        = machines.size() ;
      StringTreeNode    machineNode = null ;
      int               familyInd   = ((TypeCategoryRec)family.getData()).getInd() ;
      TypeCategoryRec   machine     = null ;
      int               count       = 0 ;

      while ( i < size ) {

         machine = (TypeCategoryRec)machines.elementAt(i) ;
         if ( TypeCategory.getParentCategory ( machine, TypeCategory.LEVEL_FAMILY ).getInd() == familyInd ) {
            machineNode = new StringTreeNode ( machine.toString(), machine ) ;
            treeModel.insertNodeInto ( machineNode, family, count++ ) ;
            insertModelsIntoModel ( treeModel, machineNode,  models ) ;
         }
         i++ ;
      }
   }

   /**
     *  This method will add each model or part number under the machine passed.
     *
     * @param treeModel The DefaultTreeModel to alter
     * @param machine The machine to use
     * @param models The list of models
     **/
   private void insertModelsIntoModel ( DefaultTreeModel treeModel, StringTreeNode machine, Vector models ) {
      int               i            = 0 ;
      int               size         = models.size() ;
      StringTreeNode    modelNode    = null ;
      int               machineInd   = ((TypeCategoryRec)machine.getData()).getInd() ;
      TypeCategoryRec   model        = null ;
      int               count        = 0 ;

      while ( i < size ) {

         model = (TypeCategoryRec)models.elementAt(i) ;
         if ( model.getParentCatInd() == machineInd ) {
            modelNode = new StringTreeNode ( model.toString(), model ) ;
            treeModel.insertNodeInto ( modelNode, machine, count ) ;
            count++ ;
         }
         i++ ;
      }
   }

   /**
     *  This method will associate the selected machines as 'All' associations
     **/
   private void associateAll () {
      TreePath[] selection = tre_MACHINES.getSelectionPaths() ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only process if we have selection                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( selection != null ) && ( selection.length > 0 ) ) {
         Vector machines           = document.getMachines() ;
         Vector applicableMachines = document.getApplicableMachines() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   We don't want notifications to go out here              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         document.setPropertyChangeEnabled ( false ) ;                        

         /* Create the vectors if the document doesn't have m  */
         if ( machines == null )
            machines = new Vector ( 1 ) ;
         if ( applicableMachines == null )
            applicableMachines = new Vector ( 1 ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through the selection                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         for ( int i = 0; i < selection.length; i++ ) {
            StringTreeNode  node       = (StringTreeNode)selection[i].getLastPathComponent() ;
            TypeCategoryRec machine    = (TypeCategoryRec) node.getData() ;


            if ( !machines.contains ( machine ) )
               machines.addElement ( machine ) ;

            if ( applicableMachines.contains ( machine ) ) {
               applicableMachines.removeElement ( machine ) ;
               removeModelsFromParent ( machine.getInd(), machine.getCategoryLevel() ) ;
            }
         }

         document.setApplicableMachines ( applicableMachines) ;
         document.setPropertyChangeEnabled ( true ) ;
         document.setMachines ( machines ) ;
      }
   }


   /**
     *  This method will remvoe all models in the vector passed that belong
     * to the machine passed
     *
     * @param machine The Machine or Subcategory to remove from
     * @param models The vector of models
     **/
   private void removeModels ( TypeCategoryRec machine, Vector models ) {
      int i    = 0 ;
      int size = models.size() ;

      while ( i < size ) {
         if ( TypeCategory.getParentCategory ( (TypeCategoryRec)models.elementAt(i), 
                                               TypeCategory.LEVEL_MACHINE ).getInd() == machine.getInd() ) {
            models.removeElementAt ( i ) ;
            size--  ;
         } else
            i++ ;
      }
   }

   /**
     *  This method will be called when the user makes a selection in the model tree
     **/
   public void valueChanged ( TreeSelectionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Model Tree                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == tre_MODELS ) {
         try {
            StringTreeNode selectedValue = (StringTreeNode)tre_MODELS.getSelectionPath().getLastPathComponent() ;

            pb_DEASSOC.setEnabled ( ( selectedValue != null ) && ( selectedValue.getData() != null ) ) ;
         } catch ( Exception e1 ) {
            // This exception is thrown when nothing is selected
            pb_DEASSOC.setEnabled ( false ) ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Machine Tree                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == tre_MACHINES ) {
         try {
            StringTreeNode  selectedValue = (StringTreeNode)tre_MACHINES.getSelectionPath().getLastPathComponent() ;
            TypeCategoryRec     machine   = (TypeCategoryRec)selectedValue.getData() ;

            pb_ASSOC.setEnabled ( machine != null ) ;
            pb_ASSOC_ALL.setEnabled ( machine != null ) ;
         } catch ( Exception e1 ) {
            // This exception is thrown when nothing is selected
            pb_ASSOC.setEnabled ( false ) ;
            pb_ASSOC_ALL.setEnabled ( false ) ;
         }

      }

   }


   /**
     *  This method will deassociate the currently selected brand, family, machine or
     * model
     **/
   private void deassociate () {
      TreePath[] selection = tre_MODELS.getSelectionPaths() ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   We don't want notifications to go out here              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      document.setPropertyChangeEnabled ( false ) ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only process if we have selection                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( selection != null ) && ( selection.length > 0 ) ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through the selection                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         for ( int i = 0; i < selection.length; i++ ) {
            StringTreeNode  node       = (StringTreeNode)selection[i].getLastPathComponent() ;
            TypeCategoryRec currentRec = (TypeCategoryRec) node.getData() ;                         


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is a model, deassociate it                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( currentRec.getCategoryLevel().equals ( TypeCategory.LEVEL_MODEL ) ) ||
                 ( currentRec.getCategoryLevel().equals ( TypeCategory.LEVEL_PARTNUMBER ) ) ) {
               Vector models = document.getModels() ;

               models.removeElement ( currentRec ) ;
               document.setModels ( models ) ;
            }

            else {
               removeModelsFromParent ( currentRec.getInd(), currentRec.getCategoryLevel() ) ;
            }
         }

         refreshData() ;

      } else
         GUISystem.printBox ( WinUtil.getParentJDialog ( this ),
                              STR_INFO,
                              STR_MUST_SELECT_ITEM ) ;

      document.setPropertyChangeEnabled ( true ) ;
   }


   /**
     *  This method will remove all models under the passed parent ind
     **/
   private void removeModelsFromParent ( int parentInd, String level ) {
      Vector models = document.getModels() ;

      if ( ( models != null ) && ( models.size() > 0 ) ) {
         int modelIndex = 0 ;
         int size       = models.size() ;

         while ( modelIndex < size ) {
            TypeCategoryRec parent = TypeCategory.getParentCategory ( (TypeCategoryRec)models.elementAt ( modelIndex ), level ) ;

            if ( ( parent != null ) && ( parent.getInd() == parentInd ) ) {
               models.removeElementAt ( modelIndex ) ;
               size-- ;
               modelIndex=-1 ;
            }
            modelIndex++ ;
         }
      }
      document.setModels ( models ) ;
   }


   /**
     *  This method will read in all descriptions for all models passed in the
     * model vector. It will then return a New vector with New TypeCategoryObjects
     *
     * @param models The vector of models
     **/
   private Vector getModelsAndDescriptions ( Vector models ) {
      Vector    newModels = new Vector ( 1 ) ;

      if ( ( models != null ) && ( models.size() > 0 ) ) {
         SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getModelsAndDescriptions", 5 ) ;

         try {
            int       size = models.size() ;
            Statement stmt = sqlMethod.createStatement() ;
            ResultSet rs   = null ;
            String    sql  = "SELECT B.TYPECATIND, A.DESCRIPTION "+
                             "FROM PRODUCT.PRODUCT A, "+
                             "     PRODUCT.CATEGORY B "+
                             "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
                             "      B.TYPECATIND IN ( " ;

            for ( int i = 0; i < size; i++ ) {
               sql+=""+((TypeCategoryRec)models.elementAt(i)).getInd() ;
               if ( i < (size-1) )
                  sql+="," ;
            }
            sql+=") FOR FETCH ONLY" ;

            rs = stmt.executeQuery ( sql ) ;                              

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, read in and create our new records                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( rs.next() ) {
               TypeCategoryRec newRec = null ;

               try {
                  newRec = (TypeCategoryRec)getCategoryRec ( rs.getInt ( 1 ), models ).clone() ;
               } catch ( NullPointerException e ) {
                  System.out.println ( "Huh, could not find category for typeCatInd:"+rs.getInt(1) ) ;
               }

               newRec.setToString ( newRec.getDescript()+" ("+rs.getString(2).trim()+")" ) ;
               newModels.addElement ( newRec ) ;

            }


            rs.close() ;

         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            sqlMethod.rollBack() ;
            GUISystem.printBox ( WinUtil.getParentJDialog ( this ), STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
         }

         sqlMethod.close() ;

      }

      return( newModels ) ;
   }


   /**
     *  This method will simply return the TypeCategoryRec representing
     * the indicator passed in
     *
     * @param getCategoryRec
     **/
   private TypeCategoryRec getCategoryRec ( int ind, Vector categories ) {
      TypeCategoryRec categoryRec = null ;

      if ( ( categories != null ) && ( categories.size() > 0 ) ) {
         int i    = 0 ;
         int size = categories.size() ;

         while ( ( categoryRec == null ) && ( i < size ) ) {
            if ( ((TypeCategoryRec)categories.elementAt(i)).getInd() == ind )
               categoryRec = (TypeCategoryRec)categories.elementAt(i) ;
            i++ ;
         }
      }

      return( categoryRec ) ;
   }
}
