package com.ibm.nzna.projects.qit.doc.docedit ;                                
/*-----------------------------------------------------------*/
/*   ProductSelectModelPanel                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductSelectModelPanel             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This panel can hold a list of models/part numbers that the 
 * user can select multiple of. 
 *
 *
 * @version 1.0, 8/15/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductSelectModelPanel                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductSelectModelPanel extends JPanel implements ListSelectionListener, Runnable, AppConst, ActionListener, DocConst {


   /**
     *  This is the vector of Products that we are displaying
     **/
   private Vector products = null ;

   /**
     *  This is the help label which tells the user what they are looking at
     **/
   private JLabel st_HELP = new JLabel ( Str.getStr ( STR_PLEASE_SELECT_SYSTEM_OPTION ) ) ;

   /**
     *  This is the Multilist which holds the models/parts
     **/
   private ProductTable cnr_PRODUCTS = new ProductTable ( ) ;

   /**
     *  This is the Ok Button
     **/
   private DButton pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     *  This is the Cancel Button
     **/
   private DButton pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;

   /**
     *  This is the ProductSelectDlg which this panel is a part of
     **/
   private ProductSelectDlg selectDlg = null ;

   /**
     *  Holds the Ok and Cancel buttons
     **/
   ButtonPanel buttonPanel = new ButtonPanel() ; 

   /**
     *  This is the Machine that we are displaying products under. Please note
     * that this could be a SubCategory OR Machine.
     **/
   private Object[] machineArray = null ;


   /**
     *  This is the document that we are editing
     **/
   private Document document = null ;

   /**
     *  Default constructor
     *
     * @param selectDlg The ProductSelectDlg this panel is in
     * @param machineArray The Machine to read products under
     * @param document The document to set products for
     **/
   protected ProductSelectModelPanel ( ProductSelectDlg selectDlg, Object[] machineArray, Document document ) {

      this.selectDlg = selectDlg ;
      this.machineArray   = machineArray ;
      this.document  = document ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      cnr_PRODUCTS.addActionListener ( this ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      setLayout ( null ) ;
      add ( st_HELP, BorderLayout.NORTH ) ;
      add ( cnr_PRODUCTS, BorderLayout.CENTER ) ;
      add ( buttonPanel, BorderLayout.SOUTH ) ;
      GUISystem.setPropertiesOnPanel ( this ) ;

      new Thread ( this ).start() ;
   }


   /**
     *  This method will position and lay out all of our controls
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       y         = 5 ;

      super.doLayout() ;

      st_HELP.setBounds ( 0, 0, size.width, rowHeight ) ;
      y+=rowHeight ;

      cnr_PRODUCTS.setBounds ( 0, y, size.width, size.height-y-29 ) ;

      buttonPanel.setBounds ( 0, size.height-25, size.width, 25 ) ;
   }


   /**
     *  This method will respond to ActionEvents being fired on us
     * 
     * @param e The Event
     **/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == pb_CANCEL )
         selectDlg.closeModelPanel () ;

      else if ( ( e.getSource() == pb_OK ) || ( e.getSource() == cnr_PRODUCTS ) ) {
         saveData() ;
         selectDlg.closeModelPanel () ;
      }
   }


   /**
     *  This method will read in all Products under the machineArray that we are showing
     **/
   private boolean readProducts () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readProducts", 5 ) ;
      boolean   rc        = false ;
      String    inClause  = "(" ;                                         
      boolean   isOption  = false ;

      pb_OK.setEnabled ( false ) ;
      pb_CANCEL.setEnabled ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the 'In' Clause containing all machine inds      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      for ( int i = 0; i < machineArray.length; i++ ) {
         inClause+=((TypeCategoryRec)machineArray[i]).getInd()+"" ;
         if ( ((TypeCategoryRec)machineArray[i] ).getCategoryLevel().equals ( TypeCategory.LEVEL_SUBCATEGORY ) )
            isOption = true ;
         if ( i < (machineArray.length-1) )
            inClause+="," ;
      }
      inClause+=")" ;

      try {
         Statement stmt = sqlMethod.createStatement () ;
         ResultSet rs   = null ;


         if ( isOption )
            rs = stmt.executeQuery ( "SELECT A.PRODUCTIND,    A.PRODNUM,   A.DESCRIPTION, "+
                                     "       A.CREATEDON,     A.CREATEDBY, A.LASTTOUCHEDBY, "+
                                     "       A.LASTTOUCHEDON, C.DESCRIPTION "+
                                     "FROM PRODUCT.PRODUCT A,  "+
                                     "     PRODUCT.CATEGORY B, "+
                                     "     PRODUCT.TYPECATEGORY C "+
                                     "WHERE B.TYPECATIND    IN "+inClause+" AND "+
                                     "      A.PRODUCTIND    = B.PRODUCTIND AND "+
                                     "      A.PUBLISH       = 'Y' AND "+
                                     "      C.PARENTCATIND  = B.TYPECATIND AND "+
                                     "      C.CATEGORYLEVEL = '"+TypeCategory.LEVEL_PARTNUMBER+"' AND "+
                                     "      C.TYPECATIND IN (SELECT TYPECATIND FROM PRODUCT.CATEGORY WHERE PRODUCTIND = A.PRODUCTIND) "+
                                     "      ORDER BY C.DESCRIPTION" ) ;
         else
            rs = stmt.executeQuery ( "SELECT A.PRODUCTIND,    A.PRODNUM,   A.DESCRIPTION, "+
                                     "       A.CREATEDON,     A.CREATEDBY, A.LASTTOUCHEDBY, "+
                                     "       A.LASTTOUCHEDON "+
                                     "FROM PRODUCT.PRODUCT A,  "+
                                     "     PRODUCT.CATEGORY B "+
                                     "WHERE B.TYPECATIND IN "+inClause+" AND "+
                                     "      A.PUBLISH    = 'Y' AND "+
                                     "      A.PRODUCTIND = B.PRODUCTIND ORDER BY A.PRODNUM" ) ;


         products = new Vector ( 1 ) ;
         while ( rs.next() ) {
            products.addElement ( new ProductRow ( rs.getInt(1),
                                                   0,
                                                   isOption ? rs.getString ( 8 ).trim() : rs.getString(2).trim(),
                                                   rs.getString(3).trim(),
                                                   rs.getString(4).trim(),
                                                   rs.getString(5).trim(),
                                                   null,
                                                   null,
                                                   rs.getString(6).trim(),
                                                   rs.getString(7).trim(),
                                                   false ) ) ;
         }

         rs.close() ;
         rc = true ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( WinUtil.getParentJDialog ( this ), STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      pb_OK.setEnabled ( true ) ;
      pb_CANCEL.setEnabled ( true ) ;

      return( rc ) ;
   }




   /**
     *  This thread will read in all of our products and populate our list
     **/
   public void run () {
      if ( machineArray.length == 1 )
         selectDlg.setText ( Str.getStr ( STR_READING ) ) ;
      else
         selectDlg.setText ( Str.getStr ( STR_READING )+" "+machineArray.length+" "+Str.getStr ( STR_MACHINES_AND_SUB_CATEGORIES ) ) ;

      try {
         if ( readProducts() )
            cnr_PRODUCTS.setProducts ( products ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

      selectDlg.setText ( null ) ;
   }


   /**
     *  This method will be called when the user selects an item in our list
     **/
   public void valueChanged ( ListSelectionEvent e ) {
      pb_OK.setEnabled ( cnr_PRODUCTS.getSelectedItem() != null ) ;
   }



   /**
     *  This method will take all selected products and add them to the document
     **/
   private void saveData () {
      Vector selectedProducts = cnr_PRODUCTS.getSelection() ;

      pb_OK.setEnabled ( false ) ;
      pb_CANCEL.setEnabled ( false ) ;

      try {
         if ( ( selectedProducts != null ) && ( selectedProducts.size() > 0 ) ) {
            selectDlg.setText ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
            readProductModels ( selectedProducts ) ;
            document.addProductRows ( selectedProducts ) ;
            selectDlg.setText ( null ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( WinUtil.getParentJDialog ( this ), STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      pb_OK.setEnabled ( true ) ;
      pb_CANCEL.setEnabled ( true ) ;
   }



   /**
     *  This method will read in and set the Model property on all ProductRow objects
     * passed in the productRows param
     *
     * @param productRows A Vector of ProductRow Objects
     **/
   private boolean readProductModels ( Vector productRows ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readProductModels", 5 ) ;
      boolean   rc        = false ;

      try {
         Statement  stmt     = sqlMethod.createStatement () ;
         ResultSet  rs       = null ;
         String     inClause = "(" ;
         int        i        = 0 ;
         int        size     = productRows.size() ;
         ProductRow row      = null ;                                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the 'In' clause we will use in our query         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {
            inClause+=((ProductRow)productRows.elementAt(i)).getProductInd()+"" ;

            if ( i < (size-1) )
               inClause+="," ;
            i++ ;
         }
         inClause+=")" ;


         rs = stmt.executeQuery ( "SELECT A.TYPECATIND, A.PRODUCTIND, B.DESCRIPTION, B.CATEGORYLEVEL, B.ORDERNUM, B.PARENTCATIND "+
                                  "FROM PRODUCT.CATEGORY A, "+
                                  "     PRODUCT.TYPECATEGORY B "+
                                  "WHERE A.PRODUCTIND IN "+inClause+" AND "+
                                  "      A.TYPECATIND = B.TYPECATIND AND "+
                                  "      B.CATEGORYLEVEL IN ('"+TypeCategory.LEVEL_MODEL+"', '"+TypeCategory.LEVEL_PARTNUMBER+"') "+
                                  "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            row = getProductRow ( rs.getInt ( 2 ), productRows ) ;
            if ( row != null ) {
               TypeCategoryRec model = new TypeCategoryRec ( rs.getInt(1) ) ;

               model.setDescript ( rs.getString(3).trim() ) ;
               model.setCategoryLevel ( rs.getString(4).trim() ) ;
               model.setOrderNum ( rs.getInt ( 5 ) ) ;
               model.setParentCatInd ( rs.getInt ( 6 ) ) ;  
               model.updateRecStatus ( TypeCategoryRec.REC_NOCHANGE ) ; // Just a hack in case
               row.setModel ( model ) ;
            }
         }

         rs.close() ;
         rc = true ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( WinUtil.getParentJDialog ( this ), STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }

   private ProductRow getProductRow ( int productInd, Vector productRows ) {
      ProductRow productRow = null ;
      int        i          = 0 ;
      int        size       = productRows.size() ;

      while ( ( i < size ) && ( productRow == null ) ) {
         if ( ((ProductRow)productRows.elementAt(i)).getProductInd() == productInd )
            productRow = (ProductRow)productRows.elementAt(i) ;
         i++ ;
      }

      return( productRow ) ;
   }



} /* ProductSelectModelPanel */
