package com.ibm.nzna.projects.qit.doc.docedit ;                             
/*-----------------------------------------------------------*/
/*   PublishingOptionsPanel                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PublishingOptionsPanel              */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The PublishingOptionsPanel holds all of the publishing options, this 
 * information includes:<br><br>
 *
 * Webable<br>
 * Online Assistant<br>
 * Hot News<br>
 * Months To Review<br>
 * Filename<br>
 *
 * @version 1.0, 5/2/2001            
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   PublishingOptionsPanel                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class PublishingOptionsPanel extends DPanel implements Runnable, DocumentPropertyListener, FocusListener, ActionListener, AppConst {

   /**
     *  this thread saves our data
     **/
   private static final String THREAD_SAVE = "SAVE" ;

   /**
     *  This is the last focus event that was sent to us
     **/
   private FocusEvent lastFocusEvent = null ;

   /**
     *  This is the Webable check box
     **/
   private JCheckBox webable = new JCheckBox ( Str.getStr ( STR_WEBABLE ) ) ;

   /**
     *  This is the Online Assistant Check Box
     **/
   private JCheckBox oa = new JCheckBox ( Str.getStr ( STR_KNOWLEDGE_BASE ) ) ;

   /**
     * This is the entry field that holds the data for Months To Review
     **/
   private JTextField monthsToReview = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 2 ), "", 0 ) ;

   /**
     *  This is the JTextField which holds the Filename
     **/
   private JTextField filename = new JTextField ( ) ;

   /**
     *  This is the ValueLabel which holds the filename
     **/
   private ValueLabel filenameLabel = new ValueLabel ( Str.getStr ( STR_FILENAME ),
                                                       filename ) ;

   /**
     *  This is the Months To Review Value Label
     **/
   private ValueLabel monthsToReviewLabel = new ValueLabel ( Str.getStr ( STR_MONTHS_TO_REVIEW ),
                                                             monthsToReview ) ;

   /**
     *  This is the value label that will hold the Hot News Information
     **/
   private ValueLabel hotNewsInfo = null ;

   /**
     *  This is the Panel that will hold the Hot News Check Box and the
     * Hot News Entry Field
     **/
   private JPanel hotNewsPanel = new JPanel ()  ;

   /**
     *  This is the check box for hot news
     **/
   private JCheckBox hotNews = new JCheckBox ( "" ) ;

   /**
     *  This is the entry field for the hot news date
     **/
   private DateEntryField hotNewsDate = new DateEntryField ( ) ;


   /**
     * This is the document we are editing
     **/
   private Document document = null ;

   /**
     * Determines if the filename can be edited throughout the life of this GUI control
     **/
   private boolean filenameCanBeChanged = false ;




   /**
     *  This constructor creates our blank panel
     **/
   public PublishingOptionsPanel () {

      /**
        *  Create the Hot News Controls
        **/
      hotNewsPanel.setLayout ( new BorderLayout () ) ;
      hotNewsPanel.add ( hotNews, BorderLayout.WEST ) ;
      hotNewsPanel.add ( hotNewsDate, BorderLayout.CENTER ) ;
      hotNewsInfo = new ValueLabel ( "Hot News/Expiration Date", hotNewsPanel ) ;

      /**
        *  Set properties
        **/
      setPreferredSize ( new Dimension ( 490, (GUISystem.getRowHeight()*3) ) ) ;
      filename.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, "VVVV-VVVVVV", filename ) ) ;

      /**
        * Add listeners
        **/
      hotNewsDate.addFocusListener  ( this ) ;
      hotNews.addActionListener ( this ) ;
      oa.addActionListener ( this ) ;
      webable.addActionListener ( this ) ;
      monthsToReview.addFocusListener ( this ) ;
      filename.addFocusListener ( this ) ;

      /**
        *  Add the GUI
        **/
      setLayout ( null ) ;
      setBackground ( Color.white );
      webable.setBackground ( Color.white );
      oa.setBackground ( Color.white );
      hotNewsInfo.setBackground ( Color.white );
      hotNewsInfo.setDescriptionWidth ( 150 ) ;
      hotNews.setBackground ( Color.white ) ;
      monthsToReviewLabel.setDescriptionWidth ( 150 ) ;
      filenameLabel.setDescriptionWidth ( 150 ) ;
      add ( webable ) ;
      add ( oa ) ;
      add ( hotNewsInfo ) ;
      add ( monthsToReviewLabel ) ;
      add ( filenameLabel ) ;

   }




   /**
     *  This method will lay out and position all of our controls
     **/
   public void doLayout ( ) {
      Dimension size      = getSize() ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight () ;

      super.doLayout() ;

      webable.setBounds ( 0, y, size.width/2, rowHeight ) ;
      hotNewsInfo.setBounds ( size.width/2, y, size.width/2, rowHeight ) ;
      y+=rowHeight ;

      oa.setBounds ( 0, y, size.width/2, rowHeight ) ;
      monthsToReviewLabel.setBounds ( size.width/2, y, 180, rowHeight ) ;
      y+=rowHeight ;

      filenameLabel.setBounds ( 0, y, 300, rowHeight ) ;
   }           


   /**
     *  This method will set the information we are displaying to the
     * information that is in the Document Record passed
     **/
   protected void setDocument ( Document document ) {

      if ( this.document != document ) {
         if ( this.document != null )
            this.document.removeDocumentPropertyListener ( this ) ;
         this.document = document ;
         document.addDocumentPropertyListener ( this ) ;
      }


      refresh() ;
   }

   private void refresh () {

      webable.setSelected ( document.getWebable() ) ;
      oa.setSelected ( document.getKnowledgeBase() ) ;
      hotNews.setSelected ( ( document.getHotNewsDate() != null ) && ( document.getHotNewsDate().length() > 0 )  ) ;
      if ( hotNews.isSelected() )
         hotNewsDate.setText ( document.getHotNewsDate() ) ;
      monthsToReview.setText ( ""+document.getMonthsToReview() ) ;

      if ( document.getFilename() != null ) {
         filename.setText ( document.getFilename () ) ;
         if ( filenameCanBeChanged )
            filename.setEditable ( true ) ;
         else
            filename.setEditable ( false ) ;
      } else {
         filename.setText ( "" ) ;
         filename.setEditable ( true ) ;
      }
   }

   /**
     *  This method will be called when the user changes the hot news check box
     **/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == hotNews ) {
         hotNewsDate.setEnabled ( hotNews.isSelected() ) ;
         if ( !hotNews.isSelected() )
            hotNewsDate.setText ( "" ) ;
         else {
            hotNewsDate.requestFocus() ;
            hotNewsDate.selectAll() ;
         }
      }

      /*-----------------------------*/
      /*                             */
      /*  Online Assistant Check Box */
      /*                             */
      /*-----------------------------*/
      else if ( e.getSource() == oa ) {
         document.setKnowledgeBase ( oa.isSelected() );
      }


      /*--------------------*/
      /*                    */
      /*  Webable Check Box */
      /*                    */
      /*--------------------*/
      else if ( e.getSource() == webable ) {
         document.setWebable ( webable.isSelected() ) ;
      }

   }


   /**
     * This method does nothing
     **/
   public void focusGained ( FocusEvent e ) {
   }

   /**
     * This method will save the data in the field that lost focus
     **/
   public void focusLost ( FocusEvent e ) {
      lastFocusEvent = e ;
      new Thread ( this, THREAD_SAVE ).start() ;
   }

   public void run () {
      String operation = Thread.currentThread().getName () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Thread Save                                             */  
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_SAVE ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Hot News                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( lastFocusEvent.getSource() == hotNewsDate ) {
            document.setHotNewsDate ( hotNewsDate.getText() ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Months to review                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( lastFocusEvent.getSource() == monthsToReview ) {
            document.setMonthsToReview ( new Integer ( monthsToReview.getText() ).intValue() ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Filename                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( lastFocusEvent.getSource() == filename ) {
            if ( filename.isEditable() ) {
               String newFilename = filename.getText() ;

               filenameCanBeChanged = true ;
               if ( newFilename.trim().length() >4 ) {
                  try {
                     if ( ( DocumentSQL.getExistingDocInd ( newFilename ) == 0 ) &&
                          ( DocumentSQL.getDraftDocInd ( newFilename ) == 0 ) ) {
                        ((DocumentDraft)document).setFilename ( newFilename ) ;
                     } else {
                        GUISystem.printBox ( "Info", "That filename is currently in use by another document" ) ;
                     }
                  } catch ( Exception e1 ) {
                     LogSystem.log ( 1, e1, false ) ;
                     filename.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, "VVVV-VVVVVV", filename ) ) ;
                     GUISystem.printBox ( "Info", "The filename you have entered cannot be checked for duplicates. The filename has been reset" ) ;

                  }
               }
            }
         }
      } /* SAVE  */


   }


   public void documentChanged ( Document document, int property ) {
      refresh() ;
   }


   public void setEnabled ( boolean enable ) {
      webable.setEnabled ( enable ) ;
      oa.setEnabled ( enable ) ;
      monthsToReview.setEnabled ( enable ) ;
      hotNews.setEnabled ( enable ) ;
      hotNewsDate.setEnabled ( enable ) ;
   }
} /* PublishingOptionsPanel */       
