package com.ibm.nzna.projects.qit.doc.docedit ;   
/*-----------------------------------------------------------*/
/*   ReimportDlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ReimportDlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.Vector ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog reports the status of re-import for a Document's Body 
 *
 *
 * @version 1.0, 2/13/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ReimportDlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ReimportDlg extends JDialog implements AppConst, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_GRAPHICS_FOUND    = null ;
   private JLabel      stt_GRAPHICS_FOUND   = null ;
   private JLabel      st_GRAPHIC_NAMES     = null ;
   private JLabel      st_LINKS_FOUND       = null ;
   private JLabel      stt_LINKS_FOUND      = null ;
   private JLabel      st_LINK_NAMES        = null ;
   private JList       graphicList          = null ;
   private JList       linkList             = null ;
   private DButton     pb_OK                = null ;      
   private JScrollPane linkScroll           = null ;
   private JScrollPane graphicScroll        = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show our window.
    *  It will not return until the user closes this window
    *  
    *
    *  @param parentWin The parent window for this dialog
    *  @param graphicVec The vector of graphics to show
    *  @param linkVec The vector of links to show
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ReimportDlg ( AppDefaultWin parentWin, Vector graphicVec, Vector linkVec ) {
      super ( parentWin, Str.getStr ( STR_IMPORT_RESULTS ), true ) ;     
      Container contentPane = getContentPane () ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Take out the deleted data                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( graphicVec != null )
         graphicVec = QuestUtil.getValidData ( graphicVec ) ;
      
      if ( linkVec != null )
         linkVec    = QuestUtil.getValidData ( linkVec ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_GRAPHICS_FOUND    = new JLabel ( Str.getStr ( STR_GRAPHICS_FOUND ) ) ;
      stt_GRAPHICS_FOUND   = (graphicVec != null ) ? new JLabel ( ""+graphicVec.size() ) : new JLabel ( "0" ) ;
      st_GRAPHIC_NAMES     = new JLabel ( Str.getStr ( STR_GRAPHIC_NAMES ) ) ;
      st_LINKS_FOUND       = new JLabel ( Str.getStr ( STR_LINKS_FOUND ) ) ;
      stt_LINKS_FOUND      = (linkVec != null ) ? new JLabel ( ""+linkVec.size() ) : new JLabel ( "0" ) ;
      st_LINK_NAMES        = new JLabel ( Str.getStr ( STR_LINK_NAMES ) ) ;
      graphicList          = (graphicVec != null ) ? new JList ( graphicVec ) : new JList() ;
      linkList             = (linkVec != null ) ? new JList ( linkVec ) : new JList() ;
      pb_OK                = new DButton ( Str.getStr ( STR_OK ) ) ;    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;  



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_GRAPHICS_FOUND ) ;
      contentPane.add ( stt_GRAPHICS_FOUND ) ;
      contentPane.add ( st_GRAPHIC_NAMES ) ;
      contentPane.add ( st_LINKS_FOUND ) ;
      contentPane.add ( stt_LINKS_FOUND ) ;
      contentPane.add ( st_LINK_NAMES ) ;
      contentPane.add ( graphicScroll = new JScrollPane ( graphicList ) ) ;
      contentPane.add ( linkScroll = new JScrollPane ( linkList ) ) ;
      contentPane.add ( pb_OK ) ;


      setSize ( 350, 300 ) ;
      WinUtil.centerChildInParent ( this, parentWin ) ;
      GUISystem.setPropertiesOnPanel ( this ) ;
      setVisible ( true ) ;
   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all
    *  of our control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout () ;

      size.width-=5 ;

      st_GRAPHICS_FOUND.setBounds ( x, y, 150, rowHeight ) ;
      stt_GRAPHICS_FOUND.setBounds ( x+150, y, 100, rowHeight ) ;
      y+=rowHeight ;

      st_GRAPHIC_NAMES.setBounds ( x, y, 150, rowHeight ) ;
      y+=rowHeight ;

      graphicScroll.setBounds ( x, y, size.width-(x*2)-5, rowHeight*3 ) ;
      y+=(rowHeight*4 ) ;

      st_LINKS_FOUND.setBounds ( x, y, 150, rowHeight ) ;
      stt_LINKS_FOUND.setBounds ( x+150, y, 100, rowHeight ) ;
      y+=rowHeight ;

      st_LINK_NAMES.setBounds ( x, y, size.width-(x*2)-5, rowHeight ) ;
      y+=rowHeight ;

      linkScroll.setBounds ( x, y, size.width-(x*2)-5, size.height-65-y ) ;

      pb_OK.setBounds ( x, size.height-50, 70, 25 ) ;

   } /* doLayout  */




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    Any action that is performed and we close
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      dispose() ;
   }

} /* ReimportDlg */
