package com.ibm.nzna.projects.qit.doc.docedit ;
/*-----------------------------------------------------------*/
/*   SaveDocDlg                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SaveDocDlg                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
/**
 * This dialog allows a user to select what type of save that they would like
 *
 *
 * @version 1.0, 4/28/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SaveDocDlg                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class SaveDocDlg extends JDialog implements DataLengths, MouseListener, DocConst, ActionListener, AppConst, Authority {


   /**
     *  This return code is given if the user cancels or
     * closes the window without saving
     **/
   public static final int SAVE_CANCEL  = 0 ;

   /**
     *  THis return code is given if the user saves the document
     * but the document should remain up
     **/
   public static final int SAVE_SAVED = 1 ;

   /**
     *  This return code is given if the user publishes the
     * document
     **/
   public static final int SAVE_PUBLISHED = 2;

   /**
     *  This return code is given if the user saves the
     * docuement as a template
     **/
   public static final int SAVE_TEMPLATE = 3 ;

   /**
     *  This return code is given if the user saves the
     * document as a cancel
     **/
   public static final int SAVE_CANCEL_DRAFT = 4 ;

   /**
     *  This return code is given if the user saves
     * the document as a new document
     **/
   public static final int SAVE_NEW_DOCUMENT = 5 ;

   /**
     *  This is the highlight button background
     **/
   private Color buttonHighlightBackground = null ;

   /**
     *  This is the default button background
     **/
   private Color buttonDefaultBackground = null ;

   /**
     *  This is the save panel we will use
     **/
   private DocSaveStatusPanel statusPanel = null ;

   /**
     * This panel holds the template entry field and name
     **/
   private JPanel templatePanel     = new JPanel() ;

   /**
     * This panel holds the comments and action buttons
     **/
   private JPanel mainPanel   = new JPanel() ;

   /**
     *  This is the JCheck Box stating that the body contents
     * have changed
     **/
   private JCheckBox ck_TRANSLATION = new JCheckBox ( Str.getStr ( STR_TRANSLATION_WARNING ) ) ;

   /**
     * This is the 'Publish' Button
     **/
   private DButton pb_PUBLISH = new DButton ( Str.getStr ( STR_PUBLISH ) ) ;

   /**
     *  This is the 'Publish Now' Button
     **/
   private DButton pb_PUBLISH_NOW = new DButton ( Str.getStr ( STR_PUBLISH_NOW ) ) ;

   /**
     * This is the 'Save' button
     **/
   private DButton pb_SAVE = new DButton ( Str.getStr ( STR_SAVE_AS_DRAFT ) ) ;

   /**
     * This is the 'Cancel Draft' button
     **/
   private DButton pb_CANCEL_DRAFT = new DButton ( Str.getStr ( STR_CANCEL_DRAFT ) ) ;

   /**
     * This is the text above the template name
     **/
   private JLabel st_TEMPLATE_NAME = new JLabel ( Str.getStr ( STR_PLEASE_ENTER_TEMPLATE_NAME ) ) ;

   /**
     * This is the text above the comments
     **/
   private JLabel st_COMMENTS = new JLabel ( Str.getStr ( STR_COMMENT ) ) ;

   /**
     * This is the JTextField that holds the template name
     **/
   private JTextField ef_TEMPLATE_NAME = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ), "", 0 ) ;

   /**
     * This is the 'Save As New Document' button
     **/
   private DButton pb_SAVE_AS_NEW = new DButton ( Str.getStr ( STR_SAVE_AS_NEW_DOCUMENT ) ) ;

   /**
     * This is the 'Save as Template' button
     **/
   private DButton pb_SAVE_AS_TEMPLATE = new DButton ( Str.getStr ( STR_SAVE_AS_TEMPLATE ) ) ;

   /**
     * This is the 'Cancel' button
     **/
   private DButton pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;


   /**
     * This is the MLE which holds the last comments
     **/
   private NoTabJTextArea mle_OLD_COMMENTS = new NoTabJTextArea () ;

   /**
     * This is the MLE which holds the comments
     **/
   private NoTabJTextArea mle_COMMENTS = new NoTabJTextArea() ;

   /**
     * This is the panel which holds the old comments and new comments
     **/
   private JPanel commentPanel = new JPanel () ;

   /**
     * This is the JScrollPane which holds the commentPanel
     **/
   private JScrollPane commentScroll = new JScrollPane ( commentPanel ) ;

   /**
     *  This holds return code which tells what the user did. This property
     * will be one of the SAVE_XXX values defined in this class
     **/
   private int returnCode = SAVE_CANCEL ;

   /**
     *  This is the button that will save the template once a name has
     * been chosen
     **/
   private DButton pb_SAVE_TEMPLATE_NOW = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     * This is the document that will be saved
     **/
   private DocumentDraft document = null ;

   /**
     * This is the picture
     **/
   private JLabel st_IMAGE = null ;

   /**
     * This is the image we are dispplaying
     **/
   private ImageIcon wizardImage = null ;

   /**
     *  These are the comments that the user is allowed to edit
     **/
   private String comments = null ;

   /**
     * This default constructor creates and shows the window. It will
     * not return until a save is complete, a save failed or the user
     * cancels. At that time, use the getReturnCode() method to determine
     * what went on
     *
     * @param parentWin - The parent window
     * @param document The DOcument to save
     * @param comments The comments that the user is allowed to edit
     **/
   public SaveDocDlg ( AppDefaultWin parentWin, DocumentDraft document, String comments ) {
      super ( parentWin, Str.getStr ( STR_SAVE ), true ) ;

      /*---------------------------------------*/
      /*                                       */
      /*  Assign our local vars and initialize */
      /*                                       */
      /*---------------------------------------*/
      this.document = document ;
      this.comments = comments ;
      initialize() ;

      setSize ( 495, 305 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;
   }


   /**
     * This method initializes all properties, including the GUI, on this
     * window 
     **/
   private void initialize () {

      /*-------------*/
      /*             */
      /*  Create GUI */
      /*             */
      /*-------------*/
      wizardImage = ImageSystem.getImageIcon ( this, ImageSystem.SAVE_DOC ) ;
      st_IMAGE = new JLabel ( wizardImage ) ;

      /*-------------------------------*/
      /*                               */
      /*  Adjust properties on the GUI */
      /*                               */
      /*-------------------------------*/
      mle_OLD_COMMENTS.setEditable ( false ) ;
      mle_OLD_COMMENTS.setBackground ( new Color ( 230, 230, 230 ) ) ;
      mle_OLD_COMMENTS.setFont ( pb_PUBLISH.getFont() ) ;
      mle_COMMENTS.setFont     ( pb_PUBLISH.getFont() ) ;
      buttonDefaultBackground   = pb_SAVE.getBackground() ;
      buttonHighlightBackground = buttonDefaultBackground.brighter() ;                                      
      if ( document.getWorkRequired() == TypeList.WORKREQ_DELETE ) {
         pb_PUBLISH.setText ( "Publish (Delete Active Doc)" ) ;
         pb_PUBLISH_NOW.setText ( "Delete Now!" ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Enable/Disable Controls                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_PUBLISH.setEnabled ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) ;
      pb_PUBLISH_NOW.setEnabled ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) ;

      if ( document.getStatus().getStatus() == Status.TYPESTATUS_CLOSEDNOTAPPROVED ) {
         pb_PUBLISH.setEnabled ( false ) ;
         pb_PUBLISH_NOW.setEnabled ( false ) ;
         pb_SAVE_AS_NEW.setEnabled ( false ) ;
         pb_SAVE.setEnabled ( false ) ;
         pb_SAVE_AS_TEMPLATE.setEnabled ( false ) ;
      }


      /*----------------*/
      /*                */
      /*  Add listeners */
      /*                */
      /*----------------*/
      pb_PUBLISH_NOW.addActionListener       ( this ) ;
      pb_SAVE_TEMPLATE_NOW.addActionListener ( this ) ;
      pb_CANCEL.addActionListener            ( this ) ;
      pb_PUBLISH.addActionListener           ( this ) ;
      pb_CANCEL_DRAFT.addActionListener      ( this ) ;
      pb_SAVE_AS_TEMPLATE.addActionListener  ( this ) ;
      pb_SAVE_AS_NEW.addActionListener       ( this ) ;
      pb_SAVE.addActionListener              ( this ) ;
      pb_SAVE_AS_NEW.addMouseListener        ( this ) ;
      pb_PUBLISH_NOW.addMouseListener        ( this ) ;
      pb_PUBLISH.addMouseListener            ( this ) ;
      pb_CANCEL_DRAFT.addMouseListener       ( this ) ;
      pb_SAVE_AS_TEMPLATE.addMouseListener   ( this ) ;
      pb_SAVE.addMouseListener               ( this ) ;
      ck_TRANSLATION.addActionListener       ( this ) ;                            


      /*--------------*/
      /*              */
      /*  Add our GUI */
      /*              */
      /*--------------*/
      mainPanel.setLayout ( null ) ;
      templatePanel.setLayout ( null ) ;
      commentPanel.setLayout ( new BorderLayout() ) ;
      commentPanel.add ( mle_OLD_COMMENTS, BorderLayout.NORTH ) ;
      commentPanel.add ( mle_COMMENTS ) ;
      mainPanel.add ( st_COMMENTS ) ;
      mainPanel.add ( commentScroll ) ;
      mainPanel.add ( ck_TRANSLATION ) ;
      mainPanel.add ( pb_SAVE ) ;
      mainPanel.add ( pb_SAVE_AS_TEMPLATE ) ;
      mainPanel.add ( pb_SAVE_AS_NEW ) ;
      mainPanel.add ( pb_CANCEL_DRAFT ) ;
      mainPanel.add ( pb_PUBLISH ) ;
      mainPanel.add ( pb_PUBLISH_NOW ) ;
      templatePanel.add ( st_TEMPLATE_NAME ) ;
      templatePanel.add ( ef_TEMPLATE_NAME ) ;
      templatePanel.add ( pb_SAVE_TEMPLATE_NOW ) ;

      getContentPane().setLayout ( null ) ;
      getContentPane().add ( mainPanel ) ;
      getContentPane().add ( templatePanel ) ;
      getContentPane().add ( pb_CANCEL ) ;
      getContentPane().add ( st_IMAGE ) ;    

      /*--------------------------*/
      /*                          */
      /*  Adjust other properties */
      /*                          */
      /*--------------------------*/
      setResizable ( false ) ;
      templatePanel.setVisible ( false ) ;       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   REfresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mle_OLD_COMMENTS.setText ( document.getOriginalComments() ) ;
      mle_COMMENTS.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_DOCCOMMENT-document.getOriginalComments().length() ) ) ;
      mle_COMMENTS.setText ( comments ) ;
      ef_TEMPLATE_NAME.setText ( document.getTitle() ) ;

   }


   /**
     * This method will layout and position all of our controls
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = wizardImage.getImage().getWidth(this)+15 ;
      int       y         = 5 ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2), 
                           wizardImage.getImage().getWidth(this), 
                           wizardImage.getImage().getHeight(this)) ;

      mainPanel.setBounds ( x, 5, size.width-x-5, size.height-90 ) ;
      templatePanel.setBounds ( x, 5, size.width-x-5, size.height-90 ) ;
      if ( statusPanel != null )
         statusPanel.setBounds ( x, 5, size.width-x-5, size.height-90 ) ;
      pb_CANCEL.setBounds ( x, size.height-68, 80, 25 ) ;                          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Size the controls on the Main panel                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      x=0 ;
      y=0 ;
      size = mainPanel.getSize() ;
      st_COMMENTS.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      commentScroll.setBounds ( x, y, size.width-15, rowHeight*5 ) ;
      y+=(rowHeight*5) ;

      ck_TRANSLATION.setBounds ( x, y, size.width-15, rowHeight ) ;
      y+=rowHeight ;

      pb_SAVE.setBounds ( x, y, 168, 25 ) ;
      pb_SAVE_AS_TEMPLATE.setBounds ( x+173, y, 168, 25 ) ;
      y+=28 ;

      pb_SAVE_AS_NEW.setBounds ( x, y, 168, 25 ) ;
      pb_CANCEL_DRAFT.setBounds ( x+173, y, 168, 25 ) ;
      y+=28 ;

      pb_PUBLISH.setBounds ( x, y, 168, 25 ) ;
      pb_PUBLISH_NOW.setBounds ( x+173, y, 168, 25 ) ;
      y+=28 ;                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, size the controls on the template panel            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      x=5;
      y=5;
      st_TEMPLATE_NAME.setBounds ( x, y, size.width-15, rowHeight ) ;
      y+=rowHeight ;
      ef_TEMPLATE_NAME.setBounds ( x, y, size.width-15, rowHeight ) ;
      y+=rowHeight+5 ;
      pb_SAVE_TEMPLATE_NOW.setBounds ( x, y, 80, 25 ) ;

   }

   /**
     *  This method will return the return code
     **/
   public int getReturnCode () {
      return( returnCode ) ;
   }


   /**
     * This method will be called when a button is hit
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Cancel                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         if ( mainPanel.isVisible() )
            dispose() ;
         else {
            mainPanel.setVisible ( true ) ;
            templatePanel.setVisible ( false ) ;
            validate() ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Cancel Draft                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CANCEL_DRAFT ) {
         document.updateRecStatus ( Document.REC_DELETE ) ;
         createSavePanel() ;
         if ( DocumentSaver.saveDraft ( document, statusPanel ) ) {
            returnCode = SAVE_CANCEL_DRAFT ;
            dispose() ;
         } else {
            destroySavePanel() ;
            document.updateRecStatus ( Document.REC_NOCHANGE ) ;
            document.updateRecStatus ( Document.REC_UPDATE ) ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save As Draft                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE ) {
         if ( DocSaveCheck.checkDraft ( document, this ) ) {
            saveComment() ;
            createSavePanel () ;
            if ( DocumentSaver.saveDraft ( document, statusPanel ) ) {
               returnCode = SAVE_SAVED ;
               dispose() ;
            } else
               destroySavePanel() ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Publish                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_PUBLISH ) {
         if ( DocSaveCheck.checkPublish ( document, this ) ) {
            saveComment() ;
            createSavePanel() ;
            if ( DocumentSaver.publishDraft ( document, statusPanel ) ) {
               returnCode = SAVE_PUBLISHED ;
               dispose() ;
            } else
               destroySavePanel () ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Publish Now                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_PUBLISH_NOW ) {
         System.out.println ( "Publish now!" ) ;
         if ( DocSaveCheck.checkPublish ( document, this ) ) {
            saveComment() ;
            createSavePanel() ;
            if ( DocumentSaver.publishDraftNow ( document, statusPanel ) ) {
               returnCode = SAVE_PUBLISHED ;
               dispose() ;
            } else
               destroySavePanel () ;
         }
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save as Template                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE_AS_TEMPLATE ) {
         if ( canSaveAsTemplate() ) {
            mainPanel.setVisible ( false ) ;
            templatePanel.setVisible ( true ) ;
         } else
            GUISystem.printBox ( this, "Info", "Templates cannot have graphics or file attachments. Please remove any graphic and file attachment and try again" ) ;
         validate() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save Template Now                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE_TEMPLATE_NOW ) {
         String newTitle = ef_TEMPLATE_NAME.getText() ;

         if ( ( newTitle != null ) && ( newTitle.length() > 0 ) ) {
            boolean cont = true ;

            if ( cont ) {
               document.setTitle ( newTitle ) ;
               document.setTemplate ( true ) ;
               saveComment() ;
               createSavePanel() ;
               if ( DocumentSaver.saveDraft ( document, statusPanel ) ) {
                  returnCode = SAVE_TEMPLATE ;
                  DocTemplateSystem.addTemplate ( document ) ;
                  GUISystem.printBox ( this, "Info", "A new Document Template is now available" ) ;
                  dispose() ;
               } else
                  destroySavePanel () ;
            }

         } else
            GUISystem.printBox ( this, "Info", "Please supply the name for this template" ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save as new document                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE_AS_NEW ) {
         document.createNew() ;
         if ( DocSaveCheck.checkDraft ( document, this ) ) {
            saveComment() ;
            createSavePanel () ;
            if ( DocumentSaver.saveDraft ( document, statusPanel ) ) {
               returnCode = SAVE_SAVED ;
               //GUISystem.printBox ( this, "Info", "A new Document Draft has been created" ) ;
               dispose() ;
            } else
               destroySavePanel() ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Translation Check Box                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ck_TRANSLATION ) {
         if ( ck_TRANSLATION.isSelected () )
            mle_COMMENTS.setText ( mle_COMMENTS.getText()+"(BODY HAS BEEN CHANGED)" ) ;
         else {
            String newComments = Text.replaceAllStrInStr ( mle_COMMENTS.getText(), "(BODY HAS BEEN CHANGED)", "" ) ;

            mle_COMMENTS.setText ( newComments ) ;
         }
      }


   }



   public void mouseClicked  ( MouseEvent e ) {
   }
   public void mousePressed  ( MouseEvent e ) {
   }
   public void mouseReleased ( MouseEvent e ) {
   }
   public void mouseEntered ( MouseEvent e ) {
      ((DButton)e.getSource()).setBackground ( buttonHighlightBackground ) ;
   }

   public void mouseExited ( MouseEvent e ) {
      ((DButton)e.getSource()).setBackground ( buttonDefaultBackground ) ;
   }


   /**
     *  This method will create our status save panel and show it
     **/
   private void createSavePanel () {
      statusPanel = new DocSaveStatusPanel ( document ) ;
      getContentPane().add ( statusPanel ) ;
      mainPanel.setVisible ( false ) ;
      templatePanel.setVisible ( false ) ;
      pb_CANCEL.setEnabled ( false ) ;
      validate() ;
   }


   /**
     *  This method will destroy the save panel and put the main panel back up
     **/
   private void destroySavePanel () {
      getContentPane().remove ( statusPanel ) ;
      statusPanel = null ;
      mainPanel.setVisible ( true ) ;
      pb_CANCEL.setEnabled ( true ) ;
      validate() ;
   }

   /**
     *  This method will save the comment that the user has entered, if any
     **/
   private void saveComment () {
      String comment = mle_COMMENTS.getText() ;

      if ( ( comment == null ) || ( comment.length() == 0 ) ) {
         String date = new CDate ( CDate.DATE_WW_DATETIME ).today() ;

         comment = "On "+date+" "+UserSystem.getUserName ( )+" "+
                   "saved this document but made no comment\n" ;
      }

      document.setComments ( comment ) ;
   }


   /**
     *  This method will return TRUE if the document can be saved as a template. Any document
     * with a graphic or file attachment cannot be saved as a template
     **/
   private boolean canSaveAsTemplate () {
      if ( ( document.getGraphics() == null ) && ( document.getFileAttachments() == null ) )
         return( true ) ;
      else {
         boolean can = true ;

         if ( ( document.getGraphics() != null ) && 
              ( QuestUtil.getValidData ( document.getGraphics() ).size() > 0 ) )
            can = false ;
         else if ( ( document.getFileAttachments() != null ) && 
                   ( QuestUtil.getValidData ( document.getFileAttachments() ).size() > 0 ) )
            can = false ;

         return( can ) ;
      }
   }
} /* SaveDocDlg */
