package com.ibm.nzna.projects.qit.doc.docmaint ;
/*-----------------------------------------------------------*/
/*   DocListPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocListPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.qit.doc.model.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/**
 * This panel will hold the Document JTable, Title, and stats on the doc list. 
 *
 *
 * @version 1.0, 9/28/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocListPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocListPanel extends JPanel implements ActionListener, DocConst, AppConst, Authority {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle        st_CNRSTATS       = null ;
   private JTitle        st_TITLE          = null ;
   private JPanel        pnl_TEXT          = null ;
   private PrevNextPanel prevNextPanel     = null ;
   private DocMaintPanel docMaintPanel     = null ;
   private DocumentTable documents         = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create and attach all of our controls
    *  to the panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocListPanel ( DocMaintPanel docMaintPanel ) {
      JScrollPane tableScroll = null ;
      ButtonPanel buttonPanel = new ButtonPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our variables                                    */           
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.docMaintPanel = docMaintPanel ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Static Text                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_CNRSTATS   = new JTitle ( Str.getStr ( STR_VIEWING )+" 0" ) ;
      st_TITLE      = new JTitle ( Str.getStr ( STR_DOCUMENTS ) ) ;                                        
      documents     = new DocumentTable() ;
      tableScroll   = new JScrollPane  ( documents ) ;
      prevNextPanel = new PrevNextPanel ( this ) ;                                       
      pnl_TEXT      = new JPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust some basic properties                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_CNRSTATS.setForeground    ( Color.black ) ;
      st_TITLE.setForeground       ( Color.black ) ;
      st_TITLE.setFont             ( FontSystem.largeTitleFont ) ;
      st_CNRSTATS.setFont          ( FontSystem.defaultFont ) ;
      tableScroll.setBackground    ( Color.white ) ;
      setBackground ( Color.white ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to the Text panel and ourselves        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.setBackground ( Color.white ) ;
      buttonPanel.add ( new JLabel ( "Unclassified", (Icon)ImageSystem.getImageIcon ( this, ImageSystem.IBM_UNCLASSIFIED ), JLabel.LEFT ) ) ;
      buttonPanel.add ( new JLabel ( "Dealer Confidential", (Icon)ImageSystem.getImageIcon ( this, ImageSystem.IBM_DEALER_CONFIDENTIAL ), JLabel.LEFT ) ) ;
      buttonPanel.add ( new JLabel ( "IBM Confidential", (Icon)ImageSystem.getImageIcon ( this, ImageSystem.IBM_CONFIDENTIAL ),  JLabel.LEFT ) ) ;

      setLayout ( new BorderLayout() ) ;           
      pnl_TEXT.setLayout ( new BorderLayout () ) ;                   
      pnl_TEXT.add ( st_TITLE, BorderLayout.NORTH ) ;
      pnl_TEXT.add ( st_CNRSTATS, BorderLayout.CENTER ) ;
      add ( tableScroll, BorderLayout.CENTER ) ;
      add ( pnl_TEXT, BorderLayout.NORTH ) ;
      add ( buttonPanel, BorderLayout.SOUTH ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set some texture specific properties                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_TEXT.setBackground ( Color.white ) ;
      setBorder ( GUISystem.loweredBorder ) ;
      tableScroll.setBorder ( GUISystem.emptyBorder ) ;

   } /* DocListPanel */


   /*------------------------------------------------------------*/
   /* getSelection
    */
   /**
    *    This method will return the currently
    *  selected documents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of docs
    *//*---------------------------------------------------------*/
   public Vector getSelection() {
      return ( documents.getSelection() ) ;

   }




   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will remove everything from
    *  the current list and add the new Vector
    *  
    *
    *  @param addVec The vector to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void add ( Vector addVec ) {
      documents.setDocuments ( addVec ) ;
   }





   /*------------------------------------------------------------*/
   /* remove
    */
   /**
    *    THis method will remove an item from the
    *  document list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void remove ( DocumentRow docRow ) {
      documents.remove ( (MultiListRow) docRow ) ;
   }

   /*------------------------------------------------------------*/
   /* getSelectedDocument
    */
   /**
    *    This method will return the currently
    *  selected element in the container
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public DocumentRow getSelectedDocument () {
      return ( (DocumentRow)documents.getSelectedItem() ) ;
   }


   /*------------------------------------------------------------*/
   /* setStats
    */
   /**
    *    This method will set the current statistics
    *  for the container
    *  
    *
    *  @param stats The string representation of the statistics
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStats ( String stats ) {
      st_CNRSTATS.setText ( stats ) ;
   }





   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set our title
    *  
    *  
    *
    *  @param title The String representation of our title
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitle ( String title ) {
      st_TITLE.setText ( title ) ;
   }




   /*------------------------------------------------------------*/
   /* showNext
    */
   /**
    *    This method will either show or hide the
    *  next button on the container
    *  
    *
    *  @param show TRUE  = show, FALSE = do not show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showNext ( boolean show ) {
      prevNextPanel.showNext ( show ) ;
   } /* showNext */


   /*------------------------------------------------------------*/
   /* showPrev
    */
   /**
    *    This method will either show or hide the
    *  prev button on the container
    *  
    *
    *  @param show TRUE  = show, FALSE = do not show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showPrev ( boolean show ) {
      prevNextPanel.showPrev ( show ) ;
   } /* showPrev */





   /*------------------------------------------------------------*/
   /* nextShowing
    */
   /**
    *    This method will return TRUE if the next
    *  button is showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean nextShowing () {
      return( prevNextPanel.nextShowing() ) ;
   }



   /*------------------------------------------------------------*/
   /* setSelected
    */
   /**
    *    This method will set which record is 
    *  currently selected in the document list
    *  
    *
    *  @param docRec The DocRec to select
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelected ( Object docRec ) {
      documents.setSelectedItem ( docRec, true ) ;
   } /* setSelection */




   /*-----------------------------------------------------------*/
   /* selectAll
    */
   /**
    *    this method will select all of the items in
    *  the list
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void selectAll () {
      if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) )
         documents.selectAll(true) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when a button
    *  on the PrevNextPanel is pressed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( Qit.getEnabled() ) {

         if ( e.getSource() == prevNextPanel.getNextButton() )
            docMaintPanel.showNextDocs() ;
         else if ( e.getSource() == prevNextPanel.getPrevButton() )
            docMaintPanel.showPrevDocs() ;
      }
   }  


   /*------------------------------------------------------------*/
   /* removeAll
    */
   /**
    *    This method will remove all of the documents
    *  from the list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeAll () {

   }





   /*------------------------------------------------------------*/
   /* dispose
    */
   /**
    *    This method will clean up all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void dispose() {
      PropertySystem.putDetailList ( PROPERTY_DOCUMENT_COLUMN_WIDTHS, documents ) ;
      removeAll() ;
   }


   public void addListSelectionListener ( ListSelectionListener listener ) {
      documents.addListSelectionListener ( listener ) ;
   }

   public void addActionListener ( ActionListener listener ) {
      documents.addActionListener ( listener ) ;
   }

   /**
     *  This method will return the instance of DocumentTable we are using to display
     * documents with
     *
     * @return DocumentTable
     **/
   public DocumentTable getDocumentTable () {
      return ( documents );
   }
} /* DocListPanel */
