package com.ibm.nzna.projects.qit.doc.docmaint ;     
/*-----------------------------------------------------------*/
/*   DocMaintPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocMaintPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * The Document Maintenance panel allows the user 
 * to navigate through the documents in the Quest database
 *
 * @version 1.0, 1/9/2001 
 * @author  David Lentz
 */
import java.awt.* ;
import java.io.* ;
import java.awt.datatransfer.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.help.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.qit.doc.docedit.* ;
import com.ibm.nzna.projects.qit.doc.fieldchanger.* ;
import com.ibm.nzna.projects.qit.doc.history.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.doc.docmaint.property.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.pom.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   DocMaintPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocMaintPanel extends JPanel  implements HelpSource, PreviewPanel, PrintPanel, CustomViewDlgListener, SelectDocListener, Runnable, ListSelectionListener, ActionListener, DocViewListener, QuestPanel, AppConst, DocConst, Authority {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESHPREVIEW        = "refpre" ;
   private static final String THREAD_COMPLETEREQUESTS      = "RCOMREQ" ;
   private static final String THREAD_UNLOCK_DOCS           = "UNLOCK" ;




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private, GUI Members                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DocListPanel   docListPane        = null ;  // The panel holding the doc list and title
   private DocEditPanel   previewPanel       = null ;  // The panel holding the preview
   private JSplitPane     splitPane          = null ;  // The Horizontal Split located on the right side of pnl_MAINSPLIT
   private DocNavPanel    navPanel           = null ;
   private ToolBarButton  pb_UNLOCK          = null ;
   private ToolBarButton  pb_FIND            = null ;
   private ToolBarButton  pb_WIZARD          = null ;
   private ToolBarButton  pb_CUSTOM_QUERY    = null ;
   private ToolBarButton  pb_PREVIEW         = null ;
   private ToolBarButton  pb_HISTORY         = null ;
   private ToolBarButton  pb_PROPERTY        = null ;
   private JToolBar       toolBar            = null ;      
   private DButton        pb_EDIT            = null ;
   private DButton        pb_CLOSE           = null ;
   private DButton        pb_NEW             = null ;
   private DButton        pb_DELETE          = null ;
   private DButton        pb_COMPLETE        = null ;




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  The Parent AppDefaultWin of the Document Maint. Panel
     **/
   private AppDefaultWin parentWin        = null ;

   /**
     *  The current view that we are displaying
     **/
   private DocViewRec    currentView      = null ;

   /**
     *  The current list of document row objects that we are displaying
     **/
   private Vector        documents        = null ;       

   /**
     *  This is the flag telling us if we are showing drafts or not
     **/
   private boolean       showingDrafts    = false ;      


   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will be called before we are shown and
    *  will close our panel if we return FALSE. If we
    *  return TRUE everything was initialized ok
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize  () {
      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         navPanel           = new DocNavPanel ( this ) ;
         pb_EDIT            = new ActionButton ( Str.getStr ( STR_EDIT ),     ImageSystem.getImageIcon ( this, ImageSystem.EDIT ),    Str.getStr ( STR_EDIT ) ) ;
         pb_NEW             = new ActionButton ( Str.getStr ( STR_NEW ),      ImageSystem.getImageIcon ( this, ImageSystem.NEW ),     Str.getStr ( STR_NEW ) ) ;
         pb_DELETE          = new ActionButton ( Str.getStr ( STR_DELETE ),   ImageSystem.getImageIcon ( this, ImageSystem.DELETE ),  Str.getStr ( STR_DELETE ) ) ;
         pb_COMPLETE        = new ActionButton ( Str.getStr ( STR_COMPLETE ), ImageSystem.getImageIcon ( this, ImageSystem.ACTION ),  Str.getStr ( STR_COMPLETE_REQUEST ) ) ;                             
         pb_CLOSE           = new ActionButton ( Str.getStr ( STR_CLOSE ),    ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),   Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;                                               
         docListPane        = new DocListPanel ( this ) ;
         previewPanel       = new DocEditPanel () ;
         splitPane          = new JSplitPane ( JSplitPane.VERTICAL_SPLIT ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our tool bar buttons                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         toolBar = new JToolBar() ;
         pb_FIND         = new ToolBarButton ( Str.getStr ( STR_FIND ),       ImageSystem.getImageIcon ( this, ImageSystem.SEARCH ),                 Str.getStr ( STR_FIND_DOCMAINT ) ) ;
         pb_WIZARD       = new ToolBarButton ( Str.getStr ( STR_WIZARD ),     ImageSystem.getImageIcon ( this, ImageSystem.WIZARD ),                 Str.getStr ( STR_WIZARD_DOCMAINT ) ) ;
         pb_CUSTOM_QUERY = new ToolBarButton ( Str.getStr ( STR_CUSTOM ),     ImageSystem.getImageIcon ( this, ImageSystem.CUSTOM_QUERY ),           Str.getStr ( STR_CUSTOM_QUERY_DOC_TIP ) ) ;
         pb_PREVIEW      = new ToolBarButton ( Str.getStr ( STR_PREVIEW_ON ), ImageSystem.getImageIcon ( this, ImageSystem.PREVIEW ),                Str.getStr ( STR_TURN_PREVIEW_ON ) ) ;
         pb_HISTORY      = new ToolBarButton ( Str.getStr ( STR_HISTORY ),    ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_HISTORY_SMALL ), Str.getStr ( STR_HISTORY ) ) ;
         pb_PROPERTY     = new ToolBarButton ( Str.getStr ( STR_PROPERTY ),   ImageSystem.getImageIcon ( this, ImageSystem.PROPERTIES ),             Str.getStr ( STR_DOCUMENT_MAINT_PROPERTY ) ) ;
         toolBar.add ( new ToolBarSeparator () )  ;
         toolBar.add ( pb_FIND ) ;
         toolBar.add ( pb_HISTORY ) ;
         toolBar.add ( new ToolBarSeparator () )  ;
         toolBar.add ( pb_PREVIEW ) ;
         toolBar.add ( new ToolBarSeparator () )  ;
         toolBar.add ( pb_CUSTOM_QUERY ) ;
         toolBar.add ( pb_WIZARD ) ;                               
         toolBar.add ( new ToolBarSeparator () ) ;
         toolBar.add ( pb_PROPERTY ) ;               


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add the 'Unlock' Feature                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( UserSystem.hasAuthority ( AUTH_UNLOCK_DOCUMENTS ) ) {
            pb_UNLOCK = new ToolBarButton ( Str.getStr ( STR_UNLOCK ), ImageSystem.getImageIcon ( this, ImageSystem.LOCK_SMALL ), Str.getStr ( STR_UNLOCK )+" "+Str.getStr ( STR_DOCUMENTS ) ) ;
            pb_UNLOCK.addActionListener ( this ) ;
            toolBar.add ( new ToolBarSeparator() ) ;
            toolBar.add ( pb_UNLOCK ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add Listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_EDIT.addActionListener                          ( this ) ;
         pb_NEW.addActionListener                           ( this ) ;
         pb_DELETE.addActionListener                        ( this ) ;
         pb_COMPLETE.addActionListener                      ( this ) ;
         pb_CLOSE.addActionListener                         ( this ) ;                                        
         pb_FIND.addActionListener                          ( this ) ;
         pb_WIZARD.addActionListener                        ( this ) ;
         pb_CUSTOM_QUERY.addActionListener                  ( this ) ;
         pb_PREVIEW.addActionListener                       ( this ) ;
         pb_HISTORY.addActionListener                       ( this ) ;
         pb_PROPERTY.addActionListener                      ( this ) ;
         docListPane.addListSelectionListener               ( this ) ;
         docListPane.addActionListener                      ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         splitPane.setContinuousLayout ( false ) ;
         splitPane.setBorder ( GUISystem.emptyBorder ) ;           
         toolBar.setBorder ( GUISystem.emptyBorder ) ;
         toolBar.setOpaque ( false ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our controls                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setLayout ( new BorderLayout() ) ;

         if ( PropertySystem.getBool ( PROPERTY_DOCMAINT_PREVIEW ) ) {
            splitPane.setTopComponent ( docListPane ) ;
            splitPane.setBottomComponent ( previewPanel ) ;        
            add ( splitPane, BorderLayout.CENTER ) ;
         } else {
            add ( docListPane, BorderLayout.CENTER ) ;
            splitPane.setBottomComponent ( previewPanel ) ;        
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         return( false ) ;
      }


      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are initialized
    *  and we are showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      boolean showingPreview = PropertySystem.getBool ( PROPERTY_DOCMAINT_PREVIEW ) ;

      parentWin = GUISystem.getParentDefWin ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_NEW ) ;
      parentWin.addActionComponent ( this, pb_EDIT ) ;
      parentWin.addActionComponent ( this, pb_DELETE ) ;
      parentWin.addActionComponent ( this, pb_COMPLETE ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our toolbar                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addToolBarButtons() ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, refresh our views                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navPanel.refresh ( true ) ;

      if ( showingPreview )
         splitPane.setDividerLocation ( (int) getSize().height-200 ) ;

      pb_PREVIEW.setText ( showingPreview ? Str.getStr ( STR_PREVIEW_OFF ) : Str.getStr ( STR_PREVIEW_ON ) ) ;
      pb_PREVIEW.setToolTipText ( showingPreview ? Str.getStr ( STR_TURN_PREVIEW_OFF ) : Str.getStr ( STR_TURN_PREVIEW_ON ) ) ;


   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the
    *  user has selected our panel from a
    *  tabbed pane. If we are in MDI mode, this method is never called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
      addToolBarButtons() ;
   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel onto another. This method
    *  will never be called in the MDI mode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
   }





   /*------------------------------------------------------------*/
   /* addToolBarButtons
    */
   /**
    *    This method will add all of our specific
    *  toolbar buttons
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addToolBarButtons () {
      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 2 ) ;
   }



   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon that represents
    *  this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_SMALL ) ) ;
   }                                               





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default navigation
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( navPanel ) ;
   }



   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_DOCUMENTS ) ) ;
   }






   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh our display with the currently
    * selected view
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      if ( currentView != null ) {
         currentView.actualCount = 0 ;
         currentView.refresh() ;
      }
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method returns FALSE since we don't
    *  support cutting and pasting
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }



   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method returns FALSE since we don't
    *  support cutting and pasting
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( pasteIntoDoc() ) ;
   }



   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method returns FALSE since we don't
    *  support cutting and pasting
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the user
    *  has shutdown the application. A previous call
    *  to stop was already made so we do nothing here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the Quest
    *  application asks us to close ourselves. we
    *  will return true
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      panelDeselected() ;
      return( true ) ;
   }


   /*------------------------------------------------------------*/
   /* docViewStarted
    */
   /**
    *    This method will be called once a view
    *  has started to refresh
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void docViewStarted ( DocViewRec viewRec ) {
      parentWin.setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
   }




   /*------------------------------------------------------------*/
   /* docViewStopped
    */
   /**
    *    This method will be called when a view
    *  has stopped refreshing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void docViewStopped ( DocViewRec viewRec ) {

      try {
         if ( viewRec.getResults() != null ) {
            int toInt           = 0 ;
            int maxRecordReturn = PropertySystem.getInt ( PROPERTY_MAX_DOCUMENT_RETURN ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Free our previous results                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            this.documents = viewRec.getResults() ;                 


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the Prev/Next buttons up                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( documents.size() < maxRecordReturn ) {
               if ( docListPane.nextShowing() ) {
                  docListPane.showNext(false) ;
                  docListPane.showPrev(true) ;
               } else {
                  docListPane.showNext(false) ;
                  docListPane.showPrev(false) ;
               }
            } else
               docListPane.showNext(true) ;                            

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Calculate the 'From' and 'To' positions                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            toInt=viewRec.fromCount+documents.size() ;
            if ( viewRec.actualCount > 0 )
               docListPane.setStats ( Str.getStr ( STR_VIEWING )+" "+viewRec.fromCount+" - "+toInt+" "+Str.getStr ( STR_OF )+" "+viewRec.actualCount ) ;
            else
               docListPane.setStats ( Str.getStr ( STR_VIEWING )+" "+viewRec.fromCount+" - "+toInt ) ;

            showingDrafts = refreshShowingDrafts ( documents ) ;
            docListPane.add ( documents ) ;                                   

            if ( viewRec.actualCount > 0 )
               parentWin.setStatus ( Str.getStr ( STR_FOUND )+" "+viewRec.actualCount, 3 ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Make sure our 'showingDraft' property is set            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            refreshShowingDrafts(documents) ;

         } /* if the documents is good! */
         else
            parentWin.setStatus  ( Str.getStr ( STR_FOUND ) +" 0", 3 ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our current view                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      currentView = viewRec ;

      parentWin.setStatus ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action event
    *  is fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( Qit.getEnabled() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_CLOSE = Frees our records and closes                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == pb_CLOSE )
            GUISystem.getParentDefWin(this).closePanel ( this, null ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_FIND = Find a document                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_FIND )
            new SelectDocDlg ( ).addSelectDocListener ( this ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CUSTOM                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_CUSTOM_QUERY )
            new CustomViewDlg ( GUISystem.getParentDefWin ( this ), this ).getResult() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_UNLOCK                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_UNLOCK )
            new Thread ( this, THREAD_UNLOCK_DOCS ).start() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_HISTORY                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_HISTORY ) {
            DocumentRow docRow = docListPane.getSelectedDocument() ;

            if ( docRow != null )
               WindowSystem.createPanel ( new DocHistPanel ( docRow.getExistingDocumentInd(), docRow.isDraft(), this ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_PROPERTY                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_PROPERTY )
            new PropertyDlg ( parentWin ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_NEW = Creates new document request                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_NEW )
            createNewDoc ( GUISystem.getParentDefWin ( this ), this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_EDIT = Edits last selected Document                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( e.getSource() == pb_EDIT ) || ( e.getSource() == docListPane.getDocumentTable() ) )
            editCurrentDoc() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_COMPLETE = Completes the selected requests           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_COMPLETE )
            completeDocs() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_DELETE = Deletes the selected documents              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_DELETE )
            deleteDocs() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_PREVIEW                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_PREVIEW )
            togglePreview() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_WIZARD                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_WIZARD )
            fieldChanger() ;

      }
   }




   /**
    *    This method will be called when the user selects
    *  an item in our document list. We only capture this
    *  because if we are showing the preview information we need
    *  to refresh it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {

      if ( !e.getValueIsAdjusting() ) {
         if ( PropertySystem.getBool ( PROPERTY_DOCMAINT_PREVIEW ) )
            new Thread ( this, THREAD_REFRESHPREVIEW ).start() ;
      }
   } /* valueChanged  */


   /*-----------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread does a couple of operations. The operation
    *  that is run depends on what the name of the
    *  thread is set to.
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void run () {
      String name = Thread.currentThread().getName() ;         

      Qit.setEnabled ( false ) ;

      try {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_COMPLETEREQUESTS                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( name.equals ( THREAD_COMPLETEREQUESTS ) )
            complete() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_UNLOCK_DOCS                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( name.equals ( THREAD_UNLOCK_DOCS ) ) {
            parentWin.setStatus ( Str.getStr ( STR_UNLOCKING ) ) ;
            unlockDocs() ;
            parentWin.setStatus ( null ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_REFRESHPREVIEW                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( name.equals ( THREAD_REFRESHPREVIEW ) ) {
            DocumentRow row = docListPane.getSelectedDocument() ;

            parentWin.setStatus ( STR_PLEASE_WAIT ) ;
            if ( row != null ) {
               int         rc =  row.readDocument ( false ) ;

               if ( rc == 0 ) {
                  previewPanel.setDocument ( row.getDocument() ) ;
                  previewPanel.setEnabled ( false ) ;
                  setPreviewDocVisible ( true ) ;
               } else
                  LogSystem.log ( 1, "Could not read the document for refresh Preview Panel. rc:"+rc ) ;
            }
            parentWin.setStatus ( null ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, "Exception occurred in Thread ( "+name+" )" ) ;
         LogSystem.log ( 1, e ) ;
      }

      Qit.setEnabled ( true ) ;

   } /* run */

   /*------------------------------------------------------------*/
   /* refreshShowingDrafts
    */
   /**
    *    This method will go through a vector of
    *  DocumentRows and tries to determine if any
    *  documents in the vector are requests. If 1 document is
    *  a request, then the showingDrafts property is updated to TRUE,
    *  otherwise it is updated to FALSE ;
    *
    *  @param vector The list of documents
    *  @param None
    *  @param None
    *
    *  @return TRUE = Showing Requets, FALSE = not showing requests
    *//*---------------------------------------------------------*/
   private boolean refreshShowingDrafts ( Vector documents ) {

      showingDrafts = false ;

      /* Ok, now if we have any data in the
         vector, check each record to see if
         any of them are requests. If any are,
         set showingDraft = true */
      if ( documents != null ) {
         int    i         = 0 ;
         int    length    = documents.size() ;

         length = documents.size() ;
         while ( ( i < length ) && ( !showingDrafts ) ) {
            showingDrafts = ((DocumentRow)documents.elementAt(i)).isDraft() ;
            i++ ;
         } /* while */

      } /* if */

      return( showingDrafts ) ;
   }

   /*------------------------------------------------------------*/
   /* editCurrentDoc
    */
   /**
    *    This method will edit the current document
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void editCurrentDoc () {
      try {
         DocumentRow docRow = docListPane.getSelectedDocument() ;                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only process if a doc is selected                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parentWin.setStatus ( "Reading "+docRow.getTitle() ) ;
         if ( docRow != null )
            EditDocument.editDocument ( docRow ) ;
         else
            GUISystem.printBox ( Str.getStr( STR_INFO ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      parentWin.setStatus ( null ) ;
   }


   /*------------------------------------------------------------*/
   /* togglePreview
    */
   /**
    *    This method will toggle the preview mode on
    *  or off
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = now showing preview, FALSE = not showing preview
    *//*---------------------------------------------------------*/
   public boolean togglePreview () {
      Dimension size           = getSize() ;
      boolean   showingPreview = PropertySystem.getBool ( PROPERTY_DOCMAINT_PREVIEW ) ;

      if ( !showingPreview ) {
         remove ( docListPane ) ;
         splitPane.setTopComponent ( docListPane ) ;
         add ( splitPane ) ;
         splitPane.setDividerLocation ( (int) size.height-200 ) ;

         /* Reset the preview information */
         PropertySystem.putBool ( PROPERTY_DOCMAINT_PREVIEW, true ) ;
         new Thread ( this, THREAD_REFRESHPREVIEW ).start() ;
      } else {
         remove ( splitPane ) ;
         splitPane.remove ( docListPane ) ;
         add ( docListPane ) ;
         splitPane.setDividerLocation ( (int) size.height ) ;
         PropertySystem.putBool ( PROPERTY_DOCMAINT_PREVIEW, false ) ;
      }

      showingPreview = PropertySystem.getBool ( PROPERTY_DOCMAINT_PREVIEW ) ;
      pb_PREVIEW.setText ( showingPreview ? Str.getStr ( STR_PREVIEW_OFF ) : Str.getStr ( STR_PREVIEW_ON ) ) ;
      pb_PREVIEW.setToolTipText ( showingPreview ? Str.getStr ( STR_TURN_PREVIEW_OFF ) : Str.getStr ( STR_TURN_PREVIEW_ON ) ) ;


      revalidate() ;
      return( showingPreview ) ;
   }





   /*------------------------------------------------------------*/
   /* getPreviewShowing
    */
   /**
    *    This method will return TRUE if we are
    *  in preview mode
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getPreviewShowing () {
      return( PropertySystem.getBool ( PROPERTY_DOCMAINT_PREVIEW ) ) ;
   }

   /*------------------------------------------------------------*/
   /* selectDocComplete
    */
   /**
    *    This method will be called when the user
    *  has made a selection in the select doc window
    *  
    *
    *  @param Vector The vector of doc recs
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectDocComplete ( Vector vec ) {
      processExternalVec ( vec ) ;
   }

   /*-----------------------------------------------------------*/
   /* processExternalVec
    */
   /**
    *    This method will process a list of documents
    *  returned from an external source. An external source is a 
    *  source outside of this panel's filters and custom queries.
    *  
    *             
    *  @param newVec The New Vector of documents returned from the External source
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void processExternalVec ( Vector newVec ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our showingDraft property                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      showingDrafts = refreshShowingDrafts ( newVec ) ;

      try {
         docListPane.add ( newVec ) ;
         setCnrTitle ( "" ) ;
         docListPane.setStats ( "" ) ;
         docListPane.showNext(false) ;
         docListPane.showPrev(false) ;
      } catch ( Exception e ) {
      }

      revalidate() ;
   }


   /*------------------------------------------------------------*/
   /* setCnrTitle
    */
   /**
    *    This method will set the title of the container. The
    *  title is the static text control with large bold letters
    *  appearing above the container.
    *
    *  @param title The new Title, null if default title
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setCnrTitle ( String title ) {
      if ( title == null )
         docListPane.setTitle ( Str.getStr ( STR_DOCUMENTS ) ) ;
      else
         docListPane.setTitle ( title ) ;
   } /* setCnrTitle */


   /*------------------------------------------------------------*/
   /* getNewCustomView
    */
   /**
    *    This method simply returns a blank OACustomView
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return CustomView
    *//*---------------------------------------------------------*/
   public CustomView getNewCustomView () {
      DocCustomView customView = new DocCustomView() ;

      customView.updateRecStatus ( PersistentRec.REC_INSERT ) ;

      return( customView ) ;
   }



   /*------------------------------------------------------------*/
   /* createNewDoc
    */
   /**
    *    This method will go through all operations
    *  needed to start a new document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void createNewDoc ( AppDefaultWin appDefaultWin, DocMaintPanel docMaintPanel ) {
      NewDocumentDlg newDlg  = new NewDocumentDlg ( appDefaultWin );
      DocumentDraft  draft   = null ;
      int            newType = newDlg.getResult() ;

      switch ( newType ) {
         case NewDocumentDlg.NEWDOC_TEMPLATE :
            draft = new DocumentDraft ( newDlg.getTemplate() ) ;
            draft.setWorkRequired ( TypeList.WORKREQ_CREATE ) ;
            break ;

         case NewDocumentDlg.NEWDOC_BLANK :
            draft = new DocumentDraft () ;
            draft.setDocumentType ( newDlg.getTypeDocRec().getInd() ) ;

            if ( ( newDlg.getTypeDocRec().getInd() == TypeList.DOCTYPE_SSD ) ||
                 ( newDlg.getTypeDocRec().getInd() == TypeList.DOCTYPE_SERVICE_PUB ) ) {
               draft.setReleaseDate ( DateSystem.getDate ( CDate.DATE_DATE ) ) ;
            }
            break ;

         case NewDocumentDlg.NEWDOC_MIGRATE :
            pasteIntoDoc() ;
            break ;
      }

      if ( draft != null )
         EditDocument.editDocument ( draft ) ;
   }





   /*------------------------------------------------------------*/
   /* delete
    */
   /**
    *    This method will try the delete operation
    *  on each selected document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void delete () {
      StatusWin statusWin = new StatusWin ( parentWin ) ;

      parentWin.setStatus ( Str.getStr ( STR_DELETING ) ) ;

      try {

         statusWin.setTitle ( Str.getStr ( STR_PLEASE_WAIT ) );
         statusWin.setVisible ( true ) ;

         /* Check to see if they selected anything */
         if ( haveSelection() ) {
            boolean      hasAuth        = UserSystem.hasAuthority ( AUTH_AUTHOR ) ;
            DocumentRow  documentRow    = null ;
            int          i              = 0 ;
            Vector       documents      = docListPane.getSelection() ;
            int          length         = documents.size() ;
            int          rc             = 0 ;

            statusWin.setMaxValue ( length ) ;                                              


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we are showing drafts, simply set the recycled flags */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( showingDrafts ) {
               StringBuffer sql = new StringBuffer ( "UPDATE QUEST.DOCUMENTS SET RECYCLED = 'Y', DBUSER = '"+UserSystem.getUserId()+"', CHANGEDTIME = CURRENT TIMESTAMP WHERE DOCIND IN (") ;

               if ( hasAuth ) {
                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Loop through and add each doc indicator                 */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  while ( i < length ) {
                     sql.append ( ((DocumentRow)documents.elementAt(i)).getDocumentInd() ) ;
                     if ( i < (length-1) )
                        sql.append ( "," ) ;
                     i++ ;
                  }
                  sql.append ( ")" ) ;  


                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Run the SQL                                             */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  rc = SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sql.toString() ) ;

                  if ( rc == 0 )
                     GUISystem.printBox ( STR_INFO, STR_COMPLETE ) ;
                  else {
                     LogSystem.log ( 1, "Could not update documents to recycled. rc="+rc ) ;
                     GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                  } 
               } else
                  GUISystem.printBox ( "Info", "You do not have the proper authority to perform this action" ) ;
            }



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we are showing Active Documents, then create a       */
            /* request for delete for them                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
               while ( i < length ) {
                  documentRow = (DocumentRow) documents.elementAt(i) ;
                  statusWin.setText ( Str.getStr ( STR_READING )+" "+documentRow.getTitle() ) ;
                  rc = documentRow.readDocument ( true ) ;
                  if ( rc == 0 ) {
                     DocumentDraft draft = EditDocument.createDraft ( documentRow.getDocument() ) ;

                     if ( draft != null ) {

                        EditDocument.setDefaultValues ( draft ) ;
                        draft.setWorkRequired ( TypeList.WORKREQ_DELETE ) ;

                        statusWin.setText ( Str.getStr ( STR_SAVING )+" "+documentRow.getTitle() ) ;
                        if ( !DocumentSaver.saveDraft ( draft, null ) ) {
                           GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                           i=length+1 ;
                        }

                     }
                  } else {
                     GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                     LogSystem.log ( 1, "Could not read document. rc:"+rc ) ;
                  }
                  i++ ;
               }
            } /* else, has documents  */
         } /* have Selection */
         else
            GUISystem.printBox ( STR_ERROR, STR_MUST_SELECT_ITEM ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

      parentWin.setStatus ( null ) ;
      statusWin.dispose() ;

      /* Refresh ourselves */
      docListPane.repaint() ;
      System.gc() ;


   }

   /*------------------------------------------------------------*/
   /* deleteDocs
    */
   /**
    *    This method will try to either create a request
    *  for deletion on all selected existing documents,
    *  or will delete the selected documen requests
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void deleteDocs() {
      if ( haveSelection() ) {
         boolean rc = false ;

         /* Check the user's authority. If they don't have author,
            they cannot delete a request, or create a request for delete */
         if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) {
            if ( showingDrafts )
               rc = GUISystem.printBox ( Str.getStr ( STR_ARE_YOU_SURE_DELETE ), Str.getStr ( STR_ARE_YOU_SURE_DELETE_DOC_REQ ) ) ;
            else
               rc = GUISystem.printBox ( "Are You Sure?", "Are you sure you wish to create drafts for the deletetion of the selected documents?" ) ;
         } else {

            if ( !showingDrafts )
               rc = GUISystem.printBox ( "Are You Sure?", "Are you sure you wish to create drafts for the deletetion of the selected documents?" ) ;
            else
               GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_NO_AUTH_DELETE_DOC ) ) ;

         }

         /* Ok, if the user answered yes to our questions, delete! */
         if ( rc )
            delete() ;

      } /* if documents are selected */
   } /* deleteDocs */


   /*-----------------------------------------------------------*/
   /* haveSelection
    */
   /**
    *    This method will refresh the selectedDocVec and return TRUE if we
    *  currently have a valid selection
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return TRUE = have selection, FALSE = do not have a selection
    */
   public boolean haveSelection() {
      boolean rc             = false ;
      Vector  selectedDocVec = docListPane.getSelection() ;

      rc = ( ( selectedDocVec != null ) && ( selectedDocVec.size() > 0 ) ) ;

      return( rc ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteIntoDoc
    */
   /**
    *    This method will create a new document by
    *  pasting the clipboard into the docs comments
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private static boolean pasteIntoDoc () {
      Clipboard cp = Toolkit.getDefaultToolkit().getSystemClipboard() ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   STEP 1 - Check the Data on the clipboard                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( cp != null ) {
         try {
            Transferable cpData = cp.getContents ( (Object)MainWindow.getInstance() ) ;

            if ( cpData != null ) {
               DocumentDraft draft = null ;
               String        data  = (String) (cpData.getTransferData ( DataFlavor.stringFlavor ) ) ;                                        

               /*------------------------------------------------------------*/
               /*                                                            */
               /*   STEP 2 - If the data is good, create a new document      */
               /*            request for it and assign it the work required  */
               /*            of TypeList.WORKREQ_CREATE, and get the type of */
               /*            document the user wants.                        */
               /*                                                            */
               /*------------------------------------------------------------*/
               if ( data != null ) {
                  DocEditPanel editPanel = null ;

                  draft = new DocumentDraft() ;

                  if ( draft.getDocumentType() == TypeList.DOCTYPE_SSD )
                     draft.setAdditionalInfo ( data ) ;
                  else
                     draft.setDocumentBody ( new DocumentBody ( data ) ) ;

                  EditDocument.editDocument ( draft ) ;
               }
            } /* if */


         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* unlockDocs
    */
   /**
    *    This method will unlock all selected documents
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void unlockDocs () {
      if ( haveSelection () ) {
         String     qualifier = showingDrafts ? "QUEST" : "TIGRIS" ;
         Vector     docs      = docListPane.getSelection() ;
         StatusWin  statusWin = null ;
         int        docInd    = 0 ;
         String     sqlString = "UPDATE "+qualifier+".DOCUMENTS SET MAINTLOCK='N' WHERE DOCIND IN (" ;

         for ( int i = 0; i < docs.size(); i++ ) {
            docInd = showingDrafts ? ((DocumentRow)docs.elementAt(i)).getDocumentInd() :
                     ((DocumentRow)docs.elementAt(i)).getExistingDocumentInd() ;
            sqlString+=+docInd ;
            if ( i < (docs.size()-1) )
               sqlString+="," ;
         }

         sqlString+=")"+SqlRunner.END_DELIM ;                                    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now make sure we take out the Checked Out property  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sqlString+="DELETE FROM "+qualifier+".DOCPROPERTIES WHERE PROPERTYIND = "+DocumentProperty.CHECKED_OUT_BY+" AND DOCIND IN (" ;
         for ( int i = 0; i < docs.size(); i++ ) {
            docInd = showingDrafts ? ((DocumentRow)docs.elementAt(i)).getDocumentInd() :
                     ((DocumentRow)docs.elementAt(i)).getExistingDocumentInd() ;
            sqlString+=+docInd ;
            if ( i < (docs.size()-1) )
               sqlString+="," ;
         }
         sqlString+=")"+SqlRunner.END_DELIM ;  



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, run it                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( SqlRunner.sqlRunnerCode ( DatabaseSystem.QUEST, sqlString ) == 0 )
            GUISystem.printBox ( Str.getStr ( STR_INFO ), "Documents Unlocked" ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;

      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;
   }

   /*------------------------------------------------------------*/
   /* completeDocs
    */
   /**
    *    This method will complete all selected
    *  requests
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void completeDocs () {
      if ( haveSelection() ) {

         /* Only complete the requests */
         if ( showingDrafts ) {

            new Thread ( this, THREAD_COMPLETEREQUESTS ).start() ;

         } else
            GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_CAN_ONLY_COMPLETE_REQUEST ) ) ;

      } /* if */
   } /* completeDocs */



   /*------------------------------------------------------------*/
   /* showPrevDocs
    */
   /**
    *    This method will show the previous x docs
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showPrevDocs () {
      int maxRecordReturn = PropertySystem.getInt ( PROPERTY_MAX_DOCUMENT_RETURN ) ;

      currentView.directionInd = currentView.PB_PREV ;
      if ( currentView.fromCount < maxRecordReturn )
         currentView.fromCount=0 ;
      else
         currentView.fromCount-=maxRecordReturn ;

      if ( currentView.fromCount < maxRecordReturn )
         docListPane.showPrev(false) ;

      if ( currentView.actualCount > maxRecordReturn )
         docListPane.showNext(true) ;

      refresh() ;
   }




   /*------------------------------------------------------------*/
   /* showNextDocs
    */
   /**
    *    This method will show the next 50 documents
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showNextDocs () {
      int maxRecordReturn = PropertySystem.getInt ( PROPERTY_MAX_DOCUMENT_RETURN ) ;

      if ( PropertySystem.getBool ( PROPERTY_DOCMAINT_PREVIEW ) )
         setPreviewDocVisible ( false ) ;

      currentView.directionInd = currentView.PB_NEXT ;
      currentView.fromCount+=maxRecordReturn ;
      refresh() ;
      docListPane.showPrev(true) ;
   }



   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    If we are passed a string, and the string
    *  is DOCMAINT, we are good to go and return TRUE
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if object is String and equals DOCMAINT
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {
      if ( object instanceof String ) {
         return( ((String)object).equals ( "DOCMAINT" ) ) ;
      }
      return( super.equals ( object ) ) ;
   }



   /*------------------------------------------------------------*/
   /* fieldChanger
    */
   /**
    *    This method will present the user with a wizard
    *  to change all of the selected document requests
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if everything went ok
    *//*---------------------------------------------------------*/
   private boolean fieldChanger () {

      try {

         /* Check if the user has author authority */
         if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) {

            /* First, check to see that they have something selected */
            if ( haveSelection() ) {
               Vector selectedDocVec = docListPane.getSelection() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If we are showing Drafts, process the Drafts            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( showingDrafts )
                  new FieldChangerWizard ( GUISystem.getParentDefWin ( this ),
                                           docListPane.getSelection() ).getResult() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise, we must create a Draft for each document     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  Vector       docRows = new Vector ( selectedDocVec.size() ) ;
                  DocumentRow  docRow     = null ;
                  int          i          = 0 ;
                  int          length     = selectedDocVec.size() ;
                  boolean      cont       = true ;
                  StatusWin    statusWin  = new StatusWin ( parentWin ) ;
                  Vector       drafts     = new Vector ( 1 )  ;

                  GUISystem.getParentDefWin(this).setStatus ( STR_CREATING_DOC_REQUEST ) ;

                  /* Loop through and process each existing
                     document */
                  while ( ( cont ) && ( i < length ) ) {

                     docRow = (DocumentRow) selectedDocVec.elementAt(i) ;

                     if ( docRow != null ) {
                        DocumentDraft draft = null ;
                        int           sqlCode = 0 ;


                        statusWin.setText ( Str.getStr ( STR_READING )+" "+docRow.getTitle() ) ;
                        sqlCode = docRow.readDocument ( false ) ;                

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   If we read it in, save it                               */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        if ( sqlCode == 0 ) {

                           statusWin.setText ( "Creating Draft for "+docRow.getTitle() ) ;

                           draft = EditDocument.createDraft ( docRow.getDocument() ) ;
                           if ( draft != null ) {
                              EditDocument.setDefaultValues ( draft ) ;

                              statusWin.setText ( Str.getStr ( STR_SAVING )+" "+docRow.getTitle() ) ;
                              if ( draft.writeToDatabase() != 0 ) {
                                 GUISystem.printBox ( "Error", "Could not save draft - "+docRow.getFilename() ) ;
                                 cont = false ;
                              } else
                                 drafts.addElement ( docRow ) ;
                           }

                        } else {
                           GUISystem.printBox ( "Error", "Could not read document - "+docRow.getFilename() ) ;
                           cont  = false ;
                        }

                     }

                     i++ ;
                  }

                  statusWin.dispose() ;
                  GUISystem.getParentDefWin(this).setStatus ( null ) ;

                  /* if we ran into no problems, launch the wizard */
                  if ( cont ) {

                     /* Set the documents in the list */
                     processExternalVec ( drafts ) ;


                     /* Pop up the wizard */
                     new FieldChangerWizard ( GUISystem.getParentDefWin ( this ),
                                              docRows ).getResult() ;
                  }

               } /* else */
            } else
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_DOC_BEFORE_FIELD_CHANGER ) ;
         } else
            GUISystem.printBox ( STR_INFO, STR_NO_AUTH_FIELD_CHANGER ) ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }
      return( true ) ;

   }




   /*------------------------------------------------------------*/
   /* getHelpFilename
    */
   /**
    *    This method will return our help filename
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getHelpFilename () {
      System.out.println ( "Returing" ) ;
      return( "Documents/Document_views.html" ) ;
   }


   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      if ( !showingDrafts ) {
         Vector selection = docListPane.getSelection() ;

         if ( ( selection != null ) && ( selection.size() > 0 ) ) {
            int    i           = 0 ;
            int    size        = selection.size() ;
            Vector bookmarkVec = new Vector ( size ) ;

            while ( i < size ) {
               Bookmark bookmark = new Bookmark() ;
               bookmark.setDescript ( ((DocumentRow)selection.elementAt(i)).getTitle() ) ;
               bookmark.setObject ( new Integer ( ((DocumentRow)selection.elementAt(i)).getExistingDocumentInd() ),
                                    BOOKMARK_DOCUMENT ) ;
               bookmark.updateRecStatus ( PersistentRec.REC_INSERT ) ;
               bookmarkVec.addElement ( bookmark ) ;
               i++ ;
            }

            BookmarkSystem.guiAddBookmarks ( bookmarkVec ) ;

         } else
            GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                                 Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_MUST_SELECT_ITEM_TO_BOOKMARK ) ) ;

      } else
         GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_CANNOT_BOOKMARK_DRAFTS ) ) ;
      return( true ) ;
   }

/**
    *  This method will give the printing support to the panel
    **/

   public Object getPrintObject() {
      return( new Print ( docListPane.getDocumentTable(), parentWin, Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }

   /**
    *  This method will give the Preview support to the panel
    **/
    
    public Object getPreviewObject() {
      return( new Preview ( docListPane.getDocumentTable(), parentWin, Str.getStr ( STR_DOCUMENTS ) ) ) ;
   }

   /**
     *  This method will either show or hide the document being displayed in the previewpanel
     **/
   private void setPreviewDocVisible ( boolean visible ) {
      previewPanel.setVisible ( visible ) ;
      revalidate() ;
   }


   public void addListSelectionListener ( ListSelectionListener listener ) {
      docListPane.addListSelectionListener ( listener ) ;
   }


   /**
     * This method will complete all selected drafts
     **/
   private void complete () {
      StatusWin statusWin = new StatusWin ( parentWin ) ;        

      try {
         int          i              = 0 ;
         Vector       selectedDocVec = docListPane.getSelection() ;
         int          length         = selectedDocVec.size() ;
         boolean      cont           = true ;
         DocumentRow  docRow         = null ;
         Vector       badDocs        = new Vector ( 1 ) ;

         String       comments           = null ;


         /*---------------------------------*/
         /*                                 */
         /*  Get the comments from the user */
         /*                                 */
         /*---------------------------------*/
         comments = new SingleMLEDlg ( parentWin, 
                                       "Please Enter Comments",
                                       (length > 1 ) ? "Please Enter Comments for publishing these drafts" : "Please Enter Comments for publishing this draft",
                                       "",
                                       "" ).getResult() ;

         if ( ( comments != null ) && ( comments.length() > 0 ) ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up our status                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            statusWin.setTitle ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
            statusWin.setMaxValue ( length ) ;                             

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through and publish each document                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( ( i < length ) && ( cont ) ) {

               docRow = (DocumentRow) selectedDocVec.elementAt(i) ;

               if ( docRow != null ) {

                  if ( docRow.isDraft() ) {
                     int rc = -1 ;


                     statusWin.setText ( Str.getStr ( STR_READING )+" "+docRow.getTitle() ) ;
                     rc = docRow.readDocument ( false ) ;

                     if ( rc == 0 ) {
                        DocumentDraft draft = (DocumentDraft)docRow.getDocument() ;


                        statusWin.setText ( Str.getStr ( STR_PUBLISHING )+" "+docRow.getTitle() ) ;
                        draft.setComments ( comments ) ;
                        draft.updateRecStatus ( DocumentDraft.REC_UPDATE ) ;
                        if ( DocSaveCheck.checkPublish ( draft, statusWin, false ) ) {
                           if ( !DocumentSaver.publishDraft ( draft, null ) ) {
                              GUISystem.printBox ( "Error", "Could not republish "+docRow.getFilename() ) ;
                              cont = false ;
                           }
                        } else
                           badDocs.addElement ( docRow ) ;

                     }
                  }

               }

               i++ ;
               statusWin.setValue ( i ) ;

               /* Run the Garbage Collector */
               System.gc() ;
            }

            refresh() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   IF we have bad documents that did not publish, show them*/
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( badDocs.size() > 0 ) {
               if ( badDocs.size() == 1 ) {
                  if ( GUISystem.printBox ( "Question", 
                                            "The document did not publish because "+
                                            "it is missing data. Would you like "+
                                            "to edit that document?" ) ) {
                     docRow = (DocumentRow)badDocs.elementAt ( 0 ) ;

                     parentWin.setStatus ( "Reading "+docRow.getTitle() ) ;
                     EditDocument.editDocument ( docRow ) ;
                  }
               } else {
                  GUISystem.printBox ( "Info", "There were "+badDocs.size()+" documents which did not publish because they are missing data. Press 'Ok' to see the list of documents" ) ;
                  new DocumentDlg ( GUISystem.getParentDefWin ( this ), badDocs ) ;
               }
            }
         } else
            GUISystem.printBox ( "Info", "You must enter a comment when completing these drafts" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1 ,e ,false ) ;
      }

      statusWin.dispose() ;
   }
} /* DocMaintPanel */
