package com.ibm.nzna.projects.qit.doc.docmaint ;              
/*-----------------------------------------------------------*/
/*   DocMaintWhatsNew                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocMaintWhatsNew                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
/**
 * This object manages all 'What's New' pop ups for the DocMaint 
 *
 *
 * @version 1.0, 12/4/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocMaintWhatsNew                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocMaintWhatsNew extends Object implements WhatsNewPopUpListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Pop Up Indexes                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int WHATSNEW_START        = 0 ;
   private static final int WHATSNEW_FIELDCHANGER = 1 ;
   private static final int WHATSNEW_CUSTOMVIEW   = 2 ;
   private static final int WHATSNEW_HISTORY      = 3 ;
   private static final int WHATSNEW_PROPERTY     = 4 ;
   private static final int WHATSNEW_STOP         = 5 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Locations -  These are the locations for each popup     */
   /* each Rectangle represents the corresponding Pop Up Index  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Point[] locations = {
      new Point ( 170, 100),        // WHATSNEW_START
      new Point ( 170, 100),        // WHATSNEW_FIELDCHANGER
      new Point ( 170, 100),        // WHATSNEW_CUSTOMVIEW
      new Point ( 170, 100),        // WHATSNEW_HISTORY
      new Point ( 170, 100),        // WHATSNEW_PROPERTY
      new Point ( 170, 100)         // WHATSNEW_STOP
   } ;                                   


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Titles - These are the titles of all Pop Ups            */
   /* each String represents the corresponding Pop Up Index     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static int[] titles = {
      STR_DOCUMENT_MAINT,                             // WHATSNEW_START
      STR_WIZARD,                                     // WHATSNEW_FIELDCHANGER
      STR_CUSTOM,                                     // WHATSNEW_CUSTOMVIEW
      STR_HISTORY,                                    // WHATSNEW_HISTORY
      STR_PROPERTIES,                                 // WHATSNEW_PROPERTY
      STR_BOOKMARK                                    // WHATSNEW_STOP
   }  ;                                   


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Content - This is the content of all Pop Ups            */
   /* each String represents the corresponding Pop Up Index     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static int[] content = {
      STR_WHATS_NEW_DOCMAINT_VIEWS,             // WHATSNEW_START
      STR_WHATS_NEW_DOCMAINT_FIELD_CHANGER,     // WHATSNEW_FIELDCHANGER
      STR_WHATS_NEW_DOCMAINT_CUSTOM_VIEW,       // WHATSNEW_CUSTOMVIEW
      STR_WHATS_NEW_DOCMAINT_HISTORY,           // WHATSNEW_HISTORY
      STR_WHATS_NEW_DOCMAINT_PROPERTY,          // WHATSNEW_PROPERTY
      STR_WHATS_NEW_DOCMAINT_BOOKMARK           // WHATSNEW_STOP
   }   ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DocMaintPanel panel         = null ;
   private int            whatsNewIndex = WHATSNEW_START ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This initial constructor starts the show
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocMaintWhatsNew ( DocMaintPanel panel ) {
      if ( PropertySystem.getBool ( PropertySystem.PROPERTY_SHOW_WHATS_NEW ) ) {
         this.panel = panel ;
         showWhatsNew ( WHATSNEW_START ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* showWhatsNew
    */
   /**
    *    This method will show the pop up with
    *  the specified index
    *
    *  @param index The WHATSNEW_XXX index to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showWhatsNew ( int index ) {
      WhatsNewPopUp popup = WhatsNewPopUp.getInstance() ;

      whatsNewIndex = index ;
      popup.removeWhatsNewPopUpListener ( this ) ;
      popup.addWhatsNewPopUpListener ( this ) ;

      popup.setRelativeParent ( GUISystem.getParentDefWin ( panel ) ) ;
      popup.setLocation  ( locations[whatsNewIndex] ) ;
      popup.setTitle   ( Str.getStr ( titles  [ whatsNewIndex ] ) ) ;
      popup.setContent ( Str.getStr ( content [ whatsNewIndex ] ) ) ;
      popup.setVisible ( true ) ;
      popup.toFront() ;
   }





   /*------------------------------------------------------------*/
   /* whatsNewPerformed
    */
   /**
    *    This method will be called when a 
    *  new whats new button is hit. We will process
    *  this by incrementing the wahtsNewIndex and showing it
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void whatsNewPerformed ( WhatsNewPopUp popup ) {
      whatsNewIndex++ ;

      if ( whatsNewIndex > WHATSNEW_STOP )
         whatsNewIndex = WHATSNEW_START ;

      showWhatsNew ( whatsNewIndex ) ;
   }
} /* DocMaintWhatsNew */
