package com.ibm.nzna.projects.qit.doc.docmaint ;
/*-----------------------------------------------------------*/
/*   DocNavPanel                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocNavPanel                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import javax.swing.tree.* ;
import javax.swing.event.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.border.BevelBorder ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.customview.* ;
/**
 * This panel will hold all of the different filters that may
 * be applied to the DocMaintPanel.
 *
 * @version 1.0, 2/18/1999 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocNavPanel                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocNavPanel extends JPanel implements ChangeListener, NavPanel, ActionListener, AppConst, DocConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_CREATE      = "Create" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTabbedPane         tabbedPane         = null ;
   private BookmarkHotLinkTree bookmarkTree       = null ;
   private DocViewPanel        viewPanel          = null ;
   private DocMaintPanel       docMaintPanel      = null ;
   private DocReviewPanel      reviewPanel        = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create an initially blank
    *  Filter Panel. A Thread will be created and
    *  run to populate the filter list
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocNavPanel ( DocMaintPanel docMaintPanel ) {
      this.docMaintPanel = docMaintPanel ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {    
      JScrollPane bookmarkScrollPane = null ;
      Color       tabColor           = ThemeSystem.getColor ( "NAVPANEL_BG_COLOR" ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Use a border layout                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;   


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      tabbedPane   = new JTabbedPane() ;
      bookmarkTree = new BookmarkHotLinkTree ( true ) ;
      viewPanel    = new DocViewPanel( );
      reviewPanel  = new DocReviewPanel ( docMaintPanel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      tabbedPane.setBorder ( GUISystem.emptyBorder ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      bookmarkTree.addActionListener ( this ) ;
      viewPanel.addDocViewListener ( docMaintPanel ) ;
      docMaintPanel.addListSelectionListener ( reviewPanel ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      tabbedPane.addTab ( Str.getStr ( STR_VIEWS ),     viewPanel ) ;
      tabbedPane.addTab ( Str.getStr ( STR_BOOKMARKS ), bookmarkScrollPane = new JScrollPane ( bookmarkTree ) ) ;
      tabbedPane.addTab ( Str.getStr ( STR_REVIEW ),    reviewPanel ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */        
      /*                                                           */
      /*-----------------------------------------------------------*/
      bookmarkScrollPane.setBorder ( GUISystem.emptyBorder ) ;
      add ( tabbedPane, BorderLayout.CENTER ) ;
      tabbedPane.addChangeListener ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will refresh our data in a new thread
    *  or the current thread
    *  
    *
    *  @param newThread TRUE if a new thread should be created, FALSE If not
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void refresh ( boolean newThread ) {
      viewPanel.refresh ( newThread ) ;
   }



   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are showing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
   }      


   /*------------------------------------------------------------*/
   /* close
    */
   /**
    *    This method will free all data we have
    *  been consuming
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void close () {
   }



   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    Returns what we prefer to be
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( new Dimension ( 153, 400 ) ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will free our resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      removeAll() ;
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title of our
    *  nav panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_SELECT_A_VIEW ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will simply return ourselves
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event that we are listening for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Bookmarks                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() instanceof Bookmark ) {
         Bookmark folder = (Bookmark)e.getSource() ;

         if ( folder.isFolder() ) {
            Vector bookmarks = BookmarkSystem.getBookmarks ( folder.getDescript() ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have bookmarks in this folder                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( bookmarks != null ) && ( bookmarks.size() > 0 ) ) {
               boolean containsDraft  = false ;
               boolean containsActive = false ;
               int     i              = 0 ;
               int     size           = bookmarks.size() ;                                       
               Vector  listeners      = new Vector ( 1 ) ;                                        

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Make sure the docmaint panel is notified of views       */
               /*                                                           */
               /*-----------------------------------------------------------*/
               listeners.addElement ( docMaintPanel ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   First, find out if we have Active, Draft or both        */
               /* Documents in this folder                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {
                  if ( ((Bookmark)bookmarks.elementAt(i)).getBookmarkType() == BOOKMARK_DOCUMENT )
                     containsActive = true ;
                  else if ( ((Bookmark)bookmarks.elementAt(i)).getBookmarkType() == BOOKMARK_DOCUMENT_DRAFT )
                     containsDraft  = true ;
                  i++ ;
               }  

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If we contain both, ask the user which one to show      */                                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( containsDraft ) && ( containsActive ) ) {
                  if ( GUISystem.printBox ( STR_QUESTION, STR_BOOKMARK_FOLDER_CONTAINS_DRAFT_ACTIVE ) )
                     containsDraft = false ;
                  else
                     containsActive = false ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If we contain none, tell the user                       */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( ( !containsDraft ) && ( !containsActive ) )
                  GUISystem.printBox ( STR_INFO, STR_BOOKMARK_FOLDER_CONTAINS_NO_DOCUMENTS ) ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, if we have active documents, proceed               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( containsActive ) {
                  Vector         activeVec = new Vector ( 10 ) ;
                  DocViewRec     viewRec   = new DocViewRec ( DocViewRec.VIEW_ACTIVE_BOOKMARKS ) ;

                  i=0 ;
                  while ( i < size ) {
                     if ( ((Bookmark)bookmarks.elementAt(i)).getBookmarkType() == BOOKMARK_DOCUMENT )
                        activeVec.addElement ( bookmarks.elementAt(i) ) ;
                     i++ ;
                  }

                  viewRec.setBookmarkVec ( activeVec ) ;
                  viewRec.setListeners ( listeners ) ;
                  viewRec.setCategoryInd ( -1 ) ;
                  viewRec.refresh() ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, if we have Draft documents, proceed                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( containsDraft ) {
                  Vector         draftVec = new Vector ( 10 ) ;
                  DocViewRec     viewRec   = new DocViewRec ( DocViewRec.VIEW_DRAFT_BOOKMARKS ) ;

                  i=0 ;
                  while ( i < size ) {
                     if ( ((Bookmark)bookmarks.elementAt(i)).getBookmarkType() == BOOKMARK_DOCUMENT_DRAFT )
                        draftVec.addElement ( bookmarks.elementAt(i) ) ;
                     i++ ;
                  }

                  viewRec.setBookmarkVec ( draftVec ) ;
                  viewRec.setListeners ( listeners ) ;
                  viewRec.setCategoryInd ( -1 ) ;
                  viewRec.refresh() ;
               }



            } else
               GUISystem.printBox ( STR_INFO, STR_BOOKMARK_FOLDER_CONTAINS_NO_DOCUMENTS ) ;

         }
      }

   }


   /**
     * This method will be called when the user selects a tab in our tabbed notebook. If
     * the page showing is our review page, we must tell the review page it is showing,
     * otherwise we need to tell the review page it is not showing
     *
     * @param e The Event
     */
   public void stateChanged ( ChangeEvent e ) {
      if ( tabbedPane.getComponent ( tabbedPane.getSelectedIndex() ) == reviewPanel ) {
          reviewPanel.setRefresh ( true ) ;
      } else
          reviewPanel.setRefresh ( false ) ;
   }

} /* DocNavPanel */
