package com.ibm.nzna.projects.qit.doc.docmaint ;
/*-----------------------------------------------------------*/
/*   DocPrevPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocPrevPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * DO NOT CONFUSE THIS PANEL! This is not the panel that draws onto the Preview Panel. 
 * This Panel simply extends the Browser and handles the setting of the current
 * document that is being viewed.
 *
 *
 * @version 1.0, 9/28/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocPrevPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocPrevPanel extends JPanel implements AppConst, DocConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESHPREVIEWINFO = "RefreshPreview" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DocumentRow   docRow        = null ;
   private DocMaintPanel docMaintPanel = null ;
   private boolean       inRefresh     = false ;
   private Browser       browser       = GUISystem.createBrowser() ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create our panel and add our control
    *  to it
    *  
    *
    *  @param docMaintPanel The DocMaintPanel we are a part of
    *  @param splitPane If this preview panel is in a JSplitPane, that JSplitPane instance
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocPrevPanel ( DocMaintPanel docMaintPanel ) {                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our layout and add our browser                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( browser.getComponent() ) ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign local vars                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.docMaintPanel = docMaintPanel ;
   }


   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return the minimum size
    *  that this component may display
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize() {
      if ( docMaintPanel.getPreviewShowing() )
         return( new Dimension ( 100, 10 ) ) ;
      return( new Dimension ( 100, 0 ) ) ;
   }



   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    Returns what we prefer to be
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( getMinimumSize() ) ;
   }


   /*------------------------------------------------------------*/
   /* setDocRec
    */
   /**
    *    This method will set our doc rec that
    *  we are currently displaying.  It will populate this panel
    * with that data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocument ( DocumentRow docRow ) {

      browser.setPageContent ( "<html><body><font size=2 face=\"Helvetica\"><center>"+Str.getStr ( STR_READING_PREVIEW )+"</center></font></body></html>" ) ;

      try {
         int rc = docRow.readDocument ( false ) ;

         if ( rc == 0 ) {
            Document doc = docRow.getDocument() ;

         } else
            LogSystem.log ( 1, "Cannot read Document Preview. rc:"+rc ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }

   /**
    *    This method will refresh our preview information
    *  if we don't already have it refreshed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readPreviewInfo ( ) {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      parentWin.setStatus ( Str.getStr ( STR_READING_PREVIEW ) ) ;


      //if ( !docRec.getPreviewInfoValid() ) {
      //   if ( docRec.readPreviewInfo() )
      //      setPreviewOutput() ;
      //   else
      browser.setPageContent ( "<html><body><center><font face=\"Helvetica, Arial\" size=2>"+Str.getStr ( STR_NO_PREVIEW_INFO )+"<br><br><h2>NOT IMPLEMENTED!</h2></font></center></body></html>" ) ;
      //}

      Qit.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;
   } /* refreshPreviewInfo  */




   /*------------------------------------------------------------*/
   /* setPreviewOutput
    */
   /**
    *    This method will set our output
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPreviewOutput () {
      //VOYif ( docRec != null ) {
      //VOY   String docBody    = "" ;
      //VOY   String previewStr = "<html>"+
      //VOY                       docRec.getFieldPreview ( PREVIEWFIELD_TITLE )+
      //VOY                       docRec.getFieldPreview ( PREVIEWFIELD_PUBLISHINGTAGS )+
      //VOY                       docRec.getFieldPreview ( PREVIEWFIELD_COMMENT )+
      //VOY                       docRec.getFieldPreview ( PREVIEWFIELD_COMMENTHISTORY )+
      //VOY                       docRec.getFieldPreview ( PREVIEWFIELD_DATABASEINFO ) ;                                          

      //VOY   /*-----------------------------------------------------------*/
      //VOY   /*                                                           */
      //VOY   /*   Add the document body if the document is not an SSD     */
      //VOY   /*                                                           */
      //VOY   /*-----------------------------------------------------------*/
      //VOY   if ( ( docRec.getFieldData ( FIELD_DOCUMENTTYPE ) == null ) ||
      //VOY        ( ((TypeDocRec)docRec.getFieldData ( FIELD_DOCUMENTTYPE )).getInd() != DocRec.DOCTYPE_SYSTEMSUPPORTDISKS ) )
      //VOY      docBody = (String) docRec.getFieldData ( FIELD_BODY ) ;

      //VOY   /*-----------------------------------------------------------*/
      //VOY   /*                                                           */
      //VOY   /*   Otherwise, add the additional information in            */
      //VOY   /*                                                           */
      //VOY   /*-----------------------------------------------------------*/
      //VOY   else
      //VOY      docBody = (String) docRec.getFieldData ( FIELD_ADDITIONALINFO ) ;

      //VOY   if ( ( docBody != null ) && ( docBody.length() > 0 ) ) {
      //VOY      docBody = DocRec.prepareBodyForPreview ( docBody ) ;


      //VOY      previewStr = previewStr+"<HR ALIGN=\"CENTER\">" ;
      //VOY      previewStr = previewStr+docBody ;
      //VOY   }

      //VOY   previewStr = previewStr+"</html>" ;

      //VOY   setPageContent ( previewStr ) ;
      //VOY   inRefresh = false ;

      //VOY} else {
      //VOY   setPageContent ( "<html><body><center><font face=\"Helvetica, Arial\" size=2>"+Str.getStr ( STR_NO_PREVIEW_INFO )+"</font></center></body></html>" ) ;
      //VOY}
   }



   /*------------------------------------------------------------*/
   /* cleanUp
    */
   /**
    *    This method will clean up all resources that
    *  this preview window is taking.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void close () {
   }



} /* DocPrevPanel */
