package com.ibm.nzna.projects.qit.doc.docmaint ;
/*-----------------------------------------------------------*/
/*   DocReviewPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocReviewPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.help.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This panel allows the user to select a year, month and brand
 * and will display information from a DocViewRec.VIEW_DOC_REVIEW. Please
 * note that for the filename and expiration date information to be
 * refreshed, you must add this panel as a ListSelectionListener to a
 * MultiList which is showing Documents.
 *
 *
 * @version 1.0, 8/31/1998 
 * @author  David Lentz
 */

public class DocReviewPanel extends JPanel implements Runnable, DocConst, ListSelectionListener , ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * This is the Year JLabel
     */
   private JLabel          st_YEAR      = null ;
   private JLabel          st_MONTH     = null ;
   private JLabel          st_BRAND     = null ;
   private JComboBox       cb_YEAR      = null ;
   private JComboBox       cb_MONTH     = null ;
   private JComboBox       cb_BRAND     = null ;
   private HotLinkLabel    pb_REFRESH   = null ;
   private DocViewListener viewListener = null ;

   /**
     * This is the Help button
     */
   private HotLinkLabel pb_HELP = null ;

   /**
     * This is the JLabel which shows 'Review'
     */
   private JLabel st_REVIEW = null ;

   /**
     * This is the 'Expired' JLabel
     */
   private JLabel st_EXPIRED = null ;

   /**
     * This is the JLabel which holds the document filename
     */
   private JLabel st_FILENAME = null ;

   /**
     * This property will turn on our refreshing or turn it off. If off
     * the Filename and expired time do not refresh
     */
   private boolean refresh = false ;

   /**
     * This is the current document that we are showing information to
     */
   private DocumentRow currentDoc = null ;


   /**
     * This constructor will initialize the panel and add all of the GUI
     */
   public DocReviewPanel ( DocViewListener viewListener ) {
      Vector           years    = new Vector ( 3 ) ;
      Vector           brands   = Brands.getBrands() ;
      CDate            date     = new CDate () ;
      int              curYear  = new Integer ( date.getDate ( CDate.DATE_YEAR ) ).intValue() ;
      TypeCategoryRec  brandRec = TypeCategory.getCategory ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEFAULT_BRAND ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our local vars                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.viewListener = viewListener;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Years in starting from 1999                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      for ( int i = 1999; i <= curYear; i++ )
         years.addElement ( ""+i ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the special 'All' brand                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      brands.insertElementAt ( Str.getStr ( STR_ALL ), 0 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_FILENAME = new JLabel ( "" ) ;
      st_YEAR     = new JLabel ( Str.getStr ( STR_YEAR ) ) ;
      st_MONTH    = new JLabel ( Str.getStr ( STR_MONTH ) ) ;
      st_BRAND    = new JLabel ( Str.getStr ( STR_BRAND ) ) ;
      cb_YEAR     = new JComboBox ( years ) ;
      cb_BRAND    = new JComboBox ( brands ) ;
      pb_REFRESH  = new HotLinkLabel ( Str.getStr ( STR_GO ) ) ;
      pb_HELP     = new HotLinkLabel ( Str.getStr ( STR_HELP ) ) ;
      st_REVIEW   = new JLabel ( Str.getStr ( STR_REVIEW ) ) ;
      st_EXPIRED  = new JLabel ( "" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_REFRESH.showSelection ( false ) ;
      pb_HELP.showSelection ( false ) ;
      st_REVIEW.setFont ( FontSystem.largeTitleFont ) ;
      setBackground ( Color.white ) ;
      setBorder ( GUISystem.loweredBorder ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our layout and add the controls                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_REVIEW ) ;
      add ( st_YEAR ) ;
      add ( cb_YEAR ) ;
      add ( st_MONTH ) ;
      add ( st_BRAND ) ;
      add ( cb_BRAND ) ;
      add ( pb_REFRESH ) ;
      add ( pb_HELP ) ;
      add ( st_FILENAME ) ;
      add ( st_EXPIRED ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our listeners                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_YEAR.addActionListener ( this ) ;
      pb_REFRESH.addActionListener ( this ) ;
      pb_HELP.addActionListener ( this ) ;

      /*------------------------------------------------------------*/
      /*                                                            */
      /*   Select this year and then refresh the months. Refreshing */
      /* the months will add the cb_MONTH drop down in              */
      /*                                                            */
      /*------------------------------------------------------------*/
      cb_YEAR.setSelectedItem ( ""+curYear ) ;
      refreshMonths ( ""+curYear, false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user has a favorite brand, select it             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( brandRec != null )
         cb_BRAND.setSelectedItem ( brandRec ) ;


   } /* Constructor  */


   /**
     * This method will lay out and position all of our components
     */
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_REVIEW.setBounds ( x, y, size.width-(x*2), rowHeight*2 ) ;
      y+=rowHeight*2 ;

      st_YEAR.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cb_YEAR.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=(rowHeight*2) ;

      st_MONTH.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      if ( cb_MONTH != null )
         cb_MONTH.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=(rowHeight*2) ;


      st_BRAND.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cb_BRAND.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=(rowHeight*2) ;

      pb_REFRESH.setBounds ( x, y, 50, rowHeight ) ;
      pb_HELP.setBounds ( size.width-70, y, 65, rowHeight ) ;
      y+=rowHeight*3 ;

      st_FILENAME.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      st_EXPIRED.setBounds ( x, y, size.width-(x*2), rowHeight*2 ) ;

   }

   /**
     * This method will destroy the current cb_MONTH control and
     * recreate it with the months for the year passed in. If the
     * year passed in is this year, then only months that are in the
     * past will show.
     *
     * @param year The year to show
     * @param forceLayout TRUE = force a layout once we are done
     */
   private void refreshMonths ( String year, boolean forceLayout ) {
      CDate   date      = new CDate () ;
      String  curYear   = date.getDate ( CDate.DATE_YEAR ) ;
      int     curMonth  = new Integer ( date.getDate ( CDate.DATE_MONTH ) ).intValue() ;
      boolean isCurYear = curYear.equals ( year ) ;
      Vector  months    = new Vector ( 10 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, kill the old list                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( cb_MONTH != null ) {
         remove ( cb_MONTH ) ;
         cb_MONTH = null ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, create the new list                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      /* Always add the 'All' in  */
      months.addElement ( Str.getStr ( STR_ALL ) ) ;

      if ( ( !isCurYear ) || ( curMonth >= 1 ) )
         months.addElement ( Str.getStr ( STR_JANUARY ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 2 ) )
         months.addElement ( Str.getStr ( STR_FEBRUARY ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 3 ) )
         months.addElement ( Str.getStr ( STR_MARCH ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 4 ) )
         months.addElement ( Str.getStr ( STR_APRIL ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 5 ) )
         months.addElement ( Str.getStr ( STR_MAY ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 6 ) )
         months.addElement ( Str.getStr ( STR_JUNE ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 7 ) )
         months.addElement ( Str.getStr ( STR_JULY ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 8 ) )
         months.addElement ( Str.getStr ( STR_AUGUST ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 9 ) )
         months.addElement ( Str.getStr ( STR_SEPTEMBER ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 10 ) )
         months.addElement ( Str.getStr ( STR_OCTOBER ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 11 ) )
         months.addElement ( Str.getStr ( STR_NOVEMBER ) ) ;
      if ( ( !isCurYear ) || ( curMonth >= 12 ) )
         months.addElement ( Str.getStr ( STR_DECEMBER ) ) ;

      cb_MONTH = new JComboBox ( months ) ;

      add ( cb_MONTH ) ;

      if ( forceLayout )
         revalidate() ;
   }

   /**
     * This method will be called when an item is selected from one of our
     * lists or the user hits the 'Go' button 
     */
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Year Drop Down Box                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == cb_YEAR )
         refreshMonths ( (String)cb_YEAR.getSelectedItem(), true ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Go Push Button                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REFRESH ) {
         DocViewRec      viewRec   = new DocViewRec ( DocViewRec.VIEW_DOC_REVIEW ) ;
         TypeCategoryRec brandRec  = null ;
         Vector          listeners = new Vector ( 1 ) ;

         try {
            brandRec = (TypeCategoryRec)cb_BRAND.getSelectedItem() ;
            viewRec.setBrandInd ( brandRec.getInd() ) ;
         } catch ( Exception ex ) {
            /* Do nothing because this is only thrown if
               the user selected 'All' as their brand  */
         }

         viewRec.setMonth ( (String)cb_MONTH.getSelectedItem() ) ;
         viewRec.setYear  ( (String)cb_YEAR.getSelectedItem() ) ;
         listeners.addElement ( viewListener ) ;
         viewRec.setListeners ( listeners ) ;
         viewRec.refresh() ;


      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The Help Pushbutton                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_HELP ) {
         HelpSystem.showHelp ( "doc/docmaint_review.html" ) ;
      }
   }

   /**
     * This method will set the refresh property
     */
   public void setRefresh ( boolean refresh ) {
      this.refresh = refresh ;
      if ( refresh )
         new Thread ( this ).start() ;
   }

   /**
     * This method will be called when a selection has occurred in a list we
     * have been added to
     */
   public void valueChanged ( ListSelectionEvent e ) {
      if ( e.getSource() instanceof MultiList ) {
         DocumentRow documentRow = (DocumentRow) ((MultiList)e.getSource()).getSelectedItem() ;

         if ( documentRow != null ) {
            st_FILENAME.setText ( "<html><body><b>"+documentRow.getFilename()+"</b></body></html>" ) ;
            st_EXPIRED.setText ( Str.getStr ( STR_READING ) ) ;
            currentDoc = documentRow ;
            
            if ( refresh )
               new Thread ( this ).start() ;
         }
      }
   }

   /**
     * This method will be run in a thread and will read the REVIEWPERIOD information
     * from the Quest database. It will then take that information and disply it in
     * the expiration date field
     */
   public void run () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "DocReviewPanel.run", 5 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = null ;

         rs = stmt.executeQuery ( "SELECT REVIEWPERIOD, CHANGEDTIME FROM TIGRIS.DOCUMENTS "+
                                  "WHERE DOCIND = "+currentDoc.getExistingDocumentInd() ) ;

         if ( rs.next() ) {
            int    reviewPeriod = rs.getInt(1) ;
            String changedTime  = rs.getString(2).trim() ;

            /* Ok, add the review period to the changed time  */
            changedTime = CDate.addMonthsToDate ( changedTime, reviewPeriod, CDate.DATE_STAMP ) ;

            st_EXPIRED.setText ( "<html><body><b>"+Str.getStr ( STR_EXPIRE_DATE )+":</b><br>"+DateSystem.prettyDateFromStampNoTime ( changedTime )+"</body></html>" ) ;


         } else
            st_EXPIRED.setText ( Str.getStr ( STR_UNKNOWN ) ) ;

         rs.close() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         st_EXPIRED.setText ( Str.getStr ( STR_UNKNOWN ) ) ;
         sqlMethod.rollBack () ;
      }

      sqlMethod.close() ;
   }

}
