package com.ibm.nzna.projects.qit.doc.docmaint ;       
/*-----------------------------------------------------------*/
/*   NewDocumentDlg                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NewDocumentDlg                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This dialog presents the user with the different ways to 
 * create a document. These ways include:<br><br>
 *
 * New Blank Document<br>
 * Document from a template<br>
 * Document from the clipboard<br>
 * Import Document<br><br>
 *
 * If a new blank document is selected, then this dialog also
 * asks which document type.
 *
 * @version 1.0, 3/9/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NewDocumentDlg                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class NewDocumentDlg extends JDialog implements  Runnable, MouseListener, TypeListListener, ListSelectionListener, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Thread Names                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  This thread will read all of the templates and populates 
     * the list of templates
     **/
   private static final String THREAD_POPULATE_TEMPLATES = "TPT" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int NEWDOC_CANCEL     = 0 ;
   public static final int NEWDOC_BLANK      = 1 ;
   public static final int NEWDOC_TEMPLATE   = 2 ;
   public static final int NEWDOC_MIGRATE    = 3 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JPanel        newTypePanel              = null ;
   private JPanel        docTypePanel              = null ;
   private JPanel        templatePanel             = null ;
   private JPanel        currentPanel              = null ;

   private ButtonPanel   buttonPanel               = null ;
   private JLabel        st_HELP                   = null ;
   private JTextArea     detailedHelp              = null ;
   private JLabel        st_IMAGE                  = null ;
   private DButton       pb_NEWDOC                 = null ;
   private DButton       pb_TEMPLATE               = null ;
   private DButton       pb_MIGRATE                = null ;
   private DButton       pb_CANCEL                 = null ;
   private DButton       pb_OK_DOC_TYPE            = null ;
   private DButton       pb_OK_DOC_TEMPLATE        = null ;
   private DList         documentTypes             = null ;
   private JTextArea     typeOutput                = null ;
   private JScrollPane   documentTypeScroll        = null ;
   private JScrollPane   outputScroll              = null ;
   private Color         buttonDefaultBackground   = Color.gray ;
   private Color         buttonHighlightBackground = new Color ( 220, 220, 220 ) ;
   private DocumentTable templates                 = null ;
   private CardLayout    cardLayout                = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int         newDocType = 0 ;
   private TypeDocRec  typeDocRec = null ;
   private DocumentRow template   = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create but not show our
    *  window. To show the window and wait for
    *  user input, please call the 'getResult' method
    *
    *  @param frame the parent AppDefaultWin
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NewDocumentDlg ( AppDefaultWin frame ) {
      super ( frame, Str.getStr ( STR_NEW_DOCUMENT ), true ) ;
      Container contentPane = getContentPane() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      newTypePanel       = new JPanel () ;
      docTypePanel       = new JPanel () ;    
      templatePanel      = new JPanel () ;

      st_IMAGE           = new JLabel  ( ImageSystem.getImageIcon ( this, ImageSystem.NEW_DOCUMENT ) ) ;
      st_HELP            = new JLabel  ( Str.getStr ( STR_CREATE_DOC_FROM ) ) ;
      detailedHelp     = new JTextArea () ;

      pb_NEWDOC          = new DButton ( Str.getStr ( STR_BLANK_DOC ),              ImageSystem.getImageIcon( this, ImageSystem.NEWBLANK ) ) ;
      pb_TEMPLATE        = new DButton ( Str.getStr ( STR_TEMPLATE ),               ImageSystem.getImageIcon( this, ImageSystem.FROMTEMPLATE ) ) ;
      pb_MIGRATE         = new DButton ( Str.getStr ( STR_MIGRATE_FROM_CLIPBOARD ), ImageSystem.getImageIcon( this, ImageSystem.FROMCLIPBOARD ) ) ;
      pb_CANCEL          = new DButton ( Str.getStr ( STR_CANCEL ) ) ;

      buttonPanel        = new ButtonPanel ();
      typeOutput         = new JTextArea () ;
      documentTypes      = new DList() ;
      pb_OK_DOC_TYPE     = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_OK_DOC_TEMPLATE = new DButton ( Str.getStr ( STR_OK ) ) ;
      outputScroll       = new JScrollPane ( typeOutput ) ;
      documentTypeScroll = new JScrollPane ( documentTypes ) ;
      templates          = new DocumentTable () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      documentTypes.addActionListener        ( this ) ;
      pb_NEWDOC.addActionListener            ( this ) ;
      pb_TEMPLATE.addActionListener          ( this ) ;
      pb_MIGRATE.addActionListener           ( this ) ;
      pb_CANCEL.addActionListener            ( this ) ;        
      pb_OK_DOC_TYPE.addActionListener       ( this ) ;
      pb_OK_DOC_TEMPLATE.addActionListener   ( this ) ;
      pb_NEWDOC.addMouseListener             ( this ) ;
      pb_TEMPLATE.addMouseListener           ( this ) ;
      pb_MIGRATE.addMouseListener            ( this ) ;
      documentTypes.addListSelectionListener ( this ) ;
      templates.addListSelectionListener     ( this ) ;
      templates.addActionListener            ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_NEWDOC.setHorizontalAlignment   ( JButton.CENTER ) ;
      pb_TEMPLATE.setHorizontalAlignment ( JButton.CENTER ) ;
      pb_MIGRATE.setHorizontalAlignment  ( JButton.CENTER ) ;
      TypeList.getInstance().refreshList ( documentTypes, TypeList.TYPE_DOC, this ) ;
      typeOutput.setLineWrap ( true ) ;
      typeOutput.setWrapStyleWord ( true ) ;
      typeOutput.setEditable ( false ) ;
      typeOutput.setFont ( FontSystem.defaultFont ) ;                        
      detailedHelp.setLineWrap ( true ) ;
      detailedHelp.setWrapStyleWord ( true ) ;
      detailedHelp.setEditable ( false ) ;
      detailedHelp.setFont ( FontSystem.defaultFont ) ;                        
      detailedHelp.setBorder ( GUISystem.emptyBorder ) ;
      detailedHelp.setOpaque ( false ) ;
      buttonDefaultBackground = pb_NEWDOC.getBackground() ;
      pb_NEWDOC.setToolTipText ( Str.getStr ( STR_NEW_BLANK_DOC_HELP ) ) ;
      pb_TEMPLATE.setToolTipText ( Str.getStr ( STR_NEW_TEMPLATE_DOC_HELP ) ) ;
      pb_MIGRATE.setToolTipText ( Str.getStr ( STR_NEW_CLIP_DOC_HELP ) ) ;
      pb_OK_DOC_TYPE.setEnabled ( false ) ;
      pb_OK_DOC_TEMPLATE.setEnabled ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add controls the the newTypePanel                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      newTypePanel.setLayout ( new GridLayout( 5, 1, 15, 5 ) ) ;
      newTypePanel.add ( pb_NEWDOC ) ;
      newTypePanel.add ( pb_TEMPLATE ) ;
      newTypePanel.add ( pb_MIGRATE ) ;
      newTypePanel.add ( detailedHelp ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to the docTypePanel                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      docTypePanel.setLayout ( null ) ;
      docTypePanel.add ( documentTypeScroll ) ;
      docTypePanel.add ( outputScroll ) ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to the Template panel                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      templatePanel.setLayout ( new BorderLayout() ) ;
      templatePanel.add ( templates ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign the current panel to the newTypePanel            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      currentPanel = new JPanel () ;
      currentPanel.setLayout ( cardLayout = new CardLayout ( 0, 0 ) ) ;
      currentPanel.add ( newTypePanel,  "newTypePanel" ) ;
      currentPanel.add ( templatePanel, "templatePanel" ) ;
      currentPanel.add ( docTypePanel,  "docTypePanel" )  ;                         
      cardLayout.first ( currentPanel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add controls to our main dialog                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.add ( pb_CANCEL ) ;
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_HELP ) ;
      contentPane.add ( buttonPanel ) ;                                   
      contentPane.add ( currentPanel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust final properties and center                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPropertiesOnPanel ( contentPane );    
      GUISystem.setPreferredButton ( pb_OK_DOC_TYPE ) ;
      GUISystem.setPreferredButton ( pb_OK_DOC_TEMPLATE ) ;
      setSize ( 495, 310 ) ;
      WinUtil.centerChildInParent ( this, frame ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Finalize init                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( DocTemplateSystem.getInstance() == null )
         new DocTemplateSystem() ;
      
   }


   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will return the result of what the
    *  user selected. It will be one of the NEWDOC_XXX
    *  ints.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getResult() {
      setVisible ( true ) ;
      return( newDocType ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all
    *  of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size       = getSize() ;
      int       y          = 10 ;
      int       rowHeight  = GUISystem.getRowHeight () ;                                          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, adjust the controls on the main window, this     */
      /* includes the graphic, the current panel and the cancel    */
      /* button                                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      super.doLayout() ;

      st_IMAGE.setBounds ( 10, 10, 118, 259 ) ;
      st_HELP.setBounds  ( 133, 10, size.width-(148), rowHeight ) ;
      y+=rowHeight ;

      currentPanel.setBounds ( 133, y, size.width-(148), size.height-y-70 ) ;
      buttonPanel.setBounds ( 133, size.height-65, size.width-(148), 25 ) ;                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, adjust the controls on the docTypePanel            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      size = docTypePanel.getSize() ;
      documentTypeScroll.setBounds ( 0, 0, size.width, size.height-(rowHeight*5) ) ;
      outputScroll.setBounds ( 0, size.height-(rowHeight*5)+5, size.width, rowHeight*4 ) ;

   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event from a GUI control is fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {                                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_TEMPLATE = Show the user the templates               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( pb_TEMPLATE == e.getSource() ) {
         newDocType = NEWDOC_TEMPLATE ;

         cardLayout.show ( currentPanel, "templatePanel" ) ;
         buttonPanel.remove ( pb_CANCEL ) ;
         buttonPanel.add ( pb_OK_DOC_TEMPLATE ) ;
         buttonPanel.add ( pb_CANCEL ) ;
         revalidate() ;
         new Thread ( this, THREAD_POPULATE_TEMPLATES ).start() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_NEWDOC = Show the user the document types            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_NEWDOC == e.getSource() ) {
         newDocType = NEWDOC_BLANK ;
         cardLayout.show ( currentPanel, "docTypePanel" ) ;
         st_HELP.setText ( Str.getStr ( STR_SELECT_DOC_TYPE ) ) ;
         buttonPanel.remove ( pb_CANCEL ) ;
         buttonPanel.add ( pb_OK_DOC_TYPE ) ;
         buttonPanel.add ( pb_CANCEL ) ;
         revalidate() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_MIGRATE = Show nothing, just return                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_MIGRATE == e.getSource() ) {
         newDocType = NEWDOC_MIGRATE ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL = close this window                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_CANCEL == e.getSource() ) {
         newDocType = NEWDOC_CANCEL ;
         dispose() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user hit the Ok button for document types        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( pb_OK_DOC_TYPE == e.getSource() ) ||
                ( documentTypes == e.getSource() ) ) {
         if ( typeDocRec != null )
            dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user hit the Ok button for the templates         */                                      
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( pb_OK_DOC_TEMPLATE == e.getSource() ) ||
                ( templates == e.getSource() ) ) {
         if ( template != null )
            dispose() ;
      }
      
   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    this method will be called when an item
    *  in the list is selected
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *///*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The list of Document Types                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == documentTypes ) {

         typeDocRec = (TypeDocRec) documentTypes.getSelectedValue() ;

         pb_OK_DOC_TYPE.setEnabled ( typeDocRec != null ) ;

         switch ( typeDocRec.getInd() ) {
            
            case TypeList.DOCTYPE_PARTS_INFO :
               typeOutput.setText ( Str.getStr ( STR_PARTS_INFO_DESCRIPT ) ) ;
               break ;

            case TypeList.DOCTYPE_PRODUCT_INFO :
               typeOutput.setText ( Str.getStr ( STR_PRODUCT_INFO_DESCRIPT ) ) ;
               break ;

            case TypeList.DOCTYPE_SERVICE_HINTS_TIPS :
               typeOutput.setText ( Str.getStr ( STR_SERVICE_HT_DESCRIPT ) ) ;
               break ;

            case TypeList.DOCTYPE_SERVICE_PUB :
               typeOutput.setText ( Str.getStr ( STR_SERVICE_PUB_DESCRIPT ) ) ;
               break ;

            case TypeList.DOCTYPE_SSD :
               typeOutput.setText ( Str.getStr ( STR_SSD_DESCRIPT ) ) ;
               break ;

            case TypeList.DOCTYPE_PRELOAD :
               typeOutput.setText ( Str.getStr ( STR_PRELOADSOFT_DESCRIPT ) ) ;
               break ;

            default:
               typeOutput.setText ( "" ) ;
               break ;

         }
      } /* if documentTypes */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The list of templates                                   */            
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == templates ) {
         template = (DocumentRow) templates.getSelectedItem() ;

         pb_OK_DOC_TEMPLATE.setEnabled ( template != null ) ;

      }
      
   }




   /*------------------------------------------------------------*/
   /* getTypeDocRec
    */
   /**
    *    This method will return the currently
    *  selected type doc rec
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TypeDocRec
    *//*---------------------------------------------------------*/
   public TypeDocRec getTypeDocRec () {
      return( typeDocRec ) ;
   }




   /*------------------------------------------------------------*/
   /* listRefreshed
    */
   /**
    *    This method will be called when our type
    *  list has been refreshed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void listRefreshed ( int typeList ) {
      docTypePanel.invalidate() ;
   }

   public void mouseClicked  ( MouseEvent e ) {
   }
   public void mousePressed  ( MouseEvent e ) {
   }
   public void mouseReleased ( MouseEvent e ) {
   }

   /**
     *  Subclassed to change the background color of a button
     *  to a highlighted look
     **/
   public void mouseEntered ( MouseEvent e ) {
      ((DButton)e.getSource()).setBackground ( buttonHighlightBackground ) ;
      detailedHelp.setText ( ((DButton)e.getSource()).getToolTipText() ) ;
   }

   /**
     *  Subclassed to change the color of a button back to normal
     **/
   public void mouseExited ( MouseEvent e ) {
      ((DButton)e.getSource()).setBackground ( buttonDefaultBackground ) ;
      detailedHelp.setText ( "" ) ;
   }

   /**
     *  This method will be run in a thread. This method has the
     * capability of doing many things, the things that is done depends
     * on the name of the thread
     **/
   public void run () {
      String operation = Thread.currentThread().getName () ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_POPULATE_TEMPLATES                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_POPULATE_TEMPLATES ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our indication that something is working            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         st_HELP.setText ( Str.getStr ( STR_READING_DOC_TEMPLATES ) ) ;
         pb_OK_DOC_TEMPLATE.setEnabled ( false ) ;
         pb_CANCEL.setEnabled ( false ) ;
         setCursor ( new Cursor ( Cursor.WAIT_CURSOR ) ) ;
         templates.setData ( DocTemplateSystem.getInstance().getTemplateVec() ) ;
         st_HELP.setText ( Str.getStr ( STR_SELECT_A_TEMPLATE ) ) ;
         setCursor ( new Cursor ( Cursor.DEFAULT_CURSOR ) ) ;
         pb_OK_DOC_TEMPLATE.setEnabled ( true ) ;
         pb_CANCEL.setEnabled ( true ) ;
      }

   }


   /**
     *  This method simply asks our contentPane to revalidate
     **/
   private void revalidate () {
      invalidate() ;
      ((JPanel)getContentPane()).revalidate() ;
      doLayout() ;
      cardLayout.layoutContainer ( currentPanel ) ;
      validate() ;
   }


   /**
     *  This method will return the template that the user selected. If the template
     *  has not yet been read in, this method will first read the template and then return
     *  the template.
     **/
   public DocumentDraft getTemplate () {
      int rc = template.readDocument ( false ) ;

      switch ( rc ) {
         case 0 :
            return ( (DocumentDraft)template.getDocument() ) ;

         case 100 :
            GUISystem.printBox ( STR_INFO, STR_DOCUMENT_NOT_FOUND ) ;
            LogSystem.log ( 1, "Document Template was not found." ) ;
            break ;

         default:
            GUISystem.printBox ( STR_INFO, STR_TEMPLATE_CANNOT_BE_READ ) ;
            LogSystem.log ( 1, "Cannot read template. Return Code:"+rc ) ;
            break ;
      }

      return ( null ) ;
   }
} /* NewDocumentDlg */
