package com.ibm.nzna.projects.qit.doc.docmaint.property ;           
/*-----------------------------------------------------------*/
/*   ViewPropertyPage                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewPropertyPage                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.help.* ;
/**
 * This page has user adjustable settings for the Document Views 
 *
 *
 * @version 1.0, 1/1/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewPropertyPage                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewPropertyPage extends JPanel implements ActionListener, DocConst, AppConst, ChangeListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel    st_MAX_ALLOWED_RETURN      = null ;
   private JSlider   maxSlider                  = null ;
   private JLabel    stt_MAX_ALLOWED_RETURN     = null ;
   private DButton   pb_HELP                    = null ;
   private JCheckBox ck_SHOW_MY_BRANDS_ONLY     = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ViewPropertyPage () {
      createGUI () ;
   }


   /*------------------------------------------------------------*/
   /* createGUI
    */
   /**
    *    This method will create and add our 
    *  GUI components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createGUI () {
      int currentMaxReturn = PropertySystem.getInt ( PROPERTY_MAX_DOCUMENT_RETURN ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_MAX_ALLOWED_RETURN  = new JLabel    ( Str.getStr ( STR_MAX_DOCUMENT_RETURN_FROM_VIEW ) ) ;
      stt_MAX_ALLOWED_RETURN = new JLabel    ( ""+currentMaxReturn ) ;
      maxSlider              = new JSlider   ( JSlider.HORIZONTAL ) ;          
      pb_HELP                = new DButton   ( Str.getStr ( STR_HELP ) ) ;
      ck_SHOW_MY_BRANDS_ONLY = new JCheckBox ( Str.getStr ( STR_SHOW_MY_BRANDS_ONLY ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      maxSlider.setMinimum ( 10 ) ;
      maxSlider.setMaximum ( 1500 ) ;
      maxSlider.setValue ( currentMaxReturn ) ;    
      ck_SHOW_MY_BRANDS_ONLY.setToolTipText ( Str.getStr ( STR_SHOW_MY_BRANDS_ONLY_TIP ) ) ;
      maxSlider.setToolTipText ( Str.getStr ( STR_MAX_DOCUMENT_RETURN_TIP ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      maxSlider.addChangeListener ( this ) ;
      pb_HELP.addActionListener   ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_MAX_ALLOWED_RETURN ) ;
      add ( stt_MAX_ALLOWED_RETURN ) ;
      add ( maxSlider ) ;
      add ( ck_SHOW_MY_BRANDS_ONLY ) ;
      add ( pb_HELP ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the data                                        */       
      /*                                                           */
      /*-----------------------------------------------------------*/
      ck_SHOW_MY_BRANDS_ONLY.setSelected ( PropertySystem.getBool ( PROPERTY_SHOW_MY_BRANDS_ONLY ) ) ;
      
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all
    *  of our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout ( ) {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = 5 ;
      int       y         = 5 ;

      super.doLayout() ;
      st_MAX_ALLOWED_RETURN.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      maxSlider.setBounds ( x, y, size.width/2, rowHeight+5 ) ;
      stt_MAX_ALLOWED_RETURN.setBounds ( size.width/2+5, y, (size.width/2)-5, rowHeight ) ;
      y+=rowHeight*2 ;

      ck_SHOW_MY_BRANDS_ONLY.setBounds ( x, y, size.width-(x*2), rowHeight ) ;

      pb_HELP.setBounds ( x, size.height-30, 70, 25 ) ;
   }





   /*------------------------------------------------------------*/
   /* stateChanged
    */
   /**
    *    Notified when the state of the slider has
    *  changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stateChanged ( ChangeEvent e ) {
      stt_MAX_ALLOWED_RETURN.setText ( ""+maxSlider.getValue() ) ;
   }

   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save the data on our
    *  screen. It will return TRUE if all data
    *  was saved
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      boolean rc = true ;

      PropertySystem.putInt  ( PROPERTY_MAX_DOCUMENT_RETURN, maxSlider.getValue() ) ;
      PropertySystem.putBool ( PROPERTY_SHOW_MY_BRANDS_ONLY, ck_SHOW_MY_BRANDS_ONLY.isSelected() ) ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_HELP                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_HELP ) {
         HelpSystem.showHelp ( "doc"+java.io.File.separator+"docmaint_prop.html" ) ;
      }

   }
} /* ViewPropertyPage */
