package com.ibm.nzna.projects.qit.doc.docview ;    
/*-----------------------------------------------------------*/
/*   DocViewPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocViewPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
/**
 * This panel holds all of the different types of 
 * views supported for searching on documents.
 *
 *
 * @version 1.0, 11/28/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocViewPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocViewPanel extends JPanel implements PropertyListener, Runnable, CustomViewListener, ActionListener, NavListExpandListener, DocConst, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_CATEGORY          = null ;
   private JComboBox    cb_CATEGORY          = null ;
   private JLabel       st_BRAND             = null ;
   private JComboBox    cb_BRAND             = null ;
   private NavList      viewList             = null ;
   private JScrollPane  viewScroll           = null ;
   private NavListGroup draftViewList        = null ;
   private NavListGroup activeViewList       = null ;           
   private NavListGroup customViewList       = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Secondary View Lists                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private NavListGroup openDraftBrandList       = null ;
   private NavListGroup openDraftBrandStatusList = null ;      
   private NavListGroup openDraftBrandOwnerList  = null ;      
   private NavListGroup activeDocTypeList        = null ;
   private NavListGroup activeDocTypeBrandList   = null ;
   private NavListGroup activeAuthorDocTypeList  = null ;
   private NavListGroup activeBrandFamilyList    = null ;
   private NavListGroup activeMachineList        = null ;
   private NavListGroup activePartNumberList     = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector listeners          = new Vector ( 1, 1 ) ;
   private HotLinkGroup hotLinkGroup = new HotLinkGroup () ;
   private boolean      initialized  = false ;
   private int          maxDocumentReturn   = PropertySystem.getInt ( PROPERTY_MAX_DOCUMENT_RETURN ) ;
   private boolean      showDeletedDocsOnly = false ;
   private DocViewRec   viewRec             = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default Constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocViewPanel () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      createGUI() ;         

   }





   /*------------------------------------------------------------*/
   /* createGUI
    */
   /**
    *    This method will create our GUI and add
    *  it to our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createGUI () {
      Vector categories = new Vector ( 0 ) ;
      Vector brands     = Brands.getBrands() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the special 'All' Category                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      categories.insertElementAt ( Str.getStr ( STR_PLEASE_SELECT_BRAND ), 0 ) ;
      brands.insertElementAt ( Str.getStr ( STR_PLEASE_SELECT_BRAND ), 0 ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_CATEGORY    = new JLabel    ( Str.getStr ( STR_CATEGORY ) ) ;
      cb_CATEGORY    = new JComboBox ( categories  ) ;
      viewList       = new NavList() ;
      viewScroll     = new JScrollPane ( viewList ) ;        
      st_BRAND       = new JLabel ( Str.getStr ( STR_BRAND ) ) ;
      cb_BRAND       = new JComboBox ( brands ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      viewScroll.setBorder ( GUISystem.emptyBorder ) ;      
      viewScroll.setBackground ( Color.white ) ;
      viewScroll.getViewport().setBackground ( Color.white ) ;
      viewList.setBackground ( Color.white ) ;
      setBackground ( Color.white ) ;
      setOpaque ( false ) ;
      viewScroll.setOpaque ( false );
      viewScroll.setBorder ( GUISystem.loweredBorder ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      CustomViewSystem.addCustomViewListener ( this ) ;
      PropertySystem.addPropertyListener ( PROPERTY_SHOW_MY_BRANDS_ONLY,          this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_DEFAULT_BRAND, this ) ;
      cb_BRAND.addActionListener ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_BRAND ) ;
      add ( cb_BRAND ) ;
      add ( st_CATEGORY ) ;
      add ( cb_CATEGORY ) ;
      add ( viewScroll ) ;
   }



   public void finalize ( ) {
      PropertySystem.removePropertyListener ( PROPERTY_SHOW_MY_BRANDS_ONLY, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_DEFAULT_BRAND, this ) ;
      CustomViewSystem.removeCustomViewListener ( this ) ;
      try {
         super.finalize() ;
      } catch ( Throwable e ) {

      }
   }


   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      //if ( initialized ) {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_BRAND.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cb_BRAND.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      st_CATEGORY.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cb_CATEGORY.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      viewScroll.setBounds ( x, y, size.width-(x*2), size.height-y ) ;
      //}
   }






   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will kick off a thread to refresh the views,
    *  or will refresh the views in the current thread.
    *  
    *
    *  @param newThread TRUE = start a new thread, FALSE = use current thread
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh ( boolean newThread) {
      if ( newThread )
         new Thread ( this ).start() ;
      else
         run() ;
   }

   public void run () {
      refreshViews() ;
      initialized = true ;
      invalidate() ;
      revalidate() ;
      repaint() ;
      viewScroll.revalidate() ;
   }

   /*------------------------------------------------------------*/
   /* refreshViews
    */
   /**
    *    This method will refresh our view list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshViews () {
      if ( draftViewList != null )
         viewList.remove ( draftViewList ) ;
      if ( activeViewList != null )
         viewList.remove ( activeViewList ) ;
      if ( customViewList != null )
         viewList.remove ( customViewList ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Main 3 Views (Draft, Active and Custom)      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftViewList  = createNavListGroup ( Str.getStr ( STR_DRAFTS )+" ("+Str.getStr ( STR_REFRESHING )+") ", 
                                            ImageSystem.getImageIcon ( this, ImageSystem.CLOSED_DRAFT_FOLDER ),
                                            ImageSystem.getImageIcon ( this, ImageSystem.OPEN_DRAFT_FOLDER ) ) ;
      activeViewList  = createNavListGroup ( Str.getStr ( STR_ACTIVE )+" ("+Str.getStr ( STR_REFRESHING )+") ", 
                                             ImageSystem.getImageIcon ( this, ImageSystem.CLOSED_ACTIVE_FOLDER ),
                                             ImageSystem.getImageIcon ( this, ImageSystem.OPEN_ACTIVE_FOLDER ) ) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add all of our main views                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      viewList.add ( draftViewList ) ;
      viewList.add ( activeViewList ) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now add our Drafts Views                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshDraftsViews() ; 
      draftViewList.setText ( Str.getStr ( STR_DRAFTS ) ) ;
      draftViewList.invalidate() ;
      invalidate() ;
      revalidate() ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Active Views                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshActiveViews() ; 
      activeViewList.setText ( Str.getStr ( STR_ACTIVE ) ) ;
      activeViewList.invalidate() ;
      invalidate() ;
      revalidate() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Custom Views                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshCustomViews() ; 
      customViewList.invalidate() ;
      invalidate() ;
      revalidate() ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Open any major view                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( PropertySystem.getBool ( PROPERTY_DRAFT_VIEW_OPEN ) )
         draftViewList.expand() ;
      if ( PropertySystem.getBool ( PROPERTY_ACTIVE_VIEW_OPEN ) )
         activeViewList.expand() ;
      if ( PropertySystem.getBool ( PROPERTY_CUSTOM_VIEW_OPEN ) )
         customViewList.expand() ;
   }





   /*------------------------------------------------------------*/
   /* refreshDraftsViews
    */
   /**
    *    This method will refresh all views under the
    *  main 'Draft' view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshDraftsViews () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   My Drafts                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftViewList.add ( createHotLinkLabel ( Str.getStr ( STR_MY_DRAFTS ),
                                               new DocViewRec ( DocViewRec.VIEW_DRAFTS_BY_ME ) ) ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Drafts by Author                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftViewList.add ( createHotLinkLabel ( Str.getStr ( STR_REQUESTOR ),
                                               new DocViewRec ( DocViewRec.VIEW_DRAFT_BY_AUTHOR ) ) ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Drafts By Brand/Status                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      openDraftBrandStatusList = createNavListGroup ( Str.getStr ( STR_BRAND_STATUS ) ) ;
      addBrandStatusToList ( openDraftBrandStatusList, new DocViewRec ( DocViewRec.VIEW_DRAFT_BY_BRAND_STATUS ) ) ;
      draftViewList.add ( openDraftBrandStatusList ) ;            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Drafts By Brand/Owner                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      openDraftBrandOwnerList = createNavListGroup ( Str.getStr ( STR_OWNER ) ) ;
      addBrandsToList   ( openDraftBrandOwnerList, new DocViewRec ( DocViewRec.VIEW_DRAFT_BY_BRAND_OWNER ) ) ;
      draftViewList.add ( openDraftBrandOwnerList ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Drafts By Comment                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftViewList.add ( createHotLinkLabel ( Str.getStr ( STR_COMMENT ),
                                               new DocViewRec ( DocViewRec.VIEW_DRAFT_BY_COMMENT ) ) ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Drafts By Source                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftViewList.add ( createHotLinkLabel ( Str.getStr ( STR_SOURCE ),
                                               new DocViewRec ( DocViewRec.VIEW_DRAFT_BY_SOURCE ) ) ) ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   All Open Drafts                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      draftViewList.add ( createHotLinkLabel ( Str.getStr ( STR_ALL_OPEN_REQUESTS ),
                                               new DocViewRec ( DocViewRec.VIEW_OPEN_DRAFTS ) ) ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Open Drafts by Brand                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      openDraftBrandList       = createNavListGroup ( Str.getStr ( STR_ALL_OPEN_REQUESTS_BRAND ) ) ;
      addBrandsToList ( openDraftBrandList, new DocViewRec ( DocViewRec.VIEW_DRAFT_ALL_BY_BRAND ) ) ;          
      draftViewList.add ( openDraftBrandList ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Auto-Open any Lists                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( PropertySystem.getBool ( PROPERTY_OPEN_DRAFT_BRAND_OPEN ) )
         openDraftBrandList.expand() ;

   }





   /*------------------------------------------------------------*/
   /* refreshActiveViews
    */
   /**
    *    This method will add all 'Active' views
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshActiveViews () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   My Documents                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeViewList.add ( createHotLinkLabel ( Str.getStr ( STR_MY_DOCS ), new DocViewRec ( DocViewRec.VIEW_MYDOCS ) ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   By Document Type                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeDocTypeList = createNavListGroup ( Str.getStr ( STR_BY_DOCTYPE ) ) ;
      addDocTypesToList ( activeDocTypeList, new DocViewRec ( DocViewRec.VIEW_BY_DOC_TYPE ) ) ;
      activeViewList.add ( activeDocTypeList ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Brand/Document Type                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeDocTypeBrandList = createNavListGroup ( Str.getStr ( STR_BY_BRAND ) ) ;
      addBrandDocTypesToList ( activeDocTypeBrandList, new DocViewRec ( DocViewRec.VIEW_BY_BRAND ) ) ;
      activeViewList.add ( activeDocTypeBrandList ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Author                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeAuthorDocTypeList = createNavListGroup ( Str.getStr ( STR_AUTHOR ) ) ;
      addDocTypesToList ( activeAuthorDocTypeList, new DocViewRec ( DocViewRec.VIEW_BY_AUTHOR ) ) ;
      activeViewList.add ( activeAuthorDocTypeList ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Brand/Family                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeBrandFamilyList = createNavListGroup ( Str.getStr ( STR_FAMILY ) ) ;
      addBrandFamilyDocTypesToList ( activeBrandFamilyList, new DocViewRec ( DocViewRec.VIEW_BY_BRANDGROUPIND ) ) ;
      activeViewList.add ( activeBrandFamilyList ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Machine                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeMachineList = createNavListGroup ( Str.getStr ( STR_PRODUCT ) ) ;
      addDocTypesToList ( activeMachineList, new DocViewRec ( DocViewRec.VIEW_BY_MACHINE ) ) ;
      activeViewList.add ( activeMachineList ) ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Option Part Number                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activePartNumberList = createNavListGroup ( Str.getStr ( STR_OPTION_PARTNUMBER ) ) ;
      addDocTypesToList ( activePartNumberList, new DocViewRec ( DocViewRec.VIEW_BY_OPTION_PARTNUMBER ) ) ;
      activeViewList.add ( activePartNumberList ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Source                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      activeViewList.add ( createHotLinkLabel ( Str.getStr ( STR_SOURCE ), new DocViewRec ( DocViewRec.VIEW_SOURCE ) ) ) ;


   }




   /*------------------------------------------------------------*/
   /* addBrandFamilyDocTypesToList
    */
   /**
    *    This method will add each Brand and Family
    *  to the group specified. The DocViewRec
    *  will be cloned down to the DocTypeRec view
    *
    *  @param group The NavListGroup to add the controls to
    *  @param viewRec The DocViewRec to clone
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addBrandFamilyDocTypesToList ( NavListGroup group, DocViewRec viewRec ) {
      Vector brands = Brands.getBrands() ;

      if ( ( brands != null ) && 
           ( brands.size() > 0 ) && 
           ( group != null ) && 
           ( viewRec != null ) ) {
         int          i          = 0 ;
         int          size       = brands.size() ;
         TypeCategoryRec brandRec   = null ;
         DocViewRec      newViewRec = null ;
         NavListGroup    brandGroup = null ;


         while ( i < size ) {
            brandRec = (TypeCategoryRec)brands.elementAt(i) ;

            if ( showBrandRec ( brandRec ) ) {
               newViewRec = (DocViewRec)viewRec.clone() ;

               newViewRec.setBrandInd ( brandRec.getInd() ) ;

               brandGroup = createNavListGroup ( brandRec.toString() ) ;
               addFamilyDocTypesToList ( brandGroup, brandRec.getInd(), newViewRec ) ;
               group.add ( brandGroup ) ;

            }
            i++ ;
         }

      }

   }




   /*------------------------------------------------------------*/
   /* addFamilyDocTypesToList
    */
   /**
    *    This method will add all families under the 
    *  brand passed to the group specified. All
    *  Doc Types will be added to each family
    *
    *  @param group The NavListGroup to add controls to
    *  @param brandInd The Brand to add families under
    *  @param viewRec The DocViewRec to clone and add to the DocTypes
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addFamilyDocTypesToList ( NavListGroup group, int brandInd, DocViewRec viewRec ) {
      Vector families = Brands.getFamilies ( brandInd ) ;

      if ( ( families != null ) && ( families.size() > 0 ) ) {
         int          i           = 0 ;
         int          size        = families.size() ;
         TypeCategoryRec familyRec   = null ;
         NavListGroup familyGroup = null ;
         DocViewRec   newViewRec  = null ;

         while ( i < size ) {
            familyRec = (TypeCategoryRec) families.elementAt(i) ;

            if ( familyRec != null ) {
               newViewRec = (DocViewRec)viewRec.clone() ;

               familyGroup = createNavListGroup ( familyRec.toString() ) ;
               addDocTypesToList ( familyGroup, newViewRec ) ;
               group.add ( familyGroup ) ;
            }
            i++ ;
         }

      }
   }


   /*------------------------------------------------------------*/
   /* addDocTypesToList
    */
   /**
    *    This method will add doc types to a NavList
    *  Group. The DocViewRec passed in will
    *  be cloned and added to each Doc Type
    *
    *  @param group The NavListGroup to add to
    *  @param viewRec The DocViewRec to clone
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addDocTypesToList ( NavListGroup group, DocViewRec viewRec ) {
      Vector types = TypeList.getInstance().getTypeList ( TypeList.TYPE_DOC ) ;

      if ( ( types != null ) && ( types.size() > 0 ) ) {
         int        i          = 0 ;
         int        size       = types.size() ;
         DocViewRec newViewRec = null ;
         TypeDocRec typeDocRec = null ;

         while ( i < size ) {
            newViewRec = (DocViewRec) viewRec.clone() ;
            typeDocRec = (TypeDocRec) types.elementAt(i) ;

            newViewRec.setDocTypeInd ( typeDocRec.getInd() ) ;
            group.add ( createHotLinkLabel ( typeDocRec.toString(), newViewRec ) ) ;
            i++ ;
         } /* while */

      } /* if */

   } /* addDocTypesToList */


   /*------------------------------------------------------------*/
   /* addBrandsToList
    */
   /**
    *    This method will add all brands to the
    *  NavListGroup passed in. Each brand will
    *  be assigned the view passed in as well
    *
    *  @param navListGroup The NavListGroup to add
    *  @param viewRec The DocViewRec that each brand makes a copy of and uses
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addBrandsToList ( NavListGroup group, DocViewRec viewRec ) {
      Vector brands = Brands.getBrands() ;

      if ( ( brands != null ) && 
           ( brands.size() > 0 ) && 
           ( group != null ) && 
           ( viewRec != null ) ) {
         int          i          = 0 ;
         int          size       = brands.size() ;
         TypeCategoryRec brandRec   = null ;
         DocViewRec   newViewRec = null ;


         while ( i < size ) {
            brandRec = (TypeCategoryRec)brands.elementAt(i) ;
            if ( showBrandRec ( brandRec ) ) {
               newViewRec = (DocViewRec)viewRec.clone() ;
               newViewRec.setBrandInd ( brandRec.getInd() ) ;
               group.add ( createHotLinkLabel ( brandRec.toString(), 
                                                newViewRec ) ) ;
            }
            i++ ;
         }

      }
   }




   /*------------------------------------------------------------*/
   /* addBrandDocTypesToList
    */
   /**
    *    This method will add all brands and DocTypes to the
    *  NavListGroup passed in. Each brand will
    *  be assigned the view passed in as well
    *
    *  @param navListGroup The NavListGroup to add
    *  @param viewRec The DocViewRec that each brand makes a copy of and uses
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addBrandDocTypesToList ( NavListGroup group, DocViewRec viewRec ) {
      Vector brands = Brands.getBrands() ;

      if ( ( brands != null ) && 
           ( brands.size() > 0 ) && 
           ( group != null ) && 
           ( viewRec != null ) ) {
         int          i          = 0 ;
         int          size       = brands.size() ;
         TypeCategoryRec brandRec   = null ;
         DocViewRec   newViewRec = null ;
         NavListGroup brandGroup = null ;


         while ( i < size ) {
            brandRec = (TypeCategoryRec)brands.elementAt(i) ;
            if ( showBrandRec ( brandRec ) ) {
               newViewRec = (DocViewRec)viewRec.clone() ;

               newViewRec.setBrandInd ( brandRec.getInd() ) ;

               brandGroup = createNavListGroup ( brandRec.toString() ) ;
               addDocTypesToList ( brandGroup, newViewRec ) ;
               group.add ( brandGroup ) ;
            }
            i++ ;
         }

      }
   }






   /*------------------------------------------------------------*/
   /* addBrandStatusToList
    */
   /**
    *    This method will add all brands and each doc status under each brand to the
    *  NavListGroup passed in. Each status will
    *  be assigned the view passed in as well
    *
    *  @param navListGroup The NavListGroup to add
    *  @param viewRec The DocViewRec that each brand makes a copy of and uses
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addBrandStatusToList ( NavListGroup group, DocViewRec viewRec ) {
      Vector brands = Brands.getBrands() ;
      Vector status = Status.getStatusList ( Status.STATUS_DOCUMENT ) ;

      if ( ( brands != null ) && 
           ( brands.size() > 0 ) && 
           ( group != null ) && 
           ( status != null ) && 
           ( status.size() > 0 ) &&
           ( viewRec != null ) ) {
         int           i           = 0 ;
         int           size        = brands.size() ;
         TypeCategoryRec  brandRec    = null ;
         DocViewRec    newViewRec  = null ;
         NavListGroup  brandList   = null ;
         TypeStatusRec statusRec   = null ;
         int           statusIndex = 0 ;
         int           statusSize  = status.size() ;


         while ( i < size ) {
            brandRec = (TypeCategoryRec)brands.elementAt(i) ;

            if ( showBrandRec ( brandRec ) ) {
               brandList = createNavListGroup ( brandRec.toString() ) ;

               statusIndex = 0 ;
               while ( statusIndex < statusSize ) {
                  statusRec = (TypeStatusRec) status.elementAt(statusIndex) ;

                  newViewRec = (DocViewRec)viewRec.clone() ;
                  newViewRec.setBrandInd ( brandRec.getInd() ) ;
                  newViewRec.setStatusInd ( statusRec.getInd() ) ;

                  brandList.add ( createHotLinkLabel ( statusRec.toString(),
                                                       newViewRec ) ) ;
                  statusIndex++ ;
               }

               group.add ( brandList ) ;
            }
            i++ ;
         }

      }
   }






   /*------------------------------------------------------------*/
   /* createNavListGroup
    */
   /**
    *    This method will create a NavListGroup that
    *  is configured to display in the views list
    *  
    *
    *  @param descript The title of the group
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private NavListGroup createNavListGroup ( String descript ) {
      return( createNavListGroup ( descript,
                                   ImageSystem.getImageIcon ( this, ImageSystem.CLOSE_TREE ),
                                   ImageSystem.getImageIcon ( this, ImageSystem.OPEN_TREE ) ) ) ;
   }





   /*------------------------------------------------------------*/
   /* createNavListGroup
    */
   /**
    *    This method will create a NavListGroup
    *  configured to work with this view control
    *  
    *
    *  @param descript The description of the NavListGroup
    *  @param closeIcon The icon for the closed status
    *  @param openIcon The icon for the open status
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private NavListGroup createNavListGroup ( String descript, ImageIcon closeIcon, ImageIcon openIcon ) {
      NavListGroup group = new NavListGroup ( descript, closeIcon ) ;

      group.addExpandListener ( this ) ;
      group.setExpandedIcon ( openIcon ) ;
      group.setBackground ( Color.white ) ;

      return( group ) ;
   }





   /*------------------------------------------------------------*/
   /* createHotLinkLabel
    */
   /**
    *    This method will create a default hot link label
    *  to be used in the views
    *  
    *
    *  @param Descript The Description of the hot link label
    *  @param docViewRec The DocViewRec to associate with the label
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private HotLinkLabel createHotLinkLabel ( String descript, DocViewRec docViewRec ) {
      HotLinkLabel hotLinkLabel = new HotLinkLabel ( descript, ImageSystem.getImageIcon ( this, ImageSystem.BLANK_TREE ), 0, docViewRec ) ;

      hotLinkLabel.addActionListener ( this ) ;
      hotLinkLabel.setBackground ( Color.white ) ;
      hotLinkGroup.add ( hotLinkLabel ) ;
      return( hotLinkLabel ) ;
   }


   /*------------------------------------------------------------*/
   /* expanded
    */
   /**
    *    This method will be called when a NavList
    *  of ours is expanded. We use this time to write down in the
    *  property file what nav list groups are expanded
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void expanded ( NavListGroup source, boolean expanded ) {
      if ( source == draftViewList )
         PropertySystem.putBool ( PROPERTY_DRAFT_VIEW_OPEN, expanded ) ;
      else if ( source == activeViewList )
         PropertySystem.putBool ( PROPERTY_ACTIVE_VIEW_OPEN, expanded ) ;
      else if ( source == customViewList )
         PropertySystem.putBool ( PROPERTY_CUSTOM_VIEW_OPEN, expanded ) ;

   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   A normal View Hit                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() instanceof HotLinkLabel ) {
         HotLinkLabel viewLabel = (HotLinkLabel) e.getSource() ;

         if ( viewLabel.getData() instanceof DocViewRec ) {
            DocViewRec viewRec = (DocViewRec)viewLabel.getData() ;
            boolean    refresh = false ;

            /* Populate the Category selected  */
            if ( cb_CATEGORY.getSelectedItem() instanceof TypeCategoryRec ) {
               viewRec.setCategoryInd ( ((TypeCategoryRec)cb_CATEGORY.getSelectedItem()).getInd() ) ;
               refresh = true ;
            } else if ( cb_CATEGORY.getSelectedItem() instanceof String ) {
               viewRec.setCategoryInd ( -1 ) ;
               refresh = true ;
            }

            if ( refresh ) {
               /* Now, set the listeners up so that they know when a view is done refreshing */
               viewRec.setListeners ( listeners ) ;
               viewRec.setActualCount ( 0 ) ;

               /* Now, launch the view */
               viewRec.setShowDeletedDocsOnly ( showDeletedDocsOnly ) ;
               viewRec.refresh() ;
            }
         }


      } /* hotLinkLabel */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The brand drop down                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == cb_BRAND ) {
         if ( cb_BRAND.getSelectedItem() instanceof TypeCategoryRec ) {
            Vector categories = TypeCategory.getChildCategories ( ((TypeCategoryRec)cb_BRAND.getSelectedItem()).getInd(), TypeCategory.LEVEL_DOC_CATEGORY ) ;

            categories.insertElementAt ( Str.getStr ( STR_ALL ), 0 ) ;
            remove ( cb_CATEGORY ) ;
            cb_CATEGORY = new JComboBox ( categories ) ;
            add ( cb_CATEGORY ) ;
            revalidate() ;
         }
      }

   } /* actionPerformed */





   /*------------------------------------------------------------*/
   /* addDocViewListener
    */
   /**
    *    This method will add a DocViewListener
    *  to this control. Any listener added will be notified
    *  once a view starts to read and when it stops
    *
    *  @param listener The DocViewListener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addDocViewListener ( DocViewListener listener ) {
      listeners.addElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* removeDocViewListener
    */
   /**
    *    This method will remove a DocViewListener
    *  from being notified of view changed
    *  
    *
    *  @param listener The DocViewListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeDocViewListener ( DocViewListener listener ) {
      listeners.removeElement ( listener ) ;
   }






   /*------------------------------------------------------------*/
   /* refreshCustomViews
    */
   /**
    *    This method will refresh all of the custom views
    *  and then add each one to the main list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshCustomViews () {
      DocCustomView customView         = null ;
      int           i                  = 0 ;
      Vector        userCustomViewVec  = null ;

      if ( customViewList != null ) {
         customViewList.collapse() ;
         viewList.remove ( customViewList ) ;
      }

      customViewList = createNavListGroup ( Str.getStr ( STR_CUSTOM_VIEWS ),
                                            ImageSystem.getImageIcon ( this, ImageSystem.CLOSED_ACTIVE_FOLDER ),
                                            ImageSystem.getImageIcon ( this, ImageSystem.OPEN_ACTIVE_FOLDER ) ) ;

      userCustomViewVec = CustomViewSystem.getViews ( CUSTOMVIEW_DOCUMENT ) ;

      /* Loop through the vector and add each item */
      if ( ( userCustomViewVec != null ) && ( userCustomViewVec.size() > 0 ) ) {
         int length = userCustomViewVec.size() ;

         while ( i < length ) {
            if ( userCustomViewVec.elementAt(i) instanceof DocCustomView ) {
               customView = (DocCustomView) userCustomViewVec.elementAt(i) ;
               if ( customView != null ) {
                  DocViewRec viewRec = new DocViewRec ( DocViewRec.VIEW_CUSTOM_VIEW ) ;

                  viewRec.setCustomView ( customView ) ;
                  customViewList.add ( createHotLinkLabel ( customView.toString(), viewRec ) ) ;
               }
            }
            i++ ;
         } /* while */
      } /* if */

      viewList.add ( customViewList ) ;

   }


   public void customViewRemoved ( CustomView customView ) {
      refreshCustomViews() ;
   }
   public void customViewUpdated ( CustomView customView ) {
      refreshCustomViews() ;
   }
   public void customViewAdded ( CustomView customView ) {
      if ( customView instanceof DocCustomView ) {
         DocViewRec viewRec = new DocViewRec ( DocViewRec.VIEW_CUSTOM_VIEW ) ;

         viewRec.setCustomView ( (DocCustomView)customView ) ;
         customViewList.add ( createHotLinkLabel ( customView.toString(), viewRec ) ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* showMyBrandsOnly
    */
   /**
    *    This method will return TRUE or FALSE
    *  depending on what the user has selected
    *  in their preferences. The 'showMyBrandsOnly'
    *  preference can make it so that this panel only
    *  shows a particular brand
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Boolean
    *//*---------------------------------------------------------*/
   private boolean showMyBrandsOnly () {
      return( PropertySystem.getBool ( PROPERTY_SHOW_MY_BRANDS_ONLY ) ) ;
   }





   /*------------------------------------------------------------*/
   /* showBrandRec
    */
   /**
    *    This method will return TRUE if the TypeCategoryRec
    *  passed should be shown in the list of views. If 
    *  the user has selected to 'Show My Brands Only', this
    *  method will return FALSE for all brands except the user's
    *  favorite brand
    *
    *  @param brandRec The TypeCategoryRec to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean showBrandRec ( TypeCategoryRec brandRec ) {
      boolean show = true ;

      if ( showMyBrandsOnly () )
         show = ( brandRec.getInd() == PropertySystem.getInt ( PropertySystem.PROPERTY_DEFAULT_BRAND ) ) ;

      return( show ) ;
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has changed. This normally
    *  makes the list refresh itself
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object newValue ) {
      refreshViews() ;
   }

   /**
     * This method will set the maximum amount of documents returned
     * from any query this panel runs
     */
   public void setMaxDocumentReturn ( int max ) {
      this.maxDocumentReturn = max ;
   }

   /**
     * This method will turn on/off the ability to display deleted documents
     * only.
     */
   public void setShowDeletedDocsOnly ( boolean show ) {
      this.showDeletedDocsOnly = show ;
   }


   /**
     * If there has been a view selected, this method will
     * refresh that view.
     */
   public void refreshCurrentView () {

      if ( viewRec != null ) {
         /* Populate the Category selected  */
         viewRec.setCategoryInd ( ((TypeDocCatRec)cb_CATEGORY.getSelectedItem()).getInd() ) ;

         /* Now, set the listeners up so that they know when a view is done refreshing */
         viewRec.setListeners ( listeners ) ;
         viewRec.setActualCount ( 0 ) ;
         viewRec.setMaxDocumentReturn ( maxDocumentReturn ) ;
         viewRec.setShowDeletedDocsOnly ( showDeletedDocsOnly ) ;

         /* Now, launch the view */
         viewRec.refresh() ;
      }

   }

} /* DocViewPanel */
