package com.ibm.nzna.projects.qit.doc.docview ;     
/*-----------------------------------------------------------*/
/*   DocViewRec                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocViewRec                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import java.util.* ;
/**
 * This record holds information about a product search 
 *
 *
 * @version 1.0, 7/16/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   DocViewRec                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocViewRec extends Object implements DocConst, AppConst {

   public static final int VIEW_DOCUMENT            = 1 ;
   public static final int VIEW_DOCTEMPLATE         = 2 ;        


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Search Types                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int VIEW_BY_BRAND                 = 0 ;
   public static final int VIEW_BY_DOC_TYPE              = 1 ;
   public static final int VIEW_BY_AUTHOR                = 2 ;
   public static final int VIEW_BY_BRANDGROUPIND         = 3 ;
   public static final int VIEW_BY_MACHINE               = 4 ;
   public static final int VIEW_DRAFT_BY_AUTHOR          = 5 ;
   public static final int VIEW_DRAFT_BY_BRAND_STATUS    = 6 ;
   public static final int VIEW_DRAFT_BY_BRAND_OWNER     = 7 ;
   public static final int VIEW_USER_TEMPLATES           = 8 ;
   public static final int VIEW_BY_TITLE                 = 9 ; 
   public static final int VIEW_DOC_BODY                 = 10 ;
   public static final int VIEW_BY_NOTESID               = 11 ;
   public static final int VIEW_BY_QUESTIND              = 12 ;
   public static final int VIEW_BY_BRAND_CATEGORY        = 13 ;
   public static final int VIEW_CUSTOM_VIEW              = 14 ;
   public static final int VIEW_DRAFTS_BY_ME             = 15 ;
   public static final int VIEW_BY_OPTION_MACHINETYPE    = 16 ;
   public static final int VIEW_OPEN_DRAFTS              = 17 ;
   public static final int VIEW_DRAFT_BY_COMMENT         = 19 ;
   public static final int VIEW_SOURCE                   = 20 ;
   public static final int VIEW_DRAFT_BY_SOURCE          = 21 ;
   public static final int VIEW_BY_FAMILY_CAT            = 22 ;
   public static final int VIEW_DRAFT_ALL_BY_BRAND       = 23 ;
   public static final int VIEW_MYDOCS                   = 24 ;
   public static final int VIEW_QUERY_TEXT               = 25 ;
   public static final int VIEW_DRAFT_BOOKMARKS          = 26 ;
   public static final int VIEW_ACTIVE_BOOKMARKS         = 27 ;
   public static final int VIEW_EXACT_SQL                = 28 ; 
   public static final int VIEW_DOCBODY                  = 29 ;
   public static final int VIEW_BY_BRANDCATEGORY         = 30 ;
   public static final int VIEW_BY_OPTION_PARTNUMBER     = 31 ;
   public static final int VIEW_DOC_REVIEW               = 32 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Directions                                              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int PB_PREV    = 1 ;
   public static final int PB_NEXT    = 2 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Column Headings                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     * This is the list of column headings needed to display the info
     */
   String[] listTitle = { 
       " ",
       Str.getStr ( STR_DOC_IND ),
       Str.getStr ( STR_TITLE ),
       Str.getStr ( STR_LOTUS_NOTES_ID ),
       Str.getStr ( STR_LAST_TOUCHED_ON )} ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public int            directionInd            = 0 ;
   private int           brandInd                = 0 ;
   private int           viewType                = 0 ;
   private int           brandGroupInd           = 0 ;
   public int            fromCount               = 0 ;
   public int            newFromCount            = 0 ;   
   public int            actualCount             = 0 ;
   public String         descript                = "" ;
   private Vector        bookmarkVec             = null ;
   private int           statusInd               = 0 ;
   private int           categoryInd             = 0 ;
   private Vector        listeners               = null ;
   private int           docTypeInd              = 0 ;
   private String        partNumber              = null ;
   private Vector        results                 = null ;
   private String        queryText               = null ;
   private String        comment                 = null ;
   private String        source                  = null ;
   private UserRec       userRec                 = null ;
   private String        machine                 = null ;
   private String        title                   = null ;
   private String        sqlQuery                = null ;
   private String        year                    = null ;
   private String        month                   = null ;
   private int           maxDocumentReturn       = PropertySystem.getInt ( PROPERTY_MAX_DOCUMENT_RETURN ) ;
   private DocCustomView customView              = null ;
   private boolean       showDeletedDocsOnly     = false ;
   private String        filename                = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize our variables to
    *  the ones passed in
    *  
    *
    *  @param viewType What type of view this is. This must be one of the VIEW_*** values
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocViewRec ( int viewType ) {
      setSearchType ( viewType ) ;
   }

   /*------------------------------------------------------------*/
   /* setSearchType
    */
   /**
    *    This method will set what type of search
    *  we are performing here. This type must be defined
    *  in this class as one of the VIEW_XXX
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSearchType ( int viewType ) {
      this.viewType = viewType ;
   }



   /*------------------------------------------------------------*/
   /* clone
    */
   /**
    *    This method creates a copy of this object
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object clone () {
      DocViewRec viewRec = new DocViewRec ( viewType ) ;

      viewRec.directionInd   = directionInd ;
      viewRec.viewType       = viewType ;
      viewRec.fromCount      = fromCount ;
      viewRec.newFromCount   = newFromCount ;
      viewRec.actualCount    = actualCount ;
      viewRec.docTypeInd     = docTypeInd ;
      viewRec.descript       = descript ;
      viewRec.statusInd      = statusInd ;
      viewRec.brandInd       = brandInd ;
      viewRec.categoryInd    = categoryInd ;
      viewRec.listeners      = listeners ;
      viewRec.comment        = comment ;
      viewRec.source         = source ;
      viewRec.brandGroupInd  = brandGroupInd ;
      viewRec.userRec        = userRec ;
      viewRec.machine        = machine ;
      viewRec.title          = title ;

      return( viewRec ) ;
   }




   /*------------------------------------------------------------*/
   /* setBrandInd
    */
   /**
    *    This method will set our brand indicator
    *  
    *  
    *
    *  @param brandInd The Brand Indicator that matches what is in TIGRIS.TYPEBRAND
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrandInd ( int brandInd ) {
      this.brandInd = brandInd ;
   }





   /*------------------------------------------------------------*/
   /* setStatusInd
    */
   /**
    *    This method will set the status indicator
    *  to use when searching.
    *  
    *
    *  @param statusInd The Status Indicator as defined in TIGRIS.TYPESTATUS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatusInd ( int statusInd ) {
      this.statusInd = statusInd ;
   }





   /*------------------------------------------------------------*/
   /* getStatusInd
    */
   /**
    *    This method will return the status indicator
    *  that this view will search with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The status indicator as defined in TIGRIS.TYPESTATUS
    *//*---------------------------------------------------------*/
   public int getStatusInd () {
      return( statusInd ) ;
   }


   /*------------------------------------------------------------*/
   /* setCategoryInd
    */
   /**
    *    This method will set the category indicator
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCategoryInd ( int categoryInd ) {
      this.categoryInd = categoryInd ;
   }




   /*------------------------------------------------------------*/
   /* setListeners
    */
   /**
    *    This method will set the all DocViewListeners
    *  that will be notified once the view is complete
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setListeners ( Vector listeners ) {
      this.listeners = listeners ;
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will attempt to start a read
    *  of the currently configured view. This will
    *  kick off a thread to do the work so this method
    *  immediatly returns
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      new RefreshDocViewThread ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshNow
    */
   /**
    *    This method will attempt to refresh
    *  the current view. This method will not return
    *  until the view is read in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshNow () {
      new RefreshDocViewThread ( this, false ).run() ;
   }



   /*------------------------------------------------------------*/
   /* getResults
    */
   /**
    *    This method will return the results of a view
    *  once the view has completed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector of DocRecs
    *//*---------------------------------------------------------*/
   public Vector getResults () {
      return( results ) ;
   }





   /*------------------------------------------------------------*/
   /* isCustomView
    */
   /**
    *    This method will return TRUE if this view
    *  is a user created view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean isCustomView () {
      return( customView != null ) ;
   }




   /*------------------------------------------------------------*/
   /* getCustomView
    */
   /**
    *    This method will return the CustomView
    *  object that describes the custom view
    *  to use in this view
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DocCustomView
    *//*---------------------------------------------------------*/
   public DocCustomView getCustomView () {
      return( customView ) ;
   }




   /*------------------------------------------------------------*/
   /* setCustomView
    */
   /**
    *    This method will set the custom query
    *  that this view will represent
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCustomView ( DocCustomView customView ) {
      this.customView = customView ;
   }




   /*------------------------------------------------------------*/
   /* getListeners
    */
   /**
    *    This method will return the list of listeners
    *  that need to be notified of this view starting
    *  and stopping
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getListeners () {
      return( listeners ) ;
   }




   /*------------------------------------------------------------*/
   /* getViewType
    */
   /**
    *    This method will return the type of view that this
    *  is. This number is represented by one of the VIEW_XXX
    *  constants defined in this class
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getViewType () {
      return( viewType ) ;
   }





   /*------------------------------------------------------------*/
   /* getFromCount
    */
   /**
    *    This method will return the 'From' count
    *  that this view is refreshing from. This number
    *  represents the first record index returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getFromCount () {
      return( fromCount ) ;
   }




   /*------------------------------------------------------------*/
   /* getCategoryInd
    */
   /**
    *    This method will return the Document Category
    *  that should be returned in a view. If a -1 is
    *  returned, then the view applies to All categories
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getCategoryInd () {
      return( categoryInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setResults
    */
   /**
    *    This method will set the results that are returned
    *  to the listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setResults ( Vector results ) {
      this.results = results ;
   }




   /*------------------------------------------------------------*/
   /* getBrandInd
    */
   /**
    *    This method will return the brand indicator
    *  that is used. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int The brand indicator as defined in TIGRIS.TYPEBRAND
    *//*---------------------------------------------------------*/
   public int getBrandInd () {
      return( brandInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getQueryText
    */
   /**
    *    This method will return our query text
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getQueryText () {
      return( queryText ) ;
   }



   /*------------------------------------------------------------*/
   /* getComment
    */
   /**
    *    This method will return the comments. 
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getComment () {
      return( getComment ( false ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getComment
    */
   /**
    *    This method will ask the user for comments
    *  and then return them.
    *  
    *
    *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the comment property
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getComment ( boolean showGUI ) {
      if ( showGUI )
         comment = new SingleEntryDlg ( MainWindow.getInstance(),
                                        Str.getStr ( STR_COMMENT ),
                                        Str.getStr ( STR_COMMENT ),
                                        comment,
                                        new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;

      return( comment ) ;
   }




   /*------------------------------------------------------------*/
   /* getSource
    */
   /**
    *    This method will return the sources. 
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getSource () {
      return( getSource ( false ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getSource
    */
   /**
    *    This method will ask the user for sources
    *  and then return them.
    *  
    *
    *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the source property
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getSource ( boolean showGUI ) {
      if ( showGUI )
         source = new SingleEntryDlg ( MainWindow.getInstance(),
                                       Str.getStr ( STR_SOURCE ),
                                       Str.getStr ( STR_SOURCE ),
                                       source,
                                       new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;

      return( source ) ;
   }






   /*------------------------------------------------------------*/
   /* getUserId
    */
   /**
    *    This method will return the userIds. 
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getUserId () {
      return( getUserId ( false ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getUserId
    */
   /**
    *    This method will ask the user for userIds
    *  and then return them.
    *  
    *
    *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the userId property
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getUserId ( boolean showGUI ) {
      if ( showGUI ) {
         Vector users = new SelectUserDlg ( MainWindow.getInstance(),
                                            userRec ).getResult() ;

         if ( ( users != null ) && ( users.size() > 0 ) )
            userRec = (UserRec)users.elementAt(0) ;
      }

      if ( userRec != null )
         return( userRec.getUserId() ) ;
      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocTypeInd
    */
   /**
    *    This method will set the Document Type Indicator
    *  
    *  
    *
    *  @param docTypeInd
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocTypeInd ( int docTypeInd ) {
      this.docTypeInd = docTypeInd ;
   }





   /*------------------------------------------------------------*/
   /* getDocTypeInd
    */
   /**
    *    This method will return the document type
    *  indicator
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getDocTypeInd () {
      return( docTypeInd ) ;
   }





   /*------------------------------------------------------------*/
   /* setBrandGroupInd
    */
   /**
    *    This method will set the BrandGroupInd (combination
    *  of a Brand and Family) that this view will
    *  show
    *
    *  @param brandGroupInd The Brand/Group Indicator as defined in TIGRIS.BRANDS
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrandGroupInd ( int brandGroupInd ) {
      this.brandGroupInd = brandGroupInd ;
   }




   /*------------------------------------------------------------*/
   /* getBrandGroupInd
    */
   /**
    *    This method will return the brand group indicator
    *  of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return brandGroupInd The BrandGroupInd as defined in TIGRIS.BRANDS
    *//*---------------------------------------------------------*/
   public int getBrandGroupInd ( ) {
      return( brandGroupInd ) ;
   }




   /*------------------------------------------------------------*/
   /* getMachine
    */
   /**
    *    This method will return the machines. 
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getMachine () {
      return( getMachine ( false ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getMachine
    */
   /**
    *    This method will ask the user for machines
    *  and then return them.
    *  
    *
    *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the machine property
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getMachine ( boolean showGUI ) {
      if ( showGUI )
         machine = new SingleEntryDlg ( MainWindow.getInstance(),
                                        Str.getStr ( STR_MACHINE ),
                                        Str.getStr ( STR_MACHINE ),
                                        machine,
                                        new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 4 ) ).getResult() ;

      return( machine ) ;
   }






   /*------------------------------------------------------------*/
   /* setBookmarkVec
    */
   /**
    *    This method will set the bookmark vec
    *  that this view is refreshing with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBookmarkVec ( Vector vec ) {
      bookmarkVec = vec ;
   }




   /*------------------------------------------------------------*/
   /* getBookmarkVec
    */
   /**
    *    This method will return the bookmark
    *  vector we are using
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getBookmarkVec () {
      return( bookmarkVec ) ;
   }





   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set the title to search on
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitle ( String title ) {
      this.title = title ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title to search on
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( title ) ;
   }




   /*------------------------------------------------------------*/
   /* getSQLQuery
    */
   /**
    *    This method will return the SQL Query to
    *  run if this view is of type VIEW_EXACT_SQL
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getSQLQuery () {
      return( sqlQuery ) ;
   }




   /*------------------------------------------------------------*/
   /* setSQLQuery
    */
   /**
    *    This method will set the SQL Query to run
    *  if this view is of type VIEW_EXACT_SQL
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSQLQuery ( String sqlQuery ) {
      this.sqlQuery = sqlQuery ;
   }




   /*------------------------------------------------------------*/
   /* setActualCount
    */
   /**
    *    This method will set the actual count
    *  property. This is the number of actual
    *  documents that fit in this view
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setActualCount ( int actualCount ) {
      this.actualCount = actualCount ;
   }




   /*------------------------------------------------------------*/
   /* setPartNumber
    */
   /**
    *    This method will set the part number for
    *  the VIEW_BY_OPTION_PARTNUMBER
    *  view
    *
    *  @param partNumber The Part Number to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPartNumber ( String partNumber ) {
      this.partNumber = partNumber ;
   }




   /*------------------------------------------------------------*/
   /* getPartNumber
    */
   /**
    *    this method will return the part number
    *  for the view VIEW_BY_OPTION_PARTNUMBER
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getPartNumber () {
      return( getPartNumber ( false ) ) ;
   }

   /*------------------------------------------------------------*/
   /* getMachine
    */
   /**
    *    This method will ask the user for machines
    *  and then return them.
    *  
    *
    *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the machine property
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getPartNumber ( boolean showGUI ) {
      if ( showGUI )
         partNumber = new SingleEntryDlg ( MainWindow.getInstance(),
                                           Str.getStr ( STR_OPTION_PARTNUMBER ),
                                           Str.getStr ( STR_OPTION_PARTNUMBER ),
                                           machine,
                                           new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 7 ) ).getResult() ;

      return( partNumber ) ;
   }


   /**
     * This method will return the maximum number of documents this view will return.
     * By Default this is the user configurable property PROPERTY_MAX_DOCUMENT_RETURN
     */
   public int getMaxDocumentReturn () {
       return( maxDocumentReturn ) ;
   }

   /**
     * This method will set the maximum number of documents this view will return.
     * By Default this is the user configurable property PROPERTY_MAX_DOCUMENT_RETURN
     *
     * @param max The number of documents to return
     */
   public void setMaxDocumentReturn ( int max ) {
       this.maxDocumentReturn = max ;
   }

   /**
     * This method will turn on/off the ability to display deleted documents
     * only.
     */
   public void setShowDeletedDocsOnly ( boolean show ) {
       this.showDeletedDocsOnly = show ;
   }

   /**
     * This method will return TRUE if we are to show only deleted docs, FALSE
     * if we show only non-deleted docs
     */
   public boolean getShowDeletedDocsOnly () {
       return( showDeletedDocsOnly ) ;
   }

   /**
     * This method will set the year that a VIEW_DOC_REVIEW view would use as
     * a baseline
     */
   public void setYear ( String year ) {
       this.year = year ;
   }

   /**
     * This method will set the month that a VIEW_DOC_REVIEW view would use as
     * a baseline
     */
   public void setMonth ( String month ) {
       this.month = month ;
   }

   /**
     * This method will return the year that this view is using
     */
   public String getYear () {
       return( year ) ;
   }

   /**
     * This method will return the month that this view is using
     */
   public String getMonth () {
       return( month ) ;
   }

   /**
     * This method will return the list of column headings that should
     * be placed in a MultiList or JTable
     */
   public String[] getColumnHeadings () {
       return( listTitle ) ;
   }

   /**
     * This method will set a title in the column headings
     */
   public void setColumnHeading ( int titleIndex, String title ) {
       listTitle[titleIndex] = title ;
   }

} /* DocViewRec */
