package com.ibm.nzna.projects.qit.doc.fieldchanger ;                    
/*-----------------------------------------------------------*/
/*   DocBFTMFieldChanger                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocBFTMFieldChanger                 */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.product.* ;
/**
 * This field changer will change the document Brand/Family/Machine/Model of 
 * a list of documents
 *
 * @version 1.0, 11/22/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocBFTMFieldChanger                                     */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocBFTMFieldChanger extends Object implements DocConst, FieldChanger, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean continueProcess = true ;





   /*------------------------------------------------------------*/
   /* beginProcess
    */
   /**
    *    This method will begin processing the document
    *  list passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean beginProcess ( Vector docList, Object data, boolean updateOnly, DocFieldChangerPanel changerPanel ) {
      boolean      rc             = false ;
      int          i              = 0 ;
      int          size           = docList.size() ;
      StringBuffer sqlBuffer      = new StringBuffer() ;
      DocRec       docRec         = null ;
      int          countryCodeInd = PropertySystem.getInt ( PropertySystem.PROPERTY_COUNTRY ) ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Form what big buffer to send to SQL Runner              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( continueProcess ) && ( i < size ) ) {
         docRec = (DocRec)docList.elementAt(i) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Delete                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         createSQL ( docRec.getDocInd(), (ProdDocRec)data, sqlBuffer, updateOnly ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Update the comments                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sqlBuffer.append ( "UPDATE QUEST.DOCUMENTS " ) ;
         sqlBuffer.append ( "SET DBUSER = '"+UserSystem.getUserId()+"', " ) ;
         sqlBuffer.append ( "CHANGEDTIME = CURRENT TIMESTAMP, " ) ;
         sqlBuffer.append ( "COMMENT = '"+Text.cleanDBString ( (String)docRec.getFieldData ( FIELD_COMMENT ) )+"' " ) ;
         sqlBuffer.append ( "WHERE DOCIND = "+docRec.getDocInd() ) ;
         sqlBuffer.append ( SqlRunner.END_DELIM ) ;

         i++ ;
      } /* while  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, send the buffer                                    */           
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( continueProcess )
         rc = QITSqlRunner.sqlRunner ( sqlBuffer.toString() ) ;

      return( rc ) ;
   } /* beginProcess  */


   /*------------------------------------------------------------*/
   /* stopProcess
    */
   /**
    *    This method will stop the processing at the next
    *  available point
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopProcess () {
      continueProcess = false ;
   }


   private void createSQL ( int docInd, ProdDocRec prodDocRec, StringBuffer docSqlString, boolean updateOnly ) {
      if ( prodDocRec != null ) {
         try {
            boolean            cont          = true ;
            Vector             familyVec     = prodDocRec.getAssociatedFamilies() ;
            Vector             machineVec    = prodDocRec.getAssociatedMachines() ;
            Vector             productVec    = prodDocRec.getAssociatedModels() ;
            int                index         = 0 ;
            int                length        = familyVec.size() ;
            DocFamilyRec       docFamilyRec  = null ;
            MachineRec         machineRec    = null ;
            ProductRec         productRec    = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   IF we are to replace all brand/family/machine/model     */
            /* entries, delete all first                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( !updateOnly ) {
               docSqlString.append ("DELETE FROM QUEST.DOCBRAND WHERE DOCIND = "+docInd+QITSqlRunner.END_DELIM) ;
               docSqlString.append ("DELETE FROM QUEST.PRODDOCS WHERE DOCIND = "+docInd+QITSqlRunner.END_DELIM) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add each Family, no matter if 'All' or 'Direct'         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            index=0 ;
            while ( index < length ) {
               docFamilyRec = (DocFamilyRec) familyVec.elementAt(index) ;
               if ( updateOnly ) {
                  docSqlString.append ( "DELETE FROM QUEST.DOCBRAND " ) ;
                  docSqlString.append ( "WHERE DOCIND = "+docInd+" AND " ) ;
                  docSqlString.append ( "      BRANDGROUPIND = "+docFamilyRec.getBrandGroupInd() ) ;
                  docSqlString.append ( QITSqlRunner.END_DELIM ) ;
               }
               docSqlString.append ("INSERT INTO QUEST.DOCBRAND "+
                                    "( BRANDGROUPIND, DOCIND, DBUSER, CHANGEDTIME ) "+
                                    "VALUES "+
                                    "( "+docFamilyRec.getBrandGroupInd()+", "+
                                    "  "+docInd+", "+
                                    " '"+UserSystem.getUserId()+"', "+
                                    "  CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ) ;

               index++ ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add the All-All product for each 'All' Family           */                                  
            /*                                                           */
            /*-----------------------------------------------------------*/
            index=0 ;
            while ( index < length ) {

               docFamilyRec = (DocFamilyRec) familyVec.elementAt(index) ;

               if ( docFamilyRec.getAll() ) {
                  int allProdInd = AppSQL.getAllAllProdInt ( docFamilyRec.getBrandGroupInd() );

                  if ( updateOnly ) {
                     docSqlString.append ( "DELETE FROM QUEST.PRODDOCS " ) ;
                     docSqlString.append ( "WHERE DOCIND = "+docInd+" AND " ) ;
                     docSqlString.append ( "      PRODINT = "+allProdInd ) ;
                     docSqlString.append ( QITSqlRunner.END_DELIM ) ;
                  }
                  docSqlString.append ("INSERT INTO QUEST.PRODDOCS "+
                                       "( PRODINT, DOCIND, DBUSER, CHANGEDTIME ) "+
                                       "VALUES "+
                                       "( "+allProdInd+", "+
                                       "  "+docInd+", "+
                                       " '"+UserSystem.getUserId()+"', "+
                                       "  CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ) ;
               }
               index++ ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add each 'XXXX-All' product for each 'All' machine      */                                       
            /*                                                           */
            /*-----------------------------------------------------------*/
            index=0 ;
            length=machineVec.size() ;
            while ( index < length ) {
               machineRec = (MachineRec) machineVec.elementAt ( index ) ;
               
               if ( machineRec.getAll() ) {
                  int allModelProdInt = AppSQL.getAllModelProdInt ( machineRec.getMachine(), machineRec.getBrandGroupInd() ) ;

                  if ( updateOnly ) {
                     docSqlString.append ( "DELETE FROM QUEST.PRODDOCS " ) ;
                     docSqlString.append ( "WHERE DOCIND = "+docInd+" AND " ) ;
                     docSqlString.append ( "      PRODINT = "+allModelProdInt ) ;
                     docSqlString.append ( QITSqlRunner.END_DELIM ) ;
                  }

                  docSqlString.append ("INSERT INTO QUEST.PRODDOCS "+
                                       "( PRODINT, DOCIND, DBUSER, CHANGEDTIME ) "+
                                       "VALUES "+
                                       "( "+allModelProdInt+", "+
                                       "  "+docInd+", "+
                                       " '"+UserSystem.getUserId()+"', "+
                                       "  CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ) ;
               }
               index++ ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add each direct product association                     */                        
            /*                                                           */
            /*-----------------------------------------------------------*/
            index=0 ;
            length=productVec.size() ;
            while ( index < length ) {
               productRec = (ProductRec) productVec.elementAt ( index ) ;

               if ( updateOnly ) {
                  docSqlString.append ( "DELETE FROM QUEST.PRODDOCS " ) ;
                  docSqlString.append ( "WHERE DOCIND = "+docInd+" AND " ) ;
                  docSqlString.append ( "      PRODINT = "+productRec.getProdInt() ) ;
                  docSqlString.append ( QITSqlRunner.END_DELIM ) ;
               }

               docSqlString.append ("INSERT INTO QUEST.PRODDOCS "+
                                    "( PRODINT, DOCIND, DBUSER, CHANGEDTIME ) "+
                                    "VALUES "+
                                    "( "+productRec.getProdInt()+", "+
                                    "  "+docInd+", "+
                                    " '"+UserSystem.getUserId()+"', "+
                                    "  CURRENT TIMESTAMP )"+QITSqlRunner.END_DELIM ) ;
               index++ ;
            }

         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }
   }

} /* DocBFTMFieldChanger */
