package com.ibm.nzna.projects.qit.doc.fieldchanger ;
/*-----------------------------------------------------------*/
/*   DocFieldChangerPanel                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocFieldChangerPanel                */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;    
import java.util.* ;
import java.awt.event.* ;    
import java.sql.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.docedit.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This panel is simply a statistics panel that 
 * will display results from a Global Change 
 *
 *
 * @version 1.0, 10/3/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocFieldChangerPanel                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocFieldChangerPanel extends JPanel implements QuestProcess, ActionListener, QuestPanel, Runnable, DocConst, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel          st_PROCESSING             = null ;
   private JLabel          st_STATUS                 = null ;
   private JLabel          st_DOCSCOMPLETED          = null ;
   private JLabel          st_DOCSFAILED             = null ;
   private MultiList       cnr_DOCCOMPLETE           = null ;
   private MultiList       cnr_DOCFAILED             = null ;
   private ActionButton    pb_CLOSE                  = null ;
   private ListActionPanel completeListPanel         = null ;
   private ListActionPanel failedListPanel           = null ;
   private HotLinkLabel    pb_EDIT_COMPLETE          = null ;
   private HotLinkLabel    pb_EDIT_FAILED            = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean        inThread           = false ;
   private String         comments           = "" ;
   private Vector         docList            = null ;
   private String         fieldName          = null ;
   private Object         dataObject         = null ;
   private boolean        updateOnly         = false ;
   private int            progressValue      = 0 ;
   private AppDefaultWin  parentWin          = null ;
   private boolean        continueProcessing = false ;

   private static final String[] listTitle = { 
      " ",
      Str.getStr ( STR_DOC_IND ),
      Str.getStr ( STR_TITLE ),
      Str.getStr ( STR_LOTUS_NOTES_ID ),
      Str.getStr ( STR_LAST_TOUCHED_ON )} ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create the blank panel
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public DocFieldChangerPanel ( Vector docList, String fieldName, Object dataObject, String comments, boolean updateOnly ) {                
      this.docList    = docList ;
      this.fieldName  = fieldName ;
      this.dataObject = dataObject ;
      this.comments   = comments ;
      this.updateOnly = updateOnly ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize and create our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Action Buttons                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the rest of the controls                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_PROCESSING     = new JLabel ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
      st_STATUS         = new JLabel ( "" ) ;
      st_DOCSCOMPLETED  = new JLabel ( Str.getStr ( STR_DOCS_COMPLETED ) ) ;
      st_DOCSFAILED     = new JLabel ( Str.getStr ( STR_DOCS_FAILED ) ) ;
      cnr_DOCCOMPLETE   = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_DOCFAILED     = new MultiList ( GUISystem.getFontUtil() ) ;        
      completeListPanel = new ListActionPanel () ;
      failedListPanel   = new ListActionPanel () ;
      pb_EDIT_COMPLETE  = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT_FAILED    = new HotLinkLabel ( Str.getStr ( STR_EDIT ) ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_PROCESSING.setOpaque ( false ) ;
      st_STATUS.setOpaque ( false ) ;
      st_DOCSCOMPLETED.setOpaque ( false ) ;
      st_DOCSFAILED.setOpaque ( false ) ;       

      st_PROCESSING.setFont ( FontSystem.textFont ) ;
      st_STATUS.setFont ( FontSystem.textFont ) ;
      st_DOCSCOMPLETED.setFont ( FontSystem.textFont ) ;
      st_DOCSFAILED.setFont ( FontSystem.textFont ) ;           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.addActionListener         ( this ) ;
      pb_EDIT_COMPLETE.addActionListener ( this ) ;
      pb_EDIT_FAILED.addActionListener   ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our lists up                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_DOCCOMPLETE.setColumnHeadings ( listTitle ) ;
      cnr_DOCCOMPLETE.showColumnHeadings ( true ) ;
      cnr_DOCCOMPLETE.setColumnWidth ( 0, 20 ) ;
      cnr_DOCCOMPLETE.setColumnWidth ( 1, 75 ) ;
      cnr_DOCCOMPLETE.setColumnWidth ( 2, 300 ) ;
      cnr_DOCCOMPLETE.setColumnWidth ( 3, 100 ) ;
      cnr_DOCCOMPLETE.setColumnWidth ( 4, 200 ) ;

      cnr_DOCFAILED.setColumnHeadings ( listTitle ) ;
      cnr_DOCFAILED.showColumnHeadings ( true ) ;
      cnr_DOCFAILED.setColumnWidth ( 0, 20 ) ;
      cnr_DOCFAILED.setColumnWidth ( 1, 75 ) ;
      cnr_DOCFAILED.setColumnWidth ( 2, 300 ) ;
      cnr_DOCFAILED.setColumnWidth ( 3, 100 ) ;
      cnr_DOCFAILED.setColumnWidth ( 4, 200 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our panel                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_PROCESSING ) ;
      add ( st_STATUS ) ;
      add ( st_DOCSCOMPLETED ) ;
      add ( st_DOCSFAILED ) ;
      add ( cnr_DOCCOMPLETE ) ;
      completeListPanel.add ( pb_EDIT_COMPLETE ) ;
      add ( completeListPanel ) ;
      add ( cnr_DOCFAILED ) ;
      failedListPanel.add ( pb_EDIT_FAILED ) ;
      add ( failedListPanel ) ;


      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing. We will start now
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {

      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;

      new Thread ( this ).start() ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the user
    *  has stopped us. We will release all resources
    *  here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      if ( inThread )
         GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_STILL_PROCESSING ) ) ;
      else {
         removeAll() ;
         st_PROCESSING             = null ;
         st_STATUS                 = null ;
         st_DOCSCOMPLETED          = null ;
         st_DOCSFAILED             = null ;
         cnr_DOCCOMPLETE           = null ;
         cnr_DOCFAILED             = null ;
         return( true ) ;
      }

      return( false ) ;
   }


   /*-----------------------------------------------------------*/
   /* layoutControls
    */
   /**
    *    This method will position and size all of our
    *  controls
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_PROCESSING.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      st_STATUS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      st_DOCSCOMPLETED.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      cnr_DOCCOMPLETE.setBounds ( x, y, size.width-(x*2), size.height-y-rowHeight*10 ) ;

      completeListPanel.setBounds ( x, size.height-rowHeight*10, size.width-(x*2), rowHeight ) ;

      st_DOCSFAILED.setBounds ( x, size.height-rowHeight*9, size.width-(x*2), rowHeight ) ;

      cnr_DOCFAILED.setBounds ( x, size.height-rowHeight*8, size.width-(x*2), rowHeight*7 ) ;

      failedListPanel.setBounds ( x, size.height-rowHeight, size.width-(x*2), rowHeight ) ;

   }

   /*-----------------------------------------------------------*/
   /* refreshAll
    */
   /**
    *    This method will refresh the text on all of our controls
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void refreshAll () {

      st_PROCESSING.setText ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
      st_DOCSCOMPLETED.setText ( Str.getStr ( STR_DOCS_COMPLETED ) ) ;
      st_DOCSFAILED.setText ( Str.getStr ( STR_DOCS_FAILED ) ) ;

   }


   /*-----------------------------------------------------------*/
   /* shutDown
    */
   /**
    *    This method will be called if the user tries
    *  to shut us down
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public boolean shutdown () {
      if ( inThread )
         GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_STILL_PROCESSING ) ) ;
      return( !inThread ) ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will simply call our saveDocFieldChange
    *  method to save all field changes to a document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      ProcessManagerSystem.addProcess ( this ) ;
      continueProcessing = true ;
      inThread = true ;

      try {
         Qit.setEnabled ( false ) ;
         parentWin.setStatus ( Str.getStr ( STR_READING_COMMENTS ) ) ;
         parentWin.setValueMax ( docList.size() ) ;
         parentWin.setStatusLock ( true ) ;                                                                               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, read in all of the comments for each document    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         //VOY readDocumentComments() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now create a FieldChanger class to change the       */
         /* correct field                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parentWin.setValue ( 0 ) ;
         if ( continueProcessing ) {

            parentWin.setStatusLock ( false ) ;
            parentWin.setStatus ( STR_PLEASE_WAIT ) ;
            parentWin.setStatusLock ( true ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Doc Type Changer                                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( fieldName.equals ( Str.getStr ( STR_DOCUMENT_TYPE ) ) )
               continueProcessing = new  DocTypeFieldChanger ().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Doc Classification                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_DOC_CLASS ) ) )
               continueProcessing = new  DocClassFieldChanger ().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Category                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_DOCUMENT_CATEGORY ) ) )
               continueProcessing = new  DocCategoryFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Status                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_STATUS ) ) )
               continueProcessing = new  DocStatusFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Brand/Family/Machine/Model                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            //VOY else if ( ( fieldName.equals ( Str.getStr ( STR_BRAND_FAMILY ) ) ) ||
            //VOY           ( fieldName.equals ( Str.getStr ( STR_MACHINETYPE_MODEL ) ) ) )
            //VOY    continueProcessing = new  DocBFTMFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Location                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_LOCATION ) ) )
               continueProcessing = new  DocLocationFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Publishing Tags                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_PUBLISHING_TAGS ) ) )
               continueProcessing = new  DocTagsFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Metrics                                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_METRICS ) ) )
               continueProcessing = new  DocMetricFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Body Replace                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_BODY ) ) )
               continueProcessing = new  DocBodyFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Title Replace                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_TITLE ) ) )
               continueProcessing = new  DocTitleFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document URL ( With No File Attachment )                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_URL_FOR_NO_FILE_ATTACH ) ) )
               continueProcessing = new  DocURLFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Publication Type                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_PUBLICATION ) ) )
               continueProcessing = new  DocPubFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Release Date                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_RELEASE_DATE ) ) )
               continueProcessing = new  DocReleaseDateFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Document Part Number                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( fieldName.equals ( Str.getStr ( STR_PART_NUMBER ) ) )
               continueProcessing = new  DocPartNumberFieldChanger().beginProcess ( docList, dataObject, updateOnly, this ) ;





            if ( continueProcessing ) {
               st_STATUS.setText ( "" ) ;
               st_PROCESSING.setText ( Str.getStr ( STR_COMPLETE ) ) ;
               cnr_DOCCOMPLETE.add ( docList ) ;
            }

         }



         parentWin.setStatusLock ( false ) ;
         parentWin.setStatus ( null ) ;
         Qit.setEnabled ( true ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      inThread = false ;
      ProcessManagerSystem.removeProcess ( this ) ;
      GUISystem.printBox ( STR_INFO, STR_GLOBAL_CHANGE_COMPLETE ) ;
   }




   /*------------------------------------------------------------*/
   /* getDocumentComments
    */
   /**
    *    This method will read in all of the document comments
    *  and sets that field in each document in the docList
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
    //VOYprivate void readDocumentComments () {
    //VOY   SQLMethod      sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDocumentComments", 5 ) ;
    //VOY   int            i         = 0 ;
    //VOY   int            size      = docList.size() ;
    //VOY   DocumentDraft  draft     = null ;
    //VOY   boolean        increment = true ;

    //VOY   try {
    //VOY      Statement stmt = sqlMethod.createStatement() ;
    //VOY      ResultSet rs   = null ;

    //VOY      while ( ( continueProcessing ) && ( i < size ) ) {
    //VOY         draft = (DocumentDraft)docList.elementAt(i) ;                          

    //VOY         /*-----------------------------------------------------------*/
    //VOY         /*                                                           */
    //VOY         /*   First, read in the current comments                     */
    //VOY         /*                                                           */
    //VOY         /*-----------------------------------------------------------*/
    //VOY         st_STATUS.setText ( Str.getStr ( STR_READING )+" '"+draft.getTitle()+"'" ) ;
    //VOY         rs = stmt.executeQuery ( "SELECT COMMENT, MAINTLOCK, DBUSER "+
    //VOY                                  "FROM QUEST.DOCUMENTS "+
    //VOY                                  "WHERE DOCIND = "+draft.getDocumentInd()+" FOR FETCH ONLY" ) ;

    //VOY         if ( rs.next() ) {
    //VOY            String dbUser    = rs.getString(3).trim() ;
    //VOY            String maintLock = rs.getString(2).trim() ;

    //VOY            draft.setL
    //VOY            draft.setFieldData ( FIELD_COMMENT, comments ) ;
    //VOY            draft.setFieldData ( FIELD_COMMENT, getComments ( draft, rs.getString(1).trim() ) ) ;

    //VOY            if ( ( maintLock.equalsIgnoreCase ( "Y" ) ) &&
    //VOY                 ( !dbUser.equals ( UserSystem.getUserId() ) ) ) {
    //VOY               GUISystem.printBox ( Str.getStr ( STR_INFO ),
    //VOY                                    Str.getStr ( STR_CANNOT_MASS_CHANGE_DOCUMENT )+" '"+draft.getFieldData ( FIELD_TITLE )+"' "+
    //VOY                                    Str.getStr ( STR_BECAUSE_IT_IS_LOCKED_BY )+" "+UserSystem.getNameFromUserId ( dbUser ) ) ;
    //VOY               increment=false ;
    //VOY               docList.removeElementAt (i) ;
    //VOY               size = docList.size() ;
    //VOY            }

    //VOY         }

    //VOY         rs.close() ;
    //VOY         parentWin.setValue ( i ) ;
    //VOY         
    //VOY         if ( increment )
    //VOY            i++ ;
    //VOY         increment=true ;
    //VOY      }

    //VOY   } catch ( Exception e ) {
    //VOY      continueProcessing = false ;
    //VOY      sqlMethod.rollBack() ;
    //VOY      LogSystem.log ( 1, e, false ) ;
    //VOY      GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
    //VOY   }

    //VOY   st_STATUS.setText ( "" ) ;
    //VOY   sqlMethod.close() ;
    //VOY} /* readDocumentComments  */





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the status text we
    *  are displaying
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatus ( String status ) {
      if ( status == null )
         status="" ;

      st_STATUS.setText ( status ) ;
   }



   /*------------------------------------------------------------*/
   /* getComments
    */
   /**
    *    This method will return the newly updated
    *  comments that includes the comments for
    *  this mass change
    *
    *  @param draft The DocRec to get the original comments from
    *  @param None
    *  @param None
    *
    *  @return String the new comments
    *//*---------------------------------------------------------*/
   private String getComments ( DocumentDraft draft, String originalComment ) {
      String comment         = draft.getComments() ;

      if ( originalComment == null )
         originalComment = "" ;

      if ( ( comment == null ) || ( comment.length() == 0 ) )
         comment = originalComment+"\nOn "+DateSystem.getDate ( CDate.DATE_DATE )+" "+UserSystem.getUserName()+" saved the document, but made no comments"  ;
      else
         comment = originalComment+"\n"+Str.getStr ( STR_ON )+DateSystem.getDate ( CDate.DATE_DATE)+" "+UserSystem.getUserName()+" "+Str.getStr ( STR_WROTE )+":\n"+comment ;

      return( comment ) ;
   }


   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name of this
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_DOC_PROCESSOR ) ) ;
   }



   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon that represents
    *  this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_SMALL ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return NULL since we
    *  do not have a Nav Panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( null ) ;
   }





   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return this component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will not cut any text
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will not copy any text
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }



   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will not paste any text
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when this panel
    *  is Selected. We will do nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }


   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when this panel
    *  is Deselected. We will do nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event needs to be handled
    *  
    *
    *  @param e The actionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( Qit.getEnabled() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_CLOSE = Frees our records and closes                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == pb_CLOSE ) {
            GUISystem.getParentDefWin(this).closePanel ( this, WindowSystem.findQuestPanel ( "DOCMAINT" ) ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_EDIT_COMPLETE                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_EDIT_COMPLETE ) {
            DocumentRow draft = (DocumentRow) cnr_DOCCOMPLETE.getSelectedItem() ;

            if ( draft != null ) {
               EditDocument.editDocument ( draft ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_EDIT_FAILED                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_EDIT_FAILED ) {
            DocumentRow draft = (DocumentRow) cnr_DOCFAILED.getSelectedItem() ;

            if ( draft != null ) {
               EditDocument.editDocument ( draft ) ;
            }
         }


      }
   }

   public boolean bookmark () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getProcessName
    */
   /**
    *    THis method will return our process' name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getProcessName () {
      return( "DocFieldChanger" ) ;
   }




   /*------------------------------------------------------------*/
   /* stopProcess
    */
   /**
    *    This method will attempt to stop the
    *  current process that is going on
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopProcess () {
      continueProcessing = false ;
   }

} /* DocFieldChangerPanel */

