package com.ibm.nzna.projects.qit.doc.fieldchanger ;                    
/*-----------------------------------------------------------*/
/*   DocLocationFieldChanger                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocLocationFieldChanger             */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.qit.doc.* ;
/**
 * This field changer will change the location of 
 * a list of documents
 *
 * @version 1.0, 11/22/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocLocationFieldChanger                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocLocationFieldChanger extends Object implements DocConst, FieldChanger, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean continueProcess = true ;





   /*------------------------------------------------------------*/
   /* beginProcess
    */
   /**
    *    This method will begin processing the document
    *  list passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean beginProcess ( Vector docList, Object data, boolean updateOnly, DocFieldChangerPanel changerPanel ) {
      boolean      rc        = false ;
      int          i         = 0 ;
      int          size      = docList.size() ;
      StringBuffer sqlBuffer = new StringBuffer() ;
      DocumentDraft       draft    = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Form what big buffer to send to SQL Runner              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( ( continueProcess ) && ( i < size ) ) {
         draft = FieldChangerUtil.getDraftFromRow ( (DocumentRow)docList.elementAt(i) ) ;

         if ( draft != null ) {

            createSQL ( draft.getDocInd(), (LocaleRec)data, sqlBuffer, updateOnly ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Update the comments                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            sqlBuffer.append ( "UPDATE QUEST.DOCUMENTS " ) ;
            sqlBuffer.append ( "SET DBUSER = '"+UserSystem.getUserId()+"', " ) ;
            sqlBuffer.append ( "CHANGEDTIME = CURRENT TIMESTAMP, " ) ;
            sqlBuffer.append ( "COMMENT = '"+Text.cleanDBString ( draft.getOriginalComments()+"\n"+draft.getComments() )+"' " ) ;
            sqlBuffer.append ( "WHERE DOCIND = "+draft.getDocInd() ) ;
            sqlBuffer.append ( SqlRunner.END_DELIM ) ;
            ((DocumentRow)docList.elementAt(i)).freeDocument() ;
         }

         i++ ;
      } /* while  */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, send the buffer                                    */           
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( continueProcess )
         rc = QITSqlRunner.sqlRunner ( sqlBuffer.toString() ) ;

      return( rc ) ;
   } /* beginProcess  */


   /*------------------------------------------------------------*/
   /* stopProcess
    */
   /**
    *    This method will stop the processing at the next
    *  available point
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopProcess () {
      continueProcess = false ;
   }


   private void createSQL ( int docInd, LocaleRec localeRec, StringBuffer docSqlString, boolean updateOnly ) {

      if ( localeRec != null ) {
         TypeCountryCodeRec countryRec   = null ;
         TypeGeoRec         geoRec       = null ;
         String             sqlStmt      = null ;

         /* First, delete all locale associations to this product now */
         if ( !updateOnly ) {
            docSqlString.append ("DELETE FROM QUEST.DOCGEO WHERE DOCIND = "+docInd+QITSqlRunner.END_DELIM) ;
            docSqlString.append ("DELETE FROM QUEST.DOCCOUNTRY WHERE DOCIND = "+docInd+QITSqlRunner.END_DELIM) ;
         }

         /* Ok, if we are world Wide, only insert 1 geography with an indicator
            of 6 */
         if ( localeRec.isWorldWide() ) {
            docSqlString.append ( "DELETE FROM QUEST.DOCGEO WHERE DOCIND = "+docInd+QITSqlRunner.END_DELIM ) ;
            sqlStmt = "INSERT INTO QUEST.DOCGEO "+
                      "( DOCIND, GEOIND, DBUSER, CHANGEDTIME )"+
                      "VALUES "+
                      " ( "+docInd+", "+
                      "   6, "+
                      "   '"+UserSystem.getUserId()+"', "+
                      "   CURRENT TIMESTAMP )" ;
            docSqlString.append (sqlStmt+QITSqlRunner.END_DELIM) ;
         }


         /* Otherwise, add the geographies that have all countries associated to them */
         else {
            Vector geoVec     = localeRec.getGeoListContainingAll() ;
            Vector countryVec = localeRec.getSingleCountryList() ;
            int    i          = 0 ;

            /* Save the geographies */
            if ( ( geoVec != null ) && ( geoVec.size() > 0 ) ) {
               int length = geoVec.size() ;

               while ( i < length ) {

                  if ( updateOnly )
                     docSqlString.append ( "DELETE FROM QUEST.DOCGEO WHERE GEOIND = "+((TypeGeoRec)geoVec.elementAt(i)).getInd()+" AND DOCIND = "+docInd+QITSqlRunner.END_DELIM ) ;

                  sqlStmt = "INSERT INTO QUEST.DOCGEO "+
                            "(DOCIND, GEOIND, DBUSER, CHANGEDTIME )"+
                            "VALUES "+
                            "( "+docInd+", "+
                            "  "+((TypeGeoRec)geoVec.elementAt(i)).getInd()+", "+
                            "  '"+UserSystem.getUserId()+"', "+
                            "  CURRENT TIMESTAMP )" ;
                  docSqlString.append (sqlStmt+QITSqlRunner.END_DELIM) ;
                  i++ ;
               }
            }

            /* Now, save the countries */
            if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
               int length = countryVec.size() ;

               i = 0 ;
               while ( i < length ) {
                  if ( updateOnly )
                     docSqlString.append ( "DELETE FROM QUEST.DOCCOUNTRY WHERE COUNTRYCODEIND = "+((TypeCountryCodeRec)countryVec.elementAt(i)).getInd()+" AND DOCIND = "+docInd+QITSqlRunner.END_DELIM ) ;

                  sqlStmt = "INSERT INTO QUEST.DOCCOUNTRY "+
                            "(DOCIND, COUNTRYCODEIND, DBUSER, CHANGEDTIME )"+
                            "VALUES "+
                            "( "+docInd+", "+
                            "  "+((TypeCountryCodeRec)countryVec.elementAt(i)).getInd()+", "+
                            "  '"+UserSystem.getUserId()+"', "+
                            "  CURRENT TIMESTAMP )" ;
                  docSqlString.append (sqlStmt+QITSqlRunner.END_DELIM) ;
                  i++ ;
               }
            } /* if countryVec  */
         } /* else, not worldwide  */
      } /* if localeRec  */
   }

} /* DocLocationFieldChanger */
