package com.ibm.nzna.projects.qit.doc.fieldchanger ;     
/*-----------------------------------------------------------*/
/*   FieldChangerWizard                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FieldChangerWizard                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.gui.popup.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import javax.swing.* ;
import java.util.* ;
import java.awt.* ;
import java.io.* ;
/**
 * The FieldChangerWizard will walk the user through the steps 
 * of changing a set of documents.
 *
 * Step 1  - Just a welcome message to the user
 * Step 2  - Asks the user for what Field to change
 * Step 2a - Only shows if field is a dynamic field and asks if they wish to overwrite data or add data
 * Step 3  - Ask the user for comments
 * Step 4  - Asks the user for the field data
 * Step 5  - Finished Msg.
 *
 *
 * @version 1.0, 1/21/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FieldChangerWizard                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class FieldChangerWizard extends JDialog implements DataLengths, AppConst, WizardListener, PropertyListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Wizard          wizardPanel          = null ;
   private boolean         rc                   = false ;
   private InfoStep        infoStep             = null ;
   private InfoStep        finishStep           = null ;
   private SelectFieldStep selectFieldStep      = new SelectFieldStep () ;
   private CommentStep     commentStep          = new CommentStep () ;
   private PopUpPanel      dataStep             = null ;
   private Vector          docList              = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  all of our controls. It will not show the dialog.
    *  Please use the 'getResult' method to show and process the dialog
    *
    *  @param parent The parent window of this wizard
    *  @param docList The list of Document Drafts to process
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FieldChangerWizard ( Frame parent, Vector docList ) {
      super ( parent, "", true ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign local vars                                       */        
      /*                                                           */
      /*-----------------------------------------------------------*/
      this.docList = docList ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Wizard control                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardPanel = new Wizard ( ImageSystem.getImageIcon ( this, "skins"+
                                                            File.separator+
                                                            PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                                                            File.separator+
                                                            "setup_wizard.gif" ) ) ;                     




      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our steps                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      infoStep   = new InfoStep       ( "" ) ;                      
      finishStep = new InfoStep       ( "" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties/Add Listeners                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;
      wizardPanel.addWizardListener ( this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our window                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( new BorderLayout () ) ;
      getContentPane().add ( wizardPanel ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      setSize ( 495, 375 ) ;
      WinUtil.centerChildInParent ( this, MainWindow.getInstance() ) ;
   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will return the results
    *  from the user processing this wizard. This
    *  method will show the wizard and will not return unless Finish or Cancel were pressed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Finish was pressed, FALSE = Cancel was pressed
    *//*---------------------------------------------------------*/
   public boolean getResult () {

      wizardPanel.setCurrentStep ( infoStep ) ;

      setVisible ( true ) ;
      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* cancelPerformed
    */
   /**
    *    This method will be called if the user
    *  hit the 'Cancel' button on the Wizard
    *  
    *
    *  @param e The wizard event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cancelPerformed ( WizardEvent e ) {
      rc = false ;
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* finishPerformed
    */
   /**
    *    This method will be called when the
    *  user presses the finish button
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finishPerformed ( WizardEvent e) {
      DocFieldChangerPanel changerPanel = new DocFieldChangerPanel ( docList,
                                                                     selectFieldStep.getSelectedField(),
                                                                     dataStep.saveData(),
                                                                     commentStep.getComments(),
                                                                     selectFieldStep.getUpdateOnly() ) ;
      setVisible ( false ) ;
      WindowSystem.createPanel ( changerPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* nextPerformed
    */
   /**
    *    This method will be called when the
    *  next button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void nextPerformed ( WizardEvent e ) {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == infoStep )
         wizardPanel.setCurrentStep ( selectFieldStep ) ;
      else if ( step == selectFieldStep ) {
         if ( selectFieldStep.getSelectedField() != null )
            wizardPanel.setCurrentStep ( commentStep ) ;
      } else if ( step == commentStep ) {
         dataStep = getFieldStep ( selectFieldStep.getSelectedField() ) ;
         wizardPanel.setCurrentStep ( dataStep ) ;
      } else 
         wizardPanel.setCurrentStep ( finishStep ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      adjustButtons () ;
   }


   /*------------------------------------------------------------*/
   /* backPerformed
    */
   /**
    *    This method will be called when the
    *  back button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void backPerformed ( WizardEvent e ) {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == selectFieldStep )
         wizardPanel.setCurrentStep ( infoStep ) ;

      else if ( step == commentStep )
         wizardPanel.setCurrentStep ( selectFieldStep ) ;

      else if ( step == finishStep )
         wizardPanel.setCurrentStep ( ( dataStep != null ) ? (WizardStep)dataStep : commentStep ) ;

      else
         wizardPanel.setCurrentStep ( commentStep ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      adjustButtons ( ) ;
   }






   /*------------------------------------------------------------*/
   /* getFieldStep
    */
   /**
    *    This method will create and return a WizardStep
    *  based on the name of the field passed in
    *  
    *
    *  @param field The name of the field ( Document Type, Document Class etc..)
    *  @param None
    *  @param None
    *
    *  @return WizardStep to edit that field
    *//*---------------------------------------------------------*/
   private PopUpPanel getFieldStep ( String fieldName ) {
      PopUpPanel returnStep = null ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Document Type                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( fieldName.equals ( Str.getStr ( STR_DOCUMENT_TYPE ) ) ) {
         PopUpList listStep = new PopUpList ( null ) ;
         Vector    docTypes = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_DOC ).clone() ;

         docTypes.removeElement ( TypeList.getInstance().objectFromInd ( TypeList.DOCTYPE_SSD, TypeList.TYPE_DOC ) ) ;
         docTypes.removeElement ( TypeList.getInstance().objectFromInd ( TypeList.DOCTYPE_SERVICE_PUB, TypeList.TYPE_DOC ) ) ;

         listStep.setMultiSelect ( false ) ;
         listStep.setTitle ( Str.getStr ( STR_SELECT_DOCUMENT_TYPE_TITLE ) ) ;
         listStep.setDescript ( Str.getStr ( STR_DOCUMENT_TYPE ) ) ;
         listStep.refreshData ( docTypes ) ;

         returnStep = listStep ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Document Classification                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_DOC_CLASS ) ) ) {
         PopUpList listStep = new PopUpList ( null ) ;
         Vector    docClass = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_DOCCLASS ).clone() ;

         listStep.setMultiSelect ( false ) ;
         listStep.setTitle ( Str.getStr ( STR_SELECT_DOCUMENT_CLASS_TITLE ) ) ;
         listStep.setDescript ( Str.getStr ( STR_DOC_CLASS ) ) ;
         listStep.refreshData ( docClass ) ;

         returnStep = listStep ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Document Catagory                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_DOCUMENT_CATEGORY ) ) ) {
         PopUpList listStep   = new PopUpList ( null ) ;
         Vector    categories = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_DOCCAT ).clone() ;

         listStep.setTitle ( Str.getStr ( STR_SELECT_DOCUMENT_CATEGORY_TITLE ) ) ;
         listStep.setDescript ( Str.getStr ( STR_DOCUMENT_CATEGORY ) ) ;
         listStep.refreshData ( categories ) ;
         listStep.setMultiSelect ( true ) ;

         returnStep = listStep ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Document Status                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_STATUS ) ) ) {
         PopUpStatus statusStep = new PopUpStatus ( null ) ;
         StatusRec   statusRec  = new StatusRec () ;

         statusStep.setTypeStatus ( Status.STATUS_DOCUMENT ) ;
         statusStep.setTitle ( Str.getStr ( STR_SELECT_DOCUMENT_STATUS_TITLE ) ) ;
         statusStep.refreshData ( statusRec ) ;

         returnStep = statusStep ;
      }


      //VOY/*-----------------------------------------------------------*/
      //VOY/*                                                           */
      //VOY/*   Brand/Family                                            */
      //VOY/*                                                           */
      //VOY/*-----------------------------------------------------------*/
      //VOYelse if ( ( fieldName.equals ( Str.getStr ( STR_BRAND_FAMILY ) ) ) ||
      //VOY          ( fieldName.equals ( Str.getStr ( STR_MACHINETYPE_MODEL ) ) ) ) {
      //VOY   PopUpBrandFamilyMachineModel step = new PopUpBrandFamilyMachineModel ( null ) ;

      //VOY   step.refreshData ( new ProdDocRec() ) ;
      //VOY   step.setRequired ( true ) ;
      //VOY   returnStep = step ;
      //VOY}

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Location                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_LOCATION ) ) ) {
         PopUpLocale localeStep = new PopUpLocale ( null ) ;
         LocaleRec   localeRec  = new LocaleRec () ;

         localeStep.refreshData ( localeRec ) ;
         localeStep.setTitle ( Str.getStr ( STR_SELECT_LOCALE_TITLE ) ) ;

         returnStep = localeStep ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Publishing Tags                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_PUBLISHING_TAGS ) ) ) {
         PopUpPublication  pubStep  = new PopUpPublication ( null ) ;
         PublishingTagsRec tagsRec  = new PublishingTagsRec () ;

         tagsRec.webable = true ;

         pubStep.refreshData ( tagsRec ) ;
         pubStep.setTitle ( Str.getStr ( STR_SELECT_PUB_TAGS_TITLE ) ) ;

         returnStep = pubStep ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Metrics                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_METRICS ) ) ) {
         PopUpList listStep = new PopUpList ( null ) ;

         listStep.setTitle ( Str.getStr ( STR_SELECT_DOCUMENT_METRIC_TITLE ) ) ;
         listStep.setDescript ( Str.getStr ( STR_METRICS ) ) ;
         listStep.refreshData ( (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_METRIC ).clone() ) ;
         listStep.setMultiSelect ( false ) ;

         returnStep = listStep ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Body                                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_BODY ) ) ) {
         PopUpBodyReplace bodyStep   = new PopUpBodyReplace ( null ) ;
         BodyReplaceRec   replaceRec = new BodyReplaceRec () ;

         bodyStep.refreshData ( replaceRec ) ;
         bodyStep.setTitle ( Str.getStr ( STR_SEARCH_REPLACE_BODY_TITLE ) ) ;

         returnStep = bodyStep ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Title                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_TITLE ) ) ) {
         PopUpBodyReplace bodyStep   = new PopUpBodyReplace ( null ) ;
         BodyReplaceRec   replaceRec = new BodyReplaceRec () ;

         bodyStep.refreshData ( replaceRec ) ;
         bodyStep.setTitle ( Str.getStr ( STR_SEARCH_REPLACE_TITLE_TITLE ) ) ;

         returnStep = bodyStep ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   URL with no File attachments                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_URL_FOR_NO_FILE_ATTACH ) ) ) {
         PopUpEntryField entryStep = new PopUpEntryField ( null ) ;

         entryStep.setTitle ( Str.getStr ( STR_ENTER_URL_NO_ATTACH_TITLE ) ) ;
         entryStep.setMaximumLength ( LENGTH_FILEURL ) ;
         entryStep.setDescript ( Str.getStr ( STR_URL_FOR_NO_FILE_ATTACH ) ) ;

         returnStep = entryStep ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Publication Type                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_PUBLICATION ) ) ) {
         PopUpList listStep   = new PopUpList ( null ) ;
         Vector    pubs       = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_PUB ).clone() ;

         listStep.setTitle ( Str.getStr ( STR_SELECT_DOCUMENT_PUBLICATION_TITLE ) ) ;
         listStep.setDescript ( Str.getStr ( STR_PUBLICATION ) ) ;
         listStep.refreshData ( pubs ) ;
         listStep.setMultiSelect ( false ) ;

         returnStep = listStep ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Release Date                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_RELEASE_DATE ) ) ) {
         PopUpEntryField entryStep = new PopUpEntryField ( null ) ;

         entryStep.setTitle ( Str.getStr ( STR_RELEASE_DATE_TITLE ) ) ;
         entryStep.getEntryField().setDocument ( EntrySystem.getDateMask ( entryStep.getEntryField() ) ) ;
         entryStep.setDescript ( Str.getStr ( STR_RELEASE_DATE ) ) ;

         returnStep = entryStep ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Part Number                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( fieldName.equals ( Str.getStr ( STR_PART_NUMBER ) ) ) {
         PopUpEntryField entryStep = new PopUpEntryField ( null ) ;

         entryStep.setTitle ( Str.getStr ( STR_PART_NUMBER_CHANGE_TITLE ) ) ;
         entryStep.setMaximumLength ( LENGTH_DOCUMENT_PART_NUMBER ) ;
         entryStep.setDescript ( Str.getStr ( STR_PART_NUMBER ) ) ;

         returnStep = entryStep ;
      }

      if ( returnStep != null )
         returnStep.setRequired ( true ) ;


      return( returnStep ) ;
   }


   /*------------------------------------------------------------*/
   /* adjustButtons
    */
   /**
    *    This method will look at what step is
    *  currently showing and will decide which
    *  buttons to show in the wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void adjustButtons () {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == infoStep )
         wizardPanel.showButtons ( true, false, true, false ) ;
      else if ( step == finishStep )
         wizardPanel.showButtons ( false, true, false, true ) ;
      else
         wizardPanel.showButtons ( true, true, true, false ) ;
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    Called when a property we are listening
    *  for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE ) {
         setText() ;
      }
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on all
    *  buttons and InfoStep panels of the 
    *  wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      wizardPanel.setNextText   ( Str.getStr ( STR_NEXT ) ) ;
      wizardPanel.setBackText   ( Str.getStr ( STR_BACK ) ) ;
      wizardPanel.setFinishText ( Str.getStr ( STR_FINISH ) ) ;
      wizardPanel.setCancelText ( Str.getStr ( STR_CANCEL ) ) ;
      setTitle ( Str.getStr ( STR_FIELD_CHANGER_WIZARD ) ) ;
      infoStep.setMessage ( Str.getStr ( STR_FIELD_CHANGER_WIZARD_INFO ) ) ;                      
      finishStep.setMessage ( Str.getStr ( STR_FIELD_CHANGER_WIZARD_FINISH ) ) ;
   }


} /* FieldChangerWizard */
