package com.ibm.nzna.projects.qit.doc.fieldchanger ;
/*-----------------------------------------------------------*/
/*   SelectFieldStep                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectFieldStep                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import javax.swing.event.* ;
import javax.swing.* ;
import java.util.* ;   
import java.awt.* ;
import java.awt.event.* ;
/**
 * This step will allow the user to specify which field to change
 *
 *
 * @version 1.0, 1/21/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectFieldStep                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectFieldStep extends WizardStep implements ListSelectionListener, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_HELP          = null ;
   private JList        fieldList        = null ;
   private JScrollPane  fieldScroll      = null ;
   private JRadioButton updateOnly       = null ;
   private JRadioButton updateAndReplace = null ;
   private String       selectedField    = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our step
    *  
    *
    *  @param message The Message to display to the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectFieldStep (  ) {
      ButtonGroup buttonGroup = new ButtonGroup () ;
      Vector      fields      = new Vector ( 10 ) ;


      fields.addElement ( Str.getStr ( STR_DOCUMENT_TYPE ) ) ;
      fields.addElement ( Str.getStr ( STR_DOC_CLASS ) ) ;
      fields.addElement ( Str.getStr ( STR_DOCUMENT_CATEGORY ) ) ;
      fields.addElement ( Str.getStr ( STR_STATUS ) ) ;
      fields.addElement ( Str.getStr ( STR_BRAND_FAMILY ) ) ;
      fields.addElement ( Str.getStr ( STR_MACHINETYPE_MODEL ) ) ;
      fields.addElement ( Str.getStr ( STR_LOCATION ) ) ;
      fields.addElement ( Str.getStr ( STR_PUBLISHING_TAGS ) ) ;
      fields.addElement ( Str.getStr ( STR_METRICS ) ) ;
      fields.addElement ( Str.getStr ( STR_BODY ) ) ;
      fields.addElement ( Str.getStr ( STR_URL_FOR_NO_FILE_ATTACH ) ) ;
      fields.addElement ( Str.getStr ( STR_PUBLICATION ) ) ;
      fields.addElement ( Str.getStr ( STR_RELEASE_DATE ) ) ;
      fields.addElement ( Str.getStr ( STR_PART_NUMBER ) ) ;
      fields.addElement ( Str.getStr ( STR_TITLE ) ) ;

      new QuickSort (fields) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP          = new JLabel ( ) ;
      fieldList        = new JList ( fields ) ;
      fieldScroll      = new JScrollPane ( fieldList ) ;
      updateOnly       = new JRadioButton ( Str.getStr ( STR_UPDATE_ONLY ) ) ;
      updateAndReplace = new JRadioButton ( Str.getStr ( STR_UPDATE_AND_REPLACE ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;
      buttonGroup.add ( updateOnly ) ;
      buttonGroup.add ( updateAndReplace ) ;
      setText() ;                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */    
      /*                                                           */
      /*-----------------------------------------------------------*/
      fieldList.addListSelectionListener ( this ) ;
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls to the panel                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_HELP ) ;
      add ( fieldScroll ) ;                                       
      add ( updateOnly ) ;
      add ( updateAndReplace ) ;

      enableRadioControls ( false ) ;
   }





   /*------------------------------------------------------------*/
   /* enableRadioControls
    */
   /**
    *    This method will either enable or disable
    *  the radio button controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void enableRadioControls ( boolean enable ) {
      updateOnly.setEnabled ( enable ) ;
      updateAndReplace.setEnabled ( enable ) ;
      if ( !enable )
         updateAndReplace.setSelected ( true ) ;
   }



   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will position and size all
    *  of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = 5 ;
      int       y         = 5 ;
      Dimension prefSize  = null ;

      super.doLayout() ;

      st_HELP.setBounds ( x, y, size.width-10, 100 ) ;
      prefSize = st_HELP.getPreferredSize() ;
      st_HELP.setBounds ( x, y, size.width-10, prefSize.height ) ;
      y+=prefSize.height+5 ;

      fieldScroll.setBounds ( x, y, size.width-10, size.height-y-15-(rowHeight*2) ) ;

      updateOnly.setBounds ( x, size.height-10-(rowHeight*2), size.width-10, rowHeight ) ;
      updateAndReplace.setBounds ( x, size.height-10-rowHeight, size.width-10, rowHeight ) ;
   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save the information
    *  and return TRUE if a language was
    *  selected.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      boolean rc = false ;


      selectedField = (String)fieldList.getSelectedValue() ;

      rc = selectedField != null ;

      return( rc ) ;
   }

   public void close () {}


   public String getTitle () {
      return( Str.getStr ( STR_SELECT_FIELD_FOR_DOCUMENT_CHANGE_TITLE ) ) ;
   }


   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of the panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      st_HELP.setText ( Str.getStr ( STR_SELECT_FIELD_FOR_DOCUMENT_CHANGE ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getSelectedField
    */
   /**
    *    This method will return the selected field
    *  that the user chose. NULL if nothing was selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected String getSelectedField () {
      return( selectedField ) ;
   }





   /*------------------------------------------------------------*/
   /* getUpdateOnly
    */
   /**
    *    This method will return TRUE or FALSE if
    *  the user wishes to Update Only.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected boolean getUpdateOnly () {
      return( updateOnly.isSelected() ) ;
   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when a value is
    *  changed in the list of document fields
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      String selectedField = (String)fieldList.getSelectedValue() ;

      if ( ( selectedField.equals ( Str.getStr ( STR_DOCUMENT_TYPE ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_DOC_CLASS ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_STATUS ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_PUBLISHING_TAGS ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_METRICS ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_BODY ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_TITLE ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_URL_FOR_NO_FILE_ATTACH ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_PUBLICATION ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_RELEASE_DATE ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_PART_NUMBER ) ) ) )
         enableRadioControls ( false ) ;
      else
         enableRadioControls ( true ) ;

   }
} /* SelectFieldStep */
