package com.ibm.nzna.projects.qit.doc.gui ;   
/*-----------------------------------------------------------*/
/*   DocumentDlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentDlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.docedit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
/**
 * This simple dialog simply shows the user a list of documents. The document
 * will be modal if the parent passed to the constructor is a JDialog. If the
 * parent is a JFrame then the dialog will not be modal and will have the 'Edit'
 * button.
 *
 *
 * @version 1.0, 8/6/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentDlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentDlg extends JDialog implements ActionListener, WindowListener, AppConst, DocConst {

   /**
     *  This is the actual list that will show the documents
     **/
   private DocumentTable documentTable = null ;

   /**
     *  This is the 'Ok' button
     **/
   private DButton pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     *  This is the Vector of documents we will be showing
     **/
   private Vector documents = null ;

   /**
     *  This button bar holds the Ok and Edit buttons
     **/
   ButtonPanel buttonBar = new ButtonPanel () ;     

   /**
     *  If our parent is an AppDefaultWin, this is that instance
     **/
   private AppDefaultWin parentWin = null ;

   /**
     *  This is the edit button. This button only shows if this
     * window is non-modal
     **/
   private DButton pb_EDIT = null ;


   /**
     *  This is the default constructor
     **/
   public DocumentDlg ( Frame parent, Vector documents ) {
      super ( parent, Str.getStr ( STR_DOCUMENTS ), true ) ;

      this.documents = documents ;
      initialize() ;
      setVisible ( true ) ;
   }

   /**
     *  This is the constructor that makes a JDialog our parent
     **/
   public DocumentDlg ( JDialog parent, Vector documents ) {
      super ( parent, Str.getStr ( STR_DOCUMENTS ), true ) ;

      this.documents = documents ;
      initialize() ;
      setVisible ( true ) ;
   }

   /**
     *  This is the constructor that makes a JFrame our parent and is
     * not modal
     **/
   public DocumentDlg ( JFrame parent, Vector documents ) {
      super ( parent, Str.getStr ( STR_DOCUMENTS ), false ) ;

      if ( parent instanceof AppDefaultWin )
         parentWin = (AppDefaultWin)parent ;

      this.documents = documents ;
      initialize() ;
      initializeEdit() ;
      setVisible ( true ) ;
   }


   /**
     *  This method will initialize our GUI and set our size/position
     **/
   private void initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      documentTable = new DocumentTable ( documents ) ;              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      addWindowListener ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      buttonBar.add ( pb_OK ) ;
      getContentPane().add ( documentTable ) ;
      getContentPane().add ( buttonBar, BorderLayout.SOUTH ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our size                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSizeFromPrevious() ;
      GUISystem.setPropertiesOnPanel ( getContentPane() );

   }


   /**
     *  This method will set our size from our previous size
     **/
   private void setSizeFromPrevious () {
      Rectangle oldSize = (Rectangle)PropertySystem.get ( PROPERTY_DOCUMENT_DLG_POSITION ) ;

      if ( oldSize == null ) {
         setSize ( 400, 300 ) ;
         WinUtil.centerWindow ( this ) ;

         oldSize = new Rectangle ( getLocation().x, getLocation().y, 400, 300 ) ;
      } else
         setBounds ( oldSize ) ;
   }


   /**
     * This method is called when the user hits the 'OK' button
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_OK ) {
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT ) {
         DocumentRow docRow = (DocumentRow) documentTable.getSelectedItem() ;

         if ( docRow != null ) {
            if ( parentWin != null ) {
               parentWin.setStatus ( Str.getStr ( STR_READING )+" "+docRow.getTitle() ) ;

               EditDocument.editDocument ( docRow ) ;
               parentWin.setStatus ( null ) ;
            }
         } else
            GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_ITEM ) ;
      }

   }


   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e ) {
   }
   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowOpened ( WindowEvent e ) {
   }
   public void windowClosing ( WindowEvent e ) {
      PropertySystem.put ( PROPERTY_DOCUMENT_DLG_POSITION, getBounds() ) ;
   }
   public void windowClosed ( WindowEvent e ) {
   }

   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;
      documentTable.setBounds ( 5, 5, size.width-15, size.height-68 ) ;

      buttonBar.setBounds ( 5, size.height-56, size.width-15, 25 ) ;
   }

   /**
     *  this method will add the edit button to our window
     **/
   public void initializeEdit() {
      pb_EDIT = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT.addActionListener ( this ) ;
      buttonBar.add ( pb_EDIT ) ;
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
   }
} /* DocumentDlg */
