package com.ibm.nzna.projects.qit.doc.gui ;
/*-----------------------------------------------------------*/
/*   DocumentFileRenderer                                    */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentFileRenderer                */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This renderer will paint a DocumentFile object in a JList
 *
 *
 * @version 1.0, 5/5/1999 
 * @author David Lentz
 */
import java.awt.* ;
import java.awt.image.* ;
import javax.swing.* ;       
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentFileRenderer                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentFileRenderer extends Component implements ListCellRenderer {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Color    selBackground = (Color)javax.swing.UIManager.getDefaults().get ( "List.selectionBackground" ) ;
   private static Color    selForeground = (Color)javax.swing.UIManager.getDefaults().get ( "List.selectionForeground" ) ;
   private static Color    oddColor      = new Color ( 240, 240, 240 ) ;
   private static Image    focusImage    = null ;
   private static FontUtil fontUtil      = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean      selected        = false ;
   private boolean      hasFocus        = false ;
   private DocumentFile file            = null ;
   private Dimension    prefSize        = new Dimension ( 450, 34 ) ;


   /*------------------------------------------------------------*/
   /* Constructor(1)
    */
   /**
    *    This constructor will create a default
    *  renderer
    *  
    *
    *  @param multiList The MultiList control that this renderer will draw in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentFileRenderer ( ) {
   }



   /*------------------------------------------------------------*/
   /* getListCellRendererComponent
    */
   /**
    *    Simply returns an instance of this
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Component getListCellRendererComponent ( JList list, Object data, int index, boolean isSelected, boolean cellHasFocus ) {

      if ( data instanceof DocumentFile )
         this.file = (DocumentFile)data ;

      if ( ((index/2)*2) != index )
         setBackground ( oddColor ) ;
      else
         setBackground ( Color.white ) ;

      this.selected = isSelected ;
      this.hasFocus = cellHasFocus ;

      return(this);
   }





   /*------------------------------------------------------------*/
   /* paint
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param g The graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paint ( Graphics g ) {
      Rectangle clip = (Rectangle) g.getClip() ;

      if ( clip == null )
         clip = new Rectangle ( 0, 0, prefSize.width, prefSize.height ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Paint our background                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !selected ) {
         g.setColor ( getBackground() ) ;
         g.fillRect ( clip.x, clip.y, clip.width, clip.height ) ;

      } else {
         g.setColor ( selBackground ) ;
         g.fillRect ( clip.x, clip.y, clip.width, clip.height ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Paint our DocumentFile                                  */             
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( file != null ) {
         ImageIcon icon      = ImageSystem.getImageIcon ( this, ImageSystem.FILE_DEFAULT ) ;
         String    ext       = file.getFilename() ;
         int       clipWidth = clip.width-clip.x ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Grab the extension                                      */         
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ext.indexOf ( "." ) != -1 )
            ext = ext.substring ( ext.indexOf ( "." )+1 ).toUpperCase() ;
         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get our icon                                            */   
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( ext.equals ( "EXE" ) ) ||
              ( ext.equals ( "BAT" ) ) ||
              ( ext.equals ( "CMD" ) ) )
            icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_EXEC ) ;
         else if ( ext.equals ( "PDF" ) )
            icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_PDF ) ;
         else if ( ( ext.equals ( "TXT" ) ) ||
                   ( ext.equals ( "1ST" ) ) ||
                   ( ext.equals ( "ME" ) ) ||
                   ( ext.equals ( "DAT" ) ) )
            icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_TEXT ) ;
         else if ( ( ext.equals ( "AVI" ) ) ||
                   ( ext.equals ( "WMF" ) ) ||
                   ( ext.equals ( "GIF" ) ) ||
                   ( ext.equals ( "JPG" ) ) ||
                   ( ext.equals ( "MPEG" ) ) )
            icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_VIDEO ) ;
         else if ( ( ext.equals ( "WAV" ) ) ||
                   ( ext.equals ( "MP3" ) ) ||
                   ( ext.equals ( "MP4" ) ) )
            icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_WAVE ) ;

                
         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our foreground color                                */               
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !selected )
            g.setColor ( getForeground() ) ;
         else
            g.setColor ( selForeground ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paint our Icon                                          */     
         /*                                                           */
         /*-----------------------------------------------------------*/
         g.drawImage ( icon.getImage(), 5, 5, this ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paint the filename and file size                        */                       
         /*                                                           */
         /*-----------------------------------------------------------*/
         g.drawString ( file.getFilename()+" ("+file.getSize()+" bytes)", 40, 12 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paint the Description                                   */            
         /*                                                           */
         /*-----------------------------------------------------------*/
         g.drawString ( file.getDescription(), 40, 28 ) ;
         

         

         
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paint the focus rectangle if we have one                */                               
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( hasFocus ) {
            if ( focusImage != null )
               focusImage.flush() ;
            createFocusImage() ;

            if ( focusImage != null )
               g.drawImage ( focusImage, 0, 0, this ) ;
         } /* hasFocus  */
      } /* file  */
   } /* paint  */





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred size
    *  of this row.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }



   /*------------------------------------------------------------*/
   /* createFocusImage
    */
   /**
    *    This method will re-create the focus image
    *  of dotted yellow lines around the row
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createFocusImage ( ) {
      if ( prefSize.height >= 0 ) {
         int               i         = 0 ;
         MemoryImageSource imgSource = null ;
         int[]             imgBytes  = new int [ prefSize.width*prefSize.height ] ;
         int               black     = 0xff000000 ;
         int               yellow    = 0xffffff9c ;
         int               color     = yellow ;

         /* Draw the horizontal lines */
         while ( i < prefSize.width ) {
            imgBytes[i]=color ;
            imgBytes[i+(prefSize.width*(prefSize.height-1))]=color ;
            i+=2 ;
         }

         i=0 ;
         while ( i < (prefSize.height*(prefSize.width-1)) ) {
            imgBytes[i]=color ;
            imgBytes[i+prefSize.width-1]=color ;
            i+=prefSize.width*2 ;
         }

         imgSource = new MemoryImageSource ( prefSize.width, prefSize.height, imgBytes, 0, prefSize.width ) ;

         focusImage = this.createImage ( imgSource ) ;
      }

   }





} /* DocumentFileRenderer */
