package com.ibm.nzna.projects.qit.doc.gui ;
/*-----------------------------------------------------------*/
/*   SaveTemplateDlg                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SaveTemplateDlg                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This dialog will only be displayed if a user tries to save a template twice. It will 
 * give the user a choice of the following actions:
 *
 * 1. Save this as a new Template
 * 2. Update the current Template
 * 3. Cancel
 *
 *
 * @version 1.0, 11/24/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SaveTemplateDlg                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class SaveTemplateDlg extends JDialog implements AppConst, ActionListener, Runnable {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea  st_HELP           = new JTextArea ( Str.getStr ( STR_UPDATE_OR_NEW_TEMPLATE ) ) ;
   private DButton    pb_NEWTEMPLATE    = new DButton   ( Str.getStr ( STR_NEW_TEMPLATE ) ) ;
   private DButton    pb_UPDATETEMPLATE = new DButton   ( Str.getStr ( STR_UPDATE_TEMPLATE ) ) ;
   private DButton    pb_CANCEL         = new DButton   ( Str.getStr ( STR_CANCEL ) ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int userAction     = PersistentRec.REC_NOCHANGE ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create the SelectTemplateDlg. To get
    *  the action that a user chooses from this window, call
    *  the getResult method.
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public SaveTemplateDlg ( JDialog parent ) {
      super ( parent, Str.getStr ( STR_TEMPLATE ), true ) ;
      setResizable ( false ) ;
      setSize ( 300, 200 ) ;
      WinUtil.centerChildInParent ( this ,parent ) ;
   }


   /*-----------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the dialog and not
    *  return until the user has chosen an action. The
    *  action returned is of type PersistentRec.REC_INSERT ( New Template ), 
    *  PersistentRec.REC_UPDATE ( Update Old Template ) and PersistentRec.REC_NOCHANGE ( User Cancelled  ).
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public int getResult() {
      new Thread ( this ).start() ;
      setVisible ( true ) ;
      return( userAction ) ;
   }


   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our
    *  controls to the dialog
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void createControls () {
      Container contentPane = getContentPane() ;

      contentPane.setLayout ( null ) ;
      WinUtil.setJTextAreaAsLabel ( st_HELP ) ;
      st_HELP.setFont ( FontSystem.defaultFont ) ;
      contentPane.add ( st_HELP ) ;
      contentPane.add ( pb_NEWTEMPLATE ) ;
      contentPane.add ( pb_UPDATETEMPLATE ) ;
      contentPane.add ( pb_CANCEL ) ;

      pb_NEWTEMPLATE.addActionListener ( this ) ;
      pb_UPDATETEMPLATE.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
   } /* createControls */



   /*-----------------------------------------------------------*/
   /* layoutControls
    */
   /**
    *    This method will be called when we are supposed 
    *  to layout and size all of our controls
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = 18 ;
      int       y         = 5 ;

      super.doLayout() ;

      if ( st_HELP != null )
         st_HELP.setBounds ( 5, 5, size.width-10, rowHeight*4 ) ;
      y+=(rowHeight*4)+5 ;

      if ( pb_NEWTEMPLATE != null )
         pb_NEWTEMPLATE.setBounds ( 20, y, size.width-40, 25 ) ;
      y+=30 ;

      if ( pb_UPDATETEMPLATE != null )
         pb_UPDATETEMPLATE.setBounds ( 20, y, size.width-40, 25 ) ;
      y+=30 ;

      if ( pb_CANCEL != null )
         pb_CANCEL.setBounds ( 20, y, size.width-40, 25 ) ;
   }



   /*-----------------------------------------------------------*/
   /* buttonHit
    */
   /**
    *    This method will respond to buttons being pressed
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_NEWTEMPLATE )
         userAction = PersistentRec.REC_INSERT ;
      else if ( e.getSource() == pb_UPDATETEMPLATE )
         userAction = PersistentRec.REC_UPDATE ;
      else if ( e.getSource() == pb_CANCEL )
         userAction = PersistentRec.REC_NOCHANGE ;

      dispose() ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run our initializing in a 
    *  thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      createControls () ;
      getContentPane().setVisible ( false ) ;
      getContentPane().invalidate() ;
      getContentPane().setVisible ( true ) ;
   }


} /* SaveTemplateDlg */

