package com.ibm.nzna.projects.qit.doc.gui ;
/*-----------------------------------------------------------*/
/*   SelectDocDlg                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectDocDlg                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import javax.swing.border.* ;
import javax.swing.event.* ;
import java.awt.datatransfer.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.docedit.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.projects.qit.doc.docmaint.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
/**
 * This window allows the user to perform searches against the database for docs, and 
 * will put the results in a list. The user is then allowed to make either
 * multiple selections, or a single selection.<br><br>
 *
 * PLEASE NOTE! Do not use the instance of this class returned from the Constructor, instead
 * call the static method 'getInstance()'
 *
 *
 * @version 1.0, 8/15/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectDocDlg                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public final class SelectDocDlg extends JDialog implements 
ActionListener,
DataLengths,
DocViewListener,
Runnable,
AppConst,
DocConst,
WindowListener,
ClipboardOwner,
ListSelectionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int SELECTTYPE_SINGLE       = 0 ;
   public static final int SELECTTYPE_MULTIPLE     = 1 ;
   public static final int DLG_SELECTDOC           = 2 ;

   private static final String THREAD_SEARCH       = "Search" ;
   private static final String THREAD_INITIALIZE   = "Init" ;

   private static SelectDocDlg instance = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private BookmarkTree   bookmarkTree      = null ;
   private JScrollPane    bookmarkScroll    = null ;
   private ButtonPanel    buttonPanel       = null ;
   private JPanel         brd_FIND          = null ;
   private JPanel         brd_DOCS          = null ;
   private JPanel         brd_SEARCHING     = null ;
   private UserEntryPanel pnl_USERENTRY     = null ;
   private DButton        pb_FIND           = null ;
   private DButton        pb_SELECT         = null ;
   private DButton        pb_CANCEL         = null ;
   private DButton        pb_EDIT           = null ;
   private DButton        pb_CLEAR          = null ;
   private DButton        pb_BOOKMARKS      = null ;
   private JLabel         st_SEARCHBY       = null ;
   private JLabel         st_IS             = null ;
   private JLabel         st_ANDTYPEIS      = null ;
   private JLabel         st_SEARCHING      = null ;
   private JComboBox      cb_DOCTYPE        = null ;
   private JComboBox      cb_SEARCHTYPE     = null ;
   private JComboBox      cb_BRANDS         = null ;
   private JComboBox      cb_FAMILY         = null ;
   private JComboBox      cb_CATEGORY       = null ;
   private JTextField     ef_SEARCHARG      = null ;
   private DocumentTable  cnr_DOCS          = null ;
   private int            searchType        = DocViewRec.VIEW_BY_NOTESID ;
   private PrevNextPanel  prevNextPanel     = null ;
   private JCheckBox      ck_REQUEST        = null ;
   private DButton        pb_COPY           = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int                  selectType       = SELECTTYPE_SINGLE ;
   private Vector               retVec           = null ;
   private TypeCategoryRec         curBrandRec      = null ;
   private TypeGroupRec         curGroupRec      = null ;
   private String               title            = null ;
   private String               body             = null ;
   private String               user             = null ;
   private String               notesId          = null ;
   private String               source           = null ;
   private String               comment          = null ;
   private int                  docInd           = 0 ;
   private TypeDocRec           typeDocRec       = null ;
   private TypeDocCatRec        curCategoryRec   = null ;
   private DocViewRec           viewRec          = null ;
   private boolean              docTypeRequired  = true ;
   private boolean              showRequests     = true ;
   private boolean              allowInput       = true ;
   private SelectDocListener    listener         = null ;
   private boolean              showingBookmarks = false ;
   private String               queryText        = null ;
   private String               lastQuery        = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a blank SelectDocDlg with a select type
    *  of single.
    *  
    *  
    *
    *  @param listener The listener to notify when we are closed
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectDocDlg ( ) {
      super ( new Frame(), Str.getStr ( STR_SELECT_DOCUMENT ), false ) ;
      initialize() ;
   } /* Constructor */





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a SelectDocRec with 
    *  a certain type of selection.
    *  
    *
    *  @param listener The listener to notify when we are closed
    *  @param selectType The Type of selection ( SELECTTYPE_MULTIPLE, SELECTTYPE_SINGLE )
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectDocDlg ( int selectType ) {
      super ( new Frame() , Str.getStr ( STR_SELECT_DOCUMENT ), false ) ;
      this.selectType = selectType ;
      initialize() ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a SelectDocRec with 
    *  a certain type of selection and show of doc requests.
    *  
    *
    *  @param listener The listener to notify when we are closed
    *  @param selectType The Type of selection ( SELECTTYPE_MULTIPLE, SELECTTYPE_SINGLE )
    *  @param showRequests TRUE = allow user to see only doc requests, false if otherwise
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectDocDlg ( int selectType, boolean showRequests ) {
      super ( new Frame(), Str.getStr ( STR_SELECT_DOCUMENT ), false ) ;
      this.selectType   = selectType ;
      this.showRequests = showRequests ;
      initialize() ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a SelectDocRec with 
    *  a certain type of selection and show of doc requests.
    *  
    *
    *  @param showRequests TRUE = allow user to see only doc requests, false if otherwise
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectDocDlg ( boolean showRequests ) {
      super ( new Frame(), Str.getStr ( STR_SELECT_DOCUMENT ), false ) ;
      this.showRequests = showRequests ;
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* initializedow
    */
   /**
    *    This method will initialize our variables for
    *  our window before it is shown
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true
    *//*---------------------------------------------------------*/
   public boolean initialize () {                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this insance is null, create a new window            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( instance == null ) {
         Vector listeners = new Vector ( 1 ) ;         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our Properties                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         listeners.addElement ( this ) ;
         setResizable ( false ) ;
         setSize ( 565, 390 ) ;
         setVisible ( true ) ;
         instance = this ;
         viewRec = new DocViewRec ( DocViewRec.VIEW_EXACT_SQL ) ;
         viewRec.setListeners ( listeners ) ;

         new Thread ( this, THREAD_INITIALIZE ).start() ;
         WinUtil.centerChildInParent ( this, GUISystem.getParentDefWin ( this ) ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, show the old window                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         instance.setVisible ( true ) ;
         dispose() ;
      }

      return( true ) ;
   }



   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {
      int x         = 0 ;
      int y         = 0 ;
      int rowHeight = 18 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our panels                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      brd_FIND      = new JPanel() ;
      brd_DOCS      = new JPanel() ;
      brd_SEARCHING = new JPanel() ;
      brd_FIND.setBorder ( new TitledBorder ( Str.getStr ( STR_FIND ) ) ) ;
      brd_DOCS.setBorder ( new TitledBorder ( Str.getStr ( STR_DOCUMENTS ) ) ) ;
      brd_SEARCHING.setBorder ( GUISystem.loweredBorder ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      bookmarkTree   = new BookmarkTree () ;
      bookmarkScroll = new JScrollPane ( bookmarkTree ) ;
      buttonPanel    = new ButtonPanel () ;
      st_SEARCHBY    = new JLabel ( Str.getStr ( STR_SEARCH_WHERE ) ) ;
      st_ANDTYPEIS   = new JLabel ( Str.getStr ( STR_AND_TYPE_IS ) ) ;
      st_IS          = new JLabel ( Str.getStr ( STR_IS ) ) ;
      st_SEARCHING   = new JLabel ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
      cnr_DOCS       = new DocumentTable() ;
      pnl_USERENTRY  = new UserEntryPanel() ;
      pb_FIND        = new DButton ( Str.getStr ( STR_FIND ) ) ;
      pb_COPY        = new DButton ( Str.getStr ( STR_COPY ) ) ;
      pb_SELECT      = new DButton ( Str.getStr ( STR_SELECT ) ) ;
      pb_CANCEL      = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      pb_EDIT        = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_CLEAR       = new DButton ( Str.getStr ( STR_CLEAR ) ) ;
      pb_BOOKMARKS   = new DButton ( Str.getStr ( STR_BOOKMARKS ) ) ;
      cb_SEARCHTYPE  = new JComboBox ( );
      cb_DOCTYPE     = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_DOC ) );
      cb_BRANDS      = new JComboBox ( Brands.getBrands() );
      cb_FAMILY      = new JComboBox ( );
      cb_CATEGORY    = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_DOCCAT ) );
      ef_SEARCHARG   = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN , LENGTH_TITLE ), "", 0 ) ;
      ck_REQUEST     = new JCheckBox ( Str.getStr ( STR_SHOW_ONLY_DRAFTS ) ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Default Doc Type                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      typeDocRec = (TypeDocRec)TypeList.getInstance().getTypeList(TypeList.TYPE_DOC).elementAt(0) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SEARCHING.setHorizontalAlignment ( st_SEARCHING.CENTER ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Prev/Next Panels                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      prevNextPanel = new PrevNextPanel ( this ) ;                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our contorls to the dialog                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      getContentPane().add ( brd_FIND ) ;
      getContentPane().add ( brd_DOCS ) ;
      getContentPane().add ( brd_SEARCHING ) ;
      getContentPane().add ( buttonPanel ) ;
      buttonPanel.add ( pb_FIND ) ;
      buttonPanel.add ( pb_EDIT ) ;
      buttonPanel.add ( pb_SELECT ) ;
      buttonPanel.add ( pb_CLEAR ) ;
      buttonPanel.add ( pb_COPY ) ;
      buttonPanel.add ( pb_BOOKMARKS ) ;
      buttonPanel.add ( pb_CANCEL ) ;                         



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to the find panel                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      brd_FIND.setLayout ( null ) ;
      brd_FIND.add ( st_SEARCHBY ) ;
      brd_FIND.add ( st_ANDTYPEIS ) ;
      brd_FIND.add ( st_IS ) ;
      brd_FIND.add ( cb_SEARCHTYPE ) ;
      brd_FIND.add ( ef_SEARCHARG ) ;
      brd_FIND.add ( cb_DOCTYPE ) ;
      brd_FIND.add ( pnl_USERENTRY ) ;
      brd_FIND.add ( cb_BRANDS ) ;
      brd_FIND.add ( cb_CATEGORY ) ;
      brd_FIND.add ( ck_REQUEST ) ;
      brd_FIND.add ( cb_FAMILY ) ;                              


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to the searching panel                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      brd_SEARCHING.setLayout ( null ) ;
      brd_SEARCHING.add ( st_SEARCHING ) ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our doc panel                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      brd_DOCS.setLayout ( new BorderLayout() ) ;
      brd_DOCS.add ( cnr_DOCS, BorderLayout.CENTER ) ;
      cnr_DOCS.addPanel ( prevNextPanel, BorderLayout.SOUTH ) ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Hide some controls for now                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_USERENTRY.setVisible ( false ) ;
      cb_BRANDS.setVisible ( false ) ;
      cb_FAMILY.setVisible ( false ) ;
      brd_SEARCHING.setVisible ( false ) ;
      cb_CATEGORY.setVisible ( false ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_SEARCHTYPE.addActionListener ( this ) ;
      cb_BRANDS.addActionListener     ( this ) ;
      cb_FAMILY.addActionListener     ( this ) ;
      cb_CATEGORY.addActionListener   ( this ) ;
      cb_DOCTYPE.addActionListener    ( this ) ;
      pb_FIND.addActionListener       ( this ) ;
      pb_EDIT.addActionListener       ( this ) ;
      pb_CANCEL.addActionListener     ( this ) ;
      pb_SELECT.addActionListener     ( this ) ;
      cnr_DOCS.addActionListener      ( this ) ;
      ef_SEARCHARG.addActionListener  ( this ) ;
      pb_CLEAR.addActionListener      ( this ) ;                           
      pb_COPY.addActionListener       ( this ) ;
      pb_BOOKMARKS.addActionListener  ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the size/position of controls                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      x = 5 ;
      y = 15 ;
      st_SEARCHBY.setBounds ( x, y, 175, rowHeight ) ;
      y+=rowHeight+2 ;

      cb_SEARCHTYPE.setBounds ( x, y, 175, rowHeight+2 ) ;
      st_IS.setBounds ( x+180, y, 50, rowHeight ) ;
      ef_SEARCHARG.setBounds ( x+235, y, 175, rowHeight ) ;
      pnl_USERENTRY.setBounds ( x+235, y, 175, rowHeight ) ;
      cb_BRANDS.setBounds ( x+235, y, 175, rowHeight+2 ) ;
      y+=rowHeight+2 ;

      st_ANDTYPEIS.setBounds ( x, y, 175, rowHeight ) ;
      y+=rowHeight+2 ;

      cb_DOCTYPE.setBounds  ( x,     y, 175, rowHeight+2 ) ;
      cb_FAMILY.setBounds   ( x+235, y, 175, rowHeight+2 ) ;
      cb_CATEGORY.setBounds ( x+235, y, 175, rowHeight+2 ) ;
      y+=rowHeight+4 ;

      ck_REQUEST.setBounds ( x, y, 200, rowHeight ) ;

      /* If we are not to allow the user to select requests,
         hide the check box */
      ck_REQUEST.setVisible ( showRequests ) ;


      GUISystem.setPropertiesOnPanel ( getContentPane()  ) ;
   } /* createControls */



   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will layout and resize all of
    *  our controls
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void doLayout () {
      try {
         Dimension size = getSize() ;

         super.doLayout() ;

         brd_FIND.setBounds ( 5, 5, size.width-15, 140 ) ;
         brd_SEARCHING.setBounds ( 5, 5, size.width-15, 140 ) ;
         brd_DOCS.setBounds ( 5, 145, size.width-15, 390-145-60 ) ;

         buttonPanel.setBounds ( 5, size.height-55, size.width-15, 25 ) ;

         size=brd_SEARCHING.getSize() ;
         st_SEARCHING.setBounds ( 0, 0, size.width, size.height ) ;
      } catch ( Exception e ) {
      }
   } /* layoutControls */



   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh all of our lists
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void refreshList () {

      /* Add our items to the search choice */
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_BY_NOTESID       , Str.getStr ( STR_LOTUS_NOTES_ID ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_QUERY_TEXT      ,  Str.getStr ( STR_QUERY_TEXT ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_BY_TITLE         , Str.getStr ( STR_TITLE ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_BY_BRAND         , Str.getStr ( STR_BRAND ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_BY_BRANDGROUPIND , Str.getStr ( STR_FAMILY ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_BY_AUTHOR        , Str.getStr ( STR_AUTHOR ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_DOCBODY         ,  Str.getStr ( STR_BODY ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_BY_QUESTIND      , Str.getStr ( STR_INTERNAL_NUM ) )  ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_BY_BRANDCATEGORY , Str.getStr ( STR_BRAND_CATEGORY ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_SOURCE ,           Str.getStr ( STR_SOURCE ) ) ) ;
      cb_SEARCHTYPE.addItem ( new GlassItem ( DocViewRec.VIEW_DRAFT_BY_COMMENT,  Str.getStr ( STR_COMMENT ) ) ) ;


      cb_SEARCHTYPE.setSelectedIndex ( 0 ) ;
   } /* refreshList */


   /*-----------------------------------------------------------*/
   /* itemStateChanged
    */
   /**
    *    This method will be called when one of
    *  our choice controls is selected
    *  
    *             
    *  @param e The ActionEvent for the change
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void comboBoxHit ( ActionEvent e ) {
      JComboBox comboBox      = (JComboBox) e.getSource() ;
      Object    selectedItem  = comboBox.getSelectedItem() ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   CB_CATEGORY                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( comboBox == cb_CATEGORY )
         curCategoryRec = (TypeDocCatRec) selectedItem ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   CB_SEARCHTYPE                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( comboBox == cb_SEARCHTYPE ) {
         GlassItem item = (GlassItem) selectedItem ;

         searchType = item.getId() ;
         setSearchFields() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   CB_BRANDS                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( comboBox == cb_BRANDS ) {
         curBrandRec = (TypeCategoryRec) selectedItem ;

         refreshFamilyList() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   CB_FAMILY                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( comboBox == cb_FAMILY )
         curGroupRec = (TypeGroupRec) selectedItem ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   CB_DOCTYPE                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( comboBox == cb_DOCTYPE )
         typeDocRec = (TypeDocRec) selectedItem ;

   } /* itemStateChanged */




   /*-----------------------------------------------------------*/
   /* setSearchFields
    */
   /**
    *    This method will adjust the entry fields
    *  and choices for each different search type
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setSearchFields () {
      int    format     = MaskDocument.MASKTYPE_UNKNOWN ;
      String mask       = null ;
      int    maxLength  = 50 ;
      String compareStr = Str.getStr ( STR_IS ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Take off all controls                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_SEARCHARG.setVisible ( false ) ;
      pnl_USERENTRY.setVisible ( false ) ;
      cb_BRANDS.setVisible ( false ) ;
      cb_FAMILY.setVisible ( false ) ;
      cb_CATEGORY.setVisible ( false ) ;                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the mask type for certain searches                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      switch ( searchType ) {
         
         case DocViewRec.VIEW_SOURCE :
         case DocViewRec.VIEW_DRAFT_BY_COMMENT :
         case DocViewRec.VIEW_BY_TITLE :
            maxLength = LENGTH_DOCUMENTTITLE ;
         case DocViewRec.VIEW_DOCBODY :
            ef_SEARCHARG.setVisible ( true ) ;
            compareStr = Str.getStr ( STR_LIKE ) ;
            docTypeRequired = true ;
            break ;

         case DocViewRec.VIEW_BY_BRAND : 
            cb_BRANDS.setVisible ( true ) ;
            docTypeRequired = true ;
            break ;

         case DocViewRec.VIEW_BY_AUTHOR :
            pnl_USERENTRY.setVisible ( true ) ;
            docTypeRequired = true ;
            break ;

         case DocViewRec.VIEW_BY_BRANDGROUPIND :
            cb_BRANDS.setVisible ( true ) ;
            cb_FAMILY.setVisible ( true ) ;
            refreshFamilyList() ;
            docTypeRequired = true ;
            break ;

         case DocViewRec.VIEW_BY_NOTESID :
            format = MaskDocument.MASKTYPE_UPPER ;
            mask="VVVV-VVVVVV" ;
            ef_SEARCHARG.setVisible ( true ) ;
            docTypeRequired = false ;
            break ;

         case DocViewRec.VIEW_BY_QUESTIND :
            ef_SEARCHARG.setVisible ( true ) ;
            format = MaskDocument.MASKTYPE_NUMERIC ;
            maxLength=12 ;
            docTypeRequired = false ;
            break ;

         case DocViewRec.VIEW_BY_BRANDCATEGORY :
            cb_BRANDS.setVisible ( true ) ;
            cb_CATEGORY.setVisible ( true ) ;
            docTypeRequired = true ;
            break ;

         case DocViewRec.VIEW_QUERY_TEXT :
            docTypeRequired = false ;
            ef_SEARCHARG.setVisible ( true ) ;
            break ;

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Clear the entry field                                   */          
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ef_SEARCHARG.isVisible() ) {
         String text = ef_SEARCHARG.getText() ;

         /* Set the entry field up */
         try {
            if ( mask != null ) {
               ef_SEARCHARG.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, mask, ef_SEARCHARG ) ) ;
            } else
               ef_SEARCHARG.setDocument ( new MaskDocument ( format, maxLength ) ) ;
         } catch ( Exception e ) {
         }

         /* Re-apply the text  */
         if ( text != null )
            ef_SEARCHARG.setText ( text ) ;
      }

      cb_DOCTYPE.setVisible ( docTypeRequired ) ;
      st_ANDTYPEIS.setVisible ( docTypeRequired ) ;
      st_IS.setText ( compareStr ) ;

      brd_FIND.revalidate() ;
      if ( ef_SEARCHARG.isVisible() )
         ef_SEARCHARG.requestFocus() ;
   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    this method will respond to buttons being
    *  pressed
    *  
    *
    *  @param id The ID of the button
    *  @param button The instance of the button
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Process DButton events                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( allowInput ) && ( e.getSource() instanceof DButton ) ) {
         DButton button = (DButton) e.getSource() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_CANCEL                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( button == pb_CANCEL ) {
            retVec = null ;
            closeWindow() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_FIND                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_FIND )
            find() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_COPY                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_COPY )
            copyToClip() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_CLEAR                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_CLEAR ) {
            cnr_DOCS.removeAll() ;
            ef_SEARCHARG.setText ( "" ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_SELECT                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_SELECT ) {
            select() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_BOOKMARKS                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_BOOKMARKS ) {
            if ( showingBookmarks ) {
               brd_DOCS.remove ( bookmarkScroll ) ;
               brd_DOCS.add ( cnr_DOCS, BorderLayout.CENTER ) ;
               pb_BOOKMARKS.setText ( Str.getStr ( STR_BOOKMARKS ) ) ;
               showingBookmarks = false ;
            } else {
               brd_DOCS.remove ( cnr_DOCS ) ;
               brd_DOCS.add ( bookmarkScroll, BorderLayout.CENTER ) ;
               showingBookmarks = true ;
               pb_BOOKMARKS.setText ( Str.getStr ( STR_BROWSE_DB ) ) ;
            }
            GUISystem.setPreferredButton ( pb_BOOKMARKS ) ;
            buttonPanel.revalidate() ;
            brd_DOCS.invalidate() ;
            repaint() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_PREV ( In PrevNextPanel )                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == prevNextPanel.getPrevButton() ) {
            int maxReturn = PropertySystem.getInt ( PROPERTY_MAX_DOCUMENT_RETURN ) ;

            if ( viewRec.fromCount < maxReturn )
               viewRec.fromCount=0 ;
            else
               viewRec.fromCount-=maxReturn ;

            if ( viewRec.fromCount < maxReturn )
               prevNextPanel.showPrev(false) ;

            if ( viewRec.actualCount > maxReturn )
               prevNextPanel.showNext(true) ;

            new Thread ( this, THREAD_SEARCH ).start() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_NEXT ( In PrevNextPanel )                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == prevNextPanel.getNextButton() ) {
            int maxReturn = PropertySystem.getInt ( PROPERTY_MAX_DOCUMENT_RETURN ) ;

            viewRec.fromCount+=maxReturn ;
            prevNextPanel.showPrev(true) ;
            viewRec.refresh() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_EDIT                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_EDIT ) {

            if ( showingBookmarks ) {
               StringTreeNode selectedObject = (StringTreeNode) bookmarkTree.getSelectionPath().getLastPathComponent() ;                                         

               //VOYif ( selectedObject != null ) {
               //VOY   Bookmark bookmark = (Bookmark) selectedObject.getData() ;

               //VOY   if ( bookmark.getBookmarkType() == BOOKMARK_DOCUMENT ) {
               //VOY      closeWindow() ;
               //VOY      new DocRec ( bookmark ).edit() ;
               //VOY   }
               //VOY}
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, just edit the document selected              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
               try {
                  DocumentRow docRow = (DocumentRow)cnr_DOCS.getSelectedItem() ;                      

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Only process if a doc is selected                       */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( docRow != null ) {
                     EditDocument.editDocument ( docRow ) ;
                     closeWindow() ;
                  }
                  else
                     GUISystem.printBox ( Str.getStr( STR_INFO ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;

               } catch ( Exception e1 ) {
                  LogSystem.log ( 1, e1 ) ;
               }
            }

         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Catch JTextField Events                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( allowInput ) && ( e.getSource() instanceof JTextField ) )
         find() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Catch JComboBox Events                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( allowInput ) && ( e.getSource() instanceof JComboBox ) )
         comboBoxHit ( e ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Catch the Double click on a doc                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( allowInput ) && ( e.getSource() instanceof MultiList ) )
         select() ;


   }


   /*------------------------------------------------------------*/
   /* newSearch
    */
   /**
    *    This method will clear out the viewRec
    *  and set the default values of it
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return nothing
    *//*---------------------------------------------------------*/
   public void newSearch() {

      /* Take away the prev/next buttons */
      prevNextPanel.showNext(false) ;
      prevNextPanel.showPrev(false) ;

   } /* newSearch */



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform certain operations
    *  in a thread. The operation that is performed
    *  depends on the name of the thread.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName() ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_SEARCH                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( name.equals ( THREAD_SEARCH ) ) {
         boolean    startSearch   = false ;
         boolean    drafts        = ck_REQUEST.isSelected() ;
         TypeDocRec curDocTypeRec = (TypeDocRec)cb_DOCTYPE.getSelectedItem() ;
         String     strSearchArg  = null ;

         switch ( searchType ) {
            case DocViewRec.VIEW_BY_BRAND :
               curBrandRec    = (TypeCategoryRec)  cb_BRANDS.getSelectedItem() ;

               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.EXISTINGDOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,          DOCUMENTS.LNDOCID,         DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM   QUEST.DOCUMENTS DOCUMENTS,"+
                              "       QUEST.DOCBRAND DOCBRAND "+
                              "WHERE  DOCBRAND.BRANDGROUPIND IN (SELECT BRANDGROUPIND FROM TIGRIS.BRANDS WHERE BRANDIND = "+curBrandRec.getInd()+") AND "+
                              "       DOCUMENTS.DOCIND     = DOCBRAND.DOCIND AND"+
                              "       DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "       DOCUMENTS.TEMPLATE   = 'N' AND "+
                              "       DOCUMENTS.RECYCLED   = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),      DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND, DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM   TIGRIS.DOCUMENTS DOCUMENTS,"+
                              "       TIGRIS.DOCBRAND DOCBRAND "+
                              "WHERE  DOCBRAND.BRANDGROUPIND IN (SELECT BRANDGROUPIND FROM TIGRIS.BRANDS WHERE BRANDIND = "+curBrandRec.getInd()+") AND "+
                              "       DOCUMENTS.DOCIND     = DOCBRAND.DOCIND AND"+
                              "       DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "       DOCUMENTS.PUBLISH    = 'Y' AND "+
                              "       DOCUMENTS.RECYCLED   = 'N' " ;


               startSearch = true ;
               break ;

            case DocViewRec.VIEW_BY_AUTHOR :
               UserRec userRec = pnl_USERENTRY.getUserRec() ;

               strSearchArg = userRec.getUserId() ;

               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,     DOCUMENTS.LNDOCID,         DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM QUEST.DOCUMENTS DOCUMENTS "+
                              "WHERE DOCUMENTS.DBUSER     = '"+strSearchArg+"'              AND "+
                              "      DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "      DOCUMENTS.RECYCLED   = 'N'                          AND "+
                              "      DOCUMENTS.TEMPLATE   = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),DOCUMENTS.DOCIND , DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,     DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM TIGRIS.DOCUMENTS DOCUMENTS "+
                              "WHERE DOCUMENTS.DBUSER     = '"+strSearchArg+"' AND "+
                              "      DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "      DOCUMENTS.RECYCLED   = 'N'      AND "+
                              "      DOCUMENTS.PUBLISH    = 'Y' " ;

               startSearch = true ;
               break ;

            case DocViewRec.VIEW_BY_BRANDGROUPIND :
               int brandGroupInd = 0 ;

               curBrandRec   = (TypeCategoryRec)  cb_BRANDS.getSelectedItem() ;
               curGroupRec   = (TypeGroupRec)  cb_FAMILY.getSelectedItem() ;
               //VOY brandGroupInd =  Brands.brandGroupIndFromBrandGroup ( curBrandRec.ind, curGroupRec.ind ) ;

               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.EXISTINGDOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,      DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM   QUEST.DOCUMENTS DOCUMENTS,"+
                              "       QUEST.DOCBRAND DOCBRAND "+
                              "WHERE  DOCBRAND.BRANDGROUPIND IN (SELECT BRANDGROUPIND FROM TIGRIS.BRANDS WHERE BRANDGROUPIND = "+brandGroupInd+") AND "+
                              "       DOCUMENTS.DOCIND     = DOCBRAND.DOCIND AND"+
                              "       DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "       DOCUMENTS.TEMPLATE   = 'N' AND "+
                              "       DOCUMENTS.RECYCLED   = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.DOCIND , DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,      DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM   TIGRIS.DOCUMENTS DOCUMENTS,"+
                              "       TIGRIS.DOCBRAND DOCBRAND "+
                              "WHERE  DOCBRAND.BRANDGROUPIND IN (SELECT BRANDGROUPIND FROM TIGRIS.BRANDS WHERE BRANDGROUPIND = "+brandGroupInd+") AND "+
                              "       DOCUMENTS.DOCIND     = DOCBRAND.DOCIND AND"+
                              "       DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "       DOCUMENTS.PUBLISH    = 'Y' AND "+
                              "       DOCUMENTS.RECYCLED   = 'N' " ;

               startSearch = true ;
               break ;

            case DocViewRec.VIEW_BY_TITLE :

               if ( drafts )
                  lastQuery = "SELECT DISTINC(DOCUMENTS.DOCIND),          DOCUMENTS.EXISTINGDOCIND , DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,     DOCUMENTS.LNDOCID,         DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM QUEST.DOCUMENTS DOCUMENTS "+
                              "WHERE translate(DOCUMENTS.TITLE) like '%"+title.toUpperCase()+"%' AND "+
                              "      DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.TEMPLATE  = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),          DOCUMENTS.DOCIND , DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,     DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM TIGRIS.DOCUMENTS DOCUMENTS "+
                              "WHERE translate(DOCUMENTS.TITLE) like '%"+title.toUpperCase()+"%' AND "+
                              "      DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.PUBLISH   = 'Y' " ;

               startSearch = true ;
               break ;

            case DocViewRec.VIEW_DRAFT_BY_COMMENT :

               strSearchArg = ef_SEARCHARG.getText() ;

               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),          DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,     DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM QUEST.DOCUMENTS DOCUMENTS "+
                              "WHERE translate(COMMENT) LIKE '%"+strSearchArg+"%' AND "+
                              "      DOCUMENTS.STOPDATE IS NULL     AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.TEMPLATE  = 'N' " ;

               if ( !ck_REQUEST.isSelected() )
                  GUISystem.printBox ( this, STR_INFO, STR_CANNOT_SEARCH_COMMENT_ON_PUBLISH ) ;
               else
                  startSearch = true ;
               break ;

            case DocViewRec.VIEW_SOURCE :

               strSearchArg = ef_SEARCHARG.getText() ;
               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),          DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,     DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM QUEST.DOCUMENTS DOCUMENTS "+
                              "WHERE translate(SOURCE) LIKE '%"+strSearchArg.toUpperCase()+"%' AND "+
                              "      DOCUMENTS.STOPDATE IS NULL     AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.TEMPLATE  = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),       DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER, "+
                              "       DOCUMENTS.DOCCLASSIND,  DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER, "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM TIGRIS.DOCUMENTS DOCUMENTS "+
                              "WHERE translate(DOCUMENTS.SOURCE) LIKE '%"+strSearchArg.toUpperCase()+"%' AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.PUBLISH   = 'Y' ";

               startSearch = true ;
               break ;

            case DocViewRec.VIEW_DOCBODY :

               strSearchArg = ef_SEARCHARG.getText() ;

               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.EXISTINGDOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,      DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM   QUEST.DOCUMENTS DOCUMENTS,"+
                              "       QUEST.DOCBODY DOCBODY "+
                              "WHERE  DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "       DOCUMENTS.DOCIND     = DOCBODY.DOCIND AND"+
                              "       DOCBODY.BODY like '%"+strSearchArg+"%' AND "+
                              "       DOCUMENTS.TEMPLATE   = 'N' AND "+
                              "       DOCUMENTS.RECYCLED   = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),      DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND, DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM   TIGRIS.DOCUMENTS DOCUMENTS,"+
                              "       TIGRIS.DOCBODY DOCBODY "+
                              "WHERE  DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "       DOCUMENTS.DOCIND     = DOCBODY.DOCIND AND"+
                              "       DOCBODY.BODY like '%"+strSearchArg+"%' AND "+
                              "       DOCUMENTS.PUBLISH    = 'Y' AND "+
                              "       DOCUMENTS.RECYCLED   = 'N' " ;

               startSearch = true ;
               break ;

            case DocViewRec.VIEW_BY_NOTESID :

               strSearchArg = ef_SEARCHARG.getText() ;

               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),          DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,     DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM QUEST.DOCUMENTS DOCUMENTS "+
                              "WHERE LNDOCID = '"+strSearchArg+"' AND "+
                              "      DOCUMENTS.STOPDATE IS NULL     AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.TEMPLATE  = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),       DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER, "+
                              "       DOCUMENTS.DOCCLASSIND,  DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER, "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM TIGRIS.DOCUMENTS DOCUMENTS "+
                              "WHERE LNDOCID = '"+strSearchArg+"' AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.PUBLISH   = 'Y' " ;

               startSearch = true ;
               break ;

            case DocViewRec.VIEW_BY_QUESTIND :
               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),          DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,     DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM QUEST.DOCUMENTS DOCUMENTS "+
                              "WHERE DOCIND = "+ef_SEARCHARG.getText()+" AND "+
                              "      DOCUMENTS.STOPDATE IS NULL     AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.TEMPLATE  = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),       DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER, "+
                              "       DOCUMENTS.DOCCLASSIND,  DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER, "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM TIGRIS.DOCUMENTS DOCUMENTS "+
                              "WHERE DOCIND = "+ef_SEARCHARG.getText()+" AND "+
                              "      DOCUMENTS.RECYCLED  = 'N'      AND "+
                              "      DOCUMENTS.PUBLISH   = 'Y' " ;
               startSearch = true ;
               break ;

            case DocViewRec.VIEW_QUERY_TEXT :
               lastQuery = formulateQueryTextSQL() ;
               startSearch = true ;
               break ;

            case DocViewRec.VIEW_BY_BRANDCATEGORY :
               TypeDocCatRec curDocCatRec = (TypeDocCatRec)cb_CATEGORY.getSelectedItem() ;


               if ( drafts )
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.EXISTINGDOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND,          DOCUMENTS.LNDOCID,         DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM   QUEST.DOCUMENTS DOCUMENTS,"+
                              "       QUEST.DOCBRAND DOCBRAND, "+
                              "       QUEST.DOCCATEGORY DOCCATEGORY "+
                              "WHERE  DOCBRAND.BRANDGROUPIND IN (SELECT BRANDGROUPIND FROM TIGRIS.BRANDS WHERE BRANDIND = "+curBrandRec.getInd()+") AND "+
                              "       DOCUMENTS.DOCIND     = DOCBRAND.DOCIND AND"+
                              "       DOCCATEGORY.DOCIND    = DOCUMENTS.DOCIND AND "+
                              "       DOCCATEGORY.DOCCATIND = "+curDocCatRec.getInd()+" AND "+
                              "       DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "       DOCUMENTS.TEMPLATE   = 'N' AND "+
                              "       DOCUMENTS.RECYCLED   = 'N' " ;
               else
                  lastQuery = "SELECT DISTINCT(DOCUMENTS.DOCIND),      DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                              "       DOCUMENTS.DOCCLASSIND, DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                              "       DOCUMENTS.CHANGEDTIME "+
                              "FROM   TIGRIS.DOCUMENTS DOCUMENTS,"+
                              "       TIGRIS.DOCBRAND DOCBRAND, "+
                              "       TIGRIS.DOCCATEGORY DOCCATEGORY "+
                              "WHERE  DOCBRAND.BRANDGROUPIND IN (SELECT BRANDGROUPIND FROM TIGRIS.BRANDS WHERE BRANDIND = "+curBrandRec.getInd()+") AND "+
                              "       DOCUMENTS.DOCIND     = DOCBRAND.DOCIND AND"+
                              "       DOCCATEGORY.DOCIND    = DOCUMENTS.DOCIND AND "+
                              "       DOCCATEGORY.DOCCATIND = "+curDocCatRec.getInd()+" AND "+
                              "       DOCUMENTS.DOCTYPEIND = "+curDocTypeRec.getInd()+" AND "+
                              "       DOCUMENTS.PUBLISH    = 'Y' AND "+
                              "       DOCUMENTS.RECYCLED   = 'N' " ;


               startSearch = true ;
               break ;
         }


         if ( startSearch ) {
            Vector listeners = new Vector ( 1 ) ;

            listeners.addElement ( this ) ;

            viewRec.setListeners ( listeners ) ; 
            viewRec.setCategoryInd ( -1 ) ;
            viewRec.setSQLQuery ( lastQuery ) ;

            viewRec.refresh() ;
         }

      } /* THREAD_SEARCH */

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_INITIALIZE                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( name.equals ( THREAD_INITIALIZE ) ) {
         setDefaultCloseOperation ( DO_NOTHING_ON_CLOSE ) ;
         addWindowListener ( this ) ;
         createControls() ;
         refreshList() ;
         doLayout() ;
         getContentPane().setVisible ( false ) ;
         getContentPane().setVisible ( true ) ;
         ef_SEARCHARG.setCaretPosition(0) ;
      }

   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save all of our search
    *  data
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if all data was entered, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      boolean rc = false ;

      curCategoryRec = (TypeDocCatRec) cb_CATEGORY.getSelectedItem() ;
      curBrandRec    = (TypeCategoryRec)  cb_BRANDS.getSelectedItem() ;
      curGroupRec    = (TypeGroupRec)  cb_FAMILY.getSelectedItem() ;

      switch ( searchType ) {
         case DocViewRec.VIEW_BY_BRAND :
            rc = ( curBrandRec != null ) ;
            if ( !rc )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_BRAND );
            break ;

         case DocViewRec.VIEW_QUERY_TEXT :
            queryText = ef_SEARCHARG.getText() ;

            if ( ( queryText != null ) && ( queryText.length() > 0 ) )
               rc = true ;
            else
               GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
            break ;


         case DocViewRec.VIEW_BY_BRANDGROUPIND :
            rc = (curGroupRec != null ) ;
            if ( !rc )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_GROUP );
            break ;

         case DocViewRec.VIEW_BY_TITLE :
            title = ef_SEARCHARG.getText() ;

            if ( ( title != null ) && ( title.length() > 2 ) ) {
               rc = true ;
               title = "%"+title+"%" ;
            }

            if ( !rc )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_VALUE ) ;
            break ;

         case DocViewRec.VIEW_DRAFT_BY_COMMENT :
            comment = ef_SEARCHARG.getText() ;

            if ( ( comment == null ) || ( comment.length() < 2 ) )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_VALUE ) ;
            else
               rc = true ;
            break ;

         case DocViewRec.VIEW_SOURCE :
            source = ef_SEARCHARG.getText() ;

            if ( ( source == null ) || ( source.length() < 2 ) )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_VALUE ) ;
            else
               rc = true ;
            break ;

         case DocViewRec.VIEW_DOCBODY :
            body = ef_SEARCHARG.getText() ;

            if ( ( body != null ) && ( body.length() >= 2 ) ) {
               rc = true ;
               body = "%"+body+"%" ;
               rc = !Text.containsHTML ( body ) ;
               if ( body.length() < 7 ) {
                  if ( rc )
                     rc = ( body.toUpperCase().indexOf ( "THE" ) == -1 ) ;
                  if ( rc )
                     rc = ( body.toUpperCase().indexOf ( "AT" ) == -1 ) ;
                  if ( rc )
                     rc = ( body.toUpperCase().indexOf ( "AND" ) == -1 ) ;
               }
            }

            if ( !rc )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_VALUE ) ;
            break ;

         case DocViewRec.VIEW_BY_AUTHOR :
            user = pnl_USERENTRY.getUserId() ;
            if ( ( user == null ) || ( user.length() == 0 ) )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_VALUE ) ;
            rc = true ;
            break ;

         case DocViewRec.VIEW_BY_QUESTIND :
            try {
               docInd = new Integer ( ef_SEARCHARG.getText() ).intValue() ;
            } catch ( Exception e ) {
               docInd = 0 ;
            }

            if ( docInd == 0 )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_VALUE ) ;
            else
               rc = true ;
            break ;

         case DocViewRec.VIEW_BY_NOTESID :
            notesId = ef_SEARCHARG.getText() ;

            if ( notesId.length() != LENGTH_NOTESID )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_ENTER_VALUE ) ;
            else
               rc = true ;

            break ;

         case DocViewRec.VIEW_BY_BRANDCATEGORY :
            rc = ( curBrandRec != null ) ;
            if ( !rc )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_BRAND ) ;
            else
               rc = ( curCategoryRec != null ) ;

            if ( !rc )
               GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_CATEGORY ) ;
            break ;
      }



      /* Ok, if everything is ok check to see if we need a doc type */
      if ( rc ) {
         if ( ( docTypeRequired ) && ( typeDocRec == null ) ) {
            GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_DOC_TYPE ) ;
            rc = false ;
         } else if ( ( docTypeRequired ) &&  ( typeDocRec != null ) ) {
            rc = true ;
         } else {
            rc = true ;
         }
      } /* if */

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* docViewStarted
    */
   /**
    *    This method will be called when a view
    *  has started to refresh
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void docViewStarted ( DocViewRec docViewRec ) {
      System.out.println ( "Started!" ) ;
      showSearchBox  ( true ) ;
   }



   /*------------------------------------------------------------*/
   /* searchComplete
    */
   /**
    *    This method will be called when a search
    *  we have started is complete
    *  
    *
    *  @param searchType The type of search ( DOCUMENT, PRODUCT etc.. )
    *  @param userData The user data returned from the search
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void docViewStopped ( DocViewRec viewRec ) {

      System.out.println ( "Stopped!" ) ;
      if ( viewRec.getResults() != null ) {
         int          toInt     = 0 ;
         boolean      showNext  = false ;
         boolean      showPrev  = false ;

         cnr_DOCS.removeAll() ;
         cnr_DOCS.add ( viewRec.getResults() ) ;

         if ( viewRec.actualCount > 50 )
            showNext = true ;

         if ( viewRec.fromCount > ( viewRec.actualCount-50 ) )
            showNext = false ;


         if ( viewRec.fromCount > 0 )
            showPrev = true ;

         prevNextPanel.showPrev(showPrev) ;
         prevNextPanel.showNext(showNext) ;

         cnr_DOCS.invalidate() ;
         cnr_DOCS.doLayout() ;

         /* Calculate the To positions */
         toInt=viewRec.fromCount+viewRec.getResults().size() ;

         ((TitledBorder)brd_DOCS.getBorder()).setTitle ( Str.getStr ( STR_DOCUMENTS )+"  "+viewRec.fromCount+" - "+toInt+" of "+viewRec.actualCount ) ;
         brd_DOCS.repaint() ;
      }

      showSearchBox ( false ) ;
   } /* searchComplete */


   /*-----------------------------------------------------------*/
   /* showSearchBox
    */
   /**
    *    This method will either show or hide the
    *  search box and find box. It will also disable all of
    *  our controls if the brd_SEARCHING box will show.
    *  
    *             
    *  @param show TRUE = show the search box, hide the find box. The inverse for FALSE
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void showSearchBox ( boolean show ) {
      brd_SEARCHING.setVisible ( show ) ;
      brd_FIND.setVisible ( !show ) ;
      allowInput = !show ;
   } /* showSearchBox */





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    Called when an item is selected
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      if ( e.getSource() == cnr_DOCS ) 
         select() ;
   }

   private void closeWindow () {
      PropertySystem.putDetailList ( PROPERTY_DOCUMENT_COLUMN_WIDTHS, cnr_DOCS ) ;

      setVisible ( false ) ;
   }





   /*------------------------------------------------------------*/
   /* find
    */
   /**
    *    This method will kick off our search
    *  thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void find () {
      if ( saveData() ) {
         newSearch() ;
         new Thread ( this, THREAD_SEARCH ).start() ;
      }
   }





   /*------------------------------------------------------------*/
   /* refreshFamilyList
    */
   /**
    *    This method will refresh our family list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshFamilyList () {
      if ( ( curBrandRec != null ) && ( searchType == DocViewRec.VIEW_BY_BRANDGROUPIND ) ) {
         Vector groupVec = Brands.getFamilies ( curBrandRec ) ;
         int    i        = 0 ;
         int    length   = 0 ;

         cb_FAMILY.removeAllItems() ;

         if ( groupVec != null ) {
            length = groupVec.size() ;

            while ( i < length )
               cb_FAMILY.addItem ( groupVec.elementAt(i++) ) ;

         }

      } /* if */
   }





   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ){
   }
   public void windowClosing ( WindowEvent e ) {
      setVisible ( false ) ;
   }
   public void windowClosed ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e ) {
   }
   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowOpened ( WindowEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* select
    */
   /**
    *    This method will get everything that is selected
    *  and will close the window, but not before
    *  notifying the listeners. If nothing is selected, nothing happens
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void select () {                                       
      Vector vec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are showing bookmarks, take data from tree        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( showingBookmarks ) {
         try {
            StringTreeNode selectedObject = (StringTreeNode) bookmarkTree.getSelectionPath().getLastPathComponent() ;                                         

            if ( selectedObject != null ) {
               Bookmark bookmark = (Bookmark) selectedObject.getData() ;

               if ( bookmark.isFolder() ) {
                  vec = BookmarkSystem.getBookmarks ( bookmark.getDescript() ) ;
               } else {
                  TreePath[] treePath = bookmarkTree.getSelectionPaths() ;

                  vec = new Vector ( 1, treePath.length ) ;
                  for ( int i = 0; i < treePath.length; i++ )
                     vec.addElement ( ((Bookmark)treePath[i].getLastPathComponent()).getObject() ) ;
               }

            }
         } catch ( Exception e ) {
         }

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, take the data from the container             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         vec = cnr_DOCS.getSelection() ;
      }

      if ( vec != null ) {
         if ( listener != null )
            listener.selectDocComplete ( vec ) ;
         closeWindow() ;
      }

   } /* select  */



   public void dispose() {
      //super.dispose() ;
      //((Frame)getParent()).dispose() ;
   }




   /*------------------------------------------------------------*/
   /* copyToClip
    */
   /**
    *    This method will create a URL hotlink
    *  to the document selected and will paste
    *  that hotlink into the clipboard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void copyToClip () {
      //DocRec docRec = (DocRec) cnr_DOCS.getSelectedItem() ;

      //VOYif ( docRec != null ) {
      //VOY   StringSelection urlLink = new StringSelection ( docRec.getFieldData ( FIELD_FILENAME )+".html" ) ;
      //VOY   Clipboard       cp      = Toolkit.getDefaultToolkit().getSystemClipboard() ;                      

      //VOY   cp.setContents ( urlLink, this ) ;
      //VOY}
      GUISystem.printBox ( "ERROR", "NOT IMPLENETED" ) ;
   }




   /*------------------------------------------------------------*/
   /* lostOwnership
    */
   /**
    *    This method will be called if this window
    *  loses ownership of the clipboard. We really
    *  don't care about that so we do nothing!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void lostOwnership (Clipboard cp, Transferable contents ) {
   }




   /*------------------------------------------------------------*/
   /* addSelectDocListener
    */
   /**
    *    This method will add a listener to this window
    *  to listen to when selections are made and
    *  the 'Select' button is pressed
    *
    *  @param listener The listener
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addSelectDocListener ( SelectDocListener listener ) {
      this.listener = listener ;
   }





   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return the instance of the SelectDocDlg
    *  that is valid and showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectDocDlg getInstance() {
      return( instance ) ;
   }





   /*------------------------------------------------------------*/
   /* removeSelectDocListener
    */
   /**
    *    This method will remove a SelectDocListener
    *  from the list of listeners that are notified
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeSelectDocListener ( SelectDocListener listener ) {
      this.listener = null ;
   }



   /*------------------------------------------------------------*/
   /* formulateQueryTextSQL
    */
   /**
    *    This method will pick apart the question
    *  given and will return SQL matching that
    *  question from the TIGRIS or QUEST.DOCQUERYTEXT
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return True if documents were returned Ok
    *//*---------------------------------------------------------*/
   private String formulateQueryTextSQL () {
      //VOYboolean         rc          = false ;
      //VOYString          inClause    = "" ;
      //VOYString          queryText   = ef_SEARCHARG.getText() ;
      //VOYStringTokenizer strtok      = new StringTokenizer ( queryText, " " ) ;
      //VOYString          curtok      = null ;
      //VOYDocRec          readDoc     = null ;
      //VOYint             count       = 0 ;
      //VOYint             actualCount = 0 ;
      //VOYString          query       = null ;

      //VOY/*-----------------------------------------------------------*/
      //VOY/*                                                           */
      //VOY/*   First, pick out all irrelevant words, forming our list  */
      //VOY/* of relevant words                                         */
      //VOY/*                                                           */
      //VOY/*-----------------------------------------------------------*/
      //VOYwhile ( strtok.hasMoreTokens() ) {
      //VOY   curtok = strtok.nextToken() ;

      //VOY   if ( ( !curtok.equalsIgnoreCase ( "the" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "i" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "who" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "what" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "when" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "where" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "why" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "is" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "in" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "can" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "to" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "too" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "want" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "do" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "meg" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "megabyte" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "gig" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "how" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "get" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "a" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "gig" ) ) &&
      //VOY        ( !curtok.equalsIgnoreCase ( "gigabyte" ) ) ) {
      //VOY      inClause+=" (DOCQUERYTEXT.QUERYTEXT LIKE '%"+cleanToken ( curtok.toUpperCase() )+"%' AND DOCUMENTS.DOCIND = DOCQUERYTEXT.DOCIND) " ;

      //VOY      if ( strtok.hasMoreTokens() )
      //VOY         inClause+=" OR " ;
      //VOY   }
      //VOY}




      //VOY/*-----------------------------------------------------------*/
      //VOY/*                                                           */
      //VOY/*   Ok, now execute the query                               */                
      //VOY/*                                                           */
      //VOY/*-----------------------------------------------------------*/
      //VOYif ( ck_REQUEST.isSelected() )
      //VOY   query = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER, "+
      //VOY           "       DOCUMENTS.DOCCLASSIND,      DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER, "+
      //VOY           "       DOCUMENTS.CHANGEDTIME "+
      //VOY           "FROM QUEST.DOCUMENTS DOCUMENTS, "+
      //VOY           "     QUEST.DOCQUERYTEXT DOCQUERYTEXT "+
      //VOY           "     WHERE ("+inClause+") " ;
      //VOYelse
      //VOY   query = "SELECT DISTINCT(DOCUMENTS.DOCIND), 0,                DOCUMENTS.TITLE,   DOCUMENTS.OWNER, "+
      //VOY           "       DOCUMENTS.DOCCLASSIND,      DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER, "+
      //VOY           "       DOCUMENTS.CHANGEDTIME "+
      //VOY           "FROM TIGRIS.DOCUMENTS DOCUMENTS, "+
      //VOY           "     TIGRIS.DOCQUERYTEXT DOCQUERYTEXT "+
      //VOY           "     WHERE ("+inClause+")  " ;

       GUISystem.printBox ( "ERROR", "NOT IMPLEMENTED forumlateQueryText" ) ;
      return( null ) ;

   }

   private String cleanToken ( String token ) {
      token = Text.replaceAllStrInStr ( token, "?", "" ) ;
      token = Text.replaceAllStrInStr ( token, "'", "" ) ;
      token = Text.replaceAllStrInStr ( token, ",", "" ) ;
      token = Text.replaceAllStrInStr ( token, ".", "" ) ;
      token = Text.replaceAllStrInStr ( token, "'", "" ) ;
      return( token.trim() ) ;
   }

} /* SelectDocDlg */
