package com.ibm.nzna.projects.qit.doc.history ;         
/*-----------------------------------------------------------*/
/*   DocHistNavPanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocHistNavPanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This nav panel is used by the DocHistPanel for navigation. It allows 
 * the user to select a date to when the document was published
 *
 * @version 1.0, 8/30/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocHistNavPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocHistNavPanel extends JPanel implements DocConst, ActionListener, ChangeListener, NavPanel, AppConst {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    /**
      *  The JTabbedPane which holds the View Panel and Result Panel
      **/
    private JTabbedPane tabbedPane   = null ;

    /**
      *  The Result page showing the find results
      **/
    private HistoryResultPanel resultPanel = null ;


    /**
      *  The Document History Panel we are tied to
      **/
    private DocHistPanel docHistPanel      = null ;

    /**
      *  The panel holding the document views
      **/
    private DocViewPanel viewPanel = null ;

    /**
      * The CheckBox allowing the user to specifiy to show deleted
      * documents or not
      */
    private JCheckBox ck_SHOW_DELETED_DOCS = null ;

    /**
      * This panel holds all of the components for the DocViewPanel
      */
    private JPanel allViewPanel = null ;






    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This constructor simply assigns our DocHistPanel
     *  property
     *  
     *
     *  @param docHistPanel The DocHistPanel that this nav panel is tied to
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public DocHistNavPanel ( DocHistPanel docHistPanel ) {
        this.docHistPanel = docHistPanel ;
    }


    /*------------------------------------------------------------*/
    /* getDocInd
     */
    /**
     *    This method will return what docInd is
     *  currently displayed in the doc indicator
     *  entry field
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return int
     *//*---------------------------------------------------------*/
    protected int getDocInd () {
        return( resultPanel.getDocInd() ) ;
    }

    /*------------------------------------------------------------*/
    /* setDocInd
     */
    /**
     *    This method will set the doc indicator that 
     *  we are displaying. This will also kick off the thread
     *  that reads in all of the history of the document
     *
     *  @param docInd The DocInd to set to
     *  @param request TRUE = This docInd represents a request
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    protected void setDocInd ( int docInd, boolean request ) {
        resultPanel.setDocInd ( docInd, request ) ;
    }



    /*------------------------------------------------------------*/
    /* getTitle
     */
    /**
     *    This method will return out title
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String getTitle () {
        return( Str.getStr ( STR_DOCUMENT_HISTORY ) ) ;
    }




    /*------------------------------------------------------------*/
    /* getJComponent
     */
    /**
     *    This method will return an instance of this
     *  class
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public JComponent getJComponent () {
        return( this ) ;
    }







    /*------------------------------------------------------------*/
    /* stop
     */
    /**
     *    This method will be called once we have
     *  been closed. Clean up time!
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void stop () {

    }

    /**
      *  Called when our panel is showing. We will set our focus to
      * the Result Panel.
      **/
    public void start () {
        resultPanel.start() ;
    }

    /**
      *  This method will initialize all of our controls
      **/
    public void initialize () {
        Color  tabColor  = ThemeSystem.getColor ( "NAVPANEL_BG_COLOR" ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create the GUI                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        tabbedPane           = new JTabbedPane () ;
        resultPanel          = new HistoryResultPanel ( docHistPanel ) ;   
        viewPanel            = new DocViewPanel () ;
        ck_SHOW_DELETED_DOCS = new JCheckBox ( Str.getStr ( STR_SHOW_DELETED_DOCS_ONLY ) ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Initialize the tabbed panelss                           */                    
        /*                                                           */
        /*-----------------------------------------------------------*/
        resultPanel.initialize() ;
        viewPanel.refresh ( true ) ;
        viewPanel.setBorder ( GUISystem.emptyBorder ) ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust properties                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        viewPanel.setMaxDocumentReturn ( 100000 ) ;
        viewPanel.addDocViewListener ( docHistPanel ) ;
        tabbedPane.addChangeListener ( this ) ;
        viewPanel.setShowDeletedDocsOnly ( true ) ;
        ck_SHOW_DELETED_DOCS.setSelected ( true ) ;
        ck_SHOW_DELETED_DOCS.addActionListener ( this ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create the panel to hold our check box and views        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        allViewPanel = new JPanel () ;
        allViewPanel.setLayout ( new BorderLayout() ) ;
        allViewPanel.add ( ck_SHOW_DELETED_DOCS, BorderLayout.NORTH ) ;
        allViewPanel.add ( viewPanel ) ;
        allViewPanel.setBorder ( GUISystem.emptyBorder ) ;
        viewPanel.setShowDeletedDocsOnly ( ck_SHOW_DELETED_DOCS.isSelected() ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our tabs                                            */
        /*                                                           */
        /*-----------------------------------------------------------*/
        tabbedPane.addTab ( Str.getStr ( STR_FIND ), resultPanel ) ;
        tabbedPane.addTab ( Str.getStr ( STR_VIEWS ), new JScrollPane ( allViewPanel ) ) ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our controls                                        */       
        /*                                                           */
        /*-----------------------------------------------------------*/
        setLayout ( new BorderLayout() ) ;
        add ( tabbedPane ) ;
    }


    /**
      * Something happenened in the JTabbedPane
      */
    public void stateChanged ( ChangeEvent e ) {
        if ( tabbedPane.getSelectedComponent() == resultPanel )
            docHistPanel.setPanel ( "docEditPanel" ) ;
        else
            docHistPanel.setPanel ( "documents" ) ;
    }


    /**
      * This method will be called if the user hits one of our buttons
      */
    public void actionPerformed ( ActionEvent e ) {
        if ( e.getSource() == ck_SHOW_DELETED_DOCS ) {
            viewPanel.setShowDeletedDocsOnly ( ck_SHOW_DELETED_DOCS.isSelected() ) ;
            viewPanel.refreshCurrentView() ;
        }
    }


    /**
      * This method will set the tab that is showing. The parameter
      * passed must be either 'find' or 'view'
      */
    protected void showTab ( String tabName ) {
        if ( tabName.equals ( "find" ) )
            tabbedPane.setSelectedComponent ( resultPanel ) ;
        else if ( tabName.equals ( "view" ) )
            tabbedPane.setSelectedComponent ( allViewPanel ) ;
    }


    /**
      *  This method will return the instance of DocViewPanel that we are showing
      **/
    public DocViewPanel getDocViewPanel (){ 
       return ( viewPanel ) ;
    }
} /* DocHistNavPanel */
