package com.ibm.nzna.projects.qit.doc.history ;
/*-----------------------------------------------------------*/
/*   DocHistPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocHistPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.io.File ;
import java.util.Vector ;
import java.sql.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.doc.docedit.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This panel will display a document's history to the user. It will 
 * also allow the user to see changes in history
 *
 *
 * @version 1.0, 10/8/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocHistPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocHistPanel extends JPanel implements DocViewListener, SelectDocListener, QuestPanel, ActionListener, AppConst, DocConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESH_PREVIEW = "RP" ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ToolBarButton        pb_FIND         = null ;
   private DocEditPanel         docEditPanel    = null ;    
   private ActionButton         pb_CLOSE        = null ;
   private ActionButton         pb_RESTORE      = null ;
   private ActionButton         pb_SELECT       = null ;
   private DocHistNavPanel      docHistNavPanel = null ;
   private JToolBar             toolBar         = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean           draft             = false ;
   private DocumentRow       documentRow       = null ;
   private boolean           showChanges       = false ;
   private boolean           startDraft        = false ;
   private int               startDocInd       = 0 ;
   private QuestPanel        previousPanel     = null ;
   private CardLayout        cardLayout        = null ;
   private DocumentTable     documents         = null ;
   private DocViewRec        viewRec           = null ;
   private AppDefaultWin     parentWin         = null ;
   private String            currentPanel      = "docEditPanel" ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a blank DocHistPanel
    *  
    *  
    *             
    *  @param previousPanel The panel to show once this panel closes, this may be NULL
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public DocHistPanel ( QuestPanel previousPanel ) {
      this.previousPanel = previousPanel ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will initialize our variables
    *  to read in the document specified
    *  
    *
    *  @param docInd The Document Indicator
    *  @param draft TRUE if this is a draft, FALSE if not
    *  @param previousPanel The panel to show once this panel closes, this may be NULL
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocHistPanel ( int docInd, boolean draft, QuestPanel previousPanel ) {
      this.startDraft  = draft ;
      this.startDocInd = docInd ;
      this.previousPanel = previousPanel ;
   }



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return true
    *//*---------------------------------------------------------*/
   public boolean initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE        = new ActionButton ( Str.getStr ( STR_CLOSE ),   ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_RESTORE      = new ActionButton ( Str.getStr ( STR_RESTORE ), ImageSystem.getImageIcon ( this, ImageSystem.NEW ),   Str.getStr ( STR_RESTORE_HISTORY_DOC ) ) ;
      pb_SELECT       = new ActionButton ( Str.getStr ( STR_SELECT ),  ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),  Str.getStr ( STR_SELECT_HISTORY_DOC ) ) ;
      docEditPanel    = new DocEditPanel() ;
      docHistNavPanel = new DocHistNavPanel ( this ) ;
      pb_FIND         = new ToolBarButton ( Str.getStr ( STR_FIND ), ImageSystem.getImageIcon ( this, ImageSystem.SEARCH ), Str.getStr ( STR_FIND ) ) ;
      toolBar         = new JToolBar() ;
      documents       = new DocumentTable() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action listeners                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_FIND.addActionListener    ( this ) ;
      pb_CLOSE.addActionListener   ( this ) ;
      pb_RESTORE.addActionListener ( this ) ;
      pb_SELECT.addActionListener  ( this ) ;
      documents.addActionListener  ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECT.setVisible ( false ) ;
      toolBar.setBorder ( null ) ;
      toolBar.setOpaque ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our panel                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( cardLayout = new CardLayout ( ) ) ;
      toolBar.add ( pb_FIND ) ;
      add ( docEditPanel, "docEditPanel" ) ;
      add ( documents,  "documents" ) ;

      return( true ) ;
   }



   /**
     * This method will be called when a button is hit
     */
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_FIND                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_FIND )
         new SelectDocDlg ( false ).getInstance().addSelectDocListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_SELECT                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == documents ) || ( e.getSource() == pb_SELECT ) ) {
         DocumentRow documentRow = (DocumentRow) documents.getSelectedItem() ;

         if ( documentRow != null ) {
            docHistNavPanel.setDocInd ( documentRow.isDraft() ? documentRow.getDocumentInd() : documentRow.getExistingDocumentInd(), 
                                        documentRow.isDraft() ) ;
            setPanel ( "docEditPanel" ) ;
            docEditPanel.setDocument ( null ) ;
            docHistNavPanel.showTab ( "find" ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CLOSE ) {
         if ( Qit.getEnabled() )
            GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_RESTORE                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_RESTORE )
         restore() ;


   } /* actionPerformed */


   /*-----------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called if the user hits the
    *  refresh button on the task bar. It will simply
    *  call refreshList
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void refresh () {
      if ( currentPanel.equals ( "docEditPanel" ) )
         new Thread ( this, THREAD_REFRESH_PREVIEW ).start() ;
      else
         docHistNavPanel.getDocViewPanel().refreshCurrentView() ;
   }

   /**
     * This method will refresh only our documents
     */
   protected void refreshDocs () {
      viewRec.refresh() ;
   }


   /*-----------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread performs many operations. The operation
    *  done is determined by the thread name
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void run () {
      String operation = Thread.currentThread().getName() ;
             
      Qit.setEnabled ( false ) ;
      clearDocument() ;
      pb_CLOSE.setEnabled ( false ) ;
      pb_RESTORE.setEnabled ( false ) ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_REFRESH_PREVIEW                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( operation.equals ( THREAD_REFRESH_PREVIEW ) )
            refreshPreview() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }


      pb_CLOSE.setEnabled ( true ) ;
      pb_RESTORE.setEnabled ( true ) ;
      Qit.setEnabled ( true ) ;
   }



   /*-----------------------------------------------------------*/
   /* selectDocComplete
    */
   /**
    *    This method will respond to the user selecting a document in the SelectDocDlg
    *  
    *  
    *             
    *  @param id The ID of the dialog
    *  @param userData The UserData returned from the dialog
    *  @param 
    *
    *  @return 
    */
   public void selectDocComplete ( Vector vec ) {
      if ( ( vec != null ) && ( vec.size() > 0 ) ) {

         System.out.println ( "Select Doc Complete!" ) ;
         if ( vec.elementAt(0) instanceof DocumentRow ) {
            DocumentRow documentRow = (DocumentRow) vec.elementAt(0) ;

            System.out.println ( "instance of document row" ) ;
            if ( !documentRow.isDraft() )
               docHistNavPanel.setDocInd ( documentRow.getDocumentInd(), false ) ;
            else
               docHistNavPanel.setDocInd ( documentRow.getDocumentInd(), true ) ;

         } else if ( vec.elementAt(0) instanceof Bookmark ) {
            Bookmark bookmark = (Bookmark)vec.elementAt(0) ;

            System.out.println ( "instance of bookmark" ) ;
            if ( bookmark.getBookmarkType() == BOOKMARK_DOCUMENT )
               docHistNavPanel.setDocInd ( ((Integer)bookmark.getObject()).intValue(), false ) ;
         } else
            System.out.println ( "UNknown instance!" );
      }
   }



   /*-----------------------------------------------------------*/
   /* setDocRec
    */
   /**
    *    This method will set the doc rec we are editing
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setDocument ( DocumentRow documentRow ) {

      this.documentRow = documentRow ;

      if ( documentRow != null ) {
         this.draft = documentRow.isDraft() ;
         new Thread ( this, THREAD_REFRESH_PREVIEW ).start() ;
      } /* if */
   } /* setDocRec */



   /*------------------------------------------------------------*/
   /* refreshPreview
    */
   /**
    *    Sets the doc rec for all preview fields
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshPreview () {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      if ( documentRow != null ) {
         int rc = -1 ;

         parentWin.setStatus ( STR_PLEASE_WAIT ) ;
         System.out.println ("Reading Document:"+documentRow.getDocumentInd() ) ;
         rc = documentRow.readDocument ( false ) ;
         if ( rc == 0 ) {
            docEditPanel.setDocument ( documentRow.getDocument() ) ;
            docEditPanel.setEnabled ( false ) ;
         } else {
            LogSystem.log ( 1, "Document History could not read document. rc:"+rc ) ;
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
         }
         System.out.println ("Done Reading Document:"+documentRow.getDocumentInd()+"  rc:"+rc  ) ;

      }

      parentWin.setStatus ( Str.getStr ( STR_DONE ), 2 ) ;
   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when we are
    *  deselected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected() {
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
   }

   /*------------------------------------------------------------*/
   /* panelSselected
    */
   /**
    *    This method will be called when we are
    *  selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected() {
      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 1 ) ;
   }




   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText() {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText() {
      return( false ) ;
   }
   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText() {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent() {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default nav panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( docHistNavPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when we need
    *  to be closed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE - Ok to close, FALSE if not ok to close
    *//*---------------------------------------------------------*/
   public boolean stop () {
      if ( Qit.getEnabled() ) {
         panelDeselected() ;
         removeAll() ;
         return( true ) ;
      }

      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will return TRUE
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( Qit.getEnabled() ) ;
   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we have
    *  been shown
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;         
      parentWin.addActionComponent ( this, pb_RESTORE ) ;
      parentWin.addActionComponent ( this, pb_SELECT ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the toolbar on                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addToolbarButton ( toolBar, 1 ) ;

      if ( startDocInd != 0 )
         docHistNavPanel.setDocInd ( startDocInd, startDraft ) ;

      new DocHistWhatsNew ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our small icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_HISTORY_SMALL ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_DOCUMENT_HISTORY ) ) ;
   }


   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      if ( documentRow != null ) {
         Bookmark bookmark = new Bookmark() ;
         bookmark.setDescript ( documentRow.getTitle() ) ;
         bookmark.setObject ( new Integer ( documentRow.getExistingDocumentInd() ),
                              BOOKMARK_DOCUMENT ) ;
         bookmark.updateRecStatus ( QITRec.REC_INSERT ) ;
         new AddBookmarkDlg ( GUISystem.getParentDefWin ( this ), bookmark ) ;

      } else
         GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_MUST_SELECT_ITEM_TO_BOOKMARK ) ) ;


      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* restore
    */
   /**
    *    This method will restore a document to it's
    *  draft state if no other draft is open for the document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void restore () {

      GUISystem.getParentDefWin ( this ).setStatus ( STR_RESTORING ) ;

      if ( documentRow != null ) {
         DocumentDraft draft     = (DocumentDraft)documentRow.getDocument() ;
         SQLMethod     sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "DocHistPanel.restore", 5 ) ;
         boolean       restore   = false ;

         try {
            Statement stmt     = sqlMethod.createStatement() ;
            String    comments = null ;                                               


            /*-------------------------------------------------------------*/
            /*                                                             */
            /*   First, make sure that there is not an open draft already  */
            /*                                                             */
            /*-------------------------------------------------------------*/
            if ( stmt != null ) {
               ResultSet rs = stmt.executeQuery ( "SELECT DBUSER, CHANGEDTIME "+
                                                  "FROM QUEST.DOCUMENTS "+
                                                  "WHERE STOPDATE IS NULL AND "+
                                                  "EXISTINGDOCIND = "+documentRow.getExistingDocumentInd() ) ;

               if ( rs.next() ) {
                  String userId    = rs.getString(1).trim() ;
                  String timeStamp = DateSystem.prettyDateFromStamp ( rs.getString(2).trim() ) ;


                  GUISystem.printBox ( Str.getStr ( STR_INFO ), 
                                       Str.getStr ( STR_CANNOT_RESTORE_HISTORY_DOC )+" "+
                                       UserSystem.getNameFromUserId ( userId )+" "+
                                       Str.getStr ( STR_ON )+" "+
                                       timeStamp ) ;
                  restore = false ;
               } else
                  restore = true ;

               rs.close() ;



               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, if we still can, restore the draft from history    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( restore ) {
                  int workInd = 0 ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   First, read in the comments from the previous edit      */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  rs = stmt.executeQuery ( "SELECT COMMENT, WORKIND "+
                                           "FROM QUEST.DOCUMENTS "+
                                           "WHERE DOCIND = "+documentRow.getDocumentInd() ) ;

                  if ( rs.next() ) {
                     comments = rs.getString(1).trim() ;
                     workInd  = rs.getInt(2) ;
                  } else
                     comments = "" ;

                  rs.close() ;



                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Add a comment                                           */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  comments+="\n<"+
                            Str.getStr ( STR_DOC_RESTORED )+" "+
                            Str.getStr ( STR_BY )+" "+
                            UserSystem.getUserName()+" "+
                            Str.getStr ( STR_ON )+" "+
                            DateSystem.getDate ( CDate.DATE_DATE )+">\n" ;

                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   If the draft was for a delete of a document, change     */
                  /* that to an Edit of a document                             */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  if ( draft.getWorkRequired() == TypeList.WORKREQ_DELETE )
                     draft.setWorkRequired ( TypeList.WORKREQ_EDIT ) ;

                  stmt.executeUpdate ( "UPDATE QUEST.DOCUMENTS "+
                                       "SET   WORKIND     = "+workInd+", "+
                                       "      STOPDATE    = NULL, "+
                                       "      COMMENT     = '"+comments+"', "+
                                       "      REQUESTOR   = '"+UserSystem.getUserId()+"', "+
                                       "      DBUSER      = '"+UserSystem.getUserId()+"', "+
                                       "      CHANGEDTIME = CURRENT TIMESTAMP "+
                                       "WHERE DOCIND      = "+documentRow.getDocumentInd() ) ;

                  GUISystem.printBox ( STR_INFO, STR_DOC_HISTORY_RESTORED ) ;
               } /* if restore */


            } /* if Stmt */

         } catch ( Exception e ) {
            sqlMethod.rollBack () ;
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
         }

         sqlMethod.close() ;
      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

      GUISystem.getParentDefWin ( this ).setStatus ( null ) ;
   } /* restore */


   /**
     *  This method will set what panel we are showing. The panel
     * name passed must be:<br><br>
     *
     * documents  - Shows the document list<br>
     * docEditPanel - Shows the Document being viewed<br>
     **/
   protected void setPanel ( String panelName ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the action buttons                               */
      /*                                                           */
      /*-----------------------------------------------------------*/

      /* If we are showing documents, hide the 'Restore' button and
         show the 'Select' button  */
      if ( panelName.equals ( "documents" ) ) {
         pb_RESTORE.setVisible ( false ) ;
         pb_SELECT.setVisible ( true ) ;
      }

      /* If we are showing the Tabbed Pane, hide the 'Select' button and
         show the 'Restore' button  */
      else if ( panelName.equals ( "docEditPanel" ) ) {
         pb_RESTORE.setVisible ( true ) ;
         pb_SELECT.setVisible ( false ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, show the panel                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cardLayout.show ( this, panelName ) ;
      currentPanel = panelName ;
   }


   /**
     * This methodw will be called when a  view has started to refresh
     */
   public void docViewStarted ( DocViewRec viewRec ) {
      this.viewRec = viewRec ;
      parentWin.setStatus ( STR_READING ) ;
   }

   /**
     * This method will be called when a view has refreshed
     */
   public void docViewStopped ( DocViewRec viewRec ) {
      documents.setDocuments ( viewRec.getResults() ) ;
      setPanel ( "documents" ) ;
      this.viewRec = viewRec ;
      parentWin.setStatus ( null ) ;
   }


   protected void clearDocument () {
      docEditPanel.setDocument( null );
   }
} /* DocHistPanel */
