package com.ibm.nzna.projects.qit.doc.history ;
/*-----------------------------------------------------------*/
/*   DocHistSQL                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocHistSQL                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.Vector ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This class holds static routines for manipulating history 
 *
 *
 * @version 1.0, 10/8/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   DocHistSQL                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocHistSQL extends Object implements DocConst, AppConst {


   /*-----------------------------------------------------------*/
   /* readHistory
    */
   /**
    *    This method will read a document's history
    *  
    *  
    *             
    *  @param docInd The Existing Document Ind to check history for
    *  @param 
    *  @param 
    *
    *  @return Vector The vector of docRecs
    */
   public static Vector readHistory ( AppDefaultWin win, int docInd ) {
      String    rtnName = "readHistory" ;
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      int       sqlCode = 0 ;
      Vector    retVec  = new Vector ( 10, 1 ) ;

      win.setStatus ( STR_PLEASE_WAIT ) ;

      if ( sql != null ) {
         try {
            DocHistoryRec newDoc = null ;
            Statement   stmt     = sql.createStatement() ;

            if ( stmt != null ) {
               ResultSet rs = stmt.executeQuery ( "SELECT A.DOCIND, A.DBUSER, A.STOPDATE, B.DOCIND "+
                                                  "FROM QUEST.DOCUMENTS A, TIGRIS.DOCUMENTS B "+
                                                  "WHERE A.EXISTINGDOCIND = "+docInd+" AND "+
                                                  "A.STOPDATE IS NOT NULL AND B.DOCIND = A.EXISTINGDOCIND" ) ;

               while ( rs.next() ) {
                  
                  newDoc = new DocHistoryRec ( rs.getInt(1),
                                               rs.getString(3),
                                               UserSystem.getNameFromUserId ( rs.getString(2).trim() ),
                                               rs.getInt(4) ) ;

                  retVec.addElement ( newDoc ) ;
               }

               rs.close() ;
            }

         } catch ( Exception e ) {
            sql.rollBack() ;
            LogSystem.log ( 1, e ) ;
         }
      } /* if */

      sql.close() ;
      win.setStatus ( null ) ;

      return( retVec ) ;
   } /* readHistory */






   /*------------------------------------------------------------*/
   /* readCommentHistory
    */
   /**
    *    This method will read the comment history of a draft
    *  . only the last 5 comments will be returned ni the string
    *  
    *
    *  @param docInd The Document Request INdicator
    *  @param existingDocInd The Document Indicator for the active, existing document
    *  @param None
    *
    *  @return String comments
    *//*---------------------------------------------------------*/
   protected static String readCommentHistory ( int docInd, int existingDocInd ) {
      SQLMethod sqlMethod      = new SQLMethod ( DatabaseSystem.QUEST, "readCommentHistory", 5 ) ;
      String    commentHistory = "" ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT STOPDATE, COMMENT "+
                                              "FROM QUEST.DOCUMENTS "+
                                              "WHERE DOCIND < "+docInd+" AND "+
                                              "      EXISTINGDOCIND = "+existingDocInd+" AND "+
                                              "      STOPDATE IS NOT NULL AND "+
                                              "      RECYCLED       = 'N' AND "+
                                              "      TEMPLATE       = 'N' "+
                                              "ORDER BY STOPDATE DESC FETCH FIRST 5 ROWS ONLY" ) ;      

         while ( rs.next() ) 
            commentHistory+=rs.getString(1).trim()+"\n"+rs.getString(2).trim()+"\n\n\n" ;

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e , false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;

      }

      sqlMethod.close() ;
      return ( commentHistory ) ;
   }



} /* DocHistSQL */

