package com.ibm.nzna.projects.qit.doc.history ;                 
/*-----------------------------------------------------------*/
/*   HistoryResultPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HistoryResultPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;/**
 * This panel shows the history results for a particular  
 * document. This panel includes a MultiList of all published
 * versions of a document.It also houses a 'Find' feature
 * so the user can easily enter in the document indicator
 * of an Active or Draft document.
 *
 *
 * @version 1.0, 4/26/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HistoryResultPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class HistoryResultPanel extends JPanel implements Runnable, AppConst, ActionListener, MouseListener  {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_READ_HISTORY = "READ_HIST" ;




   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextField ef_DOCIND           = null ;
   private JLabel     st_DOCIND           = null ;
   private JLabel     st_PUBLISHDATES     = null ;
   private MultiList  historyList         = null ;
   private DButton    pb_GO               = null ;      
   private JCheckBox  ck_REQUEST          = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DocHistPanel docHistPanel      = null ;
   private boolean      request           = false ;
   private static final String[] listTitle = { Str.getStr ( STR_DOC_IND ), 
      Str.getStr ( STR_STOP_DATE ),
      Str.getStr ( STR_LAST_TOUCHED_BY )} ;







   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor simply assigns our DocHistPanel
    *  property
    *  
    *
    *  @param docHistPanel The DocHistPanel that this nav panel is tied to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HistoryResultPanel ( DocHistPanel docHistPanel ) {
      this.docHistPanel = docHistPanel ;
   }



   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our nav list
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_GO           = new DButton ( Str.getStr ( STR_GO ) ) ;
      ef_DOCIND       = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 7 ), "", 0 ) ;
      st_DOCIND       = new JLabel ( Str.getStr ( STR_DOC_IND ) ) ;
      historyList     = new MultiList ( GUISystem.getFontUtil() ) ;
      st_PUBLISHDATES = new JLabel ( Str.getStr ( STR_PUBLISH_DATES ) ) ;        
      ck_REQUEST      = new JCheckBox ( Str.getStr ( STR_DRAFT ) ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      historyList.setBorder ( null ) ;
      historyList.setColumnHeadings ( listTitle ) ;
      historyList.setColumnHeadingsBackground ( Color.white ) ;
      historyList.setColumnWidth ( 0, 75 ) ;
      historyList.setColumnWidth ( 1, 75 ) ;
      historyList.setColumnWidth ( 2, 175 ) ;
      ck_REQUEST.setOpaque ( false ) ;
      historyList.setOpaque ( false ) ;
      ef_DOCIND.setOpaque ( true ) ;
      historyList.setBackground ( ThemeSystem.getColor ( "NAVPANEL_BG_COLOR" ) ) ;
      historyList.setOddLinesColor ( ThemeSystem.getColor ( "NAVPANEL_BG_COLOR" ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      historyList.getJList().addMouseListener ( this ) ;
      ef_DOCIND.addActionListener ( this ) ;
      pb_GO.addActionListener ( this ) ;  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_DOCIND ) ;
      add ( ef_DOCIND ) ;
      add ( pb_GO ) ;
      add ( ck_REQUEST ) ;
      add ( st_PUBLISHDATES ) ;
      add ( historyList ) ;


   } /* initialize */






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;
      Dimension size      = getSize() ;


      st_DOCIND.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      ef_DOCIND.setBounds ( x, y, size.width-(x*2)-55, rowHeight ) ;
      pb_GO.setBounds ( size.width-(x*2)-50, y, 45, rowHeight ) ;
      y+=rowHeight ;

      ck_REQUEST.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight*2 ;

      st_PUBLISHDATES.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      historyList.setBounds ( x, y, size.width-(x*2), size.height-y-10 ) ;
   }


   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing. Basically we will just give focus
    *  to the ef_DOCIND field
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      ef_DOCIND.requestFocus() ;
   }
                         




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are waiting for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_GO  or ef_DOCIND                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == pb_GO ) || ( e.getSource() == ef_DOCIND ) ) {
         try {
            setDocInd ( new Integer ( ef_DOCIND.getText() ).intValue(),
                        ck_REQUEST.isSelected() ) ;
         } catch ( NumberFormatException ex ) {
            /* Do nothing, somehow they entered alphas */
         }
      }

   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when an item in the
    *  list has been selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseClicked ( MouseEvent e ) {
      DocHistoryRec historyRec = (DocHistoryRec) historyList.getSelectedItem() ;

      if ( historyRec != null )
         docHistPanel.setDocument ( new DocumentRow ( ""+historyRec.getDocInd (), ""+historyRec.getExistingDocInd(), true ) ) ;
   }

   public void mouseEntered ( MouseEvent e ) {}
   public void mouseExited ( MouseEvent e ) {}
   public void mousePressed ( MouseEvent e ) {}
   public void mouseReleased ( MouseEvent e ) {}






   /*------------------------------------------------------------*/
   /* setDocInd
    */
   /**
    *    This method will set the doc indicator that 
    *  we are displaying. This will also kick off the thread
    *  that reads in all of the history of the document
    *
    *  @param docInd The DocInd to set to
    *  @param request TRUE = This docInd represents a request
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setDocInd ( int docInd, boolean request ) {
      ef_DOCIND.setText ( ""+docInd ) ;
      this.request = request ;
      ck_REQUEST.setSelected ( request ) ;
      historyList.removeAll() ;
      docHistPanel.clearDocument() ;
      new Thread ( this, THREAD_READ_HISTORY ).start() ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread does different operations. The operation
    *  done depends on the name of the thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_READ_HISTORY                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_READ_HISTORY ) ) {
         readHistory ( new Integer ( ef_DOCIND.getText() ).intValue() ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* readHistory
    */
   /**
    *    This method will read and display the list
    *  of documents that match
    *  
    *
    *  @param docInd The Doc Ind to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readHistory ( int docInd ) {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      pb_GO.setEnabled ( false ) ;
      ef_DOCIND.setEnabled ( false ) ;
      parentWin.setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;

      try {

         /*-----------------------------------------------------------------*/
         /*                                                                 */
         /*   If this is a request, we need to get the published doc ind    */
         /*                                                                 */
         /*-----------------------------------------------------------------*/
         if ( request ) {
            int existingDocInd = DocumentSQL.getExistingDocInd ( docInd ) ;
            ef_DOCIND.setText ( ""+existingDocInd ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have a valid doc indicator, read in the list      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( docInd != 0 ) {
            Vector historyVec = DocHistSQL.readHistory ( GUISystem.getParentDefWin ( this ), docInd ) ;                       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this vector is good, continue                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( historyVec != null ) && ( historyVec.size() > 0 ) ) {
               historyList.removeAll() ;
               historyList.add ( historyVec ) ;
            } else
               GUISystem.printBox ( STR_INFO, STR_NO_HISTORY_FOUND ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ,false ) ;
      }

      pb_GO.setEnabled ( true ) ;
      ef_DOCIND.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;

   }





   /*------------------------------------------------------------*/
   /* getDocInd
    */
   /**
    *    This method will return what docInd is
    *  currently displayed in the doc indicator
    *  entry field
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   protected int getDocInd () {
      try {
         return( new Integer ( ef_DOCIND.getText() ).intValue() ) ;
      } catch ( NumberFormatException e ) {
      }

      return( 0 ) ;
   }

} /* HistoryResultPanel */
