package com.ibm.nzna.projects.qit.doc.model ;     
/*-----------------------------------------------------------*/
/*   DocumentTableModel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocumentTableModel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.* ;
import javax.swing.table.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.*;
/**
 * A Model for a JTable that contains a list of Documents. This
 * model uses a Vector of Documents that can be changed with
 * the 'setDocuments' method.
 *
 *
 * @version 1.0, 4/3/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DocumentTableModel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocumentTableModel extends AbstractTableModel implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector documents = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default Constructor which
    *  does nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentTableModel () {
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates a new model
    *  and assigns the data. Please note that
    *  you will have to call JTable.setModel ( thisModel ) to show the data
    *
    *  @param documents The Vector of DocumentList Objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DocumentTableModel ( Vector documents ) {
      setDocuments ( documents ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    Returns the number of columns we 
    *  display for documents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getColumnCount () {
      return( 5 ) ;
   }




   /*------------------------------------------------------------*/
   /* getRowCount
    */
   /**
    *    If we have any data, returns the number
    *  of documents we display
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getRowCount () {
      if ( documents != null )
         return( documents.size() ) ;
      return( 0 ) ;
   }




   /*------------------------------------------------------------*/
   /* getValueAt
    */
   /**
    *    This method will return the Object that
    *  should be displayed in the row/column
    *  
    *
    *  @param row The row to retrieve data for
    *  @param column The column to retrieve data for
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getValueAt ( int row, int column ) {
      return( ((MultiListRow)documents.elementAt(row)).getColumnData ( column ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getColumnObject
    */
   /**
    *    This method will return what type of
    *  object should be displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Class getColumnClass(int columnIndex) {
      try {
         return( getValueAt ( 0, columnIndex ).getClass() ) ;
      } catch ( Exception e ) {
      }

      return( Object.class ) ;
   }





   /*------------------------------------------------------------*/
   /* setDocuments
    */
   /**
    *    This method will set the documents that
    *  this model represents. Please note that
    *  you must call 'setModel' on the JTable to reflect the changes
    *
    *  @param documents The Vector of DocumentList Objects to represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDocuments ( Vector documents ) {
      this.documents = documents ;
   }





   /*------------------------------------------------------------*/
   /* getColumnName
    */
   /**
    *    This method will return the name of the
    *  column specified.
    *  
    *
    *  @param column The column index to return
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getColumnName ( int column ) {
      switch ( column ) {
         case 1 :
            return( Str.getStr ( STR_DOC_IND ) ) ;
         case 2 : 
            return( Str.getStr ( STR_TITLE ) ) ;
         case 3 : 
            return( Str.getStr ( STR_LOTUS_NOTES_ID ) ) ;
         case 4 : 
            return( Str.getStr ( STR_LAST_TOUCHED_ON ) ) ;
         default :
            return( "|" ) ;
      }
   }

} /* DocumentTableModel */
