package com.ibm.nzna.projects.qit.doc.storedProc.docReader ;
/*-----------------------------------------------------------*/
/*   DocReader                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocReader                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This Stored Procedure reads in ALL information about a single document 
 *
 *
 * @version 1.0, 9/15/1999 
 * @author  David Lentz
 */
import java.sql.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Blob ;
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   DocReader                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocReader extends StoredProc {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String LOG_FILE            = "/tmp/DocReader.log" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   ReadLevel                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int READLEVEL_ALL          = 0 ; // Reads all document information
   private static final int READLEVEL_PREVIEW      = 1 ;   // Reads only preview information


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Program Vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int              sqlCode        = 0 ;          // The SQLCODE we will return
   private DocReaderBuffer  buffer         = null ;       // The Buffer to return data in
   private Connection       con            = null ;       // Our connection to DB/2
   private int              docInd         = 0 ;          // The document indicator we are working on
   private int              existingDocInd = 0 ;          // The document indicator we are working on
   private int              request        = 0 ;          // 1 = Request, 0 = Existing Document
   private String           qualifier      = "TIGRIS" ;   // The qualifier in the database to read


   /*------------------------------------------------------------*/
   /* readDocument
    */
   /**
    *    This method will be called by DB/2 when
    *  we are called by a client. It will process reading
    *  all of the document information
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*--------------------------------------------------------*/
   public void readDocument ( int docInd, int existingDocInd, int request, int readLevel, Blob results, int sqlCode, String errorString ) throws Exception  {                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the DB/2 Connection                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      con = getConnection() ;      

      try {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Copy over our vars                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         this.docInd  = docInd ;
         this.request = request ;                                                      


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our qualifier to 'QUEST' if this is a request       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( request == 1 )
            qualifier = "QUEST" ;
         else
            this.docInd = existingDocInd ;



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the buffer we will send back                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         buffer = new DocReaderBuffer() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in the Information from TIGRIS or QUEST.DOCUMENTS  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( request == 0 )
            sqlCode = readDocumentInfo () ;
         else
            sqlCode = readDocumentRequestInfo() ;

         if ( ( sqlCode == 0 ) && ( readLevel == READLEVEL_ALL ) )
            sqlCode = readDocProperties() ;

         if ( ( sqlCode == 0 ) && ( readLevel == READLEVEL_ALL ) )
            sqlCode = readDocGeo() ;

         if ( ( sqlCode == 0 ) && ( readLevel == READLEVEL_ALL ) )
            sqlCode = readDocCountry() ;

         if ( ( sqlCode == 0 ) && ( readLevel == READLEVEL_ALL ) )
            sqlCode = readDocProducts() ;

         if ( ( sqlCode == 0 ) && ( readLevel == READLEVEL_ALL ) )
            sqlCode = readDocQueryText() ;

         if ( ( sqlCode == 0 ) && ( readLevel == READLEVEL_ALL ) ) {
            if ( ( buffer.docTypeInd == 9 ) || ( buffer.docTypeInd == 8 ) )
               sqlCode = readDocMedia() ;
         }

         if ( ( sqlCode == 0 ) && ( readLevel == READLEVEL_ALL ) )
            sqlCode = readDocLinks() ;

         /* Only read the body if this document is not an SSD */
         if ( ( sqlCode == 0 ) && ( buffer.docTypeInd != 9 ) )
            sqlCode = readDocBody() ;

         if ( ( sqlCode == 0 ) && ( readLevel == READLEVEL_ALL ) )
            sqlCode = readDocCategory() ;

         if ( sqlCode == 0 )
            sqlCode = readCommentHistory() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create a blob to return data                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         results = Lob.newBlob() ;
         setObjectInBlob ( results, buffer ) ;

         set ( 5, results ) ;
         set ( 6, sqlCode ) ;

      } catch ( Exception e ) {
         StringWriter    sw      = new StringWriter () ;
         PrintWriter     pw      = new PrintWriter ( sw ) ;

         /* Get the stackTrace */
         e.printStackTrace ( pw ) ;
         set(7, e.toString() ) ;
      }

      con.commit() ;
      con.close() ;

   }



   /*------------------------------------------------------------*/
   /* setObjectInBlob
    */
   /**
    *    This method will copy a serialized object into a blob.
    *  
    *
    *  @param blob The Blob
    *  @param object The Object to serialize
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setObjectInBlob ( Blob blob, Object object ) throws Exception {
      ByteArrayOutputStream outStream = new ByteArrayOutputStream() ;
      ObjectOutputStream    s         = new ObjectOutputStream ( blob.getOutputStream() ) ;

      s.writeObject ( object ) ;
      s.flush() ;
      s.close() ;
   }





   /*------------------------------------------------------------*/
   /* readDocumentInfo
    */
   /**
    *    This method will read all information from
    *  either the QUEST.DOCUMENTS or
    *  TIGRIS.DOCUMENTS table.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return sqlCode 
    *//*---------------------------------------------------------*/
   private int readDocumentInfo () throws Exception {
      Statement stmt    = con.createStatement() ;
      String    sql     = null ;
      ResultSet rs      = null ;      
      String    addInfo = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT DOCTYPEIND,   DOCCLASSIND, TITLE,     OWNER,         REVIEWPERIOD, "+
            "       AUTHIND,      WEBABLE,     WEBONLY,   FAXABLE,       FLASHNEWS,    "+
            "       FLASHDATE,    FAQ,         EMAILABLE, EMAILABLEDATE, SUMMARY,      "+
            "       METRICIND,    ADDINFO,     PARTNUM,   PUBTYPE,       LNDOCID,      "+
            "       SOURCE,       MAINTLOCK,   SHOW,      WORKIND,       DBUSER,       "+
            "       CHANGEDTIME,  KNOWLEDGEBASE                                        "+
            "FROM   TIGRIS.DOCUMENTS                                                   "+
            "WHERE  DOCIND   = "+docInd+" AND                                          "+
            "       RECYCLED = 'N'                                                     "+
            "       FOR FETCH ONLY                                                     " ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute It!                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Copy over the data                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rs.next() ) {
         buffer.docTypeInd       = rs.getInt(1) ;
         buffer.docClassInd      = rs.getInt(2) ;
         buffer.title            = rs.getString(3).trim() ;
         buffer.owner            = rs.getString(4).trim() ;
         buffer.reviewPeriod     = rs.getInt(5) ;
         buffer.authInd          = rs.getInt(6) ;
         buffer.webable          = rs.getString(7).trim() ;
         buffer.webonly          = rs.getString(8).trim() ;
         buffer.faxable          = rs.getString(9).trim() ;
         buffer.flash            = rs.getString(10).trim() ;
         buffer.flashDate        = rs.getString(11) ;
         buffer.faq              = rs.getString(12) ;
         buffer.emailable        = rs.getString(13).trim() ;
         buffer.emailableDate    = rs.getString(14) ;
         buffer.summary          = rs.getString(15) ;
         buffer.metricInd        = rs.getInt(16) ;
         addInfo                 = rs.getString(17) ;
         buffer.partNumber       = rs.getString(18) ;
         buffer.pubTypeInd       = rs.getInt(19) ;
         buffer.lnDocId          = rs.getString(20) ;
         buffer.source           = rs.getString(21) ;
         buffer.maintLock        = rs.getString(22).trim() ;
         buffer.show             = rs.getString(23).trim() ;
         buffer.workInd          = rs.getInt(24) ;
         buffer.dbUser           = rs.getString(25).trim() ;
         buffer.changedTime      = rs.getString(26).trim() ;
         buffer.knowledgeBase    = rs.getString(27).trim() ;
         buffer.existingDocInd   = docInd ;
         buffer.docInd           = 0 ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Clean the Nullable strings                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( buffer.emailableDate != null )
            buffer.emailableDate = buffer.emailableDate.trim() ;
         if ( buffer.flashDate != null )
            buffer.flashDate = buffer.flashDate.trim() ;
         if ( addInfo != null )
            addInfo = addInfo.trim() ;
         if ( buffer.partNumber != null )
            buffer.partNumber = buffer.partNumber.trim() ;
         if ( buffer.lnDocId != null )
            buffer.lnDocId = buffer.lnDocId.trim() ;
         if ( buffer.source != null )
            buffer.source = buffer.source.trim() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Break up the Additional Info Field                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         buffer.addInfoVec = new Vector ( 1, 1 ) ;
         buffer.breakStringIntoVec ( addInfo, buffer.addInfoVec ) ;

      } else
         return( 100 ) ;

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }




   /*------------------------------------------------------------*/
   /* readDocumentRequestInfo
    */
   /**
    *    This method will read all information from
    *  either the QUEST.DOCUMENTS or
    *  TIGRIS.DOCUMENTS table.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return sqlCode 
    *//*---------------------------------------------------------*/
   private int readDocumentRequestInfo () throws Exception {
      Statement stmt    = con.createStatement() ;
      String    sql  
      = null ;
      ResultSet rs      = null ; 
      String    addInfo = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT DOCTYPEIND,   DOCCLASSIND, TITLE,     OWNER,         REVIEWPERIOD,   "+
            "       AUTHIND,      WEBABLE,     WEBONLY,   FAXABLE,       FLASHNEWS,      "+
            "       FLASHDATE,    FAQ,         EMAILABLE, EMAILABLEDATE, SUMMARY,        "+
            "       METRICIND,    ADDINFO,     PARTNUM,   PUBTYPE,       LNDOCID,        "+
            "       SOURCE,       MAINTLOCK,   SHOW,      WORKIND,       DBUSER,         "+
            "       CHANGEDTIME,  STOPDATE,    STARTDATE, REQUESTOR,     EXISTINGDOCIND, "+
            "       COMMENT,      KNOWLEDGEBASE                                          "+
            "FROM   QUEST.DOCUMENTS                                                      "+
            "WHERE  DOCIND   = "+docInd+" AND                                            "+
            "       RECYCLED = 'N'                                                       "+
            "       FOR FETCH ONLY                                                       " ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute It!                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Copy over the data                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rs.next() ) {
         buffer.docTypeInd       = rs.getInt(1) ;
         buffer.docClassInd      = rs.getInt(2) ;
         buffer.title            = rs.getString(3).trim() ;
         buffer.owner            = rs.getString(4).trim() ;
         buffer.reviewPeriod     = rs.getInt(5) ;
         buffer.authInd          = rs.getInt(6) ;
         buffer.webable          = rs.getString(7).trim() ;
         buffer.webonly          = rs.getString(8).trim() ;
         buffer.faxable          = rs.getString(9).trim() ;
         buffer.flash            = rs.getString(10).trim() ;
         buffer.flashDate        = rs.getString(11) ;
         buffer.faq              = rs.getString(12) ;
         buffer.emailable        = rs.getString(13).trim() ;
         buffer.emailableDate    = rs.getString(14) ;
         buffer.summary          = rs.getString(15) ;
         buffer.metricInd        = rs.getInt(16) ;
         addInfo                 = rs.getString(17) ;
         buffer.partNumber       = rs.getString(18) ;
         buffer.pubTypeInd       = rs.getInt(19) ;
         buffer.lnDocId          = rs.getString(20) ;
         buffer.source           = rs.getString(21) ;
         buffer.maintLock        = rs.getString(22).trim() ;
         buffer.show             = rs.getString(23).trim() ;
         buffer.workInd          = rs.getInt(24) ;
         buffer.dbUser           = rs.getString(25).trim() ;
         buffer.changedTime      = rs.getString(26).trim() ;
         buffer.startDate        = rs.getString(28) ;
         buffer.stopDate         = rs.getString(27) ;
         buffer.requestor        = rs.getString(29) ;
         buffer.existingDocInd   = rs.getInt(30) ;
         buffer.comment          = rs.getString(31) ;
         buffer.knowledgeBase    = rs.getString(32).trim() ;
         existingDocInd          = buffer.existingDocInd ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Clean the Nullable strings                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( buffer.emailableDate != null )
            buffer.emailableDate = buffer.emailableDate.trim() ;
         if ( buffer.flashDate != null )
            buffer.flashDate = buffer.flashDate.trim() ;
         if ( addInfo != null )
            addInfo = addInfo.trim() ;
         if ( buffer.partNumber != null )
            buffer.partNumber = buffer.partNumber.trim() ;
         if ( buffer.lnDocId != null )
            buffer.lnDocId = buffer.lnDocId.trim() ;
         if ( buffer.source != null )
            buffer.source = buffer.source.trim() ;
         if ( buffer.startDate != null )
            buffer.startDate = buffer.startDate.trim() ;
         if ( buffer.stopDate != null )
            buffer.stopDate = buffer.stopDate.trim() ;
         if ( buffer.requestor != null )
            buffer.requestor = buffer.requestor.trim() ;
         if ( buffer.comment != null )
            buffer.comment = buffer.comment.trim() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Break up the Additional Info Field                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         buffer.addInfoVec = new Vector ( 1, 1 ) ;
         buffer.breakStringIntoVec ( addInfo, buffer.addInfoVec ) ;


      } else
         return( 100 ) ;

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }







   /*------------------------------------------------------------*/
   /* readDocProperties
    */
   /**
    *    This method will read in all of the Doc Properties
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocProperties() throws Exception {
      Statement     stmt           = con.createStatement () ;
      String        sql            = null ;
      ResultSet     rs             = null ;                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize Some Vars                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buffer.descript = "" ;
      buffer.countryComment = "" ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT PROPERTYIND, ORDERNUM, VALUE "+
            "FROM "+qualifier+".DOCPROPERTIES "+
            "WHERE DOCIND = "+docInd+" "+
            "ORDER BY ORDERNUM "+
            "FOR FETCH ONLY" ;            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the statement                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;

      while ( rs.next() ) {
         if ( rs.getString(3) != null ) {
            switch ( rs.getInt(1) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Lotus Notes Universal Id                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               case 1 :
                  buffer.universalNotesId = rs.getString(3).trim() ;
                  break ;

               case 2 :
                  buffer.descript+= ( rs.getString(3) ) ;
                  break ;

               case 3 :
                  buffer.releaseDate = rs.getString(3).trim() ;
                  break ;

               case 4 :
                  buffer.countryComment+= ( rs.getString(3).trim() ) ;
                  break ;

               case 5 :
                  buffer.createDate = rs.getString(3).trim() ;
                  break ;

               case 6 :
                  buffer.lastUpdateCTSTips = rs.getString(3).trim() ;
                  break ;

               case 7 :
                  buffer.url = rs.getString(3).trim() ;
                  break ;

               case 8 :
                  buffer.version = rs.getString(3).trim() ;
                  break ;

            }
         }
      }

      if ( (buffer.descript != null ) && ( buffer.descript.length() > 0 ) )
         buffer.descript = buffer.descript.trim() ;

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* readDocCountry
    */
   /**
    *    This method will read all document country
    *  information
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocCountry () throws Exception {
      Statement stmt = con.createStatement() ;
      ResultSet rs   = null ;
      String    sql  = null ;                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our country vector                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buffer.countryVec = new Vector ( 10, 10 ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT COUNTRYCODEIND FROM "+qualifier+".DOCCOUNTRY "+
            "WHERE DOCIND = "+docInd+" FOR FETCH ONLY" ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the SQL                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery(sql) ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read all values                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {
         buffer.countryVec.addElement ( new Integer ( rs.getInt ( 1 ) ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close up                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }

   /*------------------------------------------------------------*/
   /* readDocGeo
    */
   /**
    *    This method will read all document geography
    *  information
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocGeo () throws Exception {
      Statement stmt = con.createStatement() ;
      ResultSet rs   = null ;
      String    sql  = null ;                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our geo vector                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buffer.geoVec = new Vector ( 10, 10 ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT GEOIND FROM "+qualifier+".DOCGEO "+
            "WHERE DOCIND = "+docInd+" FOR FETCH ONLY" ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the SQL                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery(sql) ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read all values                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {
         buffer.geoVec.addElement ( new Integer ( rs.getInt ( 1 ) ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close up                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }






   /*------------------------------------------------------------*/
   /* readDocProducts
    */
   /**
    *  This method will populate the prodVec in the DocReaderBuffer
    * with all products associated to this document.  
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocProducts () throws Exception {
      Statement           stmt    = con.createStatement() ;
      ResultSet           rs      = null ;
      DocReaderProdBuffer readRec = null ;
      String              sql     = "SELECT A.PRODINT, A.BRANDGROUPIND, A.MACHINE, A.MODEL, A.PRODNUM "+
                                    "FROM "+qualifier+".PRODDOCS B, "+
                                    "     TIGRIS.PRODUCTS A "+
                                    "WHERE A.PRODINT = B.PRODINT AND "+
                                    "      B.DOCIND  = "+docInd+" "+
                                    " FOR FETCH ONLY" ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the vector                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buffer.prodVec = new Vector ( 10, 10 ) ;

      while ( rs.next() ) {
         readRec = new DocReaderProdBuffer() ;
         readRec.prodInt       = rs.getInt(1) ;
         readRec.brandGroupInd = rs.getInt(2) ;
         readRec.machine       = rs.getString(3).trim() ;
         readRec.model         = rs.getString(4).trim() ;
         readRec.productNum    = rs.getString(5).trim() ;
         buffer.prodVec.addElement ( readRec ) ;
      }

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }




   /*------------------------------------------------------------*/
   /* readDocMedia
    */
   /**
    *    This method will read in all file attachments
    *  to this document. It will add the records to
    *  the mediaVec in the buffer
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocMedia () throws Exception {
      Statement            stmt    = con.createStatement() ;
      ResultSet            rs      = null ;
      String               sql     = null ;
      DocReaderMediaBuffer readRec = null ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT A.MULTIMEDIAIND, B.GENINFO,      B.FILENAME, B.SIZE, "+
            "       B.FILEURL,       B.FILEVERSION,  B.MEDIAIND          "+
            "FROM   "+qualifier+".DOCMEDIA   A,                          "+
            "       TIGRIS.MULTIMEDIA B                                  "+
            "WHERE  A.DOCIND = "+docInd+" AND                            "+
            "       B.MULTIMEDIAIND = A.MULTIMEDIAIND                    "+
            "       ORDER BY B.CHANGEDTIME FOR FETCH ONLY ";        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buffer.mediaVec = new Vector ( 10, 10 ) ;
      while ( rs.next() ) {
         readRec = new DocReaderMediaBuffer () ;
         readRec.fileInd   = rs.getInt(1) ;
         readRec.descript  = rs.getString(2).trim() ;
         readRec.sFilename = rs.getString(3).trim() ;
         readRec.fileSize  = rs.getInt(4) ;
         readRec.fileURL   = rs.getString(5).trim() ;
         readRec.version   = rs.getString(6).trim() ;
         buffer.mediaVec.addElement ( readRec ) ;
      }


      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }


   /*------------------------------------------------------------*/
   /* readDocLinks
    */
   /**
    *    This method will read in all of the Document
    *  Links. It will populate the docLinksVec
    *  in the Buffer. It will also read in all documents linking
    *  to this document. This data will be populated in the refDocLinksVec
    *  in the buffer.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocLinks () throws Exception {
      Statement           stmt    = con.createStatement() ;
      ResultSet           rs      = null ;
      String              sql     = null ;
      DocReaderLinkBuffer readRec = null ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the SQL To read in the doc links                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT A.DOCLINKIND, B.TITLE, B.DOCCLASSIND, B.LNDOCID "+
            "FROM   "+qualifier+".DOCLINKS A, "+
            "       TIGRIS.DOCUMENTS B "+
            "WHERE  A.DOCIND      = "+docInd+" AND "+
            "       A.DOCLINKIND  = B.DOCIND "+
            "       FOR FETCH ONLY" ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize the Vectors                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buffer.docLinksVec    = new Vector ( 10, 10 ) ;
      buffer.refDocLinksVec = new Vector ( 10, 10 ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in data                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {
         readRec = new DocReaderLinkBuffer () ;
         readRec.docInd      = rs.getInt(1) ;
         readRec.title       = rs.getString(2).trim() ;
         readRec.docClassInd = rs.getInt(3) ;
         readRec.lndocInd    = rs.getString(4).trim() ;
         buffer.docLinksVec.addElement ( readRec ) ;
      }

      rs.close() ;                                       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the Statement to read in referring doc links        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT A.DOCIND, B.TITLE, B.DOCCLASSIND, B.LNDOCID "+
            "FROM   "+qualifier+".DOCLINKS A, "+
            "       TIGRIS.DOCUMENTS B "+
            "WHERE  A.DOCLINKIND  = "+docInd+" AND "+
            "       A.DOCIND  = B.DOCIND "+
            "       FOR FETCH ONLY" ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in data                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {
         readRec = new DocReaderLinkBuffer () ;
         readRec.docInd      = rs.getInt(1) ;
         readRec.title       = rs.getString(2).trim() ;
         readRec.docClassInd = rs.getInt(3) ;
         readRec.lndocInd    = rs.getString(4).trim() ;
         buffer.refDocLinksVec.addElement ( readRec ) ;
      }

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* readDocBody
    */
   /**
    *    This method will read in the document body
    *  and will populate the body field in the buffer
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocBody () throws Exception {
      Statement    stmt     = con.createStatement() ;
      ResultSet    rs       = null ;
      String       sql      = null ;      
      int          i        = 0 ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Declare the SQL                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT BODY, RECNUM "+
            "FROM "+qualifier+".DOCBODY "+
            "WHERE DOCIND = "+docInd+" "+
            "ORDER BY RECNUM FOR FETCH ONLY" ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the body to our temp variable                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buffer.bodyVec = new Vector ( 1, 1 ) ;
      while ( rs.next() ) {
         buffer.bodyVec.addElement ( rs.getString(1) ) ;
         i++ ;
      }

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* readDocCategory
    */
   /**
    *    This method will read in all of the document's
    *  categories.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocCategory () throws Exception {
      Statement stmt = con.createStatement() ;
      ResultSet rs   = null ;
      String    sql  = null ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Format the SQL                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sql = "SELECT DOCCATIND "+
            "FROM "+qualifier+".DOCCATEGORY "+
            "WHERE DOCIND = "+docInd+" FOR FETCH ONLY" ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sql ) ;       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buffer.docCatVec = new Vector ( 10, 10 ) ;
      while ( rs.next() ) {
         buffer.docCatVec.addElement ( new Integer ( rs.getInt(1) ) ) ;
      }

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* readCommentHistory
    */
   /**
    *    This method will read up to 5 of the latest
    *  comments for this document. This method
    *  requires a valid value in the existingDocInd property
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readCommentHistory () throws Exception {
      Statement     stmt       = con.createStatement() ;
      ResultSet     rs         = null ;
      String        sqlStmt    = null ; 
      StringBuffer  comments   = new StringBuffer() ;
      int           searchInd  = 0 ;

      if ( request == 0 )
         searchInd = docInd ;
      else
         searchInd = existingDocInd ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Form the SQL                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlStmt = "SELECT STOPDATE, COMMENT "+
                "FROM QUEST.DOCUMENTS "+
                "WHERE  EXISTINGDOCIND = "+searchInd+" AND "+
                "       STOPDATE IS NOT NULL AND "+
                "       RECYCLED = 'N' AND "+
                "       TEMPLATE = 'N' "+
                "ORDER BY STOPDATE DESC FETCH FIRST 5 ROWS ONLY" ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sqlStmt ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {
         comments.append ( rs.getString(1).trim()+"\n" ) ;
         if ( rs.getString(2) != null )
            //comments.append ( formatStringLineBreaks ( rs.getString(2).trim(), 80 ) ) ;
            comments.append ( rs.getString(2).trim() ) ;
         else
            comments.append ( "No Comment" ) ;
         comments.append ( "\n\n\n" ) ;
      }

      rs.close() ;
      stmt.close() ;

      buffer.commentHistory = comments.toString() ;
      comments = null ;

      return( 0 ) ;
   }





   /*------------------------------------------------------------*/
   /* formatStringLineBreaks
    */
   /**
    *    This method will format a string to the
    *  width specified
    *  
    *
    *  @param None
    *  @param None
    
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String formatStringLineBreaks ( String string, int length) {
      boolean         cont    = true ;
      int             nextRet = length ;
      int             index   = 0 ;
      String          retStr  = "" ;
      StringTokenizer token   = null ;
      StringBuffer    buf     = null ;

      if ( string != null ) {

         /* Create our String Tokenizer */
         token = new StringTokenizer ( string, "\n" ) ;

         /* Ok, now create a string with the words and single spaces in
            between the words */
         while ( token.hasMoreTokens () ) {
            retStr = retStr.concat ( token.nextToken () ) ;
            retStr = retStr.concat ( " " ) ;
         } /* while */


         if ( retStr != null ) {

            if ( retStr.length() > length ) {

               buf = new StringBuffer ( retStr ) ;

               /* Loop through ,making each space closest to the 80 a carriage return */
               while ( cont ) {

                  /* Get the last position of a space, before
                     the length */
                  index = retStr.lastIndexOf ( " ", nextRet ) ;

                  /* If there are no more spaces, get out of here */
                  if ( index < 1 )
                     cont = false ;
                  else {

                     /* If this index is valid, put a return character
                        in and set our index back */
                     if ( index < retStr.length() ) {
                        buf.setCharAt ( index, '\n' ) ;
                        nextRet=index+length ;
                        if ( nextRet > retStr.length() )
                           cont = false ;
                     } else
                        cont = false ;
                  } /* else */

               } /* cont */
            } /* if length */
            else
               buf = new StringBuffer ( retStr ) ;
         } /* if */
      } /* if */
      return( buf.toString().trim() ) ;
   }






   /*------------------------------------------------------------*/
   /* readDocQueryText
    */
   /**
    *    This method will read the document Query Text
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private int readDocQueryText () throws Exception {
      Statement     stmt       = con.createStatement() ;
      ResultSet     rs         = null ;
      String        sqlStmt    = null ; 
      StringBuffer  comments   = new StringBuffer() ;
      int           searchInd  = 0 ;

      if ( request == 0 )
         searchInd = docInd ;
      else
         searchInd = existingDocInd ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Form the SQL                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      sqlStmt = "SELECT QUERYTEXT "+
                "FROM "+qualifier+".DOCQUERYTEXT "+
                "WHERE  DOCIND = "+docInd ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Execute the Query                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rs = stmt.executeQuery ( sqlStmt ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( rs.next() )
         buffer.queryText = rs.getString(1).trim() ;

      rs.close() ;
      stmt.close() ;

      return( 0 ) ;
   }




} /* DocReader */
