package com.ibm.nzna.projects.qit.doc.storedProc.docReader ;
/*-----------------------------------------------------------*/
/*   DocReaderBuffer                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DocReaderBuffer                     */       
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This object simply holds all data sent back from the DocReader Stored Procedure. 
 *
 *
 * @version 1.0, 9/15/1999 
 * @author  David Lentz
 */
import java.util.* ;  


/*-----------------------------------------------------------*/
/*                                                           */
/*   DocReaderBuffer                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class DocReaderBuffer extends Object implements java.io.Serializable {

   /*------------------------------------------------------------*/
   /*                                                            */
   /*   This is the serialization ID. If you add fields to this  */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is     */
   /* fine as long as you know what you are doing!               */
   /*                                                            */
   /* If you change this number, you break everyone's Bookmarks  */
   /* to the point where they will not load, so you better write */
   /* a converter or something to run on everyone's bookmarks    */
   /*                                                            */
   /*------------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int MAX_STRING_LEN    = 10000 ;  // The max allowed string length to be serialized

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Document Properties                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public String  requestor                      = null ;        // The User Id of the Requestor who created the request.
   public String  title                          = null ;        // The title of this document. 
   public String  owner                          = null ;        // The current user who should be editing this document.
   public String  source                         = null ;        // The source of the data for the document
   public String  comment                        = null ;        // Comments only for Document Requests. The requestor fills this field in
   public String  partNumber                     = null ;        // This is for the SSds and Service Pub Document part number
   public String  lnDocId                        = null ;        // The Lotus Notes Doc Id. Mapped to QUEST.LNDOCID
   public String  dbUser                         = null ;        // Maps to the DBUSER field in the database. The last user who touched this record
   public String  changedTime                    = null ;        // Maps to the CHANGEDTIME field in the database. The last time this record was updated
   public String  webable                        = null ;
   public String  webonly                        = null ;
   public String  faxable                        = null ;
   public String  emailable                      = null ;
   public String  emailableDate                  = null ;
   public String  flash                          = null ;
   public String  flashDate                      = null ;
   public String  faq                            = null ;
   public String  summary                        = null ;
   public String  maintLock                      = null ;
   public String  show                           = null ;
   public String  commentHistory                 = null ;
   public String  knowledgeBase                  = null ;
   public String  queryText                      = null ;
   public int     workInd                        = 0 ;
   public int     docClassInd                    = 0 ;
   public int     reviewPeriod                   = 0 ;
   public int     authInd                        = 0 ;
   public int     docTypeInd                     = 0 ;
   public int     metricInd                      = 0 ;
   public int     pubTypeInd                     = 0 ;
   public int     docInd                         = 0 ;          // The Doc Request Indicator that should only be used in the QUEST.DOCUMENTS table
   public int     existingDocInd                 = 0 ;          // The Doc Indicator that should only be used in the TIGRIS.DOCUMENTS table.
   public int     templateDocInd                 = 0 ;          // If this is a new document created from a template, this DocInd is the indicator of the template


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Properties only for Document Requests                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public String  startDate                      = null ;        // The original date the request was created
   public String  stopDate                       = null ;        // The stop date of this request.
   public String  returnEMail                    = null ;        // The return eMail for a request

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Properties for all documents                            */                   
   /*                                                           */
   /*-----------------------------------------------------------*/
   public String universalNotesId               = null ;
   public String descript                       = null ;
   public String releaseDate                    = null ;
   public String createDate                     = null ;
   public String countryComment                 = null ;
   public String lastUpdateCTSTips              = null ;
   public String version                        = null ;
   public String url                            = null ;

   

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Additional Info and the Document Body                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public Vector addInfoVec   = null ; // Holds 10k String segments of the Additional Info
   public Vector bodyVec      = null ; // Holds 10k String segments of the Body Vector

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Lists of elements                                       */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public Vector     graphicVec        = null ; // graphics associated to this document.
   public Vector     mediaVec          = null ; // Media associated to this document.
   public Vector     docCatVec         = null ; // Categories of this document.  Mapped to TypeDocCat.
   public Vector     docLinksVec       = null ; // List of DocLinkRecs
   public Vector     refDocLinksVec    = null ; // A list of documents that link to this document
   public Vector     countryVec        = null ; // Holds a list of country code indicators
   public Vector     geoVec            = null ; // Holds a list of geographics indicators
   public Vector     prodVec           = null ; // Holds all associated products


   /*------------------------------------------------------------*/
   /* breakStringIntoVec
    */
   /**
    *    This method will break a string into DocReaderBuffer.MAX_STRING_LEN
    *  segments and place each segment into the Vector
    *  passed in
    *
    *  @param string The String to break up
    *  @param Vector The vector to place the string segments into
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void breakStringIntoVec ( String string, Vector vector ) throws Exception {
      
      if ( ( vector != null ) && ( string != null ) ) {                            

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this string is too long, break it up                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( string.length() > MAX_STRING_LEN ) {
            int i      = 0 ;
            int length = string.length() ;

            while ( i < length ) {
               if ( (i+MAX_STRING_LEN) < length )
                  vector.addElement ( string.substring ( i, i+MAX_STRING_LEN ) ) ;
               else
                  vector.addElement ( string.substring ( i, length ) ) ;
               i+=MAX_STRING_LEN ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, add the whole string into the vector         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            vector.addElement ( string ) ;
         }
      }
   }

   /*------------------------------------------------------------*/
   /* stringFromVec
    */
   /**
    *    This method will take a vector of strings and will create 1 string
    *  by concatenating the string segments togehter  
    *
    *  @param Vector The vector to get the string segments from   
    *
    *  @return String  
    *//*---------------------------------------------------------*/
   public static String stringFromVec ( Vector vector ) {
      String retStr = "" ;
      
      if ( vector != null ) {
         int i       = 0 ;
         int length  = vector.size() ;

         while ( i < length ) {
            retStr+=((String)vector.elementAt(i)) ;
            i++ ;
         }

      }

      return ( retStr ) ;
   }

} /* DocReaderBuffer */
