package com.ibm.nzna.projects.qit.exporter ;
/*-----------------------------------------------------------*/
/*   Export                                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: Export                              */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.awt.* ;
import java.io.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * The Export object does an entire export from asking the user what
 * type of export and where to export, to exporting the data.
 *
 * @version 1.0, 6/1/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   Export                                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class Export extends Object implements Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */     
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String TAB_STR    = new Character ( (char) 9 ).toString() ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ExportRec     exportRec         = null ;
   private AppDefaultWin parentWin         = null ;
   private Frame         tempFrame         = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will start the export process. Since the
    *  export is done in a thread, this constructor will immediatly
    *  return
    *  
    *
    *  @param parentWin The parent window. Since this export has to ask the user for information
    *                   in a dialog, this parentWin will be the parent of the dialog, if a NULL
    *                   is passed, a temporary Frame will be created
    *  @param dataVec The Data Vector to export
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Export ( AppDefaultWin parentWin, Vector dataVec ) {
      exportRec = new ExportRec() ;
      exportRec.setData ( dataVec ) ;
      this.parentWin = parentWin ;

      if ( parentWin != null )
         tempFrame = new Frame() ;

      new Thread ( this ).start() ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread does the actual export process
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {                                              

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, find out what type of export and where to export */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( parentWin != null )
            new ExportDlg ( parentWin, exportRec ) ;
         else
            new ExportDlg ( tempFrame, exportRec ) ;                              

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the user gave a valid type, continue                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( exportRec.getExportType() > 0 ) {
            File             exportFileOut = new File ( exportRec.getFilename() ) ;
            FileOutputStream exportStream  = new FileOutputStream ( exportFileOut ) ;                           

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have opened the file, continue                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( exportFileOut != null ) && ( exportStream != null ) ) {
               runExport ( exportStream ) ;
               exportStream.close() ;
            } else
               GUISystem.printBox ( STR_INFO, STR_CANNOT_OPEN_EXPORT_FILE ) ;
         }

      } catch ( Exception e ) {
         GUISystem.printBox ( STR_INFO, STR_EXPORT_FAILED_SEND_REPORT ) ;
         LogSystem.log ( 1, e , false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will try and set the status to
    *  the parentWin if a valid parentWin was passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setStatus ( int stringId ) {
      setStatus ( Str.getStr ( stringId ) ) ;
   }




   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will try and set the status to
    *  the parentWin if a valid parentWin was passed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setStatus ( String status ) {
      if ( parentWin != null )
         parentWin.setStatus ( status ) ;
   }





   /*------------------------------------------------------------*/
   /* runExport
    */
   /**
    *    This method will run the actual export
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void runExport ( FileOutputStream outStream ) {
      Vector dataVec = exportRec.getData() ;
      int    i       = 0 ;
      int    size    = dataVec.size() ;                    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Tell the user what's going on                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setStatus ( STR_PLEASE_WAIT ) ;                       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Loop through, writing each value                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         if ( parentWin != null ) {
            parentWin.setValueMax ( size ) ;
            parentWin.setValue ( 0 ) ;
         }
         while ( i < size ) {
            writeRow ( dataVec.elementAt(i), outStream ) ;
            i++ ;
            if ( parentWin != null )
               parentWin.setValue ( i ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      } /* catch  */

      setStatus ( null ) ;
      if ( parentWin != null )
         parentWin.setValue ( 0 ) ;
      GUISystem.printBox ( STR_INFO, STR_EXPORT_STOPPED ) ;
   }




   /*------------------------------------------------------------*/
   /* writeRow
    */
   /**
    *    This method will write a single row to the
    *  outputstream passed
    *  
    *
    *  @param object The object to write out
    *  @param outputStream The FileOutputStream to write to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeRow ( Object object, FileOutputStream outputStream ) throws IOException {
      if ( object instanceof StringTreeNode )
         object = ((StringTreeNode)object).getData() ;
      else if ( object instanceof MutableTreeWrapper )
         object = ((MutableTreeWrapper)object).getDataObject();

      if ( object instanceof MultiListRow ) {
         MultiListRow row  = (MultiListRow) object ;
         int          col  = 0 ;
         String       text = null ;

         while ( col < exportRec.getColumnCount() ) {
            if ( row.getColumnData ( col ) != null ) {
               if ( row.getColumnData ( col ) instanceof String ) {
                  text = (String) row.getColumnData ( col ) ;
                  writeColumn ( outputStream, text ) ;
               }
            } 
            col++ ;
         } /* while  */
         outputStream.write ( new String ( "\n" ).getBytes() ) ;
      } /* if  */ 
   } /* writeRow  */





   /*------------------------------------------------------------*/
   /* writeColumn
    */
   /**
    *    This method will write the single column
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void writeColumn ( FileOutputStream outStream, String text ) throws IOException {
      String outString = null ;

      switch ( exportRec.getExportType() ) {
         case ExportRec.EXPORT_FLAT_TEXT :
            outString = text+"   " ;
            break ;

         case ExportRec.EXPORT_COMMA_SEPARATED :
            outString = text+"," ;
            break ;

         case ExportRec.EXPORT_TAB_DELIM :
            outString = text+TAB_STR ;
      }

      if ( outString != null ) {
         outStream.write ( outString.getBytes() ) ;
      }
   }

} /* Export */
