package com.ibm.nzna.projects.qit.exporter ; 
/*-----------------------------------------------------------*/
/*   ExportDlg                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ExportDlg                           */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.io.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This dialog asks the user what type of data they wish to export, and the export filename 
 *
 *
 * @version 1.0, 5/31/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ExportDlg                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class ExportDlg extends JDialog implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_IMAGE           = null ;
   private JLabel       st_EXPORTTYPE      = null ;
   private JLabel       st_EXPORTFILE      = null ;
   private JLabel       st_COLCOUNT        = null ;
   private JRadioButton rb_TAB_DELIM       = null ;
   private JRadioButton rb_COMMA_SEPARATED = null ;
   private JRadioButton rb_FLAT_TEXT       = null ;
   private DButton      pb_EXPORT          = null ;
   private DButton      pb_CANCEL          = null ;
   private JTextField   ef_EXPORTFILE      = null ;      
   private JTextField   ef_COLCOUNT        = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ExportRec exportRec   = null ;
   private ImageIcon wizardImage = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and show the window.
    *  It will not return until the user closes the window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ExportDlg ( Frame parentWin, ExportRec exportRec ) {
      super ( parentWin, Str.getStr ( STR_EXPORT ), true ) ;
      Container   contentPane = getContentPane() ;
      ButtonGroup buttonGroup = new ButtonGroup() ;

      this.exportRec = exportRec ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      wizardImage = ImageSystem.getImageIcon ( this, ImageSystem.EXPORT_WIZARD ) ;
      st_IMAGE           = new JLabel ( wizardImage ) ;
      st_EXPORTFILE      = new JLabel ( Str.getStr ( STR_EXPORT_FILENAME ) ) ;
      st_EXPORTTYPE      = new JLabel ( Str.getStr ( STR_EXPORT_TYPE ) ) ;
      st_COLCOUNT        = new JLabel ( Str.getStr ( STR_COLUMN_COUNT ) ) ;
      ef_EXPORTFILE      = new JTextField ( "" ) ;
      ef_COLCOUNT        = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 2 ), "5", 0 ) ;
      rb_COMMA_SEPARATED = new JRadioButton ( Str.getStr ( STR_COMMA_SEPARATED ) ) ;
      rb_TAB_DELIM       = new JRadioButton ( Str.getStr ( STR_TAB_DELIM ) ) ;
      rb_FLAT_TEXT       = new JRadioButton ( Str.getStr ( STR_FLAT_TEXT ) ) ;
      pb_EXPORT          = new DButton ( Str.getStr ( STR_EXPORT ) ) ;
      pb_CANCEL          = new DButton ( Str.getStr ( STR_CANCEL ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_EXPORTTYPE ) ;
      contentPane.add ( rb_TAB_DELIM ) ;
      contentPane.add ( rb_COMMA_SEPARATED ) ;
      contentPane.add ( rb_FLAT_TEXT ) ;
      contentPane.add ( st_EXPORTFILE ) ;
      contentPane.add ( ef_EXPORTFILE ) ;
      contentPane.add ( st_COLCOUNT ) ;
      contentPane.add ( ef_COLCOUNT ) ;
      contentPane.add ( pb_EXPORT ) ;
      contentPane.add ( pb_CANCEL ) ;
      GUISystem.setPropertiesOnPanel ( contentPane ) ;                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_EXPORT.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      ef_COLCOUNT.addActionListener ( this ) ;
      ef_EXPORTFILE.addActionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the radio buttons to the button group               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonGroup.add ( rb_TAB_DELIM ) ;
      buttonGroup.add ( rb_COMMA_SEPARATED ) ;
      buttonGroup.add ( rb_FLAT_TEXT ) ;
      rb_TAB_DELIM.setSelected ( true ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Center our window                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 495, 325 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;

   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = wizardImage.getImage().getWidth(this)+10 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2), 
                           wizardImage.getImage().getWidth(this), 
                           wizardImage.getImage().getHeight(this)) ;



      st_EXPORTTYPE.setBounds ( x, y, size.width-(x)-10, rowHeight ) ;
      y+=rowHeight ;

      rb_TAB_DELIM.setBounds ( x, y, size.width-(x)-10-100, rowHeight ) ;
      y+=rowHeight ;

      rb_COMMA_SEPARATED.setBounds ( x, y, size.width-(x)-10-100, rowHeight ) ;
      y+=rowHeight ;

      rb_FLAT_TEXT.setBounds ( x, y, size.width-(x)-10-100, rowHeight ) ;
      y+=rowHeight+10 ;


      st_EXPORTFILE.setBounds ( x, y, size.width-(x)-10-100, rowHeight ) ;
      y+=rowHeight ;


      ef_EXPORTFILE.setBounds ( x, y, size.width-(x)-10-100, rowHeight ) ;
      y+=rowHeight ;

      st_COLCOUNT.setBounds ( x, y, size.width-(x)-10-100, rowHeight ) ;
      y+=rowHeight ;


      ef_COLCOUNT.setBounds ( x, y, size.width-(x)-10-100, rowHeight ) ;
      y+=rowHeight ;

      pb_EXPORT.setLocation ( x, size.height-75 ) ;
      pb_CANCEL.setLocation ( x+pb_EXPORT.getMinimumSize().width+5, size.height-75 ) ;

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         exportRec.setExportType ( 0 ) ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EXPORT                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_EXPORT ) ||
                ( e.getSource() == ef_EXPORTFILE ) ||
                ( e.getSource() == ef_COLCOUNT ) ) {
         if ( saveData() )
            dispose() ;
      }
   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save the data on our screen
    *  and will make sure that the data is correct.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      boolean rc          = false ;
      String  filename    = ef_EXPORTFILE.getText() ;
      int     columnCount = new Integer ( ef_COLCOUNT.getText() ).intValue() ;

      if ( rb_TAB_DELIM.isSelected() )
         exportRec.setExportType ( ExportRec.EXPORT_TAB_DELIM ) ;
      else if ( rb_COMMA_SEPARATED.isSelected() )
         exportRec.setExportType ( ExportRec.EXPORT_COMMA_SEPARATED ) ;
      else if ( rb_FLAT_TEXT.isSelected() )
         exportRec.setExportType ( ExportRec.EXPORT_FLAT_TEXT ) ;


      if ( ( filename != null ) || ( filename.length() > 0 ) ) {
         try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Check if the file already exists                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( FileUtil.exists ( filename ) ) {

               if ( GUISystem.printBox ( STR_QUESTION, STR_FILE_EXISTS_OVERWRITE ) ) {
                  rc = true ;
                  exportRec.setFilename ( filename ) ;
               }
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, check to see if we can write to the file     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
               File             outFile   = new File ( filename ) ;
               FileOutputStream outStream = new FileOutputStream ( outFile ) ;
               byte[] outByte = {1,2,3,4} ;             

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Try and write the file                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               outStream.write ( outByte ) ;

               outStream.close() ;
               outFile.delete() ;
               rc = true ;
               exportRec.setFilename ( filename ) ;
            }

         } catch ( Exception e ) {
            ef_EXPORTFILE.requestFocus() ;
            st_EXPORTFILE.setForeground ( Color.blue ) ;
            st_EXPORTFILE.setText ( Str.getStr ( STR_CANNOT_OPEN_EXPORT_FILE ) ) ;
            rc = false ;
         }
      } else {
         ef_EXPORTFILE.requestFocus() ;
         st_EXPORTFILE.setForeground ( Color.blue ) ;
         st_EXPORTFILE.setText ( Str.getStr ( STR_CANNOT_OPEN_EXPORT_FILE ) ) ;
      }


      if ( rc ) {
         if ( columnCount > 0 )
            exportRec.setColumnCount ( columnCount ) ;
         else {
            GUISystem.printBox ( STR_INFO, STR_COLUMN_COUNT_MUST_BE_GREATER_THAN_ZERO ) ;
            ef_COLCOUNT.requestFocus() ;
            rc = false ;
         }
      }
      return( rc ) ;
   }
} /* ExportDlg */
