package com.ibm.nzna.projects.qit.exporter ; 
/*-----------------------------------------------------------*/
/*   ExportRec                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ExportRec                           */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
/**
 * This record holds information needed for an export 
 *
 *
 * @version 1.0, 5/31/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ExportRec                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class ExportRec extends Object {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Export Types                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int EXPORT_UNKNOWN         = 0 ;
   public static final int EXPORT_TAB_DELIM       = 1 ;
   public static final int EXPORT_COMMA_SEPARATED = 2 ;
   public static final int EXPORT_FLAT_TEXT       = 3 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Members                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector dataVec     = null ;
   private String filename    = null ;
   private int    exportType  = 0 ;
   private int    columnCount = 0 ;





   /*------------------------------------------------------------*/
   /* setExportType
    */
   /**
    *    This method will set the export type
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setExportType ( int exportType ) {
      if ( ( exportType < 0 ) || ( exportType > 3 ) )
         throw new IllegalArgumentException ( "Unknown export type:"+exportType ) ;
      else
         this.exportType = exportType ;
   }




   /*------------------------------------------------------------*/
   /* setData
    */
   /**
    *    This method will set the data object that
    *  we are going to export
    *  
    *
    *  @param dataVec The vector containing the data to export
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setData ( Vector dataVec ) {
      this.dataVec = dataVec ;
   }




   /*------------------------------------------------------------*/
   /* setFilename
    */
   /**
    *    This method will set the filename of the 
    *  export to save to
    *  
    *
    *  @param filename The filename that will be exported to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFilename ( String filename ) {
      this.filename = filename ;
   }





   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the vector of data
    *  to return
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getData () {
      return( dataVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getFilename
    */
   /**
    *    This method will get the filename of the
    *  export. This filename should be the file containing
    *  the exported data
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getFilename () {
      return( filename ) ;
   }




   /*------------------------------------------------------------*/
   /* getExportType
    */
   /**
    *    This method will return the export type
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getExportType () {
      return( exportType ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    This method will return the number of columns
    *  to export
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getColumnCount () {
      return( columnCount ) ;
   }




   /*------------------------------------------------------------*/
   /* setColumnCount
    */
   /**
    *    This method will set the amount of columns to
    *  export
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setColumnCount ( int columnCount ) {
      this.columnCount = columnCount ;
   }
} /* ExportRec */
