package com.ibm.nzna.projects.qit.ftp_explorer ;
/*-----------------------------------------------------------*/
/*   FTPDocDlg                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FTPDocDlg                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog holds the information that will be stored with a document. 
 * This information is the document that is associated with a file. THe
 * other information (Description and date) are not directly associated with
 * a document
 *
 *
 * @version 1.0, 11/11/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   FTPDocDlg                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPDocDlg extends JDialog implements DataLengths, ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String filename          = "" ; 
   private int    rc                = 0 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_VERSION        = null ;
   private JLabel       st_DESCRIPT       = null ;
   private JLabel       st_DOCUMENT       = null ;
   private JTextField   ef_VERSION        = null ;
   private JTextField   ef_DOCID          = null ;
   private JTextField   ef_DOCUMENT       = null ;
   private DTextArea    mle_DESCRIPT      = null ;
   private DButton      pb_OK             = null ;
   private DButton      pb_CANCEL         = null ;
   private DButton      pb_BROWSE         = null ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create but not show our window.
    *  To show the window and process the user's
    *  results please call 'getResult'
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FTPDocDlg ( Frame parent, String filename ) {
      super ( parent, Str.getStr ( STR_FILE )+" - "+filename, true ) ;

      setSize ( 300, 200 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;

      createControls() ;
   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This mehtod will show the window and waits
    *  for the user to interact with it. Once the user
    *  has closed the window, this method will return
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return -1 = User canceled, 0 if everything ok, 1 if an error occurred
    *//*---------------------------------------------------------*/
   public int getResult() {
      setVisible ( true ) ;
      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create all of our controls and
    *  attaches them to our window.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_VERSION   = new JLabel ( Str.getStr ( STR_VERSION ) ) ;
      st_DESCRIPT  = new JLabel ( Str.getStr ( STR_DESCRIPTION ) ) ;
      st_DOCUMENT  = new JLabel ( Str.getStr ( STR_DOCUMENT ) ) ;
      ef_VERSION   = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_FILEVERSION ), "", 0 ) ;
      mle_DESCRIPT = new DTextArea ( ) ;
      ef_DOCUMENT  = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 10 ), "", 0 ) ;
      pb_BROWSE    = new DButton ( Str.getStr ( STR_BROWSE ) ) ;
      pb_OK        = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL    = new DButton ( Str.getStr ( STR_CANCEL ) ) ;      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mle_DESCRIPT.setMaximumText ( LENGTH_FILEGENINFO ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_BROWSE.addActionListener ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is sent
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_CANCEL ) {
         rc=-1 ;
         dispose() ;
      }

   }



   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of
    *  our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_VERSION.setBounds ( x,y,100,rowHeight ) ;
      ef_VERSION.setBounds ( x+100,y,100,rowHeight ) ;
      y+=rowHeight ;

      st_DESCRIPT.setBounds ( x,y,200,rowHeight ) ;
      y+=rowHeight ;

      mle_DESCRIPT.setBounds ( x, y, size.width-(x*2), rowHeight*4 ) ;
      y+=(rowHeight*4) ;

      st_DOCUMENT.setBounds ( x, y, 100, rowHeight ) ;
      ef_DOCUMENT.setBounds ( x+100, y, 75, rowHeight ) ;
      pb_BROWSE.setBounds ( x+180, y, 75, 25 ) ;
      
      pb_OK.setBounds ( x, size.height-55, 70, 25 ) ;
      pb_CANCEL.setBounds ( x+75, size.height-55, 75, 25 ) ;
   }


} /* FTPDocDlg */
