package com.ibm.nzna.projects.qit.ftp_explorer ;
/*-----------------------------------------------------------*/
/*   FTPExplorerPanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FTPExplorerPanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel will hold a Windows 'Explorer' look but instead of exploring
 * the local hard drive, it will explore an FTP Site, allowing the user
 * to view files, download files, upload etc..
 *
 * @version 1.0, 10/5/1999 
 * @author  David Lentz
 */
import javax.swing.* ;
import javax.swing.tree.* ;
import javax.swing.event.* ;
import javax.swing.text.html.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import java.io.* ;
import java.net.* ;
//import com.ibm.nzna.shared.ftp.* ;
//import com.ibm.nzna.shared.ftp.FTP.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.ftp.sun.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   FTPExplorer                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPExplorerPanel extends JPanel implements 
QuestPanel,
QuestProcess,
DocConst, 
ListSelectionListener, 
AppConst, 
ActionListener, 
ItemListener, 
Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Body Preview                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String bodyTemplate =
   "<HTML><BODY>"+
   "<P><FONT FACE=\"Helvetica\" SIZE=1>"+
   "<FONT COLOR=\"#800080\">FILENAME: </FONT><b>VAL_FILENAME<b><BR>"+
   "<FONT COLOR=\"#800080\">VERSION: </FONT>VAL_VERSION<BR>"+
   "<FONT COLOR=\"#800080\">LASTTOUCHEDON: </FONT>VAL_LASTTOUCHEDON<BR>"+
   "<FONT COLOR=\"#800080\">ASSOCIATEDDOCS: </FONT>VAL_ASSOCIATEDDOCS<BR>"+
   "<FONT COLOR=\"#800080\">DESCRIPTION: </FONT>VAL_DESCRIPTION<BR>"+
   "<hr>"+
   "VAL_BODY"+
   "</P></FONT></BODY></HTML>" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Thread                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_CONNECT            = "Connect" ;
   private static final String THREAD_REFRESHDIRECTORIES = "RefDir" ;
   private static final String THREAD_REFRESHPREVIEW     = "RefPre" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Titles                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] fileTitle = { 
      "", 
      Str.getStr ( STR_FILENAME ), 
      Str.getStr ( STR_SIZE )
   } ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI                                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_FTPSITE        = new JLabel ( Str.getStr ( STR_FTP_SITE ) ) ;
   private DButton     pb_GO             = new DButton ( Str.getStr ( STR_GO ) ) ;
   private JComboBox   cb_FTPSITE        = new JComboBox() ;
   private MultiList   cnr_FILELIST      = new MultiList ( GUISystem.getFontUtil() ) ;
   private JTextPane   mle_PREVIEW       = new JTextPane() ;
   private JSplitPane  pnl_FILEPREV      = new JSplitPane ( JSplitPane.VERTICAL_SPLIT ) ;
   private JScrollPane scr_PREVIEW       = new JScrollPane ( mle_PREVIEW ) ;
   private JPanel      pnl_FILELIST      = new JPanel () ;
   private JTitle      st_DIRECTORY      = new JTitle ( "" ) ;
   private ActionButton pb_CREATE         = null ;
   private ActionButton pb_DELETE         = null ;
   private ActionButton pb_EDIT           = null ;
   private ActionButton pb_CLOSE          = null ;
   private ActionButton pb_DOWNLOAD       = null ;
   private FTPNavPanel  navPanel          = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DefaultTreeModel    model              = null ;
   private String              currentFTPSite     = "" ;
   private Vector              siteList           = null ;
   private SunFTP              ftp                = null ;
   private boolean             connected          = false ;
   private DefaultTreeRenderer treeRenderer       = null ;
   private String              curPath            = null ;
   private Vector              fileVec            = null ;
   private String              wholePath          = "" ;
   private FTPFileRec          currentFileRec     = null ;
   private boolean             firstShow          = false ;
   private AppDefaultWin       parentWin          = null ;
   private boolean             continueProcessing = false ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor and it initializes
    *  and creates the entire GUI for the FTP Explorer.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FTPExplorerPanel () {
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will add all of our controls
    *  to the panel and initialize any variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {
      Dimension size = getSize() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our action buttons                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE    = new ActionButton ( Str.getStr ( STR_CLOSE ),    ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),  Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_CREATE   = new ActionButton ( Str.getStr ( STR_NEW ),      ImageSystem.getImageIcon ( this, ImageSystem.NEW ),    Str.getStr ( STR_NEW ) ) ;
      pb_EDIT     = new ActionButton ( Str.getStr ( STR_EDIT ),     ImageSystem.getImageIcon ( this, ImageSystem.EDIT ),   Str.getStr ( STR_EDIT ) ) ;
      pb_DELETE   = new ActionButton ( Str.getStr ( STR_DELETE ),   ImageSystem.getImageIcon ( this, ImageSystem.DELETE ), Str.getStr ( STR_DELETE ) ) ;
      pb_DOWNLOAD = new ActionButton ( Str.getStr ( STR_DOWNLOAD ), ImageSystem.getImageIcon ( this, ImageSystem.SAVE ),   Str.getStr ( STR_DOWNLOAD ) ) ;
      navPanel = new FTPNavPanel ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Fill the history combo box                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      siteList = (Vector) QuestUtil.readObjectFromDisk ( "ftphist.ser" ) ;

      if ( siteList == null ) {
         siteList = new Vector ( 10, 1 ) ;
         siteList.addElement ( new String ( "f3.pc.ibm.com" ) ) ;
      }

      cb_FTPSITE = new JComboBox ( siteList ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust some properties                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_FILELIST.setBackground ( Color.white ) ;
      pnl_FILELIST.setBorder ( GUISystem.loweredBorder ) ;
      pnl_FILELIST.setLayout ( new BorderLayout() ) ;

      cnr_FILELIST.setBorder ( null ) ;
      cnr_FILELIST.setColumnHeadingBorder ( null ) ;
      cnr_FILELIST.setOpaque ( true ) ;
      cnr_FILELIST.setColumnHeadings ( fileTitle ) ;
      cnr_FILELIST.setColumnWidth ( 0, 25 ) ;
      cnr_FILELIST.setColumnWidth ( 1, 250 ) ;
      cnr_FILELIST.setColumnWidth ( 2, 50 ) ;
      cnr_FILELIST.setPreferredSize ( new Dimension ( 100, size.height-200 ) ) ;

      cb_FTPSITE.setEditable ( true ) ;

      st_DIRECTORY.setFont ( FontSystem.largeTitleFont ) ;

      mle_PREVIEW.setPreferredSize ( new Dimension ( 100, 200 ) ) ;
      mle_PREVIEW.setEditorKit ( new HTMLEditorKit() ) ;
      mle_PREVIEW.setEditable ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_FTPSITE.addItemListener ( this ) ;
      pb_GO.addActionListener ( this ) ;
      pb_CLOSE.addActionListener ( this ) ;
      pb_DOWNLOAD.addActionListener ( this ) ;
      pb_CREATE.addActionListener ( this ) ;
      cnr_FILELIST.addListSelectionListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to ourselves                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( st_FTPSITE ) ;
      add ( cb_FTPSITE ) ;
      add ( pb_GO ) ;
      add ( pnl_FILEPREV ) ;
      pnl_FILELIST.add ( st_DIRECTORY, BorderLayout.NORTH ) ;
      pnl_FILELIST.add ( cnr_FILELIST, BorderLayout.CENTER ) ;
      pnl_FILEPREV.add ( pnl_FILELIST, JSplitPane.TOP ) ;
      pnl_FILEPREV.add ( scr_PREVIEW,  JSplitPane.BOTTOM ) ;

      pnl_FILEPREV.setDividerLocation ( .9 ) ;

      return( true ) ;
   } /* initialize */




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and size our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       siteWidth = st_FTPSITE.getPreferredSize().width ;

      super.doLayout() ;

      st_FTPSITE.setBounds ( 2, 2, siteWidth, rowHeight ) ;
      cb_FTPSITE.setBounds ( 2+siteWidth, 2, size.width-60-siteWidth, rowHeight ) ;
      pb_GO.setBounds ( size.width-50, 2, 45, rowHeight ) ;

      pnl_FILEPREV.setBounds ( 0, 4+rowHeight, size.width, size.height-rowHeight ) ;


      if ( !firstShow ) {
         if ( size.height > 0 ) {
            firstShow = true ;
            pnl_FILEPREV.setDividerLocation ( .75 ) ;
         }
      }

   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are listening for is fired
    *  
    *
    *  @param e The Action Event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PB_GO                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_GO ) {
         currentFTPSite = (String) cb_FTPSITE.getSelectedItem() ;
         new Thread ( this, THREAD_CONNECT ).start() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CREATE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CREATE ) {
         if ( ftp != null )
            create() ;
         else
            GUISystem.printBox ( STR_INFO, STR_NOT_CONNECTED_TO_FTP ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_DOWNLOAD                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DOWNLOAD ) {
         try {
            if ( currentFileRec != null ) {
               File[] file = GUISystem.getFileOpen ( parentWin,
                                                     Str.getStr ( STR_SAVE_AS ),
                                                     PropertySystem.getString ( PropertySystem.PROPERTY_FTP_SAVE_DIR ),
                                                     currentFileRec.getFilename(),
                                                     false ) ;

               if ( ( file != null ) && ( file.length > 0 ) ) {
                  String dir = file[0].getAbsolutePath() ;

                  /* Strip the filename from the path */
                  dir = dir.substring ( 0, dir.lastIndexOf(File.separator)) ;

                  PropertySystem.putString ( PropertySystem.PROPERTY_FTP_SAVE_DIR, dir ) ;

                  if ( downloadCurrentFile ( dir ) )
                     GUISystem.printBox ( STR_INFO, STR_DOWNLOAD_COMPLETE ) ;
               }
            } else
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;
         } catch ( Exception ex1 ) {
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Other                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CLOSE ) {
         try {
            if ( connected )
               ftp.doCommand ( "bye" ) ;
            QuestUtil.writeObjectToDisk ( "ftphist.ser", siteList ) ;
         } catch ( Exception e1 ) {
         }
         parentWin.closePanel ( this, null ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* itemStateChanged
    */
   /**
    *    This method will be called when an item
    *  we are listening for has been changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void itemStateChanged ( ItemEvent e ) {
      if ( e.getStateChange() == ItemEvent.SELECTED ) {
         String text = (String)cb_FTPSITE.getSelectedItem() ;                                        

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add this site to our list if we don't have it yet       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !StoreUtil.vectorContains ( siteList, text ) ) {
            siteList.insertElementAt( text, 0 ) ;
         }

         currentFTPSite = text ;                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Disconnect from the current site                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         disconnect() ;             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Connect to the new one                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         new Thread ( this, THREAD_CONNECT ).start() ;

      }
   }




   /*------------------------------------------------------------*/
   /* disconnect
    */
   /**
    *    This method will disconnect from the current
    *  FTP Site
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void disconnect () {
      Qit.setEnabled ( false ) ;
      try {
         if ( ftp != null ) {
            parentWin.setStatus ( Str.getStr ( STR_DISCONNECTING ) ) ;
            ftp.doCommand ( "bye" ) ;
            ftp = null ;
            System.gc() ;
            parentWin.setStatus ( null ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
      connected = false ;
      Qit.setEnabled ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* connect
    */
   /**
    *    This method will connect to the FTP Site
    *  currently populated in the currentFTPSite
    *  property
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void connect () {

      navPanel.clear() ;
      cnr_FILELIST.removeAll() ;
      wholePath="" ;
      parentWin.setStatus ( Str.getStr ( STR_CONNECTING_TO )+" "+currentFTPSite ) ;
      Qit.setEnabled ( false ) ;
      connected = false ;

      try {
         FTPLogonDlg logonDlg = new FTPLogonDlg ( parentWin ) ;

         if ( onInternalSite() ) {
            ftp = new SunFTP ( currentFTPSite, "anonymous", UserSystem.getUserId()+"@questInput.com" ) ;
            connected = true ;
         }

         else if ( logonDlg.getResponse() ) {
            String  userId    = logonDlg.getUserId() ;
            String  password  = logonDlg.getPassword() ;
            boolean anonymous = logonDlg.getAnonymous() ;                                

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we are to logon as anonymous, send the user's        */
            /*   id as the password.                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( anonymous ) {
               userId = "anonymous" ;
               password = UserSystem.getUserId()+"@QuestInput.com" ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Log in                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            ftp = new SunFTP ( currentFTPSite, userId, password ) ; 

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Mark as connected                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            connected = true ;

         } else
            disconnect() ;

         logonDlg = null ;                                  


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are connected, refresh the current dir.           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( connected ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this is an internal server, change directories       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( onInternalSite() )
               ftp.cd ( "pub/pccbbs" ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Ok, get the initial directory listing                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            refreshDirectories() ;

         }

      } catch ( java.net.UnknownHostException ex1 ) {
         GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_COULD_NOT_CONNECT_TO_FTP_SITE )+" "+currentFTPSite )  ;
      } catch ( Exception e ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Catch the Logon Error                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.toString().indexOf ( "Login incorrect" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_USER_ID_PASSWORD_WRONG_FOR_FTP_SITE ) ;

         else if ( e.toString().indexOf ( "Connection refused" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_CONNECTION_REFUSED ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Catch all other errors                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else
            LogSystem.log ( 1, e ) ;
      }

      Qit.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will run a variety of operations
    *  inside of a thread. The operation run depends
    *  on the name of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;     

      continueProcessing = true ;
      ProcessManagerSystem.addProcess ( this ) ;

      try {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_CONNECT                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( operation.equals ( THREAD_CONNECT ) ) {
            connect() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_REFRESHDIRECTORIES                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( operation.equals ( THREAD_REFRESHDIRECTORIES ) )
            refreshDirectories() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_REFRESHPREVIEW                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( operation.equals ( THREAD_REFRESHPREVIEW ) )
            refreshPreview() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      ProcessManagerSystem.removeProcess ( this ) ;

   }




   /*------------------------------------------------------------*/
   /* refreshDirectories
    */
   /**
    *    This method will refresh the directory
    *  list from the FTP Site. It will populate both the left
    *  list of directories, and the list of files in the selected
    *  directory
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshDirectories() {
      if ( connected ) {

         /* Set our wait cursor */
         setCursor ( Cursor.getPredefinedCursor ( Cursor.WAIT_CURSOR ) ) ;

         try {
            Vector         dir         = null ;
            int            i           = 0 ;     
            long           size        = 0 ;
            int            dirIndex    = 0 ;
            Vector         dirVec      = new Vector ( 10, 10 ) ;
            int            dirSize     = 0 ;
            String         currentLine = null ;

            parentWin.setStatus ( Str.getStr ( STR_READING ) ) ;

            dir     = ftp.lsDir() ;                    
            dirSize = dir.size() ;
            fileVec = new Vector ( 10, 10 ) ;


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Walk through the list grabbing files and directories    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( ( continueProcessing ) && ( i < dirSize ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Grab the next line                                      */
               /*                                                           */
               /*-----------------------------------------------------------*/
               currentLine = (String) dir.elementAt(i) ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If a directory, add it to our directory tree            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( currentLine.startsWith ( "d" ) ) {
                  FTPFileRec dirRec = parseFilename ( (String)dir.elementAt(i) ) ;

                  if ( dirRec != null ) {
                     dirRec.setDirectory(true) ;
                     dirRec.autoSetIcon () ;
                     dirVec.addElement ( dirRec ) ;
                  }

                  dirIndex++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise it is a file. Add it to our list of files     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else {
                  FTPFileRec fileRec = parseFilename ((String)dir.elementAt(i) ) ;

                  if ( fileRec != null ) {
                     fileRec.autoSetIcon() ;
                     fileRec.setSize ( size ) ;
                     fileVec.addElement ( fileRec ) ;
                  }
               }
               i++ ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Sort the dirs and files                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            new QuickSort ( dirVec ) ;
            new QuickSort ( fileVec ) ;
            cnr_FILELIST.removeAll() ;            
            cnr_FILELIST.add ( dirVec ) ;
            cnr_FILELIST.add ( fileVec ) ;
            navPanel.clear() ;
            navPanel.refreshList ( dirVec ) ;


            revalidate() ;



         } catch ( Exception e ) {
            if ( e.toString().indexOf ( "Illegal PORT Command" ) >= 0 ) {
               GUISystem.printBox ( STR_INFO, STR_FTP_SITE_RESPONDED_WITH_ERROR ) ;
               LogSystem.log ( 1, e, false ) ;
            } else if ( e.toString().indexOf ( "No files found" ) >= 0 ) {
               navPanel.clear() ;
               cnr_FILELIST.removeAll() ;
            } else if ( e.toString().indexOf ( "EOF" ) >= 0 )
               GUISystem.printBox ( STR_INFO, STR_CANNOT_READ_DIR_FROM_FTP_SITE ) ;

            else
               LogSystem.log ( 1, e ) ;
         }
      }


      /* Return our cursor */
      setCursor ( Cursor.getPredefinedCursor ( Cursor.DEFAULT_CURSOR ) ) ;

      parentWin.setStatus ( null ) ;
   }



   /*------------------------------------------------------------*/  
   /* valueChanged 
    */ 
   /** 
    *    This method will respond to events sent to us 
    *  from the container on our panel 
    *   
    * 
    *  @param event The TreeSelectionEvent
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   File List                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == cnr_FILELIST ) {
         FTPFileRec fileRec = (FTPFileRec) cnr_FILELIST.getSelectedItem() ;

         if ( fileRec != null ) {

            if ( fileRec.getDirectory() )
               changeDir ( fileRec.getFilename() ) ;
            else {
               currentFileRec = fileRec ;
               new Thread ( this, THREAD_REFRESHPREVIEW ).start() ;
            }
         }
      }

   } /* valueChanged */





   /*------------------------------------------------------------*/
   /* parseFilename
    */
   /**
    *    This method will parse out the filename from
    *  a ls -l directory listing.
    *  
    *
    *  @param line The entire line containing permisions, size and date of a file
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private FTPFileRec parseFilename ( String line ) {
      StringTokenizer strtok  = new StringTokenizer ( line, " " ) ;
      int             size    = 0 ;
      FTPFileRec      fileRec = null ;

      try {
         /* First, ignore the permissions */
         strtok.nextElement() ;

         /* Second, ignore the number of sub dirs */
         strtok.nextElement() ;

         /* Third, ignore the User and Group */
         strtok.nextElement() ;
         strtok.nextElement() ;

         /* Fourth, grab the size */
         size = new Integer ( (String)strtok.nextElement() ).intValue() ;

         /* Fifth, ignore the date */
         strtok.nextElement() ;
         strtok.nextElement() ;
         strtok.nextElement() ;

         /* Now, finally, the filename */
         fileRec = new FTPFileRec ( ((String)strtok.nextElement()).trim() ) ;
         fileRec.setSize ( size ) ;

      } catch ( Exception e ) {
      }


      return( fileRec ) ;


   }


   /*------------------------------------------------------------*/
   /* isFileDir
    */
   /**
    *    This method will determine the file size
    *  if a file. If a size of -1 is returned, then the file is
    *  a directory
    *  
    *
    *  @param filename The name of the file
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public long getFileSize ( String filename ) {
      long rc = 0 ;

      //try {
      //   rc = ftp.size ( filename ) ;
      //} catch ( Exception e ) {
      //   if ( e.toString().indexOf ( "not a plain file" ) >= 0 )
      //      rc = -1 ;
      //   else if ( e.toString().indexOf ( "Permission denied" ) >= 0 )
      //      rc = 0 ;
      //   else if ( e.toString().indexOf ( "does not allow" ) >= 0 )
      //      rc = 0 ;
      //   else
      //      LogSystem.log ( 1, e ) ;
      //}

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* enableComponents
    */
   /**
    *    This method will either enable or disable
    *  the components on this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void enableComponents ( boolean enable ) {
      pb_GO.setEnabled ( enable ) ;
      cb_FTPSITE.setEnabled ( enable ) ;
      cnr_FILELIST.setEnabled ( enable ) ;
      pnl_FILELIST.setEnabled ( enable ) ;
      navPanel.setEnabled ( enable ) ;
   }





   /*------------------------------------------------------------*/
   /* refreshPreview
    */
   /**
    *    This method will refresh the preview information.
    *  It will populate the preview panel with whatever
    *  currentFileRec is set to
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshPreview () {
      try {
         String type      = currentFileRec.getType() ;
         String name      = currentFileRec.getFilename() ;
         String localName = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator+name ;
         String preview   = "" ;                       
         String descript  = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, we must download the file                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         parentWin.setStatus ( Str.getStr ( STR_DOWNLOADING ) ) ;

         preview="<html>" ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the beginning of our preview. If we are on the   */
         /*   internal site, we must go get more information to       */
         /*   display                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( onInternalSite() ) {
            //VOY Vector docVec = null ;           


            //VOY /*-----------------------------------------------------------*/
            //VOY /*                                                           */
            //VOY /*   Read in the extra info                                  */
            //VOY /*                                                           */
            //VOY /*-----------------------------------------------------------*/
            //VOY if ( !currentFileRec.getDatabaseRead() )
            //VOY    FTPSQL.getFileInfo ( parentWin, currentFileRec ) ;


            //VOY /*-----------------------------------------------------------*/
            //VOY /*                                                           */
            //VOY /*   Format the extra info                                   */
            //VOY /*                                                           */
            //VOY /*-----------------------------------------------------------*/
            //VOY descript = currentFileRec.getDescript() ;
            //VOY if ( descript != null )
            //VOY    descript = Text.replaceAllStrInStr ( descript, "\n", "<br>" ) ;

            //VOY preview+= "<b>"+Str.getStr ( STR_FILENAME )+":</b> "+
            //VOY           name+"<br>"+
            //VOY           "<b>"+Str.getStr ( STR_VERSION )+":</b> "+
            //VOY           currentFileRec.getVersion()+"<br>"+
            //VOY           "<b>"+Str.getStr ( STR_LAST_TOUCHED_ON )+":</b> "+
            //VOY           currentFileRec.getTimestamp()+"<br><br>"+
            //VOY           "<b><u>"+Str.getStr ( STR_ASSOCIATED_DOCS )+"</u></b><br>" ;

            //VOY /*-----------------------------------------------------------*/
            //VOY /*                                                           */
            //VOY /*   Now, add each doc rec                                   */
            //VOY /*                                                           */
            //VOY /*-----------------------------------------------------------*/
            //VOY docVec = currentFileRec.getDocAssoc() ;

            //VOY if ( ( docVec != null ) && ( docVec.size() > 0 ) ) {
            //VOY    DocRec docRec = null ;
            //VOY    int    i      = 0 ;
            //VOY    int    length = docVec.size() ;

            //VOY    /* Set up a table to display the doc info in */
            //VOY    preview+="<table BORDER=0 CELLSPACING=0 CELLPADDING=0 COLS=2 WIDTH=\"100%\" >" ;

            //VOY    while ( i < length ) {
            //VOY       docRec = (DocRec) docVec.elementAt(i) ;
            //VOY       preview+="<tr><td>"+docRec.getFieldData ( FIELD_FILENAME )+"</td>" ;
            //VOY       preview+="<td>"+docRec.getFieldData ( FIELD_TITLE )+"</td></tr>" ;
            //VOY       i++ ;
            //VOY    }

            //VOY    preview+="</table><br>" ;

            //VOY } else
            //VOY    preview+=Str.getStr ( STR_NONE )+"<br>" ;


         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, just put up our plain info                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            preview = "<b>"+Str.getStr ( STR_FILENAME )+":</b> "+name+"<b>" ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Put the description                                     */        
         /*                                                           */
         /*-----------------------------------------------------------*/
         preview+="<b><u>"+Str.getStr ( STR_DESCRIPTION )+":</u></b><br>"+
                  descript+"<br><br>" ;          

         preview+="<hr>" ;



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   GIF/JPG                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( type.equals ( "GIF" ) ) ||
              ( type.equals ( "JPG" ) ) ) {
            downloadCurrentFile ( PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator ) ;
            preview+= "<img src=\"file:/"+localName+"\">" ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Default for text based                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( !type.equals ( "EXE" ) ) &&
                   ( !type.equals ( "PDF" ) ) &&
                   ( !type.equals ( "DSK" ) ) &&
                   ( !type.equals ( "ZIP" ) ) &&
                   ( !type.equals ( "INF" ) ) &&
                   ( !type.equals ( "COM" ) ) &&
                   ( !type.equals ( "DOC" ) ) &&
                   ( !type.equals ( "SYS" ) ) &&
                   ( !type.equals ( "TAR" ) ) ) {
            String fileContent = null ;

            /* Download the file and read in the content */
            downloadCurrentFile ( PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator ) ;
            fileContent = FileUtil.readFile ( localName ) ;

            fileContent = Text.replaceAllStrInStr ( fileContent, "\n", "<br>" ) ;
            preview+=fileContent+"<br>" ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Close up the HTML and view it                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         preview+="</html>" ;
         mle_PREVIEW.setText ( preview ) ;
         revalidate() ;

      } catch ( Exception e ) {
         if ( e.toString().indexOf ( "denied" ) >= 0 )
            GUISystem.printBox ( STR_INFO, STR_NO_ACCESS_TO_FILE_ON_FTP ) ;
         else if ( e.toString().indexOf ( "expired" ) >= 0 ) {
         }

         e.printStackTrace() ;
      }

      parentWin.setStatus ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* changeDir
    */
   /**
    *    This method will change the current working
    *  directory of the FTP Explorer
    *  
    *
    *  @param path The new path for the directory change
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void changeDir ( String path ) {
      try {
         ftp.cd ( path ) ;
         new Thread ( this, THREAD_REFRESHDIRECTORIES ).start() ;
      } catch ( Exception ex1 ) {
         GUISystem.printBox ( STR_INFO, STR_CANNOT_CHANGE_TO_FTP_DIR ) ;
         LogSystem.log ( 1, ex1, false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* onInternalSite
    */
   /**
    *    This method will determine if we are exploring
    *  the internal FTP site
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   private boolean onInternalSite () {
      return( ( currentFTPSite.equals ( "f3.pc.ibm.com" ) ) ||
              ( currentFTPSite.equals ( "ftp.pc.ibm.com" ) ) ) ;
   }




   /*------------------------------------------------------------*/
   /* downloadCurrentFile
    */
   /**
    *    This method will download the current file
    *  to the user's drive
    *  
    *
    *  @param dir The directory to place the file in
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean downloadCurrentFile ( String downloadDir ) {
      String   name = currentFileRec.getFilename() ;
      boolean  rc   = false ;

      //parentWin.setStatus ( STR_DOWNLOADING ) ;
      //try {

      //   if ( currentFileRec.getType().equals ( "TXT" ) )
      //      ftp.setType ( ftp.ASCII ) ;
      //   else
      //      ftp.setType ( ftp.BINARY ) ;                  


      //   /*-----------------------------------------------------------*/
      //   /*                                                           */
      //   /*   Get the file to our temp dir.                           */
      //   /*                                                           */
      //   /*-----------------------------------------------------------*/
      //   ftp.get ( name, downloadDir+name ) ;                        

      //   rc = true ;
      //} catch ( java.io.IOException ex1 ) {
      //   GUISystem.printBox ( STR_ERROR, STR_CANNOT_DOWNLOAD_DUE_TO_DISK ) ;
      //} catch ( Exception e ) {
      //   if ( e.toString().indexOf ( "denied" ) >= 0 )
      //      GUISystem.printBox ( STR_INFO, STR_NO_ACCESS_TO_FILE_ON_FTP ) ;
      //   else if ( e.toString().indexOf ( "Timeout" ) >= 0 ) {
      //      connect() ;
      //      downloadCurrentFile ( downloadDir ) ;
      //   } else
      //      LogSystem.log ( 1, e ) ;
      //}

      //parentWin.setStatus ( null ) ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* create
    */
   /**
    *    This method will upload a file to the FTP site
    *  and if that FTP Site is the internal f3.pc.ibm.com.
    *  it will add the file to the database
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void create () {
      File[] file = null ;                                

      //try {

      //   /*-----------------------------------------------------------*/
      //   /*                                                           */
      //   /*   First, get the filename and upload the file             */
      //   /*                                                           */
      //   /*-----------------------------------------------------------*/
      //   file = GUISystem.getFileOpen ( parentWin,
      //                                  Str.getStr ( STR_NEW ), 
      //                                  PropertySystem.getString ( PropertySystem.PROPERTY_FTP_SAVE_DIR ),
      //                                  "",
      //                                  true ) ;

      //   if ( ( file != null ) && ( file.length > 0 ) ) {
      //      int    type     = new GetTypeDlg ( parentWin ).getResult() ;
      //      String fullPath = file[0].getAbsolutePath() ;

      //      if ( type != GetTypeDlg.UNKNOWN ) {
      //         ftp.setType ( (char)type ) ;

      //         parentWin.setStatus ( STR_UPLOADING ) ;
      //         ftp.put ( fullPath, Text.stripPath ( fullPath ), false );                          

      //         /*-----------------------------------------------------------*/
      //         /*                                                           */
      //         /*   Now, ask for the document information                   */
      //         /*                                                           */
      //         /*-----------------------------------------------------------*/
      //         if ( onInternalSite() ) {

      //            new FTPDocDlg( parentWin,
      //                           file[0].getAbsolutePath()).getResult() ;



      //         }
      //      } /* if type was set */

      //   }

      //} catch ( Exception e ) {
      //   if ( e.toString().indexOf ( "denied" ) >0 )
      //      GUISystem.printBox ( STR_INFO, STR_NO_ACCESS_TO_FILE_ON_FTP ) ;
      //   else
      //      LogSystem.log ( 1, e );
      //}

      parentWin.setStatus ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }
   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }
   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected() {
   }




   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will add our controls after
    *  we have been started
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_CREATE ) ;
      parentWin.addActionComponent ( this, pb_DOWNLOAD ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will free our resources if
    *  we are not doing anything
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      if ( Qit.getEnabled() ) {
         removeAll() ;
         return( true ) ;
      }
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will return TRUE if we are
    *  not busy
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( Qit.getEnabled() ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return ou default nav panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( navPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.FTP_EXPLORER_SMALL ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_FTP_EXPLORER ) ) ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    this method will do nothing for now
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh() {
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }


   public String getProcessName () {
      return( "FTPExplorerPanel " ) ;
   }

   public void stopProcess () {
      continueProcessing = false ;
   }
} /* FTPExplorer */

