package com.ibm.nzna.projects.qit.ftp_explorer ;
/*-----------------------------------------------------------*/
/*   FTPFileRec                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FTPFileRec                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class holds information about a single file on an FTP Server 
 *
 *
 * @version 1.0, 10/6/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   FTPFileRec                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPFileRec extends Object implements MultiListRow, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String  filename      = null ;
   private Image   icon          = null ;
   private long    size          = 0 ;
   private boolean directory     = false ;
   private String  descript      = "" ;
   private String  timestamp     = "" ;
   private String  version       = "" ;
   private int     multimediaInd = 0 ;
   private Vector  docVec        = null ;
   private boolean databaseRead  = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default constructor creates a file record
    *  with the filename passed in
    *  
    *
    *  @param filename The name of the file that this object represents
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FTPFileRec ( String filename ) {
      setFilename ( filename ) ;
   }





   /*------------------------------------------------------------*/
   /* getColumnData
    */
   /**
    *    This method will return the data that is
    *  supposed to be in a certain column
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object object = null ;

      switch ( column ) {
         case 0 :
            if ( icon == null )
               icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_DEFAULT ).getImage() ;
            object = icon ;
            break ;

         case 1 :
            object = filename ;
            break ;


         case 2 :
            object = ""+size ;
            break ;
      }
      return( object ) ;
   }




   /*------------------------------------------------------------*/
   /* setSize
    */
   /**
    *    This method will set the size of the file
    *  represented by this file object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSize ( long size ) {
      this.size = size ;
   }





   /*------------------------------------------------------------*/
   /* getSize
    */
   /**
    *    This method will return the size of this
    *  file
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public long getSize ( ) {
      return( size ) ;
   }





   /*------------------------------------------------------------*/
   /* setIcon
    */
   /**
    *    This method will set the icon used to
    *  represent this file
    *  
    *
    *  @param icon The new icon
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIcon ( Image icon ) {
      this.icon = icon ;
   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon representing
    *  this file
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return image
    *//*---------------------------------------------------------*/
   public Image getIcon ( ) {
      return( icon ) ;
   }




   /*------------------------------------------------------------*/
   /* getFilename
    */
   /**
    *    This method will return the filename of the
    *  file that this object represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getFilename () {
      return( filename ) ;
   }





   /*------------------------------------------------------------*/
   /* setFilename
    */
   /**
    *    This method will set the filename of the
    *  file that this object represents
    *  
    *
    *  @param filename The new filename to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFilename ( String filename ) {
      this.filename = filename ;
   }




   /*------------------------------------------------------------*/
   /* autoSetIcon
    */
   /**
    *    This method will pick an icon for this
    *  file by using the file's extension. If there
    *  is no extension and the directory property is 'No',
    *  then the file will be considered executable
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void autoSetIcon () {
      if ( !directory ) {
         if ( filename != null ) {
            if ( filename.indexOf ( "." ) > 0 ) {
               String extension = getType() ;

               if ( ( extension.equals ( "EXE" )  ) ||
                    ( extension.equals ( "BAT" )  ) ||
                    ( extension.equals ( "CMD" )  ) )
                  icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_EXEC ).getImage() ;
               else if ( extension.equals ( "PDF" ) )
                  icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_PDF ).getImage() ;
               else if ( ( extension.equals ( "TXT" )  ) ||
                         ( extension.equals ( "DAT" )  ) ||
                         ( extension.equals ( "INI" )  ) ||
                         ( extension.equals ( "JPG" )  ) ||
                         ( extension.equals ( "GIF" )  ) ||
                         ( extension.equals ( "BMP" )  ) )
                  icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_TEXT ).getImage() ;
               else if ( ( extension.equals ( "WAV" )  ) ||
                         ( extension.equals ( "MP3" )  ) ||
                         ( extension.equals ( "MIDI" )  ) ||
                         ( extension.equals ( "AU" )  ) )
                  icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_WAVE ).getImage()  ;
               else if ( ( extension.equals ( "AVI" )  ) ||
                         ( extension.equals ( "QT" )  ) )
                  icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_VIDEO ).getImage() ;
               else
                  icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_DEFAULT ).getImage() ;

            }
         } else
            icon = ImageSystem.getImageIcon ( this, ImageSystem.FILE_DEFAULT ).getImage() ;
      } else
         icon = ImageSystem.getImageIcon ( this, ImageSystem.CLOSE_FOLDER ).getImage() ;
   }





   /*------------------------------------------------------------*/
   /* setDirectory
    */
   /**
    *    This method will set the directory property
    *  of this file object. If set to TRUE, then this
    *  object represents a directory
    *
    *  @param directory TRUE = Represent a directory
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDirectory ( boolean directory ) {
      this.directory = directory ;
   }




   /*------------------------------------------------------------*/
   /* getDirectory
    */
   /**
    *    This method will get the directory property
    *  of this object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean TRUE = Represents a Directory, FALSE = Represents a file
    *//*---------------------------------------------------------*/
   public boolean getDirectory () {
      return( directory ) ;
   }






   /*------------------------------------------------------------*/
   /* getType
    */
   /**
    *    This method will return the type of 
    *  file this is. The type is the extension of the
    *  file in all uppercase
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getType () {
      if ( filename.indexOf ( "." ) > 0 )
         return( filename.substring ( filename.lastIndexOf ( "." )+1, filename.length() ).toUpperCase() ) ;
      return( "" ) ;
   }





   /*------------------------------------------------------------*/
   /* setMultimediaInd
    */
   /**
    *    This method will set the multimedia indicator
    *  property. The multimedia indicator is used in the TIGRIS.MULTIMEDIA
    *  table
    *  
    *
    *  @param multimediaInd - the new indicator
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMultimediaInd ( int multimediaInd ) {
      this.multimediaInd = multimediaInd ;
   }




   /*------------------------------------------------------------*/
   /* getMultimediaInd
    */
   /**
    *    This method will return the multimedia indicator
    *  of this file.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getMultimediaInd () {
      return( multimediaInd ) ;
   }





   /*------------------------------------------------------------*/
   /* getDescript
    */
   /**
    *    This method will return the descript property
    *  that resides in quest in TIGRIS.MULTIMEDIA
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getDescript () {
      return( descript ) ;
   }





   /*------------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    This method will set the descript property
    *  of this file
    *  
    *
    *  @param descript The new description
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      
      this.descript = descript ;
   }





   /*------------------------------------------------------------*/
   /* setVersion
    */
   /**
    *    This method will set the version property
    *  of this file
    *  
    *
    *  @param version the new version
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setVersion ( String version ) {
      this.version = version ;
   }




   /*------------------------------------------------------------*/
   /* getVersion
    */
   /**
    *    This method will return the version property
    *  of this file.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getVersion () {
      return( version ) ;
   }




   /*------------------------------------------------------------*/
   /* setTimestamp
    */
   /**
    *    This method will set the time stamp for this
    *  file
    *  
    *
    *  @param timestamp The new timestamp
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTimestamp ( String timestamp ) {
      this.timestamp = timestamp ;
   }




   /*------------------------------------------------------------*/
   /* getTimestamp
    */
   /**
    *    This method will return the current timestamp property
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTimestamp () {
      return( timestamp ) ;
   }





   /*------------------------------------------------------------*/
   /* getDocAssoc
    */
   /**
    *    THis method will return all documents associated
    *  with this file
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getDocAssoc () {
      return( docVec ) ;
   }






   /*------------------------------------------------------------*/
   /* toString
    */
   /**
    *    This method will simply return the filename
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String toString () {
      return( filename ) ;
   }






   /*------------------------------------------------------------*/
   /* setDatabaseRead
    */
   /**
    *    This method will set the 'DatabaseRead'
    *  property. This property tells if the data in this
    *  file was read from the database
    *
    *  @param TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDatabaseRead ( boolean databaseRead ) {
      this.databaseRead = databaseRead ;
   }




   /*------------------------------------------------------------*/
   /* setDatabaseRead
    */
   /**
    *    This method will get the 'DatabaseRead'
    *  property. This property tells if the data in this
    *  file was read from the database
    *
    *  @return TRUE/FALSE
    *//*---------------------------------------------------------*/
   public boolean getDatabaseRead ( ) {
      return ( databaseRead ) ;
   }




} /* FTPFileRec */
