package com.ibm.nzna.projects.qit.ftp_explorer ;
/*-----------------------------------------------------------*/
/*   FTPLogonDlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FTPLogonDlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * A Dialog that asks the user to log on to an FTP Server. It will give
 * them an option to log in as anonymous
 *
 * @version 1.0, 1/30/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   FTPLogonDlg                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class FTPLogonDlg extends JDialog implements Runnable, KeyListener, ActionListener, AppConst {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextField      ef_USERID          = new JTextField      ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 32 ), "", 0 ) ;
   private JPasswordField  ef_PASSWORD        = new JPasswordField  ( "" ) ;
   private JLabel          st_HELPTEXT        = new JLabel  ( Str.getStr(STR_LOGONHELP) ) ;
   private JLabel          st_USERID          = new JLabel  ( Str.getStr(STR_USERID) ) ;
   private JLabel          st_PASSWORD        = new JLabel  ( Str.getStr(STR_PASSWORD) ) ;
   private DButton         pb_OK              = new DButton ( Str.getStr(STR_OK) ) ;
   private DButton         pb_CANCEL          = new DButton ( Str.getStr(STR_CANCEL) ) ;
   private JCheckBox       ck_ANON            = new JCheckBox ( Str.getStr ( STR_LOGON_AS_ANONYMOUS ) ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean rc       = false ;
   private String  userId   = null ;
   private String  password = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constructor                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public FTPLogonDlg ( Frame parent ) {
      super ( parent, Str.getStr(STR_LOGON), true ) ;

      setResizable ( false ) ;

      /* Since we have entry fields, make sure that the entry fields
         are tall enough */
      setSize ( 300, 160 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
   }




   /*------------------------------------------------------------*/
   /* getResponse
    */
   /**
    *    This method will pop up our window
    *  and will wait for the user to hit enter
    *  or cancel. If they hit cancel this method returns falsse
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if Ok is pressed, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean getResponse () {
      new Thread ( this ).start() ;
      setVisible ( true ) ;
      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* getUserId
    */
   /**
    *    This method will return what id the user
    *  entered
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getUserId () {
      if ( !ck_ANON.isSelected() )
         return( userId ) ;
      return( "anonymous" ) ;
   }




   /*------------------------------------------------------------*/
   /* getPassword
    */
   /**
    *    This method will return the user's password
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getPassword () {
      return( password ) ;
   }


   /*------------------------------------------------------------*/
   /* getAnonymous
    */
   /**
    *    This method will return TRUE if the user
    *  entered the anonymous id
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getAnonymous () {
      return( ck_ANON.isSelected() ) ;
   }

   /*-----------------------------------------------------------*/
   /*   createControls                                          */          
   /*                                                           */
   /*   Description:                                            */
   /*      Creates and attaches our controls                    */                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void createControls () {
      Container contentPane = getContentPane() ;

      contentPane.setLayout ( null ) ;
      contentPane.add ( st_HELPTEXT ) ;
      contentPane.add ( st_USERID ) ;
      contentPane.add ( ef_USERID ) ;
      contentPane.add ( st_PASSWORD ) ;
      contentPane.add ( ef_PASSWORD ) ;
      contentPane.add ( ck_ANON ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_CANCEL ) ;

      pb_OK.addKeyListener ( this ) ;
      pb_CANCEL.addKeyListener ( this ) ;
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      ck_ANON.addActionListener ( this ) ;
      ef_USERID.addActionListener ( this ) ;
      ef_PASSWORD.addActionListener ( this ) ;
      ck_ANON.addActionListener ( this ) ;

   }


   /*-----------------------------------------------------------*/
   /* layoutControls
    */
   /**
    *    Moves and resizes all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       x         = 10 ;
      int       y         = 45 ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       cx        = rowHeight ;
      Dimension size      = getSize() ;

      super.doLayout() ;

      st_HELPTEXT.setBounds ( x, 0, size.width-20, 45 ) ;
      st_USERID.setBounds ( x, y, 100, cx ) ;
      ef_USERID.setBounds ( x+100, y, 100, cx ) ;
      y+=cx ;

      st_PASSWORD.setBounds ( x, y, 100, cx ) ;
      ef_PASSWORD.setBounds ( x+100, y, 100, cx ) ;
      y+=rowHeight+2 ;

      ck_ANON.setBounds ( x, y, 200, rowHeight ) ;
      y+=rowHeight+34 ;

      pb_OK.setBounds ( x, y, 70, 25 ) ;
      pb_CANCEL.setBounds ( x+75, y, 70, 25 ) ;
   }


   /*-----------------------------------------------------------*/
   /*   buttonHit                                               */     
   /*                                                           */
   /*   Description:                                            */
   /*      Responds to buttons being pressed                    */                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( e.getSource() == pb_OK ) || 
           ( e.getSource () == ef_PASSWORD ) ||
           ( e.getSource () == ef_USERID ) )
         processOk() ;
      else if ( e.getSource() == pb_CANCEL )
         dispose() ;

      else if ( e.getSource() == ck_ANON ) {
         boolean selected = ck_ANON.isSelected() ;

         ef_PASSWORD.setEnabled ( !selected ) ;
         ef_USERID.setEnabled ( !selected ) ;

         if ( selected ) {
            ef_USERID.setText ( "anonymous" ) ;
         } else
            ef_USERID.setText ( "" ) ;
      }
   } /* buttonHit */




   /*------------------------------------------------------------*/
   /* processOk
    */
   /**
    *    this method will process our Ok button
    *  hit
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void processOk () {
      if ( saveData() ) {
         rc = true ;
         dispose() ;
      }
   }



   /*-----------------------------------------------------------*/
   /*   saveData                                                */    
   /*                                                           */
   /*   Description:                                            */
   /*      Saves the data on the window                         */                   
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      TRUE if everything was saved                         */   
   /*-----------------------------------------------------------*/
   public boolean saveData () {
      boolean  rc       = false ;

      userId = ef_USERID.getText() ;
      password = ef_PASSWORD.getText() ;

      if ( !ck_ANON.isSelected() ) {

         if ( ( userId == null ) || ( userId.length() < 1 ) ) {
            GUISystem.printBox ( Str.getStr(STR_ERROR), Str.getStr(STR_MUST_ENTER_USERID) ) ;
         } else if ( ( password == null ) || ( password.length() < 1 ) ) {
            GUISystem.printBox ( Str.getStr(STR_ERROR), Str.getStr(STR_MUST_ENTER_PASSWORD) ) ;
         } else {
            rc = true ;
         }
      } else
         rc = true ;

      return( rc ) ;
   } /* saveData */




   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    Refreshes all of the data on our window
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData () {
   }

   public void keyReleased ( KeyEvent e ){}
   public void keyTyped ( KeyEvent e ){}





   /*------------------------------------------------------------*/
   /* keyPressed
    */
   /**
    *    This method will be called when a key
    *  is pressed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void keyPressed ( KeyEvent e ) {

      switch ( e.getKeyCode() ) {
         

         case KeyEvent.VK_ENTER :
            if ( e.getSource() == pb_OK )
               processOk() ;
            else if ( e.getSource() == pb_CANCEL ) {
               dispose() ;
            }
            break ;

         case KeyEvent.VK_SPACE :
            if ( e.getSource() == pb_OK )
               processOk() ;
            else if ( e.getSource() == pb_CANCEL ) {
               dispose() ;
            }
            break ;


      }
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread simply waits 200 milliseconds
    *  and then refreshes our data.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      createControls() ;

      try {
         Thread.currentThread().sleep ( 200 ) ;
      } catch ( Exception e ) {
      }
      refreshData() ;
      getContentPane().setVisible ( false ) ;
      getContentPane().setVisible ( true ) ;
   }

} /* FTPLogonDlg */
