package com.ibm.nzna.projects.qit.ftp_explorer ;
/*-----------------------------------------------------------*/
/*   GetTypeDlg                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: GetTypeDlg                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog will ask the user what type of upload/download they want (Binary or Ascii) 
 *
 *
 * @version 1.0, 11/11/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.ftp.* ;
import com.ibm.nzna.shared.ftp.FTP.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   GetTypeDlg                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class GetTypeDlg extends JDialog implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int UNKNOWN = 0 ;
   public static final int ASCII   = Ftp.ASCII ;
   public static final int BINARY  = Ftp.BINARY ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JRadioButton rb_BINARY         = null ;
   private JRadioButton rb_ASCII          = null ;
   private DButton      pb_OK             = null ;
   private DButton      pb_CANCEL         = null ;
   private JLabel       st_HELP           = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int rc                         = UNKNOWN ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize our window
    *  but will not show it. To show the window and
    *  get the data the user entered, please call 'getResult'
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public GetTypeDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_SELECT_FTP_TYPE ), true ) ;

      createControls() ;

      setSize ( 300, 200 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the user the window and
    *  will not return until the user has acted on the window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UNKNOWN, ASCII or BINARY
    *//*---------------------------------------------------------*/
   public int getResult () {
      setVisible ( true ) ;

      return ( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create all of our controls and
    *  will then add each control to the GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createControls () {        
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rb_ASCII  = new JRadioButton ( Str.getStr ( STR_ASCII ) ) ;
      rb_BINARY = new JRadioButton ( Str.getStr ( STR_BINARY ) ) ;
      pb_OK     = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      st_HELP   = new JLabel ( Str.getStr ( STR_SELECT_FTP_TYPE ) ) ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Aciton Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to the GUI                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_HELP ) ;
      contentPane.add ( rb_ASCII ) ;
      contentPane.add ( rb_BINARY ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_CANCEL ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       y         = 5 ;
      int       x         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_HELP.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight*2 ;

      rb_ASCII.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      rb_BINARY.setBounds ( x, y, size.width-(x*2), rowHeight ) ;

      pb_OK.setBounds ( x, size.height-50, 70, 25 ) ;
      pb_CANCEL.setBounds ( x+75, size.height-50, 75, 25 ) ;
   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  is performed in this window
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_OK )
         rc = ( (rb_ASCII.isSelected()) ? ASCII : BINARY ) ;
      else
         rc = UNKNOWN ;

      dispose() ;
   }


} /* GetTypeDlg */
