package com.ibm.nzna.projects.qit.gui ;       
/*-----------------------------------------------------------*/
/*   ActionButton                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ActionButton                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing.* ;
import java.awt.* ;
/**
 * This is a subclass of DButton to provide a uniform look and
 * feel to all action buttons in the Quest Application. Action
 * buttons are normally located directly above a QuestPanel and
 * contain buttons specific to that panel
 *
 *
 * @version 1.0, 1/18/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ActionButton                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class ActionButton extends DButton implements PropertyListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final Insets insets = new Insets ( 2,2,2,2 ) ;
   private int iconIndex = 0 ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the default
    *  Quest ActionButton
    *  
    *
    *  @param text The text to show on the button
    *  @param image The image to show just to the left of the text
    *  @param help The flyover help to show
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ActionButton ( String text, ImageIcon image, String help ) {
      super ( text, image ) ;

      try {
         boolean skinned = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;                 

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our property listeners                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_ACTIONBUTTONCOLOR, this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SKINNAME,          this ) ;         
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_BUTTONFONT,        this ) ;         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our properties                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         propertyChanged ( PropertySystem.PROPERTY_ACTIONBUTTONCOLOR, null ) ;
         propertyChanged ( PropertySystem.PROPERTY_SKINNAME,          null ) ;

         setToolTipText ( help ) ;
         setBorderPainted ( false ) ;
         setFocusPainted ( false ) ;
         setFont ( (Font)UIManager.getDefaults().get ( "Button.font" ) ) ;
         setOpaque ( false ) ;
         setContentAreaFilled ( false ) ;
         setHorizontalAlignment ( RIGHT ) ;
         setHorizontalAlignment ( JButton.LEADING ) ;
         setMargin ( insets ) ;
         actAsLink ( true ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* setIcon
    */
   /**
    *    Subclassed to get the actual image
    *  we are displaying
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIcon ( ImageIcon icon ) {
      super.setIcon ( icon ) ;
   }



   /*------------------------------------------------------------*/
   /* finalize
    */
   /**
    *    This method will remove all of our listeners
    *  and destroy
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finalize () {
      try {
         PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_ACTIONBUTTONCOLOR, this ) ;
         PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;
         PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_BUTTONFONT, this ) ;
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening to has changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PROPERTY_ACTIONBUTTONCOLOR                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( property == PropertySystem.PROPERTY_ACTIONBUTTONCOLOR )
         setForeground ( (Color)PropertySystem.get ( PropertySystem.PROPERTY_ACTIONBUTTONCOLOR ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PROPERTY_SKINNAME                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( property == PropertySystem.PROPERTY_SKINNAME ) {
         if ( iconIndex > 0 )
            setIcon ( ImageSystem.getImageIcon ( this, iconIndex ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   PROPERTY_BUTTONFONT                                     */          
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( property == PropertySystem.PROPERTY_BUTTONFONT ) {
         setFont ( (Font)UIManager.getDefaults().get ( "Button.font" ) ) ;
      }
      

   }
} /* ActionButton */
