package com.ibm.nzna.projects.qit.gui ;              
/*-----------------------------------------------------------*/
/*   AppDefaultMenuBar                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AppDefaultMenuBar                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;      
import java.awt.event.* ;
import java.awt.* ;
import java.io.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
/**
 * This menu bar will hold all default and QuestApplet menu items for the AppDefaultWin. 
 *
 *
 * @version 1.0, 1/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   AppDefaultMenuBar                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class AppDefaultMenuBar extends JMenuBar implements Runnable, AppConst, PropertyListener {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Menu Items                                              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JMenu     debugMenu             = null ;
   private JMenu     fileMenu              = null ;
   private JMenu     editMenu              = null ;
   private JMenu     helpMenu              = null ;
   private JMenu     createMenu            = null ;
   private JMenu     propertyMenu          = null ;
   private JMenu     cutMenu               = null ;
   private JMenu     copyMenu              = null ;
   private JMenu     pasteMenu             = null ;
   private JMenuItem refreshMenuItem       = null ;
   private JMenuItem appPropertyMenuItem   = null ;
   private JMenuItem userPropertyMenuItem  = null ;
   private JMenuItem cutTextMenuItem       = null ;
   private JMenuItem copyTextMenuItem      = null ;
   private JMenuItem pasteTextMenuItem     = null ;
   private JMenuItem cutSpecialMenuItem    = null ;
   private JMenuItem copySpecialMenuItem   = null ;
   private JMenuItem pasteSpecialMenuItem  = null ;
   private JMenuItem exitMenuItem          = null ;
   private JMenuItem aboutMenuItem         = null ;
   private JMenuItem helpIndexMenuItem     = null ;
   private JMenuItem helpOnHelpMenuItem    = null ;
   private JMenuItem tutorial              = null ;
   private JMenuItem databaseItem          = null ;
   private JMenuItem debugItem             = null ;
   private JMenuItem editBookmarkMenuItem  = null ;
   private Image[]   animationArray        = null ;
   private Thread    animationThread       = null ;
   private boolean   processAnimation      = false ;
   private int       animationStep         = 0 ;
   private int       animationMax          = 39 ;
   private boolean   stopThread            = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the default
    *  menu items
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AppDefaultMenuBar () {
      super() ;                


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Load the animation Images                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      loadImages() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our default menu                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      createDefaultMenu() ;                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add ourselves as a language property listener           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
   }





   /*------------------------------------------------------------*/
   /* createDefaultMenu
    */
   /**
    *    This method will create the default menu
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createDefaultMenu () {
      try {
         fileMenu   = new JMenu ( "" ) ;
         editMenu   = new JMenu ( "" ) ;
         helpMenu   = new JMenu ( "" ) ;
         debugMenu  = new JMenu ( "Debug" ) ;
         createMenu = new JMenu ( "" ) ;

         fileMenu.add ( propertyMenu    = new JMenu ( "" ) ) ;
         fileMenu.add ( refreshMenuItem = new JMenuItem ( "" ) ) ;
         fileMenu.addSeparator() ;
         fileMenu.add ( exitMenuItem    = new JMenuItem ( "" ) ) ;

         propertyMenu.add ( appPropertyMenuItem  = new JMenuItem ( "" ) ) ;
         propertyMenu.add ( userPropertyMenuItem = new JMenuItem ( "" ) ) ;

         editMenu.add ( cutMenu       = new JMenu ( "" ) ) ;
         editMenu.add ( copyMenu      = new JMenu ( "" ) ) ;
         editMenu.add ( pasteMenu     = new JMenu ( "" ) ) ;
         editMenu.addSeparator() ;
         editMenu.add ( editBookmarkMenuItem = new JMenuItem ( "" ) ) ;

         cutMenu.add   ( cutTextMenuItem   = new JMenuItem () ) ;
         copyMenu.add  ( copyTextMenuItem  = new JMenuItem () ) ;
         pasteMenu.add ( pasteTextMenuItem = new JMenuItem () ) ;
         cutMenu.add   ( cutSpecialMenuItem   = new JMenuItem () ) ;
         copyMenu.add  ( copySpecialMenuItem  = new JMenuItem () ) ;
         pasteMenu.add ( pasteSpecialMenuItem = new JMenuItem () ) ;

         helpMenu.add ( helpIndexMenuItem  = new JMenuItem () ) ;
         helpMenu.add ( helpOnHelpMenuItem = new JMenuItem () ) ;
         helpMenu.addSeparator() ;
         //helpMenu.add ( tutorial           = new JMenuItem ( Str.getStr ( STR_TUTORIAL ) ) ) ;
         //helpMenu.addSeparator() ;
         //helpMenu.add ( aboutMenuItem      = new JMenuItem () ) ;


         add ( fileMenu ) ;
         add ( editMenu ) ;
         add ( createMenu ) ;
         add ( helpMenu ) ;

         if ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEBUGLEVEL ) > 0 ) {
            add ( debugMenu ) ;
            debugMenu.add ( databaseItem = new JMenuItem ( "Database System" ) ) ;
            //debugMenu.add ( debugItem    = new JMenuItem ( "Debug File" ) ) ;
         }

         setText() ;
         //GUISystem.setPropertiesOnPanel ( this ) ;
         setBackground ( helpMenu.getBackground() ) ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when the
    *  language property or skin property
    *  change has been made
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {

      if ( property == PropertySystem.PROPERTY_LANGUAGE )
         setText() ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of
    *  all menu items
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText ( ) {
      fileMenu.setText ( Str.getStr ( STR_FILE ) ) ;
      editMenu.setText ( Str.getStr ( STR_EDIT ) ) ;
      helpMenu.setText ( Str.getStr ( STR_HELP ) ) ;
      createMenu.setText ( Str.getStr ( STR_CREATE ) ) ;


      propertyMenu.setText            ( Str.getStr ( STR_PROPERTIES    ) ) ;
      cutMenu.setText                 ( Str.getStr ( STR_CUT           ) ) ;
      copyMenu.setText                ( Str.getStr ( STR_COPY          ) ) ;
      pasteMenu.setText               ( Str.getStr ( STR_PASTE         ) ) ;
      refreshMenuItem.setText         ( Str.getStr ( STR_REFRESH       ) ) ;
      appPropertyMenuItem.setText     ( Str.getStr ( STR_APPLICATION   ) ) ;
      userPropertyMenuItem.setText    ( Str.getStr ( STR_USER          ) ) ;
      cutTextMenuItem.setText         ( Str.getStr ( STR_CUT_TEXT      ) ) ;
      copyTextMenuItem.setText        ( Str.getStr ( STR_COPY_TEXT     ) ) ;
      pasteTextMenuItem.setText       ( Str.getStr ( STR_PASTE_TEXT    ) ) ;
      cutSpecialMenuItem.setText      ( Str.getStr ( STR_CUT_SPECIAL   ) ) ;
      copySpecialMenuItem.setText     ( Str.getStr ( STR_COPY_SPECIAL  ) ) ;
      pasteSpecialMenuItem.setText    ( Str.getStr ( STR_PASTE_SPECIAL ) ) ;
      helpIndexMenuItem.setText       ( Str.getStr ( STR_HELP_INDEX    ) ) ;
      helpOnHelpMenuItem.setText      ( Str.getStr ( STR_HELP_ON_HELP  ) ) ;
      //aboutMenuItem.setText           ( Str.getStr ( STR_ABOUT         ) ) ;
      exitMenuItem.setText            ( Str.getStr ( STR_EXIT          ) ) ;
      editBookmarkMenuItem.setText    ( Str.getStr ( STR_BOOKMARKS     ) ) ;
      //tutorial.setText                ( Str.getStr ( STR_TUTORIAL      ) ) ;

      GUISystem.setPropertiesOnPanel ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener
    *  to all default menu items.
    *  
    *
    *  @param listener The ActionListener
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      refreshMenuItem.addActionListener      ( listener ) ;
      appPropertyMenuItem.addActionListener  ( listener ) ;
      userPropertyMenuItem.addActionListener ( listener ) ;
      cutTextMenuItem.addActionListener      ( listener ) ;
      copyTextMenuItem.addActionListener     ( listener ) ;
      pasteTextMenuItem.addActionListener    ( listener ) ;
      cutSpecialMenuItem.addActionListener   ( listener ) ;
      copySpecialMenuItem.addActionListener  ( listener ) ;
      pasteSpecialMenuItem.addActionListener ( listener ) ;
      exitMenuItem.addActionListener         ( listener ) ;
      //aboutMenuItem.addActionListener        ( listener ) ;
      helpIndexMenuItem.addActionListener    ( listener ) ;
      helpOnHelpMenuItem.addActionListener   ( listener ) ;
      editBookmarkMenuItem.addActionListener ( listener ) ;
      //tutorial.addActionListener             ( listener ) ;

      if ( databaseItem != null )
         databaseItem.addActionListener ( listener ) ;
      if ( debugItem != null )
         debugItem.addActionListener ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* addCutMenuItem
    */
   /**
    *    This method will add a custom
    *  menu item to the cut menu
    *  
    *
    *  @param menuItem The JMenuItem to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCutMenuItem ( JComponent menuItem ) {
      cutMenu.add ( menuItem ) ;
   }




   /*------------------------------------------------------------*/
   /* removeCutMenuItem
    */
   /**
    *    This method will remove a custom
    *  menu item from the cut menu
    *  
    *
    *  @param menuItem The JMenuItem to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCutMenuItem ( JComponent menuItem ) {
      cutMenu.remove ( menuItem ) ;
   }


   /*------------------------------------------------------------*/
   /* addPasteMenuItem
    */
   /**
    *    This method will add a custom
    *  menu item to the paste menu
    *  
    *
    *  @param menuItem The JMenuItem to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addPasteMenuItem ( JComponent menuItem ) {
      pasteMenu.add ( menuItem ) ;
   }




   /*------------------------------------------------------------*/
   /* removePasteMenuItem
    */
   /**
    *    This method will remove a custom
    *  menu item from the paste menu
    *  
    *
    *  @param menuItem The JMenuItem to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removePasteMenuItem ( JComponent menuItem ) {
      pasteMenu.remove ( menuItem ) ;
   }


   /*------------------------------------------------------------*/
   /* addCopyMenuItem
    */
   /**
    *    This method will add a custom
    *  menu item to the copy menu
    *  
    *
    *  @param menuItem The JMenuItem to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCopyMenuItem ( JComponent menuItem ) {
      copyMenu.add ( menuItem ) ;
   }




   /*------------------------------------------------------------*/
   /* removeCopyMenuItem
    */
   /**
    *    This method will remove a custom
    *  menu item from the copy menu
    *  
    *
    *  @param menuItem The JMenuItem to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeCopyMenuItem ( JComponent menuItem ) {
      copyMenu.remove ( menuItem ) ;
   }



   /*------------------------------------------------------------*/
   /* addFileMenuItem
    */
   /**
    *    This method will add a custom
    *  menu item to the file menu
    *  
    *
    *  @param menuItem The JMenuItem to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addFileMenuItem ( JComponent menuItem ) {
      fileMenu.add ( menuItem ) ;
   }




   /*------------------------------------------------------------*/
   /* removeFileMenuItem
    */
   /**
    *    This method will remove a custom
    *  menu item from the file menu
    *  
    *
    *  @param menuItem The JMenuItem to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeFileMenuItem ( JComponent menuItem ) {
      fileMenu.remove ( menuItem ) ;
   }


   /*------------------------------------------------------------*/
   /* addHelpMenuItem
    */
   /**
    *    This method will add a custom
    *  menu item to the help menu
    *  
    *
    *  @param menuItem The JMenuItem to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addHelpMenuItem ( JMenuItem menuItem ) {
      helpMenu.add ( menuItem ) ;
   }




   /*------------------------------------------------------------*/
   /* removeHelpMenuItem
    */
   /**
    *    This method will remove a custom
    *  menu item from the help menu
    *  
    *
    *  @param menuItem The JMenuItem to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeHelpMenuItem ( JComponent menuItem ) {
      helpMenu.remove ( menuItem ) ;
   }


   /*------------------------------------------------------------*/
   /* addEditMenuItem
    */
   /**
    *    This method will add a custom
    *  menu item to the edit menu
    *  
    *
    *  @param menuItem The JMenuItem to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addEditMenuItem ( JComponent menuItem ) {
      editMenu.add ( menuItem ) ;
   }




   /*------------------------------------------------------------*/
   /* removeEditMenuItem
    */
   /**
    *    This method will remove a custom
    *  menu item from the edit menu
    *  
    *
    *  @param menuItem The JMenuItem to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeEditMenuItem ( JComponent menuItem ) {
      editMenu.remove ( menuItem ) ;
   }


   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will repaint our component
    *  and animation image
    *  
    *
    *  @param g The Graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      Rectangle clip = (Rectangle)g.getClip() ;
      Dimension size = getSize() ;

      /* Draw our background  */
      g.setColor ( getBackground() ) ;
      g.fillRect ( clip.x, clip.y, clip.width, clip.height ) ;

      /* Draw our image  */
      if ( clip.x < size.width-100 ) {
         try {
            g.drawImage ( animationArray[animationStep], size.width-63, 2, this ) ;
         } catch ( ArrayIndexOutOfBoundsException e ) {
         }
      }

   }

   public Dimension getPreferredSize () {
      return( new Dimension ( 100, 25 ) ) ;
   }
   public Dimension getMinimumSize () {
      return( new Dimension ( 100, 25 ) ) ;
   }





   /*------------------------------------------------------------*/
   /* loadImages
    */
   /**
    *    This method will load the images that
    *  we will use to display an animation with
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void loadImages () {
      try {
         MediaTracker tracker = new MediaTracker(this);
         Toolkit toolKit = Toolkit.getDefaultToolkit() ;
         int     i       = 0 ;
         String  dir     = PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR )+File.separator+
                           "skins"+File.separator+
                           PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME ) ;

         animationArray = new Image[animationMax] ;

         while ( i < animationMax ) {
            animationArray[i] = toolKit.getImage ( dir+File.separator+"world"+(i+1)+".gif" ) ;
            tracker.addImage(animationArray[i], 0);
            tracker.waitForID(0);
            i++ ;
         }
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }





   /*------------------------------------------------------------*/
   /* finalize
    */
   /**
    *    This method has been subclassed to stop
    *  the animation thread
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finalize () {
      stopThread = true ;
      if ( animationThread != null )
         animationThread.interrupt() ;
      try {
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will run our animation. If the
    *  processAnimation flag is false, it will simply
    *  sleep for 5 minutes. If you wish animation to start, you must interupt
    *  the thread and set the processAnimation to true
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      Thread.currentThread().setPriority ( Thread.MIN_PRIORITY ) ;

      while ( !stopThread ) {

         /* If we are suppsed to process our animation, do
            so now  */
         if ( processAnimation ) {


            /* Bump our animation step  */
            for ( int i = 0; i < animationMax; i++ ) {

               /* Repaint  */
               repaint() ;

               animationStep++ ;

               if ( animationStep >= animationMax )
                  animationStep=0 ;

               /* Sleep for a sec  */
               try {
                  Thread.currentThread().sleep ( 100 ) ;
               } catch ( Exception e ) {
               }
            }
            animationStep=0 ;
            repaint() ;

         }

         /* otherwise, wait for 5 minutes  */
         else {

            try {
               Thread.currentThread().sleep ( 600000 ) ;
            } catch ( Exception e ) {
            }
         } /* else  */

      } /* while !stopThread  */

   } /* run  */





   /*------------------------------------------------------------*/
   /* startAnimation
    */
   /**
    *    This method will start the animation
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void startAnimation () {

      processAnimation = true ;

      if ( animationThread == null ) {
         animationThread = new Thread ( this ) ;
         animationThread.start() ;
      } else {
         animationThread.interrupt() ;
      }

   }





   /*------------------------------------------------------------*/
   /* stopAnimation
    */
   /**
    *    This method will stop our animation
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stopAnimation () {
      processAnimation = false ;
      repaint() ;
   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will free all of our resources
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
   }





   /*------------------------------------------------------------*/
   /* removeMenu
    */
   /**
    *    This method will remove the menu item from
    *  the menu
    *  
    *
    *  @param menuItem the menu item to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeMenu ( JMenu menu ) {
      remove ( menu ) ;
   }



   /*------------------------------------------------------------*/
   /* addMenuItem
    */
   /**
    *    This method will add a menu item in between the
    *  'Edit' and 'Help' menu items
    *  
    *
    *  @param menu The menu to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addMenu ( JMenu menu ) {
      remove ( helpMenu ) ;
      if ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEBUGLEVEL ) > 0 )
         remove ( debugMenu ) ;

      add ( menu ) ;

      add ( helpMenu ) ;

      if ( PropertySystem.getInt ( PropertySystem.PROPERTY_DEBUGLEVEL ) > 0 )
         add ( debugMenu ) ;

      GUISystem.setPropertiesOnPanel ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* addCreateMenuItem
    */
   /**
    *    This method will add an item under the
    *  'Create' menu
    *  
    *
    *  @param menuItem The JMenuItem to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addCreateMenuItem ( JMenuItem menuItem ) {
      createMenu.add ( menuItem ) ;
   }


   /**
     *  This method will enable/disable the Cut menu item
     **/
   public void enableCut ( boolean enable ) {
      cutTextMenuItem.setEnabled ( enable ) ;
      cutSpecialMenuItem.setEnabled ( enable ) ;
   }

   /**
     *  This method will enable/disable the Copy menu item
     **/
   public void enableCopy ( boolean enable ) {
      copyTextMenuItem.setEnabled ( enable ) ;
      copySpecialMenuItem.setEnabled ( enable ) ;
   }


   /**
     *  This method will enable/disable the Paste menu item
     **/
   public void enablePaste ( boolean enable ) {
      pasteTextMenuItem.setEnabled ( enable ) ;
      pasteSpecialMenuItem.setEnabled ( enable ) ;
   }

} /* AppDefaultMenuBar */
