package com.ibm.nzna.projects.qit.gui ;         
/*-----------------------------------------------------------*/
/*   AppDefaultWin                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: AppDefaultWin                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.util.* ;
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.help.* ;
import com.ibm.nzna.projects.qit.workspace.* ;
import com.ibm.nzna.projects.qit.chat.* ;
import com.ibm.nzna.projects.qit.help.tutor.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.app.appProp.* ;
import com.ibm.nzna.projects.qit.app.userProp.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import java.sql.* ;
/**
 * The Application Default Window is subclassed by every
 * window in the Quest Application. The Default Window houses
 * the Default Toolbars, Status Bars and DataPanel.
 *
 *
 * @version 1.0, 1/8/2000 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   AppDefaultWin                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class AppDefaultWin extends JFrame implements TypeCategoryListener, TypeListStatusListener, Runnable, AppConst, WindowListener, ActionListener, PropertyListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Thread names                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_MENUACTION   = "MA" ;
   private static final String THREAD_HOLD_STATUS  = "HS" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StatusBar         StatusBar            = null ;  // The status area at the bottom of the window
   private AppDefaultMenuBar defaultMenuBar       = null ;  // The default menu bar for the application
   private String            lastMenuAction       = null ;  // The last menu action that was received
   private AntiAliasPanel    contentPanel         = null ;  // The content panel holding any ApplicationPanels
   private ApplicationPanel  currentPanel         = null ;  // The current panel that is showing
   private ApplicationPanel  previousPanel        = null ;  // The panel that was previously showing
   private ButtonPanel       focusButtonPanel     = null ;  // The panel that holds all buttons representing panels
   private AntiAliasPanel    focusPanel           = null ;  // The panel which holds the focus bar and content panel
   private int               panelCount           = 0 ;     // The number of QuestPanels that this window has. 0 if in MDI mode
   private Vector            focusButtonVec       = null ;  // The vector holding all open FocusButtons
   private boolean           isDisposed           = false ; // TRUE = The Dispose method was called and the window was destroyed
   private Vector            applicationPanelVec  = null  ; // The vector holding all open Application Panels
   private boolean           statusLocked         = false ; // If TRUE, do not ever change the status text
   private boolean           showFocusPanel       = true  ; // Default to TRUE. If FALSE, the focus buttons will never show.
   private boolean           skinned              = false ; // TRUE if we are showing a skin
   private int               holdStatusSeconds    = 0 ;     // IF we are to hold the current status, for this many seconds
   private Vector            focusHistory         = null  ; // The history in which Focus changes took place.
   private boolean           statusChanged        = false ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Protected                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected AppDefaultToolBar toolBar         = null ;              

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Default Toolbar Buttons                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ToolBarButton pb_REFRESH            = null ;
   private ToolBarButton pb_BOOKMARK           = null ;
   private ToolBarButton pb_HELP               = null ;
   private ToolBarButton pb_PRINT              = null ;
   private ToolBarButton pb_CHAT               = null ;
   private ToolBarButton pb_SEND_DEBUG         = null ;
   private ToolBarButton pb_STOP               = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the window to a blank window.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AppDefaultWin () {
      initialize() ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize the window to a blank window,
    *  it will also set the read only property 'showFocusPanel'
    *
    *  @param showFocusPanel TRUE = Show the focus panel if not in MDI
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AppDefaultWin ( boolean showFocusPanel ) {
      this.showFocusPanel = showFocusPanel ;
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a new App Default
    *  Window initialized to the panel passed in. Please not
    *  that the panel passed to this constructor will be added directly
    *  to the contentPane of this JFrame so there will be no tabbed notebook
    *  for the panel. This constructor should only be called if the PropretySystem.PROPERTY_MDI
    *  is true.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public AppDefaultWin ( QuestPanel panel ) {
      super () ;
      initialize() ;
      addPanel ( panel ) ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our AppDefaultWin. It
    *  will create our panels and adjust our properties
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         skinned = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ; 
         setDefaultCloseOperation ( DO_NOTHING_ON_CLOSE ) ;                 
         addWindowListener ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create the default controls                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         StatusBar        = new StatusBar () ;       
         focusButtonPanel = new ButtonPanel () ;
         focusPanel       = new AntiAliasPanel () ;        
         focusHistory     = new Vector () ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         focusButtonPanel.setPreferredSize ( new Dimension ( 0, 13 ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our controls                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         getContentPane().setLayout ( new BorderLayout (0,0) ) ;

         focusPanel.setLayout ( new BorderLayout(0,0 ) ) ;

         if ( !PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) {
            if ( showFocusPanel )
               focusPanel.add ( focusButtonPanel, BorderLayout.NORTH ) ;
         }
         getContentPane().add ( StatusBar, BorderLayout.SOUTH ) ;       
         getContentPane().add ( focusPanel, BorderLayout.CENTER ) ;
         setDefaultMenu() ;
         setDefaultToolbar() ;    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         TypeCategory.addTypeCategoryListener ( this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_ANTI_ALIAS,             this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_TOOLBAR_SHOW_ICON_TEXT, this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_TOOLBAR_HELP,           this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_TOOLBAR_TEXT_ORIENT,    this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_BUTTONFONT,             this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE,               this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SHOWSKIN,               this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SKINNAME,               this ) ;
         TypeList.getInstance().addStatusListener ( this ) ;                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust all properties on all GUI                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* addPanel
    */
   /**
    *    This method will add a QuestPanel to this
    *  Window instance. This method will kick off a thread to do
    *  the work so it will instantly return. DO NOT CALL THIS DIRECTLY.
    *  You should only call WindowSystem.createPanel  ( questPanel ).
    *  
    *
    *  @param questPanel The QuestPanel is add
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addPanel ( QuestPanel questPanel ) {
      new QuestPanelInitializeThread ( this, questPanel ) ;
   }

   /*------------------------------------------------------------*/
   /* addPanel
    */
   /**
    *    This method will add a panel to this
    *  window. Each frame added will produce
    *  a 'Focus Button' on the task bar at the top of the window
    *
    *  @param panel The ApplicationPanel to add
    *  @param title The title of this panel
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void addPanel ( ApplicationPanel panel, String title ) {
      try {
         boolean mdi = PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are in MDI (Multiple Document Interface) mode,    */
         /*   bring up a new window, unless we do not have a panel    */
         /*   yet. If we do not have a currentPanel  here, make this   */
         /*   panel our currentPanel                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( mdi )
            setSinglePanel ( panel, title ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, add this panel to our tabbed pane            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Initialize the Tabbed Pane                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( initializeContentPanel() ) {
               FocusButton focusButton = null ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Create the Focus Button for the task bar                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               contentPanel.add ( title+panelCount, panel ) ;
               if ( panel.getQuestPanel().getIcon() != null )
                  focusButton = new FocusButton ( title, panel.getQuestPanel().getIcon(), title+panelCount, panel ) ;
               else
                  focusButton = new FocusButton ( title, title+panelCount, panel ) ;                           


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the Focus Button to the task bar                    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               focusButtonPanel.add ( focusButton ) ;
               focusButtonVec.addElement ( focusButton ) ;
               applicationPanelVec.addElement ( panel ) ;
               focusButton.setFocus ( true ) ;
               defocusFocusButtons ( panel ) ;                          

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add this as the last selected panel                     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               focusHistory.addElement ( panel ) ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the new ApplicationPanel to our window              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               ((CardLayout)contentPanel.getLayout()).show ( contentPanel, title+panelCount ) ;
               if ( currentPanel != null ) {
                  currentPanel.panelDeselected() ;
                  previousPanel = currentPanel ;
               } else
                  previousPanel = panel ;
               currentPanel = panel ; 
               refreshEditMenu() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Increment our count and repaint                         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               panelCount++ ;
               revalidate() ;
            }

         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_CANNOT_OPEN_PANEL )+" "+title ) ;
      }
   }


   /*------------------------------------------------------------*/
   /* windowDeactivated
    */
   /**
    *    This method will be called when this
    *  window is to be deactivated
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowDeactivated ( WindowEvent e ) {
   }





   /*------------------------------------------------------------*/
   /* windowActivated
    */
   /**
    *    This method will be called when the
    *  window is going to be activated
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowActivated ( WindowEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* windowOpened
    */
   /**
    *    This method will be called when this
    *  window is opened
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowOpened ( WindowEvent e ) {
   }





   /*------------------------------------------------------------*/
   /* windowClosed
    */
   /**
    *    This method will be called as one of the
    *  last methods once this window is closed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowClosed ( WindowEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* windowDeiconified
    */
   /**
    *    This method will be called when this
    *  window is risen from an icon state
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowDeiconified ( WindowEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* windowIconified
    */
   /**
    *    This method will be called when this
    *  window is iconified
    *  
    *
    *  @param e The WindowEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowIconified ( WindowEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* windowClosing
    */
   /**
    *    This method will be called when something
    *  wants to close this window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowClosing ( WindowEvent e ) {
      dispose() ;
   }





   /*------------------------------------------------------------*/
   /* revalidate
    */
   /**
    *    This method will revalidate all component
    *  positions and will repaint the window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void revalidate () {
      defaultMenuBar.revalidate() ;
      toolBar.revalidate() ;

      if ( currentPanel != null )
         currentPanel.revalidate() ;
   }



   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the status text of the
    *  status label at the bottom of a AppDefaultWin
    *  
    *
    *  @param text The text to set, NULL if nothing or needs to clear
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatus ( String text ) {
      if ( !statusLocked ) {
         setStatus ( text, true ) ;
         statusChanged = true ;
      }
   }



   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the status text of the
    *  status label at the bottom of a AppDefaultWin to the
    *  indicator passed in. This indicator will be used to look up
    *  the actual string in the language file
    *  
    *
    *  @param id The id of the string to look up in the language file
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatus ( int id ) {
      if ( !statusLocked ) {
         setStatus ( Str.getStr ( id ), true ) ;
         statusChanged=true;
      }
   }





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the text that is showing
    *  in the bottom of this window. 
    *  
    *
    *  @param text The text to display
    *  @param animate TRUE = start animation, FALSE = do not start animation
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatus ( String text, boolean animate ) {
      if ( ( defaultMenuBar != null ) && ( text != null ) && ( animate ) )
         defaultMenuBar.startAnimation() ;
      else if ( ( defaultMenuBar != null ) && ( text == null ) && ( animate ) )
         defaultMenuBar.stopAnimation() ;

      if ( ( text != null ) && ( text.length() > 0 ) && ( animate ) )
         setCursor ( new Cursor ( Cursor.WAIT_CURSOR ) ) ;
      else
         setCursor ( new Cursor ( Cursor.DEFAULT_CURSOR ) ) ;

      statusChanged=true ;
      StatusBar.setStatus ( text, animate ) ;
   }


   /*------------------------------------------------------------*/
   /* addToolbarButton
    */
   /**
    *    This method will add a toolbar button
    *  to the toolbar on this frame
    *  
    *
    *  @param comp The component to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addToolbarButton ( Component comp, int index ) {
      if ( comp != null ) {
         if ( comp instanceof ToolBarButton )
            ((ToolBarButton)comp).setProperties() ;
         else if ( comp instanceof JToolBar ) {
            ((JToolBar)comp).setFloatable ( false ) ;
            GUISystem.setPropertiesOnPanel ( (Container)comp ) ;
         }
         else if ( comp instanceof Container )
            GUISystem.setPropertiesOnPanel ( (Container)comp ) ;
         toolBar.add ( comp, index ) ;
         toolBar.repaint() ;
      }
   }






   /*------------------------------------------------------------*/
   /* removeToolbarButton
    */
   /**
    *    This method will remove a toolbar
    *  button from the toolbar
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeToolbarButton ( Component comp ) {
      try {
         toolBar.remove ( comp ) ;
         toolBar.repaint() ;
      } catch ( Exception e ) {
      }
   }


   /*------------------------------------------------------------*/
   /* showToolbarButton
    */
   /**
    *    This method will hide or show a toolbar
    *  button from the toolbar
    *  
    *
    *  @param comp The component
    *  @param visible TRUE = show, FALSE = hide
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showToolbarButton ( Component comp, boolean visible ) {
      try {
         comp.setVisible ( visible ) ;
      } catch ( Exception e ) {
      }
   }


   /*------------------------------------------------------------*/
   /* setDefaultToolbar
    */
   /**
    *    This method will set up the default
    *  toolbar
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setDefaultToolbar () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the toolbar                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar = new AppDefaultToolBar() ;
      getContentPane().add ( toolBar, BorderLayout.NORTH ) ;      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add each button                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.add ( pb_REFRESH = new ToolBarButton ( Str.getStr ( STR_REFRESH ), 
                                                     ImageSystem.getImageIcon ( this, ImageSystem.REFRESH ), 
                                                     Str.getStr ( STR_REFRESH_HELP ) ) );
      toolBar.add ( pb_STOP = ProcessManagerSystem.createStopButton() ) ;
      toolBar.add ( new ToolBarSeparator() ) ;
      //toolBar.add ( pb_CHAT = new ToolBarButton ( Str.getStr ( STR_MESSAGING ), 
      //                                            ImageSystem.getImageIcon ( this, ImageSystem.CHAT ), 
      //                                            Str.getStr ( STR_MESSAGING_FLYOVER ) ) );
      toolBar.add ( new ToolBarSeparator() ) ;
      toolBar.add ( pb_BOOKMARK = new ToolBarButton ( Str.getStr ( STR_BOOKMARK ), 
                                                      ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_SMALL ), 
                                                      Str.getStr ( STR_BOOKMARK_HELP ) ) );


      toolBar.add ( new ToolBarSeparator() ) ;
      toolBar.add ( pb_PRINT = new ToolBarButton ( Str.getStr ( STR_PRINT ),
                                                   ImageSystem.getImageIcon ( this, ImageSystem.PRINT ),
                                                   Str.getStr ( STR_PRINT ) ) ) ;
      toolBar.add ( new ToolBarSeparator() ) ;
      toolBar.add ( pb_SEND_DEBUG    = new ToolBarButton ( Str.getStr ( STR_SEND_DEBUG ), 
                                                           ImageSystem.getImageIcon ( this, ImageSystem.LARGE_MAIL ),
                                                           Str.getStr ( STR_SEND_DEBUG ) ) );                    
      toolBar.add ( new ToolBarSeparator() ) ;
      toolBar.add ( pb_HELP    = new ToolBarButton ( Str.getStr ( STR_HELP ), 
                                                     ImageSystem.getImageIcon ( this, ImageSystem.SMALL_HELP ),
                                                     Str.getStr ( STR_HELP ) ) );                            


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.setOpaque ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners for each                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_REFRESH.addActionListener    ( this ) ;
      pb_BOOKMARK.addActionListener   ( this ) ;
      pb_HELP.addActionListener       ( this ) ;
      pb_PRINT.addActionListener      ( this ) ;
      pb_SEND_DEBUG.addActionListener ( this ) ;
      //pb_CHAT.addActionListener     ( this ) ;

   }





   /*------------------------------------------------------------*/
   /* initializeMenu
    */
   /**
    *    This method will initialize the default
    *  menu for the application
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setDefaultMenu () {
      setJMenuBar ( defaultMenuBar = new AppDefaultMenuBar() ) ;
      defaultMenuBar.addActionListener ( this ) ;
      GUISystem.setPropertiesOnPanel ( defaultMenuBar ) ;

   }






   /*------------------------------------------------------------*/
   /* actionListener
    */
   /**
    *    This method will be notified when
    *  tool bar buttons and menu items
    *  are fired on us
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() instanceof JMenuItem ) {
         lastMenuAction = ((JMenuItem)e.getSource()).getText() ;
         new Thread ( this, THREAD_MENUACTION ).start() ;
      } else if ( e.getSource() instanceof ToolBarButton )
         buttonActionPerformed ( (ToolBarButton) e.getSource() ) ;
   }





   /*------------------------------------------------------------*/
   /* buttonActionPerformed
    */
   /**
    *    This method will be called when a
    *  tool bar button has been pressed
    *  
    *
    *  @param button The ToolBarButton
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void buttonActionPerformed ( ToolBarButton button ) {
      if ( Qit.getEnabled() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_REFRESH                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( button == pb_REFRESH ) {
            if ( currentPanel != null )
               currentPanel.getQuestPanel().refresh() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_SEND_DEBUG                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_SEND_DEBUG )
            new DebugSender().send() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_HELP                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_HELP ) {
            if ( ( currentPanel != null ) && ( currentPanel.getQuestPanel() != null ) ) {
               QuestPanel panel = currentPanel.getQuestPanel() ;

               if ( panel instanceof HelpSource )
                  HelpSystem.showHelp ( ((HelpSource)panel).getHelpFilename() ) ;
               else
                  HelpSystem.showHelp ( "app"+File.separator+"index.html" ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_BOOKMARK                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_BOOKMARK ) {
            if ( ( currentPanel != null ) && ( currentPanel.getQuestPanel() != null ) ) {
               QuestPanel panel = currentPanel.getQuestPanel() ;

               if ( !panel.bookmark() )
                  GUISystem.printBox ( STR_INFO, STR_BOOKMARK_NOT_SUPPORTED ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_PRINT                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_PRINT )
           new PrintDialogue(currentPanel,this);

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CHAT                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( button == pb_CHAT ) {
            new ChatDlg ( this ) ;
         }

      }
   }





   /*------------------------------------------------------------*/
   /* menuActionPerformed
    */
   /**
    *    This method will be called when a
    *  menu item is selected
    *  
    *
    *  @param text The text of the menu item
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void menuActionPerformed ( String text ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Database System Dialog (Always allow it to open!)       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( text.equals ( "Database System" ) )
         new DatabaseDlg ( ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Log File                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( text.equals ( "Debug File" ) ) {
         LogSystem.showLogWindow ( true ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only perform the following actions if the app is        */
      /* enabled                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( Qit.getEnabled() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Exit                                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( text.equals ( Str.getStr ( STR_EXIT ) ) )
            new ShutdownThread (true) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Edit bookmarks                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_BOOKMARKS ) ) )
            new BookmarkDlg ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Application Properties                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_APPLICATION ) ) )
            new ApplicationPropertyDlg(this) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   User Properties                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_USER ) ) )
            new UserPropertyDlg ( this ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Paste Special                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_PASTE_SPECIAL ) ) )
            new ClipboardDlg ( this, null ).getResult() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Copy Special                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_COPY_SPECIAL ) ) )
            new ClipboardDlg ( this, null ).getResult() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Cut Special                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_CUT_SPECIAL ) ) )
            new ClipboardDlg ( this, null ).getResult() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Help Index                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_HELP_INDEX ) ) )
            HelpSystem.showHelp ( "app/index.html" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Help On Help                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_HELP_ON_HELP ) ) )
            HelpSystem.showHelp ( "app/helponhelp.html" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Tutorial                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( text.equals ( Str.getStr ( STR_TUTORIAL ) ) )
            new TutorWin ( false, 1 ) ;



      } /* if enabled  */
   }






   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when one
    *  of the properties we are listening for
    *  has been fired
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {

      switch ( property ) {
         case PropertySystem.PROPERTY_LANGUAGE :
            setText() ;
            break ;

         case PropertySystem.PROPERTY_SHOWSKIN :
            skinned = ((Boolean)value).booleanValue() ;

            if ( skinned )
               focusButtonPanel.setBackground ( ImageSystem.getImage ( this, ImageSystem.SAVE ) ) ;
            else {
               focusButtonPanel.setBackground ( getBackground() ) ;
               focusButtonPanel.setBackground ( (Image)null ) ;
            }

            focusButtonPanel.repaint() ;
            revalidate() ;
            break ;

         default:
            GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
            contentPanel.setAntiAlias ( PropertySystem.getBool ( PropertySystem.PROPERTY_ANTI_ALIAS ) ) ;
            focusPanel.setAntiAlias ( PropertySystem.getBool ( PropertySystem.PROPERTY_ANTI_ALIAS ) ) ;
            revalidate() ;
      }
   }




   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of
    *  all default radio buttons
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText ( ) {
      pb_REFRESH.setText  ( Str.getStr ( STR_REFRESH ) ) ;
      pb_BOOKMARK.setText ( Str.getStr ( STR_BOOKMARK ) ) ;
      pb_HELP.setText     ( Str.getStr ( STR_HELP ) ) ;
      //pb_PRINT.setText    ( Str.getStr ( STR_PRINT ) ) ;
   } /* setText */





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform a variety of
    *  operations in a thread. The operation is
    *  defined in the name of the  thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName() ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_MENUACTION                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( name.equals ( THREAD_MENUACTION ) )
         menuActionPerformed ( lastMenuAction ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Hold Status                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( name.equals ( THREAD_HOLD_STATUS ) ) {
         try {
            Thread.currentThread().sleep ( holdStatusSeconds*1000 ) ;
         } catch ( Exception e ) {
         }
         if ( !statusChanged )
            setStatus ( null ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* initializeContentPane
    */
   /**
    *    This method will initialize our panel that holds content
    *  if we are not in MDI mode, and the contentPanel
    *  is not already initialized
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private boolean initializeContentPanel () {
      boolean rc = false ;

      try {
         if ( ( !PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) &&
              ( contentPanel == null ) ) {
            contentPanel = new AntiAliasPanel() ;
            contentPanel.setLayout ( new CardLayout() ) ;
            focusPanel.add ( contentPanel, BorderLayout.CENTER ) ;
            focusButtonVec = new Vector ( 1, 1 ) ;
            applicationPanelVec = new Vector ( 1, 1 ) ;

            rc = true ;
         } else
            rc = (contentPanel != null) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setSinglePanel
    */
   /**
    *    This method is only called if we are 
    *  in MDI mode. This means that this window
    *  will only show 1 panel.
    *
    *  @param panel The ApplicationPanel to show
    *  @param title The title of the panel
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSinglePanel ( ApplicationPanel panel, String title ) {
      currentPanel = panel ;
      setTitle ( Str.getApplicationName()+" - "+title ) ;
      if ( panel.getQuestPanel().getIcon()  != null )
         setIconImage ( panel.getQuestPanel().getIcon().getImage() ) ;

      focusPanel.add ( panel, BorderLayout.CENTER ) ;
      revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* getStatus
    */
   /**
    *    This method will return the status that
    *  is showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getStatus () {
      return( StatusBar.getStatus() ) ;
   }





   /*------------------------------------------------------------*/
   /* getAppDefaultMenuBar
    */
   /**
    *    This method will return the AppDefaultMenuBar that
    *  is associated to this window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return AppDefaultMenuBar
    *//*---------------------------------------------------------*/
   public AppDefaultMenuBar getAppDefaultMenuBar () {
      return( defaultMenuBar ) ;
   }



   /*------------------------------------------------------------*/
   /* setPanelFocus
    */
   /**
    *    This method will set the focus panel (the one
    *  that is viewable) to whatever QuestPanel currently has the
    *  title passed. This method does nothing in MDI mode.
    *
    *  @param title The title of the QuestPanel to show, including the panel 
    *               index number appended to the end of the title
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPanelFocus ( String title, ApplicationPanel appPanel ) {
      if ( ( appPanel != null ) && ( title != null ) ) {

         try {
            if ( currentPanel != null ) {
               currentPanel.getQuestPanel().panelDeselected() ;
               defocusFocusButtons ( appPanel ) ;
            }

            if ( !PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) {
               ((CardLayout)contentPanel.getLayout()).show ( contentPanel, title ) ;                              

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Send the notification to the QuestPanel                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               appPanel.getQuestPanel().panelSelected() ; 

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Adjust our variables                                    */
               /*                                                           */
               /*-----------------------------------------------------------*/
               focusHistory.removeElement ( appPanel ) ;     // Takes this appPanel out of history
               focusHistory.addElement ( appPanel ) ; // Bumps this appPanel to the last selected panel
               currentPanel = appPanel ;
               refreshEditMenu() ;
               revalidate() ;

               focusButtonFromApplicationPanel ( appPanel ).setFocus ( true ) ;
            }
         } catch ( Exception e ) {
            e.printStackTrace() ;
            showWorkspacePanel() ;
         }
      } else {
         showWorkspacePanel() ;
      }
   }






   /*------------------------------------------------------------*/
   /* hidePanel
    */
   /**
    *    This method is currently not supported and
    *  will not work.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void hidePanel ( QuestPanel panel, QuestPanel newPanel ) {
      try {
         ApplicationPanel appPanel    = applicationPanelFromQuestPanel ( panel ) ;
         ApplicationPanel newAppPanel = applicationPanelFromQuestPanel ( newPanel ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are not in MDI mode, remove the panel             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) {
            boolean     found       = false ;
            int         i           = 0 ;
            int         size        = focusButtonVec.size() ;
            FocusButton focusButton = null ;

            while ( ( !found ) && ( i < size ) ) {
               focusButton = (FocusButton) focusButtonVec.elementAt(i) ;
               if ( ( focusButton != null ) && ( focusButton.contains ( appPanel ) ) ) {
                  contentPanel.remove ( appPanel ) ; 
                  appPanel.getQuestPanel().panelDeselected() ;
                  if ( newAppPanel != null )
                     setPanelFocus ( titleFromApplicationPanel ( newAppPanel ), newAppPanel ) ;
                  else
                     showWorkspacePanel() ;
                  focusButtonPanel.remove ( focusButton ) ;
                  found = true ;
               }
               i++ ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, close ourselves                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else
            setVisible ( false ) ;

      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* showPanel
    */
   /**
    *    This method will show a panel that was
    *  previously hidden by the 'hidePanel'
    *  method. Please use WindowSystem.createPanel to create new panels
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showPanel ( QuestPanel panel ) {
      try {
         ApplicationPanel appPanel    = applicationPanelFromQuestPanel ( panel ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are not in MDI mode, remove the panel             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) {
            boolean     found       = false ;
            int         i           = 0 ;
            int         size        = focusButtonVec.size() ;
            FocusButton focusButton = null ;

            while ( ( !found ) && ( i < size ) ) {
               focusButton = (FocusButton) focusButtonVec.elementAt(i) ;
               if ( ( focusButton != null ) && ( focusButton.contains ( appPanel ) ) ) {
                  if ( currentPanel != null )
                     contentPanel.remove ( currentPanel ) ;

                  if ( appPanel != null ) {
                     defocusFocusButtons ( appPanel ) ;
                     focusButton.setFocus ( true ) ; 
                     contentPanel.add ( appPanel, focusButton.getFocusApplicationPanelTitle() ) ;
                     ((CardLayout)contentPanel.getLayout()).show ( appPanel, focusButton.getFocusApplicationPanelTitle() ) ;            
                     //setPanelFocus ( titleFromApplicationPanel ( appPanel ), appPanel ) ;

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Notify the QuestPanel                                   */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     appPanel.getQuestPanel().panelSelected() ;
                     refreshEditMenu() ;
                     focusHistory.removeElement ( appPanel ) ;
                     focusHistory.addElement ( appPanel ) ; // Bump this appPanel to the first item in history
                  }

                  focusButtonPanel.add ( focusButton ) ;
                  found = true ;
               }
               i++ ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, close ourselves                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else
            setVisible ( true ) ;

      } catch ( Exception e ) {
      }
   }


   /*------------------------------------------------------------*/
   /* closePanel
    */
   /**
    *    This method will close the panel that
    *  is passed in
    *  
    *
    *  @param questPanel The panel to close
    *  @param newPanel The new panel to set focus to.
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean closePanel ( QuestPanel panel, QuestPanel newPanel ) {
      boolean rc = false ;

      if ( !(panel instanceof WorkspacePanel) ) {
         try {
            ApplicationPanel appPanel    = applicationPanelFromQuestPanel ( panel ) ;
            ApplicationPanel newAppPanel = applicationPanelFromQuestPanel ( newPanel ) ;

            if ( ( appPanel != null ) && ( appPanel.close() ) ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If we are not in MDI mode, remove the panel             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( !PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) {
                  boolean     found       = false ;
                  int         i           = 0 ;
                  int         size        = focusButtonVec.size() ;
                  FocusButton focusButton = null ;

                  while ( ( !found ) && ( i < size ) ) {
                     focusButton = (FocusButton) focusButtonVec.elementAt(i) ;
                     if ( ( focusButton != null ) && ( focusButton.contains ( appPanel ) ) ) {
                        if ( currentPanel != null )
                           currentPanel.finalize() ;
                        currentPanel = null ;                                         

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Remove this panel from history and from the screen      */
                        /* and from the Focus Button Vector and from the App Vector  */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        contentPanel.remove ( appPanel ) ; 
                        focusHistory.removeElement ( appPanel ) ;
                        focusButtonVec.removeElement ( focusButton ) ;
                        applicationPanelVec.removeElement ( appPanel ) ;                                

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Always set the focus back to the previously selected    */
                        /* panel                                                     */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        if ( focusHistory.size() > 0 )
                           newAppPanel = (ApplicationPanel) focusHistory.elementAt ( focusHistory.size()-1 ) ;
                        else
                           newAppPanel = null ;

                        if ( newAppPanel != null )
                           setPanelFocus ( titleFromApplicationPanel ( newAppPanel ), newAppPanel ) ;

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   If that failed, show the workspace, it's always there   */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        else
                           showWorkspacePanel() ;                              


                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Remove the Focus Button from the window                 */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        focusButtonPanel.remove ( focusButton ) ;
                        focusButton.free() ;
                        focusButton = null ;
                        found = true ;
                     }
                     i++ ;
                  }
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Otherwise, close ourselves                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else
                  dispose() ;

               rc = true ;
            }

         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         } catch ( Throwable e1 ) {
            e1.printStackTrace() ;
         }
      }

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* closePanel
    */
   /**
    *    This method will close the panel that
    *  is passed in
    *  
    *
    *  @param appPanel The Application Panel to close
    *  @param focusButton The FocusButton that is tied to the QuestPanel
    *  @param setFocus TRUE = Set Focus to next Panel, FALSE = do not set focus
    *
    *  @return TRUE if the panel was able to close
    *//*---------------------------------------------------------*/
   protected boolean closePanel ( ApplicationPanel appPanel, FocusButton focusButton, boolean setFocus ) {
      boolean rc = false ;

      if ( !(appPanel.getQuestPanel() instanceof WorkspacePanel) ) {
         if ( ( setFocus ) && ( previousPanel != null ) && ( previousPanel.getQuestPanel() != null ) ) {
            rc = closePanel ( appPanel.getQuestPanel(), previousPanel.getQuestPanel() ) ;
         } else
            rc = closePanel ( appPanel.getQuestPanel(), null ) ;
      }

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* titleFromApplicationPanel
    */
   /**
    *    This method will get the title that the contentPanel
    *  card layout manager is using for the
    *  QuestPanel passed in
    *
    *  @param questPanel The QuestPanel to look for
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   private String titleFromApplicationPanel ( ApplicationPanel panel ) {
      String retStr = null ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only do the work if we are not in MDI mode              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( !PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) {
            int             i           = 0 ;
            int             size        = focusButtonVec.size() ;
            FocusButton     focusButton = null ;

            while ( ( retStr == null ) && ( i < size ) ) {
               focusButton = (FocusButton) focusButtonVec.elementAt(i) ;
               if ( ( focusButton != null ) && ( focusButton.contains ( panel ) ) )
                  retStr = focusButton.getFocusApplicationPanelTitle() ;
               i++ ;
            }
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( retStr ) ;
   }



   /**
     *  This method will set the focus to 'FALSE' for all FocusButtons
     * except for the Focus Button representing the Application Panel
     * passed in
     **/
   private void defocusFocusButtons ( ApplicationPanel panel ) {
      if ( focusButtonVec != null ) {
         int         i         = 0 ;
         int         size      = focusButtonVec.size() ;

         while ( i < size ) {
            if ( !((FocusButton)focusButtonVec.elementAt(i)).contains ( panel ) )
               ((FocusButton)focusButtonVec.elementAt(i)).setFocus ( false ) ;
            i++ ;
         }
      }
   }


   /*------------------------------------------------------------*/
   /* focusButtonFromApplicationPanel
    */
   /**
    *    This method will return a focus button
    *  from the application panel passed in
    *  
    *
    *  @param appPanel The ApplicationPanel to look with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private FocusButton focusButtonFromApplicationPanel ( ApplicationPanel appPanel ) {
      FocusButton retButton = null ;                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have a focus button vector, traverse it           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( focusButtonVec != null ) {
         int         i         = 0 ;
         int         size      = focusButtonVec.size() ;

         while ( ( retButton == null ) && ( i < size ) ) {
            if ( ((FocusButton)focusButtonVec.elementAt(i)).contains ( appPanel ) )
               retButton = (FocusButton)focusButtonVec.elementAt(i) ;
            i++ ;
         }
      }


      return( retButton ) ;
   }





   /*------------------------------------------------------------*/
   /* dispose
    */
   /**
    *    This method will free all resources
    *  contained inside of this window.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void dispose() {
      boolean dispose = true ;


      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are in MDI mode, only close our main panel        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( PropertySystem.getBool ( PropertySystem.PROPERTY_MDI ) ) {
            if ( currentPanel != null ) {
               dispose = currentPanel.close() ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, close all of our panels                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {

            if ( focusButtonVec != null ) {
               int          i                    = 0 ;
               int          size                 = focusButtonVec.size() ;
               FocusButton  focusButton          = null ;
               FocusButton  workSpaceFocusButton = null ;

               while ( ( dispose ) && ( i < size ) ) {

                  focusButton = (FocusButton)focusButtonVec.elementAt(i) ;
                  if ( focusButton != null ) {

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   If this is the workspace button, don't close it now     */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( focusButton.getFocusApplicationPanelTitle().indexOf ( Str.getStr ( STR_WORKSPACE ) ) >= 0 )
                        workSpaceFocusButton = focusButton ;

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Otherwise, close the panel                              */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     else {
                        dispose = closePanel ( focusButton.getFocusApplicationPanel(), focusButton, false ) ;                                               

                        /*------------------------------------------------------------*/
                        /*                                                            */
                        /*   Reset our variables since the FocusButtonVec was altered */
                        /*                                                            */
                        /*------------------------------------------------------------*/
                        i    = -1 ;
                        size = focusButtonVec.size() ;
                     }
                  }
                  i++ ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, if everything else closed, close the Workspace     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( dispose ) && ( workSpaceFocusButton != null ) )
                  closePanel ( workSpaceFocusButton.getFocusApplicationPanel(), workSpaceFocusButton, false ) ;
            }
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now if we are still going to dispose ourselves,     */                                          
         /* continue.                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( dispose ) {


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Kill the menu bar                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( defaultMenuBar != null ) {
               setJMenuBar ( null ) ;
               defaultMenuBar.free() ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Kill all our property listeners                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            TypeCategory.removeTypeCategoryListener ( this ) ;
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_TOOLBAR_SHOW_ICON_TEXT, this ) ;
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_ANTI_ALIAS,             this ) ;
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_TOOLBAR_HELP,           this ) ;
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_TOOLBAR_TEXT_ORIENT,    this ) ;
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_BUTTONFONT,             this ) ;
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE,               this ) ;         
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SHOWSKIN,               this ) ;
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SKINNAME,               this ) ;
            TypeList.removeStatusListener         ( this ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Remove the toolbar and our STOP button                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( toolBar != null ) {
               ProcessManagerSystem.destroyStopButton ( pb_STOP ) ;
               getContentPane().remove ( toolBar ) ;
            }
         } /* if dispose */


      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign our 'isDisposed' property                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      isDisposed = dispose ;

      if ( dispose )
         super.dispose() ;

   }





   /*------------------------------------------------------------*/
   /* isDisposed
    */
   /**
    *    This method will return TRUE or FALSE
    *  as to if this window has completed the
    *  dispose process.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean isDisposed () {
      return( isDisposed ) ;
   }





   /*------------------------------------------------------------*/
   /* applicationPanelFromQuestPanel
    */
   /**
    *    This method will go through all of our open
    *  ApplicationPanels and tries to find the ApplicationPanel
    *  that holds the QuestPanel passed in
    *
    *  @param questPanel The QuestPanel to look for
    *  @param None
    *  @param None
    *
    *  @return ApplicationPanel NULL if not found
    *//*---------------------------------------------------------*/
   public ApplicationPanel applicationPanelFromQuestPanel ( QuestPanel questPanel ) {
      ApplicationPanel appPanel = null ;                                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have a vector of ApplicationPanels, traverse      */
      /*   through the list, otherwise return the current panel    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( applicationPanelVec != null ) {
         int              i        = 0 ;
         int              size     = applicationPanelVec.size() ;

         if ( questPanel != null ) {
            while ( ( appPanel == null ) && ( i < size ) ) {
               if ( ((ApplicationPanel)applicationPanelVec.elementAt(i)).containsQuestPanel ( questPanel ) ) {
                  appPanel = (ApplicationPanel)applicationPanelVec.elementAt(i) ;
               }
               i++ ;
            } /* while */
         }
      } else
         appPanel = currentPanel ;

      return( appPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* addActionComponent
    */
   /**
    *    This method will add an action component
    *  to the QuestPanel passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionComponent ( QuestPanel questPanel, JComponent component ) {
      ApplicationPanel appPanel = applicationPanelFromQuestPanel ( questPanel ) ;

      if ( appPanel != null )
         appPanel.addActionComponent ( component ) ;
   }



   /*------------------------------------------------------------*/
   /* removeActionComponent
    */
   /**
    *    This method will remove an action component
    *  from the QuestPanel passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeActionComponent ( QuestPanel questPanel, JComponent component ) {
      ApplicationPanel appPanel = applicationPanelFromQuestPanel ( questPanel ) ;

      if ( appPanel != null )
         appPanel.removeActionComponent ( component ) ;
   }





   /*------------------------------------------------------------*/
   /* setNavPanel
    */
   /**
    *    This method will change the navigation panel
    *  for a certain QuestPanel
    *  
    *
    *  @param questPanel The QuestPanel to hold the navigation panel
    *  @param navPanel The new Navigation Panel
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNavPanel ( QuestPanel questPanel, NavPanel navPanel ) {
      ApplicationPanel appPanel = applicationPanelFromQuestPanel ( questPanel ) ;

      if ( appPanel != null )
         appPanel.setNavPanel ( navPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* showWorkspacePanel
    */
   /**
    *    This method will only be called in a non-MDI
    *  interface mode. This will simply select the
    *  workspace panel button and show the workspace
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showWorkspacePanel () {
      if ( focusButtonVec != null ) {
         int          i                    = 0 ;
         int          size                 = focusButtonVec.size() ;
         FocusButton  focusButton          = null ;
         FocusButton  workSpaceFocusButton = null ;

         while ( i < size ) {

            focusButton = (FocusButton)focusButtonVec.elementAt(i) ;
            if ( focusButton != null ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If this is the workspace button, don't close it now     */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( focusButton.getFocusApplicationPanelTitle().indexOf ( Str.getStr ( STR_WORKSPACE ) ) >= 0 )
                  workSpaceFocusButton = focusButton ;
            } /* if  */
            i++ ;
         } /* while  */

         if ( focusButton != null ) {
            workSpaceFocusButton.setFocus ( true ) ;
            defocusFocusButtons ( workSpaceFocusButton.getFocusApplicationPanel() ) ;
            ((CardLayout)contentPanel.getLayout()).show ( contentPanel, workSpaceFocusButton.getFocusApplicationPanelTitle() ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* setFocusButtonText
    */
   /**
    *    This method will set the focus button text
    *  for the QuestPanel passed in.
    *  
    *
    *  @param questPanel The QuestPanel who's focus button needs to be altered
    *  @param text The text to set the focus button to
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocusButtonText ( QuestPanel questPanel, String text ) {
      ApplicationPanel appPanel = applicationPanelFromQuestPanel ( questPanel ) ;

      if ( appPanel != null ) {
         FocusButton focusButton = focusButtonFromApplicationPanel ( appPanel ) ;                                     

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we don't have a focus button, set our title          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( focusButton != null )
            focusButton.setText ( text ) ;
         else
            setTitle ( Str.getApplicationName()+" - "+text ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* typeListStatusChangeBegin
    */
   /**
    *    This method will be called when the TypeList
    *  object has something to show
    *  
    *
    *  @param status The new status
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void typeListStatusChangeBegin ( String status ) {
      setStatus ( status ) ;
   }




   /*------------------------------------------------------------*/
   /* typeListStatusChangeEnd
    */
   /**
    *    This method will be called when a previous
    *  status change from the TypeList object has
    *  expired
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void typeListStatusChangeEnd () {
      setStatus ( null ) ;
   }





   /*------------------------------------------------------------*/
   /* setStatusLock
    */
   /**
    *    This method will set the status lock to TRUE
    *  or FALSE. If TRUE, no other method will be
    *  able to set the status until it is unlocked
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatusLock ( boolean statusLocked ) {
      this.statusLocked = statusLocked ;
   }





   /*------------------------------------------------------------*/
   /* findQuestPanel
    */
   /**
    *    This method will traverse through all panels
    *  on this window and will call the equals() method
    *  on each with the compareRec passed
    *
    *  @param compareRec
    *  @param None
    *  @param None
    *
    *  @return QuestPanel NULL if not found
    *//*---------------------------------------------------------*/
   public QuestPanel findQuestPanel ( Object compareRec ) {
      QuestPanel returnPanel = null ;                               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we have multiple panels, ask each one                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( applicationPanelVec != null ) {
         int         i          = 0 ;
         int         size       = applicationPanelVec.size() ;
         QuestPanel  questPanel = null ;

         while ( i < size ) {
            questPanel = ((ApplicationPanel)applicationPanelVec.elementAt(i)).getQuestPanel() ;
            if ( ( questPanel != null ) && ( questPanel.equals ( compareRec ) ) )
               returnPanel = questPanel ;
            i++ ;
         }
      } else {
         if ( currentPanel.getQuestPanel().equals ( compareRec ) ) {
            returnPanel = currentPanel.getQuestPanel() ;
         }
      }

      return( returnPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* setPanelFocus
    */
   /**
    *    This method will set the focus of this window
    *  to the panel passed in
    *  
    *
    *  @param questPanel The QuestPanel to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPanelFocus ( QuestPanel questPanel ) {
      ApplicationPanel appPanel = applicationPanelFromQuestPanel ( questPanel ) ;

      if ( appPanel != null )
         setPanelFocus ( titleFromApplicationPanel ( appPanel ), appPanel ) ;
   }






   /*------------------------------------------------------------*/
   /* containsPanel
    */
   /**
    *    This method will determine if this window
    *  currently contains the QuestPanel passed in.
    *  
    *
    *  @param questPanel The QuestPanel to look for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean containsPanel ( QuestPanel questPanel ) {
      return( applicationPanelFromQuestPanel ( questPanel ) != null ) ;
   }




   /*------------------------------------------------------------*/
   /* setValue
    */
   /**
    *    This method will set what value the progress
    *  bar at the bottom of the window is set at
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setValue ( int value ) {
      StatusBar.setStatusValue ( value ) ;
   }




   /*------------------------------------------------------------*/
   /* setValueMax
    */
   /**
    *    This method will set the max value that will
    *  be shown to the user on the progress bar
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setValueMax ( int maxValue ) {
      StatusBar.setStatusMax ( maxValue ) ;
   }





   /*------------------------------------------------------------*/
   /* print
    */
   /**
    *    This method will print whatever panel is selected if that
    *  panel implements the PrintPanel interface
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void print () {

      if ( ( currentPanel != null ) && 
           ( currentPanel.getQuestPanel() != null  ) &&
           ( currentPanel.getQuestPanel() instanceof PrintPanel ) ) {
         new Print ( ((PrintPanel)currentPanel.getQuestPanel()).getPrintObject(),
                     this,
                     currentPanel.getQuestPanel().getName() ) ;
      } else
         GUISystem.printBox ( this, STR_INFO, STR_PRINTING_NOT_SUPPORTED ) ;
   }





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the status text in the
    *  window and hold it for 'x' seconds.
    *  
    *
    *  @param status The status
    *  @param x The amount of seconds
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatus ( String status, int x ) {
      holdStatusSeconds = x ;
      setStatus ( status, false ) ;
      statusChanged=false ;
      new Thread ( this, THREAD_HOLD_STATUS ).start() ;
   }


   /**
     *  This method will be called when the TypeCategory object reads
     * information. We will display a message stating this.
     *
     * @param name The Category Name
     **/
   public void typeCategoryReadStart ( String name ) {
      setStatus ( Str.getStr ( STR_READING )+" "+name ) ;
   }

   /**
     *  This method will be called when the TypeCategory object has
     * finished reading. We will take off our message
     *
     **/
   public void typeCategoryReadStop () {
      setStatus ( null ) ;
   }

   /**
     *  This method will be called when the TypeCategory object starts
     * saving something
     *
     **/
   public void typeCategoryWriteStart () {
      setStatus ( Str.getStr ( STR_SAVING ) ) ;
   }


   /**
     *  This method will be called when the TypeCategory object has
     * finished saving. We will take off our message
     *
     **/
   public void typeCategoryWriteStop () {
      setStatus ( null ) ;
   }


   /**
     *  This method will refresh the edit menu. If the current quest panel
     * being shown does not implement ClipboardPanel then all edit
     * cut/copy/paste menu items will be disabled
     **/
   private void refreshEditMenu () {
      boolean enable =  ( currentPanel.getQuestPanel() instanceof ClipboardPanel ) ;

      defaultMenuBar.enableCut ( enable ) ;
      defaultMenuBar.enableCopy ( enable ) ;
      defaultMenuBar.enablePaste ( enable ) ;


   }
} /* AppDefaultWin */
