package com.ibm.nzna.projects.qit.gui ;            
/*-----------------------------------------------------------*/
/*   ApplicationPanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ApplicationPanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing.* ;
import javax.swing.border.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;     
/**
 * The ApplicationPanel is responsible for holding a SplitPane,
 * ContentPane and NavBar. This panel also keeps track of  
 * multiple NavPanels so that if a new NavPanel is opened and
 * set, the current NavPanel will be shown upon closure of the
 * new NavPanel. The ApplicationPanel should really only be used
 * by the PanelSystem, although you can embed the ApplicationPanel
 * wherever you see fit.
 *
 *
 * @version 1.0, 1/12/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ApplicationPanel                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ApplicationPanel extends AntiAliasPanel implements PropertyListener, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final Color darkGray = new Color ( 128, 128, 128 ) ;
   private static int GID     = 1 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JSplitPane            splitPane               = null ;
   private NavPanel              current_NavComp         = null ;
   private NavHoldPanel          navHoldPanel            = null ;
   private QuestPanel            questPanel              = null ;
   private Vector                navPanelVec             = null ;
   private ApplicationTitlePanel titlePane               = null ;
   private boolean               skinned                 = false ;
   private int                   id                      = 0 ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create and initialize
    *  all variables for the ApplicationPanel. It
    *  will however not show the panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ApplicationPanel () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Global Identifier                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GID++ ;
      this.id = GID ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the default controls                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      titlePane = new ApplicationTitlePanel ( "" ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set attributes                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      skinned = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;
      setLayout ( null ) ; 
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_ANTI_ALIAS, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add default controls                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setAntiAlias ( PropertySystem.getBool ( PropertySystem.PROPERTY_ANTI_ALIAS ) ) ;
      add ( titlePane ) ;
   }



   /*------------------------------------------------------------*/
   /* getTitlePane
    */
   /**
    *    This method will return the AppDefaultTitlePane
    *  that this panel holds
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ApplicationTitlePanel
    *//*---------------------------------------------------------*/
   public ApplicationTitlePanel getTitlePanel () {
      return( titlePane ) ;
   }



   /*------------------------------------------------------------*/
   /* setPreferredNavPanelSize
    */
   /**
    *    This method will set the preferred size
    *  of the nav panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPreferredNavPanelSize () {
      int dividerLocation = splitPane.getDividerLocation() ;                            

      if ( dividerLocation == 0 )
         dividerLocation = 200 ;
      else if ( dividerLocation < 25 )
         dividerLocation = 25 ;

      //navHoldPanel.setMinimumSize ( new Dimension ( dividerLocation, 50 ) ) ;                       
      //navHoldPanel.setPreferredSize ( new Dimension ( dividerLocation, 100 ) ) ;                       
      //splitPane.setDividerLocation ( dividerLocation ) ;
   }


   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our controls.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      try {
         Dimension size = getSize() ;

         super.doLayout() ;

         titlePane.setBounds ( 0, 5, size.width-1, 25 ) ;

         if ( splitPane != null )
            splitPane.setBounds ( 0, 32, size.width-1, size.height-33 ) ;
         else
            questPanel.getJComponent().setBounds ( 0, 32, size.width-1, size.height-33 ) ;

      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* setNavPanelWidth
    */
   /**
    *    This method will set the width of the
    *  current nav panel
    *  
    *
    *  @param width The width, in pixels, of the nav panel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNavPanelWidth ( int pixels ) {
      if ( splitPane != null )
         splitPane.setDividerLocation ( pixels ) ;
   }




   /*------------------------------------------------------------*/
   /* setContentPanel
    */
   /**
    *    This method will set the content pane. The
    *  content pane is the pane on the right of
    *  the vertical split.
    *
    *  @param panel The Content Panel to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setContentPanel ( QuestPanel questPanel ) {
      if ( splitPane != null )
         splitPane.add ( questPanel.getJComponent(), splitPane.RIGHT ) ;
      else
         add ( questPanel.getJComponent() ) ;
      this.questPanel = questPanel ;
      titlePane.setTitle ( questPanel.getName()) ;
   }




   /*------------------------------------------------------------*/
   /* setNavPanel
    */
   /**
    *    This method will set the NavPanel
    *  for the panel. The NavPanel is
    *  the pane on the left side of the vertical split
    *
    *  @param panel The nav panel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNavPanel ( NavPanel panel ) {


      if ( panel != null ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up the JSplit Pane                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         splitPane = new JSplitPane ( JSplitPane.HORIZONTAL_SPLIT ) ;
         splitPane.setBorder ( null ) ;                                     
         if ( skinned )
            splitPane.setDividerSize ( 1 ) ;
         else
            splitPane.setDividerSize ( 3 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we currently have a quest panel, remove it           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( questPanel != null ) {
            remove ( questPanel.getJComponent() ) ;
            add ( splitPane ) ;
            splitPane.add ( questPanel.getJComponent(), splitPane.RIGHT ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add this panel to the NavPanel Vector                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( navPanelVec == null )
            navPanelVec = new Vector ( 1, 1 ) ;
         navPanelVec.addElement ( panel ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Make sure the NavHoldPanel is ready                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( current_NavComp != null ) {
            navHoldPanel.remove ( panel.getJComponent() ) ;
         } else
            initializeNavHoldPanel() ;                                 

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Tell the NavPanel it is time to initialize              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         panel.initialize() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the minimum size of the panel                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setPreferredNavPanelSize() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the current panel and add it                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         current_NavComp = panel ;
         navHoldPanel.addNavPanel ( panel ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* initializeNavHoldPanel
    */
   /**
    *    
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeNavHoldPanel () {
      NavPanelHeader navHeader = new NavPanelHeader () ;                                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the NavHoldPanel. It will hold everything        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navHoldPanel = new NavHoldPanel ( ) ;                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //navHoldPanel.setBorder ( GUISystem.etchedBorder ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our panels                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navHoldPanel.addNavPanelHeader ( navHeader ) ;
      navHeader.addActionListener ( this ) ;
      navHeader.setNavPanel ( navHoldPanel ) ;
      splitPane.add ( navHoldPanel, splitPane.LEFT ) ;
   }







   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called if a NavPanelHeader
    *  has something to notify us of
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Pop out the nav panel                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getActionCommand().equals ( "NAV_POPOUT" ) )
         popOutNavPanel() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close the Nav Panel                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getActionCommand().equals ( "NAV_CLOSE" ) )
         closeNavPanel() ;
   }







   /*------------------------------------------------------------*/
   /* close
    */
   /**
    *    This method will be called from the parent
    *  when the parent wishes to close
    *  this panel. This panel will close all open NavPanels and Content Panels
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean close () {
      AppDefaultWin parent = GUISystem.getParentDefWin ( this ) ;                           
      boolean       closed = false ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we can stop the QuestPanel, continue                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( questPanel != null ) {
         if ( questPanel.stop() ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this panel has NavPanels, close each one             */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( navPanelVec != null ) {
               int        i      = 0 ;
               int        size   = navPanelVec.size() ;
               NavPanel   panel  = null ;                           

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Loop through each nav panel, telling each one           */
               /* to close, and then closing the panel or the panel's parent*/
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {
                  panel = (NavPanel)navPanelVec.elementAt(i) ;
                  if ( panel != null ) {
                     panel.stop() ;
                     if ( splitPane != null )
                        splitPane.remove ( panel.getJComponent() ) ;
                  }
                  i++ ;
               }
            }

            if ( navHoldPanel != null )
               navHoldPanel.free() ;
            if ( splitPane != null )
               remove ( splitPane ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Remove ourselves as property listeners                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;
            PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_ANTI_ALIAS, this ) ;

            closed = true ;
         }
      } else
         closed = true ;

      return( closed ) ;
   }


   /*------------------------------------------------------------*/
   /* containsQuestPanel
    */
   /**
    *    This method will return TRUE if the
    *  quest panel passed is contained inside
    *  of this application panel
    *
    *  @param QuestPanel The quest panel to look for
    *  @param None
    *  @param None
    *
    *  @return TRUE = Yes, it contains, FALSE = No, does not contain
    *//*---------------------------------------------------------*/
   public boolean containsQuestPanel ( QuestPanel questPanel ) {
      return( this.questPanel == questPanel ) ;
   }





   /*------------------------------------------------------------*/
   /* containsNavPanel
    */
   /**
    *    This method will return TRUE if the
    *  NavPanel passed in is contained in
    *  this application panel. This includes nav panels popped out.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean containsNavPanel ( NavPanel navPanel ) {
      boolean rc = false ;
      if ( navPanelVec != null ) {
         int i    = 0 ;
         int size = navPanelVec.size() ;

         while ( ( !rc ) && ( i < size ) ) {
            rc = ( ((NavPanel)navPanelVec.elementAt(i)) == navPanel ) ;
            i++ ;
         }
      }
      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* popOutNavPanel
    */
   /**
    *    This method will pop the current NavPanel
    *  out of this window into a separate
    *  window
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void popOutNavPanel () {
      try {
         if ( current_NavComp != null ) {
            Dimension     size   = current_NavComp.getJComponent().getSize() ;
            AppDefaultWin newWin = new AppDefaultWin () ;
            Insets        insets = newWin.getInsets() ;

            navHoldPanel.remove ( current_NavComp.getJComponent() ) ;
            splitPane.remove ( navHoldPanel ) ;
            navHoldPanel.free() ;
            navHoldPanel = null ;

            newWin.getContentPane().setLayout ( new BorderLayout ( 0, 0 ) ) ;
            newWin.getContentPane().add ( current_NavComp.getJComponent() ) ;
            newWin.setSize ( size.width+insets.left+insets.right,
                             size.height+insets.top+insets.bottom ) ;
            WinUtil.centerChildInParent ( newWin, GUISystem.getParentDefWin ( this ) ) ;
            newWin.setVisible ( true ) ;
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* closeNavPanel
    */
   /**
    *    This method will close the current
    *  NavPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void closeNavPanel () {

      try {

         if ( current_NavComp != null ) {
            /* Give their stop method a call */
            current_NavComp.stop() ;

            /* Remove all of our components */
            navHoldPanel.remove ( current_NavComp.getJComponent() ) ;

            /* If this is not the default nav panel for the QuestPanel,
               put up the default nav panel now */
            if ( current_NavComp != questPanel.getDefaultNavPanel() ) {
               current_NavComp = questPanel.getDefaultNavPanel() ;
               navHoldPanel.addNavPanel ( current_NavComp ) ;
            } else {
               splitPane.remove ( navHoldPanel ) ;

               /* Assign to null */
               navHoldPanel.free() ;
               navHoldPanel    = null ;
               current_NavComp =  null ;
            }

         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getNavPanelWidth
    */
   /**
    *    This method will return the width
    *  of the nav panel in pixels
    *   
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return width
    *//*---------------------------------------------------------*/
   public int getNavPanelWidth () {
      return( splitPane.getDividerLocation() ) ;
   }


   /*------------------------------------------------------------*/
   /* getQuestPanel
    */
   /**
    *    This method will return the QuestPanel
    *  that this ApplicationPanel is displaying
    *  in the content area
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return QuestPanel
    *//*---------------------------------------------------------*/
   public QuestPanel getQuestPanel ( ) {
      return( questPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param g The graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      Dimension size = getSize() ;

      super.paintComponent ( g ) ;

      g.setColor ( darkGray ) ;
      g.drawLine ( 0, 0, size.width, 0 ) ;
      g.setColor ( Color.white ) ;
      g.drawLine ( 0, 1, size.width, 1 ) ;
   }

   /*------------------------------------------------------------*/
   /* addActionComponent
    */
   /**
    *    This method will add an 'action' component
    *  to the title area of this panel
    *  
    *
    *  @param JComponent The component to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionComponent ( JComponent component ) {
      titlePane.addComponent ( component ) ;
   }

   /*------------------------------------------------------------*/
   /* removeActionComponent
    */
   /**
    *    This method will remove an 'action' component
    *  rom the title area of this panel
    *  
    *
    *  @param JComponent The component to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void removeActionComponent ( JComponent component ) {
      titlePane.removeComponent ( component ) ;
   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when this panel
    *  is no longer showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      if ( questPanel != null )
         questPanel.panelDeselected() ;
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  that we are listening for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object newValue ) {
      if ( property == PropertySystem.PROPERTY_SHOWSKIN ) {
         skinned = ((Boolean)newValue).booleanValue() ;
         revalidate() ;
         if ( skinned )
            splitPane.setDividerSize ( 1 ) ;
         else
            splitPane.setDividerSize ( 3 ) ;
      }
      setAntiAlias ( PropertySystem.getBool ( PropertySystem.PROPERTY_ANTI_ALIAS ) ) ;

   }


   public void finalize () {
      try {
         WinUtil.finalizeComponents ( this ) ;
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }


   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method will determine equality only
    *  for GIDs. If the object passed to compare 
    *  is not an Integer object, the super will be called
    *
    *  @param object The object to compare with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean equals ( Object object ) {
      if ( object instanceof Integer )
         return( ((Integer)object).intValue() == id ) ;
      return( super.equals ( object ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getId
    */
   /**
    *    This method will return the unique id for
    *  this ApplicationPanel. This ID is unique 
    *  application wide
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getId () {
      return( id ) ;
   }
} /* ApplicationPanel */
















/*------------------------------------------------------------*/
/* NavHoldPanel
 */
/**
 *    This NavHoldPanel holds the NavPanelHeader
 *  and NavPanel
 *  
 *
 *  @param None
 *  @param None
 *  @param None
 *
 *  @return Nothing
 *//*---------------------------------------------------------*/
class NavHoldPanel extends AntiAliasPanel implements NavListExpandListener, AppConst, PropertyListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private NavPanelHeader navHeader        = null ;
   private NavPanel       navPanel         = null ;
   private int            width            = 0 ;
   private int            height           = 0 ;
   private boolean        showSkin         = false ;
   private Dimension      prefSize         = new Dimension ( 200, 150 ) ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavHoldPanel () {
      try {
         setLayout ( new BorderLayout ( 0, 0 ) ) ;

         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_ANTI_ALIAS, this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE,   this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SHOWSKIN,   this ) ;
         PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SKINNAME,   this ) ;
         showSkin = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;
         propertyChanged ( PropertySystem.PROPERTY_SKINNAME, null ) ;
         setAntiAlias ( PropertySystem.getBool ( PropertySystem.PROPERTY_ANTI_ALIAS ) ) ;

         if ( !showSkin ) {
            setBorder ( GUISystem.loweredBorder ) ;
            //setBackground ( Color.white ) ;
         }

      } catch ( Exception e ) {
      }
   }


   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }

   public Dimension getMinimumSize () {
      return( prefSize ) ;
   }

   /*------------------------------------------------------------*/
   /* addNavPanelHeader
    */
   /**
    *    This method will add the nav header
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void addNavPanelHeader ( NavPanelHeader navHeader ) {
      boolean add = true ;

      this.navHeader = navHeader ;
      if ( navPanel != null ) {
         navHeader.setTitle ( navPanel.getTitle() ) ;
      }
      add ( navHeader, BorderLayout.NORTH ) ;
   }




   /*------------------------------------------------------------*/
   /* addNavPanel
    */
   /**
    *    This method will add the NavPanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void addNavPanel ( NavPanel navPanel ) {
      this.navPanel = navPanel ;

      if ( ( navPanel != null ) && ( navPanel.getJComponent() != null ) ) {
         add ( navPanel.getJComponent(), BorderLayout.CENTER ) ;
         
         if ( navHeader != null ) {
            if ( ( navPanel.getTitle() != null ) && ( navPanel.getTitle().length() > 0 ) ) {
               navHeader.setTitle ( navPanel.getTitle() ) ;
               navHeader.setVisible ( true );
            } else
               navHeader.setVisible ( false ) ;
         }
      }
   }



   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has changed
    *  
    *
    *  @param property The property that changed
    *  @param value The value of the property
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {

      try {

         switch ( property ) {
            case PropertySystem.PROPERTY_LANGUAGE :
               if ( navPanel != null ) {
                  navHeader.setTitle ( navPanel.getTitle() ) ;
               }


            case PropertySystem.PROPERTY_SKINNAME :
            case PropertySystem.PROPERTY_SHOWSKIN :
               Color bg       = ThemeSystem.getColor ( "NAVPANEL_BG_COLOR" ) ;

               showSkin = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;

               //if ( bg != null )
               //   setBackground ( bg ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   IF we are to show the skin, adjust values               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( showSkin ) {
                  GUISystem.setComponentOpaque ( this, false ) ;
                  setBorder ( null ) ;
               } else {
                  setBorder ( GUISystem.loweredBorder ) ;
                  //navHeader.setOpaque ( true ) ;
               }

               invalidate() ;
               repaint() ;

               break ;

            default:
               setAntiAlias ( PropertySystem.getBool ( PropertySystem.PROPERTY_ANTI_ALIAS ) ) ;
               break ;
         }
      } catch ( Exception e ) {
      }

   }





   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will free all of the objects
    *  tied to our panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void free () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_ANTI_ALIAS, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;
   }



   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    Subclassed to set Opaque to FALSE if
    *  we are showing a skin
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Component add ( Component c ) {
      c = super.add ( c ) ;
      addComponent ( c ) ;
      return( c) ;
   }

   public Component add ( String s, Component c ) {
      c = super.add ( s, c ) ;
      addComponent ( c ) ;
      return( c ) ;
   }

   public void add ( Component c, Object o ) {
      super.add ( c,o ) ;
      addComponent ( c ) ;
   }

   public Component add ( Component c, int i ) {
      c = super.add ( c,i ) ;
      addComponent ( c ) ;
      return( c ) ;
   }

   private void addComponent ( Component c ) {
      //if ( ( c instanceof JComponent ) && ( c != navHeader ) )
      //   GUISystem.setComponentOpaque ( (JComponent) c, false ) ;

      if ( ( c instanceof NavListGroup ) || 
           ( c instanceof NavList ) ||
           ( c instanceof Container ) )
         addNavListExpandListeners ( (Container)c ) ;
   }


   private void addNavListExpandListeners ( Container component ) {

      if ( component instanceof NavListGroup )
         ((NavListGroup)component).addExpandListener ( this ) ;
      else {
         Component   c         = null ;
         Component[] compArray = ((Container)component).getComponents() ;

         if ( compArray != null ) {
            for ( int i = 0 ; i < compArray.length; i++ ) {
               if ( ( compArray[i] instanceof NavListGroup ) || ( compArray[i] instanceof Container ) )
                  addNavListExpandListeners ( (Container) compArray[i] ) ;
            }
         }
      }

   }




   /*------------------------------------------------------------*/
   /* expanded
    */
   /**
    *    This method will be called when a nav list
    *  expands. we will invalidate and repaint due to
    *  Swing Paint Problems
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void expanded ( NavListGroup source, boolean expanded ) {
      if ( !expanded ) {
         invalidate() ;
         repaint() ;
      }

   }
}
