package com.ibm.nzna.projects.qit.gui ;                          
/*-----------------------------------------------------------*/
/*   ApplicationTitlePanel                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ApplicationTitlePanel               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import javax.swing.* ;
import java.awt.* ;
/**
 * This is the panel that sits on top of all the QuestPanel/NavPanels. It holds 
 * the title of the QuestPanel and also holds the action buttons for the panel
 *
 * @version 1.0, 3/15/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ApplicationTitlePanel                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class ApplicationTitlePanel extends JPanel implements AppConst, PropertyListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Variables                                        */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final Dimension prefSize = new Dimension ( 100, 27 ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Components                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_TITLE    = null ;
   private ButtonPanel buttonPanel = null ;
   private boolean     skinned     = false ;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor that initializes this component
    *  with a title
    *  
    *  
    *
    *  @param title The initial title to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ApplicationTitlePanel ( String title ) {
      initialize() ;
      setTitle ( title ) ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ApplicationTitlePanel ( ) {
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will create our GUI and initialize
    *  all variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE    = new JLabel ( "" ) ;
      buttonPanel = new ButtonPanel () ;            


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_PANELTITLEFONT, this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SHOWSKIN,       this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SKINNAME,       this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      skinned = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;
      st_TITLE.setOpaque ( false ) ;
      st_TITLE.setFont ( FontSystem.panelTitleFont ) ;
      buttonPanel.setOpaque ( false ) ;  
      setBackground ( ThemeSystem.getColor ( "TITLEPANEL_BG_COLOR" ) ) ;
      st_TITLE.setForeground ( ThemeSystem.getColor ( "TITLEPANEL_FG_COLOR" ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_TITLE ) ;
      add ( buttonPanel ) ;

   } /* initialize  */





   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return our minimum size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our preferredSize
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( prefSize ) ;
   }


   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize () ;

      super.doLayout () ;

      st_TITLE.setBounds ( 5, 0, 170, size.height ) ;
      buttonPanel.setBounds ( 195, 0, size.width-175, size.height ) ;
   }




   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will change the title of
    *  this TitlePanel
    *  
    *
    *  @param title The new title
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitle ( String title ) {
      st_TITLE.setText ( title ) ;
   }




   /*------------------------------------------------------------*/
   /* addComponent
    */
   /**
    *    This method will add a component to the
    *  button panel that is shown.
    *  
    *
    *  @param JComponent The component to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addComponent ( JComponent component ) {
      buttonPanel.add ( component ) ;
   }

   /*------------------------------------------------------------*/
   /* removeComponent
    */
   /**
    *    This method will remove a component from the
    *  button panel that is shown.
    *  
    *
    *  @param JComponent The component to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeComponent ( JComponent component ) {
      buttonPanel.remove ( component ) ;
   }






   /*------------------------------------------------------------*/
   /* finalize
    */
   /**
    *    This method will be called when we are disposed
    *  and no longer are needed. We will free
    *  resources here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finalize () {
      try {
         PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_PANELTITLEFONT, this ) ;
         PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;
         PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }

   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has been changed.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {

      switch ( property ) {
         
         case PropertySystem.PROPERTY_PANELTITLEFONT :
            st_TITLE.setFont ( (Font) value ) ;
            revalidate() ;
            break ;

         case PropertySystem.PROPERTY_SHOWSKIN :
            skinned = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;
            revalidate() ;
            break ;

         case PropertySystem.PROPERTY_SKINNAME :
            Color bg = ThemeSystem.getColor ( "TITLEPANEL_BG_COLOR" ) ;
            Color fg = ThemeSystem.getColor ( "TITLEPANEL_FG_COLOR" ) ;

            if ( bg == null )
               setOpaque ( false ) ;
            else {
               setOpaque ( true ) ;
               setBackground ( bg ) ;
            }
            
            if ( fg == null )
               st_TITLE.setForeground ( Color.white ) ;
            else
               st_TITLE.setForeground ( fg ) ;
            break ;

      }

   }





   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      //if ( skinned )
      //   WinUtil.wallPaper ( this, g, ImageSystem.getImage ( this, ImageSystem.SAVE ) ) ;
      //else {
         Dimension size = getSize() ;

         g.setColor ( getBackground() ) ;
         g.fillRect ( 0, 0, size.width, size.height ) ;
      //}

      paintChildren ( g ) ;
   }

} /* ApplicationTitlePanel */
