package com.ibm.nzna.projects.qit.gui ;    
/*-----------------------------------------------------------*/
/*   ClipboardDlg                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ClipboardDlg                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
/**
 * This dialog shows a user all objects in a clipboard. 
 *
 *
 * @version 1.0, 3/26/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ClipboardDlg                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class ClipboardDlg extends JDialog implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Members                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_CLIPBOARD            = null ;
   private JComboBox    cb_CLIPBOARDS           = null ;
   private MultiList    lb_DATA                 = null ;
   private DButton      pb_CANCEL               = null ;
   private DButton      pb_HELP                 = null ;
   private DButton      pb_PASTE                = null ;
   private ButtonPanel  buttonPanel             = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Members                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Object       returnObject            = null ;
   private String       clipboardName           = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create but not show the
    *  dialog. To show the dialog, you must call
    *  the method 'getResult'
    *
    *  @param parent The parent window of this dialog
    *  @param clipboardName The name of the clipboard to show by default.
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ClipboardDlg ( Frame parent, String clipboardName ) {
      super ( parent, Str.getStr ( STR_CLIPBOARDS ), true ) ;
      this.clipboardName = clipboardName ;
      setSize ( 400, 300 ) ;
      createControls() ;
      WinUtil.centerChildInParent ( this, parent ) ;
   }




   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the window and return a couple of different
    *  objects depending on what action the user
    *  takes. The method will note return until the window is dismissed.<br><br>
    *  <b>The following are return values:</b><br>
    *
    * <pre><br>
    * Object            Description<br>
    * -----------------------------------------------------------<br>
    * NULL              User pressed 'Cancel'<br>
    * Vector            User selected at least 2 items and pressed 'Paste'
    * Object            User selected 1 item, and pressed 'Paste'
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return see above
    *//*---------------------------------------------------------*/
   public Object getResult () {
      setVisible ( true ) ;
      return( returnObject ) ;
   }




   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createControls () {
      Container contentPane = getContentPane() ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_CLIPBOARD  = new JLabel ( Str.getStr ( STR_CLIPBOARDS ) ) ;
      cb_CLIPBOARDS = new JComboBox ( ClipSystem.getAllClipboardNames() ) ;
      pb_PASTE      = new DButton   ( Str.getStr ( STR_PASTE ) ) ;
      pb_CANCEL     = new DButton   ( Str.getStr ( STR_CANCEL ) ) ;
      pb_HELP       = new DButton   ( Str.getStr ( STR_HELP ) ) ;
      lb_DATA       = new MultiList ( GUISystem.getFontUtil() ) ;        
      buttonPanel   = new ButtonPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_CLIPBOARD.setFont  ( FontSystem.textFont ) ;
      cb_CLIPBOARDS.setFont ( FontSystem.listFont ) ;
      pb_PASTE.setFont      ( FontSystem.buttonFont ) ;
      pb_CANCEL.setFont     ( FontSystem.buttonFont ) ;
      pb_HELP.setFont       ( FontSystem.buttonFont ) ;
      lb_DATA.setFont       ( FontSystem.listFont ) ;    
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      GUISystem.setPreferredButton ( pb_HELP ) ;
      GUISystem.setPreferredButton ( pb_PASTE ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_PASTE.addActionListener  ( this ) ;
      pb_HELP.addActionListener   ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      lb_DATA.addActionListener   ( this ) ;  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_CLIPBOARD ) ;
      contentPane.add ( cb_CLIPBOARDS ) ;
      contentPane.add ( lb_DATA ) ;
      contentPane.add ( buttonPanel ) ;
      buttonPanel.add ( pb_PASTE ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      buttonPanel.add ( pb_HELP ) ;

   } /* createControls  */






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired. It will handle that action event
    *  
    *
    *  @param e The action event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         returnObject = null ;
         dispose() ;
      }
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will position and size all of
    *  our GUI Components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize () ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout () ;

      st_CLIPBOARD.setBounds ( 0, 0, 150, rowHeight ) ;
      cb_CLIPBOARDS.setBounds ( 150, 0, size.width-155, rowHeight ) ;

      lb_DATA.setBounds ( 0, rowHeight+5, size.width-10, size.height-80 ) ;

      buttonPanel.setBounds ( 0, size.height-55, size.width-10, 35 ) ;
   }

} /* ClipboardDlg */
