package com.ibm.nzna.projects.qit.gui ;           
/*-----------------------------------------------------------*/
/*   CountrySelectDlg                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CountrySelectDlg                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This JDialog will allow the user to select countries
 * from a JTree tree.
 *
 * @version 1.0, 5/21/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CountrySelectDlg                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class CountrySelectDlg extends JDialog implements AppConst, ActionListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Controls                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JScrollPane scr_COUNTRIES      = null ;
   private JTree       countryTree        = null ;
   private DButton     pb_OK              = null ;
   private DButton     pb_CANCEL          = null ;
   private JLabel      st_TITLE           = null ;      
   private ButtonPanel buttonPanel        = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector returnVec      = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the dialog but will
    *  not show it. To show the dialog you must call
    *  the 'getResult' method
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CountrySelectDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_SELECT_A_COUNTRY ), true ) ;

      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the window and will
    *  wait for the user to respond
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector A Vector of all selected countries
    *//*---------------------------------------------------------*/
   public Vector getResult() {
      setVisible ( true ) ;
      return( returnVec ) ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize this dialog's
    *  GUI and variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {     
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE      = new JLabel ( Str.getStr ( STR_SELECT_A_COUNTRY ) ) ;
      countryTree   = new JTree ( ) ;
      scr_COUNTRIES = new JScrollPane ( countryTree ) ;
      pb_OK         = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL     = new DButton ( Str.getStr ( STR_CANCEL ) ) ;        
      buttonPanel   = new ButtonPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      UIManager.put ( "Tree.expandedIcon",  ImageSystem.getImageIcon ( this, ImageSystem.OPEN_TREE ) ) ;
      UIManager.put ( "Tree.collapsedIcon", ImageSystem.getImageIcon ( this, ImageSystem.CLOSE_TREE ) );       
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;
      scr_COUNTRIES.setBorder ( GUISystem.loweredBorder ) ;    


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_TITLE ) ;
      contentPane.add ( scr_COUNTRIES ) ;
      contentPane.add ( buttonPanel ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_CANCEL ) ;                                          
      GUISystem.setPropertiesOnPanel ( contentPane ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the list of countries                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshList() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust window size and position                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 300, 300 ) ;
      WinUtil.centerChildInParent ( this , WinUtil.getParentFrame ( this ) ) ;



   }







   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize () ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = 5 ;
      int       y         = 0 ;

      super.doLayout() ;
      st_TITLE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+2 ;

      scr_COUNTRIES.setBounds ( x, y, size.width-(x*3), size.height-80 ) ;

      buttonPanel.setBounds ( x, size.height-55, size.width, 25 ) ;

   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method responds to action events
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         returnVec = null ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_OK ) {
         saveData() ;
         dispose() ;
      }


   }





   /*------------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method will refresh our list of countries
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshList () {
      Vector              geoVec      = TypeList.getInstance().getTypeList ( TypeList.TYPE_GEOGRAPHY ) ;
      Vector              countryVec  = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;
      StringTreeNode      root        = new StringTreeNode ( Str.getStr ( STR_SUPPORTED_COUNTRIES ) ) ;
      DefaultTreeModel    model       = new DefaultTreeModel ( root ) ;
      StringTreeNode      currentNode = null ;
      StringTreeRenderer  treeRenderer  = null ;

      countryTree.setCellRenderer ( treeRenderer = new StringTreeRenderer() ) ;
      //countryTree.setRootVisible ( false ) ;
      treeRenderer.setPreferredWidth ( 200 ) ;

      if ( geoVec != null ) {
         int                 geoIndex     = 0 ;
         int                 geoSize      = geoVec.size() ;
         int                 countryIndex = 0 ;
         int                 countrySize  = countryVec.size() ;
         TypeCountryCodeRec  countryRec   = null ;
         TypeGeoRec          geoRec       = null ;

         while ( geoIndex < geoSize ) {

            geoRec = (TypeGeoRec ) geoVec.elementAt ( geoIndex ) ;
            currentNode = new StringTreeNode ( geoRec.toString(),
                                               geoRec ) ;
            root.add ( currentNode ) ;

            countryIndex = 0 ;
            if ( countryVec != null ) {
               while ( countryIndex < countrySize ) {

                  countryRec = (TypeCountryCodeRec) countryVec.elementAt ( countryIndex ) ;

                  if ( countryRec.geoInd == geoRec.getInd() )
                     currentNode.add ( new StringTreeNode ( countryRec.toString(), countryRec ) ) ;

                  countryIndex++ ;
               } /* while countryIndex */
            } /* if countryVec != null */

            geoIndex++ ;
         } /* while geoIndex */
      } /* if geoVec != null */

      countryTree.setModel ( model ) ;
      countryTree.expandRow ( 0 ) ;
   } /* refreshList */





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save our data 
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void saveData () {
      TreePath[] treePath = countryTree.getSelectionPaths() ;               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our return Vector                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      returnVec = new Vector ( 1, 10 ) ;                                         


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, loop through the selection and grab each item      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( treePath != null ) && ( treePath.length > 0 ) ) {
         for ( int i = 0; i < treePath.length; i++ ) {
            StringTreeNode treeNode = (StringTreeNode) treePath[i].getLastPathComponent() ;
            Object         object   = treeNode.getData() ;


            /*-------------------------------------------------------------*/
            /*                                                             */
            /*   If a geography is selected, add all countries in that geo */
            /*                                                             */
            /*-------------------------------------------------------------*/
            if ( object instanceof TypeGeoRec ) {
               TypeGeoRec geoRec = (TypeGeoRec) object ;

               /*--------------------------------------------------------------*/
               /*                                                              */
               /*   If worldwide, just return, we now have all countries       */
               /*                                                              */
               /*--------------------------------------------------------------*/
               if ( geoRec.getInd() == TypeList.GEO_WORLD_WIDE ) {
                  i = treePath.length ;
                  returnVec.removeAllElements() ;
               }

               copyCountryVec ( returnVec, geoRec.getInd() ) ;

            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Otherwise, just add the single country                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( object instanceof TypeCountryCodeRec )
               returnVec.addElement ( object ) ;

         } /* for */
      } /* if treePath */
   } /* saveData */




   /*------------------------------------------------------------*/
   /* copyCountryVec
    */
   /**
    *    This method will copy all countries under
    *  a specified geography to the vector passed in
    *  
    *
    *  @param countryVec The Country Vector to add TypeCountryCodeRecs to
    *  @param geoInd The Geography indicator
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void copyCountryVec ( Vector countryVec, int geoInd ) {
      Vector typeCountryVec = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;

      if ( ( typeCountryVec != null ) && ( typeCountryVec.size() > 0 ) ) {
         int                i          = 0 ;
         int                size       = typeCountryVec.size() ;
         TypeCountryCodeRec countryRec = null ;

         while ( i < size ) {
            countryRec = (TypeCountryCodeRec) typeCountryVec.elementAt(i) ;

            if ( ( countryRec.geoInd == geoInd ) || ( geoInd == TypeList.GEO_WORLD_WIDE ) )
               countryVec.addElement ( countryRec ) ;

            i++ ;
         } /* while i */
      } /* if typeCountryVec */
   } /* copyCountryVec */

} /* CountrySelectDlg */

