package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   CustomerEntryPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomerEntryPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2001                                  */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.qit.product.* ;

/**
 * This panel holds 2 controls. The EntryField and Select Customer Button. 
 * It can be used to allow the user to enter CUSTOMER description. It will auto-populate
 * the entry field if the user stops typing for 1 second.             
 *
 * @version 1.0, 08/3/2001 
 * @author  Venu Sanka
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomerEntryPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomerEntryPanel extends JPanel implements ActionListener, Runnable, AppConst, ProductConst,
     DataLengths {

    /**
     *  This is the name of the thread that will search for customers
     **/
   private static final String THREAD_SEARCH = "SE" ;

   /**
     *  This is the name of the thread that will fire events
     **/
   private static final String THREAD_FIRE_EVENT = "FE" ;

   
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   
   private JTextField ef_CUST          = new JTextField() ;
   private DButton    pb_SELECTCUST    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This vector holds all registered listeners
     **/
   private Vector listeners = new Vector ( 1 ) ;


   /**
     *  This is the property which holds the last customer searched for.
     **/
   private String  lastCustomerSearch = "" ;

   /**
     *  If TRUE, the control is currently searching for UserRec objects
     **/
   private boolean inSearch       = false ;

   /**
     *  This is the vector of customers that can be selected
     **/
   private Vector  customerVec        = null ;

   /**
     *  This is the currently selected customer record
     **/
   private TypeCustomerRec  customerRec        = null ;

   /**
     *  If TRUE, this property tells the control to show the user's full name, not just the user id
     **/
   private boolean showFullName   = true ;

   /**
     *  This property tells the running thread to continue
     **/
   private boolean continueThread = true ;

   /**
     *  This is a temporary property to hold the current user
     **/
   private String           curTempCustomer     = null ;

   /**
     *  This property tells the control to show managers only
     **/
   private boolean showManagerOnly    = false ;


      
   
   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the Customer Panel with
    *  the default description of 'Customer'
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public CustomerEntryPanel () {
      init() ;
   }


   /*-----------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void init () {
      createControls () ;
      customerVec = (Vector)TypeList.getInstance().getTypeList( TypeList.TYPE_CUSTOMER).clone() ;
      new Thread(this, THREAD_SEARCH ).start() ;

   }

   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our controls
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void createControls () {

      setLayout ( null ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECTCUST = new DButton ( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_PERSONEL ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      
      //setBorder ( GUISystem.loweredBorder ) ;
      pb_SELECTCUST.setFocusPainted ( false ) ;
      //pb_SELECTCUST.setFocusable ( false ) ;
      pb_SELECTCUST.setBorderPainted ( false ) ;
      pb_SELECTCUST.setBackground ( Color.white ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECTCUST.addActionListener ( this ) ;
      ef_CUST.addActionListener( this ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      
      setLayout ( new BorderLayout () ) ;
      add ( ef_CUST, BorderLayout.CENTER ) ;
      add ( pb_SELECTCUST, BorderLayout.EAST ) ;

   }


   /*-----------------------------------------------------------*/
   /* getText
    */
   /**
    *    This method will simply return whatever is
    *  in the Customer Entry Field. It is not guarenteed
    *  what is in the field
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public String getText() {
      return( ef_CUST.getText() ) ;
   }


   /*-----------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of the entry field
    *  
    *  
    *             
    *  @param text The text to set
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setText ( String text ) {
      if ( ( text != null ) && ( text.length() > 0 ) ) {
         curTempCustomer = text ;
         customerRec = findCustomer ( text, true ) ;
         refreshCustomerField() ;
      } else
         ef_CUST.setText ( "" ) ;

   }


   /*-----------------------------------------------------------*/
   /* setShowFullName
    */
   /**
    *    This method will tell the panel to show the user's full name,
    *  If showFullName is TRUE, the panel will
    *  display the customer's full name.
    *  
    *             
    *  @param onTop TRUE = Show FullName FALSE = Only show user id
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setShowFullName ( boolean showFullName ) {
      this.showFullName = showFullName ;

      refreshCustomerField() ;

   }


   /*-----------------------------------------------------------*/
   /* getShowFullName
    */
   /**
    *    This method will the ShowFullName property     
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public boolean getShowFullName () {
      return( showFullName ) ;
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform our thread operations
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
       String operation = Thread.currentThread().getName() ;

       if ( operation.equals (THREAD_SEARCH ))
           search() ;
       else if ( operation.equals ( THREAD_FIRE_EVENT ))
           fireEvent() ;
   }


   /**
    *  This method will search for customers
    *
    */
   private void search () {
   
      String curCustomer  = null ;
      String curCustomer1 = null ;

      while ( continueThread ) {

         /* Ok, first sleep for 1 1/2 second and then check the
            entry field's text */
         try {
            Thread.sleep ( 1500 ) ;
         } catch ( Exception e ) {
         }
         curCustomer = ef_CUST.getText() ;

         /* Ok, if both curCustomer and lastCustomerSearch are not null
            and do not equal each other, perform a search */
         if ( ( curCustomer != null ) &&
              ( curCustomer.length() > 0 ) && 
              ( lastCustomerSearch != null ) ) {

            /* Ok, now wait for a bit longer and check it again. If the
               text has changed, loop back up. Otherwise look for the
               name */
            try {
               Thread.sleep ( 300 ) ;
            } catch ( Exception e ) {
            }
            curCustomer1 = ef_CUST.getText() ;


            if ( curCustomer1.equals ( curCustomer ) ) {

               if ( !curCustomer1.equals ( lastCustomerSearch ) ) {
                  lastCustomerSearch = curCustomer1 ;

                  customerRec = findCustomer ( curCustomer1 ) ;

                  /* Ok, if we found a record, populate it now */
                  refreshCustomerField() ;

                  if ( customerRec != null )
                      new Thread ( this, THREAD_FIRE_EVENT ).start() ;
               } /* if */

            } /* if */
         } else
            customerRec = null ;
      } /* while */
   } /* search */




   /*------------------------------------------------------------*/
   /* findCustomer
    */
   /**
    *    This method will attempt to locate a user
    *  doing a fuzzy search
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private TypeCustomerRec findCustomer ( String customer ) {
      return( findCustomer ( customer, false ) ) ;
   }




   /*------------------------------------------------------------*/
   /* findCustomer
    */
   /**
    *    This method will attempt to locate the customer
    *  in the application customer list. If a match is found
    *  then this method will return the customer record.
    *
    *  @param customer  customer  description
    *  @param exactMatch TRUE = only hit on exact matches
    *  @param None
    *
    *  @return TypeCustomerRec of the customer found, NULL if not found
    *//*---------------------------------------------------------*/
   private TypeCustomerRec findCustomer ( String customer, boolean exactMatch ) {
      TypeCustomerRec retRec = null ;

      if ( ( customer != null ) && ( customerVec != null ) ) 
          {
            int     i              = 0 ;
            int     length         = customerVec.size() ;
            TypeCustomerRec    compareCustomerRec = null ;
            Vector  foundVec       = new Vector ( 1, 10 ) ;

            /* Always upper case! */
            customer = customer.toUpperCase() ;                    

            /* Ok, walk through the vector and look for the 'customer' in the description field */
            while ( i < length ) 
                {
                    compareCustomerRec = (TypeCustomerRec) customerVec.elementAt(i) ;

                if ( compareCustomerRec != null ) 
                    {

                        if ( exactMatch ) 
                            {
                                if ( compareCustomerRec.toString().toUpperCase().equals ( customer ) )
                                foundVec.addElement ( compareCustomerRec ) ;

                  //else if ( compareUserRec.getName().toUpperCase().equals ( user ) )
                  //   foundVec.addElement ( compareUserRec ) ;
                            } 
                        else {
                                if ( compareCustomerRec.toString().toUpperCase().startsWith ( customer ) )
                                foundVec.addElement ( compareCustomerRec ) ;

                  //else if ( compareUserRec.getName().toUpperCase().startsWith ( user ) ) {
                  //   foundVec.addElement ( compareUserRec ) ;
                            //}
               }
            } /* if */

            i++ ;
         } /* while */

         foundVec.trimToSize() ;

         /* Ok, if we have more than 1 hit, let the user select, otherwise auto-populate the field */
         if ( foundVec.size() > 1 ) 
             {
                Vector vec = null ;


                vec = new SelectCustomerDlg ( GUISystem.getParentFrame(this), foundVec, false ).getResult() ;

                if ( ( vec != null ) && ( vec.size() > 0 ) )
                    compareCustomerRec = (TypeCustomerRec) vec.elementAt(0) ;

                    retRec = this.customerRec = compareCustomerRec ;

                    refreshCustomerField() ;

             } 
         else if ( foundVec.size() == 1 )
            retRec = (TypeCustomerRec) foundVec.elementAt(0) ;


      } /* if */

      return( retRec ) ;
   } /* findUser */













   /*------------------------------------------------------------*/
   /* refreshCustomerField
    */
   /**
    *    This method will refresh the data that is
    *  in the customer field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshCustomerField() {

      if ( customerRec != null ) {

         if ( showFullName ) 
            ef_CUST.setText ( customerRec.toString() ) ;
         //else
         //   ef_USERID.setText ( userRec.getUserId() ) ;

         try {
            ef_CUST.setSelectionStart ( lastCustomerSearch.length() ) ;
            ef_CUST.setSelectionEnd ( ef_CUST.getText().length() ) ;
         } catch ( Exception e ) {}

         lastCustomerSearch = ef_CUST.getText() ;

         } /* if */

      else
         ef_CUST.setText ( "" ) ;
      
   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one of our
    *  buttons is pressed
    *  
    *
    *  @param e The actionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {     

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   DButton Events                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() instanceof DButton ) {
            DButton button = (DButton) e.getSource() ;
            if ( pb_SELECTCUST == button ) {
               Vector vec = null ;

               vec = new SelectCustomerDlg ( GUISystem.getParentFrame(this), false ).getResult() ;
               if ( ( vec != null ) && ( vec.size() > 0 ) )
                  customerRec = (TypeCustomerRec) vec.elementAt(0) ;
               refreshCustomerField() ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   TextField Events                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() instanceof JTextField ) {
            String text = ef_CUST.getText() ;

            customerRec = findCustomer ( text ) ;
            refreshCustomerField() ;
         }
      } catch ( Exception ex ) {
         LogSystem.log ( 1, ex) ;
      }
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will stop our thread that is monitoring
    *  the entry field. It must be called when you are through with
    *  this control.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      continueThread = false ;
   }



   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name if it is valid.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return user The name if valid, NULL if not valid
    *//*---------------------------------------------------------*/
   public String getName () {
      String retStr = null ;

      if ( customerRec != null )
         retStr = customerRec.toString() ;

      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* getCustomerRec
    */
   /**
    *    This method will return the customer record
    *  if it is valid, If not valid, a lookup will be performed
    *  asking the user to select a description
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public TypeCustomerRec getCustomerRec () {
      if ( customerRec == null )
         customerRec = findCustomer ( ef_CUST.getText() ) ;

      return( customerRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getCustomerRecNoLookup
    */
   /**
    *    This method will return the customerRec selected
    *  by the user or nothing. It will not perform
    *  a second lookup
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public TypeCustomerRec getCustomerRecNoLookup () {

      return( customerRec ) ;
   }


   /*------------------------------------------------------------*/
   /* getDescript
    */
   /**
    *    This method will return the customer description if it is valid.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return customer The Customer description if valid, NULL if not valid
    *//*---------------------------------------------------------*/
   public String getDescript () {
      String retStr = null ;

      if ( customerRec != null )
         retStr = customerRec.toString() ;

      return( retStr ) ;
   }



   /*------------------------------------------------------------*/
   /* requestFocus
    */
   /**
    *    This method has been subclassed to pass the
    *  focus to our entry field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void requestFocus () {
      ef_CUST.requestFocus() ;
   }





   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable
    *  our component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enable ) {
      ef_CUST.setEnabled ( enable ) ;
      pb_SELECTCUST.setEnabled ( enable ) ;
      super.setEnabled ( enable ) ;
   }


   /**
    *  This method will fire a 'userSelected' event to all registered listeners
    */
   private void fireEvent () {
       int i = 0 ;
       int size = listeners.size() ;

       while ( i < size ) {
           ((CustomerEntryPanelListener)listeners.elementAt(i)).customerSelected( customerRec ) ;
           i++  ;
       }
   }



   public void addCustomerEntryPanelListener ( CustomerEntryPanelListener listener )
   {
       listeners.addElement( listener ) ;
   }


   public void removeCustomerEntryPanelListener ( CustomerEntryPanelListener listener ) {
       listeners.removeElement( listener ) ;
   }


} /* CustomerEntryPanel */

