package com.ibm.nzna.projects.qit.gui ;       
/*-----------------------------------------------------------*/
/*   DateEntryField                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DateEntryField                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The DateEntryField is a JTextField extension that allows 
 * the user to enter dates in any format. It will also format the date
 * to the MM/DD/YYYY format in the getText/setText methods.
 *
 * @version 1.0, 10/3/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DateEntryField                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class DateEntryField extends JTextField implements ActionListener, FocusListener {

   public DateEntryField () {
      super() ;
      init() ;
   }

   public DateEntryField ( String text ) {
      super ( ) ;
      init() ;
      setText ( text ) ;
   }


   /**
     *  This method will initialize our control by setting our mask
     **/
   private void init () {
      setDocument ( EntrySystem.getDateMask(this) ) ;
      addActionListener ( this ) ;
      addFocusListener ( this ) ;
   }

   /**
     *  Subclassed to take the date given and
     * format it correctly. PLEASE NOTE THAT THIS
     * DATE MUST BE IN CDate.DATE_DATE format!
     **/
   public void setText ( String date ) {
      if ( CDate.checkDate ( date, CDate.DATE_DATE ) == 0 ) {
         super.setText ( CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR, CDate.DATE_DATE ),
                                      CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                      CDate.parse ( date, CDate.DATE_DAY, CDate.DATE_DATE ),
                                      0,0,0, CDate.DATE_WW_DATE ) ) ;
      } else
         super.setText ( "           " ) ;
   }


   /**
     *  This method will return the text that the user has entered
     * in the form of CDate.DATE_DATE 
     **/
   public String getText () {
      String date = super.getText() ;

      formatDate() ;
      if ( dateCorrect() ) 
         return ( CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR, CDate.DATE_WW_DATE ),
                                     CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_WW_DATE ),
                                     CDate.parse ( date, CDate.DATE_DAY, CDate.DATE_WW_DATE ),
                                     0,0,0,CDate.DATE_DATE ) ) ;
      return ( null ) ;
   }


   /**
     *  This method will format our date
     **/
   private void formatDate () {
      String     text      = super.getText().trim() ;
      String     date      = CDate.format4DigitYear ( text, CDate.DATE_WW_DATE ) ;

      if ( date != null )
         super.setText ( date ) ;
   }


   /**
     *  This method will return TRUE if the date entered is valid
     **/
   public boolean dateCorrect() {
      return ( CDate.checkDate ( super.getText(), CDate.DATE_WW_DATE ) == 0 ) ;
   }

   public void focusGained ( FocusEvent e ) {
      if ( super.getText().trim().length() == 0 ) {
         System.out.println ( "Selecitng ALL!" ) ;
         selectAll() ;
      }

   }

   public void focusLost ( FocusEvent e ) {
      formatDate() ;
   }

   public void actionPerformed ( ActionEvent e ) {
      formatDate() ;
   }


} /* DateEntryField */
