package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   DateRangeEntryDlg                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DateRangeEntryDlg                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog allows the user to enter a date range, this dialog
 * forces the 'From' date to come before the 'To' Date. The 'getResult()'
 * method should be used to 'show' the dialog once the constructor is called
 *
 *
 * @version 1.0, 7/30/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   DateRangeEntryDlg                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class DateRangeEntryDlg extends JDialog implements AppConst, ActionListener, WindowListener, FocusListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton        pb_OK        = null ;
   private DButton        pb_CANCEL    = null ;
   private DateEntryField ef_FROM_DATE = null ;
   private DateEntryField ef_TO_DATE   = null ;
   private JLabel         st_FROM      = null ;
   private JLabel         st_TO        = null ;
   private JLabel         st_DESCRIPT  = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String  fromDate   = null ;
   private String  toDate     = null ;
   private String  descript   = Str.getStr ( STR_ENTER_DATE_RANGE ) ;
   private boolean returnCode = false ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog.
    *  The fromDate is today, and that sets the 'to' date to 30 days from today
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DateRangeEntryDlg ( Frame parent, String title, String descript ) {
      super ( parent, title, true ) ;

      this.descript = descript ;
      this.fromDate = new CDate ( CDate.DATE_DATE ).today() ;
      this.toDate   = CDate.addDaysToDate ( fromDate, 30, CDate.DATE_DATE ) ;

      initialize() ;

   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param fromDate The date to show in the 'From' field. This will automatically make the 'to' date 30 days ahead
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DateRangeEntryDlg ( Frame parent, String title, String descript, String fromDate ) {
      super ( parent, title, true ) ;

      this.descript = descript ;
      this.fromDate = fromDate ;
      this.toDate   = CDate.addDaysToDate ( fromDate, 30, CDate.DATE_DATE ) ;

      initialize() ;

   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param fromDate The date to show in the 'From' field. 
    *  @param toDate The date to show in the 'To' field
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DateRangeEntryDlg ( Frame parent, String title, String descript, String fromDate, String toDate ) {
      super ( parent, title, true ) ;

      this.descript = descript ;
      this.fromDate = fromDate ;
      this.toDate   = toDate ;

      initialize() ;


   }






   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our local variables
    *  and GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      Container contentPane = getContentPane() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_FROM_DATE = new DateEntryField () ;
      ef_TO_DATE   = new DateEntryField () ;
      st_DESCRIPT  = new JLabel ( descript ) ;
      st_TO        = new JLabel ( Str.getStr ( STR_TO ) ) ;
      st_FROM      = new JLabel ( Str.getStr ( STR_FROM ) ) ;
      pb_OK        = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL    = new DButton ( Str.getStr ( STR_CANCEL ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our window                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_DESCRIPT ) ;
      contentPane.add ( st_FROM ) ;
      contentPane.add ( ef_FROM_DATE ) ;
      contentPane.add ( st_TO ) ;
      contentPane.add ( ef_TO_DATE ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_CANCEL ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addWindowListener              ( this ) ;
      pb_OK.addActionListener        ( this ) ;
      pb_CANCEL.addActionListener    ( this ) ;
      ef_FROM_DATE.addActionListener ( this ) ;
      ef_TO_DATE.addActionListener   ( this ) ;
      ef_FROM_DATE.addFocusListener  ( this ) ;
      ef_TO_DATE.addFocusListener    ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Data                                            */   
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_FROM_DATE.setText ( fromDate ) ;
      ef_TO_DATE.setText   ( toDate ) ;
      

      setSize ( 300, 220 ) ;
      WinUtil.centerWindow ( this ) ;
   }




   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the dialog and not
    *  return until the user presses either Ok or Cancel.
    *  If Cancel is hit, a NULL is returned, Otherwise the text is returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getResult() {
      setVisible ( true ) ;
      return( returnCode ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and size each
    *  component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       y         = 5 ;
      super.doLayout() ;

      st_DESCRIPT.setBounds ( 5, 5, size.width-10, rowHeight*3 ) ;
      y+=(rowHeight*3)+5 ;

      st_FROM.setBounds ( 5,   y, 75, rowHeight ) ;
      ef_FROM_DATE.setBounds ( 80, y, 150, rowHeight ) ;
      y+=rowHeight+5 ;

      st_TO.setBounds ( 5,   y, 75, rowHeight ) ;
      ef_TO_DATE.setBounds ( 80, y, 150, rowHeight ) ;

      pb_OK.setBounds ( 5, size.height-55, 70, 25 ) ;
      pb_CANCEL.setBounds ( 75, size.height-55, 85, 25 ) ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    THis method will be called when an action
    *  is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK, or ef_TO_DATE                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == pb_OK ) || 
           ( e.getSource() == ef_TO_DATE ) ) {

         fromDate = ef_FROM_DATE.getText() ;
         toDate   = ef_TO_DATE.getText() ;

         if ( ( CDate.checkDate ( fromDate, CDate.DATE_DATE ) != 0 ) ||
              ( CDate.checkDate ( toDate, CDate.DATE_DATE ) != 0 ) )
            GUISystem.printBox ( STR_ERROR, STR_DATE_ENTERED_WRONG ) ;
         else {
            returnCode = true ;
            dispose() ;
         }

      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ef_FROM_DATE                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ef_FROM_DATE ) {
         ef_TO_DATE.requestFocus() ;
         try {
            String text = ef_TO_DATE.getText() ;

            ef_TO_DATE.setSelectionStart ( 0 ) ;
            ef_TO_DATE.setSelectionEnd   ( text.length()-1 ) ;
         } catch ( Exception ec ) {
         }
      } else {
         fromDate   = null ;
         toDate     = null ;
         returnCode = false ;
         dispose() ;
      }

   }


   public void windowClosing ( WindowEvent e ) {}
   public void windowClosed ( WindowEvent e ) {}
   public void windowIconified ( WindowEvent e ) {}
   public void windowDeiconified ( WindowEvent e ) {}
   public void windowActivated ( WindowEvent e ) {
      
      ef_FROM_DATE.requestFocus();
      
      if ( fromDate != null ) {
         try {
            ef_FROM_DATE.setSelectionStart ( 0 ) ;
            ef_FROM_DATE.setSelectionEnd   ( fromDate.length()-1 ) ;
         } catch ( Exception ex ) {
         }
      }
   }
   public void windowDeactivated ( WindowEvent e ) {}
   public void windowOpened ( WindowEvent e ) {}





   /*------------------------------------------------------------*/
   /* focusGained
    */
   /**
    *    This method will be called when the entry
    *  field has gained focus
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusGained ( FocusEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* focusLost
    */
   /**
    *    This method will be called when the focus
    *  has been lost in the entry field. It will automatically
    *  format the date
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusLost ( FocusEvent e ) {
      if ( e.getSource() instanceof JTextField )
         EntrySystem.fixDateField ( (JTextField) e.getSource() ) ;
   }





   /*------------------------------------------------------------*/
   /* getFromDate
    */
   /**
    *    This method will return the date that the
    *  user entered into the 'From' field.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String. NULL if the user cancelled the dialog
    *//*---------------------------------------------------------*/
   public String getFromDate () {
      return( fromDate ) ;
   }



   /*------------------------------------------------------------*/
   /* getToDate
    */
   /**
    *    This method will return the date that the
    *  user entered into the 'To' field.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String. NULL if the user cancelled the dialog
    *//*---------------------------------------------------------*/
   public String getToDate () {
      return( toDate ) ;
   }


} /* DateRangeEntryDlg */
