package com.ibm.nzna.projects.qit.gui ;    
/*-----------------------------------------------------------*/
/*   EditNavPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EditNavPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
/**
 * This is a generic NavPanel that holds the following information: <br><pre>
 *
 * 1. Creation Time
 * 2. Last Touched By User
 * 3. Last Touched On Timestamp<br></pre>
 *
 * This panel was designed to hold standard information for editor panels. These
 * panels usually edit an object in a database, and if the information above is available,
 * this panel should be used.<br><br>
 *
 * This panel also has a 'dynamicPanel' which can be added and changed. This 'dynamicPanel' appears
 * above all information in this panel. This provides a way for any editor to add/remove controls to
 * this panel.<br><br>
 *
 * This panel also supplies it's own JScrollPane, so no matter how large the 'dynamicPanel' is, all of it
 * will always be visible.
 *
 * @version 1.0, 4/23/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EditNavPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class EditNavPanel extends JPanel implements NavPanel, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel    st_LAST_TOUCHED_ON  = null ;
   private JLabel    st_LAST_TOUCHED_BY  = null ;
   private JLabel    st_CREATION_DATE    = null ;
   private JLabel    stt_LAST_TOUCHED_ON = null ;
   private JLabel    stt_LAST_TOUCHED_BY = null ;
   private JLabel    stt_CREATION_DATE   = null ;
   private JPanel    holdPanel           = null ;
   private JPanel    dynamicHolder       = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String title = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the EditNavPanel.
    *  
    *  
    *
    *  @param title The title of the panel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EditNavPanel ( String title ) {
      this.title = title ;
   }




   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return our title
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( title ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our controls and
    *  variables
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public void initialize () {
      Color       purple      = new Color ( 128, 0, 128 ) ;
      JPanel      panel       = null ;
      NavList     navList     = null ; 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_LAST_TOUCHED_ON       = new JLabel ( Str.getStr ( STR_LAST_TOUCHED_ON )+"  " ) ;
      st_LAST_TOUCHED_BY       = new JLabel ( Str.getStr ( STR_LAST_TOUCHED_BY )+"  " ) ;
      stt_LAST_TOUCHED_ON      = new JLabel ( "" ) ;
      stt_LAST_TOUCHED_BY      = new JLabel ( "" ) ;
      st_CREATION_DATE         = new JLabel ( Str.getStr ( STR_CREATION_DATE )+" " ) ;
      stt_CREATION_DATE        = new JLabel ( "" ) ;
      navList                  = new NavList () ;
      dynamicHolder            = new JPanel () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_LAST_TOUCHED_ON.setForeground ( purple ) ;
      st_LAST_TOUCHED_BY.setForeground ( purple ) ;
      st_CREATION_DATE.setForeground ( purple ) ;
      st_LAST_TOUCHED_ON.setOpaque ( false ) ;
      st_CREATION_DATE.setOpaque ( false ) ;
      st_LAST_TOUCHED_BY.setOpaque ( false ) ;
      stt_LAST_TOUCHED_ON.setOpaque ( false ) ;
      stt_LAST_TOUCHED_BY.setOpaque ( false ) ;
      st_CREATION_DATE.setOpaque ( false ) ;
      navList.setOpaque ( false ) ;  
      dynamicHolder.setOpaque ( false ) ;
      dynamicHolder.setLayout ( new BorderLayout() ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI to the Nav List                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navList.add ( dynamicHolder ) ;
      panel = new JPanel() ;
      panel.setOpaque ( false ) ;
      panel.setLayout ( new BorderLayout () ) ;
      panel.add ( st_LAST_TOUCHED_ON, BorderLayout.WEST ) ;
      panel.add ( stt_LAST_TOUCHED_ON, BorderLayout.CENTER ) ;
      navList.add ( panel ) ;

      panel = new JPanel() ;
      panel.setOpaque ( false ) ;
      panel.setLayout ( new BorderLayout () ) ;
      panel.add ( st_LAST_TOUCHED_BY, BorderLayout.WEST ) ;
      panel.add ( stt_LAST_TOUCHED_BY, BorderLayout.CENTER ) ;
      navList.add ( panel ) ;                                      

      panel = new JPanel() ;
      panel.setOpaque ( false ) ;
      panel.setLayout ( new BorderLayout () ) ;
      panel.add ( st_CREATION_DATE, BorderLayout.WEST ) ;
      panel.add ( stt_CREATION_DATE, BorderLayout.CENTER ) ;
      navList.add ( panel ) ;                                      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, add the 1 component (navList) to our panel         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      add ( navList, BorderLayout.CENTER ) ;

   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will remove all our items and
    *  will release our resources
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      removeAll() ;
   }




   /*------------------------------------------------------------*/
   /* setDynamicPanel
    */
   /**
    *    This method will set the dynamicPanel that
    *  we use. If a NULL is passed, no dynamicPanel
    *  will show
    *
    *  @param panel The panel to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDynamicPanel ( JPanel panel ) {
      dynamicHolder.removeAll() ;

      if ( panel != null )
         dynamicHolder.add ( panel, BorderLayout.CENTER ) ;
   }





   /*------------------------------------------------------------*/
   /* setAllData
    */
   /**
    *    This method will set the Last Touched On, Last
    *  Touched By and Creation Date data.
    *  
    *
    *  @param lastTouchedOn The time to represent Last Touched
    *  @param lastTouchedBy The person to represent who last touched
    *  @param creationdDate The creation date
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAllData ( String lastTouchedOn, String lastTouchedBy, String creationDate ) {
      setLastTouchedOn ( lastTouchedOn ) ;
      setLastTouchedBy ( lastTouchedBy ) ;
      setCreationDate  ( creationDate ) ;
   }




   /*------------------------------------------------------------*/
   /* setLastTouchedOn
    */
   /**
    *    This method will set the text value for the
    *  item 'Last Touched On'
    *  
    *
    *  @param lastTouchedOn
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLastTouchedOn ( String lastTouchedOn ) {
      stt_LAST_TOUCHED_ON.setText ( lastTouchedOn ) ;
   }


   /*------------------------------------------------------------*/
   /* setLastTouchedBy
    */
   /**
    *    This method will set the text value for the
    *  item 'Last Touched By'
    *  
    *
    *  @param lastTouchedBy
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLastTouchedBy ( String lastTouchedBy ) {
      stt_LAST_TOUCHED_BY.setText ( lastTouchedBy ) ;
   }




   /*------------------------------------------------------------*/
   /* setCreationDate
    */
   /**
    *    This method will set the creation date
    *  value
    *  
    *
    *  @param creationDate
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCreationDate ( String creationDate ) {
      stt_CREATION_DATE.setText ( creationDate ) ;
   }

} /* EditNavPanel */
