package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   EnterUserDlg                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EnterUserDlg                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This dialog will ask the user to enter a user id. 
 *
 *
 * @version 1.0, 2/26/1999 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   EnterUserDlg                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class EnterUserDlg extends JDialog implements ActionListener, AppConst, WindowListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private UserEntryPanel ef_USERID    = null ;
   private DButton        pb_OK        = null ;
   private DButton        pb_CANCEL    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String  descript = "" ;
   private UserRec userRec  = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a default EnterUserDlg
    *  with the specified title and description. This
    *  constructor will not show the dialog. To show the dialog
    *  please call the 'getUserRec()' function.
    *  
    *
    *  @param title The title to display on the title bar
    *  @param description the description to show in the window
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public EnterUserDlg ( Frame parent, String title, String descript ) {
      super ( parent ,title,true ) ;
      setSize ( 300, 150 ) ;
      setDefaultCloseOperation ( DO_NOTHING_ON_CLOSE ) ;
      this.descript = descript ;
      createControls() ;
      addWindowListener ( this ) ;
      WinUtil.centerChildInParent (this, parent) ;
   }






   /*------------------------------------------------------------*/
   /* getUserRec
    */
   /**
    *    This method will show the dialog, wait for
    *  the user to either enter a valid user id or press
    *  'Cancel' and will then return the User record.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec of user if entered, NULL if cancel or user wasn't found
    *//*---------------------------------------------------------*/
   public UserRec getUserRec ( ) {
      setVisible ( true ) ;
      return( userRec ) ;
   }



   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createControls () {

      /* Set our layout */
      getContentPane().setLayout ( null ) ;

      /* Create our controls */
      ef_USERID = new UserEntryPanel ( ) ;
      pb_OK     = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;

      /* Add our listeners */
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;


      /* Add the controls */
      getContentPane().add ( ef_USERID ) ;
      getContentPane().add ( pb_OK ) ;
      getContentPane().add ( pb_CANCEL ) ;

   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will position and size all of our
    *  components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = 18 ;

      super.doLayout() ;

      ef_USERID.setBounds ( 5, 5, size.width-(15), rowHeight*2 ) ;

      pb_OK.setBounds ( 5, size.height-60, 70, 25 ) ;
      pb_CANCEL.setBounds ( 75, size.height-60, 85, 25 ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event needs to be addressed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() instanceof DButton ) {
         DButton button = (DButton) e.getSource() ;

         if ( pb_OK == button ) {
            if ( saveData() )
               dispose() ;
         }

         else if ( pb_CANCEL == button ) {
            userRec = null ;
            dispose() ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will attempt to save the data
    *  we are displaying. If the data is good, then
    *  this method will return TRUE
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Data was good, FALSE = Data was bad
    *//*---------------------------------------------------------*/
   private boolean saveData () {

      userRec = ef_USERID.getUserRec() ;

      return( userRec != null  ) ;
   }





   /*------------------------------------------------------------*/
   /* setUserId
    */
   /**
    *    This method will set the text that appears
    *  in the user id field
    *  
    *
    *  @param userId The string representation of the user id
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setUserId ( String userId ) {
      ef_USERID.setText ( userId ) ;
   }


   public void windowClosing ( WindowEvent e ) {}
   public void windowClosed ( WindowEvent e ) {}
   public void windowIconified ( WindowEvent e ) {}
   public void windowDeiconified ( WindowEvent e ) {}
   public void windowActivated ( WindowEvent e ) {ef_USERID.requestFocus();}
   public void windowDeactivated ( WindowEvent e ) {}
   public void windowOpened ( WindowEvent e ) {}


} /* EnterUserDlg */
