package com.ibm.nzna.projects.qit.gui ;              
/*-----------------------------------------------------------*/
/*   ExtWordProcButton                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ExtWordProcButton                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import java.io.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
/**
 * This panel simply holds a button allowing the user to launch the external word processor.
 * The actual button can be a normal DButton which is gray and contains only text, or can be
 * a HotLinkLabel which is opaque and contains an Icon with text.<br><br>
 *
 * An ActionListener can be added to this control. It will fire ActionEvents when a button
 * is pressed. The ID of the button determines what action was taken. For example, an ID of 0 means
 * that the user hit the 'Launch Word Proc.' button. An ID of 1 means that the user hit the reimport button.
 *
 * @version 1.0, 4/27/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ExtWordProcButton                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class ExtWordProcButton extends JPanel implements AppConst, Runnable, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int HOTLINKLABEL         = 1 ;
   public static final int DBUTTON              = 2 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton      pb_LAUNCH         = null ;
   private HotLinkLabel st_LAUNCH         = null ;
   private HotLinkLabel st_REIMPORT       = null ;
   private DButton      pb_REIMPORT       = null ;
   private boolean      launched          = false ;
   private String       wordProcName      = null ;
   private int          buttonType        = 0 ;
   private String       saveFilename      = null ;
   private String       body              = null ;
   private Process      extProcess        = null ;
   private Vector       listenerVec       = new Vector ( 1, 1 ) ;
   private boolean      hasChanged        = false ;
   private boolean      autoLaunch        = true ;
   private int          currentId         = 0 ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the button
    *  
    *  
    *
    *  @param buttonType This must be either DBUTTON, or HOTLINKLABEL as defined in this class
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ExtWordProcButton ( int buttonType ) {
      try {
         wordProcName = PropertySystem.getString ( PropertySystem.PROPERTY_EXTERNAL_WORDPROC_NAME ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our GUI                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         this.buttonType = buttonType ;
         switch ( buttonType ) {
            case DBUTTON :
               pb_LAUNCH = new DButton ( wordProcName ) ;
               pb_REIMPORT = new DButton ( Str.getStr ( STR_REIMPORT_BODY ) ) ;
               setLayout ( new BorderLayout () ) ;
               add ( pb_LAUNCH, BorderLayout.CENTER ) ;
               pb_LAUNCH.addActionListener ( this ) ;
               pb_REIMPORT.addActionListener ( this ) ;
               break ;

            case HOTLINKLABEL :
               st_LAUNCH  = new HotLinkLabel ( wordProcName,
                                               ImageSystem.getImageIcon ( this, ImageSystem.SMALL_CHECKBOOK ) ) ;
               st_REIMPORT = new HotLinkLabel ( Str.getStr ( STR_REIMPORT_BODY ) ) ;
               setLayout ( new BorderLayout () ) ;
               add ( st_LAUNCH, BorderLayout.CENTER ) ;
               st_LAUNCH.addActionListener ( this ) ;
               st_REIMPORT.addActionListener ( this ) ;
               break ;
         }
      } catch ( Exception e ) {
      }
   } /* Constructor  */





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are waiting for has ocurred
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {                           

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Reimport if we have already launched                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == pb_REIMPORT ) || ( e.getSource() == st_REIMPORT ) ) {
         launched = false ;

         importBody() ;

         switch ( buttonType ) {
            case DBUTTON :
               remove ( pb_REIMPORT ) ;
               add ( pb_LAUNCH, BorderLayout.CENTER ) ;
               break ;

            case HOTLINKLABEL :
               remove ( st_REIMPORT ) ;
               add ( st_LAUNCH, BorderLayout.CENTER ) ;
               break ;

         }

         revalidate() ;
         currentId = 1 ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Launch if we haven't yet                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         currentId = 0 ;
         if ( autoLaunch )
            launch() ;
      } /* else  */

      new Thread ( this ).start() ;
   } /* actionPerf.  */





   /*------------------------------------------------------------*/
   /* launch
    */
   /**
    *    This method will actually launch the separate
    *  process for the word processor
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean launch () {
      String[]   params     = new String[2] ;
      boolean    rc         = false ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the Body                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Make sure we have a body                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( body == null ) || ( body.length() == 0 ) )
         body = "<html><body>Erase Me</body></html>" ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save the file to the temp dir.                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      saveFilename = System.currentTimeMillis()+".html" ;
      QuestUtil.writeTempFile ( saveFilename, body.getBytes() ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the params                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         params[0] = PropertySystem.getString ( PropertySystem.PROPERTY_EXTERNAL_WORDPROC_EXE ) ;
         params[1] = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator+saveFilename ;

         extProcess = Runtime.getRuntime().exec ( params ) ;
         rc = true ;
      } catch ( Exception e ) {
         GUISystem.printBox ( STR_INFO, STR_CANNOT_START_EXT_WORD_PROC ) ;
      }


      launched = rc ;

      if ( launched ) {
         switch ( buttonType ) {
            case DBUTTON :
               remove ( pb_LAUNCH ) ;
               add ( pb_REIMPORT, BorderLayout.CENTER ) ;
               break ;

            case HOTLINKLABEL :
               remove ( st_LAUNCH ) ;
               add ( st_REIMPORT, BorderLayout.CENTER ) ;
               break ;

         }

         revalidate() ;
      } /* if launched  */

      return( rc ) ;
   }




   /*------------------------------------------------------------*/
   /* setBody
    */
   /**
    *    This method will set the body that will be
    *  sent to the external word processor
    *  
    *
    *  @param body The body to send
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBody ( String body ) {
      this.body = body ;
   }




   /*------------------------------------------------------------*/
   /* getBody
    */
   /**
    *    This method will return the body that we
    *  got back from the external word processor. If the user
    *  has not pressed 'Import Body' yet, then we will prompt them to.
    *  If they choose not to import a body, this method will return NULL.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getBody (  ) {
      if ( launched ) {
         if ( GUISystem.printBox ( STR_QUESTION, STR_NOT_IMPORTED_BODY_YET_IMPORT ) )
            importBody() ;
         else
            body = null ;
      }
      return( body ) ;
   }




   /*------------------------------------------------------------*/
   /* importBody
    */
   /**
    *    This method will reimport the body from the
    *  file we saved before we launched the ext.
    *  word processor
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void importBody () {
      try {
         boolean killProcess = PropertySystem.getBool ( PropertySystem.PROPERTY_CLOSE_EXTERNAL_WORDPROC ) ;


         if ( killProcess )
            extProcess.destroy() ;

         body = FileUtil.readFile ( PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+
                                    File.separator+
                                    saveFilename ) ;
         extProcess = null ;
         hasChanged = true ;
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will fire our action events to
    *  all of our listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      int i    = 0 ;
      int size = listenerVec.size() ;
      ActionEvent e = new ActionEvent ( this, currentId, body );

      while ( i < size ) {
         ((ActionListener)listenerVec.elementAt(i)).actionPerformed(e) ;
         i++ ;
      }
   }




   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener e ) {
      listenerVec.addElement ( e ) ;
   }




   /*------------------------------------------------------------*/
   /* removeActionListener
    */
   /**
    *    This method will remove an actionListener
    *  from our list of listeners
    *  
    *
    *  @param listener The listener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeActionListener ( ActionListener e ) {
      listenerVec.removeElement ( e ) ;
   }

   public void finalize () {
      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Kill the word proc if we are supposed to                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( extProcess != null ) {
            boolean killProcess = PropertySystem.getBool ( PropertySystem.PROPERTY_CLOSE_EXTERNAL_WORDPROC ) ;


            if ( killProcess )
               extProcess.destroy() ;
         }
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* getLaunched
    */
   /**
    *    This method will return TRUE if the
    *  program has been launched and the user
    *  has not yet reimported the body
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getLaunched () {
      return( launched ) ;
   }





   /*------------------------------------------------------------*/
   /* getHasChanged
    */
   /**
    *    This method will return TRUE if the user
    *  has changed the text with a word processor
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean getHasChanged () {
      return( hasChanged ) ;
   }





   /*------------------------------------------------------------*/
   /* setAutoLaunch
    */
   /**
    *    This method determines if the button automatically
    *  launches the word processor when pushed. If this
    *  value is set to FALSE, then only an action performed is fired 
    *  when the button is pushed. The programmer must call the 
    *  'launch' method manually. The default value for this property
    *  is true.
    *
    *  @param autoLaunch TRUE/FALSE
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setAutoLaunch ( boolean autoLaunch ) {
      this.autoLaunch = autoLaunch ;
   }

} /* ExtWordProcButton */
