package com.ibm.nzna.projects.qit.gui ;     
/*-----------------------------------------------------------*/
/*   FocusButton                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FocusButton                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;     
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
/**
 * This button is a Focus Button which is primarily used 
 * in the AppDefaultWin's task bar at the top of the window. This
 * button extends JPanel and holds a JToggleButton so that it appears sticky. It also
 * displays a panel's icon and name, as well as the 'X' close button when
 * the mouse is over the button
 *
 * @version 1.0, 2/10/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FocusButton                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class FocusButton extends JPanel implements AppConst, ActionListener, MouseListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final Color lightBlue = new Color ( 0, 0, 230 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String            focusComponentTitle = null ;
   private String            labelText           = null ;
   private ImageIcon         labelIcon           = null ;
   private AppDefaultWin     parentWin           = null ;
   private boolean           mouseOver           = false ;
   private ApplicationPanel  focusPanel          = null ;
   private boolean           workspacePanel      = false ;
   private boolean           initialized         = false ;
   private boolean           hasFocus            = false ;
   private int               iconWidth           = 0 ;
   private int               closeIconWidth      = 0 ;
   private int               labelTextWidth      = 0 ;
   private Dimension         prefSize            = new Dimension ( 150, GUISystem.getRowHeight()+2 ) ;
   private ImageIcon         closeIcon           = null ;
   private Font              currentFont         = (Font)javax.swing.UIManager.getDefaults().get ( "Button.font" ) ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the focus button with an icon and text.
    *  
    *
    *  @param text The text to display on the button
    *  @param icon The icon to display
    *  @param focusComponentTitle The title of the component that this focus button represents
    *  @param focusPanel The panel that this button represents
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FocusButton ( String title, ImageIcon icon, String focusComponentTitle, ApplicationPanel focusPanel ) {
      setText ( title ) ;
      setIcon ( icon ) ;
      this.focusComponentTitle = focusComponentTitle ;
      this.focusPanel          = focusPanel ;
      initialize() ;
   }






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the focus button with only text
    *  
    *
    *  @param text The text to displaye
    *  @param focusComponentTitle The title of the component that this focus button represents
    *  @param focusPanel The panel that this button represents
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FocusButton ( String text, String focusComponentTitle, ApplicationPanel focusPanel ) {
      setText ( text ) ;
      this.focusComponentTitle = focusComponentTitle ;
      this.focusPanel          = focusPanel ;
      initialize() ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  our focus button with only an icon
    *  
    *
    *  @param icon The icon
    *  @param focusComponentTitle The title of the component that this focus button represents
    *  @param focusPanel The panel that this button represents
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public FocusButton ( ImageIcon icon, String focusComponentTitle, ApplicationPanel focusPanel ) {
      setIcon ( icon ) ;
      this.focusComponentTitle = focusComponentTitle ;
      this.focusPanel          = focusPanel ;
      initialize () ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will adjust all of our properties
    *  to have a conforming look
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize() {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get the information about the 'X' Close Button          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      closeIcon = ImageSystem.getImageIcon ( this, ImageSystem.SMALLCLOSE ) ;
      closeIconWidth = closeIcon.getImage().getWidth ( this )+5 ;                      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   We need to know about the mouse                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addMouseListener ( this ) ;
      setCursor( Cursor.getPredefinedCursor(Cursor.HAND_CURSOR)) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   This hack is so that the workspace panel never          */
      /* gets a close button                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      workspacePanel = (focusComponentTitle.indexOf ( Str.getStr ( STR_WORKSPACE ) ) >= 0 ) ;

      initialized = true ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when we
    *  are selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      ///*-----------------------------------------------------------*/
      ///*                                                           */
      ///*   We must have a parent win by now!                       */
      ///*                                                           */
      ///*-----------------------------------------------------------*/
      //if ( parentWin == null )
      //   parentWin = GUISystem.getParentDefWin ( this ) ;

      ///*-----------------------------------------------------------*/
      ///*                                                           */
      ///*   Toggle Button                                           */
      ///*                                                           */
      ///*-----------------------------------------------------------*/
      //if ( e.getSource() == toggleButton ) {
      //   if ( parentWin != null )
      //      parentWin.setPanelFocus ( focusComponentTitle, focusPanel ) ;
      //}

      ///*-----------------------------------------------------------*/
      ///*                                                           */
      ///*   PB_CLOSE                                                */
      ///*                                                           */
      ///*-----------------------------------------------------------*/
      //else if ( e.getSource() == closeButton ) {

      //   if ( ( parentWin != null ) && ( !workspacePanel ) )
      //      parentWin.closePanel ( focusPanel, this, true ) ;
      //}
   }





   /*------------------------------------------------------------*/
   /* mouseEntered
    */
   /**
    *    This method will be called once the mouse
    *  has entered our realm
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseEntered ( MouseEvent e ) {
      mouseOver = true ;
      invalidate() ;
      repaint();
   }




   /*------------------------------------------------------------*/
   /* mouseExited
    */
   /**
    *    This method will be called when the
    *  mouse has left our realm
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseExited ( MouseEvent e) {
      mouseOver = false ;
      invalidate() ;
      repaint();
   }




   /*------------------------------------------------------------*/
   /* mouseReleased
    */
   /**
    *    This method will be called when all
    *  mouse buttons are released
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseReleased ( MouseEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* mousePressed
    */
   /**
    *    This method will be called when a mouse
    *  button has been pressed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mousePressed ( MouseEvent e ) {

      /* Get our parent if we don't have it yet  */
      if ( parentWin == null )
         parentWin = GUISystem.getParentDefWin ( this ) ;


      if ( parentWin != null ) {
         Dimension size = getSize() ;
         Point     loc  = e.getPoint() ;

         /* If they clicked the close icon, close  */
         if ( loc.x > ((size.width-closeIconWidth)) )
            parentWin.closePanel ( focusPanel, this, true ) ;

         /* Ok, the user clicked somewhere else  */
         else
            parentWin.setPanelFocus ( focusComponentTitle, focusPanel ) ;
      }
      repaint();
   }




   /*------------------------------------------------------------*/
   /* mouseClicked
    */
   /**
    *    This method will be called when the mouse
    *  has been clicked
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseClicked ( MouseEvent e  ) {

   }





   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our preferred
    *  size
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      int width = iconWidth+labelTextWidth+closeIconWidth+8 ;

      if ( width < 100 )
         prefSize.width = 100 ;
      else
         prefSize.width = 150 ;
      return( prefSize ) ;
   }




   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return the minimum
    *  size of this control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( getPreferredSize() ) ;
   }



   /*------------------------------------------------------------*/
   /* contains
    */
   /**
    *    This method will determine if this focus
    *  button contains the specified QuestPanel.
    *  
    *
    *  @param questPanel The QuestPanel to comapre to
    *  @param None
    *  @param None
    *
    *  @return TRUE = Contains the panel, FALSE = does not contain
    *//*---------------------------------------------------------*/
   protected boolean contains ( QuestPanel questPanel ) {
      return( focusPanel.getQuestPanel() == questPanel ) ;
   }

   /*------------------------------------------------------------*/
   /* contains
    */
   /**
    *    This method will determine if this focus
    *  button contains the specified ApplicationPanel.
    *  
    *
    *  @param appPanel The QuestPanel to comapre to
    *  @param None
    *  @param None
    *
    *  @return TRUE = Contains the panel, FALSE = does not contain
    *//*---------------------------------------------------------*/
   protected boolean contains ( ApplicationPanel panel ) {
      return( focusPanel == panel ) ;
   }




   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method will free all of the resources
    *  we have allocated
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void free () {
      focusComponentTitle = null ;
      parentWin           = null ;
      focusPanel          = null ;
   }




   /*------------------------------------------------------------*/
   /* getFocusApplicationPanelTitle
    */
   /**
    *    This method will return the title of the
    *  focus component that this button is using
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getFocusApplicationPanelTitle () {
      return( focusComponentTitle ) ;
   }




   /*------------------------------------------------------------*/
   /* getFocusApplicationPanel
    */
   /**
    *    This method will return the component
    *  that this focus button represents
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ApplicationPanel
    *//*---------------------------------------------------------*/
   public ApplicationPanel getFocusApplicationPanel () {
      return( focusPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will set ourselves to be the focused component.
    *
    *  @param gotFocus - TRUE = receiving focus, FALSE = losing focus
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus ( boolean gotFocus) {
      hasFocus = gotFocus ;
      invalidate() ;
      repaint();
   }




   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text that is displayed
    *  on the focus button
    *  
    *
    *  @param text The text to set
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText ( String text ) {
      FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics (currentFont) ;
      this.labelText = text ;

      /* If this string says 'Workspace' then we must be the workspace!  */
      if ( text.indexOf ( Str.getStr ( STR_WORKSPACE ) ) != -1 )
         workspacePanel = true ;

      labelTextWidth = fm.stringWidth ( text ) ;
      if ( initialized )
         repaint();
   }


   /**
     *  This method will set the icon that we are drawing
     *
     * @param icon The Icon to set
     **/
   public void setIcon ( ImageIcon icon ) {
      this.labelIcon = icon ;

      /* Get the width of this image  */
      iconWidth = labelIcon.getImage().getWidth ( this ) ;

      if ( initialized )
         repaint();
   }


   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param g The graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      Dimension  size = getSize() ;
      int        x    = 3 ;
      //Graphics2D g2   = (Graphics2D) g ;

      /* Anti Alias  */
      //g2.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON ) ;

      super.paintComponent(g) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always draw the background lighter if we are selected   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( hasFocus ) {
         try {
            g.setColor ( new Color ( getBackground().getRed()+30, 
                                  getBackground().getGreen()+30, 
                                  getBackground().getBlue()+30 ) ) ;
         } catch ( Exception e ) {
         }
         g.fillRect ( 0, 0, size.width, size.height ) ;
         g.setColor ( lightBlue ) ;
      } else {
         g.setColor ( getBackground() ) ;
         g.fillRect ( 0, 0, size.width, size.height ) ;
         g.setColor ( Color.black ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Draw the Icon and Text and Close 'X'                    */                           
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( labelIcon != null ) {
         g.drawImage ( labelIcon.getImage(), x, ((size.height/2)-(labelIcon.getImage().getHeight(this)/2)), this ) ;
         x+=iconWidth+2 ;
      }

      if ( labelText != null ) {
         Shape clip = g.getClip() ;

         g.setClip ( new Rectangle ( 0, 0, size.width-iconWidth-2, size.height ) ) ;
         g.drawString ( labelText, x, FontUtil.getYCentered ( currentFont, size.height ) ) ;
         g.setClip ( clip ) ;
      }

      /* Only draw the X if: The mouse is over us or we are indented and we are not the workspace  */
      if ( !workspacePanel ) {
         if ( mouseOver || hasFocus )
            g.drawImage ( closeIcon.getImage(), size.width-closeIconWidth-3, ((size.height/2)-(closeIcon.getImage().getHeight(this)/2)), this ) ;
      }


      size.width-- ;
      size.height-- ;                                                                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always draw an indented rectangle if we are selected    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( hasFocus ) {
         g.setColor ( Color.darkGray ) ;
         g.drawLine ( 0, 0, 0, size.height ) ;
         g.drawLine ( 0, 0, size.width, 0 ) ;
         g.setColor ( Color.white ) ;
         g.drawLine ( 1, size.height, size.width, size.height ) ;
         g.drawLine ( size.width, size.height, size.width, 0 ) ;
      } else {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Only draw the raised bevel if the mouse is over us      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( mouseOver ) {
            g.setColor ( Color.white ) ;
            g.drawLine ( 0, 0, 0, size.height ) ;
            g.drawLine ( 0, 0, size.width, 0 ) ;
            g.setColor ( Color.darkGray ) ;
            g.drawLine ( 1, size.height, size.width, size.height ) ;
            g.drawLine ( size.width, size.height, size.width, 0 ) ;
         }
      }
   }

} /* FocusButton */
