package com.ibm.nzna.projects.qit.gui ; 
/*-----------------------------------------------------------*/
/*   HTMLPaneBrowser                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HTMLPaneBrowser                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import java.io.* ;
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.net.* ;
import horst.* ;
import java.beans.* ;
/**
 * The browser control displays HTML documents in a web browser like interface 
 *
 *
 * Currently the browser control is handled by the HTMLWindow
 *
 * @version 1.0, 2/1/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HTMLPaneBrowser                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class HTMLPaneBrowser extends JComponent implements Browser, HTMLPaneStatusListener, LinkListener, ActionListener, AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static URL dummyURL    = null ;
   private static int nextFileNum = 1 ;
   private static final String VIEWPORT_NAME = "QUEST" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private HTMLWindow             pnl_BROWSER        = null ;
   private HTMLPane               htmlPane           = null ;
   private AppDefaultWin          appDefaultWin      = null ;
   private BrowserListener        listener           = null ;
   private String                 currentURL         = null ;
   private ButtonPanel            buttonPanel        = null ;
   private int                    zoom               = 3 ;
   private boolean                firstZoomSet       = false ;
   private boolean                showZoom           = true ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the browser component.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HTMLPaneBrowser () {         

      try {
         if ( dummyURL == null )
            dummyURL = new URL ( "http://localhost/" ) ;
      } catch ( Exception e ) {
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the browser                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         zoom          = 3;//PropertySystem.getInt ( PropertySystem.PROPERTY_BROWSER_ZOOM ) ;
         buttonPanel   = new ButtonPanel () ;
         initializeHTMLPaneBrowser() ;               
      } catch ( Exception e ) {
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.setBackground ( Color.white ) ;
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;           


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      htmlPane.addLinkListener ( this ) ;
      htmlPane.addStatusListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout () ) ;
      add ( pnl_BROWSER, BorderLayout.CENTER ) ;

   }





   /*------------------------------------------------------------*/
   /* initializeHTMLPaneBrowser
    */
   /**
    *    This method will initialize the browser
    *  component that we will use
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initializeHTMLPaneBrowser () {
      pnl_BROWSER = new HTMLWindow() ;
      htmlPane    = new HTMLPane () ;
      pnl_BROWSER.setHTMLPane ( htmlPane ) ;
   }





   /*------------------------------------------------------------*/
   /* back
    */
   /**
    *    This method will force the browser
    *  to go back 1 page
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void back () {
      htmlPane.back() ;
   }





   /*------------------------------------------------------------*/
   /* forward
    */
   /**
    *    This method will force the browser
    *  to go forward a page
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void forward () {
      htmlPane.forward() ;
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will stop the browser from
    *  loading a page
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      htmlPane.stop() ;
      if ( listener != null )
         listener.pageStopped () ;
   }





   /*------------------------------------------------------------*/
   /* loadURL
    */
   /**
    *    This method will load the specified
    *  URL into the browser panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void loadURL ( String url ) {
      currentURL = url ;
      new Thread ( this ).start() ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will read in the URL we are to display
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {

      try {
         if ( listener != null )
            listener.pageStarted ( currentURL ) ;
         htmlPane.openPage ( new URL ( currentURL ) ) ;

      } catch ( Exception e ) {
         if ( listener != null )
            listener.pageStopped () ;
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_CANNOT_LOAD_URL )+" "+currentURL  ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* reload
    */
   /**
    *    This method will reload the page
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void reload () {
      new Thread ( this ).start() ;
   }





   /*------------------------------------------------------------*/
   /* copy
    */
   /**
    *    This method will attempt to copy the
    *  selected text on the web page
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE - Text was copied, FALSE - could not copy
    *//*---------------------------------------------------------*/
   public boolean copy () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* setHTMLPaneStatusListener
    */
   /**
    *    This method will set the one listener that
    *  will receive events from this HTMLPaneBrowser.
    *  
    *
    *  @param listener The listener
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addBrowserListener ( BrowserListener listener ) {
      this.listener = listener ;
   }

   public void removeBrowserListener ( BrowserListener listener ) {
      this.listener = null ;
   }


   /*------------------------------------------------------------*/
   /* setPageContent
    */
   /**
    *    This method will set the content of the
    *  browser to the string passed in
    *  
    *
    *  @param content The content to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPageContent ( String content ) {
      try {
         String filename = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+
                           File.separator+
                           (nextFileNum++)+"_"+System.currentTimeMillis()+"_temp.html" ;

         FileUtil.saveFile ( filename, content ) ;
         loadURL ( "file:/"+filename ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* showZoom
    */
   /**
    *    This method will turn the zoom controls
    *  either on or off
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showZoom ( boolean showZoom ) {

      showZoom = false ;
      buttonPanel.setVisible ( showZoom ) ;
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* canGoBack
    */
   /**
    *    This method will return TRUE/FALSE
    *  if the browser can go back 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean canGoBack () {
      return( htmlPane.canMoveBack ( ) ) ;
   }




   /*------------------------------------------------------------*/
   /* canGoForward
    */
   /**
    *    This method will return TRUE if the
    *  browser can go forward
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean canGoForward () {
      return( htmlPane.canMoveForward ( ) );
   }





   /*------------------------------------------------------------*/
   /* mouseOverLink
    */
   /**
    *    This method will be called when the HTMLWindow
    *  detects that the mouse is positioned over a link
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseOverLink ( LinkEvent e ) {
      setStatus ( e.getLink() ) ;
   }





   /*------------------------------------------------------------*/
   /* mousePressedOverLink
    */
   /**
    *    This method will be called when the HTMLWindow
    *  has processed a mouse click over a link
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean mousePressedOverLink ( LinkEvent e ) {
      boolean rc = true ;

      if ( listener != null )
         rc = listener.linkClicked ( e.getLink() ) ;

      return( rc ) ;
   }





   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set the status on the AppDefaultWin
    *  that this browser belongs to
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setStatus ( String status ) {
      if ( appDefaultWin == null )
         appDefaultWin = GUISystem.getParentDefWin ( this ) ;

      if ( appDefaultWin != null )
         appDefaultWin.setStatus ( status, false ) ;
   }


   public boolean statusChanged ( HTMLPaneStatusEvent e ) {

      switch ( e.getID() ) {
         
         case HTMLPaneStatus.APPLET_STATUS_MSG :
            break ;

         case HTMLPaneStatus.CONNECTING :
            setStatus ( Str.getStr ( STR_CONNECTING_TO ) ) ;
            break ;

         case HTMLPaneStatus.CONTENT_TYPE :
            break ;
         case HTMLPaneStatus.IDLE :
            break ;
         case HTMLPaneStatus.LAYINGOUT :
            //setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
            break ;
         case HTMLPaneStatus.LOADING_APPLET :
            setStatus ( Str.getStr ( STR_LOADING ) ) ;
            break ;
         case HTMLPaneStatus.LOADING_RESOURCES :
            //setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
            break ;
         case HTMLPaneStatus.PAGE_LOADED :
            if ( listener != null )
               listener.pageStopped () ;
            setStatus ( null ) ;
            break ;
         case HTMLPaneStatus.PARSING :
            break ;
         case HTMLPaneStatus.POPUPMENU_TRIGGER  :
            break ;

         default :
            setStatus ( null ) ;

      }
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* getSource
    */
   /**
    *    This method will return the source code
    *  of the page that is being displayed. If a null
    *  is returned, that means that the page is not yet loaded
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getSource () {
      String source       = null ;

      try {
         String tempFilename = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR )+File.separator+"temp_sf"+nextFileNum ;

         htmlPane.getSourceFile ( tempFilename )  ;
         source=FileUtil.readFile ( tempFilename ) ;
         nextFileNum++ ;

      } catch ( Exception e ) {
      }

      return( source) ;
   }

   public Component getComponent () {
      return ( this ) ;
   }

} /* HTMLPaneBrowser */
