package com.ibm.nzna.projects.qit.gui ;       
/*-----------------------------------------------------------*/
/*   NavListGroup                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NavListGroup                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This JPanel can hold many components and is expandable. This
 * means that the user can shrink the control to only show the
 * title, or expand it to show all contents. All contents are
 * shown 10 pixels to the right in relation to the title, so
 * to the user, it will look like a part of a tree structure
 *
 *
 * @version 1.0, 1/14/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NavListGroup                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class NavListGroup extends JPanel implements Runnable, ActionListener {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector          componentVec         = null ;
   private HotLinkLabel    st_TITLE             = null ;
   private NavList         navList              = null ;
   private boolean         expanded             = false ;
   private Dimension       minSize              = new Dimension ( 0, GUISystem.getRowHeight() ) ;
   private int             navListHeight        = 1 ;
   private Vector          expandListenerVec    = null ;
   private ImageIcon       collapsedIcon        = null ;
   private ImageIcon       expandedIcon         = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  the control
    *  
    *
    *  @param title The title of the NavListGroup
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavListGroup ( String title ) {
      initialize ( title, null ) ;
   }

   public NavListGroup ( String title, ImageIcon icon ) {
      initialize ( title, icon ) ;
   }

   private void initialize ( String title, ImageIcon icon ) {
      try {

         setBackground ( Color.white ) ;
         setOpaque ( false ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our controls                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( title != null ) && ( icon == null ) )
            st_TITLE = new HotLinkLabel ( title ) ;
         else if ( ( title == null ) && ( icon != null ) )
            st_TITLE = new HotLinkLabel ( icon ) ;
         else if ( ( title != null ) && ( icon != null ) )
            st_TITLE = new HotLinkLabel ( title, icon ) ;

         st_TITLE.showSelection ( false ) ;
         navList  = new NavList () ;
         this.collapsedIcon = icon ;
         this.expandedIcon  = icon ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set Properties                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setLayout ( null ) ;
         navList.setVisible ( false ) ;
         //navList.setOpaque ( isOpaque() ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add Listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         st_TITLE.addActionListener ( this );

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our controls                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         add ( st_TITLE, BorderLayout.NORTH ) ;
         add ( navList,  BorderLayout.CENTER ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all
    *  of our controls .
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout() {
      int       rowHeight = GUISystem.getRowHeight() ;
      Dimension size      = getSize () ;

      st_TITLE.setBounds ( 0, 0, size.width, rowHeight ) ;

      if ( ( expanded ) && ( navListHeight > 1 ) )
         navList.setBounds ( 18, rowHeight, size.width-15, size.height-rowHeight-1 ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an
    *  action event is fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == st_TITLE )
         if ( expanded )
            collapse () ;
         else
            expand() ;
   }





   /*------------------------------------------------------------*/
   /* expand
    */
   /**
    *    This method will either expand or
    *  contract the control
    *  
    *
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void expand ( ) {

      if ( navListHeight > 1 ) {


         if ( getParent() instanceof NavList ) {
            ((NavList)getParent()).scrollSmooth ( getLocation().y+minSize.height, 50-minSize.height ) ;
         }

         navList.setVisible ( true ) ;
         minSize.height = navListHeight+GUISystem.getRowHeight()  ;
         try {
            //((JComponent)getParent()).revalidate() ;
            invalidate() ;
         } catch ( Exception e ) {
            e.printStackTrace() ;
         }
      }

      expanded = true ;

      st_TITLE.setIcon ( expandedIcon ) ;

      if ( getParent() instanceof NavListGroup )
         ((NavListGroup)getParent()).revalidateHeight() ;

      if ( expandListenerVec != null )
         new Thread ( this ).start() ;

   }





   /*------------------------------------------------------------*/
   /* collapse
    */
   /**
    *    This method will collapse the control
    *  up to the title
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void collapse () {

      if ( navListHeight > 1 ) {
         expanded = false ;

         if ( getParent() instanceof NavList ) {
            ((NavList)getParent()).scrollSmooth ( getLocation().y+minSize.height, 0-minSize.height+GUISystem.getRowHeight() ) ;
         }

         minSize.height = GUISystem.getRowHeight() ;
         navList.setVisible ( false ) ;
         try {
            ((JComponent)getParent()).revalidate() ;
         } catch ( Exception e ) {
         }
      }

      st_TITLE.setIcon ( collapsedIcon ) ;

      if ( getParent() instanceof NavListGroup )
         ((NavListGroup)getParent()).revalidateHeight() ;

      if ( expandListenerVec != null )
         new Thread ( this ).start() ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set our test
    *  
    *  
    *
    *  @param test
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText ( String text ) {
      st_TITLE.setText ( text ) ;
   }



   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return our minimum size
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      return( getMinimumSize() ) ;
   }





   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return the minimum
    *  size of our control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      int rowHeight = GUISystem.getRowHeight() ;

      if ( ( navListHeight > 1 ) && ( expanded ) ) {
         minSize = navList.getPreferredSize() ;
         minSize.height+=rowHeight ;
      } else
         minSize.height=rowHeight ;
      return( minSize ) ;
   }





   /*------------------------------------------------------------*/
   /* add
    */
   /**
    *    This method will add a component
    *  to our group
    *  
    *
    *  @param component The component to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Component add ( Component component ) {
      navList.add ( component ) ;
      revalidateHeight() ;
      return( component ) ;
   }
   public void revalidateHeight() {
      int components = getComponentCount() ;
      int i          = 0 ;
      Component      component = null ;

      navListHeight=1 ;
      while ( i < components ) {
         component = getComponent(i) ;
         navListHeight+=component.getPreferredSize().height ;
         if ( ( navListHeight > 1 ) && ( expanded ) ) {

            minSize.height=GUISystem.getRowHeight()+navListHeight ;

            try {
               if ( getParent() instanceof NavList ) {
                  ((NavList)getParent()).scrollSmooth ( getLocation().y+minSize.height, 50-minSize.height ) ;
               }

               minSize.height = 50 ;
            } catch ( Exception e ) {
               minSize.height = 50 ;
               e.printStackTrace() ;
            }

         }
         i++ ;
      }
      invalidate() ;
      if ( getParent() != null )
         ((JComponent)getParent()).revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* setIcon
    */
   /**
    *    This method will set an icon in the
    *  title of the group
    *  
    *
    *  @param imageIcon The Icon to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setIcon ( ImageIcon imageIcon ) {
      st_TITLE.setIcon ( imageIcon ) ;
   }






   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener to
    *  each component in this group.
    *  
    *
    *  @param listener The ActionListener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      navList.addActionListener ( listener ) ;
   }



   /**
     *  This method will remove an action listener
     **/
   public void removeActionListener ( ActionListener listener ) {
      navList.removeActionListener ( this ) ;
   }


   /*------------------------------------------------------------*/
   /* getExpanded
    */
   /**
    *    This method will return TRUE if this group
    *  has been expanded to show it's components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getExpanded () {
      return( expanded ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread currently only notifies all of the
    *  expand listeners that the expansion/contraction
    *  is complete
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      notifyExpandListeners() ;
   }




   /*------------------------------------------------------------*/
   /* notifyExpandListeners
    */
   /**
    *    This method will notify all registered NavListExpandListeners
    *  that the expand/collapse is complete
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void notifyExpandListeners ( ) {
      if ( expandListenerVec != null ) {
         int i    = 0 ;
         int size = expandListenerVec.size() ;

         while ( i < size ) {
            ((NavListExpandListener)expandListenerVec.elementAt(i)).expanded ( this, expanded ) ;
            i++ ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* addExpandListener
    */
   /**
    *    This method will add a NavListExpandListener
    *  to be notified 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addExpandListener ( NavListExpandListener listener ) {
      if ( expandListenerVec == null )
         expandListenerVec = new Vector ( 1 ) ;

      expandListenerVec.addElement ( listener ) ;
   }




   /*------------------------------------------------------------*/
   /* removeExpandListener
    */
   /**
    *    This method will remove a NavListExpandListener
    *  from this control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeExpandListener ( NavListExpandListener listener ) {
      if ( expandListenerVec != null ) {
         expandListenerVec.removeElement ( listener ) ;
         if ( expandListenerVec.size() == 0 )
            expandListenerVec = null ;

      }
   }





   /*------------------------------------------------------------*/
   /* setExpandedIcon
    */
   /**
    *    This method will set the icon that is used
    *  in the expanded state
    *  
    *
    *  @param icon The icon to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setExpandedIcon ( ImageIcon icon ) {
      expandedIcon = icon ;
   }




   /*------------------------------------------------------------*/
   /* setCollapsedIcon
    */
   /**
    *    This method will set the default (collapsed)
    *  icon that appears
    *  
    *
    *  @param icon The icon to set to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setCollapsedIcon ( ImageIcon icon ) {
      collapsedIcon = icon ;
   }




   /*------------------------------------------------------------*/
   /* setData
    */
   /**
    *    This method will set a data object on this
    *  NavListGroup.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setData ( Object object ) {
      st_TITLE.setData ( object ) ;
   }




   /*------------------------------------------------------------*/
   /* getData
    */
   /**
    *    This method will return the data that is
    *  associated to this NavListGroup. The data is
    *  set by 'setData()' method
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Object
    *//*---------------------------------------------------------*/
   public Object getData () {
      return( st_TITLE.getData() ) ;
   }
} /* NavListGroup */
