package com.ibm.nzna.projects.qit.gui ;  
/*-----------------------------------------------------------*/
/*   NavPanel                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NavPanel                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
/**
 * This interface defines the methods that are needed in the
 * preparation, manipulation and closing of a navigation panel.
 * The preparation of a NavPanel includes calling two methods:
 *
 * initialize
 * start
 *
 * The manipulation occurs when the user sizes the nav bar
 * or pops the nav bar out of the ApplicationPanel it is
 * residing on. A call to the method 'popout' will be called
 * when the panel has succesfully been added to an external
 * frame.
 *
 * The closing of the nav panel calls this method:
 * stop
 *
 *
 * @version 1.0, 1/12/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NavPanel                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public interface NavPanel {





    /*------------------------------------------------------------*/
    /* initialize
     */
    /**
     *    This method should do any preparation
     *  work before the panel is shown
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public abstract void initialize () ;




    /*------------------------------------------------------------*/
    /* start
     */
    /**
     *    This method will be called after initialization
     *  and the panel is showing
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public abstract void start () ;





    /*------------------------------------------------------------*/
    /* stop
     */
    /**
     *    This method will be called once the
     *  panel is hidden and it is ready to 
     *  be destroyed
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public abstract void stop() ;





    /*------------------------------------------------------------*/
    /* getTitle
     */
    /**
     *    This method should return the User 
     *  displayable title of the NavPanel
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public abstract String getTitle () ;






    /*------------------------------------------------------------*/
    /* getJComponent
     */
    /**
     *    This method should return the JComponent
     *  that represents this NavPanel
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return JComponent
     *//*---------------------------------------------------------*/
    public abstract JComponent getJComponent() ;

} /* NavPanel */
