package com.ibm.nzna.projects.qit.gui ;        
/*-----------------------------------------------------------*/
/*   NavPanelHeader                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NavPanelHeader                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;   
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This class subclasses JPanel so that it can provide a title,
 * pop out button and close button for Nav Panels. An action listener
 * can be added to this panel to be notified if the Pop Out or Close
 * button is pressed. The Pop Out button has an Action Command of NAV_POPOUT
 * and the Close button has an Action Command of NAV_CLOSE. This panel does
 * nothing when the buttons are pressed.
 *
 * @version 1.0, 1/12/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NavPanelHeader                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class NavPanelHeader extends JPanel implements PropertyListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Dimension  minSize              = new Dimension ( 200, 19 ) ;
   private static Dimension  titleMinSize         = new Dimension ( 2000, 19 ) ;
   private static int        buttonWidth          = 25 ;
   private static Insets     insets               = new Insets ( 2, 2, 2, 2 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel     st_TITLE             = null ;
   private DButton    pb_CLOSE             = null ;
   private DButton    pb_POPOUT            = null ;
   private JComponent navPanel             = null ;
   private boolean    showSkin             = false ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will initialize and create
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanelHeader () {     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TITLE  = new JLabel ("") ;
      pb_CLOSE  = new DButton ( ImageSystem.getImageIcon ( this, ImageSystem.SMALLCLOSE ) ) ;
      pb_POPOUT = new DButton ( ImageSystem.getImageIcon ( this, ImageSystem.POPOUT ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.setActionCommand   ( "NAV_CLOSE" ) ;
      pb_CLOSE.setBorderRollover  ( true ) ;
      pb_POPOUT.setBorderRollover ( true ) ;
      pb_POPOUT.setActionCommand  ( "NAV_POPOUT" ) ;
      st_TITLE.setMinimumSize     ( titleMinSize ) ;                            
      st_TITLE.setHorizontalAlignment ( SwingConstants.CENTER ) ;
      st_TITLE.setVerticalAlignment   ( SwingConstants.BOTTOM ) ;        
      st_TITLE.setOpaque ( false ) ;
      pb_CLOSE.setOpaque ( false ) ;
      pb_POPOUT.setOpaque ( false ) ;
      propertyChanged ( PropertySystem.PROPERTY_SKINNAME, null ) ;
      propertyChanged ( PropertySystem.PROPERTY_SHOWSKIN, null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our listeners                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;

      showSkin = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our layout and add our components                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout ( 0, 0 ) ) ;//null ) ;
      //add ( pb_CLOSE   , BorderLayout.EAST) ;
      add ( st_TITLE   , BorderLayout.CENTER) ;
   }



   /*------------------------------------------------------------*/
   /* getMinimumSize
    */
   /**
    *    This method will return our minimum
    *  size
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getMinimumSize () {
      return( minSize ) ;
   }
   public Dimension getPreferredSize () {
      return( minSize ) ;
   }


   /*------------------------------------------------------------*/
   /* setNavPanel
    */
   /**
    *    This method will set what panel this
    *  NavPanelHeader is really header for
    *  
    *
    *  @param JComponent The component
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setNavPanel ( JComponent comp ) {
      this.navPanel = comp ;
   }





   /*------------------------------------------------------------*/
   /* addActionListener
    */
   /**
    *    This method will add an action listener
    *  for both the close and pop out
    *  buttons
    *
    *  @param listener The listener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionListener ( ActionListener listener ) {
      pb_POPOUT.addActionListener ( listener ) ;
      pb_CLOSE.addActionListener ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set the title that
    *  appears between the pop out and close
    *  buttons
    *
    *  @param title The new title
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitle ( String title ) {
      st_TITLE.setText ( title ) ;
   }




   /*------------------------------------------------------------*/
   /* free
    */
   /**
    *    This method should be called when this
    *  object is no longer needed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void free () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SHOWSKIN, this ) ;
   }






   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  has changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object newValue ) {

      try {
         switch ( property ) {
            case PropertySystem.PROPERTY_SHOWSKIN :
               showSkin = PropertySystem.getBool ( PropertySystem.PROPERTY_SHOWSKIN ) ;
               invalidate() ;
               break ;

            case PropertySystem.PROPERTY_SKINNAME :
               Color bg       = ThemeSystem.getColor ( "NAVPANELTITLE_BG_COLOR" ) ;
               Color fg       = ThemeSystem.getColor ( "NAVPANELTITLE_FG_COLOR" ) ;


               if ( bg != null ) {
                  setOpaque ( true ) ;
                  setBackground ( bg ) ;
               } else 
                  setOpaque ( false ) ;

               if ( fg != null ) 
                  setForeground ( fg ) ;
               else
                  setForeground ( Color.black ) ;

               break ;
         }

      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      Dimension size = getSize() ;

      super.paintComponent ( g ) ;

      //if ( showSkin )
      //   WinUtil.wallPaper ( this, g, ImageSystem.getImage ( this, ImageSystem.SAVE ) ) ;
   }




   /*------------------------------------------------------------*/
   /* setForeground
    */
   /**
    *    Subclassed to set the foreground of the JLabel
    *  holding our title
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setForeground ( Color color ) {
      if ( ( color != null ) && ( st_TITLE != null ) ) {
         st_TITLE.setForeground ( color ) ;
      }
   }
} /* NavPanelHeader */
