package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   OfferingEntryPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: OfferingEntryPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2001                                  */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.qit.product.* ;

/**
 * This panel holds 2 controls. The EntryField and Select Offering Button. 
 * It can be used to allow the user to enter offering description. It will auto-populate
 * the entry field if the user stops typing for 1 second.             
 *
 * @version 1.0, 08/3/2001 
 * @author  Venu Sanka
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   OfferingEntryPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class OfferingEntryPanel extends JPanel implements ActionListener, Runnable, AppConst, ProductConst,
     DataLengths {

    /**
     *  This is the name of the thread that will search for offerings
     **/
   private static final String THREAD_SEARCH = "SE" ;

   /**
     *  This is the name of the thread that will fire events
     **/
   private static final String THREAD_FIRE_EVENT = "FE" ;

   
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   
   private JTextField ef_OFFER          = new JTextField() ;
   private DButton    pb_SELECTOFFER    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This vector holds all registered listeners
     **/
   private Vector listeners = new Vector ( 1 ) ;


   /**
     *  This is the property which holds the last offering searched for.
     **/
   private String  lastOfferingSearch = "" ;

   /**
     *  If TRUE, the control is currently searching for UserRec objects
     **/
   private boolean inSearch       = false ;

   /**
     *  This is the vector of offerings that can be selected
     **/
   private Vector  offeringVec        = null ;

   /**
     *  This is the currently selected offering record
     **/
   private TypeOfferingRec  offeringRec        = null ;

   /**
     *  If TRUE, this property tells the control to show the user's full name, not just the user id
     **/
   private boolean showFullName   = true ;

   /**
     *  This property tells the running thread to continue
     **/
   private boolean continueThread = true ;

   /**
     *  This is a temporary property to hold the current user
     **/
   private String           curTempOffering     = null ;

   /**
     *  This property tells the control to show managers only
     **/
   private boolean showManagerOnly    = false ;


      
   
   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the Offering Panel with
    *  the default description of 'Offering'
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public OfferingEntryPanel () {
      init() ;
   }


   /*-----------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void init () {
      createControls () ;
      offeringVec = (Vector)TypeList.getInstance().getTypeList( TypeList.TYPE_OFFERING).clone() ;
      new Thread(this, THREAD_SEARCH ).start() ;

   }

   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our controls
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void createControls () {

      setLayout ( null ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECTOFFER = new DButton ( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_PERSONEL ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      
      //setBorder ( GUISystem.loweredBorder ) ;
      pb_SELECTOFFER.setFocusPainted ( false ) ;
      //pb_SELECTOFFER.setFocusable ( false ) ;
      pb_SELECTOFFER.setBorderPainted ( false ) ;
      pb_SELECTOFFER.setBackground ( Color.white ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECTOFFER.addActionListener ( this ) ;
      ef_OFFER.addActionListener( this ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      
      setLayout ( new BorderLayout () ) ;
      add ( ef_OFFER, BorderLayout.CENTER ) ;
      add ( pb_SELECTOFFER, BorderLayout.EAST ) ;

   }


   /*-----------------------------------------------------------*/
   /* getText
    */
   /**
    *    This method will simply return whatever is
    *  in the Offering Entry Field. It is not guarenteed
    *  what is in the field
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public String getText() {
      return( ef_OFFER.getText() ) ;
   }


   /*-----------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of the entry field
    *  
    *  
    *             
    *  @param text The text to set
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setText ( String text ) {
      if ( ( text != null ) && ( text.length() > 0 ) ) {
         curTempOffering = text ;
         offeringRec = findOffering ( text, true ) ;
         refreshOfferingField() ;
      } else
         ef_OFFER.setText ( "" ) ;

   }


   /*-----------------------------------------------------------*/
   /* setShowFullName
    */
   /**
    *    This method will tell the panel to show the user's full name,
    *  If showFullName is TRUE, the panel will
    *  display the offering's full name.
    *  
    *             
    *  @param onTop TRUE = Show FullName FALSE = Only show user id
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setShowFullName ( boolean showFullName ) {
      this.showFullName = showFullName ;

      refreshOfferingField() ;

   }


   /*-----------------------------------------------------------*/
   /* getShowFullName
    */
   /**
    *    This method will the ShowFullName property     
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public boolean getShowFullName () {
      return( showFullName ) ;
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform our thread operations
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
       String operation = Thread.currentThread().getName() ;

       if ( operation.equals (THREAD_SEARCH ))
           search() ;
       else if ( operation.equals ( THREAD_FIRE_EVENT ))
           fireEvent() ;
   }


   /**
    *  This method will search for offerings
    *
    */
   private void search () {
   
      String curOffering  = null ;
      String curOffering1 = null ;

      while ( continueThread ) {

         /* Ok, first sleep for 1 1/2 second and then check the
            entry field's text */
         try {
            Thread.sleep ( 1500 ) ;
         } catch ( Exception e ) {
         }
         curOffering = ef_OFFER.getText() ;

         /* Ok, if both curOffer and lastOfferSearch are not null
            and do not equal each other, perform a search */
         if ( ( curOffering != null ) &&
              ( curOffering.length() > 0 ) && 
              ( lastOfferingSearch != null ) ) {

            /* Ok, now wait for a bit longer and check it again. If the
               text has changed, loop back up. Otherwise look for the
               name */
            try {
               Thread.sleep ( 300 ) ;
            } catch ( Exception e ) {
            }
            curOffering1 = ef_OFFER.getText() ;


            if ( curOffering1.equals ( curOffering ) ) {

               if ( !curOffering1.equals ( lastOfferingSearch ) ) {
                  lastOfferingSearch = curOffering1 ;

                  offeringRec = findOffering ( curOffering1 ) ;

                  /* Ok, if we found a record, populate it now */
                  refreshOfferingField() ;

                  if ( offeringRec != null )
                      new Thread ( this, THREAD_FIRE_EVENT ).start() ;
               } /* if */

            } /* if */
         } else
            offeringRec = null ;
      } /* while */
   } /* search */




   /*------------------------------------------------------------*/
   /* findOffering
    */
   /**
    *    This method will attempt to locate a user
    *  doing a fuzzy search
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private TypeOfferingRec findOffering ( String offering ) {
      return( findOffering ( offering, false ) ) ;
   }




   /*------------------------------------------------------------*/
   /* findOffering
    */
   /**
    *    This method will attempt to locate the offering
    *  in the application offering list. If a match is found
    *  then this method will return the offering record.
    *
    *  @param offering  offering  description
    *  @param exactMatch TRUE = only hit on exact matches
    *  @param None
    *
    *  @return TypeOfferingRec of the offering found, NULL if not found
    *//*---------------------------------------------------------*/
   private TypeOfferingRec findOffering ( String offering, boolean exactMatch ) {
      TypeOfferingRec retRec = null ;

      if ( ( offering != null ) && ( offeringVec != null ) ) 
          {
            int     i              = 0 ;
            int     length         = offeringVec.size() ;
            TypeOfferingRec    compareOfferingRec = null ;
            Vector  foundVec       = new Vector ( 1, 10 ) ;

            /* Always upper case! */
            offering = offering.toUpperCase() ;                    

            /* Ok, walk through the vector and look for the 'offering' in the description field */
            while ( i < length ) 
                {
                    compareOfferingRec = (TypeOfferingRec) offeringVec.elementAt(i) ;

                if ( compareOfferingRec != null ) 
                    {

                        if ( exactMatch ) 
                            {
                                if ( compareOfferingRec.toString().toUpperCase().equals ( offering ) )
                                foundVec.addElement ( compareOfferingRec ) ;

                  //else if ( compareUserRec.getName().toUpperCase().equals ( user ) )
                  //   foundVec.addElement ( compareUserRec ) ;
                            } 
                        else {
                                if ( compareOfferingRec.toString().toUpperCase().startsWith ( offering ) )
                                foundVec.addElement ( compareOfferingRec ) ;

                  //else if ( compareUserRec.getName().toUpperCase().startsWith ( user ) ) {
                  //   foundVec.addElement ( compareUserRec ) ;
                            //}
               }
            } /* if */

            i++ ;
         } /* while */

         foundVec.trimToSize() ;

         /* Ok, if we have more than 1 hit, let the user select, otherwise auto-populate the field */
         if ( foundVec.size() > 1 ) 
             {
                Vector vec = null ;


                vec = new SelectOfferingDlg ( GUISystem.getParentFrame(this), foundVec, false ).getResult() ;

                if ( ( vec != null ) && ( vec.size() > 0 ) )
                    compareOfferingRec = (TypeOfferingRec) vec.elementAt(0) ;

                    retRec = this.offeringRec = compareOfferingRec ;

                    refreshOfferingField() ;

             } 
         else if ( foundVec.size() == 1 )
            retRec = (TypeOfferingRec) foundVec.elementAt(0) ;


      } /* if */

      return( retRec ) ;
   } /* findUser */













   /*------------------------------------------------------------*/
   /* refreshOfferingField
    */
   /**
    *    This method will refresh the data that is
    *  in the offering field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshOfferingField() {

      if ( offeringRec != null ) {

         if ( showFullName ) 
            ef_OFFER.setText ( offeringRec.toString() ) ;
         //else
         //   ef_USERID.setText ( userRec.getUserId() ) ;

         try {
            ef_OFFER.setSelectionStart ( lastOfferingSearch.length() ) ;
            ef_OFFER.setSelectionEnd ( ef_OFFER.getText().length() ) ;
         } catch ( Exception e ) {}

         lastOfferingSearch = ef_OFFER.getText() ;

         } /* if */

      else
         ef_OFFER.setText ( "" ) ;
      
   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one of our
    *  buttons is pressed
    *  
    *
    *  @param e The actionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {     

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   DButton Events                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() instanceof DButton ) {
            DButton button = (DButton) e.getSource() ;
            if ( pb_SELECTOFFER == button ) {
               Vector vec = null ;

               vec = new SelectOfferingDlg ( GUISystem.getParentFrame(this), false ).getResult() ;
               if ( ( vec != null ) && ( vec.size() > 0 ) )
                  offeringRec = (TypeOfferingRec) vec.elementAt(0) ;
               refreshOfferingField() ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   TextField Events                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() instanceof JTextField ) {
            String text = ef_OFFER.getText() ;

            offeringRec = findOffering ( text ) ;
            refreshOfferingField() ;
         }
      } catch ( Exception ex ) {
         LogSystem.log ( 1, ex) ;
      }
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will stop our thread that is monitoring
    *  the entry field. It must be called when you are through with
    *  this control.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      continueThread = false ;
   }



   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name if it is valid.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return user The name if valid, NULL if not valid
    *//*---------------------------------------------------------*/
   public String getName () {
      String retStr = null ;

      if ( offeringRec != null )
         retStr = offeringRec.toString() ;

      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* getOfferingRec
    */
   /**
    *    This method will return the offering record
    *  if it is valid, If not valid, a lookup will be performed
    *  asking the user to select a description
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public TypeOfferingRec getOfferingRec () {
      if ( offeringRec == null )
         offeringRec = findOffering ( ef_OFFER.getText() ) ;

      return( offeringRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getOfferingRecNoLookup
    */
   /**
    *    This method will return the offeringRec selected
    *  by the user or nothing. It will not perform
    *  a second lookup
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public TypeOfferingRec getOfferingRecNoLookup () {

      return( offeringRec ) ;
   }


   /*------------------------------------------------------------*/
   /* getDescript
    */
   /**
    *    This method will return the offering description if it is valid.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return offering The Offering description if valid, NULL if not valid
    *//*---------------------------------------------------------*/
   public String getDescript () {
      String retStr = null ;

      if ( offeringRec != null )
         retStr = offeringRec.toString() ;

      return( retStr ) ;
   }



   /*------------------------------------------------------------*/
   /* requestFocus
    */
   /**
    *    This method has been subclassed to pass the
    *  focus to our entry field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void requestFocus () {
      ef_OFFER.requestFocus() ;
   }





   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable
    *  our component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enable ) {
      ef_OFFER.setEnabled ( enable ) ;
      pb_SELECTOFFER.setEnabled ( enable ) ;
      super.setEnabled ( enable ) ;
   }


   /**
    *  This method will fire a 'userSelected' event to all registered listeners
    */
   private void fireEvent () {
       int i = 0 ;
       int size = listeners.size() ;

       while ( i < size ) {
           ((OfferingEntryPanelListener)listeners.elementAt(i)).offeringSelected( offeringRec ) ;
           i++  ;
       }
   }



   public void addOfferingEntryPanelListener ( OfferingEntryPanelListener listener )
   {
       listeners.addElement( listener ) ;
   }


   public void removeOfferingEntryPanelListener ( OfferingEntryPanelListener listener ) {
       listeners.removeElement( listener ) ;
   }


} /* OfferingEntryPanel */

