package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   PrevNextPanel                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PrevNextPanel                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;                   
import javax.swing.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This panel simply holds the previous and next buttons that you see on the ContainerWin. 
 *
 *
 * @version 1.0, 6/16/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PrevNextPanel                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class PrevNextPanel extends JPanel implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controsl                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton   pb_NEXT      = new DButton ( Str.getStr ( STR_NEXT ) ) ;
   private DButton   pb_PREV      = new DButton ( Str.getStr ( STR_PREVIOUS ) ) ;
   private Dimension size         = new Dimension ( 0, 18 ) ;
   private boolean   showNext     = false ;
   private boolean   showPrev     = false ;
   private Insets    buttonInsets = new Insets ( 0, 0, 0, 0 ) ;

   public PrevNextPanel () {
      init() ;
      pb_NEXT.setVisible ( showNext ) ;
      pb_PREV.setVisible ( showPrev ) ;
      showNext = false ;
      showPrev = false ;
   }

   public PrevNextPanel ( ActionListener listener ) {
      init() ;
      if ( listener != null ) {
         pb_NEXT.addActionListener ( listener ) ;
         pb_PREV.addActionListener ( listener ) ;
      }
      pb_NEXT.setFocusPainted ( false ) ;
      pb_PREV.setFocusPainted ( false ) ;
   }




   /*-----------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize the panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void init () {
      setLayout ( null ) ;
      pb_NEXT.setMargin ( buttonInsets ) ;
      pb_PREV.setMargin ( buttonInsets ) ;
      add ( pb_NEXT ) ;
      add ( pb_PREV ) ;
      setOpaque ( false ) ;
   }


   /*-----------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred size of
    *  this component. The size depends on which
    *  buttons are being shown
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize () {
      if ( ( showPrev ) && ( showNext ) ) {
         size.width = 95 ;
         size.height = GUISystem.getRowHeight() ;
      } else if ( ( ( !showPrev ) && ( showNext ) ) || 
                  ( ( showPrev ) && ( !showNext ) ) ) {
         size.width = 55 ;
         size.height=GUISystem.getRowHeight() ;
      } else {
         size.width = 0 ;
         size.height=0 ;
      }

      return( size ) ;
   }

   public Dimension getMinimumSize() {
      return( getPreferredSize() ) ;
   }

   public Dimension getMaximumSize() {
      return( getPreferredSize() ) ;
   }




   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will layout all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      try {
         size.height = getSize().height ;

         super.doLayout() ;

         if ( ( showPrev ) && ( showNext ) ) {
            pb_PREV.setBounds ( 0, 1, 40, size.height-1 ) ;
            pb_NEXT.setBounds ( 40, 1, 40, size.height-1 ) ;
         } else if ( showNext ) {
            pb_NEXT.setBounds ( 0, 1, 40, size.height-1 ) ;
            pb_NEXT.paint ( pb_NEXT.getGraphics() ) ;
         } else if ( showPrev ) {
            pb_PREV.setBounds ( 0, 1, 40, size.height-1 ) ;
            pb_PREV.paint ( pb_PREV.getGraphics() ) ;
         }

      } catch ( Exception e ) {
      }
   }



   /*-----------------------------------------------------------*/
   /* paint
    */
   /**
    *    This method will paint our window
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paintComponent ( Graphics g ) {
      super.paintComponent ( g ) ;
      g.setColor ( Color.black ) ;
      g.drawLine ( 0, 0, size.width-1, 0 ) ;
      g.drawLine ( size.width-1, 0, size.width-1, size.height ) ;
   }



   /*-----------------------------------------------------------*/
   /* showNext
    */
   /**
    *    This method will either turn the Next button on
    *  or off.
    *  
    *
    *  @param showNext TRUE = Show Next Button
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showNext ( boolean showNext ) {
      this.showNext = showNext ;
      pb_NEXT.setVisible ( showNext ) ;
      //getParent().doLayout() ;
      invalidate() ;
      doLayout() ;
   }

   /*-----------------------------------------------------------*/
   /* showPrev
    */
   /**
    *    This method will either turn the Prev button on
    *  or off.
    *  
    *
    *  @param showPrev TRUE = Show Prev Button
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void showPrev ( boolean showPrev ) {
      this.showPrev = showPrev ;
      pb_PREV.setVisible ( showPrev ) ;
      //getParent().doLayout() ;
      invalidate() ;
      doLayout() ;
   }



   /*-----------------------------------------------------------*/
   /* nextShowing
    */
   /**
    *    This method will return TRUE if the
    *  next button is showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean nextShowing() {
      return( showNext) ;
   }




   /*------------------------------------------------------------*/
   /* getNextButton
    */
   /**
    *    This method will return the DButton instance
    *  of our Next button
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DButton getNextButton ( ) {
      return( pb_NEXT ) ;
   }




   /*------------------------------------------------------------*/
   /* getPrevButton
    */
   /**
    *    This method will return the DButton instance
    *  of our Prev button
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public DButton getPrevButton () {
      return( pb_PREV ) ;
   }




} /* PrevNextPanel */
