package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   PreviewBrowser                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PreviewBrowser                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This subclass of JPanel houses a JScrollPane and JLabel. The
 * JLabel can display HTML Text as long as the text set in the 'setText'
 * method begins with '<html>' and ends with '<.html>'
 *
 *
 * @version 1.0, 7/25/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.io.* ;
import javax.swing.* ; 
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   PreviewBrowser                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class PreviewBrowser extends JPanel implements AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESH      = "REFRESH" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String  statusText        = Str.getStr ( STR_PLEASE_WAIT ) ;
   private String  emptyText         = "" ;
   private String  currentText       = "" ;
   private boolean optimizeScroll    = true ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextPane   st_PREVIEW     = null ;
   private JScrollPane pnl_SCROLLPANE = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the default PreviewBrowser
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PreviewBrowser () {
      init() ;
   }




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the default
    *  pane and will show the 'statusText' when
    *  formatting html
    *
    *  @param statusText - The text to show when formatting the HTML
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PreviewBrowser ( String statusText ) {
      this.statusText = statusText ;
      init() ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create the default
    *  pane and will show the 'statusText' when
    *  formatting html. It will also show the 'emptyText' when
    *  the preview pane is empty.
    *
    *  @param statusText - The text to show when formatting the HTML
    *  @param emptyText  - The text to show when the field is empty
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PreviewBrowser ( String statusText, String emptyText ) {
      this.statusText = statusText ;
      this.emptyText  = emptyText ;
      init() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create the default preview 
    *  browser with or without optimizations for
    *  scrolling
    *
    *  @param optimizeScroll TRUE = optimize, FALSE = do not
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PreviewBrowser ( boolean optimizeScroll ) {
      this.optimizeScroll = optimizeScroll; 
      init() ;
   }




   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our PreviewBrowser
    *  
    *  
    *
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void init () {

      /* Create our GUI  */
      st_PREVIEW     = new JTextPane (  ) ;
      pnl_SCROLLPANE = new JScrollPane ( st_PREVIEW ) ;

      /* Adjust some properties  */
      setBackground ( Color.white ) ;
      st_PREVIEW.setBackground ( Color.white ) ;
      st_PREVIEW.setEditable ( false );
      st_PREVIEW.setFont ( new Font ( "SansSerif" , Font.PLAIN, 12 ) ) ;
      pnl_SCROLLPANE.setBackground ( Color.white ) ;
      pnl_SCROLLPANE.getViewport().setBackground ( Color.white ) ;
      pnl_SCROLLPANE.getViewport().setBorder ( null ) ;
      pnl_SCROLLPANE.setBorder ( null ) ;
      if ( optimizeScroll )
         pnl_SCROLLPANE.getViewport().putClientProperty ("EnableWindowBlit", Boolean.TRUE);
      st_PREVIEW.setBorder ( null ) ;
      pnl_SCROLLPANE.setBackground ( Color.white ) ;

      setLayout ( new BorderLayout() ) ;
      add ( pnl_SCROLLPANE, BorderLayout.CENTER ) ;
   }





   /*------------------------------------------------------------*/
   /* setPage
    */
   /**
    *    This method will set the text of the control to an HTML
    *  page
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPage ( String text ) {
      if ( ( text == null ) || ( text.length() == 0 ) ) {
         text = emptyText ;
         st_PREVIEW.setText ( text ) ;
      }

      else {
         currentText = text ;
         new Thread ( this, THREAD_REFRESH ).start() ; ;
      }

   }


   public void setText ( String text ) {
      st_PREVIEW.setText ( text ) ;
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will refresh our preview with the
    *  text passed into setText
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      try {
         String tempDir      = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;
         String tempFilename = tempDir+File.separator+DateSystem.getDate(CDate.DATE_STAMP)+".html" ;

         FileUtil.saveFile ( tempFilename, currentText ) ;
         st_PREVIEW.setText ( statusText ) ;
         st_PREVIEW.setPage ( "file:/"+tempFilename ) ;
         revalidate() ;
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* dispose
    */
   /**
    *    This method will detroy everything and return
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void dispose () {
      try {
         pnl_SCROLLPANE.remove ( st_PREVIEW ) ;
         remove ( pnl_SCROLLPANE ) ;
         st_PREVIEW = null ;
         pnl_SCROLLPANE = null ;
         getParent().remove ( this ) ;
         super.finalize() ;
      } catch ( Throwable e ) {
      }
   }




   /*------------------------------------------------------------*/
   /* addHyperlinkListener
    */
   /**
    *    This method will add a hyper link listener to
    *  the JTextPane that is showing the preview
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addHyperlinkListener ( HyperlinkListener listener ) {
      st_PREVIEW.addHyperlinkListener ( listener ) ;
   }
} /* PreviewBrowser */

