package com.ibm.nzna.projects.qit.gui ;    
/*-----------------------------------------------------------*/
/*   QuestPanel                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestPanel                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.Component ;
/**
 * This interface defines the methods and static variables for
 * the QuestPanel. The QuestPanel Interface must be implemented by any
 * Panel wishing to show and be managed inside of Quest. This
 * does not include subpanels (Panels inside other panels). For
 * example, a QuestPanel would be the Workspace, but
 * any panel inside of the Workspace would not implement QuestPanel.
 *
 *
 * @version 1.0, 1/12/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestPanel                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public interface QuestPanel {



   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method should return an ImageIcon
    *  representing this panel. This icon will
    *  be used as a Frame icon, or Tab icon
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract ImageIcon  getIcon() ;




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method should return the name
    *  of the panel. This name should be user
    *  readable as it will be in the titlebar of frames and tabs
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public abstract String     getName() ;




   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method should return an instance
    *  of the default navigation panel for this
    *  panel. NULLs are allowed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent - An instance of the Navigation panel
    *//*---------------------------------------------------------*/
   public abstract NavPanel getDefaultNavPanel() ;




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method should initialize the panel
    *  but not show anything. This method is
    *  called before 'start', so any prep work should be done here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE - Continue initialization process, FALSE - Stop initialization and kill instance
    *//*---------------------------------------------------------*/
   public abstract boolean    initialize() ;




   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called when the
    *  PanelSystem has just shown the panel
    *  Any focus setting or data refreshes should be done here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void start() ;




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called to let the
    *  panel know that the PanelSystem needs
    *  to close it. This method should return FALSE if it wishes not to close
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE - Go ahead and close the panel, FALSE - Do not close the panel (I'm not ready to die)
    *//*---------------------------------------------------------*/
   public abstract boolean    stop() ;




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called on every
    *  open panel upon the user shutting down
    *  the application.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Go ahead and call 'stop', I am ready to close. FALSE = Stop shutdown process and do not close
    *//*---------------------------------------------------------*/
   public abstract boolean    shutdown() ;




   /*------------------------------------------------------------*/
   /* enable
    */
   /**
    *    This method will be called by the PanelSystem
    *  to let a panel know if it should accept
    *  user input or disregard it. Be careful with this method, if
    *  you call 'setEnabled' on all of your controls in the panel, there
    *  will be a second to two second pause while repainting everything. A
    *  good programmer should mark this as a flag, and when any user interaction
    *  has happened, check that flag to determine if you should allow
    *  user interaction.
    *
    *  @param enable TRUE = allow user input, FALSE = do not allow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void enable ( boolean enable ) ;





   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method should return the JComponent
    *  representing this QuestPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public abstract JComponent getJComponent () ;





   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Cut As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public abstract boolean cutText () ;

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Paste As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public abstract boolean pasteText () ; 

   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Copy As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public abstract boolean copyText () ;





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called when the user
    *  wishes to refresh the QuestPanel. DO NOT EVER CALL revalidate
    *  in this method, the default operation for this button is to
    *  have the AppDefaultWin holding the QuestPanel to repaint everything
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public abstract void refresh () ;



   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the
    *  user has selected our panel from a
    *  tabbed pane. If we are in MDI mode, this method is never called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () ;





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel onto another. This method
    *  will never be called in the MDI mode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () ;





   /*------------------------------------------------------------*/
   /* equals
    */
   /**
    *    This method should determine if this QuestPanel
    *  equals the Object being passed in.
    *  
    *
    *  @param compareRec The object to compare this QuestPanel with.
    *  @param None
    *  @param None
    *
    *  @return TRUE if Equal, FALSE if not
    *//*---------------------------------------------------------*/
   public abstract boolean equals ( Object compareRec ) ;





   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public abstract boolean bookmark  () ;

} /* QuestPanel */
