package com.ibm.nzna.projects.qit.gui ;                                         
/*-----------------------------------------------------------*/
/*   QuestPanelInitializeThread                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: QuestPanelInitializeThread          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The QuestPanelInitializeThread will completely create an ApplicationPanel from a QuestPanel,  
 * including calling all of the initialize methods of the QuestPanel. After the ApplicationPanel
 * has been created, this thread will call the AppDefaultWin who created the thread with the
 * completed ApplicationPanel.
 *
 * @version 1.0, 3/5/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   QuestPanelInitializeThread                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class QuestPanelInitializeThread extends Thread {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private AppDefaultWin    appDefaultWin    = null ;
   private QuestPanel       questPanel       = null ;
   private ApplicationPanel applicationPanel = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and kick off the
    *  initialize thread
    *  
    *
    *  @param AppDefaultWin The appDefaultWin that will hold the panel once created
    *  @param questPanel The QuestPanel to create
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestPanelInitializeThread ( AppDefaultWin appDefaultWin, QuestPanel questPanel ) {
      this.appDefaultWin = appDefaultWin ;
      this.questPanel    = questPanel ;

      new Thread ( this ).start() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and kick off the initialize thread if
    *  the second parameter is true
    *  
    *
    *  @param applicationPanel The ApplicationPanel to initialize with the QuestPanel
    *  @param questPanel
    *  @param kickOff TRUE = start thread, FALSE = do not start
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public QuestPanelInitializeThread ( ApplicationPanel applicationPanel, QuestPanel questPanel, boolean kickOff ) {
      this.questPanel       = questPanel ;
      this.applicationPanel = applicationPanel ;
      if ( kickOff )
         new Thread ( this ).start() ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will actually run the thread
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      try {
         if ( questPanel != null ) {
            NavPanel         navPanel = null ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Initialize the controls                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( questPanel.initialize() ) {
               navPanel = questPanel.getDefaultNavPanel() ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the controls to the panel                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( applicationPanel == null )
                  applicationPanel = new ApplicationPanel () ;
               applicationPanel.setContentPanel ( questPanel ) ;
               applicationPanel.setNavPanel ( navPanel ) ;                                                           

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Tell the appDefaultWin that this panel is ready to go!  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( appDefaultWin != null )
                  appDefaultWin.addPanel ( applicationPanel, questPanel.getName() ) ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Start the controls                                      */
               /*                                                           */
               /*-----------------------------------------------------------*/
               questPanel.start() ;
               if ( navPanel != null )
                  navPanel.start() ;
            }
         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

   }





   /*------------------------------------------------------------*/
   /* getApplicationPanel
    */
   /**
    *    This method will return the application panel
    *  that was created during the initialization process
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ApplicationPanel
    *//*---------------------------------------------------------*/
   public ApplicationPanel getApplicationPanel () {
      return( applicationPanel ) ;
   }

} /* QuestPanelInitializeThread */
