package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   SelectWarrantyDlg                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectWarrantyDlg                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * Allows the user to select a warranty  
 *
 *
 * @version 1.0, 08/03/2001 
 * @author  Venu Sanka
 */
import java.awt.* ;
import java.util.Vector ;
import javax.swing.* ;
import javax.swing.border.TitledBorder ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.qit.product.* ;



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectWarrantyDlg                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectWarrantyDlg extends JDialog implements ActionListener, AppConst, ProductConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_DESCRIPT      = new JLabel    ( Str.getStr ( STR_PLEASE_SELECT_A_WARRANTY ) ) ;
   private MultiList    warrantyList     = new MultiList ( GUISystem.getFontUtil() ) ;
   private JPanel       pnl_WARRANTYLIST = new JPanel    ( ) ;
   private JTitle       st_TITLE         = new JTitle    ( Str.getStr ( STR_WARRANTY ) ) ;
   private DButton      pb_SELECT        = new DButton   ( Str.getStr ( STR_SELECT ) ) ;
   private DButton      pb_CANCEL        = new DButton   ( Str.getStr ( STR_CANCEL ) ) ;
   private DButton      pb_SEARCH        = new DButton   ( Str.getStr ( STR_FIND ) ) ;
   private JPanel       brd_SEARCH       = new JPanel    ( ) ;
   private JPanel       brd_WARRANTY     = new JPanel    ( ) ;
   private JTextField   ef_SEARCH        = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 80 ), "", 0 ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private TypeWarrantyRec  warrantyRec          = null;
   private Vector           retVec               = null ;
   private Vector           warrantyVec          = null ;
   private boolean multiple         = false ;
   //private boolean showManagersOnly = false ;
   private int     rowHeight        = 18 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String[] warrantyTitle = {Str.getStr ( STR_NAME )} ;



   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our dialog
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectWarrantyDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_PLEASE_SELECT_A_WARRANTY ), true ) ;

      init ( ) ;
   }

   public SelectWarrantyDlg ( Frame parent, boolean multiple ) {
      super ( parent, Str.getStr ( STR_PLEASE_SELECT_A_WARRANTY ), true ) ;

      this.multiple = multiple ;
      init ( ) ;
   }

   public SelectWarrantyDlg ( Frame parent, Vector warrantyVec ) {
      super ( parent, Str.getStr ( STR_PLEASE_SELECT_A_WARRANTY ), true ) ;

      this.warrantyVec = warrantyVec ;
      this.multiple = multiple ;
      init (  ) ;
   }

   public SelectWarrantyDlg ( Frame parent, Vector warrantyVec, boolean multiple ) {
      super ( parent, Str.getStr ( STR_PLEASE_SELECT_A_WARRANTY ), true ) ;

      this.warrantyVec = warrantyVec ;
      this.multiple = multiple ;
      init ( ) ;
   }

   public SelectWarrantyDlg ( Frame parent, TypeWarrantyRec warrantyRec ) {
      super ( parent, Str.getStr ( STR_PLEASE_SELECT_A_WARRANTY ), true ) ;

      this.warrantyRec =  warrantyRec ;
      init (  ) ;
   }


   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our window
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void init () {
      setResizable ( false ) ;
      createControls() ;
      refreshList() ;
      setSize ( 340, 300 ) ;
      WinUtil.centerWindow ( this ) ;
      addWindowListener ( new ComponentPopFocus ( ef_SEARCH ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the window, waits for
    *  the user to select a warranty, and then will
    *  return a vector of what was selected.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NULL if the user canceled
    *//*---------------------------------------------------------*/
   public Vector getResult() {
      setVisible ( true ) ;
      return( retVec ) ;
   }



   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and attach all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      warrantyList.setColumnHeadings ( warrantyTitle ) ;
      warrantyList.setColumnWidth ( 1, 250 ) ;
      warrantyList.setForeground ( Color.black ) ;
      warrantyList.setRowHeight(18) ;
      brd_SEARCH.setBorder ( new TitledBorder ( Str.getStr ( STR_FIND ) ) ) ;
      brd_WARRANTY.setBorder ( new TitledBorder ( "" ) ) ;
      brd_SEARCH.setLayout ( null ) ;
      brd_WARRANTY.setLayout ( null ) ;
      warrantyList.setBorder ( null ) ;
      warrantyList.showColumnHeadings ( false ) ;
      pnl_WARRANTYLIST.setBackground ( Color.white ) ;
      pnl_WARRANTYLIST.setBorder ( GUISystem.loweredBorder ) ;
      st_TITLE.setFont ( FontSystem.defaultFont ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add controls to the bordered panels                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pnl_WARRANTYLIST.setLayout ( new BorderLayout() ) ;
      pnl_WARRANTYLIST.add ( st_TITLE, BorderLayout.NORTH ) ;
      pnl_WARRANTYLIST.add ( warrantyList ) ;
      brd_SEARCH.add ( ef_SEARCH ) ;
      brd_SEARCH.add ( pb_SEARCH ) ;
      brd_WARRANTY.add ( pnl_WARRANTYLIST ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set initial size of components                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_SEARCH.setBounds ( 10, 15, 200, rowHeight ) ;
      pb_SEARCH.setBounds ( 210, 15, 90, rowHeight ) ;
      pnl_WARRANTYLIST.setBounds ( 10, 15, 300, 150 ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      ef_SEARCH.addActionListener ( this ) ;
      pb_SEARCH.addActionListener ( this ) ;
      warrantyList.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_SELECT.addActionListener ( this ) ;      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      getContentPane().add ( brd_SEARCH ) ;
      getContentPane().add ( brd_WARRANTY ) ;
      getContentPane().add ( pb_SELECT ) ;
      getContentPane().add ( pb_CANCEL ) ;
   }




   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method is called after window initialization.
    *  We will use this time to populate our ContainerWin
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshList () {

      if ( warrantyVec == null ) {
         Vector vector     = null ;

         vector = (Vector)TypeList.getInstance().getTypeList( TypeList.TYPE_WARRANTY).clone() ;

         if ( vector != null )
            warrantyList.add ( vector ) ;
      }

      else
         warrantyList.add ( warrantyVec ) ;

      if ( warrantyRec != null )
         warrantyList.setSelectedItem ( warrantyRec, true ) ;

      ef_SEARCH.requestFocus() ;
      warrantyList.paintAll(warrantyList.getGraphics() ) ;
   }




   /*-----------------------------------------------------------*/
   /* layoutControls
    */
   /**
    *    This method will lay out all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       x         = 10 ;
      int       y         = 10 ;
      Dimension size      = getSize() ;

      super.doLayout() ;
      size.height-=30 ;

      brd_SEARCH.setBounds ( x, y, size.width-(x*2), (rowHeight*2)+5 ) ;
      y+=(rowHeight*2)+5 ;

      brd_WARRANTY.setBounds ( x, y, size.width-(x*2), size.height-50-y ) ;

      pb_SELECT.setBounds ( x, size.height-40, 70, 25 ) ;
      pb_CANCEL.setBounds ( x+75, size.height-40, 90, 25 ) ;
   } /* layoutControls */



   /*-----------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method responds to actions being fired
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == ef_SEARCH )
         search() ;
      else if ( e.getSource() instanceof DButton ) {
         DButton button = (DButton) e.getSource() ;

         if ( pb_CANCEL == button ) {
            retVec = null ;
            dispose() ;
         }

         else if ( pb_SEARCH == button )
            search() ;

         else if ( pb_SELECT == button ) {
            if ( saveData () )
               dispose() ;
         }
      } else if ( e.getSource() == warrantyList ) {
         if ( saveData() )
            dispose() ;
      }
   }




   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save all of the selections and
    *  return
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Data was saved Ok, FALSE - No data was saved
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      boolean rc     = false ;
      retVec = warrantyList.getSelection() ;

      if ( retVec == null )
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
      else
         rc = true ;

      return( rc ) ;
   }


   /*------------------------------------------------------------*/
   /* search
    */
   /**
    *    This method will search through the
    *  current list and try to find an item matching
    *  the description in the find field.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void search () {
      String            searchStr  = ef_SEARCH.getText() ;

      if ( ( searchStr != null ) && ( searchStr.length() > 0 ) ) {
         Vector warrantyVec = (Vector)TypeList.getInstance().getTypeList( TypeList.TYPE_WARRANTY).clone() ;
         int i = 0 ;

         /* Uppercase the search str */
         searchStr = searchStr.toUpperCase() ;

         if ( warrantyVec != null ) {
            int                 length      = 0 ;
            boolean             found       = false ;
            TypeWarrantyRec     warrantyRec = null ;

            length = warrantyVec.size() ;

            while ( ( !found ) && ( i < length ) ) {
               warrantyRec = (TypeWarrantyRec) warrantyVec.elementAt(i) ;
               if ( ( warrantyRec != null ) &&
                    ( warrantyRec.toString().toUpperCase().indexOf ( searchStr ) != -1 ))
                  found = true ;
               i++ ;
            }

            if ( found )
               warrantyList.setSelectedItem ( warrantyRec, true ) ;
         }
      }

   } /* find */

} /* SelectWarrantyDlg */ 
