package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   SingleDateEntryDlg                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SingleDateEntryDlg                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog simply allows the user to enter a single string of text 
 *
 *
 * @version 1.0, 7/30/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   SingleDateEntryDlg                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class SingleDateEntryDlg extends JDialog implements AppConst, ActionListener, WindowListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton        pb_OK       = null ;
   private DButton        pb_CANCEL   = null ;
   private DateEntryField ef_ENTRY    = null ;
   private JLabel         st_DESCRIPT = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String retStr = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param initialText The text to initialy show
    *  @param mask The MaskDocument to apply to the entry field
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SingleDateEntryDlg ( Frame parent, String title, String descript, String initialText ) {
      super ( parent, title, true ) ;

      ef_ENTRY = new DateEntryField () ;

      /* Add the controls  */
      getContentPane().setLayout ( null ) ;
      getContentPane().add ( st_DESCRIPT = new JLabel ( descript ) ) ;
      getContentPane().add ( ef_ENTRY ) ;
      getContentPane().add ( pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ) ;
      getContentPane().add ( pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ) ;

      /* Add the listeners  */
      addWindowListener ( this ) ;
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      ef_ENTRY.addActionListener ( this ) ;

      retStr = initialText ;

      setSize ( 250, 120 ) ;
      WinUtil.centerWindow ( this ) ;
   }




   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the dialog and not
    *  return until the user presses either Ok or Cancel.
    *  If Cancel is hit, a NULL is returned, Otherwise the text is returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getResult() {
      setVisible ( true ) ;
      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and size each
    *  component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      super.doLayout() ;

      st_DESCRIPT.setBounds ( 5, 5, size.width-10, rowHeight ) ;
      ef_ENTRY.setBounds ( 5, 5+rowHeight, size.width-10, rowHeight ) ;
      pb_OK.setBounds ( 5, size.height-55, 70, 25 ) ;
      pb_CANCEL.setBounds ( 75, size.height-55, 85, 25 ) ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    THis method will be called when an action
    *  is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( e.getSource() == pb_OK ) || ( e.getSource() == ef_ENTRY ) ) {

         retStr = ef_ENTRY.getText() ;
         if ( retStr == null )
            GUISystem.printBox ( STR_ERROR, STR_DATE_ENTERED_WRONG ) ;
         else
            dispose() ;
      } else {
         retStr = null ;
         dispose() ;
      }

   }


   public void windowClosing ( WindowEvent e ) {}
   public void windowClosed ( WindowEvent e ) {}
   public void windowIconified ( WindowEvent e ) {}
   public void windowDeiconified ( WindowEvent e ) {}
   public void windowActivated ( WindowEvent e ) {
      ef_ENTRY.requestFocus();
      if ( retStr != null ) {
         ef_ENTRY.setText ( retStr ) ;
         try {
            ef_ENTRY.setSelectionStart ( retStr.length()-1 ) ;
            ef_ENTRY.setSelectionEnd   ( retStr.length()-1 ) ;
         } catch ( Exception ex ) {
         }
      }
   }
   public void windowDeactivated ( WindowEvent e ) {}
   public void windowOpened ( WindowEvent e ) {}





   /*------------------------------------------------------------*/
   /* focusGained
    */
   /**
    *    This method will be called when the entry
    *  field has gained focus
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusGained ( FocusEvent e ) {
   }



} /* SingleDateEntryDlg */
