package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   SingleEntryDlg                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SingleEntryDlg                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog simply allows the user to enter a single string of text 
 *
 *
 * @version 1.0, 7/30/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   SingleEntryDlg                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class SingleEntryDlg extends JDialog implements Runnable, AppConst, ActionListener, WindowListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton          pb_OK           = null ;
   private DButton          pb_CANCEL       = null ;
   private JTextField       ef_ENTRY        = null ;
   private JLabel           st_DESCRIPT     = null ;
   private SpellCheckPanel  spellCheckPanel = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String retStr = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param initialText The text to initialy show
    *  @param mask The MaskDocument to apply to the entry field
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SingleEntryDlg ( Frame parent, String title, String descript, String initialText, MaskDocument mask ) {
      super ( parent, title, true ) ;

      initialize ( title, descript, initialText, null, mask ) ;
   }

   public SingleEntryDlg ( Dialog parent, String title, String descript, String initialText, MaskDocument mask ) {
      super ( parent, title, true ) ;

      initialize ( title, descript, initialText, null, mask ) ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param initialText The text to initialy show
    *  @param mask The MaskDocument to apply to the entry field
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SingleEntryDlg ( Frame parent, String title, String descript, String initialText, String mask ) {
      super ( parent, title, true ) ;

      initialize ( title, descript, initialText, mask, null ) ;

   }

   public SingleEntryDlg ( Dialog parent, String title, String descript, String initialText, String mask ) {
      super ( parent, title, true ) ;

      initialize ( title, descript, initialText, mask, null ) ;

   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param initialText The text to initialy show
    *  @param mask The MaskDocument to apply to the entry field
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SingleEntryDlg ( Frame parent, String title, String descript, String initialText ) {
      super ( parent, title, true ) ;

      initialize ( title, descript, initialText, null, null ) ;

   }
   public SingleEntryDlg ( Dialog parent, String title, String descript, String initialText ) {
      super ( parent, title, true ) ;

      initialize ( title, descript, initialText, null, null ) ;

   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize all of our local
    *  variables and GUI
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize ( String title, String descript, String initialText, String maskString, MaskDocument mask ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( mask != null ) && ( maskString != null ) ) {
         ef_ENTRY = new JTextField ( "" ) ;
         ef_ENTRY.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, maskString, ef_ENTRY ) ) ;
      } else if ( mask != null )
         ef_ENTRY = new JTextField ( mask, "", 0 ) ;
      else
         ef_ENTRY = new JTextField ( "" ) ;
      spellCheckPanel = new SpellCheckPanel () ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      spellCheckPanel.setTextComponent ( ef_ENTRY ) ;       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      getContentPane().add ( st_DESCRIPT = new JLabel ( descript ) ) ;
      getContentPane().add ( spellCheckPanel ) ;
      getContentPane().add ( ef_ENTRY ) ;
      getContentPane().add ( pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ) ;
      getContentPane().add ( pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      addWindowListener ( this ) ;
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      ef_ENTRY.addActionListener ( this ) ;

      retStr = initialText ;

      setSize ( 370, 140 ) ;
      setResizable ( false ) ;
      WinUtil.centerWindow ( this ) ;

   }

   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the dialog and not
    *  return until the user presses either Ok or Cancel.
    *  If Cancel is hit, a NULL is returned, Otherwise the text is returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getResult() {
      setVisible ( true ) ;
      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and size each
    *  component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size          = getSize() ;
      int       rowHeight     = GUISystem.getRowHeight() ;
      Dimension spellPrefSize = spellCheckPanel.getPreferredSize() ;
      int       y             = 5 ;

      super.doLayout() ;

      st_DESCRIPT.setBounds ( 5, 5, size.width-10, rowHeight ) ;
      y+=rowHeight+5 ;
      
      ef_ENTRY.setBounds ( 5, y, size.width-15, rowHeight ) ;
      y+=rowHeight ;

      spellCheckPanel.setBounds ( 5, y, size.width-15, spellPrefSize.height ) ;
      y+=spellPrefSize.height ;
      pb_OK.setBounds ( 5, size.height-55, 70, 25 ) ;
      pb_CANCEL.setBounds ( 75, size.height-55, 85, 25 ) ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    THis method will be called when an action
    *  is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( e.getSource() == pb_OK ) || ( e.getSource() == ef_ENTRY ) ) {
         retStr = ef_ENTRY.getText() ;
         new Thread ( this ).start() ;
      } else {
         retStr = null ;
         new Thread ( this ).start() ;
      }

   }


   public void windowClosing ( WindowEvent e ) {
   }
   public void windowClosed ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e ) {
   }
   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ) {
   }
   public void windowOpened ( WindowEvent e ) {
      ef_ENTRY.requestFocus();
      if ( retStr != null ) {
         ef_ENTRY.setText ( retStr ) ;
         try {
            ef_ENTRY.setSelectionStart ( retStr.length()-1 ) ;
            ef_ENTRY.setSelectionEnd   ( retStr.length()-1 ) ;
         } catch ( Exception ex ) {
         }
      }
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will simply call the dispose method
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      dispose() ;
   }

   public void setVisible ( boolean visible ) {
      addWindowListener ( this ) ;
      super.setVisible ( visible ) ;
   }
} /* SingleEntryDlg */
