package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   SingleMLEDlg                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SingleMLEDlg                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This dialog simply allows the user to enter a single string of text 
 * into a JTextArea control
 *
 * @version 1.0, 7/30/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   SingleMLEDlg                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class SingleMLEDlg extends JDialog implements AppConst, ActionListener, WindowListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton     pb_OK       = null ;
   private DButton     pb_CANCEL   = null ;
   private JTextArea   mle_ENTRY   = null ;
   private JLabel      st_DESCRIPT = null ;
   private JScrollPane scrollPane  = null ; 

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private String retStr      = null ;
   private String descript    = null ;
   private String initialText = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param initialText The text to initialy show
    *  @param mask The MaskDocument to apply to the entry field
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SingleMLEDlg ( Frame parent, String title, String descript, String initialText, MaskDocument mask ) {
      super ( parent, title, true ) ;

      this.descript = descript ;
      this.initialText = initialText ;

      if ( mask != null )
         mle_ENTRY = new JTextArea ( mask ) ;
      else
         mle_ENTRY = new JTextArea ( "" ) ;

      init ( parent ) ;
   }


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param initialText The text to initialy show
    *  @param mask The MaskDocument to apply to the entry field
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SingleMLEDlg ( Frame parent, String title, String descript, String initialText, String mask ) {
      super ( parent, title, true ) ;

      mle_ENTRY = new JTextArea ( "" ) ;

      mle_ENTRY.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, mask, mle_ENTRY ) ) ;

      this.descript = descript ;
      this.initialText = initialText ;
      init ( parent ) ;
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the dialog. Use the
    *  getResult method to get what the user types
    *  in. This constructor will return immediatly without showing the dialog
    *
    *  @param frame parent
    *  @param title The title in the window
    *  @param descript The description above the entryfield
    *  @param initialText The text to initialy show
    *  @param mask The MaskDocument to apply to the entry field
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SingleMLEDlg ( Frame parent, String title, String descript, String initialText ) {
      super ( parent, title, true ) ;

      mle_ENTRY = new JTextArea ( initialText ) ;

      this.initialText = initialText ;
      this.descript = descript ;

      init ( parent ) ;
   }





   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our window
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void init ( Frame parent ) {

      /*----------------------*/
      /*                      */      
      /*  Assign Default Vars */
      /*                      */      
      /*----------------------*/
      mle_ENTRY.setText ( initialText ) ;
      retStr = initialText ;

      /*--------------*/
      /*              */              
      /*  Add the GUI */
      /*              */              
      /*--------------*/
      getContentPane().setLayout ( null ) ;
      getContentPane().add ( st_DESCRIPT = new JLabel ( descript ) ) ;
      getContentPane().add ( scrollPane = new JScrollPane ( mle_ENTRY ) ) ;
      getContentPane().add ( pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ) ;
      getContentPane().add ( pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ) ;


      /*--------------------*/
      /*                    */        
      /*  Adjust properties */
      /*                    */        
      /*--------------------*/
      mle_ENTRY.setFont ( pb_OK.getFont() ) ;

      /*----------------*/
      /*                */            
      /*  Add listeners */
      /*                */            
      /*----------------*/
      addWindowListener ( this ) ;
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;

      setSize ( 320, 270 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
   }


   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the dialog and not
    *  return until the user presses either Ok or Cancel.
    *  If Cancel is hit, a NULL is returned, Otherwise the text is returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getResult() {
      setVisible ( true ) ;
      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and size each
    *  component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;
      st_DESCRIPT.setBounds ( 5, 5, size.width-10, rowHeight ) ;
      scrollPane.setBounds ( 5, 5+rowHeight, size.width-15, size.height-80 ) ;
      pb_OK.setBounds ( 5, size.height-55, 70, 25 ) ;
      pb_CANCEL.setBounds ( 80, size.height-55, 85, 25 ) ;
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    THis method will be called when an action
    *  is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( e.getSource() == pb_OK ) || ( e.getSource() == mle_ENTRY ) ) {
         retStr = mle_ENTRY.getText() ;
         dispose() ;
      } else {
         retStr = null ;
         dispose() ;
      }

   }


   public void windowClosing ( WindowEvent e ) {}
   public void windowClosed ( WindowEvent e ) {}
   public void windowIconified ( WindowEvent e ) {}
   public void windowDeiconified ( WindowEvent e ) {}
   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ) {}
   public void windowOpened ( WindowEvent e ) {
      mle_ENTRY.requestFocus();
      if ( retStr != null ) {
         mle_ENTRY.setText ( retStr ) ;
         try {
            mle_ENTRY.setSelectionStart ( retStr.length()-1 ) ;
            mle_ENTRY.setSelectionEnd   ( retStr.length()-1 ) ;
         } catch ( Exception ex ) {
         }
      }
   }


} /* SingleMLEDlg */
