package com.ibm.nzna.projects.qit.gui ;   
/*-----------------------------------------------------------*/
/*   StatusBar                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StatusBar                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This Label holds the busy image, status text, application
 * action buttons and the completion gauge.
 *
 *
 * @version 1.0, 1/21/1999 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   StatusBar                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class StatusBar extends JPanel implements PropertyListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static App Buttons                                      */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int            APPBUTTON_WIDTH = 20 ;
   private static final int            GAUGE_WIDTH     = 70 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int CONTROL_HEIGHT  = 23 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   protected JProgressBar   gge_STATUS      = null ;
   protected JLabel         st_STATUS       = null ;              
   private   DBStatusLight dbStatusLight    = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private   Dimension     prefSize          = new Dimension ( 0, CONTROL_HEIGHT ) ;
   private   String        status            = null ;
   private   boolean       appButtonsCreated = false ;
   private   int           appBarWidth       = 0 ;   
   private   int           width             = 0 ;
   private   int           height            = 0 ;
   private   FontMetrics   fm                = null ;
   private   int           space             = 0 ; // Used for vertically centering text
   private   int           fontY             = 0 ;


   /*------------------------------------------------------------*/
   /* StatusBar
    */
   /**
    *    This will create our default StatusBar
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public StatusBar () {
      init() ;
   }





   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize the StatusBar
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void init () {        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      gge_STATUS    = new JProgressBar ( ) ;
      st_STATUS     = new JLabel      ( ) ;
      dbStatusLight = new DBStatusLight() ;      

      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      gge_STATUS.setFont ( FontSystem.smallFont ) ;
      gge_STATUS.setForeground ( new Color ( 0, 0, 120 ) ) ;
      propertyChanged ( PropertySystem.PROPERTY_SKINNAME, null ) ;                


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to ourself                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( dbStatusLight ) ;
      add ( gge_STATUS ) ;
      add ( st_STATUS ) ;

      setBorder ( null ) ;
   }




   /*------------------------------------------------------------*/
   /* getPreferredSize
    */
   /**
    *    This method will return the preferred size
    *  of our control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Dimension
    *//*---------------------------------------------------------*/
   public Dimension getPreferredSize() {
      prefSize.width = getParent().getSize().width ;
      return( prefSize ) ;
   }

   /*------------------------------------------------------------*/
   /* setStatus
    */
   /**
    *    This method will set our status text. A
    *  status text of null will blank out the status
    *  line
    *
    *  @param statusText The new Status Text
    *  @param showBusy TRUE = Show the busy image, FALSE if not
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatus ( String statusText, boolean showBusy ) {
      this.status = statusText ;
      SwingUtilities.invokeLater ( new UpdateStatusText ( this, statusText, showBusy ) ) ;
   }



   public void removeNotify () {
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_SKINNAME, this ) ;
      super.removeNotify() ;
   }





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when a property
    *  we are listening for has been changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object newValue ) {
      switch ( property ) {
         case PropertySystem.PROPERTY_SKINNAME :
            String borderType = (String)ThemeSystem.getThemeProperty ( "STATUS_RECT" ) ;
            Color  bg         = ThemeSystem.getColor ( "STATUS_BG_COLOR" ) ;
            Color  fg         = ThemeSystem.getColor ( "STATUS_FG_COLOR" ) ;

            if ( borderType.equalsIgnoreCase ( "LOWERED" ) ) {
               st_STATUS.setBorder ( GUISystem.loweredBorder ) ;
               gge_STATUS.setBorder ( GUISystem.loweredBorder ) ;
               dbStatusLight.setBorder ( GUISystem.loweredBorder ) ;
            } else if ( borderType.equalsIgnoreCase ( "RAISED" ) ) {
               st_STATUS.setBorder ( GUISystem.raisedBorder ) ;
               gge_STATUS.setBorder ( GUISystem.raisedBorder ) ;
               dbStatusLight.setBorder ( GUISystem.raisedBorder ) ;
            } else if ( borderType.equalsIgnoreCase ( "ETCHED" ) ) {
               st_STATUS.setBorder ( GUISystem.etchedBorder ) ;
               gge_STATUS.setBorder ( GUISystem.etchedBorder ) ;
               dbStatusLight.setBorder ( GUISystem.etchedBorder ) ;
            }

            gge_STATUS.setStringPainted ( ThemeSystem.getBoolean ( "STATUS_SHOW_PERCENTAGE_TEXT" ) ) ;


            if ( bg != null ) {
               setBackground ( bg ) ;
               setOpaque ( true ) ;
            }
            else
               setOpaque ( false ) ;

            if ( fg != null )
               st_STATUS.setForeground ( fg ) ;
            else
               st_STATUS.setForeground ( Color.black ) ;

            gge_STATUS.setBackground ( getBackground() ) ;
            break ;
      }
   }



   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position
    *  all of our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = null ;
      int       x    = 2 ;
      int       y    = 2 ;

      super.doLayout() ;

      /* Get the viewable area size */
      size = getSize() ;
      height = size.height ;
      width = size.width ;

      dbStatusLight.setBounds ( x, y, 20, height-(y*2) ) ;
      x+=22 ;

      st_STATUS.setBounds ( x, y, width-GAUGE_WIDTH-x-2, height-(y*2) ) ;
      gge_STATUS.setBounds ( width-GAUGE_WIDTH, y, GAUGE_WIDTH-4, height-(y*2) ) ;

   }




   /*------------------------------------------------------------*/
   /* setStatusValue
    */
   /**
    *    This method will set the value that the gauge
    *  is indicating. Classes not members of the quest.base package
    *  should call SystemData.getMainWin().setStatusValue(x)
    *  
    *
    *  @param value The int value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatusValue(int value) {
      SwingUtilities.invokeLater ( new UpdateStatusValue ( this, value ) ) ;
   }





   /*------------------------------------------------------------*/
   /* setStatusMax
    */
   /**
    *    This method will set the maximum value for
    *  the gauge control to represent. Classes not members of
    *  quest.base should call SystemData.getMainWin().setStatusValue(x)
    *  
    *
    *  @param maxValue The maximum value
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setStatusMax ( int maxValue ) {
      gge_STATUS.setMaximum ( maxValue ) ;
   }





   /*------------------------------------------------------------*/
   /* incrementStatusValue
    */
   /**
    *    This method will increment the status
    *  value by one
    *  
    *
    *  @param increment The number to increment by
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void incrementStatusValue ( int increment) {
      try {
         gge_STATUS.setValue ( gge_STATUS.getValue()+increment ) ;
      } catch ( Exception e ) {
         LogSystem.log  ( 1, e ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* getStatus
    */
   /**
    *    This method will return the String of status
    *  text that is displayed on the status bar
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getStatus () {
      return( st_STATUS.getText() ) ;
   }

} /* StatusBar */


class UpdateStatusText extends Object implements Runnable {

   StatusBar StatusBar = null ;
   String      status      = null ;
   boolean     showBusy    = false ;

   public UpdateStatusText ( StatusBar StatusBar, String status, boolean showBusy ) {
      this.StatusBar = StatusBar ;  
      this.status      = status ;
      this.showBusy    = showBusy ;
   }

   public void run () {
      if ( status != null ) {
         StatusBar.st_STATUS.setText ( status ) ;
      } else {
         StatusBar.st_STATUS.setText ( "" ) ;
         StatusBar.st_STATUS.setIcon ( null ) ;
      }
      //StatusBar.revalidate() ;
   }
}


class UpdateStatusValue extends Object implements Runnable {

   StatusBar StatusBar = null ;
   int         value       = 0 ;

   public UpdateStatusValue ( StatusBar StatusBar, int value ) {
      this.StatusBar = StatusBar ;
      this.value       = value ;
   }


   public void run () {
      int    newPercentage = 0 ;
      int    max           = StatusBar.gge_STATUS.getMaximum() ;
      String strValue      = "" ;


      if ( ( value > 0 ) && ( max > 0 ) )
         newPercentage = new Integer ( value * 100 / (max - 0) ).intValue() ;

      if ( newPercentage > 100 )
         newPercentage = 100 ;

      StatusBar.gge_STATUS.setValue ( value ) ;
      if ( value > 0 )
         StatusBar.gge_STATUS.setString ( newPercentage+"%" ) ;
      else
         StatusBar.gge_STATUS.setString ( "" ) ;
   }
}
