package com.ibm.nzna.projects.qit.gui ;         
/*-----------------------------------------------------------*/
/*   StatusWin                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: StatusWin                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import java.awt.* ;
/**
 * This window simply displays progress to the user.
 * The status text and icon can be changed so really it could be used for anything!
 *
 * @version 1.0, 4/28/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   StatusWin                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class StatusWin extends JDialog implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel          st_IMAGE       = null ;
   private JLabel          st_TEXT        = null ;
   private JProgressBar    progressBar    = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the only constructor.
    *  
    *  
    *
    *  @param parent The parent frame of this dialog
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public StatusWin ( Frame parent ) {     
      super ( parent, Str.getStr ( STR_PLEASE_WAIT ), false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_IMAGE = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.UNPACK ) ) ;
      st_TEXT  = new JLabel ( "" ) ;
      progressBar = new JProgressBar ( 0, 1 ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the gui                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      getContentPane().add ( st_IMAGE ) ;
      getContentPane().add ( st_TEXT ) ;
      getContentPane().add ( progressBar ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      setSize ( 330, 125 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will do our gui positioning
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 10, 32, 32 ) ;

      st_TEXT.setBounds ( 50, 10, size.width-60, 30 ) ;
      progressBar.setBounds ( 50, size.height-rowHeight-35, size.width-65, rowHeight ) ;

   }





   /*------------------------------------------------------------*/
   /* setValue
    */
   /**
    *    This method will set the progress on the
    *  progress bar to the value specified
    *  
    *
    *  @param value The new value to display in the progress bar
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setValue ( int value ) {
      progressBar.setValue ( value ) ;
      repaint() ;
   }





   /*------------------------------------------------------------*/
   /* setImageIcon
    */
   /**
    *    This method will change the image that
    *  is being displayed. Please note that the image
    *  will be cut off at 32x32 pixels if it is too large
    *
    *  @param imageIcon The ImageIcon to use
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setImageIcon ( ImageIcon icon ) {
      st_IMAGE.setIcon ( icon ) ;
   }




   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text that we are
    *  displaying
    *  
    *
    *  @param text The text, should be in <html><body> </html></body> tags
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setText ( String text ) {
      st_TEXT.setText ( text ) ;
   }





   /*------------------------------------------------------------*/
   /* setMaxValue
    */
   /**
    *    This method will set the max value displayed
    *  in the gauge.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMaxValue ( int value ) {
      progressBar.setMaximum ( value ) ;
   }


} /* StatusWin */
