package com.ibm.nzna.projects.qit.gui ;       
/*-----------------------------------------------------------*/
/*   ToolBarButton                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ToolBarButton                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;  
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This subclass of DButton can be added to any JToolBar. It will
 * resepect the properties that the user has set for the tool bar buttons
 *
 *
 * @version 1.0, 1/20/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ToolBarButton                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class ToolBarButton extends DButton {



    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Static Members                                  */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static Insets       insets   = new Insets ( 0, 0, 0, 0 ) ;      

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private String    labelText   = "" ;
    private String    toolTipText = "" ;
    private ImageIcon icon        = null ;
    private int       iconIndex   = 0 ;



    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create the ToolBarButton.
     *  
     *  
     *
     *  @param text The text to display
     *  @param icon The ImageIcon to display
     *  @param helpText The help text to show
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public ToolBarButton ( String text, ImageIcon icon, String helpText ) {
        super ( text, icon ) ;                 

        try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Assign our vars                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            this.labelText   = text ;
            this.toolTipText = helpText ;
            setIcon ( icon ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set property dependant properties                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            setProperties() ;

        } catch ( Exception e ) {
        }
    }




    /**
      *  Do not respond to focus events
      **/
    //public boolean isFocusTraversable () {
    //   return ( false ) ;
    //}

    /*------------------------------------------------------------*/
    /* setText
     */
    /**
     *    This method will set our text that is
     *  being displayed
     *  
     *
     *  @param text The text to show
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setText ( String text ) {
        this.labelText = text ;
        try {
            if ( PropertySystem.getString ( PropertySystem.PROPERTY_TOOLBAR_SHOW_ICON_TEXT ).indexOf ( "TEXT" ) >= 0 )
                super.setText ( text ) ;
        } catch ( Exception e ) {
        }
    }





    /*------------------------------------------------------------*/
    /* setToolTipText
     */
    /**
     *    This method will set our tool tip text
     *  
     *  
     *
     *  @param text The text to show in a Fly over help
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setToolTipText ( String text ) {
        this.toolTipText =  text ;
        try {
            if ( PropertySystem.getBool ( PropertySystem.PROPERTY_TOOLBAR_HELP ) )
                super.setToolTipText ( text ) ;
        } catch ( Exception e ) {
        }

    }


    /*------------------------------------------------------------*/
    /* setProperties
     */
    /**
     *    This method will reset all of the
     *  properties of this button
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setProperties () {

        try {
            String orient = PropertySystem.getString ( PropertySystem.PROPERTY_TOOLBAR_TEXT_ORIENT ) ;
            String show   = PropertySystem.getString ( PropertySystem.PROPERTY_TOOLBAR_SHOW_ICON_TEXT ) ;
            
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set our image                                           */    
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( iconIndex > 0 ) 
               setIcon ( ImageSystem.getImageIcon ( this, iconIndex ) ) ;
            

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set our default properties                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            setBorderRollover ( true ) ;                        
            setMargin ( insets ) ;
            setFocusPainted ( false ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Turn text on and off                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( show.indexOf ( "TEXT" ) >= 0 ) {
                super.setText ( labelText ) ;
            } else
                super.setText ( "" ) ;                      
            if ( show.indexOf ( "ICON" ) >= 0 )
                super.setIcon ( icon ) ;
            else
                super.setIcon ( null ) ;



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set the orientation of the text                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( orient.equals ( "LEFT" ) ) {
                setHorizontalTextPosition ( SwingConstants.LEFT ) ;
                setVerticalAlignment ( SwingConstants.CENTER ) ;
                setPreferredSize ( new Dimension ( getPreferredSize().width, 20 ) ) ;
            } else if ( orient.equals ( "RIGHT" ) ) {
                setHorizontalTextPosition ( SwingConstants.RIGHT ) ;
                setVerticalAlignment ( SwingConstants.CENTER ) ;
                //setPreferredSize ( new Dimension ( getPreferredSize().width, 20 ) ) ;
            } else if ( orient.equals ( "BOTTOM" ) ) {
                setVerticalTextPosition ( SwingConstants.BOTTOM ) ;
                setVerticalAlignment ( SwingConstants.TOP ) ;
                setHorizontalTextPosition ( SwingConstants.CENTER ) ;
            } else if ( orient.equals ( "TOP" ) ) {
                setVerticalTextPosition ( SwingConstants.TOP ) ;
                setVerticalAlignment ( SwingConstants.BOTTOM ) ;
                setHorizontalTextPosition ( SwingConstants.CENTER ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set opaque on/off                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            setOpaque ( false ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Turn tool tips on/off                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( PropertySystem.getBool ( PropertySystem.PROPERTY_TOOLBAR_HELP ) )
                super.setToolTipText ( toolTipText ) ;
            else
                super.setToolTipText ( "" ) ;

        } catch ( Exception e ) {
        }
    }




    /*------------------------------------------------------------*/
    /* setIcon
     */
    /**
     *    This method will set the icon that
     *  is displayed
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setIcon ( ImageIcon icon ) {
        this.icon = icon ;
        try {
            if ( PropertySystem.getString ( PropertySystem.PROPERTY_TOOLBAR_SHOW_ICON_TEXT ).indexOf ( "ICON" ) >= 0 )
                super.setIcon ( icon ) ;
        } catch ( Exception e ) {
        }
    }




} /* ToolBarButton */
