package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   UserEntryPanel                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: UserEntryPanel                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 David Lentz                      */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/**
 * This panel holds 2 controls. The EntryField and Select User Button. 
 * It can be used to allow the user to enter either a Full Name or User Id. It will auto-populate
 * the entry field if the user stops typing for 1 second. There are methods to get the text either
 * in User Id form, or Full Name.            
 *
 * @version 1.0, 11/3/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   UserEntryPanel                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class UserEntryPanel extends JPanel implements ActionListener, Runnable, AppConst, DataLengths {

   /**
     *  This is the name of the thread that will search for users
     **/
   private static final String THREAD_SEARCH = "SE" ;

   /**
     *  This is the name of the thread that will fire events
     **/
   private static final String THREAD_FIRE_EVENT = "FE" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextField ef_USERID     = new JTextField() ;
   private DButton    pb_SELECTUSER = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This vector holds all registered listeners
     **/
   private Vector listeners = new Vector ( 1 ) ;


   /**
     *  This is the property which holds the last user searched for.
     **/
   private String  lastUserSearch = "" ;

   /**
     *  If TRUE, the control is currently searching for UserRec objects
     **/
   private boolean inSearch       = false ;

   /**
     *  This is the vector of users that can be selected
     **/
   private Vector  userVec        = null ;

   /**
     *  This is the currently selected user record
     **/
   private UserRec userRec        = null ;

   /**
     *  If TRUE, this property tells the control to show the user's full name, not just the user id
     **/
   private boolean showFullName   = true ;

   /**
     *  This property tells the running thread to continue
     **/
   private boolean continueThread = true ;

   /**
     *  This is a temporary property to hold the current user
     **/
   private String  curTempUser    = null ;

   /**
     *  This property tells the control to show managers only
     **/
   private boolean showManagerOnly    = false ;

   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the User Panel
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public UserEntryPanel () {
      init() ;
   }


   /*-----------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void init () {
      createControls () ;
      userVec = UserSystem.getAllUserVec() ;
      new Thread ( this, THREAD_SEARCH ).start() ;

   }

   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our controls
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void createControls () {

      setLayout ( null ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECTUSER = new DButton ( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_PERSONEL ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //setBorder ( GUISystem.loweredBorder ) ;
      pb_SELECTUSER.setFocusPainted ( false ) ;
      //JDK14 pb_SELECTUSER.setFocusable ( false ) ;
      pb_SELECTUSER.setBorderPainted ( false ) ;
      //ef_USERID.setBorder ( GUISystem.emptyBorder ) ;
      pb_SELECTUSER.setBackground ( Color.white ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECTUSER.addActionListener ( this ) ;
      ef_USERID.addActionListener ( this ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout () ) ;
      add ( ef_USERID, BorderLayout.CENTER ) ;
      add ( pb_SELECTUSER, BorderLayout.EAST ) ;

   }


   /*-----------------------------------------------------------*/
   /* getText
    */
   /**
    *    This method will simply return whatever is
    *  in the User Entry Field. It is not guarenteed
    *  what is in the field
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public String getText() {
      return( ef_USERID.getText() ) ;
   }





   /*-----------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of the entry field
    *  
    *  
    *             
    *  @param text The text to set
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setText ( String text ) {
      if ( ( text != null ) && ( text.length() > 0 ) ) {
         curTempUser = text ;
         userRec = findUser ( text, true ) ;
         refreshUserField() ;
      } else
         ef_USERID.setText ( "" ) ;

   }


   /*-----------------------------------------------------------*/
   /* setShowFullName
    */
   /**
    *    This method will tell the panel to show the user's full name,
    *  or just their user id. If showFullName is TRUE, the panel will
    *  display the user's full name.
    *  
    *             
    *  @param onTop TRUE = Show FullName FALSE = Only show user id
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setShowFullName ( boolean showFullName ) {
      this.showFullName = showFullName ;

      refreshUserField() ;

   }


   /*-----------------------------------------------------------*/
   /* getShowFullName
    */
   /**
    *    This method will the ShowFullName property     
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public boolean getShowFullName () {
      return( showFullName ) ;
   }


   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform our thread operations
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      if ( operation.equals ( THREAD_SEARCH ) )
         search() ;
      else if ( operation.equals ( THREAD_FIRE_EVENT ) )
         fireEvent() ;
   }


   /**
     *  This method will search for users
     **/
   private void search () {
      String curUser  = null ;
      String curUser1 = null ;

      while ( continueThread ) {

         /* Ok, first sleep for 1 1/2 second and then check the
            entry field's text */
         try {
            Thread.sleep ( 1500 ) ;
         } catch ( Exception e ) {
         }
         curUser = ef_USERID.getText() ;

         /* Ok, if both curUser and lastUserSearch are not null
            and do not equal each other, perform a search */
         if ( ( curUser != null ) &&
              ( curUser.length() > 0 ) && 
              ( lastUserSearch != null ) ) {

            /* Ok, now wait for a bit longer and check it again. If the
               text has changed, loop back up. Otherwise look for the
               name */
            try {
               Thread.sleep ( 300 ) ;
            } catch ( Exception e ) {
            }
            curUser1 = ef_USERID.getText() ;


            if ( curUser1.equals ( curUser ) ) {

               if ( !curUser1.equals ( lastUserSearch ) ) {
                  lastUserSearch = curUser1 ;

                  userRec = findUser ( curUser1 ) ;

                  /* Ok, if we found a record, populate it now */
                  refreshUserField() ;

                  if ( userRec != null )
                     new Thread ( this, THREAD_FIRE_EVENT ).start() ;
               } /* if */

            } /* if */
         } else
            userRec = null ;
      } /* while */
   } /* run */





   /*------------------------------------------------------------*/
   /* findUser
    */
   /**
    *    This method will attempt to locate a user
    *  doing a fuzzy search
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private UserRec findUser ( String user ) {
      return( findUser ( user, false ) ) ;
   }




   /*------------------------------------------------------------*/
   /* findUser
    */
   /**
    *    This method will attempt to locate the user
    *  in the application user list. If a match is found
    *  then this method will return the user record.
    *
    *  @param user Either the user id or Name of the user
    *  @param exactMatch TRUE = only hit on exact matches
    *  @param None
    *
    *  @return UserRec of the user found, NULL if not found
    *//*---------------------------------------------------------*/
   private UserRec findUser ( String user, boolean exactMatch ) {
      UserRec retRec = null ;

      try {
         if ( ( user != null ) && ( userVec != null ) ) {
            int     i              = 0 ;
            int     length         = userVec.size() ;
            UserRec compareUserRec = null ;
            Vector  foundVec       = new Vector ( 1, 10 ) ;

            /* Always upper case! */
            user = user.toUpperCase() ;                    

            /* Ok, walk through the vector and look for the 'user' in the User Id
               and name fields */
            while ( i < length ) {
               compareUserRec = (UserRec) userVec.elementAt(i) ;

               if ( compareUserRec != null ) {


                  try {

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   IF we are matching Exact, make sure it is exact         */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     if ( exactMatch ) {
                        if ( ( compareUserRec.getUserId() != null ) && 
                             ( compareUserRec.getUserId().toUpperCase().equals ( user ) ) ) {
                           foundVec.addElement ( compareUserRec ) ;
                        }

                        else if ( ( ( compareUserRec.getName() != null ) && 
                                    ( compareUserRec.getName().toUpperCase().equals ( user ) ) ) ) {
                           foundVec.addElement ( compareUserRec ) ;
                        }
                     }

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   IF we are not matching exact, match the first chars     */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     else {
                        if ( ( compareUserRec.getUserId() != null ) && 
                             ( compareUserRec.getUserId().toUpperCase().startsWith ( user ) ) ) {
                           foundVec.addElement ( compareUserRec ) ;
                        } else if ( ( ( compareUserRec.getName() != null ) && 
                                      ( compareUserRec.getName().toUpperCase().startsWith ( user ) ) ) ) {
                           foundVec.addElement ( compareUserRec ) ;
                        }
                     }
                  } catch ( Exception e ) {
                  }
               } /* if */

               i++ ;
            } /* while */

            foundVec.trimToSize() ;

            /* Ok, if we have more than 1 hit, let the user select, otherwise auto-populate the field */
            if ( foundVec.size() > 1 ) {
               Vector vec = null ;

               vec = new SelectUserDlg ( GUISystem.getParentFrame(this), foundVec, false, showManagerOnly ).getResult() ;

               if ( ( vec != null ) && ( vec.size() > 0 ) ) {
                  compareUserRec = (UserRec) vec.elementAt(0) ;

                  retRec = this.userRec = compareUserRec ;

                  refreshUserField() ;
                  if ( userRec != null )
                     new Thread ( this, THREAD_FIRE_EVENT ).start() ;
               }

            } else if ( foundVec.size() == 1 )
               retRec = (UserRec) foundVec.elementAt(0) ;


         } /* if */
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         retRec = null ;
      }

      return( retRec ) ;
   } /* findUser */






   /*------------------------------------------------------------*/
   /* refreshUserField
    */
   /**
    *    This method will refresh the data that is
    *  in the user id field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshUserField () {
      if ( userRec != null ) {
         if ( showFullName )
            ef_USERID.setText ( userRec.getName() ) ;
         else
            ef_USERID.setText ( userRec.getUserId() ) ;

         try {
            ef_USERID.setSelectionStart ( lastUserSearch.length() ) ;
            ef_USERID.setSelectionEnd ( ef_USERID.getText().length() ) ;
         } catch ( Exception e ) {
         }

         lastUserSearch = ef_USERID.getText() ;

      } /* if */

      else
         ef_USERID.setText ( "" ) ;
   }



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one of our
    *  buttons is pressed
    *  
    *
    *  @param e The actionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   DButton Events                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() instanceof DButton ) {
            DButton button = (DButton) e.getSource() ;
            if ( pb_SELECTUSER == button ) {
               Vector vec = null ;

               vec = new SelectUserDlg ( GUISystem.getParentFrame(this), false, showManagerOnly ).getResult() ;
               if ( ( vec != null ) && ( vec.size() > 0 ) ) {
                  userRec = (UserRec) vec.elementAt(0) ;
                  new Thread ( this, THREAD_FIRE_EVENT ).start() ;
               }
               refreshUserField() ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   TextField Events                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() instanceof JTextField ) {
            String text = ef_USERID.getText() ;

            userRec = findUser ( text ) ;
            refreshUserField() ;
         }
      } catch ( Exception ex ) {
         LogSystem.log ( 1, ex) ;
      }
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will stop our thread that is monitoring
    *  the entry field. It must be called when you are through with
    *  this control.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      continueThread = false ;
   }



   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name if it is valid.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return user The name if valid, NULL if not valid
    *//*---------------------------------------------------------*/
   public String getName () {
      String retStr = null ;

      if ( userRec != null )
         retStr = userRec.getName() ;

      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* getUserRec
    */
   /**
    *    This method will return the user record
    *  if it is valid, If not valid, a lookup will be performed
    *  asking the user to select an id
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public UserRec getUserRec () {
      if ( userRec == null )
         userRec = findUser ( ef_USERID.getText() ) ;

      return( userRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getUserRecNoLookup
    */
   /**
    *    This method will return the userRec selected
    *  by the user or nothing. It will not perform
    *  a second lookup
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public UserRec getUserRecNoLookup () {

      return( userRec ) ;
   }


   /*------------------------------------------------------------*/
   /* getUserId
    */
   /**
    *    This method will return the user id if it is valid.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return user The User Id if valid, NULL if not valid
    *//*---------------------------------------------------------*/
   public String getUserId () {
      String retStr = null ;

      if ( userRec != null )
         retStr = userRec.getUserId() ;

      return( retStr ) ;
   }



   /*------------------------------------------------------------*/
   /* requestFocus
    */
   /**
    *    This method has been subclassed to pass the
    *  focus to our entry field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void requestFocus () {
      ef_USERID.requestFocus() ;
   }





   /*------------------------------------------------------------*/
   /* showManagerOnly
    */
   /**
    *    This method will tell the panel only to
    *  accept Manager's User Ids. It will also only
    *  show Manager's Ids in the pop up lists
    *
    *  @param showManagerOnly  TRUE = Only Show Manager, FALSE = Show Everyone
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setShowManagerOnly ( boolean showManagerOnly ) {
      this.showManagerOnly = showManagerOnly ;
      ef_USERID.setText("");
   }




   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable
    *  our component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enable ) {
      ef_USERID.setEnabled ( enable ) ;
      pb_SELECTUSER.setEnabled ( enable ) ;
      super.setEnabled ( enable ) ;
   }


   /**
     *  This method will add a UserEntryPanelListener to this control
     *
     * @param listener The listener to be added
     **/
   public void addUserEntryPanelListener ( UserEntryPanelListener listener ) {
      listeners.addElement ( listener ) ;
   }

   /**
     *  This method will remove a UserEntryPanelListener from the control
     *
     * @param listener The listener to remove
     **/
   public void removeUserEntryPanelListener ( UserEntryPanelListener listener ) {
      listeners.removeElement ( listener ) ;
   }

   /**
     *  This method will fire a 'userSelected' event to all registered listeners
     **/
   private void fireEvent () {
      int i    = 0 ;
      int size = listeners.size() ;

      while ( i < size ) {
         ((UserEntryPanelListener)listeners.elementAt(i)).userSelected ( userRec ) ;
         i++ ;
      }
   }

   public void addFocusListener ( FocusListener listener ) {
      super.addFocusListener ( listener ) ;
      if ( ef_USERID != null )
         ef_USERID.addFocusListener ( listener ) ;
   }

   public JTextField getJTextField () {
      return( ef_USERID ) ;
   }
} /* UserEntryPanel */

