package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   LogonDlg                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: LogonDlg                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.io.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.help.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * A Dialog that asks the user to log on 
 *
 *
 * @version 1.0, 1/30/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   LogonDlg                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class UserLogonDlg extends JDialog implements AppConst, Runnable, DataLengths, KeyListener, ActionListener {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextField      ef_USERID          = new JTextField      ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_USERID ), "", 0 ) ;
   private JPasswordField  ef_PASSWORD        = new JPasswordField  ( "" ) ;
   private JLabel          st_HELPTEXT        = new JLabel          ( Str.getStr ( STR_LOGONHELP ) ) ;
   private JLabel          st_USERID          = new JLabel          ( Str.getStr ( STR_USERID    ) ) ;
   private JLabel          st_PASSWORD        = new JLabel          ( Str.getStr ( STR_PASSWORD  ) ) ;
   private JLabel          st_IMAGE           = new JLabel          ( ImageSystem.getImageIcon ( this, ImageSystem.LOGON ) ) ;
   private DButton         pb_OK              = new DButton         ( Str.getStr ( STR_OK        ) ) ;
   private DButton         pb_CANCEL          = new DButton         ( Str.getStr ( STR_CANCEL    ) ) ;
   private DButton         pb_HELP            = new DButton         ( Str.getStr ( STR_HELP      ) ) ;
   private ButtonPanel     buttonPanel        = new ButtonPanel     () ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private boolean rc = false ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constructor                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public UserLogonDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_LOGON ), true ) ;                                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write to the 'splashClose' file so that the splash      */
      /* closes                                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      FileUtil.saveFile ( "closeSplash", "close" ) ;

      setResizable ( false ) ;

      setSize ( 300, 170 ) ;
      WinUtil.centerWindow ( this) ;
   }


   /*------------------------------------------------------------*/
   /* getResponse
    */
   /**
    *    This method will pop up our window
    *  and will wait for the user to hit enter
    *  or cancel. If they hit cancel this method returns falsse
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if Ok is pressed, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean getResponse () {
      try {
         String userId   = PropertySystem.getString ( PropertySystem.PROPERTY_USERID ) ;
         String password = PropertySystem.getString ( PropertySystem.PROPERTY_PASSWORD ) ;                                               

         /*------------------------------------------------------------*/
         /*                                                            */
         /*   If we already have the user id and password, return TRUE */
         /*                                                            */
         /*------------------------------------------------------------*/
         if ( ( userId != null ) && ( userId.length() > 0 ) &&
              ( password != null ) && ( password.length() > 0 ) ) {
            dispose() ;
            rc = true ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Otherwise, prompt the user                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else {
            createControls() ;
            new Thread ( this ).start() ;
            setVisible ( true ) ;
         }
      } catch ( Exception e ) {
      }
      return( rc ) ;
   }


   /*-----------------------------------------------------------*/
   /*   createControls                                          */          
   /*                                                           */
   /*   Description:                                            */
   /*      Creates and attaches our controls                    */                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void createControls () {
      Container contentPane = getContentPane() ;

      contentPane.setLayout ( null ) ;
      contentPane.add ( st_HELPTEXT ) ;
      contentPane.add ( st_USERID ) ;
      contentPane.add ( ef_USERID ) ;
      contentPane.add ( st_PASSWORD ) ;
      contentPane.add ( ef_PASSWORD ) ;
      contentPane.add ( buttonPanel ) ;
      contentPane.add ( st_IMAGE ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      buttonPanel.add ( pb_HELP ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addKeyListener          ( this ) ;
      pb_CANCEL.addKeyListener      ( this ) ;
      pb_OK.addActionListener       ( this ) ;
      pb_CANCEL.addActionListener   ( this ) ;
      ef_USERID.addActionListener   ( this ) ;
      ef_PASSWORD.addActionListener ( this ) ;
      pb_HELP.addActionListener     ( this ) ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_OK ) ;
      GUISystem.setPreferredButton ( pb_CANCEL ) ;
      GUISystem.setPreferredButton ( pb_HELP ) ;
      st_HELPTEXT.setHorizontalAlignment ( SwingConstants.CENTER ) ;                              
   }



   /*-----------------------------------------------------------*/
   /* layoutControls
    */
   /**
    *    Moves and resizes all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       x    = 10 ;
      int       y    = 45 ;
      int       cx   = GUISystem.getRowHeight() ;
      Dimension size = getSize() ;

      super.doLayout() ;

      st_IMAGE.setBounds ( x, y-4, 44, 44 ) ;
      st_HELPTEXT.setBounds ( x, 0, size.width-20, 45 ) ;
      st_USERID.setBounds ( x+50, y, 75, cx ) ;
      ef_USERID.setBounds ( x+125, y, 100, cx ) ;
      y+=cx ;

      st_PASSWORD.setBounds ( x+50, y, 75, cx ) ;
      ef_PASSWORD.setBounds ( x+125, y, 100, cx ) ;
      y=size.height-58 ;

      buttonPanel.setBounds ( 5, y, size.width, 25 ) ;
   }


   /*-----------------------------------------------------------*/
   /*   actionPerformed                                         */     
   /*                                                           */
   /*   Description:                                            */
   /*      Responds to buttons being pressed                    */                        
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      Nothing                                              */
   /*-----------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( ( e.getSource() == pb_OK ) ||
           ( e.getSource () == ef_PASSWORD ) ||
           ( e.getSource () == ef_USERID ) )
         processOk() ;

      else if ( e.getSource() == pb_CANCEL )
         dispose() ;

      else if ( e.getSource() == pb_HELP )
         HelpSystem.showHelp ( "app"+File.separator+"logon.html" ) ;

   } /* buttonHit */




   /*------------------------------------------------------------*/
   /* processOk
    */
   /**
    *    this method will process our Ok button
    *  hit
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void processOk () {
      if ( saveData() ) {
         rc = true ;
         dispose() ;
      }
   }



   /*-----------------------------------------------------------*/
   /*   saveData                                                */    
   /*                                                           */
   /*   Description:                                            */
   /*      Saves the data on the window                         */                   
   /*                                                           */
   /*   Parameters:                                             */
   /*      None                                                 */
   /*                                                           */
   /*   Returns:                                                */
   /*      TRUE if everything was saved                         */   
   /*-----------------------------------------------------------*/
   public boolean saveData () {
      boolean  rc       = false ;
      String   userId   = ef_USERID.getText() ;
      String   password = new String ( ef_PASSWORD.getPassword() ) ;

      if ( ( userId == null ) || ( userId.length() == 0 ) ) {
         GUISystem.printBox ( this, STR_ERROR, STR_MUST_ENTER_USERID ) ;
         ef_USERID.requestFocus() ;
      } else if ( ( password == null ) || ( password.length() == 0 ) ) {
         GUISystem.printBox ( this, STR_ERROR, STR_MUST_ENTER_PASSWORD ) ;
         ef_PASSWORD.requestFocus() ;
      } else {
         try {
            PropertySystem.putString ( PropertySystem.PROPERTY_USERID,   userId.toUpperCase() ) ;
            PropertySystem.putString ( PropertySystem.PROPERTY_PASSWORD, password.toUpperCase() ) ;
         } catch ( Exception e ) {
            LogSystem.log (1,e,false ) ;
         }

         rc = true ;
      }


      return( rc ) ;
   } /* saveData */




   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    Refreshes all of the data on our window
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData () {

      try {
         if ( PropertySystem.getString ( PropertySystem.PROPERTY_USERID ) != null ) {
            ef_USERID.setText ( PropertySystem.getString ( PropertySystem.PROPERTY_USERID ) ) ;
            ef_PASSWORD.requestFocus() ;
         } else
            ef_USERID.requestFocus() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

   }

   public void keyReleased ( KeyEvent e ) {
   }
   public void keyTyped ( KeyEvent e ) {
   }

   public void keyPressed ( KeyEvent e ) {

      switch ( e.getKeyCode() ) {
         

         case KeyEvent.VK_ENTER :
            if ( e.getSource() == pb_OK )
               processOk() ;
            else if ( e.getSource() == pb_CANCEL ) {
               dispose() ;
            }
            break ;

         case KeyEvent.VK_SPACE :
            if ( e.getSource() == pb_OK )
               processOk() ;
            else if ( e.getSource() == pb_CANCEL ) {
               dispose() ;
            }
            break ;


      }
   }

   public void run () {
      try {
         Thread.currentThread().sleep ( 100 ) ;
         refreshData() ;
      } catch ( Exception e ) {
      }
   }

} /* LogonDlg */
