package com.ibm.nzna.projects.qit.gui ;   
/*-----------------------------------------------------------*/
/*   ValidateDlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ValidateDlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The Validate Dialog is really a window to show the user 
 * progress during a lengthy validation procedure. The window will have
 * an icon and status at the top, with a check list of validation steps
 * following. A progress bar will also be shown at the bottom of the window.
 * <br>
 * Each check list item will be passed as a String array into the constructor.
 * Each String in the array will be created as 1 check list item. The 'validateStep'
 * method will automatically validate the current step by putting a check box next to
 * it and then will advance the progress bar. The 'validateFailed' method will put
 * a red 'X' next to the step that failed.
 *
 * @version 1.0, 4/10/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ValidateDlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ValidateDlg extends JDialog implements AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_STATUS      = null ;
   private JLabel       st_IMAGE       = null ;
   private JProgressBar progressBar    = null ;
   private NavList      navList        = null ;      

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int    step    = 0 ;
   private Vector stepVec = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor takes the parent window,
    *  the list of initial steps, the icon and initial
    *  description
    *
    *  @param parent The parent window
    *  @param stepList An array of strings representing the steps to take
    *  @param icon The ImageIcon to display in the top left of the window
    *  @param status The initial status to show
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ValidateDlg ( Frame parent, String[] stepList, ImageIcon icon, String status ) {
      super ( parent, Str.getStr ( STR_VALIDATE ), false ) ;
      initialize ( stepList, icon, status ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
      setVisible ( true ) ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initilize our GUI
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize ( String[] stepList, ImageIcon icon, String status ) {
      Container contentPane = getContentPane() ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our static controls                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_IMAGE    = new JLabel ( icon ) ;
      st_STATUS   = new JLabel ( status ) ;
      progressBar = new JProgressBar (0, stepList.length) ;
      navList     = new NavList () ;                                            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, create the steps and place the controls in a vec   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      stepVec = new Vector ( stepList.length ) ;
      for ( int i = 0; i < stepList.length; i++ )
         stepVec.addElement ( new JLabel ( stepList[i], ImageSystem.getImageIcon ( this, ImageSystem.VALIDATE_BLANK ), JLabel.LEADING ) ) ;
      navList.add ( stepVec ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navList.setOpaque ( false ) ;
      st_IMAGE.setBorder ( GUISystem.loweredBorder ) ;
      st_STATUS.setVerticalAlignment ( SwingConstants.TOP ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_STATUS ) ;
      contentPane.add ( navList ) ;
      contentPane.add ( progressBar ) ;   


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our size                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 300, 140+((stepList.length+2)*GUISystem.getRowHeight()) ) ;               


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Initialize our variables                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      step=-1 ;
      validateStep() ;

   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all
    *  of our GUI Controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize () ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;

      super.doLayout() ;

      st_IMAGE.setBounds ( x,y, 36, 36 ) ;
      st_STATUS.setBounds ( x+38, y, size.width-((x*2)-38), 36 ) ;
      y+=51 ;

      x=25 ;
      navList.setBounds ( x, y, size.width-(x*2)-5, size.height-60 ) ;

      progressBar.setBounds ( x, size.height-40, size.width-(x*2)-5, rowHeight-5 ) ;

   }





   /*------------------------------------------------------------*/
   /* validateStep
    */
   /**
    *    This method will validate the current step
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void validateStep () {
      if ( step >= 0 ) {

         ((JLabel)stepVec.elementAt(step)).setForeground ( Color.black ) ;
         ((JLabel)stepVec.elementAt(step)).setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.VALIDATE_CHECK ) ) ;
         progressBar.setValue ( step ) ;
      }
      step++ ;
      ((JLabel)stepVec.elementAt(step)).setForeground ( Color.white ) ;
   }




   /*------------------------------------------------------------*/
   /* validateFailed
    */
   /**
    *    This method will fail the current step
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void validateFailed () {
      if ( step >= 0 ) {
         ((JLabel)stepVec.elementAt(step)).setForeground ( Color.red ) ;
         ((JLabel)stepVec.elementAt(step)).setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.VALIDATE_FAIL ) ) ;
         progressBar.setValue ( step ) ;
      }
   }

} /* ValidateDlg */
