package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   ViewerUtil                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewerUtil                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This class holds static methods usefull for painting in Field Viewers 
 *
 *
 * @version 1.0, 5/18/1999 
 * @author  David Lentz
 */
import java.awt.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewerUtil                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewerUtil extends Object implements  AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final int   infoY            = 34 ;


   /*------------------------------------------------------------*/
   /* paintMultiLine
    */
   /**
    *    This method will paint one of the multi line
    *  fields.
    *  
    *
    *  @param g The Graphics Context
    *  @param size The Size of the control
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static int paintMultiLine ( Graphics g, Dimension size, int fontHeight, String[] strArray ) {
      int i = 0 ;
      int x = 8 ;
      int y = infoY ;
      int height = 0 ;

      if ( strArray != null ) {
         try {
            while ( i <= strArray.length ) {
               if ( strArray[i] != null )
                  g.drawString ( strArray[i], x, y ) ;
               height+=fontHeight ;
               y+=fontHeight ;
               i++ ;
            }
         } catch ( ArrayIndexOutOfBoundsException e ) {
         }
      }
      return( height ) ;
   } /* paintMultiLine */

   public static int paintMultiLine ( int y, Graphics g, Dimension size, int fontHeight, String[] strArray ) {
      int i = 0 ;
      int x = 8 ;
      int height = 0 ;

      if ( strArray != null ) {
         try {
            while ( i <= strArray.length ) {
               if ( strArray[i] != null )
                  g.drawString ( strArray[i], x, y ) ;
               y+=fontHeight ;
               height+=fontHeight ;
               i++ ;
            }
         } catch ( ArrayIndexOutOfBoundsException e ) {
         }
      }
      return( height ) ;
   } /* paintMultiLine */


   public static int paintMultiLine ( int x, int y, Graphics g, Dimension size, int fontHeight, String[] strArray ) {
      int i = 0 ;
      int height = 0 ;

      if ( strArray != null ) {
         try {
            while ( i <= strArray.length ) {
               if ( strArray[i] != null )
                  g.drawString ( strArray[i], x, y ) ;
               y+=fontHeight ;
               height+=fontHeight ;
               i++ ;
            }
         } catch ( ArrayIndexOutOfBoundsException e ) {
         }
      }
      return( height ) ;
   } /* paintMultiLine */



   /*------------------------------------------------------------*/
   /* paintTitle
    */
   /**
    *    This method will paint the title of this
    *  field viewer
    *
    *
    *  @param g The Graphics context to paint into
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void paintTitle ( Graphics g, Dimension size, String title  ) {

      g.setColor ( Color.black ) ;
      g.setFont ( FontSystem.smallTitleFont ) ;
      g.drawImage ( ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.SMALL_BACK_TITLE ).getImage(), 0, 0, MainWindow.getInstance() ) ;
      g.drawString ( title, 5, 13) ;
   }


   /*------------------------------------------------------------*/
   /* paintBackground
    */
   /**
    *    This method will paint the background of our
    *  window
    *  
    *
    *  @param g The graphics context
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void paintBackground ( Graphics g, Dimension size ) {                            
      g.fillRect ( 5, 20, size.width-10, size.height-22 ) ;
   }


   /*------------------------------------------------------------*/
   /* drawDescriptValuePair
    */
   /**
    *    This method will draw a description and
    *  value pair
    *  
    *
    *  @param g The graphics context to draw in
    *  @param descript The Description to draw
    *  @param value The value to draw
    *  @param descriptWidth The width of the description. This is where the value will be drawn
    *  @param x,y The coordinates to draw the text at
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void drawDescriptValuePair ( Graphics g, String descript, Object value, int descriptWidth, int x, int y, Font valueFont, Color valueColor ) {
      g.setColor ( Color.black ) ;
      g.setFont ( FontSystem.smallTitleFont ) ;
      g.drawString ( descript, x, y ) ;
      g.setFont ( valueFont ) ;
      g.setColor ( valueColor ) ;

      if ( ( value != null ) && ( value.toString() != null ) )
         g.drawString ( value.toString(), x+descriptWidth, y ) ;
      else
         g.drawString ( "-", x+descriptWidth, y ) ;
   }


   /*------------------------------------------------------------*/
   /* yesNoFromBool
    */
   /**
    *    This method will return the word 'Yes' if
    *  passed TRUE and 'No' if passed false.
    *  
    *
    *  @param b The boolean value to use
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public static String yesNoFromBool ( boolean b ) {
      return( b?Str.getStr ( STR_YES ):Str.getStr ( STR_NO ) ) ;
   }


} /* ViewerUtil */
