package com.ibm.nzna.projects.qit.gui ;
/*-----------------------------------------------------------*/
/*   WarrantyEntryPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WarrantyEntryPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2001                                  */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.qit.product.* ;

/**
 * This panel holds 2 controls. The EntryField and Select Warranty Button. 
 * It can be used to allow the user to enter WARRANTY description. It will auto-populate
 * the entry field if the user stops typing for 1 second.             
 *
 * @version 1.0, 08/3/2001 
 * @author  Venu Sanka
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WarrantyEntryPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class WarrantyEntryPanel extends JPanel implements ActionListener, Runnable, AppConst, ProductConst,
     DataLengths {

    /**
     *  This is the name of the thread that will search for warrantys
     **/
   private static final String THREAD_SEARCH = "SE" ;

   /**
     *  This is the name of the thread that will fire events
     **/
   private static final String THREAD_FIRE_EVENT = "FE" ;

   
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   
   private JTextField ef_WARR          = new JTextField() ;
   private DButton    pb_SELECTWARR    = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This vector holds all registered listeners
     **/
   private Vector listeners = new Vector ( 1 ) ;


   /**
     *  This is the property which holds the last warranty searched for.
     **/
   private String  lastWarrantySearch = "" ;

   /**
     *  If TRUE, the control is currently searching for UserRec objects
     **/
   private boolean inSearch       = false ;

   /**
     *  This is the vector of warrantys that can be selected
     **/
   private Vector  warrantyVec        = null ;

   /**
     *  This is the currently selected warranty record
     **/
   private TypeWarrantyRec  warrantyRec        = null ;

   /**
     *  If TRUE, this property tells the control to show the user's full name, not just the user id
     **/
   private boolean showFullName   = true ;

   /**
     *  This property tells the running thread to continue
     **/
   private boolean continueThread = true ;

   /**
     *  This is a temporary property to hold the current user
     **/
   private String           curTempWarranty     = null ;

   /**
     *  This property tells the control to show managers only
     **/
   private boolean showManagerOnly    = false ;


      
   
   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create the Warranty Panel with
    *  the default description of 'Warranty'
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public WarrantyEntryPanel () {
      init() ;
   }


   /*-----------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void init () {
      createControls () ;
      warrantyVec = (Vector)TypeList.getInstance().getTypeList( TypeList.TYPE_WARRANTY).clone() ;
      new Thread(this, THREAD_SEARCH ).start() ;

   }

   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and add all of our controls
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   private void createControls () {

      setLayout ( null ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECTWARR = new DButton ( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_PERSONEL ) ) ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      
      //setBorder ( GUISystem.loweredBorder ) ;
      pb_SELECTWARR.setFocusPainted ( false ) ;
      //pb_SELECTWARR.setFocusable ( false ) ;
      pb_SELECTWARR.setBorderPainted ( false ) ;
      pb_SELECTWARR.setBackground ( Color.white ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SELECTWARR.addActionListener ( this ) ;
      ef_WARR.addActionListener( this ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      
      setLayout ( new BorderLayout () ) ;
      add ( ef_WARR, BorderLayout.CENTER ) ;
      add ( pb_SELECTWARR, BorderLayout.EAST ) ;

   }


   /*-----------------------------------------------------------*/
   /* getText
    */
   /**
    *    This method will simply return whatever is
    *  in the Warranty Entry Field. It is not guarenteed
    *  what is in the field
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public String getText() {
      return( ef_WARR.getText() ) ;
   }


   /*-----------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of the entry field
    *  
    *  
    *             
    *  @param text The text to set
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setText ( String text ) {
      if ( ( text != null ) && ( text.length() > 0 ) ) {
         curTempWarranty = text ;
         warrantyRec = findWarranty ( text, true ) ;
         refreshWarrantyField() ;
      } else
         ef_WARR.setText ( "" ) ;

   }


   /*-----------------------------------------------------------*/
   /* setShowFullName
    */
   /**
    *    This method will tell the panel to show the user's full name,
    *  If showFullName is TRUE, the panel will
    *  display the warranty's full name.
    *  
    *             
    *  @param onTop TRUE = Show FullName FALSE = Only show user id
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setShowFullName ( boolean showFullName ) {
      this.showFullName = showFullName ;

      refreshWarrantyField() ;

   }


   /*-----------------------------------------------------------*/
   /* getShowFullName
    */
   /**
    *    This method will the ShowFullName property     
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public boolean getShowFullName () {
      return( showFullName ) ;
   }



   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method will perform our thread operations
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
       String operation = Thread.currentThread().getName() ;

       if ( operation.equals (THREAD_SEARCH ))
           search() ;
       else if ( operation.equals ( THREAD_FIRE_EVENT ))
           fireEvent() ;
   }


   /**
    *  This method will search for warrantys
    *
    */
   private void search () {
   
      String curWarranty  = null ;
      String curWarranty1 = null ;

      while ( continueThread ) {

         /* Ok, first sleep for 1 1/2 second and then check the
            entry field's text */
         try {
            Thread.sleep ( 1500 ) ;
         } catch ( Exception e ) {
         }
         curWarranty = ef_WARR.getText() ;

         /* Ok, if both curWarranty and lastWarrantySearch are not null
            and do not equal each other, perform a search */
         if ( ( curWarranty != null ) &&
              ( curWarranty.length() > 0 ) && 
              ( lastWarrantySearch != null ) ) {

            /* Ok, now wait for a bit longer and check it again. If the
               text has changed, loop back up. Otherwise look for the
               name */
            try {
               Thread.sleep ( 300 ) ;
            } catch ( Exception e ) {
            }
            curWarranty1 = ef_WARR.getText() ;


            if ( curWarranty1.equals ( curWarranty ) ) {

               if ( !curWarranty1.equals ( lastWarrantySearch ) ) {
                  lastWarrantySearch = curWarranty1 ;

                  warrantyRec = findWarranty ( curWarranty1 ) ;

                  /* Ok, if we found a record, populate it now */
                  refreshWarrantyField() ;

                  if ( warrantyRec != null )
                      new Thread ( this, THREAD_FIRE_EVENT ).start() ;
               } /* if */

            } /* if */
         } else
            warrantyRec = null ;
      } /* while */
   } /* search */




   /*------------------------------------------------------------*/
   /* findWarranty
    */
   /**
    *    This method will attempt to locate a user
    *  doing a fuzzy search
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private TypeWarrantyRec findWarranty ( String warranty ) {
      return( findWarranty ( warranty, false ) ) ;
   }




   /*------------------------------------------------------------*/
   /* findWarranty
    */
   /**
    *    This method will attempt to locate the warranty
    *  in the application warranty list. If a match is found
    *  then this method will return the warranty record.
    *
    *  @param warranty  warranty  description
    *  @param exactMatch TRUE = only hit on exact matches
    *  @param None
    *
    *  @return TypeWarrantyRec of the warranty found, NULL if not found
    *//*---------------------------------------------------------*/
   private TypeWarrantyRec findWarranty ( String warranty, boolean exactMatch ) {
      TypeWarrantyRec retRec = null ;

      if ( ( warranty != null ) && ( warrantyVec != null ) ) 
          {
            int     i              = 0 ;
            int     length         = warrantyVec.size() ;
            TypeWarrantyRec    compareWarrantyRec = null ;
            Vector  foundVec       = new Vector ( 1, 10 ) ;

            /* Always upper case! */
            warranty = warranty.toUpperCase() ;                    

            /* Ok, walk through the vector and look for the 'warranty' in the description field */
            while ( i < length ) 
                {
                    compareWarrantyRec = (TypeWarrantyRec) warrantyVec.elementAt(i) ;

                if ( compareWarrantyRec != null ) 
                    {

                        if ( exactMatch ) 
                            {
                                if ( compareWarrantyRec.toString().toUpperCase().equals ( warranty ) )
                                foundVec.addElement ( compareWarrantyRec ) ;

                  //else if ( compareUserRec.getName().toUpperCase().equals ( user ) )
                  //   foundVec.addElement ( compareUserRec ) ;
                            } 
                        else {
                                if ( compareWarrantyRec.toString().toUpperCase().startsWith ( warranty ) )
                                foundVec.addElement ( compareWarrantyRec ) ;

                  //else if ( compareUserRec.getName().toUpperCase().startsWith ( user ) ) {
                  //   foundVec.addElement ( compareUserRec ) ;
                            //}
               }
            } /* if */

            i++ ;
         } /* while */

         foundVec.trimToSize() ;

         /* Ok, if we have more than 1 hit, let the user select, otherwise auto-populate the field */
         if ( foundVec.size() > 1 ) 
             {
                Vector vec = null ;


                vec = new SelectWarrantyDlg ( GUISystem.getParentFrame(this), foundVec, false ).getResult() ;

                if ( ( vec != null ) && ( vec.size() > 0 ) )
                    compareWarrantyRec = (TypeWarrantyRec) vec.elementAt(0) ;

                    retRec = this.warrantyRec = compareWarrantyRec ;

                    refreshWarrantyField() ;

             } 
         else if ( foundVec.size() == 1 )
            retRec = (TypeWarrantyRec) foundVec.elementAt(0) ;


      } /* if */

      return( retRec ) ;
   } /* findUser */













   /*------------------------------------------------------------*/
   /* refreshWarrantyField
    */
   /**
    *    This method will refresh the data that is
    *  in the warranty field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshWarrantyField() {

      if ( warrantyRec != null ) {

         if ( showFullName ) 
             ef_WARR.setText ( warrantyRec.toString() ) ;
         //else
         //   ef_USERID.setText ( userRec.getUserId() ) ;

         try {
            ef_WARR.setSelectionStart ( lastWarrantySearch.length() ) ;
            ef_WARR.setSelectionEnd ( ef_WARR.getText().length() ) ;
         } catch ( Exception e ) {}

         lastWarrantySearch = ef_WARR.getText() ;

         } /* if */

      else
         ef_WARR.setText ( "" ) ;
      
   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when one of our
    *  buttons is pressed
    *  
    *
    *  @param e The actionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {     

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   DButton Events                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() instanceof DButton ) {
            DButton button = (DButton) e.getSource() ;
            if ( pb_SELECTWARR == button ) {
               Vector vec = null ;

               vec = new SelectWarrantyDlg ( GUISystem.getParentFrame(this), false ).getResult() ;
               if ( ( vec != null ) && ( vec.size() > 0 ) )
                  warrantyRec = (TypeWarrantyRec) vec.elementAt(0) ;
               refreshWarrantyField() ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   TextField Events                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() instanceof JTextField ) {
            String text = ef_WARR.getText() ;

            warrantyRec = findWarranty ( text ) ;
            refreshWarrantyField() ;
         }
      } catch ( Exception ex ) {
         LogSystem.log ( 1, ex) ;
      }
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will stop our thread that is monitoring
    *  the entry field. It must be called when you are through with
    *  this control.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      continueThread = false ;
   }



   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name if it is valid.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return user The name if valid, NULL if not valid
    *//*---------------------------------------------------------*/
   public String getName () {
      String retStr = null ;

      if ( warrantyRec != null )
         retStr = warrantyRec.toString() ;

      return( retStr ) ;
   }




   /*------------------------------------------------------------*/
   /* getWarrantyRec
    */
   /**
    *    This method will return the warranty record
    *  if it is valid, If not valid, a lookup will be performed
    *  asking the user to select a description
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public TypeWarrantyRec getWarrantyRec () {
      if ( warrantyRec == null )
         warrantyRec = findWarranty ( ef_WARR.getText() ) ;

      return( warrantyRec ) ;
   }





   /*------------------------------------------------------------*/
   /* getWarrantyRecNoLookup
    */
   /**
    *    This method will return the warrantyRec selected
    *  by the user or nothing. It will not perform
    *  a second lookup
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return UserRec
    *//*---------------------------------------------------------*/
   public TypeWarrantyRec getWarrantyRecNoLookup () {

      return( warrantyRec ) ;
   }


   /*------------------------------------------------------------*/
   /* getDescript
    */
   /**
    *    This method will return the warranty description if it is valid.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return warranty The Warranty description if valid, NULL if not valid
    *//*---------------------------------------------------------*/
   public String getDescript () {
      String retStr = null ;

      if ( warrantyRec != null )
          retStr = warrantyRec.toString() ;

      return( retStr ) ;
   }



   /*------------------------------------------------------------*/
   /* requestFocus
    */
   /**
    *    This method has been subclassed to pass the
    *  focus to our entry field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void requestFocus () {
      ef_WARR.requestFocus() ;
   }





   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable
    *  our component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enable ) {
      ef_WARR.setEnabled ( enable ) ;
      pb_SELECTWARR.setEnabled ( enable ) ;
      super.setEnabled ( enable ) ;
   }


   /**
    *  This method will fire a 'userSelected' event to all registered listeners
    */
   private void fireEvent () {
       int i = 0 ;
       int size = listeners.size() ;

       while ( i < size ) {
           ((WarrantyEntryPanelListener)listeners.elementAt(i)).warrantySelected( warrantyRec ) ;
           i++  ;
       }
   }



   public void addWarrantyEntryPanelListener ( WarrantyEntryPanelListener listener )
   {
       listeners.addElement( listener ) ;
   }


   public void removeWarrantyEntryPanelListener ( WarrantyEntryPanelListener listener ) {
       listeners.removeElement( listener ) ;
   }


} /* WarrantyEntryPanel */

