package com.ibm.nzna.projects.qit.gui ;     
/*-----------------------------------------------------------*/
/*   WhatsNewPopUP                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: WhatsNewPopUP                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This window is the 'What's New' pop up that shows new users
 * what is new and allows them to read extensivly about changed
 * or new functionality.<br><br>
 *
 * There is only 1 instance of the WhatsNewPopUp that is ever
 * created. Since this is true, <b>never</b> use the Constructor. Instead, use the 'getInstance()'
 * method to obtain the instance.
 *
 *
 * @version 1.0, 11/24/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   WhatsNewPopUP                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class WhatsNewPopUp extends JWindow implements ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Members                                          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   //private static String contentWrapper = "<html><body><center><table align=center border=0 width=330><tr><td><font face=\"Helvetica, Arial\" size=2><center>CONTENT</center></font></td></tr></table></center></body></html>" ;
   private static String contentWrapper = "<html><body><center>CONTENT</center></body></html>" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Color        backgroundColor    = new Color ( 255, 255, 255 ) ;
   private Color        backgroundColor1   = new Color ( 235, 235, 235 ) ;
   private Color        backgroundColor2   = new Color ( 215, 215, 215 ) ;
   private Color        backgroundColor3   = new Color ( 195, 195, 195 ) ;
   private Image        mascotImage        = null ;
   private JLabel       st_WHATSNEW        = null ;
   private JLabel       st_TITLE           = null ;
   private JLabel       content            = null ;
   private DButton      pb_NEXT            = null ;
   private DButton      pb_CLOSE           = null ;
   private JCheckBox    ck_NEVER_SHOW      = null ;

   private        Frame         relativeParent  = null ;
   private        Vector        listeners       = new Vector ( 10, 10 )  ;
   private static WhatsNewPopUp instance        = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor creates and initialize
    *  the popup but will not show the pop up
    *  
    *
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public WhatsNewPopUp ( ) {

      if ( instance == null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Load our images                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         mascotImage = ImageSystem.getImage ( this, ImageSystem.WHATSNEW_MASCOT ) ;      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create Controls                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         st_WHATSNEW   = new JLabel      ( Str.getStr ( STR_WHATS_NEW ) ) ;
         st_TITLE      = new JLabel      ( "Quest Input v3.0" ) ;
         content       = new JLabel      ( "" ) ;
         pb_NEXT       = new DButton     ( Str.getStr ( STR_WHAT_ELSE_IS_NEW ) ) ;
         pb_CLOSE      = new DButton     ( Str.getStr ( STR_CLOSE ) ) ;
         ck_NEVER_SHOW = new JCheckBox   ( Str.getStr ( STR_NEVER_SHOW_AGAIN ) ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         content.setOpaque ( false ) ;
         st_WHATSNEW.setOpaque ( false ) ;
         st_TITLE.setOpaque ( false ) ;
         st_TITLE.setFont ( new Font ( "Helv", Font.BOLD, 20 ) );
         st_WHATSNEW.setFont ( FontSystem.smallTitleFont ) ;
         st_WHATSNEW.setHorizontalAlignment ( st_WHATSNEW.CENTER ) ;
         st_TITLE.setHorizontalAlignment ( st_TITLE.CENTER ) ;
         content.setVerticalAlignment ( SwingConstants.TOP ) ;
         st_TITLE.setVerticalAlignment ( SwingConstants.TOP ) ;
         setBackground ( backgroundColor ) ;
         getContentPane().setBackground ( backgroundColor ) ;       
         ck_NEVER_SHOW.setOpaque ( false ) ;
         pb_NEXT.setBackground ( new Color ( 240, 240, 240 ) ) ;
         pb_CLOSE.setOpaque ( false ) ;
         pb_NEXT.setFocusPainted ( false ) ;
         pb_CLOSE.setFocusPainted ( false ) ;    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add listeners                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_NEXT.addActionListener ( this ) ;
         pb_CLOSE.addActionListener ( this ) ;
         ck_NEVER_SHOW.addActionListener ( this ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our controls                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         getContentPane().setLayout ( null ) ;
         getContentPane().add ( st_WHATSNEW ) ;
         getContentPane().add ( st_TITLE ) ;
         getContentPane().add ( content ) ;
         getContentPane().add ( pb_NEXT ) ;
         getContentPane().add ( ck_NEVER_SHOW ) ;
         getContentPane().add ( pb_CLOSE ) ;

         instance = this ;
         setSize ( 350, 355 ) ;
      } else
         instance.setVisible ( true ) ;
   }





   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return the actual instance of
    *  this pop up
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static WhatsNewPopUp getInstance () {
      if ( instance == null )
         new WhatsNewPopUp() ;
      return( instance ) ;
   }



   /*------------------------------------------------------------*/
   /* paintComponent
    */
   /**
    *    This method will paint our component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void paint ( Graphics g ) {
      Dimension size = getSize() ;
      Graphics2D g2   = (Graphics2D) g ;

      /* Anti Alias  */
      g2.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON ) ;

      super.paint(g) ;
      g.setColor ( Color.black ) ;
      g.drawRect ( 0, 0, size.width-1, size.height-1 ) ;
      g.setColor ( backgroundColor3 ) ;
      g.drawRect ( 1, 1, size.width-3, size.height-3 ) ;
      g.setColor ( backgroundColor2 ) ;
      g.drawRect ( 2, 2, size.width-5, size.height-5 ) ;
      g.setColor ( backgroundColor1 ) ;
      g.drawRect ( 3, 3, size.width-6, size.height-6 ) ;

      g.drawImage ( mascotImage, 4, 4, this ) ;
   }






   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position our
    *  control
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       y         = 5 ;

      super.doLayout() ;
      st_WHATSNEW.setBounds ( 4, y, size.width-8, rowHeight ) ;
      y+=rowHeight+20 ;

      st_TITLE.setBounds ( 4, y, size.width-8, rowHeight*2) ;
      y+=(rowHeight*2)+5 ;

      content.setBounds ( 4, y, size.width-8, size.height-y-(rowHeight*2)-5 ) ;
      y=(size.height-(rowHeight*4)-2) ;

      pb_NEXT.setBounds ( (size.width/2)-55, y, 110, rowHeight ) ;
      y+=((rowHeight)-2) ;

      y=size.height-(rowHeight+10 ) ;
      pb_CLOSE.setBounds ( size.width-84, y, 75, rowHeight ) ;

      ck_NEVER_SHOW.setBounds ( 4, y, 150, rowHeight ) ;

   }





   /*------------------------------------------------------------*/
   /* setContent
    */
   /**
    *    This method will set the content of
    *  the What's New Pop Up
    *  
    *
    *  @param content The new content
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setContent ( String newContent ) {
      content.setText ( Text.replaceInStr ( contentWrapper, "CONTENT", newContent ) ) ;
   }





   /*------------------------------------------------------------*/
   /* setRelativeParent
    */
   /**
    *    This method will set the 'relative' parent of 
    *  this PopUp. Any calls to 'setBounds', 'setLocation'
    *  will result in the position being altered to be relative to this parent
    *
    *  @param Frame parent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRelativeParent ( Frame parent ) {
      this.relativeParent = parent ;
   }





   /*------------------------------------------------------------*/
   /* setBounds
    */
   /**
    *    This method will set the location and
    *  size of the window, taking into account
    *  the relative parent
    *
    *  @param x The x position
    *  @param y The y position
    *  @param width The width
    *  @param height The height
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBounds ( int x, int y, int width, int height ) {
      super.setBounds ( translateX(x),
                        translateY(y),
                        width,
                        height ) ;
   }


   /*------------------------------------------------------------*/
   /* setBounds
    */
   /**
    *    This method will set the location and
    *  size of the window, taking into account
    *  the relative parent
    *
    *  @param rectangle The rectangle giving the position and dimensions
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBounds ( Rectangle rectangle ) {
      super.setBounds ( translateX(rectangle.x),
                        translateY(rectangle.y),
                        rectangle.width,
                        rectangle.height ) ;
   }





   /*------------------------------------------------------------*/
   /* setLocation
    */
   /**
    *    This method will set the location
    *  of this window, taking into account the
    *  relative parent
    *
    *  @param x The x position
    *  @param y The y position
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLocation ( int x, int y ) {
      Dimension prefSize = getSize() ;

      setBounds ( x, y, prefSize.width, prefSize.height ) ;

   }


   /*------------------------------------------------------------*/
   /* setLocation
    */
   /**
    *    This method will set the location
    *  of this window, taking into account the
    *  relative parent
    *
    *  @param pt The Point of the location
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setLocation ( Point pt ) {
      setLocation ( pt.x, pt.y ) ;

   }





   /*------------------------------------------------------------*/
   /* translateX
    */
   /**
    *    This method will translate an X Coordinate
    *  so that it is relative to the relativeParent
    *  
    *
    *  @param x The X Coordinate
    *  @param None
    *  @param None
    *
    *  @return The translated coordinate
    *//*---------------------------------------------------------*/
   public int translateX ( int x ) {
      int returnX = x ;

      if ( relativeParent != null ) {
         Point parentLoc = relativeParent.getLocation() ;

         returnX=(x+parentLoc.x) ;
      }
      return( returnX ) ;
   }



   /*------------------------------------------------------------*/
   /* translateY
    */
   /**
    *    This method will translate a Y Coordinate
    *  so that it is relative to the relativeParent
    *  
    *
    *  @param y The Y Coordinate
    *  @param None
    *  @param None
    *
    *  @return The translated coordinate
    *//*---------------------------------------------------------*/
   public int translateY ( int y ) {
      int returnY = y ;

      if ( relativeParent != null ) {
         Point parentLoc = relativeParent.getLocation() ;

         returnY=(y+parentLoc.y) ;
      }

      return( returnY ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when action events
    *  are fired on us
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE ) {
         listeners.removeAllElements() ;
         setVisible ( false ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ck_NEVER_SHOW                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == ck_NEVER_SHOW ) {
         if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_NEVER_SHOW_WHATS_NEW ) ) {
            PropertySystem.putBool ( PropertySystem.PROPERTY_SHOW_WHATS_NEW, false ) ;
            dispose() ;
         } else
            ck_NEVER_SHOW.setSelected ( false ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_NEXT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_NEXT ) {
         fireWhatsNewEvent() ;
      }

   }






   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set the title we are displaying
    *  
    *  
    *
    *  @param title The new title
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitle ( String title ) {
      st_TITLE.setText ( title ) ;
   }





   /*------------------------------------------------------------*/
   /* addWhatsNewPopUpListener
    */
   /**
    *    This method will add a WhatsNewPopUpListener
    *  to this list. In the event the user closes the 
    *  panel all listeners will be nullified.
    *
    *  @param listener The WhatsNewPopUpListener to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addWhatsNewPopUpListener ( WhatsNewPopUpListener listener ) {
      listeners.addElement ( listener ) ;
   }





   /*------------------------------------------------------------*/
   /* fireWhatsNewEvent
    */
   /**
    *    THis method will fire a 'whats new' event
    *  to all registered listeners
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void fireWhatsNewEvent () {
      if ( ( listeners != null ) && ( listeners.size() > 0 ) ) {
         int i    = 0 ;
         int size = listeners.size() ;

         while ( i < size ) {
            ((WhatsNewPopUpListener)listeners.elementAt(i)).whatsNewPerformed ( this ) ;
            i++ ;
         }
      }
   }




   /*------------------------------------------------------------*/
   /* removeWhatsNewPopUpListener
    */
   /**
    *    This method will remove a WhatsNewListener
    *  from being notified
    *  
    *
    *  @param listener The WhatsNewListener to remove
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void removeWhatsNewPopUpListener ( WhatsNewPopUpListener listener ) {
      listeners.removeElement ( listener ) ;
   }
} /* WhatsNewPopUP */
