package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpBodyReplace                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpBodyReplace                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.fieldchanger.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel holds two entry field and a description. 
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpBodyReplace                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpBodyReplace extends PopUpPanel implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_DESCRIPT       = new JLabel     ( Str.getStr ( STR_BODY_REPLACE )  ) ;
   private JTextField   ef_ORIGINAL       = new JTextField ( "" ) ;
   private JTextField   ef_NEW            = new JTextField ( "" ) ;
   private JLabel       st_ORIGINAL       = new JLabel     ( Str.getStr ( STR_ORIGINAL_TEXT ) ) ;
   private JLabel       st_NEW            = new JLabel     ( Str.getStr ( STR_NEW_TEXT ) ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int            rowHeight     = 18 ;
   private boolean        requireData   = false ;
   private BodyReplaceRec replaceRec    = null ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpBodyReplacePanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpBodyReplace ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;
      setLayout ( null ) ;
      add ( st_DESCRIPT ) ;
      add ( st_ORIGINAL ) ;
      add ( ef_ORIGINAL ) ;
      add ( st_NEW ) ;
      add ( ef_NEW ) ;
      ef_NEW.addActionListener ( this ) ;
      ef_ORIGINAL.addActionListener ( this ) ;
   }


   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will be subclassed to size and
    *  position our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize();
      int       rowHeight = 18 ;
      int       y         = 0 ;

      super.doLayout() ;

      st_DESCRIPT.setBounds ( 0, 0, size.width, rowHeight*4 ) ;
      y+=rowHeight*4 ;

      st_ORIGINAL.setBounds ( 0, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      ef_ORIGINAL.setBounds ( 0, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      st_NEW.setBounds ( 0, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      ef_NEW.setBounds ( 0, y, size.width, rowHeight ) ;
      y+=rowHeight ;

   }



   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      st_DESCRIPT.setText ( descript ) ;
   }



   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData () {
      String originalStr = ef_ORIGINAL.getText() ;
      String newStr      = ef_NEW.getText() ;

      replaceRec = null ;

      if ( ( originalStr == null ) || ( originalStr.length() == 0 ) )
         GUISystem.printBox ( STR_ERROR, STR_MUST_ENTER_VALUE ) ;
      else if ( ( newStr == null ) || ( newStr.length() == 0 ) )
         GUISystem.printBox ( STR_ERROR, STR_MUST_ENTER_VALUE ) ;
      else {

         if ( ( !Text.containsHTML ( newStr ) ) &&
              ( !Text.containsHTML ( originalStr ) ) &&
              ( originalStr.length() > 1 ) &&
              ( newStr.length() > 1 ) ) {
            replaceRec = new BodyReplaceRec () ;

            replaceRec.newStr      = newStr ;
            replaceRec.originalStr = originalStr ;
         } else
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_SEARCH_REPLACE_NO_HTML ) ) ;
      }

      return( replaceRec ) ;
   }



   /*-----------------------------------------------------------*/
   /* setRequired
    */
   /**
    *    This method will tell the entry field to either
    *  require data or not. Data is always required for this pop up
    *  
    *
    *  @param requireData = TRUE if we should not close until data is entered
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRequired ( boolean requireData ) {
      this.requireData = true ;
   }



   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object dataObj ) {
      if ( dataObj != null ) {
         if ( dataObj instanceof BodyReplaceRec ) {
            BodyReplaceRec replaceRec = (BodyReplaceRec) dataObj ;

            ef_NEW.setText ( "" ) ;
            ef_ORIGINAL.setText ( replaceRec.originalStr ) ;
         } else {
            ef_NEW.setText ( "" ) ;
            ef_ORIGINAL.setText ( "" ) ;
         }
      } else {
         ef_NEW.setText ( "" ) ;
         ef_ORIGINAL.setText ( "" ) ;
      }

   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      ef_ORIGINAL.requestFocus() ;
   }



   /*-----------------------------------------------------------*/
   /* entryFieldHit
    */
   /**
    *    This method will respond to the entry
    *  field being used
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void actionPerformed ( ActionEvent e ) {
      if ( saveData() != null )
         popUpDlg.okHit ( PopUpDlg.PB_OK ) ;
   }




   /*------------------------------------------------------------*/
   /* hiding
    */
   /**
    *    This method has been subclassed to remove
    *  any attributes that the entry field has set so
    *  the next time we are shown, we are shown as the default entry field
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void hiding () {
   }



} /* PopUpBodyReplace */
