package com.ibm.nzna.projects.qit.gui.popup ;                                               
/*-----------------------------------------------------------*/
/*   PopUpBrandFamilyMachineModel                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpBrandFamilyMachineModel        */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
/**
 * This class is 1 Pop up that contains 2 others (the PopUpBrandFamily and PopUpMachineModel)
 * in a JTabbedPane. This allows the user to select the Brand/Family/Machine/Model in one popup 
 *
 *
 * @version 1.0, 11/22/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpBrandFamilyMachineModel                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpBrandFamilyMachineModel extends PopUpPanel implements PopUpBrandFamilyListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTabbedPane       tabbedPane        = null ;
   private PopUpBrandFamily  brandFamily       = null ;
   private PopUpMachineModel machineModel      = null ;
   private boolean           required          = false ;

   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpBrandFamily Panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpBrandFamilyMachineModel ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      tabbedPane = new JTabbedPane() ;
      brandFamily = new PopUpBrandFamily ( popUpDlg ) ;
      machineModel = new PopUpMachineModel ( popUpDlg ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout ()  ) ;
      tabbedPane.addTab ( Str.getStr ( STR_BRAND_FAMILY ),         brandFamily ) ;
      tabbedPane.addTab ( Str.getStr ( STR_MACHINETYPE_MODEL ),    machineModel ) ;
      add ( tabbedPane, BorderLayout.CENTER ) ;

   }





   /*------------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh our data with the
    *  object passed in
    *  
    *
    *  @param object The object to refresh with. Only ProdDocRecs are allowed
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object object ) {
      if ( object == null )
         object = new ProdDocRec() ;

      if ( object instanceof ProdDocRec ) {
         brandFamily.refreshData ( object ) ;
         brandFamily.setPopUpBrandFamilyListener ( this ) ;
         machineModel.setPopUpBrandFamilyListener ( this ) ;
         machineModel.refreshData ( object ) ;
      }
   }

   public void brandFamilyChanged ( Object source ) {
      if ( source == brandFamily )
         machineModel.refreshData ( brandFamily.getWorkingRec() ) ;
      else
         brandFamily.refreshData ( machineModel.getWorkingRec() ) ;
   }

   public void showBrandFamilyPage () {
      tabbedPane.setSelectedComponent ( brandFamily ) ;
   }

   public void showMachineModelPage () {
      tabbedPane.setSelectedComponent ( machineModel ) ;
   }

   public Object saveData () {
      if ( required ) {
         if ( brandFamily.saveData() == null ) 
            showBrandFamilyPage() ;
         else if ( machineModel.saveData() == null ) 
            showMachineModelPage() ;
         else
            return ( machineModel.saveData() ) ;
      } else
         return( machineModel.saveData() ) ;

      return( null ) ;
   }

   public void setRequired ( boolean required ) {
      brandFamily.setRequired ( required ) ;
      machineModel.setRequired ( required ) ;
      this.required = required ;
   }
} /* PopUpBrandFamilyMachineModel */
