package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpDateRange                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpDateRange                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel holds date spin buttons 
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  Maria Paredes
 * @author  David Lentz - Changed Spin Buttons to entry fields
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpDateRange                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpDateRange extends PopUpPanel implements AppConst, FocusListener, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_START          = new JLabel ( "" ) ;
   private JLabel       st_END            = new JLabel ( "" ) ;
   private JTextArea    st_DESCRIPT       = new JTextArea  ( ) ;
   private DateEntryField ef_STARTDATE      = new DateEntryField ( ) ;
   private DateEntryField ef_STOPDATE       = new DateEntryField ( ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       rowHeight     = 18 ;
   private String[]  dates         = new String[ 2 ] ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpDateRange Panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpDateRange ( PopUpDlg popUpDlg ) {
      super ( popUpDlg ) ;
      setLayout ( null ) ;

      add ( st_DESCRIPT ) ;
      add ( ef_STARTDATE ) ;
      add ( ef_STOPDATE ) ;
      add ( st_START ) ;
      add ( st_END ) ;

      st_START.setText ( Str.getStr ( STR_ENTER_START_DATE ) ) ;
      st_END.setText ( Str.getStr ( STR_ENTER_STOP_DATE ) ) ;
      WinUtil.setJTextAreaAsLabel ( st_DESCRIPT ) ;
      st_DESCRIPT.setFont ( FontSystem.defaultFont ) ;

      ef_STARTDATE.addActionListener ( this ) ;
      ef_STOPDATE.addActionListener ( this ) ;

   }

   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will be subclassed to size and
    *  position our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int y = 10 ;
      Dimension size = getSize();

      super.doLayout() ;

      st_DESCRIPT.setBounds ( 0, 0, size.width, rowHeight*4 ) ;
      y+=rowHeight*4 ;

      st_START.setBounds ( 18, y, size.width/2, rowHeight ) ;
      y+=rowHeight ;
      ef_STARTDATE.setBounds ( 20, y, size.width/2, rowHeight ) ;
      y+=(rowHeight*2) ;

      st_END.setBounds ( 18, y, size.width/2, rowHeight ) ;
      y+=rowHeight ;
      ef_STOPDATE.setBounds ( 20, y, size.width/2, rowHeight ) ;
   }



   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      st_DESCRIPT.setText ( descript ) ;
   }



   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData () {
      Object retObj = null ;

      dates[ 0 ] = ( String ) ef_STARTDATE.getText () ;
      dates[ 1 ] = ( String ) ef_STOPDATE.getText () ;

      retObj = dates ;

      return( retObj ) ;

   }


   /*------------------------------------------------------------*/
   /* focusLost
    */
   /**
    *    This method will be called when the
    *  entry field for the status date loses focus. We
    *  will use this time to format the date correctly
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusLost ( FocusEvent e ) {
      if ( e.getSource() instanceof JTextField ) {
         EntrySystem.fixDateField ( (JTextField) e.getSource() ) ;
      }
   }

   public void focusGained ( FocusEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    called when the user hits 'Enter' 
    *  in one of our fields
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      EntrySystem.fixDateField ( (JTextField)e.getSource() ) ;
      popUpDlg.okHit ( PopUpDlg.PB_OK ) ;
   }





   /*------------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will set the focus to the
    *  first date field
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus () {
      ef_STARTDATE.requestFocus() ;
   }

} /* PopUpDateRange */
