package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpDlg                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpDlg                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.io.* ;
import java.awt.event.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.border.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This dialog will be used to generate Pop Up Windows used to enter data 
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpDlg                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpDlg extends JDialog implements PopUpConst, WindowListener, ActionListener, AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_SETFOCUS  = "SetFocus" ;
   private static final Dimension wizSize  = new Dimension ( 495, 375 ) ;
   private static final Dimension prefSize = new Dimension ( 400, 280 ) ;
   public static final int PB_OK     = 0 ;
   public static final int PB_FINISH = 1 ;
   public static final int PB_CANCEL = 2 ;
   public static final int PB_NEXT   = 3 ;
   public static final int PB_BACK   = 4 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton pb_OK          = new DButton ( Str.getStr ( STR_OK ) ) ;
   private DButton pb_CANCEL      = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
   private DButton pb_BACK        = new DButton ( Str.getStr ( STR_BACK ) ) ;
   private DButton pb_FINISH      = new DButton ( Str.getStr ( STR_FINISH ) ) ;
   private DButton pb_NEXT        = new DButton ( Str.getStr ( STR_NEXT ) ) ;
   private JLabel  st_WIZIMAGE    = null ;
   private JPanel  pnl_COMPHOLD   = new JPanel() ;
   private JPanel  pnl_CARDLAYOUT = new JPanel () ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int          popupType            = POPUP ;
   private int          panelType            = -1 ;
   private Object       data                 = null ;
   private PopUpPanel   curPanel             = null ;
   private String       descript             = "" ;
   private PopUpPanel[] panelArray           = new PopUpPanel[MAX_POPUP_PANELS] ;
   private boolean      forceDestroy         = false ;
   private int          curPanelInd          = 0 ;
   private int          id                   = 0 ;
   private boolean      firstRefresh         = false ;
   private boolean      forceLayout          = false ;
   private Dimension    prevSize             = new Dimension ( 0, 0 ) ;
   private boolean      enabled              = true ;


   private transient PopUpListener listener     = null ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpDlg but not show it.
    *  You must call the RefreshControls method
    *  to show the PopUpDlg.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpDlg ( Frame parent ) {
      super ( parent, "", false ) ;

      setDefaultCloseOperation ( DO_NOTHING_ON_CLOSE ) ;
      addWindowListener ( this ) ;
      //setResizable ( false ) ;

      /* Set some properties  */
      try {
         st_WIZIMAGE = new JLabel ( ImageSystem.getImageIcon ( this, "skins"+
                                                               File.separator+
                                                               PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                                                               File.separator+
                                                               "setup_wizard.gif" ) ) ;
         st_WIZIMAGE.setBorder ( new BevelBorder ( BevelBorder.LOWERED ) ) ;
      } catch ( Exception e ) {
      }

      createControls() ;

      setSize ( wizSize ) ;
      WinUtil.centerChildInParent ( this, parent ) ;
      setVisible ( false ) ;
   }


   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and attach all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {

      getContentPane().setLayout ( new BorderLayout() ) ;
      getContentPane().add ( pnl_COMPHOLD, BorderLayout.CENTER ) ;
      pnl_COMPHOLD.setLayout ( null ) ;
      pnl_CARDLAYOUT.setLayout ( new CardLayout() ) ;
      pnl_COMPHOLD.add ( pnl_CARDLAYOUT ) ;
      pnl_COMPHOLD.add ( st_WIZIMAGE ) ;
      pnl_COMPHOLD.add ( pb_OK ) ;
      pnl_COMPHOLD.add ( pb_BACK ) ;
      pnl_COMPHOLD.add ( pb_NEXT ) ;
      pnl_COMPHOLD.add ( pb_FINISH ) ;
      pnl_COMPHOLD.add ( pb_CANCEL ) ;

      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_NEXT.addActionListener ( this ) ;
      pb_FINISH.addActionListener ( this ) ;
      pb_BACK.addActionListener ( this ) ;
      pb_BACK.setVisible ( false ) ;
   }



   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = pnl_COMPHOLD.getSize() ;

      super.doLayout() ;
      forceLayout=true ;
      if ( ( prevSize.width != size.width ) ||
           ( prevSize.height != size.height ) ||
           ( forceLayout ) ) {

         /* First, make sure we don't force each layout!  */
         forceLayout = false ;
         prevSize = size ;

         switch ( popupType ) {
            case WIZARD :

               if ( pb_NEXT.isVisible () ) {
                  pb_NEXT.setBounds   ( wizSize.width-(83*2)-30, wizSize.height-56, 78, 23 ) ;
                  if ( pb_BACK.isVisible() )
                     pb_BACK.setBounds   ( wizSize.width-(83*3)-30, wizSize.height-56, 78, 23 ) ;
               } else {
                  if ( pb_BACK.isVisible() )
                     pb_BACK.setBounds   ( wizSize.width-(83*2)-30, wizSize.height-56, 78, 23 ) ;
               }

               pb_FINISH.setBounds ( wizSize.width-103, wizSize.height-56, 90, 23 ) ;
               pb_CANCEL.setBounds ( wizSize.width-103, wizSize.height-56, 90, 23 ) ;

               st_WIZIMAGE.setVisible ( true ) ;
               st_WIZIMAGE.setLocation ( 5, 15 ) ;
               st_WIZIMAGE.setSize ( st_WIZIMAGE.getPreferredSize() ) ;
               pnl_CARDLAYOUT.setBounds ( 145, 30, wizSize.width-160, wizSize.height-128 ) ;
               setSize ( wizSize ) ;
               break ;

            case POPUP :

               pb_OK.setVisible ( true ) ; // In case it was turned off last time
               pb_CANCEL.setVisible ( true ) ;
               st_WIZIMAGE.setVisible ( false ) ;
               pb_OK.setBounds ( 10, prefSize.height-30, 70, 23 ) ;
               pb_CANCEL.setBounds ( 85, prefSize.height-30, 95, 23 ) ;
               pnl_CARDLAYOUT.setBounds ( 10, 10, prefSize.width-20, prefSize.height-50 ) ;
               setSize ( prefSize.width, prefSize.height+30 ) ;
               break ;

         }
      }
   }


   /*-----------------------------------------------------------*/
   /* refreshPopup
    */
   /**
    *    This method will refresh the pop up window
    *  to the new type and data passed in
    *  
    *
    *  @param id The new ID for the Pop Up Window
    *  @param Type The Integer Type of the pop up display panel ( POPUP_ENTRYFIELD )
    *  @param data The data object to pass to the display panel
    *  @param PType The Pop Up Type ( WIZARD or POPUP )
    *  @param title The title that should show in the title bar
    *  @param descript The description that should be shown in the panel
    *  @param listener The new listener to notify
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpPanel refreshPopUp ( int id, int panelType, Object data, int popupType, String title, String descript, PopUpListener listener ) {
      boolean forceRefresh = ( popupType != this.popupType ) ;


      this.id = id ;
      this.panelType = panelType ;
      this.data      = data;
      this.popupType = popupType ;
      setTitle ( title ) ;
      this.descript  = descript ;
      this.listener  = listener ;

      pb_OK.setText     ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL.setText ( Str.getStr ( STR_CANCEL ) ) ;
      pb_BACK.setText   ( Str.getStr ( STR_BACK ) ) ;
      pb_NEXT.setText   ( Str.getStr ( STR_NEXT ) ) ;
      pb_FINISH.setText ( Str.getStr ( STR_FINISH ) ) ;

      /* Set our panel */
      if ( setView ( panelType ) ) {
         if ( curPanel != null ) {
            curPanel.setDescript ( descript ) ;
            curPanel.setReadOnly ( false ) ;
            curPanel.refreshData ( data ) ;
         }
      }

      switch ( popupType ) {
         case WIZARD : 
            pb_OK.setVisible ( false ) ;
            break ;

         case POPUP :  {
               Dimension size = getSize() ;

               pb_OK.setVisible ( true ) ;
               pb_NEXT.setVisible ( false ) ;
               pb_BACK.setVisible ( false ) ;
            }
            break ;
      }

      ((CardLayout)pnl_CARDLAYOUT.getLayout()).show ( pnl_CARDLAYOUT, ""+curPanelInd ) ;

      setVisible ( true ) ;
      requestFocus() ;

      if ( forceRefresh ) {
         firstRefresh = false ;
         doLayout() ;
         pnl_COMPHOLD.revalidate() ;
      }



      new Thread(this, THREAD_SETFOCUS ).start() ;
      return( curPanel ) ;
   }




   /*-----------------------------------------------------------*/
   /* setView
    */
   /**
    *    This method will set the view of our window
    *  
    *  
    *
    *  @param view The panelType to switch to
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean setView ( int panelInd ) {
      int     i      = 0 ;
      int     length = MAX_POPUP_PANELS ;
      boolean rc     = true ;

      if ( panelArray[curPanelInd] != null ) {
         panelArray[curPanelInd].hiding() ;
         //   panelArray[curPanelInd].setVisible ( false ) ;
      }

      switch ( panelInd ) {
         case POPUP_ENTRYFIELD :
            if ( panelArray[POPUP_ENTRYFIELD] == null ) {
               panelArray[POPUP_ENTRYFIELD] = new PopUpEntryField ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_ENTRYFIELD] , ""+POPUP_ENTRYFIELD) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_DATERANGE :
            if ( panelArray[POPUP_DATERANGE] == null ) {
               panelArray[POPUP_DATERANGE] = new PopUpDateRange ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_DATERANGE] , ""+POPUP_DATERANGE) ;
               firstRefresh=false ;
            }
            break ;


         case POPUP_MLE :
            if ( panelArray[POPUP_MLE] == null ) {
               panelArray[POPUP_MLE] = new PopUpMLE ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_MLE] , ""+POPUP_MLE) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_TYPELIST :
            if ( panelArray[POPUP_TYPELIST] == null ) {
               panelArray[POPUP_TYPELIST] = new PopUpTypeList ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_TYPELIST] , ""+POPUP_TYPELIST) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_LOCALE :
            if ( panelArray[POPUP_LOCALE] == null ) {
               panelArray[POPUP_LOCALE] = new PopUpLocale ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_LOCALE] , ""+POPUP_LOCALE) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_LIST :
            if ( panelArray[POPUP_LIST] == null ) {
               panelArray[POPUP_LIST] = new PopUpList ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_LIST] , ""+POPUP_LIST) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_STATUS :
            if ( panelArray[POPUP_STATUS] == null ) {
               panelArray[POPUP_STATUS] = new PopUpStatus ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_STATUS] , ""+POPUP_STATUS) ;
               firstRefresh=false ;
            }
            break ;


         case POPUP_DESCRIPT :
            if ( panelArray[POPUP_DESCRIPT] == null ) {
               panelArray[POPUP_DESCRIPT] = new PopUpDescript ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_DESCRIPT] , ""+POPUP_DESCRIPT) ;
               firstRefresh=false ;
            }
            break ;
         case POPUP_PUBLICATION :
            if ( panelArray[POPUP_PUBLICATION] == null ) {
               panelArray[POPUP_PUBLICATION] = new PopUpPublication ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_PUBLICATION] , ""+POPUP_PUBLICATION) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_BODYREPLACE :
            if ( panelArray[ panelInd ] == null ) {
               panelArray[ panelInd ] = new PopUpBodyReplace ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[ panelInd ], ""+ panelInd) ;
               firstRefresh=false ;
            }
            break ;
         case POPUP_FIELDLIST :
            if ( panelArray[POPUP_FIELDLIST] == null ) {
               panelArray[POPUP_FIELDLIST] = new PopUpDocFieldList ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_FIELDLIST] , ""+POPUP_FIELDLIST) ;
               firstRefresh=false ;
            }
            break ;            

         case POPUP_CHECK :
            if ( panelArray[POPUP_CHECK] == null ) {
               panelArray[POPUP_CHECK] = new PopUpCheck ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_CHECK] , ""+POPUP_CHECK) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_SPINBUTTON :
            if ( panelArray[POPUP_SPINBUTTON] == null ) {
               panelArray[POPUP_SPINBUTTON] = new PopUpDateRange ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_SPINBUTTON] , ""+POPUP_SPINBUTTON) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_USERIDENTRY :
            if ( panelArray[POPUP_USERIDENTRY] == null ) {
               panelArray[POPUP_USERIDENTRY] = new PopUpUserEntry ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_USERIDENTRY] , ""+POPUP_USERIDENTRY) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_SPLITTYPELIST :
            if ( panelArray[POPUP_SPLITTYPELIST] == null ) {
               panelArray[POPUP_SPLITTYPELIST] = new PopUpSplitTypeList ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_SPLITTYPELIST] , ""+POPUP_SPLITTYPELIST) ;
               firstRefresh=false ;
            }
            break ;

         case POPUP_EMAILABLE :
            if ( panelArray[POPUP_EMAILABLE] == null ) {
               panelArray[POPUP_EMAILABLE] = new PopUpEmail ( this ) ;
               pnl_CARDLAYOUT.add ( panelArray[POPUP_EMAILABLE] , ""+POPUP_EMAILABLE) ;
               firstRefresh=false ;
            }
            break ;

         default :
            GUISystem.printBox ( "Programmer Error", "You passed an unsupported POPUP_XXX type of '"+panelInd+"'. Please see quest.base.Const.java for available panel types.");
            break ;

      }

      curPanel    = panelArray[panelInd] ;
      curPanelInd = panelInd ;
      return( rc ) ;
   }




   /*-----------------------------------------------------------*/
   /* buttonHit
    */
   /**
    *    This method responds to buttons being
    *  pressed
    *  
    *
    *  @param id The ID of the button
    *  @param button The instance of the button
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      boolean fireEvent = false ;
      int     command   = 0 ;

      if ( enabled ) {
         if ( e.getSource() == pb_FINISH )
            okHit ( PB_FINISH ) ;

         else if ( e.getSource() == pb_NEXT )
            okHit ( PB_NEXT ) ;

         else if ( e.getSource() == pb_OK )
            okHit( PB_OK ) ;

         else if ( e.getSource() == pb_BACK ) {
            fireEvent = true ;
            command = PB_BACK ;
         }

         else if ( e.getSource() == pb_CANCEL )
            cancelHit() ;

         if ( ( listener != null ) && ( fireEvent ) )
            listener.popupAction ( id, this, command, null ) ;
      }
   }

   public void cancelHit () {
      if ( listener != null )
         listener.popupAction ( id, this, PB_CANCEL, null ) ;
      setVisible ( false ) ;
   }

   public void okHit ( int command ) {
      Object retObj = null ;

      if ( !curPanel.readOnly ) {
         retObj = curPanel.saveData() ;
         switch ( command ) {
            case PB_NEXT :
               break ;

            case PB_CANCEL :
               setVisible ( false ) ;
               break ;

            case PB_OK :
            case PB_FINISH :
               if ( retObj == null ) {
                  if ( !curPanel.requireData )
                     setVisible ( ( popupType == WIZARD ) && ( command != PB_FINISH ) ) ;
               } else
                  setVisible ( ( popupType == WIZARD ) && ( command != PB_FINISH ) ) ;
               break ;
         }

      } else
         setVisible ( ( popupType == WIZARD ) && ( command != PB_FINISH ) ) ;

      if ( listener != null )
         listener.popupAction ( id, this, command, retObj ) ;
   }


   /*-----------------------------------------------------------*/
   /* destroyWin
    */
   /**
    *    I over rode destroyWin so that the close button
    *  on the window would not close the window. The only
    *  way to close the window is to press a button
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean destroyWin () {
      if ( enabled ) {
         cancelHit() ;
         return( forceDestroy ) ;
      }
      return( false ) ;
   }



   /*-----------------------------------------------------------*/
   /* showBackButton
    */
   /**
    *    This method will either hide or show the back
    *  button
    *  
    *             
    *  @param show TRUE = Show Back Button, FALSE = Do Not Show BACK Button
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void showBackButton ( boolean show ) {
      pb_BACK.setVisible ( show ) ;
      doLayout() ;
   }

   /*-----------------------------------------------------------*/
   /* showNextButton
    */
   /**
    *    This method will either hide or show the back
    *  button
    *  
    *             
    *  @param show TRUE = Show Next Button, FALSE = Do Not Show NEXT Button
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void showNextButton ( boolean show ) {
      pb_NEXT.setVisible ( show ) ;
      doLayout() ;
   }







   /*-----------------------------------------------------------*/
   /* forceDestroy
    */
   /**
    *    This is the only method that will actually 
    *  close the pop up.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void forceDestroy() {
      //forceDestroy = true ;
      //dispose() ;
   }


   /*-----------------------------------------------------------*/
   /* showFinish
    */
   /**
    *    This method will either show the Finish button,
    *  or the Cancel button. The two buttons toggle each other.
    *  If one is off, the other is on.
    *             
    *  @param show TRUE - Show the Finish button, FALSE - Show the Cancel button
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void showFinishButton ( boolean show ) {
      pb_FINISH.setVisible ( show ) ;
      pb_CANCEL.setVisible ( !show ) ;
      doLayout() ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will currently only set the focus
    *  to a control on our curPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName() ;

      if ( ( name.equals ( THREAD_SETFOCUS ) ) && ( curPanel != null ) ) {
         try {
            Thread.sleep(100) ;
         } catch ( InterruptedException e ) {
         }
         curPanel.setFocus() ;
      }
   }





   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable
    *  all buttons and the closing of the pop up
    *  window
    *
    *  @param enable TRUE = enable the window, FALSE = disable
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enable ) {
      pb_OK.setEnabled ( enable ) ;
      pb_CANCEL.setEnabled ( enable ) ;
      pb_BACK.setEnabled ( enable ) ;
      pb_FINISH.setEnabled ( enable ) ;
      pb_NEXT.setEnabled ( enable ) ;
      this.enabled = enable ;

   }

   public void windowOpened ( WindowEvent e ) {
   }
   public void windowClosing ( WindowEvent e ) {
      if ( enabled )
         setVisible ( false ) ;
   }
   public void windowClosed ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e ) {
   }
   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ) {
   }


} /* PopUpDlg */
