package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpEntryField                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpEntryField                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.*;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel holds an entry field and a description. 
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpEntryField                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpEntryField extends PopUpPanel implements AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea       st_DESCRIPT       = new JTextArea() ;
   private JTextField      ef_DESCRIPT       = new JTextField ( "" ) ;
   private JPanel          pnl_DESCRIPT      = new JPanel() ;
   private SpellCheckPanel pnl_SPELLCHECK    = new SpellCheckPanel() ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static MaskDocument defaultDoc = new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 1024 ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       rowHeight     = 18 ;
   private String    mask          = null ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpEntryFieldPanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpEntryField ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;
      setLayout ( null ) ;

      /* set some properties  */
      st_DESCRIPT.setLineWrap ( true ) ;
      st_DESCRIPT.setWrapStyleWord ( true ) ;
      st_DESCRIPT.setEditable ( false ) ;
      st_DESCRIPT.setOpaque ( false ) ;
      st_DESCRIPT.setFont ( FontSystem.defaultFont ) ;
      //ef_DESCRIPT.setPreferredSize ( new Dimension ( size.width-10, DALSystem.getEntryFieldHeight() ) ) ;

      pnl_DESCRIPT.setLayout ( new BorderLayout () ) ;
      pnl_DESCRIPT.add ( st_DESCRIPT,BorderLayout.CENTER ) ;
      add ( pnl_DESCRIPT ) ;
      add ( pnl_SPELLCHECK ) ;
      add ( ef_DESCRIPT ) ;
      pnl_SPELLCHECK.setTextComponent ( ef_DESCRIPT ) ;
      ef_DESCRIPT.addActionListener ( this ) ;
   }


   public void doLayout () {
      Dimension textSize = null ;
      Dimension size     = getSize() ;

      super.doLayout() ;
      pnl_DESCRIPT.setBounds ( 0, 0, size.width-20, 21 ) ;
      textSize = pnl_DESCRIPT.getPreferredSize() ;

      pnl_DESCRIPT.setBounds ( 0, 0, size.width, textSize.height ) ;
      pnl_SPELLCHECK.setBounds ( 0, textSize.height+2, pnl_SPELLCHECK.getPreferredSize().width, pnl_SPELLCHECK.getPreferredSize().height ) ;
      ef_DESCRIPT.setBounds ( 0, textSize.height+2+pnl_SPELLCHECK.getPreferredSize().height, size.width-30, rowHeight ) ;
      
   }       

   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      st_DESCRIPT.setText ( descript ) ;
   }



   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData () {
      String str = ef_DESCRIPT.getText() ;

      if ( ( ( str == null ) || ( str.length() < 1 ) ) && ( requireData ) ) {
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_ENTER_VALUE ) ) ;
         str = null ;
      } else if ( str == null )
         str = "" ;

      /* Auto format the date */
      if ( ( mask != null ) && ( mask.equals ( EntrySystem.getStringDateMask() ) ) ) {
         str = CDate.format4DigitYear ( str, CDate.DATE_DATE ) ;
         ef_DESCRIPT.setText ( str ) ;
      } /* if */


      /* If this is a date, check the date to make sure it is good */
      if ( ( str != null ) && 
           ( mask != null ) && 
           ( mask.equals ( EntrySystem.getStringDateMask() ) ) ) {
         if ( CDate.checkDate ( str, CDate.DATE_DATE ) != 0 ) {
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_DATE_ENTERED_WRONG ) ) ;
            str = null ;
         } /* if DDate */
      } /* if */


      return( str ) ;
   }


   /*-----------------------------------------------------------*/
   /* getEntryField
    */
   /**
    *    This method will return the entry field instance
    *  to the caller.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Instance of an EntryField
    *//*---------------------------------------------------------*/
   public JTextField getEntryField () {
      return( ef_DESCRIPT ) ;
   }



   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object dataObj ) {
      if ( dataObj != null ) {
         if ( dataObj instanceof String ) {
            String tempStr = (String) dataObj ;

            /* Remove line breaks */
            tempStr = Text.replaceAllStrInStr ( tempStr, "\n", " " ) ;
            tempStr = Text.replaceAllStrInStr ( tempStr, new Character ( (char)13 ).toString(), " " ) ;
            tempStr = Text.replaceAllStrInStr ( tempStr, new Character ( (char)10 ).toString(), " " ) ;

            ef_DESCRIPT.setText ( tempStr ) ;

         } else
            ef_DESCRIPT.setText ( "" ) ;
      } else
         ef_DESCRIPT.setText ( "" ) ;

   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      ef_DESCRIPT.requestFocus() ;
   }

   /*-----------------------------------------------------------*/
   /* entryFieldHit
    */
   /**
    *    This method will respond to the entry
    *  field being used
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void actionPerformed ( ActionEvent e ) {

      if ( saveData() != null )
         popUpDlg.okHit ( PopUpDlg.PB_OK ) ;
   }




   /*------------------------------------------------------------*/
   /* hiding
    */
   /**
    *    This method has been subclassed to remove
    *  any attributes that the entry field has set so
    *  the next time we are shown, we are shown as the default entry field
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void hiding () {
      ef_DESCRIPT.setDocument ( defaultDoc ) ;
      mask=null ;
      ef_DESCRIPT.setText ( "" ) ;
   }

   /*-----------------------------------------------------------*/
   /* setReadOnly
    */
   /**
    *    This method will change this pop up to be
    *  read only
    *  
    *             
    *  @param readOnly TRUE = make read only, false Make no read only
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setReadOnly ( boolean readOnly ) {
      super.setReadOnly ( readOnly ) ;
      ef_DESCRIPT.setEnabled ( !readOnly ) ;
   }


   public void setMask ( String mask ) {
      String text = ef_DESCRIPT.getText() ;
      
      this.mask = mask ;
      ef_DESCRIPT.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, mask, ef_DESCRIPT ) ) ;
      if ( text != null )
         ef_DESCRIPT.setText(text) ;
      ef_DESCRIPT.setToolTipText ( "" ) ;
   }

   public void setMaximumLength ( int max ) {
      String text = ef_DESCRIPT.getText() ;

      ef_DESCRIPT.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, max ) ) ;
      if ( text != null )
         ef_DESCRIPT.setText(text) ;
      ef_DESCRIPT.setToolTipText ( Str.getStr ( STR_MAXIMUM_LENGTH )+": "+max ) ;
   }

   public void setFormat ( int format, int max ) {
      String text = ef_DESCRIPT.getText() ;

      ef_DESCRIPT.setDocument ( new MaskDocument ( format, max ) ) ;
      if ( text != null )
         ef_DESCRIPT.setText(text) ;
      ef_DESCRIPT.setToolTipText ( Str.getStr ( STR_MAXIMUM_LENGTH )+": "+max ) ;
   }

} /* PopUpEntryField */
