package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpList                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpList                           */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
/**
 * This panel holds an entry field and a description. 
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpList                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpList extends PopUpPanel implements ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea    st_DESCRIPT   = new JTextArea() ;
   private JList        dataList      = new JList() ;
   private JScrollPane  scr_PANE      = new JScrollPane ( dataList ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpListPanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpList ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;
      setLayout ( new BorderLayout ( ) ) ;

      /* set some properties  */
      st_DESCRIPT.setLineWrap ( true ) ;
      st_DESCRIPT.setWrapStyleWord ( true ) ;
      st_DESCRIPT.setEditable ( false ) ;
      st_DESCRIPT.setOpaque ( false ) ;
      st_DESCRIPT.setFont ( FontSystem.defaultFont ) ;
      dataList.setPrototypeCellValue("WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW"); // 50 W's
      dataList.setFixedCellWidth(450);    

      add ( st_DESCRIPT, BorderLayout.NORTH ) ;
      add ( scr_PANE, BorderLayout.CENTER ) ;

      setDefaults () ;
   }


   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      st_DESCRIPT.setText ( descript ) ;
   }



   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData () {
      Object retObj = null ;

      if ( dataList.getSelectionMode() == ListSelectionModel.MULTIPLE_INTERVAL_SELECTION ) {
         Object[] values = dataList.getSelectedValues() ;

         if ( ( values == null ) || ( values.length == 0 ) ) {

            if ( requireData )
               GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
            else
               retObj = new Vector(10) ;

         } else {
            int    i = 0 ;
            int    length = values.length ;
            Vector tempVec = new Vector ( values.length ) ;

            while ( i < length )
               tempVec.addElement(values[i++]) ;
            retObj = tempVec ;
         }

      } else {
         retObj = dataList.getSelectedValue() ;

         if ( retObj == null ) {
            if ( requireData )
               GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
         }
      }

      return( retObj ) ;

   }



   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object dataObj ) {
      dataList.removeAll() ;

      if ( dataObj != null ) {
         if ( dataObj instanceof Vector ) {
            dataList.setListData ( (Vector) dataObj ) ;
         } else if ( dataObj instanceof String[] ) {
            dataList.setListData ( (Object []) dataObj ) ;
         }
      }
   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      dataList.requestFocus() ;
   }


   /*-----------------------------------------------------------*/
   /* setSelection
    */
   /**
    *    This method will set what is currently selected
    *  
    *  
    *
    *  @param data The object to be selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelection ( Object data ) {
      dataList.setSelectedValue ( data, true ) ;
   }

   /*-----------------------------------------------------------*/
   /* setSelection
    */
   /**
    *    This method will set what is currently selected
    *  
    *  
    *
    *  @param data The object to be selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelection ( Vector data ) {
      if ( data != null ) {
         for ( int dataIndex = 0 ; dataIndex < data.size() ; dataIndex++ ) {
            dataList.setSelectedValue ( data.elementAt( dataIndex ) , true ) ;
         }
      }
   }


   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() instanceof JList )
         popUpDlg.okHit ( PopUpDlg.PB_OK ) ;
   }

   public void setDefaults () {
      dataList.clearSelection() ;
      setMultiSelect ( false ) ;
   }

   public void setMultiSelect ( boolean allowMulti ) {
      if ( allowMulti )
         dataList.setSelectionMode ( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION ) ;
      else
         dataList.setSelectionMode ( ListSelectionModel.SINGLE_SELECTION ) ;
   }


} /* PopUpList */
