package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpLocale                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpLocale                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This panel holds 2 Containers, and a World Wide button. It will allow
 * the user to select multiple geographies and multiple countries from a
 * list and return the list in a vector of TypeCountryCodeRecs.
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpLocale                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpLocale extends PopUpPanel implements ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_GEOGRAPHY      = new JLabel ( Str.getStr ( STR_GEOGRAPHY ) ) ;
   private JLabel       st_ASSOCCOUNTRY   = new JLabel ( Str.getStr ( STR_ASSOC_COUNTRY ) ) ;
   private MultiList    cnr_GEO           = null ;
   private MultiList    cnr_COUNTRYCODE   = null ;
   private JCheckBox    ck_WORLDWIDE      = new JCheckBox ( Str.getStr ( STR_WORLDWIDE ) ) ;
   private DButton      pb_DESELECT       = new DButton ( Str.getStr ( STR_DESELECT ) )  ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       rowHeight     = 18 ;
   private LocaleRec locRec        = null ;
   private Vector    cnrVec        = null ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpLocalePanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpLocale ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;

      setLayout ( null ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_GEO         = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_COUNTRYCODE = new MultiList ( GUISystem.getFontUtil() ) ;
      cnr_GEO.setForeground ( Color.black ) ;
      cnr_COUNTRYCODE.setForeground ( Color.black ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_GEO.addActionListener ( this ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_GEO.setColumnWidth ( 0, 200 ) ;
      cnr_COUNTRYCODE.setColumnWidth ( 0, 200 ) ;
      cnr_GEO.setRowHeight( 18 ) ;
      cnr_COUNTRYCODE.setRowHeight ( 18 ) ;            

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add all of our controls                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( st_GEOGRAPHY ) ;
      add ( st_ASSOCCOUNTRY ) ;
      add ( cnr_GEO ) ;
      add ( cnr_COUNTRYCODE ) ;
      add ( ck_WORLDWIDE ) ;
      add ( pb_DESELECT ) ;

      /* We require the country and geo */
      requireData = true ;

      ck_WORLDWIDE.addActionListener ( this ) ;
      pb_DESELECT.addActionListener ( this ) ;
   }



   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will be subclassed to size and
    *  position our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize();
      int       y         = 0 ;
      int       center    = size.width/2 ;

      st_GEOGRAPHY.setBounds ( 0, 0, center-5, rowHeight ) ;
      st_ASSOCCOUNTRY.setBounds( center+5, 0, center-5, rowHeight ) ;
      y+=rowHeight ;

      cnr_GEO.setBounds ( 0, y, center-5, 120 ) ;
      cnr_COUNTRYCODE.setBounds ( center+5, y, center-5, 120 ) ;
      y+=130 ;

      ck_WORLDWIDE.setBounds ( 5, y, center-10, 25 ) ;
      pb_DESELECT.setBounds ( center+10, y, center-10, 25 ) ;

   }



   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
   }



   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData() {
      Object retRec = null ;

      if ( requireData ) {
         if ( ( locRec.isWorldWide() == false ) &&
              ( locRec.countryCount() == 0 ) )
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
         else {
            retRec = locRec ;
         }
      }

      return( retRec ) ;
   }



   /*-----------------------------------------------------------*/
   /* setRequired
    */
   /**
    *    This method will tell the entry field to either
    *  require data or not
    *  
    *
    *  @param requireData = TRUE if we should not close until data is entered
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRequired ( boolean requireData ) {
      this.requireData = requireData ;
   }

   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object dataObj ) {
      cnr_COUNTRYCODE.removeAll () ;

      TypeList.refreshList ( cnr_GEO, TypeList.TYPE_GEOGRAPHY, null ) ;
      if ( dataObj != null ) {
         locRec = new LocaleRec((LocaleRec) dataObj) ;
         updateCountryList() ;
      } else
         locRec = new LocaleRec() ;
   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      cnr_GEO.requestFocus() ;
   }

   /*-----------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method responds to buttons being pressed
    *  
    *  
    *
    *  @param e The ActionEvent
    *
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == ck_WORLDWIDE ) {
         if ( ck_WORLDWIDE.isSelected() ) {
            locRec.setWorldWide(true) ;
            updateCountryList() ;
         } else {
            locRec = new LocaleRec() ;
            cnr_COUNTRYCODE.removeAll() ;
         }
      }

      else if ( e.getSource() == pb_DESELECT )
         deselectCountry() ;

      else if ( e.getSource() == cnr_GEO ) {
         locRec = new SelectCountryDlg ( (TypeGeoRec) cnr_GEO.getSelectedItem(), 
                                         locRec ).getCountryList() ;
         updateCountryList() ;
      }


   }




   /*-----------------------------------------------------------*/
   /* deselectCountry
    */
   /**
    *    This method will deselect and remove the
    *  countries that are selected in the country
    *  container.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void deselectCountry () {
      Vector vec    = cnr_COUNTRYCODE.getSelection() ;

      if ( ( cnrVec != null ) && ( vec != null ) && ( vec.size() > 0 ) ) {
         int i       = 0 ;
         int length  = vec.size() ;

         while ( i < length ) {
            cnr_COUNTRYCODE.remove ( (MultiListRow)vec.elementAt(i) ) ;
            locRec.removeCountry ( (TypeCountryCodeRec) vec.elementAt(i) ) ;
            i++ ;
         }
         ck_WORLDWIDE.setSelected ( false ) ;
         vec = null ;
         System.gc() ;

      }

   }


   /*-----------------------------------------------------------*/
   /* updateCountryList
    */
   /**
    *    This method will update our cnr_COUNTRYCODE
    *  Container. It will insert all of the countries in the
    *  locRec.countryVec, and all countries associed to geo's in locRec.geoVec
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void updateCountryList () {
      cnrVec = locRec.getCountryList() ;

      /* Update the container */
      cnr_COUNTRYCODE.removeAll () ;
      if ( cnrVec != null )
         cnr_COUNTRYCODE.add ( cnrVec ) ;

      ck_WORLDWIDE.setSelected ( locRec.isWorldWide() ) ;
   } /* updateCountryList */


   /*-----------------------------------------------------------*/
   /* setReadOnly
    */
   /**
    *    This method will change this pop up to be
    *  read only
    *  
    *             
    *  @param readOnly TRUE = make read only, false Make no read only
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setReadOnly ( boolean readOnly ) {
      super.setReadOnly ( readOnly ) ;
      ck_WORLDWIDE.setEnabled ( !readOnly ) ;
      pb_DESELECT.setEnabled ( !readOnly ) ;
   }


   public void hiding () {
      locRec = null ;
      ck_WORLDWIDE.setSelected ( false ) ;
   }


} /* PopUpLocale */
