package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpMLE                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpMLE                            */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;

/**
 * This panel holds an entry field and a description. 
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpMLE                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpMLE extends PopUpPanel implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea       st_DESCRIPT       = new JTextArea () ;     
   private DTextArea       mle_DESCRIPT      = null ;
   private JPanel          pnl_DESCRIPT      = new JPanel ( ) ;
   private SpellCheckPanel pnl_SPELL         = new SpellCheckPanel () ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       rowHeight     = 18 ;
   private int       maxTextLen    = 32000 ;
   private String    descript      = "" ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpMLEPanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpMLE ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;
      setLayout ( null ) ;

      /* Create the Swing Components we will use */
      pnl_DESCRIPT.setLayout ( new BorderLayout() ) ;
      pnl_DESCRIPT.add ( st_DESCRIPT, BorderLayout.CENTER ) ;

      mle_DESCRIPT      = new DTextArea() ;

      st_DESCRIPT.setLineWrap ( true ) ;
      st_DESCRIPT.setWrapStyleWord ( true ) ;
      st_DESCRIPT.setEditable ( false ) ;
      st_DESCRIPT.setOpaque ( false ) ;
      st_DESCRIPT.setFont ( FontSystem.defaultFont ) ;


      add ( pnl_DESCRIPT ) ;
      add ( pnl_SPELL ) ;
      add ( mle_DESCRIPT ) ;
      mle_DESCRIPT.setFont ( FontSystem.defaultFont ) ;

      pnl_SPELL.setTextComponent ( mle_DESCRIPT.getTextArea() ) ;
   }



   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will be subclassed to size and
    *  position our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size     = getSize() ;
      Dimension textSize = null ;

      super.doLayout() ;
      pnl_DESCRIPT.setBounds ( 0, 0, size.width, 21 ) ;
      textSize = pnl_DESCRIPT.getPreferredSize() ;

      pnl_DESCRIPT.setBounds ( 0, 0, size.width, textSize.height ) ;
      pnl_SPELL.setBounds    ( 0, textSize.height+2, pnl_SPELL.getPreferredSize().width, pnl_SPELL.getPreferredSize().height ) ;
      mle_DESCRIPT.setBounds ( 0, textSize.height+2+pnl_SPELL.getPreferredSize().height, size.width-20, size.height-textSize.height-4-pnl_SPELL.getPreferredSize().height ) ;
   }



   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      String text = descript+".  "+Str.getStr ( STR_MAXIMUM_LENGTH ) ;

      this.descript = descript ;
      if ( maxTextLen > 0 )
         text+=""+maxTextLen ;
      else
         text+="?" ;

      st_DESCRIPT.setText ( text ) ;
   }



   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData () {
      String str = mle_DESCRIPT.getText() ;

      if ( str != null )
         str = str.trim() ;

      if ( ( ( str == null ) || ( str.length() < 1 ) ) && ( requireData ) ) {
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_ENTER_VALUE ) ) ;
         str = null ;
      } else if ( str == null )
         str = "" ;


      return( str ) ;
   }




   /*-----------------------------------------------------------*/
   /* getEntryField
    */
   /**
    *    This method will return the entry field instance
    *  to the caller.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Instance of an EntryField
    *//*---------------------------------------------------------*/
   public JTextArea getMLE () {
      return( mle_DESCRIPT.getTextArea() ) ;
   }



   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object dataObj ) {
      if ( dataObj != null ) {
         if ( dataObj instanceof String ) {
            mle_DESCRIPT.setText ( (String) dataObj ) ;
         }
         else
            mle_DESCRIPT.setText ( "" ) ;
      } else
         mle_DESCRIPT.setText ( "" ) ;

   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      mle_DESCRIPT.requestFocus() ;
   }




   /*------------------------------------------------------------*/
   /* setMaxTextLen
    */
   /**
    *    This method will set our maximum text length
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setMaximumLength ( int maxTextLen ) {
      mle_DESCRIPT.getTextArea().setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, maxTextLen ) ) ;

      mle_DESCRIPT.setMaximumText ( maxTextLen ) ;
      this.maxTextLen = maxTextLen ;
      setDescript ( descript ) ;
   }


   /*-----------------------------------------------------------*/
   /* setReadOnly
    */
   /**
    *    This method will change this pop up to be
    *  read only
    *  
    *             
    *  @param readOnly TRUE = make read only, false Make no read only
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setReadOnly ( boolean readOnly ) {
      super.setReadOnly ( readOnly ) ;
      mle_DESCRIPT.setEnabled ( !readOnly ) ;
   }

} /* PopUpMLE */
