package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpMachineModel                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpMachineModel                   */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This panel allows the associations of machines and models
 *
 *
 * @version 1.0, 6/2/99
 * @author  David Lentz
 */
import java.awt.* ;
import java.sql.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.prodmaint.* ;
import com.ibm.nzna.projects.qit.doc.docmaint.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpMachineModel                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpMachineModel extends PopUpPanel implements ProductConstants, BookmarkDlgListener, ListSelectionListener, ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final FontUtil  fontUtil      = GUISystem.getFontUtil() ;
   public static final Insets    buttonMargin  = new Insets ( 0, 0, 0 ,0  ); 
   public static final String[]  prodListTitle = { 
      Str.getStr ( STR_PRODUCT_NUMBER  ), 
      Str.getStr ( STR_DESCRIPTION ),
      Str.getStr ( STR_STATUS   ),
      Str.getStr ( STR_STATUS_DATE )
   } ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel      st_MACHINES      = new JLabel ( Str.getStr ( STR_MACHINE ) ) ;
   private JLabel      st_MODELS        = new JLabel ( Str.getStr ( STR_PRODUCTS ) ) ;
   private JLabel      st_STATUS        = new JLabel ( "" ) ;
   private JLabel      st_SELECTPRODS   = new JLabel ( Str.getStr ( STR_SELECT_PRODS ) ) ;
   private JLabel      st_SELECTHELP    = new JLabel ( Str.getStr ( STR_IDIOT_DIRECTIONS ) ) ;
   private DList       cnr_MACHINES     = new DList () ;
   private DList       cnr_MODELS       = new DList () ;
   private MultiList   cnr_PRODUCTS     = new MultiList ( fontUtil, prodListTitle ) ;
   private JScrollPane scr_MACHINES     = new JScrollPane ( cnr_MACHINES ) ;
   private JScrollPane scr_MODELS       = new JScrollPane ( cnr_MODELS ) ;
   private DButton     pb_ASSOCIATEALL  = new DButton ( Str.getStr ( STR_ASSOC_TO_ALL ) ) ;
   private DButton     pb_ASSOCIATENONE = new DButton ( Str.getStr ( STR_DIRECT_ASSOC ) ) ;
   private DButton     pb_SELECT        = new DButton ( Str.getStr ( STR_SELECT ) ) ;
   private DButton     pb_CANCEL        = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
   private DButton     pb_RIGHT         = new DButton ( ">>" ) ;
   private DButton     pb_RIGHTALL      = new DButton ( ">>" ) ;
   private DButton     pb_LEFT          = new DButton ( "<<" ) ;
   private DButton     pb_BOOKMARKS     = new DButton ( Str.getStr ( STR_BOOKMARKS ), ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.DOCUMENT_SMALL ) ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ProdDocRec               workRec         = null ;
   private ProdDocRec               originalRec     = null ;
   private boolean                  prodListVisible = false ;
   private PopUpBrandFamilyListener listener        = null ;

   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpLocalePanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpMachineModel ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;

      setLayout ( null ) ;
      add ( st_SELECTHELP ) ;
      add ( st_MACHINES ) ;
      add ( st_MODELS ) ;
      add ( st_STATUS ) ;
      add ( st_SELECTPRODS ) ;
      add ( scr_MACHINES ) ;
      add ( pb_LEFT ) ;
      add ( pb_RIGHT ) ;
      add ( pb_RIGHTALL ) ;
      add ( scr_MODELS ) ;
      add ( st_STATUS ) ;
      add ( pb_ASSOCIATEALL ) ;
      add ( pb_ASSOCIATENONE ) ;
      add ( cnr_PRODUCTS ) ;
      add ( pb_BOOKMARKS ) ;
      add ( pb_SELECT ) ;
      add ( pb_CANCEL ) ;

      /* Hide some controls */
      setProdListVisible ( false ) ;
      pb_ASSOCIATEALL.setVisible ( false ) ;
      pb_ASSOCIATENONE.setVisible ( false ) ;

      /* Adjust some properties */
      pb_RIGHTALL.setForeground ( Color.blue ) ;
      pb_RIGHTALL.setMargin ( buttonMargin ) ;
      pb_RIGHT.setMargin ( buttonMargin ) ;
      pb_LEFT.setMargin ( buttonMargin ) ;
      cnr_PRODUCTS.setColumnWidth ( 1, 450 ) ;


      /* Add our listeners */
      cnr_MACHINES.addListSelectionListener ( this ) ;
      pb_RIGHTALL.addActionListener ( this ) ;
      pb_RIGHT.addActionListener ( this ) ;
      pb_LEFT.addActionListener ( this ) ;
      pb_ASSOCIATEALL.addActionListener ( this ) ;
      pb_ASSOCIATENONE.addActionListener ( this ) ;
      pb_SELECT.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      cnr_MACHINES.addActionListener ( this ) ;
      pb_BOOKMARKS.addActionListener ( this ) ;


      revalidate() ;
   }


   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will be subclassed to size and
    *  position our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size            = getSize();
      int       y               = 0 ;
      int       rowHeight       = GUISystem.getRowHeight() ;
      int       half            = (size.width/2)-10 ;
      int       halfHeight      = size.height/2 ;
      int       x               = 10 ;
      int       halfButtonWidth = 25 ;

      super.doLayout() ;

      st_SELECTHELP.setBounds ( x, 0, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      st_MACHINES.setBounds ( x, y, half-halfButtonWidth-(x*2), rowHeight ) ;
      st_MODELS.setBounds ( half+halfButtonWidth+x, y, half-halfButtonWidth-x, rowHeight ) ;
      y+=rowHeight ;

      scr_MACHINES.setBounds ( x, y, half-halfButtonWidth-(x*2), size.height-80 ) ;
      scr_MODELS.setBounds ( half+halfButtonWidth+x, y, half-halfButtonWidth-x, size.height-105 ) ;
      pb_BOOKMARKS.setBounds ( half+halfButtonWidth+x, size.height-60, half-halfButtonWidth-x, rowHeight ) ;

      pb_LEFT.setBounds ( half-halfButtonWidth, halfHeight-50,     halfButtonWidth*2, 25 ) ;
      pb_RIGHT.setBounds ( half-halfButtonWidth, halfHeight-15,    halfButtonWidth*2, 25 ) ;
      pb_RIGHTALL.setBounds ( half-halfButtonWidth, halfHeight+20, halfButtonWidth*2, 25 ) ;

      st_STATUS.setBounds ( half+halfButtonWidth+x, size.height-30, half-halfButtonWidth-x, 25 ) ;

      if ( !prodListVisible ) {
         pb_ASSOCIATEALL.setBounds ( x, size.height-30, half-halfButtonWidth-x, 25 ) ;
         pb_ASSOCIATENONE.setBounds ( x, size.height-30, half-halfButtonWidth-x, 25 ) ;
      } else {
         pb_ASSOCIATEALL.setBounds ( 0,0,0,0) ;
         pb_ASSOCIATENONE.setBounds ( 0,0,0,0 ) ;
      }

      st_SELECTPRODS.setBounds ( x, 0, size.width-(x*2), rowHeight ) ;
      cnr_PRODUCTS.setBounds ( x, rowHeight, size.width-(x*2), size.height-55 ) ;
      pb_SELECT.setBounds ( x, size.height-30, 80, 25 ) ;
      pb_CANCEL.setBounds ( 85, size.height-30, 80, 25 ) ;

   }






   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData() {

      if ( requireData ) {
         if ( ( !workRec.hasFamilyAll() ) &&
              ( !workRec.hasMachineAll() ) &&
              ( !workRec.hasModels() ) ) {
            GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
            return ( null ) ;
         }
      }
      return( workRec.clone() ) ;
   }



   /*-----------------------------------------------------------*/
   /* setRequired
    */
   /**
    *    This method will tell the entry field to either
    *  require data or not
    *  
    *
    *  @param requireData = TRUE if we should not close until data is entered
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setRequired ( boolean requireData ) {
      this.requireData = requireData ;
   }

   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object data ) {

      try {

         if ( data instanceof ProdDocRec ) {
            ProdDocRec prodDocRec = (ProdDocRec) data ;

            /* If we were passed a null, create a blank record  */
            if ( data == null )
               workRec = new ProdDocRec() ;

            /* Otherwise, work with a copy of the record  */
            else
               workRec = (ProdDocRec) prodDocRec.clone() ;
         } else
            workRec = new ProdDocRec() ;


         /* Now, popuplate our lists */
         setProdListVisible ( false ) ;
         cnr_MODELS.setListData ( workRec.getAssociatedModels() ) ;
         refreshMachineList() ;
         pb_ASSOCIATEALL.setVisible ( false ) ;
         pb_ASSOCIATENONE.setVisible ( false ) ;
         revalidate() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      cnr_MACHINES.requestFocus() ;
   }


   /*-----------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method responds to buttons being pressed
    *  
    *  
    *
    *  @param e The ActionEvent
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Handle the Double Clicks in the lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() instanceof DList ) {
         associateMachines ( true ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Handle the Button clicks                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() instanceof DButton ) {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_RIGHT                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == pb_RIGHT )
            associateMachines( true ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CANCEL                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_CANCEL ) {
            setProdListVisible ( false ) ;
            cnr_PRODUCTS.removeAll() ;
            revalidate() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_SELECT                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_SELECT )
            associateModels() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_LEFT                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_LEFT )
            deAssociateModels() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_RIGHTALL                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_RIGHTALL )
            associateMachinesAll() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_ASSOCIATEALL                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_ASSOCIATEALL )
            associateMachinesAll() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_ASSOCIATENONE                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_ASSOCIATENONE )
            associateMachines ( false ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_BOOKMARKS                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_BOOKMARKS )
            new BookmarkDlg ( GUISystem.getParentDefWin ( this ), this ) ;

      }
   }


   /*------------------------------------------------------------*/
   /* applyFamilyToDups
    */
   /**
    *    Since a machine type may appear in 2
    *  different families, we will print out the family
    *  with the machine type. This will alter the description of the machine
    *
    *  @param vector The vector of machine recs
    *
    *//*---------------------------------------------------------*/
   private void applyFamilyToDups(Vector vec) {
      if ( vec != null ) {
         int        i          = 0 ;
         int        k          = 0 ;
         int        length     = vec.size() ;
         MachineRec curMachRec = null ;
         MachineRec machRec    = null ;

         while ( i < length ) {
            if ( vec.elementAt(i) instanceof MachineRec ) {
               curMachRec = (MachineRec) vec.elementAt(i) ;
               k=i+1 ;
               while ( k < length ) {
                  machRec = (MachineRec) vec.elementAt(k) ;
                  if ( ( machRec.getMachine().equals ( curMachRec.getMachine() ) ) )
                     machRec.setDescript ( machRec.getMachine()+", "+Brands.groupRecFromBrandGroupInd ( machRec.getBrandGroupInd() ) ) ;
                  k++ ;
               }
            }
            i++;
         }
      }
   }


   /*------------------------------------------------------------*/
   /* highlightAssocModels
    */
   /**
    *    This method will highlight all models in the
    *  models container that are associated to the
    *  currently selected machine
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void highlightAssocModels () {
      Object selectedObject = cnr_MACHINES.getSelectedValue() ;

      if ( selectedObject instanceof MachineRec ) {
         MachineRec machineRec  = (MachineRec) selectedObject ;
         Vector     modelVec    = workRec.getAssociatedModels() ;
         int        i           = 0 ;
         int        k           = 0 ;
         int        length      = modelVec.size() ;
         int        selectCount = 0 ;
         int[]      selectArray = null ;

         /* Count the number of items to select */
         while ( i < length ) {
            if ( ((ProductRec)modelVec.elementAt(i)).equals ( machineRec ) )
               selectCount++ ;
            i++ ;
         }

         if ( selectCount > 0 ) {
            selectArray = new int[selectCount] ;

            i=0 ;
            while ( i < length ) {
               if ( ((ProductRec)modelVec.elementAt(i)).equals ( machineRec ) ) {
                  selectArray[k]=i ;
                  k++ ;
               }
               i++ ;
            }

            cnr_MODELS.setSelectedIndices ( selectArray ) ;
         }

      }
   }





   /*------------------------------------------------------------*/
   /* hiding
    */
   /**
    *    This method will be called when the dialog
    *  we are in hides itself. We will clean up
    *  our resources here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void hiding () {
      workRec = null ;

      cnr_MACHINES.removeAll() ;
      cnr_MODELS.removeAll() ;
      cnr_PRODUCTS.removeAll() ;
   }




   /*------------------------------------------------------------*/
   /* setProdListVisible
    */
   /**
    *    This method will either turn on or off
    *  the product list visibility
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setProdListVisible ( boolean visible ) {
      cnr_PRODUCTS.setVisible ( visible ) ;
      pb_SELECT.setVisible ( visible ) ;
      pb_CANCEL.setVisible ( visible ) ;
      st_SELECTPRODS.setVisible ( visible ) ;
      scr_MACHINES.setVisible ( !visible ) ;
      scr_MODELS.setVisible ( !visible ) ;
      pb_BOOKMARKS.setVisible ( !visible ) ;
      st_MACHINES.setVisible ( !visible ) ;
      st_MODELS.setVisible ( !visible ) ;
      pb_RIGHT.setVisible ( !visible ) ;
      pb_RIGHTALL.setVisible ( !visible ) ;
      pb_LEFT.setVisible ( !visible ) ;
      st_SELECTHELP.setVisible ( !visible ) ;

      if ( popUpDlg != null )
         popUpDlg.setEnabled ( !visible ) ;

      if ( !visible ) {
         add ( pb_ASSOCIATEALL ) ;
         add ( pb_ASSOCIATENONE ) ;
      } else {
         remove ( pb_ASSOCIATEALL ) ;
         remove ( pb_ASSOCIATENONE ) ;
      }

      prodListVisible = visible ;
   }





   /*------------------------------------------------------------*/
   /* setEnabled
    */
   /**
    *    This method will either enable or disable
    *  all of our controls
    *  
    *
    *  @param enabled TRUE = Set to enabled, FALSE = disable
    *
    *//*---------------------------------------------------------*/
   public void setEnabled ( boolean enabled ) {
      pb_RIGHT.setEnabled ( enabled ) ;
      pb_RIGHTALL.setEnabled ( enabled ) ;
      pb_LEFT.setEnabled ( enabled ) ;
      pb_SELECT.setEnabled ( enabled ) ;
      pb_CANCEL.setEnabled ( enabled ) ;
      pb_ASSOCIATEALL.setEnabled ( enabled ) ;
      pb_ASSOCIATENONE.setEnabled ( enabled ) ;
      cnr_PRODUCTS.setEnabled ( enabled ) ;
      if ( popUpDlg != null )
         popUpDlg.setEnabled ( enabled ) ;
   }





   /*------------------------------------------------------------*/
   /* showProductList
    */
   /**
    *    This method will show the product list. Before
    *  The list is shown, all products will be read for
    *  the machine passed in.
    *
    *  @param machineRec The MachineRec to read with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showProductList ( MachineRec machineRec ) {
      Vector         prodList          = null ;

      setEnabled ( false ) ;
      st_STATUS.setText ( Str.getStr ( STR_READING_PRODS_FOR_MACH )+" "+machineRec.toString() ) ;

      if ( machineRec.isOption() ) {
         prodList = readModelsForOptions ( machineRec.getMachine(), machineRec.getBrandGroupInd() ) ;
      }
      else
         prodList = readModels ( machineRec.getSubMachine(), machineRec.getBrandGroupInd() ) ;
      

      setEnabled ( true ) ;

      if ( ( prodList != null ) && ( prodList.size() > 0 ) ) {
         cnr_PRODUCTS.removeAll () ;
         cnr_PRODUCTS.setData ( prodList ) ;
         setProdListVisible ( true ) ;
         revalidate() ;
      }

      st_STATUS.setText ( "" ) ;
   }







   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when a selection
    *  occurs in a list
    *  
    *
    *  @param e the event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      Object[] values    = cnr_MACHINES.getSelectedValues() ;
      int      allCount  = 0 ;
      int      normCount = 0 ;

      if ( values != null ) {
         int i = 0 ;

         /* First, figure out how many alls we have
            because this determines which button we
            will show  */
         while ( i < values.length ) {
            if ( values[i] instanceof MachineRec ) {
               if ( ((MachineRec)values[i]).getAll() )
                  allCount++ ;
               else
                  normCount++ ;
            }
            i++ ;
         }

         pb_ASSOCIATEALL.setVisible  ( normCount > allCount ) ;
         pb_ASSOCIATENONE.setVisible ( allCount >= normCount ) ;
      } else {
         pb_ASSOCIATEALL.setVisible ( false ) ;
         pb_ASSOCIATENONE.setVisible ( false ) ;
      }

      highlightAssocModels() ;
   }



   /*------------------------------------------------------------*/
   /* associateMachines
    */
   /**
    *    This method will make the machine
    *  associations and allow the user to select the models
    *  
    *
    *  @param showProdList - Shows the product list after associating
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void associateMachines ( boolean showProdList ) {

      if ( showProdList ) {
         Object selectedObject = cnr_MACHINES.getSelectedValue() ;

         if ( selectedObject instanceof MachineRec ) {
            MachineRec machineRec = (MachineRec) selectedObject ;

            if ( machineRec.getAll() )
               workRec.addMachineAssociation ( machineRec ) ; // Take off the 'All'

            showProductList ( machineRec ) ;
         }
      }

      else {
         Object[] values = cnr_MACHINES.getSelectedValues() ;
         int      i      = 0 ;

         if ( ( values != null ) && ( values.length > 0 ) ) {
            while ( i < values.length ) {
               if ( values[i] instanceof MachineRec )
                  workRec.addMachineAssociation ( (MachineRec) values[i] ) ;
               i++ ;
            }
            pb_ASSOCIATEALL.setVisible ( true ) ;
            pb_ASSOCIATENONE.setVisible ( false ) ;
            revalidate() ;
            repaint() ;
         }

      }

      cnr_MODELS.setListData ( workRec.getAssociatedModels() ) ;

   }





   /*------------------------------------------------------------*/
   /* associateMachinesAll
    */
   /**
    *    This method will associate all selected machines
    *  to all.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void associateMachinesAll () {
      Object[] values = cnr_MACHINES.getSelectedValues() ;

      if ( ( values != null ) && ( values.length > 0 ) ) {
         int i = 0 ;

         while ( i < values.length ) {
            if ( values[i] instanceof MachineRec )
               workRec.addMachineAllAssociation ( (MachineRec) values[i] ) ;
            i++ ;
         }
      }
      cnr_MODELS.setListData ( workRec.getAssociatedModels() ) ;
      pb_ASSOCIATEALL.setVisible ( false ) ;
      pb_ASSOCIATENONE.setVisible ( true ) ;
      revalidate() ;
      repaint() ;
   }



   /*------------------------------------------------------------*/
   /* associateModels
    */
   /**
    *    This method will associate all models
    *  selected to the document
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void associateModels () {
      Object[] values = cnr_PRODUCTS.getJList().getSelectedValues() ;
      int      i              = 0 ;

      if ( ( values != null ) && ( values.length > 0 ) ) {

         while ( i < values.length ) {
            if ( values[i] instanceof ProductRec ) {
               workRec.addModelAssociation ( (ProductRec) values[i] ) ; 
            }
            i++ ;
         }
      }

      cnr_MODELS.setListData ( workRec.getAssociatedModels() ) ;
      setProdListVisible ( false ) ;
      revalidate() ;

   }





   /*------------------------------------------------------------*/
   /* deAssociateModels
    */
   /**
    *    this method will de associate all selected
    *  models
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void deAssociateModels () {
      Object[] values = cnr_MODELS.getSelectedValues() ;
      int      i              = 0 ;

      if ( ( values != null ) && ( values.length > 0 ) ) {

         while ( i < values.length ) {
            if ( values[i] instanceof ProductRec ) {
               workRec.deAssociateModel ( (ProductRec) values[i] ) ; 
            }
            i++ ;
         }
      }

      cnr_MODELS.setListData ( workRec.getAssociatedModels() ) ;
      revalidate() ;
   }



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   refreshMachineList - Populate cnr_MACHINES              */                                 
   /*                                                           */
   /*-----------------------------------------------------------*/
   private void refreshMachineList () {
      Vector vec = workRec.getMachineFamilyList() ;

      applyFamilyToDups ( vec ) ;

      cnr_MACHINES.setListData ( vec ) ;
   }





   /*------------------------------------------------------------*/
   /* bookmarkSelected
    */
   /**
    *    This method will be called when the user
    *  has selected a bunch of bookmarks
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void bookmarkSelected ( Vector bookmarkVec ) {
      if ( ( bookmarkVec != null ) && ( bookmarkVec.size() > 0 ) ) {
         int      i        = 0 ;
         int      size     = bookmarkVec.size() ;
         Bookmark bookmark = null ;

         while ( i < size ) {
            bookmark = (Bookmark) bookmarkVec.elementAt(i) ;

            if ( bookmark.getBookmarkType() == BOOKMARK_PRODUCT ) {
               Integer    prodInt = (Integer) bookmark.getObject() ;
               ProductRec prodRec = new ProductRec ( prodInt.intValue() ) ;

               st_STATUS.setText ( Str.getStr ( STR_READING )+" "+prodInt ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Read the product, and add it                            */                   
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( prodRec.readFromDatabase() == 0 ) {
                  workRec.addModelAssociation ( prodRec ) ;
                  cnr_MODELS.setListData ( workRec.getAssociatedModels() ) ;
                  refreshMachineList() ;
               }
            }


            i++ ;
         }
      }

      st_STATUS.setText ( "" ) ;
      
      if ( listener != null )
         listener.brandFamilyChanged ( this ) ;

   }







   /*------------------------------------------------------------*/
   /* readModelsForOptions
    */
   /**
    *    This method will read all of the models under
    *  the machine given. The machine must be an Option
    *  
    *
    *  @param machine The machine to read models under
    *  @param None
    *  @param None
    *
    *  @return Vector of StringTreeNodes. Each StringTreeNode representing a the following structure: Machine-Model  (Description)
    *//*---------------------------------------------------------*/
   private Vector readModelsForOptions ( String machine, int brandGroupInd) {
      Vector         modelVec       = new Vector ( 1, 1 ) ;
      SQLMethod      sqlMethod      = new SQLMethod ( DatabaseSystem.QUEST, "readModels", 10 ) ;                                          
      StringTreeNode stringTreeNode = null ;

      try {
         Statement  stmt    = sqlMethod.createStatement() ;
         ProductRec prodRec = null ;
         ResultSet rs       = stmt.executeQuery ( "SELECT A.PRODINT, A.PRODNUM, B.DESCRIPT, A.MODEL, A.PRODDESC, A.STATUSIND, A.STATUSDATE "+
                                                  "FROM TIGRIS.PRODUCTS A, "+
                                                  "     TIGRIS.PRODMACHINEDESC B "+
                                                  "WHERE A.BRANDGROUPIND = "+brandGroupInd+" AND "+
                                                  "      B.DESCRIPT      = '"+machine+"' AND "+
                                                  "      A.MODEL        <> 'All' AND "+
                                                  "      A.RECYCLED      = 'N' AND "+
                                                  "      A.PRODINT       = B.PRODINT order by A.PRODNUM" ) ;

         while ( rs.next() ) {

            prodRec = new ProductRec ( rs.getInt(1),             // PRODINT
                                       rs.getString(2).trim(),   // PRODNUM
                                       rs.getString(3).trim(),   // MACHINE
                                       rs.getString(4).trim() ); // MODEL
            prodRec.setFieldData ( PRODFIELD_DESCRIPTION, rs.getString(5).trim() ) ;
            prodRec.setFieldData ( PRODFIELD_STATUS,      new StatusRec ( rs.getInt(6), CDate.convertDate ( CDate.DATE_DBDATE, CDate.DATE_DATE, rs.getString(7).trim() ) ) ) ;
            prodRec.setFieldData ( PRODFIELD_BRANDGROUPIND, new Integer ( brandGroupInd ) ) ;
            modelVec.addElement ( prodRec ) ;

         }

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_CANNOT_READ_MODEL_LIST ) ) ;
      }

      sqlMethod.close() ;

      return( modelVec ) ;
   }



   /*------------------------------------------------------------*/
   /* readModels
    */
   /**
    *    This method will read all of the models under
    *  the machine given
    *  
    *
    *  @param machine The machine to read models under
    *  @param None
    *  @param None
    *
    *  @return Vector of StringTreeNode. Each StringTreeNode representing a the following structure: Machine-Model  (Description)
    *//*---------------------------------------------------------*/
   private Vector readModels ( String machine, int brandGroupInd ) {
      Vector         modelVec       = new Vector ( 1, 1 ) ;
      SQLMethod      sqlMethod      = new SQLMethod ( DatabaseSystem.QUEST, "readModels", 10 ) ;
      StringTreeNode stringTreeNode = null ;

      try {
         Statement  stmt    = sqlMethod.createStatement() ;
         ProductRec prodRec = null ;
         ResultSet  rs      = stmt.executeQuery ( "SELECT PRODINT, PRODNUM, MACHINE, MODEL, PRODDESC, STATUSIND, STATUSDATE "+
                                                  "FROM TIGRIS.PRODUCTS "+
                                                  "WHERE BRANDGROUPIND = "+brandGroupInd+" AND "+
                                                  "      MACHINE       = '"+machine+"' AND "+
                                                  "      MODEL        <> 'All' AND "+
                                                  "      RECYCLED      = 'N' order by PRODNUM" ) ;

         while ( rs.next() ) {
            prodRec = new ProductRec ( rs.getInt(1),             // PRODINT
                                       rs.getString(2).trim(),   // PRODNUM
                                       rs.getString(3).trim(),   // MACHINE
                                       rs.getString(4).trim() ); // MODEL
            prodRec.setFieldData ( PRODFIELD_DESCRIPTION, rs.getString(5).trim() ) ;
            prodRec.setFieldData ( PRODFIELD_STATUS,      new StatusRec ( rs.getInt(6), CDate.convertDate ( CDate.DATE_DBDATE, CDate.DATE_DATE, rs.getString(7).trim() ) ) ) ;
            prodRec.setFieldData ( PRODFIELD_BRANDGROUPIND, new Integer ( brandGroupInd ) ) ;
            modelVec.addElement ( prodRec ) ;

         }

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_CANNOT_READ_MODEL_LIST ) ) ;
      }

      sqlMethod.close() ;

      return( modelVec ) ;
   }

   /*------------------------------------------------------------*/
   /* setPopUpBrandFamilyListener
    */
   /**
    *    This method will set the listener who is
    *  notified of changes made to the Brand/Family
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPopUpBrandFamilyListener ( PopUpBrandFamilyListener listener ) {
      this.listener = listener ;
   }

   public ProdDocRec getWorkingRec () {
      return ( workRec ) ;
   }

} /* PopUpMachineModel */



