package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpSplitTypeList                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpSplitTypeList                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.Vector ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This panel holds a two lists and a description. The lists are separated by
 * a button marking '>>' and a button marking '<<'
 *
 * To use this popup, two Vectors must be passed inside of another vector. For 
 * example:
 *
 * Vector vecOfVector     = new Vector(2) ;
 * Vector typeAuth        = TypeList.getTypeList ( TYPE_AUTH ) ;
 * Vector associatedAuths = new Vector(1) ;
 *
 * vecOfVector.add ( typeAuth ) ;        // First element MUST be complete list
 * vecOfVector.add ( associatedAuths ) ; // Second element MUST be associated list
 *
 * SystemData.showPopUp ( 0, POPUP_TYPELIST, vecOfVector, WIZARD, this )
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpSplitTypeList                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpSplitTypeList extends PopUpPanel implements ActionListener, Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea   st_DESCRIPT       = new JTextArea() ;
   private JList       cnr_DATALIST      = new JList() ;
   private JList       cnr_ASSOCLIST     = new JList() ;
   private JScrollPane scr_DATALIST      = new JScrollPane ( cnr_DATALIST ) ;
   private JScrollPane scr_ASSOCLIST     = new JScrollPane ( cnr_ASSOCLIST ) ;
   private DButton     pb_ASSOC          = new DButton ( ">>" ) ;
   private DButton     pb_DEASSOC        = new DButton ( "<<" ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       rowHeight     = 18 ;
   private Vector    assocVec      = null ;

   private Object    refreshObject = null ;

   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpSplitTypeListPanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpSplitTypeList ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;
      setLayout ( null ) ;                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      WinUtil.setJTextAreaAsLabel ( st_DESCRIPT ) ;
      st_DESCRIPT.setFont ( FontSystem.defaultFont ) ;
      cnr_DATALIST.setPrototypeCellValue("WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW"); // 50 W's
      cnr_ASSOCLIST.setPrototypeCellValue("WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW"); // 50 W's
      cnr_DATALIST.setFixedCellWidth(450);    
      cnr_ASSOCLIST.setFixedCellWidth(450);      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_ASSOC.addActionListener ( this ) ;
      pb_DEASSOC.addActionListener ( this ) ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( st_DESCRIPT ) ;
      add ( scr_DATALIST ) ;
      add ( scr_ASSOCLIST ) ;
      add ( pb_ASSOC ) ;
      add ( pb_DEASSOC ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    this method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size       = getSize() ;
      int       descHeight = 0 ;

      super.doLayout () ;

      st_DESCRIPT.setBounds ( 0, 0, size.width, rowHeight ) ;
      descHeight = st_DESCRIPT.getPreferredSize().height ;
      st_DESCRIPT.setBounds ( 0, 0, size.width, descHeight ) ;

      scr_DATALIST.setBounds ( 0, descHeight+2, (size.width/2)-35, size.height-descHeight-2 ) ;
      scr_ASSOCLIST.setBounds ( (size.width/2)+35, descHeight+2, (size.width/2)-35, size.height-descHeight-2 ) ;

      pb_DEASSOC.setBounds ( (size.width/2)-30, 100, 60, rowHeight+5 ) ;
      pb_ASSOC.setBounds ( (size.width/2)-30, 100+rowHeight+7, 60, rowHeight+5 ) ;



   }


   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      st_DESCRIPT.setText ( descript ) ;
   }

   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData () {
      Object retObj = null ;

      if ( ( assocVec == null ) || ( assocVec.size() == 0 ) ) {

         if ( requireData )
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
         else
            retObj = new Vector(10) ;

      } else
         retObj = assocVec ;

      PropertySystem.put ( PropertySystem.PROPERTY_LAST_SPLIT_SELECTION, cnr_DATALIST.getSelectedIndices() ) ;
      return( retObj ) ;
   }

   /*********************************************************
   * <b><font color="maroon">Description:</b></font>
   * Set the control to original status.
   *
   * <p><b><font color="teal">Author:</b></font> <i>Chad Meadows</i>
   * <p><b><font color="teal">Created:</b></font> 10/9/1998  1:52pm
   * @param
   * @return
   *********************************************************/
   public void setDefaults() {
      //cnr_DATALIST.clearSelection ( ) ;
   }


   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object dataObj ) {
      refreshObject = dataObj ;
      new Thread ( this ).start() ;
   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      cnr_DATALIST.requestFocus() ;
   }     


   /*-----------------------------------------------------------*/
   /* setSelection
    */
   /**
    *    This method will set what is currently selected
    *  
    *  
    *
    *  @param data The object to be selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelection ( Object data ) {
      cnr_DATALIST.setSelectedValue ( data, true ) ;
   }

   public Dimension getPreferredSize () {
      return( new Dimension ( 100, 100 ) ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    Runs our thread to refresh our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      Vector listVec     = (Vector) refreshObject ;
      Vector dataVec     = (Vector)listVec.elementAt(0) ;
      Vector newAssocVec = (Vector)listVec.elementAt(1) ;
      int[]  selectIndex = (int[])PropertySystem.get ( PropertySystem.PROPERTY_LAST_SPLIT_SELECTION ) ;

      if ( newAssocVec == null )
         newAssocVec = new Vector ( 10, 10 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Copy the vector                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      assocVec = (Vector) newAssocVec.clone() ;

      cnr_DATALIST.setListData ( dataVec ) ;
      cnr_ASSOCLIST.setListData ( assocVec ) ;

      if ( ( selectIndex != null ) && ( selectIndex.length > 0 ) ) {
         cnr_DATALIST.setSelectedIndices ( selectIndex ) ;
         cnr_DATALIST.ensureIndexIsVisible ( selectIndex[0] ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an actionevent
    *  is fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == pb_ASSOC )
         associate() ;
      else if ( e.getSource() == pb_DEASSOC )
         deassociate() ;
   }





   /*------------------------------------------------------------*/
   /* associate
    */
   /**
    *    This method will take all selected items from 
    *  the left list box and associates them in the
    *  right. Duplicates will not be associated
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void associate () {
      Object items[] = cnr_DATALIST.getSelectedValues() ;
      Object assoc[] = cnr_ASSOCLIST.getSelectedValues() ;

      if ( items != null ) {
         if ( assoc != null ) {
            int i    = 0 ;

            while ( i < items.length ) {
               if ( !StoreUtil.vectorContains ( assocVec, items[i] ) )
                  assocVec.addElement ( items[i] ) ;
               i++ ;
            }

            cnr_ASSOCLIST.setListData ( assocVec ) ;

         } else
            cnr_ASSOCLIST.setListData ( items ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* deassociate
    */
   /**
    *    This method will remove all selected associations
    *  from the right list box
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void deassociate () {
      Object[] assoc = cnr_ASSOCLIST.getSelectedValues () ;

      if ( ( assoc != null ) && ( assoc.length > 0 ) ) {
         int i    = 0 ;

         while ( i < assoc.length ) {
            assocVec.removeElement ( assoc[i] ) ;
            i++ ;
         }
      }

      cnr_ASSOCLIST.setListData ( assocVec ) ;
   }


} /* PopUpSplitTypeList */
