package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpStatus                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpStatus                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.event.* ;
import java.awt.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel holds an entry field and a description. 
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpStatus                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpStatus extends PopUpPanel implements FocusListener, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_DESCRIPT       = new JLabel ( Str.getStr ( STR_PLEASE_SELECT_STATUS ) ) ;
   private JLabel       st_DATE           = new JLabel ( Str.getStr ( STR_DATE ) ) ;
   private DateEntryField ef_DATE           = new DateEntryField() ;
   private JComboBox    cb_STATUS         = new JComboBox() ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       rowHeight = GUISystem.getRowHeight() ;
   private StatusRec statusRec = null ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpStatusPanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpStatus ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;
      setLayout ( null ) ;

      /* Adjust some properties  */
      ef_DATE.addActionListener ( this ) ;

      add ( st_DESCRIPT ) ;
      add ( cb_STATUS ) ;
      add ( st_DATE ) ;
      add ( ef_DATE ) ;
   }                            

   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will be subclassed to size and
    *  position our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize();
      int       y    = 0 ;

      super.doLayout() ;

      st_DESCRIPT.setBounds ( 0, 0, size.width, rowHeight ) ;
      y+=rowHeight+2 ;

      cb_STATUS.setBounds ( 0, y, 230, rowHeight ) ;
      y+=rowHeight+7 ;

      st_DATE.setBounds ( 0, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      ef_DATE.setBounds ( 0, y, 150, rowHeight ) ;
      y+=rowHeight+2 ;

   }



   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
   }



   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData () {
      String        statusDate     = ef_DATE.getText().trim() ;
      StatusRec     retStatus      = null ;
      TypeStatusRec typeStatus     = (TypeStatusRec)cb_STATUS.getSelectedItem() ;

      if ( typeStatus == null )
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
      else if ( CDate.checkDate ( statusDate, CDate.DATE_DATE ) != 0 ) {
         GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_DATE_ENTERED_WRONG ) ) ;
      } else {

         if ( statusRec == null )
            statusRec = new StatusRec () ;

         statusRec.setStatusInd  ( typeStatus.getInd() ) ;
         statusRec.setDate       ( ef_DATE.getText() ) ;

         retStatus = statusRec ;
      } 

      return( retStatus ) ;
   }

   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object dataObj ) {
      if ( ( dataObj != null ) && (  dataObj instanceof StatusRec ) ) {
         StatusRec statusRec = (StatusRec) dataObj ;

         this.statusRec = (StatusRec)statusRec.clone() ;
         cb_STATUS.setSelectedItem ( TypeList.getInstance().objectFromInd ( statusRec.getStatusInd(), TypeList.TYPE_STATUS ) ) ;
         ef_DATE.setText ( statusRec.getDate() ) ;
      } else
         ef_DATE.setText ( new CDate ( CDate.DATE_DATE ).today() ) ;
   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      if ( cb_STATUS != null )
         cb_STATUS.requestFocus() ;
   }



   /*-----------------------------------------------------------*/
   /* StatusHit
    */
   /**
    *    This method will respond to the entry
    *  field being used
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void actionPerformed ( ActionEvent e ) {
      String text = ef_DATE.getText().trim() ;
      String date = CDate.format4DigitYear ( text, CDate.DATE_DATE ) ;

      ef_DATE.setText ( date ) ;

      if ( saveData() != null )
         popUpDlg.okHit ( PopUpDlg.PB_OK ) ;
   }



   /*-----------------------------------------------------------*/
   /* setTypeStatus
    */
   /**
    *    This method will set the type of status
    *  records that display
    *  
    *
    *  @param typeStatus The type of status to refresh with
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTypeStatus ( int typeStatus ) {
      if ( cb_STATUS.getItemCount() > 0 )
         cb_STATUS.removeAllItems() ;
      Status.refreshList ( cb_STATUS, typeStatus ) ;
      cb_STATUS.requestFocus() ;
   }

   /*-----------------------------------------------------------*/
   /* setReadOnly
    */
   /**
    *    This method will change this pop up to be
    *  read only
    *  
    *             
    *  @param readOnly TRUE = make read only, false Make no read only
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void setReadOnly ( boolean readOnly ) {
      super.setReadOnly ( readOnly ) ;
      ef_DATE.setEnabled ( !readOnly ) ;
      cb_STATUS.setEnabled ( !readOnly ) ;
   }

   public void focusGained ( FocusEvent e ) {
   }




   /*------------------------------------------------------------*/
   /* focusLost
    */
   /**
    *    This method will be called when the
    *  entry field for the status date loses focus. We
    *  will use this time to format the date correctly
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void focusLost ( FocusEvent e ) {
   }

} /* PopUpStatus */
