package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   PopUpTypeList                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PopUpTypeList                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.Vector ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel holds a ContainerWin and a description. The ContainerWin is
 * automatically populated with the type list Integer that is passed to the
 * SystemData.showPopUpWin method. For example, this call will create this panel
 * with the Type Authorities.
 *
 * SystemData.showPopUp ( 0, POPUP_TYPELIST, new Integer ( TYPEAUTH ), WIZARD, this )
 *
 *
 * @version 1.0, 6/28/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   PopUpTypeList                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class PopUpTypeList extends PopUpPanel implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTextArea   st_DESCRIPT       = new JTextArea() ;
   private JList       dataList          = new JList() ;
   private JScrollPane scr_PANE          = new JScrollPane ( dataList ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int       rowHeight     = 18 ;
   private int       lastTypeList  = -1 ;
   private Vector    typeList      = null ;

   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a PopUpTypeListPanel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public PopUpTypeList ( PopUpDlg popUpDlg ) {
      super(popUpDlg) ;
      setLayout ( new BorderLayout ( ) ) ;

      /* set some properties  */
      st_DESCRIPT.setLineWrap ( true ) ;
      st_DESCRIPT.setWrapStyleWord ( true ) ;
      st_DESCRIPT.setEditable ( false ) ;
      st_DESCRIPT.setOpaque ( false ) ;
      st_DESCRIPT.setFont ( FontSystem.defaultFont ) ;
      dataList.setPrototypeCellValue("WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW"); // 50 W's
      dataList.setFixedCellWidth(450);    

      add ( st_DESCRIPT, BorderLayout.NORTH ) ;
      add ( scr_PANE, BorderLayout.CENTER ) ;



      setDefaults() ;
   }

   /*-----------------------------------------------------------*/
   /* setDescript
    */
   /**
    *    We subclass this method to set our description
    *  
    *  
    *
    *  @param descript
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setDescript ( String descript ) {
      st_DESCRIPT.setText ( descript ) ;
   }

   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will determine if the user has
    *  entered the data needed to close the popup
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return The string entered if any
    *//*---------------------------------------------------------*/
   public Object saveData () {
      Object retObj = null ;

      if ( dataList.getSelectionMode() == ListSelectionModel.MULTIPLE_INTERVAL_SELECTION ) {
         Object[] values = dataList.getSelectedValues() ;

         if ( ( values == null ) || ( values.length == 0 ) ) {

            if ( requireData )
               GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
            else
               retObj = new Vector(10) ;

         } else {
            int    i = 0 ;
            int    length = values.length ;
            Vector tempVec = new Vector ( values.length ) ;

            while ( i < length )
               tempVec.addElement(values[i++]) ;
            retObj = tempVec ;
         }

      } else {
         retObj = dataList.getSelectedValue() ;

         if ( retObj == null ) {
            if ( requireData )
               GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
         }
      }

      return( retObj ) ;
   }

   /*********************************************************
   * <b><font color="maroon">Description:</b></font>
   * Set the control to original status.
   *
   * <p><b><font color="teal">Author:</b></font> <i>Chad Meadows</i>
   * <p><b><font color="teal">Created:</b></font> 10/9/1998  1:52pm
   * @param
   * @return
   *********************************************************/
   public void setDefaults() {
      //dataList.clearSelection ( ) ;
      //setMultiSelect ( false ) ;
   }



   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh all of our data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshData ( Object dataObj ) {
      dataList.clearSelection();

      try {
         if ( dataObj instanceof Integer ) {
            Integer integer = (Integer) dataObj ;

            if ( integer.intValue() != lastTypeList ) {
               lastTypeList = integer.intValue() ;
               typeList = TypeList.getInstance().getTypeList ( lastTypeList ) ;
               dataList.setListData ( typeList ) ;
            }

         } else
            dataList.setListData ( typeList = new Vector(10) ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

   }




   /*-----------------------------------------------------------*/
   /* setFocus
    */
   /**
    *    This method will be called when the PopUpDlg
    *  is done initializing and is now ready to let
    *  the user enter data.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setFocus() {
      dataList.requestFocus() ;
   }   

   public void refreshInfo() {
      setFocus() ;
   }


   /*-----------------------------------------------------------*/
   /* setSelection
    */
   /**
    *    This method will set what is currently selected
    *  
    *  
    *
    *  @param data The object to be selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelection ( Object data ) {
      dataList.setSelectionMode ( ListSelectionModel.SINGLE_SELECTION ) ;
      dataList.setSelectedValue ( data, true ) ;
   }
   /*-----------------------------------------------------------*/
   /* setSelection
    */
   /**
    *    This method will set what is currently selected
    *  
    *  
    *
    *  @param data The object to be selected
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSelection ( Vector data ) {

      dataList.setSelectionMode ( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION ) ;
      if ( data != null ) {
         int   i       = 0 ;
         int   length  = data.size() ;
         int[] indices = new int[length] ;

         while ( i < length ) {
            indices[i]=typeList.indexOf ( data.elementAt(i) ) ;
            i++ ;
         }
         dataList.setSelectedIndices ( indices ) ;
      }
   }




   public void setMultiSelect ( boolean allowMulti ) {
      if ( allowMulti )
         dataList.setSelectionMode ( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION ) ;
      else
         dataList.setSelectionMode ( ListSelectionModel.SINGLE_SELECTION ) ;
   }




   /*------------------------------------------------------------*/
   /* limitDocType
    */
   /**
    *    This method can be used to limit the document
    *  types shown. This is only used for the wizard
    *  in DocMaintPanel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void limitDocType () {
   }

} /* PopUpTypeList */
