package com.ibm.nzna.projects.qit.gui.popup ;
/*-----------------------------------------------------------*/
/*   SelectCountryDlg                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectCountryDlg                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.Vector ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * Allows the user to select a country that is associated to a geography 
 *
 *
 * @version 1.0, 7/1/1998 
 * @author  David Lentz
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectCountryDlg                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectCountryDlg extends JDialog implements ActionListener, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JLabel       st_DESCRIPT      = new JLabel    ( Str.getStr ( STR_COUNTRY ) ) ;
   private MultiList    cnr_COUNTRYLIST  = new MultiList ( GUISystem.getFontUtil() ) ;
   private DButton      pb_OK            = new DButton   ( Str.getStr ( STR_OK ) ) ;
   private DButton      pb_CANCEL        = new DButton   ( Str.getStr ( STR_CANCEL ) ) ;
   private DButton      pb_ALL           = new DButton   ( Str.getStr ( STR_ALL ) ) ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private TypeGeoRec geoRec = null;
   private LocaleRec  retRec = null ;
   private int        id     = 0 ;


   /*-----------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our dialog
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectCountryDlg ( TypeGeoRec geoRec, LocaleRec retRec ) {
      super ( MainWindow.getInstance(), Str.getStr ( STR_SELECT_COUNTRY ), true ) ;

      try {
         this.retRec = retRec ;
         this.geoRec = geoRec ;
         setResizable ( false ) ;
         setSize ( 300, 400 ) ;
         WinUtil.centerWindow ( this ) ;
         createControls() ;
         refreshList() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* getCountryList
    */
   /**
    *    This method will show the window and
    *  waits for the user to press Ok, ALl or Cancel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return LocaleRec with the countries added
    *//*---------------------------------------------------------*/
   public LocaleRec getCountryList () {
      setVisible ( true ) ;
      return( retRec ) ;
   }


   /*-----------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will create and attach all of our
    *  controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {
      try {
         Container contentPane = getContentPane() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set some properties                                     */        
         /*                                                           */
         /*-----------------------------------------------------------*/
         cnr_COUNTRYLIST.setColumnWidth ( 0, 250 ) ;
         cnr_COUNTRYLIST.setForeground ( Color.black ) ;
         cnr_COUNTRYLIST.setRowHeight ( 18 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our controls                                        */     
         /*                                                           */
         /*-----------------------------------------------------------*/
         contentPane.setLayout ( null ) ;
         contentPane.add ( st_DESCRIPT ) ;
         contentPane.add ( cnr_COUNTRYLIST ) ;
         contentPane.add ( pb_ALL ) ;
         contentPane.add ( pb_OK ) ;
         contentPane.add ( pb_CANCEL ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add Listeners                                           */  
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_ALL.addActionListener    ( this ) ;
         pb_OK.addActionListener     ( this ) ;
         pb_CANCEL.addActionListener ( this ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }




   /*-----------------------------------------------------------*/
   /* refreshList
    */
   /**
    *    This method is called after window initialization.
    *  We will use this time to populate our Countries
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshList () {
      Vector             vector     = TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ;
      TypeCountryCodeRec countryRec = null ;
      int                i          = 0;
      int                length     = vector.size() ;

      try {

         while ( i < length ) {
            countryRec = (TypeCountryCodeRec) vector.elementAt(i) ;
            if ( countryRec.getGeoInd() == geoRec.getInd() )
               cnr_COUNTRYLIST.add ( countryRec ) ;
            i++ ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

   }




   /*-----------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out all of our controls
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       x         = 10 ;
      int       y         = 10 ;
      int       rowHeight = 18 ;
      Dimension size      = getSize() ;

      super.doLayout() ;

      try {
         st_DESCRIPT.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
         y+=rowHeight ;

         cnr_COUNTRYLIST.setBounds ( x, y, size.width-(x*2), size.height-125 ) ;

         pb_ALL.setBounds ( x, size.height-90, size.width-(x*2), 25 ) ;

         pb_OK.setBounds ( x, size.height-55, 70, 25 ) ;
         pb_CANCEL.setBounds ( x+75, size.height-55, 70, 25 ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }

   /*-----------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method responds to buttons being pressed
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      try {

         if ( e.getSource() == pb_CANCEL )
            dispose() ;

         else if ( e.getSource() == pb_ALL ) {
            selectAll() ;
            dispose() ;
         }

         else if ( e.getSource() == pb_OK ) {
            if ( saveData () )
               dispose() ;
         }

      } catch ( Exception ex) {
         LogSystem.log ( 1, ex ) ;
      }
   }

                      

   /*-----------------------------------------------------------*/
   /* selectAll
    */
   /**
    *    This method will add all of our countries to the
    *  vector and return
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectAll () {
      try {
         retRec.addGeography ( geoRec ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }



   /*-----------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save all of the selections and
    *  return
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Data was saved Ok, FALSE - No data was saved
    *//*---------------------------------------------------------*/
   public boolean saveData () {
      boolean rc     = false ;

      try {
         Vector countryVec = cnr_COUNTRYLIST.getSelection() ;

         if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
            int i       = 0 ;
            int length  = countryVec.size() ;

            while ( i < length )
               retRec.addCountry ( (TypeCountryCodeRec) countryVec.elementAt(i++) ) ;

            rc = true ;

         } else
            GUISystem.printBox ( Str.getStr ( STR_ERROR ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }

      return( rc ) ;
   }



} /* SelectCountryDlg */ 
