package com.ibm.nzna.projects.qit.help ;            
/*-----------------------------------------------------------*/
/*   HelpContentPanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HelpContentPanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.* ;     
import com.ibm.nzna.projects.qit.app.* ;     
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel holds the content of the Help Window, 
 * basically it is just the browser window and action buttons
 *
 *
 * @version 1.0, 1/13/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HelpContentPanel                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class HelpContentPanel extends JPanel implements BrowserListener, ActionListener, AppConst, QuestPanel {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Browser               browser              = null ;
   private HelpNavPanel          navPanel             = null ;
   private boolean               enabled              = true ;
   private HelpPage              helpPage             = null ;
   private ActionButton          pb_CLOSE             = null ;
   private ActionButton          pb_INDEX             = null ;
   private ActionButton          pb_HELP              = null ;
   private HelpWindow            helpWin              = null ;
   private ApplicationTitlePanel titlePanel           = null ;
   private String                helpUrl              = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our Help Content
    *  Panel. This will not initialize anything, since
    *  the PanelSystem will call our Initialize method
    *
    *  @param contents The initial contents to display
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HelpContentPanel ( String helpURL ) {
      this.helpUrl = helpUrl ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will be called before
    *  we are shown. It is threaded so we
    *  are safe to do heavy processing.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean initialize () {          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      browser  = GUISystem.createBrowser() ;
      navPanel = new HelpNavPanel ( this, helpPage ) ;                 
      pb_CLOSE = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),       Str.getStr ( STR_HELP_ON_CLOSE ) ) ;                
      pb_INDEX = new ActionButton ( Str.getStr ( STR_INDEX ), ImageSystem.getImageIcon ( this, ImageSystem.INDEX ),       Str.getStr ( STR_HELP_ON_INDEX ) ) ;                
      pb_HELP  = new ActionButton ( Str.getStr ( STR_HELP ),  ImageSystem.getImageIcon ( this, ImageSystem.SMALL_HELP ),  Str.getStr ( STR_HELP_ON_HELP ) ) ;                


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      browser.addBrowserListener ( this ) ;
      pb_CLOSE.addActionListener ( this ) ;
      pb_INDEX.addActionListener ( this ) ;
      pb_HELP.addActionListener  ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the components                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout () );
      add ( browser.getComponent()) ;

      return( true ) ;
   }






   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {

      if ( helpUrl == null ) {
         setTitle ( Str.getStr ( STR_HELP ) ) ;
      }

      if ( helpWin == null )
         helpWin = HelpSystem.getHelpWindow() ;

      helpWin.addActionComponent ( pb_CLOSE ) ;
      helpWin.addActionComponent ( pb_INDEX ) ;
      helpWin.addActionComponent ( pb_HELP ) ;
      System.out.println ( "Started!" ) ;

      invalidate() ;
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called after we
    *  have been hidden so we can free
    *  resources
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remove controls                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      remove ( browser.getComponent() ) ;     


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Free resources                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      browser = null ;

      return( true ) ;
   }






   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the
    *  application is wanting to shut down
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE - Since we don't care if we shutdown
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return the default
    *  navigation panel for this content pane.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NavPanel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel ( ) {
      return( navPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* searchAll
    */
   /**
    *    This method will show the 'Search All' navigation
    *  panel
    *  
    *
    *  @param show TRUE = show the nav panel, FALSE = hide it
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void searchAll ( boolean show ) {
      //if ( show )
      //   GUISystem.getParentDefWin(this).setNavPanel ( this, new HelpSearchNavPanel ( this ) ) ;
      //else
      //   GUISystem.getParentDefWin(this).setNavPanel ( this, getDefaultNavPanel() ) ;
   }



   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return the JComponent
    *  which is really just an instance of this
    *  class
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* enableInput
    */
   /**
    *    This method will enable a flag which
    *  we will use when the user acts on
    *  our buttons.
    *
    *  @param enable TRUE = allow user input, FALSE = do not allow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void enableInput ( boolean enable ) {
      this.enabled = enabled ;
   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon
    *  associated with this content pane
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.HELP ) ) ;
   }





   /*------------------------------------------------------------*/
   /* setPageContent
    */
   /**
    *    This method will set the help page's
    *  content. Keep in mind that all content is wrapped up
    *  in the default help page.
    *  
    *
    *  @param helpUrl The URL that the help file is at
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setPageContent ( String helpUrl ) {

      this.helpUrl = helpUrl ;
      helpPage = null ;

      if ( browser != null )
         browser.loadURL ( helpUrl ) ;

   }




   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when
    *  an action event is fired upon us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Close                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE )
         WinUtil.getParentJDialog ( this ).dispose() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Index                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_INDEX ) {
         HelpSystem.showHelp ( "app/index.html" ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Help on Help                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_HELP ) {
         HelpSystem.showHelp ( "app/helponhelp.html" ) ;
      }
   }



   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Cut As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Paste As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Copy As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called when the user
    *  wishes to refresh the QuestPanel. DO NOT EVER CALL revalidate
    *  in this method, the default operation for this button is to
    *  have the AppDefaultWin holding the QuestPanel to repaint everything
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
   }


   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the
    *  user has selected our panel from a
    *  tabbed pane. If we are in MDI mode, this method is never called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {

   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel onto another. This method
    *  will never be called in the MDI mode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }




   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    this method will return the name of this panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_HELP ) ) ;
   }





   /*------------------------------------------------------------*/
   /* setHelpWin
    */
   /**
    *    This method will set the HelpWindow instance
    *  that this HelpContentPanel is a child of
    *  
    *
    *  @param helpWin The HelpWindow instance that this panel is a child of
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setHelpWin ( HelpWindow helpWin ) {
      this.helpWin = helpWin ;
   }

   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* setTitlePanel
    */
   /**
    *    This method will set the title panel that we
    *  will display the title of help pages on
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setTitlePanel ( ApplicationTitlePanel titlePanel ) {
      this.titlePanel = titlePanel ;
   }




   /*------------------------------------------------------------*/
   /* pageStopped
    */
   /**
    *    When the page is done loading, we will do
    *  our own parsing to get the left navigation
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void pageStopped () {

      helpPage = new HelpPage ( browser.getSource() ) ;                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we got a 404, put up a nice error message            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( helpPage.getTitle().indexOf ( "404" ) >=0 ) {
         browser.setPageContent ( Str.getStr ( STR_HELP_NOT_FOUND ) ) ;
         setTitle ( Str.getStr ( STR_HELP_NOT_FOUND_TITLE ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, refresh the nav list                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {

         if ( navPanel != null ) {
            navPanel.setHelpPage ( helpPage ) ;
            navPanel.refreshNavList() ;
         }

         setTitle ( helpPage.getTitle() ) ;
      }

   }

   public void mouseLinkExit() {
   }

   public void mouseOverLink ( String string ) {
   }

   public void pageStarted ( String string ) {
      setTitle ( "Loading..." ) ;
   }

   public boolean linkClicked ( String link ) {
      return( true ) ;
   }






   /*------------------------------------------------------------*/
   /* setTitle
    */
   /**
    *    This method will set the text on our title
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setTitle ( String title ) {
      if ( titlePanel != null )
         titlePanel.setTitle ( title ) ;
   }


   public void statusChange ( String status, int value, int max ) {
   }
} /* HelpContentPanel */
