package com.ibm.nzna.projects.qit.help ;      
/*-----------------------------------------------------------*/
/*   HelpNavPanel                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HelpNavPanel                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.* ; 
import com.ibm.nzna.projects.qit.app.* ; 
import com.ibm.nzna.projects.qit.help.tutor.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This is the navigation panel for the Help Window 
 *
 *
 * @version 1.0, 1/13/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HelpNavPanel                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class HelpNavPanel extends JPanel implements ActionListener, AppConst, NavPanel {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private NavList          navList         = null ;
   private NavListGroup     searchGroup     = null ;
   private NavListGroup     howToGroup      = null ;
   private NavListGroup     shortCutGroup   = null ;
   private NavListGroup     tutorialGroup   = null ;
   private NavListGroup     seeAlsoGroup    = null ;
   private HelpPage         helpPage        = null ;
   private HotLinkLabel     st_HELPFULHINTS = null ;
   private HotLinkLabel     st_SEARCHALL    = null ;
   private HelpContentPanel sibPanel        = null ;
   private JScrollPane      navListScroll   = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will construct our nav panel
    *  
    *  
    *
    *  @param helpPage The Help Page to use
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HelpNavPanel ( HelpContentPanel sibPanel, HelpPage helpPage ) {
      this.sibPanel = sibPanel ;
      setHelpPage ( helpPage ) ;
   }


   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will be called before the
    *  nav panel is shown. We will initialize GUI,
    *  and other vars here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {               

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() );
      setBackground ( Color.white ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Populate with initial values                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshNavList() ;
   }






   /*------------------------------------------------------------*/
   /* setHelpPage
    */
   /**
    *    This method will set the HelpPage
    *  that is being displayed. It will not refresh
    *  the navigation list though, a separate call to refreshNavList would be needed
    *
    *  @param helpPage The new HelpPage to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setHelpPage ( HelpPage helpPage ) {
      this.helpPage = helpPage ;
   }



   /*------------------------------------------------------------*/
   /* refreshNavList
    */
   /**
    *    This method will refresh the nav list
    *  to the current help page being displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshNavList() {

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, kill the old list and create a new one           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( navListScroll == null ) {
            navListScroll = new JScrollPane() ;
            add ( navListScroll, BorderLayout.CENTER ) ;
         }

         if ( navList != null )
            navListScroll.remove ( navList ) ;

         navList = new NavList() ;
         navListScroll.getViewport().setView ( navList ) ;                       


         if ( helpPage != null ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now process each set in the list                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( helpPage.getHowTos() != null ) {
               navList.add ( howToGroup    = new NavListGroup ( Str.getStr ( STR_HOWTO ) ) ) ;
               addHowTos() ;
            }

            if ( helpPage.getShortCuts() != null ) {
               navList.add ( shortCutGroup = new NavListGroup ( Str.getStr ( STR_SHORTCUTS ) ) ) ;
               addShortCuts() ;
            }

            if ( helpPage.getSeeAlso() != null ) {
               navList.add ( seeAlsoGroup  = new NavListGroup ( Str.getStr ( STR_SEEALSO ) ) ) ;
               addSeeAlso() ;
            }

            if ( helpPage.getTutorial() != null ) {
               navList.add ( tutorialGroup = new NavListGroup ( Str.getStr ( STR_TUTORIAL ) ) ) ;
               addTutorials() ;
            }

            //navList.add ( searchGroup = new NavListGroup ( Str.getStr ( STR_SEARCH ) ) ) ;
            //searchGroup.add ( st_SEARCHALL    = new HotLinkLabel ( Str.getStr ( STR_ALLHELP ) )  ) ;    

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set our icons                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            //searchGroup.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_SEARCH ) ) ;
            if ( seeAlsoGroup != null )
               seeAlsoGroup.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.SEE_ALSO ) ) ;
            if ( shortCutGroup != null )
               shortCutGroup.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.SHORT_CUT ) ) ;
            if ( howToGroup != null )
               howToGroup.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_HELP ) ) ;
            if ( tutorialGroup != null )
               tutorialGroup.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.SMALL_MASCOT ) ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add action listeners                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            //st_SEARCHALL.addActionListener    ( this ) ;         


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Expand our groups                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( howToGroup != null )
               howToGroup.expand() ;


            if ( seeAlsoGroup != null )
               seeAlsoGroup.expand() ;

            if ( shortCutGroup != null )
               shortCutGroup.expand() ;

            if ( tutorialGroup != null )
               tutorialGroup.expand() ;

         }
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once the
    *  panel is shown.
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
   }

   public void stop () {
   }





   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title of the
    *  nav panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_HELP ) ) ;
   }



   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance
    *  of this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* addHowTos
    */
   /**
    *    This method will add all of the 'How To'
    *  items to the 'How To' list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addHowTos () {
      String[][] howToArray = helpPage.getHowTos() ;

      if ( ( howToArray != null ) && ( howToArray.length > 0 ) ) {
         int          i     = 0 ;
         HotLinkLabel label = null ;

         while ( i < howToArray.length ) {
            howToGroup.add ( label = new HotLinkLabel ( howToArray[i][1] ) ) ;
            label.addActionListener ( this ) ;
            label.setActionCommand ( "HOW_TO" ) ;
            i++ ;
         }
      }
   }


   /*------------------------------------------------------------*/
   /* addShortCuts
    */
   /**
    *    This method will add all of the 'How To'
    *  items to the 'How To' list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addShortCuts () {
      String[][] shortCutArray = helpPage.getShortCuts() ;

      if ( ( shortCutArray != null ) && ( shortCutArray.length > 0 ) ) {
         int          i     = 0 ;
         HotLinkLabel label = null ;

         while ( i < shortCutArray.length ) {
            shortCutGroup.add ( label = new HotLinkLabel ( shortCutArray[i][1] ) ) ;
            label.addActionListener ( this ) ;
            label.setActionCommand ( "SHORT_CUT" ) ;
            i++ ;
         }
      }
   }



   /*------------------------------------------------------------*/
   /* addSeeAlso
    */
   /**
    *    This method will add all of the 'See Also'
    *  items to the 'See Also' list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addSeeAlso () {
      String[][] seeAlsoArray = helpPage.getSeeAlso() ;

      if ( ( seeAlsoArray != null ) && ( seeAlsoArray.length > 0 ) ) {
         int             i     = 0 ;
         HotLinkLabel    label = null ;

         while ( i < seeAlsoArray.length ) {
            seeAlsoGroup.add ( label = new HotLinkLabel ( seeAlsoArray[i][1] ) ) ;
            label.addActionListener ( this ) ;
            label.setActionCommand ( "SEE_ALSO" ) ;
            i++ ;
         }
      }
   }


   /*------------------------------------------------------------*/
   /* addTutorials
    */
   /**
    *    This method will add all of the 'Tutorials'
    *  items to the 'See Also' list
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addTutorials () {
      String[][] tutorialArray = helpPage.getTutorial() ;

      if ( ( tutorialArray != null ) && ( tutorialArray.length > 0 ) ) {
         int             i     = 0 ;
         HotLinkLabel    label = null ;

         while ( i < tutorialArray.length ) {
            tutorialGroup.add ( label = new HotLinkLabel ( tutorialArray[i][1] ) ) ;
            label.addActionListener ( this ) ;
            label.setActionCommand ( "TUTORIAL" ) ;
            i++ ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called once an
    *  action event is fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) { 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Search All                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == st_SEARCHALL ) {
         sibPanel.searchAll(true) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Handle any How Tos.                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getActionCommand().equals ( "HOW_TO" ) ) {
         String destination = helpPage.destFromHowTo ( ((HotLinkLabel)e.getSource()).getText() ) ;

         HelpSystem.showHelp ( destination ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Handle any 'See Also'                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getActionCommand().equals ( "SEE_ALSO" ) ) {
         String destination = helpPage.destFromSeeAlso ( ((HotLinkLabel)e.getSource()).getText() ) ;

         HelpSystem.showHelp ( destination ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Short Cuts                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getActionCommand().equals ( "SHORT_CUT" ) ) {
         String destination = helpPage.destFromShortCut ( ((HotLinkLabel)e.getSource()).getText() ) ;

         HelpSystem.showHelp ( destination ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Tutorials                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getActionCommand().equals ( "TUTORIAL" ) ) {
         int tutorNum = helpPage.destFromTutorial ( ((HotLinkLabel)e.getSource()).getText() ) ;

         new TutorWin ( false, tutorNum ) ;

      }

   }


} /* HelpNavPanel */
