package com.ibm.nzna.projects.qit.help ;  
/*-----------------------------------------------------------*/
/*   HelpPage                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HelpPage                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.io.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * The HelpPage object holds information about a specific help
 * page. This information includes the 'How To', 'Generic Help',
 * 'See Also' etc.. The constructor itself will parse the
 * help contents given. This parser is in no way forgiving at
 * all. Every tag MUST BE IN UPPER CASE, and have an ending tag.
 *
 * @version 1.0, 1/15/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HelpPage                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class HelpPage extends Object implements AppConst {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Static Members                                          */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static final String wrapper = "<TABLE BORDER=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\" WIDTH=\"90%\">"+
                                          //"    <TR>"+
                                          //"	    <TD COLSPAN=\"2\" BGCOLOR=\"navy\"><FONT SIZE=\"6\" COLOR=\"white\" FACE=\"Arial, Helvetica, sans-serif\"> #TITLE</FONT></TD>"+
                                          //"    </TR>"+
                                          "    <TR>"+
                                          "	    <TD WIDTH=\"3%\">&nbsp;&nbsp</TD>"+
                                          "	    <TD WIDTH=\"97%\"><p><FONT SIZE=\"2\" COLOR=\"black\" FACE=\"Arial, Helvetica, sans-serif\">#CONTENT</font></TD>"+
                                          "    </TR>"+
                                          "/TABLE>" ;

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private String      generalHelp          = "Malformed Help File. No GENERAL HELP tag defined" ;
    private String      title                = "No title" ;
    private String[][]  seeAlsoArray         = null ;
    private String[][]  howToArray           = null ;
    private String[][]  shortCutArray        = null ;
    private String[][]  tutorialArray        = null ;
    private String      helpfulHintDest      = null ;






    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will take the help contents
     *  of a help file, and will parse them into
     *  their correct places
     *
     *  @param content The content to parse and create an object from
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public HelpPage ( String contents ) {
        parse ( contents ) ;
    }




    /*------------------------------------------------------------*/
    /* parse
     */
    /**
     *    This method will parse a string down
     *  into the contents of this object
     *  
     *
     *  @param contents The contents to parse
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void parse ( String contents ) {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Parse the Generic Help                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( contents.indexOf ( "<GENERAL HELP>" ) >= 0 ) {
            generalHelp = contents.substring ( contents.indexOf ( "<GENERAL HELP>" )+14 ) ;
            generalHelp = generalHelp.substring ( 0, generalHelp.indexOf ( "</GENERAL HELP>" ) ).trim() ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Get the title                                           */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( contents.indexOf ( "<TITLE>" ) >= 0 ) {
            title = contents.substring ( contents.indexOf ( "<TITLE>" )+7 ) ;
            title = title.substring ( 0, title.indexOf ( "</TITLE>" ) ).trim() ;
        }
        else if ( contents.indexOf ( "<title>" ) >= 0 ) {
            title = contents.substring ( contents.indexOf ( "<title>" )+7 ) ;
            title = title.substring ( 0, title.indexOf ( "</title>" ) ).trim() ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Get the Helpful  Hints                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( contents.indexOf ( "<HELPFUL HINTS>" ) >= 0 ) {
            helpfulHintDest = contents.substring ( contents.indexOf ( "<HELPFUL HINTS>" )+16 ) ;
            helpfulHintDest = helpfulHintDest.substring ( 0, helpfulHintDest.indexOf ( "</HELPFUL HINTS>" ) ).trim() ;
        }



        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Parse the 'See Also' tags                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        parseSeeAlso ( contents ) ;              

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Parse the 'How To' tags                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        parseHowTo ( contents ) ;              

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Parse the 'Short Cur' tags                              */
        /*                                                           */
        /*-----------------------------------------------------------*/
        parseShortCut ( contents ) ;                             

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Parse the 'Tutorial' Tags                               */                
        /*                                                           */
        /*-----------------------------------------------------------*/
        parseTutorial ( contents ) ;
        


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Wrap our table around the general help                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        //generalHelp = wrapHelp ( generalHelp ) ;
    }





    /*------------------------------------------------------------*/
    /* parseSeeAlso
     */
    /**
     *    This method will parse the 'See Also'
     *  tags from the help file
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void parseSeeAlso ( String contents ) {
        String beginTag = "<SEE ALSO DESTINATION" ;
        String endTag   = "</SEE ALSO>" ;
        int    index    = contents.indexOf ( beginTag ) ;
        int    count    = 0 ;
        String dest     = "" ;
        String descript = "" ;      

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Count the items                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        while ( index >= 0 ) {
            index+=10 ;
            count++ ;
            index = contents.indexOf ( beginTag, index ) ;
        }

        if ( count > 0 ) {

            seeAlsoArray = new String [count][2] ;
            count=0 ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through and process each See ALso                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            index    = contents.indexOf ( beginTag ) ;
            while ( index >= 0 ) {
                index+=beginTag.length()+2  ;   // Bump the index passed the string and =" 
                dest = contents.substring ( index, contents.indexOf ( "\"", index ) ) ;
                index=contents.indexOf ( ">", index )+1 ;
                descript = contents.substring ( index, contents.indexOf ( endTag, index ) ) ;

                seeAlsoArray[count][0] = dest.trim() ;
                seeAlsoArray[count][1] = descript.trim() ;

                index = contents.indexOf ( beginTag, index ) ;
                count++ ;
            }
        }
    }

    /*------------------------------------------------------------*/
    /* parseHowTo
     */
    /**
     *    This method will parse the 'See Also'
     *  tags from the help file
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void parseHowTo ( String contents ) {
        String beginTag = "<HOW TO DESTINATION" ;
        String endTag   = "</HOW TO>" ;
        int    index    = contents.indexOf ( beginTag ) ;
        int    count    = 0 ;
        String dest     = "" ;
        String descript = "" ;      

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Count the items                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        while ( index >= 0 ) {
            index+=10 ;
            count++ ;
            index = contents.indexOf ( beginTag, index ) ;
        }

        if ( count > 0 ) {

            howToArray = new String [count][2] ;
            count=0 ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through and process each See ALso                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            index    = contents.indexOf ( beginTag ) ;
            while ( index >= 0 ) {
                index+=beginTag.length()+2  ;   // Bump the index passed the string and =" 
                dest = contents.substring ( index, contents.indexOf ( "\"", index ) ) ;
                index=contents.indexOf ( ">", index )+1 ;
                descript = contents.substring ( index, contents.indexOf ( endTag, index ) ) ;

                howToArray[count][0] = dest.trim() ;
                howToArray[count][1] = descript.trim() ;

                index = contents.indexOf ( beginTag, index ) ;
                count++ ;
            }
        }
    }

    /*------------------------------------------------------------*/
    /* parseShortCut
     */
    /**
     *    This method will parse the 'See Also'
     *  tags from the help file
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void parseShortCut ( String contents ) {
        String beginTag = "<SHORT CUT DESTINATION" ;
        String endTag   = "</SHORT CUT>" ;
        int    index    = contents.indexOf ( beginTag ) ;
        int    count    = 0 ;
        String dest     = "" ;
        String descript = "" ;      

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Count the items                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        while ( index >= 0 ) {
            index+=10 ;
            count++ ;
            index = contents.indexOf ( beginTag, index ) ;
        }

        if ( count > 0 ) {

            shortCutArray = new String [count][2] ;
            count=0 ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through and process each See ALso                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            index    = contents.indexOf ( beginTag ) ;
            while ( index >= 0 ) {
                index+=beginTag.length()+2  ;   // Bump the index passed the string and =" 
                dest = contents.substring ( index, contents.indexOf ( "\"", index ) ) ;
                index=contents.indexOf ( ">", index )+1 ;
                descript = contents.substring ( index, contents.indexOf ( endTag, index ) ) ;

                shortCutArray[count][0] = dest.trim() ;
                shortCutArray[count][1] = descript.trim() ;

                index = contents.indexOf ( beginTag, index ) ;
                count++ ;
            }
        }
    }



    /*------------------------------------------------------------*/
    /* parseTutorial
     */
    /**
     *    This method will parse the 'Tutorials'
     *  tags from the help file
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void parseTutorial ( String contents ) {
        String beginTag = "<TUTORIAL" ;
        String endTag   = "</TUTORIAL>" ;
        int    index    = contents.indexOf ( beginTag ) ;
        int    count    = 0 ;
        String dest     = "" ;
        String descript = "" ;      

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Count the items                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        while ( index >= 0 ) {
            index+=10 ;
            count++ ;
            index = contents.indexOf ( beginTag, index ) ;
        }

        if ( count > 0 ) {

            tutorialArray = new String [count][2] ;
            count=0 ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through and process each Tutorial                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            index    = contents.indexOf ( beginTag ) ;
            while ( index >= 0 ) {
                index+=beginTag.length()+1  ;   // Bump the index passed the string and =" 
                dest = contents.substring ( index, contents.indexOf ( ">", index ) ) ;
                index=contents.indexOf ( ">", index )+1 ;
                descript = contents.substring ( index, contents.indexOf ( endTag, index ) ) ;

                tutorialArray[count][0] = dest.trim() ;
                tutorialArray[count][1] = descript.trim() ;

                index = contents.indexOf ( beginTag, index ) ;
                count++ ;
            }
        }
    }






    /*------------------------------------------------------------*/
    /* getGeneralHelp
     */
    /**
     *    This method will return the General
     *  Help section of this help page
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getGeneralHelp () {
        return( "<html><body>"+generalHelp+"</body></html>" ) ;
    }




    /*------------------------------------------------------------*/
    /* getTitle
     */
    /**
     *    This method will return the title
     *  of this help page
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getTitle () {
        return( title ) ;
    }




    /*------------------------------------------------------------*/
    /* getHelpfulHint
     */
    /**
     *    This method will return the helpful hint
     *  destination file.
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getHelpfulHint () {
        return( helpfulHintDest ) ;
    }




    /*------------------------------------------------------------*/
    /* getSeeAlso
     */
    /**
     *    This method will return a two dimensional
     *  array of See Also. The first dimension holds
     *  the See Also destination, the second holds the description
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String[][]
     *//*---------------------------------------------------------*/
    public String[][]getSeeAlso() {
        return( seeAlsoArray ) ;
    }


    /*------------------------------------------------------------*/
    /* getHowTos
     */
    /**
     *    This method will return a two dimensional
     *  array of how tos. The first dimension holds
     *  the How To destination, the second holds the description
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String[][]
     *//*---------------------------------------------------------*/
    public String[][]getHowTos() {
        return( howToArray ) ;
    }

    /*------------------------------------------------------------*/
    /* getShortCuts
     */
    /**
     *    This method will return a two dimensional
     *  array of short cuts. The first dimension holds
     *  the Short Cut destination, the second holds the description
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String[][]
     *//*---------------------------------------------------------*/
    public String[][]getShortCuts() {
        return( shortCutArray ) ;
    }

    /*------------------------------------------------------------*/
    /* getTutorial
     */
    /**
     *    This method will return a two dimensional
     *  array of short cuts. The first dimension holds
     *  the Short Cut destination, the second holds the description
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String[][]
     *//*---------------------------------------------------------*/
    public String[][]getTutorial() {
        return( tutorialArray ) ;
    }


    /*------------------------------------------------------------*/
    /* wrapHelp
     */
    /**
     *    This method will wrap the contents
     *  of a help page inside of our table
     *  
     *
     *  @param contents The contents to wrap
     *  @param title The title of the page
     *  @param None
     *
     *  @return Wrapped Content
     *//*---------------------------------------------------------*/
    private String wrapHelp ( String contents ) {
        String wrapped  = contents+"" ;
        String homeDir  = "file:/"+PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR )+File.separator ;
        String language = PropertySystem.getString ( PropertySystem.PROPERTY_LANGUAGEFILE ) ;

        //if ( contents != null ) {
        //    wrapped = Text.replaceAllStrInStr ( wrapper, "#TITLE", title ) ;

        //    wrapped = Text.replaceAllStrInStr ( wrapped, "#CONTENT", contents ) ;
        //} else {
        //    wrapped = Text.replaceAllStrInStr ( wrapper, 
        //                                        "<p><center><FONT SIZE=\"4\" COLOR=\"white\" FACE=\"Arial, Helvetica\">"+
        //                                        Str.getStr ( STR_NOHELPFOUND )+"<br><br>Quest</center></p>", title ) ;
        //    wrapped = Text.replaceAllStrInStr ( wrapped, "#CONTENT", Str.getStr ( STR_INFO ) ) ;
        //}

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Ok, finally replace all img tags                        */                       
        /*                                                           */
        /*-----------------------------------------------------------*/
        //wrapped = Text.replaceAllStrInStr ( wrapped, "<img src=\"", "<img src=\""+homeDir ) ;
        //wrapped = Text.replaceAllStrInStr ( wrapped, "<IMG SRC=\"", "<img src=\""+homeDir ) ;
        
        return( wrapped ) ;
    }







    /*------------------------------------------------------------*/
    /* destFromSeeAlso
     */
    /**
     *    This method will take a descriptive version
     *  of a how to string and return's the HTML
     *  destination. (Where the SeeAlso points)
     *
     *  @param descript The descriptive text
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String destFromSeeAlso ( String descript ) {
        String ret = null ;

        if ( seeAlsoArray != null ) {
            int i = 0 ;

            while ( ( ret == null ) && ( i < seeAlsoArray.length ) ) {
                if ( seeAlsoArray[i][1].equals ( descript ) )
                    ret = seeAlsoArray[i][0] ;
                i++ ;
            }
        }

        return ( ret ) ;
    }


    /*------------------------------------------------------------*/
    /* destFromShortCut
     */
    /**
     *    This method will take a descriptive version
     *  of a how to string and return's the HTML
     *  destination. (Where the ShortCut points)
     *
     *  @param descript The descriptive text
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String destFromShortCut ( String descript ) {
        String ret = null ;

        if ( shortCutArray != null ) {
            int i = 0 ;

            while ( ( ret == null ) && ( i < shortCutArray.length ) ) {
                if ( shortCutArray[i][1].equals ( descript ) )
                    ret = shortCutArray[i][0] ;
                i++ ;
            }
        }

        return ( ret ) ;
    }

    /*------------------------------------------------------------*/
    /* destFromTutorial
     */
    /**
     *    This method will take a descriptive version
     *  of a Tutorial string and return's the tutor number
     *  destination. (Where the Tutorial points)
     *
     *  @param descript The descriptive text
     *  @param None
     *  @param None
     *
     *  @return int
     *//*---------------------------------------------------------*/
    public int destFromTutorial ( String descript ) {
        int ret = -2 ;

        if ( tutorialArray != null ) {
            int i = 0 ;

            while ( ( ret == -2 ) && ( i < tutorialArray.length ) ) {
                if ( tutorialArray[i][1].equals ( descript ) )
                    ret = new Integer ( tutorialArray[i][0] ).intValue() ;
                i++ ;
            }
        }

        return ( ret ) ;
    }




    /*------------------------------------------------------------*/
    /* destFromHowTo
     */
    /**
     *    This method will take a descriptive version
     *  of a how to string and return's the HTML
     *  destination. (Where the HowTo points)
     *
     *  @param descript The descriptive text
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String destFromHowTo ( String descript ) {
        String ret = null ;

        if ( howToArray != null ) {
            int i = 0 ;

            while ( ( ret == null ) && ( i < howToArray.length ) ) {
                if ( howToArray[i][1].equals ( descript ) )
                    ret = howToArray[i][0] ;
                i++ ;
            }
        }

        return ( ret ) ;
    }

} /* HelpPage */
