package com.ibm.nzna.projects.qit.help ;      
/*-----------------------------------------------------------*/
/*   HelpSearchNavPanel                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HelpSearchNavPanel                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.* ; 
import com.ibm.nzna.projects.qit.app.* ; 
/**
 * This is the search panel that will show in the navigational
 * area of the search window. This panel will actually just send
 * a message to the content panel to initiate a search.
 *
 *
 * @version 1.0, 1/13/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HelpSearchNavPanel                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class HelpSearchNavPanel extends JPanel implements ActionListener, AppConst, NavPanel {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Vars                                            */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private QuestPanel   sibPanel            = null ;
    private JLabel       st_SEARCHTITLE      = null ;
    private JLabel       st_CRITERIA         = null ;
    private JLabel       st_SEARCHTEXT       = null ;
    private JTextField   ef_SEARCHTEXT       = null ;
    private JRadioButton rb_MATCHPHRASE      = null ;
    private JRadioButton rb_CONTAINSALLWORDS = null ;
    private JRadioButton rb_CONTAINSANYWORD  = null ;
    private ButtonGroup  buttonGroup         = null ;






    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will construct our nav panel
     *  
     *  
     *
     *  @param helpPage The Help Page to use
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public HelpSearchNavPanel ( QuestPanel sibPanel ) {
        this.sibPanel = sibPanel ;
    }


    /*------------------------------------------------------------*/
    /* initialize
     */
    /**
     *    This method will be called before the
     *  nav panel is shown. We will initialize GUI,
     *  and other vars here
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void initialize () {                    

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create our controls                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        buttonGroup = new ButtonGroup() ;
        st_SEARCHTITLE      = new JLabel       ( Str.getStr ( STR_SEARCH ) ) ;        
        rb_MATCHPHRASE      = new JRadioButton ( Str.getStr ( STR_MATCHPHRASEEXACTLY ) ) ;
        rb_CONTAINSALLWORDS = new JRadioButton ( Str.getStr ( STR_CONTAINSALLWORDS ) ) ;
        rb_CONTAINSANYWORD  = new JRadioButton ( Str.getStr ( STR_CONTAINSANYWORD ) ) ;
        st_CRITERIA         = new JLabel       ( Str.getStr ( STR_CRITERIA ) ) ;
        st_SEARCHTEXT       = new JLabel       ( Str.getStr ( STR_SEARCHTEXT ) ) ;
        ef_SEARCHTEXT       = new JTextField   ( "" ) ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust Properties                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        st_SEARCHTITLE.setIcon ( ImageSystem.getImageIcon ( this, ImageSystem.SEARCH ) ) ;       
        st_SEARCHTITLE.setFont ( FontSystem.largeTitleFont ) ;
        st_SEARCHTEXT.setFont  ( FontSystem.smallTitleFont ) ;
        st_CRITERIA.setFont    ( FontSystem.smallTitleFont ) ;
        setBackground ( Color.white ) ;
        rb_MATCHPHRASE.setBackground ( Color.white ) ;
        rb_CONTAINSALLWORDS.setBackground ( Color.white ) ;
        rb_CONTAINSANYWORD.setBackground ( Color.white ) ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our controls                                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setLayout ( null ) ;
        buttonGroup.add ( rb_CONTAINSALLWORDS ) ;
        buttonGroup.add ( rb_MATCHPHRASE ) ;
        buttonGroup.add ( rb_CONTAINSANYWORD ) ;
        add ( st_SEARCHTITLE ) ;
        add ( st_CRITERIA ) ;
        add ( rb_CONTAINSALLWORDS ) ;
        add ( rb_MATCHPHRASE ) ;
        add ( rb_CONTAINSANYWORD ) ;
        add ( st_SEARCHTEXT ) ;
        add ( ef_SEARCHTEXT ) ;
    }






    /*------------------------------------------------------------*/
    /* doLayout
     */
    /**
     *    This method will lay out and position
     *  all of our components
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void doLayout () {
        Dimension size      = getSize() ;
        int       rowHeight = GUISystem.getRowHeight() ;
        int       y         = rowHeight*3 ;

        super.doLayout() ;

        st_SEARCHTITLE.setBounds ( 5, 0, size.width, rowHeight*2 ) ;
        
        st_CRITERIA.setBounds ( 5, y, size.width, rowHeight ) ;
        y+=rowHeight ;

        rb_CONTAINSALLWORDS.setBounds ( 10, y, size.width-10, rowHeight ) ;
        y+=rowHeight ;

        rb_MATCHPHRASE.setBounds ( 10, y, size.width-10, rowHeight ) ;
        y+=rowHeight ;

        rb_CONTAINSANYWORD.setBounds ( 10, y, size.width-10, rowHeight ) ;
        y+=rowHeight*2 ;

        st_SEARCHTEXT.setBounds ( 5, y, size.width-5, rowHeight ) ;
        y+=rowHeight ;

        ef_SEARCHTEXT.setBounds ( 10, y, size.width-15, rowHeight ) ;
    }

    /*------------------------------------------------------------*/
    /* start
     */
    /**
     *    This method will be called once the
     *  panel is shown.
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void start () {
        rb_CONTAINSALLWORDS.setSelected ( true ) ;
        ef_SEARCHTEXT.requestFocus() ;
    }

    public void stop () {
    }





    /*------------------------------------------------------------*/
    /* getTitle
     */
    /**
     *    This method will return the title of the
     *  nav panel
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getTitle () {
        return( Str.getStr ( STR_SEARCH ) ) ;
    }



    /*------------------------------------------------------------*/
    /* getJComponent
     */
    /**
     *    This method will return an instance
     *  of this panel
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return JComponent
     *//*---------------------------------------------------------*/
    public JComponent getJComponent () {
        return( this ) ;
    }



    /*------------------------------------------------------------*/
    /* actionPerformed
     */
    /**
     *    This method will be called once an
     *  action event is fired
     *  
     *
     *  @param e The event
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void actionPerformed ( ActionEvent e ) { 

    }





    /*------------------------------------------------------------*/
    /* getPreferredSize
     */
    /**
     *    This method will return the preferred
     *  size of our panel
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Dimension getMinimumSize () {
        return( GUISystem.defaultNavSize ) ;
    }

} /* HelpSearchNavPanel */
