package com.ibm.nzna.projects.qit.help ;    
/*-----------------------------------------------------------*/
/*   HelpSystem                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HelpSystem                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import java.io.* ;
import java.awt.* ;
/**
 *  The Help system is used to create a consistent look and 
 * feel across the Quest application for on line help. Each QuestApplet 
 * is responsible for the actual Help Content. The Help window 
 * and navigation is handled through the HelpWindow which 
 * is part of the Help System.
 *
 *
 * @version 1.0, 1/12/2000
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HelpSystem                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class HelpSystem extends Object implements Runnable, AppConst {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private        HelpWindow helpWindow       = null ;
   private static HelpSystem helpSystem       = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Creates and initializes the HelpSystem
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HelpSystem() {
      helpSystem = this ;
   }





   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return an instance
    *  of the HelpSystem
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return HelpSystem
    *//*---------------------------------------------------------*/
   public static HelpSystem getInstance() {
      return( helpSystem ) ;
   }




   /*------------------------------------------------------------*/
   /* showHelp
    */
   /**
    *    This method will show a specific
    *  help file. It will be done inside of another thread so this
    *  method will immediatly return.
    *  
    *
    *  @param filename The name of the help file to look. This filename
    *                  should be relative to the userdir/help directory.
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void showHelp ( String filename ) {
      new Thread ( getInstance(), filename ).start() ;
   }




   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This method is the thread that will show
    *  the help window and help file
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String filename = Thread.currentThread().getName() ;

      try {
         String   helpContents = null ;
         String   url          = null ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Prepend the URL to the filename                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         url = Constants.getStrConst ( "IT_HELP_URL" ) ;
         if ( url == null )
            url = URL_ONLINE_HELP_NO_DATABASE ;

         filename = url+"/"+
                    PropertySystem.getString ( PropertySystem.PROPERTY_LANGUAGEFILE )+"/"+
                    filename ;
         System.out.println ( "Filename:"+filename ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Replace any backslashes with forward slashes            */                                   
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( filename.indexOf ( "\\" ) >=0 )
            filename = Text.replaceAllStrInStr ( filename, "\\", "/" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Form the HelpPage                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( helpWindow == null ) {
            helpWindow = new HelpWindow ( filename ) ;
            helpWindow.setVisible ( true ) ;
            helpWindow = null ;
         } else
            helpWindow.showHelp ( filename ) ;

      } catch ( Exception e ) {
         if ( helpWindow != null )
            helpWindow.dispose() ;

         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_HELPFILENOTFOUND )+" "+filename ) ;
      }
   }






   /*------------------------------------------------------------*/
   /* getHelpWindow
    */
   /**
    *    This method will return an instance
    *  to the help window currently being
    *  shown
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static HelpWindow getHelpWindow () {
      return( getInstance().helpWindow ) ;
   }

} /* HelpSystem */
