package com.ibm.nzna.projects.qit.help ;    
/*-----------------------------------------------------------*/
/*   HelpWindow                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: HelpWindow                          */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import java.awt.event.* ;
import java.awt.* ;
import javax.swing.* ;
/**
 * This Help Window is just the frame that holds a toolbar, 
 * and application panel
 *
 * @version 1.0, 1/15/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   HelpWindow                                              */
/*                                                           */
/*-----------------------------------------------------------*/
public class HelpWindow extends JDialog implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private HelpContentPanel contentPanel         = null ;
   private String           contents             = null ;
   private String           filename             = null ;
   private ApplicationPanel helpApplicationPanel = null ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create/show the help
    *  window
    *  
    *
    *  @param helpPage The help page to show initially
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public HelpWindow ( String filename ) {
      super ( MainWindow.getInstance(), Str.getStr ( STR_HELP ), true ) ;
      QuestPanelInitializeThread initThread = null ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our controls                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         contentPanel = new HelpContentPanel ( filename ) ;
         contentPanel.setHelpWin ( this ) ;
         helpApplicationPanel = new ApplicationPanel () ;
         initThread   = new QuestPanelInitializeThread ( helpApplicationPanel, contentPanel, false ) ;
         initThread.run() ;
         helpApplicationPanel = initThread.getApplicationPanel() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust our properties                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setSize ( 630, 450 ) ;
         //setIconImage ( ImageSystem.getImage ( this, ImageSystem.HELP ) ) ;               

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Assign our vars                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         this.filename = filename ;
         this.contents = contents ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our content Pane                                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         getContentPane().setLayout ( new BorderLayout () ) ;
         getContentPane().add ( helpApplicationPanel, BorderLayout.CENTER ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust our position                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
         WinUtil.centerWindow ( this ) ;
         setTitle ( Str.getStr ( STR_HELP ) ) ;
         contentPanel.setTitlePanel ( helpApplicationPanel.getTitlePanel() ) ;

         if ( filename != null )
            showHelp ( filename ) ;

      } catch ( Exception e ) {
         setVisible ( false ) ;
         LogSystem.log ( 1, e ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* showHelp
    */
   /**
    *    This method will show a help file, along
    *  with the default navigation on the left
    *  
    *
    *  @param helpUrl The URL to the Help File
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void showHelp ( String helpUrl ) {
      if ( helpApplicationPanel != null )
         helpApplicationPanel.setPreferredNavPanelSize () ;
      contentPanel.setPageContent ( helpUrl ) ;
   }





   /*------------------------------------------------------------*/
   /* windowClosing
    */
   /**
    *    This method will be called if the user
    *  chooses to close this window, or if
    *  anything else chooses to close  this window. This method is
    *  subclassed so that we don't close, we just disappear
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void windowClosing ( WindowEvent e ) {
      dispose() ;
   }


   /*------------------------------------------------------------*/
   /* addActionComponent
    */
   /**
    *    This method will add an action component
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addActionComponent ( JComponent component ) {
      helpApplicationPanel.addActionComponent ( component ) ;
   }


}

