package com.ibm.nzna.projects.qit.help.tutor ;
/*-----------------------------------------------------------*/
/*   TutorWin                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: TutorWin                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.io.* ;
import javax.swing.* ;
import java.awt.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This window shows the user the Quest Tutorial 
 *
 *
 * @version 1.0, 7/7/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   TutorWin                                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class TutorWin extends JDialog implements BrowserListener, AppConst, ActionListener {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Members                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Browser      browser     = null ;
   private DButton      pb_NEXT     = null ;
   private DButton      pb_PREV     = null ;
   private DButton      pb_CLOSE    = null ;
   private ButtonPanel  buttonPanel = null ;      


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int currentTutorPage = 0 ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will start the tutorial from the
    *  beginning
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TutorWin () {
      super ( new Frame(), Str.getStr ( STR_TUTOR ), false ) ;

      currentTutorPage = 0 ;
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will start the tutorial and will
    *  not return until the tutorial is canceled
    *  
    *
    *  @param modal TRUE = Do not return, FALSE = Return immediatly
    *  @param tutorPage The page to start the tutorial on
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public TutorWin ( boolean modal, int tutorPage ) {
      super ( new Frame(), Str.getStr ( STR_TUTOR ), modal ) ;

      currentTutorPage = tutorPage ;
      initialize () ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our GUI and
    *  show the window
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {
      Container contentPane = getContentPane() ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      browser  = GUISystem.createBrowser() ;
      pb_NEXT  = new DButton ( Str.getStr ( STR_NEXT ) ) ;
      pb_PREV  = new DButton ( Str.getStr ( STR_BACK ) ) ;
      pb_CLOSE = new DButton ( Str.getStr ( STR_CLOSE ) ) ;
      buttonPanel = new ButtonPanel () ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.setArrowImage ( ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_LEFT ),
                                  ImageSystem.getImageIcon ( this, ImageSystem.BUTTONPANEL_RIGHT ) ) ;  
      pb_NEXT.addActionListener ( this ) ;
      pb_PREV.addActionListener ( this ) ;
      pb_CLOSE.addActionListener ( this ) ;
      browser.addBrowserListener ( this ) ;




      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( browser.getComponent(), BorderLayout.CENTER ) ;
      contentPane.add ( buttonPanel, BorderLayout.SOUTH ) ;
      buttonPanel.add ( pb_PREV ) ;
      buttonPanel.add ( pb_NEXT ) ;
      buttonPanel.add ( pb_CLOSE ) ;

      GUISystem.setPropertiesOnPanel ( this ) ;                       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the size and show the window                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSize ( 630, 470 ) ;
      WinUtil.centerWindow ( this ) ;

      setVisible ( true ) ;
      showTutorPage ( currentTutorPage ) ;
   }





   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out our components
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;
      browser.getComponent().setBounds ( 15, 15, size.width-45, size.height-90 ) ;
      buttonPanel.setBounds ( 15, size.height-68, size.width-30, 25 ) ;
   }





   /*------------------------------------------------------------*/
   /* showTutorPage
    */
   /**
    *    This method will show the page number
    *  passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showTutorPage ( int tutorPage ) {
      try {
         String page = (String) PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR ) ;                                       
         String notFoundPage = (String) PropertySystem.getString ( PropertySystem.PROPERTY_HOMEDIR ) ;                                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our tutor directory and filename to the path        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         page+=File.separator+
               "help"+File.separator+
               "tutor"+File.separator+
               "content"+File.separator+
               PropertySystem.getString ( PropertySystem.PROPERTY_LANGUAGEFILE )+File.separator+
               "tutor_"+tutorPage+".html" ;                      

         notFoundPage+=File.separator+
                       "help"+File.separator+
                       "tutor"+File.separator+
                       "content"+File.separator+
                       PropertySystem.getString ( PropertySystem.PROPERTY_LANGUAGEFILE )+File.separator+
                       "notfound.html" ;                      

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Make sure the file is there                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( FileUtil.exists ( page ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Show the file                                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            browser.loadURL ( "file:/"+page ) ;

         } else
            browser.loadURL ( "file:/"+notFoundPage ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, disable some of the buttons                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_PREV.setEnabled ( tutorPage > 0 ) ;


      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are listening for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_NEXT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( pb_NEXT == e.getSource() ) {
         if ( currentTutorPage < 0 )
            currentTutorPage=0 ;
         showTutorPage ( ++currentTutorPage ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_PREV                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_PREV == e.getSource() )
         showTutorPage ( --currentTutorPage ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_CLOSE == e.getSource() )
         dispose() ;


   } /* actionPerformed  */


   public void pageStopped() {}
   public void pageStarted ( String page ) {}
   public void mouseLinkExit() {}
   public void mouseOverLink ( String link ) {}




   /*------------------------------------------------------------*/
   /* linkClicked
    */
   /**
    *    This method will determine if the new
    *  page being loaded is a tutor page and
    *  will update our currentTutorPage property if so
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean linkClicked(String link) {
      if ( link.indexOf ( "tutor_" ) >= 0 ) {
         String tutorNum = link.substring ( link.indexOf ( "tutor_" )+6 ) ;

         tutorNum = tutorNum.substring ( 0, tutorNum.indexOf ( "." ) ) ;                  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now make the assignment                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         try {
            currentTutorPage = new Integer ( tutorNum ).intValue() ;
         } catch ( Exception e ) {
         }
      }

      return ( true ) ;
   }

   public void statusChange ( String status, int value, int max ) {
   }

} /* TutorWin */
