package com.ibm.nzna.projects.qit.notify ;
/*-----------------------------------------------------------*/
/*   NotificationRec                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NotificationRec                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.sql.* ;
import java.util.* ;
import java.awt.datatransfer.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.db.* ;
/**
 * This object holds specific information for a Notification. 
 * The Notifications are stored in QUEST.NOTIFICATIONS table. 
 *
 *
 * @version 1.0, 9/21/1998 
 * @author  David Lentz
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   NotificationRec                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class NotificationRec extends PersistentRec implements MultiListSort, MultiListRow, java.io.Serializable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   This is the serialization ID. If you add fields to this */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is    */
   /* fine as long as you know what you are doing!              */
   /*                                                           */
   /*-----------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Body Preview                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String bodyTemplate =
   "<HTML><BODY>"+
   "<P><FONT FACE=\"Helvetica\" SIZE=2>"+
   "<FONT COLOR=\"#800080\">TO: </FONT>VAL_TO<BR>"+
   "<FONT COLOR=\"#800080\">FROM: </FONT>VAL_FROM<BR>"+
   "<FONT COLOR=\"#800080\">DATE: </FONT>VAL_DATE<BR>"+
   "<FONT COLOR=\"#800080\">BRAND: </FONT>VAL_BRAND<BR>"+
   "<FONT COLOR=\"#800080\">RESOLUTION: </FONT>VAL_RESOL<BR>"+
   "<FONT COLOR=\"#800080\">DOCUMENT: </FONT>VAL_DOC<BR><br>"+
   "VAL_BODY"+
   "</P></FONT></BODY></HTML>" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static                                                  */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int SEND_NEW       = 0 ;
   public static final int SEND_REPLY     = 1 ;
   public static final int SEND_FORWARD   = 2 ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public String            toUser         = null ; 
   public String            fromUser       = null ; 
   public String            subject        = null ;
   public String            prettyFrom     = null ;
   public TypeCategoryRec   brandRec       = null ;
   public TypeResolutionRec resolRec       = null ;
   public String            comments       = null ;
   public String            openDate       = null ;
   public String            closeDate      = null ;
   public int               docInd         = 0 ;
   public int               existingDocInd = 0 ;
   public int               noteInd        = 0 ;
   public boolean           read           = false ;
   public int               sendType       = SEND_NEW ;
   public boolean           notifySent     = false ;
   public boolean           mustSend       = false ;
   public int               brandInd       = 0 ;
   public int               resolInd       = 0 ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This default constructor initializes no data
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NotificationRec () {
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create and initialize
    *  this object with the values of the NotificationRec
    *  passed in
    *
    *  @param noteRec The NotificationRec to copy
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NotificationRec ( NotificationRec noteRec ) {
      toUser         = noteRec.toUser ;
      fromUser       = noteRec.fromUser ;
      subject        = noteRec.subject ;
      prettyFrom     = noteRec.prettyFrom ;
      brandRec       = noteRec.brandRec ;
      resolRec       = noteRec.resolRec ;
      comments       = noteRec.comments ;
      openDate       = noteRec.openDate ;
      closeDate      = noteRec.closeDate ;
      docInd         = noteRec.docInd ;
      existingDocInd = noteRec.existingDocInd ;
      noteInd        = noteRec.noteInd ;
      read           = noteRec.read ;
      sendType       = noteRec.sendType ;
      notifySent     = noteRec.notifySent ;
      mustSend       = noteRec.mustSend ;
      brandInd       = noteRec.brandInd ;
      resolInd       = noteRec.resolInd ;
   }


   /*-----------------------------------------------------------*/
   /*   getColumnData                                           */
   /*                                                           */
   /*   Description:                                            */
   /*      Returns a specific column's data                     */
   /*                                                           */
   /*   Parameters:                                             */
   /*      column The Column to return data to                  */
   /*                                                           */
   /*   Returns:                                                */
   /*      Object                                               */
   /*-----------------------------------------------------------*/
   public Object getColumnData ( int column ) {
      Object retObj = null ;

      switch ( column ) {
         case 0 :
            if ( read )
               retObj = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.OPEN_MAIL ).getImage() ;
            else
               retObj = ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.MAIL ).getImage() ;
            break ;

         case 1 :
            retObj = subject ;
            break ;

         case 2 :
            retObj = prettyFrom ;
            break ;

         case 3 :
            retObj = openDate ;
            break ;

      }

      if ( retObj == null )
         retObj = new String ( "" ) ;

      return( retObj ) ;
   }



   /*-----------------------------------------------------------*/
   /* getBody
    */
   /**
    *    This method will return the body of a
    *  notification.
    *  
    *             
    *  @param notifyRec The NotificationRec to get the body from
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public String getBody ( NotificationRec notifyRec ) {
      String retStr = null ;

      if ( notifyRec != null ) {
         String newComments = null ;                   


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust nofication Rec Values                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         newComments = Text.replaceAllStrInStr ( notifyRec.comments, "\n", "<br>" ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Replace the field labels                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retStr = Text.replaceInStr ( bodyTemplate, "TO",         Str.getStr ( STR_TO ) ) ;
         retStr = Text.replaceInStr ( retStr,       "FROM",       Str.getStr ( STR_FROM ) ) ;
         retStr = Text.replaceInStr ( retStr,       "DATE",       Str.getStr ( STR_OPEN_DATE ) ) ;
         retStr = Text.replaceInStr ( retStr,       "BRAND",      Str.getStr ( STR_BRAND ) ) ;
         retStr = Text.replaceInStr ( retStr,       "RESOLUTION", Str.getStr ( STR_RESOLUTION ) ) ;
         retStr = Text.replaceInStr ( retStr,       "DOCUMENT",   Str.getStr ( STR_DOCUMENT ) ) ;                 

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, replace the variables                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         retStr = Text.replaceInStr ( retStr, "VAL_TO",    UserSystem.getNameFromUserId ( notifyRec.toUser ) ) ;
         retStr = Text.replaceInStr ( retStr, "VAL_FROM",  notifyRec.prettyFrom ) ;
         retStr = Text.replaceInStr ( retStr, "VAL_DATE",  notifyRec.openDate ) ;
         retStr = Text.replaceInStr ( retStr, "VAL_BRAND", notifyRec.brandRec.toString() ) ;
         retStr = Text.replaceInStr ( retStr, "VAL_RESOL", notifyRec.resolRec.toString() ) ;
         retStr = Text.replaceInStr ( retStr, "VAL_DOC",   "<a href=\"quest://"+notifyRec.docInd+"\">"+notifyRec.docInd+" ("+getDocTitle ( notifyRec.docInd )+") </a>" ) ;
         retStr = Text.replaceInStr ( retStr, "VAL_BODY",  newComments ) ;

      } /* if */

      return( retStr ) ;
   }

   /*------------------------------------------------------------*/
   /* getDocIndStr
    */
   /**
    *    This method will return the string needed
    *  for an update or delete of this item in the
    *  QUEST.NOTIFICATION table.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getDocIndStr () {
      String retStr = "NULL" ;

      if ( docInd != 0 )
         retStr = ""+docInd ;

      return( retStr ) ;
   }


   /*------------------------------------------------------------*/
   /* getCloseDateStr
    */
   /**
    *    This method will return the string needed
    *  for an update or delete of this item in the
    *  QUEST.NOTIFICATION table.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getCloseDateStr () {
      String retStr = "NULL" ;

      if ( ( closeDate != null ) && ( closeDate.length() > 6 ) )
         retStr = "'"+closeDate+"'" ;

      return( retStr ) ;
   }


   /*-----------------------------------------------------------*/
   /* copyToClipboard
    */
   /**
    *    This method will copy this record to the
    *  clip board in a human readable way
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void copyToClipboard ( Clipboard cp ) {
      if ( cp != null ) {
         StringSelection strSel = new StringSelection ( getBody ( this ) ) ;

         cp.setContents ( strSel, strSel ) ;
      } else
         LogSystem.log ( 1, "Clipboard Not Available" ) ;
   }





   /*------------------------------------------------------------*/
   /* cleanDBChars
    */
   /**
    *    This method will clean all strings in a Notification
    *  record so they may be put into the database
    *  
    *
    *  @param sql The Database to clean for
    *  @param notifyRec The NotificationRec to clean
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void cleanDBChars ( NotificationRec notifyRec ) {
      notifyRec.toUser    = Text.cleanDBString ( notifyRec.toUser ) ;
      notifyRec.subject   = Text.cleanDBString ( notifyRec.subject ) ;
      notifyRec.fromUser  = Text.cleanDBString ( notifyRec.fromUser ) ;
      notifyRec.comments  = Text.cleanDBString ( notifyRec.comments ) ;
      notifyRec.openDate  = Text.cleanDBString ( notifyRec.openDate ) ;
      notifyRec.closeDate = Text.cleanDBString ( notifyRec.closeDate ) ;

   } /* cleanDBChars */

   /*------------------------------------------------------------*/
   /* saveToDatabase
    */
   /**
    *    This method will save a notification to the
    *  database
    *  
    *
    *  @param notifyRec The notification record to save
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveToDatabase ( NotificationRec notifyRec ) {
      String   rtnName = "saveToDatabase (Notification)" ;
      SQLMethod sql     = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
      boolean  rc      = false ;
      String   strStmt = null ;

      Qit.setEnabled ( false ) ;
      if ( ( notifyRec != null ) && 
           ( notifyRec.getRecStatus() != REC_NOINSERT ) &&
           ( notifyRec.getRecStatus() != REC_NOCHANGE ) ) {

         try {
            Statement stmt = sql.createStatement () ;
            String    read = notifyRec.read ? "Y" : "N" ;       

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Clean the record                                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            notifyRec.cleanDBChars ( notifyRec ) ;           


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the statement                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            switch ( notifyRec.getRecStatus() ) {
               

               case REC_INSERT:
                  if ( notifyRec.noteInd == 0 )
                     notifyRec.noteInd = Counter.getCounter ( LogSystem.getInstance(), "NOTEIND" ) ;

                  strStmt = "INSERT INTO QUEST.NOTIFICATION "+
                            "( NOTEIND, SUBJECT,  BRANDIND,  RESOLIND, COMMENT, TOUSER, "+
                            "  DOCIND,  OPENDATE, CLOSEDATE, READ,     DBUSER,  CHANGEDTIME )" +
                            "values (  "+notifyRec.noteInd+",                     "+
                            "         '"+notifyRec.subject+"',                    "+
                            "          "+notifyRec.brandRec.getInd()+",           "+
                            "          "+notifyRec.resolRec.getInd()+",           "+
                            "         '"+notifyRec.comments+"',                   "+
                            "         '"+notifyRec.toUser+"',                     "+
                            "          "+notifyRec.getDocIndStr()+",              "+
                            "         '"+notifyRec.openDate+"',                   "+
                            "          "+notifyRec.getCloseDateStr()+",           "+
                            "         '"+read+"', "+
                            "         '"+UserSystem.getUserId()+"',               "+
                            "         CURRENT TIMESTAMP )" ;
                  break ;

               case REC_UPDATE :  
                  /* Do not update dbUser or Changedtime since we use the
                     dbUser field to know who the note came from, and the
                     changed time field to order on when we query */
                  strStmt = "UPDATE QUEST.NOTIFICATION "+
                            "SET   SUBJECT      = '"+notifyRec.subject+"', "+
                            "      BRANDIND     =  "+notifyRec.brandRec.getInd()+", "+
                            "      RESOLIND     =  "+notifyRec.resolRec.getInd()+", "+
                            "      COMMENT      = '"+notifyRec.comments+"', "+
                            "      TOUSER       = '"+notifyRec.toUser+"', "+
                            "      DOCIND       =  "+notifyRec.getDocIndStr()+", "+
                            "      OPENDATE     = '"+notifyRec.openDate+"', "+
                            "      CLOSEDATE    =  "+notifyRec.getCloseDateStr()+", "+
                            "      READ         = '"+read+"'  "+
                            "WHERE NOTEIND      = "+notifyRec.noteInd ;
                  break ;

               case REC_DELETE :
                  boolean deleteRequest = GUISystem.printBox ( STR_QUESTION, STR_DELETE_DOC_REQUEST_WITH_NOTIFY ) ;


                  strStmt = "DELETE FROM QUEST.NOTIFICATION "+
                            "WHERE NOTEIND = "+notifyRec.noteInd ;

                  if ( deleteRequest ) {
                     stmt.executeUpdate ( strStmt ) ;
                     strStmt = "UPDATE QUEST.DOCUMENT "+
                               "SET RECYCLED = 'Y', "+
                               "DBUSER       = '"+UserSystem.getUserId()+"', "+
                               "CHANGEDTIME  = CURRENT TIMESTAMP "+
                               "WHERE DOCIND = "+docInd ;
                  }
                  break ;

            } /* switch */

            stmt.executeUpdate ( strStmt ) ;

            /* If we got here then everything must have went ok! */
            rc = true ;

         } catch ( Exception e ) {
            sql.rollBack() ;
            LogSystem.log (1, e) ;
         } /* catch */

      } /* if */

      sql.close() ;
      Qit.setEnabled ( true ) ;
      return( rc ) ;

   } /* saveToDatabase */





   /*------------------------------------------------------------*/
   /* readListFromDatabase
    */
   /**
    *    This method will read all notifications for
    *  a user from the database. 
    *  
    *
    *  @param userId The User Id to read notifications for
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public static Vector readListFromDatabase ( String userId ) {
      Vector            returnVec  = new Vector ( 1 ) ;
      SQLMethod         sqlMethod  = new SQLMethod ( DatabaseSystem.QUEST, "NotifyReader.read", 3 ) ;

      try {
         Statement       stmt     = sqlMethod.createStatement() ;
         String          sql      = null ;
         ResultSet       rs       = null ;
         NotificationRec readRec  = null ;

         sql = "SELECT NOTEIND, SUBJECT, BRANDIND, RESOLIND,  COMMENT, "+
               "       TOUSER,  DOCIND,  OPENDATE, CLOSEDATE, READ,    "+
               "       DBUSER,  CHANGEDTIME "+
               "FROM QUEST.NOTIFICATION "+
               "WHERE TOUSER = '"+userId+"' "+
               "ORDER BY OPENDATE FOR FETCH ONLY" ;

         rs = stmt.executeQuery ( sql ) ;                       

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through and read all values                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {

            readRec = new NotificationRec () ;
            readRec.noteInd   = rs.getInt(1) ;
            readRec.subject   = rs.getString(2).trim() ;
            readRec.brandInd  = rs.getInt(3) ;
            readRec.resolInd  = rs.getInt(4) ;
            readRec.comments  = rs.getString(5).trim() ;
            readRec.toUser    = rs.getString(6).trim() ;
            readRec.docInd    = rs.getInt(7) ;
            readRec.openDate  = CDate.convertDate ( CDate.DATE_DBDATE, CDate.DATE_DATE, rs.getString(8).trim() ) ;
            readRec.closeDate = rs.getString(9) ;
            readRec.read      = rs.getString(10).equals("Y") ? true : false ;
            readRec.fromUser  = rs.getString(11).trim() ;
            readRec.brandRec   = Brands.getBrand ( readRec.brandInd ) ;
            readRec.resolRec   = (TypeResolutionRec) TypeList.getInstance().objectFromInd ( readRec.resolInd, TypeList.TYPE_RESOLUTION ) ;
            readRec.prettyFrom = UserSystem.getNameFromUserId ( readRec.fromUser ) ;

            if ( readRec.closeDate != null )
               readRec.closeDate = readRec.closeDate.trim() ;

            returnVec.addElement ( readRec ) ;
         } 

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e ) ;
      }

      sqlMethod.close( ) ;

      return( returnVec ) ;
   }





   /*------------------------------------------------------------*/
   /* getDocInd
    */
   /**
    *    This method will return the document indicator
    *  that is represented by this notification
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getDocInd () {
      return( docInd ) ;
   }




   /*------------------------------------------------------------*/
   /* compareTo
    */
   /**
    *    This method will compare this object
    *  to another NOtificationRec. This method is
    *  really only intended to be called when a MultiList reorders
    *
    *  @param MultiListSort The object to compare with
    *  @param column The column to compare
    *  @param None
    *
    *  @return 1 if greater, 0 if equal, -1 if less
    *//*---------------------------------------------------------*/
   public int compareTo ( MultiListSort object, int column ) {
      int rc = 0 ;

      switch ( column ) {
         
         case 1 :
            rc = subject.compareTo ( ((NotificationRec)object).subject ) ;
            break ;

         case 2 :
            rc = prettyFrom.compareTo ( ((NotificationRec)object).prettyFrom ) ;
            break ;

         case 3 :
            rc = CDate.compareDate ( openDate, ((NotificationRec)object).openDate, CDate.DATE_DATE ) ;
            break ;

      }

      return( rc ) ;

   }





   /*------------------------------------------------------------*/
   /* getDocTitle
    */
   /**
    *    This method will return a document title
    *  from a Doc Ind
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private String getDocTitle ( int docInd ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getDocTitle", 5 ) ;
      String    title     = "" ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT TITLE FROM QUEST.DOCUMENTS WHERE DOCIND = "+docInd ) ;

         if ( rs.next() )
            title = rs.getString(1).trim() ;

         rs.close() ;

      } catch ( Exception e ) {
         sqlMethod.rollBack() ;
         LogSystem.log ( 1, e, false ) ;
      }

      sqlMethod.close() ;

      return( title ) ;
   }
} /* NotificationRec */
