package com.ibm.nzna.projects.qit.notify ;   
/*-----------------------------------------------------------*/
/*   NotifyPanel                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NotifyPanel                         */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import java.io.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.docedit.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.qit.storedProc.sqlRunner.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This panel holds the user's list of Notifications. It allows the user to delete, reply to or forward them 
 *
 *
 * @version 1.0, 3/31/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NotifyPanel                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class NotifyPanel extends JPanel implements PropertyListener, ActionListener, ListSelectionListener, BrowserListener, QuestPanel, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Controls                                                */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTitle          st_SUBJECT                 = null ;
   private MultiList       cnr_NOTES                  = null ;
   private Browser         browser                    = null ;
   private ActionButton    pb_CLOSE                   = null ;
   private ActionButton    pb_FORWARD                 = null ;
   private ActionButton    pb_REPLY                   = null ;
   private ActionButton    pb_DELETE                  = null ;
   private JSplitPane      splitPane                  = null ;
   private AppDefaultWin   parentWin                  = null ; 




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if initialized, FALSE if not
    *//*---------------------------------------------------------*/
   public boolean initialize () {
      JPanel previewPanel = new JPanel() ;
      String[] title = { "", Str.getStr ( STR_SUBJECT ), Str.getStr ( STR_FROM ), Str.getStr ( STR_DATE )} ;                       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, create our GUI components                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_NOTES      = new MultiList  ( GUISystem.getFontUtil () ) ;
      st_SUBJECT     = new JTitle     ( "Subject" ) ;
      browser        = GUISystem.createBrowser() ;
      splitPane      = new JSplitPane ( JSplitPane.VERTICAL_SPLIT ) ;        
      pb_CLOSE       = new ActionButton ( Str.getStr ( STR_CLOSE ),   ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),          Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_FORWARD     = new ActionButton ( Str.getStr ( STR_FORWARD ), ImageSystem.getImageIcon ( this, ImageSystem.FORWARD_NOTIFY ), Str.getStr ( STR_FORWARD_NOTIFY ) ) ;
      pb_REPLY       = new ActionButton ( Str.getStr ( STR_REPLY ),   ImageSystem.getImageIcon ( this, ImageSystem.REPLY_NOTIFY ),   Str.getStr ( STR_REPLY_NOTIFY ) ) ;
      pb_DELETE      = new ActionButton ( Str.getStr ( STR_DELETE ),  ImageSystem.getImageIcon ( this, ImageSystem.DELETE ),         Str.getStr ( STR_DELETE_NOTIFY ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      cnr_NOTES.setColumnHeadings ( title ) ;
      cnr_NOTES.setColumnWidth ( 0, 23 ) ;
      cnr_NOTES.setColumnWidth ( 1, 250 ) ;     


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      browser.addBrowserListener ( this ) ;
      cnr_NOTES.addListSelectionListener ( this ) ;
      pb_CLOSE.addActionListener   ( this ) ;
      pb_REPLY.addActionListener   ( this ) ;
      pb_FORWARD.addActionListener ( this ) ;
      pb_DELETE.addActionListener  ( this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout              ( new BorderLayout () ) ;
      previewPanel.setLayout ( new BorderLayout () ) ;

      previewPanel.add ( st_SUBJECT, BorderLayout.NORTH ) ;
      previewPanel.add ( browser.getComponent(), BorderLayout.CENTER ) ;

      add ( splitPane, BorderLayout.CENTER ) ;

      splitPane.add ( previewPanel, JSplitPane.BOTTOM ) ;
      splitPane.add ( cnr_NOTES,    JSplitPane.TOP ) ;


      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the name of this panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_IN_BOX ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return the icon that this
    *  panel has
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon ( this, ImageSystem.NOTIFY_SMALL ) ) ;
   }




   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called once this panel
    *  has been selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }


   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called once this panel
    *  has been selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called when we need
    *  to refresh our view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;
      Vector        notes     = null ;

      parentWin.setStatus ( Str.getStr ( STR_READING_NOTIFICATIONS ) ) ;
      Qit.setEnabled ( false ) ;
      try {
         notes = NotificationRec.readListFromDatabase ( UserSystem.getUserId() ) ;
         cnr_NOTES.removeAll() ;
         cnr_NOTES.add ( notes ) ;
         browser.setPageContent ( "<html><head><title></title></head><body bgcolor=\"#FFFFFF\"></body></html>" ) ;
         st_SUBJECT.setText("" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
      Qit.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;
   }




   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }



   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will do nothing
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }






   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will always return TRUE
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when we have
    *  been closed and we need to free resources
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {
      removeAll() ;
      st_SUBJECT                 = null ;
      cnr_NOTES                  = null ;
      browser                    = null ;
      pb_CLOSE                   = null ;
      pb_FORWARD                 = null ;
      pb_REPLY                   = null ;
      pb_DELETE                  = null ;
      splitPane                  = null ;
      PropertySystem.removePropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return this component
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return the default navigation
    *  panel for this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel ( ) {
      return( null ) ;
   }



   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are added
    *  to a window and are now showing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      parentWin   = GUISystem.getParentDefWin ( this ) ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_REPLY ) ;
      parentWin.addActionComponent ( this, pb_FORWARD ) ;
      parentWin.addActionComponent ( this, pb_DELETE ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our split pane size                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      splitPane.setDividerLocation ( .55 ) ;                        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in our notifications to show                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refresh() ;
   }




   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when the user
    *  has selected an item from the list
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {

      if ( !e.getValueIsAdjusting() ) {
         try {
            NotificationRec noteRec = (NotificationRec) cnr_NOTES.getSelectedItem() ;

            if ( noteRec != null ) {
               String body     = noteRec.getBody ( noteRec ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set the data in the browser and title                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               st_SUBJECT.setText ( noteRec.subject ) ;
               browser.setPageContent ( body ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Set this mail to 'read' and update the db               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               noteRec.read = true ;
               cnr_NOTES.invalidate() ;
               cnr_NOTES.revalidate() ;
               noteRec.updateRecStatus ( noteRec.REC_NOCHANGE ) ;
               noteRec.updateRecStatus ( noteRec.REC_UPDATE ) ;
               noteRec.saveToDatabase ( noteRec ) ;

            }
         } catch ( Exception ex1 ) {
            LogSystem.log ( 1, ex1 ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* deleteNotify
    */
   /**
    *    This method will delete the selected
    *  notification records
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void deleteNotify () {
      Vector selItems = cnr_NOTES.getSelection() ;                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If they haven't selected anything , tell them           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( selItems == null ) || ( selItems.size() == 0 ) )
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Otherwise, walk the vector and remove the items         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_DELETE_NOTIFY ) ) {
            Vector          data          = cnr_NOTES.getData() ;
            int             selIndex      = 0 ;
            int             selSize       = selItems.size() ;
            NotificationRec noteRec       = null ;
            AppDefaultWin   win           = GUISystem.getParentDefWin ( this ) ;
            String          sqlStmt       = "DELETE FROM QUEST.NOTIFICATION WHERE NOTEIND IN (" ;
            String          draftSqlStmt  = "UPDATE QUEST.DOCUMENTS SET RECYCLED = 'Y', DBUSER = '"+UserSystem.getUserId()+"', CHANGEDTIME = CURRENT TIMESTAMP WHERE DOCIND IN ( ";
            boolean         deleteRequest = GUISystem.printBox ( STR_QUESTION, STR_DELETE_REQUEST_WITH_NOTIFY ) ;

            win.setStatus ( STR_DELETING ) ;                                               

            /*-------------------------------------------------------------*/
            /*                                                             */
            /*   Walk the items while creating the list of items to delete */
            /*                                                             */
            /*-------------------------------------------------------------*/
            while ( selIndex < selSize ) {
               noteRec = (NotificationRec) selItems.elementAt ( selIndex) ;
               data.removeElement ( noteRec ) ;
               sqlStmt+=""+noteRec.noteInd ;
               draftSqlStmt+=""+noteRec.getDocInd() ;

               if ( selIndex < (selSize-1) ) {
                  sqlStmt+=", " ;
                  draftSqlStmt+=", " ;
               }

               selIndex++ ;   
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Put the ending parens in                                */              
            /*                                                           */
            /*-----------------------------------------------------------*/
            sqlStmt+=")"+SqlRunner.END_DELIM ;                          
            draftSqlStmt+=")"+SqlRunner.END_DELIM ;                          


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Send the statements to the database                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            QITSqlRunner.sqlRunner ( sqlStmt ) ;

            if ( deleteRequest )
               QITSqlRunner.sqlRunner ( draftSqlStmt ) ;




            win.setStatus ( null ) ;                                 

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Now, remove all of the items from our list              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            cnr_NOTES.removeAll() ;
            cnr_NOTES.add ( data ) ;

         } /* if they are sure  */
      } /* else  */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Finally, take out the preview information               */                                
      /*                                                           */
      /*-----------------------------------------------------------*/
      browser.setPageContent ( "<html><head><title></title></head><body bgcolor=\"#FFFFFF\"></body></html>" ) ;
      st_SUBJECT.setText("") ;


   } /* deleteNotify  */

   /*-----------------------------------------------------------*/
   /* replyForward
    */
   /**
    *    This method will reply to a selected message or
    *  forward a message
    *  
    *  
    *             
    *  @param forward TRUE = Forward the item, false = REPLY
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void replyForward ( boolean forward ) {
      NotificationRec selRec = (NotificationRec) cnr_NOTES.getSelectedItem() ;

      if ( selRec != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Make a copy of the original                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         selRec = new NotificationRec ( selRec ) ;


         if ( !forward ) {
            selRec.sendType = NotificationRec.SEND_REPLY ;
            selRec.comments = Text.replaceAllStrInStr ( selRec.comments, "\n", "<<" ) ;
            selRec.comments = Text.replaceAllStrInStr ( selRec.comments, "<<", "\n>" ) ;
            selRec.comments = "\n\n"+selRec.prettyFrom+" "+Str.getStr ( STR_WROTE )+":\n"+selRec.comments ;
            selRec.toUser   = selRec.fromUser ;
         } else {
            selRec.toUser     = "" ;
            selRec.sendType = NotificationRec.SEND_FORWARD ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get this record ready for insert                        */                       
         /*                                                           */
         /*-----------------------------------------------------------*/
         selRec.updateRecStatus ( selRec.REC_NOCHANGE ) ;
         selRec.updateRecStatus ( selRec.REC_INSERT ) ;
         selRec.fromUser   = UserSystem.getUserId() ;
         selRec.prettyFrom = UserSystem.getUserName() ;
         selRec.closeDate  = null ;
         selRec.openDate   = DateSystem.getDate ( CDate.DATE_DATE ) ;
         selRec.noteInd    = 0 ;


         new SendNotifyDlg ( GUISystem.getParentDefWin ( this ), selRec ).getResult() ;

      } else
         GUISystem.printBox ( STR_ERROR, STR_MUST_SELECT_ITEM ) ;
   } /* replyForward  */


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  we are waiting for has been fired
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_DELETE                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_DELETE )
         deleteNotify() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REPLY / pb_FORWARD                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_REPLY ) ||
                ( e.getSource() == pb_FORWARD ) )
         replyForward ( e.getSource() == pb_FORWARD ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CLOSE ) {
         GUISystem.getParentDefWin(this).closePanel ( this, null ) ;
      }
   } /* actionPerformed  */





   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    This method will be called when properties
    *  we listen to are changed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE )
         setText() ;
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on all of 
    *  our GUI Components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      String[] title = { "", Str.getStr ( STR_SUBJECT ), Str.getStr ( STR_FROM ), Str.getStr ( STR_DATE )} ;                       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, create our GUI components                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_SUBJECT.setText ( "" ) ;
      pb_CLOSE.setText          ( Str.getStr ( STR_CLOSE ) ) ;
      pb_CLOSE.setToolTipText   ( Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      pb_FORWARD.setText        ( Str.getStr ( STR_FORWARD ) ) ;
      pb_FORWARD.setToolTipText ( Str.getStr ( STR_FORWARD_NOTIFY ) ) ;
      pb_REPLY.setText          ( Str.getStr ( STR_REPLY ) ) ;
      pb_REPLY.setToolTipText   ( Str.getStr ( STR_REPLY_NOTIFY ) ) ;
      pb_DELETE.setText         ( Str.getStr ( STR_DELETE ) ) ;
      pb_DELETE.setToolTipText  ( Str.getStr ( STR_DELETE_NOTIFY ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cnr_NOTES.setColumnHeadings ( title ) ;
      cnr_NOTES.setColumnWidth ( 0, 23 ) ;
      cnr_NOTES.setColumnWidth ( 1, 250 ) ;     
   }



   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }




   /*------------------------------------------------------------*/
   /* linkClicked
    */
   /**
    *    This method will be called when the user
    *  clicks on a link
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean linkClicked ( String url ) {
      DocumentDraft draft = new DocumentDraft ( ((NotificationRec)cnr_NOTES.getSelectedItem()).getDocInd() ) ;

      EditDocument.editDocument ( draft ) ;

      return( true ) ;
   }

   public void mouseLinkExit () {
   }
   public void mouseLinkEnter( ) {
   }
   public void pageStarted ( String str ) {
   }
   public void pageStopped () {
   }
   public void mouseOverLink ( String str ) {
   }

   public void statusChange ( String status, int value, int max ) {
      parentWin.setStatus ( status ) ;
      parentWin.setValue ( value ) ;
      parentWin.setValueMax ( max ) ;
   }

} /* NotifyPanel */
