package com.ibm.nzna.projects.qit.notify ;
/*-----------------------------------------------------------*/
/*   SendNotifyDlg                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SendNotifyDlg                       */
/*                                                           */
/*                                                           */
/*         @Copyright  1997 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.*;
import java.awt.datatransfer.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
/**
 * This dialog holds the fields needed to send a notification 
 *
 *
 * @version 1.0, 9/21/1998 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SendNotifyDlg                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class SendNotifyDlg extends JDialog implements DataLengths, TypeListListener, SelectDocListener, ActionListener, Runnable, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Vars                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_SAVEANDEXIT = "SaveExit" ;  

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI Members                                             */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private DButton         pb_FIND             = null ;
   private DButton         pb_PASTE            = null ;
   private DButton         pb_CANCEL           = null ;
   private DButton         pb_SEND             = null ;
   private JComboBox       cb_BRAND            = null ;
   private JComboBox       cb_RESOLUTION       = null ;
   private UserEntryPanel  ef_USERID           = null ;
   private JTextField      ef_SUBJECT          = null ;
   private JTextArea       mle_COMMENT         = null ;
   private JTextField      ef_DOCIND           = null ;
   private JScrollPane     scr_COMMENT         = null ;
   private JLabel          st_BRAND            = null ;
   private JLabel          st_SUBJECT          = null ;
   private JLabel          st_BRAND_PM         = null ;
   private JLabel          st_RESOLUTION       = null ;
   private JLabel          st_DOCUMENTID       = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JPanel            pnl_HOLDER = null ;
   private NotificationRec   notifyRec  = null ;
   private boolean           allowClose = false ;
   private boolean           allowInput = true ;
   private int               id         = 0 ;


   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a blank send dialog. 
    * This method will not show the window, you must call 'getResult' to show it.
    *  
    *  
    *
    *  @param listener The listener to notify when we close
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SendNotifyDlg ( Frame parent ) {
      super ( parent, Str.getStr ( STR_SEND_NOTIFICATION ), true ) ;

      initialize ( parent ) ;

   } /* SendNotifyDlg */

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This will create a populated send dialog
    * This method will not show the window, you must call 'getResult' to show it.
    *  
    *  
    *
    *  @param listener The listener to notify when we close
    *  @param notifyRec The notification record to populate with
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SendNotifyDlg ( Frame parent, NotificationRec notifyRec ) {
      super ( parent, Str.getStr ( STR_SEND_NOTIFICATION ), true ) ;

      this.notifyRec = notifyRec ;
      initialize ( parent ) ;

   } /* SendNotifyDlg */





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our window
    * This method will not show the window, you must call 'getResult' to show it.
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize ( Frame parent ) {

      createControls() ;

      setSize ( 450, 350 ) ;
      WinUtil.centerChildInParent ( this, parent ) ;

      if ( notifyRec != null )
         refreshData() ;

      setDefaultCloseOperation ( JDialog.DO_NOTHING_ON_CLOSE ) ;

   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will show the window and
    *  waits for the user to act on the window. When
    *  the user closes the window, a notification rec will be returned
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NotificationRec
    *//*---------------------------------------------------------*/
   public NotificationRec getResult () {
      setVisible ( true ) ;
      return( notifyRec ) ;
   }

   /*------------------------------------------------------------*/
   /* createControls
    */
   /**
    *    This method will be called to create and
    *  add all of our controls.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void createControls () {     
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SEND       = new DButton ( Str.getStr ( STR_SEND ) ) ;
      pb_PASTE      = new DButton ( Str.getStr ( STR_PASTE ) ) ;
      pb_CANCEL     = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      pb_FIND       = new DButton ( Str.getStr ( STR_FIND ) ) ;
      cb_BRAND      = new JComboBox ( Brands.getBrands() ) ; 
      cb_RESOLUTION = new JComboBox ( ) ;
      ef_USERID     = new UserEntryPanel () ;
      ef_SUBJECT    = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_SUBJECT ), "", 0 ) ;
      ef_DOCIND     = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC, 10 ), "", 0 ) ;
      mle_COMMENT  = new JTextArea  ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_COMMENT ) ) ;
      scr_COMMENT  = new JScrollPane ( mle_COMMENT ) ;
      st_BRAND      = new JLabel ( Str.getStr ( STR_BRAND ) ) ;
      st_SUBJECT    = new JLabel ( Str.getStr ( STR_SUBJECT ) ) ;
      st_BRAND_PM   = new JLabel ( Str.getStr ( STR_BRAND_PROGRAM_MANAGER ) ) ;
      st_RESOLUTION = new JLabel ( Str.getStr ( STR_RESOLUTION ) ) ;
      st_DOCUMENTID = new JLabel ( Str.getStr ( STR_INTERNAL_NUM ) ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_FIND.addActionListener   ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_PASTE.addActionListener  ( this ) ;
      pb_SEND.addActionListener   ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls                                        */       
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_BRAND ) ;
      contentPane.add ( st_SUBJECT ) ;
      contentPane.add ( st_BRAND_PM ) ;
      contentPane.add ( st_RESOLUTION ) ;
      contentPane.add ( st_DOCUMENTID ) ;
      contentPane.add ( ef_USERID ) ;
      contentPane.add ( cb_BRAND ) ;
      contentPane.add ( cb_RESOLUTION ) ;
      contentPane.add ( ef_SUBJECT ) ;
      contentPane.add ( ef_DOCIND ) ;
      contentPane.add ( pb_FIND ) ;
      contentPane.add ( scr_COMMENT ) ;
      contentPane.add ( pb_SEND ) ;
      contentPane.add ( pb_PASTE ) ;
      contentPane.add ( pb_CANCEL ) ;
      
   } /* createControls */




   /*-----------------------------------------------------------*/
   /* refreshData
    */
   /**
    *    This method will refresh our data
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void refreshData () {
      if ( notifyRec != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we don't have to send a note, enable cancel          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_CANCEL.setVisible ( !notifyRec.mustSend ) ;                                         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we are supposed to get the user from the brand,      */
         /*   do so now.                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( notifyRec.toUser != null ) {
            if ( notifyRec.toUser.equals ( "GET_USER_FROM_BRAND" ) ) {
               ef_USERID.setEnabled ( false ) ;
            } else if ( notifyRec.toUser != null ) {
               ef_USERID.setText ( notifyRec.toUser ) ;
            }
         } else
            ef_USERID.setText ( "" ) ;   


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set the text                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         ef_SUBJECT.setText ( notifyRec.subject ) ;                                   
         cb_BRAND.setSelectedItem ( (Object) notifyRec.brandRec ) ;

         if ( notifyRec.docInd > 0 ) {
            ef_DOCIND.setText ( ""+notifyRec.docInd ) ;
            ef_DOCIND.setEnabled ( false ) ;
            pb_FIND.setEnabled ( false ) ;
         }

         mle_COMMENT.setText ( notifyRec.comments ) ;

         if ( notifyRec.sendType == NotificationRec.SEND_REPLY )
            setTitle ( Str.getStr ( STR_REPLY ) ) ;
         else if ( notifyRec.sendType == NotificationRec.SEND_FORWARD )
            setTitle ( Str.getStr ( STR_FORWARD ) ) ;

         ef_SUBJECT.requestFocus() ;

         TypeList.refreshList ( cb_RESOLUTION, TypeList.TYPE_RESOLUTION, this ) ;
      }
   }





   /*------------------------------------------------------------*/
   /* layoutControls
    */
   /**
    *    This method will be called to lay out and
    *  position all of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout() {
      Dimension size = getSize() ;
      int rowHeight = GUISystem.getRowHeight() ;
      int y         = 5 ;

      super.doLayout() ;

      st_BRAND_PM.setBounds ( 12, y, 285, rowHeight ) ;
      ef_USERID.setBounds ( 112, y, 185, rowHeight+2 ) ;
      y+=rowHeight+4 ;

      ef_SUBJECT.setBounds (  112, y , 185, rowHeight) ;
      st_SUBJECT.setBounds (  12, y , 47, rowHeight) ;
      y+=rowHeight+2 ;

      cb_BRAND.setBounds (  112, y, 185, rowHeight ) ;
      st_BRAND.setBounds (  12, y, 67, rowHeight) ;
      y+=rowHeight+8 ;

      cb_RESOLUTION.setBounds (  112, y, 185, rowHeight) ;
      st_RESOLUTION.setBounds (  12, y, 100, rowHeight) ;
      y+=rowHeight+8 ;

      ef_DOCIND.setBounds (  112, y, 91, rowHeight) ;
      st_DOCUMENTID.setBounds (  12, y, 93, rowHeight) ;
      pb_FIND.setBounds ( 205, y, 50, rowHeight ) ;
      y+=rowHeight+2 ;

      scr_COMMENT.setBounds (  12, y, 400, size.height-70-y ) ;

      pb_SEND.setBounds (  12, size.height-60, 70, 25 ) ;
      pb_PASTE.setBounds ( 87, size.height-60, 90, 25 ) ;
      pb_CANCEL.setBounds (  87+95, size.height-60, 90, 25 ) ;

   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will respond to buttons being
    *  pressed
    *  
    *
    *  @param id The ID of the button
    *  @param button The instance of the button
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      if ( ( allowInput ) && ( e.getSource() instanceof DButton ) ) {
         DButton button = (DButton) e.getSource() ;                           

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_FIND - Brings up the SelectDocDlg                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( pb_FIND == button )
            new SelectDocDlg ( SelectDocDlg.SELECTTYPE_SINGLE, true).addSelectDocListener ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_SEND - Sends this notification                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_SEND == button ) {
            String user = ef_USERID.getUserId() ;

            if ( user != null ) {
               if ( saveData() ) {
                  enableComponents ( false ) ;
                  new Thread ( this, THREAD_SAVEANDEXIT ).start() ;
               } else
                  enableComponents ( true ) ;

            } else {
               setTitle ( Str.getStr ( STR_MUST_ENTER_TO_USER ) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_CANCEL - Will cancel this notification               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_CANCEL == button ) {
            if ( !notifyRec.mustSend ) {
               notifyRec = null ;
               allowClose = true ;
               closeWindow() ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_PASTE - Pastes the clipboard into the text           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( pb_PASTE == button )
            paste() ;

      } /* if it is a DButton  */
   } /* buttonHit */




   /*------------------------------------------------------------*/
   /* saveData
    */
   /**
    *    This method will save the data on our screen
    *  and return TRUE if the data is valid to go to
    *  the database
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Valid, FALSE = Not Valid
    *//*---------------------------------------------------------*/
   private boolean saveData () {
      TypeCategoryRec   brandRec = (TypeCategoryRec) cb_BRAND.getSelectedItem();
      TypeResolutionRec resolRec = (TypeResolutionRec) cb_RESOLUTION.getSelectedItem() ;
      String            subject  = ef_SUBJECT.getText() ;
      String            comments = mle_COMMENT.getText() ;
      int               docInd   = 0 ;
      String            toUser   = ef_USERID.getUserId() ;
      boolean           rc       = false ;

      try {
         docInd = new Integer ( ef_DOCIND.getText() ).intValue() ;
      } catch ( NumberFormatException e ) {
      }


      if ( brandRec == null ) {
         GUISystem.printBox ( STR_ERROR, STR_MUST_SELECT_BRAND ) ;
      } else if ( resolRec == null )
         setTitle ( Str.getStr ( STR_MUST_ENTER_RESOLUTION ) ) ;
      else if ( ( subject == null ) || ( subject.length() == 0 ) )
         setTitle ( Str.getStr ( STR_MUST_ENTER_SUBJECT ) ) ;
      else if ( ( comments == null ) || ( comments.length() == 0 ) )
         setTitle ( Str.getStr ( STR_MUST_ENTER_COMMENT ) ) ;
      else if ( ( toUser == null ) || ( toUser.length() == 0 ) )
         setTitle ( Str.getStr ( STR_MUST_ENTER_TO_USER ) ) ;
      else {

         /* Save the data */
         if ( notifyRec == null )
            notifyRec = new NotificationRec() ;
         notifyRec.openDate  = DateSystem.getDate ( CDate.DATE_DATE ) ;
         notifyRec.comments  = comments ;
         notifyRec.toUser    = toUser ;
         notifyRec.fromUser  = UserSystem.getUserId() ;
         notifyRec.brandRec  = brandRec ;
         notifyRec.resolRec  = resolRec ;
         notifyRec.docInd    = docInd ;
         notifyRec.subject   = subject ;
         notifyRec.updateRecStatus ( notifyRec.REC_NOCHANGE ) ;
         notifyRec.updateRecStatus ( notifyRec.REC_INSERT ) ;

         rc = true ;

      }



      return( rc ) ;
   } /* saveData */





   /*------------------------------------------------------------*/
   /* notifyClose
    */
   /**
    *    This method will notify our listener that we
    *  are closed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void notifyClose() {
      //listener.stdDialogClosed ( id, notifyRec ) ;
   }





   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread will perform a variety of operations.
    *  The operation that is performed depends on
    *  what you name the thread ( THREAD_SAVEANDEXIT etc.. )
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String name = Thread.currentThread().getName() ;       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_SAVEANDEXIT                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( name.equals ( THREAD_SAVEANDEXIT ) ) {
         enableComponents ( false ) ;
         if ( notifyRec.saveToDatabase ( notifyRec ) ) {
            enableComponents ( true ) ; // Must reenable the components or we won't close!
            notifyRec.notifySent = true ;
            allowClose = true ;
            closeWindow() ;
         }
         enableComponents  ( true ) ;
      } /* if */
   }



   /*-----------------------------------------------------------*/
   /* paste
    */
   /**
    *    This method will paste what is in the clip board
    *  to the comments field
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public void paste() {
      Clipboard cp = Toolkit.getDefaultToolkit().getSystemClipboard() ;

      if ( cp != null ) {
         try {
            Transferable cpData = cp.getContents ( (Object)this ) ;

            if ( cpData != null ) {
               String       data   = (String) (cpData.getTransferData ( DataFlavor.stringFlavor ) ) ;

               if ( data != null )
                  mle_COMMENT.setText ( data ) ;
            } /* if */


         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* destroyWin
    */
   /**
    *    This method will return TRUE so our window
    *  closes, but we need to call the STOP method
    *  in the UserPanel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean destroyWin () {
      ef_USERID.stop() ;
      return( true ) ;
   }





   /*------------------------------------------------------------*/
   /* closeWindow
    */
   /**
    *    This method will be called when we need
    *  to close our window. If it is allowed to close,
    *  we will close it
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void closeWindow () {
      if ( allowClose )
         dispose() ;
   }





   /*------------------------------------------------------------*/
   /* enableComponents
    */
   /**
    *    This method will either enable or disable
    *  all of our components
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void enableComponents ( boolean enable ) {
      this.allowInput = enable ;
   }





   /*------------------------------------------------------------*/
   /* typeListRefreshed
    */
   /**
    *    This method will be called when a type list
    *  we have asked for has been refreshed
    *  
    *
    *  @param typeList The type list
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void listRefreshed ( int typeList ) {
      if ( notifyRec != null ) {

         if ( ( typeList == TypeList.TYPE_RESOLUTION ) && 
              ( notifyRec.resolRec != null ) ) {
            cb_RESOLUTION.setSelectedItem ( notifyRec.resolRec ) ;
         }
      }
   }





   /*------------------------------------------------------------*/
   /* selectDocComplete
    */
   /**
    *    This method will be called if the user
    *  hits the 'Find' button, and finds a document
    *  
    *
    *  @param docVec all of the selected documents that were found
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void selectDocComplete ( Vector docVec ) {
      if ( docVec != null ) {
         DocumentRow document = (DocumentRow) docVec.elementAt(0) ;

         if ( document.isDraft() ) 
            ef_DOCIND.setText ( ""+document.getDocumentInd() ) ;
      }
   }
} /* SendNotifyDlg */
