package com.ibm.nzna.projects.qit.notify.storedProc.notifyReader ;   
/*-----------------------------------------------------------*/
/*   NotifyReader                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NotifyReader                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.projects.qit.notify.* ;
import com.ibm.nzna.shared.util.CDate ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.Blob ;
import COM.ibm.db2.app.* ;
import java.sql.ResultSet ;
import java.sql.Connection ;
import java.sql.Statement ;
import java.io.* ;
import java.util.* ;
/**
 * This stored procedure will read all notifications for a specific user. The
 * notifications are returned in a Vector of NotificationRec. 
 * 
 * Main Execution Point: read()  
 *
 * Param Number   Descript
 * -------------- -------------------------------------------
 * 1              User Id to read notifications for
 * 2              Vector containing Notification Recs
 * 3              StoredProcRec ( Allocated and returned by NotifyReader )
 *
 * 
 *
 * @version 1.0, 2/23/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NotifyReader                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class NotifyReader extends StoredProc {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private StoredProcRec returnRec         = new StoredProcRec () ;
   private Vector        returnVec         = null ;
   private Connection    con               = null ;





   /*------------------------------------------------------------*/
   /* logonUser
    */
   /**
    *    This method read all notifications
    *
    *  @param userId The user id to read notifications for
    *  @param retNotifyReaderRec The BLOB used to return the Vector
    *  @param storedProcRetRec The BLOB used to return the StoredProcRetRec
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void read ( String userId, Blob retNotifyReaderRec, Blob storedProcRetRec ) throws Exception  {                            

      try {
         try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Get our connection to DB/2                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            con = getConnection() ;         

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Read everything in                                      */
            /*                                                           */
            /*-----------------------------------------------------------*/
            readNotifications ( userId ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Write out our NotifyReaderRec                           */
            /*                                                           */
            /*-----------------------------------------------------------*/
            retNotifyReaderRec = Lob.newBlob() ;

            StoredProcUtil.setObjectInBlob ( retNotifyReaderRec, returnVec );
            set ( 2, retNotifyReaderRec ) ;


         } catch ( Exception e ) {
            com.ibm.nzna.shared.util.FileUtil.writeException ( "c:\\notifyReader.out", e ) ;
            StoredProcUtil.writeException ( e, returnRec ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Commit                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con.commit() ;
         con.close() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Write out our return information                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         storedProcRetRec = Lob.newBlob() ;
         StoredProcUtil.setObjectInBlob ( storedProcRetRec, returnRec ) ;
         set ( 3, storedProcRetRec ) ;
      } catch ( Exception e1 ) {
         com.ibm.nzna.shared.util.FileUtil.writeException ( "c:\\notifyReader1.out", e1 ) ;

      }

   }






   /*------------------------------------------------------------*/
   /* readUsers
    */
   /**
    *    This method will read in all notifications for a user
    *
    *  @param userId The user id to read notifications for
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readNotifications ( String userId ) throws Exception {
      Statement       stmt     = con.createStatement() ;
      String          sql      = null ;
      ResultSet       rs       = null ;
      NotificationRec readRec  = null ;

      returnVec = new Vector ( 1, 10 ) ;

      sql = "SELECT NOTEIND, SUBJECT, BRANDIND, RESOLIND,  COMMENT, "+
            "       TOUSER,  DOCIND,  OPENDATE, CLOSEDATE, READ,    "+
            "       DBUSER,  CHANGEDTIME "+
            "FROM QUEST.NOTIFICATION "+
            "WHERE TOUSER = '"+userId+"' "+
            "ORDER BY OPENDATE FOR FETCH ONLY" ;

      rs = stmt.executeQuery ( sql ) ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Loop through and read all values                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rs.next() ) {

         readRec = new NotificationRec () ;
         readRec.noteInd   = rs.getInt(1) ;
         readRec.subject   = rs.getString(2).trim() ;
         readRec.brandInd  = rs.getInt(3) ;
         readRec.resolInd  = rs.getInt(4) ;
         readRec.comments  = rs.getString(5).trim() ;
         readRec.toUser    = rs.getString(6).trim() ;
         readRec.docInd    = rs.getInt(7) ;
         readRec.openDate  = CDate.convertDate ( CDate.DATE_DBDATE, CDate.DATE_DATE, rs.getString(8).trim() ) ;
         readRec.closeDate = rs.getString(9) ;
         readRec.read      = rs.getString(10).equals("Y") ? true : false ;
         readRec.fromUser  = rs.getString(11).trim() ;

         if ( readRec.closeDate != null )
            readRec.closeDate = readRec.closeDate.trim() ;

         returnVec.addElement ( readRec ) ;
      } 

      rs.close() ;
      stmt.close() ;
   }



} /* NotifyReader */
