package com.ibm.nzna.projects.qit.product ;              
/*-----------------------------------------------------------*/
/*   ProductTemplateSystem                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductTemplateSystem               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
/**
 * This system maintains all of the product templates for a user 
 *
 *
 * @version 1.0, 3/28/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductTemplateSystem                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductTemplateSystem extends Object implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   private members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static Vector                templateVec    = null ;
   private static boolean               templatesRead  = false ;
   private static ProductTemplateSystem instance       = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will simply create this object. Templates
    *  will not be read until they are requested for
    *  the first time
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductTemplateSystem () {
      templateVec = new Vector ( 1, 1 ) ;
      instance = this ;
   }




   /*------------------------------------------------------------*/
   /* getTemplateVec
    */
   /**
    *    This method will return the vector of templates.
    *  If the templates have not been read in yet, they
    *  will be read in now.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Vector getTemplateVec ( ) {
      System.out.println ( "Get TEmplate Vec!" ) ;
      if ( !templatesRead )
         readAllTemplates ( ) ;

      return( templateVec ) ;
   }




   /*------------------------------------------------------------*/
   /* readAllTemplates
    */
   /**
    *    This method will read in all of our templates
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector readAllTemplates ( ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readAllTemplates", 5 ) ;

      System.out.println ( "Reading All Templates!" ) ;
      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT PRODUCTIND, PRODNUM, DESCRIPTION, CREATEDON, CREATEDBY, LASTTOUCHEDBY, LASTTOUCHEDON   "+
                                              "FROM PRODRAFT.PRODUCT "+
                                              "WHERE LASTTOUCHEDBY = '"+UserSystem.getUserId()+"' AND "+
                                              "      TEMPLATE = 'Y' " ) ;

         templateVec = new Vector ( 1 ) ;
         while ( rs.next() ) {
            templateVec.addElement ( new ProductRow ( rs.getInt ( 1 ),
                                                      0,
                                                      rs.getString ( 2 ),
                                                      rs.getString ( 3 ).trim(),    // Description
                                                      DateSystem.prettyDateFromStamp ( rs.getString ( 4 ).trim() ),  // Created On
                                                      UserSystem.getNameFromUserId ( rs.getString ( 5 ).trim() ),    // Created By
                                                      "",
                                                      "",
                                                      UserSystem.getNameFromUserId ( rs.getString ( 6 ).trim() ),
                                                      DateSystem.prettyDateFromStamp ( rs.getString ( 7 ) ),
                                                      true ) ) ;
         }

         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }


      sqlMethod.close() ;
      return( templateVec ) ;
   }




   /*------------------------------------------------------------*/
   /* getInstance
    */
   /**
    *    This method will return the one instance
    *  of the ProductTemplateSystem
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ProductTemplateSystem
    *//*---------------------------------------------------------*/
   public static ProductTemplateSystem getInstance () {
      return( instance ) ;
   }


   /*------------------------------------------------------------*/
   /* addTemplate
    */
   /**
    *    This method will add a template to our list
    *  
    *  
    *
    *  @param docRec The template to add
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void addTemplate ( ProductDraft productRec ) {
      if ( templateVec == null )
         templateVec = new Vector ( 1, 1 ) ;

      templateVec.addElement ( new ProductRow ( productRec ) ) ;
   }




} /* ProductTemplateSystem */

