package com.ibm.nzna.projects.qit.product.compare;

/*-----------------------------------------------------------*/
/*   ProductCompare                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductCompare                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/

/**
 * ProductCompare is a QuestApplet that contains the
 * ProductComparePanel.  This whole applet will compare two
 * products of any type and display differences.  This class
 * will only create the Panel  
 *
 * @author Brian Bokor
 * @version 1.0
 */

import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.compare.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.product.*;
import com.ibm.nzna.projects.qit.product.*;
import com.ibm.nzna.projects.qit.product.gui.*;
import com.ibm.nzna.projects.qit.product.productEdit.*;
import com.ibm.nzna.projects.qit.product.manager.*;


/*----------------------------------------------------*/
/*                                                    */
/* ProductCompare                                     */
/*                                                    */
/*----------------------------------------------------*/
public class ProductCompare implements QuestApplet, AppConst, ProductConst {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
     * This is the Quest Panel that will be used to hold all
     * items for the Product Compare
     */
    private ProductComparePanel productComparePanel = null;  
    /**
     * This is where the ProductCompare came from
     */
    private QuestPanel          previousPanel       = null;
    /**
     * This is the left Product object
     */
    private Product             leftProduct         = null;
    /**
     * This is the right Product object
     */
    private Product             rightProduct        = null;
    
    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     * This constructor like all good Quest
     * Applets does nothing
     *
     * @param None
     * @param None
     * @param None
     *
     * @return Nothing
     *//*---------------------------------------------------------*/
    public ProductCompare() {
    }/*constructor*/

    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     * This constructor will be used if you select
     * Product Compare and pass it one current Product Object
     * .
     *
     * @param Product
     * @param None
     * @param None
     *
     * @return Nothing
     *//*---------------------------------------------------------*/
    public ProductCompare( Product leftProduct ) {
        this.leftProduct = leftProduct;
    }/*constructor*/

    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     * This constructor will be used if you pass it two product objects
     * that you need to use to create the Panel with a Draft and Active
     * Product.
     *
     * @param Product
     * @param Product
     * @param none
     *
     * @return nothing
     *//*---------------------------------------------------------*/
    public ProductCompare( Product leftProduct, Product rightProduct ) {
        this.leftProduct = leftProduct;
        this.rightProduct = rightProduct;
    }/*constructor*/

    /*------------------------------------------------------------*/
    /* initialize
     */
    /**
     * This method will initialize our browser bean as well as get
     * everything else ready
     *
     * @param none
     * @param none
     * @param none
     *
     * @return boolean
     *//*---------------------------------------------------------*/
    public boolean initialize() {
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   First,  create our panel that holds the browser         */
        /*                                                           */
        /*-----------------------------------------------------------*/

      if ( leftProduct != null && rightProduct != null )
        productComparePanel = new ProductComparePanel ( leftProduct, rightProduct, previousPanel ) ;

      else if ( leftProduct != null && rightProduct == null ) 
        productComparePanel = new ProductComparePanel ( leftProduct, previousPanel ) ;    

      else
        productComparePanel = new ProductComparePanel ( previousPanel );
     
      return( true ) ;
    }/*initialize*/

    /*------------------------------------------------------------*/
    /* start
     */
    /**
     * This method will be called when our initialization has completed,
     * and we can now add ourselves to a window
     *
     * @param none
     * @param none
     * @param none
     *
     * @return Nothing
     *//*---------------------------------------------------------*/
    public void start() {
         WindowSystem.createPanel ( productComparePanel ) ;
    }/*start*/

    /*------------------------------------------------------------*/
    /* stop
     */
    /**
     * This method will be called when we need to
     * clean up and stop our Applet.
     *
     * @param none
     * @param none
     * @param none
     *
     * @return boolean
     *//*---------------------------------------------------------*/
    public boolean stop() {
      return ( true );
    }/*stop*/

    /*------------------------------------------------------------*/
    /* shutdown
     */
    /**
     * This method will be called on every open panel upon
     * the user shutting down the application.
     *
     * @param none
     * @param none
     * @param none
     *
     * @return TRUE = Go ahead and call 'stop', I am ready to close.
     *         FALSE = Stop shutdown process and do not close
     *//*---------------------------------------------------------*/
     public boolean shutdown() {
      return ( true );
     }/*shutdown*/

    /*------------------------------------------------------------*/
    /* getName
     */
    /**
     *    This method will return the name
     *  of this Quest Applet
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getName() {
      return( Str.getStr ( STR_PRODUCT_COMPARE ) ) ;
    }/*getName*/

    /*------------------------------------------------------------*/
    /* getDescription
     */
    /**
     *    This method will return a user readable
     *  description of this Quest Applet
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getDescription() {
      return( Str.getStr ( STR_PRODUCT_COMPARE_DESCRIPTION ) ) ;
    }/*getDesription*/

    /*------------------------------------------------------------*/
    /* getAuthority
     */
    /**
     *    This method will return 0, since everyone
     *  has access to the browser
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return int
     *//*---------------------------------------------------------*/
    public int getAuthority() {
      return ( 0 );
    }/*getAuthority*/

    /*------------------------------------------------------------*/
    /* canChat
     */
    /**
     *    This method will return TRUE, since
     *  the web browser can chat with other
     *  web browsers
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return boolean
     *//*---------------------------------------------------------*/
    public boolean canChat() {
      return ( false );
    }/*canChat*/

    /*------------------------------------------------------------*/
    /* getSmallIcon
     */
    /**
     * This method will return the small icon representing
     * this Quest Applet
     *
     * @param none
     * @param none
     * @param none
     *
     * @return ImageIcon
     *
     *//*---------------------------------------------------------*/
    public ImageIcon getSmallIcon() {
      return( ImageSystem.getImageIcon ( this, ImageSystem.PRODUCT_COMPARE ) ) ;
    }/*getSmallIcon*/

    /*------------------------------------------------------------*/
    /* getIcon
     */
    /**
     * This method will return our normal sized icon
     *
     * @param none
     * @param none
     * @param none
     *
     * @return ImageIcon
     *//*---------------------------------------------------------*/
    public ImageIcon getIcon() {
      return( ImageSystem.getImageIcon ( this, ImageSystem.PRODUCT_COMPARE_LARGE ) ) ;
    }/*getIcon*/
}/*ProductCompare*/
