package com.ibm.nzna.projects.qit.product.compare;

/*-----------------------------------------------------------*/
/*   ProductCompareEditPanel                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductCompareEditPanel             */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/

/**
 *
 * The ProductCompareEditPanel will contain a ProductEditPanel.
 * This Object will control all of the features needed for each
 * Panel holding a product.  Some functionality will not be
 * used due to the Product's status(Active or Draft).  IF the 
 * two ProductCompareEditPanels being compared have different 
 * fields then this class will also handle adding a star to the
 * appropriate JLabel and make the text red.
 *
 * @author  Brian R Bokor
 * @version 1.0
 */

import java.awt.* ;
import java.awt.event.* ;
import java.io.* ;
import java.util.* ;
import java.sql.* ; 
import java.awt.datatransfer.* ;
import javax.swing.event.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.qit.product.manager.* ;
import com.ibm.nzna.projects.qit.product.gui.*;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.product.*;
import com.ibm.nzna.projects.qit.product.*; 
import com.ibm.nzna.projects.qit.product.gui.*;
import com.ibm.nzna.projects.qit.product.productEdit.*;
import com.ibm.nzna.projects.qit.product.manager.*;   
/*----------------------------------------------------*/
/*                                                    */
/* ProductCompareEditPanel                           */
/*                                                    */
/*----------------------------------------------------*/
public class ProductCompareEditPanel extends JPanel implements ProductConst, AppConst, Runnable {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Public static final Members                             */
   /*                                                           */
   /*-----------------------------------------------------------*/

  /**
   * This is the Thread variable used for refresh.
   */
   private static final String THREAD_REFRESH_PREVIEW       = "RP"; 
   /**
    * This will be used if the Base Model is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int BASE_MODEL          = 0 ;
   /**
    * This will be used if the Brand is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int BRAND               = 1 ;
   /**
    * This will be used if the Created On is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int CREATED             = 2 ;
   /**
    * This will be used if the Created By is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int CREATED_BY          = 3 ;
   /**
    * This will be used if the Customer is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int CUSTOMER            = 4 ;
   /**
    * This will be used if the Dates are different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int DATES_FOR_COUNTRY   = 5 ;
   /**
    * This will be used if the Family is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int FAMILY              = 6 ;
   /**
    * This will be used if the Family Type is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int FAMILYTYPE          = 7 ;
   /**
    * This will be used if the Input is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int INPUT               = 8 ;
   /**
    * This will be used if the Last OPICM Update is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int LAST_OPICM_UPDATE   = 9 ;
   /**
    * This will be used if the Last Touched By is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int LAST_TOUCHED_BY     = 10 ;
   /**
    * This will be used if the Machine is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int MACHINE             = 11 ;
   /**
    * This will be used if the Model is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int MODEL               = 12 ;
   /**
    * This will be used if the Offering is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int OFFERING            = 13 ;
   /**
    * This will be used if the OPICM Descript is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int OPICM_DESCRIPT      = 14 ;
   /**
    * This will be used if the Part Num is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int PARTNUM             = 15 ;
   /**
    * This will be used if the Product Draft is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int PRODUCT_DRAFT       = 16 ;
   /**
    * This will be used if the Product Ind is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int PRODUCT_IND         = 17 ;
   /**
    * This will be used if the Project Name is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int PROJECTNAME         = 18 ;
   /**
    * This will be used if the Quest Descript is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int QUEST_DESCRIPT      = 19 ;
   /**
    * This will be used if the Source is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int SOURCE              = 20 ;
   /**
    * This will be used if the Warranty is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int WARRANTY            = 21 ;
   /**
    * This will be used if the Location is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int LOCATION            = 22 ;
   /**
    * This will be used if the ProductDraftInd is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int PRODUCT_DRAFT_IND   = 23 ;
   /**
    * This will be used if the DraftCreatedOn is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int DRAFT_CREATED_ON    = 24 ;
   /**
    * This will be used if the LastTouchedOn is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int LAST_TOUCHED_ON     = 25 ;
   /**
    * This will be used if the ProductNum is different to know
    * what JLabel to change in the ProductEditorPanel
    */
   public static final int PROD_NUM            = 26 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This is the object that will be used to create an editable
    * or read-only Panel
    */
   private  ProductEditPanel      editPanel     = null;
   /**
    * This is the Product Object that will be used to fill the
    * Scroll Pane with information
    */
   private Product                product       = null;
                             
   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    * This constructor like all good Quest
    * Applets does nothing
    *
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public ProductCompareEditPanel( ) {
      initialize();
   }/*constuctor*/                      

   /*------------------------------------------------------------*/
   /* init
    */
   /**
    *    This method will be called before we are shown and
    *  will close our panel if we return FALSE. If we
    *  return TRUE everything was initialized ok
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean initialize  () {
       LogSystem.log( 1, "Initializing ProductCompareEditPanel" );
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI from ProductEditPanel                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      editPanel    = new ProductEditPanel();

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setProperties();

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Our Controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout( new BorderLayout() );
      add( editPanel);

      return( true );

   }/*initialize*/

   /*------------------------------------------------------------*/
   /* setProperties
    */
   /**
    *    This method will set the product and our properties
    *    for the ProductEditPanel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setProperties() {
      LogSystem.log( 1, "Setting Properties: ProductCompareEditPanel");

      if ( product != null ) { 
         if ( product instanceof ProductDraft ){
            editPanel.setEditable( true );
         }else if ( product instanceof Product ){ 
            editPanel.setEditable( false );
         }/*if*/
      }/*if*/
   }/*setProperties*/

   /*------------------------------------------------------------*/
   /* saveProduct
    */
   /**
    *    This method will call the save Product method in the
    *    ProductEditPanel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int saveProduct() {
       LogSystem.log( 1, "SaveProduct: ProductCompareEditPanel");
       return( editPanel.saveProduct() );
   }/*saveProduct*/

   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this );
   }/*getJComponent*/

   /*------------------------------------------------------------*/
   /* setChangedLabel
    */
   /**
    *  This method will contain a switch case that will come from
    *  Product Compare.  It will send the Public Int for what value
    *  has changed.  It will know what value has changed from POMRec
    *  having a boolean stating true or false
    *
    * @param int
    * @param none
    * @param none
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public void setChangedLabel( int changeNumber ) {

      LogSystem.beginLogSection("ProductCompareEditPanel- setChangedLabel");
      LogSystem.log( 1, "ChangeNumber: " + changeNumber);

      switch ( changeNumber ) {
         case PROD_NUM :
            editPanel.st_PROD_PARTNUM.setText( Str.getStr( STR_RED_STAR ) + editPanel.st_PROD_PARTNUM.getText() );
            editPanel.st_PROD_PARTNUM.setForeground( Color.red );
            break;
         case BASE_MODEL :
            editPanel.otherInfoPanel.st_BASE.setText( Str.getStr( STR_RED_STAR ) + editPanel.otherInfoPanel.st_BASE.getText() );
            editPanel.otherInfoPanel.st_BASE.setForeground( Color.red );
            break;
         case BRAND :
            editPanel.productInformation.st_BRAND.setText( Str.getStr( STR_RED_STAR ) + editPanel.productInformation.st_BRAND.getText() );
            editPanel.productInformation.st_BRAND.setForeground( Color.red );
            break;
         case CREATED :            
            editPanel.databaseInfoPanel.createdOn.setText( Str.getStr( STR_RED_STAR ) + editPanel.databaseInfoPanel.createdOn.getJLabel().getText() );
            editPanel.databaseInfoPanel.createdOn.setForeground( Color.red );
            break;
         case CREATED_BY :       
            editPanel.databaseInfoPanel.createdBy.setText( Str.getStr( STR_RED_STAR ) + editPanel.databaseInfoPanel.createdBy.getJLabel().getText() );
            editPanel.databaseInfoPanel.createdBy.setForeground( Color.red );
            break;
         case CUSTOMER :           
            editPanel.otherInfoPanel.st_CUSTOMER.setText( Str.getStr( STR_RED_STAR ) + editPanel.otherInfoPanel.st_CUSTOMER.getText() );
            editPanel.otherInfoPanel.st_CUSTOMER.setForeground( Color.red );
            break;
         case DATES_FOR_COUNTRY :
            editPanel.datePanel.st_COUNTRY.setText( Str.getStr( STR_RED_STAR ) + editPanel.datePanel.st_COUNTRY.getText() );
            editPanel.datePanel.st_COUNTRY.setForeground( Color.red );
            break;
         case FAMILY :             
            editPanel.productInformation.st_FAMILY.setText( Str.getStr( STR_RED_STAR ) + editPanel.productInformation.st_FAMILY.getText() );
            editPanel.productInformation.st_FAMILY.setForeground( Color.red );
            break;
         case FAMILYTYPE :         
            editPanel.productInformation.st_MACHINE.setText( Str.getStr( STR_RED_STAR ) + editPanel.productInformation.st_MACHINE.getText() );
            editPanel.productInformation.st_MACHINE.setForeground( Color.red );
            break;
         case INPUT :              
            editPanel.otherInfoPanel.st_INPUT.setText( Str.getStr( STR_RED_STAR ) + editPanel.otherInfoPanel.st_INPUT.getText() );
            editPanel.otherInfoPanel.st_INPUT.setForeground( Color.red );
            break;
         case LAST_TOUCHED_BY :    
            editPanel.databaseInfoPanel.lastTouchedBy.setText( Str.getStr( STR_RED_STAR ) + editPanel.databaseInfoPanel.lastTouchedBy.getJLabel().getText() );
            editPanel.databaseInfoPanel.lastTouchedBy.setForeground( Color.red );
            break;
         case MACHINE :            
            editPanel.productInformation.st_MACHINE.setText( Str.getStr( STR_RED_STAR ) + editPanel.productInformation.st_MACHINE.getText() );
            editPanel.productInformation.st_MACHINE.setForeground( Color.red );
            break;
         case MODEL :              
            editPanel.productInformation.st_MODEL.setText( Str.getStr( STR_RED_STAR ) + editPanel.productInformation.st_MODEL.getText() );
            editPanel.productInformation.st_MODEL.setForeground( Color.red );
            break;
         case OFFERING :          
            editPanel.otherInfoPanel.st_OFFERING.setText( Str.getStr( STR_RED_STAR ) + editPanel.otherInfoPanel.st_OFFERING.getText() );
            editPanel.otherInfoPanel.st_OFFERING.setForeground( Color.red );
            break;
         case OPICM_DESCRIPT :     
            editPanel.description.st_OPICM.setText( Str.getStr( STR_RED_STAR ) + editPanel.description.st_OPICM.getText() );
            editPanel.description.st_OPICM.setForeground( Color.red );
            break;
         case PARTNUM :            
            editPanel.productInformation.st_MODEL.setText( Str.getStr( STR_RED_STAR ) + editPanel.productInformation.st_MODEL.getText() );
            editPanel.productInformation.st_MODEL.setForeground( Color.red );  
            break;
         case PRODUCT_IND :        
            editPanel.databaseInfoPanel.activeProductInd.setText( Str.getStr( STR_RED_STAR ) + editPanel.databaseInfoPanel.activeProductInd.getJLabel().getText() );
            editPanel.databaseInfoPanel.activeProductInd.setForeground( Color.red );
            break;
         case PROJECTNAME :        
            editPanel.productInformation.st_PROJECT.setText( Str.getStr( STR_RED_STAR ) + editPanel.productInformation.st_PROJECT.getText() );
            editPanel.productInformation.st_PROJECT.setForeground( Color.red );
            break;
         case QUEST_DESCRIPT :     
            editPanel.description.st_QUEST.setText( Str.getStr( STR_RED_STAR ) + editPanel.description.st_QUEST.getText() );
            editPanel.description.st_QUEST.setForeground( Color.red );
            break;
         case SOURCE :             
            editPanel.otherInfoPanel.st_SOURCE.setText( Str.getStr( STR_RED_STAR ) + editPanel.otherInfoPanel.st_SOURCE.getText() );
            editPanel.otherInfoPanel.st_SOURCE.setForeground( Color.red );
            break;
         case WARRANTY :           
            editPanel.otherInfoPanel.st_WARRANTY.setText( Str.getStr( STR_RED_STAR ) + editPanel.otherInfoPanel.st_WARRANTY.getText() );
            editPanel.otherInfoPanel.st_WARRANTY.setForeground( Color.red );
            break;
         case LOCATION :
            editPanel.localePanel.help.setText( Str.getStr( STR_RED_STAR ) + editPanel.localePanel.help.getText() );
            editPanel.localePanel.help.setForeground( Color.red );
            break;
         case PRODUCT_DRAFT_IND:
            editPanel.databaseInfoPanel.draftProductInd.setText( Str.getStr( STR_RED_STAR ) + editPanel.databaseInfoPanel.draftProductInd.getJLabel().getText() );
            editPanel.databaseInfoPanel.draftProductInd.setForeground( Color.red ); 
            break;
         case DRAFT_CREATED_ON:
            editPanel.databaseInfoPanel.draftCreatedOn.setText( Str.getStr( STR_RED_STAR ) + editPanel.databaseInfoPanel.draftCreatedOn.getJLabel().getText() );
            editPanel.databaseInfoPanel.draftCreatedOn.setForeground( Color.red );
            break;
         case LAST_TOUCHED_ON:
            editPanel.databaseInfoPanel.lastTouchedOn.setText( Str.getStr( STR_RED_STAR ) + editPanel.databaseInfoPanel.lastTouchedOn.getJLabel().getText() );
            editPanel.databaseInfoPanel.lastTouchedOn.setForeground( Color.red );
            break;
      }/*switch*/   
      LogSystem.endLogSection();
   }/*setChangedLabel*/

   /*------------------------------------------------------------*/
   /* getProduct
    */
   /**
    *  This method will get the product that is currently
    *  being used by this Class
    *
    * @param none
    * @param none
    * @param none
    *
    * @return Product
    *//*---------------------------------------------------------*/
   public Product getProduct(  ) {
      return( product );
   }/*getProduct*/

   /*------------------------------------------------------------*/
   /* setProduct
    */
   /**
    *  This method will set the current product and it is
    *  private only to this Class
    *
    * @param Product
    * @param none
    * @param none
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public void setProduct( Product product ) {
      LogSystem.log( 1, "Setting Product: ProductCompareEditPanel");
      this.product = product;
      new Thread ( this, THREAD_REFRESH_PREVIEW ).start() ; 
   }/*setProduct*/

   /*------------------------------------------------------------*/
   /* refreshPreview
    */
   /**
    *    This method will refresh data for the Product.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/ 
   private void refreshPreview(){  
      editPanel.setProduct( product );
      setProperties();
   }

   /*-----------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread performs many operations. The operation
    *  done is determined by the thread name
    *
    *
    *  @param none
    *  @param none
    *  @param none
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run ( ) {

      String operation = Thread.currentThread().getName() ;

      Qit.setEnabled ( false ) ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_REFRESH_PREVIEW                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( operation.equals ( THREAD_REFRESH_PREVIEW ) )
            refreshPreview() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
      Qit.setEnabled ( true ) ;
   }/* run */   

   /*------------------------------------------------------------*/
   /* getFormPanel
    */
   /**
    *  This method will get the DPanel from the 
    *  ProductEditPanel
    *
    * @param none
    * @param none
    * @param none
    *
    * @return DPanel
    *//*---------------------------------------------------------*/
   public DPanel getFormPanel () {
      editPanel.getFormPanel().setPreferredSize( new Dimension ( -1, -1 ) ) ;
      return( editPanel.getFormPanel() ) ;
   }/*getFormPanel*/

   /*------------------------------------------------------------*/
   /* getPanel
    */
   /**
    *  This method will get an instance of the ProductEditPanel
    *
    * @param none
    * @param none
    * @param none
    *
    * @return ProductEditPanel
    *//*---------------------------------------------------------*/
   public ProductEditPanel getPanel () {
      return( editPanel ) ;
   }/*getPanel*/        
} /* ProductCompareEditPanel */
