package com.ibm.nzna.projects.qit.product.compare;

/*-----------------------------------------------------------*/
/*   ProductComparePOMRecCompare                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductComparePOMRecCompare         */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/

/**
 *
 * The ProductComparePOMRecCompare is set up to compare the two
 * objects and will only be created if you have two Product Objects.
 * It will compare all the attributes from a Product object that will
 * be contained in the ProductEditPanel that need to be compared.
 * There will be boolean variables that will state if there is a
 * difference between the two attributes being compared.  Variables
 * will public so that they can be easily accessed from the
 * ProductComparePanel.
 *
 * All the compare will be encapsulated in here so you will
 * not see anything but the boolean variables to state if there
 * is a difference or not.
 *
 * @author  Brian R Bokor
 * @version 1.0
 */

import java.io.* ;
import java.util.* ;
import java.util.Vector ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.common.quest.*;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.product.*;
import com.ibm.nzna.projects.qit.product.productEdit.*;
import com.ibm.nzna.projects.qit.product.compare.*;

/*----------------------------------------------------*/
/*                                                    */
/* ProductComparePOMRecCompare                        */
/*                                                    */
/*----------------------------------------------------*/

public class ProductComparePOMRecCompare extends Object {

    /**
     * This will be the hard coded value for an Option
     */                     
    private static final int OPTION     = 6 ;

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
     * This will be the left Product being compared
     */
    private Product           leftProduct           = null;
    /**
     * This will be the right Product being compared
     */
    private Product           rightProduct          = null;
    /**
     * This will be the left base model needed for moving changes
     */
    private String            L_baseModel           = null;
    /**
     * This will be the right base model needed for moving changes
     */
    private String            R_baseModel           = null;
    /**
     * This will be the left brand needed for moving changes
     */
    private TypeCategoryRec   L_brand               = null;
    /**
     * This will be the right brand needed for moving changes
     */
    private TypeCategoryRec   R_brand               = null;
    /**
     * This will be the left created needed for moving changes
     */
    private String            L_created             = null;
    /**
     * This will be the right created needed for moving changes
     */
    private String            R_created             = null;
    /**
     * This will be the left Created By needed for moving changes
     */
    private String            L_createdBy           = null;
    /**
     * This will be the right Created By needed for moving changes
     */
    private String            R_createdBy           = null;
    /**
     * This will be the left Customer needed for moving changes
     */
    private int               L_customer            = 0;
    /**
     * This will be the right Customer needed for moving changes
     */
    private int               R_customer            = 0;
    /**
     * This will be the left dates needed for moving changes
     */
    private Vector            L_dates               = null;
    /**
     * This will be the right dates needed for moving changes
     */
    private Vector            R_dates               = null;
    /**
     * This will be the left Family needed for moving changes
     */
    private TypeCategoryRec   L_family              = null;
    /**
     * This will be the right Family needed for moving changes
     */
    private TypeCategoryRec   R_family              = null;
    /**
     * This will be the left Family Type needed for moving changes
     */
    private TypeCategoryRec   L_familyType          = null;
    /**
     * This will be the right Family Type needed for moving changes
     */
    private TypeCategoryRec   R_familyType          = null;
    /**
     * This will be the left Input needed for moving changes
     */
    private String            L_input               = null;
    /**
     * This will be the right Input needed for moving changes
     */
    private String            R_input               = null;
    /**
     * This will be the left Last Touched By needed for moving changes
     */
    private String            L_lastTouchedBy       = null;
    /**
     * This will be the right Last Touched By needed for moving changes
     */
    private String            R_lastTouchedBy       = null;
    /**
     * This will be the left Last Touched On needed for moving changes
     */
    private String            L_lastTouchedOn       = null;
    /**
     * This will be the right Last Touched On needed for moving changes
     */
    private String            R_lastTouchedOn       = null;
    /**
     * This will be the left Machine needed for moving changes
     */
    private TypeCategoryRec   L_machine             = null;
    /**
     * This will be the right Machine needed for moving changes
     */
    private TypeCategoryRec   R_machine             = null;
    /**
     * This will be the left Model needed for moving changes
     */
    private TypeCategoryRec   L_model               = null;
    /**
     * This will be the right Model needed for moving changes
     */
    private TypeCategoryRec   R_model               = null;
    /**
     * This will be the left Offering needed for moving changes
     */
    private int               L_offering            = 0;
    /**
     * This will be the right Offering needed for moving changes
     */
    private int               R_offering            = 0;
    /**
     * This will be the left Part Num needed for moving changes
     */
    private TypeCategoryRec   L_partNum             = null;
    /**
     * This will be the right Part Num needed for moving changes
     */
    private TypeCategoryRec   R_partNum             = null;
    /**
     * This will be the left Product Ind needed for moving changes
     */
    private int               L_productInd          = 0;
    /**
     * This will be the right Product Ind needed for moving changes
     */
    private int               R_productInd          = 0;
    /**
     * This will be the left DraftProduct Ind needed for moving changes
     */
    private int               L_draftProductInd     = 0;
    /**
     * This will be the right DraftProduct Ind needed for moving changes
     */
    private int               R_draftProductInd     = 0;
    /**
     * This will be the left Project Name needed for moving changes
     */
    private TypeCategoryRec   L_projectName         = null;
    /**
     * This will be the right Project Name needed for moving changes
     */
    private TypeCategoryRec   R_projectName         = null;
    /**
     * This will be the left Quest Description needed for moving changes
     */
    private String            L_questDescript       = null;
    /**
     * This will be the right Quest Description needed for moving changes
     */
    private String            R_questDescript       = null;
    /**
     * This will be the right OPICM Description needed for moving changes
     */ 
    private String            L_opicmDescript       = null;
    /**
     * This will be the right OPICM Description needed for moving changes
     */ 
    private String            R_opicmDescript       = null;    
    /**
     * This will be the left Source needed for moving changes
     */
    private String            L_source              = null;
    /**
     * This will be the right Source needed for moving changes
     */
    private String            R_source              = null;
    /**
     * This will be the left Warranty needed for moving changes
     */
    private int               L_warranty            = 0;
    /**
     * This will be the right Warranty needed for moving changes
     */
    private int               R_warranty            = 0;
    /**
     * This will be the left Location needed for moving changes
     */
    private LocaleRec         L_locale              = null;
    /**
     * This will be the right Location needed for moving changes
     */
    private LocaleRec         R_locale              = null;
    /**
     * This will be the Left Ind to show if it is a System or Option
     */
    private int               L_brandInd            = 0;
    /**
     * This will be the right Ind to show if it is a System or Option
     */    
    private int               R_brandInd            = 0;  
    /**
     * This will be the left DraftCreatedOn needed for moving changes
     */
    private String            L_draftCreatedOn      = null;
    /**
     * This will be the right DraftCreatedOn needed for moving changes
     */
    private String            R_draftCreatedOn      = null; 
    /**
     * This will be the left ProdNum needed for moving changes
     */
    private String            L_prodNum             = null;
    /**
     * This will be the right ProdNum needed for moving changes
     */
    private String            R_prodNum             = null;

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Public Members                                          */
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
     * This boolean will show if the Base Models are different
     */
    public boolean BASE_MODEL             = true ; //String
    /**
     * This boolean will show if the Brand are different
     */
    public boolean BRAND                  = true ; //TypeBrandRec
    /**
     * This boolean will show if the Created are different
     */
    public boolean CREATED                = true ; //String
    /**
     * This boolean will show if the Created By are different
     */
    public boolean CREATED_BY             = true ; //String
    /**
     * This boolean will show if the Customer are different
     */
    public boolean CUSTOMER               = true ; //int -> TypeList
    /**
     * This boolean will show if the Dates are different
     */
    public boolean DATES_FOR_COUNTRY      = true ; //Vector
    /**
     * This boolean will show if the Family are different
     */
    public boolean FAMILY                 = true ; //TypeFamilyRec
    /**
     * This boolean will show if the FamilyType are different
     */
    public boolean FAMILYTYPE             = true ; //TypeFamilyTypeRec
    /**
     * This boolean will show if the Input are different
     */
    public boolean INPUT                  = true ; //String
    /**
     * This boolean will show if the Last Touched By are different
     */
    public boolean LAST_TOUCHED_BY        = true ; //String
    /**
     * This boolean will show if the Machine are different
     */
    public boolean MACHINE                = true ; //String
    /**
     * This boolean will show if the Model are different
     */
    public boolean MODEL                  = true ; //String
    /**
     * This boolean will show if the Offering are different
     */
    public boolean OFFERING               = true ; //int ->offeringTypeind
    /**
     * This boolean will show if the OPIC/M Description are different
     */
    public boolean OPICM_DESCRIPT         = true ;
    /**
     * This boolean will show if the PartNum are different
     */
    public boolean PARTNUM                = true ; //String
    /**
     * This boolean will show if the DraftProductInd are different
     */
    public boolean PRODUCT_DRAFT_IND      = true ;  //int
    /**
     * This boolean will show if the Product Ind are different
     */
    public boolean PRODUCT_IND            = true ; //int
    /**
     * This boolean will show if the Project Name are different
     */
    public boolean PROJECTNAME            = true ; //TypeProjectRec
    /**
     * This boolean will show if the Quest Description are different
     */
    public boolean QUEST_DESCRIPT         = true ; //String
    /**
     * This boolean will show if the Source are different
     */
    public boolean SOURCE                 = true ; //String
    /**
     * This boolean will show if the Warranty By are different
     */
    public boolean WARRANTY               = true ; //int ->warrentyTypeInd
    /**
     * This boolean will show if the Location By are different
     */
    public boolean LOCATION               = true ; //LocaleRec 
    /**
     * This boolean will show if the DraftCreatedOn are different
     */
    public boolean DRAFT_CREATED_ON       = true ; //String
    /**
     * This boolean will show if the LastTouchedOn are different
     */
    public boolean LAST_TOUCHED_ON        = true ; //String
    /**
     * This boolean will show if the ProdNum are different
     */
    public boolean PROD_NUM               = true ; //String



    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     * This constructor will have two products being compared.  There
     * will be no other way to instanciate this object
     *
     * @param None
     * @param None
     * @param None
     *
     * @return Nothing
     *//*---------------------------------------------------------*/
    public ProductComparePOMRecCompare( Product leftProduct, Product rightProduct ) {
      this.leftProduct  = leftProduct;
      this.rightProduct = rightProduct;
      LogSystem.beginLogSection ( "Comparing Products..." );
      getValues();
      recCompare();
      LogSystem.endLogSection() ;
    }

    /*------------------------------------------------------------*/
    /* recCompare
     */
    /**
     * This method will contain all the booleans and there compare
     * method.
     *
     * @param None
     * @param None
     * @param None
     *
     * @return Nothing
     *//*---------------------------------------------------------*/
    private void recCompare(){

        LogSystem.log( 1, "Started: Comparing Values" );

        BASE_MODEL         = stringCompare( L_baseModel, R_baseModel ); 
        BRAND              = typeCategoryRecCompare( L_brand, R_brand );

        if ( L_brandInd != OPTION ) {
             if ( R_brandInd != OPTION ){
                MACHINE            = typeCategoryRecCompare( L_machine, R_machine );  
                MODEL              = typeCategoryRecCompare( L_model, R_model );      
             }
             else{
                 if ( R_brandInd == OPTION ){
                    MACHINE       = false;
                    MODEL         = false;
                 }/*if*/
             }/*if*/
         }/*if*/
         else { 
             if ( R_brandInd == OPTION ){
                 PARTNUM            = typeCategoryRecCompare( L_partNum, R_partNum );
                 FAMILYTYPE         = typeCategoryRecCompare( L_familyType, R_familyType );
             }
             else{
                 MACHINE    = false;
                 MODEL      = false;
             }/*if*/
         }/*if*/

        CREATED            = stringCompare( L_created, R_created ); 
        CREATED_BY         = stringCompare( L_createdBy, R_createdBy );
        CUSTOMER           = intCompare( L_customer, R_customer ); 
        DATES_FOR_COUNTRY  = vectorCompare( L_dates, R_dates ); 
        FAMILY             = typeCategoryRecCompare( L_family, R_family ); 
        INPUT              = stringCompare( L_input, R_input ); 
        LAST_TOUCHED_BY    = stringCompare( L_lastTouchedBy, R_lastTouchedBy );
        LAST_TOUCHED_ON    = stringCompare( L_lastTouchedOn, R_lastTouchedOn );
        OFFERING           = intCompare( L_offering, R_offering );
        OPICM_DESCRIPT     = stringCompare( L_opicmDescript, R_opicmDescript );
        PRODUCT_IND        = intCompare( L_productInd, R_productInd ); 
        PROJECTNAME        = typeCategoryRecCompare( L_projectName, R_projectName ); 
        QUEST_DESCRIPT     = stringCompare( L_questDescript, R_questDescript );
        SOURCE             = stringCompare( L_source, R_source ); 
        WARRANTY           = intCompare( L_warranty, R_warranty );
        LOCATION           = localeRecCompare( L_locale, R_locale );
        DRAFT_CREATED_ON   = stringCompare( L_draftCreatedOn, R_draftCreatedOn );
        PRODUCT_DRAFT_IND  = intCompare( L_draftProductInd, R_draftProductInd ); 
        PROD_NUM           = stringCompare( L_prodNum, R_prodNum );

        LogSystem.log( 1, "Finished: Comparing Values" );
    }/*recCompare*/

    /*------------------------------------------------------------*/
    /* getValues
     */
    /**
     * This method will get all the values needed to compare with.
     *
     * @param None
     * @param None
     * @param None
     *
     * @return Nothing
     *//*---------------------------------------------------------*/
    private void getValues(){
        LogSystem.log ( 1, "Started:  Populating Product Values" ) ;

        L_baseModel     =  leftProduct.getBaseModel();       //   String            
        R_baseModel     =  rightProduct.getBaseModel();      //   String            
        L_brand         =  leftProduct.getBrand();           //   TypeCategoryRec      
        R_brand         =  rightProduct.getBrand();          //   TypeCategoryRec  
        L_created       =  leftProduct.getCreatedOn();       //   String            
        R_created       =  rightProduct.getCreatedOn();      //   String            
        L_createdBy     =  leftProduct.getCreatedBy();       //   String            
        R_createdBy     =  rightProduct.getCreatedBy();      //   String            
        L_customer      =  leftProduct.getCustomer();        //   int               
        R_customer      =  rightProduct.getCustomer();       //   int               
        L_dates         =  leftProduct.getDates();           //   Vector            
        R_dates         =  rightProduct.getDates();          //   Vector            
        L_family        =  leftProduct.getFamily();          //   TypeCategoryRec     
        R_family        =  rightProduct.getFamily();         //   TypeCategoryRec 
        L_input         =  leftProduct.getInput();           //   String            
        R_input         =  rightProduct.getInput();          //   String            
        L_lastTouchedBy =  leftProduct.getLastTouchedBy();   //   String            
        R_lastTouchedBy =  rightProduct.getLastTouchedBy();  //   String 
        L_lastTouchedOn =  leftProduct.getLastTouchedOn();   //   String            
        R_lastTouchedOn =  rightProduct.getLastTouchedOn();  //   String 
        L_offering      =  leftProduct.getOffering();        //   int               
        R_offering      =  rightProduct.getOffering();       //   int 
        L_projectName   =  leftProduct.getProject();         //   TypeCategoryRec    
        R_projectName   =  rightProduct.getProject();        //   TypeCategoryRec    
        L_questDescript =  leftProduct.getDescription();     //   String            
        R_questDescript =  rightProduct.getDescription();    //   String      
        L_source        =  leftProduct.getSource();          //   String            
        R_source        =  rightProduct.getSource();         //   String            
        L_warranty      =  leftProduct.getWarranty();        //   int               
        R_warranty      =  rightProduct.getWarranty();       //   int               
        L_locale        =  leftProduct.getLocale();          //   LocaleRec         
        R_locale        =  rightProduct.getLocale();         //   LocaleRec 
        L_prodNum       =  leftProduct.getProdNum();         //   String
        R_prodNum       =  rightProduct.getProdNum();        //   String

        L_brandInd      = L_brand.getInd();                  //   int
        R_brandInd      = R_brand.getInd();                  //   int 

        if ( L_brandInd != OPTION ) {
            L_machine       =  leftProduct.getMachine();     //   TypeCategoryRec   
            L_model         =  leftProduct.getModel();       //   TypeCategoryRec   
        }
        else{
            L_familyType    =  leftProduct.getSubCategory(); //   TypeCategoryRec 
            L_partNum       =  leftProduct.getPartNumber();  //   String            
        }/*if*/

        if ( R_brandInd != OPTION ) {
            R_machine       =  rightProduct.getMachine();    //   TypeCategoryRec   
            R_model         =  rightProduct.getModel();      //   TypeCategoryRec            
        }
        else{
            R_familyType    =  rightProduct.getSubCategory();//   TypeCategoryRec
            R_partNum       =  rightProduct.getPartNumber(); //   String 
        }/*if*/

        if ( leftProduct instanceof ProductDraft ) {
            L_draftProductInd  = ((ProductDraft)leftProduct).getProductInd();         //int
            L_productInd       = ((ProductDraft)leftProduct).getExistingProductInd(); //int
        }
        else{
            L_draftProductInd  = 0;
            L_productInd       = leftProduct.getProductInd();                         //int
        }/*if*/

        if ( rightProduct instanceof ProductDraft ) {
            R_draftProductInd  = ((ProductDraft)rightProduct).getProductInd();        //int
            R_productInd       = ((ProductDraft)rightProduct).getExistingProductInd();//int
        }
        else{
            R_draftProductInd  = 0;
            R_productInd       = rightProduct.getProductInd();                        //int
        }/*if*/


        if ( leftProduct instanceof ProductDraft ){
            L_opicmDescript =  ((ProductDraft)leftProduct).getOPICMDescript();        //String
            L_draftCreatedOn = ((ProductDraft)leftProduct).getStartDate();            //String
        }/*if*/
        if ( rightProduct instanceof ProductDraft ){
            R_opicmDescript =  ((ProductDraft)rightProduct).getOPICMDescript();       //String
            R_draftCreatedOn = ((ProductDraft)rightProduct).getStartDate();           //String
        }/*if*/                      

        LogSystem.log ( 1, "Finished:  Product Values Populated" ) ;
    }/*getValues*/

    /*------------------------------------------------------------*/
    /* stringCompare
     */
    /**
     * This method will do a simple string compare for two strings
     * that are passed in
     *
     * @param String
     * @param String
     * @param None
     *
     * @return boolean  False if they are different
     *                  True if the are the same
     *//*---------------------------------------------------------*/
    private boolean stringCompare( String x, String y ){

        if( x == null ) 
            x = "";

        if( y == null )
            y = "";

        if ( x.compareTo ( y ) == 0 )
            return ( true );
        else
            return ( false );
        
    }/*stringCompare*/

    /*------------------------------------------------------------*/
    /* intCompare
     */
    /**
     * This method will do a simple int compare for two strings
     * that are passed in
     *
     * @param int
     * @param int
     * @param None
     *
     * @return boolean  False if they are different
     *                  True if the are the same
     *//*---------------------------------------------------------*/
    private boolean intCompare( int x, int y ){

        if ( x == y )
            return ( true );
        else
            return ( false );
    }/*intCompare*/

    /*------------------------------------------------------------*/
    /* typeCategoryRecCompare
     */
    /**
     * This method will compare the type Categories for brand, 
     * family, familytype, machine, model and project
     *
     * @param TypeCategoryRec
     * @param TypeCategoryRec
     * @param none
     *
     * @return boolean  False if they are different
     *                  True if the are the same
     *//*---------------------------------------------------------*/
    private boolean typeCategoryRecCompare( TypeCategoryRec x, TypeCategoryRec y ){

        return( stringCompare( x.getDescript(), y.getDescript() ) ); 
    }/*typeCategoryRecCompare*/

    /*------------------------------------------------------------*/
    /* localeRecCompare
     */
    /**
     * This method will do a localeRec compare
     *
     * @param TypeProjectRec
     * @param TypeProjectRec
     * @param none
     *
     * @return boolean  False if they are different
     *                  True if the are the same
     *//*---------------------------------------------------------*/
    private boolean localeRecCompare( LocaleRec x, LocaleRec y ){

        return( stringCompare( x.getLocaleList(), y.getLocaleList() ) );
    }/*localeRecCompare*/

    /*------------------------------------------------------------*/
    /* vectorCompare
     */
    /**
     * This method will do a vector compare
     *
     * @param TypeProjectRec
     * @param TypeProjectRec
     * @param none
     *
     * @return boolean  False if they are different
     *                  True if the are the same
     *//*---------------------------------------------------------*/
    private boolean vectorCompare( Vector x, Vector y ){

        return ( stringCompare( x.toString(), y.toString() ) );
    }/*VectorCompare*/

    /*------------------------------------------------------------*/
    /* leftMoveChanges
     */
    /**
     * This method will be called if the changes need moved over
     * and return the Product with new values
     *
     * @param none
     * @param none
     * @param none
     *
     * @return Product
     *//*---------------------------------------------------------*/
    public Product leftMoveChanges(){

        LogSystem.beginLogSection(" LeftMoveChanges..." );


        if ( ( L_brandInd != OPTION ) && ( R_brandInd != OPTION )  ) {
            if ( !MACHINE )
                rightProduct.setMachine( L_machine );
            if ( !MODEL ) 
                rightProduct.setModel( L_model );
        }
        else{
            if ( ( L_brandInd == OPTION ) && ( R_brandInd == OPTION ) ) {
                if ( !FAMILYTYPE )
                    rightProduct.setSubCategory( L_familyType );
                if ( !PARTNUM ) 
                    rightProduct.setPartNumber( L_partNum );
            }/*if*/
        }/*if*/

        if ( !BRAND )
            rightProduct.setBrand( L_brand );
        if ( !FAMILY )
            rightProduct.setFamily( L_family );
        if ( !PROJECTNAME ) 
            rightProduct.setProject( L_projectName );
        if ( !BASE_MODEL )        
            rightProduct.setBaseModel( L_baseModel );
        if ( !CUSTOMER )                                     
            rightProduct.setCustomer( L_customer );       
        if ( !DATES_FOR_COUNTRY )                            
            rightProduct.setDates( L_dates );
        if ( !INPUT )                                        
            rightProduct.setInput( L_input );    
        if ( !OFFERING )                                     
            rightProduct.setOffering( L_offering );
        if ( !PROJECTNAME )                                  
            rightProduct.setProject( L_projectName );    
        if ( !QUEST_DESCRIPT )                            
            rightProduct.setDescription( L_questDescript );  
        if ( !WARRANTY )                                  
            rightProduct.setWarranty( L_warranty );       
        if ( !LOCATION )                                  
            rightProduct.setLocale( L_locale );

        LogSystem.log( 1, "MovedChanges");

        LogSystem.endLogSection();

        return ( rightProduct );
    }/*leftMoveChanges*/

    /*------------------------------------------------------------*/
    /* rightMoveChanges
     */
    /**
     * This method will be called if the changes need moved over
     * and return the Product with new values
     *
     * @param none
     * @param none
     * @param none
     *
     * @return Product
     *//*---------------------------------------------------------*/
    public Product rightMoveChanges(){

        LogSystem.beginLogSection(" RightMoveChanges..." );

        if ( ( L_brandInd != OPTION ) && ( R_brandInd != OPTION )  ) {
            if ( !MACHINE )
                leftProduct.setMachine( R_machine );
            if ( !MODEL ) 
                leftProduct.setModel( R_model );
        }
        else{
            if ( ( L_brandInd == OPTION ) && ( R_brandInd == OPTION ) ) {
                if ( !FAMILYTYPE )
                    leftProduct.setSubCategory( R_familyType );
                if ( !PARTNUM ) 
                    leftProduct.setPartNumber( R_partNum );
            }/*if*/
        }/*if*/

        if ( !BRAND )
            leftProduct.setBrand( R_brand );
        if ( !FAMILY )
            leftProduct.setFamily( R_family );
        if ( !PROJECTNAME ) 
            leftProduct.setProject( R_projectName );
        if ( !BASE_MODEL )        
            leftProduct.setBaseModel( R_baseModel );
        if ( !CUSTOMER )                                     
            leftProduct.setCustomer( R_customer );       
        if ( !DATES_FOR_COUNTRY )                            
            leftProduct.setDates( R_dates );
        if ( !FAMILYTYPE )                                   
            leftProduct.setSubCategory( R_familyType );
        if ( !INPUT )                                        
            leftProduct.setInput( R_input );    
        if ( !OFFERING )                                     
            leftProduct.setOffering( R_offering );
        if ( !PROJECTNAME )                                  
            leftProduct.setProject( R_projectName );    
        if ( !QUEST_DESCRIPT )                            
            leftProduct.setDescription( R_questDescript );  
        if ( !WARRANTY )                                  
            leftProduct.setWarranty( R_warranty );       
        if ( !LOCATION )                                  
            leftProduct.setLocale( R_locale );

        LogSystem.log( 1, "MovedChanges");

        LogSystem.endLogSection();

        return ( leftProduct );
    }/*rightMoveChange*/
} /* ProductComparePOMRecCompare */
