package com.ibm.nzna.projects.qit.product.compare;

/*-----------------------------------------------------------*/
/*   ProductComparePanel                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductComparePanel                 */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/

/**
 * The ProductComparePanel is used to compare any two ProductCompareEditPanels. The
 * Product Objects being compared can either be Draft or Active.
 * It will also containa ProductComparePOMRecCompare Object that
 * will compare all the attributes of a Product.  If there is an attribute
 * that is different from one product to another then there will
 * be a Red Star placed in front of that JLabel in which the attribute is different.
 *
 *
 * @author Brian Bokor
 * @version 1.0
 */


import java.awt.* ;
import java.io.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.help.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.product.*;
import com.ibm.nzna.projects.qit.product.*;
import com.ibm.nzna.projects.qit.product.gui.*;
import com.ibm.nzna.projects.qit.product.productEdit.*;
import com.ibm.nzna.projects.qit.product.manager.*;
import com.ibm.nzna.projects.qit.product.compare.*;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;

/*----------------------------------------------------*/
/*                                                    */
/* ProductComparePanel                                */
/*                                                    */
/*----------------------------------------------------*/

public class ProductComparePanel extends JPanel implements Authority, ProductEditPanelListener, AdjustmentListener, QuestPanel, ProductConst, AppConst, ActionListener, FindProductListener, Runnable, HelpSource {

   /**
   * This is the Thread variable used for refresh.
   */
   private static final String THREAD_REFRESH_PREVIEW       = "RP" ;
   /**
    * This is the Thread variable used for refresh.
    */
   private static final String THREAD_REFRESH_RIGHT_PRODUCT = "RIGHT_PRODUCT_THREAD" ;
   /**
    * This is the Thread variable used for refresh.
    */
   private static final String THREAD_REFRESH_LEFT_PRODUCT  = "LEFT_PRODUCT_THREAD" ;
   /**
    * This is the String variable that says we are dealing with the
    * left side of things
    */
   private static final String left                         = "LEFT" ;
   /**
    * This is the String variable that says we are dealing with the
    * right side of things
    */
   private static final String right                        = "RIGHT" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private, GUI Members                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This Panel will be used to contain a NavListGroup and ProductCompareScrollPane for the left Product.
    */
   private JPanel         leftPanel                    = null; //Will be used to hold the ScrollPanes for scrolling purposes
   /**
     *  This is the JScrollPane which holds the left panel
     **/
   private JScrollPane leftScrollPanel                 = null ;
   /**
     *  This is the JScrollPane which holds the right panel
     **/
   private JScrollPane rightScrollPanel                = null ;
   /**
    * This Panel will be used to contain a NavListGroup and ProductCompareScrollPane for the right Product
    */
   private JPanel         rightPanel                   = null; //Will be used to hold the ScrollPanes for scrolling purposes
   /**
    * This will hold the HotLinkLabels for the left side of the JSplitPane
    */
   private ListPanel      leftListPanel                = null; //ListPanel will contain the HotLinkLabels
   /**
    * This will hold the HotLinkLabels for the right side of the JSplitPane
    */
   private ListPanel      rightListPanel               = null; //ListPanel will contain the HotLinkLabels
   /**
    * This will contain the text "Draft" and will not have any action
    */
   private HotLinkLabel   hll_LEFT_TITLE               = null; //the HotLinkLabels will be used for the action buttons
   /**
   * This will contain the text "Draft" and will not have any action
   */
   private HotLinkLabel   hll_RIGHT_TITLE              = null; //the HotLinkLabels will be used for the action buttons
   /**
    * This will contain the text "Change Product" and will pull up the ProductFindDlg
    */
   private HotLinkLabel   hll_LEFT_CHANGE_PRODUCT      = null; //the HotLinkLabels will be used for the action buttons
   /**
    * This will contain the text "Change Product" and will pull up the ProductFindDlg
    */
   private HotLinkLabel   hll_RIGHT_CHANGE_PRODUCT     = null; //the HotLinkLabels will be used for the action buttons
   /**
    * This will contain the text "Move Changes >>" and will use the ProductComparePOMRecCompare to get changes
    */
   private HotLinkLabel   hll_LEFT_MOVE_CHANGES        = null; //the HotLinkLabels will be used for the action buttons
   /**
    * This will contain the text "<<Move Changes" and will use the ProductComparePOMRecCompare to get changes
    */
   private HotLinkLabel   hll_RIGHT_MOVE_CHANGES       = null; //the HotLinkLabels will be used for the action buttons
   /**
    * This will pull up the ProductSaveDlg
    */
   private HotLinkLabel   hll_LEFT_SAVE                = null; //the HotLinkLabels will be used for the action buttons
   /**
    * This will be the ProductSaveDlg object to save the current Product
    */
   private HotLinkLabel   hll_RIGHT_SAVE               = null;  //the HotLinkLabels will be used for the action buttons
   /**
    * This will be the ActionButton to close the current Applet/Panel
    */
   private ActionButton   pb_CLOSE                     = null;
   /**
    * This is used when the Product is being saved
    */
   private ProductSaveDlg         saveDialog           = null;
   /**
    * This is where the ProductCompare came from.
    */
   private QuestPanel             previousPanel        = null;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This will be the variable to check it the object
    * is initialized
    */
   private boolean                     initialized         = false;  // If TRUE, we have initialized Ok
   /**
    * This will be the variable to check if the left Product is a Draft
    */
   private boolean                     leftDraft           = false;
   /**
    * This will be the variable to check if the left Product is a Active
    */
   private boolean                     leftActive          = false;
   /**
    * This will be the variable to check if the right Product is a Draft
    */
   private boolean                     rightDraft          = false;
   /**
    * This will be the variable to check if the right Product is a Active
    */
   private boolean                     rightActive         = false;
   /**
    * This will be the variable to determine what ChangeProduct
    * HotLinkLabel was pushed
    */
   private boolean                     leftChangeProduct   = false;
   /**
    * This will be the variable to determine what ChangeProduct
    * HotLinkLabel was pushed
    */
   private boolean                     rightChangeProduct  = false;
   /**
    * This is the variable to handle a returnCode from a
    * save for a product
    */
   private int                         saveType            = 0;
   /**
    * This is the variable to handle a returnCode from a
    * readFromDatabase for a product
    */
   private int                         returnCode          = 0;
   /**
    * This will be the left JPanel if there is a left Product
    */
   private ProductCompareEditPanel    leftEditPanel        = null ;
   /**
    * This will be the left JPanel if there is a right Product
    */
   private ProductCompareEditPanel    rightEditPanel       = null ;
   /**
    * This will be the object used to compare records
    */
   private ProductComparePOMRecCompare recCompare          = null ;
   /**
    * This will be the left Product object if present
    */
   private Product                     leftProduct         = null ;
   /**
    * This will be the right Product object if present
    */
   private Product                     rightProduct        = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    * This constructor like all good Quest
    * Applets does nothing
    *
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public ProductComparePanel() {
      leftProduct  = null;
      rightProduct = null;
   }/*constructor*/

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    * This constructor like all good Quest
    * Applets does nothing
    *
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public ProductComparePanel( QuestPanel previousPanel ) {
      leftProduct  = null;
      rightProduct = null;
      this.previousPanel = previousPanel;
   }/*constructor*/

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    * This constructor will be used if you select
    * Product Compare and pass it one current Product Object
    * .
    *
    * @param Product
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public ProductComparePanel( Product leftProduct, QuestPanel previousPanel ) {

      if ( leftProduct instanceof ProductDraft ) {
         if ( !findActive( (ProductDraft)leftProduct ) )
            rightProduct = null;
      }

      this.leftProduct   = leftProduct;
      this.previousPanel = previousPanel;
   }/*constructor*/

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    * This constructor will be used if you pass it two product objects
    * that you need to use to create the Panel with a Draft and Active
    * Product.
    *
    * @param Product
    * @param Product
    * @param none
    *
    * @return nothing
    *//*---------------------------------------------------------*/
   public ProductComparePanel( Product leftProduct, Product rightProduct, QuestPanel previousPanel ) {
      this.leftProduct   = leftProduct;
      this.rightProduct  = rightProduct;
      this.previousPanel = previousPanel;
   }/*constructor*/

   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will be called before we are shown and
    *  will close our panel if we return FALSE. If we
    *  return TRUE everything was initialized ok
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean initialize() {
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Controls                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.log ( 1, "Initializing the ProductComparePanel");

      pb_CLOSE                 = new ActionButton ( Str.getStr ( STR_CLOSE ),
                                                    ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),
                                                    Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
      leftPanel                = new JPanel( );
      rightPanel               = new JPanel( );

      leftListPanel            = new ListPanel( );
      rightListPanel           = new ListPanel( );

      hll_LEFT_TITLE           = new HotLinkLabel( Str.getStr( STR_INIT_TITLE ) );
      hll_RIGHT_TITLE          = new HotLinkLabel( Str.getStr( STR_INIT_TITLE ) );
      hll_LEFT_CHANGE_PRODUCT  = new HotLinkLabel( Str.getStr( STR_CHANGE_PRODUCT ) );
      hll_RIGHT_CHANGE_PRODUCT = new HotLinkLabel( Str.getStr( STR_CHANGE_PRODUCT ) );
      hll_LEFT_MOVE_CHANGES    = new HotLinkLabel( Str.getStr( STR_LEFT_MOVE_PRODUCT ) );
      hll_RIGHT_MOVE_CHANGES   = new HotLinkLabel( Str.getStr( STR_RIGHT_MOVE_PRODUCT ) );
      hll_LEFT_SAVE            = new HotLinkLabel( Str.getStr( STR_SAVE ) );
      hll_RIGHT_SAVE           = new HotLinkLabel( Str.getStr( STR_SAVE ) );

      leftEditPanel            = new ProductCompareEditPanel( );
      rightEditPanel           = new ProductCompareEditPanel( );

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Preparing Controls for the QuestPanel                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.log ( 1, "Setting up controls on ProductComparePanel");
      leftListPanel.add( hll_LEFT_TITLE );
      leftListPanel.add( hll_LEFT_CHANGE_PRODUCT );
      leftListPanel.add( hll_LEFT_MOVE_CHANGES );
      leftListPanel.add( hll_LEFT_SAVE );

      rightListPanel.add( hll_RIGHT_TITLE );
      rightListPanel.add( hll_RIGHT_CHANGE_PRODUCT );
      rightListPanel.add( hll_RIGHT_MOVE_CHANGES );
      rightListPanel.add( hll_RIGHT_SAVE );

      leftListPanel.setBackground( new Color ( 240, 240, 240 ) );
      rightListPanel.setBackground( new Color ( 240, 240, 240 ) );     
      leftListPanel.doLayout();
      rightListPanel.doLayout();

      leftPanel.setLayout(  new BorderLayout() );
      rightPanel.setLayout( new BorderLayout() );
      leftPanel.add( leftEditPanel.getFormPanel(), BorderLayout.CENTER );
      rightPanel.add( rightEditPanel.getFormPanel(), BorderLayout.CENTER );


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Controls to the Quest Panel                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.log ( 1, "Adding Components to the ProductComparePanel");

      setLayout( null ) ;
      setBackground( Color.white );

      add( leftListPanel );
      add( rightListPanel );
      add( leftScrollPanel = new JScrollPane ( leftPanel ) ) ;
      add( rightScrollPanel = new JScrollPane ( rightPanel ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust Properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      leftScrollPanel.getVerticalScrollBar().setUnitIncrement ( 18 ) ;
      rightScrollPanel.getVerticalScrollBar().setUnitIncrement ( 18 ) ;
      hll_LEFT_TITLE.setVisible( false );
      hll_LEFT_TITLE.setEnabled( false );
      hll_LEFT_TITLE.showSelection( false );
      hll_RIGHT_TITLE.setVisible( false );
      hll_RIGHT_TITLE.setEnabled( false );
      hll_RIGHT_TITLE.showSelection( false );
      hll_LEFT_CHANGE_PRODUCT.setEnabled( true );
      hll_LEFT_CHANGE_PRODUCT.showSelection( false );
      hll_RIGHT_CHANGE_PRODUCT.setEnabled( true );
      hll_RIGHT_CHANGE_PRODUCT.showSelection( false );
      hll_LEFT_MOVE_CHANGES.setVisible( false );
      hll_LEFT_MOVE_CHANGES.showSelection( false );   
      hll_RIGHT_MOVE_CHANGES.setVisible( false );
      hll_RIGHT_MOVE_CHANGES.showSelection( false );  
      hll_LEFT_SAVE.setVisible( false );
      hll_LEFT_SAVE.showSelection( false );           
      hll_RIGHT_SAVE.setVisible( false );
      hll_RIGHT_SAVE.showSelection( false );

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Listeners                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.log ( 1, "Adding Listeners to ProductComparePanel");
      pb_CLOSE.addActionListener                           ( this );
      hll_LEFT_CHANGE_PRODUCT.addActionListener            ( this );
      hll_RIGHT_CHANGE_PRODUCT.addActionListener           ( this );
      hll_LEFT_MOVE_CHANGES.addActionListener              ( this );
      hll_RIGHT_MOVE_CHANGES.addActionListener             ( this );
      hll_LEFT_SAVE.addActionListener                      ( this );
      hll_RIGHT_SAVE.addActionListener                     ( this );
      leftEditPanel.getPanel().addProductEditPanelListener ( this );
      leftScrollPanel.getVerticalScrollBar().addAdjustmentListener     ( this ) ;
      leftScrollPanel.getHorizontalScrollBar().addAdjustmentListener   ( this ) ;
      rightScrollPanel.getVerticalScrollBar().addAdjustmentListener    ( this ) ;
      rightScrollPanel.getHorizontalScrollBar().addAdjustmentListener  ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setProperties();


      return( true );
   }/* initialize */

   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    * This method will lay out and position all of the components
    *
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      LogSystem.log ( 1, "doLayout: ProductComparePanel");
      Dimension size       = getSize() ;
      int       listHeight  = (int)leftListPanel.getPreferredSize( ).getHeight();
      int       halfWidth  = size.width/2 ;

      leftListPanel.setBounds   ( 0, 0, halfWidth-2, listHeight );
      rightListPanel.setBounds  ( halfWidth+2, 0, halfWidth-2, listHeight );

      leftListPanel.setPreferredSize ( new Dimension ( halfWidth-2, GUISystem.getRowHeight()*4) );
      rightListPanel.setPreferredSize ( new Dimension ( halfWidth-2, GUISystem.getRowHeight()*4) );

      leftScrollPanel.setBounds ( 0, listHeight, halfWidth-2, size.height - listHeight ) ;
      rightScrollPanel.setBounds( halfWidth+2, listHeight, halfWidth-2, size.height - listHeight );
   }/*doLayout*/

   /*------------------------------------------------------------*/
   /* adjustmentValueChanged
    */
   /**
    * This method will be called when the scroll panes are scrolled
    *
    * @param AdjustmentEvent
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public void adjustmentValueChanged ( AdjustmentEvent e ) {

      if ( e.getSource() == leftScrollPanel.getVerticalScrollBar() )
         rightScrollPanel.getVerticalScrollBar().setValue ( leftScrollPanel.getVerticalScrollBar().getValue() ) ;

      else if ( e.getSource() == rightScrollPanel.getVerticalScrollBar() )
         leftScrollPanel.getVerticalScrollBar().setValue ( rightScrollPanel.getVerticalScrollBar().getValue() ) ;

      else if ( e.getSource() == leftScrollPanel.getHorizontalScrollBar() )
         rightScrollPanel.getHorizontalScrollBar().setValue ( leftScrollPanel.getHorizontalScrollBar().getValue() ) ;

      else if ( e.getSource() == rightScrollPanel.getHorizontalScrollBar() )
         leftScrollPanel.getHorizontalScrollBar().setValue ( rightScrollPanel.getHorizontalScrollBar().getValue() ) ;
   }/*adjustmentValueChanged*/

   /*------------------------------------------------------------*/
   /* changeHotLinkLabelStatus
    */
   /**
    *    This method will change the status of the buttons to
    *    either enabled/disabled or will change the text on the
    *    buttons;
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void changeHotLinkLabelStatus() {
       LogSystem.beginLogSection( "ChangingHotLinkLabelStatus");

      if ( leftActive && rightActive ) {
         hll_LEFT_TITLE.setText( Str.getStr( STR_ACTIVE ) );
         hll_LEFT_TITLE.setVisible( true );
         hll_RIGHT_TITLE.setText ( Str.getStr( STR_ACTIVE ) );
         hll_RIGHT_TITLE.setVisible( true );
         hll_LEFT_MOVE_CHANGES.setVisible( false );
         hll_LEFT_MOVE_CHANGES.setEnabled( false );
         hll_RIGHT_MOVE_CHANGES.setVisible( false );
         hll_RIGHT_MOVE_CHANGES.setEnabled( false );
         hll_LEFT_SAVE.setVisible( false );
         hll_LEFT_SAVE.setEnabled( false );
         hll_RIGHT_SAVE.setVisible( false );
         hll_RIGHT_SAVE.setEnabled( false );
         LogSystem.log( 1, "Both Active");

      } else if ( leftDraft && rightDraft ) {
         hll_LEFT_TITLE.setText( Str.getStr( STR_DRAFT ) );
         hll_LEFT_TITLE.setVisible( true );
         hll_RIGHT_TITLE.setText( Str.getStr( STR_DRAFT ) );
         hll_RIGHT_TITLE.setVisible( true );      
         
         if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) {                                
             hll_LEFT_SAVE.setVisible( true );             
             hll_LEFT_SAVE.setEnabled( true );
             hll_RIGHT_SAVE.setVisible( true );
             hll_RIGHT_SAVE.setEnabled( true );
             hll_LEFT_MOVE_CHANGES.setVisible( true );
             hll_LEFT_MOVE_CHANGES.setEnabled( true );
             hll_RIGHT_MOVE_CHANGES.setVisible( true );
             hll_RIGHT_MOVE_CHANGES.setEnabled( true );
         }
         else{
             hll_LEFT_SAVE.setVisible( false );
             hll_LEFT_SAVE.setEnabled( false );
             hll_RIGHT_SAVE.setVisible( false );
             hll_RIGHT_SAVE.setEnabled( false );
             hll_LEFT_MOVE_CHANGES.setVisible( false );
             hll_LEFT_MOVE_CHANGES.setEnabled( false );
             hll_RIGHT_MOVE_CHANGES.setVisible( false );
             hll_RIGHT_MOVE_CHANGES.setEnabled( false );
         }

         LogSystem.log( 1, "Both Draft");

      } else if ( leftActive && rightDraft ) {
         hll_LEFT_TITLE.setText( Str.getStr( STR_ACTIVE ) );
         hll_LEFT_TITLE.setVisible( true );
         hll_RIGHT_TITLE.setText( Str.getStr( STR_DRAFT ) );
         hll_RIGHT_TITLE.setVisible( true );  
         hll_RIGHT_MOVE_CHANGES.setVisible( false );
         hll_RIGHT_MOVE_CHANGES.setEnabled( false ); 
         hll_LEFT_SAVE.setVisible( false );
         hll_LEFT_SAVE.setEnabled( false );

         if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) {
             hll_RIGHT_SAVE.setVisible( true );
             hll_RIGHT_SAVE.setEnabled( true ); 
             hll_LEFT_MOVE_CHANGES.setVisible( true );
             hll_LEFT_MOVE_CHANGES.setEnabled( true );
         }
         else{
             hll_RIGHT_SAVE.setVisible( false );    
             hll_RIGHT_SAVE.setEnabled( false );
             hll_LEFT_MOVE_CHANGES.setVisible( false );
             hll_LEFT_MOVE_CHANGES.setEnabled( false );
         }
         
         LogSystem.log( 1, "Active/Draft");

      }else if ( leftDraft && rightActive ) {
         hll_LEFT_TITLE.setText( Str.getStr( STR_DRAFT ) );
         hll_LEFT_TITLE.setVisible( true );
         hll_RIGHT_TITLE.setText ( Str.getStr( STR_ACTIVE ) );
         hll_RIGHT_TITLE.setVisible( true );
         hll_LEFT_MOVE_CHANGES.setVisible( false );
         hll_LEFT_MOVE_CHANGES.setEnabled( false );
         
         if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ){
             hll_LEFT_SAVE.setVisible( true );
             hll_LEFT_SAVE.setEnabled( true ); 
             hll_RIGHT_MOVE_CHANGES.setVisible( true );
             hll_RIGHT_MOVE_CHANGES.setEnabled( true );
         }
         else{ 
             hll_LEFT_SAVE.setVisible( false );
             hll_RIGHT_MOVE_CHANGES.setVisible( false );
             hll_RIGHT_MOVE_CHANGES.setEnabled( false );
         }

         hll_RIGHT_SAVE.setVisible( false );
         hll_RIGHT_SAVE.setEnabled( false );
         LogSystem.log( 1, "Draft/Active");

      }else if ( leftActive && !rightActive && !rightDraft ) {
         hll_LEFT_TITLE.setText( Str.getStr( STR_ACTIVE ) ); 
         hll_LEFT_TITLE.setVisible( true );
         hll_LEFT_MOVE_CHANGES.setVisible( false );
         hll_LEFT_SAVE.setVisible( false );
         LogSystem.log( 1, "Just Left Active");

      } else if ( leftDraft && !rightActive && !rightDraft ) {
         hll_LEFT_TITLE.setText( Str.getStr( STR_DRAFT ) );
         hll_LEFT_TITLE.setVisible( true );
         hll_LEFT_MOVE_CHANGES.setVisible( false );
         hll_LEFT_MOVE_CHANGES.setEnabled( false );
         
         if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) {
             hll_LEFT_SAVE.setVisible( true );
             hll_LEFT_SAVE.setEnabled( true );
         }
         else{
             hll_LEFT_SAVE.setVisible( false );
             hll_LEFT_SAVE.setEnabled( false );
         }
         
         LogSystem.log( 1, "Just Left Draft");

      } else if ( rightActive && !leftActive && !leftDraft ) {
         hll_RIGHT_TITLE.setText ( Str.getStr( STR_ACTIVE ) );
         hll_RIGHT_TITLE.setVisible( true );
         hll_RIGHT_MOVE_CHANGES.setVisible( false );
         hll_RIGHT_MOVE_CHANGES.setEnabled( false );
         hll_RIGHT_SAVE.setVisible( false );
         hll_RIGHT_SAVE.setEnabled( false );
         LogSystem.log( 1, "Just Right Active");

      } else if ( rightDraft && !leftActive && !leftDraft ) {
         hll_RIGHT_TITLE.setText ( Str.getStr( STR_DRAFT ) );
         hll_RIGHT_TITLE.setVisible( true );
         hll_RIGHT_MOVE_CHANGES.setVisible( false );

         if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) {
             hll_RIGHT_SAVE.setVisible( true );
             hll_RIGHT_SAVE.setEnabled( true );
         }
         else{
             hll_RIGHT_SAVE.setVisible( false );
             hll_RIGHT_SAVE.setEnabled( false );
         }
         
         LogSystem.log( 1, "Just Left Draft");
      }/*if*/

      LogSystem.endLogSection();
   }/*changeHotLinkLabelStatus*/

   /*------------------------------------------------------------*/
   /* findProductStatus
    */
   /**
    * This method will find the product status of each product
    * that is currently being used
    *
    * @param none
    * @param none
    * @param none
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   private void findProductStatus( ) {

      LogSystem.log( 1, "Finding Product Status: Draft or Active" );

      leftDraft    = false;
      leftActive   = false;
      rightDraft   = false;
      rightActive  = false;


      if ( leftProduct != null ) {
         if ( leftProduct instanceof ProductDraft )
            leftDraft  = true;
         else if ( leftProduct instanceof Product )
            leftActive   = true;
      }/*if*/

      if ( rightProduct != null ) {
        if ( rightProduct instanceof ProductDraft )
            rightDraft  = true;
        else if ( rightProduct instanceof Product )
            rightActive = true;
     }/*if*/
   }/*findProductStatus*/

   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are initialized
    *  and we are showing
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {

      AppDefaultWin parentWin = GUISystem.getParentDefWin( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent( this, pb_CLOSE ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the toolbar on                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //parentWin.addToolbarButton( toolBar, 1 ) ;

      if ( leftProduct != null ) {
         leftEditPanel.setProduct ( leftProduct ) ;
      }
      if ( rightProduct != null ) {
         rightEditPanel.setProduct ( rightProduct );
      }
   }/*start*/

   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the user
    *  has shutdown the application. A previous call
    *  to stop was already made so we do nothing here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean shutdown () {
      return( Qit.getEnabled () ) ;
   }/*shutdown*/

   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the Quest
    *  application asks us to close ourselves. we
    *  will return true
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return boolean
    *//*---------------------------------------------------------*/
   public boolean stop () {
      panelDeselected() ;
      removeAll() ;
      return( Qit.getEnabled() ) ;
   }/*stop*/

   /*-----------------------------------------------------------*/
   /* findProductComplete
    */
   /**
    *   This method will be called when the user hits
    *  'Find' on the ProductSearch window.
    *
    *  @param none
    *  @param none
    *  @param none
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void findProductComplete( Vector vec ) {

      LogSystem.beginLogSection( "FindProductComplete" );
      
      if ( ( vec != null ) && ( vec.size() > 0 ) ) {
          ProductRow productRow = ( (ProductRow)vec.elementAt(0) );
          int returnCode = productRow.readProduct( true );

          if ( returnCode == 0 ) {
            if ( productRow.isDraft() ) {
                ProductDraft pd    = ( (ProductDraft)productRow.getProduct() );
                if ( leftChangeProduct ) {
                    setLeftProduct ( pd ) ;
                } else if ( rightChangeProduct ) {
                    setRightProduct ( pd ) ;
                }/*if ChangeProduct*/

            } else if ( !productRow.isDraft() ) {
                Product p    = productRow.getProduct();

                if ( leftChangeProduct ) {
                    setLeftProduct ( p ) ;
                } else if ( rightChangeProduct ) {
                    setRightProduct ( p ) ;
                }/*if changeProduct*/
            }/*if draft*/
          }/*if returncode*/
      }/*if vec*/

      refresh();
      LogSystem.endLogSection();
   }/* findProductComplete */

   /*-----------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread performs many operations. The operation
    *  done is determined by the thread name
    *
    *
    *  @param none
    *  @param none
    *  @param none
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run ( ) {

      String operation = Thread.currentThread().getName() ;

      Qit.setEnabled ( false ) ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   THREAD_REFRESH_PREVIEW                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( operation.equals ( THREAD_REFRESH_PREVIEW ) )
            refreshPreview() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }
      Qit.setEnabled ( true ) ;
   }/* run */

   /*------------------------------------------------------------*/
   /* refreshPreview
    */
   /**
    *    This method will refresh data for the both products
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return void
    *//*---------------------------------------------------------*/
   private void refreshPreview () {
      if ( leftProduct != null )
        leftEditPanel.setProduct( getLeftProduct() );
      if ( rightProduct != null )
        rightEditPanel.setProduct( getRightProduct() );
      setProperties();
   }/*refreshPreview*/

   /*-----------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called if the user hits the
    *  refresh button on the task bar. It will simply
    *  call refreshList
    *
    *  @param none
    *  @param none
    *  @param none
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh ( ) {
      new Thread ( this, THREAD_REFRESH_PREVIEW ).start() ;
   }/* refresh */

   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the
    *  user has selected our panel from a
    *  tabbed pane. If we are in MDI mode, this method is never called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
   }/*panelSelected*/

   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel onto another. This method
    *  will never be called in the MDI mode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
   }/*panelDeselected*/

   /*------------------------------------------------------------*/
   /* setProperties
    */
   /**
    *    This method will set properties for the current set
    *    up with Products and Product drafts.  Sets any variables
    *    that might be used to determine what functionality is
    *    needed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setProperties() {

      LogSystem.log( 1, "Setting Properties: ProductComparePanel " );

      findProductStatus();        //finds whether the product is active or Draft
      changeHotLinkLabelStatus(); //adjusts the HotLinkLabel status
      revalidate();
      
      leftChangeProduct  = false; //initializing so that we can start fresh when the user clicks change product(see ActionPerformed)
      rightChangeProduct = false; //initializing so that we can start fresh when the user clicks change product(see ActionPerformed)

   }/*setProperties*/

   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method will return our icon that represents
    *  this panel
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {
      return( ImageSystem.getImageIcon( this, ImageSystem.PRODUCT_COMPARE ) );
   }/*getIcon*/

   /*------------------------------------------------------------*/
   /* getSmallIcon
    */
   /**
    * This method will return the small icon representing
    * this Quest Applet
    *
    * @param none
    * @param none
    * @param none
    *
    * @return ImageIcon
    *
    *//*---------------------------------------------------------*/
   public ImageIcon getSmallIcon() {
      return( ImageSystem.getImageIcon( this, ImageSystem.PRODUCT_COMPARE ) );
   }/*getSmallIcon*/

   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default navigation
    *  panel
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NavPanel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( null );
   }/*getDefaultNavPanel*/

   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return our name
    *
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr( STR_PRODUCT_COMPARE ) );
   }/*getName*/

   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this );
   }/*getJComponent*/

   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action event
    *  is fired on us
    *
    *
    *  @param ActionEvent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( ( Qit.getEnabled() ) && ( e.getSource() instanceof HotLinkLabel ) ) {
         HotLinkLabel label = (HotLinkLabel) e.getSource() ;
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   hll_LEFT_CHANGE_PRODUCT                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( label == hll_LEFT_CHANGE_PRODUCT ) {
            leftChangeProduct = true;
            findProductComplete( new ProductSearchDlg().getResult() );
            //new ProductSearchDlg().addFindProductListener( this );
         }
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   hll_RIGHT_CHANGE_PRODUCT                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( label == hll_RIGHT_CHANGE_PRODUCT ) {
            rightChangeProduct = true;
            findProductComplete( new ProductSearchDlg().getResult() );
           // new ProductSearchDlg().addFindProductListener( this );
         }
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   hll_LEFT_MOVE_CHANGES                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( label == hll_LEFT_MOVE_CHANGES ) {
            moveChanges( left );
            rightEditPanel.setProduct( rightProduct );
         }
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   hll_RIGHT_MOVE_CHANGES                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( label == hll_RIGHT_MOVE_CHANGES ) {
            moveChanges( right );
            leftEditPanel.setProduct( leftProduct );
         }                             
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   hll_LEFT_SAVE                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( label == hll_LEFT_SAVE ) 
             saveType = leftEditPanel.saveProduct();
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   hll_RIGHT_SAVE                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( label == hll_RIGHT_SAVE ) 
             saveType = rightEditPanel.saveProduct();
      }/*if HotLinkLabel*/

      if ( ( Qit.getEnabled() ) && ( e.getSource() instanceof DButton ) ) {
         DButton button = (DButton)e.getSource();
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CLOSE                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( button == pb_CLOSE )
            GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;
      }/*if DButton*/
   }/*actionPerformed*/

   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button.
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark() {
      return( false ) ;
   }/*bookmark*/

   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    * This method will be called when the user wishes to
    * 'Copy As Text' from this panel
    *
    * @param none
    * @param none
    * @param none
    *
    * @return TRUE = Action Performed
    *         FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean copyText() {
      return( false ) ;
   }/*copyText*/

   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    * This method will be called when the user wishes to
    * 'Cut As Text' from this panel
    *
    * @param none
    * @param none
    * @param none
    *
    * @return TRUE = Action Performed
    *         FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean cutText() {
      return( false ) ;
   }/*cutText*/

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    * This method will be called when the user wishes to
    * 'Paste As Text' from this panel
    *
    * @param none
    * @param none
    * @param none
    *
    * @return TRUE = Action Performed
    *         FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean pasteText() {
      return( false ) ;
   }/*pasteText*/

   /*------------------------------------------------------------*/
   /* setRightProduct
    */
   /**
    * This method will set the Right Product to be compared to
    *
    * @param Product
    * @param none
    * @param none
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public void setRightProduct( Product rightProduct ) {
      this.rightProduct = rightProduct;
   }/*setRightProduct*/

   /*------------------------------------------------------------*/
   /* setLeftProduct
    */
   /**
    * This method will set the Left Product to be compared to
    *
    * @param Product
    * @param none
    * @param none
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public void setLeftProduct( Product leftProduct ) {
      this.leftProduct = leftProduct;
   }/*setLeftProduct*/

   /*------------------------------------------------------------*/
   /* getRightProduct
    */
   /**
    * This method will get the Right Product to be compared to
    *
    * @param Product
    * @param none
    * @param none
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public Product getRightProduct( ) {
      return( rightProduct );
   }/*getRightProduct*/

   /*------------------------------------------------------------*/
   /* getLeftProduct
    */
   /**
    * This method will get the Left Product to be compared to
    *
    * @param Product
    * @param none
    * @param none
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public Product getLeftProduct( ) {
      return( leftProduct );
   }/*getLeftProduct*/


   /*------------------------------------------------------------*/
   /* findActive
    */
   /**
    * This method will find whether or not there is an active
    * product for the draft that has been passed in.
    *
    * @param Product
    * @param none
    * @param none
    *
    * @return boolean
    *//*---------------------------------------------------------*/
   private boolean findActive( ProductDraft productDraft ) {

      GUISystem.getParentDefWin ( this ).setStatus ( STR_FINDING_ACTIVE ) ;
      int existingProductInd = 0;

      if ( productDraft != null ) {
         existingProductInd = productDraft.getExistingProductInd();
         if ( existingProductInd != 0 ) {
            LogSystem.beginLogSection ( "Searching for an Active Product....." ) ;
            Product product = new Product( existingProductInd );
            returnCode = product.readFromDatabase();
            switch ( returnCode ) {
               case 0 :
                  LogSystem.log ( 1, "Found Active Product" ) ;
                  this.rightProduct = product;
                  break ;
               case 100 :
                  LogSystem.log ( 1, "Could not find the Active Product" ) ;
                  break ;
               default :
                  LogSystem.log ( 1, "Return Code: "+ returnCode ) ;
                  break ;
            }/*switch*/
         }/*if*/

         LogSystem.endLogSection() ;

         if ( returnCode != 0 ) {
            GUISystem.getParentDefWin ( this ).setStatus ( STR_DID_NOT_FIND_ACTIVE ) ;
            return( false );
         } else {
            GUISystem.getParentDefWin ( this ).setStatus ( STR_FOUND_ACTIVE ) ;
            return( true );
         }

      } else
         return( false );


   }/*findActive*/


   /*------------------------------------------------------------*/
   /* moveChanges
    */
   /**
    * This method will move the changes from the product selected
    * to the other product
    *
    * @param String
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   private void moveChanges( String location ) {

      LogSystem.log( 1, "MoveChanges: ProductComparePanel" );

      if ( location.equals( left ) ) {
         rightProduct = recCompare.leftMoveChanges();
         hll_LEFT_MOVE_CHANGES.setVisible( false );
         hll_LEFT_MOVE_CHANGES.setEnabled( false );
         hll_RIGHT_MOVE_CHANGES.setVisible( false );
         hll_RIGHT_MOVE_CHANGES.setEnabled( false ); 
         setRightProduct( rightProduct );
      }/*if*/

      if ( location.equals( right ) ) {
         leftProduct = recCompare.rightMoveChanges();
         hll_LEFT_MOVE_CHANGES.setVisible( false );
         hll_LEFT_MOVE_CHANGES.setEnabled( false );
         hll_RIGHT_MOVE_CHANGES.setVisible( false );
         hll_RIGHT_MOVE_CHANGES.setEnabled( false );
         setLeftProduct( leftProduct );
      }/*if*/
      revalidate();
   }/*moveChanges*/

   /*------------------------------------------------------------*/
   /* recordCompare
    */
   /**
    * This method will check the public variables of
    * ProductComparePOMRecCompare and change the appropriate JLabel
    * from ProductCompareEditPanel
    *
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   private void recordCompare() {

      LogSystem.beginLogSection("ProductComparePanel - recordCompare");

      recCompare = new ProductComparePOMRecCompare( leftProduct, rightProduct );

      if ( !recCompare.PROD_NUM )
         leftEditPanel.setChangedLabel( leftEditPanel.PROD_NUM );
      if ( !recCompare.BASE_MODEL )
         leftEditPanel.setChangedLabel( leftEditPanel.BASE_MODEL );
      if ( !recCompare.BRAND )
         leftEditPanel.setChangedLabel( leftEditPanel.BRAND );
      if ( !recCompare.CREATED )
         leftEditPanel.setChangedLabel( leftEditPanel.CREATED );
      if ( !recCompare.CREATED_BY )
         leftEditPanel.setChangedLabel( leftEditPanel.CREATED_BY );
      if ( !recCompare.CUSTOMER )
         leftEditPanel.setChangedLabel( leftEditPanel.CUSTOMER );
      if ( !recCompare.DATES_FOR_COUNTRY )
         leftEditPanel.setChangedLabel( leftEditPanel.DATES_FOR_COUNTRY );
      if ( !recCompare.FAMILY )
         leftEditPanel.setChangedLabel( leftEditPanel.FAMILY );
      if ( !recCompare.FAMILYTYPE )
         leftEditPanel.setChangedLabel( leftEditPanel.FAMILYTYPE );
      if ( !recCompare.INPUT )
         leftEditPanel.setChangedLabel( leftEditPanel.INPUT );
      if ( !recCompare.LAST_TOUCHED_BY )
         leftEditPanel.setChangedLabel( leftEditPanel.LAST_TOUCHED_BY );
      if ( !recCompare.MACHINE )
         leftEditPanel.setChangedLabel( leftEditPanel.MACHINE );
      if ( !recCompare.MODEL )
         leftEditPanel.setChangedLabel( leftEditPanel.MODEL );
      if ( !recCompare.OFFERING )
         leftEditPanel.setChangedLabel( leftEditPanel.OFFERING );
      if ( !recCompare.OPICM_DESCRIPT )
         leftEditPanel.setChangedLabel( leftEditPanel.OPICM_DESCRIPT );
      if ( !recCompare.PARTNUM )
         leftEditPanel.setChangedLabel( leftEditPanel.PARTNUM );
      if ( !recCompare.PRODUCT_IND )
         leftEditPanel.setChangedLabel( leftEditPanel.PRODUCT_IND );
      if ( !recCompare.PROJECTNAME )
         leftEditPanel.setChangedLabel( leftEditPanel.PROJECTNAME );
      if ( !recCompare.QUEST_DESCRIPT )
         leftEditPanel.setChangedLabel( leftEditPanel.QUEST_DESCRIPT );
      if ( !recCompare.SOURCE )
         leftEditPanel.setChangedLabel( leftEditPanel.SOURCE );
      if ( !recCompare.WARRANTY )
         leftEditPanel.setChangedLabel( leftEditPanel.WARRANTY );
      if ( !recCompare.DRAFT_CREATED_ON )
         leftEditPanel.setChangedLabel( leftEditPanel.DRAFT_CREATED_ON );
      if ( !recCompare.PRODUCT_DRAFT_IND )
         leftEditPanel.setChangedLabel( leftEditPanel.PRODUCT_DRAFT_IND );
      if ( !recCompare.LAST_TOUCHED_ON )
         leftEditPanel.setChangedLabel( leftEditPanel.LAST_TOUCHED_ON );      

      LogSystem.log( 1, "finished compare");
      LogSystem.endLogSection();
   }/*recordCompare*/

   /*------------------------------------------------------------*/
   /* refreshComplete
    */
   /**
    * This method will be called when the Thread in the
    * ProductEditPanel is done loading all the Panels and Labels.
    * It will allow us to then compare and change the labels.
    *
    * @param None
    * @param None
    * @param None
    *
    * @return Nothing
    *//*---------------------------------------------------------*/
   public void refreshComplete(){
       LogSystem.log( 1, "RefreshComplete: ProductComparePanel");
       if ( ( leftProduct != null ) && ( rightProduct!= null ) )
           recordCompare();
   }/*refreshComplete*/

   /*------------------------------------------------------------*/
   /* getHelpFilename
    */
   /**
    *    This method will return our help filename
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getHelpFilename () {
      return( "Products/ProdCompare.html" ) ;
   }

}/* ProductComparePanel */
