package com.ibm.nzna.projects.qit.product.gui ;   
/*-----------------------------------------------------------*/
/*   ProductDlg                                              */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductDlg                          */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.productEdit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
/**
 * This simple dialog simply shows the user a list of products. The dialog
 * will be modal if the parent passed to the constructor is a JDialog. If the
 * parent is a JFrame then the dialog will not be modal and will have the 'Edit'
 * button.
 *
 *
 * @version 1.0, 8/6/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductDlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductDlg extends JDialog implements ActionListener, WindowListener, AppConst, ProductConst {

   /**
     *  This is the actual list that will show the products
     **/
   private ProductTable productTable = null ;

   /**
     *  This is the 'Ok' button
     **/
   private DButton pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     *  This is the Vector of products we will be showing
     **/
   private Vector products = null ;

   /**
     *  This button bar holds the Ok and Edit buttons
     **/
   ButtonPanel buttonBar = new ButtonPanel () ;     

   /**
     *  If our parent is an AppDefaultWin, this is that instance
     **/
   private AppDefaultWin parentWin = null ;

   /**
     *  This is the edit button. This button only shows if this
     * window is non-modal
     **/
   private DButton pb_EDIT = null ;


   /**
     *  This is the default constructor
     **/
   public ProductDlg ( Frame parent, Vector products ) {
      super ( parent, Str.getStr ( STR_PRODUCTS ), true ) ;

      this.products = products ;
      initialize() ;
      setVisible ( true ) ;
   }

   /**
     *  This is the constructor that makes a JDialog our parent
     **/
   public ProductDlg ( JDialog parent, Vector products ) {
      super ( parent, Str.getStr ( STR_PRODUCTS ), true ) ;

      this.products = products ;
      initialize() ;
      setVisible ( true ) ;
   }

   /**
     *  This is the constructor that makes a JFrame our parent and is
     * not modal
     **/
   public ProductDlg ( JFrame parent, Vector products ) {
      super ( parent, Str.getStr ( STR_DOCUMENTS ), false ) ;

      if ( parent instanceof AppDefaultWin )
         parentWin = (AppDefaultWin)parent ;

      this.products = products ;
      initialize() ;
      initializeEdit() ;
      setVisible ( true ) ;
   }


   /**
     *  This method will initialize our GUI and set our size/position
     **/
   private void initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      productTable = new ProductTable ( products ) ;              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      addWindowListener ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      buttonBar.add ( pb_OK ) ;
      getContentPane().add ( productTable ) ;
      getContentPane().add ( buttonBar, BorderLayout.SOUTH ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our size                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSizeFromPrevious() ;
      GUISystem.setPropertiesOnPanel ( getContentPane() );

   }


   /**
     *  This method will set our size from our previous size
     **/
   private void setSizeFromPrevious () {
      Rectangle oldSize = (Rectangle)PropertySystem.get ( com.ibm.nzna.projects.qit.doc.DocConst.PROPERTY_DOCUMENT_DLG_POSITION ) ;

      if ( oldSize == null ) {
         setSize ( 400, 300 ) ;
         WinUtil.centerWindow ( this ) ;

         oldSize = new Rectangle ( getLocation().x, getLocation().y, 400, 300 ) ;
      } else
         setBounds ( oldSize ) ;
   }


   /**
     * This method is called when the user hits the 'OK' button
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_OK ) {
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT ) {
         ProductRow productRow = (ProductRow) productTable.getSelectedItem() ;

         if ( productRow != null ) {
            if ( parentWin != null ) {
               parentWin.setStatus ( Str.getStr ( STR_READING )+" "+productRow.getDescription() ) ;

               EditProduct.editProduct ( productRow ) ;
            }
         } else
            GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_ITEM ) ;
      }

   }


   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e ) {
   }
   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowOpened ( WindowEvent e ) {
   }
   public void windowClosing ( WindowEvent e ) {
      PropertySystem.put ( com.ibm.nzna.projects.qit.doc.DocConst.PROPERTY_DOCUMENT_DLG_POSITION, getBounds() ) ;
   }
   public void windowClosed ( WindowEvent e ) {
   }

   public void doLayout () {
      Dimension size = getSize() ;

      super.doLayout() ;
      productTable.setBounds ( 5, 5, size.width-15, size.height-68 ) ;

      buttonBar.setBounds ( 5, size.height-56, size.width-15, 25 ) ;
   }

   /**
     *  this method will add the edit button to our window
     **/
   public void initializeEdit() {
      pb_EDIT = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT.addActionListener ( this ) ;
      buttonBar.add ( pb_EDIT ) ;
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
   }
} /* ProductDlg */
