package com.ibm.nzna.projects.qit.product.gui ;
/*-----------------------------------------------------------*/
/*   ProductSearchDlg                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductSearchDlg                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import java.awt.* ;
import java.io.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.border.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.common.storedProc.*;
import com.ibm.nzna.projects.qit.help.*;
import com.ibm.nzna.projects.common.quest.type.*;
import com.ibm.nzna.projects.common.quest.product.* ;



/**
 * This dialog allows the user to navigate through the database.
 * The results from the 'Find' button being hit will be passed back in a Vector
 * to each registered Listener. Each time the window disappears, all
 * listeners will be removed.<br>
 *
 *
 * @version 1.0, 5/13/2001
 * @author  Venu Sanka
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductSearchDlg                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductSearchDlg extends JDialog implements Runnable, AppConst,
ProductConst, ActionListener, ListSelectionListener, DataLengths {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   GUI Members                                             */
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
     * The Field To Search JLabel
     */
    private JLabel        st_FIELDTOSEARCH      = null ;

    /**
     * The Search For JLabel
     */
    private JLabel        st_SEARCHFOR   = null ;

    /**
     * The Geography JLabel
     */
    private JLabel        st_GEO         = null ;

    /**
     * The Country JLabel
     */
    private JLabel        st_COUNTRY     = null ;


    /**
     * The Search Entry Field
     */
    private JTextField    ef_SEARCH      = null ;

    /**
     * The Exact Match Check Box
     */
    private JCheckBox     ck_EXACTMATCH  = null ;

    /**
     * The Button Group for Search All and Search Selected Radio Buttons
     */
    private ButtonGroup   schGroup       = null ;

    /**
     * The Search All Radio Button
     */
    private JRadioButton  rb_SEARCHALL   = null ;

    /**
     * The Search Selected Radio Button
     */
    private JRadioButton  rb_SEARCHSEL   = null ;

    /**
     * The Button Panel
     */
    private ButtonPanel   buttonPanel    = null ;

    /**
     * The Find Button
     */
    private DButton       pb_FIND        = null ;

    /**
     * The Cancel Button
     */
    private DButton       pb_CANCEL      = null ;

    /**
     * The Help Button
     */
    private DButton       pb_HELP        = null ;

    /**
     * The Geography List Box
     */
    private JList         list_GEO       = null ;

    /**
     * The Country List Box
     */
    private JList         list_COUNTRY   = null ;

    /**
     * The JList for holding the search cirteria
     */
    private JList         list           = null ;

    /**
     * The Check box for selecting Product Requests
     */
    private JCheckBox            ck_REQUEST     = null ;

    private UserEntryPanel       pnl_USERENTRY  = null ;

    private WarrantyEntryPanel   pnl_WARRANTY   = null ;

    private OfferingEntryPanel   pnl_OFFERING   = null ;

    private CustomerEntryPanel   pnl_CUSTOMER   = null ;

    private String               user           = null ;

    /**
     *   Constants representing the field to search. Uses same ordering in getSearchTypes()
     */

    private static final int INDEX_PRODUCT_NUM   = 0;
    private static final int INDEX_PRODUCT_IND   = 1;
    private static final int INDEX_PART_NUM      = 2;
    private static final int INDEX_DESCRIPTION   = 3;
    private static final int INDEX_MACHINE       = 4;
    private static final int INDEX_MODEL         = 5;
    private static final int INDEX_USERID        = 6;
    private static final int INDEX_PROJECT       = 7;
    private static final int INDEX_BASE_MODEL    = 8;
    private static final int INDEX_CUSTOMER      = 9;
    private static final int INDEX_OFFERING      =10;
    private static final int INDEX_WARRANTY      =11;



    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private static   ProductSearchDlg instance      = null ;
    private Vector   listenerVec                    = new Vector ( 1, 1 ) ;
    private Vector   prodSelectVec                  = null ;
    private boolean  showRequests                   = true ;
    private Vector   foundVec                       = null ;


    private static final String THREAD_SEARCH_ALL = "SA" ;

    private static final String THREAD_SEARCH_SELECT = "SS" ;




    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This will create the default ProductSearchDlg
     *
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public ProductSearchDlg () {
        super ( new Frame(), Str.getStr ( STR_PRODUCTSEARCH ), true ) ;
        initialize() ;
    } /* Constructor */





    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This will create a ProductSearchDlg with
     *  a show of product requests.
     *
     *
     *  @param showRequests TRUE = allow user to see only product requests, false if otherwise
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public ProductSearchDlg ( boolean showRequests ) {
        super ( new Frame(), Str.getStr ( STR_PRODUCTSEARCH ), true ) ;
        this.showRequests = showRequests ;
        initialize() ;
    }


    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This will create a ProductSearchDlg with
     *  a selection of products passed as a parameter.
     *
     *
     *  @param prodVec   the vector of products to confine the search
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public ProductSearchDlg ( Vector prodSelectVec ) {
        super ( new Frame(), Str.getStr ( STR_PRODUCTSEARCH ), true ) ;
        this.prodSelectVec = prodSelectVec ;
        initialize() ;
    }




    /*------------------------------------------------------------*/
    /* initialize
     */
    /**
     *    This method will initialize our variables for
     *  our window before it is shown
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return true
     *//*---------------------------------------------------------*/
    public boolean initialize () {


        Vector listeners = new Vector ( 1 ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Set our Properties                                      */
        /*                                                           */
        /*-----------------------------------------------------------*/

        setResizable ( false ) ;
        setSize(533,260) ;
        WinUtil.centerWindow ( this ) ;
        createControls() ;
        //setVisible ( true ) ;
        instance = this ;
        listeners.addElement ( this ) ;

        return( true ) ;
    }








    /*------------------------------------------------------------*/
    /* createControls
     */
    /**
     *    This method will create and add all of our controls
     *
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/

    private void createControls () {

        Vector geo     =  (Vector)TypeList.getInstance().getTypeList( TypeList.TYPE_GEOGRAPHY).clone() ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create our GUI                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        pb_FIND         = new DButton      ( Str.getStr ( STR_FIND ) ) ;
        pb_CANCEL       = new DButton      ( Str.getStr ( STR_CANCEL ) ) ;
        pb_HELP         = new DButton      ( Str.getStr ( STR_HELP ) ) ;
        list                = new JList(getSearchTypes()) ;
        list.setSelectedIndex(0) ;
        st_FIELDTOSEARCH    = new JLabel(Str.getStr  ( STR_FIELDTOSEARCH) ) ;
        st_SEARCHFOR        = new JLabel(Str.getStr (STR_SEARCH_FOR) ) ;
        ef_SEARCH           = new JTextField("", 20);
        ck_EXACTMATCH       = new JCheckBox(Str.getStr (STR_EXACTMATCH)) ;
        schGroup            = new ButtonGroup() ;
        rb_SEARCHALL        = new JRadioButton (Str.getStr (STR_SEARCHALL)) ;
        rb_SEARCHSEL        = new JRadioButton(Str.getStr (STR_SEARCHSEL)) ;
        list_GEO            = new JList();
        list_COUNTRY        = new JList();
        st_GEO              = new JLabel(Str.getStr (STR_GEOGRAPHY));
        st_COUNTRY          = new JLabel(Str.getStr (STR_COUNTRY));
        buttonPanel         = new ButtonPanel() ;
        ck_REQUEST          = new JCheckBox ( Str.getStr ( STR_SHOW_ONLY_DRAFTS )) ;

        pnl_USERENTRY       = new UserEntryPanel() ;
        pnl_OFFERING        = new OfferingEntryPanel() ;
        pnl_WARRANTY        = new WarrantyEntryPanel() ;
        pnl_CUSTOMER        = new CustomerEntryPanel() ;





        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust our properties                                   */
        /*                                                           */
        /*-----------------------------------------------------------*/
        list_GEO.setModel( new DefaultListModel() ) ;
        list_COUNTRY.setModel( new DefaultListModel() ) ;
        setListData( list_GEO, getGeo() ) ;
        list_GEO.setSelectedIndex(0) ;
        setListData( list_COUNTRY, getCountry() ) ;
        list_COUNTRY.setSelectedIndex(0) ;
        GUISystem.setPreferredButton ( pb_FIND ) ;
        GUISystem.setPreferredButton ( pb_HELP ) ;
        GUISystem.setPreferredButton ( pb_CANCEL ) ;
        rb_SEARCHALL.setSelected( true ) ;
        if ( prodSelectVec != null )
            rb_SEARCHSEL.setEnabled( true ) ;
        else
            rb_SEARCHSEL.setEnabled( false) ;




        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our listeners                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        pb_FIND.addActionListener      ( this ) ;
        pb_HELP.addActionListener      ( this ) ;
        pb_CANCEL.addActionListener    ( this ) ;
        ef_SEARCH.addActionListener    ( this ) ;
        list.addListSelectionListener  ( this ) ;
        list_GEO.addListSelectionListener( this ) ;
        list_COUNTRY.addListSelectionListener( this ) ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our controls                                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        getContentPane().setLayout ( null ) ;
        JScrollPane pane = new JScrollPane(list);
        getContentPane().add(pane);
        pane.setBounds(5,24,200,160);
        JScrollPane geoPane = new JScrollPane(list_GEO) ;
        getContentPane().add(geoPane) ;
        JScrollPane countryPane = new JScrollPane(list_COUNTRY) ;
        getContentPane().add(countryPane) ;
        getContentPane().add(st_FIELDTOSEARCH);
        getContentPane().add(st_SEARCHFOR);
        getContentPane().add(ef_SEARCH);
        getContentPane().add(ck_EXACTMATCH);
        schGroup.add(rb_SEARCHALL);
        schGroup.add(rb_SEARCHSEL);
        getContentPane().add(rb_SEARCHALL);
        getContentPane().add(rb_SEARCHSEL);
        getContentPane().add(st_GEO);
        getContentPane().add(st_COUNTRY);
        getContentPane().add(ck_REQUEST) ;
        buttonPanel.add ( pb_FIND ) ;
        buttonPanel.add ( pb_CANCEL ) ;
        buttonPanel.add ( pb_HELP ) ;
        getContentPane().add(buttonPanel) ;
        getContentPane().add(pnl_USERENTRY ) ;
        getContentPane().add(pnl_OFFERING ) ;
        getContentPane().add(pnl_WARRANTY ) ;
        getContentPane().add(pnl_CUSTOMER ) ;



        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Hide some controls for now                              */
        /*                                                           */
        /*-----------------------------------------------------------*/
        pnl_USERENTRY.setVisible( false ) ;
        pnl_OFFERING.setVisible( false ) ;
        pnl_WARRANTY.setVisible( false ) ;
        pnl_CUSTOMER.setVisible( false ) ;






        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust the size/position of controls                    */
        /*                                                           */
        /*-----------------------------------------------------------*/


        int x = 5 ;
        int y = 5 ;
        int rowHeight = 19 ;
        Dimension size = getSize() ;

        st_FIELDTOSEARCH.setBounds( x, y, 200, rowHeight );
        st_SEARCHFOR.setBounds(x+210, y, 223, rowHeight);
        y+=rowHeight;

        pane.setBounds(x,y,200,160);
        ef_SEARCH.setBounds(x+210, y, 310, rowHeight);
        pnl_USERENTRY.setBounds( x+210, y, 310, rowHeight) ;
        pnl_OFFERING.setBounds( x+210, y, 310, rowHeight) ;
        pnl_WARRANTY.setBounds( x+210, y, 310, rowHeight) ;
        pnl_CUSTOMER.setBounds( x+210, y, 310, rowHeight) ;

        y+=rowHeight;


        ck_EXACTMATCH.setBounds(x+210, y, 100, rowHeight);
        ck_REQUEST.setBounds(x+310, y, 200, rowHeight) ;
        y+=rowHeight;

        rb_SEARCHALL.setBounds(x+210,y,223,rowHeight);
        y+=rowHeight;

        rb_SEARCHSEL.setBounds(x+210,y,223,rowHeight);


        st_GEO.setBounds(215, 102, 66, rowHeight);

        st_COUNTRY.setBounds(215, 150, 66, rowHeight);

        geoPane.setBounds(282,102,243,40);

        countryPane.setBounds(282,150,243,40);

        buttonPanel.setBounds(5, size.height-70, size.width-15, 25) ;

        GUISystem.setPropertiesOnPanel ( getContentPane() ) ;

    }




    /*------------------------------------------------------------*/
    /* getSearchTypes
     */
    /**
     *    This method will return all of the search types
     *
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private Vector getSearchTypes () {
        Vector vec = new Vector ( 1, 1 ) ;


        vec.addElement ( Str.getStr ( STR_PRODUCT_NUMBER ) ) ;
        vec.addElement ( Str.getStr ( STR_PRODUCT_IND ) ) ;
        vec.addElement ( Str.getStr ( STR_PARTNUM ) ) ;
        vec.addElement ( Str.getStr ( STR_DESCRIPTION ) ) ;
        vec.addElement ( Str.getStr ( STR_MACHINE ) ) ;
        vec.addElement ( Str.getStr ( STR_MODEL ) ) ;
        vec.addElement ( Str.getStr ( STR_USERID ) ) ;
        vec.addElement ( Str.getStr ( STR_PROJECTNAME ) ) ;
        vec.addElement ( Str.getStr ( STR_BASEMODEL ) ) ;
        vec.addElement ( Str.getStr ( STR_CUSTOMER ) ) ;
        vec.addElement ( Str.getStr ( STR_OFFERING ) ) ;
        vec.addElement ( Str.getStr ( STR_WARRANTY ) ) ;



        return( vec ) ;
    }


    /*------------------------------------------------------------*/
    /* getResult
     */
    /**
     *  This method will show the window, waits for
     *  the user to hit find, and then will
     *  return a vector of what was selected.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return NULL if the user cancelled
     *//*---------------------------------------------------------*/
    public Vector getResult() {
        setVisible ( true ) ;
        return( foundVec ) ;
    }




    /*------------------------------------------------------------*/
    /* actionPerformed
     */
    /**
     *    This method will be called when a button
     *  is selected
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void actionPerformed ( ActionEvent e  ) {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   pb_CANCEL                                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( e.getSource() == pb_CANCEL )
            setVisible ( false ) ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   PB_FIND or EF_SEARCH                                    */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( ( e.getSource() == pb_FIND ) ||
                  ( e.getSource() == ef_SEARCH ) ) {

            if ( rb_SEARCHALL.isSelected() ) {
                System.out.println("searchall is selected: "+rb_SEARCHALL.isSelected()) ;
                new Thread(this, THREAD_SEARCH_ALL ).start() ;
            }

            else if ( rb_SEARCHSEL.isSelected() ) {
                System.out.println("searchSEL is selected: "+rb_SEARCHSEL.isSelected()) ;
                new Thread(this, THREAD_SEARCH_SELECT ).start() ;

            }

        }




        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   pb_HELP                                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( e.getSource() == pb_HELP ) {
            HelpSystem.showHelp ( "Products/ProdSearch.html" ) ;

        }

    }

    /*------------------------------------------------------------*/
    /* valueChanged
     */
    /**
     *    This method is invoked when a selection occurs in the list
     *
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void valueChanged(ListSelectionEvent e) {
        if ( e.getSource() == list )
            refreshSearchFields() ;

        else if (e.getSource() == list_GEO)
            getCountryFromGeo();


    }

    /*------------------------------------------------------------*/
    /* refreshSearchFields
     */
    /**
     *    This method will refresh the search entry
     *  field with the correct formatting
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void refreshSearchFields () {

        String searchType  = (String) list.getSelectedValue() ;

        
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*  Part Number                                              */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if (  searchType.equals ( Str.getStr (STR_PARTNUM)) ) {
            ef_SEARCH.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;


            ef_SEARCH.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_PART_NUMBER ) ) ;

        }


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Product Number                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_PRODUCT_NUMBER ) ) ) {
            ef_SEARCH.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;


            ef_SEARCH.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_PRODNUM ) ) ;

        }





        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Internal Number                                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_PRODUCT_IND ) ) ) {
            ef_SEARCH.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;

            ef_SEARCH.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC ) ) ;
        }


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Model                                                   */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_MODEL ) ) ) {
            ef_SEARCH.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;

            ef_SEARCH.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, "VVV", ef_SEARCH ) ) ;
        }



        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   BaseModel                                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals (Str.getStr ( STR_BASEMODEL ) ) ) {
            ef_SEARCH.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;

            ef_SEARCH.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, "VVVVVVV", ef_SEARCH ) ) ;
        }



        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Machine                                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_MACHINE ) ) ) {
            ef_SEARCH.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;
            ef_SEARCH.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_STRING, "VVVV", ef_SEARCH ) ) ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Description                                             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_DESCRIPTION ) ) ) {
            ef_SEARCH.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;
            ef_SEARCH.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_PRODDESC ) ) ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   UserId                                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_USERID ) ) ) {


            ef_SEARCH.setVisible( false ) ;
            pnl_USERENTRY.setVisible( true ) ;
            pnl_WARRANTY.setVisible(false) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;

        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Offering                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_OFFERING ) ) ) {


            ef_SEARCH.setVisible( false ) ;
            pnl_OFFERING.setVisible( true ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;

        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Warranty                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_WARRANTY ) ) ) {


            ef_SEARCH.setVisible( false ) ;
            pnl_WARRANTY.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Customer                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_CUSTOMER ) ) ) {


            ef_SEARCH.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;


        }




        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Project Name                                            */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( searchType.equals ( Str.getStr ( STR_PROJECTNAME ) ) ) {
            ef_SEARCH.setVisible( true ) ;
            pnl_OFFERING.setVisible( false ) ;
            pnl_USERENTRY.setVisible( false ) ;
            pnl_WARRANTY.setVisible( false ) ;
            pnl_CUSTOMER.setVisible( false ) ;
            ef_SEARCH.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, LENGTH_PROJECT ) ) ;
        }

        ef_SEARCH.requestFocus() ;
    }

    /*------------------------------------------------------------*/
    /* searchFromSelection
     */
    /**
     *
     *    This method will search through the vector of products selected
     *  and try to find an item matching the description in the find field.
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/

    private void searchFromSelection () {


        ProductRow        productRow      = null ;
        int               returnCode      = 0    ;
        String            offering        = null ;
        String            customer        = null ;
        boolean           match           = false;

        String     searchArg        = ef_SEARCH.getText() ;
        if (searchArg != null)
            searchArg = searchArg.trim() ;
        System.out.println("searcharg1"+searchArg) ;
        String     warrArg          = pnl_WARRANTY.getDescript() ;
        if (warrArg != null)
            warrArg = warrArg.trim() ;
        String     userArg          = pnl_USERENTRY.getUserId() ;
        if (userArg != null)
            userArg = userArg.trim() ;
        String     offeringArg         = pnl_OFFERING.getDescript() ;
        if (offeringArg != null)
            offeringArg = offeringArg.trim() ;
        String     customerArg          = pnl_CUSTOMER.getDescript() ;
        if (customerArg != null)
            customerArg = customerArg.trim() ;

        if (foundVec == null)
            foundVec = new Vector(10);
        else
            foundVec.removeAllElements();
        enableGUI ( false ) ;

        if ( list.isSelectionEmpty() )
            GUISystem.printBox ( "Info", "You must select a field to search"  ) ;


        else {

            if ( ( searchArg != null ) && ( searchArg.length() > 0 ) ||
                 ( userArg != null ) && ( userArg.length() > 0 )     ||
                 ( offeringArg != null ) && ( offeringArg.length() > 0 ) ||
                 ( customerArg != null ) && ( customerArg.length() > 0 ) ||
                 ( warrArg != null ) && ( warrArg.length() > 0 )        ) {


                if ( ( prodSelectVec != null ) && (prodSelectVec.size() > 0 ) ) {

                    int i = 0 ;
                    int size = prodSelectVec.size() ;
                    System.out.println("size"+prodSelectVec.size()) ;

                    while ( i < size ) {
                        System.out.println("Testing1") ;
                        productRow = (ProductRow) prodSelectVec.elementAt(i) ;
                        if ( productRow != null )
                            returnCode = productRow.readProduct(true) ;
                        System.out.println("returncode"+returnCode) ;


                        switch (list.getSelectedIndex()) {
                        case INDEX_PRODUCT_NUM:

                            if ((productRow.getProduct().getProdNum() != null) && (productRow.getProduct().getProdNum().indexOf ( searchArg ) != -1 ))
                                match = true;
                            break;

                        case INDEX_PRODUCT_IND:


                            if ( (productRow.getProduct().getProductInd() != 0) && (productRow.getProduct().getProductInd() == Integer.parseInt(searchArg)  ))
                                match = true;
                            break;

                        case INDEX_PART_NUM:

                            if ( (productRow.getProduct().getPartNumber() != null) && (productRow.getProduct().getPartNumber().toString().indexOf ( searchArg ) != -1 ))
                                match = true;
                            break;

                        case INDEX_DESCRIPTION:

                            if ( (productRow.getProduct().getDescription() != null) && (productRow.getProduct().getDescription().indexOf (searchArg ) != -1  ))
                                match = true;
                            break;

                        case INDEX_MACHINE:

                            if ( (productRow.getProduct().getMachine() != null) && (productRow.getProduct().getMachine().toString().indexOf ( searchArg ) != -1 ))
                                match = true;
                            break;

                        case INDEX_MODEL:

                            if ( (productRow.getProduct().getModel() != null) && (productRow.getProduct().getModel().toString().indexOf ( searchArg ) != -1 ))
                                match = true;
                            break;

                        case INDEX_USERID:
                            if ( (productRow.getProduct().getLastTouchedBy() != null) && (productRow.getProduct().getLastTouchedBy().indexOf ( userArg ) != -1 ))
                                match = true;
                            break;


                        case INDEX_PROJECT:

                            if ( (productRow.getProduct().getProject() != null) && (productRow.getProduct().getProject().toString().indexOf ( searchArg ) != -1 ))
                                match = true;
                            break;

                        case INDEX_BASE_MODEL:
                            if ( (productRow.getProduct().getBaseModel() != null) && (productRow.getProduct().getBaseModel().indexOf ( searchArg ) != -1 ))
                                match = true;
                            break;

                        case INDEX_CUSTOMER:

                            int customerInd =  productRow.getProduct().getCustomer() ;
                            TypeCustomerRec customerRec = (TypeCustomerRec)(TypeList.getInstance().objectFromInd( customerInd, TypeList.TYPE_CUSTOMER)) ;
                            if (customerRec != null )
                                if (customerRec.getDescript().indexOf( customerArg ) != -1 )
                                    match = true;

                            break;

                        case INDEX_OFFERING:
                            int offeringInd =  productRow.getProduct().getOffering() ;
                            TypeOfferingRec offeringRec = (TypeOfferingRec)(TypeList.getInstance().objectFromInd( offeringInd, TypeList.TYPE_OFFERING)) ;
                            if (offeringRec != null)
                                if (offeringRec.getDescript().indexOf( offeringArg ) != -1 )
                                    match = true;
                            break;


                        case INDEX_WARRANTY:
                            int warrantyInd =  productRow.getProduct().getWarranty() ;
                            TypeWarrantyRec warrantyRec = (TypeWarrantyRec)(TypeList.getInstance().objectFromInd( warrantyInd, TypeList.TYPE_WARRANTY)) ;
                            if (warrantyRec != null)
                                if (warrantyRec.getDescript().indexOf( warrArg ) != -1 )
                                    match = true;

                            break;

                        default:

                            break;

                        } // end switch
                        if (match)
                            foundVec.addElement(prodSelectVec.elementAt(i) ) ;

                        match = false;
                        i++;

                    } /* while */



                    if ( (foundVec ==null) || (foundVec.size() == 0) )
                        GUISystem.printBox ( STR_INFO, STR_NO_RESULTS_FROM_SEARCH ) ;

                    dispose() ;

                } /* if prodSelectVec...   */

            } else {
                GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
                dispose() ;
            }

        }

    } /* searchFromSelection */


    /*------------------------------------------------------------*/
    /* search
     */
    /**
     *    This method will perform a search based
     *  on what is selected in the List box
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void search () {
        SQLMethod  sqlMethod        = new SQLMethod ( DatabaseSystem.QUEST, "ProductSearchDlg.search", 5 ) ;
        String     search           = (String) list.getSelectedValue() ;
        String     searchArg        = ef_SEARCH.getText() ;
        if (searchArg != null) searchArg = searchArg.trim() ;
        System.out.println("searcharg1"+searchArg) ;
        String     warrArg          = pnl_WARRANTY.getDescript() ;
        if (warrArg != null) warrArg = warrArg.trim() ;
        String     userArg          = pnl_USERENTRY.getUserId() ;
        if (userArg != null) userArg = userArg.trim() ;
        String     offerArg         = pnl_OFFERING.getDescript() ;
        if (offerArg != null) offerArg = offerArg.trim() ;
        String     custArg          = pnl_CUSTOMER.getDescript() ;
        if (custArg != null) custArg = custArg.trim() ;
        boolean    drafts           = ck_REQUEST.isSelected() ;
        boolean    exactMatch       = ck_EXACTMATCH.isSelected() ;
        String     operation        = null ;
        String     operation1       = null ;
        String     operation2       = null ;
        String     operation3       = null ;
        String     operation4       = null ;

        String     fromClause       = null ;
        String     selectClause     = null ;
        String     query1           = null ;

        String     whereClause      = "" ;
        boolean    queryMachine     = false ;
        boolean    queryModel       = false ;
        boolean    queryProjectName = false ;
        boolean    queryCustomer    = false ;
        boolean    queryOffering    = false ;
        boolean    queryWarranty    = false ;
        boolean    queryPartNumber  = false ;
        boolean    queryProductind  = false ;

        boolean    queryProdnum     = false ;
        boolean    queryDesc        = false ;
        boolean    queryUser        = false ;
        boolean    queryBase        = false ;

        Vector     products1        = new Vector (1) ;

        String     geoFromClause        = "" ;
        String     geoWhereClause       = "" ;
        String     countryFromClause    = "" ;
        String     countryWhereClause   = "" ;

        enableGUI ( false ) ;

        if (foundVec == null)
            foundVec = new Vector(10);
        else
            foundVec.removeAllElements();


        try {
            if (list.isSelectionEmpty())
                GUISystem.printBox ( "Info", "You must select a field to search" )  ;
            else if (list_GEO.isSelectionEmpty())
                GUISystem.printBox ( "Info", "You must select at least 1 geography" ) ;
            else if (list_COUNTRY.isSelectionEmpty())
                GUISystem.printBox ( "Info", "You must select at least 1 country" ) ;

            else if ( (( searchArg != null ) && ( searchArg.length() > 0 ) ) ||
                      (( userArg != null) && (userArg.length() > 0 )) ||
                      (( offerArg != null) && (offerArg.length() > 0 )) ||
                      (( warrArg != null) && (warrArg.length() > 0 )) ||
                      (( custArg != null) && (custArg.length() > 0 ))
                    ) {
                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Replace the wildcards with DB/2 wildcards               */
                /*                                                           */
                /*-----------------------------------------------------------*/
                searchArg = Text.cleanDBString ( searchArg ) ;
                searchArg = Text.replaceAllStrInStr ( searchArg, "*", "%" ) ;
                warrArg = Text.cleanDBString ( warrArg ) ;
                warrArg = Text.replaceAllStrInStr ( warrArg, "*", "%" ) ;
                userArg = Text.cleanDBString ( userArg ) ;
                userArg = Text.replaceAllStrInStr ( userArg, "*", "%" ) ;
                offerArg = Text.cleanDBString ( offerArg ) ;
                offerArg = Text.replaceAllStrInStr ( offerArg, "*", "%" ) ;
                custArg = Text.cleanDBString ( custArg ) ;
                custArg = Text.replaceAllStrInStr ( custArg, "*", "%" ) ;
                if ((! exactMatch) && ( searchArg.indexOf ( "%" ) == -1 ))
                    searchArg="%"+searchArg+"%" ;
                if ((! exactMatch) && ( userArg.indexOf ( "%" ) == -1 ))
                    userArg="%"+userArg+"%" ;

                if ((! exactMatch) && ( offerArg.indexOf ( "%" ) == -1 ))
                    offerArg="%"+offerArg+"%" ;

                if ((! exactMatch) && ( custArg.indexOf ( "%" ) == -1 ))
                    custArg="%"+custArg+"%" ;


                if ((! exactMatch) && ( warrArg.indexOf ( "%" ) == -1 ))
                    warrArg="%"+warrArg+"%" ;

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Assign our comparison operator                          */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( searchArg.indexOf ( "%" ) >= 0 )
                    operation = "LIKE" ;
                else
                    operation = "=" ;

                if ( warrArg.indexOf ( "%" ) >= 0 )
                    operation1 = "LIKE" ;
                else
                    operation1 = "=" ;

                if ( userArg.indexOf ( "%" ) >= 0 )
                    operation2 = "LIKE" ;
                else
                    operation2 = "=" ;


                if ( offerArg.indexOf ( "%" ) >= 0 )
                    operation3 = "LIKE" ;
                else
                    operation3 = "=" ;


                if ( custArg.indexOf ( "%" ) >= 0 )
                    operation4 = "LIKE" ;
                else
                    operation4 = "=" ;




                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Form the 'Select' clause                                */
                /*                                                           */
                /*-----------------------------------------------------------*/

                selectClause =        "SELECT DISTINCT(PRODUCT.PRODUCTIND), PRODUCT.PRODUCTIND," +
                                      "PRODUCT.PRODNUM,  PRODUCT.DESCRIPTION, " +
                                      "PRODUCT.CREATEDON, PRODUCT.CREATEDBY, " +
                                      "DATES.ACTUAL, DATES.TARGET, PRODUCT.LASTTOUCHEDBY, " +
                                      "PRODUCT.LASTTOUCHEDON, PRODUCT.CHECKEDOUTBY, PRODUCT.PUBLISH "                                ;

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Form the 'Where' clause                                 */
                /*                                                           */
                /*-----------------------------------------------------------*/

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Product Indicator                                       */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( search.equals ( Str.getStr ( STR_PRODUCT_IND ) ) ) {

                    if (! exactMatch)
                        searchArg = Text.replaceAllStrInStr ( searchArg, "%", "" ) ;
                    whereClause+= " PRODUCT.PRODUCTIND = "+searchArg+" " ;
                    queryProductind = true ;
                }


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Product Number                                          */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( search.equals ( Str.getStr ( STR_PRODUCT_NUMBER ) ) ) {

                    whereClause+= " PRODUCT.PRODNUM "+operation+" '"+searchArg+"' " ;
                    queryProdnum = true ;
                }

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Machine                                                 */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_MACHINE ) ) ) {


                    whereClause+= " TC.DESCRIPTION "+operation+" '"+searchArg+"' AND " +
                                  "TC.CATEGORYLEVEL = '"+TypeCategory.LEVEL_MACHINE+"' " ;
                    queryMachine = true ;
                }
                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Model                                                   */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_MODEL ) ) ) {


                    whereClause+= " TC.DESCRIPTION "+operation+" '"+searchArg+"' AND " +
                                  "TC.CATEGORYLEVEL = '"+TypeCategory.LEVEL_MODEL+"'  " ;
                    queryModel = true ;
                }


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Project Name                                            */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_PROJECTNAME ) ) ) {


                    whereClause+= " TC.DESCRIPTION "+operation+" '"+searchArg+"' AND " +
                                  "TC.CATEGORYLEVEL = '"+TypeCategory.LEVEL_PROJECT+"' " ;
                    queryProjectName = true ;
                }

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Part Number                                             */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_PARTNUM ) ) ) {


                    whereClause+= " TC.DESCRIPTION "+operation+" '"+searchArg+"' AND " +
                                  "TC.CATEGORYLEVEL = '"+TypeCategory.LEVEL_PARTNUMBER+"' " ;

                    queryPartNumber = true ;
                }



                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Description                                             */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_DESCRIPTION ) ) ) {

                    whereClause+= " PRODUCT.DESCRIPTION "+operation+" '"+searchArg+"' " ;
                    queryDesc = true ;
                }


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Base Model                                              */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_BASEMODEL ) ) ) {

                    whereClause+= " PRODUCT.BASEMODEL "+operation+" '"+searchArg+"' " ;
                    queryBase = true ;
                }



                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   User Id                                                 */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_USERID ) ) ) {


                    whereClause+= " PRODUCT.LASTTOUCHEDBY "+operation2+" '"+userArg+"' " ;
                    queryUser = true ;
                }




                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Customer                                                */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_CUSTOMER ) ) ) {

                    whereClause+= " TCU.DESCRIPTION " +operation4+" ' "+custArg+"' " ;
                    queryCustomer = true ;
                }


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Offering                                                */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_OFFERING ) ) ) {

                    whereClause+= " TO.DESCRIPTION " +operation3+" ' "+offerArg+"' " ;
                    queryOffering = true ;
                }



                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Warranty                                                */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else if ( search.equals ( Str.getStr ( STR_WARRANTY ) ) ) {


                    whereClause+= " TW.DESCRIPTION " +operation1+" '"+warrArg+"' " ;
                    queryWarranty = true ;
                }

                
                if ( whereClause != null ) {

                    if ( (((TypeGeoRec)list_GEO.getSelectedValue()).getInd() == TypeList.GEO_WORLD_WIDE) &&
                         (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() == TypeList.COUNTRY_WORLD_WIDE) ) {
                        geoWhereClause="" ;


                    }

                    
                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Now, add the Geo/Country selections to the where clause */
                    /*                                                           */
                    /*-----------------------------------------------------------*/

                    /*---------------------------------------------------------------*/
                    /*                                                               */
                    /*   If the Geo selected is 'WorldWide' and Country is not 'All' */
                    /*                                                               */
                    /*---------------------------------------------------------------*/
                    else if ( (((TypeGeoRec)list_GEO.getSelectedValue()).getInd() == TypeList.GEO_WORLD_WIDE) &&
                              (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE) ) {


                        if (list_COUNTRY.getSelectedValues() != null) {
                            System.out.println("geoInd"+getGeoInd()) ;
                            System.out.println("countryInd"+getCountryInd()) ;
                            System.out.println("countryDes"+getCountryDescript()) ;

                            
                            geoWhereClause+=" AND GEO.GEOIND IN (" +getCountryGeoInd()+","+TypeList.GEO_WORLD_WIDE+") AND " +
                                                  "GEO.PRODUCTIND = PRODUCT.PRODUCTIND " ;
                                                  
                            countryWhereClause+=" AND COUNTRY.COUNTRYCODEIND IN (" +getCountryInd()+ ") AND " +
                                                  "COUNTRY.PRODUCTIND = PRODUCT.PRODUCTIND " ;

                            if ( drafts ){
                            
                                geoFromClause+=", PRODRAFT.GEO GEO " ;
                                countryFromClause+=", PRODRAFT.COUNTRY COUNTRY " ;
                            }
                            else
                            {
                            
                                geoFromClause+=", PRODUCT.GEO GEO " ;
                                countryFromClause+=", PRODUCT.COUNTRY COUNTRY " ;
                            }

                        }

                    }

                    /*---------------------------------------------------------------*/
                    /*                                                               */
                    /*   If the Geo selected is not 'WorldWide' and Country is 'All' */
                    /*                                                               */
                    /*---------------------------------------------------------------*/



                    else if ( (((TypeGeoRec)list_GEO.getSelectedValue()).getInd() != TypeList.GEO_WORLD_WIDE) &&
                              (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() == TypeList.COUNTRY_WORLD_WIDE) ) {


                        if ( ( list_GEO.getSelectedValues() != null ) ) {
                            System.out.println("geoDescript"+getGeoInd()) ;

                            geoWhereClause+=" AND GEO.GEOIND IN (" +getGeoInd()+") AND " +
                                                   "GEO.PRODUCTIND = PRODUCT.PRODUCTIND " ;


                            if ( drafts )
                                geoFromClause+=", PRODRAFT.GEO GEO " ;
                            else
                                geoFromClause+=", PRODUCT.GEO GEO " ;

                         }

                    }


                    /*-------------------------------------------------------------------*/
                    /*                                                                   */
                    /*   If the Geo selected is not 'WorldWide' and Country is not 'All' */
                    /*                                                                   */
                    /*-------------------------------------------------------------------*/

                    else if ( (((TypeGeoRec)list_GEO.getSelectedValue()).getInd() != TypeList.GEO_WORLD_WIDE) &&
                              (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE) ) {


                        if ( ( list_GEO.getSelectedValues() != null ) && (list_COUNTRY.getSelectedValues() != null) ) {
                            System.out.println("geoInd"+getGeoInd()) ;
                            System.out.println("countryInd"+getCountryInd()) ;
                            System.out.println("countryDes"+getCountryDescript()) ;


                            geoWhereClause+=" AND GEO.GEOIND IN (" +getGeoInd()+","+TypeList.GEO_WORLD_WIDE+") AND " +
                                                   "GEO.PRODUCTIND = PRODUCT.PRODUCTIND " ;

                            countryWhereClause+=" AND COUNTRY.COUNTRYCODEIND IN (" +getCountryInd()+ ") AND " +
                                                   "COUNTRY.PRODUCTIND = PRODUCT.PRODUCTIND " ;



                            if ( drafts )
                            {
                            
                                geoFromClause+=", PRODRAFT.GEO GEO " ;
                                countryFromClause+=", PRODRAFT.COUNTRY COUNTRY " ;
                            }
                            else
                            {
                            
                                geoFromClause+=", PRODUCT.GEO GEO " ;
                                countryFromClause+=", PRODUCT.COUNTRY COUNTRY " ;
                            }


                            
                        }

                    }


                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Form the 'From' clause                                  */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( drafts ) {

                        if ( queryProductind)
                            fromClause = " FROM PRODRAFT.PRODUCT PRODUCT LEFT OUTER JOIN PRODRAFT.DATES DATES ON "+
                                         " PRODUCT.PRODUCTIND = DATES.PRODUCTIND AND DATES.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" "+
                                         " WHERE "+whereClause+ " " ;

                        if ( ( queryProdnum) || (queryDesc) || (queryUser) || (queryBase) ) 
                        {
                            if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT "+geoFromClause+" WHERE "+
                                         " "+whereClause+geoWhereClause+" UNION SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT "+countryFromClause+" WHERE "+
                                         " "+whereClause+countryWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else

                            fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT "+geoFromClause+" WHERE "+
                                         " "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ; 
                        }

                        if ( ( queryMachine) || (queryModel) || (queryProjectName) || (queryPartNumber) ) 
                        {
                            if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.TYPECATEGORY TC, PRODRAFT.CATEGORY PRODUCT "+geoFromClause+" WHERE "+
                                         " TC.TYPECATIND = PRODUCT.TYPECATIND AND "+whereClause+geoWhereClause+" UNION "+
                                         " SELECT  PRODUCT.PRODUCTIND FROM PRODRAFT.TYPECATEGORY TC, PRODRAFT.CATEGORY PRODUCT "+
                                         ""+countryFromClause+" WHERE TC.TYPECATIND = PRODUCT.TYPECATIND AND "+
                                         ""+whereClause+countryWhereClause+"  ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        
                        else

                            fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.TYPECATEGORY TC, PRODRAFT.CATEGORY PRODUCT "+geoFromClause+" WHERE "+
                                         " TC.TYPECATIND = PRODUCT.TYPECATIND AND "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        }

                        if ( queryCustomer ) 
                        {
                            if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT, PRODUCT.TYPECUSTOMER TCU "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPECUSTOMERIND = TCU.TYPECUSTOMERIND AND "+whereClause+geoWhereClause+" UNION "+
                                         " SELECT  PRODUCT.PRODUCTIND FROM PRODRAFT.PRODUCT PRODUCT, PRODUCT.TYPECUSTOMER TCU "+
                                         ""+countryFromClause+" WHERE PRODUCT.TYPECUSTOMERIND = TCU.TYPECUSTOMERIND AND "+
                                         ""+whereClause+countryWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT, PRODUCT.TYPECUSTOMER TCU "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPECUSTOMERIND = TCU.TYPECUSTOMERIND AND "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        }

                        if ( queryOffering ) 
                        {
                            if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT, PRODUCT.TYPEOFFERING TO "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPEOFFERINGIND = TO.TYPEOFFERINGIND AND "+whereClause+geoWhereClause+" UNION "+
                                         " SELECT PRODUCT.PRODUCTIND FROM PRODRAFT.PRODUCT PRODUCT, PRODUCT.TYPEOFFERING TO "+
                                         ""+countryFromClause+" WHERE PRODUCT.TYPEOFFERINGIND = TO.TYPEOFFERINGIND AND "+
                                         ""+whereClause+countryWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else
                            
                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT, PRODUCT.TYPEOFFERING TO "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPEOFFERINGIND = TO.TYPEOFFERINGIND AND "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        }



                        if ( queryWarranty ) 
                        {
                            if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT, PRODUCT.TYPEWARRANTY TW "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPEWARRANTYIND = TW.TYPEWARRANTYIND AND "+whereClause+geoWhereClause+" UNION "+
                                         " SELECT PRODUCT.PRODUCTIND FROM PRODRAFT.PRODUCT, PRODUCT.TYPEWARRANTY TW "+
                                         ""+countryFromClause+" WHERE PRODUCT.TYPEWARRANTYIND = TW.TYPEWARRANTYIND AND "+
                                         ""+whereClause+countryWhereClause+"  ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODRAFT.PRODUCT PRODUCT, PRODUCT.TYPEWARRANTY TW "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPEWARRANTYIND = TW.TYPEWARRANTYIND AND "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        }



                        


                    }





                else
                {

                    if ( queryProductind)
                        fromClause = " FROM PRODUCT.PRODUCT PRODUCT LEFT OUTER JOIN PRODUCT.DATES DATES ON "+
                                     " PRODUCT.PRODUCTIND = DATES.PRODUCTIND AND DATES.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" "+
                                     " WHERE "+whereClause+ " " ;

                    if ( ( queryProdnum) || (queryDesc) || (queryUser) || (queryBase) )
                    {

                        if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT "+geoFromClause+" WHERE "+
                                         " "+whereClause+geoWhereClause+" UNION SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT "+countryFromClause+" WHERE "+
                                         " "+whereClause+countryWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else

                            fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT "+geoFromClause+" WHERE "+
                                         " "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;

                    }


                    if ( ( queryMachine) || (queryModel) || (queryProjectName) || (queryPartNumber) )
                    {

                        if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.TYPECATEGORY TC, PRODUCT.CATEGORY PRODUCT "+geoFromClause+" WHERE "+
                                         " TC.TYPECATIND = PRODUCT.TYPECATIND AND "+whereClause+geoWhereClause+" UNION "+
                                         " SELECT  PRODUCT.PRODUCTIND FROM PRODUCT.TYPECATEGORY TC, PRODUCT.CATEGORY PRODUCT "+
                                         ""+countryFromClause+" WHERE TC.TYPECATIND = PRODUCT.TYPECATIND AND "+
                                         ""+whereClause+countryWhereClause+"  ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else

                            fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.TYPECATEGORY TC, PRODUCT.CATEGORY PRODUCT "+geoFromClause+" WHERE "+
                                         " TC.TYPECATIND = PRODUCT.TYPECATIND AND "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                    }

                    if ( queryCustomer )
                    {
                        if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)


                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT, PRODUCT.TYPECUSTOMER TCU "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPECUSTOMERIND = TCU.TYPECUSTOMERIND AND "+whereClause+geoWhereClause+" UNION "+
                                         " SELECT  PRODUCT.PRODUCTIND FROM PRODUCT.PRODUCT PRODUCT, PRODUCT.TYPECUSTOMER TCU "+
                                         ""+countryFromClause+" WHERE PRODUCT.TYPECUSTOMERIND = TCU.TYPECUSTOMERIND AND "+
                                         ""+whereClause+countryWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT, PRODUCT.TYPECUSTOMER TCU "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPECUSTOMERIND = TCU.TYPECUSTOMERIND AND "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                    }


                    if ( queryOffering )
                    {

                        if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT, PRODUCT.TYPEOFFERING TO "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPEOFFERINGIND = TO.TYPEOFFERINGIND AND "+whereClause+geoWhereClause+" UNION "+
                                         " SELECT PRODUCT.PRODUCTIND FROM PRODUCT.PRODUCT PRODUCT, PRODUCT.TYPEOFFERING TO "+
                                         ""+countryFromClause+" WHERE PRODUCT.TYPEOFFERINGIND = TO.TYPEOFFERINGIND AND "+
                                         ""+whereClause+countryWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT, PRODUCT.TYPEOFFERING TO "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPEOFFERINGIND = TO.TYPEOFFERINGIND AND "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                    }


                    if ( queryWarranty )
                    {

                        if (((TypeCountryCodeRec)list_COUNTRY.getSelectedValue()).getInd() != TypeList.COUNTRY_WORLD_WIDE)

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT, PRODUCT.TYPEWARRANTY TW "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPEWARRANTYIND = TW.TYPEWARRANTYIND AND "+whereClause+geoWhereClause+" UNION "+
                                         " SELECT PRODUCT.PRODUCTIND FROM PRODUCT.PRODUCT, PRODUCT.TYPEWARRANTY TW "+
                                         ""+countryFromClause+" WHERE PRODUCT.TYPEWARRANTYIND = TW.TYPEWARRANTYIND AND "+
                                         ""+whereClause+countryWhereClause+"  ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                        else

                            fromClause=  " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                                         " PRODUCT.PRODUCT PRODUCT, PRODUCT.TYPEWARRANTY TW "+geoFromClause+" WHERE "+
                                         " PRODUCT.TYPEWARRANTYIND = TW.TYPEWARRANTYIND AND "+whereClause+geoWhereClause+" ) J1 LEFT OUTER JOIN "+
                                         " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                                         " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                                         " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;
                    }



                }




                    LogSystem.log ( 1, "Executing Product Search:"+whereClause ) ;


                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Form the SQL Query                                      */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    query1 = selectClause+fromClause ;
                    query1+=" ORDER BY PRODUCT.PRODUCTIND FOR FETCH ONLY " ;
                    //System.out.println("query"+ query1 ) ;


                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Execute the stored procedure                            */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    Statement stmt = sqlMethod.createStatement() ;
                    ResultSet rs   = stmt.executeQuery(query1) ;


                    while ( rs.next() ) {
                        String date = "" ;
                        if ( (rs.getString(7)) != null  )
                            date =  DateSystem.prettyDateFromDBDate ( rs.getString( 7 )) ;
                        else if ( (rs.getString(8)) != null )
                            date =  DateSystem.prettyDateFromDBDate ( rs.getString( 8 )) ;

                        else
                            date = rs.getString(7) ;

                        ProductRow  productRow = new ProductRow (rs.getInt ( 1 ),                                 //Product Ind
                                                                 0,                                               //Existing Product Ind
                                                                 rs.getString ( 3 ),   //PRoduct number or Part Number
                                                                 rs.getString ( 4 ),                              //Description
                                                                 DateSystem.prettyDateFromStamp ( rs.getString ( 5 ).trim() ),//Created on
                                                                 UserSystem.getNameFromUserId ( rs.getString ( 6 ).trim()),//Created     By
                                                                 date,                                            //Actual Announce Date
                                                                 "",                                              //Target Announce Date
                                                                 UserSystem.getNameFromUserId ( rs.getString ( 9 ).trim()), //LAstTouched By
                                                                 DateSystem.prettyDateFromStamp ( rs.getString ( 10 ).trim() ),//LastTouched on
                                                                 drafts ) ;
                        if ( rs.getString(11) != null )
                            productRow.setImage ( ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.SMALL_PRODUCT_LOCK ) ) ;

                        if ( rs.getString(12).equals("N" ))
                            productRow.setImage ( ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.SMALL_RECYCLE_BIN ) ) ;

                        foundVec.addElement( productRow ) ;

                    } /* while */

                    rs.close() ;

                    System.out.println("size"+foundVec.size() ) ;

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   If FoundVec is null                                     */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( (foundVec == null) || (foundVec.size() == 0) )
                        GUISystem.printBox ( STR_INFO, STR_NO_RESULTS_FROM_SEARCH ) ;

                    dispose() ;

                }/* If where clause */






            } else {

                System.out.println("entered else") ;
                GUISystem.printBox ( STR_INFO, STR_MUST_ENTER_VALUE ) ;
                dispose() ;
            }


        }/* try */

        catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            sqlMethod.rollBack() ;
            GUISystem.printBox( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            dispose() ;
        }

        enableGUI ( true ) ;
        ef_SEARCH.requestFocus() ;
        sqlMethod.close() ;
    }/* search */

    /*------------------------------------------------------------*/
    /* enableGUI
     */
    /**
     *    This method will either enable or disable all
     *  of our controls
     *
     *
     *  @param enable TRUE = enable our components, FALSE = disable them
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void enableGUI ( boolean enable ) {
        pb_FIND.setEnabled    ( enable ) ;
        pb_HELP.setEnabled    ( enable ) ;
        pb_CANCEL.setEnabled  ( enable ) ;

    }

    /*------------------------------------------------------------*/
    /* removeAllListeners
     */
    /**
     *    This method will remove all listeners for
     *  this window. We do this everytime we hide
     *  ourselves
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void removeAllListeners () {
        listenerVec.removeAllElements() ;
    }

    /*------------------------------------------------------------*/
    /* setVisible
     */
    /**
     *    This method has been subclassed to remove
     *  all listeners once we are hidden
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setVisible ( boolean visible ) {
        super.setVisible ( visible ) ;
        //  if ( !visible )
        //      removeAllListeners() ;
    }

    /*------------------------------------------------------------*/
    /* getInstance
     */
    /**
     *    This method will return the instance of this
     *  class. Since this class really only instatiates and
     *  shows once, this method is usefull in getting that one instance
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return ProductSearchDlg
     *//*---------------------------------------------------------*/
    public static ProductSearchDlg getInstance() {
        return( instance ) ;
    }

    /*------------------------------------------------------------*/
    /* run
     */
    /**
     *    This thread initiates a search
     *
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void run () {
        //search() ;
        String operation = Thread.currentThread().getName() ;

        if ( operation.equals (THREAD_SEARCH_ALL ))
            search() ;
        else if ( operation.equals ( THREAD_SEARCH_SELECT ))
            searchFromSelection() ;

    }

    /*------------------------------------------------------------*/
    /* fireProductFindEvent
     */
    /**
     *    This method will fire the Product Find
     *  event to all registered listeners
     *
     *
     *  @param vec The vector of products returned
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void fireProductFindEvent ( Vector vec ) {
        int i    = 0 ;
        int size = listenerVec.size() ;

        while ( vec != null && ( i < size ) ) {
            ((FindProductListener)listenerVec.elementAt(i)).findProductComplete ( vec ) ;
            i++ ;
        }
    }

    /*------------------------------------------------------------*/
    /* addFindProductListener
     */
    /**
     *    This method will add a listener to this window
     *    to listen to when the 'Find' button is pressed
     *
     *
     *
     *
     *  @param listener   The listener to add
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void addFindProductListener ( FindProductListener  listener ) {
        listenerVec.addElement ( listener ) ;
    }

    /*------------------------------------------------------------*/
    /* removeFindProductListener
     */
    /**
     *    This method will remove a FindProductListener from this window
     *
     *
     *
     *
     *
     *  @param listener   The listener to remove
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void removeFindProductListener ( FindProductListener  listener ) {
        listenerVec.removeElement ( listener ) ;

    }

    /*------------------------------------------------------------*/
    /* getCountry
     */
    /**
     *    This method is invoked for selecting a list of countries
     *    after selecting a geo
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private Vector getCountry() {

        Vector vec = TypeList.getInstance().getTypeList( TypeList.TYPE_COUNTRYCODE ) ;
        new QuickSort(vec);

        vec.removeElement( TypeList.getInstance().objectFromInd( TypeList.COUNTRY_WORLD_WIDE, TypeList.TYPE_COUNTRYCODE )) ;
        TypeCountryCodeRec allCountry    = (TypeCountryCodeRec)TypeList.getInstance().objectFromInd ( TypeList.COUNTRY_WORLD_WIDE,
                                                                                                      TypeList.TYPE_COUNTRYCODE ) ;

        if ( allCountry == null )
            allCountry = new TypeCountryCodeRec ( TypeList.COUNTRY_WORLD_WIDE, "All" ) ;

        vec.add ( 0, allCountry ) ;


        return( vec ) ;


    }

    /*------------------------------------------------------------*/
    /* getCountryFromGeo
     */
    /**
     *    This method is invoked for selecting a list of countries
     *    after selecting a geo
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/

    private void getCountryFromGeo() {

        Vector geoSelectVec = null;
        TypeGeoRec geoRec = null;
        Vector countries = new Vector(10,10);


        geoSelectVec = getGeoSelectVec();

        // don't show any countries if a geo isn't selected
        if ( geoSelectVec == null ) {
            countries.trimToSize();
            setListData(list_COUNTRY, countries);
        }

        // if worldwide is selected show all countries
        else if (((TypeGeoRec)list_GEO.getSelectedValue()).getInd() == TypeList.GEO_WORLD_WIDE) {
            setListData( list_COUNTRY, getCountry());
            list_COUNTRY.setSelectedIndex(0) ;
        }

        else {

            if ((geoSelectVec != null) && (geoSelectVec.size() > 0)) {
                // add all the countries for each geo that is selected
                for (int i= 0; i< geoSelectVec.size(); i++) {
                    geoRec     =  (TypeGeoRec)geoSelectVec.elementAt(i);
                    if ( (geoRec != null)  && ( geoRec instanceof TypeGeoRec )) {
                        int size = geoRec.getCountryVec().size();
                        countries.addAll(geoRec.getCountryVec());
                    }
                }

                //sort the countries, in alpha order
                if ((countries != null) && (countries.size() > 0) )
                    new QuickSort(countries);
                TypeCountryCodeRec allCountry    = (TypeCountryCodeRec)TypeList.getInstance().objectFromInd ( TypeList.COUNTRY_WORLD_WIDE,
                                                                                                              TypeList.TYPE_COUNTRYCODE ) ;
                // make sure all countries is shown first in the list
                if ( allCountry == null )
                    allCountry = new TypeCountryCodeRec ( TypeList.COUNTRY_WORLD_WIDE, "All" ) ;
                countries.add(0, allCountry);
                countries.trimToSize();
                setListData( list_COUNTRY, countries ) ;
                list_COUNTRY.setSelectedIndex(0) ;
            }
        }
    }


    /*------------------------------------------------------------*/
    /* getGeo
     */
    /**
     *    This method will return all geos except the World-wide one
     *    ProductRow object
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Vector
     *//*---------------------------------------------------------*/
    private Vector getGeo() {
        Vector vec = TypeList.getInstance().getTypeList( TypeList.TYPE_GEOGRAPHY ) ;

        vec.removeElement( TypeList.getInstance().objectFromInd( 6, TypeList.TYPE_GEOGRAPHY));
        TypeGeoRec allGeo    = (TypeGeoRec)TypeList.getInstance().objectFromInd ( TypeList.GEO_WORLD_WIDE,
                                                                                  TypeList.TYPE_GEOGRAPHY ) ;

        if ( allGeo == null )
            allGeo = new TypeGeoRec ( TypeList.GEO_WORLD_WIDE, "World-Wide" ) ;

        vec.add( 0, allGeo ) ;

        return( vec ) ;
    }


    /**
      *  This method will set the data in a JComboBox. Please note that
      * you must have previously set the CombBox model to a new DefaultComboBoxModel
      * for this method to function. (JComboBox.setModel ( new DefaultComboBoxModel ) ;
      *
      * @param comboBox The JComboBox to put data in
      * @param data The data to insert
      **/
    public void setListData ( JList list, Vector data ) {
        DefaultListModel model = (DefaultListModel) list.getModel() ;

        if ( model != null ) {

            model.removeAllElements() ;

            if ( data != null ) {
                int i    = 0 ;
                int size = data.size() ;

                while ( i < size ) {
                    model.addElement ( data.elementAt(i++ ) ) ;
                }
            }
        }

    }

    /**
     *  This method will return a Vector of TypeGeoRecs for the geographies selected in
     *  the geography list box.
     **/
    public Vector getGeoSelectVec() {
        Vector geoSelectVec = null ;
        Object[] selectedGeo =  list_GEO.getSelectedValues() ;

        if ((  selectedGeo != null ) && ( selectedGeo.length > 0 )) {
            geoSelectVec = new Vector ( selectedGeo.length ) ;

            for ( int d = 0; d < selectedGeo.length; d++ ) {
                try {
                    Object object = (TypeGeoRec)selectedGeo[d] ;

                    if ( object instanceof TypeGeoRec )
                        geoSelectVec.addElement( object ) ;
                } catch (ClassCastException e ) {
                }
            }
        }
        return( geoSelectVec ) ;


    }

    /**
     *  This method will return a String that contains a comma separated
     *  list of geography indicators
     **/
    public String getGeoInd() {

        String geoInds = "";

        if ( (getGeoSelectVec() != null)  && (getGeoSelectVec().size() > 0) ) {
            for (int i = 0; i < getGeoSelectVec().size() ; i++) {
                geoInds+=((TypeGeoRec)getGeoSelectVec().elementAt(i)).getInd()+"," ;
            }
        }
        geoInds = geoInds.substring ( 0, geoInds.length()-1 ) ;

        return(geoInds) ;
    }

    /**
     *  This method will return a String that contains a comma separated
     *  list of geography descriptions
     **/
    public String getGeoDescript() {

        String geos = "" ;

        if ( (getGeoSelectVec() != null)  && (getGeoSelectVec().size() > 0) ) {
            for (int i = 0; i < getGeoSelectVec().size() ; i++) {
                geos+=((TypeGeoRec)getGeoSelectVec().elementAt(i)).getDescript()+"," ;
            }
        }
        geos = geos.substring ( 0, geos.length()-1 ) ;

        return(geos) ;
    }



    /**
     *  This method will return a Vector of TypeCountryCodeRecs for the countries selected in
     *  the country list box.
     **/
    public Vector getCountrySelectVec() {
        Vector countrySelectVec = null ;
        Object[] selectedCountry =  list_COUNTRY.getSelectedValues() ;

        if ((  selectedCountry != null ) && ( selectedCountry.length > 0 )) {
            countrySelectVec = new Vector ( selectedCountry.length ) ;

            for ( int d = 0; d < selectedCountry.length; d++ ) {
                try {
                    Object object = (TypeCountryCodeRec)selectedCountry[d] ;

                    if ( object instanceof TypeCountryCodeRec )
                        countrySelectVec.addElement( object ) ;
                } catch (ClassCastException e ) {
                }
            }
        }
        return( countrySelectVec ) ;


    }

    /**
     *  This method will return a String that contains a comma separated
     *  list of country code indicators
     **/
    public String getCountryInd() {

        String countryInds = "";

        if ( (getCountrySelectVec() != null)  && (getCountrySelectVec().size() > 0) ) {
            for (int i = 0; i < getCountrySelectVec().size() ; i++) {
                countryInds+=((TypeCountryCodeRec)getCountrySelectVec().elementAt(i)).getInd()+"," ;
            }
        }
        countryInds = countryInds.substring ( 0, countryInds.length()-1 ) ;

        return(countryInds) ;
    }


    /**
     *  This method will return a String that contains a comma separated
     *  list of country descriptions
     **/
    public String getCountryDescript() {

        String countries = "" ;

        if ( (getCountrySelectVec() != null)  && (getCountrySelectVec().size() > 0) ) {
            for (int i = 0; i < getCountrySelectVec().size() ; i++) {
                countries+=((TypeCountryCodeRec)getCountrySelectVec().elementAt(i)).getDescript()+"," ;
            }
        }
        countries = countries.substring ( 0, countries.length()-1 ) ;

        return(countries) ;
    }


    /**
     *  This method will return a String that contains a comma separated
     *  list of geo indicators for the countries selected
     **/
    public String getCountryGeoInd() {

        String countryGeoInds = "" ;

        if ( (getCountrySelectVec() != null)  && (getCountrySelectVec().size() > 0) ) {
            for (int i = 0; i < getCountrySelectVec().size() ; i++) {
                countryGeoInds+=((TypeCountryCodeRec)getCountrySelectVec().elementAt(i)).getGeoInd()+"," ;
            }
        }
        countryGeoInds = countryGeoInds.substring ( 0, countryGeoInds.length()-1 ) ;

        return(countryGeoInds) ;
    }





} /* ProductSearchdDlg */



