package com.ibm.nzna.projects.qit.product.gui ;     
/*-----------------------------------------------------------*/
/*   ProductTable                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductTable                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.product.ProductRow ;
/**
 * This subclass of MultiList is used to display products. Any
 * QuestApplet that needs to display a list of products should
 * do so with this control.
 *
 *
 * @version 1.0, 4/4/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductTable                                            */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductTable extends MultiList implements ProductConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Default constructor creates the default
    *  table
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductTable () {
      super ( GUISystem.getFontUtil() ) ;
      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create a ProductTable
    *  with the products that are in the vector passed
    *  
    *
    *  @param products A Vector of ProductList objects to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductTable ( Vector products ) {
      super ( GUISystem.getFontUtil() ) ;
      initialize() ;
      setProducts ( products ) ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our default control
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {

      setForeground      ( Color.black ) ;
      setWordWrap        ( true ) ;
      setOddLinesColor   ( new Color ( 230, 230, 230 ) ) ;
      initializeColumns() ;

   } /* initialize  */




   /*------------------------------------------------------------*/
   /* setProducts
    */
   /**
    *    This method will set the products that
    *  we have. The current set of products being
    *  viewed will be erased
    *
    *  @param products A Vector of ProductList objects to show
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setProducts ( Vector products ) {
      setData ( products ) ;
      initializeColumns() ;
   }


   /**
     * This Method will set up and initialie all of our columns
     **/
   private void initializeColumns () {
      String[] listTitle = { 
         "",
         hasOptions() ? Str.getStr ( STR_PART_NUMBER ) : Str.getStr ( STR_PRODUCT_NUMBER ),
         Str.getStr ( STR_DESCRIPTION ),
         Str.getStr ( STR_LAST_TOUCHED_BY ),
         Str.getStr ( STR_LAST_TOUCHED_ON )} ;

      setColumnHeadings  ( listTitle ) ;
      showColumnHeadings ( true ) ;

      if ( !PropertySystem.getDetailList ( PROPERTY_PRODUCT_COLUMN_WIDTHS, this ) ) {
         setColumnWidth     ( 0, 20 ) ;
         setColumnWidth     ( 1, 100 ) ;
         setColumnWidth     ( 2, 300  ) ;
         setColumnWidth     ( 3, 100 ) ;
         setColumnWidth     ( 4, 100 ) ;
      } /* if  */
   }


   /**
     * This method will determine if the list we are displaying contains
     * options
     **/
   private boolean hasOptions () {
      Vector  data            = getData() ;
      boolean containsOptions = false ;

      if ( ( data != null ) && ( data.size() > 0 ) ) {

         if ( ((ProductRow)data.elementAt(0)).getColumnData(1).toString().indexOf ( "-" ) == -1 )
            containsOptions = true ;
      }


      return( containsOptions ) ;
   }
} /* ProductTable */
