package com.ibm.nzna.projects.qit.product.history ;
/*-----------------------------------------------------------*/
/*   ProductHistoryNavPanel                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductHistoryNavPanel              */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.common.quest.product.*;
import com.ibm.nzna.projects.qit.product.gui.*;
import com.ibm.nzna.projects.qit.product.*;
/**
 * This nav panel is used by the ProductHistoryPanel for navigation.
 *  It allows the user to select a date to when the product
 *  was published
 *
 * @version 1.0, 05/21/2001
 * @author  Brian Bokor
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductHistoryNavPanel                                  */
/*                                                           */
/*-----------------------------------------------------------*/

public class ProductHistoryNavPanel extends JPanel implements Runnable, NavPanel, AppConst, ProductConst, ActionListener, MouseListener {
   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/

  /**
    * This will be the variable used to for the reading of history thread
    */
   private static final String THREAD_READ_HISTORY = "READ_HIST" ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Static Final Variable                           */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * These are the titles of the left nav panel.
    */
   private static final String[] listTitle      = { Str.getStr ( STR_PROD_IND ),
                                                    Str.getStr ( STR_STOP_DATE ),
                                                    Str.getStr ( STR_LAST_TOUCHED_BY )} ;
   private ProductDraft          productDraft   = null ;
   private Product               product        = null ;

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This is the JTextField that you will use to enter in the
    * Prod Ind
    */
   private JTextField ef_PRODIND          = null ;

   /**
    * This will be the JTitle of the JTextField
    */
   private JLabel     st_PRODIND          = null;

   /**
    * This will be the JTitle of the Publish Dates
    */
   private JLabel     st_PUBLISHDATES     = null ;

   /**
    * This will be the List Produced from the search.  It
    * will list all of the previous products.
    */
   private MultiList  historyList         = null ;

   /**
    * This is the GO button used to complete the search
    */
   private DButton    pb_GO               = null ;

   /**
    * This is the JCheckBox that will say if it is a request or not.
    */
   private JCheckBox  ck_REQUEST          = null ;


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/

   /**
    * This is the Right Panel that houses the Product Selected
    */
   private ProductHistoryPanel productHistoryPanel      = null ;

   /**
    * This is whether a request is true or false
    */
   private boolean             request            = false ;
   private int                 prodInd            = 0 ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor simply assigns our ProductHistoryPanel
    *  property
    *
    *
    *  @param ProductHistoryPanel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductHistoryNavPanel ( ProductHistoryPanel productHistoryPanel ) {
      this.productHistoryPanel = productHistoryPanel ;
   }/* constructor */

   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our nav list
    *
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_GO           = new DButton ( Str.getStr ( STR_GO ) ) ;
      ef_PRODIND      = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_NUMERIC ), "", 0 ) ;
      st_PRODIND      = new JLabel ( Str.getStr ( STR_PROD_IND ) ) ;
      historyList     = new MultiList ( GUISystem.getFontUtil() ) ;
      st_PUBLISHDATES = new JLabel ( Str.getStr ( STR_PUBLISH_DATES ) ) ;
      ck_REQUEST      = new JCheckBox ( Str.getStr ( STR_DRAFT ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_PUBLISHDATES.setFont ( FontSystem.smallTitleFont ) ;
      st_PRODIND.setFont ( FontSystem.smallTitleFont ) ;
      historyList.setBorder ( null ) ;
      historyList.setColumnHeadings ( listTitle ) ;
      historyList.setColumnHeadingsBackground ( Color.white ) ;
      historyList.setColumnWidth ( 0, 75 ) ;
      historyList.setColumnWidth ( 1, 75 ) ;
      historyList.setColumnWidth ( 2, 175 ) ;
      ck_REQUEST.setOpaque ( false ) ;
      historyList.setOpaque ( false ) ;
      ef_PRODIND.setOpaque ( true ) ;
      historyList.setBackground ( ThemeSystem.getColor ( "NAVPANEL_BG_COLOR" ) ) ;
      historyList.setOddLinesColor ( ThemeSystem.getColor ( "NAVPANEL_BG_COLOR" ) ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      historyList.getJList().addMouseListener ( this ) ;
      ef_PRODIND.addActionListener ( this ) ;
      pb_GO.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_PRODIND ) ;
      add ( ef_PRODIND ) ;
      add ( pb_GO ) ;
      add ( ck_REQUEST ) ;
      add ( st_PUBLISHDATES ) ;
      add ( historyList ) ;
   }/* initialize */

   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all of our
    *  controls
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       x         = 5 ;
      int       y         = 5 ;
      int       rowHeight = GUISystem.getRowHeight() ;
      Dimension size      = getSize() ;


      st_PRODIND.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      ef_PRODIND.setBounds ( x, y, size.width-(x*2)-55, rowHeight ) ;
      pb_GO.setBounds ( size.width-(x*2)-50, y, 45, rowHeight ) ;
      y+=rowHeight ;

      ck_REQUEST.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight*2 ;

      st_PUBLISHDATES.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      historyList.setBounds ( x, y, size.width-(x*2), size.height-y-10 ) ;

   }/* doLayout */

   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing. Basically we will just give focus
    *  to the ef_PRODIND field
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
      ef_PRODIND.requestFocus() ;
   }/* start */

   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called once we have
    *  been closed. Clean up time!
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
   }/* stop */

   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event we are waiting for has been fired
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_GO  or ef_PRODIND                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == pb_GO ) || ( e.getSource() == ef_PRODIND ) ) {

         try {

             setProdInd( new Integer( ef_PRODIND.getText() ).intValue(), ck_REQUEST.isSelected() );

         } catch ( NumberFormatException ex ) {
            /* Do nothing, somehow they entered alphas */
         }
      }
   }/* actionPerformed */

   /*------------------------------------------------------------*/
   /* mouseClicked
    */
   /**
    *    This method will be called when an item in the
    *  list has been selected
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void mouseClicked ( MouseEvent e ) {

       ProductHistoryRow historyRow = (ProductHistoryRow) historyList.getSelectedItem() ;

       if ( historyRow != null ) {
         productDraft = new ProductDraft( historyRow.getProductInd() );
         readDatabase( productDraft );
       }
   }/* mouseClicked */

   public void mouseEntered ( MouseEvent e ) {}
   public void mouseExited ( MouseEvent e ) {}
   public void mousePressed ( MouseEvent e ) {}
   public void mouseReleased ( MouseEvent e ) {}

   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return out title
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_PRODUCT_HISTORY ) ) ;
   }/* getTitle */

   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will return an instance of this
    *  class
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }/* getJComponent */

   /*------------------------------------------------------------*/
   /* setProduct
    */
   /**
    *    This method will set a ProductDraft Object
    *
    *  @param ProductDraft
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
    public void setProduct ( ProductDraft productDraft ) {
      this.productDraft = productDraft ;
   }/* setProduct */

   /*------------------------------------------------------------*/
   /* getProduct
    */
   /**
    *    This method will get a ProductDraft Object
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Product
    *//*---------------------------------------------------------*/
    public ProductDraft getProduct (  ) {
      return( productDraft ) ;
   }/* getProduct */

   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread does different operations. The operation
    *  done depends on the name of the thread
    *
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_READ_HISTORY                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_READ_HISTORY ) ) {
         readHistory ( new Integer ( ef_PRODIND.getText() ).intValue() ) ;
      }
   }/* run */

   /*------------------------------------------------------------*/
   /* readHistory
    */
   /**
    *    This method will read and display the list
    *  of products that match
    *
    *
    *  @param int
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void readHistory ( int prodInd ) {
      AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

      pb_GO.setEnabled ( false ) ;
      ef_PRODIND.setEnabled ( false ) ;
      parentWin.setStatus ( Str.getStr ( STR_PLEASE_WAIT ) ) ;

      try {

         /*-----------------------------------------------------------------*/
         /*                                                                 */
         /*   If this is a request, we need to get the published prod ind   */
         /*                                                                 */
         /*-----------------------------------------------------------------*/
         if ( request ) {  //draft true
            
             int existingProdInd = ProductSQL.getExistingProductInd( prodInd ) ;
             ef_PRODIND.setText ( ""+existingProdInd ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If we have a valid prod indicator, read in the list     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( prodInd != 0 ) {
            Vector historyVec = ProductHistorySQL.readHistory ( GUISystem.getParentDefWin ( this ), prodInd ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If this vector is good, continue                        */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( historyVec != null ) && ( historyVec.size() > 0 ) ) {
               historyList.removeAll() ;
               //might need to create a row here with the vector items
               historyList.add ( historyVec ) ;
            } else
               GUISystem.printBox ( STR_INFO, STR_NO_PRODUCT_HISTORY_FOUND ) ;
         }

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ,false ) ;
      }

      pb_GO.setEnabled ( true ) ;
      ef_PRODIND.setEnabled ( true ) ;
      parentWin.setStatus ( null ) ;
   }/* readHistory */

   /*------------------------------------------------------------*/
   /* getProdInd
    */
   /**
    *    This method will return what prodInd is
    *  currently displayed in the prod indicator
    *  entry field
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   protected int getProdInd () {  
       try {
         return( new Integer ( ef_PRODIND.getText() ).intValue() ) ;
      } catch ( NumberFormatException e ) {
      }        
      return( 0 ) ;
   }/* getProdInd */

   /*------------------------------------------------------------*/
   /* setProdInd
    */
   /**
    *    This method will set what prodInd is
    *  currently displayed in the prod indicator
    *  entry field
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   protected void setProdInd ( int prodInd, boolean request ) {
       ef_PRODIND.setText(""+prodInd );
       this.request = request ; 
       ck_REQUEST.setSelected( request );
       historyList.removeAll();
       new Thread ( this, THREAD_READ_HISTORY ).start();
   }/* setProdInd */

   /*------------------------------------------------------------*/
   /* readDatabase
    */
   /**
    *    This method will read in the product from the DB
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   private boolean readDatabase ( ProductDraft productDraftNew ) {

       int returnCode = 0;
       
       if ( productDraftNew != null ){

           LogSystem.beginLogSection ( "Reading Product Selected" ) ;
             
           returnCode = productDraftNew.readFromDatabase(); 

           switch ( returnCode ) {
            case 0 :
               LogSystem.log ( 1, "Successful Read" ) ;
               productHistoryPanel.setProduct( productDraftNew );
               break ; 
            case 100 :
               LogSystem.log ( 1, "Unsuccessful Read" ) ;
               GUISystem.printBox ( STR_INFO, STR_NO_PRODUCT_FOUND ) ;
               break ; 
            default :
               LogSystem.log ( 1, "Return Code: "+ returnCode ) ;
               break ;
           }/*switch*/
       }     

       if( returnCode != 0 )
           return( false );
       else
           return( true );

   }/* readDatabase */           
}/* ProductHistoryNavPanel */

