package com.ibm.nzna.projects.qit.product.history ;                                          
/*-----------------------------------------------------------*/
/*   ProductHistoryPanel                                     */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductHistoryPanel                 */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.io.File ;
import java.util.Vector ;
import java.sql.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.help.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.qit.product.manager.* ;
import com.ibm.nzna.projects.qit.product.gui.*;
import com.ibm.nzna.projects.common.quest.product.*;
import com.ibm.nzna.projects.qit.product.*;
import com.ibm.nzna.projects.qit.product.productEdit.*;
/**
 * This panel will display a product's history to the user. It will
 * also allow the user to see changes in history
 *
 * @author  Brian R Bokor
 * @version 1.0
 */

/*----------------------------------------------------*/
/*                                                    */
/* ProductHistoryPanel                                */
/*                                                    */
/*----------------------------------------------------*/
public class ProductHistoryPanel extends JPanel implements QuestPanel, ActionListener, FindProductListener, ProductConst, AppConst, Runnable, HelpSource {
    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Static Final Variable                           */
    /*                                                           */
    /*-----------------------------------------------------------*/

    /**
     * This is the Thread variable used for refresh.
     */
    private static final String THREAD_REFRESH_PREVIEW = "RP" ;
    private static final String timestamp              = "CURRENT TIMESTAMP";

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/

    private ToolBarButton           pb_FIND                = null ;
    //private JScrollPane             scr_PREVIEW            = null ;
    private ActionButton            pb_CLOSE               = null ;
    private ActionButton            pb_RESTORE             = null ;
    private ProductHistoryNavPanel  productHistoryNavPanel = null ;
    private JToolBar                toolBar                = null ;
    private String                  commentHistory         = null ;

    /**
     * This will let us know if we are currently working with a Product
     * that is a draft or not
     */
    private boolean           draft             = false ;

    /**
     * This will let us know if the changes should be shown or not
     */
    private boolean           showChanges       = false ;

    /**
     * This will be the Prod Ind that we are starting with
     */
    private int               startProdInd      = 0 ;


    private boolean           startDraft        = false;

    /**
     * This is the Panel that we previous came from
     */
    private QuestPanel        previousPanel     = null ;

    /**
     * This will be the Editor in this Panel
     */
    private ProductEditPanel  editPanel         = null ;

    /**
     * This will be the ProductDraft we are currently working with.
     */
    private ProductDraft      productDraft      = null ;

    /**
     * This will be the ProductDraft we are currently working with.
     */
    //private Product           product           = null ;

    /*-----------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This will create a blank ProductHistoryPanel
     *
     *
     *
     *  @param Quest Panel
     *  @param
     *  @param
     *
     *  @return
     *//*---------------------------------------------------------*/
    public ProductHistoryPanel ( QuestPanel previousPanel ) {
        this.previousPanel = previousPanel ;
    }/* constructor */

    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This constructor will initialize our variables
     *  to read in the product specified
     *
     *
     *  @param Product
     *  @param QuestPanel  can be null
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public ProductHistoryPanel ( int prodInd, boolean draft, QuestPanel previousPanel ) {
        this.startDraft    = draft;
        this.startProdInd  = prodInd;
        this.previousPanel = previousPanel ;
    }/* constructor */


    /*------------------------------------------------------------*/
    /* initialize
     */
    /**
     *    This method will initialize our panel
     *
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return true
     *//*---------------------------------------------------------*/
    public boolean initialize ( ) {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create the controls                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        pb_CLOSE                = new ActionButton ( Str.getStr ( STR_CLOSE ),
                                                     ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),
                                                     Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
        pb_RESTORE              = new ActionButton ( Str.getStr ( STR_RESTORE ),
                                                     ImageSystem.getImageIcon ( this, ImageSystem.NEW ),
                                                     Str.getStr ( STR_RESTORE_HISTORY_PROD ) ) ; 

        productHistoryNavPanel  = new ProductHistoryNavPanel( this ) ;
        pb_FIND                 = new ToolBarButton ( Str.getStr ( STR_FIND ),
                                                      ImageSystem.getImageIcon ( this, ImageSystem.SEARCH ),
                                                      Str.getStr ( STR_FIND ) ) ;
        toolBar                 = new JToolBar( ) ;
        editPanel               = new ProductEditPanel ( ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our action listeners                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        pb_FIND.addActionListener    ( this ) ;
        pb_CLOSE.addActionListener   ( this ) ;
        pb_RESTORE.addActionListener ( this ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add the controls to our panel                           */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setLayout ( new BorderLayout ( ) ) ;
        setBackground( Color.white );
        toolBar.add ( pb_FIND ) ;
        add( editPanel );

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust properties                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        toolBar.setBorder ( null ) ;
        toolBar.setOpaque ( false ) ;
        //editPanel.setBackground ( Color.white ) ;

        return( true ) ;
    }/* initialize */

    /*-----------------------------------------------------------*/
    /* actionPerformed
     */
    /**
     *    This method will respond to buttons being
     *  pressed
     *
     *
     *  @param ActionEvent
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void actionPerformed ( ActionEvent e ) {

        if ( ( Qit.getEnabled() ) && ( e.getSource() instanceof DButton ) ) {
            DButton button = (DButton) e.getSource() ;
            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   PB_FIND                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( button == pb_FIND ) {
                findProductComplete( new ProductSearchDlg().getResult() );
                //ProductSearchDlg searchDlg = new ProductSearchDlg() ;

                //if ( searchDlg != null )
                //    searchDlg.addFindProductListener ( this ) ;

            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   PB_CLOSE                                                */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( button == pb_CLOSE )
                GUISystem.getParentDefWin ( this ).closePanel ( this, previousPanel ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   pb_RESTORE                                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( button == pb_RESTORE )
                restore() ;
        } /* if */
    }/* actionPerformed  */

    /*-----------------------------------------------------------*/
    /* refresh
     */
    /**
     *    This method will be called if the user hits the
     *  refresh button on the task bar. It will simply
     *  call refreshList
     *
     *  @param none
     *  @param none
     *  @param none
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void refresh ( ) {
        new Thread ( this, THREAD_REFRESH_PREVIEW ).start() ;
    }/* refresh */

    /*-----------------------------------------------------------*/
    /* run
     */
    /**
     *    This thread performs many operations. The operation
     *  done is determined by the thread name
     *
     *
     *  @param none
     *  @param none
     *  @param none
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void run ( ) {

        String operation = Thread.currentThread().getName() ;

        Qit.setEnabled ( false ) ;

        try {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   THREAD_REFRESH_PREVIEW                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( operation.equals ( THREAD_REFRESH_PREVIEW ) )
                refreshPreview() ;
        } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
        }
        Qit.setEnabled ( true ) ;
    }/* run */

    /*-----------------------------------------------------------*/
    /* findProductComplete
     */
    /**
     *   This method will be called when the user hits
     *  'Find' on the ProductSearch window.
     *
     *  @param none
     *  @param none
     *  @param none
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void findProductComplete( Vector vec ){ 

        ProductRow productRow = null;
        int returnCode = 0;

        if ( ( vec != null ) && ( vec.size() > 0 ) ) {
            productRow = ( (ProductRow)vec.elementAt(0) );
            returnCode = productRow.readProduct( true );

            if ( returnCode == 0 ) {
                if ( productRow.isDraft() ) {
                    ProductDraft pd  = ( (ProductDraft)productRow.getProduct() );
                    productHistoryNavPanel.setProdInd ( pd.getExistingProductInd(), false ) ;

                } else if ( !productRow.isDraft() ) {
                    com.ibm.nzna.projects.common.quest.product.Product p        = productRow.getProduct() ;
                    productHistoryNavPanel.setProdInd ( p.getProductInd(), true ) ;
                }/*if*/
            }/*if returncode*/
        }/*if vec*/
    }/* findProductComplete */

    /*-----------------------------------------------------------*/
    /* setProduct
     */
    /**
     *    This method will set the ProductDraft we are editing
     *
     *
     *
     *  @param ProductDraft
     *  @param none
     *  @param none
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setProduct ( ProductDraft productDraft ) {

        this.productDraft = productDraft ;

        if ( productDraft != null ) {
            new Thread ( this, THREAD_REFRESH_PREVIEW ).start() ;
        } /* if */
    }/* setProduct */

    /*------------------------------------------------------------*/
    /* refreshPreview
     */
    /**
     *    Sets the products for all preview fields
     *
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void refreshPreview ( ) {

        AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;

        if (  productDraft != null ) {
            parentWin.setStatus ( STR_PLEASE_WAIT ) ;
            editPanel.setEditable( false );
            editPanel.setProduct( productDraft );
        }/*if*/

        parentWin.setStatus ( Str.getStr ( STR_DONE ), 2 ) ;
    }/* refreshPreview */

    /*------------------------------------------------------------*/
    /* panelDeselected
     */
    /**
     *    This method will be called when we are
     *  deselected
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void panelDeselected( ) {
        GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
    }/* panelDeselected */

    /*------------------------------------------------------------*/
    /* panelSelected
     */
    /**
     *    This method will be called when we are
     *  selected
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void panelSelected( ) {
        GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 1 ) ;
    }/* panelSelected */

    /*------------------------------------------------------------*/
    /* copyText
     */
    /**
     *    This method will do nothing
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return boolean
     *//*---------------------------------------------------------*/
    public boolean copyText( ) {
        return( false ) ;
    }/* copyText */

    /*------------------------------------------------------------*/
    /* cutText
     */
    /**
     *    This method will do nothing
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return boolean
     *//*---------------------------------------------------------*/
    public boolean cutText( ) {
        return( false ) ;
    }/* cutText */

    /*------------------------------------------------------------*/
    /* pasteText
     */
    /**
     *    This method will do nothing
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return boolean
     *//*---------------------------------------------------------*/
    public boolean pasteText( ) {
        return( false ) ;
    }/* pasteText */

    /*------------------------------------------------------------*/
    /* getJComponent
     */
    /**
     *    This method will return an instance of this
     *  class
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return JComponent
     *//*---------------------------------------------------------*/
    public JComponent getJComponent( ) {
        return( this );
    }/* getJComponenet */

    /*------------------------------------------------------------*/
    /* getDefaultNavPanel
     */
    /**
     *    This method will return our default nav panel
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return NavPanel
     *//*---------------------------------------------------------*/
    public NavPanel getDefaultNavPanel ( ) {
        return( productHistoryNavPanel ) ;
    }/* getDefaultNavPanel */
    /*------------------------------------------------------------*/
    /* stop
     */
    /**
     *    This method will be called when we need
     *  to be closed
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return boolean
     *//*---------------------------------------------------------*/
    public boolean stop ( ) {
        panelDeselected() ;
        removeAll() ;
        return( Qit.getEnabled() ) ;
    }/* stop */

    /*------------------------------------------------------------*/
    /* shutdown
     */
    /**
     *    This method will return TRUE
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return boolean
     *//*---------------------------------------------------------*/
    public boolean shutdown () {
        return( Qit.getEnabled() ) ;
    }/* shutdown */

    /*------------------------------------------------------------*/
    /* start
     */
    /**
     *    This method will be called once we have
     *  been shown
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void start ( ) {

        AppDefaultWin parentWin = GUISystem.getParentDefWin ( this ) ;
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add our action buttons                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        parentWin.addActionComponent ( this, pb_CLOSE ) ;
        parentWin.addActionComponent ( this, pb_RESTORE ) ;
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Set the toolbar on                                      */
        /*                                                           */
        /*-----------------------------------------------------------*/
        parentWin.addToolbarButton ( toolBar, 1 ) ;

        if ( startProdInd != 0 ) {
            productHistoryNavPanel.setProdInd ( startProdInd, startDraft ) ;
        }
    }/* start */

    /*------------------------------------------------------------*/
    /* getIcon
     */
    /**
     *    This method will return our small icon
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return ImageIcon
     *//*---------------------------------------------------------*/
    public ImageIcon getIcon ( ) {
        return( ImageSystem.getImageIcon ( this, ImageSystem.PRODUCT_HISTORY_SMALL ) ) ;
    }/* getIcon */

    /*------------------------------------------------------------*/
    /* getName
     */
    /**
     *    This method will return our name
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String getName ( ) {
        return( Str.getStr ( STR_PRODUCT_HISTORY ) ) ;
    }/* getName */

    /*------------------------------------------------------------*/
    /* bookmark
     */
    /**
     *    This method will be called if the user
     *  has hit the 'Bookmark' button.
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return boolean
     *//*---------------------------------------------------------*/
    public boolean bookmark ( ) {
        return( false ) ;
    }/* bookmark */

    /*------------------------------------------------------------*/
    /* restore
     */
    /**
     *    This method will restore a product to it's
     *  draft state if no other draft is open for the product
     *
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void restore ( ) {

        GUISystem.getParentDefWin ( this ).setStatus ( STR_RESTORING ) ;

        if ( productDraft != null ) {
            SQLMethod sqlMethod  = new SQLMethod ( DatabaseSystem.QUEST, "ProductHistoryPanel.restore", 5 ) ;
            boolean   restore    = false ;
            int       returnCode = 0;

            try {
                Statement stmt          = sqlMethod.createStatement() ;
                String    comments      = null ;
                String    lastTouchedBy = null ;


                /*-------------------------------------------------------------*/
                /*                                                             */
                /*   First, make sure that there is not an open draft already  */
                /*                                                             */
                /*-------------------------------------------------------------*/
                if ( stmt != null ) {
                    ResultSet rs = stmt.executeQuery ( "SELECT LASTTOUCHEDBY, LASTTOUCHEDON "+
                                                       "FROM PRODRAFT.PRODUCT "+
                                                       "WHERE STOPDATE IS NULL AND "+
                                                       "EXISTINGPRODUCTIND = "+ productHistoryNavPanel.getProdInd() ) ;
                    if ( rs.next() ) {
                        lastTouchedBy    = rs.getString(1).trim() ;
                        String timeStamp = DateSystem.prettyDateFromStamp ( rs.getString(2).trim() ) ;

                        GUISystem.printBox ( Str.getStr ( STR_INFO ),
                                             Str.getStr ( STR_CANNOT_RESTORE_PRODUCT )+" "+
                                             lastTouchedBy +" "+
                                             Str.getStr ( STR_LAST_TOUCHED_ON )+" "+
                                             timeStamp ) ;
                        restore = false ;
                    } else
                        restore = true ;

                    rs.close() ;

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Now, if we still can, restore the draft from history    */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( restore ) {                                         

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   First, read in the comments from the previous edit      */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        rs = stmt.executeQuery ( "SELECT COMMENTS "+
                                                 "FROM PRODRAFT.PRODUCT "+
                                                 "WHERE PRODUCTIND = "+productDraft.getProductInd() ) ;

                        if ( rs.next() )
                            comments = rs.getString(1).trim() ;
                        else
                            comments = "" ;

                        rs.close() ;

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Add a comment                                           */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        comments+="\n<"+
                                  Str.getStr ( STR_PRODUCT_RESTORED )+" "+
                                  Str.getStr ( STR_LAST_TOUCHED_BY )+" "+
                                  UserSystem.getUserId()+" "+
                                  Str.getStr ( STR_LAST_TOUCHED_ON )+" "+
                                  DateSystem.getDate ( CDate.DATE_DATE )+">\n" ;


                        stmt.executeUpdate ( "UPDATE PRODRAFT.PRODUCT "+
                                             "SET   STOPDATE      = NULL, "+
                                             "      COMMENTS       = '"+comments+"', "+
                                             "      LASTTOUCHEDBY = '"+UserSystem.getUserId()+"', "+
                                             "      LASTTOUCHEDON = CURRENT TIMESTAMP "+
                                             "WHERE PRODUCTIND    = "+productDraft.getProductInd() ) ;

                        GUISystem.printBox ( STR_INFO, STR_PRODUCT_RESTORED ) ;
                    } /* if restore */
                } /* if Stmt */

            } catch ( Exception e ) {
                sqlMethod.rollBack() ;
                LogSystem.log ( 1, e, false ) ;
                GUISystem.printBox ( STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
            }

            sqlMethod.close() ;
        } else
            GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;

        GUISystem.getParentDefWin ( this ).setStatus ( null ) ;
    }/* restore */

    /*------------------------------------------------------------*/
    /* getHelpFilename
     */
    /**
     *    This method will return our help filename
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String getHelpFilename () {
        return( "Products/ProdHistory.html" ) ;
    }

}/* ProductHistoryPanel */
