package com.ibm.nzna.projects.qit.product.manager ;   
/*-----------------------------------------------------------*/
/*   CountryMapDlg                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CountryMapDlg                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.gui.* ;
import com.ibm.nzna.projects.common.quest.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
/**
 * This window allows the user to make a mapping between OPIC/M 
 * Geographies/Countries and Quest countries
 *
 *
 * @version 1.0, 9/3/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CountryMapDlg                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class CountryMapDlg extends JDialog implements ActionListener, AppConst, ProductConst {


   /**
     *  This is the mapping that we are editing
     **/
   private CountryMappingRow mapRow = null ;

   /**
     *  This is the Ok Button
     **/
   private DButton pb_OK      = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     *  This is the Cancel button
     **/
   private DButton pb_CANCEL  = new DButton ( Str.getStr ( STR_CANCEL ) ) ;

   /**
     *  This is the text for the OPIC/M Geo
     **/
   private JLabel st_OGEOGRAPHY   = new JLabel ( "OGeography" ) ;

   /**
     *  This is the text for the OPIC/M Country
     **/
   private JLabel st_OCOUNTRY = new JLabel ( "OCountry" ) ;

   /**
     *  This is the JLabel for the Quest Country
     **/
   private JLabel st_COUNTRY = new JLabel ( Str.getStr ( STR_COUNTRY ) ) ;

   /**
     *  This is the Drop down for OPIC/M Brands
     **/
   private JComboBox cb_OGEOGRAPHY = new JComboBox ( ) ;

   /**
     *  This is the Drop Down for the OPIC/M Families
     **/
   private JComboBox cb_OCOUNTRY = new JComboBox ( ) ;

   /**
     *  This is the Drop Down for the Quest Brand
     **/
   private JComboBox cb_COUNTRY = new JComboBox ( ) ;

   /**
     *  This button creates a new OPIC/M Geography
     **/
   private HotLinkLabel pb_NEW_GEOGRAPHY = new HotLinkLabel ( Str.getStr ( STR_NEW ) ) ;

   /**
     *  THis button creates a new OPIC/M Country
     **/
   private HotLinkLabel pb_NEW_COUNTRY = new HotLinkLabel ( Str.getStr ( STR_NEW ) ) ;

   /**
     *  Default constructor shows the dialog and does not return
     * until the user closes the window
     **/
   public CountryMapDlg ( Frame parent, CountryMappingRow mapRow ) {
      super ( parent, Str.getStr ( STR_DATA_MAPPING ), true ) ;
      this.mapRow = mapRow ;
      initialize () ;

      setVisible ( true ) ;
   }

   /**
     *  This method will initialize and add our GUI
     **/
   private void initialize () {
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setResizable ( false ) ;  
      cb_OGEOGRAPHY.setModel ( new DefaultComboBoxModel() ) ;
      cb_OCOUNTRY.setModel ( new DefaultComboBoxModel() ) ;
      cb_COUNTRY.setModel ( new DefaultComboBoxModel() ) ;
      pb_NEW_GEOGRAPHY.showSelection ( false );
      pb_NEW_COUNTRY.showSelection ( false );       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add default data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      WinUtil.setJComboBoxData ( cb_OGEOGRAPHY, TypeCategory.getCategories ( TypeCategory.LEVEL_OPICM_GEOGRAPHY ) ) ;
      WinUtil.setJComboBoxData ( cb_COUNTRY,    TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ) ;
      WinUtil.setJComboBoxData ( cb_OCOUNTRY,   createEmptyList() ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener            ( this ) ;
      pb_CANCEL.addActionListener        ( this ) ;
      cb_OGEOGRAPHY.addActionListener    ( this ) ;
      pb_NEW_GEOGRAPHY.addActionListener ( this ) ;
      pb_NEW_COUNTRY.addActionListener   ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_OGEOGRAPHY ) ;
      contentPane.add ( cb_OGEOGRAPHY ) ;
      contentPane.add ( pb_NEW_GEOGRAPHY ) ;
      contentPane.add ( st_OCOUNTRY ) ;
      contentPane.add ( cb_OCOUNTRY ) ;
      contentPane.add ( pb_NEW_COUNTRY ) ;
      contentPane.add ( st_COUNTRY ) ;
      contentPane.add ( cb_COUNTRY ) ;
      contentPane.add ( pb_OK ) ;
      contentPane.add ( pb_CANCEL ) ;                                       


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Select the value from the mapping we were passed        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( mapRow != null ) {
         if ( mapRow.oGeo != null )
            cb_OGEOGRAPHY.setSelectedItem ( mapRow.oGeo ) ;
         if ( mapRow.oCountry != null )
            cb_OCOUNTRY.setSelectedItem ( mapRow.oCountry ) ;
         if ( mapRow.countryRec != null )
            cb_COUNTRY.setSelectedItem ( mapRow.countryRec ) ;


         cb_OGEOGRAPHY.setEnabled  ( mapRow.oGeo  == null ) ;
         cb_OCOUNTRY.setEnabled ( mapRow.oCountry == null ) ;
      }


      setSize ( 400, 200 );
      WinUtil.centerWindow ( this ) ;
   }


   /**
     *  This method will lay out and position all of our controls
     **/
   public void doLayout ( ) {
      Dimension size      = getSize () ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = 35 ;
      int       y         = 5 ;

      super.doLayout() ;

      st_OGEOGRAPHY.setBounds ( x, y, 100, rowHeight ) ;
      cb_OGEOGRAPHY.setBounds ( x+100, y, 200, rowHeight ) ;
      pb_NEW_GEOGRAPHY.setBounds ( x+305, y, 70, rowHeight ) ;
      y+=rowHeight+5 ;

      st_OCOUNTRY.setBounds ( x, y, 100, rowHeight ) ;
      cb_OCOUNTRY.setBounds ( x+100, y, 200, rowHeight ) ;
      pb_NEW_COUNTRY.setBounds ( x+305, y, 70, rowHeight ) ;
      y+=rowHeight+5 ;

      st_COUNTRY.setBounds ( x, y, 100, rowHeight ) ;
      cb_COUNTRY.setBounds ( x+100, y, 200, rowHeight ) ;
      y+=rowHeight+5 ;

      pb_OK.setBounds ( 5, size.height-60, 75, 25 ) ;
      pb_CANCEL.setBounds ( 85, size.height-60, 75, 25 ) ;
   }


   /**
     *  This method will be called when the user selects a
     * drop down
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   cb_OGEOGRAPHY                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == cb_OGEOGRAPHY )
         refreshOCountries ( (TypeCategoryRec)cb_OGEOGRAPHY.getSelectedItem() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_OK ) {
         if ( saveData() )
            dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CANCEL )
         dispose() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_NEW_GEOGRAPHY                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_NEW_GEOGRAPHY )
         createNewGeography() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_NEW_COUNTRY                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_NEW_COUNTRY )
         createNewCountry() ;

   }


   /**
     *  This method will return an empty list with a 'Select an Item'
     * in it
     **/
   private Vector createEmptyList () {
      Vector list = new Vector () ;
      list.addElement ( Str.getStr ( STR_SELECT_ITEM ) ) ;
      return( list ) ;
   }

   /**
     *  This method will determine if an item in a JComboBox is valid
     * for a selection. It determines this by looking if the selected item
     * is an instance of TypeCategoryRec
     **/
   private boolean selectionValid ( JComboBox comboBox ) {
      return( comboBox.getSelectedItem() instanceof TypeCategoryRec ) ;
   }


   /**
     *  This method will refresh the OPIC/M Families under the brand
     * passed in
     **/
   private void refreshOCountries ( TypeCategoryRec geo ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, recreate them                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( geo != null ) {
         WinUtil.setJComboBoxData ( cb_OCOUNTRY, TypeCategory.getChildCategories ( geo.getInd() )  ) ;
      } else {
         WinUtil.setJComboBoxData ( cb_OCOUNTRY, createEmptyList() ) ;
      }

      getContentPane().add ( cb_OCOUNTRY ) ;
      doLayout() ;
   }


   /**
     *  This method will save our data and will return TRUE if all data is present
     **/
   private boolean saveData () {
      TypeCategoryRec    oGeo      = (TypeCategoryRec)cb_OGEOGRAPHY.getSelectedItem() ;
      TypeCategoryRec    oCountry  = (TypeCategoryRec)cb_OCOUNTRY.getSelectedItem() ;
      TypeCountryCodeRec country   = (TypeCountryCodeRec)cb_COUNTRY.getSelectedItem() ;
      boolean            saved     = false ;

      if ( ( oGeo != null ) &&
           ( oCountry != null ) &&
           ( country != null ) ) {
         StatusWin statusWin = new StatusWin ( GUISystem.getParentDefWin ( this ) ) ;

         statusWin.setVisible ( true ) ;
         statusWin.setText ( "Creating OPIC/M Geo/Country Mapping" ) ;
         mapRow.oGeo         = oGeo ;
         mapRow.oCountry     = oCountry ;
         mapRow.countryRec   = country ;
         saved = saveMappingToDatabase() ;

         if ( saved ) {
            statusWin.setText ( "Converting all current misfits ..." ) ;
            saved = convertMisfits ( statusWin ) ;
         }

         statusWin.dispose() ;

      } else
         GUISystem.printBox ( this, "Info", "Please select a value for all fields" ) ;

      return( saved ) ;
   }


   /**
     *  This method will save the mapping to the OPICM.BRANDFAMILYMAP table
     **/
   private boolean saveMappingToDatabase () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "saveMappingToDatabase", 5 ) ;
      boolean   rc        = false ;

      try {
         Statement stmt = sqlMethod.createStatement() ;

         stmt.executeUpdate ( "DELETE FROM OPICM.GEOCOUNTRYMAP WHERE TYPECATIND = "+mapRow.oCountry.getInd() ) ;
         stmt.executeUpdate ( "INSERT INTO OPICM.GEOCOUNTRYMAP VALUES ( "+mapRow.oCountry.getInd()+", "+mapRow.countryRec.getInd()+")" ) ;

         rc = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( this, STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }


   /**
     *  This method will add an OPIC/M Geography to the database
     **/
   private void createNewGeography () {
      String geo = new SingleEntryDlg ( this, 
                                        Str.getStr ( STR_NEW ), 
                                        Str.getStr ( STR_GEOGRAPHY ), 
                                        "", 
                                        new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN,
                                                           254 ) ).getResult() ;

      if ( ( geo != null ) && ( geo.length() > 0 ) ) {
         TypeCategoryRec geoRec = TypeCategory.createTypeCategory ( 0,
                                                                    geo,
                                                                    TypeCategory.LEVEL_OPICM_GEOGRAPHY,
                                                                    0,
                                                                    "N",
                                                                    UserSystem.getUserId() ) ;

         TypeCategory.createDraftTypeCategory ( geoRec.getInd(),
                                                0,
                                                geo,
                                                TypeCategory.LEVEL_OPICM_GEOGRAPHY,
                                                0,
                                                "N",
                                                UserSystem.getUserId() ) ;

         if ( ( geoRec != null ) && ( TypeCategory.writeToDatabase() == 0 ) ) {
            ((DefaultComboBoxModel)cb_OGEOGRAPHY.getModel()).addElement ( geoRec ) ;
            cb_OGEOGRAPHY.setSelectedItem ( geoRec ) ;
         }

      }
   } /* createNewGeography  */


   /**
     *  This method will add an OPIC/M Country to the database
     **/
   private void createNewCountry () {
      String country = new SingleEntryDlg ( this, 
                                            Str.getStr ( STR_NEW ), 
                                            Str.getStr ( STR_COUNTRY ), 
                                            "", 
                                            new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN,
                                                               254 ) ).getResult() ;

      if ( ( country != null ) && ( country.length() > 0 ) ) {
         TypeCategoryRec oGeoRec  = ((TypeCategoryRec)cb_OGEOGRAPHY.getSelectedItem());
         TypeCategoryRec countryRec = TypeCategory.createTypeCategory ( oGeoRec.getInd(),
                                                                        country,
                                                                        TypeCategory.LEVEL_OPICM_COUNTRY,
                                                                        0,
                                                                        "N",
                                                                        UserSystem.getUserId() ) ;

         TypeCategory.createDraftTypeCategory ( countryRec.getInd(),
                                                oGeoRec.getInd(),
                                                country,
                                                TypeCategory.LEVEL_OPICM_COUNTRY,
                                                0,
                                                "N",
                                                UserSystem.getUserId() ) ;

         if ( ( countryRec != null ) && ( TypeCategory.writeToDatabase() == 0 ) ) {
            ((DefaultComboBoxModel)cb_OCOUNTRY.getModel()).addElement ( countryRec ) ;
            cb_OCOUNTRY.setSelectedItem ( countryRec ) ;
         }

      }
   } /* createNewCountry  */



   /**
     *  This method will convert all current misfits that fall under this
     * new mapping
     **/
   private boolean convertMisfits ( StatusWin statusWin ) {
      Vector    misfits   = getMisfits ( statusWin ) ;
      boolean   rc        = true ;

      if ( misfits.size() > 0 ) {
         int             i              = 0 ;
         int             size           = misfits.size() ;
         ProductDraft    draft          = null ;
         int             converted      = 0 ;
         int             readRC         = 0 ;                  

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Ok, now process each misfit                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( ( i < size ) && ( rc ) ) {
            statusWin.setText ( "Reading Misfit "+(i+1)+" of "+size ) ;
            draft = new ProductDraft ( ((Integer)misfits.elementAt(i)).intValue() ) ;
            readRC = draft.readFromDatabase() ;

            switch ( readRC ) {
               
               case 0 :
                  int       saveRC     = 0 ;                          
                  LocaleRec localeRec  = draft.getLocale() ;

                  localeRec.addCountry ( mapRow.countryRec ) ;
                  draft.setLocale ( localeRec ) ;

                  statusWin.setText ( "Saving Misfit "+(i+1)+" of "+size ) ;
                  saveRC = draft.writeToDatabase() ;
                  try {
                     draft.checkOut ( null ) ;
                  } catch ( Exception e ) {
                     LogSystem.log ( 1, e, false ) ;
                  }

                  if ( saveRC != 0 )
                     LogSystem.log ( 1, "Couldn't save draft to database. ProductInd:"+((Integer)misfits.elementAt(i)).intValue()+". rc:"+saveRC ) ;
                  else
                     converted++ ;
                  break ;

               default:
                  LogSystem.log ( 1, "Couldn't read in misfit productInd "+((Integer)misfits.elementAt(i)).intValue()+". rc:"+rc ) ;
                  rc = false ;
                  break ;
            }

            i++ ;
         }

         GUISystem.printBox ( this, "Info", "Converted "+converted+" misfits of "+size ) ;
      }

      return( rc ) ;   
   }


   /**
     *  This method will find all misfits and will return a Vector of their
     * product indicators
     **/
   private Vector getMisfits ( StatusWin statusWin ) {
      Vector     misfits   = new Vector ( 1 ) ;
      SQLMethod  sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getMisfits", 5 ) ;

      try {
         int       i    = 0 ;
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT A.PRODUCTIND "+
                                              "FROM PRODRAFT.CATEGORY A, "+
                                              "     PRODRAFT.PRODUCT B "+
                                              "WHERE A.TYPECATIND = "+mapRow.oCountry.getInd()+" AND "+
                                              "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      B.MISFIT = 'Y'" ) ;

         while ( rs.next() ) {
            misfits.addElement ( new Integer ( rs.getInt(1) ) ) ;
            i++ ;
            statusWin.setText ( "Found Misfits ... "+i ) ;
         }

         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         misfits = new Vector ( 1 ) ;
         GUISystem.printBox ( this, STR_ERROR, STR_ERROR_IN_LAST_ACTION ) ;
      }

      return( misfits ) ;
   }

} /* CountryMapDlg */
