package com.ibm.nzna.projects.qit.product.manager ;     
/*-----------------------------------------------------------*/
/*   CustomViewPanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: CustomViewPanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.product.* ;

/**
 * This is the CustomView panel in ProductManagement. 
 *
 *
 * @version 1.0, 8/30/2001 
 * @author  Venu Sanka
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   CustomViewPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class CustomViewPanel extends JPanel implements CustomViewListener, NavPanel, ProductConst, 
    AppConst, ActionListener {



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   

   private NavListGroup    customQueryView                  = null ;
   private NavList         navList                          = null ;
   private Vector          customViewComponents             = new Vector ( 1, 1 ) ;

   private ProductManagementPanel productManagementPanel    = null ;





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method does nothing but assign our
    *  local vars
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomViewPanel ( ProductManagementPanel productManagementPanel ) {
      this.productManagementPanel = productManagementPanel ;

      initialize() ;
   }





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our panel. This includes
    *  creating all of our controls and attaching
    *  them to this panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {
      JScrollPane scrollPane = null ;     
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      navList    = new NavList() ;
      scrollPane = new JScrollPane ( navList ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our views                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      createCustomQueryViews ( ) ;

      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Action Listeners                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //cb_BRANDS.addActionListener ( this ) ;
      CustomViewSystem.addCustomViewListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout(5,5) ) ;
      add ( scrollPane, BorderLayout.CENTER ) ;


   }




   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are
    *  showing. We will do nothing here
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {

      }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will remove all of our controls
    *  and will free all of our resources
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
      CustomViewSystem.removeCustomViewListener ( this ) ;
      removeAll() ;
   }


   /*------------------------------------------------------------*/
   /* getTitle
    */
   /**
    *    This method will return the title of this nav
    *  panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_VIEWS ) ) ;
   }




   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method will simply return this class
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   

   /*------------------------------------------------------------*/
   /* createCustomQueryViews
    */
   /**
    *    This method will create all of the necessary
    *  views for the custom queries
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void createCustomQueryViews ( ) {            
      Vector viewVec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the Base views                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      customQueryView = new NavListGroup ( Str.getStr ( STR_CUSTOM_VIEWS ) ) ;
      navList.add ( customQueryView ) ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the main custom query views                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      
      viewVec = CustomViewSystem.getViews ( CUSTOMVIEW_PRODUCT ) ;
      addCustomQueries ( customQueryView, viewVec ) ;
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add action listeners to all nav lists                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      customQueryView.addActionListener  ( this ) ;
      
   } /* createCustomQueryViews  */





   /*------------------------------------------------------------*/
   /* addCustomQueries
    */
   /**
    *    This method will add custom queries to
    *  the nav list specified. The queries should be in
    *  the queryVec passed
    *
    *  @param navList The nav list to add the views to
    *  @param queryVec The vector containing CustomView records
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void addCustomQueries ( NavListGroup navList, Vector queryVec ) {
      if ( ( queryVec != null ) && ( queryVec.size() > 0 ) ) {
         int           i            = 0 ;
         int           size         = queryVec.size() ;
         HotLinkLabel  hotLinkLabel = null ;
         HotLinkGroup 	group	= new HotLinkGroup () ;

         while ( i < size ) {

            navList.add ( hotLinkLabel = new HotLinkLabel ( ((CustomView)queryVec.elementAt(i)).getDescript() ,
                                                            null,
                                                            "CUSTOM_VIEW" ) ) ;
            group.add( hotLinkLabel ) ;
            customViewComponents.addElement ( hotLinkLabel ) ;
            i++ ;
         }
      }
   }

   



   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event is fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

     if ( e.getSource() instanceof HotLinkLabel ) {
         String actionCommand = ((HotLinkLabel)e.getSource()).getActionCommand() ;                                        

         /*-------------------------------------------------------------------------*/
         /*                                                                         */
         /*   If this is a custom view, it's action command will be CUSTOM_VIEW     */
         /*                                                                         */
         /*-------------------------------------------------------------------------*/
         if ( actionCommand.equals ( "CUSTOM_VIEW" ) )
            productManagementPanel.setCustomView ( ((HotLinkLabel)e.getSource()).getText() );
     }

      
   } /* actionPerformed  */



   /*------------------------------------------------------------*/
   /* customViewRemoved
    */
   /**
    *    This method will be called when the user
    *  erases a custom view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewRemoved ( CustomView customView ) {
      HotLinkLabel label = hotLinkFromCustomView ( customView ) ;
      
      if ( label != null ) 
         label.getParent().remove ( label ) ;
      revalidate() ;
   }





   /*------------------------------------------------------------*/
   /* customViewAdded
    */
   /**
    *    This method is called when the user adds
    *  a custom view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewAdded ( CustomView customView ) {
      HotLinkLabel hotLinkLabel = null ;

       switch ( customView.getViewType() ) {
        
         case CUSTOMVIEW_PRODUCT :
            customQueryView.add ( hotLinkLabel = new HotLinkLabel ( customView.getDescript(), null, "CUSTOM_VIEW" ) ) ;
            break ;

         }

      if ( hotLinkLabel != null )
         customViewComponents.addElement ( hotLinkLabel ) ;
      revalidate() ;
   }




   /*------------------------------------------------------------*/
   /* customViewUpdated
    */
   /**
    *    This method will be called when a custom view
    *  is updated
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void customViewUpdated ( CustomView customView ) {
      HotLinkLabel label = hotLinkFromCustomView ( customView ) ;
      
      if ( label != null ) 
         label.setText ( customView.getDescript() ) ;
      revalidate() ;
   }

   




   



   
   /*------------------------------------------------------------*/
   /* hotLinkFromCustomView
    */
   /**
    *    This method will go through all of the
    *  custom view components and will return
    *  the HotLinkLabel for the custom view
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return HotLinkLabel
    *//*---------------------------------------------------------*/
   public HotLinkLabel hotLinkFromCustomView ( CustomView customView ) {
      int          i        = 0 ;
      int          size     = customViewComponents.size() ;
      HotLinkLabel retLabel = null ;

      while ( ( retLabel != null ) && ( i < size ) ) {
         if ( ((HotLinkLabel)customViewComponents.elementAt(i)).getText().equals ( customView.getDescript() ) )
            retLabel = (HotLinkLabel)customViewComponents.elementAt(i) ;

         i++ ;
      }

      return( retLabel ) ;
   }

   } /* CustomViewPanel */
