package com.ibm.nzna.projects.qit.product.manager ;
/*-----------------------------------------------------------*/
/*   NewProductDlg                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: NewProductDlg                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.io.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.* ;
/**
 * This dialog presents the user with the different ways to 
 * create a product. These ways include:<br><br>
 *
 * New Blank Product<br>
 * Product from a template<br>
 * Import Product<br><br>
 *
 * If a new blank product is selected, then this dialog also
 * asks which product type.
 *
 * @version 1.0, 3/9/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   NewProductDlg                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class NewProductDlg extends JDialog implements  Runnable, MouseListener, TypeListListener, ListSelectionListener, AppConst, ActionListener, ProductConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Thread Names                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  This thread will read all of the templates and populates 
     * the list of templates
     **/
   private static final String THREAD_POPULATE_TEMPLATES = "TPT" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Constants                                               */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int NEWPROD_CANCEL     = 0 ;
   public static final int NEWPROD_BLANK      = 1 ;
   public static final int NEWPROD_TEMPLATE   = 2 ;
   public static final int NEWPROD_IMPORT     = 3 ;
   public static final int IMPORT_CREATE_UPDATE = 1 ;
   public static final int IMPORT_CREATE        = 2 ;
   public static final int IMPORT_UPDATE        = 3 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JPanel        newTypePanel              = null ;
   private JPanel        prodTypePanel             = null ;
   private JPanel        templatePanel             = null ;
   private JPanel        currentPanel              = null ;
   private JPanel        importPanel               = null ;

   private JLabel        st_IMPORT_FILE            = null ;
   private JLabel        st_IMPORT_TYPE            = null ;
   private JLabel        st_PRODUCT_TYPE           = null ;
   private JRadioButton  rb_CREATE_UPDATE          = null ;
   private JRadioButton  rb_CREATE_ONLY            = null ;
   private JRadioButton  rb_UPDATE_ONLY            = null ;
   private DList         importProductTypes        = null ;
   private JScrollPane   importProductTypesScroll  = null ;
   private DButton       pb_BROWSE                 = null ;
   private JTextField    ef_IMPORT_FILE            = null ;
   private DButton       pb_OK_IMPORT              = null ;

   private ButtonPanel   buttonPanel               = null ;
   private JLabel        st_HELP                   = null ;
   private JTextArea     detailedHelp              = null ;
   private JLabel        st_IMAGE                  = null ;
   private DButton       pb_NEWPROD                = null ;
   private DButton       pb_TEMPLATE               = null ;
   private DButton       pb_IMPORT                 = null ;
   private DButton       pb_CANCEL                 = null ;
   private DButton       pb_OK_PROD_TYPE           = null ;
   private DButton       pb_OK_PROD_TEMPLATE       = null ;
   private DList         productTypes              = null ;
   private JTextArea     typeOutput                = null ;
   private JScrollPane   productTypeScroll         = null ;
   private JScrollPane   outputScroll              = null ;
   private Color         buttonDefaultBackground   = Color.gray ;
   private Color         buttonHighlightBackground = new Color ( 220, 220, 220 ) ;
   private ProductTable  templates                 = null ;
   private CardLayout    cardLayout                = null ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Vars                                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int             newProdType     = 0 ;
   private TypeProductRec  typeProductRec  = null ;
   private ProductRow      template        = null ;




   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create but not show our
    *  window. To show the window and wait for
    *  user input, please call the 'getResult' method
    *
    *  @param frame the parent AppDefaultWin
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public NewProductDlg ( AppDefaultWin frame ) {
      super ( frame, Str.getStr ( STR_NEW_PRODUCT ), true ) ;
      Container   contentPane = getContentPane() ;     
      ButtonGroup buttonGroup = new ButtonGroup () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      newTypePanel       = new JPanel () ;
      prodTypePanel      = new JPanel () ;    
      templatePanel      = new JPanel () ;
      importPanel        = new JPanel () ;

      st_IMAGE           = new JLabel  ( ImageSystem.getImageIcon ( this, ImageSystem.NEW_PRODUCT_WIZARD ) ) ;
      st_HELP            = new JLabel  ( Str.getStr ( STR_CREATE_NEW_PRODUCT_FROM ) ) ;
      detailedHelp       = new JTextArea () ;

      st_IMPORT_FILE            = new JLabel ( Str.getStr ( STR_IMPORT_FILE ) ) ;
      st_IMPORT_TYPE            = new JLabel ( Str.getStr ( STR_IMPORT_TYPE ) ) ;
      st_PRODUCT_TYPE           = new JLabel ( Str.getStr ( STR_PRODUCT_TYPE ) ) ;
      rb_CREATE_UPDATE          = new JRadioButton ( Str.getStr ( STR_INSERT_UPDATE ) ) ;
      rb_CREATE_ONLY            = new JRadioButton ( Str.getStr ( STR_INSERT ) ) ;
      rb_UPDATE_ONLY            = new JRadioButton ( Str.getStr ( STR_UPDATE ) ) ;
      importProductTypes        = new DList ( TypeList.getInstance().getTypeList ( TypeList.TYPE_PRODUCT ) ) ;
      importProductTypesScroll  = new JScrollPane ( importProductTypes ) ;
      ef_IMPORT_FILE            = new JTextField () ;
      pb_BROWSE                 = new DButton ( Str.getStr ( STR_BROWSE ) ) ;
      pb_OK_IMPORT              = new DButton ( Str.getStr ( STR_OK ) ) ;

      pb_NEWPROD          = new DButton ( Str.getStr ( STR_BLANK_PRODUCT ), ImageSystem.getImageIcon( this, ImageSystem.NEWBLANK ) ) ;
      pb_TEMPLATE         = new DButton ( Str.getStr ( STR_TEMPLATE ),      ImageSystem.getImageIcon( this, ImageSystem.FROMTEMPLATE ) ) ;
      pb_IMPORT           = new DButton ( Str.getStr ( STR_IMPORT ),        ImageSystem.getImageIcon( this, ImageSystem.DOCIMPORT ) ) ;
      pb_CANCEL           = new DButton ( Str.getStr ( STR_CANCEL ) ) ;

      buttonPanel         = new ButtonPanel ();
      typeOutput          = new JTextArea () ;
      productTypes        = new DList() ;
      pb_OK_PROD_TYPE     = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_OK_PROD_TEMPLATE = new DButton ( Str.getStr ( STR_OK ) ) ;
      outputScroll        = new JScrollPane ( typeOutput ) ;
      productTypeScroll   = new JScrollPane ( productTypes ) ;
      templates           = new ProductTable () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      productTypes.addActionListener         ( this ) ;
      pb_NEWPROD.addActionListener           ( this ) ;
      pb_TEMPLATE.addActionListener          ( this ) ;
      pb_IMPORT.addActionListener            ( this ) ;
      pb_CANCEL.addActionListener            ( this ) ;        
      pb_OK_PROD_TYPE.addActionListener      ( this ) ;
      pb_OK_PROD_TEMPLATE.addActionListener  ( this ) ;
      pb_NEWPROD.addMouseListener            ( this ) ;
      pb_TEMPLATE.addMouseListener           ( this ) ;
      pb_IMPORT.addMouseListener             ( this ) ;
      productTypes.addListSelectionListener  ( this ) ;
      templates.addListSelectionListener     ( this ) ;
      templates.addActionListener            ( this ) ;
      pb_BROWSE.addActionListener            ( this ) ;
      pb_OK_IMPORT.addActionListener         ( this ) ;
      importProductTypes.addActionListener   ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_NEWPROD.setHorizontalAlignment   ( JButton.CENTER ) ;
      pb_TEMPLATE.setHorizontalAlignment ( JButton.CENTER ) ;
      pb_IMPORT.setHorizontalAlignment  ( JButton.CENTER ) ;
      TypeList.getInstance().refreshList ( productTypes, TypeList.TYPE_PRODUCT, this ) ;
      typeOutput.setLineWrap ( true ) ;
      typeOutput.setWrapStyleWord ( true ) ;
      typeOutput.setEditable ( false ) ;
      typeOutput.setFont ( FontSystem.defaultFont ) ;                        
      detailedHelp.setLineWrap ( true ) ;
      detailedHelp.setWrapStyleWord ( true ) ;
      detailedHelp.setEditable ( false ) ;
      detailedHelp.setFont ( FontSystem.defaultFont ) ;                        
      detailedHelp.setBorder ( GUISystem.emptyBorder ) ;
      detailedHelp.setOpaque ( false ) ;
      buttonDefaultBackground = pb_NEWPROD.getBackground() ;
      pb_NEWPROD.setToolTipText ( Str.getStr ( STR_NEW_BLANK_PRODUCT_HELP ) ) ;
      pb_TEMPLATE.setToolTipText ( Str.getStr ( STR_NEW_TEMPLATE_PRODUCT_HELP ) ) ;
      pb_IMPORT.setToolTipText ( Str.getStr ( STR_IMPORT_PRODUCT_HELP ) ) ;
      pb_OK_PROD_TYPE.setEnabled ( false ) ;
      pb_OK_PROD_TEMPLATE.setEnabled ( false ) ;
      rb_CREATE_UPDATE.setSelected ( true ) ;

      buttonGroup.add ( rb_CREATE_UPDATE ) ;
      buttonGroup.add ( rb_CREATE_ONLY ) ;
      buttonGroup.add ( rb_UPDATE_ONLY ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add controls the the newTypePanel                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      newTypePanel.setLayout ( new GridLayout( 5, 1, 15, 5 ) ) ;
      newTypePanel.add ( pb_NEWPROD ) ;
      newTypePanel.add ( pb_TEMPLATE ) ;
      newTypePanel.add ( pb_IMPORT ) ;
      newTypePanel.add ( detailedHelp ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to the prodTypePanel                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      prodTypePanel.setLayout ( null ) ;
      prodTypePanel.add ( productTypeScroll ) ;
      prodTypePanel.add ( outputScroll ) ;                             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to the Template panel                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      templatePanel.setLayout ( new BorderLayout() ) ;
      templatePanel.add ( templates ) ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADd controls to the Import panel                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      importPanel.setLayout ( null ) ;
      importPanel.add ( st_IMPORT_FILE ) ;
      importPanel.add ( ef_IMPORT_FILE ) ;
      importPanel.add ( pb_BROWSE ) ;
      importPanel.add ( st_IMPORT_TYPE ) ;
      importPanel.add ( rb_CREATE_UPDATE ) ;
      importPanel.add ( rb_CREATE_ONLY ) ;
      importPanel.add ( rb_UPDATE_ONLY ) ;
      importPanel.add ( st_PRODUCT_TYPE ) ;
      importPanel.add ( importProductTypesScroll ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign the current panel to the newTypePanel            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      currentPanel = new JPanel () ;
      currentPanel.setLayout ( cardLayout = new CardLayout ( 0, 0 ) ) ;
      currentPanel.add ( newTypePanel,   "newTypePanel" ) ;
      currentPanel.add ( templatePanel,  "templatePanel" ) ;
      currentPanel.add ( prodTypePanel,  "prodTypePanel" )  ;                         
      currentPanel.add ( importPanel,    "importPanel" )  ;                         
      cardLayout.first ( currentPanel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add controls to our main dialog                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.add ( pb_CANCEL ) ;
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_HELP ) ;
      contentPane.add ( buttonPanel ) ;                                   
      contentPane.add ( currentPanel ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust final properties and center                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPropertiesOnPanel ( contentPane );    
      GUISystem.setPreferredButton ( pb_OK_PROD_TYPE ) ;
      GUISystem.setPreferredButton ( pb_OK_IMPORT ) ;
      GUISystem.setPreferredButton ( pb_OK_PROD_TEMPLATE ) ;
      setSize ( 495, 310 ) ;
      WinUtil.centerChildInParent ( this, frame ) ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Finalize init                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      //VOY if ( DocTemplateSystem.getInstance() == null )
      //VOY    new DocTemplateSystem() ;

   }


   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will return the result of what the
    *  user selected. It will be one of the NEWPROD_XXX
    *  ints.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getResult() {
      setVisible ( true ) ;
      return( newProdType ) ;
   }




   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will lay out and position all
    *  of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size       = getSize() ;
      int       y          = 10 ;
      int       x          = 10 ;
      int       rowHeight  = GUISystem.getRowHeight () ;                                          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, adjust the controls on the main window, this     */
      /* includes the graphic, the current panel and the cancel    */
      /* button                                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      super.doLayout() ;

      st_IMAGE.setBounds ( 10, 10, 118, 259 ) ;
      st_HELP.setBounds  ( 133, 10, size.width-(148), rowHeight ) ;
      y+=rowHeight ;

      currentPanel.setBounds ( 133, y, size.width-(148), size.height-y-70 ) ;
      buttonPanel.setBounds ( 133, size.height-65, size.width-(148), 25 ) ;                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, adjust the controls on the prodTypePanel           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      size = prodTypePanel.getSize() ;
      productTypeScroll.setBounds ( 0, 0, size.width, size.height-(rowHeight*5) ) ;
      outputScroll.setBounds ( 0, size.height-(rowHeight*5)+5, size.width, rowHeight*4 ) ;                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, adjust the controls on the import panel            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      size = importPanel.getSize() ;
      x=10;
      y=10;
      st_IMPORT_FILE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      ef_IMPORT_FILE.setBounds ( x, y, size.width-(x*2)-80, rowHeight ) ;
      pb_BROWSE.setBounds ( size.width-80, y, 75, rowHeight ) ;
      y+=rowHeight*2 ;

      st_IMPORT_TYPE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      rb_CREATE_UPDATE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      rb_CREATE_ONLY.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      rb_UPDATE_ONLY.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight*2 ;

      st_PRODUCT_TYPE.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      importProductTypesScroll.setBounds ( x, y, size.width-(x*2), size.height-y-5 ) ;

   }


   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event from a GUI control is fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_TEMPLATE = Show the user the templates               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( pb_TEMPLATE == e.getSource() ) {
         newProdType = NEWPROD_TEMPLATE ;

         cardLayout.show ( currentPanel, "templatePanel" ) ;
         buttonPanel.remove ( pb_CANCEL ) ;
         buttonPanel.add ( pb_OK_PROD_TEMPLATE ) ;
         buttonPanel.add ( pb_CANCEL ) ;
         revalidate() ;
         new Thread ( this, THREAD_POPULATE_TEMPLATES ).start() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_IMPORT                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_IMPORT == e.getSource() ) {
         newProdType = NEWPROD_IMPORT ;

         st_HELP.setText ( Str.getStr ( STR_PRODUCT_IMPORT ) ) ;
         cardLayout.show ( currentPanel, "importPanel" ) ;
         buttonPanel.remove ( pb_CANCEL ) ;
         buttonPanel.add ( pb_OK_IMPORT ) ;
         buttonPanel.add ( pb_CANCEL ) ;
         revalidate() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_NEWPROD = Show the user the document types            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_NEWPROD == e.getSource() ) {
         newProdType = NEWPROD_BLANK ;
         cardLayout.show ( currentPanel, "prodTypePanel" ) ;
         st_HELP.setText ( Str.getStr ( STR_SELECT_PROD_TYPE ) ) ;
         buttonPanel.remove ( pb_CANCEL ) ;
         buttonPanel.add ( pb_OK_PROD_TYPE ) ;
         buttonPanel.add ( pb_CANCEL ) ;
         revalidate() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_IMPORT = Show nothing, just return                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_IMPORT == e.getSource() ) {
         newProdType = NEWPROD_IMPORT ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL = close this window                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_CANCEL == e.getSource() ) {
         newProdType = NEWPROD_CANCEL ;
         dispose() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user hit the Ok button for document types        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( pb_OK_PROD_TYPE == e.getSource() ) ||
                ( productTypes == e.getSource() ) ) {
         if ( typeProductRec != null )
            dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user hit the Ok button for the templates         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( pb_OK_PROD_TEMPLATE == e.getSource() ) ||
                ( templates == e.getSource() ) ) {
         if ( template != null )
            dispose() ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Browse                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( pb_BROWSE == e.getSource() ) {
         String currentDir = PropertySystem.getString ( PropertySystem.PROPERTY_IMPORT_DIRECTORY ) ;

         if ( currentDir == null )
            currentDir = PropertySystem.getString ( PropertySystem.PROPERTY_TEMP_DIR ) ;

         try {
            File[] files = GUISystem.getFileOpen ( GUISystem.getParentDefWin ( this ),
                                                   Str.getStr ( STR_IMPORT ),
                                                   currentDir,
                                                   true ) ;

            if ( files != null ) {
               if ( files[0].exists() )
                  ef_IMPORT_FILE.setText ( files[0].toString() ) ;
               else {
                  GUISystem.printBox ( this, "Error", "Please select a valid file" ) ;
                  ef_IMPORT_FILE.setText ( "" ) ;
               }
            }


         } catch ( Exception e1 ) {
            GUISystem.printBox ( this, "Error", "The Import file could not be opened" ) ;
            ef_IMPORT_FILE.setText ( "" ) ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK_IMPORT                                            */   
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( pb_OK_IMPORT == e.getSource() ) || 
                ( importProductTypes == e.getSource() ) ) {
         String  filename           = ef_IMPORT_FILE.getText() ;
         boolean prodTypeSelected   = ( importProductTypes.getSelectedValue() != null ) ;
         File    importFile         = new File ( filename ) ;
         boolean fileExists         = importFile.exists() ;

         if ( ( fileExists ) && ( prodTypeSelected ) ) {
            PropertySystem.putString ( PropertySystem.PROPERTY_IMPORT_DIRECTORY, importFile.getAbsolutePath() ) ;
            System.out.println ( "ABPAT:"+importFile.getPath() ) ;
            dispose() ;
         }
         else
            GUISystem.printBox ( this, "More information", "Please enter a valid <b>File</b> for import and also select a valid <b>Product Type</b> for import" ) ;
      }
      

   }





   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    this method will be called when an item
    *  in the list is selected
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *///*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The list of Document Types                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == productTypes ) {

         typeProductRec = (TypeProductRec) productTypes.getSelectedValue() ;

         pb_OK_PROD_TYPE.setEnabled ( typeProductRec != null ) ;

         switch ( typeProductRec.getInd() ) {
            
            case TypeList.PRODUCTTYPE_SYSTEM :
               typeOutput.setText ( "A complete system" ) ;
               break ;

            case TypeList.PRODUCTTYPE_OPTION :
               typeOutput.setText ( "An Accessory, Peripheral etc... including Mice, Keyboards, Network Adapters, Hard Drives, Memory ..." ) ;
               break ;

            case TypeList.PRODUCTTYPE_MONITOR :
               typeOutput.setText ( "Visual displays" ) ;
               break ;

            default:
               typeOutput.setText ( "" ) ;
               break ;

         }
      } /* if productTypes */


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   The list of templates                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == templates ) {
         template = (ProductRow) templates.getSelectedItem() ;

         pb_OK_PROD_TEMPLATE.setEnabled ( template != null ) ;
      }

   }




   /*------------------------------------------------------------*/
   /* getTypeProductRec
    */
   /**
    *    This method will return the currently
    *  selected type doc rec
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TypeProductRec
    *//*---------------------------------------------------------*/
   public TypeProductRec getTypeProductRec () {
      return( typeProductRec ) ;
   }




   /*------------------------------------------------------------*/
   /* listRefreshed
    */
   /**
    *    This method will be called when our type
    *  list has been refreshed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void listRefreshed ( int typeList ) {
      prodTypePanel.invalidate() ;
   }

   public void mouseClicked  ( MouseEvent e ) {
   }
   public void mousePressed  ( MouseEvent e ) {
   }
   public void mouseReleased ( MouseEvent e ) {
   }

   /**
     *  Subclassed to change the background color of a button
     *  to a highlighted look
     **/
   public void mouseEntered ( MouseEvent e ) {
      ((DButton)e.getSource()).setBackground ( buttonHighlightBackground ) ;
      detailedHelp.setText ( ((DButton)e.getSource()).getToolTipText() ) ;
   }

   /**
     *  Subclassed to change the color of a button back to normal
     **/
   public void mouseExited ( MouseEvent e ) {
      ((DButton)e.getSource()).setBackground ( buttonDefaultBackground ) ;
      detailedHelp.setText ( "" ) ;
   }

   /**
     *  This method will be run in a thread. This method has the
     * capability of doing many things, the things that is done depends
     * on the name of the thread
     **/
   public void run () {
      String operation = Thread.currentThread().getName () ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_POPULATE_TEMPLATES                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_POPULATE_TEMPLATES ) ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our indication that something is working            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         st_HELP.setText ( Str.getStr ( STR_READING_PROD_TEMPLATES ) ) ;
         pb_OK_PROD_TEMPLATE.setEnabled ( false ) ;
         pb_CANCEL.setEnabled ( false ) ;
         setCursor ( new Cursor ( Cursor.WAIT_CURSOR ) ) ;
         templates.setData ( ProductTemplateSystem.getInstance().getTemplateVec() ) ;
         st_HELP.setText ( Str.getStr ( STR_SELECT_A_TEMPLATE ) ) ;
         setCursor ( new Cursor ( Cursor.DEFAULT_CURSOR ) ) ;
         pb_OK_PROD_TEMPLATE.setEnabled ( true ) ;
         pb_CANCEL.setEnabled ( true ) ;
      }

   }


   /**
     *  This method simply asks our contentPane to revalidate
     **/
   private void revalidate () {
      invalidate() ;
      ((JPanel)getContentPane()).revalidate() ;
      doLayout() ;
      cardLayout.layoutContainer ( currentPanel ) ;
      validate() ;
   }


   /**
     *  This method will return the template that the user selected. If the template
     *  has not yet been read in, this method will first read the template and then return
     *  the template.
     **/
   public ProductDraft getTemplate () {
      int rc = template.readProduct ( false ) ;

      switch ( rc ) {
         case 0 :
            return( (ProductDraft)template.getProduct() ) ;

         case 100 :
            GUISystem.printBox ( STR_INFO, STR_DOCUMENT_NOT_FOUND ) ;
            LogSystem.log ( 1, "Document Template was not found." ) ;
            break ;

         default:
            GUISystem.printBox ( STR_INFO, STR_TEMPLATE_CANNOT_BE_READ ) ;
            LogSystem.log ( 1, "Cannot read template. Return Code:"+rc ) ;
            break ;
      }

      return( null ) ;
   }


   /**
     *  This method will return the import type
     **/
   public int getImportType () {
      if ( rb_CREATE_ONLY.isSelected() )
         return( IMPORT_CREATE ) ;
      else if ( rb_UPDATE_ONLY.isSelected() )
         return( IMPORT_UPDATE ) ;

      return( IMPORT_CREATE_UPDATE ) ;
   }


   /**
     *  This method will return the filename of the file to import
     **/
   public String getImportFilename () {
      return( ef_IMPORT_FILE.getText() ) ;
   }

   /**
     *  This method will return the product type of the import
     **/
   public int getImportProductType () {
      return( ((TypeProductRec)importProductTypes.getSelectedValue()).getInd() ) ;
   }
} /* NewProductDlg */
