package com.ibm.nzna.projects.qit.product.manager ;
/*-----------------------------------------------------------*/
/*   ProductCustomView                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductCustomView                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import java.awt.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.doc.docview.* ;
import com.ibm.nzna.projects.qit.customview.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.qit.product.gui.* ;

/**
 * This object represents the Product custom view 
 *
 *
 * @version 1.0, 05/18/2001 
 * @author  Venu Sanka
 */

/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductCustomView                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductCustomView extends CustomView implements AppConst, ProductConst {

   /*------------------------------------------------------------*/
   /*                                                            */
   /*   This is the serialization ID. If you add fields to this  */
   /* record, YOU MUST UPDATE THIS NUMBER. Adding methods is     */
   /* fine as long as you know what you are doing!               */
   /*                                                            */
   /* If you change this number, you break everyone's Product    */
   /* Custom views to the point where they will not load, so you */
   /* better write a converter or something to run on everyone's */
   /* bookmarks                                                  */
   /*                                                            */
   /*------------------------------------------------------------*/
   static final long serialVersionUID = 1000000 ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/


   private int[]     columnWidths   = { 20, 75, 300, 100, 200} ;
   private int       columnCount    = 5 ;
   private String    fromClause     = null ;
   private String    selectClause   = null ;
   private String    standardWhereClause = "" ;
   private String    geoFromClause = "" ;
   private String    countryFromClause = "" ;
   private String    dateFromClause = "" ;
   private boolean   request        = false ;
   private boolean   queryMachine   = false ;
   private boolean   queryModel     = false ;
   private boolean   queryGeo       = false ;
   private boolean   queryCountry   = false ;

   private boolean   queryFamily        = false ;
   private boolean   queryBrand         = false ;
   private boolean   queryOpicmBrand    = false ;
   private boolean   queryOpicmFamily   = false ;
   private boolean   queryDates         = false ;
   
   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default constructor
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductCustomView () {
   }

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor will create a blank
    *  ProductCustomView based on the type passed
    *  in
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductCustomView ( int viewType ) {
      setViewType ( viewType ) ;
   }


   /*------------------------------------------------------------*/
   /* getSelect
    */
   /**
    *    This method will return the entire select
    *  clause including the 'SELECT' keyword
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getSelect () {
      return( selectClause ) ;
   }

   /*------------------------------------------------------------*/
   /* getWhere
    */
   /**
    *    This method will return the entire where clause
    *  including the 'WHERE' statement
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getWhere () {
      Vector          fields                = getFields() ;
      int             i                     = 0 ;
      int             size                  = fields.size() ;
      String          whereClause           = null ;
      String          brandWhereClause      = "" ;
      String          familyWhereClause     = "" ;
      String          opicmBrandWhereClause = "" ;
      String          opicmFamilyWhereClause = "" ;
      CustomViewField field                 = null ;

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Start everything off real small                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         whereClause        = "" ;
         queryFamily        = false ;
         queryBrand         = false ;
         queryGeo           = false ;
         queryCountry       = false ;
         queryOpicmBrand    = false ;
         queryOpicmFamily   = false ;
         queryMachine       = false ;
         queryModel         = false ;
         queryDates         = false ;
         

         while ( i < size )
             {
            field = (CustomViewField) fields.elementAt(i) ;                  

            if ( field.getDatabaseOperator() != null ) 
                {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the field database name                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               //whereClause+=field.getColumnName()+" " ;  
                


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Add the operator                                        */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( !field.getColumnDescript().equals ( Str.getStr ( STR_BRAND ) ) )  &&
                    ( !field.getColumnDescript().equals ( Str.getStr ( STR_FAMILY ) ) ) &&
                    ( !field.getColumnDescript().equals ( Str.getStr ( STR_GEOGRAPHY ) ) ) &&
                    ( !field.getColumnDescript().equals ( Str.getStr ( STR_COUNTRY ) ) ) &&
                    ( !field.getColumnDescript().equals ( Str.getStr ( STR_OPICM_BRAND ) ) ) &&
                    ( !field.getColumnDescript().equals ( Str.getStr ( STR_OPICM_FAMILY ) ) ) &&
                    ( !field.getColumnDescript().equals ( Str.getStr ( STR_ANNOUNCE_DATE ) ) ) && 
                    ( !field.getColumnDescript().equals ( Str.getStr ( STR_REQUEST ) ) ) 
                  )
                  whereClause+=" AND "+field.getColumnName()+" "+field.getDatabaseOperator()+" " ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, depending on the column, put the value in          */
               /*                                                           */
               /*-----------------------------------------------------------*/

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Title, Changed time, Additional information             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( field.getColumnName().indexOf ( "LASTTOUCHEDON" ) >= 0 ) ||
                    ( field.getColumnName().indexOf ( "DESCRIPTION" ) >= 0 ) ||
                    //( field.getColumnName().indexOf ( "INPUT" ) >= 0 )  ||
                    ( field.getColumnName().indexOf ( "SOURCE" ) >= 0 ) ) 

                  {
                    
                  if ( field.getDatabaseOperator().equals ( "LIKE" ) )
                     whereClause+="'%"+field.getValue()+"%' " ;
                  else if ( field.getDatabaseOperator().equals ( "IN" ) )
                     whereClause+="( "+field.getValue()+" ) " ;
                  else
                     whereClause+="'"+field.getValue()+"' " ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Brand                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_BRAND ) ) ) {

                 brandWhereClause=" "+field.getColumnName()+" "+field.getDatabaseOperator()+"  ( "+((TypeCategoryRec)field.getValue()).getInd()+ " ) AND "+
                            " TYPECATEGORY.CATEGORYLEVEL = '" +TypeCategory.LEVEL_BRAND+ " ' " ;

                 queryBrand = true ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Family                                                  */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_FAMILY ) ) ) {

                   familyWhereClause=" "+field.getColumnName()+" "+field.getDatabaseOperator()+" ( "+((TypeCategoryRec)field.getValue()).getInd()+ " ) AND "+
                               " TYPECATEGORY.CATEGORYLEVEL = '" +TypeCategory.LEVEL_FAMILY+ " ' " ;

                  if ( queryBrand == true )
                  {  brandWhereClause = "" ;
                  }
                  queryFamily = true ;
                  
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   OPICM Brand                                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_OPICM_BRAND ) ) ) {

                   opicmBrandWhereClause=" "+field.getColumnName()+" "+field.getDatabaseOperator()+" ( "+((TypeCategoryRec)field.getValue()).getInd()+ " ) AND "+
                               " TYPECATEGORY.CATEGORYLEVEL = '" +TypeCategory.LEVEL_OPICM_BRAND+ " ' " ;
                   if ( queryFamily == true )
                   {  familyWhereClause = "" ;
                   }
                  
                  queryOpicmBrand = true ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   OPICM Family                                            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_OPICM_FAMILY ) ) ) {

                   opicmFamilyWhereClause=" "+field.getColumnName()+" "+field.getDatabaseOperator()+" ( "+((TypeCategoryRec)field.getValue()).getInd()+ " ) AND "+
                               " TYPECATEGORY.CATEGORYLEVEL = '" +TypeCategory.LEVEL_OPICM_FAMILY+ " ' " ;

                   if ( queryOpicmBrand == true )
                   {  opicmBrandWhereClause = "" ;
                   }
                   
                  queryOpicmFamily = true ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Geography                                               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_GEOGRAPHY ) ) ) {
                  whereClause+=" AND "+field.getColumnName()+" IN ("+getGeoIndValues ( (String)field.getValue() )+") AND PRODUCT.PRODUCTIND = GEO.PRODUCTIND " ;

                  queryGeo = true ;
               }


               
               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Country                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_COUNTRY ) ) ) {
                  whereClause+=" AND "+field.getColumnName()+" IN ("+getCountryCodeIndValues ( (String)field.getValue() )+") AND PRODUCT.PRODUCTIND = COUNTRY.PRODUCTIND " ;
                  queryCountry = true ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   ActualAnnounceDate                                      */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnDescript().equals ( Str.getStr ( STR_ANNOUNCE_DATE ) ) )  {

                   
                   whereClause+=" AND "+field.getColumnName()+" "+field.getDatabaseOperator()+" '"+field.getValue()+" ' AND "+
                               "  PRODUCT.PRODUCTIND =  DATES.PRODUCTIND  AND "+
                               " DATES.TYPEDATEIND  =  " +TypeList.DATE_ANNOUNCED+ " " ;

                   
                   queryDates = true ;
               }


               

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   LastTouchedBy                                           */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "LASTTOUCHEDBY" ) >= 0 ) {
                   
                  if ( field.getDatabaseOperator().equals ( "IN" ) )
                     whereClause+="( "+field.getValue()+" ) " ;
                  else
                     whereClause+="'"+field.getValue()+"' " ;
                  
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   ProductInd                                              */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "PRODUCTIND" ) >= 0 ) {
                  if ( field.getDatabaseOperator().equals ( "IN" ) )
                     whereClause+="( "+field.getValue()+" ) " ;
                  else
                     whereClause+=""+field.getValue()+" " ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   ProductType                                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "TYPEPRODUCTIND" ) >=0 )
               {
                  System.out.println("next line in producttype") ;
                  whereClause+=" "+((TypeProductRec)field.getValue()).getInd()+" " ;

                  System.out.println("next next line in producttype") ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Warranty                                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "TYPEWARRANTYIND" ) >=0 )
                  whereClause+=" "+((TypeWarrantyRec)field.getValue()).getInd()+" " ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Offering                                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "TYPEOFFERINGIND" ) >=0 )
                  whereClause+=" "+((TypeOfferingRec)field.getValue()).getInd()+" " ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Customer                                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( field.getColumnName().indexOf ( "TYPECUSTOMERIND" ) >=0 )
                  whereClause+="  "+((TypeCustomerRec)field.getValue()).getInd()+" " ;


               
                                              
            } /* if -- getDatabaseOperator */

            i++ ;
         } /* while -- i < size */

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Take the last 'AND' from the where clause off           */
         /*                                                           */
         /*-----------------------------------------------------------*/
        // whereClause = whereClause.substring( 0, whereClause.length()-4 ) ;

        // if( whereClause.indexOf("AND AND") >= 0 )
        //     whereClause = whereClause.substring(4, whereClause.length() ) ;

        // System.out.println("whereClause"+whereClause) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, adjust the From clause                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( request ) {
            

            if ( queryCountry ) 
                countryFromClause=", PRODRAFT.COUNTRY COUNTRY " ;
            if ( queryGeo )
                geoFromClause=", PRODRAFT.GEO GEO " ;
            if ( queryDates )
                dateFromClause=", PRODRAFT.DATES DATES " ;

            if ( ( queryBrand ) || ( queryFamily )||( queryOpicmBrand ) || ( queryOpicmFamily ) )
              fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                               " PRODRAFT.TYPECATEGORY TYPECATEGORY, PRODRAFT.CATEGORY CA, PRODRAFT.PRODUCT PRODUCT "+
                               ""+geoFromClause+countryFromClause+dateFromClause+"  WHERE "+
                               " PRODUCT.PRODUCTIND = CA.PRODUCTIND AND TYPECATEGORY.TYPECATIND = CA.TYPECATIND AND "+
                               ""+brandWhereClause+familyWhereClause+opicmBrandWhereClause+opicmFamilyWhereClause+whereClause+" ) J1 LEFT OUTER JOIN "+
                               " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                               " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                               " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;

              
            
            else if ( 
                !( ( queryBrand ) && ( queryFamily ) && ( queryOpicmBrand ) && ( queryOpicmFamily ) )
               )

                fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                               " PRODRAFT.PRODUCT PRODUCT "+geoFromClause+countryFromClause+dateFromClause+"  WHERE "+
                               ""+whereClause.substring(4, whereClause.length())+" ) J1 LEFT OUTER JOIN "+
                               " PRODRAFT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                               " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODRAFT.PRODUCT PRODUCT "+
                               " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;



            
         } else {

              if ( queryCountry ) 
                countryFromClause=" , PRODUCT.COUNTRY COUNTRY " ;
              if ( queryGeo )
                geoFromClause=" , PRODUCT.GEO GEO " ;
              if ( queryDates )
                dateFromClause=" , PRODUCT.DATES DATES " ;
              
            
              if ( ( queryBrand ) || ( queryFamily )||( queryOpicmBrand ) || ( queryOpicmFamily ) )
              fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                               " PRODUCT.TYPECATEGORY TYPECATEGORY, PRODUCT.CATEGORY CA, PRODUCT.PRODUCT PRODUCT "+
                               ""+geoFromClause+countryFromClause+dateFromClause+"  WHERE "+
                               " PRODUCT.PRODUCTIND = CA.PRODUCTIND AND TYPECATEGORY.TYPECATIND = CA.TYPECATIND AND "+
                               ""+brandWhereClause+familyWhereClause+opicmBrandWhereClause+opicmFamilyWhereClause+whereClause+" ) J1 LEFT OUTER JOIN "+
                               " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                               " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                               " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;

              
            
            else if ( 
                !( ( queryBrand ) && ( queryFamily ) && ( queryOpicmBrand ) && ( queryOpicmFamily ) )
               )

                fromClause = " FROM ( SELECT J1.PRODUCTIND, D.ACTUAL, D.TARGET FROM ( SELECT PRODUCT.PRODUCTIND FROM "+
                               " PRODUCT.PRODUCT PRODUCT "+geoFromClause+countryFromClause+dateFromClause+"  WHERE "+
                               ""+whereClause.substring(4, whereClause.length())+" ) J1 LEFT OUTER JOIN "+
                               " PRODUCT.DATES D ON J1.PRODUCTIND = D.PRODUCTIND AND "+
                               " D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" ) DATES INNER JOIN PRODUCT.PRODUCT PRODUCT "+
                               " ON PRODUCT.PRODUCTIND = DATES.PRODUCTIND " ;

            
                 
            
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, add the standard restrictions to the WHERE clause  */                                            
         /*                                                           */
         /*-----------------------------------------------------------*/
         //whereClause+=" PRODUCT.PUBLISH = 'Y' " ;
         standardWhereClause=" ORDER BY PRODUCT.PRODUCTIND FOR FETCH ONLY " ;
         
      } catch ( Exception e ) {
         LogSystem.log ( 1, e ) ;
      }


      return ( whereClause ) ;
   }

   /*------------------------------------------------------------*/
   /* getTables
    */
   /**
    *    This method will return all of the tables that
    *  this view will select from. This includes the
    *  keyword 'FROM'
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTables () {
      getWhere() ;
      return( fromClause ) ;
   }

   /*------------------------------------------------------------*/
   /* getColumnWidths
    */
   /**
    *    This method will return the column widths
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int[]
    *//*---------------------------------------------------------*/
   public int[] getColumnWidths () {
      return( columnWidths ) ;
   }

   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    This method will return the column count
    *  for this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getColumnCount () {
      return( columnCount ) ;
   }

   /*------------------------------------------------------------*/
   /* instantiateObject
    */
   /**
    *    This method will instaniate an object
    *  of the type passed in
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public CustomView instantiateObject ( int customType ) {
      return( null ) ;
   }

   /*------------------------------------------------------------*/
   /* getQuery
    */
   /**
    *    This method will return our full query
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return String
    *//*---------------------------------------------------------*/
   public String getQuery () {
      //return( getSelect()+" "+getTables()+" "+getWhere() ) ;
       return( getSelect()+" "+getTables()+" "+standardWhereClause ) ;
   }


   /*------------------------------------------------------------*/
   /* getResults
    */
   /**
    *    This method will kick off the query and will
    *  return the results from the query
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   public Vector getResults () {
      Vector           retVec  = new Vector ( 1, 1 ) ;   
      String           query2  = null ;
      Vector           dateVec = new Vector ( 1 ) ;
      Vector           productList = new Vector ( 1 ) ; 
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If this is going against a Request, we need a different */
      /*   Select and From clause   
      /*                                                           */
      /*-----------------------------------------------------------*/
       if (request) {
      
         selectClause = " SELECT DISTINCT(PRODUCT.PRODUCTIND), PRODUCT.EXISTINGPRODUCTIND, "+
                        "        PRODUCT.PRODNUM,  PRODUCT.DESCRIPTION, "+
                        "        PRODUCT.CREATEDON, PRODUCT.CREATEDBY, "+
                        "        DATES.ACTUAL, DATES.TARGET, PRODUCT.LASTTOUCHEDBY, "+  
                        "        PRODUCT.LASTTOUCHEDON, PRODUCT.CHECKEDOUTBY " ;
                        

      }  else  {
      
         selectClause = " SELECT DISTINCT(PRODUCT.PRODUCTIND), 0 ,  "+
                        "        PRODUCT.PRODNUM,  PRODUCT.DESCRIPTION, "+
                        "        PRODUCT.CREATEDON, PRODUCT.CREATEDBY, "+
                        "        DATES.ACTUAL, DATES.TARGET, PRODUCT.LASTTOUCHEDBY, "+ 
                        "        PRODUCT.LASTTOUCHEDON, PRODUCT.CHECKEDOUTBY "  ;
                }

      LogSystem.log( 1, "Custom View Kicked off. SQL:"+getQuery() ) ;
      
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "getResults ()", 5 ) ;
      

      try {
         
         Statement stmt   = sqlMethod.createStatement() ;
         ResultSet rs     = stmt.executeQuery ( getQuery() ) ;

         while ( rs.next() ) {
             String date = "" ;
                   if ( (rs.getString(7)) != null  )
                       date =  DateSystem.prettyDateFromDBDate ( rs.getString( 7 )) ;
                   else if ( (rs.getString(8)) != null )
                       date =  DateSystem.prettyDateFromDBDate ( rs.getString( 8 )) ;
                   
                   else 
                       date = rs.getString(7) ;

            ProductRow productRow = new ProductRow ( rs.getInt ( 1 ),                               //Product Ind
                                                     0,                                             //Existing Product Ind
                                                     rs.getString(3) ,                              //PRoduct number or Part Number
                                                     rs.getString ( 4 ),                            //Description
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 5 ).trim() ),//Created on
                                                     UserSystem.getNameFromUserId ( rs.getString ( 6 ).trim()),//Created     By
                                                     date,                                          //Actual Announce Date
                                                     "",  
                                                     UserSystem.getNameFromUserId ( rs.getString ( 9 ).trim()), //LAstTouched By                                              //Target Announce Date
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 10 ).trim() ),//LastTouched on
                                                     request ) ;

            if ( rs.getString(11) != null )
               productRow.setImage ( ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.SMALL_PRODUCT_LOCK ) ) ;
            retVec.addElement ( productRow ) ;

            

         }

         rs.close() ;


               
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;

      System.out.println("size"+retVec.size()) ;
      
      return( retVec ) ;
   }


   /*------------------------------------------------------------*/
   /* setViewType
    */
   /**
    *    This method will set what type of query this
    *  is
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setViewType ( int type ) {
      super.setViewType( type ) ;

      fields = new Vector ( 5 ) ;
      
      
      fields.addElement ( new CustomViewField ( "PRODUCT.TYPECUSTOMERIND",      Str.getStr ( STR_CUSTOMER ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.DESCRIPTION",          Str.getStr ( STR_DESCRIPTION ) ) ) ;
      //fields.addElement ( new CustomViewField ( "PRODUCT.INPUT",                Str.getStr ( STR_INPUT ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.TYPEOFFERINGIND",      Str.getStr ( STR_OFFERING ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.PRODUCTIND ",          Str.getStr ( STR_PRODUCT_IND ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.TYPEWARRANTYIND",      Str.getStr ( STR_WARRANTY ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.SOURCE",               Str.getStr ( STR_SOURCE ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.TYPEPRODUCTIND",       Str.getStr ( STR_PRODUCT_TYPE ) ) ) ;
      fields.addElement ( new CustomViewField ( "DATES.ACTUAL",                 Str.getStr ( STR_ANNOUNCE_DATE ) ) ) ;
      fields.addElement ( new CustomViewField ( "GEO.GEOIND",                   Str.getStr ( STR_GEOGRAPHY ) ) ) ;
      fields.addElement ( new CustomViewField ( "COUNTRY.COUNTRYCODEIND",       Str.getStr ( STR_COUNTRY ) ) ) ;
      fields.addElement ( new CustomViewField ( "TYPECATEGORY.TYPECATIND",      Str.getStr ( STR_OPICM_BRAND ) ) ) ;
      fields.addElement ( new CustomViewField ( "TYPECATEGORY.TYPECATIND",      Str.getStr ( STR_OPICM_FAMILY ) ) ) ;
      fields.addElement ( new CustomViewField ( "TYPECATEGORY.TYPECATIND",      Str.getStr ( STR_BRAND ) ) ) ;
      fields.addElement ( new CustomViewField ( "TYPECATEGORY.TYPECATIND",      Str.getStr ( STR_FAMILY ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.LASTTOUCHEDON",        Str.getStr ( STR_LAST_TOUCHED_ON ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.LASTTOUCHEDBY",        Str.getStr ( STR_LAST_TOUCHED_BY ) ) ) ;
      fields.addElement ( new CustomViewField ( "PRODUCT.REQUEST",              Str.getStr ( STR_REQUEST ) ) ) ;
      
      new QuickSort ( fields ) ;
   }

   /*------------------------------------------------------------*/
   /* addFieldRestriction
    */
   /**
    *    This method will add a field restriction
    *  to this query
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void addFieldRestriction ( CustomViewField field ) {
      Frame  tempFrame   = new Frame() ;  

      Vector operatorVec = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Always ask the user for what type of operator first for */
      /* most fields                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( field.getColumnName().indexOf ( "COUNTRYCODEIND" ) == -1 ) &&
           ( field.getColumnName().indexOf ( "GEOIND" ) == -1 ) &&
           ( field.getColumnName().indexOf ( "REQUEST" ) == -1 ) 
         )
           
         operatorVec = new JListDlg ( tempFrame, Str.getStr ( STR_OPERATOR ), field.getOperators() ).getResult() ;
         
      else {
         operatorVec = new Vector ( 1 ) ;
         operatorVec.addElement ( Str.getStr ( STR_EQUALS ) ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only continue if they selected an operator              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( operatorVec != null ) && ( operatorVec.size() > 0 ) ) {

         field.setOperator ( (String)operatorVec.elementAt(0) ) ;

         
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Description                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( field.getColumnName().indexOf ( "DESCRIPTION" ) >= 0 ) {
            String comment = new SingleEntryDlg ( tempFrame,
                                                  Str.getStr ( STR_DESCRIPTION ),
                                                  Str.getStr ( STR_DESCRIPTION ),
                                                  "",
                                                  "" ).getResult() ;
            if ( ( comment != null ) && ( comment.length() > 0 ) ) {
               field.setValue ( comment ) ;
            }

         } /* description  */

         
         
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Source                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "SOURCE" ) >= 0 ) {
            String source = new SingleEntryDlg ( tempFrame,
                                                 Str.getStr ( STR_SOURCE ),
                                                 Str.getStr ( STR_SOURCE ),
                                                 "", "" ).getResult() ;
            if ( ( source != null ) && ( source.length() > 0 ) )
               field.setValue ( source ) ;

         } /* source */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Input                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         //else if ( field.getColumnName().indexOf ( "INPUT" ) >= 0 ) {
         //   String source = new SingleEntryDlg ( tempFrame,
         //                                        Str.getStr ( STR_INPUT ),
         //                                        Str.getStr ( STR_INPUT ),
         //                                        "", "" ).getResult() ;
         //   if ( ( source != null ) && ( source.length() > 0 ) )
         //      field.setValue ( source ) ;
         //
         //} /* input */


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   LastTouchedOn                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "LASTTOUCHEDON" ) >= 0 ) {
            String date = new SingleDateEntryDlg ( tempFrame,
                                                   Str.getStr ( STR_LAST_TOUCHED_ON ),
                                                   Str.getStr ( STR_LAST_TOUCHED_ON ),
                                                   new CDate ().today() ).getResult() ;
            if ( ( date != null ) && ( date.length() > 0 ) ) {
              date = CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR,  CDate.DATE_DATE ),
                                 CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                 CDate.parse ( date, CDate.DATE_DAY,   CDate.DATE_DATE ),
                                 0,
                                 0,
                                 0,
                                 CDate.DATE_DBDATE )  ;
               date = Text.replaceAllStrInStr ( date, "/", "-" ) ;
               field.setValue ( date+"-07.00.00.000000" ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   LastTouchedBy                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "LASTTOUCHEDBY" ) >= 0 ) {
            Vector  vector  = new SelectUserDlg ( tempFrame ).getResult() ;
            UserRec userRec = null ;

            if ( ( vector != null ) && ( vector.size() > 0 ) ) {
               int     i    = 0 ;
               int     size = vector.size() ;
               String  str  = "" ;

               while ( i < size ) {
                  str+=((UserRec)vector.elementAt(i)) ;
                  if ( i != (size-1) )
                     str+="," ;
                  i++ ;
               } /* while  */

               field.setValue ( str ) ;

            } /* if  */
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Country                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "COUNTRYCODEIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_COUNTRY ),
                                                    TypeList.getInstance().getTypeList( TypeList.TYPE_COUNTRYCODE ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               int    i     = 0 ;
               int    size  = selectedItemVec.size() ;
               String value = "" ;

               while ( i < size ) {
                  value+=""+((TypeCountryCodeRec)selectedItemVec.elementAt(i)).toString() ;
                  if ( i < (size-1) )
                     value+="," ;
                  i++ ;
               }
               field.setValue ( value ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Geo                                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "GEOIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_GEOGRAPHY ),
                                                    TypeList.getInstance().getTypeList( TypeList.TYPE_GEOGRAPHY)).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               int    i     = 0 ;
               int    size  = selectedItemVec.size() ;
               String value = "" ;

               while ( i < size ) {
                  value+=""+((TypeGeoRec)selectedItemVec.elementAt(i)).toString() ;
                  if ( i < (size-1) )
                     value+="," ;
                  i++ ;
               }
               field.setValue ( value ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Actual  Announce Date                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "ACTUAL" ) >= 0 ) {
            String date = new SingleDateEntryDlg ( tempFrame,
                                                   Str.getStr ( STR_ANNOUNCE_DATE ),
                                                   Str.getStr ( STR_ANNOUNCE_DATE ),
                                                   new CDate ().today() ).getResult() ;
            if ( ( date != null ) && ( date.length() > 0 ) ) {
              date = CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR,  CDate.DATE_DATE ),
                                 CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                 CDate.parse ( date, CDate.DATE_DAY,   CDate.DATE_DATE ),
                                 0,
                                 0,
                                 0,
                                 CDate.DATE_DBDATE )  ;
               date = Text.replaceAllStrInStr ( date, "/", "-" ) ;
               field.setValue ( date ) ;
            }

            
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   ProductType                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "TYPEPRODUCTIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_PRODUCT_TYPE ),
                                                    TypeList.getInstance().getTypeList ( TypeList.TYPE_PRODUCT ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Product Ind                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "PRODUCTIND" ) >= 0 ) {
            String productind = new SingleEntryDlg ( tempFrame,
                                                  Str.getStr ( STR_PRODUCT_IND ),
                                                  Str.getStr ( STR_PRODUCT_IND ),
                                                  "", "" ).getResult() ;
            if ( ( productind != null ) && ( productind.length() > 0 ) )
               field.setValue ( productind ) ;

         } /* productind */      



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Offering                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "TYPEOFFERINGIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_OFFERING ),
                                                    TypeList.getInstance().getTypeList ( TypeList.TYPE_OFFERING ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Warranty                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "TYPEWARRANTYIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_WARRANTY ),
                                                    TypeList.getInstance().getTypeList( TypeList.TYPE_WARRANTY ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Customer                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "TYPECUSTOMERIND" ) >= 0 ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_CUSTOMER ),
                                                    TypeList.getInstance().getTypeList( TypeList.TYPE_CUSTOMER ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Request                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( field.getColumnName().indexOf ( "REQUEST" ) >= 0 ) {
            request = GUISystem.printBox ( tempFrame, STR_QUESTION, STR_CUSTOM_VIEW_FOR_REQUESTS ) ;

            field.setValue ( ""+request ) ;
         }

         
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Brand                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( field.getColumnName().indexOf ( "TYPECATIND" ) >= 0 ) &&
                   ( field.getColumnDescript().equals ( Str.getStr ( STR_BRAND ) ) ) ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_BRAND ),
                                                    TypeCategory.getCategories(TypeCategory.LEVEL_BRAND ) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Family                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( field.getColumnName().indexOf ( "TYPECATIND" ) >= 0 ) &&
                   ( field.getColumnDescript().equals ( Str.getStr ( STR_FAMILY ) ) ) ) {
            Vector family =  TypeCategory.getCategories(TypeCategory.LEVEL_FAMILY ) ;
            new QuickSort(family) ;
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_FAMILY ),
                                                    family ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   OPIC/M Brand                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( field.getColumnName().indexOf ( "TYPECATIND" ) >= 0 ) &&
                   ( field.getColumnDescript().equals ( Str.getStr ( STR_OPICM_BRAND ) ) ) ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_OPICM_BRAND ),
                                                    TypeCategory.getCategories(TypeCategory.LEVEL_OPICM_BRAND)).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   OPICM Family                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( field.getColumnName().indexOf ( "TYPECATIND" ) >= 0 ) &&
                   ( field.getColumnDescript().equals ( Str.getStr ( STR_OPICM_FAMILY ) ) ) ) {
            Vector selectedItemVec = new JListDlg ( tempFrame,
                                                    Str.getStr ( STR_OPICM_FAMILY ),
                                                    TypeCategory.getCategories(TypeCategory.LEVEL_OPICM_FAMILY) ).getResult() ;

            if ( ( selectedItemVec != null ) && ( selectedItemVec.size() > 0 ) ) {
               field.setValue ( selectedItemVec.elementAt(0) ) ;
            }
         }

         
      
      } /* if they selected an operator  */

      tempFrame.dispose() ;

         }

      
   /*------------------------------------------------------------*/
   /* getRequest
    */
   /**
    *    This method will return TRUE if this query
    *  pulls Product Requests
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getRequest () {
      return( request ) ;
   }


   /**
     *  This method will take a String that contains a comma separated
     *  list of Country Descriptions and will return a String with the
     *  Country Code Indicators separated by commas. Each Country Code
     *  Indicator will replace the country's Description.
     **/
   private String getCountryCodeIndValues ( String countries ) {
      StringTokenizer    strtok     = new StringTokenizer ( countries, "," ) ;
      String             retStr     = "" ;
      TypeCountryCodeRec countryRec = null ;

      while ( strtok.hasMoreTokens() ) {
         countryRec = (TypeCountryCodeRec)TypeList.getInstance().objectFromString ( strtok.nextToken(), TypeList.TYPE_COUNTRYCODE ) ;
         if ( countryRec != null )
            retStr+=""+countryRec.getInd()+"," ;
      }

      /* Take the last comma out  */
      retStr = retStr.substring ( 0, retStr.length()-1) ;

      return ( retStr ) ;
   }

   /**
     *  This method will take a String that contains a comma separated
     *  list of Geo Descriptions and will return a String with the
     *  Geo Code Indicators separated by commas. Each Geo Code
     *  Indicator will replace the geo's Description.
     **/
   private String getGeoIndValues ( String geos ) {
      StringTokenizer strtok  = new StringTokenizer ( geos, "," ) ;
      String          retStr  = "" ;
      TypeGeoRec      geoRec  = null ;

      while ( strtok.hasMoreTokens() ) {
         geoRec = (TypeGeoRec)TypeList.getInstance().objectFromString ( strtok.nextToken(), TypeList.TYPE_GEOGRAPHY ) ;
         if ( geoRec != null )
            retStr+=""+geoRec.getInd()+"," ;
      }

      /* Take the last comma out  */
      retStr = retStr.substring ( 0, retStr.length()-1) ;

      return ( retStr ) ;
   }



   


} /* ProductCustomView */
