package com.ibm.nzna.projects.qit.product.manager ;   
/*-----------------------------------------------------------*/
/*   ProductManagementPanel                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductManagementPanel              */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.sql.* ;
import javax.swing.* ;
import javax.swing.tree.*;
import javax.swing.event.* ;
import java.awt.event.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;     // MultiList, DButton etc..
import com.ibm.nzna.shared.util.* ;    // LogSystem, Text etc,,
import com.ibm.nzna.shared.db.* ;      // DatabaseSystem, SQLMethod
import com.ibm.nzna.shared.batch.* ;
import com.ibm.nzna.shared.pom.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.qit.*;
import com.ibm.nzna.projects.qit.customview.CustomViewDlg ;
import com.ibm.nzna.projects.qit.customview.*;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.help.* ;
import com.ibm.nzna.projects.qit.dbgui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.manager.property.* ;
import com.ibm.nzna.projects.qit.product.gui.*;
import com.ibm.nzna.projects.qit.product.history.*;
import com.ibm.nzna.projects.qit.product.compare.*;
import com.ibm.nzna.projects.qit.product.manager.views.* ;
import com.ibm.nzna.projects.qit.product.productImport.* ;
import com.ibm.nzna.projects.qit.batchrunner.* ;
import com.ibm.nzna.projects.qit.product.productEdit.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.product.manager.masschange.*;
import com.ibm.nzna.projects.common.storedProc.sqlRunner.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.product.* ;            
import com.ibm.nzna.projects.common.quest.brand.* ;            

/**
 * The Product Management panel allows the user
 * to navigate through the products in the Quest database
 *
 *
 * @version 1.0, 6/25/2001 
 * @author  Venu Sanka
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductManagementPanel                                  */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductManagementPanel extends JPanel implements 
Runnable, ProductConst, ActionListener, PreviewPanel, AppConst, FindProductListener,
CustomViewDlgListener, PrintPanel, TreeSelectionListener, TreeExpansionListener, QuestPanel, Authority, HelpSource {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_REFRESH_VIEW     = "REFRESH_VIEW" ; 
   private static final String THREAD_REFRESHVIEW_NODE = "REFRESH_VIEW_NODE" ;
   private static final String THREAD_UNLOCK_PRODUCTS  = "UNLOCK" ;
   private static final String THREAD_REFRESH_PREVIEW  = "REFRESH_PREVIEW" ;
   private static final String THREAD_COMPLETEREQUESTS = "RCOMREQ" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private GUI Members                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private ActionButton    pb_SAVE              = null ;
   private ToolBarButton   pb_PROPERTY          = null ;

   /**
     * This menu holds the columns that are visible
     **/
   private JMenu showMenu  = new JMenu ( "Show" ) ;

   /**
     * This menu shows/hides the Part Number field
     **/
   private JCheckBoxMenuItem ck_SHOW_PARTNUM = new JCheckBoxMenuItem ( "Part/Product Number" ) ;

   /**
     * This menu shows/hides the Description field
     **/
   private JCheckBoxMenuItem ck_SHOW_DESCRIPTION = new JCheckBoxMenuItem ( "Description" ) ;


   /**
     * This menu shows/hides the Announce Date
     **/
   private JCheckBoxMenuItem ck_SHOW_ANNOUNCE = new JCheckBoxMenuItem ( "Announce Date" ) ;

   /**
     * This menu shows/hides the CreateDate
     **/
   private JCheckBoxMenuItem ck_SHOW_CREATEDATE = new JCheckBoxMenuItem ( "Create Date" ) ;

   /**
     * This menu shows/hide sthe Last Touched by
     **/
   private JCheckBoxMenuItem ck_SHOW_LASTTOUCHEDBY = new JCheckBoxMenuItem ( "Last Touched By" ) ;

   /**
     * This menu shows/hide sthe Last Touched on
     **/
   private JCheckBoxMenuItem ck_SHOW_LASTTOUCHEDON = new JCheckBoxMenuItem ( "Last Touched On" ) ;


   /**
    * The ProductNavPanel 
    */
   private ProductNavPanel    navPanel          = null ;

   /**
    * The UNLOCK ToolBarButton. 
    */
   private ToolBarButton  pb_UNLOCK         = null ;

   /**
    * The FIND ToolBarButton 
    */
   private ToolBarButton  pb_FIND           = null ;

   /**
    * The Wizard ToolBarButton 
    */
   private ToolBarButton  pb_WIZARD         = null ;

   /**
    * The Custom Query ToolBarButton 
    */
   private ToolBarButton  pb_CUSTOM_QUERY   = null ;

   /**
    * The HISTORY ToolBarButton 
    */
   private ToolBarButton  pb_HISTORY        = null ;

   /**
    * The HISTORY ToolBarButton 
    */
   private ToolBarButton  pb_PREVIEW        = null ;

   /**
    * The COMPARE ToolBarButton 
    */
   private ToolBarButton  pb_COMPARE        = null ;

   /**
    * The ToolBar for the ToolBarButtons 
    */
   private JToolBar       toolBar           = null ;      

   /**
    * The Edit Button 
    */
   private DButton        pb_EDIT           = null ;

   /**
    * The CLOSE Button 
    */
   private DButton        pb_CLOSE          = null ;

   /**
    * The NEW Button 
    */
   private DButton        pb_NEW            = null ;

   /**
    * The DELETE Button 
    */
   private DButton        pb_DELETE         = null ;

   /**
    * The COMPLETE Button 
    */
   private DButton        pb_COMPLETE       = null ;

   /**
    * The CANCEL Button 
    */
   private DButton        pb_CANCEL         = null ;

   /**
     *  This is the export button
     **/
   private ToolBarButton pb_EXPORT = null ;


   /**
     *  This is the Descriptio text which appears first when this applet is opened
     **/
   private JLabel st_FIRST_TEXT = new JLabel ( Str.getStr ( STR_PRODUCT_MGMT_FIRST_TEXT ) ) ;

   /**
     * This is the split pane which holds the preview window
     **/
   private JSplitPane splitPane = new JSplitPane ( JSplitPane.VERTICAL_SPLIT ) ;

   /**
     * This is the ProductEditPanel which shows the preview
     **/
   private ProductEditPanel previewPanel = new ProductEditPanel () ;

   /**
     *  This is the flag telling us if we are showing drafts or not
     **/
   private boolean       showingDrafts    = false ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private AppDefaultWin          parentWin              = null ;
   private String                 currentView            = null ;  // The view currently showing
   private PMView                 currentPMView          = null ;
   private DefaultTreeModel       treeModel              = null ;
   private DetailTreeCellRenderer treeRenderer           = null ;  // The tree renderer for the view
   private JTitle                 st_TITLE               = new JTitle() ;
   private JLabel                 st_KEY                 = null ;
   private TypeCategoryRec        currentBrandRec        = null ;  // The brand for the view being displayed
   private int                    sort                   = 0 ;
   private DetailJTree            productTree            = new DetailJTree() ;
   private ViewTreeNode           currentViewNode        = null ;
   private JPanel                 contentPane            = new JPanel () ;
   private CustomView             currentCustomView      = null ;
   private AppDefaultWin          defaultWin             = GUISystem.getParentDefWin( this ) ;





   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method is called first before anything
    *  else. We are not showing at this time so it is
    *  a good idea to create our GUI now
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE if we initialize, FALSE if we do not
    *//*---------------------------------------------------------*/
   public boolean initialize () {
      boolean rc = false ;

      try {


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our GUI Controls                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         navPanel          = new ProductNavPanel ( this ) ;
         pb_EDIT           = new ActionButton ( Str.getStr ( STR_EDIT ),     ImageSystem.getImageIcon ( this, ImageSystem.EDIT ),    Str.getStr ( STR_EDIT ) ) ;
         pb_NEW            = new ActionButton ( Str.getStr ( STR_NEW ),      ImageSystem.getImageIcon ( this, ImageSystem.NEW ),     Str.getStr ( STR_NEW ) ) ;
         pb_DELETE         = new ActionButton ( Str.getStr ( STR_DELETE ),   ImageSystem.getImageIcon ( this, ImageSystem.DELETE ),  Str.getStr ( STR_DELETE ) ) ;
         pb_COMPLETE       = new ActionButton ( Str.getStr ( STR_COMPLETE ), ImageSystem.getImageIcon ( this, ImageSystem.ACTION ),  Str.getStr ( STR_COMPLETE_PRODUCT_REQUEST ) ) ;
         pb_CLOSE          = new ActionButton ( Str.getStr ( STR_CLOSE ),    ImageSystem.getImageIcon ( this, ImageSystem.CLOSE ),   Str.getStr ( STR_CLOSE_THIS_PANEL ) ) ;
         //pb_CANCEL        	= new ActionButton ( Str.getStr ( STR_CANCEL ),   ImageSystem.getImageIcon ( this, BMP_CANCEL ),  Str.getStr ( STR_CANCEL) ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create our tool bar buttons                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         toolBar           = new JToolBar() ;
         pb_FIND           = new ToolBarButton ( Str.getStr ( STR_FIND ),       ImageSystem.getImageIcon ( this, ImageSystem.SEARCH ),                 Str.getStr ( STR_FIND_PRODUCT ) ) ;
         pb_EXPORT         = new ToolBarButton ( Str.getStr ( STR_EXPORT ),     ImageSystem.getImageIcon ( this, ImageSystem.EXPORT ),                 Str.getStr ( STR_EXPORT ) ) ;
         pb_WIZARD         = new ToolBarButton ( Str.getStr ( STR_WIZARD ),     ImageSystem.getImageIcon ( this, ImageSystem.WIZARD ),                 Str.getStr ( STR_PRODUCT_WIZARD ) ) ;
         pb_CUSTOM_QUERY   = new ToolBarButton ( Str.getStr ( STR_CUSTOM ),     ImageSystem.getImageIcon ( this, ImageSystem.CUSTOM_QUERY ),           Str.getStr ( STR_CUSTOM_QUERY_DOC_TIP ) ) ;
         pb_COMPARE        = new ToolBarButton ( Str.getStr ( STR_COMPARE ),    ImageSystem.getImageIcon ( this, ImageSystem.PRODUCT_COMPARE ),        Str.getStr ( STR_COMPARE )) ;
         pb_HISTORY        = new ToolBarButton ( Str.getStr ( STR_HISTORY ),    ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_HISTORY_SMALL ), Str.getStr ( STR_HISTORY ) ) ;
         pb_PROPERTY       = new ToolBarButton ( Str.getStr ( STR_PROPERTY ),   ImageSystem.getImageIcon ( this, ImageSystem.PROPERTIES ),             Str.getStr ( STR_DOCUMENT_MAINT_PROPERTY ) ) ;


         if ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_PREVIEW ) )
            pb_PREVIEW        = new ToolBarButton ( Str.getStr ( STR_PREVIEW_OFF ),    ImageSystem.getImageIcon ( this, ImageSystem.PREVIEW ), Str.getStr ( STR_PREVIEW ) ) ;
         else
            pb_PREVIEW        = new ToolBarButton ( Str.getStr ( STR_PREVIEW_ON ),    ImageSystem.getImageIcon ( this, ImageSystem.PREVIEW ), Str.getStr ( STR_PREVIEW ) ) ;

         toolBar.add ( new ToolBarSeparator () )  ;
         toolBar.add ( pb_FIND ) ;
         toolBar.add ( pb_PREVIEW ) ;
         toolBar.add ( pb_HISTORY ) ;
         toolBar.add ( pb_EXPORT ) ;
         toolBar.add ( pb_COMPARE ) ;
         toolBar.add ( pb_WIZARD ) ;
         toolBar.add ( pb_CUSTOM_QUERY ) ;
         toolBar.add ( new ToolBarSeparator () )  ;
         toolBar.add ( pb_PROPERTY ) ;

         if ( UserSystem.hasAuthority ( Authority.AUTH_UNLOCK_PRODUCTS ) ) {
            pb_UNLOCK = new ToolBarButton ( Str.getStr ( STR_UNLOCK ), ImageSystem.getImageIcon ( this, ImageSystem.LOCK_SMALL ), Str.getStr ( STR_UNLOCK )+" "+Str.getStr ( STR_PRODUCTS ) ) ;
            pb_UNLOCK.addActionListener ( this ) ;
            toolBar.add ( pb_UNLOCK ) ;  
         }


         /*----------------------------------*/
         /*                                  */
         /*  Add our Menu items for the Show */
         /*                                  */
         /*----------------------------------*/
         showMenu.add ( ck_SHOW_PARTNUM ) ;
         showMenu.add ( ck_SHOW_DESCRIPTION ) ;
         showMenu.add ( ck_SHOW_ANNOUNCE ) ;
         showMenu.add ( ck_SHOW_CREATEDATE ) ;
         showMenu.add ( ck_SHOW_LASTTOUCHEDBY ) ;
         showMenu.add ( ck_SHOW_LASTTOUCHEDON ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Adjust properties                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         toolBar.setOpaque ( false ) ;                   // Always make your toolbars see-through
         toolBar.setBorder ( GUISystem.emptyBorder ) ;   // Always make a blank border on your toolbar
         st_FIRST_TEXT.setBackground ( Color.white ) ;
         st_FIRST_TEXT.setHorizontalAlignment ( JLabel.CENTER ) ;
         st_FIRST_TEXT.setVerticalAlignment ( JLabel.TOP ) ;
         contentPane.setBackground ( Color.white ) ;
         previewPanel.setEditable ( false ) ;
         ck_SHOW_PARTNUM.setSelected ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_PARTNUM ) ) ;
         ck_SHOW_DESCRIPTION.setSelected ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_DESCRIPTION ) ) ;
         ck_SHOW_ANNOUNCE.setSelected ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_ANNOUNCE ) ) ;
         ck_SHOW_CREATEDATE.setSelected ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_CREATEDATE ) ) ;
         ck_SHOW_LASTTOUCHEDBY.setSelected ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDBY ) ) ;
         ck_SHOW_LASTTOUCHEDON.setSelected ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDON ) ) ;
         ck_SHOW_PARTNUM.addActionListener ( this ) ;
         ck_SHOW_DESCRIPTION.addActionListener ( this ) ;
         ck_SHOW_ANNOUNCE.addActionListener ( this ) ;
         ck_SHOW_CREATEDATE.addActionListener ( this ) ;
         ck_SHOW_LASTTOUCHEDBY.addActionListener ( this ) ;
         ck_SHOW_LASTTOUCHEDON.addActionListener ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our listeners                                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         pb_CLOSE.addActionListener                      ( this ) ; // lines all line up. Pretty Sweet!
         pb_EDIT.addActionListener                       ( this ) ;
         pb_NEW.addActionListener                        ( this ) ;
         pb_DELETE.addActionListener                     ( this ) ;
         pb_COMPLETE.addActionListener                   ( this ) ;
         pb_EXPORT.addActionListener                     ( this ) ;
         productTree.getJTree().addTreeExpansionListener ( this ) ;
         productTree.getJTree().addTreeSelectionListener ( this ) ;

         pb_FIND.addActionListener ( this ) ;
         pb_WIZARD.addActionListener ( this ) ;
         pb_CUSTOM_QUERY.addActionListener ( this ) ;
         pb_COMPARE.addActionListener ( this ) ;
         pb_HISTORY.addActionListener ( this ) ;      
         pb_PREVIEW.addActionListener ( this ) ;
         pb_PROPERTY.addActionListener ( this ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set up our tree                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         productTree.getJTree().setCellRenderer ( treeRenderer = new DetailTreeCellRenderer ( GUISystem.getFontUtil(), 7 ) ) ;
         productTree.getJTree().setModel ( new DefaultTreeModel ( new StringTreeNode ( Str.getStr ( STR_SELECT_A_VIEW ) ) ) ) ;
         productTree.setBorder ( null ) ;
         treeRenderer.setWordWrap ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_WORD_WRAP ) ) ;
         productTree.setWordWrap ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_WORD_WRAP ) ) ;
         treeRenderer.setPreferredWidth ( 800 ) ;
         productTree.addActionListener ( this ) ;
         productTree.getJTree().setRowHeight ( 0 ) ;



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Add our GUI                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setLayout ( new BorderLayout () ) ;
         splitPane.setBottomComponent ( previewPanel ) ;

         contentPane.setLayout( new BorderLayout () ) ;
         contentPane.add ( st_TITLE,    BorderLayout.NORTH ) ;
         contentPane.add ( st_FIRST_TEXT, BorderLayout.CENTER ) ;

         if ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_PREVIEW ) ) {
            splitPane.setTopComponent ( contentPane ) ;
            add ( splitPane, BorderLayout.CENTER ) ;
         } else
            add ( contentPane,BorderLayout.CENTER );

      } catch ( Exception e ) {
         LogSystem.log( 1, e, false ) ;
         return( false ) ;
      }

      rc = true ;

      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called once we are added
    *  to a Window and we are showing.
    *  We use this time to add our toolbars, action buttons, etc..
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get our parent window                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin = GUISystem.getParentDefWin ( this ) ;             

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Action Buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentWin.addActionComponent ( this, pb_CLOSE ) ;   // Always put Close on first
      parentWin.addActionComponent ( this, pb_NEW ) ; 
      parentWin.addActionComponent ( this, pb_EDIT ) ; 
      parentWin.addActionComponent ( this, pb_DELETE ) ; 
      parentWin.addActionComponent ( this, pb_COMPLETE ) ;                                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Hide these action buttons until a view is selected      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_EDIT.setVisible ( false ) ;
      pb_DELETE.setVisible ( false ) ;
      pb_COMPLETE.setVisible ( false ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Put our tool bar up on the window                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      panelSelected() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we are showing preview, adjust the position          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_PREVIEW ) )
         splitPane.setDividerLocation ( (int) getSize().height-200 ) ;


   }





   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when this panel
    *  has once again gained focus on the main window.
    *  This happens when the user clicks off of this panel
    *  and then clicks back on.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {

      parentWin.getAppDefaultMenuBar().add ( showMenu ) ;

      /* We are now showing again, show our toolbar buttons  */
      showToolbar() ;
   }




   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when our panel
    *  is clicked off of. We will take our toolbar
    *  off
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      parentWin.getAppDefaultMenuBar().remove ( showMenu ) ;
      parentWin.removeToolbarButton ( toolBar ) ;
   }





   /*------------------------------------------------------------*/
   /* showToolbar
    */
   /**
    *    This method will add and show our toolbar
    *  on our parent AppDefaultWin
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void showToolbar () {
      parentWin.addToolbarButton ( toolBar, 2 ) ;
   }





   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method should return the name
    *  of this panel that is displayed in the top left
    *  of the main window
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_PRODUCT_MANAGEMENT ) ) ;
   }





   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method should copy text from this
    *  panel. We do nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method should cut text from this
    *  panel. We do nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method should paste text onto this
    *  panel. We do nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    Since QuestPanels can really be made
    *  of anything, we need to return the actual
    *  JComponent that represents our QuestPanel.
    *  However, since our QuestPanel is actually a JPanel,
    *  just return an instance of ourselves
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called when the user
    *  wants to shut down the app. Return FALSE from
    *  this if it is Not OK to shutdown and shutdown will stop.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean shutdown () {

      return( true ) ; // We return TRUE Since we are Ok to shutdown, however
                       // If you had unsaved data on this window, you could use
                       // this time to save it or something
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when the user
    *  wants to close this panel. You can return FALSE
    *  from this method and you are not closed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean stop () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save our widths                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( currentPMView != null )
         PropertySystem.putDetailList ( PROPERTY_PRODUCT_TREE_WIDTHS, productTree ) ;
      panelDeselected() ;

      return( true ) ; // We return TRUE Since we are Ok to close, however
                       // If you had unsaved data on this window, you could use
                       // this time to save it or something
   }






   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method will return our default
    *  navigation panel. This NavPanel is shown
    *  on the left side of us. It is Ok to return a NULL. If you
    *  return a NULL, your panel will take the full width of the window
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return NavPanel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel () {
      return( navPanel ) ;
   }




   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method should return a small icon
    *  that represents this panel. This icon is displayed
    *  in the task bar of the tool
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return ImageIcon
    *//*---------------------------------------------------------*/
   public ImageIcon getIcon () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remember that you should always use the                 */
      /* ImageSystem to get images. Also, the DOCUMENT_SMALL       */
      /* is defined in AppConst.java                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      return( ImageSystem.getImageIcon ( this, 
                                         ImageSystem.PRODUCT_SMALL ) ) ;
   }






   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an ActionEvent
    *  from our GUI is fired on us
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
      Vector  selectVec   = getSelection() ;
      boolean isBrandView = isBrandView() ;


      showingDrafts       = refreshShowingDrafts(selectVec) ;


      if ( Qit.getEnabled() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CLOSE = Frees our products and closes                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( e.getSource() == pb_CLOSE )
            parentWin.closePanel( this, null ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_FIND  = Find a Product                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_FIND ) {
            if ( ( selectVec != null ) && (selectVec.size() > 0 ) ) {
               findProductComplete( new ProductSearchDlg( getSelection() ).getResult() );
               //new ProductSearchDlg( getSelection() ) ;
               //ProductSearchDlg.getInstance().addFindProductListener( this ) ;
            }

            else {
               findProductComplete( new ProductSearchDlg().getResult() );
               //new ProductSearchDlg() ;
               //ProductSearchDlg.getInstance().addFindProductListener( this ) ;
            }

         }

         /*--------------*/
         /*              */
         /*  pb_PROPERTY */
         /*              */
         /*--------------*/
         else if ( e.getSource() == pb_PROPERTY ) {
            new PropertyDlg ( parentWin ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_CUSTOM_QUERY                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_CUSTOM_QUERY )
            new CustomViewDlg ( parentWin, this ).getResult() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_UNLOCK = Unlocks products                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_UNLOCK )
            new Thread ( this, THREAD_UNLOCK_PRODUCTS ).start() ;



         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_WIZARD                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_WIZARD )
            fieldChanger() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_HISTORY                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_HISTORY ) {
            if ( ( getSelection() != null ) && (getSelection().size() > 0) ) {
               ProductRow productRow = (ProductRow)getSelection().elementAt(0) ;

               if ( productRow != null )
                  WindowSystem.createPanel ( new ProductHistoryPanel ( productRow.getProductInd(), productRow.isDraft(), this ) ) ;
            }
         }

         /*-------------*/
         /*             */
         /*  pb_PREVIEW */
         /*             */
         /*-------------*/
         else if ( e.getSource() == pb_PREVIEW )
            togglePreview () ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_COMPARE                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_COMPARE ) {
            Product left          = null;
            Product right         = null;
            Vector  productSelect = getSelection() ;
            int     vecSize       = 0 ;

            if ( productSelect != null ) {
               vecSize = productSelect.size();

               if ( vecSize > 2 )
                  vecSize = 2;

               for ( int x = 0; x < vecSize; x++ ) {
                  ProductRow productRow = (ProductRow)productSelect.elementAt(x);
                  int returnCode = productRow.readProduct( true );

                  if ( returnCode == 0 ) {
                     if ( productRow.isDraft() ) {
                        ProductDraft productDraft = (ProductDraft)productRow.getProduct() ;
                        if ( x == 0 )
                           left  = productDraft;
                        else
                           right = productDraft;
                     } else {
                        Product product  = productRow.getProduct() ;
                        if ( x == 0 )
                           left = product;
                        else
                           right = product;
                     }
                  } /* if  */

               } /* for  */

               if ( left != null && right != null )
                  WindowSystem.createPanel ( new ProductComparePanel ( left, right, this ) );
               else if ( left != null && right == null )
                  WindowSystem.createPanel ( new ProductComparePanel ( left, this ) );
               else
                  WindowSystem.createPanel ( new ProductComparePanel ( this ) );  
            } else
               GUISystem.printBox ( "Info", "You must select at least 1 product before attempting to compare. If you select <b>2</b>"+
                                    "products, then you will be comparing those 2 products" ) ;

         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_NEW = Creates new Product request                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_NEW )
            performNew() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_EDIT = Edits last selected Product                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( e.getSource() == pb_EDIT ) || ( e.getSource() == productTree ) ) {
            if ( pb_EDIT.isVisible() )
               edit() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_EXPORT                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_EXPORT ) {
            if ( PMViews.getViewName ( currentView ).equals ( Str.getStr ( STR_BRAND_FAMILY_SERIES ) ) ) {
               GUISystem.printBox ( "Info", "Exporting of Brand/Family/Series Mapping is not supported" ) ;
            } else if ( PMViews.getViewName ( currentView ).equals ( Str.getStr ( STR_GEOGRAPHY_COUNTRY ) ) ) {
               GUISystem.printBox ( "Info", "Exporting of Geography/Country Mapping is not supported" ) ;
            } else {
               Vector selectedProducts = getSelection() ;
               Vector allProducts      = getAllViewableProducts() ;

               if ( ( ( selectedProducts == null ) || ( selectedProducts.size() == 0 ) ) &&
                    ( ( allProducts == null ) || ( allProducts.size() == 0 ) ) &&
                    ( currentPMView == null ) )
                  GUISystem.printBox ( "Info", "You must have one of the following to export:<br><br>1. A view selected<br>2. At least 1 product selected<br>3. At least 1 product viewable" ) ;
               else {
                  if ( currentPMView.getFullSQL() != null )
                     new com.ibm.nzna.projects.qit.product.productExport.ExportWizard ( selectedProducts, 
                                                                                        allProducts, 
                                                                                        currentPMView ).getResult() ;
                  else
                     GUISystem.printBox ( "Info", "Exporting is not supported under the current view" ) ;
               }
            }
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_COMPLETE = Completes the selected Product requests   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_COMPLETE )
            completeProducts() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   PB_DELETE = Deletes the selected products               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_DELETE )
            deleteProducts() ;



         /*------------------*/
         /*                  */
         /*  ck_SHOW_PARTNUM */
         /*                  */
         /*------------------*/
         else if ( ( e.getSource() == ck_SHOW_PARTNUM ) && ( isBrandView ) ) {
            if ( !ck_SHOW_PARTNUM.isSelected() )
               productTree.setColumnWidth ( 1, 0 ) ;
            else
               productTree.setColumnWidth ( 1, 90 ) ;
            PropertySystem.putBool ( PROPERTY_PRODUCT_MGMT_SHOW_PARTNUM, ck_SHOW_PARTNUM.isSelected() ) ;
         }

         /*----------------------*/
         /*                      */
         /*  ck_SHOW_DESCRIPTION */
         /*                      */
         /*----------------------*/
         else if ( ( e.getSource() == ck_SHOW_DESCRIPTION ) && ( isBrandView ) ) {
            if ( !ck_SHOW_DESCRIPTION.isSelected() )
               productTree.setColumnWidth ( 2, 0 ) ;
            else
               productTree.setColumnWidth ( 2, 250 ) ;
            PropertySystem.putBool ( PROPERTY_PRODUCT_MGMT_SHOW_DESCRIPTION, ck_SHOW_DESCRIPTION.isSelected() ) ;
         }


         /*-------------------*/
         /*                   */
         /*  ck_SHOW_ANNOUNCE */
         /*                   */
         /*-------------------*/
         else if ( ( e.getSource() == ck_SHOW_ANNOUNCE ) && ( isBrandView ) ) {
            if ( !ck_SHOW_ANNOUNCE.isSelected() )
               productTree.setColumnWidth ( 3, 0 ) ;
            else
               productTree.setColumnWidth ( 3, 125 ) ;
            PropertySystem.putBool ( PROPERTY_PRODUCT_MGMT_SHOW_ANNOUNCE, ck_SHOW_ANNOUNCE.isSelected() ) ;
         }

         /*---------------------*/
         /*                     */
         /*  ck_SHOW_CREATEDATE */
         /*                     */
         /*---------------------*/
         else if ( ( e.getSource() == ck_SHOW_CREATEDATE ) && ( isBrandView ) ) {
            if ( !ck_SHOW_CREATEDATE.isSelected() )
               productTree.setColumnWidth ( 4, 0 ) ;
            else
               productTree.setColumnWidth ( 4, 160 ) ;
            PropertySystem.putBool ( PROPERTY_PRODUCT_MGMT_SHOW_CREATEDATE, ck_SHOW_CREATEDATE.isSelected() ) ;
         }

         /*------------------------*/
         /*                        */
         /*  ck_SHOW_LASTTOUCHEDBY */
         /*                        */
         /*------------------------*/
         else if ( ( e.getSource() == ck_SHOW_LASTTOUCHEDBY ) && ( isBrandView ) ) {
            if ( !ck_SHOW_LASTTOUCHEDBY.isSelected() )
               productTree.setColumnWidth ( 5, 0 ) ;
            else
               productTree.setColumnWidth ( 5, 120 ) ;
            PropertySystem.putBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDBY, ck_SHOW_LASTTOUCHEDBY.isSelected() ) ;
         }

         /*------------------------*/
         /*                        */
         /*  ck_SHOW_LASTTOUCHEDON */
         /*                        */
         /*------------------------*/
         else if ( ( e.getSource() == ck_SHOW_LASTTOUCHEDON ) && ( isBrandView ) ) {
            if ( !ck_SHOW_LASTTOUCHEDON.isSelected() )
               productTree.setColumnWidth ( 6, 0 ) ;
            else
               productTree.setColumnWidth ( 6, 160 ) ;
            PropertySystem.putBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDON, ck_SHOW_LASTTOUCHEDON.isSelected() ) ;
         }



      }
   }


   /*------------------------------------------------------------*/
   /* findProductComplete
    */
   /**
    *    This method will be called when the ProductSearchDlg makes a 
    *  search
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void findProductComplete ( Vector productVec ) {

      Vector insertVec = new Vector ( 10 ) ;

      if ( ( productVec != null ) && ( productVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = productVec.size() ;

         insertVec = new Vector ( 10, 10 ) ;

         while ( i < size ) {
            if ( productVec.elementAt(i) instanceof ProductRow )
               insertVec.addElement ( productVec.elementAt(i) ) ;
            i++ ;
         }

         setProductList ( Str.getStr ( STR_RESULTS ), insertVec ) ;

      }


   } /* findProductComplete */


   /*------------------------------------------------------------*/
   /* setProductList
    */
   /**
    *    This method will set the product list that
    *  is displayed.
    *  
    *
    *  @param descript The description of the root node that will show above the products in the tree
    *  @param productVec The Vector of products
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setProductList ( String descript, Vector productVec ) {
      StringTreeNode   rootNode       = new StringTreeNode ( descript ) ;
      DefaultTreeModel model          = new DefaultTreeModel ( rootNode ) ;
      String[]         columnHeadings = {
         "",
         Str.getStr ( STR_PRODUCT_PART_NUMBER ),
         Str.getStr ( STR_DESCRIPTION ),
         Str.getStr ( STR_ANNOUNCE_DATE ),
         Str.getStr ( STR_CREATE_DATE ),
         Str.getStr ( STR_LAST_TOUCHED_BY ),
         Str.getStr ( STR_LAST_TOUCHED_ON )
      };               


      refreshView() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our Action Buttons                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_CLOSE.setVisible(true) ;
      pb_NEW.setVisible(true);
      pb_EDIT.setVisible(true);
      pb_DELETE.setVisible(true);
      if ( ((ProductRow)productVec.elementAt(0)).isDraft() ) {
         pb_COMPLETE.setVisible(true) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our columns headings                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      productTree.setColumnHeadings ( columnHeadings ) ;

      /*------------------------------------------*/
      /*                                          */
      /*  Blank out the title and insert our data */
      /*                                          */
      /*------------------------------------------*/
      st_TITLE.setText ( "" ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   if productVec                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( productVec != null ) && ( productVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = productVec.size() ;

         while ( i < size ) {
            rootNode.add ( new MutableTreeWrapper ( productVec.elementAt(i) ) ) ;
            i++ ;
         } /* while */

      } /* if */



      /*---------------------------*/
      /*                           */
      /*  Set our data in the tree */
      /*                           */
      /*---------------------------*/
      productTree.setTitlePadding ( 10 ) ;
      productTree.getJTree().setModel ( model ) ;
      st_TITLE.setText ( descript ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the size                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      productTree.setColumnWidth ( 0, 15 ) ;
      productTree.setColumnWidth ( 1, PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_PARTNUM ) ? 90 : 0 ) ;
      productTree.setColumnWidth ( 2, PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_DESCRIPTION ) ? 250 : 0 ) ;
      productTree.setColumnWidth ( 3, PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_ANNOUNCE ) ? 125 : 0) ;
      productTree.setColumnWidth ( 4, PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_CREATEDATE ) ? 120 : 0) ;
      productTree.setColumnWidth ( 5, PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDON ) ? 120 : 0) ;
      productTree.setColumnWidth ( 6, PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDBY ) ? 140 : 0) ;
      if ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_DESCRIPTION ) )
         productTree.setColumnWidth ( 2, 251 ) ;

   } /* setProductList */



   /*------------------------------------------------------------*/
   /* getNewCustomView
    */
   /**
    *    This method simply returns a blank CustomView
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return CustomView
    *//*---------------------------------------------------------*/
   public CustomView getNewCustomView () {
      ProductCustomView customView = new ProductCustomView() ;

      customView.updateRecStatus ( QITRec.REC_INSERT ) ;

      return( customView ) ;
   }






   /*------------------------------------------------------------*/
   /* run
    */
   /**
    *    This thread can perform a variety of operations.
    *  The operation performed is picked from the name
    *  of the thread
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void run () {
      String operation = Thread.currentThread().getName() ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH_DATA                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_REFRESH_VIEW ) ) {
         parentWin.setStatus ( STR_PLEASE_WAIT ) ;
         refreshView() ;
         parentWin.setStatus ( null ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESHVIEW_NODE                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( operation.equals ( THREAD_REFRESHVIEW_NODE ) )
         refreshViewNode() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_UNLOCK_PRODUCTS                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( operation.equals ( THREAD_UNLOCK_PRODUCTS ) ) {
         parentWin.setStatus ( Str.getStr ( STR_UNLOCKING ) ) ;
         unlockProducts() ;
         parentWin.setStatus ( null ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH_PREVIEW                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( operation.equals ( THREAD_REFRESH_PREVIEW ) ) {
         refreshPreview () ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_COMPLETEREQUESTS                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( operation.equals ( THREAD_COMPLETEREQUESTS ) )
         completeProductsThread() ;
   } /* run  */



   /**
     *  This method will refresh the view under a node
     **/
   private void refreshViewNode () {
      parentWin.setStatus ( STR_PLEASE_WAIT ) ;
      if ( currentViewNode != null ) {
         currentPMView.refreshViewTree ( treeModel, 
                                         (ViewTreeNode)currentViewNode ) ;

         if ( ((ViewTreeNode)currentViewNode).getChildCount() == 1 )
            WinUtil.expandTreeNode ( productTree,
                                     ((DefaultMutableTreeNode)currentViewNode.getChildAt(0)) ) ;

         productTree.reset() ;
      }
      parentWin.setStatus ( null );
   }



   /*------------------------------------------------------------*/
   /* refreshView
    */
   /**
    *    If you notice, this method is only called from
    *  the run method. This is because all queries
    *  and anything that takes more than 1 second should be in a thread!
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshView () {
      PMViewModel viewModel = PMViews.getViewModel ( currentView, currentBrandRec ) ;                                          

      showTree() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Get our Model and PMView                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( viewModel != null ) {
         treeModel     = viewModel.treeModel ;
         currentPMView = viewModel.pmView ;                                  
      } else {
         currentPMView = null ;
         treeModel     = null ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set the properties on the tree                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setProductTreeProperties() ;
      setActionButtons() ;

      if ( treeModel != null ) {
         DefaultMutableTreeNode root = ((DefaultMutableTreeNode)treeModel.getRoot()) ;

         productTree.getJTree().setModel ( treeModel ) ;
         if ( currentBrandRec != null )
            st_TITLE.setText ( currentBrandRec.getDescript()+" - "+PMViews.getViewName ( currentView ) ) ;
         else
            st_TITLE.setText ( PMViews.getViewName ( currentView ) ) ;

         /*---------------------------------------------*/
         /*                                             */
         /*  Now, auto-expand any single row            */
         /*                                             */
         /*---------------------------------------------*/
         if ( root.getChildCount() == 1 )
            WinUtil.expandTreeNode ( productTree, (DefaultMutableTreeNode) root.getChildAt(0)) ;
      }

   }



   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set what brand all of the
    *  views are currently being refreshed with
    *  
    *
    *  @param brandRec The TypeCategoryRec
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setBrand ( TypeCategoryRec typeBrandRec ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the brand as changed, then change our tree           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( currentBrandRec != typeBrandRec ) {
         this.currentBrandRec = typeBrandRec ;
         currentView          = null ;
         productTree.getJTree().setModel ( createEmptyModel() ) ;    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our title                                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( currentBrandRec != null ) {
            st_TITLE.setText ( currentBrandRec.toString () ) ;
            PropertySystem.putInt ( PropertySystem.PROPERTY_LAST_SELECTED_BRAND, currentBrandRec.getInd() ) ;
         }


      }
   }



   /*------------------------------------------------------------*/
   /* setView
    */
   /**
    *    This method will set our view if it is not
    *  already set to the value specified
    *  
    *
    *  @param view The view as defined in OAViews
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setView ( String view ) {
      currentView = view ;
      new Thread ( this, THREAD_REFRESH_VIEW ).start() ;
   }


   /*------------------------------------------------------------*/
   /* setSort
    */
   /**
    *    This method will set the current sort that
    *  we are using
    *  
    *
    *  @param sort The new sort. This must be one of the values defined in OASort
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setSort ( int sort ) {
      this.sort = sort ;
      //PropertySystem.putInt ( PROPERTY_OANAV_SORT, sort ) ;                         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the view with the new sort                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      refreshViewWithSort() ;

   }




   /*------------------------------------------------------------*/
   /* refreshViewWithSort
    */
   /**
    *    This method will refresh all exanded data
    *  with the current sort
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void refreshViewWithSort() {
      int rowCount = productTree.getJTree().getRowCount() ;
      int rowIndex = 1 ;                              

      parentWin.setStatus ( STR_SORTING ) ;
      parentWin.setValueMax ( rowCount ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Walk each row, asking if it is expanded                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rowIndex < rowCount ) {
         if ( productTree.getJTree().isExpanded ( rowIndex ) ) {

            try {
               TreePath               pathRoot  = productTree.getJTree().getPathForRow ( rowIndex ) ;
               TreePath               path      = productTree.getJTree().getPathForRow ( rowIndex+1 ) ;
               DefaultMutableTreeNode node      = (DefaultMutableTreeNode)pathRoot.getLastPathComponent() ;                                             

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Ok, if the Object on this row is a MutableTreeWrapper,  */
               /* then we can sort this view                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( path.getLastPathComponent() instanceof MutableTreeNode ) {
                  int      childCount = node.getChildCount() ;
                  Vector   newDataVec = new Vector ( childCount ) ;

                  for ( int i = 0; i < childCount; i++ )
                     newDataVec.addElement ( ((MutableTreeWrapper)node.getChildAt(i)).getDataObject() ) ;

                  new RefreshTreeViewThread().populateTree ( treeModel, node, newDataVec ) ;
               }
            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
            }

         }

         rowIndex++ ;
         parentWin.setValue ( rowIndex ) ;
      }

      parentWin.setValue ( 0 ) ;
      parentWin.setStatus ( null ) ;
   }




   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called when the user
    *  wishes to refresh this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      new Thread ( this, THREAD_REFRESH_VIEW ).start() ;
   }



   /*------------------------------------------------------------*/
   /* closeTree
    */
   /**
    *    This method will close the tree items defined
    *  in the treePath param
    *  
    *
    *  @param treePath The path to close
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void closeTree ( TreePath treePath ) {
      productTree.getJTree().collapsePath ( treePath ) ;
   }




   /*-----------------------------------------------------------*/
   /* newProduct
    */
   /**
    *    This method will create a new product
    *  
    *  
    *             
    *  @param 
    *  @param 
    *  @param 
    *
    *  @return 
    */
   public static void createNewProduct ( AppDefaultWin appDefaultWin, ProductManagementPanel productManagementPanel ) {
      NewProductDlg newDlg  = new NewProductDlg ( appDefaultWin ) ;
      ProductDraft  draft   = null ;
      int           newType = newDlg.getResult() ;

      switch ( newType ) {
         case NewProductDlg.NEWPROD_TEMPLATE :
            draft = new ProductDraft ( newDlg.getTemplate() ) ;
            draft.setWorkRequired ( TypeList.WORKREQ_CREATE ) ;
            break ;

         case NewProductDlg.NEWPROD_BLANK :
            draft = new ProductDraft () ;
            draft.setProductType ( newDlg.getTypeProductRec().getInd() ) ;
            break ;

         case NewProductDlg.NEWPROD_IMPORT :
            String     args[]    = new String[3] ;

            if ( newDlg.getImportFilename() != null ) {
               args[0] = "/FILE:"+newDlg.getImportFilename() ;

               // get the Import Type
               if ( newDlg.getImportType()== newDlg.IMPORT_CREATE )
                  args[1] = "/INSERT" ;
               else if ( newDlg.getImportType() == newDlg.IMPORT_UPDATE )
                  args[1] = "/UPDATE" ;
               else
                  args[1] = "/INSERTUPDATE" ;                     

               //get the Product Type
               if ( newDlg.getImportProductType() == ProductImport.MONITORSONLY )
                  args[2] = "/MONITORS" ;
               else if ( newDlg.getImportProductType() == ProductImport.OPTIONSONLY )
                  args[2] = "/OPTIONS" ;
               else
                  args[2] = "/SYSTEMS" ;

               ProductImport batch = new ProductImport ( args ) ;

               BatchRunnerPanel  runner = new BatchRunnerPanel ( (Batch)batch ) ;  

               /* Make the Commit and Rerun buttons visible on the BatchRunnerPanel */
               runner.showCommitBtn(true);
               runner.showRerunBtn(true);                    

               batch.setButtons( runner.getCommitButton(), runner.getRerunButton()) ;

               WindowSystem.createPanel ( runner ) ;

            }
            break ;
      }

      if ( draft != null )
         EditProduct.editProduct ( draft ) ;

   } /* newProduct */


   /*------------------------------------------------------------*/
   /* treeCollapsed
    */
   /**
    *    This method will be called when a tree
    *  has been collapsed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void treeCollapsed ( TreeExpansionEvent e ) {
   }



   /*------------------------------------------------------------*/
   /* treeExpanded
    */
   /**
    *    This method will be called when a branch
    *  in our tree has been expanded
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void treeExpanded ( TreeExpansionEvent e ) {
      Object expandedObject = e.getPath().getLastPathComponent() ;

      if ( ( expandedObject != null ) && ( expandedObject instanceof ViewTreeNode ) ) {
         currentViewNode = (ViewTreeNode)expandedObject ;

         new Thread ( this, THREAD_REFRESHVIEW_NODE ).start() ;
      }
   }



   /**
     *  This method will edit the currently selected product
     **/
   private void editCurrentProduct () {
      TreePath path = productTree.getJTree().getSelectionPath() ;

      if ( path != null ) {
         MutableTreeWrapper selectedItem = (MutableTreeWrapper)path.getLastPathComponent() ;

         try {
            ProductRow         productRow   = (ProductRow) selectedItem.getDataObject() ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Only process if a product is selected                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            parentWin.setStatus ( "Reading "+productRow.getDescription() ) ;
            if ( productRow != null )
               EditProduct.editProduct ( productRow ) ;

         } catch ( Exception e ) {
         }

      }

      parentWin.setStatus ( null );
   }


   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      if ( !showingDrafts ) {
         Vector selection = getSelection();

         if ( ( selection != null ) && ( selection.size() > 0 ) ) {
            int    i           = 0 ;
            int    size        = selection.size() ;
            Vector bookmarkVec = new Vector ( size ) ;



            while ( i < size ) {
               Bookmark bookmark = new Bookmark() ;
               bookmark.setDescript ( ((ProductRow)selection.elementAt(i)).getDescription() ) ;
               bookmark.setObject ( new Integer ( ((ProductRow)selection.elementAt(i)).getProductInd() ),
                                    BOOKMARK_PRODUCT ) ;
               bookmark.updateRecStatus ( PersistentRec.REC_INSERT ) ;
               bookmarkVec.addElement ( bookmark ) ;
               i++ ;
            }

            BookmarkSystem.guiAddBookmarks ( bookmarkVec ) ;

         } else
            GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                                 Str.getStr ( STR_INFO ),
                                 Str.getStr ( STR_MUST_SELECT_ITEM_TO_BOOKMARK ) ) ;

      } else
         GUISystem.printBox ( GUISystem.getParentDefWin ( this ),
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_CANNOT_BOOKMARK_DRAFTS ) ) ;
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* getSelection           
    */
   /**
    *    This method will return a vector of all
    *  selected products
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector getSelection() {
      Vector   prodSelectVec       = null ;
      int[]    selectedRows = productTree.getJTree().getSelectionRows() ;


      if ( ( selectedRows != null ) && ( selectedRows.length > 0 ) ) {
         prodSelectVec = new Vector ( selectedRows.length ) ;

         for ( int i = 0; i < selectedRows.length; i++ ) {
            try {
               Object object = ((MutableTreeWrapper)productTree.getJTree().getPathForRow ( selectedRows[i] ).getLastPathComponent()).getDataObject() ;

               if ( object instanceof ProductRow )
                  prodSelectVec.addElement ( object  ) ;
            } catch ( ClassCastException e ) {
               // Do nothing
            }

         }
      }


      return( prodSelectVec ) ;
   }


   /*------------------------------------------------------------*/
   /* getSelection           
    */
   /**
    *    This method will return a vector of all
    *  selected products
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private Vector getAllViewableProducts() {
      Vector   viewableProducts = null ;
      int      rowCount      = productTree.getJTree().getRowCount() ;


      if ( rowCount > 0 ) {
         viewableProducts = new Vector ( rowCount ) ;

         for ( int i = 0; i < rowCount; i++ ) {
            try {
               Object object = ((MutableTreeWrapper)productTree.getJTree().getPathForRow ( i ).getLastPathComponent()).getDataObject() ;

               if ( object instanceof ProductRow )
                  viewableProducts.addElement ( object  ) ;
            } catch ( ClassCastException e ) {
               // Do nothing
            }
         }
      }


      return( viewableProducts ) ;
   }









   /**
     *  This method will simply create an empty DefaultTreeModel
     **/
   private DefaultTreeModel createEmptyModel () {
      StringTreeNode   root  = new StringTreeNode ( Str.getStr ( STR_SELECT_A_VIEW ) ) ;
      DefaultTreeModel model = new DefaultTreeModel ( root ) ;

      return( model ) ;
   }



   /**
     *  This method will adjust the productTree JTree settings for the current brand
     * and view based on the currentBrandRec. 
     **/
   private void setProductTreeProperties () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the currentView is not null, contrinue               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( currentView != null ) {
         int[] columnWidths = currentPMView.getColumnWidths() ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the column headings and widths from the view        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         productTree.setColumnHeadings ( currentPMView.getColumnNames() ) ;

         for ( int i = 0; i < columnWidths.length ; i++ )
            productTree.setColumnWidth ( i, columnWidths[i] ) ;

         productTree.setTitlePadding ( currentPMView.getTitlePadding() ) ;      

      } /* if currentView  */
   }


   /**
     *  This method will adjust the New,Edit,Complete buttons according to the view select
     **/
   private void setActionButtons() {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Data Mapping                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( PMViews.getViewName ( currentView ).equals ( Str.getStr ( STR_BRAND_FAMILY_SERIES ) ) ) ||
           ( PMViews.getViewName ( currentView ).equals ( Str.getStr ( STR_GEOGRAPHY_COUNTRY ) ) ) ) {
         pb_COMPLETE.setVisible ( false ) ;
         pb_NEW.setVisible ( true ) ;
         pb_EDIT.setVisible ( true ) ;
         pb_DELETE.setVisible ( false ) ;
         pb_WIZARD.setEnabled ( false ) ;
         pb_HISTORY.setEnabled ( false ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Misfit Views                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( PMViews.getViewName ( currentView ).indexOf ( Str.getStr ( STR_MISFITS ) ) != -1 ) {
         pb_COMPLETE.setVisible ( false ) ;
         pb_NEW.setVisible ( false ) ;
         pb_EDIT.setVisible ( false ) ;
         pb_DELETE.setVisible( false ) ;
         pb_WIZARD.setEnabled ( false ) ;
         pb_HISTORY.setEnabled ( false ) ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Any Draft View                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( PMViews.getViewName ( currentView ).indexOf ( Str.getStr ( STR_DRAFT ) ) != -1 ) {
         pb_COMPLETE.setVisible ( true ) ;
         pb_NEW.setVisible ( true ) ;
         pb_EDIT.setVisible ( true ) ;
         pb_DELETE.setVisible ( true ) ;
         pb_WIZARD.setEnabled ( true ) ;
         pb_HISTORY.setEnabled ( true ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Any other view                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else {
         pb_COMPLETE.setVisible ( false ) ;
         pb_NEW.setVisible ( true ) ;
         pb_EDIT.setVisible ( true ) ;
         pb_DELETE.setVisible ( true ) ;
         pb_WIZARD.setEnabled ( true ) ;
         pb_HISTORY.setEnabled ( true ) ;
      }


   }




   /*------------------------------------------------------------*/
   /* setCustomView
    */
   /**
    *    This method will set our custom view if it is not
    *  already set to the value specified
    *  
    *
    *  @param view The custom view description
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected void setCustomView ( String view ) {

      Vector dataVec = new Vector (1) ;
      CustomView customView = CustomViewSystem.getView( view ) ;

      if ( customView != null )
         dataVec = customView.getResults() ;

      setProductList ( Str.getStr ( STR_RESULTS ), dataVec ) ;

   }



   /**
     * This method kicks off the Product Managmenet Mass Field Changer
     **/
   private void fieldChanger() {
      Vector       productDraftVec       = new Vector();
      Vector       selectedProductRowVec = new Vector();
      ProductDraft productDraft          = null;
      StatusWin    statusWin             = new StatusWin ( GUISystem.getParentDefWin ( this ) )  ;
      Vector finalDraftVec=new Vector();

      if ( UserSystem.hasAuthority(AUTH_AUTHOR) ) {
         try {
            /* First, check to see that they have something selected */
            if ( haveSelection() ) {
               selectedProductRowVec = getSelection() ;
            }

            statusWin.setVisible ( true ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Procees ProductDrafts for selected productrows          */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( (selectedProductRowVec!=null) && (selectedProductRowVec.size()>0) ) {

               statusWin.setMaxValue ( selectedProductRowVec.size() ) ;

               for ( int i = 0; i < selectedProductRowVec.size();i++ ) {
                  if ( selectedProductRowVec.elementAt(i) instanceof ProductRow ) {
                     ProductRow productRow=(ProductRow)selectedProductRowVec.elementAt(i);

                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Checking whether selected product rows are ProductDrafts*/
                     /*   or ExistingProducts.If they are drafts get those drafts */
                     /*   for masschange.                                         */
                     /*-----------------------------------------------------------*/
                     if ( productRow.isDraft() ) {
                        int returnCode = 0;

                        statusWin.setText ( Str.getStr ( STR_READING )+" "+productRow.getDescription() ) ;
                        returnCode = productRow.readProduct(true);


                        if ( returnCode==0 ) {
                           productDraft=(ProductDraft)productRow.getProduct();
                           productDraftVec.addElement(productDraft);
                        } else {
                           LogSystem.log(1,"Problem while reading productDraft"+productDraft.getDescription());
                        }

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   if selected product rows are existing products then we  */
                        /*   need to creates product drafts for them for masschange  */
                        /*                                                           */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                     } else {
                        statusWin.setText ( Str.getStr ( STR_READING )+" "+productRow.getDescription() ) ;
                        CreateProductDraft createProductDraft=ProductDraft.createProductDraft(productRow.getProductInd());
                        if ( createProductDraft!=null ) {
                           ProductDraft newProductDraft= createProductDraft.getProductDraft();
                           int code  = 0 ;


                           statusWin.setText ( Str.getStr ( STR_SAVING )+" "+productRow.getDescription() ) ;
                           code = newProductDraft.writeToDatabase();

                           if ( code==0 ) {
                              productDraftVec.addElement(newProductDraft);
                           }

                           else {
                              LogSystem.log(1,"Problem while writing productDraft"+productDraft.getProductInd());
                           }
                        }
                     }
                  }
                  statusWin.setValue ( i ) ;
               }
            } else {
               GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM );
            }

            statusWin.dispose() ;
            if ( (productDraftVec!=null) && (productDraftVec.size()>0) ) {
               for ( int i=0;i<productDraftVec.size();i++ ) {
                  ProductDraft draft=(ProductDraft)productDraftVec.elementAt(i);
                  if ( !productCheckedOut(draft) ) {
                     try {
                        draft.checkOut(UserSystem.getUserId());
                        finalDraftVec.addElement(productDraftVec.elementAt(i));
                     } catch ( Exception e ) {
                        LogSystem.log(1,e,false);
                     }
                  } else {
                     GUISystem.printBox("Info","Your selection has locked products."+
                                        "Those are not processed for masschange."+
                                        "Press OK to continue masschange for unlocked products!!!");
                  }
               }
            }

            if ( (finalDraftVec!=null) && (finalDraftVec.size()>0) ) {
               FieldChangerWizard fieldChangerWizard=new FieldChangerWizard(GUISystem.getParentDefWin ( this ),productDraftVec);
               fieldChangerWizard.getResult();
            }

            /* After Performing Masschange  on productDrafts set the checkOut to null*/

            if ( (finalDraftVec!=null) && (finalDraftVec.size()>0) ) {
               for ( int i=0;i<finalDraftVec.size();i++ ) {
                  ProductDraft draft=(ProductDraft)finalDraftVec.elementAt(i);
                  try {
                     draft.checkOut(null);
                  } catch ( Exception e ) {
                     LogSystem.log(1,e,false);
                  }
               }
            }



         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
         }


         statusWin.dispose() ;

      } else
         GUISystem.printBox ( STR_INFO, STR_NO_AUTH_FIELD_CHANGER ) ;
   } /* fieldChanger  */


   /**
     *  This method will will give the printing support to this panel.
     **/
   public Object getPrintObject() {
      if ( treeModel != null )
         return( new Print ( productTree.getJTree(), parentWin, Str.getStr ( STR_PRODUCTS ) ) ) ;
      else {
         GUISystem.printBox ( "Info", "Please navigate to a Product View before printing" ) ;
      }
      return( null ) ;
   }

   /**
     *  This method will will give the Preview support to this panel.
     **/
   public Object getPreviewObject() {
      if ( treeModel != null )
         return( new Preview ( productTree.getJTree(), parentWin, Str.getStr ( STR_PRODUCTS ) ) ) ;
      else {
         GUISystem.printBox ( "Info", "Please navigate to a Product View before printing" ) ;
      }
      return( null ) ;
   }



   /**
     * This method simply returns TRUE if at least 1 item is selected in the
     * product list
     **/
   private boolean haveSelection() {
      if ( (getSelection()!=null) && (getSelection().size()>0) ) {
         return(true);
      } else
         return(false);
   }


   /*------------------------------------------------------------*/
   /* unlockProducts
    */
   /**
    *    This method will unlock all selected products
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void unlockProducts () {
      if ( haveSelection () ) {
         String     qualifier  = showingDrafts ? "PRODRAFT" : "PRODUCT" ;
         Vector     products   = getSelection() ;
         StatusWin  statusWin  = null ;
         int        productInd = 0 ;
         String     sqlString  = "UPDATE "+qualifier+".PRODUCT SET CHECKEDOUTBY=NULL WHERE PRODUCTIND IN (" ;

         for ( int i = 0; i < products.size(); i++ ) {
            productInd = showingDrafts ? ((ProductRow)products.elementAt(i)).getExistingProductInd() :
                         ((ProductRow)products.elementAt(i)).getExistingProductInd() ;
            sqlString+=+productInd ;
            if ( i < (products.size()-1) )
               sqlString+="," ;
         }

         sqlString+=")" ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, run it                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sqlString ) == 0 )
            GUISystem.printBox ( Str.getStr ( STR_INFO ), "Products Unlocked" ) ;
         else
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;

      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;
   }




   /**
     * If we are showing our help start page, shut that off and
     * show the JTree. Otherwise do nothing
     **/
   private void showTree () {
      if ( st_FIRST_TEXT != null ) {
         contentPane.remove ( st_FIRST_TEXT ) ;
         st_FIRST_TEXT=null ;
         contentPane.add ( new JScrollPane ( productTree ), BorderLayout.CENTER ) ;
         revalidate() ;
      }
   }



   /**
     * This method will toggle the preview on or off depending on the current
     * preview setting
     **/
   private void togglePreview () {
      boolean currentPreview = PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_PREVIEW ) ;


      /*------------------------------------*/
      /*                                    */
      /*  If the preview is on, turn it off */
      /*                                    */
      /*------------------------------------*/
      if ( currentPreview ) {
         pb_PREVIEW.setText ( Str.getStr ( STR_PREVIEW_ON ) ) ;
         remove ( splitPane ) ;
         splitPane.remove ( contentPane ) ;
         add ( contentPane, BorderLayout.CENTER ) ;
         revalidate() ;
      }


      /*------------------------------------*/
      /*                                    */
      /*  If the preview is off, turn it on */
      /*                                    */
      /*------------------------------------*/
      else {
         pb_PREVIEW.setText ( Str.getStr ( STR_PREVIEW_OFF ) ) ;
         remove ( contentPane ) ;
         splitPane.setTopComponent ( contentPane ) ;
         add ( splitPane, BorderLayout.CENTER ) ;
         splitPane.setDividerLocation ( (int) getSize().height-200 ) ;
         revalidate() ;

         if ( haveSelection() )
            new Thread ( this, THREAD_REFRESH_PREVIEW ).start() ;
      }

      PropertySystem.putBool ( PROPERTY_PRODUCT_MGMT_PREVIEW, !currentPreview ) ;
   }


   /**
     *  This method will refresh the preview window with whatever product is
     * selected.
     **/
   private void refreshPreview () {
      TreePath path = productTree.getJTree().getSelectionPath() ;

      pb_EDIT.setEnabled ( false ) ;
      pb_HISTORY.setEnabled ( false ) ;
      pb_COMPARE.setEnabled ( false ) ;
      pb_DELETE.setEnabled  ( false ) ;

      if ( path != null ) {
         MutableTreeWrapper selectedItem = (MutableTreeWrapper)path.getLastPathComponent() ;

         try {
            ProductRow         productRow   = (ProductRow) selectedItem.getDataObject() ;

            if ( productRow.readProduct ( false ) == 0 )
               previewPanel.setProduct  ( productRow.getProduct() ) ;

         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
         }
      }
      pb_EDIT.setEnabled ( true ) ;
      pb_HISTORY.setEnabled ( true ) ;
      pb_COMPARE.setEnabled ( true ) ;
      pb_DELETE.setEnabled  ( true ) ;
   }

   public void valueChanged ( TreeSelectionEvent e ) {
      if ( PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_PREVIEW ) )
         new Thread ( this, THREAD_REFRESH_PREVIEW ).start() ;
   }




   /**
     *  This method will perform the edit on whatever view is currently up
     **/
   private void edit () {

      if ( productTree.getJTree().getSelectionPath() != null ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Brand/Family/Series Mapping Edit                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( PMViews.getViewName ( currentView ).equals ( Str.getStr ( STR_BRAND_FAMILY_SERIES ) ) ) {
            TreePath path = productTree.getJTree().getSelectionPath() ;

            if ( path != null ) {
               MutableTreeWrapper selectedItem = (MutableTreeWrapper)path.getLastPathComponent() ;
               BrandMappingRow    mapRow       = (BrandMappingRow) selectedItem.getDataObject() ;
               new BrandMapDlg ( parentWin, mapRow ) ;
               productTree.invalidate() ;
               revalidate() ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Geo/Country Edit                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( PMViews.getViewName ( currentView ).equals ( Str.getStr ( STR_GEOGRAPHY_COUNTRY ) ) ) {
            TreePath path = productTree.getJTree().getSelectionPath() ;

            if ( path != null ) {
               MutableTreeWrapper selectedItem = (MutableTreeWrapper)path.getLastPathComponent() ;
               CountryMappingRow  mapRow       = (CountryMappingRow) selectedItem.getDataObject() ;
               new CountryMapDlg ( parentWin, mapRow ) ;
               productTree.invalidate() ;
               revalidate() ;
            }
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Normal Product Edit                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else
            editCurrentProduct() ;
      } else
         GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
   }


   /**
     * This method will determine if the view we are currently showing is a brand view
     **/
   private boolean isBrandView () {
      if ( currentView != null ) {
         return( ( !PMViews.getViewName(currentView).equals ( Str.getStr ( STR_GEOGRAPHY_COUNTRY ) ) ) &&
                 ( !PMViews.getViewName(currentView).equals ( Str.getStr ( STR_BRAND_FAMILY_SERIES ) ) ) ) ;
      }

      return( false ) ;
   }

   /**
     *  This method will perform the 'New' operation for whatever view is currently selected
     **/
   private void performNew () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Brand/Family mapping                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( PMViews.getViewName ( currentView ).equals ( Str.getStr ( STR_BRAND_FAMILY_SERIES ) ) ) {
         BrandMappingRow mapRow = new BrandMappingRow () ;

         new BrandMapDlg ( parentWin, mapRow ) ;
         productTree.invalidate() ;
         refresh() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Geo/CountryMapping                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( PMViews.getViewName ( currentView ).equals ( Str.getStr ( STR_GEOGRAPHY_COUNTRY ) ) ) {
         CountryMappingRow mapRow = new CountryMappingRow () ;

         new CountryMapDlg ( parentWin, mapRow ) ;
         productTree.invalidate() ;
         refresh() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Default Product Creation                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else
         createNewProduct ( GUISystem.getParentDefWin ( this ), this ) ;

   }

   /*------------------------------------------------------------*/
   /* delete
    */
   /**
    *    This method will try the delete operation
    *  on each selected product
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void delete () {
      StatusWin statusWin = new StatusWin ( parentWin ) ;

      parentWin.setStatus ( Str.getStr ( STR_DELETING ) ) ;

      try {

         statusWin.setTitle ( Str.getStr ( STR_PLEASE_WAIT ) );
         statusWin.setVisible ( true ) ;

         /* Check to see if they selected anything */
         if ( haveSelection() ) {
            boolean      hasAuth        = UserSystem.hasAuthority ( AUTH_AUTHOR ) ;
            ProductRow   productRow     = null ;
            int          i              = 0 ;
            Vector       products       = getSelection() ;
            int          length         = products.size() ;
            int          rc             = 0 ;
            int          actualAdded    = 0 ;

            showingDrafts  = refreshShowingDrafts(products) ; 

            statusWin.setMaxValue ( length ) ;                                              


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we are showing drafts, simply set the recycled flags */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( showingDrafts ) {
               StringBuffer sql = new StringBuffer ( "UPDATE PRODRAFT.PRODUCT SET RECYCLED = 'Y', LASTTOUCHEDBY = '"+UserSystem.getUserId()+"', LASTTOUCHEDON = CURRENT TIMESTAMP WHERE PRODUCTIND IN (") ;

               if ( hasAuth ) {
                  /*-----------------------------------------------------------*/
                  /*                                                           */
                  /*   Loop through and add each PRODUCT indicator, ONLY if    */
                  /* product indicator goes to a draft whose source is not     */
                  /* opicm.                                                    */
                  /*                                                           */
                  /*-----------------------------------------------------------*/
                  while ( i < length ) {
                     if ( !productSourceIsOPICM ( (ProductRow)products.elementAt(i) ) ) {
                        sql.append ( ((ProductRow)products.elementAt(i)).getProductInd() ) ;
                        if ( i < (length-1) )
                           sql.append ( "," ) ;
                        actualAdded++ ;
                     } else
                        GUISystem.printBox ( "Info", "You cannot delete product '"+((ProductRow)products.elementAt(i)).getDescription()+"' "+
                                             "because it's source is OPIC/M." ) ;
                     i++ ;
                  }
                  sql.append ( ")" ) ;  


                  if ( actualAdded > 0 ) {
                     /*-----------------------------------------------------------*/
                     /*                                                           */
                     /*   Run the SQL                                             */
                     /*                                                           */
                     /*-----------------------------------------------------------*/
                     rc = SqlRunner.runSingleSQL ( DatabaseSystem.QUEST, sql.toString() ) ;

                     if ( rc == 0 )
                        GUISystem.printBox ( STR_INFO, STR_COMPLETE ) ;
                     else {
                        LogSystem.log ( 1, "Could not update products to recycled. rc="+rc ) ;
                        GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                     } 
                  }
               } else
                  GUISystem.printBox ( "Info", "You do not have the proper authority to perform this action" ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we are showing Active Products, then create a        */
            /* request for delete for them                               */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {
               int success = 0 ;

               while ( i < length ) {
                  productRow = (ProductRow) products.elementAt(i) ;
                  statusWin.setText ( Str.getStr ( STR_READING )+" "+productRow.getDescription() ) ;
                  rc = productRow.readProduct( true ) ;
                  if ( rc == 0 ) {
                     ProductDraft draft = EditProduct.createDraft ( productRow.getProduct() ) ;

                     if ( draft != null ) {
                        EditProduct.setDefaultValues ( draft ) ;
                        draft.setWorkRequired ( TypeList.WORKREQ_DELETE ) ;


                        statusWin.setText ( Str.getStr ( STR_SAVING )+" "+productRow.getDescription() ) ;
                        if ( !ProductSaver.saveDraft ( draft, null ) ) {
                           GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                           i=length+1 ;
                        } else {
                           try {
                              draft.checkOut ( null ) ;
                              success++ ;
                           } catch ( Exception e ) {
                              LogSystem.log ( 1, e, false ) ;
                           }
                        }


                     }
                  } else {
                     GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                     LogSystem.log ( 1, "Could not read product. rc:"+rc ) ;
                  }
                  i++ ;
               }

               if ( success > 0 ) {
                  if ( success > 1 )
                     GUISystem.printBox ( "Info", "You have just created "+success+" drafts for deletion. To complete the deletion process please complete the drafts" ) ;
                  else
                     GUISystem.printBox ( "Info", "You have just created 1 drafts for deletion. To complete the deletion process please complete the drafts" ) ;
               }
            } /* else, has products  */
         } /* have Selection */
         else
            GUISystem.printBox ( STR_ERROR, STR_MUST_SELECT_ITEM ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

      parentWin.setStatus ( null ) ;
      statusWin.dispose() ;


   }

   /*------------------------------------------------------------*/
   /* deleteProducts
    */
   /**
    *    This method will try to either create a request
    *  for deletion on all selected existing products,
    *  or will delete the selected product requests
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void deleteProducts() {
      if ( haveSelection() ) {
         boolean rc = false ;

         /* Check the user's authority. If they don't have author,
            they cannot delete a request, or create a request for delete */
         if ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) {
            if ( showingDrafts )
               rc = GUISystem.printBox ( Str.getStr ( STR_ARE_YOU_SURE_DELETE ), "Are you sure you wish to move the product drafts to the recycle bin?" ) ;
            else
               rc = GUISystem.printBox ( "Are You Sure?", "Are you sure you wish to create drafts for the deletetion of the selected products?" ) ;
         } else {

            if ( !showingDrafts )
               rc = GUISystem.printBox ( "Are You Sure?", "Are you sure you wish to create drafts for the deletetion of the selected products?" ) ;
            else
               GUISystem.printBox ( Str.getStr ( STR_INFO ), "You don't have the authority to do that" ) ;

         }

         /* Ok, if the user answered yes to our questions, delete! */
         if ( rc )
            delete() ;

      } /* if products are selected */
      else
         GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
   } /* deleteProducts */


   /**
     * This is the actual method that will complete products in a thread
     **/
   private void completeProductsThread() {
      StatusWin statusWin = new StatusWin ( parentWin ) ;        

      try {
         int          i                  = 0 ;
         Vector       selectedProductVec = getSelection() ;
         int          length             = selectedProductVec.size() ;
         boolean      cont               = true ;
         ProductRow   productRow         = null ;
         Vector       badProducts        = new Vector ( 1 ) ;
         String       comments           = null ;


         /*---------------------------------*/
         /*                                 */
         /*  Get the comments from the user */
         /*                                 */
         /*---------------------------------*/
         comments = new SingleMLEDlg ( parentWin, 
                                       Str.getStr ( STR_PLEASE_ENTER_COMMENTS ),
                                       Str.getStr ( STR_PLEASE_ENTER_COMMENTS_FOR_PUBLISH ),
                                       "",
                                       "" ).getResult() ;

         if ( ( comments != null ) && ( comments.length() > 0 ) ) {

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Set up our status                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            statusWin.setTitle ( Str.getStr ( STR_PLEASE_WAIT ) ) ;
            statusWin.setMaxValue ( length ) ;                             

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Loop through and publish each product                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            while ( ( i < length ) && ( cont ) ) {

               productRow = (ProductRow) selectedProductVec.elementAt(i) ;

               if ( productRow != null ) {

                  if ( productRow.isDraft() ) {
                     int rc = -1 ;


                     statusWin.setText ( Str.getStr ( STR_READING )+" "+productRow.getDescription() ) ;
                     rc = productRow.readProduct ( false ) ;

                     if ( rc == 0 ) {
                        ProductDraft draft = (ProductDraft)productRow.getProduct() ;


                        statusWin.setText ( Str.getStr ( STR_PUBLISHING )+" "+productRow.getDescription() ) ;
                        draft.setComment ( comments ) ;
                        draft.updateRecStatus ( ProductDraft.REC_UPDATE ) ;
                        if ( ProductSaveCheck.checkPublish ( draft, statusWin, false ) ) {
                           if ( !ProductSaver.publishDraft ( draft, null ) ) {
                              GUISystem.printBox ( "Error", "Could not republish "+productRow.getProductInd() ) ;
                              cont = false ;
                           }
                        } else
                           badProducts.addElement ( productRow ) ;

                     }
                  }

               }

               i++ ;
               statusWin.setValue ( i ) ;

               /* Run the Garbage Collector */
               System.gc() ;
            }

            refresh() ;                                     

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   IF we have bad products that did not publish, show them */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( badProducts.size() > 0 ) {
               if ( badProducts.size() == 1 ) {
                  if ( GUISystem.printBox ( "Question", 
                                            "The product did not publish because "+
                                            "it is missing data. Would you like "+
                                            "to edit that product?" ) ) {
                     productRow = (ProductRow)badProducts.elementAt ( 0 ) ;

                     parentWin.setStatus ( "Reading "+productRow.getDescription() ) ;
                     EditProduct.editProduct ( productRow ) ;
                  }
               } else {
                  GUISystem.printBox ( "Info", "There were "+badProducts.size()+" products which did not publish because they are missing data. Press 'Ok' to see the list of products" ) ;
                  new ProductDlg ( GUISystem.getParentDefWin ( this ), badProducts ) ;
               }
            }
         } else
            GUISystem.printBox ( "Info", "You must enter a comment before completing the drafts" ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1 ,e ,false ) ;
      }

      statusWin.dispose() ;
   }



   /*------------------------------------------------------------*/
   /* completeProducts
    */
   /**
    *    This method will complete all selected
    *  requests
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void completeProducts () {
      if ( haveSelection() ) {

         /* Only complete the requests */
         if ( showingDrafts ) {

            new Thread ( this, THREAD_COMPLETEREQUESTS ).start() ;

         } else
            GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_CAN_ONLY_COMPLETE_REQUEST ) ) ;

      } /* if */
      else
         GUISystem.printBox ( Str.getStr ( STR_INFO ), Str.getStr ( STR_MUST_SELECT_ITEM ) ) ;
   } /* completeProducts */   

   /*------------------------------------------------------------*/
   /* refreshShowingDrafts
    */
   /**
    *    This method will go through a vector of
    *  ProductRows and tries to determine if any
    *  products in the vector are requests. If 1 product is
    *  a request, then the showingDrafts property is updated to TRUE,
    *  otherwise it is updated to FALSE ;
    *
    *  @param vector The list of products
    *  @param None
    *  @param None
    *
    *  @return TRUE = Showing Requets, FALSE = not showing requests
    *//*---------------------------------------------------------*/
   private boolean refreshShowingDrafts ( Vector products ) {

      showingDrafts = false ;

      /* Ok, now if we have any data in the
         vector, check each record to see if
         any of them are requests. If any are,
         set showingDraft = true */
      if ( products != null ) {
         int    i         = 0 ;
         int    length    = products.size() ;

         length = products.size() ;
         while ( ( i < length ) && ( !showingDrafts ) ) {
            if ( products.elementAt(i) instanceof ProductRow )
               showingDrafts = ((ProductRow)products.elementAt(i)).isDraft() ;
            i++ ;
         } /* while */

      } /* if */

      return( showingDrafts ) ;
   }


   /*------------------------------------------------------------*/
   /* getHelpFilename
    */
   /**
    *    This method will return our help filename
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getHelpFilename () {
      return( "Products/ProdMgmt.html" ) ;
   }

   /**
     *  This method will return TRUE if the product being edited is locked
     **/
   private boolean productCheckedOut(ProductDraft draft) {
      String checkedOutBy = draft.getCheckedOutBy () ;

      if ( checkedOutBy == null )
         return( false );

      return( !checkedOutBy.equals (UserSystem.getUserId() ) ) ;
   }


   /**
     * This method will return TRUE if the source of a product Row
     * passed in is OPICM
     **/
   private boolean productSourceIsOPICM ( ProductRow productRow ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "productSourceIsOPICM", 5 ) ;
      boolean   rc        = false ;

      try {
         Statement stmt = sqlMethod.createStatement () ;
         ResultSet rs   = stmt.executeQuery ( "SELECT SOURCE FROM PRODRAFT.PRODUCT WHERE PRODUCTIND = "+productRow.getProductInd() ) ;

         if ( rs.next() ) {
            String source = rs.getString(1) ;

            rc = ( source.indexOf ( "OPIC/M" ) != -1)  ;
         }

         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e ,false ) ;
      }

      sqlMethod.close() ;

      return( rc ) ;
   }




   /**
     * This method will simply refresh the twisties that are already opened and
     * have at least 1 Product Row showing
     **/
   public void refreshProductRows () {
      int rowCount = productTree.getJTree().getRowCount() ;
      int rowIndex = 1 ;                              

      parentWin.setStatus ( STR_REFRESHING ) ;
      parentWin.setValueMax ( rowCount ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Walk each row, asking if it is expanded                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( rowIndex < rowCount ) {
         if ( productTree.getJTree().isExpanded ( rowIndex ) ) {

            try {
               TreePath               pathRoot  = productTree.getJTree().getPathForRow ( rowIndex ) ;
               TreePath               path      = productTree.getJTree().getPathForRow ( rowIndex+1 ) ;
               DefaultMutableTreeNode node      = (DefaultMutableTreeNode)pathRoot.getLastPathComponent() ;                                             


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Ok, if the Object on this row is a MutableTreeWrapper,  */
               /* then we can sort this view                                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( path.getLastPathComponent() instanceof MutableTreeWrapper ) {
                  Object object = ((MutableTreeWrapper)path.getLastPathComponent()).getDataObject() ;

                  if ( object instanceof ProductRow ) {
                     currentViewNode = (ViewTreeNode) pathRoot.getLastPathComponent() ;
                     refreshViewNode() ;
                  }
               }
            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
            }

         }

         rowIndex++ ;
         parentWin.setValue ( rowIndex ) ;
      }

      parentWin.setValue ( 0 ) ;
      parentWin.setStatus ( null ) ;
   }




} /* ProductManagementPanel */

