package com.ibm.nzna.projects.qit.product.manager ; 
/*-----------------------------------------------------------*/
/*   ProductNavPanel                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductNavPanel                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.qit.product.*;
import com.ibm.nzna.projects.qit.product.manager.views.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import java.lang.Integer ;
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.qit.product.gui.* ;


/**
 * This panel will hold all of the different filters that may
 * be applied to the ProductManagementPanel.
 *
 *
 * @version 1.0, 5/13/2001 
 * @author  Venu Sanka
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductNavPanel                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductNavPanel extends JPanel implements ActionListener, NavPanel, 
    ProductConst, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private JTabbedPane              tabbedPane             = null ;
   private BookmarkHotLinkTree      bookmarkTree           = null ;
   private PMViewPanel              pmViewPanel            = null ;
   private CustomViewPanel          customViewPanel        = null ;
   private ProductManagementPanel   productManagementPanel = null ;
   private String                   query                  = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    Usually the NavPanel of a QuestPanel
    *  takes the QuestPanel in the constructor
    *  so that when the nav panel needs to call a 
    *  method or something in the QuestPanel it can
    *
    *  @param samplePanel The SamplePanel that is using us as it's NavPanel
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductNavPanel ( ProductManagementPanel productManagementPanel ) {
      this.productManagementPanel = productManagementPanel ;
   }

   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    Just like a QuestPanel and QuestApplet, the
    *  initialize method is called before anything else and
    *  before this NavPanel is shown. Initialize the GUI here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void initialize () {     

       JScrollPane  bookmarkScrollPane = null ;
       Color tabColor                  = ThemeSystem.getColor( "NAVPANEL_BG_COLOR" ) ; 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Use a border layout                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      tabbedPane            = new JTabbedPane() ;
      bookmarkTree          = new BookmarkHotLinkTree ( true ) ;
      pmViewPanel           = new PMViewPanel(productManagementPanel) ;
      customViewPanel       = new CustomViewPanel(productManagementPanel) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set Properties                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      tabbedPane.setBorder( GUISystem.emptyBorder ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our action listeners                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      bookmarkTree.addActionListener( this ) ;
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our controls                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout() ) ;
      tabbedPane.addTab( Str.getStr ( STR_VIEWS ), pmViewPanel ) ;
      tabbedPane.addTab( Str.getStr ( STR_BOOKMARKS ), bookmarkScrollPane = new JScrollPane ( bookmarkTree ) ) ;
      tabbedPane.addTab( Str.getStr ( STR_CUSTOM ), customViewPanel ) ;
      bookmarkScrollPane.setBorder ( GUISystem.emptyBorder ) ;

      add (tabbedPane, BorderLayout.CENTER ) ;

   }





   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    Just like the QuestPanel, the start
    *  method is called once we are showing. We
    *  could use this time to refresh data etc..
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start () {
   }


   /*------------------------------------------------------------*/
   /* getTItle
    */
   /**
    *    This method will return the name of this
    *  NavPanel. The name is displayed above
    *  the nav panel
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getTitle () {
      return( Str.getStr ( STR_PRODUCT_VIEWS ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    Since a NavPanel can be implemented by
    *  anything, the main window requires the
    *  JComponent that actually represents the NavPanel's GUI
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ; // We are the GUI, so return ourselves!
   }





   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called when we have
    *  been closed. Cleanup work is done here
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void stop () {
       removeAll() ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event that we are listening for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {
       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Bookmarks                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() instanceof Bookmark ) {
         Bookmark folder = (Bookmark)e.getSource() ;

         if ( folder.isFolder() ) {
            Vector bookmarks = BookmarkSystem.getBookmarks ( folder.getDescript() ) ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   If we have bookmarks in this folder                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( bookmarks != null ) && ( bookmarks.size() > 0 ) ) {
                 
               boolean containsDraft  = false ;
               boolean containsActive = false ;
               int     i              = 0 ;
               int     size           = bookmarks.size() ;                                       
               Vector  listeners      = new Vector ( 1 ) ;                                        

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Make sure the product management panel is notified of   */
               /*   views                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               listeners.addElement ( productManagementPanel ) ;

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   First, find out if we have Active, Draft or both        */
               /* Products in this folder                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               while ( i < size ) {
                  if ( ((Bookmark)bookmarks.elementAt(i)).getBookmarkType() == BOOKMARK_PRODUCT )
                     containsActive = true ;
                  else if ( ((Bookmark)bookmarks.elementAt(i)).getBookmarkType() == BOOKMARK_PRODUCT_DRAFT )
                     containsDraft  = true ;
                  i++ ;
               }  

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If we contain both, ask the user which one to show      */                                         
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( ( containsDraft ) && ( containsActive ) ) {
                  if ( GUISystem.printBox ( STR_QUESTION, STR_BOOKMARK_FOLDER_PRODUCT_DRAFT_ACTIVE ) )
                     containsDraft = false ;
                  else
                     containsActive = false ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   If we contain none, tell the user                       */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( ( !containsDraft ) && ( !containsActive ) )
                  GUISystem.printBox ( STR_INFO, STR_BOOKMARK_FOLDER_CONTAINS_NO_PRODUCTS ) ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, if we have active products, proceed               */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( containsActive ) {
                  Vector         activeVec = new Vector ( 10 ) ;
                  int activeIndex = 0 ;
                  
                  i=0 ;
                  while ( i < size ) {
                     if ( ((Bookmark)bookmarks.elementAt(i)).getBookmarkType() == BOOKMARK_PRODUCT )
                        activeVec.addElement ( bookmarks.elementAt(i) ) ;
                        i++ ;
                  }

                  int activeSize = activeVec.size() ;

                  query = "SELECT DISTINCT(A.PRODUCTIND), 0, A.PRODNUM, "+
                          "       A.DESCRIPTION, A.CREATEDON, A.CREATEDBY, A.CREATEDBY, "+
                          "       A.CREATEDBY, A.LASTTOUCHEDBY, A.LASTTOUCHEDON, A.CHECKEDOUTBY "+
                          "FROM PRODUCT.PRODUCT A "+
                          "WHERE A.PUBLISH = 'Y' AND " +
                          "      A.PRODUCTIND IN ( " ;

                  while ( activeIndex < activeSize ) {
                      Integer intRec = (Integer)((Bookmark)activeVec.elementAt(activeIndex)).getObject() ;
                      query+=" "+intRec.intValue() ;
                      if (activeIndex < (activeSize - 1) )
                          query+="," ;
                      activeIndex++ ;
                  }

                  query+=") " ;

                  LogSystem.log( 1, "Bookmark View Kicked off. SQL:"+query ) ;



                  getBookMarkList(query) ;
               }


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Now, if we have Draft documents, proceed                */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( containsDraft ) {
                  Vector         draftVec = new Vector ( 10 ) ;
                  i=0 ;
                  while ( i < size ) {
                     if ( ((Bookmark)bookmarks.elementAt(i)).getBookmarkType() == BOOKMARK_PRODUCT_DRAFT )
                        draftVec.addElement ( bookmarks.elementAt(i) ) ;
                     i++ ;
                  }
             
             }
              
            } else
               GUISystem.printBox ( STR_INFO, STR_BOOKMARK_FOLDER_CONTAINS_NO_PRODUCTS ) ;

         }
      }

   }

   /*------------------------------------------------------------*/
   /* getBookMarkList
    */
   /**
    *    This method will kick off the query and will
    *  return the results from the query
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
   private Vector getBookMarkList ( String query ) {
       this.query           = query ;
       String query2        = null ;
      SQLMethod sqlMethod   = new SQLMethod ( DatabaseSystem.QUEST, "getBookMarkList", 5 ) ;
      Vector    retVec      = new Vector ( 1 ) ;
      Vector    dateVec     = new Vector ( 1 ) ;

      try {
         
         Statement stmt     = sqlMethod.createStatement() ;
         ResultSet rs       = stmt.executeQuery ( query ) ;
         while ( rs.next() ) {
            ProductRow productRow = new ProductRow ( rs.getInt ( 1 ),                                               //Product Ind
                                                     0,                                                             //Existing Product Ind
                                                     rs.getString(3),                                               // PRoduct number or Part Number
                                                     rs.getString ( 4 ),                                            //Description
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 5 ).trim() ),  //Created on
                                                     UserSystem.getNameFromUserId ( rs.getString ( 6 ).trim()),     //Created     By
                                                     "",//DateSystem.prettyDateFromDBDate ( date ),                 //Actual Announce Date
                                                     "",                                                            //Target Announce Date
                                                     UserSystem.getNameFromUserId ( rs.getString ( 9 ).trim()),   //LAstTouched By
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 10 ).trim() ),//LastTouched on
                                                           
                                                     false ) ;

            if ( rs.getString(11) != null )
               productRow.setImage ( ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.SMALL_PRODUCT_LOCK ) ) ;
            retVec.addElement ( productRow ) ;

        }

         rs.close() ;


               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Form the SQL Query for getting the actual dates         */
               /*                                                           */
               /*-----------------------------------------------------------*/
               query2 = "SELECT PRODUCTIND, ACTUAL FROM PRODUCT.DATES WHERE TYPEDATEIND ="+TypeList.DATE_ANNOUNCED+ " AND "+
                        "PRODUCTIND IN ( " ;
               for ( int i = 0; i < retVec.size(); i++ ) {
                   query2+=" "+((ProductRow)retVec.elementAt(i)).getProductInd() ;

                   if ( i < (retVec.size() - 1 ) ) 
                       query2+=","  ;
                   
               }
               query2+=")" ;

               ResultSet rs2 = stmt.executeQuery(query2) ;

               while (rs2.next()) 
                   {
                   dateVec.addElement(new ProductDateCompare(rs2.getInt(1),
                                                               rs2.getString(2))) ;
                } /* while */
               rs2.close() ;
               System.out.println("size"+dateVec.size()) ;


               Vector productList = getFilteredRows( retVec, dateVec ); 

               if ( ( productList != null ) && ( productList.size() > 0 ) ) 
                productManagementPanel.setProductList(Str.getStr ( STR_RESULTS ),productList) ;




      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( retVec ) ;

   }



   /*------------------------------------------------------------*/
   /* getFilteredRows
    */
   /**
    *    This method is invoked for inserting actual dates in the 
    *    ProductRow object
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Vector
    *//*---------------------------------------------------------*/
    private Vector getFilteredRows( Vector vec1, Vector vec2 ) {
        Vector finalVector=new Vector();
        Vector replaceVector = new Vector( vec1 ) ;

        if ((vec1 != null) && (vec1.size() > 0)) {
            for ( int i = 0; i < vec1.size() ; i++ ) {
                ProductRow productRow=(ProductRow)vec1.elementAt(i);
                int prodInt=productRow.getProductInd();




                for(int j=0; j<vec2.size();j++){

                    ProductDateCompare productDateCompare=(ProductDateCompare)vec2.elementAt(j);
                    int productCompareInd=productDateCompare.getProductInd();
                    if(prodInt==productCompareInd){
                       int index = vec1.indexOf(vec1.elementAt(i)) ;
                       replaceVector.removeElementAt(index) ;
                         //String actualDate=productDateCompare.getActualDate();
                       String actualDate=DateSystem.prettyDateFromDBDate(productDateCompare.getActualDate());
                         productRow.setActualDate(actualDate);
                         replaceVector.insertElementAt(productRow, index) ;

                         
                    }
                    
                }
                            }
        }
        return (replaceVector);

    }



} /* ProductNavPanel */
