package com.ibm.nzna.projects.qit.product.manager.masschange ;     
/*-----------------------------------------------------------*/
/*   FieldChangerWizard                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: FieldChangerWizard                  */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.gui.popup.* ;
import com.ibm.nzna.projects.qit.doc.docedit.*;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.projects.qit.product.manager.masschange.*;
import com.ibm.nzna.projects.qit.product.*;
import com.ibm.nzna.projects.common.quest.brand.*;
import com.ibm.nzna.projects.common.quest.*;
import javax.swing.* ;
import java.util.* ;
import java.awt.* ;
import java.io.* ;
/**
 * The FieldChangerWizard will walk the user through the steps 
 * of changing a set of products.
 *
 * Step 1  - Just a welcome message to the user
 * Step 2  - Asks the user for what Field to change
 * Step 2a - Only shows if field is a dynamic field and asks if they wish to overwrite data or add data
 * Step 3  - Ask the user for comments
 * Step 4  - Asks the user for the field data
 * Step 5  - Finished Msg.
 *
 *
 * @version 1.0, 1/21/100 
 * @author  Jag.
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   FieldChangerWizard                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class FieldChangerWizard extends JDialog implements AppConst, WizardListener, PropertyListener,ProductConst,DataLengths {


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Private Members                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private Wizard          wizardPanel          = null ;
    private boolean         rc                   = false ;
    private InfoStep        infoStep             = null ;
    private InfoStep        finishStep           = null ;
    private SelectFieldStep selectFieldStep      = new SelectFieldStep () ;
    private CommentStep     commentStep          = new CommentStep () ;
    private PopUpPanel      dataStep             = null ;
    private PopUpPanel      dataStep1            = null ;
    private Vector          prodList             = null ;
    private TypeBrandRec    currentBrandRec      = null ;
    private TypeCategoryRec currentCategoryRec   = null ;

    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will create and initialize
     *  all of our controls. It will not show the dialog.
     *  Please use the 'getResult' method to show and process the dialog
     *
     *  @param parent The parent window of this wizard
     *  @param prodList The list of product Drafts to process
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public FieldChangerWizard ( Frame parent, Vector prodList ) {
        super ( parent, "", true ) ;          
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Assign local vars                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        this.prodList = prodList ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create our Wizard control                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        wizardPanel = new Wizard ( ImageSystem.getImageIcon ( this, "skins"+
                                                              File.separator+
                                                              PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                                                              File.separator+
                                                              "setup_wizard.gif" ) ) ;                     



        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Create our steps                                        */
        /*                                                           */
        /*-----------------------------------------------------------*/
        infoStep   = new InfoStep       ( "" ) ;                      
        finishStep = new InfoStep       ( "" ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Adjust properties/Add Listeners                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        setText() ;
        wizardPanel.addWizardListener ( this ) ;
        PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add the controls to our window                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        getContentPane().setLayout ( new BorderLayout () ) ;
        getContentPane().add ( wizardPanel ) ;

        GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
        //setSize ( 495, 375 ) ;
        setSize ( 510, 375 ) ;
        WinUtil.centerChildInParent ( this, MainWindow.getInstance() ) ;
    }





    /*------------------------------------------------------------*/
    /* getResult
     */
    /**
     *    This method will return the results
     *  from the user processing this wizard. This
     *  method will show the wizard and will not return unless Finish or Cancel were pressed
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return TRUE = Finish was pressed, FALSE = Cancel was pressed
     *//*---------------------------------------------------------*/
    public boolean getResult () {
        wizardPanel.setCurrentStep ( infoStep ) ;
        setVisible ( true ) ;
        return( rc ) ;
    }






    /*------------------------------------------------------------*/
    /* cancelPerformed
     */
    /**
     *    This method will be called if the user
     *  hit the 'Cancel' button on the Wizard
     *  
     *
     *  @param e The wizard event
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void cancelPerformed ( WizardEvent e ) {
        rc = false ;
       
        setVisible(false);
    }




    /*------------------------------------------------------------*/
    /* finishPerformed
     */
    /**
     *    This method will be called when the
     *  user presses the finish button
     *  
     *
     *  @param e The event
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void finishPerformed(WizardEvent e){
        boolean isSuccess=false;
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Product Basemodel                                       */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_BASE_MODEL ) ) ) {
            isSuccess=  new  ProductBaseModelFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Customer                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_CUSTOMER ) ) ) {
            isSuccess=  new  ProductCustomerFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Offering                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_OFFERING ) ) ) {
            isSuccess=  new  ProductOfferingFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Warranty                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_WARRANTY ) ) ) {
            isSuccess=  new  ProductWarrantyFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Product Type                                            */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_PRODUCT_TYPE ) ) ) {
            isSuccess=  new  ProductTypeFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Input                                                   */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_INPUT ) ) ) {
            isSuccess=  new  ProductInputFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Source                                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_SOURCE ) ) ) {
            isSuccess=  new  ProductSourceFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Description                                             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_DESC ) ) ) {
            isSuccess=  new  ProductDescFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Brand/Family                                            */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_BRAND_FAMILY ) ) ) {
            new  ProductBrandFamilyFieldChanger ().beginProcess ( prodList, dataStep.saveData(),dataStep1.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
            isSuccess=true;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Machine                                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_MACHINE ) ) ) {
            new  ProductMachineFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
            isSuccess=true;
        }
        
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   SubCategory                                             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_SUB_CATEGORY ) ) ) {
            new  ProductSubCategoryFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
            isSuccess=true;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   DateFields                                              */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_DATE_FIELDS ) ) ) {
            new  ProductDateFieldsChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
            isSuccess=true;
        }
        
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Product Locale                                          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_LOCATION ) ) ) {
            new  ProductLocaleFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
            isSuccess=true;
        }
        
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Project                                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( selectFieldStep.getSelectedField().equals ( Str.getStr ( STR_PROJECT ) ) ) {
            new  ProductProjectFieldChanger ().beginProcess ( prodList, dataStep.saveData(), selectFieldStep.getActionType(), commentStep.getComments()  ) ;
            isSuccess=true;
        }
        
        if (isSuccess) {
            dispose();
        }
    }



    /*------------------------------------------------------------*/
    /* nextPerformed
     */
    /**
     *    This method will be called when the
     *  next button is performed
     *  
     *
     *  @param e The event
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void nextPerformed ( WizardEvent e ) {
        WizardStep step = wizardPanel.getCurrentStep() ;
        PopUpPanel familyDatastep=null;
        if ( step == infoStep ) {
            wizardPanel.setCurrentStep ( selectFieldStep ) ;
        } else if ( step == selectFieldStep ) {
            if ( selectFieldStep.getSelectedField() != null )
                wizardPanel.setCurrentStep ( commentStep ) ;
        } else if ( step == commentStep ) {
            if (selectFieldStep.getSelectedField().equals(Str.getStr(STR_BRAND_FAMILY))) {
                dataStep = getFieldStep ( Str.getStr(STR_BRAND_FAMILY))  ;
                wizardPanel.setCurrentStep ( dataStep ) ;
            } else {
                dataStep = getFieldStep ( selectFieldStep.getSelectedField() ) ;
                wizardPanel.setCurrentStep ( dataStep ) ;
            }
        } else if ((step==dataStep)&&(selectFieldStep.getSelectedField().equals(Str.getStr(STR_BRAND_FAMILY)))) {
            currentCategoryRec=(TypeCategoryRec)dataStep.saveData();
            dataStep1 = getFieldStep ( Str.getStr(STR_FAMILY));
            wizardPanel.setCurrentStep ( dataStep1 ) ;

        } else if (step==dataStep1) {
            wizardPanel.setCurrentStep(finishStep);
        } else
            wizardPanel.setCurrentStep ( finishStep ) ;

        GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
        adjustButtons () ;
    }


    /*------------------------------------------------------------*/
    /* backPerformed
     */
    /**
     *    This method will be called when the
     *  back button is performed
     *  
     *
     *  @param e The event
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void backPerformed ( WizardEvent e ) {
        WizardStep step = wizardPanel.getCurrentStep() ;

        if ( step == selectFieldStep )
            wizardPanel.setCurrentStep ( infoStep ) ;

        else if ( step == commentStep )
            wizardPanel.setCurrentStep ( selectFieldStep ) ;

        else if (step==dataStep)
            wizardPanel.setCurrentStep ( commentStep );

        else if ( step == dataStep1 ) {
            if ((selectFieldStep.getSelectedField().equals(Str.getStr(STR_BRAND_FAMILY)))) {
                wizardPanel.setCurrentStep(dataStep);
            }
        }

        else if (step==finishStep) {
            if ((selectFieldStep.getSelectedField().equals(Str.getStr(STR_BRAND_FAMILY)))) {

                wizardPanel.setCurrentStep(dataStep1);
            } else
                wizardPanel.setCurrentStep(dataStep);
        } else
            wizardPanel.setCurrentStep(commentStep);


        GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
        adjustButtons ( ) ;
    }






    /*------------------------------------------------------------*/
    /* getFieldStep
     */
    /**
     *    This method will create and return a WizardStep
     *  based on the name of the field passed in
     *  
     *
     *  @param field The name of the field ( Document Type, Document Class etc..)
     *  @param None
     *  @param None
     *
     *  @return WizardStep to edit that field
     *//*---------------------------------------------------------*/
    private PopUpPanel getFieldStep ( String fieldName ) {
        PopUpPanel returnStep = null ;    

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Description                                             */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( fieldName.equals ( Str.getStr ( STR_DESC ) ) ) {
            PopUpDescReplace bodyStep   = new PopUpDescReplace ( null ) ;
            DescReplaceRec   replaceRec = new DescReplaceRec () ;

            bodyStep.refreshData ( replaceRec ) ;
            bodyStep.setTitle ( Str.getStr ( STR_SEARCH_REPLACE_DESCRIPTION_TITLE ) ) ;

            returnStep = bodyStep ;
        }







        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Machine/Subcategory                                     */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if (( fieldName.equals ( Str.getStr ( STR_MACHINE ) ) )||( fieldName.equals ( Str.getStr ( STR_SUB_CATEGORY ) ) )) {
            PopUpEntryField entryStep = new PopUpEntryField ( null ) ;
            if (fieldName.equals ( Str.getStr ( STR_MACHINE ) )) {
                entryStep.setTitle ( Str.getStr ( STR_PLEASE_ENTER_MACHINE ) ) ;
                entryStep.setMaximumLength ( LENGTH_MACHINE ) ;
                entryStep.setDescript ( Str.getStr ( STR_MACHINE ) ) ;

                returnStep = entryStep ;
            }
            else {
                entryStep.setTitle ( Str.getStr ( STR_PLEASE_ENTER_SUBCATEGORY ) ) ;
                entryStep.setMaximumLength ( LENGTH_SUBCATEGORY ) ;
                entryStep.setDescript ( Str.getStr ( STR_SUB_CATEGORY ) ) ;

                returnStep = entryStep ;
            }
        }


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   BaseModel                                               */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_BASE_MODEL ) ) ) {
            PopUpEntryField entryStep = new PopUpEntryField ( null ) ;

            entryStep.setTitle ( Str.getStr ( STR_PLEASE_ENTER_BASE_MODEL ) ) ;
            entryStep.setMaximumLength ( LENGTH_BASE_MODEL ) ;
            entryStep.setDescript ( Str.getStr ( STR_BASE_MODEL ) ) ;
            returnStep = entryStep ;

        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Project                                                 */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_PROJECT ) ) ) {
            PopUpEntryField entryStep = new PopUpEntryField ( null ) ;

            entryStep.setTitle ( Str.getStr ( STR_ENTER_PROJECT ) ) ;
            entryStep.setMaximumLength ( LENGTH_PROJECT ) ;
            entryStep.setDescript ( Str.getStr ( STR_PROJECT ) ) ;
            returnStep = entryStep ;

        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Brand/Family                                            */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_BRAND_FAMILY ) ) ) {
            PopUpList listStep   = new PopUpList ( null ) ;
            Vector    brands = TypeCategory.getCategories("brand") ;
            listStep.setTitle ( Str.getStr ( STR_PLEASE_SELECT_BRAND ) ) ;
            listStep.setDescript ( Str.getStr ( STR_BRAND ) ) ;
            listStep.refreshData ( brands ) ;
            listStep.setMultiSelect ( false ) ;

            returnStep = listStep ;
        }


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Family                                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_FAMILY ) ) ) {
            PopUpList listStep   = new PopUpList ( null ) ;
            Vector    families = TypeCategory.getChildCategories(currentCategoryRec.getInd()) ;
            listStep.setTitle ( Str.getStr ( STR_PLEASE_SELECT_FAMILY ) ) ;
            listStep.setDescript ( Str.getStr ( STR_FAMILY ) ) ;
            listStep.refreshData ( families ) ;
            listStep.setMultiSelect ( false ) ;

            returnStep = listStep ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Customer                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_CUSTOMER ) ) ) {
            PopUpList listStep   = new PopUpList ( null ) ;
            Vector    customers = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_CUSTOMER ).clone() ;
            listStep.setTitle ( Str.getStr ( STR_PLEASE_SELECT_CUSTOMER ) ) ;
            listStep.setDescript ( Str.getStr ( STR_CUSTOMER ) ) ;
            listStep.refreshData ( customers ) ;
            listStep.setMultiSelect ( false ) ;

            returnStep = listStep ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Offering                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_OFFERING ) ) ) {
            PopUpList listStep   = new PopUpList ( null ) ;
            Vector    offerings = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_OFFERING ).clone() ;
            listStep.setTitle ( Str.getStr ( STR_PLEASE_SELECT_OFFERING ) ) ;
            listStep.setDescript ( Str.getStr ( STR_OFFERING ) ) ;
            listStep.refreshData ( offerings ) ;
            listStep.setMultiSelect ( false ) ;

            returnStep = listStep ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Warranty                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_WARRANTY ) ) ) {
            PopUpList listStep   = new PopUpList ( null ) ;
            Vector    warranty = (Vector)TypeList.getInstance().getTypeList ( TypeList.TYPE_WARRANTY ).clone() ;
            listStep.setTitle ( Str.getStr ( STR_PLEASE_SELECT_WARRANTY ) ) ;
            listStep.setDescript ( Str.getStr ( STR_WARRANTY ) ) ;
            listStep.refreshData ( warranty ) ;
            listStep.setMultiSelect ( false ) ;

            returnStep = listStep ;
        }

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   DateFields                                              */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_DATE_FIELDS ) ) ) {
            PopUpDates listStep   = new PopUpDates ( null ) ;
            listStep.refreshData(null);
            returnStep = listStep ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Location                                                */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_LOCATION ) ) ) {
            PopUpLocation listStep   = new PopUpLocation ( null ) ;
            com.ibm.nzna.projects.common.quest.LocaleRec localeRec=new com.ibm.nzna.projects.common.quest.LocaleRec();
            listStep.refreshData(localeRec);
            returnStep = listStep ;
        }
        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Source                                                  */
        /*                                                           */
        /*-----------------------------------------------------------*/
        else if ( fieldName.equals ( Str.getStr ( STR_SOURCE ) ) ) {
            PopUpList listStep   = new PopUpList ( null ) ;
            Vector    source = new Vector();
            source.addElement(Str.getStr(STR_OPICM));
            source.addElement(Str.getStr(STR_AUTHOR));
            listStep.setTitle ( Str.getStr ( STR_PLEASE_SELECT_SOURCE ) ) ;
            listStep.setDescript ( Str.getStr ( STR_SOURCE ) ) ;
            listStep.refreshData ( source ) ;
            listStep.setMultiSelect ( false ) ;

            returnStep = listStep ;
        }
        if ( returnStep != null )
            returnStep.setRequired ( true ) ;


        return( returnStep ) ;
    }


    /*------------------------------------------------------------*/
    /* adjustButtons
     */
    /**
     *    This method will look at what step is
     *  currently showing and will decide which
     *  buttons to show in the wizard
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void adjustButtons () {
        WizardStep step = wizardPanel.getCurrentStep() ;

        if ( step == infoStep )
            wizardPanel.showButtons ( true, false, true, false ) ;
        else if ( step == finishStep )
            wizardPanel.showButtons ( false, true, false, true ) ;
        else
            wizardPanel.showButtons ( true, true, true, false ) ;
    }




    /*------------------------------------------------------------*/
    /* propertyChanged
     */
    /**
     *    Called when a property we are listening
     *  for has been fired
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void propertyChanged ( int property, Object value ) {
        if ( property == PropertySystem.PROPERTY_LANGUAGE ) {
            setText() ;
        }
    }





    /*------------------------------------------------------------*/
    /* setText
     */
    /**
     *    This method will set the text on all
     *  buttons and InfoStep panels of the 
     *  wizard
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void setText () {
        wizardPanel.setNextText   ( Str.getStr ( STR_NEXT ) ) ;
        wizardPanel.setBackText   ( Str.getStr ( STR_BACK ) ) ;
        wizardPanel.setFinishText ( Str.getStr ( STR_FINISH ) ) ;
        wizardPanel.setCancelText ( Str.getStr ( STR_CANCEL ) ) ;
        setTitle ( Str.getStr ( STR_FIELD_CHANGER_WIZARD ) ) ;
        infoStep.setMessage ( Str.getStr ( STR_PRODUCT_FIELD_CHANGER_WIZARD_INFO ) ) ;                      
        finishStep.setMessage ( Str.getStr ( STR_FIELD_CHANGER_WIZARD_FINISH ) ) ;
    }

} /* FieldChangerWizard */
