package com.ibm.nzna.projects.qit.product.manager.masschange ;
/*-----------------------------------------------------------*/
/*   SelectFieldStep                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SelectFieldStep                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.qit.product.*;
import javax.swing.event.* ;
import javax.swing.* ;
import java.util.* ;   
import java.awt.* ;
import java.awt.event.* ;
/**
 * This step will allow the user to specify which field to change
 *
 *
 * @version 1.0, 05/20/2001 
 * @author  Jag.
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SelectFieldStep                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class SelectFieldStep extends WizardStep implements ListSelectionListener, AppConst,ProductConst  {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * JLabel for Help. 
    */
   private JLabel       st_HELP          = null ;

   /**
    * fieldList contains list of editable fields. 
    */
   private JList        fieldList        = null ;

   /**
    * fieldList ScrollPane. 
    */
   private JScrollPane  fieldScroll      = null ;

   /**
    * The FieldChanger should only append information, not  Replace and Remove  information
    */
   private JRadioButton append       = null ;

   /**
    *  The FieldChanger should replace the information
    */
   private JRadioButton replace      = null ;

   /**
    * The FieldChanger should remove the information 
    */
   private JRadioButton remove       = null ;

   /**
    * this var contains which editable field we are selected. 
    */
   private String       selectedField    = null ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create our step
    *  
    *
    *  @param message The Message to display to the user
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public SelectFieldStep (  ) {
       ButtonGroup buttonGroup = new ButtonGroup () ;
       Vector      fields      = new Vector ( 10 ) ;
       fields.addElement ( Str.getStr ( STR_BASE_MODEL ) ) ;
       fields.addElement ( Str.getStr ( STR_BRAND_FAMILY ) ) ;
       fields.addElement ( Str.getStr ( STR_CUSTOMER ) ) ;
       fields.addElement ( Str.getStr ( STR_DATE_FIELDS ) ) ;
       fields.addElement ( Str.getStr ( STR_DESC ) ) ;
       fields.addElement ( Str.getStr ( STR_MACHINE ) ) ;
       fields.addElement ( Str.getStr ( STR_LOCALE ) ) ;
       fields.addElement ( Str.getStr ( STR_OFFERING ) ) ;
       fields.addElement ( Str.getStr ( STR_SUB_CATEGORY ) ) ;
       fields.addElement ( Str.getStr ( STR_SOURCE ) ) ;
       fields.addElement ( Str.getStr ( STR_WARRANTY ) ) ;
       fields.addElement ( Str.getStr ( STR_PROJECT ) );

       new QuickSort (fields) ;

       /*-----------------------------------------------------------*/
       /*                                                           */
       /*   Create our controls                                     */
       /*                                                           */
       /*-----------------------------------------------------------*/
       st_HELP          = new JLabel       ( ) ;
       fieldList        = new JList        ( fields ) ;
       fieldScroll      = new JScrollPane  ( fieldList ) ;
       append           = new JRadioButton ( Str.getStr ( STR_APPEND ) ) ;
       replace          = new JRadioButton ( Str.getStr ( STR_REPLACE ) ) ;
       remove           = new JRadioButton ( Str.getStr( STR_REMOVE ) );
       /*-----------------------------------------------------------*/
       /*                                                           */
       /*   Adjust properties                                       */
       /*                                                           */
       /*-----------------------------------------------------------*/
       st_HELP.setVerticalAlignment ( SwingConstants.TOP ) ;
       buttonGroup.add ( append ) ;
       buttonGroup.add ( replace ) ;
       buttonGroup.add ( remove ) ;
       setText() ;                           

       /*-----------------------------------------------------------*/
       /*                                                           */
       /*   Add listeners                                           */    
       /*                                                           */
       /*-----------------------------------------------------------*/
       fieldList.addListSelectionListener ( this ) ;

       /*-----------------------------------------------------------*/
       /*                                                           */
       /*   Add our controls to the panel                           */
       /*                                                           */
       /*-----------------------------------------------------------*/
       setLayout ( null ) ;
       add ( st_HELP ) ;
       add ( fieldScroll ) ;                                       
       add ( append ) ;
       add ( replace ) ;
       add ( remove ) ;
       enableRadioControls ( false ) ;

   }





   /*------------------------------------------------------------*/
   /* enableRadioControls
    */
   /**
    *    This method will either enable or disable
    *  the radio button controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void enableRadioControls ( boolean enable ) {
      append.setEnabled ( enable ) ;
      replace.setEnabled ( enable ) ;
      remove.setEnabled( enable );
      if ( !enable )
         replace.setSelected ( true ) ;
   }


   /*------------------------------------------------------------*/
   /* doLayout
    */
   /**
    *    This method will position and size all
    *  of our controls
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight() ;
      int       x         = 5 ;
      int       y         = 5 ;
      Dimension prefSize  = null ;

      super.doLayout() ;

      st_HELP.setBounds ( x, y, size.width-10, 100 ) ;
      prefSize = st_HELP.getPreferredSize() ;
      st_HELP.setBounds ( x, y, size.width-10, prefSize.height ) ;
      y+=prefSize.height+5 ;

      fieldScroll.setBounds ( x, y, size.width-10, size.height-y-15-(rowHeight*3) ) ;

      append.setBounds ( x, size.height-10-(rowHeight*3), size.width-10, rowHeight ) ;
      replace.setBounds ( x, size.height-10-(rowHeight*2), size.width-10, rowHeight ) ;
      remove.setBounds( x,size.height-10-rowHeight , size.width-10, rowHeight ) ;
   }





   /*------------------------------------------------------------*/
   /* saveInfo
    */
   /**
    *    This method will save the information
    *  and return TRUE if a language was
    *  selected.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean saveInfo () {
      boolean rc = false ;


      selectedField = (String)fieldList.getSelectedValue() ;

      rc = selectedField != null ;

      return( rc ) ;
   }

   public void close () {}


   public String getTitle () {
      return( Str.getStr ( STR_SELECT_FIELD_FOR_DOCUMENT_CHANGE_TITLE ) ) ;
   }


   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text of the panel
    *  
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      st_HELP.setText ( Str.getStr ( STR_SELECT_FIELD_FOR_PRODUCT_CHANGE ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getSelectedField
    */
   /**
    *    This method will return the selected field
    *  that the user chose. NULL if nothing was selected
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   protected String getSelectedField () {
      return( selectedField ) ;
   }





   /*------------------------------------------------------------*/
   /* getActionType
    */
   /**
    *    This method will return String  if
    *  the user wishes to Append ,replace,Remove Only.
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public  String getActionType () {
       if(append.isSelected()){
           return( Str.getStr ( STR_APPEND ) ) ;
       }
       else if(replace.isSelected()){
           return( Str.getStr ( STR_REPLACE ) ) ;
       }
       else if(remove.isSelected()){
           return( Str.getStr ( STR_REMOVE ) ) ;
       }
       else
           return null;

   }


   /*------------------------------------------------------------*/
   /* valueChanged
    */
   /**
    *    This method will be called when a value is
    *  changed in the list of document fields
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void valueChanged ( ListSelectionEvent e ) {
      String selectedField = (String)fieldList.getSelectedValue() ;
      if ( ( selectedField.equals ( Str.getStr ( STR_DATE_FIELDS ) ) ) ||
           ( selectedField.equals ( Str.getStr ( STR_LOCALE ) ) ) ) 
         enableRadioControls(true);
      else 
         enableRadioControls ( false ) ;
   }
} /* SelectFieldStep */
