package com.ibm.nzna.projects.qit.product.manager.productview ;     
/*-----------------------------------------------------------*/
/*   ProductViewRec                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductViewRec                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import java.util.* ;
import com.ibm.nzna.projects.qit.product.manager.*;
/**
 * This record holds information about a product search 
 *
 *
 * @version 1.0,  
 * @author  Venu Sanka
 */


/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductViewRec                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductViewRec extends Object implements AppConst {
    public static final int VIEW_PRODUCT        = 1 ;
    public static final int VIEW_PRODUCTEMPLATE = 2 ;

    /*----------------------------------------------------------------------*/
    /*		Static Search Types												*/
    /*----------------------------------------------------------------------*/
    public static final int BRAND_VIEW_DRAFT_BY_ANN_DATE    = 0 ;
    public static final int BRAND_VIEW_DRAFT_BY_PROJECT     = 1 ;
    public static final int BRAND_VIEW_PRODUCT_BY_FAMILY    = 2 ;
    public static final int BRAND_VIEW_PRODUCT_BY_PROJECT   = 3 ;
    public static final int BRAND_VIEW_PRODUCT_BY_ANN_DATE  = 4 ;
    public static final int MISFIT_VIEW_BY_ANN_DATE         = 5 ;
    public static final int MISFIT_VIEW_BY_PROJECT          = 6 ;
    public static final int DATA_MAPPING_VIEW_BY_BFS        = 7 ;
    public static final int DATA_MAPPING_VIEW_BY_GE_CO      = 8 ;

    public static final int VIEW_EXACT_SQL                  = 9 ;

    public static final int VIEW_CUSTOM_VIEW                = 10 ;


    /*----------------------------------------------------------------------*/
    /*		Directions     												    */
    /*----------------------------------------------------------------------*/
    public static final int PB_PREV                         = 1 ;
    public static final int PB_NEXT                         = 2 ;

    /*----------------------------------------------------------------------*/
    /*		Vars												            */
    /*----------------------------------------------------------------------*/
    public  int     directionInd                              = 0 ;
    private int     typeCatInd                                = 0 ;
    private int     parentCatInd                              = 0 ;
    private int     viewType                                  = 0 ;

    public  int     fromCount                                 = 0 ;
    public  int     actualCount                               = 0 ;
    public  int     newFromCount                              = 0 ;

    public  String  description                               = "" ;

    private Vector  bookmarkVec                               = null ;
    private int     statusInd                                 = 0 ;
    private Vector  listeners                                 = null ;
    private String  partNumber                                = null ;
    private Vector  results                                   = null ;
    private String  queryText                                 = null ;
    private String  comment                                   = null ;
    private String  source                                    = null ;
    private UserRec userRec                                   = null ;
    private String  machine                                   = null ;
    private String  title                                     = null ;
    private String  sqlQuery                                  = null ;
    
     
    private ProductCustomView     customView                  = null ;


    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Column Headings                                         */
    /*                                                           */
    /*-----------------------------------------------------------*/
    /**
      * This is the list of column headings needed to display the info
      */
    String[] listTitle = { 
        " ",
        Str.getStr ( STR_PRODUCT_NUMBER ),
        Str.getStr ( STR_DESCRIPTION ),
        Str.getStr ( STR_STATUS ),
        Str.getStr ( STR_STATUS_DATE )} ;


        
    /*------------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This method will initialize our variables to
     *  the ones passed in
     *  
     *
     *  @param viewType What type of view this is. This must be one of the VIEW_*** values
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public ProductViewRec ( int viewType ) {
        setSearchType ( viewType ) ;
    }

    /*------------------------------------------------------------*/
    /* setSearchType
     */
    /**
     *    This method will set what type of search
     *  we are performing here. This type must be defined
     *  in this class as one of the VIEW_XXX
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setSearchType ( int viewType ) {
        this.viewType = viewType ;
    }



    /*------------------------------------------------------------*/
    /* clone
     */
    /**
     *    This method creates a copy of this object
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Object clone () {
        ProductViewRec viewRec = new ProductViewRec ( viewType ) ;

        viewRec.directionInd  =  directionInd ;
        viewRec.typeCatInd    =  typeCatInd ;
        viewRec.parentCatInd  =  parentCatInd ;
        viewRec.viewType      =  viewType ;
        viewRec.fromCount     =  fromCount ;
        viewRec.newFromCount  =  newFromCount ;
        viewRec.actualCount   =  actualCount ;
        viewRec.description   =  description ;
        viewRec.statusInd     =  statusInd ;
        viewRec.listeners     =  listeners ;
        viewRec.comment       =  comment ;
        viewRec.source        =  source ;
        viewRec.userRec       =  userRec ;
        viewRec.machine       =  machine ;
        viewRec.title         =  title ;

        return( viewRec ) ;
    }


    /*------------------------------------------------------------*/
    /* getTypeCatInd
     */
    /**
     *    This method will return the type category indicator
     *  that is used. 
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return int The type category indicator as defined in PRODUCT.TYPECATEGORY
     *//*---------------------------------------------------------*/
    public int getTypeCatInd () {
        return( typeCatInd ) ;
    }


    /*------------------------------------------------------------*/
    /* setTypeCatInd
     */
    /**
     *    This method will set our type category indicator
     *  
     *  
     *
     *  @param typeCatInd The Type Category Indicator that matches what is in PRODUCT.TYPECATEGORY
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setTypeCatInd ( int typeCatInd ) {
        this.typeCatInd = typeCatInd ;
    }


    /*------------------------------------------------------------*/
    /* getParentCatInd
     */
    /**
     *    This method will return the parent category indicator
     *  that is used. 
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return int The parent category indicator as defined in PRODUCT.TYPECATCHILD
     *//*---------------------------------------------------------*/
    public int getParentCatInd () {
        return( parentCatInd ) ;
    }


    /*------------------------------------------------------------*/
    /* setParentCatInd
     */
    /**
     *    This method will set our parent category indicator
     *  
     *  
     *
     *  @param typeCatInd The Type Category Indicator that matches what is in PRODUCT.TYPECATCHILD
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setParentCatInd ( int parentCatInd ) {
        this.parentCatInd = parentCatInd ;
    }


    /*------------------------------------------------------------*/
    /* getBookmarkVec
     */
    /**
     *    This method will return the bookmark
     *  vector we are using
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Vector getBookmarkVec () {
        return( bookmarkVec ) ;
    }


    /*------------------------------------------------------------*/
    /* setBookmarkVec
     */
    /**
     *    This method will set the bookmark vec
     *  that this view is refreshing with
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setBookmarkVec ( Vector vec ) {
        bookmarkVec = vec ;
    }



    /*------------------------------------------------------------*/
    /* getStatusInd
     */
    /**
     *    This method will return the status indicator
     *  that this view will search with
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return int The status indicator as defined in TIGRIS.TYPESTATUS
     *//*---------------------------------------------------------*/
    public int getStatusInd () {
        return( statusInd ) ;
    }

   

    /*------------------------------------------------------------*/
    /* setStatusInd
     */
    /**
     *    This method will set the status indicator
     *  to use when searching.
     *  
     *
     *  @param statusInd The Status Indicator as defined in TIGRIS.TYPESTATUS
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setStatusInd ( int statusInd ) {
        this.statusInd = statusInd ;
    }


    /*------------------------------------------------------------*/
    /* getListeners
     */
    /**
     *    This method will return the list of listeners
     *  that need to be notified of this view starting
     *  and stopping
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public Vector getListeners () {
        return( listeners ) ;
    }



    /*------------------------------------------------------------*/
    /* setListeners
     */
    /**
     *    This method will set the all DocViewListeners
     *  that will be notified once the view is complete
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setListeners ( Vector listeners ) {
        this.listeners = listeners ;
    }



    /*------------------------------------------------------------*/
    /* getPartNumber
     */
    /**
     *    this method will return the part number
     *  for the view VIEW_BY_OPTION_PARTNUMBER
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getPartNumber () {
        return( getPartNumber ( false ) ) ;
    }

    /*------------------------------------------------------------*/
    /* getPartNumber
     */
    /**
     *    This method will ask the user for part numbers
     *  and then return them.
     *  
     *
     *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the machine property
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getPartNumber ( boolean showGUI ) {
        if ( showGUI )
            partNumber = new SingleEntryDlg ( MainWindow.getInstance(),
                                              Str.getStr ( STR_OPTION_PARTNUMBER ),
                                              Str.getStr ( STR_OPTION_PARTNUMBER ),
                                              machine,
                                              new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;

        return( partNumber ) ;
    }


    

    /*------------------------------------------------------------*/
    /* setPartNumber
     */
    /**
     *    This method will set the part number for
     *  the VIEW_BY_OPTION_PARTNUMBER
     *  view
     *
     *  @param partNumber The Part Number to look for
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setPartNumber ( String partNumber ) {
        this.partNumber = partNumber ;
    }


    /*------------------------------------------------------------*/
    /* getResults
     */
    /**
     *    This method will return the results of a view
     *  once the view has completed
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Vector of DocRecs
     *//*---------------------------------------------------------*/
    public Vector getResults () {
        return( results ) ;
    }



    /*------------------------------------------------------------*/
    /* setResults
     */
    /**
     *    This method will set the results that are returned
     *  to the listeners
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    protected void setResults ( Vector results ) {
        this.results = results ;
    }




    /*------------------------------------------------------------*/
    /* getQueryText
     */
    /**
     *    This method will return our query text
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String getQueryText () {
        return( queryText ) ;
    }



    /*------------------------------------------------------------*/
    /* getComment
     */
    /**
     *    This method will return the comments. 
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getComment () {
        return( getComment ( false ) ) ;
    }




    /*------------------------------------------------------------*/
    /* getComment
     */
    /**
     *    This method will ask the user for comments
     *  and then return them.
     *  
     *
     *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the comment property
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getComment ( boolean showGUI ) {
        if ( showGUI )
            comment = new SingleEntryDlg ( MainWindow.getInstance(),
                                           Str.getStr ( STR_COMMENT ),
                                           Str.getStr ( STR_COMMENT ),
                                           comment,
                                           new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;

        return( comment ) ;
    }




    /*------------------------------------------------------------*/
    /* getSource
     */
    /**
     *    This method will return the sources. 
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getSource () {
        return( getSource ( false ) ) ;
    }




    /*------------------------------------------------------------*/
    /* getSource
     */
    /**
     *    This method will ask the user for sources
     *  and then return them.
     *  
     *
     *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the source property
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getSource ( boolean showGUI ) {
        if ( showGUI )
            source = new SingleEntryDlg ( MainWindow.getInstance(),
                                          Str.getStr ( STR_SOURCE ),
                                          Str.getStr ( STR_SOURCE ),
                                          source,
                                          new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ).getResult() ;

        return( source ) ;
    }



    /*------------------------------------------------------------*/
    /* getUserId
     */
    /**
     *    This method will return the userIds. 
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getUserId () {
        return( getUserId ( false ) ) ;
    }




    /*------------------------------------------------------------*/
    /* getUserId
     */
    /**
     *    This method will ask the user for userIds
     *  and then return them.
     *  
     *
     *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the userId property
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getUserId ( boolean showGUI ) {
        if ( showGUI ) {
            Vector users = new SelectUserDlg ( MainWindow.getInstance(),
                                               userRec ).getResult() ;

            if ( ( users != null ) && ( users.size() > 0 ) )
                userRec = (UserRec)users.elementAt(0) ;
        }

        if ( userRec != null )
            return( userRec.getUserId() ) ;
        return( null ) ;
    }


    /*------------------------------------------------------------*/
    /* getMachine
     */
    /**
     *    This method will return the machines. 
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getMachine () {
        return( getMachine ( false ) ) ;
    }




    /*------------------------------------------------------------*/
    /* getMachine
     */
    /**
     *    This method will ask the user for machines
     *  and then return them.
     *  
     *
     *  @param showGUI TRUE = ask user, FALSE = just return whatever is in the machine property
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getMachine ( boolean showGUI ) {
        if ( showGUI )
            machine = new SingleEntryDlg ( MainWindow.getInstance(),
                                           Str.getStr ( STR_MACHINE ),
                                           Str.getStr ( STR_MACHINE ),
                                           machine,
                                           new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 4 ) ).getResult() ;

        return( machine ) ;
    }



    /*------------------------------------------------------------*/
    /* getTitle
     */
    /**
     *    This method will return the title to search on
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public String getTitle () {
        return( title ) ;
    }


    /*------------------------------------------------------------*/
    /* setTitle
     */
    /**
     *    This method will set the title to search on
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setTitle ( String title ) {
        this.title = title ;
    }
          

    /*------------------------------------------------------------*/
    /* getSQLQuery
     */
    /**
     *    This method will return the SQL Query to
     *  run if this view is of type VIEW_EXACT_SQL
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getSQLQuery () {
        return( sqlQuery ) ;
    }

          
    /*------------------------------------------------------------*/
    /* setSQLQuery
     */
    /**
     *    This method will set the SQL Query to run
     *  if this view is of type VIEW_EXACT_SQL
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setSQLQuery ( String sqlQuery ) {
        this.sqlQuery = sqlQuery ;
    }


    /*------------------------------------------------------------*/
    /* getCustomView
     */
    /**
     *    This method will return the CustomView
     *  object that describes the custom view
     *  to use in this view
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return DocCustomView
     *//*---------------------------------------------------------*/
    public ProductCustomView getCustomView () {
        return( customView ) ;
    }
      
    /*------------------------------------------------------------*/
    /* setCustomView
     */
    /**
     *    This method will set the custom query
     *  that this view will represent
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setCustomView ( ProductCustomView customView ) {
        this.customView = customView ;
    }
      
    /*------------------------------------------------------------*/
    /* getViewType
     */
    /**
     *    This method will return the type of view that this
     *  is. This number is represented by one of the VIEW_XXX
     *  constants defined in this class
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public int getViewType () {
        return( viewType ) ;
    }
      
    /*------------------------------------------------------------*/
    /* getFromCount
     */
    /**
     *    This method will return the 'From' count
     *  that this view is refreshing from. This number
     *  represents the first record index returned
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return int
     *//*---------------------------------------------------------*/
    public int getFromCount () {
        return( fromCount ) ;
    }

    /*------------------------------------------------------------*/
    /* setActualCount
     */
    /**
     *    This method will set the actual count
     *  property. This is the number of actual
     *  documents that fit in this view
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void setActualCount ( int actualCount ) {
        this.actualCount = actualCount ;
    }


    /*------------------------------------------------------------*/
    /* isCustomView
     */
    /**
     *    This method will return TRUE if this view
     *  is a user created view
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean isCustomView () {
        return( customView != null ) ;
    }

    
    /**
      * This method will return the list of column headings that should
      * be placed in a MultiList or JTable
      */
    public String[] getColumnHeadings () {
        return( listTitle ) ;
    }

    /**
      * This method will set a title in the column headings
      */
    public void setColumnHeading ( int titleIndex, String title ) {
        listTitle[titleIndex] = title ;
    }

         
    /*------------------------------------------------------------*/
    /* refresh
     */
    /**
     *    This method will attempt to start a read
     *  of the currently configured view. This will
     *  kick off a thread to do the work so this method
     *  immediatly returns
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void refresh () {
      //  new RefreshProductViewThread ( this ) ;
    }

    /*------------------------------------------------------------*/
    /* refreshNow
     */
    /**
     *    This method will attempt to refresh
     *  the current view. This method will not return
     *  until the view is read in
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void refreshNow () {
      //  new RefreshProductViewThread ( this, false ).run() ;
    }



    
      
    

    } /* ProductViewRec */
