package com.ibm.nzna.projects.qit.product.manager.productview ;
/*-----------------------------------------------------------*/
/*   RefreshProductViewThread                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: RefreshProductViewThread            */
/*                                                           */
/*                                                           */
/*         @Copyright                                        */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.projects.qit.* ;
import com.ibm.nzna.projects.qit.doc.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.bookmark.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.projects.common.storedProc.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;


/**
 * This thread will refresh a Product view in a Thread 
 *
 *
 * @version 1.0,  
 * @author  
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   RefreshProductViewThread                                */
/*                                                           */
/*-----------------------------------------------------------*/
public class RefreshProductViewThread extends Thread  implements QuestProcess, DocConst, AppConst {

    /*-----------------------------------------------------------*/
    /*                                                           */
    /*   Vars                                                    */
    /*                                                           */
    /*-----------------------------------------------------------*/
    private boolean           onlyRequest      = false ;
    private CallableStatement stmt             = null ;
    private Statement         sqlStmt          = null ;
    private boolean           continueProcess  = true ;

    private Vector            listeners        = null ;
    private DocViewRec        viewRec          = null ;



    /*-----------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This will create and run the thread that will
     *  read all of the documents with the restrictions
     *  set in the searchRec passed in
     *             
     *  @param viewRec The ProductViewRec to refresh
     *  @param 
     *  @param 
     *
     *  @return 
     */
    public RefreshProductViewThread ( ProductViewRec viewRec ) {
        this.viewRec = viewRec ;
        listeners = viewRec.getListeners() ;
        start () ;
    } /* RefreshProductViewThread */


    /*-----------------------------------------------------------*/
    /* Constructor
     */
    /**
     *    This will create and run the thread that will
     *  read all of the products with the restrictions
     *  set in the searchRec passed in
     *             
     *  @param viewRec The ProductViewRec to refresh
     *  @param autoStart TRUE = start the Thread, FALSE = Do Not Start
     *  @param 
     *
     *  @return 
     */
    public RefreshProductViewThread ( ProductViewRec viewRec, boolean autoStart ) {
        this.viewRec = viewRec ;
        listeners = viewRec.getListeners() ;
        if ( autoStart )
            start () ;
    } /* RefreshProductViewThread */



    /*-----------------------------------------------------------*/
    /* run
     */
    /**
     *    This method will actually do all of the work
     *  that the thread is going to do
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void run () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Add ourselves as a QuestProcess so the user can stop us */
        /* with the stop button                                      */
        /*                                                           */
        /*-----------------------------------------------------------*/
        ProcessManagerSystem.addProcess ( this ) ;
        fireStartEvent() ;

        if ( continueProcess ) {


            Qit.setEnabled ( false ) ;
            if ( viewRec != null ) {

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Search for either templates or user queries             */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( viewRec.isCustomView() ) {
                    viewRec.setResults ( viewRec.getCustomView().getResults() ) ;
                    fireStopEvent() ;
                }


                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Search by standard view                                 */
                /*                                                           */
                /*-----------------------------------------------------------*/
                else {

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   Run the view                                            */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    readProductData() ;


                    fireStopEvent() ;
                }

            } /* if viewRec */

        } /* if continueProcess  */

        if ( continueProcess )
            Qit.setEnabled ( true ) ;

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Ok, we are done, remove ourselves from the              */
        /* process Manager                                           */
        /*                                                           */
        /*-----------------------------------------------------------*/
        if ( continueProcess )
            ProcessManagerSystem.removeProcess ( this ) ;


    } /* run */


    /*------------------------------------------------------------*/
    /* readProductData
     */
    /**
     *    This method will perform certain SQL queries
     *  for products using JDBC
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private boolean readProductData () {
        boolean   rc        = false ;
        String    query     = null ;

        if ( continueProcess ) {
            //boolean includeCategory = ( viewRec.getCategoryInd() >= 0 ) ;
            String  qualifier       = "QUEST" ;
            //String  publish         = viewRec.getShowDeletedDocsOnly() ? "N" : "Y" ;
            //String  recycle         = viewRec.getShowDeletedDocsOnly() ? "Y" : "N" ;

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Create the Query to read based on the view. Please note */
            /* The following rules for building the SQL String!!!!       */
            /*                                                           */
            /* 1. The values in the SELECT clause of each statement MUST */
            /*    remain as they are now (just copy and paste the select */
            /*    clause.                                                */
            /* 2. Always use the ViewRec to pull latest properties for   */
            /*    your view (properties include brand, machine, show     */
            /*    delete docs etc..)                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            switch ( viewRec.getViewType() ) {
                
                
                case viewRec.VIEW_EXACT_SQL :
                    query = viewRec.getSQLQuery () ;
                    break ;

                
                case viewRec.VIEW_BY_MACHINE :
                    String machine = viewRec.getMachine ( true ) ;

                    if ( ( machine != null ) && ( machine.length() > 0 ) ) {

                        if ( machine.length() > 4 )
                            machine = machine.substring ( 0, 4 ) ;

                        query = "SELECT DISTINCT(DOCUMENTS.DOCIND),      DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                                "       DOCUMENTS.DOCCLASSIND, DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                                "       DOCUMENTS.CHANGEDTIME "+
                                "FROM   TIGRIS.DOCUMENTS DOCUMENTS,"+
                                "       TIGRIS.PRODDOCS PRODDOC, "+
                                "       TIGRIS.PRODUCTS PRODUCTS "+
                                "WHERE  DOCUMENTS.DOCTYPEIND = "+viewRec.getDocTypeInd()+" AND "+
                                "       DOCUMENTS.PUBLISH    = '"+publish+"' AND "+
                                "       DOCUMENTS.RECYCLED   = '"+recycle+"' AND "+
                                "       PRODDOC.DOCIND       = DOCUMENTS.DOCIND AND "+
                                "       PRODUCTS.PRODINT     = PRODDOC.PRODINT AND "+
                                "       translate(PRODUCTS.MACHINE) like '%"+machine+"%' ";
                    }
                    break ;


                case viewRec.VIEW_BY_BRANDGROUPIND :
                    query = "SELECT DISTINCT(DOCUMENTS.DOCIND),      DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                            "       DOCUMENTS.DOCCLASSIND, DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                            "       DOCUMENTS.CHANGEDTIME "+
                            "FROM   TIGRIS.DOCUMENTS DOCUMENTS,"+
                            "       TIGRIS.DOCBRAND DOCBRAND "+
                            "WHERE  DOCBRAND.BRANDGROUPIND = "+viewRec.getBrandGroupInd()+" AND "+
                            "       DOCUMENTS.DOCIND     = DOCBRAND.DOCIND AND"+
                            "       DOCUMENTS.DOCTYPEIND = "+viewRec.getDocTypeInd()+" AND "+
                            "       DOCUMENTS.PUBLISH    = '"+publish+"' AND "+
                            "       DOCUMENTS.RECYCLED   = '"+recycle+"' " ;
                    qualifier="TIGRIS" ;
                    break ;

                case viewRec.VIEW_BY_OPTION_PARTNUMBER :
                    String partNumber = viewRec.getPartNumber ( true ) ;

                    if ( ( partNumber != null ) && ( partNumber.length() > 0 ) )
                        query = "SELECT DISTINCT(DOCUMENTS.DOCIND),      DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                                "       DOCUMENTS.DOCCLASSIND, DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                                "       DOCUMENTS.CHANGEDTIME "+
                                "FROM   TIGRIS.DOCUMENTS DOCUMENTS, "+
                                "       TIGRIS.PRODDOCS  PRODDOC, "+
                                "       TIGRIS.PRODUCTS  PRODUCTS "+
                                "WHERE  PRODUCTS.PRODNUM     = '"+partNumber+"' AND "+
                                "       DOCUMENTS.DOCTYPEIND = "+viewRec.getDocTypeInd()+" AND "+
                                "       PRODUCTS.PRODINT     = PRODDOC.PRODINT AND "+
                                "       DOCUMENTS.DOCIND     = PRODDOC.DOCIND AND "+
                                "       DOCUMENTS.RECYCLED   = '"+recycle+"' AND "+
                                "       DOCUMENTS.PUBLISH    = '"+publish+"' " ;
                    break ;


                
                case viewRec.VIEW_SOURCE :
                    String activeSource = viewRec.getSource ( true ) ;

                    if ( ( activeSource != null ) && ( activeSource.length() > 0 ) ) {
                        query = "SELECT DISTINCT(DOCUMENTS.DOCIND),       DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER, "+
                                "       DOCUMENTS.DOCCLASSIND,  DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER, "+
                                "       DOCUMENTS.CHANGEDTIME "+
                                "FROM TIGRIS.DOCUMENTS DOCUMENTS "+
                                "WHERE translate(DOCUMENTS.SOURCE) LIKE '%"+activeSource.toUpperCase()+"%' AND "+
                                "      DOCUMENTS.RECYCLED  = '"+recycle+"' AND "+
                                "      DOCUMENTS.PUBLISH   = '"+publish+"' ";

                        qualifier = "TIGRIS" ;
                    }
                    break ;

                
                case viewRec.VIEW_ACTIVE_BOOKMARKS :
                    int    activeIndex = 0 ;
                    Vector bookmarkVec = viewRec.getBookmarkVec() ;
                    int    activeSize  = bookmarkVec.size() ;

                    query = "SELECT DISTINCT(DOCUMENTS.DOCIND),      DOCUMENTS.DOCIND,  DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                            "       DOCUMENTS.DOCCLASSIND, DOCUMENTS.LNDOCID, DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                            "       DOCUMENTS.CHANGEDTIME "+
                            "FROM TIGRIS.DOCUMENTS DOCUMENTS "+
                            "WHERE DOCUMENTS.RECYCLED   = '"+recycle+"' AND "+
                            "      DOCUMENTS.PUBLISH    = '"+publish+"'      AND "+
                            "      DOCUMENTS.DOCIND IN ( " ;

                    while ( activeIndex < activeSize ) {
                        Integer intRec = (Integer)((Bookmark)bookmarkVec.elementAt(activeIndex)).getObject() ;
                        query+=" "+intRec.intValue() ;
                        if ( activeIndex < (activeSize-1) )
                            query+="," ;
                        activeIndex++ ;
                    }

                    query+=") " ;
                    break ;


                case viewRec.VIEW_DRAFT_BOOKMARKS :
                    int    draftIndex = 0 ;
                    Vector draftBookmarkVec = viewRec.getBookmarkVec() ;
                    int    draftSize  = draftBookmarkVec.size() ;

                    query = "SELECT DISTINCT(DOCUMENTS.DOCIND), DOCUMENTS.EXISTINGDOCIND, DOCUMENTS.TITLE,   DOCUMENTS.OWNER,      "+
                            "       DOCUMENTS.DOCCLASSIND,      DOCUMENTS.LNDOCID,        DOCUMENTS.WORKIND, DOCUMENTS.DBUSER,     "+
                            "       DOCUMENTS.CHANGEDTIME "+
                            "FROM QUEST.DOCUMENTS DOCUMENTS, "+
                            "WHERE DOCUMENTS.STOPDATE IS NULL     AND "+
                            "      DOCUMENTS.RECYCLED  = '"+recycle+"' AND "+
                            "      DOCUMENTS.TEMPLATE  = 'N'      AND "+
                            "      DOCUMENTS.DOCIND IN ( " ;

                    while ( draftIndex < draftSize ) {
                        Integer intRec = (Integer)((Bookmark)draftBookmarkVec.elementAt(draftIndex)).getObject() ;
                        query+=" "+intRec.intValue() ;
                        if ( draftIndex < (draftSize-1) )
                            query+="," ;
                        draftIndex++ ;
                    }

                    query+=") " ;
                    break ;


            }


            if ( query != null ) {
                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Always add the category specification to the query      */
                /*                                                           */
                /*-----------------------------------------------------------*/
                if ( includeCategory ) {
                    query = Text.replaceInStr ( query, "WHERE", ", "+qualifier+".DOCCATEGORY DOCCATEGORY WHERE" ) ;
                    query+=" AND DOCCATEGORY.DOCIND    = DOCUMENTS.DOCIND "+
                           " AND DOCCATEGORY.DOCCATIND = "+viewRec.getCategoryInd()+" "+
                           "ORDER BY DOCUMENTS.CHANGEDTIME DESC " ;
                } else
                    query+="ORDER BY DOCUMENTS.CHANGEDTIME DESC " ;          

                /*-----------------------------------------------------------*/
                /*                                                           */
                /*   Now, call the query                                     */
                /*                                                           */
                /*-----------------------------------------------------------*/
                rc = callDocListReader ( viewRec, query ) ;
            }

        } /* if continue  */

        return( rc ) ;

    } 





    /*------------------------------------------------------------*/
    /* callProductListReader
     */
    /**
     *    This method will call the DocListReader
     *  stored procedure with the query passed in. It
     *  will populate the 'resultVec', 'actualCount', and 'fromCount'
     *  properties of the search rec.
     *  
     *
     *  @param viewRec The DocSearchRec to populate
     *  @param query The query to run
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public boolean callProductListReader ( ProductViewRec viewRec, String query ) {
        String    rtnName   = "callProductListReader" ;
        SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, rtnName, 5 ) ;
        boolean   rc        = false ;

        if ( ( continueProcess ) && ( query != null ) ) {

            if ( sqlMethod != null ) {
                try {
                    CallableStatement stmt  = sqlMethod.createStoredProc ( "QUEST.DocListReader", 6 ) ;                                    

                    /*-----------------------------------------------------------*/
                    /*                                                           */
                    /*   If we have a query and can continueProcessing           */
                    /* then call the stored procedure                            */
                    /*                                                           */
                    /*-----------------------------------------------------------*/
                    if ( continueProcess ) {
                        int count        = 0 ;
                        int actualCount  = 0 ;

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Set up the call                                         */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        stmt.setBytes             ( 1, query.getBytes() ) ;
                        stmt.setInt               ( 2, viewRec.getMaxDocumentReturn() ) ;
                        stmt.setInt               ( 3, viewRec.getFromCount() ) ;
                        stmt.registerOutParameter ( 4, Types.INTEGER ) ;
                        stmt.registerOutParameter ( 5, Types.LONGVARBINARY ) ;
                        stmt.registerOutParameter ( 6, Types.LONGVARBINARY ) ;

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Execute                                                 */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        LogSystem.log ( 1, "Executing Query:"+query ) ;
                        stmt.execute() ;                                     

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Get the StoredProcRec from the Lob                      */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        StoredProcRec retRec = (StoredProcRec)StoredProcUtil.getObjectFromStatement ( stmt, 6 ) ;                


                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Throw an error if need be                               */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        if ( ( retRec != null ) && ( retRec.error ) ) {
                            LogSystem.beginLogSection ( "DocListReader Returned Error" ) ;
                            LogSystem.log ( 1, "sqlCode:   "+retRec.sqlCode ) ;
                            LogSystem.log ( 1, "userError: "+retRec.userError ) ;
                            LogSystem.log ( 1, "errorStr:  "+retRec.errorStr ) ;
                            LogSystem.log ( 1, "error:     "+retRec.error ) ;
                            LogSystem.endLogSection () ;
                        }

                        /*-----------------------------------------------------------*/
                        /*                                                           */
                        /*   Otherwise, continue                                     */
                        /*                                                           */
                        /*-----------------------------------------------------------*/
                        else {

                            /*-----------------------------------------------------------*/
                            /*                                                           */
                            /*   Get our return object                                   */
                            /*                                                           */
                            /*-----------------------------------------------------------*/
                            viewRec.setResults ( (Vector) StoredProcUtil.getObjectFromStatement ( stmt, 5 ) ) ;
                            if ( viewRec.getResults() != null )
                                LogSystem.log ( 1, "DocListReader returned "+viewRec.getResults().size()+" hit(s)" ) ;
                            else
                                LogSystem.log ( 1, "DocListReader returned no hits" ) ;

                            if ( viewRec.actualCount == 0 )
                                viewRec.actualCount = stmt.getInt ( 4 ) ;

                            rc = true ;

                        } /* Else if we have no error  */

                    } /* if Query is good  */

                    rc = true ;

                } catch ( Exception e ) {
                    sqlMethod.rollBack() ;
                    LogSystem.log ( 1, e, false ) ;
                    GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
                }
            } /* if sqlMethod is not null  */

        } /* if continueProcess  */

        sqlMethod.close() ;

        return( rc ) ;
    }








    /*------------------------------------------------------------*/
    /* getProcessName
     */
    /**
     *    This method will return the name of this
     *  process
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return String
     *//*---------------------------------------------------------*/
    public String getProcessName () {
        return( "Refresh Document List" ) ;
    }




    /*------------------------------------------------------------*/
    /* stopProcess
     */
    /**
     *    This method will attempt to stop the processing
     *  
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    public void stopProcess () {

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   First, set our proccessing flag                         */
        /*                                                           */
        /*-----------------------------------------------------------*/
        continueProcess = false ;                                           

        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Ok, now send the last message to the listeners          */
        /*                                                           */
        /*-----------------------------------------------------------*/
        System.out.println ( "Firing Stop Event!" ) ;
        fireStopEvent() ;

        Qit.setEnabled ( true ) ;
        ProcessManagerSystem.removeProcess ( this ) ;


        /*-----------------------------------------------------------*/
        /*                                                           */
        /*   Now, if any of the statements are running, stop them    */
        /*                                                           */
        /*-----------------------------------------------------------*/
        try {
            if ( sqlStmt != null )
                sqlStmt.cancel() ;
            else if ( stmt != null )
                stmt.cancel() ;
        } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
        }

    }





    /*------------------------------------------------------------*/
    /* fireStopEvent
     */
    /**
     *    This method will fire a stop event to
     *  all registered listeners
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void fireStopEvent () {
        if ( listeners != null ) {
            int i    = 0 ;
            int size = listeners.size() ;

            while ( i < size ) {
                ((DocViewListener)listeners.elementAt(i)).docViewStopped ( viewRec ) ;
                i++ ;
            }

        } /* if */
    }



    /*------------------------------------------------------------*/
    /* fireStartEvent
     */
    /**
     *    This method will fire a start event to
     *  all registered listeners
     *  
     *
     *  @param None
     *  @param None
     *  @param None
     *
     *  @return Nothing
     *//*---------------------------------------------------------*/
    private void fireStartEvent () {
        if ( ( continueProcess ) && ( listeners != null ) ) {
            int i    = 0 ;
            int size = listeners.size() ;

            while ( ( continueProcess ) && ( i < size ) ) {
                ((DocViewListener)listeners.elementAt(i)).docViewStarted ( viewRec ) ;
                i++ ;
            }

        } /* if */
    }



} /* RefreshDocViewThread */





