package com.ibm.nzna.projects.qit.product.manager.views ;
/*-----------------------------------------------------------*/
/*   PMViews                                                 */
/*                                                           */
/*   Description:                                            */
/*      Source code for: PMViews                             */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import java.util.* ;
import javax.swing.tree.* ;
import javax.swing.* ;
/**
 * This class holds static methods usefull in manipulating Views in the Navigator.
 * Since there are so many views, the views can be looked at this way. A view has a
 * main view type which is Active, Draft, Orphan or Archived. A View has a sub view type
 * which is Action, Question or Symptom.
 *
 * @version 1.0,  
 * @author  
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   PMViews                                                 */
/*                                                           */
/*-----------------------------------------------------------*/
public class PMViews extends Object implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Static Integer Views                                    */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final int DRAFT_PRODUCT              = 1 ;
   public static final int EXISTING_PRODUCT           = 2 ;
   public static final int DRAFT_BY_TYPE              = 3 ;
   

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Views                                                   */
   /*                                                           */
   /*-----------------------------------------------------------*/
   public static final String VIEW_BRAND_DRAFT_ANNOUNCE_DATE   = "com.ibm.nzna.projects.qit.product.manager.views.ViewBrandDraftADate" ;
   public static final String VIEW_BRAND_DRAFT_PROJECT         = "com.ibm.nzna.projects.qit.product.manager.views.ViewBrandDraftProject" ;
   public static final String VIEW_BRAND_DRAFT_TYPE_NEW        = "com.ibm.nzna.projects.qit.product.manager.views.ViewBrandDraftNew" ;
   public static final String VIEW_BRAND_DRAFT_TYPE_UPDATE     = "com.ibm.nzna.projects.qit.product.manager.views.ViewBrandDraftUpdate" ;
   public static final String VIEW_BRAND_DRAFT_TYPE_DELETE     = "com.ibm.nzna.projects.qit.product.manager.views.ViewBrandDraftDelete" ;

   public static final String VIEW_BRAND_PRODUCT_FAMILY        = "com.ibm.nzna.projects.qit.product.manager.views.ViewBrandFamily" ;
   public static final String VIEW_BRAND_PRODUCT_PROJECT       = "com.ibm.nzna.projects.qit.product.manager.views.ViewBrandProject" ;
   public static final String VIEW_BRAND_PRODUCT_ANNOUNCE_DATE = "com.ibm.nzna.projects.qit.product.manager.views.ViewBrandADate" ;
   
   public static final String VIEW_MISFIT_OPICM_PROJECT        = "com.ibm.nzna.projects.qit.product.manager.views.ViewMisfitOpicmProject" ;
   public static final String VIEW_MISFIT_ANNOUNCE_DATE        = "com.ibm.nzna.projects.qit.product.manager.views.ViewMisfitADate" ;

   public static final String VIEW_DATAMAP_BRAND_FAMILY_SERIES = "com.ibm.nzna.projects.qit.product.manager.views.ViewDataMapBFS" ;
   public static final String VIEW_DATAMAP_GEO_COUNTRY         = "com.ibm.nzna.projects.qit.product.manager.views.ViewDataMapGC" ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static DefaultTreeModel model        = null ;
   private static StringTreeNode   root         = null ;



   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the entire JTree
    *  Model that needs to be in a JTree to support
    *  the view passed in
    *
    *  @param view One of the VIEW_XXX strings defined in this class
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static PMViewModel getViewModel ( String view, TypeCategoryRec typeCategoryRec ) {       
      PMViewModel      viewModel = new PMViewModel () ;
      DefaultTreeModel model     = null ;
      PMNavView        navView   = null ;                                

      if ( view != null ) {
         try {

            LogSystem.log ( 1, "Creating PMView :"+view+":" ) ;
            try {
               navView = (PMNavView) Class.forName ( view ).newInstance() ;
            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
            }

            navView.setBrand     ( typeCategoryRec ) ;
            model = navView.getJTreeModel() ;

         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }

      viewModel.treeModel = model ;
      viewModel.pmView    = navView ;

      return( viewModel ) ;
   }




   /*------------------------------------------------------------*/
   /* getViewName
    */
   /**
    *    This method will return the user readable
    *  description of a view.
    *  
    *
    *  @param view The view to check
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getViewName ( String view ) {
      String retStr = "" ;

      if ( view != null ) {

         try {
            PMNavView navView = (PMNavView) Class.forName ( view ).newInstance() ;

            retStr = navView.getName() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e ) ;
         }
      }

      return( retStr );
   }


   /*------------------------------------------------------------*/
   /* getViewShortName
    */
   /**
    *    This method will return the user readable
    *  description of a view.
    *  
    *
    *  @param view The view to check
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static String getViewShortName ( String view ) {
      String retStr = "" ;

      if ( view != null ) {

         try {
            PMNavView navView = (PMNavView) Class.forName ( view ).newInstance() ;

            retStr = navView.getShortName() ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
         }
      }

      return( retStr );
   }




   

} /* OAs */
