package com.ibm.nzna.projects.qit.product.manager.views ;         
/*-----------------------------------------------------------*/
/*   ViewBrandADate                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewBrandADate                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This view will list Product Drafts by Announce Date.
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewBrandADate                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewBrandADate  extends Object implements PMView, ProductConst, PMNavView, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   These are the different levels, or drill downs          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String LEVEL_DATE    = "DATE" ;
   private static final String LEVEL_FAMILY  = "FAMILY" ;
   private static final String LEVEL_MACHINE = "MACHINE" ;
   private static final String LEVEL_MODEL   = "MODEL" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  DefaultMutableTreeNode root     = null ;
   private  TypeCategoryRec        brandRec = null ;


   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_BY_ANNOUNCE_DATE ) ) ;
   }                         

   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( "SHORT NAME!" ) ;
   }

   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the brand that this
    *  view will be based on
    *  
    *
    *  @param brandRec The BrandRec to base the view on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( TypeCategoryRec brandRec ) {
      this.brandRec = brandRec ;
   }





   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the complete DefaultTreeModel
    *  needed to show this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DefaultTreeModel
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getJTreeModel () {
      DefaultTreeModel model        = null ;
      ViewTreeNode     viewTreeNode = null ;
      Vector           dates        = readDistinctDates () ;

      model = new DefaultTreeModel ( root = new DefaultMutableTreeNode("") ) ;                                                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, go through the list of dates and add each          */
      /* date as a ViewTreeNode. This will allow the user to selec */
      /* each one. We will also put the SQL needed to read that    */
      /* Date under each one.                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( dates != null ) && ( dates.size() > 0 ) ) {
         int i    = 0 ;
         int size = dates.size() ;

         while ( i < size ) {
            viewTreeNode = new ViewTreeNode ( DateSystem.prettyDateFromDBDate ( dates.elementAt(i).toString() ), "" ) ;
            viewTreeNode.setViewLevel ( LEVEL_DATE ) ;
            viewTreeNode.setDataObject ( dates.elementAt(i) ) ;

            viewTreeNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            model.insertNodeInto ( viewTreeNode, root, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      return( model ) ;
   }



   /**
     *  This method will refresh this view under the ViewTreeNode passed in 
     **/
   public void refreshViewTree ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Families under Dates                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( treeNode.getViewLevel().equals ( LEVEL_DATE ) )
         refreshFamiliesUnderDate ( treeModel, treeNode ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Machines Under Family                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( treeNode.getViewLevel().equals ( LEVEL_FAMILY ) )
         refreshMachinesUnderFamily ( treeModel, treeNode ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Models under machine                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( treeNode.getViewLevel().equals ( LEVEL_MACHINE ) )
         refreshModelsUnderMachine ( treeModel, treeNode ) ;

   }




   /**
     *  This method will read in all distinct dates into a vector and returns each
     * date as a string
     **/
   private Vector readDistinctDates () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctDates", 5 ) ;
      Vector    dates     = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = null ;

         rs = stmt.executeQuery ( "SELECT DISTINCT ( ACTUAL ) "+
                                  "FROM PRODUCT.DATES A,        "+
                                  "     PRODUCT.CATEGORY B, "+
                                  "     PRODUCT.PRODUCT C "+
                                  "WHERE TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                  "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                  "      A.PRODUCTIND = C.PRODUCTIND AND "+
                                  "      C.PUBLISH    = 'Y' AND "+
                                  "      B.TYPECATIND = "+brandRec.getInd()+" AND "+
                                  "      ACTUAL IS NOT NULL "+
                                  "ORDER BY ACTUAL DESC FOR FETCH ONLY" ) ;

         while ( rs.next() )
            dates.addElement ( rs.getString(1) ) ;

         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( dates ) ;
   }


   /**
     *  This method will return a list of distinct families (in the form of TypeCategoryRec) objects
     * that fall under the date passed
     *
     * @param date The date to look for
     **/
   private Vector readDistinctFamilies ( String date ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctFamilies ( "+date+" )", 5 ) ;
      Vector    families  = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT ( B.TYPECATIND ) "+
                                              "FROM PRODUCT.DATES A,        "+
                                              "     PRODUCT.CATEGORY B, "+
                                              "     PRODUCT.TYPECATEGORY C, "+
                                              "     PRODUCT.PRODUCT D "+
                                              "WHERE TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                              "      ACTUAL       = '"+date+"' AND "+
                                              "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      B.TYPECATIND = C.TYPECATIND AND "+
                                              "      C.CATEGORYLEVEL = '"+TypeCategory.LEVEL_FAMILY+"' AND "+
                                              "      C.PARENTCATIND  = "+brandRec.getInd()+" AND "+
                                              "      D.PRODUCTIND    = A.PRODUCTIND AND "+
                                              "      D.PUBLISH       = 'Y' "+
                                              "FOR FETCH ONLY" ) ;

         while ( rs.next() )
            families.addElement ( TypeCategory.getCategory ( rs.getInt(1) ) ) ;

         rs.close() ;
         new com.ibm.nzna.shared.sort.QuickSort ( families ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( families ) ;

   }



   /**
     *  This method will return a list of distinct machines (in the form of TypeCategoryRec) objects
     * that fall under the date and family passed
     *
     * @param date The date to look for
     **/
   private Vector readDistinctMachines ( String date, TypeCategoryRec family ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctMachines ( "+date+" )", 5 ) ;
      Vector    machines  = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT ( B.TYPECATIND ) "+
                                              "FROM PRODUCT.DATES A, "+
                                              "     PRODUCT.CATEGORY B, "+
                                              "     PRODUCT.TYPECATEGORY C, "+
                                              "     PRODUCT.TYPECATCHILD D, "+
                                              "     PRODUCT.PRODUCT E "+
                                              "WHERE TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                              "      ACTUAL       = '"+date+"' AND "+
                                              "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      B.TYPECATIND = C.TYPECATIND AND "+
                                              "      C.CATEGORYLEVEL IN ( '"+TypeCategory.LEVEL_SUBCATEGORY+"', '"+TypeCategory.LEVEL_MACHINE+"' ) AND "+
                                              "      C.TYPECATIND = D.TYPECATIND AND "+
                                              "      D.PARENTCATIND  = "+family.getInd()+" AND "+
                                              "      E.PRODUCTIND    = A.PRODUCTIND AND "+
                                              "      E.PUBLISH       = 'Y' "+
                                              "FOR FETCH ONLY" ) ;

         while ( rs.next() )
            machines.addElement ( TypeCategory.getCategory ( rs.getInt(1) ) ) ;

         new com.ibm.nzna.shared.sort.QuickSort ( machines ) ;
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( machines ) ;

   }




   /**
     *  This method will return a list of distinct models (in the form of TypeCategoryRec) objects
     * that fall under the date, machine and family passed
     *
     * @param date The date to look for
     **/
   private Vector readDistinctModels ( String date, TypeCategoryRec family, TypeCategoryRec machine ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctModels ( "+date+" )", 5 ) ;
      Vector    models  = new Vector ( 1 ) ;

      try {
         boolean   option = machine.getCategoryLevel().equals ( TypeCategory.LEVEL_SUBCATEGORY ) ;
         Statement stmt   = sqlMethod.createStatement() ;
         ResultSet rs     = stmt.executeQuery ( "SELECT A.PRODUCTIND, A.PRODNUM,       A.DESCRIPTION,   A.CREATEDON, "+
                                                "       A.CREATEDBY,  A.LASTTOUCHEDON, A.LASTTOUCHEDBY, C.DESCRIPTION, "+
                                                "       A.CHECKEDOUTBY "+
                                                "FROM PRODUCT.PRODUCT A, "+
                                                "     PRODUCT.CATEGORY B, "+
                                                "     PRODUCT.TYPECATEGORY C, "+
                                                "     PRODUCT.DATES D "+
                                                "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
                                                "      A.PUBLISH    = 'Y' AND "+
                                                "      B.TYPECATIND IN (SELECT TYPECATIND "+
                                                "                       FROM PRODUCT.TYPECATEGORY "+
                                                "                       WHERE PARENTCATIND = "+machine.getInd()+") AND "+
                                                "      C.TYPECATIND  = B.TYPECATIND AND "+
                                                "      D.PRODUCTIND  = A.PRODUCTIND AND "+
                                                "      D.ACTUAL      = '"+date+"' AND "+
                                                "      D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" FOR FETCH ONLY" ) ;


         while ( rs.next() ) {
            ProductRow productRow = new ProductRow ( rs.getInt ( 1 ),                                 //Product Ind
                                                     0,                                               //Existing Product Ind
                                                     option ? rs.getString(8) : rs.getString ( 2 ),   //PRoduct number or Part Number
                                                     rs.getString ( 3 ),                              //Description
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 4 ).trim() ),//Created on
                                                     UserSystem.getNameFromUserId ( rs.getString ( 5 ).trim()),//Created     By
                                                     DateSystem.prettyDateFromDBDate ( date ),        //Actual Announce Date
                                                     "",                                              //Target Announce Date
                                                     UserSystem.getNameFromUserId ( rs.getString ( 7 ).trim()),  //LAstTouched By
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 6 ).trim() ),//LastTouched on
                                                     false ) ;

            if ( rs.getString(9) != null )
               productRow.setImage ( ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.SMALL_PRODUCT_LOCK ) ) ;


            productRow.setColumnOrderData ( 0, productRow.COL_LOCKED  ) ;
            productRow.setColumnOrderData ( 1, productRow.COL_PRODNUM  ) ;
            productRow.setColumnOrderData ( 2, productRow.COL_DESCRIPTION  ) ;
            productRow.setColumnOrderData ( 3, productRow.COL_PROJECT  ) ;
            productRow.setColumnOrderData ( 4, productRow.COL_ANNOUNCEDATE  ) ;
            productRow.setColumnOrderData ( 5, productRow.COL_CREATEDATE  ) ;
            productRow.setColumnOrderData ( 6, productRow.COL_LASTTOUCHEDBY  ) ;
            productRow.setColumnOrderData ( 7, productRow.COL_LASTTOUCHEDON  ) ;


            models.addElement ( productRow ) ;

         }

         rs.close() ;
         new com.ibm.nzna.shared.sort.QuickSort ( models ) ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;


      /*------------------------------------------------------------*/
      /*                                                            */
      /*  Now, we must read in the project for each product read in */
      /*                                                            */
      /*------------------------------------------------------------*/
      ViewUtil.readProjectForProducts ( "PRODUCT", models, false ) ;

      return( models ) ;

   }



   /**
     *  This method will refresh the list of families under the date represented by
     * the ViewTreeNode passed in
     **/
   private void refreshFamiliesUnderDate ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      String date       = (String)treeNode.getDataObject() ;
      Vector families   = readDistinctFamilies ( date ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have families, add them                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( families != null ) && ( families.size() > 0 ) ) {
         int          i       = 0 ;
         int          size    = families.size() ;
         ViewTreeNode newNode = null ;

         while ( i < size ) {
            newNode = new ViewTreeNode ( ((TypeCategoryRec)families.elementAt(i)).getDescript(), "" ) ;
            newNode.setDataObject ( families.elementAt(i) ) ;
            newNode.setViewLevel ( LEVEL_FAMILY ) ;

            newNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            treeNode.add  ( newNode ) ;
            //treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshFamiliesUnderDate  */



   /**
     *  This method will refresh the list of machines under the family represented by
     * the ViewTreeNode passed in
     **/
   private void refreshMachinesUnderFamily ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      TypeCategoryRec family     = (TypeCategoryRec)treeNode.getDataObject() ;
      String          date       = (String)((ViewTreeNode)treeNode.getParent()).getDataObject() ;
      Vector          machines   = readDistinctMachines ( date, family ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have machines, add them                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( machines != null ) && ( machines.size() > 0 ) ) {
         int          i       = 0 ;
         int          size    = machines.size() ;
         ViewTreeNode newNode = null ;

         while ( i < size ) {
            newNode = new ViewTreeNode ( ((TypeCategoryRec)machines.elementAt(i)).getDescript(), "" ) ;
            newNode.setDataObject ( machines.elementAt(i) ) ;
            newNode.setViewLevel ( LEVEL_MACHINE ) ;

            newNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            treeNode.add ( newNode ) ;
            //treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshMachinesUnderFamily  */

   /**
     *  This method will refresh the list of Models/Parts under the machine represented by
     * the ViewTreeNode passed in
     **/
   private void refreshModelsUnderMachine ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      ViewTreeNode    familyNode = ((ViewTreeNode)treeNode.getParent()) ;
      TypeCategoryRec family     = (TypeCategoryRec)familyNode.getDataObject() ;
      TypeCategoryRec machine    = (TypeCategoryRec)treeNode.getDataObject() ;
      String          date       = (String)((ViewTreeNode)familyNode.getParent()).getDataObject() ;
      Vector          models     = readDistinctModels ( date, family, machine ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have models, add them                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( models != null ) && ( models.size() > 0 ) ) {
         int                i       = 0 ;
         int                size    = models.size() ;
         MutableTreeWrapper newNode = null ;

         while ( i < size ) {
            newNode = new MutableTreeWrapper () ;
            newNode.setDataObject ( models.elementAt(i) ) ;
            treeNode.add ( newNode ) ;

            //treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshModelsUnderMachine  */



   /**
     *  This method should return the width in pixels of the padding/buffer space
     * before the title columns begin in the DetailJTree
     **/
   public int getTitlePadding () {
      return( 80 ) ;
   }

   public TypeCategoryRec getBrand () {
      return( brandRec ) ;
   }


   /**
     *  This method will return the SQL that will pull all products that are visible
     * in this view
     **/
   public String getFullSQL () {
      return("SELECT A.PRODUCTIND, A.PRODNUM,       A.DESCRIPTION,   A.CREATEDON, "+
             "       A.CREATEDBY,  A.LASTTOUCHEDON, A.LASTTOUCHEDBY, C.DESCRIPTION, "+
             "       A.CHECKEDOUTBY "+
             "FROM PRODUCT.PRODUCT A, "+
             "     PRODUCT.CATEGORY B, "+
             "     PRODUCT.TYPECATEGORY C, "+
             "     PRODUCT.DATES D "+
             "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
             "      A.PUBLISH    = 'Y' AND "+
             "      B.TYPECATIND IN (SELECT TYPECATIND "+
             "                       FROM PRODUCT.TYPECATEGORY "+
             "                       WHERE PARENTCATIND = "+brandRec.getInd()+") AND "+
             "      C.TYPECATIND  = B.TYPECATIND AND "+
             "      D.PRODUCTIND  = A.PRODUCTIND AND "+
             "      D.ACTUAL      IS NOT NULL AND "+
             "      D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" FOR FETCH ONLY" ) ;
   }



   /**
     * This method will return the column names for this view
     **/
   public String[] getColumnNames () {
      String[] columnHeadings = {
         "",
         Str.getStr ( STR_PRODUCT_NUMBER ),
         Str.getStr ( STR_DESCRIPTION ),
         Str.getStr ( STR_PROJECT ),
         Str.getStr ( STR_CREATE_DATE ),
         Str.getStr ( STR_LAST_TOUCHED_BY ),
         Str.getStr ( STR_LAST_TOUCHED_ON )
      };       

      if ( Brands.isBrandOption ( brandRec ) ) 
         columnHeadings[1] = Str.getStr ( STR_PART_NUMBER ) ;


      return( columnHeadings ) ;
   } /* getColumnNames  */


   /**
     * This method will return the width (in pixels) of each column
     **/
   public int[] getColumnWidths () {
      int[] widths = {
         15 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_PARTNUM ) ? 100 : 0 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_DESCRIPTION ) ? 250 : 0 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_PROJECT ) ? 150 : 0 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_CREATEDATE ) ? 120 : 0,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDON ) ? 120 : 0,
         160} ;

      return( widths ) ;
   }


} /* ViewBrandADate */

