package com.ibm.nzna.projects.qit.product.manager.views ;         
/*-----------------------------------------------------------*/
/*   ViewBrandProductFamily                                  */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewBrandFamily                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This view will list Products by Family.
 *
 * @version 1.0, 8/31/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewBrandFamily                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewBrandFamily  extends Object implements PMView, ProductConst, PMNavView, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   These are the different levels, or drill downs          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String LEVEL_FAMILY  = "FAMILY" ;
   private static final String LEVEL_MACHINE = "MACHINE" ;
   private static final String LEVEL_MODEL   = "MODEL" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  DefaultMutableTreeNode root     = null ;
   private  TypeCategoryRec        brandRec = null ;


   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( "By Family" ) ;
      // return ( Str.getStr ( STR_BY_FAMILY ) ) ;
   }                         

   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( "SHORT NAME!" ) ;
   }

   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the brand that this
    *  view will be based on
    *  
    *
    *  @param brandRec The BrandRec to base the view on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( TypeCategoryRec brandRec ) {
      this.brandRec = brandRec ;
   }





   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the complete DefaultTreeModel
    *  needed to show this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DefaultTreeModel
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getJTreeModel () {
      DefaultTreeModel model        = null ;
      ViewTreeNode     viewTreeNode = null ;
      Vector           families     = readDistinctFamilies () ;

      model = new DefaultTreeModel ( root = new DefaultMutableTreeNode("") ) ;                                                          

      /*--------------------------------------------------------------*/
      /*                                                              */
      /*   Now, go through the list of families and add each          */
      /* family as a ViewTreeNode. This will allow the user to select */
      /* each one. We will also put the SQL needed to read that       */
      /* Family under each one.                                       */
      /*                                                              */
      /*--------------------------------------------------------------*/
      if ( ( families != null ) && ( families.size() > 0 ) ) {
         int i    = 0 ;
         int size = families.size() ;

         while ( i < size ) {
            viewTreeNode = new ViewTreeNode ( ((TypeCategoryRec) families.elementAt(i)).getDescript() , "" ) ;
            viewTreeNode.setViewLevel ( LEVEL_FAMILY ) ;
            viewTreeNode.setDataObject ( families.elementAt(i) ) ;

            viewTreeNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            model.insertNodeInto ( viewTreeNode, root, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      return( model ) ;
   }



   /**
     *  This method will refresh this view under the ViewTreeNode passed in 
     **/
   public void refreshViewTree ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Machines under Families                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( treeNode.getViewLevel().equals ( LEVEL_FAMILY ) )
         refreshMachinesUnderFamily ( treeModel, treeNode ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Models under machine                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( treeNode.getViewLevel().equals ( LEVEL_MACHINE ) )
         refreshModelsUnderMachine ( treeModel, treeNode ) ;

   }


   /**
     *  This method will return a list of distinct families (in the form of TypeCategoryRec) objects
     *
     **/
   private Vector readDistinctFamilies ( ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctFamilies", 5 ) ;
      Vector    families  = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT ( A.TYPECATIND ) "+
                                              "FROM PRODUCT.CATEGORY A, "+
                                              "     PRODUCT.TYPECATEGORY B, "+
                                              "     PRODUCT.PRODUCT C "+
                                              "WHERE A.TYPECATIND = B.TYPECATIND AND "+
                                              "      C.PUBLISH    = 'Y' AND "+
                                              "      A.PRODUCTIND = C.PRODUCTIND AND "+
                                              "      B.CATEGORYLEVEL = '"+TypeCategory.LEVEL_FAMILY+"' AND "+
                                              "      B.PARENTCATIND  = "+brandRec.getInd()+" "+
                                              "FOR FETCH ONLY" ) ;

         while ( rs.next() )
            families.addElement ( TypeCategory.getCategory ( rs.getInt(1) ) ) ;

         new com.ibm.nzna.shared.sort.QuickSort ( families ) ;
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( families ) ;

   }



   /**
     *  This method will return a list of distinct machines (in the form of TypeCategoryRec) objects
     * that fall under the family passed
     *
     * @param family The family to look for
     **/
   private Vector readDistinctMachines ( TypeCategoryRec family ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctMachines ( "+family+" )", 5 ) ;
      Vector    machines  = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT ( B.TYPECATIND ), C.DESCRIPTION, C.PARENTCATIND, C.ORDERNUM, C.CATEGORYLEVEL "+
                                              "FROM PRODUCT.CATEGORY B, "+
                                              "     PRODUCT.TYPECATEGORY C, "+
                                              "     PRODUCT.TYPECATCHILD D, "+
                                              "     PRODUCT.PRODUCT E "+
                                              "WHERE B.TYPECATIND = C.TYPECATIND AND "+
                                              "      E.PUBLISH    = 'Y' AND "+
                                              "      E.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      C.CATEGORYLEVEL IN ( '"+TypeCategory.LEVEL_SUBCATEGORY+"', '"+TypeCategory.LEVEL_MACHINE+"' ) AND "+
                                              "      C.TYPECATIND = D.TYPECATIND AND "+
                                              "      D.PARENTCATIND  = "+family.getInd()+" "+
                                              "ORDER BY C.ORDERNUM FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            TypeCategoryRec machine = new TypeCategoryRec ( rs.getInt(1) ) ;

            machine.setDescript ( rs.getString(2).trim() ) ;
            machine.setParentCatInd ( rs.getInt(3) ) ;
            machine.setOrderNum ( rs.getInt(4) ) ;
            machine.setCategoryLevel ( rs.getString(5).trim() ) ;
            machines.addElement ( machine ) ;
         }

         new com.ibm.nzna.shared.sort.QuickSort ( machines ) ;
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( machines ) ;

   }


   /**
     *  This method will return a list of distinct models (in the form of TypeCategoryRec) objects
     * that fall under the family and machine passed
     *
     * @param family  The family to look for
     * @param machine The machine to look for
     **/
   private Vector readDistinctModels ( TypeCategoryRec family, TypeCategoryRec machine ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctModels ( "+family+","+machine+" )", 5 ) ;
      Vector    models  = new Vector ( 1 ) ;

      try {
         boolean   option = machine.getCategoryLevel().equals ( TypeCategory.LEVEL_SUBCATEGORY ) ;
         Statement stmt   = sqlMethod.createStatement() ;
         ResultSet rs     = stmt.executeQuery( "SELECT A.PRODUCTIND,  A.PRODNUM,       A.DESCRIPTION,   A.CREATEDON,    "+
                                               "       A.CREATEDBY,   A.LASTTOUCHEDON, A.LASTTOUCHEDBY, A.CHECKEDOUTBY, "+
                                               "       B.DESCRIPTION                                                    "+
                                               "FROM PRODUCT.PRODUCT A, "+
                                               "     PRODUCT.CATEGORY C, "+
                                               "     PRODUCT.TYPECATEGORY B "+
                                               "WHERE A.PRODUCTIND = C.PRODUCTIND AND "+
                                               "      A.PUBLISH    = 'Y' AND "+
                                               "      B.TYPECATIND = C.TYPECATIND AND "+
                                               "      B.PARENTCATIND = "+machine.getInd()+" "+
                                               "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            ProductRow productRow = new ProductRow ( rs.getInt ( 1 ),                                             //Product Ind
                                                     0,                                                           //Existing Product Ind
                                                     option ? rs.getString(9) : rs.getString ( 2 ),               //PRoduct number or Part Number
                                                     rs.getString ( 3 ),                                          //Description
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 4 ).trim() ),//Created on
                                                     UserSystem.getNameFromUserId ( rs.getString ( 5 ).trim()),   //Created     By
                                                     "",                                                          //Actual Announce Date
                                                     "",                                                          //Target Announce Date
                                                     UserSystem.getNameFromUserId ( rs.getString ( 7 ).trim()),  //LAstTouched By
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 6 ).trim() ),//LastTouched on
                                                     false ) ;

            if ( rs.getString(8) != null )
               productRow.setImage ( ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.SMALL_PRODUCT_LOCK ) ) ;
            

            productRow.setColumnOrderData ( 0, productRow.COL_LOCKED  ) ;
            productRow.setColumnOrderData ( 1, productRow.COL_PRODNUM  ) ;
            productRow.setColumnOrderData ( 2, productRow.COL_DESCRIPTION  ) ;
            productRow.setColumnOrderData ( 3, productRow.COL_PROJECT  ) ;
            productRow.setColumnOrderData ( 4, productRow.COL_CREATEDATE  ) ;
            productRow.setColumnOrderData ( 5, productRow.COL_LASTTOUCHEDBY  ) ;
            productRow.setColumnOrderData ( 6, productRow.COL_LASTTOUCHEDON  ) ;

            models.addElement ( productRow ) ;

         }

         new com.ibm.nzna.shared.sort.QuickSort ( models ) ;
         rs.close() ;


      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;

      /*--------------------------------------------------------*/
      /*                                                        */
      /*  Now, we must read in the Actual/Target Announce Dates */
      /*                                                        */
      /*--------------------------------------------------------*/
      ViewUtil.readAnnounceDatesForProducts ( "PRODUCT", models, false ) ;


      /*------------------------------------------------------------*/
      /*                                                            */
      /*  Now, we must read in the project for each product read in */
      /*                                                            */
      /*------------------------------------------------------------*/
      ViewUtil.readProjectForProducts ( "PRODUCT", models, false ) ;

      return( models ) ;

   }



   /**
     *  This method will refresh the list of machines under the family represented by
     * the ViewTreeNode passed in
     **/
   private void refreshMachinesUnderFamily ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      TypeCategoryRec family     = (TypeCategoryRec)treeNode.getDataObject() ;
      Vector          machines   = readDistinctMachines ( family ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have machines, add them                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( machines != null ) && ( machines.size() > 0 ) ) {
         int          i       = 0 ;
         int          size    = machines.size() ;
         ViewTreeNode newNode = null ;

         while ( i < size ) {
            newNode = new ViewTreeNode ( ((TypeCategoryRec)machines.elementAt(i)).getDescript(), "" ) ;
            newNode.setDataObject ( machines.elementAt(i) ) ;
            newNode.setViewLevel ( LEVEL_MACHINE ) ;

            newNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            //treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            treeNode.add ( newNode ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshMachinesUnderFamily  */

   /**
     *  This method will refresh the list of Models/Parts under the machine represented by
     * the ViewTreeNode passed in
     **/
   private void refreshModelsUnderMachine ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      ViewTreeNode    familyNode = ((ViewTreeNode)treeNode.getParent()) ;
      TypeCategoryRec family     = (TypeCategoryRec)familyNode.getDataObject() ;
      TypeCategoryRec machine    = (TypeCategoryRec)treeNode.getDataObject() ;
      Vector          models     = readDistinctModels ( family, machine ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have models, add them                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( models != null ) && ( models.size() > 0 ) ) {
         int                i       = 0 ;
         int                size    = models.size() ;
         MutableTreeWrapper newNode = null ;

         while ( i < size ) {
            newNode = new MutableTreeWrapper () ;
            newNode.setDataObject ( models.elementAt(i) ) ;
            treeNode.add ( newNode ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshModelsUnderMachine  */



   /**
     *  This method should return the width in pixels of the padding/buffer space
     * before the title columns begin in the DetailJTree
     **/
   public int getTitlePadding () {
      return( 60 ) ;
   }


   /**
     *  This method will return the SQL that will pull all products that are visible
     * in this view
     **/
   public String getFullSQL () {
      return("SELECT A.PRODUCTIND, A.PRODNUM,       A.DESCRIPTION,   A.CREATEDON, "+
             "       A.CREATEDBY,  A.LASTTOUCHEDON, A.LASTTOUCHEDBY, C.DESCRIPTION, "+
             "       A.CHECKEDOUTBY "+
             "FROM PRODUCT.PRODUCT A, "+
             "     PRODUCT.CATEGORY B, "+
             "     PRODUCT.TYPECATEGORY C "+
             "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
             "      A.PUBLISH    = 'Y' AND "+
             "      B.TYPECATIND IN (SELECT TYPECATIND "+
             "                       FROM PRODUCT.TYPECATEGORY "+
             "                       WHERE PARENTCATIND = "+brandRec.getInd()+") AND "+
             "      C.TYPECATIND  = B.TYPECATIND ");
   }


   public TypeCategoryRec getBrand () {
      return( brandRec ) ;
   }




   /**
     * This method will return the column names for this view
     **/
   public String[] getColumnNames () {
      String[] columnHeadings = {
         "",
         Str.getStr ( STR_PRODUCT_NUMBER ),
         Str.getStr ( STR_DESCRIPTION ),
         Str.getStr ( STR_PROJECT ),
         Str.getStr ( STR_CREATE_DATE ),
         Str.getStr ( STR_LAST_TOUCHED_BY ),
         Str.getStr ( STR_LAST_TOUCHED_ON )
      };       

      if ( Brands.isBrandOption ( brandRec ) )
         columnHeadings[1] = Str.getStr ( STR_PART_NUMBER ) ;


      return( columnHeadings ) ;
   } /* getColumnNames  */


   /**
     * This method will return the width (in pixels) of each column
     **/
   public int[] getColumnWidths () {
      int[] widths = {
         15 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_PARTNUM ) ? 100 : 0 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_DESCRIPTION ) ? 250 : 0 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_PROJECT ) ? 150 : 0 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_CREATEDATE ) ? 120 : 0,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDON ) ? 120 : 0,
         160} ;

      return( widths ) ;
   }

} /* ViewBrandFamily */