package com.ibm.nzna.projects.qit.product.manager.views ;         
/*-----------------------------------------------------------*/
/*   ViewSymptomType                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewSymptomType                     */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This view will list all OPIC/M Brands/Families/Series and their
 * Corresponding Quest Brands/Families
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewDataMapBFS                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewDataMapBFS extends Object implements ProductConst, PMNavView, AppConst {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  TypeCategoryRec           typeCategoryRec = null ;
   private  DefaultMutableTreeNode    root            = null ;

   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_BRAND_FAMILY_SERIES ) ) ;
   }

   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable  short
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( Str.getStr ( STR_BRAND_FAMILY_SERIES ) ) ;
   }


   /**
     *  This view does not support brands, so this method does nothing
     **/
   public void setBrand ( TypeCategoryRec categoryRec ) {

   }

   /**
     *  This method will return the Complete JTree Model for this view
     **/
   public DefaultTreeModel getJTreeModel () {
      DefaultMutableTreeNode root      = null ;
      Vector                 mapVector = createMappingVector() ;
      DefaultTreeModel       model     = new DefaultTreeModel ( root = new DefaultMutableTreeNode() ) ;       

      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add each mapping                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( mapVector != null ) && ( mapVector.size() > 0 ) ) {
         int i    = 0 ;
         int size = mapVector.size() ;

         while ( i < size ) {
            MutableTreeWrapper newNode = new MutableTreeWrapper () ;

            newNode.setDataObject ( mapVector.elementAt(i) ) ;
            model.insertNodeInto ( newNode, root, 0 ) ;
            i++ ;
         }
      }
      return( model ) ;
   }


   /**
     *  This method will refresh the view in a tree. This method does not 
     * support this so this method does nothing
     **/
   public void refreshViewTree ( DefaultTreeModel model, ViewTreeNode node ) {
   }


   /**
     *  This method will return the amount of pixels that should be in place
     * before the first column of this view
     **/
   public int getTitlePadding () {
      return( 10 ) ;
   }


   /**
     *  This method will create a vector of all OPIC/M Brands, Families, Series
     * and their corresponding Quest Brands/Families
     *
     * @return Vector
     **/
   private Vector createMappingVector () {
      Vector       seriesVec  = TypeCategory.getCategories ( TypeCategory.LEVEL_OPICM_SERIES ) ;
      Vector       mappingVec = new Vector ( 1 ) ;          
      OPICMMapper  mapper     = new OPICMMapper () ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Process each series                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( seriesVec != null ) && ( seriesVec.size() > 0 ) ) {
         int i    = 0 ;
         int size = seriesVec.size() ;

         while ( i < size ) {
            try {
               BrandMappingRow mapRow = new BrandMappingRow() ;
               TypeCategoryRec series = (TypeCategoryRec)seriesVec.elementAt(i) ;

               if ( series != null ) {
                  mapRow.oSeries = series ;
                  mapRow.oFamily = TypeCategory.getCategory ( series.getParentCatInd() ) ;
                  mapRow.oBrand  = TypeCategory.getCategory ( TypeCategory.getCategory ( series.getParentCatInd() ).getParentCatInd() ) ;
                  mapRow.family  = mapper.getQuestFamilyFromOPICM ( mapRow.oBrand.toString(), mapRow.oFamily.toString(), mapRow.oSeries.toString() ) ;
                  if ( mapRow.family != null )
                     mapRow.brand   = TypeCategory.getCategory ( mapRow.family.getParentCatInd() ) ;
                  mappingVec.addElement ( mapRow ) ;
               }
            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
            }
            i++ ;
         }
      }

      return( mappingVec ) ;
   }


   public TypeCategoryRec getBrand () {
      return( null ) ;
   }

   public String getFullSQL () {
      return( null ) ;
   }


   /**
     * This method will return the column names for this view
     **/
   public String[] getColumnNames () {
      String[] columnHeadings = {
         "OBrand",
         "OFamily",
         "OSeries",
         Str.getStr ( STR_BRAND ),
         Str.getStr ( STR_FAMILY )
      };

      return( columnHeadings );
   } /* getColumnNames  */


   /**
     * This method will return the width (in pixels) of each column
     **/
   public int[] getColumnWidths () {
      int[] widths = {
         150,
         150,
         150,
         150,
         150} ;

      return( widths ) ;
   }


} /* ViewSymptomType */
