package com.ibm.nzna.projects.qit.product.manager.views ;         
/*-----------------------------------------------------------*/
/*   ViewMisfitADate                                         */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewMisfitADate                     */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This view will list Product Drafts by Announce Date.
 *
 * @version 1.0, 4/18/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ViewMisfitADate                                         */
/*                                                           */
/*-----------------------------------------------------------*/
public class ViewMisfitADate  extends Object implements PMView, ProductConst, PMNavView, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   These are the different levels, or drill downs          */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String LEVEL_DATE    = "DATE" ;
   private static final String LEVEL_FAMILY  = "FAMILY" ;
   private static final String LEVEL_SERIES  = "SERIES" ;
   private static final String LEVEL_PROJECT = "PROJECT" ;
   private static final String LEVEL_MACHINE = "MACHINE" ;
   private static final String LEVEL_MODEL   = "MODEL" ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private  DefaultMutableTreeNode root     = null ;
   private  TypeCategoryRec        brandRec = null ;


   /*------------------------------------------------------------*/
   /* getName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getName () {
      return( Str.getStr ( STR_MISFITS ) +" - "+Str.getStr ( STR_BY_ANNOUNCE_DATE ) ) ;
   }                         

   /*------------------------------------------------------------*/
   /* getShortName
    */
   /**
    *    This method will return the user readable
    *  description of this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getShortName () {
      return( "SHORT NAME!" ) ;
   }

   /*------------------------------------------------------------*/
   /* setBrand
    */
   /**
    *    This method will set the brand that this
    *  view will be based on
    *  
    *
    *  @param brandRec The BrandRec to base the view on
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setBrand ( TypeCategoryRec brandRec ) {
      this.brandRec = brandRec ;
   }





   /*------------------------------------------------------------*/
   /* getJTreeModel
    */
   /**
    *    This method will return the complete DefaultTreeModel
    *  needed to show this view
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return DefaultTreeModel
    *//*---------------------------------------------------------*/
   public DefaultTreeModel getJTreeModel () {
      DefaultTreeModel model        = null ;
      ViewTreeNode     viewTreeNode = null ;
      Vector           dates        = readDistinctDates () ;

      model = new DefaultTreeModel ( root = new DefaultMutableTreeNode("") ) ;                                                          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, go through the list of dates and add each          */
      /* date as a ViewTreeNode. This will allow the user to selec */
      /* each one. We will also put the SQL needed to read that    */
      /* Date under each one.                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/

      if ( ( dates != null ) && ( dates.size() > 0 ) ) {
         int i    = 0 ;
         int size = dates.size() ;

         while ( i < size ) {
            try {
               viewTreeNode = new ViewTreeNode ( DateSystem.prettyDateFromDBDate ( dates.elementAt(i).toString() ), "" ) ;
               viewTreeNode.setViewLevel ( LEVEL_DATE ) ;
               viewTreeNode.setDataObject ( CDate.dateFromDBDate ( (String)dates.elementAt(i) ) ) ;

               viewTreeNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
               model.insertNodeInto ( viewTreeNode, root, 0 ) ;
            } catch ( Exception e ) {
            }
            i++ ;
         } /* while  */
      } /* if  */

      return( model ) ;
   }



   /**
     *  This method will refresh this view under the ViewTreeNode passed in 
     **/
   public void refreshViewTree ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Families under Dates                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( treeNode.getViewLevel().equals ( LEVEL_DATE ) )
         refreshFamiliesUnderDate ( treeModel, treeNode ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Series under Family                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( treeNode.getViewLevel().equals ( LEVEL_FAMILY ) )
         refreshSeriesUnderFamily ( treeModel, treeNode ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Projects under Series                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( treeNode.getViewLevel().equals ( LEVEL_SERIES ) )
         refreshProjectsUnderSeries ( treeModel, treeNode ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Machines under Project                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( treeNode.getViewLevel().equals ( LEVEL_PROJECT ) )
         refreshMachinesUnderProject ( treeModel, treeNode ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh Models under machine                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( treeNode.getViewLevel().equals ( LEVEL_MACHINE ) )
         refreshModelsUnderMachine ( treeModel, treeNode ) ;

   }




   /**
     *  This method will read in all distinct dates into a vector and returns each
     * date as a string
     **/
   private Vector readDistinctDates () {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctDates", 5 ) ;
      Vector    dates     = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = null ;

         rs = stmt.executeQuery ( "SELECT DISTINCT ( ACTUAL ), TARGET "+
                                  "FROM PRODRAFT.DATES A,        "+
                                  "     PRODRAFT.PRODUCT B, "+
                                  "     PRODRAFT.CATEGORY C "+
                                  "WHERE TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                  "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                  "      B.MISFIT = 'Y' AND "+
                                  "      B.PRODUCTIND = C.PRODUCTIND AND "+
                                  "      C.TYPECATIND = "+brandRec.getInd()+" "+
                                  "ORDER BY ACTUAL DESC FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            String date = null ;
            if ( (rs.getString(1)) != null )
               date =  rs.getString( 1 )  ;
            else if ( (rs.getString(2)) != null )
               date = ( rs.getString( 2 ))  ;
            else
               date = rs.getString( 1 ) ;

            dates.addElement ( date ) ;
         }

         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( dates ) ;
   }


   /**
     *  This method will return a list of distinct families (in the form of TypeCategoryRec) objects
     * that fall under the date passed
     *
     * @param date The date to look for
     **/
   private Vector readDistinctFamilies ( String date ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctFamilies ( "+date+" )", 5 ) ;
      Vector    families  = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT ( B.TYPECATIND ) "+
                                              "FROM PRODRAFT.DATES A,        "+
                                              "     PRODRAFT.CATEGORY B, "+
                                              "     PRODRAFT.TYPECATEGORY C, "+
                                              "     PRODRAFT.PRODUCT D "+
                                              "WHERE A.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                              "      (A.ACTUAL = '"+date+"' OR A.TARGET = '"+date+"' ) AND "+
                                              "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      B.TYPECATIND = C.TYPECATIND AND "+
                                              "      C.CATEGORYLEVEL = '"+TypeCategory.LEVEL_OPICM_FAMILY+"' AND "+
                                              "      C.PARENTCATIND = "+brandRec.getInd()+" AND "+
                                              "      A.PRODUCTIND = D.PRODUCTIND AND "+
                                              "      D.MISFIT = 'Y' "+
                                              "FOR FETCH ONLY" ) ;

         while ( rs.next() )
            families.addElement ( TypeCategory.getCategory ( rs.getInt(1) ) ) ;

         new com.ibm.nzna.shared.sort.QuickSort ( families ) ;
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( families ) ;

   }


   /**
     *  This method will return a list of distinct Series (in the form of TypeCategoryRec) objects
     * that fall under the family passed
     *
     * @param date The date to look for
     * @param family  The family to look for
     **/
   private Vector readDistinctSeries( String date, TypeCategoryRec family ) {
      SQLMethod sqlMethod = new SQLMethod (DatabaseSystem.QUEST, "readDistinctSeries ()", 5) ;
      Vector    series    = new Vector(1) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT ( B.TYPECATIND ) "+
                                              "FROM PRODRAFT.DATES A,        "+
                                              "     PRODRAFT.CATEGORY B, "+
                                              "     PRODRAFT.TYPECATEGORY C, "+
                                              "     PRODRAFT.PRODUCT D "+
                                              "WHERE TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                              "      (ACTUAL = '"+date+"' OR TARGET = '"+date+"' ) AND "+
                                              "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      B.TYPECATIND = C.TYPECATIND AND "+
                                              "      C.PARENTCATIND  = "+family.getInd()+" AND "+
                                              "      C.CATEGORYLEVEL = '"+TypeCategory.LEVEL_OPICM_SERIES+"' AND " + 
                                              "      D.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      D.MISFIT = 'Y' "+
                                              "FOR FETCH ONLY" ) ;


         while ( rs.next() )
            series.addElement( TypeCategory.getDraftCategory( rs.getInt(1) ) ) ;

         new com.ibm.nzna.shared.sort.QuickSort ( series ) ;
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log(1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox(STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sql method */
      sqlMethod.close() ;
      return(series) ;
   }




   /**
     *  This method will return a list of distinct projects (in the form of TypeCategoryRec) objects
     * that fall under the series passed
     *
     * @param date The family to look for
     **/
   private Vector readDistinctProjects( String date, TypeCategoryRec series ) {
      SQLMethod sqlMethod = new SQLMethod (DatabaseSystem.QUEST, "readDistinctProjects ()", 5) ;
      Vector    projects  = new Vector(1) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT ( B.TYPECATIND ) "+
                                              "FROM PRODRAFT.DATES A,        "+
                                              "     PRODRAFT.CATEGORY B, "+
                                              "     PRODRAFT.TYPECATEGORY C, "+
                                              "     PRODRAFT.PRODUCT D "+
                                              "WHERE TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                              "      (ACTUAL = '"+date+"' OR TARGET = '"+date+"' ) AND "+
                                              "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      B.TYPECATIND = C.TYPECATIND AND "+
                                              "      C.PARENTCATIND  = "+series.getInd()+" AND "+
                                              "      C.CATEGORYLEVEL = '"+TypeCategory.LEVEL_PROJECT+"' AND " + 
                                              "      D.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      D.MISFIT = 'Y' "+
                                              "FOR FETCH ONLY" ) ;


         while ( rs.next() )
            projects.addElement( TypeCategory.getDraftCategory( rs.getInt(1) ) ) ;

         new com.ibm.nzna.shared.sort.QuickSort ( projects ) ;
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log(1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox(STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sql method */
      sqlMethod.close() ;
      return(projects) ;
   }



   /**
     *  This method will return a list of distinct machines (in the form of TypeCategoryRec) objects
     * that fall under the family and project passed
     *
     * @param family  The family to look for
     * @param project The project to look for
     **/
   private Vector readDistinctMachines ( String date, TypeCategoryRec project  ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctMachines ( )", 5 ) ;
      Vector    machines  = new Vector ( 1 ) ;

      try {
         Statement stmt = sqlMethod.createStatement() ;
         ResultSet rs   = stmt.executeQuery ( "SELECT DISTINCT ( B.TYPECATIND ) "+
                                              "FROM PRODRAFT.DATES A,        "+
                                              "     PRODRAFT.CATEGORY B, "+
                                              "     PRODRAFT.TYPECATEGORY C, "+
                                              "     PRODRAFT.PRODUCT D "+
                                              "WHERE TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                              "      (ACTUAL = '"+date+"' OR TARGET = '"+date+"' ) AND "+
                                              "      A.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      B.TYPECATIND = C.TYPECATIND AND "+
                                              "      C.PARENTCATIND  = "+project.getInd()+" AND "+
                                              "      C.CATEGORYLEVEL IN ( '"+TypeCategory.LEVEL_MACHINE+"', '"+TypeCategory.LEVEL_SUBCATEGORY+"') AND " +
                                              "      D.PRODUCTIND = B.PRODUCTIND AND "+
                                              "      D.MISFIT = 'Y' "+
                                              "FOR FETCH ONLY" ) ;


         while ( rs.next() )
            machines.addElement ( TypeCategory.getDraftCategory ( rs.getInt(1) ) ) ;

         new com.ibm.nzna.shared.sort.QuickSort ( machines ) ;
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( machines ) ;

   }




   /**
     *  This method will return a list of distinct models (in the form of TypeCategoryRec) objects
     * that fall under the family, project and machine passed
     *
     * @param date The date to look for
     **/
   private Vector readDistinctModels ( String date, TypeCategoryRec project, TypeCategoryRec machine ) {
      SQLMethod sqlMethod = new SQLMethod ( DatabaseSystem.QUEST, "readDistinctModels ( )", 5 ) ;
      Vector    models  = new Vector ( 1 ) ;

      try {
         boolean   option = machine.getCategoryLevel().equals ( TypeCategory.LEVEL_SUBCATEGORY ) ;
         Statement stmt   = sqlMethod.createStatement() ;
         ResultSet rs     = stmt.executeQuery( "SELECT A.PRODUCTIND,  A.PRODNUM,       A.DESCRIPTION,   A.CREATEDON,    "+
                                               "       A.CREATEDBY,   A.LASTTOUCHEDON, A.LASTTOUCHEDBY, A.CHECKEDOUTBY, "+
                                               "       B.DESCRIPTION, D.ACTUAL,        D.TARGET                         "+
                                               "FROM PRODRAFT.PRODUCT A, "+
                                               "     PRODRAFT.CATEGORY C, "+
                                               "     PRODRAFT.TYPECATEGORY B, "+
                                               "     PRODRAFT.DATES D "+
                                               "WHERE A.PRODUCTIND = C.PRODUCTIND AND "+
                                               "      A.PUBLISH    = 'Y' AND "+
                                               "      A.RECYCLED   = 'N' AND "+
                                               "      A.TEMPLATE   = 'N' AND "+
                                               "      A.MISFIT     = 'Y' AND "+
                                               "      A.STOPDATE IS NULL AND "+
                                               "      B.TYPECATIND = C.TYPECATIND AND "+
                                               "      B.PARENTCATIND = "+machine.getInd()+" AND "+
                                               "      A.PRODUCTIND = D.PRODUCTIND AND "+
                                               "      (D.ACTUAL = '"+date+"' OR D.TARGET = '"+date+"' ) AND "+
                                               "      D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" "+
                                               "FOR FETCH ONLY" ) ;

         while ( rs.next() ) {
            String target = rs.getString ( 11 ) ;
            String actual = rs.getString ( 10 ) ;

            if ( target != null )
               target = DateSystem.prettyDateFromDBDate ( target ) ;
            if ( actual != null )
               actual = DateSystem.prettyDateFromDBDate ( actual ) ;

            ProductRow productRow = new ProductRow ( rs.getInt ( 1 ),                                             //Product Ind
                                                     0,                                                           //Existing Product Ind
                                                     option ? rs.getString(9) : rs.getString(2),                  //PRoduct number or Part Number
                                                     rs.getString ( 3 ),                                          //Description
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 4 ).trim() ),//Created on
                                                     UserSystem.getNameFromUserId ( rs.getString ( 5 ).trim()),   //Created     By
                                                     actual,
                                                     target,
                                                     UserSystem.getNameFromUserId ( rs.getString ( 7 ).trim()),  //LAstTouched By
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 6 ).trim() ),//LastTouched on
                                                     true ) ;
            if ( rs.getString(8) != null )
               productRow.setImage ( ImageSystem.getImage ( MainWindow.getInstance(), ImageSystem.SMALL_PRODUCT_LOCK ) ) ;

            models.addElement ( productRow ) ;

         }

         new com.ibm.nzna.shared.sort.QuickSort ( models ) ;
         rs.close() ;

      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack() ;
         GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
      }

      /* Always close your sqlMethod  */
      sqlMethod.close() ;
      return( models ) ;

   }

   /**
     *  This method will refresh the list of families under the date represented by
     * the ViewTreeNode passed in
     **/
   private void refreshFamiliesUnderDate ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      String date       = (String)treeNode.getDataObject() ;
      Vector families   = readDistinctFamilies ( date ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have families, add them                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( families != null ) && ( families.size() > 0 ) ) {
         int          i       = 0 ;
         int          size    = families.size() ;
         ViewTreeNode newNode = null ;

         while ( i < size ) {
            newNode = new ViewTreeNode ( ((TypeCategoryRec)families.elementAt(i)).getDescript(), "" ) ;
            newNode.setDataObject ( families.elementAt(i) ) ;
            newNode.setViewLevel ( LEVEL_FAMILY ) ;

            newNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshFamiliesUnderDate  */




   /**
     *  This method will refresh the list of series under the family represented by
     * the ViewTreeNode passed in
     **/
   private void refreshSeriesUnderFamily ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      TypeCategoryRec family   = (TypeCategoryRec)treeNode.getDataObject() ;
      ViewTreeNode    familyNode = treeNode ;
      String          date       = (String)((ViewTreeNode)familyNode.getParent()).getDataObject() ;
      Vector          series = readDistinctSeries ( date, family ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have series, add them                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( series != null ) && ( series.size() > 0 ) ) {
         int          i       = 0 ;
         int          size    = series.size() ;
         ViewTreeNode newNode = null ;

         while ( i < size ) {
            newNode = new ViewTreeNode ( ((TypeCategoryRec)series.elementAt(i)).getDescript(), "" ) ;
            newNode.setDataObject ( series.elementAt(i) ) ;
            newNode.setViewLevel ( LEVEL_SERIES ) ;

            newNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshSeriesUnderFamily  */



   /**
     *  This method will refresh the list of projects under the series represented by
     * the ViewTreeNode passed in
     **/
   private void refreshProjectsUnderSeries ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      ViewTreeNode    familyNode    = ((ViewTreeNode)treeNode.getParent()) ;
      String          date       = (String)((ViewTreeNode)familyNode.getParent()).getDataObject() ;
      TypeCategoryRec series        = (TypeCategoryRec)treeNode.getDataObject() ;
      Vector          projects      = readDistinctProjects ( date, series ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have projects, add them                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( projects != null ) && ( projects.size() > 0 ) ) {
         int          i       = 0 ;
         int          size    = projects.size() ;
         ViewTreeNode newNode = null ;

         while ( i < size ) {
            newNode = new ViewTreeNode ( ((TypeCategoryRec)projects.elementAt(i)).getDescript(), "" ) ;
            newNode.setDataObject ( projects.elementAt(i) ) ;
            newNode.setViewLevel ( LEVEL_PROJECT ) ;

            newNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshProjectsUnderSeries  */



   /**
     *  This method will refresh the list of machines under the project represented by
     * the ViewTreeNode passed in
     **/
   private void refreshMachinesUnderProject ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      ViewTreeNode    seriesNode  = ((ViewTreeNode)treeNode.getParent()) ;
      ViewTreeNode    familyNode  = ((ViewTreeNode)seriesNode.getParent()) ;
      String          date       = (String)((ViewTreeNode)familyNode.getParent()).getDataObject() ;
      TypeCategoryRec project     = (TypeCategoryRec)treeNode.getDataObject() ;
      Vector          machines    = readDistinctMachines ( date, project ) ;                              

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have machines, add them                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( machines != null ) && ( machines.size() > 0 ) ) {
         int          i       = 0 ;
         int          size    = machines.size() ;
         ViewTreeNode newNode = null ;

         while ( i < size ) {
            newNode = new ViewTreeNode ( ((TypeCategoryRec)machines.elementAt(i)).getDescript(), "" ) ;
            newNode.setDataObject ( machines.elementAt(i) ) ;
            newNode.setViewLevel ( LEVEL_MACHINE ) ;

            newNode.add ( new StringTreeNode ( Str.getStr ( STR_PLEASE_WAIT ) ) ) ;
            treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshMachinesUnderProject  */

   /**
     *  This method will refresh the list of Models/Parts under the machine represented by
     * the ViewTreeNode passed in
     **/
   private void refreshModelsUnderMachine ( DefaultTreeModel treeModel, ViewTreeNode treeNode ) {
      ViewTreeNode    projectNode = ((ViewTreeNode)treeNode.getParent()) ;
      ViewTreeNode    seriesNode  = ((ViewTreeNode)projectNode.getParent()) ;
      ViewTreeNode    familyNode  = ((ViewTreeNode)seriesNode.getParent()) ;
      TypeCategoryRec project     = (TypeCategoryRec)projectNode.getDataObject() ;
      TypeCategoryRec machine     = (TypeCategoryRec)treeNode.getDataObject() ;
      String          date       = (String)((ViewTreeNode)familyNode.getParent()).getDataObject() ;
      Vector          models      = readDistinctModels ( date, project, machine ) ;  


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, now that we have models, add them                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      treeNode.removeAll() ;
      if ( ( models != null ) && ( models.size() > 0 ) ) {
         int                i       = 0 ;
         int                size    = models.size() ;
         MutableTreeWrapper newNode = null ;

         while ( i < size ) {
            newNode = new MutableTreeWrapper () ;
            newNode.setDataObject ( models.elementAt(i) ) ;

            treeModel.insertNodeInto ( newNode, treeNode, 0 ) ;
            i++ ;
         } /* while  */
      } /* if  */

      treeModel.reload ( treeNode ) ;
   } /* refreshModelsUnderMachine  */



   /**
     *  This method should return the width in pixels of the padding/buffer space
     * before the title columns begin in the DetailJTree
     **/
   public int getTitlePadding () {
      return( 80 ) ;
   }



   /**
     *  This method will return the SQL that will pull all products that are visible
     * in this view
     **/
   public String getFullSQL () {
      return("SELECT A.PRODUCTIND, A.PRODNUM,       A.DESCRIPTION,   A.CREATEDON, "+
             "       A.CREATEDBY,  A.LASTTOUCHEDON, A.LASTTOUCHEDBY, C.DESCRIPTION, "+
             "       A.CHECKEDOUTBY "+
             "FROM PRODRAFT.PRODUCT A, "+
             "     PRODRAFT.CATEGORY B, "+
             "     PRODRAFT.TYPECATEGORY C, "+
             "     PRODRAFT.DATES D "+
             "WHERE A.PRODUCTIND = B.PRODUCTIND AND "+
             "      A.TEMPLATE   = 'N' AND "+
             "      A.PUBLISH    = 'Y' AND "+
             "      A.RECYCLED   = 'N' AND "+
             "      A.MISFIT     = 'Y' AND "+
             "      A.STOPDATE   IS NULL AND "+
             "      B.TYPECATIND IN (SELECT TYPECATIND "+
             "                       FROM PRODUCT.TYPECATEGORY "+
             "                       WHERE PARENTCATIND = "+brandRec.getInd()+") AND "+
             "      C.TYPECATIND  = B.TYPECATIND AND "+
             "      D.PRODUCTIND  = A.PRODUCTIND AND "+
             "      D.ACTUAL      IS NOT NULL AND "+
             "      D.TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" FOR FETCH ONLY" ) ;
   }


   public TypeCategoryRec getBrand () {
      return( brandRec ) ;
   }



   /**
     * This method will return the column names for this view
     **/
   public String[] getColumnNames () {
      String[] returnHeadings = null ;

      if ( Brands.isBrandOption ( brandRec ) ) {
         String[] columnHeadings = {
            "",
            Str.getStr ( STR_PART_NUMBER ),
            Str.getStr ( STR_DESCRIPTION ),
            Str.getStr ( STR_ANNOUNCE_DATE ),
            Str.getStr ( STR_CREATE_DATE ),
            Str.getStr ( STR_LAST_TOUCHED_BY ),
            Str.getStr ( STR_LAST_TOUCHED_ON )
         };    

         returnHeadings = columnHeadings ;

      } else {
         String[] columnHeadings = {
            "",
            Str.getStr ( STR_PRODUCT_NUMBER ),
            Str.getStr ( STR_DESCRIPTION ),
            Str.getStr ( STR_ANNOUNCE_DATE ),
            Str.getStr ( STR_CREATE_DATE ),
            Str.getStr ( STR_LAST_TOUCHED_BY ),
            Str.getStr ( STR_LAST_TOUCHED_ON )
         };       

         returnHeadings = columnHeadings ;
      }

      return( returnHeadings ) ;
   } /* getColumnNames  */


   /**
     * This method will return the width (in pixels) of each column
     **/
   public int[] getColumnWidths () {
      int[] widths = {
         15 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_PARTNUM ) ? 100 : 0 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_DESCRIPTION ) ? 250 : 0 ,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_ANNOUNCE ) ? 125 : 0,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_CREATEDATE ) ? 120 : 0,
         PropertySystem.getBool ( PROPERTY_PRODUCT_MGMT_SHOW_LASTTOUCHEDON ) ? 120 : 0,
         160} ;

      return( widths ) ;
   }
} /* ViewMisfitADate */
