package com.ibm.nzna.projects.qit.product.manager.views ;         
/*-----------------------------------------------------------*/
/*   ViewUtil                                                */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ViewUtil                            */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.sql.* ;
import java.util.* ;
import javax.swing.tree.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This class holds simple util methods
 *
 * @version 1.0, 8/31/2001 
 * @author  David Lentz
 */

public class ViewUtil extends Object {




   /**
     * This method will populate all Target and Actual Announce dates for each Product Row passed in. 
     *
     * @param products A Vector of ProductRow objects
     * @param deleteIfNotFound If the product was not found to have an announce date, delete it from the Vector
     * @param schema The Database schema to look for the dates in. This is usually PRODRAFT or PRODUCT
     *
     * @return The populated Vector
     **/
   public static Vector readAnnounceDatesForProducts ( String schema, Vector products, boolean deleteIfNotFound ) {
      Vector readProducts = null ;

      if ( ( products != null ) && ( products.size() > 0 ) ) {
         SQLMethod sqlMethod   = new SQLMethod ( DatabaseSystem.QUEST, "readAnnounceDatesFromProducts", 5 ) ;
         String    productInds = "" ;
         int       i           = 0 ;
         int       baseIndex   = 0 ;
         int       size        = products.size() ;
         int       MAX_SEND    = 199 ;

         while ( baseIndex < size ) {


            i=0 ;
            productInds = "" ;
            while ( ( baseIndex < size )  && ( i < MAX_SEND ) ) {

               productInds+=""+((ProductRow)products.elementAt(baseIndex)).getProductInd() ;

               if ( size < MAX_SEND ) {
                  if ( i < (size-1) )
                     productInds+="," ;
               }

               else {
                  if ( ( i < (MAX_SEND-1) ) && ( baseIndex < (size-1) ) )
                     productInds+="," ;
               }
               i++ ;
               baseIndex++ ;
            }

            try {
               Statement stmt = sqlMethod.createStatement () ;
               ResultSet rs   = stmt.executeQuery ( "SELECT PRODUCTIND, ACTUAL, TARGET "+
                                                    "FROM   "+schema+".DATES "+
                                                    "WHERE TYPEDATEIND = "+TypeList.DATE_ANNOUNCED+" AND "+
                                                    "      PRODUCTIND IN ("+productInds+") FOR FETCH ONLY" ) ;

               while ( rs.next() ) {
                  ProductRow productRow = getProductRowFromInd ( rs.getInt ( 1 ), products ) ;

                  if ( productRow != null ) {
                     if ( rs.getString(2) != null )
                        productRow.setActualDate ( DateSystem.prettyDateFromDBDate ( rs.getString ( 2 ) ) ) ;
                     if ( rs.getString(3) != null )
                        productRow.setTargetDate ( DateSystem.prettyDateFromDBDate ( rs.getString ( 3 ) ) ) ;
                  }
               }

               rs.close() ;
               stmt.close() ;

            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
               sqlMethod.rollBack() ;
            }
         }


         sqlMethod.close() ;
      }

      /*------------------------------------------------*/
      /*                                                */
      /*  Now, if there were any in here without dates, */
      /* delete them if we should                       */
      /*                                                */
      /*------------------------------------------------*/
      if ( deleteIfNotFound ) {
         int i    = 0 ;
         int size = products.size() ;

         while ( i < size ) {
            if ( ( ((ProductRow)products.elementAt(i)).getActualDate() == null ) &&
                 ( ((ProductRow)products.elementAt(i)).getTargetDate() == null ) ) {
               products.removeElementAt(i) ;
               i=0 ;
               size = products.size() ;
            } else
               i++ ;
         }
      }

      return( readProducts ) ;

   } /* readAnnounceDatesFromProducts  */




   /**
     * This method will populate all Target and Actual Announce dates for each Product Row passed in. 
     *
     * @param products A Vector of ProductRow objects
     * @param deleteIfNotFound If the product was not found to have an announce date, delete it from the Vector
     * @param schema The Database schema to look for the dates in. This is usually PRODRAFT or PRODUCT
     *
     * @return The populated Vector
     **/
   public static Vector readProjectForProducts ( String schema, Vector products, boolean deleteIfNotFound ) {
      Vector readProducts = null ;

      if ( ( products != null ) && ( products.size() > 0 ) ) {
         SQLMethod sqlMethod   = new SQLMethod ( DatabaseSystem.QUEST, "readAnnounceDatesFromProducts", 5 ) ;
         String    productInds = "" ;
         int       i           = 0 ;
         int       baseIndex   = 0 ;
         int       size        = products.size() ;
         int       MAX_SEND    = 199 ;

         while ( baseIndex < size ) {


            i=0 ;
            productInds = "" ;
            while ( ( baseIndex < size )  && ( i < MAX_SEND ) ) {

               productInds+=""+((ProductRow)products.elementAt(baseIndex)).getProductInd() ;

               if ( size < MAX_SEND ) {
                  if ( i < (size-1) )
                     productInds+="," ;
               }

               else {
                  if ( ( i < (MAX_SEND-1) ) && ( baseIndex < (size-1) ) )
                     productInds+="," ;
               }
               i++ ;
               baseIndex++ ;
            }

            try {
               Statement stmt = sqlMethod.createStatement () ;
               ResultSet rs   = stmt.executeQuery ( "SELECT B.PRODUCTIND, A.DESCRIPTION "+
                                                    "FROM   "+schema+".TYPECATEGORY A, "+
                                                    "       "+schema+".CATEGORY B "+
                                                    "WHERE A.TYPECATIND = B.TYPECATIND AND "+
                                                    "      B.PRODUCTIND IN ("+productInds+") AND "+
                                                    "      A.CATEGORYLEVEL = '"+TypeCategory.LEVEL_PROJECT+"' FOR FETCH ONLY" ) ;

               while ( rs.next() ) {
                  ProductRow productRow = getProductRowFromInd ( rs.getInt ( 1 ), products ) ;

                  if ( productRow != null )
                     productRow.setProject ( rs.getString(2).trim() ) ;
               }

               rs.close() ;
               stmt.close() ;

            } catch ( Exception e ) {
               LogSystem.log ( 1, e, false ) ;
               sqlMethod.rollBack() ;
            }
         }


         sqlMethod.close() ;
      }

      /*------------------------------------------------*/
      /*                                                */
      /*  Now, if there were any in here without dates, */
      /* delete them if we should                       */
      /*                                                */
      /*------------------------------------------------*/
      if ( deleteIfNotFound ) {
         int i    = 0 ;
         int size = products.size() ;

         while ( i < size ) {
            if ( ( ((ProductRow)products.elementAt(i)).getActualDate() == null ) &&
                 ( ((ProductRow)products.elementAt(i)).getTargetDate() == null ) ) {
               products.removeElementAt(i) ;
               i=0 ;
               size = products.size() ;
            } else
               i++ ;
         }
      }

      return( readProducts ) ;

   } /* readProjectForProducts  */




   /**
     * This method will find a ProductRow from the product indicator
     **/
   private static ProductRow getProductRowFromInd ( int productInd, Vector products ) {
      ProductRow productRow = null ;
      int        i          = 0 ;
      int        size       = products.size() ;

      while ( ( i < size ) && ( productRow == null ) ) {
         if ( ((ProductRow)products.elementAt(i)).getProductInd() == productInd )
            productRow = (ProductRow)products.elementAt(i) ;
         i++ ;
      }

      return( productRow ) ;
   }
} /* ViewUtil  */
