package com.ibm.nzna.projects.qit.product.productEdit ;
/*-----------------------------------------------------------*/
/*   ConfirmDocumentDlg                                      */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ConfirmDocumentDlg                  */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.doc.docedit.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
/**
 * This simple dialog simply shows the user a listof documents. The document
 * will be modal if the parent passed to the constructor is a JDialog. If the
 * parent is a JFrame then the dialog will not be modal and will have the 'Edit'
 * button.
 *
 *
 * @version 1.0, 8/6/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ConfirmDocumentDlg                                      */
/*                                                           */
/*-----------------------------------------------------------*/
public class ConfirmDocumentDlg extends JDialog implements ActionListener, WindowListener, AppConst, ProductConst {

   public static final int CONTINUE = 1 ;
   public static final int CANCEL   = 2 ;

   /**
     *  This is the return code which can be retrieved using getResult
     **/
   private int rc = CANCEL ;

   /**
     *  This is the actual list that will show the documents
     **/
   private DocumentTable documentTable = null ;

   /**
     *  The Cancel button
     **/
   private DButton pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;

   /**
     *  This is the 'Ok' button
     **/
   private DButton pb_OK = new DButton ( "Continue" ) ;

   /**
     *  This is the Vector of documents we will be showing
     **/
   private Vector documents = null ;

   /**
     *  This button bar holds the Ok and Edit buttons
     **/
   ButtonPanel buttonBar = new ButtonPanel () ;     

   /**
     *  If our parent is an AppDefaultWin, this is that instance
     **/
   private AppDefaultWin parentWin = null ;

   /**
     *  This is the count text
     **/
   private JLabel st_COUNT = new JLabel ( "Count:" ) ;

   /**
     *  This is the edit button. This button only shows if this
     * window is non-modal
     **/
   private DButton pb_EDIT = null ;


   /**
     *  This is the default constructor
     **/
   public ConfirmDocumentDlg ( Frame parent, Vector documents ) {
      super ( parent, "The following documents are associated", true ) ;

      this.documents = documents ;
      initialize() ;
      setVisible ( true ) ;
   }

   /**
     *  This is the constructor that makes a JDialog our parent
     **/
   public ConfirmDocumentDlg ( JDialog parent, Vector documents ) {
      super ( parent, "The following documents are associated", true ) ;

      this.documents = documents ;
      initialize() ;
      setVisible ( true ) ;
   }

   /**
     *  This is the constructor that makes a JFrame our parent and is
     * not modal
     **/
   public ConfirmDocumentDlg ( JFrame parent, Vector documents ) {
      super ( parent, "The following documents are associated", false ) ;

      if ( parent instanceof AppDefaultWin )
         parentWin = (AppDefaultWin)parent ;

      this.documents = documents ;
      initialize() ;
      initializeEdit() ;
      setVisible ( true ) ;
   }


   /**
     *  This method will initialize our GUI and set our size/position
     **/
   private void initialize () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      documentTable = new DocumentTable ( documents ) ; 
      st_COUNT.setText ( "Count: "+documents.size() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      addWindowListener ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( null ) ;
      buttonBar.add ( pb_OK ) ;
      buttonBar.add ( pb_CANCEL ) ;
      getContentPane().add ( documentTable ) ;
      getContentPane().add ( st_COUNT ) ;
      getContentPane().add ( buttonBar, BorderLayout.SOUTH ) ;   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set our size                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setSizeFromPrevious() ;
      GUISystem.setPropertiesOnPanel ( getContentPane() );

   }


   /**
     *  This method will set our size from our previous size
     **/
   private void setSizeFromPrevious () {
      setSize ( 400, 300 ) ;
      WinUtil.centerWindow ( this ) ;
   }


   /**
     * This method is called when the user hits the 'OK' button
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_OK ) {
         rc = CONTINUE ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_EDIT                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_EDIT ) {
         DocumentRow docRow = (DocumentRow) documentTable.getSelectedItem() ;

         if ( docRow != null ) {
            if ( parentWin != null ) {
               parentWin.setStatus ( Str.getStr ( STR_READING )+" "+docRow.getTitle() ) ;

               EditDocument.editDocument ( docRow ) ;
            }
         } else
            GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_ITEM ) ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CANCEL                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CANCEL ) {
         rc = CANCEL ;
         dispose() ;
      }
      
   }


   public void windowActivated ( WindowEvent e ) {
   }
   public void windowDeactivated ( WindowEvent e ) {
   }
   public void windowIconified ( WindowEvent e ) {
   }
   public void windowDeiconified ( WindowEvent e ) {
   }
   public void windowOpened ( WindowEvent e ) {
   }
   public void windowClosing ( WindowEvent e ) {
   }
   public void windowClosed ( WindowEvent e ) {
   }

   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;

      super.doLayout() ;
      documentTable.setBounds ( 5, 5, size.width-15, size.height-68-rowHeight ) ;
      st_COUNT.setBounds ( 5, (size.height-68-rowHeight)+5, size.width-15, rowHeight ) ;

      buttonBar.setBounds ( 5, size.height-56, size.width-15, 25 ) ;
   }

   /**
     *  this method will add the edit button to our window
     **/
   public void initializeEdit() {
      pb_EDIT = new DButton ( Str.getStr ( STR_EDIT ) ) ;
      pb_EDIT.addActionListener ( this ) ;
      buttonBar.add ( pb_EDIT ) ;
      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
   }


   public int getResult () {
      return ( rc );
   }
} /* ConfirmDocumentDlg */
