package com.ibm.nzna.projects.qit.product.productEdit ;
/*-----------------------------------------------------------*/
/*   DatabaseInfoPanel                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DatabaseInfoPanel                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.Dimension ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
/**
 * This FormComponent shows the following product information:<br><br> 
 *
 * Active Product Indicator<br>
 * Draft Product Indicator<br>
 * Draft Created On<br>
 * Last Touched By<br>
 * Last Touched On<br>
 * Product Created On<br>
 * Product Created By<br>
 * Last OPIC/M Update<br>
 *
 *
 * @version 1.0, 5/1/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DatabaseInfoPanel                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class DatabaseInfoPanel extends ListPanel implements Authority, ActionListener, ProductPropertyListener, AppConst, ProductConst {

   /**
     *  This is the Value Label holds the Active Product Indicator
     **/
   public ValueLabel activeProductInd = new ValueLabel ( Str.getStr ( STR_ACTIVE_PRODUCT_IND ),
                                                         new JLabel(""+0) ) ;

   /**
     *  This is the Value Label which holds the Draft Product Indicator
     **/
   public ValueLabel draftProductInd = new ValueLabel ( Str.getStr ( STR_DRAFT_PRODUCT_IND ),
                                                        new JLabel ( ""+0 ) ) ;

   /**
     *  This is the Value label which holds the last touched by
     **/
   public ValueLabel lastTouchedBy = new ValueLabel ( Str.getStr ( STR_LAST_TOUCHED_BY ),
                                                      new JLabel ( "-" ) ) ;

   /**
     *  This is thet value label which holds the Last Touched ON
     **/
   public ValueLabel lastTouchedOn = new ValueLabel ( Str.getStr ( STR_LAST_TOUCHED_ON ),
                                                      new JLabel ( "-" ) ) ;

   /**
     * This is the Product Created On
     **/
   public ValueLabel createdOn = new ValueLabel ( Str.getStr ( STR_CREATED_ON ),
                                                  new JLabel ( "-" ) ) ;

   /**
     * This is the product creatd by
     **/
   public ValueLabel createdBy = new ValueLabel ( Str.getStr ( STR_CREATED_BY ),
                                                  new JLabel ( "-" ) ) ;


   /**
     * This is the last opicm update
     **/
   public ValueLabel OPICMKey = new ValueLabel ( "OPIC/M Key",
                                                 new JLabel ( "-" ) ) ;

   /**
     * This is the Draft Created By
     **/
   public ValueLabel draftCreatedBy = new ValueLabel ( "Draft Created By",
                                                       new JLabel ( "-" ) ) ;

   /**
     * This is the Draft Created On
     **/
   public ValueLabel draftCreatedOn = new ValueLabel ( "Draft Created On",
                                                       new JLabel ( "-" ) ) ;

   /**
     *  This is the CHeck box to allow OPIC/M Updates
     **/
   public JCheckBox ck_ALLOW_OPICM = new JCheckBox ( "Allow OPIC/M to update" ) ;


   /**
     *  This is the product we are displaying
     **/
   public Product product = null ;

   /**
     *  This method will create a blank DocumetnInfoFormComponent
     **/
   public DatabaseInfoPanel () {
      super() ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust the GUI properties                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setOpaque ( false ) ;
      setPreferredSize ( new Dimension ( 550, GUISystem.getRowHeight()*15 ) ) ;
      setValueLabelProperties ( activeProductInd ) ;
      setValueLabelProperties ( draftProductInd ) ;
      setValueLabelProperties ( lastTouchedBy ) ;
      setValueLabelProperties ( lastTouchedOn ) ;
      setValueLabelProperties ( draftCreatedOn ) ;
      setValueLabelProperties ( draftCreatedBy ) ;
      setValueLabelProperties ( createdOn ) ;
      setValueLabelProperties ( createdBy ) ;
      setValueLabelProperties ( OPICMKey ) ;
      ck_ALLOW_OPICM.setBackground ( Color.white ) ;
      setPadding ( 3 ) ;
      ck_ALLOW_OPICM.addActionListener ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      add ( activeProductInd ) ;
      add ( createdOn ) ;
      add ( createdBy ) ;
      add ( new JLabel ( "______________________________" ) ) ;
      add ( draftProductInd ) ;
      add ( draftCreatedBy ) ;
      add ( draftCreatedOn ) ;     
      add ( new JLabel ( "______________________________" ) ) ;
      add ( lastTouchedBy ) ;
      add ( lastTouchedOn ) ;     
      add ( new JLabel ( "______________________________" ) ) ;
      add ( OPICMKey ) ;

      if ( UserSystem.hasBPMAuth() ) 
         add ( ck_ALLOW_OPICM ) ;
   }


   /**
     *  This method will set all of the properties for each Value Label
     *  passed in
     **/
   private void setValueLabelProperties ( ValueLabel valueLabel ) {
      valueLabel.setDescriptionWidth ( 150 ) ;
   }

   /**
     *  This method will set the information being displayed to the
     * document passed in
     **/
   public void setProduct ( Product product ) {
      if ( this.product != product ) {
         if ( this.product != null )
            this.product.removeProductPropertyListener ( this ) ;

         this.product = product ;
         product.addProductPropertyListener ( this ) ;
      }

      refresh () ;
   }

   public void productChanged ( Product product, int propertyInd ) {
      setProduct ( product ) ;
   }


   /**
     *  This method will refresh our data
     **/
   private void refresh () {
      if ( product instanceof ProductDraft ) {
         activeProductInd.setValue ( ""+((ProductDraft)product).getExistingProductInd() ) ;
         draftProductInd.setValue ( ""+((ProductDraft)product).getProductInd() ) ;
         draftCreatedOn.setValue ( DateSystem.prettyDateFromStamp ( ((ProductDraft)product).getStartDate() ) ) ;
         draftCreatedBy.setValue ( UserSystem.getNameFromUserId ( ((ProductDraft)product).getDraftCreatedBy() ) ) ;
         if ( ((ProductDraft)product).getOPICMKey() != null )
            OPICMKey.setValue ( ((ProductDraft)product).getOPICMKey() ) ;
         else
            OPICMKey.setValue ( "NA" ) ;
      } else {
         activeProductInd.setValue ( ""+product.getProductInd() ) ;
         draftProductInd.setValue ( "NA" ) ;
         draftCreatedOn.setValue ( "NA" ) ;
         draftCreatedBy.setValue ( "NA" ) ;
         OPICMKey.setValue ( "NA" ) ;
      }

      if ( product.getLastTouchedOn() != null )
         lastTouchedOn.setValue ( DateSystem.prettyDateFromStamp ( product.getLastTouchedOn() ) ) ;
      else
         lastTouchedOn.setValue ( "-" ) ;
      lastTouchedBy.setValue ( UserSystem.getNameFromUserId ( product.getLastTouchedBy() ) ) ;
      createdOn.setValue ( DateSystem.prettyDateFromStamp ( product.getCreatedOn() ) ) ;
      createdBy.setValue ( UserSystem.getNameFromUserId ( product.getCreatedBy() ) ) ;

      ck_ALLOW_OPICM.setSelected ( product.getOPICMImport() ) ;
   }


   /**
     *  This method will be called when the ck_ALLOW_OPICM is selected
     **/
   public void actionPerformed ( ActionEvent e ) {
      if ( e.getSource() == ck_ALLOW_OPICM )
         product.setOPICMImport ( ck_ALLOW_OPICM.isSelected() ) ;
   }


   /**
     *  This method will enable/disable our controls
     **/
   public void setEditable ( boolean editable ) {
      ck_ALLOW_OPICM.setEnabled ( editable ) ;
   }
} /* DatabaseInfoPanel */
