package com.ibm.nzna.projects.qit.product.productEdit ; 
/*-----------------------------------------------------------*/
/*   DatePanel                                               */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DatePanel                           */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.projects.qit.gui.*;
import com.ibm.nzna.projects.qit.app.*;
import com.ibm.nzna.projects.qit.product.*;
import com.ibm.nzna.projects.qit.print.*;
import com.ibm.nzna.projects.qit.product.productEdit.dateEdit.*;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.shared.util.* ;
/**
 * This panel holds a small list of product dates and allows the 
 * user to select to add/remove a country and to edit the dates
 *
 * @version 1.0, 8/26/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DatePanel                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class DatePanel extends DPanel implements ActionListener, ProductPropertyListener, AppConst, ProductConst {

   /**
     *  This property determines if this panel can be edited in any way
     **/
   private boolean editable = true ;

   /**
     *  This is the label for the countries
     **/
   public JLabel st_COUNTRY = new JLabel ( Str.getStr ( STR_COUNTRY ) ) ;

   /**
     *  This is the button for Show All Dates
     **/
   private DButton pb_SHOW_ALL = new DButton ( Str.getStr ( STR_SHOW_ALL_DATES ) ) ;

   /**
     *  This is the drop down for country
     **/
   private JComboBox cb_COUNTRY = new JComboBox () ;

   /**
     *  This is the MultiList for the dates
     **/
   private MultiList cnr_DATES = new MultiList ( GUISystem.getFontUtil() ) ;

   /**
     *  This is the Add Country button
     **/
   private DButton pb_ADD_COUNTRY = new DButton ( Str.getStr ( STR_ADD_COUNTRY ) ) ;

   /**
     *  THis is the edit button
     **/
   private DButton pb_EDIT = new DButton ( Str.getStr ( STR_EDIT ) ) ;

   /**
     *  This is the Remove Country button
     **/
   private DButton pb_REMOVE_COUNTRY = new DButton ( Str.getStr ( STR_REMOVE_COUNTRY ) ) ;

   /**
     * This is the JLabel showing 'Required'
     **/
   private JLabel st_REQUIRED =  null;

   /**
     *  This holds the product we are editing/displaying
     **/
   private Product product = null ;




   /**
     *  Default Constructor
     **/
   public DatePanel () {
      String[] columnHeaders = { Str.getStr (STR_TYPE_DATE), Str.getStr ( STR_DATE )} ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setBackground ( Color.white ) ;
      setPreferredSize ( new Dimension ( 550, 200 ) ) ;   
      cnr_DATES.setColumnHeadings ( columnHeaders ) ;
      cnr_DATES.setColumnWidth ( 0, 140 ) ;
      cnr_DATES.setColumnWidth ( 1, 140 ) ;
      cnr_DATES.setBorder ( GUISystem.blackBorder ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_COUNTRY.addActionListener        ( this ) ;
      pb_EDIT.addActionListener           ( this ) ;
      pb_SHOW_ALL.addActionListener       ( this ) ;
      pb_ADD_COUNTRY.addActionListener    ( this ) ;
      pb_REMOVE_COUNTRY.addActionListener ( this ) ;
      cnr_DATES.addActionListener         ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADd our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_REQUIRED = GUISystem.getRequiredJLabel() ) ;
      add ( st_COUNTRY ) ;
      add ( cb_COUNTRY ) ;
      add ( pb_SHOW_ALL ) ;
      add ( cnr_DATES ) ;
      add ( pb_ADD_COUNTRY ) ;
      add ( pb_EDIT ) ;
      add ( pb_REMOVE_COUNTRY ) ;

      st_REQUIRED.setText ( "Required For Publish" ) ;
   }



   /**
     *  This method will position and size all of our GUI
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       x         = 0 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight () ;


      st_REQUIRED.setBounds ( x, y, size.width, 15 ) ;
      y+=17 ;

      st_COUNTRY.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      cb_COUNTRY.setBounds ( x, y, size.width-92, rowHeight ) ;
      pb_SHOW_ALL.setBounds ( size.width-90, y, 90, rowHeight ) ;
      y+=rowHeight ;

      cnr_DATES.setBounds ( x, y, size.width, size.height-y-37 ) ;

      pb_ADD_COUNTRY.setBounds ( x, size.height-25, 80, 20 ) ;
      pb_EDIT.setBounds ( x+85, size.height-25, 70, 20 ) ;
      pb_REMOVE_COUNTRY.setBounds ( x+85+75, size.height-25, 100, 20 ) ;
   }


   /**
     *  This method will set the product that we are working on
     **/
   public void setProduct ( Product product ) {
      if ( this.product != product ) {
         if ( this.product != null )
            this.product.removeProductPropertyListener ( this ) ;

         this.product = product ;
         product.addProductPropertyListener ( this ) ;
      }

      refresh () ;
   }


   /**
     *  This method will refresh our data
     **/
   private void refresh () {
      Vector countries = getUniqueCountries() ;

      if ( countries.size() == 0 )
         countries.addElement ( "No countries, please press 'Add Country' to add a country" ) ;
      cb_COUNTRY.removeActionListener ( this ) ;
      remove ( cb_COUNTRY ) ;
      cb_COUNTRY = new JComboBox ( countries ) ;
      add ( cb_COUNTRY ) ;
      refreshDatesFromCountry() ;
      cb_COUNTRY.addActionListener ( this ) ;

      revalidate() ;
   }


   /**
     *  This method will go through all of the dates and pulls out each unique
     * country. It will then return a Vector of the results
     *
     * @return Vector of TypeCountryCodeRec objects
     **/
   private Vector getUniqueCountries () {
      Vector countries = new Vector ( 1 ) ;
      Vector dates     = product.getDates() ;

      if ( ( dates != null ) && ( dates.size() > 0 ) ) {
         int                i       = 0 ;
         int                size    = dates.size() ;
         ProductDate        date    = null ;
         TypeCountryCodeRec country = null ;

         while ( i < size ) {
            date = (ProductDate)dates.elementAt(i) ;
            country = (TypeCountryCodeRec)TypeList.getInstance().objectFromInd ( date.getCountryCodeInd(), TypeList.TYPE_COUNTRYCODE ) ;
            if ( !countries.contains ( country ) )
               countries.addElement ( country ) ;
            i++ ;
         }
      }

      dates.removeAllElements() ;
      dates=null ;
      new QuickSort ( countries ) ;

      return( countries ) ;
   }

   /**
     *  this method will be called when a property on the product has changed
     **/
   public void productChanged ( Product product, int propertyInd ) {
      if ( propertyInd == Product.DATES )
         refresh() ;
   }


   /**
     *  This method will refresh all of the dates shown in our MultiList with the 
     * dates under the selected country
     **/
   private void refreshDatesFromCountry () {
      if ( cb_COUNTRY.getSelectedItem() instanceof TypeCountryCodeRec ) {
         Vector              dateTypes   = TypeList.getInstance().getTypeList ( TypeList.TYPE_DATE ) ;
         TypeCountryCodeRec  countryRec  = (TypeCountryCodeRec) cb_COUNTRY.getSelectedItem() ;
         Vector              dates       = product.getDates() ;
         Vector              insertDates = new Vector ( 1 ) ;

         if ( ( dateTypes != null ) && 
              ( dateTypes.size() > 0 ) &&
              ( countryRec != null ) ) {
            int                i       = 0 ;
            int                size    = dateTypes.size() ;
            ProductDate        date    = null ;
            ProductDateRow     row     = null ;

            while ( i < size ) {

               row = getProductDateRowFromCountryType ( dates, 
                                                        countryRec.getInd(), 
                                                        ((TypeDateRec)dateTypes.elementAt(i)).getInd() ) ;

               if ( row == null )
                  row = new ProductDateRow ( (TypeDateRec)dateTypes.elementAt(i) ) ;

               insertDates.addElement ( row ) ;

               i++ ;
            } /* while  */
         } /* if  */

         cnr_DATES.setData ( insertDates ) ;
      }
   } /* refreshDatesFromCountry  */

   /**
     *  This method will return all dates under the specified country and date type
     **/
   private ProductDateRow getProductDateRowFromCountryType ( Vector dates, int countryInd, int dateInd ) {
      ProductDateRow row = null ;

      if ( ( dates != null ) && ( dates.size() > 0 ) ) {
         int i     = 0 ;
         int size  = dates.size() ;

         while ( ( row == null ) && ( i < size ) ) {
            if ( ( ((ProductDate)dates.elementAt(i)).getCountryCodeInd() == countryInd ) &&
                 ( ((ProductDate)dates.elementAt(i)).getDateType() == dateInd ) )
               row = new ProductDateRow ( ((ProductDate)dates.elementAt(i)) ) ;
            i++ ;
         }
      }

      return( row ) ;
   }

   /**
     *  This method will be called when the user selects a country or hits a button
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If the user selected a country, refresh!                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == cb_COUNTRY )
         refreshDatesFromCountry() ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Show all dates                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SHOW_ALL ) {
         new DateEditDlg ( GUISystem.getParentDefWin ( this ), product, editable ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Allow the user to edit a date                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_EDIT ) ||
                ( e.getSource() == cnr_DATES ) )
         editDate() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Allow the user to add a country                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_COUNTRY )
         addCountry () ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Allow the user to remove a country                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_COUNTRY )
         removeCountry() ;


   }



   /**
     *  This method will edit the currently selected date
     **/
   private void editDate () {
      ProductDateRow dateRow = (ProductDateRow)cnr_DATES.getSelectedItem() ;

      if ( dateRow != null ) {
         ProductDate productDate = dateRow.getProductDate() ;
         String      tempDate    = null ;

         if ( productDate == null ) {
            productDate = new ProductDate ( product.getProductInd(),
                                            dateRow.getDateType() ) ;
            productDate.setCountryCodeInd ( ((TypeCountryCodeRec)cb_COUNTRY.getSelectedItem()).getInd() ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the user to edit the date                           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         tempDate = new SingleDateEntryDlg ( GUISystem.getParentDefWin ( this ),
                                             (String)dateRow.getColumnData(0),
                                             (String)dateRow.getColumnData(0),
                                             (productDate.getActualDate() == null) ? productDate.getTargetDate() : productDate.getActualDate() ).getResult() ;

         if ( tempDate != null ) {
            Vector dates = product.getDates() ;

            productDate.setActualDate ( tempDate ) ;

            if ( !dates.contains ( productDate ) )
               dates.addElement ( productDate ) ;
            else {
               ((ProductDate)dates.elementAt ( dates.indexOf ( productDate ) )).setActualDate ( tempDate ) ;
            }
            product.setPropertyChangeEnabled ( false ) ;
            product.setDates ( dates ) ;
            product.setPropertyChangeEnabled ( true ) ;

            refreshDatesFromCountry() ;
         }


      } else
         GUISystem.printBox ( STR_INFO, STR_MUST_SELECT_ITEM ) ;
   }


   /**
     *  This method will ask the user for what country to add, then add it
     * and will then refresh our date list with that new country
     **/
   private void addCountry () {
      Vector   countryVec = null ;
      JListDlg listDlg    = new JListDlg ( GUISystem.getParentDefWin ( this ),
                                           Str.getStr ( STR_ADD_COUNTRY ),
                                           TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ) ;

      listDlg.setSelectionMode ( ListSelectionModel.SINGLE_SELECTION ) ;
      countryVec = listDlg.getResult() ;

      if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
         TypeCountryCodeRec addCountry    = (TypeCountryCodeRec)countryVec.elementAt(0) ;
         Vector             uniqueCountry = getUniqueCountries() ;

         if ( !uniqueCountry.contains ( addCountry ) ) {
            uniqueCountry.insertElementAt ( addCountry, 0 ) ;
            new QuickSort ( uniqueCountry ) ;
            cb_COUNTRY.removeActionListener ( this ) ;
            remove ( cb_COUNTRY ) ;
            cb_COUNTRY = new JComboBox ( uniqueCountry ) ;
            add ( cb_COUNTRY ) ;
            cb_COUNTRY.setSelectedItem ( addCountry ) ;
            cb_COUNTRY.addActionListener ( this ) ;
            refreshDatesFromCountry() ;
            revalidate() ;
         } else
            GUISystem.printBox ( STR_INFO, STR_COUNTRY_ALREADY_ADDED_TO_PRODUCT ) ;
      }
   }


   /**
     *  This method will remove a country
     **/
   private void removeCountry () {
      TypeCountryCodeRec countryRec = (TypeCountryCodeRec)cb_COUNTRY.getSelectedItem() ;

      if ( countryRec != null ) {
         if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_REMOVE_COUNTRY_DATES ) ) {
            Vector dates = product.getDates() ;
            int    i     = 0 ;
            int    size  = dates.size() ;

            while ( i < size ) {
               if ( ((ProductDate)dates.elementAt(i)).getCountryCodeInd() == countryRec.getInd() ) {
                  dates.removeElementAt(i) ;
                  size-- ;
               } else
                  i++ ;
            } /* while  */

            cnr_DATES.removeAll() ;
            product.setDates ( dates ) ;
         }
      } else
         GUISystem.printBox ( STR_INFO, STR_PLEASE_SELECT_COUNTRY ) ;
   }


   protected void setEditable ( boolean editable ) {
      pb_ADD_COUNTRY.setEnabled ( editable ) ;
      pb_REMOVE_COUNTRY.setEnabled ( editable ) ;
      pb_EDIT.setEnabled ( editable ) ;
      this.editable = editable ;
   }
} /* DatePanel */
