package com.ibm.nzna.projects.qit.product.productEdit ;        
/*-----------------------------------------------------------*/
/*   DescriptionPanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DescriptionPanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.app.* ;   
import com.ibm.nzna.projects.common.quest.product.* ;
/**
 * This panel holds two JTextAreas for viewing the OPIC/M Desciption 
 *
 *
 * @version 1.0, 8/26/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DescriptionPanel                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class DescriptionPanel extends DPanel implements FocusListener, ProductPropertyListener, ActionListener, AppConst, ProductConst {


   /**
     *  This is the button which moves the OPIC/M Description to the Quest Description
     **/
   private DButton pb_MOVE    = null ;

   /**
     * This is the static text that reads 'Required'
     **/
   private JLabel st_REQUIRED = null ;

   /**
     *  This is the static text for the OPIC/M Description
     **/
   public JLabel st_OPICM = new JLabel ( Str.getStr ( STR_OPICM ) ) ;

   /**
     *  This is the static text for the Quest Description
     **/
   public JLabel st_QUEST = new JLabel ( Str.getStr ( STR_QUEST ) ) ;

   /**
     *  This is the opcim JTextArea
     **/
   private NoTabJTextArea mle_OPICM = new NoTabJTextArea () ;

   /**
     *  This is the quest JTextArea
     **/
   private NoTabJTextArea mle_QUEST = new NoTabJTextArea () ;

   /**
     *  This is the JScrollPane for the OPICM
     **/
   private JScrollPane scr_OPICM = new JScrollPane ( mle_OPICM ) ;

   /**
     *  This is the JScrollPane for the QUest
     **/
   private JScrollPane scr_QUEST = new JScrollPane ( mle_QUEST ) ;

   /**
     *  This is the counter
     **/
   private JTextAreaCounter mle_QUEST_COUNTER = null ;

   /**
     *  This is the product we are editing
     **/
   private Product product = null ;


   /**
     *  Default Constructor
     **/
   public DescriptionPanel () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create GUI                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_MOVE = new DButton ( ImageSystem.getImageIcon ( this, ImageSystem.RIGHT_ARROW ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setPreferredSize ( new Dimension ( 550, 130 ) ) ;
      setBackground ( Color.white ) ;   
      pb_MOVE.setBorderPainted ( false ) ;
      pb_MOVE.setBackground ( Color.white ) ;
      mle_OPICM.setFont ( pb_MOVE.getFont() ) ;
      mle_QUEST.setFont ( pb_MOVE.getFont() ) ;
      mle_OPICM.setEditable ( false ) ;
      mle_OPICM.setLineWrap ( true ) ;
      mle_OPICM.setWrapStyleWord ( true ) ;
      mle_QUEST.setLineWrap ( true ) ;
      mle_QUEST.setWrapStyleWord ( true ) ;    
      mle_QUEST.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ) ) ;
      mle_QUEST_COUNTER = new JTextAreaCounter ( mle_QUEST );

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_MOVE.addActionListener ( this ) ;
      mle_QUEST.addFocusListener ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add controls                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_REQUIRED = GUISystem.getRequiredJLabel() ) ;
      add ( st_OPICM ) ;
      add ( st_QUEST ) ;
      add ( scr_OPICM ) ;
      add ( pb_MOVE ) ;
      add ( scr_QUEST ) ;
      add ( mle_QUEST_COUNTER ) ;
   }



   /**
     *  This method will lay out and position our controls
     **/
   public void doLayout () {
      Dimension size = getSize() ;
      int       halfWidth   = size.width/2 ;
      int       rowHeight   = GUISystem.getRowHeight () ;
      int       buttonWidth = 20 ;
      int       y           = 0 ;

      st_OPICM.setBounds ( 0, 0, halfWidth-buttonWidth, rowHeight ) ;
      st_QUEST.setBounds ( halfWidth+buttonWidth, 0, size.width-(halfWidth+buttonWidth), rowHeight ) ;
      y+=rowHeight ;

      scr_OPICM.setBounds ( 0, y, halfWidth-buttonWidth, size.height-y ) ;
      scr_QUEST.setBounds ( halfWidth+buttonWidth, y, size.width-(halfWidth+buttonWidth), size.height-y-40 ) ;
      mle_QUEST_COUNTER.setBounds ( halfWidth+buttonWidth, size.height-39, size.width-(halfWidth+buttonWidth), 15 ) ;
      st_REQUIRED.setBounds ( halfWidth+buttonWidth, size.height-15, size.width-(halfWidth+buttonWidth), 15 ) ;


      pb_MOVE.setBounds ( halfWidth-buttonWidth, rowHeight+30, buttonWidth*2, 25 ) ;

   }



   /**
     *  This method will be called when the user hits the move button
     **/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == pb_MOVE ) {
         mle_QUEST.setText ( mle_OPICM.getText() ) ;
      }
   }


   /**
     *  This method will set the product that we are working on
     **/
   public void setProduct ( Product product ) {
      if ( this.product != product ) {
         if ( this.product != null )
            this.product.removeProductPropertyListener ( this ) ;

         this.product = product ;
         product.addProductPropertyListener ( this ) ;
      }

      refresh () ;
   }


   /**
     *  This method will refresh our data
     **/
   private void refresh () {
      if ( product instanceof ProductDraft ) {
         String opicmDescript = ((ProductDraft)product).getOPICMDescript() ;

         if ( opicmDescript != null ) {
            mle_OPICM.setText ( opicmDescript ) ;
            pb_MOVE.setEnabled ( true ) ;
         } else {
            mle_OPICM.setText ( "" ) ;
            pb_MOVE.setEnabled ( false ) ;
         }

      } else {
         mle_OPICM.setText ( "" ) ;
      }

      if ( product.getDescription() != null )
         mle_QUEST.setText ( product.getDescription() ) ;
      else
         mle_QUEST.setText ( "" ) ;


   }


   /**
     *  this method will be called when a property on the product changes
     **/
   public void productChanged ( Product product, int property ) {
      if ( property == Product.DESCRIPTION )
         refresh() ;
   }

   /**
     *  This method will be called when focus is gained. We will do nothing
     **/
   public void focusGained ( FocusEvent e ) {
   }

   /**
     *  This method will be called when focus is lost. We will save the description
     * of the Product now
     **/
   public void focusLost ( FocusEvent e ) {
      product.setDescription ( mle_QUEST.getText().trim() ) ;
   }


   /**
     *  This method will either enable/disable all GUI on this control
     **/
   protected void setEditable ( boolean editable ) {
      mle_QUEST.setEditable ( editable ) ;

      if ( !editable )
         pb_MOVE.setEnabled ( editable ) ;
      else {
         pb_MOVE.setEnabled ( false ) ;
         if ( product instanceof ProductDraft ) {
            String opicmDescript = ((ProductDraft)product).getOPICMDescript() ;

            if ( opicmDescript != null ) {
               pb_MOVE.setEnabled ( true ) ;
            }
         }

      }
   }
} /* DescriptionPanel */
