package com.ibm.nzna.projects.qit.product.productEdit ;
/*-----------------------------------------------------------*/
/*   EditProduct                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: EditProduct                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.sql.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.* ;
import java.util.* ;
/**
 * This class holds static routines useful when wanting
 * to start the ProductEditPanel from anywhere in the Quest Input Tool
 *
  *
 * @version 1.0, 2/21/2001 
 * @author  David Lentz
 */


public class EditProduct implements AppConst, ProductConst {


   /**
     * This method will start the product editor editing the
     * ProductRow object passed. If the ProductRow object
     * passed holds a Product object (not a ProductDraft Object)
     * then a ProductDraft will be created and edited
     */
   public static void editProduct ( ProductRow productRow ) {
      int sqlCode = 0 ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, read in the product and edit it                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      LogSystem.log ( 1, "Starting Read of product for Edit. Product Row Draft:"+productRow.isDraft() ) ;
      sqlCode = productRow.readProduct ( true ) ;

      switch ( sqlCode ) {
         case 0 :
            if ( productRow.getProduct() instanceof ProductDraft )
               editProduct ( (ProductDraft)productRow.getProduct() ) ;
            else
               editProduct ( productRow.getProduct() ) ;
            break ;

         case 100 :
            GUISystem.printBox ( STR_INFO, STR_PRODUCT_NOT_FOUND ) ;
            break ;

         default:
            LogSystem.log ( 1, "Cannot Read Product For Editing. SQLCode returned from ProductRow.readProduct: "+sqlCode ) ;
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            break ;
      }

      LogSystem.log ( 1, "editProduct Method Complete" ) ;
   } /* editProduct  */


   /**
     * This method will start the product editor editing the
     * Product object passed in. A ProductDraft object will
     * be created.
     */
   public static void editProduct ( Product product ) {
      ProductDraft draft = createDraft ( product ) ;

      if ( draft != null ) {
         editProduct ( draft ) ;
      }
   }

   /**
     *  This method will create a draft. It will also tell the user
     * about 'Draft Already Exist', 'Recycled' etc.. It will return
     * a draft if the user wants to edit it
     *
     * @param product The product to create a draft for
     **/
   public static ProductDraft createDraft ( Product product ) {
      CreateProductDraft createDraft = ProductDraft.createProductDraft ( product ) ;
      ProductDraft       returnDraft = null ;

      switch ( createDraft.getReturnCode() ) {
         case CreateProductDraft.DRAFT_NONE :
            GUISystem.printBox ( STR_INFO, STR_ERROR_IN_LAST_ACTION ) ;
            break ;

         case CreateProductDraft.DRAFT_ALREADY_EXISTS :
            if ( GUISystem.printBox ( Str.getStr ( STR_QUESTION ), 
                                      UserSystem.getNameFromUserId ( createDraft.getProductDraft().getLastTouchedBy() )+" "+
                                      Str.getStr ( STR_EDIT_EXISTING_REQUEST ) ) )
               returnDraft = createDraft.getProductDraft() ;
            returnDraft.updateRecStatus ( ProductDraft.REC_NOCHANGE ) ;
            returnDraft.updateRecStatus ( ProductDraft.REC_UPDATE ) ;
            break ;

         case CreateProductDraft.DRAFT_RECYCLED :
            if ( GUISystem.printBox ( Str.getStr ( STR_QUESTION ),
                                      UserSystem.getNameFromUserId ( product.getLastTouchedBy() )+" "+
                                      Str.getStr ( STR_EDIT_RECYCLED_REQUEST )+
                                      createDraft.getProductDraft().getComment() ) ) {
               returnDraft = createDraft.getProductDraft() ;
            }
            break ;

         case CreateProductDraft.DRAFT_CREATED :
            returnDraft = createDraft.getProductDraft() ;
            break ;
      }

      return( returnDraft ) ;
   } /* editProduct  */


   /**
     * This method will start the product editor editing the
     * ProductDraft object passed in. If the ProductDraft
     * object has not been read in then it will be read now. The
     * way of determining if a ProductDraft object has been read
     * in is to check the title. If the title is NULL, the ProductDraft
     * object will be read in
     */
   public static void editProduct ( ProductDraft productDraft ) {
      QuestPanel panel = null ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok, if we are read in the draft, do so now              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( productDraft.getDescription() == null )
         productDraft.readFromDatabase() ;

      if ( !productDraft.getMisfit() ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Set our default values                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         setDefaultValues ( productDraft ) ;



         /*-----------------------------------------------*/
         /*                                               */
         /*  Find an editor already up with this product  */
         /*                                               */
         /*-----------------------------------------------*/
         panel = WindowSystem.findQuestPanel ( productDraft ) ;

         /*----------------------------------------------------------*/
         /*                                                          */
         /*  If we found a panel, just show it, otherwise create one */
         /*                                                          */
         /*----------------------------------------------------------*/
         if ( panel != null )
            WindowSystem.showPanel ( panel ) ;
         else
            WindowSystem.createPanel ( new ProductEditor ( productDraft ) ) ;
      } else
         GUISystem.printBox ( "Info", "You cannot edit a Misfit Product" ) ;
   } /* editProduct  */


   /**
     *  This method will set the default values for the product
     **/
   public static void setDefaultValues ( ProductDraft productDraft ) {


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign Start Date and Create Date                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( productDraft.getCreatedOn() == null )
         productDraft.setCreatedOn ( new CDate ( CDate.DATE_STAMP ).today() ) ;

      if ( productDraft.getCreatedBy() == null ) 
         productDraft.setCreatedBy ( UserSystem.getUserId() ) ;
      
      if ( productDraft.getDraftCreatedBy() == null ) 
         productDraft.setDraftCreatedBy ( UserSystem.getUserId() ) ;
      
      if ( productDraft.getLastTouchedBy() == null ) 
         productDraft.setLastTouchedBy ( UserSystem.getUserId() ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign the Requestor if it hasn't already been assigned */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( productDraft.getInput() == null )
         productDraft.setInput ( UserSystem.getUserId() ) ;
   }
}
