package com.ibm.nzna.projects.qit.product.productEdit ;                
/*-----------------------------------------------------------*/
/*   ProductEditPanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductEditPanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.gui.form.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
/**
 * This edit panel allows the user to alter any properties on a product 
 *
 *
 * @version 1.0, 8/24/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductEditPanel                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductEditPanel extends JPanel implements PropertyListener, ProductPropertyListener, FocusListener, Runnable, AppConst, ProductConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Threads                                                 */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private static final String THREAD_CREATE_FORM           = "CF" ;
   private static final String THREAD_FIRE_REFRESH_COMPLETE = "RFC" ;



   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   These are the forms                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/
   /**
     *  Product/Part Number static text
     **/
   public static final String FORM_PRODUCT_PARTNUM     = "" ;

   /**
     *  This is the description form
     **/
   public static final String FORM_DESCRIPTION        = Str.getStr ( STR_DESCRIPTION ) ;

   /**
     *  This is the product information form
     **/
   public static final String FORM_PRODUCT_INFORMATION = Str.getStr ( STR_PRODUCT_INFORMATION ) ;

   /**
     *  This is the Comment form
     **/
   public static final String FORM_COMMENT = Str.getStr ( STR_COMMENT ) ;

   /**
     *  This is the Comment HIstory form
     **/
   public static final String FORM_COMMENT_HISTORY = Str.getStr ( STR_COMMENT_HISTORY ) ;

   /**
     *  This is the form for the dates
     **/
   public static final String FORM_DATES = Str.getStr ( STR_DATES ) ;

   /**
     *  This is the form for the 'Other' Information
     **/
   public static final String FORM_OTHER_INFORMATION = Str.getStr ( STR_OTHER_INFORMATION ) ;

   /**
     *  This is the location 
     **/
   public static final String FORM_LOCATION = Str.getStr ( STR_LOCATION ) ;

   /**
     *  This is the XRef stuff
     **/
   public static final String FORM_XREF = Str.getStr ( STR_XREF ) ;

   /**
     *  This is the internal record ifnormation form
     **/
   public static final String FORM_DATABASE_INFORMATION = Str.getStr ( STR_DATABASE_INFO ) ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   GUI                                                     */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This is the Vector of ProductEditListeners
     **/
   private Vector listeners = new Vector() ;

   /**
     *  This is the Static Text component which holds the product/part number
     **/
   public JLabel st_PROD_PARTNUM = null ;   

   /**
     * This is the Locale panel
     **/
   public LocalePanel localePanel = null ;


   /**
     * This is the 'Other' Information panel
     **/
   public OtherInfoPanel otherInfoPanel = null ;


   /**
     *  This is the panel which holds all of our components in a list
     **/
   private ListPanel dataPanel = null ;

   /**
     *  This is the JScrollPane which holds the dataPanel
     **/
   private JScrollPane scrollPane = null ;


   /**
     *  This is the Descrition component which holds the OPIC/M and Quest Descriptions
     **/
   public DescriptionPanel description = null ;

   /**
     * This is the database information panel
     **/
   public DatabaseInfoPanel databaseInfoPanel = null ;

   /**
     *  This is the current comment that can be edited
     **/
   private NoTabJTextArea currentComment = null ;

   /**
     *  This is the current comment history that cannot be edited
     **/
   private NoTabJTextArea currentCommentHistory = null ;

   /**
     * This is the panel which holds all dates
     **/
   public DatePanel datePanel = null ;

   /**
     *  This is the panel which holds product information
     **/
   public ProductInfoPanel productInformation = null ;

   /**
     *  This is the comment history panel
     **/
   public MultiLineFormComponent commentHistory = null ;


   /**
     *  This is the Vectro which holds all of our GUI that we have focus listeners on
     **/
   private Vector focusGUI = new Vector ( 1 ) ;

   /**
     *  Governs if any field is editable in this form
     **/
   private boolean editable = true ;


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Program vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
     *  This is the product that we are editing
     **/
   private Product product = null ;

   /**
     *  The default constructor
     **/
   public ProductEditPanel ( ) {
      initialize() ;
   }

   /**
     *  This constructor must be passed a valid product record, already
     * read in
     **/
   public ProductEditPanel ( Product product ) {

      initialize() ;

      setProduct ( product ) ;
   }



   /**
     *  This method will set the product that is being displayed
     **/
   public void setProduct ( Product product ) {

      if ( this.product != null )
         this.product.removeProductPropertyListener ( this ) ;

      this.product = product ;

      if ( product != null ) {
         product.addProductPropertyListener ( this ) ;
         new Thread ( this, THREAD_CREATE_FORM ).start() ;
      }
   }


   /**
     *  This method will either enable/disable the editing of the fields
     * in this editor
     *
     * @param enable TRUE = allow editing, FALSE = do not allow editing
     **/
   public void setEditable ( boolean editable ) {

      this.editable = editable ;
      try {
         if ( localePanel != null )
            localePanel.setEditable             ( editable ) ;

         if ( otherInfoPanel != null )
            otherInfoPanel.setEditable          ( editable ) ;

         if ( description != null )
            description.setEditable             ( editable ) ;

         if ( currentComment != null )
            currentComment.setEditable          ( editable ) ;

         if ( datePanel != null )
            datePanel.setEditable               ( editable ) ;

         if ( productInformation != null )
            productInformation.setEditable      ( editable ) ;

         if ( databaseInfoPanel != null )
            databaseInfoPanel.setEditable ( editable ) ;


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, disable the fields if this is an OPIC/M Product    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( product instanceof ProductDraft ) {

            if ( ((ProductDraft)product).isOPICM() ) {
               localePanel.setEditable ( false ) ;
               datePanel.setEditable ( false ) ;
               productInformation.setEditable ( false ) ;
            }
         }
      } catch ( NullPointerException e ) {
      } catch ( Exception e1 ) {
         LogSystem.log ( 1, e1 ) ;
      }
   }



   /**
     *  This method will start the save process. It will return one of
     * the returnCodes from the ProductSaveDlg 
     **/
   public int saveProduct () {
      boolean saved = false ;
      boolean cont  = true ;                      


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Continue                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( cont ) {

         /*---------------------------------------------------------*/
         /*                                                         */
         /*  First, save all of our information that might not have */
         /* been saved yet.                                         */
         /*                                                         */
         /*---------------------------------------------------------*/
         ((ProductDraft)product).setComment ( currentComment.getText() ) ;


         /*------------------------------------------*/
         /*                                          */
         /*  Now, bring up the 'SaveProduct'  window */
         /*                                          */
         /*------------------------------------------*/
         SaveProductDlg saveProductDlg = new SaveProductDlg ( GUISystem.getParentDefWin ( this ), 
                                                              ((ProductDraft)product),
                                                              currentComment.getText() ) ;


         if ( saveProductDlg.getReturnCode() == SaveProductDlg.SAVE_NEW_PRODUCT )
            setProduct ( saveProductDlg.getProduct() ) ;
         return( saveProductDlg.getReturnCode() ) ;

      }

      return( 0 ) ;
   }



   /**
     *  This method will initialize all of our fields. 
     **/
   private void initialize () {

      PropertySystem.addPropertyListener ( PROPERTY_PRODUCT_EDITOR_LAYOUT, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Set up the default layout for the DocEditor             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( PropertySystem.get ( PROPERTY_PRODUCT_EDITOR_LAYOUT ) == null ) {
         Vector layout = new Vector () ;

         layout.addElement ( FORM_PRODUCT_PARTNUM ) ;
         layout.addElement ( FORM_DESCRIPTION ) ;
         layout.addElement ( FORM_PRODUCT_INFORMATION ) ;
         layout.addElement ( FORM_COMMENT ) ;
         layout.addElement ( FORM_COMMENT_HISTORY ) ;
         layout.addElement ( FORM_DATES ) ;
         layout.addElement ( FORM_OTHER_INFORMATION ) ;
         layout.addElement ( FORM_LOCATION ) ;
         //layout.addElement ( FORM_XREF ) ;
         layout.addElement ( FORM_DATABASE_INFORMATION ) ;

         PropertySystem.put ( PROPERTY_PRODUCT_EDITOR_LAYOUT, layout ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      dataPanel = new ListPanel () ;
      scrollPane = new JScrollPane ( dataPanel ) ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      dataPanel.setBackground  ( Color.white ) ;
      scrollPane.setBackground ( Color.white ) ;  
      scrollPane.setBorder     ( GUISystem.emptyBorder ) ;
      scrollPane.getVerticalScrollBar().setUnitIncrement ( GUISystem.getRowHeight() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( new BorderLayout () ) ;
      add ( scrollPane ) ;



   }



   /**
     *  This thread performs multiple operations. The operation that is 
     * preformed is chosen by the name of the thread
     **/
   public void run () {
      String operation = Thread.currentThread().getName() ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_CREATE_FORM                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_CREATE_FORM ) )
         addFormComponents() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_FIRE_REFRESH_COMPLETE                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( operation.equals ( THREAD_FIRE_REFRESH_COMPLETE ) ) {
         fireRefreshComplete() ;
      }


   }




   /**
     *  This method will clear all current form components and recreates
     * them
     **/
   private void addFormComponents () {
      Vector        layout    = (Vector)PropertySystem.get ( PROPERTY_PRODUCT_EDITOR_LAYOUT ) ;
      int           i         = 0 ;
      int           size      = layout.size() ;
      String        current   = null ;                                        
      Container     component = null ;               

      try {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, remove everything                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         removeAllFocusListeners() ;
         try {
            dataPanel.removeAll() ;
         } catch ( Exception e ) {
            //e.printStackTrace() ;
         }

         if ( dataPanel == null )
            dataPanel = new ListPanel () ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Loop through each item and add the form components      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( i < size ) {

            current = (String)layout.elementAt(i) ;         
            component = null ;                 


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   The Product/Part Number                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( current.equals ( FORM_PRODUCT_PARTNUM ) ) {
               String prodNum = "" ;

               if ( ( product != null ) && ( product.getBrand() != null ) ) {
                  if ( Brands.getOptionBrand().getInd() == product.getBrand().getInd() )
                     prodNum = product.getPartNumber().toString() ;
                  else
                     prodNum = product.getProdNum() ;
               }

               if ( prodNum != null )
                  component = st_PROD_PARTNUM = new JLabel ( prodNum ) ;
               else
                  component = st_PROD_PARTNUM = new JLabel ( "" ) ;

               st_PROD_PARTNUM.setBackground ( Color.white ) ;
               st_PROD_PARTNUM.setForeground ( new Color ( 0, 0, 150 ) ) ;
               st_PROD_PARTNUM.setFont ( FontSystem.largeTitleFont ) ;
               dataPanel.add ( st_PROD_PARTNUM ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*    The Description Form Component                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( current.equals ( FORM_DESCRIPTION ) ) {
               description = new DescriptionPanel () ;
               component = new FormComponent ( Str.getStr ( STR_DESCRIPTION ),
                                               description ) ;
               description.setProduct ( product ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Comment                                                 */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( current.equals ( FORM_COMMENT ) ) {
               if ( ( product != null ) && ( product instanceof ProductDraft ) ) {
                  JPanel           holdPanel          = new JPanel ( ) ;
                  JPanel           scrollHoldPanel    = new JPanel ( ) ;
                  int              commentSize        = 1024 ;
                  String           originalComments   = null ;
                  JTextAreaCounter commentCounter     = null ;

                  try {
                     originalComments = ((ProductDraft)product).getOriginalComments() ;
                  } catch ( Exception e ) {
                     originalComments= "" ;
                  }

                  if ( originalComments != null )
                     commentSize-=originalComments.length() ;
                  currentCommentHistory = new NoTabJTextArea() ;
                  currentComment        = new NoTabJTextArea() ;

                  currentComment.setFont ( FontSystem.defaultFont ) ;
                  currentComment.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, commentSize ) ) ;
                  currentCommentHistory.setFont ( FontSystem.defaultFont ) ;
                  currentCommentHistory.setBackground ( new Color ( 240, 240, 240 ) );
                  currentCommentHistory.setEditable ( false ) ;
                  currentComment.setEditable ( editable ) ;
                  commentCounter = new JTextAreaCounter ( currentComment ) ;

                  holdPanel.setLayout ( new BorderLayout() ) ;
                  scrollHoldPanel.setLayout ( new BorderLayout() ) ;
                  scrollHoldPanel.setBackground ( Color.white ) ;
                  holdPanel.add ( currentCommentHistory, BorderLayout.NORTH  ) ;
                  holdPanel.add ( currentComment, BorderLayout.CENTER ) ;

                  scrollHoldPanel.add ( new JScrollPane ( holdPanel ) ) ;
                  scrollHoldPanel.add ( commentCounter, BorderLayout.SOUTH ) ;

                  component = new FormComponent ( Str.getStr ( STR_COMMENT ), scrollHoldPanel ) ;
                  ((FormComponent)component).setPreferredSize ( new Dimension ( 470, 
                                                                                (GUISystem.getRowHeight()*5)+22 ) ) ;

                  if ( ((ProductDraft)product).getComment() != null )
                     currentComment.setText ( ((ProductDraft)product).getComment().toString() ) ;
                  if ( originalComments != null )
                     currentCommentHistory.setText ( originalComments ) ;
               }
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Date Panel                                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( current.equals ( FORM_DATES ) ) {
               component = new FormComponent ( Str.getStr ( STR_DATES ),
                                               datePanel = new DatePanel() ) ;
               datePanel.setProduct ( product ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Product Information                                     */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( current.equals ( FORM_PRODUCT_INFORMATION ) ) {
               productInformation = new ProductInfoPanel () ;
               component = new FormComponent ( Str.getStr ( STR_PRODUCT_INFORMATION ),
                                               productInformation ) ;

               if ( product != null )
                  productInformation.setProduct ( product ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Locale                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( current.equals ( FORM_LOCATION ) ) {
               localePanel = new LocalePanel () ;
               component = new FormComponent ( Str.getStr ( STR_LOCATION ),
                                               localePanel ) ;

               localePanel.setProduct ( product ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Other Info                                              */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( current.equals ( FORM_OTHER_INFORMATION ) ) {
               component = new FormComponent ( Str.getStr ( STR_OTHER_INFORMATION ),
                                               otherInfoPanel = new OtherInfoPanel () ) ;
               otherInfoPanel.setProduct ( product ) ;

            }


            /*-----------------------*/
            /*                       */
            /*  Database Information */
            /*                       */
            /*-----------------------*/
            else if ( current.equals ( FORM_DATABASE_INFORMATION ) ) {
               component = new FormComponent ( Str.getStr ( STR_DATABASE_INFO ),
                                               databaseInfoPanel = new DatabaseInfoPanel () ) ;
               databaseInfoPanel.setProduct ( product ) ;
            }


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Comment History                                         */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( ( current.equals ( FORM_COMMENT_HISTORY ) ) && ( product instanceof ProductDraft ) ) {
               component = commentHistory = new MultiLineFormComponent ( FORM_COMMENT_HISTORY ) ;
               commentHistory.setText ( ((ProductDraft)product).getCommentHistoryAsText() ) ;
               commentHistory.getJTextArea().setEditable ( false ) ;
               ((MultiLineFormComponent)component).setPreferredSize ( new Dimension ( 470, 
                                                                                      (GUISystem.getRowHeight()*5)+22 ) ) ;

            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   XRef                                                    */
            /*                                                           */
            /*-----------------------------------------------------------*/
            //else if ( current.equals ( FORM_XREF ) ) {
            //   component = new FormComponent ( Str.getStr ( STR_XREF ),
            //                                   new JLabel ( "" ) ) ;
            //}



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Add the Component                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( component != null ) {

               if ( ( component instanceof FormComponent ) &&
                    ( component != st_PROD_PARTNUM ) )
                  GUISystem.setPropertiesOnFormComponent ( (FormComponent)component ) ;

               dataPanel.add ( component ) ;

               addComponentFocus ( component ) ;
            } else
               LogSystem.log ( 1, "Cannot add FormComponent:"+current ) ;

            i++ ;
         }

         setEditable ( editable ) ;
         revalidate() ;

         new Thread ( this, THREAD_FIRE_REFRESH_COMPLETE ).start() ;

      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

   }



   /**
     *  This method will go through all children of the component
     * passed and will add us as a FocusListener. This way, if a control
     * that is not visible in our JScrollPane receives focus, we can scroll
     * to it.
     **/
   private void addComponentFocus ( Container component ) {
      Component[] compArray = component.getComponents() ;

      if ( compArray != null ) {

         for ( int i = 0; i < compArray.length; i++ ) {
            if ( ( compArray[i] instanceof JCheckBox ) ||
                 ( compArray[i] instanceof JRadioButton ) ||
                 ( compArray[i] instanceof JList ) ||
                 ( compArray[i] instanceof JTextArea ) ||
                 ( compArray[i] instanceof JTextField ) ||
                 ( compArray[i] instanceof JButton ) ) {
               compArray[i].addFocusListener ( this ) ;
               focusGUI.addElement ( compArray[i] ) ;
            }

            else if ( compArray[i] instanceof Container )
               addComponentFocus ( (Container)compArray[i] ) ;

         }
      }

   }


   /**
     *  This method will be called when a component we have has gained focus
     **/
   public void focusGained ( FocusEvent e ) {
      boolean   c         = true ;
      Component component = (Component)e.getSource() ;
      Rectangle bounds    = component.getBounds() ;
      Rectangle viewRect  = scrollPane.getViewport().getViewRect() ;                                    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, we have to figure out what position this           */
      /* control is in relative to the Data Panel                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( c ) {
         Component parent = component.getParent() ;
         Rectangle parentBounds = parent.getBounds() ;

         c = ( parent != dataPanel ) ;

         if ( c ) {
            bounds.y+=parentBounds.y ;
            component = parent ;
         }
      }

      if ( !viewRect.contains ( bounds.x, bounds.y ) )
         scrollPane.getViewport().setViewPosition ( new Point ( bounds.x, bounds.y ) ) ;


   }

   /**
     *  This method will be notified when any field we are displaying loses focus. We will
     * use this time to save data that appears in non-standard form components.
     **/
   public void focusLost ( FocusEvent e ) {
   }



   /**
     *  This method will be called when any property on the product has changed. We will use
     * this to update any non-standard form components, like the Product or Part Number
     **/
   public void productChanged ( Product product, int property ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Our product number                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( property == Product.MODEL ) ||
           ( property == Product.MACHINE ) ||
           ( property == Product.PARTNUMBER ) ) {
         String prodNum = product.getProdNum() ;

         if ( prodNum == null ) {
            if ( product.getPartNumber() != null )
               prodNum = product.getPartNumber().toString() ;
            else
               prodNum = "" ;
         }

         st_PROD_PARTNUM.setText ( prodNum ) ;
      }
   }


   /**
     *  This method will remove us as a FocusListener from everything.
     **/
   private void removeAllFocusListeners () {
      int i    = 0 ;
      int size = focusGUI.size() ;

      while ( i < size ) {
         ((Component)focusGUI.elementAt(i)).removeFocusListener ( this ) ;
         i++ ;
      }

      focusGUI.removeAllElements() ;
   }



   /**
     *  This method will return the actual DPanel which holds all Form
     * compoents for this editor
     **/
   public DPanel getFormPanel ( ) {
      return( dataPanel );
   }

   public void propertyChanged ( int property, Object value ) {
      addFormComponents() ;
   }


   /**
     *  This method must be called whenever this panel is no longer used
     **/
   public void free () {
      PropertySystem.removePropertyListener ( PROPERTY_PRODUCT_EDITOR_LAYOUT, this ) ;
      product.removeProductPropertyListener ( this ) ;
   }

   public void addProductEditPanelListener ( ProductEditPanelListener listener ) {
      listeners.addElement ( listener ) ;
   }

   public void removeProductEditPanelListener ( ProductEditPanelListener listener ) {
      listeners.removeElement ( listener ) ;
   }


   /**
     *  This method will fire off an event to all registered listeners
     **/
   public void fireRefreshComplete () {
      if ( listeners.size() > 0 ) {
         int i    = 0;
         int size = listeners.size() ;

         while ( i < size ) {
            ((ProductEditPanelListener)listeners.elementAt(i)).refreshComplete() ;
            i++ ;
         }
      }
   }

   public JPanel getContentPanel () {
      return( dataPanel );
   }


   public Product getProduct () {
      return ( product ) ;
   }
} /* ProductEditPanel */
