package com.ibm.nzna.projects.qit.product.productEdit ;     
/*-----------------------------------------------------------*/
/*   ProductEditor                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductEditor                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.compare.* ;
import com.ibm.nzna.projects.qit.product.history.* ;
import com.ibm.nzna.projects.qit.product.productEdit.property.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This is the panel which allows the user to save a product  
 * to the Quest Database. This panel simply holds a ProductEditPanel
 * to allow the manipulation of Data and also holds the Save, Close
 * and toolbar buttons
 *
 *
 * @version 1.0, 8/24/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductEditor                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductEditor extends JPanel implements PreviewPanel, PrintPanel, ActionListener, ProductPropertyListener, QuestPanel, AppConst, ProductConst {




   /**
     *  This is the draft that we are currently editing
     **/
   private ProductDraft draft = null ;


   /**
     *  This is the ProductEditPanel that we will use
     **/
   private ProductEditPanel prodEditPanel = null ;

   /**
     *  TRUE if we have set our focus button text
     **/
   private boolean focusButtonSet = false ;

   /**
     *  This is the property toolbar button
     **/
   private   ToolBarButton   pb_PROPERTY = null ;

   /**
     *  This is the property toolbar button
     **/
   private   ToolBarButton   pb_COMPARE = null ;

   /**
     *  This is the history toolbar button
     **/
   private   ToolBarButton   pb_HISTORY = null ;

   /**
     *  This is the toolbar that our panel uses
     **/
   private   JToolBar        toolBar              = null ;

   /**
     *  This is the Save Button
     **/
   private ActionButton pb_SAVE = new ActionButton ( Str.getStr ( STR_SAVE ), ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.SAVE ), Str.getStr ( STR_SAVE )) ;

   /**
     *  This is the close Button
     **/
   private ActionButton pb_CLOSE = new ActionButton ( Str.getStr ( STR_CLOSE ), ImageSystem.getImageIcon ( MainWindow.getInstance(), ImageSystem.CLOSE ), Str.getStr ( STR_CLOSE )) ;

   /**
     *  This is our parent
     **/
   private AppDefaultWin parentWin  = null ;

   /**
     *  If the product we are editing has changed
     **/
   private boolean productChanged = false ;



   /**
     *  Default constructor
     **/
   public ProductEditor ( ProductDraft draft ) {
      this.draft = draft ;
      draft.addProductPropertyListener ( this ) ;
      productChanged ( draft, 0 ) ;
   }

   /*------------------------------------------------------------*/
   /* getIcon
    */
   /**
    *    This method should return an ImageIcon
    *  representing this panel. This icon will
    *  be used as a Frame icon, or Tab icon
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ImageIcon  getIcon() {
      return( ImageSystem.getImageIcon ( this, ImageSystem.PRODUCT_SMALL ) ) ;
   }





   /**
     *  This method will return our title
     **/
   public String getName () {
      return( Str.getStr ( STR_EDIT  ) ) ;
   }





   /*------------------------------------------------------------*/
   /* getDefaultNavPanel
    */
   /**
    *    This method should return an instance
    *  of the default navigation panel for this
    *  panel. NULLs are allowed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent - An instance of the Navigation panel
    *//*---------------------------------------------------------*/
   public NavPanel getDefaultNavPanel() {
      return( null ) ;
   }




   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method should initialize the panel
    *  but not show anything. This method is
    *  called before 'start', so any prep work should be done here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE - Continue initialization process, FALSE - Stop initialization and kill instance
    *//*---------------------------------------------------------*/
   public boolean    initialize() {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      prodEditPanel = new ProductEditPanel () ;      
      pb_PROPERTY   = new ToolBarButton ( Str.getStr ( STR_PROPERTY ), ImageSystem.getImageIcon ( this, ImageSystem.PROPERTIES ),             Str.getStr ( STR_PROPERTY ) ) ;
      pb_COMPARE    = new ToolBarButton ( Str.getStr ( STR_COMPARE ),  ImageSystem.getImageIcon ( this, ImageSystem.PRODUCT_COMPARE ),        Str.getStr ( STR_COMPARE ) ) ;
      pb_HISTORY    = new ToolBarButton ( Str.getStr ( STR_HISTORY ),  ImageSystem.getImageIcon ( this, ImageSystem.DOCUMENT_HISTORY_SMALL ), Str.getStr ( STR_HISTORY ) ) ;
      toolBar       = new JToolBar() ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.setBorder ( GUISystem.emptyBorder ) ;
      toolBar.setOpaque ( false ) ; 
      setBackground ( Color.white ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our Toolbar                                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      toolBar.add ( pb_PROPERTY ) ;
      toolBar.add ( pb_COMPARE ) ;
      toolBar.add ( pb_HISTORY ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_SAVE.addActionListener     ( this ) ;
      pb_CLOSE.addActionListener    ( this ) ;
      pb_PROPERTY.addActionListener ( this ) ;
      pb_COMPARE.addActionListener  ( this ) ;
      pb_HISTORY.addActionListener  ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( prodEditPanel ) ;

      return( true ) ;
   }



   public void doLayout () {
      Dimension size = getSize() ;

      prodEditPanel.setBounds ( 5, 5, size.width-6, size.height-6 ) ;
   }


   /*------------------------------------------------------------*/
   /* start
    */
   /**
    *    This method will be called when the
    *  PanelSystem has just shown the panel
    *  Any focus setting or data refreshes should be done here
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void start() {

      parentWin = GUISystem.getParentDefWin ( this ) ;
      prodEditPanel.setProduct ( draft ) ;

      parentWin.addActionComponent ( this, pb_CLOSE ) ;
      parentWin.addActionComponent ( this, pb_SAVE ) ;
      panelSelected() ;



      if ( productCheckedOut() ) {
         pb_SAVE.setVisible ( false ) ;
         GUISystem.printBox ( "Info", "The product is currently locked by "+UserSystem.getNameFromUserId ( draft.getLastTouchedBy() ) ) ;
      }
   }




   /*------------------------------------------------------------*/
   /* stop
    */
   /**
    *    This method will be called to let the
    *  panel know that the PanelSystem needs
    *  to close it. This method should return FALSE if it wishes not to close
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE - Go ahead and close the panel, FALSE - Do not close the panel (I'm not ready to die)
    *//*---------------------------------------------------------*/
   public boolean    stop() {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   If we should save, ask the user                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( !productCheckedOut() ) {
         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the document has changed, offer to save              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( productChanged ) {
            if ( GUISystem.printBox ( "Question", "Do you wish to save your product?" ) )
               return( saveProduct() ) ;
         }

         try {
            draft.checkOut ( null ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
         }


      }


      panelDeselected() ;
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* shutdown
    */
   /**
    *    This method will be called on every
    *  open panel upon the user shutting down
    *  the application.
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Go ahead and call 'stop', I am ready to close. FALSE = Stop shutdown process and do not close
    *//*---------------------------------------------------------*/
   public boolean    shutdown() {
      return( true ) ;
   }




   /*------------------------------------------------------------*/
   /* enable
    */
   /**
    *    This method will be called by the PanelSystem
    *  to let a panel know if it should accept
    *  user input or disregard it. Be careful with this method, if
    *  you call 'setEnabled' on all of your controls in the panel, there
    *  will be a second to two second pause while repainting everything. A
    *  good programmer should mark this as a flag, and when any user interaction
    *  has happened, check that flag to determine if you should allow
    *  user interaction.
    *
    *  @param enable TRUE = allow user input, FALSE = do not allow
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void enable ( boolean enable ) {
   }





   /*------------------------------------------------------------*/
   /* getJComponent
    */
   /**
    *    This method should return the JComponent
    *  representing this QuestPanel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return JComponent
    *//*---------------------------------------------------------*/
   public JComponent getJComponent () {
      return( this ) ;
   }





   /*------------------------------------------------------------*/
   /* cutText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Cut As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean cutText () {
      return( false ) ;
   }


   /*------------------------------------------------------------*/
   /* pasteText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Paste As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean pasteText () {
      return( false ) ;
   }

   /*------------------------------------------------------------*/
   /* copyText
    */
   /**
    *    This method will be called when the user
    *  wishes to 'Copy As Text' from this panel
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Action Performed, FALSE = Action not allowed
    *//*---------------------------------------------------------*/
   public boolean copyText () {
      return( false ) ;
   }





   /*------------------------------------------------------------*/
   /* refresh
    */
   /**
    *    This method will be called when the user
    *  wishes to refresh the QuestPanel. DO NOT EVER CALL revalidate
    *  in this method, the default operation for this button is to
    *  have the AppDefaultWin holding the QuestPanel to repaint everything
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void refresh () {
      GUISystem.printBox ( "Info", "Refresh not supported here" ) ;
   }



   /*------------------------------------------------------------*/
   /* panelSelected
    */
   /**
    *    This method will be called when the
    *  user has selected our panel from a
    *  tabbed pane. If we are in MDI mode, this method is never called
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelSelected () {
      GUISystem.getParentDefWin ( this ).addToolbarButton ( toolBar, 2 ) ;
   }





   /*------------------------------------------------------------*/
   /* panelDeselected
    */
   /**
    *    This method will be called when the user
    *  clicks off of our panel onto another. This method
    *  will never be called in the MDI mode
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void panelDeselected () {
      GUISystem.getParentDefWin ( this ).removeToolbarButton ( toolBar ) ;
   }




   /*------------------------------------------------------------*/
   /* bookmark
    */
   /**
    *    This method will be called if the user
    *  has hit the 'Bookmark' button. 
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return FALSE if Bookmarking is supported, TRUE if bookmarked
    *//*---------------------------------------------------------*/
   public boolean bookmark  () {
      return( false ) ;
   }



   /**
     * This method will be called when the product we are editing has changed a property,
     * we will use this time to format the focus button text for our panel. This is done
     * by first taking the Product Number/Part NUmber. If that is null, then a
     * product indicator or existing product indicator is used. If both are zero, then
     * a substring of the description is taken. We also make a note that something changed so when
     * we go to close we can ask the user if they wish to save their changes.
     **/
   public void productChanged ( Product product, int propertyInd  ) {
      productChanged = true ;

      try {
         if ( !focusButtonSet ) {
            String prodNum    = null ;
            String descript   = product.getDescription() ;
            int    productInd = product.getProductInd () ;

            if ( product.getProductType() == TypeList.PRODUCTTYPE_OPTION ) {
               if ( product.getPartNumber() != null )
                  prodNum = product.getPartNumber().toString() ;
            } else
               prodNum = product.getProdNum() ;

            if ( ( prodNum != null ) && ( prodNum.length() > 0 ) ) {
               parentWin.setFocusButtonText ( this, prodNum ) ;
            }

            else if ( productInd > 0 ) {
               parentWin.setFocusButtonText ( this, ""+productInd ) ;
            }

            else if ( ( descript != null ) && ( descript.length() > 10 ) ) {
               parentWin.setFocusButtonText ( this, descript.substring ( 0, 10 )+"..." ) ;
            }

         }
      } catch ( Exception e ) {
      }
   }


   /**
     * This method will return TRUE if we are editing the product passed.
     *
     * @param Object The Object to compare to.
     **/
   public boolean equals ( Object object ) {
      if ( object instanceof ProductDraft ) {
         ProductDraft compareDraft = (ProductDraft) object ;

         if ( compareDraft.getExistingProductInd() == 0 )
            return( false ) ;
         else {
            if ( ( compareDraft.getExistingProductInd() == 0 ) &&
                 ( draft.getExistingProductInd() == 0 ) )
               return( false ) ;
            else
               return( compareDraft.getExistingProductInd() == draft.getExistingProductInd() ) ;
         }
      }

      return( object == this ) ;
   }



   /**
     *  This method will be called when action events are fired on us
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_CLOSE                                                */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CLOSE )
         parentWin.closePanel ( this, null ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_SAVE                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE ) {
         saveProduct() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Launch the property window                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_PROPERTY )
         new PropertyDlg ( parentWin ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_COMPARE                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_COMPARE )
         WindowSystem.createPanel ( new ProductComparePanel ( draft, this ) ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_HISTORY                                              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_HISTORY )
         WindowSystem.createPanel ( new ProductHistoryPanel ( draft.getExistingProductInd(), false, this ) ) ;


   }


   /**
     *  This method will make attempt to save the document
     **/
   private boolean saveProduct () {

      switch ( prodEditPanel.saveProduct() ) {
         case SaveProductDlg.SAVE_PUBLISHED :
         case SaveProductDlg.SAVE_CANCEL_DRAFT :
         case SaveProductDlg.SAVE_TEMPLATE :
            productChanged = false ;
            parentWin.closePanel ( this, null ) ;
            return( true ) ;

         case SaveProductDlg.SAVE_SAVED :
            productChanged = false ;
            return( true ) ;

         case SaveProductDlg.SAVE_NEW_PRODUCT :
            productChanged = false ;
            focusButtonSet = false ;
            this.draft.removeProductPropertyListener ( this ) ;
            this.draft = (ProductDraft)prodEditPanel.getProduct() ;
            draft.addProductPropertyListener ( this ) ;
            productChanged ( draft, 0 ) ;

            return( true ) ;
      }

      return( false ) ;
   }


   /**
     *  This method will return TRUE if the product being edited is locked
     **/
   private boolean productCheckedOut() {
      String checkedOutBy = draft.getCheckedOutBy () ;

      if ( checkedOutBy == null )
         return( false );

      return( !checkedOutBy.equals (UserSystem.getUserId() ) ) ;
   }

   /**
     *  This method will give the printing support to the panel
     **/
   public Object getPrintObject() {
      return( new Print ( prodEditPanel.getFormPanel(), GUISystem.getParentDefWin(this), Str.getStr ( STR_PRODUCTS ) ) ) ;
   }
   /**
     *  This method will give the preview support to the panel
     **/
   public Object getPreviewObject() {
      return( new Preview ( prodEditPanel.getFormPanel(), GUISystem.getParentDefWin(this), Str.getStr ( STR_PRODUCTS ) ) ) ;
   }
} /* ProductEditor */
