package com.ibm.nzna.projects.qit.product.productEdit ;           
/*-----------------------------------------------------------*/
/*   ProductInfoPanel                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductInfoPanel                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.product.* ;

/**
 * This panel holds the Brand, Family, Project, SubCategory, Machine and Model 
 * also the part number
 *
 * @version 1.0, 8/26/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductInfoPanel                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductInfoPanel extends DPanel implements FocusListener, Runnable, DataLengths, ProductPropertyListener, ActionListener, AppConst, ProductConst {

   /**
     * Refreshes the entire control
     **/
   private static final String THREAD_REFRESH = "REFRESH" ;

   private static final String THREAD_SAVE_MODEL = "saveModel" ;

   /**
     *  This property tells the actionPerformed method if it should
     * process any action
     **/
   private boolean processActions = false ;

   /**
     *  This is the Brand drop down
     **/
   private JComboBox cb_BRAND = null ;

   /**
     *  This is the Family Drop down
     **/
   private JComboBox cb_FAMILY = null ;

   /**
     *  This is teh machine/sub category drop down
     **/
   private JComboBox cb_MACHINE = null ;

   /**
     *  This is the Machine drop down
     **/
   private JTextField ef_MODEL = null ;

   /**
     *  This is the Project drop down
     **/
   private JComboBox cb_PROJECT = null ;

   /**
     *  This is the JLabel for brand
     **/
   public JLabel st_BRAND = null ;

   /**
     *  This is te JLabel for the family
     **/
   public JLabel st_FAMILY = null ;

   /**
     *  This is the JLabel for Sub Category/Machine
     **/
   public JLabel st_MACHINE = null ;

   /**
     *  This is the JLabel for Part Number/Model
     **/
   public JLabel st_MODEL = null ;

   /**
     *  This is the JLabel for Project Name
     **/
   public JLabel st_PROJECT = null ;

   /**
     *  This is the new button for Projects
     **/
   private HotLinkLabel pb_NEW_PROJECT = new HotLinkLabel ( Str.getStr ( STR_NEW ) ) ;

   /**
     *  This is the new button for Machines
     **/
   private HotLinkLabel pb_NEW_MACHINE = new HotLinkLabel ( Str.getStr ( STR_NEW ) ) ;

   /**
     *  This is the product we are editing
     **/
   private Product product =  null ;

   /**
     *  This property determines if this panel is editable
     **/
   private boolean enableInput = true  ;


   /**
     *  This property identifies the product being edited as either an Option or Other
     **/
   private boolean isOption =  false ; 

   /**
     *  Default Constructor
     **/
   public ProductInfoPanel () {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Craete the GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_BRAND   = new JComboBox () ;
      cb_FAMILY  = new JComboBox () ;
      cb_PROJECT = new JComboBox () ;
      ef_MODEL   = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 3 ), "", 0 ) ;
      cb_MACHINE = new JComboBox () ;
      st_BRAND   = new JLabel ( "*"+Str.getStr ( STR_BRAND ) ) ;
      st_FAMILY  = new JLabel ( "*"+Str.getStr ( STR_FAMILY ) ) ;
      st_PROJECT = new JLabel ( "*"+Str.getStr ( STR_PROJECT ) ) ;
      st_MODEL   = new JLabel ( "*"+Str.getStr ( STR_MODEL ) ) ;
      st_MACHINE = new JLabel ( "*"+Str.getStr ( STR_MACHINE ) ) ;        


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setPreferredSize ( new Dimension ( 550, 90 ) ) ;
      setBackground ( Color.white ) ;
      pb_NEW_PROJECT.showSelection ( false ) ;
      pb_NEW_MACHINE.showSelection ( false ) ;
      cb_BRAND.setModel ( new DefaultComboBoxModel() ) ;
      cb_FAMILY.setModel ( new DefaultComboBoxModel() ) ;
      cb_PROJECT.setModel ( new DefaultComboBoxModel() ) ;
      cb_MACHINE.setModel ( new DefaultComboBoxModel() ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add Listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      cb_BRAND.addActionListener       ( this ) ;  
      pb_NEW_PROJECT.addActionListener ( this ) ;
      pb_NEW_MACHINE.addActionListener ( this ) ;
      cb_MACHINE.addActionListener     ( this ) ;
      cb_FAMILY.addActionListener      ( this ) ;
      cb_PROJECT.addActionListener     ( this ) ;
      ef_MODEL.addFocusListener        ( this ) ;
      ef_MODEL.addActionListener       ( this ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our gui                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_BRAND ) ;
      add ( cb_BRAND ) ;
      add ( st_FAMILY ) ;
      add ( cb_FAMILY ) ;
      add ( st_PROJECT ) ;
      add ( cb_PROJECT ) ;
      add ( pb_NEW_PROJECT ) ;
      add ( st_MACHINE ) ;
      add ( cb_MACHINE ) ;
      add ( pb_NEW_MACHINE ) ;
      add ( st_MODEL ) ;
      add ( ef_MODEL ) ;

      enableInput() ;
   }


   /**
     *  This method will set the product we are editing
     **/
   public void setProduct ( Product product ) {
      if ( this.product != product ) {
         if ( this.product != null )
            this.product.removeProductPropertyListener ( this ) ;

         this.product = product ;
         product.addProductPropertyListener ( this ) ;
      }

      isOption = ( product.getProductType() == TypeList.PRODUCTTYPE_OPTION ) ;
      new Thread ( this, THREAD_REFRESH ).start() ;
   }

   /**
     *  This method will layout and position our componets
     **/
   public void doLayout ( ) {
      Dimension size      = getSize() ;
      int       x         = 0 ;
      int       y         = 0 ;
      int       rowHeight = GUISystem.getRowHeight () ;

      try {
         st_BRAND.setBounds ( x, y, 75, rowHeight ) ;
         cb_BRAND.setBounds ( x+75, y, 150, rowHeight ) ;
         y+=rowHeight ;

         st_FAMILY.setBounds ( x, y, 75, rowHeight ) ;
         cb_FAMILY.setBounds ( x+75, y, 170, rowHeight ) ;
         y+=rowHeight ;

         st_PROJECT.setBounds ( x, y, 75, rowHeight ) ;
         cb_PROJECT.setBounds ( x+75, y, 250, rowHeight ) ;
         pb_NEW_PROJECT.setBounds ( x+75+255, y, 70, rowHeight ) ;
         y+=rowHeight ;

         st_MACHINE.setBounds ( x, y, 75, rowHeight ) ;
         cb_MACHINE.setBounds ( x+75, y, 150, rowHeight ) ;
         pb_NEW_MACHINE.setBounds ( x+75+155, y, 70, rowHeight ) ;
         y+=rowHeight ;

         st_MODEL.setBounds ( x, y, 75, rowHeight ) ;
         ef_MODEL.setBounds ( x+75, y, 75, rowHeight ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }

   }



   /**
     * This method will be called when action events are fired on us
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Only process the action if we are supposed to           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( processActions ) {

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   cb_BRAND                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( e.getSource() == cb_BRAND ) && ( comboBoxHasValidData ( cb_BRAND ) ) ) {
            System.out.println ( "Set Brand to :"+(TypeCategoryRec)cb_BRAND.getSelectedItem() ) ;
            product.setBrand ( (TypeCategoryRec)cb_BRAND.getSelectedItem() ) ;
            product.setFamily      ( null ) ;
            product.setMachine     ( null ) ;
            product.setSubCategory ( null ) ;
            product.setProject     ( null ) ;
            product.setModel       ( null ) ;
            new Thread ( this, THREAD_REFRESH ).start() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   cb_FAMILY                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( e.getSource() == cb_FAMILY ) && ( comboBoxHasValidData ( cb_FAMILY ) ) ) {
            product.setFamily ( (TypeCategoryRec)cb_FAMILY.getSelectedItem() ) ;
            product.setMachine     ( null ) ;
            product.setSubCategory ( null ) ;
            product.setProject     ( null ) ;
            product.setModel       ( null ) ;
            new Thread ( this, THREAD_REFRESH ).start() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   cb_PROJECT                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( e.getSource() == cb_PROJECT ) && ( comboBoxHasValidData ( cb_PROJECT ) ) ) {
            product.setProject ( (TypeCategoryRec)cb_PROJECT.getSelectedItem() ) ;
            product.setMachine     ( null ) ;
            product.setSubCategory ( null ) ;
            product.setModel       ( null ) ;
            new Thread ( this, THREAD_REFRESH ).start() ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   cb_MACHINE                                              */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( e.getSource() == cb_MACHINE ) && ( comboBoxHasValidData ( cb_MACHINE ) ) ) {
            if ( isOption )
               product.setSubCategory ( (TypeCategoryRec)cb_MACHINE.getSelectedItem() ) ;
            else
               product.setMachine ( (TypeCategoryRec)cb_MACHINE.getSelectedItem() ) ;
            product.setModel       ( null ) ;
            new Thread ( this, THREAD_REFRESH ).start() ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   ef_MODEL                                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == ef_MODEL )
            saveModelField() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_NEW_PROJECT                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_NEW_PROJECT ) {
            if ( ( cb_FAMILY.getSelectedItem() != null ) && ( !(cb_FAMILY.getSelectedItem() instanceof String ) ) ) {
               TypeCategoryRec family     = (TypeCategoryRec)cb_FAMILY.getSelectedItem() ;
               String          newProject = null ;

               /* ask the user for the new project  */
               newProject = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                                 Str.getStr ( STR_NEW ),
                                                 Str.getStr ( STR_PROJECT ),
                                                 "",
                                                 new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 50 ) ).getResult() ;

               if ( newProject != null ) {
                  if ( !duplicateProject ( newProject, family ) ) {
                     TypeCategoryRec newProjectRec = TypeCategory.createDraftTypeCategory ( family.getInd(),
                                                                                            newProject,
                                                                                            TypeCategory.LEVEL_PROJECT,
                                                                                            0,
                                                                                            "N",
                                                                                            UserSystem.getUserId() ) ;
                     TypeCategory.writeToDatabase() ;
                     ((DefaultComboBoxModel)cb_PROJECT.getModel()).addElement ( newProjectRec ) ;
                     cb_PROJECT.setSelectedItem ( newProjectRec ) ;
                  } else
                     GUISystem.printBox ( "Info", "That Project already exists under the family "+family.toString() ) ;
               }
            } else
               GUISystem.printBox ( STR_INFO, STR_PLEASE_SELECT_FAMILY ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   pb_NEW_MACHINE                                          */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( e.getSource() == pb_NEW_MACHINE ) {
            if ( ( cb_PROJECT.getSelectedItem() != null ) && 
                 ( !(cb_PROJECT.getSelectedItem() instanceof String ) ) ) {
               TypeCategoryRec project     = (TypeCategoryRec)cb_PROJECT.getSelectedItem() ;
               String          newMachine  = null ;
               int             fieldLength = isOption ? LENGTH_SUBCATEGORY : LENGTH_MACHINE ;
               String          descript    = isOption ? Str.getStr ( STR_SUB_CATEGORY ) : Str.getStr ( STR_MACHINE ) ;
               String          level       = isOption ? TypeCategory.LEVEL_SUBCATEGORY : TypeCategory.LEVEL_MACHINE ;

               /* ask the user for the new project  */
               newMachine = new SingleEntryDlg ( GUISystem.getParentDefWin ( this ),
                                                 Str.getStr ( STR_NEW ),
                                                 descript,
                                                 "",
                                                 new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, fieldLength ) ).getResult() ;

               if ( newMachine != null ) {
                  if ( !duplicateMachine ( newMachine, project ) ) {
                     TypeCategoryRec newMachineRec = TypeCategory.createDraftTypeCategory ( project.getInd(),
                                                                                            newMachine,
                                                                                            level,
                                                                                            0,
                                                                                            "N",
                                                                                            UserSystem.getUserId() ) ;
                     newMachineRec.writeToDatabase() ;
                     if ( isOption )
                        product.setSubCategory ( newMachineRec ) ;
                     else
                        product.setMachine ( newMachineRec ) ;
                     refresh() ;
                     cb_MACHINE.setSelectedItem ( newMachineRec ) ;
                     enableInput() ;
                  } else
                     GUISystem.printBox ( "Info", "That machine already exists under the project "+project.toString() ) ;
               }
            } else
               GUISystem.printBox ( STR_INFO, STR_PLEASE_SELECT_PROJECT ) ;
         }


      }

   }


   /**
     *  This method will refrsh all of our data
     **/
   private void refresh () {

      //printDebugInfo() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   First, we do not want to process Action events while    */
      /* we are setting the data in the controls.                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      processActions = false ;                     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the Brands accordingly                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( isOption ) {
         Vector optionVec = new Vector ( 1 ) ;

         optionVec.addElement ( Brands.getOptionBrand() ) ;
         WinUtil.setJComboBoxData ( cb_BRAND, optionVec ) ;
         product.setBrand ( Brands.getOptionBrand() ) ;
      } else {
         Vector brands = Brands.getBrands() ;
         int    type   = product.getProductType() ;

         if ( type == TypeList.PRODUCTTYPE_SYSTEM ) {
            brands.removeElement ( Brands.getOptionBrand() ) ;
            brands.removeElement ( Brands.getMonitorBrand() ) ;
            brands.removeElement ( Brands.getCrossBrand() ) ;
            WinUtil.setJComboBoxData ( cb_BRAND, createDisplayVec ( brands ) ) ;
         } else if ( type == TypeList.PRODUCTTYPE_MONITOR ) {
            WinUtil.setJComboBoxData ( cb_BRAND, createDisplayVec ( brands ) ) ;
            product.setBrand ( Brands.getMonitorBrand() ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Select the correct brand                                */               
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( product.getBrand() != null ) {
         cb_BRAND.setSelectedItem ( product.getBrand() ) ;
      }




      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the Families and select the correct family      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( comboBoxHasValidData ( cb_BRAND ) ) {
         WinUtil.setJComboBoxData ( cb_FAMILY, createDisplayVec ( Brands.getFamilies ( (TypeCategoryRec)cb_BRAND.getSelectedItem() ) ) ) ;
         if ( product.getFamily() != null )
            cb_FAMILY.setSelectedItem ( product.getFamily() ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the projects and select the correct project     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( comboBoxHasValidData ( cb_FAMILY ) ) {
         WinUtil.setJComboBoxData ( cb_PROJECT, 
                                    createDisplayVec ( TypeCategory.getChildDraftCategories ( ((TypeCategoryRec)cb_FAMILY.getSelectedItem()).getInd(),
                                                                                              TypeCategory.LEVEL_PROJECT ) ) ) ;
         if ( product.getProject() != null )
            cb_PROJECT.setSelectedItem ( product.getProject() ) ;
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Refresh the machines                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( comboBoxHasValidData ( cb_PROJECT ) ) {
         WinUtil.setJComboBoxData ( cb_MACHINE,
                                    createDisplayVec ( TypeCategory.getChildDraftCategories ( ((TypeCategoryRec)cb_PROJECT.getSelectedItem()).getInd(),
                                                                                              isOption ? TypeCategory.LEVEL_SUBCATEGORY : TypeCategory.LEVEL_MACHINE ) ) ) ;

         if ( isOption ) {
            if ( product.getSubCategory() != null )
               cb_MACHINE.setSelectedItem ( product.getSubCategory() ) ;
         } else {
            if ( product.getMachine() != null )
               cb_MACHINE.setSelectedItem ( product.getMachine() ) ;
         }


      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, refresh the model/part number and static text      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( isOption ) {
         ef_MODEL.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_PART_NUMBER ) ) ;
         if ( product.getPartNumber() != null )
            ef_MODEL.setText ( product.getPartNumber().toString() ) ;
         else
            ef_MODEL.setText ( "" ) ;
         st_MODEL.setText ( "*"+Str.getStr ( STR_PART_NUMBER ) ) ;
         st_MACHINE.setText ( "*"+Str.getStr ( STR_SUB_CATEGORY ) ) ;
      } else {
         ef_MODEL.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_MODEL ) ) ;
         st_MACHINE.setText ( "*"+Str.getStr ( STR_MACHINE ) ) ;
         st_MODEL.setText ( "*"+Str.getStr ( STR_MODEL ) ) ;
         if ( product.getModel() != null )
            ef_MODEL.setText ( product.getModel().toString() ) ;
         else
            ef_MODEL.setText ( "" ) ;
      }

      processActions = true ;
      enableInput() ;

   }


   /**
     *  This method will be called whenever a property in the product changes
     **/
   public void productChanged ( Product product, int property ) {
   }


   /**
     *  This method will perform a variety of operations. The operation that is done
     * depends on the name
     **/
   public void run () {
      String operation = Thread.currentThread().getName() ;                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   THREAD_REFRESH                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( operation.equals ( THREAD_REFRESH ) )
         refresh() ;

      /*--------------------*/
      /*                    */
      /*  THREAD_SAVE_MODEL */
      /*                    */
      /*--------------------*/
      else if ( operation.equals ( THREAD_SAVE_MODEL ) )
         saveModelField() ;
   } /* run  */



   /**
     *  This method will do nothing
     **/
   public void focusGained ( FocusEvent e ) {
   }


   /**
     *  This method will save the data in our entry fields
     **/
   public void focusLost ( FocusEvent e ) {

      if ( e.getSource() == ef_MODEL )
         new Thread ( this, THREAD_SAVE_MODEL ).start() ;
   }


   /**
     *  This method will either enable or disable the ef_MODEL field
     * based on if a valid Machine/SubCategory is selected. It will
     * also enable/disable all controls based on enableInput property
     **/
   private void enableInput () {
      if ( enableInput ) {

         if ( ( product != null ) && ( product.getProductType() == TypeList.PRODUCTTYPE_MONITOR ) )
            cb_BRAND.setEnabled   ( false ) ;
         else if ( isOption )
            cb_BRAND.setEnabled   ( false ) ;

         cb_FAMILY.setEnabled  ( comboBoxHasValidData ( cb_BRAND ) ) ;
         cb_PROJECT.setEnabled ( comboBoxHasValidData ( cb_FAMILY ) ) ;
         cb_MACHINE.setEnabled ( comboBoxHasValidData ( cb_PROJECT ) ) ;
         ef_MODEL.setEnabled   ( comboBoxHasValidData ( cb_MACHINE ) ) ;

         pb_NEW_MACHINE.setEnabled ( cb_MACHINE.isEnabled() ) ;
         pb_NEW_PROJECT.setEnabled ( cb_PROJECT.isEnabled() ) ;

      } else {
         ef_MODEL.setEnabled ( false ) ;
         cb_BRAND.setEnabled ( false ) ;
         cb_FAMILY.setEnabled ( false ) ;
         cb_PROJECT.setEnabled ( false ) ;
         cb_MACHINE.setEnabled ( false ) ;
      }

      pb_NEW_PROJECT.setEnabled ( cb_PROJECT.isEnabled() ) ;
      pb_NEW_MACHINE.setEnabled ( cb_MACHINE.isEnabled() ) ;
   }


   /**
     *  this method will turn input on/off for this panel
     **/
   public void setEditable ( boolean enableInput ) {
      this.enableInput = enableInput ;
      enableInput() ;
   }

   /**
     *  This method will take the vector passed, insert the 'Select Item' string at the 
     * top and return it
     **/
   private Vector createDisplayVec ( Vector data ) {
      if ( data == null )
         data = new Vector ( 1 ) ;

      data.insertElementAt ( Str.getStr ( STR_SELECT_ITEM ), 0 ) ;

      return( data ) ;
   }


   /**
     *  This method will attempt to find the string project passed in under the
     * family passed in. It will return TRUE if the match was found
     **/
   private boolean duplicateProject ( String projectName, TypeCategoryRec family ) {
      if ( ( projectName != null ) && ( projectName.length() > 0 ) )
         return( ( TypeCategory.getDraftCategoryFromDescription ( projectName, TypeCategory.LEVEL_PROJECT, family.getInd() ) != null ) &&
                 ( TypeCategory.getCategoryFromDescription ( projectName, TypeCategory.LEVEL_PROJECT, family.getInd() ) != null ) ) ;
      return( false ) ;
   }

   /**
     *  This method will attempt to find the PartNumber/Model passed in under the
     * machine/subcategory passed in. It will return TRUE if the match was found
     **/
   private boolean duplicateModel ( String model, TypeCategoryRec machine ) {
      boolean duplicate = false ;

      duplicate = ( ( TypeCategory.getDraftCategoryFromDescription ( model, 
                                                                     TypeCategory.LEVEL_MODEL, 
                                                                     machine.getInd() ) ) != null ) ;

      if ( !duplicate ) {
         duplicate = ( ( TypeCategory.getDraftCategoryFromDescription ( model, 
                                                                        TypeCategory.LEVEL_PARTNUMBER, 
                                                                        machine.getInd() ) ) != null ) ;
      }

      return( duplicate ) ;
   }

   /**
     *  This method will attempt to find the string machine/subcategory passed in under the
     * project passed in. It will return TRUE if the match was found
     **/
   private boolean duplicateMachine ( String machineName, TypeCategoryRec project ) {
      return( ( TypeCategory.getDraftCategoryFromDescription ( machineName, TypeCategory.LEVEL_MACHINE,     project.getInd() ) != null ) &&
              ( TypeCategory.getDraftCategoryFromDescription ( machineName, TypeCategory.LEVEL_SUBCATEGORY, project.getInd() ) != null ) ) ;
   }


   /**
     *  This method will return TRUE if the JComboBox passed has 1 item selected and
     * that item is not a String
     **/
   private boolean comboBoxHasValidData ( JComboBox comboBox ) {
      return( ( comboBox.getSelectedItem() != null ) &&
              ( !(comboBox.getSelectedItem() instanceof String) ) ) ;
   }


   void printDebugInfo () {
      try {
         if ( product.getBrand() != null )
            System.out.println ( "Brand:"+product.getBrand()+" "+product.getBrand().getInd()+" draft:"+product.getBrand().getIsDraft() ) ;
         else
            System.out.println ( "product.getBrand() is null" ) ;
         if ( product.getFamily() != null )
            System.out.println ( "Family:"+product.getFamily()+" "+product.getFamily().getInd()+" draft:"+product.getFamily().getIsDraft() ) ;
         else
            System.out.println ( "product.getFamily() is null" ) ;
         if ( product.getProject() != null )
            System.out.println ( "Project:"+product.getProject()+" "+product.getProject().getInd()+" draft:"+product.getProject().getIsDraft() ) ;
         else
            System.out.println ( "product.getProject() is null" ) ;
         if ( product.getSubCategory() != null )
            System.out.println ( "SubCategory:"+product.getSubCategory()+" "+product.getSubCategory().getInd()+" draft:"+product.getSubCategory().getIsDraft() ) ;
         else
            System.out.println ( "product.getSubCategory() is null" ) ;
         if ( product.getPartNumber() != null )
            System.out.println ( "Part:"+product.getPartNumber()+" "+product.getPartNumber().getInd()+" draft:"+product.getPartNumber().getIsDraft() ) ;
         else
            System.out.println ( "product.getPartNumber() is null" ) ;
         if ( product.getMachine() != null )
            System.out.println ( "Machine:"+product.getMachine()+" "+product.getMachine().getInd()+" draft:"+product.getMachine().getIsDraft() ) ;
         else
            System.out.println ( "product.getMachine() is null" ) ;
         if ( product.getModel() != null )
            System.out.println ( "Model:"+product.getModel()+" "+product.getModel().getInd()+" draft:"+product.getModel().getIsDraft() ) ;
         else
            System.out.println ( "product.getModel() is null" ) ;
      } catch ( Exception e ) {
         e.printStackTrace() ;
      }
   }



   /**
     *  This method will save the information in the model field to either the
     * Part Number or Model property of the product
     **/
   private void saveModelField () {
      if ( comboBoxHasValidData ( cb_MACHINE ) ) {
         String          model       = ef_MODEL.getText() ;
         String          level       = isOption ? TypeCategory.LEVEL_PARTNUMBER : TypeCategory.LEVEL_MODEL ;
         String          fieldName   = isOption ? "Part Number" : "Model" ;
         String          parentName  = isOption ? "Sub Category" : "Machine" ;
         TypeCategoryRec modelParent = (TypeCategoryRec)cb_MACHINE.getSelectedItem() ;                                    

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, make sure they changed somethine in it           */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( product.getModel() == null ) ||
              ( !product.getModel().toString().equals ( model ) ) ) {

            if ( ( model != null ) && ( model.length() > 0 ) ) {
               /*---------------------------------------------------------------*/
               /*                                                               */
               /*   Now, make sure that part number/model isn't already there   */
               /*                                                               */
               /*---------------------------------------------------------------*/
               GUISystem.getParentDefWin ( this ).setStatus ( "Checking "+fieldName ) ;
               if ( ( TypeCategory.getCategoryFromDescription ( model, level, modelParent.getInd() ) == null ) &&
                    ( TypeCategory.getDraftCategoryFromDescription ( model, level, modelParent.getInd() ) == null ) ) {
                  TypeCategoryRec modelRec    = new TypeCategoryRec ( model,
                                                                      level ) ;

                  modelRec.setParentCatInd ( modelParent.getInd() ) ;

                  if ( isOption )
                     product.setPartNumber ( modelRec ) ;
                  else
                     product.setModel ( modelRec ) ;
               } else {
                  GUISystem.printBox ( "Info", "That "+fieldName+" already exists under the "+parentName+", Please choose another" ) ;
                  ef_MODEL.setText ( "" ) ;
                  ef_MODEL.requestFocus() ;
               }
            } else {
               GUISystem.printBox ( "Info", "The "+fieldName+" cannot be blank. Please specify a valid "+fieldName ) ;
               
               if ( isOption ) {
                 if ( product.getPartNumber() != null )
                    ef_MODEL.setText ( product.getPartNumber().toString() ) ;
               } else {
                  if ( product.getModel() != null )
                     ef_MODEL.setText ( product.getModel().toString() ) ;
               }

            }
         }
      }

      GUISystem.getParentDefWin ( this ).setStatus ( null );
   }
} /* ProductInfoPanel */
