package com.ibm.nzna.projects.qit.product.productEdit ;
/*-----------------------------------------------------------*/
/*   ProductSaveCheck                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductSaveCheck                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.JDialog ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.doc.gui.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.doc.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class checks to see if a product is Ok to save or publish. 
 *
 *
 * @version 1.0, 8/21/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductSaveCheck                                        */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductSaveCheck extends Object implements ProductConst, AppConst {


   /**
     *  This method will check to make sure that a product is ok
     * to save as a draft.
     *
     * @param draft The draft to check
     * @param parent The JDialog that is the parent of any error message
     * @param showErrors If TRUE , this method will show PrintBoxs with error messages
     *
     * @return TRUE if the draft was ok
     **/
   public static boolean checkDraft ( ProductDraft draft, JDialog parent, boolean showErrors ) {
      boolean rc       = false ;


      /*---------------------------------------------------*/
      /*                                                   */
      /*  Do not check information for a publish of delete */
      /*                                                   */
      /*---------------------------------------------------*/
      if ( draft.getWorkRequired() != TypeList.WORKREQ_DELETE ) {

         boolean isOption = ( draft.getProductType() == TypeList.PRODUCTTYPE_OPTION ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Brand                                                   */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( draft.getBrand () == null ) {
            if ( showErrors )
               GUISystem.printBox ( parent, STR_ERROR, STR_MUST_SELECT_BRAND ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Family                                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( draft.getFamily() == null ) {
            if ( showErrors )
               GUISystem.printBox ( parent, STR_ERROR, STR_MUST_SELECT_FAMILY ) ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Description                                             */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( ( draft.getDescription() == null ) || ( draft.getDescription().length() == 0 ) ) {
            if ( showErrors )
               GUISystem.printBox ( parent, STR_ERROR, STR_MUST_ENTER_DESCRIPTION ) ;
         }

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Project                                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         else if ( draft.getProject() == null ) {
            if ( showErrors )
               GUISystem.printBox ( parent, STR_ERROR, STR_MUST_SELECT_PROJECT ) ;
         }

         else
            rc = true ;                                       


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If this is an option, a SubCategory, Part Number        */
         /* is required                                               */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( rc ) {
            rc = false ;   

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Check Option                                            */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( isOption ) {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Sub Category                                            */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( draft.getSubCategory() == null ) {
                  if ( showErrors )
                     GUISystem.printBox ( parent, STR_ERROR, STR_MUST_SELECT_SUBCATEGORY ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Part Number                                             */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( draft.getPartNumber() == null ) {
                  if ( showErrors )
                     GUISystem.printBox ( parent, STR_ERROR, STR_MUST_ENTER_PART_NUMBER ) ;
               } else
                  rc = true ;
            }



            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   System                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else {

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Machine                                                 */
               /*                                                           */
               /*-----------------------------------------------------------*/
               if ( draft.getMachine() == null ) {
                  if ( showErrors )
                     GUISystem.printBox ( parent, STR_ERROR, STR_MUST_SELECT_MACHINE ) ;
               }

               /*-----------------------------------------------------------*/
               /*                                                           */
               /*   Model                                                   */
               /*                                                           */
               /*-----------------------------------------------------------*/
               else if ( draft.getModel() == null ) {
                  if ( showErrors )
                     GUISystem.printBox ( parent, STR_ERROR, STR_MUST_ENTER_MODEL ) ;
               } else
                  rc = true ;


            }


         }

      } else {
         rc = (draft.getSource().indexOf ( "OPIC" ) == -1 ) ;
         if ( !rc )
            GUISystem.printBox ( "Info", "You cannot delete products whose source is OPIC/M" ) ;
      }

      return( rc )  ;
   }



   /**
     *  This method will check to see if a document can be published.
     *
     * @param ProductDraft draft The Draft to check
     * @param parent The JDialog that is the parent of any error message
     * @param showErrors If TRUE , this method will show PrintBoxs with error messages
     *
     * @return TRUE if Ok, FALSE If not Ok
     **/
   public static boolean checkPublish ( ProductDraft draft, JDialog parent, boolean showErrors ) {
      boolean rc = false ;


      /*---------------------------------------------------*/
      /*                                                   */
      /*  Do not check information for a publish of delete */
      /*                                                   */
      /*---------------------------------------------------*/
      if ( draft.getWorkRequired() != TypeList.WORKREQ_DELETE ) {
         rc = checkDraft ( draft, parent ) ; // This takes care of checking Title, Category and BFTM

         if ( rc ) {

            rc = false ; // Now, assume it was bad


            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Locale                                                  */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( draft.getLocale() == null ) ||
                 ( ( draft.getLocale().countryCount() == 0 ) &&
                   ( !draft.getLocale().isWorldWide() ) &&
                   ( draft.getLocale().geoCount() == 0 ) ) ) {
               if ( showErrors )
                  GUISystem.printBox ( parent, STR_INFO, STR_MUST_SELECT_ONE_COUNTRY ) ;
            }

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Dates                                                   */
            /*                                                           */
            /*-----------------------------------------------------------*/
            else if ( draft.getDates().size() == 0 ) {
               if ( showErrors )
                  GUISystem.printBox ( parent, STR_INFO, STR_MUST_ENTER_ONE_DATE ) ;
            } else
               rc = true ;

         }
      }

      /*----------------------------------------------------*/
      /*                                                    */
      /*  If this is a delete, make sure there are no other */
      /* links                                              */
      /*                                                    */
      /*----------------------------------------------------*/
      else {
         StatusWin  statusWin = new StatusWin ( MainWindow.getInstance() ) ;
         
         try {
            int        typeInd = 0 ;
            Vector     links   = null ;

            statusWin.setText ( "Checking for Document Associations" ) ;

            if ( Brands.getOptionBrand().getInd() == draft.getBrand().getInd() )
               typeInd = draft.getPartNumber().getInd() ;
            else
               typeInd = draft.getModel().getInd() ;

            links = DocumentSQL.getDocumentsFromTypeCategory ( typeInd ) ;
            if ( ( links != null ) && ( links.size() > 0 ) ) {
                int cont = new ConfirmDocumentDlg ( parent, links ).getResult() ;

                rc = ( cont == ConfirmDocumentDlg.CONTINUE ) ;
                
            } else
               rc = true ;

         } catch ( Exception e ) {
            LogSystem.log ( 1, e, false ) ;
            GUISystem.printBox ( "Error", "Cannot check links to product "+draft.getProductInd()+". Delete not allowed" ) ;
            rc = false ;
         }


         if ( rc ) {
            rc = (draft.getSource().indexOf ( "OPIC" ) == -1 ) ;
            if ( !rc )
               GUISystem.printBox ( "Info", "You cannot delete products whose source is OPIC/M" ) ;
         }
         statusWin.dispose() ;
      }


      return( rc ) ;

   }


   public static boolean checkPublish ( ProductDraft draft, JDialog parent ) {
      return( checkPublish ( draft, parent, true ) ) ;
   }

   public static boolean checkDraft ( ProductDraft draft, JDialog parent ) {
      return( checkDraft ( draft, parent, true ) ) ;
   }
} /* ProductSaveCheck */
