package com.ibm.nzna.projects.qit.product.productEdit ;
/*-----------------------------------------------------------*/
/*   SaveProductDlg                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: SaveProductDlg                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.app.util.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.gui.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.status.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.event.* ;
import javax.swing.* ;
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
/**
 * This dialog allows a user to select what type of save that they would like
 *
 *
 * @version 1.0, 4/28/2000 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   SaveProductDlg                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class SaveProductDlg extends JDialog implements DataLengths, ListSelectionListener, MouseListener, ProductConst, ActionListener, AppConst, Authority {


   /**
     *  This return code is given if the user cancels or
     * closes the window without saving
     **/
   public static final int SAVE_CANCEL  = 0 ;

   /**
     *  THis return code is given if the user saves the product
     * but the product should remain up
     **/
   public static final int SAVE_SAVED = 1 ;

   /**
     *  This return code is given if the user publishes the
     * product
     **/
   public static final int SAVE_PUBLISHED = 2;

   /**
     *  This return code is given if the user saves the
     * docuement as a template
     **/
   public static final int SAVE_TEMPLATE = 3 ;

   /**
     *  This return code is given if the user saves the
     * product as a cancel
     **/
   public static final int SAVE_CANCEL_DRAFT = 4 ;

   /**
     *  This return code is given if the user saves
     * the product as a new product
     **/
   public static final int SAVE_NEW_PRODUCT = 5 ;

   /**
     *  This is the highlight button background
     **/
   private Color buttonHighlightBackground = null ;

   /**
     *  This is the default button background
     **/
   private Color buttonDefaultBackground = null ;

   /**
     *  This is the save panel we will use
     **/
   private ProductSaveStatusPanel statusPanel = null ;

   /**
     * This panel holds the template entry field and name
     **/
   private JPanel templatePanel     = new JPanel() ;

   /**
     * This panel holds the comments and action buttons
     **/
   private JPanel mainPanel   = new JPanel() ;

   /**
     *  This is the counter for the comments
     **/
   private JTextAreaCounter commentCounter = null ;

   /**
     *  This is the panel which holds the entry field for PartNumber/Model when
     * saving as a new product
     **/
   private JPanel saveNewPanel = new JPanel () ;

   /**
     *  This is the Entry field for the model/part number in the save new panel
     **/
   private JTextField ef_SAVE_NEW_MODEL = new JTextField() ;

   /**
     *  This is the help text above the entry field in the save new panel
     **/
   private JLabel st_SAVE_NEW_HELP = new JLabel ( Str.getStr ( STR_PLEASE_ENTER_NEW_MODEL ) ) ;

   /**
     *  This is the Ok button for the save/new
     **/
   private DButton pb_OK_SAVE_NEW = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     * This is the 'Publish' Button
     **/
   private DButton pb_PUBLISH = new DButton ( Str.getStr ( STR_PUBLISH ) ) ;

   /**
     * This is the 'Save' button
     **/
   private DButton pb_SAVE = new DButton ( "Save As Draft" ) ;

   /**
     * This is the 'Cancel Draft' button
     **/
   private DButton pb_CANCEL_DRAFT = new DButton ( Str.getStr ( STR_CANCEL_DRAFT ) ) ;

   /**
     * This is the text above the template name
     **/
   private JLabel st_TEMPLATE_NAME = new JLabel ( Str.getStr ( STR_PLEASE_ENTER_TEMPLATE_NAME ) ) ;

   /**
     * This is the text above the comments
     **/
   private JLabel st_COMMENTS = new JLabel ( Str.getStr ( STR_COMMENT ) ) ;

   /**
     * This is the JTextField that holds the template name
     **/
   private JTextField ef_TEMPLATE_NAME = new JTextField ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, 254 ), "", 0 ) ;

   /**
     * This is the 'Save As New Product' button
     **/
   private DButton pb_SAVE_AS_NEW = new DButton ( Str.getStr ( STR_SAVE_AS_NEW_PRODUCT ) ) ;

   /**
     * This is the 'Save as Template' button
     **/
   private DButton pb_SAVE_AS_TEMPLATE = new DButton ( Str.getStr ( STR_SAVE_AS_TEMPLATE ) ) ;

   /**
     * This is the 'Cancel' button
     **/
   private DButton pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;


   /**
     * This is the MLE which holds the last comments
     **/
   private NoTabJTextArea mle_OLD_COMMENTS = new NoTabJTextArea () ;

   /**
     * This is the MLE which holds the comments
     **/
   private NoTabJTextArea mle_COMMENTS = new NoTabJTextArea() ;

   /**
     * This is the panel which holds the old comments and new comments
     **/
   private JPanel commentPanel = new JPanel () ;

   /**
     * This is the JScrollPane which holds the commentPanel
     **/
   private JScrollPane commentScroll = new JScrollPane ( commentPanel ) ;

   /**
     *  This is the panel which holds the list of cancel reasons
     **/
   private JPanel cancelPanel = new JPanel () ;

   /**
     *  This is the JList which holds the list of cancel reasons
     **/
   private JList lb_CANCEL = new JList ( TypeList.getInstance().getTypeList ( TypeList.TYPE_CANCEL ) ) ;

   /**
     *  This is the JScrollPane which holds the lb_CANCEL
     **/
   private JScrollPane scr_CANCEL = new JScrollPane ( lb_CANCEL ) ;

   /**
     *  This is the Ok button for the cancel
     **/
   private DButton pb_OK_CANCEL = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     *  This holds return code which tells what the user did. This property
     * will be one of the SAVE_XXX values defined in this class
     **/
   private int returnCode = SAVE_CANCEL ;

   /**
     *  This is the button that will save the template once a name has
     * been chosen
     **/
   private DButton pb_SAVE_TEMPLATE_NOW = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     * This is the product that will be saved
     **/
   private ProductDraft product = null ;

   /**
     * This is the picture
     **/
   private JLabel st_IMAGE = null ;

   /**
     * This is the image we are dispplaying
     **/
   private ImageIcon wizardImage = null ;

   /**
     *  These are the comments that the user is allowed to edit
     **/
   private String comments = null ;

   /**
     * This default constructor creates and shows the window. It will
     * not return until a save is complete, a save failed or the user
     * cancels. At that time, use the getReturnCode() method to determine
     * what went on
     *
     * @param parentWin - The parent window
     * @param product The DOcument to save
     * @param comments The comments that the user is allowed to edit
     **/
   public SaveProductDlg ( AppDefaultWin parentWin, ProductDraft product, String comments ) {
      super ( parentWin, Str.getStr ( STR_SAVE ), true ) ;

      /*---------------------------------------*/
      /*                                       */
      /*  Assign our local vars and initialize */
      /*                                       */
      /*---------------------------------------*/
      this.product = product ;
      this.comments = comments ;
      initialize() ;

      setSize ( 495, 305 ) ;
      WinUtil.centerWindow ( this ) ;
      setVisible ( true ) ;
   }


   /**
     * This method initializes all properties, including the GUI, on this
     * window 
     **/
   private void initialize () {
      int    commentSize        = 1024 ;
      String currentCommentStr  = product.getOriginalComments();

      if ( currentCommentStr != null )
         commentSize-=currentCommentStr.length() ;

      /*-------------*/
      /*             */
      /*  Create GUI */
      /*             */
      /*-------------*/
      wizardImage = ImageSystem.getImageIcon ( this, ImageSystem.SAVE_DOC ) ;
      st_IMAGE = new JLabel ( wizardImage ) ;

      /*-------------------------------*/
      /*                               */
      /*  Adjust properties on the GUI */
      /*                               */
      /*-------------------------------*/
      mle_OLD_COMMENTS.setEditable ( false ) ;
      mle_OLD_COMMENTS.setBackground ( new Color ( 230, 230, 230 ) ) ;
      mle_OLD_COMMENTS.setFont ( pb_PUBLISH.getFont() ) ;
      mle_COMMENTS.setFont     ( pb_PUBLISH.getFont() ) ;
      buttonDefaultBackground   = pb_SAVE.getBackground() ;
      buttonHighlightBackground = buttonDefaultBackground.brighter() ;                                      
      if ( product.getWorkRequired() == TypeList.WORKREQ_DELETE )
         pb_PUBLISH.setText ( "Publish (Delete Active Product)" ) ;
      if ( product.getProductType() == TypeList.PRODUCTTYPE_OPTION ) {
         st_SAVE_NEW_HELP.setText ( Str.getStr ( STR_PLEASE_ENTER_NEW_PARTNUM ) ) ;
         ef_SAVE_NEW_MODEL.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, LENGTH_PART_NUMBER ) ) ;
      } else
         ef_SAVE_NEW_MODEL.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UPPER, 3 ) ) ;


      mle_COMMENTS.setDocument ( new MaskDocument ( MaskDocument.MASKTYPE_UNKNOWN, commentSize ) ) ;
      commentCounter = new JTextAreaCounter ( mle_COMMENTS) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Enable/Disable Controls                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_PUBLISH.setEnabled ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) ;
      pb_CANCEL_DRAFT.setEnabled  ( UserSystem.hasAuthority ( AUTH_AUTHOR ) ) ;
      pb_OK_CANCEL.setEnabled ( false ) ;

      /*----------------*/
      /*                */
      /*  Add listeners */
      /*                */
      /*----------------*/
      lb_CANCEL.addListSelectionListener ( this ) ;
      pb_SAVE_TEMPLATE_NOW.addActionListener ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_PUBLISH.addActionListener ( this ) ;
      pb_CANCEL_DRAFT.addActionListener ( this ) ;
      pb_SAVE_AS_TEMPLATE.addActionListener ( this ) ;
      pb_SAVE_AS_NEW.addActionListener ( this );
      pb_SAVE.addActionListener ( this ) ;
      pb_SAVE_AS_NEW.addMouseListener ( this ) ;
      pb_PUBLISH.addMouseListener ( this ) ;
      pb_CANCEL_DRAFT.addMouseListener ( this ) ;
      pb_SAVE_AS_TEMPLATE.addMouseListener ( this ) ;
      pb_SAVE.addMouseListener ( this ) ;
      pb_OK_CANCEL.addActionListener ( this ) ;
      ef_SAVE_NEW_MODEL.addActionListener ( this ) ;
      pb_OK_SAVE_NEW.addActionListener ( this ) ;


      /*--------------*/
      /*              */
      /*  Add our GUI */
      /*              */
      /*--------------*/
      mainPanel.setLayout ( null ) ;
      templatePanel.setLayout ( null ) ;
      commentPanel.setLayout ( new BorderLayout() ) ;
      commentPanel.add ( mle_OLD_COMMENTS, BorderLayout.NORTH ) ;
      commentPanel.add ( mle_COMMENTS ) ;
      cancelPanel.setLayout ( null ) ;
      cancelPanel.add ( scr_CANCEL ) ;
      cancelPanel.add ( pb_OK_CANCEL ) ;
      mainPanel.add ( st_COMMENTS ) ;
      mainPanel.add ( commentScroll ) ;
      mainPanel.add ( commentCounter ) ;
      mainPanel.add ( pb_SAVE ) ;
      mainPanel.add ( pb_SAVE_AS_TEMPLATE ) ;
      mainPanel.add ( pb_SAVE_AS_NEW ) ;
      mainPanel.add ( pb_CANCEL_DRAFT ) ;
      mainPanel.add ( pb_PUBLISH ) ;
      templatePanel.add ( st_TEMPLATE_NAME ) ;
      templatePanel.add ( ef_TEMPLATE_NAME ) ;
      templatePanel.add ( pb_SAVE_TEMPLATE_NOW ) ;
      saveNewPanel.setLayout ( null ) ;
      saveNewPanel.add ( st_SAVE_NEW_HELP ) ;
      saveNewPanel.add ( ef_SAVE_NEW_MODEL ) ;
      saveNewPanel.add ( pb_OK_SAVE_NEW ) ;

      getContentPane().setLayout ( null ) ;
      getContentPane().add ( mainPanel ) ;
      getContentPane().add ( templatePanel ) ;
      getContentPane().add ( cancelPanel ) ;
      getContentPane().add ( saveNewPanel ) ;
      getContentPane().add ( pb_CANCEL ) ;
      getContentPane().add ( st_IMAGE ) ;    

      /*--------------------------*/
      /*                          */
      /*  Adjust other properties */
      /*                          */
      /*--------------------------*/
      setResizable ( false ) ;
      templatePanel.setVisible ( false ) ;       
      cancelPanel.setVisible ( false ) ;
      saveNewPanel.setVisible ( false ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   REfresh our data                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( product.getOriginalComments() != null )
         mle_OLD_COMMENTS.setText ( product.getOriginalComments() ) ;
      mle_COMMENTS.setText ( product.getComment().toString() ) ;
      ef_TEMPLATE_NAME.setText ( product.getDescription() ) ;

   }


   /**
     * This method will layout and position all of our controls
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = wizardImage.getImage().getWidth(this)+15 ;
      int       y         = 5 ;

      super.doLayout() ;

      st_IMAGE.setBounds ( 5, 
                           ((size.height-40)/2)-(wizardImage.getImage().getHeight(this)/2), 
                           wizardImage.getImage().getWidth(this), 
                           wizardImage.getImage().getHeight(this)) ;

      mainPanel.setBounds ( x, 5, size.width-x-5, size.height-90 ) ;
      templatePanel.setBounds ( x, 5, size.width-x-5, size.height-90 ) ;
      cancelPanel.setBounds ( x, 5, size.width-x-5, size.height-90 ) ;
      saveNewPanel.setBounds ( x, 5, size.width-x-5, size.height-90 ) ;
      if ( statusPanel != null )
         statusPanel.setBounds ( x, 5, size.width-x-5, size.height-90 ) ;
      pb_CANCEL.setBounds ( x, size.height-68, 80, 25 ) ;                          


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Size the controls on the Main panel                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      x=0 ;
      y=0 ;
      size = mainPanel.getSize() ;
      st_COMMENTS.setBounds ( x, y, size.width, rowHeight ) ;
      y+=rowHeight ;

      commentScroll.setBounds ( x, y, size.width-15, rowHeight*5 ) ;
      y+=(rowHeight*5) ;

      commentCounter.setBounds ( x, y, size.width-15, 14 ) ;
      y+=16 ;

      pb_SAVE.setBounds ( x, y, 168, 25 ) ;
      pb_SAVE_AS_TEMPLATE.setBounds ( x+173, y, 168, 25 ) ;
      y+=28 ;

      pb_SAVE_AS_NEW.setBounds ( x, y, 168, 25 ) ;
      pb_CANCEL_DRAFT.setBounds ( x+173, y, 168, 25 ) ;
      y+=28 ;

      pb_PUBLISH.setBounds ( x, y, size.width-16, 25 ) ;
      y+=28 ;                                   

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, size the controls on the template panel            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      x=5;
      y=5;
      st_TEMPLATE_NAME.setBounds ( x, y, size.width-15, rowHeight ) ;
      y+=rowHeight ;
      ef_TEMPLATE_NAME.setBounds ( x, y, size.width-15, rowHeight ) ;
      y+=rowHeight+5 ;
      pb_SAVE_TEMPLATE_NOW.setBounds ( x, y, 80, 25 ) ;                                 


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, size the controls on the cancel panel              */
      /*                                                           */
      /*-----------------------------------------------------------*/
      x=5;
      y=5;
      size=cancelPanel.getSize() ;
      scr_CANCEL.setBounds ( x, y, size.width-(x*2), rowHeight*10 ) ;
      y+=rowHeight*10 ;
      pb_OK_CANCEL.setBounds ( x,y,80,24 ) ;                                   


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, size the controls on the save new panel            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      x=5;
      y=5;
      size=saveNewPanel.getSize() ;
      st_SAVE_NEW_HELP.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight ;

      ef_SAVE_NEW_MODEL.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+5 ;

      pb_OK_SAVE_NEW.setBounds ( x, y, 70, 25 ) ;




   }

   /**
     *  This method will return the return code
     **/
   public int getReturnCode () {
      return( returnCode ) ;
   }


   /**
     * This method will be called when a button is hit
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Cancel                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         if ( mainPanel.isVisible() )
            dispose() ;
         else {
            mainPanel.setVisible ( true ) ;
            templatePanel.setVisible ( false ) ;
            validate() ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save As Draft                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE ) {
         if ( ProductSaveCheck.checkDraft ( product, this ) ) {
            saveComment() ;
            createSavePanel () ;
            if ( ProductSaver.saveDraft ( product, statusPanel ) ) {
               returnCode = SAVE_SAVED ;
               dispose() ;
            } else
               destroySavePanel() ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Publish                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_PUBLISH ) {
         if ( ProductSaveCheck.checkPublish ( product, this ) ) {
            saveComment() ;
            createSavePanel() ;
            if ( ProductSaver.publishDraft ( product, statusPanel ) ) {
               returnCode = SAVE_PUBLISHED ;
               dispose() ;
            } else
               destroySavePanel () ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save as Template                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE_AS_TEMPLATE ) {
         if ( canSaveAsTemplate() ) {
            mainPanel.setVisible     ( false ) ;
            templatePanel.setVisible ( true ) ;
            saveNewPanel.setVisible  ( false ) ;
            cancelPanel.setVisible   ( false ) ;
         } else
            GUISystem.printBox ( this, "Info", "Templates cannot have graphics or file attachments. Please remove any graphic and file attachment and try again" ) ;
         validate() ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save Template Now                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE_TEMPLATE_NOW ) {
         String newTitle = ef_TEMPLATE_NAME.getText() ;

         if ( ( newTitle != null ) && ( newTitle.length() > 0 ) ) {
            product.setDescription ( newTitle ) ;
            product.setTemplate ( true ) ;
            saveComment() ;
            createSavePanel() ;
            if ( ProductSaver.saveDraft ( product, statusPanel ) ) {
               returnCode = SAVE_TEMPLATE ;
               ProductTemplateSystem.addTemplate ( product ) ;
               GUISystem.printBox ( this, "Info", "A new Product Template is now available" ) ;
               dispose() ;
            } else
               destroySavePanel () ;
         } else
            GUISystem.printBox ( this, "Info", "Please supply the name for this template" ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save Cancel Draft                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_CANCEL_DRAFT ) {
         mainPanel.setVisible     ( false ) ;
         templatePanel.setVisible ( false ) ;
         saveNewPanel.setVisible  ( false ) ;
         cancelPanel.setVisible   ( true ) ;
         validate() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_OK_CANCEL                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_OK_CANCEL ) {
         TypeCancelRec cancelRec = (TypeCancelRec) lb_CANCEL.getSelectedValue() ;

         if ( cancelRec != null ) {
            ((ProductDraft)product).setTypeCancelInd ( cancelRec.getInd() ) ;
            ((ProductDraft)product).setStopDate ( new CDate ( CDate.DATE_STAMP ).today() ) ;

            createSavePanel() ;
            if ( ProductSaver.saveDraft ( product, statusPanel ) ) {
               returnCode = SAVE_TEMPLATE ;
               dispose() ;
            } else
               destroySavePanel() ;
         }
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save as new product                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_SAVE_AS_NEW ) {
         if ( ProductSaveCheck.checkDraft ( product, this ) ) {
            mainPanel.setVisible     ( false ) ;
            templatePanel.setVisible ( false ) ;
            saveNewPanel.setVisible  ( true ) ;
            cancelPanel.setVisible   ( false ) ;
         }
      }



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Save as New Product (From SaveNewPanel)                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == ef_SAVE_NEW_MODEL ) ||
                ( e.getSource() == pb_OK_SAVE_NEW ) ) {
         String model = ef_SAVE_NEW_MODEL.getText() ;

         if ( model.length() > 0 ) {
            boolean         isOption    = (product.getProductType() == TypeList.PRODUCTTYPE_OPTION ) ;
            TypeCategoryRec modelParent = isOption ? product.getSubCategory() : product.getMachine() ;
            String          level       = isOption ? TypeCategory.LEVEL_PARTNUMBER : TypeCategory.LEVEL_MODEL ;
            String          parentName  = isOption ? "Sub Category" : "Machine" ;
            String          fieldName   = isOption ? "Part Number" : "Model" ;        

            /*-----------------------------------------------------------*/
            /*                                                           */
            /*   Do not allow dups                                       */
            /*                                                           */
            /*-----------------------------------------------------------*/
            if ( ( TypeCategory.getCategoryFromDescription ( model, level, modelParent.getInd() ) == null ) &&
                 ( TypeCategory.getDraftCategoryFromDescription ( model, level, modelParent.getInd() ) == null ) ) {
               ProductDraft    newDraft    = null ;
               TypeCategoryRec modelRec    = new TypeCategoryRec ( model,
                                                                   level ) ;

               modelRec.setParentCatInd ( modelParent.getInd() ) ;

               newDraft = product.createNew ( modelRec ) ;

               if ( ProductSaveCheck.checkDraft ( newDraft, this ) ) {
                  product.setCheckedOutBy ( null ) ;
                  this.product = newDraft ;

                  saveComment() ;
                  createSavePanel() ;

                  if ( ProductSaver.saveDraft ( product, statusPanel ) ) {
                     returnCode = SAVE_NEW_PRODUCT ;
                     GUISystem.printBox ( this, "Info", "A new Product Draft has been created" ) ;
                     dispose() ;
                  } else
                     destroySavePanel() ;

               }
            } else {
               GUISystem.printBox ( this, "Info", "That "+fieldName+" already exists under the "+parentName+", Please choose another" ) ;
            }

         } else
            GUISystem.printBox ( "Info", "Please select a model/part number for this new product" ) ;

      }
   }



   public void mouseClicked  ( MouseEvent e ) {
   }
   public void mousePressed  ( MouseEvent e ) {
   }
   public void mouseReleased ( MouseEvent e ) {
   }
   public void mouseEntered ( MouseEvent e ) {
      ((DButton)e.getSource()).setBackground ( buttonHighlightBackground ) ;
   }

   public void mouseExited ( MouseEvent e ) {
      ((DButton)e.getSource()).setBackground ( buttonDefaultBackground ) ;
   }


   /**
     *  This method will create our status save panel and show it
     **/
   private void createSavePanel () {
      statusPanel = new ProductSaveStatusPanel ( product ) ;
      getContentPane().add ( statusPanel ) ;
      mainPanel.setVisible ( false ) ;
      templatePanel.setVisible ( false ) ;
      pb_CANCEL.setEnabled ( false ) ;
      validate() ;
   }


   /**
     *  This method will destroy the save panel and put the main panel back up
     **/
   private void destroySavePanel () {
      getContentPane().remove ( statusPanel ) ;
      statusPanel = null ;
      mainPanel.setVisible ( true ) ;
      pb_CANCEL.setEnabled ( true ) ;
      validate() ;
   }

   /**
     *  This method will save the comment that the user has entered, if any
     **/
   private void saveComment () {
      String comment = mle_COMMENTS.getText() ;

      if ( ( comment == null ) || ( comment.length() == 0 ) ) {
         String date = new CDate ( CDate.DATE_WW_DATETIME ).today() ;

         comment = "On "+date+" "+UserSystem.getUserName()+" "+
                   "saved this product but made no comment\n" ;
      }

      product.setComment ( comment ) ;
   }


   /**
     *  This method will return TRUE if the product can be saved as a template. Any product
     * with a graphic or file attachment cannot be saved as a template
     **/
   private boolean canSaveAsTemplate () {
      return( true ) ;
   }


   public void valueChanged ( ListSelectionEvent e ) {
      if ( e.getSource() == lb_CANCEL )
         pb_OK_CANCEL.setEnabled ( lb_CANCEL.getSelectedValue() != null ) ;
   }


   public ProductDraft getProduct () {
      return( product ) ;
   }
} /* SaveProductDlg */
