package com.ibm.nzna.projects.qit.product.productEdit.dateEdit ;   
/*-----------------------------------------------------------*/
/*   DateEditDlg                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: DateEditDlg                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.awt.* ;
import java.util.* ;
import java.awt.event.* ;
import javax.swing.* ;
import javax.swing.event.* ;
import javax.swing.table.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.print.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This dialog allows the user to edit any date in any country for a product 
 *
 *
 * @version 1.0, 8/30/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   DateEditDlg                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class DateEditDlg extends JDialog implements MouseListener, ListSelectionListener, ActionListener, AppConst, ProductConst {

   /**
     *  The dreaded JTable
     **/
   private JTable dateTable = null ;

   /**
     *  This is the ProductDateTableModel which holds all the logic
     **/
   private ProductDateTableModel dateModel = null ;

   /**
     *  This is the product we are displaying dates for
     **/
   private Product product = null ;

   /**
     *  This is the JSCrollPane which holds the JTable
     **/
   private JScrollPane scrollPane = null ;

   /**
     *  This Ok Button
     **/
   private DButton pb_OK = new DButton ( Str.getStr ( STR_OK ) ) ;

   /**
     *  The Add Country Button
     **/
   private DButton pb_ADD_COUNTRY = new DButton ( Str.getStr ( STR_ADD_COUNTRY ) ) ;

   /**
     *  The remove country button
     **/
   private DButton pb_REMOVE_COUNTRY = new DButton ( Str.getStr ( STR_REMOVE_COUNTRY ) ) ;

   /**
     *  The print button
     **/
   private DButton pb_PRINT = new DButton ( Str.getStr ( STR_PRINT ) ) ;

   /**
     * This is our help text
     **/
   private JLabel st_HELP = new JLabel ( "To edit a single date, double click on that date" ) ;

   /**
     *  The Cancel Button
     **/
   private DButton pb_CANCEL = new DButton ( Str.getStr ( STR_CANCEL ) ) ;

   /**
     * This is the button panel
     **/
   private ButtonPanel buttonPanel = new ButtonPanel () ;

   /**
     *  This is the Date text
     **/
   private JLabel st_DATE = new JLabel ( Str.getStr ( STR_DATE ) ) ;

   /**
     *  This is the Drop down with the date types
     **/
   private JComboBox cb_DATE = new JComboBox ( TypeList.getInstance().getTypeList ( TypeList.TYPE_DATE ) );

   /**
     *  This is the set button
     **/
   private DButton pb_SET = new DButton ( "Set" ) ;


   /**
     *  This is the EntryField for all dates
     **/
   private DateEntryField ef_DATE = new DateEntryField () ;

   /**
     *  This determines if anything is allowed to be edited
     **/
   private boolean editable = true ;

   /**
     *  Default Constructor
     **/
   public DateEditDlg ( Frame parent, Product product, boolean editable ) {
      super ( parent, Str.getStr ( STR_DATES ), true ) ;

      this.editable = editable ;
      this.product = product ;
      initialize() ;
      setVisible ( true ) ;
   }


   /**
     *  This method will initialize all our GUI and properties
     **/
   private void initialize () {
      Rectangle   bounds      = (Rectangle)PropertySystem.get ( PROPERTY_DATE_EDIT_BOUNDS ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      dateTable = new JTable ( dateModel = new ProductDateTableModel ( product.getDates() )) ;        
      scrollPane = new JScrollPane ( dateTable ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */

      /*-----------------------------------------------------------*/
      dateTable.setAutoResizeMode ( JTable.AUTO_RESIZE_OFF ) ;
      dateTable.setShowHorizontalLines ( false ) ;
      scrollPane.setBackground ( Color.white ) ;    
      dateTable.setColumnSelectionAllowed ( true ) ;
      dateTable.addMouseListener ( this ) ;
      pb_SET.setEnabled ( editable ) ;
      pb_OK.setEnabled ( editable ) ;
      pb_ADD_COUNTRY.setEnabled ( editable ) ;
      pb_REMOVE_COUNTRY.setEnabled ( editable ) ;
      cb_DATE.setEnabled ( editable ) ;
      if ( !editable )
         pb_CANCEL.setText ( Str.getStr ( STR_CLOSE ) ) ;

      ef_DATE.addFocusListener  ( EntrySystem.getDateFieldFixer() ) ;
      ef_DATE.addActionListener ( EntrySystem.getDateFieldFixer() ) ;
      dateTable.setPreferredScrollableViewportSize ( new Dimension ( 300, 400 ) ) ;
      WinUtil.setBackgroundAll ( scrollPane, Color.white ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener             ( this ) ;
      pb_CANCEL.addActionListener         ( this ) ;
      pb_ADD_COUNTRY.addActionListener    ( this ) ;
      pb_REMOVE_COUNTRY.addActionListener ( this ) ;
      cb_DATE.addActionListener           ( this ) ;
      dateTable.getSelectionModel().addListSelectionListener ( this ) ;
      ef_DATE.addActionListener           ( this ) ;
      pb_SET.addActionListener            ( this ) ;
      pb_PRINT.addActionListener          ( this ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add GUI                                                 */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_ADD_COUNTRY ) ;
      buttonPanel.add ( pb_REMOVE_COUNTRY ) ;
      buttonPanel.add ( pb_CANCEL ) ;
      buttonPanel.add ( pb_PRINT ) ;
      getContentPane().setLayout ( null );
      getContentPane().add ( st_HELP,     BorderLayout.NORTH ) ;
      getContentPane().add ( scrollPane,  BorderLayout.CENTER ) ;
      getContentPane().add ( st_DATE ) ;
      getContentPane().add ( cb_DATE ) ;
      getContentPane().add ( ef_DATE ) ;
      getContentPane().add ( pb_SET ) ;
      getContentPane().add ( buttonPanel, BorderLayout.SOUTH ) ;
      GUISystem.setPropertiesOnPanel (getContentPane() ) ;

      if ( bounds == null ) {
         setSize ( 500, 400 ) ;
         WinUtil.centerWindow ( this ) ;
      } else
         setBounds ( bounds ) ;

   }


   /**
     *  this method will be called when an action event is fired on us
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Cancel                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( e.getSource() == pb_CANCEL ) {
         PropertySystem.put ( PROPERTY_DATE_EDIT_BOUNDS, getBounds() ) ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Ok                                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_OK ) {
         product.setDates ( dateModel.getDates() ) ;
         PropertySystem.put ( PROPERTY_DATE_EDIT_BOUNDS, getBounds() ) ;
         dispose() ;
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add country                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_ADD_COUNTRY )
         addCountry() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Print                                                   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_PRINT )
         getPrintObject() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Date Type                                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == cb_DATE ) {
         int selectedRow = dateTable.getSelectedRow() ;

         if ( selectedRow >= 0 ) {
            TypeCountryCodeRec countryRec  = dateModel.getCountryForRow ( selectedRow ) ;
            refreshDate ( dateModel.getProductDateFromCountryType ( dateModel.getCountryForRow ( selectedRow ).getInd(),
                                                                    ((TypeDateRec)cb_DATE.getSelectedItem()).getInd() ) ) ;

            ef_DATE.requestFocus() ;
         } else
            GUISystem.printBox ( this, "Info", "Please select a date from one of the countries listed" ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Date Entry Field                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_SET ) ||
                ( e.getSource() == ef_DATE ) ) {
         int                selectedRow = dateTable.getSelectedRow () ;
         if ( selectedRow >= 0 ) {
            TypeCountryCodeRec countryRec  = dateModel.getCountryForRow ( selectedRow ) ;

            if ( countryRec != null )
               editDate ( countryRec ) ;
            else
               GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_ITEM ) ;
         } else
            GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_ITEM ) ;
      }


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   pb_REMOVE_COUNTRY                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_REMOVE_COUNTRY ) {
         int selectedRow = dateTable.getSelectedRow () ;

         if ( selectedRow >= 0 ) {
            TypeCountryCodeRec countryRec  = dateModel.getCountryForRow ( selectedRow ) ;

            if ( countryRec != null )
               removeCountry ( countryRec ) ;

         } else
            GUISystem.printBox ( this, STR_INFO, STR_MUST_SELECT_ITEM ) ;
      }



   }


   /**
     *  This method will add a country
     **/
   private void addCountry () {
      Vector   countryVec = null ;
      JListDlg listDlg    = new JListDlg ( GUISystem.getParentDefWin ( this ),
                                           Str.getStr ( STR_ADD_COUNTRY ),
                                           TypeList.getInstance().getTypeList ( TypeList.TYPE_COUNTRYCODE ) ) ;

      listDlg.setSelectionMode ( ListSelectionModel.SINGLE_SELECTION ) ;
      countryVec = listDlg.getResult() ;

      if ( ( countryVec != null ) && ( countryVec.size() > 0 ) ) {
         TypeCountryCodeRec addCountry    = (TypeCountryCodeRec)countryVec.elementAt(0) ;


         if ( !dateModel.contains ( addCountry ) ) {
            dateModel.addCountry ( addCountry ) ;
            dateTable.revalidate(); 
         } else
            GUISystem.printBox ( this, "Info", "That country is already in the list. Please select another country" ) ;
      }

   }

   /**
     *  This method will print whatever information is there in the dialog
     **/

   private Object getPrintObject () {

      return(new Print ( this, GUISystem.getParentDefWin(this), Str.getStr ( STR_DOCUMENTS )  )) ;
   }

   /**
     * This method will lay out and position our controls
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = 5 ;


      super.doLayout() ;
      st_HELP.setBounds ( 5, 5, size.width-15, rowHeight ) ;

      scrollPane.setBounds ( 5, rowHeight+5, size.width-15, size.height-113 ) ;

      st_DATE.setBounds ( x, size.height-86, 50, rowHeight ) ;
      x+=55 ;
      cb_DATE.setBounds ( x, size.height-86, 150, rowHeight ) ;
      x+=155 ;
      ef_DATE.setBounds ( x, size.height-86, 90, rowHeight  ) ;
      x+=95 ;
      pb_SET.setBounds  ( x, size.height-85, 60, rowHeight-2 ) ;

      buttonPanel.setBounds ( 5, size.height-60, size.width-15, 25 ) ;
   }



   /**
     *  This method will refrsh the date that is entered into the date field and
     * apply it
     **/
   private void editDate ( TypeCountryCodeRec country ) {
      String      date     = null ;
      TypeDateRec typeDate = (TypeDateRec)cb_DATE.getSelectedItem() ;

      date = ef_DATE.getText() ;

      if ( CDate.checkDate ( date, CDate.DATE_DATE ) == 0 ) {


         ProductDate productDate = dateModel.getProductDateFromCountryType ( country.getInd(), typeDate.getInd() ) ;

         if ( productDate == null ) {
            productDate = new ProductDate ( product.getProductInd(),
                                            date,
                                            date,
                                            typeDate.getInd(),
                                            country.getInd() ) ;
            dateModel.addDate ( productDate ) ;
         } else
            productDate.setActualDate ( date ) ;

         productDate.setToString ( "<html><body><font color=\"blue\">"+date+"</font></body></html>" ) ;
         repaintTable() ;
      } else
         GUISystem.printBox ( this, STR_INFO, STR_DATE_ENTERED_WRONG ) ;
   }



   /**
     *  This method will be called when a date is selected
     **/
   public void valueChanged ( ListSelectionEvent e ) {
      mouseClicked ( null ) ;
   }


   /**
     *  This method will refresh the date that is selected
     **/
   private void refreshDate ( ProductDate productDate ) {
      if ( productDate != null ) {
         cb_DATE.setSelectedItem ( TypeList.getInstance().objectFromInd ( productDate.getDateType(), TypeList.TYPE_DATE ) ) ;
         if ( productDate.getActualDate() != null )
            ef_DATE.setText ( productDate.getActualDate() ) ;
         else
            ef_DATE.setText ( productDate.getTargetDate() ) ;
      } else {
         ef_DATE.setText ( "" ) ;
      }
   }


   public void mouseClicked ( MouseEvent e ) {
      if ( editable ) {
         int                selectedRow    = dateTable.getSelectedRow() ;
         int                selectedColumn = dateTable.getSelectedColumn() ;

         if ( ( selectedRow >= 0 ) && ( selectedColumn >= 1 ) ) {
            ProductDate productDate = (ProductDate)dateModel.getValueAt ( selectedRow, selectedColumn ) ;
            refreshDate ( productDate ) ;
            cb_DATE.setSelectedItem ( dateModel.getDateTypeFromColumn ( selectedColumn ) ) ;
         }
      }
   }

   public void mousePressed ( MouseEvent e ) {
   }
   public void mouseReleased ( MouseEvent e ) {
   }
   public void mouseMoved ( MouseEvent e ) {
   }
   public void mouseEntered ( MouseEvent e ) {
   }
   public void mouseExited ( MouseEvent e ) {
   }

   public void repaintTable () {
      dateTable.setShowGrid ( true ) ;
      dateTable.setShowHorizontalLines ( false ) ;
      scrollPane.invalidate() ;
      ((JComponent)getContentPane()).revalidate() ;
   }


   /**
     *  This method will remove all dates for a product under a country
     **/
   private void removeCountry ( TypeCountryCodeRec countryRec ) {
      if ( countryRec != null ) {
         if ( GUISystem.printBox ( STR_QUESTION, STR_ARE_YOU_SURE_REMOVE_COUNTRY_DATES ) ) {
            Vector dates = dateModel.getDates() ;
            int    i     = 0 ;
            int    size  = dates.size() ;

            while ( i < size ) {
               if ( ((ProductDate)dates.elementAt(i)).getCountryCodeInd() == countryRec.getInd() ) {
                  dates.removeElementAt(i) ;
                  size-- ;
               } else
                  i++ ;
            } /* while  */

            dateModel.setProductDates ( dates ) ;
         }
      } /* if  */
   } /* removeCountry  */
} /* DateEditDlg */
