package com.ibm.nzna.projects.qit.product.productEdit.dateEdit ;
/*-----------------------------------------------------------*/
/*   ProductDateTableModel                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductDateTableModel               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import javax.swing.* ;
import javax.swing.table.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.projects.qit.app.*;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * A Model for a JTable that contains a list of ProductDate objects. This
 * model uses a Vector of ProductDate Objects that can be changed with
 * the 'setProductDates' method. 
 *
 *
 * @version 1.0, 4/3/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductDateTableModel                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductDateTableModel extends AbstractTableModel implements AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Vector productDates    = null ;
   private Vector typeDates       = TypeList.getInstance().getTypeList ( TypeList.TYPE_DATE ) ;
   private Vector uniqueCountries = null ;

   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This is the default Constructor which
    *  does nothing
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductDateTableModel () {
   }





   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This constructor creates a new model
    *  and assigns the data. Please note that
    *  you will have to call JTable.setModel ( thisModel ) to show the data
    *
    *  @param productDates The Vector of DocumentList Objects
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductDateTableModel ( Vector productDates ) {
      setProductDates ( productDates ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnCount
    */
   /**
    *    Returns the number of columns we 
    *  display for productDates
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public int getColumnCount () {
      return( typeDates.size()+1 ) ;
   }




   /*------------------------------------------------------------*/
   /* getRowCount
    */
   /**
    *    If we have any data, returns the number
    *  of productDates we display
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return int
    *//*---------------------------------------------------------*/
   public int getRowCount () {
      if ( uniqueCountries == null )
         uniqueCountries = getUniqueCountries() ;

      if ( uniqueCountries != null )
         return( uniqueCountries.size() ) ;
      return( 0 ) ;
   }




   /*------------------------------------------------------------*/
   /* getValueAt
    */
   /**
    *    This method will return the Object that
    *  should be displayed in the row/column
    *  
    *
    *  @param row The row to retrieve data for
    *  @param column The column to retrieve data for
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Object getValueAt ( int row, int column ) {
      try {
         TypeCountryCodeRec countryRec = (TypeCountryCodeRec) uniqueCountries.elementAt(row) ;

         if ( countryRec != null ) {
            if ( column == 0 )
               return( countryRec.toString() ) ;
            else {
               TypeDateRec typeDateRec = (TypeDateRec) typeDates.elementAt ( column-1 ) ;
               ProductDate productDate = getProductDateFromCountryType ( countryRec.getInd(), typeDateRec.getInd() ) ;


               if ( productDate != null ) {
                  if ( productDate.getActualDate() != null )
                     productDate.setToString ( "<html><body>"+convertDate ( productDate.getActualDate() )+"</body></html>" ) ;
                  else if ( productDate.getTargetDate() != null )
                     productDate.setToString ( "<html><body><font color=red>"+convertDate ( productDate.getTargetDate() )+"</font></body></html>" ) ;

                  return(productDate) ;
               }
            }
         }
      } catch ( Exception e ) {
      }

      return( null ) ;
   }


   /**
     *  This method will return all dates under the specified country and date type
     **/
   protected ProductDate getProductDateFromCountryType (int countryInd, int dateInd ) {
      ProductDate row = null ;

      if ( ( productDates != null ) && ( productDates.size() > 0 ) ) {
         int i     = 0 ;
         int size  = productDates.size() ;

         while ( ( row == null ) && ( i < size ) ) {
            if ( ( ((ProductDate)productDates.elementAt(i)).getCountryCodeInd() == countryInd ) &&
                 ( ((ProductDate)productDates.elementAt(i)).getDateType() == dateInd ) )
               row = ((ProductDate)productDates.elementAt(i)) ;
            i++ ;
         }
      }

      return( row ) ;
   }


   /*------------------------------------------------------------*/
   /* getColumnObject
    */
   /**
    *    This method will return what type of
    *  object should be displayed
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public Class getColumnClass(int columnIndex) {
      try {
         return( getValueAt ( 0, columnIndex ).getClass() ) ;
      } catch ( Exception e ) {
      }

      return( Object.class ) ;
   }





   /*------------------------------------------------------------*/
   /* setProductDates
    */
   /**
    *    This method will set the productDates that
    *  this model represents. Please note that
    *  you must call 'setModel' on the JTable to reflect the changes
    *
    *  @param productDates The Vector of DocumentList Objects to represent
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void setProductDates ( Vector productDates ) {
      this.productDates = productDates ;
      uniqueCountries = getUniqueCountries() ;
   }

   public boolean isCellEditable ( int row, int col ) {
      return( false ) ; //return( col > 0 ) ;
   }



   public void setValueAt ( Object value, int row, int col ) {
   }

   /*------------------------------------------------------------*/
   /* getColumnName
    */
   /**
    *    This method will return the name of the
    *  column specified.
    *  
    *
    *  @param column The column index to return
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getColumnName ( int column ) {
      switch ( column ) {
         case 0 :
            return( Str.getStr ( STR_COUNTRY ) ) ;

         default:
            return( typeDates.elementAt(column-1).toString() ) ;
      }
   }


   /**
     *  This method will go through all of the dates and pulls out each unique
     * country. It will then return a Vector of the results
     *
     * @return Vector of TypeCountryCodeRec objects
     **/
   private Vector getUniqueCountries () {
      Vector countries = new Vector ( 1 ) ;

      if ( ( productDates != null ) && ( productDates.size() > 0 ) ) {
         int                i       = 0 ;
         int                size    = productDates.size() ;
         ProductDate        date    = null ;
         TypeCountryCodeRec country = null ;

         while ( i < size ) {
            date = (ProductDate)productDates.elementAt(i) ;
            country = (TypeCountryCodeRec)TypeList.getInstance().objectFromInd ( date.getCountryCodeInd(), TypeList.TYPE_COUNTRYCODE ) ;
            if ( !countries.contains ( country ) )
               countries.addElement ( country ) ;
            i++ ;
         }
      }
      new QuickSort ( countries ) ;

      return( countries ) ;
   }


   public void addCountry ( TypeCountryCodeRec countryCodeRec ) {
      if ( uniqueCountries == null )
         uniqueCountries = getUniqueCountries() ;

      uniqueCountries.insertElementAt ( countryCodeRec, 0 ) ;
      new QuickSort ( uniqueCountries ) ;

   }


   /**
     *  This method will return the country that is displaying on a particular row
     **/
   public TypeCountryCodeRec getCountryForRow ( int row ) {
      if ( uniqueCountries != null ) {
         return( (TypeCountryCodeRec)uniqueCountries.elementAt(row) ) ;
      }
      return( null ) ;
   }


   /**
     *  This method will add a date to our list of dates
     *
     * @param ProductDate The ProductDate to add
     **/
   public void addDate ( ProductDate date ) {
      if ( productDates == null )
         productDates = new Vector ( 1 ) ;

      productDates.addElement ( date ) ;
   }

   /**
     *  This method will return the date type for a particular column
     **/
   public TypeDateRec getDateTypeFromColumn ( int column ) {
      if ( column > 0 )
         return( (TypeDateRec)typeDates.elementAt(column-1) ) ;
      return( null ) ;
   }

   /**
     *  This method will return our dates
     **/
   public Vector getDates () {
      return( productDates ) ;
   }

   public boolean contains ( TypeCountryCodeRec countryRec ) {
      if ( uniqueCountries == null )
         return ( false ) ;
      return ( uniqueCountries.contains ( countryRec ) ) ;
   }

   private String convertDate ( String date ) {
      return(  CDate.createDate ( CDate.parse ( date, CDate.DATE_YEAR, CDate.DATE_DATE ),
                                  CDate.parse ( date, CDate.DATE_MONTH, CDate.DATE_DATE ),
                                  CDate.parse ( date, CDate.DATE_DAY, CDate.DATE_DATE ),
                                  0,0,0,CDate.DATE_WW_DATE ) ) ;
   }
} /* ProductDateTableModel */
