package com.ibm.nzna.projects.qit.product.productExport ;
/*-----------------------------------------------------------*/
/*   ExportWizard                                            */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ExportWizard                        */
/*                                                           */
/*                                                           */
/*         @Copyright  1999 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.manager.views.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import javax.swing.* ;
import java.util.* ;
import java.awt.* ;
import java.io.* ;
/**
 * The Export Wizard will walk the user through the steps 
 * of exporting data.
 *
 * Step 1 - Ask the user what data set to export
 * Step 2 - Asks the user for what fields to export in what order
 * Step 3 - Ask the user what file to export to
 * Step 4 - show the user the export progress
 * Step 5 - Finished Msg.
 *
 *
 * @version 1.0, 1/21/100 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ExportWizard                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ExportWizard extends JDialog implements ProductConst, AppConst, WizardListener, PropertyListener {


   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private Wizard                wizardPanel          = null ;
   private boolean               rc                   = false ;
   private InfoStep              welcomeStep          = null ;
   private InfoStep              finishStep           = null ;
   private Frame                 parentFrame          = null ;
   private ProductSetStep        setStep              = null ;
   private ProductFieldsStep     fieldsStep           = null ;
   private ProductFileStep       filenameStep         = null ;
   private ProcessStep           processStep          = null ;
   private ProductExportProperty exportProperty       = new ProductExportProperty () ;



   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize
    *  all of our controls. It will not show the dialog.
    *  Please use the 'getResult' method to show and process the dialog
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ExportWizard ( Vector selectedProducts, Vector viewableProducts, PMView view ) {
      super ( new Frame(), "", true ) ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Assign Local Vars                                       */        
      /*                                                           */
      /*-----------------------------------------------------------*/
      parentFrame = (Frame) getParent() ;
      CreateWizardWin statusWin = new CreateWizardWin ( parentFrame,6 ) ;
      exportProperty.setPMView ( view ) ;
      exportProperty.setSelectedProducts ( selectedProducts ) ;
      exportProperty.setViewableProducts ( viewableProducts ) ;


      WinUtil.centerWindow ( statusWin ) ;
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our Wizard control                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      try {
         wizardPanel = new Wizard ( ImageSystem.getImageIcon ( this, "skins"+
                                                               File.separator+
                                                               PropertySystem.getString ( PropertySystem.PROPERTY_SKINNAME )+
                                                               File.separator+
                                                               "large_export.gif" ) ) ;                     
         statusWin.setValue ( 1 ) ;
      } catch ( Exception e ) {
         wizardPanel = new Wizard () ;
      }




      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our steps                                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      welcomeStep = new InfoStep ( "Press Next to begin", Str.getStr ( STR_EXPORT_WIZ_WELCOME ) ) ;                      
      statusWin.setValue ( 2 ) ;
      setStep    = new ProductSetStep ( wizardPanel, exportProperty ) ;
      statusWin.setValue ( 3 ) ;
      fieldsStep = new ProductFieldsStep ( wizardPanel, exportProperty ) ;
      statusWin.setValue ( 3 ) ;
      filenameStep = new ProductFileStep ( wizardPanel, exportProperty ) ;
      statusWin.setValue ( 4 ) ;
      processStep  = new ProcessStep ( wizardPanel, exportProperty ) ;
      statusWin.setValue ( 5 ) ;
      finishStep = new InfoStep ( "", Str.getStr ( STR_EXPORT_WIZ_TITLE_COMPLETE ) ) ;
      statusWin.setValue ( 6 ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties/Add Listeners                         */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setText() ;
      wizardPanel.addWizardListener ( this ) ;
      PropertySystem.addPropertyListener ( PropertySystem.PROPERTY_LANGUAGE, this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the controls to our window                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      getContentPane().setLayout ( new BorderLayout () ) ;
      getContentPane().add ( wizardPanel ) ;
      statusWin.setValue ( 6 ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      setSize ( 495, 375 ) ;
      statusWin.dispose() ;
      WinUtil.centerWindow ( this ) ;
   }





   /*------------------------------------------------------------*/
   /* getResult
    */
   /**
    *    This method will return the results
    *  from the user processing this wizard. This
    *  method will show the wizard and will not return unless Finish or Cancel were pressed
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return TRUE = Finish was pressed, FALSE = Cancel was pressed
    *//*---------------------------------------------------------*/
   public boolean getResult () {

      wizardPanel.setCurrentStep ( welcomeStep ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;

      setVisible ( true ) ;
      parentFrame.dispose() ;
      return( rc ) ;
   }






   /*------------------------------------------------------------*/
   /* cancelPerformed
    */
   /**
    *    This method will be called if the user
    *  hit the 'Cancel' button on the Wizard
    *  
    *
    *  @param e The wizard event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void cancelPerformed ( WizardEvent e ) {
      rc = false ;
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* finishPerformed
    */
   /**
    *    This method will be called when the
    *  user presses the finish button
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void finishPerformed ( WizardEvent e) {
      rc = true ;
      dispose() ;
   }




   /*------------------------------------------------------------*/
   /* nextPerformed
    */
   /**
    *    This method will be called when the
    *  next button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void nextPerformed ( WizardEvent e ) {
      WizardStep step = wizardPanel.getCurrentStep() ;


      wizardPanel.setVisible ( false ) ;
      if ( step == welcomeStep )
         wizardPanel.setCurrentStep ( setStep ) ;
      else if ( step == setStep )
         wizardPanel.setCurrentStep ( fieldsStep ) ;
      else if ( step == fieldsStep )
         wizardPanel.setCurrentStep ( filenameStep ) ;
      else if ( step == filenameStep )
         wizardPanel.setCurrentStep ( processStep ) ;
      else if ( step == processStep )
         wizardPanel.setCurrentStep ( finishStep ) ;

      GUISystem.setPropertiesOnPanel ( getContentPane() ) ;
      wizardPanel.setVisible ( true ) ;
      adjustButtons () ;
   }


   /*------------------------------------------------------------*/
   /* backPerformed
    */
   /**
    *    This method will be called when the
    *  back button is performed
    *  
    *
    *  @param e The event
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void backPerformed ( WizardEvent e ) {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == setStep )
         wizardPanel.setCurrentStep ( welcomeStep ) ;
      else if ( step == fieldsStep )
         wizardPanel.setCurrentStep ( setStep ) ;
      else if ( step == filenameStep )
         wizardPanel.setCurrentStep ( fieldsStep ) ;
      else if ( step == processStep ) 
         wizardPanel.setCurrentStep ( setStep ) ;

      adjustButtons ( ) ;
   }





   /*------------------------------------------------------------*/
   /* adjustButtons
    */
   /**
    *    This method will look at what step is
    *  currently showing and will decide which
    *  buttons to show in the wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void adjustButtons () {
      WizardStep step = wizardPanel.getCurrentStep() ;

      if ( step == welcomeStep )
         wizardPanel.showButtons ( true, false, true, false ) ;
      else if ( step == finishStep )
         wizardPanel.showButtons ( false, true, false, true ) ;
      else if ( step == processStep )
         wizardPanel.showButtons ( false, false, false, false ) ;
      else
         wizardPanel.showButtons ( true, true, true, false ) ;
   }




   /*------------------------------------------------------------*/
   /* propertyChanged
    */
   /**
    *    Called when a property we are listening
    *  for has been fired
    *  
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void propertyChanged ( int property, Object value ) {
      if ( property == PropertySystem.PROPERTY_LANGUAGE ) {
         setText() ;
      }
   }





   /*------------------------------------------------------------*/
   /* setText
    */
   /**
    *    This method will set the text on all
    *  buttons and InfoStep panels of the 
    *  wizard
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void setText () {
      wizardPanel.setNextText   ( Str.getStr ( STR_NEXT ) ) ;
      wizardPanel.setBackText   ( Str.getStr ( STR_BACK ) ) ;
      wizardPanel.setFinishText ( Str.getStr ( STR_FINISH ) ) ;
      wizardPanel.setCancelText ( Str.getStr ( STR_CANCEL ) ) ;
      setTitle ( Str.getStr ( STR_PRODUCT_EXPORT ) ) ;
      welcomeStep.setMessage ( "<html><body><center>Welcome to the Product Set Export Wizard. This wizard allows you to export products to a CSV(Comma Separated Value) file format. Press 'Next' to begin.</center></body></html>" ) ;                      
      finishStep.setMessage ( "The Export is complete" ) ;
   }

} /* ExportWizard */
