package com.ibm.nzna.projects.qit.product.productExport ;       
/*-----------------------------------------------------------*/
/*   ProcessStep                                             */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProcessStep                         */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.app.* ;
/**
 * This wizard step runs the export and shows the user what is happeneing
 *
 *
 * @version 1.0, 9/9/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProcessStep                                             */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProcessStep extends WizardStep implements AppConst, ProductConst, ProductExportListener {

   /**
     *  This is the property object which we will save our information to
     **/
   private ProductExportProperty exportProperty = null ;

   /**
     *  This is the Wizard we belong to
     **/
   private Wizard wizardPanel = null ;


   /**
     *  This is the JLabel which holds our status
     **/
   private JLabel st_STATUS = new JLabel ( "Please wait, exporting the products you selected" ) ;

   /**
     *  This is the JLabel which says 'Products Exported'
     **/
   private JLabel st_PRODUCTS = new JLabel ( "Products Exported: " ) ;

   /**
     *  This is the JLabel which says the number
     **/
   private JLabel st_TOTAL = new JLabel ( "0" ) ;

   /**
     *  This is the JLabel which shows the status
     **/
   private JLabel st_EXPORT_STATUS = new JLabel ( "" ) ;

   /**
     *  This property is TRUE if we are currently exporting
     **/
   private boolean exportInProgress = false ;

   /**
     *  Default Constructor
     **/
   public ProcessStep ( Wizard wizard, ProductExportProperty exportProperty ) {

      this.wizardPanel = wizard ;
      this.exportProperty = exportProperty ;        

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust props                                            */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_TOTAL.setFont ( new Font ( "Helv", Font.BOLD, 10 ) ) ;
      st_EXPORT_STATUS.setHorizontalAlignment ( st_EXPORT_STATUS.CENTER ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_STATUS ) ;
      add ( st_PRODUCTS ) ;
      add ( st_TOTAL ) ;    
      add ( st_EXPORT_STATUS ) ;
   }


   /**
     *  This method will lay out and size our controls
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = 0 ;
      int       y         = 30 ;


      st_STATUS.setBounds ( 30, y, size.width-60, rowHeight ) ;
      y+=rowHeight*4 ;

      st_PRODUCTS.setBounds ( 40, y, 120, rowHeight ) ;
      st_TOTAL.setBounds ( 160, y, 80, rowHeight ) ;
      y+=rowHeight*2 ;

      st_EXPORT_STATUS.setBounds ( 0, y, size.width, rowHeight ) ;
   }

   /**
     *  This method will save our information and return TRUE
     **/
   public boolean saveInfo () {
      return( true ) ;
   }


   /**
     *  This method will refresh our information
     **/
   public void refreshInfo () {
      if ( !exportInProgress ) {
         exportInProgress = true ;

         new ProductExport ( exportProperty, this ) ;
      }
   }


   /**
     *  This method will return our title text
     **/
   public String getTitle () {
      return( Str.getStr ( STR_EXPORT_WIZ_TITLE_EXPORTING ) ) ;
   }


   /**
     *  This method will be called when a product has been exported
     **/
   public void productExported ( int total ) {
      st_TOTAL.setText ( ""+total ) ;
   }

   /**
     *  This method will be called when a product status has changed
     **/
   public void productExportStatusChange ( String status ) {
      st_EXPORT_STATUS.setText ( status ) ;
   }


   /**
     *  This method will be called when the export is complete
     **/
   public void productExportComplete () {
      st_EXPORT_STATUS.setText ( Str.getStr ( STR_COMPLETE )) ;
      wizardPanel.showButtons ( false, true, false, true ) ;
      exportInProgress = false ;
   }
} /* ProcessStep */
