package com.ibm.nzna.projects.qit.product.productExport ;     
/*-----------------------------------------------------------*/
/*   ProductExport                                           */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductExport                       */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.io.* ;
import java.util.* ;
import java.sql.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.db.* ;
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.product.manager.views.* ;
import com.ibm.nzna.projects.common.quest.product.* ;
import com.ibm.nzna.projects.common.quest.brand.* ;
import com.ibm.nzna.projects.common.quest.type.* ;
/**
 * This class runs any product export and notifies progress 
 * through the ProductExportListener
 *
 * @version 1.0, 9/21/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductExport                                           */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductExport extends Object implements AppConst, ProductConst, Runnable {


   /**
     *  This is the Export property file which holds all information we need
     **/
   private ProductExportProperty exportProperty = null ;

   /**
     *  This is the Listener that is notified when we export a product
     **/
   private ProductExportListener listener = null ;

   /**
     *  This is the file output stream that will be pointing to the export file
     **/
   private FileOutputStream outFile = null ;


   /**
     *  This is the default constructor
     **/
   public ProductExport ( ProductExportProperty exportProperty, ProductExportListener listener ) {
      this.exportProperty = exportProperty ;
      this.listener       = listener ;

      new Thread ( this ).start() ;
   }



   /**
     *  This method will actually run the export
     **/
   public void run () {
      Vector productRows = null ;                

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Open the file for writing                               */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listener.productExportStatusChange ( "Opening export file" ) ;                                           
      if ( openFile() ) {


         listener.productExportStatusChange ( "Gathering list of products" ) ;                                           


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, we have to get a vector of ProductRow objects    */
         /* that we can export                                        */
         /*                                                           */
         /*-----------------------------------------------------------*/
         switch ( exportProperty.getExportType() ) {
            case ProductExportProperty.EXPORT_TYPE_VIEWABLE_PRODUCTS :
               productRows = exportProperty.getViewableProducts() ;
               break ;

            case ProductExportProperty.EXPORT_TYPE_SELECTED_PRODUCTS :
               productRows = exportProperty.getSelectedProducts() ;
               break ;

            case ProductExportProperty.EXPORT_TYPE_ALL_PRODUCTS_IN_VIEW :
               productRows = getRowsFromView() ;
               break ;
         }


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, export them                                        */       
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( productRows != null ) && ( productRows.size() > 0 ) ) {
            int i    = 0 ;
            int size = productRows.size() ;

            while ( i < size ) {
               writeProduct ( ((ProductRow)productRows.elementAt(i)) ) ;
               i++ ;
               listener.productExported ( i ) ;
            }

         } else
            listener.productExportStatusChange ( "No products to export" ) ;
      } else
         listener.productExportStatusChange ( "Could not open export file" ) ;

      listener.productExportComplete () ;
   }


   /**
     *  This method will open the file and write out the first line of content,
     * which will be a comma separted list of field names
     **/
   private boolean openFile () {
      boolean opened = false ;

      try {
         File   file          = new File ( exportProperty.getFilename() ) ;
         Vector columnNames   = exportProperty.getFieldNames() ;             

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Open the output stream                                  */
         /*                                                           */
         /*-----------------------------------------------------------*/
         outFile = new FileOutputStream ( file ) ;               


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Now, write out the names                                */
         /*                                                           */
         /*-----------------------------------------------------------*/
         if ( ( columnNames != null ) && ( columnNames.size() > 0 ) ) {
            int i       = 0 ;
            int size    = columnNames.size() ;

            while ( i < size ) {
               outFile.write ( columnNames.elementAt(i).toString().getBytes() ) ;
               if ( i < (size-1) )
                  outFile.write ( ",".getBytes() ) ;
               i++ ;
            }

            outFile.write ( "\n".getBytes() ) ;
         }

         opened = true ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
      }

      return( opened ) ;
   }



   /**
     *  This method will write out a product row to the outFile
     **/
   private void writeProduct ( ProductRow productRow ) {
      Vector fieldNames = exportProperty.getFieldNames() ;
      int    i          = 0 ;
      int    size       = fieldNames.size() ;          

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Read in the product                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      listener.productExportStatusChange ( "Reading "+productRow.getDescription() ) ;
      productRow.readProduct ( false ) ;         

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Write out the data                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      while ( i < size ) {
         try {
            outFile.write ( getFieldString ( productRow, fieldNames.elementAt(i).toString() ).getBytes() ) ;
            if ( i < (size-1) )
               outFile.write ( ",".getBytes() ) ;
         } catch ( Exception e ) {
            LogSystem.log ( 1, "Could not write product out. Product Ind:"+productRow.getProductInd() ) ;
         }
         i++ ;
      }
      try {
         outFile.write ( "\n".getBytes() ) ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, "Could not write product out. Product Ind:"+productRow.getProductInd() ) ;
      }

   }


   /**
     *  This method will return the data that should be written out for a particular product
     * for a particular field
     **/
   private String getFieldString ( ProductRow productRow, String fieldName ) {
      String fieldString = "" ;

      if ( fieldName.equals ( Str.getStr ( STR_PRODUCT_NUMBER ) ) )
         fieldString = productRow.getProduct().getProdNum() ;

      else if ( fieldName.equals ( Str.getStr ( STR_PARTNUM ) ) ) {
         if ( productRow.getProduct().getPartNumber() != null )
            fieldString = productRow.getProduct().getPartNumber().toString() ;
         else
            fieldString = "NA" ;
      }

      if ( fieldName.equals ( Str.getStr ( STR_DESCRIPTION ) ) )
         fieldString = productRow.getProduct().getDescription() ;

      else if ( fieldName.equals ( Str.getStr ( STR_BRAND ) ) ) {
         if ( productRow.getProduct().getBrand() != null )
            fieldString = productRow.getProduct().getBrand().toString() ;
         else
            fieldString = "NA" ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_FAMILY ) ) ) {
         if ( productRow.getProduct().getFamily() != null )
            fieldString = productRow.getProduct().getFamily().toString() ;
         else
            fieldString = "NA" ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_MACHINE ) ) ) {
         if ( productRow.getProduct().getMachine() != null )
            fieldString = productRow.getProduct().getMachine().toString() ;
         else
            fieldString = "NA" ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_MODEL ) ) ) {
         if ( productRow.getProduct().getModel() != null )
            fieldString = productRow.getProduct().getModel().toString() ;
         else
            fieldString = "NA" ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_SUBCATEGORY ) ) ) {
         if ( productRow.getProduct().getSubCategory() != null )
            fieldString = productRow.getProduct().getSubCategory().toString() ;
         else
            fieldString = "NA" ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_DATES ) ) ) {
         fieldString = "NA" ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_XREF ) ) ) {
         fieldString = "NA" ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_COUNTRY ) ) ) {
         fieldString = "NA" ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_LAST_TOUCHED_BY ) ) ) {
         fieldString = UserSystem.getNameFromUserId ( productRow.getProduct().getLastTouchedBy() ) ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_LAST_TOUCHED_ON ) ) ) {
         fieldString = DateSystem.prettyDateFromStamp ( productRow.getProduct().getLastTouchedOn() ) ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_CREATED_ON ) ) ) {
         fieldString = DateSystem.prettyDateFromStamp ( productRow.getProduct().getCreatedOn() ) ;
      }

      else if ( fieldName.equals ( Str.getStr ( STR_CREATED_BY ) ) ) {
         fieldString = UserSystem.getNameFromUserId ( productRow.getProduct().getLastTouchedBy() ) ;
      }

      return( fieldString ) ;
   }




   /**
     *  This method will read in the ProductRow objects from the view that is defined in the
     * ProductExportProperty object
     **/
   private Vector getRowsFromView () {
      String    viewName    = exportProperty.getPMView().getName() ;
      Vector    productRows = new Vector ( 1 ) ;      
      String    sql         = null ;
      SQLMethod sqlMethod   = new SQLMethod ( DatabaseSystem.QUEST, "getRowsFromView", 5 ) ;


      try {
         Statement stmt   = sqlMethod.createStatement() ;
         ResultSet rs     = null ;
         boolean   option = Brands.getOptionBrand().getInd() == exportProperty.getPMView().getBrand().getInd() ; 
         boolean   draft  = ( viewName.indexOf ( Str.getStr ( STR_DRAFT ) ) != -1 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   First, form the SQL for the query                       */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sql = exportProperty.getPMView().getFullSQL () ;                           


         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   If the SQL is good, read in the data                    */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery (sql ) ;

         while ( rs.next() ) {
            ProductRow productRow = new ProductRow ( rs.getInt ( 1 ),                                             //Product Ind
                                                     rs.getInt ( 1 ) ,                                            //Existing Product Ind
                                                     option ? rs.getString(8) : rs.getString ( 2 ),               //PRoduct number or Part Number
                                                     rs.getString ( 3 ).trim(),                                   //Description
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 4 ).trim() ),//Created on
                                                     UserSystem.getNameFromUserId ( rs.getString ( 5 ).trim()),   //Created     By
                                                     "",                                                          //Actual Announce Date
                                                     "",                                                          //Target Announce Date
                                                     DateSystem.prettyDateFromStamp ( rs.getString ( 6 ).trim() ),//LastTouched on
                                                     UserSystem.getNameFromUserId ( rs.getString ( 7 ).trim()),  //LAstTouched By
                                                     draft ) ;
            productRows.addElement ( productRow ) ;
         }

         rs.close() ;
      } catch ( Exception e ) {
         LogSystem.log ( 1, e, false ) ;
         sqlMethod.rollBack () ;
      }

      sqlMethod.close() ;
      return( productRows ) ;
   }
} /* ProductExport */
