package com.ibm.nzna.projects.qit.product.productExport ;                          
/*-----------------------------------------------------------*/
/*   ProductExportProperty                                   */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductExportProperty               */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import java.util.* ;
import com.ibm.nzna.projects.qit.product.manager.views.* ;
/**
 * This object holds the different properties that an export can 
 * have.
 *
 * @version 1.0, 9/12/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductExportProperty                                   */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductExportProperty extends Object {


   public static final int EXPORT_TYPE_ALL_PRODUCTS_IN_VIEW = 1 ;
   public static final int EXPORT_TYPE_SELECTED_PRODUCTS    = 2 ;
   public static final int EXPORT_TYPE_VIEWABLE_PRODUCTS    = 3 ;


   /**
     *  This is the filename of the file to export to
     **/
   private String filename    = null ;

   /**
     *  This is the vector of products that were selceted or
     * in the view from Product Management. This does not include
     * the products that are read from the database when a full
     * export of a view is done
     **/
   private Vector productVec = null ;


   /**
     *  This is the vector of all selected products in product mgmt.
     **/
   private Vector selectedProducts = null ;

   /**
     *  This is the vector of all viewable products
     **/
   private Vector viewableProducts = null;

   /**
     *  This is the current view
     **/
   private PMView view = null ;

   /**
     *  This vector contains a list of Strings. Each string represents
     * a field that is to be exported
     **/
   private Vector fields = null ;


   /**
     *  This is the number of products that were exported
     **/
   private int exportedProducts = 0 ;

   /**
     *  This is the type of export that is to be done
     **/
   private int exportType = 0 ;


   /**
     *  This method will set the filename to export
     *
     * @param filename The filename
     **/
   public void setFilename ( String filename ) {
      this.filename = filename ;
   }

   /**
     *  This method will return the filename for the
     * export
     *
     * @return String
     **/
   public String getFilename ( ) {
      return( filename ) ;
   }


   /**
     *  This method will set the vector of products
     *
     * @param productVec The vector of products
     **/
   public void setProductVec ( Vector productVec ) {
      this.productVec =  productVec ;
   }

   /**
     *  This method will return the vector of products
     **/
   public Vector getProductVec ( ) {
      return( productVec ) ;
   }


   /**
     *  This method returns the vector of field names
     * chosen by the user
     **/
   public Vector getFieldNames () {
      return( fields ) ;
   }

   /**
     *  This method will set the vector of field names
     *
     * @param fields The Vector of fields
     **/
   public void setFieldNames ( Vector fields ) {
      this.fields = fields ;
   }

   /**
     *  This method sets the number of products that were
     * exported
     *
     * @param int exportedProducts The Number of exported products
     **/
   public void setExportedProducts ( int exportedProducts ) {
      this.exportedProducts = exportedProducts ;
   }


   /**
     *  This method will return the number of exported products
     *
     * @return int
     **/
   public int getExportedProducts () {
      return( exportedProducts ) ;
   }

   /**
     *  This method sets the type of export to do
     *
     * @param int exportType The type of export
     **/
   public void setExportType ( int exportType ) {
      this.exportType = exportType ;
   }


   /**
     *  This method will return the export type
     *
     * @return int
     **/
   public int getExportType () {
      return( exportType ) ;
   }


   /**
     *  This method will set the vector of viewable products
     **/
   public void setViewableProducts ( Vector viewableProducts ) {
      this.viewableProducts = viewableProducts ;
   }

   /**
     *  This methodw will return a Vector of ProductRow objects
     * that are currently viewable
     **/
   public Vector getViewableProducts ( ) {
      return ( viewableProducts ) ;
   }


   /**
     *  This method will set the vector of selected products
     **/
   public void setSelectedProducts ( Vector selectedProducts ) {
      this.selectedProducts = selectedProducts ;
   }

   /**
     *  This methodw will return a Vector of ProductRow objects
     * that are currently selected
     **/
   public Vector getSelectedProducts ( ) {
      return ( selectedProducts ) ;
   }


   /**
     *  This method will set our current view
     **/
   public void setPMView ( PMView view ) {
      this.view = view ;
   }

   public PMView getPMView ( ) {
      return ( view ) ;
   }
} /* ProductExportProperty */
