package com.ibm.nzna.projects.qit.product.productExport ;       
/*-----------------------------------------------------------*/
/*   ProductFieldsStep                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductFieldsStep                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import javax.swing.event.* ;
import java.awt.* ;
import java.awt.event.* ;
import java.util.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import com.ibm.nzna.shared.sort.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.app.* ;

/**
 * This wizard step allows the user to select what fields are
 * to be exported
 *
 *
 * @version 1.0, 9/9/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductFieldsStep                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductFieldsStep extends WizardStep implements ListSelectionListener, ActionListener, AppConst, ProductConst {

   /**
     *  This is the property object which we will save our information to
     **/
   private ProductExportProperty exportProperty = null ;

   /**
     *  This is the JLabel which shows the available
     **/
   private JLabel st_AVAILABLE = new JLabel ( Str.getStr ( STR_AVAILABLE ) ) ;

   /**
     *  This is the JLabel which shows the exported fields
     **/
   private JLabel st_EXPORTED = new JLabel ( Str.getStr ( STR_EXPORTED_FIELDS ) ) ;

   /**
     *  This is the List box which holds available fields
     **/
   private DList lb_AVAILABLE = new DList ( ) ;

   /**
     *  This is the list box which holds the exported Fields
     **/
   private DList lb_EXPORTED = new DList ( ) ;

   /**
     *  This is the JScrollPane which holds the available list
     **/
   private JScrollPane scr_AVAILABLE = new JScrollPane ( lb_AVAILABLE ) ;

   /**
     *  This is the JScrollPane which holds the exported fields
     **/
   private JScrollPane scr_EXPORTED = new JScrollPane ( lb_EXPORTED ) ;

   /**
     *  This is the add button
     **/
   private DButton pb_ADD = new DButton ( ">>" ) ;

   /**
     *  This is the remove button
     **/
   private DButton pb_REMOVE = new DButton ( "<<" ) ;

   /**
     *  This is the help text
     **/
   private JTextArea mle_OUTPUT = new JTextArea ( "" ) ;

   /**
     *  This is the data model for the available list
     **/
   private DefaultListModel availableModel = new DefaultListModel () ;

   /**
     *  This is the data model for the exported list
     **/
   private DefaultListModel exportedModel = new DefaultListModel () ;

   /**
     *  This is the up button
     **/
   private DButton pb_UP = new DButton ( Str.getStr  ( STR_UP ) ) ;

   /**
     *  This is the down button
     **/
   private DButton pb_DOWN = new DButton ( Str.getStr ( STR_DOWN ) ) ;

   /**
     * This is the JScrollPane which holds the description
     **/
   private JScrollPane scr_OUTPUT = new JScrollPane ( mle_OUTPUT ) ;


   /**
     *  Default Constructor
     **/
   public ProductFieldsStep ( Wizard wizard, ProductExportProperty exportProperty ) {

      setWizard ( wizard ) ;
      this.exportProperty = exportProperty ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      mle_OUTPUT.setBorder        ( GUISystem.loweredBorder ) ;
      mle_OUTPUT.setFont          ( pb_REMOVE.getFont() ) ;
      mle_OUTPUT.setEditable      ( false ) ;
      mle_OUTPUT.setWrapStyleWord ( true ) ;
      mle_OUTPUT.setLineWrap      ( true ) ;
      lb_AVAILABLE.setModel ( availableModel ) ;
      lb_EXPORTED.setModel ( exportedModel ) ;
      populateAvailable() ;    

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   ADd listeners                                           */
      /*                                                           */
      /*-----------------------------------------------------------*/
      lb_AVAILABLE.addListSelectionListener ( this ) ;
      lb_EXPORTED.addListSelectionListener ( this ) ;
      pb_UP.addActionListener ( this ) ;
      pb_DOWN.addActionListener ( this ) ;
      pb_ADD.addActionListener ( this ) ;
      pb_REMOVE.addActionListener ( this ) ;
      lb_AVAILABLE.addActionListener ( this ) ;
      lb_EXPORTED.addActionListener ( this ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( st_AVAILABLE ) ;
      add ( st_EXPORTED ) ;
      add ( scr_AVAILABLE ) ;
      add ( pb_ADD ) ;
      add ( pb_REMOVE ) ;
      add ( scr_EXPORTED ) ;
      add ( pb_UP ) ;
      add ( pb_DOWN ) ;
      add ( scr_OUTPUT ) ;

   }


   /**
     *  This method will lay out and size our controls
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = 5 ;
      int       y         = 5 ;
      int       half      = size.width/2 ;

      st_AVAILABLE.setBounds ( x, y, 150, rowHeight ) ;
      st_EXPORTED.setBounds ( half+20, y, 150, rowHeight ) ;
      y+=rowHeight ;

      scr_AVAILABLE.setBounds ( x, y, half-x-20, size.height-90 ) ;
      scr_EXPORTED.setBounds  ( half+20, y, half-x-20, size.height-110 ) ;
      pb_UP.setBounds ( half+40, size.height-80, 60, rowHeight ) ;
      pb_DOWN.setBounds ( half+105, size.height-80, 60, rowHeight ) ;
      y+=rowHeight*3 ;

      pb_ADD.setBounds ( half-15, y, 30, 25 ) ;
      y+=30 ;
      pb_REMOVE.setBounds ( half-15, y, 30, 25 ) ;

      y=size.height-60 ;

      scr_OUTPUT.setBounds ( x, y, size.width-(x*2), size.height-y-5 ) ;


   }

   /**
     *  This method will save our information and return TRUE
     **/
   public boolean saveInfo () {
      Vector   fields = WinUtil.getListData ( lb_EXPORTED ) ;
      boolean  rc     = false ;

      if ( ( fields != null ) && ( fields.size() > 0 ) ) {
         exportProperty.setFieldNames ( fields ) ;
         rc = true ;
      } else
         GUISystem.printBox ( WinUtil.getParentJDialog ( this ),
                              Str.getStr ( STR_INFO ),
                              Str.getStr ( STR_PLEASE_SELECT_FIELD ) ) ;
      return( rc ) ;
   }


   /**
     *  This method will refresh our information
     **/
   public void refreshInfo () {
   }


   /**
     *  This method will return our title text
     **/
   public String getTitle () {
      return( Str.getStr ( STR_EXPORT_WIZ_TITLE_FIELDS ) ) ;
   }


   /**
     *  This method will return the vector of fields that is available
     * to export
     *
     * @return Vector
     **/
   private void populateAvailable() {
      Vector fields = new Vector ( 1 ) ;

      fields.addElement ( Str.getStr ( STR_PRODUCT_NUMBER ) ) ;
      fields.addElement ( Str.getStr ( STR_PARTNUM ) ) ;
      fields.addElement ( Str.getStr ( STR_DESCRIPTION ) ) ;
      fields.addElement ( Str.getStr ( STR_BRAND ) ) ;
      fields.addElement ( Str.getStr ( STR_FAMILY ) ) ;
      fields.addElement ( Str.getStr ( STR_MACHINE ) ) ;
      fields.addElement ( Str.getStr ( STR_MODEL ) ) ;
      fields.addElement ( Str.getStr ( STR_SUBCATEGORY ) ) ;
      fields.addElement ( Str.getStr ( STR_DATES ) ) ;
      fields.addElement ( Str.getStr ( STR_XREF ) ) ;
      fields.addElement ( Str.getStr ( STR_COUNTRY ) ) ;
      fields.addElement ( Str.getStr ( STR_CREATED_ON ) ) ;
      fields.addElement ( Str.getStr ( STR_CREATED_BY ) ) ;
      fields.addElement ( Str.getStr ( STR_LAST_TOUCHED_ON ) ) ;
      fields.addElement ( Str.getStr ( STR_LAST_TOUCHED_BY ) ) ;
      new QuickSort ( fields ) ;

      WinUtil.setListData ( fields, availableModel ) ;
   }


   /**
     *  This method will be called when the user hits a button
     **/
   public void actionPerformed ( ActionEvent e ) {

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add                                                     */
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( e.getSource() == pb_ADD ) || ( e.getSource() == lb_AVAILABLE ) )
         addFields() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Remove                                                  */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( ( e.getSource() == pb_REMOVE ) || ( e.getSource() == lb_EXPORTED ) )
         removeFields() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Up                                                      */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_UP )
         WinUtil.moveItemUp ( lb_EXPORTED ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Down                                                    */
      /*                                                           */
      /*-----------------------------------------------------------*/
      else if ( e.getSource() == pb_DOWN )
         WinUtil.moveItemDown ( lb_EXPORTED ) ;




   }


   public void valueChanged ( ListSelectionEvent e ) {
      pb_ADD.setEnabled ( lb_AVAILABLE.getSelectedValue() != null ) ;
      pb_REMOVE.setEnabled ( lb_EXPORTED.getSelectedValue() != null ) ;

      if ( e.getSource() == lb_AVAILABLE ) {
         mle_OUTPUT.setText ( getFieldDescription ( (String)lb_AVAILABLE.getSelectedValue() ) ) ;
      }
   }


   /**
     *  This method will add all currently selected fields
     **/
   private void addFields () {
      Object[] fields = lb_AVAILABLE.getSelectedValues() ;

      if ( ( fields != null ) && ( fields.length > 0 ) ) {
         for ( int i= 0; i < fields.length; i++ ) {
            exportedModel.addElement ( fields[i] ) ;
            availableModel.removeElement ( fields[i] ) ;
         }
      }

   }


   /**
     *  This method will remove all currently selected fields
     **/
   private void removeFields () {
      Object[] fields = lb_EXPORTED.getSelectedValues() ;

      if ( ( fields != null ) && ( fields.length > 0 ) ) {
         for ( int i= 0; i < fields.length; i++ ) {
            exportedModel.removeElement ( fields[i] ) ;
            availableModel.addElement ( fields[i] ) ;
         }
      }

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Now, alphabetize!                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      Vector temp = WinUtil.getListData ( lb_AVAILABLE ) ;
      new QuickSort ( temp ) ;
      WinUtil.setListData ( temp, availableModel ) ;

   }


   /**
     *  This method will return the field description of the field passed in
     **/
   private String getFieldDescription ( String field ) {
      String retStr = null ;

      if ( field.equals ( Str.getStr ( STR_DATES ) ) )
         retStr = "When exporting dates, each value will read 'Date Name: Date: Country'. For example, if a product"+
                  "has a GA Date of 1/1/2001 in Russia, the value in the file written would read 'GA Date: 1/1/2001:Russia'" ;

      else if ( field.equals ( Str.getStr ( STR_XREF ) ) )
         retStr = "When exported Cross Reference Product Numbers, each value will read 'Cross Reference Name: number'. For "+
                  "example, if a product has a Cross Reference Product Number for 'CRU' set to '123456' then the file written "+
                  "would be 'CRU: 123456'" ;
      else if ( field.equals ( Str.getStr ( STR_COUNTRY ) ) )
         retStr = "When exporting countries, each country will be separated by a semi-colon (;)" ;

      else if ( field.equals ( Str.getStr ( STR_CREATED_ON ) ) )
         retStr = "This is the date that the product or product draft was created on" ;
      
      else if ( field.equals ( Str.getStr ( STR_CREATED_BY ) ) )
         retStr = "This is the person who created the product or product draft" ;

      else if ( field.equals ( Str.getStr ( STR_DESCRIPTION ) ) )
         retStr = "This is the description of the product" ;

      else if ( field.equals ( Str.getStr ( STR_BRAND ) ) )
         retStr = "This is the IBM Brand (Aptiva, Options etc..) that the product belongs to" ;

      else if ( field.equals ( Str.getStr ( STR_FAMILY ) ) )
         retStr = "This is the IBM Family (Thinkpad 380, Hard Drives etc..) that the product belongs to" ;

      else if ( field.equals ( Str.getStr ( STR_MACHINE ) ) )
         retStr = "This is the machine type of the product (Only for Systems and Monitors)" ;

      else if ( field.equals ( Str.getStr ( STR_SUBCATEGORY ) ) )
         retStr = "This is the Sub Category of the Option (Only for options)" ;

      else if ( field.equals ( Str.getStr ( STR_MODEL ) ) )
         retStr = "This is the product's model number. (Only for Systems and Monitors)" ;

      else if ( field.equals ( Str.getStr ( STR_PART_NUMBER ) ) )
         retStr = "This is the product's part number. (Only for Options)" ;
      
      else if ( field.equals ( Str.getStr ( STR_LAST_TOUCHED_ON ) ) )
         retStr = "This is the date that the product or product draft was last changed" ;
      
      else if ( field.equals ( Str.getStr ( STR_LAST_TOUCHED_BY ) ) )
         retStr = "This is the person who last saved the product or product draft" ;

      else
         retStr = "The "+field+" of the product" ;

      return( retStr ) ;
   }
} /* ProductFieldsStep */

