package com.ibm.nzna.projects.qit.product.productExport ;       
/*-----------------------------------------------------------*/
/*   ProductSetStep                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductSetStep                      */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import javax.swing.* ;
import java.awt.* ;
import com.ibm.nzna.shared.gui.wizard.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.projects.qit.app.* ;
/**
 * This wizard step allows the user to select what set of products to export 
 *
 *
 * @version 1.0, 9/9/2001 
 * @author  David Lentz
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductSetStep                                          */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductSetStep extends WizardStep implements AppConst, ProductConst {

   /**
     *  This is the property object which we will save our information to
     **/
   private ProductExportProperty exportProperty = null ;

   /**
     *  Selected Radio Button
     **/
   private JRadioButton rb_SELECTED_PRODUCTS = null ;

   /**
     *  This is the viewable radio button
     **/
   private JRadioButton rb_VIEWABLE_PRODUCTS = null ;

   /**
     *  This is the all radio button
     **/
   private JRadioButton rb_ALL_PRODUCTS = null ;

   /**
     *  Default Constructor
     **/
   public ProductSetStep ( Wizard wizard, ProductExportProperty exportProperty ) {
      ButtonGroup group = new ButtonGroup () ;

      setWizard ( wizard ) ;
      this.exportProperty = exportProperty ;     

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      rb_SELECTED_PRODUCTS = new JRadioButton ( Str.getStr ( STR_ONLY_PRODUCTS_I_SELECTED ) ) ;
      rb_VIEWABLE_PRODUCTS = new JRadioButton ( Str.getStr ( STR_ONLY_PRODUCTS_VIEWABLE ) ) ;
      rb_ALL_PRODUCTS      = new JRadioButton ( Str.getStr ( STR_ALL_PRODUCTS_IN_VIEW ) ) ;  

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust props                                            */   
      /*                                                           */
      /*-----------------------------------------------------------*/
      rb_ALL_PRODUCTS.setSelected ( true ) ;
      
      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      setLayout ( null ) ;
      add ( rb_SELECTED_PRODUCTS ) ;
      add ( rb_VIEWABLE_PRODUCTS ) ;
      add ( rb_ALL_PRODUCTS ) ;                       

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the JRadioButtons to a group                        */
      /*                                                           */
      /*-----------------------------------------------------------*/
      group.add ( rb_SELECTED_PRODUCTS ) ;
      group.add ( rb_VIEWABLE_PRODUCTS ) ;
      group.add ( rb_ALL_PRODUCTS ) ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust editable                                         */      
      /*                                                           */
      /*-----------------------------------------------------------*/
      if ( ( exportProperty.getViewableProducts() == null ) ||
           ( exportProperty.getViewableProducts().size() == 0 ) )
         rb_VIEWABLE_PRODUCTS.setEnabled ( false ) ;
      if ( ( exportProperty.getSelectedProducts() == null ) ||
           ( exportProperty.getSelectedProducts().size() == 0 ) )
         rb_SELECTED_PRODUCTS.setEnabled ( false ) ;
      if ( exportProperty.getPMView() == null )
         rb_ALL_PRODUCTS.setEnabled ( false ) ;
      
   }


   /**
     *  This method will lay out and size our controls
     **/
   public void doLayout () {
      Dimension size      = getSize() ;
      int       rowHeight = GUISystem.getRowHeight () ;
      int       x         = 20 ;
      int       y         = 30 ;

      rb_SELECTED_PRODUCTS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+2 ;

      rb_VIEWABLE_PRODUCTS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+2 ;

      rb_ALL_PRODUCTS.setBounds ( x, y, size.width-(x*2), rowHeight ) ;
      y+=rowHeight+2 ;
   }

   /**
     *  This method will save our information and return TRUE
     **/
   public boolean saveInfo () {
      if ( rb_ALL_PRODUCTS.isSelected() )
         exportProperty.setExportType ( ProductExportProperty.EXPORT_TYPE_ALL_PRODUCTS_IN_VIEW ) ;
      else if ( rb_SELECTED_PRODUCTS.isSelected() )
         exportProperty.setExportType ( ProductExportProperty.EXPORT_TYPE_SELECTED_PRODUCTS ) ;
      else if ( rb_VIEWABLE_PRODUCTS.isSelected() )
         exportProperty.setExportType ( ProductExportProperty.EXPORT_TYPE_VIEWABLE_PRODUCTS ) ;
      return( true ) ;
   }


   /**
     *  This method will refresh our information
     **/
   public void refreshInfo () {
      switch ( exportProperty.getExportType() ) {
         case ProductExportProperty.EXPORT_TYPE_ALL_PRODUCTS_IN_VIEW :
            rb_ALL_PRODUCTS.setSelected ( true ) ;
            break ;

         case ProductExportProperty.EXPORT_TYPE_SELECTED_PRODUCTS :
            rb_SELECTED_PRODUCTS.setSelected ( true ) ;
            break ;

         case ProductExportProperty.EXPORT_TYPE_VIEWABLE_PRODUCTS :
            rb_VIEWABLE_PRODUCTS.setSelected ( true ) ;
            break ;
      }
   }


   /**
     *  This method will return our title text
     **/
   public String getTitle () {
      return( Str.getStr ( STR_EXPORT_WIZ_TITLE_SET ) ) ;
   }


} /* ProductSetStep */
