package com.ibm.nzna.projects.qit.product.productImport ;
/*-----------------------------------------------------------*/
/*   ProductImportDlg                                        */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductImportDlg                    */
/*                                                           */
/*                                                           */
/*         @Copyright  2000 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
import com.ibm.nzna.projects.qit.app.* ;
import com.ibm.nzna.projects.qit.gui.* ;
import com.ibm.nzna.projects.qit.product.* ;
import com.ibm.nzna.shared.gui.* ;
import com.ibm.nzna.shared.util.* ;
import javax.swing.* ;
import java.awt.* ;
import java.io.* ;
import java.awt.event.* ;

/**
 * This dialog asks the users which parameters to send to the ProductImport batch
 * .
 * This class is based off of the HTMLProductImportDlg created by David Lentz on
 * 4/4/2000
 *
 * @version 1.0, 5/9/2001
 * @author  Tracey Watkins
 */



/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductImportDlg                                    */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductImportDlg extends JDialog implements ActionListener, ProductConst, AppConst {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Private Members                                         */
   /*                                                           */
   /*-----------------------------------------------------------*/

   /**
    * This is the Label for the filename entry field. 
    */
   private JLabel       st_FILENAME     = null ;

   /**
    * This is the filename entry field. 
    */
   private JTextField   ef_FILENAME     = null ;

   /**
    * This is the browse push button. 
    */
   private DButton      pb_BROWSE       = null ;


   /**
    * This is the Label for buttongroup radio buttons.  It is for the "What type of Import?" question.
    */
   private JLabel       st_ACTIONTYPE     = null ;

   /**
    * This is the insert radio button. 
    */
   private JRadioButton rb_INSERT       = null ;

   /**
    * This is the update radio button. 
    */
   private JRadioButton rb_UPDATE       = null ;

   /**
    * This is the insert/update radio button. 
    */
   private JRadioButton rb_INSERTUPDATE = null ;

   /**
    * This is the Label for buttongroup2 radio buttons.  It is for the "Import what type of IBM product?" question.
    */

   private JLabel       st_IMPORTTYPE     = null ;

   /**
    * This is the systems radio button. 
    */
   private JRadioButton	rb_SYSTEMS		= null;

   /**
    * This is the monitors radio button.
    */
   private JRadioButton	rb_MONITORS		= null;

   /**
    * This is the options radio button. 
    */
   private JRadioButton	rb_OPTIONS		= null;

   /**
    * This is the cancel push button. 
    */
   private DButton      pb_CANCEL       = null ;

   /**
    * This is the ok push button. 
    */
   private DButton      pb_OK           = null ;

   /**
    * This is the BMP Product image.
    */
   private JLabel       st_IMAGE        = null ;

    /**
    * This is the button panel that holds the OK and CANCEL buttons.
    */
   private ButtonPanel  buttonPanel     = null ;

   /**
    * This is the button group that holds the action type (INSERT, UPDATE, INSERT/UPDATE) radio buttons.
    */
   private ButtonGroup  buttonGroup     = null ;

   /**
    * This is the button group that holds the import type (SYSTEMS, MONITORS, OPTIONS) radio buttons.
    */
   private ButtonGroup  buttonGroup2     = null ;

   /**
    * This is the name of the file being imported.
    */
   private String       importFilename  = null ;

   /**
    * This is the insert only flag.  If this flag is set, then only inserts will be performed.
    */
   private boolean      insert          = false ;

   /**
    * This is the update only flag.  If this flag is set, then only updates will be performed.
    */
   private boolean      update          = false ;

   /**
    * This is the insert and update flag.  If this flag is set, then both inserts and updates will be performed.
    */
   private boolean      insertUpdate    = false ;

   /**
    * This is the systems only flag.  If this is set to true, then only systems will be imported.
    */
   private boolean		systems			= false ;

   /**
    * This is monitors only flag.  If this is set to true, the only monitors will be imported.
    */
   private boolean		monitors      		= false ;

   /**
    * This is the options only flag.  If this is set to true, then only options will be imported.
    */
   private boolean		options			= false;






   /*------------------------------------------------------------*/
   /* Constructor
    */
   /**
    *    This method will create and initialize the window
    *  and will show it. Please call 'getXXXResult'
    *  to get the data that the user entered. This method does not
    *  return until the window is closed
    *
    *  @param parent The parent to show
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public ProductImportDlg ( JFrame parent ) {
      super ( parent, Str.getStr ( STR_PRODUCT_IMPORT ), true ) ;
      initialize() ;
      WinUtil.centerChildInParent ( this, parent ) ;
      setVisible ( true ) ;
   }






   /*------------------------------------------------------------*/
   /* initialize
    */
   /**
    *    This method will initialize our window
    *  
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void initialize () {    
      Container contentPane = getContentPane() ;

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create our GUI                                          */
      /*                                                           */
      /*-----------------------------------------------------------*/
      st_FILENAME     = new JLabel ( Str.getStr ( STR_FILENAME ) ) ;
      st_ACTIONTYPE   = new JLabel ( Str.getStr ( STR_ACTIONTYPE ) ) ;
      st_IMPORTTYPE   = new JLabel ( Str.getStr ( STR_IMPORTTYPE ) ) ;
      ef_FILENAME     = new JTextField () ;
      pb_BROWSE       = new DButton ( Str.getStr ( STR_BROWSE ) ) ;
      rb_INSERT       = new JRadioButton ( Str.getStr ( STR_INSERT ) ) ;
      rb_UPDATE       = new JRadioButton ( Str.getStr ( STR_UPDATE ) ) ;
      rb_INSERTUPDATE = new JRadioButton ( Str.getStr ( STR_INSERT_UPDATE ) ) ;
      rb_SYSTEMS      = new JRadioButton ( Str.getStr ( STR_SYSTEMS ) ) ;
      rb_MONITORS     = new JRadioButton ( Str.getStr ( STR_MONITORS ) ) ;
      rb_OPTIONS      = new JRadioButton ( Str.getStr ( STR_OPTIONS ) ) ;
      pb_OK           = new DButton ( Str.getStr ( STR_OK ) ) ;
      pb_CANCEL       = new DButton ( Str.getStr ( STR_CANCEL ) ) ;
      st_IMAGE        = new JLabel ( ImageSystem.getImageIcon ( this, ImageSystem.PRODUCT_LARGE ) ) ;  
      buttonPanel     = new ButtonPanel () ;                                 

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the group for the insert/update type radio       */
      /*   buttons   											   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonGroup = new ButtonGroup () ;
      buttonGroup.add ( rb_INSERT ) ;
      buttonGroup.add ( rb_UPDATE ) ;
      buttonGroup.add ( rb_INSERTUPDATE ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Create the group for the systems/options import type    */
      /*   radio buttons 										   */
      /*                                                           */
      /*-----------------------------------------------------------*/
      buttonGroup2 = new ButtonGroup () ;
      buttonGroup2.add ( rb_SYSTEMS ) ;
      buttonGroup2.add ( rb_MONITORS ) ;
      buttonGroup2.add ( rb_OPTIONS ) ;


      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add the action listeners                                */               
      /*                                                           */
      /*-----------------------------------------------------------*/
      pb_OK.addActionListener     ( this ) ;
      pb_CANCEL.addActionListener ( this ) ;
      pb_BROWSE.addActionListener ( this ) ;
      

      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Adjust properties                                       */
      /*                                                           */
      /*-----------------------------------------------------------*/
      GUISystem.setPreferredButton ( pb_OK ) ;
      GUISystem.setPreferredButton ( pb_CANCEL ) ;



      /*-----------------------------------------------------------*/
      /*                                                           */
      /*   Add our GUI                                             */
      /*                                                           */
      /*-----------------------------------------------------------*/
      contentPane.setLayout ( null ) ;
      contentPane.add ( st_IMAGE ) ;
      contentPane.add ( st_FILENAME ) ;
      contentPane.add ( ef_FILENAME ) ;
      contentPane.add ( pb_BROWSE ) ;
      contentPane.add ( rb_INSERT ) ;
      contentPane.add ( rb_UPDATE ) ;
      contentPane.add ( rb_INSERTUPDATE ) ;
      contentPane.add ( rb_SYSTEMS ) ;
      contentPane.add ( rb_MONITORS ) ;
      contentPane.add ( rb_OPTIONS ) ;
      contentPane.add ( buttonPanel ) ;
      buttonPanel.add ( pb_OK ) ;
      buttonPanel.add ( pb_CANCEL ) ;


      setSize ( 300, 220 ) ;

   }





   /*------------------------------------------------------------*/
   /* doLayout 
    */
   /**
    *    This method will lay out and position all of
    *  our controls
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void doLayout () {
      int       rowHeight     = GUISystem.getRowHeight () ;
      int       x             = 40 ;
      int       y             = 5 ;
      Dimension size          = getSize() ;

      super.doLayout() ;
      st_IMAGE.setBounds ( 0,5,32,32 ) ;

      st_FILENAME.setBounds ( x, y, 150, rowHeight ) ;
      y+=rowHeight ;

      ef_FILENAME.setBounds ( x, y, 140, rowHeight ) ;
      pb_BROWSE.setBounds ( x+142, y, 60, rowHeight ) ;
      y+=(rowHeight*2) ;

      st_ACTIONTYPE.setBounds ( x, y, 150, rowHeight ) ;
      st_IMPORTTYPE.setBounds ( x, y, 60, rowHeight );
      y+=rowHeight ;

      rb_INSERT.setBounds ( x, y, 150, rowHeight ) ;
      rb_SYSTEMS.setBounds ( x+150, y, 60, rowHeight ) ;
      y+=rowHeight ;

      rb_UPDATE.setBounds ( x, y, 150, rowHeight ) ;
      rb_MONITORS.setBounds ( x+150, y, 60, rowHeight ) ;
      y+=rowHeight ;

      rb_INSERTUPDATE.setBounds ( x, y, 150, rowHeight ) ;
      rb_OPTIONS.setBounds ( x+150, y, 60, rowHeight ) ;
      y+=rowHeight ;

      buttonPanel.setBounds ( 5, size.height-50, size.width-10, 25 ) ;
   }


   /*------------------------------------------------------------*/
   /* browseFiles
    */
   /**
    *    This method will bring up the file open
    *  dialog and allow the user to select a file,
    *  it will then populate the entry field with the file they select
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   private void browseFiles() {
      FileDialog fd       = new FileDialog ( GUISystem.getParentDefWin ( this ), Str.getStr ( STR_SELECT_FILE ), FileDialog.LOAD ) ;
      String     filename = null ;
      String     dir      = null ;

      try {
         fd.setDirectory ( PropertySystem.getString ( PropertySystem.PROPERTY_IMPORT_PRODUCT_DIR ) ) ;
         fd.show() ;
         filename = fd.getDirectory()+fd.getFile() ;

         if ( ( filename != null ) && ( filename.length() > 0 ) )
            ef_FILENAME.setText ( filename ) ;
      } catch ( Exception e ) {
      }
   }





   /*------------------------------------------------------------*/
   /* getInsert
    */
   /**
    *    This method will return our insert value
    *  
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getInsert () {
      return( insert ) ;
   }




   /*------------------------------------------------------------*/
   /* getUpdate
    */
   /**
    *    This method will return our update value
    *  
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getUpdate () {
      return( update ) ;
   }




   /*------------------------------------------------------------*/
   /* getInsertUpdate
    */
   /**
    *    This method will return our insert/update
    *  value
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getInsertUpdate () {
      return( insertUpdate ) ;
   }





   /*------------------------------------------------------------*/
   /* getSystems
    */
   /**
    *    This method will return our systems value
    *  
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getSystems () {
      return( systems ) ;
   }





   /*------------------------------------------------------------*/
   /* getMonitors
    */
   /**
    *    This method will return our monitors value
    *  
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getMonitors () {
      return( monitors ) ;
   }





   /*------------------------------------------------------------*/
   /* getOptions
    */
   /**
    *    This method will return our options value
    *  
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public boolean getOptions () {
      return( options ) ;
   }




   /*------------------------------------------------------------*/
   /* getImportFile
    */
   /**
    *    This method will get the import filename
    *  
    *  
    *
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public String getImportFile () {
      return( importFilename ) ;
   }





   /*------------------------------------------------------------*/
   /* actionPerformed
    */
   /**
    *    This method will be called when an action
    *  event that we are listening for has been fired
    *  
    *
    *  @param e The event
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void actionPerformed ( ActionEvent e ) {

      if ( e.getSource() == pb_OK ) {
         insert         = rb_INSERT.isSelected() ;
         update         = rb_UPDATE.isSelected() ;
         insertUpdate   = rb_INSERTUPDATE.isSelected() ;
         systems        = rb_SYSTEMS.isSelected() ;
         monitors       = rb_MONITORS.isSelected() ;
         options        = rb_OPTIONS.isSelected() ;
         importFilename = ef_FILENAME.getText() ;

         if ( ( importFilename == null ) || ( importFilename.length() == 0 ) ) {
            GUISystem.printBox ( STR_ERROR, STR_MUST_ENTER_IMPORT_FILE ) ;
            importFilename = null ;
         } else {
            File file = new File ( importFilename ) ;

            if ( file.exists() )
               dispose() ;
            else {
               GUISystem.printBox ( STR_ERROR, STR_MUST_ENTER_IMPORT_FILE ) ;
               importFilename = null ;
            } /* else  */
         } /* else  */
      } /* if  */

      else if ( e.getSource() == pb_BROWSE )
         browseFiles() ;

      else if ( e.getSource() == pb_CANCEL ) {
         importFilename = null ;
         dispose() ;
      }
   }
} /* ProductImportDlg */

