package com.ibm.nzna.projects.qit.product.storedProc.prodListReader ;                       
/*-----------------------------------------------------------*/
/*   ProdListReader                                          */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProdListReader                      */
/*                                                           */
/*                                                           */
/*         @Copyright  1998 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This Stored Procedure reads in ALL information about a single document 
 *
 *
 * @version 1.0, 9/15/1999 
 * @author  David Lentz
 */
import java.sql.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Blob ;
import java.io.* ;
import java.util.* ;

/*-----------------------------------------------------------*/
/*                                                           */
/*   DocWriter                                               */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProdListReader extends StoredProc {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Program Vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int                        sqlCode        = 0 ;          // The SQLCODE we will return
   private Connection                 con            = null ;       // Our connection to DB/2
   private int                        request        = 0 ;          // 1 = Request, 0 = Existing Document
   private String                     qualifier      = "TIGRIS" ;   // The qualifier in the database to read
   private Blob                       results        = null ;       // The blob passed to us


   /*------------------------------------------------------------*/
   /* readList
    */
   /**
    *    This method will be called by DB/2 when
    *  we are called by a client. It will process reading
    *  a product list
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readList ( String whereClause, Blob results, int sqlCode, String errorString ) throws Exception  {                            

      try {
         ProdListReaderBuffer     buffer  = new ProdListReaderBuffer() ;
         ProdListReaderProdBuffer readRec = null ;
         String               columns = "BRANDGROUPIND, PRODINT,    PRODNUM, "+
                                        "PRODDESC,      MACHINE,    MODEL,   "+
                                        "STATUSIND,     STATUSDATE, DBUSER,  "+
                                        "CHANGEDTIME" ;
         Vector               prodVec = new Vector ( 100, 10 )  ;
         String               sqlStmt = null ;
         Statement            stmt    = null ;
         ResultSet            rs      = null ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the DB/2 Connection                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;                                                                           
         stmt = con.createStatement() ;   

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Form the SQL                                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         sqlStmt = "SELECT "+columns+" FROM TIGRIS.PRODUCTS WHERE "+whereClause ;                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Initialize our vars                                     */
         /*                                                           */
         /*-----------------------------------------------------------*/
         buffer.retVec = new Vector() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute the SQL                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( sqlStmt ) ;         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in our values                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {
            readRec = new ProdListReaderProdBuffer () ;
            readRec.brandGroupInd = rs.getInt(1) ;
            readRec.prodInt       = rs.getInt(2) ;
            readRec.prodNum       = rs.getString(3).trim() ;
            readRec.prodDesc      = rs.getString(4) ;
            readRec.machine       = rs.getString(5) ;
            readRec.model         = rs.getString(6) ;
            readRec.statusInd     = rs.getInt(7) ;
            readRec.statusDate    = rs.getString(8) ;
            readRec.dbUser        = rs.getString(9) ;
            readRec.changedTime   = rs.getString(10) ;
            buffer.retVec.addElement ( readRec ) ;
         }

         rs.close() ;
         stmt.close() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create a blob to return data                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         results = Lob.newBlob() ;
         if ( results != null ) {
            setObjectInBlob ( results, buffer ) ;
            set ( 2, results ) ;
         }
         set ( 3, sqlCode ) ;

         con.close() ;

      } catch ( Exception e ) {
         StringWriter    sw      = new StringWriter () ;
         PrintWriter     pw      = new PrintWriter ( sw ) ;

         /* Get the stackTrace */
         e.printStackTrace ( pw ) ;
         set(4, e.toString() ) ;
      }

   }

   /*------------------------------------------------------------*/
   /* setObjectInBlob
    */
   /**
    *    This method will copy a serialized object into a blob.
    *  
    *
    *  @param blob The Blob
    *  @param object The Object to serialize
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setObjectInBlob ( Blob blob, Object object ) throws Exception {
      ByteArrayOutputStream outStream = new ByteArrayOutputStream() ;
      ObjectOutputStream    s         = new ObjectOutputStream ( blob.getOutputStream() ) ;

      s.writeObject ( object ) ;
      s.flush() ;
      s.close() ;
   }



} /* DocWriter */
