package com.ibm.nzna.projects.qit.product.storedProc.prodListReader ;                       
/*-----------------------------------------------------------*/
/*   ProductListReader                                       */
/*                                                           */
/*   Description:                                            */
/*      Source code for: ProductListReader                   */
/*                                                           */
/*                                                           */
/*         @Copyright  2001 IBM                              */
/*                                                           */
/*-----------------------------------------------------------*/
/**
 * This Stored Procedure reads in information about a list of products 
 *
 *
 * @version 1.0, 7/15/2001 
 * @author  
 */
import java.sql.* ;
import COM.ibm.db2.jdbc.app.* ;
import COM.ibm.db2.app.* ;
import COM.ibm.db2.app.Blob ;
import java.io.* ;
import java.util.* ;
import com.ibm.nzna.projects.common.quest.product.*;

/*-----------------------------------------------------------*/
/*                                                           */
/*   ProductListReader                                       */
/*                                                           */
/*-----------------------------------------------------------*/
public class ProductListReader extends StoredProc {

   /*-----------------------------------------------------------*/
   /*                                                           */
   /*   Program Vars                                            */
   /*                                                           */
   /*-----------------------------------------------------------*/
   private int                        sqlCode        = 0 ;          // The SQLCODE we will return
   private Connection                 con            = null ;       // Our connection to DB/2
   private Blob                       results        = null ;       // The blob passed to us
   private String                     query          = null ;

   /*------------------------------------------------------------*/
   /* readList
    */
   /**
    *    This method will be called by DB/2 when
    *  we are called by a client. It will process reading
    *  a product list
    *
    *  @param None
    *  @param None
    *  @param None
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public void readList ( String query, Blob results, int sqlCode, String errorString ) throws Exception  {                            

      try {
         Vector               returnVec = new Vector( 100 , 10) ;
         Statement            stmt      = null ;
         ResultSet            rs        = null ;
         boolean              isRequest = ( query.toUpperCase().indexOf( "PRODRAFT.PRODUCT" ) >= 0 ) ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Get the DB/2 Connection                                 */
         /*                                                           */
         /*-----------------------------------------------------------*/
         con = getConnection() ;                                                                           
         stmt = con.createStatement() ;   
         this.query = query ;                

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Execute the SQL                                         */
         /*                                                           */
         /*-----------------------------------------------------------*/
         rs = stmt.executeQuery ( query ) ;         

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Read in our values                                      */
         /*                                                           */
         /*-----------------------------------------------------------*/
         while ( rs.next() ) {
             returnVec.addElement ( new ProductRow ( rs.getInt(1),               // Product Indicator
                                                     rs.getInt(2),               // Existing Product Indicator
                                                     rs.getString(3),            // Product Number
                                                     rs.getString(4).trim(),     // Description
                                                     rs.getString(5),            // Created On
                                                     rs.getString(6).trim(),     // Created By
                                                     rs.getString(7),            // Actual Date
                                                     rs.getString(8),            // Target Date
                                                     rs.getString(9).trim(),     // Last Touched By
                                                     rs.getString(10),           // Last Touched On
                                                     isRequest ) ) ;

         }

         rs.close() ;
         stmt.close() ;

         /*-----------------------------------------------------------*/
         /*                                                           */
         /*   Create a blob to return data                            */
         /*                                                           */
         /*-----------------------------------------------------------*/
         results = Lob.newBlob() ;
         if ( results != null ) {
            setObjectInBlob ( results, returnVec ) ;
            set ( 2, results ) ;
         }
         set ( 3, sqlCode ) ;

         con.close() ;

      } catch ( Exception e ) {
         StringWriter    sw      = new StringWriter () ;
         PrintWriter     pw      = new PrintWriter ( sw ) ;

         /* Get the stackTrace */
         e.printStackTrace ( pw ) ;
         set(4, e.toString() ) ;
      }

   }

   /*------------------------------------------------------------*/
   /* setObjectInBlob
    */
   /**
    *    This method will copy a serialized object into a blob.
    *  
    *
    *  @param blob The Blob
    *  @param object The Object to serialize
    *
    *  @return Nothing
    *//*---------------------------------------------------------*/
   public static void setObjectInBlob ( Blob blob, Object object ) throws Exception {
      ByteArrayOutputStream outStream = new ByteArrayOutputStream() ;
      ObjectOutputStream    s         = new ObjectOutputStream ( blob.getOutputStream() ) ;

      s.writeObject ( object ) ;
      s.flush() ;
      s.close() ;
   }



} /* ProductListReader */
